=== modified file '.bzrignore'
--- .bzrignore	2010-04-01 04:41:18 +0000
+++ .bzrignore	2010-10-13 00:26:41 +0000
@@ -19,7 +19,8 @@
 ./doc/**/*.html
 ./doc/developers/performance.png
 ./doc/en/user-reference/*.txt
-./doc/en/release-notes/*.txt
+./doc/en/release-notes/index.txt
+./doc/en/release-notes/NEWS.txt
 BRANCH-INFO
 # setup.py working directory
 ./build

=== modified file '.testr.conf'
--- .testr.conf	2010-02-28 10:08:29 +0000
+++ .testr.conf	2010-12-20 11:57:14 +0000
@@ -1,3 +1,4 @@
 [DEFAULT]
-test_command=./bzr selftest --subunit $IDOPTION
+test_command=./bzr selftest --subunit $IDOPTION $LISTOPT
 test_id_option=--load-list $IDFILE
+test_list_option=--list

=== modified file 'Makefile'
--- Makefile	2010-04-06 06:59:03 +0000
+++ Makefile	2011-09-06 11:01:30 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 PLUGIN_TARGET=plugin-release
 PYTHON_BUILDFLAGS=
 
-.PHONY: all clean extensions pyflakes api-docs check-nodocs check
+.PHONY: all clean realclean extensions pyflakes api-docs check-nodocs check
 
 all: extensions
 
@@ -39,8 +39,16 @@
 check: docs check-nodocs
 
 check-nodocs: extensions
+	set -e
 	# Generate a stream for PQM to watch.
-	$(PYTHON) -Werror -O ./bzr selftest --subunit $(tests) | tee selftest.log
+	-$(RM) -f selftest.log
+	echo `date` ": selftest starts" 1>&2
+	$(PYTHON) -Werror -Wignore::ImportWarning -O ./bzr selftest --subunit $(tests) | tee selftest.log
+	echo `date` ": selftest ends" 1>&2
+	# An empty log file should catch errors in the $(PYTHON)
+	# command above (the '|' swallow any errors since 'make'
+	# sees the 'tee' exit code for the whole line
+	if [ ! -s selftest.log ] ; then exit 1 ; fi
 	# Check that there were no errors reported.
 	subunit-stats < selftest.log
 
@@ -60,13 +68,16 @@
 	$(PYTHON) setup.py clean
 	-find . -name "*.pyc" -o -name "*.pyo" -o -name "*.so" | xargs rm -f
 
+realclean: clean
+	# Remove files which are autogenerated but included by the tarball.
+	rm -f bzrlib/*_pyx.c
+	rm -f bzrlib/_simple_set_pyx.h bzrlib/_simple_set_pyx_api.h
+
 # Build API documentation
 docfiles = bzr bzrlib
 api-docs:
 	mkdir -p api/html
-	PYTHONPATH=$(PWD) $(PYTHON) tools/bzr_epydoc --html -o api/html --docformat 'restructuredtext en' $(docfiles)
-check-api-docs:
-	PYTHONPATH=$(PWD) $(PYTHON) tools/bzr_epydoc --check --docformat 'restructuredtext en' $(docfiles)
+	pydoctor --make-html --docformat='restructuredtext' --html-output=api/html $(docfiles)
 
 # build tags for emacs and vim
 TAGS:
@@ -133,11 +144,13 @@
 	doc/developers/Makefile \
 	doc/developers/make.bat
 
+NEWS_FILES = $(wildcard doc/en/release-notes/bzr-*.txt)
+
 doc/en/user-reference/index.txt: $(MAN_DEPENDENCIES)
 	$(PYTHON) tools/generate_docs.py -o $@ rstx
 
-doc/en/release-notes/index.txt: NEWS tools/generate_release_notes.py
-	$(PYTHON) tools/generate_release_notes.py NEWS $@
+doc/en/release-notes/index.txt: $(NEWS_FILES) tools/generate_release_notes.py
+	$(PYTHON) tools/generate_release_notes.py $@ $(NEWS_FILES)
 
 doc/%/Makefile: doc/en/Makefile
 	$(PYTHON) -c "import shutil; shutil.copyfile('$<', '$@')"
@@ -296,10 +309,10 @@
 	$(rst2html) --stylesheet=default.css $< $@
 
 %.html: %.txt
-	$(rst2html) --stylesheet=../../default.css $< $@
+	$(rst2html) --stylesheet=../../default.css $< "$@"
 
-doc/en/release-notes/NEWS.txt: NEWS
-	$(PYTHON) -c "import shutil; shutil.copyfile('$<', '$@')"
+doc/en/release-notes/NEWS.txt: $(NEWS_FILES) tools/generate_release_notes.py
+	$(PYTHON) tools/generate_release_notes.py "$@" $(NEWS_FILES)
 
 upgrade_guide_dependencies =  $(wildcard $(addsuffix /*.txt, doc/en/upgrade-guide)) 
 
@@ -407,6 +420,26 @@
 	$(PYTHON) tools/win32/ostools.py remove dist
 
 
+# i18n targets
+
+.PHONY: update-pot po/bzr.pot
+update-pot: po/bzr.pot
+
+TRANSLATABLE_PYFILES:=$(shell find bzrlib -name '*.py' \
+    		| grep -v 'bzrlib/tests/' \
+    		| grep -v 'bzrlib/doc' \
+		)
+
+po/bzr.pot: $(PYFILES) $(DOCFILES)
+	$(PYTHON) ./bzr export-pot > po/bzr.pot
+	echo $(TRANSLATABLE_PYFILES) | xargs \
+	  xgettext --package-name "bzr" \
+	  --msgid-bugs-address "<bazaar@canonical.com>" \
+	  --copyright-holder "Canonical" \
+	  --from-code ISO-8859-1 --join --sort-by-file --add-comments=i18n: \
+	  -d bzr -p po -o bzr.pot
+
+
 ### Packaging Targets ###
 
 .PHONY: dist check-dist-tarball

=== added file 'NEWS'
--- NEWS	1970-01-01 00:00:00 +0000
+++ NEWS	2010-10-08 08:15:14 +0000
@@ -0,0 +1,2 @@
+The NEWS file has been moved and split into multiple files (one per release
+series).  The NEWS files are now found in doc/en/release-notes/.

=== modified file 'README'
--- README	2010-01-29 14:09:05 +0000
+++ README	2010-10-07 06:08:01 +0000
@@ -36,7 +36,7 @@
 It also directly supports and encourages a large number of development best
 practices like refactoring and pre-commit regression testing. Users can
 choose between our command line tool and our cross-platform GUI application.
-For further details, see our website at http://bazaar-vcs.org/en.
+For further details, see our website at http://bazaar.canonical.com/en/
 
 Feedback
 ========

=== modified file 'bzr'
--- bzr	2010-07-13 07:44:02 +0000
+++ bzr	2011-09-13 12:29:22 +0000
@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,16 +23,16 @@
 import warnings
 
 # update this on each release
-_script_version = (2, 3, 0)
+_script_version = (2, 5, 0)
 
 try:
     version_info = sys.version_info
 except AttributeError:
-    version_info = 1, 5 # 1.5 or older
+    version_info = 1, 5  # 1.5 or older
 
 REINVOKE = "__BZR_REINVOKE"
-NEED_VERS = (2, 4)
-KNOWN_PYTHONS = ('python2.4', 'python2.5', 'python2.6')
+NEED_VERS = (2, 6)
+KNOWN_PYTHONS = ('python2.6', 'python2.7')
 
 if version_info < NEED_VERS:
     if not os.environ.has_key(REINVOKE):
@@ -75,6 +75,16 @@
 else:
     import locale
 
+if os.name == "posix":
+    try:
+        locale.setlocale(locale.LC_ALL, '')
+    except locale.Error, e:
+        sys.stderr.write('bzr: warning: %s\n'
+            '  bzr could not set the application locale.\n'
+            '  Although this should be no problem for bzr itself, it might\n'
+            '  cause problems with some plugins. To investigate the issue,\n'
+            '  look at the output of the locale(1p) tool.\n' % e)
+
 
 # The python2.6 release includes some libraries that have deprecation warnings
 # against the interpreter - see https://bugs.launchpad.net/bzr/+bug/387139
@@ -86,17 +96,6 @@
     )
 
 
-try:
-    locale.setlocale(locale.LC_ALL, '')
-except locale.Error, e:
-    sys.stderr.write('bzr: warning: %s\n'
-                     '  bzr could not set the application locale.\n'
-                     '  Although this should be no problem for bzr itself,\n'
-                     '  it might cause problems with some plugins.\n'
-                     '  To investigate the issue, look at the output\n'
-                     '  of the locale(1p) tool available on POSIX systems.\n'
-                     % e)
-
 # instruct bzrlib/__init__.py to install lazy_regex
 sys._bzr_lazy_regex = True
 try:
@@ -109,10 +108,16 @@
     raise
 
 if bzrlib.version_info[:3] != _script_version:
-    sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
-            "This may indicate an installation problem.\n"
-            "bzrlib from %s is version %r\n"
-            % (bzrlib.__path__, bzrlib.version_info))
+    sys.stderr.write(
+        "bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
+        "This may indicate an installation problem.\n"
+        "bzrlib is version %s from %s\n"
+        "bzr is version %s from %s\n" % (
+        bzrlib._format_version_tuple(bzrlib.version_info),
+        bzrlib.__path__[0],
+        bzrlib._format_version_tuple(_script_version),
+        __file__))
+
 
 import bzrlib.inspect_for_copy
 bzrlib.inspect_for_copy.import_copy_with_hacked_inspect()

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2010-07-13 07:44:02 +0000
+++ bzrlib/__init__.py	2011-10-06 09:20:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 IGNORE_FILENAME = ".bzrignore"
 
 
-__copyright__ = "Copyright 2005-2010 Canonical Ltd."
+__copyright__ = "Copyright 2005-2011 Canonical Ltd."
 
 # same format as sys.version_info: "A tuple containing the five components of
 # the version number: major, minor, micro, releaselevel, and serial. All
@@ -52,10 +52,10 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (2, 3, 0, 'dev', 1)
+version_info = (2, 5, 0, 'dev', 3)
 
 # API compatibility version
-api_minimum_version = (2, 2, 0)
+api_minimum_version = (2, 4, 0)
 
 
 def _format_version_tuple(version_info):
@@ -71,24 +71,20 @@
     1.0.0
     >>> print _format_version_tuple((1, 2, 0, 'dev', 0))
     1.2.0dev
-    >>> print bzrlib._format_version_tuple((1, 2, 0, 'dev', 1))
+    >>> print _format_version_tuple((1, 2, 0, 'dev', 1))
     1.2.0dev1
     >>> print _format_version_tuple((1, 1, 1, 'candidate', 2))
     1.1.1rc2
-    >>> print bzrlib._format_version_tuple((2, 1, 0, 'beta', 1))
+    >>> print _format_version_tuple((2, 1, 0, 'beta', 1))
     2.1b1
     >>> print _format_version_tuple((1, 4, 0))
     1.4.0
     >>> print _format_version_tuple((1, 4))
     1.4
-    >>> print bzrlib._format_version_tuple((2, 1, 0, 'final', 1))
-    Traceback (most recent call last):
-    ...
-    ValueError: version_info (2, 1, 0, 'final', 1) not valid
+    >>> print _format_version_tuple((2, 1, 0, 'final', 42))
+    2.1.0.42
     >>> print _format_version_tuple((1, 4, 0, 'wibble', 0))
-    Traceback (most recent call last):
-    ...
-    ValueError: version_info (1, 4, 0, 'wibble', 0) not valid
+    1.4.0.wibble.0
     """
     if len(version_info) == 2:
         main_version = '%d.%d' % version_info[:2]
@@ -100,9 +96,10 @@
     release_type = version_info[3]
     sub = version_info[4]
 
-    # check they're consistent
     if release_type == 'final' and sub == 0:
         sub_string = ''
+    elif release_type == 'final':
+        sub_string = '.' + str(sub)
     elif release_type == 'dev' and sub == 0:
         sub_string = 'dev'
     elif release_type == 'dev':
@@ -114,7 +111,7 @@
     elif release_type == 'candidate':
         sub_string = 'rc' + str(sub)
     else:
-        raise ValueError("version_info %r not valid" % (version_info,))
+        return '.'.join(map(str, version_info))
 
     return main_version + sub_string
 
@@ -140,7 +137,7 @@
 # variables that need to be hunted down. Accessing the current BzrLibraryState
 # through this variable is not encouraged: it is better to pass it around as
 # part of the context of an operation than to look it up directly, but when
-# that is too hard, it is better to use this variable than to make a branch new
+# that is too hard, it is better to use this variable than to make a brand new
 # global variable.
 # If using this variable by looking it up (because it can't be easily obtained)
 # it is important to store the reference you get, rather than looking it up
@@ -157,13 +154,18 @@
 
     More options may be added in future so callers should use named arguments.
 
+    The object returned by this function can be used as a contex manager
+    through the 'with' statement to automatically shut down when the process
+    is finished with bzrlib.  However (from bzr 2.4) it's not necessary to
+    separately enter the context as well as starting bzr: bzrlib is ready to
+    go when this function returns.
+
     :param setup_ui: If true (default) use a terminal UI; otherwise 
         some other ui_factory must be assigned to `bzrlib.ui.ui_factory` by
         the caller.
     :param stdin, stdout, stderr: If provided, use these for terminal IO;
         otherwise use the files in `sys`.
-    :return: A context manager for the use of bzrlib. The __enter__ method of
-        this context needs to be called before it takes effect, and the __exit__
+    :return: A context manager for the use of bzrlib. The __exit__
         should be called by the caller before exiting their process or
         otherwise stopping use of bzrlib. Advanced callers can use
         BzrLibraryState directly.
@@ -178,7 +180,10 @@
     else:
         ui_factory = None
     tracer = trace.DefaultConfig()
-    return library_state.BzrLibraryState(ui=ui_factory, trace=tracer)
+    state = library_state.BzrLibraryState(ui=ui_factory, trace=tracer)
+    # Start automatically in case people don't realize this returns a context.
+    state._start()
+    return state
 
 
 def test_suite():

=== modified file 'bzrlib/_btree_serializer_pyx.pyx'
--- bzrlib/_btree_serializer_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_btree_serializer_pyx.pyx	2011-05-11 11:35:28 +0000
@@ -33,6 +33,7 @@
     char *PyString_AsString(object p) except NULL
     object PyString_FromStringAndSize(char *, Py_ssize_t)
     PyObject *PyString_FromStringAndSize_ptr "PyString_FromStringAndSize" (char *, Py_ssize_t)
+    object PyString_FromFormat(char *, ...)
     int PyString_CheckExact(object s)
     int PyString_CheckExact_ptr "PyString_CheckExact" (PyObject *)
     Py_ssize_t PyString_Size(object p)
@@ -49,19 +50,31 @@
     PyObject *PyTuple_GET_ITEM_ptr_object "PyTuple_GET_ITEM" (object tpl, int index)
     void Py_INCREF(object)
     void Py_DECREF_ptr "Py_DECREF" (PyObject *)
+    void *PyMem_Malloc(size_t nbytes)
+    void PyMem_Free(void *)
+    void memset(void *, int, size_t)
 
 cdef extern from "string.h":
     void *memcpy(void *dest, void *src, size_t n)
     void *memchr(void *s, int c, size_t n)
+    int memcmp(void *s1, void *s2, size_t n)
     # GNU extension
     # void *memrchr(void *s, int c, size_t n)
     int strncmp(char *s1, char *s2, size_t n)
+    unsigned long strtoul(char *s1, char **out, int base)
+    long long strtoll(char *s1, char **out, int base)
+
 
 # It seems we need to import the definitions so that the pyrex compiler has
 # local names to access them.
 from _static_tuple_c cimport StaticTuple, \
     import_static_tuple_c, StaticTuple_New, \
-    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact
+    StaticTuple_Intern, StaticTuple_SET_ITEM, StaticTuple_CheckExact, \
+    StaticTuple_GET_SIZE, StaticTuple_GET_ITEM
+# This tells the test infrastructure that StaticTuple is a class, so we don't
+# have to worry about exception checking.
+## extern cdef class StaticTuple
+import sys
 
 
 # TODO: Find some way to import this from _dirstate_helpers
@@ -103,7 +116,6 @@
     Py_DECREF_ptr(py_str)
     return result
 
-from bzrlib import _static_tuple_c
 # This sets up the StaticTuple C_API functionality
 import_static_tuple_c()
 
@@ -315,6 +327,521 @@
     return parser.parse()
 
 
+# TODO: We can go from 8 byte offset + 4 byte length to a simple lookup,
+#       because the block_offset + length is likely to be repeated. However,
+#       the big win there is to cache across pages, and not just one page
+#       Though if we did cache in a page, we could certainly use a short int.
+#       And this goes from 40 bytes to 30 bytes.
+#       One slightly ugly option would be to cache block offsets in a global.
+#       However, that leads to thread-safety issues, etc.
+ctypedef struct gc_chk_sha1_record:
+    long long block_offset
+    unsigned int block_length
+    unsigned int record_start
+    unsigned int record_end
+    char sha1[20]
+
+
+cdef int _unhexbuf[256]
+cdef char *_hexbuf
+_hexbuf = '0123456789abcdef'
+
+cdef _populate_unhexbuf():
+    cdef int i
+    for i from 0 <= i < 256:
+        _unhexbuf[i] = -1
+    for i from 0 <= i < 10: # 0123456789 => map to the raw number
+        _unhexbuf[(i + c'0')] = i
+    for i from 10 <= i < 16: # abcdef => 10, 11, 12, 13, 14, 15, 16
+        _unhexbuf[(i - 10 + c'a')] = i
+    for i from 10 <= i < 16: # ABCDEF => 10, 11, 12, 13, 14, 15, 16
+        _unhexbuf[(i - 10 + c'A')] = i
+_populate_unhexbuf()
+
+
+cdef int _unhexlify_sha1(char *as_hex, char *as_bin): # cannot_raise
+    """Take the hex sha1 in as_hex and make it binary in as_bin
+    
+    Same as binascii.unhexlify, but working on C strings, not Python objects.
+    """
+    cdef int top
+    cdef int bot
+    cdef int i, j
+    cdef char *cur
+    
+    # binascii does this using isupper() and tolower() and ?: syntax. I'm
+    # guessing a simple lookup array should be faster.
+    j = 0
+    for i from 0 <= i < 20:
+        top = _unhexbuf[<unsigned char>(as_hex[j])]
+        j = j + 1
+        bot = _unhexbuf[<unsigned char>(as_hex[j])]
+        j = j + 1
+        if top == -1 or bot == -1:
+            return 0
+        as_bin[i] = <unsigned char>((top << 4) + bot);
+    return 1
+
+
+def _py_unhexlify(as_hex):
+    """For the test infrastructure, just thunks to _unhexlify_sha1"""
+    if len(as_hex) != 40 or not PyString_CheckExact(as_hex):
+        raise ValueError('not a 40-byte hex digest')
+    as_bin = PyString_FromStringAndSize(NULL, 20)
+    if _unhexlify_sha1(PyString_AS_STRING(as_hex), PyString_AS_STRING(as_bin)):
+        return as_bin
+    return None
+
+
+cdef void _hexlify_sha1(char *as_bin, char *as_hex): # cannot_raise
+    cdef int i, j
+    cdef char c
+
+    j = 0
+    for i from 0 <= i < 20:
+        c = as_bin[i]
+        as_hex[j] = _hexbuf[(c>>4)&0xf]
+        j = j + 1
+        as_hex[j] = _hexbuf[(c)&0xf]
+        j = j + 1
+
+
+def _py_hexlify(as_bin):
+    """For test infrastructure, thunk to _hexlify_sha1"""
+    if len(as_bin) != 20 or not PyString_CheckExact(as_bin):
+        raise ValueError('not a 20-byte binary digest')
+    as_hex = PyString_FromStringAndSize(NULL, 40)
+    _hexlify_sha1(PyString_AS_STRING(as_bin), PyString_AS_STRING(as_hex))
+    return as_hex
+
+
+cdef int _key_to_sha1(key, char *sha1): # cannot_raise
+    """Map a key into its sha1 content.
+
+    :param key: A tuple of style ('sha1:abcd...',)
+    :param sha1: A char buffer of 20 bytes
+    :return: 1 if this could be converted, 0 otherwise
+    """
+    cdef char *c_val
+    cdef PyObject *p_val
+
+    if StaticTuple_CheckExact(key) and StaticTuple_GET_SIZE(key) == 1:
+        p_val = <PyObject *>StaticTuple_GET_ITEM(key, 0)
+    elif (PyTuple_CheckExact(key) and PyTuple_GET_SIZE(key) == 1):
+        p_val = PyTuple_GET_ITEM_ptr_object(key, 0)
+    else:
+        # Not a tuple or a StaticTuple
+        return 0
+    if (PyString_CheckExact_ptr(p_val) and PyString_GET_SIZE_ptr(p_val) == 45):
+        c_val = PyString_AS_STRING_ptr(p_val)
+    else:
+        return 0
+    if strncmp(c_val, 'sha1:', 5) != 0:
+        return 0
+    if not _unhexlify_sha1(c_val + 5, sha1):
+        return 0
+    return 1
+
+
+def _py_key_to_sha1(key):
+    """Map a key to a simple sha1 string.
+
+    This is a testing thunk to the C function.
+    """
+    as_bin_sha = PyString_FromStringAndSize(NULL, 20)
+    if _key_to_sha1(key, PyString_AS_STRING(as_bin_sha)):
+        return as_bin_sha
+    return None
+
+
+cdef StaticTuple _sha1_to_key(char *sha1):
+    """Compute a ('sha1:abcd',) key for a given sha1."""
+    cdef StaticTuple key
+    cdef object hexxed
+    cdef char *c_buf
+    hexxed = PyString_FromStringAndSize(NULL, 45)
+    c_buf = PyString_AS_STRING(hexxed)
+    memcpy(c_buf, 'sha1:', 5)
+    _hexlify_sha1(sha1, c_buf+5)
+    key = StaticTuple_New(1)
+    Py_INCREF(hexxed)
+    StaticTuple_SET_ITEM(key, 0, hexxed)
+    # This is a bit expensive. To parse 120 keys takes 48us, to return them all
+    # can be done in 66.6us (so 18.6us to build them all).
+    # Adding simple hash() here brings it to 76.6us (so computing the hash
+    # value of 120keys is 10us), Intern is 86.9us (another 10us to look and add
+    # them to the intern structure.)
+    # However, since we only intern keys that are in active use, it is probably
+    # a win. Since they would have been read from elsewhere anyway.
+    # We *could* hang the PyObject form off of the gc_chk_sha1_record for ones
+    # that we have deserialized. Something to think about, at least.
+    key = StaticTuple_Intern(key)
+    return key
+
+
+def _py_sha1_to_key(sha1_bin):
+    """Test thunk to check the sha1 mapping."""
+    if not PyString_CheckExact(sha1_bin) or PyString_GET_SIZE(sha1_bin) != 20:
+        raise ValueError('sha1_bin must be a str of exactly 20 bytes')
+    return _sha1_to_key(PyString_AS_STRING(sha1_bin))
+
+
+cdef unsigned int _sha1_to_uint(char *sha1): # cannot_raise
+    cdef unsigned int val
+    # Must be in MSB, because that is how the content is sorted
+    val = (((<unsigned int>(sha1[0]) & 0xff) << 24)
+           | ((<unsigned int>(sha1[1]) & 0xff) << 16)
+           | ((<unsigned int>(sha1[2]) & 0xff) << 8)
+           | ((<unsigned int>(sha1[3]) & 0xff) << 0))
+    return val
+
+
+cdef _format_record(gc_chk_sha1_record *record):
+    # This is inefficient to go from a logical state back to a
+    # string, but it makes things work a bit better internally for now.
+    if record.block_offset >= 0xFFFFFFFF:
+        # %llu is what we really want, but unfortunately it was only added
+        # in python 2.7... :(
+        block_offset_str = str(record.block_offset)
+        value = PyString_FromFormat('%s %u %u %u',
+                                PyString_AS_STRING(block_offset_str),
+                                record.block_length,
+                                record.record_start, record.record_end)
+    else:
+        value = PyString_FromFormat('%lu %u %u %u',
+                                    <unsigned long>record.block_offset,
+                                    record.block_length,
+                                    record.record_start, record.record_end)
+    return value
+
+
+cdef class GCCHKSHA1LeafNode:
+    """Track all the entries for a given leaf node."""
+
+    cdef gc_chk_sha1_record *records
+    cdef public object last_key
+    cdef gc_chk_sha1_record *last_record
+    cdef public int num_records
+    # This is the number of bits to shift to get to the interesting byte. A
+    # value of 24 means that the very first byte changes across all keys.
+    # Anything else means that there is a common prefix of bits that we can
+    # ignore. 0 means that at least the first 3 bytes are identical, though
+    # that is going to be very rare
+    cdef public unsigned char common_shift
+    # This maps an interesting byte to the first record that matches.
+    # Equivalent to bisect.bisect_left(self.records, sha1), though only taking
+    # into account that one byte.
+    cdef unsigned char offsets[257]
+
+    def __sizeof__(self):
+        # :( Why doesn't Pyrex let me do a simple sizeof(GCCHKSHA1LeafNode)
+        # like Cython? Explicitly enumerating everything here seems to leave my
+        # size off by 2 (286 bytes vs 288 bytes actual). I'm guessing it is an
+        # alignment/padding issue. Oh well- at least we scale properly with
+        # num_records and are very close to correct, which is what I care
+        # about.
+        # If we ever decide to require cython:
+        # return (sizeof(GCCHKSHA1LeafNode)
+        #     + sizeof(gc_chk_sha1_record)*self.num_records)
+        return (sizeof(PyObject) + sizeof(void*) + sizeof(int)
+            + sizeof(gc_chk_sha1_record*) + sizeof(PyObject *)
+            + sizeof(gc_chk_sha1_record*) + sizeof(char)
+            + sizeof(unsigned char)*257
+            + sizeof(gc_chk_sha1_record)*self.num_records)
+
+    def __dealloc__(self):
+        if self.records != NULL:
+            PyMem_Free(self.records)
+            self.records = NULL
+
+    def __init__(self, bytes):
+        self._parse_bytes(bytes)
+        self.last_key = None
+        self.last_record = NULL
+
+    property min_key:
+        def __get__(self):
+            if self.num_records > 0:
+                return _sha1_to_key(self.records[0].sha1)
+            return None
+
+    property max_key:
+        def __get__(self):
+            if self.num_records > 0:
+                return _sha1_to_key(self.records[self.num_records-1].sha1)
+            return None
+
+    cdef StaticTuple _record_to_value_and_refs(self,
+                                               gc_chk_sha1_record *record):
+        """Extract the refs and value part of this record."""
+        cdef StaticTuple value_and_refs
+        cdef StaticTuple empty
+        value_and_refs = StaticTuple_New(2)
+        value = _format_record(record)
+        Py_INCREF(value)
+        StaticTuple_SET_ITEM(value_and_refs, 0, value)
+        # Always empty refs
+        empty = StaticTuple_New(0)
+        Py_INCREF(empty)
+        StaticTuple_SET_ITEM(value_and_refs, 1, empty)
+        return value_and_refs
+
+    cdef StaticTuple _record_to_item(self, gc_chk_sha1_record *record):
+        """Turn a given record back into a fully fledged item.
+        """
+        cdef StaticTuple item
+        cdef StaticTuple key
+        cdef StaticTuple value_and_refs
+        cdef object value
+        key = _sha1_to_key(record.sha1)
+        item = StaticTuple_New(2)
+        Py_INCREF(key)
+        StaticTuple_SET_ITEM(item, 0, key)
+        value_and_refs = self._record_to_value_and_refs(record)
+        Py_INCREF(value_and_refs)
+        StaticTuple_SET_ITEM(item, 1, value_and_refs)
+        return item
+
+    cdef gc_chk_sha1_record* _lookup_record(self, char *sha1) except? NULL:
+        """Find a gc_chk_sha1_record that matches the sha1 supplied."""
+        cdef int lo, hi, mid, the_cmp
+        cdef int offset
+
+        # TODO: We can speed up misses by comparing this sha1 to the common
+        #       bits, and seeing if the common prefix matches, if not, we don't
+        #       need to search for anything because it cannot match
+        # Use the offset array to find the closest fit for this entry
+        # follow that up with bisecting, since multiple keys can be in one
+        # spot
+        # Bisecting dropped us from 7000 comparisons to 582 (4.8/key), using
+        # the offset array dropped us from 23us to 20us and 156 comparisions
+        # (1.3/key)
+        offset = self._offset_for_sha1(sha1)
+        lo = self.offsets[offset]
+        hi = self.offsets[offset+1]
+        if hi == 255:
+            # if hi == 255 that means we potentially ran off the end of the
+            # list, so push it up to num_records
+            # note that if 'lo' == 255, that is ok, because we can start
+            # searching from that part of the list.
+            hi = self.num_records
+        local_n_cmp = 0
+        while lo < hi:
+            mid = (lo + hi) / 2
+            the_cmp = memcmp(self.records[mid].sha1, sha1, 20)
+            if the_cmp == 0:
+                return &self.records[mid]
+            elif the_cmp < 0:
+                lo = mid + 1
+            else:
+                hi = mid
+        return NULL
+
+    def __contains__(self, key):
+        cdef char sha1[20]
+        cdef gc_chk_sha1_record *record
+        if _key_to_sha1(key, sha1):
+            # If it isn't a sha1 key, then it won't be in this leaf node
+            record = self._lookup_record(sha1)
+            if record != NULL:
+                self.last_key = key
+                self.last_record = record
+                return True
+        return False
+
+    def __getitem__(self, key):
+        cdef char sha1[20]
+        cdef gc_chk_sha1_record *record
+        record = NULL
+        if self.last_record != NULL and key is self.last_key:
+            record = self.last_record
+        elif _key_to_sha1(key, sha1):
+            record = self._lookup_record(sha1)
+        if record == NULL:
+            raise KeyError('key %r is not present' % (key,))
+        return self._record_to_value_and_refs(record)
+
+    def __len__(self):
+        return self.num_records
+
+    def all_keys(self):
+        cdef int i
+        result = []
+        for i from 0 <= i < self.num_records:
+            PyList_Append(result, _sha1_to_key(self.records[i].sha1))
+        return result
+
+    def all_items(self):
+        cdef int i
+        result = []
+        for i from 0 <= i < self.num_records:
+            item = self._record_to_item(&self.records[i])
+            PyList_Append(result, item)
+        return result
+
+    cdef int _count_records(self, char *c_content, char *c_end): # cannot_raise
+        """Count how many records are in this section."""
+        cdef char *c_cur
+        cdef int num_records
+
+        c_cur = c_content
+        num_records = 0
+        while c_cur != NULL and c_cur < c_end:
+            c_cur = <char *>memchr(c_cur, c'\n', c_end - c_cur);
+            if c_cur == NULL:
+                break
+            c_cur = c_cur + 1
+            num_records = num_records + 1
+        return num_records
+
+    cdef _parse_bytes(self, bytes):
+        """Parse the string 'bytes' into content."""
+        cdef char *c_bytes
+        cdef char *c_cur
+        cdef char *c_end
+        cdef Py_ssize_t n_bytes
+        cdef int num_records
+        cdef int entry
+        cdef gc_chk_sha1_record *cur_record
+
+        if not PyString_CheckExact(bytes):
+            raise TypeError('We only support parsing plain 8-bit strings.')
+        # Pass 1, count how many records there will be
+        n_bytes = PyString_GET_SIZE(bytes)
+        c_bytes = PyString_AS_STRING(bytes)
+        c_end = c_bytes + n_bytes
+        if strncmp(c_bytes, 'type=leaf\n', 10):
+            raise ValueError("bytes did not start with 'type=leaf\\n': %r"
+                             % (bytes[:10],))
+        c_cur = c_bytes + 10
+        num_records = self._count_records(c_cur, c_end)
+        # Now allocate the memory for these items, and go to town
+        self.records = <gc_chk_sha1_record*>PyMem_Malloc(num_records *
+            (sizeof(unsigned short) + sizeof(gc_chk_sha1_record)))
+        self.num_records = num_records
+        cur_record = self.records
+        entry = 0
+        while c_cur != NULL and c_cur < c_end and entry < num_records:
+            c_cur = self._parse_one_entry(c_cur, c_end, cur_record)
+            cur_record = cur_record + 1
+            entry = entry + 1
+        if (entry != self.num_records
+            or c_cur != c_end
+            or cur_record != self.records + self.num_records):
+            raise ValueError('Something went wrong while parsing.')
+        # Pass 3: build the offset map
+        self._compute_common()
+
+    cdef char *_parse_one_entry(self, char *c_cur, char *c_end,
+                                gc_chk_sha1_record *cur_record) except NULL:
+        """Read a single sha record from the bytes.
+        :param c_cur: The pointer to the start of bytes
+        :param cur_record: 
+        """
+        cdef char *c_next
+        if strncmp(c_cur, 'sha1:', 5):
+            raise ValueError('line did not start with sha1: %r'
+                % (safe_string_from_size(c_cur, 10),))
+        c_cur = c_cur + 5
+        c_next = <char *>memchr(c_cur, c'\0', c_end - c_cur)
+        if c_next == NULL or (c_next - c_cur != 40):
+            raise ValueError('Line did not contain 40 hex bytes')
+        if not _unhexlify_sha1(c_cur, cur_record.sha1):
+            raise ValueError('We failed to unhexlify')
+        c_cur = c_next + 1
+        if c_cur[0] != c'\0':
+            raise ValueError('only 1 null, not 2 as expected')
+        c_cur = c_cur + 1
+        cur_record.block_offset = strtoll(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c' ':
+            raise ValueError('Failed to parse block offset')
+        c_cur = c_next + 1
+        cur_record.block_length = strtoul(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c' ':
+            raise ValueError('Failed to parse block length')
+        c_cur = c_next + 1
+        cur_record.record_start = strtoul(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c' ':
+            raise ValueError('Failed to parse block length')
+        c_cur = c_next + 1
+        cur_record.record_end = strtoul(c_cur, &c_next, 10)
+        if c_cur == c_next or c_next[0] != c'\n':
+            raise ValueError('Failed to parse record end')
+        c_cur = c_next + 1
+        return c_cur
+
+    cdef int _offset_for_sha1(self, char *sha1) except -1:
+        """Find the first interesting 8-bits of this sha1."""
+        cdef int this_offset
+        cdef unsigned int as_uint
+        as_uint = _sha1_to_uint(sha1)
+        this_offset = (as_uint >> self.common_shift) & 0xFF
+        return this_offset
+
+    def _get_offset_for_sha1(self, sha1):
+        return self._offset_for_sha1(PyString_AS_STRING(sha1))
+
+    cdef _compute_common(self):
+        cdef unsigned int first
+        cdef unsigned int this
+        cdef unsigned int common_mask
+        cdef unsigned char common_shift
+        cdef int i
+        cdef int offset, this_offset
+        cdef int max_offset
+        # The idea with the offset map is that we should be able to quickly
+        # jump to the key that matches a gives sha1. We know that the keys are
+        # in sorted order, and we know that a lot of the prefix is going to be
+        # the same across them.
+        # By XORing the records together, we can determine what bits are set in
+        # all of them
+        if self.num_records < 2:
+            # Everything is in common if you have 0 or 1 leaves
+            # So we'll always just shift to the first byte
+            self.common_shift = 24
+        else:
+            common_mask = 0xFFFFFFFF
+            first = _sha1_to_uint(self.records[0].sha1)
+            for i from 0 < i < self.num_records:
+                this = _sha1_to_uint(self.records[i].sha1)
+                common_mask = (~(first ^ this)) & common_mask
+            common_shift = 24
+            while common_mask & 0x80000000 and common_shift > 0:
+                common_mask = common_mask << 1
+                common_shift = common_shift - 1
+            self.common_shift = common_shift
+        offset = 0
+        max_offset = self.num_records
+        # We cap this loop at 254 records. All the other offsets just get
+        # filled with 0xff as the singleton saying 'too many'.
+        # It means that if we have >255 records we have to bisect the second
+        # half of the list, but this is going to be very rare in practice.
+        if max_offset > 255:
+            max_offset = 255
+        for i from 0 <= i < max_offset:
+            this_offset = self._offset_for_sha1(self.records[i].sha1)
+            while offset <= this_offset:
+                self.offsets[offset] = i
+                offset = offset + 1
+        while offset < 257:
+            self.offsets[offset] = max_offset
+            offset = offset + 1
+
+    def _get_offsets(self):
+        cdef int i
+        result = []
+        for i from 0 <= i < 257:
+            PyList_Append(result, self.offsets[i])
+        return result
+
+
+def _parse_into_chk(bytes, key_length, ref_list_length):
+    """Parse into a format optimized for chk records."""
+    assert key_length == 1
+    assert ref_list_length == 0
+    return GCCHKSHA1LeafNode(bytes)
+
+
 def _flatten_node(node, reference_lists):
     """Convert a node into the serialized form.
 

=== modified file 'bzrlib/_chk_map_pyx.pyx'
--- bzrlib/_chk_map_pyx.pyx	2010-05-16 15:18:43 +0000
+++ bzrlib/_chk_map_pyx.pyx	2010-10-08 04:25:10 +0000
@@ -413,7 +413,7 @@
     cdef Py_ssize_t byte_size, pos, file_id_len
 
     if not PyString_CheckExact(bytes):
-        raise TypeError('bytes must be a string')
+        raise TypeError('bytes must be a string, got %r' % (type(bytes),))
     byte_str = PyString_AS_STRING(bytes)
     byte_size = PyString_GET_SIZE(bytes)
     byte_end = byte_str + byte_size
@@ -421,7 +421,8 @@
     if cur_end == NULL:
         raise ValueError('No kind section found.')
     if cur_end[1] != c' ':
-        raise ValueError('Kind section should end with ": "')
+        raise ValueError(
+            'Kind section should end with ": ", got %r' % str(cur_end[:2],))
     file_id_str = cur_end + 2
     # file_id is now the data up until the next newline
     cur_end = <char*>memchr(file_id_str, c'\n', byte_end - file_id_str)

=== modified file 'bzrlib/_dirstate_helpers_py.py'
--- bzrlib/_dirstate_helpers_py.py	2009-06-22 14:32:48 +0000
+++ bzrlib/_dirstate_helpers_py.py	2011-09-28 16:50:40 +0000
@@ -16,7 +16,9 @@
 
 """Python implementations of Dirstate Helper functions."""
 
+import binascii
 import os
+import struct
 
 # We cannot import the dirstate module, because it loads this module
 # All we really need is the IN_MEMORY_MODIFIED constant
@@ -24,6 +26,30 @@
 from bzrlib.dirstate import DirState
 
 
+def pack_stat(st, _b64=binascii.b2a_base64, _pack=struct.Struct('>6L').pack):
+    """Convert stat values into a packed representation
+
+    Not all of the fields from the stat included are strictly needed, and by
+    just encoding the mtime and mode a slight speed increase could be gained.
+    However, using the pyrex version instead is a bigger win.
+    """
+    # base64 encoding always adds a final newline, so strip it off
+    return _b64(_pack(st.st_size & 0xFFFFFFFF, int(st.st_mtime) & 0xFFFFFFFF,
+        int(st.st_ctime) & 0xFFFFFFFF, st.st_dev & 0xFFFFFFFF,
+        st.st_ino & 0xFFFFFFFF, st.st_mode))[:-1]
+
+
+def _unpack_stat(packed_stat):
+    """Turn a packed_stat back into the stat fields.
+
+    This is meant as a debugging tool, should not be used in real code.
+    """
+    (st_size, st_mtime, st_ctime, st_dev, st_ino,
+     st_mode) = struct.unpack('>6L', binascii.a2b_base64(packed_stat))
+    return dict(st_size=st_size, st_mtime=st_mtime, st_ctime=st_ctime,
+                st_dev=st_dev, st_ino=st_ino, st_mode=st_mode)
+
+
 def _bisect_path_left(paths, path):
     """Return the index where to insert path into paths.
 

=== modified file 'bzrlib/_dirstate_helpers_pyx.pyx'
--- bzrlib/_dirstate_helpers_pyx.pyx	2010-05-20 02:57:52 +0000
+++ bzrlib/_dirstate_helpers_pyx.pyx	2011-10-06 16:41:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008, 2010 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -97,6 +97,7 @@
     object PyTuple_GetItem_void_object "PyTuple_GET_ITEM" (void* tpl, int index)
     object PyTuple_GET_ITEM(object tpl, Py_ssize_t index)
 
+    unsigned long PyInt_AsUnsignedLongMask(object number) except? -1
 
     char *PyString_AsString(object p)
     char *PyString_AsString_obj "PyString_AsString" (PyObject *string)
@@ -118,6 +119,11 @@
     # ??? memrchr is a GNU extension :(
     # void *memrchr(void *s, int c, size_t len)
 
+# cimport all of the definitions we will need to access
+from _static_tuple_c cimport import_static_tuple_c, StaticTuple, \
+    StaticTuple_New, StaticTuple_SET_ITEM
+
+import_static_tuple_c()
 
 cdef void* _my_memrchr(void *s, int c, size_t n): # cannot_raise
     # memrchr seems to be a GNU extension, so we have to implement it ourselves
@@ -610,7 +616,8 @@
         :param new_block: This is to let the caller know that it needs to
             create a new directory block to store the next entry.
         """
-        cdef object path_name_file_id_key
+        cdef StaticTuple path_name_file_id_key
+        cdef StaticTuple tmp
         cdef char *entry_size_cstr
         cdef unsigned long int entry_size
         cdef char* executable_cstr
@@ -650,10 +657,20 @@
         # Build up the key that will be used.
         # By using <object>(void *) Pyrex will automatically handle the
         # Py_INCREF that we need.
-        path_name_file_id_key = (<object>p_current_dirname[0],
-                                 self.get_next_str(),
-                                 self.get_next_str(),
-                                )
+        cur_dirname = <object>p_current_dirname[0]
+        # Use StaticTuple_New to pre-allocate, rather than creating a regular
+        # tuple and passing it to the StaticTuple constructor.
+        # path_name_file_id_key = StaticTuple(<object>p_current_dirname[0],
+        #                          self.get_next_str(),
+        #                          self.get_next_str(),
+        #                         )
+        tmp = StaticTuple_New(3)
+        Py_INCREF(cur_dirname); StaticTuple_SET_ITEM(tmp, 0, cur_dirname)
+        cur_basename = self.get_next_str()
+        cur_file_id = self.get_next_str()
+        Py_INCREF(cur_basename); StaticTuple_SET_ITEM(tmp, 1, cur_basename)
+        Py_INCREF(cur_file_id); StaticTuple_SET_ITEM(tmp, 2, cur_file_id)
+        path_name_file_id_key = tmp
 
         # Parse all of the per-tree information. current has the information in
         # the same location as parent trees. The only difference is that 'info'
@@ -677,7 +694,20 @@
             executable_cstr = self.get_next(&cur_size)
             is_executable = (executable_cstr[0] == c'y')
             info = self.get_next_str()
-            PyList_Append(trees, (
+            # TODO: If we want to use StaticTuple_New here we need to be pretty
+            #       careful. We are relying on a bit of Pyrex
+            #       automatic-conversion from 'int' to PyInt, and that doesn't
+            #       play well with the StaticTuple_SET_ITEM macro.
+            #       Timing doesn't (yet) show a worthwile improvement in speed
+            #       versus complexity and maintainability.
+            # tmp = StaticTuple_New(5)
+            # Py_INCREF(minikind); StaticTuple_SET_ITEM(tmp, 0, minikind)
+            # Py_INCREF(fingerprint); StaticTuple_SET_ITEM(tmp, 1, fingerprint)
+            # Py_INCREF(entry_size); StaticTuple_SET_ITEM(tmp, 2, entry_size)
+            # Py_INCREF(is_executable); StaticTuple_SET_ITEM(tmp, 3, is_executable)
+            # Py_INCREF(info); StaticTuple_SET_ITEM(tmp, 4, info)
+            # PyList_Append(trees, tmp)
+            PyList_Append(trees, StaticTuple(
                 minikind,     # minikind
                 fingerprint,  # fingerprint
                 entry_size,   # size
@@ -782,7 +812,6 @@
 _encode = binascii.b2a_base64
 
 
-from struct import pack
 cdef _pack_stat(stat_value):
     """return a string representing the stat value's key fields.
 
@@ -792,16 +821,22 @@
     cdef char result[6*4] # 6 long ints
     cdef int *aliased
     aliased = <int *>result
-    aliased[0] = htonl(stat_value.st_size)
-    aliased[1] = htonl(int(stat_value.st_mtime))
-    aliased[2] = htonl(int(stat_value.st_ctime))
-    aliased[3] = htonl(stat_value.st_dev)
-    aliased[4] = htonl(stat_value.st_ino & 0xFFFFFFFF)
-    aliased[5] = htonl(stat_value.st_mode)
+    aliased[0] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_size))
+    # mtime and ctime will often be floats but get converted to PyInt within
+    aliased[1] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_mtime))
+    aliased[2] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_ctime))
+    aliased[3] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_dev))
+    aliased[4] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_ino))
+    aliased[5] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_mode))
     packed = PyString_FromStringAndSize(result, 6*4)
     return _encode(packed)[:-1]
 
 
+def pack_stat(stat_value):
+    """Convert stat value into a packed representation quickly with pyrex"""
+    return _pack_stat(stat_value)
+
+
 def update_entry(self, entry, abspath, stat_value):
     """Update the entry based on what is actually on disk.
 
@@ -837,6 +872,7 @@
     # _st mode of the compiled stat objects.
     cdef int minikind, saved_minikind
     cdef void * details
+    cdef int worth_saving
     minikind = minikind_from_mode(stat_value.st_mode)
     if 0 == minikind:
         return None
@@ -871,6 +907,7 @@
     # If we have gotten this far, that means that we need to actually
     # process this entry.
     link_or_sha1 = None
+    worth_saving = 1
     if minikind == c'f':
         executable = self._is_executable(stat_value.st_mode,
                                          saved_executable)
@@ -887,10 +924,15 @@
             entry[1][0] = ('f', link_or_sha1, stat_value.st_size,
                            executable, packed_stat)
         else:
-            entry[1][0] = ('f', '', stat_value.st_size,
-                           executable, DirState.NULLSTAT)
+            # This file is not worth caching the sha1. Either it is too new, or
+            # it is newly added. Regardless, the only things we are changing
+            # are derived from the stat, and so are not worth caching. So we do
+            # *not* set the IN_MEMORY_MODIFIED flag. (But we'll save the
+            # updated values if there is *other* data worth saving.)
+            entry[1][0] = ('f', '', stat_value.st_size, executable,
+                           DirState.NULLSTAT)
+            worth_saving = 0
     elif minikind == c'd':
-        link_or_sha1 = None
         entry[1][0] = ('d', '', 0, False, packed_stat)
         if saved_minikind != c'd':
             # This changed from something into a directory. Make sure we
@@ -900,7 +942,17 @@
                 self._get_block_entry_index(entry[0][0], entry[0][1], 0)
             self._ensure_block(block_index, entry_index,
                                pathjoin(entry[0][0], entry[0][1]))
+        else:
+            # Any changes are derived trivially from the stat object, not worth
+            # re-writing a dirstate for just this
+            worth_saving = 0
     elif minikind == c'l':
+        if saved_minikind == c'l':
+            # If the object hasn't changed kind, it isn't worth saving the
+            # dirstate just for a symlink. The default is 'fast symlinks' which
+            # save the target in the inode entry, rather than separately. So to
+            # stat, we've already read everything off disk.
+            worth_saving = 0
         link_or_sha1 = self._read_link(abspath, saved_link_or_sha1)
         if self._cutoff_time is None:
             self._sha_cutoff_time()
@@ -911,7 +963,10 @@
         else:
             entry[1][0] = ('l', '', stat_value.st_size,
                            False, DirState.NULLSTAT)
-    self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+    if worth_saving:
+        # Note, even though _mark_modified will only set
+        # IN_MEMORY_HASH_MODIFIED, it still isn't worth 
+        self._mark_modified([entry])
     return link_or_sha1
 
 
@@ -1744,6 +1799,7 @@
                 advance_entry = -1
                 advance_path = -1
                 result = None
+                changed = None
                 path_handled = 0
                 if current_entry is None:
                     # unversioned -  the check for path_handled when the path

=== modified file 'bzrlib/_groupcompress_pyx.pyx'
--- bzrlib/_groupcompress_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_groupcompress_pyx.pyx	2011-05-16 14:26:54 +0000
@@ -22,6 +22,8 @@
 
 
 cdef extern from "Python.h":
+    ctypedef struct PyObject:
+        pass
     ctypedef int Py_ssize_t # Required for older pyrex versions
     int PyString_CheckExact(object)
     char * PyString_AS_STRING(object)
@@ -44,16 +46,35 @@
         unsigned long agg_offset
     struct delta_index:
         pass
-    delta_index * create_delta_index(source_info *src, delta_index *old) nogil
-    delta_index * create_delta_index_from_delta(source_info *delta,
-                                                delta_index *old) nogil
+    ctypedef enum delta_result:
+        DELTA_OK
+        DELTA_OUT_OF_MEMORY
+        DELTA_INDEX_NEEDED
+        DELTA_SOURCE_EMPTY
+        DELTA_SOURCE_BAD
+        DELTA_BUFFER_EMPTY
+        DELTA_SIZE_TOO_BIG
+    delta_result create_delta_index(source_info *src,
+                                    delta_index *old,
+                                    delta_index **fresh,
+                                    int max_entries) nogil
+    delta_result create_delta_index_from_delta(source_info *delta,
+                                               delta_index *old,
+                                               delta_index **fresh) nogil
     void free_delta_index(delta_index *index) nogil
-    void *create_delta(delta_index *indexes,
-             void *buf, unsigned long bufsize,
-             unsigned long *delta_size, unsigned long max_delta_size) nogil
+    delta_result create_delta(delta_index *indexes,
+                              void *buf, unsigned long bufsize,
+                              unsigned long *delta_size,
+                              unsigned long max_delta_size,
+                              void **delta_data) nogil
     unsigned long get_delta_hdr_size(unsigned char **datap,
                                      unsigned char *top) nogil
+    unsigned long sizeof_delta_index(delta_index *index)
     Py_ssize_t DELTA_SIZE_MIN
+    int get_hash_offset(delta_index *index, int pos, unsigned int *hash_offset)
+    int get_entry_summary(delta_index *index, int pos,
+                          unsigned int *global_offset, unsigned int *hash_val)
+    unsigned int rabin_hash (unsigned char *data)
 
 
 cdef void *safe_malloc(size_t count) except NULL:
@@ -83,6 +104,29 @@
     return DeltaIndex(source)
 
 
+cdef object _translate_delta_failure(delta_result result):
+    if result == DELTA_OUT_OF_MEMORY:
+        return MemoryError("Delta function failed to allocate memory")
+    elif result == DELTA_INDEX_NEEDED:
+        return ValueError("Delta function requires delta_index param")
+    elif result == DELTA_SOURCE_EMPTY:
+        return ValueError("Delta function given empty source_info param")
+    elif result == DELTA_SOURCE_BAD:
+        return RuntimeError("Delta function given invalid source_info param")
+    elif result == DELTA_BUFFER_EMPTY:
+        return ValueError("Delta function given empty buffer params")
+    return AssertionError("Unrecognised delta result code: %d" % result)
+
+
+def _rabin_hash(content):
+    if not PyString_CheckExact(content):
+        raise ValueError('content must be a string')
+    if len(content) < 16:
+        raise ValueError('content must be at least 16 bytes long')
+    # Try to cast it to an int, if it can fit
+    return int(rabin_hash(<unsigned char*>(PyString_AS_STRING(content))))
+
+
 cdef class DeltaIndex:
 
     # We need Pyrex 0.9.8+ to understand a 'list' definition, and this object
@@ -91,20 +135,40 @@
     cdef readonly object _sources
     cdef source_info *_source_infos
     cdef delta_index *_index
+    cdef public unsigned long _source_offset
     cdef readonly unsigned int _max_num_sources
-    cdef public unsigned long _source_offset
+    cdef public int _max_bytes_to_index
 
-    def __init__(self, source=None):
+    def __init__(self, source=None, max_bytes_to_index=None):
         self._sources = []
         self._index = NULL
         self._max_num_sources = 65000
         self._source_infos = <source_info *>safe_malloc(sizeof(source_info)
                                                         * self._max_num_sources)
         self._source_offset = 0
+        self._max_bytes_to_index = 0
+        if max_bytes_to_index is not None:
+            self._max_bytes_to_index = max_bytes_to_index
 
         if source is not None:
             self.add_source(source, 0)
 
+    def __sizeof__(self):
+        # We want to track the _source_infos allocations, but the referenced
+        # void* are actually tracked in _sources itself.
+        # XXX: Cython is capable of doing sizeof(class) and returning the size
+        #      of the underlying struct. Pyrex (<= 0.9.9) refuses, so we need
+        #      to do it manually. *sigh* Note that we might get it wrong
+        #      because of alignment issues.
+        cdef Py_ssize_t size
+        # PyObject start, vtable *, 3 object pointers, 2 C ints
+        size = ((sizeof(PyObject) + sizeof(void*) + 3*sizeof(PyObject*)
+                 + sizeof(unsigned long)
+                 + sizeof(unsigned int))
+                + (sizeof(source_info) * self._max_num_sources)
+                + sizeof_delta_index(self._index))
+        return size
+
     def __repr__(self):
         return '%s(%d, %d)' % (self.__class__.__name__,
             len(self._sources), self._source_offset)
@@ -118,6 +182,44 @@
     def _has_index(self):
         return (self._index != NULL)
 
+    def _dump_index(self):
+        """Dump the pointers in the index.
+
+        This is an arbitrary layout, used for testing. It is not meant to be
+        used in production code.
+
+        :return: (hash_list, entry_list)
+            hash_list   A list of offsets, so hash[i] points to the 'hash
+                        bucket' starting at the given offset and going until
+                        hash[i+1]
+            entry_list  A list of (text_offset, hash_val). text_offset is the
+                        offset in the "source" texts, and hash_val is the RABIN
+                        hash for that offset.
+                        Note that the entry should be in the hash bucket
+                        defined by
+                        hash[(hash_val & mask)] && hash[(hash_val & mask) + 1]
+        """
+        cdef int pos
+        cdef unsigned int text_offset
+        cdef unsigned int hash_val
+        cdef unsigned int hash_offset
+        if self._index == NULL:
+            return None
+        hash_list = []
+        pos = 0
+        while get_hash_offset(self._index, pos, &hash_offset):
+            hash_list.append(int(hash_offset))
+            pos += 1
+        entry_list = []
+        pos = 0
+        while get_entry_summary(self._index, pos, &text_offset, &hash_val):
+            # Map back using 'int' so that we don't get Long everywhere, when
+            # almost everything is <2**31.
+            val = tuple(map(int, [text_offset, hash_val]))
+            entry_list.append(val)
+            pos += 1
+        return hash_list, entry_list
+
     def add_delta_source(self, delta, unadded_bytes):
         """Add a new delta to the source texts.
 
@@ -128,6 +230,7 @@
         cdef char *c_delta
         cdef Py_ssize_t c_delta_size
         cdef delta_index *index
+        cdef delta_result res
         cdef unsigned int source_location
         cdef source_info *src
         cdef unsigned int num_indexes
@@ -146,9 +249,11 @@
         src.size = c_delta_size
         src.agg_offset = self._source_offset + unadded_bytes
         with nogil:
-            index = create_delta_index_from_delta(src, self._index)
+            res = create_delta_index_from_delta(src, self._index, &index)
+        if res != DELTA_OK:
+            raise _translate_delta_failure(res)
         self._source_offset = src.agg_offset + src.size
-        if index != NULL:
+        if index != self._index:
             free_delta_index(self._index)
             self._index = index
 
@@ -158,13 +263,19 @@
         :param source: The text in question, this must be a byte string
         :param unadded_bytes: Assume there are this many bytes that didn't get
             added between this source and the end of the previous source.
+        :param max_pointers: Add no more than this many entries to the index.
+            By default, we sample every 16 bytes, if that would require more
+            than max_entries, we will reduce the sampling rate.
+            A value of 0 means unlimited, None means use the default limit.
         """
         cdef char *c_source
         cdef Py_ssize_t c_source_size
         cdef delta_index *index
+        cdef delta_result res
         cdef unsigned int source_location
         cdef source_info *src
         cdef unsigned int num_indexes
+        cdef int max_num_entries
 
         if not PyString_CheckExact(source):
             raise TypeError('source is not a str')
@@ -187,22 +298,29 @@
         # We delay creating the index on the first insert
         if source_location != 0:
             with nogil:
-                index = create_delta_index(src, self._index)
-            if index != NULL:
+                res = create_delta_index(src, self._index, &index,
+                                         self._max_bytes_to_index)
+            if res != DELTA_OK:
+                raise _translate_delta_failure(res)
+            if index != self._index:
                 free_delta_index(self._index)
                 self._index = index
 
     cdef _populate_first_index(self):
         cdef delta_index *index
+        cdef delta_result res
         if len(self._sources) != 1 or self._index != NULL:
             raise AssertionError('_populate_first_index should only be'
                 ' called when we have a single source and no index yet')
 
-        # We know that self._index is already NULL, so whatever
-        # create_delta_index returns is fine
+        # We know that self._index is already NULL, so create_delta_index
+        # will always create a new index unless there's a malloc failure
         with nogil:
-            self._index = create_delta_index(&self._source_infos[0], NULL)
-        assert self._index != NULL
+            res = create_delta_index(&self._source_infos[0], NULL, &index,
+                                     self._max_bytes_to_index)
+        if res != DELTA_OK:
+            raise _translate_delta_failure(res)
+        self._index = index
 
     cdef _expand_sources(self):
         raise RuntimeError('if we move self._source_infos, then we need to'
@@ -219,6 +337,7 @@
         cdef void * delta
         cdef unsigned long delta_size
         cdef unsigned long c_max_delta_size
+        cdef delta_result res
 
         if self._index == NULL:
             if len(self._sources) == 0:
@@ -237,13 +356,14 @@
         #       allocate the bytes into the final string
         c_max_delta_size = max_delta_size
         with nogil:
-            delta = create_delta(self._index,
-                                 target, target_size,
-                                 &delta_size, c_max_delta_size)
+            res = create_delta(self._index, target, target_size,
+                               &delta_size, c_max_delta_size, &delta)
         result = None
-        if delta:
+        if res == DELTA_OK:
             result = PyString_FromStringAndSize(<char *>delta, delta_size)
             free(delta)
+        elif res != DELTA_SIZE_TOO_BIG:
+            raise _translate_delta_failure(res)
         return result
 
 
@@ -350,8 +470,8 @@
                 # Copy instruction
                 data = _decode_copy_instruction(data, cmd, &cp_off, &cp_size)
                 if (cp_off + cp_size < cp_size or
-                    cp_off + cp_size > source_size or
-                    cp_size > size):
+                    cp_off + cp_size > <unsigned int>source_size or
+                    cp_size > <unsigned int>size):
                     failed = 1
                     break
                 memcpy(out, source + cp_off, cp_size)

=== modified file 'bzrlib/_patiencediff_c.c'
--- bzrlib/_patiencediff_c.c	2010-03-11 04:33:41 +0000
+++ bzrlib/_patiencediff_c.c	2010-12-02 09:46:15 +0000
@@ -47,8 +47,8 @@
 
 
 /* malloc returns NULL on some platforms if you try to allocate nothing,
- * causing <https://bugs.edge.launchpad.net/bzr/+bug/511267> and
- * <https://bugs.edge.launchpad.net/bzr/+bug/331095>.  On glibc it passes, but
+ * causing <https://bugs.launchpad.net/bzr/+bug/511267> and
+ * <https://bugs.launchpad.net/bzr/+bug/331095>.  On glibc it passes, but
  * let's make it fail to aid testing. */
 #define guarded_malloc(x) ( (x) ? malloc(x) : NULL )
 

=== modified file 'bzrlib/_simple_set_pyx.pyx'
--- bzrlib/_simple_set_pyx.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_simple_set_pyx.pyx	2010-08-02 17:16:12 +0000
@@ -115,6 +115,20 @@
             raise MemoryError()
         memset(self._table, 0, n_bytes)
 
+    def __sizeof__(self):
+        # Note: Pyrex doesn't allow sizeof(class) so we re-implement it here.
+        # Bits are:
+        #   1: PyObject
+        #   2: vtable *
+        #   3: 3 Py_ssize_t
+        #   4: PyObject**
+        # Note that we might get alignment, etc, wrong, but at least this is
+        # better than no estimate at all
+        # return sizeof(SimpleSet) + (self._mask + 1) * (sizeof(PyObject*))
+        return (sizeof(PyObject) + sizeof(void*)
+                + 3*sizeof(Py_ssize_t) + sizeof(PyObject**)
+                + (self._mask + 1) * sizeof(PyObject*))
+
     def __dealloc__(self):
         if self._table != NULL:
             PyMem_Free(self._table)

=== modified file 'bzrlib/_static_tuple_c.pxd'
--- bzrlib/_static_tuple_c.pxd	2010-05-11 14:13:31 +0000
+++ bzrlib/_static_tuple_c.pxd	2010-08-04 14:28:10 +0000
@@ -33,6 +33,7 @@
     int import_static_tuple_c() except -1
     StaticTuple StaticTuple_New(Py_ssize_t)
     StaticTuple StaticTuple_Intern(StaticTuple)
+    StaticTuple StaticTuple_FromSequence(object)
 
     # Steals a reference and val must be a valid type, no checking is done
     void StaticTuple_SET_ITEM(StaticTuple key, Py_ssize_t offset, object val)

=== modified file 'bzrlib/_walkdirs_win32.pyx'
--- bzrlib/_walkdirs_win32.pyx	2010-02-17 17:11:16 +0000
+++ bzrlib/_walkdirs_win32.pyx	2011-04-05 12:15:34 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -69,9 +69,13 @@
 
 
 import operator
+import os
 import stat
 
-from bzrlib import osutils, _readdir_py
+from bzrlib import _readdir_py
+
+cdef object osutils
+osutils = None
 
 
 cdef class _Win32Stat:
@@ -170,6 +174,9 @@
 
     def top_prefix_to_starting_dir(self, top, prefix=""):
         """See DirReader.top_prefix_to_starting_dir."""
+        global osutils
+        if osutils is None:
+            from bzrlib import osutils
         return (osutils.safe_utf8(prefix), None, None, None,
                 osutils.safe_unicode(top))
 
@@ -250,3 +257,37 @@
                 #       earlier Exception, so for now, I'm ignoring this
         dirblock.sort(key=operator.itemgetter(1))
         return dirblock
+
+
+def lstat(path):
+    """Equivalent to os.lstat, except match Win32ReadDir._get_stat_value.
+    """
+    return wrap_stat(os.lstat(path))
+
+
+def fstat(fd):
+    """Like os.fstat, except match Win32ReadDir._get_stat_value
+
+    :seealso: wrap_stat
+    """
+    return wrap_stat(os.fstat(fd))
+
+
+def wrap_stat(st):
+    """Return a _Win32Stat object, based on the given stat result.
+
+    On Windows, os.fstat(open(fname).fileno()) != os.lstat(fname). This is
+    generally because os.lstat and os.fstat differ in what they put into st_ino
+    and st_dev. What gets set where seems to also be dependent on the python
+    version. So we always set it to 0 to avoid worrying about it.
+    """
+    cdef _Win32Stat statvalue
+    statvalue = _Win32Stat()
+    statvalue.st_mode = st.st_mode
+    statvalue.st_ctime = st.st_ctime
+    statvalue.st_mtime = st.st_mtime
+    statvalue.st_atime = st.st_atime
+    statvalue._st_size = st.st_size
+    statvalue.st_ino = 0
+    statvalue.st_dev = 0
+    return statvalue

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2010-06-23 08:19:28 +0000
+++ bzrlib/add.py	2011-09-16 12:04:16 +0000
@@ -17,9 +17,13 @@
 """Helper functions for adding files to working trees."""
 
 import sys
-
-import bzrlib.osutils
-
+import os
+
+from bzrlib import (
+    osutils,
+    ui, 
+    )
+from bzrlib.i18n import gettext
 
 class AddAction(object):
     """A class which defines what action to take when adding a file."""
@@ -38,7 +42,7 @@
         if should_print is not None:
             self.should_print = should_print
 
-    def __call__(self, inv, parent_ie, path, kind, _quote=bzrlib.osutils.quotefn):
+    def __call__(self, inv, parent_ie, path, kind, _quote=osutils.quotefn):
         """Add path to inventory.
 
         The default action does nothing.
@@ -48,9 +52,51 @@
         :param kind: The kind of the object being added.
         """
         if self.should_print:
-            self._to_file.write('adding %s\n' % _quote(path.raw_path))
+            self._to_file.write('adding %s\n' % _quote(path))
         return None
 
+    def skip_file(self, tree, path, kind, stat_value = None):
+        """Test whether the given file should be skipped or not.
+        
+        The default action never skips. Note this is only called during
+        recursive adds
+        
+        :param tree: The tree we are working in
+        :param path: The path being added
+        :param kind: The kind of object being added.
+        :param stat: Stat result for this file, if available already
+        :return bool. True if the file should be skipped (not added)
+        """
+        return False
+
+
+class AddWithSkipLargeAction(AddAction):
+    """A class that can decide to skip a file if it's considered too large"""
+
+    # default 20 MB
+    _DEFAULT_MAX_FILE_SIZE = 20000000
+    _optionName = 'add.maximum_file_size'
+    _maxSize = None
+
+    def skip_file(self, tree, path, kind, stat_value = None):
+        if kind != 'file':
+            return False            
+        if self._maxSize is None:
+            config = tree.branch.get_config()
+            self._maxSize = config.get_user_option_as_int_from_SI(
+                self._optionName,  
+                self._DEFAULT_MAX_FILE_SIZE)
+        if stat_value is None:
+            file_size = os.path.getsize(path);
+        else:
+            file_size = stat_value.st_size;
+        if self._maxSize > 0 and file_size > self._maxSize:
+            ui.ui_factory.show_warning(gettext(
+                "skipping {0} (larger than {1} of {2} bytes)").format(
+                path, self._optionName,  self._maxSize))
+            return True
+        return False
+
 
 class AddFromBaseAction(AddAction):
     """This class will try to extract file ids from another tree."""
@@ -68,7 +114,7 @@
         if file_id is not None:
             if self.should_print:
                 self._to_file.write('adding %s w/ file id from %s\n'
-                                    % (path.raw_path, base_path))
+                                    % (path, base_path))
         else:
             # we aren't doing anything special, so let the default
             # reporter happen
@@ -84,12 +130,13 @@
         Else, we look for an entry in the base tree with the same path.
         """
 
-        if (parent_ie.file_id in self.base_tree):
+        if self.base_tree.has_id(parent_ie.file_id):
             base_parent_ie = self.base_tree.inventory[parent_ie.file_id]
-            base_child_ie = base_parent_ie.children.get(path.base_path)
+            base_child_ie = base_parent_ie.children.get(
+                osutils.basename(path))
             if base_child_ie is not None:
                 return (base_child_ie.file_id,
                         self.base_tree.id2path(base_child_ie.file_id))
-        full_base_path = bzrlib.osutils.pathjoin(self.base_path, path.raw_path)
+        full_base_path = osutils.pathjoin(self.base_path, path)
         # This may return None, but it is our last attempt
         return self.base_tree.path2id(full_base_path), full_base_path

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2010-06-17 09:23:19 +0000
+++ bzrlib/annotate.py	2011-06-27 15:52:06 +0000
@@ -41,9 +41,17 @@
     )
 from bzrlib.config import extract_email_address
 from bzrlib.repository import _strip_NULL_ghosts
-from bzrlib.revision import CURRENT_REVISION, Revision
-
-
+from bzrlib.revision import (
+    CURRENT_REVISION,
+    Revision,
+    )
+from bzrlib.symbol_versioning import (
+    deprecated_function,
+    deprecated_in,
+    )
+
+
+@deprecated_function(deprecated_in((2, 4, 0)))
 def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
                   to_file=None, show_ids=False):
     """Annotate file_id at revision rev_id in branch.
@@ -60,21 +68,13 @@
         used.
     :param show_ids: Show revision ids in the annotation output.
     """
-    if to_file is None:
-        to_file = sys.stdout
-
-    # Handle the show_ids case
-    annotations = _annotations(branch.repository, file_id, rev_id)
-    if show_ids:
-        return _show_id_annotations(annotations, to_file, full)
-
-    # Calculate the lengths of the various columns
-    annotation = list(_expand_annotations(annotations, branch))
-    _print_annotations(annotation, verbose, to_file, full)
+    tree = branch.repository.revision_tree(rev_id)
+    annotate_file_tree(tree, file_id, to_file, verbose=verbose,
+        full=full, show_ids=show_ids, branch=branch)
 
 
 def annotate_file_tree(tree, file_id, to_file, verbose=False, full=False,
-    show_ids=False):
+    show_ids=False, branch=None):
     """Annotate file_id in a tree.
 
     The tree should already be read_locked() when annotate_file_tree is called.
@@ -86,25 +86,34 @@
         reasonable text width.
     :param full: XXXX Not sure what this does.
     :param show_ids: Show revision ids in the annotation output.
+    :param branch: Branch to use for revision revno lookups
     """
-    rev_id = tree.last_revision()
-    branch = tree.branch
+    if branch is None:
+        branch = tree.branch
+    if to_file is None:
+        to_file = sys.stdout
 
     # Handle the show_ids case
     annotations = list(tree.annotate_iter(file_id))
     if show_ids:
         return _show_id_annotations(annotations, to_file, full)
 
-    # Create a virtual revision to represent the current tree state.
-    # Should get some more pending commit attributes, like pending tags,
-    # bugfixes etc.
-    current_rev = Revision(CURRENT_REVISION)
-    current_rev.parent_ids = tree.get_parent_ids()
-    current_rev.committer = tree.branch.get_config().username()
-    current_rev.message = "?"
-    current_rev.timestamp = round(time.time(), 3)
-    current_rev.timezone = osutils.local_time_offset()
-    annotation = list(_expand_annotations(annotations, tree.branch,
+    if not getattr(tree, "get_revision_id", False):
+        # Create a virtual revision to represent the current tree state.
+        # Should get some more pending commit attributes, like pending tags,
+        # bugfixes etc.
+        current_rev = Revision(CURRENT_REVISION)
+        current_rev.parent_ids = tree.get_parent_ids()
+        try:
+            current_rev.committer = branch.get_config().username()
+        except errors.NoWhoami:
+            current_rev.committer = 'local user'
+        current_rev.message = "?"
+        current_rev.timestamp = round(time.time(), 3)
+        current_rev.timezone = osutils.local_time_offset()
+    else:
+        current_rev = None
+    annotation = list(_expand_annotations(annotations, branch,
         current_rev))
     _print_annotations(annotation, verbose, to_file, full)
 
@@ -170,13 +179,6 @@
     return
 
 
-def _annotations(repo, file_id, rev_id):
-    """Return the list of (origin_revision_id, line_text) for a revision of a file in a repository."""
-    annotations = repo.texts.annotate((file_id, rev_id))
-    #
-    return [(key[-1], line) for (key, line) in annotations]
-
-
 def _expand_annotations(annotations, branch, current_rev=None):
     """Expand a file's annotations into command line UI ready tuples.
 
@@ -189,8 +191,8 @@
     """
     repository = branch.repository
     if current_rev is not None:
-        # This can probably become a function on MutableTree, get_revno_map there,
-        # or something.
+        # This can probably become a function on MutableTree, get_revno_map
+        # there, or something.
         last_revision = current_rev.revision_id
         # XXX: Partially Cloned from branch, uses the old_get_graph, eep.
         # XXX: The main difficulty is that we need to inject a single new node

=== modified file 'bzrlib/api.py'
--- bzrlib/api.py	2009-03-23 14:59:43 +0000
+++ bzrlib/api.py	2011-04-07 10:36:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,10 +22,7 @@
 """
 
 import bzrlib
-from bzrlib.lazy_import import lazy_import
-lazy_import(globals(), """
 from bzrlib.errors import IncompatibleAPI
-""")
 
 
 def get_current_api_version(object_with_api):

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2010-04-30 11:03:59 +0000
+++ bzrlib/atomicfile.py	2011-07-15 09:04:54 +0000
@@ -20,7 +20,6 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import stat
-import socket
 import warnings
 
 from bzrlib import (
@@ -81,13 +80,6 @@
             if stat.S_IMODE(st.st_mode) != new_mode:
                 os.chmod(self.tmpfilename, new_mode)
 
-    def _get_closed(self):
-        symbol_versioning.warn('AtomicFile.closed deprecated in bzr 0.10',
-                               DeprecationWarning, stacklevel=2)
-        return self._fd is None
-
-    closed = property(_get_closed)
-
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__,
                            self.realfilename)
@@ -119,7 +111,3 @@
         """Discard the file unless already committed."""
         if self._fd is not None:
             self.abort()
-
-    def __del__(self):
-        if self._fd is not None:
-            warnings.warn("%r leaked" % self)

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2010-07-29 11:17:57 +0000
+++ bzrlib/branch.py	2011-10-14 13:56:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,41 +14,46 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import bzrlib.bzrdir
 
 from cStringIO import StringIO
-import sys
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from itertools import chain
+import itertools
 from bzrlib import (
-        bzrdir,
-        cache_utf8,
-        config as _mod_config,
-        debug,
-        errors,
-        lockdir,
-        lockable_files,
-        remote,
-        repository,
-        revision as _mod_revision,
-        rio,
-        symbol_versioning,
-        transport,
-        tsort,
-        ui,
-        urlutils,
-        )
-from bzrlib.config import BranchConfig, TransportConfig
-from bzrlib.repofmt.pack_repo import RepositoryFormatKnitPack5RichRoot
-from bzrlib.tag import (
-    BasicTags,
-    DisabledTags,
+    bzrdir,
+    controldir,
+    cache_utf8,
+    cleanup,
+    config as _mod_config,
+    debug,
+    errors,
+    fetch,
+    graph as _mod_graph,
+    lockdir,
+    lockable_files,
+    remote,
+    repository,
+    revision as _mod_revision,
+    rio,
+    tag as _mod_tag,
+    transport,
+    ui,
+    urlutils,
     )
+from bzrlib.i18n import gettext, ngettext
 """)
 
-from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
-from bzrlib.hooks import HookPoint, Hooks
+from bzrlib import (
+    controldir,
+    )
+from bzrlib.decorators import (
+    needs_read_lock,
+    needs_write_lock,
+    only_raises,
+    )
+from bzrlib.hooks import Hooks
 from bzrlib.inter import InterObject
 from bzrlib.lock import _RelockDebugMixin, LogicalLockResult
 from bzrlib import registry
@@ -59,19 +64,15 @@
 from bzrlib.trace import mutter, mutter_callsite, note, is_quiet
 
 
-BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
-BZR_BRANCH_FORMAT_5 = "Bazaar-NG branch, format 5\n"
-BZR_BRANCH_FORMAT_6 = "Bazaar Branch Format 6 (bzr 0.15)\n"
-
-
-class Branch(bzrdir.ControlComponent):
+class Branch(controldir.ControlComponent):
     """Branch holding a history of revisions.
 
     :ivar base:
         Base directory/url of the branch; using control_url and
         control_transport is more standardized.
-
-    hooks: An instance of BranchHooks.
+    :ivar hooks: An instance of BranchHooks.
+    :ivar _master_branch_cache: cached result of get_master_branch, see
+        _clear_cached_state.
     """
     # this is really an instance variable - FIXME move it there
     # - RBC 20060112
@@ -91,7 +92,9 @@
         self._revision_id_to_revno_cache = None
         self._partial_revision_id_to_revno_cache = {}
         self._partial_revision_history_cache = []
+        self._tags_bytes = None
         self._last_revision_info_cache = None
+        self._master_branch_cache = None
         self._merge_sorted_revisions_cache = None
         self._open_hook()
         hooks = Branch.hooks['open']
@@ -103,6 +106,13 @@
 
     def _activate_fallback_location(self, url):
         """Activate the branch/repository from url as a fallback repository."""
+        for existing_fallback_repo in self.repository._fallback_repositories:
+            if existing_fallback_repo.user_url == url:
+                # This fallback is already configured.  This probably only
+                # happens because ControlDir.sprout is a horrible mess.  To avoid
+                # confusing _unstack we don't add this a second time.
+                mutter('duplicate activation of fallback %r on %r', url, self)
+                return
         repo = self._get_fallback_repository(url)
         if repo.has_same_location(self.repository):
             raise errors.UnstackableLocationError(self.user_url, url)
@@ -163,14 +173,14 @@
         For instance, if the branch is at URL/.bzr/branch,
         Branch.open(URL) -> a Branch instance.
         """
-        control = bzrdir.BzrDir.open(base, _unsupported,
+        control = controldir.ControlDir.open(base, _unsupported,
                                      possible_transports=possible_transports)
         return control.open_branch(unsupported=_unsupported)
 
     @staticmethod
     def open_from_transport(transport, name=None, _unsupported=False):
         """Open the branch rooted at transport"""
-        control = bzrdir.BzrDir.open_from_transport(transport, _unsupported)
+        control = controldir.ControlDir.open_from_transport(transport, _unsupported)
         return control.open_branch(name=name, unsupported=_unsupported)
 
     @staticmethod
@@ -185,7 +195,7 @@
         format, UnknownFormatError or UnsupportedFormatError are raised.
         If there is one, it is returned, along with the unused portion of url.
         """
-        control, relpath = bzrdir.BzrDir.open_containing(url,
+        control, relpath = controldir.ControlDir.open_containing(url,
                                                          possible_transports)
         return control.open_branch(), relpath
 
@@ -205,7 +215,17 @@
 
         :return: A bzrlib.config.BranchConfig.
         """
-        return BranchConfig(self)
+        return _mod_config.BranchConfig(self)
+
+    def get_config_stack(self):
+        """Get a bzrlib.config.BranchStack for this Branch.
+
+        This can then be used to get and set configuration options for the
+        branch.
+
+        :return: A bzrlib.config.BranchStack.
+        """
+        return _mod_config.BranchStack(self)
 
     def _get_config(self):
         """Get the concrete config for just the config in this branch.
@@ -226,6 +246,7 @@
             possible_transports=[self.bzrdir.root_transport])
         return a_branch.repository
 
+    @needs_read_lock
     def _get_tags_bytes(self):
         """Get the bytes of a serialised tags dict.
 
@@ -238,7 +259,9 @@
         :return: The bytes of the tags file.
         :seealso: Branch._set_tags_bytes.
         """
-        return self._transport.get_bytes('tags')
+        if self._tags_bytes is None:
+            self._tags_bytes = self._transport.get_bytes('tags')
+        return self._tags_bytes
 
     def _get_nick(self, local=False, possible_transports=None):
         config = self.get_config()
@@ -435,6 +458,7 @@
             after. If None, the rest of history is included.
         :param stop_rule: if stop_revision_id is not None, the precise rule
             to use for termination:
+
             * 'exclude' - leave the stop revision out of the result (default)
             * 'include' - the stop revision is the last item in the result
             * 'with-merges' - include the stop revision and all of its
@@ -442,6 +466,7 @@
             * 'with-merges-without-common-ancestry' - filter out revisions 
               that are in both ancestries
         :param direction: either 'reverse' or 'forward':
+
             * reverse means return the start_revision_id first, i.e.
               start at the most recent revision and go backwards in history
             * forward returns tuples in the opposite order to reverse.
@@ -491,31 +516,31 @@
         rev_iter = iter(merge_sorted_revisions)
         if start_revision_id is not None:
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id != start_revision_id:
                     continue
                 else:
                     # The decision to include the start or not
                     # depends on the stop_rule if a stop is provided
                     # so pop this node back into the iterator
-                    rev_iter = chain(iter([node]), rev_iter)
+                    rev_iter = itertools.chain(iter([node]), rev_iter)
                     break
         if stop_revision_id is None:
             # Yield everything
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 yield (rev_id, node.merge_depth, node.revno,
                        node.end_of_merge)
         elif stop_rule == 'exclude':
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id == stop_revision_id:
                     return
                 yield (rev_id, node.merge_depth, node.revno,
                        node.end_of_merge)
         elif stop_rule == 'include':
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 yield (rev_id, node.merge_depth, node.revno,
                        node.end_of_merge)
                 if rev_id == stop_revision_id:
@@ -527,7 +552,7 @@
             ancestors = graph.find_unique_ancestors(start_revision_id,
                                                     [stop_revision_id])
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id not in ancestors:
                     continue
                 yield (rev_id, node.merge_depth, node.revno,
@@ -543,7 +568,7 @@
             reached_stop_revision_id = False
             revision_id_whitelist = []
             for node in rev_iter:
-                rev_id = node.key[-1]
+                rev_id = node.key
                 if rev_id == left_parent:
                     # reached the left parent after the stop_revision
                     return
@@ -629,6 +654,14 @@
         """
         raise errors.UpgradeRequired(self.user_url)
 
+    def get_append_revisions_only(self):
+        """Whether it is only possible to append revisions to the history.
+        """
+        if not self._format.supports_set_append_revisions_only():
+            return False
+        return self.get_config(
+            ).get_user_option_as_bool('append_revisions_only')
+
     def set_append_revisions_only(self, enabled):
         if not self._format.supports_set_append_revisions_only():
             raise errors.UpgradeRequired(self.user_url)
@@ -648,31 +681,16 @@
         raise errors.UnsupportedOperation(self.get_reference_info, self)
 
     @needs_write_lock
-    def fetch(self, from_branch, last_revision=None, pb=None):
+    def fetch(self, from_branch, last_revision=None, limit=None):
         """Copy revisions from from_branch into this branch.
 
         :param from_branch: Where to copy from.
         :param last_revision: What revision to stop at (None for at the end
                               of the branch.
-        :param pb: An optional progress bar to use.
+        :param limit: Optional rough limit of revisions to fetch
         :return: None
         """
-        if self.base == from_branch.base:
-            return (0, [])
-        if pb is not None:
-            symbol_versioning.warn(
-                symbol_versioning.deprecated_in((1, 14, 0))
-                % "pb parameter to fetch()")
-        from_branch.lock_read()
-        try:
-            if last_revision is None:
-                last_revision = from_branch.last_revision()
-                last_revision = _mod_revision.ensure_null(last_revision)
-            return self.repository.fetch(from_branch.repository,
-                                         revision_id=last_revision,
-                                         pb=pb)
-        finally:
-            from_branch.unlock()
+        return InterBranch.get(from_branch, self).fetch(last_revision, limit=limit)
 
     def get_bound_location(self):
         """Return the URL of the branch we are bound to.
@@ -689,7 +707,7 @@
 
     def get_commit_builder(self, parents, config=None, timestamp=None,
                            timezone=None, committer=None, revprops=None,
-                           revision_id=None):
+                           revision_id=None, lossy=False):
         """Obtain a CommitBuilder for this branch.
 
         :param parents: Revision ids of the parents of the new revision.
@@ -699,13 +717,16 @@
         :param committer: Optional committer to set for commit.
         :param revprops: Optional dictionary of revision properties.
         :param revision_id: Optional revision id.
+        :param lossy: Whether to discard data that can not be natively
+            represented, when pushing to a foreign VCS 
         """
 
         if config is None:
             config = self.get_config()
 
         return self.repository.get_commit_builder(self, parents, config,
-            timestamp, timezone, committer, revprops, revision_id)
+            timestamp, timezone, committer, revprops, revision_id,
+            lossy)
 
     def get_master_branch(self, possible_transports=None):
         """Return the branch we are bound to.
@@ -714,16 +735,18 @@
         """
         return None
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def get_revision_delta(self, revno):
         """Return the delta for one revision.
 
         The delta is relative to its mainline predecessor, or the
         empty tree for revision 1.
         """
-        rh = self.revision_history()
-        if not (1 <= revno <= len(rh)):
+        try:
+            revid = self.get_rev_id(revno)
+        except errors.NoSuchRevision:
             raise errors.InvalidRevisionNumber(revno)
-        return self.repository.get_revision_delta(rh[revno-1])
+        return self.repository.get_revision_delta(revid)
 
     def get_stacked_on_url(self):
         """Get the URL this branch is stacked against.
@@ -738,8 +761,54 @@
         """Print `file` to stdout."""
         raise NotImplementedError(self.print_file)
 
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def set_revision_history(self, rev_history):
-        raise NotImplementedError(self.set_revision_history)
+        """See Branch.set_revision_history."""
+        self._set_revision_history(rev_history)
+
+    @needs_write_lock
+    def _set_revision_history(self, rev_history):
+        if len(rev_history) == 0:
+            revid = _mod_revision.NULL_REVISION
+        else:
+            revid = rev_history[-1]
+        if rev_history != self._lefthand_history(revid):
+            raise errors.NotLefthandHistory(rev_history)
+        self.set_last_revision_info(len(rev_history), revid)
+        self._cache_revision_history(rev_history)
+        for hook in Branch.hooks['set_rh']:
+            hook(self, rev_history)
+
+    @needs_write_lock
+    def set_last_revision_info(self, revno, revision_id):
+        """Set the last revision of this branch.
+
+        The caller is responsible for checking that the revno is correct
+        for this revision id.
+
+        It may be possible to set the branch last revision to an id not
+        present in the repository.  However, branches can also be
+        configured to check constraints on history, in which case this may not
+        be permitted.
+        """
+        raise NotImplementedError(self.set_last_revision_info)
+
+    @needs_write_lock
+    def generate_revision_history(self, revision_id, last_rev=None,
+                                  other_branch=None):
+        """See Branch.generate_revision_history"""
+        graph = self.repository.get_graph()
+        (last_revno, last_revid) = self.last_revision_info()
+        known_revision_ids = [
+            (last_revid, last_revno),
+            (_mod_revision.NULL_REVISION, 0),
+            ]
+        if last_rev is not None:
+            if not graph.is_ancestor(last_rev, revision_id):
+                # our previous tip is not merged into stop_revision
+                raise errors.DivergedBranches(self, other_branch)
+        revno = graph.find_distance_to_null(revision_id, known_revision_ids)
+        self.set_last_revision_info(revno, revision_id)
 
     @needs_write_lock
     def set_parent(self, url):
@@ -789,12 +858,12 @@
 
     def _unstack(self):
         """Change a branch to be unstacked, copying data as needed.
-        
+
         Don't call this directly, use set_stacked_on_url(None).
         """
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            pb.update("Unstacking")
+            pb.update(gettext("Unstacking"))
             # The basic approach here is to fetch the tip of the branch,
             # including all available ghosts, from the existing stacked
             # repository into a new repository object without the fallbacks. 
@@ -804,7 +873,8 @@
             old_repository = self.repository
             if len(old_repository._fallback_repositories) != 1:
                 raise AssertionError("can't cope with fallback repositories "
-                    "of %r" % (self.repository,))
+                    "of %r (fallbacks: %r)" % (old_repository,
+                        old_repository._fallback_repositories))
             # Open the new repository object.
             # Repositories don't offer an interface to remove fallback
             # repositories today; take the conceptually simpler option and just
@@ -813,7 +883,8 @@
             # stream from one of them to the other.  This does mean doing
             # separate SSH connection setup, but unstacking is not a
             # common operation so it's tolerable.
-            new_bzrdir = bzrdir.BzrDir.open(self.bzrdir.root_transport.base)
+            new_bzrdir = controldir.ControlDir.open(
+                self.bzrdir.root_transport.base)
             new_repository = new_bzrdir.find_repository()
             if new_repository._fallback_repositories:
                 raise AssertionError("didn't expect %r to have "
@@ -858,13 +929,14 @@
                 # XXX: If you unstack a branch while it has a working tree
                 # with a pending merge, the pending-merged revisions will no
                 # longer be present.  You can (probably) revert and remerge.
-                #
-                # XXX: This only fetches up to the tip of the repository; it
-                # doesn't bring across any tags.  That's fairly consistent
-                # with how branch works, but perhaps not ideal.
-                self.repository.fetch(old_repository,
-                    revision_id=self.last_revision(),
-                    find_ghosts=True)
+                try:
+                    tags_to_fetch = set(self.tags.get_reverse_tag_dict())
+                except errors.TagsNotSupported:
+                    tags_to_fetch = set()
+                fetch_spec = _mod_graph.NotInOtherForRevs(self.repository,
+                    old_repository, required_ids=[self.last_revision()],
+                    if_present_ids=tags_to_fetch, find_ghosts=True).execute()
+                self.repository.fetch(old_repository, fetch_spec=fetch_spec)
             finally:
                 old_repository.unlock()
         finally:
@@ -875,8 +947,13 @@
 
         :seealso: Branch._get_tags_bytes.
         """
-        return _run_with_write_locked_target(self, self._transport.put_bytes,
-            'tags', bytes)
+        op = cleanup.OperationWithCleanups(self._set_tags_bytes_locked)
+        op.add_cleanup(self.lock_write().unlock)
+        return op.run_simple(bytes)
+
+    def _set_tags_bytes_locked(self, bytes):
+        self._tags_bytes = bytes
+        return self._transport.put_bytes('tags', bytes)
 
     def _cache_revision_history(self, rev_history):
         """Set the cached revision history to rev_history.
@@ -909,9 +986,11 @@
         self._revision_history_cache = None
         self._revision_id_to_revno_cache = None
         self._last_revision_info_cache = None
+        self._master_branch_cache = None
         self._merge_sorted_revisions_cache = None
         self._partial_revision_history_cache = []
         self._partial_revision_id_to_revno_cache = {}
+        self._tags_bytes = None
 
     def _gen_revision_history(self):
         """Return sequence of revision hashes on to this branch.
@@ -928,6 +1007,7 @@
         """
         raise NotImplementedError(self._gen_revision_history)
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     @needs_read_lock
     def revision_history(self):
         """Return sequence of revision ids on this branch.
@@ -935,6 +1015,9 @@
         This method will cache the revision history for as long as it is safe to
         do so.
         """
+        return self._revision_history()
+
+    def _revision_history(self):
         if 'evil' in debug.debug_flags:
             mutter_callsite(3, "revision_history scales with history.")
         if self._revision_history_cache is not None:
@@ -967,61 +1050,16 @@
         :return: A tuple (revno, revision_id).
         """
         if self._last_revision_info_cache is None:
-            self._last_revision_info_cache = self._last_revision_info()
+            self._last_revision_info_cache = self._read_last_revision_info()
         return self._last_revision_info_cache
 
-    def _last_revision_info(self):
-        rh = self.revision_history()
-        revno = len(rh)
-        if revno:
-            return (revno, rh[-1])
-        else:
-            return (0, _mod_revision.NULL_REVISION)
-
-    @deprecated_method(deprecated_in((1, 6, 0)))
-    def missing_revisions(self, other, stop_revision=None):
-        """Return a list of new revisions that would perfectly fit.
-
-        If self and other have not diverged, return a list of the revisions
-        present in other, but missing from self.
-        """
-        self_history = self.revision_history()
-        self_len = len(self_history)
-        other_history = other.revision_history()
-        other_len = len(other_history)
-        common_index = min(self_len, other_len) -1
-        if common_index >= 0 and \
-            self_history[common_index] != other_history[common_index]:
-            raise errors.DivergedBranches(self, other)
-
-        if stop_revision is None:
-            stop_revision = other_len
-        else:
-            if stop_revision > other_len:
-                raise errors.NoSuchRevision(self, stop_revision)
-        return other_history[self_len:stop_revision]
-
-    def update_revisions(self, other, stop_revision=None, overwrite=False,
-                         graph=None):
-        """Pull in new perfect-fit revisions.
-
-        :param other: Another Branch to pull from
-        :param stop_revision: Updated until the given revision
-        :param overwrite: Always set the branch pointer, rather than checking
-            to see if it is a proper descendant.
-        :param graph: A Graph object that can be used to query history
-            information. This can be None.
-        :return: None
-        """
-        return InterBranch.get(other, self).update_revisions(stop_revision,
-            overwrite, graph)
-
+    def _read_last_revision_info(self):
+        raise NotImplementedError(self._read_last_revision_info)
+
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def import_last_revision_info(self, source_repo, revno, revid):
         """Set the last revision info, importing from another repo if necessary.
 
-        This is used by the bound branch code to upload a revision to
-        the master branch first before updating the tip of the local branch.
-
         :param source_repo: Source repository to optionally fetch from
         :param revno: Revision number of the new tip
         :param revid: Revision id of the new tip
@@ -1030,11 +1068,32 @@
             self.repository.fetch(source_repo, revision_id=revid)
         self.set_last_revision_info(revno, revid)
 
+    def import_last_revision_info_and_tags(self, source, revno, revid,
+                                           lossy=False):
+        """Set the last revision info, importing from another repo if necessary.
+
+        This is used by the bound branch code to upload a revision to
+        the master branch first before updating the tip of the local branch.
+        Revisions referenced by source's tags are also transferred.
+
+        :param source: Source branch to optionally fetch from
+        :param revno: Revision number of the new tip
+        :param revid: Revision id of the new tip
+        :param lossy: Whether to discard metadata that can not be
+            natively represented
+        :return: Tuple with the new revision number and revision id
+            (should only be different from the arguments when lossy=True)
+        """
+        if not self.repository.has_same_location(source.repository):
+            self.fetch(source, revid)
+        self.set_last_revision_info(revno, revid)
+        return (revno, revid)
+
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
         if _mod_revision.is_null(revision_id):
             return 0
-        history = self.revision_history()
+        history = self._revision_history()
         try:
             return history.index(revision_id) + 1
         except ValueError:
@@ -1067,35 +1126,14 @@
             stop_revision=stop_revision,
             possible_transports=possible_transports, *args, **kwargs)
 
-    def push(self, target, overwrite=False, stop_revision=None, *args,
-        **kwargs):
+    def push(self, target, overwrite=False, stop_revision=None, lossy=False,
+            *args, **kwargs):
         """Mirror this branch into target.
 
         This branch is considered to be 'local', having low latency.
         """
         return InterBranch.get(self, target).push(overwrite, stop_revision,
-            *args, **kwargs)
-
-    def lossy_push(self, target, stop_revision=None):
-        """Push deltas into another branch.
-
-        :note: This does not, like push, retain the revision ids from 
-            the source branch and will, rather than adding bzr-specific 
-            metadata, push only those semantics of the revision that can be 
-            natively represented by this branch' VCS.
-
-        :param target: Target branch
-        :param stop_revision: Revision to push, defaults to last revision.
-        :return: BranchPushResult with an extra member revidmap: 
-            A dictionary mapping revision ids from the target branch 
-            to new revision ids in the target branch, for each 
-            revision that was pushed.
-        """
-        inter = InterBranch.get(self, target)
-        lossy_push = getattr(inter, "lossy_push", None)
-        if lossy_push is None:
-            raise errors.LossyPushToSameVCS(self, target)
-        return lossy_push(stop_revision)
+            lossy, *args, **kwargs)
 
     def basis_tree(self):
         """Return `Tree` object for last revision."""
@@ -1256,7 +1294,8 @@
         return result
 
     @needs_read_lock
-    def sprout(self, to_bzrdir, revision_id=None, repository_policy=None):
+    def sprout(self, to_bzrdir, revision_id=None, repository_policy=None,
+            repository=None):
         """Create a new line of development from the branch, into to_bzrdir.
 
         to_bzrdir controls the branch format.
@@ -1267,13 +1306,17 @@
         if (repository_policy is not None and
             repository_policy.requires_stacking()):
             to_bzrdir._format.require_stacking(_skip_repo=True)
-        result = to_bzrdir.create_branch()
+        result = to_bzrdir.create_branch(repository=repository)
         result.lock_write()
         try:
             if repository_policy is not None:
                 repository_policy.configure_branch(result)
             self.copy_content_into(result, revision_id=revision_id)
-            result.set_parent(self.bzrdir.root_transport.base)
+            master_url = self.get_bound_location()
+            if master_url is None:
+                result.set_parent(self.bzrdir.root_transport.base)
+            else:
+                result.set_parent(master_url)
         finally:
             result.unlock()
         return result
@@ -1353,25 +1396,21 @@
         # TODO: We should probably also check that self.revision_history
         # matches the repository for older branch formats.
         # If looking for the code that cross-checks repository parents against
-        # the iter_reverse_revision_history output, that is now a repository
+        # the Graph.iter_lefthand_ancestry output, that is now a repository
         # specific check.
         return result
 
-    def _get_checkout_format(self):
+    def _get_checkout_format(self, lightweight=False):
         """Return the most suitable metadir for a checkout of this branch.
         Weaves are used if this branch's repository uses weaves.
         """
-        if isinstance(self.bzrdir, bzrdir.BzrDirPreSplitOut):
-            from bzrlib.repofmt import weaverepo
-            format = bzrdir.BzrDirMetaFormat1()
-            format.repository_format = weaverepo.RepositoryFormat7()
-        else:
-            format = self.repository.bzrdir.checkout_metadir()
-            format.set_branch_format(self._format)
+        format = self.repository.bzrdir.checkout_metadir()
+        format.set_branch_format(self._format)
         return format
 
     def create_clone_on_transport(self, to_transport, revision_id=None,
-        stacked_on=None, create_prefix=False, use_existing_dir=False):
+        stacked_on=None, create_prefix=False, use_existing_dir=False,
+        no_tree=None):
         """Create a clone of this branch and its bzrdir.
 
         :param to_transport: The transport to clone onto.
@@ -1390,7 +1429,8 @@
             revision_id = self.last_revision()
         dir_to = self.bzrdir.clone_on_transport(to_transport,
             revision_id=revision_id, stacked_on=stacked_on,
-            create_prefix=create_prefix, use_existing_dir=use_existing_dir)
+            create_prefix=create_prefix, use_existing_dir=use_existing_dir,
+            no_tree=no_tree)
         return dir_to.open_branch()
 
     def create_checkout(self, to_location, revision_id=None,
@@ -1401,7 +1441,7 @@
         :param to_location: The url to produce the checkout at
         :param revision_id: The revision to check out
         :param lightweight: If True, produce a lightweight checkout, otherwise,
-        produce a bound branch (heavyweight checkout)
+            produce a bound branch (heavyweight checkout)
         :param accelerator_tree: A tree which can be used for retrieving file
             contents more quickly than the revision tree, i.e. a workingtree.
             The revision tree will be used for cases where accelerator_tree's
@@ -1412,14 +1452,13 @@
         """
         t = transport.get_transport(to_location)
         t.ensure_base()
+        format = self._get_checkout_format(lightweight=lightweight)
         if lightweight:
-            format = self._get_checkout_format()
             checkout = format.initialize_on_transport(t)
             from_branch = BranchReferenceFormat().initialize(checkout, 
                 target_branch=self)
         else:
-            format = self._get_checkout_format()
-            checkout_branch = bzrdir.BzrDir.create_branch_convenience(
+            checkout_branch = controldir.ControlDir.create_branch_convenience(
                 to_location, force_new_tree=False, format=format)
             checkout = checkout_branch.bzrdir
             checkout_branch.bind(self)
@@ -1453,6 +1492,7 @@
 
     def reference_parent(self, file_id, path, possible_transports=None):
         """Return the parent branch for a tree-reference file_id
+
         :param file_id: The file_id of the tree reference
         :param path: The path of the file_id in the tree
         :return: A branch associated with the file_id
@@ -1511,8 +1551,33 @@
         else:
             raise AssertionError("invalid heads: %r" % (heads,))
 
-
-class BranchFormat(object):
+    def heads_to_fetch(self):
+        """Return the heads that must and that should be fetched to copy this
+        branch into another repo.
+
+        :returns: a 2-tuple of (must_fetch, if_present_fetch).  must_fetch is a
+            set of heads that must be fetched.  if_present_fetch is a set of
+            heads that must be fetched if present, but no error is necessary if
+            they are not present.
+        """
+        # For bzr native formats must_fetch is just the tip, and if_present_fetch
+        # are the tags.
+        must_fetch = set([self.last_revision()])
+        if_present_fetch = set()
+        c = self.get_config()
+        include_tags = c.get_user_option_as_bool('branch.fetch_tags',
+                                                 default=False)
+        if include_tags:
+            try:
+                if_present_fetch = set(self.tags.get_reverse_tag_dict())
+            except errors.TagsNotSupported:
+                pass
+        must_fetch.discard(_mod_revision.NULL_REVISION)
+        if_present_fetch.discard(_mod_revision.NULL_REVISION)
+        return must_fetch, if_present_fetch
+
+
+class BranchFormat(controldir.ControlComponentFormat):
     """An encapsulation of the initialization and open routines for a format.
 
     Formats provide three things:
@@ -1521,7 +1586,7 @@
      * an open routine.
 
     Formats are placed in an dict by their format string for reference
-    during branch opening. Its not required that these be instances, they
+    during branch opening. It's not required that these be instances, they
     can be classes themselves with class methods - it simply depends on
     whether state is needed for a given format or not.
 
@@ -1530,14 +1595,6 @@
     object will be created every time regardless.
     """
 
-    _default_format = None
-    """The default format used for new branches."""
-
-    _formats = {}
-    """The known formats."""
-
-    can_set_append_revisions_only = True
-
     def __eq__(self, other):
         return self.__class__ is other.__class__
 
@@ -1545,61 +1602,55 @@
         return not (self == other)
 
     @classmethod
-    def find_format(klass, a_bzrdir, name=None):
-        """Return the format for the branch object in a_bzrdir."""
+    def find_format(klass, controldir, name=None):
+        """Return the format for the branch object in controldir."""
         try:
-            transport = a_bzrdir.get_branch_transport(None, name=name)
+            transport = controldir.get_branch_transport(None, name=name)
             format_string = transport.get_bytes("format")
-            format = klass._formats[format_string]
-            if isinstance(format, MetaDirBranchFormatFactory):
-                return format()
-            return format
+            return format_registry.get(format_string)
         except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
+            raise errors.NotBranchError(path=transport.base, bzrdir=controldir)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string, kind='branch')
 
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def get_default_format(klass):
         """Return the current default format."""
-        return klass._default_format
+        return format_registry.get_default()
 
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def get_formats(klass):
         """Get all the known formats.
 
         Warning: This triggers a load of all lazy registered formats: do not
         use except when that is desireed.
         """
-        result = []
-        for fmt in klass._formats.values():
-            if isinstance(fmt, MetaDirBranchFormatFactory):
-                fmt = fmt()
-            result.append(fmt)
-        return result
+        return format_registry._get_all()
 
-    def get_reference(self, a_bzrdir, name=None):
-        """Get the target reference of the branch in a_bzrdir.
+    def get_reference(self, controldir, name=None):
+        """Get the target reference of the branch in controldir.
 
         format probing must have been completed before calling
         this method - it is assumed that the format of the branch
-        in a_bzrdir is correct.
+        in controldir is correct.
 
-        :param a_bzrdir: The bzrdir to get the branch data from.
+        :param controldir: The controldir to get the branch data from.
         :param name: Name of the colocated branch to fetch
         :return: None if the branch is not a reference branch.
         """
         return None
 
     @classmethod
-    def set_reference(self, a_bzrdir, name, to_branch):
-        """Set the target reference of the branch in a_bzrdir.
+    def set_reference(self, controldir, name, to_branch):
+        """Set the target reference of the branch in controldir.
 
         format probing must have been completed before calling
         this method - it is assumed that the format of the branch
-        in a_bzrdir is correct.
+        in controldir is correct.
 
-        :param a_bzrdir: The bzrdir to set the branch reference for.
+        :param controldir: The controldir to set the branch reference for.
         :param name: Name of colocated branch to set, None for default
         :param to_branch: branch that the checkout is to reference
         """
@@ -1613,62 +1664,18 @@
         """Return the short format description for this format."""
         raise NotImplementedError(self.get_format_description)
 
-    def _run_post_branch_init_hooks(self, a_bzrdir, name, branch):
+    def _run_post_branch_init_hooks(self, controldir, name, branch):
         hooks = Branch.hooks['post_branch_init']
         if not hooks:
             return
-        params = BranchInitHookParams(self, a_bzrdir, name, branch)
+        params = BranchInitHookParams(self, controldir, name, branch)
         for hook in hooks:
             hook(params)
 
-    def _initialize_helper(self, a_bzrdir, utf8_files, name=None,
-                           lock_type='metadir', set_format=True):
-        """Initialize a branch in a bzrdir, with specified files
-
-        :param a_bzrdir: The bzrdir to initialize the branch in
-        :param utf8_files: The files to create as a list of
-            (filename, content) tuples
-        :param name: Name of colocated branch to create, if any
-        :param set_format: If True, set the format with
-            self.get_format_string.  (BzrBranch4 has its format set
-            elsewhere)
-        :return: a branch in this format
-        """
-        mutter('creating branch %r in %s', self, a_bzrdir.user_url)
-        branch_transport = a_bzrdir.get_branch_transport(self, name=name)
-        lock_map = {
-            'metadir': ('lock', lockdir.LockDir),
-            'branch4': ('branch-lock', lockable_files.TransportLock),
-        }
-        lock_name, lock_class = lock_map[lock_type]
-        control_files = lockable_files.LockableFiles(branch_transport,
-            lock_name, lock_class)
-        control_files.create_lock()
-        try:
-            control_files.lock_write()
-        except errors.LockContention:
-            if lock_type != 'branch4':
-                raise
-            lock_taken = False
-        else:
-            lock_taken = True
-        if set_format:
-            utf8_files += [('format', self.get_format_string())]
-        try:
-            for (filename, content) in utf8_files:
-                branch_transport.put_bytes(
-                    filename, content,
-                    mode=a_bzrdir._get_file_mode())
-        finally:
-            if lock_taken:
-                control_files.unlock()
-        branch = self.open(a_bzrdir, name, _found=True)
-        self._run_post_branch_init_hooks(a_bzrdir, name, branch)
-        return branch
-
-    def initialize(self, a_bzrdir, name=None):
-        """Create a branch of this format in a_bzrdir.
-        
+    def initialize(self, controldir, name=None, repository=None,
+                   append_revisions_only=None):
+        """Create a branch of this format in controldir.
+
         :param name: Name of the colocated branch to create.
         """
         raise NotImplementedError(self.initialize)
@@ -1694,7 +1701,7 @@
         Note that it is normal for branch to be a RemoteBranch when using tags
         on a RemoteBranch.
         """
-        return DisabledTags(branch)
+        return _mod_tag.DisabledTags(branch)
 
     def network_name(self):
         """A simple byte string uniquely identifying this format for RPC calls.
@@ -1706,10 +1713,11 @@
         """
         raise NotImplementedError(self.network_name)
 
-    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False):
-        """Return the branch object for a_bzrdir
+    def open(self, controldir, name=None, _found=False, ignore_fallbacks=False,
+            found_repository=None):
+        """Return the branch object for controldir.
 
-        :param a_bzrdir: A BzrDir that contains a branch.
+        :param controldir: A ControlDir that contains a branch.
         :param name: Name of colocated branch to open
         :param _found: a private parameter, do not use it. It is used to
             indicate if format probing has already be done.
@@ -1719,24 +1727,19 @@
         raise NotImplementedError(self.open)
 
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def register_format(klass, format):
         """Register a metadir format.
-        
+
         See MetaDirBranchFormatFactory for the ability to register a format
         without loading the code the format needs until it is actually used.
         """
-        klass._formats[format.get_format_string()] = format
-        # Metadir formats have a network name of their format string, and get
-        # registered as factories.
-        if isinstance(format, MetaDirBranchFormatFactory):
-            network_format_registry.register(format.get_format_string(), format)
-        else:
-            network_format_registry.register(format.get_format_string(),
-                format.__class__)
+        format_registry.register(format)
 
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def set_default_format(klass, format):
-        klass._default_format = format
+        format_registry.set_default(format)
 
     def supports_set_append_revisions_only(self):
         """True if this format supports set_append_revisions_only."""
@@ -1746,9 +1749,14 @@
         """True if this format records a stacked-on branch."""
         return False
 
+    def supports_leaving_lock(self):
+        """True if this format supports leaving locks in place."""
+        return False # by default
+
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def unregister_format(klass, format):
-        del klass._formats[format.get_format_string()]
+        format_registry.remove(format)
 
     def __str__(self):
         return self.get_format_description().rstrip()
@@ -1757,6 +1765,14 @@
         """True if this format supports tags stored in the branch"""
         return False  # by default
 
+    def tags_are_versioned(self):
+        """Whether the tag container for this branch versions tags."""
+        return False
+
+    def supports_tags_referencing_ghosts(self):
+        """True if tags can reference ghost revisions."""
+        return True
+
 
 class MetaDirBranchFormatFactory(registry._LazyObjectGetter):
     """A factory for a BranchFormat object, permitting simple lazy registration.
@@ -1799,26 +1815,26 @@
         These are all empty initially, because by default nothing should get
         notified.
         """
-        Hooks.__init__(self)
-        self.create_hook(HookPoint('set_rh',
+        Hooks.__init__(self, "bzrlib.branch", "Branch.hooks")
+        self.add_hook('set_rh',
             "Invoked whenever the revision history has been set via "
             "set_revision_history. The api signature is (branch, "
             "revision_history), and the branch will be write-locked. "
             "The set_rh hook can be expensive for bzr to trigger, a better "
-            "hook to use is Branch.post_change_branch_tip.", (0, 15), None))
-        self.create_hook(HookPoint('open',
+            "hook to use is Branch.post_change_branch_tip.", (0, 15))
+        self.add_hook('open',
             "Called with the Branch object that has been opened after a "
-            "branch is opened.", (1, 8), None))
-        self.create_hook(HookPoint('post_push',
+            "branch is opened.", (1, 8))
+        self.add_hook('post_push',
             "Called after a push operation completes. post_push is called "
             "with a bzrlib.branch.BranchPushResult object and only runs in the "
-            "bzr client.", (0, 15), None))
-        self.create_hook(HookPoint('post_pull',
+            "bzr client.", (0, 15))
+        self.add_hook('post_pull',
             "Called after a pull operation completes. post_pull is called "
             "with a bzrlib.branch.PullResult object and only runs in the "
-            "bzr client.", (0, 15), None))
-        self.create_hook(HookPoint('pre_commit',
-            "Called after a commit is calculated but before it is is "
+            "bzr client.", (0, 15))
+        self.add_hook('pre_commit',
+            "Called after a commit is calculated but before it is "
             "completed. pre_commit is called with (local, master, old_revno, "
             "old_revid, future_revno, future_revid, tree_delta, future_tree"
             "). old_revid is NULL_REVISION for the first commit to a branch, "
@@ -1826,29 +1842,29 @@
             "basis revision. hooks MUST NOT modify this delta. "
             " future_tree is an in-memory tree obtained from "
             "CommitBuilder.revision_tree() and hooks MUST NOT modify this "
-            "tree.", (0,91), None))
-        self.create_hook(HookPoint('post_commit',
+            "tree.", (0,91))
+        self.add_hook('post_commit',
             "Called in the bzr client after a commit has completed. "
             "post_commit is called with (local, master, old_revno, old_revid, "
             "new_revno, new_revid). old_revid is NULL_REVISION for the first "
-            "commit to a branch.", (0, 15), None))
-        self.create_hook(HookPoint('post_uncommit',
+            "commit to a branch.", (0, 15))
+        self.add_hook('post_uncommit',
             "Called in the bzr client after an uncommit completes. "
             "post_uncommit is called with (local, master, old_revno, "
             "old_revid, new_revno, new_revid) where local is the local branch "
             "or None, master is the target branch, and an empty branch "
-            "receives new_revno of 0, new_revid of None.", (0, 15), None))
-        self.create_hook(HookPoint('pre_change_branch_tip',
+            "receives new_revno of 0, new_revid of None.", (0, 15))
+        self.add_hook('pre_change_branch_tip',
             "Called in bzr client and server before a change to the tip of a "
             "branch is made. pre_change_branch_tip is called with a "
             "bzrlib.branch.ChangeBranchTipParams. Note that push, pull, "
-            "commit, uncommit will all trigger this hook.", (1, 6), None))
-        self.create_hook(HookPoint('post_change_branch_tip',
+            "commit, uncommit will all trigger this hook.", (1, 6))
+        self.add_hook('post_change_branch_tip',
             "Called in bzr client and server after a change to the tip of a "
             "branch is made. post_change_branch_tip is called with a "
             "bzrlib.branch.ChangeBranchTipParams. Note that push, pull, "
-            "commit, uncommit will all trigger this hook.", (1, 4), None))
-        self.create_hook(HookPoint('transform_fallback_location',
+            "commit, uncommit will all trigger this hook.", (1, 4))
+        self.add_hook('transform_fallback_location',
             "Called when a stacked branch is activating its fallback "
             "locations. transform_fallback_location is called with (branch, "
             "url), and should return a new url. Returning the same url "
@@ -1859,23 +1875,23 @@
             "fallback locations have not been activated. When there are "
             "multiple hooks installed for transform_fallback_location, "
             "all are called with the url returned from the previous hook."
-            "The order is however undefined.", (1, 9), None))
-        self.create_hook(HookPoint('automatic_tag_name',
-            "Called to determine an automatic tag name for a revision."
+            "The order is however undefined.", (1, 9))
+        self.add_hook('automatic_tag_name',
+            "Called to determine an automatic tag name for a revision. "
             "automatic_tag_name is called with (branch, revision_id) and "
             "should return a tag name or None if no tag name could be "
             "determined. The first non-None tag name returned will be used.",
-            (2, 2), None))
-        self.create_hook(HookPoint('post_branch_init',
+            (2, 2))
+        self.add_hook('post_branch_init',
             "Called after new branch initialization completes. "
             "post_branch_init is called with a "
             "bzrlib.branch.BranchInitHookParams. "
             "Note that init, branch and checkout (both heavyweight and "
-            "lightweight) will all trigger this hook.", (2, 2), None))
-        self.create_hook(HookPoint('post_switch',
+            "lightweight) will all trigger this hook.", (2, 2))
+        self.add_hook('post_switch',
             "Called after a checkout switches branch. "
             "post_switch is called with a "
-            "bzrlib.branch.SwitchHookParams.", (2, 2), None))
+            "bzrlib.branch.SwitchHookParams.", (2, 2))
 
 
 
@@ -1884,7 +1900,7 @@
 
 
 class ChangeBranchTipParams(object):
-    """Object holding parameters passed to *_change_branch_tip hooks.
+    """Object holding parameters passed to `*_change_branch_tip` hooks.
 
     There are 5 fields that hooks may wish to access:
 
@@ -1922,12 +1938,12 @@
 
 
 class BranchInitHookParams(object):
-    """Object holding parameters passed to *_branch_init hooks.
+    """Object holding parameters passed to `*_branch_init` hooks.
 
     There are 4 fields that hooks may wish to access:
 
     :ivar format: the branch format
-    :ivar bzrdir: the BzrDir where the branch will be/has been initialized
+    :ivar bzrdir: the ControlDir where the branch will be/has been initialized
     :ivar name: name of colocated branch, if any (or None)
     :ivar branch: the branch created
 
@@ -1936,11 +1952,11 @@
     branch, which refer to the original branch.
     """
 
-    def __init__(self, format, a_bzrdir, name, branch):
+    def __init__(self, format, controldir, name, branch):
         """Create a group of BranchInitHook parameters.
 
         :param format: the branch format
-        :param a_bzrdir: the BzrDir where the branch will be/has been
+        :param controldir: the ControlDir where the branch will be/has been
             initialized
         :param name: name of colocated branch, if any (or None)
         :param branch: the branch created
@@ -1950,7 +1966,7 @@
         in branch, which refer to the original branch.
         """
         self.format = format
-        self.bzrdir = a_bzrdir
+        self.bzrdir = controldir
         self.name = name
         self.branch = branch
 
@@ -1958,20 +1974,15 @@
         return self.__dict__ == other.__dict__
 
     def __repr__(self):
-        if self.branch:
-            return "<%s of %s>" % (self.__class__.__name__, self.branch)
-        else:
-            return "<%s of format:%s bzrdir:%s>" % (
-                self.__class__.__name__, self.branch,
-                self.format, self.bzrdir)
+        return "<%s of %s>" % (self.__class__.__name__, self.branch)
 
 
 class SwitchHookParams(object):
-    """Object holding parameters passed to *_switch hooks.
+    """Object holding parameters passed to `*_switch` hooks.
 
     There are 4 fields that hooks may wish to access:
 
-    :ivar control_dir: BzrDir of the checkout to change
+    :ivar control_dir: ControlDir of the checkout to change
     :ivar to_branch: branch that the checkout is to reference
     :ivar force: skip the check for local commits in a heavy checkout
     :ivar revision_id: revision ID to switch to (or None)
@@ -1980,7 +1991,7 @@
     def __init__(self, control_dir, to_branch, force, revision_id):
         """Create a group of SwitchHook parameters.
 
-        :param control_dir: BzrDir of the checkout to change
+        :param control_dir: ControlDir of the checkout to change
         :param to_branch: branch that the checkout is to reference
         :param force: skip the check for local commits in a heavy checkout
         :param revision_id: revision ID to switch to (or None)
@@ -1999,49 +2010,6 @@
             self.revision_id)
 
 
-class BzrBranchFormat4(BranchFormat):
-    """Bzr branch format 4.
-
-    This format has:
-     - a revision-history file.
-     - a branch-lock lock file [ to be shared with the bzrdir ]
-    """
-
-    def get_format_description(self):
-        """See BranchFormat.get_format_description()."""
-        return "Branch format 4"
-
-    def initialize(self, a_bzrdir, name=None):
-        """Create a branch of this format in a_bzrdir."""
-        utf8_files = [('revision-history', ''),
-                      ('branch-name', ''),
-                      ]
-        return self._initialize_helper(a_bzrdir, utf8_files, name=name,
-                                       lock_type='branch4', set_format=False)
-
-    def __init__(self):
-        super(BzrBranchFormat4, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirFormat6()
-
-    def network_name(self):
-        """The network name for this format is the control dirs disk label."""
-        return self._matchingbzrdir.get_format_string()
-
-    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False):
-        """See BranchFormat.open()."""
-        if not _found:
-            # we are being called directly and must probe.
-            raise NotImplementedError
-        return BzrBranch(_format=self,
-                         _control_files=a_bzrdir._control_files,
-                         a_bzrdir=a_bzrdir,
-                         name=name,
-                         _repository=a_bzrdir.open_repository())
-
-    def __str__(self):
-        return "Bazaar-NG branch format 4"
-
-
 class BranchFormatMetadir(BranchFormat):
     """Common logic for meta-dir based branch formats."""
 
@@ -2049,6 +2017,43 @@
         """What class to instantiate on open calls."""
         raise NotImplementedError(self._branch_class)
 
+    def _get_initial_config(self, append_revisions_only=None):
+        if append_revisions_only:
+            return "append_revisions_only = True\n"
+        else:
+            # Avoid writing anything if append_revisions_only is disabled,
+            # as that is the default.
+            return ""
+
+    def _initialize_helper(self, a_bzrdir, utf8_files, name=None,
+                           repository=None):
+        """Initialize a branch in a bzrdir, with specified files
+
+        :param a_bzrdir: The bzrdir to initialize the branch in
+        :param utf8_files: The files to create as a list of
+            (filename, content) tuples
+        :param name: Name of colocated branch to create, if any
+        :return: a branch in this format
+        """
+        mutter('creating branch %r in %s', self, a_bzrdir.user_url)
+        branch_transport = a_bzrdir.get_branch_transport(self, name=name)
+        control_files = lockable_files.LockableFiles(branch_transport,
+            'lock', lockdir.LockDir)
+        control_files.create_lock()
+        control_files.lock_write()
+        try:
+            utf8_files += [('format', self.get_format_string())]
+            for (filename, content) in utf8_files:
+                branch_transport.put_bytes(
+                    filename, content,
+                    mode=a_bzrdir._get_file_mode())
+        finally:
+            control_files.unlock()
+        branch = self.open(a_bzrdir, name, _found=True,
+                found_repository=repository)
+        self._run_post_branch_init_hooks(a_bzrdir, name, branch)
+        return branch
+
     def network_name(self):
         """A simple byte string uniquely identifying this format for RPC calls.
 
@@ -2056,7 +2061,8 @@
         """
         return self.get_format_string()
 
-    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False):
+    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
+            found_repository=None):
         """See BranchFormat.open()."""
         if not _found:
             format = BranchFormat.find_format(a_bzrdir, name=name)
@@ -2067,23 +2073,29 @@
         try:
             control_files = lockable_files.LockableFiles(transport, 'lock',
                                                          lockdir.LockDir)
+            if found_repository is None:
+                found_repository = a_bzrdir.find_repository()
             return self._branch_class()(_format=self,
                               _control_files=control_files,
                               name=name,
                               a_bzrdir=a_bzrdir,
-                              _repository=a_bzrdir.find_repository(),
+                              _repository=found_repository,
                               ignore_fallbacks=ignore_fallbacks)
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
 
-    def __init__(self):
-        super(BranchFormatMetadir, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
-        self._matchingbzrdir.set_branch_format(self)
+    @property
+    def _matchingbzrdir(self):
+        ret = bzrdir.BzrDirMetaFormat1()
+        ret.set_branch_format(self)
+        return ret
 
     def supports_tags(self):
         return True
 
+    def supports_leaving_lock(self):
+        return True
+
 
 class BzrBranchFormat5(BranchFormatMetadir):
     """Bzr branch format 5.
@@ -2109,12 +2121,15 @@
         """See BranchFormat.get_format_description()."""
         return "Branch format 5"
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
+        if append_revisions_only:
+            raise errors.UpgradeRequired(a_bzrdir.user_url)
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
-        return self._initialize_helper(a_bzrdir, utf8_files, name)
+        return self._initialize_helper(a_bzrdir, utf8_files, name, repository)
 
     def supports_tags(self):
         return False
@@ -2142,17 +2157,19 @@
         """See BranchFormat.get_format_description()."""
         return "Branch format 6"
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         utf8_files = [('last-revision', '0 null:\n'),
-                      ('branch.conf', ''),
+                      ('branch.conf',
+                          self._get_initial_config(append_revisions_only)),
                       ('tags', ''),
                       ]
-        return self._initialize_helper(a_bzrdir, utf8_files, name)
+        return self._initialize_helper(a_bzrdir, utf8_files, name, repository)
 
     def make_tags(self, branch):
         """See bzrlib.branch.BranchFormat.make_tags()."""
-        return BasicTags(branch)
+        return _mod_tag.BasicTags(branch)
 
     def supports_set_append_revisions_only(self):
         return True
@@ -2172,23 +2189,20 @@
         """See BranchFormat.get_format_description()."""
         return "Branch format 8"
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         utf8_files = [('last-revision', '0 null:\n'),
-                      ('branch.conf', ''),
+                      ('branch.conf',
+                          self._get_initial_config(append_revisions_only)),
                       ('tags', ''),
                       ('references', '')
                       ]
-        return self._initialize_helper(a_bzrdir, utf8_files, name)
-
-    def __init__(self):
-        super(BzrBranchFormat8, self).__init__()
-        self._matchingbzrdir.repository_format = \
-            RepositoryFormatKnitPack5RichRoot()
+        return self._initialize_helper(a_bzrdir, utf8_files, name, repository)
 
     def make_tags(self, branch):
         """See bzrlib.branch.BranchFormat.make_tags()."""
-        return BasicTags(branch)
+        return _mod_tag.BasicTags(branch)
 
     def supports_set_append_revisions_only(self):
         return True
@@ -2199,7 +2213,7 @@
     supports_reference_locations = True
 
 
-class BzrBranchFormat7(BzrBranchFormat8):
+class BzrBranchFormat7(BranchFormatMetadir):
     """Branch format with last-revision, tags, and a stacked location pointer.
 
     The stacked location pointer is passed down to the repository and requires
@@ -2208,13 +2222,15 @@
     This format was introduced in bzr 1.6.
     """
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         utf8_files = [('last-revision', '0 null:\n'),
-                      ('branch.conf', ''),
+                      ('branch.conf',
+                          self._get_initial_config(append_revisions_only)),
                       ('tags', ''),
                       ]
-        return self._initialize_helper(a_bzrdir, utf8_files, name)
+        return self._initialize_helper(a_bzrdir, utf8_files, name, repository)
 
     def _branch_class(self):
         return BzrBranch7
@@ -2230,10 +2246,17 @@
     def supports_set_append_revisions_only(self):
         return True
 
+    def supports_stacking(self):
+        return True
+
+    def make_tags(self, branch):
+        """See bzrlib.branch.BranchFormat.make_tags()."""
+        return _mod_tag.BasicTags(branch)
+
     supports_reference_locations = False
 
 
-class BranchReferenceFormat(BranchFormat):
+class BranchReferenceFormat(BranchFormatMetadir):
     """Bzr branch reference format.
 
     Branch references are used in implementing checkouts, they
@@ -2262,13 +2285,16 @@
         transport = a_bzrdir.get_branch_transport(None, name=name)
         location = transport.put_bytes('location', to_branch.base)
 
-    def initialize(self, a_bzrdir, name=None, target_branch=None):
+    def initialize(self, a_bzrdir, name=None, target_branch=None,
+            repository=None, append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         if target_branch is None:
             # this format does not implement branch itself, thus the implicit
             # creation contract must see it as uninitializable
             raise errors.UninitializableFormat(self)
         mutter('creating branch reference in %s', a_bzrdir.user_url)
+        if a_bzrdir._format.fixed_components:
+            raise errors.IncompatibleFormat(self, a_bzrdir._format)
         branch_transport = a_bzrdir.get_branch_transport(self, name=name)
         branch_transport.put_bytes('location',
             target_branch.bzrdir.user_url)
@@ -2279,11 +2305,6 @@
         self._run_post_branch_init_hooks(a_bzrdir, name, branch)
         return branch
 
-    def __init__(self):
-        super(BranchReferenceFormat, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
-        self._matchingbzrdir.set_branch_format(self)
-
     def _make_reference_clone_function(format, a_branch):
         """Create a clone() routine for a branch dynamically."""
         def clone(to_bzrdir, revision_id=None,
@@ -2296,7 +2317,8 @@
         return clone
 
     def open(self, a_bzrdir, name=None, _found=False, location=None,
-             possible_transports=None, ignore_fallbacks=False):
+             possible_transports=None, ignore_fallbacks=False,
+             found_repository=None):
         """Return the branch that the branch reference in a_bzrdir points at.
 
         :param a_bzrdir: A BzrDir that contains a branch.
@@ -2317,7 +2339,7 @@
                     (format, self))
         if location is None:
             location = self.get_reference(a_bzrdir, name)
-        real_bzrdir = bzrdir.BzrDir.open(
+        real_bzrdir = controldir.ControlDir.open(
             location, possible_transports=possible_transports)
         result = real_bzrdir.open_branch(name=name, 
             ignore_fallbacks=ignore_fallbacks)
@@ -2333,6 +2355,20 @@
         return result
 
 
+class BranchFormatRegistry(controldir.ControlComponentFormatRegistry):
+    """Branch format registry."""
+
+    def __init__(self, other_registry=None):
+        super(BranchFormatRegistry, self).__init__(other_registry)
+        self._default_format = None
+
+    def set_default(self, format):
+        self._default_format = format
+
+    def get_default(self):
+        return self._default_format
+
+
 network_format_registry = registry.FormatRegistry()
 """Registry of formats indexed by their network name.
 
@@ -2341,6 +2377,8 @@
 BranchFormat.network_name() for more detail.
 """
 
+format_registry = BranchFormatRegistry(network_format_registry)
+
 
 # formats which have no format string are not discoverable
 # and not independently creatable, so are not registered.
@@ -2348,16 +2386,12 @@
 __format6 = BzrBranchFormat6()
 __format7 = BzrBranchFormat7()
 __format8 = BzrBranchFormat8()
-BranchFormat.register_format(__format5)
-BranchFormat.register_format(BranchReferenceFormat())
-BranchFormat.register_format(__format6)
-BranchFormat.register_format(__format7)
-BranchFormat.register_format(__format8)
-BranchFormat.set_default_format(__format7)
-_legacy_formats = [BzrBranchFormat4(),
-    ]
-network_format_registry.register(
-    _legacy_formats[0].network_name(), _legacy_formats[0].__class__)
+format_registry.register(__format5)
+format_registry.register(BranchReferenceFormat())
+format_registry.register(__format6)
+format_registry.register(__format7)
+format_registry.register(__format8)
+format_registry.set_default(__format7)
 
 
 class BranchWriteLockResult(LogicalLockResult):
@@ -2430,7 +2464,7 @@
     base = property(_get_base, doc="The URL for the root of this branch.")
 
     def _get_config(self):
-        return TransportConfig(self._transport, 'branch.conf')
+        return _mod_config.TransportConfig(self._transport, 'branch.conf')
 
     def is_locked(self):
         return self.control_files.is_locked()
@@ -2511,110 +2545,19 @@
         """See Branch.print_file."""
         return self.repository.print_file(file, revision_id)
 
-    def _write_revision_history(self, history):
-        """Factored out of set_revision_history.
-
-        This performs the actual writing to disk.
-        It is intended to be called by BzrBranch5.set_revision_history."""
-        self._transport.put_bytes(
-            'revision-history', '\n'.join(history),
-            mode=self.bzrdir._get_file_mode())
-
-    @needs_write_lock
-    def set_revision_history(self, rev_history):
-        """See Branch.set_revision_history."""
-        if 'evil' in debug.debug_flags:
-            mutter_callsite(3, "set_revision_history scales with history.")
-        check_not_reserved_id = _mod_revision.check_not_reserved_id
-        for rev_id in rev_history:
-            check_not_reserved_id(rev_id)
-        if Branch.hooks['post_change_branch_tip']:
-            # Don't calculate the last_revision_info() if there are no hooks
-            # that will use it.
-            old_revno, old_revid = self.last_revision_info()
-        if len(rev_history) == 0:
-            revid = _mod_revision.NULL_REVISION
-        else:
-            revid = rev_history[-1]
-        self._run_pre_change_branch_tip_hooks(len(rev_history), revid)
-        self._write_revision_history(rev_history)
-        self._clear_cached_state()
-        self._cache_revision_history(rev_history)
-        for hook in Branch.hooks['set_rh']:
-            hook(self, rev_history)
-        if Branch.hooks['post_change_branch_tip']:
-            self._run_post_change_branch_tip_hooks(old_revno, old_revid)
-
-    def _synchronize_history(self, destination, revision_id):
-        """Synchronize last revision and revision history between branches.
-
-        This version is most efficient when the destination is also a
-        BzrBranch5, but works for BzrBranch6 as long as the revision
-        history is the true lefthand parent history, and all of the revisions
-        are in the destination's repository.  If not, set_revision_history
-        will fail.
-
-        :param destination: The branch to copy the history into
-        :param revision_id: The revision-id to truncate history at.  May
-          be None to copy complete history.
-        """
-        if not isinstance(destination._format, BzrBranchFormat5):
-            super(BzrBranch, self)._synchronize_history(
-                destination, revision_id)
-            return
-        if revision_id == _mod_revision.NULL_REVISION:
-            new_history = []
-        else:
-            new_history = self.revision_history()
-        if revision_id is not None and new_history != []:
-            try:
-                new_history = new_history[:new_history.index(revision_id) + 1]
-            except ValueError:
-                rev = self.repository.get_revision(revision_id)
-                new_history = rev.get_history(self.repository)[1:]
-        destination.set_revision_history(new_history)
-
     @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
-        """Set the last revision of this branch.
-
-        The caller is responsible for checking that the revno is correct
-        for this revision id.
-
-        It may be possible to set the branch last revision to an id not
-        present in the repository.  However, branches can also be
-        configured to check constraints on history, in which case this may not
-        be permitted.
-        """
+        if not revision_id or not isinstance(revision_id, basestring):
+            raise errors.InvalidRevisionId(revision_id=revision_id, branch=self)
         revision_id = _mod_revision.ensure_null(revision_id)
-        # this old format stores the full history, but this api doesn't
-        # provide it, so we must generate, and might as well check it's
-        # correct
-        history = self._lefthand_history(revision_id)
-        if len(history) != revno:
-            raise AssertionError('%d != %d' % (len(history), revno))
-        self.set_revision_history(history)
-
-    def _gen_revision_history(self):
-        history = self._transport.get_bytes('revision-history').split('\n')
-        if history[-1:] == ['']:
-            # There shouldn't be a trailing newline, but just in case.
-            history.pop()
-        return history
-
-    @needs_write_lock
-    def generate_revision_history(self, revision_id, last_rev=None,
-        other_branch=None):
-        """Create a new revision history that will finish with revision_id.
-
-        :param revision_id: the new tip to use.
-        :param last_rev: The previous last_revision. If not None, then this
-            must be a ancestory of revision_id, or DivergedBranches is raised.
-        :param other_branch: The other branch that DivergedBranches should
-            raise with respect to.
-        """
-        self.set_revision_history(self._lefthand_history(revision_id,
-            last_rev, other_branch))
+        old_revno, old_revid = self.last_revision_info()
+        if self.get_append_revisions_only():
+            self._check_history_violation(revision_id)
+        self._run_pre_change_branch_tip_hooks(revno, revision_id)
+        self._write_last_revision_info(revno, revision_id)
+        self._clear_cached_state()
+        self._last_revision_info_cache = revno, revision_id
+        self._run_post_change_branch_tip_hooks(old_revno, old_revid)
 
     def basis_tree(self):
         """See Branch.basis_tree."""
@@ -2629,28 +2572,6 @@
                 pass
         return None
 
-    def _basic_push(self, target, overwrite, stop_revision):
-        """Basic implementation of push without bound branches or hooks.
-
-        Must be called with source read locked and target write locked.
-        """
-        result = BranchPushResult()
-        result.source_branch = self
-        result.target_branch = target
-        result.old_revno, result.old_revid = target.last_revision_info()
-        self.update_references(target)
-        if result.old_revid != self.last_revision():
-            # We assume that during 'push' this repository is closer than
-            # the target.
-            graph = self.repository.get_graph(target.repository)
-            target.update_revisions(self, stop_revision,
-                overwrite=overwrite, graph=graph)
-        if self._push_should_merge_tags():
-            result.tag_conflicts = self.tags.merge_to(target.tags,
-                overwrite)
-        result.new_revno, result.new_revid = target.last_revision_info()
-        return result
-
     def get_stacked_on_url(self):
         raise errors.UnstackableBranchFormat(self._format, self.user_url)
 
@@ -2667,55 +2588,10 @@
             self._transport.put_bytes('parent', url + '\n',
                 mode=self.bzrdir._get_file_mode())
 
-
-class BzrBranch5(BzrBranch):
-    """A format 5 branch. This supports new features over plain branches.
-
-    It has support for a master_branch which is the data for bound branches.
-    """
-
-    def get_bound_location(self):
-        try:
-            return self._transport.get_bytes('bound')[:-1]
-        except errors.NoSuchFile:
-            return None
-
-    @needs_read_lock
-    def get_master_branch(self, possible_transports=None):
-        """Return the branch we are bound to.
-
-        :return: Either a Branch, or None
-
-        This could memoise the branch, but if thats done
-        it must be revalidated on each new lock.
-        So for now we just don't memoise it.
-        # RBC 20060304 review this decision.
-        """
-        bound_loc = self.get_bound_location()
-        if not bound_loc:
-            return None
-        try:
-            return Branch.open(bound_loc,
-                               possible_transports=possible_transports)
-        except (errors.NotBranchError, errors.ConnectionError), e:
-            raise errors.BoundBranchConnectionFailure(
-                    self, bound_loc, e)
-
     @needs_write_lock
-    def set_bound_location(self, location):
-        """Set the target where this branch is bound to.
-
-        :param location: URL to the target branch
-        """
-        if location:
-            self._transport.put_bytes('bound', location+'\n',
-                mode=self.bzrdir._get_file_mode())
-        else:
-            try:
-                self._transport.delete('bound')
-            except errors.NoSuchFile:
-                return False
-            return True
+    def unbind(self):
+        """If bound, unbind"""
+        return self.set_bound_location(None)
 
     @needs_write_lock
     def bind(self, other):
@@ -2743,10 +2619,50 @@
         # history around
         self.set_bound_location(other.base)
 
+    def get_bound_location(self):
+        try:
+            return self._transport.get_bytes('bound')[:-1]
+        except errors.NoSuchFile:
+            return None
+
+    @needs_read_lock
+    def get_master_branch(self, possible_transports=None):
+        """Return the branch we are bound to.
+
+        :return: Either a Branch, or None
+        """
+        if self._master_branch_cache is None:
+            self._master_branch_cache = self._get_master_branch(
+                possible_transports)
+        return self._master_branch_cache
+
+    def _get_master_branch(self, possible_transports):
+        bound_loc = self.get_bound_location()
+        if not bound_loc:
+            return None
+        try:
+            return Branch.open(bound_loc,
+                               possible_transports=possible_transports)
+        except (errors.NotBranchError, errors.ConnectionError), e:
+            raise errors.BoundBranchConnectionFailure(
+                    self, bound_loc, e)
+
     @needs_write_lock
-    def unbind(self):
-        """If bound, unbind"""
-        return self.set_bound_location(None)
+    def set_bound_location(self, location):
+        """Set the target where this branch is bound to.
+
+        :param location: URL to the target branch
+        """
+        self._master_branch_cache = None
+        if location:
+            self._transport.put_bytes('bound', location+'\n',
+                mode=self.bzrdir._get_file_mode())
+        else:
+            try:
+                self._transport.delete('bound')
+            except errors.NoSuchFile:
+                return False
+            return True
 
     @needs_write_lock
     def update(self, possible_transports=None):
@@ -2765,8 +2681,135 @@
             return old_tip
         return None
 
-
-class BzrBranch8(BzrBranch5):
+    def _read_last_revision_info(self):
+        revision_string = self._transport.get_bytes('last-revision')
+        revno, revision_id = revision_string.rstrip('\n').split(' ', 1)
+        revision_id = cache_utf8.get_cached_utf8(revision_id)
+        revno = int(revno)
+        return revno, revision_id
+
+    def _write_last_revision_info(self, revno, revision_id):
+        """Simply write out the revision id, with no checks.
+
+        Use set_last_revision_info to perform this safely.
+
+        Does not update the revision_history cache.
+        """
+        revision_id = _mod_revision.ensure_null(revision_id)
+        out_string = '%d %s\n' % (revno, revision_id)
+        self._transport.put_bytes('last-revision', out_string,
+            mode=self.bzrdir._get_file_mode())
+
+
+class FullHistoryBzrBranch(BzrBranch):
+    """Bzr branch which contains the full revision history."""
+
+    @needs_write_lock
+    def set_last_revision_info(self, revno, revision_id):
+        if not revision_id or not isinstance(revision_id, basestring):
+            raise errors.InvalidRevisionId(revision_id=revision_id, branch=self)
+        revision_id = _mod_revision.ensure_null(revision_id)
+        # this old format stores the full history, but this api doesn't
+        # provide it, so we must generate, and might as well check it's
+        # correct
+        history = self._lefthand_history(revision_id)
+        if len(history) != revno:
+            raise AssertionError('%d != %d' % (len(history), revno))
+        self._set_revision_history(history)
+
+    def _read_last_revision_info(self):
+        rh = self._revision_history()
+        revno = len(rh)
+        if revno:
+            return (revno, rh[-1])
+        else:
+            return (0, _mod_revision.NULL_REVISION)
+
+    @deprecated_method(deprecated_in((2, 4, 0)))
+    @needs_write_lock
+    def set_revision_history(self, rev_history):
+        """See Branch.set_revision_history."""
+        self._set_revision_history(rev_history)
+
+    def _set_revision_history(self, rev_history):
+        if 'evil' in debug.debug_flags:
+            mutter_callsite(3, "set_revision_history scales with history.")
+        check_not_reserved_id = _mod_revision.check_not_reserved_id
+        for rev_id in rev_history:
+            check_not_reserved_id(rev_id)
+        if Branch.hooks['post_change_branch_tip']:
+            # Don't calculate the last_revision_info() if there are no hooks
+            # that will use it.
+            old_revno, old_revid = self.last_revision_info()
+        if len(rev_history) == 0:
+            revid = _mod_revision.NULL_REVISION
+        else:
+            revid = rev_history[-1]
+        self._run_pre_change_branch_tip_hooks(len(rev_history), revid)
+        self._write_revision_history(rev_history)
+        self._clear_cached_state()
+        self._cache_revision_history(rev_history)
+        for hook in Branch.hooks['set_rh']:
+            hook(self, rev_history)
+        if Branch.hooks['post_change_branch_tip']:
+            self._run_post_change_branch_tip_hooks(old_revno, old_revid)
+
+    def _write_revision_history(self, history):
+        """Factored out of set_revision_history.
+
+        This performs the actual writing to disk.
+        It is intended to be called by set_revision_history."""
+        self._transport.put_bytes(
+            'revision-history', '\n'.join(history),
+            mode=self.bzrdir._get_file_mode())
+
+    def _gen_revision_history(self):
+        history = self._transport.get_bytes('revision-history').split('\n')
+        if history[-1:] == ['']:
+            # There shouldn't be a trailing newline, but just in case.
+            history.pop()
+        return history
+
+    def _synchronize_history(self, destination, revision_id):
+        if not isinstance(destination, FullHistoryBzrBranch):
+            super(BzrBranch, self)._synchronize_history(
+                destination, revision_id)
+            return
+        if revision_id == _mod_revision.NULL_REVISION:
+            new_history = []
+        else:
+            new_history = self._revision_history()
+        if revision_id is not None and new_history != []:
+            try:
+                new_history = new_history[:new_history.index(revision_id) + 1]
+            except ValueError:
+                rev = self.repository.get_revision(revision_id)
+                new_history = rev.get_history(self.repository)[1:]
+        destination._set_revision_history(new_history)
+
+    @needs_write_lock
+    def generate_revision_history(self, revision_id, last_rev=None,
+        other_branch=None):
+        """Create a new revision history that will finish with revision_id.
+
+        :param revision_id: the new tip to use.
+        :param last_rev: The previous last_revision. If not None, then this
+            must be a ancestory of revision_id, or DivergedBranches is raised.
+        :param other_branch: The other branch that DivergedBranches should
+            raise with respect to.
+        """
+        self._set_revision_history(self._lefthand_history(revision_id,
+            last_rev, other_branch))
+
+
+class BzrBranch5(FullHistoryBzrBranch):
+    """A format 5 branch. This supports new features over plain branches.
+
+    It has support for a master_branch which is the data for bound branches.
+    """
+
+
+class BzrBranch8(BzrBranch):
     """A branch that stores tree-reference locations."""
 
     def _open_hook(self):
@@ -2798,57 +2841,16 @@
         self._last_revision_info_cache = None
         self._reference_info = None
 
-    def _last_revision_info(self):
-        revision_string = self._transport.get_bytes('last-revision')
-        revno, revision_id = revision_string.rstrip('\n').split(' ', 1)
-        revision_id = cache_utf8.get_cached_utf8(revision_id)
-        revno = int(revno)
-        return revno, revision_id
-
-    def _write_last_revision_info(self, revno, revision_id):
-        """Simply write out the revision id, with no checks.
-
-        Use set_last_revision_info to perform this safely.
-
-        Does not update the revision_history cache.
-        Intended to be called by set_last_revision_info and
-        _write_revision_history.
-        """
-        revision_id = _mod_revision.ensure_null(revision_id)
-        out_string = '%d %s\n' % (revno, revision_id)
-        self._transport.put_bytes('last-revision', out_string,
-            mode=self.bzrdir._get_file_mode())
-
-    @needs_write_lock
-    def set_last_revision_info(self, revno, revision_id):
-        revision_id = _mod_revision.ensure_null(revision_id)
-        old_revno, old_revid = self.last_revision_info()
-        if self._get_append_revisions_only():
-            self._check_history_violation(revision_id)
-        self._run_pre_change_branch_tip_hooks(revno, revision_id)
-        self._write_last_revision_info(revno, revision_id)
-        self._clear_cached_state()
-        self._last_revision_info_cache = revno, revision_id
-        self._run_post_change_branch_tip_hooks(old_revno, old_revid)
-
-    def _synchronize_history(self, destination, revision_id):
-        """Synchronize last revision and revision history between branches.
-
-        :see: Branch._synchronize_history
-        """
-        # XXX: The base Branch has a fast implementation of this method based
-        # on set_last_revision_info, but BzrBranch/BzrBranch5 have a slower one
-        # that uses set_revision_history.  This class inherits from BzrBranch5,
-        # but wants the fast implementation, so it calls
-        # Branch._synchronize_history directly.
-        Branch._synchronize_history(self, destination, revision_id)
-
     def _check_history_violation(self, revision_id):
-        last_revision = _mod_revision.ensure_null(self.last_revision())
+        current_revid = self.last_revision()
+        last_revision = _mod_revision.ensure_null(current_revid)
         if _mod_revision.is_null(last_revision):
             return
-        if last_revision not in self._lefthand_history(revision_id):
-            raise errors.AppendRevisionsOnlyViolation(self.user_url)
+        graph = self.repository.get_graph()
+        for lh_ancestor in graph.iter_lefthand_ancestry(revision_id):
+            if lh_ancestor == current_revid:
+                return
+        raise errors.AppendRevisionsOnlyViolation(self.user_url)
 
     def _gen_revision_history(self):
         """Generate the revision history from last revision
@@ -2857,22 +2859,6 @@
         self._extend_partial_history(stop_index=last_revno-1)
         return list(reversed(self._partial_revision_history_cache))
 
-    def _write_revision_history(self, history):
-        """Factored out of set_revision_history.
-
-        This performs the actual writing to disk, with format-specific checks.
-        It is intended to be called by BzrBranch5.set_revision_history.
-        """
-        if len(history) == 0:
-            last_revision = 'null:'
-        else:
-            if history != self._lefthand_history(history[-1]):
-                raise errors.NotLefthandHistory(history)
-            last_revision = history[-1]
-        if self._get_append_revisions_only():
-            self._check_history_violation(last_revision)
-        self._write_last_revision_info(len(history), last_revision)
-
     @needs_write_lock
     def _set_parent_location(self, url):
         """Set the parent branch"""
@@ -2964,6 +2950,7 @@
 
     def set_bound_location(self, location):
         """See Branch.set_push_location."""
+        self._master_branch_cache = None
         result = None
         config = self.get_config()
         if location is None:
@@ -3000,23 +2987,15 @@
         # you can always ask for the URL; but you might not be able to use it
         # if the repo can't support stacking.
         ## self._check_stackable_repo()
-        stacked_url = self._get_config_location('stacked_on_location')
+        # stacked_on_location is only ever defined in branch.conf, so don't
+        # waste effort reading the whole stack of config files.
+        config = self.get_config()._get_branch_data_config()
+        stacked_url = self._get_config_location('stacked_on_location',
+            config=config)
         if stacked_url is None:
             raise errors.NotStacked(self)
         return stacked_url
 
-    def _get_append_revisions_only(self):
-        return self.get_config(
-            ).get_user_option_as_bool('append_revisions_only')
-
-    @needs_write_lock
-    def generate_revision_history(self, revision_id, last_rev=None,
-                                  other_branch=None):
-        """See BzrBranch5.generate_revision_history"""
-        history = self._lefthand_history(revision_id, last_rev, other_branch)
-        revno = len(history)
-        self.set_last_revision_info(revno, revision_id)
-
     @needs_read_lock
     def get_rev_id(self, revno, history=None):
         """Find the revision id of the specified revno."""
@@ -3046,7 +3025,10 @@
         try:
             index = self._partial_revision_history_cache.index(revision_id)
         except ValueError:
-            self._extend_partial_history(stop_revision=revision_id)
+            try:
+                self._extend_partial_history(stop_revision=revision_id)
+            except errors.RevisionNotPresent, e:
+                raise errors.GhostRevisionsHaveNoRevno(revision_id, e.revision_id)
             index = len(self._partial_revision_history_cache) - 1
             if self._partial_revision_history_cache[index] != revision_id:
                 raise errors.NoSuchRevision(self, revision_id)
@@ -3105,18 +3087,30 @@
     :ivar local_branch: target branch if there is a Master, else None
     :ivar target_branch: Target/destination branch object. (write locked)
     :ivar tag_conflicts: A list of tag conflicts, see BasicTags.merge_to
+    :ivar tag_updates: A dict with new tags, see BasicTags.merge_to
     """
 
+    @deprecated_method(deprecated_in((2, 3, 0)))
     def __int__(self):
-        # DEPRECATED: pull used to return the change in revno
+        """Return the relative change in revno.
+
+        :deprecated: Use `new_revno` and `old_revno` instead.
+        """
         return self.new_revno - self.old_revno
 
     def report(self, to_file):
+        tag_conflicts = getattr(self, "tag_conflicts", None)
+        tag_updates = getattr(self, "tag_updates", None)
         if not is_quiet():
-            if self.old_revid == self.new_revid:
-                to_file.write('No revisions to pull.\n')
-            else:
+            if self.old_revid != self.new_revid:
                 to_file.write('Now on revision %d.\n' % self.new_revno)
+            if tag_updates:
+                to_file.write('%d tag(s) updated.\n' % len(tag_updates))
+            if self.old_revid == self.new_revid and not tag_updates:
+                if not tag_conflicts:
+                    to_file.write('No revisions or tags to pull.\n')
+                else:
+                    to_file.write('No revisions to pull.\n')
         self._show_tag_conficts(to_file)
 
 
@@ -3139,16 +3133,31 @@
         target, otherwise it will be None.
     """
 
+    @deprecated_method(deprecated_in((2, 3, 0)))
     def __int__(self):
-        # DEPRECATED: push used to return the change in revno
+        """Return the relative change in revno.
+
+        :deprecated: Use `new_revno` and `old_revno` instead.
+        """
         return self.new_revno - self.old_revno
 
     def report(self, to_file):
-        """Write a human-readable description of the result."""
-        if self.old_revid == self.new_revid:
-            note('No new revisions to push.')
-        else:
-            note('Pushed up to revision %d.' % self.new_revno)
+        # TODO: This function gets passed a to_file, but then
+        # ignores it and calls note() instead. This is also
+        # inconsistent with PullResult(), which writes to stdout.
+        # -- JRV20110901, bug #838853
+        tag_conflicts = getattr(self, "tag_conflicts", None)
+        tag_updates = getattr(self, "tag_updates", None)
+        if not is_quiet():
+            if self.old_revid != self.new_revid:
+                note(gettext('Pushed up to revision %d.') % self.new_revno)
+            if tag_updates:
+                note(ngettext('%d tag updated.', '%d tags updated.', len(tag_updates)) % len(tag_updates))
+            if self.old_revid == self.new_revid and not tag_updates:
+                if not tag_conflicts:
+                    note(gettext('No new revisions or tags to push.'))
+                else:
+                    note(gettext('No new revisions to push.'))
         self._show_tag_conficts(to_file)
 
 
@@ -3168,10 +3177,10 @@
         :param verbose: Requests more detailed display of what was checked,
             if any.
         """
-        note('checked branch %s format %s', self.branch.user_url,
-            self.branch._format)
+        note(gettext('checked branch {0} format {1}').format(
+                                self.branch.user_url, self.branch._format))
         for error in self.errors:
-            note('found error:%s', error)
+            note(gettext('found error:%s'), error)
 
 
 class Converter5to6(object):
@@ -3216,7 +3225,7 @@
 
 
 class Converter7to8(object):
-    """Perform an in-place upgrade of format 6 to format 7"""
+    """Perform an in-place upgrade of format 7 to format 8"""
 
     def convert(self, branch):
         format = BzrBranchFormat8()
@@ -3225,39 +3234,6 @@
         branch._transport.put_bytes('format', format.get_format_string())
 
 
-def _run_with_write_locked_target(target, callable, *args, **kwargs):
-    """Run ``callable(*args, **kwargs)``, write-locking target for the
-    duration.
-
-    _run_with_write_locked_target will attempt to release the lock it acquires.
-
-    If an exception is raised by callable, then that exception *will* be
-    propagated, even if the unlock attempt raises its own error.  Thus
-    _run_with_write_locked_target should be preferred to simply doing::
-
-        target.lock_write()
-        try:
-            return callable(*args, **kwargs)
-        finally:
-            target.unlock()
-
-    """
-    # This is very similar to bzrlib.decorators.needs_write_lock.  Perhaps they
-    # should share code?
-    target.lock_write()
-    try:
-        result = callable(*args, **kwargs)
-    except:
-        exc_info = sys.exc_info()
-        try:
-            target.unlock()
-        finally:
-            raise exc_info[0], exc_info[1], exc_info[2]
-    else:
-        target.unlock()
-        return result
-
-
 class InterBranch(InterObject):
     """This class represents operations taking place between two branches.
 
@@ -3291,21 +3267,7 @@
         raise NotImplementedError(self.pull)
 
     @needs_write_lock
-    def update_revisions(self, stop_revision=None, overwrite=False,
-                         graph=None):
-        """Pull in new perfect-fit revisions.
-
-        :param stop_revision: Updated until the given revision
-        :param overwrite: Always set the branch pointer, rather than checking
-            to see if it is a proper descendant.
-        :param graph: A Graph object that can be used to query history
-            information. This can be None.
-        :return: None
-        """
-        raise NotImplementedError(self.update_revisions)
-
-    @needs_write_lock
-    def push(self, overwrite=False, stop_revision=None,
+    def push(self, overwrite=False, stop_revision=None, lossy=False,
              _override_hook_source_branch=None):
         """Mirror the source branch into the target branch.
 
@@ -3322,6 +3284,15 @@
         """
         raise NotImplementedError(self.copy_content_into)
 
+    @needs_write_lock
+    def fetch(self, stop_revision=None, limit=None):
+        """Fetch revisions.
+
+        :param stop_revision: Last revision to fetch
+        :param limit: Optional rough limit of revisions to fetch
+        """
+        raise NotImplementedError(self.fetch)
+
 
 class GenericInterBranch(InterBranch):
     """InterBranch implementation that uses public Branch functions."""
@@ -3333,14 +3304,14 @@
 
     @classmethod
     def _get_branch_formats_to_test(klass):
-        return [(BranchFormat._default_format, BranchFormat._default_format)]
+        return [(format_registry.get_default(), format_registry.get_default())]
 
     @classmethod
     def unwrap_format(klass, format):
         if isinstance(format, remote.RemoteBranchFormat):
             format._ensure_real()
             return format._custom_format
-        return format                                                                                                  
+        return format
 
     @needs_write_lock
     def copy_content_into(self, revision_id=None):
@@ -3362,9 +3333,27 @@
             self.source.tags.merge_to(self.target.tags)
 
     @needs_write_lock
-    def update_revisions(self, stop_revision=None, overwrite=False,
-        graph=None):
-        """See InterBranch.update_revisions()."""
+    def fetch(self, stop_revision=None, limit=None):
+        if self.target.base == self.source.base:
+            return (0, [])
+        self.source.lock_read()
+        try:
+            fetch_spec_factory = fetch.FetchSpecFactory()
+            fetch_spec_factory.source_branch = self.source
+            fetch_spec_factory.source_branch_stop_revision_id = stop_revision
+            fetch_spec_factory.source_repo = self.source.repository
+            fetch_spec_factory.target_repo = self.target.repository
+            fetch_spec_factory.target_repo_kind = fetch.TargetRepoKinds.PREEXISTING
+            fetch_spec_factory.limit = limit
+            fetch_spec = fetch_spec_factory.make_fetch_spec()
+            return self.target.repository.fetch(self.source.repository,
+                fetch_spec=fetch_spec)
+        finally:
+            self.source.unlock()
+
+    @needs_write_lock
+    def _update_revisions(self, stop_revision=None, overwrite=False,
+            graph=None):
         other_revno, other_last_revision = self.source.last_revision_info()
         stop_revno = None # unknown
         if stop_revision is None:
@@ -3381,7 +3370,7 @@
         # case of having something to pull, and so that the check for
         # already merged can operate on the just fetched graph, which will
         # be cached in memory.
-        self.target.fetch(self.source, stop_revision)
+        self.fetch(stop_revision=stop_revision)
         # Check to see if one is an ancestor of the other
         if not overwrite:
             if graph is None:
@@ -3415,7 +3404,17 @@
         if local and not bound_location:
             raise errors.LocalRequiresBoundBranch()
         master_branch = None
-        if not local and bound_location and self.source.user_url != bound_location:
+        source_is_master = False
+        if bound_location:
+            # bound_location comes from a config file, some care has to be
+            # taken to relate it to source.user_url
+            normalized = urlutils.normalize_url(bound_location)
+            try:
+                relpath = self.source.user_transport.relpath(normalized)
+                source_is_master = (relpath == '')
+            except (errors.PathNotChild, errors.InvalidURL):
+                source_is_master = False
+        if not local and bound_location and not source_is_master:
             # not pulling from master, so we need to update master.
             master_branch = self.target.get_master_branch(possible_transports)
             master_branch.lock_write()
@@ -3427,34 +3426,57 @@
             return self._pull(overwrite,
                 stop_revision, _hook_master=master_branch,
                 run_hooks=run_hooks,
-                _override_hook_target=_override_hook_target)
+                _override_hook_target=_override_hook_target,
+                merge_tags_to_master=not source_is_master)
         finally:
             if master_branch:
                 master_branch.unlock()
 
-    def push(self, overwrite=False, stop_revision=None,
+    def push(self, overwrite=False, stop_revision=None, lossy=False,
              _override_hook_source_branch=None):
         """See InterBranch.push.
 
         This is the basic concrete implementation of push()
 
-        :param _override_hook_source_branch: If specified, run
-        the hooks passing this Branch as the source, rather than self.
-        This is for use of RemoteBranch, where push is delegated to the
-        underlying vfs-based Branch.
+        :param _override_hook_source_branch: If specified, run the hooks
+            passing this Branch as the source, rather than self.  This is for
+            use of RemoteBranch, where push is delegated to the underlying
+            vfs-based Branch.
         """
+        if lossy:
+            raise errors.LossyPushToSameVCS(self.source, self.target)
         # TODO: Public option to disable running hooks - should be trivial but
         # needs tests.
-        self.source.lock_read()
-        try:
-            return _run_with_write_locked_target(
-                self.target, self._push_with_bound_branches, overwrite,
-                stop_revision,
-                _override_hook_source_branch=_override_hook_source_branch)
-        finally:
-            self.source.unlock()
-
-    def _push_with_bound_branches(self, overwrite, stop_revision,
+
+        op = cleanup.OperationWithCleanups(self._push_with_bound_branches)
+        op.add_cleanup(self.source.lock_read().unlock)
+        op.add_cleanup(self.target.lock_write().unlock)
+        return op.run(overwrite, stop_revision,
+            _override_hook_source_branch=_override_hook_source_branch)
+
+    def _basic_push(self, overwrite, stop_revision):
+        """Basic implementation of push without bound branches or hooks.
+
+        Must be called with source read locked and target write locked.
+        """
+        result = BranchPushResult()
+        result.source_branch = self.source
+        result.target_branch = self.target
+        result.old_revno, result.old_revid = self.target.last_revision_info()
+        self.source.update_references(self.target)
+        if result.old_revid != stop_revision:
+            # We assume that during 'push' this repository is closer than
+            # the target.
+            graph = self.source.repository.get_graph(self.target.repository)
+            self._update_revisions(stop_revision, overwrite=overwrite,
+                    graph=graph)
+        if self.source._push_should_merge_tags():
+            result.tag_updates, result.tag_conflicts = (
+                self.source.tags.merge_to(self.target.tags, overwrite))
+        result.new_revno, result.new_revid = self.target.last_revision_info()
+        return result
+
+    def _push_with_bound_branches(self, operation, overwrite, stop_revision,
             _override_hook_source_branch=None):
         """Push from source into target, and into target's master if any.
         """
@@ -3472,35 +3494,32 @@
             # be bound to itself? -- mbp 20070507
             master_branch = self.target.get_master_branch()
             master_branch.lock_write()
-            try:
-                # push into the master from the source branch.
-                self.source._basic_push(master_branch, overwrite, stop_revision)
-                # and push into the target branch from the source. Note that we
-                # push from the source branch again, because its considered the
-                # highest bandwidth repository.
-                result = self.source._basic_push(self.target, overwrite,
-                    stop_revision)
-                result.master_branch = master_branch
-                result.local_branch = self.target
-                _run_hooks()
-                return result
-            finally:
-                master_branch.unlock()
+            operation.add_cleanup(master_branch.unlock)
+            # push into the master from the source branch.
+            master_inter = InterBranch.get(self.source, master_branch)
+            master_inter._basic_push(overwrite, stop_revision)
+            # and push into the target branch from the source. Note that
+            # we push from the source branch again, because it's considered
+            # the highest bandwidth repository.
+            result = self._basic_push(overwrite, stop_revision)
+            result.master_branch = master_branch
+            result.local_branch = self.target
         else:
+            master_branch = None
             # no master branch
-            result = self.source._basic_push(self.target, overwrite,
-                stop_revision)
+            result = self._basic_push(overwrite, stop_revision)
             # TODO: Why set master_branch and local_branch if there's no
             # binding?  Maybe cleaner to just leave them unset? -- mbp
             # 20070504
             result.master_branch = self.target
             result.local_branch = None
-            _run_hooks()
-            return result
+        _run_hooks()
+        return result
 
     def _pull(self, overwrite=False, stop_revision=None,
              possible_transports=None, _hook_master=None, run_hooks=True,
-             _override_hook_target=None, local=False):
+             _override_hook_target=None, local=False,
+             merge_tags_to_master=True):
         """See Branch.pull.
 
         This function is the core worker, used by GenericInterBranch.pull to
@@ -3511,6 +3530,8 @@
         :param run_hooks: Private parameter - if false, this branch
             is being called because it's the master of the primary branch,
             so it should not run its hooks.
+            is being called because it's the master of the primary branch,
+            so it should not run its hooks.
         :param _override_hook_target: Private parameter - set the branch to be
             supplied as the target_branch to pull hooks.
         :param local: Only update the local branch, and not the bound branch.
@@ -3535,13 +3556,14 @@
             # -- JRV20090506
             result.old_revno, result.old_revid = \
                 self.target.last_revision_info()
-            self.target.update_revisions(self.source, stop_revision,
-                overwrite=overwrite, graph=graph)
+            self._update_revisions(stop_revision, overwrite=overwrite,
+                graph=graph)
             # TODO: The old revid should be specified when merging tags, 
             # so a tags implementation that versions tags can only 
             # pull in the most recent changes. -- JRV20090506
-            result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
-                overwrite)
+            result.tag_updates, result.tag_conflicts = (
+                self.source.tags.merge_to(self.target.tags, overwrite,
+                    ignore_master=not merge_tags_to_master))
             result.new_revno, result.new_revid = self.target.last_revision_info()
             if _hook_master:
                 result.master_branch = _hook_master

=== modified file 'bzrlib/branchbuilder.py'
--- bzrlib/branchbuilder.py	2010-02-27 12:27:33 +0000
+++ bzrlib/branchbuilder.py	2011-10-11 12:01:51 +0000
@@ -17,10 +17,11 @@
 """Utility for create branches with particular contents."""
 
 from bzrlib import (
-    bzrdir,
+    controldir,
     commit,
     errors,
     memorytree,
+    revision,
     )
 
 
@@ -63,7 +64,7 @@
             If the path of the transport does not exist but its parent does
             it will be created.
         :param format: Either a BzrDirFormat, or the name of a format in the
-            bzrdir format registry for the branch to be built.
+            controldir format registry for the branch to be built.
         :param branch: An already constructed branch to use.  This param is
             mutually exclusive with the transport and format params.
         """
@@ -81,8 +82,8 @@
             if format is None:
                 format = 'default'
             if isinstance(format, str):
-                format = bzrdir.format_registry.make_bzrdir(format)
-            self._branch = bzrdir.BzrDir.create_branch_convenience(
+                format = controldir.format_registry.make_bzrdir(format)
+            self._branch = controldir.ControlDir.create_branch_convenience(
                 transport.base, format=format, force_new_tree=False)
         self._tree = None
 
@@ -165,6 +166,11 @@
         committer=None, timezone=None, message_callback=None):
         """Build a commit, shaped in a specific way.
 
+        Most of the actions are self-explanatory.  'flush' is special action to
+        break a series of actions into discrete steps so that complex changes
+        (such as unversioning a file-id and re-adding it with a different kind)
+        can be expressed in a way that will clearly work.
+
         :param revision_id: The handle for the new commit, can be None
         :param parent_ids: A list of parent_ids to use for the commit.
             It can be None, which indicates to use the last commit.
@@ -173,6 +179,7 @@
             ('modify', ('file-id', 'new-content'))
             ('unversion', 'file-id')
             ('rename', ('orig-path', 'new-path'))
+            ('flush', None)
         :param message: An optional commit message, if not supplied, a default
             commit message will be written.
         :param message_callback: A message callback to use for the commit, as
@@ -186,7 +193,10 @@
         :return: The revision_id of the new commit
         """
         if parent_ids is not None:
-            base_id = parent_ids[0]
+            if len(parent_ids) == 0:
+                base_id = revision.NULL_REVISION
+            else:
+                base_id = parent_ids[0]
             if base_id != self._branch.last_revision():
                 self._move_branch_pointer(base_id,
                     allow_leftmost_as_ghost=allow_leftmost_as_ghost)
@@ -204,53 +214,76 @@
             # inventory entry. And the only public function to create a
             # directory is MemoryTree.mkdir() which creates the directory, but
             # also always adds it. So we have to use a multi-pass setup.
-            to_add_directories = []
-            to_add_files = []
-            to_add_file_ids = []
-            to_add_kinds = []
-            new_contents = {}
-            to_unversion_ids = []
-            to_rename = []
+            pending = _PendingActions()
             for action, info in actions:
                 if action == 'add':
                     path, file_id, kind, content = info
                     if kind == 'directory':
-                        to_add_directories.append((path, file_id))
+                        pending.to_add_directories.append((path, file_id))
                     else:
-                        to_add_files.append(path)
-                        to_add_file_ids.append(file_id)
-                        to_add_kinds.append(kind)
+                        pending.to_add_files.append(path)
+                        pending.to_add_file_ids.append(file_id)
+                        pending.to_add_kinds.append(kind)
                         if content is not None:
-                            new_contents[file_id] = content
+                            pending.new_contents[file_id] = content
                 elif action == 'modify':
                     file_id, content = info
-                    new_contents[file_id] = content
+                    pending.new_contents[file_id] = content
                 elif action == 'unversion':
-                    to_unversion_ids.append(info)
+                    pending.to_unversion_ids.add(info)
                 elif action == 'rename':
                     from_relpath, to_relpath = info
-                    to_rename.append((from_relpath, to_relpath))
+                    pending.to_rename.append((from_relpath, to_relpath))
+                elif action == 'flush':
+                    self._flush_pending(tree, pending)
+                    pending = _PendingActions()
                 else:
                     raise ValueError('Unknown build action: "%s"' % (action,))
-            if to_unversion_ids:
-                tree.unversion(to_unversion_ids)
-            for path, file_id in to_add_directories:
-                if path == '':
-                    # Special case, because the path already exists
-                    tree.add([path], [file_id], ['directory'])
-                else:
-                    tree.mkdir(path, file_id)
-            for from_relpath, to_relpath in to_rename:
-                tree.rename_one(from_relpath, to_relpath)
-            tree.add(to_add_files, to_add_file_ids, to_add_kinds)
-            for file_id, content in new_contents.iteritems():
-                tree.put_file_bytes_non_atomic(file_id, content)
+            self._flush_pending(tree, pending)
             return self._do_commit(tree, message=message, rev_id=revision_id,
                 timestamp=timestamp, timezone=timezone, committer=committer,
                 message_callback=message_callback)
         finally:
             tree.unlock()
 
+    def _flush_pending(self, tree, pending):
+        """Flush the pending actions in 'pending', i.e. apply them to 'tree'."""
+        for path, file_id in pending.to_add_directories:
+            if path == '':
+                old_id = tree.path2id(path)
+                if old_id is not None and old_id in pending.to_unversion_ids:
+                    # We're overwriting this path, no need to unversion
+                    pending.to_unversion_ids.discard(old_id)
+                # Special case, because the path already exists
+                tree.add([path], [file_id], ['directory'])
+            else:
+                tree.mkdir(path, file_id)
+        for from_relpath, to_relpath in pending.to_rename:
+            tree.rename_one(from_relpath, to_relpath)
+        if pending.to_unversion_ids:
+            tree.unversion(pending.to_unversion_ids)
+        tree.add(pending.to_add_files, pending.to_add_file_ids, pending.to_add_kinds)
+        for file_id, content in pending.new_contents.iteritems():
+            tree.put_file_bytes_non_atomic(file_id, content)
+
     def get_branch(self):
         """Return the branch created by the builder."""
         return self._branch
+
+
+class _PendingActions(object):
+    """Pending actions for build_snapshot to take.
+
+    This is just a simple class to hold a bunch of the intermediate state of
+    build_snapshot in single object.
+    """
+
+    def __init__(self):
+        self.to_add_directories = []
+        self.to_add_files = []
+        self.to_add_file_ids = []
+        self.to_add_kinds = []
+        self.new_contents = {}
+        self.to_unversion_ids = set()
+        self.to_rename = []
+

=== modified file 'bzrlib/breakin.py'
--- bzrlib/breakin.py	2010-06-17 09:23:19 +0000
+++ bzrlib/breakin.py	2011-02-19 17:37:45 +0000
@@ -41,19 +41,6 @@
         signal.signal(_breakin_signal_number, _debug)
 
 
-def hook_sigquit():
-    # We import this late because breakin.py is loaded as part of the main
-    # 'bzr' script, so we want it to load as little as possible until things
-    # are up and running
-    from bzrlib import symbol_versioning, trace
-    trace.mutter_callsite(2, 'Deprecated function called')
-    symbol_versioning.warn(symbol_versioning.deprecation_string(
-        hook_sigquit, symbol_versioning.deprecated_in((1, 18, 0))),
-        DeprecationWarning, stacklevel=2)
-
-    return hook_debugger_to_signal()
-
-
 def determine_signal():
     global _breakin_signal_number
     global _breakin_signal_name

=== modified file 'bzrlib/btree_index.py'
--- bzrlib/btree_index.py	2010-06-20 11:18:38 +0000
+++ bzrlib/btree_index.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,10 +18,14 @@
 """B+Tree indices"""
 
 import cStringIO
-from bisect import bisect_right
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import bisect
 import math
 import tempfile
 import zlib
+""")
 
 from bzrlib import (
     chunk_writer,
@@ -158,7 +162,7 @@
         :param references: An iterable of iterables of keys. Each is a
             reference to another key.
         :param value: The value to associate with the key. It may be any
-            bytes as long as it does not contain \0 or \n.
+            bytes as long as it does not contain \\0 or \\n.
         """
         # Ensure that 'key' is a StaticTuple
         key = static_tuple.StaticTuple.from_sequence(key).intern()
@@ -193,7 +197,7 @@
             new_backing_file, size = self._spill_mem_keys_without_combining()
         # Note: The transport here isn't strictly needed, because we will use
         #       direct access to the new_backing._file object
-        new_backing = BTreeGraphIndex(transport.get_transport('.'),
+        new_backing = BTreeGraphIndex(transport.get_transport_from_path('.'),
                                       '<temp>', size)
         # GC will clean up the file
         new_backing._file = new_backing_file
@@ -602,10 +606,10 @@
         """In memory index's have no known corruption at the moment."""
 
 
-class _LeafNode(object):
+class _LeafNode(dict):
     """A leaf node for a serialised B+Tree index."""
 
-    __slots__ = ('keys', 'min_key', 'max_key')
+    __slots__ = ('min_key', 'max_key', '_keys')
 
     def __init__(self, bytes, key_length, ref_list_length):
         """Parse bytes to create a leaf node object."""
@@ -617,7 +621,20 @@
             self.max_key = key_list[-1][0]
         else:
             self.min_key = self.max_key = None
-        self.keys = dict(key_list)
+        super(_LeafNode, self).__init__(key_list)
+        self._keys = dict(self)
+
+    def all_items(self):
+        """Return a sorted list of (key, (value, refs)) items"""
+        items = self.items()
+        items.sort()
+        return items
+
+    def all_keys(self):
+        """Return a sorted list of all keys."""
+        keys = self.keys()
+        keys.sort()
+        return keys
 
 
 class _InternalNode(object):
@@ -672,6 +689,7 @@
         self._recommended_pages = self._compute_recommended_pages()
         self._root_node = None
         self._base_offset = offset
+        self._leaf_factory = _LeafNode
         # Default max size is 100,000 leave values
         self._leaf_value_cache = None # lru_cache.LRUCache(100*1000)
         if unlimited_cache:
@@ -950,7 +968,7 @@
         """Cache directly from key => value, skipping the btree."""
         if self._leaf_value_cache is not None:
             for node in nodes.itervalues():
-                for key, value in node.keys.iteritems():
+                for key, value in node.all_items():
                     if key in self._leaf_value_cache:
                         # Don't add the rest of the keys, we've seen this node
                         # before.
@@ -980,10 +998,10 @@
         if self._row_offsets[-1] == 1:
             # There is only the root node, and we read that via key_count()
             if self.node_ref_lists:
-                for key, (value, refs) in sorted(self._root_node.keys.items()):
+                for key, (value, refs) in self._root_node.all_items():
                     yield (self, key, value, refs)
             else:
-                for key, (value, refs) in sorted(self._root_node.keys.items()):
+                for key, (value, refs) in self._root_node.all_items():
                     yield (self, key, value)
             return
         start_of_leaves = self._row_offsets[-2]
@@ -999,11 +1017,11 @@
         # for spilling index builds to disk.
         if self.node_ref_lists:
             for _, node in nodes:
-                for key, (value, refs) in sorted(node.keys.items()):
+                for key, (value, refs) in node.all_items():
                     yield (self, key, value, refs)
         else:
             for _, node in nodes:
-                for key, (value, refs) in sorted(node.keys.items()):
+                for key, (value, refs) in node.all_items():
                     yield (self, key, value)
 
     @staticmethod
@@ -1033,7 +1051,7 @@
         # iter_steps = len(in_keys) + len(fixed_keys)
         # bisect_steps = len(in_keys) * math.log(len(fixed_keys), 2)
         if len(in_keys) == 1: # Bisect will always be faster for M = 1
-            return [(bisect_right(fixed_keys, in_keys[0]), in_keys)]
+            return [(bisect.bisect_right(fixed_keys, in_keys[0]), in_keys)]
         # elif bisect_steps < iter_steps:
         #     offsets = {}
         #     for key in in_keys:
@@ -1170,8 +1188,8 @@
                 continue
             node = nodes[node_index]
             for next_sub_key in sub_keys:
-                if next_sub_key in node.keys:
-                    value, refs = node.keys[next_sub_key]
+                if next_sub_key in node:
+                    value, refs = node[next_sub_key]
                     if self.node_ref_lists:
                         yield (self, next_sub_key, value, refs)
                     else:
@@ -1245,14 +1263,13 @@
             # sub_keys is all of the keys we are looking for that should exist
             # on this page, if they aren't here, then they won't be found
             node = nodes[node_index]
-            node_keys = node.keys
             parents_to_check = set()
             for next_sub_key in sub_keys:
-                if next_sub_key not in node_keys:
+                if next_sub_key not in node:
                     # This one is just not present in the index at all
                     missing_keys.add(next_sub_key)
                 else:
-                    value, refs = node_keys[next_sub_key]
+                    value, refs = node[next_sub_key]
                     parent_keys = refs[ref_list_num]
                     parent_map[next_sub_key] = parent_keys
                     parents_to_check.update(parent_keys)
@@ -1265,8 +1282,8 @@
             while parents_to_check:
                 next_parents_to_check = set()
                 for key in parents_to_check:
-                    if key in node_keys:
-                        value, refs = node_keys[key]
+                    if key in node:
+                        value, refs = node[key]
                         parent_keys = refs[ref_list_num]
                         parent_map[key] = parent_keys
                         next_parents_to_check.update(parent_keys)
@@ -1546,7 +1563,8 @@
                     continue
             bytes = zlib.decompress(data)
             if bytes.startswith(_LEAF_FLAG):
-                node = _LeafNode(bytes, self._key_length, self.node_ref_lists)
+                node = self._leaf_factory(bytes, self._key_length,
+                                          self.node_ref_lists)
             elif bytes.startswith(_INTERNAL_FLAG):
                 node = _InternalNode(bytes)
             else:
@@ -1571,8 +1589,11 @@
             pass
 
 
+_gcchk_factory = _LeafNode
+
 try:
     from bzrlib import _btree_serializer_pyx as _btree_serializer
+    _gcchk_factory = _btree_serializer._parse_into_chk
 except ImportError, e:
     osutils.failed_to_load_extension(e)
     from bzrlib import _btree_serializer_py as _btree_serializer

=== modified file 'bzrlib/bugtracker.py'
--- bzrlib/bugtracker.py	2010-04-30 11:03:59 +0000
+++ bzrlib/bugtracker.py	2011-09-01 15:26:30 +0000
@@ -48,8 +48,14 @@
 
     bzr commit --fixes <tracker>:<id>
 
+or::
+
+    bzr commit --fixes <id>
+
 where "<tracker>" is an identifier for the bug tracker, and "<id>" is the
 identifier for that bug within the bugtracker, usually the bug number.
+If "<tracker>" is not specified the ``bugtracker`` set in the branch
+or global configuration is used.
 
 Bazaar knows about a few bug trackers that have many users. If
 you use one of these bug trackers then there is no setup required to
@@ -93,7 +99,7 @@
 --fixes`` to mark bugs in that tracker as being fixed by that commit. For
 example::
 
-    bugzilla_squid_url = http://www.squid-cache.org/bugs
+    bugzilla_squid_url = http://bugs.squid-cache.org
 
 would allow ``bzr commit --fixes squid:1234`` to mark Squid's bug 1234 as
 fixed.
@@ -127,7 +133,13 @@
 
     bugtracker_cpan_url = http://rt.cpan.org/Public/Bug/Display.html?id={id}
 
-for CPAN's RT bug tracker.
+would allow ``bzr commit --fixes cpan:1234`` to mark bug 1234 in CPAN's
+RT bug tracker as fixed, or::
+
+    bugtracker_hudson_url = http://issues.hudson-ci.org/browse/{id}
+
+would allow ``bzr commit --fixes hudson:HUDSON-1234`` to mark bug HUDSON-1234
+in Hudson's JIRA bug tracker as fixed.
 """
 
 
@@ -225,23 +237,23 @@
 
 
 tracker_registry.register('gnome',
-    UniqueIntegerBugTracker('gnome', 'http://bugzilla.gnome.org/show_bug.cgi?id='))
-
-
-class URLParametrizedIntegerBugTracker(IntegerBugTracker):
+    UniqueIntegerBugTracker('gnome',
+                            'http://bugzilla.gnome.org/show_bug.cgi?id='))
+
+
+class URLParametrizedBugTracker(BugTracker):
     """A type of bug tracker that can be found on a variety of different sites,
     and thus needs to have the base URL configured.
 
     Looks for a config setting in the form '<type_name>_<abbreviation>_url'.
-    `type_name` is the name of the type of tracker (e.g. 'bugzilla' or 'trac')
-    and `abbreviation` is a short name for the particular instance (e.g.
-    'squid' or 'apache').
+    `type_name` is the name of the type of tracker and `abbreviation`
+    is a short name for the particular instance.
     """
 
     def get(self, abbreviation, branch):
         config = branch.get_config()
         url = config.get_user_option(
-            "%s_%s_url" % (self.type_name, abbreviation))
+            "%s_%s_url" % (self.type_name, abbreviation), expand=False)
         if url is None:
             return None
         self._base_url = url
@@ -256,6 +268,19 @@
         return urlutils.join(self._base_url, self._bug_area) + str(bug_id)
 
 
+class URLParametrizedIntegerBugTracker(IntegerBugTracker,
+                                       URLParametrizedBugTracker):
+    """A type of bug tracker that  only allows integer bug IDs.
+
+    This can be found on a variety of different sites, and thus needs to have
+    the base URL configured.
+
+    Looks for a config setting in the form '<type_name>_<abbreviation>_url'.
+    `type_name` is the name of the type of tracker (e.g. 'bugzilla' or 'trac')
+    and `abbreviation` is a short name for the particular instance (e.g.
+    'squid' or 'apache').
+    """
+
 tracker_registry.register(
     'trac', URLParametrizedIntegerBugTracker('trac', 'ticket/'))
 
@@ -264,7 +289,7 @@
     URLParametrizedIntegerBugTracker('bugzilla', 'show_bug.cgi?id='))
 
 
-class GenericBugTracker(URLParametrizedIntegerBugTracker):
+class GenericBugTracker(URLParametrizedBugTracker):
     """Generic bug tracker specified by an URL template."""
 
     def __init__(self):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-07-29 11:17:57 +0000
+++ bzrlib/builtins.py	2011-10-14 13:56:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,9 +18,10 @@
 
 import os
 
+import bzrlib.bzrdir
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import codecs
 import cStringIO
 import sys
 import time
@@ -30,10 +31,10 @@
     bugtracker,
     bundle,
     btree_index,
-    bzrdir,
+    controldir,
     directory_service,
     delta,
-    config,
+    config as _mod_config,
     errors,
     globbing,
     hooks,
@@ -45,12 +46,12 @@
     rename_map,
     revision as _mod_revision,
     static_tuple,
-    symbol_versioning,
     timestamp,
     transport,
     ui,
     urlutils,
     views,
+    gpg,
     )
 from bzrlib.branch import Branch
 from bzrlib.conflicts import ConflictList
@@ -58,6 +59,7 @@
 from bzrlib.revisionspec import RevisionSpec, RevisionInfo
 from bzrlib.smtp_connection import SMTPConnection
 from bzrlib.workingtree import WorkingTree
+from bzrlib.i18n import gettext, ngettext
 """)
 
 from bzrlib.commands import (
@@ -73,6 +75,9 @@
     _parse_revision_str,
     )
 from bzrlib.trace import mutter, note, warning, is_quiet, get_verbosity_level
+from bzrlib import (
+    symbol_versioning,
+    )
 
 
 @symbol_versioning.deprecated_function(symbol_versioning.deprecated_in((2, 3, 0)))
@@ -110,7 +115,7 @@
             if view_files:
                 file_list = view_files
                 view_str = views.view_display_str(view_files)
-                note("Ignoring files outside view. View is %s" % view_str)
+                note(gettext("Ignoring files outside view. View is %s") % view_str)
     return tree, file_list
 
 
@@ -118,8 +123,8 @@
     if revisions is None:
         return None
     if len(revisions) != 1:
-        raise errors.BzrCommandError(
-            'bzr %s --revision takes exactly one revision identifier' % (
+        raise errors.BzrCommandError(gettext(
+            'bzr %s --revision takes exactly one revision identifier') % (
                 command_name,))
     return revisions[0]
 
@@ -194,7 +199,7 @@
     the --directory option is used to specify a different branch."""
     if directory is not None:
         return (None, Branch.open(directory), filename)
-    return bzrdir.BzrDir.open_containing_tree_or_branch(filename)
+    return controldir.ControlDir.open_containing_tree_or_branch(filename)
 
 
 # TODO: Make sure no commands unconditionally use the working directory as a
@@ -230,9 +235,10 @@
     unknown
         Not versioned and not matching an ignore pattern.
 
-    Additionally for directories, symlinks and files with an executable
-    bit, Bazaar indicates their type using a trailing character: '/', '@'
-    or '*' respectively.
+    Additionally for directories, symlinks and files with a changed
+    executable bit, Bazaar indicates their type using a trailing
+    character: '/', '@' or '*' respectively. These decorations can be
+    disabled using the '--no-classify' option.
 
     To see ignored files use 'bzr ignored'.  For details on the
     changes to file texts, use 'bzr diff'.
@@ -251,8 +257,12 @@
     To skip the display of pending merge information altogether, use
     the no-pending option or specify a file/directory.
 
-    If a revision argument is given, the status is calculated against
-    that revision, or between two revisions if two are provided.
+    To compare the working directory to a specific revision, pass a
+    single revision to the revision argument.
+
+    To see which files have changed in a specific revision, or between
+    two revisions, pass a revision range to the revision argument.
+    This will produce the same results as calling 'bzr diff --summarize'.
     """
 
     # TODO: --no-recurse, --recurse options
@@ -265,6 +275,9 @@
                             short_name='V'),
                      Option('no-pending', help='Don\'t show pending merges.',
                            ),
+                     Option('no-classify',
+                            help='Do not mark object type using indicator.',
+                           ),
                      ]
     aliases = ['st', 'stat']
 
@@ -273,12 +286,13 @@
 
     @display_command
     def run(self, show_ids=False, file_list=None, revision=None, short=False,
-            versioned=False, no_pending=False, verbose=False):
+            versioned=False, no_pending=False, verbose=False,
+            no_classify=False):
         from bzrlib.status import show_tree_status
 
         if revision and len(revision) > 2:
-            raise errors.BzrCommandError('bzr status --revision takes exactly'
-                                         ' one or two revision specifiers')
+            raise errors.BzrCommandError(gettext('bzr status --revision takes exactly'
+                                         ' one or two revision specifiers'))
 
         tree, relfile_list = WorkingTree.open_containing_paths(file_list)
         # Avoid asking for specific files when that is not needed.
@@ -293,7 +307,8 @@
         show_tree_status(tree, show_ids=show_ids,
                          specific_files=relfile_list, revision=revision,
                          to_file=self.outf, short=short, versioned=versioned,
-                         show_pending=(not no_pending), verbose=verbose)
+                         show_pending=(not no_pending), verbose=verbose,
+                         classify=not no_classify)
 
 
 class cmd_cat_revision(Command):
@@ -320,17 +335,18 @@
     @display_command
     def run(self, revision_id=None, revision=None, directory=u'.'):
         if revision_id is not None and revision is not None:
-            raise errors.BzrCommandError('You can only supply one of'
-                                         ' revision_id or --revision')
+            raise errors.BzrCommandError(gettext('You can only supply one of'
+                                         ' revision_id or --revision'))
         if revision_id is None and revision is None:
-            raise errors.BzrCommandError('You must supply either'
-                                         ' --revision or a revision_id')
-        b = WorkingTree.open_containing(directory)[0].branch
+            raise errors.BzrCommandError(gettext('You must supply either'
+                                         ' --revision or a revision_id'))
+
+        b = controldir.ControlDir.open_containing_tree_or_branch(directory)[1]
 
         revisions = b.repository.revisions
         if revisions is None:
-            raise errors.BzrCommandError('Repository %r does not support '
-                'access to raw revision texts')
+            raise errors.BzrCommandError(gettext('Repository %r does not support '
+                'access to raw revision texts'))
 
         b.repository.lock_read()
         try:
@@ -340,14 +356,14 @@
                 try:
                     self.print_revision(revisions, revision_id)
                 except errors.NoSuchRevision:
-                    msg = "The repository %s contains no revision %s." % (
+                    msg = gettext("The repository {0} contains no revision {1}.").format(
                         b.repository.base, revision_id)
                     raise errors.BzrCommandError(msg)
             elif revision is not None:
                 for rev in revision:
                     if rev is None:
                         raise errors.BzrCommandError(
-                            'You cannot specify a NULL revision.')
+                            gettext('You cannot specify a NULL revision.'))
                     rev_id = rev.as_revision_id(b)
                     self.print_revision(revisions, rev_id)
         finally:
@@ -409,9 +425,12 @@
                 self.outf.write(page_bytes[:header_end])
                 page_bytes = data
             self.outf.write('\nPage %d\n' % (page_idx,))
-            decomp_bytes = zlib.decompress(page_bytes)
-            self.outf.write(decomp_bytes)
-            self.outf.write('\n')
+            if len(page_bytes) == 0:
+                self.outf.write('(empty)\n');
+            else:
+                decomp_bytes = zlib.decompress(page_bytes)
+                self.outf.write(decomp_bytes)
+                self.outf.write('\n')
 
     def _dump_entries(self, trans, basename):
         try:
@@ -456,15 +475,15 @@
             location_list=['.']
 
         for location in location_list:
-            d = bzrdir.BzrDir.open(location)
-            
+            d = controldir.ControlDir.open(location)
+
             try:
                 working = d.open_workingtree()
             except errors.NoWorkingTree:
-                raise errors.BzrCommandError("No working tree to remove")
+                raise errors.BzrCommandError(gettext("No working tree to remove"))
             except errors.NotLocalUrl:
-                raise errors.BzrCommandError("You cannot remove the working tree"
-                                             " of a remote path")
+                raise errors.BzrCommandError(gettext("You cannot remove the working tree"
+                                             " of a remote path"))
             if not force:
                 if (working.has_changes()):
                     raise errors.UncommittedChanges(working)
@@ -472,12 +491,64 @@
                     raise errors.ShelvedChanges(working)
 
             if working.user_url != working.branch.user_url:
-                raise errors.BzrCommandError("You cannot remove the working tree"
-                                             " from a lightweight checkout")
+                raise errors.BzrCommandError(gettext("You cannot remove the working tree"
+                                             " from a lightweight checkout"))
 
             d.destroy_workingtree()
 
 
+class cmd_repair_workingtree(Command):
+    __doc__ = """Reset the working tree state file.
+
+    This is not meant to be used normally, but more as a way to recover from
+    filesystem corruption, etc. This rebuilds the working inventory back to a
+    'known good' state. Any new modifications (adding a file, renaming, etc)
+    will be lost, though modified files will still be detected as such.
+
+    Most users will want something more like "bzr revert" or "bzr update"
+    unless the state file has become corrupted.
+
+    By default this attempts to recover the current state by looking at the
+    headers of the state file. If the state file is too corrupted to even do
+    that, you can supply --revision to force the state of the tree.
+    """
+
+    takes_options = ['revision', 'directory',
+        Option('force',
+               help='Reset the tree even if it doesn\'t appear to be'
+                    ' corrupted.'),
+    ]
+    hidden = True
+
+    def run(self, revision=None, directory='.', force=False):
+        tree, _ = WorkingTree.open_containing(directory)
+        self.add_cleanup(tree.lock_tree_write().unlock)
+        if not force:
+            try:
+                tree.check_state()
+            except errors.BzrError:
+                pass # There seems to be a real error here, so we'll reset
+            else:
+                # Refuse
+                raise errors.BzrCommandError(gettext(
+                    'The tree does not appear to be corrupt. You probably'
+                    ' want "bzr revert" instead. Use "--force" if you are'
+                    ' sure you want to reset the working tree.'))
+        if revision is None:
+            revision_ids = None
+        else:
+            revision_ids = [r.as_revision_id(tree.branch) for r in revision]
+        try:
+            tree.reset_state(revision_ids)
+        except errors.BzrError, e:
+            if revision_ids is None:
+                extra = (gettext(', the header appears corrupt, try passing -r -1'
+                         ' to set the state to the last commit'))
+            else:
+                extra = ''
+            raise errors.BzrCommandError(gettext('failed to reset the tree state{0}').format(extra))
+
+
 class cmd_revno(Command):
     __doc__ = """Show current revision number.
 
@@ -488,28 +559,41 @@
     takes_args = ['location?']
     takes_options = [
         Option('tree', help='Show revno of working tree'),
+        'revision',
         ]
 
     @display_command
-    def run(self, tree=False, location=u'.'):
+    def run(self, tree=False, location=u'.', revision=None):
+        if revision is not None and tree:
+            raise errors.BzrCommandError(gettext("--tree and --revision can "
+                "not be used together"))
+
         if tree:
             try:
                 wt = WorkingTree.open_containing(location)[0]
                 self.add_cleanup(wt.lock_read().unlock)
             except (errors.NoWorkingTree, errors.NotLocalUrl):
                 raise errors.NoWorkingTree(location)
+            b = wt.branch
             revid = wt.last_revision()
-            try:
-                revno_t = wt.branch.revision_id_to_dotted_revno(revid)
-            except errors.NoSuchRevision:
-                revno_t = ('???',)
-            revno = ".".join(str(n) for n in revno_t)
         else:
             b = Branch.open_containing(location)[0]
             self.add_cleanup(b.lock_read().unlock)
-            revno = b.revno()
+            if revision:
+                if len(revision) != 1:
+                    raise errors.BzrCommandError(gettext(
+                        "Tags can only be placed on a single revision, "
+                        "not on a range"))
+                revid = revision[0].as_revision_id(b)
+            else:
+                revid = b.last_revision()
+        try:
+            revno_t = b.revision_id_to_dotted_revno(revid)
+        except errors.NoSuchRevision:
+            revno_t = ('???',)
+        revno = ".".join(str(n) for n in revno_t)
         self.cleanup_now()
-        self.outf.write(str(revno) + '\n')
+        self.outf.write(revno + '\n')
 
 
 class cmd_revision_info(Command):
@@ -584,6 +668,9 @@
     are added.  This search proceeds recursively into versioned
     directories.  If no names are given '.' is assumed.
 
+    A warning will be printed when nested trees are encountered,
+    unless they are explicitly ignored.
+
     Therefore simply saying 'bzr add' will version all files that
     are currently unknown.
 
@@ -605,6 +692,10 @@
     
     Any files matching patterns in the ignore list will not be added
     unless they are explicitly mentioned.
+    
+    In recursive mode, files larger than the configuration option 
+    add.maximum_file_size will be skipped. Named items are never skipped due
+    to file size.
     """
     takes_args = ['file*']
     takes_options = [
@@ -637,7 +728,7 @@
             action = bzrlib.add.AddFromBaseAction(base_tree, base_path,
                           to_file=self.outf, should_print=(not is_quiet()))
         else:
-            action = bzrlib.add.AddAction(to_file=self.outf,
+            action = bzrlib.add.AddWithSkipLargeAction(to_file=self.outf,
                 should_print=(not is_quiet()))
 
         if base_tree:
@@ -650,8 +741,9 @@
             if verbose:
                 for glob in sorted(ignored.keys()):
                     for path in ignored[glob]:
-                        self.outf.write("ignored %s matching \"%s\"\n"
-                                        % (path, glob))
+                        self.outf.write(
+                         gettext("ignored {0} matching \"{1}\"\n").format(
+                         path, glob))
 
 
 class cmd_mkdir(Command):
@@ -671,7 +763,8 @@
             if id != None:
                 os.mkdir(d)
                 wt.add([dd])
-                self.outf.write('added %s\n' % d)
+                if not is_quiet():
+                    self.outf.write(gettext('added %s\n') % d)
             else:
                 raise errors.NotVersionedError(path=base)
 
@@ -715,7 +808,7 @@
     @display_command
     def run(self, revision=None, show_ids=False, kind=None, file_list=None):
         if kind and kind not in ['file', 'directory', 'symlink']:
-            raise errors.BzrCommandError('invalid kind %r specified' % (kind,))
+            raise errors.BzrCommandError(gettext('invalid kind %r specified') % (kind,))
 
         revision = _get_one_revision('inventory', revision)
         work_tree, file_list = WorkingTree.open_containing_paths(file_list)
@@ -734,8 +827,9 @@
                                       require_versioned=True)
             # find_ids_across_trees may include some paths that don't
             # exist in 'tree'.
-            entries = sorted((tree.id2path(file_id), tree.inventory[file_id])
-                             for file_id in file_ids if file_id in tree)
+            entries = sorted(
+                (tree.id2path(file_id), tree.inventory[file_id])
+                for file_id in file_ids if tree.has_id(file_id))
         else:
             entries = tree.inventory.entries()
 
@@ -784,22 +878,22 @@
         if auto:
             return self.run_auto(names_list, after, dry_run)
         elif dry_run:
-            raise errors.BzrCommandError('--dry-run requires --auto.')
+            raise errors.BzrCommandError(gettext('--dry-run requires --auto.'))
         if names_list is None:
             names_list = []
         if len(names_list) < 2:
-            raise errors.BzrCommandError("missing file argument")
+            raise errors.BzrCommandError(gettext("missing file argument"))
         tree, rel_names = WorkingTree.open_containing_paths(names_list, canonicalize=False)
         self.add_cleanup(tree.lock_tree_write().unlock)
         self._run(tree, names_list, rel_names, after)
 
     def run_auto(self, names_list, after, dry_run):
         if names_list is not None and len(names_list) > 1:
-            raise errors.BzrCommandError('Only one path may be specified to'
-                                         ' --auto.')
+            raise errors.BzrCommandError(gettext('Only one path may be specified to'
+                                         ' --auto.'))
         if after:
-            raise errors.BzrCommandError('--after cannot be specified with'
-                                         ' --auto.')
+            raise errors.BzrCommandError(gettext('--after cannot be specified with'
+                                         ' --auto.'))
         work_tree, file_list = WorkingTree.open_containing_paths(
             names_list, default_directory='.')
         self.add_cleanup(work_tree.lock_tree_write().unlock)
@@ -835,9 +929,9 @@
                     self.outf.write("%s => %s\n" % (src, dest))
         else:
             if len(names_list) != 2:
-                raise errors.BzrCommandError('to mv multiple files the'
+                raise errors.BzrCommandError(gettext('to mv multiple files the'
                                              ' destination must be a versioned'
-                                             ' directory')
+                                             ' directory'))
 
             # for cicp file-systems: the src references an existing inventory
             # item:
@@ -902,10 +996,14 @@
     match the remote one, use pull --overwrite. This will work even if the two
     branches have diverged.
 
-    If there is no default location set, the first pull will set it.  After
-    that, you can omit the location to use the default.  To change the
-    default, use --remember. The value will only be saved if the remote
-    location can be accessed.
+    If there is no default location set, the first pull will set it (use
+    --no-remember to avoid setting it). After that, you can omit the
+    location to use the default.  To change the default, use --remember. The
+    value will only be saved if the remote location can be accessed.
+
+    The --verbose option will display the revisions pulled using the log_format
+    configuration option. You can use a different format by overriding it with
+    -Olog_format=<other_format>.
 
     Note: The location can be specified either in the form of a branch,
     or in the form of a path to a file containing a merge directive generated
@@ -924,13 +1022,16 @@
                  "branch.  Local pulls are not applied to "
                  "the master branch."
             ),
+        Option('show-base',
+            help="Show base revision text in conflicts.")
         ]
     takes_args = ['location?']
     encoding_type = 'replace'
 
-    def run(self, location=None, remember=False, overwrite=False,
+    def run(self, location=None, remember=None, overwrite=False,
             revision=None, verbose=False,
-            directory=None, local=False):
+            directory=None, local=False,
+            show_base=False):
         # FIXME: too much stuff is in the command class
         revision_id = None
         mergeable = None
@@ -945,6 +1046,9 @@
             branch_to = Branch.open_containing(directory)[0]
             self.add_cleanup(branch_to.lock_write().unlock)
 
+        if tree_to is None and show_base:
+            raise errors.BzrCommandError(gettext("Need working tree for --show-base."))
+
         if local and not branch_to.get_bound_location():
             raise errors.LocalRequiresBoundBranch()
 
@@ -959,20 +1063,20 @@
         stored_loc = branch_to.get_parent()
         if location is None:
             if stored_loc is None:
-                raise errors.BzrCommandError("No pull location known or"
-                                             " specified.")
+                raise errors.BzrCommandError(gettext("No pull location known or"
+                                             " specified."))
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
                 if not is_quiet():
-                    self.outf.write("Using saved parent location: %s\n" % display_url)
+                    self.outf.write(gettext("Using saved parent location: %s\n") % display_url)
                 location = stored_loc
 
         revision = _get_one_revision('pull', revision)
         if mergeable is not None:
             if revision is not None:
-                raise errors.BzrCommandError(
-                    'Cannot use -r with merge directives or bundles')
+                raise errors.BzrCommandError(gettext(
+                    'Cannot use -r with merge directives or bundles'))
             mergeable.install_revisions(branch_to.repository)
             base_revision_id, revision_id, verified = \
                 mergeable.get_merge_request(branch_to.repository)
@@ -981,8 +1085,9 @@
             branch_from = Branch.open(location,
                 possible_transports=possible_transports)
             self.add_cleanup(branch_from.lock_read().unlock)
-
-            if branch_to.get_parent() is None or remember:
+            # Remembers if asked explicitly or no previous location is set
+            if (remember
+                or (remember is None and branch_to.get_parent() is None)):
                 branch_to.set_parent(branch_from.base)
 
         if revision is not None:
@@ -995,7 +1100,7 @@
                 view_info=view_info)
             result = tree_to.pull(
                 branch_from, overwrite, revision_id, change_reporter,
-                possible_transports=possible_transports, local=local)
+                local=local, show_base=show_base)
         else:
             result = branch_to.pull(
                 branch_from, overwrite, revision_id, local=local)
@@ -1005,6 +1110,10 @@
             log.show_branch_change(
                 branch_to, self.outf, result.old_revno,
                 result.old_revid)
+        if getattr(result, 'tag_conflicts', None):
+            return 1
+        else:
+            return 0
 
 
 class cmd_push(Command):
@@ -1027,10 +1136,14 @@
     do a merge (see bzr help merge) from the other branch, and commit that.
     After that you will be able to do a push without '--overwrite'.
 
-    If there is no default push location set, the first push will set it.
-    After that, you can omit the location to use the default.  To change the
-    default, use --remember. The value will only be saved if the remote
-    location can be accessed.
+    If there is no default push location set, the first push will set it (use
+    --no-remember to avoid setting it).  After that, you can omit the
+    location to use the default.  To change the default, use --remember. The
+    value will only be saved if the remote location can be accessed.
+
+    The --verbose option will display the revisions pushed using the log_format
+    configuration option. You can use a different format by overriding it with
+    -Olog_format=<other_format>.
     """
 
     _see_also = ['pull', 'update', 'working-trees']
@@ -1057,21 +1170,24 @@
         Option('strict',
                help='Refuse to push if there are uncommitted changes in'
                ' the working tree, --no-strict disables the check.'),
+        Option('no-tree',
+               help="Don't populate the working tree, even for protocols"
+               " that support it."),
         ]
     takes_args = ['location?']
     encoding_type = 'replace'
 
-    def run(self, location=None, remember=False, overwrite=False,
+    def run(self, location=None, remember=None, overwrite=False,
         create_prefix=False, verbose=False, revision=None,
         use_existing_dir=False, directory=None, stacked_on=None,
-        stacked=False, strict=None):
+        stacked=False, strict=None, no_tree=False):
         from bzrlib.push import _show_push_branch
 
         if directory is None:
             directory = '.'
         # Get the source branch
         (tree, br_from,
-         _unused) = bzrdir.BzrDir.open_containing_tree_or_branch(directory)
+         _unused) = controldir.ControlDir.open_containing_tree_or_branch(directory)
         # Get the tip's revision_id
         revision = _get_one_revision('push', revision)
         if revision is not None:
@@ -1098,25 +1214,25 @@
                     # error by the feedback given to them. RBC 20080227.
                     stacked_on = parent_url
             if not stacked_on:
-                raise errors.BzrCommandError(
-                    "Could not determine branch to refer to.")
+                raise errors.BzrCommandError(gettext(
+                    "Could not determine branch to refer to."))
 
         # Get the destination location
         if location is None:
             stored_loc = br_from.get_push_location()
             if stored_loc is None:
-                raise errors.BzrCommandError(
-                    "No push location known or specified.")
+                raise errors.BzrCommandError(gettext(
+                    "No push location known or specified."))
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
-                self.outf.write("Using saved push location: %s\n" % display_url)
+                note(gettext("Using saved push location: %s") % display_url)
                 location = stored_loc
 
         _show_push_branch(br_from, revision_id, location, self.outf,
             verbose=verbose, overwrite=overwrite, remember=remember,
             stacked_on=stacked_on, create_prefix=create_prefix,
-            use_existing_dir=use_existing_dir)
+            use_existing_dir=use_existing_dir, no_tree=no_tree)
 
 
 class cmd_branch(Command):
@@ -1131,6 +1247,8 @@
 
     To retrieve the branch as of a particular revision, supply the --revision
     parameter, as in "branch foo/bar -r 5".
+
+    The synonyms 'clone' and 'get' for this command are deprecated.
     """
 
     _see_also = ['checkout']
@@ -1166,7 +1284,13 @@
             files_from=None):
         from bzrlib import switch as _mod_switch
         from bzrlib.tag import _merge_tags_if_possible
-        accelerator_tree, br_from = bzrdir.BzrDir.open_tree_or_branch(
+        if self.invoked_as in ['get', 'clone']:
+            ui.ui_factory.show_user_warning(
+                'deprecated_command',
+                deprecated_name=self.invoked_as,
+                recommended_name='branch',
+                deprecated_in_version='2.4')
+        accelerator_tree, br_from = controldir.ControlDir.open_tree_or_branch(
             from_location)
         if not (hardlink or files_from):
             # accelerator_tree is usually slower because you have to read N
@@ -1191,56 +1315,100 @@
             to_transport.mkdir('.')
         except errors.FileExists:
             if not use_existing_dir:
-                raise errors.BzrCommandError('Target directory "%s" '
-                    'already exists.' % to_location)
+                raise errors.BzrCommandError(gettext('Target directory "%s" '
+                    'already exists.') % to_location)
             else:
                 try:
-                    bzrdir.BzrDir.open_from_transport(to_transport)
+                    to_dir = controldir.ControlDir.open_from_transport(
+                        to_transport)
                 except errors.NotBranchError:
-                    pass
+                    to_dir = None
                 else:
-                    raise errors.AlreadyBranchError(to_location)
+                    try:
+                        to_dir.open_branch()
+                    except errors.NotBranchError:
+                        pass
+                    else:
+                        raise errors.AlreadyBranchError(to_location)
         except errors.NoSuchFile:
-            raise errors.BzrCommandError('Parent of "%s" does not exist.'
+            raise errors.BzrCommandError(gettext('Parent of "%s" does not exist.')
                                          % to_location)
-        try:
-            # preserve whatever source format we have.
-            dir = br_from.bzrdir.sprout(to_transport.base, revision_id,
-                                        possible_transports=[to_transport],
-                                        accelerator_tree=accelerator_tree,
-                                        hardlink=hardlink, stacked=stacked,
-                                        force_new_repo=standalone,
-                                        create_tree_if_local=not no_tree,
-                                        source_branch=br_from)
-            branch = dir.open_branch()
-        except errors.NoSuchRevision:
-            to_transport.delete_tree('.')
-            msg = "The branch %s has no revision %s." % (from_location,
-                revision)
-            raise errors.BzrCommandError(msg)
+        else:
+            to_dir = None
+        if to_dir is None:
+            try:
+                # preserve whatever source format we have.
+                to_dir = br_from.bzrdir.sprout(to_transport.base, revision_id,
+                                            possible_transports=[to_transport],
+                                            accelerator_tree=accelerator_tree,
+                                            hardlink=hardlink, stacked=stacked,
+                                            force_new_repo=standalone,
+                                            create_tree_if_local=not no_tree,
+                                            source_branch=br_from)
+                branch = to_dir.open_branch()
+            except errors.NoSuchRevision:
+                to_transport.delete_tree('.')
+                msg = gettext("The branch {0} has no revision {1}.").format(
+                    from_location, revision)
+                raise errors.BzrCommandError(msg)
+        else:
+            branch = br_from.sprout(to_dir, revision_id=revision_id)
         _merge_tags_if_possible(br_from, branch)
         # If the source branch is stacked, the new branch may
         # be stacked whether we asked for that explicitly or not.
         # We therefore need a try/except here and not just 'if stacked:'
         try:
-            note('Created new stacked branch referring to %s.' %
+            note(gettext('Created new stacked branch referring to %s.') %
                 branch.get_stacked_on_url())
         except (errors.NotStacked, errors.UnstackableBranchFormat,
             errors.UnstackableRepositoryFormat), e:
-            note('Branched %d revision(s).' % branch.revno())
+            note(ngettext('Branched %d revision.', 'Branched %d revisions.', branch.revno()) % branch.revno())
         if bind:
             # Bind to the parent
             parent_branch = Branch.open(from_location)
             branch.bind(parent_branch)
-            note('New branch bound to %s' % from_location)
+            note(gettext('New branch bound to %s') % from_location)
         if switch:
             # Switch to the new branch
             wt, _ = WorkingTree.open_containing('.')
             _mod_switch.switch(wt.bzrdir, branch)
-            note('Switched to branch: %s',
+            note(gettext('Switched to branch: %s'),
                 urlutils.unescape_for_display(branch.base, 'utf-8'))
 
 
+class cmd_branches(Command):
+    __doc__ = """List the branches available at the current location.
+
+    This command will print the names of all the branches at the current
+    location.
+    """
+
+    takes_args = ['location?']
+    takes_options = [
+                  Option('recursive', short_name='R',
+                         help='Recursively scan for branches rather than '
+                              'just looking in the specified location.')]
+
+    def run(self, location=".", recursive=False):
+        if recursive:
+            t = transport.get_transport(location)
+            if not t.listable():
+                raise errors.BzrCommandError(
+                    "Can't scan this type of location.")
+            for b in controldir.ControlDir.find_branches(t):
+                self.outf.write("%s\n" % urlutils.unescape_for_display(
+                    urlutils.relative_url(t.base, b.base),
+                    self.outf.encoding).rstrip("/"))
+        else:
+            dir = controldir.ControlDir.open_containing(location)[0]
+            for branch in dir.list_branches():
+                if branch.name is None:
+                    self.outf.write(gettext(" (default)\n"))
+                else:
+                    self.outf.write(" %s\n" % branch.name.encode(
+                        self.outf.encoding))
+
+
 class cmd_checkout(Command):
     __doc__ = """Create a new checkout of an existing branch.
 
@@ -1285,7 +1453,7 @@
         if branch_location is None:
             branch_location = osutils.getcwd()
             to_location = branch_location
-        accelerator_tree, source = bzrdir.BzrDir.open_tree_or_branch(
+        accelerator_tree, source = controldir.ControlDir.open_tree_or_branch(
             branch_location)
         if not (hardlink or files_from):
             # accelerator_tree is usually slower because you have to read N
@@ -1346,29 +1514,56 @@
 
 
 class cmd_update(Command):
-    __doc__ = """Update a tree to have the latest code committed to its branch.
-
-    This will perform a merge into the working tree, and may generate
-    conflicts. If you have any local changes, you will still
-    need to commit them after the update for the update to be complete.
-
-    If you want to discard your local changes, you can just do a
-    'bzr revert' instead of 'bzr commit' after the update.
-
-    If the tree's branch is bound to a master branch, it will also update
+    __doc__ = """Update a working tree to a new revision.
+
+    This will perform a merge of the destination revision (the tip of the
+    branch, or the specified revision) into the working tree, and then make
+    that revision the basis revision for the working tree.  
+
+    You can use this to visit an older revision, or to update a working tree
+    that is out of date from its branch.
+    
+    If there are any uncommitted changes in the tree, they will be carried
+    across and remain as uncommitted changes after the update.  To discard
+    these changes, use 'bzr revert'.  The uncommitted changes may conflict
+    with the changes brought in by the change in basis revision.
+
+    If the tree's branch is bound to a master branch, bzr will also update
     the branch from the master.
+
+    You cannot update just a single file or directory, because each Bazaar
+    working tree has just a single basis revision.  If you want to restore a
+    file that has been removed locally, use 'bzr revert' instead of 'bzr
+    update'.  If you want to restore a file to its state in a previous
+    revision, use 'bzr revert' with a '-r' option, or use 'bzr cat' to write
+    out the old content of that file to a new location.
+
+    The 'dir' argument, if given, must be the location of the root of a
+    working tree to update.  By default, the working tree that contains the 
+    current working directory is used.
     """
 
     _see_also = ['pull', 'working-trees', 'status-flags']
     takes_args = ['dir?']
-    takes_options = ['revision']
+    takes_options = ['revision',
+                     Option('show-base',
+                            help="Show base revision text in conflicts."),
+                     ]
     aliases = ['up']
 
-    def run(self, dir='.', revision=None):
+    def run(self, dir=None, revision=None, show_base=None):
         if revision is not None and len(revision) != 1:
-            raise errors.BzrCommandError(
-                        "bzr update --revision takes exactly one revision")
-        tree = WorkingTree.open_containing(dir)[0]
+            raise errors.BzrCommandError(gettext(
+                "bzr update --revision takes exactly one revision"))
+        if dir is None:
+            tree = WorkingTree.open_containing('.')[0]
+        else:
+            tree, relpath = WorkingTree.open_containing(dir)
+            if relpath:
+                # See bug 557886.
+                raise errors.BzrCommandError(gettext(
+                    "bzr update can only update a whole tree, "
+                    "not a file or subdirectory"))
         branch = tree.branch
         possible_transports = []
         master = branch.get_master_branch(
@@ -1398,8 +1593,8 @@
             revision_id = branch.last_revision()
         if revision_id == _mod_revision.ensure_null(tree.last_revision()):
             revno = branch.revision_id_to_dotted_revno(revision_id)
-            note("Tree is up to date at revision %s of branch %s" %
-                ('.'.join(map(str, revno)), branch_location))
+            note(gettext("Tree is up to date at revision {0} of branch {1}"
+                        ).format('.'.join(map(str, revno)), branch_location))
             return 0
         view_info = _get_view_info_for_change_reporter(tree)
         change_reporter = delta._ChangeReporter(
@@ -1410,21 +1605,22 @@
                 change_reporter,
                 possible_transports=possible_transports,
                 revision=revision_id,
-                old_tip=old_tip)
+                old_tip=old_tip,
+                show_base=show_base)
         except errors.NoSuchRevision, e:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                                   "branch has no revision %s\n"
                                   "bzr update --revision only works"
-                                  " for a revision in the branch history"
+                                  " for a revision in the branch history")
                                   % (e.revision))
         revno = tree.branch.revision_id_to_dotted_revno(
             _mod_revision.ensure_null(tree.last_revision()))
-        note('Updated to revision %s of branch %s' %
-             ('.'.join(map(str, revno)), branch_location))
+        note(gettext('Updated to revision {0} of branch {1}').format(
+             '.'.join(map(str, revno)), branch_location))
         parent_ids = tree.get_parent_ids()
         if parent_ids[1:] and parent_ids[1:] != existing_pending_merges:
-            note('Your local commits will now show as pending merges with '
-                 "'bzr status', and can be committed with 'bzr commit'.")
+            note(gettext('Your local commits will now show as pending merges with '
+                 "'bzr status', and can be committed with 'bzr commit'."))
         if conflicts != 0:
             return 1
         else:
@@ -1471,17 +1667,18 @@
         else:
             noise_level = 0
         from bzrlib.info import show_bzrdir_info
-        show_bzrdir_info(bzrdir.BzrDir.open_containing(location)[0],
+        show_bzrdir_info(controldir.ControlDir.open_containing(location)[0],
                          verbose=noise_level, outfile=self.outf)
 
 
 class cmd_remove(Command):
     __doc__ = """Remove files or directories.
 
-    This makes bzr stop tracking changes to the specified files. bzr will delete
-    them if they can easily be recovered using revert. If no options or
-    parameters are given bzr will scan for files that are being tracked by bzr
-    but missing in your tree and stop tracking them for you.
+    This makes Bazaar stop tracking changes to the specified files. Bazaar will
+    delete them if they can easily be recovered using revert otherwise they
+    will be backed up (adding an extention of the form .~#~). If no options or
+    parameters are given Bazaar will scan for files that are being tracked by
+    Bazaar but missing in your tree and stop tracking them for you.
     """
     takes_args = ['file*']
     takes_options = ['verbose',
@@ -1489,16 +1686,22 @@
         RegistryOption.from_kwargs('file-deletion-strategy',
             'The file deletion mode to be used.',
             title='Deletion Strategy', value_switches=True, enum_switch=False,
-            safe='Only delete files if they can be'
-                 ' safely recovered (default).',
+            safe='Backup changed files (default).',
             keep='Delete from bzr but leave the working copy.',
+            no_backup='Don\'t backup changed files.',
             force='Delete all the specified files, even if they can not be '
-                'recovered and even if they are non-empty directories.')]
+                'recovered and even if they are non-empty directories. '
+                '(deprecated, use no-backup)')]
     aliases = ['rm', 'del']
     encoding_type = 'replace'
 
     def run(self, file_list, verbose=False, new=False,
         file_deletion_strategy='safe'):
+        if file_deletion_strategy == 'force':
+            note(gettext("(The --force option is deprecated, rather use --no-backup "
+                "in future.)"))
+            file_deletion_strategy = 'no-backup'
+
         tree, file_list = WorkingTree.open_containing_paths(file_list)
 
         if file_list is not None:
@@ -1512,7 +1715,7 @@
                 specific_files=file_list).added
             file_list = sorted([f[0] for f in added], reverse=True)
             if len(file_list) == 0:
-                raise errors.BzrCommandError('No matching files.')
+                raise errors.BzrCommandError(gettext('No matching files.'))
         elif file_list is None:
             # missing files show up in iter_changes(basis) as
             # versioned-with-no-kind.
@@ -1525,7 +1728,7 @@
             file_deletion_strategy = 'keep'
         tree.remove(file_list, verbose=verbose, to_file=self.outf,
             keep_files=file_deletion_strategy=='keep',
-            force=file_deletion_strategy=='force')
+            force=(file_deletion_strategy=='no-backup'))
 
 
 class cmd_file_id(Command):
@@ -1593,11 +1796,17 @@
 
     _see_also = ['check']
     takes_args = ['branch?']
+    takes_options = [
+        Option('canonicalize-chks',
+               help='Make sure CHKs are in canonical form (repairs '
+                    'bug 522637).',
+               hidden=True),
+        ]
 
-    def run(self, branch="."):
+    def run(self, branch=".", canonicalize_chks=False):
         from bzrlib.reconcile import reconcile
-        dir = bzrdir.BzrDir.open(branch)
-        reconcile(dir)
+        dir = controldir.ControlDir.open(branch)
+        reconcile(dir, canonicalize_chks=canonicalize_chks)
 
 
 class cmd_revision_history(Command):
@@ -1611,7 +1820,11 @@
     @display_command
     def run(self, location="."):
         branch = Branch.open_containing(location)[0]
-        for revid in branch.revision_history():
+        self.add_cleanup(branch.lock_read().unlock)
+        graph = branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+        for revid in reversed(history):
             self.outf.write(revid)
             self.outf.write('\n')
 
@@ -1635,9 +1848,13 @@
             b = wt.branch
             last_revision = wt.last_revision()
 
-        revision_ids = b.repository.get_ancestry(last_revision)
-        revision_ids.pop(0)
-        for revision_id in revision_ids:
+        self.add_cleanup(b.repository.lock_read().unlock)
+        graph = b.repository.get_graph()
+        revisions = [revid for revid, parents in
+            graph.iter_ancestry([last_revision])]
+        for revision_id in reversed(revisions):
+            if _mod_revision.is_null(revision_id):
+                continue
             self.outf.write(revision_id + '\n')
 
 
@@ -1674,18 +1891,20 @@
                 help='Specify a format for this branch. '
                 'See "help formats".',
                 lazy_registry=('bzrlib.bzrdir', 'format_registry'),
-                converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
+                converter=lambda name: controldir.format_registry.make_bzrdir(name),
                 value_switches=True,
                 title="Branch format",
                 ),
          Option('append-revisions-only',
                 help='Never change revnos or the existing log.'
-                '  Append revisions to it only.')
+                '  Append revisions to it only.'),
+         Option('no-tree',
+                'Create a branch without a working tree.')
          ]
     def run(self, location=None, format=None, append_revisions_only=False,
-            create_prefix=False):
+            create_prefix=False, no_tree=False):
         if format is None:
-            format = bzrdir.format_registry.make_bzrdir('default')
+            format = controldir.format_registry.make_bzrdir('default')
         if location is None:
             location = u'.'
 
@@ -1700,20 +1919,25 @@
             to_transport.ensure_base()
         except errors.NoSuchFile:
             if not create_prefix:
-                raise errors.BzrCommandError("Parent directory of %s"
+                raise errors.BzrCommandError(gettext("Parent directory of %s"
                     " does not exist."
                     "\nYou may supply --create-prefix to create all"
-                    " leading parent directories."
+                    " leading parent directories.")
                     % location)
             to_transport.create_prefix()
 
         try:
-            a_bzrdir = bzrdir.BzrDir.open_from_transport(to_transport)
+            a_bzrdir = controldir.ControlDir.open_from_transport(to_transport)
         except errors.NotBranchError:
             # really a NotBzrDir error...
-            create_branch = bzrdir.BzrDir.create_branch_convenience
+            create_branch = controldir.ControlDir.create_branch_convenience
+            if no_tree:
+                force_new_tree = False
+            else:
+                force_new_tree = None
             branch = create_branch(to_transport.base, format=format,
-                                   possible_transports=[to_transport])
+                                   possible_transports=[to_transport],
+                                   force_new_tree=force_new_tree)
             a_bzrdir = branch.bzrdir
         else:
             from bzrlib.transport.local import LocalTransport
@@ -1723,13 +1947,14 @@
                         raise errors.BranchExistsWithoutWorkingTree(location)
                 raise errors.AlreadyBranchError(location)
             branch = a_bzrdir.create_branch()
-            a_bzrdir.create_workingtree()
+            if not no_tree and not a_bzrdir.has_workingtree():
+                a_bzrdir.create_workingtree()
         if append_revisions_only:
             try:
                 branch.set_append_revisions_only(True)
             except errors.UpgradeRequired:
-                raise errors.BzrCommandError('This branch format cannot be set'
-                    ' to append-revisions-only.  Try --default.')
+                raise errors.BzrCommandError(gettext('This branch format cannot be set'
+                    ' to append-revisions-only.  Try --default.'))
         if not is_quiet():
             from bzrlib.info import describe_layout, describe_format
             try:
@@ -1739,7 +1964,8 @@
             repository = branch.repository
             layout = describe_layout(repository, branch, tree).lower()
             format = describe_format(a_bzrdir, repository, branch, tree)
-            self.outf.write("Created a %s (format: %s)\n" % (layout, format))
+            self.outf.write(gettext("Created a {0} (format: {1})\n").format(
+                  layout, format))
             if repository.is_shared():
                 #XXX: maybe this can be refactored into transport.path_or_url()
                 url = repository.bzrdir.root_transport.external_url()
@@ -1747,7 +1973,7 @@
                     url = urlutils.local_path_from_url(url)
                 except errors.InvalidURL:
                     pass
-                self.outf.write("Using shared repository: %s\n" % url)
+                self.outf.write(gettext("Using shared repository: %s\n") % url)
 
 
 class cmd_init_repository(Command):
@@ -1783,8 +2009,8 @@
     takes_options = [RegistryOption('format',
                             help='Specify a format for this repository. See'
                                  ' "bzr help formats" for details.',
-                            lazy_registry=('bzrlib.bzrdir', 'format_registry'),
-                            converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
+                            lazy_registry=('bzrlib.controldir', 'format_registry'),
+                            converter=lambda name: controldir.format_registry.make_bzrdir(name),
                             value_switches=True, title='Repository format'),
                      Option('no-trees',
                              help='Branches in the repository will default to'
@@ -1794,7 +2020,7 @@
 
     def run(self, location, format=None, no_trees=False):
         if format is None:
-            format = bzrdir.format_registry.make_bzrdir('default')
+            format = controldir.format_registry.make_bzrdir('default')
 
         if location is None:
             location = '.'
@@ -1823,6 +2049,13 @@
     "bzr diff -p1" is equivalent to "bzr diff --prefix old/:new/", and
     produces patches suitable for "patch -p1".
 
+    Note that when using the -r argument with a range of revisions, the
+    differences are computed between the two specified revisions.  That
+    is, the command does not show the changes introduced by the first 
+    revision in the range.  This differs from the interpretation of 
+    revision ranges used by "bzr log" which includes the first revision
+    in the range.
+
     :Exit values:
         1 - changed
         2 - unrepresentable changes
@@ -1846,7 +2079,11 @@
 
             bzr diff -r1..3 xxx
 
-        To see the changes introduced in revision X::
+        The changes introduced by revision 2 (equivalent to -r1..2)::
+
+            bzr diff -c2
+
+        To see the changes introduced by revision X::
         
             bzr diff -cX
 
@@ -1856,9 +2093,10 @@
 
             bzr diff -r<chosen_parent>..X
 
-        The changes introduced by revision 2 (equivalent to -r1..2)::
+        The changes between the current revision and the previous revision
+        (equivalent to -c-1 and -r-2..-1)
 
-            bzr diff -c2
+            bzr diff -r-2..
 
         Show just the differences for file NEWS::
 
@@ -1879,12 +2117,16 @@
         Same as 'bzr diff' but prefix paths with old/ and new/::
 
             bzr diff --prefix old/:new/
+            
+        Show the differences using a custom diff program with options::
+        
+            bzr diff --using /usr/bin/diff --diff-options -wu
     """
     _see_also = ['status']
     takes_args = ['file*']
     takes_options = [
         Option('diff-options', type=str,
-               help='Pass these options to the diff program.'),
+               help='Pass these options to the external diff program.'),
         Option('prefix', type=str,
                short_name='p',
                help='Set prefixes added to old and new filenames, as '
@@ -1904,9 +2146,10 @@
             type=unicode,
             ),
         RegistryOption('format',
+            short_name='F',
             help='Diff format to use.',
             lazy_registry=('bzrlib.diff', 'format_registry'),
-            value_switches=False, title='Diff format'),
+            title='Diff format'),
         ]
     aliases = ['di', 'dif']
     encoding_type = 'exact'
@@ -1927,21 +2170,18 @@
         elif ':' in prefix:
             old_label, new_label = prefix.split(":")
         else:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 '--prefix expects two values separated by a colon'
-                ' (eg "old/:new/")')
-
-        if using is not None and diff_options is not None:
-            raise errors.BzrCommandError(
-            '--diff-options and --using are mutually exclusive.')
+                ' (eg "old/:new/")'))
 
         if revision and len(revision) > 2:
-            raise errors.BzrCommandError('bzr diff --revision takes exactly'
-                                         ' one or two revision specifiers')
+            raise errors.BzrCommandError(gettext('bzr diff --revision takes exactly'
+                                         ' one or two revision specifiers'))
 
         if using is not None and format is not None:
-            raise errors.BzrCommandError('--using and --format are mutually '
-                'exclusive.')
+            raise errors.BzrCommandError(gettext(
+                '{0} and {1} are mutually exclusive').format(
+                '--using', '--format'))
 
         (old_tree, new_tree,
          old_branch, new_branch,
@@ -1997,7 +2237,9 @@
     @display_command
     def run(self, null=False, directory=u'.'):
         tree = WorkingTree.open_containing(directory)[0]
+        self.add_cleanup(tree.lock_read().unlock)
         td = tree.changes_from(tree.basis_tree())
+        self.cleanup_now()
         for path, id, kind, text_modified, meta_modified in td.modified:
             if null:
                 self.outf.write(path + '\0')
@@ -2022,7 +2264,7 @@
         basis_inv = basis.inventory
         inv = wt.inventory
         for file_id in inv:
-            if file_id in basis_inv:
+            if basis_inv.has_id(file_id):
                 continue
             if inv.is_root(file_id) and len(basis_inv) == 0:
                 continue
@@ -2053,7 +2295,7 @@
     try:
         return int(limitstring)
     except ValueError:
-        msg = "The limit argument must be an integer."
+        msg = gettext("The limit argument must be an integer.")
         raise errors.BzrCommandError(msg)
 
 
@@ -2061,7 +2303,7 @@
     try:
         return int(s)
     except ValueError:
-        msg = "The levels argument must be an integer."
+        msg = gettext("The levels argument must be an integer.")
         raise errors.BzrCommandError(msg)
 
 
@@ -2177,8 +2419,11 @@
 
     :Other filtering:
 
-      The --message option can be used for finding revisions that match a
-      regular expression in a commit message.
+      The --match option can be used for finding revisions that match a
+      regular expression in a commit message, committer, author or bug.
+      Specifying the option several times will match any of the supplied
+      expressions. --match-author, --match-bugs, --match-committer and
+      --match-message can be used to only match a specific field.
 
     :Tips & tricks:
 
@@ -2244,10 +2489,10 @@
                    argname='N',
                    type=_parse_levels),
             Option('message',
-                   short_name='m',
                    help='Show revisions whose message matches this '
                         'regular expression.',
-                   type=str),
+                   type=str,
+                   hidden=True),
             Option('limit',
                    short_name='l',
                    help='Limit the output to the first N revisions.',
@@ -2256,12 +2501,39 @@
             Option('show-diff',
                    short_name='p',
                    help='Show changes made in each revision as a patch.'),
-            Option('include-merges',
+            Option('include-merged',
                    help='Show merged revisions like --levels 0 does.'),
+            Option('include-merges', hidden=True,
+                   help='Historical alias for --include-merged.'),
+            Option('omit-merges',
+                   help='Do not report commits with more than one parent.'),
             Option('exclude-common-ancestry',
                    help='Display only the revisions that are not part'
                    ' of both ancestries (require -rX..Y)'
-                   )
+                   ),
+            Option('signatures',
+                   help='Show digital signature validity'),
+            ListOption('match',
+                short_name='m',
+                help='Show revisions whose properties match this '
+                'expression.',
+                type=str),
+            ListOption('match-message',
+                   help='Show revisions whose message matches this '
+                   'expression.',
+                type=str),
+            ListOption('match-committer',
+                   help='Show revisions whose committer matches this '
+                   'expression.',
+                type=str),
+            ListOption('match-author',
+                   help='Show revisions whose authors match this '
+                   'expression.',
+                type=str),
+            ListOption('match-bugs',
+                   help='Show revisions whose bugs match this '
+                   'expression.',
+                type=str)
             ]
     encoding_type = 'replace'
 
@@ -2277,9 +2549,17 @@
             message=None,
             limit=None,
             show_diff=False,
-            include_merges=False,
+            include_merged=None,
             authors=None,
             exclude_common_ancestry=False,
+            signatures=False,
+            match=None,
+            match_message=None,
+            match_committer=None,
+            match_author=None,
+            match_bugs=None,
+            omit_merges=False,
+            include_merges=symbol_versioning.DEPRECATED_PARAMETER,
             ):
         from bzrlib.log import (
             Logger,
@@ -2287,23 +2567,40 @@
             _get_info_for_log_files,
             )
         direction = (forward and 'forward') or 'reverse'
+        if symbol_versioning.deprecated_passed(include_merges):
+            ui.ui_factory.show_user_warning(
+                'deprecated_command_option',
+                deprecated_name='--include-merges',
+                recommended_name='--include-merged',
+                deprecated_in_version='2.5',
+                command=self.invoked_as)
+            if include_merged is None:
+                include_merged = include_merges
+            else:
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--include-merges', '--include-merged'))
+        if include_merged is None:
+            include_merged = False
         if (exclude_common_ancestry
             and (revision is None or len(revision) != 2)):
-            raise errors.BzrCommandError(
-                '--exclude-common-ancestry requires -r with two revisions')
-        if include_merges:
+            raise errors.BzrCommandError(gettext(
+                '--exclude-common-ancestry requires -r with two revisions'))
+        if include_merged:
             if levels is None:
                 levels = 0
             else:
-                raise errors.BzrCommandError(
-                    '--levels and --include-merges are mutually exclusive')
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--levels', '--include-merged'))
 
         if change is not None:
             if len(change) > 1:
                 raise errors.RangeInChangeOption()
             if revision is not None:
-                raise errors.BzrCommandError(
-                    '--revision and --change are mutually exclusive')
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--revision', '--change'))
             else:
                 revision = change
 
@@ -2315,8 +2612,8 @@
                 revision, file_list, self.add_cleanup)
             for relpath, file_id, kind in file_info_list:
                 if file_id is None:
-                    raise errors.BzrCommandError(
-                        "Path unknown at end or start of revision range: %s" %
+                    raise errors.BzrCommandError(gettext(
+                        "Path unknown at end or start of revision range: %s") %
                         relpath)
                 # If the relpath is the top of the tree, we log everything
                 if relpath == '':
@@ -2334,11 +2631,18 @@
                 location = revision[0].get_branch()
             else:
                 location = '.'
-            dir, relpath = bzrdir.BzrDir.open_containing(location)
+            dir, relpath = controldir.ControlDir.open_containing(location)
             b = dir.open_branch()
             self.add_cleanup(b.lock_read().unlock)
             rev1, rev2 = _get_revision_range(revision, b, self.name())
 
+        if b.get_config().validate_signatures_in_log():
+            signatures = True
+
+        if signatures:
+            if not gpg.GPGStrategy.verify_signatures_available():
+                raise errors.GpgmeNotInstalled(None)
+
         # Decide on the type of delta & diff filtering to use
         # TODO: add an --all-files option to make this configurable & consistent
         if not verbose:
@@ -2381,6 +2685,18 @@
         match_using_deltas = (len(file_ids) != 1 or filter_by_dir
             or delta_type or partial_history)
 
+        match_dict = {}
+        if match:
+            match_dict[''] = match
+        if match_message:
+            match_dict['message'] = match_message
+        if match_committer:
+            match_dict['committer'] = match_committer
+        if match_author:
+            match_dict['author'] = match_author
+        if match_bugs:
+            match_dict['bugs'] = match_bugs
+
         # Build the LogRequest and execute it
         if len(file_ids) == 0:
             file_ids = None
@@ -2389,7 +2705,8 @@
             start_revision=rev1, end_revision=rev2, limit=limit,
             message_search=message, delta_type=delta_type,
             diff_type=diff_type, _match_using_deltas=match_using_deltas,
-            exclude_common_ancestry=exclude_common_ancestry,
+            exclude_common_ancestry=exclude_common_ancestry, match=match_dict,
+            signature=signatures, omit_merges=omit_merges,
             )
         Logger(b, rqst).show(lf)
 
@@ -2412,9 +2729,9 @@
             # b is taken from revision[0].get_branch(), and
             # show_log will use its revision_history. Having
             # different branches will lead to weird behaviors.
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "bzr %s doesn't accept two revisions in different"
-                " branches." % command_name)
+                " branches.") % command_name)
         if start_spec.spec is None:
             # Avoid loading all the history.
             rev1 = RevisionInfo(branch, None, None)
@@ -2428,8 +2745,8 @@
         else:
             rev2 = end_spec.in_history(branch)
     else:
-        raise errors.BzrCommandError(
-            'bzr %s --revision takes one or two values.' % command_name)
+        raise errors.BzrCommandError(gettext(
+            'bzr %s --revision takes one or two values.') % command_name)
     return rev1, rev2
 
 
@@ -2506,10 +2823,10 @@
             null=False, kind=None, show_ids=False, path=None, directory=None):
 
         if kind and kind not in ('file', 'directory', 'symlink'):
-            raise errors.BzrCommandError('invalid kind specified')
+            raise errors.BzrCommandError(gettext('invalid kind specified'))
 
         if verbose and null:
-            raise errors.BzrCommandError('Cannot set both --verbose and --null')
+            raise errors.BzrCommandError(gettext('Cannot set both --verbose and --null'))
         all = not (unknown or versioned or ignored)
 
         selection = {'I':ignored, '?':unknown, 'V':versioned}
@@ -2518,8 +2835,8 @@
             fs_path = '.'
         else:
             if from_root:
-                raise errors.BzrCommandError('cannot specify both --from-root'
-                                             ' and PATH')
+                raise errors.BzrCommandError(gettext('cannot specify both --from-root'
+                                             ' and PATH'))
             fs_path = path
         tree, branch, relpath = \
             _open_directory_or_containing_tree_or_branch(fs_path, directory)
@@ -2541,7 +2858,7 @@
             if view_files:
                 apply_view = True
                 view_str = views.view_display_str(view_files)
-                note("Ignoring files outside view. View is %s" % view_str)
+                note(gettext("Ignoring files outside view. View is %s") % view_str)
 
         self.add_cleanup(tree.lock_read().unlock)
         for fp, fc, fkind, fid, entry in tree.list_files(include_root=False,
@@ -2633,8 +2950,13 @@
     Patterns prefixed with '!!' act as regular ignore patterns, but have
     precedence over the '!' exception patterns.
 
-    Note: ignore patterns containing shell wildcards must be quoted from
-    the shell on Unix.
+    :Notes: 
+        
+    * Ignore patterns containing shell wildcards must be quoted from
+      the shell on Unix.
+
+    * Ignore patterns starting with "#" act as comments in the ignore file.
+      To ignore patterns that begin with that character, use the "RE:" prefix.
 
     :Examples:
         Ignore the top level Makefile::
@@ -2649,6 +2971,10 @@
 
             bzr ignore "!special.class"
 
+        Ignore files whose name begins with the "#" character::
+
+            bzr ignore "RE:^#"
+
         Ignore .o files under the lib directory::
 
             bzr ignore "lib/**/*.o"
@@ -2662,7 +2988,7 @@
             bzr ignore "RE:(?!debian/).*"
         
         Ignore everything except the "local" toplevel directory,
-        but always ignore "*~" autosave files, even under local/::
+        but always ignore autosave files ending in ~, even under local/::
         
             bzr ignore "*"
             bzr ignore "!./local"
@@ -2685,23 +3011,27 @@
                 self.outf.write("%s\n" % pattern)
             return
         if not name_pattern_list:
-            raise errors.BzrCommandError("ignore requires at least one "
-                "NAME_PATTERN or --default-rules.")
+            raise errors.BzrCommandError(gettext("ignore requires at least one "
+                "NAME_PATTERN or --default-rules."))
         name_pattern_list = [globbing.normalize_pattern(p)
                              for p in name_pattern_list]
         bad_patterns = ''
+        bad_patterns_count = 0
         for p in name_pattern_list:
             if not globbing.Globster.is_pattern_valid(p):
+                bad_patterns_count += 1
                 bad_patterns += ('\n  %s' % p)
         if bad_patterns:
-            msg = ('Invalid ignore pattern(s) found. %s' % bad_patterns)
+            msg = (ngettext('Invalid ignore pattern found. %s', 
+                            'Invalid ignore patterns found. %s',
+                            bad_patterns_count) % bad_patterns)
             ui.ui_factory.show_error(msg)
             raise errors.InvalidPattern('')
         for name_pattern in name_pattern_list:
             if (name_pattern[0] == '/' or
                 (len(name_pattern) > 1 and name_pattern[1] == ':')):
-                raise errors.BzrCommandError(
-                    "NAME_PATTERN should not be an absolute path")
+                raise errors.BzrCommandError(gettext(
+                    "NAME_PATTERN should not be an absolute path"))
         tree, relpath = WorkingTree.open_containing(directory)
         ignores.tree_ignores_add_patterns(tree, name_pattern_list)
         ignored = globbing.Globster(name_pattern_list)
@@ -2714,10 +3044,10 @@
                 if ignored.match(filename):
                     matches.append(filename)
         if len(matches) > 0:
-            self.outf.write("Warning: the following files are version controlled and"
-                  " match your ignore pattern:\n%s"
+            self.outf.write(gettext("Warning: the following files are version "
+                  "controlled and match your ignore pattern:\n%s"
                   "\nThese files will continue to be version controlled"
-                  " unless you 'bzr remove' them.\n" % ("\n".join(matches),))
+                  " unless you 'bzr remove' them.\n") % ("\n".join(matches),))
 
 
 class cmd_ignored(Command):
@@ -2762,7 +3092,7 @@
         try:
             revno = int(revno)
         except ValueError:
-            raise errors.BzrCommandError("not a valid revision-number: %r"
+            raise errors.BzrCommandError(gettext("not a valid revision-number: %r")
                                          % revno)
         revid = WorkingTree.open_containing(directory)[0].branch.get_rev_id(revno)
         self.outf.write("%s\n" % revid)
@@ -2796,6 +3126,7 @@
          zip                          .zip
       =================       =========================
     """
+    encoding = 'exact'
     takes_args = ['dest', 'branch_or_subdir?']
     takes_options = ['directory',
         Option('format',
@@ -2828,7 +3159,7 @@
             export(rev_tree, dest, format, root, subdir, filtered=filters,
                    per_file_timestamps=per_file_timestamps)
         except errors.NoSuchExportFormat, e:
-            raise errors.BzrCommandError('Unsupported export format: %s' % e.format)
+            raise errors.BzrCommandError(gettext('Unsupported export format: %s') % e.format)
 
 
 class cmd_cat(Command):
@@ -2854,8 +3185,8 @@
     def run(self, filename, revision=None, name_from_revision=False,
             filters=False, directory=None):
         if revision is not None and len(revision) != 1:
-            raise errors.BzrCommandError("bzr cat --revision takes exactly"
-                                         " one revision specifier")
+            raise errors.BzrCommandError(gettext("bzr cat --revision takes exactly"
+                                         " one revision specifier"))
         tree, branch, relpath = \
             _open_directory_or_containing_tree_or_branch(filename, directory)
         self.add_cleanup(branch.lock_read().unlock)
@@ -2871,48 +3202,37 @@
 
         old_file_id = rev_tree.path2id(relpath)
 
+        # TODO: Split out this code to something that generically finds the
+        # best id for a path across one or more trees; it's like
+        # find_ids_across_trees but restricted to find just one. -- mbp
+        # 20110705.
         if name_from_revision:
             # Try in revision if requested
             if old_file_id is None:
-                raise errors.BzrCommandError(
-                    "%r is not present in revision %s" % (
+                raise errors.BzrCommandError(gettext(
+                    "{0!r} is not present in revision {1}").format(
                         filename, rev_tree.get_revision_id()))
             else:
-                content = rev_tree.get_file_text(old_file_id)
+                actual_file_id = old_file_id
         else:
             cur_file_id = tree.path2id(relpath)
-            found = False
-            if cur_file_id is not None:
-                # Then try with the actual file id
-                try:
-                    content = rev_tree.get_file_text(cur_file_id)
-                    found = True
-                except errors.NoSuchId:
-                    # The actual file id didn't exist at that time
-                    pass
-            if not found and old_file_id is not None:
-                # Finally try with the old file id
-                content = rev_tree.get_file_text(old_file_id)
-                found = True
-            if not found:
-                # Can't be found anywhere
-                raise errors.BzrCommandError(
-                    "%r is not present in revision %s" % (
+            if cur_file_id is not None and rev_tree.has_id(cur_file_id):
+                actual_file_id = cur_file_id
+            elif old_file_id is not None:
+                actual_file_id = old_file_id
+            else:
+                raise errors.BzrCommandError(gettext(
+                    "{0!r} is not present in revision {1}").format(
                         filename, rev_tree.get_revision_id()))
         if filtered:
-            from bzrlib.filters import (
-                ContentFilterContext,
-                filtered_output_bytes,
-                )
-            filters = rev_tree._content_filter_stack(relpath)
-            chunks = content.splitlines(True)
-            content = filtered_output_bytes(chunks, filters,
-                ContentFilterContext(relpath, rev_tree))
-            self.cleanup_now()
-            self.outf.writelines(content)
+            from bzrlib.filter_tree import ContentFilterTree
+            filter_tree = ContentFilterTree(rev_tree,
+                rev_tree._content_filter_stack)
+            content = filter_tree.get_file_text(actual_file_id)
         else:
-            self.cleanup_now()
-            self.outf.write(content)
+            content = rev_tree.get_file_text(actual_file_id)
+        self.cleanup_now()
+        self.outf.write(content)
 
 
 class cmd_local_time_offset(Command):
@@ -2979,32 +3299,7 @@
       to trigger updates to external systems like bug trackers. The --fixes
       option can be used to record the association between a revision and
       one or more bugs. See ``bzr help bugs`` for details.
-
-      A selective commit may fail in some cases where the committed
-      tree would be invalid. Consider::
-  
-        bzr init foo
-        mkdir foo/bar
-        bzr add foo/bar
-        bzr commit foo -m "committing foo"
-        bzr mv foo/bar foo/baz
-        mkdir foo/bar
-        bzr add foo/bar
-        bzr commit foo/bar -m "committing bar but not baz"
-  
-      In the example above, the last commit will fail by design. This gives
-      the user the opportunity to decide whether they want to commit the
-      rename at the same time, separately first, or not at all. (As a general
-      rule, when in doubt, Bazaar has a policy of Doing the Safe Thing.)
     """
-    # TODO: Run hooks on tree to-be-committed, and after commit.
-
-    # TODO: Strict commit that fails if there are deleted files.
-    #       (what does "deleted files" mean ??)
-
-    # TODO: Give better message for -s, --summary, used by tla people
-
-    # XXX: verbose currently does nothing
 
     _see_also = ['add', 'bugs', 'hooks', 'uncommit']
     takes_args = ['selected*']
@@ -3042,31 +3337,52 @@
              Option('show-diff', short_name='p',
                     help='When no message is supplied, show the diff along'
                     ' with the status summary in the message editor.'),
+             Option('lossy', 
+                    help='When committing to a foreign version control '
+                    'system do not push data that can not be natively '
+                    'represented.'),
              ]
     aliases = ['ci', 'checkin']
 
     def _iter_bug_fix_urls(self, fixes, branch):
+        default_bugtracker  = None
         # Configure the properties for bug fixing attributes.
         for fixed_bug in fixes:
             tokens = fixed_bug.split(':')
-            if len(tokens) != 2:
-                raise errors.BzrCommandError(
+            if len(tokens) == 1:
+                if default_bugtracker is None:
+                    branch_config = branch.get_config()
+                    default_bugtracker = branch_config.get_user_option(
+                        "bugtracker")
+                if default_bugtracker is None:
+                    raise errors.BzrCommandError(gettext(
+                        "No tracker specified for bug %s. Use the form "
+                        "'tracker:id' or specify a default bug tracker "
+                        "using the `bugtracker` option.\nSee "
+                        "\"bzr help bugs\" for more information on this "
+                        "feature. Commit refused.") % fixed_bug)
+                tag = default_bugtracker
+                bug_id = tokens[0]
+            elif len(tokens) != 2:
+                raise errors.BzrCommandError(gettext(
                     "Invalid bug %s. Must be in the form of 'tracker:id'. "
                     "See \"bzr help bugs\" for more information on this "
-                    "feature.\nCommit refused." % fixed_bug)
-            tag, bug_id = tokens
+                    "feature.\nCommit refused.") % fixed_bug)
+            else:
+                tag, bug_id = tokens
             try:
                 yield bugtracker.get_bug_url(tag, branch, bug_id)
             except errors.UnknownBugTrackerAbbreviation:
-                raise errors.BzrCommandError(
-                    'Unrecognized bug %s. Commit refused.' % fixed_bug)
+                raise errors.BzrCommandError(gettext(
+                    'Unrecognized bug %s. Commit refused.') % fixed_bug)
             except errors.MalformedBugIdentifier, e:
-                raise errors.BzrCommandError(
-                    "%s\nCommit refused." % (str(e),))
+                raise errors.BzrCommandError(gettext(
+                    "%s\nCommit refused.") % (str(e),))
 
     def run(self, message=None, file=None, verbose=False, selected_list=None,
             unchanged=False, strict=False, local=False, fixes=None,
-            author=None, show_diff=False, exclude=None, commit_time=None):
+            author=None, show_diff=False, exclude=None, commit_time=None,
+            lossy=False):
         from bzrlib.errors import (
             PointlessCommit,
             ConflictsInTree,
@@ -3075,7 +3391,8 @@
         from bzrlib.msgeditor import (
             edit_commit_message_encoded,
             generate_commit_message_template,
-            make_commit_message_template_encoded
+            make_commit_message_template_encoded,
+            set_commit_message,
         )
 
         commit_stamp = offset = None
@@ -3083,14 +3400,8 @@
             try:
                 commit_stamp, offset = timestamp.parse_patch_date(commit_time)
             except ValueError, e:
-                raise errors.BzrCommandError(
-                    "Could not parse --commit-time: " + str(e))
-
-        # TODO: Need a blackbox test for invoking the external editor; may be
-        # slightly problematic to run this cross-platform.
-
-        # TODO: do more checks that the commit will succeed before
-        # spending the user's valuable time typing a commit message.
+                raise errors.BzrCommandError(gettext(
+                    "Could not parse --commit-time: " + str(e)))
 
         properties = {}
 
@@ -3129,15 +3440,15 @@
                 message = message.replace('\r\n', '\n')
                 message = message.replace('\r', '\n')
             if file:
-                raise errors.BzrCommandError(
-                    "please specify either --message or --file")
+                raise errors.BzrCommandError(gettext(
+                    "please specify either --message or --file"))
 
         def get_message(commit_obj):
             """Callback to get commit message"""
             if file:
-                f = codecs.open(file, 'rt', osutils.get_user_encoding())
+                f = open(file)
                 try:
-                    my_message = f.read()
+                    my_message = f.read().decode(osutils.get_user_encoding())
                 finally:
                     f.close()
             elif message is not None:
@@ -3153,14 +3464,19 @@
                 # make_commit_message_template_encoded returns user encoding.
                 # We probably want to be using edit_commit_message instead to
                 # avoid this.
-                start_message = generate_commit_message_template(commit_obj)
-                my_message = edit_commit_message_encoded(text,
-                    start_message=start_message)
-                if my_message is None:
-                    raise errors.BzrCommandError("please specify a commit"
-                        " message with either --message or --file")
-            if my_message == "":
-                raise errors.BzrCommandError("empty commit message specified")
+                my_message = set_commit_message(commit_obj)
+                if my_message is None:
+                    start_message = generate_commit_message_template(commit_obj)
+                    my_message = edit_commit_message_encoded(text,
+                        start_message=start_message)
+                if my_message is None:
+                    raise errors.BzrCommandError(gettext("please specify a commit"
+                        " message with either --message or --file"))
+                if my_message == "":
+                    raise errors.BzrCommandError(gettext("Empty commit message specified."
+                            " Please specify a commit message with either"
+                            " --message or --file or leave a blank message"
+                            " with --message \"\"."))
             return my_message
 
         # The API permits a commit with a filter of [] to mean 'select nothing'
@@ -3174,22 +3490,24 @@
                         reporter=None, verbose=verbose, revprops=properties,
                         authors=author, timestamp=commit_stamp,
                         timezone=offset,
-                        exclude=tree.safe_relpath_files(exclude))
+                        exclude=tree.safe_relpath_files(exclude),
+                        lossy=lossy)
         except PointlessCommit:
-            raise errors.BzrCommandError("No changes to commit."
-                              " Use --unchanged to commit anyhow.")
+            raise errors.BzrCommandError(gettext("No changes to commit."
+                " Please 'bzr add' the files you want to commit, or use"
+                " --unchanged to force an empty commit."))
         except ConflictsInTree:
-            raise errors.BzrCommandError('Conflicts detected in working '
+            raise errors.BzrCommandError(gettext('Conflicts detected in working '
                 'tree.  Use "bzr conflicts" to list, "bzr resolve FILE" to'
-                ' resolve.')
+                ' resolve.'))
         except StrictCommitFailed:
-            raise errors.BzrCommandError("Commit refused because there are"
-                              " unknown files in the working tree.")
+            raise errors.BzrCommandError(gettext("Commit refused because there are"
+                              " unknown files in the working tree."))
         except errors.BoundBranchOutOfDate, e:
-            e.extra_help = ("\n"
+            e.extra_help = (gettext("\n"
                 'To commit to master branch, run update and then commit.\n'
                 'You can also pass --local to commit to continue working '
-                'disconnected.')
+                'disconnected.'))
             raise
 
 
@@ -3264,27 +3582,62 @@
 
 
 class cmd_upgrade(Command):
-    __doc__ = """Upgrade branch storage to current format.
-
-    The check command or bzr developers may sometimes advise you to run
-    this command. When the default format has changed you may also be warned
-    during other operations to upgrade.
+    __doc__ = """Upgrade a repository, branch or working tree to a newer format.
+
+    When the default format has changed after a major new release of
+    Bazaar, you may be informed during certain operations that you
+    should upgrade. Upgrading to a newer format may improve performance
+    or make new features available. It may however limit interoperability
+    with older repositories or with older versions of Bazaar.
+
+    If you wish to upgrade to a particular format rather than the
+    current default, that can be specified using the --format option.
+    As a consequence, you can use the upgrade command this way to
+    "downgrade" to an earlier format, though some conversions are
+    a one way process (e.g. changing from the 1.x default to the
+    2.x default) so downgrading is not always possible.
+
+    A backup.bzr.~#~ directory is created at the start of the conversion
+    process (where # is a number). By default, this is left there on
+    completion. If the conversion fails, delete the new .bzr directory
+    and rename this one back in its place. Use the --clean option to ask
+    for the backup.bzr directory to be removed on successful conversion.
+    Alternatively, you can delete it by hand if everything looks good
+    afterwards.
+
+    If the location given is a shared repository, dependent branches
+    are also converted provided the repository converts successfully.
+    If the conversion of a branch fails, remaining branches are still
+    tried.
+
+    For more information on upgrades, see the Bazaar Upgrade Guide,
+    http://doc.bazaar.canonical.com/latest/en/upgrade-guide/.
     """
 
-    _see_also = ['check']
+    _see_also = ['check', 'reconcile', 'formats']
     takes_args = ['url?']
     takes_options = [
-                    RegistryOption('format',
-                        help='Upgrade to a specific format.  See "bzr help'
-                             ' formats" for details.',
-                        lazy_registry=('bzrlib.bzrdir', 'format_registry'),
-                        converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
-                        value_switches=True, title='Branch format'),
-                    ]
+        RegistryOption('format',
+            help='Upgrade to a specific format.  See "bzr help'
+                 ' formats" for details.',
+            lazy_registry=('bzrlib.controldir', 'format_registry'),
+            converter=lambda name: controldir.format_registry.make_bzrdir(name),
+            value_switches=True, title='Branch format'),
+        Option('clean',
+            help='Remove the backup.bzr directory if successful.'),
+        Option('dry-run',
+            help="Show what would be done, but don't actually do anything."),
+    ]
 
-    def run(self, url='.', format=None):
+    def run(self, url='.', format=None, clean=False, dry_run=False):
         from bzrlib.upgrade import upgrade
-        upgrade(url, format)
+        exceptions = upgrade(url, format, clean_up=clean, dry_run=dry_run)
+        if exceptions:
+            if len(exceptions) == 1:
+                # Compatibility with historical behavior
+                raise exceptions[0]
+            else:
+                return 3
 
 
 class cmd_whoami(Command):
@@ -3317,7 +3670,7 @@
                 try:
                     c = Branch.open_containing(u'.')[0].get_config()
                 except errors.NotBranchError:
-                    c = config.GlobalConfig()
+                    c = _mod_config.GlobalConfig()
             else:
                 c = Branch.open(directory).get_config()
             if email:
@@ -3326,9 +3679,13 @@
                 self.outf.write(c.username() + '\n')
             return
 
+        if email:
+            raise errors.BzrCommandError(gettext("--email can only be used to display existing "
+                                         "identity"))
+
         # display a warning if an email address isn't included in the given name.
         try:
-            config.extract_email_address(name)
+            _mod_config.extract_email_address(name)
         except errors.NoEmailInUsername, e:
             warning('"%s" does not seem to contain an email address.  '
                     'This is allowed, but not recommended.', name)
@@ -3340,7 +3697,7 @@
             else:
                 c = Branch.open(directory).get_config()
         else:
-            c = config.GlobalConfig()
+            c = _mod_config.GlobalConfig()
         c.set_user_option('email', name)
 
 
@@ -3409,17 +3766,17 @@
 
     def remove_alias(self, alias_name):
         if alias_name is None:
-            raise errors.BzrCommandError(
-                'bzr alias --remove expects an alias to remove.')
+            raise errors.BzrCommandError(gettext(
+                'bzr alias --remove expects an alias to remove.'))
         # If alias is not found, print something like:
         # unalias: foo: not found
-        c = config.GlobalConfig()
+        c = _mod_config.GlobalConfig()
         c.unset_alias(alias_name)
 
     @display_command
     def print_aliases(self):
         """Print out the defined aliases in a similar format to bash."""
-        aliases = config.GlobalConfig().get_aliases()
+        aliases = _mod_config.GlobalConfig().get_aliases()
         for key, value in sorted(aliases.iteritems()):
             self.outf.write('bzr alias %s="%s"\n' % (key, value))
 
@@ -3435,7 +3792,7 @@
 
     def set_alias(self, alias_name, alias_command):
         """Save the alias in the global config."""
-        c = config.GlobalConfig()
+        c = _mod_config.GlobalConfig()
         c.set_alias(alias_name, alias_command)
 
 
@@ -3476,6 +3833,9 @@
     If you set BZR_TEST_PDB=1 when running selftest, failing tests will drop
     into a pdb postmortem session.
 
+    The --coverage=DIRNAME global option produces a report with covered code
+    indicated.
+
     :Examples:
         Run only tests relating to 'ignore'::
 
@@ -3492,10 +3852,10 @@
         if typestring == "sftp":
             from bzrlib.tests import stub_sftp
             return stub_sftp.SFTPAbsoluteServer
-        if typestring == "memory":
+        elif typestring == "memory":
             from bzrlib.tests import test_server
             return memory.MemoryServer
-        if typestring == "fakenfs":
+        elif typestring == "fakenfs":
             from bzrlib.tests import test_server
             return test_server.FakeNFSServer
         msg = "No known transport type %s. Supported types are: sftp\n" %\
@@ -3535,10 +3895,10 @@
                      Option('randomize', type=str, argname="SEED",
                             help='Randomize the order of tests using the given'
                                  ' seed or "now" for the current time.'),
-                     Option('exclude', type=str, argname="PATTERN",
-                            short_name='x',
-                            help='Exclude tests that match this regular'
-                                 ' expression.'),
+                     ListOption('exclude', type=str, argname="PATTERN",
+                                short_name='x',
+                                help='Exclude tests that match this regular'
+                                ' expression.'),
                      Option('subunit',
                         help='Output test progress via subunit.'),
                      Option('strict', help='Fail on missing dependencies or '
@@ -3551,6 +3911,9 @@
                                 param_name='starting_with', short_name='s',
                                 help=
                                 'Load only the tests starting with TESTID.'),
+                     Option('sync',
+                            help="By default we disable fsync and fdatasync"
+                                 " while running the test suite.")
                      ]
     encoding_type = 'replace'
 
@@ -3564,11 +3927,9 @@
             first=False, list_only=False,
             randomize=None, exclude=None, strict=False,
             load_list=None, debugflag=None, starting_with=None, subunit=False,
-            parallel=None, lsprof_tests=False):
-        from bzrlib.tests import selftest
-
-        # Make deprecation warnings visible, unless -Werror is set
-        symbol_versioning.activate_deprecation_warnings(override=False)
+            parallel=None, lsprof_tests=False,
+            sync=False):
+        from bzrlib import tests
 
         if testspecs_list is not None:
             pattern = '|'.join(testspecs_list)
@@ -3578,8 +3939,8 @@
             try:
                 from bzrlib.tests import SubUnitBzrRunner
             except ImportError:
-                raise errors.BzrCommandError("subunit not available. subunit "
-                    "needs to be installed to use --subunit.")
+                raise errors.BzrCommandError(gettext("subunit not available. subunit "
+                    "needs to be installed to use --subunit."))
             self.additional_selftest_args['runner_class'] = SubUnitBzrRunner
             # On Windows, disable automatic conversion of '\n' to '\r\n' in
             # stdout, which would corrupt the subunit stream. 
@@ -3594,10 +3955,16 @@
             self.additional_selftest_args.setdefault(
                 'suite_decorators', []).append(parallel)
         if benchmark:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "--benchmark is no longer supported from bzr 2.2; "
-                "use bzr-usertest instead")
+                "use bzr-usertest instead"))
         test_suite_factory = None
+        if not exclude:
+            exclude_pattern = None
+        else:
+            exclude_pattern = '(' + '|'.join(exclude) + ')'
+        if not sync:
+            self._disable_fsync()
         selftest_kwargs = {"verbose": verbose,
                           "pattern": pattern,
                           "stop_on_failure": one,
@@ -3608,16 +3975,32 @@
                           "matching_tests_first": first,
                           "list_only": list_only,
                           "random_seed": randomize,
-                          "exclude_pattern": exclude,
+                          "exclude_pattern": exclude_pattern,
                           "strict": strict,
                           "load_list": load_list,
                           "debug_flags": debugflag,
                           "starting_with": starting_with
                           }
         selftest_kwargs.update(self.additional_selftest_args)
-        result = selftest(**selftest_kwargs)
+
+        # Make deprecation warnings visible, unless -Werror is set
+        cleanup = symbol_versioning.activate_deprecation_warnings(
+            override=False)
+        try:
+            result = tests.selftest(**selftest_kwargs)
+        finally:
+            cleanup()
         return int(not result)
 
+    def _disable_fsync(self):
+        """Change the 'os' functionality to not synchronize."""
+        self._orig_fsync = getattr(os, 'fsync', None)
+        if self._orig_fsync is not None:
+            os.fsync = lambda filedes: None
+        self._orig_fdatasync = getattr(os, 'fdatasync', None)
+        if self._orig_fdatasync is not None:
+            os.fdatasync = lambda filedes: None
+
 
 class cmd_version(Command):
     __doc__ = """Show version of bzr."""
@@ -3643,7 +4026,7 @@
 
     @display_command
     def run(self):
-        self.outf.write("It sure does!\n")
+        self.outf.write(gettext("It sure does!\n"))
 
 
 class cmd_find_merge_base(Command):
@@ -3667,7 +4050,7 @@
         graph = branch1.repository.get_graph(branch2.repository)
         base_rev_id = graph.find_unique_lca(last1, last2)
 
-        self.outf.write('merge base is revision %s\n' % base_rev_id)
+        self.outf.write(gettext('merge base is revision %s\n') % base_rev_id)
 
 
 class cmd_merge(Command):
@@ -3676,18 +4059,27 @@
     The source of the merge can be specified either in the form of a branch,
     or in the form of a path to a file containing a merge directive generated
     with bzr send. If neither is specified, the default is the upstream branch
-    or the branch most recently merged using --remember.
-
-    When merging a branch, by default the tip will be merged. To pick a different
-    revision, pass --revision. If you specify two values, the first will be used as
-    BASE and the second one as OTHER. Merging individual revisions, or a subset of
-    available revisions, like this is commonly referred to as "cherrypicking".
-
-    Revision numbers are always relative to the branch being merged.
-
-    By default, bzr will try to merge in all new work from the other
-    branch, automatically determining an appropriate base.  If this
-    fails, you may need to give an explicit base.
+    or the branch most recently merged using --remember.  The source of the
+    merge may also be specified in the form of a path to a file in another
+    branch:  in this case, only the modifications to that file are merged into
+    the current working tree.
+
+    When merging from a branch, by default bzr will try to merge in all new
+    work from the other branch, automatically determining an appropriate base
+    revision.  If this fails, you may need to give an explicit base.
+
+    To pick a different ending revision, pass "--revision OTHER".  bzr will
+    try to merge in all new work up to and including revision OTHER.
+
+    If you specify two values, "--revision BASE..OTHER", only revisions BASE
+    through OTHER, excluding BASE but including OTHER, will be merged.  If this
+    causes some revisions to be skipped, i.e. if the destination branch does
+    not already contain revision BASE, such a merge is commonly referred to as
+    a "cherrypick". Unlike a normal merge, Bazaar does not currently track
+    cherrypicks. The changes look like a normal commit, and the history of the
+    changes from the other branch is not stored in the commit.
+
+    Revision numbers are always relative to the source branch.
 
     Merge will do its best to combine the changes in two branches, but there
     are some kinds of problems only a human can fix.  When it encounters those,
@@ -3696,17 +4088,19 @@
 
     Use bzr resolve when you have fixed a problem.  See also bzr conflicts.
 
-    If there is no default branch set, the first merge will set it. After
-    that, you can omit the branch to use the default.  To change the
-    default, use --remember. The value will only be saved if the remote
-    location can be accessed.
+    If there is no default branch set, the first merge will set it (use
+    --no-remember to avoid setting it). After that, you can omit the branch
+    to use the default.  To change the default, use --remember. The value will
+    only be saved if the remote location can be accessed.
 
     The results of the merge are placed into the destination working
     directory, where they can be reviewed (with bzr diff), tested, and then
     committed to record the result of the merge.
 
     merge refuses to run if there are any uncommitted changes, unless
-    --force is given. The --force option can also be used to create a
+    --force is given.  If --force is given, then the changes from the source 
+    will be merged with the current working tree, including any uncommitted
+    changes in the tree.  The --force option can also be used to create a
     merge revision which has more than two parents.
 
     If one would like to merge changes from the working tree of the other
@@ -3717,7 +4111,7 @@
     you to apply each diff hunk and file change, similar to "shelve".
 
     :Examples:
-        To merge the latest revision from bzr.dev::
+        To merge all new revisions from bzr.dev::
 
             bzr merge ../bzr.dev
 
@@ -3770,7 +4164,7 @@
     ]
 
     def run(self, location=None, revision=None, force=False,
-            merge_type=None, show_base=False, reprocess=None, remember=False,
+            merge_type=None, show_base=False, reprocess=None, remember=None,
             uncommitted=False, pull=False,
             directory=None,
             preview=False,
@@ -3784,7 +4178,11 @@
         merger = None
         allow_pending = True
         verified = 'inapplicable'
+
         tree = WorkingTree.open_containing(directory)[0]
+        if tree.branch.revno() == 0:
+            raise errors.BzrCommandError(gettext('Merging into empty branches not currently supported, '
+                                         'https://bugs.launchpad.net/bzr/+bug/308562'))
 
         try:
             basis_tree = tree.revision_tree(tree.last_revision())
@@ -3810,19 +4208,19 @@
                 mergeable = None
             else:
                 if uncommitted:
-                    raise errors.BzrCommandError('Cannot use --uncommitted'
-                        ' with bundles or merge directives.')
+                    raise errors.BzrCommandError(gettext('Cannot use --uncommitted'
+                        ' with bundles or merge directives.'))
 
                 if revision is not None:
-                    raise errors.BzrCommandError(
-                        'Cannot use -r with merge directives or bundles')
+                    raise errors.BzrCommandError(gettext(
+                        'Cannot use -r with merge directives or bundles'))
                 merger, verified = _mod_merge.Merger.from_mergeable(tree,
                    mergeable, None)
 
         if merger is None and uncommitted:
             if revision is not None and len(revision) > 0:
-                raise errors.BzrCommandError('Cannot use --uncommitted and'
-                    ' --revision at the same time.')
+                raise errors.BzrCommandError(gettext('Cannot use --uncommitted and'
+                    ' --revision at the same time.'))
             merger = self.get_merger_from_uncommitted(tree, location, None)
             allow_pending = False
 
@@ -3836,20 +4234,27 @@
         self.sanity_check_merger(merger)
         if (merger.base_rev_id == merger.other_rev_id and
             merger.other_rev_id is not None):
-            note('Nothing to do.')
+            # check if location is a nonexistent file (and not a branch) to
+            # disambiguate the 'Nothing to do'
+            if merger.interesting_files:
+                if not merger.other_tree.has_filename(
+                    merger.interesting_files[0]):
+                    note(gettext("merger: ") + str(merger))
+                    raise errors.PathsDoNotExist([location])
+            note(gettext('Nothing to do.'))
             return 0
-        if pull:
+        if pull and not preview:
             if merger.interesting_files is not None:
-                raise errors.BzrCommandError('Cannot pull individual files')
+                raise errors.BzrCommandError(gettext('Cannot pull individual files'))
             if (merger.base_rev_id == tree.last_revision()):
                 result = tree.pull(merger.other_branch, False,
                                    merger.other_rev_id)
                 result.report(self.outf)
                 return 0
         if merger.this_basis is None:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "This branch has no commits."
-                " (perhaps you would prefer 'bzr pull')")
+                " (perhaps you would prefer 'bzr pull')"))
         if preview:
             return self._do_preview(merger)
         elif interactive:
@@ -3906,8 +4311,8 @@
     def sanity_check_merger(self, merger):
         if (merger.show_base and
             not merger.merge_type is _mod_merge.Merge3Merger):
-            raise errors.BzrCommandError("Show-base is not supported for this"
-                                         " merge type. %s" % merger.merge_type)
+            raise errors.BzrCommandError(gettext("Show-base is not supported for this"
+                                         " merge type. %s") % merger.merge_type)
         if merger.reprocess is None:
             if merger.show_base:
                 merger.reprocess = False
@@ -3915,12 +4320,12 @@
                 # Use reprocess if the merger supports it
                 merger.reprocess = merger.merge_type.supports_reprocess
         if merger.reprocess and not merger.merge_type.supports_reprocess:
-            raise errors.BzrCommandError("Conflict reduction is not supported"
-                                         " for merge type %s." %
+            raise errors.BzrCommandError(gettext("Conflict reduction is not supported"
+                                         " for merge type %s.") %
                                          merger.merge_type)
         if merger.reprocess and merger.show_base:
-            raise errors.BzrCommandError("Cannot do conflict reduction and"
-                                         " show base.")
+            raise errors.BzrCommandError(gettext("Cannot do conflict reduction and"
+                                         " show base."))
 
     def _get_merger_from_branch(self, tree, location, revision, remember,
                                 possible_transports, pb):
@@ -3953,11 +4358,19 @@
         if other_revision_id is None:
             other_revision_id = _mod_revision.ensure_null(
                 other_branch.last_revision())
-        # Remember where we merge from
-        if ((remember or tree.branch.get_submit_branch() is None) and
-             user_location is not None):
+        # Remember where we merge from. We need to remember if:
+        # - user specify a location (and we don't merge from the parent
+        #   branch)
+        # - user ask to remember or there is no previous location set to merge
+        #   from and user didn't ask to *not* remember
+        if (user_location is not None
+            and ((remember
+                  or (remember is None
+                      and tree.branch.get_submit_branch() is None)))):
             tree.branch.set_submit_branch(other_branch.base)
-        _merge_tags_if_possible(other_branch, tree.branch)
+        # Merge tags (but don't set them in the master branch yet, the user
+        # might revert this merge).  Commit will propagate them.
+        _merge_tags_if_possible(other_branch, tree.branch, ignore_master=True)
         merger = _mod_merge.Merger.from_revision_ids(pb, tree,
             other_revision_id, base_revision_id, other_branch, base_branch)
         if other_path != '':
@@ -4022,10 +4435,10 @@
             stored_location_type = "parent"
         mutter("%s", stored_location)
         if stored_location is None:
-            raise errors.BzrCommandError("No location specified or remembered")
+            raise errors.BzrCommandError(gettext("No location specified or remembered"))
         display_url = urlutils.unescape_for_display(stored_location, 'utf-8')
-        note(u"%s remembered %s location %s", verb_string,
-                stored_location_type, display_url)
+        note(gettext("{0} remembered {1} location {2}").format(verb_string,
+                stored_location_type, display_url))
         return stored_location
 
 
@@ -4068,9 +4481,9 @@
         self.add_cleanup(tree.lock_write().unlock)
         parents = tree.get_parent_ids()
         if len(parents) != 2:
-            raise errors.BzrCommandError("Sorry, remerge only works after normal"
+            raise errors.BzrCommandError(gettext("Sorry, remerge only works after normal"
                                          " merges.  Not cherrypicking or"
-                                         " multi-merges.")
+                                         " multi-merges."))
         repository = tree.branch.repository
         interesting_ids = None
         new_conflicts = []
@@ -4131,9 +4544,10 @@
     last committed revision is used.
 
     To remove only some changes, without reverting to a prior version, use
-    merge instead.  For example, "merge . --revision -2..-3" will remove the
-    changes introduced by -2, without affecting the changes introduced by -1.
-    Or to remove certain changes on a hunk-by-hunk basis, see the Shelf plugin.
+    merge instead.  For example, "merge . -r -2..-3" (don't forget the ".")
+    will remove the changes introduced by the second last commit (-2), without
+    affecting the changes introduced by the last commit (-1).  To remove
+    certain changes on a hunk-by-hunk basis, see the shelve command.
 
     By default, any files that have been manually changed will be backed up
     first.  (Files changed only by merge are not backed up.)  Backup files have
@@ -4169,7 +4583,7 @@
     target branches.
     """
 
-    _see_also = ['cat', 'export']
+    _see_also = ['cat', 'export', 'merge', 'shelve']
     takes_options = [
         'revision',
         Option('no-backup', "Do not save backups of reverted files."),
@@ -4294,8 +4708,10 @@
             type=_parse_revision_str,
             help='Filter on local branch revisions (inclusive). '
                 'See "help revisionspec" for details.'),
-        Option('include-merges',
+        Option('include-merged',
                'Show all revisions in addition to the mainline ones.'),
+        Option('include-merges', hidden=True,
+               help='Historical alias for --include-merged.'),
         ]
     encoding_type = 'replace'
 
@@ -4304,13 +4720,29 @@
             theirs_only=False,
             log_format=None, long=False, short=False, line=False,
             show_ids=False, verbose=False, this=False, other=False,
-            include_merges=False, revision=None, my_revision=None,
-            directory=u'.'):
+            include_merged=None, revision=None, my_revision=None,
+            directory=u'.',
+            include_merges=symbol_versioning.DEPRECATED_PARAMETER):
         from bzrlib.missing import find_unmerged, iter_log_revisions
         def message(s):
             if not is_quiet():
                 self.outf.write(s)
 
+        if symbol_versioning.deprecated_passed(include_merges):
+            ui.ui_factory.show_user_warning(
+                'deprecated_command_option',
+                deprecated_name='--include-merges',
+                recommended_name='--include-merged',
+                deprecated_in_version='2.5',
+                command=self.invoked_as)
+            if include_merged is None:
+                include_merged = include_merges
+            else:
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--include-merges', '--include-merged'))
+        if include_merged is None:
+            include_merged = False
         if this:
             mine_only = this
         if other:
@@ -4331,12 +4763,12 @@
         if other_branch is None:
             other_branch = parent
             if other_branch is None:
-                raise errors.BzrCommandError("No peer location known"
-                                             " or specified.")
+                raise errors.BzrCommandError(gettext("No peer location known"
+                                             " or specified."))
             display_url = urlutils.unescape_for_display(parent,
                                                         self.outf.encoding)
-            message("Using saved parent location: "
-                    + display_url + "\n")
+            message(gettext("Using saved parent location: {0}\n").format(
+                    display_url))
 
         remote_branch = Branch.open(other_branch)
         if remote_branch.base == local_branch.base:
@@ -4355,7 +4787,7 @@
         local_extra, remote_extra = find_unmerged(
             local_branch, remote_branch, restrict,
             backward=not reverse,
-            include_merges=include_merges,
+            include_merged=include_merged,
             local_revid_range=local_revid_range,
             remote_revid_range=remote_revid_range)
 
@@ -4368,7 +4800,9 @@
 
         status_code = 0
         if local_extra and not theirs_only:
-            message("You have %d extra revision(s):\n" %
+            message(ngettext("You have %d extra revision:\n",
+                             "You have %d extra revisions:\n", 
+                             len(local_extra)) %
                 len(local_extra))
             for revision in iter_log_revisions(local_extra,
                                 local_branch.repository,
@@ -4382,7 +4816,9 @@
         if remote_extra and not mine_only:
             if printed_local is True:
                 message("\n\n\n")
-            message("You are missing %d revision(s):\n" %
+            message(ngettext("You are missing %d revision:\n",
+                             "You are missing %d revisions:\n",
+                             len(remote_extra)) %
                 len(remote_extra))
             for revision in iter_log_revisions(remote_extra,
                                 remote_branch.repository,
@@ -4392,15 +4828,15 @@
 
         if mine_only and not local_extra:
             # We checked local, and found nothing extra
-            message('This branch is up to date.\n')
+            message(gettext('This branch has no new revisions.\n'))
         elif theirs_only and not remote_extra:
             # We checked remote, and found nothing extra
-            message('Other branch is up to date.\n')
+            message(gettext('Other branch has no new revisions.\n'))
         elif not (mine_only or theirs_only or local_extra or
                   remote_extra):
             # We checked both branches, and neither one had extra
             # revisions
-            message("Branches are up to date.\n")
+            message(gettext("Branches are up to date.\n"))
         self.cleanup_now()
         if not status_code and parent is None and other_branch is not None:
             self.add_cleanup(local_branch.lock_write().unlock)
@@ -4436,7 +4872,7 @@
         ]
 
     def run(self, branch_or_repo='.', clean_obsolete_packs=False):
-        dir = bzrdir.BzrDir.open_containing(branch_or_repo)[0]
+        dir = controldir.ControlDir.open_containing(branch_or_repo)[0]
         try:
             branch = dir.open_branch()
             repository = branch.repository
@@ -4468,26 +4904,10 @@
 
     @display_command
     def run(self, verbose=False):
-        import bzrlib.plugin
-        from inspect import getdoc
-        result = []
-        for name, plugin in bzrlib.plugin.plugins().items():
-            version = plugin.__version__
-            if version == 'unknown':
-                version = ''
-            name_ver = '%s %s' % (name, version)
-            d = getdoc(plugin.module)
-            if d:
-                doc = d.split('\n')[0]
-            else:
-                doc = '(no description)'
-            result.append((name_ver, doc, plugin.path()))
-        for name_ver, doc, path in sorted(result):
-            self.outf.write("%s\n" % name_ver)
-            self.outf.write("   %s\n" % doc)
-            if verbose:
-                self.outf.write("   %s\n" % path)
-            self.outf.write("\n")
+        from bzrlib import plugin
+        # Don't give writelines a generator as some codecs don't like that
+        self.outf.writelines(
+            list(plugin.describe_plugins(show_paths=verbose)))
 
 
 class cmd_testament(Command):
@@ -4546,7 +4966,9 @@
     @display_command
     def run(self, filename, all=False, long=False, revision=None,
             show_ids=False, directory=None):
-        from bzrlib.annotate import annotate_file, annotate_file_tree
+        from bzrlib.annotate import (
+            annotate_file_tree,
+            )
         wt, branch, relpath = \
             _open_directory_or_containing_tree_or_branch(filename, directory)
         if wt is not None:
@@ -4555,21 +4977,20 @@
             self.add_cleanup(branch.lock_read().unlock)
         tree = _get_one_revision_tree('annotate', revision, branch=branch)
         self.add_cleanup(tree.lock_read().unlock)
-        if wt is not None:
+        if wt is not None and revision is None:
             file_id = wt.path2id(relpath)
         else:
             file_id = tree.path2id(relpath)
         if file_id is None:
             raise errors.NotVersionedError(filename)
-        file_version = tree.inventory[file_id].revision
         if wt is not None and revision is None:
             # If there is a tree and we're not annotating historical
             # versions, annotate the working tree's content.
             annotate_file_tree(wt, file_id, self.outf, long, all,
                 show_ids=show_ids)
         else:
-            annotate_file(branch, file_version, file_id, long, all, self.outf,
-                          show_ids=show_ids)
+            annotate_file_tree(tree, file_id, self.outf, long, all,
+                show_ids=show_ids, branch=branch)
 
 
 class cmd_re_sign(Command):
@@ -4582,9 +5003,9 @@
 
     def run(self, revision_id_list=None, revision=None, directory=u'.'):
         if revision_id_list is not None and revision is not None:
-            raise errors.BzrCommandError('You can only supply one of revision_id or --revision')
+            raise errors.BzrCommandError(gettext('You can only supply one of revision_id or --revision'))
         if revision_id_list is None and revision is None:
-            raise errors.BzrCommandError('You must supply either --revision or a revision_id')
+            raise errors.BzrCommandError(gettext('You must supply either --revision or a revision_id'))
         b = WorkingTree.open_containing(directory)[0].branch
         self.add_cleanup(b.lock_write().unlock)
         return self._run(b, revision_id_list, revision)
@@ -4622,7 +5043,7 @@
                 if to_revid is None:
                     to_revno = b.revno()
                 if from_revno is None or to_revno is None:
-                    raise errors.BzrCommandError('Cannot sign a range of non-revision-history revisions')
+                    raise errors.BzrCommandError(gettext('Cannot sign a range of non-revision-history revisions'))
                 b.repository.start_write_group()
                 try:
                     for revno in range(from_revno, to_revno + 1):
@@ -4634,7 +5055,7 @@
                 else:
                     b.repository.commit_write_group()
             else:
-                raise errors.BzrCommandError('Please supply either one revision, or a range.')
+                raise errors.BzrCommandError(gettext('Please supply either one revision, or a range.'))
 
 
 class cmd_bind(Command):
@@ -4659,21 +5080,21 @@
             try:
                 location = b.get_old_bound_location()
             except errors.UpgradeRequired:
-                raise errors.BzrCommandError('No location supplied.  '
-                    'This format does not remember old locations.')
+                raise errors.BzrCommandError(gettext('No location supplied.  '
+                    'This format does not remember old locations.'))
             else:
                 if location is None:
                     if b.get_bound_location() is not None:
-                        raise errors.BzrCommandError('Branch is already bound')
+                        raise errors.BzrCommandError(gettext('Branch is already bound'))
                     else:
-                        raise errors.BzrCommandError('No location supplied '
-                            'and no previous location known')
+                        raise errors.BzrCommandError(gettext('No location supplied '
+                            'and no previous location known'))
         b_other = Branch.open(location)
         try:
             b.bind(b_other)
         except errors.DivergedBranches:
-            raise errors.BzrCommandError('These branches have diverged.'
-                                         ' Try merging, and then bind again.')
+            raise errors.BzrCommandError(gettext('These branches have diverged.'
+                                         ' Try merging, and then bind again.'))
         if b.get_config().has_explicit_nickname():
             b.nick = b_other.nick
 
@@ -4692,7 +5113,7 @@
     def run(self, directory=u'.'):
         b, relpath = Branch.open_containing(directory)
         if not b.unbind():
-            raise errors.BzrCommandError('Local branch is not bound')
+            raise errors.BzrCommandError(gettext('Local branch is not bound'))
 
 
 class cmd_uncommit(Command):
@@ -4719,6 +5140,8 @@
     takes_options = ['verbose', 'revision',
                     Option('dry-run', help='Don\'t actually make changes.'),
                     Option('force', help='Say yes to all questions.'),
+                    Option('keep-tags',
+                           help='Keep tags that point to removed revisions.'),
                     Option('local',
                            help="Only remove the commits from the local branch"
                                 " when in a checkout."
@@ -4728,12 +5151,11 @@
     aliases = []
     encoding_type = 'replace'
 
-    def run(self, location=None,
-            dry_run=False, verbose=False,
-            revision=None, force=False, local=False):
+    def run(self, location=None, dry_run=False, verbose=False,
+            revision=None, force=False, local=False, keep_tags=False):
         if location is None:
             location = u'.'
-        control, relpath = bzrdir.BzrDir.open_containing(location)
+        control, relpath = controldir.ControlDir.open_containing(location)
         try:
             tree = control.open_workingtree()
             b = tree.branch
@@ -4745,9 +5167,11 @@
             self.add_cleanup(tree.lock_write().unlock)
         else:
             self.add_cleanup(b.lock_write().unlock)
-        return self._run(b, tree, dry_run, verbose, revision, force, local=local)
+        return self._run(b, tree, dry_run, verbose, revision, force,
+                         local, keep_tags)
 
-    def _run(self, b, tree, dry_run, verbose, revision, force, local=False):
+    def _run(self, b, tree, dry_run, verbose, revision, force, local,
+             keep_tags):
         from bzrlib.log import log_formatter, show_log
         from bzrlib.uncommit import uncommit
 
@@ -4768,7 +5192,7 @@
                 rev_id = b.get_rev_id(revno)
 
         if rev_id is None or _mod_revision.is_null(rev_id):
-            self.outf.write('No revisions to uncommit.\n')
+            self.outf.write(gettext('No revisions to uncommit.\n'))
             return 1
 
         lf = log_formatter('short',
@@ -4783,26 +5207,32 @@
                  end_revision=last_revno)
 
         if dry_run:
-            self.outf.write('Dry-run, pretending to remove'
-                            ' the above revisions.\n')
+            self.outf.write(gettext('Dry-run, pretending to remove'
+                            ' the above revisions.\n'))
         else:
-            self.outf.write('The above revision(s) will be removed.\n')
+            self.outf.write(gettext('The above revision(s) will be removed.\n'))
 
         if not force:
-            if not ui.ui_factory.get_boolean('Are you sure'):
-                self.outf.write('Canceled')
+            if not ui.ui_factory.confirm_action(
+                    gettext(u'Uncommit these revisions'),
+                    'bzrlib.builtins.uncommit',
+                    {}):
+                self.outf.write(gettext('Canceled\n'))
                 return 0
 
         mutter('Uncommitting from {%s} to {%s}',
                last_rev_id, rev_id)
         uncommit(b, tree=tree, dry_run=dry_run, verbose=verbose,
-                 revno=revno, local=local)
-        self.outf.write('You can restore the old tip by running:\n'
-             '  bzr pull . -r revid:%s\n' % last_rev_id)
+                 revno=revno, local=local, keep_tags=keep_tags)
+        self.outf.write(gettext('You can restore the old tip by running:\n'
+             '  bzr pull . -r revid:%s\n') % last_rev_id)
 
 
 class cmd_break_lock(Command):
-    __doc__ = """Break a dead lock on a repository, branch or working directory.
+    __doc__ = """Break a dead lock.
+
+    This command breaks a lock on a repository, branch, working directory or
+    config file.
 
     CAUTION: Locks should only be broken when you are sure that the process
     holding the lock has been stopped.
@@ -4813,17 +5243,33 @@
     :Examples:
         bzr break-lock
         bzr break-lock bzr+ssh://example.com/bzr/foo
+        bzr break-lock --conf ~/.bazaar
     """
+
     takes_args = ['location?']
+    takes_options = [
+        Option('config',
+               help='LOCATION is the directory where the config lock is.'),
+        Option('force',
+            help='Do not ask for confirmation before breaking the lock.'),
+        ]
 
-    def run(self, location=None, show=False):
+    def run(self, location=None, config=False, force=False):
         if location is None:
             location = u'.'
-        control, relpath = bzrdir.BzrDir.open_containing(location)
-        try:
-            control.break_lock()
-        except NotImplementedError:
-            pass
+        if force:
+            ui.ui_factory = ui.ConfirmationUserInterfacePolicy(ui.ui_factory,
+                None,
+                {'bzrlib.lockdir.break': True})
+        if config:
+            conf = _mod_config.LockableConfig(file_name=location)
+            conf.break_lock()
+        else:
+            control, relpath = controldir.ControlDir.open_containing(location)
+            try:
+                control.break_lock()
+            except NotImplementedError:
+                pass
 
 
 class cmd_wait_until_signalled(Command):
@@ -4869,6 +5315,8 @@
                     'option leads to global uncontrolled write access to your '
                     'file system.'
                 ),
+        Option('client-timeout', type=float,
+               help='Override the default idle client timeout (5min).'),
         ]
 
     def get_host_and_port(self, port):
@@ -4891,7 +5339,7 @@
         return host, port
 
     def run(self, port=None, inet=False, directory=None, allow_writes=False,
-            protocol=None):
+            protocol=None, client_timeout=None):
         from bzrlib import transport
         if directory is None:
             directory = os.getcwd()
@@ -4902,7 +5350,19 @@
         if not allow_writes:
             url = 'readonly+' + url
         t = transport.get_transport(url)
-        protocol(t, host, port, inet)
+        try:
+            protocol(t, host, port, inet, client_timeout)
+        except TypeError, e:
+            # We use symbol_versioning.deprecated_in just so that people
+            # grepping can find it here.
+            # symbol_versioning.deprecated_in((2, 5, 0))
+            symbol_versioning.warn(
+                'Got TypeError(%s)\ntrying to call protocol: %s.%s\n'
+                'Most likely it needs to be updated to support a'
+                ' "timeout" parameter (added in bzr 2.5.0)'
+                % (e, protocol.__module__, protocol),
+                DeprecationWarning)
+            protocol(t, host, port, inet)
 
 
 class cmd_join(Command):
@@ -4914,7 +5374,7 @@
     not part of it.  (Such trees can be produced by "bzr split", but also by
     running "bzr branch" with the target inside a tree.)
 
-    The result is a combined tree, with the subtree no longer an independant
+    The result is a combined tree, with the subtree no longer an independent
     part.  This is marked as a merge of the subtree into the containing tree,
     and all history is preserved.
     """
@@ -4931,9 +5391,9 @@
         containing_tree = WorkingTree.open_containing(parent_dir)[0]
         repo = containing_tree.branch.repository
         if not repo.supports_rich_root():
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "Can't join trees because %s doesn't support rich root data.\n"
-                "You can use bzr upgrade on the repository."
+                "You can use bzr upgrade on the repository.")
                 % (repo,))
         if reference:
             try:
@@ -4941,14 +5401,14 @@
             except errors.BadReferenceTarget, e:
                 # XXX: Would be better to just raise a nicely printable
                 # exception from the real origin.  Also below.  mbp 20070306
-                raise errors.BzrCommandError("Cannot join %s.  %s" %
-                                             (tree, e.reason))
+                raise errors.BzrCommandError(
+                       gettext("Cannot join {0}.  {1}").format(tree, e.reason))
         else:
             try:
                 containing_tree.subsume(sub_tree)
             except errors.BadSubsumeSource, e:
-                raise errors.BzrCommandError("Cannot join %s.  %s" %
-                                             (tree, e.reason))
+                raise errors.BzrCommandError(
+                       gettext("Cannot join {0}.  {1}").format(tree, e.reason))
 
 
 class cmd_split(Command):
@@ -5038,7 +5498,7 @@
         if submit_branch is None:
             submit_branch = branch.get_parent()
         if submit_branch is None:
-            raise errors.BzrCommandError('No submit branch specified or known')
+            raise errors.BzrCommandError(gettext('No submit branch specified or known'))
 
         stored_public_branch = branch.get_public_branch()
         if public_branch is None:
@@ -5046,13 +5506,13 @@
         elif stored_public_branch is None:
             branch.set_public_branch(public_branch)
         if not include_bundle and public_branch is None:
-            raise errors.BzrCommandError('No public branch specified or'
-                                         ' known')
+            raise errors.BzrCommandError(gettext('No public branch specified or'
+                                         ' known'))
         base_revision_id = None
         if revision is not None:
             if len(revision) > 2:
-                raise errors.BzrCommandError('bzr merge-directive takes '
-                    'at most two one revision identifiers')
+                raise errors.BzrCommandError(gettext('bzr merge-directive takes '
+                    'at most two one revision identifiers'))
             revision_id = revision[-1].as_revision_id(branch)
             if len(revision) == 2:
                 base_revision_id = revision[0].as_revision_id(branch)
@@ -5060,7 +5520,7 @@
             revision_id = branch.last_revision()
         revision_id = ensure_null(revision_id)
         if revision_id == NULL_REVISION:
-            raise errors.BzrCommandError('No revisions to bundle.')
+            raise errors.BzrCommandError(gettext('No revisions to bundle.'))
         directive = merge_directive.MergeDirective2.from_objects(
             branch.repository, revision_id, time.time(),
             osutils.local_time_offset(), submit_branch,
@@ -5110,6 +5570,12 @@
     source branch defaults to that containing the working directory, but can
     be changed using --from.
 
+    Both the submit branch and the public branch follow the usual behavior with
+    respect to --remember: If there is no default location set, the first send
+    will set it (use --no-remember to avoid setting it). After that, you can
+    omit the location to use the default.  To change the default, use
+    --remember. The value will only be saved if the location can be accessed.
+
     In order to calculate those changes, bzr must analyse the submit branch.
     Therefore it is most efficient for the submit branch to be a local mirror.
     If a public location is known for the submit_branch, that location is used
@@ -5184,7 +5650,7 @@
         ]
 
     def run(self, submit_branch=None, public_branch=None, no_bundle=False,
-            no_patch=False, revision=None, remember=False, output=None,
+            no_patch=False, revision=None, remember=None, output=None,
             format=None, mail_to=None, message=None, body=None,
             strict=None, **kwargs):
         from bzrlib.send import send
@@ -5314,27 +5780,37 @@
         self.add_cleanup(branch.lock_write().unlock)
         if delete:
             if tag_name is None:
-                raise errors.BzrCommandError("No tag specified to delete.")
+                raise errors.BzrCommandError(gettext("No tag specified to delete."))
             branch.tags.delete_tag(tag_name)
-            self.outf.write('Deleted tag %s.\n' % tag_name)
+            note(gettext('Deleted tag %s.') % tag_name)
         else:
             if revision:
                 if len(revision) != 1:
-                    raise errors.BzrCommandError(
+                    raise errors.BzrCommandError(gettext(
                         "Tags can only be placed on a single revision, "
-                        "not on a range")
+                        "not on a range"))
                 revision_id = revision[0].as_revision_id(branch)
             else:
                 revision_id = branch.last_revision()
             if tag_name is None:
                 tag_name = branch.automatic_tag_name(revision_id)
                 if tag_name is None:
-                    raise errors.BzrCommandError(
-                        "Please specify a tag name.")
-            if (not force) and branch.tags.has_tag(tag_name):
+                    raise errors.BzrCommandError(gettext(
+                        "Please specify a tag name."))
+            try:
+                existing_target = branch.tags.lookup_tag(tag_name)
+            except errors.NoSuchTag:
+                existing_target = None
+            if not force and existing_target not in (None, revision_id):
                 raise errors.TagAlreadyExists(tag_name)
-            branch.tags.set_tag(tag_name, revision_id)
-            self.outf.write('Created tag %s.\n' % tag_name)
+            if existing_target == revision_id:
+                note(gettext('Tag %s already exists for that revision.') % tag_name)
+            else:
+                branch.tags.set_tag(tag_name, revision_id)
+                if existing_target is None:
+                    note(gettext('Created tag %s.') % tag_name)
+                else:
+                    note(gettext('Updated tag %s.') % tag_name)
 
 
 class cmd_tags(Command):
@@ -5347,22 +5823,17 @@
     takes_options = [
         custom_help('directory',
             help='Branch whose tags should be displayed.'),
-        RegistryOption.from_kwargs('sort',
+        RegistryOption('sort',
             'Sort tags by different criteria.', title='Sorting',
-            alpha='Sort tags lexicographically (default).',
-            time='Sort tags chronologically.',
+            lazy_registry=('bzrlib.tag', 'tag_sort_methods')
             ),
         'show-ids',
         'revision',
     ]
 
     @display_command
-    def run(self,
-            directory='.',
-            sort='alpha',
-            show_ids=False,
-            revision=None,
-            ):
+    def run(self, directory='.', sort=None, show_ids=False, revision=None):
+        from bzrlib.tag import tag_sort_methods
         branch, relpath = Branch.open_containing(directory)
 
         tags = branch.tags.get_tag_dict().items()
@@ -5371,25 +5842,11 @@
 
         self.add_cleanup(branch.lock_read().unlock)
         if revision:
-            graph = branch.repository.get_graph()
-            rev1, rev2 = _get_revision_range(revision, branch, self.name())
-            revid1, revid2 = rev1.rev_id, rev2.rev_id
-            # only show revisions between revid1 and revid2 (inclusive)
-            tags = [(tag, revid) for tag, revid in tags if
-                graph.is_between(revid, revid1, revid2)]
-        if sort == 'alpha':
-            tags.sort()
-        elif sort == 'time':
-            timestamps = {}
-            for tag, revid in tags:
-                try:
-                    revobj = branch.repository.get_revision(revid)
-                except errors.NoSuchRevision:
-                    timestamp = sys.maxint # place them at the end
-                else:
-                    timestamp = revobj.timestamp
-                timestamps[revid] = timestamp
-            tags.sort(key=lambda x: timestamps[x[1]])
+            # Restrict to the specified range
+            tags = self._tags_for_range(branch, revision)
+        if sort is None:
+            sort = tag_sort_methods.get()
+        sort(branch, tags)
         if not show_ids:
             # [ (tag, revid), ... ] -> [ (tag, dotted_revno), ... ]
             for index, (tag, revid) in enumerate(tags):
@@ -5397,7 +5854,9 @@
                     revno = branch.revision_id_to_dotted_revno(revid)
                     if isinstance(revno, tuple):
                         revno = '.'.join(map(str, revno))
-                except errors.NoSuchRevision:
+                except (errors.NoSuchRevision,
+                        errors.GhostRevisionsHaveNoRevno,
+                        errors.UnsupportedOperation):
                     # Bad tag data/merges can lead to tagged revisions
                     # which are not in this branch. Fail gracefully ...
                     revno = '?'
@@ -5406,6 +5865,32 @@
         for tag, revspec in tags:
             self.outf.write('%-20s %s\n' % (tag, revspec))
 
+    def _tags_for_range(self, branch, revision):
+        range_valid = True
+        rev1, rev2 = _get_revision_range(revision, branch, self.name())
+        revid1, revid2 = rev1.rev_id, rev2.rev_id
+        # _get_revision_range will always set revid2 if it's not specified.
+        # If revid1 is None, it means we want to start from the branch
+        # origin which is always a valid ancestor. If revid1 == revid2, the
+        # ancestry check is useless.
+        if revid1 and revid1 != revid2:
+            # FIXME: We really want to use the same graph than
+            # branch.iter_merge_sorted_revisions below, but this is not
+            # easily available -- vila 2011-09-23
+            if branch.repository.get_graph().is_ancestor(revid2, revid1):
+                # We don't want to output anything in this case...
+                return []
+        # only show revisions between revid1 and revid2 (inclusive)
+        tagged_revids = branch.tags.get_reverse_tag_dict()
+        found = []
+        for r in branch.iter_merge_sorted_revisions(
+            start_revision_id=revid2, stop_revision_id=revid1,
+            stop_rule='include'):
+            revid_tags = tagged_revids.get(r[0], None)
+            if revid_tags:
+                found.extend([(tag, r[0]) for tag in revid_tags])
+        return found
+
 
 class cmd_reconfigure(Command):
     __doc__ = """Reconfigure the type of a bzr directory.
@@ -5425,18 +5910,30 @@
     takes_args = ['location?']
     takes_options = [
         RegistryOption.from_kwargs(
-            'target_type',
-            title='Target type',
-            help='The type to reconfigure the directory to.',
+            'tree_type',
+            title='Tree type',
+            help='The relation between branch and tree.',
             value_switches=True, enum_switch=False,
             branch='Reconfigure to be an unbound branch with no working tree.',
             tree='Reconfigure to be an unbound branch with a working tree.',
             checkout='Reconfigure to be a bound branch with a working tree.',
             lightweight_checkout='Reconfigure to be a lightweight'
                 ' checkout (with no local history).',
+            ),
+        RegistryOption.from_kwargs(
+            'repository_type',
+            title='Repository type',
+            help='Location fo the repository.',
+            value_switches=True, enum_switch=False,
             standalone='Reconfigure to be a standalone branch '
                 '(i.e. stop using shared repository).',
             use_shared='Reconfigure to use a shared repository.',
+            ),
+        RegistryOption.from_kwargs(
+            'repository_trees',
+            title='Trees in Repository',
+            help='Whether new branches in the repository have trees.',
+            value_switches=True, enum_switch=False,
             with_trees='Reconfigure repository to create '
                 'working trees on branches by default.',
             with_no_trees='Reconfigure repository to not create '
@@ -5456,12 +5953,12 @@
             ),
         ]
 
-    def run(self, location=None, target_type=None, bind_to=None, force=False,
-            stacked_on=None,
-            unstacked=None):
-        directory = bzrdir.BzrDir.open(location)
+    def run(self, location=None, bind_to=None, force=False,
+            tree_type=None, repository_type=None, repository_trees=None,
+            stacked_on=None, unstacked=None):
+        directory = controldir.ControlDir.open(location)
         if stacked_on and unstacked:
-            raise BzrCommandError("Can't use both --stacked-on and --unstacked")
+            raise errors.BzrCommandError(gettext("Can't use both --stacked-on and --unstacked"))
         elif stacked_on is not None:
             reconfigure.ReconfigureStackedOn().apply(directory, stacked_on)
         elif unstacked:
@@ -5469,33 +5966,44 @@
         # At the moment you can use --stacked-on and a different
         # reconfiguration shape at the same time; there seems no good reason
         # to ban it.
-        if target_type is None:
+        if (tree_type is None and
+            repository_type is None and
+            repository_trees is None):
             if stacked_on or unstacked:
                 return
             else:
-                raise errors.BzrCommandError('No target configuration '
-                    'specified')
-        elif target_type == 'branch':
+                raise errors.BzrCommandError(gettext('No target configuration '
+                    'specified'))
+        reconfiguration = None
+        if tree_type == 'branch':
             reconfiguration = reconfigure.Reconfigure.to_branch(directory)
-        elif target_type == 'tree':
+        elif tree_type == 'tree':
             reconfiguration = reconfigure.Reconfigure.to_tree(directory)
-        elif target_type == 'checkout':
+        elif tree_type == 'checkout':
             reconfiguration = reconfigure.Reconfigure.to_checkout(
                 directory, bind_to)
-        elif target_type == 'lightweight-checkout':
+        elif tree_type == 'lightweight-checkout':
             reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
                 directory, bind_to)
-        elif target_type == 'use-shared':
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
+        if repository_type == 'use-shared':
             reconfiguration = reconfigure.Reconfigure.to_use_shared(directory)
-        elif target_type == 'standalone':
+        elif repository_type == 'standalone':
             reconfiguration = reconfigure.Reconfigure.to_standalone(directory)
-        elif target_type == 'with-trees':
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
+        if repository_trees == 'with-trees':
             reconfiguration = reconfigure.Reconfigure.set_repository_trees(
                 directory, True)
-        elif target_type == 'with-no-trees':
+        elif repository_trees == 'with-no-trees':
             reconfiguration = reconfigure.Reconfigure.set_repository_trees(
                 directory, False)
-        reconfiguration.apply(force)
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
 
 
 class cmd_switch(Command):
@@ -5536,11 +6044,11 @@
         from bzrlib import switch
         tree_location = directory
         revision = _get_one_revision('switch', revision)
-        control_dir = bzrdir.BzrDir.open_containing(tree_location)[0]
+        control_dir = controldir.ControlDir.open_containing(tree_location)[0]
         if to_location is None:
             if revision is None:
-                raise errors.BzrCommandError('You must supply either a'
-                                             ' revision or a location')
+                raise errors.BzrCommandError(gettext('You must supply either a'
+                                             ' revision or a location'))
             to_location = tree_location
         try:
             branch = control_dir.open_branch()
@@ -5550,8 +6058,8 @@
             had_explicit_nick = False
         if create_branch:
             if branch is None:
-                raise errors.BzrCommandError('cannot create branch without'
-                                             ' source branch')
+                raise errors.BzrCommandError(gettext('cannot create branch without'
+                                             ' source branch'))
             to_location = directory_service.directories.dereference(
                               to_location)
             if '/' not in to_location and '\\' not in to_location:
@@ -5574,7 +6082,7 @@
         if had_explicit_nick:
             branch = control_dir.open_branch() #get the new branch!
             branch.nick = to_branch.nick
-        note('Switched to branch: %s',
+        note(gettext('Switched to branch: %s'),
             urlutils.unescape_for_display(to_branch.base, 'utf-8'))
 
     def _get_branch_location(self, control_dir):
@@ -5689,38 +6197,38 @@
             name = current_view
         if delete:
             if file_list:
-                raise errors.BzrCommandError(
-                    "Both --delete and a file list specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --delete and a file list specified"))
             elif switch:
-                raise errors.BzrCommandError(
-                    "Both --delete and --switch specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --delete and --switch specified"))
             elif all:
                 tree.views.set_view_info(None, {})
-                self.outf.write("Deleted all views.\n")
+                self.outf.write(gettext("Deleted all views.\n"))
             elif name is None:
-                raise errors.BzrCommandError("No current view to delete")
+                raise errors.BzrCommandError(gettext("No current view to delete"))
             else:
                 tree.views.delete_view(name)
-                self.outf.write("Deleted '%s' view.\n" % name)
+                self.outf.write(gettext("Deleted '%s' view.\n") % name)
         elif switch:
             if file_list:
-                raise errors.BzrCommandError(
-                    "Both --switch and a file list specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --switch and a file list specified"))
             elif all:
-                raise errors.BzrCommandError(
-                    "Both --switch and --all specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --switch and --all specified"))
             elif switch == 'off':
                 if current_view is None:
-                    raise errors.BzrCommandError("No current view to disable")
+                    raise errors.BzrCommandError(gettext("No current view to disable"))
                 tree.views.set_view_info(None, view_dict)
-                self.outf.write("Disabled '%s' view.\n" % (current_view))
+                self.outf.write(gettext("Disabled '%s' view.\n") % (current_view))
             else:
                 tree.views.set_view_info(switch, view_dict)
                 view_str = views.view_display_str(tree.views.lookup_view())
-                self.outf.write("Using '%s' view: %s\n" % (switch, view_str))
+                self.outf.write(gettext("Using '{0}' view: {1}\n").format(switch, view_str))
         elif all:
             if view_dict:
-                self.outf.write('Views defined:\n')
+                self.outf.write(gettext('Views defined:\n'))
                 for view in sorted(view_dict):
                     if view == current_view:
                         active = "=>"
@@ -5729,25 +6237,25 @@
                     view_str = views.view_display_str(view_dict[view])
                     self.outf.write('%s %-20s %s\n' % (active, view, view_str))
             else:
-                self.outf.write('No views defined.\n')
+                self.outf.write(gettext('No views defined.\n'))
         elif file_list:
             if name is None:
                 # No name given and no current view set
                 name = 'my'
             elif name == 'off':
-                raise errors.BzrCommandError(
-                    "Cannot change the 'off' pseudo view")
+                raise errors.BzrCommandError(gettext(
+                    "Cannot change the 'off' pseudo view"))
             tree.views.set_view(name, sorted(file_list))
             view_str = views.view_display_str(tree.views.lookup_view())
-            self.outf.write("Using '%s' view: %s\n" % (name, view_str))
+            self.outf.write(gettext("Using '{0}' view: {1}\n").format(name, view_str))
         else:
             # list the files
             if name is None:
                 # No name given and no current view set
-                self.outf.write('No current view.\n')
+                self.outf.write(gettext('No current view.\n'))
             else:
                 view_str = views.view_display_str(tree.views.lookup_view(name))
-                self.outf.write("'%s' view is: %s\n" % (name, view_str))
+                self.outf.write(gettext("'{0}' view is: {1}\n").format(name, view_str))
 
 
 class cmd_hooks(Command):
@@ -5767,7 +6275,7 @@
                         self.outf.write("    %s\n" %
                                         (some_hooks.get_hook_name(hook),))
                 else:
-                    self.outf.write("    <no hooks installed>\n")
+                    self.outf.write(gettext("    <no hooks installed>\n"))
 
 
 class cmd_remove_branch(Command):
@@ -5793,7 +6301,7 @@
             location = "."
         branch = Branch.open_containing(location)[0]
         branch.bzrdir.destroy_branch()
-        
+
 
 class cmd_shelve(Command):
     __doc__ = """Temporarily set aside some changes from the current tree.
@@ -5818,6 +6326,18 @@
 
     You can put multiple items on the shelf, and by default, 'unshelve' will
     restore the most recently shelved changes.
+
+    For complicated changes, it is possible to edit the changes in a separate
+    editor program to decide what the file remaining in the working copy
+    should look like.  To do this, add the configuration option
+
+        change_editor = PROGRAM @new_path @old_path
+
+    where @new_path is replaced with the path of the new version of the 
+    file and @old_path is replaced with the path of the old version of 
+    the file.  The PROGRAM should save the new file with the desired 
+    contents of the file in the working tree.
+        
     """
 
     takes_args = ['file*']
@@ -5835,12 +6355,12 @@
         Option('destroy',
                help='Destroy removed changes instead of shelving them.'),
     ]
-    _see_also = ['unshelve']
+    _see_also = ['unshelve', 'configuration']
 
     def run(self, revision=None, all=False, file_list=None, message=None,
-            writer=None, list=False, destroy=False, directory=u'.'):
+            writer=None, list=False, destroy=False, directory=None):
         if list:
-            return self.run_for_list()
+            return self.run_for_list(directory=directory)
         from bzrlib.shelf_ui import Shelver
         if writer is None:
             writer = bzrlib.option.diff_writer_registry.get()
@@ -5854,13 +6374,15 @@
         except errors.UserAbort:
             return 0
 
-    def run_for_list(self):
-        tree = WorkingTree.open_containing('.')[0]
+    def run_for_list(self, directory=None):
+        if directory is None:
+            directory = u'.'
+        tree = WorkingTree.open_containing(directory)[0]
         self.add_cleanup(tree.lock_read().unlock)
         manager = tree.get_shelf_manager()
         shelves = manager.active_shelves()
         if len(shelves) == 0:
-            note('No shelved changes.')
+            note(gettext('No shelved changes.'))
             return 0
         for shelf_id in reversed(shelves):
             message = manager.get_metadata(shelf_id).get('message')
@@ -5920,7 +6442,7 @@
     """
     takes_options = ['directory',
                      Option('ignored', help='Delete all ignored files.'),
-                     Option('detritus', help='Delete conflict files, merge'
+                     Option('detritus', help='Delete conflict files, merge and revert'
                             ' backups, and failed selftest dirs.'),
                      Option('unknown',
                             help='Delete files unknown to bzr (default).'),
@@ -5955,7 +6477,7 @@
         if path is not None:
             branchdir = path
         tree, branch, relpath =(
-            bzrdir.BzrDir.open_containing_tree_or_branch(branchdir))
+            controldir.ControlDir.open_containing_tree_or_branch(branchdir))
         if path is not None:
             path = relpath
         if tree is None:
@@ -5985,15 +6507,33 @@
             self.outf.write('%s %s\n' % (path, location))
 
 
+class cmd_export_pot(Command):
+    __doc__ = """Export command helps and error messages in po format."""
+
+    hidden = True
+    takes_options = [Option('plugin', 
+                            help='Export help text from named command '\
+                                 '(defaults to all built in commands).',
+                            type=str)]
+
+    def run(self, plugin=None):
+        from bzrlib.export_pot import export_pot
+        export_pot(self.outf, plugin)
+
+
 def _register_lazy_builtins():
     # register lazy builtins from other modules; called at startup and should
     # be only called once.
     for (name, aliases, module_name) in [
         ('cmd_bundle_info', [], 'bzrlib.bundle.commands'),
+        ('cmd_config', [], 'bzrlib.config'),
         ('cmd_dpush', [], 'bzrlib.foreign'),
         ('cmd_version_info', [], 'bzrlib.cmd_version_info'),
         ('cmd_resolve', ['resolved'], 'bzrlib.conflicts'),
         ('cmd_conflicts', [], 'bzrlib.conflicts'),
-        ('cmd_sign_my_commits', [], 'bzrlib.sign_my_commits'),
+        ('cmd_sign_my_commits', [], 'bzrlib.commit_signature_commands'),
+        ('cmd_verify_signatures', [],
+                                        'bzrlib.commit_signature_commands'),
+        ('cmd_test_script', [], 'bzrlib.cmd_test_script'),
         ]:
         builtin_command_registry.register_lazy(name, aliases, module_name)

=== modified file 'bzrlib/bundle/__init__.py'
--- bzrlib/bundle/__init__.py	2010-07-15 12:53:44 +0000
+++ bzrlib/bundle/__init__.py	2011-09-15 14:21:33 +0000
@@ -20,14 +20,12 @@
 lazy_import(globals(), """
 from bzrlib import (
     errors,
+    transport as _mod_transport,
     urlutils,
     )
 from bzrlib.bundle import serializer as _serializer
 from bzrlib.merge_directive import MergeDirective
-from bzrlib.transport import (
-    do_catching_redirections,
-    get_transport,
-    )
+from bzrlib.i18n import gettext
 """)
 from bzrlib.trace import note
 
@@ -38,7 +36,7 @@
     :return: An object supporting get_target_revision.  Raises NotABundle if
         the target is not a mergeable type.
     """
-    child_transport = get_transport(url,
+    child_transport = _mod_transport.get_transport(url,
         possible_transports=possible_transports)
     transport = child_transport.clone('..')
     filename = transport.relpath(child_transport.base)
@@ -56,12 +54,12 @@
         url, filename = urlutils.split(exception.target,
                                        exclude_trailing_slash=False)
         if not filename:
-            raise errors.NotABundle('A directory cannot be a bundle')
-        return get_transport(url)
+            raise errors.NotABundle(gettext('A directory cannot be a bundle'))
+        return _mod_transport.get_transport_from_url(url)
 
     try:
-        bytef, transport = do_catching_redirections(get_bundle, transport,
-                                                    redirected_transport)
+        bytef, transport = _mod_transport.do_catching_redirections(
+            get_bundle, transport, redirected_transport)
     except errors.TooManyRedirections:
         raise errors.NotABundle(transport.clone(filename).base)
     except (errors.ConnectionReset, errors.ConnectionError), e:

=== modified file 'bzrlib/bundle/apply_bundle.py'
--- bzrlib/bundle/apply_bundle.py	2010-02-10 17:52:08 +0000
+++ bzrlib/bundle/apply_bundle.py	2011-09-20 11:46:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,23 +17,24 @@
 This contains functionality for installing bundles into repositories
 """
 
-import bzrlib.ui
+from bzrlib import ui
+from bzrlib.i18n import gettext
+from bzrlib.merge import Merger
 from bzrlib.progress import ProgressPhase
-from bzrlib.merge import Merger
-from bzrlib.repository import install_revision
 from bzrlib.trace import note
+from bzrlib.vf_repository import install_revision
 
 
 def install_bundle(repository, bundle_reader):
     custom_install = getattr(bundle_reader, 'install', None)
     if custom_install is not None:
         return custom_install(repository)
-    pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    pb = ui.ui_factory.nested_progress_bar()
     repository.lock_write()
     try:
         real_revisions = bundle_reader.real_revisions
         for i, revision in enumerate(reversed(real_revisions)):
-            pb.update("Install revisions",i, len(real_revisions))
+            pb.update(gettext("Install revisions"),i, len(real_revisions))
             if repository.has_revision(revision.revision_id):
                 continue
             cset_tree = bundle_reader.revision_tree(repository,
@@ -47,7 +48,7 @@
 def merge_bundle(reader, tree, check_clean, merge_type,
                     reprocess, show_base, change_reporter=None):
     """Merge a revision bundle into the current tree."""
-    pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    pb = ui.ui_factory.nested_progress_bar()
     try:
         pp = ProgressPhase("Merge phase", 6, pb)
         pp.next_phase()
@@ -64,7 +65,7 @@
         merger.pp.next_phase()
         merger.find_base()
         if merger.base_rev_id == merger.other_rev_id:
-            note("Nothing to do.")
+            note(gettext("Nothing to do."))
             return 0
         merger.merge_type = merge_type
         merger.show_base = show_base

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2010-06-16 12:47:51 +0000
+++ bzrlib/bundle/bundle_data.py	2011-05-13 14:02:14 +0000
@@ -25,20 +25,22 @@
     osutils,
     timestamp,
     )
-import bzrlib.errors
 from bzrlib.bundle import apply_bundle
-from bzrlib.errors import (TestamentMismatch, BzrError,
-                           MalformedHeader, MalformedPatches, NotABundle)
-from bzrlib.inventory import (Inventory, InventoryEntry,
-                              InventoryDirectory, InventoryFile,
-                              InventoryLink)
-from bzrlib.osutils import sha_file, sha_string, pathjoin
+from bzrlib.errors import (
+    TestamentMismatch,
+    BzrError,
+    )
+from bzrlib.inventory import (
+    Inventory,
+    InventoryDirectory,
+    InventoryFile,
+    InventoryLink,
+    )
+from bzrlib.osutils import sha_string, pathjoin
 from bzrlib.revision import Revision, NULL_REVISION
 from bzrlib.testament import StrictTestament
 from bzrlib.trace import mutter, warning
-import bzrlib.transport
 from bzrlib.tree import Tree
-import bzrlib.urlutils
 from bzrlib.xml5 import serializer_v5
 
 
@@ -206,7 +208,7 @@
 
         inv = bundle_tree.inventory
         self._validate_inventory(inv, revision_id)
-        self._validate_revision(inv, revision_id)
+        self._validate_revision(bundle_tree, revision_id)
 
         return bundle_tree
 
@@ -286,10 +288,10 @@
             warning('Inventory sha hash mismatch for revision %s. %s'
                     ' != %s' % (revision_id, sha1, rev.inventory_sha1))
 
-    def _validate_revision(self, inventory, revision_id):
+    def _validate_revision(self, tree, revision_id):
         """Make sure all revision entries match their checksum."""
 
-        # This is a mapping from each revision id to it's sha hash
+        # This is a mapping from each revision id to its sha hash
         rev_to_sha1 = {}
 
         rev = self.get_revision(revision_id)
@@ -298,7 +300,7 @@
             raise AssertionError()
         if not (rev.revision_id == revision_id):
             raise AssertionError()
-        sha1 = self._testament_sha1(rev, inventory)
+        sha1 = self._testament_sha1(rev, tree)
         if sha1 != rev_info.sha1:
             raise TestamentMismatch(rev.revision_id, rev_info.sha1, sha1)
         if rev.revision_id in rev_to_sha1:
@@ -462,6 +464,7 @@
 
 
 class BundleTree(Tree):
+
     def __init__(self, base_tree, revision_id):
         self.base_tree = base_tree
         self._renamed = {} # Mapping from old_path => new_path
@@ -638,10 +641,11 @@
                 'Malformed patch for %s, %r' % (file_id, file_patch))
         return patched_file(file_patch, patch_original)
 
-    def get_symlink_target(self, file_id):
-        new_path = self.id2path(file_id)
+    def get_symlink_target(self, file_id, path=None):
+        if path is None:
+            path = self.id2path(file_id)
         try:
-            return self._targets[new_path]
+            return self._targets[path]
         except KeyError:
             return self.base_tree.get_symlink_target(file_id)
 
@@ -661,7 +665,7 @@
         path = self.id2path(file_id)
         if path in self._last_changed:
             return self._last_changed[path]
-        return self.base_tree.inventory[file_id].revision
+        return self.base_tree.get_file_revision(file_id)
 
     def get_size_and_sha1(self, file_id):
         """Return the size and sha1 hash of the given file id.
@@ -712,15 +716,14 @@
                 ie.executable = self.is_executable(file_id)
             elif kind == 'symlink':
                 ie = InventoryLink(file_id, name, parent_id)
-                ie.symlink_target = self.get_symlink_target(file_id)
+                ie.symlink_target = self.get_symlink_target(file_id, path)
             ie.revision = revision_id
 
-            if kind in ('directory', 'symlink'):
-                ie.text_size, ie.text_sha1 = None, None
-            else:
+            if kind == 'file':
                 ie.text_size, ie.text_sha1 = self.get_size_and_sha1(file_id)
-            if (ie.text_size is None) and (kind == 'file'):
-                raise BzrError('Got a text_size of None for file_id %r' % file_id)
+                if ie.text_size is None:
+                    raise BzrError(
+                        'Got a text_size of None for file_id %r' % file_id)
             inv.add(ie)
 
         sorted_entries = self.sorted_path_id()
@@ -740,11 +743,28 @@
         for path, entry in self.inventory.iter_entries():
             yield entry.file_id
 
+    def list_files(self, include_root=False, from_dir=None, recursive=True):
+        # The only files returned by this are those from the version
+        inv = self.inventory
+        if from_dir is None:
+            from_dir_id = None
+        else:
+            from_dir_id = inv.path2id(from_dir)
+            if from_dir_id is None:
+                # Directory not versioned
+                return
+        entries = inv.iter_entries(from_dir=from_dir_id, recursive=recursive)
+        if inv.root is not None and not include_root and from_dir is None:
+            # skip the root for compatability with the current apis.
+            entries.next()
+        for path, entry in entries:
+            yield path, 'V', entry.kind, entry.file_id, entry
+
     def sorted_path_id(self):
         paths = []
         for result in self._new_id.iteritems():
             paths.append(result)
-        for id in self.base_tree:
+        for id in self.base_tree.all_file_ids():
             path = self.id2path(id)
             if path is None:
                 continue

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2010-04-02 19:12:58 +0000
+++ bzrlib/bundle/commands.py	2011-09-20 13:31:00 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 and for applying a changeset.
 """
 
-import sys
 from cStringIO import StringIO
 
 from bzrlib.lazy_import import lazy_import
@@ -33,11 +32,10 @@
     urlutils,
     transport,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.commands import Command
-from bzrlib.option import Option
-from bzrlib.trace import note
 
 
 class cmd_bundle_info(Command):
@@ -59,11 +57,11 @@
             bundle_info = read_bundle(bundle_file)
         else:
             if verbose:
-                raise errors.BzrCommandError('--verbose requires a merge'
-                    ' directive')
+                raise errors.BzrCommandError(gettext(
+                            '--verbose requires a merge directive'))
         reader_method = getattr(bundle_info, 'get_bundle_reader', None)
         if reader_method is None:
-            raise errors.BzrCommandError('Bundle format not supported')
+            raise errors.BzrCommandError(gettext('Bundle format not supported'))
 
         by_kind = {}
         file_ids = set()
@@ -73,13 +71,13 @@
                 (bytes, parents, repo_kind, revision_id, file_id))
             if file_id is not None:
                 file_ids.add(file_id)
-        self.outf.write('Records\n')
+        self.outf.write(gettext('Records\n'))
         for kind, records in sorted(by_kind.iteritems()):
             multiparent = sum(1 for b, m, k, r, f in records if
                               len(m.get('parents', [])) > 1)
-            self.outf.write('%s: %d (%d multiparent)\n' % \
-                (kind, len(records), multiparent))
-        self.outf.write('unique files: %d\n' % len(file_ids))
+            self.outf.write(gettext('{0}: {1} ({2} multiparent)\n').format(
+                kind, len(records), multiparent))
+        self.outf.write(gettext('unique files: %d\n') % len(file_ids))
         self.outf.write('\n')
         nicks = set()
         committers = set()
@@ -88,10 +86,10 @@
                 nicks.add(revision.properties['branch-nick'])
             committers.add(revision.committer)
 
-        self.outf.write('Revisions\n')
-        self.outf.write(('nicks: %s\n'
+        self.outf.write(gettext('Revisions\n'))
+        self.outf.write((gettext('nicks: %s\n')
             % ', '.join(sorted(nicks))).encode(term_encoding, 'replace'))
-        self.outf.write(('committers: \n%s\n' %
+        self.outf.write((gettext('committers: \n%s\n') %
         '\n'.join(sorted(committers)).encode(term_encoding, 'replace')))
         if verbose:
             self.outf.write('\n')
@@ -99,6 +97,6 @@
             line = bundle_file.readline()
             line = bundle_file.readline()
             content = bundle_file.read().decode('bz2')
-            self.outf.write("Decoded contents\n")
+            self.outf.write(gettext("Decoded contents\n"))
             self.outf.write(content)
             self.outf.write('\n')

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2010-07-15 12:53:00 +0000
+++ bzrlib/bundle/serializer/__init__.py	2011-06-16 11:58:22 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,10 @@
 from StringIO import StringIO
 import re
 
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    pyutils,
+    )
 from bzrlib.diff import internal_diff
 from bzrlib.revision import NULL_REVISION
 # For backwards-compatibility
@@ -156,10 +159,9 @@
         forced_bases = {revision_id:base_revision_id}
         if base_revision_id is NULL_REVISION:
             base_revision_id = None
-        revision_ids = set(repository.get_ancestry(revision_id,
-                           topo_sorted=False))
-        revision_ids.difference_update(repository.get_ancestry(
-            base_revision_id, topo_sorted=False))
+        graph = repository.get_graph()
+        revision_ids = graph.find_unique_ancestors(revision_id,
+            [base_revision_id])
         revision_ids = list(repository.get_graph().iter_topo_order(
             revision_ids))
         revision_ids.reverse()
@@ -191,8 +193,7 @@
     :param overwrite: Should this version override a default
     """
     def _loader(version):
-        mod = __import__(module, globals(), locals(), [classname])
-        klass = getattr(mod, classname)
+        klass = pyutils.get_named_object(module, classname)
         return klass(version)
     register(version, _loader, overwrite=overwrite)
 

=== modified file 'bzrlib/bundle/serializer/v08.py'
--- bzrlib/bundle/serializer/v08.py	2009-06-05 23:15:23 +0000
+++ bzrlib/bundle/serializer/v08.py	2011-06-14 02:21:41 +0000
@@ -263,7 +263,7 @@
 
         def do_diff(file_id, old_path, new_path, action, force_binary):
             def tree_lines(tree, require_text=False):
-                if file_id in tree:
+                if tree.has_id(file_id):
                     tree_file = tree.get_file(file_id)
                     if require_text is True:
                         tree_file = text_file(tree_file)
@@ -553,5 +553,5 @@
         testament = StrictTestament.from_revision(repository, revision_id)
         return testament.as_sha1()
 
-    def _testament_sha1(self, revision, inventory):
-        return StrictTestament(revision, inventory).as_sha1()
+    def _testament_sha1(self, revision, tree):
+        return StrictTestament(revision, tree).as_sha1()

=== modified file 'bzrlib/bundle/serializer/v09.py'
--- bzrlib/bundle/serializer/v09.py	2009-03-23 14:59:43 +0000
+++ bzrlib/bundle/serializer/v09.py	2011-04-18 15:53:40 +0000
@@ -63,8 +63,8 @@
         testament = StrictTestament3.from_revision(repository, revision_id)
         return testament.as_sha1()
 
-    def _testament_sha1(self, revision, inventory):
-        return StrictTestament3(revision, inventory).as_sha1()
+    def _testament_sha1(self, revision, tree):
+        return StrictTestament3(revision, tree).as_sha1()
 
 
 class BundleReaderV09(BundleReader):

=== modified file 'bzrlib/bundle/serializer/v4.py'
--- bzrlib/bundle/serializer/v4.py	2009-08-04 14:08:32 +0000
+++ bzrlib/bundle/serializer/v4.py	2011-09-24 12:57:04 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 import re
 
 from bzrlib import (
-    diff,
     errors,
     iterablefile,
     lru_cache,
@@ -30,11 +29,50 @@
     serializer,
     trace,
     ui,
+    versionedfile as _mod_versionedfile,
     )
 from bzrlib.bundle import bundle_data, serializer as bundle_serializer
+from bzrlib.i18n import ngettext
 from bzrlib import bencode
 
 
+class _MPDiffInventoryGenerator(_mod_versionedfile._MPDiffGenerator):
+    """Generate Inventory diffs serialized inventories."""
+
+    def __init__(self, repo, inventory_keys):
+        super(_MPDiffInventoryGenerator, self).__init__(repo.inventories,
+            inventory_keys)
+        self.repo = repo
+        self.sha1s = {}
+
+    def iter_diffs(self):
+        """Compute the diffs one at a time."""
+        # This is instead of compute_diffs() since we guarantee our ordering of
+        # inventories, we don't have to do any buffering
+        self._find_needed_keys()
+        # We actually use a slightly different ordering. We grab all of the
+        # parents first, and then grab the ordered requests.
+        needed_ids = [k[-1] for k in self.present_parents]
+        needed_ids.extend([k[-1] for k in self.ordered_keys])
+        inv_to_str = self.repo._serializer.write_inventory_to_string
+        for inv in self.repo.iter_inventories(needed_ids):
+            revision_id = inv.revision_id
+            key = (revision_id,)
+            if key in self.present_parents:
+                # Not a key we will transmit, which is a shame, since because
+                # of that bundles don't work with stacked branches
+                parent_ids = None
+            else:
+                parent_ids = [k[-1] for k in self.parent_map[key]]
+            as_bytes = inv_to_str(inv)
+            self._process_one_record(key, (as_bytes,))
+            if parent_ids is None:
+                continue
+            diff = self.diffs.pop(key)
+            sha1 = osutils.sha_string(as_bytes)
+            yield revision_id, parent_ids, sha1, diff
+
+
 class BundleWriter(object):
     """Writer for bundle-format files.
 
@@ -284,7 +322,8 @@
 
     def do_write(self):
         """Write all data to the bundle"""
-        trace.note('Bundling %d revision(s).', len(self.revision_ids))
+        trace.note(ngettext('Bundling %d revision.', 'Bundling %d revisions.',
+                            len(self.revision_ids)), len(self.revision_ids))
         self.repository.lock_read()
         try:
             self.bundle.begin()
@@ -348,48 +387,10 @@
         the other side.
         """
         inventory_key_order = [(r,) for r in revision_order]
-        parent_map = self.repository.inventories.get_parent_map(
-                            inventory_key_order)
-        missing_keys = set(inventory_key_order).difference(parent_map)
-        if missing_keys:
-            raise errors.RevisionNotPresent(list(missing_keys)[0],
-                                            self.repository.inventories)
-        inv_to_str = self.repository._serializer.write_inventory_to_string
-        # Make sure that we grab the parent texts first
-        just_parents = set()
-        map(just_parents.update, parent_map.itervalues())
-        just_parents.difference_update(parent_map)
-        # Ignore ghost parents
-        present_parents = self.repository.inventories.get_parent_map(
-                            just_parents)
-        ghost_keys = just_parents.difference(present_parents)
-        needed_inventories = list(present_parents) + inventory_key_order
-        needed_inventories = [k[-1] for k in needed_inventories]
-        all_lines = {}
-        for inv in self.repository.iter_inventories(needed_inventories):
-            revision_id = inv.revision_id
-            key = (revision_id,)
-            as_bytes = inv_to_str(inv)
-            # The sha1 is validated as the xml/textual form, not as the
-            # form-in-the-repository
-            sha1 = osutils.sha_string(as_bytes)
-            as_lines = osutils.split_lines(as_bytes)
-            del as_bytes
-            all_lines[key] = as_lines
-            if key in just_parents:
-                # We don't transmit those entries
-                continue
-            # Create an mpdiff for this text, and add it to the output
-            parent_keys = parent_map[key]
-            # See the comment in VF.make_mpdiffs about how this effects
-            # ordering when there are ghosts present. I think we have a latent
-            # bug
-            parent_lines = [all_lines[p_key] for p_key in parent_keys
-                            if p_key not in ghost_keys]
-            diff = multiparent.MultiParent.from_lines(
-                as_lines, parent_lines)
+        generator = _MPDiffInventoryGenerator(self.repository,
+                                              inventory_key_order)
+        for revision_id, parent_ids, sha1, diff in generator.iter_diffs():
             text = ''.join(diff.to_patch())
-            parent_ids = [k[-1] for k in parent_keys]
             self.bundle.add_multiparent_record(text, sha1, parent_ids,
                                                'inventory', revision_id, None)
 

=== added file 'bzrlib/bzr_distutils.py'
--- bzrlib/bzr_distutils.py	1970-01-01 00:00:00 +0000
+++ bzrlib/bzr_distutils.py	2011-09-27 15:19:38 +0000
@@ -0,0 +1,116 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2007,2009,2011 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+# This code is from bzr-explorer and modified for bzr.
+
+"""build_mo command for setup.py"""
+
+from distutils import log
+from distutils.core import Command
+from distutils.dep_util import newer
+from distutils.spawn import find_executable
+import os
+import re
+
+
+class build_mo(Command):
+    """Subcommand of build command: build_mo"""
+
+    description = 'compile po files to mo files'
+
+    # List of options:
+    #   - long name,
+    #   - short name (None if no short name),
+    #   - help string.
+    user_options = [('build-dir=', 'd', 'Directory to build locale files'),
+                    ('output-base=', 'o', 'mo-files base name'),
+                    ('source-dir=', None, 'Directory with sources po files'),
+                    ('force', 'f', 'Force creation of mo files'),
+                    ('lang=', None, 'Comma-separated list of languages '
+                                    'to process'),
+                   ]
+
+    boolean_options = ['force']
+
+    def initialize_options(self):
+        self.build_dir = None
+        self.output_base = None
+        self.source_dir = None
+        self.force = None
+        self.lang = None
+
+    def finalize_options(self):
+        self.set_undefined_options('build', ('force', 'force'))
+        self.prj_name = self.distribution.get_name()
+        if self.build_dir is None:
+            self.build_dir = 'bzrlib/locale'
+        if not self.output_base:
+            self.output_base = self.prj_name or 'messages'
+        if self.source_dir is None:
+            self.source_dir = 'po'
+        if self.lang is None:
+            re_po = re.compile(r'^([a-zA-Z_]+)\.po$')
+            self.lang = []
+            for i in os.listdir(self.source_dir):
+                mo = re_po.match(i)
+                if mo:
+                    self.lang.append(mo.group(1))
+        else:
+            self.lang = [i.strip() for i in self.lang.split(',') if i.strip()]
+
+    def run(self):
+        """Run msgfmt for each language"""
+        if not self.lang:
+            return
+
+        if find_executable('msgfmt') is None:
+            log.warn("GNU gettext msgfmt utility not found!")
+            log.warn("Skip compiling po files.")
+            return
+
+        if 'en' in self.lang:
+            if find_executable('msginit') is None:
+                log.warn("GNU gettext msginit utility not found!")
+                log.warn("Skip creating English PO file.")
+            else:
+                log.info('Creating English PO file...')
+                pot = (self.prj_name or 'messages') + '.pot'
+                en_po = 'en.po'
+                self.spawn(['msginit',
+                    '--no-translator',
+                    '-l', 'en',
+                    '-i', os.path.join(self.source_dir, pot),
+                    '-o', os.path.join(self.source_dir, en_po),
+                    ])
+
+        basename = self.output_base
+        if not basename.endswith('.mo'):
+            basename += '.mo'
+
+        for lang in self.lang:
+            po = os.path.join('po', lang + '.po')
+            if not os.path.isfile(po):
+                po = os.path.join('po', lang + '.po')
+            dir_ = os.path.join(self.build_dir, lang, 'LC_MESSAGES')
+            self.mkpath(dir_)
+            mo = os.path.join(dir_, basename)
+            if self.force or newer(po, mo):
+                log.info('Compile: %s -> %s' % (po, mo))
+                self.spawn(['msgfmt', '-o', mo, po])
+
+

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2010-06-28 03:12:48 +0000
+++ bzrlib/bzrdir.py	2011-10-14 10:48:50 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,103 +25,56 @@
 objects returned.
 """
 
-# TODO: Move old formats into a plugin to make this file smaller.
-
-import os
 import sys
-import warnings
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from stat import S_ISDIR
-import textwrap
-
 import bzrlib
 from bzrlib import (
-    branch,
-    config,
+    branch as _mod_branch,
+    cleanup,
     errors,
+    fetch,
     graph,
     lockable_files,
     lockdir,
     osutils,
+    pyutils,
     remote,
     repository,
     revision as _mod_revision,
+    transport as _mod_transport,
     ui,
     urlutils,
-    versionedfile,
     win32utils,
-    workingtree,
+    workingtree_3,
     workingtree_4,
-    xml4,
-    xml5,
-    )
-from bzrlib.osutils import (
-    sha_string,
-    )
-from bzrlib.push import (
-    PushResult,
-    )
-from bzrlib.repofmt import pack_repo
-from bzrlib.smart.client import _SmartClient
-from bzrlib.store.versioned import WeaveStore
-from bzrlib.transactions import WriteTransaction
+    )
+from bzrlib.repofmt import knitpack_repo
 from bzrlib.transport import (
     do_catching_redirections,
-    get_transport,
     local,
     )
-from bzrlib.weave import Weave
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.trace import (
     mutter,
     note,
-    warning,
     )
 
 from bzrlib import (
-    hooks,
+    config,
+    controldir,
     registry,
-    symbol_versioning,
-    )
-    
-    
-class ControlComponent(object):
-    """Abstract base class for control directory components.
-    
-    This provides interfaces that are common across bzrdirs, 
-    repositories, branches, and workingtree control directories.
-    
-    They all expose two urls and transports: the *user* URL is the 
-    one that stops above the control directory (eg .bzr) and that 
-    should normally be used in messages, and the *control* URL is
-    under that in eg .bzr/checkout and is used to read the control
-    files.
-    
-    This can be used as a mixin and is intended to fit with 
-    foreign formats.
-    """
-    
-    @property
-    def control_transport(self):
-        raise NotImplementedError
-   
-    @property
-    def control_url(self):
-        return self.control_transport.base
-    
-    @property
-    def user_transport(self):
-        raise NotImplementedError
-        
-    @property
-    def user_url(self):
-        return self.user_transport.base
-    
-
-class BzrDir(ControlComponent):
+    )
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    deprecated_method,
+    )
+
+
+class BzrDir(controldir.ControlDir):
     """A .bzr control diretory.
 
     BzrDir instances let you create or open any of the things that can be
@@ -158,10 +111,6 @@
                     return
         thing_to_unlock.break_lock()
 
-    def can_convert_format(self):
-        """Return true if this bzrdir is one whose format we can convert from."""
-        return True
-
     def check_conversion_target(self, target_format):
         """Check that a bzrdir as a whole can be converted to a new format."""
         # The only current restriction is that the repository content can be 
@@ -174,56 +123,9 @@
             # No repo, no problem.
             pass
 
-    @staticmethod
-    def _check_supported(format, allow_unsupported,
-        recommend_upgrade=True,
-        basedir=None):
-        """Give an error or warning on old formats.
-
-        :param format: may be any kind of format - workingtree, branch,
-        or repository.
-
-        :param allow_unsupported: If true, allow opening
-        formats that are strongly deprecated, and which may
-        have limited functionality.
-
-        :param recommend_upgrade: If true (default), warn
-        the user through the ui object that they may wish
-        to upgrade the object.
-        """
-        # TODO: perhaps move this into a base Format class; it's not BzrDir
-        # specific. mbp 20070323
-        if not allow_unsupported and not format.is_supported():
-            # see open_downlevel to open legacy branches.
-            raise errors.UnsupportedFormatError(format=format)
-        if recommend_upgrade \
-            and getattr(format, 'upgrade_recommended', False):
-            ui.ui_factory.recommend_upgrade(
-                format.get_format_description(),
-                basedir)
-
-    def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
-        """Clone this bzrdir and its contents to url verbatim.
-
-        :param url: The url create the clone at.  If url's last component does
-            not exist, it will be created.
-        :param revision_id: The tip revision-id to use for any branch or
-            working tree.  If not None, then the clone operation may tune
-            itself to download less data.
-        :param force_new_repo: Do not use a shared repository for the target
-                               even if one is available.
-        :param preserve_stacking: When cloning a stacked branch, stack the
-            new branch on top of the other branch's stacked-on branch.
-        """
-        return self.clone_on_transport(get_transport(url),
-                                       revision_id=revision_id,
-                                       force_new_repo=force_new_repo,
-                                       preserve_stacking=preserve_stacking)
-
     def clone_on_transport(self, transport, revision_id=None,
         force_new_repo=False, preserve_stacking=False, stacked_on=None,
-        create_prefix=False, use_existing_dir=True):
+        create_prefix=False, use_existing_dir=True, no_tree=False):
         """Clone this bzrdir and its contents to transport verbatim.
 
         :param transport: The transport for the location to produce the clone
@@ -238,15 +140,16 @@
         :param create_prefix: Create any missing directories leading up to
             to_transport.
         :param use_existing_dir: Use an existing directory if one exists.
+        :param no_tree: If set to true prevents creation of a working tree.
         """
         # Overview: put together a broad description of what we want to end up
         # with; then make as few api calls as possible to do it.
-        
+
         # We may want to create a repo/branch/tree, if we do so what format
         # would we want for each:
         require_stacking = (stacked_on is not None)
         format = self.cloning_metadir(require_stacking)
-        
+
         # Figure out what objects we want:
         try:
             local_repo = self.find_repository()
@@ -271,7 +174,7 @@
         # we should look up the policy needs first, or just use it as a hint,
         # or something.
         if local_repo:
-            make_working_trees = local_repo.make_working_trees()
+            make_working_trees = local_repo.make_working_trees() and not no_tree
             want_shared = local_repo.is_shared()
             repo_format_name = format.repository_format.network_name()
         else:
@@ -318,7 +221,7 @@
             # the tree and fail.
             result.root_transport.local_abspath('.')
             if result_repo is None or result_repo.make_working_trees():
-                self.open_workingtree().clone(result)
+                self.open_workingtree().clone(result, revision_id=revision_id)
         except (errors.NoWorkingTree, errors.NotLocalUrl):
             pass
         return result
@@ -326,148 +229,8 @@
     # TODO: This should be given a Transport, and should chdir up; otherwise
     # this will open a new connection.
     def _make_tail(self, url):
-        t = get_transport(url)
-        t.ensure_base()
-
-    @classmethod
-    def create(cls, base, format=None, possible_transports=None):
-        """Create a new BzrDir at the url 'base'.
-
-        :param format: If supplied, the format of branch to create.  If not
-            supplied, the default is used.
-        :param possible_transports: If supplied, a list of transports that
-            can be reused to share a remote connection.
-        """
-        if cls is not BzrDir:
-            raise AssertionError("BzrDir.create always creates the default"
-                " format, not one of %r" % cls)
-        t = get_transport(base, possible_transports)
-        t.ensure_base()
-        if format is None:
-            format = BzrDirFormat.get_default_format()
-        return format.initialize_on_transport(t)
-
-    @staticmethod
-    def find_bzrdirs(transport, evaluate=None, list_current=None):
-        """Find bzrdirs recursively from current location.
-
-        This is intended primarily as a building block for more sophisticated
-        functionality, like finding trees under a directory, or finding
-        branches that use a given repository.
-        :param evaluate: An optional callable that yields recurse, value,
-            where recurse controls whether this bzrdir is recursed into
-            and value is the value to yield.  By default, all bzrdirs
-            are recursed into, and the return value is the bzrdir.
-        :param list_current: if supplied, use this function to list the current
-            directory, instead of Transport.list_dir
-        :return: a generator of found bzrdirs, or whatever evaluate returns.
-        """
-        if list_current is None:
-            def list_current(transport):
-                return transport.list_dir('')
-        if evaluate is None:
-            def evaluate(bzrdir):
-                return True, bzrdir
-
-        pending = [transport]
-        while len(pending) > 0:
-            current_transport = pending.pop()
-            recurse = True
-            try:
-                bzrdir = BzrDir.open_from_transport(current_transport)
-            except (errors.NotBranchError, errors.PermissionDenied):
-                pass
-            else:
-                recurse, value = evaluate(bzrdir)
-                yield value
-            try:
-                subdirs = list_current(current_transport)
-            except (errors.NoSuchFile, errors.PermissionDenied):
-                continue
-            if recurse:
-                for subdir in sorted(subdirs, reverse=True):
-                    pending.append(current_transport.clone(subdir))
-
-    def list_branches(self):
-        """Return a sequence of all branches local to this control directory.
-
-        """
-        try:
-            return [self.open_branch()]
-        except (errors.NotBranchError, errors.NoRepositoryPresent):
-            return []
-
-    @staticmethod
-    def find_branches(transport):
-        """Find all branches under a transport.
-
-        This will find all branches below the transport, including branches
-        inside other branches.  Where possible, it will use
-        Repository.find_branches.
-
-        To list all the branches that use a particular Repository, see
-        Repository.find_branches
-        """
-        def evaluate(bzrdir):
-            try:
-                repository = bzrdir.open_repository()
-            except errors.NoRepositoryPresent:
-                pass
-            else:
-                return False, ([], repository)
-            return True, (bzrdir.list_branches(), None)
-        ret = []
-        for branches, repo in BzrDir.find_bzrdirs(transport,
-                                                  evaluate=evaluate):
-            if repo is not None:
-                ret.extend(repo.find_branches())
-            if branches is not None:
-                ret.extend(branches)
-        return ret
-
-    def destroy_repository(self):
-        """Destroy the repository in this BzrDir"""
-        raise NotImplementedError(self.destroy_repository)
-
-    def create_branch(self, name=None):
-        """Create a branch in this BzrDir.
-
-        :param name: Name of the colocated branch to create, None for
-            the default branch.
-
-        The bzrdir's format will control what branch format is created.
-        For more control see BranchFormatXX.create(a_bzrdir).
-        """
-        raise NotImplementedError(self.create_branch)
-
-    def destroy_branch(self, name=None):
-        """Destroy a branch in this BzrDir.
-        
-        :param name: Name of the branch to destroy, None for the default 
-            branch.
-        """
-        raise NotImplementedError(self.destroy_branch)
-
-    @staticmethod
-    def create_branch_and_repo(base, force_new_repo=False, format=None):
-        """Create a new BzrDir, Branch and Repository at the url 'base'.
-
-        This will use the current default BzrDirFormat unless one is
-        specified, and use whatever
-        repository format that that uses via bzrdir.create_branch and
-        create_repository. If a shared repository is available that is used
-        preferentially.
-
-        The created Branch object is returned.
-
-        :param base: The URL to create the branch at.
-        :param force_new_repo: If True a new repository is always created.
-        :param format: If supplied, the format of branch to create.  If not
-            supplied, the default is used.
-        """
-        bzrdir = BzrDir.create(base, format)
-        bzrdir._find_or_create_repository(force_new_repo)
-        return bzrdir.create_branch()
+        t = _mod_transport.get_transport(url)
+        t.ensure_base()
 
     def determine_repository_policy(self, force_new_repo=False, stack_on=None,
                                     stack_on_pwd=None, require_stacking=False):
@@ -535,109 +298,185 @@
         policy = self.determine_repository_policy(force_new_repo)
         return policy.acquire_repository()[0]
 
-    @staticmethod
-    def create_branch_convenience(base, force_new_repo=False,
-                                  force_new_tree=None, format=None,
-                                  possible_transports=None):
-        """Create a new BzrDir, Branch and Repository at the url 'base'.
-
-        This is a convenience function - it will use an existing repository
-        if possible, can be told explicitly whether to create a working tree or
-        not.
-
-        This will use the current default BzrDirFormat unless one is
-        specified, and use whatever
-        repository format that that uses via bzrdir.create_branch and
-        create_repository. If a shared repository is available that is used
-        preferentially. Whatever repository is used, its tree creation policy
-        is followed.
-
-        The created Branch object is returned.
-        If a working tree cannot be made due to base not being a file:// url,
-        no error is raised unless force_new_tree is True, in which case no
-        data is created on disk and NotLocalUrl is raised.
-
-        :param base: The URL to create the branch at.
-        :param force_new_repo: If True a new repository is always created.
-        :param force_new_tree: If True or False force creation of a tree or
-                               prevent such creation respectively.
-        :param format: Override for the bzrdir format to create.
-        :param possible_transports: An optional reusable transports list.
+    def _find_source_repo(self, add_cleanup, source_branch):
+        """Find the source branch and repo for a sprout operation.
+        
+        This is helper intended for use by _sprout.
+
+        :returns: (source_branch, source_repository).  Either or both may be
+            None.  If not None, they will be read-locked (and their unlock(s)
+            scheduled via the add_cleanup param).
         """
-        if force_new_tree:
-            # check for non local urls
-            t = get_transport(base, possible_transports)
-            if not isinstance(t, local.LocalTransport):
-                raise errors.NotLocalUrl(base)
-        bzrdir = BzrDir.create(base, format, possible_transports)
-        repo = bzrdir._find_or_create_repository(force_new_repo)
-        result = bzrdir.create_branch()
-        if force_new_tree or (repo.make_working_trees() and
-                              force_new_tree is None):
+        if source_branch is not None:
+            add_cleanup(source_branch.lock_read().unlock)
+            return source_branch, source_branch.repository
+        try:
+            source_branch = self.open_branch()
+            source_repository = source_branch.repository
+        except errors.NotBranchError:
+            source_branch = None
             try:
-                bzrdir.create_workingtree()
-            except errors.NotLocalUrl:
-                pass
-        return result
-
-    @staticmethod
-    def create_standalone_workingtree(base, format=None):
-        """Create a new BzrDir, WorkingTree, Branch and Repository at 'base'.
-
-        'base' must be a local path or a file:// url.
-
-        This will use the current default BzrDirFormat unless one is
-        specified, and use whatever
-        repository format that that uses for bzrdirformat.create_workingtree,
-        create_branch and create_repository.
-
-        :param format: Override for the bzrdir format to create.
-        :return: The WorkingTree object.
-        """
-        t = get_transport(base)
-        if not isinstance(t, local.LocalTransport):
-            raise errors.NotLocalUrl(base)
-        bzrdir = BzrDir.create_branch_and_repo(base,
-                                               force_new_repo=True,
-                                               format=format).bzrdir
-        return bzrdir.create_workingtree()
-
-    def create_workingtree(self, revision_id=None, from_branch=None,
-        accelerator_tree=None, hardlink=False):
-        """Create a working tree at this BzrDir.
-
-        :param revision_id: create it as of this revision id.
-        :param from_branch: override bzrdir branch (for lightweight checkouts)
+                source_repository = self.open_repository()
+            except errors.NoRepositoryPresent:
+                source_repository = None
+            else:
+                add_cleanup(source_repository.lock_read().unlock)
+        else:
+            add_cleanup(source_branch.lock_read().unlock)
+        return source_branch, source_repository
+
+    def sprout(self, url, revision_id=None, force_new_repo=False,
+               recurse='down', possible_transports=None,
+               accelerator_tree=None, hardlink=False, stacked=False,
+               source_branch=None, create_tree_if_local=True):
+        """Create a copy of this controldir prepared for use as a new line of
+        development.
+
+        If url's last component does not exist, it will be created.
+
+        Attributes related to the identity of the source branch like
+        branch nickname will be cleaned, a working tree is created
+        whether one existed before or not; and a local branch is always
+        created.
+
+        if revision_id is not None, then the clone operation may tune
+            itself to download less data.
+
         :param accelerator_tree: A tree which can be used for retrieving file
             contents more quickly than the revision tree, i.e. a workingtree.
             The revision tree will be used for cases where accelerator_tree's
             content is different.
-        """
-        raise NotImplementedError(self.create_workingtree)
+        :param hardlink: If true, hard-link files from accelerator_tree,
+            where possible.
+        :param stacked: If true, create a stacked branch referring to the
+            location of this control directory.
+        :param create_tree_if_local: If true, a working-tree will be created
+            when working locally.
+        """
+        operation = cleanup.OperationWithCleanups(self._sprout)
+        return operation.run(url, revision_id=revision_id,
+            force_new_repo=force_new_repo, recurse=recurse,
+            possible_transports=possible_transports,
+            accelerator_tree=accelerator_tree, hardlink=hardlink,
+            stacked=stacked, source_branch=source_branch,
+            create_tree_if_local=create_tree_if_local)
+
+    def _sprout(self, op, url, revision_id=None, force_new_repo=False,
+               recurse='down', possible_transports=None,
+               accelerator_tree=None, hardlink=False, stacked=False,
+               source_branch=None, create_tree_if_local=True):
+        add_cleanup = op.add_cleanup
+        fetch_spec_factory = fetch.FetchSpecFactory()
+        if revision_id is not None:
+            fetch_spec_factory.add_revision_ids([revision_id])
+            fetch_spec_factory.source_branch_stop_revision_id = revision_id
+        target_transport = _mod_transport.get_transport(url,
+            possible_transports)
+        target_transport.ensure_base()
+        cloning_format = self.cloning_metadir(stacked)
+        # Create/update the result branch
+        result = cloning_format.initialize_on_transport(target_transport)
+        source_branch, source_repository = self._find_source_repo(
+            add_cleanup, source_branch)
+        fetch_spec_factory.source_branch = source_branch
+        # if a stacked branch wasn't requested, we don't create one
+        # even if the origin was stacked
+        if stacked and source_branch is not None:
+            stacked_branch_url = self.root_transport.base
+        else:
+            stacked_branch_url = None
+        repository_policy = result.determine_repository_policy(
+            force_new_repo, stacked_branch_url, require_stacking=stacked)
+        result_repo, is_new_repo = repository_policy.acquire_repository()
+        add_cleanup(result_repo.lock_write().unlock)
+        fetch_spec_factory.source_repo = source_repository
+        fetch_spec_factory.target_repo = result_repo
+        if stacked or (len(result_repo._fallback_repositories) != 0):
+            target_repo_kind = fetch.TargetRepoKinds.STACKED
+        elif is_new_repo:
+            target_repo_kind = fetch.TargetRepoKinds.EMPTY
+        else:
+            target_repo_kind = fetch.TargetRepoKinds.PREEXISTING
+        fetch_spec_factory.target_repo_kind = target_repo_kind
+        if source_repository is not None:
+            fetch_spec = fetch_spec_factory.make_fetch_spec()
+            result_repo.fetch(source_repository, fetch_spec=fetch_spec)
+
+        if source_branch is None:
+            # this is for sprouting a controldir without a branch; is that
+            # actually useful?
+            # Not especially, but it's part of the contract.
+            result_branch = result.create_branch()
+        else:
+            result_branch = source_branch.sprout(result,
+                revision_id=revision_id, repository_policy=repository_policy,
+                repository=result_repo)
+        mutter("created new branch %r" % (result_branch,))
+
+        # Create/update the result working tree
+        if (create_tree_if_local and
+            isinstance(target_transport, local.LocalTransport) and
+            (result_repo is None or result_repo.make_working_trees())):
+            wt = result.create_workingtree(accelerator_tree=accelerator_tree,
+                hardlink=hardlink, from_branch=result_branch)
+            wt.lock_write()
+            try:
+                if wt.path2id('') is None:
+                    try:
+                        wt.set_root_id(self.open_workingtree.get_root_id())
+                    except errors.NoWorkingTree:
+                        pass
+            finally:
+                wt.unlock()
+        else:
+            wt = None
+        if recurse == 'down':
+            basis = None
+            if wt is not None:
+                basis = wt.basis_tree()
+            elif result_branch is not None:
+                basis = result_branch.basis_tree()
+            elif source_branch is not None:
+                basis = source_branch.basis_tree()
+            if basis is not None:
+                add_cleanup(basis.lock_read().unlock)
+                subtrees = basis.iter_references()
+            else:
+                subtrees = []
+            for path, file_id in subtrees:
+                target = urlutils.join(url, urlutils.escape(path))
+                sublocation = source_branch.reference_parent(file_id, path)
+                sublocation.bzrdir.sprout(target,
+                    basis.get_reference_revision(file_id, path),
+                    force_new_repo=force_new_repo, recurse=recurse,
+                    stacked=stacked)
+        return result
+
+    @deprecated_method(deprecated_in((2, 3, 0)))
+    def generate_backup_name(self, base):
+        return self._available_backup_name(base)
+
+    def _available_backup_name(self, base):
+        """Find a non-existing backup file name based on base.
+
+        See bzrlib.osutils.available_backup_name about race conditions.
+        """
+        return osutils.available_backup_name(base, self.root_transport.has)
 
     def backup_bzrdir(self):
         """Backup this bzr control directory.
 
         :return: Tuple with old path name and new path name
         """
-        def name_gen(base='backup.bzr'):
-            counter = 1
-            name = "%s.~%d~" % (base, counter)
-            while self.root_transport.has(name):
-                counter += 1
-                name = "%s.~%d~" % (base, counter)
-            return name
 
-        backup_dir=name_gen()
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            # FIXME: bug 300001 -- the backup fails if the backup directory
-            # already exists, but it should instead either remove it or make
-            # a new backup directory.
-            #
             old_path = self.root_transport.abspath('.bzr')
+            backup_dir = self._available_backup_name('backup.bzr')
             new_path = self.root_transport.abspath(backup_dir)
-            ui.ui_factory.note('making backup of %s\n  to %s' % (old_path, new_path,))
+            ui.ui_factory.note(gettext('making backup of {0}\n  to {1}').format(
+                urlutils.unescape_for_display(old_path, 'utf-8'),
+                urlutils.unescape_for_display(new_path, 'utf-8')))
             self.root_transport.copy_tree('.bzr', backup_dir)
             return (old_path, new_path)
         finally:
@@ -658,8 +497,8 @@
             try:
                 to_path = '.bzr.retired.%d' % i
                 self.root_transport.rename('.bzr', to_path)
-                note("renamed %s to %s"
-                    % (self.root_transport.abspath('.bzr'), to_path))
+                note(gettext("renamed {0} to {1}").format(
+                    self.root_transport.abspath('.bzr'), to_path))
                 return
             except (errors.TransportError, IOError, errors.PathError):
                 i += 1
@@ -668,21 +507,6 @@
                 else:
                     pass
 
-    def destroy_workingtree(self):
-        """Destroy the working tree at this BzrDir.
-
-        Formats that do not support this may raise UnsupportedOperation.
-        """
-        raise NotImplementedError(self.destroy_workingtree)
-
-    def destroy_workingtree_metadata(self):
-        """Destroy the control files for the working tree at this BzrDir.
-
-        The contents of working tree files are not affected.
-        Formats that do not support this may raise UnsupportedOperation.
-        """
-        raise NotImplementedError(self.destroy_workingtree_metadata)
-
     def _find_containing(self, evaluate):
         """Find something in a containing control directory.
 
@@ -707,7 +531,7 @@
                 return None
             # find the next containing bzrdir
             try:
-                found_bzrdir = BzrDir.open_containing_from_transport(
+                found_bzrdir = self.open_containing_from_transport(
                     next_transport)[0]
             except errors.NotBranchError:
                 return None
@@ -737,33 +561,6 @@
             raise errors.NoRepositoryPresent(self)
         return found_repo
 
-    def get_branch_reference(self, name=None):
-        """Return the referenced URL for the branch in this bzrdir.
-
-        :param name: Optional colocated branch name
-        :raises NotBranchError: If there is no Branch.
-        :raises NoColocatedBranchSupport: If a branch name was specified
-            but colocated branches are not supported.
-        :return: The URL the branch in this bzrdir references if it is a
-            reference branch, or None for regular branches.
-        """
-        if name is not None:
-            raise errors.NoColocatedBranchSupport(self)
-        return None
-
-    def get_branch_transport(self, branch_format, name=None):
-        """Get the transport for use by branch format in this BzrDir.
-
-        Note that bzr dirs that do not support format strings will raise
-        IncompatibleFormat if the branch format they are given has
-        a format string, and vice versa.
-
-        If branch_format is None, the transport is returned with no
-        checking. If it is not None, then the returned transport is
-        guaranteed to point to an existing directory ready for use.
-        """
-        raise NotImplementedError(self.get_branch_transport)
-
     def _find_creation_modes(self):
         """Determine the appropriate modes for files and directories.
 
@@ -808,32 +605,6 @@
             self._find_creation_modes()
         return self._dir_mode
 
-    def get_repository_transport(self, repository_format):
-        """Get the transport for use by repository format in this BzrDir.
-
-        Note that bzr dirs that do not support format strings will raise
-        IncompatibleFormat if the repository format they are given has
-        a format string, and vice versa.
-
-        If repository_format is None, the transport is returned with no
-        checking. If it is not None, then the returned transport is
-        guaranteed to point to an existing directory ready for use.
-        """
-        raise NotImplementedError(self.get_repository_transport)
-
-    def get_workingtree_transport(self, tree_format):
-        """Get the transport for use by workingtree format in this BzrDir.
-
-        Note that bzr dirs that do not support format strings will raise
-        IncompatibleFormat if the workingtree format they are given has a
-        format string, and vice versa.
-
-        If workingtree_format is None, the transport is returned with no
-        checking. If it is not None, then the returned transport is
-        guaranteed to point to an existing directory ready for use.
-        """
-        raise NotImplementedError(self.get_workingtree_transport)
-
     def get_config(self):
         """Get configuration for this BzrDir."""
         return config.BzrDirConfig(self)
@@ -857,11 +628,11 @@
         self.transport = _transport.clone('.bzr')
         self.root_transport = _transport
         self._mode_check_done = False
-        
+
     @property 
     def user_transport(self):
         return self.root_transport
-        
+
     @property
     def control_transport(self):
         return self.transport
@@ -873,9 +644,7 @@
 
         This is true IF and ONLY IF the filename is part of the namespace reserved
         for bzr control dirs. Currently this is the '.bzr' directory in the root
-        of the root_transport. it is expected that plugins will need to extend
-        this in the future - for instance to make bzr talk with svn working
-        trees.
+        of the root_transport. 
         """
         # this might be better on the BzrDirFormat class because it refers to
         # all the possible bzrdir disk formats.
@@ -885,245 +654,6 @@
         # add new tests for it to the appropriate place.
         return filename == '.bzr' or filename.startswith('.bzr/')
 
-    def needs_format_conversion(self, format=None):
-        """Return true if this bzrdir needs convert_format run on it.
-
-        For instance, if the repository format is out of date but the
-        branch and working tree are not, this should return True.
-
-        :param format: Optional parameter indicating a specific desired
-                       format we plan to arrive at.
-        """
-        raise NotImplementedError(self.needs_format_conversion)
-
-    @staticmethod
-    def open_unsupported(base):
-        """Open a branch which is not supported."""
-        return BzrDir.open(base, _unsupported=True)
-
-    @staticmethod
-    def open(base, _unsupported=False, possible_transports=None):
-        """Open an existing bzrdir, rooted at 'base' (url).
-
-        :param _unsupported: a private parameter to the BzrDir class.
-        """
-        t = get_transport(base, possible_transports=possible_transports)
-        return BzrDir.open_from_transport(t, _unsupported=_unsupported)
-
-    @staticmethod
-    def open_from_transport(transport, _unsupported=False,
-                            _server_formats=True):
-        """Open a bzrdir within a particular directory.
-
-        :param transport: Transport containing the bzrdir.
-        :param _unsupported: private.
-        """
-        for hook in BzrDir.hooks['pre_open']:
-            hook(transport)
-        # Keep initial base since 'transport' may be modified while following
-        # the redirections.
-        base = transport.base
-        def find_format(transport):
-            return transport, BzrDirFormat.find_format(
-                transport, _server_formats=_server_formats)
-
-        def redirected(transport, e, redirection_notice):
-            redirected_transport = transport._redirected_to(e.source, e.target)
-            if redirected_transport is None:
-                raise errors.NotBranchError(base)
-            note('%s is%s redirected to %s',
-                 transport.base, e.permanently, redirected_transport.base)
-            return redirected_transport
-
-        try:
-            transport, format = do_catching_redirections(find_format,
-                                                         transport,
-                                                         redirected)
-        except errors.TooManyRedirections:
-            raise errors.NotBranchError(base)
-
-        BzrDir._check_supported(format, _unsupported)
-        return format.open(transport, _found=True)
-
-    def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
-        """Open the branch object at this BzrDir if one is present.
-
-        If unsupported is True, then no longer supported branch formats can
-        still be opened.
-
-        TODO: static convenience version of this?
-        """
-        raise NotImplementedError(self.open_branch)
-
-    @staticmethod
-    def open_containing(url, possible_transports=None):
-        """Open an existing branch which contains url.
-
-        :param url: url to search from.
-        See open_containing_from_transport for more detail.
-        """
-        transport = get_transport(url, possible_transports)
-        return BzrDir.open_containing_from_transport(transport)
-
-    @staticmethod
-    def open_containing_from_transport(a_transport):
-        """Open an existing branch which contains a_transport.base.
-
-        This probes for a branch at a_transport, and searches upwards from there.
-
-        Basically we keep looking up until we find the control directory or
-        run into the root.  If there isn't one, raises NotBranchError.
-        If there is one and it is either an unrecognised format or an unsupported
-        format, UnknownFormatError or UnsupportedFormatError are raised.
-        If there is one, it is returned, along with the unused portion of url.
-
-        :return: The BzrDir that contains the path, and a Unicode path
-                for the rest of the URL.
-        """
-        # this gets the normalised url back. I.e. '.' -> the full path.
-        url = a_transport.base
-        while True:
-            try:
-                result = BzrDir.open_from_transport(a_transport)
-                return result, urlutils.unescape(a_transport.relpath(url))
-            except errors.NotBranchError, e:
-                pass
-            try:
-                new_t = a_transport.clone('..')
-            except errors.InvalidURLJoin:
-                # reached the root, whatever that may be
-                raise errors.NotBranchError(path=url)
-            if new_t.base == a_transport.base:
-                # reached the root, whatever that may be
-                raise errors.NotBranchError(path=url)
-            a_transport = new_t
-
-    def _get_tree_branch(self, name=None):
-        """Return the branch and tree, if any, for this bzrdir.
-
-        :param name: Name of colocated branch to open.
-
-        Return None for tree if not present or inaccessible.
-        Raise NotBranchError if no branch is present.
-        :return: (tree, branch)
-        """
-        try:
-            tree = self.open_workingtree()
-        except (errors.NoWorkingTree, errors.NotLocalUrl):
-            tree = None
-            branch = self.open_branch(name=name)
-        else:
-            if name is not None:
-                branch = self.open_branch(name=name)
-            else:
-                branch = tree.branch
-        return tree, branch
-
-    @classmethod
-    def open_tree_or_branch(klass, location):
-        """Return the branch and working tree at a location.
-
-        If there is no tree at the location, tree will be None.
-        If there is no branch at the location, an exception will be
-        raised
-        :return: (tree, branch)
-        """
-        bzrdir = klass.open(location)
-        return bzrdir._get_tree_branch()
-
-    @classmethod
-    def open_containing_tree_or_branch(klass, location):
-        """Return the branch and working tree contained by a location.
-
-        Returns (tree, branch, relpath).
-        If there is no tree at containing the location, tree will be None.
-        If there is no branch containing the location, an exception will be
-        raised
-        relpath is the portion of the path that is contained by the branch.
-        """
-        bzrdir, relpath = klass.open_containing(location)
-        tree, branch = bzrdir._get_tree_branch()
-        return tree, branch, relpath
-
-    @classmethod
-    def open_containing_tree_branch_or_repository(klass, location):
-        """Return the working tree, branch and repo contained by a location.
-
-        Returns (tree, branch, repository, relpath).
-        If there is no tree containing the location, tree will be None.
-        If there is no branch containing the location, branch will be None.
-        If there is no repository containing the location, repository will be
-        None.
-        relpath is the portion of the path that is contained by the innermost
-        BzrDir.
-
-        If no tree, branch or repository is found, a NotBranchError is raised.
-        """
-        bzrdir, relpath = klass.open_containing(location)
-        try:
-            tree, branch = bzrdir._get_tree_branch()
-        except errors.NotBranchError:
-            try:
-                repo = bzrdir.find_repository()
-                return None, None, repo, relpath
-            except (errors.NoRepositoryPresent):
-                raise errors.NotBranchError(location)
-        return tree, branch, branch.repository, relpath
-
-    def open_repository(self, _unsupported=False):
-        """Open the repository object at this BzrDir if one is present.
-
-        This will not follow the Branch object pointer - it's strictly a direct
-        open facility. Most client code should use open_branch().repository to
-        get at a repository.
-
-        :param _unsupported: a private parameter, not part of the api.
-        TODO: static convenience version of this?
-        """
-        raise NotImplementedError(self.open_repository)
-
-    def open_workingtree(self, _unsupported=False,
-                         recommend_upgrade=True, from_branch=None):
-        """Open the workingtree object at this BzrDir if one is present.
-
-        :param recommend_upgrade: Optional keyword parameter, when True (the
-            default), emit through the ui module a recommendation that the user
-            upgrade the working tree when the workingtree being opened is old
-            (but still fully supported).
-        :param from_branch: override bzrdir branch (for lightweight checkouts)
-        """
-        raise NotImplementedError(self.open_workingtree)
-
-    def has_branch(self, name=None):
-        """Tell if this bzrdir contains a branch.
-
-        Note: if you're going to open the branch, you should just go ahead
-        and try, and not ask permission first.  (This method just opens the
-        branch and discards it, and that's somewhat expensive.)
-        """
-        try:
-            self.open_branch(name)
-            return True
-        except errors.NotBranchError:
-            return False
-
-    def has_workingtree(self):
-        """Tell if this bzrdir contains a working tree.
-
-        This will still raise an exception if the bzrdir has a workingtree that
-        is remote & inaccessible.
-
-        Note: if you're going to open the working tree, you should just go ahead
-        and try, and not ask permission first.  (This method just opens the
-        workingtree and discards it, and that's somewhat expensive.)
-        """
-        try:
-            self.open_workingtree(recommend_upgrade=False)
-            return True
-        except errors.NoWorkingTree:
-            return False
-
     def _cloning_metadir(self):
         """Produce a metadir suitable for cloning with.
 
@@ -1169,7 +699,7 @@
 
         :require_stacking: If True, non-stackable formats will be upgraded
             to similar stackable formats.
-        :returns: a BzrDirFormat with all component formats either set
+        :returns: a ControlDirFormat with all component formats either set
             appropriately or set to None if that component should not be
             created.
         """
@@ -1187,515 +717,59 @@
             format.require_stacking()
         return format
 
-    def checkout_metadir(self):
-        return self.cloning_metadir()
-
-    def sprout(self, url, revision_id=None, force_new_repo=False,
-               recurse='down', possible_transports=None,
-               accelerator_tree=None, hardlink=False, stacked=False,
-               source_branch=None, create_tree_if_local=True):
-        """Create a copy of this bzrdir prepared for use as a new line of
-        development.
-
-        If url's last component does not exist, it will be created.
-
-        Attributes related to the identity of the source branch like
-        branch nickname will be cleaned, a working tree is created
-        whether one existed before or not; and a local branch is always
-        created.
-
-        if revision_id is not None, then the clone operation may tune
-            itself to download less data.
-        :param accelerator_tree: A tree which can be used for retrieving file
-            contents more quickly than the revision tree, i.e. a workingtree.
-            The revision tree will be used for cases where accelerator_tree's
-            content is different.
-        :param hardlink: If true, hard-link files from accelerator_tree,
-            where possible.
-        :param stacked: If true, create a stacked branch referring to the
-            location of this control directory.
-        :param create_tree_if_local: If true, a working-tree will be created
-            when working locally.
-        """
-        target_transport = get_transport(url, possible_transports)
-        target_transport.ensure_base()
-        cloning_format = self.cloning_metadir(stacked)
-        # Create/update the result branch
-        result = cloning_format.initialize_on_transport(target_transport)
-        # if a stacked branch wasn't requested, we don't create one
-        # even if the origin was stacked
-        stacked_branch_url = None
-        if source_branch is not None:
-            if stacked:
-                stacked_branch_url = self.root_transport.base
-            source_repository = source_branch.repository
-        else:
-            try:
-                source_branch = self.open_branch()
-                source_repository = source_branch.repository
-                if stacked:
-                    stacked_branch_url = self.root_transport.base
-            except errors.NotBranchError:
-                source_branch = None
-                try:
-                    source_repository = self.open_repository()
-                except errors.NoRepositoryPresent:
-                    source_repository = None
-        repository_policy = result.determine_repository_policy(
-            force_new_repo, stacked_branch_url, require_stacking=stacked)
-        result_repo, is_new_repo = repository_policy.acquire_repository()
-        is_stacked = stacked or (len(result_repo._fallback_repositories) != 0)
-        if is_new_repo and revision_id is not None and not is_stacked:
-            fetch_spec = graph.PendingAncestryResult(
-                [revision_id], source_repository)
-        else:
-            fetch_spec = None
-        if source_repository is not None:
-            # Fetch while stacked to prevent unstacked fetch from
-            # Branch.sprout.
-            if fetch_spec is None:
-                result_repo.fetch(source_repository, revision_id=revision_id)
-            else:
-                result_repo.fetch(source_repository, fetch_spec=fetch_spec)
-
-        if source_branch is None:
-            # this is for sprouting a bzrdir without a branch; is that
-            # actually useful?
-            # Not especially, but it's part of the contract.
-            result_branch = result.create_branch()
-        else:
-            result_branch = source_branch.sprout(result,
-                revision_id=revision_id, repository_policy=repository_policy)
-        mutter("created new branch %r" % (result_branch,))
-
-        # Create/update the result working tree
-        if (create_tree_if_local and
-            isinstance(target_transport, local.LocalTransport) and
-            (result_repo is None or result_repo.make_working_trees())):
-            wt = result.create_workingtree(accelerator_tree=accelerator_tree,
-                hardlink=hardlink)
-            wt.lock_write()
-            try:
-                if wt.path2id('') is None:
-                    try:
-                        wt.set_root_id(self.open_workingtree.get_root_id())
-                    except errors.NoWorkingTree:
-                        pass
-            finally:
-                wt.unlock()
-        else:
-            wt = None
-        if recurse == 'down':
-            if wt is not None:
-                basis = wt.basis_tree()
-                basis.lock_read()
-                subtrees = basis.iter_references()
-            elif result_branch is not None:
-                basis = result_branch.basis_tree()
-                basis.lock_read()
-                subtrees = basis.iter_references()
-            elif source_branch is not None:
-                basis = source_branch.basis_tree()
-                basis.lock_read()
-                subtrees = basis.iter_references()
-            else:
-                subtrees = []
-                basis = None
-            try:
-                for path, file_id in subtrees:
-                    target = urlutils.join(url, urlutils.escape(path))
-                    sublocation = source_branch.reference_parent(file_id, path)
-                    sublocation.bzrdir.sprout(target,
-                        basis.get_reference_revision(file_id, path),
-                        force_new_repo=force_new_repo, recurse=recurse,
-                        stacked=stacked)
-            finally:
-                if basis is not None:
-                    basis.unlock()
-        return result
-
-    def push_branch(self, source, revision_id=None, overwrite=False, 
-        remember=False, create_prefix=False):
-        """Push the source branch into this BzrDir."""
-        br_to = None
-        # If we can open a branch, use its direct repository, otherwise see
-        # if there is a repository without a branch.
-        try:
-            br_to = self.open_branch()
-        except errors.NotBranchError:
-            # Didn't find a branch, can we find a repository?
-            repository_to = self.find_repository()
-        else:
-            # Found a branch, so we must have found a repository
-            repository_to = br_to.repository
-
-        push_result = PushResult()
-        push_result.source_branch = source
-        if br_to is None:
-            # We have a repository but no branch, copy the revisions, and then
-            # create a branch.
-            repository_to.fetch(source.repository, revision_id=revision_id)
-            br_to = source.clone(self, revision_id=revision_id)
-            if source.get_push_location() is None or remember:
-                source.set_push_location(br_to.base)
-            push_result.stacked_on = None
-            push_result.branch_push_result = None
-            push_result.old_revno = None
-            push_result.old_revid = _mod_revision.NULL_REVISION
-            push_result.target_branch = br_to
-            push_result.master_branch = None
-            push_result.workingtree_updated = False
-        else:
-            # We have successfully opened the branch, remember if necessary:
-            if source.get_push_location() is None or remember:
-                source.set_push_location(br_to.base)
-            try:
-                tree_to = self.open_workingtree()
-            except errors.NotLocalUrl:
-                push_result.branch_push_result = source.push(br_to, 
-                    overwrite, stop_revision=revision_id)
-                push_result.workingtree_updated = False
-            except errors.NoWorkingTree:
-                push_result.branch_push_result = source.push(br_to,
-                    overwrite, stop_revision=revision_id)
-                push_result.workingtree_updated = None # Not applicable
-            else:
-                tree_to.lock_write()
-                try:
-                    push_result.branch_push_result = source.push(
-                        tree_to.branch, overwrite, stop_revision=revision_id)
-                    tree_to.update()
-                finally:
-                    tree_to.unlock()
-                push_result.workingtree_updated = True
-            push_result.old_revno = push_result.branch_push_result.old_revno
-            push_result.old_revid = push_result.branch_push_result.old_revid
-            push_result.target_branch = \
-                push_result.branch_push_result.target_branch
-        return push_result
-
-
-class BzrDirHooks(hooks.Hooks):
-    """Hooks for BzrDir operations."""
-
-    def __init__(self):
-        """Create the default hooks."""
-        hooks.Hooks.__init__(self)
-        self.create_hook(hooks.HookPoint('pre_open',
-            "Invoked before attempting to open a BzrDir with the transport "
-            "that the open will use.", (1, 14), None))
-        self.create_hook(hooks.HookPoint('post_repo_init',
-            "Invoked after a repository has been initialized. "
-            "post_repo_init is called with a "
-            "bzrlib.bzrdir.RepoInitHookParams.",
-            (2, 2), None))
-
-# install the default hooks
-BzrDir.hooks = BzrDirHooks()
-
-
-class RepoInitHookParams(object):
-    """Object holding parameters passed to *_repo_init hooks.
-
-    There are 4 fields that hooks may wish to access:
-
-    :ivar repository: Repository created
-    :ivar format: Repository format
-    :ivar bzrdir: The bzrdir for the repository
-    :ivar shared: The repository is shared
-    """
-
-    def __init__(self, repository, format, a_bzrdir, shared):
-        """Create a group of RepoInitHook parameters.
-
-        :param repository: Repository created
-        :param format: Repository format
-        :param bzrdir: The bzrdir for the repository
-        :param shared: The repository is shared
-        """
-        self.repository = repository
-        self.format = format
-        self.bzrdir = a_bzrdir
-        self.shared = shared
-
-    def __eq__(self, other):
-        return self.__dict__ == other.__dict__
-
-    def __repr__(self):
-        if self.repository:
-            return "<%s for %s>" % (self.__class__.__name__,
-                self.repository)
-        else:
-            return "<%s for %s>" % (self.__class__.__name__,
-                self.bzrdir)
-
-
-class BzrDirPreSplitOut(BzrDir):
-    """A common class for the all-in-one formats."""
-
-    def __init__(self, _transport, _format):
-        """See BzrDir.__init__."""
-        super(BzrDirPreSplitOut, self).__init__(_transport, _format)
-        self._control_files = lockable_files.LockableFiles(
-                                            self.get_branch_transport(None),
-                                            self._format._lock_file_name,
-                                            self._format._lock_class)
-
-    def break_lock(self):
-        """Pre-splitout bzrdirs do not suffer from stale locks."""
-        raise NotImplementedError(self.break_lock)
-
-    def cloning_metadir(self, require_stacking=False):
-        """Produce a metadir suitable for cloning with."""
-        if require_stacking:
-            return format_registry.make_bzrdir('1.6')
-        return self._format.__class__()
-
-    def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
-        """See BzrDir.clone().
-
-        force_new_repo has no effect, since this family of formats always
-        require a new repository.
-        preserve_stacking has no effect, since no source branch using this
-        family of formats can be stacked, so there is no stacking to preserve.
-        """
-        self._make_tail(url)
-        result = self._format._initialize_for_clone(url)
-        self.open_repository().clone(result, revision_id=revision_id)
-        from_branch = self.open_branch()
-        from_branch.clone(result, revision_id=revision_id)
-        try:
-            tree = self.open_workingtree()
-        except errors.NotLocalUrl:
-            # make a new one, this format always has to have one.
-            result._init_workingtree()
-        else:
-            tree.clone(result)
-        return result
-
-    def create_branch(self, name=None):
-        """See BzrDir.create_branch."""
-        return self._format.get_branch_format().initialize(self, name=name)
-
-    def destroy_branch(self, name=None):
-        """See BzrDir.destroy_branch."""
-        raise errors.UnsupportedOperation(self.destroy_branch, self)
-
-    def create_repository(self, shared=False):
-        """See BzrDir.create_repository."""
-        if shared:
-            raise errors.IncompatibleFormat('shared repository', self._format)
-        return self.open_repository()
-
-    def destroy_repository(self):
-        """See BzrDir.destroy_repository."""
-        raise errors.UnsupportedOperation(self.destroy_repository, self)
-
-    def create_workingtree(self, revision_id=None, from_branch=None,
-                           accelerator_tree=None, hardlink=False):
-        """See BzrDir.create_workingtree."""
-        # The workingtree is sometimes created when the bzrdir is created,
-        # but not when cloning.
-
-        # this looks buggy but is not -really-
-        # because this format creates the workingtree when the bzrdir is
-        # created
-        # clone and sprout will have set the revision_id
-        # and that will have set it for us, its only
-        # specific uses of create_workingtree in isolation
-        # that can do wonky stuff here, and that only
-        # happens for creating checkouts, which cannot be
-        # done on this format anyway. So - acceptable wart.
-        if hardlink:
-            warning("can't support hardlinked working trees in %r"
-                % (self,))
-        try:
-            result = self.open_workingtree(recommend_upgrade=False)
-        except errors.NoSuchFile:
-            result = self._init_workingtree()
-        if revision_id is not None:
-            if revision_id == _mod_revision.NULL_REVISION:
-                result.set_parent_ids([])
-            else:
-                result.set_parent_ids([revision_id])
-        return result
-
-    def _init_workingtree(self):
-        from bzrlib.workingtree import WorkingTreeFormat2
-        try:
-            return WorkingTreeFormat2().initialize(self)
-        except errors.NotLocalUrl:
-            # Even though we can't access the working tree, we need to
-            # create its control files.
-            return WorkingTreeFormat2()._stub_initialize_on_transport(
-                self.transport, self._control_files._file_mode)
-
-    def destroy_workingtree(self):
-        """See BzrDir.destroy_workingtree."""
-        raise errors.UnsupportedOperation(self.destroy_workingtree, self)
-
-    def destroy_workingtree_metadata(self):
-        """See BzrDir.destroy_workingtree_metadata."""
-        raise errors.UnsupportedOperation(self.destroy_workingtree_metadata,
-                                          self)
-
     def get_branch_transport(self, branch_format, name=None):
-        """See BzrDir.get_branch_transport()."""
-        if name is not None:
-            raise errors.NoColocatedBranchSupport(self)
-        if branch_format is None:
-            return self.transport
-        try:
-            branch_format.get_format_string()
-        except NotImplementedError:
-            return self.transport
-        raise errors.IncompatibleFormat(branch_format, self._format)
+        """Get the transport for use by branch format in this BzrDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the branch format they are given has
+        a format string, and vice versa.
+
+        If branch_format is None, the transport is returned with no
+        checking. If it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_branch_transport)
 
     def get_repository_transport(self, repository_format):
-        """See BzrDir.get_repository_transport()."""
-        if repository_format is None:
-            return self.transport
-        try:
-            repository_format.get_format_string()
-        except NotImplementedError:
-            return self.transport
-        raise errors.IncompatibleFormat(repository_format, self._format)
-
-    def get_workingtree_transport(self, workingtree_format):
-        """See BzrDir.get_workingtree_transport()."""
-        if workingtree_format is None:
-            return self.transport
-        try:
-            workingtree_format.get_format_string()
-        except NotImplementedError:
-            return self.transport
-        raise errors.IncompatibleFormat(workingtree_format, self._format)
-
-    def needs_format_conversion(self, format=None):
-        """See BzrDir.needs_format_conversion()."""
-        # if the format is not the same as the system default,
-        # an upgrade is needed.
-        if format is None:
-            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
-                % 'needs_format_conversion(format=None)')
-            format = BzrDirFormat.get_default_format()
-        return not isinstance(self._format, format.__class__)
-
-    def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
-        """See BzrDir.open_branch."""
-        from bzrlib.branch import BzrBranchFormat4
-        format = BzrBranchFormat4()
-        self._check_supported(format, unsupported)
-        return format.open(self, name, _found=True)
-
-    def sprout(self, url, revision_id=None, force_new_repo=False,
-               possible_transports=None, accelerator_tree=None,
-               hardlink=False, stacked=False, create_tree_if_local=True,
-               source_branch=None):
-        """See BzrDir.sprout()."""
-        if source_branch is not None:
-            my_branch = self.open_branch()
-            if source_branch.base != my_branch.base:
-                raise AssertionError(
-                    "source branch %r is not within %r with branch %r" %
-                    (source_branch, self, my_branch))
-        if stacked:
-            raise errors.UnstackableBranchFormat(
-                self._format, self.root_transport.base)
-        if not create_tree_if_local:
-            raise errors.MustHaveWorkingTree(
-                self._format, self.root_transport.base)
-        from bzrlib.workingtree import WorkingTreeFormat2
-        self._make_tail(url)
-        result = self._format._initialize_for_clone(url)
-        try:
-            self.open_repository().clone(result, revision_id=revision_id)
-        except errors.NoRepositoryPresent:
-            pass
-        try:
-            self.open_branch().sprout(result, revision_id=revision_id)
-        except errors.NotBranchError:
-            pass
-
-        # we always want a working tree
-        WorkingTreeFormat2().initialize(result,
-                                        accelerator_tree=accelerator_tree,
-                                        hardlink=hardlink)
-        return result
-
-
-class BzrDir4(BzrDirPreSplitOut):
-    """A .bzr version 4 control object.
-
-    This is a deprecated format and may be removed after sept 2006.
-    """
-
-    def create_repository(self, shared=False):
-        """See BzrDir.create_repository."""
-        return self._format.repository_format.initialize(self, shared)
-
-    def needs_format_conversion(self, format=None):
-        """Format 4 dirs are always in need of conversion."""
-        if format is None:
-            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
-                % 'needs_format_conversion(format=None)')
-        return True
-
-    def open_repository(self):
-        """See BzrDir.open_repository."""
-        from bzrlib.repofmt.weaverepo import RepositoryFormat4
-        return RepositoryFormat4().open(self, _found=True)
-
-
-class BzrDir5(BzrDirPreSplitOut):
-    """A .bzr version 5 control object.
-
-    This is a deprecated format and may be removed after sept 2006.
-    """
-
-    def has_workingtree(self):
-        """See BzrDir.has_workingtree."""
-        return True
-    
-    def open_repository(self):
-        """See BzrDir.open_repository."""
-        from bzrlib.repofmt.weaverepo import RepositoryFormat5
-        return RepositoryFormat5().open(self, _found=True)
-
-    def open_workingtree(self, _unsupported=False,
-            recommend_upgrade=True):
-        """See BzrDir.create_workingtree."""
-        from bzrlib.workingtree import WorkingTreeFormat2
-        wt_format = WorkingTreeFormat2()
-        # we don't warn here about upgrades; that ought to be handled for the
-        # bzrdir as a whole
-        return wt_format.open(self, _found=True)
-
-
-class BzrDir6(BzrDirPreSplitOut):
-    """A .bzr version 6 control object.
-
-    This is a deprecated format and may be removed after sept 2006.
-    """
-
-    def has_workingtree(self):
-        """See BzrDir.has_workingtree."""
-        return True
-    
-    def open_repository(self):
-        """See BzrDir.open_repository."""
-        from bzrlib.repofmt.weaverepo import RepositoryFormat6
-        return RepositoryFormat6().open(self, _found=True)
-
-    def open_workingtree(self, _unsupported=False,
-        recommend_upgrade=True):
-        """See BzrDir.create_workingtree."""
-        # we don't warn here about upgrades; that ought to be handled for the
-        # bzrdir as a whole
-        from bzrlib.workingtree import WorkingTreeFormat2
-        return WorkingTreeFormat2().open(self, _found=True)
+        """Get the transport for use by repository format in this BzrDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the repository format they are given has
+        a format string, and vice versa.
+
+        If repository_format is None, the transport is returned with no
+        checking. If it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_repository_transport)
+
+    def get_workingtree_transport(self, tree_format):
+        """Get the transport for use by workingtree format in this BzrDir.
+
+        Note that bzr dirs that do not support format strings will raise
+        IncompatibleFormat if the workingtree format they are given has a
+        format string, and vice versa.
+
+        If workingtree_format is None, the transport is returned with no
+        checking. If it is not None, then the returned transport is
+        guaranteed to point to an existing directory ready for use.
+        """
+        raise NotImplementedError(self.get_workingtree_transport)
+
+    @classmethod
+    def create(cls, base, format=None, possible_transports=None):
+        """Create a new BzrDir at the url 'base'.
+
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
+        :param possible_transports: If supplied, a list of transports that
+            can be reused to share a remote connection.
+        """
+        if cls is not BzrDir:
+            raise AssertionError("BzrDir.create always creates the "
+                "default format, not one of %r" % cls)
+        return controldir.ControlDir.create(base, format=format,
+                possible_transports=possible_transports)
 
 
 class BzrDirMeta1(BzrDir):
@@ -1711,9 +785,12 @@
         """See BzrDir.can_convert_format()."""
         return True
 
-    def create_branch(self, name=None):
+    def create_branch(self, name=None, repository=None,
+            append_revisions_only=None):
         """See BzrDir.create_branch."""
-        return self._format.get_branch_format().initialize(self, name=name)
+        return self._format.get_branch_format().initialize(self, name=name,
+                repository=repository,
+                append_revisions_only=append_revisions_only)
 
     def destroy_branch(self, name=None):
         """See BzrDir.create_branch."""
@@ -1741,7 +818,10 @@
         wt = self.open_workingtree(recommend_upgrade=False)
         repository = wt.branch.repository
         empty = repository.revision_tree(_mod_revision.NULL_REVISION)
-        wt.revert(old_tree=empty)
+        # We ignore the conflicts returned by wt.revert since we're about to
+        # delete the wt metadata anyway, all that should be left here are
+        # detritus. But see bug #634470 about subtree .bzr dirs.
+        conflicts = wt.revert(old_tree=empty)
         self.destroy_workingtree_metadata()
 
     def destroy_workingtree_metadata(self):
@@ -1816,27 +896,20 @@
     def has_workingtree(self):
         """Tell if this bzrdir contains a working tree.
 
-        This will still raise an exception if the bzrdir has a workingtree that
-        is remote & inaccessible.
-
         Note: if you're going to open the working tree, you should just go
         ahead and try, and not ask permission first.
         """
         from bzrlib.workingtree import WorkingTreeFormat
         try:
-            WorkingTreeFormat.find_format(self)
+            WorkingTreeFormat.find_format_string(self)
         except errors.NoWorkingTree:
             return False
         return True
 
-    def needs_format_conversion(self, format=None):
+    def needs_format_conversion(self, format):
         """See BzrDir.needs_format_conversion()."""
-        if format is None:
-            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
-                % 'needs_format_conversion(format=None)')
-        if format is None:
-            format = BzrDirFormat.get_default_format()
-        if not isinstance(self._format, format.__class__):
+        if (not isinstance(self._format, format.__class__) or
+            self._format.get_format_string() != format.get_format_string()):
             # it is not a meta dir format, conversion is needed.
             return True
         # we might want to push this down to the repository?
@@ -1866,7 +939,7 @@
                     ignore_fallbacks=False):
         """See BzrDir.open_branch."""
         format = self.find_branch_format(name=name)
-        self._check_supported(format, unsupported)
+        format.check_support_status(unsupported)
         return format.open(self, name=name,
             _found=True, ignore_fallbacks=ignore_fallbacks)
 
@@ -1874,7 +947,7 @@
         """See BzrDir.open_repository."""
         from bzrlib.repository import RepositoryFormat
         format = RepositoryFormat.find_format(self)
-        self._check_supported(format, unsupported)
+        format.check_support_status(unsupported)
         return format.open(self, _found=True)
 
     def open_workingtree(self, unsupported=False,
@@ -1882,8 +955,7 @@
         """See BzrDir.open_workingtree."""
         from bzrlib.workingtree import WorkingTreeFormat
         format = WorkingTreeFormat.find_format(self)
-        self._check_supported(format, unsupported,
-            recommend_upgrade,
+        format.check_support_status(unsupported, recommend_upgrade,
             basedir=self.root_transport.base)
         return format.open(self, _found=True)
 
@@ -1891,13 +963,200 @@
         return config.TransportConfig(self.transport, 'control.conf')
 
 
-class BzrDirFormat(object):
-    """An encapsulation of the initialization and open routines for a format.
-
-    Formats provide three things:
-     * An initialization routine,
-     * a format string,
-     * an open routine.
+class BzrDirMeta1Colo(BzrDirMeta1):
+    """BzrDirMeta1 with support for colocated branches.
+
+    This format is experimental, and will eventually be merged back into
+    BzrDirMeta1.
+    """
+
+    def __init__(self, _transport, _format):
+        super(BzrDirMeta1Colo, self).__init__(_transport, _format)
+        self.control_files = lockable_files.LockableFiles(_transport,
+            self._format._lock_file_name, self._format._lock_class)
+
+    def _get_branch_path(self, name):
+        """Obtain the branch path to use.
+
+        This uses the API specified branch name first, and then falls back to
+        the branch name specified in the URL. If neither of those is specified,
+        it uses the default branch.
+
+        :param name: Optional branch name to use
+        :return: Relative path to branch, branch name
+        """
+        if name is None:
+            name = self._get_selected_branch()
+        if name is None:
+            return 'branch', None
+        return urlutils.join('branches', name), name
+
+    def _read_branch_list(self):
+        """Read the branch list.
+
+        :return: List of utf-8 encoded branch names.
+        """
+        try:
+            f = self.control_transport.get('branch-list')
+        except errors.NoSuchFile:
+            return []
+
+        ret = []
+        try:
+            for name in f:
+                ret.append(name.rstrip("\n"))
+        finally:
+            f.close()
+        return ret
+
+    def _write_branch_list(self, branches):
+        """Write out the branch list.
+
+        :param branches: List of utf-8 branch names to write
+        """
+        self.transport.put_bytes('branch-list',
+            "".join([name+"\n" for name in branches]))
+
+    def destroy_branch(self, name=None):
+        """See BzrDir.create_branch."""
+        path, name = self._get_branch_path(name)
+        if name is not None:
+            self.control_files.lock_write()
+            try:
+                branches = self._read_branch_list()
+                try:
+                    branches.remove(name)
+                except ValueError:
+                    raise errors.NotBranchError(name)
+                self._write_branch_list(name)
+            finally:
+                self.control_files.unlock()
+        self.transport.delete_tree(path)
+
+    def list_branches(self):
+        """See ControlDir.list_branches."""
+        ret = []
+        # Default branch
+        try:
+            ret.append(self.open_branch())
+        except (errors.NotBranchError, errors.NoRepositoryPresent):
+            pass
+
+        # colocated branches
+        ret.extend([self.open_branch(name) for name in
+                    self._read_branch_list()])
+
+        return ret
+
+    def get_branch_transport(self, branch_format, name=None):
+        """See BzrDir.get_branch_transport()."""
+        path, name = self._get_branch_path(name)
+        # XXX: this shouldn't implicitly create the directory if it's just
+        # promising to get a transport -- mbp 20090727
+        if branch_format is None:
+            return self.transport.clone(path)
+        try:
+            branch_format.get_format_string()
+        except NotImplementedError:
+            raise errors.IncompatibleFormat(branch_format, self._format)
+        if name is not None:
+            try:
+                self.transport.mkdir('branches', mode=self._get_mkdir_mode())
+            except errors.FileExists:
+                pass
+            branches = self._read_branch_list()
+            if not name in branches:
+                self.control_files.lock_write()
+                try:
+                    branches = self._read_branch_list()
+                    branches.append(name)
+                    self._write_branch_list(branches)
+                finally:
+                    self.control_files.unlock()
+        try:
+            self.transport.mkdir(path, mode=self._get_mkdir_mode())
+        except errors.FileExists:
+            pass
+        return self.transport.clone(path)
+
+
+class BzrProber(controldir.Prober):
+    """Prober for formats that use a .bzr/ control directory."""
+
+    formats = registry.FormatRegistry(controldir.network_format_registry)
+    """The known .bzr formats."""
+
+    @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
+    def register_bzrdir_format(klass, format):
+        klass.formats.register(format.get_format_string(), format)
+
+    @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
+    def unregister_bzrdir_format(klass, format):
+        klass.formats.remove(format.get_format_string())
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return the .bzrdir style format present in a directory."""
+        try:
+            format_string = transport.get_bytes(".bzr/branch-format")
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base)
+        try:
+            return klass.formats.get(format_string)
+        except KeyError:
+            raise errors.UnknownFormatError(format=format_string, kind='bzrdir')
+
+    @classmethod
+    def known_formats(cls):
+        result = set()
+        for name, format in cls.formats.iteritems():
+            if callable(format):
+                format = format()
+            result.add(format)
+        return result
+
+
+controldir.ControlDirFormat.register_prober(BzrProber)
+
+
+class RemoteBzrProber(controldir.Prober):
+    """Prober for remote servers that provide a Bazaar smart server."""
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return a RemoteBzrDirFormat object if it looks possible."""
+        try:
+            medium = transport.get_smart_medium()
+        except (NotImplementedError, AttributeError,
+                errors.TransportNotPossible, errors.NoSmartMedium,
+                errors.SmartProtocolError):
+            # no smart server, so not a branch for this format type.
+            raise errors.NotBranchError(path=transport.base)
+        else:
+            # Decline to open it if the server doesn't support our required
+            # version (3) so that the VFS-based transport will do it.
+            if medium.should_probe():
+                try:
+                    server_version = medium.protocol_version()
+                except errors.SmartProtocolError:
+                    # Apparently there's no usable smart server there, even though
+                    # the medium supports the smart protocol.
+                    raise errors.NotBranchError(path=transport.base)
+                if server_version != '2':
+                    raise errors.NotBranchError(path=transport.base)
+            from bzrlib.remote import RemoteBzrDirFormat
+            return RemoteBzrDirFormat()
+
+    @classmethod
+    def known_formats(cls):
+        from bzrlib.remote import RemoteBzrDirFormat
+        return set([RemoteBzrDirFormat()])
+
+
+class BzrDirFormat(controldir.ControlDirFormat):
+    """ControlDirFormat base class for .bzr/ directories.
 
     Formats are placed in a dict by their format string for reference
     during bzrdir opening. These should be subclasses of BzrDirFormat
@@ -1906,103 +1165,17 @@
     Once a format is deprecated, just deprecate the initialize and open
     methods on the format class. Do not deprecate the object, as the
     object will be created every system load.
-
-    :cvar colocated_branches: Whether this formats supports colocated branches.
-    """
-
-    _default_format = None
-    """The default format used for new .bzr dirs."""
-
-    _formats = {}
-    """The known formats."""
-
-    _control_formats = []
-    """The registered control formats - .bzr, ....
-
-    This is a list of BzrDirFormat objects.
-    """
-
-    _control_server_formats = []
-    """The registered control server formats, e.g. RemoteBzrDirs.
-
-    This is a list of BzrDirFormat objects.
     """
 
     _lock_file_name = 'branch-lock'
 
-    colocated_branches = False
-    """Whether co-located branches are supported for this control dir format.
-    """
-
     # _lock_class must be set in subclasses to the lock type, typ.
     # TransportLock or LockDir
 
     @classmethod
-    def find_format(klass, transport, _server_formats=True):
-        """Return the format present at transport."""
-        if _server_formats:
-            formats = klass._control_server_formats + klass._control_formats
-        else:
-            formats = klass._control_formats
-        for format in formats:
-            try:
-                return format.probe_transport(transport)
-            except errors.NotBranchError:
-                # this format does not find a control dir here.
-                pass
-        raise errors.NotBranchError(path=transport.base)
-
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return the .bzrdir style format present in a directory."""
-        try:
-            format_string = transport.get_bytes(".bzr/branch-format")
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base)
-        try:
-            return klass._formats[format_string]
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string, kind='bzrdir')
-
-    @classmethod
-    def get_default_format(klass):
-        """Return the current default format."""
-        return klass._default_format
-
-    def get_format_string(self):
+    def get_format_string(cls):
         """Return the ASCII format string that identifies this format."""
-        raise NotImplementedError(self.get_format_string)
-
-    def get_format_description(self):
-        """Return the short description for this format."""
-        raise NotImplementedError(self.get_format_description)
-
-    def get_converter(self, format=None):
-        """Return the converter to use to convert bzrdirs needing converts.
-
-        This returns a bzrlib.bzrdir.Converter object.
-
-        This should return the best upgrader to step this format towards the
-        current default format. In the case of plugins we can/should provide
-        some means for them to extend the range of returnable converters.
-
-        :param format: Optional format to override the default format of the
-                       library.
-        """
-        raise NotImplementedError(self.get_converter)
-
-    def initialize(self, url, possible_transports=None):
-        """Create a bzr control dir at this url and return an opened copy.
-
-        While not deprecated, this method is very specific and its use will
-        lead to many round trips to setup a working environment. See
-        initialize_on_transport_ex for a [nearly] all-in-one method.
-
-        Subclasses should typically override initialize_on_transport
-        instead of this method.
-        """
-        return self.initialize_on_transport(get_transport(url,
-                                                          possible_transports))
+        raise NotImplementedError(cls.get_format_string)
 
     def initialize_on_transport(self, transport):
         """Initialize a new bzrdir in the base directory of a Transport."""
@@ -2018,6 +1191,7 @@
             # metadir1
             if type(self) != BzrDirMetaFormat1:
                 return self._initialize_on_transport_vfs(transport)
+            from bzrlib.remote import RemoteBzrDirFormat
             remote_format = RemoteBzrDirFormat()
             self._supply_sub_formats_to(remote_format)
             return remote_format.initialize_on_transport(transport)
@@ -2061,6 +1235,7 @@
             except errors.NoSmartMedium:
                 pass
             else:
+                from bzrlib.remote import RemoteBzrDirFormat
                 # TODO: lookup the local format from a server hint.
                 remote_dir_format = RemoteBzrDirFormat()
                 remote_dir_format._network_name = self.network_name()
@@ -2141,7 +1316,7 @@
         utf8_files = [('README',
                        "This is a Bazaar control directory.\n"
                        "Do not change any files in this directory.\n"
-                       "See http://bazaar-vcs.org/ for more information about Bazaar.\n"),
+                       "See http://bazaar.canonical.com/ for more information about Bazaar.\n"),
                       ('branch-format', self.get_format_string()),
                       ]
         # NB: no need to escape relative paths that are url safe.
@@ -2157,55 +1332,13 @@
             control_files.unlock()
         return self.open(transport, _found=True)
 
-    def is_supported(self):
-        """Is this format supported?
-
-        Supported formats must be initializable and openable.
-        Unsupported formats may not support initialization or committing or
-        some other features depending on the reason for not being supported.
-        """
-        return True
-
-    def network_name(self):
-        """A simple byte string uniquely identifying this format for RPC calls.
-
-        Bzr control formats use thir disk format string to identify the format
-        over the wire. Its possible that other control formats have more
-        complex detection requirements, so we permit them to use any unique and
-        immutable string they desire.
-        """
-        raise NotImplementedError(self.network_name)
-
-    def same_model(self, target_format):
-        return (self.repository_format.rich_root_data ==
-            target_format.rich_root_data)
-
-    @classmethod
-    def known_formats(klass):
-        """Return all the known formats.
-
-        Concrete formats should override _known_formats.
-        """
-        # There is double indirection here to make sure that control
-        # formats used by more than one dir format will only be probed
-        # once. This can otherwise be quite expensive for remote connections.
-        result = set()
-        for format in klass._control_formats:
-            result.update(format._known_formats())
-        return result
-
-    @classmethod
-    def _known_formats(klass):
-        """Return the known format instances for this control format."""
-        return set(klass._formats.values())
-
     def open(self, transport, _found=False):
         """Return an instance of this format for the dir transport points at.
 
         _found is a private parameter, do not use it.
         """
         if not _found:
-            found_format = BzrDirFormat.find_format(transport)
+            found_format = controldir.ControlDirFormat.find_format(transport)
             if not isinstance(found_format, self.__class__):
                 raise AssertionError("%s was asked to open %s, but it seems to need "
                         "format %s"
@@ -2223,43 +1356,6 @@
         """
         raise NotImplementedError(self._open)
 
-    @classmethod
-    def register_format(klass, format):
-        klass._formats[format.get_format_string()] = format
-        # bzr native formats have a network name of their format string.
-        network_format_registry.register(format.get_format_string(), format.__class__)
-
-    @classmethod
-    def register_control_format(klass, format):
-        """Register a format that does not use '.bzr' for its control dir.
-
-        TODO: This should be pulled up into a 'ControlDirFormat' base class
-        which BzrDirFormat can inherit from, and renamed to register_format
-        there. It has been done without that for now for simplicity of
-        implementation.
-        """
-        klass._control_formats.append(format)
-
-    @classmethod
-    def register_control_server_format(klass, format):
-        """Register a control format for client-server environments.
-
-        These formats will be tried before ones registered with
-        register_control_format.  This gives implementations that decide to the
-        chance to grab it before anything looks at the contents of the format
-        file.
-        """
-        klass._control_server_formats.append(format)
-
-    @classmethod
-    def _set_default_format(klass, format):
-        """Set default format (for testing behavior of defaults only)"""
-        klass._default_format = format
-
-    def __str__(self):
-        # Trim the newline
-        return self.get_format_description().rstrip()
-
     def _supply_sub_formats_to(self, other_format):
         """Give other_format the same values for sub formats as this has.
 
@@ -2272,214 +1368,9 @@
         :return: None.
         """
 
-    @classmethod
-    def unregister_format(klass, format):
-        del klass._formats[format.get_format_string()]
-
-    @classmethod
-    def unregister_control_format(klass, format):
-        klass._control_formats.remove(format)
-
-
-class BzrDirFormat4(BzrDirFormat):
-    """Bzr dir format 4.
-
-    This format is a combined format for working tree, branch and repository.
-    It has:
-     - Format 1 working trees [always]
-     - Format 4 branches [always]
-     - Format 4 repositories [always]
-
-    This format is deprecated: it indexes texts using a text it which is
-    removed in format 5; write support for this format has been removed.
-    """
-
-    _lock_class = lockable_files.TransportLock
-
-    def get_format_string(self):
-        """See BzrDirFormat.get_format_string()."""
-        return "Bazaar-NG branch, format 0.0.4\n"
-
-    def get_format_description(self):
-        """See BzrDirFormat.get_format_description()."""
-        return "All-in-one format 4"
-
-    def get_converter(self, format=None):
-        """See BzrDirFormat.get_converter()."""
-        # there is one and only one upgrade path here.
-        return ConvertBzrDir4To5()
-
-    def initialize_on_transport(self, transport):
-        """Format 4 branches cannot be created."""
-        raise errors.UninitializableFormat(self)
-
-    def is_supported(self):
-        """Format 4 is not supported.
-
-        It is not supported because the model changed from 4 to 5 and the
-        conversion logic is expensive - so doing it on the fly was not
-        feasible.
-        """
-        return False
-
-    def network_name(self):
-        return self.get_format_string()
-
-    def _open(self, transport):
-        """See BzrDirFormat._open."""
-        return BzrDir4(transport, self)
-
-    def __return_repository_format(self):
-        """Circular import protection."""
-        from bzrlib.repofmt.weaverepo import RepositoryFormat4
-        return RepositoryFormat4()
-    repository_format = property(__return_repository_format)
-
-
-class BzrDirFormatAllInOne(BzrDirFormat):
-    """Common class for formats before meta-dirs."""
-
-    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
-        create_prefix=False, force_new_repo=False, stacked_on=None,
-        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
-        shared_repo=False):
-        """See BzrDirFormat.initialize_on_transport_ex."""
-        require_stacking = (stacked_on is not None)
-        # Format 5 cannot stack, but we've been asked to - actually init
-        # a Meta1Dir
-        if require_stacking:
-            format = BzrDirMetaFormat1()
-            return format.initialize_on_transport_ex(transport,
-                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
-                force_new_repo=force_new_repo, stacked_on=stacked_on,
-                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
-                make_working_trees=make_working_trees, shared_repo=shared_repo)
-        return BzrDirFormat.initialize_on_transport_ex(self, transport,
-            use_existing_dir=use_existing_dir, create_prefix=create_prefix,
-            force_new_repo=force_new_repo, stacked_on=stacked_on,
-            stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
-            make_working_trees=make_working_trees, shared_repo=shared_repo)
-
-
-class BzrDirFormat5(BzrDirFormatAllInOne):
-    """Bzr control format 5.
-
-    This format is a combined format for working tree, branch and repository.
-    It has:
-     - Format 2 working trees [always]
-     - Format 4 branches [always]
-     - Format 5 repositories [always]
-       Unhashed stores in the repository.
-    """
-
-    _lock_class = lockable_files.TransportLock
-
-    def get_format_string(self):
-        """See BzrDirFormat.get_format_string()."""
-        return "Bazaar-NG branch, format 5\n"
-
-    def get_branch_format(self):
-        from bzrlib import branch
-        return branch.BzrBranchFormat4()
-
-    def get_format_description(self):
-        """See BzrDirFormat.get_format_description()."""
-        return "All-in-one format 5"
-
-    def get_converter(self, format=None):
-        """See BzrDirFormat.get_converter()."""
-        # there is one and only one upgrade path here.
-        return ConvertBzrDir5To6()
-
-    def _initialize_for_clone(self, url):
-        return self.initialize_on_transport(get_transport(url), _cloning=True)
-
-    def initialize_on_transport(self, transport, _cloning=False):
-        """Format 5 dirs always have working tree, branch and repository.
-
-        Except when they are being cloned.
-        """
-        from bzrlib.branch import BzrBranchFormat4
-        from bzrlib.repofmt.weaverepo import RepositoryFormat5
-        result = (super(BzrDirFormat5, self).initialize_on_transport(transport))
-        RepositoryFormat5().initialize(result, _internal=True)
-        if not _cloning:
-            branch = BzrBranchFormat4().initialize(result)
-            result._init_workingtree()
-        return result
-
-    def network_name(self):
-        return self.get_format_string()
-
-    def _open(self, transport):
-        """See BzrDirFormat._open."""
-        return BzrDir5(transport, self)
-
-    def __return_repository_format(self):
-        """Circular import protection."""
-        from bzrlib.repofmt.weaverepo import RepositoryFormat5
-        return RepositoryFormat5()
-    repository_format = property(__return_repository_format)
-
-
-class BzrDirFormat6(BzrDirFormatAllInOne):
-    """Bzr control format 6.
-
-    This format is a combined format for working tree, branch and repository.
-    It has:
-     - Format 2 working trees [always]
-     - Format 4 branches [always]
-     - Format 6 repositories [always]
-    """
-
-    _lock_class = lockable_files.TransportLock
-
-    def get_format_string(self):
-        """See BzrDirFormat.get_format_string()."""
-        return "Bazaar-NG branch, format 6\n"
-
-    def get_format_description(self):
-        """See BzrDirFormat.get_format_description()."""
-        return "All-in-one format 6"
-
-    def get_branch_format(self):
-        from bzrlib import branch
-        return branch.BzrBranchFormat4()
-
-    def get_converter(self, format=None):
-        """See BzrDirFormat.get_converter()."""
-        # there is one and only one upgrade path here.
-        return ConvertBzrDir6ToMeta()
-
-    def _initialize_for_clone(self, url):
-        return self.initialize_on_transport(get_transport(url), _cloning=True)
-
-    def initialize_on_transport(self, transport, _cloning=False):
-        """Format 6 dirs always have working tree, branch and repository.
-
-        Except when they are being cloned.
-        """
-        from bzrlib.branch import BzrBranchFormat4
-        from bzrlib.repofmt.weaverepo import RepositoryFormat6
-        result = super(BzrDirFormat6, self).initialize_on_transport(transport)
-        RepositoryFormat6().initialize(result, _internal=True)
-        if not _cloning:
-            branch = BzrBranchFormat4().initialize(result)
-            result._init_workingtree()
-        return result
-
-    def network_name(self):
-        return self.get_format_string()
-
-    def _open(self, transport):
-        """See BzrDirFormat._open."""
-        return BzrDir6(transport, self)
-
-    def __return_repository_format(self):
-        """Circular import protection."""
-        from bzrlib.repofmt.weaverepo import RepositoryFormat6
-        return RepositoryFormat6()
-    repository_format = property(__return_repository_format)
+    def supports_transport(self, transport):
+        # bzr formats can be opened over all known transports
+        return True
 
 
 class BzrDirMetaFormat1(BzrDirFormat):
@@ -2487,14 +1378,20 @@
 
     This is the first format with split out working tree, branch and repository
     disk storage.
+
     It has:
-     - Format 3 working trees [optional]
-     - Format 5 branches [optional]
-     - Format 7 repositories [optional]
+
+    - Format 3 working trees [optional]
+    - Format 5 branches [optional]
+    - Format 7 repositories [optional]
     """
 
     _lock_class = lockdir.LockDir
 
+    fixed_components = False
+
+    colocated_branches = False
+
     def __init__(self):
         self._workingtree_format = None
         self._branch_format = None
@@ -2514,8 +1411,8 @@
 
     def get_branch_format(self):
         if self._branch_format is None:
-            from bzrlib.branch import BranchFormat
-            self._branch_format = BranchFormat.get_default_format()
+            from bzrlib.branch import format_registry as branch_format_registry
+            self._branch_format = branch_format_registry.get_default()
         return self._branch_format
 
     def set_branch_format(self, format):
@@ -2576,9 +1473,9 @@
                     # stack_on is inaccessible, JFDI.
                     # TODO: bad monkey, hard-coded formats...
                     if self.repository_format.rich_root_data:
-                        new_repo_format = pack_repo.RepositoryFormatKnitPack5RichRoot()
+                        new_repo_format = knitpack_repo.RepositoryFormatKnitPack5RichRoot()
                     else:
-                        new_repo_format = pack_repo.RepositoryFormatKnitPack5()
+                        new_repo_format = knitpack_repo.RepositoryFormatKnitPack5()
             else:
                 # If the target already supports stacking, then we know the
                 # project is already able to use stacking, so auto-upgrade
@@ -2590,8 +1487,8 @@
                     new_repo_format = None
             if new_repo_format is not None:
                 self.repository_format = new_repo_format
-                note('Source repository format does not support stacking,'
-                     ' using format:\n  %s',
+                note(gettext('Source repository format does not support stacking,'
+                     ' using format:\n  %s'),
                      new_repo_format.get_format_description())
 
         if not self.get_branch_format().supports_stacking():
@@ -2601,7 +1498,8 @@
             if target_branch is None:
                 if do_upgrade:
                     # TODO: bad monkey, hard-coded formats...
-                    new_branch_format = branch.BzrBranchFormat7()
+                    from bzrlib.branch import BzrBranchFormat7
+                    new_branch_format = BzrBranchFormat7()
             else:
                 new_branch_format = target_branch._format
                 if not new_branch_format.supports_stacking():
@@ -2609,20 +1507,27 @@
             if new_branch_format is not None:
                 # Does support stacking, use its format.
                 self.set_branch_format(new_branch_format)
-                note('Source branch format does not support stacking,'
-                     ' using format:\n  %s',
+                note(gettext('Source branch format does not support stacking,'
+                     ' using format:\n  %s'),
                      new_branch_format.get_format_description())
 
     def get_converter(self, format=None):
         """See BzrDirFormat.get_converter()."""
         if format is None:
             format = BzrDirFormat.get_default_format()
+        if (type(self) is BzrDirMetaFormat1 and
+            type(format) is BzrDirMetaFormat1Colo):
+            return ConvertMetaToColo(format)
+        if (type(self) is BzrDirMetaFormat1Colo and
+            type(format) is BzrDirMetaFormat1):
+            return ConvertMetaRemoveColo(format)
         if not isinstance(self, format.__class__):
             # converting away from metadir is not implemented
             raise NotImplementedError(self.get_converter)
         return ConvertMetaToMeta(format)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG meta directory, format 1\n"
 
@@ -2646,8 +1551,8 @@
         """Circular import protection."""
         if self._repository_format:
             return self._repository_format
-        from bzrlib.repository import RepositoryFormat
-        return RepositoryFormat.get_default_format()
+        from bzrlib.repository import format_registry
+        return format_registry.get_default()
 
     def _set_repository_format(self, value):
         """Allow changing the repository format for metadir formats."""
@@ -2676,8 +1581,10 @@
 
     def __get_workingtree_format(self):
         if self._workingtree_format is None:
-            from bzrlib.workingtree import WorkingTreeFormat
-            self._workingtree_format = WorkingTreeFormat.get_default_format()
+            from bzrlib.workingtree import (
+                format_registry as wt_format_registry,
+                )
+            self._workingtree_format = wt_format_registry.get_default()
         return self._workingtree_format
 
     def __set_workingtree_format(self, wt_format):
@@ -2687,481 +1594,41 @@
                                   __set_workingtree_format)
 
 
-network_format_registry = registry.FormatRegistry()
-"""Registry of formats indexed by their network name.
-
-The network name for a BzrDirFormat is an identifier that can be used when
-referring to formats with smart server operations. See
-BzrDirFormat.network_name() for more detail.
-"""
-
-
-# Register bzr control format
-BzrDirFormat.register_control_format(BzrDirFormat)
-
 # Register bzr formats
-BzrDirFormat.register_format(BzrDirFormat4())
-BzrDirFormat.register_format(BzrDirFormat5())
-BzrDirFormat.register_format(BzrDirFormat6())
-__default_format = BzrDirMetaFormat1()
-BzrDirFormat.register_format(__default_format)
-BzrDirFormat._default_format = __default_format
-
-
-class Converter(object):
-    """Converts a disk format object from one format to another."""
-
-    def convert(self, to_convert, pb):
-        """Perform the conversion of to_convert, giving feedback via pb.
-
-        :param to_convert: The disk object to convert.
-        :param pb: a progress bar to use for progress information.
-        """
-
-    def step(self, message):
-        """Update the pb by a step."""
-        self.count +=1
-        self.pb.update(message, self.count, self.total)
-
-
-class ConvertBzrDir4To5(Converter):
-    """Converts format 4 bzr dirs to format 5."""
-
-    def __init__(self):
-        super(ConvertBzrDir4To5, self).__init__()
-        self.converted_revs = set()
-        self.absent_revisions = set()
-        self.text_count = 0
-        self.revisions = {}
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        self.bzrdir = to_convert
-        if pb is not None:
-            warnings.warn("pb parameter to convert() is deprecated")
-        self.pb = ui.ui_factory.nested_progress_bar()
-        try:
-            ui.ui_factory.note('starting upgrade from format 4 to 5')
-            if isinstance(self.bzrdir.transport, local.LocalTransport):
-                self.bzrdir.get_workingtree_transport(None).delete('stat-cache')
-            self._convert_to_weaves()
-            return BzrDir.open(self.bzrdir.user_url)
-        finally:
-            self.pb.finished()
-
-    def _convert_to_weaves(self):
-        ui.ui_factory.note('note: upgrade may be faster if all store files are ungzipped first')
-        try:
-            # TODO permissions
-            stat = self.bzrdir.transport.stat('weaves')
-            if not S_ISDIR(stat.st_mode):
-                self.bzrdir.transport.delete('weaves')
-                self.bzrdir.transport.mkdir('weaves')
-        except errors.NoSuchFile:
-            self.bzrdir.transport.mkdir('weaves')
-        # deliberately not a WeaveFile as we want to build it up slowly.
-        self.inv_weave = Weave('inventory')
-        # holds in-memory weaves for all files
-        self.text_weaves = {}
-        self.bzrdir.transport.delete('branch-format')
-        self.branch = self.bzrdir.open_branch()
-        self._convert_working_inv()
-        rev_history = self.branch.revision_history()
-        # to_read is a stack holding the revisions we still need to process;
-        # appending to it adds new highest-priority revisions
-        self.known_revisions = set(rev_history)
-        self.to_read = rev_history[-1:]
-        while self.to_read:
-            rev_id = self.to_read.pop()
-            if (rev_id not in self.revisions
-                and rev_id not in self.absent_revisions):
-                self._load_one_rev(rev_id)
-        self.pb.clear()
-        to_import = self._make_order()
-        for i, rev_id in enumerate(to_import):
-            self.pb.update('converting revision', i, len(to_import))
-            self._convert_one_rev(rev_id)
-        self.pb.clear()
-        self._write_all_weaves()
-        self._write_all_revs()
-        ui.ui_factory.note('upgraded to weaves:')
-        ui.ui_factory.note('  %6d revisions and inventories' % len(self.revisions))
-        ui.ui_factory.note('  %6d revisions not present' % len(self.absent_revisions))
-        ui.ui_factory.note('  %6d texts' % self.text_count)
-        self._cleanup_spare_files_after_format4()
-        self.branch._transport.put_bytes(
-            'branch-format',
-            BzrDirFormat5().get_format_string(),
-            mode=self.bzrdir._get_file_mode())
-
-    def _cleanup_spare_files_after_format4(self):
-        # FIXME working tree upgrade foo.
-        for n in 'merged-patches', 'pending-merged-patches':
-            try:
-                ## assert os.path.getsize(p) == 0
-                self.bzrdir.transport.delete(n)
-            except errors.NoSuchFile:
-                pass
-        self.bzrdir.transport.delete_tree('inventory-store')
-        self.bzrdir.transport.delete_tree('text-store')
-
-    def _convert_working_inv(self):
-        inv = xml4.serializer_v4.read_inventory(
-                self.branch._transport.get('inventory'))
-        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv, working=True)
-        self.branch._transport.put_bytes('inventory', new_inv_xml,
-            mode=self.bzrdir._get_file_mode())
-
-    def _write_all_weaves(self):
-        controlweaves = WeaveStore(self.bzrdir.transport, prefixed=False)
-        weave_transport = self.bzrdir.transport.clone('weaves')
-        weaves = WeaveStore(weave_transport, prefixed=False)
-        transaction = WriteTransaction()
-
-        try:
-            i = 0
-            for file_id, file_weave in self.text_weaves.items():
-                self.pb.update('writing weave', i, len(self.text_weaves))
-                weaves._put_weave(file_id, file_weave, transaction)
-                i += 1
-            self.pb.update('inventory', 0, 1)
-            controlweaves._put_weave('inventory', self.inv_weave, transaction)
-            self.pb.update('inventory', 1, 1)
-        finally:
-            self.pb.clear()
-
-    def _write_all_revs(self):
-        """Write all revisions out in new form."""
-        self.bzrdir.transport.delete_tree('revision-store')
-        self.bzrdir.transport.mkdir('revision-store')
-        revision_transport = self.bzrdir.transport.clone('revision-store')
-        # TODO permissions
-        from bzrlib.xml5 import serializer_v5
-        from bzrlib.repofmt.weaverepo import RevisionTextStore
-        revision_store = RevisionTextStore(revision_transport,
-            serializer_v5, False, versionedfile.PrefixMapper(),
-            lambda:True, lambda:True)
-        try:
-            for i, rev_id in enumerate(self.converted_revs):
-                self.pb.update('write revision', i, len(self.converted_revs))
-                text = serializer_v5.write_revision_to_string(
-                    self.revisions[rev_id])
-                key = (rev_id,)
-                revision_store.add_lines(key, None, osutils.split_lines(text))
-        finally:
-            self.pb.clear()
-
-    def _load_one_rev(self, rev_id):
-        """Load a revision object into memory.
-
-        Any parents not either loaded or abandoned get queued to be
-        loaded."""
-        self.pb.update('loading revision',
-                       len(self.revisions),
-                       len(self.known_revisions))
-        if not self.branch.repository.has_revision(rev_id):
-            self.pb.clear()
-            ui.ui_factory.note('revision {%s} not present in branch; '
-                         'will be converted as a ghost' %
-                         rev_id)
-            self.absent_revisions.add(rev_id)
-        else:
-            rev = self.branch.repository.get_revision(rev_id)
-            for parent_id in rev.parent_ids:
-                self.known_revisions.add(parent_id)
-                self.to_read.append(parent_id)
-            self.revisions[rev_id] = rev
-
-    def _load_old_inventory(self, rev_id):
-        f = self.branch.repository.inventory_store.get(rev_id)
-        try:
-            old_inv_xml = f.read()
-        finally:
-            f.close()
-        inv = xml4.serializer_v4.read_inventory_from_string(old_inv_xml)
-        inv.revision_id = rev_id
-        rev = self.revisions[rev_id]
-        return inv
-
-    def _load_updated_inventory(self, rev_id):
-        inv_xml = self.inv_weave.get_text(rev_id)
-        inv = xml5.serializer_v5.read_inventory_from_string(inv_xml, rev_id)
-        return inv
-
-    def _convert_one_rev(self, rev_id):
-        """Convert revision and all referenced objects to new format."""
-        rev = self.revisions[rev_id]
-        inv = self._load_old_inventory(rev_id)
-        present_parents = [p for p in rev.parent_ids
-                           if p not in self.absent_revisions]
-        self._convert_revision_contents(rev, inv, present_parents)
-        self._store_new_inv(rev, inv, present_parents)
-        self.converted_revs.add(rev_id)
-
-    def _store_new_inv(self, rev, inv, present_parents):
-        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv)
-        new_inv_sha1 = sha_string(new_inv_xml)
-        self.inv_weave.add_lines(rev.revision_id,
-                                 present_parents,
-                                 new_inv_xml.splitlines(True))
-        rev.inventory_sha1 = new_inv_sha1
-
-    def _convert_revision_contents(self, rev, inv, present_parents):
-        """Convert all the files within a revision.
-
-        Also upgrade the inventory to refer to the text revision ids."""
-        rev_id = rev.revision_id
-        mutter('converting texts of revision {%s}',
-               rev_id)
-        parent_invs = map(self._load_updated_inventory, present_parents)
-        entries = inv.iter_entries()
-        entries.next()
-        for path, ie in entries:
-            self._convert_file_version(rev, ie, parent_invs)
-
-    def _convert_file_version(self, rev, ie, parent_invs):
-        """Convert one version of one file.
-
-        The file needs to be added into the weave if it is a merge
-        of >=2 parents or if it's changed from its parent.
-        """
-        file_id = ie.file_id
-        rev_id = rev.revision_id
-        w = self.text_weaves.get(file_id)
-        if w is None:
-            w = Weave(file_id)
-            self.text_weaves[file_id] = w
-        text_changed = False
-        parent_candiate_entries = ie.parent_candidates(parent_invs)
-        heads = graph.Graph(self).heads(parent_candiate_entries.keys())
-        # XXX: Note that this is unordered - and this is tolerable because
-        # the previous code was also unordered.
-        previous_entries = dict((head, parent_candiate_entries[head]) for head
-            in heads)
-        self.snapshot_ie(previous_entries, ie, w, rev_id)
-        del ie.text_id
-
-    def get_parent_map(self, revision_ids):
-        """See graph.StackedParentsProvider.get_parent_map"""
-        return dict((revision_id, self.revisions[revision_id])
-                    for revision_id in revision_ids
-                     if revision_id in self.revisions)
-
-    def snapshot_ie(self, previous_revisions, ie, w, rev_id):
-        # TODO: convert this logic, which is ~= snapshot to
-        # a call to:. This needs the path figured out. rather than a work_tree
-        # a v4 revision_tree can be given, or something that looks enough like
-        # one to give the file content to the entry if it needs it.
-        # and we need something that looks like a weave store for snapshot to
-        # save against.
-        #ie.snapshot(rev, PATH, previous_revisions, REVISION_TREE, InMemoryWeaveStore(self.text_weaves))
-        if len(previous_revisions) == 1:
-            previous_ie = previous_revisions.values()[0]
-            if ie._unchanged(previous_ie):
-                ie.revision = previous_ie.revision
-                return
-        if ie.has_text():
-            f = self.branch.repository._text_store.get(ie.text_id)
-            try:
-                file_lines = f.readlines()
-            finally:
-                f.close()
-            w.add_lines(rev_id, previous_revisions, file_lines)
-            self.text_count += 1
-        else:
-            w.add_lines(rev_id, previous_revisions, [])
-        ie.revision = rev_id
-
-    def _make_order(self):
-        """Return a suitable order for importing revisions.
-
-        The order must be such that an revision is imported after all
-        its (present) parents.
-        """
-        todo = set(self.revisions.keys())
-        done = self.absent_revisions.copy()
-        order = []
-        while todo:
-            # scan through looking for a revision whose parents
-            # are all done
-            for rev_id in sorted(list(todo)):
-                rev = self.revisions[rev_id]
-                parent_ids = set(rev.parent_ids)
-                if parent_ids.issubset(done):
-                    # can take this one now
-                    order.append(rev_id)
-                    todo.remove(rev_id)
-                    done.add(rev_id)
-        return order
-
-
-class ConvertBzrDir5To6(Converter):
-    """Converts format 5 bzr dirs to format 6."""
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        self.bzrdir = to_convert
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            ui.ui_factory.note('starting upgrade from format 5 to 6')
-            self._convert_to_prefixed()
-            return BzrDir.open(self.bzrdir.user_url)
-        finally:
-            pb.finished()
-
-    def _convert_to_prefixed(self):
-        from bzrlib.store import TransportStore
-        self.bzrdir.transport.delete('branch-format')
-        for store_name in ["weaves", "revision-store"]:
-            ui.ui_factory.note("adding prefixes to %s" % store_name)
-            store_transport = self.bzrdir.transport.clone(store_name)
-            store = TransportStore(store_transport, prefixed=True)
-            for urlfilename in store_transport.list_dir('.'):
-                filename = urlutils.unescape(urlfilename)
-                if (filename.endswith(".weave") or
-                    filename.endswith(".gz") or
-                    filename.endswith(".sig")):
-                    file_id, suffix = os.path.splitext(filename)
-                else:
-                    file_id = filename
-                    suffix = ''
-                new_name = store._mapper.map((file_id,)) + suffix
-                # FIXME keep track of the dirs made RBC 20060121
-                try:
-                    store_transport.move(filename, new_name)
-                except errors.NoSuchFile: # catches missing dirs strangely enough
-                    store_transport.mkdir(osutils.dirname(new_name))
-                    store_transport.move(filename, new_name)
-        self.bzrdir.transport.put_bytes(
-            'branch-format',
-            BzrDirFormat6().get_format_string(),
-            mode=self.bzrdir._get_file_mode())
-
-
-class ConvertBzrDir6ToMeta(Converter):
-    """Converts format 6 bzr dirs to metadirs."""
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        from bzrlib.repofmt.weaverepo import RepositoryFormat7
-        from bzrlib.branch import BzrBranchFormat5
-        self.bzrdir = to_convert
-        self.pb = ui.ui_factory.nested_progress_bar()
-        self.count = 0
-        self.total = 20 # the steps we know about
-        self.garbage_inventories = []
-        self.dir_mode = self.bzrdir._get_dir_mode()
-        self.file_mode = self.bzrdir._get_file_mode()
-
-        ui.ui_factory.note('starting upgrade from format 6 to metadir')
-        self.bzrdir.transport.put_bytes(
-                'branch-format',
-                "Converting to format 6",
-                mode=self.file_mode)
-        # its faster to move specific files around than to open and use the apis...
-        # first off, nuke ancestry.weave, it was never used.
-        try:
-            self.step('Removing ancestry.weave')
-            self.bzrdir.transport.delete('ancestry.weave')
-        except errors.NoSuchFile:
-            pass
-        # find out whats there
-        self.step('Finding branch files')
-        last_revision = self.bzrdir.open_branch().last_revision()
-        bzrcontents = self.bzrdir.transport.list_dir('.')
-        for name in bzrcontents:
-            if name.startswith('basis-inventory.'):
-                self.garbage_inventories.append(name)
-        # create new directories for repository, working tree and branch
-        repository_names = [('inventory.weave', True),
-                            ('revision-store', True),
-                            ('weaves', True)]
-        self.step('Upgrading repository  ')
-        self.bzrdir.transport.mkdir('repository', mode=self.dir_mode)
-        self.make_lock('repository')
-        # we hard code the formats here because we are converting into
-        # the meta format. The meta format upgrader can take this to a
-        # future format within each component.
-        self.put_format('repository', RepositoryFormat7())
-        for entry in repository_names:
-            self.move_entry('repository', entry)
-
-        self.step('Upgrading branch      ')
-        self.bzrdir.transport.mkdir('branch', mode=self.dir_mode)
-        self.make_lock('branch')
-        self.put_format('branch', BzrBranchFormat5())
-        branch_files = [('revision-history', True),
-                        ('branch-name', True),
-                        ('parent', False)]
-        for entry in branch_files:
-            self.move_entry('branch', entry)
-
-        checkout_files = [('pending-merges', True),
-                          ('inventory', True),
-                          ('stat-cache', False)]
-        # If a mandatory checkout file is not present, the branch does not have
-        # a functional checkout. Do not create a checkout in the converted
-        # branch.
-        for name, mandatory in checkout_files:
-            if mandatory and name not in bzrcontents:
-                has_checkout = False
-                break
-        else:
-            has_checkout = True
-        if not has_checkout:
-            ui.ui_factory.note('No working tree.')
-            # If some checkout files are there, we may as well get rid of them.
-            for name, mandatory in checkout_files:
-                if name in bzrcontents:
-                    self.bzrdir.transport.delete(name)
-        else:
-            from bzrlib.workingtree import WorkingTreeFormat3
-            self.step('Upgrading working tree')
-            self.bzrdir.transport.mkdir('checkout', mode=self.dir_mode)
-            self.make_lock('checkout')
-            self.put_format(
-                'checkout', WorkingTreeFormat3())
-            self.bzrdir.transport.delete_multi(
-                self.garbage_inventories, self.pb)
-            for entry in checkout_files:
-                self.move_entry('checkout', entry)
-            if last_revision is not None:
-                self.bzrdir.transport.put_bytes(
-                    'checkout/last-revision', last_revision)
-        self.bzrdir.transport.put_bytes(
-            'branch-format',
-            BzrDirMetaFormat1().get_format_string(),
-            mode=self.file_mode)
-        self.pb.finished()
-        return BzrDir.open(self.bzrdir.user_url)
-
-    def make_lock(self, name):
-        """Make a lock for the new control dir name."""
-        self.step('Make %s lock' % name)
-        ld = lockdir.LockDir(self.bzrdir.transport,
-                             '%s/lock' % name,
-                             file_modebits=self.file_mode,
-                             dir_modebits=self.dir_mode)
-        ld.create()
-
-    def move_entry(self, new_dir, entry):
-        """Move then entry name into new_dir."""
-        name = entry[0]
-        mandatory = entry[1]
-        self.step('Moving %s' % name)
-        try:
-            self.bzrdir.transport.move(name, '%s/%s' % (new_dir, name))
-        except errors.NoSuchFile:
-            if mandatory:
-                raise
-
-    def put_format(self, dirname, format):
-        self.bzrdir.transport.put_bytes('%s/format' % dirname,
-            format.get_format_string(),
-            self.file_mode)
-
-
-class ConvertMetaToMeta(Converter):
+BzrProber.formats.register(BzrDirMetaFormat1.get_format_string(),
+    BzrDirMetaFormat1)
+controldir.ControlDirFormat._default_format = BzrDirMetaFormat1()
+
+
+class BzrDirMetaFormat1Colo(BzrDirMetaFormat1):
+    """BzrDirMeta1 format with support for colocated branches."""
+
+    colocated_branches = True
+
+    @classmethod
+    def get_format_string(cls):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar meta directory, format 1 (with colocated branches)\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "Meta directory format 1 with support for colocated branches"
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        # Create a new format instance because otherwise initialisation of new
+        # metadirs share the global default format object leading to alias
+        # problems.
+        format = BzrDirMetaFormat1Colo()
+        self._supply_sub_formats_to(format)
+        return BzrDirMeta1Colo(transport, format)
+
+
+BzrProber.formats.register(BzrDirMetaFormat1Colo.get_format_string(),
+    BzrDirMetaFormat1Colo)
+
+
+class ConvertMetaToMeta(controldir.Converter):
     """Converts the components of metadirs."""
 
     def __init__(self, target_format):
@@ -3185,14 +1652,13 @@
         else:
             if not isinstance(repo._format, self.target_format.repository_format.__class__):
                 from bzrlib.repository import CopyConverter
-                ui.ui_factory.note('starting repository conversion')
+                ui.ui_factory.note(gettext('starting repository conversion'))
                 converter = CopyConverter(self.target_format.repository_format)
                 converter.convert(repo, pb)
         for branch in self.bzrdir.list_branches():
             # TODO: conversions of Branch and Tree should be done by
             # InterXFormat lookups/some sort of registry.
             # Avoid circular imports
-            from bzrlib import branch as _mod_branch
             old = branch._format.__class__
             new = self.target_format.get_branch_format().__class__
             while old != new:
@@ -3221,7 +1687,7 @@
         else:
             # TODO: conversions of Branch and Tree should be done by
             # InterXFormat lookups
-            if (isinstance(tree, workingtree.WorkingTree3) and
+            if (isinstance(tree, workingtree_3.WorkingTree3) and
                 not isinstance(tree, workingtree_4.DirStateWorkingTree) and
                 isinstance(self.target_format.workingtree_format,
                     workingtree_4.DirStateWorkingTreeFormat)):
@@ -3240,452 +1706,50 @@
         return to_convert
 
 
-# This is not in remote.py because it's relatively small, and needs to be
-# registered. Putting it in remote.py creates a circular import problem.
-# we can make it a lazy object if the control formats is turned into something
-# like a registry.
-class RemoteBzrDirFormat(BzrDirMetaFormat1):
-    """Format representing bzrdirs accessed via a smart server"""
-
-    def __init__(self):
-        BzrDirMetaFormat1.__init__(self)
-        # XXX: It's a bit ugly that the network name is here, because we'd
-        # like to believe that format objects are stateless or at least
-        # immutable,  However, we do at least avoid mutating the name after
-        # it's returned.  See <https://bugs.launchpad.net/bzr/+bug/504102>
-        self._network_name = None
-
-    def __repr__(self):
-        return "%s(_network_name=%r)" % (self.__class__.__name__,
-            self._network_name)
-
-    def get_format_description(self):
-        if self._network_name:
-            real_format = network_format_registry.get(self._network_name)
-            return 'Remote: ' + real_format.get_format_description()
-        return 'bzr remote bzrdir'
-
-    def get_format_string(self):
-        raise NotImplementedError(self.get_format_string)
-
-    def network_name(self):
-        if self._network_name:
-            return self._network_name
-        else:
-            raise AssertionError("No network name set.")
-
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return a RemoteBzrDirFormat object if it looks possible."""
-        try:
-            medium = transport.get_smart_medium()
-        except (NotImplementedError, AttributeError,
-                errors.TransportNotPossible, errors.NoSmartMedium,
-                errors.SmartProtocolError):
-            # no smart server, so not a branch for this format type.
-            raise errors.NotBranchError(path=transport.base)
-        else:
-            # Decline to open it if the server doesn't support our required
-            # version (3) so that the VFS-based transport will do it.
-            if medium.should_probe():
-                try:
-                    server_version = medium.protocol_version()
-                except errors.SmartProtocolError:
-                    # Apparently there's no usable smart server there, even though
-                    # the medium supports the smart protocol.
-                    raise errors.NotBranchError(path=transport.base)
-                if server_version != '2':
-                    raise errors.NotBranchError(path=transport.base)
-            return klass()
-
-    def initialize_on_transport(self, transport):
-        try:
-            # hand off the request to the smart server
-            client_medium = transport.get_smart_medium()
-        except errors.NoSmartMedium:
-            # TODO: lookup the local format from a server hint.
-            local_dir_format = BzrDirMetaFormat1()
-            return local_dir_format.initialize_on_transport(transport)
-        client = _SmartClient(client_medium)
-        path = client.remote_path_from_transport(transport)
-        try:
-            response = client.call('BzrDirFormat.initialize', path)
-        except errors.ErrorFromSmartServer, err:
-            remote._translate_error(err, path=path)
-        if response[0] != 'ok':
-            raise errors.SmartProtocolError('unexpected response code %s' % (response,))
-        format = RemoteBzrDirFormat()
-        self._supply_sub_formats_to(format)
-        return remote.RemoteBzrDir(transport, format)
-
-    def parse_NoneTrueFalse(self, arg):
-        if not arg:
-            return None
-        if arg == 'False':
-            return False
-        if arg == 'True':
-            return True
-        raise AssertionError("invalid arg %r" % arg)
-
-    def _serialize_NoneTrueFalse(self, arg):
-        if arg is False:
-            return 'False'
-        if arg:
-            return 'True'
-        return ''
-
-    def _serialize_NoneString(self, arg):
-        return arg or ''
-
-    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
-        create_prefix=False, force_new_repo=False, stacked_on=None,
-        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
-        shared_repo=False):
-        try:
-            # hand off the request to the smart server
-            client_medium = transport.get_smart_medium()
-        except errors.NoSmartMedium:
-            do_vfs = True
-        else:
-            # Decline to open it if the server doesn't support our required
-            # version (3) so that the VFS-based transport will do it.
-            if client_medium.should_probe():
-                try:
-                    server_version = client_medium.protocol_version()
-                    if server_version != '2':
-                        do_vfs = True
-                    else:
-                        do_vfs = False
-                except errors.SmartProtocolError:
-                    # Apparently there's no usable smart server there, even though
-                    # the medium supports the smart protocol.
-                    do_vfs = True
-            else:
-                do_vfs = False
-        if not do_vfs:
-            client = _SmartClient(client_medium)
-            path = client.remote_path_from_transport(transport)
-            if client_medium._is_remote_before((1, 16)):
-                do_vfs = True
-        if do_vfs:
-            # TODO: lookup the local format from a server hint.
-            local_dir_format = BzrDirMetaFormat1()
-            self._supply_sub_formats_to(local_dir_format)
-            return local_dir_format.initialize_on_transport_ex(transport,
-                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
-                force_new_repo=force_new_repo, stacked_on=stacked_on,
-                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
-                make_working_trees=make_working_trees, shared_repo=shared_repo,
-                vfs_only=True)
-        return self._initialize_on_transport_ex_rpc(client, path, transport,
-            use_existing_dir, create_prefix, force_new_repo, stacked_on,
-            stack_on_pwd, repo_format_name, make_working_trees, shared_repo)
-
-    def _initialize_on_transport_ex_rpc(self, client, path, transport,
-        use_existing_dir, create_prefix, force_new_repo, stacked_on,
-        stack_on_pwd, repo_format_name, make_working_trees, shared_repo):
-        args = []
-        args.append(self._serialize_NoneTrueFalse(use_existing_dir))
-        args.append(self._serialize_NoneTrueFalse(create_prefix))
-        args.append(self._serialize_NoneTrueFalse(force_new_repo))
-        args.append(self._serialize_NoneString(stacked_on))
-        # stack_on_pwd is often/usually our transport
-        if stack_on_pwd:
-            try:
-                stack_on_pwd = transport.relpath(stack_on_pwd)
-                if not stack_on_pwd:
-                    stack_on_pwd = '.'
-            except errors.PathNotChild:
-                pass
-        args.append(self._serialize_NoneString(stack_on_pwd))
-        args.append(self._serialize_NoneString(repo_format_name))
-        args.append(self._serialize_NoneTrueFalse(make_working_trees))
-        args.append(self._serialize_NoneTrueFalse(shared_repo))
-        request_network_name = self._network_name or \
-            BzrDirFormat.get_default_format().network_name()
-        try:
-            response = client.call('BzrDirFormat.initialize_ex_1.16',
-                request_network_name, path, *args)
-        except errors.UnknownSmartMethod:
-            client._medium._remember_remote_is_before((1,16))
-            local_dir_format = BzrDirMetaFormat1()
-            self._supply_sub_formats_to(local_dir_format)
-            return local_dir_format.initialize_on_transport_ex(transport,
-                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
-                force_new_repo=force_new_repo, stacked_on=stacked_on,
-                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
-                make_working_trees=make_working_trees, shared_repo=shared_repo,
-                vfs_only=True)
-        except errors.ErrorFromSmartServer, err:
-            remote._translate_error(err, path=path)
-        repo_path = response[0]
-        bzrdir_name = response[6]
-        require_stacking = response[7]
-        require_stacking = self.parse_NoneTrueFalse(require_stacking)
-        format = RemoteBzrDirFormat()
-        format._network_name = bzrdir_name
-        self._supply_sub_formats_to(format)
-        bzrdir = remote.RemoteBzrDir(transport, format, _client=client)
-        if repo_path:
-            repo_format = remote.response_tuple_to_repo_format(response[1:])
-            if repo_path == '.':
-                repo_path = ''
-            if repo_path:
-                repo_bzrdir_format = RemoteBzrDirFormat()
-                repo_bzrdir_format._network_name = response[5]
-                repo_bzr = remote.RemoteBzrDir(transport.clone(repo_path),
-                    repo_bzrdir_format)
-            else:
-                repo_bzr = bzrdir
-            final_stack = response[8] or None
-            final_stack_pwd = response[9] or None
-            if final_stack_pwd:
-                final_stack_pwd = urlutils.join(
-                    transport.base, final_stack_pwd)
-            remote_repo = remote.RemoteRepository(repo_bzr, repo_format)
-            if len(response) > 10:
-                # Updated server verb that locks remotely.
-                repo_lock_token = response[10] or None
-                remote_repo.lock_write(repo_lock_token, _skip_rpc=True)
-                if repo_lock_token:
-                    remote_repo.dont_leave_lock_in_place()
-            else:
-                remote_repo.lock_write()
-            policy = UseExistingRepository(remote_repo, final_stack,
-                final_stack_pwd, require_stacking)
-            policy.acquire_repository()
-        else:
-            remote_repo = None
-            policy = None
-        bzrdir._format.set_branch_format(self.get_branch_format())
-        if require_stacking:
-            # The repo has already been created, but we need to make sure that
-            # we'll make a stackable branch.
-            bzrdir._format.require_stacking(_skip_repo=True)
-        return remote_repo, bzrdir, require_stacking, policy
-
-    def _open(self, transport):
-        return remote.RemoteBzrDir(transport, self)
-
-    def __eq__(self, other):
-        if not isinstance(other, RemoteBzrDirFormat):
-            return False
-        return self.get_format_description() == other.get_format_description()
-
-    def __return_repository_format(self):
-        # Always return a RemoteRepositoryFormat object, but if a specific bzr
-        # repository format has been asked for, tell the RemoteRepositoryFormat
-        # that it should use that for init() etc.
-        result = remote.RemoteRepositoryFormat()
-        custom_format = getattr(self, '_repository_format', None)
-        if custom_format:
-            if isinstance(custom_format, remote.RemoteRepositoryFormat):
-                return custom_format
-            else:
-                # We will use the custom format to create repositories over the
-                # wire; expose its details like rich_root_data for code to
-                # query
-                result._custom_format = custom_format
-        return result
-
-    def get_branch_format(self):
-        result = BzrDirMetaFormat1.get_branch_format(self)
-        if not isinstance(result, remote.RemoteBranchFormat):
-            new_result = remote.RemoteBranchFormat()
-            new_result._custom_format = result
-            # cache the result
-            self.set_branch_format(new_result)
-            result = new_result
-        return result
-
-    repository_format = property(__return_repository_format,
-        BzrDirMetaFormat1._set_repository_format) #.im_func)
-
-
-BzrDirFormat.register_control_server_format(RemoteBzrDirFormat)
-
-
-class BzrDirFormatInfo(object):
-
-    def __init__(self, native, deprecated, hidden, experimental):
-        self.deprecated = deprecated
-        self.native = native
-        self.hidden = hidden
-        self.experimental = experimental
-
-
-class BzrDirFormatRegistry(registry.Registry):
-    """Registry of user-selectable BzrDir subformats.
-
-    Differs from BzrDirFormat._control_formats in that it provides sub-formats,
-    e.g. BzrDirMeta1 with weave repository.  Also, it's more user-oriented.
-    """
-
-    def __init__(self):
-        """Create a BzrDirFormatRegistry."""
-        self._aliases = set()
-        self._registration_order = list()
-        super(BzrDirFormatRegistry, self).__init__()
-
-    def aliases(self):
-        """Return a set of the format names which are aliases."""
-        return frozenset(self._aliases)
-
-    def register_metadir(self, key,
-             repository_format, help, native=True, deprecated=False,
-             branch_format=None,
-             tree_format=None,
-             hidden=False,
-             experimental=False,
-             alias=False):
-        """Register a metadir subformat.
-
-        These all use a BzrDirMetaFormat1 bzrdir, but can be parameterized
-        by the Repository/Branch/WorkingTreeformats.
-
-        :param repository_format: The fully-qualified repository format class
-            name as a string.
-        :param branch_format: Fully-qualified branch format class name as
-            a string.
-        :param tree_format: Fully-qualified tree format class name as
-            a string.
-        """
-        # This should be expanded to support setting WorkingTree and Branch
-        # formats, once BzrDirMetaFormat1 supports that.
-        def _load(full_name):
-            mod_name, factory_name = full_name.rsplit('.', 1)
-            try:
-                mod = __import__(mod_name, globals(), locals(),
-                        [factory_name])
-            except ImportError, e:
-                raise ImportError('failed to load %s: %s' % (full_name, e))
-            try:
-                factory = getattr(mod, factory_name)
-            except AttributeError:
-                raise AttributeError('no factory %s in module %r'
-                    % (full_name, mod))
-            return factory()
-
-        def helper():
-            bd = BzrDirMetaFormat1()
-            if branch_format is not None:
-                bd.set_branch_format(_load(branch_format))
-            if tree_format is not None:
-                bd.workingtree_format = _load(tree_format)
-            if repository_format is not None:
-                bd.repository_format = _load(repository_format)
-            return bd
-        self.register(key, helper, help, native, deprecated, hidden,
-            experimental, alias)
-
-    def register(self, key, factory, help, native=True, deprecated=False,
-                 hidden=False, experimental=False, alias=False):
-        """Register a BzrDirFormat factory.
-
-        The factory must be a callable that takes one parameter: the key.
-        It must produce an instance of the BzrDirFormat when called.
-
-        This function mainly exists to prevent the info object from being
-        supplied directly.
-        """
-        registry.Registry.register(self, key, factory, help,
-            BzrDirFormatInfo(native, deprecated, hidden, experimental))
-        if alias:
-            self._aliases.add(key)
-        self._registration_order.append(key)
-
-    def register_lazy(self, key, module_name, member_name, help, native=True,
-        deprecated=False, hidden=False, experimental=False, alias=False):
-        registry.Registry.register_lazy(self, key, module_name, member_name,
-            help, BzrDirFormatInfo(native, deprecated, hidden, experimental))
-        if alias:
-            self._aliases.add(key)
-        self._registration_order.append(key)
-
-    def set_default(self, key):
-        """Set the 'default' key to be a clone of the supplied key.
-
-        This method must be called once and only once.
-        """
-        registry.Registry.register(self, 'default', self.get(key),
-            self.get_help(key), info=self.get_info(key))
-        self._aliases.add('default')
-
-    def set_default_repository(self, key):
-        """Set the FormatRegistry default and Repository default.
-
-        This is a transitional method while Repository.set_default_format
-        is deprecated.
-        """
-        if 'default' in self:
-            self.remove('default')
-        self.set_default(key)
-        format = self.get('default')()
-
-    def make_bzrdir(self, key):
-        return self.get(key)()
-
-    def help_topic(self, topic):
-        output = ""
-        default_realkey = None
-        default_help = self.get_help('default')
-        help_pairs = []
-        for key in self._registration_order:
-            if key == 'default':
-                continue
-            help = self.get_help(key)
-            if help == default_help:
-                default_realkey = key
-            else:
-                help_pairs.append((key, help))
-
-        def wrapped(key, help, info):
-            if info.native:
-                help = '(native) ' + help
-            return ':%s:\n%s\n\n' % (key,
-                textwrap.fill(help, initial_indent='    ',
-                    subsequent_indent='    ',
-                    break_long_words=False))
-        if default_realkey is not None:
-            output += wrapped(default_realkey, '(default) %s' % default_help,
-                              self.get_info('default'))
-        deprecated_pairs = []
-        experimental_pairs = []
-        for key, help in help_pairs:
-            info = self.get_info(key)
-            if info.hidden:
-                continue
-            elif info.deprecated:
-                deprecated_pairs.append((key, help))
-            elif info.experimental:
-                experimental_pairs.append((key, help))
-            else:
-                output += wrapped(key, help, info)
-        output += "\nSee :doc:`formats-help` for more about storage formats."
-        other_output = ""
-        if len(experimental_pairs) > 0:
-            other_output += "Experimental formats are shown below.\n\n"
-            for key, help in experimental_pairs:
-                info = self.get_info(key)
-                other_output += wrapped(key, help, info)
-        else:
-            other_output += \
-                "No experimental formats are available.\n\n"
-        if len(deprecated_pairs) > 0:
-            other_output += "\nDeprecated formats are shown below.\n\n"
-            for key, help in deprecated_pairs:
-                info = self.get_info(key)
-                other_output += wrapped(key, help, info)
-        else:
-            other_output += \
-                "\nNo deprecated formats are available.\n\n"
-        other_output += \
-                "\nSee :doc:`formats-help` for more about storage formats."
-
-        if topic == 'other-formats':
-            return other_output
-        else:
-            return output
+class ConvertMetaToColo(controldir.Converter):
+    """Add colocated branch support."""
+
+    def __init__(self, target_format):
+        """Create a converter.that upgrades a metadir to the colo format.
+
+        :param target_format: The final metadir format that is desired.
+        """
+        self.target_format = target_format
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        to_convert.transport.put_bytes('branch-format',
+            self.target_format.get_format_string())
+        return BzrDir.open_from_transport(to_convert.root_transport)
+
+
+class ConvertMetaRemoveColo(controldir.Converter):
+    """Remove colocated branch support from a bzrdir."""
+
+    def __init__(self, target_format):
+        """Create a converter.that downgrades a colocated branch metadir
+        to a regular metadir.
+
+        :param target_format: The final metadir format that is desired.
+        """
+        self.target_format = target_format
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        to_convert.control_files.lock_write()
+        try:
+            branches = to_convert.list_branches()
+            if len(branches) > 1:
+                raise errors.BzrError("remove all but a single "
+                    "colocated branch when downgrading")
+        finally:
+            to_convert.control_files.unlock()
+        to_convert.transport.put_bytes('branch-format',
+            self.target_format.get_format_string())
+        return BzrDir.open_from_transport(to_convert.root_transport)
+
+
+controldir.ControlDirFormat.register_server_prober(RemoteBzrProber)
 
 
 class RepositoryAcquisitionPolicy(object):
@@ -3772,6 +1836,7 @@
 
         Implementations may create a new repository or use a pre-exising
         repository.
+
         :param make_working_trees: If creating a repository, set
             make_working_trees to this value (if non-None)
         :param shared: If creating a repository, make it shared if True
@@ -3786,8 +1851,8 @@
 
     def __init__(self, bzrdir, stack_on=None, stack_on_pwd=None,
                  require_stacking=False):
-        """
-        Constructor.
+        """Constructor.
+
         :param bzrdir: The bzrdir to create the repository on.
         :param stack_on: A location to stack on
         :param stack_on_pwd: If stack_on is relative, the location it is
@@ -3809,8 +1874,8 @@
                                     possible_transports=[self._bzrdir.root_transport])
             if not self._require_stacking:
                 # We have picked up automatic stacking somewhere.
-                note('Using default stacking branch %s at %s', self._stack_on,
-                    self._stack_on_pwd)
+                note(gettext('Using default stacking branch {0} at {1}').format(
+                    self._stack_on, self._stack_on_pwd))
         repository = self._bzrdir.create_repository(shared=shared)
         self._add_fallback(repository,
                            possible_transports=[self._bzrdir.transport])
@@ -3845,161 +1910,172 @@
         return self._repository, False
 
 
-# Please register new formats after old formats so that formats
-# appear in chronological order and format descriptions can build
-# on previous ones.
-format_registry = BzrDirFormatRegistry()
-# The pre-0.8 formats have their repository format network name registered in
-# repository.py. MetaDir formats have their repository format network name
-# inferred from their disk format string.
-format_registry.register('weave', BzrDirFormat6,
-    'Pre-0.8 format.  Slower than knit and does not'
-    ' support checkouts or shared repositories.',
-    hidden=True,
-    deprecated=True)
-format_registry.register_metadir('metaweave',
-    'bzrlib.repofmt.weaverepo.RepositoryFormat7',
-    'Transitional format in 0.8.  Slower than knit.',
-    branch_format='bzrlib.branch.BzrBranchFormat5',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat3',
-    hidden=True,
-    deprecated=True)
-format_registry.register_metadir('knit',
+def register_metadir(registry, key,
+         repository_format, help, native=True, deprecated=False,
+         branch_format=None,
+         tree_format=None,
+         hidden=False,
+         experimental=False,
+         alias=False, bzrdir_format=None):
+    """Register a metadir subformat.
+
+    These all use a meta bzrdir, but can be parameterized by the
+    Repository/Branch/WorkingTreeformats.
+
+    :param repository_format: The fully-qualified repository format class
+        name as a string.
+    :param branch_format: Fully-qualified branch format class name as
+        a string.
+    :param tree_format: Fully-qualified tree format class name as
+        a string.
+    """
+    if bzrdir_format is None:
+        bzrdir_format = BzrDirMetaFormat1
+    # This should be expanded to support setting WorkingTree and Branch
+    # formats, once the API supports that.
+    def _load(full_name):
+        mod_name, factory_name = full_name.rsplit('.', 1)
+        try:
+            factory = pyutils.get_named_object(mod_name, factory_name)
+        except ImportError, e:
+            raise ImportError('failed to load %s: %s' % (full_name, e))
+        except AttributeError:
+            raise AttributeError('no factory %s in module %r'
+                % (full_name, sys.modules[mod_name]))
+        return factory()
+
+    def helper():
+        bd = bzrdir_format()
+        if branch_format is not None:
+            bd.set_branch_format(_load(branch_format))
+        if tree_format is not None:
+            bd.workingtree_format = _load(tree_format)
+        if repository_format is not None:
+            bd.repository_format = _load(repository_format)
+        return bd
+    registry.register(key, helper, help, native, deprecated, hidden,
+        experimental, alias)
+
+register_metadir(controldir.format_registry, 'knit',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
     'Format using knits.  Recommended for interoperation with bzr <= 0.14.',
     branch_format='bzrlib.branch.BzrBranchFormat5',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat3',
+    tree_format='bzrlib.workingtree_3.WorkingTreeFormat3',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('dirstate',
+register_metadir(controldir.format_registry, 'dirstate',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
     help='New in 0.15: Fast local operations. Compatible with bzr 0.8 and '
         'above when accessed over the network.',
     branch_format='bzrlib.branch.BzrBranchFormat5',
-    # this uses bzrlib.workingtree.WorkingTreeFormat4 because importing
-    # directly from workingtree_4 triggers a circular import.
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('dirstate-tags',
+register_metadir(controldir.format_registry, 'dirstate-tags',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
     help='New in 0.15: Fast local operations and improved scaling for '
         'network operations. Additionally adds support for tags.'
         ' Incompatible with bzr < 0.15.',
     branch_format='bzrlib.branch.BzrBranchFormat6',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('rich-root',
+register_metadir(controldir.format_registry, 'rich-root',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit4',
     help='New in 1.0.  Better handling of tree roots.  Incompatible with'
         ' bzr < 1.0.',
     branch_format='bzrlib.branch.BzrBranchFormat6',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
-format_registry.register_metadir('dirstate-with-subtree',
+register_metadir(controldir.format_registry, 'dirstate-with-subtree',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit3',
     help='New in 0.15: Fast local operations and improved scaling for '
         'network operations. Additionally adds support for versioning nested '
         'bzr branches. Incompatible with bzr < 0.15.',
     branch_format='bzrlib.branch.BzrBranchFormat6',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     experimental=True,
     hidden=True,
     )
-format_registry.register_metadir('pack-0.92',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack1',
+register_metadir(controldir.format_registry, 'pack-0.92',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack1',
     help='New in 0.92: Pack-based format with data compatible with '
         'dirstate-tags format repositories. Interoperates with '
         'bzr repositories before 0.92 but cannot be read by bzr < 0.92. '
         ,
     branch_format='bzrlib.branch.BzrBranchFormat6',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     )
-format_registry.register_metadir('pack-0.92-subtree',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack3',
+register_metadir(controldir.format_registry, 'pack-0.92-subtree',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack3',
     help='New in 0.92: Pack-based format with data compatible with '
         'dirstate-with-subtree format repositories. Interoperates with '
         'bzr repositories before 0.92 but cannot be read by bzr < 0.92. '
         ,
     branch_format='bzrlib.branch.BzrBranchFormat6',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     experimental=True,
     )
-format_registry.register_metadir('rich-root-pack',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack4',
+register_metadir(controldir.format_registry, 'rich-root-pack',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack4',
     help='New in 1.0: A variant of pack-0.92 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
     branch_format='bzrlib.branch.BzrBranchFormat6',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.6',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack5',
+register_metadir(controldir.format_registry, '1.6',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack5',
     help='A format that allows a branch to indicate that there is another '
          '(stacked) repository that should be used to access data that is '
          'not present locally.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.6.1-rich-root',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack5RichRoot',
+register_metadir(controldir.format_registry, '1.6.1-rich-root',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack5RichRoot',
     help='A variant of 1.6 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.9',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6',
+register_metadir(controldir.format_registry, '1.9',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6',
     help='A repository format using B+tree indexes. These indexes '
          'are smaller in size, have smarter caching and provide faster '
          'performance for most operations.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.9-rich-root',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6RichRoot',
+register_metadir(controldir.format_registry, '1.9-rich-root',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
     help='A variant of 1.9 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat4',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     )
-format_registry.register_metadir('1.14',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6',
+register_metadir(controldir.format_registry, '1.14',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6',
     help='A working-tree format that supports content filtering.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat5',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat5',
     )
-format_registry.register_metadir('1.14-rich-root',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatKnitPack6RichRoot',
+register_metadir(controldir.format_registry, '1.14-rich-root',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
     help='A variant of 1.14 that supports rich-root data '
          '(needed for bzr-svn and bzr-git).',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat5',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat5',
     )
 # The following un-numbered 'development' formats should always just be aliases.
-format_registry.register_metadir('development-rich-root',
-    'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK1',
-    help='Current development format. Supports rich roots. Can convert data '
-        'to and from rich-root-pack (and anything compatible with '
-        'rich-root-pack) format repositories. Repositories and branches in '
-        'this format can only be read by bzr.dev. Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    experimental=True,
-    alias=True,
-    hidden=True,
-    )
-format_registry.register_metadir('development-subtree',
-    'bzrlib.repofmt.pack_repo.RepositoryFormatPackDevelopment2Subtree',
+register_metadir(controldir.format_registry, 'development-subtree',
+    'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2aSubtree',
     help='Current development format, subtree variant. Can convert data to and '
         'from pack-0.92-subtree (and anything compatible with '
         'pack-0.92-subtree) format repositories. Repositories and branches in '
@@ -4007,40 +2083,42 @@
         'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
         'before use.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
     experimental=True,
     hidden=True,
     alias=False, # Restore to being an alias when an actual development subtree format is added
                  # This current non-alias status is simply because we did not introduce a
                  # chk based subtree format.
     )
+register_metadir(controldir.format_registry, 'development5-subtree',
+    'bzrlib.repofmt.knitpack_repo.RepositoryFormatPackDevelopment2Subtree',
+    help='Development format, subtree variant. Can convert data to and '
+        'from pack-0.92-subtree (and anything compatible with '
+        'pack-0.92-subtree) format repositories. Repositories and branches in '
+        'this format can only be read by bzr.dev. Please read '
+        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
+        'before use.',
+    branch_format='bzrlib.branch.BzrBranchFormat7',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
+    experimental=True,
+    hidden=True,
+    alias=False,
+    )
+
+register_metadir(controldir.format_registry, 'development-colo',
+    'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
+    help='The 2a format with experimental support for colocated branches.\n',
+    branch_format='bzrlib.branch.BzrBranchFormat7',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
+    experimental=False,
+    bzrdir_format=BzrDirMetaFormat1Colo,
+    )
+
 
 # And the development formats above will have aliased one of the following:
-format_registry.register_metadir('development6-rich-root',
-    'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK1',
-    help='pack-1.9 with 255-way hashed CHK inv, group compress, rich roots '
-        'Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    hidden=True,
-    experimental=True,
-    )
-
-format_registry.register_metadir('development7-rich-root',
-    'bzrlib.repofmt.groupcompress_repo.RepositoryFormatCHK2',
-    help='pack-1.9 with 255-way hashed CHK inv, bencode revision, group compress, '
-        'rich roots. Please read '
-        'http://doc.bazaar.canonical.com/latest/developers/development-repo.html '
-        'before use.',
-    branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    hidden=True,
-    experimental=True,
-    )
-
-format_registry.register_metadir('2a',
+
+# Finally, the current format.
+register_metadir(controldir.format_registry, '2a',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
     help='First format for bzr 2.0 series.\n'
         'Uses group-compress storage.\n'
@@ -4048,19 +2126,25 @@
         # 'storage in packs, 255-way hashed CHK inventory, bencode revision, group compress, '
         # 'rich roots. Supported by bzr 1.16 and later.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
-    experimental=True,
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
+    experimental=False,
     )
 
 # The following format should be an alias for the rich root equivalent 
 # of the default format
-format_registry.register_metadir('default-rich-root',
+register_metadir(controldir.format_registry, 'default-rich-root',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
     branch_format='bzrlib.branch.BzrBranchFormat7',
-    tree_format='bzrlib.workingtree.WorkingTreeFormat6',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
     alias=True,
     hidden=True,
     help='Same as 2a.')
 
 # The current format that is made on 'bzr init'.
-format_registry.set_default('2a')
+format_name = config.GlobalStack().get('default_format')
+controldir.format_registry.set_default(format_name)
+
+# XXX 2010-08-20 JRV: There is still a lot of code relying on
+# bzrlib.bzrdir.format_registry existing. When BzrDir.create/BzrDir.open/etc
+# get changed to ControlDir.create/ControlDir.open/etc this should be removed.
+format_registry = controldir.format_registry

=== added file 'bzrlib/cethread.py'
--- bzrlib/cethread.py	1970-01-01 00:00:00 +0000
+++ bzrlib/cethread.py	2011-10-07 14:51:42 +0000
@@ -0,0 +1,154 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import sys
+import threading
+
+
+class CatchingExceptionThread(threading.Thread):
+    """A thread that keeps track of exceptions.
+
+    If an exception occurs during the thread execution, it's caught and
+    re-raised when the thread is joined().
+    """
+
+    def __init__(self, *args, **kwargs):
+        # There are cases where the calling thread must wait, yet, if an
+        # exception occurs, the event should be set so the caller is not
+        # blocked. The main example is a calling thread that want to wait for
+        # the called thread to be in a given state before continuing.
+        try:
+            sync_event = kwargs.pop('sync_event')
+        except KeyError:
+            # If the caller didn't pass a specific event, create our own
+            sync_event = threading.Event()
+        super(CatchingExceptionThread, self).__init__(*args, **kwargs)
+        self.set_sync_event(sync_event)
+        self.exception = None
+        self.ignored_exceptions = None # see set_ignored_exceptions
+        self.lock = threading.Lock()
+
+    # compatibility thunk for python-2.4 and python-2.5...
+    if sys.version_info < (2, 6):
+        name = property(threading.Thread.getName, threading.Thread.setName)
+
+    def set_sync_event(self, event):
+        """Set the ``sync_event`` event used to synchronize exception catching.
+
+        When the thread uses an event to synchronize itself with another thread
+        (setting it when the other thread can wake up from a ``wait`` call),
+        the event must be set after catching an exception or the other thread
+        will hang.
+
+        Some threads require multiple events and should set the relevant one
+        when appropriate.
+
+        Note that the event should be initially cleared so the caller can
+        wait() on him and be released when the thread set the event.
+
+        Also note that the thread can use multiple events, setting them as it
+        progress, while the caller can chose to wait on any of them. What
+        matters is that there is always one event set so that the caller is
+        always released when an exception is caught. Re-using the same event is
+        therefore risky as the thread itself has no idea about which event the
+        caller is waiting on. If the caller has already been released then a
+        cleared event won't guarantee that the caller is still waiting on it.
+        """
+        self.sync_event = event
+
+    def switch_and_set(self, new):
+        """Switch to a new ``sync_event`` and set the current one.
+
+        Using this method protects against race conditions while setting a new
+        ``sync_event``.
+
+        Note that this allows a caller to wait either on the old or the new
+        event depending on whether it wants a fine control on what is happening
+        inside a thread.
+
+        :param new: The event that will become ``sync_event``
+        """
+        cur = self.sync_event
+        self.lock.acquire()
+        try: # Always release the lock
+            try:
+                self.set_sync_event(new)
+                # From now on, any exception will be synced with the new event
+            except:
+                # Unlucky, we couldn't set the new sync event, try restoring a
+                # safe state
+                self.set_sync_event(cur)
+                raise
+            # Setting the current ``sync_event`` will release callers waiting
+            # on it, note that it will also be set in run() if an exception is
+            # raised
+            cur.set()
+        finally:
+            self.lock.release()
+
+    def set_ignored_exceptions(self, ignored):
+        """Declare which exceptions will be ignored.
+
+        :param ignored: Can be either:
+        
+           - None: all exceptions will be raised,
+           - an exception class: the instances of this class will be ignored,
+           - a tuple of exception classes: the instances of any class of the
+             list will be ignored,
+           - a callable: that will be passed the exception object
+             and should return True if the exception should be ignored
+        """
+        if ignored is None:
+            self.ignored_exceptions = None
+        elif isinstance(ignored, (Exception, tuple)):
+            self.ignored_exceptions = lambda e: isinstance(e, ignored)
+        else:
+            self.ignored_exceptions = ignored
+
+    def run(self):
+        """Overrides Thread.run to capture any exception."""
+        self.sync_event.clear()
+        try:
+            try:
+                super(CatchingExceptionThread, self).run()
+            except:
+                self.exception = sys.exc_info()
+        finally:
+            # Make sure the calling thread is released
+            self.sync_event.set()
+
+
+    def join(self, timeout=None):
+        """Overrides Thread.join to raise any exception caught.
+
+        Calling join(timeout=0) will raise the caught exception or return None
+        if the thread is still alive.
+        """
+        super(CatchingExceptionThread, self).join(timeout)
+        if self.exception is not None:
+            exc_class, exc_value, exc_tb = self.exception
+            self.exception = None # The exception should be raised only once
+            if (self.ignored_exceptions is None
+                or not self.ignored_exceptions(exc_value)):
+                # Raise non ignored exceptions
+                raise exc_class, exc_value, exc_tb
+
+    def pending_exception(self):
+        """Raise the caught exception.
+
+        This does nothing if no exception occurred.
+        """
+        self.join(timeout=0)

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2010-04-30 11:35:43 +0000
+++ bzrlib/check.py	2011-10-11 10:10:23 +0000
@@ -39,6 +39,7 @@
 so that when the dependent object is checked, matches can be pulled out and
 evaluated in-line rather than re-reading the same data many times.
 check_refs are tuples (kind, value). Currently defined kinds are:
+
 * 'trees', where value is a revid and the looked up objects are revision trees.
 * 'lefthand-distance', where value is a revid and the looked up objects are the
   distance along the lefthand path to NULL for that revid.
@@ -46,18 +47,30 @@
   indicating that the revision was found/not found.
 """
 
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    ui,
+    )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
+from bzrlib.controldir import ControlDir
 from bzrlib.revision import NULL_REVISION
-from bzrlib.symbol_versioning import deprecated_function, deprecated_in
 from bzrlib.trace import note
-import bzrlib.ui
 from bzrlib.workingtree import WorkingTree
+from bzrlib.i18n import gettext
 
 class Check(object):
     """Check a repository"""
 
+    def __init__(self, repository, check_repo=True):
+        self.repository = repository
+
+    def report_results(self, verbose):
+        raise NotImplementedError(self.report_results)
+
+
+class VersionedFileCheck(Check):
+    """Check a versioned file repository"""
+
     # The Check object interacts with InventoryEntry.check, etc.
 
     def __init__(self, repository, check_repo=True):
@@ -88,19 +101,19 @@
         if callback_refs is None:
             callback_refs = {}
         self.repository.lock_read()
-        self.progress = bzrlib.ui.ui_factory.nested_progress_bar()
+        self.progress = ui.ui_factory.nested_progress_bar()
         try:
-            self.progress.update('check', 0, 4)
+            self.progress.update(gettext('check'), 0, 4)
             if self.check_repo:
-                self.progress.update('checking revisions', 0)
+                self.progress.update(gettext('checking revisions'), 0)
                 self.check_revisions()
-                self.progress.update('checking commit contents', 1)
+                self.progress.update(gettext('checking commit contents'), 1)
                 self.repository._check_inventories(self)
-                self.progress.update('checking file graphs', 2)
+                self.progress.update(gettext('checking file graphs'), 2)
                 # check_weaves is done after the revision scan so that
                 # revision index is known to be valid.
                 self.check_weaves()
-            self.progress.update('checking branches and trees', 3)
+            self.progress.update(gettext('checking branches and trees'), 3)
             if callback_refs:
                 repo = self.repository
                 # calculate all refs, and callback the objects requesting them.
@@ -170,7 +183,7 @@
         # - we can fill out existence flags at this point
         # - we can read the revision inventory sha at this point
         # - we can check properties and serialisers etc.
-        if not self.repository.revision_graph_can_have_wrong_parents():
+        if not self.repository._format.revision_graph_can_have_wrong_parents:
             # The check against the index isn't needed.
             self.revs_with_bad_parents_in_index = None
             for thing in revision_iterator:
@@ -187,56 +200,57 @@
             result.report_results(verbose)
 
     def _report_repo_results(self, verbose):
-        note('checked repository %s format %s',
+        note(gettext('checked repository {0} format {1}').format(
             self.repository.user_url,
-            self.repository._format)
-        note('%6d revisions', self.checked_rev_cnt)
-        note('%6d file-ids', len(self.checked_weaves))
+            self.repository._format))
+        note(gettext('%6d revisions'), self.checked_rev_cnt)
+        note(gettext('%6d file-ids'), len(self.checked_weaves))
         if verbose:
-            note('%6d unreferenced text versions',
+            note(gettext('%6d unreferenced text versions'),
                 len(self.unreferenced_versions))
         if verbose and len(self.unreferenced_versions):
                 for file_id, revision_id in self.unreferenced_versions:
-                    note('unreferenced version: {%s} in %s', revision_id,
-                        file_id)
+                    note(gettext('unreferenced version: {{{0}}} in {1}').format(revision_id,
+                        file_id))
         if self.missing_inventory_sha_cnt:
-            note('%6d revisions are missing inventory_sha1',
+            note(gettext('%6d revisions are missing inventory_sha1'),
                  self.missing_inventory_sha_cnt)
         if self.missing_revision_cnt:
-            note('%6d revisions are mentioned but not present',
+            note(gettext('%6d revisions are mentioned but not present'),
                  self.missing_revision_cnt)
         if len(self.ghosts):
-            note('%6d ghost revisions', len(self.ghosts))
+            note(gettext('%6d ghost revisions'), len(self.ghosts))
             if verbose:
                 for ghost in self.ghosts:
                     note('      %s', ghost)
         if len(self.missing_parent_links):
-            note('%6d revisions missing parents in ancestry',
+            note(gettext('%6d revisions missing parents in ancestry'),
                  len(self.missing_parent_links))
             if verbose:
                 for link, linkers in self.missing_parent_links.items():
-                    note('      %s should be in the ancestry for:', link)
+                    note(gettext('      %s should be in the ancestry for:'), link)
                     for linker in linkers:
                         note('       * %s', linker)
         if len(self.inconsistent_parents):
-            note('%6d inconsistent parents', len(self.inconsistent_parents))
+            note(gettext('%6d inconsistent parents'), len(self.inconsistent_parents))
             if verbose:
                 for info in self.inconsistent_parents:
                     revision_id, file_id, found_parents, correct_parents = info
-                    note('      * %s version %s has parents %r '
-                         'but should have %r'
-                         % (file_id, revision_id, found_parents,
+                    note(gettext('      * {0} version {1} has parents {2!r} '
+                         'but should have {3!r}').format(
+                         file_id, revision_id, found_parents,
                              correct_parents))
         if self.revs_with_bad_parents_in_index:
-            note('%6d revisions have incorrect parents in the revision index',
+            note(gettext(
+                 '%6d revisions have incorrect parents in the revision index'),
                  len(self.revs_with_bad_parents_in_index))
             if verbose:
                 for item in self.revs_with_bad_parents_in_index:
                     revision_id, index_parents, actual_parents = item
-                    note(
-                        '       %s has wrong parents in index: '
-                        '%r should be %r',
-                        revision_id, index_parents, actual_parents)
+                    note(gettext(
+                        '       {0} has wrong parents in index: '
+                        '{1!r} should be {2!r}').format(
+                        revision_id, index_parents, actual_parents))
         for item in self._report_items:
             note(item)
 
@@ -247,8 +261,8 @@
         :param rev: A revision or None to indicate a missing revision.
         """
         if rev.revision_id != rev_id:
-            self._report_items.append(
-                'Mismatched internal revid {%s} and index revid {%s}' % (
+            self._report_items.append(gettext(
+                'Mismatched internal revid {{{0}}} and index revid {{{1}}}').format(
                 rev.revision_id, rev_id))
             rev_id = rev.revision_id
         # Check this revision tree etc, and count as seen when we encounter a
@@ -277,8 +291,8 @@
         existing = self.pending_keys.get(key)
         if existing:
             if sha1 != existing[1]:
-                self._report_items.append('Multiple expected sha1s for %s. {%s}'
-                    ' expects {%s}, {%s} expects {%s}', (
+                self._report_items.append(gettext('Multiple expected sha1s for {0}. {{{1}}}'
+                    ' expects {{{2}}}, {{{3}}} expects {{{4}}}').format(
                     key, referer, sha1, existing[1], existing[0]))
         else:
             self.pending_keys[key] = (kind, sha1, referer)
@@ -287,7 +301,7 @@
         """Check all the weaves we can get our hands on.
         """
         weave_ids = []
-        storebar = bzrlib.ui.ui_factory.nested_progress_bar()
+        storebar = ui.ui_factory.nested_progress_bar()
         try:
             self._check_weaves(storebar)
         finally:
@@ -328,39 +342,6 @@
             self.text_key_references[key] = True
 
 
-@deprecated_function(deprecated_in((1,6,0)))
-def check(branch, verbose):
-    """Run consistency checks on a branch.
-
-    Results are reported through logging.
-
-    Deprecated in 1.6.  Please use check_dwim instead.
-
-    :raise BzrCheckError: if there's a consistency error.
-    """
-    check_branch(branch, verbose)
-
-
-@deprecated_function(deprecated_in((1,16,0)))
-def check_branch(branch, verbose):
-    """Run consistency checks on a branch.
-
-    Results are reported through logging.
-
-    :raise BzrCheckError: if there's a consistency error.
-    """
-    branch.lock_read()
-    try:
-        needed_refs = {}
-        for ref in branch._get_check_refs():
-            needed_refs.setdefault(ref, []).append(branch)
-        result = branch.repository.check([branch.last_revision()], needed_refs)
-        branch_result = result.other_results[0]
-    finally:
-        branch.unlock()
-    branch_result.report_results(verbose)
-
-
 def scan_branch(branch, needed_refs, to_unlock):
     """Scan a branch for refs.
 
@@ -368,7 +349,7 @@
     :param needed_refs: Refs we are accumulating.
     :param to_unlock: The unlock list accumulating.
     """
-    note("Checking branch at '%s'." % (branch.base,))
+    note(gettext("Checking branch at '%s'.") % (branch.base,))
     branch.lock_read()
     to_unlock.append(branch)
     branch_refs = branch._get_check_refs()
@@ -388,7 +369,7 @@
     """
     if base_tree is not None and tree.basedir == base_tree.basedir:
         return
-    note("Checking working tree at '%s'." % (tree.basedir,))
+    note(gettext("Checking working tree at '%s'.") % (tree.basedir,))
     tree.lock_read()
     to_unlock.append(tree)
     tree_refs = tree._get_check_refs()
@@ -405,7 +386,7 @@
     """
     try:
         base_tree, branch, repo, relpath = \
-                        BzrDir.open_containing_tree_branch_or_repository(path)
+                        ControlDir.open_containing_tree_branch_or_repository(path)
     except errors.NotBranchError:
         base_tree = branch = repo = None
 
@@ -441,23 +422,23 @@
                     if do_branch:
                         scan_branch(branch, needed_refs, to_unlock)
             if do_branch and not branches:
-                note("No branch found at specified location.")
+                note(gettext("No branch found at specified location."))
             if do_tree and base_tree is None and not saw_tree:
-                note("No working tree found at specified location.")
+                note(gettext("No working tree found at specified location."))
             if do_repo or do_branch or do_tree:
                 if do_repo:
-                    note("Checking repository at '%s'."
+                    note(gettext("Checking repository at '%s'.")
                          % (repo.user_url,))
                 result = repo.check(None, callback_refs=needed_refs,
                     check_repo=do_repo)
                 result.report_results(verbose)
         else:
             if do_tree:
-                note("No working tree found at specified location.")
+                note(gettext("No working tree found at specified location."))
             if do_branch:
-                note("No branch found at specified location.")
+                note(gettext("No branch found at specified location."))
             if do_repo:
-                note("No repository found at specified location.")
+                note(gettext("No repository found at specified location."))
     finally:
         for thing in to_unlock:
             thing.unlock()

=== modified file 'bzrlib/chk_map.py'
--- bzrlib/chk_map.py	2010-07-21 09:58:42 +0000
+++ bzrlib/chk_map.py	2011-05-20 14:46:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,10 +44,10 @@
 lazy_import.lazy_import(globals(), """
 from bzrlib import (
     errors,
-    versionedfile,
     )
 """)
 from bzrlib import (
+    errors,
     lru_cache,
     osutils,
     registry,
@@ -1369,7 +1369,7 @@
         return self._search_prefix
 
     def unmap(self, store, key, check_remap=True):
-        """Remove key from this node and it's children."""
+        """Remove key from this node and its children."""
         if not len(self._items):
             raise AssertionError("can't unmap in an empty InternalNode.")
         children = [node for node, _

=== modified file 'bzrlib/chk_serializer.py'
--- bzrlib/chk_serializer.py	2010-02-17 17:11:16 +0000
+++ bzrlib/chk_serializer.py	2011-02-19 22:39:03 +0000
@@ -19,10 +19,8 @@
 from bzrlib import (
     bencode,
     cache_utf8,
-    inventory,
     revision as _mod_revision,
-    xml6,
-    xml7,
+    xml8,
     )
 
 
@@ -131,36 +129,7 @@
         return self.read_revision_from_string(f.read())
 
 
-class CHKSerializerSubtree(BEncodeRevisionSerializer1, xml7.Serializer_v7):
-    """A CHKInventory based serializer that supports tree references"""
-
-    supported_kinds = set(['file', 'directory', 'symlink', 'tree-reference'])
-    format_num = '9'
-    revision_format_num = None
-    support_altered_by_hack = False
-
-    def _unpack_entry(self, elt, entry_cache=None, return_from_cache=False):
-        kind = elt.tag
-        if not kind in self.supported_kinds:
-            raise AssertionError('unsupported entry kind %s' % kind)
-        if kind == 'tree-reference':
-            file_id = elt.attrib['file_id']
-            name = elt.attrib['name']
-            parent_id = elt.attrib['parent_id']
-            revision = elt.get('revision')
-            reference_revision = elt.get('reference_revision')
-            return inventory.TreeReference(file_id, name, parent_id, revision,
-                                           reference_revision)
-        else:
-            return xml7.Serializer_v7._unpack_entry(self, elt,
-                entry_cache=entry_cache, return_from_cache=return_from_cache)
-
-    def __init__(self, node_size, search_key_name):
-        self.maximum_size = node_size
-        self.search_key_name = search_key_name
-
-
-class CHKSerializer(xml6.Serializer_v6):
+class CHKSerializer(xml8.Serializer_v8):
     """A CHKInventory based serializer with 'plain' behaviour."""
 
     format_num = '9'

=== modified file 'bzrlib/chunk_writer.py'
--- bzrlib/chunk_writer.py	2009-03-23 14:59:43 +0000
+++ bzrlib/chunk_writer.py	2011-05-19 09:32:38 +0000
@@ -122,14 +122,14 @@
         bytes that did not fit in the chunk.
 
         :return: (compressed_bytes, unused_bytes, num_nulls_needed)
-            compressed_bytes    a list of bytes that were output from the
-                                compressor. If the compressed length was not
-                                exactly chunk_size, the final string will be a
-                                string of all null bytes to pad this to
-                                chunk_size
-            unused_bytes        None, or the last bytes that were added, which
-                                we could not fit.
-            num_nulls_needed    How many nulls are padded at the end
+
+            * compressed_bytes: a list of bytes that were output from the
+              compressor. If the compressed length was not exactly chunk_size,
+              the final string will be a string of all null bytes to pad this
+              to chunk_size
+            * unused_bytes: None, or the last bytes that were added, which we
+              could not fit.
+            * num_nulls_needed: How many nulls are padded at the end
         """
         self.bytes_in = None # Free the data cached so far, we don't need it
         out = self.compressor.flush(Z_FINISH)
@@ -164,10 +164,11 @@
         :param extra_bytes: Optional, if supplied we will add it with
             Z_SYNC_FLUSH
         :return: (bytes_out, bytes_out_len, alt_compressed)
-            bytes_out   is the compressed bytes returned from the compressor
-            bytes_out_len the length of the compressed output
-            compressor  An object with everything packed in so far, and
-                        Z_SYNC_FLUSH called.
+
+            * bytes_out: is the compressed bytes returned from the compressor
+            * bytes_out_len: the length of the compressed output
+            * compressor: An object with everything packed in so far, and
+              Z_SYNC_FLUSH called.
         """
         compressor = zlib.compressobj()
         bytes_out = []

=== modified file 'bzrlib/clean_tree.py'
--- bzrlib/clean_tree.py	2010-05-20 02:57:52 +0000
+++ bzrlib/clean_tree.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,14 +15,19 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+import errno
 import os
 import shutil
 
-from bzrlib import bzrdir, errors
+from bzrlib import (
+    controldir,
+    errors,
+    ui,
+    )
 from bzrlib.osutils import isdir
 from bzrlib.trace import note
 from bzrlib.workingtree import WorkingTree
-
+from bzrlib.i18n import gettext
 
 def is_detritus(subp):
     """Return True if the supplied path is detritus, False otherwise"""
@@ -54,7 +59,7 @@
             ignored=ignored, detritus=detritus))
         deletables = _filter_out_nested_bzrdirs(deletables)
         if len(deletables) == 0:
-            note('Nothing to delete.')
+            note(gettext('Nothing to delete.'))
             return 0
         if not no_prompt:
             for path, subp in deletables:
@@ -80,7 +85,7 @@
         # directory and therefore delete it. (worth to FIXME?)
         if isdir(path):
             try:
-                bzrdir.BzrDir.open(path)
+                controldir.ControlDir.open(path)
             except errors.NotBranchError:
                 result.append((path,subp))
             else:
@@ -93,16 +98,34 @@
 
 def delete_items(deletables, dry_run=False):
     """Delete files in the deletables iterable"""
+    def onerror(function, path, excinfo):
+        """Show warning for errors seen by rmtree.
+        """
+        # Handle only permission error while removing files.
+        # Other errors are re-raised.
+        if function is not os.remove or excinfo[1].errno != errno.EACCES:
+            raise
+        ui.ui_factory.show_warning(gettext('unable to remove %s') % path)
     has_deleted = False
     for path, subp in deletables:
         if not has_deleted:
-            note("deleting paths:")
+            note(gettext("deleting paths:"))
             has_deleted = True
-        note('  ' + subp)
         if not dry_run:
             if isdir(path):
-                shutil.rmtree(path)
+                shutil.rmtree(path, onerror=onerror)
             else:
-                os.unlink(path)
+                try:
+                    os.unlink(path)
+                    note('  ' + subp)
+                except OSError, e:
+                    # We handle only permission error here
+                    if e.errno != errno.EACCES:
+                        raise e
+                    ui.ui_factory.show_warning(gettext(
+                        'unable to remove "{0}": {1}.').format(
+                                                    path, e.strerror))
+        else:
+            note('  ' + subp)
     if not has_deleted:
-        note("No files deleted.")
+        note(gettext("No files deleted."))

=== modified file 'bzrlib/cleanup.py'
--- bzrlib/cleanup.py	2010-04-27 07:29:11 +0000
+++ bzrlib/cleanup.py	2011-05-26 21:02:47 +0000
@@ -188,7 +188,10 @@
                 # but don't propagate them.
                 _run_cleanup(cleanup, *c_args, **kwargs)
         if exc_info is not None:
-            raise exc_info[0], exc_info[1], exc_info[2]
+            try:
+                raise exc_info[0], exc_info[1], exc_info[2]
+            finally:
+                del exc_info
         # No error, so we can return the result
         return result
 

=== added file 'bzrlib/cmd_test_script.py'
--- bzrlib/cmd_test_script.py	1970-01-01 00:00:00 +0000
+++ bzrlib/cmd_test_script.py	2010-11-08 10:53:53 +0000
@@ -0,0 +1,65 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Front-end command for shell-like test scripts.
+
+See doc/developers/testing.txt for more explanations.
+This module should be importable even if testtools aren't available.
+"""
+
+import os
+
+from bzrlib import (
+    commands,
+    option,
+    )
+
+
+class cmd_test_script(commands.Command):
+    """Run a shell-like test from a file."""
+
+    hidden = True
+    takes_args = ['infile']
+    takes_options = [
+        option.Option('null-output',
+                       help='Null command outputs match any output.'),
+        ]
+
+    @commands.display_command
+    def run(self, infile, null_output=False):
+        # local imports to defer testtools dependency
+        from bzrlib import tests
+        from bzrlib.tests.script import TestCaseWithTransportAndScript
+
+        f = open(infile)
+        try:
+            script = f.read()
+        finally:
+            f.close()
+
+        class Test(TestCaseWithTransportAndScript):
+
+            script = None # Set before running
+
+            def test_it(self):
+                self.run_script(script,
+                                null_output_matches_anything=null_output)
+
+        runner = tests.TextTestRunner(stream=self.outf)
+        test = Test('test_it')
+        test.path = os.path.realpath(infile)
+        res = runner.run(test)
+        return len(res.errors) + len(res.failures)

=== modified file 'bzrlib/cmd_version_info.py'
--- bzrlib/cmd_version_info.py	2010-04-02 19:12:58 +0000
+++ bzrlib/cmd_version_info.py	2011-10-06 10:25:48 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,10 +22,10 @@
 from bzrlib import (
     branch,
     errors,
-    ui,
     version_info_formats,
     workingtree,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.commands import Command
@@ -42,9 +42,9 @@
         return version_info_formats.get_builder(format)
     except KeyError:
         formats = version_info_formats.get_builder_formats()
-        raise errors.BzrCommandError('No known version info format %s.'
-                                     ' Supported types are: %s'
-                                     % (format, formats))
+        raise errors.BzrCommandError(gettext('No known version info format {0}.'
+                                     ' Supported types are: {1}').format(
+                                     format, formats))
 
 
 class cmd_version_info(Command):
@@ -83,6 +83,7 @@
                      Option('include-file-revisions',
                             help='Include the last revision for each file.'),
                      Option('template', type=str, help='Template for the output.'),
+                     'revision',
                      ]
     takes_args = ['location?']
 
@@ -90,7 +91,13 @@
 
     def run(self, location=None, format=None,
             all=False, check_clean=False, include_history=False,
-            include_file_revisions=False, template=None):
+            include_file_revisions=False, template=None,
+            revision=None):
+
+        if revision and len(revision) > 1:
+            raise errors.BzrCommandError(
+                gettext('bzr version-info --revision takes exactly'
+                        ' one revision specifier'))
 
         if location is None:
             location = '.'
@@ -109,11 +116,16 @@
         if all or template:
             include_history = True
             check_clean = True
-            include_file_revisions=True
+            include_file_revisions = True
+
+        if revision is not None:
+            revision_id = revision[0].as_revision_id(b)
+        else:
+            revision_id = None
 
         builder = format(b, working_tree=wt,
                 check_for_clean=check_clean,
                 include_revision_history=include_history,
                 include_file_revisions=include_file_revisions,
-                template=template)
-        builder.generate(ui.ui_factory.make_output_stream())
+                template=template, revision_id=revision_id)
+        builder.generate(self.outf)

=== modified file 'bzrlib/cmdline.py'
--- bzrlib/cmdline.py	2010-03-01 09:02:18 +0000
+++ bzrlib/cmdline.py	2011-02-07 01:39:42 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Unicode-compatible command-line splitter for all platforms."""
+"""Unicode-compatible command-line splitter for all platforms.
+
+The user-visible behaviour of this module is described in
+configuring_bazaar.txt.
+"""
 
 import re
 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2010-06-18 11:07:07 +0000
+++ bzrlib/commands.py	2011-09-29 15:50:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,10 +27,8 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import codecs
 import errno
 import threading
-from warnings import warn
 
 import bzrlib
 from bzrlib import (
@@ -38,15 +36,16 @@
     cmdline,
     debug,
     errors,
+    i18n,
     option,
     osutils,
     trace,
     ui,
-    win32utils,
     )
 """)
 
-from bzrlib.hooks import HookPoint, Hooks
+from bzrlib.hooks import Hooks
+from bzrlib.i18n import gettext
 # Compatibility - Option used to be in commands.
 from bzrlib.option import Option
 from bzrlib.plugin import disable_plugins, load_plugins
@@ -165,16 +164,6 @@
     return cmd[4:].replace('_','-')
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def _builtin_commands():
-    """Return a dict of {name: cmd_class} for builtin commands.
-
-    :deprecated: Use the builtin_command_registry registry instead
-    """
-    # return dict(name: cmd_class)
-    return dict(builtin_command_registry.items())
-
-
 def _register_builtin_commands():
     if builtin_command_registry.keys():
         # only load once
@@ -222,6 +211,7 @@
     Use of all_command_names() is encouraged rather than builtin_command_names
     and/or plugin_command_names.
     """
+    _register_builtin_commands()
     return builtin_command_registry.keys()
 
 
@@ -239,7 +229,7 @@
     try:
         return _get_cmd_object(cmd_name, plugins_override)
     except KeyError:
-        raise errors.BzrCommandError('unknown command "%s"' % cmd_name)
+        raise errors.BzrCommandError(gettext('unknown command "%s"') % cmd_name)
 
 
 def _get_cmd_object(cmd_name, plugins_override=True, check_missing=True):
@@ -275,6 +265,8 @@
     # Allow plugins to extend commands
     for hook in Command.hooks['extend_command']:
         hook(cmd)
+    if getattr(cmd, 'invoked_as', None) is None:
+        cmd.invoked_as = cmd_name
     return cmd
 
 
@@ -357,57 +349,58 @@
     summary, then a complete description of the command.  A grammar
     description will be inserted.
 
-    aliases
-        Other accepted names for this command.
-
-    takes_args
-        List of argument forms, marked with whether they are optional,
-        repeated, etc.
-
-                Examples:
-
-                ['to_location', 'from_branch?', 'file*']
-
-                'to_location' is required
-                'from_branch' is optional
-                'file' can be specified 0 or more times
-
-    takes_options
-        List of options that may be given for this command.  These can
-        be either strings, referring to globally-defined options,
-        or option objects.  Retrieve through options().
-
-    hidden
-        If true, this command isn't advertised.  This is typically
+    :cvar aliases: Other accepted names for this command.
+
+    :cvar takes_args: List of argument forms, marked with whether they are
+        optional, repeated, etc.  Examples::
+
+            ['to_location', 'from_branch?', 'file*']
+
+        * 'to_location' is required
+        * 'from_branch' is optional
+        * 'file' can be specified 0 or more times
+
+    :cvar takes_options: List of options that may be given for this command.
+        These can be either strings, referring to globally-defined options, or
+        option objects.  Retrieve through options().
+
+    :cvar hidden: If true, this command isn't advertised.  This is typically
         for commands intended for expert users.
 
-    encoding_type
-        Command objects will get a 'outf' attribute, which has been
-        setup to properly handle encoding of unicode strings.
-        encoding_type determines what will happen when characters cannot
-        be encoded
-            strict - abort if we cannot decode
-            replace - put in a bogus character (typically '?')
-            exact - do not encode sys.stdout
-
-            NOTE: by default on Windows, sys.stdout is opened as a text
-            stream, therefore LF line-endings are converted to CRLF.
-            When a command uses encoding_type = 'exact', then
-            sys.stdout is forced to be a binary stream, and line-endings
-            will not mangled.
+    :cvar encoding_type: Command objects will get a 'outf' attribute, which has
+        been setup to properly handle encoding of unicode strings.
+        encoding_type determines what will happen when characters cannot be
+        encoded:
+
+        * strict - abort if we cannot decode
+        * replace - put in a bogus character (typically '?')
+        * exact - do not encode sys.stdout
+
+        NOTE: by default on Windows, sys.stdout is opened as a text stream,
+        therefore LF line-endings are converted to CRLF.  When a command uses
+        encoding_type = 'exact', then sys.stdout is forced to be a binary
+        stream, and line-endings will not mangled.
+
+    :cvar invoked_as:
+        A string indicating the real name under which this command was
+        invoked, before expansion of aliases.
+        (This may be None if the command was constructed and run in-process.)
 
     :cvar hooks: An instance of CommandHooks.
-    :ivar __doc__: The help shown by 'bzr help command' for this command.
+
+    :cvar __doc__: The help shown by 'bzr help command' for this command.
         This is set by assigning explicitly to __doc__ so that -OO can
         be used::
 
-        class Foo(Command):
-            __doc__ = "My help goes here"
+            class Foo(Command):
+                __doc__ = "My help goes here"
     """
     aliases = []
     takes_args = []
     takes_options = []
     encoding_type = 'strict'
+    invoked_as = None
+    l10n = True
 
     hidden = False
 
@@ -439,19 +432,6 @@
         """
         self._operation.cleanup_now()
 
-    @deprecated_method(deprecated_in((2, 1, 0)))
-    def _maybe_expand_globs(self, file_list):
-        """Glob expand file_list if the platform does not do that itself.
-
-        Not used anymore, now that the bzr command-line parser globs on
-        Windows.
-
-        :return: A possibly empty list of unicode paths.
-
-        Introduced in bzrlib 0.18.
-        """
-        return file_list
-
     def _usage(self):
         """Return single-line grammar for this command.
 
@@ -485,9 +465,18 @@
             usage help (e.g. Purpose, Usage, Options) with a
             message explaining how to obtain full help.
         """
+        if self.l10n:
+            i18n.install()  # Install i18n only for get_help_text for now.
         doc = self.help()
-        if not doc:
-            doc = "No help for this command."
+        if doc:
+            # Note: If self.gettext() translates ':Usage:\n', the section will
+            # be shown after "Description" section and we don't want to
+            # translate the usage string.
+            # Though, bzr export-pot don't exports :Usage: section and it must
+            # not be translated.
+            doc = self.gettext(doc)
+        else:
+            doc = gettext("No help for this command.")
 
         # Extract the summary (purpose) and sections out from the text
         purpose,sections,order = self._get_help_parts(doc)
@@ -500,11 +489,11 @@
 
         # The header is the purpose and usage
         result = ""
-        result += ':Purpose: %s\n' % purpose
+        result += gettext(':Purpose: %s\n') % (purpose,)
         if usage.find('\n') >= 0:
-            result += ':Usage:\n%s\n' % usage
+            result += gettext(':Usage:\n%s\n') % (usage,)
         else:
-            result += ':Usage:   %s\n' % usage
+            result += gettext(':Usage:   %s\n') % (usage,)
         result += '\n'
 
         # Add the options
@@ -512,19 +501,18 @@
         # XXX: optparse implicitly rewraps the help, and not always perfectly,
         # so we get <https://bugs.launchpad.net/bzr/+bug/249908>.  -- mbp
         # 20090319
-        options = option.get_optparser(self.options()).format_option_help()
-        # XXX: According to the spec, ReST option lists actually don't support 
-        # options like --1.9 so that causes syntax errors (in Sphinx at least).
-        # As that pattern always appears in the commands that break, we trap
-        # on that and then format that block of 'format' options as a literal
-        # block.
-        if not plain and options.find('  --1.9  ') != -1:
+        parser = option.get_optparser(self.options())
+        options = parser.format_option_help()
+        # FIXME: According to the spec, ReST option lists actually don't
+        # support options like --1.14 so that causes syntax errors (in Sphinx
+        # at least).  As that pattern always appears in the commands that
+        # break, we trap on that and then format that block of 'format' options
+        # as a literal block. We use the most recent format still listed so we
+        # don't have to do that too often -- vila 20110514
+        if not plain and options.find('  --1.14  ') != -1:
             options = options.replace(' format:\n', ' format::\n\n', 1)
         if options.startswith('Options:'):
-            result += ':' + options
-        elif options.startswith('options:'):
-            # Python 2.4 version of optparse
-            result += ':Options:' + options[len('options:'):]
+            result += gettext(':Options:%s') % (options[len('options:'):],)
         else:
             result += options
         result += '\n'
@@ -535,26 +523,26 @@
             if sections.has_key(None):
                 text = sections.pop(None)
                 text = '\n  '.join(text.splitlines())
-                result += ':%s:\n  %s\n\n' % ('Description',text)
+                result += gettext(':Description:\n  %s\n\n') % (text,)
 
             # Add the custom sections (e.g. Examples). Note that there's no need
             # to indent these as they must be indented already in the source.
             if sections:
                 for label in order:
-                    if sections.has_key(label):
-                        result += ':%s:\n%s\n' % (label,sections[label])
+                    if label in sections:
+                        result += ':%s:\n%s\n' % (label, sections[label])
                 result += '\n'
         else:
-            result += ("See bzr help %s for more details and examples.\n\n"
+            result += (gettext("See bzr help %s for more details and examples.\n\n")
                 % self.name())
 
         # Add the aliases, source (plug-in) and see also links, if any
         if self.aliases:
-            result += ':Aliases:  '
+            result += gettext(':Aliases:  ')
             result += ', '.join(self.aliases) + '\n'
         plugin_name = self.plugin_name()
         if plugin_name is not None:
-            result += ':From:     plugin "%s"\n' % plugin_name
+            result += gettext(':From:     plugin "%s"\n') % plugin_name
         see_also = self.get_see_also(additional_see_also)
         if see_also:
             if not plain and see_also_as_links:
@@ -566,11 +554,11 @@
                         see_also_links.append(item)
                     else:
                         # Use a Sphinx link for this entry
-                        link_text = ":doc:`%s <%s-help>`" % (item, item)
+                        link_text = gettext(":doc:`{0} <{1}-help>`").format(
+                                                                    item, item)
                         see_also_links.append(link_text)
                 see_also = see_also_links
-            result += ':See also: '
-            result += ', '.join(see_also) + '\n'
+            result += gettext(':See also: %s') % ', '.join(see_also) + '\n'
 
         # If this will be rendered as plain text, convert it
         if plain:
@@ -657,13 +645,14 @@
     def run_argv_aliases(self, argv, alias_argv=None):
         """Parse the command line and run with extra aliases in alias_argv."""
         args, opts = parse_args(self, argv, alias_argv)
+        self._setup_outf()
 
         # Process the standard options
         if 'help' in opts:  # e.g. bzr add --help
-            sys.stdout.write(self.get_help_text())
+            self.outf.write(self.get_help_text())
             return 0
         if 'usage' in opts:  # e.g. bzr add --usage
-            sys.stdout.write(self.get_help_text(verbose=False))
+            self.outf.write(self.get_help_text(verbose=False))
             return 0
         trace.set_verbosity_level(option._verbosity_level)
         if 'verbose' in self.supported_std_options:
@@ -674,7 +663,6 @@
             opts['quiet'] = trace.is_quiet()
         elif opts.has_key('quiet'):
             del opts['quiet']
-
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -684,9 +672,15 @@
         all_cmd_args = cmdargs.copy()
         all_cmd_args.update(cmdopts)
 
-        self._setup_outf()
-
-        return self.run(**all_cmd_args)
+        try:
+            return self.run(**all_cmd_args)
+        finally:
+            # reset it, so that other commands run in the same process won't
+            # inherit state. Before we reset it, log any activity, so that it
+            # gets properly tracked.
+            ui.ui_factory.log_transport_activity(
+                display=('bytes' in debug.debug_flags))
+            trace.set_verbosity_level(0)
 
     def _setup_run(self):
         """Wrap the defined run method on self with a cleanup.
@@ -706,11 +700,6 @@
                 del self._operation
         self.run = run
 
-    @deprecated_method(deprecated_in((2, 2, 0)))
-    def run_direct(self, *args, **kwargs):
-        """Deprecated thunk from bzrlib 2.1."""
-        return self.run(*args, **kwargs)
-
     def run(self):
         """Actually run the command.
 
@@ -742,7 +731,19 @@
             return None
         return getdoc(self)
 
+    def gettext(self, message):
+        """Returns the gettext function used to translate this command's help.
+
+        Commands provided by plugins should override this to use their
+        own i18n system.
+        """
+        return i18n.gettext_per_paragraph(message)
+
     def name(self):
+        """Return the canonical name for this command.
+
+        The name under which it was actually invoked is available in invoked_as.
+        """
         return _unsquish_command_name(self.__class__.__name__)
 
     def plugin_name(self):
@@ -766,30 +767,30 @@
         These are all empty initially, because by default nothing should get
         notified.
         """
-        Hooks.__init__(self)
-        self.create_hook(HookPoint('extend_command',
+        Hooks.__init__(self, "bzrlib.commands", "Command.hooks")
+        self.add_hook('extend_command',
             "Called after creating a command object to allow modifications "
             "such as adding or removing options, docs etc. Called with the "
-            "new bzrlib.commands.Command object.", (1, 13), None))
-        self.create_hook(HookPoint('get_command',
+            "new bzrlib.commands.Command object.", (1, 13))
+        self.add_hook('get_command',
             "Called when creating a single command. Called with "
             "(cmd_or_None, command_name). get_command should either return "
             "the cmd_or_None parameter, or a replacement Command object that "
             "should be used for the command. Note that the Command.hooks "
             "hooks are core infrastructure. Many users will prefer to use "
             "bzrlib.commands.register_command or plugin_cmds.register_lazy.",
-            (1, 17), None))
-        self.create_hook(HookPoint('get_missing_command',
+            (1, 17))
+        self.add_hook('get_missing_command',
             "Called when creating a single command if no command could be "
             "found. Called with (command_name). get_missing_command should "
             "either return None, or a Command object to be used for the "
-            "command.", (1, 17), None))
-        self.create_hook(HookPoint('list_commands',
+            "command.", (1, 17))
+        self.add_hook('list_commands',
             "Called when enumerating commands. Called with a set of "
             "cmd_name strings for all the commands found so far. This set "
             " is safe to mutate - e.g. to remove a command. "
             "list_commands should return the updated set of command names.",
-            (1, 17), None))
+            (1, 17))
 
 Command.hooks = CommandHooks()
 
@@ -809,7 +810,14 @@
     else:
         args = argv
 
-    options, args = parser.parse_args(args)
+    # for python 2.5 and later, optparse raises this exception if a non-ascii
+    # option name is given.  See http://bugs.python.org/issue2931
+    try:
+        options, args = parser.parse_args(args)
+    except UnicodeEncodeError,e:
+        raise errors.BzrCommandError(
+            gettext('Only ASCII permitted in option names'))
+
     opts = dict([(k, v) for k, v in options.__dict__.iteritems() if
                  v is not option.OptionParser.DEFAULT_VALUE])
     return args, opts
@@ -832,29 +840,33 @@
                 argdict[argname + '_list'] = None
         elif ap[-1] == '+':
             if not args:
-                raise errors.BzrCommandError("command %r needs one or more %s"
-                                             % (cmd, argname.upper()))
+                raise errors.BzrCommandError(gettext(
+                      "command {0!r} needs one or more {1}").format(
+                      cmd, argname.upper()))
             else:
                 argdict[argname + '_list'] = args[:]
                 args = []
         elif ap[-1] == '$': # all but one
             if len(args) < 2:
-                raise errors.BzrCommandError("command %r needs one or more %s"
-                                             % (cmd, argname.upper()))
+                raise errors.BzrCommandError(
+                      gettext("command {0!r} needs one or more {1}").format(
+                                             cmd, argname.upper()))
             argdict[argname + '_list'] = args[:-1]
             args[:-1] = []
         else:
             # just a plain arg
             argname = ap
             if not args:
-                raise errors.BzrCommandError("command %r requires argument %s"
-                               % (cmd, argname.upper()))
+                raise errors.BzrCommandError(
+                     gettext("command {0!r} requires argument {1}").format(
+                               cmd, argname.upper()))
             else:
                 argdict[argname] = args.pop(0)
 
     if args:
-        raise errors.BzrCommandError("extra argument to command %s: %s"
-                                     % (cmd, args[0]))
+        raise errors.BzrCommandError( gettext(
+                              "extra argument to command {0}: {1}").format(
+                                       cmd, args[0]) )
 
     return argdict
 
@@ -943,21 +955,17 @@
 
 def apply_lsprofiled(filename, the_callable, *args, **kwargs):
     from bzrlib.lsprof import profile
-    ret, stats = profile(exception_to_return_code, the_callable, *args, **kwargs)
+    ret, stats = profile(exception_to_return_code, the_callable,
+                         *args, **kwargs)
     stats.sort()
     if filename is None:
         stats.pprint()
     else:
         stats.save(filename)
-        trace.note('Profile data written to "%s".', filename)
+        trace.note(gettext('Profile data written to "%s".'), filename)
     return ret
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def shlex_split_unicode(unsplit):
-    return cmdline.split(unsplit)
-
-
 def get_alias(cmd, config=None):
     """Return an expanded alias, or None if no alias exists.
 
@@ -1019,11 +1027,11 @@
         Specify the number of processes that can be run concurrently (selftest).
     """
     trace.mutter("bazaar version: " + bzrlib.__version__)
-    argv = list(argv)
+    argv = _specified_or_unicode_argv(argv)
     trace.mutter("bzr arguments: %r", argv)
 
-    opt_lsprof = opt_profile = opt_no_plugins = opt_builtin =  \
-                opt_no_aliases = False
+    opt_lsprof = opt_profile = opt_no_plugins = opt_builtin = \
+            opt_no_l10n = opt_no_aliases = False
     opt_lsprof_file = opt_coverage_dir = None
 
     # --no-plugins is handled specially at a very early stage. We need
@@ -1032,6 +1040,7 @@
 
     argv_copy = []
     i = 0
+    override_config = []
     while i < len(argv):
         a = argv[i]
         if a == '--profile':
@@ -1046,6 +1055,8 @@
             opt_no_plugins = True
         elif a == '--no-aliases':
             opt_no_aliases = True
+        elif a == '--no-l10n':
+            opt_no_l10n = True
         elif a == '--builtin':
             opt_builtin = True
         elif a == '--concurrency':
@@ -1058,10 +1069,14 @@
             pass # already handled in startup script Bug #588277
         elif a.startswith('-D'):
             debug.debug_flags.add(a[2:])
+        elif a.startswith('-O'):
+            override_config.append(a[2:])
         else:
             argv_copy.append(a)
         i += 1
 
+    bzrlib.global_state.cmdline_overrides._from_cmdline(override_config)
+
     debug.set_debug_flags_from_config()
 
     if not opt_no_plugins:
@@ -1087,6 +1102,8 @@
 
     cmd = argv.pop(0)
     cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
+    if opt_no_l10n:
+        cmd.l10n = False
     run = cmd_obj.run_argv_aliases
     run_argv = [argv, alias_argv]
 
@@ -1117,6 +1134,8 @@
         if 'memory' in debug.debug_flags:
             trace.debug_memory('Process status after command:', short=False)
         option._verbosity_level = saved_verbosity_level
+        # Reset the overrides 
+        bzrlib.global_state.cmdline_overrides._reset()
 
 
 def display_command(func):
@@ -1151,8 +1170,9 @@
         "bzr plugin commands")
     Command.hooks.install_named_hook("get_command", _get_external_command,
         "bzr external command lookup")
-    Command.hooks.install_named_hook("get_missing_command", _try_plugin_provider,
-        "bzr plugin-provider-db check")
+    Command.hooks.install_named_hook("get_missing_command",
+                                     _try_plugin_provider,
+                                     "bzr plugin-provider-db check")
 
 
 
@@ -1165,7 +1185,7 @@
         new_argv = []
         try:
             # ensure all arguments are unicode strings
-            for a in argv[1:]:
+            for a in argv:
                 if isinstance(a, unicode):
                     new_argv.append(a)
                 else:
@@ -1187,11 +1207,10 @@
 
     :return: exit code of bzr command.
     """
-    argv = _specified_or_unicode_argv(argv)
+    if argv is not None:
+        argv = argv[1:]
     _register_builtin_commands()
     ret = run_bzr_catch_errors(argv)
-    bzrlib.ui.ui_factory.log_transport_activity(
-        display=('bytes' in debug.debug_flags))
     trace.mutter("return code %d", ret)
     return ret
 
@@ -1250,19 +1269,19 @@
 
 
 class Provider(object):
-    '''Generic class to be overriden by plugins'''
+    """Generic class to be overriden by plugins"""
 
     def plugin_for_command(self, cmd_name):
-        '''Takes a command and returns the information for that plugin
+        """Takes a command and returns the information for that plugin
 
         :return: A dictionary with all the available information
-        for the requested plugin
-        '''
+            for the requested plugin
+        """
         raise NotImplementedError
 
 
 class ProvidersRegistry(registry.Registry):
-    '''This registry exists to allow other providers to exist'''
+    """This registry exists to allow other providers to exist"""
 
     def __iter__(self):
         for key, provider in self.iteritems():

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2010-05-10 11:34:20 +0000
+++ bzrlib/commit.py	2011-09-16 10:08:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,9 +52,9 @@
 from bzrlib import (
     debug,
     errors,
-    revision,
     trace,
     tree,
+    ui,
     )
 from bzrlib.branch import Branch
 from bzrlib.cleanup import OperationWithCleanups
@@ -72,8 +72,7 @@
 from bzrlib.inventory import Inventory, InventoryEntry, make_entry
 from bzrlib import symbol_versioning
 from bzrlib.urlutils import unescape_for_display
-import bzrlib.ui
-
+from bzrlib.i18n import gettext
 
 class NullCommitReporter(object):
     """I report on progress of a commit."""
@@ -114,7 +113,7 @@
         note(format, *args)
 
     def snapshot_change(self, change, path):
-        if path == '' and change in ('added', 'modified'):
+        if path == '' and change in (gettext('added'), gettext('modified')):
             return
         self._note("%s %s", change, path)
 
@@ -128,10 +127,10 @@
                                    "to started.", DeprecationWarning,
                                    stacklevel=2)
             location = ''
-        self._note('Committing%s', location)
+        self._note(gettext('Committing%s'), location)
 
     def completed(self, revno, rev_id):
-        self._note('Committed revision %d.', revno)
+        self._note(gettext('Committed revision %d.'), revno)
         # self._note goes to the console too; so while we want to log the
         # rev_id, we can't trivially only log it. (See bug 526425). Long
         # term we should rearrange the reporting structure, but for now
@@ -140,10 +139,10 @@
         mutter('Committed revid %s as revno %d.', rev_id, revno)
 
     def deleted(self, path):
-        self._note('deleted %s', path)
+        self._note(gettext('deleted %s'), path)
 
     def missing(self, path):
-        self._note('missing %s', path)
+        self._note(gettext('missing %s'), path)
 
     def renamed(self, change, old_path, new_path):
         self._note('%s %s => %s', change, old_path, new_path)
@@ -229,7 +228,8 @@
                message_callback=None,
                recursive='down',
                exclude=None,
-               possible_master_transports=None):
+               possible_master_transports=None,
+               lossy=False):
         """Commit working copy as a new revision.
 
         :param message: the commit message (it or message_callback is required)
@@ -262,6 +262,8 @@
         :param exclude: None or a list of relative paths to exclude from the
             commit. Pending changes to excluded files will be ignored by the
             commit.
+        :param lossy: When committing to a foreign VCS, ignore any
+            data that can not be natively represented.
         """
         operation = OperationWithCleanups(self._commit)
         self.revprops = revprops or {}
@@ -283,12 +285,13 @@
                message_callback=message_callback,
                recursive=recursive,
                exclude=exclude,
-               possible_master_transports=possible_master_transports)
+               possible_master_transports=possible_master_transports,
+               lossy=lossy)
 
     def _commit(self, operation, message, timestamp, timezone, committer,
             specific_files, rev_id, allow_pointless, strict, verbose,
             working_tree, local, reporter, message_callback, recursive,
-            exclude, possible_master_transports):
+            exclude, possible_master_transports, lossy):
         mutter('preparing to commit')
 
         if working_tree is None:
@@ -326,7 +329,7 @@
                 minimum_path_selection(specific_files))
         else:
             self.specific_files = None
-            
+
         self.allow_pointless = allow_pointless
         self.message_callback = message_callback
         self.timestamp = timestamp
@@ -346,7 +349,7 @@
             not self.branch.repository._format.supports_tree_reference and
             (self.branch.repository._format.fast_deltas or
              len(self.parents) < 2))
-        self.pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        self.pb = ui.ui_factory.nested_progress_bar()
         operation.add_cleanup(self.pb.finished)
         self.basis_revid = self.work_tree.last_revision()
         self.basis_tree = self.work_tree.basis_tree()
@@ -380,7 +383,9 @@
         self.pb_stage_count = 0
         self.pb_stage_total = 5
         if self.bound_branch:
-            self.pb_stage_total += 1
+            # 2 extra stages: "Uploading data to master branch" and "Merging
+            # tags to master branch"
+            self.pb_stage_total += 2
         self.pb.show_pct = False
         self.pb.show_spinner = False
         self.pb.show_eta = False
@@ -398,8 +403,13 @@
 
         # Collect the changes
         self._set_progress_stage("Collecting changes", counter=True)
+        self._lossy = lossy
         self.builder = self.branch.get_commit_builder(self.parents,
-            self.config, timestamp, timezone, committer, self.revprops, rev_id)
+            self.config, timestamp, timezone, committer, self.revprops,
+            rev_id, lossy=lossy)
+        if not self.builder.supports_record_entry_contents and self.exclude:
+            self.builder.abort()
+            raise errors.ExcludesUnsupported(self.branch.repository)
 
         try:
             self.builder.will_record_deletes()
@@ -432,7 +442,6 @@
         except Exception, e:
             mutter("aborting commit write group because of exception:")
             trace.log_exception_quietly()
-            note("aborting commit write group: %r" % (e,))
             self.builder.abort()
             raise
 
@@ -444,12 +453,24 @@
             self._set_progress_stage("Uploading data to master branch")
             # 'commit' to the master first so a timeout here causes the
             # local branch to be out of date
-            self.master_branch.import_last_revision_info(
-                self.branch.repository, new_revno, self.rev_id)
+            (new_revno, self.rev_id) = self.master_branch.import_last_revision_info_and_tags(
+                self.branch, new_revno, self.rev_id, lossy=lossy)
+            if lossy:
+                self.branch.fetch(self.master_branch, self.rev_id)
 
         # and now do the commit locally.
         self.branch.set_last_revision_info(new_revno, self.rev_id)
 
+        # Merge local tags to remote
+        if self.bound_branch:
+            self._set_progress_stage("Merging tags to master branch")
+            tag_updates, tag_conflicts = self.branch.tags.merge_to(
+                self.master_branch.tags)
+            if tag_conflicts:
+                warning_lines = ['    ' + name for name, _, _ in tag_conflicts]
+                note( gettext("Conflicting tags in bound branch:\n{0}".format(
+                    "\n".join(warning_lines))) )
+
         # Make the working tree be up to date with the branch. This
         # includes automatic changes scheduled to be made to the tree, such
         # as updating its basis and unversioning paths that were missing.
@@ -473,15 +494,6 @@
         # A merge with no effect on files
         if len(self.parents) > 1:
             return
-        # TODO: we could simplify this by using self.builder.basis_delta.
-
-        # The initial commit adds a root directory, but this in itself is not
-        # a worthwhile commit.
-        if (self.basis_revid == revision.NULL_REVISION and
-            ((self.builder.new_inventory is not None and
-             len(self.builder.new_inventory) == 1) or
-            len(self.builder._basis_delta) == 1)):
-            raise PointlessCommit()
         if self.builder.any_changes():
             return
         raise PointlessCommit()
@@ -683,6 +695,8 @@
                 # Reset the new path (None) and new versioned flag (False)
                 change = (change[0], (change[1][0], None), change[2],
                     (change[3][0], False)) + change[4:]
+                new_path = change[1][1]
+                versioned = False
             elif kind == 'tree-reference':
                 if self.recursive == 'down':
                     self._commit_nested_tree(change[0], change[1][1])
@@ -692,15 +706,15 @@
                     if new_path is None:
                         reporter.deleted(old_path)
                     elif old_path is None:
-                        reporter.snapshot_change('added', new_path)
+                        reporter.snapshot_change(gettext('added'), new_path)
                     elif old_path != new_path:
-                        reporter.renamed('renamed', old_path, new_path)
+                        reporter.renamed(gettext('renamed'), old_path, new_path)
                     else:
                         if (new_path or 
                             self.work_tree.branch.repository._format.rich_root_data):
                             # Don't report on changes to '' in non rich root
                             # repositories.
-                            reporter.snapshot_change('modified', new_path)
+                            reporter.snapshot_change(gettext('modified'), new_path)
             self._next_progress_entry()
         # Unversion IDs that were found to be deleted
         self.deleted_ids = deleted_ids
@@ -712,7 +726,7 @@
         if self.specific_files or self.exclude:
             specific_files = self.specific_files or []
             for path, old_ie in self.basis_inv.iter_entries():
-                if old_ie.file_id in self.builder.new_inventory:
+                if self.builder.new_inventory.has_id(old_ie.file_id):
                     # already added - skip.
                     continue
                 if (is_inside_any(specific_files, path)
@@ -929,7 +943,7 @@
             self.reporter.renamed(change, old_path, path)
             self._next_progress_entry()
         else:
-            if change == 'unchanged':
+            if change == gettext('unchanged'):
                 return
             self.reporter.snapshot_change(change, path)
             self._next_progress_entry()
@@ -951,10 +965,10 @@
 
     def _emit_progress(self):
         if self.pb_entries_count is not None:
-            text = "%s [%d] - Stage" % (self.pb_stage_name,
+            text = gettext("{0} [{1}] - Stage").format(self.pb_stage_name,
                 self.pb_entries_count)
         else:
-            text = "%s - Stage" % (self.pb_stage_name, )
+            text = gettext("%s - Stage") % (self.pb_stage_name, )
         self.pb.update(text, self.pb_stage_count, self.pb_stage_total)
 
     def _set_specific_file_ids(self):

=== renamed file 'bzrlib/sign_my_commits.py' => 'bzrlib/commit_signature_commands.py'
--- bzrlib/sign_my_commits.py	2010-04-23 08:51:52 +0000
+++ bzrlib/commit_signature_commands.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,14 +20,15 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
-    config,
+    controldir,
+    errors,
     gpg,
+    revision as _mod_revision,
     )
-from bzrlib.bzrdir import BzrDir
 """)
 from bzrlib.commands import Command
 from bzrlib.option import Option
-
+from bzrlib.i18n import gettext, ngettext
 
 class cmd_sign_my_commits(Command):
     __doc__ = """Sign all commits by a given committer.
@@ -50,10 +51,10 @@
 
     def run(self, location=None, committer=None, dry_run=False):
         if location is None:
-            bzrdir = BzrDir.open_containing('.')[0]
+            bzrdir = controldir.ControlDir.open_containing('.')[0]
         else:
             # Passed in locations should be exact
-            bzrdir = BzrDir.open(location)
+            bzrdir = controldir.ControlDir.open(location)
         branch = bzrdir.open_branch()
         repo = branch.repository
         branch_config = branch.get_config()
@@ -65,9 +66,16 @@
         count = 0
         repo.lock_write()
         try:
+            graph = repo.get_graph()
             repo.start_write_group()
             try:
-                for rev_id in repo.get_ancestry(branch.last_revision())[1:]:
+                for rev_id, parents in graph.iter_ancestry(
+                        [branch.last_revision()]):
+                    if _mod_revision.is_null(rev_id):
+                        continue
+                    if parents is None:
+                        # Ignore ghosts
+                        continue
                     if repo.has_signature_for_revision_id(rev_id):
                         continue
                     rev = repo.get_revision(rev_id)
@@ -89,3 +97,98 @@
         print 'Signed %d revisions' % (count,)
 
 
+class cmd_verify_signatures(Command):
+    __doc__ = """Verify all commit signatures.
+
+    Verifies that all commits in the branch are signed by known GnuPG keys.
+    """
+
+    takes_options = [
+            Option('acceptable-keys',
+                   help='Comma separated list of GPG key patterns which are'
+                        ' acceptable for verification.',
+                   short_name='k',
+                   type=str,),
+            'revision', 
+            'verbose',
+          ]
+    takes_args = ['location?']
+
+    def run(self, acceptable_keys=None, revision=None, verbose=None,
+                                                            location=u'.'):
+        bzrdir = controldir.ControlDir.open_containing(location)[0]
+        branch = bzrdir.open_branch()
+        repo = branch.repository
+        branch_config = branch.get_config()
+        gpg_strategy = gpg.GPGStrategy(branch_config)
+
+        gpg_strategy.set_acceptable_keys(acceptable_keys)
+
+        def write(string):
+            self.outf.write(string + "\n")
+        def write_verbose(string):
+            self.outf.write("  " + string + "\n")
+
+        #get our list of revisions
+        revisions = []
+        if revision is not None:
+            if len(revision) == 1:
+                revno, rev_id = revision[0].in_history(branch)
+                revisions.append(rev_id)
+            elif len(revision) == 2:
+                from_revno, from_revid = revision[0].in_history(branch)
+                to_revno, to_revid = revision[1].in_history(branch)
+                if to_revid is None:
+                    to_revno = branch.revno()
+                if from_revno is None or to_revno is None:
+                    raise errors.BzrCommandError(gettext(
+                    'Cannot verify a range of non-revision-history revisions'))
+                for revno in range(from_revno, to_revno + 1):
+                    revisions.append(branch.get_rev_id(revno))
+        else:
+            #all revisions by default including merges
+            graph = repo.get_graph()
+            revisions = []
+            repo.lock_read()
+            for rev_id, parents in graph.iter_ancestry(
+                    [branch.last_revision()]):
+                if _mod_revision.is_null(rev_id):
+                    continue
+                if parents is None:
+                    # Ignore ghosts
+                    continue
+                revisions.append(rev_id)
+            repo.unlock()
+        count, result, all_verifiable =\
+                                gpg_strategy.do_verifications(revisions, repo)
+        if all_verifiable:
+               write(gettext(
+                            "All commits signed with verifiable keys"))
+               if verbose:
+                   write(gpg_strategy.verbose_valid_message(result))
+               return 0
+        else:
+            write(gpg_strategy.valid_commits_message(count))
+            if verbose:
+               for message in gpg_strategy.verbose_valid_message(result):
+                   write_verbose(message)
+            write(gpg_strategy.expired_commit_message(count))
+            if verbose:
+               for message in gpg_strategy.verbose_expired_key_message(result,
+                                                                          repo):
+                   write_verbose(message)
+            write(gpg_strategy.unknown_key_message(count))
+            if verbose:
+                for message in gpg_strategy.verbose_missing_key_message(result):
+                    write_verbose(message)
+            write(gpg_strategy.commit_not_valid_message(count))
+            if verbose:
+                for message in gpg_strategy.verbose_not_valid_message(result,
+                                                                        repo):
+                   write_verbose(message)
+            write(gpg_strategy.commit_not_signed_message(count))
+            if verbose:
+                for message in gpg_strategy.verbose_not_signed_message(result,
+                                                                          repo):
+                    write_verbose(message)
+            return 1

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2010-07-21 03:14:56 +0000
+++ bzrlib/config.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #            and others
 #
@@ -29,6 +29,9 @@
 create_signatures=always|never|when-required(default)
 gpg_signing_command=name-of-program
 log_format=name-of-format
+validate_signatures_in_log=true|false(default)
+acceptable_keys=pattern1,pattern2
+gpg_signing_key=amy@example.com
 
 in locations.conf, you specify the url of a branch and options for it.
 Wildcards may be used - * and ? as normal in shell completion. Options
@@ -39,19 +42,26 @@
 email= as above
 check_signatures= as above
 create_signatures= as above.
+validate_signatures_in_log=as above
+acceptable_keys=as above
 
 explanation of options
 ----------------------
 editor - this option sets the pop up editor to use during commits.
 email - this option sets the user id bzr will use when committing.
-check_signatures - this option controls whether bzr will require good gpg
+check_signatures - this option will control whether bzr will require good gpg
                    signatures, ignore them, or check them if they are
-                   present.
+                   present.  Currently it is unused except that check_signatures
+                   turns on create_signatures.
 create_signatures - this option controls whether bzr will always create
-                    gpg signatures, never create them, or create them if the
-                    branch is configured to require them.
+                    gpg signatures or not on commits.  There is an unused
+                    option which in future is expected to work if               
+                    branch settings require signatures.
 log_format - this option sets the default log format.  Possible values are
              long, short, line, or a plugin can register new formats.
+validate_signatures_in_log - show GPG signature validity in log output
+acceptable_keys - comma separated list of key patterns acceptable for
+                  verify-signatures command
 
 In bazaar.conf you can also define aliases in the ALIASES sections, example
 
@@ -63,31 +73,49 @@
 """
 
 import os
+import string
 import sys
 
+
+import bzrlib
+from bzrlib.decorators import needs_write_lock
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import errno
-from fnmatch import fnmatch
+import fnmatch
 import re
 from cStringIO import StringIO
 
-import bzrlib
 from bzrlib import (
     atomicfile,
+    controldir,
     debug,
     errors,
+    lazy_regex,
+    library_state,
+    lockdir,
     mail_client,
+    mergetools,
     osutils,
-    registry,
     symbol_versioning,
     trace,
+    transport,
     ui,
     urlutils,
     win32utils,
     )
+from bzrlib.i18n import gettext
 from bzrlib.util.configobj import configobj
 """)
+from bzrlib import (
+    commands,
+    hooks,
+    lazy_regex,
+    registry,
+    )
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    deprecated_method,
+    )
 
 
 CHECK_IF_POSSIBLE=0
@@ -123,25 +151,47 @@
 STORE_BRANCH = 3
 STORE_GLOBAL = 4
 
-_ConfigObj = None
-def ConfigObj(*args, **kwargs):
-    global _ConfigObj
-    if _ConfigObj is None:
-        class ConfigObj(configobj.ConfigObj):
-
-            def get_bool(self, section, key):
-                return self[section].as_bool(key)
-
-            def get_value(self, section, name):
-                # Try [] for the old DEFAULT section.
-                if section == "DEFAULT":
-                    try:
-                        return self[name]
-                    except KeyError:
-                        pass
-                return self[section][name]
-        _ConfigObj = ConfigObj
-    return _ConfigObj(*args, **kwargs)
+
+class ConfigObj(configobj.ConfigObj):
+
+    def __init__(self, infile=None, **kwargs):
+        # We define our own interpolation mechanism calling it option expansion
+        super(ConfigObj, self).__init__(infile=infile,
+                                        interpolation=False,
+                                        **kwargs)
+
+    def get_bool(self, section, key):
+        return self[section].as_bool(key)
+
+    def get_value(self, section, name):
+        # Try [] for the old DEFAULT section.
+        if section == "DEFAULT":
+            try:
+                return self[name]
+            except KeyError:
+                pass
+        return self[section][name]
+
+
+# FIXME: Until we can guarantee that each config file is loaded once and
+# only once for a given bzrlib session, we don't want to re-read the file every
+# time we query for an option so we cache the value (bad ! watch out for tests
+# needing to restore the proper value). -- vila 20110219
+_expand_default_value = None
+def _get_expand_default_value():
+    global _expand_default_value
+    if _expand_default_value is not None:
+        return _expand_default_value
+    conf = GlobalConfig()
+    # Note that we must not use None for the expand value below or we'll run
+    # into infinite recursion. Using False really would be quite silly ;)
+    expand = conf.get_user_option_as_bool('bzr.config.expand', expand=True)
+    if expand is None:
+        # This is an opt-in feature, you *really* need to clearly say you want
+        # to activate it !
+        expand = False
+    _expand_default_value = expand
+    return expand
 
 
 class Config(object):
@@ -150,6 +200,11 @@
     def __init__(self):
         super(Config, self).__init__()
 
+    def config_id(self):
+        """Returns a unique ID for the config."""
+        raise NotImplementedError(self.config_id)
+
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def get_editor(self):
         """Get the users pop up editor."""
         raise NotImplementedError
@@ -162,7 +217,6 @@
         return diff.DiffFromTool.from_string(cmd, old_tree, new_tree,
                                              sys.stdout)
 
-
     def get_mail_client(self):
         """Get a mail client to use"""
         selected_client = self.get_user_option('mail_client')
@@ -179,24 +233,171 @@
     def _get_signing_policy(self):
         """Template method to override signature creation policy."""
 
+    option_ref_re = None
+
+    def expand_options(self, string, env=None):
+        """Expand option references in the string in the configuration context.
+
+        :param string: The string containing option to expand.
+
+        :param env: An option dict defining additional configuration options or
+            overriding existing ones.
+
+        :returns: The expanded string.
+        """
+        return self._expand_options_in_string(string, env)
+
+    def _expand_options_in_list(self, slist, env=None, _ref_stack=None):
+        """Expand options in  a list of strings in the configuration context.
+
+        :param slist: A list of strings.
+
+        :param env: An option dict defining additional configuration options or
+            overriding existing ones.
+
+        :param _ref_stack: Private list containing the options being
+            expanded to detect loops.
+
+        :returns: The flatten list of expanded strings.
+        """
+        # expand options in each value separately flattening lists
+        result = []
+        for s in slist:
+            value = self._expand_options_in_string(s, env, _ref_stack)
+            if isinstance(value, list):
+                result.extend(value)
+            else:
+                result.append(value)
+        return result
+
+    def _expand_options_in_string(self, string, env=None, _ref_stack=None):
+        """Expand options in the string in the configuration context.
+
+        :param string: The string to be expanded.
+
+        :param env: An option dict defining additional configuration options or
+            overriding existing ones.
+
+        :param _ref_stack: Private list containing the options being
+            expanded to detect loops.
+
+        :returns: The expanded string.
+        """
+        if string is None:
+            # Not much to expand there
+            return None
+        if _ref_stack is None:
+            # What references are currently resolved (to detect loops)
+            _ref_stack = []
+        if self.option_ref_re is None:
+            # We want to match the most embedded reference first (i.e. for
+            # '{{foo}}' we will get '{foo}',
+            # for '{bar{baz}}' we will get '{baz}'
+            self.option_ref_re = re.compile('({[^{}]+})')
+        result = string
+        # We need to iterate until no more refs appear ({{foo}} will need two
+        # iterations for example).
+        while True:
+            raw_chunks = self.option_ref_re.split(result)
+            if len(raw_chunks) == 1:
+                # Shorcut the trivial case: no refs
+                return result
+            chunks = []
+            list_value = False
+            # Split will isolate refs so that every other chunk is a ref
+            chunk_is_ref = False
+            for chunk in raw_chunks:
+                if not chunk_is_ref:
+                    if chunk:
+                        # Keep only non-empty strings (or we get bogus empty
+                        # slots when a list value is involved).
+                        chunks.append(chunk)
+                    chunk_is_ref = True
+                else:
+                    name = chunk[1:-1]
+                    if name in _ref_stack:
+                        raise errors.OptionExpansionLoop(string, _ref_stack)
+                    _ref_stack.append(name)
+                    value = self._expand_option(name, env, _ref_stack)
+                    if value is None:
+                        raise errors.ExpandingUnknownOption(name, string)
+                    if isinstance(value, list):
+                        list_value = True
+                        chunks.extend(value)
+                    else:
+                        chunks.append(value)
+                    _ref_stack.pop()
+                    chunk_is_ref = False
+            if list_value:
+                # Once a list appears as the result of an expansion, all
+                # callers will get a list result. This allows a consistent
+                # behavior even when some options in the expansion chain
+                # defined as strings (no comma in their value) but their
+                # expanded value is a list.
+                return self._expand_options_in_list(chunks, env, _ref_stack)
+            else:
+                result = ''.join(chunks)
+        return result
+
+    def _expand_option(self, name, env, _ref_stack):
+        if env is not None and name in env:
+            # Special case, values provided in env takes precedence over
+            # anything else
+            value = env[name]
+        else:
+            # FIXME: This is a limited implementation, what we really need is a
+            # way to query the bzr config for the value of an option,
+            # respecting the scope rules (That is, once we implement fallback
+            # configs, getting the option value should restart from the top
+            # config, not the current one) -- vila 20101222
+            value = self.get_user_option(name, expand=False)
+            if isinstance(value, list):
+                value = self._expand_options_in_list(value, env, _ref_stack)
+            else:
+                value = self._expand_options_in_string(value, env, _ref_stack)
+        return value
+
     def _get_user_option(self, option_name):
         """Template method to provide a user option."""
         return None
 
-    def get_user_option(self, option_name):
-        """Get a generic option - no special process, no default."""
-        return self._get_user_option(option_name)
-
-    def get_user_option_as_bool(self, option_name):
-        """Get a generic option as a boolean - no special process, no default.
-
+    def get_user_option(self, option_name, expand=None):
+        """Get a generic option - no special process, no default.
+
+        :param option_name: The queried option.
+
+        :param expand: Whether options references should be expanded.
+
+        :returns: The value of the option.
+        """
+        if expand is None:
+            expand = _get_expand_default_value()
+        value = self._get_user_option(option_name)
+        if expand:
+            if isinstance(value, list):
+                value = self._expand_options_in_list(value)
+            elif isinstance(value, dict):
+                trace.warning('Cannot expand "%s":'
+                              ' Dicts do not support option expansion'
+                              % (option_name,))
+            else:
+                value = self._expand_options_in_string(value)
+        for hook in OldConfigHooks['get']:
+            hook(self, option_name, value)
+        return value
+
+    def get_user_option_as_bool(self, option_name, expand=None, default=None):
+        """Get a generic option as a boolean.
+
+        :param expand: Allow expanding references to other config values.
+        :param default: Default value if nothing is configured
         :return None if the option doesn't exist or its value can't be
             interpreted as a boolean. Returns True or False otherwise.
         """
-        s = self._get_user_option(option_name)
+        s = self.get_user_option(option_name, expand=expand)
         if s is None:
             # The option doesn't exist
-            return None
+            return default
         val = ui.bool_from_string(s)
         if val is None:
             # The value can't be interpreted as a boolean
@@ -204,17 +405,57 @@
                           s, option_name)
         return val
 
-    def get_user_option_as_list(self, option_name):
+    def get_user_option_as_list(self, option_name, expand=None):
         """Get a generic option as a list - no special process, no default.
 
         :return None if the option doesn't exist. Returns the value as a list
             otherwise.
         """
-        l = self._get_user_option(option_name)
+        l = self.get_user_option(option_name, expand=expand)
         if isinstance(l, (str, unicode)):
-            # A single value, most probably the user forgot the final ','
+            # A single value, most probably the user forgot (or didn't care to
+            # add) the final ','
             l = [l]
         return l
+        
+    def get_user_option_as_int_from_SI(self,  option_name,  default=None):
+        """Get a generic option from a human readable size in SI units, e.g 10MB
+        
+        Accepted suffixes are K,M,G. It is case-insensitive and may be followed
+        by a trailing b (i.e. Kb, MB). This is intended to be practical and not
+        pedantic.
+        
+        :return Integer, expanded to its base-10 value if a proper SI unit is 
+            found. If the option doesn't exist, or isn't a value in 
+            SI units, return default (which defaults to None)
+        """
+        val = self.get_user_option(option_name)
+        if isinstance(val, list):
+            val = val[0]
+        if val is None:
+            val = default
+        else:
+            p = re.compile("^(\d+)([kmg])*b*$", re.IGNORECASE)
+            try:
+                m = p.match(val)
+                if m is not None:
+                    val = int(m.group(1))
+                    if m.group(2) is not None:
+                        if m.group(2).lower() == 'k':
+                            val *= 10**3
+                        elif m.group(2).lower() == 'm':
+                            val *= 10**6
+                        elif m.group(2).lower() == 'g':
+                            val *= 10**9
+                else:
+                    ui.ui_factory.show_warning(gettext('Invalid config value for "{0}" '
+                                               ' value {1!r} is not an SI unit.').format(
+                                                option_name, val))
+                    val = default
+            except TypeError:
+                val = default
+        return val
+        
 
     def gpg_signing_command(self):
         """What program should be used to sign signatures?"""
@@ -238,6 +479,29 @@
         """See log_format()."""
         return None
 
+    def validate_signatures_in_log(self):
+        """Show GPG signature validity in log"""
+        result = self._validate_signatures_in_log()
+        if result == "true":
+            result = True
+        else:
+            result = False
+        return result
+
+    def _validate_signatures_in_log(self):
+        """See validate_signatures_in_log()."""
+        return None
+
+    def acceptable_keys(self):
+        """Comma separated list of key patterns acceptable to 
+        verify-signatures command"""
+        result = self._acceptable_keys()
+        return result
+
+    def _acceptable_keys(self):
+        """See acceptable_keys()."""
+        return None
+
     def post_commit(self):
         """An ordered list of python functions to call.
 
@@ -262,21 +526,21 @@
         the concrete policy type is checked, and finally
         $EMAIL is examined.
         If no username can be found, errors.NoWhoami exception is raised.
-
-        TODO: Check it's reasonably well-formed.
         """
         v = os.environ.get('BZR_EMAIL')
         if v:
             return v.decode(osutils.get_user_encoding())
-
         v = self._get_user_id()
         if v:
             return v
-
         v = os.environ.get('EMAIL')
         if v:
             return v.decode(osutils.get_user_encoding())
-
+        name, email = _auto_user_id()
+        if name and email:
+            return '%s <%s>' % (name, email)
+        elif email:
+            return email
         raise errors.NoWhoami()
 
     def ensure_username(self):
@@ -306,14 +570,22 @@
         if policy is None:
             policy = self._get_signature_checking()
             if policy is not None:
+                #this warning should go away once check_signatures is
+                #implemented (if not before)
                 trace.warning("Please use create_signatures,"
                               " not check_signatures to set signing policy.")
-            if policy == CHECK_ALWAYS:
-                return True
         elif policy == SIGN_ALWAYS:
             return True
         return False
 
+    def gpg_signing_key(self):
+        """GPG user-id to sign commits"""
+        key = self.get_user_option('gpg_signing_key')
+        if key == "default" or key == None:
+            return self.user_email()
+        else:
+            return key
+
     def get_alias(self, value):
         return self._get_alias(value)
 
@@ -348,28 +620,178 @@
         else:
             return True
 
+    def get_merge_tools(self):
+        tools = {}
+        for (oname, value, section, conf_id, parser) in self._get_options():
+            if oname.startswith('bzr.mergetool.'):
+                tool_name = oname[len('bzr.mergetool.'):]
+                tools[tool_name] = self.get_user_option(oname)
+        trace.mutter('loaded merge tools: %r' % tools)
+        return tools
+
+    def find_merge_tool(self, name):
+        # We fake a defaults mechanism here by checking if the given name can
+        # be found in the known_merge_tools if it's not found in the config.
+        # This should be done through the proposed config defaults mechanism
+        # when it becomes available in the future.
+        command_line = (self.get_user_option('bzr.mergetool.%s' % name,
+                                             expand=False)
+                        or mergetools.known_merge_tools.get(name, None))
+        return command_line
+
+
+class _ConfigHooks(hooks.Hooks):
+    """A dict mapping hook names and a list of callables for configs.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        super(_ConfigHooks, self).__init__('bzrlib.config', 'ConfigHooks')
+        self.add_hook('load',
+                      'Invoked when a config store is loaded.'
+                      ' The signature is (store).',
+                      (2, 4))
+        self.add_hook('save',
+                      'Invoked when a config store is saved.'
+                      ' The signature is (store).',
+                      (2, 4))
+        # The hooks for config options
+        self.add_hook('get',
+                      'Invoked when a config option is read.'
+                      ' The signature is (stack, name, value).',
+                      (2, 4))
+        self.add_hook('set',
+                      'Invoked when a config option is set.'
+                      ' The signature is (stack, name, value).',
+                      (2, 4))
+        self.add_hook('remove',
+                      'Invoked when a config option is removed.'
+                      ' The signature is (stack, name).',
+                      (2, 4))
+ConfigHooks = _ConfigHooks()
+
+
+class _OldConfigHooks(hooks.Hooks):
+    """A dict mapping hook names and a list of callables for configs.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        super(_OldConfigHooks, self).__init__('bzrlib.config', 'OldConfigHooks')
+        self.add_hook('load',
+                      'Invoked when a config store is loaded.'
+                      ' The signature is (config).',
+                      (2, 4))
+        self.add_hook('save',
+                      'Invoked when a config store is saved.'
+                      ' The signature is (config).',
+                      (2, 4))
+        # The hooks for config options
+        self.add_hook('get',
+                      'Invoked when a config option is read.'
+                      ' The signature is (config, name, value).',
+                      (2, 4))
+        self.add_hook('set',
+                      'Invoked when a config option is set.'
+                      ' The signature is (config, name, value).',
+                      (2, 4))
+        self.add_hook('remove',
+                      'Invoked when a config option is removed.'
+                      ' The signature is (config, name).',
+                      (2, 4))
+OldConfigHooks = _OldConfigHooks()
+
 
 class IniBasedConfig(Config):
     """A configuration policy that draws from ini files."""
 
-    def __init__(self, get_filename):
+    def __init__(self, get_filename=symbol_versioning.DEPRECATED_PARAMETER,
+                 file_name=None):
+        """Base class for configuration files using an ini-like syntax.
+
+        :param file_name: The configuration file path.
+        """
         super(IniBasedConfig, self).__init__()
-        self._get_filename = get_filename
+        self.file_name = file_name
+        if symbol_versioning.deprecated_passed(get_filename):
+            symbol_versioning.warn(
+                'IniBasedConfig.__init__(get_filename) was deprecated in 2.3.'
+                ' Use file_name instead.',
+                DeprecationWarning,
+                stacklevel=2)
+            if get_filename is not None:
+                self.file_name = get_filename()
+        else:
+            self.file_name = file_name
+        self._content = None
         self._parser = None
 
-    def _get_parser(self, file=None):
+    @classmethod
+    def from_string(cls, str_or_unicode, file_name=None, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This will
+            be utf-8 encoded.
+
+        :param file_name: The configuration file path.
+
+        :param _save: Whether the file should be saved upon creation.
+        """
+        conf = cls(file_name=file_name)
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
+    def _create_from_string(self, str_or_unicode, save):
+        self._content = StringIO(str_or_unicode.encode('utf-8'))
+        # Some tests use in-memory configs, some other always need the config
+        # file to exist on disk.
+        if save:
+            self._write_config_file()
+
+    def _get_parser(self, file=symbol_versioning.DEPRECATED_PARAMETER):
         if self._parser is not None:
             return self._parser
-        if file is None:
-            input = self._get_filename()
+        if symbol_versioning.deprecated_passed(file):
+            symbol_versioning.warn(
+                'IniBasedConfig._get_parser(file=xxx) was deprecated in 2.3.'
+                ' Use IniBasedConfig(_content=xxx) instead.',
+                DeprecationWarning,
+                stacklevel=2)
+        if self._content is not None:
+            co_input = self._content
+        elif self.file_name is None:
+            raise AssertionError('We have no content to create the config')
         else:
-            input = file
+            co_input = self.file_name
         try:
-            self._parser = ConfigObj(input, encoding='utf-8')
+            self._parser = ConfigObj(co_input, encoding='utf-8')
         except configobj.ConfigObjError, e:
             raise errors.ParseConfigError(e.errors, e.config.filename)
+        except UnicodeDecodeError:
+            raise errors.ConfigContentError(self.file_name)
+        # Make sure self.reload() will use the right file name
+        self._parser.filename = self.file_name
+        for hook in OldConfigHooks['load']:
+            hook(self)
         return self._parser
 
+    def reload(self):
+        """Reload the config file from disk."""
+        if self.file_name is None:
+            raise AssertionError('We need a file name to reload the config')
+        if self._parser is not None:
+            self._parser.reload()
+        for hook in ConfigHooks['load']:
+            hook(self)
+
     def _get_matching_sections(self):
         """Return an ordered list of (section_name, extra_path) pairs.
 
@@ -386,6 +808,55 @@
         """Override this to define the section used by the config."""
         return "DEFAULT"
 
+    def _get_sections(self, name=None):
+        """Returns an iterator of the sections specified by ``name``.
+
+        :param name: The section name. If None is supplied, the default
+            configurations are yielded.
+
+        :return: A tuple (name, section, config_id) for all sections that will
+            be walked by user_get_option() in the 'right' order. The first one
+            is where set_user_option() will update the value.
+        """
+        parser = self._get_parser()
+        if name is not None:
+            yield (name, parser[name], self.config_id())
+        else:
+            # No section name has been given so we fallback to the configobj
+            # itself which holds the variables defined outside of any section.
+            yield (None, parser, self.config_id())
+
+    def _get_options(self, sections=None):
+        """Return an ordered list of (name, value, section, config_id) tuples.
+
+        All options are returned with their associated value and the section
+        they appeared in. ``config_id`` is a unique identifier for the
+        configuration file the option is defined in.
+
+        :param sections: Default to ``_get_matching_sections`` if not
+            specified. This gives a better control to daughter classes about
+            which sections should be searched. This is a list of (name,
+            configobj) tuples.
+        """
+        opts = []
+        if sections is None:
+            parser = self._get_parser()
+            sections = []
+            for (section_name, _) in self._get_matching_sections():
+                try:
+                    section = parser[section_name]
+                except KeyError:
+                    # This could happen for an empty file for which we define a
+                    # DEFAULT section. FIXME: Force callers to provide sections
+                    # instead ? -- vila 20100930
+                    continue
+                sections.append((section_name, section))
+        config_id = self.config_id()
+        for (section_name, section) in sections:
+            for (name, value) in section.iteritems():
+                yield (name, parser._quote(value), section_name,
+                       config_id, parser)
+
     def _get_option_policy(self, section, option_name):
         """Return the policy for the given (section, option_name) pair."""
         return POLICY_NONE
@@ -442,6 +913,14 @@
         """See Config.log_format."""
         return self._get_user_option('log_format')
 
+    def _validate_signatures_in_log(self):
+        """See Config.validate_signatures_in_log."""
+        return self._get_user_option('validate_signatures_in_log')
+
+    def _acceptable_keys(self):
+        """See Config.acceptable_keys."""
+        return self._get_user_option('acceptable_keys')
+
     def _post_commit(self):
         """See Config.post_commit."""
         return self._get_user_option('post_commit')
@@ -478,24 +957,143 @@
     def _get_nickname(self):
         return self.get_user_option('nickname')
 
+    def remove_user_option(self, option_name, section_name=None):
+        """Remove a user option and save the configuration file.
+
+        :param option_name: The option to be removed.
+
+        :param section_name: The section the option is defined in, default to
+            the default section.
+        """
+        self.reload()
+        parser = self._get_parser()
+        if section_name is None:
+            section = parser
+        else:
+            section = parser[section_name]
+        try:
+            del section[option_name]
+        except KeyError:
+            raise errors.NoSuchConfigOption(option_name)
+        self._write_config_file()
+        for hook in OldConfigHooks['remove']:
+            hook(self, option_name)
+
     def _write_config_file(self):
-        filename = self._get_filename()
-        atomic_file = atomicfile.AtomicFile(filename)
+        if self.file_name is None:
+            raise AssertionError('We cannot save, self.file_name is None')
+        conf_dir = os.path.dirname(self.file_name)
+        ensure_config_dir_exists(conf_dir)
+        atomic_file = atomicfile.AtomicFile(self.file_name)
         self._get_parser().write(atomic_file)
         atomic_file.commit()
         atomic_file.close()
-        osutils.copy_ownership_from_path(filename)
-
-
-class GlobalConfig(IniBasedConfig):
+        osutils.copy_ownership_from_path(self.file_name)
+        for hook in OldConfigHooks['save']:
+            hook(self)
+
+
+class LockableConfig(IniBasedConfig):
+    """A configuration needing explicit locking for access.
+
+    If several processes try to write the config file, the accesses need to be
+    serialized.
+
+    Daughter classes should decorate all methods that update a config with the
+    ``@needs_write_lock`` decorator (they call, directly or indirectly, the
+    ``_write_config_file()`` method. These methods (typically ``set_option()``
+    and variants must reload the config file from disk before calling
+    ``_write_config_file()``), this can be achieved by calling the
+    ``self.reload()`` method. Note that the lock scope should cover both the
+    reading and the writing of the config file which is why the decorator can't
+    be applied to ``_write_config_file()`` only.
+
+    This should be enough to implement the following logic:
+    - lock for exclusive write access,
+    - reload the config file from disk,
+    - set the new value
+    - unlock
+
+    This logic guarantees that a writer can update a value without erasing an
+    update made by another writer.
+    """
+
+    lock_name = 'lock'
+
+    def __init__(self, file_name):
+        super(LockableConfig, self).__init__(file_name=file_name)
+        self.dir = osutils.dirname(osutils.safe_unicode(self.file_name))
+        # FIXME: It doesn't matter that we don't provide possible_transports
+        # below since this is currently used only for local config files ;
+        # local transports are not shared. But if/when we start using
+        # LockableConfig for other kind of transports, we will need to reuse
+        # whatever connection is already established -- vila 20100929
+        self.transport = transport.get_transport_from_path(self.dir)
+        self._lock = lockdir.LockDir(self.transport, self.lock_name)
+
+    def _create_from_string(self, unicode_bytes, save):
+        super(LockableConfig, self)._create_from_string(unicode_bytes, False)
+        if save:
+            # We need to handle the saving here (as opposed to IniBasedConfig)
+            # to be able to lock
+            self.lock_write()
+            self._write_config_file()
+            self.unlock()
+
+    def lock_write(self, token=None):
+        """Takes a write lock in the directory containing the config file.
+
+        If the directory doesn't exist it is created.
+        """
+        ensure_config_dir_exists(self.dir)
+        return self._lock.lock_write(token)
+
+    def unlock(self):
+        self._lock.unlock()
+
+    def break_lock(self):
+        self._lock.break_lock()
+
+    @needs_write_lock
+    def remove_user_option(self, option_name, section_name=None):
+        super(LockableConfig, self).remove_user_option(option_name,
+                                                       section_name)
+
+    def _write_config_file(self):
+        if self._lock is None or not self._lock.is_held:
+            # NB: if the following exception is raised it probably means a
+            # missing @needs_write_lock decorator on one of the callers.
+            raise errors.ObjectNotLocked(self)
+        super(LockableConfig, self)._write_config_file()
+
+
+class GlobalConfig(LockableConfig):
     """The configuration that should be used for a specific location."""
 
+    def __init__(self):
+        super(GlobalConfig, self).__init__(file_name=config_filename())
+
+    def config_id(self):
+        return 'bazaar'
+
+    @classmethod
+    def from_string(cls, str_or_unicode, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This
+            will be utf-8 encoded.
+
+        :param save: Whether the file should be saved upon creation.
+        """
+        conf = cls()
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def get_editor(self):
         return self._get_user_option('editor')
 
-    def __init__(self):
-        super(GlobalConfig, self).__init__(config_filename)
-
+    @needs_write_lock
     def set_user_option(self, option, value):
         """Save option and its value in the configuration."""
         self._set_option(option, value, 'DEFAULT')
@@ -507,12 +1105,15 @@
         else:
             return {}
 
+    @needs_write_lock
     def set_alias(self, alias_name, alias_command):
         """Save the alias in the configuration."""
         self._set_option(alias_name, alias_command, 'ALIASES')
 
+    @needs_write_lock
     def unset_alias(self, alias_name):
         """Unset an existing alias."""
+        self.reload()
         aliases = self._get_parser().get('ALIASES')
         if not aliases or alias_name not in aliases:
             raise errors.NoSuchAlias(alias_name)
@@ -520,30 +1121,93 @@
         self._write_config_file()
 
     def _set_option(self, option, value, section):
-        # FIXME: RBC 20051029 This should refresh the parser and also take a
-        # file lock on bazaar.conf.
-        conf_dir = os.path.dirname(self._get_filename())
-        ensure_config_dir_exists(conf_dir)
+        self.reload()
         self._get_parser().setdefault(section, {})[option] = value
         self._write_config_file()
-
-
-class LocationConfig(IniBasedConfig):
+        for hook in OldConfigHooks['set']:
+            hook(self, option, value)
+
+    def _get_sections(self, name=None):
+        """See IniBasedConfig._get_sections()."""
+        parser = self._get_parser()
+        # We don't give access to options defined outside of any section, we
+        # used the DEFAULT section by... default.
+        if name in (None, 'DEFAULT'):
+            # This could happen for an empty file where the DEFAULT section
+            # doesn't exist yet. So we force DEFAULT when yielding
+            name = 'DEFAULT'
+            if 'DEFAULT' not in parser:
+               parser['DEFAULT']= {}
+        yield (name, parser[name], self.config_id())
+
+    @needs_write_lock
+    def remove_user_option(self, option_name, section_name=None):
+        if section_name is None:
+            # We need to force the default section.
+            section_name = 'DEFAULT'
+        # We need to avoid the LockableConfig implementation or we'll lock
+        # twice
+        super(LockableConfig, self).remove_user_option(option_name,
+                                                       section_name)
+
+def _iter_for_location_by_parts(sections, location):
+    """Keep only the sessions matching the specified location.
+
+    :param sections: An iterable of section names.
+
+    :param location: An url or a local path to match against.
+
+    :returns: An iterator of (section, extra_path, nb_parts) where nb is the
+        number of path components in the section name, section is the section
+        name and extra_path is the difference between location and the section
+        name.
+
+    ``location`` will always be a local path and never a 'file://' url but the
+    section names themselves can be in either form.
+    """
+    location_parts = location.rstrip('/').split('/')
+
+    for section in sections:
+        # location is a local path if possible, so we need to convert 'file://'
+        # urls in section names to local paths if necessary.
+
+        # This also avoids having file:///path be a more exact
+        # match than '/path'.
+
+        # FIXME: This still raises an issue if a user defines both file:///path
+        # *and* /path. Should we raise an error in this case -- vila 20110505
+
+        if section.startswith('file://'):
+            section_path = urlutils.local_path_from_url(section)
+        else:
+            section_path = section
+        section_parts = section_path.rstrip('/').split('/')
+
+        matched = True
+        if len(section_parts) > len(location_parts):
+            # More path components in the section, they can't match
+            matched = False
+        else:
+            # Rely on zip truncating in length to the length of the shortest
+            # argument sequence.
+            names = zip(location_parts, section_parts)
+            for name in names:
+                if not fnmatch.fnmatch(name[0], name[1]):
+                    matched = False
+                    break
+        if not matched:
+            continue
+        # build the path difference between the section and the location
+        extra_path = '/'.join(location_parts[len(section_parts):])
+        yield section, extra_path, len(section_parts)
+
+
+class LocationConfig(LockableConfig):
     """A configuration object that gives the policy for a location."""
 
     def __init__(self, location):
-        name_generator = locations_config_filename
-        if (not os.path.exists(name_generator()) and
-                os.path.exists(branches_config_filename())):
-            if sys.platform == 'win32':
-                trace.warning('Please rename %s to %s'
-                              % (branches_config_filename(),
-                                 locations_config_filename()))
-            else:
-                trace.warning('Please rename ~/.bazaar/branches.conf'
-                              ' to ~/.bazaar/locations.conf')
-            name_generator = branches_config_filename
-        super(LocationConfig, self).__init__(name_generator)
+        super(LocationConfig, self).__init__(
+            file_name=locations_config_filename())
         # local file locations are looked up by local path, rather than
         # by file url. This is because the config file is a user
         # file, and we would rather not expose the user to file urls.
@@ -551,50 +1215,48 @@
             location = urlutils.local_path_from_url(location)
         self.location = location
 
+    def config_id(self):
+        return 'locations'
+
+    @classmethod
+    def from_string(cls, str_or_unicode, location, save=False):
+        """Create a config object from a string.
+
+        :param str_or_unicode: A string representing the file content. This will
+            be utf-8 encoded.
+
+        :param location: The location url to filter the configuration.
+
+        :param save: Whether the file should be saved upon creation.
+        """
+        conf = cls(location)
+        conf._create_from_string(str_or_unicode, save)
+        return conf
+
     def _get_matching_sections(self):
         """Return an ordered list of section names matching this location."""
-        sections = self._get_parser()
-        location_names = self.location.split('/')
-        if self.location.endswith('/'):
-            del location_names[-1]
-        matches=[]
-        for section in sections:
-            # location is a local path if possible, so we need
-            # to convert 'file://' urls to local paths if necessary.
-            # This also avoids having file:///path be a more exact
-            # match than '/path'.
-            if section.startswith('file://'):
-                section_path = urlutils.local_path_from_url(section)
-            else:
-                section_path = section
-            section_names = section_path.split('/')
-            if section.endswith('/'):
-                del section_names[-1]
-            names = zip(location_names, section_names)
-            matched = True
-            for name in names:
-                if not fnmatch(name[0], name[1]):
-                    matched = False
-                    break
-            if not matched:
-                continue
-            # so, for the common prefix they matched.
-            # if section is longer, no match.
-            if len(section_names) > len(location_names):
-                continue
-            matches.append((len(section_names), section,
-                            '/'.join(location_names[len(section_names):])))
-        matches.sort(reverse=True)
-        sections = []
-        for (length, section, extra_path) in matches:
-            sections.append((section, extra_path))
+        matches = list(_iter_for_location_by_parts(self._get_parser(),
+                                                   self.location))
+        # put the longest (aka more specific) locations first
+        matches.sort(
+            key=lambda (section, extra_path, length): (length, section),
+            reverse=True)
+        for (section, extra_path, length) in matches:
+            yield section, extra_path
             # should we stop looking for parent configs here?
             try:
                 if self._get_parser()[section].as_bool('ignore_parents'):
                     break
             except KeyError:
                 pass
-        return sections
+
+    def _get_sections(self, name=None):
+        """See IniBasedConfig._get_sections()."""
+        # We ignore the name here as the only sections handled are named with
+        # the location path and we don't expose embedded sections either.
+        parser = self._get_parser()
+        for name, extra_path in self._get_matching_sections():
+            yield (name, parser[name], self.config_id())
 
     def _get_option_policy(self, section, option_name):
         """Return the policy for the given (section, option_name) pair."""
@@ -644,6 +1306,7 @@
             if policy_key in self._get_parser()[section]:
                 del self._get_parser()[section][policy_key]
 
+    @needs_write_lock
     def set_user_option(self, option, value, store=STORE_LOCATION):
         """Save option and its value in the configuration."""
         if store not in [STORE_LOCATION,
@@ -651,30 +1314,43 @@
                          STORE_LOCATION_APPENDPATH]:
             raise ValueError('bad storage policy %r for %r' %
                 (store, option))
-        # FIXME: RBC 20051029 This should refresh the parser and also take a
-        # file lock on locations.conf.
-        conf_dir = os.path.dirname(self._get_filename())
-        ensure_config_dir_exists(conf_dir)
+        self.reload()
         location = self.location
         if location.endswith('/'):
             location = location[:-1]
-        if (not location in self._get_parser() and
-            not location + '/' in self._get_parser()):
-            self._get_parser()[location]={}
-        elif location + '/' in self._get_parser():
+        parser = self._get_parser()
+        if not location in parser and not location + '/' in parser:
+            parser[location] = {}
+        elif location + '/' in parser:
             location = location + '/'
-        self._get_parser()[location][option]=value
+        parser[location][option]=value
         # the allowed values of store match the config policies
         self._set_option_policy(location, option, store)
         self._write_config_file()
+        for hook in OldConfigHooks['set']:
+            hook(self, option, value)
 
 
 class BranchConfig(Config):
     """A configuration object giving the policy for a branch."""
 
+    def __init__(self, branch):
+        super(BranchConfig, self).__init__()
+        self._location_config = None
+        self._branch_data_config = None
+        self._global_config = None
+        self.branch = branch
+        self.option_sources = (self._get_location_config,
+                               self._get_branch_data_config,
+                               self._get_global_config)
+
+    def config_id(self):
+        return 'branch'
+
     def _get_branch_data_config(self):
         if self._branch_data_config is None:
             self._branch_data_config = TreeConfig(self.branch)
+            self._branch_data_config.config_id = self.config_id
         return self._branch_data_config
 
     def _get_location_config(self):
@@ -724,7 +1400,7 @@
             return (self.branch._transport.get_bytes("email")
                     .decode(osutils.get_user_encoding())
                     .rstrip("\r\n"))
-        except errors.NoSuchFile, e:
+        except (errors.NoSuchFile, errors.PermissionDenied), e:
             pass
 
         return self._get_best_value('_get_user_id')
@@ -748,6 +1424,32 @@
                 return value
         return None
 
+    def _get_sections(self, name=None):
+        """See IniBasedConfig.get_sections()."""
+        for source in self.option_sources:
+            for section in source()._get_sections(name):
+                yield section
+
+    def _get_options(self, sections=None):
+        opts = []
+        # First the locations options
+        for option in self._get_location_config()._get_options():
+            yield option
+        # Then the branch options
+        branch_config = self._get_branch_data_config()
+        if sections is None:
+            sections = [('DEFAULT', branch_config._get_parser())]
+        # FIXME: We shouldn't have to duplicate the code in IniBasedConfig but
+        # Config itself has no notion of sections :( -- vila 20101001
+        config_id = self.config_id()
+        for (section_name, section) in sections:
+            for (name, value) in section.iteritems():
+                yield (name, value, section_name,
+                       config_id, branch_config._get_parser())
+        # Then the global options
+        for option in self._get_global_config()._get_options():
+            yield option
+
     def set_user_option(self, name, value, store=STORE_BRANCH,
         warn_masked=False):
         if store == STORE_BRANCH:
@@ -771,20 +1473,13 @@
                         trace.warning('Value "%s" is masked by "%s" from'
                                       ' branch.conf', value, mask_value)
 
+    def remove_user_option(self, option_name, section_name=None):
+        self._get_branch_data_config().remove_option(option_name, section_name)
+
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
         return self._get_safe_value('_gpg_signing_command')
 
-    def __init__(self, branch):
-        super(BranchConfig, self).__init__()
-        self._location_config = None
-        self._branch_data_config = None
-        self._global_config = None
-        self.branch = branch
-        self.option_sources = (self._get_location_config,
-                               self._get_branch_data_config,
-                               self._get_global_config)
-
     def _post_commit(self):
         """See Config.post_commit."""
         return self._get_safe_value('_post_commit')
@@ -806,6 +1501,14 @@
         """See Config.log_format."""
         return self._get_best_value('_log_format')
 
+    def _validate_signatures_in_log(self):
+        """See Config.validate_signatures_in_log."""
+        return self._get_best_value('_validate_signatures_in_log')
+
+    def _acceptable_keys(self):
+        """See Config.acceptable_keys."""
+        return self._get_best_value('_acceptable_keys')
+
 
 def ensure_config_dir_exists(path=None):
     """Make sure a configuration directory exists.
@@ -820,7 +1523,7 @@
             parent_dir = os.path.dirname(path)
             if not os.path.isdir(parent_dir):
                 trace.mutter('creating config parent directory: %r', parent_dir)
-            os.mkdir(parent_dir)
+                os.mkdir(parent_dir)
         trace.mutter('creating config directory: %r', path)
         os.mkdir(path)
         osutils.copy_ownership_from_path(path)
@@ -829,22 +1532,46 @@
 def config_dir():
     """Return per-user configuration directory.
 
-    By default this is ~/.bazaar/
+    By default this is %APPDATA%/bazaar/2.0 on Windows, ~/.bazaar on Mac OS X
+    and Linux.  On Linux, if there is a $XDG_CONFIG_HOME/bazaar directory,
+    that will be used instead.
 
     TODO: Global option --config-dir to override this.
     """
     base = os.environ.get('BZR_HOME', None)
     if sys.platform == 'win32':
+        # environ variables on Windows are in user encoding/mbcs. So decode
+        # before using one
+        if base is not None:
+            base = base.decode('mbcs')
         if base is None:
             base = win32utils.get_appdata_location_unicode()
         if base is None:
             base = os.environ.get('HOME', None)
+            if base is not None:
+                base = base.decode('mbcs')
         if base is None:
             raise errors.BzrError('You must have one of BZR_HOME, APPDATA,'
                                   ' or HOME set')
         return osutils.pathjoin(base, 'bazaar', '2.0')
     else:
-        if base is None:
+        if base is not None:
+            base = base.decode(osutils._fs_enc)
+    if sys.platform == 'darwin':
+        if base is None:
+            # this takes into account $HOME
+            base = os.path.expanduser("~")
+        return osutils.pathjoin(base, '.bazaar')
+    else:
+        if base is None:
+            xdg_dir = os.environ.get('XDG_CONFIG_HOME', None)
+            if xdg_dir is None:
+                xdg_dir = osutils.pathjoin(os.path.expanduser("~"), ".config")
+            xdg_dir = osutils.pathjoin(xdg_dir, 'bazaar')
+            if osutils.isdir(xdg_dir):
+                trace.mutter(
+                    "Using configuration in XDG directory %s." % xdg_dir)
+                return xdg_dir
             base = os.path.expanduser("~")
         return osutils.pathjoin(base, ".bazaar")
 
@@ -854,11 +1581,6 @@
     return osutils.pathjoin(config_dir(), 'bazaar.conf')
 
 
-def branches_config_filename():
-    """Return per-user configuration ini file filename."""
-    return osutils.pathjoin(config_dir(), 'branches.conf')
-
-
 def locations_config_filename():
     """Return per-user configuration ini file filename."""
     return osutils.pathjoin(config_dir(), 'locations.conf')
@@ -901,6 +1623,86 @@
         return os.path.expanduser('~/.cache')
 
 
+def _get_default_mail_domain():
+    """If possible, return the assumed default email domain.
+
+    :returns: string mail domain, or None.
+    """
+    if sys.platform == 'win32':
+        # No implementation yet; patches welcome
+        return None
+    try:
+        f = open('/etc/mailname')
+    except (IOError, OSError), e:
+        return None
+    try:
+        domain = f.read().strip()
+        return domain
+    finally:
+        f.close()
+
+
+def _auto_user_id():
+    """Calculate automatic user identification.
+
+    :returns: (realname, email), either of which may be None if they can't be
+    determined.
+
+    Only used when none is set in the environment or the id file.
+
+    This only returns an email address if we can be fairly sure the 
+    address is reasonable, ie if /etc/mailname is set on unix.
+
+    This doesn't use the FQDN as the default domain because that may be 
+    slow, and it doesn't use the hostname alone because that's not normally 
+    a reasonable address.
+    """
+    if sys.platform == 'win32':
+        # No implementation to reliably determine Windows default mail
+        # address; please add one.
+        return None, None
+
+    default_mail_domain = _get_default_mail_domain()
+    if not default_mail_domain:
+        return None, None
+
+    import pwd
+    uid = os.getuid()
+    try:
+        w = pwd.getpwuid(uid)
+    except KeyError:
+        trace.mutter('no passwd entry for uid %d?' % uid)
+        return None, None
+
+    # we try utf-8 first, because on many variants (like Linux),
+    # /etc/passwd "should" be in utf-8, and because it's unlikely to give
+    # false positives.  (many users will have their user encoding set to
+    # latin-1, which cannot raise UnicodeError.)
+    try:
+        gecos = w.pw_gecos.decode('utf-8')
+        encoding = 'utf-8'
+    except UnicodeError:
+        try:
+            encoding = osutils.get_user_encoding()
+            gecos = w.pw_gecos.decode(encoding)
+        except UnicodeError, e:
+            trace.mutter("cannot decode passwd entry %s" % w)
+            return None, None
+    try:
+        username = w.pw_name.decode(encoding)
+    except UnicodeError, e:
+        trace.mutter("cannot decode passwd entry %s" % w)
+        return None, None
+
+    comma = gecos.find(',')
+    if comma == -1:
+        realname = gecos
+    else:
+        realname = gecos[:comma]
+
+    return realname, (username + '@' + default_mail_domain)
+
+
 def parse_username(username):
     """Parse e-mail username and return a (name, address) tuple."""
     match = re.match(r'(.*?)\s*<?([\w+.-]+@[\w+.-]+)>?', username)
@@ -949,12 +1751,23 @@
 
     def set_option(self, value, name, section=None):
         """Set a per-branch configuration option"""
+        # FIXME: We shouldn't need to lock explicitly here but rather rely on
+        # higher levels providing the right lock -- vila 20101004
         self.branch.lock_write()
         try:
             self._config.set_option(value, name, section)
         finally:
             self.branch.unlock()
 
+    def remove_option(self, option_name, section_name=None):
+        # FIXME: We shouldn't need to lock explicitly here but rather rely on
+        # higher levels providing the right lock -- vila 20101004
+        self.branch.lock_write()
+        try:
+            self._config.remove_option(option_name, section_name)
+        finally:
+            self.branch.unlock()
+
 
 class AuthenticationConfig(object):
     """The authentication configuration file based on a ini file.
@@ -986,6 +1799,8 @@
             self._config = ConfigObj(self._input, encoding='utf-8')
         except configobj.ConfigObjError, e:
             raise errors.ParseConfigError(e.errors, e.config.filename)
+        except UnicodeError:
+            raise errors.ConfigContentError(self._filename)
         return self._config
 
     def _save(self):
@@ -1008,7 +1823,7 @@
         section[option_name] = value
         self._save()
 
-    def get_credentials(self, scheme, host, port=None, user=None, path=None, 
+    def get_credentials(self, scheme, host, port=None, user=None, path=None,
                         realm=None):
         """Returns the matching credentials from authentication.conf file.
 
@@ -1182,7 +1997,7 @@
             if ask:
                 if prompt is None:
                     # Create a default prompt suitable for most cases
-                    prompt = scheme.upper() + ' %(host)s username'
+                    prompt = u'%s' % (scheme.upper(),) + u' %(host)s username'
                 # Special handling for optional fields in the prompt
                 if port is not None:
                     prompt_host = '%s:%d' % (host, port)
@@ -1226,7 +2041,7 @@
         if password is None:
             if prompt is None:
                 # Create a default prompt suitable for most cases
-                prompt = '%s' % scheme.upper() + ' %(user)s@%(host)s password'
+                prompt = u'%s' % scheme.upper() + u' %(user)s@%(host)s password'
             # Special handling for optional fields in the prompt
             if port is not None:
                 prompt_host = '%s:%d' % (host, port)
@@ -1403,8 +2218,8 @@
     """A Config that reads/writes a config file on a Transport.
 
     It is a low-level object that considers config data to be name/value pairs
-    that may be associated with a section.  Assigning meaning to the these
-    values is done at higher levels like TreeConfig.
+    that may be associated with a section.  Assigning meaning to these values
+    is done at higher levels like TreeConfig.
     """
 
     def __init__(self, transport, filename):
@@ -1427,7 +2242,10 @@
                 section_obj = configobj[section]
             except KeyError:
                 return default
-        return section_obj.get(name, default)
+        value = section_obj.get(name, default)
+        for hook in OldConfigHooks['get']:
+            hook(self, name, value)
+        return value
 
     def set_option(self, value, name, section=None):
         """Set the value associated with a named option.
@@ -1441,23 +2259,1307 @@
             configobj[name] = value
         else:
             configobj.setdefault(section, {})[name] = value
+        for hook in OldConfigHooks['set']:
+            hook(self, name, value)
+        self._set_configobj(configobj)
+
+    def remove_option(self, option_name, section_name=None):
+        configobj = self._get_configobj()
+        if section_name is None:
+            del configobj[option_name]
+        else:
+            del configobj[section_name][option_name]
+        for hook in OldConfigHooks['remove']:
+            hook(self, option_name)
         self._set_configobj(configobj)
 
     def _get_config_file(self):
         try:
-            return StringIO(self._transport.get_bytes(self._filename))
+            f = StringIO(self._transport.get_bytes(self._filename))
+            for hook in OldConfigHooks['load']:
+                hook(self)
+            return f
         except errors.NoSuchFile:
             return StringIO()
+        except errors.PermissionDenied, e:
+            trace.warning("Permission denied while trying to open "
+                "configuration file %s.", urlutils.unescape_for_display(
+                urlutils.join(self._transport.base, self._filename), "utf-8"))
+            return StringIO()
+
+    def _external_url(self):
+        return urlutils.join(self._transport.external_url(), self._filename)
 
     def _get_configobj(self):
         f = self._get_config_file()
         try:
-            return ConfigObj(f, encoding='utf-8')
+            try:
+                conf = ConfigObj(f, encoding='utf-8')
+            except configobj.ConfigObjError, e:
+                raise errors.ParseConfigError(e.errors, self._external_url())
+            except UnicodeDecodeError:
+                raise errors.ConfigContentError(self._external_url())
         finally:
             f.close()
+        return conf
 
     def _set_configobj(self, configobj):
         out_file = StringIO()
         configobj.write(out_file)
         out_file.seek(0)
         self._transport.put_file(self._filename, out_file)
+        for hook in OldConfigHooks['save']:
+            hook(self)
+
+
+class Option(object):
+    """An option definition.
+
+    The option *values* are stored in config files and found in sections.
+
+    Here we define various properties about the option itself, its default
+    value, how to convert it from stores, what to do when invalid values are
+    encoutered, in which config files it can be stored.
+    """
+
+    def __init__(self, name, default=None, default_from_env=None,
+                 help=None,
+                 from_unicode=None, invalid=None):
+        """Build an option definition.
+
+        :param name: the name used to refer to the option.
+
+        :param default: the default value to use when none exist in the config
+            stores. This is either a string that ``from_unicode`` will convert
+            into the proper type or a python object that can be stringified (so
+            only the empty list is supported for example).
+
+        :param default_from_env: A list of environment variables which can
+           provide a default value. 'default' will be used only if none of the
+           variables specified here are set in the environment.
+
+        :param help: a doc string to explain the option to the user.
+
+        :param from_unicode: a callable to convert the unicode string
+            representing the option value in a store. This is not called for
+            the default value.
+
+        :param invalid: the action to be taken when an invalid value is
+            encountered in a store. This is called only when from_unicode is
+            invoked to convert a string and returns None or raise ValueError or
+            TypeError. Accepted values are: None (ignore invalid values),
+            'warning' (emit a warning), 'error' (emit an error message and
+            terminates).
+        """
+        if default_from_env is None:
+            default_from_env = []
+        self.name = name
+        # Convert the default value to a unicode string so all values are
+        # strings internally before conversion (via from_unicode) is attempted.
+        if default is None:
+            self.default = None
+        elif isinstance(default, list):
+            # Only the empty list is supported
+            if default:
+                raise AssertionError(
+                    'Only empty lists are supported as default values')
+            self.default = u','
+        elif isinstance(default, (str, unicode, bool, int, float)):
+            # Rely on python to convert strings, booleans and integers
+            self.default = u'%s' % (default,)
+        else:
+            # other python objects are not expected
+            raise AssertionError('%r is not supported as a default value'
+                                 % (default,))
+        self.default_from_env = default_from_env
+        self.help = help
+        self.from_unicode = from_unicode
+        if invalid and invalid not in ('warning', 'error'):
+            raise AssertionError("%s not supported for 'invalid'" % (invalid,))
+        self.invalid = invalid
+
+    def convert_from_unicode(self, unicode_value):
+        if self.from_unicode is None or unicode_value is None:
+            # Don't convert or nothing to convert
+            return unicode_value
+        try:
+            converted = self.from_unicode(unicode_value)
+        except (ValueError, TypeError):
+            # Invalid values are ignored
+            converted = None
+        if converted is None and self.invalid is not None:
+            # The conversion failed
+            if self.invalid == 'warning':
+                trace.warning('Value "%s" is not valid for "%s"',
+                              unicode_value, self.name)
+            elif self.invalid == 'error':
+                raise errors.ConfigOptionValueError(self.name, unicode_value)
+        return converted
+
+    def get_default(self):
+        value = None
+        for var in self.default_from_env:
+            try:
+                # If the env variable is defined, its value is the default one
+                value = os.environ[var]
+                break
+            except KeyError:
+                continue
+        if value is None:
+            # Otherwise, fallback to the value defined at registration
+            value = self.default
+        return value
+
+    def get_help_text(self, additional_see_also=None, plain=True):
+        result = self.help
+        from bzrlib import help_topics
+        result += help_topics._format_see_also(additional_see_also)
+        if plain:
+            result = help_topics.help_as_plain_text(result)
+        return result
+
+
+# Predefined converters to get proper values from store
+
+def bool_from_store(unicode_str):
+    return ui.bool_from_string(unicode_str)
+
+
+def int_from_store(unicode_str):
+    return int(unicode_str)
+
+
+def float_from_store(unicode_str):
+    return float(unicode_str)
+
+
+
+# Use a an empty dict to initialize an empty configobj avoiding all
+# parsing and encoding checks
+_list_converter_config = configobj.ConfigObj(
+    {}, encoding='utf-8', list_values=True, interpolation=False)
+
+
+def list_from_store(unicode_str):
+    if not isinstance(unicode_str, basestring):
+        raise TypeError
+    # Now inject our string directly as unicode. All callers got their value
+    # from configobj, so values that need to be quoted are already properly
+    # quoted.
+    _list_converter_config.reset()
+    _list_converter_config._parse([u"list=%s" % (unicode_str,)])
+    maybe_list = _list_converter_config['list']
+    # ConfigObj return '' instead of u''. Use 'str' below to catch all cases.
+    if isinstance(maybe_list, basestring):
+        if maybe_list:
+            # A single value, most probably the user forgot (or didn't care to
+            # add) the final ','
+            l = [maybe_list]
+        else:
+            # The empty string, convert to empty list
+            l = []
+    else:
+        # We rely on ConfigObj providing us with a list already
+        l = maybe_list
+    return l
+
+
+class OptionRegistry(registry.Registry):
+    """Register config options by their name.
+
+    This overrides ``registry.Registry`` to simplify registration by acquiring
+    some information from the option object itself.
+    """
+
+    def register(self, option):
+        """Register a new option to its name.
+
+        :param option: The option to register. Its name is used as the key.
+        """
+        super(OptionRegistry, self).register(option.name, option,
+                                             help=option.help)
+
+    def register_lazy(self, key, module_name, member_name):
+        """Register a new option to be loaded on request.
+
+        :param key: the key to request the option later. Since the registration
+            is lazy, it should be provided and match the option name.
+
+        :param module_name: the python path to the module. Such as 'os.path'.
+
+        :param member_name: the member of the module to return.  If empty or 
+                None, get() will return the module itself.
+        """
+        super(OptionRegistry, self).register_lazy(key,
+                                                  module_name, member_name)
+
+    def get_help(self, key=None):
+        """Get the help text associated with the given key"""
+        option = self.get(key)
+        the_help = option.help
+        if callable(the_help):
+            return the_help(self, key)
+        return the_help
+
+
+option_registry = OptionRegistry()
+
+
+# Registered options in lexicographical order
+
+option_registry.register(
+    Option('bzr.workingtree.worth_saving_limit', default=10,
+           from_unicode=int_from_store,  invalid='warning',
+           help='''\
+How many changes before saving the dirstate.
+
+-1 means that we will never rewrite the dirstate file for only
+stat-cache changes. Regardless of this setting, we will always rewrite
+the dirstate file if a file is added/removed/renamed/etc. This flag only
+affects the behavior of updating the dirstate file after we notice that
+a file has been touched.
+'''))
+option_registry.register(
+    Option('dirstate.fdatasync', default=True,
+           from_unicode=bool_from_store,
+           help='''\
+Flush dirstate changes onto physical disk?
+
+If true (default), working tree metadata changes are flushed through the
+OS buffers to physical disk.  This is somewhat slower, but means data
+should not be lost if the machine crashes.  See also repository.fdatasync.
+'''))
+option_registry.register(
+    Option('debug_flags', default=[], from_unicode=list_from_store,
+           help='Debug flags to activate.'))
+option_registry.register(
+    Option('default_format', default='2a',
+           help='Format used when creating branches.'))
+option_registry.register(
+    Option('dpush_strict', default=None,
+           from_unicode=bool_from_store,
+           help='''\
+The default value for ``dpush --strict``.
+
+If present, defines the ``--strict`` option default value for checking
+uncommitted changes before pushing into a different VCS without any
+custom bzr metadata.
+'''))
+option_registry.register(
+    Option('editor',
+           help='The command called to launch an editor to enter a message.'))
+option_registry.register(
+    Option('ignore_missing_extensions', default=False,
+           from_unicode=bool_from_store,
+           help='''\
+Control the missing extensions warning display.
+
+The warning will not be emitted if set to True.
+'''))
+option_registry.register(
+    Option('language',
+           help='Language to translate messages into.'))
+option_registry.register(
+    Option('locks.steal_dead', default=False, from_unicode=bool_from_store,
+           help='''\
+Steal locks that appears to be dead.
+
+If set to True, bzr will check if a lock is supposed to be held by an
+active process from the same user on the same machine. If the user and
+machine match, but no process with the given PID is active, then bzr
+will automatically break the stale lock, and create a new lock for
+this process.
+Otherwise, bzr will prompt as normal to break the lock.
+'''))
+option_registry.register(
+    Option('log_format', default='long',
+           help= '''\
+Log format to use when displaying revisions.
+
+Standard log formats are ``long``, ``short`` and ``line``. Additional formats
+may be provided by plugins.
+'''))
+option_registry.register(
+    Option('output_encoding',
+           help= 'Unicode encoding for output'
+           ' (terminal encoding if not specified).'))
+option_registry.register(
+    Option('push_strict', default=None,
+           from_unicode=bool_from_store,
+           help='''\
+The default value for ``push --strict``.
+
+If present, defines the ``--strict`` option default value for checking
+uncommitted changes before sending a merge directive.
+'''))
+option_registry.register(
+    Option('repository.fdatasync', default=True,
+           from_unicode=bool_from_store,
+           help='''\
+Flush repository changes onto physical disk?
+
+If true (default), repository changes are flushed through the OS buffers
+to physical disk.  This is somewhat slower, but means data should not be
+lost if the machine crashes.  See also dirstate.fdatasync.
+'''))
+option_registry.register(
+    Option('send_strict', default=None,
+           from_unicode=bool_from_store,
+           help='''\
+The default value for ``send --strict``.
+
+If present, defines the ``--strict`` option default value for checking
+uncommitted changes before pushing.
+'''))
+
+option_registry.register(
+    Option('serve.client_timeout',
+           default=300.0, from_unicode=float_from_store,
+           help="If we wait for a new request from a client for more than"
+                " X seconds, consider the client idle, and hangup."))
+
+
+class Section(object):
+    """A section defines a dict of option name => value.
+
+    This is merely a read-only dict which can add some knowledge about the
+    options. It is *not* a python dict object though and doesn't try to mimic
+    its API.
+    """
+
+    def __init__(self, section_id, options):
+        self.id = section_id
+        # We re-use the dict-like object received
+        self.options = options
+
+    def get(self, name, default=None):
+        return self.options.get(name, default)
+
+    def __repr__(self):
+        # Mostly for debugging use
+        return "<config.%s id=%s>" % (self.__class__.__name__, self.id)
+
+
+_NewlyCreatedOption = object()
+"""Was the option created during the MutableSection lifetime"""
+
+
+class MutableSection(Section):
+    """A section allowing changes and keeping track of the original values."""
+
+    def __init__(self, section_id, options):
+        super(MutableSection, self).__init__(section_id, options)
+        self.orig = {}
+
+    def set(self, name, value):
+        if name not in self.options:
+            # This is a new option
+            self.orig[name] = _NewlyCreatedOption
+        elif name not in self.orig:
+            self.orig[name] = self.get(name, None)
+        self.options[name] = value
+
+    def remove(self, name):
+        if name not in self.orig:
+            self.orig[name] = self.get(name, None)
+        del self.options[name]
+
+
+class CommandLineSection(MutableSection):
+    """A section used to carry command line overrides for the config options."""
+
+    def __init__(self, opts=None):
+        if opts is None:
+            opts = {}
+        super(CommandLineSection, self).__init__('cmdline-overrides', opts)
+
+    def _reset(self):
+        # The dict should be cleared but not replaced so it can be shared.
+        self.options.clear()
+
+    def _from_cmdline(self, overrides):
+        # Reset before accepting new definitions
+        self._reset()
+        for over in overrides:
+            try:
+                name, value = over.split('=', 1)
+            except ValueError:
+                raise errors.BzrCommandError(
+                    gettext("Invalid '%s', should be of the form 'name=value'")
+                    % (over,))
+            self.set(name, value)
+
+
+class Store(object):
+    """Abstract interface to persistent storage for configuration options."""
+
+    readonly_section_class = Section
+    mutable_section_class = MutableSection
+
+    def is_loaded(self):
+        """Returns True if the Store has been loaded.
+
+        This is used to implement lazy loading and ensure the persistent
+        storage is queried only when needed.
+        """
+        raise NotImplementedError(self.is_loaded)
+
+    def load(self):
+        """Loads the Store from persistent storage."""
+        raise NotImplementedError(self.load)
+
+    def _load_from_string(self, bytes):
+        """Create a store from a string in configobj syntax.
+
+        :param bytes: A string representing the file content.
+        """
+        raise NotImplementedError(self._load_from_string)
+
+    def unload(self):
+        """Unloads the Store.
+
+        This should make is_loaded() return False. This is used when the caller
+        knows that the persistent storage has changed or may have change since
+        the last load.
+        """
+        raise NotImplementedError(self.unload)
+
+    def save(self):
+        """Saves the Store to persistent storage."""
+        raise NotImplementedError(self.save)
+
+    def external_url(self):
+        raise NotImplementedError(self.external_url)
+
+    def get_sections(self):
+        """Returns an ordered iterable of existing sections.
+
+        :returns: An iterable of (name, dict).
+        """
+        raise NotImplementedError(self.get_sections)
+
+    def get_mutable_section(self, section_name=None):
+        """Returns the specified mutable section.
+
+        :param section_name: The section identifier
+        """
+        raise NotImplementedError(self.get_mutable_section)
+
+    def __repr__(self):
+        # Mostly for debugging use
+        return "<config.%s(%s)>" % (self.__class__.__name__,
+                                    self.external_url())
+
+
+class IniFileStore(Store):
+    """A config Store using ConfigObj for storage.
+
+    :ivar transport: The transport object where the config file is located.
+
+    :ivar file_name: The config file basename in the transport directory.
+
+    :ivar _config_obj: Private member to hold the ConfigObj instance used to
+        serialize/deserialize the config file.
+    """
+
+    def __init__(self, transport, file_name):
+        """A config Store using ConfigObj for storage.
+
+        :param transport: The transport object where the config file is located.
+
+        :param file_name: The config file basename in the transport directory.
+        """
+        super(IniFileStore, self).__init__()
+        self.transport = transport
+        self.file_name = file_name
+        self._config_obj = None
+
+    def is_loaded(self):
+        return self._config_obj != None
+
+    def unload(self):
+        self._config_obj = None
+
+    def load(self):
+        """Load the store from the associated file."""
+        if self.is_loaded():
+            return
+        try:
+            content = self.transport.get_bytes(self.file_name)
+        except errors.PermissionDenied:
+            trace.warning("Permission denied while trying to load "
+                          "configuration store %s.", self.external_url())
+            raise
+        self._load_from_string(content)
+        for hook in ConfigHooks['load']:
+            hook(self)
+
+    def _load_from_string(self, bytes):
+        """Create a config store from a string.
+
+        :param bytes: A string representing the file content.
+        """
+        if self.is_loaded():
+            raise AssertionError('Already loaded: %r' % (self._config_obj,))
+        co_input = StringIO(bytes)
+        try:
+            # The config files are always stored utf8-encoded
+            self._config_obj = ConfigObj(co_input, encoding='utf-8',
+                                         list_values=False)
+        except configobj.ConfigObjError, e:
+            self._config_obj = None
+            raise errors.ParseConfigError(e.errors, self.external_url())
+        except UnicodeDecodeError:
+            raise errors.ConfigContentError(self.external_url())
+
+    def save(self):
+        if not self.is_loaded():
+            # Nothing to save
+            return
+        out = StringIO()
+        self._config_obj.write(out)
+        self.transport.put_bytes(self.file_name, out.getvalue())
+        for hook in ConfigHooks['save']:
+            hook(self)
+
+    def external_url(self):
+        # FIXME: external_url should really accepts an optional relpath
+        # parameter (bug #750169) :-/ -- vila 2011-04-04
+        # The following will do in the interim but maybe we don't want to
+        # expose a path here but rather a config ID and its associated
+        # object </hand wawe>.
+        return urlutils.join(self.transport.external_url(), self.file_name)
+
+    def get_sections(self):
+        """Get the configobj section in the file order.
+
+        :returns: An iterable of (name, dict).
+        """
+        # We need a loaded store
+        try:
+            self.load()
+        except (errors.NoSuchFile, errors.PermissionDenied):
+            # If the file can't be read, there is no sections
+            return
+        cobj = self._config_obj
+        if cobj.scalars:
+            yield self.readonly_section_class(None, cobj)
+        for section_name in cobj.sections:
+            yield self.readonly_section_class(section_name, cobj[section_name])
+
+    def get_mutable_section(self, section_name=None):
+        # We need a loaded store
+        try:
+            self.load()
+        except errors.NoSuchFile:
+            # The file doesn't exist, let's pretend it was empty
+            self._load_from_string('')
+        if section_name is None:
+            section = self._config_obj
+        else:
+            section = self._config_obj.setdefault(section_name, {})
+        return self.mutable_section_class(section_name, section)
+
+
+# Note that LockableConfigObjStore inherits from ConfigObjStore because we need
+# unlockable stores for use with objects that can already ensure the locking
+# (think branches). If different stores (not based on ConfigObj) are created,
+# they may face the same issue.
+
+
+class LockableIniFileStore(IniFileStore):
+    """A ConfigObjStore using locks on save to ensure store integrity."""
+
+    def __init__(self, transport, file_name, lock_dir_name=None):
+        """A config Store using ConfigObj for storage.
+
+        :param transport: The transport object where the config file is located.
+
+        :param file_name: The config file basename in the transport directory.
+        """
+        if lock_dir_name is None:
+            lock_dir_name = 'lock'
+        self.lock_dir_name = lock_dir_name
+        super(LockableIniFileStore, self).__init__(transport, file_name)
+        self._lock = lockdir.LockDir(self.transport, self.lock_dir_name)
+
+    def lock_write(self, token=None):
+        """Takes a write lock in the directory containing the config file.
+
+        If the directory doesn't exist it is created.
+        """
+        # FIXME: This doesn't check the ownership of the created directories as
+        # ensure_config_dir_exists does. It should if the transport is local
+        # -- vila 2011-04-06
+        self.transport.create_prefix()
+        return self._lock.lock_write(token)
+
+    def unlock(self):
+        self._lock.unlock()
+
+    def break_lock(self):
+        self._lock.break_lock()
+
+    @needs_write_lock
+    def save(self):
+        # We need to be able to override the undecorated implementation
+        self.save_without_locking()
+
+    def save_without_locking(self):
+        super(LockableIniFileStore, self).save()
+
+
+# FIXME: global, bazaar, shouldn't that be 'user' instead or even
+# 'user_defaults' as opposed to 'user_overrides', 'system_defaults'
+# (/etc/bzr/bazaar.conf) and 'system_overrides' ? -- vila 2011-04-05
+
+# FIXME: Moreover, we shouldn't need classes for these stores either, factory
+# functions or a registry will make it easier and clearer for tests, focusing
+# on the relevant parts of the API that needs testing -- vila 20110503 (based
+# on a poolie's remark)
+class GlobalStore(LockableIniFileStore):
+
+    def __init__(self, possible_transports=None):
+        t = transport.get_transport_from_path(
+            config_dir(), possible_transports=possible_transports)
+        super(GlobalStore, self).__init__(t, 'bazaar.conf')
+
+
+class LocationStore(LockableIniFileStore):
+
+    def __init__(self, possible_transports=None):
+        t = transport.get_transport_from_path(
+            config_dir(), possible_transports=possible_transports)
+        super(LocationStore, self).__init__(t, 'locations.conf')
+
+
+class BranchStore(IniFileStore):
+
+    def __init__(self, branch):
+        super(BranchStore, self).__init__(branch.control_transport,
+                                          'branch.conf')
+        self.branch = branch
+
+    def lock_write(self, token=None):
+        return self.branch.lock_write(token)
+
+    def unlock(self):
+        return self.branch.unlock()
+
+    @needs_write_lock
+    def save(self):
+        # We need to be able to override the undecorated implementation
+        self.save_without_locking()
+
+    def save_without_locking(self):
+        super(BranchStore, self).save()
+
+
+class ControlStore(LockableIniFileStore):
+
+    def __init__(self, bzrdir):
+        super(ControlStore, self).__init__(bzrdir.transport,
+                                          'control.conf',
+                                           lock_dir_name='branch_lock')
+
+
+class SectionMatcher(object):
+    """Select sections into a given Store.
+
+    This is intended to be used to postpone getting an iterable of sections
+    from a store.
+    """
+
+    def __init__(self, store):
+        self.store = store
+
+    def get_sections(self):
+        # This is where we require loading the store so we can see all defined
+        # sections.
+        sections = self.store.get_sections()
+        # Walk the revisions in the order provided
+        for s in sections:
+            if self.match(s):
+                yield s
+
+    def match(self, section):
+        """Does the proposed section match.
+
+        :param section: A Section object.
+
+        :returns: True if the section matches, False otherwise.
+        """
+        raise NotImplementedError(self.match)
+
+
+class NameMatcher(SectionMatcher):
+
+    def __init__(self, store, section_id):
+        super(NameMatcher, self).__init__(store)
+        self.section_id = section_id
+
+    def match(self, section):
+        return section.id == self.section_id
+
+
+class LocationSection(Section):
+
+    def __init__(self, section, length, extra_path):
+        super(LocationSection, self).__init__(section.id, section.options)
+        self.length = length
+        self.extra_path = extra_path
+
+    def get(self, name, default=None):
+        value = super(LocationSection, self).get(name, default)
+        if value is not None:
+            policy_name = self.get(name + ':policy', None)
+            policy = _policy_value.get(policy_name, POLICY_NONE)
+            if policy == POLICY_APPENDPATH:
+                value = urlutils.join(value, self.extra_path)
+        return value
+
+
+class LocationMatcher(SectionMatcher):
+
+    def __init__(self, store, location):
+        super(LocationMatcher, self).__init__(store)
+        if location.startswith('file://'):
+            location = urlutils.local_path_from_url(location)
+        self.location = location
+
+    def _get_matching_sections(self):
+        """Get all sections matching ``location``."""
+        # We slightly diverge from LocalConfig here by allowing the no-name
+        # section as the most generic one and the lower priority.
+        no_name_section = None
+        all_sections = []
+        # Filter out the no_name_section so _iter_for_location_by_parts can be
+        # used (it assumes all sections have a name).
+        for section in self.store.get_sections():
+            if section.id is None:
+                no_name_section = section
+            else:
+                all_sections.append(section)
+        # Unfortunately _iter_for_location_by_parts deals with section names so
+        # we have to resync.
+        filtered_sections = _iter_for_location_by_parts(
+            [s.id for s in all_sections], self.location)
+        iter_all_sections = iter(all_sections)
+        matching_sections = []
+        if no_name_section is not None:
+            matching_sections.append(
+                LocationSection(no_name_section, 0, self.location))
+        for section_id, extra_path, length in filtered_sections:
+            # a section id is unique for a given store so it's safe to take the
+            # first matching section while iterating. Also, all filtered
+            # sections are part of 'all_sections' and will always be found
+            # there.
+            while True:
+                section = iter_all_sections.next()
+                if section_id == section.id:
+                    matching_sections.append(
+                        LocationSection(section, length, extra_path))
+                    break
+        return matching_sections
+
+    def get_sections(self):
+        # Override the default implementation as we want to change the order
+        matching_sections = self._get_matching_sections()
+        # We want the longest (aka more specific) locations first
+        sections = sorted(matching_sections,
+                          key=lambda section: (section.length, section.id),
+                          reverse=True)
+        # Sections mentioning 'ignore_parents' restrict the selection
+        for section in sections:
+            # FIXME: We really want to use as_bool below -- vila 2011-04-07
+            ignore = section.get('ignore_parents', None)
+            if ignore is not None:
+                ignore = ui.bool_from_string(ignore)
+            if ignore:
+                break
+            # Finally, we have a valid section
+            yield section
+
+
+class Stack(object):
+    """A stack of configurations where an option can be defined"""
+
+    _option_ref_re = lazy_regex.lazy_compile('({[^{}]+})')
+    """Describes an exandable option reference.
+
+    We want to match the most embedded reference first.
+
+    I.e. for '{{foo}}' we will get '{foo}',
+    for '{bar{baz}}' we will get '{baz}'
+    """
+
+    def __init__(self, sections_def, store=None, mutable_section_name=None):
+        """Creates a stack of sections with an optional store for changes.
+
+        :param sections_def: A list of Section or callables that returns an
+            iterable of Section. This defines the Sections for the Stack and
+            can be called repeatedly if needed.
+
+        :param store: The optional Store where modifications will be
+            recorded. If none is specified, no modifications can be done.
+
+        :param mutable_section_name: The name of the MutableSection where
+            changes are recorded. This requires the ``store`` parameter to be
+            specified.
+        """
+        self.sections_def = sections_def
+        self.store = store
+        self.mutable_section_name = mutable_section_name
+
+    def get(self, name, expand=None):
+        """Return the *first* option value found in the sections.
+
+        This is where we guarantee that sections coming from Store are loaded
+        lazily: the loading is delayed until we need to either check that an
+        option exists or get its value, which in turn may require to discover
+        in which sections it can be defined. Both of these (section and option
+        existence) require loading the store (even partially).
+
+        :param name: The queried option.
+
+        :param expand: Whether options references should be expanded.
+
+        :returns: The value of the option.
+        """
+        # FIXME: No caching of options nor sections yet -- vila 20110503
+        if expand is None:
+            expand = _get_expand_default_value()
+        value = None
+        # Ensuring lazy loading is achieved by delaying section matching (which
+        # implies querying the persistent storage) until it can't be avoided
+        # anymore by using callables to describe (possibly empty) section
+        # lists.
+        for section_or_callable in self.sections_def:
+            # Each section can expand to multiple ones when a callable is used
+            if callable(section_or_callable):
+                sections = section_or_callable()
+            else:
+                sections = [section_or_callable]
+            for section in sections:
+                value = section.get(name)
+                if value is not None:
+                    break
+            if value is not None:
+                break
+        # If the option is registered, it may provide additional info about
+        # value handling
+        try:
+            opt = option_registry.get(name)
+        except KeyError:
+            # Not registered
+            opt = None
+        def expand_and_convert(val):
+            # This may need to be called twice if the value is None or ends up
+            # being None during expansion or conversion.
+            if val is not None:
+                if expand:
+                    if isinstance(val, basestring):
+                        val = self._expand_options_in_string(val)
+                    else:
+                        trace.warning('Cannot expand "%s":'
+                                      ' %s does not support option expansion'
+                                      % (name, type(val)))
+                if opt is not None:
+                    val = opt.convert_from_unicode(val)
+            return val
+        value = expand_and_convert(value)
+        if opt is not None and value is None:
+            # If the option is registered, it may provide a default value
+            value = opt.get_default()
+            value = expand_and_convert(value)
+        for hook in ConfigHooks['get']:
+            hook(self, name, value)
+        return value
+
+    def expand_options(self, string, env=None):
+        """Expand option references in the string in the configuration context.
+
+        :param string: The string containing option(s) to expand.
+
+        :param env: An option dict defining additional configuration options or
+            overriding existing ones.
+
+        :returns: The expanded string.
+        """
+        return self._expand_options_in_string(string, env)
+
+    def _expand_options_in_string(self, string, env=None, _refs=None):
+        """Expand options in the string in the configuration context.
+
+        :param string: The string to be expanded.
+
+        :param env: An option dict defining additional configuration options or
+            overriding existing ones.
+
+        :param _refs: Private list (FIFO) containing the options being expanded
+            to detect loops.
+
+        :returns: The expanded string.
+        """
+        if string is None:
+            # Not much to expand there
+            return None
+        if _refs is None:
+            # What references are currently resolved (to detect loops)
+            _refs = []
+        result = string
+        # We need to iterate until no more refs appear ({{foo}} will need two
+        # iterations for example).
+        while True:
+            raw_chunks = Stack._option_ref_re.split(result)
+            if len(raw_chunks) == 1:
+                # Shorcut the trivial case: no refs
+                return result
+            chunks = []
+            # Split will isolate refs so that every other chunk is a ref
+            chunk_is_ref = False
+            for chunk in raw_chunks:
+                if not chunk_is_ref:
+                    chunks.append(chunk)
+                    chunk_is_ref = True
+                else:
+                    name = chunk[1:-1]
+                    if name in _refs:
+                        raise errors.OptionExpansionLoop(string, _refs)
+                    _refs.append(name)
+                    value = self._expand_option(name, env, _refs)
+                    if value is None:
+                        raise errors.ExpandingUnknownOption(name, string)
+                    chunks.append(value)
+                    _refs.pop()
+                    chunk_is_ref = False
+            result = ''.join(chunks)
+        return result
+
+    def _expand_option(self, name, env, _refs):
+        if env is not None and name in env:
+            # Special case, values provided in env takes precedence over
+            # anything else
+            value = env[name]
+        else:
+            # FIXME: This is a limited implementation, what we really need is a
+            # way to query the bzr config for the value of an option,
+            # respecting the scope rules (That is, once we implement fallback
+            # configs, getting the option value should restart from the top
+            # config, not the current one) -- vila 20101222
+            value = self.get(name, expand=False)
+            value = self._expand_options_in_string(value, env, _refs)
+        return value
+
+    def _get_mutable_section(self):
+        """Get the MutableSection for the Stack.
+
+        This is where we guarantee that the mutable section is lazily loaded:
+        this means we won't load the corresponding store before setting a value
+        or deleting an option. In practice the store will often be loaded but
+        this allows helps catching some programming errors.
+        """
+        section = self.store.get_mutable_section(self.mutable_section_name)
+        return section
+
+    def set(self, name, value):
+        """Set a new value for the option."""
+        section = self._get_mutable_section()
+        section.set(name, value)
+        for hook in ConfigHooks['set']:
+            hook(self, name, value)
+
+    def remove(self, name):
+        """Remove an existing option."""
+        section = self._get_mutable_section()
+        section.remove(name)
+        for hook in ConfigHooks['remove']:
+            hook(self, name)
+
+    def __repr__(self):
+        # Mostly for debugging use
+        return "<config.%s(%s)>" % (self.__class__.__name__, id(self))
+
+    def _get_overrides(self):
+        # Hack around library_state.initialize never called
+        if bzrlib.global_state is not None:
+            return [bzrlib.global_state.cmdline_overrides]
+        return []
+
+
+class _CompatibleStack(Stack):
+    """Place holder for compatibility with previous design.
+
+    This is intended to ease the transition from the Config-based design to the
+    Stack-based design and should not be used nor relied upon by plugins.
+
+    One assumption made here is that the daughter classes will all use Stores
+    derived from LockableIniFileStore).
+
+    It implements set() by re-loading the store before applying the
+    modification and saving it.
+
+    The long term plan being to implement a single write by store to save
+    all modifications, this class should not be used in the interim.
+    """
+
+    def set(self, name, value):
+        # Force a reload
+        self.store.unload()
+        super(_CompatibleStack, self).set(name, value)
+        # Force a write to persistent storage
+        self.store.save()
+
+
+class GlobalStack(_CompatibleStack):
+    """Global options only stack."""
+
+    def __init__(self):
+        # Get a GlobalStore
+        gstore = GlobalStore()
+        super(GlobalStack, self).__init__(
+            [self._get_overrides, gstore.get_sections],
+            gstore)
+
+
+class LocationStack(_CompatibleStack):
+    """Per-location options falling back to global options stack."""
+
+    def __init__(self, location):
+        """Make a new stack for a location and global configuration.
+        
+        :param location: A URL prefix to """
+        lstore = LocationStore()
+        matcher = LocationMatcher(lstore, location)
+        gstore = GlobalStore()
+        super(LocationStack, self).__init__(
+            [self._get_overrides,
+             matcher.get_sections, gstore.get_sections],
+            lstore)
+
+
+class BranchStack(_CompatibleStack):
+    """Per-location options falling back to branch then global options stack."""
+
+    def __init__(self, branch):
+        bstore = BranchStore(branch)
+        lstore = LocationStore()
+        matcher = LocationMatcher(lstore, branch.base)
+        gstore = GlobalStore()
+        super(BranchStack, self).__init__(
+            [self._get_overrides,
+             matcher.get_sections, bstore.get_sections, gstore.get_sections],
+            bstore)
+        self.branch = branch
+
+
+class RemoteControlStack(_CompatibleStack):
+    """Remote control-only options stack."""
+
+    def __init__(self, bzrdir):
+        cstore = ControlStore(bzrdir)
+        super(RemoteControlStack, self).__init__(
+            [cstore.get_sections],
+            cstore)
+        self.bzrdir = bzrdir
+
+
+class RemoteBranchStack(_CompatibleStack):
+    """Remote branch-only options stack."""
+
+    def __init__(self, branch):
+        bstore = BranchStore(branch)
+        super(RemoteBranchStack, self).__init__(
+            [bstore.get_sections],
+            bstore)
+        self.branch = branch
+
+
+class cmd_config(commands.Command):
+    __doc__ = """Display, set or remove a configuration option.
+
+    Display the active value for a given option.
+
+    If --all is specified, NAME is interpreted as a regular expression and all
+    matching options are displayed mentioning their scope. The active value
+    that bzr will take into account is the first one displayed for each option.
+
+    If no NAME is given, --all .* is implied.
+
+    Setting a value is achieved by using name=value without spaces. The value
+    is set in the most relevant scope and can be checked by displaying the
+    option again.
+    """
+
+    takes_args = ['name?']
+
+    takes_options = [
+        'directory',
+        # FIXME: This should be a registry option so that plugins can register
+        # their own config files (or not) -- vila 20101002
+        commands.Option('scope', help='Reduce the scope to the specified'
+                        ' configuration file',
+                        type=unicode),
+        commands.Option('all',
+            help='Display all the defined values for the matching options.',
+            ),
+        commands.Option('remove', help='Remove the option from'
+                        ' the configuration file'),
+        ]
+
+    _see_also = ['configuration']
+
+    @commands.display_command
+    def run(self, name=None, all=False, directory=None, scope=None,
+            remove=False):
+        if directory is None:
+            directory = '.'
+        directory = urlutils.normalize_url(directory)
+        if remove and all:
+            raise errors.BzrError(
+                '--all and --remove are mutually exclusive.')
+        elif remove:
+            # Delete the option in the given scope
+            self._remove_config_option(name, directory, scope)
+        elif name is None:
+            # Defaults to all options
+            self._show_matching_options('.*', directory, scope)
+        else:
+            try:
+                name, value = name.split('=', 1)
+            except ValueError:
+                # Display the option(s) value(s)
+                if all:
+                    self._show_matching_options(name, directory, scope)
+                else:
+                    self._show_value(name, directory, scope)
+            else:
+                if all:
+                    raise errors.BzrError(
+                        'Only one option can be set.')
+                # Set the option value
+                self._set_config_option(name, value, directory, scope)
+
+    def _get_configs(self, directory, scope=None):
+        """Iterate the configurations specified by ``directory`` and ``scope``.
+
+        :param directory: Where the configurations are derived from.
+
+        :param scope: A specific config to start from.
+        """
+        if scope is not None:
+            if scope == 'bazaar':
+                yield GlobalConfig()
+            elif scope == 'locations':
+                yield LocationConfig(directory)
+            elif scope == 'branch':
+                (_, br, _) = (
+                    controldir.ControlDir.open_containing_tree_or_branch(
+                        directory))
+                yield br.get_config()
+        else:
+            try:
+                (_, br, _) = (
+                    controldir.ControlDir.open_containing_tree_or_branch(
+                        directory))
+                yield br.get_config()
+            except errors.NotBranchError:
+                yield LocationConfig(directory)
+                yield GlobalConfig()
+
+    def _show_value(self, name, directory, scope):
+        displayed = False
+        for c in self._get_configs(directory, scope):
+            if displayed:
+                break
+            for (oname, value, section, conf_id, parser) in c._get_options():
+                if name == oname:
+                    # Display only the first value and exit
+
+                    # FIXME: We need to use get_user_option to take policies
+                    # into account and we need to make sure the option exists
+                    # too (hence the two for loops), this needs a better API
+                    # -- vila 20101117
+                    value = c.get_user_option(name)
+                    # Quote the value appropriately
+                    value = parser._quote(value)
+                    self.outf.write('%s\n' % (value,))
+                    displayed = True
+                    break
+        if not displayed:
+            raise errors.NoSuchConfigOption(name)
+
+    def _show_matching_options(self, name, directory, scope):
+        name = lazy_regex.lazy_compile(name)
+        # We want any error in the regexp to be raised *now* so we need to
+        # avoid the delay introduced by the lazy regexp.  But, we still do
+        # want the nicer errors raised by lazy_regex.
+        name._compile_and_collapse()
+        cur_conf_id = None
+        cur_section = None
+        for c in self._get_configs(directory, scope):
+            for (oname, value, section, conf_id, parser) in c._get_options():
+                if name.search(oname):
+                    if cur_conf_id != conf_id:
+                        # Explain where the options are defined
+                        self.outf.write('%s:\n' % (conf_id,))
+                        cur_conf_id = conf_id
+                        cur_section = None
+                    if (section not in (None, 'DEFAULT')
+                        and cur_section != section):
+                        # Display the section if it's not the default (or only)
+                        # one.
+                        self.outf.write('  [%s]\n' % (section,))
+                        cur_section = section
+                    self.outf.write('  %s = %s\n' % (oname, value))
+
+    def _set_config_option(self, name, value, directory, scope):
+        for conf in self._get_configs(directory, scope):
+            conf.set_user_option(name, value)
+            break
+        else:
+            raise errors.NoSuchConfig(scope)
+
+    def _remove_config_option(self, name, directory, scope):
+        if name is None:
+            raise errors.BzrCommandError(
+                '--remove expects an option to remove.')
+        removed = False
+        for conf in self._get_configs(directory, scope):
+            for (section_name, section, conf_id) in conf._get_sections():
+                if scope is not None and conf_id != scope:
+                    # Not the right configuration file
+                    continue
+                if name in section:
+                    if conf_id != conf.config_id():
+                        conf = self._get_configs(directory, conf_id).next()
+                    # We use the first section in the first config where the
+                    # option is defined to remove it
+                    conf.remove_user_option(name, section_name)
+                    removed = True
+                    break
+            break
+        else:
+            raise errors.NoSuchConfig(scope)
+        if not removed:
+            raise errors.NoSuchConfigOption(name)
+
+# Test registries
+#
+# We need adapters that can build a Store or a Stack in a test context. Test
+# classes, based on TestCaseWithTransport, can use the registry to parametrize
+# themselves. The builder will receive a test instance and should return a
+# ready-to-use store or stack.  Plugins that define new store/stacks can also
+# register themselves here to be tested against the tests defined in
+# bzrlib.tests.test_config. Note that the builder can be called multiple times
+# for the same tests.
+
+# The registered object should be a callable receiving a test instance
+# parameter (inheriting from tests.TestCaseWithTransport) and returning a Store
+# object.
+test_store_builder_registry = registry.Registry()
+
+# The registered object should be a callable receiving a test instance
+# parameter (inheriting from tests.TestCaseWithTransport) and returning a Stack
+# object.
+test_stack_builder_registry = registry.Registry()

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2010-07-16 15:35:43 +0000
+++ bzrlib/conflicts.py	2011-09-16 15:39:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 
 from bzrlib import (
     cleanup,
-    commands,
     errors,
     osutils,
     rio,
@@ -33,8 +32,10 @@
     transform,
     workingtree,
     )
+from bzrlib.i18n import gettext, ngettext
 """)
 from bzrlib import (
+    commands,
     option,
     registry,
     )
@@ -72,7 +73,7 @@
                     continue
                 self.outf.write(conflict.path + '\n')
             else:
-                self.outf.write(str(conflict) + '\n')
+                self.outf.write(unicode(conflict) + '\n')
 
 
 resolve_action_registry = registry.Registry()
@@ -117,17 +118,19 @@
             ResolveActionOption(),
             ]
     _see_also = ['conflicts']
-    def run(self, file_list=None, all=False, action=None, directory=u'.'):
+    def run(self, file_list=None, all=False, action=None, directory=None):
         if all:
             if file_list:
-                raise errors.BzrCommandError("If --all is specified,"
-                                             " no FILE may be provided")
+                raise errors.BzrCommandError(gettext("If --all is specified,"
+                                             " no FILE may be provided"))
+            if directory is None:
+                directory = u'.'
             tree = workingtree.WorkingTree.open_containing(directory)[0]
             if action is None:
                 action = 'done'
         else:
             tree, file_list = workingtree.WorkingTree.open_containing_paths(
-                file_list)
+                file_list, directory)
             if file_list is None:
                 if action is None:
                     # FIXME: There is a special case here related to the option
@@ -143,13 +146,15 @@
             if file_list is None:
                 un_resolved, resolved = tree.auto_resolve()
                 if len(un_resolved) > 0:
-                    trace.note('%d conflict(s) auto-resolved.', len(resolved))
-                    trace.note('Remaining conflicts:')
+                    trace.note(ngettext('%d conflict auto-resolved.',
+                        '%d conflicts auto-resolved.', len(resolved)),
+                        len(resolved))
+                    trace.note(gettext('Remaining conflicts:'))
                     for conflict in un_resolved:
-                        trace.note(conflict)
+                        trace.note(unicode(conflict))
                     return 1
                 else:
-                    trace.note('All conflicts resolved.')
+                    trace.note(gettext('All conflicts resolved.'))
                     return 0
             else:
                 # FIXME: This can never occur but the block above needs some
@@ -157,7 +162,10 @@
                 # conflict.auto(tree) --vila 091242
                 pass
         else:
-            resolve(tree, file_list, action=action)
+            before, after = resolve(tree, file_list, action=action)
+            trace.note(ngettext('{0} conflict resolved, {1} remaining',
+                                '{0} conflicts resolved, {1} remaining',
+                                before-after).format(before - after, after))
 
 
 def resolve(tree, paths=None, ignore_misses=False, recursive=False,
@@ -176,8 +184,10 @@
     :param action: How the conflict should be resolved,
     """
     tree.lock_tree_write()
+    nb_conflicts_after = None
     try:
         tree_conflicts = tree.conflicts()
+        nb_conflicts_before = len(tree_conflicts)
         if paths is None:
             new_conflicts = ConflictList()
             to_process = tree_conflicts
@@ -191,11 +201,15 @@
             except NotImplementedError:
                 new_conflicts.append(conflict)
         try:
+            nb_conflicts_after = len(new_conflicts)
             tree.set_conflicts(new_conflicts)
         except errors.UnsupportedOperation:
             pass
     finally:
         tree.unlock()
+    if nb_conflicts_after is None:
+        nb_conflicts_after = nb_conflicts_before
+    return nb_conflicts_before, nb_conflicts_after
 
 
 def restore(filename):
@@ -281,7 +295,7 @@
     def to_strings(self):
         """Generate strings for the provided conflicts"""
         for conflict in self:
-            yield str(conflict)
+            yield unicode(conflict)
 
     def remove_files(self, tree):
         """Remove the THIS, BASE and OTHER files for listed conflicts"""
@@ -380,7 +394,7 @@
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def __str__(self):
+    def __unicode__(self):
         return self.format % self.__dict__
 
     def __repr__(self):
@@ -497,14 +511,13 @@
         if path_to_create is not None:
             tid = tt.trans_id_tree_path(path_to_create)
             transform.create_from_tree(
-                tt, tt.trans_id_tree_path(path_to_create),
-                self._revision_tree(tt._tree, revid), file_id)
+                tt, tid, self._revision_tree(tt._tree, revid), file_id)
             tt.version_file(file_id, tid)
-
+        else:
+            tid = tt.trans_id_file_id(file_id)
         # Adjust the path for the retained file id
-        tid = tt.trans_id_file_id(file_id)
         parent_tid = tt.get_tree_parent(tid)
-        tt.adjust_path(path, parent_tid, tid)
+        tt.adjust_path(osutils.basename(path), parent_tid, tid)
         tt.apply()
 
     def _revision_tree(self, tree, revid):
@@ -573,7 +586,7 @@
         :param tt: The TreeTransform where the conflict is resolved.
         :param suffix_to_remove: Either 'THIS' or 'OTHER'
 
-        The resolution is symmetric, when taking THIS, OTHER is deleted and
+        The resolution is symmetric: when taking THIS, OTHER is deleted and
         item.THIS is renamed into item and vice-versa.
         """
         try:
@@ -586,12 +599,23 @@
             # never existed or was already deleted (including the case
             # where the user deleted it)
             pass
-        # Rename 'item.suffix_to_remove' (note that if
-        # 'item.suffix_to_remove' has been deleted, this is a no-op)
-        this_tid = tt.trans_id_file_id(self.file_id)
-        parent_tid = tt.get_tree_parent(this_tid)
-        tt.adjust_path(self.path, parent_tid, this_tid)
-        tt.apply()
+        try:
+            this_path = tt._tree.id2path(self.file_id)
+        except errors.NoSuchId:
+            # The file is not present anymore. This may happen if the user
+            # deleted the file either manually or when resolving a conflict on
+            # the parent.  We may raise some exception to indicate that the
+            # conflict doesn't exist anymore and as such doesn't need to be
+            # resolved ? -- vila 20110615 
+            this_tid = None
+        else:
+            this_tid = tt.trans_id_tree_path(this_path)
+        if this_tid is not None:
+            # Rename 'item.suffix_to_remove' (note that if
+            # 'item.suffix_to_remove' has been deleted, this is a no-op)
+            parent_tid = tt.get_tree_parent(this_tid)
+            tt.adjust_path(osutils.basename(self.path), parent_tid, this_tid)
+            tt.apply()
 
     def action_take_this(self, tree):
         self._resolve_with_cleanups(tree, 'OTHER')
@@ -600,12 +624,9 @@
         self._resolve_with_cleanups(tree, 'THIS')
 
 
-# FIXME: TextConflict is about a single file-id, there never is a conflict_path
-# attribute so we shouldn't inherit from PathConflict but simply from Conflict
-
 # TODO: There should be a base revid attribute to better inform the user about
 # how the conflicts were generated.
-class TextConflict(PathConflict):
+class TextConflict(Conflict):
     """The merge algorithm could not resolve all differences encountered."""
 
     has_files = True
@@ -614,9 +635,43 @@
 
     format = 'Text conflict in %(path)s'
 
+    rformat = '%(class)s(%(path)r, %(file_id)r)'
+
     def associated_filenames(self):
         return [self.path + suffix for suffix in CONFLICT_SUFFIXES]
 
+    def _resolve(self, tt, winner_suffix):
+        """Resolve the conflict by copying one of .THIS or .OTHER into file.
+
+        :param tt: The TreeTransform where the conflict is resolved.
+        :param winner_suffix: Either 'THIS' or 'OTHER'
+
+        The resolution is symmetric, when taking THIS, item.THIS is renamed
+        into item and vice-versa. This takes one of the files as a whole
+        ignoring every difference that could have been merged cleanly.
+        """
+        # To avoid useless copies, we switch item and item.winner_suffix, only
+        # item will exist after the conflict has been resolved anyway.
+        item_tid = tt.trans_id_file_id(self.file_id)
+        item_parent_tid = tt.get_tree_parent(item_tid)
+        winner_path = self.path + '.' + winner_suffix
+        winner_tid = tt.trans_id_tree_path(winner_path)
+        winner_parent_tid = tt.get_tree_parent(winner_tid)
+        # Switch the paths to preserve the content
+        tt.adjust_path(osutils.basename(self.path),
+                       winner_parent_tid, winner_tid)
+        tt.adjust_path(osutils.basename(winner_path), item_parent_tid, item_tid)
+        # Associate the file_id to the right content
+        tt.unversion_file(item_tid)
+        tt.version_file(self.file_id, winner_tid)
+        tt.apply()
+
+    def action_take_this(self, tree):
+        self._resolve_with_cleanups(tree, 'THIS')
+
+    def action_take_other(self, tree):
+        self._resolve_with_cleanups(tree, 'OTHER')
+
 
 class HandledConflict(Conflict):
     """A path problem that has been provisionally resolved.
@@ -715,18 +770,15 @@
         pass
 
     def action_take_other(self, tree):
-        # FIXME: We shouldn't have to manipulate so many paths here (and there
-        # is probably a bug or two...)
-        base_path = osutils.basename(self.path)
-        conflict_base_path = osutils.basename(self.conflict_path)
         tt = transform.TreeTransform(tree)
         try:
             p_tid = tt.trans_id_file_id(self.file_id)
             parent_tid = tt.get_tree_parent(p_tid)
             cp_tid = tt.trans_id_file_id(self.conflict_file_id)
             cparent_tid = tt.get_tree_parent(cp_tid)
-            tt.adjust_path(base_path, cparent_tid, cp_tid)
-            tt.adjust_path(conflict_base_path, parent_tid, p_tid)
+            tt.adjust_path(osutils.basename(self.path), cparent_tid, cp_tid)
+            tt.adjust_path(osutils.basename(self.conflict_path),
+                           parent_tid, p_tid)
             tt.apply()
         finally:
             tt.finalize()

=== added file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/controldir.py	2011-10-13 16:35:57 +0000
@@ -0,0 +1,1425 @@
+# Copyright (C) 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""ControlDir is the basic control directory class.
+
+The ControlDir class is the base for the control directory used
+by all bzr and foreign formats. For the ".bzr" implementation,
+see bzrlib.bzrdir.BzrDir.
+
+"""
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import textwrap
+
+from bzrlib import (
+    errors,
+    hooks,
+    revision as _mod_revision,
+    transport as _mod_transport,
+    trace,
+    ui,
+    urlutils,
+    )
+from bzrlib.transport import local
+from bzrlib.push import (
+    PushResult,
+    )
+
+from bzrlib.i18n import gettext
+""")
+
+from bzrlib import registry
+
+
+class ControlComponent(object):
+    """Abstract base class for control directory components.
+
+    This provides interfaces that are common across controldirs,
+    repositories, branches, and workingtree control directories.
+
+    They all expose two urls and transports: the *user* URL is the
+    one that stops above the control directory (eg .bzr) and that
+    should normally be used in messages, and the *control* URL is
+    under that in eg .bzr/checkout and is used to read the control
+    files.
+
+    This can be used as a mixin and is intended to fit with
+    foreign formats.
+    """
+
+    @property
+    def control_transport(self):
+        raise NotImplementedError
+
+    @property
+    def control_url(self):
+        return self.control_transport.base
+
+    @property
+    def user_transport(self):
+        raise NotImplementedError
+
+    @property
+    def user_url(self):
+        return self.user_transport.base
+
+
+class ControlDir(ControlComponent):
+    """A control directory.
+
+    While this represents a generic control directory, there are a few
+    features that are present in this interface that are currently only
+    supported by one of its implementations, BzrDir.
+
+    These features (bound branches, stacked branches) are currently only
+    supported by Bazaar, but could be supported by other version control
+    systems as well. Implementations are required to raise the appropriate
+    exceptions when an operation is requested that is not supported.
+
+    This also makes life easier for API users who can rely on the
+    implementation always allowing a particular feature to be requested but
+    raising an exception when it is not supported, rather than requiring the
+    API users to check for magic attributes to see what features are supported.
+    """
+
+    def can_convert_format(self):
+        """Return true if this controldir is one whose format we can convert
+        from."""
+        return True
+
+    def list_branches(self):
+        """Return a sequence of all branches local to this control directory.
+
+        """
+        try:
+            return [self.open_branch()]
+        except (errors.NotBranchError, errors.NoRepositoryPresent):
+            return []
+
+    def is_control_filename(self, filename):
+        """True if filename is the name of a path which is reserved for
+        controldirs.
+
+        :param filename: A filename within the root transport of this
+            controldir.
+
+        This is true IF and ONLY IF the filename is part of the namespace reserved
+        for bzr control dirs. Currently this is the '.bzr' directory in the root
+        of the root_transport. it is expected that plugins will need to extend
+        this in the future - for instance to make bzr talk with svn working
+        trees.
+        """
+        raise NotImplementedError(self.is_control_filename)
+
+    def needs_format_conversion(self, format=None):
+        """Return true if this controldir needs convert_format run on it.
+
+        For instance, if the repository format is out of date but the
+        branch and working tree are not, this should return True.
+
+        :param format: Optional parameter indicating a specific desired
+                       format we plan to arrive at.
+        """
+        raise NotImplementedError(self.needs_format_conversion)
+
+    def create_repository(self, shared=False):
+        """Create a new repository in this control directory.
+
+        :param shared: If a shared repository should be created
+        :return: The newly created repository
+        """
+        raise NotImplementedError(self.create_repository)
+
+    def destroy_repository(self):
+        """Destroy the repository in this ControlDir."""
+        raise NotImplementedError(self.destroy_repository)
+
+    def create_branch(self, name=None, repository=None,
+                      append_revisions_only=None):
+        """Create a branch in this ControlDir.
+
+        :param name: Name of the colocated branch to create, None for
+            the default branch.
+        :param append_revisions_only: Whether this branch should only allow
+            appending new revisions to its history.
+
+        The controldirs format will control what branch format is created.
+        For more control see BranchFormatXX.create(a_controldir).
+        """
+        raise NotImplementedError(self.create_branch)
+
+    def destroy_branch(self, name=None):
+        """Destroy a branch in this ControlDir.
+
+        :param name: Name of the branch to destroy, None for the default 
+            branch.
+        """
+        raise NotImplementedError(self.destroy_branch)
+
+    def create_workingtree(self, revision_id=None, from_branch=None,
+        accelerator_tree=None, hardlink=False):
+        """Create a working tree at this ControlDir.
+
+        :param revision_id: create it as of this revision id.
+        :param from_branch: override controldir branch 
+            (for lightweight checkouts)
+        :param accelerator_tree: A tree which can be used for retrieving file
+            contents more quickly than the revision tree, i.e. a workingtree.
+            The revision tree will be used for cases where accelerator_tree's
+            content is different.
+        """
+        raise NotImplementedError(self.create_workingtree)
+
+    def destroy_workingtree(self):
+        """Destroy the working tree at this ControlDir.
+
+        Formats that do not support this may raise UnsupportedOperation.
+        """
+        raise NotImplementedError(self.destroy_workingtree)
+
+    def destroy_workingtree_metadata(self):
+        """Destroy the control files for the working tree at this ControlDir.
+
+        The contents of working tree files are not affected.
+        Formats that do not support this may raise UnsupportedOperation.
+        """
+        raise NotImplementedError(self.destroy_workingtree_metadata)
+
+    def find_branch_format(self, name=None):
+        """Find the branch 'format' for this controldir.
+
+        This might be a synthetic object for e.g. RemoteBranch and SVN.
+        """
+        raise NotImplementedError(self.find_branch_format)
+
+    def get_branch_reference(self, name=None):
+        """Return the referenced URL for the branch in this controldir.
+
+        :param name: Optional colocated branch name
+        :raises NotBranchError: If there is no Branch.
+        :raises NoColocatedBranchSupport: If a branch name was specified
+            but colocated branches are not supported.
+        :return: The URL the branch in this controldir references if it is a
+            reference branch, or None for regular branches.
+        """
+        if name is not None:
+            raise errors.NoColocatedBranchSupport(self)
+        return None
+
+    def open_branch(self, name=None, unsupported=False,
+                    ignore_fallbacks=False):
+        """Open the branch object at this ControlDir if one is present.
+
+        If unsupported is True, then no longer supported branch formats can
+        still be opened.
+
+        TODO: static convenience version of this?
+        """
+        raise NotImplementedError(self.open_branch)
+
+    def open_repository(self, _unsupported=False):
+        """Open the repository object at this ControlDir if one is present.
+
+        This will not follow the Branch object pointer - it's strictly a direct
+        open facility. Most client code should use open_branch().repository to
+        get at a repository.
+
+        :param _unsupported: a private parameter, not part of the api.
+
+        TODO: static convenience version of this?
+        """
+        raise NotImplementedError(self.open_repository)
+
+    def find_repository(self):
+        """Find the repository that should be used.
+
+        This does not require a branch as we use it to find the repo for
+        new branches as well as to hook existing branches up to their
+        repository.
+        """
+        raise NotImplementedError(self.find_repository)
+
+    def open_workingtree(self, _unsupported=False,
+                         recommend_upgrade=True, from_branch=None):
+        """Open the workingtree object at this ControlDir if one is present.
+
+        :param recommend_upgrade: Optional keyword parameter, when True (the
+            default), emit through the ui module a recommendation that the user
+            upgrade the working tree when the workingtree being opened is old
+            (but still fully supported).
+        :param from_branch: override controldir branch (for lightweight
+            checkouts)
+        """
+        raise NotImplementedError(self.open_workingtree)
+
+    def has_branch(self, name=None):
+        """Tell if this controldir contains a branch.
+
+        Note: if you're going to open the branch, you should just go ahead
+        and try, and not ask permission first.  (This method just opens the
+        branch and discards it, and that's somewhat expensive.)
+        """
+        try:
+            self.open_branch(name)
+            return True
+        except errors.NotBranchError:
+            return False
+
+    def _get_selected_branch(self):
+        """Return the name of the branch selected by the user.
+
+        :return: Name of the branch selected by the user, or None.
+        """
+        branch = self.root_transport.get_segment_parameters().get("branch")
+        if branch is not None:
+            branch = urlutils.unescape(branch)
+        return branch
+
+    def has_workingtree(self):
+        """Tell if this controldir contains a working tree.
+
+        This will still raise an exception if the controldir has a workingtree
+        that is remote & inaccessible.
+
+        Note: if you're going to open the working tree, you should just go ahead
+        and try, and not ask permission first.  (This method just opens the
+        workingtree and discards it, and that's somewhat expensive.)
+        """
+        try:
+            self.open_workingtree(recommend_upgrade=False)
+            return True
+        except errors.NoWorkingTree:
+            return False
+
+    def cloning_metadir(self, require_stacking=False):
+        """Produce a metadir suitable for cloning or sprouting with.
+
+        These operations may produce workingtrees (yes, even though they're
+        "cloning" something that doesn't have a tree), so a viable workingtree
+        format must be selected.
+
+        :require_stacking: If True, non-stackable formats will be upgraded
+            to similar stackable formats.
+        :returns: a ControlDirFormat with all component formats either set
+            appropriately or set to None if that component should not be
+            created.
+        """
+        raise NotImplementedError(self.cloning_metadir)
+
+    def checkout_metadir(self):
+        """Produce a metadir suitable for checkouts of this controldir."""
+        return self.cloning_metadir()
+
+    def sprout(self, url, revision_id=None, force_new_repo=False,
+               recurse='down', possible_transports=None,
+               accelerator_tree=None, hardlink=False, stacked=False,
+               source_branch=None, create_tree_if_local=True):
+        """Create a copy of this controldir prepared for use as a new line of
+        development.
+
+        If url's last component does not exist, it will be created.
+
+        Attributes related to the identity of the source branch like
+        branch nickname will be cleaned, a working tree is created
+        whether one existed before or not; and a local branch is always
+        created.
+
+        :param revision_id: if revision_id is not None, then the clone
+            operation may tune itself to download less data.
+        :param accelerator_tree: A tree which can be used for retrieving file
+            contents more quickly than the revision tree, i.e. a workingtree.
+            The revision tree will be used for cases where accelerator_tree's
+            content is different.
+        :param hardlink: If true, hard-link files from accelerator_tree,
+            where possible.
+        :param stacked: If true, create a stacked branch referring to the
+            location of this control directory.
+        :param create_tree_if_local: If true, a working-tree will be created
+            when working locally.
+        """
+        raise NotImplementedError(self.sprout)
+
+    def push_branch(self, source, revision_id=None, overwrite=False, 
+        remember=False, create_prefix=False):
+        """Push the source branch into this ControlDir."""
+        br_to = None
+        # If we can open a branch, use its direct repository, otherwise see
+        # if there is a repository without a branch.
+        try:
+            br_to = self.open_branch()
+        except errors.NotBranchError:
+            # Didn't find a branch, can we find a repository?
+            repository_to = self.find_repository()
+        else:
+            # Found a branch, so we must have found a repository
+            repository_to = br_to.repository
+
+        push_result = PushResult()
+        push_result.source_branch = source
+        if br_to is None:
+            # We have a repository but no branch, copy the revisions, and then
+            # create a branch.
+            if revision_id is None:
+                # No revision supplied by the user, default to the branch
+                # revision
+                revision_id = source.last_revision()
+            repository_to.fetch(source.repository, revision_id=revision_id)
+            br_to = source.clone(self, revision_id=revision_id)
+            if source.get_push_location() is None or remember:
+                source.set_push_location(br_to.base)
+            push_result.stacked_on = None
+            push_result.branch_push_result = None
+            push_result.old_revno = None
+            push_result.old_revid = _mod_revision.NULL_REVISION
+            push_result.target_branch = br_to
+            push_result.master_branch = None
+            push_result.workingtree_updated = False
+        else:
+            # We have successfully opened the branch, remember if necessary:
+            if source.get_push_location() is None or remember:
+                source.set_push_location(br_to.base)
+            try:
+                tree_to = self.open_workingtree()
+            except errors.NotLocalUrl:
+                push_result.branch_push_result = source.push(br_to, 
+                    overwrite, stop_revision=revision_id)
+                push_result.workingtree_updated = False
+            except errors.NoWorkingTree:
+                push_result.branch_push_result = source.push(br_to,
+                    overwrite, stop_revision=revision_id)
+                push_result.workingtree_updated = None # Not applicable
+            else:
+                tree_to.lock_write()
+                try:
+                    push_result.branch_push_result = source.push(
+                        tree_to.branch, overwrite, stop_revision=revision_id)
+                    tree_to.update()
+                finally:
+                    tree_to.unlock()
+                push_result.workingtree_updated = True
+            push_result.old_revno = push_result.branch_push_result.old_revno
+            push_result.old_revid = push_result.branch_push_result.old_revid
+            push_result.target_branch = \
+                push_result.branch_push_result.target_branch
+        return push_result
+
+    def _get_tree_branch(self, name=None):
+        """Return the branch and tree, if any, for this controldir.
+
+        :param name: Name of colocated branch to open.
+
+        Return None for tree if not present or inaccessible.
+        Raise NotBranchError if no branch is present.
+        :return: (tree, branch)
+        """
+        try:
+            tree = self.open_workingtree()
+        except (errors.NoWorkingTree, errors.NotLocalUrl):
+            tree = None
+            branch = self.open_branch(name=name)
+        else:
+            if name is not None:
+                branch = self.open_branch(name=name)
+            else:
+                branch = tree.branch
+        return tree, branch
+
+    def get_config(self):
+        """Get configuration for this ControlDir."""
+        raise NotImplementedError(self.get_config)
+
+    def check_conversion_target(self, target_format):
+        """Check that a controldir as a whole can be converted to a new format."""
+        raise NotImplementedError(self.check_conversion_target)
+
+    def clone(self, url, revision_id=None, force_new_repo=False,
+              preserve_stacking=False):
+        """Clone this controldir and its contents to url verbatim.
+
+        :param url: The url create the clone at.  If url's last component does
+            not exist, it will be created.
+        :param revision_id: The tip revision-id to use for any branch or
+            working tree.  If not None, then the clone operation may tune
+            itself to download less data.
+        :param force_new_repo: Do not use a shared repository for the target
+                               even if one is available.
+        :param preserve_stacking: When cloning a stacked branch, stack the
+            new branch on top of the other branch's stacked-on branch.
+        """
+        return self.clone_on_transport(_mod_transport.get_transport(url),
+                                       revision_id=revision_id,
+                                       force_new_repo=force_new_repo,
+                                       preserve_stacking=preserve_stacking)
+
+    def clone_on_transport(self, transport, revision_id=None,
+        force_new_repo=False, preserve_stacking=False, stacked_on=None,
+        create_prefix=False, use_existing_dir=True, no_tree=False):
+        """Clone this controldir and its contents to transport verbatim.
+
+        :param transport: The transport for the location to produce the clone
+            at.  If the target directory does not exist, it will be created.
+        :param revision_id: The tip revision-id to use for any branch or
+            working tree.  If not None, then the clone operation may tune
+            itself to download less data.
+        :param force_new_repo: Do not use a shared repository for the target,
+                               even if one is available.
+        :param preserve_stacking: When cloning a stacked branch, stack the
+            new branch on top of the other branch's stacked-on branch.
+        :param create_prefix: Create any missing directories leading up to
+            to_transport.
+        :param use_existing_dir: Use an existing directory if one exists.
+        :param no_tree: If set to true prevents creation of a working tree.
+        """
+        raise NotImplementedError(self.clone_on_transport)
+
+    @classmethod
+    def find_bzrdirs(klass, transport, evaluate=None, list_current=None):
+        """Find control dirs recursively from current location.
+
+        This is intended primarily as a building block for more sophisticated
+        functionality, like finding trees under a directory, or finding
+        branches that use a given repository.
+
+        :param evaluate: An optional callable that yields recurse, value,
+            where recurse controls whether this controldir is recursed into
+            and value is the value to yield.  By default, all bzrdirs
+            are recursed into, and the return value is the controldir.
+        :param list_current: if supplied, use this function to list the current
+            directory, instead of Transport.list_dir
+        :return: a generator of found bzrdirs, or whatever evaluate returns.
+        """
+        if list_current is None:
+            def list_current(transport):
+                return transport.list_dir('')
+        if evaluate is None:
+            def evaluate(controldir):
+                return True, controldir
+
+        pending = [transport]
+        while len(pending) > 0:
+            current_transport = pending.pop()
+            recurse = True
+            try:
+                controldir = klass.open_from_transport(current_transport)
+            except (errors.NotBranchError, errors.PermissionDenied):
+                pass
+            else:
+                recurse, value = evaluate(controldir)
+                yield value
+            try:
+                subdirs = list_current(current_transport)
+            except (errors.NoSuchFile, errors.PermissionDenied):
+                continue
+            if recurse:
+                for subdir in sorted(subdirs, reverse=True):
+                    pending.append(current_transport.clone(subdir))
+
+    @classmethod
+    def find_branches(klass, transport):
+        """Find all branches under a transport.
+
+        This will find all branches below the transport, including branches
+        inside other branches.  Where possible, it will use
+        Repository.find_branches.
+
+        To list all the branches that use a particular Repository, see
+        Repository.find_branches
+        """
+        def evaluate(controldir):
+            try:
+                repository = controldir.open_repository()
+            except errors.NoRepositoryPresent:
+                pass
+            else:
+                return False, ([], repository)
+            return True, (controldir.list_branches(), None)
+        ret = []
+        for branches, repo in klass.find_bzrdirs(
+                transport, evaluate=evaluate):
+            if repo is not None:
+                ret.extend(repo.find_branches())
+            if branches is not None:
+                ret.extend(branches)
+        return ret
+
+    @classmethod
+    def create_branch_and_repo(klass, base, force_new_repo=False, format=None):
+        """Create a new ControlDir, Branch and Repository at the url 'base'.
+
+        This will use the current default ControlDirFormat unless one is
+        specified, and use whatever
+        repository format that that uses via controldir.create_branch and
+        create_repository. If a shared repository is available that is used
+        preferentially.
+
+        The created Branch object is returned.
+
+        :param base: The URL to create the branch at.
+        :param force_new_repo: If True a new repository is always created.
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
+        """
+        controldir = klass.create(base, format)
+        controldir._find_or_create_repository(force_new_repo)
+        return controldir.create_branch()
+
+    @classmethod
+    def create_branch_convenience(klass, base, force_new_repo=False,
+                                  force_new_tree=None, format=None,
+                                  possible_transports=None):
+        """Create a new ControlDir, Branch and Repository at the url 'base'.
+
+        This is a convenience function - it will use an existing repository
+        if possible, can be told explicitly whether to create a working tree or
+        not.
+
+        This will use the current default ControlDirFormat unless one is
+        specified, and use whatever
+        repository format that that uses via ControlDir.create_branch and
+        create_repository. If a shared repository is available that is used
+        preferentially. Whatever repository is used, its tree creation policy
+        is followed.
+
+        The created Branch object is returned.
+        If a working tree cannot be made due to base not being a file:// url,
+        no error is raised unless force_new_tree is True, in which case no
+        data is created on disk and NotLocalUrl is raised.
+
+        :param base: The URL to create the branch at.
+        :param force_new_repo: If True a new repository is always created.
+        :param force_new_tree: If True or False force creation of a tree or
+                               prevent such creation respectively.
+        :param format: Override for the controldir format to create.
+        :param possible_transports: An optional reusable transports list.
+        """
+        if force_new_tree:
+            # check for non local urls
+            t = _mod_transport.get_transport(base, possible_transports)
+            if not isinstance(t, local.LocalTransport):
+                raise errors.NotLocalUrl(base)
+        controldir = klass.create(base, format, possible_transports)
+        repo = controldir._find_or_create_repository(force_new_repo)
+        result = controldir.create_branch()
+        if force_new_tree or (repo.make_working_trees() and
+                              force_new_tree is None):
+            try:
+                controldir.create_workingtree()
+            except errors.NotLocalUrl:
+                pass
+        return result
+
+    @classmethod
+    def create_standalone_workingtree(klass, base, format=None):
+        """Create a new ControlDir, WorkingTree, Branch and Repository at 'base'.
+
+        'base' must be a local path or a file:// url.
+
+        This will use the current default ControlDirFormat unless one is
+        specified, and use whatever
+        repository format that that uses for bzrdirformat.create_workingtree,
+        create_branch and create_repository.
+
+        :param format: Override for the controldir format to create.
+        :return: The WorkingTree object.
+        """
+        t = _mod_transport.get_transport(base)
+        if not isinstance(t, local.LocalTransport):
+            raise errors.NotLocalUrl(base)
+        controldir = klass.create_branch_and_repo(base,
+                                               force_new_repo=True,
+                                               format=format).bzrdir
+        return controldir.create_workingtree()
+
+    @classmethod
+    def open_unsupported(klass, base):
+        """Open a branch which is not supported."""
+        return klass.open(base, _unsupported=True)
+
+    @classmethod
+    def open(klass, base, _unsupported=False, possible_transports=None):
+        """Open an existing controldir, rooted at 'base' (url).
+
+        :param _unsupported: a private parameter to the ControlDir class.
+        """
+        t = _mod_transport.get_transport(base, possible_transports)
+        return klass.open_from_transport(t, _unsupported=_unsupported)
+
+    @classmethod
+    def open_from_transport(klass, transport, _unsupported=False,
+                            _server_formats=True):
+        """Open a controldir within a particular directory.
+
+        :param transport: Transport containing the controldir.
+        :param _unsupported: private.
+        """
+        for hook in klass.hooks['pre_open']:
+            hook(transport)
+        # Keep initial base since 'transport' may be modified while following
+        # the redirections.
+        base = transport.base
+        def find_format(transport):
+            return transport, ControlDirFormat.find_format(
+                transport, _server_formats=_server_formats)
+
+        def redirected(transport, e, redirection_notice):
+            redirected_transport = transport._redirected_to(e.source, e.target)
+            if redirected_transport is None:
+                raise errors.NotBranchError(base)
+            trace.note(gettext('{0} is{1} redirected to {2}').format(
+                 transport.base, e.permanently, redirected_transport.base))
+            return redirected_transport
+
+        try:
+            transport, format = _mod_transport.do_catching_redirections(
+                find_format, transport, redirected)
+        except errors.TooManyRedirections:
+            raise errors.NotBranchError(base)
+
+        format.check_support_status(_unsupported)
+        return format.open(transport, _found=True)
+
+    @classmethod
+    def open_containing(klass, url, possible_transports=None):
+        """Open an existing branch which contains url.
+
+        :param url: url to search from.
+
+        See open_containing_from_transport for more detail.
+        """
+        transport = _mod_transport.get_transport(url, possible_transports)
+        return klass.open_containing_from_transport(transport)
+
+    @classmethod
+    def open_containing_from_transport(klass, a_transport):
+        """Open an existing branch which contains a_transport.base.
+
+        This probes for a branch at a_transport, and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into the root.  If there isn't one, raises NotBranchError.
+        If there is one and it is either an unrecognised format or an unsupported
+        format, UnknownFormatError or UnsupportedFormatError are raised.
+        If there is one, it is returned, along with the unused portion of url.
+
+        :return: The ControlDir that contains the path, and a Unicode path
+                for the rest of the URL.
+        """
+        # this gets the normalised url back. I.e. '.' -> the full path.
+        url = a_transport.base
+        while True:
+            try:
+                result = klass.open_from_transport(a_transport)
+                return result, urlutils.unescape(a_transport.relpath(url))
+            except errors.NotBranchError, e:
+                pass
+            try:
+                new_t = a_transport.clone('..')
+            except errors.InvalidURLJoin:
+                # reached the root, whatever that may be
+                raise errors.NotBranchError(path=url)
+            if new_t.base == a_transport.base:
+                # reached the root, whatever that may be
+                raise errors.NotBranchError(path=url)
+            a_transport = new_t
+
+    @classmethod
+    def open_tree_or_branch(klass, location):
+        """Return the branch and working tree at a location.
+
+        If there is no tree at the location, tree will be None.
+        If there is no branch at the location, an exception will be
+        raised
+        :return: (tree, branch)
+        """
+        controldir = klass.open(location)
+        return controldir._get_tree_branch()
+
+    @classmethod
+    def open_containing_tree_or_branch(klass, location):
+        """Return the branch and working tree contained by a location.
+
+        Returns (tree, branch, relpath).
+        If there is no tree at containing the location, tree will be None.
+        If there is no branch containing the location, an exception will be
+        raised
+        relpath is the portion of the path that is contained by the branch.
+        """
+        controldir, relpath = klass.open_containing(location)
+        tree, branch = controldir._get_tree_branch()
+        return tree, branch, relpath
+
+    @classmethod
+    def open_containing_tree_branch_or_repository(klass, location):
+        """Return the working tree, branch and repo contained by a location.
+
+        Returns (tree, branch, repository, relpath).
+        If there is no tree containing the location, tree will be None.
+        If there is no branch containing the location, branch will be None.
+        If there is no repository containing the location, repository will be
+        None.
+        relpath is the portion of the path that is contained by the innermost
+        ControlDir.
+
+        If no tree, branch or repository is found, a NotBranchError is raised.
+        """
+        controldir, relpath = klass.open_containing(location)
+        try:
+            tree, branch = controldir._get_tree_branch()
+        except errors.NotBranchError:
+            try:
+                repo = controldir.find_repository()
+                return None, None, repo, relpath
+            except (errors.NoRepositoryPresent):
+                raise errors.NotBranchError(location)
+        return tree, branch, branch.repository, relpath
+
+    @classmethod
+    def create(klass, base, format=None, possible_transports=None):
+        """Create a new ControlDir at the url 'base'.
+
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
+        :param possible_transports: If supplied, a list of transports that
+            can be reused to share a remote connection.
+        """
+        if klass is not ControlDir:
+            raise AssertionError("ControlDir.create always creates the"
+                "default format, not one of %r" % klass)
+        t = _mod_transport.get_transport(base, possible_transports)
+        t.ensure_base()
+        if format is None:
+            format = ControlDirFormat.get_default_format()
+        return format.initialize_on_transport(t)
+
+
+class ControlDirHooks(hooks.Hooks):
+    """Hooks for ControlDir operations."""
+
+    def __init__(self):
+        """Create the default hooks."""
+        hooks.Hooks.__init__(self, "bzrlib.controldir", "ControlDir.hooks")
+        self.add_hook('pre_open',
+            "Invoked before attempting to open a ControlDir with the transport "
+            "that the open will use.", (1, 14))
+        self.add_hook('post_repo_init',
+            "Invoked after a repository has been initialized. "
+            "post_repo_init is called with a "
+            "bzrlib.controldir.RepoInitHookParams.",
+            (2, 2))
+
+# install the default hooks
+ControlDir.hooks = ControlDirHooks()
+
+
+class ControlComponentFormat(object):
+    """A component that can live inside of a .bzr meta directory."""
+
+    upgrade_recommended = False
+
+    def get_format_string(self):
+        """Return the format of this format, if usable in meta directories."""
+        raise NotImplementedError(self.get_format_string)
+
+    def get_format_description(self):
+        """Return the short description for this format."""
+        raise NotImplementedError(self.get_format_description)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats must be initializable and openable.
+        Unsupported formats may not support initialization or committing or
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
+        basedir=None):
+        """Give an error or warning on old formats.
+
+        :param allow_unsupported: If true, allow opening
+            formats that are strongly deprecated, and which may
+            have limited functionality.
+
+        :param recommend_upgrade: If true (default), warn
+            the user through the ui object that they may wish
+            to upgrade the object.
+        """
+        if not allow_unsupported and not self.is_supported():
+            # see open_downlevel to open legacy branches.
+            raise errors.UnsupportedFormatError(format=self)
+        if recommend_upgrade and self.upgrade_recommended:
+            ui.ui_factory.recommend_upgrade(
+                self.get_format_description(), basedir)
+
+
+class ControlComponentFormatRegistry(registry.FormatRegistry):
+    """A registry for control components (branch, workingtree, repository)."""
+
+    def __init__(self, other_registry=None):
+        super(ControlComponentFormatRegistry, self).__init__(other_registry)
+        self._extra_formats = []
+
+    def register(self, format):
+        """Register a new format."""
+        super(ControlComponentFormatRegistry, self).register(
+            format.get_format_string(), format)
+
+    def remove(self, format):
+        """Remove a registered format."""
+        super(ControlComponentFormatRegistry, self).remove(
+            format.get_format_string())
+
+    def register_extra(self, format):
+        """Register a format that can not be used in a metadir.
+
+        This is mainly useful to allow custom repository formats, such as older
+        Bazaar formats and foreign formats, to be tested.
+        """
+        self._extra_formats.append(registry._ObjectGetter(format))
+
+    def remove_extra(self, format):
+        """Remove an extra format.
+        """
+        self._extra_formats.remove(registry._ObjectGetter(format))
+
+    def register_extra_lazy(self, module_name, member_name):
+        """Register a format lazily.
+        """
+        self._extra_formats.append(
+            registry._LazyObjectGetter(module_name, member_name))
+
+    def _get_extra(self):
+        """Return all "extra" formats, not usable in meta directories."""
+        result = []
+        for getter in self._extra_formats:
+            f = getter.get_obj()
+            if callable(f):
+                f = f()
+            result.append(f)
+        return result
+
+    def _get_all(self):
+        """Return all formats, even those not usable in metadirs.
+        """
+        result = []
+        for name in self.keys():
+            fmt = self.get(name)
+            if callable(fmt):
+                fmt = fmt()
+            result.append(fmt)
+        return result + self._get_extra()
+
+    def _get_all_modules(self):
+        """Return a set of the modules providing objects."""
+        modules = set()
+        for name in self.keys():
+            modules.add(self._get_module(name))
+        for getter in self._extra_formats:
+            modules.add(getter.get_module())
+        return modules
+
+
+class Converter(object):
+    """Converts a disk format object from one format to another."""
+
+    def convert(self, to_convert, pb):
+        """Perform the conversion of to_convert, giving feedback via pb.
+
+        :param to_convert: The disk object to convert.
+        :param pb: a progress bar to use for progress information.
+        """
+
+    def step(self, message):
+        """Update the pb by a step."""
+        self.count +=1
+        self.pb.update(message, self.count, self.total)
+
+
+class ControlDirFormat(object):
+    """An encapsulation of the initialization and open routines for a format.
+
+    Formats provide three things:
+     * An initialization routine,
+     * a format string,
+     * an open routine.
+
+    Formats are placed in a dict by their format string for reference
+    during controldir opening. These should be subclasses of ControlDirFormat
+    for consistency.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the
+    object will be created every system load.
+
+    :cvar colocated_branches: Whether this formats supports colocated branches.
+    :cvar supports_workingtrees: This control directory can co-exist with a
+        working tree.
+    """
+
+    _default_format = None
+    """The default format used for new control directories."""
+
+    _server_probers = []
+    """The registered server format probers, e.g. RemoteBzrProber.
+
+    This is a list of Prober-derived classes.
+    """
+
+    _probers = []
+    """The registered format probers, e.g. BzrProber.
+
+    This is a list of Prober-derived classes.
+    """
+
+    colocated_branches = False
+    """Whether co-located branches are supported for this control dir format.
+    """
+
+    supports_workingtrees = True
+    """Whether working trees can exist in control directories of this format.
+    """
+
+    fixed_components = False
+    """Whether components can not change format independent of the control dir.
+    """
+
+    upgrade_recommended = False
+    """Whether an upgrade from this format is recommended."""
+
+    def get_format_description(self):
+        """Return the short description for this format."""
+        raise NotImplementedError(self.get_format_description)
+
+    def get_converter(self, format=None):
+        """Return the converter to use to convert controldirs needing converts.
+
+        This returns a bzrlib.controldir.Converter object.
+
+        This should return the best upgrader to step this format towards the
+        current default format. In the case of plugins we can/should provide
+        some means for them to extend the range of returnable converters.
+
+        :param format: Optional format to override the default format of the
+                       library.
+        """
+        raise NotImplementedError(self.get_converter)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats must be openable.
+        Unsupported formats may not support initialization or committing or
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def is_initializable(self):
+        """Whether new control directories of this format can be initialized.
+        """
+        return self.is_supported()
+
+    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
+        basedir=None):
+        """Give an error or warning on old formats.
+
+        :param allow_unsupported: If true, allow opening
+            formats that are strongly deprecated, and which may
+            have limited functionality.
+
+        :param recommend_upgrade: If true (default), warn
+            the user through the ui object that they may wish
+            to upgrade the object.
+        """
+        if not allow_unsupported and not self.is_supported():
+            # see open_downlevel to open legacy branches.
+            raise errors.UnsupportedFormatError(format=self)
+        if recommend_upgrade and self.upgrade_recommended:
+            ui.ui_factory.recommend_upgrade(
+                self.get_format_description(), basedir)
+
+    def same_model(self, target_format):
+        return (self.repository_format.rich_root_data ==
+            target_format.rich_root_data)
+
+    @classmethod
+    def register_format(klass, format):
+        """Register a format that does not use '.bzr' for its control dir.
+
+        """
+        raise errors.BzrError("ControlDirFormat.register_format() has been "
+            "removed in Bazaar 2.4. Please upgrade your plugins.")
+
+    @classmethod
+    def register_prober(klass, prober):
+        """Register a prober that can look for a control dir.
+
+        """
+        klass._probers.append(prober)
+
+    @classmethod
+    def unregister_prober(klass, prober):
+        """Unregister a prober.
+
+        """
+        klass._probers.remove(prober)
+
+    @classmethod
+    def register_server_prober(klass, prober):
+        """Register a control format prober for client-server environments.
+
+        These probers will be used before ones registered with
+        register_prober.  This gives implementations that decide to the
+        chance to grab it before anything looks at the contents of the format
+        file.
+        """
+        klass._server_probers.append(prober)
+
+    def __str__(self):
+        # Trim the newline
+        return self.get_format_description().rstrip()
+
+    @classmethod
+    def known_formats(klass):
+        """Return all the known formats.
+        """
+        result = set()
+        for prober_kls in klass._probers + klass._server_probers:
+            result.update(prober_kls.known_formats())
+        return result
+
+    @classmethod
+    def find_format(klass, transport, _server_formats=True):
+        """Return the format present at transport."""
+        if _server_formats:
+            _probers = klass._server_probers + klass._probers
+        else:
+            _probers = klass._probers
+        for prober_kls in _probers:
+            prober = prober_kls()
+            try:
+                return prober.probe_transport(transport)
+            except errors.NotBranchError:
+                # this format does not find a control dir here.
+                pass
+        raise errors.NotBranchError(path=transport.base)
+
+    def initialize(self, url, possible_transports=None):
+        """Create a control dir at this url and return an opened copy.
+
+        While not deprecated, this method is very specific and its use will
+        lead to many round trips to setup a working environment. See
+        initialize_on_transport_ex for a [nearly] all-in-one method.
+
+        Subclasses should typically override initialize_on_transport
+        instead of this method.
+        """
+        return self.initialize_on_transport(
+            _mod_transport.get_transport(url, possible_transports))
+
+    def initialize_on_transport(self, transport):
+        """Initialize a new controldir in the base directory of a Transport."""
+        raise NotImplementedError(self.initialize_on_transport)
+
+    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
+        create_prefix=False, force_new_repo=False, stacked_on=None,
+        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
+        shared_repo=False, vfs_only=False):
+        """Create this format on transport.
+
+        The directory to initialize will be created.
+
+        :param force_new_repo: Do not use a shared repository for the target,
+                               even if one is available.
+        :param create_prefix: Create any missing directories leading up to
+            to_transport.
+        :param use_existing_dir: Use an existing directory if one exists.
+        :param stacked_on: A url to stack any created branch on, None to follow
+            any target stacking policy.
+        :param stack_on_pwd: If stack_on is relative, the location it is
+            relative to.
+        :param repo_format_name: If non-None, a repository will be
+            made-or-found. Should none be found, or if force_new_repo is True
+            the repo_format_name is used to select the format of repository to
+            create.
+        :param make_working_trees: Control the setting of make_working_trees
+            for a new shared repository when one is made. None to use whatever
+            default the format has.
+        :param shared_repo: Control whether made repositories are shared or
+            not.
+        :param vfs_only: If True do not attempt to use a smart server
+        :return: repo, controldir, require_stacking, repository_policy. repo is
+            None if none was created or found, controldir is always valid.
+            require_stacking is the result of examining the stacked_on
+            parameter and any stacking policy found for the target.
+        """
+        raise NotImplementedError(self.initialize_on_transport_ex)
+
+    def network_name(self):
+        """A simple byte string uniquely identifying this format for RPC calls.
+
+        Bzr control formats use this disk format string to identify the format
+        over the wire. Its possible that other control formats have more
+        complex detection requirements, so we permit them to use any unique and
+        immutable string they desire.
+        """
+        raise NotImplementedError(self.network_name)
+
+    def open(self, transport, _found=False):
+        """Return an instance of this format for the dir transport points at.
+        """
+        raise NotImplementedError(self.open)
+
+    @classmethod
+    def _set_default_format(klass, format):
+        """Set default format (for testing behavior of defaults only)"""
+        klass._default_format = format
+
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+    def supports_transport(self, transport):
+        """Check if this format can be opened over a particular transport.
+        """
+        raise NotImplementedError(self.supports_transport)
+
+
+class Prober(object):
+    """Abstract class that can be used to detect a particular kind of
+    control directory.
+
+    At the moment this just contains a single method to probe a particular
+    transport, but it may be extended in the future to e.g. avoid
+    multiple levels of probing for Subversion repositories.
+
+    See BzrProber and RemoteBzrProber in bzrlib.bzrdir for the
+    probers that detect .bzr/ directories and Bazaar smart servers,
+    respectively.
+
+    Probers should be registered using the register_server_prober or
+    register_prober methods on ControlDirFormat.
+    """
+
+    def probe_transport(self, transport):
+        """Return the controldir style format present in a directory.
+
+        :raise UnknownFormatError: If a control dir was found but is
+            in an unknown format.
+        :raise NotBranchError: If no control directory was found.
+        :return: A ControlDirFormat instance.
+        """
+        raise NotImplementedError(self.probe_transport)
+
+    @classmethod
+    def known_formats(klass):
+        """Return the control dir formats known by this prober.
+
+        Multiple probers can return the same formats, so this should
+        return a set.
+
+        :return: A set of known formats.
+        """
+        raise NotImplementedError(klass.known_formats)
+
+
+class ControlDirFormatInfo(object):
+
+    def __init__(self, native, deprecated, hidden, experimental):
+        self.deprecated = deprecated
+        self.native = native
+        self.hidden = hidden
+        self.experimental = experimental
+
+
+class ControlDirFormatRegistry(registry.Registry):
+    """Registry of user-selectable ControlDir subformats.
+
+    Differs from ControlDirFormat._formats in that it provides sub-formats,
+    e.g. BzrDirMeta1 with weave repository.  Also, it's more user-oriented.
+    """
+
+    def __init__(self):
+        """Create a ControlDirFormatRegistry."""
+        self._aliases = set()
+        self._registration_order = list()
+        super(ControlDirFormatRegistry, self).__init__()
+
+    def aliases(self):
+        """Return a set of the format names which are aliases."""
+        return frozenset(self._aliases)
+
+    def register(self, key, factory, help, native=True, deprecated=False,
+                 hidden=False, experimental=False, alias=False):
+        """Register a ControlDirFormat factory.
+
+        The factory must be a callable that takes one parameter: the key.
+        It must produce an instance of the ControlDirFormat when called.
+
+        This function mainly exists to prevent the info object from being
+        supplied directly.
+        """
+        registry.Registry.register(self, key, factory, help,
+            ControlDirFormatInfo(native, deprecated, hidden, experimental))
+        if alias:
+            self._aliases.add(key)
+        self._registration_order.append(key)
+
+    def register_lazy(self, key, module_name, member_name, help, native=True,
+        deprecated=False, hidden=False, experimental=False, alias=False):
+        registry.Registry.register_lazy(self, key, module_name, member_name,
+            help, ControlDirFormatInfo(native, deprecated, hidden, experimental))
+        if alias:
+            self._aliases.add(key)
+        self._registration_order.append(key)
+
+    def set_default(self, key):
+        """Set the 'default' key to be a clone of the supplied key.
+
+        This method must be called once and only once.
+        """
+        registry.Registry.register(self, 'default', self.get(key),
+            self.get_help(key), info=self.get_info(key))
+        self._aliases.add('default')
+
+    def set_default_repository(self, key):
+        """Set the FormatRegistry default and Repository default.
+
+        This is a transitional method while Repository.set_default_format
+        is deprecated.
+        """
+        if 'default' in self:
+            self.remove('default')
+        self.set_default(key)
+        format = self.get('default')()
+
+    def make_bzrdir(self, key):
+        return self.get(key)()
+
+    def help_topic(self, topic):
+        output = ""
+        default_realkey = None
+        default_help = self.get_help('default')
+        help_pairs = []
+        for key in self._registration_order:
+            if key == 'default':
+                continue
+            help = self.get_help(key)
+            if help == default_help:
+                default_realkey = key
+            else:
+                help_pairs.append((key, help))
+
+        def wrapped(key, help, info):
+            if info.native:
+                help = '(native) ' + help
+            return ':%s:\n%s\n\n' % (key,
+                textwrap.fill(help, initial_indent='    ',
+                    subsequent_indent='    ',
+                    break_long_words=False))
+        if default_realkey is not None:
+            output += wrapped(default_realkey, '(default) %s' % default_help,
+                              self.get_info('default'))
+        deprecated_pairs = []
+        experimental_pairs = []
+        for key, help in help_pairs:
+            info = self.get_info(key)
+            if info.hidden:
+                continue
+            elif info.deprecated:
+                deprecated_pairs.append((key, help))
+            elif info.experimental:
+                experimental_pairs.append((key, help))
+            else:
+                output += wrapped(key, help, info)
+        output += "\nSee :doc:`formats-help` for more about storage formats."
+        other_output = ""
+        if len(experimental_pairs) > 0:
+            other_output += "Experimental formats are shown below.\n\n"
+            for key, help in experimental_pairs:
+                info = self.get_info(key)
+                other_output += wrapped(key, help, info)
+        else:
+            other_output += \
+                "No experimental formats are available.\n\n"
+        if len(deprecated_pairs) > 0:
+            other_output += "\nDeprecated formats are shown below.\n\n"
+            for key, help in deprecated_pairs:
+                info = self.get_info(key)
+                other_output += wrapped(key, help, info)
+        else:
+            other_output += \
+                "\nNo deprecated formats are available.\n\n"
+        other_output += \
+                "\nSee :doc:`formats-help` for more about storage formats."
+
+        if topic == 'other-formats':
+            return other_output
+        else:
+            return output
+
+
+class RepoInitHookParams(object):
+    """Object holding parameters passed to `*_repo_init` hooks.
+
+    There are 4 fields that hooks may wish to access:
+
+    :ivar repository: Repository created
+    :ivar format: Repository format
+    :ivar bzrdir: The controldir for the repository
+    :ivar shared: The repository is shared
+    """
+
+    def __init__(self, repository, format, controldir, shared):
+        """Create a group of RepoInitHook parameters.
+
+        :param repository: Repository created
+        :param format: Repository format
+        :param controldir: The controldir for the repository
+        :param shared: The repository is shared
+        """
+        self.repository = repository
+        self.format = format
+        self.bzrdir = controldir
+        self.shared = shared
+
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
+
+    def __repr__(self):
+        if self.repository:
+            return "<%s for %s>" % (self.__class__.__name__,
+                self.repository)
+        else:
+            return "<%s for %s>" % (self.__class__.__name__,
+                self.bzrdir)
+
+
+# Please register new formats after old formats so that formats
+# appear in chronological order and format descriptions can build
+# on previous ones.
+format_registry = ControlDirFormatRegistry()
+
+network_format_registry = registry.FormatRegistry()
+"""Registry of formats indexed by their network name.
+
+The network name for a ControlDirFormat is an identifier that can be used when
+referring to formats with smart server operations. See
+ControlDirFormat.network_name() for more detail.
+"""

=== modified file 'bzrlib/crash.py'
--- bzrlib/crash.py	2010-05-20 18:23:17 +0000
+++ bzrlib/crash.py	2011-05-20 14:46:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 A crash is an exception propagated up almost to the top level of Bazaar.
 
 If we have apport <https://launchpad.net/apport/>, we store a report of the
-crash using apport into it's /var/crash spool directory, from where the user
+crash using apport into its /var/crash spool directory, from where the user
 can either manually send it to Launchpad.  In some cases (at least Ubuntu
 development releases), Apport may pop up a window asking if they want
 to send it.
@@ -84,19 +84,27 @@
     """Report a bug by just printing a message to the user."""
     trace.print_exception(exc_info, err_file)
     err_file.write('\n')
-    err_file.write('bzr %s on python %s (%s)\n' % \
-                       (bzrlib.__version__,
-                        bzrlib._format_version_tuple(sys.version_info),
-                        platform.platform(aliased=1)))
-    err_file.write('arguments: %r\n' % sys.argv)
-    err_file.write(
+    import textwrap
+    def print_wrapped(l):
+        err_file.write(textwrap.fill(l,
+            width=78, subsequent_indent='    ') + '\n')
+    print_wrapped('bzr %s on python %s (%s)\n' % \
+        (bzrlib.__version__,
+        bzrlib._format_version_tuple(sys.version_info),
+        platform.platform(aliased=1)))
+    print_wrapped('arguments: %r\n' % sys.argv)
+    print_wrapped(textwrap.fill(
+        'plugins: ' + plugin.format_concise_plugin_list(),
+        width=78,
+        subsequent_indent='    ',
+        ) + '\n')
+    print_wrapped(
         'encoding: %r, fsenc: %r, lang: %r\n' % (
             osutils.get_user_encoding(), sys.getfilesystemencoding(),
             os.environ.get('LANG')))
-    err_file.write("plugins:\n")
-    err_file.write(_format_plugin_list())
+    # We used to show all the plugins here, but it's too verbose.
     err_file.write(
-        "\n\n"
+        "\n"
         "*** Bazaar has encountered an internal error.  This probably indicates a\n"
         "    bug in Bazaar.  You can help us fix it by filing a bug report at\n"
         "        https://bugs.launchpad.net/bzr/+filebug\n"
@@ -112,8 +120,9 @@
     # this function is based on apport_package_hook.py, but omitting some of the
     # Ubuntu-specific policy about what to report and when
 
-    # if the import fails, the exception will be caught at a higher level and
-    # we'll report the error by other means
+    # This import is apparently not used, but we're doing it so that if the
+    # import fails, the exception will be caught at a higher level and we'll
+    # report the error by other means.
     import apport
 
     crash_filename = _write_apport_report_to_file(exc_info)
@@ -143,10 +152,11 @@
     exc_type, exc_object, exc_tb = exc_info
 
     pr = Report()
-    # add_proc_info gives you the memory map of the process, which is not so
-    # useful for Bazaar but does tell you what binary libraries are loaded.
-    # More importantly it sets the ExecutablePath, InterpreterPath, etc.
+    # add_proc_info sets the ExecutablePath, InterpreterPath, etc.
     pr.add_proc_info()
+    # It also adds ProcMaps which for us is rarely useful and mostly noise, so
+    # let's remove it.
+    del pr['ProcMaps']
     pr.add_user_info()
 
     # Package and SourcePackage are needed so that apport will report about even
@@ -254,11 +264,7 @@
 
 
 def _format_plugin_list():
-    plugin_lines = []
-    for name, a_plugin in sorted(plugin.plugins().items()):
-        plugin_lines.append("  %-20s %s [%s]" %
-            (name, a_plugin.path(), a_plugin.__version__))
-    return '\n'.join(plugin_lines)
+    return ''.join(plugin.describe_plugins(show_paths=True))
 
 
 def _format_module_list():

=== modified file 'bzrlib/debug.py'
--- bzrlib/debug.py	2009-03-23 14:59:43 +0000
+++ bzrlib/debug.py	2011-08-20 07:49:15 +0000
@@ -24,22 +24,35 @@
 for a list of the available options.
 """
 
-
 debug_flags = set()
 
 
 def set_debug_flags_from_config():
     """Turn on debug flags based on the global configuration"""
 
-    from bzrlib.config import GlobalConfig
-
-    c = GlobalConfig()
-    value = c.get_user_option("debug_flags")
-    if value is not None:
-        # configobject gives us either a string if there's just one or a list
-        # if there's multiple
-        if isinstance(value, basestring):
-            value = [value]
-        for w in value:
-            w = w.strip()
-            debug_flags.add(w)
+    from bzrlib import config
+
+    c = config.GlobalStack()
+    for f in c.get('debug_flags'):
+        debug_flags.add(f)
+
+
+def set_trace():
+    """Pdb using original stdin and stdout.
+
+    When debugging blackbox tests, sys.stdin and sys.stdout are captured for
+    test purposes and cannot be used for interactive debugging. This class uses
+    the origianl stdin/stdout to allow such use.
+
+    Instead of doing:
+
+       import pdb; pdb.set_trace()
+
+    you can do:
+
+       from bzrlib import debug; debug.set_trace()
+    """
+    import pdb
+    import sys
+    pdb.Pdb(stdin=sys.__stdin__, stdout=sys.__stdout__
+            ).set_trace(sys._getframe().f_back)

=== modified file 'bzrlib/decorators.py'
--- bzrlib/decorators.py	2010-02-17 17:11:16 +0000
+++ bzrlib/decorators.py	2011-05-26 21:02:47 +0000
@@ -30,14 +30,16 @@
 def _get_parameters(func):
     """Recreate the parameters for a function using introspection.
 
-    :return: (function_params, calling_params)
+    :return: (function_params, calling_params, default_values)
         function_params: is a string representing the parameters of the
             function. (such as "a, b, c=None, d=1")
             This is used in the function declaration.
         calling_params: is another string representing how you would call the
             function with the correct parameters. (such as "a, b, c=c, d=d")
-            Assuming you sued function_params in the function declaration, this
+            Assuming you used function_params in the function declaration, this
             is the parameters to put in the function call.
+        default_values_block: a dict with the default values to be passed as
+            the scope for the 'exec' statement.
 
         For example:
 
@@ -49,9 +51,15 @@
     # it globally.
     import inspect
     args, varargs, varkw, defaults = inspect.getargspec(func)
+    defaults_dict = {}
+    def formatvalue(value):
+        default_name = '__default_%d' % len(defaults_dict)
+        defaults_dict[default_name] = value
+        return '=' + default_name
     formatted = inspect.formatargspec(args, varargs=varargs,
                                       varkw=varkw,
-                                      defaults=defaults)
+                                      defaults=defaults,
+                                      formatvalue=formatvalue)
     if defaults is None:
         args_passed = args
     else:
@@ -65,7 +73,7 @@
         args_passed.append('**' + varkw)
     args_passed = ', '.join(args_passed)
 
-    return formatted[1:-1], args_passed
+    return formatted[1:-1], args_passed, defaults_dict
 
 
 def _pretty_needs_read_lock(unbound):
@@ -101,20 +109,26 @@
         try:
             self.unlock()
         finally:
-            raise exc_info[0], exc_info[1], exc_info[2]
+            try:
+                raise exc_info[0], exc_info[1], exc_info[2]
+            finally:
+                del exc_info
     else:
         self.unlock()
         return result
 read_locked = %(name)s_read_locked
 """
-    params, passed_params = _get_parameters(unbound)
+    params, passed_params, defaults_dict = _get_parameters(unbound)
     variables = {'name':unbound.__name__,
                  'params':params,
                  'passed_params':passed_params,
                 }
     func_def = template % variables
 
-    exec func_def in locals()
+    scope = dict(defaults_dict)
+    scope['unbound'] = unbound
+    exec func_def in scope
+    read_locked = scope['read_locked']
 
     read_locked.__doc__ = unbound.__doc__
     read_locked.__name__ = unbound.__name__
@@ -144,7 +158,10 @@
             try:
                 self.unlock()
             finally:
-                raise exc_info[0], exc_info[1], exc_info[2]
+                try:
+                    raise exc_info[0], exc_info[1], exc_info[2]
+                finally:
+                    del exc_info
         else:
             self.unlock()
             return result
@@ -166,20 +183,26 @@
         try:
             self.unlock()
         finally:
-            raise exc_info[0], exc_info[1], exc_info[2]
+            try:
+                raise exc_info[0], exc_info[1], exc_info[2]
+            finally:
+                del exc_info
     else:
         self.unlock()
         return result
 write_locked = %(name)s_write_locked
 """
-    params, passed_params = _get_parameters(unbound)
+    params, passed_params, defaults_dict = _get_parameters(unbound)
     variables = {'name':unbound.__name__,
                  'params':params,
                  'passed_params':passed_params,
                 }
     func_def = template % variables
 
-    exec func_def in locals()
+    scope = dict(defaults_dict)
+    scope['unbound'] = unbound
+    exec func_def in scope
+    write_locked = scope['write_locked']
 
     write_locked.__doc__ = unbound.__doc__
     write_locked.__name__ = unbound.__name__
@@ -197,7 +220,10 @@
             try:
                 self.unlock()
             finally:
-                raise exc_info[0], exc_info[1], exc_info[2]
+                try:
+                    raise exc_info[0], exc_info[1], exc_info[2]
+                finally:
+                    del exc_info
         else:
             self.unlock()
             return result

=== modified file 'bzrlib/delta.h'
--- bzrlib/delta.h	2009-04-09 20:23:07 +0000
+++ bzrlib/delta.h	2011-05-16 14:26:54 +0000
@@ -21,33 +21,53 @@
                                  aggregate source */
 };
 
+/* result type for functions that have multiple failure modes */
+typedef enum {
+    DELTA_OK,             /* Success */
+    DELTA_OUT_OF_MEMORY,  /* Could not allocate required memory */
+    DELTA_INDEX_NEEDED,   /* A delta_index must be passed */
+    DELTA_SOURCE_EMPTY,   /* A source_info had no content */
+    DELTA_SOURCE_BAD,     /* A source_info had invalid or corrupt content */
+    DELTA_BUFFER_EMPTY,   /* A buffer pointer and size */
+    DELTA_SIZE_TOO_BIG,   /* Delta data is larger than the max requested */
+} delta_result;
+
+
 /*
  * create_delta_index: compute index data from given buffer
  *
- * This returns a pointer to a struct delta_index that should be passed to
- * subsequent create_delta() calls, or to free_delta_index().  A NULL pointer
- * is returned on failure.  The given buffer must not be freed nor altered
- * before free_delta_index() is called.  The returned pointer must be freed
- * using free_delta_index().
+ * Returns a delta_result status, when DELTA_OK then *fresh is set to a struct
+ * delta_index that should be passed to subsequent create_delta() calls, or to
+ * free_delta_index().  Other values are a failure, and *fresh is unset.
+ * The given buffer must not be freed nor altered before free_delta_index() is
+ * called. The resultant struct must be freed using free_delta_index().
+ *
+ * :param max_bytes_to_index: Limit the number of regions to sample to this
+ *      amount of text. We will store at most max_bytes_to_index / RABIN_WINDOW
+ *      pointers into the source text.  Useful if src can be unbounded in size,
+ *      and you are willing to trade match accuracy for peak memory.
  */
-extern struct delta_index *
+extern delta_result
 create_delta_index(const struct source_info *src,
-                   struct delta_index *old);
+                   struct delta_index *old,
+                   struct delta_index **fresh,
+                   int max_bytes_to_index);
 
 
 /*
  * create_delta_index_from_delta: compute index data from given buffer
  *
- * This returns a pointer to a struct delta_index that should be passed to
- * subsequent create_delta() calls, or to free_delta_index().  A NULL pointer
- * is returned on failure.
+ * Returns a delta_result status, when DELTA_OK then *fresh is set to a struct
+ * delta_index that should be passed to subsequent create_delta() calls, or to
+ * free_delta_index().  Other values are a failure, and *fresh is unset.
  * The bytes must be in the form of a delta structure, as generated by
  * create_delta(). The generated index will only index the insert bytes, and
  * not any of the control structures.
  */
-extern struct delta_index *
+extern delta_result
 create_delta_index_from_delta(const struct source_info *delta,
-                              struct delta_index *old);
+                              struct delta_index *old,
+                              struct delta_index **fresh);
 /*
  * free_delta_index: free the index created by create_delta_index()
  *
@@ -67,15 +87,16 @@
  *
  * This function may be called multiple times with different buffers using
  * the same delta_index pointer.  If max_delta_size is non-zero and the
- * resulting delta is to be larger than max_delta_size then NULL is returned.
- * On success, a non-NULL pointer to the buffer with the delta data is
- * returned and *delta_size is updated with its size.  The returned buffer
- * must be freed by the caller.
+ * resulting delta is to be larger than max_delta_size then DELTA_SIZE_TOO_BIG
+ * is returned.  Otherwise on success, DELTA_OK is returned and *delta_data is
+ * set to a new buffer with the delta data and *delta_size is updated with its
+ * size.  That buffer must be freed by the caller.
  */
-extern void *
+extern delta_result
 create_delta(const struct delta_index *index,
-         const void *buf, unsigned long bufsize,
-         unsigned long *delta_size, unsigned long max_delta_size);
+             const void *buf, unsigned long bufsize,
+             unsigned long *delta_size, unsigned long max_delta_size,
+             void **delta_data);
 
 /* the smallest possible delta size is 3 bytes
  * Target size, Copy command, Copy length
@@ -102,4 +123,32 @@
     return size;
 }
 
+/*
+ * Return the basic information about a given delta index.
+ * :param index: The delta_index object
+ * :param pos: The offset in the entry list. Start at 0, and walk until you get
+ *      0 as a return code.
+ * :param global_offset: return value, distance to the beginning of all sources
+ * :param hash_val: return value, the RABIN hash associated with this pointer
+ * :param hash_offset: Location for this entry in the hash array.
+ * :return: 1 if pos != -1 (there was data produced)
+ */
+extern int
+get_entry_summary(const struct delta_index *index, int pos,
+                  unsigned int *text_offset, unsigned int *hash_val);
+
+/*
+ * Determine what entry index->hash[X] points to.
+ */
+extern int
+get_hash_offset(const struct delta_index *index, int pos,
+                unsigned int *entry_offset);
+
+/*
+ * Compute the rabin_hash of the given data, it is assumed the data is at least
+ * RABIN_WINDOW wide (16 bytes).
+ */
+extern unsigned int
+rabin_hash(const unsigned char *data);
+
 #endif

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2010-04-30 11:35:43 +0000
+++ bzrlib/delta.py	2011-06-01 12:53:56 +0000
@@ -61,6 +61,7 @@
         self.modified = []
         self.unchanged = []
         self.unversioned = []
+        self.missing = []
 
     def __eq__(self, other):
         if not isinstance(other, TreeDelta):
@@ -137,7 +138,7 @@
             else:
                 delta.removed.append((path[0], file_id, kind[0]))
         elif fully_present[0] is False:
-            continue
+            delta.missing.append((path[1], file_id, kind[1]))
         elif name[0] != name[1] or parent_id[0] != parent_id[1]:
             # If the name changes, or the parent_id changes, we have a rename
             # (if we move a parent, that doesn't count as a rename for the
@@ -160,6 +161,7 @@
     delta.removed.sort()
     delta.added.sort()
     delta.renamed.sort()
+    delta.missing.sort()
     # TODO: jam 20060529 These lists shouldn't need to be sorted
     #       since we added them in alphabetical order.
     delta.modified.sort()
@@ -172,7 +174,8 @@
     """Report changes between two trees"""
 
     def __init__(self, output=None, suppress_root_add=True,
-                 output_file=None, unversioned_filter=None, view_info=None):
+                 output_file=None, unversioned_filter=None, view_info=None,
+                 classify=True):
         """Constructor
 
         :param output: a function with the signature of trace.note, i.e.
@@ -187,6 +190,7 @@
         :param view_info: A tuple of view_name,view_files if only
             items inside a view are to be reported on, or None for
             no view filtering.
+        :param classify: Add special symbols to indicate file kind.
         """
         if output_file is not None:
             if output is not None:
@@ -202,13 +206,19 @@
                              'unchanged': ' ',
                              'created': 'N',
                              'modified': 'M',
-                             'deleted': 'D'}
+                             'deleted': 'D',
+                             'missing': '!',
+                             }
         self.versioned_map = {'added': '+', # versioned target
                               'unchanged': ' ', # versioned in both
                               'removed': '-', # versioned in source
                               'unversioned': '?', # versioned in neither
                               }
         self.unversioned_filter = unversioned_filter
+        if classify:
+            self.kind_marker = osutils.kind_marker
+        else:
+            self.kind_marker = lambda kind: ''
         if view_info is None:
             self.view_name = None
             self.view_files = []
@@ -263,7 +273,7 @@
             # if the file is not missing in the source, we show its kind
             # when we show two paths.
             if kind[0] is not None:
-                old_path += osutils.kind_marker(kind[0])
+                old_path += self.kind_marker(kind[0])
             old_path += " => "
         elif versioned == 'removed':
             # not present in target
@@ -278,10 +288,10 @@
             rename = self.versioned_map[versioned]
         # we show the old kind on the new path when the content is deleted.
         if modified == 'deleted':
-            path += osutils.kind_marker(kind[0])
+            path += self.kind_marker(kind[0])
         # otherwise we always show the current kind when there is one
         elif kind[1] is not None:
-            path += osutils.kind_marker(kind[1])
+            path += self.kind_marker(kind[1])
         if exe_change:
             exe = '*'
         else:
@@ -325,6 +335,8 @@
         else:
             if content_change:
                 modified = "modified"
+            elif kind[0] is None:
+                modified = "missing"
             else:
                 modified = "unchanged"
             if kind[1] == "file":
@@ -334,7 +346,7 @@
                         exe_change, kind)
 
 def report_delta(to_file, delta, short_status=False, show_ids=False, 
-         show_unchanged=False, indent='', filter=None):
+         show_unchanged=False, indent='', filter=None, classify=True):
     """Output this delta in status-like form to to_file.
 
     :param to_file: A file-like object where the output is displayed.
@@ -352,9 +364,13 @@
 
     :param filter: A callable receiving a path and a file id and
         returning True if the path should be displayed.
+
+    :param classify: Add special symbols to indicate file kind.
     """
 
     def decorate_path(path, kind, meta_modified=None):
+        if not classify:
+            return path
         if kind == 'directory':
             path += '/'
         elif kind == 'symlink':
@@ -417,6 +433,7 @@
 
     show_list(delta.removed, 'removed', 'D')
     show_list(delta.added, 'added', 'A')
+    show_list(delta.missing, 'missing', '!')
     extra_modified = []
     # Reorder delta.renamed tuples so that all lists share the same
     # order for their 3 first fields and that they also begin like

=== removed file 'bzrlib/deprecated_graph.py'
--- bzrlib/deprecated_graph.py	2009-03-23 14:59:43 +0000
+++ bzrlib/deprecated_graph.py	1970-01-01 00:00:00 +0000
@@ -1,161 +0,0 @@
-# Copyright (C) 2005, 2008 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-
-def max_distance(node, ancestors, distances, root_descendants):
-    """Calculate the max distance to an ancestor.
-    Return None if not all possible ancestors have known distances"""
-    best = None
-    if node in distances:
-        best = distances[node]
-    for ancestor in ancestors[node]:
-        # skip ancestors we will never traverse:
-        if root_descendants is not None and ancestor not in root_descendants:
-            continue
-        # An ancestor which is not listed in ancestors will never be in
-        # distances, so we pretend it never existed.
-        if ancestor not in ancestors:
-            continue
-        if ancestor not in distances:
-            return None
-        if best is None or distances[ancestor]+1 > best:
-            best = distances[ancestor] + 1
-    return best
-
-
-def node_distances(graph, ancestors, start, root_descendants=None):
-    """Produce a list of nodes, sorted by distance from a start node.
-    This is an algorithm devised by Aaron Bentley, because applying Dijkstra
-    backwards seemed too complicated.
-
-    For each node, we walk its descendants.  If all the descendant's ancestors
-    have a max-distance-to-start, (excluding ones that can never reach start),
-    we calculate their max-distance-to-start, and schedule their descendants.
-
-    So when a node's last parent acquires a distance, it will acquire a
-    distance on the next iteration.
-
-    Once we know the max distances for all nodes, we can return a list sorted
-    by distance, farthest first.
-    """
-    distances = {start: 0}
-    lines = set([start])
-    while len(lines) > 0:
-        new_lines = set()
-        for line in lines:
-            line_descendants = graph[line]
-            for descendant in line_descendants:
-                distance = max_distance(descendant, ancestors, distances,
-                                        root_descendants)
-                if distance is None:
-                    continue
-                distances[descendant] = distance
-                new_lines.add(descendant)
-        lines = new_lines
-    return distances
-
-def nodes_by_distance(distances):
-    """Return a list of nodes sorted by distance"""
-    def by_distance(n):
-        return distances[n],n
-
-    node_list = distances.keys()
-    node_list.sort(key=by_distance, reverse=True)
-    return node_list
-
-def select_farthest(distances, common):
-    """Return the farthest common node, or None if no node qualifies."""
-    node_list = nodes_by_distance(distances)
-    for node in node_list:
-        if node in common:
-            return node
-
-def all_descendants(descendants, start):
-    """Produce a set of all descendants of the start node.
-    The input is a map of node->list of descendants for a graph encompassing
-    start.
-    """
-    result = set()
-    lines = set([start])
-    while len(lines) > 0:
-        new_lines = set()
-        for line in lines:
-            if line not in descendants:
-                continue
-            for descendant in descendants[line]:
-                if descendant in result:
-                    continue
-                result.add(descendant)
-                new_lines.add(descendant)
-        lines = new_lines
-    return result
-
-
-class Graph(object):
-    """A graph object which can memoise and cache results for performance."""
-
-    def __init__(self):
-        super(Graph, self).__init__()
-        self.roots = set([])
-        self.ghosts = set([])
-        self._graph_ancestors = {}
-        self._graph_descendants = {}
-
-    def add_ghost(self, node_id):
-        """Add a ghost to the graph."""
-        self.ghosts.add(node_id)
-        self._ensure_descendant(node_id)
-
-    def add_node(self, node_id, parent_ids):
-        """Add node_id to the graph with parent_ids as its parents."""
-        if len(parent_ids) == 0:
-            self.roots.add(node_id)
-        self._graph_ancestors[node_id] = list(parent_ids)
-        self._ensure_descendant(node_id)
-        for parent in parent_ids:
-            self._ensure_descendant(parent)
-            self._graph_descendants[parent][node_id] = 1
-
-    def _ensure_descendant(self, node_id):
-        """Ensure that a descendant lookup for node_id will work."""
-        if not node_id in self._graph_descendants:
-            self._graph_descendants[node_id] = {}
-
-    def get_ancestors(self):
-        """Return a dictionary of graph node:ancestor_list entries."""
-        return dict(self._graph_ancestors.items())
-
-    def get_ancestry(self, node_id, topo_sorted=True):
-        """Return the inclusive ancestors of node_id in topological order."""
-        # maybe optimise this ?
-        from bzrlib.tsort import topo_sort
-        result = {}
-        pending = set([node_id])
-        while len(pending):
-            current = pending.pop()
-            parents = self._graph_ancestors[current]
-            parents = [parent for parent in parents if parent not in self.ghosts]
-            result[current] = parents
-            for parent in parents:
-                if parent not in result and parent not in pending:
-                    pending.add(parent)
-        if not topo_sorted:
-            return result.keys()
-        return topo_sort(result.items())
-
-    def get_descendants(self):
-        """Return a dictionary of graph node:child_node:distance entries."""
-        return dict(self._graph_descendants.items())

=== modified file 'bzrlib/diff-delta.c'
--- bzrlib/diff-delta.c	2009-12-14 15:54:42 +0000
+++ bzrlib/diff-delta.c	2011-09-30 16:29:17 +0000
@@ -4,7 +4,7 @@
  * This code was greatly inspired by parts of LibXDiff from Davide Libenzi
  * http://www.xmailserver.org/xdiff-lib.html
  *
- * Rewritten for GIT by Nicolas Pitre <nico@cam.org>, (C) 2005-2007
+ * Rewritten for GIT by Nicolas Pitre <nico@fluxnic.net>, (C) 2005-2007
  * Adapted for Bazaar by John Arbash Meinel <john@arbash-meinel.com> (C) 2009
  *
  * This program is free software; you can redistribute it and/or modify
@@ -280,8 +280,11 @@
         if (fit_in_old) {
             // fprintf(stderr, "Fit all %d entries into old index\n",
             //                 copied_count);
-            /* No need to allocate a new buffer */
-            return NULL;
+            /*
+             * No need to allocate a new buffer, but return old_index ptr so
+             * callers can distinguish this from an OOM failure.
+             */
+            return old_index;
         } else {
             // fprintf(stderr, "Fit only %d entries into old index,"
             //                 " reallocating\n", copied_count);
@@ -370,12 +373,14 @@
 }
 
 
-struct delta_index *
+delta_result
 create_delta_index(const struct source_info *src,
-                   struct delta_index *old)
+                   struct delta_index *old,
+                   struct delta_index **fresh,
+                   int max_bytes_to_index)
 {
     unsigned int i, hsize, hmask, num_entries, prev_val, *hash_count;
-    unsigned int total_num_entries;
+    unsigned int total_num_entries, stride, max_entries;
     const unsigned char *data, *buffer;
     struct delta_index *index;
     struct unpacked_index_entry *entry, **hash;
@@ -383,13 +388,24 @@
     unsigned long memsize;
 
     if (!src->buf || !src->size)
-        return NULL;
+        return DELTA_SOURCE_EMPTY;
     buffer = src->buf;
 
     /* Determine index hash size.  Note that indexing skips the
-       first byte to allow for optimizing the Rabin's polynomial
-       initialization in create_delta(). */
+       first byte so we subtract 1 to get the edge cases right.
+     */
+    stride = RABIN_WINDOW;
     num_entries = (src->size - 1)  / RABIN_WINDOW;
+    if (max_bytes_to_index > 0) {
+        max_entries = (unsigned int) (max_bytes_to_index / RABIN_WINDOW);
+        if (num_entries > max_entries) {
+            /* Limit the max number of matching entries. This reduces the 'best'
+             * possible match, but means we don't consume all of ram.
+             */
+            num_entries = max_entries;
+            stride = (src->size - 1) / num_entries;
+        }
+    }
     if (old != NULL)
         total_num_entries = num_entries + old->num_entries;
     else
@@ -408,7 +424,7 @@
           sizeof(*entry) * total_num_entries;
     mem = malloc(memsize);
     if (!mem)
-        return NULL;
+        return DELTA_OUT_OF_MEMORY;
     hash = mem;
     mem = hash + hsize;
     entry = mem;
@@ -419,14 +435,14 @@
     hash_count = calloc(hsize, sizeof(*hash_count));
     if (!hash_count) {
         free(hash);
-        return NULL;
+        return DELTA_OUT_OF_MEMORY;
     }
 
     /* then populate the index for the new data */
     prev_val = ~0;
-    for (data = buffer + num_entries * RABIN_WINDOW - RABIN_WINDOW;
+    for (data = buffer + num_entries * stride - RABIN_WINDOW;
          data >= buffer;
-         data -= RABIN_WINDOW) {
+         data -= stride) {
         unsigned int val = 0;
         for (i = 1; i <= RABIN_WINDOW; i++)
             val = ((val << 8) | data[i]) ^ T[val >> RABIN_SHIFT];
@@ -450,16 +466,15 @@
     total_num_entries = limit_hash_buckets(hash, hash_count, hsize,
                                            total_num_entries);
     free(hash_count);
-    if (old) {
-        old->last_src = src;
-    }
     index = pack_delta_index(hash, hsize, total_num_entries, old);
     free(hash);
+    /* pack_delta_index only returns NULL on malloc failure */
     if (!index) {
-        return NULL;
+        return DELTA_OUT_OF_MEMORY;
     }
     index->last_src = src;
-    return index;
+    *fresh = index;
+    return DELTA_OK;
 }
 
 /* Take some entries, and put them into a custom hash.
@@ -473,7 +488,7 @@
                        unsigned int hsize)
 {
     unsigned int hash_offset, hmask, memsize;
-    struct index_entry *entry, *last_entry;
+    struct index_entry *entry;
     struct index_entry_linked_list *out_entry, **hash;
     void *mem;
 
@@ -493,7 +508,6 @@
     /* We know that entries are in the order we want in the output, but they
      * aren't "grouped" by hash bucket yet.
      */
-    last_entry = entries + num_entries;
     for (entry = entries + num_entries - 1; entry >= entries; --entry) {
         hash_offset = entry->val & hmask;
         out_entry->p_entry = entry;
@@ -518,7 +532,7 @@
     unsigned int i, j, hsize, hmask, total_num_entries;
     struct delta_index *index;
     struct index_entry *entry, *packed_entry, **packed_hash;
-    struct index_entry *last_entry, null_entry = {0};
+    struct index_entry null_entry = {0};
     void *mem;
     unsigned long memsize;
     struct index_entry_linked_list *unpacked_entry, **mini_hash;
@@ -569,7 +583,6 @@
         free(index);
         return NULL;
     }
-    last_entry = entries + num_entries;
     for (i = 0; i < hsize; i++) {
         /*
          * Coalesce all entries belonging in one hash bucket
@@ -679,9 +692,10 @@
     }
 }
 
-struct delta_index *
+delta_result
 create_delta_index_from_delta(const struct source_info *src,
-                              struct delta_index *old_index)
+                              struct delta_index *old_index,
+                              struct delta_index **fresh)
 {
     unsigned int i, num_entries, max_num_entries, prev_val, num_inserted;
     unsigned int hash_offset;
@@ -690,8 +704,10 @@
     struct delta_index *new_index;
     struct index_entry *entry, *entries;
 
+    if (!old_index)
+        return DELTA_INDEX_NEEDED;
     if (!src->buf || !src->size)
-        return NULL;
+        return DELTA_SOURCE_EMPTY;
     buffer = src->buf;
     top = buffer + src->size;
 
@@ -704,10 +720,15 @@
 
     max_num_entries = (src->size - 1)  / RABIN_WINDOW;
 
+    if (!max_num_entries) {
+        *fresh = old_index;
+        return DELTA_OK;
+    }
+
     /* allocate an array to hold whatever entries we find */
     entries = malloc(sizeof(*entry) * max_num_entries);
     if (!entries) /* malloc failure */
-        return NULL;
+        return DELTA_OUT_OF_MEMORY;
 
     /* then populate the index for the new data */
     prev_val = ~0;
@@ -774,16 +795,16 @@
         }
     }
     if (data != top) {
-        /* Something was wrong with this delta */
+        /* The source_info data passed was corrupted or otherwise invalid */
         free(entries);
-        return NULL;
+        return DELTA_SOURCE_BAD;
     }
     if (num_entries == 0) {
         /** Nothing to index **/
         free(entries);
-        return NULL;
+        *fresh = old_index;
+        return DELTA_OK;
     }
-    assert(old_index != NULL);
     old_index->last_src = src;
     /* See if we can fill in these values into the holes in the array */
     entry = entries;
@@ -841,11 +862,15 @@
         new_index = create_index_from_old_and_new_entries(old_index,
             entry, num_entries);
     } else {
-        new_index = NULL;
+        new_index = old_index;
         // fprintf(stderr, "inserted %d without resizing\n", num_inserted);
     }
     free(entries);
-    return new_index;
+    /* create_index_from_old_and_new_entries returns NULL on malloc failure */
+    if (!new_index)
+        return DELTA_OUT_OF_MEMORY;
+    *fresh = new_index;
+    return DELTA_OK;
 }
 
 void free_delta_index(struct delta_index *index)
@@ -853,7 +878,8 @@
     free(index);
 }
 
-unsigned long sizeof_delta_index(struct delta_index *index)
+unsigned long
+sizeof_delta_index(struct delta_index *index)
 {
     if (index)
         return index->memsize;
@@ -867,12 +893,14 @@
  */
 #define MAX_OP_SIZE (5 + 5 + 1 + RABIN_WINDOW + 7)
 
-void *
+delta_result
 create_delta(const struct delta_index *index,
              const void *trg_buf, unsigned long trg_size,
-             unsigned long *delta_size, unsigned long max_size)
+             unsigned long *delta_size, unsigned long max_size,
+             void **delta_data)
 {
-    unsigned int i, outpos, outsize, moff, msize, val;
+    unsigned int i, outpos, outsize, moff, val;
+    int msize;
     const struct source_info *msource;
     int inscnt;
     const unsigned char *ref_data, *ref_top, *data, *top;
@@ -880,9 +908,9 @@
     unsigned long source_size;
 
     if (!trg_buf || !trg_size)
-        return NULL;
+        return DELTA_BUFFER_EMPTY;
     if (index == NULL)
-        return NULL;
+        return DELTA_INDEX_NEEDED;
 
     outpos = 0;
     outsize = 8192;
@@ -890,7 +918,7 @@
         outsize = max_size + MAX_OP_SIZE + 1;
     out = malloc(outsize);
     if (!out)
-        return NULL;
+        return DELTA_OUT_OF_MEMORY;
 
     source_size = index->last_src->size + index->last_src->agg_offset;
 
@@ -943,7 +971,7 @@
                  entry++) {
                 const unsigned char *ref;
                 const unsigned char *src;
-                unsigned int ref_size;
+                int ref_size;
                 if (entry->val != val)
                     continue;
                 ref = entry->ptr;
@@ -956,14 +984,14 @@
                  * match more bytes with this location that we have already
                  * matched.
                  */
-                if (ref_size > top - src)
+                if (ref_size > (top - src))
                     ref_size = top - src;
                 if (ref_size <= msize)
                     break;
                 /* See how many bytes actually match at this location. */
                 while (ref_size-- && *src++ == *ref)
                     ref++;
-                if (msize < ref - entry->ptr) {
+                if (msize < (ref - entry->ptr)) {
                     /* this is our best match so far */
                     msize = ref - entry->ptr;
                     msource = entry->src;
@@ -1069,7 +1097,7 @@
             out = realloc(out, outsize);
             if (!out) {
                 free(tmp);
-                return NULL;
+                return DELTA_OUT_OF_MEMORY;
             }
         }
     }
@@ -1079,11 +1107,81 @@
 
     if (max_size && outpos > max_size) {
         free(out);
-        return NULL;
+        return DELTA_SIZE_TOO_BIG;
     }
 
     *delta_size = outpos;
-    return out;
+    *delta_data = out;
+    return DELTA_OK;
+}
+
+
+int
+get_entry_summary(const struct delta_index *index, int pos,
+                  unsigned int *text_offset, unsigned int *hash_val)
+{
+    int hsize;
+    const struct index_entry *entry;
+    const struct index_entry *start_of_entries;
+    unsigned int offset;
+    if (pos < 0 || text_offset == NULL || hash_val == NULL
+        || index == NULL)
+    {
+        return 0;
+    }
+    hsize = index->hash_mask + 1;
+    start_of_entries = (struct index_entry *)(((struct index_entry **)index->hash) + (hsize + 1));
+    entry = start_of_entries + pos;
+    if (entry > index->last_entry) {
+        return 0;
+    }
+    if (entry->ptr == NULL) {
+        *text_offset = 0;
+        *hash_val = 0;
+    } else {
+        offset = entry->src->agg_offset;
+        offset += (entry->ptr - ((unsigned char *)entry->src->buf));
+        *text_offset = offset;
+        *hash_val = entry->val;
+    }
+    return 1;
+}
+
+
+int
+get_hash_offset(const struct delta_index *index, int pos,
+                unsigned int *entry_offset)
+{
+    int hsize;
+    const struct index_entry *entry;
+    const struct index_entry *start_of_entries;
+    if (pos < 0 || index == NULL || entry_offset == NULL)
+    {
+        return 0;
+    }
+    hsize = index->hash_mask + 1;
+    start_of_entries = (struct index_entry *)(((struct index_entry **)index->hash) + (hsize + 1));
+    if (pos >= hsize) {
+        return 0;
+    }
+    entry = index->hash[pos];
+    if (entry == NULL) {
+        *entry_offset = -1;
+    } else {
+        *entry_offset = (entry - start_of_entries);
+    }
+    return 1;
+}
+
+
+unsigned int
+rabin_hash(const unsigned char *data)
+{
+    int i;
+    unsigned int val = 0;
+    for (i = 0; i < RABIN_WINDOW; i++)
+        val = ((val << 8) | data[i]) ^ T[val >> RABIN_SHIFT];
+    return val;
 }
 
 /* vim: et ts=4 sw=4 sts=4

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2010-07-16 15:20:17 +0000
+++ bzrlib/diff.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd.
+# Copyright (C) 2005-2011 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,13 +25,11 @@
 import errno
 import subprocess
 import tempfile
-import time
 
 from bzrlib import (
-    branch as _mod_branch,
-    bzrdir,
+    cleanup,
     cmdline,
-    cleanup,
+    controldir,
     errors,
     osutils,
     patiencediff,
@@ -41,15 +39,12 @@
     )
 
 from bzrlib.workingtree import WorkingTree
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.registry import (
     Registry,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    )
 from bzrlib.trace import mutter, note, warning
 
 
@@ -290,39 +285,6 @@
                         new_abspath, e)
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def get_trees_and_branches_to_diff(path_list, revision_specs, old_url, new_url,
-                                   apply_view=True):
-    """Get the trees and specific files to diff given a list of paths.
-
-    This method works out the trees to be diff'ed and the files of
-    interest within those trees.
-
-    :param path_list:
-        the list of arguments passed to the diff command
-    :param revision_specs:
-        Zero, one or two RevisionSpecs from the diff command line,
-        saying what revisions to compare.
-    :param old_url:
-        The url of the old branch or tree. If None, the tree to use is
-        taken from the first path, if any, or the current working tree.
-    :param new_url:
-        The url of the new branch or tree. If None, the tree to use is
-        taken from the first path, if any, or the current working tree.
-    :param apply_view:
-        if True and a view is set, apply the view or check that the paths
-        are within it
-    :returns:
-        a tuple of (old_tree, new_tree, old_branch, new_branch,
-        specific_files, extra_trees) where extra_trees is a sequence of
-        additional trees to search in for file-ids.  The trees and branches
-        are not locked.
-    """
-    op = cleanup.OperationWithCleanups(get_trees_and_branches_to_diff_locked)
-    return op.run_simple(path_list, revision_specs, old_url, new_url,
-            op.add_cleanup, apply_view=apply_view)
-    
-
 def get_trees_and_branches_to_diff_locked(
     path_list, revision_specs, old_url, new_url, add_cleanup, apply_view=True):
     """Get the trees and specific files to diff given a list of paths.
@@ -394,7 +356,7 @@
     if old_url is None:
         old_url = default_location
     working_tree, branch, relpath = \
-        bzrdir.BzrDir.open_containing_tree_or_branch(old_url)
+        controldir.ControlDir.open_containing_tree_or_branch(old_url)
     lock_tree_or_branch(working_tree, branch)
     if consider_relpath and relpath != '':
         if working_tree is not None and apply_view:
@@ -408,7 +370,7 @@
         new_url = default_location
     if new_url != old_url:
         working_tree, branch, relpath = \
-            bzrdir.BzrDir.open_containing_tree_or_branch(new_url)
+            controldir.ControlDir.open_containing_tree_or_branch(new_url)
         lock_tree_or_branch(working_tree, branch)
         if consider_relpath and relpath != '':
             if working_tree is not None and apply_view:
@@ -432,13 +394,14 @@
             if view_files:
                 specific_files = view_files
                 view_str = views.view_display_str(view_files)
-                note("*** Ignoring files outside view. View is %s" % view_str)
+                note(gettext("*** Ignoring files outside view. View is %s") % view_str)
 
     # Get extra trees that ought to be searched for file-ids
     extra_trees = None
     if working_tree is not None and working_tree not in (old_tree, new_tree):
         extra_trees = (working_tree,)
-    return old_tree, new_tree, old_branch, new_branch, specific_files, extra_trees
+    return (old_tree, new_tree, old_branch, new_branch,
+            specific_files, extra_trees)
 
 
 def _get_tree_to_diff(spec, tree=None, branch=None, basis_is_default=True):
@@ -465,7 +428,7 @@
     """Show in text form the changes from one tree to another.
 
     :param to_file: The output stream.
-    :param specific_files:Include only changes to these files - None for all
+    :param specific_files: Include only changes to these files - None for all
         changes.
     :param external_diff_options: If set, use an external GNU diff and pass 
         these options.
@@ -736,16 +699,29 @@
                      path_encoding)
 
     @classmethod
-    def make_from_diff_tree(klass, command_string):
+    def make_from_diff_tree(klass, command_string, external_diff_options=None):
         def from_diff_tree(diff_tree):
-            return klass.from_string(command_string, diff_tree.old_tree,
+            full_command_string = [command_string]
+            if external_diff_options is not None:
+                full_command_string += ' ' + external_diff_options
+            return klass.from_string(full_command_string, diff_tree.old_tree,
                                      diff_tree.new_tree, diff_tree.to_file)
         return from_diff_tree
 
     def _get_command(self, old_path, new_path):
         my_map = {'old_path': old_path, 'new_path': new_path}
-        return [AtTemplate(t).substitute(my_map) for t in
-                self.command_template]
+        command = [AtTemplate(t).substitute(my_map) for t in
+                   self.command_template]
+        if sys.platform == 'win32': # Popen doesn't accept unicode on win32
+            command_encoded = []
+            for c in command:
+                if isinstance(c, unicode):
+                    command_encoded.append(c.encode('mbcs'))
+                else:
+                    command_encoded.append(c)
+            return command_encoded
+        else:
+            return command
 
     def _execute(self, old_path, new_path):
         command = self._get_command(old_path, new_path)
@@ -771,12 +747,42 @@
                 raise
         return True
 
+    @staticmethod
+    def _fenc():
+        """Returns safe encoding for passing file path to diff tool"""
+        if sys.platform == 'win32':
+            return 'mbcs'
+        else:
+            # Don't fallback to 'utf-8' because subprocess may not be able to
+            # handle utf-8 correctly when locale is not utf-8.
+            return sys.getfilesystemencoding() or 'ascii'
+
+    def _is_safepath(self, path):
+        """Return true if `path` may be able to pass to subprocess."""
+        fenc = self._fenc()
+        try:
+            return path == path.encode(fenc).decode(fenc)
+        except UnicodeError:
+            return False
+
+    def _safe_filename(self, prefix, relpath):
+        """Replace unsafe character in `relpath` then join `self._root`,
+        `prefix` and `relpath`."""
+        fenc = self._fenc()
+        # encoded_str.replace('?', '_') may break multibyte char.
+        # So we should encode, decode, then replace(u'?', u'_')
+        relpath_tmp = relpath.encode(fenc, 'replace').decode(fenc, 'replace')
+        relpath_tmp = relpath_tmp.replace(u'?', u'_')
+        return osutils.pathjoin(self._root, prefix, relpath_tmp)
+
     def _write_file(self, file_id, tree, prefix, relpath, force_temp=False,
                     allow_write=False):
         if not force_temp and isinstance(tree, WorkingTree):
-            return tree.abspath(tree.id2path(file_id))
-        
-        full_path = osutils.pathjoin(self._root, prefix, relpath)
+            full_path = tree.abspath(tree.id2path(file_id))
+            if self._is_safepath(full_path):
+                return full_path
+
+        full_path = self._safe_filename(prefix, relpath)
         if not force_temp and self._try_symlink_root(tree, prefix):
             return full_path
         parent_dir = osutils.dirname(full_path)
@@ -839,13 +845,13 @@
         """
         old_path = self.old_tree.id2path(file_id)
         new_path = self.new_tree.id2path(file_id)
-        new_abs_path = self._prepare_files(file_id, old_path, new_path,
-                                           allow_write_new=True,
-                                           force_temp=True)[1]
-        command = self._get_command(osutils.pathjoin('old', old_path),
-                                    osutils.pathjoin('new', new_path))
+        old_abs_path, new_abs_path = self._prepare_files(
+                                            file_id, old_path, new_path,
+                                            allow_write_new=True,
+                                            force_temp=True)
+        command = self._get_command(old_abs_path, new_abs_path)
         subprocess.call(command, cwd=self._root)
-        new_file = open(new_abs_path, 'r')
+        new_file = open(new_abs_path, 'rb')
         try:
             return new_file.read()
         finally:
@@ -901,6 +907,7 @@
         """Factory for producing a DiffTree.
 
         Designed to accept options used by show_diff_trees.
+
         :param old_tree: The tree to show as old in the comparison
         :param new_tree: The tree to show as new in the comparison
         :param to_file: File to write comparisons to
@@ -912,7 +919,7 @@
         :param using: Commandline to use to invoke an external diff tool
         """
         if using is not None:
-            extra_factories = [DiffFromTool.make_from_diff_tree(using)]
+            extra_factories = [DiffFromTool.make_from_diff_tree(using, external_diff_options)]
         else:
             extra_factories = []
         if external_diff_options:

=== modified file 'bzrlib/directory_service.py'
--- bzrlib/directory_service.py	2009-03-23 14:59:43 +0000
+++ bzrlib/directory_service.py	2011-04-07 10:36:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,11 +20,16 @@
 to true URLs.  Examples include lp:urls and per-user location aliases.
 """
 
-from bzrlib import errors, registry
+from bzrlib import (
+    errors,
+    registry,
+    )
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib.branch import Branch
-from bzrlib import urlutils
+from bzrlib import (
+    branch as _mod_branch,
+    urlutils,
+    )
 """)
 
 
@@ -68,7 +73,7 @@
     """
 
     def look_up(self, name, url):
-        branch = Branch.open_containing('.')[0]
+        branch = _mod_branch.Branch.open_containing('.')[0]
         lookups = {
             'parent': branch.get_parent,
             'submit': branch.get_submit_branch,

=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2010-01-13 23:06:42 +0000
+++ bzrlib/dirstate.py	2011-09-28 16:50:40 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,91 +20,105 @@
 lines by NL. The field delimiters are ommitted in the grammar, line delimiters
 are not - this is done for clarity of reading. All string data is in utf8.
 
-MINIKIND = "f" | "d" | "l" | "a" | "r" | "t";
-NL = "\n";
-NULL = "\0";
-WHOLE_NUMBER = {digit}, digit;
-BOOLEAN = "y" | "n";
-REVISION_ID = a non-empty utf8 string;
-
-dirstate format = header line, full checksum, row count, parent details,
- ghost_details, entries;
-header line = "#bazaar dirstate flat format 3", NL;
-full checksum = "crc32: ", ["-"], WHOLE_NUMBER, NL;
-row count = "num_entries: ", WHOLE_NUMBER, NL;
-parent_details = WHOLE NUMBER, {REVISION_ID}* NL;
-ghost_details = WHOLE NUMBER, {REVISION_ID}*, NL;
-entries = {entry};
-entry = entry_key, current_entry_details, {parent_entry_details};
-entry_key = dirname,  basename, fileid;
-current_entry_details = common_entry_details, working_entry_details;
-parent_entry_details = common_entry_details, history_entry_details;
-common_entry_details = MINIKIND, fingerprint, size, executable
-working_entry_details = packed_stat
-history_entry_details = REVISION_ID;
-executable = BOOLEAN;
-size = WHOLE_NUMBER;
-fingerprint = a nonempty utf8 sequence with meaning defined by minikind.
-
-Given this definition, the following is useful to know:
-entry (aka row) - all the data for a given key.
-entry[0]: The key (dirname, basename, fileid)
-entry[0][0]: dirname
-entry[0][1]: basename
-entry[0][2]: fileid
-entry[1]: The tree(s) data for this path and id combination.
-entry[1][0]: The current tree
-entry[1][1]: The second tree
-
-For an entry for a tree, we have (using tree 0 - current tree) to demonstrate:
-entry[1][0][0]: minikind
-entry[1][0][1]: fingerprint
-entry[1][0][2]: size
-entry[1][0][3]: executable
-entry[1][0][4]: packed_stat
-OR (for non tree-0)
-entry[1][1][4]: revision_id
+::
+
+    MINIKIND = "f" | "d" | "l" | "a" | "r" | "t";
+    NL = "\\n";
+    NULL = "\\0";
+    WHOLE_NUMBER = {digit}, digit;
+    BOOLEAN = "y" | "n";
+    REVISION_ID = a non-empty utf8 string;
+    
+    dirstate format = header line, full checksum, row count, parent details,
+     ghost_details, entries;
+    header line = "#bazaar dirstate flat format 3", NL;
+    full checksum = "crc32: ", ["-"], WHOLE_NUMBER, NL;
+    row count = "num_entries: ", WHOLE_NUMBER, NL;
+    parent_details = WHOLE NUMBER, {REVISION_ID}* NL;
+    ghost_details = WHOLE NUMBER, {REVISION_ID}*, NL;
+    entries = {entry};
+    entry = entry_key, current_entry_details, {parent_entry_details};
+    entry_key = dirname,  basename, fileid;
+    current_entry_details = common_entry_details, working_entry_details;
+    parent_entry_details = common_entry_details, history_entry_details;
+    common_entry_details = MINIKIND, fingerprint, size, executable
+    working_entry_details = packed_stat
+    history_entry_details = REVISION_ID;
+    executable = BOOLEAN;
+    size = WHOLE_NUMBER;
+    fingerprint = a nonempty utf8 sequence with meaning defined by minikind.
+
+Given this definition, the following is useful to know::
+
+    entry (aka row) - all the data for a given key.
+    entry[0]: The key (dirname, basename, fileid)
+    entry[0][0]: dirname
+    entry[0][1]: basename
+    entry[0][2]: fileid
+    entry[1]: The tree(s) data for this path and id combination.
+    entry[1][0]: The current tree
+    entry[1][1]: The second tree
+
+For an entry for a tree, we have (using tree 0 - current tree) to demonstrate::
+
+    entry[1][0][0]: minikind
+    entry[1][0][1]: fingerprint
+    entry[1][0][2]: size
+    entry[1][0][3]: executable
+    entry[1][0][4]: packed_stat
+
+OR (for non tree-0)::
+
+    entry[1][1][4]: revision_id
 
 There may be multiple rows at the root, one per id present in the root, so the
-in memory root row is now:
-self._dirblocks[0] -> ('', [entry ...]),
-and the entries in there are
-entries[0][0]: ''
-entries[0][1]: ''
-entries[0][2]: file_id
-entries[1][0]: The tree data for the current tree for this fileid at /
-etc.
-
-Kinds:
-'r' is a relocated entry: This path is not present in this tree with this id,
-    but the id can be found at another location. The fingerprint is used to
-    point to the target location.
-'a' is an absent entry: In that tree the id is not present at this path.
-'d' is a directory entry: This path in this tree is a directory with the
-    current file id. There is no fingerprint for directories.
-'f' is a file entry: As for directory, but it's a file. The fingerprint is the
-    sha1 value of the file's canonical form, i.e. after any read filters have
-    been applied to the convenience form stored in the working tree.
-'l' is a symlink entry: As for directory, but a symlink. The fingerprint is the
-    link target.
-'t' is a reference to a nested subtree; the fingerprint is the referenced
-    revision.
+in memory root row is now::
+
+    self._dirblocks[0] -> ('', [entry ...]),
+
+and the entries in there are::
+
+    entries[0][0]: ''
+    entries[0][1]: ''
+    entries[0][2]: file_id
+    entries[1][0]: The tree data for the current tree for this fileid at /
+    etc.
+
+Kinds::
+
+    'r' is a relocated entry: This path is not present in this tree with this
+        id, but the id can be found at another location. The fingerprint is
+        used to point to the target location.
+    'a' is an absent entry: In that tree the id is not present at this path.
+    'd' is a directory entry: This path in this tree is a directory with the
+        current file id. There is no fingerprint for directories.
+    'f' is a file entry: As for directory, but it's a file. The fingerprint is
+        the sha1 value of the file's canonical form, i.e. after any read
+        filters have been applied to the convenience form stored in the working
+        tree.
+    'l' is a symlink entry: As for directory, but a symlink. The fingerprint is
+        the link target.
+    't' is a reference to a nested subtree; the fingerprint is the referenced
+        revision.
 
 Ordering:
 
-The entries on disk and in memory are ordered according to the following keys:
+The entries on disk and in memory are ordered according to the following keys::
 
     directory, as a list of components
     filename
     file-id
 
 --- Format 1 had the following different definition: ---
-rows = dirname, NULL, basename, NULL, MINIKIND, NULL, fileid_utf8, NULL,
-    WHOLE NUMBER (* size *), NULL, packed stat, NULL, sha1|symlink target,
-    {PARENT ROW}
-PARENT ROW = NULL, revision_utf8, NULL, MINIKIND, NULL, dirname, NULL,
-    basename, NULL, WHOLE NUMBER (* size *), NULL, "y" | "n", NULL,
-    SHA1
+
+::
+
+    rows = dirname, NULL, basename, NULL, MINIKIND, NULL, fileid_utf8, NULL,
+        WHOLE NUMBER (* size *), NULL, packed stat, NULL, sha1|symlink target,
+        {PARENT ROW}
+    PARENT ROW = NULL, revision_utf8, NULL, MINIKIND, NULL, dirname, NULL,
+        basename, NULL, WHOLE NUMBER (* size *), NULL, "y" | "n", NULL,
+        SHA1
 
 PARENT ROW's are emitted for every parent that is not in the ghosts details
 line. That is, if the parents are foo, bar, baz, and the ghosts are bar, then
@@ -135,19 +149,21 @@
 ----
 
 Design priorities:
- 1) Fast end to end use for bzr's top 5 uses cases. (commmit/diff/status/merge/???)
- 2) fall back current object model as needed.
- 3) scale usably to the largest trees known today - say 50K entries. (mozilla
+ 1. Fast end to end use for bzr's top 5 uses cases. (commmit/diff/status/merge/???)
+ 2. fall back current object model as needed.
+ 3. scale usably to the largest trees known today - say 50K entries. (mozilla
     is an example of this)
 
 
 Locking:
+
  Eventually reuse dirstate objects across locks IFF the dirstate file has not
  been modified, but will require that we flush/ignore cached stat-hit data
  because we won't want to restat all files on disk just because a lock was
  acquired, yet we cannot trust the data after the previous lock was released.
 
-Memory representation:
+Memory representation::
+
  vector of all directories, and vector of the childen ?
    i.e.
      root_entrie = (direntry for root, [parent_direntries_for_root]),
@@ -167,6 +183,7 @@
     - What's the risk of error here? Once we have the base format being processed
       we should have a net win regardless of optimality. So we are going to
       go with what seems reasonable.
+
 open questions:
 
 Maybe we should do a test profile of the core structure - 10K simulated
@@ -202,25 +219,26 @@
 """
 
 import bisect
-import binascii
 import errno
 import operator
 import os
 from stat import S_IEXEC
 import stat
-import struct
 import sys
 import time
 import zlib
 
 from bzrlib import (
     cache_utf8,
+    config,
     debug,
     errors,
     inventory,
     lock,
     osutils,
+    static_tuple,
     trace,
+    urlutils,
     )
 
 
@@ -231,39 +249,6 @@
 ERROR_DIRECTORY = 267
 
 
-if not getattr(struct, '_compile', None):
-    # Cannot pre-compile the dirstate pack_stat
-    def pack_stat(st, _encode=binascii.b2a_base64, _pack=struct.pack):
-        """Convert stat values into a packed representation."""
-        return _encode(_pack('>LLLLLL', st.st_size, int(st.st_mtime),
-            int(st.st_ctime), st.st_dev, st.st_ino & 0xFFFFFFFF,
-            st.st_mode))[:-1]
-else:
-    # compile the struct compiler we need, so as to only do it once
-    from _struct import Struct
-    _compiled_pack = Struct('>LLLLLL').pack
-    def pack_stat(st, _encode=binascii.b2a_base64, _pack=_compiled_pack):
-        """Convert stat values into a packed representation."""
-        # jam 20060614 it isn't really worth removing more entries if we
-        # are going to leave it in packed form.
-        # With only st_mtime and st_mode filesize is 5.5M and read time is 275ms
-        # With all entries, filesize is 5.9M and read time is maybe 280ms
-        # well within the noise margin
-
-        # base64 encoding always adds a final newline, so strip it off
-        # The current version
-        return _encode(_pack(st.st_size, int(st.st_mtime), int(st.st_ctime),
-            st.st_dev, st.st_ino & 0xFFFFFFFF, st.st_mode))[:-1]
-        # This is 0.060s / 1.520s faster by not encoding as much information
-        # return _encode(_pack('>LL', int(st.st_mtime), st.st_mode))[:-1]
-        # This is not strictly faster than _encode(_pack())[:-1]
-        # return '%X.%X.%X.%X.%X.%X' % (
-        #      st.st_size, int(st.st_mtime), int(st.st_ctime),
-        #      st.st_dev, st.st_ino, st.st_mode)
-        # Similar to the _encode(_pack('>LL'))
-        # return '%X.%X' % (int(st.st_mtime), st.st_mode)
-
-
 class SHA1Provider(object):
     """An interface for getting sha1s of a file."""
 
@@ -354,20 +339,25 @@
     NOT_IN_MEMORY = 0
     IN_MEMORY_UNMODIFIED = 1
     IN_MEMORY_MODIFIED = 2
+    IN_MEMORY_HASH_MODIFIED = 3 # Only hash-cache updates
 
     # A pack_stat (the x's) that is just noise and will never match the output
     # of base64 encode.
     NULLSTAT = 'x' * 32
-    NULL_PARENT_DETAILS = ('a', '', 0, False, '')
+    NULL_PARENT_DETAILS = static_tuple.StaticTuple('a', '', 0, False, '')
 
     HEADER_FORMAT_2 = '#bazaar dirstate flat format 2\n'
     HEADER_FORMAT_3 = '#bazaar dirstate flat format 3\n'
 
-    def __init__(self, path, sha1_provider):
+    def __init__(self, path, sha1_provider, worth_saving_limit=0):
         """Create a  DirState object.
 
         :param path: The path at which the dirstate file on disk should live.
         :param sha1_provider: an object meeting the SHA1Provider interface.
+        :param worth_saving_limit: when the exact number of hash changed
+            entries is known, only bother saving the dirstate if more than
+            this count of entries have changed.
+            -1 means never save hash changes, 0 means always save hash changes.
         """
         # _header_state and _dirblock_state represent the current state
         # of the dirstate metadata and the per-row data respectiely.
@@ -410,11 +400,48 @@
         # during commit.
         self._last_block_index = None
         self._last_entry_index = None
+        # The set of known hash changes
+        self._known_hash_changes = set()
+        # How many hash changed entries can we have without saving
+        self._worth_saving_limit = worth_saving_limit
+        self._config_stack = config.LocationStack(urlutils.local_path_to_url(
+            path))
 
     def __repr__(self):
         return "%s(%r)" % \
             (self.__class__.__name__, self._filename)
 
+    def _mark_modified(self, hash_changed_entries=None, header_modified=False):
+        """Mark this dirstate as modified.
+
+        :param hash_changed_entries: if non-None, mark just these entries as
+            having their hash modified.
+        :param header_modified: mark the header modified as well, not just the
+            dirblocks.
+        """
+        #trace.mutter_callsite(3, "modified hash entries: %s", hash_changed_entries)
+        if hash_changed_entries:
+            self._known_hash_changes.update([e[0] for e in hash_changed_entries])
+            if self._dirblock_state in (DirState.NOT_IN_MEMORY,
+                                        DirState.IN_MEMORY_UNMODIFIED):
+                # If the dirstate is already marked a IN_MEMORY_MODIFIED, then
+                # that takes precedence.
+                self._dirblock_state = DirState.IN_MEMORY_HASH_MODIFIED
+        else:
+            # TODO: Since we now have a IN_MEMORY_HASH_MODIFIED state, we
+            #       should fail noisily if someone tries to set
+            #       IN_MEMORY_MODIFIED but we don't have a write-lock!
+            # We don't know exactly what changed so disable smart saving
+            self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        if header_modified:
+            self._header_state = DirState.IN_MEMORY_MODIFIED
+
+    def _mark_unmodified(self):
+        """Mark this dirstate as unmodified."""
+        self._header_state = DirState.IN_MEMORY_UNMODIFIED
+        self._dirblock_state = DirState.IN_MEMORY_UNMODIFIED
+        self._known_hash_changes = set()
+
     def add(self, path, file_id, kind, stat, fingerprint):
         """Add a path to be tracked.
 
@@ -546,9 +573,9 @@
         if kind == 'directory':
            # insert a new dirblock
            self._ensure_block(block_index, entry_index, utf8path)
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified()
         if self._id_index:
-            self._id_index.setdefault(entry_key[2], set()).add(entry_key)
+            self._add_to_id_index(self._id_index, entry_key)
 
     def _bisect(self, paths):
         """Bisect through the disk structure for specific rows.
@@ -1018,8 +1045,7 @@
 
         self._ghosts = []
         self._parents = [parents[0]]
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
-        self._header_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified(header_modified=True)
 
     def _empty_parent_info(self):
         return [DirState.NULL_PARENT_DETAILS] * (len(self._parents) -
@@ -1154,9 +1180,10 @@
     def _fields_per_entry(self):
         """How many null separated fields should be in each entry row.
 
-        Each line now has an extra '\n' field which is not used
+        Each line now has an extra '\\n' field which is not used
         so we just skip over it
-        entry size:
+
+        entry size::
             3 fields for the key
             + number of fields per tree_data (5) * tree count
             + newline
@@ -1275,6 +1302,14 @@
             raise
         return result
 
+    def _check_delta_is_valid(self, delta):
+        return list(inventory._check_delta_unique_ids(
+                    inventory._check_delta_unique_old_paths(
+                    inventory._check_delta_unique_new_paths(
+                    inventory._check_delta_ids_match_entry(
+                    inventory._check_delta_ids_are_valid(
+                    inventory._check_delta_new_path_entry_both_or_None(delta)))))))
+
     def update_by_delta(self, delta):
         """Apply an inventory delta to the dirstate for tree 0
 
@@ -1298,15 +1333,10 @@
         new_ids = set()
         # This loop transforms the delta to single atomic operations that can
         # be executed and validated.
-        for old_path, new_path, file_id, inv_entry in sorted(
-            inventory._check_delta_unique_old_paths(
-            inventory._check_delta_unique_new_paths(
-            inventory._check_delta_ids_match_entry(
-            inventory._check_delta_ids_are_valid(
-            inventory._check_delta_new_path_entry_both_or_None(delta))))),
-            reverse=True):
+        delta = sorted(self._check_delta_is_valid(delta), reverse=True)
+        for old_path, new_path, file_id, inv_entry in delta:
             if (file_id in insertions) or (file_id in removals):
-                raise errors.InconsistentDelta(old_path or new_path, file_id,
+                self._raise_invalid(old_path or new_path, file_id,
                     "repeated file_id")
             if old_path is not None:
                 old_path = old_path.encode('utf-8')
@@ -1315,7 +1345,7 @@
                 new_ids.add(file_id)
             if new_path is not None:
                 if inv_entry is None:
-                    raise errors.InconsistentDelta(new_path, file_id,
+                    self._raise_invalid(new_path, file_id,
                         "new_path with no entry")
                 new_path = new_path.encode('utf-8')
                 dirname_utf8, basename = osutils.split(new_path)
@@ -1362,7 +1392,8 @@
             # _get_entry raises BzrError when a request is inconsistent; we
             # want such errors to be shown as InconsistentDelta - and that 
             # fits the behaviour we trigger.
-            raise errors.InconsistentDeltaDelta(delta, "error from _get_entry.")
+            raise errors.InconsistentDeltaDelta(delta,
+                "error from _get_entry. %s" % (e,))
 
     def _apply_removals(self, removals):
         for file_id, path in sorted(removals, reverse=True,
@@ -1373,16 +1404,13 @@
             try:
                 entry = self._dirblocks[block_i][1][entry_i]
             except IndexError:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(path, file_id,
+                self._raise_invalid(path, file_id,
                     "Wrong path for old path.")
             if not f_present or entry[1][0][0] in 'ar':
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(path, file_id,
+                self._raise_invalid(path, file_id,
                     "Wrong path for old path.")
             if file_id != entry[0][2]:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(path, file_id,
+                self._raise_invalid(path, file_id,
                     "Attempt to remove path has wrong id - found %r."
                     % entry[0][2])
             self._make_absent(entry)
@@ -1398,8 +1426,7 @@
                 # be due to it being in a parent tree, or a corrupt delta.
                 for child_entry in self._dirblocks[block_i][1]:
                     if child_entry[1][0][0] not in ('r', 'a'):
-                        self._changes_aborted = True
-                        raise errors.InconsistentDelta(path, entry[0][2],
+                        self._raise_invalid(path, entry[0][2],
                             "The file id was deleted but its children were "
                             "not deleted.")
 
@@ -1409,8 +1436,7 @@
                 self.update_minimal(key, minikind, executable, fingerprint,
                                     path_utf8=path_utf8)
         except errors.NotVersionedError:
-            self._changes_aborted = True
-            raise errors.InconsistentDelta(path_utf8.decode('utf8'), key[2],
+            self._raise_invalid(path_utf8.decode('utf8'), key[2],
                 "Missing parent")
 
     def update_basis_by_delta(self, delta, new_revid):
@@ -1424,9 +1450,6 @@
         Note that an exception during the operation of this method will leave
         the dirstate in a corrupt state where it should not be saved.
 
-        Finally, we expect all changes to be synchronising the basis tree with
-        the working tree.
-
         :param new_revid: The new revision id for the trees parent.
         :param delta: An inventory delta (see apply_inventory_delta) describing
             the changes from the current left most parent revision to new_revid.
@@ -1444,7 +1467,7 @@
 
         self._parents[0] = new_revid
 
-        delta = sorted(delta, reverse=True)
+        delta = sorted(self._check_delta_is_valid(delta), reverse=True)
         adds = []
         changes = []
         deletes = []
@@ -1471,11 +1494,13 @@
         new_ids = set()
         for old_path, new_path, file_id, inv_entry in delta:
             if inv_entry is not None and file_id != inv_entry.file_id:
-                raise errors.InconsistentDelta(new_path, file_id,
+                self._raise_invalid(new_path, file_id,
                     "mismatched entry file_id %r" % inv_entry)
-            if new_path is not None:
+            if new_path is None:
+                new_path_utf8 = None
+            else:
                 if inv_entry is None:
-                    raise errors.InconsistentDelta(new_path, file_id,
+                    self._raise_invalid(new_path, file_id,
                         "new_path with no entry")
                 new_path_utf8 = encode(new_path)
                 # note the parent for validation
@@ -1483,12 +1508,27 @@
                 if basename_utf8:
                     parents.add((dirname_utf8, inv_entry.parent_id))
             if old_path is None:
-                adds.append((None, encode(new_path), file_id,
+                old_path_utf8 = None
+            else:
+                old_path_utf8 = encode(old_path)
+            if old_path is None:
+                adds.append((None, new_path_utf8, file_id,
                     inv_to_entry(inv_entry), True))
                 new_ids.add(file_id)
             elif new_path is None:
-                deletes.append((encode(old_path), None, file_id, None, True))
-            elif (old_path, new_path) != root_only:
+                deletes.append((old_path_utf8, None, file_id, None, True))
+            elif (old_path, new_path) == root_only:
+                # change things in-place
+                # Note: the case of a parent directory changing its file_id
+                #       tends to break optimizations here, because officially
+                #       the file has actually been moved, it just happens to
+                #       end up at the same path. If we can figure out how to
+                #       handle that case, we can avoid a lot of add+delete
+                #       pairs for objects that stay put.
+                # elif old_path == new_path:
+                changes.append((old_path_utf8, new_path_utf8, file_id,
+                                inv_to_entry(inv_entry)))
+            else:
                 # Renames:
                 # Because renames must preserve their children we must have
                 # processed all relocations and removes before hand. The sort
@@ -1504,36 +1544,31 @@
                 self._update_basis_apply_deletes(deletes)
                 deletes = []
                 # Split into an add/delete pair recursively.
-                adds.append((None, new_path_utf8, file_id,
-                    inv_to_entry(inv_entry), False))
+                adds.append((old_path_utf8, new_path_utf8, file_id,
+                             inv_to_entry(inv_entry), False))
                 # Expunge deletes that we've seen so that deleted/renamed
                 # children of a rename directory are handled correctly.
-                new_deletes = reversed(list(self._iter_child_entries(1,
-                    encode(old_path))))
+                new_deletes = reversed(list(
+                    self._iter_child_entries(1, old_path_utf8)))
                 # Remove the current contents of the tree at orig_path, and
                 # reinsert at the correct new path.
                 for entry in new_deletes:
-                    if entry[0][0]:
-                        source_path = entry[0][0] + '/' + entry[0][1]
+                    child_dirname, child_basename, child_file_id = entry[0]
+                    if child_dirname:
+                        source_path = child_dirname + '/' + child_basename
                     else:
-                        source_path = entry[0][1]
+                        source_path = child_basename
                     if new_path_utf8:
                         target_path = new_path_utf8 + source_path[len(old_path):]
                     else:
                         if old_path == '':
                             raise AssertionError("cannot rename directory to"
-                                " itself")
+                                                 " itself")
                         target_path = source_path[len(old_path) + 1:]
                     adds.append((None, target_path, entry[0][2], entry[1][1], False))
                     deletes.append(
                         (source_path, target_path, entry[0][2], None, False))
-                deletes.append(
-                    (encode(old_path), new_path, file_id, None, False))
-            else:
-                # changes to just the root should not require remove/insertion
-                # of everything.
-                changes.append((encode(old_path), encode(new_path), file_id,
-                    inv_to_entry(inv_entry)))
+                deletes.append((old_path_utf8, new_path, file_id, None, False))
         self._check_delta_ids_absent(new_ids, delta, 1)
         try:
             # Finish expunging deletes/first half of renames.
@@ -1549,14 +1584,12 @@
             if 'integrity error' not in str(e):
                 raise
             # _get_entry raises BzrError when a request is inconsistent; we
-            # want such errors to be shown as InconsistentDelta - and that 
-            # fits the behaviour we trigger. Partof this is driven by dirstate
-            # only supporting deltas that turn the basis into a closer fit to
-            # the active tree.
-            raise errors.InconsistentDeltaDelta(delta, "error from _get_entry.")
+            # want such errors to be shown as InconsistentDelta - and that
+            # fits the behaviour we trigger.
+            raise errors.InconsistentDeltaDelta(delta,
+                "error from _get_entry. %s" % (e,))
 
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
-        self._header_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified(header_modified=True)
         self._id_index = None
         return
 
@@ -1566,7 +1599,7 @@
             return
         id_index = self._get_id_index()
         for file_id in new_ids:
-            for key in id_index.get(file_id, []):
+            for key in id_index.get(file_id, ()):
                 block_i, entry_i, d_present, f_present = \
                     self._get_block_entry_index(key[0], key[1], tree_index)
                 if not f_present:
@@ -1576,13 +1609,14 @@
                 if entry[0][2] != file_id:
                     # Different file_id, so not what we want.
                     continue
-                # NB: No changes made before this helper is called, so no need
-                # to set the _changes_aborted flag.
-                raise errors.InconsistentDelta(
-                    ("%s/%s" % key[0:2]).decode('utf8'), file_id,
+                self._raise_invalid(("%s/%s" % key[0:2]).decode('utf8'), file_id,
                     "This file_id is new in the delta but already present in "
                     "the target")
 
+    def _raise_invalid(self, path, file_id, reason):
+        self._changes_aborted = True
+        raise errors.InconsistentDelta(path, file_id, reason)
+
     def _update_basis_apply_adds(self, adds):
         """Apply a sequence of adds to tree 1 during update_basis_by_delta.
 
@@ -1596,26 +1630,112 @@
         """
         # Adds are accumulated partly from renames, so can be in any input
         # order - sort it.
-        adds.sort()
+        # TODO: we may want to sort in dirblocks order. That way each entry
+        #       will end up in the same directory, allowing the _get_entry
+        #       fast-path for looking up 2 items in the same dir work.
+        adds.sort(key=lambda x: x[1])
         # adds is now in lexographic order, which places all parents before
         # their children, so we can process it linearly.
         absent = 'ar'
+        st = static_tuple.StaticTuple
         for old_path, new_path, file_id, new_details, real_add in adds:
-            # the entry for this file_id must be in tree 0.
-            entry = self._get_entry(0, file_id, new_path)
-            if entry[0] is None or entry[0][2] != file_id:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(new_path, file_id,
-                    'working tree does not contain new entry')
-            if real_add and entry[1][1][0] not in absent:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(new_path, file_id,
-                    'The entry was considered to be a genuinely new record,'
-                    ' but there was already an old record for it.')
-            # We don't need to update the target of an 'r' because the handling
-            # of renames turns all 'r' situations into a delete at the original
-            # location.
-            entry[1][1] = new_details
+            dirname, basename = osutils.split(new_path)
+            entry_key = st(dirname, basename, file_id)
+            block_index, present = self._find_block_index_from_key(entry_key)
+            if not present:
+                self._raise_invalid(new_path, file_id,
+                    "Unable to find block for this record."
+                    " Was the parent added?")
+            block = self._dirblocks[block_index][1]
+            entry_index, present = self._find_entry_index(entry_key, block)
+            if real_add:
+                if old_path is not None:
+                    self._raise_invalid(new_path, file_id,
+                        'considered a real add but still had old_path at %s'
+                        % (old_path,))
+            if present:
+                entry = block[entry_index]
+                basis_kind = entry[1][1][0]
+                if basis_kind == 'a':
+                    entry[1][1] = new_details
+                elif basis_kind == 'r':
+                    raise NotImplementedError()
+                else:
+                    self._raise_invalid(new_path, file_id,
+                        "An entry was marked as a new add"
+                        " but the basis target already existed")
+            else:
+                # The exact key was not found in the block. However, we need to
+                # check if there is a key next to us that would have matched.
+                # We only need to check 2 locations, because there are only 2
+                # trees present.
+                for maybe_index in range(entry_index-1, entry_index+1):
+                    if maybe_index < 0 or maybe_index >= len(block):
+                        continue
+                    maybe_entry = block[maybe_index]
+                    if maybe_entry[0][:2] != (dirname, basename):
+                        # Just a random neighbor
+                        continue
+                    if maybe_entry[0][2] == file_id:
+                        raise AssertionError(
+                            '_find_entry_index didnt find a key match'
+                            ' but walking the data did, for %s'
+                            % (entry_key,))
+                    basis_kind = maybe_entry[1][1][0]
+                    if basis_kind not in 'ar':
+                        self._raise_invalid(new_path, file_id,
+                            "we have an add record for path, but the path"
+                            " is already present with another file_id %s"
+                            % (maybe_entry[0][2],))
+
+                entry = (entry_key, [DirState.NULL_PARENT_DETAILS,
+                                     new_details])
+                block.insert(entry_index, entry)
+
+            active_kind = entry[1][0][0]
+            if active_kind == 'a':
+                # The active record shows up as absent, this could be genuine,
+                # or it could be present at some other location. We need to
+                # verify.
+                id_index = self._get_id_index()
+                # The id_index may not be perfectly accurate for tree1, because
+                # we haven't been keeping it updated. However, it should be
+                # fine for tree0, and that gives us enough info for what we
+                # need
+                keys = id_index.get(file_id, ())
+                for key in keys:
+                    block_i, entry_i, d_present, f_present = \
+                        self._get_block_entry_index(key[0], key[1], 0)
+                    if not f_present:
+                        continue
+                    active_entry = self._dirblocks[block_i][1][entry_i]
+                    if (active_entry[0][2] != file_id):
+                        # Some other file is at this path, we don't need to
+                        # link it.
+                        continue
+                    real_active_kind = active_entry[1][0][0]
+                    if real_active_kind in 'ar':
+                        # We found a record, which was not *this* record,
+                        # which matches the file_id, but is not actually
+                        # present. Something seems *really* wrong.
+                        self._raise_invalid(new_path, file_id,
+                            "We found a tree0 entry that doesnt make sense")
+                    # Now, we've found a tree0 entry which matches the file_id
+                    # but is at a different location. So update them to be
+                    # rename records.
+                    active_dir, active_name = active_entry[0][:2]
+                    if active_dir:
+                        active_path = active_dir + '/' + active_name
+                    else:
+                        active_path = active_name
+                    active_entry[1][1] = st('r', new_path, 0, False, '')
+                    entry[1][0] = st('r', active_path, 0, False, '')
+            elif active_kind == 'r':
+                raise NotImplementedError()
+
+            new_kind = new_details[0]
+            if new_kind == 'd':
+                self._ensure_block(block_index, entry_index, new_path)
 
     def _update_basis_apply_changes(self, changes):
         """Apply a sequence of changes to tree 1 during update_basis_by_delta.
@@ -1626,16 +1746,10 @@
         absent = 'ar'
         for old_path, new_path, file_id, new_details in changes:
             # the entry for this file_id must be in tree 0.
-            entry = self._get_entry(0, file_id, new_path)
-            if entry[0] is None or entry[0][2] != file_id:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(new_path, file_id,
-                    'working tree does not contain new entry')
-            if (entry[1][0][0] in absent or
-                entry[1][1][0] in absent):
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(new_path, file_id,
-                    'changed considered absent')
+            entry = self._get_entry(1, file_id, new_path)
+            if entry[0] is None or entry[1][1][0] in 'ar':
+                self._raise_invalid(new_path, file_id,
+                    'changed entry considered not present')
             entry[1][1] = new_details
 
     def _update_basis_apply_deletes(self, deletes):
@@ -1653,44 +1767,67 @@
         null = DirState.NULL_PARENT_DETAILS
         for old_path, new_path, file_id, _, real_delete in deletes:
             if real_delete != (new_path is None):
-                self._changes_aborted = True
-                raise AssertionError("bad delete delta")
+                self._raise_invalid(old_path, file_id, "bad delete delta")
             # the entry for this file_id must be in tree 1.
             dirname, basename = osutils.split(old_path)
             block_index, entry_index, dir_present, file_present = \
                 self._get_block_entry_index(dirname, basename, 1)
             if not file_present:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(old_path, file_id,
+                self._raise_invalid(old_path, file_id,
                     'basis tree does not contain removed entry')
             entry = self._dirblocks[block_index][1][entry_index]
+            # The state of the entry in the 'active' WT
+            active_kind = entry[1][0][0]
             if entry[0][2] != file_id:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(old_path, file_id,
+                self._raise_invalid(old_path, file_id,
                     'mismatched file_id in tree 1')
-            if real_delete:
-                if entry[1][0][0] != 'a':
-                    self._changes_aborted = True
-                    raise errors.InconsistentDelta(old_path, file_id,
-                            'This was marked as a real delete, but the WT state'
-                            ' claims that it still exists and is versioned.')
+            dir_block = ()
+            old_kind = entry[1][1][0]
+            if active_kind in 'ar':
+                # The active tree doesn't have this file_id.
+                # The basis tree is changing this record. If this is a
+                # rename, then we don't want the record here at all
+                # anymore. If it is just an in-place change, we want the
+                # record here, but we'll add it if we need to. So we just
+                # delete it
+                if active_kind == 'r':
+                    active_path = entry[1][0][1]
+                    active_entry = self._get_entry(0, file_id, active_path)
+                    if active_entry[1][1][0] != 'r':
+                        self._raise_invalid(old_path, file_id,
+                            "Dirstate did not have matching rename entries")
+                    elif active_entry[1][0][0] in 'ar':
+                        self._raise_invalid(old_path, file_id,
+                            "Dirstate had a rename pointing at an inactive"
+                            " tree0")
+                    active_entry[1][1] = null
                 del self._dirblocks[block_index][1][entry_index]
+                if old_kind == 'd':
+                    # This was a directory, and the active tree says it
+                    # doesn't exist, and now the basis tree says it doesn't
+                    # exist. Remove its dirblock if present
+                    (dir_block_index,
+                     present) = self._find_block_index_from_key(
+                        (old_path, '', ''))
+                    if present:
+                        dir_block = self._dirblocks[dir_block_index][1]
+                        if not dir_block:
+                            # This entry is empty, go ahead and just remove it
+                            del self._dirblocks[dir_block_index]
             else:
-                if entry[1][0][0] == 'a':
-                    self._changes_aborted = True
-                    raise errors.InconsistentDelta(old_path, file_id,
-                        'The entry was considered a rename, but the source path'
-                        ' is marked as absent.')
-                    # For whatever reason, we were asked to rename an entry
-                    # that was originally marked as deleted. This could be
-                    # because we are renaming the parent directory, and the WT
-                    # current state has the file marked as deleted.
-                elif entry[1][0][0] == 'r':
-                    # implement the rename
-                    del self._dirblocks[block_index][1][entry_index]
-                else:
-                    # it is being resurrected here, so blank it out temporarily.
-                    self._dirblocks[block_index][1][entry_index][1][1] = null
+                # There is still an active record, so just mark this
+                # removed.
+                entry[1][1] = null
+                block_i, entry_i, d_present, f_present = \
+                    self._get_block_entry_index(old_path, '', 1)
+                if d_present:
+                    dir_block = self._dirblocks[block_i][1]
+            for child_entry in dir_block:
+                child_basis_kind = child_entry[1][1][0]
+                if child_basis_kind not in 'ar':
+                    self._raise_invalid(old_path, file_id,
+                        "The file id was deleted but its children were "
+                        "not deleted.")
 
     def _after_delta_check_parents(self, parents, index):
         """Check that parents required by the delta are all intact.
@@ -1705,17 +1842,15 @@
             # has the right file id.
             entry = self._get_entry(index, file_id, dirname_utf8)
             if entry[1] is None:
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(dirname_utf8.decode('utf8'),
+                self._raise_invalid(dirname_utf8.decode('utf8'),
                     file_id, "This parent is not present.")
             # Parents of things must be directories
             if entry[1][index][0] != 'd':
-                self._changes_aborted = True
-                raise errors.InconsistentDelta(dirname_utf8.decode('utf8'),
+                self._raise_invalid(dirname_utf8.decode('utf8'),
                     file_id, "This parent is not a directory.")
 
     def _observed_sha1(self, entry, sha1, stat_value,
-        _stat_to_minikind=_stat_to_minikind, _pack_stat=pack_stat):
+        _stat_to_minikind=_stat_to_minikind):
         """Note the sha1 of a file.
 
         :param entry: The entry the sha1 is for.
@@ -1727,15 +1862,14 @@
         except KeyError:
             # Unhandled kind
             return None
-        packed_stat = _pack_stat(stat_value)
         if minikind == 'f':
             if self._cutoff_time is None:
                 self._sha_cutoff_time()
             if (stat_value.st_mtime < self._cutoff_time
                 and stat_value.st_ctime < self._cutoff_time):
-                entry[1][0] = ('f', sha1, entry[1][0][2], entry[1][0][3],
-                    packed_stat)
-                self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+                entry[1][0] = ('f', sha1, stat_value.st_size, entry[1][0][3],
+                               pack_stat(stat_value))
+                self._mark_modified([entry])
 
     def _sha_cutoff_time(self):
         """Return cutoff time.
@@ -1799,14 +1933,13 @@
         """Serialise the entire dirstate to a sequence of lines."""
         if (self._header_state == DirState.IN_MEMORY_UNMODIFIED and
             self._dirblock_state == DirState.IN_MEMORY_UNMODIFIED):
-            # read whats on disk.
+            # read what's on disk.
             self._state_file.seek(0)
             return self._state_file.readlines()
         lines = []
         lines.append(self._get_parents_line(self.get_parent_ids()))
         lines.append(self._get_ghosts_line(self._ghosts))
-        # append the root line which is special cased
-        lines.extend(map(self._entry_to_line, self._iter_entries()))
+        lines.extend(self._get_entry_lines())
         return self._get_output_lines(lines)
 
     def _get_ghosts_line(self, ghost_ids):
@@ -1817,6 +1950,10 @@
         """Create a line for the state file for parents information."""
         return '\0'.join([str(len(parent_ids))] + parent_ids)
 
+    def _get_entry_lines(self):
+        """Create lines for entries."""
+        return map(self._entry_to_line, self._iter_entries())
+
     def _get_fields_to_entry(self):
         """Get a function which converts entry fields into a entry record.
 
@@ -1940,7 +2077,8 @@
             entry_index += 1
         return block_index, entry_index, True, False
 
-    def _get_entry(self, tree_index, fileid_utf8=None, path_utf8=None, include_deleted=False):
+    def _get_entry(self, tree_index, fileid_utf8=None, path_utf8=None,
+                   include_deleted=False):
         """Get the dirstate entry for path in tree tree_index.
 
         If either file_id or path is supplied, it is used as the key to lookup.
@@ -1980,7 +2118,7 @@
                                           ' tree_index, file_id and path')
             return entry
         else:
-            possible_keys = self._get_id_index().get(fileid_utf8, None)
+            possible_keys = self._get_id_index().get(fileid_utf8, ())
             if not possible_keys:
                 return None, None
             for key in possible_keys:
@@ -2088,7 +2226,8 @@
             executable = False
         else:
             raise Exception("can't pack %s" % inv_entry)
-        return (minikind, fingerprint, size, executable, tree_data)
+        return static_tuple.StaticTuple(minikind, fingerprint, size,
+                                        executable, tree_data)
 
     def _iter_child_entries(self, tree_index, path_utf8):
         """Iterate over all the entries that are children of path_utf.
@@ -2143,14 +2282,47 @@
                 yield entry
 
     def _get_id_index(self):
-        """Get an id index of self._dirblocks."""
+        """Get an id index of self._dirblocks.
+
+        This maps from file_id => [(directory, name, file_id)] entries where
+        that file_id appears in one of the trees.
+        """
         if self._id_index is None:
             id_index = {}
             for key, tree_details in self._iter_entries():
-                id_index.setdefault(key[2], set()).add(key)
+                self._add_to_id_index(id_index, key)
             self._id_index = id_index
         return self._id_index
 
+    def _add_to_id_index(self, id_index, entry_key):
+        """Add this entry to the _id_index mapping."""
+        # This code used to use a set for every entry in the id_index. However,
+        # it is *rare* to have more than one entry. So a set is a large
+        # overkill. And even when we do, we won't ever have more than the
+        # number of parent trees. Which is still a small number (rarely >2). As
+        # such, we use a simple tuple, and do our own uniqueness checks. While
+        # the 'in' check is O(N) since N is nicely bounded it shouldn't ever
+        # cause quadratic failure.
+        file_id = entry_key[2]
+        entry_key = static_tuple.StaticTuple.from_sequence(entry_key)
+        if file_id not in id_index:
+            id_index[file_id] = static_tuple.StaticTuple(entry_key,)
+        else:
+            entry_keys = id_index[file_id]
+            if entry_key not in entry_keys:
+                id_index[file_id] = entry_keys + (entry_key,)
+
+    def _remove_from_id_index(self, id_index, entry_key):
+        """Remove this entry from the _id_index mapping.
+
+        It is an programming error to call this when the entry_key is not
+        already present.
+        """
+        file_id = entry_key[2]
+        entry_keys = list(id_index[file_id])
+        entry_keys.remove(entry_key)
+        id_index[file_id] = static_tuple.StaticTuple.from_sequence(entry_keys)
+
     def _get_output_lines(self, lines):
         """Format lines for final output.
 
@@ -2176,18 +2348,22 @@
         """The number of parent entries in each record row."""
         return len(self._parents) - len(self._ghosts)
 
-    @staticmethod
-    def on_file(path, sha1_provider=None):
+    @classmethod
+    def on_file(cls, path, sha1_provider=None, worth_saving_limit=0):
         """Construct a DirState on the file at path "path".
 
         :param path: The path at which the dirstate file on disk should live.
         :param sha1_provider: an object meeting the SHA1Provider interface.
             If None, a DefaultSHA1Provider is used.
+        :param worth_saving_limit: when the exact number of hash changed
+            entries is known, only bother saving the dirstate if more than
+            this count of entries have changed. -1 means never save.
         :return: An unlocked DirState object, associated with the given path.
         """
         if sha1_provider is None:
             sha1_provider = DefaultSHA1Provider()
-        result = DirState(path, sha1_provider)
+        result = cls(path, sha1_provider,
+                     worth_saving_limit=worth_saving_limit)
         return result
 
     def _read_dirblocks_if_needed(self):
@@ -2251,9 +2427,9 @@
             raise errors.BzrError('missing num_entries line')
         self._num_entries = int(num_entries_line[len('num_entries: '):-1])
 
-    def sha1_from_stat(self, path, stat_result, _pack_stat=pack_stat):
+    def sha1_from_stat(self, path, stat_result):
         """Find a sha1 given a stat lookup."""
-        return self._get_packed_stat_index().get(_pack_stat(stat_result), None)
+        return self._get_packed_stat_index().get(pack_stat(stat_result), None)
 
     def _get_packed_stat_index(self):
         """Get a packed_stat index of self._dirblocks."""
@@ -2285,35 +2461,65 @@
             trace.mutter('Not saving DirState because '
                     '_changes_aborted is set.')
             return
-        if (self._header_state == DirState.IN_MEMORY_MODIFIED or
-            self._dirblock_state == DirState.IN_MEMORY_MODIFIED):
+        # TODO: Since we now distinguish IN_MEMORY_MODIFIED from
+        #       IN_MEMORY_HASH_MODIFIED, we should only fail quietly if we fail
+        #       to save an IN_MEMORY_HASH_MODIFIED, and fail *noisily* if we
+        #       fail to save IN_MEMORY_MODIFIED
+        if not self._worth_saving():
+            return
 
-            grabbed_write_lock = False
-            if self._lock_state != 'w':
-                grabbed_write_lock, new_lock = self._lock_token.temporary_write_lock()
-                # Switch over to the new lock, as the old one may be closed.
+        grabbed_write_lock = False
+        if self._lock_state != 'w':
+            grabbed_write_lock, new_lock = self._lock_token.temporary_write_lock()
+            # Switch over to the new lock, as the old one may be closed.
+            # TODO: jam 20070315 We should validate the disk file has
+            #       not changed contents, since temporary_write_lock may
+            #       not be an atomic operation.
+            self._lock_token = new_lock
+            self._state_file = new_lock.f
+            if not grabbed_write_lock:
+                # We couldn't grab a write lock, so we switch back to a read one
+                return
+        try:
+            lines = self.get_lines()
+            self._state_file.seek(0)
+            self._state_file.writelines(lines)
+            self._state_file.truncate()
+            self._state_file.flush()
+            self._maybe_fdatasync()
+            self._mark_unmodified()
+        finally:
+            if grabbed_write_lock:
+                self._lock_token = self._lock_token.restore_read_lock()
+                self._state_file = self._lock_token.f
                 # TODO: jam 20070315 We should validate the disk file has
-                #       not changed contents. Since temporary_write_lock may
-                #       not be an atomic operation.
-                self._lock_token = new_lock
-                self._state_file = new_lock.f
-                if not grabbed_write_lock:
-                    # We couldn't grab a write lock, so we switch back to a read one
-                    return
-            try:
-                self._state_file.seek(0)
-                self._state_file.writelines(self.get_lines())
-                self._state_file.truncate()
-                self._state_file.flush()
-                self._header_state = DirState.IN_MEMORY_UNMODIFIED
-                self._dirblock_state = DirState.IN_MEMORY_UNMODIFIED
-            finally:
-                if grabbed_write_lock:
-                    self._lock_token = self._lock_token.restore_read_lock()
-                    self._state_file = self._lock_token.f
-                    # TODO: jam 20070315 We should validate the disk file has
-                    #       not changed contents. Since restore_read_lock may
-                    #       not be an atomic operation.
+                #       not changed contents. Since restore_read_lock may
+                #       not be an atomic operation.                
+
+    def _maybe_fdatasync(self):
+        """Flush to disk if possible and if not configured off."""
+        if self._config_stack.get('dirstate.fdatasync'):
+            osutils.fdatasync(self._state_file.fileno())
+
+    def _worth_saving(self):
+        """Is it worth saving the dirstate or not?"""
+        if (self._header_state == DirState.IN_MEMORY_MODIFIED
+            or self._dirblock_state == DirState.IN_MEMORY_MODIFIED):
+            return True
+        if self._dirblock_state == DirState.IN_MEMORY_HASH_MODIFIED:
+            if self._worth_saving_limit == -1:
+                # We never save hash changes when the limit is -1
+                return False
+            # If we're using smart saving and only a small number of
+            # entries have changed their hash, don't bother saving. John has
+            # suggested using a heuristic here based on the size of the
+            # changed files and/or tree. For now, we go with a configurable
+            # number of changes, keeping the calculation time
+            # as low overhead as possible. (This also keeps all existing
+            # tests passing as the default is 0, i.e. always save.)
+            if len(self._known_hash_changes) >= self._worth_saving_limit:
+                return True
+        return False
 
     def _set_data(self, parent_ids, dirblocks):
         """Set the full dirstate data in memory.
@@ -2328,8 +2534,7 @@
         """
         # our memory copy is now authoritative.
         self._dirblocks = dirblocks
-        self._header_state = DirState.IN_MEMORY_MODIFIED
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified(header_modified=True)
         self._parents = list(parent_ids)
         self._id_index = None
         self._packed_stat_index = None
@@ -2355,7 +2560,14 @@
         self._make_absent(entry)
         self.update_minimal(('', '', new_id), 'd',
             path_utf8='', packed_stat=entry[1][0][4])
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified()
+        # XXX: This was added by Ian, we need to make sure there
+        #      are tests for it, because it isn't in bzr.dev TRUNK
+        #      It looks like the only place it is called is in setting the root
+        #      id of the tree. So probably we never had an _id_index when we
+        #      don't even have a root yet.
+        if self._id_index is not None:
+            self._add_to_id_index(self._id_index, entry[0])
 
     def set_parent_trees(self, trees, ghosts):
         """Set the parent trees for the dirstate.
@@ -2406,6 +2618,7 @@
         parent_trees = [tree for rev_id, tree in trees if rev_id not in ghosts]
         # how many trees do we end up with
         parent_count = len(parent_trees)
+        st = static_tuple.StaticTuple
 
         # one: the current tree
         for entry in self._iter_entries():
@@ -2414,7 +2627,9 @@
                 continue
             by_path[entry[0]] = [entry[1][0]] + \
                 [DirState.NULL_PARENT_DETAILS] * parent_count
-            id_index[entry[0][2]] = set([entry[0]])
+            # TODO: Possibly inline this, since we know it isn't present yet
+            #       id_index[entry[0][2]] = (entry[0],)
+            self._add_to_id_index(id_index, entry[0])
 
         # now the parent trees:
         for tree_index, tree in enumerate(parent_trees):
@@ -2426,7 +2641,8 @@
             # the suffix is from tree_index+1:parent_count+1.
             new_location_suffix = [DirState.NULL_PARENT_DETAILS] * (parent_count - tree_index)
             # now stitch in all the entries from this tree
-            for path, entry in tree.inventory.iter_entries_by_dir():
+            last_dirname = None
+            for path, entry in tree.iter_entries_by_dir():
                 # here we process each trees details for each item in the tree.
                 # we first update any existing entries for the id at other paths,
                 # then we either create or update the entry for the id at the
@@ -2439,10 +2655,16 @@
                 file_id = entry.file_id
                 path_utf8 = path.encode('utf8')
                 dirname, basename = osutils.split(path_utf8)
-                new_entry_key = (dirname, basename, file_id)
+                if dirname == last_dirname:
+                    # Try to re-use objects as much as possible
+                    dirname = last_dirname
+                else:
+                    last_dirname = dirname
+                new_entry_key = st(dirname, basename, file_id)
                 # tree index consistency: All other paths for this id in this tree
                 # index must point to the correct path.
-                for entry_key in id_index.setdefault(file_id, set()):
+                entry_keys = id_index.get(file_id, ())
+                for entry_key in entry_keys:
                     # TODO:PROFILING: It might be faster to just update
                     # rather than checking if we need to, and then overwrite
                     # the one we are located at.
@@ -2451,11 +2673,14 @@
                         # other trees, so put absent pointers there
                         # This is the vertical axis in the matrix, all pointing
                         # to the real path.
-                        by_path[entry_key][tree_index] = ('r', path_utf8, 0, False, '')
-                # by path consistency: Insert into an existing path record (trivial), or
-                # add a new one with relocation pointers for the other tree indexes.
-                if new_entry_key in id_index[file_id]:
-                    # there is already an entry where this data belongs, just insert it.
+                        by_path[entry_key][tree_index] = st('r', path_utf8, 0,
+                                                            False, '')
+                # by path consistency: Insert into an existing path record
+                # (trivial), or add a new one with relocation pointers for the
+                # other tree indexes.
+                if new_entry_key in entry_keys:
+                    # there is already an entry where this data belongs, just
+                    # insert it.
                     by_path[new_entry_key][tree_index] = \
                         self._inv_entry_to_details(entry)
                 else:
@@ -2465,27 +2690,26 @@
                     new_details = []
                     for lookup_index in xrange(tree_index):
                         # boundary case: this is the first occurence of file_id
-                        # so there are no id_indexs, possibly take this out of
+                        # so there are no id_indexes, possibly take this out of
                         # the loop?
-                        if not len(id_index[file_id]):
+                        if not len(entry_keys):
                             new_details.append(DirState.NULL_PARENT_DETAILS)
                         else:
                             # grab any one entry, use it to find the right path.
-                            # TODO: optimise this to reduce memory use in highly
-                            # fragmented situations by reusing the relocation
-                            # records.
-                            a_key = iter(id_index[file_id]).next()
+                            a_key = iter(entry_keys).next()
                             if by_path[a_key][lookup_index][0] in ('r', 'a'):
-                                # its a pointer or missing statement, use it as is.
+                                # its a pointer or missing statement, use it as
+                                # is.
                                 new_details.append(by_path[a_key][lookup_index])
                             else:
                                 # we have the right key, make a pointer to it.
                                 real_path = ('/'.join(a_key[0:2])).strip('/')
-                                new_details.append(('r', real_path, 0, False, ''))
+                                new_details.append(st('r', real_path, 0, False,
+                                                      ''))
                     new_details.append(self._inv_entry_to_details(entry))
                     new_details.extend(new_location_suffix)
                     by_path[new_entry_key] = new_details
-                    id_index[file_id].add(new_entry_key)
+                    self._add_to_id_index(id_index, new_entry_key)
         # --- end generation of full tree mappings
 
         # sort and output all the entries
@@ -2493,8 +2717,7 @@
         self._entries_to_current_state(new_entries)
         self._parents = [rev_id for rev_id, tree in trees]
         self._ghosts = list(ghosts)
-        self._header_state = DirState.IN_MEMORY_MODIFIED
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified(header_modified=True)
         self._id_index = id_index
 
     def _sort_entries(self, entry_list):
@@ -2504,9 +2727,20 @@
         try to keep everything in sorted blocks all the time, but sometimes
         it's easier to sort after the fact.
         """
-        def _key(entry):
+        # When sorting, we usually have 10x more entries than directories. (69k
+        # total entries, 4k directories). So cache the results of splitting.
+        # Saving time and objects. Also, use StaticTuple to avoid putting all
+        # of these object into python's garbage collector.
+        split_dirs = {}
+        def _key(entry, _split_dirs=split_dirs, _st=static_tuple.StaticTuple):
             # sort by: directory parts, file name, file id
-            return entry[0][0].split('/'), entry[0][1], entry[0][2]
+            dirpath, fname, file_id = entry[0]
+            try:
+                split = _split_dirs[dirpath]
+            except KeyError:
+                split = _st.from_sequence(dirpath.split('/'))
+                _split_dirs[dirpath] = split
+            return _st(split, fname, file_id)
         return sorted(entry_list, key=_key)
 
     def set_state_from_inventory(self, new_inv):
@@ -2637,12 +2871,29 @@
                         current_old[0][1].decode('utf8'))
                 self._make_absent(current_old)
                 current_old = advance(old_iterator)
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified()
         self._id_index = None
         self._packed_stat_index = None
         if tracing:
             trace.mutter("set_state_from_inventory complete.")
 
+    def set_state_from_scratch(self, working_inv, parent_trees, parent_ghosts):
+        """Wipe the currently stored state and set it to something new.
+
+        This is a hard-reset for the data we are working with.
+        """
+        # Technically, we really want a write lock, but until we write, we
+        # don't really need it.
+        self._requires_lock()
+        # root dir and root dir contents with no children. We have to have a
+        # root for set_state_from_inventory to work correctly.
+        empty_root = (('', '', inventory.ROOT_ID),
+                      [('d', '', 0, False, DirState.NULLSTAT)])
+        empty_tree_dirblocks = [('', [empty_root]), ('', [])]
+        self._set_data([], empty_tree_dirblocks)
+        self.set_state_from_inventory(working_inv)
+        self.set_parent_trees(parent_trees, parent_ghosts)
+
     def _make_absent(self, current_old):
         """Mark current_old - an entry - as absent for tree 0.
 
@@ -2673,7 +2924,7 @@
             block[1].pop(entry_index)
             # if we have an id_index in use, remove this key from it for this id.
             if self._id_index is not None:
-                self._id_index[current_old[0][2]].remove(current_old[0])
+                self._remove_from_id_index(self._id_index, current_old[0])
         # update all remaining keys for this id to record it as absent. The
         # existing details may either be the record we are marking as deleted
         # (if there were other trees with the id present at this path), or may
@@ -2692,7 +2943,7 @@
             if update_tree_details[0][0] == 'a': # absent
                 raise AssertionError('bad row %r' % (update_tree_details,))
             update_tree_details[0] = DirState.NULL_PARENT_DETAILS
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified()
         return last_reference
 
     def update_minimal(self, key, minikind, executable=False, fingerprint='',
@@ -2740,7 +2991,7 @@
                             # This entry has the same path (but a different id) as
                             # the new entry we're adding, and is present in ths
                             # tree.
-                            raise errors.InconsistentDelta(
+                            self._raise_invalid(
                                 ("%s/%s" % key[0:2]).decode('utf8'), key[2],
                                 "Attempt to add item at path already occupied by "
                                 "id %r" % entry[0][2])
@@ -2748,7 +2999,7 @@
                     else:
                         break
             # new entry, synthesis cross reference here,
-            existing_keys = id_index.setdefault(key[2], set())
+            existing_keys = id_index.get(key[2], ())
             if not existing_keys:
                 # not currently in the state, simplest case
                 new_entry = key, [new_details] + self._empty_parent_info()
@@ -2785,8 +3036,11 @@
                     # loop.
                     other_entry = other_block[other_entry_index]
                     other_entry[1][0] = ('r', path_utf8, 0, False, '')
-                    self._maybe_remove_row(other_block, other_entry_index,
-                        id_index)
+                    if self._maybe_remove_row(other_block, other_entry_index,
+                                              id_index):
+                        # If the row holding this was removed, we need to
+                        # recompute where this entry goes
+                        entry_index, _ = self._find_entry_index(key, block)
 
                 # This loop:
                 # adds a tuple to the new details for each column
@@ -2794,6 +3048,8 @@
                 #  - or by creating a new pointer to the right row inside that column
                 num_present_parents = self._num_present_parents()
                 if num_present_parents:
+                    # TODO: This re-evaluates the existing_keys set, do we need
+                    #       to do that ourselves?
                     other_key = list(existing_keys)[0]
                 for lookup_index in xrange(1, num_present_parents + 1):
                     # grab any one entry, use it to find the right path.
@@ -2818,7 +3074,7 @@
                         pointer_path = osutils.pathjoin(*other_key[0:2])
                         new_entry[1].append(('r', pointer_path, 0, False, ''))
             block.insert(entry_index, new_entry)
-            existing_keys.add(key)
+            self._add_to_id_index(id_index, key)
         else:
             # Does the new state matter?
             block[entry_index][1][0] = new_details
@@ -2833,7 +3089,12 @@
             # converted to relocated.
             if path_utf8 is None:
                 raise AssertionError('no path')
-            for entry_key in id_index.setdefault(key[2], set()):
+            existing_keys = id_index.get(key[2], ())
+            if key not in existing_keys:
+                raise AssertionError('We found the entry in the blocks, but'
+                    ' the key is not in the id_index.'
+                    ' key: %s, existing_keys: %s' % (key, existing_keys))
+            for entry_key in existing_keys:
                 # TODO:PROFILING: It might be faster to just update
                 # rather than checking if we need to, and then overwrite
                 # the one we are located at.
@@ -2857,12 +3118,13 @@
             if not present:
                 self._dirblocks.insert(block_index, (subdir_key[0], []))
 
-        self._dirblock_state = DirState.IN_MEMORY_MODIFIED
+        self._mark_modified()
 
     def _maybe_remove_row(self, block, index, id_index):
         """Remove index if it is absent or relocated across the row.
         
         id_index is updated accordingly.
+        :return: True if we removed the row, False otherwise
         """
         present_in_row = False
         entry = block[index]
@@ -2872,7 +3134,9 @@
                 break
         if not present_in_row:
             block.pop(index)
-            id_index[entry[0][2]].remove(entry[0])
+            self._remove_from_id_index(id_index, entry[0])
+            return True
+        return False
 
     def _validate(self):
         """Check that invariants on the dirblock are correct.
@@ -3020,6 +3284,10 @@
                         raise AssertionError(
                             'file_id %r did not match entry key %s'
                             % (file_id, entry_key))
+                if len(entry_keys) != len(set(entry_keys)):
+                    raise AssertionError(
+                        'id_index contained non-unique data for %s'
+                        % (entry_keys,))
 
     def _wipe_state(self):
         """Forget all state information about the dirstate."""
@@ -3082,8 +3350,7 @@
 
 
 def py_update_entry(state, entry, abspath, stat_value,
-                 _stat_to_minikind=DirState._stat_to_minikind,
-                 _pack_stat=pack_stat):
+                 _stat_to_minikind=DirState._stat_to_minikind):
     """Update the entry based on what is actually on disk.
 
     This function only calculates the sha if it needs to - if the entry is
@@ -3102,7 +3369,7 @@
     except KeyError:
         # Unhandled kind
         return None
-    packed_stat = _pack_stat(stat_value)
+    packed_stat = pack_stat(stat_value)
     (saved_minikind, saved_link_or_sha1, saved_file_size,
      saved_executable, saved_packed_stat) = entry[1][0]
 
@@ -3122,6 +3389,7 @@
     # If we have gotten this far, that means that we need to actually
     # process this entry.
     link_or_sha1 = None
+    worth_saving = True
     if minikind == 'f':
         executable = state._is_executable(stat_value.st_mode,
                                          saved_executable)
@@ -3143,6 +3411,7 @@
         else:
             entry[1][0] = ('f', '', stat_value.st_size,
                            executable, DirState.NULLSTAT)
+            worth_saving = False
     elif minikind == 'd':
         link_or_sha1 = None
         entry[1][0] = ('d', '', 0, False, packed_stat)
@@ -3154,7 +3423,11 @@
                 state._get_block_entry_index(entry[0][0], entry[0][1], 0)
             state._ensure_block(block_index, entry_index,
                                osutils.pathjoin(entry[0][0], entry[0][1]))
+        else:
+            worth_saving = False
     elif minikind == 'l':
+        if saved_minikind == 'l':
+            worth_saving = False
         link_or_sha1 = state._read_link(abspath, saved_link_or_sha1)
         if state._cutoff_time is None:
             state._sha_cutoff_time()
@@ -3165,7 +3438,8 @@
         else:
             entry[1][0] = ('l', '', stat_value.st_size,
                            False, DirState.NULLSTAT)
-    state._dirblock_state = DirState.IN_MEMORY_MODIFIED
+    if worth_saving:
+        state._mark_modified([entry])
     return link_or_sha1
 
 
@@ -3480,7 +3754,6 @@
             raise AssertionError("don't know how to compare "
                 "source_minikind=%r, target_minikind=%r"
                 % (source_minikind, target_minikind))
-            ## import pdb;pdb.set_trace()
         return None, None
 
     def __iter__(self):
@@ -3975,6 +4248,7 @@
         _bisect_path_left,
         _bisect_path_right,
         cmp_by_dirs,
+        pack_stat,
         ProcessEntryC as _process_entry,
         update_entry as update_entry,
         )
@@ -3986,6 +4260,7 @@
         _bisect_path_left,
         _bisect_path_right,
         cmp_by_dirs,
+        pack_stat,
         )
     # FIXME: It would be nice to be able to track moved lines so that the
     # corresponding python code can be moved to the _dirstate_helpers_py

=== modified file 'bzrlib/doc/api/__init__.py'
--- bzrlib/doc/api/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/doc/api/__init__.py	2011-05-18 16:42:48 +0000
@@ -18,7 +18,7 @@
 
 This documentation is made up of doctest testable examples.
 
-Look for bzrlib/doc/api/*.txt to read it.
+Look for `bzrlib/doc/api/*.txt` to read it.
 
 This documentation documents the current best practice in using the library.
 For details on specific apis, see pydoc on the api, or read the source.

=== modified file 'bzrlib/doc/api/branch.txt'
--- bzrlib/doc/api/branch.txt	2006-10-05 22:42:09 +0000
+++ bzrlib/doc/api/branch.txt	2011-10-05 18:11:19 +0000
@@ -25,8 +25,8 @@
 
 A branch has a history of revisions on it:
 
-   >>> new_branch.revision_history()
-   []
+   >>> new_branch.last_revision()
+   'null:'
 
 We need to write some more documentation, showing
 push and pull operations. Cloning might also be nice.

=== modified file 'bzrlib/doc/api/transport.txt'
--- bzrlib/doc/api/transport.txt	2006-06-30 21:22:07 +0000
+++ bzrlib/doc/api/transport.txt	2011-08-19 22:34:02 +0000
@@ -6,14 +6,14 @@
    >>> from bzrlib.urlutils import local_path_from_url
    >>> import bzrlib.transport as transport
    >>> if sys.platform == 'win32':
-   ...     root = transport.get_transport('file:///C:/')
+   ...     root = transport.get_transport_from_url('file:///C:/')
    ... else:
-   ...     root = transport.get_transport('file:///')
+   ...     root = transport.get_transport_from_url('file:///')
    >>>
 
 Each Transport instance represents a single logical directory.
 
-   >>> dir = transport.get_transport(".")
+   >>> dir = transport.get_transport_from_path(".")
    >>> local_path_from_url(dir.base) == getcwd() + '/'
    True
 

=== modified file 'bzrlib/doc_generate/autodoc_man.py'
--- bzrlib/doc_generate/autodoc_man.py	2010-06-02 04:50:52 +0000
+++ bzrlib/doc_generate/autodoc_man.py	2011-07-22 00:03:11 +0000
@@ -186,10 +186,14 @@
 .B "help"
 .I "command"
 .SH "DESCRIPTION"
-Bazaar (or %(bzrcmd)s) is a project of Canonical to develop an free
-distributed version control system that is powerful, friendly, and scalable.
-Version control means a system that keeps track of previous revisions
-of software source code or similar information and helps people work on it in teams.
+
+Bazaar (or %(bzrcmd)s) is a distributed version control system that is powerful, 
+friendly, and scalable.  Bazaar is a project of Canonical Ltd and part of 
+the GNU Project to develop a free operating system.
+
+Bazaar keeps track of changes to software source code (or similar information);
+lets you explore who changed it, when, and why; merges concurrent changes; and
+helps people work together in a team.
 """
 
 man_foot = """\
@@ -241,7 +245,7 @@
 .br
 log10 = log --short -r -10..-1
 .SH "SEE ALSO"
-.UR http://www.bazaar.canonical.com/
-.BR http://www.bazaar.canonical.com/
+.UR http://bazaar.canonical.com/
+.BR http://bazaar.canonical.com/
 """
 

=== modified file 'bzrlib/doc_generate/conf.py'
--- bzrlib/doc_generate/conf.py	2010-07-07 11:32:03 +0000
+++ bzrlib/doc_generate/conf.py	2011-05-16 10:19:33 +0000
@@ -38,7 +38,7 @@
 
 # General information about the project.
 project = u'Bazaar'
-copyright = u'2009, 2010 Canonical Ltd'
+copyright = u'2009-2011 Canonical Ltd'
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the

=== modified file 'bzrlib/doc_generate/writers/texinfo.py'
--- bzrlib/doc_generate/writers/texinfo.py	2010-07-21 08:07:14 +0000
+++ bzrlib/doc_generate/writers/texinfo.py	2011-03-11 12:24:10 +0000
@@ -123,6 +123,17 @@
     def depart_topic(self, node):
         pass
 
+    def visit_compound(self, node):
+        # compound is new in sphinx >= 1.0 and just add a optional layer so we
+        # relay the text to the parent when it occurs. This may requires a
+        # cleaner approach once we settle on which sphinx versions we want to
+        # support.
+        set_item_list_collector(node, 'text')
+
+    def depart_compound(self, node):
+        text = ''.join(node['text'])
+        node.parent.collect_text(text)
+
     def visit_paragraph(self, node):
         set_item_list_collector(node, 'text')
 
@@ -143,9 +154,10 @@
         # FIXME: Using a different visitor specific to toctree may be a better
         # design and makes code clearer. -- vila 20100708
         if node.has_key('toctree'):
-            node.parent.collect_text('@menu\n')
-            node.parent.collect_text(''.join(node['text']))
-            node.parent.collect_text('@end menu\n')
+            if node['text']:
+                node.parent.collect_text('@menu\n')
+                node.parent.collect_text(''.join(node['text']))
+                node.parent.collect_text('@end menu\n')
             self.in_toctree = False
         elif self.in_toctree:
             # * FIRST-ENTRY-NAME:(FILENAME)NODENAME.     DESCRIPTION

=== modified file 'bzrlib/email_message.py'
--- bzrlib/email_message.py	2009-03-23 14:59:43 +0000
+++ bzrlib/email_message.py	2011-05-19 09:32:38 +0000
@@ -181,7 +181,7 @@
     def string_with_encoding(string_):
         """Return a str object together with an encoding.
 
-        :param string_: A str or unicode object.
+        :param string\\_: A str or unicode object.
         :return: A tuple (str, encoding), where encoding is one of 'ascii',
             'utf-8', or '8-bit', in that preferred order.
         """

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2010-07-29 11:17:57 +0000
+++ bzrlib/errors.py	2011-10-12 16:00:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,10 @@
 from bzrlib import (
     osutils,
     symbol_versioning,
+    i18n,
+    trace,
     )
+from bzrlib.i18n import gettext
 from bzrlib.patches import (
     MalformedHunkHeader,
     MalformedLine,
@@ -54,12 +57,12 @@
     Base class for errors raised by bzrlib.
 
     :cvar internal_error: if True this was probably caused by a bzr bug and
-    should be displayed with a traceback; if False (or absent) this was
-    probably a user or environment error and they don't need the gory details.
-    (That can be overridden by -Derror on the command line.)
+        should be displayed with a traceback; if False (or absent) this was
+        probably a user or environment error and they don't need the gory
+        details.  (That can be overridden by -Derror on the command line.)
 
     :cvar _fmt: Format string to display the error; this is expanded
-    by the instance's dict.
+        by the instance's dict.
     """
 
     internal_error = False
@@ -140,7 +143,11 @@
         """Return format string for this exception or None"""
         fmt = getattr(self, '_fmt', None)
         if fmt is not None:
-            return fmt
+            i18n.install()
+            unicode_fmt = unicode(fmt) #_fmt strings should be ascii
+            if type(fmt) == unicode:
+                trace.mutter("Unicode strings in error.fmt are deprecated")
+            return gettext(unicode_fmt)
         fmt = getattr(self, '__doc__', None)
         if fmt is not None:
             symbol_versioning.warn("%s uses its docstring as a format, "
@@ -304,7 +311,7 @@
 class RootMissing(InternalBzrError):
 
     _fmt = ("The root entry of a tree must be the first entry supplied to "
-        "record_entry_contents.")
+        "the commit builder.")
 
 
 class NoPublicBranch(BzrError):
@@ -621,7 +628,7 @@
 
     _fmt = 'Unsupported protocol for url "%(path)s"%(extra)s'
 
-    def __init__(self, url, extra):
+    def __init__(self, url, extra=""):
         PathError.__init__(self, url, extra=extra)
 
 
@@ -713,6 +720,9 @@
        self.bzrdir = bzrdir
        PathError.__init__(self, path=path)
 
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.__dict__)
+
     def _format(self):
         # XXX: Ideally self.detail would be a property, but Exceptions in
         # Python 2.4 have to be old-style classes so properties don't work.
@@ -723,6 +733,15 @@
                     self.bzrdir.open_repository()
                 except NoRepositoryPresent:
                     self.detail = ''
+                except Exception:
+                    # Just ignore unexpected errors.  Raising arbitrary errors
+                    # during str(err) can provoke strange bugs.  Concretely
+                    # Launchpad's codehosting managed to raise NotBranchError
+                    # here, and then get stuck in an infinite loop/recursion
+                    # trying to str() that error.  All this error really cares
+                    # about that there's no working repository there, and if
+                    # open_repository() fails, there probably isn't.
+                    self.detail = ''
                 else:
                     self.detail = ': location is a repository'
             else:
@@ -852,9 +871,9 @@
         """Construct a new AlreadyVersionedError.
 
         :param path: This is the path which is versioned,
-        which should be in a user friendly form.
+            which should be in a user friendly form.
         :param context_info: If given, this is information about the context,
-        which could explain why this is expected to not be versioned.
+            which could explain why this is expected to not be versioned.
         """
         BzrError.__init__(self)
         self.path = path
@@ -873,9 +892,9 @@
         """Construct a new NotVersionedError.
 
         :param path: This is the path which is not versioned,
-        which should be in a user friendly form.
+            which should be in a user friendly form.
         :param context_info: If given, this is information about the context,
-        which could explain why this is expected to be versioned.
+            which could explain why this is expected to be versioned.
         """
         BzrError.__init__(self)
         self.path = path
@@ -1072,6 +1091,18 @@
         self.target = target
 
 
+class LockCorrupt(LockError):
+
+    _fmt = ("Lock is apparently held, but corrupted: %(corruption_info)s\n"
+            "Use 'bzr break-lock' to clear it")
+
+    internal_error = False
+
+    def __init__(self, corruption_info, file_data=None):
+        self.corruption_info = corruption_info
+        self.file_data = file_data
+
+
 class LockNotHeld(LockError):
 
     _fmt = "Lock not held: %(lock)s"
@@ -1116,6 +1147,15 @@
         BzrError.__init__(self, files=files, files_str=files_str)
 
 
+class ExcludesUnsupported(BzrError):
+
+    _fmt = ('Excluding paths during commit is not supported by '
+            'repository at %(repository)r.')
+
+    def __init__(self, repository):
+        BzrError.__init__(self, repository=repository)
+
+
 class BadCommitMessageEncoding(BzrError):
 
     _fmt = 'The specified commit message contains characters unsupported by '\
@@ -1383,7 +1423,7 @@
 
 class WeaveInvalidChecksum(WeaveError):
 
-    _fmt = "Text did not match it's checksum: %(msg)s"
+    _fmt = "Text did not match its checksum: %(msg)s"
 
 
 class WeaveTextDiffers(WeaveError):
@@ -1626,6 +1666,7 @@
 
     def __init__(self, exc_info):
         import traceback
+        # GZ 2010-08-10: Cycle with exc_tb/exc_info affects at least one test
         self.exc_type, self.exc_value, self.exc_tb = exc_info
         self.exc_info = exc_info
         traceback_strings = traceback.format_exception(
@@ -1670,6 +1711,11 @@
     _fmt = "Connection closed: %(msg)s %(orig_error)s"
 
 
+class ConnectionTimeout(ConnectionError):
+
+    _fmt = "Connection Timeout: %(msg)s%(orig_error)s"
+
+
 class InvalidRange(TransportError):
 
     _fmt = "Invalid range access in %(path)s at %(offset)s: %(msg)s"
@@ -1682,10 +1728,16 @@
 
 class InvalidHttpResponse(TransportError):
 
-    _fmt = "Invalid http response for %(path)s: %(msg)s"
+    _fmt = "Invalid http response for %(path)s: %(msg)s%(orig_error)s"
 
     def __init__(self, path, msg, orig_error=None):
         self.path = path
+        if orig_error is None:
+            orig_error = ''
+        else:
+            # This is reached for obscure and unusual errors so we want to
+            # preserve as much info as possible to ease debug.
+            orig_error = ': %r' % (orig_error,)
         TransportError.__init__(self, msg, orig_error=orig_error)
 
 
@@ -1698,6 +1750,19 @@
         InvalidHttpResponse.__init__(self, path, msg)
 
 
+class HttpBoundaryMissing(InvalidHttpResponse):
+    """A multipart response ends with no boundary marker.
+
+    This is a special case caused by buggy proxies, described in
+    <https://bugs.launchpad.net/bzr/+bug/198646>.
+    """
+
+    _fmt = "HTTP MIME Boundary missing for %(path)s: %(msg)s"
+
+    def __init__(self, path, msg):
+        InvalidHttpResponse.__init__(self, path, msg)
+
+
 class InvalidHttpContentType(InvalidHttpResponse):
 
     _fmt = 'Invalid http Content-type "%(ctype)s" for %(path)s: %(msg)s'
@@ -1731,14 +1796,31 @@
     _fmt = "Working tree has conflicts."
 
 
+class ConfigContentError(BzrError):
+
+    _fmt = "Config file %(filename)s is not UTF-8 encoded\n"
+
+    def __init__(self, filename):
+        BzrError.__init__(self)
+        self.filename = filename
+
+
 class ParseConfigError(BzrError):
 
+    _fmt = "Error(s) parsing config file %(filename)s:\n%(errors)s"
+
     def __init__(self, errors, filename):
-        if filename is None:
-            filename = ""
-        message = "Error(s) parsing config file %s:\n%s" % \
-            (filename, ('\n'.join(e.msg for e in errors)))
-        BzrError.__init__(self, message)
+        BzrError.__init__(self)
+        self.filename = filename
+        self.errors = '\n'.join(e.msg for e in errors)
+
+
+class ConfigOptionValueError(BzrError):
+
+    _fmt = """Bad value "%(value)s" for option "%(name)s"."""
+
+    def __init__(self, name, value):
+        BzrError.__init__(self, name=name, value=value)
 
 
 class NoEmailInUsername(BzrError):
@@ -1752,12 +1834,36 @@
 
 class SigningFailed(BzrError):
 
-    _fmt = 'Failed to gpg sign data with command "%(command_line)s"'
+    _fmt = 'Failed to GPG sign data with command "%(command_line)s"'
 
     def __init__(self, command_line):
         BzrError.__init__(self, command_line=command_line)
 
 
+class SignatureVerificationFailed(BzrError):
+
+    _fmt = 'Failed to verify GPG signature data with error "%(error)s"'
+
+    def __init__(self, error):
+        BzrError.__init__(self, error=error)
+
+
+class DependencyNotPresent(BzrError):
+
+    _fmt = 'Unable to import library "%(library)s": %(error)s'
+
+    def __init__(self, library, error):
+        BzrError.__init__(self, library=library, error=error)
+
+
+class GpgmeNotInstalled(DependencyNotPresent):
+
+    _fmt = 'python-gpgme is not installed, it is needed to verify signatures'
+
+    def __init__(self, error):
+        DependencyNotPresent.__init__(self, 'gpgme', error)
+
+
 class WorkingTreeNotRevision(BzrError):
 
     _fmt = ("The working tree for %(basedir)s has changed since"
@@ -1936,15 +2042,16 @@
 
 class BzrMoveFailedError(BzrError):
 
-    _fmt = "Could not move %(from_path)s%(operator)s %(to_path)s%(extra)s"
+    _fmt = ("Could not move %(from_path)s%(operator)s %(to_path)s"
+        "%(_has_extra)s%(extra)s")
 
     def __init__(self, from_path='', to_path='', extra=None):
         from bzrlib.osutils import splitpath
         BzrError.__init__(self)
         if extra:
-            self.extra = ': ' + str(extra)
+            self.extra, self._has_extra = extra, ': '
         else:
-            self.extra = ''
+            self.extra = self._has_extra = ''
 
         has_from = len(from_path) > 0
         has_to = len(to_path) > 0
@@ -1971,11 +2078,13 @@
 
 class BzrRenameFailedError(BzrMoveFailedError):
 
-    _fmt = "Could not rename %(from_path)s%(operator)s %(to_path)s%(extra)s"
+    _fmt = ("Could not rename %(from_path)s%(operator)s %(to_path)s"
+        "%(_has_extra)s%(extra)s")
 
     def __init__(self, from_path, to_path, extra=None):
         BzrMoveFailedError.__init__(self, from_path, to_path, extra)
 
+
 class BzrRemoveChangedFilesError(BzrError):
     """Used when user is trying to remove changed files."""
 
@@ -1984,6 +2093,8 @@
         "Use --keep to not delete them, or --force to delete them regardless.")
 
     def __init__(self, tree_delta):
+        symbol_versioning.warn(symbol_versioning.deprecated_in((2, 3, 0)) %
+            "BzrRemoveChangedFilesError", DeprecationWarning, stacklevel=2)
         BzrError.__init__(self)
         self.changes_as_text = tree_delta.get_changes_as_text()
         #self.paths_as_string = '\n'.join(changed_files)
@@ -1997,7 +2108,7 @@
 
 class BzrBadParameterMissing(BzrBadParameter):
 
-    _fmt = "Parameter $(param)s is required but not present."
+    _fmt = "Parameter %(param)s is required but not present."
 
 
 class BzrBadParameterUnicode(BzrBadParameter):
@@ -2011,14 +2122,6 @@
     _fmt = "Parameter %(param)s contains a newline."
 
 
-class DependencyNotPresent(BzrError):
-
-    _fmt = 'Unable to import library "%(library)s": %(error)s'
-
-    def __init__(self, library, error):
-        BzrError.__init__(self, library=library, error=error)
-
-
 class ParamikoNotPresent(DependencyNotPresent):
 
     _fmt = "Unable to import paramiko (required for sftp support): %(error)s"
@@ -2259,6 +2362,14 @@
     """
 
 
+class GhostTagsNotSupported(BzrError):
+
+    _fmt = "Ghost tags not supported by format %(format)r."
+
+    def __init__(self, format):
+        self.format = format
+
+
 class BinaryFile(BzrError):
 
     _fmt = "File is binary but should be text."
@@ -2627,11 +2738,12 @@
 
     This is distinct from ErrorFromSmartServer so that it is possible to
     distinguish between the following two cases:
-      - ErrorFromSmartServer was uncaught.  This is logic error in the client
-        and so should provoke a traceback to the user.
-      - ErrorFromSmartServer was caught but its error_tuple could not be
-        translated.  This is probably because the server sent us garbage, and
-        should not provoke a traceback.
+
+    - ErrorFromSmartServer was uncaught.  This is logic error in the client
+      and so should provoke a traceback to the user.
+    - ErrorFromSmartServer was caught but its error_tuple could not be
+      translated.  This is probably because the server sent us garbage, and
+      should not provoke a traceback.
     """
 
     _fmt = "Server sent an unexpected error: %(error_tuple)r"
@@ -2693,7 +2805,7 @@
     _fmt = "Container has multiple records with the same name: %(name)s"
 
     def __init__(self, name):
-        self.name = name
+        self.name = name.decode("utf-8")
 
 
 class NoDestinationAddress(InternalBzrError):
@@ -2843,8 +2955,11 @@
         else:
             more = ' ' + more
         import bzrlib.urlutils as urlutils
-        display_url = urlutils.unescape_for_display(
-            tree.user_url, 'ascii')
+        user_url = getattr(tree, "user_url", None)
+        if user_url is None:
+            display_url = str(tree)
+        else:
+            display_url = urlutils.unescape_for_display(user_url, 'ascii')
         BzrError.__init__(self, tree=tree, display_url=display_url, more=more)
 
 
@@ -2928,6 +3043,22 @@
         self.user_encoding = osutils.get_user_encoding()
 
 
+class NoSuchConfig(BzrError):
+
+    _fmt = ('The "%(config_id)s" configuration does not exist.')
+
+    def __init__(self, config_id):
+        BzrError.__init__(self, config_id=config_id)
+
+
+class NoSuchConfigOption(BzrError):
+
+    _fmt = ('The "%(option_name)s" configuration option does not exist.')
+
+    def __init__(self, option_name):
+        BzrError.__init__(self, option_name=option_name)
+
+
 class NoSuchAlias(BzrError):
 
     _fmt = ('The alias "%(alias_name)s" does not exist.')
@@ -3017,6 +3148,18 @@
     _fmt = "Shelf corrupt."
 
 
+class DecompressCorruption(BzrError):
+
+    _fmt = "Corruption while decompressing repository file%(orig_error)s"
+
+    def __init__(self, orig_error=None):
+        if orig_error is not None:
+            self.orig_error = ", %s" % (orig_error,)
+        else:
+            self.orig_error = ""
+        BzrError.__init__(self)
+
+
 class NoSuchShelfId(BzrError):
 
     _fmt = 'No changes are shelved with id "%(shelf_id)d".'
@@ -3173,3 +3316,42 @@
     def __init__(self, branch_url):
         self.branch_url = branch_url
 
+
+# FIXME: I would prefer to define the config related exception classes in
+# config.py but the lazy import mechanism proscribes this -- vila 20101222
+class OptionExpansionLoop(BzrError):
+
+    _fmt = 'Loop involving %(refs)r while expanding "%(string)s".'
+
+    def __init__(self, string, refs):
+        self.string = string
+        self.refs = '->'.join(refs)
+
+
+class ExpandingUnknownOption(BzrError):
+
+    _fmt = 'Option %(name)s is not defined while expanding "%(string)s".'
+
+    def __init__(self, name, string):
+        self.name = name
+        self.string = string
+
+
+class NoCompatibleInter(BzrError):
+
+    _fmt = ('No compatible object available for operations from %(source)r '
+            'to %(target)r.')
+
+    def __init__(self, source, target):
+        self.source = source
+        self.target = target
+
+
+class HpssVfsRequestNotAllowed(BzrError):
+
+    _fmt = ("VFS requests over the smart server are not allowed. Encountered: "
+            "%(method)s, %(arguments)s.")
+
+    def __init__(self, method, arguments):
+        self.method = method
+        self.arguments = arguments

=== added file 'bzrlib/estimate_compressed_size.py'
--- bzrlib/estimate_compressed_size.py	1970-01-01 00:00:00 +0000
+++ bzrlib/estimate_compressed_size.py	2011-09-01 16:54:46 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Code to estimate the entropy of content"""
+
+import zlib
+
+
+class ZLibEstimator(object):
+    """Uses zlib.compressobj to estimate compressed size."""
+
+    def __init__(self, target_size, min_compression=2.0):
+        """Create a new estimator.
+
+        :param target_size: The desired size of the compressed content.
+        :param min_compression: Estimated minimum compression. By default we
+            assume that the content is 'text', which means a min compression of
+            about 2:1.
+        """
+        self._target_size = target_size
+        self._compressor = zlib.compressobj()
+        self._uncompressed_size_added = 0
+        self._compressed_size_added = 0
+        self._unflushed_size_added = 0
+        self._estimated_compression = 2.0
+
+    def add_content(self, content):
+        self._uncompressed_size_added += len(content)
+        self._unflushed_size_added += len(content)
+        z_size = len(self._compressor.compress(content))
+        if z_size > 0:
+            self._record_z_len(z_size)
+
+    def _record_z_len(self, count):
+        # We got some compressed bytes, update the counters
+        self._compressed_size_added += count
+        self._unflushed_size_added = 0
+        # So far we've read X uncompressed bytes, and written Y compressed
+        # bytes. We should have a decent estimate of the final compression.
+        self._estimated_compression = (float(self._uncompressed_size_added)
+            / self._compressed_size_added)
+
+    def full(self):
+        """Have we reached the target size?"""
+        if self._unflushed_size_added:
+            remaining_size = self._target_size - self._compressed_size_added
+            # Estimate how much compressed content the unflushed data will
+            # consume
+            est_z_size = (self._unflushed_size_added /
+                          self._estimated_compression)
+            if est_z_size >= remaining_size:
+                # We estimate we are close to remaining
+                z_size = len(self._compressor.flush(zlib.Z_SYNC_FLUSH))
+                self._record_z_len(z_size)
+        return self._compressed_size_added >= self._target_size

=== modified file 'bzrlib/export/__init__.py'
--- bzrlib/export/__init__.py	2010-03-25 09:39:03 +0000
+++ bzrlib/export/__init__.py	2011-07-08 23:38:46 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,19 +14,25 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Export functionality, which can take a Tree and create a different representation.
-
-Such as non-controlled directories, tarfiles, zipfiles, etc.
+"""Export trees to tarballs, non-controlled directories, zipfiles, etc.
 """
 
 import os
-import bzrlib.errors as errors
+import time
+import warnings
+
+from bzrlib import (
+    errors,
+    pyutils,
+    trace,
+    )
 
 # Maps format name => export function
 _exporters = {}
 # Maps filename extensions => export format name
 _exporter_extensions = {}
 
+
 def register_exporter(format, extensions, func, override=False):
     """Register an exporter.
 
@@ -54,41 +60,51 @@
 
     When requesting a specific type of export, load the respective path.
     """
-    def _loader(tree, dest, root, subdir, filtered, per_file_timestamps):
-        mod = __import__(module, globals(), locals(), [funcname])
-        func = getattr(mod, funcname)
-        return func(tree, dest, root, subdir, filtered=filtered,
-                    per_file_timestamps=per_file_timestamps)
+    def _loader(tree, dest, root, subdir, force_mtime, fileobj):
+        func = pyutils.get_named_object(module, funcname)
+        return func(tree, dest, root, subdir, force_mtime=force_mtime,
+            fileobj=fileobj)
+
     register_exporter(scheme, extensions, _loader)
 
 
-def export(tree, dest, format=None, root=None, subdir=None, filtered=False,
-           per_file_timestamps=False):
-    """Export the given Tree to the specific destination.
+def get_export_generator(tree, dest=None, format=None, root=None, subdir=None,
+                         filtered=False, per_file_timestamps=False,
+                         fileobj=None):
+    """Returns a generator that exports the given tree.
+
+    The generator is expected to yield None while exporting the tree while the
+    actual export is written to ``fileobj``.
 
     :param tree: A Tree (such as RevisionTree) to export
-    :param dest: The destination where the files,etc should be put
+
+    :param dest: The destination where the files, etc should be put
+
     :param format: The format (dir, zip, etc), if None, it will check the
-                   extension on dest, looking for a match
-    :param root: The root location inside the format.
-                 It is common practise to have zipfiles and tarballs
-                 extract into a subdirectory, rather than into the
-                 current working directory.
-                 If root is None, the default root will be
-                 selected as the destination without its
-                 extension.
+        extension on dest, looking for a match
+
+    :param root: The root location inside the format.  It is common practise to
+        have zipfiles and tarballs extract into a subdirectory, rather than
+        into the current working directory.  If root is None, the default root
+        will be selected as the destination without its extension.
+
     :param subdir: A starting directory within the tree. None means to export
         the entire tree, and anything else should specify the relative path to
         a directory to start exporting from.
-    :param filtered: If True, content filtering is applied to the
-                     files exported.
-    :param per_file_timestamps: Whether to use the timestamp stored in the 
-        tree rather than now(). This will do a revision lookup 
-        for every file so will be significantly slower.
+
+    :param filtered: If True, content filtering is applied to the exported
+        files.  Deprecated in favour of passing a ContentFilterTree
+        as the source.
+
+    :param per_file_timestamps: Whether to use the timestamp stored in the tree
+        rather than now(). This will do a revision lookup for every file so
+        will be significantly slower.
+
+    :param fileobj: Optional file object to use
     """
     global _exporters, _exporter_extensions
 
-    if format is None:
+    if format is None and dest is not None:
         for ext in _exporter_extensions:
             if dest.endswith(ext):
                 format = _exporter_extensions[ext]
@@ -101,37 +117,71 @@
 
     if format not in _exporters:
         raise errors.NoSuchExportFormat(format)
+
+    if not per_file_timestamps:
+        force_mtime = time.time()
+    else:
+        force_mtime = None
+
+    trace.mutter('export version %r', tree)
+
+    if filtered:
+        from bzrlib.filter_tree import ContentFilterTree
+        warnings.warn(
+            "passing filtered=True to export is deprecated in bzr 2.4",
+            stacklevel=2)
+        tree = ContentFilterTree(tree, tree._content_filter_stack)
+        # We don't want things re-filtered by the specific exporter.
+        filtered = False
+
     tree.lock_read()
     try:
-        return _exporters[format](tree, dest, root, subdir, filtered=filtered,
-                                  per_file_timestamps=per_file_timestamps)
+        for _ in _exporters[format](
+            tree, dest, root, subdir,
+            force_mtime=force_mtime, fileobj=fileobj):
+            yield
     finally:
         tree.unlock()
 
 
+def export(tree, dest, format=None, root=None, subdir=None, filtered=False,
+           per_file_timestamps=False, fileobj=None):
+    """Export the given Tree to the specific destination.
+
+    :param tree: A Tree (such as RevisionTree) to export
+    :param dest: The destination where the files,etc should be put
+    :param format: The format (dir, zip, etc), if None, it will check the
+                   extension on dest, looking for a match
+    :param root: The root location inside the format.
+                 It is common practise to have zipfiles and tarballs
+                 extract into a subdirectory, rather than into the
+                 current working directory.
+                 If root is None, the default root will be
+                 selected as the destination without its
+                 extension.
+    :param subdir: A starting directory within the tree. None means to export
+        the entire tree, and anything else should specify the relative path to
+        a directory to start exporting from.
+    :param filtered: If True, content filtering is applied to the
+        files exported.  Deprecated in favor of passing an ContentFilterTree.
+    :param per_file_timestamps: Whether to use the timestamp stored in the
+        tree rather than now(). This will do a revision lookup
+        for every file so will be significantly slower.
+    :param fileobj: Optional file object to use
+    """
+    for _ in get_export_generator(tree, dest, format, root, subdir, filtered,
+                                  per_file_timestamps, fileobj):
+        pass
+
+
 def get_root_name(dest):
     """Get just the root name for an export.
 
-    >>> get_root_name('../mytest.tar')
-    'mytest'
-    >>> get_root_name('mytar.tar')
-    'mytar'
-    >>> get_root_name('mytar.tar.bz2')
-    'mytar'
-    >>> get_root_name('tar.tar.tar.tgz')
-    'tar.tar.tar'
-    >>> get_root_name('bzr-0.0.5.tar.gz')
-    'bzr-0.0.5'
-    >>> get_root_name('bzr-0.0.5.zip')
-    'bzr-0.0.5'
-    >>> get_root_name('bzr-0.0.5')
-    'bzr-0.0.5'
-    >>> get_root_name('a/long/path/mytar.tgz')
-    'mytar'
-    >>> get_root_name('../parent/../dir/other.tbz2')
-    'other'
     """
     global _exporter_extensions
+    if dest == '-':
+        # Exporting to -/foo doesn't make sense so use relative paths.
+        return ''
     dest = os.path.basename(dest)
     for ext in _exporter_extensions:
         if dest.endswith(ext):
@@ -139,47 +189,55 @@
     return dest
 
 
-def _export_iter_entries(tree, subdir):
+def _export_iter_entries(tree, subdir, skip_special=True):
     """Iter the entries for tree suitable for exporting.
 
     :param tree: A tree object.
     :param subdir: None or the path of an entry to start exporting from.
+    :param skip_special: Whether to skip .bzr files.
     """
-    inv = tree.inventory
-    if subdir is None:
-        subdir_object = None
-    else:
-        subdir_id = inv.path2id(subdir)
-        if subdir_id is not None:
-            subdir_object = inv[subdir_id]
-        # XXX: subdir is path not an id, so NoSuchId isn't proper error
-        else:
-            raise errors.NoSuchId(tree, subdir)
-    if subdir_object is not None and subdir_object.kind != 'directory':
-        yield subdir_object.name, subdir_object
-        return
-    else:
-        entries = inv.iter_entries(subdir_object)
-    if subdir is None:
-        entries.next() # skip root
-    for entry in entries:
+    if subdir == '':
+        subdir = None
+    if subdir is not None:
+        subdir = subdir.rstrip('/')
+    entries = tree.iter_entries_by_dir()
+    entries.next()  # skip root
+    for path, entry in entries:
         # The .bzr* namespace is reserved for "magic" files like
         # .bzrignore and .bzrrules - do not export these
-        if entry[0].startswith(".bzr"):
+        if skip_special and path.startswith(".bzr"):
             continue
-        if subdir is None:
-            if not tree.has_filename(entry[0]):
+        if path == subdir:
+            if entry.kind == 'directory':
+                continue
+            final_path = entry.name
+        elif subdir is not None:
+            if path.startswith(subdir + '/'):
+                final_path = path[len(subdir) + 1:]
+            else:
                 continue
         else:
-            if not tree.has_filename(os.path.join(subdir, entry[0])):
-                continue
-        yield entry
-
-
-register_lazy_exporter(None, [], 'bzrlib.export.dir_exporter', 'dir_exporter')
-register_lazy_exporter('dir', [], 'bzrlib.export.dir_exporter', 'dir_exporter')
-register_lazy_exporter('tar', ['.tar'], 'bzrlib.export.tar_exporter', 'tar_exporter')
-register_lazy_exporter('tgz', ['.tar.gz', '.tgz'], 'bzrlib.export.tar_exporter', 'tgz_exporter')
-register_lazy_exporter('tbz2', ['.tar.bz2', '.tbz2'], 'bzrlib.export.tar_exporter', 'tbz_exporter')
-register_lazy_exporter('zip', ['.zip'], 'bzrlib.export.zip_exporter', 'zip_exporter')
-
+            final_path = path
+        if not tree.has_filename(path):
+            continue
+
+        yield final_path, entry
+
+
+register_lazy_exporter(None, [], 'bzrlib.export.dir_exporter',
+                       'dir_exporter_generator')
+register_lazy_exporter('dir', [], 'bzrlib.export.dir_exporter',
+                       'dir_exporter_generator')
+register_lazy_exporter('tar', ['.tar'], 'bzrlib.export.tar_exporter',
+                       'plain_tar_exporter_generator')
+register_lazy_exporter('tgz', ['.tar.gz', '.tgz'],
+                       'bzrlib.export.tar_exporter',
+                       'tgz_exporter_generator')
+register_lazy_exporter('tbz2', ['.tar.bz2', '.tbz2'],
+                       'bzrlib.export.tar_exporter', 'tbz_exporter_generator')
+register_lazy_exporter('tlzma', ['.tar.lzma'], 'bzrlib.export.tar_exporter',
+                       'tar_lzma_exporter_generator')
+register_lazy_exporter('txz', ['.tar.xz'], 'bzrlib.export.tar_exporter',
+                       'tar_xz_exporter_generator')
+register_lazy_exporter('zip', ['.zip'], 'bzrlib.export.zip_exporter',
+                       'zip_exporter_generator')

=== modified file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	2010-05-25 17:27:52 +0000
+++ bzrlib/export/dir_exporter.py	2011-07-04 21:55:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,35 +18,31 @@
 
 import errno
 import os
-import time
 
 from bzrlib import errors, osutils
 from bzrlib.export import _export_iter_entries
-from bzrlib.filters import (
-    ContentFilterContext,
-    filtered_output_bytes,
-    )
-from bzrlib.trace import mutter
-
-
-def dir_exporter(tree, dest, root, subdir, filtered=False,
-                 per_file_timestamps=False):
-    """Export this tree to a new directory.
+
+
+def dir_exporter_generator(tree, dest, root, subdir=None,
+                           force_mtime=None, fileobj=None):
+    """Return a generator that exports this tree to a new directory.
 
     `dest` should either not exist or should be empty. If it does not exist it
     will be created holding the contents of this tree.
 
+    :param fileobj: Is not used in this exporter
+
     :note: If the export fails, the destination directory will be
            left in an incompletely exported state: export is not transactional.
     """
-    mutter('export version %r', tree)
     try:
         os.mkdir(dest)
     except OSError, e:
         if e.errno == errno.EEXIST:
             # check if directory empty
             if os.listdir(dest) != []:
-                raise errors.BzrError("Can't export tree to non-empty directory.")
+                raise errors.BzrError(
+                    "Can't export tree to non-empty directory.")
         else:
             raise
     # Iterate everything, building up the files we will want to export, and
@@ -66,22 +62,19 @@
             try:
                 symlink_target = tree.get_symlink_target(ie.file_id)
                 os.symlink(symlink_target, fullpath)
-            except OSError,e:
+            except OSError, e:
                 raise errors.BzrError(
                     "Failed to create symlink %r -> %r, error: %s"
                     % (fullpath, symlink_target, e))
         else:
             raise errors.BzrError("don't know how to export {%s} of kind %r" %
                (ie.file_id, ie.kind))
+
+        yield
     # The data returned here can be in any order, but we've already created all
     # the directories
     flags = os.O_CREAT | os.O_TRUNC | os.O_WRONLY | getattr(os, 'O_BINARY', 0)
-    now = time.time()
     for (relpath, executable), chunks in tree.iter_files_bytes(to_fetch):
-        if filtered:
-            filters = tree._content_filter_stack(relpath)
-            context = ContentFilterContext(relpath, tree, ie)
-            chunks = filtered_output_bytes(chunks, filters, context)
         fullpath = osutils.pathjoin(dest, relpath)
         # We set the mode and let the umask sort out the file info
         mode = 0666
@@ -92,8 +85,14 @@
             out.writelines(chunks)
         finally:
             out.close()
-        if per_file_timestamps:
-            mtime = tree.get_file_mtime(tree.path2id(relpath), relpath)
+        if force_mtime is not None:
+            mtime = force_mtime
         else:
-            mtime = now
+            if subdir is None:
+                file_id = tree.path2id(relpath)
+            else:
+                file_id = tree.path2id(osutils.pathjoin(subdir, relpath))
+            mtime = tree.get_file_mtime(file_id, relpath)
         os.utime(fullpath, (mtime, mtime))
+
+        yield

=== modified file 'bzrlib/export/tar_exporter.py'
--- bzrlib/export/tar_exporter.py	2010-04-14 10:12:22 +0000
+++ bzrlib/export/tar_exporter.py	2011-07-21 07:08:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,98 +14,215 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Export a Tree to a non-versioned directory.
-"""
+"""Export a tree to a tarball."""
 
+import os
 import StringIO
 import sys
 import tarfile
-import time
 
-from bzrlib import export, osutils
+from bzrlib import (
+    errors,
+    osutils,
+    )
 from bzrlib.export import _export_iter_entries
-from bzrlib.filters import (
-    ContentFilterContext,
-    filtered_output_bytes,
-    )
-from bzrlib.trace import mutter
-
-
-def tar_exporter(tree, dest, root, subdir, compression=None, filtered=False,
-                 per_file_timestamps=False):
-    """Export this tree to a new tar file.
-
-    `dest` will be created holding the contents of this tree; if it
-    already exists, it will be clobbered, like with "tar -c".
-    """
-    mutter('export version %r', tree)
-    now = time.time()
-    compression = str(compression or '')
+
+
+def prepare_tarball_item(tree, root, final_path, entry, force_mtime=None):
+    """Prepare a tarball item for exporting
+
+    :param tree: Tree to export
+
+    :param final_path: Final path to place item
+
+    :param entry: Entry to export
+
+    :param force_mtime: Option mtime to force, instead of using tree
+        timestamps.
+
+    Returns a (tarinfo, fileobj) tuple
+    """
+    filename = osutils.pathjoin(root, final_path).encode('utf8')
+    item = tarfile.TarInfo(filename)
+    if force_mtime is not None:
+        item.mtime = force_mtime
+    else:
+        item.mtime = tree.get_file_mtime(entry.file_id, final_path)
+    if entry.kind == "file":
+        item.type = tarfile.REGTYPE
+        if tree.is_executable(entry.file_id):
+            item.mode = 0755
+        else:
+            item.mode = 0644
+        # This brings the whole file into memory, but that's almost needed for
+        # the tarfile contract, which wants the size of the file up front.  We
+        # want to make sure it doesn't change, and we need to read it in one
+        # go for content filtering.
+        content = tree.get_file_text(entry.file_id)
+        item.size = len(content)
+        fileobj = StringIO.StringIO(content)
+    elif entry.kind == "directory":
+        item.type = tarfile.DIRTYPE
+        item.name += '/'
+        item.size = 0
+        item.mode = 0755
+        fileobj = None
+    elif entry.kind == "symlink":
+        item.type = tarfile.SYMTYPE
+        item.size = 0
+        item.mode = 0755
+        item.linkname = tree.get_symlink_target(entry.file_id)
+        fileobj = None
+    else:
+        raise errors.BzrError("don't know how to export {%s} of kind %r"
+                              % (entry.file_id, entry.kind))
+    return (item, fileobj)
+
+
+def export_tarball_generator(tree, ball, root, subdir=None, force_mtime=None):
+    """Export tree contents to a tarball.
+
+    :returns: A generator that will repeatedly produce None as each file is
+        emitted.  The entire generator must be consumed to complete writing
+        the file.
+
+    :param tree: Tree to export
+
+    :param ball: Tarball to export to; it will be closed when writing is
+        complete.
+
+    :param subdir: Sub directory to export
+
+    :param force_mtime: Option mtime to force, instead of using tree
+        timestamps.
+    """
+    try:
+        for final_path, entry in _export_iter_entries(tree, subdir):
+            (item, fileobj) = prepare_tarball_item(
+                tree, root, final_path, entry, force_mtime)
+            ball.addfile(item, fileobj)
+            yield
+    finally:
+        ball.close()
+
+
+def tgz_exporter_generator(tree, dest, root, subdir, force_mtime=None,
+    fileobj=None):
+    """Export this tree to a new tar file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be clobbered, like with "tar -c".
+    """
+    import gzip
+    if force_mtime is not None:
+        root_mtime = force_mtime
+    elif (getattr(tree, "repository", None) and
+          getattr(tree, "get_revision_id", None)):
+        # If this is a revision tree, use the revisions' timestamp
+        rev = tree.repository.get_revision(tree.get_revision_id())
+        root_mtime = rev.timestamp
+    elif tree.get_root_id() is not None:
+        root_mtime = tree.get_file_mtime(tree.get_root_id())
+    else:
+        root_mtime = None
+
+    is_stdout = False
+    basename = None
+    if fileobj is not None:
+        stream = fileobj
+    elif dest == '-':
+        stream = sys.stdout
+        is_stdout = True
+    else:
+        stream = open(dest, 'wb')
+        # gzip file is used with an explicit fileobj so that
+        # the basename can be stored in the gzip file rather than
+        # dest. (bug 102234)
+        basename = os.path.basename(dest)
+    try:
+        zipstream = gzip.GzipFile(basename, 'w', fileobj=stream,
+                                  mtime=root_mtime)
+    except TypeError:
+        # Python < 2.7 doesn't support the mtime argument
+        zipstream = gzip.GzipFile(basename, 'w', fileobj=stream)
+    ball = tarfile.open(None, 'w|', fileobj=zipstream)
+    for _ in export_tarball_generator(
+        tree, ball, root, subdir, force_mtime):
+        yield
+    # Closing zipstream may trigger writes to stream
+    zipstream.close()
+    if not is_stdout:
+        # Now we can safely close the stream
+        stream.close()
+
+
+def tbz_exporter_generator(tree, dest, root, subdir,
+                           force_mtime=None, fileobj=None):
+    """Export this tree to a new tar file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be clobbered, like with "tar -c".
+    """
+    if fileobj is not None:
+        ball = tarfile.open(None, 'w|bz2', fileobj)
+    elif dest == '-':
+        ball = tarfile.open(None, 'w|bz2', sys.stdout)
+    else:
+        # tarfile.open goes on to do 'os.getcwd() + dest' for opening the
+        # tar file. With dest being unicode, this throws UnicodeDecodeError
+        # unless we encode dest before passing it on. This works around
+        # upstream python bug http://bugs.python.org/issue8396 (fixed in
+        # Python 2.6.5 and 2.7b1)
+        ball = tarfile.open(dest.encode(osutils._fs_enc), 'w:bz2')
+    return export_tarball_generator(
+        tree, ball, root, subdir, force_mtime)
+
+
+def plain_tar_exporter_generator(tree, dest, root, subdir, compression=None,
+    force_mtime=None, fileobj=None):
+    """Export this tree to a new tar file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be clobbered, like with "tar -c".
+    """
+    if fileobj is not None:
+        stream = fileobj
+    elif dest == '-':
+        stream = sys.stdout
+    else:
+        stream = open(dest, 'wb')
+    ball = tarfile.open(None, 'w|', stream)
+    return export_tarball_generator(
+        tree, ball, root, subdir, force_mtime)
+
+
+def tar_xz_exporter_generator(tree, dest, root, subdir,
+                              force_mtime=None, fileobj=None):
+    return tar_lzma_exporter_generator(tree, dest, root, subdir,
+                                       force_mtime, fileobj, "xz")
+
+
+def tar_lzma_exporter_generator(tree, dest, root, subdir,
+                      force_mtime=None, fileobj=None,
+                                compression_format="alone"):
+    """Export this tree to a new .tar.lzma file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be clobbered, like with "tar -c".
+    """
     if dest == '-':
-        # XXX: If no root is given, the output tarball will contain files
-        # named '-/foo'; perhaps this is the most reasonable thing.
-        ball = tarfile.open(None, 'w|' + compression, sys.stdout)
-    else:
-        if root is None:
-            root = export.get_root_name(dest)
-
-        # tarfile.open goes on to do 'os.getcwd() + dest' for opening
-        # the tar file. With dest being unicode, this throws UnicodeDecodeError
-        # unless we encode dest before passing it on. This works around
-        # upstream python bug http://bugs.python.org/issue8396
-        # (fixed in Python 2.6.5 and 2.7b1)
-        ball = tarfile.open(dest.encode(osutils._fs_enc), 'w:' + compression)
-
-    for dp, ie in _export_iter_entries(tree, subdir):
-        filename = osutils.pathjoin(root, dp).encode('utf8')
-        item = tarfile.TarInfo(filename)
-        if per_file_timestamps:
-            item.mtime = tree.get_file_mtime(ie.file_id, dp)
-        else:
-            item.mtime = now
-        if ie.kind == "file":
-            item.type = tarfile.REGTYPE
-            if tree.is_executable(ie.file_id):
-                item.mode = 0755
-            else:
-                item.mode = 0644
-            if filtered:
-                chunks = tree.get_file_lines(ie.file_id)
-                filters = tree._content_filter_stack(dp)
-                context = ContentFilterContext(dp, tree, ie)
-                contents = filtered_output_bytes(chunks, filters, context)
-                content = ''.join(contents)
-                item.size = len(content)
-                fileobj = StringIO.StringIO(content)
-            else:
-                item.size = ie.text_size
-                fileobj = tree.get_file(ie.file_id)
-        elif ie.kind == "directory":
-            item.type = tarfile.DIRTYPE
-            item.name += '/'
-            item.size = 0
-            item.mode = 0755
-            fileobj = None
-        elif ie.kind == "symlink":
-            item.type = tarfile.SYMTYPE
-            item.size = 0
-            item.mode = 0755
-            item.linkname = ie.symlink_target
-            fileobj = None
-        else:
-            raise BzrError("don't know how to export {%s} of kind %r" %
-                           (ie.file_id, ie.kind))
-        ball.addfile(item, fileobj)
-    ball.close()
-
-
-def tgz_exporter(tree, dest, root, subdir, filtered=False,
-                 per_file_timestamps=False):
-    tar_exporter(tree, dest, root, subdir, compression='gz',
-                 filtered=filtered, per_file_timestamps=per_file_timestamps)
-
-
-def tbz_exporter(tree, dest, root, subdir, filtered=False,
-                 per_file_timestamps=False):
-    tar_exporter(tree, dest, root, subdir, compression='bz2',
-                 filtered=filtered, per_file_timestamps=per_file_timestamps)
+        raise errors.BzrError("Writing to stdout not supported for .tar.lzma")
+
+    if fileobj is not None:
+        raise errors.BzrError(
+            "Writing to fileobject not supported for .tar.lzma")
+    try:
+        import lzma
+    except ImportError, e:
+        raise errors.DependencyNotPresent('lzma', e)
+
+    stream = lzma.LZMAFile(dest.encode(osutils._fs_enc), 'w',
+        options={"format": compression_format})
+    ball = tarfile.open(None, 'w:', fileobj=stream)
+    return export_tarball_generator(
+        tree, ball, root, subdir, force_mtime=force_mtime)

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2010-03-08 02:23:17 +0000
+++ bzrlib/export/zip_exporter.py	2011-07-04 21:55:35 +0000
@@ -14,11 +14,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Export a Tree to a non-versioned directory.
+"""Export a Tree to a zip file.
 """
 
 import os
 import stat
+import sys
 import time
 import zipfile
 
@@ -26,36 +27,33 @@
     osutils,
     )
 from bzrlib.export import _export_iter_entries
-from bzrlib.filters import (
-    ContentFilterContext,
-    filtered_output_bytes,
-    )
 from bzrlib.trace import mutter
 
 
-# Windows expects this bit to be set in the 'external_attr' section
-# Or it won't consider the entry a directory
+# Windows expects this bit to be set in the 'external_attr' section,
+# or it won't consider the entry a directory.
 ZIP_DIRECTORY_BIT = (1 << 4)
 FILE_PERMISSIONS = (0644 << 16)
+DIR_PERMISSIONS = (0755 << 16)
 
 _FILE_ATTR = stat.S_IFREG | FILE_PERMISSIONS
-_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT
-
-
-def zip_exporter(tree, dest, root, subdir, filtered=False,
-                 per_file_timestamps=False):
+_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT | DIR_PERMISSIONS
+
+
+def zip_exporter_generator(tree, dest, root, subdir=None,
+    force_mtime=None, fileobj=None):
     """ Export this tree to a new zip file.
 
     `dest` will be created holding the contents of this tree; if it
     already exists, it will be overwritten".
     """
-    mutter('export version %r', tree)
-
-    now = time.localtime()[:6]
 
     compression = zipfile.ZIP_DEFLATED
+    if fileobj is not None:
+        dest = fileobj
+    elif dest == "-":
+        dest = sys.stdout
     zipf = zipfile.ZipFile(dest, "w", compression)
-
     try:
         for dp, ie in _export_iter_entries(tree, subdir):
             file_id = ie.file_id
@@ -63,25 +61,19 @@
 
             # zipfile.ZipFile switches all paths to forward
             # slashes anyway, so just stick with that.
-            if per_file_timestamps:
+            if force_mtime is not None:
+                mtime = force_mtime
+            else:
                 mtime = tree.get_file_mtime(ie.file_id, dp)
-            else:
-                mtime = now
+            date_time = time.localtime(mtime)[:6]
             filename = osutils.pathjoin(root, dp).encode('utf8')
             if ie.kind == "file":
                 zinfo = zipfile.ZipInfo(
                             filename=filename,
-                            date_time=mtime)
+                            date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _FILE_ATTR
-                if filtered:
-                    chunks = tree.get_file_lines(file_id)
-                    filters = tree._content_filter_stack(dp)
-                    context = ContentFilterContext(dp, tree, ie)
-                    contents = filtered_output_bytes(chunks, filters, context)
-                    content = ''.join(contents)
-                else:
-                    content = tree.get_file_text(file_id)
+                content = tree.get_file_text(file_id)
                 zipf.writestr(zinfo, content)
             elif ie.kind == "directory":
                 # Directories must contain a trailing slash, to indicate
@@ -89,17 +81,18 @@
                 # not just empty files.
                 zinfo = zipfile.ZipInfo(
                             filename=filename + '/',
-                            date_time=mtime)
+                            date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _DIR_ATTR
-                zipf.writestr(zinfo,'')
+                zipf.writestr(zinfo, '')
             elif ie.kind == "symlink":
                 zinfo = zipfile.ZipInfo(
                             filename=(filename + '.lnk'),
-                            date_time=mtime)
+                            date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _FILE_ATTR
-                zipf.writestr(zinfo, ie.symlink_target)
+                zipf.writestr(zinfo, tree.get_symlink_target(file_id))
+            yield
 
         zipf.close()
 

=== added file 'bzrlib/export_pot.py'
--- bzrlib/export_pot.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export_pot.py	2011-09-27 11:31:35 +0000
@@ -0,0 +1,264 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+# The normalize function is taken from pygettext which is distributed
+# with Python under the Python License, which is GPL compatible.
+
+"""Extract docstrings from Bazaar commands.
+"""
+
+import inspect
+import os
+
+from bzrlib import (
+    commands as _mod_commands,
+    errors,
+    help_topics,
+    plugin,
+    help,
+    )
+from bzrlib.trace import (
+    mutter,
+    note,
+    )
+from bzrlib.i18n import gettext
+
+
+def _escape(s):
+    s = (s.replace('\\', '\\\\')
+        .replace('\n', '\\n')
+        .replace('\r', '\\r')
+        .replace('\t', '\\t')
+        .replace('"', '\\"')
+        )
+    return s
+
+def _normalize(s):
+    # This converts the various Python string types into a format that
+    # is appropriate for .po files, namely much closer to C style.
+    lines = s.split('\n')
+    if len(lines) == 1:
+        s = '"' + _escape(s) + '"'
+    else:
+        if not lines[-1]:
+            del lines[-1]
+            lines[-1] = lines[-1] + '\n'
+        lines = map(_escape, lines)
+        lineterm = '\\n"\n"'
+        s = '""\n"' + lineterm.join(lines) + '"'
+    return s
+
+
+_FOUND_MSGID = None # set by entry function.
+
+def _poentry(outf, path, lineno, s, comment=None):
+    if s in _FOUND_MSGID:
+        return
+    _FOUND_MSGID.add(s)
+    if comment is None:
+        comment = ''
+    else:
+        comment = "# %s\n" % comment
+    mutter("Exporting msg %r at line %d in %r", s[:20], lineno, path)
+    print >>outf, ('#: %s:%d\n' % (path, lineno) +
+           comment+
+           'msgid %s\n' % _normalize(s) +
+           'msgstr ""\n')
+
+def _poentry_per_paragraph(outf, path, lineno, msgid, filter=lambda x: False):
+    # TODO: How to split long help?
+    paragraphs = msgid.split('\n\n')
+    for p in paragraphs:
+        if filter(p):
+            continue
+        _poentry(outf, path, lineno, p)
+        lineno += p.count('\n') + 2
+
+_LAST_CACHE = _LAST_CACHED_SRC = None
+
+def _offsets_of_literal(src):
+    global _LAST_CACHE, _LAST_CACHED_SRC
+    if src == _LAST_CACHED_SRC:
+        return _LAST_CACHE.copy()
+
+    import ast
+    root = ast.parse(src)
+    offsets = {}
+    for node in ast.walk(root):
+        if not isinstance(node, ast.Str):
+            continue
+        offsets[node.s] = node.lineno - node.s.count('\n')
+
+    _LAST_CACHED_SRC = src
+    _LAST_CACHE = offsets.copy()
+    return offsets
+
+def _standard_options(outf):
+    from bzrlib.option import Option
+    src = inspect.findsource(Option)[0]
+    src = ''.join(src)
+    path = 'bzrlib/option.py'
+    offsets = _offsets_of_literal(src)
+
+    for name in sorted(Option.OPTIONS.keys()):
+        opt = Option.OPTIONS[name]
+        if getattr(opt, 'hidden', False):
+            continue
+        if getattr(opt, 'title', None):
+            lineno = offsets.get(opt.title, 9999)
+            if lineno == 9999:
+                note(gettext("%r is not found in bzrlib/option.py") % opt.title)
+            _poentry(outf, path, lineno, opt.title,
+                     'title of %r option' % name)
+        if getattr(opt, 'help', None):
+            lineno = offsets.get(opt.help, 9999)
+            if lineno == 9999:
+                note(gettext("%r is not found in bzrlib/option.py") % opt.help)
+            _poentry(outf, path, lineno, opt.help,
+                     'help of %r option' % name)
+
+def _command_options(outf, path, cmd):
+    src, default_lineno = inspect.findsource(cmd.__class__)
+    offsets = _offsets_of_literal(''.join(src))
+    for opt in cmd.takes_options:
+        if isinstance(opt, str):
+            continue
+        if getattr(opt, 'hidden', False):
+            continue
+        name = opt.name
+        if getattr(opt, 'title', None):
+            lineno = offsets.get(opt.title, default_lineno)
+            _poentry(outf, path, lineno, opt.title,
+                     'title of %r option of %r command' % (name, cmd.name()))
+        if getattr(opt, 'help', None):
+            lineno = offsets.get(opt.help, default_lineno)
+            _poentry(outf, path, lineno, opt.help,
+                     'help of %r option of %r command' % (name, cmd.name()))
+
+
+def _write_command_help(outf, cmd):
+    path = inspect.getfile(cmd.__class__)
+    if path.endswith('.pyc'):
+        path = path[:-1]
+    path = os.path.relpath(path)
+    src, lineno = inspect.findsource(cmd.__class__)
+    offsets = _offsets_of_literal(''.join(src))
+    lineno = offsets[cmd.__doc__]
+    doc = inspect.getdoc(cmd)
+
+    def filter(p):
+        # ':Usage:' has special meaning in help topics.
+        # This is usage example of command and should not be translated.
+        if p.splitlines()[0] == ':Usage:':
+            return True
+
+    _poentry_per_paragraph(outf, path, lineno, doc, filter)
+    _command_options(outf, path, cmd)
+
+
+def _command_helps(outf, plugin_name=None):
+    """Extract docstrings from path.
+
+    This respects the Bazaar cmdtable/table convention and will
+    only extract docstrings from functions mentioned in these tables.
+    """
+    from glob import glob
+
+    # builtin commands
+    for cmd_name in _mod_commands.builtin_command_names():
+        command = _mod_commands.get_cmd_object(cmd_name, False)
+        if command.hidden:
+            continue
+        if plugin_name is not None:
+            # only export builtins if we are not exporting plugin commands
+            continue
+        note(gettext("Exporting messages from builtin command: %s"), cmd_name)
+        _write_command_help(outf, command)
+
+    plugin_path = plugin.get_core_plugin_path()
+    core_plugins = glob(plugin_path + '/*/__init__.py')
+    core_plugins = [os.path.basename(os.path.dirname(p))
+                        for p in core_plugins]
+    # plugins
+    for cmd_name in _mod_commands.plugin_command_names():
+        command = _mod_commands.get_cmd_object(cmd_name, False)
+        if command.hidden:
+            continue
+        if plugin_name is not None and command.plugin_name() != plugin_name:
+            # if we are exporting plugin commands, skip plugins we have not specified.
+            continue
+        if plugin_name is None and command.plugin_name() not in core_plugins:
+            # skip non-core plugins
+            # TODO: Support extracting from third party plugins.
+            continue
+        note(gettext("Exporting messages from plugin command: {0} in {1}").format(
+             cmd_name, command.plugin_name() ))
+        _write_command_help(outf, command)
+
+
+def _error_messages(outf):
+    """Extract fmt string from bzrlib.errors."""
+    path = errors.__file__
+    if path.endswith('.pyc'):
+        path = path[:-1]
+    offsets = _offsets_of_literal(open(path).read())
+
+    base_klass = errors.BzrError
+    for name in dir(errors):
+        klass = getattr(errors, name)
+        if not inspect.isclass(klass):
+            continue
+        if not issubclass(klass, base_klass):
+            continue
+        if klass is base_klass:
+            continue
+        if klass.internal_error:
+            continue
+        fmt = getattr(klass, "_fmt", None)
+        if fmt:
+            note(gettext("Exporting message from error: %s"), name)
+            _poentry(outf, 'bzrlib/errors.py',
+                     offsets.get(fmt, 9999), fmt)
+
+def _help_topics(outf):
+    topic_registry = help_topics.topic_registry
+    for key in topic_registry.keys():
+        doc = topic_registry.get(key)
+        if isinstance(doc, str):
+            _poentry_per_paragraph(
+                    outf,
+                    'dummy/help_topics/'+key+'/detail.txt',
+                    1, doc)
+        elif callable(doc): # help topics from files
+            _poentry_per_paragraph(
+                    outf,
+                    'en/help_topics/'+key+'.txt',
+                    1, doc(key))
+        summary = topic_registry.get_summary(key)
+        if summary is not None:
+            _poentry(outf, 'dummy/help_topics/'+key+'/summary.txt',
+                     1, summary)
+
+def export_pot(outf, plugin=None):
+    global _FOUND_MSGID
+    _FOUND_MSGID = set()
+    if plugin is None:
+        _standard_options(outf)
+        _command_helps(outf)
+        _error_messages(outf)
+        _help_topics(outf)
+    else:
+        _command_helps(outf, plugin)

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2010-05-13 10:08:32 +0000
+++ bzrlib/fetch.py	2011-09-16 15:32:49 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,15 +28,16 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
+    graph as _mod_graph,
     tsort,
     versionedfile,
     )
 """)
-import bzrlib
 from bzrlib import (
     errors,
     ui,
     )
+from bzrlib.i18n import gettext
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
 
@@ -54,6 +55,8 @@
 
         :param last_revision: If set, try to limit to the data this revision
             references.
+        :param fetch_spec: A SearchResult specifying which revisions to fetch.
+            If set, this overrides last_revision.
         :param find_ghosts: If True search the entire history for ghosts.
         """
         # repository.fetch has the responsibility for short-circuiting
@@ -91,12 +94,13 @@
         pb = ui.ui_factory.nested_progress_bar()
         pb.show_pct = pb.show_count = False
         try:
-            pb.update("Finding revisions", 0, 2)
-            search = self._revids_to_fetch()
-            if search is None:
+            pb.update(gettext("Finding revisions"), 0, 2)
+            search_result = self._revids_to_fetch()
+            mutter('fetching: %s', search_result)
+            if search_result.is_empty():
                 return
-            pb.update("Fetching revisions", 1, 2)
-            self._fetch_everything_for_search(search)
+            pb.update(gettext("Fetching revisions"), 1, 2)
+            self._fetch_everything_for_search(search_result)
         finally:
             pb.finished()
 
@@ -125,9 +129,6 @@
             pb.update("Inserting stream")
             resume_tokens, missing_keys = self.sink.insert_stream(
                 stream, from_format, [])
-            if self.to_repository._fallback_repositories:
-                missing_keys.update(
-                    self._parent_inventories(search.get_keys()))
             if missing_keys:
                 pb.update("Missing keys")
                 stream = source.get_stream_for_missing_keys(missing_keys)
@@ -151,29 +152,24 @@
         """Determines the exact revisions needed from self.from_repository to
         install self._last_revision in self.to_repository.
 
-        If no revisions need to be fetched, then this just returns None.
+        :returns: A SearchResult of some sort.  (Possibly a
+            PendingAncestryResult, EmptySearchResult, etc.)
         """
         if self._fetch_spec is not None:
+            # The fetch spec is already a concrete search result.
             return self._fetch_spec
-        mutter('fetch up to rev {%s}', self._last_revision)
-        if self._last_revision is NULL_REVISION:
+        elif self._last_revision == NULL_REVISION:
+            # fetch_spec is None + last_revision is null => empty fetch.
             # explicit limit of no revisions needed
-            return None
-        return self.to_repository.search_missing_revision_ids(
-            self.from_repository, self._last_revision,
-            find_ghosts=self.find_ghosts)
-
-    def _parent_inventories(self, revision_ids):
-        # Find all the parent revisions referenced by the stream, but
-        # not present in the stream, and make sure we send their
-        # inventories.
-        parent_maps = self.to_repository.get_parent_map(revision_ids)
-        parents = set()
-        map(parents.update, parent_maps.itervalues())
-        parents.discard(NULL_REVISION)
-        parents.difference_update(revision_ids)
-        missing_keys = set(('inventories', rev_id) for rev_id in parents)
-        return missing_keys
+            return _mod_graph.EmptySearchResult()
+        elif self._last_revision is not None:
+            return _mod_graph.NotInOtherForRevs(self.to_repository,
+                self.from_repository, [self._last_revision],
+                find_ghosts=self.find_ghosts).execute()
+        else: # self._last_revision is None:
+            return _mod_graph.EverythingNotInOther(self.to_repository,
+                self.from_repository,
+                find_ghosts=self.find_ghosts).execute()
 
 
 class Inter1and2Helper(object):
@@ -182,6 +178,9 @@
     This is for use by fetchers and converters.
     """
 
+    # This is a class variable so that the test suite can override it.
+    known_graph_threshold = 100
+
     def __init__(self, source):
         """Constructor.
 
@@ -243,20 +242,13 @@
         # yet, and are unlikely to in non-rich-root environments anyway.
         root_id_order.sort(key=operator.itemgetter(0))
         # Create a record stream containing the roots to create.
-        if len(revs) > 100:
-            # XXX: not covered by tests, should have a flag to always run
-            # this. -- mbp 20100129
-            graph = self.source_repo.get_known_graph_ancestry(revs)
+        if len(revs) > self.known_graph_threshold:
+            graph = self.source.get_known_graph_ancestry(revs)
         new_roots_stream = _new_root_data_stream(
             root_id_order, rev_id_to_root_id, parent_map, self.source, graph)
         return [('texts', new_roots_stream)]
 
 
-def _get_rich_root_heads_graph(source_repo, revision_ids):
-    """Get a Graph object suitable for asking heads() for new rich roots."""
-    return 
-
-
 def _new_root_data_stream(
     root_keys_to_create, rev_id_to_root_id_map, parent_map, repo, graph=None):
     """Generate a texts substream of synthesised root entries.
@@ -326,7 +318,7 @@
                 pass
             else:
                 try:
-                    parent_ids.append(tree.inventory[root_id].revision)
+                    parent_ids.append(tree.get_file_revision(root_id))
                 except errors.NoSuchId:
                     # not in the tree
                     pass
@@ -340,3 +332,98 @@
             selected_ids.append(parent_id)
     parent_keys = [(root_id, parent_id) for parent_id in selected_ids]
     return parent_keys
+
+
+class TargetRepoKinds(object):
+    """An enum-like set of constants.
+    
+    They are the possible values of FetchSpecFactory.target_repo_kinds.
+    """
+    
+    PREEXISTING = 'preexisting'
+    STACKED = 'stacked'
+    EMPTY = 'empty'
+
+
+class FetchSpecFactory(object):
+    """A helper for building the best fetch spec for a sprout call.
+
+    Factors that go into determining the sort of fetch to perform:
+     * did the caller specify any revision IDs?
+     * did the caller specify a source branch (need to fetch its
+       heads_to_fetch(), usually the tip + tags)
+     * is there an existing target repo (don't need to refetch revs it
+       already has)
+     * target is stacked?  (similar to pre-existing target repo: even if
+       the target itself is new don't want to refetch existing revs)
+
+    :ivar source_branch: the source branch if one specified, else None.
+    :ivar source_branch_stop_revision_id: fetch up to this revision of
+        source_branch, rather than its tip.
+    :ivar source_repo: the source repository if one found, else None.
+    :ivar target_repo: the target repository acquired by sprout.
+    :ivar target_repo_kind: one of the TargetRepoKinds constants.
+    """
+
+    def __init__(self):
+        self._explicit_rev_ids = set()
+        self.source_branch = None
+        self.source_branch_stop_revision_id = None
+        self.source_repo = None
+        self.target_repo = None
+        self.target_repo_kind = None
+        self.limit = None
+
+    def add_revision_ids(self, revision_ids):
+        """Add revision_ids to the set of revision_ids to be fetched."""
+        self._explicit_rev_ids.update(revision_ids)
+
+    def make_fetch_spec(self):
+        """Build a SearchResult or PendingAncestryResult or etc."""
+        if self.target_repo_kind is None or self.source_repo is None:
+            raise AssertionError(
+                'Incomplete FetchSpecFactory: %r' % (self.__dict__,))
+        if len(self._explicit_rev_ids) == 0 and self.source_branch is None:
+            if self.limit is not None:
+                raise NotImplementedError(
+                    "limit is only supported with a source branch set")
+            # Caller hasn't specified any revisions or source branch
+            if self.target_repo_kind == TargetRepoKinds.EMPTY:
+                return _mod_graph.EverythingResult(self.source_repo)
+            else:
+                # We want everything not already in the target (or target's
+                # fallbacks).
+                return _mod_graph.EverythingNotInOther(
+                    self.target_repo, self.source_repo).execute()
+        heads_to_fetch = set(self._explicit_rev_ids)
+        if self.source_branch is not None:
+            must_fetch, if_present_fetch = self.source_branch.heads_to_fetch()
+            if self.source_branch_stop_revision_id is not None:
+                # Replace the tip rev from must_fetch with the stop revision
+                # XXX: this might be wrong if the tip rev is also in the
+                # must_fetch set for other reasons (e.g. it's the tip of
+                # multiple loom threads?), but then it's pretty unclear what it
+                # should mean to specify a stop_revision in that case anyway.
+                must_fetch.discard(self.source_branch.last_revision())
+                must_fetch.add(self.source_branch_stop_revision_id)
+            heads_to_fetch.update(must_fetch)
+        else:
+            if_present_fetch = set()
+        if self.target_repo_kind == TargetRepoKinds.EMPTY:
+            # PendingAncestryResult does not raise errors if a requested head
+            # is absent.  Ideally it would support the
+            # required_ids/if_present_ids distinction, but in practice
+            # heads_to_fetch will almost certainly be present so this doesn't
+            # matter much.
+            all_heads = heads_to_fetch.union(if_present_fetch)
+            ret = _mod_graph.PendingAncestryResult(all_heads, self.source_repo)
+            if self.limit is not None:
+                graph = self.source_repo.get_graph()
+                topo_order = list(graph.iter_topo_order(ret.get_keys()))
+                result_set = topo_order[:self.limit]
+                ret = self.source_repo.revision_ids_to_search_result(result_set)
+            return ret
+        else:
+            return _mod_graph.NotInOtherForRevs(self.target_repo, self.source_repo,
+                required_ids=heads_to_fetch, if_present_ids=if_present_fetch,
+                limit=self.limit).execute()

=== added file 'bzrlib/filter_tree.py'
--- bzrlib/filter_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/filter_tree.py	2011-07-04 21:10:37 +0000
@@ -0,0 +1,82 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Content-filtered view of any tree.
+"""
+
+
+from bzrlib import (
+    tree,
+    )
+from bzrlib.filters import (
+    ContentFilter,
+    ContentFilterContext,
+    filtered_input_file,
+    filtered_output_bytes,
+    _get_filter_stack_for,
+    _get_registered_names,
+    internal_size_sha_file_byname,
+    register_filter_stack_map,
+    )
+
+
+class ContentFilterTree(tree.Tree):
+    """A virtual tree that applies content filters to an underlying tree.
+    
+    Not every operation is supported yet.
+    """
+
+    def __init__(self, backing_tree, filter_stack_callback):
+        """Construct a new filtered tree view.
+
+        :param filter_stack_callback: A callable taking a path that returns
+            the filter stack that should be used for that path.
+        :param backing_tree: An underlying tree to wrap.
+        """
+        self.backing_tree = backing_tree
+        self.filter_stack_callback = filter_stack_callback
+
+    def get_file_text(self, file_id, path=None):
+        chunks = self.backing_tree.get_file_lines(file_id, path)
+        filters = self.filter_stack_callback(path)
+        if path is None:
+            path = self.backing_tree.id2path(file_id)
+        context = ContentFilterContext(path, self, None)
+        contents = filtered_output_bytes(chunks, filters, context)
+        content = ''.join(contents)
+        return content
+
+    def has_filename(self, filename):
+        return self.backing_tree.has_filename
+
+    def is_executable(self, file_id, path=None):
+        return self.backing_tree.is_executable(file_id, path)
+
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=None):
+        # NB: This simply returns the parent tree's entries; the length may be
+        # wrong but it can't easily be calculated without filtering the whole
+        # text.  Currently all callers cope with this; perhaps they should be
+        # updated to a narrower interface that only provides things guaranteed
+        # cheaply available across all trees. -- mbp 20110705
+        return self.backing_tree.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids,
+            yield_parents=yield_parents)
+
+    def lock_read(self):
+        return self.backing_tree.lock_read()
+
+    def unlock(self):
+        return self.backing_tree.unlock()

=== modified file 'bzrlib/filters/__init__.py'
--- bzrlib/filters/__init__.py	2009-06-10 08:54:56 +0000
+++ bzrlib/filters/__init__.py	2011-04-07 10:36:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,7 +39,6 @@
 Note that context is currently only supported for write converters.
 """
 
-
 from cStringIO import StringIO
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """

=== modified file 'bzrlib/foreign.py'
--- bzrlib/foreign.py	2010-04-28 10:33:44 +0000
+++ bzrlib/foreign.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
 from bzrlib.branch import (
     Branch,
-    InterBranch,
     )
 from bzrlib.commands import Command, Option
 from bzrlib.repository import Repository
@@ -29,10 +28,10 @@
 lazy_import(globals(), """
 from bzrlib import (
     errors,
-    osutils,
     registry,
     transform,
     )
+from bzrlib.i18n import gettext
 """)
 
 class VcsMapping(object):
@@ -284,7 +283,7 @@
     def run(self, location=None, remember=False, directory=None,
             no_rebase=False, strict=None):
         from bzrlib import urlutils
-        from bzrlib.bzrdir import BzrDir
+        from bzrlib.controldir import ControlDir
         from bzrlib.errors import BzrCommandError, NoWorkingTree
         from bzrlib.workingtree import WorkingTree
 
@@ -304,23 +303,24 @@
         stored_loc = source_branch.get_push_location()
         if location is None:
             if stored_loc is None:
-                raise BzrCommandError("No push location known or specified.")
+                raise BzrCommandError(gettext("No push location known or specified."))
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
-                self.outf.write("Using saved location: %s\n" % display_url)
+                self.outf.write(
+                       gettext("Using saved location: %s\n") % display_url)
                 location = stored_loc
 
-        bzrdir = BzrDir.open(location)
-        target_branch = bzrdir.open_branch()
+        controldir = ControlDir.open(location)
+        target_branch = controldir.open_branch()
         target_branch.lock_write()
         try:
             try:
-                push_result = source_branch.lossy_push(target_branch)
+                push_result = source_branch.push(target_branch, lossy=True)
             except errors.LossyPushToSameVCS:
-                raise BzrCommandError("%r and %r are in the same VCS, lossy "
-                    "push not necessary. Please use regular push." %
-                    (source_branch, target_branch))
+                raise BzrCommandError(gettext("{0!r} and {1!r} are in the same VCS, lossy "
+                    "push not necessary. Please use regular push.").format(
+                    source_branch, target_branch))
             # We successfully created the target, remember it
             if source_branch.get_push_location() is None or remember:
                 source_branch.set_push_location(target_branch.base)
@@ -339,23 +339,3 @@
             push_result.report(self.outf)
         finally:
             target_branch.unlock()
-
-
-class InterToForeignBranch(InterBranch):
-
-    def lossy_push(self, stop_revision=None):
-        """Push deltas into another branch.
-
-        :note: This does not, like push, retain the revision ids from 
-            the source branch and will, rather than adding bzr-specific 
-            metadata, push only those semantics of the revision that can be 
-            natively represented by this branch' VCS.
-
-        :param target: Target branch
-        :param stop_revision: Revision to push, defaults to last revision.
-        :return: BranchPushResult with an extra member revidmap: 
-            A dictionary mapping revision ids from the target branch 
-            to new revision ids in the target branch, for each 
-            revision that was pushed.
-        """
-        raise NotImplementedError(self.lossy_push)

=== modified file 'bzrlib/generate_ids.py'
--- bzrlib/generate_ids.py	2009-03-23 14:59:43 +0000
+++ bzrlib/generate_ids.py	2011-04-07 10:36:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import time
-import unicodedata
 
 from bzrlib import (
     config,
@@ -93,10 +92,10 @@
 def gen_revision_id(username, timestamp=None):
     """Return new revision-id.
 
-    :param username: This is the value returned by config.username(), which is
-        typically a real name, followed by an email address. If found, we will
-        use just the email address portion. Otherwise we flatten the real name,
-        and use that.
+    :param username: The username of the committer, in the format returned by
+        config.username().  This is typically a real name, followed by an
+        email address. If found, we will use just the email address portion.
+        Otherwise we flatten the real name, and use that.
     :return: A new revision id.
     """
     try:

=== modified file 'bzrlib/globbing.py'
--- bzrlib/globbing.py	2010-07-21 10:17:11 +0000
+++ bzrlib/globbing.py	2011-06-28 22:25:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,10 @@
 
 import re
 
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    lazy_regex,
+    )
 from bzrlib.trace import (
     mutter,
     warning,
@@ -38,7 +41,7 @@
     must not contain capturing groups.
     """
 
-    _expand = re.compile(ur'\\&')
+    _expand = lazy_regex.lazy_compile(ur'\\&')
 
     def __init__(self, source=None):
         self._pat = None
@@ -74,7 +77,7 @@
 
     def __call__(self, text):
         if not self._pat:
-            self._pat = re.compile(
+            self._pat = lazy_regex.lazy_compile(
                     u'|'.join([u'(%s)' % p for p in self._pats]),
                     re.UNICODE)
         return self._pat.sub(self._do_sub, text)
@@ -217,9 +220,13 @@
 
     def _add_patterns(self, patterns, translator, prefix=''):
         while patterns:
-            grouped_rules = ['(%s)' % translator(pat) for pat in patterns[:99]]
+            grouped_rules = [
+                '(%s)' % translator(pat) for pat in patterns[:99]]
             joined_rule = '%s(?:%s)$' % (prefix, '|'.join(grouped_rules))
-            self._regex_patterns.append((re.compile(joined_rule, re.UNICODE),
+            # Explicitly use lazy_compile here, because we count on its
+            # nicer error reporting.
+            self._regex_patterns.append((
+                lazy_regex.lazy_compile(joined_rule, re.UNICODE),
                 patterns[:99]))
             patterns = patterns[99:]
 
@@ -275,7 +282,7 @@
         translator = Globster.pattern_info[Globster.identify(pattern)]["translator"]
         tpattern = '(%s)' % translator(pattern)
         try:
-            re_obj = re.compile(tpattern, re.UNICODE)
+            re_obj = lazy_regex.lazy_compile(tpattern, re.UNICODE)
             re_obj.search("") # force compile
         except errors.InvalidPattern, e:
             result = False
@@ -334,7 +341,7 @@
                 Globster.pattern_info[t]["prefix"])
 
 
-_slashes = re.compile(r'[\\/]+')
+_slashes = lazy_regex.lazy_compile(r'[\\/]+')
 def normalize_pattern(pattern):
     """Converts backslashes in path patterns to forward slashes.
 

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2009-03-23 14:59:43 +0000
+++ bzrlib/gpg.py	2011-09-16 10:41:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2011 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,7 @@
 
 import os
 import sys
+from StringIO import StringIO
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -30,21 +31,49 @@
     trace,
     ui,
     )
+from bzrlib.i18n import (
+    gettext, 
+    ngettext,
+    )
 """)
 
+#verification results
+SIGNATURE_VALID = 0
+SIGNATURE_KEY_MISSING = 1
+SIGNATURE_NOT_VALID = 2
+SIGNATURE_NOT_SIGNED = 3
+SIGNATURE_EXPIRED = 4
+
 
 class DisabledGPGStrategy(object):
     """A GPG Strategy that makes everything fail."""
 
+    @staticmethod
+    def verify_signatures_available():
+        return True
+
     def __init__(self, ignored):
         """Real strategies take a configuration."""
 
     def sign(self, content):
         raise errors.SigningFailed('Signing is disabled.')
 
+    def verify(self, content, testament):
+        raise errors.SignatureVerificationFailed('Signature verification is \
+disabled.')
+
+    def set_acceptable_keys(self, command_line_input):
+        pass
+
 
 class LoopbackGPGStrategy(object):
-    """A GPG Strategy that acts like 'cat' - data is just passed through."""
+    """A GPG Strategy that acts like 'cat' - data is just passed through.
+    Used in tests.
+    """
+
+    @staticmethod
+    def verify_signatures_available():
+        return True
 
     def __init__(self, ignored):
         """Real strategies take a configuration."""
@@ -53,6 +82,64 @@
         return ("-----BEGIN PSEUDO-SIGNED CONTENT-----\n" + content +
                 "-----END PSEUDO-SIGNED CONTENT-----\n")
 
+    def verify(self, content, testament):
+        return SIGNATURE_VALID, None
+
+    def set_acceptable_keys(self, command_line_input):
+        if command_line_input is not None:
+            patterns = command_line_input.split(",")
+            self.acceptable_keys = []
+            for pattern in patterns:
+                if pattern == "unknown":
+                    pass
+                else:
+                    self.acceptable_keys.append(pattern)
+
+    def do_verifications(self, revisions, repository):
+        count = {SIGNATURE_VALID: 0,
+                 SIGNATURE_KEY_MISSING: 0,
+                 SIGNATURE_NOT_VALID: 0,
+                 SIGNATURE_NOT_SIGNED: 0,
+                 SIGNATURE_EXPIRED: 0}
+        result = []
+        all_verifiable = True
+        for rev_id in revisions:
+            verification_result, uid =\
+                                repository.verify_revision(rev_id,self)
+            result.append([rev_id, verification_result, uid])
+            count[verification_result] += 1
+            if verification_result != SIGNATURE_VALID:
+                all_verifiable = False
+        return (count, result, all_verifiable)
+
+    def valid_commits_message(self, count):
+        return gettext(u"{0} commits with valid signatures").format(
+                                        count[SIGNATURE_VALID])            
+
+    def unknown_key_message(self, count):
+        return ngettext(u"{0} commit with unknown key",
+                             u"{0} commits with unknown keys",
+                             count[SIGNATURE_KEY_MISSING]).format(
+                                        count[SIGNATURE_KEY_MISSING])
+
+    def commit_not_valid_message(self, count):
+        return ngettext(u"{0} commit not valid",
+                             u"{0} commits not valid",
+                             count[SIGNATURE_NOT_VALID]).format(
+                                            count[SIGNATURE_NOT_VALID])
+
+    def commit_not_signed_message(self, count):
+        return ngettext(u"{0} commit not signed",
+                             u"{0} commits not signed",
+                             count[SIGNATURE_NOT_SIGNED]).format(
+                                        count[SIGNATURE_NOT_SIGNED])
+
+    def expired_commit_message(self, count):
+        return ngettext(u"{0} commit with key now expired",
+                        u"{0} commits with key now expired",
+                        count[SIGNATURE_EXPIRED]).format(
+                                        count[SIGNATURE_EXPIRED])
+
 
 def _set_gpg_tty():
     tty = os.environ.get('TTY')
@@ -70,11 +157,33 @@
 class GPGStrategy(object):
     """GPG Signing and checking facilities."""
 
+    acceptable_keys = None
+
+    @staticmethod
+    def verify_signatures_available():
+        """
+        check if this strategy can verify signatures
+
+        :return: boolean if this strategy can verify signatures
+        """
+        try:
+            import gpgme
+            return True
+        except ImportError, error:
+            return False
+
     def _command_line(self):
-        return [self._config.gpg_signing_command(), '--clearsign']
+        
+        return [self._config.gpg_signing_command(), '--clearsign', '-u',
+                                                self._config.gpg_signing_key()]
 
     def __init__(self, config):
         self._config = config
+        try:
+            import gpgme
+            self.context = gpgme.Context()
+        except ImportError, error:
+            pass # can't use verify()
 
     def sign(self, content):
         if isinstance(content, unicode):
@@ -111,3 +220,264 @@
                 raise errors.SigningFailed(self._command_line())
             else:
                 raise
+
+    def verify(self, content, testament):
+        """Check content has a valid signature.
+        
+        :param content: the commit signature
+        :param testament: the valid testament string for the commit
+        
+        :return: SIGNATURE_VALID or a failed SIGNATURE_ value, key uid if valid
+        """
+        try:
+            import gpgme
+        except ImportError, error:
+            raise errors.GpgmeNotInstalled(error)
+
+        signature = StringIO(content)
+        plain_output = StringIO()
+        
+        try:
+            result = self.context.verify(signature, None, plain_output)
+        except gpgme.GpgmeError,error:
+            raise errors.SignatureVerificationFailed(error[2])
+
+        # No result if input is invalid.
+        # test_verify_invalid()
+        if len(result) == 0:
+            return SIGNATURE_NOT_VALID, None
+        # User has specified a list of acceptable keys, check our result is in it.
+        # test_verify_unacceptable_key()
+        fingerprint = result[0].fpr
+        if self.acceptable_keys is not None:
+            if not fingerprint in self.acceptable_keys:                
+                return SIGNATURE_KEY_MISSING, fingerprint[-8:]
+        # Check the signature actually matches the testament.
+        # test_verify_bad_testament()
+        if testament != plain_output.getvalue():
+            return SIGNATURE_NOT_VALID, None 
+        # Yay gpgme set the valid bit.
+        # Can't write a test for this one as you can't set a key to be
+        # trusted using gpgme.
+        if result[0].summary & gpgme.SIGSUM_VALID:
+            key = self.context.get_key(fingerprint)
+            name = key.uids[0].name
+            email = key.uids[0].email
+            return SIGNATURE_VALID, name + " <" + email + ">"
+        # Sigsum_red indicates a problem, unfortunatly I have not been able
+        # to write any tests which actually set this.
+        if result[0].summary & gpgme.SIGSUM_RED:
+            return SIGNATURE_NOT_VALID, None
+        # GPG does not know this key.
+        # test_verify_unknown_key()
+        if result[0].summary & gpgme.SIGSUM_KEY_MISSING:
+            return SIGNATURE_KEY_MISSING, fingerprint[-8:]
+        # Summary isn't set if sig is valid but key is untrusted
+        # but if user has explicity set the key as acceptable we can validate it.
+        if result[0].summary == 0 and self.acceptable_keys is not None:
+            if fingerprint in self.acceptable_keys:
+                # test_verify_untrusted_but_accepted()
+                return SIGNATURE_VALID, None 
+        # test_verify_valid_but_untrusted()
+        if result[0].summary == 0 and self.acceptable_keys is None:
+            return SIGNATURE_NOT_VALID, None
+        if result[0].summary & gpgme.SIGSUM_KEY_EXPIRED:
+            expires = self.context.get_key(result[0].fpr).subkeys[0].expires
+            if expires > result[0].timestamp:
+                # The expired key was not expired at time of signing.
+                # test_verify_expired_but_valid()
+                return SIGNATURE_EXPIRED, fingerprint[-8:]
+            else:
+                # I can't work out how to create a test where the signature
+                # was expired at the time of signing.
+                return SIGNATURE_NOT_VALID, None
+        # A signature from a revoked key gets this.
+        # test_verify_revoked_signature()
+        if result[0].summary & gpgme.SIGSUM_SYS_ERROR:
+            return SIGNATURE_NOT_VALID, None
+        # Other error types such as revoked keys should (I think) be caught by
+        # SIGSUM_RED so anything else means something is buggy.
+        raise errors.SignatureVerificationFailed("Unknown GnuPG key "\
+                                                 "verification result")
+
+    def set_acceptable_keys(self, command_line_input):
+        """sets the acceptable keys for verifying with this GPGStrategy
+        
+        :param command_line_input: comma separated list of patterns from
+                                command line
+        :return: nothing
+        """
+        key_patterns = None
+        acceptable_keys_config = self._config.acceptable_keys()
+        try:
+            if isinstance(acceptable_keys_config, unicode):
+                acceptable_keys_config = str(acceptable_keys_config)
+        except UnicodeEncodeError:
+            #gpg Context.keylist(pattern) does not like unicode
+            raise errors.BzrCommandError(gettext('Only ASCII permitted in option names'))
+
+        if acceptable_keys_config is not None:
+            key_patterns = acceptable_keys_config
+        if command_line_input is not None: #command line overrides config
+            key_patterns = command_line_input
+        if key_patterns is not None:
+            patterns = key_patterns.split(",")
+
+            self.acceptable_keys = []
+            for pattern in patterns:
+                result = self.context.keylist(pattern)
+                found_key = False
+                for key in result:
+                    found_key = True
+                    self.acceptable_keys.append(key.subkeys[0].fpr)
+                    trace.mutter("Added acceptable key: " + key.subkeys[0].fpr)
+                if not found_key:
+                    trace.note(gettext(
+                            "No GnuPG key results for pattern: {0}"
+                                ).format(pattern))
+
+    def do_verifications(self, revisions, repository,
+                            process_events_callback=None):
+        """do verifications on a set of revisions
+        
+        :param revisions: list of revision ids to verify
+        :param repository: repository object
+        :param process_events_callback: method to call for GUI frontends that
+                                                want to keep their UI refreshed
+        
+        :return: count dictionary of results of each type,
+                 result list for each revision,
+                 boolean True if all results are verified successfully
+        """
+        count = {SIGNATURE_VALID: 0,
+                 SIGNATURE_KEY_MISSING: 0,
+                 SIGNATURE_NOT_VALID: 0,
+                 SIGNATURE_NOT_SIGNED: 0,
+                 SIGNATURE_EXPIRED: 0}
+        result = []
+        all_verifiable = True
+        for rev_id in revisions:
+            verification_result, uid =\
+                                repository.verify_revision(rev_id,self)
+            result.append([rev_id, verification_result, uid])
+            count[verification_result] += 1
+            if verification_result != SIGNATURE_VALID:
+                all_verifiable = False
+            if process_events_callback is not None:
+                process_events_callback()
+        return (count, result, all_verifiable)
+
+    def verbose_valid_message(self, result):
+        """takes a verify result and returns list of signed commits strings"""
+        signers = {}
+        for rev_id, validity, uid in result:
+            if validity == SIGNATURE_VALID:
+                signers.setdefault(uid, 0)
+                signers[uid] += 1
+        result = []
+        for uid, number in signers.items():
+             result.append( ngettext(u"{0} signed {1} commit", 
+                             u"{0} signed {1} commits",
+                             number).format(uid, number) )
+        return result
+
+
+    def verbose_not_valid_message(self, result, repo):
+        """takes a verify result and returns list of not valid commit info"""
+        signers = {}
+        for rev_id, validity, empty in result:
+            if validity == SIGNATURE_NOT_VALID:
+                revision = repo.get_revision(rev_id)
+                authors = ', '.join(revision.get_apparent_authors())
+                signers.setdefault(authors, 0)
+                signers[authors] += 1
+        result = []
+        for authors, number in signers.items():
+            result.append( ngettext(u"{0} commit by author {1}", 
+                                 u"{0} commits by author {1}",
+                                 number).format(number, authors) )
+        return result
+
+    def verbose_not_signed_message(self, result, repo):
+        """takes a verify result and returns list of not signed commit info"""
+        signers = {}
+        for rev_id, validity, empty in result:
+            if validity == SIGNATURE_NOT_SIGNED:
+                revision = repo.get_revision(rev_id)
+                authors = ', '.join(revision.get_apparent_authors())
+                signers.setdefault(authors, 0)
+                signers[authors] += 1
+        result = []
+        for authors, number in signers.items():
+            result.append( ngettext(u"{0} commit by author {1}", 
+                                 u"{0} commits by author {1}",
+                                 number).format(number, authors) )
+        return result
+
+    def verbose_missing_key_message(self, result):
+        """takes a verify result and returns list of missing key info"""
+        signers = {}
+        for rev_id, validity, fingerprint in result:
+            if validity == SIGNATURE_KEY_MISSING:
+                signers.setdefault(fingerprint, 0)
+                signers[fingerprint] += 1
+        result = []
+        for fingerprint, number in signers.items():
+            result.append( ngettext(u"Unknown key {0} signed {1} commit", 
+                                 u"Unknown key {0} signed {1} commits",
+                                 number).format(fingerprint, number) )
+        return result
+
+    def verbose_expired_key_message(self, result, repo):
+        """takes a verify result and returns list of expired key info"""
+        signers = {}
+        fingerprint_to_authors = {}
+        for rev_id, validity, fingerprint in result:
+            if validity == SIGNATURE_EXPIRED:
+                revision = repo.get_revision(rev_id)
+                authors = ', '.join(revision.get_apparent_authors())
+                signers.setdefault(fingerprint, 0)
+                signers[fingerprint] += 1
+                fingerprint_to_authors[fingerprint] = authors
+        result = []
+        for fingerprint, number in signers.items():
+            result.append(ngettext(u"{0} commit by author {1} with "\
+                                    "key {2} now expired", 
+                                   u"{0} commits by author {1} with key {2} now "\
+                                    "expired",
+                                    number).format(number,
+                            fingerprint_to_authors[fingerprint], fingerprint) )
+        return result
+
+    def valid_commits_message(self, count):
+        """returns message for number of commits"""
+        return gettext(u"{0} commits with valid signatures").format(
+                                        count[SIGNATURE_VALID])
+
+    def unknown_key_message(self, count):
+        """returns message for number of commits"""
+        return ngettext(u"{0} commit with unknown key",
+                        u"{0} commits with unknown keys",
+                        count[SIGNATURE_KEY_MISSING]).format(
+                                        count[SIGNATURE_KEY_MISSING])
+
+    def commit_not_valid_message(self, count):
+        """returns message for number of commits"""
+        return ngettext(u"{0} commit not valid",
+                        u"{0} commits not valid",
+                        count[SIGNATURE_NOT_VALID]).format(
+                                            count[SIGNATURE_NOT_VALID])
+
+    def commit_not_signed_message(self, count):
+        """returns message for number of commits"""
+        return ngettext(u"{0} commit not signed",
+                        u"{0} commits not signed",
+                        count[SIGNATURE_NOT_SIGNED]).format(
+                                        count[SIGNATURE_NOT_SIGNED])
+
+    def expired_commit_message(self, count):
+        """returns message for number of commits"""
+        return ngettext(u"{0} commit with key now expired",
+                        u"{0} commits with key now expired",
+                        count[SIGNATURE_EXPIRED]).format(
+                                    count[SIGNATURE_EXPIRED])

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2010-04-10 01:22:00 +0000
+++ bzrlib/graph.py	2011-08-26 11:35:50 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
     revision,
     trace,
     )
-from bzrlib.symbol_versioning import deprecated_function, deprecated_in
 
 STEP_UNIQUE_SEARCHER_EVERY = 5
 
@@ -59,21 +58,23 @@
     def __repr__(self):
         return 'DictParentsProvider(%r)' % self.ancestry
 
+    # Note: DictParentsProvider does not implement get_cached_parent_map
+    #       Arguably, the data is clearly cached in memory. However, this class
+    #       is mostly used for testing, and it keeps the tests clean to not
+    #       change it.
+
     def get_parent_map(self, keys):
         """See StackedParentsProvider.get_parent_map"""
         ancestry = self.ancestry
-        return dict((k, ancestry[k]) for k in keys if k in ancestry)
+        return dict([(k, ancestry[k]) for k in keys if k in ancestry])
 
-@deprecated_function(deprecated_in((1, 16, 0)))
-def _StackedParentsProvider(*args, **kwargs):
-    return StackedParentsProvider(*args, **kwargs)
 
 class StackedParentsProvider(object):
     """A parents provider which stacks (or unions) multiple providers.
-    
+
     The providers are queries in the order of the provided parent_providers.
     """
-    
+
     def __init__(self, parent_providers):
         self._parent_providers = parent_providers
 
@@ -95,6 +96,22 @@
         """
         found = {}
         remaining = set(keys)
+        # This adds getattr() overhead to each get_parent_map call. However,
+        # this is StackedParentsProvider, which means we're dealing with I/O
+        # (either local indexes, or remote RPCs), so CPU overhead should be
+        # minimal.
+        for parents_provider in self._parent_providers:
+            get_cached = getattr(parents_provider, 'get_cached_parent_map',
+                                 None)
+            if get_cached is None:
+                continue
+            new_found = get_cached(remaining)
+            found.update(new_found)
+            remaining.difference_update(new_found)
+            if not remaining:
+                break
+        if not remaining:
+            return found
         for parents_provider in self._parent_providers:
             new_found = parents_provider.get_parent_map(remaining)
             found.update(new_found)
@@ -154,6 +171,17 @@
             return None
         return dict(self._cache)
 
+    def get_cached_parent_map(self, keys):
+        """Return items from the cache.
+
+        This returns the same info as get_parent_map, but explicitly does not
+        invoke the supplied ParentsProvider to search for uncached values.
+        """
+        cache = self._cache
+        if cache is None:
+            return {}
+        return dict([(key, cache[key]) for key in keys if key in cache])
+
     def get_parent_map(self, keys):
         """See StackedParentsProvider.get_parent_map."""
         cache = self._cache
@@ -183,6 +211,23 @@
             self.missing_keys.add(key)
 
 
+class CallableToParentsProviderAdapter(object):
+    """A parents provider that adapts any callable to the parents provider API.
+
+    i.e. it accepts calls to self.get_parent_map and relays them to the
+    callable it was constructed with.
+    """
+
+    def __init__(self, a_callable):
+        self.callable = a_callable
+
+    def __repr__(self):
+        return "%s(%r)" % (self.__class__.__name__, self.callable)
+
+    def get_parent_map(self, keys):
+        return self.callable(keys)
+
+
 class Graph(object):
     """Provide incremental access to revision graphs.
 
@@ -237,7 +282,8 @@
         common ancestor of all border ancestors, because this shows that it
         cannot be a descendant of any border ancestor.
 
-        The scaling of this operation should be proportional to
+        The scaling of this operation should be proportional to:
+
         1. The number of uncommon ancestors
         2. The number of border ancestors
         3. The length of the shortest path between a border ancestor and an
@@ -258,6 +304,40 @@
         right = searchers[1].seen
         return (left.difference(right), right.difference(left))
 
+    def find_descendants(self, old_key, new_key):
+        """Find descendants of old_key that are ancestors of new_key."""
+        child_map = self.get_child_map(self._find_descendant_ancestors(
+            old_key, new_key))
+        graph = Graph(DictParentsProvider(child_map))
+        searcher = graph._make_breadth_first_searcher([old_key])
+        list(searcher)
+        return searcher.seen
+
+    def _find_descendant_ancestors(self, old_key, new_key):
+        """Find ancestors of new_key that may be descendants of old_key."""
+        stop = self._make_breadth_first_searcher([old_key])
+        descendants = self._make_breadth_first_searcher([new_key])
+        for revisions in descendants:
+            old_stop = stop.seen.intersection(revisions)
+            descendants.stop_searching_any(old_stop)
+            seen_stop = descendants.find_seen_ancestors(stop.step())
+            descendants.stop_searching_any(seen_stop)
+        return descendants.seen.difference(stop.seen)
+
+    def get_child_map(self, keys):
+        """Get a mapping from parents to children of the specified keys.
+
+        This is simply the inversion of get_parent_map.  Only supplied keys
+        will be discovered as children.
+        :return: a dict of key:child_list for keys.
+        """
+        parent_map = self._parents_provider.get_parent_map(keys)
+        parent_child = {}
+        for child, parents in sorted(parent_map.items()):
+            for parent in parents:
+                parent_child.setdefault(parent, []).append(child)
+        return parent_child
+
     def find_distance_to_null(self, target_revision_id, known_revision_ids):
         """Find the left-hand distance to the NULL_REVISION.
 
@@ -341,8 +421,8 @@
 
         :param unique_revision: The revision_id whose ancestry we are
             interested in.
-            XXX: Would this API be better if we allowed multiple revisions on
-                 to be searched here?
+            (XXX: Would this API be better if we allowed multiple revisions on
+            to be searched here?)
         :param common_revisions: Revision_ids of ancestries to exclude.
         :return: A set of revisions in the ancestry of unique_revision
         """
@@ -862,6 +942,26 @@
                 stop.add(parent_id)
         return found
 
+    def find_lefthand_merger(self, merged_key, tip_key):
+        """Find the first lefthand ancestor of tip_key that merged merged_key.
+
+        We do this by first finding the descendants of merged_key, then
+        walking through the lefthand ancestry of tip_key until we find a key
+        that doesn't descend from merged_key.  Its child is the key that
+        merged merged_key.
+
+        :return: The first lefthand ancestor of tip_key to merge merged_key.
+            merged_key if it is a lefthand ancestor of tip_key.
+            None if no ancestor of tip_key merged merged_key.
+        """
+        descendants = self.find_descendants(merged_key, tip_key)
+        candidate_iterator = self.iter_lefthand_ancestry(tip_key)
+        last_candidate = None
+        for candidate in candidate_iterator:
+            if candidate not in descendants:
+                return last_candidate
+            last_candidate = candidate
+
     def find_unique_lca(self, left_revision, right_revision,
                         count_steps=False):
         """Find a unique LCA.
@@ -919,6 +1019,25 @@
                 yield (ghost, None)
             pending = next_pending
 
+    def iter_lefthand_ancestry(self, start_key, stop_keys=None):
+        if stop_keys is None:
+            stop_keys = ()
+        next_key = start_key
+        def get_parents(key):
+            try:
+                return self._parents_provider.get_parent_map([key])[key]
+            except KeyError:
+                raise errors.RevisionNotPresent(next_key, self)
+        while True:
+            if next_key in stop_keys:
+                return
+            parents = get_parents(next_key)
+            yield next_key
+            if len(parents) == 0:
+                return
+            else:
+                next_key = parents[0]
+
     def iter_topo_order(self, revisions):
         """Iterate through the input revisions in topological order.
 
@@ -1332,13 +1451,14 @@
         parents_of_found = set()
         # revisions may contain nodes that point to other nodes in revisions:
         # we want to filter them out.
-        self.seen.update(revisions)
+        seen = self.seen
+        seen.update(revisions)
         parent_map = self._parents_provider.get_parent_map(revisions)
         found_revisions.update(parent_map)
         for rev_id, parents in parent_map.iteritems():
             if parents is None:
                 continue
-            new_found_parents = [p for p in parents if p not in self.seen]
+            new_found_parents = [p for p in parents if p not in seen]
             if new_found_parents:
                 # Calling set.update() with an empty generator is actually
                 # rather expensive.
@@ -1463,7 +1583,69 @@
             return revs, ghosts
 
 
-class SearchResult(object):
+class AbstractSearchResult(object):
+    """The result of a search, describing a set of keys.
+    
+    Search results are typically used as the 'fetch_spec' parameter when
+    fetching revisions.
+
+    :seealso: AbstractSearch
+    """
+
+    def get_recipe(self):
+        """Return a recipe that can be used to replay this search.
+
+        The recipe allows reconstruction of the same results at a later date.
+
+        :return: A tuple of `(search_kind_str, *details)`.  The details vary by
+            kind of search result.
+        """
+        raise NotImplementedError(self.get_recipe)
+
+    def get_network_struct(self):
+        """Return a tuple that can be transmitted via the HPSS protocol."""
+        raise NotImplementedError(self.get_network_struct)
+
+    def get_keys(self):
+        """Return the keys found in this search.
+
+        :return: A set of keys.
+        """
+        raise NotImplementedError(self.get_keys)
+
+    def is_empty(self):
+        """Return false if the search lists 1 or more revisions."""
+        raise NotImplementedError(self.is_empty)
+
+    def refine(self, seen, referenced):
+        """Create a new search by refining this search.
+
+        :param seen: Revisions that have been satisfied.
+        :param referenced: Revision references observed while satisfying some
+            of this search.
+        :return: A search result.
+        """
+        raise NotImplementedError(self.refine)
+
+
+class AbstractSearch(object):
+    """A search that can be executed, producing a search result.
+
+    :seealso: AbstractSearchResult
+    """
+
+    def execute(self):
+        """Construct a network-ready search result from this search description.
+
+        This may take some time to search repositories, etc.
+
+        :return: A search result (an object that implements
+            AbstractSearchResult's API).
+        """
+        raise NotImplementedError(self.execute)
+
+
+class SearchResult(AbstractSearchResult):
     """The result of a breadth first search.
 
     A SearchResult provides the ability to reconstruct the search or access a
@@ -1484,6 +1666,19 @@
         self._recipe = ('search', start_keys, exclude_keys, key_count)
         self._keys = frozenset(keys)
 
+    def __repr__(self):
+        kind, start_keys, exclude_keys, key_count = self._recipe
+        if len(start_keys) > 5:
+            start_keys_repr = repr(list(start_keys)[:5])[:-1] + ', ...]'
+        else:
+            start_keys_repr = repr(start_keys)
+        if len(exclude_keys) > 5:
+            exclude_keys_repr = repr(list(exclude_keys)[:5])[:-1] + ', ...]'
+        else:
+            exclude_keys_repr = repr(exclude_keys)
+        return '<%s %s:(%s, %s, %d)>' % (self.__class__.__name__,
+            kind, start_keys_repr, exclude_keys_repr, key_count)
+
     def get_recipe(self):
         """Return a recipe that can be used to replay this search.
 
@@ -1507,6 +1702,12 @@
         """
         return self._recipe
 
+    def get_network_struct(self):
+        start_keys = ' '.join(self._recipe[1])
+        stop_keys = ' '.join(self._recipe[2])
+        count = str(self._recipe[3])
+        return (self._recipe[0], '\n'.join((start_keys, stop_keys, count)))
+
     def get_keys(self):
         """Return the keys found in this search.
 
@@ -1544,7 +1745,7 @@
         return SearchResult(pending_refs, exclude, count, keys)
 
 
-class PendingAncestryResult(object):
+class PendingAncestryResult(AbstractSearchResult):
     """A search result that will reconstruct the ancestry for some graph heads.
 
     Unlike SearchResult, this doesn't hold the complete search result in
@@ -1561,6 +1762,15 @@
         self.heads = frozenset(heads)
         self.repo = repo
 
+    def __repr__(self):
+        if len(self.heads) > 5:
+            heads_repr = repr(list(self.heads)[:5])[:-1]
+            heads_repr += ', <%d more>...]' % (len(self.heads) - 5,)
+        else:
+            heads_repr = repr(self.heads)
+        return '<%s heads:%s repo:%r>' % (
+            self.__class__.__name__, heads_repr, self.repo)
+
     def get_recipe(self):
         """Return a recipe that can be used to replay this search.
 
@@ -1574,6 +1784,11 @@
         """
         return ('proxy-search', self.heads, set(), -1)
 
+    def get_network_struct(self):
+        parts = ['ancestry-of']
+        parts.extend(self.heads)
+        return parts
+
     def get_keys(self):
         """See SearchResult.get_keys.
 
@@ -1606,6 +1821,263 @@
         return PendingAncestryResult(referenced - seen, self.repo)
 
 
+class EmptySearchResult(AbstractSearchResult):
+    """An empty search result."""
+
+    def is_empty(self):
+        return True
+    
+
+class EverythingResult(AbstractSearchResult):
+    """A search result that simply requests everything in the repository."""
+
+    def __init__(self, repo):
+        self._repo = repo
+
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, self._repo)
+
+    def get_recipe(self):
+        raise NotImplementedError(self.get_recipe)
+
+    def get_network_struct(self):
+        return ('everything',)
+
+    def get_keys(self):
+        if 'evil' in debug.debug_flags:
+            from bzrlib import remote
+            if isinstance(self._repo, remote.RemoteRepository):
+                # warn developers (not users) not to do this
+                trace.mutter_callsite(
+                    2, "EverythingResult(RemoteRepository).get_keys() is slow.")
+        return self._repo.all_revision_ids()
+
+    def is_empty(self):
+        # It's ok for this to wrongly return False: the worst that can happen
+        # is that RemoteStreamSource will initiate a get_stream on an empty
+        # repository.  And almost all repositories are non-empty.
+        return False
+
+    def refine(self, seen, referenced):
+        heads = set(self._repo.all_revision_ids())
+        heads.difference_update(seen)
+        heads.update(referenced)
+        return PendingAncestryResult(heads, self._repo)
+
+
+class EverythingNotInOther(AbstractSearch):
+    """Find all revisions in that are in one repo but not the other."""
+
+    def __init__(self, to_repo, from_repo, find_ghosts=False):
+        self.to_repo = to_repo
+        self.from_repo = from_repo
+        self.find_ghosts = find_ghosts
+
+    def execute(self):
+        return self.to_repo.search_missing_revision_ids(
+            self.from_repo, find_ghosts=self.find_ghosts)
+
+
+class NotInOtherForRevs(AbstractSearch):
+    """Find all revisions missing in one repo for a some specific heads."""
+
+    def __init__(self, to_repo, from_repo, required_ids, if_present_ids=None,
+            find_ghosts=False, limit=None):
+        """Constructor.
+
+        :param required_ids: revision IDs of heads that must be found, or else
+            the search will fail with NoSuchRevision.  All revisions in their
+            ancestry not already in the other repository will be included in
+            the search result.
+        :param if_present_ids: revision IDs of heads that may be absent in the
+            source repository.  If present, then their ancestry not already
+            found in other will be included in the search result.
+        :param limit: maximum number of revisions to fetch
+        """
+        self.to_repo = to_repo
+        self.from_repo = from_repo
+        self.find_ghosts = find_ghosts
+        self.required_ids = required_ids
+        self.if_present_ids = if_present_ids
+        self.limit = limit
+
+    def __repr__(self):
+        if len(self.required_ids) > 5:
+            reqd_revs_repr = repr(list(self.required_ids)[:5])[:-1] + ', ...]'
+        else:
+            reqd_revs_repr = repr(self.required_ids)
+        if self.if_present_ids and len(self.if_present_ids) > 5:
+            ifp_revs_repr = repr(list(self.if_present_ids)[:5])[:-1] + ', ...]'
+        else:
+            ifp_revs_repr = repr(self.if_present_ids)
+
+        return ("<%s from:%r to:%r find_ghosts:%r req'd:%r if-present:%r"
+                "limit:%r>") % (
+                self.__class__.__name__, self.from_repo, self.to_repo,
+                self.find_ghosts, reqd_revs_repr, ifp_revs_repr,
+                self.limit)
+
+    def execute(self):
+        return self.to_repo.search_missing_revision_ids(
+            self.from_repo, revision_ids=self.required_ids,
+            if_present_ids=self.if_present_ids, find_ghosts=self.find_ghosts,
+            limit=self.limit)
+
+
+def invert_parent_map(parent_map):
+    """Given a map from child => parents, create a map of parent=>children"""
+    child_map = {}
+    for child, parents in parent_map.iteritems():
+        for p in parents:
+            # Any given parent is likely to have only a small handful
+            # of children, many will have only one. So we avoid mem overhead of
+            # a list, in exchange for extra copying of tuples
+            if p not in child_map:
+                child_map[p] = (child,)
+            else:
+                child_map[p] = child_map[p] + (child,)
+    return child_map
+
+
+def _find_possible_heads(parent_map, tip_keys, depth):
+    """Walk backwards (towards children) through the parent_map.
+
+    This finds 'heads' that will hopefully succinctly describe our search
+    graph.
+    """
+    child_map = invert_parent_map(parent_map)
+    heads = set()
+    current_roots = tip_keys
+    walked = set(current_roots)
+    while current_roots and depth > 0:
+        depth -= 1
+        children = set()
+        children_update = children.update
+        for p in current_roots:
+            # Is it better to pre- or post- filter the children?
+            try:
+                children_update(child_map[p])
+            except KeyError:
+                heads.add(p)
+        # If we've seen a key before, we don't want to walk it again. Note that
+        # 'children' stays relatively small while 'walked' grows large. So
+        # don't use 'difference_update' here which has to walk all of 'walked'.
+        # '.difference' is smart enough to walk only children and compare it to
+        # walked.
+        children = children.difference(walked)
+        walked.update(children)
+        current_roots = children
+    if current_roots:
+        # We walked to the end of depth, so these are the new tips.
+        heads.update(current_roots)
+    return heads
+
+
+def _run_search(parent_map, heads, exclude_keys):
+    """Given a parent map, run a _BreadthFirstSearcher on it.
+
+    Start at heads, walk until you hit exclude_keys. As a further improvement,
+    watch for any heads that you encounter while walking, which means they were
+    not heads of the search.
+
+    This is mostly used to generate a succinct recipe for how to walk through
+    most of parent_map.
+
+    :return: (_BreadthFirstSearcher, set(heads_encountered_by_walking))
+    """
+    g = Graph(DictParentsProvider(parent_map))
+    s = g._make_breadth_first_searcher(heads)
+    found_heads = set()
+    while True:
+        try:
+            next_revs = s.next()
+        except StopIteration:
+            break
+        for parents in s._current_parents.itervalues():
+            f_heads = heads.intersection(parents)
+            if f_heads:
+                found_heads.update(f_heads)
+        stop_keys = exclude_keys.intersection(next_revs)
+        if stop_keys:
+            s.stop_searching_any(stop_keys)
+    for parents in s._current_parents.itervalues():
+        f_heads = heads.intersection(parents)
+        if f_heads:
+            found_heads.update(f_heads)
+    return s, found_heads
+
+
+def limited_search_result_from_parent_map(parent_map, missing_keys, tip_keys,
+                                          depth):
+    """Transform a parent_map that is searching 'tip_keys' into an
+    approximate SearchResult.
+
+    We should be able to generate a SearchResult from a given set of starting
+    keys, that covers a subset of parent_map that has the last step pointing at
+    tip_keys. This is to handle the case that really-long-searches shouldn't be
+    started from scratch on each get_parent_map request, but we *do* want to
+    filter out some of the keys that we've already seen, so we don't get
+    information that we already know about on every request.
+
+    The server will validate the search (that starting at start_keys and
+    stopping at stop_keys yields the exact key_count), so we have to be careful
+    to give an exact recipe.
+
+    Basic algorithm is:
+        1) Invert parent_map to get child_map (todo: have it cached and pass it
+           in)
+        2) Starting at tip_keys, walk towards children for 'depth' steps.
+        3) At that point, we have the 'start' keys.
+        4) Start walking parent_map from 'start' keys, counting how many keys
+           are seen, and generating stop_keys for anything that would walk
+           outside of the parent_map.
+
+    :param parent_map: A map from {child_id: (parent_ids,)}
+    :param missing_keys: parent_ids that we know are unavailable
+    :param tip_keys: the revision_ids that we are searching
+    :param depth: How far back to walk.
+    """
+    if not parent_map:
+        # No search to send, because we haven't done any searching yet.
+        return [], [], 0
+    heads = _find_possible_heads(parent_map, tip_keys, depth)
+    s, found_heads = _run_search(parent_map, heads, set(tip_keys))
+    _, start_keys, exclude_keys, key_count = s.get_result().get_recipe()
+    if found_heads:
+        # Anything in found_heads are redundant start_keys, we hit them while
+        # walking, so we can exclude them from the start list.
+        start_keys = set(start_keys).difference(found_heads)
+    return start_keys, exclude_keys, key_count
+
+
+def search_result_from_parent_map(parent_map, missing_keys):
+    """Transform a parent_map into SearchResult information."""
+    if not parent_map:
+        # parent_map is empty or None, simple search result
+        return [], [], 0
+    # start_set is all the keys in the cache
+    start_set = set(parent_map)
+    # result set is all the references to keys in the cache
+    result_parents = set()
+    for parents in parent_map.itervalues():
+        result_parents.update(parents)
+    stop_keys = result_parents.difference(start_set)
+    # We don't need to send ghosts back to the server as a position to
+    # stop either.
+    stop_keys.difference_update(missing_keys)
+    key_count = len(parent_map)
+    if (revision.NULL_REVISION in result_parents
+        and revision.NULL_REVISION in missing_keys):
+        # If we pruned NULL_REVISION from the stop_keys because it's also
+        # in our cache of "missing" keys we need to increment our key count
+        # by 1, because the reconsitituted SearchResult on the server will
+        # still consider NULL_REVISION to be an included key.
+        key_count += 1
+    included_keys = start_set.intersection(result_parents)
+    start_set.difference_update(included_keys)
+    return start_set, stop_keys, key_count
+
+
 def collapse_linear_regions(parent_map):
     """Collapse regions of the graph that are 'linear'.
 
@@ -1695,7 +2167,13 @@
         return set([h[0] for h in head_keys])
 
     def merge_sort(self, tip_revision):
-        return self._graph.merge_sort((tip_revision,))
+        nodes = self._graph.merge_sort((tip_revision,))
+        for node in nodes:
+            node.key = node.key[0]
+        return nodes
+
+    def add_node(self, revision, parents):
+        self._graph.add_node((revision,), [(p,) for p in parents])
 
 
 _counters = [0,0,0,0,0,0,0]

=== modified file 'bzrlib/groupcompress.py'
--- bzrlib/groupcompress.py	2010-05-20 02:57:52 +0000
+++ bzrlib/groupcompress.py	2011-09-15 11:31:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,26 +23,34 @@
 except ImportError:
     pylzma = None
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from bzrlib import (
     annotate,
+    config,
     debug,
     errors,
     graph as _mod_graph,
-    knit,
     osutils,
     pack,
     static_tuple,
     trace,
+    tsort,
     )
+
+from bzrlib.repofmt import pack_repo
+from bzrlib.i18n import gettext
+""")
+
 from bzrlib.btree_index import BTreeBuilder
 from bzrlib.lru_cache import LRUSizeCache
-from bzrlib.tsort import topo_sort
 from bzrlib.versionedfile import (
+    _KeyRefs,
     adapter_registry,
     AbsentContentFactory,
     ChunkedContentFactory,
     FulltextContentFactory,
-    VersionedFiles,
+    VersionedFilesWithFallbacks,
     )
 
 # Minimum number of uncompressed bytes to try fetch at once when retrieving
@@ -77,7 +85,7 @@
 
     present_keys = []
     for prefix in sorted(per_prefix_map):
-        present_keys.extend(reversed(topo_sort(per_prefix_map[prefix])))
+        present_keys.extend(reversed(tsort.topo_sort(per_prefix_map[prefix])))
     return present_keys
 
 
@@ -101,7 +109,7 @@
     def __init__(self):
         # map by key? or just order in file?
         self._compressor_name = None
-        self._z_content = None
+        self._z_content_chunks = None
         self._z_content_decompressor = None
         self._z_content_length = None
         self._content_length = None
@@ -135,26 +143,30 @@
                 self._content = ''.join(self._content_chunks)
                 self._content_chunks = None
         if self._content is None:
-            if self._z_content is None:
+            # We join self._z_content_chunks here, because if we are
+            # decompressing, then it is *very* likely that we have a single
+            # chunk
+            if self._z_content_chunks is None:
                 raise AssertionError('No content to decompress')
-            if self._z_content == '':
+            z_content = ''.join(self._z_content_chunks)
+            if z_content == '':
                 self._content = ''
             elif self._compressor_name == 'lzma':
                 # We don't do partial lzma decomp yet
-                self._content = pylzma.decompress(self._z_content)
+                self._content = pylzma.decompress(z_content)
             elif self._compressor_name == 'zlib':
                 # Start a zlib decompressor
                 if num_bytes * 4 > self._content_length * 3:
                     # If we are requesting more that 3/4ths of the content,
                     # just extract the whole thing in a single pass
                     num_bytes = self._content_length
-                    self._content = zlib.decompress(self._z_content)
+                    self._content = zlib.decompress(z_content)
                 else:
                     self._z_content_decompressor = zlib.decompressobj()
                     # Seed the decompressor with the uncompressed bytes, so
                     # that the rest of the code is simplified
                     self._content = self._z_content_decompressor.decompress(
-                        self._z_content, num_bytes + _ZLIB_DECOMP_WINDOW)
+                        z_content, num_bytes + _ZLIB_DECOMP_WINDOW)
                     if not self._z_content_decompressor.unconsumed_tail:
                         self._z_content_decompressor = None
             else:
@@ -207,7 +219,17 @@
             # XXX: Define some GCCorrupt error ?
             raise AssertionError('Invalid bytes: (%d) != %d + %d' %
                                  (len(bytes), pos, self._z_content_length))
-        self._z_content = bytes[pos:]
+        self._z_content_chunks = (bytes[pos:],)
+
+    @property
+    def _z_content(self):
+        """Return z_content_chunks as a simple string.
+
+        Meant only to be used by the test suite.
+        """
+        if self._z_content_chunks is not None:
+            return ''.join(self._z_content_chunks)
+        return None
 
     @classmethod
     def from_bytes(cls, bytes):
@@ -269,13 +291,13 @@
         self._content_length = length
         self._content_chunks = content_chunks
         self._content = None
-        self._z_content = None
+        self._z_content_chunks = None
 
     def set_content(self, content):
         """Set the content of this block."""
         self._content_length = len(content)
         self._content = content
-        self._z_content = None
+        self._z_content_chunks = None
 
     def _create_z_content_using_lzma(self):
         if self._content_chunks is not None:
@@ -283,39 +305,49 @@
             self._content_chunks = None
         if self._content is None:
             raise AssertionError('Nothing to compress')
-        self._z_content = pylzma.compress(self._content)
-        self._z_content_length = len(self._z_content)
+        z_content = pylzma.compress(self._content)
+        self._z_content_chunks = (z_content,)
+        self._z_content_length = len(z_content)
 
-    def _create_z_content_from_chunks(self):
+    def _create_z_content_from_chunks(self, chunks):
         compressor = zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION)
-        compressed_chunks = map(compressor.compress, self._content_chunks)
+        # Peak in this point is 1 fulltext, 1 compressed text, + zlib overhead
+        # (measured peak is maybe 30MB over the above...)
+        compressed_chunks = map(compressor.compress, chunks)
         compressed_chunks.append(compressor.flush())
-        self._z_content = ''.join(compressed_chunks)
-        self._z_content_length = len(self._z_content)
+        # Ignore empty chunks
+        self._z_content_chunks = [c for c in compressed_chunks if c]
+        self._z_content_length = sum(map(len, self._z_content_chunks))
 
     def _create_z_content(self):
-        if self._z_content is not None:
+        if self._z_content_chunks is not None:
             return
         if _USE_LZMA:
             self._create_z_content_using_lzma()
             return
         if self._content_chunks is not None:
-            self._create_z_content_from_chunks()
-            return
-        self._z_content = zlib.compress(self._content)
-        self._z_content_length = len(self._z_content)
+            chunks = self._content_chunks
+        else:
+            chunks = (self._content,)
+        self._create_z_content_from_chunks(chunks)
 
-    def to_bytes(self):
-        """Encode the information into a byte stream."""
+    def to_chunks(self):
+        """Create the byte stream as a series of 'chunks'"""
         self._create_z_content()
         if _USE_LZMA:
             header = self.GCB_LZ_HEADER
         else:
             header = self.GCB_HEADER
-        chunks = [header,
-                  '%d\n%d\n' % (self._z_content_length, self._content_length),
-                  self._z_content,
+        chunks = ['%s%d\n%d\n'
+                  % (header, self._z_content_length, self._content_length),
                  ]
+        chunks.extend(self._z_content_chunks)
+        total_len = sum(map(len, chunks))
+        return total_len, chunks
+
+    def to_bytes(self):
+        """Encode the information into a byte stream."""
+        total_len, chunks = self.to_chunks()
         return ''.join(chunks)
 
     def _dump(self, include_text=False):
@@ -435,7 +467,10 @@
                 # Grab and cache the raw bytes for this entry
                 # and break the ref-cycle with _manager since we don't need it
                 # anymore
-                self._manager._prepare_for_extract()
+                try:
+                    self._manager._prepare_for_extract()
+                except zlib.error as value:
+                    raise errors.DecompressCorruption("zlib: " + str(value))
                 block = self._manager._block
                 self._bytes = block.extract(self.key, self._start, self._end)
                 # There are code paths that first extract as fulltext, and then
@@ -460,11 +495,25 @@
     _full_enough_block_size = 3*1024*1024 # size at which we won't repack
     _full_enough_mixed_block_size = 2*768*1024 # 1.5MB
 
-    def __init__(self, block):
+    def __init__(self, block, get_compressor_settings=None):
         self._block = block
         # We need to preserve the ordering
         self._factories = []
         self._last_byte = 0
+        self._get_settings = get_compressor_settings
+        self._compressor_settings = None
+
+    def _get_compressor_settings(self):
+        if self._compressor_settings is not None:
+            return self._compressor_settings
+        settings = None
+        if self._get_settings is not None:
+            settings = self._get_settings()
+        if settings is None:
+            vf = GroupCompressVersionedFiles
+            settings = vf._DEFAULT_COMPRESSOR_SETTINGS
+        self._compressor_settings = settings
+        return self._compressor_settings
 
     def add_factory(self, key, parents, start, end):
         if not self._factories:
@@ -503,9 +552,12 @@
         new_block.set_content(self._block._content[:last_byte])
         self._block = new_block
 
+    def _make_group_compressor(self):
+        return GroupCompressor(self._get_compressor_settings())
+
     def _rebuild_block(self):
         """Create a new GroupCompressBlock with only the referenced texts."""
-        compressor = GroupCompressor()
+        compressor = self._make_group_compressor()
         tstart = time.time()
         old_length = self._block._content_length
         end_point = 0
@@ -523,6 +575,11 @@
         #       block? It seems hard to come up with a method that it would
         #       expand, since we do full compression again. Perhaps based on a
         #       request that ends up poorly ordered?
+        # TODO: If the content would have expanded, then we would want to
+        #       handle a case where we need to split the block.
+        #       Now that we have a user-tweakable option
+        #       (max_bytes_to_index), it is possible that one person set it
+        #       to a very low value, causing poor compression.
         delta = time.time() - tstart
         self._block = new_block
         trace.mutter('creating new compressed block on-the-fly in %.3fs'
@@ -679,18 +736,21 @@
         z_header_bytes = zlib.compress(header_bytes)
         del header_bytes
         z_header_bytes_len = len(z_header_bytes)
-        block_bytes = self._block.to_bytes()
+        block_bytes_len, block_chunks = self._block.to_chunks()
         lines.append('%d\n%d\n%d\n' % (z_header_bytes_len, header_bytes_len,
-                                       len(block_bytes)))
+                                       block_bytes_len))
         lines.append(z_header_bytes)
-        lines.append(block_bytes)
-        del z_header_bytes, block_bytes
+        lines.extend(block_chunks)
+        del z_header_bytes, block_chunks
+        # TODO: This is a point where we will double the memory consumption. To
+        #       avoid this, we probably have to switch to a 'chunked' api
         return ''.join(lines)
 
     @classmethod
     def from_bytes(cls, bytes):
         # TODO: This does extra string copying, probably better to do it a
-        #       different way
+        #       different way. At a minimum this creates 2 copies of the
+        #       compressed content
         (storage_kind, z_header_len, header_len,
          block_len, rest) = bytes.split('\n', 4)
         del bytes
@@ -748,7 +808,7 @@
 
 class _CommonGroupCompressor(object):
 
-    def __init__(self):
+    def __init__(self, settings=None):
         """Create a GroupCompressor."""
         self.chunks = []
         self._last = None
@@ -757,6 +817,10 @@
         self.labels_deltas = {}
         self._delta_index = None # Set by the children
         self._block = GroupCompressBlock()
+        if settings is None:
+            self._settings = {}
+        else:
+            self._settings = settings
 
     def compress(self, key, bytes, expected_sha, nostore_sha=None, soft=False):
         """Compress lines with label key.
@@ -854,14 +918,6 @@
 
         After calling this, the compressor should no longer be used
         """
-        # TODO: this causes us to 'bloat' to 2x the size of content in the
-        #       group. This has an impact for 'commit' of large objects.
-        #       One possibility is to use self._content_chunks, and be lazy and
-        #       only fill out self._content as a full string when we actually
-        #       need it. That would at least drop the peak memory consumption
-        #       for 'commit' down to ~1x the size of the largest file, at a
-        #       cost of increased complexity within this code. 2x is still <<
-        #       3x the size of the largest file, so we are doing ok.
         self._block.set_chunked_content(self.chunks, self.endpoint)
         self.chunks = None
         self._delta_index = None
@@ -885,12 +941,12 @@
 
 class PythonGroupCompressor(_CommonGroupCompressor):
 
-    def __init__(self):
+    def __init__(self, settings=None):
         """Create a GroupCompressor.
 
         Used only if the pyrex version is not available.
         """
-        super(PythonGroupCompressor, self).__init__()
+        super(PythonGroupCompressor, self).__init__(settings)
         self._delta_index = LinesDeltaIndex([])
         # The actual content is managed by LinesDeltaIndex
         self.chunks = self._delta_index.lines
@@ -933,20 +989,22 @@
 
     It contains code very similar to SequenceMatcher because of having a similar
     task. However some key differences apply:
-     - there is no junk, we want a minimal edit not a human readable diff.
-     - we don't filter very common lines (because we don't know where a good
-       range will start, and after the first text we want to be emitting minmal
-       edits only.
-     - we chain the left side, not the right side
-     - we incrementally update the adjacency matrix as new lines are provided.
-     - we look for matches in all of the left side, so the routine which does
-       the analagous task of find_longest_match does not need to filter on the
-       left side.
+
+    * there is no junk, we want a minimal edit not a human readable diff.
+    * we don't filter very common lines (because we don't know where a good
+      range will start, and after the first text we want to be emitting minmal
+      edits only.
+    * we chain the left side, not the right side
+    * we incrementally update the adjacency matrix as new lines are provided.
+    * we look for matches in all of the left side, so the routine which does
+      the analagous task of find_longest_match does not need to filter on the
+      left side.
     """
 
-    def __init__(self):
-        super(PyrexGroupCompressor, self).__init__()
-        self._delta_index = DeltaIndex()
+    def __init__(self, settings=None):
+        super(PyrexGroupCompressor, self).__init__(settings)
+        max_bytes_to_index = self._settings.get('max_bytes_to_index', 0)
+        self._delta_index = DeltaIndex(max_bytes_to_index=max_bytes_to_index)
 
     def _compress(self, key, bytes, max_delta_size, soft=False):
         """see _CommonGroupCompressor._compress"""
@@ -1027,7 +1085,7 @@
         index = _GCGraphIndex(graph_index, lambda:True, parents=parents,
             add_callback=graph_index.add_nodes,
             inconsistency_fatal=inconsistency_fatal)
-        access = knit._DirectPackAccess({})
+        access = pack_repo._DirectPackAccess({})
         access.set_writer(writer, graph_index, (transport, 'newpack'))
         result = GroupCompressVersionedFiles(index, access, delta)
         result.stream = stream
@@ -1043,12 +1101,12 @@
 
 class _BatchingBlockFetcher(object):
     """Fetch group compress blocks in batches.
-    
+
     :ivar total_bytes: int of expected number of bytes needed to fetch the
         currently pending batch.
     """
 
-    def __init__(self, gcvf, locations):
+    def __init__(self, gcvf, locations, get_compressor_settings=None):
         self.gcvf = gcvf
         self.locations = locations
         self.keys = []
@@ -1057,10 +1115,11 @@
         self.total_bytes = 0
         self.last_read_memo = None
         self.manager = None
+        self._get_compressor_settings = get_compressor_settings
 
     def add_key(self, key):
         """Add another to key to fetch.
-        
+
         :return: The estimated number of bytes needed to fetch the batch so
             far.
         """
@@ -1091,7 +1150,7 @@
             # and then.
             self.batch_memos[read_memo] = cached_block
         return self.total_bytes
-        
+
     def _flush_manager(self):
         if self.manager is not None:
             for factory in self.manager.get_record_stream():
@@ -1102,7 +1161,7 @@
     def yield_factories(self, full_flush=False):
         """Yield factories for keys added since the last yield.  They will be
         returned in the order they were added via add_key.
-        
+
         :param full_flush: by default, some results may not be returned in case
             they can be part of the next batch.  If full_flush is True, then
             all results are returned.
@@ -1136,7 +1195,8 @@
                     memos_to_get_stack.pop()
                 else:
                     block = self.batch_memos[read_memo]
-                self.manager = _LazyGroupContentManager(block)
+                self.manager = _LazyGroupContentManager(block,
+                    get_compressor_settings=self._get_compressor_settings)
                 self.last_read_memo = read_memo
             start, end = index_memo[3:5]
             self.manager.add_factory(key, parents, start, end)
@@ -1149,16 +1209,30 @@
         self.total_bytes = 0
 
 
-class GroupCompressVersionedFiles(VersionedFiles):
+class GroupCompressVersionedFiles(VersionedFilesWithFallbacks):
     """A group-compress based VersionedFiles implementation."""
 
-    def __init__(self, index, access, delta=True, _unadded_refs=None):
+    # This controls how the GroupCompress DeltaIndex works. Basically, we
+    # compute hash pointers into the source blocks (so hash(text) => text).
+    # However each of these references costs some memory in trade against a
+    # more accurate match result. For very large files, they either are
+    # pre-compressed and change in bulk whenever they change, or change in just
+    # local blocks. Either way, 'improved resolution' is not very helpful,
+    # versus running out of memory trying to track everything. The default max
+    # gives 100% sampling of a 1MB file.
+    _DEFAULT_MAX_BYTES_TO_INDEX = 1024 * 1024
+    _DEFAULT_COMPRESSOR_SETTINGS = {'max_bytes_to_index':
+                                     _DEFAULT_MAX_BYTES_TO_INDEX}
+
+    def __init__(self, index, access, delta=True, _unadded_refs=None,
+                 _group_cache=None):
         """Create a GroupCompressVersionedFiles object.
 
         :param index: The index object storing access and graph data.
         :param access: The access object storing raw data.
         :param delta: Whether to delta compress or just entropy compress.
         :param _unadded_refs: private parameter, don't use.
+        :param _group_cache: private parameter, don't use.
         """
         self._index = index
         self._access = access
@@ -1166,13 +1240,17 @@
         if _unadded_refs is None:
             _unadded_refs = {}
         self._unadded_refs = _unadded_refs
-        self._group_cache = LRUSizeCache(max_size=50*1024*1024)
-        self._fallback_vfs = []
+        if _group_cache is None:
+            _group_cache = LRUSizeCache(max_size=50*1024*1024)
+        self._group_cache = _group_cache
+        self._immediate_fallback_vfs = []
+        self._max_bytes_to_index = None
 
     def without_fallbacks(self):
         """Return a clone of this object without any fallbacks configured."""
         return GroupCompressVersionedFiles(self._index, self._access,
-            self._delta, _unadded_refs=dict(self._unadded_refs))
+            self._delta, _unadded_refs=dict(self._unadded_refs),
+            _group_cache=self._group_cache)
 
     def add_lines(self, key, parents, lines, parent_texts=None,
         left_matching_blocks=None, nostore_sha=None, random_id=False,
@@ -1182,11 +1260,11 @@
         :param key: The key tuple of the text to add.
         :param parents: The parents key tuples of the text to add.
         :param lines: A list of lines. Each line must be a bytestring. And all
-            of them except the last must be terminated with \n and contain no
-            other \n's. The last line may either contain no \n's or a single
-            terminating \n. If the lines list does meet this constraint the add
-            routine may error or may succeed - but you will be unable to read
-            the data back accurately. (Checking the lines have been split
+            of them except the last must be terminated with \\n and contain no
+            other \\n's. The last line may either contain no \\n's or a single
+            terminating \\n. If the lines list does meet this constraint the
+            add routine may error or may succeed - but you will be unable to
+            read the data back accurately. (Checking the lines have been split
             correctly is expensive and extremely unlikely to catch bugs so it
             is not done at runtime unless check_content is True.)
         :param parent_texts: An optional dictionary containing the opaque
@@ -1247,7 +1325,7 @@
 
         :param a_versioned_files: A VersionedFiles object.
         """
-        self._fallback_vfs.append(a_versioned_files)
+        self._immediate_fallback_vfs.append(a_versioned_files)
 
     def annotate(self, key):
         """See VersionedFiles.annotate."""
@@ -1287,22 +1365,6 @@
             self._check_lines_not_unicode(lines)
             self._check_lines_are_lines(lines)
 
-    def get_known_graph_ancestry(self, keys):
-        """Get a KnownGraph instance with the ancestry of keys."""
-        # Note that this is identical to
-        # KnitVersionedFiles.get_known_graph_ancestry, but they don't share
-        # ancestry.
-        parent_map, missing_keys = self._index.find_ancestry(keys)
-        for fallback in self._fallback_vfs:
-            if not missing_keys:
-                break
-            (f_parent_map, f_missing_keys) = fallback._index.find_ancestry(
-                                                missing_keys)
-            parent_map.update(f_parent_map)
-            missing_keys = f_missing_keys
-        kg = _mod_graph.KnownGraph(parent_map)
-        return kg
-
     def get_parent_map(self, keys):
         """Get a map of the graph parents of keys.
 
@@ -1323,7 +1385,7 @@
             and so on.
         """
         result = {}
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         source_results = []
         missing = set(keys)
         for source in sources:
@@ -1430,7 +1492,7 @@
         parent_map = {}
         key_to_source_map = {}
         source_results = []
-        for source in self._fallback_vfs:
+        for source in self._immediate_fallback_vfs:
             if not missing:
                 break
             source_parents = source.get_parent_map(missing)
@@ -1446,12 +1508,13 @@
 
         The returned objects should be in the order defined by 'ordering',
         which can weave between different sources.
+
         :param ordering: Must be one of 'topological' or 'groupcompress'
         :return: List of [(source, [keys])] tuples, such that all keys are in
             the defined order, regardless of source.
         """
         if ordering == 'topological':
-            present_keys = topo_sort(parent_map)
+            present_keys = tsort.topo_sort(parent_map)
         else:
             # ordering == 'groupcompress'
             # XXX: This only optimizes for the target ordering. We may need
@@ -1546,7 +1609,8 @@
         #  - we encounter an unadded ref, or
         #  - we run out of keys, or
         #  - the total bytes to retrieve for this batch > BATCH_SIZE
-        batcher = _BatchingBlockFetcher(self, locations)
+        batcher = _BatchingBlockFetcher(self, locations,
+            get_compressor_settings=self._get_compressor_settings)
         for source, keys in source_keys:
             if source is self:
                 for key in keys:
@@ -1598,6 +1662,30 @@
         for _ in self._insert_record_stream(stream, random_id=False):
             pass
 
+    def _get_compressor_settings(self):
+        if self._max_bytes_to_index is None:
+            # TODO: VersionedFiles don't know about their containing
+            #       repository, so they don't have much of an idea about their
+            #       location. So for now, this is only a global option.
+            c = config.GlobalConfig()
+            val = c.get_user_option('bzr.groupcompress.max_bytes_to_index')
+            if val is not None:
+                try:
+                    val = int(val)
+                except ValueError, e:
+                    trace.warning('Value for '
+                                  '"bzr.groupcompress.max_bytes_to_index"'
+                                  ' %r is not an integer'
+                                  % (val,))
+                    val = None
+            if val is None:
+                val = self._DEFAULT_MAX_BYTES_TO_INDEX
+            self._max_bytes_to_index = val
+        return {'max_bytes_to_index': self._max_bytes_to_index}
+
+    def _make_group_compressor(self):
+        return GroupCompressor(self._get_compressor_settings())
+
     def _insert_record_stream(self, stream, random_id=False, nostore_sha=None,
                               reuse_blocks=True):
         """Internal core to insert a record stream into this container.
@@ -1626,12 +1714,23 @@
                 return adapter
         # This will go up to fulltexts for gc to gc fetching, which isn't
         # ideal.
-        self._compressor = GroupCompressor()
+        self._compressor = self._make_group_compressor()
         self._unadded_refs = {}
         keys_to_add = []
         def flush():
-            bytes = self._compressor.flush().to_bytes()
-            self._compressor = GroupCompressor()
+            bytes_len, chunks = self._compressor.flush().to_chunks()
+            self._compressor = self._make_group_compressor()
+            # Note: At this point we still have 1 copy of the fulltext (in
+            #       record and the var 'bytes'), and this generates 2 copies of
+            #       the compressed text (one for bytes, one in chunks)
+            # TODO: Push 'chunks' down into the _access api, so that we don't
+            #       have to double compressed memory here
+            # TODO: Figure out how to indicate that we would be happy to free
+            #       the fulltext content at this point. Note that sometimes we
+            #       will want it later (streaming CHK pages), but most of the
+            #       time we won't (everything else)
+            bytes = ''.join(chunks)
+            del chunks
             index, start, length = self._access.add_raw_records(
                 [(None, len(bytes))], bytes)[0]
             nodes = []
@@ -1656,8 +1755,8 @@
                 raise errors.RevisionNotPresent(record.key, self)
             if random_id:
                 if record.key in inserted_keys:
-                    trace.note('Insert claimed random_id=True,'
-                               ' but then inserted %r two times', record.key)
+                    trace.note(gettext('Insert claimed random_id=True,'
+                               ' but then inserted %r two times'), record.key)
                     continue
                 inserted_keys.add(record.key)
             if reuse_blocks:
@@ -1802,13 +1901,61 @@
         """See VersionedFiles.keys."""
         if 'evil' in debug.debug_flags:
             trace.mutter_callsite(2, "keys scales with size of history")
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         result = set()
         for source in sources:
             result.update(source.keys())
         return result
 
 
+class _GCBuildDetails(object):
+    """A blob of data about the build details.
+
+    This stores the minimal data, which then allows compatibility with the old
+    api, without taking as much memory.
+    """
+
+    __slots__ = ('_index', '_group_start', '_group_end', '_basis_end',
+                 '_delta_end', '_parents')
+
+    method = 'group'
+    compression_parent = None
+
+    def __init__(self, parents, position_info):
+        self._parents = parents
+        (self._index, self._group_start, self._group_end, self._basis_end,
+         self._delta_end) = position_info
+
+    def __repr__(self):
+        return '%s(%s, %s)' % (self.__class__.__name__,
+            self.index_memo, self._parents)
+
+    @property
+    def index_memo(self):
+        return (self._index, self._group_start, self._group_end,
+                self._basis_end, self._delta_end)
+
+    @property
+    def record_details(self):
+        return static_tuple.StaticTuple(self.method, None)
+
+    def __getitem__(self, offset):
+        """Compatibility thunk to act like a tuple."""
+        if offset == 0:
+            return self.index_memo
+        elif offset == 1:
+            return self.compression_parent # Always None
+        elif offset == 2:
+            return self._parents
+        elif offset == 3:
+            return self.record_details
+        else:
+            raise IndexError('offset out of range')
+            
+    def __len__(self):
+        return 4
+
+
 class _GCGraphIndex(object):
     """Mapper from GroupCompressVersionedFiles needs into GraphIndex storage."""
 
@@ -1843,7 +1990,7 @@
         # repeated over and over, this creates a surplus of ints
         self._int_cache = {}
         if track_external_parent_refs:
-            self._key_dependencies = knit._KeyRefs(
+            self._key_dependencies = _KeyRefs(
                 track_new_keys=track_new_keys)
         else:
             self._key_dependencies = None
@@ -1989,16 +2136,14 @@
         :param keys: An iterable of keys.
         :return: A dict of key:
             (index_memo, compression_parent, parents, record_details).
-            index_memo
-                opaque structure to pass to read_records to extract the raw
-                data
-            compression_parent
-                Content that this record is built upon, may be None
-            parents
-                Logical parents of this node
-            record_details
-                extra information about the content which needs to be passed to
-                Factory.parse_record
+
+            * index_memo: opaque structure to pass to read_records to extract
+              the raw data
+            * compression_parent: Content that this record is built upon, may
+              be None
+            * parents: Logical parents of this node
+            * record_details: extra information about the content which needs
+              to be passed to Factory.parse_record
         """
         self._check_read()
         result = {}
@@ -2009,9 +2154,8 @@
                 parents = None
             else:
                 parents = entry[3][0]
-            method = 'group'
-            result[key] = (self._node_to_position(entry),
-                                  None, parents, (method, None))
+            details = _GCBuildDetails(parents, self._node_to_position(entry))
+            result[key] = details
         return result
 
     def keys(self):
@@ -2033,7 +2177,7 @@
         # each, or about 7MB. Note that it might be even more when you consider
         # how PyInt is allocated in separate slabs. And you can't return a slab
         # to the OS if even 1 int on it is in use. Note though that Python uses
-        # a LIFO when re-using PyInt slots, which probably causes more
+        # a LIFO when re-using PyInt slots, which might cause more
         # fragmentation.
         start = int(bits[0])
         start = self._int_cache.setdefault(start, start)

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2009-09-20 22:12:36 +0000
+++ bzrlib/hashcache.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2010-05-26 07:19:50 +0000
+++ bzrlib/help.py	2011-08-31 10:22:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 # TODO: `help commands --all` should show hidden commands
 
 import sys
-import textwrap
 
 from bzrlib import (
     commands as _mod_commands,
@@ -30,6 +29,7 @@
     help_topics,
     osutils,
     plugin,
+    utextwrap,
     )
 
 
@@ -96,7 +96,7 @@
         else:
             firstline = ''
         helpstring = '%-*s %s%s' % (max_name, cmd_name, firstline, plugin_name)
-        lines = textwrap.wrap(
+        lines = utextwrap.wrap(
             helpstring, subsequent_indent=indent,
             width=width,
             break_long_words=False)
@@ -135,6 +135,7 @@
             help_topics.HelpTopicIndex(),
             _mod_commands.HelpCommandIndex(),
             plugin.PluginsHelpIndex(),
+            help_topics.ConfigOptionHelpIndex(),
             ]
 
     def _check_prefix_uniqueness(self):

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2010-06-02 04:50:35 +0000
+++ bzrlib/help_topics/__init__.py	2011-10-10 20:15:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,8 +37,10 @@
 
 import bzrlib
 from bzrlib import (
+    config,
     osutils,
     registry,
+    i18n,
     )
 
 
@@ -65,7 +67,7 @@
         :param section: Section in reference manual - see SECT_* identifiers.
         """
         # The detail is stored as the 'object' and the metadata as the info
-        info=(summary,section)
+        info = (summary, section)
         super(HelpTopicRegistry, self).register(topic, detail, info=info)
 
     def register_lazy(self, topic, module_name, member_name, summary,
@@ -79,7 +81,7 @@
         :param section: Section in reference manual - see SECT_* identifiers.
         """
         # The detail is stored as the 'object' and the metadata as the info
-        info=(summary,section)
+        info = (summary, section)
         super(HelpTopicRegistry, self).register_lazy(topic, module_name,
                                                      member_name, info=info)
 
@@ -270,8 +272,8 @@
 
 would refer to ``/home/remote/myproject/trunk``.
 
-Many commands that accept URLs also accept location aliases too.  See
-::doc:`location-alias-help`.
+Many commands that accept URLs also accept location aliases too.
+See :doc:`location-alias-help` and :doc:`url-special-chars-help`.
 """
 
     return out
@@ -279,7 +281,7 @@
 
 _basic_help = \
 """Bazaar %s -- a free distributed version-control tool
-http://www.bazaar.canonical.com/
+http://bazaar.canonical.com/
 
 Basic commands:
   bzr init           makes this directory a versioned branch
@@ -316,6 +318,7 @@
 --builtin      Use the built-in version of a command, not the plugin version.
                This does not suppress other plugin effects.
 --no-plugins   Do not process any plugins.
+--no-l10n      Do not translate messages.
 --concurrency  Number of processes that can be run concurrently (selftest).
 
 --profile      Profile execution using the hotshot profiler.
@@ -328,6 +331,10 @@
                will be a pickle.
 --coverage     Generate line coverage report in the specified directory.
 
+-Oname=value   Override the ``name`` config option setting it to ``value`` for
+               the duration of the command.  This can be used multiple times if
+               several options need to be overridden.
+
 See http://doc.bazaar.canonical.com/developers/profiling.html for more
 information on profiling.
 
@@ -512,7 +519,7 @@
 
   checkout     Create a working tree when a branch does not have one.
   remove-tree  Removes the working tree from a branch when it is safe to do so.
-  update       When a working tree is out of sync with it's associated branch
+  update       When a working tree is out of sync with its associated branch
                this will update the tree to match the branch.
 """
 
@@ -591,8 +598,9 @@
 _env_variables = \
 """Environment Variables
 
-=================== =======================================================================
-BZRPATH             Path where bzr is to look for shell plugin external commands.
+=================== ===========================================================
+BZRPATH             Path where bzr is to look for shell plugin external
+                    commands.
 BZR_EMAIL           E-Mail address of the user. Overrides EMAIL.
 EMAIL               E-Mail address of the user.
 BZR_EDITOR          Editor for editing commit messages. Overrides EDITOR.
@@ -601,17 +609,22 @@
 BZR_DISABLE_PLUGINS Plugins that bzr should not load.
 BZR_PLUGINS_AT      Plugins to load from a directory not in BZR_PLUGIN_PATH.
 BZR_HOME            Directory holding .bazaar config dir. Overrides HOME.
-BZR_HOME (Win32)    Directory holding bazaar config dir. Overrides APPDATA and HOME.
+BZR_HOME (Win32)    Directory holding bazaar config dir. Overrides APPDATA and
+                    HOME.
 BZR_REMOTE_PATH     Full name of remote 'bzr' command (for bzr+ssh:// URLs).
-BZR_SSH             Path to SSH client, or one of paramiko, openssh, sshcorp, plink.
+BZR_SSH             Path to SSH client, or one of paramiko, openssh, sshcorp,
+                    plink or lsh.
 BZR_LOG             Location of .bzr.log (use '/dev/null' to suppress log).
 BZR_LOG (Win32)     Location of .bzr.log (use 'NUL' to suppress log).
 BZR_COLUMNS         Override implicit terminal width.
-BZR_CONCURRENCY     Number of processes that can be run concurrently (selftest).
-BZR_PROGRESS_BAR    Override the progress display. Values are 'none', 'dots', or 'tty'.
+BZR_CONCURRENCY     Number of processes that can be run concurrently (selftest)
+BZR_PROGRESS_BAR    Override the progress display. Values are 'none' or 'text'.
 BZR_PDB             Control whether to launch a debugger on error.
-BZR_SIGQUIT_PDB     Control whether SIGQUIT behaves normally or invokes a breakin debugger.
-=================== =======================================================================
+BZR_SIGQUIT_PDB     Control whether SIGQUIT behaves normally or invokes a
+                    breakin debugger.
+BZR_TEXTUI_INPUT    Force console input mode for prompts to line-based (instead
+                    of char-based).
+=================== ===========================================================
 """
 
 
@@ -772,6 +785,8 @@
                         'Aliases for remembered locations')
 topic_registry.register('log-formats', _load_from_file,
                         'Details on the logging formats available')
+topic_registry.register('url-special-chars', _load_from_file,
+                        'Special character handling in URLs')
 
 
 # Register concept topics.
@@ -836,6 +851,15 @@
             return []
 
 
+def _format_see_also(see_also):
+    result = ''
+    if see_also:
+        result += '\n:See also: '
+        result += ', '.join(sorted(set(see_also)))
+        result += '\n'
+    return result
+
+
 class RegisteredTopic(object):
     """A help topic which has been registered in the HelpTopicRegistry.
 
@@ -859,19 +883,11 @@
             returned instead of plain text.
         """
         result = topic_registry.get_detail(self.topic)
-        # there is code duplicated here and in bzrlib/plugin.py's
-        # matching Topic code. This should probably be factored in
-        # to a helper function and a common base class.
-        if additional_see_also is not None:
-            see_also = sorted(set(additional_see_also))
-        else:
-            see_also = None
-        if see_also:
-            result += '\n:See also: '
-            result += ', '.join(see_also)
-            result += '\n'
+        result += _format_see_also(additional_see_also)
         if plain:
             result = help_as_plain_text(result)
+        i18n.install()
+        result = i18n.gettext_per_paragraph(result)
         return result
 
     def get_help_topic(self):
@@ -892,6 +908,31 @@
         elif line.endswith('::'):
             line = line[:-1]
         # Map :doc:`xxx-help` to ``bzr help xxx``
-        line = re.sub(":doc:`(.+)-help`", r'``bzr help \1``', line)
+        line = re.sub(":doc:`(.+?)-help`", r'``bzr help \1``', line)
         result.append(line)
     return "\n".join(result) + "\n"
+
+
+class ConfigOptionHelpIndex(object):
+    """A help index that returns help topics for config options."""
+
+    def __init__(self):
+        self.prefix = 'configuration/'
+
+    def get_topics(self, topic):
+        """Search for topic in the registered config options.
+
+        :param topic: A topic to search for.
+        :return: A list which is either empty or contains a single
+            config.Option entry.
+        """
+        if topic is None:
+            return []
+        elif topic.startswith(self.prefix):
+            topic = topic[len(self.prefix):]
+        if topic in config.option_registry:
+            return [config.option_registry.get(topic)]
+        else:
+            return []
+
+

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2010-07-06 01:26:59 +0000
+++ bzrlib/help_topics/en/configuration.txt	2011-08-19 22:26:03 +0000
@@ -14,12 +14,14 @@
 
   "John Doe <jdoe@example.com>"
 
-See also the ``email`` configuration value.
+See also the ``email`` configuration option.
 
 BZR_PROGRESS_BAR
 ~~~~~~~~~~~~~~~~
 
-Override the progress display.  Possible values are "none", "dots", "tty"
+Override the progress display.  Possible values are "none" or "text".  If
+the value is "none" then no progress bar is displayed.  The value "text" draws
+the ordinary command line progress bar.
 
 BZR_SIGQUIT_PDB
 ~~~~~~~~~~~~~~~
@@ -52,7 +54,7 @@
 
 Path to the Bazaar executable to use when using the bzr+ssh protocol.
 
-See also the ``bzr_remote_path`` configuration value.
+See also the ``bzr_remote_path`` configuration option.
 
 BZR_EDITOR
 ~~~~~~~~~~
@@ -256,7 +258,14 @@
 variable name, an equals sign and a value.  For example::
 
     email            = John Doe <jdoe@isp.com>
-    check_signatures = require
+    gpg_signing_key  = Amy Pond <amy@example.com>
+
+A variable can reference other variables **in the same configuration file** by
+enclosing them in curly brackets::
+
+    my_branch_name = feature_x
+    my_server      = bzr+ssh://example.com
+    push_location   = {my_server}/project/{my_branch_name}
 
 
 Variable policies
@@ -302,7 +311,6 @@
     [DEFAULT]
     email             = John Doe <jdoe@isp.com>
     editor            = /usr/bin/vim
-    check_signatures  = check-available
     create_signatures = when-required
 
 
@@ -320,10 +328,6 @@
 
     [http://hypothetical.site.com/branches/devel-branch]
     create_signatures = always
-    check_signatures  = always
-
-    [http://bazaar-vcs.org/bzr/*]
-    check_signatures  = require
 
 The authentication configuration file, authentication.conf
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -351,6 +355,10 @@
 
     debug_flags = hpss
 
+or::
+
+    debug_flags = hpss,evil
+
 email
 ~~~~~
 
@@ -377,7 +385,8 @@
 check_signatures
 ~~~~~~~~~~~~~~~~
 
-Defines the behavior for signatures.
+Reserved for future use.  These options will allow a policy for branches to
+require signatures.
 
 require
     The gnupg signature for revisions must be present and must be valid.
@@ -393,18 +402,36 @@
 create_signatures
 ~~~~~~~~~~~~~~~~~
 
-Defines the behaviour of signing revisions.
+Defines the behaviour of signing revisions on commits.  By default bzr will not
+sign new commits.
 
 always
-    Sign every new revision that is committed.
+    Sign every new revision that is committed.  If the signing fails then the
+    commit will not be made.
 
 when-required
-    (default) Sign newly committed revisions only when the branch requires
-    signed revisions.
+    Reserved for future use.
 
 never
-    Refuse to sign newly committed revisions, even if the branch
-    requires signatures.
+    Reserved for future use.
+
+In future it is planned that ``when-required`` will sign newly
+committed revisions only when the branch requires them.  ``never`` will refuse
+to sign newly committed revisions, even if the branch requires signatures.
+
+dirstate.fdatasync
+~~~~~~~~~~~~~~~~~~
+
+If true (default), working tree metadata changes are flushed through the
+OS buffers to physical disk.  This is somewhat slower, but means data
+should not be lost if the machine crashes.  See also repository.fdatasync.
+
+gpg_signing_key
+~~~~~~~~~~~~~~~
+
+The GnuPG user identity to use when signing commits.  Can be an e-mail
+address, key fingerprint or full key ID.  When unset or when set to
+"default" Bazaar will use the user e-mail set with ``whoami``.
 
 recurse
 ~~~~~~~
@@ -427,6 +454,8 @@
 
     gpg_signing_command = /usr/bin/gnpg
 
+The specified command must accept the options "--clearsign" and "-u <email>".
+
 bzr_remote_path
 ~~~~~~~~~~~~~~~
 
@@ -453,6 +482,14 @@
 These settings are only needed if the SMTP server requires authentication
 to send mail.
 
+locks.steal_dead
+~~~~~~~~~~~~~~~~
+
+If set to true, bzr will automatically break locks held by processes from
+the same machine and user that are no longer alive.  Otherwise, it will
+print a message and you can break the lock manually, if you are satisfied
+the object is no longer in use.
+
 mail_client
 ~~~~~~~~~~~
 
@@ -479,6 +516,13 @@
 :mapi: Use your preferred e-mail client on Windows.
 :xdg-email: Use xdg-email to run your preferred mail program
 
+repository.fdatasync
+~~~~~~~~~~~~~~~~~~~~
+
+If true (default), repository changes are flushed through the OS buffers
+to physical disk.  This is somewhat slower, but means data should not be
+lost if the machine crashes.  See also dirstate.fdatasync.
+
 submit_branch
 ~~~~~~~~~~~~~
 
@@ -504,6 +548,12 @@
     whether the format deprecation warning is shown on repositories that are
     using deprecated formats.
 
+default_format
+~~~~~~~~~~~~~~
+
+A format name for the default format used when creating branches.  See ``bzr
+help formats`` for possible values.
+
 
 Unicode options
 ---------------
@@ -537,15 +587,15 @@
 parent_location
 ~~~~~~~~~~~~~~~
 
-If present, the location of the default branch for pull or merge.
-This option is normally set by ``pull --remember`` or ``merge
+If present, the location of the default branch for pull or merge.  This option
+is normally set when creating a branch, the first ``pull`` or by ``pull
 --remember``.
 
 push_location
 ~~~~~~~~~~~~~
 
 If present, the location of the default branch for push.  This option
-is normally set by ``push --remember``.
+is normally set by the first ``push`` or ``push --remember``.
 
 push_strict
 ~~~~~~~~~~~
@@ -578,3 +628,45 @@
 If present, defines the ``--strict`` option default value for checking
 uncommitted changes before sending a merge directive.
 
+add.maximum_file_size
+~~~~~~~~~~~~~~~~~~~~~
+
+Defines the maximum file size the command line "add" operation will allow
+in recursive mode, with files larger than this value being skipped. You may 
+specify this value as an integer (in which case it is interpreted as bytes), 
+or you may specify the value using SI units, i.e. 10KB, 20MB, 1G. A value of 0 
+will disable skipping.
+
+External Merge Tools
+--------------------
+
+bzr.mergetool.<name>
+~~~~~~~~~~~~~~~~~~~~
+
+Defines an external merge tool called <name> with the given command-line.
+Arguments containing spaces should be quoted using single or double quotes. The
+executable may omit its path if it can be found on the PATH.
+
+The following markers can be used in the command-line to substitute filenames
+involved in the merge conflict::
+
+  {base}      file.BASE
+  {this}      file.THIS
+  {other}     file.OTHER
+  {result}    output file
+  {this_temp} temp copy of file.THIS, used to overwrite output file if merge
+              succeeds.
+
+For example::
+
+  bzr.mergetool.kdiff3 = kdiff3 {base} {this} {other} -o {result}
+
+bzr.default_mergetool
+~~~~~~~~~~~~~~~~~~~~~
+
+Specifies which external merge tool (as defined above) should be selected by
+default in tools such as ``bzr qconflicts``.
+
+For example::
+
+  bzr.default_mergetool = kdiff3

=== modified file 'bzrlib/help_topics/en/conflict-types.txt'
--- bzrlib/help_topics/en/conflict-types.txt	2010-04-28 13:15:14 +0000
+++ bzrlib/help_topics/en/conflict-types.txt	2011-04-01 06:58:37 +0000
@@ -26,12 +26,12 @@
 present the conflicting versions, and it's up to you to find the correct
 resolution.
 
-Whatever the conflict is, resolving it is roughly done in two steps::
-
- - modify the working tree content so that the conflicted item is now in the
-   state you want to keep,
-
- - inform Bazaar that the conflict is now solved and ask to cleanup any
+Whatever the conflict is, resolving it is roughly done in two steps:
+
+1. Modify the working tree content so that the conflicted item is now in the
+   state you want to keep, then
+
+2. Inform Bazaar that the conflict is now solved and ask to cleanup any
    remaining generated information (``bzr resolve <item>``).
 
 For most conflict types, there are some obvious ways to modify the working
@@ -87,11 +87,11 @@
 
 The correct resolution would be "Martin Pool released Bazaar."
 
-You can handle text conflicts either by editing the main copy of the file, or
-by invoking external tools on the THIS, OTHER and BASE versions.  It's worth
-mentioning that resolving text conflicts rarely involves picking one set of
-changes over the other.  More often, the two sets of changes must be
-intelligently combined.
+You can handle text conflicts either by editing the main copy of the file,
+or by invoking external tools on the THIS, OTHER and BASE versions.  It's
+worth mentioning that resolving text conflicts rarely involves picking one
+set of changes over the other (but see below when you encounter these
+cases).  More often, the two sets of changes must be intelligently combined.
 
 If you edit the main copy, be sure to remove the herringbone markers.  When
 you are done editing, the file should look like it never had a conflict, and be
@@ -104,6 +104,16 @@
 ``.BASE``, ``.THIS`` and ``.OTHER`` files if they are still present in the
 working tree.
 
+
+When you want to pick one set of changes over the other, you can use ``bzr
+resolve`` with one of the following actions:
+
+* ``--action=take-this`` will issue ``mv FILE.THIS FILE``,
+* ``--action=take-other`` will issue ``mv FILE.OTHER FILE``.
+
+Note that if you have modified ``FILE.THIS`` or ``FILE.OTHER``, these
+modifications will be taken into account.
+
 Content conflicts
 -----------------
 
@@ -126,9 +136,9 @@
 
 ``bzr resolve`` recognizes the following actions:
 
- - ``--action=take-this`` will issue ``bzr mv FILE.THIS FILE``,
- - ``--action=take-other`` will issue ``bzr mv FILE.OTHER FILE``,
- - ``--action=done`` will just mark the conflict as resolved.
+* ``--action=take-this`` will issue ``bzr mv FILE.THIS FILE``,
+* ``--action=take-other`` will issue ``bzr mv FILE.OTHER FILE``,
+* ``--action=done`` will just mark the conflict as resolved.
 
 Any action will also delete the previously generated ``.BASE``, ``.THIS`` and
 ``.OTHER`` files if they are still present in the working tree.
@@ -172,11 +182,11 @@
 To resolve that kind of conflict, you should rebuild FILE from either version
 or a combination of both.
 
-``bzr resolve`` recognizes the following actions::
+``bzr resolve`` recognizes the following actions:
 
- - ``--action=take-this`` will issue ``bzr rm FILE ; bzr mv FILE.moved FILE``,
- - ``--action=take-other`` will issue ``bzr rm FILE.moved``,
- - ``--action=done`` will just mark the conflict as resolved.
+* ``--action=take-this`` will issue ``bzr rm FILE ; bzr mv FILE.moved FILE``,
+* ``--action=take-other`` will issue ``bzr rm FILE.moved``,
+* ``--action=done`` will just mark the conflict as resolved.
 
 Note that you must get rid of FILE.moved before using ``--action=done``.
 
@@ -213,13 +223,13 @@
 To resolve that kind of conflict, you should either remove or rename the
 children or the directory or a combination of both.
 
-``bzr resolve`` recognizes the following actions::
+``bzr resolve`` recognizes the following actions:
 
- - ``--action=take-this`` will issue ``bzr rm directory`` including the
-   children,
- - ``--action=take-other`` will acknowledge Bazaar choice to keep the children
-   and restoring the directory,
- - ``--action=done`` will just mark the conflict as resolved.
+* ``--action=take-this`` will issue ``bzr rm directory`` including the
+  children,
+* ``--action=take-other`` will acknowledge Bazaar choice to keep the children
+  and restoring the directory,
+* ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -231,18 +241,37 @@
   Conflict: can't delete DIR because it is not empty.  Not deleting.
 
 This is the opposite of "missing parent".  A directory is deleted in the
-source, but has new children in the target.  Bazaar will retain the directory.
-Resolving this issue depends very much on the particular scenario.
+source, but has new children in the target (either because a directory
+deletion is merged or because the merge introduce new children).  Bazaar
+will retain the directory.  Resolving this issue depends very much on the
+particular scenario.
 
 To resolve that kind of conflict, you should either remove or rename the
 children or the directory or a combination of both.
 
-``bzr resolve`` recognizes the following actions::
-
- - ``--action=take-this`` will acknowledge Bazaar choice to keep the directory,
- - ``--action=take-other`` will issue ``bzr rm directory`` including the 
-   children,
- - ``--action=done`` will just mark the conflict as resolved.
+``bzr resolve`` recognizes the following actions:
+
+* ``--action=take-this`` will acknowledge Bazaar choice to keep the directory,
+
+* ``--action=take-other`` will issue ``bzr rm directory`` including the 
+  children,
+
+* ``--action=done`` will just mark the conflict as resolved.
+
+Note that when merging a directory deletion, if unversioned files are
+present, they become potential orphans has they don't have a directory
+parent anymore.
+
+Handling such orphans, *before* the conflict is created, is controlled by
+setting the ``bzr.transform.orphan_policy`` configuration option.
+
+There are two possible values for this option:
+
+* ``conflict`` (the default): will leave the orphans in place and
+  generate a conflicts,
+
+* ``move``: will move the orphans to a ``bzr-orphans`` directory at the root
+  of the working tree with names like ``<file>.~#~``.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -259,12 +288,12 @@
 To resolve that kind of conflict, you just have to decide what name should be
 retained for the file involved.
 
-``bzr resolve`` recognizes the following actions::
+``bzr resolve`` recognizes the following actions:
 
- - ``--action=take-this`` will revert Bazaar choice and keep ``PATH1`` by
-   issuing ``bzr mv PATH2 PATH1``,
- - ``--action=take-other`` will acknowledge Bazaar choice of keeping ``PATH2``,
- - ``--action=done`` will just mark the conflict as resolved.
+* ``--action=take-this`` will revert Bazaar choice and keep ``PATH1`` by
+  issuing ``bzr mv PATH2 PATH1``,
+* ``--action=take-other`` will acknowledge Bazaar choice of keeping ``PATH2``,
+* ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -294,13 +323,13 @@
 ``black``.  To resolve that kind of conflict, you just have to decide what
 name should be retained for the directories involved.
 
-``bzr resolve`` recognizes the following actions::
+``bzr resolve`` recognizes the following actions:
 
- - ``--action=take-this`` will acknowledge Bazaar choice of leaving ``white`` 
-   in ``black``,
- - ``--action=take-other`` will revert Bazaar choice and move ``black`` in
+* ``--action=take-this`` will acknowledge Bazaar choice of leaving ``white`` 
+  in ``black``,
+* ``--action=take-other`` will revert Bazaar choice and move ``black`` in
    ``white`` by issuing ``bzr mv black/white white ; bzr mv black white``,
- - ``--action=done`` will just mark the conflict as resolved.
+* ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 
@@ -329,13 +358,13 @@
 To resolve that kind of conflict, you have to decide what name should be
 retained for the file, directory or symlink involved.
 
-``bzr resolve`` recognizes the following actions::
+``bzr resolve`` recognizes the following actions:
 
- - ``--action=take-this`` will issue ``bzr rm --force foo.new`` and 
-   ``bzr add foo``,
- - ``--action=take-other`` will issue ``bzr rm --force foo`` and 
-   ``bzr mv foo.new foo``,
- - ``--action=done`` will just mark the conflict as resolved.
+* ``--action=take-this`` will issue ``bzr rm --force foo.new`` and 
+  ``bzr add foo``,
+* ``--action=take-other`` will issue ``bzr rm --force foo`` and 
+  ``bzr mv foo.new foo``,
+* ``--action=done`` will just mark the conflict as resolved.
 
 Bazaar cannot auto-detect when conflicts of this kind have been resolved.
 

=== modified file 'bzrlib/help_topics/en/debug-flags.txt'
--- bzrlib/help_topics/en/debug-flags.txt	2010-01-05 04:30:07 +0000
+++ bzrlib/help_topics/en/debug-flags.txt	2011-05-27 17:10:05 +0000
@@ -25,6 +25,7 @@
 -Dknit            Trace knit operations.
 -Dlock            Trace when lockdir locks are taken or released.
 -Dprogress        Trace progress bar operations.
+-Dmem_dump        Dump memory to a file upon an out of memory error.
 -Dmerge           Emit information for debugging merges.
 -Dno_apport       Don't use apport to report crashes.
 -Dno_activity 	  Don't show transport activity indicator in progress bar.

=== modified file 'bzrlib/help_topics/en/log-formats.txt'
--- bzrlib/help_topics/en/log-formats.txt	2010-01-03 01:41:00 +0000
+++ bzrlib/help_topics/en/log-formats.txt	2011-08-31 11:39:56 +0000
@@ -19,6 +19,7 @@
   branch nick             yes            -             -
   foreign vcs properties  yes            yes           -
   preferred levels        all            1             1
+  digital signature       optional       -             -
 
 The default format is ``long``. To change this, define the ``log_format``
 setting in the ``[DEFAULT]`` section of ``bazaar.conf`` like this (say)::

=== modified file 'bzrlib/help_topics/en/rules.txt'
--- bzrlib/help_topics/en/rules.txt	2010-01-03 03:33:10 +0000
+++ bzrlib/help_topics/en/rules.txt	2011-04-17 22:48:14 +0000
@@ -9,7 +9,7 @@
 matching that pattern(s). For example::
 
     [name *.bat]
-    eol = dos
+    eol = native
 
     [name *.html *.xml]
     keywords = xml_escape

=== added file 'bzrlib/help_topics/en/url-special-chars.txt'
--- bzrlib/help_topics/en/url-special-chars.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/help_topics/en/url-special-chars.txt	2011-07-23 11:31:21 +0000
@@ -0,0 +1,48 @@
+Special character handling in URLs
+==================================
+
+Bazaar allows locations to be specified in multiple ways, either:
+
+ * Fully qualified URLs
+
+ * File system paths, relative or absolute
+
+Internally bzr treats all locations as URLs. For any file system paths
+that are specified it will automatically determine the appropriate URL
+representation, and escape special characters where necessary.
+
+There are a few characters which have special meaning in URLs and need careful
+handling to avoid ambiguities. Characters can be escaped with a % and a hex
+value in URLs. Any non-ASCII characters in a file path will automatically be
+urlencoded when the path is converted to a URL.
+
+URLs represent non-ASCII characters in an encoding defined by the server, but
+usually UTF-8.  The % escapes should be of the UTF-8 bytes.  Bazaar tries to be
+generous in what it accepts as a URL and to print them in a way that
+will be readable.
+
+For example, if you have a directory named '/tmp/%2False' these are all valid
+ways of accessing the content (0x2F, or 47, is the ASCII code for forward slash)::
+
+  cd /tmp
+  bzr log /tmp/%2False
+  bzr log %2False
+  bzr log file:///tmp/%252False
+  bzr log file://localhost/tmp/%252False
+  bzr log file:%252False
+
+These are valid but do not refer to the same file::
+
+  bzr log file:///tmp/%2False (refers to a file called /tmp/\/alse)
+  bzr log %252False (refers to a file called /tmp/%252False)
+
+Comma also has special meaning in URLs, because it denotes `segment parameters`_
+
+_`segment parameters`: http://www.ietf.org/rfc/rfc3986.txt (section 3.3)
+
+Comma is also special in any file system paths that are specified. To use a literal
+comma in a file system path, specify a URL and URL encode the comma::
+
+  bzr log foo,branch=bla # path "foo" with the segment parameter "branch" set to "bla"
+  bzr log file:foo%2Cbranch=bla # path "foo,branch=bla"
+  bzr log file:foo,branch=bla # path "foo" with segment parameter "branch" set to "bla"

=== modified file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	2010-05-26 04:26:59 +0000
+++ bzrlib/hooks.py	2011-10-11 10:21:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,50 +16,82 @@
 
 
 """Support for plugin hooking logic."""
-from bzrlib import registry
+
+from bzrlib import (
+    registry,
+    symbol_versioning,
+    )
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import textwrap
 
 from bzrlib import (
-        _format_version_tuple,
-        errors,
-        )
-from bzrlib.help_topics import help_as_plain_text
+    _format_version_tuple,
+    errors,
+    pyutils,
+    )
+from bzrlib.i18n import gettext
 """)
 
 
-known_hooks = registry.Registry()
-# known_hooks registry contains
-# tuple of (module, member name) which is the hook point
-# module where the specific hooks are defined
-# callable to get the empty specific Hooks for that attribute
-known_hooks.register_lazy(('bzrlib.branch', 'Branch.hooks'), 'bzrlib.branch',
-    'BranchHooks')
-known_hooks.register_lazy(('bzrlib.bzrdir', 'BzrDir.hooks'), 'bzrlib.bzrdir',
-    'BzrDirHooks')
-known_hooks.register_lazy(('bzrlib.commands', 'Command.hooks'),
-    'bzrlib.commands', 'CommandHooks')
-known_hooks.register_lazy(('bzrlib.info', 'hooks'),
-    'bzrlib.info', 'InfoHooks')
-known_hooks.register_lazy(('bzrlib.lock', 'Lock.hooks'), 'bzrlib.lock',
-    'LockHooks')
-known_hooks.register_lazy(('bzrlib.merge', 'Merger.hooks'), 'bzrlib.merge',
-    'MergeHooks')
-known_hooks.register_lazy(('bzrlib.msgeditor', 'hooks'), 'bzrlib.msgeditor',
-    'MessageEditorHooks')
-known_hooks.register_lazy(('bzrlib.mutabletree', 'MutableTree.hooks'),
-    'bzrlib.mutabletree', 'MutableTreeHooks')
-known_hooks.register_lazy(('bzrlib.smart.client', '_SmartClient.hooks'),
-    'bzrlib.smart.client', 'SmartClientHooks')
-known_hooks.register_lazy(('bzrlib.smart.server', 'SmartTCPServer.hooks'),
-    'bzrlib.smart.server', 'SmartServerHooks')
-known_hooks.register_lazy(
-    ('bzrlib.version_info_formats.format_rio', 'RioVersionInfoBuilder.hooks'),
-    'bzrlib.version_info_formats.format_rio', 'RioVersionInfoBuilderHooks')
-known_hooks.register_lazy(
-    ('bzrlib.merge_directive', 'BaseMergeDirective.hooks'),
-    'bzrlib.merge_directive', 'MergeDirectiveHooks')
+class KnownHooksRegistry(registry.Registry):
+    # known_hooks registry contains
+    # tuple of (module, member name) which is the hook point
+    # module where the specific hooks are defined
+    # callable to get the empty specific Hooks for that attribute
+
+    def register_lazy_hook(self, hook_module_name, hook_member_name,
+            hook_factory_member_name):
+        self.register_lazy((hook_module_name, hook_member_name),
+            hook_module_name, hook_factory_member_name)
+
+    def iter_parent_objects(self):
+        """Yield (hook_key, (parent_object, attr)) tuples for every registered
+        hook, where 'parent_object' is the object that holds the hook
+        instance.
+
+        This is useful for resetting/restoring all the hooks to a known state,
+        as is done in bzrlib.tests.TestCase._clear_hooks.
+        """
+        for key in self.keys():
+            yield key, self.key_to_parent_and_attribute(key)
+
+    def key_to_parent_and_attribute(self, (module_name, member_name)):
+        """Convert a known_hooks key to a (parent_obj, attr) pair.
+
+        :param key: A tuple (module_name, member_name) as found in the keys of
+            the known_hooks registry.
+        :return: The parent_object of the hook and the name of the attribute on
+            that parent object where the hook is kept.
+        """
+        parent_mod, parent_member, attr = pyutils.calc_parent_name(module_name,
+            member_name)
+        return pyutils.get_named_object(parent_mod, parent_member), attr
+
+
+_builtin_known_hooks = (
+    ('bzrlib.branch', 'Branch.hooks', 'BranchHooks'),
+    ('bzrlib.controldir', 'ControlDir.hooks', 'ControlDirHooks'),
+    ('bzrlib.commands', 'Command.hooks', 'CommandHooks'),
+    ('bzrlib.config', 'ConfigHooks', '_ConfigHooks'),
+    ('bzrlib.info', 'hooks', 'InfoHooks'),
+    ('bzrlib.lock', 'Lock.hooks', 'LockHooks'),
+    ('bzrlib.merge', 'Merger.hooks', 'MergeHooks'),
+    ('bzrlib.msgeditor', 'hooks', 'MessageEditorHooks'),
+    ('bzrlib.mutabletree', 'MutableTree.hooks', 'MutableTreeHooks'),
+    ('bzrlib.smart.client', '_SmartClient.hooks', 'SmartClientHooks'),
+    ('bzrlib.smart.server', 'SmartTCPServer.hooks', 'SmartServerHooks'),
+    ('bzrlib.status', 'hooks', 'StatusHooks'),
+    ('bzrlib.version_info_formats.format_rio', 'RioVersionInfoBuilder.hooks',
+        'RioVersionInfoBuilderHooks'),
+    ('bzrlib.merge_directive', 'BaseMergeDirective.hooks',
+        'MergeDirectiveHooks'),
+    )
+
+known_hooks = KnownHooksRegistry()
+for (_hook_module, _hook_attribute, _hook_class) in _builtin_known_hooks:
+    known_hooks.register_lazy_hook(_hook_module, _hook_attribute, _hook_class)
+del _builtin_known_hooks, _hook_module, _hook_attribute, _hook_class
 
 
 def known_hooks_key_to_object((module_name, member_name)):
@@ -69,24 +101,13 @@
         the known_hooks registry.
     :return: The object this specifies.
     """
-    return registry._LazyObjectGetter(module_name, member_name).get_obj()
-
-
-def known_hooks_key_to_parent_and_attribute((module_name, member_name)):
-    """Convert a known_hooks key to a object.
-
-    :param key: A tuple (module_name, member_name) as found in the keys of
-        the known_hooks registry.
-    :return: The object this specifies.
-    """
-    member_list = member_name.rsplit('.', 1)
-    if len(member_list) == 2:
-        parent_name, attribute = member_list
-    else:
-        parent_name = None
-        attribute = member_name
-    parent = known_hooks_key_to_object((module_name, parent_name))
-    return parent, attribute
+    return pyutils.get_named_object(module_name, member_name)
+
+
+@symbol_versioning.deprecated_function(symbol_versioning.deprecated_in((2, 3)))
+def known_hooks_key_to_parent_and_attribute(key):
+    """See KnownHooksRegistry.key_to_parent_and_attribute."""
+    return known_hooks.key_to_parent_and_attribute(key)
 
 
 class Hooks(dict):
@@ -96,10 +117,40 @@
     FOO hook is triggered.
     """
 
-    def __init__(self):
+    def __init__(self, module=None, member_name=None):
+        """Create a new hooks dictionary.
+
+        :param module: The module from which this hooks dictionary should be loaded
+            (used for lazy hooks)
+        :param member_name: Name under which this hooks dictionary should be loaded.
+            (used for lazy hooks)
+        """
         dict.__init__(self)
         self._callable_names = {}
-
+        self._module = module
+        self._member_name = member_name
+
+    def add_hook(self, name, doc, introduced, deprecated=None):
+        """Add a hook point to this dictionary.
+
+        :param name: The name of the hook, for clients to use when registering.
+        :param doc: The docs for the hook.
+        :param introduced: When the hook was introduced (e.g. (0, 15)).
+        :param deprecated: When the hook was deprecated, None for
+            not-deprecated.
+        """
+        if name in self:
+            raise errors.DuplicateKey(name)
+        if self._module:
+            callbacks = _lazy_hooks.setdefault(
+                (self._module, self._member_name, name), [])
+        else:
+            callbacks = None
+        hookpoint = HookPoint(name=name, doc=doc, introduced=introduced,
+                              deprecated=deprecated, callbacks=callbacks)
+        self[name] = hookpoint
+
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4)))
     def create_hook(self, hook):
         """Create a hook which can have callbacks registered for it.
 
@@ -147,14 +198,38 @@
         """
         return self._callable_names.get(a_callable, "No hook name")
 
+    def install_named_hook_lazy(self, hook_name, callable_module,
+        callable_member, name):
+        """Install a_callable in to the hook hook_name lazily, and label it.
+
+        :param hook_name: A hook name. See the __init__ method for the complete
+            list of hooks.
+        :param callable_module: Name of the module in which the callable is
+            present.
+        :param callable_member: Member name of the callable.
+        :param name: A name to associate the callable with, to show users what
+            is running.
+        """
+        try:
+            hook = self[hook_name]
+        except KeyError:
+            raise errors.UnknownHook(self.__class__.__name__, hook_name)
+        try:
+            hook_lazy = getattr(hook, "hook_lazy")
+        except AttributeError:
+            raise errors.UnsupportedOperation(self.install_named_hook_lazy,
+                self)
+        else:
+            hook_lazy(callable_module, callable_member, name)
+
     def install_named_hook(self, hook_name, a_callable, name):
         """Install a_callable in to the hook hook_name, and label it name.
 
-        :param hook_name: A hook name. See the __init__ method of BranchHooks
-            for the complete list of hooks.
+        :param hook_name: A hook name. See the __init__ method for the complete
+            list of hooks.
         :param a_callable: The callable to be invoked when the hook triggers.
             The exact signature will depend on the hook - see the __init__
-            method of BranchHooks for details on each hook.
+            method for details on each hook.
         :param name: A name to associate a_callable with, to show users what is
             running.
         """
@@ -170,6 +245,23 @@
         if name is not None:
             self.name_hook(a_callable, name)
 
+    def uninstall_named_hook(self, hook_name, label):
+        """Uninstall named hooks.
+
+        :param hook_name: Hook point name
+        :param label: Label of the callable to uninstall
+        """
+        try:
+            hook = self[hook_name]
+        except KeyError:
+            raise errors.UnknownHook(self.__class__.__name__, hook_name)
+        try:
+            uninstall = getattr(hook, "uninstall")
+        except AttributeError:
+            raise errors.UnsupportedOperation(self.uninstall_named_hook, self)
+        else:
+            uninstall(label)
+
     def name_hook(self, a_callable, name):
         """Associate name with a_callable to show users what is running."""
         self._callable_names[a_callable] = name
@@ -179,16 +271,16 @@
     """A single hook that clients can register to be called back when it fires.
 
     :ivar name: The name of the hook.
+    :ivar doc: The docs for using the hook.
     :ivar introduced: A version tuple specifying what version the hook was
         introduced in. None indicates an unknown version.
     :ivar deprecated: A version tuple specifying what version the hook was
         deprecated or superseded in. None indicates that the hook is not
         superseded or deprecated. If the hook is superseded then the doc
         should describe the recommended replacement hook to register for.
-    :ivar doc: The docs for using the hook.
     """
 
-    def __init__(self, name, doc, introduced, deprecated):
+    def __init__(self, name, doc, introduced, deprecated=None, callbacks=None):
         """Create a HookPoint.
 
         :param name: The name of the hook, for clients to use when registering.
@@ -201,8 +293,10 @@
         self.__doc__ = doc
         self.introduced = introduced
         self.deprecated = deprecated
-        self._callbacks = []
-        self._callback_names = {}
+        if callbacks is None:
+            self._callbacks = []
+        else:
+            self._callbacks = callbacks
 
     def docs(self):
         """Generate the documentation for this HookPoint.
@@ -217,10 +311,10 @@
             introduced_string = _format_version_tuple(self.introduced)
         else:
             introduced_string = 'unknown'
-        strings.append('Introduced in: %s' % introduced_string)
+        strings.append(gettext('Introduced in: %s') % introduced_string)
         if self.deprecated:
             deprecated_string = _format_version_tuple(self.deprecated)
-            strings.append('Deprecated in: %s' % deprecated_string)
+            strings.append(gettext('Deprecated in: %s') % deprecated_string)
         strings.append('')
         strings.extend(textwrap.wrap(self.__doc__,
             break_long_words=False))
@@ -228,8 +322,20 @@
         return '\n'.join(strings)
 
     def __eq__(self, other):
-        return (type(other) == type(self) and 
-            other.__dict__ == self.__dict__)
+        return (type(other) == type(self) and other.__dict__ == self.__dict__)
+
+    def hook_lazy(self, callback_module, callback_member, callback_label):
+        """Lazily register a callback to be called when this HookPoint fires.
+
+        :param callback_module: Module of the callable to use when this
+            HookPoint fires.
+        :param callback_member: Member name of the callback.
+        :param callback_label: A label to show in the UI while this callback is
+            processing.
+        """
+        obj_getter = registry._LazyObjectGetter(callback_module,
+            callback_member)
+        self._callbacks.append((obj_getter, callback_label))
 
     def hook(self, callback, callback_label):
         """Register a callback to be called when this HookPoint fires.
@@ -238,12 +344,26 @@
         :param callback_label: A label to show in the UI while this callback is
             processing.
         """
-        self._callbacks.append(callback)
-        if callback_label is not None:
-            self._callback_names[callback] = callback_label
+        obj_getter = registry._ObjectGetter(callback)
+        self._callbacks.append((obj_getter, callback_label))
+
+    def uninstall(self, label):
+        """Uninstall the callback with the specified label.
+
+        :param label: Label of the entry to uninstall
+        """
+        entries_to_remove = []
+        for entry in self._callbacks:
+            (entry_callback, entry_label) = entry
+            if entry_label == label:
+                entries_to_remove.append(entry)
+        if entries_to_remove == []:
+            raise KeyError("No entry with label %r" % label)
+        for entry in entries_to_remove:
+            self._callbacks.remove(entry)
 
     def __iter__(self):
-        return iter(self._callbacks)
+        return (callback.get_obj() for callback, name in self._callbacks)
 
     def __len__(self):
         return len(self._callbacks)
@@ -253,12 +373,13 @@
         strings.append("<%s(" % type(self).__name__)
         strings.append(self.name)
         strings.append("), callbacks=[")
-        for callback in self._callbacks:
-            strings.append(repr(callback))
+        callbacks = self._callbacks
+        for (callback, callback_name) in callbacks:
+            strings.append(repr(callback.get_obj()))
             strings.append("(")
-            strings.append(self._callback_names[callback])
+            strings.append(callback_name)
             strings.append("),")
-        if len(self._callbacks) == 1:
+        if len(callbacks) == 1:
             strings[-1] = ")"
         strings.append("]>")
         return ''.join(strings)
@@ -303,3 +424,24 @@
         hooks = known_hooks_key_to_object(hook_key)
         segments.append(hooks.docs())
     return '\n'.join(segments)
+
+
+# Lazily registered hooks. Maps (module, name, hook_name) tuples
+# to lists of tuples with objectgetters and names
+_lazy_hooks = {}
+
+
+def install_lazy_named_hook(hookpoints_module, hookpoints_name, hook_name,
+    a_callable, name):
+    """Install a callable in to a hook lazily, and label it name.
+
+    :param hookpoints_module: Module name of the hook points.
+    :param hookpoints_name: Name of the hook points.
+    :param hook_name: A hook name.
+    :param callable: a callable to call for the hook.
+    :param name: A name to associate a_callable with, to show users what is
+        running.
+    """
+    key = (hookpoints_module, hookpoints_name, hook_name)
+    obj_getter = registry._ObjectGetter(a_callable)
+    _lazy_hooks.setdefault(key, []).append((obj_getter, name))

=== added file 'bzrlib/i18n.py'
--- bzrlib/i18n.py	1970-01-01 00:00:00 +0000
+++ bzrlib/i18n.py	2011-10-04 22:20:49 +0000
@@ -0,0 +1,205 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2007 Lukáš Lalinský <lalinsky@gmail.com>
+# Copyright (C) 2007,2009 Alexander Belchenko <bialix@ukr.net>
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+# This module is copied from Bazaar Explorer and modified for bzr.
+
+"""i18n and l10n support for Bazaar."""
+
+import gettext as _gettext
+import os
+import sys
+
+
+_translations = None
+
+
+def gettext(message):
+    """Translate message. 
+    
+    :returns: translated message as unicode.
+    """
+    install()
+    return _translations.ugettext(message)
+
+
+def ngettext(singular, plural, number):
+    """Translate message with plural forms based on `number`.
+
+    :param singular: English language message in singular form
+    :param plural: English language message in plural form
+    :param number: the number this message should be translated for
+
+    :returns: translated message as unicode.
+    """
+    install()
+    return _translations.ungettext(singular, plural, number)
+
+
+def N_(msg):
+    """Mark message for translation but don't translate it right away."""
+    return msg
+
+
+def gettext_per_paragraph(message):
+    """Translate message per paragraph.
+
+    :returns: concatenated translated message as unicode.
+    """
+    install()
+    paragraphs = message.split(u'\n\n')
+    ugettext = _translations.ugettext
+    # Be careful not to translate the empty string -- it holds the
+    # meta data of the .po file.
+    return u'\n\n'.join(ugettext(p) if p else u'' for p in paragraphs)
+
+
+def disable_i18n():
+    """Do not allow i18n to be enabled.  Useful for third party users
+    of bzrlib."""
+    global _translations
+    _translations = _gettext.NullTranslations()
+
+
+def installed():
+    """Returns whether translations are in use or not."""
+    return _translations is not None
+
+
+def install(lang=None):
+    """Enables gettext translations in bzr."""
+    global _translations
+    if installed():
+        return
+    _translations = install_translations(lang)
+
+
+def install_translations(lang=None, domain='bzr', locale_base=None):
+    """Create a gettext translation object.
+    
+    :param lang: language to install.
+    :param domain: translation domain to install.
+    :param locale_base: plugins can specify their own directory.
+
+    :returns: a gettext translations object to use
+    """
+    if lang is None:
+        lang = _get_current_locale()
+    if lang is not None:
+        languages = lang.split(':')
+    else:
+        languages = None
+    translation = _gettext.translation(
+            domain,
+            localedir=_get_locale_dir(locale_base),
+            languages=languages,
+            fallback=True)
+    return translation
+
+
+def add_fallback(fallback):
+    """
+    Add a fallback translations object.  Typically used by plugins.
+
+    :param fallback: gettext.GNUTranslations object
+    """
+    install()
+    _translations.add_fallback(fallback)
+
+
+def uninstall():
+    """Disables gettext translations."""
+    global _translations
+    _translations = None
+
+
+def _get_locale_dir(base):
+    """Returns directory to find .mo translations file in, either local or system
+
+    :param base: plugins can specify their own local directory
+    """
+    if hasattr(sys, 'frozen'):
+        if base is None:
+            base = os.path.dirname(
+                unicode(sys.executable, sys.getfilesystemencoding()))
+        return os.path.join(base, u'locale')
+    else:
+        if base is None:
+            base = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
+        dirpath = os.path.realpath(os.path.join(base, u'locale'))
+        if os.path.exists(dirpath):
+            return dirpath
+        else:
+            return '/usr/share/locale'
+
+
+def _check_win32_locale():
+    for i in ('LANGUAGE','LC_ALL','LC_MESSAGES','LANG'):
+        if os.environ.get(i):
+            break
+    else:
+        lang = None
+        import locale
+        try:
+            import ctypes
+        except ImportError:
+            # use only user's default locale
+            lang = locale.getdefaultlocale()[0]
+        else:
+            # using ctypes to determine all locales
+            lcid_user = ctypes.windll.kernel32.GetUserDefaultLCID()
+            lcid_system = ctypes.windll.kernel32.GetSystemDefaultLCID()
+            if lcid_user != lcid_system:
+                lcid = [lcid_user, lcid_system]
+            else:
+                lcid = [lcid_user]
+            lang = [locale.windows_locale.get(i) for i in lcid]
+            lang = ':'.join([i for i in lang if i])
+        # set lang code for gettext
+        if lang:
+            os.environ['LANGUAGE'] = lang
+
+
+def _get_current_locale():
+    if not os.environ.get('LANGUAGE'):
+        from bzrlib import config
+        lang = config.GlobalStack().get('language')
+        if lang:
+            os.environ['LANGUAGE'] = lang
+            return lang
+    if sys.platform == 'win32':
+        _check_win32_locale()
+    for i in ('LANGUAGE','LC_ALL','LC_MESSAGES','LANG'):
+        lang = os.environ.get(i)
+        if lang:
+            return lang
+    return None
+
+
+def load_plugin_translations(domain):
+    """Load the translations for a specific plugin.
+
+    :param domain: Gettext domain name (usually 'bzr-PLUGINNAME')
+    """
+    locale_base = os.path.dirname(
+        unicode(__file__, sys.getfilesystemencoding()))
+    translation = install_translations(domain=domain,
+        locale_base=locale_base)
+    add_fallback(translation)
+    return translation

=== modified file 'bzrlib/ignores.py'
--- bzrlib/ignores.py	2010-05-03 04:51:58 +0000
+++ bzrlib/ignores.py	2011-04-07 10:36:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 from cStringIO import StringIO
 
 import bzrlib
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from bzrlib import (
     atomicfile,
     config,
@@ -28,6 +30,7 @@
     )
 
 from trace import warning
+""")
 
 # ~/.bazaar/ignore will be filled out using
 # this ignore list, if it does not exist
@@ -41,6 +44,8 @@
     '*~',
     '.#*',
     '[#]*#',
+    '__pycache__',
+    'bzr-orphans',
 ]
 
 

=== modified file 'bzrlib/index.py'
--- bzrlib/index.py	2010-05-25 17:27:52 +0000
+++ bzrlib/index.py	2011-07-05 06:41:44 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,10 +31,11 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib import trace
-from bzrlib.bisect_multi import bisect_multi_bytes
-from bzrlib.revision import NULL_REVISION
-from bzrlib.trace import mutter
+from bzrlib import (
+    bisect_multi,
+    revision as _mod_revision,
+    trace,
+    )
 """)
 from bzrlib import (
     debug,
@@ -69,20 +70,20 @@
 class GraphIndexBuilder(object):
     """A builder that can build a GraphIndex.
 
-    The resulting graph has the structure:
+    The resulting graph has the structure::
 
-    _SIGNATURE OPTIONS NODES NEWLINE
-    _SIGNATURE     := 'Bazaar Graph Index 1' NEWLINE
-    OPTIONS        := 'node_ref_lists=' DIGITS NEWLINE
-    NODES          := NODE*
-    NODE           := KEY NULL ABSENT? NULL REFERENCES NULL VALUE NEWLINE
-    KEY            := Not-whitespace-utf8
-    ABSENT         := 'a'
-    REFERENCES     := REFERENCE_LIST (TAB REFERENCE_LIST){node_ref_lists - 1}
-    REFERENCE_LIST := (REFERENCE (CR REFERENCE)*)?
-    REFERENCE      := DIGITS  ; digits is the byte offset in the index of the
-                              ; referenced key.
-    VALUE          := no-newline-no-null-bytes
+      _SIGNATURE OPTIONS NODES NEWLINE
+      _SIGNATURE     := 'Bazaar Graph Index 1' NEWLINE
+      OPTIONS        := 'node_ref_lists=' DIGITS NEWLINE
+      NODES          := NODE*
+      NODE           := KEY NULL ABSENT? NULL REFERENCES NULL VALUE NEWLINE
+      KEY            := Not-whitespace-utf8
+      ABSENT         := 'a'
+      REFERENCES     := REFERENCE_LIST (TAB REFERENCE_LIST){node_ref_lists - 1}
+      REFERENCE_LIST := (REFERENCE (CR REFERENCE)*)?
+      REFERENCE      := DIGITS  ; digits is the byte offset in the index of the
+                                ; referenced key.
+      VALUE          := no-newline-no-null-bytes
     """
 
     def __init__(self, reference_lists=0, key_elements=1):
@@ -184,11 +185,12 @@
         :param value: The value associate with this key. Must not contain
             newlines or null characters.
         :return: (node_refs, absent_references)
-            node_refs   basically a packed form of 'references' where all
-                        iterables are tuples
-            absent_references   reference keys that are not in self._nodes.
-                                This may contain duplicates if the same key is
-                                referenced in multiple lists.
+        
+            * node_refs: basically a packed form of 'references' where all
+              iterables are tuples
+            * absent_references: reference keys that are not in self._nodes.
+              This may contain duplicates if the same key is referenced in
+              multiple lists.
         """
         as_st = StaticTuple.from_sequence
         self._check_key(key)
@@ -219,7 +221,7 @@
         :param references: An iterable of iterables of keys. Each is a
             reference to another key.
         :param value: The value to associate with the key. It may be any
-            bytes as long as it does not contain \0 or \n.
+            bytes as long as it does not contain \\0 or \\n.
         """
         (node_refs,
          absent_references) = self._check_key_ref_value(key, references, value)
@@ -243,6 +245,11 @@
         """
         
     def finish(self):
+        """Finish the index.
+
+        :returns: cStringIO holding the full context of the index as it 
+        should be written to disk.
+        """
         lines = [_SIGNATURE]
         lines.append(_OPTION_NODE_REFS + str(self.reference_lists) + '\n')
         lines.append(_OPTION_KEY_ELEMENTS + str(self._key_length) + '\n')
@@ -444,7 +451,8 @@
             # We already did this
             return
         if 'index' in debug.debug_flags:
-            mutter('Reading entire index %s', self._transport.abspath(self._name))
+            trace.mutter('Reading entire index %s',
+                          self._transport.abspath(self._name))
         if stream is None:
             stream = self._transport.get(self._name)
             if self._base_offset != 0:
@@ -671,7 +679,7 @@
         if self._nodes is not None:
             return self._iter_entries_from_total_buffer(keys)
         else:
-            return (result[1] for result in bisect_multi_bytes(
+            return (result[1] for result in bisect_multi.bisect_multi_bytes(
                 self._lookup_keys_via_location, self._size, keys))
 
     def iter_entries_prefix(self, keys):
@@ -1288,15 +1296,15 @@
     def get_parent_map(self, keys):
         """See graph.StackedParentsProvider.get_parent_map"""
         search_keys = set(keys)
-        if NULL_REVISION in search_keys:
-            search_keys.discard(NULL_REVISION)
-            found_parents = {NULL_REVISION:[]}
+        if _mod_revision.NULL_REVISION in search_keys:
+            search_keys.discard(_mod_revision.NULL_REVISION)
+            found_parents = {_mod_revision.NULL_REVISION:[]}
         else:
             found_parents = {}
         for index, key, value, refs in self.iter_entries(search_keys):
             parents = refs[0]
             if not parents:
-                parents = (NULL_REVISION,)
+                parents = (_mod_revision.NULL_REVISION,)
             found_parents[key] = parents
         return found_parents
 
@@ -1434,8 +1442,8 @@
         """
         indices_info = zip(self._index_names, self._indices)
         if 'index' in debug.debug_flags:
-            mutter('CombinedGraphIndex reordering: currently %r, promoting %r',
-                   indices_info, hit_indices)
+            trace.mutter('CombinedGraphIndex reordering: currently %r, '
+                         'promoting %r', indices_info, hit_indices)
         hit_names = []
         unhit_names = []
         new_hit_indices = []
@@ -1458,7 +1466,7 @@
         self._indices = new_hit_indices + unhit_indices
         self._index_names = hit_names + unhit_names
         if 'index' in debug.debug_flags:
-            mutter('CombinedGraphIndex reordered: %r', self._indices)
+            trace.mutter('CombinedGraphIndex reordered: %r', self._indices)
         return hit_names
 
     def _move_to_front_by_name(self, hit_names):

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2010-04-30 11:35:43 +0000
+++ bzrlib/info.py	2011-10-11 12:01:51 +0000
@@ -22,6 +22,7 @@
 
 from bzrlib import (
     bzrdir,
+    controldir,
     errors,
     hooks as _mod_hooks,
     osutils,
@@ -221,8 +222,10 @@
     """Show missing revisions in working tree."""
     branch = working.branch
     basis = working.basis_tree()
-    work_inv = working.inventory
-    branch_revno, branch_last_revision = branch.last_revision_info()
+    try:
+        branch_revno, branch_last_revision = branch.last_revision_info()
+    except errors.UnsupportedOperation:
+        return
     try:
         tree_last_id = working.get_parent_ids()[0]
     except IndexError:
@@ -239,7 +242,6 @@
 def _show_working_stats(working, outfile):
     """Show statistics about a working tree."""
     basis = working.basis_tree()
-    work_inv = working.inventory
     delta = working.changes_from(basis, want_unchanged=True)
 
     outfile.write('\n')
@@ -260,9 +262,9 @@
     outfile.write('  %8d ignored\n' % ignore_cnt)
 
     dir_cnt = 0
-    for file_id in work_inv:
-        if (work_inv.get_file_kind(file_id) == 'directory' and
-            not work_inv.is_root(file_id)):
+    root_id = working.get_root_id()
+    for path, entry in working.iter_entries_by_dir():
+        if entry.kind == 'directory' and entry.file_id != root_id:
             dir_cnt += 1
     outfile.write('  %8d versioned %s\n' % (dir_cnt,
         plural(dir_cnt, 'subdirectory', 'subdirectories')))
@@ -270,7 +272,10 @@
 
 def _show_branch_stats(branch, verbose, outfile):
     """Show statistics about a branch."""
-    revno, head = branch.last_revision_info()
+    try:
+        revno, head = branch.last_revision_info()
+    except errors.UnsupportedOperation:
+        return {}
     outfile.write('\n')
     outfile.write('Branch history:\n')
     outfile.write('  %8d revision%s\n' % (revno, plural(revno)))
@@ -332,7 +337,7 @@
                 repository = a_bzrdir.open_repository()
             except NoRepositoryPresent:
                 # Return silently; cmd_info already returned NotBranchError
-                # if no bzrdir could be opened.
+                # if no controldir could be opened.
                 return
             else:
                 lockable = repository
@@ -447,10 +452,10 @@
         branch.user_url != tree.user_url):
         branch = None
         repository = None
-    non_aliases = set(bzrdir.format_registry.keys())
-    non_aliases.difference_update(bzrdir.format_registry.aliases())
+    non_aliases = set(controldir.format_registry.keys())
+    non_aliases.difference_update(controldir.format_registry.aliases())
     for key in non_aliases:
-        format = bzrdir.format_registry.make_bzrdir(key)
+        format = controldir.format_registry.make_bzrdir(key)
         if isinstance(format, bzrdir.BzrDirMetaFormat1):
             if (tree and format.workingtree_format !=
                 tree._format):
@@ -468,7 +473,7 @@
         return 'unnamed'
     candidates.sort()
     new_candidates = [c for c in candidates if not
-        bzrdir.format_registry.get_info(c).hidden]
+        controldir.format_registry.get_info(c).hidden]
     if len(new_candidates) > 0:
         # If there are any non-hidden formats that match, only return those to
         # avoid listing hidden formats except when only a hidden format will
@@ -481,12 +486,11 @@
     """Hooks for the info command."""
 
     def __init__(self):
-        super(InfoHooks, self).__init__()
-        self.create_hook(_mod_hooks.HookPoint('repository',
+        super(InfoHooks, self).__init__("bzrlib.info", "hooks")
+        self.add_hook('repository',
             "Invoked when displaying the statistics for a repository. "
             "repository is called with a statistics dictionary as returned "
-            "by the repository and a file-like object to write to.", (1, 15), 
-            None))
+            "by the repository and a file-like object to write to.", (1, 15))
 
 
 hooks = InfoHooks()

=== modified file 'bzrlib/inter.py'
--- bzrlib/inter.py	2009-03-23 14:59:43 +0000
+++ bzrlib/inter.py	2011-05-19 09:32:38 +0000
@@ -18,6 +18,9 @@
 """Inter-object utility class."""
 
 
+from bzrlib.errors import NoCompatibleInter
+
+
 class InterObject(object):
     """This class represents operations taking place between two objects.
 
@@ -75,13 +78,14 @@
                        the InterObject instance.
         :param target: the object to be the 'target' member of
                        the InterObject instance.
+
         If an optimised worker exists it will be used otherwise
         a default Inter worker instance will be created.
         """
         for provider in reversed(klass._optimisers):
             if provider.is_compatible(source, target):
                 return provider(source, target)
-        return klass(source, target)
+        raise NoCompatibleInter(source, target)
 
     def lock_read(self):
         """Take out a logical read lock.

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2010-05-11 07:52:16 +0000
+++ bzrlib/inventory.py	2011-08-12 09:49:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,27 +31,27 @@
 lazy_import(globals(), """
 import collections
 import copy
-import os
 import re
 import tarfile
 
-import bzrlib
 from bzrlib import (
     chk_map,
     errors,
     generate_ids,
     osutils,
-    symbol_versioning,
     )
 """)
 
-from bzrlib.errors import (
-    BzrCheckError,
-    BzrError,
+from bzrlib import (
+    lazy_regex,
+    trace,
     )
-from bzrlib.symbol_versioning import deprecated_in, deprecated_method
-from bzrlib.trace import mutter
+
 from bzrlib.static_tuple import StaticTuple
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    deprecated_method,
+    )
 
 
 class InventoryEntry(object):
@@ -104,8 +104,6 @@
     InventoryDirectory('2325', 'wibble', parent_id='123', revision=None)
     >>> i.path2id('src/wibble')
     '2325'
-    >>> '2325' in i
-    True
     >>> i.add(InventoryFile('2326', 'wibble.c', '2325'))
     InventoryFile('2326', 'wibble.c', parent_id='2325', sha1=None, len=None, revision=None)
     >>> i['2326']
@@ -131,7 +129,23 @@
     RENAMED = 'renamed'
     MODIFIED_AND_RENAMED = 'modified and renamed'
 
-    __slots__ = []
+    __slots__ = ['file_id', 'revision', 'parent_id', 'name']
+
+    # Attributes that all InventoryEntry instances are expected to have, but
+    # that don't vary for all kinds of entry.  (e.g. symlink_target is only
+    # relevant to InventoryLink, so there's no reason to make every
+    # InventoryFile instance allocate space to hold a value for it.)
+    # Attributes that only vary for files: executable, text_sha1, text_size,
+    # text_id
+    executable = False
+    text_sha1 = None
+    text_size = None
+    text_id = None
+    # Attributes that only vary for symlinks: symlink_target
+    symlink_target = None
+    # Attributes that only vary for tree-references: reference_revision
+    reference_revision = None
+
 
     def detect_changes(self, old_entry):
         """Return a (text_modified, meta_modified) from this to old_entry.
@@ -158,7 +172,7 @@
         candidates = {}
         # identify candidate head revision ids.
         for inv in previous_inventories:
-            if self.file_id in inv:
+            if inv.has_id(self.file_id):
                 ie = inv[self.file_id]
                 if ie.revision in candidates:
                     # same revision value in two different inventories:
@@ -176,16 +190,6 @@
                     candidates[ie.revision] = ie
         return candidates
 
-    @deprecated_method(deprecated_in((1, 6, 0)))
-    def get_tar_item(self, root, dp, now, tree):
-        """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(osutils.pathjoin(root, dp).encode('utf8'))
-        # TODO: would be cool to actually set it to the timestamp of the
-        # revision it was last changed
-        item.mtime = now
-        fileobj = self._put_in_tar(item, tree)
-        return item, fileobj
-
     def has_text(self):
         """Return true if the object this entry represents has textual data.
 
@@ -197,7 +201,7 @@
         """
         return False
 
-    def __init__(self, file_id, name, parent_id, text_id=None):
+    def __init__(self, file_id, name, parent_id):
         """Create an InventoryEntry
 
         The filename must be a single component, relative to the
@@ -214,46 +218,17 @@
         """
         if '/' in name or '\\' in name:
             raise errors.InvalidEntryName(name=name)
-        self.executable = False
+        self.file_id = file_id
         self.revision = None
-        self.text_sha1 = None
-        self.text_size = None
-        self.file_id = file_id
         self.name = name
-        self.text_id = text_id
         self.parent_id = parent_id
-        self.symlink_target = None
-        self.reference_revision = None
 
     def kind_character(self):
         """Return a short kind indicator useful for appending to names."""
-        raise BzrError('unknown kind %r' % self.kind)
+        raise errors.BzrError('unknown kind %r' % self.kind)
 
     known_kinds = ('file', 'directory', 'symlink')
 
-    def _put_in_tar(self, item, tree):
-        """populate item for stashing in a tar, and return the content stream.
-
-        If no content is available, return None.
-        """
-        raise BzrError("don't know how to export {%s} of kind %r" %
-                       (self.file_id, self.kind))
-
-    @deprecated_method(deprecated_in((1, 6, 0)))
-    def put_on_disk(self, dest, dp, tree):
-        """Create a representation of self on disk in the prefix dest.
-
-        This is a template method - implement _put_on_disk in subclasses.
-        """
-        fullpath = osutils.pathjoin(dest, dp)
-        self._put_on_disk(fullpath, tree)
-        # mutter("  export {%s} kind %s to %s", self.file_id,
-        #         self.kind, fullpath)
-
-    def _put_on_disk(self, fullpath, tree):
-        """Put this entry onto disk at fullpath, from tree tree."""
-        raise BzrError("don't know how to export {%s} of kind %r" % (self.file_id, self.kind))
-
     def sorted_children(self):
         return sorted(self.children.items())
 
@@ -276,8 +251,9 @@
         """
         if self.parent_id is not None:
             if not inv.has_id(self.parent_id):
-                raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
-                        % (self.parent_id, rev_id))
+                raise errors.BzrCheckError(
+                    'missing parent {%s} in inventory for revision {%s}' % (
+                        self.parent_id, rev_id))
         checker._add_entry_to_text_key_references(inv, self)
         self._check(checker, rev_id)
 
@@ -397,47 +373,15 @@
         pass
 
 
-class RootEntry(InventoryEntry):
-
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
-
-    def _check(self, checker, rev_id):
-        """See InventoryEntry._check"""
-
-    def __init__(self, file_id):
-        self.file_id = file_id
-        self.children = {}
-        self.kind = 'directory'
-        self.parent_id = None
-        self.name = u''
-        self.revision = None
-        symbol_versioning.warn('RootEntry is deprecated as of bzr 0.10.'
-                               '  Please use InventoryDirectory instead.',
-                               DeprecationWarning, stacklevel=2)
-
-    def __eq__(self, other):
-        if not isinstance(other, RootEntry):
-            return NotImplemented
-
-        return (self.file_id == other.file_id) \
-               and (self.children == other.children)
-
-
 class InventoryDirectory(InventoryEntry):
     """A directory in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['children']
+
+    kind = 'directory'
 
     def _check(self, checker, rev_id):
         """See InventoryEntry._check"""
-        if (self.text_sha1 is not None or self.text_size is not None or
-            self.text_id is not None):
-            checker._report_items.append('directory {%s} has text in revision {%s}'
-                                % (self.file_id, rev_id))
         # In non rich root repositories we do not expect a file graph for the
         # root.
         if self.name == '' and not checker.rich_roots:
@@ -459,32 +403,25 @@
     def __init__(self, file_id, name, parent_id):
         super(InventoryDirectory, self).__init__(file_id, name, parent_id)
         self.children = {}
-        self.kind = 'directory'
 
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return '/'
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.DIRTYPE
-        fileobj = None
-        item.name += '/'
-        item.size = 0
-        item.mode = 0755
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        os.mkdir(fullpath)
-
 
 class InventoryFile(InventoryEntry):
     """A file in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['text_sha1', 'text_size', 'text_id', 'executable']
+
+    kind = 'file'
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryFile, self).__init__(file_id, name, parent_id)
+        self.text_sha1 = None
+        self.text_size = None
+        self.text_id = None
+        self.executable = False
 
     def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
@@ -533,31 +470,10 @@
         """See InventoryEntry.has_text."""
         return True
 
-    def __init__(self, file_id, name, parent_id):
-        super(InventoryFile, self).__init__(file_id, name, parent_id)
-        self.kind = 'file'
-
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return ''
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.REGTYPE
-        fileobj = tree.get_file(self.file_id)
-        item.size = self.text_size
-        if tree.is_executable(self.file_id):
-            item.mode = 0755
-        else:
-            item.mode = 0644
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        osutils.pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
-        if tree.is_executable(self.file_id):
-            os.chmod(fullpath, 0755)
-
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.text_sha1 = work_tree.get_file_sha1(self.file_id, path=path)
@@ -595,16 +511,16 @@
 class InventoryLink(InventoryEntry):
     """A file in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision']
+    __slots__ = ['symlink_target']
+
+    kind = 'symlink'
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryLink, self).__init__(file_id, name, parent_id)
+        self.symlink_target = None
 
     def _check(self, checker, tree_revision_id):
         """See InventoryEntry._check"""
-        if self.text_sha1 is not None or self.text_size is not None or self.text_id is not None:
-            checker._report_items.append(
-               'symlink {%s} has text in revision {%s}'
-                    % (self.file_id, tree_revision_id))
         if self.symlink_target is None:
             checker._report_items.append(
                 'symlink {%s} has no target in revision {%s}'
@@ -625,7 +541,7 @@
         # FIXME: which _modified field should we use ? RBC 20051003
         text_modified = (self.symlink_target != old_entry.symlink_target)
         if text_modified:
-            mutter("    symlink target changed")
+            trace.mutter("    symlink target changed")
         meta_modified = False
         return text_modified, meta_modified
 
@@ -648,30 +564,10 @@
         differ = DiffSymlink(old_tree, new_tree, output_to)
         return differ.diff_symlink(old_target, new_target)
 
-    def __init__(self, file_id, name, parent_id):
-        super(InventoryLink, self).__init__(file_id, name, parent_id)
-        self.kind = 'symlink'
-
     def kind_character(self):
         """See InventoryEntry.kind_character."""
         return ''
 
-    def _put_in_tar(self, item, tree):
-        """See InventoryEntry._put_in_tar."""
-        item.type = tarfile.SYMTYPE
-        fileobj = None
-        item.size = 0
-        item.mode = 0755
-        item.linkname = self.symlink_target
-        return fileobj
-
-    def _put_on_disk(self, fullpath, tree):
-        """See InventoryEntry._put_on_disk."""
-        try:
-            os.symlink(self.symlink_target, fullpath)
-        except OSError,e:
-            raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
-
     def _read_tree_state(self, path, work_tree):
         """See InventoryEntry._read_tree_state."""
         self.symlink_target = work_tree.get_symlink_target(self.file_id)
@@ -689,6 +585,8 @@
 
 class TreeReference(InventoryEntry):
 
+    __slots__ = ['reference_revision']
+
     kind = 'tree-reference'
 
     def __init__(self, file_id, name, parent_id, revision=None,
@@ -733,15 +631,16 @@
     inserted, other than through the Inventory API.
     """
 
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def __contains__(self, file_id):
         """True if this entry contains a file with given id.
 
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
         InventoryFile('123', 'foo.c', parent_id='TREE_ROOT', sha1=None, len=None, revision=None)
-        >>> '123' in inv
+        >>> inv.has_id('123')
         True
-        >>> '456' in inv
+        >>> inv.has_id('456')
         False
 
         Note that this method along with __iter__ are not encouraged for use as
@@ -822,6 +721,14 @@
                 # if we finished all children, pop it off the stack
                 stack.pop()
 
+    def _preload_cache(self):
+        """Populate any caches, we are about to access all items.
+        
+        The default implementation does nothing, because CommonInventory doesn't
+        have a cache.
+        """
+        pass
+    
     def iter_entries_by_dir(self, from_dir=None, specific_file_ids=None,
         yield_parents=False):
         """Iterate over the entries in a directory first order.
@@ -840,6 +747,11 @@
             specific_file_ids = set(specific_file_ids)
         # TODO? Perhaps this should return the from_dir so that the root is
         # yielded? or maybe an option?
+        if from_dir is None and specific_file_ids is None:
+            # They are iterating from the root, and have not specified any
+            # specific entries to look at. All current callers fully consume the
+            # iterator, so we can safely assume we are accessing all entries
+            self._preload_cache()
         if from_dir is None:
             if self.root is None:
                 return
@@ -847,7 +759,7 @@
             if (not yield_parents and specific_file_ids is not None and
                 len(specific_file_ids) == 1):
                 file_id = list(specific_file_ids)[0]
-                if file_id in self:
+                if self.has_id(file_id):
                     yield self.id2path(file_id), self[file_id]
                 return
             from_dir = self.root
@@ -863,7 +775,7 @@
             parents = set()
             byid = self
             def add_ancestors(file_id):
-                if file_id not in byid:
+                if not byid.has_id(file_id):
                     return
                 parent_id = byid[file_id].parent_id
                 if parent_id is None:
@@ -913,14 +825,6 @@
                     file_id, self[file_id]))
         return delta
 
-    def _get_mutable_inventory(self):
-        """Returns a mutable copy of the object.
-
-        Some inventories are immutable, yet working trees, for example, needs
-        to mutate exisiting inventories instead of creating a new one.
-        """
-        raise NotImplementedError(self._get_mutable_inventory)
-
     def make_entry(self, kind, name, parent_id, file_id=None):
         """Simple thunk to bzrlib.inventory.make_entry."""
         return make_entry(kind, name, parent_id, file_id)
@@ -940,7 +844,8 @@
                 if ie.kind == 'directory':
                     descend(ie, child_path)
 
-        descend(self.root, u'')
+        if self.root is not None:
+            descend(self.root, u'')
         return accum
 
     def directories(self):
@@ -1060,7 +965,7 @@
 
     >>> inv.path2id('hello.c')
     '123-123'
-    >>> '123-123' in inv
+    >>> inv.has_id('123-123')
     True
 
     There are iterators over the contents:
@@ -1223,10 +1128,6 @@
             other.add(entry.copy())
         return other
 
-    def _get_mutable_inventory(self):
-        """See CommonInventory._get_mutable_inventory."""
-        return copy.deepcopy(self)
-
     def __iter__(self):
         """Iterate over all file-ids."""
         return iter(self._byid)
@@ -1272,8 +1173,9 @@
     def _add_child(self, entry):
         """Add an entry to the inventory, without adding it to its parent"""
         if entry.file_id in self._byid:
-            raise BzrError("inventory already contains entry with id {%s}" %
-                           entry.file_id)
+            raise errors.BzrError(
+                "inventory already contains entry with id {%s}" %
+                entry.file_id)
         self._byid[entry.file_id] = entry
         for child in getattr(entry, 'children', {}).itervalues():
             self._add_child(child)
@@ -1282,9 +1184,6 @@
     def add(self, entry):
         """Add entry to inventory.
 
-        To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this.
-
         :return: entry
         """
         if entry.file_id in self._byid:
@@ -1335,10 +1234,10 @@
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
         InventoryFile('123', 'foo.c', parent_id='TREE_ROOT', sha1=None, len=None, revision=None)
-        >>> '123' in inv
+        >>> inv.has_id('123')
         True
         >>> del inv['123']
-        >>> '123' in inv
+        >>> inv.has_id('123')
         False
         """
         ie = self[file_id]
@@ -1446,15 +1345,17 @@
         """
         new_name = ensure_normalized_name(new_name)
         if not is_valid_name(new_name):
-            raise BzrError("not an acceptable filename: %r" % new_name)
+            raise errors.BzrError("not an acceptable filename: %r" % new_name)
 
         new_parent = self._byid[new_parent_id]
         if new_name in new_parent.children:
-            raise BzrError("%r already exists in %r" % (new_name, self.id2path(new_parent_id)))
+            raise errors.BzrError("%r already exists in %r" %
+                (new_name, self.id2path(new_parent_id)))
 
         new_parent_idpath = self.get_idpath(new_parent_id)
         if file_id in new_parent_idpath:
-            raise BzrError("cannot move directory %r into a subdirectory of itself, %r"
+            raise errors.BzrError(
+                "cannot move directory %r into a subdirectory of itself, %r"
                     % (self.id2path(file_id), self.id2path(new_parent_id)))
 
         file_ie = self._byid[file_id]
@@ -1496,6 +1397,7 @@
     def __init__(self, search_key_name):
         CommonInventory.__init__(self)
         self._fileid_to_entry_cache = {}
+        self._fully_cached = False
         self._path_to_fileid_cache = {}
         self._search_key_name = search_key_name
         self.root_id = None
@@ -1588,8 +1490,8 @@
             if entry.kind == 'directory':
                 directories_to_expand.add(entry.file_id)
             interesting.add(entry.parent_id)
-            children_of_parent_id.setdefault(entry.parent_id, []
-                                             ).append(entry.file_id)
+            children_of_parent_id.setdefault(entry.parent_id, set()
+                                             ).add(entry.file_id)
 
         # Now, interesting has all of the direct parents, but not the
         # parents of those parents. It also may have some duplicates with
@@ -1603,8 +1505,8 @@
             next_parents = set()
             for entry in self._getitems(remaining_parents):
                 next_parents.add(entry.parent_id)
-                children_of_parent_id.setdefault(entry.parent_id, []
-                                                 ).append(entry.file_id)
+                children_of_parent_id.setdefault(entry.parent_id, set()
+                                                 ).add(entry.file_id)
             # Remove any search tips we've already processed
             remaining_parents = next_parents.difference(interesting)
             interesting.update(remaining_parents)
@@ -1623,8 +1525,8 @@
             for entry in self._getitems(next_file_ids):
                 if entry.kind == 'directory':
                     directories_to_expand.add(entry.file_id)
-                children_of_parent_id.setdefault(entry.parent_id, []
-                                                 ).append(entry.file_id)
+                children_of_parent_id.setdefault(entry.parent_id, set()
+                                                 ).add(entry.file_id)
         return interesting, children_of_parent_id
 
     def filter(self, specific_fileids):
@@ -1708,14 +1610,6 @@
         self._fileid_to_entry_cache[result.file_id] = result
         return result
 
-    def _get_mutable_inventory(self):
-        """See CommonInventory._get_mutable_inventory."""
-        entries = self.iter_entries()
-        inv = Inventory(None, self.revision_id)
-        for path, inv_entry in entries:
-            inv.add(inv_entry.copy())
-        return inv
-
     def create_by_apply_delta(self, inventory_delta, new_revision_id,
         propagate_caches=False):
         """Create a new CHKInventory by applying inventory_delta to this one.
@@ -2062,7 +1956,7 @@
 
     def iter_just_entries(self):
         """Iterate over all entries.
-        
+
         Unlike iter_entries(), just the entries are returned (not (path, ie))
         and the order of entries is undefined.
 
@@ -2076,6 +1970,59 @@
                 self._fileid_to_entry_cache[file_id] = ie
             yield ie
 
+    def _preload_cache(self):
+        """Make sure all file-ids are in _fileid_to_entry_cache"""
+        if self._fully_cached:
+            return # No need to do it again
+        # The optimal sort order is to use iteritems() directly
+        cache = self._fileid_to_entry_cache
+        for key, entry in self.id_to_entry.iteritems():
+            file_id = key[0]
+            if file_id not in cache:
+                ie = self._bytes_to_entry(entry)
+                cache[file_id] = ie
+            else:
+                ie = cache[file_id]
+        last_parent_id = last_parent_ie = None
+        pid_items = self.parent_id_basename_to_file_id.iteritems()
+        for key, child_file_id in pid_items:
+            if key == ('', ''): # This is the root
+                if child_file_id != self.root_id:
+                    raise ValueError('Data inconsistency detected.'
+                        ' We expected data with key ("","") to match'
+                        ' the root id, but %s != %s'
+                        % (child_file_id, self.root_id))
+                continue
+            parent_id, basename = key
+            ie = cache[child_file_id]
+            if parent_id == last_parent_id:
+                parent_ie = last_parent_ie
+            else:
+                parent_ie = cache[parent_id]
+            if parent_ie.kind != 'directory':
+                raise ValueError('Data inconsistency detected.'
+                    ' An entry in the parent_id_basename_to_file_id map'
+                    ' has parent_id {%s} but the kind of that object'
+                    ' is %r not "directory"' % (parent_id, parent_ie.kind))
+            if parent_ie._children is None:
+                parent_ie._children = {}
+            basename = basename.decode('utf-8')
+            if basename in parent_ie._children:
+                existing_ie = parent_ie._children[basename]
+                if existing_ie != ie:
+                    raise ValueError('Data inconsistency detected.'
+                        ' Two entries with basename %r were found'
+                        ' in the parent entry {%s}'
+                        % (basename, parent_id))
+            if basename != ie.name:
+                raise ValueError('Data inconsistency detected.'
+                    ' In the parent_id_basename_to_file_id map, file_id'
+                    ' {%s} is listed as having basename %r, but in the'
+                    ' id_to_entry map it is %r'
+                    % (child_file_id, basename, ie.name))
+            parent_ie._children[basename] = ie
+        self._fully_cached = True
+
     def iter_changes(self, basis):
         """Generate a Tree.iter_changes change list between this and basis.
 
@@ -2241,17 +2188,13 @@
 class CHKInventoryDirectory(InventoryDirectory):
     """A directory in an inventory."""
 
-    __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', '_children', 'executable',
-                 'revision', 'symlink_target', 'reference_revision',
-                 '_chk_inventory']
+    __slots__ = ['_children', '_chk_inventory']
 
     def __init__(self, file_id, name, parent_id, chk_inventory):
         # Don't call InventoryDirectory.__init__ - it isn't right for this
         # class.
         InventoryEntry.__init__(self, file_id, name, parent_id)
         self._children = None
-        self.kind = 'directory'
         self._chk_inventory = chk_inventory
 
     @property
@@ -2342,13 +2285,9 @@
     return name
 
 
-_NAME_RE = None
+_NAME_RE = lazy_regex.lazy_compile(r'^[^/\\]+$')
 
 def is_valid_name(name):
-    global _NAME_RE
-    if _NAME_RE is None:
-        _NAME_RE = re.compile(r'^[^/\\]+$')
-
     return bool(_NAME_RE.match(name))
 
 
@@ -2444,3 +2383,15 @@
             raise errors.InconsistentDelta(new_path, item[1],
                 "new_path with no entry")
         yield item
+
+
+def mutable_inventory_from_tree(tree):
+    """Create a new inventory that has the same contents as a specified tree.
+
+    :param tree: Revision tree to create inventory from
+    """
+    entries = tree.iter_entries_by_dir()
+    inv = Inventory(None, tree.get_revision_id())
+    for path, inv_entry in entries:
+        inv.add(inv_entry.copy())
+    return inv

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2010-06-04 03:09:35 +0000
+++ bzrlib/knit.py	2011-09-24 09:03:00 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,58 +56,55 @@
 from itertools import izip
 import operator
 import os
-import sys
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import gzip
+
 from bzrlib import (
-    annotate,
     debug,
     diff,
     graph as _mod_graph,
     index as _mod_index,
-    lru_cache,
     pack,
     patiencediff,
-    progress,
     static_tuple,
     trace,
     tsort,
     tuned_gzip,
     ui,
     )
+
+from bzrlib.repofmt import pack_repo
+from bzrlib.i18n import gettext
 """)
 from bzrlib import (
+    annotate,
     errors,
     osutils,
     )
 from bzrlib.errors import (
-    FileExists,
     NoSuchFile,
-    KnitError,
     InvalidRevisionId,
     KnitCorrupt,
     KnitHeaderError,
     RevisionNotPresent,
-    RevisionAlreadyPresent,
     SHA1KnitCorrupt,
     )
 from bzrlib.osutils import (
     contains_whitespace,
-    contains_linebreaks,
     sha_string,
     sha_strings,
     split_lines,
     )
 from bzrlib.versionedfile import (
+    _KeyRefs,
     AbsentContentFactory,
     adapter_registry,
     ConstantMapper,
     ContentFactory,
-    ChunkedContentFactory,
     sort_groupcompress,
-    VersionedFile,
-    VersionedFiles,
+    VersionedFilesWithFallbacks,
     )
 
 
@@ -412,7 +409,7 @@
 class KnitContent(object):
     """Content of a knit version to which deltas can be applied.
 
-    This is always stored in memory as a list of lines with \n at the end,
+    This is always stored in memory as a list of lines with \\n at the end,
     plus a flag saying if the final ending is really there or not, because that
     corresponds to the on-disk knit representation.
     """
@@ -805,7 +802,7 @@
         writer.begin()
         index = _KnitGraphIndex(graph_index, lambda:True, parents=parents,
             deltas=delta, add_callback=graph_index.add_nodes)
-        access = _DirectPackAccess({})
+        access = pack_repo._DirectPackAccess({})
         access.set_writer(writer, graph_index, (transport, 'newpack'))
         result = KnitVersionedFiles(index, access,
             max_delta_chain=max_delta_chain)
@@ -849,7 +846,7 @@
                 in all_build_index_memos.itervalues()])
 
 
-class KnitVersionedFiles(VersionedFiles):
+class KnitVersionedFiles(VersionedFilesWithFallbacks):
     """Storage for many versioned files using knit compression.
 
     Backend storage is managed by indices and data objects.
@@ -882,7 +879,7 @@
             self._factory = KnitAnnotateFactory()
         else:
             self._factory = KnitPlainFactory()
-        self._fallback_vfs = []
+        self._immediate_fallback_vfs = []
         self._reload_func = reload_func
 
     def __repr__(self):
@@ -891,12 +888,18 @@
             self._index,
             self._access)
 
+    def without_fallbacks(self):
+        """Return a clone of this object without any fallbacks configured."""
+        return KnitVersionedFiles(self._index, self._access,
+            self._max_delta_chain, self._factory.annotated,
+            self._reload_func)
+
     def add_fallback_versioned_files(self, a_versioned_files):
         """Add a source of texts for texts not present in this knit.
 
         :param a_versioned_files: A VersionedFiles object.
         """
-        self._fallback_vfs.append(a_versioned_files)
+        self._immediate_fallback_vfs.append(a_versioned_files)
 
     def add_lines(self, key, parents, lines, parent_texts=None,
         left_matching_blocks=None, nostore_sha=None, random_id=False,
@@ -1069,7 +1072,7 @@
                     raise errors.KnitCorrupt(self,
                         "Missing basis parent %s for %s" % (
                         compression_parent, key))
-        for fallback_vfs in self._fallback_vfs:
+        for fallback_vfs in self._immediate_fallback_vfs:
             fallback_vfs.check()
 
     def _check_add(self, key, lines, random_id, check_content):
@@ -1153,14 +1156,15 @@
 
         A dict of key to (record_details, index_memo, next, parents) is
         returned.
-        method is the way referenced data should be applied.
-        index_memo is the handle to pass to the data access to actually get the
-            data
-        next is the build-parent of the version, or None for fulltexts.
-        parents is the version_ids of the parents of this version
-
-        :param allow_missing: If True do not raise an error on a missing component,
-            just ignore it.
+
+        * method is the way referenced data should be applied.
+        * index_memo is the handle to pass to the data access to actually get
+          the data
+        * next is the build-parent of the version, or None for fulltexts.
+        * parents is the version_ids of the parents of this version
+
+        :param allow_missing: If True do not raise an error on a missing
+            component, just ignore it.
         """
         component_data = {}
         pending_components = keys
@@ -1192,19 +1196,6 @@
         generator = _VFContentMapGenerator(self, [key])
         return generator._get_content(key)
 
-    def get_known_graph_ancestry(self, keys):
-        """Get a KnownGraph instance with the ancestry of keys."""
-        parent_map, missing_keys = self._index.find_ancestry(keys)
-        for fallback in self._fallback_vfs:
-            if not missing_keys:
-                break
-            (f_parent_map, f_missing_keys) = fallback._index.find_ancestry(
-                                                missing_keys)
-            parent_map.update(f_parent_map)
-            missing_keys = f_missing_keys
-        kg = _mod_graph.KnownGraph(parent_map)
-        return kg
-
     def get_parent_map(self, keys):
         """Get a map of the graph parents of keys.
 
@@ -1225,7 +1216,7 @@
             and so on.
         """
         result = {}
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         source_results = []
         missing = set(keys)
         for source in sources:
@@ -1241,15 +1232,13 @@
         """Produce a dictionary of knit records.
 
         :return: {key:(record, record_details, digest, next)}
-            record
-                data returned from read_records (a KnitContentobject)
-            record_details
-                opaque information to pass to parse_record
-            digest
-                SHA1 digest of the full text after all steps are done
-            next
-                build-parent of the version, i.e. the leftmost ancestor.
+
+            * record: data returned from read_records (a KnitContentobject)
+            * record_details: opaque information to pass to parse_record
+            * digest: SHA1 digest of the full text after all steps are done
+            * next: build-parent of the version, i.e. the leftmost ancestor.
                 Will be None if the record is not a delta.
+
         :param keys: The keys to build a map for
         :param allow_missing: If some records are missing, rather than
             error, just return the data that could be generated.
@@ -1525,7 +1514,7 @@
                         yield KnitContentFactory(key, global_map[key],
                             record_details, None, raw_data, self._factory.annotated, None)
                 else:
-                    vf = self._fallback_vfs[parent_maps.index(source) - 1]
+                    vf = self._immediate_fallback_vfs[parent_maps.index(source) - 1]
                     for record in vf.get_record_stream(keys, ordering,
                         include_delta_closure):
                         yield record
@@ -1541,7 +1530,7 @@
             # record entry 2 is the 'digest'.
             result[key] = details[2]
         missing.difference_update(set(result))
-        for source in self._fallback_vfs:
+        for source in self._immediate_fallback_vfs:
             if not missing:
                 break
             new_result = source.get_sha1s(missing)
@@ -1618,7 +1607,7 @@
                 raise RevisionNotPresent([record.key], self)
             elif ((record.storage_kind in knit_types)
                   and (compression_parent is None
-                       or not self._fallback_vfs
+                       or not self._immediate_fallback_vfs
                        or self._index.has_key(compression_parent)
                        or not self.has_key(compression_parent))):
                 # we can insert the knit record literally if either it has no
@@ -1772,7 +1761,7 @@
                         key_records.append((key, details[0]))
                 records_iter = enumerate(self._read_records_iter(key_records))
                 for (key_idx, (key, data, sha_value)) in records_iter:
-                    pb.update('Walking content', key_idx, total)
+                    pb.update(gettext('Walking content'), key_idx, total)
                     compression_parent = build_details[key][1]
                     if compression_parent is None:
                         # fulltext
@@ -1796,11 +1785,11 @@
         # vfs, and hope to find them there.  Note that if the keys are found
         # but had no changes or no content, the fallback may not return
         # anything.
-        if keys and not self._fallback_vfs:
+        if keys and not self._immediate_fallback_vfs:
             # XXX: strictly the second parameter is meant to be the file id
             # but it's not easily accessible here.
             raise RevisionNotPresent(keys, repr(self))
-        for source in self._fallback_vfs:
+        for source in self._immediate_fallback_vfs:
             if not keys:
                 break
             source_keys = set()
@@ -1808,7 +1797,7 @@
                 source_keys.add(key)
                 yield line, key
             keys.difference_update(source_keys)
-        pb.update('Walking content', total, total)
+        pb.update(gettext('Walking content'), total, total)
 
     def _make_line_delta(self, delta_seq, new_content):
         """Generate a line delta from delta_seq and new_content."""
@@ -1879,7 +1868,7 @@
         :return: the header and the decompressor stream.
                  as (stream, header_record)
         """
-        df = tuned_gzip.GzipFile(mode='rb', fileobj=StringIO(raw_data))
+        df = gzip.GzipFile(mode='rb', fileobj=StringIO(raw_data))
         try:
             # Current serialise
             rec = self._check_header(key, df.readline())
@@ -1894,7 +1883,7 @@
         # 4168 calls in 2880 217 internal
         # 4168 calls to _parse_record_header in 2121
         # 4168 calls to readlines in 330
-        df = tuned_gzip.GzipFile(mode='rb', fileobj=StringIO(data))
+        df = gzip.GzipFile(mode='rb', fileobj=StringIO(data))
         try:
             record_contents = df.readlines()
         except Exception, e:
@@ -1922,6 +1911,7 @@
         The result will be returned in whatever is the fastest to read.
         Not by the order requested. Also, multiple requests for the same
         record will only yield 1 response.
+
         :param records: A list of (key, access_memo) entries
         :return: Yields (key, contents, digest) in the order
                  read, not the order requested
@@ -1985,11 +1975,11 @@
         :param key: The key of the record. Currently keys are always serialised
             using just the trailing component.
         :param dense_lines: The bytes of lines but in a denser form. For
-            instance, if lines is a list of 1000 bytestrings each ending in \n,
-            dense_lines may be a list with one line in it, containing all the
-            1000's lines and their \n's. Using dense_lines if it is already
-            known is a win because the string join to create bytes in this
-            function spends less time resizing the final string.
+            instance, if lines is a list of 1000 bytestrings each ending in
+            \\n, dense_lines may be a list with one line in it, containing all
+            the 1000's lines and their \\n's. Using dense_lines if it is
+            already known is a win because the string join to create bytes in
+            this function spends less time resizing the final string.
         :return: (len, a StringIO instance with the raw data ready to read.)
         """
         chunks = ["version %s %d %s\n" % (key[-1], len(lines), digest)]
@@ -2015,7 +2005,7 @@
         """See VersionedFiles.keys."""
         if 'evil' in debug.debug_flags:
             trace.mutter_callsite(2, "keys scales with size of history")
-        sources = [self._index] + self._fallback_vfs
+        sources = [self._index] + self._immediate_fallback_vfs
         result = set()
         for source in sources:
             result.update(source.keys())
@@ -2061,7 +2051,7 @@
 
         missing_keys = set(nonlocal_keys)
         # Read from remote versioned file instances and provide to our caller.
-        for source in self.vf._fallback_vfs:
+        for source in self.vf._immediate_fallback_vfs:
             if not missing_keys:
                 break
             # Loop over fallback repositories asking them for texts - ignore
@@ -2786,64 +2776,6 @@
         return key[:-1], key[-1]
 
 
-class _KeyRefs(object):
-
-    def __init__(self, track_new_keys=False):
-        # dict mapping 'key' to 'set of keys referring to that key'
-        self.refs = {}
-        if track_new_keys:
-            # set remembering all new keys
-            self.new_keys = set()
-        else:
-            self.new_keys = None
-
-    def clear(self):
-        if self.refs:
-            self.refs.clear()
-        if self.new_keys:
-            self.new_keys.clear()
-
-    def add_references(self, key, refs):
-        # Record the new references
-        for referenced in refs:
-            try:
-                needed_by = self.refs[referenced]
-            except KeyError:
-                needed_by = self.refs[referenced] = set()
-            needed_by.add(key)
-        # Discard references satisfied by the new key
-        self.add_key(key)
-
-    def get_new_keys(self):
-        return self.new_keys
-    
-    def get_unsatisfied_refs(self):
-        return self.refs.iterkeys()
-
-    def _satisfy_refs_for_key(self, key):
-        try:
-            del self.refs[key]
-        except KeyError:
-            # No keys depended on this key.  That's ok.
-            pass
-
-    def add_key(self, key):
-        # satisfy refs for key, and remember that we've seen this key.
-        self._satisfy_refs_for_key(key)
-        if self.new_keys is not None:
-            self.new_keys.add(key)
-
-    def satisfy_refs_for_keys(self, keys):
-        for key in keys:
-            self._satisfy_refs_for_key(key)
-
-    def get_referrers(self):
-        result = set()
-        for referrers in self.refs.itervalues():
-            result.update(referrers)
-        return result
-
-
 class _KnitGraphIndex(object):
     """A KnitVersionedFiles index layered on GraphIndex."""
 
@@ -3278,145 +3210,6 @@
                 yield data
 
 
-class _DirectPackAccess(object):
-    """Access to data in one or more packs with less translation."""
-
-    def __init__(self, index_to_packs, reload_func=None, flush_func=None):
-        """Create a _DirectPackAccess object.
-
-        :param index_to_packs: A dict mapping index objects to the transport
-            and file names for obtaining data.
-        :param reload_func: A function to call if we determine that the pack
-            files have moved and we need to reload our caches. See
-            bzrlib.repo_fmt.pack_repo.AggregateIndex for more details.
-        """
-        self._container_writer = None
-        self._write_index = None
-        self._indices = index_to_packs
-        self._reload_func = reload_func
-        self._flush_func = flush_func
-
-    def add_raw_records(self, key_sizes, raw_data):
-        """Add raw knit bytes to a storage area.
-
-        The data is spooled to the container writer in one bytes-record per
-        raw data item.
-
-        :param sizes: An iterable of tuples containing the key and size of each
-            raw data segment.
-        :param raw_data: A bytestring containing the data.
-        :return: A list of memos to retrieve the record later. Each memo is an
-            opaque index memo. For _DirectPackAccess the memo is (index, pos,
-            length), where the index field is the write_index object supplied
-            to the PackAccess object.
-        """
-        if type(raw_data) is not str:
-            raise AssertionError(
-                'data must be plain bytes was %s' % type(raw_data))
-        result = []
-        offset = 0
-        for key, size in key_sizes:
-            p_offset, p_length = self._container_writer.add_bytes_record(
-                raw_data[offset:offset+size], [])
-            offset += size
-            result.append((self._write_index, p_offset, p_length))
-        return result
-
-    def flush(self):
-        """Flush pending writes on this access object.
-
-        This will flush any buffered writes to a NewPack.
-        """
-        if self._flush_func is not None:
-            self._flush_func()
-            
-    def get_raw_records(self, memos_for_retrieval):
-        """Get the raw bytes for a records.
-
-        :param memos_for_retrieval: An iterable containing the (index, pos,
-            length) memo for retrieving the bytes. The Pack access method
-            looks up the pack to use for a given record in its index_to_pack
-            map.
-        :return: An iterator over the bytes of the records.
-        """
-        # first pass, group into same-index requests
-        request_lists = []
-        current_index = None
-        for (index, offset, length) in memos_for_retrieval:
-            if current_index == index:
-                current_list.append((offset, length))
-            else:
-                if current_index is not None:
-                    request_lists.append((current_index, current_list))
-                current_index = index
-                current_list = [(offset, length)]
-        # handle the last entry
-        if current_index is not None:
-            request_lists.append((current_index, current_list))
-        for index, offsets in request_lists:
-            try:
-                transport, path = self._indices[index]
-            except KeyError:
-                # A KeyError here indicates that someone has triggered an index
-                # reload, and this index has gone missing, we need to start
-                # over.
-                if self._reload_func is None:
-                    # If we don't have a _reload_func there is nothing that can
-                    # be done
-                    raise
-                raise errors.RetryWithNewPacks(index,
-                                               reload_occurred=True,
-                                               exc_info=sys.exc_info())
-            try:
-                reader = pack.make_readv_reader(transport, path, offsets)
-                for names, read_func in reader.iter_records():
-                    yield read_func(None)
-            except errors.NoSuchFile:
-                # A NoSuchFile error indicates that a pack file has gone
-                # missing on disk, we need to trigger a reload, and start over.
-                if self._reload_func is None:
-                    raise
-                raise errors.RetryWithNewPacks(transport.abspath(path),
-                                               reload_occurred=False,
-                                               exc_info=sys.exc_info())
-
-    def set_writer(self, writer, index, transport_packname):
-        """Set a writer to use for adding data."""
-        if index is not None:
-            self._indices[index] = transport_packname
-        self._container_writer = writer
-        self._write_index = index
-
-    def reload_or_raise(self, retry_exc):
-        """Try calling the reload function, or re-raise the original exception.
-
-        This should be called after _DirectPackAccess raises a
-        RetryWithNewPacks exception. This function will handle the common logic
-        of determining when the error is fatal versus being temporary.
-        It will also make sure that the original exception is raised, rather
-        than the RetryWithNewPacks exception.
-
-        If this function returns, then the calling function should retry
-        whatever operation was being performed. Otherwise an exception will
-        be raised.
-
-        :param retry_exc: A RetryWithNewPacks exception.
-        """
-        is_error = False
-        if self._reload_func is None:
-            is_error = True
-        elif not self._reload_func():
-            # The reload claimed that nothing changed
-            if not retry_exc.reload_occurred:
-                # If there wasn't an earlier reload, then we really were
-                # expecting to find changes. We didn't find them, so this is a
-                # hard error
-                is_error = True
-        if is_error:
-            exc_class, exc_value, exc_traceback = retry_exc.exc_info
-            raise exc_class, exc_value, exc_traceback
-
-
 def annotate_knit(knit, revision_id):
     """Annotate a knit with no cached annotations.
 
@@ -3520,8 +3313,8 @@
         return records, ann_keys
 
     def _get_needed_texts(self, key, pb=None):
-        # if True or len(self._vf._fallback_vfs) > 0:
-        if len(self._vf._fallback_vfs) > 0:
+        # if True or len(self._vf._immediate_fallback_vfs) > 0:
+        if len(self._vf._immediate_fallback_vfs) > 0:
             # If we have fallbacks, go to the generic path
             for v in annotate.Annotator._get_needed_texts(self, key, pb=pb):
                 yield v
@@ -3532,7 +3325,7 @@
                 for idx, (sub_key, text, num_lines) in enumerate(
                                                 self._extract_texts(records)):
                     if pb is not None:
-                        pb.update('annotating', idx, len(records))
+                        pb.update(gettext('annotating'), idx, len(records))
                     yield sub_key, text, num_lines
                 for sub_key in ann_keys:
                     text = self._text_cache[sub_key]

=== modified file 'bzrlib/lazy_import.py'
--- bzrlib/lazy_import.py	2009-03-23 14:59:43 +0000
+++ bzrlib/lazy_import.py	2011-08-19 18:02:37 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,8 @@
 This includes waiting to import a module until it is actually used.
 
 Most commonly, the 'lazy_import' function is used to import other modules
-in an on-demand fashion. Typically use looks like:
+in an on-demand fashion. Typically use looks like::
+
     from bzrlib.lazy_import import lazy_import
     lazy_import(globals(), '''
     from bzrlib import (
@@ -30,14 +31,14 @@
     import bzrlib.branch
     ''')
 
-    Then 'errors, osutils, branch' and 'bzrlib' will exist as lazy-loaded
-    objects which will be replaced with a real object on first use.
+Then 'errors, osutils, branch' and 'bzrlib' will exist as lazy-loaded
+objects which will be replaced with a real object on first use.
 
-    In general, it is best to only load modules in this way. This is because
-    it isn't safe to pass these variables to other functions before they
-    have been replaced. This is especially true for constants, sometimes
-    true for classes or functions (when used as a factory, or you want
-    to inherit from them).
+In general, it is best to only load modules in this way. This is because
+it isn't safe to pass these variables to other functions before they
+have been replaced. This is especially true for constants, sometimes
+true for classes or functions (when used as a factory, or you want
+to inherit from them).
 """
 
 
@@ -87,6 +88,9 @@
                           " to another variable?",
                 extra=e)
         obj = factory(self, scope, name)
+        if obj is self:
+            raise errors.IllegalUseOfScopeReplacer(name, msg="Object tried"
+                " to replace itself, check it's not using its own scope.")
         if ScopeReplacer._should_proxy:
             object.__setattr__(self, '_real_obj', obj)
         scope[name] = obj
@@ -94,8 +98,19 @@
 
     def _cleanup(self):
         """Stop holding on to all the extra stuff"""
-        del self._factory
-        del self._scope
+        try:
+            del self._factory
+        except AttributeError:
+            # Oops, we just lost a race with another caller of _cleanup.  Just
+            # ignore it.
+            pass
+
+        try:
+            del self._scope
+        except AttributeError:
+            # Another race loss.  See above.
+            pass
+
         # We keep _name, so that we can report errors
         # del self._name
 
@@ -157,10 +172,14 @@
             None, indicating the module is being imported.
         :param children: Children entries to be imported later.
             This should be a map of children specifications.
-            {'foo':(['bzrlib', 'foo'], None,
-                {'bar':(['bzrlib', 'foo', 'bar'], None {})})
-            }
-        Examples:
+            ::
+            
+                {'foo':(['bzrlib', 'foo'], None,
+                    {'bar':(['bzrlib', 'foo', 'bar'], None {})})
+                }
+
+        Examples::
+
             import foo => name='foo' module_path='foo',
                           member=None, children={}
             import foo.bar => name='foo' module_path='foo', member=None,
@@ -367,17 +386,18 @@
 def lazy_import(scope, text, lazy_import_class=None):
     """Create lazy imports for all of the imports in text.
 
-    This is typically used as something like:
-    from bzrlib.lazy_import import lazy_import
-    lazy_import(globals(), '''
-    from bzrlib import (
-        foo,
-        bar,
-        baz,
-        )
-    import bzrlib.branch
-    import bzrlib.transport
-    ''')
+    This is typically used as something like::
+
+        from bzrlib.lazy_import import lazy_import
+        lazy_import(globals(), '''
+        from bzrlib import (
+            foo,
+            bar,
+            baz,
+            )
+        import bzrlib.branch
+        import bzrlib.transport
+        ''')
 
     Then 'foo, bar, baz' and 'bzrlib' will exist as lazy-loaded
     objects which will be replaced with a real object on first use.

=== modified file 'bzrlib/lazy_regex.py'
--- bzrlib/lazy_regex.py	2010-06-30 12:03:30 +0000
+++ bzrlib/lazy_regex.py	2011-06-13 22:39:49 +0000
@@ -16,8 +16,12 @@
 
 """Lazily compiled regex objects.
 
-This module defines a class which creates proxy objects for regex compilation.
-This allows overriding re.compile() to return lazily compiled objects.
+This module defines a class which creates proxy objects for regex
+compilation.  This allows overriding re.compile() to return lazily compiled
+objects.  
+
+We do this rather than just providing a new interface so that it will also
+be used by existing Python modules that create regexs.
 """
 
 import re
@@ -44,8 +48,8 @@
     def __init__(self, args=(), kwargs={}):
         """Create a new proxy object, passing in the args to pass to re.compile
 
-        :param args: The *args to pass to re.compile
-        :param kwargs: The **kwargs to pass to re.compile
+        :param args: The `*args` to pass to re.compile
+        :param kwargs: The `**kwargs` to pass to re.compile
         """
         self._real_regex = None
         self._regex_args = args

=== modified file 'bzrlib/library_state.py'
--- bzrlib/library_state.py	2010-06-26 01:07:16 +0000
+++ bzrlib/library_state.py	2011-09-26 15:40:02 +0000
@@ -23,6 +23,10 @@
 import sys
 
 import bzrlib
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import config
+""")
 
 
 class BzrLibraryState(object):
@@ -31,7 +35,7 @@
     This is the core state needed to make use of bzr. The current instance is
     currently always exposed as bzrlib.global_state, but we desired to move
     to a point where no global state is needed at all.
-    
+
     :ivar saved_state: The bzrlib.global_state at the time __enter__ was
         called.
     :ivar cleanups: An ObjectWithCleanups which can be used for cleanups that
@@ -61,8 +65,18 @@
         """
         self._ui = ui
         self._trace = trace
+        # There is no overrides by default, they are set later when the command
+        # arguments are parsed.
+        self.cmdline_overrides = config.CommandLineSection()
+        self.started = False
 
     def __enter__(self):
+        if not self.started:
+            self._start()
+        return self # This is bound to the 'as' clause in a with statement.
+
+    def _start(self):
+        """Do all initialization."""
         # NB: This function tweaks so much global state it's hard to test it in
         # isolation within the same interpreter.  It's not reached on normal
         # in-process run_bzr calls.  If it's broken, we expect that
@@ -86,7 +100,7 @@
 
         self.saved_state = bzrlib.global_state
         bzrlib.global_state = self
-        return self # This is bound to the 'as' clause in a with statement.
+        self.started = True
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.cleanups.cleanup_now()

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2010-05-11 08:36:16 +0000
+++ bzrlib/lock.py	2011-09-19 15:59:40 +0000
@@ -45,22 +45,22 @@
     osutils,
     trace,
     )
-from bzrlib.hooks import HookPoint, Hooks
-
+from bzrlib.hooks import Hooks
+from bzrlib.i18n import gettext
 
 class LockHooks(Hooks):
 
     def __init__(self):
-        Hooks.__init__(self)
-        self.create_hook(HookPoint('lock_acquired',
-            "Called with a bzrlib.lock.LockResult when a physical lock is "
-            "acquired.", (1, 8), None))
-        self.create_hook(HookPoint('lock_released',
-            "Called with a bzrlib.lock.LockResult when a physical lock is "
-            "released.", (1, 8), None))
-        self.create_hook(HookPoint('lock_broken',
-            "Called with a bzrlib.lock.LockResult when a physical lock is "
-            "broken.", (1, 15), None))
+        Hooks.__init__(self, "bzrlib.lock", "Lock.hooks")
+        self.add_hook('lock_acquired',
+            "Called with a bzrlib.lock.LockResult when a physical lock is "
+            "acquired.", (1, 8))
+        self.add_hook('lock_released',
+            "Called with a bzrlib.lock.LockResult when a physical lock is "
+            "released.", (1, 8))
+        self.add_hook('lock_broken',
+            "Called with a bzrlib.lock.LockResult when a physical lock is "
+            "broken.", (1, 15))
 
 
 class Lock(object):
@@ -171,12 +171,6 @@
             self.f.close()
             self.f = None
 
-    def __del__(self):
-        if self.f:
-            from warnings import warn
-            warn("lock on %r not released" % self.f)
-            self.unlock()
-
     def unlock(self):
         raise NotImplementedError()
 
@@ -541,7 +535,7 @@
     locked the same way), and -Drelock is set, then this will trace.note a
     message about it.
     """
-    
+
     _prev_lock = None
 
     def _note_lock(self, lock_type):
@@ -550,6 +544,6 @@
                 type_name = 'read'
             else:
                 type_name = 'write'
-            trace.note('%r was %s locked again', self, type_name)
+            trace.note(gettext('{0!r} was {1} locked again'), self, type_name)
         self._prev_lock = lock_type
 

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2010-04-30 11:03:59 +0000
+++ bzrlib/lockable_files.py	2011-06-28 21:47:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import codecs
 import warnings
 
 from bzrlib import (
@@ -34,27 +33,6 @@
     )
 
 
-# XXX: The tracking here of lock counts and whether the lock is held is
-# somewhat redundant with what's done in LockDir; the main difference is that
-# LockableFiles permits reentrancy.
-
-class _LockWarner(object):
-    """Hold a counter for a lock and warn if GCed while the count is >= 1.
-
-    This is separate from LockableFiles because putting a __del__ on
-    LockableFiles can result in uncollectable cycles.
-    """
-
-    def __init__(self, repr):
-        self.lock_count = 0
-        self.repr = repr
-
-    def __del__(self):
-        if self.lock_count >= 1:
-            # There should have been a try/finally to unlock this.
-            warnings.warn("%r was gc'd while locked" % self.repr)
-
-
 class LockableFiles(object):
     """Object representing a set of related files locked within the same scope.
 
@@ -69,17 +47,14 @@
     This class is now deprecated; code should move to using the Transport
     directly for file operations and using the lock or CountedLock for
     locking.
-    
+
     :ivar _lock: The real underlying lock (e.g. a LockDir)
-    :ivar _counted_lock: A lock decorated with a semaphore, so that it 
-        can be re-entered.
+    :ivar _lock_count: If _lock_mode is true, a positive count of the number
+        of times the lock has been taken (and not yet released) *by this
+        process*, through this particular object instance.
+    :ivar _lock_mode: None, or 'r' or 'w'
     """
 
-    # _lock_mode: None, or 'r' or 'w'
-
-    # _lock_count: If _lock_mode is true, a positive count of the number of
-    # times the lock has been taken *by this process*.
-
     def __init__(self, transport, lock_name, lock_class):
         """Create a LockableFiles group
 
@@ -93,7 +68,7 @@
         self.lock_name = lock_name
         self._transaction = None
         self._lock_mode = None
-        self._lock_warner = _LockWarner(repr(self))
+        self._lock_count = 0
         self._find_modes()
         esc_name = self._escape(lock_name)
         self._lock = lock_class(transport, esc_name,
@@ -112,6 +87,7 @@
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__,
                            self._transport)
+
     def __str__(self):
         return 'LockableFiles(%s, %s)' % (self.lock_name, self._transport.base)
 
@@ -175,19 +151,18 @@
         some other way, and need to synchronise this object's state with that
         fact.
         """
-        # TODO: Upgrade locking to support using a Transport,
-        # and potentially a remote locking protocol
         if self._lock_mode:
-            if self._lock_mode != 'w' or not self.get_transaction().writeable():
+            if (self._lock_mode != 'w'
+                or not self.get_transaction().writeable()):
                 raise errors.ReadOnlyError(self)
             self._lock.validate_token(token)
-            self._lock_warner.lock_count += 1
+            self._lock_count += 1
             return self._token_from_lock
         else:
             token_from_lock = self._lock.lock_write(token=token)
             #traceback.print_stack()
             self._lock_mode = 'w'
-            self._lock_warner.lock_count = 1
+            self._lock_count = 1
             self._set_write_transaction()
             self._token_from_lock = token_from_lock
             return token_from_lock
@@ -196,12 +171,12 @@
         if self._lock_mode:
             if self._lock_mode not in ('r', 'w'):
                 raise ValueError("invalid lock mode %r" % (self._lock_mode,))
-            self._lock_warner.lock_count += 1
+            self._lock_count += 1
         else:
             self._lock.lock_read()
             #traceback.print_stack()
             self._lock_mode = 'r'
-            self._lock_warner.lock_count = 1
+            self._lock_count = 1
             self._set_read_transaction()
 
     def _set_read_transaction(self):
@@ -218,23 +193,19 @@
     def unlock(self):
         if not self._lock_mode:
             return lock.cant_unlock_not_held(self)
-        if self._lock_warner.lock_count > 1:
-            self._lock_warner.lock_count -= 1
+        if self._lock_count > 1:
+            self._lock_count -= 1
         else:
             #traceback.print_stack()
             self._finish_transaction()
             try:
                 self._lock.unlock()
             finally:
-                self._lock_mode = self._lock_warner.lock_count = None
-
-    @property
-    def _lock_count(self):
-        return self._lock_warner.lock_count
+                self._lock_mode = self._lock_count = None
 
     def is_locked(self):
         """Return true if this LockableFiles group is locked"""
-        return self._lock_warner.lock_count >= 1
+        return self._lock_count >= 1
 
     def get_physical_lock_status(self):
         """Return physical lock status.
@@ -326,4 +297,3 @@
     def validate_token(self, token):
         if token is not None:
             raise errors.TokenLockingNotSupported(self)
-

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2010-06-15 09:53:42 +0000
+++ bzrlib/lockdir.py	2011-09-20 13:31:00 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -92,6 +92,10 @@
 >>> # do something here
 >>> l.unlock()
 
+Some classes of stale locks can be predicted by checking: the host name is the
+same as the local host name; the user name is the same as the local user; the
+process id no longer exists.  The check on user name is not strictly necessary
+but helps protect against colliding host names.
 """
 
 
@@ -103,16 +107,19 @@
 # the existing locking code and needs a new format of the containing object.
 # -- robertc, mbp 20070628
 
+import errno
 import os
 import time
 
 from bzrlib import (
+    config,
     debug,
     errors,
     lock,
     osutils,
+    ui,
+    urlutils,
     )
-import bzrlib.config
 from bzrlib.decorators import only_raises
 from bzrlib.errors import (
         DirectoryNotEmpty,
@@ -120,6 +127,7 @@
         LockBreakMismatch,
         LockBroken,
         LockContention,
+        LockCorrupt,
         LockFailed,
         LockNotHeld,
         NoSuchFile,
@@ -129,7 +137,7 @@
         )
 from bzrlib.trace import mutter, note
 from bzrlib.osutils import format_delta, rand_chars, get_host_name
-import bzrlib.ui
+from bzrlib.i18n import gettext
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -161,7 +169,8 @@
 
     __INFO_NAME = '/info'
 
-    def __init__(self, transport, path, file_modebits=0644, dir_modebits=0755):
+    def __init__(self, transport, path, file_modebits=0644, dir_modebits=0755,
+        extra_holder_info=None):
         """Create a new LockDir object.
 
         The LockDir is initially unlocked - this just creates the object.
@@ -170,6 +179,10 @@
 
         :param path: Path to the lock within the base directory of the
             transport.
+
+        :param extra_holder_info: If passed, {str:str} dict of extra or
+            updated information to insert into the info file when the lock is
+            taken.
         """
         self.transport = transport
         self.path = path
@@ -180,8 +193,9 @@
         self._held_info_path = self._held_dir + self.__INFO_NAME
         self._file_modebits = file_modebits
         self._dir_modebits = dir_modebits
-
         self._report_function = note
+        self.extra_holder_info = extra_holder_info
+        self._warned_about_lock_holder = None
 
     def __repr__(self):
         return '%s(%s%s)' % (self.__class__.__name__,
@@ -202,7 +216,6 @@
         except (TransportError, PathError), e:
             raise LockFailed(self, e)
 
-
     def _attempt_lock(self):
         """Make the pending directory and attempt to rename into place.
 
@@ -215,8 +228,8 @@
 
         :returns: The nonce of the lock, if it was successfully acquired.
 
-        :raises LockContention: If the lock is held by someone else.  The exception
-            contains the info of the current holder of the lock.
+        :raises LockContention: If the lock is held by someone else.  The
+            exception contains the info of the current holder of the lock.
         """
         self._trace("lock_write...")
         start_time = time.time()
@@ -225,17 +238,24 @@
         except (errors.TransportError, PathError), e:
             self._trace("... failed to create pending dir, %s", e)
             raise LockFailed(self, e)
-        try:
-            self.transport.rename(tmpname, self._held_dir)
-        except (errors.TransportError, PathError, DirectoryNotEmpty,
-                FileExists, ResourceBusy), e:
-            self._trace("... contention, %s", e)
-            self._remove_pending_dir(tmpname)
-            raise LockContention(self)
-        except Exception, e:
-            self._trace("... lock failed, %s", e)
-            self._remove_pending_dir(tmpname)
-            raise
+        while True:
+            try:
+                self.transport.rename(tmpname, self._held_dir)
+                break
+            except (errors.TransportError, PathError, DirectoryNotEmpty,
+                    FileExists, ResourceBusy), e:
+                self._trace("... contention, %s", e)
+                other_holder = self.peek()
+                self._trace("other holder is %r" % other_holder)
+                try:
+                    self._handle_lock_contention(other_holder)
+                except:
+                    self._remove_pending_dir(tmpname)
+                    raise
+            except Exception, e:
+                self._trace("... lock failed, %s", e)
+                self._remove_pending_dir(tmpname)
+                raise
         # We must check we really got the lock, because Launchpad's sftp
         # server at one time had a bug were the rename would successfully
         # move the new directory into the existing directory, which was
@@ -261,6 +281,31 @@
                 (time.time() - start_time) * 1000)
         return self.nonce
 
+    def _handle_lock_contention(self, other_holder):
+        """A lock we want to take is held by someone else.
+
+        This function can: tell the user about it; possibly detect that it's
+        safe or appropriate to steal the lock, or just raise an exception.
+
+        If this function returns (without raising an exception) the lock will
+        be attempted again.
+
+        :param other_holder: A LockHeldInfo for the current holder; note that
+            it might be None if the lock can be seen to be held but the info
+            can't be read.
+        """
+        if (other_holder is not None):
+            if other_holder.is_lock_holder_known_dead():
+                if self.get_config().get('locks.steal_dead'):
+                    ui.ui_factory.show_user_warning(
+                        'locks_steal_dead',
+                        lock_url=urlutils.join(self.transport.base, self.path),
+                        other_holder_info=unicode(other_holder))
+                    self.force_break(other_holder)
+                    self._trace("stole lock from dead holder")
+                    return
+        raise LockContention(self)
+
     def _remove_pending_dir(self, tmpname):
         """Remove the pending directory
 
@@ -272,7 +317,7 @@
             self.transport.delete(tmpname + self.__INFO_NAME)
             self.transport.rmdir(tmpname)
         except PathError, e:
-            note("error removing pending lock: %s", e)
+            note(gettext("error removing pending lock: %s"), e)
 
     def _create_pending_dir(self):
         tmpname = '%s/%s.tmp' % (self.path, rand_chars(10))
@@ -286,14 +331,14 @@
             self.create(mode=self._dir_modebits)
             # After creating the lock directory, try again
             self.transport.mkdir(tmpname)
-        self.nonce = rand_chars(20)
-        info_bytes = self._prepare_info()
+        info = LockHeldInfo.for_this_process(self.extra_holder_info)
+        self.nonce = info.get('nonce')
         # We use put_file_non_atomic because we just created a new unique
         # directory so we don't have to worry about files existing there.
         # We'll rename the whole directory into place to get atomic
         # properties
         self.transport.put_bytes_non_atomic(tmpname + self.__INFO_NAME,
-                                            info_bytes)
+            info.to_bytes())
         return tmpname
 
     @only_raises(LockNotHeld, LockBroken)
@@ -343,16 +388,29 @@
     def break_lock(self):
         """Break a lock not held by this instance of LockDir.
 
-        This is a UI centric function: it uses the bzrlib.ui.ui_factory to
+        This is a UI centric function: it uses the ui.ui_factory to
         prompt for input if a lock is detected and there is any doubt about
-        it possibly being still active.
+        it possibly being still active.  force_break is the non-interactive
+        version.
+
+        :returns: LockResult for the broken lock.
         """
         self._check_not_locked()
-        holder_info = self.peek()
+        try:
+            holder_info = self.peek()
+        except LockCorrupt, e:
+            # The lock info is corrupt.
+            if ui.ui_factory.get_boolean(u"Break (corrupt %r)" % (self,)):
+                self.force_break_corrupt(e.file_data)
+            return
         if holder_info is not None:
-            lock_info = '\n'.join(self._format_lock_info(holder_info))
-            if bzrlib.ui.ui_factory.get_boolean("Break %s" % lock_info):
-                self.force_break(holder_info)
+            if ui.ui_factory.confirm_action(
+                u"Break %(lock_info)s",
+                'bzrlib.lockdir.break',
+                dict(lock_info=unicode(holder_info))):
+                result = self.force_break(holder_info)
+                ui.ui_factory.show_message(
+                    "Broke lock %s" % result.lock_url)
 
     def force_break(self, dead_holder_info):
         """Release a lock held by another process.
@@ -361,16 +419,19 @@
         it still thinks it has the lock there will be two concurrent writers.
         In general the user's approval should be sought for lock breaks.
 
-        dead_holder_info must be the result of a previous LockDir.peek() call;
-        this is used to check that it's still held by the same process that
-        the user decided was dead.  If this is not the current holder,
-        LockBreakMismatch is raised.
-
         After the lock is broken it will not be held by any process.
         It is possible that another process may sneak in and take the
         lock before the breaking process acquires it.
+
+        :param dead_holder_info:
+            Must be the result of a previous LockDir.peek() call; this is used
+            to check that it's still held by the same process that the user
+            decided was dead.  If this is not the current holder,
+            LockBreakMismatch is raised.
+
+        :returns: LockResult for the broken lock.
         """
-        if not isinstance(dead_holder_info, dict):
+        if not isinstance(dead_holder_info, LockHeldInfo):
             raise ValueError("dead_holder_info: %r" % dead_holder_info)
         self._check_not_locked()
         current_info = self.peek()
@@ -394,6 +455,36 @@
                                  current_info.get('nonce'))
         for hook in self.hooks['lock_broken']:
             hook(result)
+        return result
+
+    def force_break_corrupt(self, corrupt_info_lines):
+        """Release a lock that has been corrupted.
+
+        This is very similar to force_break, it except it doesn't assume that
+        self.peek() can work.
+
+        :param corrupt_info_lines: the lines of the corrupted info file, used
+            to check that the lock hasn't changed between reading the (corrupt)
+            info file and calling force_break_corrupt.
+        """
+        # XXX: this copes with unparseable info files, but what about missing
+        # info files?  Or missing lock dirs?
+        self._check_not_locked()
+        tmpname = '%s/broken.%s.tmp' % (self.path, rand_chars(20))
+        self.transport.rename(self._held_dir, tmpname)
+        # check that we actually broke the right lock, not someone else;
+        # there's a small race window between checking it and doing the
+        # rename.
+        broken_info_path = tmpname + self.__INFO_NAME
+        broken_content = self.transport.get_bytes(broken_info_path)
+        broken_lines = osutils.split_lines(broken_content)
+        if broken_lines != corrupt_info_lines:
+            raise LockBreakMismatch(self, broken_lines, corrupt_info_lines)
+        self.transport.delete(broken_info_path)
+        self.transport.rmdir(tmpname)
+        result = lock.LockResult(self.transport.abspath(self.path))
+        for hook in self.hooks['lock_broken']:
+            hook(result)
 
     def _check_not_locked(self):
         """If the lock is held by this instance, raise an error."""
@@ -425,7 +516,8 @@
 
         peek() reads the info file of the lock holder, if any.
         """
-        return self._parse_info(self.transport.get_bytes(path))
+        return LockHeldInfo.from_info_file_bytes(
+            self.transport.get_bytes(path))
 
     def peek(self):
         """Check if the lock is held by anyone.
@@ -444,29 +536,6 @@
     def _prepare_info(self):
         """Write information about a pending lock to a temporary file.
         """
-        # XXX: is creating this here inefficient?
-        config = bzrlib.config.GlobalConfig()
-        try:
-            user = config.username()
-        except errors.NoWhoami:
-            user = osutils.getuser_unicode()
-        s = rio.Stanza(hostname=get_host_name(),
-                   pid=str(os.getpid()),
-                   start_time=str(int(time.time())),
-                   nonce=self.nonce,
-                   user=user,
-                   )
-        return s.to_string()
-
-    def _parse_info(self, info_bytes):
-        stanza = rio.read_stanza(osutils.split_lines(info_bytes))
-        if stanza is None:
-            # see bug 185013; we fairly often end up with the info file being
-            # empty after an interruption; we could log a message here but
-            # there may not be much we can say
-            return {}
-        else:
-            return stanza.as_dict()
 
     def attempt_lock(self):
         """Take the lock; fail if it's already held.
@@ -486,6 +555,17 @@
             hook(hook_result)
         return result
 
+    def lock_url_for_display(self):
+        """Give a nicely-printable representation of the URL of this lock."""
+        # As local lock urls are correct we display them.
+        # We avoid displaying remote lock urls.
+        lock_url = self.transport.abspath(self.path)
+        if lock_url.startswith('file://'):
+            lock_url = lock_url.split('.bzr/')[0]
+        else:
+            lock_url = ''
+        return lock_url
+
     def wait_lock(self, timeout=None, poll=None, max_attempts=None):
         """Wait a certain period for a lock.
 
@@ -515,6 +595,7 @@
         deadline_str = None
         last_info = None
         attempt_count = 0
+        lock_url = self.lock_url_for_display()
         while True:
             attempt_count += 1
             try:
@@ -531,35 +612,21 @@
             new_info = self.peek()
             if new_info is not None and new_info != last_info:
                 if last_info is None:
-                    start = 'Unable to obtain'
+                    start = gettext('Unable to obtain')
                 else:
-                    start = 'Lock owner changed for'
+                    start = gettext('Lock owner changed for')
                 last_info = new_info
-                formatted_info = self._format_lock_info(new_info)
+                msg = gettext('{0} lock {1} {2}.').format(start, lock_url,
+                                                                    new_info)
                 if deadline_str is None:
                     deadline_str = time.strftime('%H:%M:%S',
-                                                 time.localtime(deadline))
-                # As local lock urls are correct we display them.
-                # We avoid displaying remote lock urls.
-                lock_url = self.transport.abspath(self.path)
-                if lock_url.startswith('file://'):
-                    lock_url = lock_url.split('.bzr/')[0]
-                else:
-                    lock_url = ''
-                user, hostname, pid, time_ago = formatted_info
-                msg = ('%s lock %s '        # lock_url
-                    'held by '              # start
-                    '%s\n'                  # user
-                    'at %s '                # hostname
-                    '[process #%s], '       # pid
-                    'acquired %s.')         # time ago
-                msg_args = [start, lock_url, user, hostname, pid, time_ago]
+                                                    time.localtime(deadline))
                 if timeout > 0:
-                    msg += ('\nWill continue to try until %s, unless '
-                        'you press Ctrl-C.')
-                    msg_args.append(deadline_str)
-                msg += '\nSee "bzr help break-lock" for more.'
-                self._report_function(msg, *msg_args)
+                    msg += '\n' + gettext(
+                             'Will continue to try until %s, unless '
+                             'you press Ctrl-C.') % deadline_str
+                msg += '\n' + gettext('See "bzr help break-lock" for more.')
+                self._report_function(msg)
             if (max_attempts is not None) and (attempt_count >= max_attempts):
                 self._trace("exceeded %d attempts")
                 raise LockContention(self)
@@ -620,23 +687,6 @@
             raise LockContention(self)
         self._fake_read_lock = True
 
-    def _format_lock_info(self, info):
-        """Turn the contents of peek() into something for the user"""
-        start_time = info.get('start_time')
-        if start_time is None:
-            time_ago = '(unknown)'
-        else:
-            time_ago = format_delta(time.time() - int(info['start_time']))
-        user = info.get('user', '<unknown>')
-        hostname = info.get('hostname', '<unknown>')
-        pid = info.get('pid', '<unknown>')
-        return [
-            user,
-            hostname,
-            pid,
-            time_ago,
-            ]
-
     def validate_token(self, token):
         if token is not None:
             info = self.peek()
@@ -654,3 +704,159 @@
         if 'lock' not in debug.debug_flags:
             return
         mutter(str(self) + ": " + (format % args))
+
+    def get_config(self):
+        """Get the configuration that governs this lockdir."""
+        # XXX: This really should also use the locationconfig at least, but
+        # that seems a bit hard to hook up at the moment. -- mbp 20110329
+        # FIXME: The above is still true ;) -- vila 20110811
+        return config.GlobalStack()
+
+
+class LockHeldInfo(object):
+    """The information recorded about a held lock.
+
+    This information is recorded into the lock when it's taken, and it can be
+    read back by any process with access to the lockdir.  It can be used, for
+    example, to tell the user who holds the lock, or to try to detect whether
+    the lock holder is still alive.
+
+    Prior to bzr 2.4 a simple dict was used instead of an object.
+    """
+
+    def __init__(self, info_dict):
+        self.info_dict = info_dict
+
+    def __repr__(self):
+        """Return a debugging representation of this object."""
+        return "%s(%r)" % (self.__class__.__name__, self.info_dict)
+
+    def __unicode__(self):
+        """Return a user-oriented description of this object."""
+        d = self.to_readable_dict()
+        return ( gettext(
+            u'held by %(user)s on %(hostname)s (process #%(pid)s), '
+            u'acquired %(time_ago)s') % d)
+
+    def to_readable_dict(self):
+        """Turn the holder info into a dict of human-readable attributes.
+
+        For example, the start time is presented relative to the current time,
+        rather than as seconds since the epoch.
+
+        Returns a list of [user, hostname, pid, time_ago] all as readable
+        strings.
+        """
+        start_time = self.info_dict.get('start_time')
+        if start_time is None:
+            time_ago = '(unknown)'
+        else:
+            time_ago = format_delta(
+                time.time() - int(self.info_dict['start_time']))
+        user = self.info_dict.get('user', '<unknown>')
+        hostname = self.info_dict.get('hostname', '<unknown>')
+        pid = self.info_dict.get('pid', '<unknown>')
+        return dict(
+            user=user,
+            hostname=hostname,
+            pid=pid,
+            time_ago=time_ago)
+
+    def get(self, field_name):
+        """Return the contents of a field from the lock info, or None."""
+        return self.info_dict.get(field_name)
+
+    @classmethod
+    def for_this_process(cls, extra_holder_info):
+        """Return a new LockHeldInfo for a lock taken by this process.
+        """
+        info = dict(
+            hostname=get_host_name(),
+            pid=str(os.getpid()),
+            nonce=rand_chars(20),
+            start_time=str(int(time.time())),
+            user=get_username_for_lock_info(),
+            )
+        if extra_holder_info is not None:
+            info.update(extra_holder_info)
+        return cls(info)
+
+    def to_bytes(self):
+        s = rio.Stanza(**self.info_dict)
+        return s.to_string()
+
+    @classmethod
+    def from_info_file_bytes(cls, info_file_bytes):
+        """Construct from the contents of the held file."""
+        lines = osutils.split_lines(info_file_bytes)
+        try:
+            stanza = rio.read_stanza(lines)
+        except ValueError, e:
+            mutter('Corrupt lock info file: %r', lines)
+            raise LockCorrupt("could not parse lock info file: " + str(e),
+                lines)
+        if stanza is None:
+            # see bug 185013; we fairly often end up with the info file being
+            # empty after an interruption; we could log a message here but
+            # there may not be much we can say
+            return cls({})
+        else:
+            return cls(stanza.as_dict())
+
+    def __cmp__(self, other):
+        """Value comparison of lock holders."""
+        return (
+            cmp(type(self), type(other))
+            or cmp(self.info_dict, other.info_dict))
+
+    def is_locked_by_this_process(self):
+        """True if this process seems to be the current lock holder."""
+        return (
+            self.get('hostname') == get_host_name()
+            and self.get('pid') == str(os.getpid())
+            and self.get('user') == get_username_for_lock_info())
+
+    def is_lock_holder_known_dead(self):
+        """True if the lock holder process is known to be dead.
+
+        False if it's either known to be still alive, or if we just can't tell.
+
+        We can be fairly sure the lock holder is dead if it declared the same
+        hostname and there is no process with the given pid alive.  If people
+        have multiple machines with the same hostname this may cause trouble.
+
+        This doesn't check whether the lock holder is in fact the same process
+        calling this method.  (In that case it will return true.)
+        """
+        if self.get('hostname') != get_host_name():
+            return False
+        if self.get('hostname') == 'localhost':
+            # Too ambiguous.
+            return False
+        if self.get('user') != get_username_for_lock_info():
+            # Could well be another local process by a different user, but
+            # just to be safe we won't conclude about this either.
+            return False
+        pid_str = self.info_dict.get('pid', None)
+        if not pid_str:
+            mutter("no pid recorded in %r" % (self, ))
+            return False
+        try:
+            pid = int(pid_str)
+        except ValueError:
+            mutter("can't parse pid %r from %r"
+                % (pid_str, self))
+            return False
+        return osutils.is_local_pid_dead(pid)
+
+
+def get_username_for_lock_info():
+    """Get a username suitable for putting into a lock.
+
+    It's ok if what's written here is not a proper email address as long
+    as it gives some clue who the user is.
+    """
+    try:
+        return config.GlobalConfig().username()
+    except errors.NoWhoami:
+        return osutils.getuser_unicode()

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2010-07-16 13:29:07 +0000
+++ bzrlib/log.py	2011-10-14 13:56:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,33 +65,30 @@
 lazy_import(globals(), """
 
 from bzrlib import (
-    bzrdir,
     config,
+    controldir,
     diff,
     errors,
     foreign,
-    osutils,
     repository as _mod_repository,
     revision as _mod_revision,
     revisionspec,
-    trace,
     tsort,
     )
+from bzrlib.i18n import gettext, ngettext
 """)
 
 from bzrlib import (
+    lazy_regex,
     registry,
     )
 from bzrlib.osutils import (
     format_date,
     format_date_with_offset_in_original_timezone,
+    get_diff_header_encoding,
     get_terminal_encoding,
     terminal_width,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    )
 
 
 def find_touching_revisions(branch, file_id):
@@ -108,9 +105,12 @@
     last_ie = None
     last_path = None
     revno = 1
-    for revision_id in branch.revision_history():
+    graph = branch.repository.get_graph()
+    history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
+        [_mod_revision.NULL_REVISION]))
+    for revision_id in reversed(history):
         this_inv = branch.repository.get_inventory(revision_id)
-        if file_id in this_inv:
+        if this_inv.has_id(file_id):
             this_ie = this_inv[file_id]
             this_path = this_inv.id2path(file_id)
         else:
@@ -156,7 +156,8 @@
              end_revision=None,
              search=None,
              limit=None,
-             show_diff=False):
+             show_diff=False,
+             match=None):
     """Write out human-readable log of commits to this branch.
 
     This function is being retained for backwards compatibility but
@@ -185,6 +186,9 @@
         if None or 0.
 
     :param show_diff: If True, output a diff after each revision.
+
+    :param match: Dictionary of search lists to use when matching revision
+      properties.
     """
     # Convert old-style parameters to new-style parameters
     if specific_fileid is not None:
@@ -214,11 +218,11 @@
     Logger(branch, rqst).show(lf)
 
 
-# Note: This needs to be kept this in sync with the defaults in
+# Note: This needs to be kept in sync with the defaults in
 # make_log_request_dict() below
 _DEFAULT_REQUEST_PARAMS = {
     'direction': 'reverse',
-    'levels': 1,
+    'levels': None,
     'generate_tags': True,
     'exclude_common_ancestry': False,
     '_match_using_deltas': True,
@@ -227,10 +231,11 @@
 
 def make_log_request_dict(direction='reverse', specific_fileids=None,
                           start_revision=None, end_revision=None, limit=None,
-                          message_search=None, levels=1, generate_tags=True,
+                          message_search=None, levels=None, generate_tags=True,
                           delta_type=None,
                           diff_type=None, _match_using_deltas=True,
-                          exclude_common_ancestry=False,
+                          exclude_common_ancestry=False, match=None,
+                          signature=False, omit_merges=False,
                           ):
     """Convenience function for making a logging request dictionary.
 
@@ -257,10 +262,11 @@
       matching commit messages
 
     :param levels: the number of levels of revisions to
-      generate; 1 for just the mainline; 0 for all levels.
+      generate; 1 for just the mainline; 0 for all levels, or None for
+      a sensible default.
 
     :param generate_tags: If True, include tags for matched revisions.
-
+`
     :param delta_type: Either 'full', 'partial' or None.
       'full' means generate the complete delta - adds/deletes/modifies/etc;
       'partial' means filter the delta using specific_fileids;
@@ -278,19 +284,40 @@
 
     :param exclude_common_ancestry: Whether -rX..Y should be interpreted as a
       range operator or as a graph difference.
+
+    :param signature: show digital signature information
+
+    :param match: Dictionary of list of search strings to use when filtering
+      revisions. Keys can be 'message', 'author', 'committer', 'bugs' or
+      the empty string to match any of the preceding properties.
+
+    :param omit_merges: If True, commits with more than one parent are
+      omitted.
+
     """
+    # Take care of old style message_search parameter
+    if message_search:
+        if match:
+            if 'message' in match:
+                match['message'].append(message_search)
+            else:
+                match['message'] = [message_search]
+        else:
+            match={ 'message': [message_search] }
     return {
         'direction': direction,
         'specific_fileids': specific_fileids,
         'start_revision': start_revision,
         'end_revision': end_revision,
         'limit': limit,
-        'message_search': message_search,
         'levels': levels,
         'generate_tags': generate_tags,
         'delta_type': delta_type,
         'diff_type': diff_type,
         'exclude_common_ancestry': exclude_common_ancestry,
+        'signature': signature,
+        'match': match,
+        'omit_merges': omit_merges,
         # Add 'private' attributes for features that may be deprecated
         '_match_using_deltas': _match_using_deltas,
     }
@@ -298,12 +325,33 @@
 
 def _apply_log_request_defaults(rqst):
     """Apply default values to a request dictionary."""
-    result = _DEFAULT_REQUEST_PARAMS
+    result = _DEFAULT_REQUEST_PARAMS.copy()
     if rqst:
         result.update(rqst)
     return result
 
 
+def format_signature_validity(rev_id, repo):
+    """get the signature validity
+
+    :param rev_id: revision id to validate
+    :param repo: repository of revision
+    :return: human readable string to print to log
+    """
+    from bzrlib import gpg
+
+    gpg_strategy = gpg.GPGStrategy(None)
+    result = repo.verify_revision(rev_id, gpg_strategy)
+    if result[0] == gpg.SIGNATURE_VALID:
+        return "valid signature from {0}".format(result[1])
+    if result[0] == gpg.SIGNATURE_KEY_MISSING:
+        return "unknown key {0}".format(result[1])
+    if result[0] == gpg.SIGNATURE_NOT_VALID:
+        return "invalid signature!"
+    if result[0] == gpg.SIGNATURE_NOT_SIGNED:
+        return "no signature"
+
+
 class LogGenerator(object):
     """A generator of log revisions."""
 
@@ -354,13 +402,18 @@
         # Tweak the LogRequest based on what the LogFormatter can handle.
         # (There's no point generating stuff if the formatter can't display it.)
         rqst = self.rqst
-        rqst['levels'] = lf.get_levels()
+        if rqst['levels'] is None or lf.get_levels() > rqst['levels']:
+            # user didn't specify levels, use whatever the LF can handle:
+            rqst['levels'] = lf.get_levels()
+
         if not getattr(lf, 'supports_tags', False):
             rqst['generate_tags'] = False
         if not getattr(lf, 'supports_delta', False):
             rqst['delta_type'] = None
         if not getattr(lf, 'supports_diff', False):
             rqst['diff_type'] = None
+        if not getattr(lf, 'supports_signatures', False):
+            rqst['signature'] = False
 
         # Find and print the interesting revisions
         generator = self._generator_factory(self.branch, rqst)
@@ -370,7 +423,7 @@
 
     def _generator_factory(self, branch, rqst):
         """Make the LogGenerator object to use.
-        
+
         Subclasses may wish to override this.
         """
         return _DefaultLogGenerator(branch, rqst)
@@ -400,6 +453,8 @@
         levels = rqst.get('levels')
         limit = rqst.get('limit')
         diff_type = rqst.get('diff_type')
+        show_signature = rqst.get('signature')
+        omit_merges = rqst.get('omit_merges')
         log_count = 0
         revision_iterator = self._create_log_revision_iterator()
         for revs in revision_iterator:
@@ -407,12 +462,19 @@
                 # 0 levels means show everything; merge_depth counts from 0
                 if levels != 0 and merge_depth >= levels:
                     continue
+                if omit_merges and len(rev.parent_ids) > 1:
+                    continue
                 if diff_type is None:
                     diff = None
                 else:
                     diff = self._format_diff(rev, rev_id, diff_type)
+                if show_signature:
+                    signature = format_signature_validity(rev_id,
+                                                self.branch.repository)
+                else:
+                    signature = None
                 yield LogRevision(rev, revno, merge_depth, delta,
-                    self.rev_tag_dict.get(rev_id), diff)
+                    self.rev_tag_dict.get(rev_id), diff, signature)
                 if limit:
                     log_count += 1
                     if log_count >= limit:
@@ -432,7 +494,7 @@
         else:
             specific_files = None
         s = StringIO()
-        path_encoding = osutils.get_diff_header_encoding()
+        path_encoding = get_diff_header_encoding()
         diff.show_diff_trees(tree_1, tree_2, s, specific_files, old_label='',
             new_label='', path_encoding=path_encoding)
         return s.getvalue()
@@ -473,7 +535,7 @@
 
         # Apply the other filters
         return make_log_rev_iterator(self.branch, view_revisions,
-            rqst.get('delta_type'), rqst.get('message_search'),
+            rqst.get('delta_type'), rqst.get('match'),
             file_ids=rqst.get('specific_fileids'),
             direction=rqst.get('direction'))
 
@@ -492,7 +554,7 @@
             rqst.get('specific_fileids')[0], view_revisions,
             include_merges=rqst.get('levels') != 1)
         return make_log_rev_iterator(self.branch, view_revisions,
-            rqst.get('delta_type'), rqst.get('message_search'))
+            rqst.get('delta_type'), rqst.get('match'))
 
 
 def _calc_view_revisions(branch, start_rev_id, end_rev_id, direction,
@@ -506,10 +568,10 @@
              a list of the same tuples.
     """
     if (exclude_common_ancestry and start_rev_id == end_rev_id):
-        raise errors.BzrCommandError(
-            '--exclude-common-ancestry requires two different revisions')
+        raise errors.BzrCommandError(gettext(
+            '--exclude-common-ancestry requires two different revisions'))
     if direction not in ('reverse', 'forward'):
-        raise ValueError('invalid direction %r' % direction)
+        raise ValueError(gettext('invalid direction %r') % direction)
     br_revno, br_rev_id = branch.last_revision_info()
     if br_revno == 0:
         return []
@@ -540,8 +602,7 @@
         # It's the tip
         return [(br_rev_id, br_revno, 0)]
     else:
-        revno = branch.revision_id_to_dotted_revno(rev_id)
-        revno_str = '.'.join(str(n) for n in revno)
+        revno_str = _compute_revno_str(branch, rev_id)
         return [(rev_id, revno_str, 0)]
 
 
@@ -557,8 +618,8 @@
         try:
             result = list(result)
         except _StartNotLinearAncestor:
-            raise errors.BzrCommandError('Start revision not found in'
-                ' left-hand history of end revision.')
+            raise errors.BzrCommandError(gettext('Start revision not found in'
+                ' left-hand history of end revision.'))
     return result
 
 
@@ -603,8 +664,8 @@
         except _StartNotLinearAncestor:
             # A merge was never detected so the lower revision limit can't
             # be nested down somewhere
-            raise errors.BzrCommandError('Start revision not found in'
-                ' history of end revision.')
+            raise errors.BzrCommandError(gettext('Start revision not found in'
+                ' history of end revision.'))
 
     # We exit the loop above because we encounter a revision with merges, from
     # this revision, we need to switch to _graph_view_revisions.
@@ -627,11 +688,30 @@
     return len(parents) > 1
 
 
+def _compute_revno_str(branch, rev_id):
+    """Compute the revno string from a rev_id.
+
+    :return: The revno string, or None if the revision is not in the supplied
+        branch.
+    """
+    try:
+        revno = branch.revision_id_to_dotted_revno(rev_id)
+    except errors.NoSuchRevision:
+        # The revision must be outside of this branch
+        return None
+    else:
+        return '.'.join(str(n) for n in revno)
+
+
 def _is_obvious_ancestor(branch, start_rev_id, end_rev_id):
     """Is start_rev_id an obvious ancestor of end_rev_id?"""
     if start_rev_id and end_rev_id:
-        start_dotted = branch.revision_id_to_dotted_revno(start_rev_id)
-        end_dotted = branch.revision_id_to_dotted_revno(end_rev_id)
+        try:
+            start_dotted = branch.revision_id_to_dotted_revno(start_rev_id)
+            end_dotted = branch.revision_id_to_dotted_revno(end_rev_id)
+        except errors.NoSuchRevision:
+            # one or both is not in the branch; not obvious
+            return False
         if len(start_dotted) == 1 and len(end_dotted) == 1:
             # both on mainline
             return start_dotted[0] <= end_dotted[0]
@@ -661,18 +741,20 @@
     """
     br_revno, br_rev_id = branch.last_revision_info()
     repo = branch.repository
+    graph = repo.get_graph()
     if start_rev_id is None and end_rev_id is None:
         cur_revno = br_revno
-        for revision_id in repo.iter_reverse_revision_history(br_rev_id):
+        for revision_id in graph.iter_lefthand_ancestry(br_rev_id,
+            (_mod_revision.NULL_REVISION,)):
             yield revision_id, str(cur_revno), 0
             cur_revno -= 1
     else:
         if end_rev_id is None:
             end_rev_id = br_rev_id
         found_start = start_rev_id is None
-        for revision_id in repo.iter_reverse_revision_history(end_rev_id):
-            revno = branch.revision_id_to_dotted_revno(revision_id)
-            revno_str = '.'.join(str(n) for n in revno)
+        for revision_id in graph.iter_lefthand_ancestry(end_rev_id,
+                (_mod_revision.NULL_REVISION,)):
+            revno_str = _compute_revno_str(branch, revision_id)
             if not found_start and revision_id == start_rev_id:
                 if not exclude_common_ancestry:
                     yield revision_id, revno_str, 0
@@ -729,25 +811,6 @@
             yield rev_id, '.'.join(map(str, revno)), merge_depth
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def calculate_view_revisions(branch, start_revision, end_revision, direction,
-        specific_fileid, generate_merge_revisions):
-    """Calculate the revisions to view.
-
-    :return: An iterator of (revision_id, dotted_revno, merge_depth) tuples OR
-             a list of the same tuples.
-    """
-    start_rev_id, end_rev_id = _get_revision_limits(branch, start_revision,
-        end_revision)
-    view_revisions = list(_calc_view_revisions(branch, start_rev_id, end_rev_id,
-        direction, generate_merge_revisions or specific_fileid))
-    if specific_fileid:
-        view_revisions = _filter_revisions_touching_file_id(branch,
-            specific_fileid, view_revisions,
-            include_merges=generate_merge_revisions)
-    return _rebase_merge_depth(view_revisions)
-
-
 def _rebase_merge_depth(view_revisions):
     """Adjust depths upwards so the top level is 0."""
     # If either the first or last revision have a merge_depth of 0, we're done
@@ -797,31 +860,49 @@
     return log_rev_iterator
 
 
-def _make_search_filter(branch, generate_delta, search, log_rev_iterator):
+def _make_search_filter(branch, generate_delta, match, log_rev_iterator):
     """Create a filtered iterator of log_rev_iterator matching on a regex.
 
     :param branch: The branch being logged.
     :param generate_delta: Whether to generate a delta for each revision.
-    :param search: A user text search string.
+    :param match: A dictionary with properties as keys and lists of strings
+        as values. To match, a revision may match any of the supplied strings
+        within a single property but must match at least one string for each
+        property.
     :param log_rev_iterator: An input iterator containing all revisions that
         could be displayed, in lists.
     :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
         delta).
     """
-    if search is None:
+    if match is None:
         return log_rev_iterator
-    searchRE = re.compile(search, re.IGNORECASE)
-    return _filter_message_re(searchRE, log_rev_iterator)
-
-
-def _filter_message_re(searchRE, log_rev_iterator):
+    searchRE = [(k, [re.compile(x, re.IGNORECASE) for x in v])
+                for (k,v) in match.iteritems()]
+    return _filter_re(searchRE, log_rev_iterator)
+
+
+def _filter_re(searchRE, log_rev_iterator):
     for revs in log_rev_iterator:
-        new_revs = []
-        for (rev_id, revno, merge_depth), rev, delta in revs:
-            if searchRE.search(rev.message):
-                new_revs.append(((rev_id, revno, merge_depth), rev, delta))
-        yield new_revs
-
+        new_revs = [rev for rev in revs if _match_filter(searchRE, rev[1])]
+        if new_revs:
+            yield new_revs
+
+def _match_filter(searchRE, rev):
+    strings = {
+               'message': (rev.message,),
+               'committer': (rev.committer,),
+               'author': (rev.get_apparent_authors()),
+               'bugs': list(rev.iter_bugs())
+               }
+    strings[''] = [item for inner_list in strings.itervalues()
+                   for item in inner_list]
+    for (k,v) in searchRE:
+        if k in strings and not _match_any_filter(strings[k], v):
+            return False
+    return True
+
+def _match_any_filter(strings, res):
+    return any([filter(None, map(re.search, strings)) for re in res])
 
 def _make_delta_filter(branch, generate_delta, search, log_rev_iterator,
     fileids=None, direction='reverse'):
@@ -900,7 +981,7 @@
 
 def _update_fileids(delta, fileids, stop_on):
     """Update the set of file-ids to search based on file lifecycle events.
-    
+
     :param fileids: a set of fileids to update
     :param stop_on: either 'add' or 'remove' - take file-ids out of the
       fileids set once their add or remove entry is detected respectively
@@ -947,7 +1028,6 @@
     :return: An iterator over lists of ((rev_id, revno, merge_depth), rev,
         delta).
     """
-    repository = branch.repository
     num = 9
     for batch in log_rev_iterator:
         batch = iter(batch)
@@ -1002,10 +1082,10 @@
     if branch_revno != 0:
         if (start_rev_id == _mod_revision.NULL_REVISION
             or end_rev_id == _mod_revision.NULL_REVISION):
-            raise errors.BzrCommandError('Logging revision 0 is invalid.')
+            raise errors.BzrCommandError(gettext('Logging revision 0 is invalid.'))
         if start_revno > end_revno:
-            raise errors.BzrCommandError("Start revision must be older than "
-                                         "the end revision.")
+            raise errors.BzrCommandError(gettext("Start revision must be "
+                                         "older than the end revision."))
     return (start_rev_id, end_rev_id)
 
 
@@ -1060,18 +1140,19 @@
 
     if ((start_rev_id == _mod_revision.NULL_REVISION)
         or (end_rev_id == _mod_revision.NULL_REVISION)):
-        raise errors.BzrCommandError('Logging revision 0 is invalid.')
+        raise errors.BzrCommandError(gettext('Logging revision 0 is invalid.'))
     if start_revno > end_revno:
-        raise errors.BzrCommandError("Start revision must be older than "
-                                     "the end revision.")
+        raise errors.BzrCommandError(gettext("Start revision must be older "
+                                     "than the end revision."))
 
     if end_revno < start_revno:
         return None, None, None, None
     cur_revno = branch_revno
     rev_nos = {}
     mainline_revs = []
-    for revision_id in branch.repository.iter_reverse_revision_history(
-                        branch_last_revision):
+    graph = branch.repository.get_graph()
+    for revision_id in graph.iter_lefthand_ancestry(
+            branch_last_revision, (_mod_revision.NULL_REVISION,)):
         if cur_revno < start_revno:
             # We have gone far enough, but we always add 1 more revision
             rev_nos[revision_id] = cur_revno
@@ -1091,50 +1172,6 @@
     return mainline_revs, rev_nos, start_rev_id, end_rev_id
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def _filter_revision_range(view_revisions, start_rev_id, end_rev_id):
-    """Filter view_revisions based on revision ranges.
-
-    :param view_revisions: A list of (revision_id, dotted_revno, merge_depth)
-            tuples to be filtered.
-
-    :param start_rev_id: If not NONE specifies the first revision to be logged.
-            If NONE then all revisions up to the end_rev_id are logged.
-
-    :param end_rev_id: If not NONE specifies the last revision to be logged.
-            If NONE then all revisions up to the end of the log are logged.
-
-    :return: The filtered view_revisions.
-    """
-    if start_rev_id or end_rev_id:
-        revision_ids = [r for r, n, d in view_revisions]
-        if start_rev_id:
-            start_index = revision_ids.index(start_rev_id)
-        else:
-            start_index = 0
-        if start_rev_id == end_rev_id:
-            end_index = start_index
-        else:
-            if end_rev_id:
-                end_index = revision_ids.index(end_rev_id)
-            else:
-                end_index = len(view_revisions) - 1
-        # To include the revisions merged into the last revision,
-        # extend end_rev_id down to, but not including, the next rev
-        # with the same or lesser merge_depth
-        end_merge_depth = view_revisions[end_index][2]
-        try:
-            for index in xrange(end_index+1, len(view_revisions)+1):
-                if view_revisions[index][2] <= end_merge_depth:
-                    end_index = index - 1
-                    break
-        except IndexError:
-            # if the search falls off the end then log to the end as well
-            end_index = len(view_revisions) - 1
-        view_revisions = view_revisions[start_index:end_index+1]
-    return view_revisions
-
-
 def _filter_revisions_touching_file_id(branch, file_id, view_revisions,
     include_merges=True):
     r"""Return the list of revision ids which touch a given file id.
@@ -1143,6 +1180,7 @@
     This includes the revisions which directly change the file id,
     and the revisions which merge these changes. So if the
     revision graph is::
+
         A-.
         |\ \
         B C E
@@ -1175,6 +1213,8 @@
     """
     # Lookup all possible text keys to determine which ones actually modified
     # the file.
+    graph = branch.repository.get_file_graph()
+    get_parent_map = graph.get_parent_map
     text_keys = [(file_id, rev_id) for rev_id, revno, depth in view_revisions]
     next_keys = None
     # Looking up keys in batches of 1000 can cut the time in half, as well as
@@ -1184,7 +1224,6 @@
     #       indexing layer. We might consider passing in hints as to the known
     #       access pattern (sparse/clustered, high success rate/low success
     #       rate). This particular access is clustered with a low success rate.
-    get_parent_map = branch.repository.texts.get_parent_map
     modified_text_revisions = set()
     chunk_size = 1000
     for start in xrange(0, len(text_keys), chunk_size):
@@ -1217,47 +1256,6 @@
     return result
 
 
-@deprecated_function(deprecated_in((2, 2, 0)))
-def get_view_revisions(mainline_revs, rev_nos, branch, direction,
-                       include_merges=True):
-    """Produce an iterator of revisions to show
-    :return: an iterator of (revision_id, revno, merge_depth)
-    (if there is no revno for a revision, None is supplied)
-    """
-    if not include_merges:
-        revision_ids = mainline_revs[1:]
-        if direction == 'reverse':
-            revision_ids.reverse()
-        for revision_id in revision_ids:
-            yield revision_id, str(rev_nos[revision_id]), 0
-        return
-    graph = branch.repository.get_graph()
-    # This asks for all mainline revisions, which means we only have to spider
-    # sideways, rather than depth history. That said, its still size-of-history
-    # and should be addressed.
-    # mainline_revisions always includes an extra revision at the beginning, so
-    # don't request it.
-    parent_map = dict(((key, value) for key, value in
-        graph.iter_ancestry(mainline_revs[1:]) if value is not None))
-    # filter out ghosts; merge_sort errors on ghosts.
-    rev_graph = _mod_repository._strip_NULL_ghosts(parent_map)
-    merge_sorted_revisions = tsort.merge_sort(
-        rev_graph,
-        mainline_revs[-1],
-        mainline_revs,
-        generate_revno=True)
-
-    if direction == 'forward':
-        # forward means oldest first.
-        merge_sorted_revisions = reverse_by_depth(merge_sorted_revisions)
-    elif direction != 'reverse':
-        raise ValueError('invalid direction %r' % direction)
-
-    for (sequence, rev_id, merge_depth, revno, end_of_merge
-         ) in merge_sorted_revisions:
-        yield rev_id, '.'.join(map(str, revno)), merge_depth
-
-
 def reverse_by_depth(merge_sorted_revisions, _depth=0):
     """Reverse revisions by depth.
 
@@ -1298,13 +1296,17 @@
     """
 
     def __init__(self, rev=None, revno=None, merge_depth=0, delta=None,
-                 tags=None, diff=None):
+                 tags=None, diff=None, signature=None):
         self.rev = rev
-        self.revno = str(revno)
+        if revno is None:
+            self.revno = None
+        else:
+            self.revno = str(revno)
         self.merge_depth = merge_depth
         self.delta = delta
         self.tags = tags
         self.diff = diff
+        self.signature = signature
 
 
 class LogFormatter(object):
@@ -1319,27 +1321,31 @@
     to indicate which LogRevision attributes it supports:
 
     - supports_delta must be True if this log formatter supports delta.
-        Otherwise the delta attribute may not be populated.  The 'delta_format'
-        attribute describes whether the 'short_status' format (1) or the long
-        one (2) should be used.
+      Otherwise the delta attribute may not be populated.  The 'delta_format'
+      attribute describes whether the 'short_status' format (1) or the long
+      one (2) should be used.
 
     - supports_merge_revisions must be True if this log formatter supports
-        merge revisions.  If not, then only mainline revisions will be passed
-        to the formatter.
+      merge revisions.  If not, then only mainline revisions will be passed
+      to the formatter.
 
     - preferred_levels is the number of levels this formatter defaults to.
-        The default value is zero meaning display all levels.
-        This value is only relevant if supports_merge_revisions is True.
+      The default value is zero meaning display all levels.
+      This value is only relevant if supports_merge_revisions is True.
 
     - supports_tags must be True if this log formatter supports tags.
-        Otherwise the tags attribute may not be populated.
+      Otherwise the tags attribute may not be populated.
 
     - supports_diff must be True if this log formatter supports diffs.
-        Otherwise the diff attribute may not be populated.
+      Otherwise the diff attribute may not be populated.
+
+    - supports_signatures must be True if this log formatter supports GPG
+      signatures.
 
     Plugins can register functions to show custom revision properties using
     the properties_handler_registry. The registered function
-    must respect the following interface description:
+    must respect the following interface description::
+
         def my_show_properties(properties_dict):
             # code that returns a dict {'name':'value'} of the properties
             # to be shown
@@ -1352,7 +1358,7 @@
         """Create a LogFormatter.
 
         :param to_file: the file to output to
-        :param to_exact_file: if set, gives an output stream to which 
+        :param to_exact_file: if set, gives an output stream to which
              non-Unicode diffs are written.
         :param show_ids: if True, revision-ids are to be displayed
         :param show_timezone: the timezone to use
@@ -1409,7 +1415,7 @@
             if advice_sep:
                 self.to_file.write(advice_sep)
             self.to_file.write(
-                "Use --include-merges or -n0 to see merged revisions.\n")
+                "Use --include-merged or -n0 to see merged revisions.\n")
 
     def get_advice_separator(self):
         """Get the text separating the log from the closing advice."""
@@ -1532,6 +1538,7 @@
     supports_delta = True
     supports_tags = True
     supports_diff = True
+    supports_signatures = True
 
     def __init__(self, *args, **kwargs):
         super(LongLogFormatter, self).__init__(*args, **kwargs)
@@ -1557,8 +1564,9 @@
                 self.merge_marker(revision)))
         if revision.tags:
             lines.append('tags: %s' % (', '.join(revision.tags)))
-        if self.show_ids:
+        if self.show_ids or revision.revno is None:
             lines.append('revision-id: %s' % (revision.rev.revision_id,))
+        if self.show_ids:
             for parent_id in revision.rev.parent_ids:
                 lines.append('parent: %s' % (parent_id,))
         lines.extend(self.custom_properties(revision.rev))
@@ -1575,6 +1583,9 @@
 
         lines.append('timestamp: %s' % (self.date_string(revision.rev),))
 
+        if revision.signature is not None:
+            lines.append('signature: ' + revision.signature)
+
         lines.append('message:')
         if not revision.rev.message:
             lines.append('  (no message)')
@@ -1589,7 +1600,7 @@
         if revision.delta is not None:
             # Use the standard status output to display changes
             from bzrlib.delta import report_delta
-            report_delta(to_file, revision.delta, short_status=False, 
+            report_delta(to_file, revision.delta, short_status=False,
                          show_ids=self.show_ids, indent=indent)
         if revision.diff is not None:
             to_file.write(indent + 'diff:\n')
@@ -1627,7 +1638,7 @@
         indent = '    ' * depth
         revno_width = self.revno_width_by_depth.get(depth)
         if revno_width is None:
-            if revision.revno.find('.') == -1:
+            if revision.revno is None or revision.revno.find('.') == -1:
                 # mainline revno, e.g. 12345
                 revno_width = 5
             else:
@@ -1641,14 +1652,14 @@
         if revision.tags:
             tags = ' {%s}' % (', '.join(revision.tags))
         to_file.write(indent + "%*s %s\t%s%s%s\n" % (revno_width,
-                revision.revno, self.short_author(revision.rev),
+                revision.revno or "", self.short_author(revision.rev),
                 format_date(revision.rev.timestamp,
                             revision.rev.timezone or 0,
                             self.show_timezone, date_fmt="%Y-%m-%d",
                             show_offset=False),
                 tags, self.merge_marker(revision)))
         self.show_properties(revision.rev, indent+offset)
-        if self.show_ids:
+        if self.show_ids or revision.revno is None:
             to_file.write(indent + offset + 'revision-id:%s\n'
                           % (revision.rev.revision_id,))
         if not revision.rev.message:
@@ -1661,8 +1672,8 @@
         if revision.delta is not None:
             # Use the standard status output to display changes
             from bzrlib.delta import report_delta
-            report_delta(to_file, revision.delta, 
-                         short_status=self.delta_format==1, 
+            report_delta(to_file, revision.delta,
+                         short_status=self.delta_format==1,
                          show_ids=self.show_ids, indent=indent + offset)
         if revision.diff is not None:
             self.show_diff(self.to_exact_file, revision.diff, '      ')
@@ -1707,19 +1718,23 @@
 
     def log_string(self, revno, rev, max_chars, tags=None, prefix=''):
         """Format log info into one string. Truncate tail of string
-        :param  revno:      revision number or None.
-                            Revision numbers counts from 1.
-        :param  rev:        revision object
-        :param  max_chars:  maximum length of resulting string
-        :param  tags:       list of tags or None
-        :param  prefix:     string to prefix each line
-        :return:            formatted truncated string
+
+        :param revno:      revision number or None.
+                           Revision numbers counts from 1.
+        :param rev:        revision object
+        :param max_chars:  maximum length of resulting string
+        :param tags:       list of tags or None
+        :param prefix:     string to prefix each line
+        :return:           formatted truncated string
         """
         out = []
         if revno:
             # show revno only when is not None
             out.append("%s:" % revno)
-        out.append(self.truncate(self.short_author(rev), 20))
+        if max_chars is not None:
+            out.append(self.truncate(self.short_author(rev), (max_chars+3)/4))
+        else:
+            out.append(self.short_author(rev))
         out.append(self.date_string(rev))
         if len(rev.parent_ids) > 1:
             out.append('[merge]')
@@ -1784,7 +1799,8 @@
         return self.get(name)(*args, **kwargs)
 
     def get_default(self, branch):
-        return self.get(branch.get_config().log_format())
+        c = branch.get_config_stack()
+        return self.get(c.get('log_format'))
 
 
 log_formatter_registry = LogFormatterRegistry()
@@ -1813,7 +1829,7 @@
     try:
         return log_formatter_registry.make_formatter(name, *args, **kwargs)
     except KeyError:
-        raise errors.BzrCommandError("unknown log formatter: %r" % name)
+        raise errors.BzrCommandError(gettext("unknown log formatter: %r") % name)
 
 
 def author_list_all(rev):
@@ -1844,12 +1860,6 @@
                               'The committer')
 
 
-def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
-    # deprecated; for compatibility
-    lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
-    lf.show(revno, rev, delta)
-
-
 def show_changed_revisions(branch, old_rh, new_rh, to_file=None,
                            log_format='long'):
     """Show the change in revision history comparing the old revision history to the new one.
@@ -1918,8 +1928,9 @@
     old_revisions = set()
     new_history = []
     new_revisions = set()
-    new_iter = repository.iter_reverse_revision_history(new_revision_id)
-    old_iter = repository.iter_reverse_revision_history(old_revision_id)
+    graph = repository.get_graph()
+    new_iter = graph.iter_lefthand_ancestry(new_revision_id)
+    old_iter = graph.iter_lefthand_ancestry(old_revision_id)
     stop_revision = None
     do_old = True
     do_new = True
@@ -2018,7 +2029,8 @@
       branch will be read-locked.
     """
     from builtins import _get_revision_range
-    tree, b, path = bzrdir.BzrDir.open_containing_tree_or_branch(file_list[0])
+    tree, b, path = controldir.ControlDir.open_containing_tree_or_branch(
+        file_list[0])
     add_cleanup(b.lock_read().unlock)
     # XXX: It's damn messy converting a list of paths to relative paths when
     # those paths might be deleted ones, they might be on a case-insensitive
@@ -2113,7 +2125,8 @@
                           len(row) > 1 and row[1] == 'fixed']
 
         if fixed_bug_urls:
-            return {'fixes bug(s)': ' '.join(fixed_bug_urls)}
+            return {ngettext('fixes bug', 'fixes bugs', len(fixed_bug_urls)):\
+                    ' '.join(fixed_bug_urls)}
     return {}
 
 properties_handler_registry.register('bugs_properties_handler',

=== modified file 'bzrlib/lsprof.py'
--- bzrlib/lsprof.py	2010-07-04 06:22:11 +0000
+++ bzrlib/lsprof.py	2011-04-12 14:55:57 +0000
@@ -112,14 +112,21 @@
 
 
 class Stats(object):
-    """XXX docstring"""
+    """Wrapper around the collected data.
+
+    A Stats instance is created when the profiler finishes. Normal
+    usage is to use save() to write out the data to a file, or pprint()
+    to write human-readable information to the command line.
+    """
 
     def __init__(self, data, threads):
         self.data = data
         self.threads = threads
 
     def sort(self, crit="inlinetime"):
-        """XXX docstring"""
+        """Sort the data by the supplied critera.
+
+        :param crit: the data attribute used as the sort key."""
         if crit not in profiler_entry.__dict__:
             raise ValueError, "Can't sort by %s" % crit
         self.data.sort(lambda b, a: cmp(getattr(a, crit),
@@ -130,7 +137,12 @@
                                               getattr(b, crit)))
 
     def pprint(self, top=None, file=None):
-        """XXX docstring"""
+        """Pretty-print the data as plain text for human consumption.
+
+        :param top: only output the top n entries.
+            The default value of None means output all data.
+        :param file: the output file; if None, output will
+            default to stdout."""
         if file is None:
             file = sys.stdout
         d = self.data
@@ -263,12 +275,13 @@
         code = subentry.code
         totaltime = int(subentry.totaltime * 1000)
         #out_file.write('cob=%s\n' % (code.co_filename,))
-        out_file.write('cfn=%s\n' % (label(code, True),))
         if isinstance(code, str):
             out_file.write('cfi=~\n')
+            out_file.write('cfn=%s\n' % (label(code, True),))
             out_file.write('calls=%d 0\n' % (subentry.callcount,))
         else:
             out_file.write('cfi=%s\n' % (code.co_filename,))
+            out_file.write('cfn=%s\n' % (label(code, True),))
             out_file.write('calls=%d %d\n' % (
                 subentry.callcount, code.co_firstlineno))
         out_file.write('%d %d\n' % (lineno, totaltime))

=== modified file 'bzrlib/memorytree.py'
--- bzrlib/memorytree.py	2010-04-30 11:03:59 +0000
+++ bzrlib/memorytree.py	2011-04-19 10:42:59 +0000
@@ -28,12 +28,13 @@
     revision as _mod_revision,
     )
 from bzrlib.decorators import needs_read_lock
+from bzrlib.inventory import Inventory
 from bzrlib.osutils import sha_file
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.transport.memory import MemoryTransport
 
 
-class MemoryTree(mutabletree.MutableTree):
+class MemoryTree(mutabletree.MutableInventoryTree):
     """A MemoryTree is a specialisation of MutableTree.
 
     It maintains nearly no state outside of read_lock and write_lock
@@ -49,6 +50,10 @@
         self._locks = 0
         self._lock_mode = None
 
+    def is_control_filename(self, filename):
+        # Memory tree doesn't have any control filenames
+        return False
+
     @needs_tree_write_lock
     def _add(self, files, ids, kinds):
         """See MutableTree._add."""
@@ -215,11 +220,12 @@
             self._parent_ids = []
         else:
             self._parent_ids = [self._branch_revision_id]
-        self._inventory = self._basis_tree._inventory._get_mutable_inventory()
+        self._inventory = Inventory(None, self._basis_tree.get_revision_id())
         self._file_transport = MemoryTransport()
         # TODO copy the revision trees content, or do it lazy, or something.
-        inventory_entries = self._inventory.iter_entries()
+        inventory_entries = self._basis_tree.iter_entries_by_dir()
         for path, entry in inventory_entries:
+            self._inventory.add(entry.copy())
             if path == '':
                 continue
             if entry.kind == 'directory':

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2010-07-29 03:56:54 +0000
+++ bzrlib/merge.py	2011-09-16 10:08:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 lazy_import(globals(), """
 from bzrlib import (
     branch as _mod_branch,
+    cleanup,
     conflicts as _mod_conflicts,
     debug,
     generate_ids,
@@ -37,7 +38,7 @@
     versionedfile,
     workingtree,
     )
-from bzrlib.cleanup import OperationWithCleanups
+from bzrlib.i18n import gettext
 """)
 from bzrlib import (
     decorators,
@@ -53,7 +54,7 @@
 
 def transform_tree(from_tree, to_tree, interesting_ids=None):
     from_tree.lock_tree_write()
-    operation = OperationWithCleanups(merge_inner)
+    operation = cleanup.OperationWithCleanups(merge_inner)
     operation.add_cleanup(from_tree.unlock)
     operation.run_simple(from_tree.branch, to_tree, from_tree,
         ignore_zero=True, interesting_ids=interesting_ids, this_tree=from_tree)
@@ -62,8 +63,8 @@
 class MergeHooks(hooks.Hooks):
 
     def __init__(self):
-        hooks.Hooks.__init__(self)
-        self.create_hook(hooks.HookPoint('merge_file_content',
+        hooks.Hooks.__init__(self, "bzrlib.merge", "Merger.hooks")
+        self.add_hook('merge_file_content',
             "Called with a bzrlib.merge.Merger object to create a per file "
             "merge object when starting a merge. "
             "Should return either None or a subclass of "
@@ -73,7 +74,7 @@
             "side has deleted the file and the other has changed it). "
             "See the AbstractPerFileMerger API docs for details on how it is "
             "used by merge.",
-            (2, 1), None))
+            (2, 1))
 
 
 class AbstractPerFileMerger(object):
@@ -92,7 +93,7 @@
         """Attempt to merge the contents of a single file.
         
         :param merge_params: A bzrlib.merge.MergeHookParams
-        :return : A tuple of (status, chunks), where status is one of
+        :return: A tuple of (status, chunks), where status is one of
             'not_applicable', 'success', 'conflicted', or 'delete'.  If status
             is 'success' or 'conflicted', then chunks should be an iterable of
             strings for the new file contents.
@@ -458,16 +459,13 @@
     @deprecated_method(deprecated_in((2, 1, 0)))
     def file_revisions(self, file_id):
         self.ensure_revision_trees()
-        def get_id(tree, file_id):
-            revision_id = tree.inventory[file_id].revision
-            return revision_id
         if self.this_rev_id is None:
             if self.this_basis_tree.get_file_sha1(file_id) != \
                 self.this_tree.get_file_sha1(file_id):
                 raise errors.WorkingTreeNotRevision(self.this_tree)
 
         trees = (self.this_basis_tree, self.other_tree)
-        return [get_id(tree, file_id) for tree in trees]
+        return [tree.get_file_revision(file_id) for tree in trees]
 
     @deprecated_method(deprecated_in((2, 1, 0)))
     def check_basis(self, check_clean, require_commits=True):
@@ -501,7 +499,8 @@
     def _add_parent(self):
         new_parents = self.this_tree.get_parent_ids() + [self.other_rev_id]
         new_parent_trees = []
-        operation = OperationWithCleanups(self.this_tree.set_parent_trees)
+        operation = cleanup.OperationWithCleanups(
+            self.this_tree.set_parent_trees)
         for revision_id in new_parents:
             try:
                 tree = self.revision_tree(revision_id)
@@ -582,6 +581,7 @@
             elif len(lcas) == 1:
                 self.base_rev_id = list(lcas)[0]
             else: # len(lcas) > 1
+                self._is_criss_cross = True
                 if len(lcas) > 2:
                     # find_unique_lca can only handle 2 nodes, so we have to
                     # start back at the beginning. It is a shame to traverse
@@ -592,22 +592,30 @@
                 else:
                     self.base_rev_id = self.revision_graph.find_unique_lca(
                                             *lcas)
-                self._is_criss_cross = True
+                sorted_lca_keys = self.revision_graph.find_merge_order(                
+                    revisions[0], lcas)
+                if self.base_rev_id == _mod_revision.NULL_REVISION:
+                    self.base_rev_id = sorted_lca_keys[0]
+                
             if self.base_rev_id == _mod_revision.NULL_REVISION:
                 raise errors.UnrelatedBranches()
             if self._is_criss_cross:
                 trace.warning('Warning: criss-cross merge encountered.  See bzr'
                               ' help criss-cross.')
                 trace.mutter('Criss-cross lcas: %r' % lcas)
-                interesting_revision_ids = [self.base_rev_id]
-                interesting_revision_ids.extend(lcas)
+                if self.base_rev_id in lcas:
+                    trace.mutter('Unable to find unique lca. '
+                                 'Fallback %r as best option.' % self.base_rev_id)
+                interesting_revision_ids = set(lcas)
+                interesting_revision_ids.add(self.base_rev_id)
                 interesting_trees = dict((t.get_revision_id(), t)
                     for t in self.this_branch.repository.revision_trees(
                         interesting_revision_ids))
                 self._cached_trees.update(interesting_trees)
-                self.base_tree = interesting_trees.pop(self.base_rev_id)
-                sorted_lca_keys = self.revision_graph.find_merge_order(
-                    revisions[0], lcas)
+                if self.base_rev_id in lcas:
+                    self.base_tree = interesting_trees[self.base_rev_id]
+                else:
+                    self.base_tree = interesting_trees.pop(self.base_rev_id)
                 self._lca_trees = [interesting_trees[key]
                                    for key in sorted_lca_keys]
             else:
@@ -692,7 +700,7 @@
         return merge
 
     def do_merge(self):
-        operation = OperationWithCleanups(self._do_merge_to)
+        operation = cleanup.OperationWithCleanups(self._do_merge_to)
         self.this_tree.lock_tree_write()
         operation.add_cleanup(self.this_tree.unlock)
         if self.base_tree is not None:
@@ -704,9 +712,9 @@
         merge = operation.run_simple()
         if len(merge.cooked_conflicts) == 0:
             if not self.ignore_zero and not trace.is_quiet():
-                trace.note("All changes applied successfully.")
+                trace.note(gettext("All changes applied successfully."))
         else:
-            trace.note("%d conflicts encountered."
+            trace.note(gettext("%d conflicts encountered.")
                        % len(merge.cooked_conflicts))
 
         return len(merge.cooked_conflicts)
@@ -804,7 +812,7 @@
             warnings.warn("pb argument to Merge3Merger is deprecated")
 
     def do_merge(self):
-        operation = OperationWithCleanups(self._do_merge)
+        operation = cleanup.OperationWithCleanups(self._do_merge)
         self.this_tree.lock_tree_write()
         operation.add_cleanup(self.this_tree.unlock)
         self.base_tree.lock_read()
@@ -825,7 +833,7 @@
             pass
 
     def make_preview_transform(self):
-        operation = OperationWithCleanups(self._make_preview_transform)
+        operation = cleanup.OperationWithCleanups(self._make_preview_transform)
         self.base_tree.lock_read()
         operation.add_cleanup(self.base_tree.unlock)
         self.other_tree.lock_read()
@@ -851,7 +859,7 @@
             self.active_hooks = [hook for hook in hooks if hook is not None]
             for num, (file_id, changed, parents3, names3,
                       executable3) in enumerate(entries):
-                child_pb.update('Preparing file merge', num, len(entries))
+                child_pb.update(gettext('Preparing file merge'), num, len(entries))
                 self._merge_names(file_id, parents3, names3, resolver=resolver)
                 if changed:
                     file_status = self._do_merge_contents(file_id)
@@ -861,7 +869,7 @@
                     executable3, file_status, resolver=resolver)
         finally:
             child_pb.finished()
-        self.fix_root()
+        self.tt.fixup_new_roots()
         self._finish_computing_transform()
 
     def _finish_computing_transform(self):
@@ -881,7 +889,7 @@
                 self.tt.iter_changes(), self.change_reporter)
         self.cook_conflicts(fs_conflicts)
         for conflict in self.cooked_conflicts:
-            trace.warning(conflict)
+            trace.warning(unicode(conflict))
 
     def _entries3(self):
         """Gather data about files modified between three trees.
@@ -894,7 +902,7 @@
         """
         result = []
         iterator = self.other_tree.iter_changes(self.base_tree,
-                include_unchanged=True, specific_files=self.interesting_files,
+                specific_files=self.interesting_files,
                 extra_trees=[self.this_tree])
         this_entries = dict((e.file_id, e) for p, e in
                             self.this_tree.iter_entries_by_dir(
@@ -926,14 +934,16 @@
         it then compares with THIS and BASE.
 
         For the multi-valued entries, the format will be (BASE, [lca1, lca2])
-        :return: [(file_id, changed, parents, names, executable)]
-            file_id     Simple file_id of the entry
-            changed     Boolean, True if the kind or contents changed
-                        else False
-            parents     ((base, [parent_id, in, lcas]), parent_id_other,
-                         parent_id_this)
-            names       ((base, [name, in, lcas]), name_in_other, name_in_this)
-            executable  ((base, [exec, in, lcas]), exec_in_other, exec_in_this)
+
+        :return: [(file_id, changed, parents, names, executable)], where:
+
+            * file_id: Simple file_id of the entry
+            * changed: Boolean, True if the kind or contents changed else False
+            * parents: ((base, [parent_id, in, lcas]), parent_id_other,
+                        parent_id_this)
+            * names:   ((base, [name, in, lcas]), name_in_other, name_in_this)
+            * executable: ((base, [exec, in, lcas]), exec_in_other,
+                        exec_in_this)
         """
         if self.interesting_files is not None:
             lookup_trees = [self.this_tree, self.base_tree]
@@ -981,12 +991,12 @@
                 else:
                     lca_entries.append(lca_ie)
 
-            if file_id in base_inventory:
+            if base_inventory.has_id(file_id):
                 base_ie = base_inventory[file_id]
             else:
                 base_ie = _none_entry
 
-            if file_id in this_inventory:
+            if this_inventory.has_id(file_id):
                 this_ie = this_inventory[file_id]
             else:
                 this_ie = _none_entry
@@ -1084,10 +1094,9 @@
                           ))
         return result
 
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def fix_root(self):
-        try:
-            self.tt.final_kind(self.tt.root)
-        except errors.NoSuchFile:
+        if self.tt.final_kind(self.tt.root) is None:
             self.tt.cancel_deletion(self.tt.root)
         if self.tt.final_file_id(self.tt.root) is None:
             self.tt.version_file(self.tt.tree_file_id(self.tt.root),
@@ -1098,35 +1107,34 @@
         other_root = self.tt.trans_id_file_id(other_root_file_id)
         if other_root == self.tt.root:
             return
-        if self.other_tree.inventory.root.file_id in self.this_tree.inventory:
-            # the other tree's root is a non-root in the current tree (as when
-            # a previously unrelated branch is merged into another)
+        if self.this_tree.inventory.has_id(
+            self.other_tree.inventory.root.file_id):
+            # the other tree's root is a non-root in the current tree (as
+            # when a previously unrelated branch is merged into another)
             return
-        try:
-            self.tt.final_kind(other_root)
+        if self.tt.final_kind(other_root) is not None:
             other_root_is_present = True
-        except errors.NoSuchFile:
+        else:
             # other_root doesn't have a physical representation. We still need
             # to move any references to the actual root of the tree.
             other_root_is_present = False
         # 'other_tree.inventory.root' is not present in this tree. We are
         # calling adjust_path for children which *want* to be present with a
         # correct place to go.
-        for thing, child in self.other_tree.inventory.root.children.iteritems():
+        for _, child in self.other_tree.inventory.root.children.iteritems():
             trans_id = self.tt.trans_id_file_id(child.file_id)
             if not other_root_is_present:
-                # FIXME: Make final_kind returns None instead of raising
-                # NoSuchFile to avoid the ugly construct below -- vila 20100402
-                try:
-                    self.tt.final_kind(trans_id)
+                if self.tt.final_kind(trans_id) is not None:
                     # The item exist in the final tree and has a defined place
                     # to go already.
                     continue
-                except errors.NoSuchFile, e:
-                    pass
             # Move the item into the root
-            self.tt.adjust_path(self.tt.final_name(trans_id),
-                                self.tt.root, trans_id)
+            try:
+                final_name = self.tt.final_name(trans_id)
+            except errors.NoFinalPath:
+                # This file is not present anymore, ignore it.
+                continue
+            self.tt.adjust_path(final_name, self.tt.root, trans_id)
         if other_root_is_present:
             self.tt.cancel_creation(other_root)
             self.tt.cancel_versioning(other_root)
@@ -1160,7 +1168,7 @@
     @staticmethod
     def contents_sha1(tree, file_id):
         """Determine the sha1 of the file contents (used as a key method)."""
-        if file_id not in tree:
+        if not tree.has_id(file_id):
             return None
         return tree.get_file_sha1(file_id)
 
@@ -1311,22 +1319,32 @@
             self._raw_conflicts.append(('path conflict', trans_id, file_id,
                                         this_parent, this_name,
                                         other_parent, other_name))
-        if other_name is None:
+        if not self.other_tree.has_id(file_id):
             # it doesn't matter whether the result was 'other' or
-            # 'conflict'-- if there's no 'other', we leave it alone.
+            # 'conflict'-- if it has no file id, we leave it alone.
             return
         parent_id = parents[self.winner_idx[parent_id_winner]]
-        if parent_id is not None:
+        name = names[self.winner_idx[name_winner]]
+        if parent_id is not None or name is not None:
             # if we get here, name_winner and parent_winner are set to safe
             # values.
-            self.tt.adjust_path(names[self.winner_idx[name_winner]],
-                                self.tt.trans_id_file_id(parent_id),
+            if parent_id is None and name is not None:
+                # if parent_id is None and name is non-None, current file is
+                # the tree root.
+                if names[self.winner_idx[parent_id_winner]] != '':
+                    raise AssertionError(
+                        'File looks like a root, but named %s' %
+                        names[self.winner_idx[parent_id_winner]])
+                parent_trans_id = transform.ROOT_PARENT
+            else:
+                parent_trans_id = self.tt.trans_id_file_id(parent_id)
+            self.tt.adjust_path(name, parent_trans_id,
                                 self.tt.trans_id_file_id(file_id))
 
     def _do_merge_contents(self, file_id):
         """Performs a merge on file_id contents."""
         def contents_pair(tree):
-            if file_id not in tree:
+            if not tree.has_id(file_id):
                 return (None, None)
             kind = tree.kind(file_id)
             if kind == "file":
@@ -1405,10 +1423,7 @@
             self.tt.version_file(file_id, trans_id)
         # The merge has been performed, so the old contents should not be
         # retained.
-        try:
-            self.tt.delete_contents(trans_id)
-        except errors.NoSuchFile:
-            pass
+        self.tt.delete_contents(trans_id)
         return result
 
     def _default_other_winner_merge(self, merge_hook_params):
@@ -1586,10 +1601,7 @@
         if winner == 'this' and file_status != "modified":
             return
         trans_id = self.tt.trans_id_file_id(file_id)
-        try:
-            if self.tt.final_kind(trans_id) != "file":
-                return
-        except errors.NoSuchFile:
+        if self.tt.final_kind(trans_id) != "file":
             return
         if winner == "this":
             executability = this_executable
@@ -1606,8 +1618,8 @@
 
     def cook_conflicts(self, fs_conflicts):
         """Convert all conflicts into a form that doesn't depend on trans_id"""
-        self.cooked_conflicts.extend(transform.cook_conflicts(
-                fs_conflicts, self.tt))
+        content_conflict_file_ids = set()
+        cooked_conflicts = transform.cook_conflicts(fs_conflicts, self.tt)
         fp = transform.FinalPaths(self.tt)
         for conflict in self._raw_conflicts:
             conflict_type = conflict[0]
@@ -1624,8 +1636,14 @@
                 if other_parent is None or other_name is None:
                     other_path = '<deleted>'
                 else:
-                    parent_path =  fp.get_path(
-                        self.tt.trans_id_file_id(other_parent))
+                    if other_parent == self.other_tree.get_root_id():
+                        # The tree transform doesn't know about the other root,
+                        # so we special case here to avoid a NoFinalPath
+                        # exception
+                        parent_path = ''
+                    else:
+                        parent_path =  fp.get_path(
+                            self.tt.trans_id_file_id(other_parent))
                     other_path = osutils.pathjoin(parent_path, other_name)
                 c = _mod_conflicts.Conflict.factory(
                     'path conflict', path=this_path,
@@ -1643,6 +1661,7 @@
                         break
                 c = _mod_conflicts.Conflict.factory(conflict_type,
                                                     path=path, file_id=file_id)
+                content_conflict_file_ids.add(file_id)
             elif conflict_type == 'text conflict':
                 trans_id = conflict[1]
                 path = fp.get_path(trans_id)
@@ -1651,6 +1670,17 @@
                                                     path=path, file_id=file_id)
             else:
                 raise AssertionError('bad conflict type: %r' % (conflict,))
+            cooked_conflicts.append(c)
+
+        self.cooked_conflicts = []
+        # We want to get rid of path conflicts when a corresponding contents
+        # conflict exists. This can occur when one branch deletes a file while
+        # the other renames *and* modifies it. In this case, the content
+        # conflict is enough.
+        for c in cooked_conflicts:
+            if (c.typestring == 'path conflict'
+                and c.file_id in content_conflict_file_ids):
+                continue
             self.cooked_conflicts.append(c)
         self.cooked_conflicts.sort(key=_mod_conflicts.Conflict.sort_key)
 
@@ -1877,7 +1907,7 @@
             entries = self._entries_to_incorporate()
             entries = list(entries)
             for num, (entry, parent_id) in enumerate(entries):
-                child_pb.update('Preparing file merge', num, len(entries))
+                child_pb.update(gettext('Preparing file merge'), num, len(entries))
                 parent_trans_id = self.tt.trans_id_file_id(parent_id)
                 trans_id = transform.new_by_entry(self.tt, entry,
                     parent_trans_id, self.other_tree)
@@ -1901,7 +1931,7 @@
         name_in_target = osutils.basename(self._target_subdir)
         merge_into_root = subdir.copy()
         merge_into_root.name = name_in_target
-        if merge_into_root.file_id in self.this_tree.inventory:
+        if self.this_tree.inventory.has_id(merge_into_root.file_id):
             # Give the root a new file-id.
             # This can happen fairly easily if the directory we are
             # incorporating is the root, and both trees have 'TREE_ROOT' as
@@ -1944,7 +1974,7 @@
     """
     if this_tree is None:
         raise errors.BzrError("bzrlib.merge.merge_inner requires a this_tree "
-                              "parameter as of bzrlib version 0.8.")
+                              "parameter")
     merger = Merger(this_branch, other_tree, base_tree, this_tree=this_tree,
                     pb=pb, change_reporter=change_reporter)
     merger.backup_files = backup_files
@@ -2404,6 +2434,7 @@
 class _PlanLCAMerge(_PlanMergeBase):
     """
     This merge algorithm differs from _PlanMerge in that:
+
     1. comparisons are done against LCAs only
     2. cases where a contested line is new versus one LCA but old versus
        another are marked as conflicts, by emitting the line as conflicted-a
@@ -2450,10 +2481,11 @@
 
         If a line is killed and new, this indicates that the two merge
         revisions contain differing conflict resolutions.
+
         :param revision_id: The id of the revision in which the lines are
             unique
         :param unique_line_numbers: The line numbers of unique lines.
-        :return a tuple of (new_this, killed_other):
+        :return: a tuple of (new_this, killed_other)
         """
         new = set()
         killed = set()

=== modified file 'bzrlib/merge_directive.py'
--- bzrlib/merge_directive.py	2010-03-13 02:49:14 +0000
+++ bzrlib/merge_directive.py	2011-04-07 10:36:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,9 +18,12 @@
 from StringIO import StringIO
 import re
 
+from bzrlib import lazy_import
+lazy_import.lazy_import(globals(), """
 from bzrlib import (
     branch as _mod_branch,
     diff,
+    email_message,
     errors,
     gpg,
     hooks,
@@ -34,7 +37,7 @@
 from bzrlib.bundle import (
     serializer as bundle_serializer,
     )
-from bzrlib.email_message import EmailMessage
+""")
 
 
 class MergeRequestBodyParams(object):
@@ -56,12 +59,12 @@
     """Hooks for MergeDirective classes."""
 
     def __init__(self):
-        hooks.Hooks.__init__(self)
-        self.create_hook(hooks.HookPoint('merge_request_body',
+        hooks.Hooks.__init__(self, "bzrlib.merge_directive", "BaseMergeDirective.hooks")
+        self.add_hook('merge_request_body',
             "Called with a MergeRequestBodyParams when a body is needed for"
             " a merge request.  Callbacks must return a body.  If more"
             " than one callback is registered, the output of one callback is"
-            " provided to the next.", (1, 15, 0), False))
+            " provided to the next.", (1, 15, 0))
 
 
 class BaseMergeDirective(object):
@@ -263,7 +266,8 @@
             body = self.to_signed(branch)
         else:
             body = ''.join(self.to_lines())
-        message = EmailMessage(mail_from, mail_to, subject, body)
+        message = email_message.EmailMessage(mail_from, mail_to, subject,
+            body)
         return message
 
     def install_revisions(self, target_repo):

=== added file 'bzrlib/mergetools.py'
--- bzrlib/mergetools.py	1970-01-01 00:00:00 +0000
+++ bzrlib/mergetools.py	2011-01-21 23:51:15 +0000
@@ -0,0 +1,116 @@
+# Copyright (C) 2010 Canonical Ltd.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Utility functions for managing external merge tools such as kdiff3."""
+
+import os
+import shutil
+import subprocess
+import sys
+import tempfile
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    cmdline,
+    osutils,
+    trace,
+)
+""")
+
+
+known_merge_tools = {
+    'bcompare': 'bcompare {this} {other} {base} {result}',
+    'kdiff3': 'kdiff3 {base} {this} {other} -o {result}',
+    'xdiff': 'xxdiff -m -O -M {result} {this} {base} {other}',
+    'meld': 'meld {base} {this_temp} {other}',
+    'opendiff': 'opendiff {this} {other} -ancestor {base} -merge {result}',
+    'winmergeu': 'winmergeu {result}',
+}
+
+
+def check_availability(command_line):
+    cmd_list = cmdline.split(command_line)
+    exe = cmd_list[0]
+    if sys.platform == 'win32':
+        if os.path.isabs(exe):
+            base, ext = os.path.splitext(exe)
+            path_ext = [unicode(s.lower())
+                        for s in os.getenv('PATHEXT', '').split(os.pathsep)]
+            return os.path.exists(exe) and ext in path_ext
+        else:
+            return osutils.find_executable_on_path(exe) is not None
+    else:
+        return (os.access(exe, os.X_OK)
+                or osutils.find_executable_on_path(exe) is not None)
+
+
+def invoke(command_line, filename, invoker=None):
+    """Invokes the given merge tool command line, substituting the given
+    filename according to the embedded substitution markers. Optionally, it
+    will use the given invoker function instead of the default
+    subprocess_invoker.
+    """
+    if invoker is None:
+        invoker = subprocess_invoker
+    cmd_list = cmdline.split(command_line)
+    args, tmp_file = _subst_filename(cmd_list, filename)
+    def cleanup(retcode):
+        if tmp_file is not None:
+            if retcode == 0: # on success, replace file with temp file
+                shutil.move(tmp_file, filename)
+            else: # otherwise, delete temp file
+                os.remove(tmp_file)
+    return invoker(args[0], args[1:], cleanup)
+
+
+def _subst_filename(args, filename):
+    subst_names = {
+        'base': filename + u'.BASE',
+        'this': filename + u'.THIS',
+        'other': filename + u'.OTHER',
+        'result': filename,
+    }
+    tmp_file = None
+    subst_args = []
+    for arg in args:
+        if '{this_temp}' in arg and not 'this_temp' in subst_names:
+            fh, tmp_file = tempfile.mkstemp(u"_bzr_mergetools_%s.THIS" %
+                                            os.path.basename(filename))
+            trace.mutter('fh=%r, tmp_file=%r', fh, tmp_file)
+            os.close(fh)
+            shutil.copy(filename + u".THIS", tmp_file)
+            subst_names['this_temp'] = tmp_file
+        arg = _format_arg(arg, subst_names)
+        subst_args.append(arg)
+    return subst_args, tmp_file
+
+
+# This would be better implemented using format() from python 2.6
+def _format_arg(arg, subst_names):
+    arg = arg.replace('{base}', subst_names['base'])
+    arg = arg.replace('{this}', subst_names['this'])
+    arg = arg.replace('{other}', subst_names['other'])
+    arg = arg.replace('{result}', subst_names['result'])
+    if subst_names.has_key('this_temp'):
+        arg = arg.replace('{this_temp}', subst_names['this_temp'])
+    return arg
+
+
+def subprocess_invoker(executable, args, cleanup):
+    retcode = subprocess.call([executable] + args)
+    cleanup(retcode)
+    return retcode

=== modified file 'bzrlib/missing.py'
--- bzrlib/missing.py	2010-04-30 11:03:59 +0000
+++ bzrlib/missing.py	2011-09-19 08:42:37 +0000
@@ -18,6 +18,7 @@
 
 from bzrlib import (
     log,
+    symbol_versioning,
     )
 import bzrlib.revision as _mod_revision
 
@@ -42,8 +43,9 @@
 
 
 def find_unmerged(local_branch, remote_branch, restrict='all',
-                  include_merges=False, backward=False,
-                  local_revid_range=None, remote_revid_range=None):
+                  include_merged=None, backward=False,
+                  local_revid_range=None, remote_revid_range=None,
+                  include_merges=symbol_versioning.DEPRECATED_PARAMETER):
     """Find revisions from each side that have not been merged.
 
     :param local_branch: Compare the history of local_branch
@@ -53,7 +55,7 @@
         unique revisions from both sides. If 'local', we will return None
         for the remote revisions, similarly if 'remote' we will return None for
         the local revisions.
-    :param include_merges: Show mainline revisions only if False,
+    :param include_merged: Show mainline revisions only if False,
         all revisions otherwise.
     :param backward: Show oldest versions first when True, newest versions
         first when False.
@@ -61,17 +63,27 @@
         revisions (lower bound, upper bound)
     :param remote_revid_range: Revision-id range for filtering remote_branch
         revisions (lower bound, upper bound)
+    :param include_merges: Deprecated historical alias for include_merged
 
     :return: A list of [(revno, revision_id)] for the mainline revisions on
         each side.
     """
+    if symbol_versioning.deprecated_passed(include_merges):
+        symbol_versioning.warn(
+            'include_merges was deprecated in 2.5.'
+            ' Use include_merged instead.',
+            DeprecationWarning, stacklevel=2)
+        if include_merged is None:
+            include_merged = include_merges
+    if include_merged is None:
+        include_merged = False
     local_branch.lock_read()
     try:
         remote_branch.lock_read()
         try:
             return _find_unmerged(
                 local_branch, remote_branch, restrict=restrict,
-                include_merges=include_merges, backward=backward,
+                include_merged=include_merged, backward=backward,
                 local_revid_range=local_revid_range,
                 remote_revid_range=remote_revid_range)
         finally:
@@ -160,7 +172,7 @@
 
 
 def _find_unmerged(local_branch, remote_branch, restrict,
-                   include_merges, backward,
+                   include_merged, backward,
                    local_revid_range=None, remote_revid_range=None):
     """See find_unmerged.
 
@@ -186,7 +198,7 @@
                              ' "remote": %r' % (restrict,))
         local_extra, remote_extra = graph.find_difference(local_revision_id,
                                                           remote_revision_id)
-    if include_merges:
+    if include_merged:
         locals = _enumerate_with_merges(local_branch, local_extra,
                                         graph, local_revno,
                                         local_revision_id, backward)

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2010-04-30 11:35:43 +0000
+++ bzrlib/msgeditor.py	2011-07-23 16:33:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 import sys
 
 from bzrlib import (
+    cmdline,
     config,
     osutils,
     trace,
@@ -30,7 +31,7 @@
     ui,
     )
 from bzrlib.errors import BzrError, BadCommitMessageEncoding
-from bzrlib.hooks import HookPoint, Hooks
+from bzrlib.hooks import Hooks
 
 
 def _get_editor():
@@ -40,7 +41,7 @@
     except KeyError:
         pass
 
-    e = config.GlobalConfig().get_editor()
+    e = config.GlobalStack().get('editor')
     if e is not None:
         yield e, config.config_filename()
 
@@ -59,7 +60,7 @@
 def _run_editor(filename):
     """Try to execute an editor to edit the commit message."""
     for candidate, candidate_source in _get_editor():
-        edargs = candidate.split(' ')
+        edargs = cmdline.split(candidate)
         try:
             ## mutter("trying editor: %r", (edargs +[filename]))
             x = call(edargs + [filename])
@@ -143,14 +144,16 @@
         if not msgfilename:
             return None
         basename = osutils.basename(msgfilename)
-        msg_transport = transport.get_transport(osutils.dirname(msgfilename))
+        msg_transport = transport.get_transport_from_path(osutils.dirname(msgfilename))
         reference_content = msg_transport.get_bytes(basename)
         if not _run_editor(msgfilename):
             return None
         edited_content = msg_transport.get_bytes(basename)
         if edited_content == reference_content:
-            if not ui.ui_factory.get_boolean(
-                "Commit message was not edited, use anyway"):
+            if not ui.ui_factory.confirm_action(
+                u"Commit message was not edited, use anyway",
+                "bzrlib.msgeditor.unchanged",
+                {}):
                 # Returning "" makes cmd_commit raise 'empty commit message
                 # specified' which is a reasonable error, given the user has
                 # rejected using the unedited template.
@@ -206,28 +209,25 @@
 
 def _create_temp_file_with_commit_template(infotext,
                                            ignoreline=DEFAULT_IGNORE_LINE,
-                                           start_message=None):
+                                           start_message=None,
+                                           tmpdir=None):
     """Create temp file and write commit template in it.
 
-    :param infotext:    Text to be displayed at bottom of message
-                        for the user's reference;
-                        currently similar to 'bzr status'.
-                        The text is already encoded.
+    :param infotext: Text to be displayed at bottom of message for the
+        user's reference; currently similar to 'bzr status'.  The text is
+        already encoded.
 
     :param ignoreline:  The separator to use above the infotext.
 
-    :param start_message:   The text to place above the separator, if any.
-                            This will not be removed from the message
-                            after the user has edited it.
-                            The string is already encoded
+    :param start_message: The text to place above the separator, if any.
+        This will not be removed from the message after the user has edited
+        it.  The string is already encoded
 
     :return:    2-tuple (temp file name, hasinfo)
     """
     import tempfile
     tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.',
-                                               dir='.',
-                                               text=True)
-    msgfilename = osutils.basename(msgfilename)
+                                               dir=tmpdir, text=True)
     msgfile = os.fdopen(tmp_fileno, 'w')
     try:
         if start_message is not None:
@@ -302,21 +302,39 @@
 
         These are all empty initially.
         """
-        Hooks.__init__(self)
-        self.create_hook(HookPoint('commit_message_template',
+        Hooks.__init__(self, "bzrlib.msgeditor", "hooks")
+        self.add_hook('set_commit_message',
+            "Set a fixed commit message. "
+            "set_commit_message is called with the "
+            "bzrlib.commit.Commit object (so you can also change e.g. revision "
+            "properties by editing commit.builder._revprops) and the message "
+            "so far. set_commit_message must return the message to use or None"
+            " if it should use the message editor as normal.", (2, 4))
+        self.add_hook('commit_message_template',
             "Called when a commit message is being generated. "
             "commit_message_template is called with the bzrlib.commit.Commit "
             "object and the message that is known so far. "
             "commit_message_template must return a new message to use (which "
-            "could be the same as it was given. When there are multiple "
+            "could be the same as it was given). When there are multiple "
             "hooks registered for commit_message_template, they are chained "
             "with the result from the first passed into the second, and so "
-            "on.", (1, 10), None))
+            "on.", (1, 10))
 
 
 hooks = MessageEditorHooks()
 
 
+def set_commit_message(commit, start_message=None):
+    """Sets the commit message.
+    :param commit: Commit object for the active commit.
+    :return: The commit message or None to continue using the message editor
+    """
+    start_message = None
+    for hook in hooks['set_commit_message']:
+        start_message = hook(commit, start_message)
+    return start_message
+
+
 def generate_commit_message_template(commit, start_message=None):
     """Generate a commit message template.
 

=== modified file 'bzrlib/multiparent.py'
--- bzrlib/multiparent.py	2009-06-10 03:56:49 +0000
+++ bzrlib/multiparent.py	2011-09-16 10:08:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,19 +18,18 @@
 
 lazy_import(globals(), """
 import errno
+import gzip
 import itertools
 import os
 from StringIO import StringIO
 
 from bzrlib import (
+    bencode,
     errors,
     patiencediff,
-    trace,
     ui,
     )
-from bzrlib import bencode
 """)
-from bzrlib.tuned_gzip import GzipFile
 
 
 def topo_iter_keys(vf, keys=None):
@@ -76,6 +75,8 @@
 class MultiParent(object):
     """A multi-parent diff"""
 
+    __slots__ = ['hunks']
+
     def __init__(self, hunks=None):
         if hunks is not None:
             self.hunks = hunks
@@ -258,6 +259,8 @@
 class NewText(object):
     """The contents of text that is introduced by this text"""
 
+    __slots__ = ['lines']
+
     def __init__(self, lines):
         self.lines = lines
 
@@ -279,24 +282,30 @@
 class ParentText(object):
     """A reference to text present in a parent text"""
 
+    __slots__ = ['parent', 'parent_pos', 'child_pos', 'num_lines']
+
     def __init__(self, parent, parent_pos, child_pos, num_lines):
         self.parent = parent
         self.parent_pos = parent_pos
         self.child_pos = child_pos
         self.num_lines = num_lines
 
+    def _as_dict(self):
+        return dict(parent=self.parent, parent_pos=self.parent_pos,
+                    child_pos=self.child_pos, num_lines=self.num_lines)
+
     def __repr__(self):
-        return 'ParentText(%(parent)r, %(parent_pos)r, %(child_pos)r,'\
-            ' %(num_lines)r)' % self.__dict__
+        return ('ParentText(%(parent)r, %(parent_pos)r, %(child_pos)r,'
+                ' %(num_lines)r)' % self._as_dict())
 
     def __eq__(self, other):
         if self.__class__ is not other.__class__:
             return False
-        return (self.__dict__ == other.__dict__)
+        return self._as_dict() == other._as_dict()
 
     def to_patch(self):
-        yield 'c %(parent)d %(parent_pos)d %(child_pos)d %(num_lines)d\n'\
-            % self.__dict__
+        yield ('c %(parent)d %(parent_pos)d %(child_pos)d %(num_lines)d\n'
+               % self._as_dict())
 
 
 class BaseVersionedFile(object):
@@ -412,7 +421,7 @@
                             if not (lines == self.get_line_list([revision])[0]):
                                 raise AssertionError()
                             self.clear_cache()
-                    pb.update('Importing revisions',
+                    pb.update(gettext('Importing revisions'),
                               (total - len(revisions)) + len(added), total)
                 revisions = [r for r in revisions if r not in added]
         finally:
@@ -551,10 +560,11 @@
             sio = StringIO(infile.read(count))
         finally:
             infile.close()
-        zip_file = GzipFile(None, mode='rb', fileobj=sio)
+        zip_file = gzip.GzipFile(None, mode='rb', fileobj=sio)
         try:
             file_version_id = zip_file.readline()
-            return MultiParent.from_patch(zip_file.read())
+            content = zip_file.read()
+            return MultiParent.from_patch(content)
         finally:
             zip_file.close()
 
@@ -566,7 +576,7 @@
                                     # before any write returns 0
             start = outfile.tell()
             try:
-                zipfile = GzipFile(None, mode='ab', fileobj=outfile)
+                zipfile = gzip.GzipFile(None, mode='ab', fileobj=outfile)
                 zipfile.writelines(itertools.chain(
                     ['version %s\n' % version_id], diff.to_patch()))
             finally:
@@ -663,7 +673,7 @@
 
 def gzip_string(lines):
     sio = StringIO()
-    data_file = GzipFile(None, mode='wb', fileobj=sio)
+    data_file = gzip.GzipFile(None, mode='wb', fileobj=sio)
     data_file.writelines(lines)
     data_file.close()
     return sio.getvalue()

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2010-07-29 11:17:57 +0000
+++ bzrlib/mutabletree.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,18 +22,20 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import operator
 import os
 import re
 
 from bzrlib import (
     add,
-    bzrdir,
+    controldir,
     errors,
     hooks,
+    inventory as _mod_inventory,
     osutils,
     revisiontree,
-    symbol_versioning,
     trace,
+    transport as _mod_transport,
     tree,
     )
 """)
@@ -69,7 +71,7 @@
     conformance tests for - rather we are testing MemoryTree specifically, and
     interface testing implementations of WorkingTree.
 
-    A mutable tree always has an associated Branch and BzrDir object - the
+    A mutable tree always has an associated Branch and ControlDir object - the
     branch and bzrdir attributes.
     """
     def __init__(self, *args, **kw):
@@ -79,6 +81,18 @@
         # used on media which doesn't differentiate the case of names.
         self.case_sensitive = True
 
+    def is_control_filename(self, filename):
+        """True if filename is the name of a control file in this tree.
+
+        :param filename: A filename within the tree. This is a relative path
+            from the root of this tree.
+
+        This is true IF and ONLY IF the filename is part of the meta data
+        that bzr controls in this tree. I.E. a random .bzr directory placed
+        on disk will not be a control file for this tree.
+        """
+        raise NotImplementedError(self.is_control_filename)
+
     @needs_tree_write_lock
     def add(self, files, ids=None, kinds=None):
         """Add paths to the set of versioned paths.
@@ -148,7 +162,7 @@
         if sub_tree_id == self.get_root_id():
             raise errors.BadReferenceTarget(self, sub_tree,
                                      'Trees have the same root id.')
-        if sub_tree_id in self.inventory:
+        if self.has_id(sub_tree_id):
             raise errors.BadReferenceTarget(self, sub_tree,
                                             'Root id already present in tree')
         self._add([sub_tree_path], [sub_tree_id], ['tree-reference'])
@@ -163,7 +177,6 @@
         """
         raise NotImplementedError(self._add)
 
-    @needs_tree_write_lock
     def apply_inventory_delta(self, changes):
         """Apply changes to the inventory as an atomic operation.
 
@@ -172,10 +185,7 @@
         :return None:
         :seealso Inventory.apply_delta: For details on the changes parameter.
         """
-        self.flush()
-        inv = self.inventory
-        inv.apply_delta(changes)
-        self._write_inventory(inv)
+        raise NotImplementedError(self.apply_inventory_delta)
 
     @needs_write_lock
     def commit(self, message=None, revprops=None, *args,
@@ -250,7 +260,7 @@
         :param more_warning: Details about what is happening.
         """
         if strict is None:
-            strict = self.branch.get_config().get_user_option_as_bool(opt_name)
+            strict = self.branch.get_config_stack().get(opt_name)
         if strict is not False:
             err_class = None
             if (self.has_changes()):
@@ -333,12 +343,6 @@
         :return: None
         """
 
-    def _fix_case_of_inventory_path(self, path):
-        """If our tree isn't case sensitive, return the canonical path"""
-        if not self.case_sensitive:
-            path = self.get_canonical_inventory_path(path)
-        return path
-
     @needs_write_lock
     def put_file_bytes_non_atomic(self, file_id, bytes):
         """Update the content of a file in the tree.
@@ -368,7 +372,6 @@
         """
         raise NotImplementedError(self.set_parent_trees)
 
-    @needs_tree_write_lock
     def smart_add(self, file_list, recurse=True, action=None, save=True):
         """Version file_list, optionally recursing into directories.
 
@@ -389,189 +392,68 @@
             of added files, and ignored_files is a dict mapping files that were
             ignored to the rule that caused them to be ignored.
         """
-        # not in an inner loop; and we want to remove direct use of this,
-        # so here as a reminder for now. RBC 20070703
-        from bzrlib.inventory import InventoryEntry
-        if action is None:
-            action = add.AddAction()
-
-        if not file_list:
-            # no paths supplied: add the entire tree.
-            # FIXME: this assumes we are running in a working tree subdir :-/
-            # -- vila 20100208
-            file_list = [u'.']
-        # mutter("smart add of %r")
+        raise NotImplementedError(self.smart_add)
+
+
+class MutableInventoryTree(MutableTree, tree.InventoryTree):
+
+    @needs_tree_write_lock
+    def apply_inventory_delta(self, changes):
+        """Apply changes to the inventory as an atomic operation.
+
+        :param changes: An inventory delta to apply to the working tree's
+            inventory.
+        :return None:
+        :seealso Inventory.apply_delta: For details on the changes parameter.
+        """
+        self.flush()
         inv = self.inventory
-        added = []
-        ignored = {}
-        dirs_to_add = []
-        user_dirs = set()
-        conflicts_related = set()
+        inv.apply_delta(changes)
+        self._write_inventory(inv)
+
+    def _fix_case_of_inventory_path(self, path):
+        """If our tree isn't case sensitive, return the canonical path"""
+        if not self.case_sensitive:
+            path = self.get_canonical_inventory_path(path)
+        return path
+
+    @needs_tree_write_lock
+    def smart_add(self, file_list, recurse=True, action=None, save=True):
+        """Version file_list, optionally recursing into directories.
+
+        This is designed more towards DWIM for humans than API clarity.
+        For the specific behaviour see the help for cmd_add().
+
+        :param file_list: List of zero or more paths.  *NB: these are 
+            interpreted relative to the process cwd, not relative to the 
+            tree.*  (Add and most other tree methods use tree-relative
+            paths.)
+        :param action: A reporter to be called with the inventory, parent_ie,
+            path and kind of the path being added. It may return a file_id if
+            a specific one should be used.
+        :param save: Save the inventory after completing the adds. If False
+            this provides dry-run functionality by doing the add and not saving
+            the inventory.
+        :return: A tuple - files_added, ignored_files. files_added is the count
+            of added files, and ignored_files is a dict mapping files that were
+            ignored to the rule that caused them to be ignored.
+        """
         # Not all mutable trees can have conflicts
         if getattr(self, 'conflicts', None) is not None:
             # Collect all related files without checking whether they exist or
             # are versioned. It's cheaper to do that once for all conflicts
             # than trying to find the relevant conflict for each added file.
+            conflicts_related = set()
             for c in self.conflicts():
                 conflicts_related.update(c.associated_filenames())
-
-        # validate user file paths and convert all paths to tree
-        # relative : it's cheaper to make a tree relative path an abspath
-        # than to convert an abspath to tree relative, and it's cheaper to
-        # perform the canonicalization in bulk.
-        for filepath in osutils.canonical_relpaths(self.basedir, file_list):
-            rf = _FastPath(filepath)
-            # validate user parameters. Our recursive code avoids adding new
-            # files that need such validation
-            if self.is_control_filename(rf.raw_path):
-                raise errors.ForbiddenControlFileError(filename=rf.raw_path)
-
-            abspath = self.abspath(rf.raw_path)
-            kind = osutils.file_kind(abspath)
-            if kind == 'directory':
-                # schedule the dir for scanning
-                user_dirs.add(rf)
-            else:
-                if not InventoryEntry.versionable_kind(kind):
-                    raise errors.BadFileKindError(filename=abspath, kind=kind)
-            # ensure the named path is added, so that ignore rules in the later
-            # directory walk dont skip it.
-            # we dont have a parent ie known yet.: use the relatively slower
-            # inventory probing method
-            versioned = inv.has_filename(rf.raw_path)
-            if versioned:
-                continue
-            added.extend(_add_one_and_parent(self, inv, None, rf, kind, action))
-
-        if not recurse:
-            # no need to walk any directories at all.
-            if len(added) > 0 and save:
-                self._write_inventory(inv)
-            return added, ignored
-
-        # only walk the minimal parents needed: we have user_dirs to override
-        # ignores.
-        prev_dir = None
-
-        is_inside = osutils.is_inside_or_parent_of_any
-        for path in sorted(user_dirs):
-            if (prev_dir is None or not is_inside([prev_dir], path.raw_path)):
-                dirs_to_add.append((path, None))
-            prev_dir = path.raw_path
-
-        illegalpath_re = re.compile(r'[\r\n]')
-        # dirs_to_add is initialised to a list of directories, but as we scan
-        # directories we append files to it.
-        # XXX: We should determine kind of files when we scan them rather than
-        # adding to this list. RBC 20070703
-        for directory, parent_ie in dirs_to_add:
-            # directory is tree-relative
-            abspath = self.abspath(directory.raw_path)
-
-            # get the contents of this directory.
-
-            # find the kind of the path being added.
-            kind = osutils.file_kind(abspath)
-
-            if not InventoryEntry.versionable_kind(kind):
-                trace.warning("skipping %s (can't add file of kind '%s')",
-                              abspath, kind)
-                continue
-            if illegalpath_re.search(directory.raw_path):
-                trace.warning("skipping %r (contains \\n or \\r)" % abspath)
-                continue
-            if directory.raw_path in conflicts_related:
-                # If the file looks like one generated for a conflict, don't
-                # add it.
-                trace.warning(
-                    'skipping %s (generated to help resolve conflicts)',
-                    abspath)
-                continue
-
-            if parent_ie is not None:
-                versioned = directory.base_path in parent_ie.children
-            else:
-                # without the parent ie, use the relatively slower inventory
-                # probing method
-                versioned = inv.has_filename(
-                        self._fix_case_of_inventory_path(directory.raw_path))
-
-            if kind == 'directory':
-                try:
-                    sub_branch = bzrdir.BzrDir.open(abspath)
-                    sub_tree = True
-                except errors.NotBranchError:
-                    sub_tree = False
-                except errors.UnsupportedFormatError:
-                    sub_tree = True
-            else:
-                sub_tree = False
-
-            if directory.raw_path == '':
-                # mutter("tree root doesn't need to be added")
-                sub_tree = False
-            elif versioned:
-                pass
-                # mutter("%r is already versioned", abspath)
-            elif sub_tree:
-                # XXX: This is wrong; people *might* reasonably be trying to
-                # add subtrees as subtrees.  This should probably only be done
-                # in formats which can represent subtrees, and even then
-                # perhaps only when the user asked to add subtrees.  At the
-                # moment you can add them specially through 'join --reference',
-                # which is perhaps reasonable: adding a new reference is a
-                # special operation and can have a special behaviour.  mbp
-                # 20070306
-                trace.mutter("%r is a nested bzr tree", abspath)
-            else:
-                _add_one(self, inv, parent_ie, directory, kind, action)
-                added.append(directory.raw_path)
-
-            if kind == 'directory' and not sub_tree:
-                if parent_ie is not None:
-                    # must be present:
-                    this_ie = parent_ie.children[directory.base_path]
-                else:
-                    # without the parent ie, use the relatively slower inventory
-                    # probing method
-                    this_id = inv.path2id(
-                        self._fix_case_of_inventory_path(directory.raw_path))
-                    if this_id is None:
-                        this_ie = None
-                    else:
-                        this_ie = inv[this_id]
-
-                for subf in sorted(os.listdir(abspath)):
-                    # here we could use TreeDirectory rather than
-                    # string concatenation.
-                    subp = osutils.pathjoin(directory.raw_path, subf)
-                    # TODO: is_control_filename is very slow. Make it faster.
-                    # TreeDirectory.is_control_filename could also make this
-                    # faster - its impossible for a non root dir to have a
-                    # control file.
-                    if self.is_control_filename(subp):
-                        trace.mutter("skip control directory %r", subp)
-                    elif subf in this_ie.children:
-                        # recurse into this already versioned subdir.
-                        dirs_to_add.append((_FastPath(subp, subf), this_ie))
-                    else:
-                        # user selection overrides ignoes
-                        # ignore while selecting files - if we globbed in the
-                        # outer loop we would ignore user files.
-                        ignore_glob = self.is_ignored(subp)
-                        if ignore_glob is not None:
-                            # mutter("skip ignored sub-file %r", subp)
-                            ignored.setdefault(ignore_glob, []).append(subp)
-                        else:
-                            #mutter("queue to add sub-file %r", subp)
-                            dirs_to_add.append((_FastPath(subp, subf), this_ie))
-
-        if len(added) > 0:
-            if save:
-                self._write_inventory(inv)
-            else:
-                self.read_working_inventory()
-        return added, ignored
+        else:
+            conflicts_related = None
+        adder = _SmartAddHelper(self, action, conflicts_related)
+        adder.add(file_list, recurse=recurse)
+        if save:
+            invdelta = adder.get_inventory_delta()
+            self.apply_inventory_delta(invdelta)
+        return adder.added, adder.ignored
 
     def update_basis_by_delta(self, new_revid, delta):
         """Update the parents of this tree after a commit.
@@ -609,10 +491,10 @@
         # TODO: Consider re-evaluating the need for this with CHKInventory
         # we don't strictly need to mutate an inventory for this
         # it only makes sense when apply_delta is cheaper than get_inventory()
-        inventory = basis.inventory._get_mutable_inventory()
+        inventory = _mod_inventory.mutable_inventory_from_tree(basis)
         basis.unlock()
         inventory.apply_delta(delta)
-        rev_tree = revisiontree.RevisionTree(self.branch.repository,
+        rev_tree = revisiontree.InventoryRevisionTree(self.branch.repository,
                                              inventory, new_revid)
         self.set_parent_trees([(new_revid, rev_tree)])
 
@@ -626,17 +508,17 @@
         """Create the default hooks.
 
         """
-        hooks.Hooks.__init__(self)
-        self.create_hook(hooks.HookPoint('start_commit',
+        hooks.Hooks.__init__(self, "bzrlib.mutabletree", "MutableTree.hooks")
+        self.add_hook('start_commit',
             "Called before a commit is performed on a tree. The start commit "
             "hook is able to change the tree before the commit takes place. "
             "start_commit is called with the bzrlib.mutabletree.MutableTree "
-            "that the commit is being performed on.", (1, 4), None))
-        self.create_hook(hooks.HookPoint('post_commit',
+            "that the commit is being performed on.", (1, 4))
+        self.add_hook('post_commit',
             "Called after a commit is performed on a tree. The hook is "
             "called with a bzrlib.mutabletree.PostCommitHookParams object. "
             "The mutable tree the commit was performed on is available via "
-            "the mutable_tree attribute of that object.", (2, 0), None))
+            "the mutable_tree attribute of that object.", (2, 0))
 
 
 # install the default hooks into the MutableTree class.
@@ -656,70 +538,250 @@
         self.mutable_tree = mutable_tree
 
 
-class _FastPath(object):
-    """A path object with fast accessors for things like basename."""
-
-    __slots__ = ['raw_path', 'base_path']
-
-    def __init__(self, path, base_path=None):
-        """Construct a FastPath from path."""
-        if base_path is None:
-            self.base_path = osutils.basename(path)
-        else:
-            self.base_path = base_path
-        self.raw_path = path
-
-    def __cmp__(self, other):
-        return cmp(self.raw_path, other.raw_path)
-
-    def __hash__(self):
-        return hash(self.raw_path)
-
-
-def _add_one_and_parent(tree, inv, parent_ie, path, kind, action):
-    """Add a new entry to the inventory and automatically add unversioned parents.
-
-    :param inv: Inventory which will receive the new entry.
-    :param parent_ie: Parent inventory entry if known, or None.  If
-        None, the parent is looked up by name and used if present, otherwise it
-        is recursively added.
-    :param kind: Kind of new entry (file, directory, etc)
-    :param action: callback(inv, parent_ie, path, kind); return ignored.
-    :return: A list of paths which have been added.
-    """
-    # Nothing to do if path is already versioned.
-    # This is safe from infinite recursion because the tree root is
-    # always versioned.
-    if parent_ie is not None:
-        # we have a parent ie already
-        added = []
-    else:
-        # slower but does not need parent_ie
-        if inv.has_filename(tree._fix_case_of_inventory_path(path.raw_path)):
-            return []
-        # its really not there : add the parent
-        # note that the dirname use leads to some extra str copying etc but as
-        # there are a limited number of dirs we can be nested under, it should
-        # generally find it very fast and not recurse after that.
-        added = _add_one_and_parent(tree, inv, None,
-            _FastPath(osutils.dirname(path.raw_path)), 'directory', action)
-        parent_id = inv.path2id(osutils.dirname(path.raw_path))
-        parent_ie = inv[parent_id]
-    _add_one(tree, inv, parent_ie, path, kind, action)
-    return added + [path.raw_path]
-
-
-def _add_one(tree, inv, parent_ie, path, kind, file_id_callback):
-    """Add a new entry to the inventory.
-
-    :param inv: Inventory which will receive the new entry.
-    :param parent_ie: Parent inventory entry.
-    :param kind: Kind of new entry (file, directory, etc)
-    :param file_id_callback: callback(inv, parent_ie, path, kind); return a
-        file_id or None to generate a new file id
-    :returns: None
-    """
-    file_id = file_id_callback(inv, parent_ie, path, kind)
-    entry = inv.make_entry(kind, path.base_path, parent_ie.file_id,
-        file_id=file_id)
-    inv.add(entry)
+class _SmartAddHelper(object):
+    """Helper for MutableTree.smart_add."""
+
+    def get_inventory_delta(self):
+        return self._invdelta.values()
+
+    def _get_ie(self, inv_path):
+        """Retrieve the most up to date inventory entry for a path.
+
+        :param inv_path: Normalized inventory path
+        :return: Inventory entry (with possibly invalid .children for
+            directories)
+        """
+        entry = self._invdelta.get(inv_path)
+        if entry is not None:
+            return entry[3]
+        # Find a 'best fit' match if the filesystem is case-insensitive
+        inv_path = self.tree._fix_case_of_inventory_path(inv_path)
+        file_id = self.tree.path2id(inv_path)
+        if file_id is not None:
+            return self.tree.iter_entries_by_dir([file_id]).next()[1]
+        return None
+
+    def _convert_to_directory(self, this_ie, inv_path):
+        """Convert an entry to a directory.
+
+        :param this_ie: Inventory entry
+        :param inv_path: Normalized path for the inventory entry
+        :return: The new inventory entry
+        """
+        # Same as in _add_one below, if the inventory doesn't
+        # think this is a directory, update the inventory
+        this_ie = _mod_inventory.InventoryDirectory(
+            this_ie.file_id, this_ie.name, this_ie.parent_id)
+        self._invdelta[inv_path] = (inv_path, inv_path, this_ie.file_id,
+            this_ie)
+        return this_ie
+
+    def _add_one_and_parent(self, parent_ie, path, kind, inv_path):
+        """Add a new entry to the inventory and automatically add unversioned parents.
+
+        :param parent_ie: Parent inventory entry if known, or None.  If
+            None, the parent is looked up by name and used if present, otherwise it
+            is recursively added.
+        :param path: 
+        :param kind: Kind of new entry (file, directory, etc)
+        :param inv_path:
+        :return: Inventory entry for path and a list of paths which have been added.
+        """
+        # Nothing to do if path is already versioned.
+        # This is safe from infinite recursion because the tree root is
+        # always versioned.
+        inv_dirname = osutils.dirname(inv_path)
+        dirname, basename = osutils.split(path)
+        if parent_ie is None:
+            # slower but does not need parent_ie
+            this_ie = self._get_ie(inv_path)
+            if this_ie is not None:
+                return this_ie
+            # its really not there : add the parent
+            # note that the dirname use leads to some extra str copying etc but as
+            # there are a limited number of dirs we can be nested under, it should
+            # generally find it very fast and not recurse after that.
+            parent_ie = self._add_one_and_parent(None,
+                dirname, 'directory', 
+                inv_dirname)
+        # if the parent exists, but isn't a directory, we have to do the
+        # kind change now -- really the inventory shouldn't pretend to know
+        # the kind of wt files, but it does.
+        if parent_ie.kind != 'directory':
+            # nb: this relies on someone else checking that the path we're using
+            # doesn't contain symlinks.
+            parent_ie = self._convert_to_directory(parent_ie, inv_dirname)
+        file_id = self.action(self.tree.inventory, parent_ie, path, kind)
+        entry = _mod_inventory.make_entry(kind, basename, parent_ie.file_id,
+            file_id=file_id)
+        self._invdelta[inv_path] = (None, inv_path, entry.file_id, entry)
+        self.added.append(inv_path)
+        return entry
+
+    def _gather_dirs_to_add(self, user_dirs):
+        # only walk the minimal parents needed: we have user_dirs to override
+        # ignores.
+        prev_dir = None
+
+        is_inside = osutils.is_inside_or_parent_of_any
+        for path, (inv_path, this_ie) in sorted(
+                user_dirs.iteritems(), key=operator.itemgetter(0)):
+            if (prev_dir is None or not is_inside([prev_dir], path)):
+                yield (path, inv_path, this_ie, None)
+            prev_dir = path
+        
+    def __init__(self, tree, action, conflicts_related=None):
+        self.tree = tree
+        if action is None:
+            self.action = add.AddAction()
+        else:
+            self.action = action
+        self._invdelta = {}
+        self.added = []
+        self.ignored = {}
+        if conflicts_related is None:
+            self.conflicts_related = frozenset()
+        else:
+            self.conflicts_related = conflicts_related
+
+    def add(self, file_list, recurse=True):
+        from bzrlib.inventory import InventoryEntry
+        if not file_list:
+            # no paths supplied: add the entire tree.
+            # FIXME: this assumes we are running in a working tree subdir :-/
+            # -- vila 20100208
+            file_list = [u'.']
+
+        # expand any symlinks in the directory part, while leaving the
+        # filename alone
+        # only expanding if symlinks are supported avoids windows path bugs
+        if osutils.has_symlinks():
+            file_list = map(osutils.normalizepath, file_list)
+
+        user_dirs = {}
+        # validate user file paths and convert all paths to tree
+        # relative : it's cheaper to make a tree relative path an abspath
+        # than to convert an abspath to tree relative, and it's cheaper to
+        # perform the canonicalization in bulk.
+        for filepath in osutils.canonical_relpaths(self.tree.basedir, file_list):
+            # validate user parameters. Our recursive code avoids adding new
+            # files that need such validation
+            if self.tree.is_control_filename(filepath):
+                raise errors.ForbiddenControlFileError(filename=filepath)
+
+            abspath = self.tree.abspath(filepath)
+            kind = osutils.file_kind(abspath)
+            # ensure the named path is added, so that ignore rules in the later
+            # directory walk dont skip it.
+            # we dont have a parent ie known yet.: use the relatively slower
+            # inventory probing method
+            inv_path, _ = osutils.normalized_filename(filepath)
+            this_ie = self._get_ie(inv_path)
+            if this_ie is None:
+                this_ie = self._add_one_and_parent(None, filepath, kind, inv_path)
+            if kind == 'directory':
+                # schedule the dir for scanning
+                user_dirs[filepath] = (inv_path, this_ie)
+
+        if not recurse:
+            # no need to walk any directories at all.
+            return
+
+        things_to_add = list(self._gather_dirs_to_add(user_dirs))
+
+        illegalpath_re = re.compile(r'[\r\n]')
+        for directory, inv_path, this_ie, parent_ie in things_to_add:
+            # directory is tree-relative
+            abspath = self.tree.abspath(directory)
+
+            # get the contents of this directory.
+
+            # find the kind of the path being added, and save stat_value
+            # for reuse
+            stat_value = None
+            if this_ie is None:
+                stat_value = osutils.file_stat(abspath)
+                kind = osutils.file_kind_from_stat_mode(stat_value.st_mode)
+            else:
+                kind = this_ie.kind
+            
+            # allow AddAction to skip this file
+            if self.action.skip_file(self.tree,  abspath,  kind,  stat_value):
+                continue
+            if not InventoryEntry.versionable_kind(kind):
+                trace.warning("skipping %s (can't add file of kind '%s')",
+                              abspath, kind)
+                continue
+            if illegalpath_re.search(directory):
+                trace.warning("skipping %r (contains \\n or \\r)" % abspath)
+                continue
+            if directory in self.conflicts_related:
+                # If the file looks like one generated for a conflict, don't
+                # add it.
+                trace.warning(
+                    'skipping %s (generated to help resolve conflicts)',
+                    abspath)
+                continue
+
+            if kind == 'directory' and directory != '':
+                try:
+                    transport = _mod_transport.get_transport_from_path(abspath)
+                    controldir.ControlDirFormat.find_format(transport)
+                    sub_tree = True
+                except errors.NotBranchError:
+                    sub_tree = False
+                except errors.UnsupportedFormatError:
+                    sub_tree = True
+            else:
+                sub_tree = False
+
+            if this_ie is not None:
+                pass
+            elif sub_tree:
+                # XXX: This is wrong; people *might* reasonably be trying to
+                # add subtrees as subtrees.  This should probably only be done
+                # in formats which can represent subtrees, and even then
+                # perhaps only when the user asked to add subtrees.  At the
+                # moment you can add them specially through 'join --reference',
+                # which is perhaps reasonable: adding a new reference is a
+                # special operation and can have a special behaviour.  mbp
+                # 20070306
+                trace.warning("skipping nested tree %r", abspath)
+            else:
+                this_ie = self._add_one_and_parent(parent_ie, directory, kind,
+                    inv_path)
+
+            if kind == 'directory' and not sub_tree:
+                if this_ie.kind != 'directory':
+                    this_ie = self._convert_to_directory(this_ie, inv_path)
+
+                for subf in sorted(os.listdir(abspath)):
+                    inv_f, _ = osutils.normalized_filename(subf)
+                    # here we could use TreeDirectory rather than
+                    # string concatenation.
+                    subp = osutils.pathjoin(directory, subf)
+                    # TODO: is_control_filename is very slow. Make it faster.
+                    # TreeDirectory.is_control_filename could also make this
+                    # faster - its impossible for a non root dir to have a
+                    # control file.
+                    if self.tree.is_control_filename(subp):
+                        trace.mutter("skip control directory %r", subp)
+                        continue
+                    sub_invp = osutils.pathjoin(inv_path, inv_f)
+                    entry = self._invdelta.get(sub_invp)
+                    if entry is not None:
+                        sub_ie = entry[3]
+                    else:
+                        sub_ie = this_ie.children.get(inv_f)
+                    if sub_ie is not None:
+                        # recurse into this already versioned subdir.
+                        things_to_add.append((subp, sub_invp, sub_ie, this_ie))
+                    else:
+                        # user selection overrides ignores
+                        # ignore while selecting files - if we globbed in the
+                        # outer loop we would ignore user files.
+                        ignore_glob = self.tree.is_ignored(subp)
+                        if ignore_glob is not None:
+                            self.ignored.setdefault(ignore_glob, []).append(subp)
+                        else:
+                            things_to_add.append((subp, sub_invp, None, this_ie))

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2010-05-23 18:57:38 +0000
+++ bzrlib/option.py	2011-09-27 13:18:53 +0000
@@ -25,6 +25,7 @@
 from bzrlib import (
     errors,
     revisionspec,
+    i18n,
     )
 """)
 
@@ -312,7 +313,7 @@
 
     def __init__(self, name, help, registry=None, converter=None,
         value_switches=False, title=None, enum_switch=True,
-        lazy_registry=None):
+        lazy_registry=None, short_name=None, short_value_switches=None):
         """
         Constructor.
 
@@ -328,8 +329,11 @@
             which takes a value.
         :param lazy_registry: A tuple of (module name, attribute name) for a
             registry to be lazily loaded.
+        :param short_name: The short name for the enum switch, if any
+        :param short_value_switches: A dict mapping values to short names
         """
-        Option.__init__(self, name, help, type=self.convert)
+        Option.__init__(self, name, help, type=self.convert,
+                        short_name=short_name)
         self._registry = registry
         if registry is None:
             if lazy_registry is None:
@@ -344,6 +348,7 @@
         self.converter = converter
         self.value_switches = value_switches
         self.enum_switch = enum_switch
+        self.short_value_switches = short_value_switches
         self.title = title
         if self.title is None:
             self.title = name
@@ -387,6 +392,10 @@
                     help = optparse.SUPPRESS_HELP
                 else:
                     help = self.registry.get_help(key)
+                if (self.short_value_switches and
+                    key in self.short_value_switches):
+                    option_strings.append('-%s' %
+                                          self.short_value_switches[key])
                 parser.add_option(action='callback',
                               callback=self._optparse_value_callback(key),
                                   help=help,
@@ -422,10 +431,26 @@
 
     DEFAULT_VALUE = object()
 
+    def __init__(self):
+        optparse.OptionParser.__init__(self)
+        self.formatter = GettextIndentedHelpFormatter()
+
     def error(self, message):
         raise errors.BzrCommandError(message)
 
 
+class GettextIndentedHelpFormatter(optparse.IndentedHelpFormatter):
+    """Adds gettext() call to format_option()"""
+    def __init__(self):
+        optparse.IndentedHelpFormatter.__init__(self)
+
+    def format_option(self, option):
+        """code taken from Python's optparse.py"""
+        if option.help:
+            option.help = i18n.gettext(option.help)
+        return optparse.IndentedHelpFormatter.format_option(self, option)
+
+
 def get_optparser(options):
     """Generate an optparse parser for bzrlib-style options"""
 
@@ -450,6 +475,12 @@
     Option.STD_OPTIONS[name] = Option(name, **kwargs)
     Option.OPTIONS[name] = Option.STD_OPTIONS[name]
 
+def _standard_list_option(name, **kwargs):
+    """Register a standard option."""
+    # All standard options are implicitly 'global' ones
+    Option.STD_OPTIONS[name] = ListOption(name, **kwargs)
+    Option.OPTIONS[name] = Option.STD_OPTIONS[name]
+
 
 def _global_option(name, **kwargs):
     """Register a global option."""
@@ -494,87 +525,91 @@
 
 
 _merge_type_registry = MergeTypeRegistry()
+_merge_type_registry.register_lazy('diff3', 'bzrlib.merge', 'Diff3Merger',
+                                   "Merge using external diff3")
+_merge_type_registry.register_lazy('lca', 'bzrlib.merge', 'LCAMerger',
+                                   "LCA-newness merge")
 _merge_type_registry.register_lazy('merge3', 'bzrlib.merge', 'Merge3Merger',
                                    "Native diff3-style merge")
-_merge_type_registry.register_lazy('diff3', 'bzrlib.merge', 'Diff3Merger',
-                                   "Merge using external diff3")
 _merge_type_registry.register_lazy('weave', 'bzrlib.merge', 'WeaveMerger',
                                    "Weave-based merge")
-_merge_type_registry.register_lazy('lca', 'bzrlib.merge', 'LCAMerger',
-                                   "LCA-newness merge")
 
 # Declare the standard options
 _standard_option('help', short_name='h',
                  help='Show help message.')
+_standard_option('quiet', short_name='q',
+                 help="Only display errors and warnings.",
+                 custom_callback=_verbosity_level_callback)
 _standard_option('usage',
                  help='Show usage message and options.')
 _standard_option('verbose', short_name='v',
                  help='Display more information.',
                  custom_callback=_verbosity_level_callback)
-_standard_option('quiet', short_name='q',
-                 help="Only display errors and warnings.",
-                 custom_callback=_verbosity_level_callback)
 
 # Declare commonly used options
 _global_option('all')
-_global_option('overwrite', help='Ignore differences between branches and '
-               'overwrite unconditionally.')
 _global_option('basis', type=str)
 _global_option('bound')
+_global_option('change',
+               type=_parse_change_str,
+               short_name='c',
+               param_name='revision',
+               help='Select changes introduced by the specified revision. See also "help revisionspec".')
 _global_option('diff-options', type=str)
+_global_option('directory', short_name='d', type=unicode,
+               help='Branch to operate on, instead of working directory')
+_global_option('dry-run',
+               help="Show what would be done, but don't actually do anything.")
+_global_option('email')
 _global_option('file', type=unicode, short_name='F')
 _global_option('force')
 _global_option('format', type=unicode)
 _global_option('forward')
+_global_option('kind', type=str)
+_global_option('line', help='Use log format with one line per revision.'
+               ' Same as --log-format line')
+_global_registry_option('log-format', "Use specified log format.",
+                        lazy_registry=('bzrlib.log', 'log_formatter_registry'),
+                        value_switches=True, title='Log format',
+                        short_value_switches={'short': 'S'})
+_global_option('long', help='Use detailed log format.'
+               ' Same as --log-format long',
+               short_name='l')
+_global_registry_option('merge-type', 'Select a particular merge algorithm.',
+                        _merge_type_registry, value_switches=True,
+                        title='Merge algorithm')
 _global_option('message', type=unicode,
                short_name='m',
                help='Message string.')
+_global_option('name-from-revision', help='The path name in the old tree.')
+_global_option('no-backup')
 _global_option('no-recurse')
 _global_option('null', short_name='0',
                  help='Use an ASCII NUL (\\0) separator rather than '
                       'a newline.')
+_global_option('overwrite', help='Ignore differences between branches and '
+               'overwrite unconditionally.')
+_global_option('pattern', type=str)
 _global_option('profile',
                help='Show performance profiling information.')
+_global_option('reprocess', help='Reprocess to reduce spurious conflicts.')
+_global_option('remember', help='Remember the specified location as a'
+               ' default.')
 _global_option('revision',
                type=_parse_revision_str,
                short_name='r',
                help='See "help revisionspec" for details.')
-_global_option('change',
-               type=_parse_change_str,
-               short_name='c',
-               param_name='revision',
-               help='Select changes introduced by the specified revision. See also "help revisionspec".')
+_global_option('short', help='Use moderately short log format.'
+               ' Same as --log-format short')
 _global_option('show-ids',
                help='Show internal object ids.')
 _global_option('timezone',
                type=str,
                help='Display timezone as local, original, or utc.')
+_global_option('root', type=str)
 _global_option('unbound')
+_global_option('update')
 _global_option('version')
-_global_option('email')
-_global_option('update')
-_global_registry_option('log-format', "Use specified log format.",
-                        lazy_registry=('bzrlib.log', 'log_formatter_registry'),
-                        value_switches=True, title='Log format')
-_global_option('long', help='Use detailed log format. Same as --log-format long',
-               short_name='l')
-_global_option('short', help='Use moderately short log format. Same as --log-format short')
-_global_option('line', help='Use log format with one line per revision. Same as --log-format line')
-_global_option('root', type=str)
-_global_option('no-backup')
-_global_registry_option('merge-type', 'Select a particular merge algorithm.',
-                        _merge_type_registry, value_switches=True,
-                        title='Merge algorithm')
-_global_option('pattern', type=str)
-_global_option('remember', help='Remember the specified location as a'
-               ' default.')
-_global_option('reprocess', help='Reprocess to reduce spurious conflicts.')
-_global_option('kind', type=str)
-_global_option('dry-run',
-               help="Show what would be done, but don't actually do anything.")
-_global_option('name-from-revision', help='The path name in the old tree.')
-_global_option('directory', short_name='d', type=unicode,
-               help='Branch to operate on, instead of working directory')
 
 diff_writer_registry = _mod_registry.Registry()
 diff_writer_registry.register('plain', lambda x: x, 'Plaintext diff output.')

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-07-09 16:16:11 +0000
+++ bzrlib/osutils.py	2011-10-12 16:00:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,10 +42,12 @@
 
 from bzrlib import (
     cache_utf8,
+    config,
     errors,
     trace,
     win32utils,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.symbol_versioning import (
@@ -53,18 +55,10 @@
     deprecated_in,
     )
 
-# sha and md5 modules are deprecated in python2.6 but hashlib is available as
-# of 2.5
-if sys.version_info < (2, 5):
-    import md5 as _mod_md5
-    md5 = _mod_md5.new
-    import sha as _mod_sha
-    sha = _mod_sha.new
-else:
-    from hashlib import (
-        md5,
-        sha1 as sha,
-        )
+from hashlib import (
+    md5,
+    sha1 as sha,
+    )
 
 
 import bzrlib
@@ -96,8 +90,8 @@
         user_encoding = get_user_encoding()
         return [a.decode(user_encoding) for a in sys.argv[1:]]
     except UnicodeDecodeError:
-        raise errors.BzrError(("Parameter '%r' is unsupported by the current "
-                                                            "encoding." % a))
+        raise errors.BzrError(gettext("Parameter {0!r} encoding is unsupported by {1} "
+            "application locale.").format(a, user_encoding))
 
 
 def make_readonly(filename):
@@ -197,7 +191,7 @@
             if e.errno == errno.ENOENT:
                 return False;
             else:
-                raise errors.BzrError("lstat/stat of (%r): %r" % (f, e))
+                raise errors.BzrError(gettext("lstat/stat of ({0!r}): {1!r}").format(f, e))
 
 
 def fancy_rename(old, new, rename_func, unlink_func):
@@ -269,7 +263,10 @@
             else:
                 rename_func(tmp_name, new)
     if failure_exc is not None:
-        raise failure_exc[0], failure_exc[1], failure_exc[2]
+        try:
+            raise failure_exc[0], failure_exc[1], failure_exc[2]
+        finally:
+            del failure_exc
 
 
 # In Python 2.4.2 and older, os.path.abspath and os.path.realpath
@@ -282,13 +279,28 @@
     # copy posixpath.abspath, but use os.getcwdu instead
     if not posixpath.isabs(path):
         path = posixpath.join(getcwd(), path)
-    return posixpath.normpath(path)
+    return _posix_normpath(path)
 
 
 def _posix_realpath(path):
     return posixpath.realpath(path.encode(_fs_enc)).decode(_fs_enc)
 
 
+def _posix_normpath(path):
+    path = posixpath.normpath(path)
+    # Bug 861008: posixpath.normpath() returns a path normalized according to
+    # the POSIX standard, which stipulates (for compatibility reasons) that two
+    # leading slashes must not be simplified to one, and only if there are 3 or
+    # more should they be simplified as one. So we treat the leading 2 slashes
+    # as a special case here by simply removing the first slash, as we consider
+    # that breaking POSIX compatibility for this obscure feature is acceptable.
+    # This is not a paranoid precaution, as we notably get paths like this when
+    # the repo is hosted at the root of the filesystem, i.e. in "/".    
+    if path.startswith('//'):
+        path = path[1:]
+    return path
+
+
 def _win32_fixdrive(path):
     """Force drive letters to be consistent.
 
@@ -382,7 +394,7 @@
 abspath = _posix_abspath
 realpath = _posix_realpath
 pathjoin = os.path.join
-normpath = os.path.normpath
+normpath = _posix_normpath
 getcwd = os.getcwdu
 rename = os.rename
 dirname = os.path.dirname
@@ -392,6 +404,12 @@
 # These were already lazily imported into local scope
 # mkdtemp = tempfile.mkdtemp
 # rmtree = shutil.rmtree
+lstat = os.lstat
+fstat = os.fstat
+
+def wrap_stat(st):
+    return st
+
 
 MIN_ABS_PATHLENGTH = 1
 
@@ -407,6 +425,14 @@
     getcwd = _win32_getcwd
     mkdtemp = _win32_mkdtemp
     rename = _win32_rename
+    try:
+        from bzrlib import _walkdirs_win32
+    except ImportError:
+        pass
+    else:
+        lstat = _walkdirs_win32.lstat
+        fstat = _walkdirs_win32.fstat
+        wrap_stat = _walkdirs_win32.wrap_stat
 
     MIN_ABS_PATHLENGTH = 3
 
@@ -915,7 +941,7 @@
     rps = []
     for f in ps:
         if f == '..':
-            raise errors.BzrError("sorry, %r not allowed in path" % f)
+            raise errors.BzrError(gettext("sorry, %r not allowed in path") % f)
         elif (f == '.') or (f == ''):
             pass
         else:
@@ -926,7 +952,7 @@
 def joinpath(p):
     for f in p:
         if (f == '..') or (f is None) or (f == ''):
-            raise errors.BzrError("sorry, %r not allowed in path" % f)
+            raise errors.BzrError(gettext("sorry, %r not allowed in path") % f)
     return pathjoin(*p)
 
 
@@ -967,7 +993,6 @@
     # they tend to happen very early in startup when we can't check config
     # files etc, and also we want to report all failures but not spam the user
     # with 10 warnings.
-    from bzrlib import trace
     exception_str = str(exception)
     if exception_str not in _extension_load_failures:
         trace.mutter("failed to load compiled extension: %s" % exception_str)
@@ -977,8 +1002,7 @@
 def report_extension_load_failures():
     if not _extension_load_failures:
         return
-    from bzrlib.config import GlobalConfig
-    if GlobalConfig().get_user_option_as_bool('ignore_missing_extensions'):
+    if config.GlobalStack().get('ignore_missing_extensions'):
         return
     # the warnings framework should by default show this only once
     from bzrlib.trace import warning
@@ -1146,7 +1170,7 @@
 
     if len(base) < MIN_ABS_PATHLENGTH:
         # must have space for e.g. a drive letter
-        raise ValueError('%r is too short to calculate a relative path'
+        raise ValueError(gettext('%r is too short to calculate a relative path')
             % (base,))
 
     rp = abspath(path)
@@ -1462,10 +1486,16 @@
     # a similar effect.
 
     # If BZR_COLUMNS is set, take it, user is always right
+    # Except if they specified 0 in which case, impose no limit here
     try:
-        return int(os.environ['BZR_COLUMNS'])
+        width = int(os.environ['BZR_COLUMNS'])
     except (KeyError, ValueError):
-        pass
+        width = None
+    if width is not None:
+        if width > 0:
+            return width
+        else:
+            return None
 
     isatty = getattr(sys.stdout, 'isatty', None)
     if isatty is None or not isatty():
@@ -1875,7 +1905,10 @@
         s = os.stat(src)
         chown(dst, s.st_uid, s.st_gid)
     except OSError, e:
-        trace.warning("Unable to copy ownership from '%s' to '%s': IOError: %s." % (src, dst, e))
+        trace.warning(
+            'Unable to copy ownership from "%s" to "%s". '
+            'You may want to set it manually.', src, dst)
+        trace.log_exception_quietly()
 
 
 def path_prefix_key(path):
@@ -1993,6 +2026,14 @@
 # data at once.
 MAX_SOCKET_CHUNK = 64 * 1024
 
+_end_of_stream_errors = [errno.ECONNRESET]
+for _eno in ['WSAECONNRESET', 'WSAECONNABORTED']:
+    _eno = getattr(errno, _eno, None)
+    if _eno is not None:
+        _end_of_stream_errors.append(_eno)
+del _eno
+
+
 def read_bytes_from_socket(sock, report_activity=None,
         max_read_size=MAX_SOCKET_CHUNK):
     """Read up to max_read_size of bytes from sock and notify of progress.
@@ -2006,7 +2047,7 @@
             bytes = sock.recv(max_read_size)
         except socket.error, e:
             eno = e.args[0]
-            if eno == getattr(errno, "WSAECONNRESET", errno.ECONNRESET):
+            if eno in _end_of_stream_errors:
                 # The connection was closed by the other side.  Callers expect
                 # an empty string to signal end-of-stream.
                 return ""
@@ -2065,6 +2106,29 @@
             report_activity(sent, 'write')
 
 
+def connect_socket(address):
+    # Slight variation of the socket.create_connection() function (provided by
+    # python-2.6) that can fail if getaddrinfo returns an empty list. We also
+    # provide it for previous python versions. Also, we don't use the timeout
+    # parameter (provided by the python implementation) so we don't implement
+    # it either).
+    err = socket.error('getaddrinfo returns an empty list')
+    host, port = address
+    for res in socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM):
+        af, socktype, proto, canonname, sa = res
+        sock = None
+        try:
+            sock = socket.socket(af, socktype, proto)
+            sock.connect(sa)
+            return sock
+
+        except socket.error, err:
+            # 'err' is now the most recent error
+            if sock is not None:
+                sock.close()
+    raise err
+
+
 def dereference_path(path):
     """Determine the real path to a file.
 
@@ -2130,15 +2194,18 @@
     return file_kind_from_stat_mode(mode)
 file_kind_from_stat_mode = file_kind_from_stat_mode_thunk
 
-
-def file_kind(f, _lstat=os.lstat):
+def file_stat(f, _lstat=os.lstat):
     try:
-        return file_kind_from_stat_mode(_lstat(f).st_mode)
+        # XXX cache?
+        return _lstat(f)
     except OSError, e:
         if getattr(e, 'errno', None) in (errno.ENOENT, errno.ENOTDIR):
             raise errors.NoSuchFile(f)
         raise
 
+def file_kind(f, _lstat=os.lstat):
+    stat_value = file_stat(f, _lstat)
+    return file_kind_from_stat_mode(stat_value.st_mode)
 
 def until_no_eintr(f, *a, **kw):
     """Run f(*a, **kw), retrying if an EINTR error occurs.
@@ -2204,20 +2271,17 @@
             termios.tcsetattr(fd, termios.TCSADRAIN, settings)
         return ch
 
-
-if sys.platform == 'linux2':
+if sys.platform.startswith('linux'):
     def _local_concurrency():
-        concurrency = None
-        prefix = 'processor'
-        for line in file('/proc/cpuinfo', 'rb'):
-            if line.startswith(prefix):
-                concurrency = int(line[line.find(':')+1:]) + 1
-        return concurrency
+        try:
+            return os.sysconf('SC_NPROCESSORS_ONLN')
+        except (ValueError, OSError, AttributeError):
+            return None
 elif sys.platform == 'darwin':
     def _local_concurrency():
         return subprocess.Popen(['sysctl', '-n', 'hw.availcpu'],
                                 stdout=subprocess.PIPE).communicate()[0]
-elif sys.platform[0:7] == 'freebsd':
+elif "bsd" in sys.platform:
     def _local_concurrency():
         return subprocess.Popen(['sysctl', '-n', 'hw.ncpu'],
                                 stdout=subprocess.PIPE).communicate()[0]
@@ -2251,9 +2315,15 @@
     concurrency = os.environ.get('BZR_CONCURRENCY', None)
     if concurrency is None:
         try:
-            concurrency = _local_concurrency()
-        except (OSError, IOError):
-            pass
+            import multiprocessing
+        except ImportError:
+            # multiprocessing is only available on Python >= 2.6
+            try:
+                concurrency = _local_concurrency()
+            except (OSError, IOError):
+                pass
+        else:
+            concurrency = multiprocessing.cpu_count()
     try:
         concurrency = int(concurrency)
     except (TypeError, ValueError):
@@ -2330,4 +2400,120 @@
     except UnicodeDecodeError:
         raise errors.BzrError("Can't decode username as %s." % \
                 user_encoding)
+    except ImportError, e:
+        if sys.platform != 'win32':
+            raise
+        if str(e) != 'No module named pwd':
+            raise
+        # https://bugs.launchpad.net/bzr/+bug/660174
+        # getpass.getuser() is unable to return username on Windows
+        # if there is no USERNAME environment variable set.
+        # That could be true if bzr is running as a service,
+        # e.g. running `bzr serve` as a service on Windows.
+        # We should not fail with traceback in this case.
+        username = u'UNKNOWN'
     return username
+
+
+def available_backup_name(base, exists):
+    """Find a non-existing backup file name.
+
+    This will *not* create anything, this only return a 'free' entry.  This
+    should be used for checking names in a directory below a locked
+    tree/branch/repo to avoid race conditions. This is LBYL (Look Before You
+    Leap) and generally discouraged.
+
+    :param base: The base name.
+
+    :param exists: A callable returning True if the path parameter exists.
+    """
+    counter = 1
+    name = "%s.~%d~" % (base, counter)
+    while exists(name):
+        counter += 1
+        name = "%s.~%d~" % (base, counter)
+    return name
+
+
+def set_fd_cloexec(fd):
+    """Set a Unix file descriptor's FD_CLOEXEC flag.  Do nothing if platform
+    support for this is not available.
+    """
+    try:
+        import fcntl
+        old = fcntl.fcntl(fd, fcntl.F_GETFD)
+        fcntl.fcntl(fd, fcntl.F_SETFD, old | fcntl.FD_CLOEXEC)
+    except (ImportError, AttributeError):
+        # Either the fcntl module or specific constants are not present
+        pass
+
+
+def find_executable_on_path(name):
+    """Finds an executable on the PATH.
+    
+    On Windows, this will try to append each extension in the PATHEXT
+    environment variable to the name, if it cannot be found with the name
+    as given.
+    
+    :param name: The base name of the executable.
+    :return: The path to the executable found or None.
+    """
+    path = os.environ.get('PATH')
+    if path is None:
+        return None
+    path = path.split(os.pathsep)
+    if sys.platform == 'win32':
+        exts = os.environ.get('PATHEXT', '').split(os.pathsep)
+        exts = [ext.lower() for ext in exts]
+        base, ext = os.path.splitext(name)
+        if ext != '':
+            if ext.lower() not in exts:
+                return None
+            name = base
+            exts = [ext]
+    else:
+        exts = ['']
+    for ext in exts:
+        for d in path:
+            f = os.path.join(d, name) + ext
+            if os.access(f, os.X_OK):
+                return f
+    return None
+
+
+def _posix_is_local_pid_dead(pid):
+    """True if pid doesn't correspond to live process on this machine"""
+    try:
+        # Special meaning of unix kill: just check if it's there.
+        os.kill(pid, 0)
+    except OSError, e:
+        if e.errno == errno.ESRCH:
+            # On this machine, and really not found: as sure as we can be
+            # that it's dead.
+            return True
+        elif e.errno == errno.EPERM:
+            # exists, though not ours
+            return False
+        else:
+            mutter("os.kill(%d, 0) failed: %s" % (pid, e))
+            # Don't really know.
+            return False
+    else:
+        # Exists and our process: not dead.
+        return False
+
+if sys.platform == "win32":
+    is_local_pid_dead = win32utils.is_local_pid_dead
+else:
+    is_local_pid_dead = _posix_is_local_pid_dead
+
+
+def fdatasync(fileno):
+    """Flush file contents to disk if possible.
+    
+    :param fileno: Integer OS file handle.
+    :raises TransportNotPossible: If flushing to disk is not possible.
+    """
+    fn = getattr(os, 'fdatasync', getattr(os, 'fsync', None))
+    if fn is not None:
+        fn(fileno)

=== modified file 'bzrlib/pack.py'
--- bzrlib/pack.py	2010-03-15 06:54:44 +0000
+++ bzrlib/pack.py	2011-09-07 11:46:16 +0000
@@ -333,7 +333,7 @@
                 # risk that the same unicode string has been encoded two
                 # different ways.
                 if name_tuple in all_names:
-                    raise errors.DuplicateRecordNameError(name_tuple)
+                    raise errors.DuplicateRecordNameError(name_tuple[0])
                 all_names.add(name_tuple)
         excess_bytes = self.reader_func(1)
         if excess_bytes != '':

=== modified file 'bzrlib/patiencediff.py'
--- bzrlib/patiencediff.py	2009-03-23 14:59:43 +0000
+++ bzrlib/patiencediff.py	2011-04-05 01:12:15 +0000
@@ -73,7 +73,6 @@
      four
     """
     if sequencematcher is None:
-        import difflib
         sequencematcher = difflib.SequenceMatcher
 
     if fromfiledate:

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2010-06-11 08:02:42 +0000
+++ bzrlib/plugin.py	2011-09-20 11:01:37 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,7 +36,6 @@
 from bzrlib import osutils
 
 from bzrlib.lazy_import import lazy_import
-
 lazy_import(globals(), """
 import imp
 import re
@@ -49,20 +48,21 @@
     errors,
     trace,
     )
+from bzrlib.i18n import gettext
 from bzrlib import plugins as _mod_plugins
 """)
 
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    )
-
 
 DEFAULT_PLUGIN_PATH = None
 _loaded = False
 _plugins_disabled = False
 
 
+plugin_warnings = {}
+# Map from plugin name, to list of string warnings about eg plugin
+# dependencies.
+
+
 def are_plugins_disabled():
     return _plugins_disabled
 
@@ -77,6 +77,43 @@
     load_plugins([])
 
 
+def describe_plugins(show_paths=False):
+    """Generate text description of plugins.
+
+    Includes both those that have loaded, and those that failed to 
+    load.
+
+    :param show_paths: If true,
+    :returns: Iterator of text lines (including newlines.)
+    """
+    from inspect import getdoc
+    loaded_plugins = plugins()
+    all_names = sorted(list(set(
+        loaded_plugins.keys() + plugin_warnings.keys())))
+    for name in all_names:
+        if name in loaded_plugins:
+            plugin = loaded_plugins[name]
+            version = plugin.__version__
+            if version == 'unknown':
+                version = ''
+            yield '%s %s\n' % (name, version)
+            d = getdoc(plugin.module)
+            if d:
+                doc = d.split('\n')[0]
+            else:
+                doc = '(no description)'
+            yield ("  %s\n" % doc)
+            if show_paths:
+                yield ("   %s\n" % plugin.path())
+            del plugin
+        else:
+            yield "%s (failed to load)\n" % name
+        if name in plugin_warnings:
+            for line in plugin_warnings[name]:
+                yield "  ** " + line + '\n'
+        yield '\n'
+
+
 def _strip_trailing_sep(path):
     return path.rstrip("\\/")
 
@@ -101,8 +138,8 @@
         try:
             name, path = spec.split('@')
         except ValueError:
-            raise errors.BzrCommandError(
-                '"%s" is not a valid <plugin_name>@<plugin_path> description '
+            raise errors.BzrCommandError(gettext(
+                '"%s" is not a valid <plugin_name>@<plugin_path> description ')
                 % spec)
         specs.append((name, path))
     return specs
@@ -236,9 +273,9 @@
     """Load bzrlib plugins.
 
     The environment variable BZR_PLUGIN_PATH is considered a delimited
-    set of paths to look through. Each entry is searched for *.py
+    set of paths to look through. Each entry is searched for `*.py`
     files (and whatever other extensions are used in the platform,
-    such as *.pyd).
+    such as `*.pyd`).
 
     load_from_path() provides the underlying mechanism and is called with
     the default directory list to provide the normal behaviour.
@@ -327,6 +364,11 @@
     return None, None, (None, None, None)
 
 
+def record_plugin_warning(plugin_name, warning_message):
+    trace.mutter(warning_message)
+    plugin_warnings.setdefault(plugin_name, []).append(warning_message)
+
+
 def _load_plugin_module(name, dir):
     """Load plugin name from dir.
 
@@ -340,10 +382,12 @@
     except KeyboardInterrupt:
         raise
     except errors.IncompatibleAPI, e:
-        trace.warning("Unable to load plugin %r. It requested API version "
+        warning_message = (
+            "Unable to load plugin %r. It requested API version "
             "%s of module %s but the minimum exported version is %s, and "
             "the maximum is %s" %
             (name, e.wanted, e.api, e.minimum, e.current))
+        record_plugin_warning(name, warning_message)
     except Exception, e:
         trace.warning("%s" % e)
         if re.search('\.|-| ', name):
@@ -354,7 +398,9 @@
                     "file path isn't a valid module name; try renaming "
                     "it to %r." % (name, dir, sanitised_name))
         else:
-            trace.warning('Unable to load plugin %r from %r' % (name, dir))
+            record_plugin_warning(
+                name,
+                'Unable to load plugin %r from %r' % (name, dir))
         trace.log_exception_quietly()
         if 'error' in debug.debug_flags:
             trace.print_exception(sys.exc_info(), sys.stderr)
@@ -400,6 +446,17 @@
     return result
 
 
+def format_concise_plugin_list():
+    """Return a string holding a concise list of plugins and their version.
+    """
+    items = []
+    for name, a_plugin in sorted(plugins().items()):
+        items.append("%s[%s]" %
+            (name, a_plugin.__version__))
+    return ', '.join(items)
+
+
+
 class PluginsHelpIndex(object):
     """A help index that returns help topics for plugins."""
 
@@ -450,21 +507,12 @@
             result = self.module.__doc__
         if result[-1] != '\n':
             result += '\n'
-        # there is code duplicated here and in bzrlib/help_topic.py's
-        # matching Topic code. This should probably be factored in
-        # to a helper function and a common base class.
-        if additional_see_also is not None:
-            see_also = sorted(set(additional_see_also))
-        else:
-            see_also = None
-        if see_also:
-            result += 'See also: '
-            result += ', '.join(see_also)
-            result += '\n'
+        from bzrlib import help_topics
+        result += help_topics._format_see_also(additional_see_also)
         return result
 
     def get_help_topic(self):
-        """Return the modules help topic - its __name__ after bzrlib.plugins.."""
+        """Return the module help topic: its basename."""
         return self.module.__name__[len('bzrlib.plugins.'):]
 
 

=== modified file 'bzrlib/plugins/bash_completion/tests/test_bashcomp.py'
--- bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2010-05-29 10:30:53 +0000
+++ bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2011-01-28 10:28:35 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import sys
+
 import bzrlib
 from bzrlib import commands, tests
 from bzrlib.tests import features
@@ -27,6 +29,7 @@
     """Component for testing execution of a bash completion script."""
 
     _test_needs_features = [features.bash_feature]
+    script = None
 
     def complete(self, words, cword=-1):
         """Perform a bash completion.
@@ -95,10 +98,6 @@
 class TestBashCompletion(tests.TestCase, BashCompletionMixin):
     """Test bash completions that don't execute bzr."""
 
-    def __init__(self, methodName='testMethod'):
-        super(TestBashCompletion, self).__init__(methodName)
-        self.script = None
-
     def test_simple_scipt(self):
         """Ensure that the test harness works as expected"""
         self.script = """
@@ -155,9 +154,11 @@
     will be replaced by the bzr instance running this selftest.
     """
 
-    def __init__(self, methodName='testMethod'):
-        super(TestBashCompletionInvoking, self).__init__(methodName)
-        self.script = None
+    def setUp(self):
+        super(TestBashCompletionInvoking, self).setUp()
+        if sys.platform == 'win32':
+            raise tests.KnownFailure(
+                'see bug #709104, completion is broken on windows')
 
     def get_script(self):
         s = super(TestBashCompletionInvoking, self).get_script()

=== added directory 'bzrlib/plugins/changelog_merge'
=== added file 'bzrlib/plugins/changelog_merge/__init__.py'
--- bzrlib/plugins/changelog_merge/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/changelog_merge/__init__.py	2011-03-31 15:10:05 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+__doc__ = """Merge hook for GNU-format ChangeLog files
+
+To enable this plugin, add a section to your location.conf
+like::
+
+    [/home/user/proj]
+    changelog_merge_files = ChangeLog
+
+Or add an entry to your branch.conf like::
+
+    changelog_merge_files = ChangeLog
+
+The changelog_merge_files config option takes a list of file names (not paths),
+separated by commas.  (This is unlike the news_merge plugin, which matches
+paths.)  e.g. the above config examples would match both
+``src/foolib/ChangeLog`` and ``docs/ChangeLog``.
+
+The algorithm used to merge the changes can be summarised as:
+
+ * new entries added to the top of OTHER are emitted first
+ * all other additions, deletions and edits from THIS and OTHER are preserved
+ * edits (e.g. to fix typos) at the top of OTHER are hard to distinguish from
+   adding and deleting independent entries; the algorithm tries to guess which
+   based on how similar the old and new entries are.
+
+Caveats
+-------
+
+Most changes can be merged, but conflicts are possible if the plugin finds
+edits at the top of OTHER to entries that have been deleted (or also edited) by
+THIS.  In that case the plugin gives up and bzr's default merge logic will be
+used.
+
+No effort is made to deduplicate entries added by both sides.
+
+The results depend on the choice of the 'base' version, so it might give
+strange results if there is a criss-cross merge.
+"""
+
+# Since we are a built-in plugin we share the bzrlib version
+from bzrlib import version_info
+from bzrlib.hooks import install_lazy_named_hook
+
+# Put most of the code in a separate module that we lazy-import to keep the
+# overhead of this plugin as minimal as possible.
+def changelog_merge_hook(merger):
+    """Merger.merge_file_content hook for GNU-format ChangeLog files."""
+    from bzrlib.plugins.changelog_merge.changelog_merge import ChangeLogMerger
+    return ChangeLogMerger(merger)
+
+install_lazy_named_hook("bzrlib.merge", "Merger.hooks", "merge_file_content",
+    changelog_merge_hook, 'GNU ChangeLog file merge')
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'tests',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== added file 'bzrlib/plugins/changelog_merge/changelog_merge.py'
--- bzrlib/plugins/changelog_merge/changelog_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/changelog_merge/changelog_merge.py	2011-05-13 06:50:20 +0000
@@ -0,0 +1,194 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Merge logic for changelog_merge plugin."""
+
+import difflib
+
+from bzrlib import merge
+from bzrlib import debug
+from bzrlib.merge3 import Merge3
+from bzrlib.trace import mutter
+
+
+def changelog_entries(lines):
+    """Return a list of changelog entries.
+
+    :param lines: lines of a changelog file.
+    :returns: list of entries.  Each entry is a tuple of lines.
+    """
+    entries = []
+    for line in lines:
+        if line[0] not in (' ', '\t', '\n'):
+            # new entry
+            entries.append([line])
+        else:
+            try:
+                entry = entries[-1]
+            except IndexError:
+                # Cope with leading blank lines.
+                entries.append([])
+                entry = entries[-1]
+            entry.append(line)
+    return map(tuple, entries)
+
+
+def entries_to_lines(entries):
+    """Turn a list of entries into a flat iterable of lines."""
+    for entry in entries:
+        for line in entry:
+            yield line
+
+
+class ChangeLogMerger(merge.ConfigurableFileMerger):
+    """Merge GNU-format ChangeLog files."""
+
+    name_prefix = "changelog"
+
+    def get_filepath(self, params, tree):
+        """Calculate the path to the file in a tree.
+
+        This is overridden to return just the basename, rather than full path,
+        so that e.g. if the config says ``changelog_merge_files = ChangeLog``,
+        then all ChangeLog files in the tree will match (not just one in the
+        root of the tree).
+        
+        :param params: A MergeHookParams describing the file to merge
+        :param tree: a Tree, e.g. self.merger.this_tree.
+        """
+        return tree.inventory[params.file_id].name
+
+    def merge_text(self, params):
+        """Merge changelog changes.
+
+         * new entries from other will float to the top
+         * edits to older entries are preserved
+        """
+        # Transform files into lists of changelog entries
+        this_entries = changelog_entries(params.this_lines)
+        other_entries = changelog_entries(params.other_lines)
+        base_entries = changelog_entries(params.base_lines)
+        try:
+            result_entries = merge_entries(
+                base_entries, this_entries, other_entries)
+        except EntryConflict:
+            # XXX: generating a nice conflict file would be better
+            return 'not_applicable', None
+        # Transform the merged elements back into real blocks of lines.
+        return 'success', entries_to_lines(result_entries)
+
+
+class EntryConflict(Exception):
+    pass
+
+
+def default_guess_edits(new_entries, deleted_entries, entry_as_str=''.join):
+    """Default implementation of guess_edits param of merge_entries.
+
+    This algorithm does O(N^2 * logN) SequenceMatcher.ratio() calls, which is
+    pretty bad, but it shouldn't be used very often.
+    """
+    deleted_entries_as_strs = map(entry_as_str, deleted_entries)
+    new_entries_as_strs = map(entry_as_str, new_entries)
+    result_new = list(new_entries)
+    result_deleted = list(deleted_entries)
+    result_edits = []
+    sm = difflib.SequenceMatcher()
+    CUTOFF = 0.8
+    while True:
+        best = None
+        best_score = CUTOFF
+        # Compare each new entry with each old entry to find the best match
+        for new_entry_as_str in new_entries_as_strs:
+            sm.set_seq1(new_entry_as_str)
+            for old_entry_as_str in deleted_entries_as_strs:
+                sm.set_seq2(old_entry_as_str)
+                score = sm.ratio()
+                if score > best_score:
+                    best = new_entry_as_str, old_entry_as_str
+                    best_score = score
+        if best is not None:
+            # Add the best match to the list of edits, and remove it from the
+            # the list of new/old entries.  Also remove it from the new/old
+            # lists for the next round.
+            del_index = deleted_entries_as_strs.index(best[1])
+            new_index = new_entries_as_strs.index(best[0])
+            result_edits.append(
+                (result_deleted[del_index], result_new[new_index]))
+            del deleted_entries_as_strs[del_index], result_deleted[del_index]
+            del new_entries_as_strs[new_index], result_new[new_index]
+        else:
+            # No match better than CUTOFF exists in the remaining new and old
+            # entries.
+            break
+    return result_new, result_deleted, result_edits
+
+
+def merge_entries(base_entries, this_entries, other_entries,
+        guess_edits=default_guess_edits):
+    """Merge changelog given base, this, and other versions."""
+    m3 = Merge3(base_entries, this_entries, other_entries, allow_objects=True)
+    result_entries = []
+    at_top = True
+    for group in m3.merge_groups():
+        if 'changelog_merge' in debug.debug_flags:
+            mutter('merge group:\n%r', group)
+        group_kind = group[0]
+        if group_kind == 'conflict':
+            _, base, this, other = group
+            # Find additions
+            new_in_other = [
+                entry for entry in other if entry not in base]
+            # Find deletions
+            deleted_in_other = [
+                entry for entry in base if entry not in other]
+            if at_top and deleted_in_other:
+                # Magic!  Compare deletions and additions to try spot edits
+                new_in_other, deleted_in_other, edits_in_other = guess_edits(
+                    new_in_other, deleted_in_other)
+            else:
+                # Changes not made at the top are always preserved as is, no
+                # need to try distinguish edits from adds and deletes.
+                edits_in_other = []
+            if 'changelog_merge' in debug.debug_flags:
+                mutter('at_top: %r', at_top)
+                mutter('new_in_other: %r', new_in_other)
+                mutter('deleted_in_other: %r', deleted_in_other)
+                mutter('edits_in_other: %r', edits_in_other)
+            # Apply deletes and edits
+            updated_this = [
+                entry for entry in this if entry not in deleted_in_other]
+            for old_entry, new_entry in edits_in_other:
+                try:
+                    index = updated_this.index(old_entry)
+                except ValueError:
+                    # edited entry no longer present in this!  Just give up and
+                    # declare a conflict.
+                    raise EntryConflict()
+                updated_this[index] = new_entry
+            if 'changelog_merge' in debug.debug_flags:
+                mutter('updated_this: %r', updated_this)
+            if at_top:
+                # Float new entries from other to the top
+                result_entries = new_in_other + result_entries
+            else:
+                result_entries.extend(new_in_other)
+            result_entries.extend(updated_this)
+        else: # unchanged, same, a, or b.
+            lines = group[1]
+            result_entries.extend(lines)
+        at_top = False
+    return result_entries

=== added directory 'bzrlib/plugins/changelog_merge/tests'
=== added file 'bzrlib/plugins/changelog_merge/tests/__init__.py'
--- bzrlib/plugins/changelog_merge/tests/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/changelog_merge/tests/__init__.py	2011-03-10 08:02:30 +0000
@@ -0,0 +1,24 @@
+# Copyright (C) 2011 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'test_changelog_merge',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests
+

=== added file 'bzrlib/plugins/changelog_merge/tests/test_changelog_merge.py'
--- bzrlib/plugins/changelog_merge/tests/test_changelog_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/changelog_merge/tests/test_changelog_merge.py	2011-05-13 06:34:55 +0000
@@ -0,0 +1,219 @@
+# Copyright (C) 2011 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from bzrlib import (
+    merge,
+    tests,
+    )
+from bzrlib.tests import test_merge_core
+from bzrlib.plugins.changelog_merge import changelog_merge
+
+
+sample_base_entries = [
+    'Base entry B1',
+    'Base entry B2',
+    'Base entry B3',
+    ]
+
+sample_this_entries = [
+    'This entry T1',
+    'This entry T2',
+    #'Base entry B1 updated',
+    'Base entry B1',
+    'Base entry B2',
+    'Base entry B3',
+    ]
+
+sample_other_entries = [
+    'Other entry O1',
+    #'Base entry B1',
+    'Base entry B1',
+    'Base entry B2 updated',
+    'Base entry B3',
+    ]
+
+
+sample2_base_entries = [
+    'Base entry B1',
+    'Base entry B2',
+    'Base entry B3',
+    ]
+
+sample2_this_entries = [
+    'This entry T1',
+    'This entry T2',
+    #'Base entry B1 updated',
+    'Base entry B1',
+    'Base entry B2',
+    ]
+
+sample2_other_entries = [
+    'Other entry O1',
+    #'Base entry B1',
+    'Base entry B1 edit',  # > 80% similar according to difflib
+    'Base entry B2',
+    ]
+
+
+class TestMergeCoreLogic(tests.TestCase):
+
+    def test_new_in_other_floats_to_top(self):
+        """Changes at the top of 'other' float to the top.
+
+        Given a changelog in THIS containing::
+
+          NEW-1
+          OLD-1
+
+        and a changelog in OTHER containing::
+
+          NEW-2
+          OLD-1
+
+        it will merge as::
+
+          NEW-2
+          NEW-1
+          OLD-1
+        """
+        base_entries = ['OLD-1']
+        this_entries = ['NEW-1', 'OLD-1']
+        other_entries = ['NEW-2', 'OLD-1']
+        result_entries = changelog_merge.merge_entries(
+            base_entries, this_entries, other_entries)
+        self.assertEqual(
+            ['NEW-2', 'NEW-1', 'OLD-1'], result_entries)
+
+    def test_acceptance_bug_723968(self):
+        """Merging a branch that:
+
+         1. adds a new entry, and
+         2. edits an old entry (e.g. to fix a typo or twiddle formatting)
+
+        will:
+
+         1. add the new entry to the top
+         2. keep the edit, without duplicating the edited entry or moving it.
+        """
+        result_entries = changelog_merge.merge_entries(
+            sample_base_entries, sample_this_entries, sample_other_entries)
+        self.assertEqual([
+            'Other entry O1',
+            'This entry T1',
+            'This entry T2',
+            'Base entry B1',
+            'Base entry B2 updated',
+            'Base entry B3',
+            ],
+            list(result_entries))
+
+    def test_more_complex_conflict(self):
+        """Like test_acceptance_bug_723968, but with a more difficult conflict:
+        the new entry and the edited entry are adjacent.
+        """
+        def guess_edits(new, deleted):
+            #import pdb; pdb.set_trace()
+            return changelog_merge.default_guess_edits(new, deleted,
+                    entry_as_str=lambda x: x)
+        result_entries = changelog_merge.merge_entries(
+            sample2_base_entries, sample2_this_entries, sample2_other_entries,
+            guess_edits=guess_edits)
+        self.assertEqual([
+            'Other entry O1',
+            'This entry T1',
+            'This entry T2',
+            'Base entry B1 edit',
+            'Base entry B2',
+            ],
+            list(result_entries))
+
+    def test_too_hard(self):
+        """A conflict this plugin cannot resolve raises EntryConflict.
+        """
+        # An entry edited in other but deleted in this is a conflict we can't
+        # resolve.  (Ideally perhaps we'd generate a nice conflict file, but
+        # for now we just give up.)
+        self.assertRaises(changelog_merge.EntryConflict,
+            changelog_merge.merge_entries,
+            sample2_base_entries, [], sample2_other_entries)
+
+    def test_default_guess_edits(self):
+        """default_guess_edits matches a new entry only once.
+        
+        (Even when that entry is the best match for multiple old entries.)
+        """
+        new_in_other = [('AAAAA',), ('BBBBB',)]
+        deleted_in_other = [('DDDDD',), ('BBBBBx',), ('BBBBBxx',)]
+        # BBBBB is the best match for both BBBBBx and BBBBBxx
+        result = changelog_merge.default_guess_edits(
+            new_in_other, deleted_in_other)
+        self.assertEqual(
+            ([('AAAAA',)], # new
+             [('DDDDD',), ('BBBBBxx',)], # deleted
+             [(('BBBBBx',), ('BBBBB',))]), # edits
+            result)
+
+
+class TestChangeLogMerger(tests.TestCaseWithTransport):
+    """Tests for ChangeLogMerger class.
+    
+    Most tests should be unit tests for merge_entries (and its helpers).
+    This class is just to cover the handful of lines of code in ChangeLogMerger
+    itself.
+    """
+
+    def make_builder(self):
+        builder = test_merge_core.MergeBuilder(self.test_base_dir)
+        self.addCleanup(builder.cleanup)
+        return builder
+
+    def make_changelog_merger(self, base_text, this_text, other_text):
+        builder = self.make_builder()
+        builder.add_file('clog-id', builder.tree_root, 'ChangeLog',
+            base_text, True)
+        builder.change_contents('clog-id', other=other_text, this=this_text)
+        merger = builder.make_merger(merge.Merge3Merger, ['clog-id'])
+        merger.this_branch.get_config().set_user_option(
+            'changelog_merge_files', 'ChangeLog')
+        merge_hook_params = merge.MergeHookParams(merger, 'clog-id', None,
+            'file', 'file', 'conflict')
+        changelog_merger = changelog_merge.ChangeLogMerger(merger)
+        return changelog_merger, merge_hook_params
+
+    def test_merge_text_returns_not_applicable(self):
+        """A conflict this plugin cannot resolve returns (not_applicable, None).
+        """
+        # Build same example as TestMergeCoreLogic.test_too_hard: edit an entry
+        # in other but delete it in this.
+        def entries_as_str(entries):
+            return ''.join(entry + '\n' for entry in entries)
+        changelog_merger, merge_hook_params = self.make_changelog_merger(
+            entries_as_str(sample2_base_entries),
+            '',
+            entries_as_str(sample2_other_entries))
+        self.assertEqual(
+            ('not_applicable', None),
+            changelog_merger.merge_contents(merge_hook_params))
+
+    def test_merge_text_returns_success(self):
+        """A successful merge returns ('success', lines)."""
+        changelog_merger, merge_hook_params = self.make_changelog_merger(
+            '', 'this text\n', 'other text\n')
+        status, lines = changelog_merger.merge_contents(merge_hook_params)
+        self.assertEqual(
+            ('success', ['other text\n', 'this text\n']),
+            (status, list(lines)))
+

=== modified file 'bzrlib/plugins/launchpad/__init__.py'
--- bzrlib/plugins/launchpad/__init__.py	2010-05-21 04:27:11 +0000
+++ bzrlib/plugins/launchpad/__init__.py	2011-09-20 10:19:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,33 +14,56 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Launchpad.net integration plugin for Bazaar."""
+"""Launchpad.net integration plugin for Bazaar.
+
+This plugin provides facilities for working with Bazaar branches that are
+hosted on Launchpad (http://launchpad.net).  It provides a directory service 
+for referring to Launchpad branches using the "lp:" prefix.  For example,
+lp:bzr refers to the Bazaar's main development branch and
+lp:~username/project/branch-name can be used to refer to a specific branch.
+
+This plugin provides a bug tracker so that "bzr commit --fixes lp:1234" will
+record that revision as fixing Launchpad's bug 1234.
+
+The plugin also provides the following commands:
+
+    launchpad-login: Show or set the Launchpad user ID
+    launchpad-open: Open a Launchpad branch page in your web browser
+    lp-propose-merge: Propose merging a branch on Launchpad
+    register-branch: Register a branch with launchpad.net
+    launchpad-mirror: Ask Launchpad to mirror a branch now
+
+"""
 
 # The XMLRPC server address can be overridden by setting the environment
 # variable $BZR_LP_XMLRPC_URL
 
-# see http://bazaar-vcs.org/Specs/BranchRegistrationTool
-
-# Since we are a built-in plugin we share the bzrlib version
-from bzrlib import version_info
+# see http://wiki.bazaar.canonical.com/Specs/BranchRegistrationTool
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
-    branch as _mod_branch,
+    ui,
     trace,
     )
+from bzrlib.i18n import gettext
 """)
 
-from bzrlib import bzrdir
+from bzrlib import (
+    branch as _mod_branch,
+    bzrdir,
+    lazy_regex,
+    # Since we are a built-in plugin we share the bzrlib version
+    version_info,
+    )
 from bzrlib.commands import (
-        Command,
-        register_command,
-)
+    Command,
+    register_command,
+    )
 from bzrlib.directory_service import directories
 from bzrlib.errors import (
     BzrCommandError,
-    DependencyNotPresent,
+    InvalidRevisionSpec,
     InvalidURL,
     NoPublicBranch,
     NotBranchError,
@@ -81,7 +104,7 @@
                 'Launchpad project short name to associate with the branch.',
                 unicode),
          Option('product',
-                'Launchpad product short name to associate with the branch.', 
+                'Launchpad product short name to associate with the branch.',
                 unicode,
                 hidden=True),
          Option('branch-name',
@@ -122,14 +145,16 @@
             try:
                 b = _mod_branch.Branch.open_containing('.')[0]
             except NotBranchError:
-                raise BzrCommandError('register-branch requires a public '
-                    'branch url - see bzr help register-branch.')
+                raise BzrCommandError(gettext(
+                            'register-branch requires a public '
+                            'branch url - see bzr help register-branch.'))
             public_url = b.get_public_branch()
             if public_url is None:
                 raise NoPublicBranch(b)
         if product is not None:
             project = product
-            trace.note('--product is deprecated; please use --project.')
+            trace.note(gettext(
+                '--product is deprecated; please use --project.'))
 
 
         rego = BranchRegistrationRequest(branch_url=public_url,
@@ -201,7 +226,7 @@
         if location is None:
             location = u'.'
         web_url = self._get_web_url(LaunchpadService(), location)
-        trace.note('Opening %s in web browser' % web_url)
+        trace.note(gettext('Opening %s in web browser') % web_url)
         if not dry_run:
             import webbrowser   # this import should not be lazy
                                 # otherwise bzr.exe lacks this module
@@ -245,22 +270,23 @@
                 if check_account:
                     account.check_lp_login(username)
                     if verbose:
-                        self.outf.write(
-                            "Launchpad user ID exists and has SSH keys.\n")
+                        self.outf.write(gettext(
+                            "Launchpad user ID exists and has SSH keys.\n"))
                 self.outf.write(username + '\n')
             else:
-                self.outf.write('No Launchpad user ID configured.\n')
+                self.outf.write(gettext('No Launchpad user ID configured.\n'))
                 return 1
         else:
             name = name.lower()
             if check_account:
                 account.check_lp_login(name)
                 if verbose:
-                    self.outf.write(
-                        "Launchpad user ID exists and has SSH keys.\n")
+                    self.outf.write(gettext(
+                        "Launchpad user ID exists and has SSH keys.\n"))
             account.set_lp_login(name)
             if verbose:
-                self.outf.write("Launchpad user ID set to '%s'.\n" % (name,))
+                self.outf.write(gettext("Launchpad user ID set to '%s'.\n") %
+                                                                        (name,))
 
 register_command(cmd_launchpad_login)
 
@@ -275,11 +301,12 @@
     def run(self, location='.'):
         from bzrlib.plugins.launchpad import lp_api
         from bzrlib.plugins.launchpad.lp_registration import LaunchpadService
-        branch = _mod_branch.Branch.open(location)
+        branch, _ = _mod_branch.Branch.open_containing(location)
         service = LaunchpadService()
         launchpad = lp_api.login(service)
-        lp_branch = lp_api.load_branch(launchpad, branch)
-        lp_branch.requestMirror()
+        lp_branch = lp_api.LaunchpadBranch.from_bzr(launchpad, branch,
+                create_missing=False)
+        lp_branch.lp.requestMirror()
 
 
 register_command(cmd_launchpad_mirror)
@@ -348,18 +375,163 @@
 register_command(cmd_lp_propose_merge)
 
 
+class cmd_lp_find_proposal(Command):
+
+    __doc__ = """Find the proposal to merge this revision.
+
+    Finds the merge proposal(s) that discussed landing the specified revision.
+    This works only if the selected branch was the merge proposal target, and
+    if the merged_revno is recorded for the merge proposal.  The proposal(s)
+    are opened in a web browser.
+
+    Any revision involved in the merge may be specified-- the revision in
+    which the merge was performed, or one of the revisions that was merged.
+
+    So, to find the merge proposal that reviewed line 1 of README::
+
+      bzr lp-find-proposal -r annotate:README:1
+    """
+
+    takes_options = ['revision']
+
+    def run(self, revision=None):
+        from bzrlib.plugins.launchpad import lp_api
+        import webbrowser
+        b = _mod_branch.Branch.open_containing('.')[0]
+        pb = ui.ui_factory.nested_progress_bar()
+        b.lock_read()
+        try:
+            revno = self._find_merged_revno(revision, b, pb)
+            merged = self._find_proposals(revno, b, pb)
+            if len(merged) == 0:
+                raise BzrCommandError(gettext('No review found.'))
+            trace.note(gettext('%d proposals(s) found.') % len(merged))
+            for mp in merged:
+                webbrowser.open(lp_api.canonical_url(mp))
+        finally:
+            b.unlock()
+            pb.finished()
+
+    def _find_merged_revno(self, revision, b, pb):
+        if revision is None:
+            return b.revno()
+        pb.update(gettext('Finding revision-id'))
+        revision_id = revision[0].as_revision_id(b)
+        # a revno spec is necessarily on the mainline.
+        if self._is_revno_spec(revision[0]):
+            merging_revision = revision_id
+        else:
+            graph = b.repository.get_graph()
+            pb.update(gettext('Finding merge'))
+            merging_revision = graph.find_lefthand_merger(
+                revision_id, b.last_revision())
+            if merging_revision is None:
+                raise InvalidRevisionSpec(revision[0].user_spec, b)
+        pb.update(gettext('Finding revno'))
+        return b.revision_id_to_revno(merging_revision)
+
+    def _find_proposals(self, revno, b, pb):
+        launchpad = lp_api.login(lp_registration.LaunchpadService())
+        pb.update(gettext('Finding Launchpad branch'))
+        lpb = lp_api.LaunchpadBranch.from_bzr(launchpad, b,
+                                              create_missing=False)
+        pb.update(gettext('Finding proposals'))
+        return list(lpb.lp.getMergeProposals(status=['Merged'],
+                                             merged_revnos=[revno]))
+
+
+    @staticmethod
+    def _is_revno_spec(spec):
+        try:
+            int(spec.user_spec)
+        except ValueError:
+            return False
+        else:
+            return True
+
+
+register_command(cmd_lp_find_proposal)
+
+
 def _register_directory():
     directories.register_lazy('lp:', 'bzrlib.plugins.launchpad.lp_directory',
                               'LaunchpadDirectory',
                               'Launchpad-based directory service',)
+    directories.register_lazy(
+        'debianlp:', 'bzrlib.plugins.launchpad.lp_directory',
+        'LaunchpadDirectory',
+        'debianlp: shortcut')
+    directories.register_lazy(
+        'ubuntu:', 'bzrlib.plugins.launchpad.lp_directory',
+        'LaunchpadDirectory',
+        'ubuntu: shortcut')
+
 _register_directory()
 
+# This is kept in __init__ so that we don't load lp_api_lite unless the branch
+# actually matches. That way we can avoid importing extra dependencies like
+# json.
+_package_branch = lazy_regex.lazy_compile(
+    r'bazaar.launchpad.net.*?/'
+    r'(?P<user>~[^/]+/)?(?P<archive>ubuntu|debian)/(?P<series>[^/]+/)?'
+    r'(?P<project>[^/]+)(?P<branch>/[^/]+)?'
+    )
+
+def _get_package_branch_info(url):
+    """Determine the packaging information for this URL.
+
+    :return: If this isn't a packaging branch, return None. If it is, return
+        (archive, series, project)
+    """
+    if url is None:
+        return None
+    m = _package_branch.search(url)
+    if m is None:
+        return None
+    archive, series, project, user = m.group('archive', 'series',
+                                             'project', 'user')
+    if series is not None:
+        # series is optional, so the regex includes the extra '/', we don't
+        # want to send that on (it causes Internal Server Errors.)
+        series = series.strip('/')
+    if user is not None:
+        user = user.strip('~/')
+        if user != 'ubuntu-branches':
+            return None
+    return archive, series, project
+
+
+def _check_is_up_to_date(the_branch):
+    info = _get_package_branch_info(the_branch.base)
+    if info is None:
+        return
+    c = the_branch.get_config()
+    verbosity = c.get_user_option('launchpad.packaging_verbosity')
+    if verbosity is not None:
+        verbosity = verbosity.lower()
+    if verbosity == 'off':
+        trace.mutter('not checking %s because verbosity is turned off'
+                     % (the_branch.base,))
+        return
+    archive, series, project = info
+    from bzrlib.plugins.launchpad import lp_api_lite
+    latest_pub = lp_api_lite.LatestPublication(archive, series, project)
+    lp_api_lite.report_freshness(the_branch, verbosity, latest_pub)
+
+
+def _register_hooks():
+    _mod_branch.Branch.hooks.install_named_hook('open',
+        _check_is_up_to_date, 'package-branch-up-to-date')
+
+
+_register_hooks()
 
 def load_tests(basic_tests, module, loader):
     testmod_names = [
         'test_account',
         'test_register',
         'test_lp_api',
+        'test_lp_api_lite',
         'test_lp_directory',
         'test_lp_login',
         'test_lp_open',

=== modified file 'bzrlib/plugins/launchpad/account.py'
--- bzrlib/plugins/launchpad/account.py	2010-06-20 11:18:38 +0000
+++ bzrlib/plugins/launchpad/account.py	2011-09-20 10:47:22 +0000
@@ -26,7 +26,7 @@
     transport,
     )
 from bzrlib.config import AuthenticationConfig, GlobalConfig
-
+from bzrlib.i18n import gettext
 
 LAUNCHPAD_BASE = 'https://launchpad.net/'
 
@@ -61,7 +61,7 @@
         auth_username = _get_auth_user(auth)
         # Auto-upgrading
         if auth_username is None:
-            trace.note('Setting ssh/sftp usernames for launchpad.net.')
+            trace.note(gettext('Setting ssh/sftp usernames for launchpad.net.'))
             _set_auth_user(username, auth)
         elif auth_username != username:
             raise MismatchedUsernames()
@@ -100,7 +100,7 @@
     uploaded SSH keys.
     """
     if _transport is None:
-        _transport = transport.get_transport(LAUNCHPAD_BASE)
+        _transport = transport.get_transport_from_url(LAUNCHPAD_BASE)
 
     try:
         data = _transport.get_bytes('~%s/+sshkeys' % username)

=== modified file 'bzrlib/plugins/launchpad/lp_api.py'
--- bzrlib/plugins/launchpad/lp_api.py	2010-03-05 08:55:12 +0000
+++ bzrlib/plugins/launchpad/lp_api.py	2011-09-20 10:19:58 +0000
@@ -23,6 +23,7 @@
 
 import os
 import re
+import urlparse
 
 from bzrlib import (
     branch,
@@ -32,9 +33,9 @@
     trace,
     transport,
     )
+from bzrlib.i18n import gettext
 from bzrlib.plugins.launchpad.lp_registration import (
     InvalidLaunchpadInstance,
-    NotLaunchpadBranch,
     )
 
 try:
@@ -43,7 +44,6 @@
     raise errors.DependencyNotPresent('launchpadlib', e)
 
 from launchpadlib.launchpad import (
-    EDGE_SERVICE_ROOT,
     STAGING_SERVICE_ROOT,
     Launchpad,
     )
@@ -74,19 +74,35 @@
             installed_version, installed_version)
 
 
+# The older versions of launchpadlib only provided service root constants for
+# edge and staging, whilst newer versions drop edge. Therefore service root
+# URIs for which we do not always have constants are derived from the staging
+# one, which does always exist.
+#
+# It is necessary to derive, rather than use hardcoded URIs because
+# launchpadlib <= 1.5.4 requires service root URIs that end in a path of
+# /beta/, whilst launchpadlib >= 1.5.5 requires service root URIs with no path
+# info.
+#
+# Once we have a hard dependency on launchpadlib >= 1.5.4 we can replace all of
+# bzr's local knowledge of individual Launchpad instances with use of the
+# launchpadlib.uris module.
 LAUNCHPAD_API_URLS = {
-    'production': 'https://api.launchpad.net/beta/',
-    'edge': EDGE_SERVICE_ROOT,
+    'production': STAGING_SERVICE_ROOT.replace('api.staging.launchpad.net',
+        'api.launchpad.net'),
+    'qastaging': STAGING_SERVICE_ROOT.replace('api.staging.launchpad.net',
+        'api.qastaging.launchpad.net'),
     'staging': STAGING_SERVICE_ROOT,
-    'dev': 'https://api.launchpad.dev/beta/',
+    'dev': STAGING_SERVICE_ROOT.replace('api.staging.launchpad.net',
+        'api.launchpad.dev'),
     }
 
 
 def _get_api_url(service):
     """Return the root URL of the Launchpad API.
 
-    e.g. For the 'edge' Launchpad service, this function returns
-    launchpadlib.launchpad.EDGE_SERVICE_ROOT.
+    e.g. For the 'staging' Launchpad service, this function returns
+    launchpadlib.launchpad.STAGING_SERVICE_ROOT.
 
     :param service: A `LaunchpadService` object.
     :return: A URL as a string.
@@ -101,6 +117,13 @@
         raise InvalidLaunchpadInstance(lp_instance)
 
 
+class NoLaunchpadBranch(errors.BzrError):
+    _fmt = 'No launchpad branch could be found for branch "%(url)s".'
+
+    def __init__(self, branch):
+        errors.BzrError.__init__(self, branch=branch, url=branch.base)
+
+
 def login(service, timeout=None, proxy_info=None):
     """Log in to the Launchpad API.
 
@@ -174,20 +197,24 @@
         url = bzr_branch.get_push_location()
         if url is not None:
             yield url
+        url = bzr_branch.get_parent()
+        if url is not None:
+            yield url
         yield bzr_branch.base
 
     @staticmethod
     def tweak_url(url, launchpad):
         """Adjust a URL to work with staging, if needed."""
-        if str(launchpad._root_uri) != STAGING_SERVICE_ROOT:
-            return url
-        if url is None:
-            return None
-        return url.replace('bazaar.launchpad.net',
-                           'bazaar.staging.launchpad.net')
+        if str(launchpad._root_uri) == STAGING_SERVICE_ROOT:
+            return url.replace('bazaar.launchpad.net',
+                               'bazaar.staging.launchpad.net')
+        elif str(launchpad._root_uri) == LAUNCHPAD_API_URLS['qastaging']:
+            return url.replace('bazaar.launchpad.net',
+                               'bazaar.qastaging.launchpad.net')
+        return url
 
     @classmethod
-    def from_bzr(cls, launchpad, bzr_branch):
+    def from_bzr(cls, launchpad, bzr_branch, create_missing=True):
         """Find a Launchpad branch from a bzr branch."""
         check_update = True
         for url in cls.candidate_urls(bzr_branch):
@@ -198,6 +225,8 @@
             if lp_branch is not None:
                 break
         else:
+            if not create_missing:
+                raise NoLaunchpadBranch(bzr_branch)
             lp_branch = cls.create_now(launchpad, bzr_branch)
             check_update = False
         return cls(lp_branch, bzr_branch.base, bzr_branch, check_update)
@@ -207,25 +236,38 @@
         """Create a Bazaar branch on Launchpad for the supplied branch."""
         url = cls.tweak_url(bzr_branch.get_push_location(), launchpad)
         if not cls.plausible_launchpad_url(url):
-            raise errors.BzrError('%s is not registered on Launchpad' %
+            raise errors.BzrError(gettext('%s is not registered on Launchpad') %
                                   bzr_branch.base)
         bzr_branch.create_clone_on_transport(transport.get_transport(url))
         lp_branch = launchpad.branches.getByUrl(url=url)
         if lp_branch is None:
-            raise errors.BzrError('%s is not registered on Launchpad' % url)
+            raise errors.BzrError(gettext('%s is not registered on Launchpad') %
+                                                                            url)
         return lp_branch
 
-    def get_dev_focus(self):
-        """Return the 'LaunchpadBranch' for the dev focus of this one."""
+    def get_target(self):
+        """Return the 'LaunchpadBranch' for the target of this one."""
         lp_branch = self.lp
-        if lp_branch.project is None:
-            raise errors.BzrError('%s has no product.' %
-                                  lp_branch.bzr_identity)
-        dev_focus = lp_branch.project.development_focus.branch
-        if dev_focus is None:
-            raise errors.BzrError('%s has no development focus.' %
-                                  lp_branch.bzr_identity)
-        return LaunchpadBranch(dev_focus, dev_focus.bzr_identity)
+        if lp_branch.project is not None:
+            dev_focus = lp_branch.project.development_focus
+            if dev_focus is None:
+                raise errors.BzrError(gettext('%s has no development focus.') %
+                                  lp_branch.bzr_identity)
+            target = dev_focus.branch
+            if target is None:
+                raise errors.BzrError(gettext(
+                        'development focus %s has no branch.') % dev_focus)
+        elif lp_branch.sourcepackage is not None:
+            target = lp_branch.sourcepackage.getBranch(pocket="Release")
+            if target is None:
+                raise errors.BzrError(gettext(
+                                      'source package %s has no branch.') %
+                                      lp_branch.sourcepackage)
+        else:
+            raise errors.BzrError(gettext(
+                        '%s has no associated product or source package.') %
+                                  lp_branch.bzr_identity)
+        return LaunchpadBranch(target, target.bzr_identity)
 
     def update_lp(self):
         """Update the Launchpad copy of this branch."""
@@ -235,14 +277,14 @@
         try:
             if self.lp.last_scanned_id is not None:
                 if self.bzr.last_revision() == self.lp.last_scanned_id:
-                    trace.note('%s is already up-to-date.' %
+                    trace.note(gettext('%s is already up-to-date.') %
                                self.lp.bzr_identity)
                     return
                 graph = self.bzr.repository.get_graph()
                 if not graph.is_ancestor(self.lp.last_scanned_id,
                                          self.bzr.last_revision()):
                     raise errors.DivergedBranches(self.bzr, self.push_bzr)
-                trace.note('Pushing to %s' % self.lp.bzr_identity)
+                trace.note(gettext('Pushing to %s') % self.lp.bzr_identity)
             self.bzr.push(self.push_bzr)
         finally:
             self.bzr.unlock()
@@ -259,24 +301,11 @@
         return self.bzr.repository.revision_tree(lca)
 
 
-def load_branch(launchpad, branch):
-    """Return the launchpadlib Branch object corresponding to 'branch'.
-
-    :param launchpad: The root `Launchpad` object from launchpadlib.
-    :param branch: A `bzrlib.branch.Branch`.
-    :raise NotLaunchpadBranch: If we cannot determine the Launchpad URL of
-        `branch`.
-    :return: A launchpadlib Branch object.
-    """
-    # XXX: This duplicates the "What are possible URLs for the branch that
-    # Launchpad might recognize" logic found in cmd_lp_open.
-
-    # XXX: This makes multiple roundtrips to Launchpad for what is
-    # conceptually a single operation -- get me the branches that match these
-    # URLs. Unfortunately, Launchpad's support for such operations is poor, so
-    # we have to allow multiple roundtrips.
-    for url in branch.get_public_branch(), branch.get_push_location():
-        lp_branch = launchpad.branches.getByUrl(url=url)
-        if lp_branch:
-            return lp_branch
-    raise NotLaunchpadBranch(url)
+def canonical_url(object):
+    """Return the canonical URL for a branch."""
+    scheme, netloc, path, params, query, fragment = urlparse.urlparse(
+        str(object.self_link))
+    path = '/'.join(path.split('/')[2:])
+    netloc = netloc.replace('api.', 'code.')
+    return urlparse.urlunparse((scheme, netloc, path, params, query,
+                                fragment))

=== added file 'bzrlib/plugins/launchpad/lp_api_lite.py'
--- bzrlib/plugins/launchpad/lp_api_lite.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/launchpad/lp_api_lite.py	2011-08-21 09:24:01 +0000
@@ -0,0 +1,286 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tools for dealing with the Launchpad API without using launchpadlib.
+
+The api itself is a RESTful interface, so we can make HTTP queries directly.
+loading launchpadlib itself has a fairly high overhead (just calling
+Launchpad.login_anonymously() takes a 500ms once the WADL is cached, and 5+s to
+get the WADL.
+"""
+
+try:
+    # Use simplejson if available, much faster, and can be easily installed in
+    # older versions of python
+    import simplejson as json
+except ImportError:
+    # Is present since python 2.6
+    try:
+        import json
+    except ImportError:
+        json = None
+
+import time
+import urllib
+import urllib2
+
+from bzrlib import (
+    revision,
+    trace,
+    )
+
+
+class LatestPublication(object):
+    """Encapsulate how to find the latest publication for a given project."""
+
+    LP_API_ROOT = 'https://api.launchpad.net/1.0'
+
+    def __init__(self, archive, series, project):
+        self._archive = archive
+        self._project = project
+        self._setup_series_and_pocket(series)
+
+    def _setup_series_and_pocket(self, series):
+        """Parse the 'series' info into a series and a pocket.
+
+        eg::
+            _setup_series_and_pocket('natty-proposed')
+            => _series == 'natty'
+               _pocket == 'Proposed'
+        """
+        self._series = series
+        self._pocket = None
+        if self._series is not None and '-' in self._series:
+            self._series, self._pocket = self._series.split('-', 1)
+            self._pocket = self._pocket.title()
+        else:
+            self._pocket = 'Release'
+
+    def _archive_URL(self):
+        """Return the Launchpad 'Archive' URL that we will query.
+        This is everything in the URL except the query parameters.
+        """
+        return '%s/%s/+archive/primary' % (self.LP_API_ROOT, self._archive)
+
+    def _publication_status(self):
+        """Handle the 'status' field.
+        It seems that Launchpad tracks all 'debian' packages as 'Pending', while
+        for 'ubuntu' we care about the 'Published' packages.
+        """
+        if self._archive == 'debian':
+            # Launchpad only tracks debian packages as "Pending", it doesn't mark
+            # them Published
+            return 'Pending'
+        return 'Published'
+
+    def _query_params(self):
+        """Get the parameters defining our query.
+        This defines the actions we are making against the archive.
+        :return: A dict of query parameters.
+        """
+        params = {'ws.op': 'getPublishedSources',
+                  'exact_match': 'true',
+                  # If we need to use "" shouldn't we quote the project somehow?
+                  'source_name': '"%s"' % (self._project,),
+                  'status': self._publication_status(),
+                  # We only need the latest one, the results seem to be properly
+                  # most-recent-debian-version sorted
+                  'ws.size': '1',
+        }
+        if self._series is not None:
+            params['distro_series'] = '/%s/%s' % (self._archive, self._series)
+        if self._pocket is not None:
+            params['pocket'] = self._pocket
+        return params
+
+    def _query_URL(self):
+        """Create the full URL that we need to query, including parameters."""
+        params = self._query_params()
+        # We sort to give deterministic results for testing
+        encoded = urllib.urlencode(sorted(params.items()))
+        return '%s?%s' % (self._archive_URL(), encoded)
+
+    def _get_lp_info(self):
+        """Place an actual HTTP query against the Launchpad service."""
+        if json is None:
+            return None
+        query_URL = self._query_URL()
+        try:
+            req = urllib2.Request(query_URL)
+            response = urllib2.urlopen(req)
+            json_info = response.read()
+        # TODO: We haven't tested the HTTPError
+        except (urllib2.URLError, urllib2.HTTPError), e:
+            trace.mutter('failed to place query to %r' % (query_URL,))
+            trace.log_exception_quietly()
+            return None
+        return json_info
+
+    def _parse_json_info(self, json_info):
+        """Parse the json response from Launchpad into objects."""
+        if json is None:
+            return None
+        try:
+            return json.loads(json_info)
+        except Exception:
+            trace.mutter('Failed to parse json info: %r' % (json_info,))
+            trace.log_exception_quietly()
+            return None
+
+    def get_latest_version(self):
+        """Get the latest published version for the given package."""
+        json_info = self._get_lp_info()
+        if json_info is None:
+            return None
+        info = self._parse_json_info(json_info)
+        if info is None:
+            return None
+        try:
+            entries = info['entries']
+            if len(entries) == 0:
+                return None
+            return entries[0]['source_package_version']
+        except KeyError:
+            trace.log_exception_quietly()
+            return None
+
+    def place(self):
+        """Text-form for what location this represents.
+
+        Example::
+            ubuntu, natty => Ubuntu Natty
+            ubuntu, natty-proposed => Ubuntu Natty Proposed
+        :return: A string representing the location we are checking.
+        """
+        place = self._archive
+        if self._series is not None:
+            place = '%s %s' % (place, self._series)
+        if self._pocket is not None and self._pocket != 'Release':
+            place = '%s %s' % (place, self._pocket)
+        return place.title()
+
+
+def get_latest_publication(archive, series, project):
+    """Get the most recent publication for a given project.
+
+    :param archive: Either 'ubuntu' or 'debian'
+    :param series: Something like 'natty', 'sid', etc. Can be set as None. Can
+        also include a pocket such as 'natty-proposed'.
+    :param project: Something like 'bzr'
+    :return: A version string indicating the most-recent version published in
+        Launchpad. Might return None if there is an error.
+    """
+    lp = LatestPublication(archive, series, project)
+    return lp.get_latest_version()
+
+
+def get_most_recent_tag(tag_dict, the_branch):
+    """Get the most recent revision that has been tagged."""
+    # Note: this assumes that a given rev won't get tagged multiple times. But
+    #       it should be valid for the package importer branches that we care
+    #       about
+    reverse_dict = dict((rev, tag) for tag, rev in tag_dict.iteritems())
+    the_branch.lock_read()
+    try:
+        last_rev = the_branch.last_revision()
+        graph = the_branch.repository.get_graph()
+        stop_revisions = (None, revision.NULL_REVISION)
+        for rev_id in graph.iter_lefthand_ancestry(last_rev, stop_revisions):
+            if rev_id in reverse_dict:
+                return reverse_dict[rev_id]
+    finally:
+        the_branch.unlock()
+
+
+def _get_newest_versions(the_branch, latest_pub):
+    """Get information about how 'fresh' this packaging branch is.
+
+    :param the_branch: The Branch to check
+    :param latest_pub: The LatestPublication used to check most recent
+        published version.
+    :return: (latest_ver, branch_latest_ver)
+    """
+    t = time.time()
+    latest_ver = latest_pub.get_latest_version()
+    t_latest_ver = time.time() - t
+    trace.mutter('LatestPublication.get_latest_version took: %.3fs'
+                 % (t_latest_ver,))
+    if latest_ver is None:
+        return None, None
+    t = time.time()
+    tags = the_branch.tags.get_tag_dict()
+    t_tag_dict = time.time() - t
+    trace.mutter('LatestPublication.get_tag_dict took: %.3fs' % (t_tag_dict,))
+    if latest_ver in tags:
+        # branch might have a newer tag, but we don't really care
+        return latest_ver, latest_ver
+    else:
+        best_tag = get_most_recent_tag(tags, the_branch)
+        return latest_ver, best_tag
+
+
+def _report_freshness(latest_ver, branch_latest_ver, place, verbosity,
+                      report_func):
+    """Report if the branch is up-to-date."""
+    if latest_ver is None:
+        if verbosity == 'all':
+            report_func('Most recent %s version: MISSING' % (place,))
+        elif verbosity == 'short':
+            report_func('%s is MISSING a version' % (place,))
+        return
+    elif latest_ver == branch_latest_ver:
+        if verbosity == 'minimal':
+            return
+        elif verbosity == 'short':
+            report_func('%s is CURRENT in %s' % (latest_ver, place))
+        else:
+            report_func('Most recent %s version: %s\n'
+                       'Packaging branch status: CURRENT'
+                       % (place, latest_ver))
+    else:
+        if verbosity in ('minimal', 'short'):
+            if branch_latest_ver is None:
+                branch_latest_ver = 'Branch'
+            report_func('%s is OUT-OF-DATE, %s has %s'
+                        % (branch_latest_ver, place, latest_ver))
+        else:
+            report_func('Most recent %s version: %s\n'
+                        'Packaging branch version: %s\n'
+                        'Packaging branch status: OUT-OF-DATE'
+                        % (place, latest_ver, branch_latest_ver))
+
+
+def report_freshness(the_branch, verbosity, latest_pub):
+    """Report to the user how up-to-date the packaging branch is.
+
+    :param the_branch: A Branch object
+    :param verbosity: Can be one of:
+        off: Do not print anything, and skip all checks.
+        all: Print all information that we have in a verbose manner, this
+             includes misses, etc.
+        short: Print information, but only one-line summaries
+        minimal: Only print a one-line summary when the package branch is
+                 out-of-date
+    :param latest_pub: A LatestPublication instance
+    """
+    if verbosity == 'off':
+        return
+    if verbosity is None:
+        verbosity = 'all'
+    latest_ver, branch_ver = _get_newest_versions(the_branch, latest_pub)
+    place = latest_pub.place()
+    _report_freshness(latest_ver, branch_ver, place, verbosity,
+                      trace.note)

=== modified file 'bzrlib/plugins/launchpad/lp_directory.py'
--- bzrlib/plugins/launchpad/lp_directory.py	2010-07-26 17:19:33 +0000
+++ bzrlib/plugins/launchpad/lp_directory.py	2011-10-06 06:45:46 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,11 +24,9 @@
     debug,
     errors,
     trace,
-    )
-from bzrlib.transport import (
-    get_transport,
-    register_urlparse_netloc_protocol,
-    )
+    transport,
+    )
+from bzrlib.i18n import gettext
 
 from bzrlib.plugins.launchpad.lp_registration import (
     LaunchpadService, ResolveLaunchpadPathRequest)
@@ -37,8 +35,18 @@
 
 # As bzrlib.transport.remote may not be loaded yet, make sure bzr+ssh
 # is counted as a netloc protocol.
-register_urlparse_netloc_protocol('bzr+ssh')
-register_urlparse_netloc_protocol('lp')
+transport.register_urlparse_netloc_protocol('bzr+ssh')
+transport.register_urlparse_netloc_protocol('lp')
+
+_ubuntu_series_shortcuts = {
+    'n': 'natty',
+    'm': 'maverick',
+    'l': 'lucid',
+    'k': 'karmic',
+    'j': 'jaunty',
+    'h': 'hardy',
+    'd': 'dapper',
+    }
 
 
 class LaunchpadDirectory(object):
@@ -58,27 +66,110 @@
         """See DirectoryService.look_up"""
         return self._resolve(url)
 
-    def _resolve(self, url,
-                 _request_factory=ResolveLaunchpadPathRequest,
-                 _lp_login=None):
-        """Resolve the base URL for this transport."""
+    def _resolve_locally(self, path, url, _request_factory):
+        # This is the best I could work out about XMLRPC. If an lp: url
+        # includes ~user, then it is specially validated. Otherwise, it is just
+        # sent to +branch/$path.
+        _, netloc, _, _, _ = urlsplit(url)
+        if netloc == '':
+            netloc = LaunchpadService.DEFAULT_INSTANCE
+        base_url = LaunchpadService.LAUNCHPAD_DOMAINS[netloc]
+        base = 'bzr+ssh://bazaar.%s/' % (base_url,)
+        maybe_invalid = False
+        if path.startswith('~'):
+            # A ~user style path, validate it a bit.
+            # If a path looks fishy, fall back to asking XMLRPC to
+            # resolve it for us. That way we still get their nicer error
+            # messages.
+            parts = path.split('/')
+            if (len(parts) < 3
+                or (parts[1] in ('ubuntu', 'debian') and len(parts) < 5)):
+                # This special case requires 5-parts to be valid.
+                maybe_invalid = True
+        else:
+            base += '+branch/'
+        if maybe_invalid:
+            return self._resolve_via_xmlrpc(path, url, _request_factory)
+        return {'urls': [base + path]}
+
+    def _resolve_via_xmlrpc(self, path, url, _request_factory):
         service = LaunchpadService.for_url(url)
-        result = urlsplit(url)
-        if _lp_login is None:
-            _lp_login = get_lp_login()
-        path = result[2].strip('/')
-        if path.startswith('~/'):
-            if _lp_login is None:
-                raise errors.InvalidURL(path=url,
-                    extra='Cannot resolve "~" to your username.'
-                          ' See "bzr help launchpad-login"')
-            path = '~' + _lp_login + path[1:]
         resolve = _request_factory(path)
         try:
             result = resolve.submit(service)
         except xmlrpclib.Fault, fault:
             raise errors.InvalidURL(
                 path=url, extra=fault.faultString)
+        return result
+
+    def _update_url_scheme(self, url):
+        # Do ubuntu: and debianlp: expansions.
+        scheme, netloc, path, query, fragment = urlsplit(url)
+        if scheme in ('ubuntu', 'debianlp'):
+            if scheme == 'ubuntu':
+                distro = 'ubuntu'
+                distro_series = _ubuntu_series_shortcuts
+            elif scheme == 'debianlp':
+                distro = 'debian'
+                # No shortcuts for Debian distroseries.
+                distro_series = {}
+            else:
+                raise AssertionError('scheme should be ubuntu: or debianlp:')
+            # Split the path.  It's either going to be 'project' or
+            # 'series/project', but recognize that it may be a series we don't
+            # know about.
+            path_parts = path.split('/')
+            if len(path_parts) == 1:
+                # It's just a project name.
+                lp_url_template = 'lp:%(distro)s/%(project)s'
+                project = path_parts[0]
+                series = None
+            elif len(path_parts) == 2:
+                # It's a series and project.
+                lp_url_template = 'lp:%(distro)s/%(series)s/%(project)s'
+                series, project = path_parts
+            else:
+                # There are either 0 or > 2 path parts, neither of which is
+                # supported for these schemes.
+                raise errors.InvalidURL('Bad path: %s' % url)
+            # Expand any series shortcuts, but keep unknown series.
+            series = distro_series.get(series, series)
+            # Hack the url and let the following do the final resolution.
+            url = lp_url_template % dict(
+                distro=distro,
+                series=series,
+                project=project)
+            scheme, netloc, path, query, fragment = urlsplit(url)
+        return url, path
+
+    def _expand_user(self, path, url, lp_login):
+        if path.startswith('~/'):
+            if lp_login is None:
+                raise errors.InvalidURL(path=url,
+                    extra='Cannot resolve "~" to your username.'
+                          ' See "bzr help launchpad-login"')
+            path = '~' + lp_login + path[1:]
+        return path
+
+    def _resolve(self, url,
+                 _request_factory=ResolveLaunchpadPathRequest,
+                 _lp_login=None):
+        """Resolve the base URL for this transport."""
+        url, path = self._update_url_scheme(url)
+        if _lp_login is None:
+            _lp_login = get_lp_login()
+        path = path.strip('/')
+        path = self._expand_user(path, url, _lp_login)
+        if _lp_login is not None:
+            result = self._resolve_locally(path, url, _request_factory)
+            if 'launchpad' in debug.debug_flags:
+                local_res = result
+                result = self._resolve_via_xmlrpc(path, url, _request_factory)
+                trace.note(gettext(
+                    'resolution for {0}\n  local: {1}\n remote: {2}').format(
+                           url, local_res['urls'], result['urls']))
+        else:
+            result = self._resolve_via_xmlrpc(path, url, _request_factory)
 
         if 'launchpad' in debug.debug_flags:
             trace.mutter("resolve_lp_path(%r) == %r", url, result)
@@ -101,7 +192,7 @@
             else:
                 # Use the URL if we can create a transport for it.
                 try:
-                    get_transport(url)
+                    transport.get_transport(url)
                 except (errors.PathError, errors.TransportError):
                     pass
                 else:

=== modified file 'bzrlib/plugins/launchpad/lp_propose.py'
--- bzrlib/plugins/launchpad/lp_propose.py	2010-05-21 11:01:38 +0000
+++ bzrlib/plugins/launchpad/lp_propose.py	2011-09-20 10:19:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,40 +14,34 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
-import urlparse
-import webbrowser
-
 from bzrlib import (
     errors,
     hooks,
+    )
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import webbrowser
+
+from bzrlib import (
     msgeditor,
-)
+    )
 from bzrlib.plugins.launchpad import (
     lp_api,
     lp_registration,
-)
-
-from lazr.restfulclient import errors as restful_errors
+    )
+""")
 
 
 class ProposeMergeHooks(hooks.Hooks):
     """Hooks for proposing a merge on Launchpad."""
 
     def __init__(self):
-        hooks.Hooks.__init__(self)
-        self.create_hook(
-            hooks.HookPoint(
-                'get_prerequisite',
-                "Return the prerequisite branch for proposing as merge.",
-                (2, 1), None),
-        )
-        self.create_hook(
-            hooks.HookPoint(
-                'merge_proposal_body',
-                "Return an initial body for the merge proposal message.",
-                (2, 1), None),
-        )
+        hooks.Hooks.__init__(self, "bzrlib.plugins.launchpad.lp_propose",
+            "Proposer.hooks")
+        self.add_hook('get_prerequisite',
+            "Return the prerequisite branch for proposing as merge.", (2, 1))
+        self.add_hook('merge_proposal_body',
+            "Return an initial body for the merge proposal message.", (2, 1))
 
 
 class Proposer(object):
@@ -74,23 +68,20 @@
         if staging:
             lp_instance = 'staging'
         else:
-            lp_instance = 'edge'
+            lp_instance = 'production'
         service = lp_registration.LaunchpadService(lp_instance=lp_instance)
         self.launchpad = lp_api.login(service)
         self.source_branch = lp_api.LaunchpadBranch.from_bzr(
             self.launchpad, source_branch)
         if target_branch is None:
-            self.target_branch = self.source_branch.get_dev_focus()
+            self.target_branch = self.source_branch.get_target()
         else:
             self.target_branch = lp_api.LaunchpadBranch.from_bzr(
                 self.launchpad, target_branch)
         self.commit_message = message
         # XXX: this is where bug lp:583638 could be tackled.
         if reviews == []:
-            target_reviewer = self.target_branch.lp.reviewer
-            if target_reviewer is None:
-                raise errors.BzrCommandError('No reviewer specified')
-            self.reviews = [(target_reviewer, '')]
+            self.reviews = []
         else:
             self.reviews = [(self.launchpad.people[reviewer], review_type)
                             for reviewer, review_type in
@@ -151,9 +142,9 @@
             if mp.queue_status in ('Merged', 'Rejected'):
                 continue
             if mp.target_branch.self_link == self.target_branch.lp.self_link:
-                raise errors.BzrCommandError(
-                    'There is already a branch merge proposal: %s' %
-                    canonical_url(mp))
+                raise errors.BzrCommandError(gettext(
+                    'There is already a branch merge proposal: %s') %
+                    lp_api.canonical_url(mp))
 
     def _get_prerequisite_branch(self):
         hooks = self.hooks['get_prerequisite']
@@ -174,6 +165,7 @@
         :param **kwargs: **kwargs for the call.
         :return: The result of calling call(*args, *kwargs).
         """
+        from lazr.restfulclient import errors as restful_errors
         try:
             return call(*args, **kwargs)
         except restful_errors.HTTPError, e:
@@ -208,7 +200,7 @@
             review_types=review_types)
         if self.approve:
             self.call_webservice(mp.setStatus, status='Approved')
-        webbrowser.open(canonical_url(mp))
+        webbrowser.open(lp_api.canonical_url(mp))
 
 
 def modified_files(old_tree, new_tree):
@@ -217,13 +209,3 @@
         old_tree):
         if c and k == 'file':
             yield str(path)
-
-
-def canonical_url(object):
-    """Return the canonical URL for a branch."""
-    scheme, netloc, path, params, query, fragment = urlparse.urlparse(
-        str(object.self_link))
-    path = '/'.join(path.split('/')[2:])
-    netloc = netloc.replace('api.', 'code.')
-    return urlparse.urlunparse((scheme, netloc, path, params, query,
-                                fragment))

=== modified file 'bzrlib/plugins/launchpad/lp_registration.py'
--- bzrlib/plugins/launchpad/lp_registration.py	2010-05-20 18:23:10 +0000
+++ bzrlib/plugins/launchpad/lp_registration.py	2011-03-24 11:41:42 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -86,8 +86,8 @@
 
     LAUNCHPAD_DOMAINS = {
         'production': 'launchpad.net',
-        'edge': 'edge.launchpad.net',
         'staging': 'staging.launchpad.net',
+        'qastaging': 'qastaging.launchpad.net',
         'demo': 'demo.launchpad.net',
         'dev': 'launchpad.dev',
         }
@@ -98,7 +98,8 @@
     for instance, domain in LAUNCHPAD_DOMAINS.iteritems():
         LAUNCHPAD_INSTANCE[instance] = 'https://xmlrpc.%s/bazaar/' % domain
 
-    # Previously 'edge' was used to avoid a launchpad bug with redirection
+    # We use production as the default because edge has been deprecated circa
+    # 2010-11 (see bug https://bugs.launchpad.net/bzr/+bug/583667)
     DEFAULT_INSTANCE = 'production'
     DEFAULT_SERVICE_URL = LAUNCHPAD_INSTANCE[DEFAULT_INSTANCE]
 

=== modified file 'bzrlib/plugins/launchpad/test_account.py'
--- bzrlib/plugins/launchpad/test_account.py	2009-03-23 14:59:43 +0000
+++ bzrlib/plugins/launchpad/test_account.py	2010-08-30 07:34:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,9 +26,7 @@
 class LaunchpadAccountTests(TestCaseInTempDir):
 
     def setup_config(self, text):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(text)
-        my_config._get_parser(config_file)
+        my_config = config.GlobalConfig.from_string(text)
         return my_config
 
     def test_get_lp_login_unconfigured(self):

=== modified file 'bzrlib/plugins/launchpad/test_lp_api.py'
--- bzrlib/plugins/launchpad/test_lp_api.py	2010-03-05 08:55:12 +0000
+++ bzrlib/plugins/launchpad/test_lp_api.py	2011-07-11 07:02:20 +0000
@@ -15,13 +15,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from bzrlib import commands, config, errors, osutils
-from bzrlib.plugins.launchpad import cmd_launchpad_mirror
+from bzrlib import config, errors, osutils
 from bzrlib.tests import (
-    ModuleAvailableFeature,
     TestCase,
     TestCaseWithTransport,
     )
+from bzrlib.tests.features import (
+    ModuleAvailableFeature,
+    )
 
 
 launchpadlib_feature = ModuleAvailableFeature('launchpadlib')

=== added file 'bzrlib/plugins/launchpad/test_lp_api_lite.py'
--- bzrlib/plugins/launchpad/test_lp_api_lite.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/launchpad/test_lp_api_lite.py	2011-08-21 09:24:01 +0000
@@ -0,0 +1,549 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tools for dealing with the Launchpad API without using launchpadlib.
+"""
+
+import doctest
+import socket
+
+from bzrlib import tests
+from bzrlib.tests import features
+from bzrlib.plugins import launchpad
+from bzrlib.plugins.launchpad import lp_api_lite
+from testtools.matchers import DocTestMatches
+
+
+class _JSONParserFeature(features.Feature):
+
+    def _probe(self):
+        return lp_api_lite.json is not None
+
+    def feature_name(self):
+        return 'simplejson or json'
+
+
+JSONParserFeature = _JSONParserFeature()
+
+
+_example_response = r"""
+{
+    "total_size": 2,
+    "start": 0,
+    "next_collection_link": "https://api.launchpad.net/1.0/ubuntu/+archive/primary?distro_series=%2Fubuntu%2Flucid&exact_match=true&source_name=%22bzr%22&status=Published&ws.op=getPublishedSources&ws.start=1&ws.size=1",
+    "entries": [
+        {
+            "package_creator_link": "https://api.launchpad.net/1.0/~maxb",
+            "package_signer_link": "https://api.launchpad.net/1.0/~jelmer",
+            "source_package_name": "bzr",
+            "removal_comment": null,
+            "display_name": "bzr 2.1.4-0ubuntu1 in lucid",
+            "date_made_pending": null,
+            "source_package_version": "2.1.4-0ubuntu1",
+            "date_superseded": null,
+            "http_etag": "\"9ba966152dec474dc0fe1629d0bbce2452efaf3b-5f4c3fbb3eaf26d502db4089777a9b6a0537ffab\"",
+            "self_link": "https://api.launchpad.net/1.0/ubuntu/+archive/primary/+sourcepub/1750327",
+            "distro_series_link": "https://api.launchpad.net/1.0/ubuntu/lucid",
+            "component_name": "main",
+            "status": "Published",
+            "date_removed": null,
+            "pocket": "Updates",
+            "date_published": "2011-05-30T06:09:58.653984+00:00",
+            "removed_by_link": null,
+            "section_name": "devel",
+            "resource_type_link": "https://api.launchpad.net/1.0/#source_package_publishing_history",
+            "archive_link": "https://api.launchpad.net/1.0/ubuntu/+archive/primary",
+            "package_maintainer_link": "https://api.launchpad.net/1.0/~ubuntu-devel-discuss-lists",
+            "date_created": "2011-05-30T05:19:12.233621+00:00",
+            "scheduled_deletion_date": null
+        }
+    ]
+}"""
+
+_no_versions_response = '{"total_size": 0, "start": 0, "entries": []}'
+
+
+class TestLatestPublication(tests.TestCase):
+
+    def make_latest_publication(self, archive='ubuntu', series='natty',
+                                project='bzr'):
+        return lp_api_lite.LatestPublication(archive, series, project)
+
+    def assertPlace(self, place, archive, series, project):
+        lp = lp_api_lite.LatestPublication(archive, series, project)
+        self.assertEqual(place, lp.place())
+
+    def test_init(self):
+        latest_pub = self.make_latest_publication()
+        self.assertEqual('ubuntu', latest_pub._archive)
+        self.assertEqual('natty', latest_pub._series)
+        self.assertEqual('bzr', latest_pub._project)
+        self.assertEqual('Release', latest_pub._pocket)
+
+    def test__archive_URL(self):
+        latest_pub = self.make_latest_publication()
+        self.assertEqual(
+            'https://api.launchpad.net/1.0/ubuntu/+archive/primary',
+            latest_pub._archive_URL())
+
+    def test__publication_status_for_ubuntu(self):
+        latest_pub = self.make_latest_publication()
+        self.assertEqual('Published', latest_pub._publication_status())
+
+    def test__publication_status_for_debian(self):
+        latest_pub = self.make_latest_publication(archive='debian')
+        self.assertEqual('Pending', latest_pub._publication_status())
+
+    def test_pocket(self):
+        latest_pub = self.make_latest_publication(series='natty-proposed')
+        self.assertEqual('natty', latest_pub._series)
+        self.assertEqual('Proposed', latest_pub._pocket)
+
+    def test_series_None(self):
+        latest_pub = self.make_latest_publication(series=None)
+        self.assertEqual('ubuntu', latest_pub._archive)
+        self.assertEqual(None, latest_pub._series)
+        self.assertEqual('bzr', latest_pub._project)
+        self.assertEqual('Release', latest_pub._pocket)
+
+    def test__query_params(self):
+        latest_pub = self.make_latest_publication()
+        self.assertEqual({'ws.op': 'getPublishedSources',
+                          'exact_match': 'true',
+                          'source_name': '"bzr"',
+                          'status': 'Published',
+                          'ws.size': '1',
+                          'distro_series': '/ubuntu/natty',
+                          'pocket': 'Release',
+                         }, latest_pub._query_params())
+
+    def test__query_params_no_series(self):
+        latest_pub = self.make_latest_publication(series=None)
+        self.assertEqual({'ws.op': 'getPublishedSources',
+                          'exact_match': 'true',
+                          'source_name': '"bzr"',
+                          'status': 'Published',
+                          'ws.size': '1',
+                          'pocket': 'Release',
+                         }, latest_pub._query_params())
+
+    def test__query_params_pocket(self):
+        latest_pub = self.make_latest_publication(series='natty-proposed')
+        self.assertEqual({'ws.op': 'getPublishedSources',
+                          'exact_match': 'true',
+                          'source_name': '"bzr"',
+                          'status': 'Published',
+                          'ws.size': '1',
+                          'distro_series': '/ubuntu/natty',
+                          'pocket': 'Proposed',
+                         }, latest_pub._query_params())
+
+    def test__query_URL(self):
+        latest_pub = self.make_latest_publication()
+        # we explicitly sort params, so we can be sure this URL matches exactly
+        self.assertEqual(
+            'https://api.launchpad.net/1.0/ubuntu/+archive/primary'
+            '?distro_series=%2Fubuntu%2Fnatty&exact_match=true'
+            '&pocket=Release&source_name=%22bzr%22&status=Published'
+            '&ws.op=getPublishedSources&ws.size=1',
+            latest_pub._query_URL())
+
+    def DONT_test__gracefully_handle_failed_rpc_connection(self):
+        # TODO: This test kind of sucks. We intentionally create an arbitrary
+        #       port and don't listen to it, because we want the request to fail.
+        #       However, it seems to take 1s for it to timeout. Is there a way
+        #       to make it fail faster?
+        latest_pub = self.make_latest_publication()
+        s = socket.socket()
+        s.bind(('127.0.0.1', 0))
+        addr, port = s.getsockname()
+        latest_pub.LP_API_ROOT = 'http://%s:%s/' % (addr, port)
+        s.close()
+        self.assertIs(None, latest_pub._get_lp_info())
+
+    def DONT_test__query_launchpad(self):
+        # TODO: This is a test that we are making a valid request against
+        #       launchpad. This seems important, but it is slow, requires net
+        #       access, and requires launchpad to be up and running. So for
+        #       now, it is commented out for production tests.
+        latest_pub = self.make_latest_publication()
+        json_txt = latest_pub._get_lp_info()
+        self.assertIsNot(None, json_txt)
+        if lp_api_lite.json is None:
+            # We don't have a way to parse the text
+            return
+        # The content should be a valid json result
+        content = lp_api_lite.json.loads(json_txt)
+        entries = content['entries']  # It should have an 'entries' field.
+        # ws.size should mean we get 0 or 1, and there should be something
+        self.assertEqual(1, len(entries))
+        entry = entries[0]
+        self.assertEqual('bzr', entry['source_package_name'])
+        version = entry['source_package_version']
+        self.assertIsNot(None, version)
+
+    def test__get_lp_info_no_json(self):
+        # If we can't parse the json, we don't make the query.
+        self.overrideAttr(lp_api_lite, 'json', None)
+        latest_pub = self.make_latest_publication()
+        self.assertIs(None, latest_pub._get_lp_info())
+
+    def test__parse_json_info_no_module(self):
+        # If a json parsing module isn't available, we just return None here.
+        self.overrideAttr(lp_api_lite, 'json', None)
+        latest_pub = self.make_latest_publication()
+        self.assertIs(None, latest_pub._parse_json_info(_example_response))
+
+    def test__parse_json_example_response(self):
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        content = latest_pub._parse_json_info(_example_response)
+        self.assertIsNot(None, content)
+        self.assertEqual(2, content['total_size'])
+        entries = content['entries']
+        self.assertEqual(1, len(entries))
+        entry = entries[0]
+        self.assertEqual('bzr', entry['source_package_name'])
+        self.assertEqual("2.1.4-0ubuntu1", entry["source_package_version"])
+
+    def test__parse_json_not_json(self):
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        self.assertIs(None, latest_pub._parse_json_info('Not_valid_json'))
+
+    def test_get_latest_version_no_response(self):
+        latest_pub = self.make_latest_publication()
+        latest_pub._get_lp_info = lambda: None
+        self.assertEqual(None, latest_pub.get_latest_version())
+
+    def test_get_latest_version_no_json(self):
+        self.overrideAttr(lp_api_lite, 'json', None)
+        latest_pub = self.make_latest_publication()
+        self.assertEqual(None, latest_pub.get_latest_version())
+
+    def test_get_latest_version_invalid_json(self):
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        latest_pub._get_lp_info = lambda: "not json"
+        self.assertEqual(None, latest_pub.get_latest_version())
+
+    def test_get_latest_version_no_versions(self):
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        latest_pub._get_lp_info = lambda: _no_versions_response
+        self.assertEqual(None, latest_pub.get_latest_version())
+
+    def test_get_latest_version_missing_entries(self):
+        # Launchpad's no-entries response does have an empty entries value.
+        # However, lets test that we handle other failures without tracebacks
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        latest_pub._get_lp_info = lambda: '{}'
+        self.assertEqual(None, latest_pub.get_latest_version())
+
+    def test_get_latest_version_invalid_entries(self):
+        # Make sure we sanely handle a json response we don't understand
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        latest_pub._get_lp_info = lambda: '{"entries": {"a": 1}}'
+        self.assertEqual(None, latest_pub.get_latest_version())
+
+    def test_get_latest_version_example(self):
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        latest_pub._get_lp_info = lambda: _example_response
+        self.assertEqual("2.1.4-0ubuntu1", latest_pub.get_latest_version())
+
+    def DONT_test_get_latest_version_from_launchpad(self):
+        self.requireFeature(JSONParserFeature)
+        latest_pub = self.make_latest_publication()
+        self.assertIsNot(None, latest_pub.get_latest_version())
+
+    def test_place(self):
+        self.assertPlace('Ubuntu', 'ubuntu', None, 'bzr')
+        self.assertPlace('Ubuntu Natty', 'ubuntu', 'natty', 'bzr')
+        self.assertPlace('Ubuntu Natty Proposed', 'ubuntu', 'natty-proposed',
+                         'bzr')
+        self.assertPlace('Debian', 'debian', None, 'bzr')
+        self.assertPlace('Debian Sid', 'debian', 'sid', 'bzr')
+
+
+class TestIsUpToDate(tests.TestCase):
+
+    def assertPackageBranchRe(self, url, user, archive, series, project):
+        m = launchpad._package_branch.search(url)
+        if m is None:
+            self.fail('package_branch regex did not match url: %s' % (url,))
+        self.assertEqual(
+            (user, archive, series, project),
+            m.group('user', 'archive', 'series', 'project'))
+
+    def assertNotPackageBranch(self, url):
+        self.assertIs(None, launchpad._get_package_branch_info(url))
+
+    def assertBranchInfo(self, url, archive, series, project):
+        self.assertEqual((archive, series, project),
+            launchpad._get_package_branch_info(url))
+
+    def test_package_branch_regex(self):
+        self.assertPackageBranchRe(
+            'http://bazaar.launchpad.net/+branch/ubuntu/foo',
+            None, 'ubuntu', None, 'foo')
+        self.assertPackageBranchRe(
+            'bzr+ssh://bazaar.launchpad.net/+branch/ubuntu/natty/foo',
+            None, 'ubuntu', 'natty/', 'foo')
+        self.assertPackageBranchRe(
+            'sftp://bazaar.launchpad.net/+branch/debian/foo',
+            None, 'debian', None, 'foo')
+        self.assertPackageBranchRe(
+            'http://bazaar.launchpad.net/+branch/debian/sid/foo',
+            None, 'debian', 'sid/', 'foo')
+        self.assertPackageBranchRe(
+            'http://bazaar.launchpad.net/+branch'
+            '/~ubuntu-branches/ubuntu/natty/foo/natty',
+            '~ubuntu-branches/', 'ubuntu', 'natty/', 'foo')
+        self.assertPackageBranchRe(
+            'http://bazaar.launchpad.net/+branch'
+            '/~user/ubuntu/natty/foo/test',
+            '~user/', 'ubuntu', 'natty/', 'foo')
+
+    def test_package_branch_doesnt_match(self):
+        self.assertNotPackageBranch('http://example.com/ubuntu/foo')
+        self.assertNotPackageBranch(
+            'http://bazaar.launchpad.net/+branch/bzr')
+        self.assertNotPackageBranch(
+            'http://bazaar.launchpad.net/+branch/~bzr-pqm/bzr/bzr.dev')
+        # Not a packaging branch because ~user isn't ~ubuntu-branches
+        self.assertNotPackageBranch(
+            'http://bazaar.launchpad.net/+branch'
+            '/~user/ubuntu/natty/foo/natty')
+        # Older versions of bzr-svn/hg/git did not set Branch.base until after
+        # they called Branch.__init__().
+        self.assertNotPackageBranch(None)
+
+    def test__get_package_branch_info(self):
+        self.assertBranchInfo(
+            'bzr+ssh://bazaar.launchpad.net/+branch/ubuntu/natty/foo',
+            'ubuntu', 'natty', 'foo')
+        self.assertBranchInfo(
+            'bzr+ssh://bazaar.launchpad.net/+branch'
+            '/~ubuntu-branches/ubuntu/natty/foo/natty',
+            'ubuntu', 'natty', 'foo')
+        self.assertBranchInfo(
+            'http://bazaar.launchpad.net/+branch'
+            '/~ubuntu-branches/debian/sid/foo/sid',
+            'debian', 'sid', 'foo')
+
+
+class TestGetMostRecentTag(tests.TestCaseWithMemoryTransport):
+
+    def make_simple_builder(self):
+        builder = self.make_branch_builder('tip')
+        builder.build_snapshot('A', [], [
+            ('add', ('', 'root-id', 'directory', None))])
+        b = builder.get_branch()
+        b.tags.set_tag('tip-1.0', 'A')
+        return builder, b, b.tags.get_tag_dict()
+
+    def test_get_most_recent_tag_tip(self):
+        builder, b, tag_dict = self.make_simple_builder()
+        self.assertEqual('tip-1.0',
+                         lp_api_lite.get_most_recent_tag(tag_dict, b))
+
+    def test_get_most_recent_tag_older(self):
+        builder, b, tag_dict = self.make_simple_builder()
+        builder.build_snapshot('B', ['A'], [])
+        self.assertEqual('B', b.last_revision())
+        self.assertEqual('tip-1.0',
+                         lp_api_lite.get_most_recent_tag(tag_dict, b))
+
+
+class StubLatestPublication(object):
+
+    def __init__(self, latest):
+        self.called = False
+        self.latest = latest
+
+    def get_latest_version(self):
+        self.called = True
+        return self.latest
+
+    def place(self):
+        return 'Ubuntu Natty'
+
+
+class TestReportFreshness(tests.TestCaseWithMemoryTransport):
+
+    def setUp(self):
+        super(TestReportFreshness, self).setUp()
+        builder = self.make_branch_builder('tip')
+        builder.build_snapshot('A', [], [
+            ('add', ('', 'root-id', 'directory', None))])
+        self.branch = builder.get_branch()
+
+    def assertFreshnessReports(self, verbosity, latest_version, content):
+        """Assert that lp_api_lite.report_freshness reports the given content.
+
+        :param verbosity: The reporting level
+        :param latest_version: The version reported by StubLatestPublication
+        :param content: The expected content. This should be in DocTest form.
+        """
+        orig_log_len = len(self.get_log())
+        lp_api_lite.report_freshness(self.branch, verbosity,
+            StubLatestPublication(latest_version))
+        new_content = self.get_log()[orig_log_len:]
+        # Strip out lines that have LatestPublication.get_* because those are
+        # timing related lines. While interesting to log for now, they aren't
+        # something we want to be testing
+        new_content = new_content.split('\n')
+        for i in range(2):
+            if (len(new_content) > 0
+                and 'LatestPublication.get_' in new_content[0]):
+                new_content = new_content[1:]
+        new_content = '\n'.join(new_content)
+        self.assertThat(new_content,
+            DocTestMatches(content,
+                doctest.ELLIPSIS | doctest.REPORT_UDIFF))
+
+    def test_verbosity_off_skips_check(self):
+        # We force _get_package_branch_info so that we know it would otherwise
+        # try to connect to launcphad
+        self.overrideAttr(launchpad, '_get_package_branch_info',
+            lambda x: ('ubuntu', 'natty', 'bzr'))
+        self.overrideAttr(lp_api_lite, 'LatestPublication',
+            lambda *args: self.fail('Tried to query launchpad'))
+        c = self.branch.get_config()
+        c.set_user_option('launchpad.packaging_verbosity', 'off')
+        orig_log_len = len(self.get_log())
+        launchpad._check_is_up_to_date(self.branch)
+        new_content = self.get_log()[orig_log_len:]
+        self.assertContainsRe(new_content,
+            'not checking memory.*/tip/ because verbosity is turned off')
+
+    def test_verbosity_off(self):
+        latest_pub = StubLatestPublication('1.0-1ubuntu2')
+        lp_api_lite.report_freshness(self.branch, 'off', latest_pub)
+        self.assertFalse(latest_pub.called)
+
+    def test_verbosity_all_out_of_date_smoke(self):
+        self.branch.tags.set_tag('1.0-1ubuntu1', 'A')
+        self.assertFreshnessReports('all', '1.0-1ubuntu2',
+             '    INFO  Most recent Ubuntu Natty version: 1.0-1ubuntu2\n'
+             'Packaging branch version: 1.0-1ubuntu1\n'
+             'Packaging branch status: OUT-OF-DATE\n')
+
+
+class Test_GetNewestVersions(tests.TestCaseWithMemoryTransport):
+
+    def setUp(self):
+        super(Test_GetNewestVersions, self).setUp()
+        builder = self.make_branch_builder('tip')
+        builder.build_snapshot('A', [], [
+            ('add', ('', 'root-id', 'directory', None))])
+        self.branch = builder.get_branch()
+
+    def assertLatestVersions(self, latest_branch_version, pub_version):
+        if latest_branch_version is not None:
+            self.branch.tags.set_tag(latest_branch_version, 'A')
+        latest_pub = StubLatestPublication(pub_version)
+        self.assertEqual((pub_version, latest_branch_version),
+            lp_api_lite._get_newest_versions(self.branch, latest_pub))
+
+    def test_no_tags(self):
+        self.assertLatestVersions(None, '1.0-1ubuntu2')
+
+    def test_out_of_date(self):
+        self.assertLatestVersions('1.0-1ubuntu1', '1.0-1ubuntu2')
+
+    def test_up_to_date(self):
+        self.assertLatestVersions('1.0-1ubuntu2', '1.0-1ubuntu2')
+
+    def test_missing(self):
+        self.assertLatestVersions(None, None)
+
+
+class Test_ReportFreshness(tests.TestCase):
+
+    def assertReportedFreshness(self, verbosity, latest_ver, branch_latest_ver,
+                               content, place='Ubuntu Natty'):
+        """Assert that lp_api_lite.report_freshness reports the given content.
+        """
+        reported = []
+        def report_func(value):
+            reported.append(value)
+
+        lp_api_lite._report_freshness(latest_ver, branch_latest_ver, place,
+                                      verbosity, report_func)
+        new_content = '\n'.join(reported)
+        self.assertThat(new_content,
+            DocTestMatches(content,
+                doctest.ELLIPSIS | doctest.REPORT_UDIFF))
+
+    def test_verbosity_minimal_no_tags(self):
+        self.assertReportedFreshness('minimal', '1.0-1ubuntu2', None,
+            'Branch is OUT-OF-DATE, Ubuntu Natty has 1.0-1ubuntu2\n')
+
+    def test_verbosity_minimal_out_of_date(self):
+        self.assertReportedFreshness('minimal', '1.0-1ubuntu2', '1.0-1ubuntu1',
+            '1.0-1ubuntu1 is OUT-OF-DATE,'
+            ' Ubuntu Natty has 1.0-1ubuntu2\n')
+
+    def test_verbosity_minimal_up_to_date(self):
+        self.assertReportedFreshness('minimal', '1.0-1ubuntu2', '1.0-1ubuntu2',
+             '')
+
+    def test_verbosity_minimal_missing(self):
+        self.assertReportedFreshness('minimal', None, None,
+             '')
+
+    def test_verbosity_short_out_of_date(self):
+        self.assertReportedFreshness('short', '1.0-1ubuntu2', '1.0-1ubuntu1',
+            '1.0-1ubuntu1 is OUT-OF-DATE,'
+            ' Ubuntu Natty has 1.0-1ubuntu2\n')
+
+    def test_verbosity_short_up_to_date(self):
+        self.assertReportedFreshness('short', '1.0-1ubuntu2', '1.0-1ubuntu2',
+             '1.0-1ubuntu2 is CURRENT in Ubuntu Natty')
+
+    def test_verbosity_short_missing(self):
+        self.assertReportedFreshness('short', None, None,
+             'Ubuntu Natty is MISSING a version')
+
+    def test_verbosity_all_no_tags(self):
+        self.assertReportedFreshness('all', '1.0-1ubuntu2', None,
+             'Most recent Ubuntu Natty version: 1.0-1ubuntu2\n'
+             'Packaging branch version: None\n'
+             'Packaging branch status: OUT-OF-DATE\n')
+
+    def test_verbosity_all_out_of_date(self):
+        self.assertReportedFreshness('all', '1.0-1ubuntu2', '1.0-1ubuntu1',
+             'Most recent Ubuntu Natty version: 1.0-1ubuntu2\n'
+             'Packaging branch version: 1.0-1ubuntu1\n'
+             'Packaging branch status: OUT-OF-DATE\n')
+
+    def test_verbosity_all_up_to_date(self):
+        self.assertReportedFreshness('all', '1.0-1ubuntu2', '1.0-1ubuntu2',
+             'Most recent Ubuntu Natty version: 1.0-1ubuntu2\n'
+             'Packaging branch status: CURRENT\n')
+
+    def test_verbosity_all_missing(self):
+        self.assertReportedFreshness('all', None, None,
+             'Most recent Ubuntu Natty version: MISSING\n')
+
+    def test_verbosity_None_is_all(self):
+        self.assertReportedFreshness(None, '1.0-1ubuntu2', '1.0-1ubuntu2',
+             'Most recent Ubuntu Natty version: 1.0-1ubuntu2\n'
+             'Packaging branch status: CURRENT\n')

=== modified file 'bzrlib/plugins/launchpad/test_lp_directory.py'
--- bzrlib/plugins/launchpad/test_lp_directory.py	2010-07-26 17:19:33 +0000
+++ bzrlib/plugins/launchpad/test_lp_directory.py	2011-10-06 06:45:46 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 import xmlrpclib
 
 from bzrlib import (
+    debug,
     errors,
     tests,
     transport,
@@ -27,6 +28,7 @@
 from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
 from bzrlib.tests import (
+    features,
     TestCaseInTempDir,
     TestCaseWithMemoryTransport
 )
@@ -37,10 +39,7 @@
 from bzrlib.plugins.launchpad.lp_directory import (
     LaunchpadDirectory)
 from bzrlib.plugins.launchpad.account import get_lp_login, set_lp_login
-from bzrlib.tests import (
-    http_server,
-    http_utils,
-    )
+from bzrlib.tests import http_server
 
 
 def load_tests(standard_tests, module, loader):
@@ -52,7 +51,7 @@
     transport_scenarios = [
         ('http', dict(server_class=PreCannedHTTPServer,)),
         ]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         transport_scenarios.append(
             ('https', dict(server_class=PreCannedHTTPSServer,)),
             )
@@ -65,10 +64,12 @@
 
 
 class FakeResolveFactory(object):
+
     def __init__(self, test, expected_path, result):
         self._test = test
         self._expected_path = expected_path
         self._result = result
+        self._submitted = False
 
     def __call__(self, path):
         self._test.assertEqual(self._expected_path, path)
@@ -76,9 +77,147 @@
 
     def submit(self, service):
         self._service_url = service.service_url
+        self._submitted = True
         return self._result
 
 
+class LocalDirectoryURLTests(TestCaseInTempDir):
+    """Tests for branch urls that we try to pass through local resolution."""
+
+    def assertResolve(self, expected, url, submitted=False):
+        path = url[url.index(':')+1:].lstrip('/')
+        factory = FakeResolveFactory(self, path,
+                    dict(urls=['bzr+ssh://fake-resolved']))
+        directory = LaunchpadDirectory()
+        self.assertEqual(expected,
+            directory._resolve(url, factory, _lp_login='user'))
+        # We are testing local resolution, and the fallback when necessary.
+        self.assertEqual(submitted, factory._submitted)
+
+    def test_short_form(self):
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/apt',
+                           'lp:apt')
+
+    def test_two_part_form(self):
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/apt/2.2',
+                           'lp:apt/2.2')
+
+    def test_two_part_plus_subdir(self):
+        # We allow you to pass more than just what resolves. That way you can
+        # do things like "bzr log lp:apt/2.2/BUGS"
+        # Though the virtual FS implementation currently aborts when given a
+        # URL like this, rather than letting you recurse upwards to find the
+        # real branch at lp:apt/2.2
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/apt/2.2/BUGS',
+                           'lp:apt/2.2/BUGS')
+
+    def test_user_expansion(self):
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/~user/apt/foo',
+                           'lp:~/apt/foo')
+
+    def test_ubuntu(self):
+        # Confirmed against xmlrpc. If you don't have a ~user, xmlrpc doesn't
+        # care that you are asking for 'ubuntu'
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/ubuntu',
+                           'lp:ubuntu')
+
+    def test_ubuntu_invalid(self):
+        """Invalid ubuntu urls don't crash.
+
+        :seealso: http://pad.lv/843900
+        """
+        # This ought to be natty-updates.
+        self.assertRaises(errors.InvalidURL,
+            self.assertResolve,
+            '',
+            'ubuntu:natty/updates/smartpm')
+
+    def test_ubuntu_apt(self):
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/ubuntu/apt',
+                           'lp:ubuntu/apt')
+
+    def test_ubuntu_natty_apt(self):
+        self.assertResolve(
+            'bzr+ssh://bazaar.launchpad.net/+branch/ubuntu/natty/apt',
+            'lp:ubuntu/natty/apt')
+
+    def test_ubuntu_natty_apt_filename(self):
+        self.assertResolve(
+            'bzr+ssh://bazaar.launchpad.net/+branch/ubuntu/natty/apt/filename',
+            'lp:ubuntu/natty/apt/filename')
+
+    def test_user_two_part(self):
+        # We fall back to the ResolveFactory. The real Launchpad one will raise
+        # InvalidURL for this case.
+        self.assertResolve('bzr+ssh://fake-resolved', 'lp:~jameinel/apt',
+                           submitted=True)
+
+    def test_user_three_part(self):
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/~jameinel/apt/foo',
+                           'lp:~jameinel/apt/foo')
+
+    def test_user_three_part_plus_filename(self):
+        self.assertResolve(
+            'bzr+ssh://bazaar.launchpad.net/~jameinel/apt/foo/fname',
+            'lp:~jameinel/apt/foo/fname')
+
+    def test_user_ubuntu_two_part(self):
+        self.assertResolve('bzr+ssh://fake-resolved', 'lp:~jameinel/ubuntu',
+                           submitted=True)
+        self.assertResolve('bzr+ssh://fake-resolved', 'lp:~jameinel/debian',
+                           submitted=True)
+
+    def test_user_ubuntu_three_part(self):
+        self.assertResolve('bzr+ssh://fake-resolved',
+                           'lp:~jameinel/ubuntu/natty', submitted=True)
+        self.assertResolve('bzr+ssh://fake-resolved',
+                           'lp:~jameinel/debian/sid', submitted=True)
+
+    def test_user_ubuntu_four_part(self):
+        self.assertResolve('bzr+ssh://fake-resolved',
+                           'lp:~jameinel/ubuntu/natty/project', submitted=True)
+        self.assertResolve('bzr+ssh://fake-resolved',
+                           'lp:~jameinel/debian/sid/project', submitted=True)
+
+    def test_user_ubuntu_five_part(self):
+        self.assertResolve(
+            'bzr+ssh://bazaar.launchpad.net/~jameinel/ubuntu/natty/apt/branch',
+            'lp:~jameinel/ubuntu/natty/apt/branch')
+        self.assertResolve(
+            'bzr+ssh://bazaar.launchpad.net/~jameinel/debian/sid/apt/branch',
+            'lp:~jameinel/debian/sid/apt/branch')
+
+    def test_user_ubuntu_five_part_plus_subdir(self):
+        self.assertResolve(
+            'bzr+ssh://bazaar.launchpad.net/~jameinel/ubuntu/natty/apt/branch/f',
+            'lp:~jameinel/ubuntu/natty/apt/branch/f')
+        self.assertResolve(
+            'bzr+ssh://bazaar.launchpad.net/~jameinel/debian/sid/apt/branch/f',
+            'lp:~jameinel/debian/sid/apt/branch/f')
+
+    def test_handles_special_lp(self):
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/apt', 'lp:apt')
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/apt',
+                           'lp:///apt')
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/apt',
+                           'lp://production/apt')
+        self.assertResolve('bzr+ssh://bazaar.launchpad.dev/+branch/apt',
+                           'lp://dev/apt')
+        self.assertResolve('bzr+ssh://bazaar.staging.launchpad.net/+branch/apt',
+                           'lp://staging/apt')
+        self.assertResolve('bzr+ssh://bazaar.qastaging.launchpad.net/+branch/apt',
+                           'lp://qastaging/apt')
+        self.assertResolve('bzr+ssh://bazaar.demo.launchpad.net/+branch/apt',
+                           'lp://demo/apt')
+
+    def test_debug_launchpad_uses_resolver(self):
+        self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/bzr',
+                           'lp:bzr', submitted=False)
+        debug.debug_flags.add('launchpad')
+        self.addCleanup(debug.debug_flags.discard, 'launchpad')
+        self.assertResolve('bzr+ssh://fake-resolved', 'lp:bzr', submitted=True)
+
+
 class DirectoryUrlTests(TestCaseInTempDir):
     """Tests for branch urls through Launchpad.net directory"""
 
@@ -94,6 +233,19 @@
         self.assertEquals('https://xmlrpc.launchpad.net/bazaar/',
                           factory._service_url)
 
+    def test_qastaging(self):
+        """A launchpad url should map to a http url"""
+        factory = FakeResolveFactory(
+            self, 'apt', dict(urls=[
+                    'http://bazaar.qastaging.launchpad.net/~apt/apt/devel']))
+        url = 'lp://qastaging/apt'
+        directory = LaunchpadDirectory()
+        self.assertEquals('http://bazaar.qastaging.launchpad.net/~apt/apt/devel',
+                          directory._resolve(url, factory))
+        # Make sure that resolve went to the qastaging server.
+        self.assertEquals('https://xmlrpc.qastaging.launchpad.net/bazaar/',
+                          factory._service_url)
+
     def test_staging(self):
         """A launchpad url should map to a http url"""
         factory = FakeResolveFactory(
@@ -170,16 +322,16 @@
         self.assertEquals('http://bazaar.launchpad.net/~apt/apt/devel',
                           directory._resolve('lp:///apt', factory))
 
-    def test_rewrite_bzr_ssh_launchpad_net(self):
+    def test_with_login_avoid_resolve_factory(self):
         # Test that bzr+ssh URLs get rewritten to include the user's
         # Launchpad ID (assuming we know the Launchpad ID).
         factory = FakeResolveFactory(
             self, 'apt', dict(urls=[
-                    'bzr+ssh://bazaar.launchpad.net/~apt/apt/devel',
+                    'bzr+ssh://my-super-custom/special/devel',
                     'http://bazaar.launchpad.net/~apt/apt/devel']))
         directory = LaunchpadDirectory()
         self.assertEquals(
-            'bzr+ssh://bazaar.launchpad.net/~apt/apt/devel',
+            'bzr+ssh://bazaar.launchpad.net/+branch/apt',
             directory._resolve('lp:///apt', factory, _lp_login='username'))
 
     def test_no_rewrite_of_other_bzr_ssh(self):
@@ -202,7 +354,7 @@
     def test_resolve_tilde_to_user(self):
         factory = FakeResolveFactory(
             self, '~username/apt/test', dict(urls=[
-                    'bzr+ssh://bazaar.launchpad.net/~username/apt/test']))
+                'bzr+ssh://bazaar.launchpad.net/~username/apt/test']))
         directory = LaunchpadDirectory()
         self.assertEquals(
             'bzr+ssh://bazaar.launchpad.net/~username/apt/test',
@@ -219,8 +371,8 @@
                     'bzr+ssh://bazaar.launchpad.net/~username/apt/test']))
         self.assertIs(None, get_lp_login())
         directory = LaunchpadDirectory()
-        e = self.assertRaises(errors.InvalidURL,
-            directory._resolve, 'lp:~/apt/test', factory)
+        self.assertRaises(errors.InvalidURL,
+                          directory._resolve, 'lp:~/apt/test', factory)
 
 
 class DirectoryOpenBranchTests(TestCaseWithMemoryTransport):
@@ -237,6 +389,8 @@
                 return '!unexpected look_up value!'
 
         directories.remove('lp:')
+        directories.remove('ubuntu:')
+        directories.remove('debianlp:')
         directories.register('lp:', FooService, 'Map lp URLs to local urls')
         self.addCleanup(_register_directory)
         self.addCleanup(directories.remove, 'lp:')
@@ -259,11 +413,11 @@
     def handle_one_request(self):
         tcs = self.server.test_case_server
         requestline = self.rfile.readline()
-        headers = self.MessageClass(self.rfile, 0)
+        self.MessageClass(self.rfile, 0)
         if requestline.startswith('POST'):
             # The body should be a single line (or we don't know where it ends
             # and we don't want to issue a blocking read)
-            body = self.rfile.readline()
+            self.rfile.readline()
 
         self.wfile.write(tcs.canned_response)
 
@@ -283,7 +437,7 @@
     pass
 
 
-if tests.HTTPSServerFeature.available():
+if features.HTTPSServerFeature.available():
     from bzrlib.tests import https_server
     class PreCannedHTTPSServer(PreCannedServerMixin, https_server.HTTPSServer):
         pass
@@ -299,7 +453,7 @@
         self.server = self.server_class()
         self.server.start_server()
         # Ensure we don't clobber env
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def tearDown(self):
         self.server.stop_server()
@@ -356,3 +510,125 @@
     # FIXME: we need to test with a real proxy, I can't find a way so simulate
     # CONNECT without leaving one server hanging the test :-/ Since that maybe
     # related to the leaking tests problems, I'll punt for now -- vila 20091030
+
+
+class TestDebuntuExpansions(TestCaseInTempDir):
+    """Test expansions for ubuntu: and debianlp: schemes."""
+
+    def setUp(self):
+        super(TestDebuntuExpansions, self).setUp()
+        self.directory = LaunchpadDirectory()
+
+    def _make_factory(self, package='foo', distro='ubuntu', series=None):
+        if series is None:
+            path = '%s/%s' % (distro, package)
+            url_suffix = '~branch/%s/%s' % (distro, package)
+        else:
+            path = '%s/%s/%s' % (distro, series, package)
+            url_suffix = '~branch/%s/%s/%s' % (distro, series, package)
+        return FakeResolveFactory(
+            self, path, dict(urls=[
+                'http://bazaar.launchpad.net/' + url_suffix]))
+
+    def assertURL(self, expected_url, shortcut, package='foo', distro='ubuntu',
+                  series=None):
+        factory = self._make_factory(package=package, distro=distro,
+                                     series=series)
+        self.assertEqual('http://bazaar.launchpad.net/~branch/' + expected_url,
+                         self.directory._resolve(shortcut, factory))
+
+    # Bogus distro.
+
+    def test_bogus_distro(self):
+        self.assertRaises(errors.InvalidURL,
+                          self.directory._resolve, 'gentoo:foo')
+
+    def test_trick_bogus_distro_u(self):
+        self.assertRaises(errors.InvalidURL,
+                          self.directory._resolve, 'utube:foo')
+
+    def test_trick_bogus_distro_d(self):
+        self.assertRaises(errors.InvalidURL,
+                          self.directory._resolve, 'debuntu:foo')
+
+    def test_missing_ubuntu_distroseries_without_project(self):
+        # Launchpad does not hold source packages for Intrepid.  Missing or
+        # bogus distroseries with no project name is treated like a project.
+        self.assertURL('ubuntu/intrepid', 'ubuntu:intrepid', package='intrepid')
+
+    def test_missing_ubuntu_distroseries_with_project(self):
+        # Launchpad does not hold source packages for Intrepid.  Missing or
+        # bogus distroseries with a project name is treated like an unknown
+        # series (i.e. we keep it verbatim).
+        self.assertURL('ubuntu/intrepid/foo',
+                       'ubuntu:intrepid/foo', series='intrepid')
+
+    def test_missing_debian_distroseries(self):
+        # Launchpad does not hold source packages for unstable.  Missing or
+        # bogus distroseries is treated like a project.
+        self.assertURL('debian/sid',
+                       'debianlp:sid', package='sid', distro='debian')
+
+    # Ubuntu Default distro series.
+
+    def test_ubuntu_default_distroseries_expansion(self):
+        self.assertURL('ubuntu/foo', 'ubuntu:foo')
+
+    def test_ubuntu_natty_distroseries_expansion(self):
+        self.assertURL('ubuntu/natty/foo', 'ubuntu:natty/foo', series='natty')
+
+    def test_ubuntu_n_distroseries_expansion(self):
+        self.assertURL('ubuntu/natty/foo', 'ubuntu:n/foo', series='natty')
+
+    def test_ubuntu_maverick_distroseries_expansion(self):
+        self.assertURL('ubuntu/maverick/foo', 'ubuntu:maverick/foo',
+                       series='maverick')
+
+    def test_ubuntu_m_distroseries_expansion(self):
+        self.assertURL('ubuntu/maverick/foo', 'ubuntu:m/foo', series='maverick')
+
+    def test_ubuntu_lucid_distroseries_expansion(self):
+        self.assertURL('ubuntu/lucid/foo', 'ubuntu:lucid/foo', series='lucid')
+
+    def test_ubuntu_l_distroseries_expansion(self):
+        self.assertURL('ubuntu/lucid/foo', 'ubuntu:l/foo', series='lucid')
+
+    def test_ubuntu_karmic_distroseries_expansion(self):
+        self.assertURL('ubuntu/karmic/foo', 'ubuntu:karmic/foo',
+                       series='karmic')
+
+    def test_ubuntu_k_distroseries_expansion(self):
+        self.assertURL('ubuntu/karmic/foo', 'ubuntu:k/foo', series='karmic')
+
+    def test_ubuntu_jaunty_distroseries_expansion(self):
+        self.assertURL('ubuntu/jaunty/foo', 'ubuntu:jaunty/foo',
+                       series='jaunty')
+
+    def test_ubuntu_j_distroseries_expansion(self):
+        self.assertURL('ubuntu/jaunty/foo', 'ubuntu:j/foo', series='jaunty')
+
+    def test_ubuntu_hardy_distroseries_expansion(self):
+        self.assertURL('ubuntu/hardy/foo', 'ubuntu:hardy/foo', series='hardy')
+
+    def test_ubuntu_h_distroseries_expansion(self):
+        self.assertURL('ubuntu/hardy/foo', 'ubuntu:h/foo', series='hardy')
+
+    def test_ubuntu_dapper_distroseries_expansion(self):
+        self.assertURL('ubuntu/dapper/foo', 'ubuntu:dapper/foo',
+                       series='dapper')
+
+    def test_ubuntu_d_distroseries_expansion(self):
+        self.assertURL('ubuntu/dapper/foo', 'ubuntu:d/foo', series='dapper')
+
+    # Debian default distro series.
+
+    def test_debian_default_distroseries_expansion(self):
+        self.assertURL('debian/foo', 'debianlp:foo', distro='debian')
+
+    def test_debian_squeeze_distroseries_expansion(self):
+        self.assertURL('debian/squeeze/foo', 'debianlp:squeeze/foo',
+                       distro='debian', series='squeeze')
+
+    def test_debian_lenny_distroseries_expansion(self):
+        self.assertURL('debian/lenny/foo', 'debianlp:lenny/foo',
+                       distro='debian', series='lenny')

=== modified file 'bzrlib/plugins/launchpad/test_lp_service.py'
--- bzrlib/plugins/launchpad/test_lp_service.py	2010-05-20 18:23:10 +0000
+++ bzrlib/plugins/launchpad/test_lp_service.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
     def setUp(self):
         super(LaunchpadServiceTests, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_default_service(self):
         service = LaunchpadService()
@@ -54,11 +54,6 @@
         self.assertEqual('https://xmlrpc.staging.launchpad.net/bazaar/',
                          service.service_url)
 
-    def test_edge_service(self):
-        service = LaunchpadService(lp_instance='edge')
-        self.assertEqual('https://xmlrpc.edge.launchpad.net/bazaar/',
-                         service.service_url)
-
     def test_dev_service(self):
         service = LaunchpadService(lp_instance='dev')
         self.assertEqual('https://xmlrpc.launchpad.dev/bazaar/',
@@ -171,13 +166,6 @@
         self.assertEqual(
             'https://code.staging.launchpad.net/~foo/bar/baz', web_url)
 
-    def test_edge_url(self):
-        service = LaunchpadService(lp_instance='edge')
-        web_url = service.get_web_url_from_branch_url(
-            'bzr+ssh://bazaar.launchpad.net/~foo/bar/baz')
-        self.assertEqual(
-            'https://code.edge.launchpad.net/~foo/bar/baz', web_url)
-
     def test_dev_url(self):
         service = LaunchpadService(lp_instance='dev')
         web_url = service.get_web_url_from_branch_url(

=== modified file 'bzrlib/plugins/launchpad/test_register.py'
--- bzrlib/plugins/launchpad/test_register.py	2010-05-20 18:23:10 +0000
+++ bzrlib/plugins/launchpad/test_register.py	2011-03-14 10:11:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,18 +15,16 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import base64
-import os
 from StringIO import StringIO
 import urlparse
 import xmlrpclib
 
 from bzrlib import (
     config,
-    osutils,
     tests,
     ui,
     )
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests import TestCaseWithTransport
 
 # local import
 from bzrlib.plugins.launchpad.lp_registration import (
@@ -61,6 +59,28 @@
         """
         return (200, 'OK', [])
 
+    def getresponse(self, buffering=True):
+        """Fake the http reply.
+
+        This is used when running on Python 2.7, where xmlrpclib uses
+        httplib.HTTPConnection in a different way than before.
+        """
+        class FakeHttpResponse(object):
+
+            def __init__(self, status, reason, body):
+                self.status = status
+                self.reason = reason
+                self.body = body
+
+            def read(self, size=-1):
+                return self.body.read(size)
+
+            def getheader(self, name, default):
+                # We don't have headers
+                return default
+
+        return FakeHttpResponse(200, 'OK', self.getfile())
+
     def getfile(self):
         """Return a fake file containing the response content."""
         return StringIO('''\
@@ -85,6 +105,7 @@
     def __init__(self, testcase, expect_auth):
         self.testcase = testcase
         self.expect_auth = expect_auth
+        self._connection = (None, None)
 
     def make_connection(self, host):
         host, http_headers, x509 = self.get_host_info(host)
@@ -133,14 +154,11 @@
 
 
 class TestBranchRegistration(TestCaseWithTransport):
-    SAMPLE_URL = 'http://bazaar-vcs.org/bzr/bzr.dev/'
-    SAMPLE_OWNER = 'jhacker@foo.com'
-    SAMPLE_BRANCH_ID = 'bzr.dev'
 
     def setUp(self):
         super(TestBranchRegistration, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_register_help(self):
         """register-branch accepts --help"""
@@ -175,7 +193,7 @@
         self.assertEquals(out, 'Branch registered.\n')
 
     def test_onto_transport(self):
-        """Test how the request is sent by transmitting across a mock Transport"""
+        """How the request is sent by transmitting across a mock Transport"""
         # use a real transport, but intercept at the http/xml layer
         transport = InstrumentedXMLRPCTransport(self, expect_auth=True)
         service = LaunchpadService(transport)
@@ -200,7 +218,7 @@
         self.assertTrue(transport.got_request)
 
     def test_onto_transport_unauthenticated(self):
-        """Test how an unauthenticated request is transmitted across a mock Transport"""
+        """An unauthenticated request is transmitted across a mock Transport"""
         transport = InstrumentedXMLRPCTransport(self, expect_auth=False)
         service = LaunchpadService(transport)
         resolve = ResolveLaunchpadPathRequest('bzr')
@@ -299,7 +317,7 @@
     def setUp(self):
         super(TestGatherUserCredentials, self).setUp()
         # make sure we have a reproducible standard environment
-        self._captureVar('BZR_LP_XMLRPC_URL', None)
+        self.overrideEnv('BZR_LP_XMLRPC_URL', None)
 
     def test_gather_user_credentials_has_password(self):
         service = LaunchpadService()

=== modified file 'bzrlib/plugins/netrc_credential_store/__init__.py'
--- bzrlib/plugins/netrc_credential_store/__init__.py	2010-04-23 08:51:52 +0000
+++ bzrlib/plugins/netrc_credential_store/__init__.py	2011-04-07 10:36:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,12 +25,12 @@
     )
 
 lazy_import.lazy_import(globals(), """
-    import errno
-    import netrc
+import errno
+import netrc
 
-    from bzrlib import (
-        errors,
-        )
+from bzrlib import (
+    errors,
+    )
 """)
 
 

=== modified file 'bzrlib/plugins/news_merge/__init__.py'
--- bzrlib/plugins/news_merge/__init__.py	2010-05-03 04:08:50 +0000
+++ bzrlib/plugins/news_merge/__init__.py	2011-03-30 14:23:59 +0000
@@ -33,26 +33,17 @@
 
 # Since we are a built-in plugin we share the bzrlib version
 from bzrlib import version_info
-
-# Put most of the code in a separate module that we lazy-import to keep the
-# overhead of this plugin as minimal as possible.
-from bzrlib.lazy_import import lazy_import
-lazy_import(globals(), """
-from bzrlib.plugins.news_merge import news_merge as _mod_news_merge
-""")
-
-from bzrlib.merge import Merger
+from bzrlib.hooks import install_lazy_named_hook
 
 
 def news_merge_hook(merger):
     """Merger.merge_file_content hook for bzr-format NEWS files."""
-    return _mod_news_merge.NewsMerger(merger)
-
-
-def install_hook():
-    Merger.hooks.install_named_hook(
-        'merge_file_content', news_merge_hook, 'NEWS file merge')
-install_hook()
+    from bzrlib.plugins.news_merge.news_merge import NewsMerger
+    return NewsMerger(merger)
+
+
+install_lazy_named_hook("bzrlib.merge", "Merger.hooks", "merge_file_content",
+    news_merge_hook, "NEWS file merge")
 
 
 def load_tests(basic_tests, module, loader):

=== added directory 'bzrlib/plugins/weave_fmt'
=== added file 'bzrlib/plugins/weave_fmt/__init__.py'
--- bzrlib/plugins/weave_fmt/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/__init__.py	2011-05-09 09:01:38 +0000
@@ -0,0 +1,126 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Weave formats.
+
+These were formats present in pre-1.0 version of Bazaar.
+"""
+
+# Since we are a built-in plugin we share the bzrlib version
+from bzrlib import version_info
+
+from bzrlib import (
+    branch as _mod_branch,
+    controldir,
+    repository as _mod_repository,
+    serializer,
+    workingtree as _mod_workingtree,
+    )
+from bzrlib.bzrdir import (
+    BzrProber,
+    register_metadir,
+    )
+
+# Pre-0.8 formats that don't have a disk format string (because they are
+# versioned by the matching control directory). We use the control directories
+# disk format string as a key for the network_name because they meet the
+# constraints (simple string, unique, immutable).
+_mod_repository.network_format_registry.register_lazy(
+    "Bazaar-NG branch, format 5\n",
+    'bzrlib.plugins.weave_fmt.repository',
+    'RepositoryFormat5',
+)
+_mod_repository.network_format_registry.register_lazy(
+    "Bazaar-NG branch, format 6\n",
+    'bzrlib.plugins.weave_fmt.repository',
+    'RepositoryFormat6',
+)
+
+# weave formats which has no format string and are not discoverable or independently
+# creatable on disk, so are not registered in format_registry.  They're
+# all in bzrlib.plugins.weave_fmt.repository now.  When an instance of one of these is
+# needed, it's constructed directly by the BzrDir.  Non-native formats where
+# the repository is not separately opened are similar.
+
+_mod_repository.format_registry.register_lazy(
+    'Bazaar-NG Repository format 7',
+    'bzrlib.plugins.weave_fmt.repository',
+    'RepositoryFormat7'
+    )
+
+_mod_repository.format_registry.register_extra_lazy(
+    'bzrlib.plugins.weave_fmt.repository',
+    'RepositoryFormat4')
+_mod_repository.format_registry.register_extra_lazy(
+    'bzrlib.plugins.weave_fmt.repository',
+    'RepositoryFormat5')
+_mod_repository.format_registry.register_extra_lazy(
+    'bzrlib.plugins.weave_fmt.repository',
+    'RepositoryFormat6')
+
+
+# The pre-0.8 formats have their repository format network name registered in
+# repository.py. MetaDir formats have their repository format network name
+# inferred from their disk format string.
+controldir.format_registry.register_lazy('weave',
+    "bzrlib.plugins.weave_fmt.bzrdir", "BzrDirFormat6",
+    'Pre-0.8 format.  Slower than knit and does not'
+    ' support checkouts or shared repositories.',
+    hidden=True,
+    deprecated=True)
+register_metadir(controldir.format_registry, 'metaweave',
+    'bzrlib.plugins.weave_fmt.repository.RepositoryFormat7',
+    'Transitional format in 0.8.  Slower than knit.',
+    branch_format='bzrlib.branch.BzrBranchFormat5',
+    tree_format='bzrlib.workingtree_3.WorkingTreeFormat3',
+    hidden=True,
+    deprecated=True)
+
+
+BzrProber.formats.register_lazy(
+    "Bazaar-NG branch, format 0.0.4\n", "bzrlib.plugins.weave_fmt.bzrdir",
+    "BzrDirFormat4")
+BzrProber.formats.register_lazy(
+    "Bazaar-NG branch, format 5\n", "bzrlib.plugins.weave_fmt.bzrdir",
+    "BzrDirFormat5")
+BzrProber.formats.register_lazy(
+    "Bazaar-NG branch, format 6\n", "bzrlib.plugins.weave_fmt.bzrdir",
+    "BzrDirFormat6")
+
+
+_mod_branch.format_registry.register_extra_lazy(
+    'bzrlib.plugins.weave_fmt.branch', 'BzrBranchFormat4')
+_mod_branch.network_format_registry.register_lazy(
+    "Bazaar-NG branch, format 6\n",
+    'bzrlib.plugins.weave_fmt.branch', "BzrBranchFormat4")
+
+
+_mod_workingtree.format_registry.register_extra_lazy(
+    'bzrlib.plugins.weave_fmt.workingtree',
+    'WorkingTreeFormat2')
+
+serializer.format_registry.register_lazy('4', 'bzrlib.plugins.weave_fmt.xml4',
+    'serializer_v4')
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'test_bzrdir',
+        'test_repository',
+        'test_workingtree',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== added file 'bzrlib/plugins/weave_fmt/branch.py'
--- bzrlib/plugins/weave_fmt/branch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/branch.py	2011-09-12 09:51:52 +0000
@@ -0,0 +1,156 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Weave-era branch implementations."""
+
+from bzrlib import (
+    errors,
+    lockable_files,
+    )
+
+from bzrlib.trace import mutter
+
+from bzrlib.branch import (
+    BranchFormat,
+    FullHistoryBzrBranch,
+    )
+
+
+class BzrBranch4(FullHistoryBzrBranch):
+    """Branch format 4."""
+
+    def _get_checkout_format(self, lightweight=False):
+        """Return the most suitable metadir for a checkout of this branch.
+        """
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat7
+        from bzrlib.bzrdir import BzrDirMetaFormat1
+        format = BzrDirMetaFormat1()
+        if lightweight:
+            format.set_branch_format(self._format)
+            format.repository_format = self.bzrdir._format.repository_format
+        else:
+            format.repository_format = RepositoryFormat7()
+        return format
+
+    def unbind(self):
+        raise errors.UpgradeRequired(self.user_url)
+
+    def bind(self, other):
+        raise errors.UpgradeRequired(self.user_url)
+
+    def set_bound_location(self, location):
+        raise NotImplementedError(self.set_bound_location)
+
+    def get_bound_location(self):
+        return None
+
+    def update(self):
+        return None
+
+    def get_master_branch(self, possible_transports=None):
+        return None
+
+
+class BzrBranchFormat4(BranchFormat):
+    """Bzr branch format 4.
+
+    This format has:
+     - a revision-history file.
+     - a branch-lock lock file [ to be shared with the bzrdir ]
+
+    It does not support binding.
+    """
+
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
+        """Create a branch of this format in a_bzrdir.
+
+        :param a_bzrdir: The bzrdir to initialize the branch in
+        :param name: Name of colocated branch to create, if any
+        :param repository: Repository for this branch (unused)
+        """
+        if append_revisions_only:
+            raise errors.UpgradeRequired(a_bzrdir.user_url)
+        if repository is not None:
+            raise NotImplementedError(
+                "initialize(repository=<not None>) on %r" % (self,))
+        if not [isinstance(a_bzrdir._format, format) for format in
+                self._compatible_bzrdirs]:
+            raise errors.IncompatibleFormat(self, a_bzrdir._format)
+        utf8_files = [('revision-history', ''),
+                      ('branch-name', ''),
+                      ]
+        mutter('creating branch %r in %s', self, a_bzrdir.user_url)
+        branch_transport = a_bzrdir.get_branch_transport(self, name=name)
+        control_files = lockable_files.LockableFiles(branch_transport,
+            'branch-lock', lockable_files.TransportLock)
+        control_files.create_lock()
+        try:
+            control_files.lock_write()
+        except errors.LockContention:
+            lock_taken = False
+        else:
+            lock_taken = True
+        try:
+            for (filename, content) in utf8_files:
+                branch_transport.put_bytes(
+                    filename, content,
+                    mode=a_bzrdir._get_file_mode())
+        finally:
+            if lock_taken:
+                control_files.unlock()
+        branch = self.open(a_bzrdir, name, _found=True,
+                found_repository=None)
+        self._run_post_branch_init_hooks(a_bzrdir, name, branch)
+        return branch
+
+    def __init__(self):
+        super(BzrBranchFormat4, self).__init__()
+        from bzrlib.plugins.weave_fmt.bzrdir import (
+            BzrDirFormat4, BzrDirFormat5, BzrDirFormat6,
+            )
+        self._matchingbzrdir = BzrDirFormat6()
+        self._compatible_bzrdirs = [BzrDirFormat4, BzrDirFormat5,
+            BzrDirFormat6]
+
+    def network_name(self):
+        """The network name for this format is the control dirs disk label."""
+        return self._matchingbzrdir.get_format_string()
+
+    def get_format_description(self):
+        return "Branch format 4"
+
+    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
+            found_repository=None):
+        """See BranchFormat.open()."""
+        if name is not None:
+            raise errors.NoColocatedBranchSupport(self)
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        if found_repository is None:
+            found_repository = a_bzrdir.open_repository()
+        return BzrBranch4(_format=self,
+                         _control_files=a_bzrdir._control_files,
+                         a_bzrdir=a_bzrdir,
+                         name=name,
+                         _repository=found_repository)
+
+    def __str__(self):
+        return "Bazaar-NG branch format 4"
+
+    def supports_leaving_lock(self):
+        return False

=== added file 'bzrlib/plugins/weave_fmt/bzrdir.py'
--- bzrlib/plugins/weave_fmt/bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/bzrdir.py	2011-10-06 00:14:01 +0000
@@ -0,0 +1,982 @@
+# Copyright (C) 2006-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Weave-era BzrDir formats."""
+
+from bzrlib.bzrdir import (
+    BzrDir,
+    BzrDirFormat,
+    BzrDirMetaFormat1,
+    )
+from bzrlib.controldir import (
+    Converter,
+    format_registry,
+    )
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import os
+import warnings
+
+from bzrlib import (
+    errors,
+    graph,
+    lockable_files,
+    lockdir,
+    osutils,
+    revision as _mod_revision,
+    trace,
+    ui,
+    urlutils,
+    versionedfile,
+    weave,
+    xml5,
+    )
+from bzrlib.i18n import gettext
+from bzrlib.store.versioned import VersionedFileStore
+from bzrlib.transactions import WriteTransaction
+from bzrlib.transport import (
+    get_transport,
+    local,
+    )
+from bzrlib.plugins.weave_fmt import xml4
+""")
+
+
+class BzrDirFormatAllInOne(BzrDirFormat):
+    """Common class for formats before meta-dirs."""
+
+    fixed_components = True
+
+    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
+        create_prefix=False, force_new_repo=False, stacked_on=None,
+        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
+        shared_repo=False):
+        """See BzrDirFormat.initialize_on_transport_ex."""
+        require_stacking = (stacked_on is not None)
+        # Format 5 cannot stack, but we've been asked to - actually init
+        # a Meta1Dir
+        if require_stacking:
+            format = BzrDirMetaFormat1()
+            return format.initialize_on_transport_ex(transport,
+                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
+                force_new_repo=force_new_repo, stacked_on=stacked_on,
+                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
+                make_working_trees=make_working_trees, shared_repo=shared_repo)
+        return BzrDirFormat.initialize_on_transport_ex(self, transport,
+            use_existing_dir=use_existing_dir, create_prefix=create_prefix,
+            force_new_repo=force_new_repo, stacked_on=stacked_on,
+            stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
+            make_working_trees=make_working_trees, shared_repo=shared_repo)
+
+
+class BzrDirFormat5(BzrDirFormatAllInOne):
+    """Bzr control format 5.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 2 working trees [always]
+     - Format 4 branches [always]
+     - Format 5 repositories [always]
+       Unhashed stores in the repository.
+    """
+
+    _lock_class = lockable_files.TransportLock
+
+    def __eq__(self, other):
+        return type(self) == type(other)
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 5\n"
+
+    def get_branch_format(self):
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        return BzrBranchFormat4()
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 5"
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir5To6()
+
+    def _initialize_for_clone(self, url):
+        return self.initialize_on_transport(get_transport(url), _cloning=True)
+
+    def initialize_on_transport(self, transport, _cloning=False):
+        """Format 5 dirs always have working tree, branch and repository.
+
+        Except when they are being cloned.
+        """
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
+        result = (super(BzrDirFormat5, self).initialize_on_transport(transport))
+        RepositoryFormat5().initialize(result, _internal=True)
+        if not _cloning:
+            branch = BzrBranchFormat4().initialize(result)
+            result._init_workingtree()
+        return result
+
+    def network_name(self):
+        return self.get_format_string()
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir5(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
+        return RepositoryFormat5()
+    repository_format = property(__return_repository_format)
+
+
+class BzrDirFormat6(BzrDirFormatAllInOne):
+    """Bzr control format 6.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 2 working trees [always]
+     - Format 4 branches [always]
+     - Format 6 repositories [always]
+    """
+
+    _lock_class = lockable_files.TransportLock
+
+    def __eq__(self, other):
+        return type(self) == type(other)
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 6\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 6"
+
+    def get_branch_format(self):
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        return BzrBranchFormat4()
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir6ToMeta()
+
+    def _initialize_for_clone(self, url):
+        return self.initialize_on_transport(get_transport(url), _cloning=True)
+
+    def initialize_on_transport(self, transport, _cloning=False):
+        """Format 6 dirs always have working tree, branch and repository.
+
+        Except when they are being cloned.
+        """
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
+        result = super(BzrDirFormat6, self).initialize_on_transport(transport)
+        RepositoryFormat6().initialize(result, _internal=True)
+        if not _cloning:
+            branch = BzrBranchFormat4().initialize(result)
+            result._init_workingtree()
+        return result
+
+    def network_name(self):
+        return self.get_format_string()
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir6(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
+        return RepositoryFormat6()
+    repository_format = property(__return_repository_format)
+
+
+class ConvertBzrDir4To5(Converter):
+    """Converts format 4 bzr dirs to format 5."""
+
+    def __init__(self):
+        super(ConvertBzrDir4To5, self).__init__()
+        self.converted_revs = set()
+        self.absent_revisions = set()
+        self.text_count = 0
+        self.revisions = {}
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        if pb is not None:
+            warnings.warn(gettext("pb parameter to convert() is deprecated"))
+        self.pb = ui.ui_factory.nested_progress_bar()
+        try:
+            ui.ui_factory.note(gettext('starting upgrade from format 4 to 5'))
+            if isinstance(self.bzrdir.transport, local.LocalTransport):
+                self.bzrdir.get_workingtree_transport(None).delete('stat-cache')
+            self._convert_to_weaves()
+            return BzrDir.open(self.bzrdir.user_url)
+        finally:
+            self.pb.finished()
+
+    def _convert_to_weaves(self):
+        ui.ui_factory.note(gettext(
+          'note: upgrade may be faster if all store files are ungzipped first'))
+        try:
+            # TODO permissions
+            stat = self.bzrdir.transport.stat('weaves')
+            if not S_ISDIR(stat.st_mode):
+                self.bzrdir.transport.delete('weaves')
+                self.bzrdir.transport.mkdir('weaves')
+        except errors.NoSuchFile:
+            self.bzrdir.transport.mkdir('weaves')
+        # deliberately not a WeaveFile as we want to build it up slowly.
+        self.inv_weave = weave.Weave('inventory')
+        # holds in-memory weaves for all files
+        self.text_weaves = {}
+        self.bzrdir.transport.delete('branch-format')
+        self.branch = self.bzrdir.open_branch()
+        self._convert_working_inv()
+        rev_history = self.branch._revision_history()
+        # to_read is a stack holding the revisions we still need to process;
+        # appending to it adds new highest-priority revisions
+        self.known_revisions = set(rev_history)
+        self.to_read = rev_history[-1:]
+        while self.to_read:
+            rev_id = self.to_read.pop()
+            if (rev_id not in self.revisions
+                and rev_id not in self.absent_revisions):
+                self._load_one_rev(rev_id)
+        self.pb.clear()
+        to_import = self._make_order()
+        for i, rev_id in enumerate(to_import):
+            self.pb.update(gettext('converting revision'), i, len(to_import))
+            self._convert_one_rev(rev_id)
+        self.pb.clear()
+        self._write_all_weaves()
+        self._write_all_revs()
+        ui.ui_factory.note(gettext('upgraded to weaves:'))
+        ui.ui_factory.note('  ' + gettext('%6d revisions and inventories') %
+                                                        len(self.revisions))
+        ui.ui_factory.note('  ' + gettext('%6d revisions not present') %
+                                                    len(self.absent_revisions))
+        ui.ui_factory.note('  ' + gettext('%6d texts') % self.text_count)
+        self._cleanup_spare_files_after_format4()
+        self.branch._transport.put_bytes(
+            'branch-format',
+            BzrDirFormat5().get_format_string(),
+            mode=self.bzrdir._get_file_mode())
+
+    def _cleanup_spare_files_after_format4(self):
+        # FIXME working tree upgrade foo.
+        for n in 'merged-patches', 'pending-merged-patches':
+            try:
+                ## assert os.path.getsize(p) == 0
+                self.bzrdir.transport.delete(n)
+            except errors.NoSuchFile:
+                pass
+        self.bzrdir.transport.delete_tree('inventory-store')
+        self.bzrdir.transport.delete_tree('text-store')
+
+    def _convert_working_inv(self):
+        inv = xml4.serializer_v4.read_inventory(
+                self.branch._transport.get('inventory'))
+        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv, working=True)
+        self.branch._transport.put_bytes('inventory', new_inv_xml,
+            mode=self.bzrdir._get_file_mode())
+
+    def _write_all_weaves(self):
+        controlweaves = VersionedFileStore(self.bzrdir.transport, prefixed=False,
+            versionedfile_class=weave.WeaveFile)
+        weave_transport = self.bzrdir.transport.clone('weaves')
+        weaves = VersionedFileStore(weave_transport, prefixed=False,
+                versionedfile_class=weave.WeaveFile)
+        transaction = WriteTransaction()
+
+        try:
+            i = 0
+            for file_id, file_weave in self.text_weaves.items():
+                self.pb.update(gettext('writing weave'), i,
+                                                        len(self.text_weaves))
+                weaves._put_weave(file_id, file_weave, transaction)
+                i += 1
+            self.pb.update(gettext('inventory'), 0, 1)
+            controlweaves._put_weave('inventory', self.inv_weave, transaction)
+            self.pb.update(gettext('inventory'), 1, 1)
+        finally:
+            self.pb.clear()
+
+    def _write_all_revs(self):
+        """Write all revisions out in new form."""
+        self.bzrdir.transport.delete_tree('revision-store')
+        self.bzrdir.transport.mkdir('revision-store')
+        revision_transport = self.bzrdir.transport.clone('revision-store')
+        # TODO permissions
+        from bzrlib.xml5 import serializer_v5
+        from bzrlib.plugins.weave_fmt.repository import RevisionTextStore
+        revision_store = RevisionTextStore(revision_transport,
+            serializer_v5, False, versionedfile.PrefixMapper(),
+            lambda:True, lambda:True)
+        try:
+            for i, rev_id in enumerate(self.converted_revs):
+                self.pb.update(gettext('write revision'), i,
+                                                len(self.converted_revs))
+                text = serializer_v5.write_revision_to_string(
+                    self.revisions[rev_id])
+                key = (rev_id,)
+                revision_store.add_lines(key, None, osutils.split_lines(text))
+        finally:
+            self.pb.clear()
+
+    def _load_one_rev(self, rev_id):
+        """Load a revision object into memory.
+
+        Any parents not either loaded or abandoned get queued to be
+        loaded."""
+        self.pb.update(gettext('loading revision'),
+                       len(self.revisions),
+                       len(self.known_revisions))
+        if not self.branch.repository.has_revision(rev_id):
+            self.pb.clear()
+            ui.ui_factory.note(gettext('revision {%s} not present in branch; '
+                         'will be converted as a ghost') %
+                         rev_id)
+            self.absent_revisions.add(rev_id)
+        else:
+            rev = self.branch.repository.get_revision(rev_id)
+            for parent_id in rev.parent_ids:
+                self.known_revisions.add(parent_id)
+                self.to_read.append(parent_id)
+            self.revisions[rev_id] = rev
+
+    def _load_old_inventory(self, rev_id):
+        f = self.branch.repository.inventory_store.get(rev_id)
+        try:
+            old_inv_xml = f.read()
+        finally:
+            f.close()
+        inv = xml4.serializer_v4.read_inventory_from_string(old_inv_xml)
+        inv.revision_id = rev_id
+        rev = self.revisions[rev_id]
+        return inv
+
+    def _load_updated_inventory(self, rev_id):
+        inv_xml = self.inv_weave.get_text(rev_id)
+        inv = xml5.serializer_v5.read_inventory_from_string(inv_xml, rev_id)
+        return inv
+
+    def _convert_one_rev(self, rev_id):
+        """Convert revision and all referenced objects to new format."""
+        rev = self.revisions[rev_id]
+        inv = self._load_old_inventory(rev_id)
+        present_parents = [p for p in rev.parent_ids
+                           if p not in self.absent_revisions]
+        self._convert_revision_contents(rev, inv, present_parents)
+        self._store_new_inv(rev, inv, present_parents)
+        self.converted_revs.add(rev_id)
+
+    def _store_new_inv(self, rev, inv, present_parents):
+        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv)
+        new_inv_sha1 = osutils.sha_string(new_inv_xml)
+        self.inv_weave.add_lines(rev.revision_id,
+                                 present_parents,
+                                 new_inv_xml.splitlines(True))
+        rev.inventory_sha1 = new_inv_sha1
+
+    def _convert_revision_contents(self, rev, inv, present_parents):
+        """Convert all the files within a revision.
+
+        Also upgrade the inventory to refer to the text revision ids."""
+        rev_id = rev.revision_id
+        trace.mutter('converting texts of revision {%s}', rev_id)
+        parent_invs = map(self._load_updated_inventory, present_parents)
+        entries = inv.iter_entries()
+        entries.next()
+        for path, ie in entries:
+            self._convert_file_version(rev, ie, parent_invs)
+
+    def _convert_file_version(self, rev, ie, parent_invs):
+        """Convert one version of one file.
+
+        The file needs to be added into the weave if it is a merge
+        of >=2 parents or if it's changed from its parent.
+        """
+        file_id = ie.file_id
+        rev_id = rev.revision_id
+        w = self.text_weaves.get(file_id)
+        if w is None:
+            w = weave.Weave(file_id)
+            self.text_weaves[file_id] = w
+        text_changed = False
+        parent_candiate_entries = ie.parent_candidates(parent_invs)
+        heads = graph.Graph(self).heads(parent_candiate_entries.keys())
+        # XXX: Note that this is unordered - and this is tolerable because
+        # the previous code was also unordered.
+        previous_entries = dict((head, parent_candiate_entries[head]) for head
+            in heads)
+        self.snapshot_ie(previous_entries, ie, w, rev_id)
+
+    def get_parent_map(self, revision_ids):
+        """See graph.StackedParentsProvider.get_parent_map"""
+        return dict((revision_id, self.revisions[revision_id])
+                    for revision_id in revision_ids
+                     if revision_id in self.revisions)
+
+    def snapshot_ie(self, previous_revisions, ie, w, rev_id):
+        # TODO: convert this logic, which is ~= snapshot to
+        # a call to:. This needs the path figured out. rather than a work_tree
+        # a v4 revision_tree can be given, or something that looks enough like
+        # one to give the file content to the entry if it needs it.
+        # and we need something that looks like a weave store for snapshot to
+        # save against.
+        #ie.snapshot(rev, PATH, previous_revisions, REVISION_TREE, InMemoryWeaveStore(self.text_weaves))
+        if len(previous_revisions) == 1:
+            previous_ie = previous_revisions.values()[0]
+            if ie._unchanged(previous_ie):
+                ie.revision = previous_ie.revision
+                return
+        if ie.has_text():
+            f = self.branch.repository._text_store.get(ie.text_id)
+            try:
+                file_lines = f.readlines()
+            finally:
+                f.close()
+            w.add_lines(rev_id, previous_revisions, file_lines)
+            self.text_count += 1
+        else:
+            w.add_lines(rev_id, previous_revisions, [])
+        ie.revision = rev_id
+
+    def _make_order(self):
+        """Return a suitable order for importing revisions.
+
+        The order must be such that an revision is imported after all
+        its (present) parents.
+        """
+        todo = set(self.revisions.keys())
+        done = self.absent_revisions.copy()
+        order = []
+        while todo:
+            # scan through looking for a revision whose parents
+            # are all done
+            for rev_id in sorted(list(todo)):
+                rev = self.revisions[rev_id]
+                parent_ids = set(rev.parent_ids)
+                if parent_ids.issubset(done):
+                    # can take this one now
+                    order.append(rev_id)
+                    todo.remove(rev_id)
+                    done.add(rev_id)
+        return order
+
+
+class ConvertBzrDir5To6(Converter):
+    """Converts format 5 bzr dirs to format 6."""
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            ui.ui_factory.note(gettext('starting upgrade from format 5 to 6'))
+            self._convert_to_prefixed()
+            return BzrDir.open(self.bzrdir.user_url)
+        finally:
+            pb.finished()
+
+    def _convert_to_prefixed(self):
+        from bzrlib.store import TransportStore
+        self.bzrdir.transport.delete('branch-format')
+        for store_name in ["weaves", "revision-store"]:
+            ui.ui_factory.note(gettext("adding prefixes to %s") % store_name)
+            store_transport = self.bzrdir.transport.clone(store_name)
+            store = TransportStore(store_transport, prefixed=True)
+            for urlfilename in store_transport.list_dir('.'):
+                filename = urlutils.unescape(urlfilename)
+                if (filename.endswith(".weave") or
+                    filename.endswith(".gz") or
+                    filename.endswith(".sig")):
+                    file_id, suffix = os.path.splitext(filename)
+                else:
+                    file_id = filename
+                    suffix = ''
+                new_name = store._mapper.map((file_id,)) + suffix
+                # FIXME keep track of the dirs made RBC 20060121
+                try:
+                    store_transport.move(filename, new_name)
+                except errors.NoSuchFile: # catches missing dirs strangely enough
+                    store_transport.mkdir(osutils.dirname(new_name))
+                    store_transport.move(filename, new_name)
+        self.bzrdir.transport.put_bytes(
+            'branch-format',
+            BzrDirFormat6().get_format_string(),
+            mode=self.bzrdir._get_file_mode())
+
+
+class ConvertBzrDir6ToMeta(Converter):
+    """Converts format 6 bzr dirs to metadirs."""
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat7
+        from bzrlib.branch import BzrBranchFormat5
+        self.bzrdir = to_convert
+        self.pb = ui.ui_factory.nested_progress_bar()
+        self.count = 0
+        self.total = 20 # the steps we know about
+        self.garbage_inventories = []
+        self.dir_mode = self.bzrdir._get_dir_mode()
+        self.file_mode = self.bzrdir._get_file_mode()
+
+        ui.ui_factory.note(gettext('starting upgrade from format 6 to metadir'))
+        self.bzrdir.transport.put_bytes(
+                'branch-format',
+                "Converting to format 6",
+                mode=self.file_mode)
+        # its faster to move specific files around than to open and use the apis...
+        # first off, nuke ancestry.weave, it was never used.
+        try:
+            self.step(gettext('Removing ancestry.weave'))
+            self.bzrdir.transport.delete('ancestry.weave')
+        except errors.NoSuchFile:
+            pass
+        # find out whats there
+        self.step(gettext('Finding branch files'))
+        last_revision = self.bzrdir.open_branch().last_revision()
+        bzrcontents = self.bzrdir.transport.list_dir('.')
+        for name in bzrcontents:
+            if name.startswith('basis-inventory.'):
+                self.garbage_inventories.append(name)
+        # create new directories for repository, working tree and branch
+        repository_names = [('inventory.weave', True),
+                            ('revision-store', True),
+                            ('weaves', True)]
+        self.step(gettext('Upgrading repository') + '  ')
+        self.bzrdir.transport.mkdir('repository', mode=self.dir_mode)
+        self.make_lock('repository')
+        # we hard code the formats here because we are converting into
+        # the meta format. The meta format upgrader can take this to a
+        # future format within each component.
+        self.put_format('repository', RepositoryFormat7())
+        for entry in repository_names:
+            self.move_entry('repository', entry)
+
+        self.step(gettext('Upgrading branch') + '      ')
+        self.bzrdir.transport.mkdir('branch', mode=self.dir_mode)
+        self.make_lock('branch')
+        self.put_format('branch', BzrBranchFormat5())
+        branch_files = [('revision-history', True),
+                        ('branch-name', True),
+                        ('parent', False)]
+        for entry in branch_files:
+            self.move_entry('branch', entry)
+
+        checkout_files = [('pending-merges', True),
+                          ('inventory', True),
+                          ('stat-cache', False)]
+        # If a mandatory checkout file is not present, the branch does not have
+        # a functional checkout. Do not create a checkout in the converted
+        # branch.
+        for name, mandatory in checkout_files:
+            if mandatory and name not in bzrcontents:
+                has_checkout = False
+                break
+        else:
+            has_checkout = True
+        if not has_checkout:
+            ui.ui_factory.note(gettext('No working tree.'))
+            # If some checkout files are there, we may as well get rid of them.
+            for name, mandatory in checkout_files:
+                if name in bzrcontents:
+                    self.bzrdir.transport.delete(name)
+        else:
+            from bzrlib.workingtree_3 import WorkingTreeFormat3
+            self.step(gettext('Upgrading working tree'))
+            self.bzrdir.transport.mkdir('checkout', mode=self.dir_mode)
+            self.make_lock('checkout')
+            self.put_format(
+                'checkout', WorkingTreeFormat3())
+            self.bzrdir.transport.delete_multi(
+                self.garbage_inventories, self.pb)
+            for entry in checkout_files:
+                self.move_entry('checkout', entry)
+            if last_revision is not None:
+                self.bzrdir.transport.put_bytes(
+                    'checkout/last-revision', last_revision)
+        self.bzrdir.transport.put_bytes(
+            'branch-format',
+            BzrDirMetaFormat1().get_format_string(),
+            mode=self.file_mode)
+        self.pb.finished()
+        return BzrDir.open(self.bzrdir.user_url)
+
+    def make_lock(self, name):
+        """Make a lock for the new control dir name."""
+        self.step(gettext('Make %s lock') % name)
+        ld = lockdir.LockDir(self.bzrdir.transport,
+                             '%s/lock' % name,
+                             file_modebits=self.file_mode,
+                             dir_modebits=self.dir_mode)
+        ld.create()
+
+    def move_entry(self, new_dir, entry):
+        """Move then entry name into new_dir."""
+        name = entry[0]
+        mandatory = entry[1]
+        self.step(gettext('Moving %s') % name)
+        try:
+            self.bzrdir.transport.move(name, '%s/%s' % (new_dir, name))
+        except errors.NoSuchFile:
+            if mandatory:
+                raise
+
+    def put_format(self, dirname, format):
+        self.bzrdir.transport.put_bytes('%s/format' % dirname,
+            format.get_format_string(),
+            self.file_mode)
+
+
+class BzrDirFormat4(BzrDirFormat):
+    """Bzr dir format 4.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 1 working trees [always]
+     - Format 4 branches [always]
+     - Format 4 repositories [always]
+
+    This format is deprecated: it indexes texts using a text it which is
+    removed in format 5; write support for this format has been removed.
+    """
+
+    _lock_class = lockable_files.TransportLock
+
+    def __eq__(self, other):
+        return type(self) == type(other)
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 0.0.4\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 4"
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir4To5()
+
+    def initialize_on_transport(self, transport):
+        """Format 4 branches cannot be created."""
+        raise errors.UninitializableFormat(self)
+
+    def is_supported(self):
+        """Format 4 is not supported.
+
+        It is not supported because the model changed from 4 to 5 and the
+        conversion logic is expensive - so doing it on the fly was not
+        feasible.
+        """
+        return False
+
+    def network_name(self):
+        return self.get_format_string()
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir4(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat4
+        return RepositoryFormat4()
+    repository_format = property(__return_repository_format)
+
+
+class BzrDirPreSplitOut(BzrDir):
+    """A common class for the all-in-one formats."""
+
+    def __init__(self, _transport, _format):
+        """See BzrDir.__init__."""
+        super(BzrDirPreSplitOut, self).__init__(_transport, _format)
+        self._control_files = lockable_files.LockableFiles(
+                                            self.get_branch_transport(None),
+                                            self._format._lock_file_name,
+                                            self._format._lock_class)
+
+    def break_lock(self):
+        """Pre-splitout bzrdirs do not suffer from stale locks."""
+        raise NotImplementedError(self.break_lock)
+
+    def cloning_metadir(self, require_stacking=False):
+        """Produce a metadir suitable for cloning with."""
+        if require_stacking:
+            return format_registry.make_bzrdir('1.6')
+        return self._format.__class__()
+
+    def clone(self, url, revision_id=None, force_new_repo=False,
+              preserve_stacking=False):
+        """See BzrDir.clone().
+
+        force_new_repo has no effect, since this family of formats always
+        require a new repository.
+        preserve_stacking has no effect, since no source branch using this
+        family of formats can be stacked, so there is no stacking to preserve.
+        """
+        self._make_tail(url)
+        result = self._format._initialize_for_clone(url)
+        self.open_repository().clone(result, revision_id=revision_id)
+        from_branch = self.open_branch()
+        from_branch.clone(result, revision_id=revision_id)
+        try:
+            tree = self.open_workingtree()
+        except errors.NotLocalUrl:
+            # make a new one, this format always has to have one.
+            result._init_workingtree()
+        else:
+            tree.clone(result)
+        return result
+
+    def create_branch(self, name=None, repository=None,
+                      append_revisions_only=None):
+        """See BzrDir.create_branch."""
+        if repository is not None:
+            raise NotImplementedError(
+                "create_branch(repository=<not None>) on %r" % (self,))
+        return self._format.get_branch_format().initialize(self, name=name,
+            append_revisions_only=append_revisions_only)
+
+    def destroy_branch(self, name=None):
+        """See BzrDir.destroy_branch."""
+        raise errors.UnsupportedOperation(self.destroy_branch, self)
+
+    def create_repository(self, shared=False):
+        """See BzrDir.create_repository."""
+        if shared:
+            raise errors.IncompatibleFormat('shared repository', self._format)
+        return self.open_repository()
+
+    def destroy_repository(self):
+        """See BzrDir.destroy_repository."""
+        raise errors.UnsupportedOperation(self.destroy_repository, self)
+
+    def create_workingtree(self, revision_id=None, from_branch=None,
+                           accelerator_tree=None, hardlink=False):
+        """See BzrDir.create_workingtree."""
+        # The workingtree is sometimes created when the bzrdir is created,
+        # but not when cloning.
+
+        # this looks buggy but is not -really-
+        # because this format creates the workingtree when the bzrdir is
+        # created
+        # clone and sprout will have set the revision_id
+        # and that will have set it for us, its only
+        # specific uses of create_workingtree in isolation
+        # that can do wonky stuff here, and that only
+        # happens for creating checkouts, which cannot be
+        # done on this format anyway. So - acceptable wart.
+        if hardlink:
+            warning("can't support hardlinked working trees in %r"
+                % (self,))
+        try:
+            result = self.open_workingtree(recommend_upgrade=False)
+        except errors.NoSuchFile:
+            result = self._init_workingtree()
+        if revision_id is not None:
+            if revision_id == _mod_revision.NULL_REVISION:
+                result.set_parent_ids([])
+            else:
+                result.set_parent_ids([revision_id])
+        return result
+
+    def _init_workingtree(self):
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        try:
+            return WorkingTreeFormat2().initialize(self)
+        except errors.NotLocalUrl:
+            # Even though we can't access the working tree, we need to
+            # create its control files.
+            return WorkingTreeFormat2()._stub_initialize_on_transport(
+                self.transport, self._control_files._file_mode)
+
+    def destroy_workingtree(self):
+        """See BzrDir.destroy_workingtree."""
+        raise errors.UnsupportedOperation(self.destroy_workingtree, self)
+
+    def destroy_workingtree_metadata(self):
+        """See BzrDir.destroy_workingtree_metadata."""
+        raise errors.UnsupportedOperation(self.destroy_workingtree_metadata,
+                                          self)
+
+    def get_branch_transport(self, branch_format, name=None):
+        """See BzrDir.get_branch_transport()."""
+        if name is not None:
+            raise errors.NoColocatedBranchSupport(self)
+        if branch_format is None:
+            return self.transport
+        try:
+            branch_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(branch_format, self._format)
+
+    def get_repository_transport(self, repository_format):
+        """See BzrDir.get_repository_transport()."""
+        if repository_format is None:
+            return self.transport
+        try:
+            repository_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(repository_format, self._format)
+
+    def get_workingtree_transport(self, workingtree_format):
+        """See BzrDir.get_workingtree_transport()."""
+        if workingtree_format is None:
+            return self.transport
+        try:
+            workingtree_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(workingtree_format, self._format)
+
+    def needs_format_conversion(self, format=None):
+        """See BzrDir.needs_format_conversion()."""
+        # if the format is not the same as the system default,
+        # an upgrade is needed.
+        if format is None:
+            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
+                % 'needs_format_conversion(format=None)')
+            format = BzrDirFormat.get_default_format()
+        return not isinstance(self._format, format.__class__)
+
+    def open_branch(self, name=None, unsupported=False,
+                    ignore_fallbacks=False):
+        """See BzrDir.open_branch."""
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        format = BzrBranchFormat4()
+        format.check_support_status(unsupported)
+        return format.open(self, name, _found=True)
+
+    def sprout(self, url, revision_id=None, force_new_repo=False,
+               possible_transports=None, accelerator_tree=None,
+               hardlink=False, stacked=False, create_tree_if_local=True,
+               source_branch=None):
+        """See BzrDir.sprout()."""
+        if source_branch is not None:
+            my_branch = self.open_branch()
+            if source_branch.base != my_branch.base:
+                raise AssertionError(
+                    "source branch %r is not within %r with branch %r" %
+                    (source_branch, self, my_branch))
+        if stacked:
+            raise errors.UnstackableBranchFormat(
+                self._format, self.root_transport.base)
+        if not create_tree_if_local:
+            raise errors.MustHaveWorkingTree(
+                self._format, self.root_transport.base)
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        self._make_tail(url)
+        result = self._format._initialize_for_clone(url)
+        try:
+            self.open_repository().clone(result, revision_id=revision_id)
+        except errors.NoRepositoryPresent:
+            pass
+        try:
+            self.open_branch().sprout(result, revision_id=revision_id)
+        except errors.NotBranchError:
+            pass
+
+        # we always want a working tree
+        WorkingTreeFormat2().initialize(result,
+                                        accelerator_tree=accelerator_tree,
+                                        hardlink=hardlink)
+        return result
+
+
+class BzrDir4(BzrDirPreSplitOut):
+    """A .bzr version 4 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def create_repository(self, shared=False):
+        """See BzrDir.create_repository."""
+        return self._format.repository_format.initialize(self, shared)
+
+    def needs_format_conversion(self, format=None):
+        """Format 4 dirs are always in need of conversion."""
+        if format is None:
+            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
+                % 'needs_format_conversion(format=None)')
+        return True
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat4
+        return RepositoryFormat4().open(self, _found=True)
+
+
+class BzrDir5(BzrDirPreSplitOut):
+    """A .bzr version 5 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def has_workingtree(self):
+        """See BzrDir.has_workingtree."""
+        return True
+    
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
+        return RepositoryFormat5().open(self, _found=True)
+
+    def open_workingtree(self, _unsupported=False,
+            recommend_upgrade=True):
+        """See BzrDir.create_workingtree."""
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        wt_format = WorkingTreeFormat2()
+        # we don't warn here about upgrades; that ought to be handled for the
+        # bzrdir as a whole
+        return wt_format.open(self, _found=True)
+
+
+class BzrDir6(BzrDirPreSplitOut):
+    """A .bzr version 6 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def has_workingtree(self):
+        """See BzrDir.has_workingtree."""
+        return True
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
+        return RepositoryFormat6().open(self, _found=True)
+
+    def open_workingtree(self, _unsupported=False,
+        recommend_upgrade=True):
+        """See BzrDir.create_workingtree."""
+        # we don't warn here about upgrades; that ought to be handled for the
+        # bzrdir as a whole
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        return WorkingTreeFormat2().open(self, _found=True)

=== renamed file 'bzrlib/repofmt/weaverepo.py' => 'bzrlib/plugins/weave_fmt/repository.py'
--- bzrlib/repofmt/weaverepo.py	2010-05-14 12:55:13 +0000
+++ bzrlib/plugins/weave_fmt/repository.py	2011-09-22 00:44:49 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,26 +20,30 @@
 ghosts.
 """
 
+import gzip
 import os
 from cStringIO import StringIO
 import urllib
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import itertools
+
 from bzrlib import (
     xml5,
     graph as _mod_graph,
+    ui,
     )
 """)
 from bzrlib import (
-    bzrdir,
     debug,
     errors,
     lockable_files,
     lockdir,
     osutils,
-    revision as _mod_revision,
+    symbol_versioning,
     trace,
+    tuned_gzip,
     urlutils,
     versionedfile,
     weave,
@@ -47,22 +51,28 @@
     )
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
-    CommitBuilder,
-    MetaDirVersionedFileRepository,
-    MetaDirRepositoryFormat,
-    Repository,
+    InterRepository,
     RepositoryFormat,
     )
 from bzrlib.store.text import TextStore
-from bzrlib.tuned_gzip import GzipFile, bytes_to_gzip
 from bzrlib.versionedfile import (
     AbsentContentFactory,
     FulltextContentFactory,
     VersionedFiles,
     )
-
-
-class AllInOneRepository(Repository):
+from bzrlib.vf_repository import (
+    InterSameDataRepository,
+    VersionedFileCommitBuilder,
+    VersionedFileRepository,
+    VersionedFileRepositoryFormat,
+    MetaDirVersionedFileRepository,
+    MetaDirVersionedFileRepositoryFormat,
+    )
+
+from bzrlib.plugins.weave_fmt import bzrdir as weave_bzrdir
+
+
+class AllInOneRepository(VersionedFileRepository):
     """Legacy support - the repository behaviour for all-in-one branches."""
 
     @property
@@ -140,10 +150,10 @@
 
     def get_commit_builder(self, branch, parents, config, timestamp=None,
                            timezone=None, committer=None, revprops=None,
-                           revision_id=None):
+                           revision_id=None, lossy=False):
         self._check_ascii_revisionid(revision_id, self.get_commit_builder)
-        result = CommitBuilder(self, parents, config, timestamp, timezone,
-                              committer, revprops, revision_id)
+        result = VersionedFileCommitBuilder(self, parents, config, timestamp,
+            timezone, committer, revprops, revision_id, lossy=lossy)
         self.start_write_group()
         return result
 
@@ -183,11 +193,6 @@
         """Returns the policy for making working trees on new branches."""
         return True
 
-    def revision_graph_can_have_wrong_parents(self):
-        # XXX: This is an old format that we don't support full checking on, so
-        # just claim that checking for this inconsistency is not required.
-        return False
-
 
 class WeaveMetaDirRepository(MetaDirVersionedFileRepository):
     """A subclass of MetaDirRepository to set weave specific policy."""
@@ -234,10 +239,10 @@
 
     def get_commit_builder(self, branch, parents, config, timestamp=None,
                            timezone=None, committer=None, revprops=None,
-                           revision_id=None):
+                           revision_id=None, lossy=False):
         self._check_ascii_revisionid(revision_id, self.get_commit_builder)
-        result = CommitBuilder(self, parents, config, timestamp, timezone,
-                              committer, revprops, revision_id)
+        result = VersionedFileCommitBuilder(self, parents, config, timestamp,
+            timezone, committer, revprops, revision_id, lossy=lossy)
         self.start_write_group()
         return result
 
@@ -258,11 +263,8 @@
         return self.inventories.add_lines((revision_id,), final_parents, lines,
             check_content=check_content)[0]
 
-    def revision_graph_can_have_wrong_parents(self):
-        return False
-
-
-class PreSplitOutRepositoryFormat(RepositoryFormat):
+
+class PreSplitOutRepositoryFormat(VersionedFileRepositoryFormat):
     """Base class for the pre split out repository formats."""
 
     rich_root_data = False
@@ -270,9 +272,14 @@
     supports_ghosts = False
     supports_external_lookups = False
     supports_chks = False
+    supports_nesting_repositories = True
     _fetch_order = 'topological'
     _fetch_reconcile = True
     fast_deltas = False
+    supports_leaving_lock = False
+    # XXX: This is an old format that we don't support full checking on, so
+    # just claim that checking for this inconsistency is not required.
+    revision_graph_can_have_wrong_parents = False
 
     def initialize(self, a_bzrdir, shared=False, _internal=False):
         """Create a weave repository."""
@@ -315,7 +322,6 @@
             raise NotImplementedError
 
         repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = a_bzrdir._control_files
         result = AllInOneRepository(_format=self, a_bzrdir=a_bzrdir)
         result.revisions = self._get_revisions(repo_transport, result)
         result.signatures = self._get_signatures(repo_transport, result)
@@ -324,6 +330,9 @@
         result.chk_bytes = None
         return result
 
+    def is_deprecated(self):
+        return True
+
 
 class RepositoryFormat4(PreSplitOutRepositoryFormat):
     """Bzr repository format 4.
@@ -337,7 +346,9 @@
     has been removed.
     """
 
-    _matchingbzrdir = bzrdir.BzrDirFormat4()
+    supports_funky_characters = False
+
+    _matchingbzrdir = weave_bzrdir.BzrDirFormat4()
 
     def get_format_description(self):
         """See RepositoryFormat.get_format_description()."""
@@ -361,7 +372,7 @@
         return None
 
     def _get_revisions(self, repo_transport, repo):
-        from bzrlib.xml4 import serializer_v4
+        from bzrlib.plugins.weave_fmt.xml4 import serializer_v4
         return RevisionTextStore(repo_transport.clone('revision-store'),
             serializer_v4, True, versionedfile.PrefixMapper(),
             repo.is_locked, repo.is_write_locked)
@@ -385,7 +396,9 @@
     """
 
     _versionedfile_class = weave.WeaveFile
-    _matchingbzrdir = bzrdir.BzrDirFormat5()
+    _matchingbzrdir = weave_bzrdir.BzrDirFormat5()
+    supports_funky_characters = False
+
     @property
     def _serializer(self):
         return xml5.serializer_v5
@@ -430,7 +443,8 @@
     """
 
     _versionedfile_class = weave.WeaveFile
-    _matchingbzrdir = bzrdir.BzrDirFormat6()
+    _matchingbzrdir = weave_bzrdir.BzrDirFormat6()
+    supports_funky_characters = False
     @property
     def _serializer(self):
         return xml5.serializer_v5
@@ -465,7 +479,7 @@
             weave.WeaveFile, mapper, repo.is_locked)
 
 
-class RepositoryFormat7(MetaDirRepositoryFormat):
+class RepositoryFormat7(MetaDirVersionedFileRepositoryFormat):
     """Bzr repository 7.
 
     This repository format has:
@@ -480,6 +494,8 @@
     _versionedfile_class = weave.WeaveFile
     supports_ghosts = False
     supports_chks = False
+    supports_funky_characters = False
+    revision_graph_can_have_wrong_parents = False
 
     _fetch_order = 'topological'
     _fetch_reconcile = True
@@ -562,6 +578,9 @@
         result._transport = repo_transport
         return result
 
+    def is_deprecated(self):
+        return True
+
 
 class TextVersionedFiles(VersionedFiles):
     """Just-a-bunch-of-files based VersionedFile stores."""
@@ -587,7 +606,7 @@
             raise ValueError('bad idea to put / in %r' % (key,))
         text = ''.join(lines)
         if self._compressed:
-            text = bytes_to_gzip(text)
+            text = tuned_gzip.bytes_to_gzip(text)
         path = self._map(key)
         self._transport.put_bytes_non_atomic(path, text, create_parent_dir=True)
 
@@ -613,7 +632,7 @@
                     record, record.get_bytes_as(record.storage_kind)))
                 try:
                     self.add_lines(record.key, None, lines)
-                except RevisionAlreadyPresent:
+                except errors.RevisionAlreadyPresent:
                     pass
 
     def _load_text(self, key):
@@ -635,7 +654,7 @@
             else:
                 return None
         if compressed:
-            text = GzipFile(mode='rb', fileobj=StringIO(text)).read()
+            text = gzip.GzipFile(mode='rb', fileobj=StringIO(text)).read()
         return text
 
     def _map(self, key):
@@ -738,6 +757,125 @@
         paths = list(relpaths)
         return set([self._mapper.unmap(path) for path in paths])
 
-_legacy_formats = [RepositoryFormat4(),
-                   RepositoryFormat5(),
-                   RepositoryFormat6()]
+
+class InterWeaveRepo(InterSameDataRepository):
+    """Optimised code paths between Weave based repositories.
+    """
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return RepositoryFormat7()
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with known Weave formats.
+
+        We don't test for the stores being of specific types because that
+        could lead to confusing results, and there is no need to be
+        overly general.
+        """
+        try:
+            return (isinstance(source._format, (RepositoryFormat5,
+                                                RepositoryFormat6,
+                                                RepositoryFormat7)) and
+                    isinstance(target._format, (RepositoryFormat5,
+                                                RepositoryFormat6,
+                                                RepositoryFormat7)))
+        except AttributeError:
+            return False
+
+    @needs_write_lock
+    def copy_content(self, revision_id=None):
+        """See InterRepository.copy_content()."""
+        # weave specific optimised path:
+        try:
+            self.target.set_make_working_trees(self.source.make_working_trees())
+        except (errors.RepositoryUpgradeRequired, NotImplemented):
+            pass
+        # FIXME do not peek!
+        if self.source._transport.listable():
+            pb = ui.ui_factory.nested_progress_bar()
+            try:
+                self.target.texts.insert_record_stream(
+                    self.source.texts.get_record_stream(
+                        self.source.texts.keys(), 'topological', False))
+                pb.update('Copying inventory', 0, 1)
+                self.target.inventories.insert_record_stream(
+                    self.source.inventories.get_record_stream(
+                        self.source.inventories.keys(), 'topological', False))
+                self.target.signatures.insert_record_stream(
+                    self.source.signatures.get_record_stream(
+                        self.source.signatures.keys(),
+                        'unordered', True))
+                self.target.revisions.insert_record_stream(
+                    self.source.revisions.get_record_stream(
+                        self.source.revisions.keys(),
+                        'topological', True))
+            finally:
+                pb.finished()
+        else:
+            self.target.fetch(self.source, revision_id=revision_id)
+
+    @needs_read_lock
+    def search_missing_revision_ids(self,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None,
+            limit=None):
+        """See InterRepository.search_missing_revision_ids()."""
+        # we want all revisions to satisfy revision_id in source.
+        # but we don't want to stat every file here and there.
+        # we want then, all revisions other needs to satisfy revision_id
+        # checked, but not those that we have locally.
+        # so the first thing is to get a subset of the revisions to
+        # satisfy revision_id in source, and then eliminate those that
+        # we do already have.
+        # this is slow on high latency connection to self, but as this
+        # disk format scales terribly for push anyway due to rewriting
+        # inventory.weave, this is considered acceptable.
+        # - RBC 20060209
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.4.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        del revision_id
+        source_ids_set = self._present_source_revisions_for(
+            revision_ids, if_present_ids)
+        # source_ids is the worst possible case we may need to pull.
+        # now we want to filter source_ids against what we actually
+        # have in target, but don't try to check for existence where we know
+        # we do not have a revision as that would be pointless.
+        target_ids = set(self.target._all_possible_ids())
+        possibly_present_revisions = target_ids.intersection(source_ids_set)
+        actually_present_revisions = set(
+            self.target._eliminate_revisions_not_present(possibly_present_revisions))
+        required_revisions = source_ids_set.difference(actually_present_revisions)
+        if revision_ids is not None:
+            # we used get_ancestry to determine source_ids then we are assured all
+            # revisions referenced are present as they are installed in topological order.
+            # and the tip revision was validated by get_ancestry.
+            result_set = required_revisions
+        else:
+            # if we just grabbed the possibly available ids, then
+            # we only have an estimate of whats available and need to validate
+            # that against the revision records.
+            result_set = set(
+                self.source._eliminate_revisions_not_present(required_revisions))
+        if limit is not None:
+            topo_ordered = self.get_graph().iter_topo_order(result_set)
+            result_set = set(itertools.islice(topo_ordered, limit))
+        return self.source.revision_ids_to_search_result(result_set)
+
+
+InterRepository.register_optimiser(InterWeaveRepo)
+
+
+def get_extra_interrepo_test_combinations():
+    from bzrlib.repofmt import knitrepo
+    return [(InterRepository, RepositoryFormat5(),
+        knitrepo.RepositoryFormatKnit3())]

=== added file 'bzrlib/plugins/weave_fmt/test_bzrdir.py'
--- bzrlib/plugins/weave_fmt/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/test_bzrdir.py	2011-10-06 00:48:21 +0000
@@ -0,0 +1,581 @@
+# Copyright (C) 2006-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for the weave-era BzrDir formats.
+
+For interface contract tests, see tests/per_bzr_dir.
+"""
+
+import os
+import sys
+
+from bzrlib import (
+    branch,
+    bzrdir,
+    errors,
+    repository,
+    upgrade,
+    urlutils,
+    workingtree,
+    )
+from bzrlib.osutils import (
+    getcwd,
+    )
+from bzrlib.tests.test_bundle import V4BundleTester
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.tests import (
+    TestCaseWithTransport,
+    )
+
+from bzrlib.plugins.weave_fmt.branch import (
+    BzrBranchFormat4,
+    )
+from bzrlib.plugins.weave_fmt.bzrdir import (
+    BzrDirFormat5,
+    BzrDirFormat6,
+    )
+
+
+class TestFormat5(TestCaseWithTransport):
+    """Tests specific to the version 5 bzrdir format."""
+
+    def test_same_lockfiles_between_tree_repo_branch(self):
+        # this checks that only a single lockfiles instance is created
+        # for format 5 objects
+        dir = BzrDirFormat5().initialize(self.get_url())
+        def check_dir_components_use_same_lock(dir):
+            ctrl_1 = dir.open_repository().control_files
+            ctrl_2 = dir.open_branch().control_files
+            ctrl_3 = dir.open_workingtree()._control_files
+            self.assertTrue(ctrl_1 is ctrl_2)
+            self.assertTrue(ctrl_2 is ctrl_3)
+        check_dir_components_use_same_lock(dir)
+        # and if we open it normally.
+        dir = bzrdir.BzrDir.open(self.get_url())
+        check_dir_components_use_same_lock(dir)
+
+    def test_can_convert(self):
+        # format 5 dirs are convertable
+        dir = BzrDirFormat5().initialize(self.get_url())
+        self.assertTrue(dir.can_convert_format())
+
+    def test_needs_conversion(self):
+        # format 5 dirs need a conversion if they are not the default,
+        # and they aren't
+        dir = BzrDirFormat5().initialize(self.get_url())
+        # don't need to convert it to itself
+        self.assertFalse(dir.needs_format_conversion(BzrDirFormat5()))
+        # do need to convert it to the current default
+        self.assertTrue(dir.needs_format_conversion(
+            bzrdir.BzrDirFormat.get_default_format()))
+
+
+class TestFormat6(TestCaseWithTransport):
+    """Tests specific to the version 6 bzrdir format."""
+
+    def test_same_lockfiles_between_tree_repo_branch(self):
+        # this checks that only a single lockfiles instance is created
+        # for format 6 objects
+        dir = BzrDirFormat6().initialize(self.get_url())
+        def check_dir_components_use_same_lock(dir):
+            ctrl_1 = dir.open_repository().control_files
+            ctrl_2 = dir.open_branch().control_files
+            ctrl_3 = dir.open_workingtree()._control_files
+            self.assertTrue(ctrl_1 is ctrl_2)
+            self.assertTrue(ctrl_2 is ctrl_3)
+        check_dir_components_use_same_lock(dir)
+        # and if we open it normally.
+        dir = bzrdir.BzrDir.open(self.get_url())
+        check_dir_components_use_same_lock(dir)
+
+    def test_can_convert(self):
+        # format 6 dirs are convertable
+        dir = BzrDirFormat6().initialize(self.get_url())
+        self.assertTrue(dir.can_convert_format())
+
+    def test_needs_conversion(self):
+        # format 6 dirs need an conversion if they are not the default.
+        dir = BzrDirFormat6().initialize(self.get_url())
+        self.assertTrue(dir.needs_format_conversion(
+            bzrdir.BzrDirFormat.get_default_format()))
+
+
+class TestBreakLockOldBranch(TestCaseWithTransport):
+
+    def test_break_lock_format_5_bzrdir(self):
+        # break lock on a format 5 bzrdir should just return
+        self.make_branch_and_tree('foo', format=BzrDirFormat5())
+        out, err = self.run_bzr('break-lock foo')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
+
+_upgrade1_template = \
+     [
+     ('foo', 'new contents\n'),
+     ('.bzr/',),
+     ('.bzr/README',
+      'This is a Bazaar control directory.\n'
+      'Do not change any files in this directory.\n'
+      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'),
+     ('.bzr/branch-format', 'Bazaar-NG branch, format 0.0.4\n'),
+     ('.bzr/revision-history',
+      'mbp@sourcefrog.net-20051004035611-176b16534b086b3c\n'
+      'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd\n'),
+     ('.bzr/merged-patches', ''),
+     ('.bzr/pending-merged-patches', ''),
+     ('.bzr/branch-name', ''),
+     ('.bzr/branch-lock', ''),
+     ('.bzr/pending-merges', ''),
+     ('.bzr/inventory',
+      '<inventory>\n'
+      '<entry file_id="foo-20051004035605-91e788d1875603ae" kind="file" name="foo" />\n'
+      '</inventory>\n'),
+     ('.bzr/stat-cache',
+      '### bzr hashcache v5\n'
+      'foo// be9f309239729f69a6309e970ef24941d31e042c 13 1128398176 1128398176 303464 770\n'),
+     ('.bzr/text-store/',),
+     ('.bzr/text-store/foo-20051004035611-1591048e9dc7c2d4.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\xcb\xcc\xcb,\xc9L\xccQH\xce\xcf+I\xcd+)\xe6\x02\x00\xdd\xcc\xf90\x11\x00\x00\x00'),
+     ('.bzr/text-store/foo-20051004035756-4081373d897c3453.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\xcbK-WH\xce\xcf+I\xcd+)\xe6\x02\x00g\xc3\xdf\xc9\r\x00\x00\x00'),
+     ('.bzr/inventory-store/',),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xffm\x8f\xcd\n\xc20\x10\x84\xef>E\xc8\xbdt7?M\x02\xad\xaf"\xa1\x99`P[\xa8E\xacOo\x14\x05\x0f\xdef\xe1\xfbv\x98\xbeL7L\xeb\xbcl\xfb]_\xc3\xb2\x89\\\xce8\x944\xc8<\xcf\x8d"\xb2LdH\xdb\x8el\x13\x18\xce\xfb\xc4\xde\xd5SGHq*\xd3\x0b\xad\x8e\x14S\xbc\xe0\xadI\xb1\xe2\xbe\xfe}\xc2\xdc\xb0\rL\xc6#\xa4\xd1\x8d*\x99\x0f}=F\x1e$8G\x9d\xa0\x02\xa1rP9\x01c`FV\xda1qg\x98"\x02}\xa5\xf2\xa8\x95\xec\xa4h\xeb\x80\xf6g\xcd\x13\xb3\x01\xcc\x98\xda\x00\x00\x00'),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xffm\x8f\xc1\n\xc20\x10D\xef~E\xc8\xbd\xb8\x9bM\x9a,\xb4\xfe\x8a\xc4f\x83Am\xa1\x16\xb1~\xbdQ\x14<x\x9b\x81y3LW\xc6\x9b\x8c\xcb4\xaf\xbbMW\xc5\xbc\xaa\\\xce\xb2/\xa9\xd7y\x9a\x1a\x03\xe0\x10\xc0\x02\xb9\x16\\\xc3(>\x84\x84\xc1WKQ\xb4:\x95\xf1\x15\xad\x8cVc\xbc\xc8\x1b\xd3j\x91\xfb\xf2\xaf\xa4r\x8d\x85\x80\xe4)\x05\xf6\x03YG\x9f\xf4\xf5\x18\xb1\xd7\x07\xe1L\xc0\x86\xd8\x1b\xce-\xc7\xb6:a\x0f\x92\x8de\x8b\x89P\xc0\x9a\xe1\x0b\x95G\x9d\xc4\xda\xb1\xad\x07\xb6?o\x9e\xb5\xff\xf0\xf9\xda\x00\x00\x00'),
+     ('.bzr/revision-store/',),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\x9d\x8eKj\xc30\x14E\xe7^\x85\xd0 \xb3$\xefI\xd1\x8f\xd8\xa6\x1b(t\x07E?\xbb\x82H\n\xb2\x1ahW\xdfB1\x14:\xeb\xf4r\xee\xbdgl\xf1\x91\xb6T\x0b\xf15\xe7\xd4{l\x13}\xb6\xad\xa7B^j\xbd\x91\xc3\xad_\xb3\xbb?m\xf5\xbd\xf9\xb8\xb4\xba\x9eJ\xec\x87\xb5_)I\xe5\x11K\xaf\xed\xe35\x85\x89\xfe\xa5\x8e\x0c@ \xc0\x05\xb8\x90\x88GT\xd2\xa1\x14\xfc\xe2@K\xc7\xfd\xef\x85\xed\xcd\xe2D\x95\x8d\x1a\xa47<\x02c2\xb0 \xbc\xd0\x8ay\xa3\xbcp\x8a\x83\x12A3\xb7XJv\xef\x7f_\xf7\x94\xe3\xd6m\xbeO\x14\x91in4*<\x812\x88\xc60\xfc\x01>k\x89\x13\xe5\x12\x00\xe8<\x8c\xdf\x8d\xcd\xaeq\xb6!\x90\xa5\xd6\xf1\xbc\x07\xc3x\xde\x85\xe6\xe1\x0b\xc8\x8a\x98\x03T\x01\x00\x00'),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\x9d\x90Kj\x031\x0c\x86\xf79\xc5\xe0Ev\xe9\xc8o\x9b\xcc\x84^\xa0\xd0\x1b\x14\xbf&5d\xec`\xbb\x81\xf6\xf45\x84\xa4\x81\xaeZ\xa1\x85\x84^\xdf\xaf\xa9\x84K\xac1\xa7\xc1\xe5u\x8d\xad\x852\xa3\x17SZL\xc3k\xce\xa7a{j\xfb\xd5\x9e\x9fk\xfe(.,%\x1f\x9fRh\xdbc\xdb\xa3!\xa6KH-\x97\xcf\xb7\xe8g\xf4\xbbkG\x008\x06`@\xb9\xe4bG(_\x88\x95\xde\xf9n\xca\xfb\xc7\r\xf5\xdd\xe0\x19\xa9\x85)\x81\xf5"\xbd\x04j\xb8\x02b\xa8W\\\x0b\xc9\x14\xf4\xbc\xbb\xd7\xd6H4\xdc\xb8\xff}\xba\xc55\xd4f\xd6\xf3\x8c0&\x8ajE\xa4x\xe2@\xa5\xa6\x9a\xf3k\xc3WNaFT\x00\x00:l\xa6>Q\xcd1\x1cjp9\xf9;\xc34\xde\n\x9b\xe9lJWT{t\',a\xf9\x0b\xae\xc0x\x87\xa5\xb0Xp\xca,(a\xa9{\xd0{}\xd4\x12\x04(\xc5\xbb$\xc5$V\xceaI\x19\x01\xa2\x1dh\xed\x82d\x8c.\xccr@\xc3\xd8Q\xc6\x1f\xaa\xf1\xb6\xe8\xb0\xf9\x06QR\r\xf9\xfc\x01\x00\x00')]
+
+
+_ghost_template = [
+    ( './foo',
+        'hello\n'
+    ),
+    ( './.bzr/', ),
+    ( './.bzr/README',
+      'This is a Bazaar control directory.\n'
+      'Do not change any files in this directory.\n'
+      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="foo-20051004104918-0379cb7c76354cde" kind="file" name="foo" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\n'
+        'mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+        'foo// f572d396fae9206628714fb2ce00f72e94f2258f 6 1128422956 1128422956 306900 770\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/text-store/foo-20051004104921-8de8118a71be45ba.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03foo-20051004104921-8de8118a71be45ba\x00\xcbH\xcd\xc9\xc9\xe7\x02\x00 0:6\x06\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00\x9d\x8eMj\xc30\x14\x84\xf7>\x85\xd0"\xbb$\xef\xc9\xb6,\x11\xdb\xf4\x02\x85\xde\xa0\xe8\xe7\xd9\x11\xc4R\x90\xd4@{\xfa\x06\x8a\xa1\xd0]\x97\x03\xdf\xcc|c\xa6G(!E\xe6\xd2\xb6\x85Z)O\xfc\xd5\xe4\x1a"{K\xe9\xc6\x0e\xb7z\xd9\xec\xfd\xa5\xa4\x8f\xech\xc9i=E\xaa\x87\xb5^8\x0b\xf1A\xb1\xa6\xfc\xf9\x1e\xfc\xc4\xffRG\x01\xd0#@\x87\xd0i\x81G\xa3\x95%!\x06\xe5}\x0bv\xb0\xbf\x17\xca\xd5\xe0\xc4-\xa0\xb1\x8b\xb6`\xc0I\xa4\xc5\xf4\x9el\xef\x95v [\x94\xcf\x8e\xd5\xcay\xe4l\xf7\xfe\xf7u\r'
+        '\x1b\x95j\xb6\xfb\xc4\x11\x85\xea\x84\xd0\x12O\x03t\x83D\xad\xc4\x0f\xf0\x95"M\xbc\x95\x00\xc0\xe7f|6\x8aYi^B.u<\xef\xb1\x19\xcf\xbb\xce\xdc|\x038=\xc7\xe6R\x01\x00\x00'
+    ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00\x9d\x90\xc1j\xc30\x0c\x86\xef}\n'
+        "\xe3Coie'\xb1c\x9a\x94\xbe\xc0`o0,[N\x03M\\\x1c\xafe{\xfae\x94n\x85\xc1`;Y\x88O\xd2\xff\xb9Mt\x19\xe6!N\xcc\xc5q\x1cr\xa6\xd4\xf1'\x9b\xf20\xb1\xe7\x18Ol}\xca\xbb\x11\xcf\x879\xbe&G!\xc5~3Q^\xf7y\xc7\xd90]h\xca1\xbd\xbd\x0c\xbe\xe3?\xa9B\x02\xd4\x02\xa0\x12P\x99R\x17\xce\xa0\xb6\x1a\x83s\x80(\xa5\x7f\xdc0\x1f\xad\xe88\x82\xb0\x18\x0c\x82\x05\xa7\x04\x05[{\xc2\xda7\xc6\x81*\x85B\x8dh\x1a\xe7\x05g\xf7\xdc\xff>\x9d\x87\x91\xe6l\xc7s\xc7\x85\x90M%\xa5\xd1z#\x85\xa8\x9b\x1a\xaa\xfa\x06\xbc\xc7\x89:^*\x00\xe0\xfbU\xbbL\xcc\xb6\xa7\xfdH\xa9'\x16\x03\xeb\x8fq\xce\xed\xf6\xde_\xb5g\x9b\x16\xa1y\xa9\xbe\x02&\n"
+        '\x7fJ+EaM\x83$\xa5n\xbc/a\x91~\xd0\xbd\xfd\x135\n'
+        '\xd0\x9a`\x0c*W\x1aR\xc1\x94du\x08(\t\xb0\x91\xdeZ\xa3\x9cU\x9cm\x7f\x8dr\x1d\x10Ot\xb8\xc6\xcf\xa7\x907|\xfb-\xb1\xbd\xd3\xfb\xd5\x07\xeeD\xee\x08*\x02\x00\x00'
+    ),
+]
+
+_upgrade_dir_template = [
+    ( './.bzr/', ),
+    ( './.bzr/README',
+      'This is a Bazaar control directory.\n'
+      'Do not change any files in this directory.\n'
+      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="dir-20051005095101-da1441ea3fa6917a" kind="directory" name="dir" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xb3\xc9\xcc+K\xcd+\xc9/\xaa\xb4\xe3\xb2\x012\x8a*\x15\xd22sR\xe33Sl\x95R2\x8bt\x8d\x0c\x0cL\r'
+        "\x81\xd8\xc0\x12H\x19\xea\xa6$\x1a\x9a\x98\x18\xa6&\x1a\xa7%\x9aY\x1a\x9a'*)dg\xe6A\x94\xa6&\x83LQR\xc8K\xccM\x05\x0b()\xe8\x03\xcd\xd4G\xb2\x00\x00\xc2<\x94\xb1m\x00\x00\x00"
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xa5OKj\xc30\x14\xdc\xfb\x14B\x8b\xec\x92<I\xd6\xc7\xc42\x85\xde\xa0\x17(\xb6\xf4\x9c\n'
+        'l\xa9H"\x90\x9c\xbe\xa6\xa9\xa1\x9b\xae\xbax\x0c\xcc\xe71\xd3g\xbc\x85\x12R$.\xadk\xa8\x15\xb3\xa5oi\xc2\\\xc9kZ\x96\x10\x0b9,\xf5\x92\xbf)\xf7\xf2\x83O\xe5\x14\xb1\x1e\xae\xf5BI\x887\x8c5\xe5\xfb{\xf0\x96\xfei>r\x00\xc9\xb6\x83n\x03sT\xa0\xe4<y\x83\xda\x1b\xc54\xfe~T>Ff\xe9\xcc:\xdd\x8e\xa6E\xc7@\xa2\x82I\xaaNL\xbas\\313)\x00\xb9\xe6\xe0(\xd9\x87\xfc\xb7A\r'
+        "+\x96:\xae\x9f\x962\xc6\x8d\x04i\x949\x01\x97R\xb7\x1d\x17O\xc3#E\xb4T(\x00\xa0C\xd3o\x892^q\x18\xbd'>\xe4\xfe\xbc\x13M\x7f\xde{\r"
+        '\xcd\x17\x85\xea\xba\x03l\x01\x00\x00'
+    ),
+    ( './dir/', ),
+]
+
+
+class TestUpgrade(TestCaseWithTransport):
+
+    def test_upgrade_v6_to_meta_no_workingtree(self):
+        # Some format6 branches do not have checkout files. Upgrading
+        # such a branch to metadir must not setup a working tree.
+        self.build_tree_contents(_upgrade1_template)
+        upgrade.upgrade('.', BzrDirFormat6())
+        t = self.get_transport('.')
+        t.delete_multi(['.bzr/pending-merges', '.bzr/inventory'])
+        self.assertFalse(t.has('.bzr/stat-cache'))
+        t.delete_tree('backup.bzr.~1~')
+        # At this point, we have a format6 branch without checkout files.
+        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
+        # The upgrade should not have set up a working tree.
+        control = bzrdir.BzrDir.open('.')
+        self.assertFalse(control.has_workingtree())
+        # We have covered the scope of this test, we may as well check that
+        # upgrade has not eaten our data, even if it's a bit redundant with
+        # other tests.
+        self.assertIsInstance(control._format, bzrdir.BzrDirMetaFormat1)
+        b = control.open_branch()
+        self.addCleanup(b.lock_read().unlock)
+        self.assertEquals(b._revision_history(),
+           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
+            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+
+    def test_upgrade_simple(self):
+        """Upgrade simple v0.0.4 format to latest format"""
+        eq = self.assertEquals
+        self.build_tree_contents(_upgrade1_template)
+        upgrade.upgrade(u'.')
+        control = bzrdir.BzrDir.open('.')
+        b = control.open_branch()
+        # tsk, peeking under the covers.
+        self.assertIsInstance(
+            control._format,
+            bzrdir.BzrDirFormat.get_default_format().__class__)
+        self.addCleanup(b.lock_read().unlock)
+        rh = b._revision_history()
+        eq(rh,
+           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
+            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+        rt = b.repository.revision_tree(rh[0])
+        foo_id = 'foo-20051004035605-91e788d1875603ae'
+        rt.lock_read()
+        try:
+            eq(rt.get_file_text(foo_id), 'initial contents\n')
+        finally:
+            rt.unlock()
+        rt = b.repository.revision_tree(rh[1])
+        rt.lock_read()
+        try:
+            eq(rt.get_file_text(foo_id), 'new contents\n')
+        finally:
+            rt.unlock()
+        # check a backup was made:
+        backup_dir = 'backup.bzr.~1~'
+        t = self.get_transport('.')
+        t.stat(backup_dir)
+        t.stat(backup_dir + '/README')
+        t.stat(backup_dir + '/branch-format')
+        t.stat(backup_dir + '/revision-history')
+        t.stat(backup_dir + '/merged-patches')
+        t.stat(backup_dir + '/pending-merged-patches')
+        t.stat(backup_dir + '/pending-merges')
+        t.stat(backup_dir + '/branch-name')
+        t.stat(backup_dir + '/branch-lock')
+        t.stat(backup_dir + '/inventory')
+        t.stat(backup_dir + '/stat-cache')
+        t.stat(backup_dir + '/text-store')
+        t.stat(backup_dir + '/text-store/foo-20051004035611-1591048e9dc7c2d4.gz')
+        t.stat(backup_dir + '/text-store/foo-20051004035756-4081373d897c3453.gz')
+        t.stat(backup_dir + '/inventory-store/')
+        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+        t.stat(backup_dir + '/revision-store/')
+        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+
+    def test_upgrade_with_ghosts(self):
+        """Upgrade v0.0.4 tree containing ghost references.
+
+        That is, some of the parents of revisions mentioned in the branch
+        aren't present in the branch's storage.
+
+        This shouldn't normally happen in branches created entirely in
+        bzr, but can happen in branches imported from baz and arch, or from
+        other systems, where the importer knows about a revision but not
+        its contents."""
+        eq = self.assertEquals
+        self.build_tree_contents(_ghost_template)
+        upgrade.upgrade(u'.')
+        b = branch.Branch.open(u'.')
+        self.addCleanup(b.lock_read().unlock)
+        revision_id = b._revision_history()[1]
+        rev = b.repository.get_revision(revision_id)
+        eq(len(rev.parent_ids), 2)
+        eq(rev.parent_ids[1], 'wibble@wobble-2')
+
+    def test_upgrade_makes_dir_weaves(self):
+        self.build_tree_contents(_upgrade_dir_template)
+        old_repodir = bzrdir.BzrDir.open_unsupported('.')
+        old_repo_format = old_repodir.open_repository()._format
+        upgrade.upgrade('.')
+        # this is the path to the literal file. As format changes
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
+        repo = repository.Repository.open('.')
+        # it should have changed the format
+        self.assertNotEqual(old_repo_format.__class__, repo._format.__class__)
+        # and we should be able to read the names for the file id
+        # 'dir-20051005095101-da1441ea3fa6917a'
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        text_keys = repo.texts.keys()
+        dir_keys = [key for key in text_keys if key[0] ==
+                'dir-20051005095101-da1441ea3fa6917a']
+        self.assertNotEqual([], dir_keys)
+
+    def test_upgrade_to_meta_sets_workingtree_last_revision(self):
+        self.build_tree_contents(_upgrade_dir_template)
+        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
+        tree = workingtree.WorkingTree.open('.')
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([tree.branch._revision_history()[-1]],
+            tree.get_parent_ids())
+
+
+class SFTPBranchTest(TestCaseWithSFTPServer):
+    """Test some stuff when accessing a bzr Branch over sftp"""
+
+    def test_lock_file(self):
+        # old format branches use a special lock file on sftp.
+        b = self.make_branch('', format=BzrDirFormat6())
+        b = branch.Branch.open(self.get_url())
+        self.assertPathExists('.bzr/')
+        self.assertPathExists('.bzr/branch-format')
+        self.assertPathExists('.bzr/branch-lock')
+
+        self.assertPathDoesNotExist('.bzr/branch-lock.write-lock')
+        b.lock_write()
+        self.assertPathExists('.bzr/branch-lock.write-lock')
+        b.unlock()
+        self.assertPathDoesNotExist('.bzr/branch-lock.write-lock')
+
+
+class TestInfo(TestCaseWithTransport):
+
+    def test_info_locking_oslocks(self):
+        if sys.platform == "win32":
+            self.skip("don't use oslocks on win32 in unix manner")
+        # This test tests old (all-in-one, OS lock using) behaviour which
+        # simply cannot work on windows (and is indeed why we changed our
+        # design. As such, don't try to remove the thisFailsStrictLockCheck
+        # call here.
+        self.thisFailsStrictLockCheck()
+
+        tree = self.make_branch_and_tree('branch',
+                                         format=BzrDirFormat6())
+
+        # Test all permutations of locking the working tree, branch and repository
+        # XXX: Well not yet, as we can't query oslocks yet. Currently, it's
+        # implemented by raising NotImplementedError and get_physical_lock_status()
+        # always returns false. This makes bzr info hide the lock status.  (Olaf)
+        # W B R
+
+        # U U U
+        out, err = self.run_bzr('info -v branch')
+        self.assertEqualDiff(
+"""Standalone tree (format: weave)
+Location:
+  branch root: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Repository:
+         0 revisions
+""" % ('branch', tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        # L L L
+        tree.lock_write()
+        out, err = self.run_bzr('info -v branch')
+        self.assertEqualDiff(
+"""Standalone tree (format: weave)
+Location:
+  branch root: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Repository:
+         0 revisions
+""" % ('branch', tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        tree.unlock()
+
+
+class TestBranchFormat4(TestCaseWithTransport):
+    """Tests specific to branch format 4"""
+
+    def test_no_metadir_support(self):
+        url = self.get_url()
+        bdir = bzrdir.BzrDirMetaFormat1().initialize(url)
+        bdir.create_repository()
+        self.assertRaises(errors.IncompatibleFormat,
+            BzrBranchFormat4().initialize, bdir)
+
+    def test_supports_bzrdir_6(self):
+        url = self.get_url()
+        bdir = BzrDirFormat6().initialize(url)
+        bdir.create_repository()
+        BzrBranchFormat4().initialize(bdir)
+
+
+class TestBoundBranch(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestBoundBranch, self).setUp()
+        self.build_tree(['master/', 'child/'])
+        self.make_branch_and_tree('master')
+        self.make_branch_and_tree('child',
+                        format=bzrdir.format_registry.make_bzrdir('weave'))
+        os.chdir('child')
+
+    def test_bind_format_6_bzrdir(self):
+        # bind on a format 6 bzrdir should error
+        out,err = self.run_bzr('bind ../master', retcode=3)
+        self.assertEqual('', out)
+        # TODO: jam 20060427 Probably something like this really should
+        #       print out the actual path, rather than the URL
+        cwd = urlutils.local_path_to_url(getcwd())
+        self.assertEqual('bzr: ERROR: To use this feature you must '
+                         'upgrade your branch at %s/.\n' % cwd, err)
+
+    def test_unbind_format_6_bzrdir(self):
+        # bind on a format 6 bzrdir should error
+        out,err = self.run_bzr('unbind', retcode=3)
+        self.assertEqual('', out)
+        cwd = urlutils.local_path_to_url(getcwd())
+        self.assertEqual('bzr: ERROR: To use this feature you must '
+                         'upgrade your branch at %s/.\n' % cwd, err)
+
+
+class TestInit(TestCaseWithTransport):
+
+    def test_init_weave(self):
+        # --format=weave should be accepted to allow interoperation with
+        # old releases when desired.
+        out, err = self.run_bzr('init --format=weave')
+        self.assertEqual("""Created a standalone tree (format: weave)\n""",
+            out)
+        self.assertEqual('', err)
+
+
+class V4WeaveBundleTester(V4BundleTester):
+
+    def bzrdir_format(self):
+        return 'metaweave'

=== added file 'bzrlib/plugins/weave_fmt/test_repository.py'
--- bzrlib/plugins/weave_fmt/test_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/test_repository.py	2011-06-28 15:39:43 +0000
@@ -0,0 +1,329 @@
+# Copyright (C) 2006-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for weave repositories.
+
+For interface tests see tests/per_repository/*.py.
+
+"""
+
+from cStringIO import StringIO
+from stat import S_ISDIR
+import sys
+
+from bzrlib.bzrdir import (
+    BzrDirMetaFormat1,
+    )
+from bzrlib.errors import (
+    IllegalPath,
+    NoSuchFile,
+    )
+from bzrlib.repository import (
+    InterRepository,
+    Repository,
+    )
+from bzrlib.serializer import (
+    format_registry as serializer_format_registry,
+    )
+from bzrlib.tests import (
+    TestCase,
+    TestCaseWithTransport,
+    )
+
+from bzrlib.plugins.weave_fmt import xml4
+from bzrlib.plugins.weave_fmt.bzrdir import (
+    BzrDirFormat6,
+    )
+from bzrlib.plugins.weave_fmt.repository import (
+    InterWeaveRepo,
+    RepositoryFormat4,
+    RepositoryFormat5,
+    RepositoryFormat6,
+    RepositoryFormat7,
+    )
+
+
+class TestFormat6(TestCaseWithTransport):
+
+    def test_attribute__fetch_order(self):
+        """Weaves need topological data insertion."""
+        control = BzrDirFormat6().initialize(self.get_url())
+        repo = RepositoryFormat6().initialize(control)
+        self.assertEqual('topological', repo._format._fetch_order)
+
+    def test_attribute__fetch_uses_deltas(self):
+        """Weaves do not reuse deltas."""
+        control = BzrDirFormat6().initialize(self.get_url())
+        repo = RepositoryFormat6().initialize(control)
+        self.assertEqual(False, repo._format._fetch_uses_deltas)
+
+    def test_attribute__fetch_reconcile(self):
+        """Weave repositories need a reconcile after fetch."""
+        control = BzrDirFormat6().initialize(self.get_url())
+        repo = RepositoryFormat6().initialize(control)
+        self.assertEqual(True, repo._format._fetch_reconcile)
+
+    def test_no_ancestry_weave(self):
+        control = BzrDirFormat6().initialize(self.get_url())
+        repo = RepositoryFormat6().initialize(control)
+        # We no longer need to create the ancestry.weave file
+        # since it is *never* used.
+        self.assertRaises(NoSuchFile,
+                          control.transport.get,
+                          'ancestry.weave')
+
+    def test_supports_external_lookups(self):
+        control = BzrDirFormat6().initialize(self.get_url())
+        repo = RepositoryFormat6().initialize(control)
+        self.assertFalse(repo._format.supports_external_lookups)
+
+
+
+
+class TestFormat7(TestCaseWithTransport):
+
+    def test_attribute__fetch_order(self):
+        """Weaves need topological data insertion."""
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control)
+        self.assertEqual('topological', repo._format._fetch_order)
+
+    def test_attribute__fetch_uses_deltas(self):
+        """Weaves do not reuse deltas."""
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control)
+        self.assertEqual(False, repo._format._fetch_uses_deltas)
+
+    def test_attribute__fetch_reconcile(self):
+        """Weave repositories need a reconcile after fetch."""
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control)
+        self.assertEqual(True, repo._format._fetch_reconcile)
+
+    def test_disk_layout(self):
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control)
+        # in case of side effects of locking.
+        repo.lock_write()
+        repo.unlock()
+        # we want:
+        # format 'Bazaar-NG Repository format 7'
+        # lock ''
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Repository format 7',
+                             t.get('format').read())
+        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
+        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
+        self.assertEqualDiff('# bzr weave file v5\n'
+                             'w\n'
+                             'W\n',
+                             t.get('inventory.weave').read())
+        # Creating a file with id Foo:Bar results in a non-escaped file name on
+        # disk.
+        control.create_branch()
+        tree = control.create_workingtree()
+        tree.add(['foo'], ['Foo:Bar'], ['file'])
+        tree.put_file_bytes_non_atomic('Foo:Bar', 'content\n')
+        try:
+            tree.commit('first post', rev_id='first')
+        except IllegalPath:
+            if sys.platform != 'win32':
+                raise
+            self.knownFailure('Foo:Bar cannot be used as a file-id on windows'
+                              ' in repo format 7')
+            return
+        self.assertEqualDiff(
+            '# bzr weave file v5\n'
+            'i\n'
+            '1 7fe70820e08a1aac0ef224d9c66ab66831cc4ab1\n'
+            'n first\n'
+            '\n'
+            'w\n'
+            '{ 0\n'
+            '. content\n'
+            '}\n'
+            'W\n',
+            t.get('weaves/74/Foo%3ABar.weave').read())
+
+    def test_shared_disk_layout(self):
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control, shared=True)
+        # we want:
+        # format 'Bazaar-NG Repository format 7'
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        # a 'shared-storage' marker file.
+        # lock is not present when unlocked
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Repository format 7',
+                             t.get('format').read())
+        self.assertEqualDiff('', t.get('shared-storage').read())
+        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
+        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
+        self.assertEqualDiff('# bzr weave file v5\n'
+                             'w\n'
+                             'W\n',
+                             t.get('inventory.weave').read())
+        self.assertFalse(t.has('branch-lock'))
+
+    def test_creates_lockdir(self):
+        """Make sure it appears to be controlled by a LockDir existence"""
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control, shared=True)
+        t = control.get_repository_transport(None)
+        # TODO: Should check there is a 'lock' toplevel directory,
+        # regardless of contents
+        self.assertFalse(t.has('lock/held/info'))
+        repo.lock_write()
+        try:
+            self.assertTrue(t.has('lock/held/info'))
+        finally:
+            # unlock so we don't get a warning about failing to do so
+            repo.unlock()
+
+    def test_uses_lockdir(self):
+        """repo format 7 actually locks on lockdir"""
+        base_url = self.get_url()
+        control = BzrDirMetaFormat1().initialize(base_url)
+        repo = RepositoryFormat7().initialize(control, shared=True)
+        t = control.get_repository_transport(None)
+        repo.lock_write()
+        repo.unlock()
+        del repo
+        # make sure the same lock is created by opening it
+        repo = Repository.open(base_url)
+        repo.lock_write()
+        self.assertTrue(t.has('lock/held/info'))
+        repo.unlock()
+        self.assertFalse(t.has('lock/held/info'))
+
+    def test_shared_no_tree_disk_layout(self):
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control, shared=True)
+        repo.set_make_working_trees(False)
+        # we want:
+        # format 'Bazaar-NG Repository format 7'
+        # lock ''
+        # inventory.weave == empty_weave
+        # empty revision-store directory
+        # empty weaves directory
+        # a 'shared-storage' marker file.
+        t = control.get_repository_transport(None)
+        self.assertEqualDiff('Bazaar-NG Repository format 7',
+                             t.get('format').read())
+        ## self.assertEqualDiff('', t.get('lock').read())
+        self.assertEqualDiff('', t.get('shared-storage').read())
+        self.assertEqualDiff('', t.get('no-working-trees').read())
+        repo.set_make_working_trees(True)
+        self.assertFalse(t.has('no-working-trees'))
+        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
+        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
+        self.assertEqualDiff('# bzr weave file v5\n'
+                             'w\n'
+                             'W\n',
+                             t.get('inventory.weave').read())
+
+    def test_supports_external_lookups(self):
+        control = BzrDirMetaFormat1().initialize(self.get_url())
+        repo = RepositoryFormat7().initialize(control)
+        self.assertFalse(repo._format.supports_external_lookups)
+
+
+class TestInterWeaveRepo(TestCaseWithTransport):
+
+    def test_is_compatible_and_registered(self):
+        # InterWeaveRepo is compatible when either side
+        # is a format 5/6/7 branch
+        from bzrlib.repofmt import knitrepo
+        formats = [RepositoryFormat5(),
+                   RepositoryFormat6(),
+                   RepositoryFormat7()]
+        incompatible_formats = [RepositoryFormat4(),
+                                knitrepo.RepositoryFormatKnit1(),
+                                ]
+        repo_a = self.make_repository('a')
+        repo_b = self.make_repository('b')
+        is_compatible = InterWeaveRepo.is_compatible
+        for source in incompatible_formats:
+            # force incompatible left then right
+            repo_a._format = source
+            repo_b._format = formats[0]
+            self.assertFalse(is_compatible(repo_a, repo_b))
+            self.assertFalse(is_compatible(repo_b, repo_a))
+        for source in formats:
+            repo_a._format = source
+            for target in formats:
+                repo_b._format = target
+                self.assertTrue(is_compatible(repo_a, repo_b))
+        self.assertEqual(InterWeaveRepo,
+                         InterRepository.get(repo_a, repo_b).__class__)
+
+
+_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
+<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
+<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
+<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
+</inventory>"""
+
+
+_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.212"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
+</parents>
+</revision>
+"""
+
+
+class TestSerializer(TestCase):
+    """Test serializer"""
+
+    def test_registry(self):
+        self.assertIs(xml4.serializer_v4,
+                      serializer_format_registry.get('4'))
+
+    def test_canned_inventory(self):
+        """Test unpacked a canned inventory v4 file."""
+        inp = StringIO(_working_inventory_v4)
+        inv = xml4.serializer_v4.read_inventory(inp)
+        self.assertEqual(len(inv), 4)
+        self.assert_(inv.has_id('bar-20050901064931-73b4b1138abc9cd2'))
+
+    def test_unpack_revision(self):
+        """Test unpacking a canned revision v4"""
+        inp = StringIO(_revision_v4)
+        rev = xml4.serializer_v4.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parent_ids), 1)
+        eq(rev.parent_ids[0],
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+

=== added file 'bzrlib/plugins/weave_fmt/test_workingtree.py'
--- bzrlib/plugins/weave_fmt/test_workingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/test_workingtree.py	2011-03-10 13:29:54 +0000
@@ -0,0 +1,87 @@
+# Copyright (C) 2005-2011 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for weave-era working tree formats."""
+
+import os
+
+from bzrlib import (
+    conflicts,
+    errors,
+    )
+
+from bzrlib.tests import (
+    TestCaseWithTransport,
+    )
+
+from bzrlib.plugins.weave_fmt.bzrdir import BzrDirFormat6
+
+
+class TestFormat2WorkingTree(TestCaseWithTransport):
+    """Tests that are specific to format 2 trees."""
+
+    def create_format2_tree(self, url):
+        return self.make_branch_and_tree(
+            url, format=BzrDirFormat6())
+
+    def test_conflicts(self):
+        # test backwards compatability
+        tree = self.create_format2_tree('.')
+        self.assertRaises(errors.UnsupportedOperation, tree.set_conflicts,
+                          None)
+        file('lala.BASE', 'wb').write('labase')
+        expected = conflicts.ContentsConflict('lala')
+        self.assertEqual(list(tree.conflicts()), [expected])
+        file('lala', 'wb').write('la')
+        tree.add('lala', 'lala-id')
+        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
+        self.assertEqual(list(tree.conflicts()), [expected])
+        file('lala.THIS', 'wb').write('lathis')
+        file('lala.OTHER', 'wb').write('laother')
+        # When "text conflict"s happen, stem, THIS and OTHER are text
+        expected = conflicts.TextConflict('lala', file_id='lala-id')
+        self.assertEqual(list(tree.conflicts()), [expected])
+        os.unlink('lala.OTHER')
+        os.mkdir('lala.OTHER')
+        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
+        self.assertEqual(list(tree.conflicts()), [expected])
+
+    def test_detect_conflicts(self):
+        """Conflicts are detected properly"""
+        tree = self.create_format2_tree('.')
+        self.build_tree_contents([('hello', 'hello world4'),
+                                  ('hello.THIS', 'hello world2'),
+                                  ('hello.BASE', 'hello world1'),
+                                  ('hello.OTHER', 'hello world3'),
+                                  ('hello.sploo.BASE', 'yellowworld'),
+                                  ('hello.sploo.OTHER', 'yellowworld2'),
+                                  ])
+        tree.lock_read()
+        self.assertLength(6, list(tree.list_files()))
+        tree.unlock()
+        tree_conflicts = tree.conflicts()
+        self.assertLength(2, tree_conflicts)
+        self.assertTrue('hello' in tree_conflicts[0].path)
+        self.assertTrue('hello.sploo' in tree_conflicts[1].path)
+        conflicts.restore('hello')
+        conflicts.restore('hello.sploo')
+        self.assertLength(0, tree.conflicts())
+        self.assertFileEqual('hello world2', 'hello')
+        self.assertFalse(os.path.lexists('hello.sploo'))
+        self.assertRaises(errors.NotConflicted, conflicts.restore, 'hello')
+        self.assertRaises(errors.NotConflicted,
+                          conflicts.restore, 'hello.sploo')

=== added directory 'bzrlib/plugins/weave_fmt/tests'
=== added file 'bzrlib/plugins/weave_fmt/workingtree.py'
--- bzrlib/plugins/weave_fmt/workingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/workingtree.py	2011-08-30 17:32:29 +0000
@@ -0,0 +1,238 @@
+# Copyright (C) 2005-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Weave-era working tree objects."""
+
+from cStringIO import StringIO
+
+from bzrlib import (
+    conflicts as _mod_conflicts,
+    errors,
+    inventory,
+    osutils,
+    revision as _mod_revision,
+    transform,
+    xml5,
+    )
+from bzrlib.decorators import needs_read_lock
+from bzrlib.transport.local import LocalTransport
+from bzrlib.workingtree import (
+    WorkingTreeFormat,
+    )
+from bzrlib.workingtree_3 import (
+    PreDirStateWorkingTree,
+    )
+
+
+def get_conflicted_stem(path):
+    for suffix in _mod_conflicts.CONFLICT_SUFFIXES:
+        if path.endswith(suffix):
+            return path[:-len(suffix)]
+
+
+class WorkingTreeFormat2(WorkingTreeFormat):
+    """The second working tree format.
+
+    This format modified the hash cache from the format 1 hash cache.
+    """
+
+    upgrade_recommended = True
+
+    requires_normalized_unicode_filenames = True
+
+    case_sensitive_filename = "Branch-FoRMaT"
+
+    missing_parent_conflicts = False
+
+    supports_versioned_directories = True
+
+    def get_format_description(self):
+        """See WorkingTreeFormat.get_format_description()."""
+        return "Working tree format 2"
+
+    def _stub_initialize_on_transport(self, transport, file_mode):
+        """Workaround: create control files for a remote working tree.
+
+        This ensures that it can later be updated and dealt with locally,
+        since BzrDirFormat6 and BzrDirFormat5 cannot represent dirs with
+        no working tree.  (See bug #43064).
+        """
+        sio = StringIO()
+        inv = inventory.Inventory()
+        xml5.serializer_v5.write_inventory(inv, sio, working=True)
+        sio.seek(0)
+        transport.put_file('inventory', sio, file_mode)
+        transport.put_bytes('pending-merges', '', file_mode)
+
+    def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
+                   accelerator_tree=None, hardlink=False):
+        """See WorkingTreeFormat.initialize()."""
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        if from_branch is not None:
+            branch = from_branch
+        else:
+            branch = a_bzrdir.open_branch()
+        if revision_id is None:
+            revision_id = _mod_revision.ensure_null(branch.last_revision())
+        branch.lock_write()
+        try:
+            branch.generate_revision_history(revision_id)
+        finally:
+            branch.unlock()
+        inv = inventory.Inventory()
+        wt = WorkingTree2(a_bzrdir.root_transport.local_abspath('.'),
+                         branch,
+                         inv,
+                         _internal=True,
+                         _format=self,
+                         _bzrdir=a_bzrdir,
+                         _control_files=branch.control_files)
+        basis_tree = branch.repository.revision_tree(revision_id)
+        if basis_tree.inventory.root is not None:
+            wt.set_root_id(basis_tree.get_root_id())
+        # set the parent list and cache the basis tree.
+        if _mod_revision.is_null(revision_id):
+            parent_trees = []
+        else:
+            parent_trees = [(revision_id, basis_tree)]
+        wt.set_parent_trees(parent_trees)
+        transform.build_tree(basis_tree, wt)
+        return wt
+
+    def __init__(self):
+        super(WorkingTreeFormat2, self).__init__()
+        from bzrlib.plugins.weave_fmt.bzrdir import BzrDirFormat6
+        self._matchingbzrdir = BzrDirFormat6()
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the WorkingTree object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already been done.
+        """
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        wt = WorkingTree2(a_bzrdir.root_transport.local_abspath('.'),
+                           _internal=True,
+                           _format=self,
+                           _bzrdir=a_bzrdir,
+                           _control_files=a_bzrdir.open_branch().control_files)
+        return wt
+
+
+class WorkingTree2(PreDirStateWorkingTree):
+    """This is the Format 2 working tree.
+
+    This was the first weave based working tree.
+     - uses os locks for locking.
+     - uses the branch last-revision.
+    """
+
+    def __init__(self, basedir, *args, **kwargs):
+        super(WorkingTree2, self).__init__(basedir, *args, **kwargs)
+        # WorkingTree2 has more of a constraint that self._inventory must
+        # exist. Because this is an older format, we don't mind the overhead
+        # caused by the extra computation here.
+
+        # Newer WorkingTree's should only have self._inventory set when they
+        # have a read lock.
+        if self._inventory is None:
+            self.read_working_inventory()
+
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree."""
+        return [('trees', self.last_revision())]
+
+
+    def lock_tree_write(self):
+        """See WorkingTree.lock_tree_write().
+
+        In Format2 WorkingTrees we have a single lock for the branch and tree
+        so lock_tree_write() degrades to lock_write().
+
+        :return: An object with an unlock method which will release the lock
+            obtained.
+        """
+        self.branch.lock_write()
+        try:
+            self._control_files.lock_write()
+            return self
+        except:
+            self.branch.unlock()
+            raise
+
+    def unlock(self):
+        # we share control files:
+        if self._control_files._lock_count == 3:
+            # do non-implementation specific cleanup
+            self._cleanup()
+            # _inventory_is_modified is always False during a read lock.
+            if self._inventory_is_modified:
+                self.flush()
+            self._write_hashcache_if_dirty()
+
+        # reverse order of locking.
+        try:
+            return self._control_files.unlock()
+        finally:
+            self.branch.unlock()
+
+    def _iter_conflicts(self):
+        conflicted = set()
+        for info in self.list_files():
+            path = info[0]
+            stem = get_conflicted_stem(path)
+            if stem is None:
+                continue
+            if stem not in conflicted:
+                conflicted.add(stem)
+                yield stem
+
+    @needs_read_lock
+    def conflicts(self):
+        conflicts = _mod_conflicts.ConflictList()
+        for conflicted in self._iter_conflicts():
+            text = True
+            try:
+                if osutils.file_kind(self.abspath(conflicted)) != "file":
+                    text = False
+            except errors.NoSuchFile:
+                text = False
+            if text is True:
+                for suffix in ('.THIS', '.OTHER'):
+                    try:
+                        kind = osutils.file_kind(self.abspath(conflicted+suffix))
+                        if kind != "file":
+                            text = False
+                    except errors.NoSuchFile:
+                        text = False
+                    if text == False:
+                        break
+            ctype = {True: 'text conflict', False: 'contents conflict'}[text]
+            conflicts.append(_mod_conflicts.Conflict.factory(ctype,
+                             path=conflicted,
+                             file_id=self.path2id(conflicted)))
+        return conflicts
+
+    def set_conflicts(self, arg):
+        raise errors.UnsupportedOperation(self.set_conflicts, self)
+
+    def add_conflicts(self, arg):
+        raise errors.UnsupportedOperation(self.add_conflicts, self)

=== renamed file 'bzrlib/xml4.py' => 'bzrlib/plugins/weave_fmt/xml4.py'
=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2010-07-17 16:39:20 +0000
+++ bzrlib/progress.py	2010-09-14 04:13:48 +0000
@@ -166,48 +166,6 @@
             self.ui_factory.clear_term()
 
 
-# NOTE: This is also deprecated; you should provide a ProgressView instead.
-class _BaseProgressBar(object):
-
-    def __init__(self,
-                 to_file=None,
-                 show_pct=False,
-                 show_spinner=False,
-                 show_eta=False,
-                 show_bar=True,
-                 show_count=True,
-                 to_messages_file=None,
-                 _stack=None):
-        object.__init__(self)
-        if to_file is None:
-            to_file = sys.stderr
-        if to_messages_file is None:
-            to_messages_file = sys.stdout
-        self.to_file = to_file
-        self.to_messages_file = to_messages_file
-        self.last_msg = None
-        self.last_cnt = None
-        self.last_total = None
-        self.show_pct = show_pct
-        self.show_spinner = show_spinner
-        self.show_eta = show_eta
-        self.show_bar = show_bar
-        self.show_count = show_count
-        self._stack = _stack
-        # seed throttler
-        self.MIN_PAUSE = 0.1 # seconds
-        now = time.time()
-        # starting now
-        self.start_time = now
-        # next update should not throttle
-        self.last_update = now - self.MIN_PAUSE - 1
-
-    def finished(self):
-        """Return this bar to its progress stack."""
-        self.clear()
-        self._stack.return_pb(self)
-
-
 class DummyProgress(object):
     """Progress-bar standin that does nothing.
 

=== modified file 'bzrlib/push.py'
--- bzrlib/push.py	2010-04-30 11:03:59 +0000
+++ bzrlib/push.py	2011-10-11 12:01:51 +0000
@@ -17,7 +17,7 @@
 """UI helper for the push command."""
 
 from bzrlib import (
-    bzrdir,
+    controldir,
     errors,
     revision as _mod_revision,
     transport,
@@ -26,6 +26,7 @@
     note,
     warning,
     )
+from bzrlib.i18n import gettext
 
 
 class PushResult(object):
@@ -47,17 +48,17 @@
         """Write a human-readable description of the result."""
         if self.branch_push_result is None:
             if self.stacked_on is not None:
-                note('Created new stacked branch referring to %s.' %
+                note(gettext('Created new stacked branch referring to %s.') %
                     self.stacked_on)
             else:
-                note('Created new branch.')
+                note(gettext('Created new branch.'))
         else:
             self.branch_push_result.report(to_file)
 
 
 def _show_push_branch(br_from, revision_id, location, to_file, verbose=False,
     overwrite=False, remember=False, stacked_on=None, create_prefix=False,
-    use_existing_dir=False):
+    use_existing_dir=False, no_tree=False):
     """Push a branch to a location.
 
     :param br_from: the source branch
@@ -78,7 +79,7 @@
     """
     to_transport = transport.get_transport(location)
     try:
-        dir_to = bzrdir.BzrDir.open_from_transport(to_transport)
+        dir_to = controldir.ControlDir.open_from_transport(to_transport)
     except errors.NotBranchError:
         # Didn't find anything
         dir_to = None
@@ -87,26 +88,27 @@
         try:
             br_to = br_from.create_clone_on_transport(to_transport,
                 revision_id=revision_id, stacked_on=stacked_on,
-                create_prefix=create_prefix, use_existing_dir=use_existing_dir)
+                create_prefix=create_prefix, use_existing_dir=use_existing_dir,
+                no_tree=no_tree)
         except errors.FileExists, err:
             if err.path.endswith('/.bzr'):
-                raise errors.BzrCommandError(
+                raise errors.BzrCommandError(gettext(
                     "Target directory %s already contains a .bzr directory, "
-                    "but it is not valid." % (location,))
+                    "but it is not valid.") % (location,))
             if not use_existing_dir:
-                raise errors.BzrCommandError("Target directory %s"
+                raise errors.BzrCommandError(gettext("Target directory %s"
                      " already exists, but does not have a .bzr"
                      " directory. Supply --use-existing-dir to push"
-                     " there anyway." % location)
+                     " there anyway.") % location)
             # This shouldn't occur, but if it does the FileExists error will be
             # more informative than an UnboundLocalError for br_to.
             raise
         except errors.NoSuchFile:
             if not create_prefix:
-                raise errors.BzrCommandError("Parent directory of %s"
+                raise errors.BzrCommandError(gettext("Parent directory of %s"
                     " does not exist."
                     "\nYou may supply --create-prefix to create all"
-                    " leading parent directories."
+                    " leading parent directories.")
                     % location)
             # This shouldn't occur (because create_prefix is true, so
             # create_clone_on_transport should be catching NoSuchFile and
@@ -115,8 +117,8 @@
             # UnboundLocalError for br_to.
             raise
         except errors.TooManyRedirections:
-            raise errors.BzrCommandError("Too many redirections trying "
-                                         "to make %s." % location)
+            raise errors.BzrCommandError(gettext("Too many redirections trying "
+                                         "to make %s.") % location)
         push_result = PushResult()
         # TODO: Some more useful message about what was copied
         try:
@@ -128,7 +130,9 @@
         push_result.target_branch = br_to
         push_result.old_revid = _mod_revision.NULL_REVISION
         push_result.old_revno = 0
-        if br_from.get_push_location() is None or remember:
+        # Remembers if asked explicitly or no previous location is set
+        if (remember
+            or (remember is None and br_from.get_push_location() is None)):
             br_from.set_push_location(br_to.base)
     else:
         if stacked_on is not None:
@@ -138,20 +142,20 @@
             push_result = dir_to.push_branch(br_from, revision_id, overwrite, 
                 remember, create_prefix)
         except errors.DivergedBranches:
-            raise errors.BzrCommandError('These branches have diverged.'
+            raise errors.BzrCommandError(gettext('These branches have diverged.'
                                     '  See "bzr help diverged-branches"'
-                                    ' for more information.')
+                                    ' for more information.'))
         except errors.NoRoundtrippingSupport, e:
-            raise errors.BzrCommandError("It is not possible to losslessly "
-                "push to %s. You may want to use dpush instead." % 
+            raise errors.BzrCommandError(gettext("It is not possible to losslessly "
+                "push to %s. You may want to use dpush instead.") % 
                     e.target_branch.mapping.vcs.abbreviation)
         except errors.NoRepositoryPresent:
-            # we have a bzrdir but no branch or repository
+            # we have a controldir but no branch or repository
             # XXX: Figure out what to do other than complain.
-            raise errors.BzrCommandError("At %s you have a valid .bzr"
+            raise errors.BzrCommandError(gettext("At %s you have a valid .bzr"
                 " control directory, but not a branch or repository. This"
                 " is an unsupported configuration. Please move the target"
-                " directory out of the way and try again." % location)
+                " directory out of the way and try again.") % location)
         if push_result.workingtree_updated == False:
             warning("This transport does not update the working " 
                     "tree of: %s. See 'bzr help working-trees' for "

=== modified file 'bzrlib/python-compat.h'
--- bzrlib/python-compat.h	2009-10-21 20:53:21 +0000
+++ bzrlib/python-compat.h	2010-08-23 21:36:06 +0000
@@ -74,6 +74,9 @@
 
 #ifdef _MSC_VER
 #define  snprintf  _snprintf
+/* gcc (mingw32) has strtoll, while the MSVC compiler uses _strtoi64 */
+#define strtoll _strtoi64
+#define strtoull _strtoui64
 #endif
 
 /* Introduced in Python 2.6 */

=== added file 'bzrlib/pyutils.py'
--- bzrlib/pyutils.py	1970-01-01 00:00:00 +0000
+++ bzrlib/pyutils.py	2010-10-15 12:35:00 +0000
@@ -0,0 +1,90 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""General Python convenience functions."""
+
+
+import sys
+
+
+def get_named_object(module_name, member_name=None):
+    """Get the Python object named by a given module and member name.
+
+    This is usually much more convenient than dealing with ``__import__``
+    directly::
+
+        >>> doc = get_named_object('bzrlib.pyutils', 'get_named_object.__doc__')
+        >>> doc.splitlines()[0]
+        'Get the Python object named by a given module and member name.'
+
+    :param module_name: a module name, as would be found in sys.modules if
+        the module is already imported.  It may contain dots.  e.g. 'sys' or
+        'os.path'.
+    :param member_name: (optional) a name of an attribute in that module to
+        return.  It may contain dots.  e.g. 'MyClass.some_method'.  If not
+        given, the named module will be returned instead.
+    :raises: ImportError or AttributeError.
+    """
+    # We may have just a module name, or a module name and a member name,
+    # and either may contain dots.  __import__'s return value is a bit
+    # unintuitive, so we need to take care to always return the object
+    # specified by the full combination of module name + member name.
+    if member_name:
+        # Give __import__ a from_list.  It will return the last module in
+        # the dotted module name.
+        attr_chain = member_name.split('.')
+        from_list = attr_chain[:1]
+        obj = __import__(module_name, {}, {}, from_list)
+        for attr in attr_chain:
+            obj = getattr(obj, attr)
+    else:
+        # We're just importing a module, no attributes, so we have no
+        # from_list.  __import__ will return the first module in the dotted
+        # module name, so we look up the module from sys.modules.
+        __import__(module_name, globals(), locals(), [])
+        obj = sys.modules[module_name]
+    return obj
+
+
+def calc_parent_name(module_name, member_name=None):
+    """Determine the 'parent' of a given dotted module name and (optional)
+    member name.
+
+    The idea is that ``getattr(parent_obj, final_attr)`` will equal
+    get_named_object(module_name, member_name).
+
+    :return: (module_name, member_name, final_attr) tuple.
+    """
+# +SKIP is not recognized by python2.4
+# Typical use is::
+# 
+#     >>> parent_mod, parent_member, final_attr = calc_parent_name(
+#     ...     module_name, member_name) # doctest: +SKIP
+#     >>> parent_obj = get_named_object(parent_mod, parent_member)
+#     ... # doctest: +SKIP
+    if member_name is not None:
+        split_name = member_name.rsplit('.', 1)
+        if len(split_name) == 1:
+            return (module_name, None, member_name)
+        else:
+            return (module_name, split_name[0], split_name[1])
+    else:
+        split_name = module_name.rsplit('.', 1)
+        if len(split_name) == 1:
+            raise AssertionError(
+                'No parent object for top-level module %r' % (module_name,))
+        else:
+            return (split_name[0], None, split_name[1])

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2010-05-20 18:23:17 +0000
+++ bzrlib/reconcile.py	2011-09-19 15:59:40 +0000
@@ -29,14 +29,16 @@
 from bzrlib import (
     cleanup,
     errors,
+    revision as _mod_revision,
     ui,
     )
 from bzrlib.trace import mutter
 from bzrlib.tsort import topo_sort
 from bzrlib.versionedfile import AdapterFactory, FulltextContentFactory
-
-
-def reconcile(dir, other=None):
+from bzrlib.i18n import gettext
+
+
+def reconcile(dir, canonicalize_chks=False):
     """Reconcile the data in dir.
 
     Currently this is limited to a inventory 'reweave'.
@@ -46,30 +48,32 @@
     Directly using Reconciler is recommended for library users that
     desire fine grained control or analysis of the found issues.
 
-    :param other: another bzrdir to reconcile against.
+    :param canonicalize_chks: Make sure CHKs are in canonical form.
     """
-    reconciler = Reconciler(dir, other=other)
+    reconciler = Reconciler(dir, canonicalize_chks=canonicalize_chks)
     reconciler.reconcile()
 
 
 class Reconciler(object):
     """Reconcilers are used to reconcile existing data."""
 
-    def __init__(self, dir, other=None):
+    def __init__(self, dir, other=None, canonicalize_chks=False):
         """Create a Reconciler."""
         self.bzrdir = dir
+        self.canonicalize_chks = canonicalize_chks
 
     def reconcile(self):
         """Perform reconciliation.
 
         After reconciliation the following attributes document found issues:
-        inconsistent_parents: The number of revisions in the repository whose
-                              ancestry was being reported incorrectly.
-        garbage_inventories: The number of inventory objects without revisions
-                             that were garbage collected.
-        fixed_branch_history: None if there was no branch, False if the branch
-                              history was correct, True if the branch history
-                              needed to be re-normalized.
+
+        * `inconsistent_parents`: The number of revisions in the repository
+          whose ancestry was being reported incorrectly.
+        * `garbage_inventories`: The number of inventory objects without
+          revisions that were garbage collected.
+        * `fixed_branch_history`: None if there was no branch, False if the
+          branch history was correct, True if the branch history needed to be
+          re-normalized.
         """
         self.pb = ui.ui_factory.nested_progress_bar()
         try:
@@ -89,25 +93,33 @@
             # Nothing to check here
             self.fixed_branch_history = None
             return
-        ui.ui_factory.note('Reconciling branch %s' % self.branch.base)
+        ui.ui_factory.note(gettext('Reconciling branch %s') % self.branch.base)
         branch_reconciler = self.branch.reconcile(thorough=True)
         self.fixed_branch_history = branch_reconciler.fixed_history
 
     def _reconcile_repository(self):
         self.repo = self.bzrdir.find_repository()
-        ui.ui_factory.note('Reconciling repository %s' %
+        ui.ui_factory.note(gettext('Reconciling repository %s') %
             self.repo.user_url)
-        self.pb.update("Reconciling repository", 0, 1)
-        repo_reconciler = self.repo.reconcile(thorough=True)
+        self.pb.update(gettext("Reconciling repository"), 0, 1)
+        if self.canonicalize_chks:
+            try:
+                self.repo.reconcile_canonicalize_chks
+            except AttributeError:
+                raise errors.BzrError(
+                    gettext("%s cannot canonicalize CHKs.") % (self.repo,))
+            repo_reconciler = self.repo.reconcile_canonicalize_chks()
+        else:
+            repo_reconciler = self.repo.reconcile(thorough=True)
         self.inconsistent_parents = repo_reconciler.inconsistent_parents
         self.garbage_inventories = repo_reconciler.garbage_inventories
         if repo_reconciler.aborted:
-            ui.ui_factory.note(
-                'Reconcile aborted: revision index has inconsistent parents.')
-            ui.ui_factory.note(
-                'Run "bzr check" for more details.')
+            ui.ui_factory.note(gettext(
+                'Reconcile aborted: revision index has inconsistent parents.'))
+            ui.ui_factory.note(gettext(
+                'Run "bzr check" for more details.'))
         else:
-            ui.ui_factory.note('Reconciliation complete.')
+            ui.ui_factory.note(gettext('Reconciliation complete.'))
 
 
 class BranchReconciler(object):
@@ -134,12 +146,12 @@
         self._reconcile_revision_history()
 
     def _reconcile_revision_history(self):
-        repo = self.branch.repository
         last_revno, last_revision_id = self.branch.last_revision_info()
         real_history = []
+        graph = self.branch.repository.get_graph()
         try:
-            for revid in repo.iter_reverse_revision_history(
-                    last_revision_id):
+            for revid in graph.iter_lefthand_ancestry(
+                    last_revision_id, (_mod_revision.NULL_REVISION,)):
                 real_history.append(revid)
         except errors.RevisionNotPresent:
             pass # Hit a ghost left hand parent
@@ -150,13 +162,14 @@
             # set_revision_history, as this will regenerate it again.
             # Not really worth a whole BranchReconciler class just for this,
             # though.
-            ui.ui_factory.note('Fixing last revision info %s => %s' % (
-                 last_revno, len(real_history)))
+            ui.ui_factory.note(gettext('Fixing last revision info {0} '\
+                                       ' => {1}').format(
+                                       last_revno, len(real_history)))
             self.branch.set_last_revision_info(len(real_history),
                                                last_revision_id)
         else:
             self.fixed_history = False
-            ui.ui_factory.note('revision_history ok.')
+            ui.ui_factory.note(gettext('revision_history ok.'))
 
 
 class RepoReconciler(object):
@@ -187,10 +200,11 @@
         """Perform reconciliation.
 
         After reconciliation the following attributes document found issues:
-        inconsistent_parents: The number of revisions in the repository whose
-                              ancestry was being reported incorrectly.
-        garbage_inventories: The number of inventory objects without revisions
-                             that were garbage collected.
+
+        * `inconsistent_parents`: The number of revisions in the repository
+          whose ancestry was being reported incorrectly.
+        * `garbage_inventories`: The number of inventory objects without
+          revisions that were garbage collected.
         """
         operation = cleanup.OperationWithCleanups(self._reconcile)
         self.add_cleanup = operation.add_cleanup
@@ -215,10 +229,8 @@
         only data-loss causing issues (!self.thorough) or all issues
         (self.thorough) are treated as requiring the reweave.
         """
-        # local because needing to know about WeaveFile is a wart we want to hide
-        from bzrlib.weave import WeaveFile, Weave
         transaction = self.repo.get_transaction()
-        self.pb.update('Reading inventory data')
+        self.pb.update(gettext('Reading inventory data'))
         self.inventory = self.repo.inventories
         self.revisions = self.repo.revisions
         # the total set of revisions to process
@@ -238,11 +250,11 @@
         # (no garbage inventories or we are not doing a thorough check)
         if (not self.inconsistent_parents and
             (not self.garbage_inventories or not self.thorough)):
-            ui.ui_factory.note('Inventory ok.')
+            ui.ui_factory.note(gettext('Inventory ok.'))
             return
-        self.pb.update('Backing up inventory', 0, 0)
+        self.pb.update(gettext('Backing up inventory'), 0, 0)
         self.repo._backup_inventory()
-        ui.ui_factory.note('Backup inventory created.')
+        ui.ui_factory.note(gettext('Backup inventory created.'))
         new_inventories = self.repo._temp_inventories()
 
         # we have topological order of revisions and non ghost parents ready.
@@ -258,10 +270,10 @@
         if not (set(new_inventories.keys()) ==
             set([(revid,) for revid in self.pending])):
             raise AssertionError()
-        self.pb.update('Writing weave')
+        self.pb.update(gettext('Writing weave'))
         self.repo._activate_new_inventory()
         self.inventory = None
-        ui.ui_factory.note('Inventory regenerated.')
+        ui.ui_factory.note(gettext('Inventory regenerated.'))
 
     def _new_inv_parents(self, revision_key):
         """Lookup ghost-filtered parents for revision_key."""
@@ -355,24 +367,24 @@
     def _load_indexes(self):
         """Load indexes for the reconciliation."""
         self.transaction = self.repo.get_transaction()
-        self.pb.update('Reading indexes', 0, 2)
+        self.pb.update(gettext('Reading indexes'), 0, 2)
         self.inventory = self.repo.inventories
-        self.pb.update('Reading indexes', 1, 2)
+        self.pb.update(gettext('Reading indexes'), 1, 2)
         self.repo._check_for_inconsistent_revision_parents()
         self.revisions = self.repo.revisions
-        self.pb.update('Reading indexes', 2, 2)
+        self.pb.update(gettext('Reading indexes'), 2, 2)
 
     def _gc_inventory(self):
         """Remove inventories that are not referenced from the revision store."""
-        self.pb.update('Checking unused inventories', 0, 1)
+        self.pb.update(gettext('Checking unused inventories'), 0, 1)
         self._check_garbage_inventories()
-        self.pb.update('Checking unused inventories', 1, 3)
+        self.pb.update(gettext('Checking unused inventories'), 1, 3)
         if not self.garbage_inventories:
-            ui.ui_factory.note('Inventory ok.')
+            ui.ui_factory.note(gettext('Inventory ok.'))
             return
-        self.pb.update('Backing up inventory', 0, 0)
+        self.pb.update(gettext('Backing up inventory'), 0, 0)
         self.repo._backup_inventory()
-        ui.ui_factory.note('Backup Inventory created')
+        ui.ui_factory.note(gettext('Backup Inventory created'))
         # asking for '' should never return a non-empty weave
         new_inventories = self.repo._temp_inventories()
         # we have topological order of revisions and non ghost parents ready.
@@ -389,10 +401,10 @@
         # the revisionds list
         if not(set(new_inventories.keys()) == set(revision_keys)):
             raise AssertionError()
-        self.pb.update('Writing weave')
+        self.pb.update(gettext('Writing weave'))
         self.repo._activate_new_inventory()
         self.inventory = None
-        ui.ui_factory.note('Inventory regenerated.')
+        ui.ui_factory.note(gettext('Inventory regenerated.'))
 
     def _fix_text_parents(self):
         """Fix bad versionedfile parent entries.
@@ -430,7 +442,7 @@
             versions_list.append(text_key[1])
         # Do the reconcile of individual weaves.
         for num, file_id in enumerate(per_id_bad_parents):
-            self.pb.update('Fixing text parents', num,
+            self.pb.update(gettext('Fixing text parents'), num,
                            len(per_id_bad_parents))
             versions_with_bad_parents = per_id_bad_parents[file_id]
             id_unused_versions = set(key[-1] for key in unused_versions
@@ -496,6 +508,12 @@
     #  - unlock the names list
     # https://bugs.launchpad.net/bzr/+bug/154173
 
+    def __init__(self, repo, other=None, thorough=False,
+            canonicalize_chks=False):
+        super(PackReconciler, self).__init__(repo, other=other,
+            thorough=thorough)
+        self.canonicalize_chks = canonicalize_chks
+
     def _reconcile_steps(self):
         """Perform the steps to reconcile this repository."""
         if not self.thorough:
@@ -509,8 +527,12 @@
         total_inventories = len(list(
             collection.inventory_index.combined_index.iter_all_entries()))
         if len(all_revisions):
-            new_pack =  self.repo._reconcile_pack(collection, packs,
-                ".reconcile", all_revisions, self.pb)
+            if self.canonicalize_chks:
+                reconcile_meth = self.repo._canonicalize_chks_pack
+            else:
+                reconcile_meth = self.repo._reconcile_pack
+            new_pack = reconcile_meth(collection, packs, ".reconcile",
+                all_revisions, self.pb)
             if new_pack is not None:
                 self._discard_and_save(packs)
         else:

=== modified file 'bzrlib/reconfigure.py'
--- bzrlib/reconfigure.py	2010-04-23 00:44:15 +0000
+++ bzrlib/reconfigure.py	2011-10-11 12:01:51 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Reconfigure a bzrdir into a new tree/branch/repository layout.
+"""Reconfigure a controldir into a new tree/branch/repository layout.
 
 Various types of reconfiguration operation are available either by
 constructing a class or using a factory method on Reconfigure.
@@ -23,13 +23,13 @@
 
 from bzrlib import (
     branch,
-    bzrdir,
+    controldir,
     errors,
     trace,
     ui,
     urlutils,
     )
-
+from bzrlib.i18n import gettext
 
 # TODO: common base class for all reconfigure operations, making no
 # assumptions about what kind of change will be done.
@@ -48,9 +48,9 @@
         try:
             branch.set_stacked_on_url(on_url)
             if not trace.is_quiet():
-                ui.ui_factory.note(
-                    "%s is now stacked on %s\n"
-                    % (branch.base, branch.get_stacked_on_url()))
+                ui.ui_factory.note(gettext(
+                    "{0} is now stacked on {1}\n").format(
+                      branch.base, branch.get_stacked_on_url()))
         finally:
             branch.unlock()
 
@@ -63,8 +63,8 @@
         try:
             branch.set_stacked_on_url(None)
             if not trace.is_quiet():
-                ui.ui_factory.note(
-                    "%s is now not stacked\n"
+                ui.ui_factory.note(gettext(
+                    "%s is now not stacked\n")
                     % (branch.base,))
         finally:
             branch.unlock()
@@ -344,7 +344,8 @@
                 reference_branch.repository.fetch(self.repository)
             elif self.local_branch is not None and not self._destroy_branch:
                 up = self.local_branch.user_transport.clone('..')
-                up_bzrdir = bzrdir.BzrDir.open_containing_from_transport(up)[0]
+                up_bzrdir = controldir.ControlDir.open_containing_from_transport(
+                    up)[0]
                 new_repo = up_bzrdir.find_repository()
                 new_repo.fetch(self.repository)
         last_revision_info = None

=== modified file 'bzrlib/registry.py'
--- bzrlib/registry.py	2009-09-16 10:37:29 +0000
+++ bzrlib/registry.py	2011-05-12 16:32:48 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,9 @@
 """Classes to provide name-to-object registry-like support."""
 
 
+from bzrlib.pyutils import get_named_object
+
+
 class _ObjectGetter(object):
     """Maintain a reference to an object, and return the object on request.
 
@@ -32,6 +35,10 @@
     def __init__(self, obj):
         self._obj = obj
 
+    def get_module(self):
+        """Get the module the object was loaded from."""
+        return self._obj.__module__
+
     def get_obj(self):
         """Get the object that was saved at creation time"""
         return self._obj
@@ -51,6 +58,11 @@
         self._imported = False
         super(_LazyObjectGetter, self).__init__(None)
 
+    def get_module(self):
+        """Get the module the referenced object will be loaded from.
+        """
+        return self._module_name
+
     def get_obj(self):
         """Get the referenced object.
 
@@ -58,26 +70,14 @@
         return the imported object.
         """
         if not self._imported:
-            self._do_import()
+            self._obj = get_named_object(self._module_name, self._member_name)
+            self._imported = True
         return super(_LazyObjectGetter, self).get_obj()
 
-    def _do_import(self):
-        if self._member_name:
-            segments = self._member_name.split('.')
-            names = segments[0:1]
-        else:
-            names = [self._member_name]
-        obj = __import__(self._module_name, globals(), locals(), names)
-        if self._member_name:
-            for segment in segments:
-                obj = getattr(obj, segment)
-        self._obj = obj
-        self._imported = True
-
     def __repr__(self):
-        return "<%s.%s object at %x, module=%r attribute=%r>" % (
+        return "<%s.%s object at %x, module=%r attribute=%r imported=%r>" % (
             self.__class__.__module__, self.__class__.__name__, id(self),
-            self._module_name, self._member_name)
+            self._module_name, self._member_name, self._imported)
 
 
 class Registry(object):
@@ -132,12 +132,15 @@
                       override_existing=False):
         """Register a new object to be loaded on request.
 
+        :param key: This is the key to use to request the object later.
         :param module_name: The python path to the module. Such as 'os.path'.
         :param member_name: The member of the module to return.  If empty or
                 None, get() will return the module itself.
         :param help: Help text for this entry. This may be a string or
                 a callable.
-        :param info: More information for this entry. Registry
+        :param info: More information for this entry. Registry.get_info()
+                can be used to get this information. Registry treats this as an
+                opaque storage location (it is defined by the caller).
         :param override_existing: If True, replace the existing object
                 with the new one. If False, if there is already something
                 registered with the same key, raise a KeyError
@@ -172,6 +175,14 @@
         """
         return self._dict[self._get_key_or_default(key)].get_obj()
 
+    def _get_module(self, key):
+        """Return the module the object will be or was loaded from.
+
+        :param key: The key to obtain the module for.
+        :return: The name of the module
+        """
+        return self._dict[key].get_module()
+
     def get_prefix(self, fullname):
         """Return an object whose key is a prefix of the supplied value.
 
@@ -248,6 +259,14 @@
         Registry.__init__(self)
         self._other_registry = other_registry
 
+    def register(self, key, obj, help=None, info=None,
+                 override_existing=False):
+        Registry.register(self, key, obj, help=help, info=info,
+            override_existing=override_existing)
+        if self._other_registry is not None:
+            self._other_registry.register(key, obj, help=help,
+                info=info, override_existing=override_existing)
+
     def register_lazy(self, key, module_name, member_name,
                       help=None, info=None,
                       override_existing=False):
@@ -259,10 +278,13 @@
             self._other_registry.register_lazy(key, module_name, member_name,
                 help=help, info=info, override_existing=override_existing)
 
+    def remove(self, key):
+        Registry.remove(self, key)
+        if self._other_registry is not None:
+            self._other_registry.remove(key)
+
     def get(self, format_string):
         r = Registry.get(self, format_string)
         if callable(r):
             r = r()
         return r
-
-

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2010-06-11 07:56:46 +0000
+++ bzrlib/remote.py	2011-09-18 14:57:54 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,34 +19,39 @@
 from bzrlib import (
     bencode,
     branch,
-    bzrdir,
+    bzrdir as _mod_bzrdir,
     config,
+    controldir,
     debug,
     errors,
     graph,
     lock,
     lockdir,
-    repository,
     repository as _mod_repository,
-    revision,
     revision as _mod_revision,
     static_tuple,
     symbol_versioning,
-)
+    urlutils,
+    vf_repository,
+    )
 from bzrlib.branch import BranchReferenceFormat, BranchWriteLockResult
-from bzrlib.bzrdir import BzrDir, RemoteBzrDirFormat
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.errors import (
     NoSuchRevision,
     SmartProtocolError,
     )
+from bzrlib.i18n import gettext
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.smart import client, vfs, repository as smart_repo
-from bzrlib.revision import ensure_null, NULL_REVISION
-from bzrlib.repository import RepositoryWriteLockResult
+from bzrlib.smart.client import _SmartClient
+from bzrlib.revision import NULL_REVISION
+from bzrlib.repository import RepositoryWriteLockResult, _LazyListJoin
 from bzrlib.trace import mutter, note, warning
 
 
+_DEFAULT_SEARCH_DEPTH = 100
+
+
 class _RpcHelper(object):
     """Mixin class that helps with issuing RPCs."""
 
@@ -87,9 +92,246 @@
     return format
 
 
-# Note: RemoteBzrDirFormat is in bzrdir.py
-
-class RemoteBzrDir(BzrDir, _RpcHelper):
+# Note that RemoteBzrDirProber lives in bzrlib.bzrdir so bzrlib.remote
+# does not have to be imported unless a remote format is involved.
+
+class RemoteBzrDirFormat(_mod_bzrdir.BzrDirMetaFormat1):
+    """Format representing bzrdirs accessed via a smart server"""
+
+    supports_workingtrees = False
+
+    def __init__(self):
+        _mod_bzrdir.BzrDirMetaFormat1.__init__(self)
+        # XXX: It's a bit ugly that the network name is here, because we'd
+        # like to believe that format objects are stateless or at least
+        # immutable,  However, we do at least avoid mutating the name after
+        # it's returned.  See <https://bugs.launchpad.net/bzr/+bug/504102>
+        self._network_name = None
+
+    def __repr__(self):
+        return "%s(_network_name=%r)" % (self.__class__.__name__,
+            self._network_name)
+
+    def get_format_description(self):
+        if self._network_name:
+            real_format = controldir.network_format_registry.get(self._network_name)
+            return 'Remote: ' + real_format.get_format_description()
+        return 'bzr remote bzrdir'
+
+    def get_format_string(self):
+        raise NotImplementedError(self.get_format_string)
+
+    def network_name(self):
+        if self._network_name:
+            return self._network_name
+        else:
+            raise AssertionError("No network name set.")
+
+    def initialize_on_transport(self, transport):
+        try:
+            # hand off the request to the smart server
+            client_medium = transport.get_smart_medium()
+        except errors.NoSmartMedium:
+            # TODO: lookup the local format from a server hint.
+            local_dir_format = _mod_bzrdir.BzrDirMetaFormat1()
+            return local_dir_format.initialize_on_transport(transport)
+        client = _SmartClient(client_medium)
+        path = client.remote_path_from_transport(transport)
+        try:
+            response = client.call('BzrDirFormat.initialize', path)
+        except errors.ErrorFromSmartServer, err:
+            _translate_error(err, path=path)
+        if response[0] != 'ok':
+            raise errors.SmartProtocolError('unexpected response code %s' % (response,))
+        format = RemoteBzrDirFormat()
+        self._supply_sub_formats_to(format)
+        return RemoteBzrDir(transport, format)
+
+    def parse_NoneTrueFalse(self, arg):
+        if not arg:
+            return None
+        if arg == 'False':
+            return False
+        if arg == 'True':
+            return True
+        raise AssertionError("invalid arg %r" % arg)
+
+    def _serialize_NoneTrueFalse(self, arg):
+        if arg is False:
+            return 'False'
+        if arg:
+            return 'True'
+        return ''
+
+    def _serialize_NoneString(self, arg):
+        return arg or ''
+
+    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
+        create_prefix=False, force_new_repo=False, stacked_on=None,
+        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
+        shared_repo=False):
+        try:
+            # hand off the request to the smart server
+            client_medium = transport.get_smart_medium()
+        except errors.NoSmartMedium:
+            do_vfs = True
+        else:
+            # Decline to open it if the server doesn't support our required
+            # version (3) so that the VFS-based transport will do it.
+            if client_medium.should_probe():
+                try:
+                    server_version = client_medium.protocol_version()
+                    if server_version != '2':
+                        do_vfs = True
+                    else:
+                        do_vfs = False
+                except errors.SmartProtocolError:
+                    # Apparently there's no usable smart server there, even though
+                    # the medium supports the smart protocol.
+                    do_vfs = True
+            else:
+                do_vfs = False
+        if not do_vfs:
+            client = _SmartClient(client_medium)
+            path = client.remote_path_from_transport(transport)
+            if client_medium._is_remote_before((1, 16)):
+                do_vfs = True
+        if do_vfs:
+            # TODO: lookup the local format from a server hint.
+            local_dir_format = _mod_bzrdir.BzrDirMetaFormat1()
+            self._supply_sub_formats_to(local_dir_format)
+            return local_dir_format.initialize_on_transport_ex(transport,
+                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
+                force_new_repo=force_new_repo, stacked_on=stacked_on,
+                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
+                make_working_trees=make_working_trees, shared_repo=shared_repo,
+                vfs_only=True)
+        return self._initialize_on_transport_ex_rpc(client, path, transport,
+            use_existing_dir, create_prefix, force_new_repo, stacked_on,
+            stack_on_pwd, repo_format_name, make_working_trees, shared_repo)
+
+    def _initialize_on_transport_ex_rpc(self, client, path, transport,
+        use_existing_dir, create_prefix, force_new_repo, stacked_on,
+        stack_on_pwd, repo_format_name, make_working_trees, shared_repo):
+        args = []
+        args.append(self._serialize_NoneTrueFalse(use_existing_dir))
+        args.append(self._serialize_NoneTrueFalse(create_prefix))
+        args.append(self._serialize_NoneTrueFalse(force_new_repo))
+        args.append(self._serialize_NoneString(stacked_on))
+        # stack_on_pwd is often/usually our transport
+        if stack_on_pwd:
+            try:
+                stack_on_pwd = transport.relpath(stack_on_pwd)
+                if not stack_on_pwd:
+                    stack_on_pwd = '.'
+            except errors.PathNotChild:
+                pass
+        args.append(self._serialize_NoneString(stack_on_pwd))
+        args.append(self._serialize_NoneString(repo_format_name))
+        args.append(self._serialize_NoneTrueFalse(make_working_trees))
+        args.append(self._serialize_NoneTrueFalse(shared_repo))
+        request_network_name = self._network_name or \
+            _mod_bzrdir.BzrDirFormat.get_default_format().network_name()
+        try:
+            response = client.call('BzrDirFormat.initialize_ex_1.16',
+                request_network_name, path, *args)
+        except errors.UnknownSmartMethod:
+            client._medium._remember_remote_is_before((1,16))
+            local_dir_format = _mod_bzrdir.BzrDirMetaFormat1()
+            self._supply_sub_formats_to(local_dir_format)
+            return local_dir_format.initialize_on_transport_ex(transport,
+                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
+                force_new_repo=force_new_repo, stacked_on=stacked_on,
+                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
+                make_working_trees=make_working_trees, shared_repo=shared_repo,
+                vfs_only=True)
+        except errors.ErrorFromSmartServer, err:
+            _translate_error(err, path=path)
+        repo_path = response[0]
+        bzrdir_name = response[6]
+        require_stacking = response[7]
+        require_stacking = self.parse_NoneTrueFalse(require_stacking)
+        format = RemoteBzrDirFormat()
+        format._network_name = bzrdir_name
+        self._supply_sub_formats_to(format)
+        bzrdir = RemoteBzrDir(transport, format, _client=client)
+        if repo_path:
+            repo_format = response_tuple_to_repo_format(response[1:])
+            if repo_path == '.':
+                repo_path = ''
+            if repo_path:
+                repo_bzrdir_format = RemoteBzrDirFormat()
+                repo_bzrdir_format._network_name = response[5]
+                repo_bzr = RemoteBzrDir(transport.clone(repo_path),
+                    repo_bzrdir_format)
+            else:
+                repo_bzr = bzrdir
+            final_stack = response[8] or None
+            final_stack_pwd = response[9] or None
+            if final_stack_pwd:
+                final_stack_pwd = urlutils.join(
+                    transport.base, final_stack_pwd)
+            remote_repo = RemoteRepository(repo_bzr, repo_format)
+            if len(response) > 10:
+                # Updated server verb that locks remotely.
+                repo_lock_token = response[10] or None
+                remote_repo.lock_write(repo_lock_token, _skip_rpc=True)
+                if repo_lock_token:
+                    remote_repo.dont_leave_lock_in_place()
+            else:
+                remote_repo.lock_write()
+            policy = _mod_bzrdir.UseExistingRepository(remote_repo, final_stack,
+                final_stack_pwd, require_stacking)
+            policy.acquire_repository()
+        else:
+            remote_repo = None
+            policy = None
+        bzrdir._format.set_branch_format(self.get_branch_format())
+        if require_stacking:
+            # The repo has already been created, but we need to make sure that
+            # we'll make a stackable branch.
+            bzrdir._format.require_stacking(_skip_repo=True)
+        return remote_repo, bzrdir, require_stacking, policy
+
+    def _open(self, transport):
+        return RemoteBzrDir(transport, self)
+
+    def __eq__(self, other):
+        if not isinstance(other, RemoteBzrDirFormat):
+            return False
+        return self.get_format_description() == other.get_format_description()
+
+    def __return_repository_format(self):
+        # Always return a RemoteRepositoryFormat object, but if a specific bzr
+        # repository format has been asked for, tell the RemoteRepositoryFormat
+        # that it should use that for init() etc.
+        result = RemoteRepositoryFormat()
+        custom_format = getattr(self, '_repository_format', None)
+        if custom_format:
+            if isinstance(custom_format, RemoteRepositoryFormat):
+                return custom_format
+            else:
+                # We will use the custom format to create repositories over the
+                # wire; expose its details like rich_root_data for code to
+                # query
+                result._custom_format = custom_format
+        return result
+
+    def get_branch_format(self):
+        result = _mod_bzrdir.BzrDirMetaFormat1.get_branch_format(self)
+        if not isinstance(result, RemoteBranchFormat):
+            new_result = RemoteBranchFormat()
+            new_result._custom_format = result
+            # cache the result
+            self.set_branch_format(new_result)
+            result = new_result
+        return result
+
+    repository_format = property(__return_repository_format,
+        _mod_bzrdir.BzrDirMetaFormat1._set_repository_format) #.im_func)
+
+
+class RemoteBzrDir(_mod_bzrdir.BzrDir, _RpcHelper):
     """Control directory on a remote server, accessed via bzr:// or similar."""
 
     def __init__(self, transport, format, _client=None, _force_probe=False):
@@ -98,7 +340,7 @@
         :param _client: Private parameter for testing. Disables probing and the
             use of a real bzrdir.
         """
-        BzrDir.__init__(self, transport, format)
+        _mod_bzrdir.BzrDir.__init__(self, transport, format)
         # this object holds a delegated bzrdir that uses file-level operations
         # to talk to the other side
         self._real_bzrdir = None
@@ -164,7 +406,7 @@
                 import traceback
                 warning('VFS BzrDir access triggered\n%s',
                     ''.join(traceback.format_stack()))
-            self._real_bzrdir = BzrDir.open_from_transport(
+            self._real_bzrdir = _mod_bzrdir.BzrDir.open_from_transport(
                 self.root_transport, _server_formats=False)
             self._format._network_name = \
                 self._real_bzrdir._format.network_name()
@@ -176,7 +418,7 @@
         # Prevent aliasing problems in the next_open_branch_result cache.
         # See create_branch for rationale.
         self._next_open_branch_result = None
-        return BzrDir.break_lock(self)
+        return _mod_bzrdir.BzrDir.break_lock(self)
 
     def _vfs_cloning_metadir(self, require_stacking=False):
         self._ensure_real()
@@ -213,14 +455,14 @@
         if len(branch_info) != 2:
             raise errors.UnexpectedSmartServerResponse(response)
         branch_ref, branch_name = branch_info
-        format = bzrdir.network_format_registry.get(control_name)
+        format = controldir.network_format_registry.get(control_name)
         if repo_name:
-            format.repository_format = repository.network_format_registry.get(
+            format.repository_format = _mod_repository.network_format_registry.get(
                 repo_name)
         if branch_ref == 'ref':
             # XXX: we need possible_transports here to avoid reopening the
             # connection to the referenced location
-            ref_bzrdir = BzrDir.open(branch_name)
+            ref_bzrdir = _mod_bzrdir.BzrDir.open(branch_name)
             branch_format = ref_bzrdir.cloning_metadir().get_branch_format()
             format.set_branch_format(branch_format)
         elif branch_ref == 'branch':
@@ -245,14 +487,19 @@
         self._ensure_real()
         self._real_bzrdir.destroy_repository()
 
-    def create_branch(self, name=None):
+    def create_branch(self, name=None, repository=None,
+                      append_revisions_only=None):
         # as per meta1 formats - just delegate to the format object which may
         # be parameterised.
         real_branch = self._format.get_branch_format().initialize(self,
-            name=name)
+            name=name, repository=repository,
+            append_revisions_only=append_revisions_only)
         if not isinstance(real_branch, RemoteBranch):
-            result = RemoteBranch(self, self.find_repository(), real_branch,
-                                  name=name)
+            if not isinstance(repository, RemoteRepository):
+                raise AssertionError(
+                    'need a RemoteRepository to use with RemoteBranch, got %r'
+                    % (repository,))
+            result = RemoteBranch(self, repository, real_branch, name=name)
         else:
             result = real_branch
         # BzrDir.clone_on_transport() uses the result of create_branch but does
@@ -270,7 +517,8 @@
         self._real_bzrdir.destroy_branch(name=name)
         self._next_open_branch_result = None
 
-    def create_workingtree(self, revision_id=None, from_branch=None):
+    def create_workingtree(self, revision_id=None, from_branch=None,
+        accelerator_tree=None, hardlink=False):
         raise errors.NotLocalUrl(self.transport.base)
 
     def find_branch_format(self, name=None):
@@ -427,7 +675,8 @@
 
     def _path_for_remote_call(self, client):
         """Return the path to be used for this bzrdir in a remote call."""
-        return client.remote_path_from_transport(self.root_transport)
+        return urlutils.split_segment_parameters_raw(
+            client.remote_path_from_transport(self.root_transport))[0]
 
     def get_branch_transport(self, branch_format, name=None):
         self._ensure_real()
@@ -445,11 +694,8 @@
         """Upgrading of remote bzrdirs is not supported yet."""
         return False
 
-    def needs_format_conversion(self, format=None):
+    def needs_format_conversion(self, format):
         """Upgrading of remote bzrdirs is not supported yet."""
-        if format is None:
-            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
-                % 'needs_format_conversion(format=None)')
         return False
 
     def clone(self, url, revision_id=None, force_new_repo=False,
@@ -462,7 +708,7 @@
         return RemoteBzrDirConfig(self)
 
 
-class RemoteRepositoryFormat(repository.RepositoryFormat):
+class RemoteRepositoryFormat(vf_repository.VersionedFileRepositoryFormat):
     """Format for repositories accessed over a _SmartClient.
 
     Instances of this repository are represented by RemoteRepository
@@ -483,15 +729,20 @@
     """
 
     _matchingbzrdir = RemoteBzrDirFormat()
+    supports_full_versioned_files = True
+    supports_leaving_lock = True
 
     def __init__(self):
-        repository.RepositoryFormat.__init__(self)
+        _mod_repository.RepositoryFormat.__init__(self)
         self._custom_format = None
         self._network_name = None
         self._creating_bzrdir = None
+        self._revision_graph_can_have_wrong_parents = None
         self._supports_chks = None
         self._supports_external_lookups = None
         self._supports_tree_reference = None
+        self._supports_funky_characters = None
+        self._supports_nesting_repositories = None
         self._rich_root_data = None
 
     def __repr__(self):
@@ -526,6 +777,22 @@
         return self._supports_external_lookups
 
     @property
+    def supports_funky_characters(self):
+        if self._supports_funky_characters is None:
+            self._ensure_real()
+            self._supports_funky_characters = \
+                self._custom_format.supports_funky_characters
+        return self._supports_funky_characters
+
+    @property
+    def supports_nesting_repositories(self):
+        if self._supports_nesting_repositories is None:
+            self._ensure_real()
+            self._supports_nesting_repositories = \
+                self._custom_format.supports_nesting_repositories
+        return self._supports_nesting_repositories
+
+    @property
     def supports_tree_reference(self):
         if self._supports_tree_reference is None:
             self._ensure_real()
@@ -533,6 +800,14 @@
                 self._custom_format.supports_tree_reference
         return self._supports_tree_reference
 
+    @property
+    def revision_graph_can_have_wrong_parents(self):
+        if self._revision_graph_can_have_wrong_parents is None:
+            self._ensure_real()
+            self._revision_graph_can_have_wrong_parents = \
+                self._custom_format.revision_graph_can_have_wrong_parents
+        return self._revision_graph_can_have_wrong_parents
+
     def _vfs_initialize(self, a_bzrdir, shared):
         """Helper for common code in initialize."""
         if self._custom_format:
@@ -573,7 +848,7 @@
             network_name = self._network_name
         else:
             # Select the current bzrlib default and ask for that.
-            reference_bzrdir_format = bzrdir.format_registry.get('default')()
+            reference_bzrdir_format = _mod_bzrdir.format_registry.get('default')()
             reference_format = reference_bzrdir_format.repository_format
             network_name = reference_format.network_name()
         # 2) try direct creation via RPC
@@ -605,7 +880,7 @@
 
     def _ensure_real(self):
         if self._custom_format is None:
-            self._custom_format = repository.network_format_registry.get(
+            self._custom_format = _mod_repository.network_format_registry.get(
                 self._network_name)
 
     @property
@@ -648,7 +923,7 @@
 
 
 class RemoteRepository(_RpcHelper, lock._RelockDebugMixin,
-    bzrdir.ControlComponent):
+    controldir.ControlComponent):
     """Repository accessed over rpc.
 
     For the moment most operations are performed using local transport-backed
@@ -707,7 +982,7 @@
         # transport, but I'm not sure it's worth making this method
         # optional -- mbp 2010-04-21
         return self.bzrdir.get_repository_transport(None)
-        
+
     def __str__(self):
         return "%s(%s)" % (self.__class__.__name__, self.base)
 
@@ -821,9 +1096,6 @@
     def find_text_key_references(self):
         """Find the text key references within the repository.
 
-        :return: a dictionary mapping (file_id, revision_id) tuples to altered file-ids to an iterable of
-        revision_ids. Each altered file-ids has the exact revision_ids that
-        altered it listed explicitly.
         :return: A dictionary mapping text keys ((fileid, revision_id) tuples)
             to whether they were referred to by the inventory of the
             revision_id that they contain. The inventory texts from all present
@@ -847,7 +1119,7 @@
         """Private method for using with old (< 1.2) servers to fallback."""
         if revision_id is None:
             revision_id = ''
-        elif revision.is_null(revision_id):
+        elif _mod_revision.is_null(revision_id):
             return {}
 
         path = self.bzrdir._path_for_remote_call(self._client)
@@ -877,6 +1149,10 @@
         return RemoteStreamSource(self, to_format)
 
     @needs_read_lock
+    def get_file_graph(self):
+        return graph.Graph(self.texts)
+
+    @needs_read_lock
     def has_revision(self, revision_id):
         """True if this repository has a copy of the revision."""
         # Copy of bzrlib.repository.Repository.has_revision
@@ -934,7 +1210,7 @@
         """See Repository.gather_stats()."""
         path = self.bzrdir._path_for_remote_call(self._client)
         # revid can be None to indicate no revisions, not just NULL_REVISION
-        if revid is None or revision.is_null(revid):
+        if revid is None or _mod_revision.is_null(revid):
             fmt_revid = ''
         else:
             fmt_revid = revid
@@ -1223,14 +1499,15 @@
 
     def get_commit_builder(self, branch, parents, config, timestamp=None,
                            timezone=None, committer=None, revprops=None,
-                           revision_id=None):
+                           revision_id=None, lossy=False):
         # FIXME: It ought to be possible to call this without immediately
         # triggering _ensure_real.  For now it's the easiest thing to do.
         self._ensure_real()
         real_repo = self._real_repository
         builder = real_repo.get_commit_builder(branch, parents,
                 config, timestamp=timestamp, timezone=timezone,
-                committer=committer, revprops=revprops, revision_id=revision_id)
+                committer=committer, revprops=revprops,
+                revision_id=revision_id, lossy=lossy)
         return builder
 
     def add_fallback_repository(self, repository):
@@ -1244,12 +1521,13 @@
         # We need to accumulate additional repositories here, to pass them in
         # on various RPC's.
         #
+        # Make the check before we lock: this raises an exception.
+        self._check_fallback_repository(repository)
         if self.is_locked():
             # We will call fallback.unlock() when we transition to the unlocked
             # state, so always add a lock here. If a caller passes us a locked
             # repository, they are responsible for unlocking it later.
             repository.lock_read()
-        self._check_fallback_repository(repository)
         self._fallback_repositories.append(repository)
         # If self._real_repository was parameterised already (e.g. because a
         # _real_branch had its get_stacked_on_url method called), then the
@@ -1342,17 +1620,32 @@
         return result
 
     @needs_read_lock
-    def search_missing_revision_ids(self, other, revision_id=None, find_ghosts=True):
+    def search_missing_revision_ids(self, other,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None,
+            limit=None):
         """Return the revision ids that other has that this does not.
 
         These are returned in topological order.
 
         revision_id: only return revision ids included by revision_id.
         """
-        return repository.InterRepository.get(
-            other, self).search_missing_revision_ids(revision_id, find_ghosts)
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.4.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        inter_repo = _mod_repository.InterRepository.get(other, self)
+        return inter_repo.search_missing_revision_ids(
+            find_ghosts=find_ghosts, revision_ids=revision_ids,
+            if_present_ids=if_present_ids, limit=limit)
 
-    def fetch(self, source, revision_id=None, pb=None, find_ghosts=False,
+    def fetch(self, source, revision_id=None, find_ghosts=False,
             fetch_spec=None):
         # No base implementation to use as RemoteRepository is not a subclass
         # of Repository; so this is a copy of Repository.fetch().
@@ -1369,14 +1662,14 @@
             # check that last_revision is in 'from' and then return a
             # no-operation.
             if (revision_id is not None and
-                not revision.is_null(revision_id)):
+                not _mod_revision.is_null(revision_id)):
                 self.get_revision(revision_id)
             return 0, []
         # if there is no specific appropriate InterRepository, this will get
         # the InterRepository base class, which raises an
         # IncompatibleRepositories when asked to fetch.
-        inter = repository.InterRepository.get(source, self)
-        return inter.fetch(revision_id=revision_id, pb=pb,
+        inter = _mod_repository.InterRepository.get(source, self)
+        return inter.fetch(revision_id=revision_id,
             find_ghosts=find_ghosts, fetch_spec=fetch_spec)
 
     def create_bundle(self, target, base, fileobj, format=None):
@@ -1384,6 +1677,8 @@
         self._real_repository.create_bundle(target, base, fileobj, format)
 
     @needs_read_lock
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def get_ancestry(self, revision_id, topo_sorted=True):
         self._ensure_real()
         return self._real_repository.get_ancestry(revision_id, topo_sorted)
@@ -1403,6 +1698,10 @@
         self._ensure_real()
         return self._real_repository.iter_files_bytes(desired_files)
 
+    def get_cached_parent_map(self, revision_ids):
+        """See bzrlib.CachingParentsProvider.get_cached_parent_map"""
+        return self._unstacked_provider.get_cached_parent_map(revision_ids)
+
     def get_parent_map(self, revision_ids):
         """See bzrlib.Graph.get_parent_map()."""
         return self._make_parents_provider().get_parent_map(revision_ids)
@@ -1466,26 +1765,15 @@
         if parents_map is None:
             # Repository is not locked, so there's no cache.
             parents_map = {}
-        # start_set is all the keys in the cache
-        start_set = set(parents_map)
-        # result set is all the references to keys in the cache
-        result_parents = set()
-        for parents in parents_map.itervalues():
-            result_parents.update(parents)
-        stop_keys = result_parents.difference(start_set)
-        # We don't need to send ghosts back to the server as a position to
-        # stop either.
-        stop_keys.difference_update(self._unstacked_provider.missing_keys)
-        key_count = len(parents_map)
-        if (NULL_REVISION in result_parents
-            and NULL_REVISION in self._unstacked_provider.missing_keys):
-            # If we pruned NULL_REVISION from the stop_keys because it's also
-            # in our cache of "missing" keys we need to increment our key count
-            # by 1, because the reconsitituted SearchResult on the server will
-            # still consider NULL_REVISION to be an included key.
-            key_count += 1
-        included_keys = start_set.intersection(result_parents)
-        start_set.difference_update(included_keys)
+        if _DEFAULT_SEARCH_DEPTH <= 0:
+            (start_set, stop_keys,
+             key_count) = graph.search_result_from_parent_map(
+                parents_map, self._unstacked_provider.missing_keys)
+        else:
+            (start_set, stop_keys,
+             key_count) = graph.limited_search_result_from_parent_map(
+                parents_map, self._unstacked_provider.missing_keys,
+                keys, depth=_DEFAULT_SEARCH_DEPTH)
         recipe = ('manual', start_set, stop_keys, key_count)
         body = self._serialise_search_recipe(recipe)
         path = self.bzrdir._path_for_remote_call(self._client)
@@ -1595,7 +1883,7 @@
         from bzrlib import osutils
         import tarfile
         # TODO: Maybe a progress bar while streaming the tarball?
-        note("Copying repository content as tarball...")
+        note(gettext("Copying repository content as tarball..."))
         tar_file = self._get_tarball('bz2')
         if tar_file is None:
             return None
@@ -1606,7 +1894,7 @@
             tmpdir = osutils.mkdtemp()
             try:
                 _extract_tar(tar, tmpdir)
-                tmp_bzrdir = BzrDir.open(tmpdir)
+                tmp_bzrdir = _mod_bzrdir.BzrDir.open(tmpdir)
                 tmp_repo = tmp_bzrdir.open_repository()
                 tmp_repo.copy_content_into(destination, revision_id)
             finally:
@@ -1697,6 +1985,7 @@
     def supports_rich_root(self):
         return self._format.rich_root_data
 
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def iter_reverse_revision_history(self, revision_id):
         self._ensure_real()
         return self._real_repository.iter_reverse_revision_history(revision_id)
@@ -1723,11 +2012,6 @@
         return self._real_repository.item_keys_introduced_by(revision_ids,
             _files_pb=_files_pb)
 
-    def revision_graph_can_have_wrong_parents(self):
-        # The answer depends on the remote repo format.
-        self._ensure_real()
-        return self._real_repository.revision_graph_can_have_wrong_parents()
-
     def _find_inconsistent_revision_parents(self, revisions_iterator=None):
         self._ensure_real()
         return self._real_repository._find_inconsistent_revision_parents(
@@ -1741,9 +2025,8 @@
         providers = [self._unstacked_provider]
         if other is not None:
             providers.insert(0, other)
-        providers.extend(r._make_parents_provider() for r in
-                         self._fallback_repositories)
-        return graph.StackedParentsProvider(providers)
+        return graph.StackedParentsProvider(_LazyListJoin(
+            providers, self._fallback_repositories))
 
     def _serialise_search_recipe(self, recipe):
         """Serialise a graph search recipe.
@@ -1757,12 +2040,7 @@
         return '\n'.join((start_keys, stop_keys, count))
 
     def _serialise_search_result(self, search_result):
-        if isinstance(search_result, graph.PendingAncestryResult):
-            parts = ['ancestry-of']
-            parts.extend(search_result.heads)
-        else:
-            recipe = search_result.get_recipe()
-            parts = [recipe[0], self._serialise_search_recipe(recipe)]
+        parts = search_result.get_network_struct()
         return '\n'.join(parts)
 
     def autopack(self):
@@ -1778,7 +2056,7 @@
             raise errors.UnexpectedSmartServerResponse(response)
 
 
-class RemoteStreamSink(repository.StreamSink):
+class RemoteStreamSink(vf_repository.StreamSink):
 
     def _insert_real(self, stream, src_format, resume_tokens):
         self.target_repo._ensure_real()
@@ -1885,7 +2163,7 @@
         self._last_substream and self._last_stream so that the stream can be
         resumed by _resume_stream_with_vfs.
         """
-                    
+
         stream_iter = iter(stream)
         for substream_kind, substream in stream_iter:
             if substream_kind == 'inventory-deltas':
@@ -1894,9 +2172,9 @@
                 return
             else:
                 yield substream_kind, substream
-            
-
-class RemoteStreamSource(repository.StreamSource):
+
+
+class RemoteStreamSource(vf_repository.StreamSource):
     """Stream data from a remote server."""
 
     def get_stream(self, search):
@@ -1962,6 +2240,7 @@
         candidate_verbs = [
             ('Repository.get_stream_1.19', (1, 19)),
             ('Repository.get_stream', (1, 13))]
+
         found_verb = False
         for verb, version in candidate_verbs:
             if medium._is_remote_before(version):
@@ -1971,6 +2250,17 @@
                     verb, args, search_bytes)
             except errors.UnknownSmartMethod:
                 medium._remember_remote_is_before(version)
+            except errors.UnknownErrorFromSmartServer, e:
+                if isinstance(search, graph.EverythingResult):
+                    error_verb = e.error_from_smart_server.error_verb
+                    if error_verb == 'BadSearch':
+                        # Pre-2.4 servers don't support this sort of search.
+                        # XXX: perhaps falling back to VFS on BadSearch is a
+                        # good idea in general?  It might provide a little bit
+                        # of protection against client-side bugs.
+                        medium._remember_remote_is_before((2, 4))
+                        break
+                raise
             else:
                 response_tuple, response_handler = response
                 found_verb = True
@@ -2073,7 +2363,7 @@
         return a_bzrdir.open_branch(name=name, 
             ignore_fallbacks=ignore_fallbacks)
 
-    def _vfs_initialize(self, a_bzrdir, name):
+    def _vfs_initialize(self, a_bzrdir, name, append_revisions_only):
         # Initialisation when using a local bzrdir object, or a non-vfs init
         # method is not available on the server.
         # self._custom_format is always set - the start of initialize ensures
@@ -2081,32 +2371,36 @@
         if isinstance(a_bzrdir, RemoteBzrDir):
             a_bzrdir._ensure_real()
             result = self._custom_format.initialize(a_bzrdir._real_bzrdir,
-                name)
+                name, append_revisions_only=append_revisions_only)
         else:
             # We assume the bzrdir is parameterised; it may not be.
-            result = self._custom_format.initialize(a_bzrdir, name)
+            result = self._custom_format.initialize(a_bzrdir, name,
+                append_revisions_only=append_revisions_only)
         if (isinstance(a_bzrdir, RemoteBzrDir) and
             not isinstance(result, RemoteBranch)):
             result = RemoteBranch(a_bzrdir, a_bzrdir.find_repository(), result,
                                   name=name)
         return result
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         # 1) get the network name to use.
         if self._custom_format:
             network_name = self._custom_format.network_name()
         else:
             # Select the current bzrlib default and ask for that.
-            reference_bzrdir_format = bzrdir.format_registry.get('default')()
+            reference_bzrdir_format = _mod_bzrdir.format_registry.get('default')()
             reference_format = reference_bzrdir_format.get_branch_format()
             self._custom_format = reference_format
             network_name = reference_format.network_name()
         # Being asked to create on a non RemoteBzrDir:
         if not isinstance(a_bzrdir, RemoteBzrDir):
-            return self._vfs_initialize(a_bzrdir, name=name)
+            return self._vfs_initialize(a_bzrdir, name=name,
+                append_revisions_only=append_revisions_only)
         medium = a_bzrdir._client._medium
         if medium._is_remote_before((1, 13)):
-            return self._vfs_initialize(a_bzrdir, name=name)
+            return self._vfs_initialize(a_bzrdir, name=name,
+                append_revisions_only=append_revisions_only)
         # Creating on a remote bzr dir.
         # 2) try direct creation via RPC
         path = a_bzrdir._path_for_remote_call(a_bzrdir._client)
@@ -2119,21 +2413,36 @@
         except errors.UnknownSmartMethod:
             # Fallback - use vfs methods
             medium._remember_remote_is_before((1, 13))
-            return self._vfs_initialize(a_bzrdir, name=name)
+            return self._vfs_initialize(a_bzrdir, name=name,
+                    append_revisions_only=append_revisions_only)
         if response[0] != 'ok':
             raise errors.UnexpectedSmartServerResponse(response)
         # Turn the response into a RemoteRepository object.
         format = RemoteBranchFormat(network_name=response[1])
         repo_format = response_tuple_to_repo_format(response[3:])
-        if response[2] == '':
-            repo_bzrdir = a_bzrdir
+        repo_path = response[2]
+        if repository is not None:
+            remote_repo_url = urlutils.join(a_bzrdir.user_url, repo_path)
+            url_diff = urlutils.relative_url(repository.user_url,
+                    remote_repo_url)
+            if url_diff != '.':
+                raise AssertionError(
+                    'repository.user_url %r does not match URL from server '
+                    'response (%r + %r)'
+                    % (repository.user_url, a_bzrdir.user_url, repo_path))
+            remote_repo = repository
         else:
-            repo_bzrdir = RemoteBzrDir(
-                a_bzrdir.root_transport.clone(response[2]), a_bzrdir._format,
-                a_bzrdir._client)
-        remote_repo = RemoteRepository(repo_bzrdir, repo_format)
+            if repo_path == '':
+                repo_bzrdir = a_bzrdir
+            else:
+                repo_bzrdir = RemoteBzrDir(
+                    a_bzrdir.root_transport.clone(repo_path), a_bzrdir._format,
+                    a_bzrdir._client)
+            remote_repo = RemoteRepository(repo_bzrdir, repo_format)
         remote_branch = RemoteBranch(a_bzrdir, remote_repo,
             format=format, setup_stacking=False, name=name)
+        if append_revisions_only:
+            remote_branch.set_append_revisions_only(append_revisions_only)
         # XXX: We know this is a new branch, so it must have revno 0, revid
         # NULL_REVISION. Creating the branch locked would make this be unable
         # to be wrong; here its simply very unlikely to be wrong. RBC 20090225
@@ -2158,6 +2467,19 @@
         self._ensure_real()
         return self._custom_format.supports_set_append_revisions_only()
 
+    def _use_default_local_heads_to_fetch(self):
+        # If the branch format is a metadir format *and* its heads_to_fetch
+        # implementation is not overridden vs the base class, we can use the
+        # base class logic rather than use the heads_to_fetch RPC.  This is
+        # usually cheaper in terms of net round trips, as the last-revision and
+        # tags info fetched is cached and would be fetched anyway.
+        self._ensure_real()
+        if isinstance(self._custom_format, branch.BranchFormatMetadir):
+            branch_class = self._custom_format._branch_class()
+            heads_to_fetch_impl = branch_class.heads_to_fetch.im_func
+            if heads_to_fetch_impl is branch.Branch.heads_to_fetch.im_func:
+                return True
+        return False
 
 class RemoteBranch(branch.Branch, _RpcHelper, lock._RelockDebugMixin):
     """Branch stored on a server accessed by HPSS RPC.
@@ -2321,9 +2643,16 @@
                 self.bzrdir, self._client)
         return self._control_files
 
-    def _get_checkout_format(self):
+    def _get_checkout_format(self, lightweight=False):
         self._ensure_real()
-        return self._real_branch._get_checkout_format()
+        if lightweight:
+            format = RemoteBzrDirFormat()
+            self.bzrdir._format._supply_sub_formats_to(format)
+            format.workingtree_format = self._real_branch._get_checkout_format(
+                lightweight=lightweight).workingtree_format
+            return format
+        else:
+            return self._real_branch._get_checkout_format(lightweight=False)
 
     def get_physical_lock_status(self):
         """See Branch.get_physical_lock_status()."""
@@ -2362,12 +2691,18 @@
             self._is_stacked = False
         else:
             self._is_stacked = True
-        
+
     def _vfs_get_tags_bytes(self):
         self._ensure_real()
         return self._real_branch._get_tags_bytes()
 
+    @needs_read_lock
     def _get_tags_bytes(self):
+        if self._tags_bytes is None:
+            self._tags_bytes = self._get_tags_bytes_via_hpss()
+        return self._tags_bytes
+
+    def _get_tags_bytes_via_hpss(self):
         medium = self._client._medium
         if medium._is_remote_before((1, 13)):
             return self._vfs_get_tags_bytes()
@@ -2383,6 +2718,8 @@
         return self._real_branch._set_tags_bytes(bytes)
 
     def _set_tags_bytes(self, bytes):
+        if self.is_locked():
+            self._tags_bytes = bytes
         medium = self._client._medium
         if medium._is_remote_before((1, 18)):
             self._vfs_set_tags_bytes(bytes)
@@ -2542,7 +2879,7 @@
             missing_parent = parent_map[missing_parent]
         raise errors.RevisionNotPresent(missing_parent, self.repository)
 
-    def _last_revision_info(self):
+    def _read_last_revision_info(self):
         response = self._call('Branch.last_revision_info', self._remote_path())
         if response[0] != 'ok':
             raise SmartProtocolError('unexpected response code %s' % (response,))
@@ -2611,8 +2948,14 @@
             raise errors.UnexpectedSmartServerResponse(response)
         self._run_post_change_branch_tip_hooks(old_revno, old_revid)
 
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     @needs_write_lock
     def set_revision_history(self, rev_history):
+        """See Branch.set_revision_history."""
+        self._set_revision_history(rev_history)
+
+    @needs_write_lock
+    def _set_revision_history(self, rev_history):
         # Send just the tip revision of the history; the server will generate
         # the full history from that.  If the revision doesn't exist in this
         # branch, NoSuchRevision will be raised.
@@ -2676,10 +3019,10 @@
             _override_hook_target=self, **kwargs)
 
     @needs_read_lock
-    def push(self, target, overwrite=False, stop_revision=None):
+    def push(self, target, overwrite=False, stop_revision=None, lossy=False):
         self._ensure_real()
         return self._real_branch.push(
-            target, overwrite=overwrite, stop_revision=stop_revision,
+            target, overwrite=overwrite, stop_revision=stop_revision, lossy=lossy,
             _override_hook_source_branch=self)
 
     def is_locked(self):
@@ -2695,7 +3038,8 @@
         # XXX: These should be returned by the set_last_revision_info verb
         old_revno, old_revid = self.last_revision_info()
         self._run_pre_change_branch_tip_hooks(revno, revision_id)
-        revision_id = ensure_null(revision_id)
+        if not revision_id or not isinstance(revision_id, basestring):
+            raise errors.InvalidRevisionId(revision_id=revision_id, branch=self)
         try:
             response = self._call('Branch.set_last_revision_info',
                 self._remote_path(), self._lock_token, self._repo_lock_token,
@@ -2730,13 +3074,40 @@
             except errors.UnknownSmartMethod:
                 medium._remember_remote_is_before((1, 6))
         self._clear_cached_state_of_remote_branch_only()
-        self.set_revision_history(self._lefthand_history(revision_id,
+        self._set_revision_history(self._lefthand_history(revision_id,
             last_rev=last_rev,other_branch=other_branch))
 
     def set_push_location(self, location):
         self._ensure_real()
         return self._real_branch.set_push_location(location)
 
+    def heads_to_fetch(self):
+        if self._format._use_default_local_heads_to_fetch():
+            # We recognise this format, and its heads-to-fetch implementation
+            # is the default one (tip + tags).  In this case it's cheaper to
+            # just use the default implementation rather than a special RPC as
+            # the tip and tags data is cached.
+            return branch.Branch.heads_to_fetch(self)
+        medium = self._client._medium
+        if medium._is_remote_before((2, 4)):
+            return self._vfs_heads_to_fetch()
+        try:
+            return self._rpc_heads_to_fetch()
+        except errors.UnknownSmartMethod:
+            medium._remember_remote_is_before((2, 4))
+            return self._vfs_heads_to_fetch()
+
+    def _rpc_heads_to_fetch(self):
+        response = self._call('Branch.heads_to_fetch', self._remote_path())
+        if len(response) != 2:
+            raise errors.UnexpectedSmartServerResponse(response)
+        must_fetch, if_present_fetch = response
+        return set(must_fetch), set(if_present_fetch)
+
+    def _vfs_heads_to_fetch(self):
+        self._ensure_real()
+        return self._real_branch.heads_to_fetch()
+
 
 class RemoteConfig(object):
     """A Config that reads and writes from smart verbs.
@@ -2756,22 +3127,32 @@
         """
         try:
             configobj = self._get_configobj()
+            section_obj = None
             if section is None:
                 section_obj = configobj
             else:
                 try:
                     section_obj = configobj[section]
                 except KeyError:
-                    return default
-            return section_obj.get(name, default)
+                    pass
+            if section_obj is None:
+                value = default
+            else:
+                value = section_obj.get(name, default)
         except errors.UnknownSmartMethod:
-            return self._vfs_get_option(name, section, default)
+            value = self._vfs_get_option(name, section, default)
+        for hook in config.OldConfigHooks['get']:
+            hook(self, name, value)
+        return value
 
     def _response_to_configobj(self, response):
         if len(response[0]) and response[0][0] != 'ok':
             raise errors.UnexpectedSmartServerResponse(response)
         lines = response[1].read_body_bytes().splitlines()
-        return config.ConfigObj(lines, encoding='utf-8')
+        conf = config.ConfigObj(lines, encoding='utf-8')
+        for hook in config.OldConfigHooks['load']:
+            hook(self)
+        return conf
 
 
 class RemoteBranchConfig(RemoteConfig):
@@ -2929,10 +3310,7 @@
                     'Missing key %r in context %r', key_err.args[0], context)
                 raise err
 
-    if err.error_verb == 'IncompatibleRepositories':
-        raise errors.IncompatibleRepositories(err.error_args[0],
-            err.error_args[1], err.error_args[2])
-    elif err.error_verb == 'NoSuchRevision':
+    if err.error_verb == 'NoSuchRevision':
         raise NoSuchRevision(find('branch'), err.error_args[0])
     elif err.error_verb == 'nosuchrevision':
         raise NoSuchRevision(find('repository'), err.error_args[0])
@@ -2945,22 +3323,12 @@
             detail=extra)
     elif err.error_verb == 'norepository':
         raise errors.NoRepositoryPresent(find('bzrdir'))
-    elif err.error_verb == 'LockContention':
-        raise errors.LockContention('(remote lock)')
     elif err.error_verb == 'UnlockableTransport':
         raise errors.UnlockableTransport(find('bzrdir').root_transport)
-    elif err.error_verb == 'LockFailed':
-        raise errors.LockFailed(err.error_args[0], err.error_args[1])
     elif err.error_verb == 'TokenMismatch':
         raise errors.TokenMismatch(find('token'), '(remote token)')
     elif err.error_verb == 'Diverged':
         raise errors.DivergedBranches(find('branch'), find('other_branch'))
-    elif err.error_verb == 'TipChangeRejected':
-        raise errors.TipChangeRejected(err.error_args[0].decode('utf8'))
-    elif err.error_verb == 'UnstackableBranchFormat':
-        raise errors.UnstackableBranchFormat(*err.error_args)
-    elif err.error_verb == 'UnstackableRepositoryFormat':
-        raise errors.UnstackableRepositoryFormat(*err.error_args)
     elif err.error_verb == 'NotStacked':
         raise errors.NotStacked(branch=find('branch'))
     elif err.error_verb == 'PermissionDenied':
@@ -2976,6 +3344,24 @@
     elif err.error_verb == 'NoSuchFile':
         path = get_path()
         raise errors.NoSuchFile(path)
+    _translate_error_without_context(err)
+
+
+def _translate_error_without_context(err):
+    """Translate any ErrorFromSmartServer values that don't require context"""
+    if err.error_verb == 'IncompatibleRepositories':
+        raise errors.IncompatibleRepositories(err.error_args[0],
+            err.error_args[1], err.error_args[2])
+    elif err.error_verb == 'LockContention':
+        raise errors.LockContention('(remote lock)')
+    elif err.error_verb == 'LockFailed':
+        raise errors.LockFailed(err.error_args[0], err.error_args[1])
+    elif err.error_verb == 'TipChangeRejected':
+        raise errors.TipChangeRejected(err.error_args[0].decode('utf8'))
+    elif err.error_verb == 'UnstackableBranchFormat':
+        raise errors.UnstackableBranchFormat(*err.error_args)
+    elif err.error_verb == 'UnstackableRepositoryFormat':
+        raise errors.UnstackableRepositoryFormat(*err.error_args)
     elif err.error_verb == 'FileExists':
         raise errors.FileExists(err.error_args[0])
     elif err.error_verb == 'DirectoryNotEmpty':
@@ -3000,4 +3386,7 @@
             raise UnicodeEncodeError(encoding, val, start, end, reason)
     elif err.error_verb == 'ReadOnlyError':
         raise errors.TransportNotPossible('readonly transport')
+    elif err.error_verb == 'MemoryError':
+        raise errors.BzrError("remote server out of memory\n"
+            "Retry non-remotely, or contact the server admin for details.")
     raise errors.UnknownErrorFromSmartServer(err)

=== modified file 'bzrlib/rename_map.py'
--- bzrlib/rename_map.py	2009-03-24 15:47:32 +0000
+++ bzrlib/rename_map.py	2011-09-16 10:08:09 +0000
@@ -23,7 +23,7 @@
     trace,
 )
 from bzrlib.ui import ui_factory
-
+from bzrlib.i18n import gettext
 
 class RenameMap(object):
     """Determine a mapping of renames."""
@@ -65,7 +65,7 @@
         try:
             for num, (file_id, contents) in enumerate(
                 tree.iter_files_bytes(desired_files)):
-                task.update('Calculating hashes', num, len(file_ids))
+                task.update(gettext('Calculating hashes'), num, len(file_ids))
                 s = StringIO()
                 s.writelines(contents)
                 s.seek(0)
@@ -103,7 +103,7 @@
         task = ui_factory.nested_progress_bar()
         try:
             for num, path in enumerate(paths):
-                task.update('Determining hash hits', num, len(paths))
+                task.update(gettext('Determining hash hits'), num, len(paths))
                 hits = self.hitcounts(self.tree.get_file_lines(None,
                                                                path=path))
                 all_hits.extend((v, path, k) for k, v in hits.items())
@@ -238,7 +238,7 @@
             pp.next_phase()
             delta = rn._make_inventory_delta(matches)
             for old, new, file_id, entry in delta:
-                trace.note("%s => %s", old, new)
+                trace.note( gettext("{0} => {1}").format(old, new) )
             if not dry_run:
                 tree.add(required_parents)
                 tree.apply_inventory_delta(delta)

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2010-05-14 13:25:05 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2011-05-09 12:01:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,32 +26,36 @@
     errors,
     index as _mod_index,
     inventory,
-    knit,
     osutils,
     pack,
     revision as _mod_revision,
     trace,
     ui,
+    versionedfile,
     )
 from bzrlib.btree_index import (
     BTreeGraphIndex,
     BTreeBuilder,
     )
+from bzrlib.decorators import needs_write_lock
 from bzrlib.groupcompress import (
     _GCGraphIndex,
     GroupCompressVersionedFiles,
     )
 from bzrlib.repofmt.pack_repo import (
+    _DirectPackAccess,
     Pack,
     NewPack,
-    KnitPackRepository,
-    KnitPackStreamSource,
+    PackRepository,
     PackRootCommitBuilder,
     RepositoryPackCollection,
     RepositoryFormatPack,
     ResumedPack,
     Packer,
     )
+from bzrlib.vf_repository import (
+    StreamSource,
+    )
 from bzrlib.static_tuple import StaticTuple
 
 
@@ -350,8 +354,8 @@
         """Build a VersionedFiles instance on top of this group of packs."""
         index_name = index_name + '_index'
         index_to_pack = {}
-        access = knit._DirectPackAccess(index_to_pack,
-                                        reload_func=self._reload_func)
+        access = _DirectPackAccess(index_to_pack,
+                                   reload_func=self._reload_func)
         if for_write:
             # Use new_pack
             if self.new_pack is None:
@@ -419,14 +423,26 @@
         inventory_keys = source_vf.keys()
         missing_inventories = set(self.revision_keys).difference(inventory_keys)
         if missing_inventories:
-            missing_inventories = sorted(missing_inventories)
-            raise ValueError('We are missing inventories for revisions: %s'
-                % (missing_inventories,))
+            # Go back to the original repo, to see if these are really missing
+            # https://bugs.launchpad.net/bzr/+bug/437003
+            # If we are packing a subset of the repo, it is fine to just have
+            # the data in another Pack file, which is not included in this pack
+            # operation.
+            inv_index = self._pack_collection.repo.inventories._index
+            pmap = inv_index.get_parent_map(missing_inventories)
+            really_missing = missing_inventories.difference(pmap)
+            if really_missing:
+                missing_inventories = sorted(really_missing)
+                raise ValueError('We are missing inventories for revisions: %s'
+                    % (missing_inventories,))
         self._copy_stream(source_vf, target_vf, inventory_keys,
                           'inventories', self._get_filtered_inv_stream, 2)
 
+    def _get_chk_vfs_for_copy(self):
+        return self._build_vfs('chk', False, False)
+
     def _copy_chk_texts(self):
-        source_vf, target_vf = self._build_vfs('chk', False, False)
+        source_vf, target_vf = self._get_chk_vfs_for_copy()
         # TODO: This is technically spurious... if it is a performance issue,
         #       remove it
         total_keys = source_vf.keys()
@@ -578,10 +594,117 @@
         return new_pack.data_inserted() and self._data_changed
 
 
+class GCCHKCanonicalizingPacker(GCCHKPacker):
+    """A packer that ensures inventories have canonical-form CHK maps.
+    
+    Ideally this would be part of reconcile, but it's very slow and rarely
+    needed.  (It repairs repositories affected by
+    https://bugs.launchpad.net/bzr/+bug/522637).
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(GCCHKCanonicalizingPacker, self).__init__(*args, **kwargs)
+        self._data_changed = False
+
+    def _exhaust_stream(self, source_vf, keys, message, vf_to_stream, pb_offset):
+        """Create and exhaust a stream, but don't insert it.
+
+        This is useful to get the side-effects of generating a stream.
+        """
+        self.pb.update('scanning %s' % (message,), pb_offset)
+        child_pb = ui.ui_factory.nested_progress_bar()
+        try:
+            list(vf_to_stream(source_vf, keys, message, child_pb))
+        finally:
+            child_pb.finished()
+
+    def _copy_inventory_texts(self):
+        source_vf, target_vf = self._build_vfs('inventory', True, True)
+        source_chk_vf, target_chk_vf = self._get_chk_vfs_for_copy()
+        inventory_keys = source_vf.keys()
+        # First, copy the existing CHKs on the assumption that most of them
+        # will be correct.  This will save us from having to reinsert (and
+        # recompress) these records later at the cost of perhaps preserving a
+        # few unused CHKs. 
+        # (Iterate but don't insert _get_filtered_inv_stream to populate the
+        # variables needed by GCCHKPacker._copy_chk_texts.)
+        self._exhaust_stream(source_vf, inventory_keys, 'inventories',
+                self._get_filtered_inv_stream, 2)
+        GCCHKPacker._copy_chk_texts(self)
+        # Now copy and fix the inventories, and any regenerated CHKs.
+        def chk_canonicalizing_inv_stream(source_vf, keys, message, pb=None):
+            return self._get_filtered_canonicalizing_inv_stream(
+                source_vf, keys, message, pb, source_chk_vf, target_chk_vf)
+        self._copy_stream(source_vf, target_vf, inventory_keys,
+                          'inventories', chk_canonicalizing_inv_stream, 4)
+
+    def _copy_chk_texts(self):
+        # No-op; in this class this happens during _copy_inventory_texts.
+        pass
+
+    def _get_filtered_canonicalizing_inv_stream(self, source_vf, keys, message,
+            pb=None, source_chk_vf=None, target_chk_vf=None):
+        """Filter the texts of inventories, regenerating CHKs to make sure they
+        are canonical.
+        """
+        total_keys = len(keys)
+        target_chk_vf = versionedfile.NoDupeAddLinesDecorator(target_chk_vf)
+        def _filtered_inv_stream():
+            stream = source_vf.get_record_stream(keys, 'groupcompress', True)
+            search_key_name = None
+            for idx, record in enumerate(stream):
+                # Inventories should always be with revisions; assume success.
+                bytes = record.get_bytes_as('fulltext')
+                chk_inv = inventory.CHKInventory.deserialise(
+                    source_chk_vf, bytes, record.key)
+                if pb is not None:
+                    pb.update('inv', idx, total_keys)
+                chk_inv.id_to_entry._ensure_root()
+                if search_key_name is None:
+                    # Find the name corresponding to the search_key_func
+                    search_key_reg = chk_map.search_key_registry
+                    for search_key_name, func in search_key_reg.iteritems():
+                        if func == chk_inv.id_to_entry._search_key_func:
+                            break
+                canonical_inv = inventory.CHKInventory.from_inventory(
+                    target_chk_vf, chk_inv,
+                    maximum_size=chk_inv.id_to_entry._root_node._maximum_size,
+                    search_key_name=search_key_name)
+                if chk_inv.id_to_entry.key() != canonical_inv.id_to_entry.key():
+                    trace.mutter(
+                        'Non-canonical CHK map for id_to_entry of inv: %s '
+                        '(root is %s, should be %s)' % (chk_inv.revision_id,
+                        chk_inv.id_to_entry.key()[0],
+                        canonical_inv.id_to_entry.key()[0]))
+                    self._data_changed = True
+                p_id_map = chk_inv.parent_id_basename_to_file_id
+                p_id_map._ensure_root()
+                canon_p_id_map = canonical_inv.parent_id_basename_to_file_id
+                if p_id_map.key() != canon_p_id_map.key():
+                    trace.mutter(
+                        'Non-canonical CHK map for parent_id_to_basename of '
+                        'inv: %s (root is %s, should be %s)'
+                        % (chk_inv.revision_id, p_id_map.key()[0],
+                           canon_p_id_map.key()[0]))
+                    self._data_changed = True
+                yield versionedfile.ChunkedContentFactory(record.key,
+                        record.parents, record.sha1,
+                        canonical_inv.to_lines())
+            # We have finished processing all of the inventory records, we
+            # don't need these sets anymore
+        return _filtered_inv_stream()
+
+    def _use_pack(self, new_pack):
+        """Override _use_pack to check for reconcile having changed content."""
+        return new_pack.data_inserted() and self._data_changed
+
+
 class GCRepositoryPackCollection(RepositoryPackCollection):
 
     pack_factory = GCPack
     resumed_pack_factory = ResumedGCPack
+    normal_packer_class = GCCHKPacker
+    optimising_packer_class = GCCHKPacker
 
     def _check_new_inventories(self):
         """Detect missing inventories or chk root entries for the new revisions
@@ -669,56 +792,15 @@
                 % (sorted(missing_text_keys),))
         return problems
 
-    def _execute_pack_operations(self, pack_operations,
-                                 _packer_class=GCCHKPacker,
-                                 reload_func=None):
-        """Execute a series of pack operations.
-
-        :param pack_operations: A list of [revision_count, packs_to_combine].
-        :param _packer_class: The class of packer to use (default: Packer).
-        :return: None.
-        """
-        # XXX: Copied across from RepositoryPackCollection simply because we
-        #      want to override the _packer_class ... :(
-        for revision_count, packs in pack_operations:
-            # we may have no-ops from the setup logic
-            if len(packs) == 0:
-                continue
-            packer = GCCHKPacker(self, packs, '.autopack',
-                                 reload_func=reload_func)
-            try:
-                result = packer.pack()
-            except errors.RetryWithNewPacks:
-                # An exception is propagating out of this context, make sure
-                # this packer has cleaned up. Packer() doesn't set its new_pack
-                # state into the RepositoryPackCollection object, so we only
-                # have access to it directly here.
-                if packer.new_pack is not None:
-                    packer.new_pack.abort()
-                raise
-            if result is None:
-                return
-            for pack in packs:
-                self._remove_pack_from_memory(pack)
-        # record the newly available packs and stop advertising the old
-        # packs
-        to_be_obsoleted = []
-        for _, packs in pack_operations:
-            to_be_obsoleted.extend(packs)
-        result = self._save_pack_names(clear_obsolete_packs=True,
-                                       obsolete_packs=to_be_obsoleted)
-        return result
-
-
-class CHKInventoryRepository(KnitPackRepository):
-    """subclass of KnitPackRepository that uses CHK based inventories."""
+
+class CHKInventoryRepository(PackRepository):
+    """subclass of PackRepository that uses CHK based inventories."""
 
     def __init__(self, _format, a_bzrdir, control_files, _commit_builder_class,
         _serializer):
         """Overridden to change pack collection class."""
-        KnitPackRepository.__init__(self, _format, a_bzrdir, control_files,
-            _commit_builder_class, _serializer)
-        # and now replace everything it did :)
+        super(CHKInventoryRepository, self).__init__(_format, a_bzrdir,
+            control_files, _commit_builder_class, _serializer)
         index_transport = self._transport.clone('indices')
         self._pack_collection = GCRepositoryPackCollection(self,
             self._transport, index_transport,
@@ -999,10 +1081,24 @@
         finally:
             pb.finished()
 
+    @needs_write_lock
+    def reconcile_canonicalize_chks(self):
+        """Reconcile this repository to make sure all CHKs are in canonical
+        form.
+        """
+        from bzrlib.reconcile import PackReconciler
+        reconciler = PackReconciler(self, thorough=True, canonicalize_chks=True)
+        reconciler.reconcile()
+        return reconciler
+
     def _reconcile_pack(self, collection, packs, extension, revs, pb):
         packer = GCCHKReconcilePacker(collection, packs, extension)
         return packer.pack(pb)
 
+    def _canonicalize_chks_pack(self, collection, packs, extension, revs, pb):
+        packer = GCCHKCanonicalizingPacker(collection, packs, extension, revs)
+        return packer.pack(pb)
+
     def _get_source(self, to_format):
         """Return a source for streaming from this repository."""
         if self._format._serializer == to_format._serializer:
@@ -1013,8 +1109,39 @@
             return GroupCHKStreamSource(self, to_format)
         return super(CHKInventoryRepository, self)._get_source(to_format)
 
-
-class GroupCHKStreamSource(KnitPackStreamSource):
+    def _find_inconsistent_revision_parents(self, revisions_iterator=None):
+        """Find revisions with different parent lists in the revision object
+        and in the index graph.
+
+        :param revisions_iterator: None, or an iterator of (revid,
+            Revision-or-None). This iterator controls the revisions checked.
+        :returns: an iterator yielding tuples of (revison-id, parents-in-index,
+            parents-in-revision).
+        """
+        if not self.is_locked():
+            raise AssertionError()
+        vf = self.revisions
+        if revisions_iterator is None:
+            revisions_iterator = self._iter_revisions(None)
+        for revid, revision in revisions_iterator:
+            if revision is None:
+                pass
+            parent_map = vf.get_parent_map([(revid,)])
+            parents_according_to_index = tuple(parent[-1] for parent in
+                parent_map[(revid,)])
+            parents_according_to_revision = tuple(revision.parent_ids)
+            if parents_according_to_index != parents_according_to_revision:
+                yield (revid, parents_according_to_index,
+                    parents_according_to_revision)
+
+    def _check_for_inconsistent_revision_parents(self):
+        inconsistencies = list(self._find_inconsistent_revision_parents())
+        if inconsistencies:
+            raise errors.BzrCheckError(
+                "Revision index has inconsistent parents.")
+
+
+class GroupCHKStreamSource(StreamSource):
     """Used when both the source and target repo are GroupCHK repos."""
 
     def __init__(self, from_repository, to_format):
@@ -1107,6 +1234,13 @@
             self._chk_p_id_roots = None
         yield 'chk_bytes', _get_parent_id_basename_to_file_id_pages()
 
+    def _get_text_stream(self):
+        # Note: We know we don't have to handle adding root keys, because both
+        # the source and target are the identical network name.
+        text_stream = self.from_repository.texts.get_record_stream(
+                        self._text_keys, self._text_fetch_order, False)
+        return ('texts', text_stream)
+
     def get_stream(self, search):
         def wrap_and_count(pb, rc, stream):
             """Yield records from stream while showing progress."""
@@ -1127,17 +1261,20 @@
             yield (stream_info[0],
                 wrap_and_count(pb, rc, stream_info[1]))
         self._revision_keys = [(rev_id,) for rev_id in revision_ids]
-        self.from_repository.revisions.clear_cache()
-        self.from_repository.signatures.clear_cache()
-        s = self._get_inventory_stream(self._revision_keys)
-        yield (s[0], wrap_and_count(pb, rc, s[1]))
-        self.from_repository.inventories.clear_cache()
         # TODO: The keys to exclude might be part of the search recipe
         # For now, exclude all parents that are at the edge of ancestry, for
         # which we have inventories
         from_repo = self.from_repository
         parent_keys = from_repo._find_parent_keys_of_revisions(
                         self._revision_keys)
+        self.from_repository.revisions.clear_cache()
+        self.from_repository.signatures.clear_cache()
+        # Clear the repo's get_parent_map cache too.
+        self.from_repository._unstacked_provider.disable_cache()
+        self.from_repository._unstacked_provider.enable_cache()
+        s = self._get_inventory_stream(self._revision_keys)
+        yield (s[0], wrap_and_count(pb, rc, s[1]))
+        self.from_repository.inventories.clear_cache()
         for stream_info in self._get_filtered_chk_streams(parent_keys):
             yield (stream_info[0], wrap_and_count(pb, rc, stream_info[1]))
         self.from_repository.chk_bytes.clear_cache()
@@ -1215,19 +1352,15 @@
         yield record
 
 
-
-
-class RepositoryFormatCHK1(RepositoryFormatPack):
-    """A hashed CHK+group compress pack repository."""
+class RepositoryFormat2a(RepositoryFormatPack):
+    """A CHK repository that uses the bencode revision serializer."""
 
     repository_class = CHKInventoryRepository
     supports_external_lookups = True
     supports_chks = True
-    # For right now, setting this to True gives us InterModel1And2 rather
-    # than InterDifferingSerializer
     _commit_builder_class = PackRootCommitBuilder
     rich_root_data = True
-    _serializer = chk_serializer.chk_serializer_255_bigpage
+    _serializer = chk_serializer.chk_bencode_serializer
     _commit_inv_deltas = True
     # What index classes to use
     index_builder_class = BTreeBuilder
@@ -1244,52 +1377,6 @@
     pack_compresses = True
 
     def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir('development6-rich-root')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return ('Bazaar development format - group compression and chk inventory'
-                ' (needs bzr.dev from 1.14)\n')
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return ("Development repository format - rich roots, group compression"
-            " and chk inventories")
-
-
-class RepositoryFormatCHK2(RepositoryFormatCHK1):
-    """A CHK repository that uses the bencode revision serializer."""
-
-    _serializer = chk_serializer.chk_bencode_serializer
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir('development7-rich-root')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return ('Bazaar development format - chk repository with bencode '
-                'revision serialization (needs bzr.dev from 1.16)\n')
-
-
-class RepositoryFormat2a(RepositoryFormatCHK2):
-    """A CHK repository that uses the bencode revision serializer.
-
-    This is the same as RepositoryFormatCHK2 but with a public name.
-    """
-
-    _serializer = chk_serializer.chk_bencode_serializer
-
-    def _get_matching_bzrdir(self):
         return bzrdir.format_registry.make_bzrdir('2a')
 
     def _ignore_setting_bzrdir(self, format):
@@ -1304,3 +1391,28 @@
         """See RepositoryFormat.get_format_description()."""
         return ("Repository format 2a - rich roots, group compression"
             " and chk inventories")
+
+
+class RepositoryFormat2aSubtree(RepositoryFormat2a):
+    """A 2a repository format that supports nested trees.
+
+    """
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir('development-subtree')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        return ('Bazaar development format 8\n')
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return ("Development repository format 8 - nested trees, "
+                "group compression and chk inventories")
+
+    experimental = True
+    supports_tree_reference = True

=== added file 'bzrlib/repofmt/knitpack_repo.py'
--- bzrlib/repofmt/knitpack_repo.py	1970-01-01 00:00:00 +0000
+++ bzrlib/repofmt/knitpack_repo.py	2011-05-02 15:29:58 +0000
@@ -0,0 +1,1145 @@
+# Copyright (C) 2007-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Knit-based pack repository formats."""
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from itertools import izip
+import time
+
+from bzrlib import (
+    bzrdir,
+    debug,
+    errors,
+    knit,
+    osutils,
+    pack,
+    revision as _mod_revision,
+    trace,
+    tsort,
+    ui,
+    xml5,
+    xml6,
+    xml7,
+    )
+from bzrlib.knit import (
+    _KnitGraphIndex,
+    KnitPlainFactory,
+    KnitVersionedFiles,
+    )
+""")
+
+from bzrlib import (
+    btree_index,
+    )
+from bzrlib.index import (
+    CombinedGraphIndex,
+    GraphIndex,
+    GraphIndexPrefixAdapter,
+    InMemoryGraphIndex,
+    )
+from bzrlib.repofmt.knitrepo import (
+    KnitRepository,
+    )
+from bzrlib.repofmt.pack_repo import (
+    _DirectPackAccess,
+    NewPack,
+    RepositoryFormatPack,
+    ResumedPack,
+    Packer,
+    PackCommitBuilder,
+    PackRepository,
+    PackRootCommitBuilder,
+    RepositoryPackCollection,
+    )
+from bzrlib.vf_repository import (
+    StreamSource,
+    )
+
+
+class KnitPackRepository(PackRepository, KnitRepository):
+
+    def __init__(self, _format, a_bzrdir, control_files, _commit_builder_class,
+        _serializer):
+        PackRepository.__init__(self, _format, a_bzrdir, control_files,
+            _commit_builder_class, _serializer)
+        if self._format.supports_chks:
+            raise AssertionError("chk not supported")
+        index_transport = self._transport.clone('indices')
+        self._pack_collection = KnitRepositoryPackCollection(self,
+            self._transport,
+            index_transport,
+            self._transport.clone('upload'),
+            self._transport.clone('packs'),
+            _format.index_builder_class,
+            _format.index_class,
+            use_chk_index=False,
+            )
+        self.inventories = KnitVersionedFiles(
+            _KnitGraphIndex(self._pack_collection.inventory_index.combined_index,
+                add_callback=self._pack_collection.inventory_index.add_callback,
+                deltas=True, parents=True, is_locked=self.is_locked),
+            data_access=self._pack_collection.inventory_index.data_access,
+            max_delta_chain=200)
+        self.revisions = KnitVersionedFiles(
+            _KnitGraphIndex(self._pack_collection.revision_index.combined_index,
+                add_callback=self._pack_collection.revision_index.add_callback,
+                deltas=False, parents=True, is_locked=self.is_locked,
+                track_external_parent_refs=True),
+            data_access=self._pack_collection.revision_index.data_access,
+            max_delta_chain=0)
+        self.signatures = KnitVersionedFiles(
+            _KnitGraphIndex(self._pack_collection.signature_index.combined_index,
+                add_callback=self._pack_collection.signature_index.add_callback,
+                deltas=False, parents=False, is_locked=self.is_locked),
+            data_access=self._pack_collection.signature_index.data_access,
+            max_delta_chain=0)
+        self.texts = KnitVersionedFiles(
+            _KnitGraphIndex(self._pack_collection.text_index.combined_index,
+                add_callback=self._pack_collection.text_index.add_callback,
+                deltas=True, parents=True, is_locked=self.is_locked),
+            data_access=self._pack_collection.text_index.data_access,
+            max_delta_chain=200)
+        self.chk_bytes = None
+        # True when the repository object is 'write locked' (as opposed to the
+        # physical lock only taken out around changes to the pack-names list.)
+        # Another way to represent this would be a decorator around the control
+        # files object that presents logical locks as physical ones - if this
+        # gets ugly consider that alternative design. RBC 20071011
+        self._write_lock_count = 0
+        self._transaction = None
+        # for tests
+        self._reconcile_does_inventory_gc = True
+        self._reconcile_fixes_text_parents = True
+        self._reconcile_backsup_inventory = False
+
+    def _get_source(self, to_format):
+        if to_format.network_name() == self._format.network_name():
+            return KnitPackStreamSource(self, to_format)
+        return PackRepository._get_source(self, to_format)
+
+    def _reconcile_pack(self, collection, packs, extension, revs, pb):
+        packer = KnitReconcilePacker(collection, packs, extension, revs)
+        return packer.pack(pb)
+
+
+class RepositoryFormatKnitPack1(RepositoryFormatPack):
+    """A no-subtrees parameterized Pack repository.
+
+    This format was introduced in 0.92.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackCommitBuilder
+    @property
+    def _serializer(self):
+        return xml5.serializer_v5
+    # What index classes to use
+    index_builder_class = InMemoryGraphIndex
+    index_class = GraphIndex
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir('pack-0.92')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar pack repository format 1 (needs bzr 0.92)\n"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Packs containing knits without subtree support"
+
+
+class RepositoryFormatKnitPack3(RepositoryFormatPack):
+    """A subtrees parameterized Pack repository.
+
+    This repository format uses the xml7 serializer to get:
+     - support for recording full info about the tree root
+     - support for recording tree-references
+
+    This format was introduced in 0.92.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackRootCommitBuilder
+    rich_root_data = True
+    experimental = True
+    supports_tree_reference = True
+    @property
+    def _serializer(self):
+        return xml7.serializer_v7
+    # What index classes to use
+    index_builder_class = InMemoryGraphIndex
+    index_class = GraphIndex
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir(
+            'pack-0.92-subtree')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar pack repository format 1 with subtree support (needs bzr 0.92)\n"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Packs containing knits with subtree support\n"
+
+
+class RepositoryFormatKnitPack4(RepositoryFormatPack):
+    """A rich-root, no subtrees parameterized Pack repository.
+
+    This repository format uses the xml6 serializer to get:
+     - support for recording full info about the tree root
+
+    This format was introduced in 1.0.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackRootCommitBuilder
+    rich_root_data = True
+    supports_tree_reference = False
+    @property
+    def _serializer(self):
+        return xml6.serializer_v6
+    # What index classes to use
+    index_builder_class = InMemoryGraphIndex
+    index_class = GraphIndex
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir(
+            'rich-root-pack')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return ("Bazaar pack repository format 1 with rich root"
+                " (needs bzr 1.0)\n")
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Packs containing knits with rich root support\n"
+
+
+class RepositoryFormatKnitPack5(RepositoryFormatPack):
+    """Repository that supports external references to allow stacking.
+
+    New in release 1.6.
+
+    Supports external lookups, which results in non-truncated ghosts after
+    reconcile compared to pack-0.92 formats.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackCommitBuilder
+    supports_external_lookups = True
+    # What index classes to use
+    index_builder_class = InMemoryGraphIndex
+    index_class = GraphIndex
+
+    @property
+    def _serializer(self):
+        return xml5.serializer_v5
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir('1.6')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar RepositoryFormatKnitPack5 (bzr 1.6)\n"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Packs 5 (adds stacking support, requires bzr 1.6)"
+
+
+class RepositoryFormatKnitPack5RichRoot(RepositoryFormatPack):
+    """A repository with rich roots and stacking.
+
+    New in release 1.6.1.
+
+    Supports stacking on other repositories, allowing data to be accessed
+    without being stored locally.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackRootCommitBuilder
+    rich_root_data = True
+    supports_tree_reference = False # no subtrees
+    supports_external_lookups = True
+    # What index classes to use
+    index_builder_class = InMemoryGraphIndex
+    index_class = GraphIndex
+
+    @property
+    def _serializer(self):
+        return xml6.serializer_v6
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir(
+            '1.6.1-rich-root')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6.1)\n"
+
+    def get_format_description(self):
+        return "Packs 5 rich-root (adds stacking support, requires bzr 1.6.1)"
+
+
+class RepositoryFormatKnitPack5RichRootBroken(RepositoryFormatPack):
+    """A repository with rich roots and external references.
+
+    New in release 1.6.
+
+    Supports external lookups, which results in non-truncated ghosts after
+    reconcile compared to pack-0.92 formats.
+
+    This format was deprecated because the serializer it uses accidentally
+    supported subtrees, when the format was not intended to. This meant that
+    someone could accidentally fetch from an incorrect repository.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackRootCommitBuilder
+    rich_root_data = True
+    supports_tree_reference = False # no subtrees
+
+    supports_external_lookups = True
+    # What index classes to use
+    index_builder_class = InMemoryGraphIndex
+    index_class = GraphIndex
+
+    @property
+    def _serializer(self):
+        return xml7.serializer_v7
+
+    def _get_matching_bzrdir(self):
+        matching = bzrdir.format_registry.make_bzrdir(
+            '1.6.1-rich-root')
+        matching.repository_format = self
+        return matching
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6)\n"
+
+    def get_format_description(self):
+        return ("Packs 5 rich-root (adds stacking support, requires bzr 1.6)"
+                " (deprecated)")
+
+    def is_deprecated(self):
+        return True
+
+
+class RepositoryFormatKnitPack6(RepositoryFormatPack):
+    """A repository with stacking and btree indexes,
+    without rich roots or subtrees.
+
+    This is equivalent to pack-1.6 with B+Tree indices.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackCommitBuilder
+    supports_external_lookups = True
+    # What index classes to use
+    index_builder_class = btree_index.BTreeBuilder
+    index_class = btree_index.BTreeGraphIndex
+
+    @property
+    def _serializer(self):
+        return xml5.serializer_v5
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir('1.9')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar RepositoryFormatKnitPack6 (bzr 1.9)\n"
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return "Packs 6 (uses btree indexes, requires bzr 1.9)"
+
+
+class RepositoryFormatKnitPack6RichRoot(RepositoryFormatPack):
+    """A repository with rich roots, no subtrees, stacking and btree indexes.
+
+    1.6-rich-root with B+Tree indices.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackRootCommitBuilder
+    rich_root_data = True
+    supports_tree_reference = False # no subtrees
+    supports_external_lookups = True
+    # What index classes to use
+    index_builder_class = btree_index.BTreeBuilder
+    index_class = btree_index.BTreeGraphIndex
+
+    @property
+    def _serializer(self):
+        return xml6.serializer_v6
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir(
+            '1.9-rich-root')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Bazaar RepositoryFormatKnitPack6RichRoot (bzr 1.9)\n"
+
+    def get_format_description(self):
+        return "Packs 6 rich-root (uses btree indexes, requires bzr 1.9)"
+
+
+class RepositoryFormatPackDevelopment2Subtree(RepositoryFormatPack):
+    """A subtrees development repository.
+
+    This format should be retained in 2.3, to provide an upgrade path from this
+    to RepositoryFormat2aSubtree.  It can be removed in later releases.
+
+    1.6.1-subtree[as it might have been] with B+Tree indices.
+    """
+
+    repository_class = KnitPackRepository
+    _commit_builder_class = PackRootCommitBuilder
+    rich_root_data = True
+    experimental = True
+    supports_tree_reference = True
+    supports_external_lookups = True
+    # What index classes to use
+    index_builder_class = btree_index.BTreeBuilder
+    index_class = btree_index.BTreeGraphIndex
+
+    @property
+    def _serializer(self):
+        return xml7.serializer_v7
+
+    def _get_matching_bzrdir(self):
+        return bzrdir.format_registry.make_bzrdir(
+            'development5-subtree')
+
+    def _ignore_setting_bzrdir(self, format):
+        pass
+
+    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return ("Bazaar development format 2 with subtree support "
+            "(needs bzr.dev from before 1.8)\n")
+
+    def get_format_description(self):
+        """See RepositoryFormat.get_format_description()."""
+        return ("Development repository format, currently the same as "
+            "1.6.1-subtree with B+Tree indices.\n")
+
+
+class KnitPackStreamSource(StreamSource):
+    """A StreamSource used to transfer data between same-format KnitPack repos.
+
+    This source assumes:
+        1) Same serialization format for all objects
+        2) Same root information
+        3) XML format inventories
+        4) Atomic inserts (so we can stream inventory texts before text
+           content)
+        5) No chk_bytes
+    """
+
+    def __init__(self, from_repository, to_format):
+        super(KnitPackStreamSource, self).__init__(from_repository, to_format)
+        self._text_keys = None
+        self._text_fetch_order = 'unordered'
+
+    def _get_filtered_inv_stream(self, revision_ids):
+        from_repo = self.from_repository
+        parent_ids = from_repo._find_parent_ids_of_revisions(revision_ids)
+        parent_keys = [(p,) for p in parent_ids]
+        find_text_keys = from_repo._serializer._find_text_key_references
+        parent_text_keys = set(find_text_keys(
+            from_repo._inventory_xml_lines_for_keys(parent_keys)))
+        content_text_keys = set()
+        knit = KnitVersionedFiles(None, None)
+        factory = KnitPlainFactory()
+        def find_text_keys_from_content(record):
+            if record.storage_kind not in ('knit-delta-gz', 'knit-ft-gz'):
+                raise ValueError("Unknown content storage kind for"
+                    " inventory text: %s" % (record.storage_kind,))
+            # It's a knit record, it has a _raw_record field (even if it was
+            # reconstituted from a network stream).
+            raw_data = record._raw_record
+            # read the entire thing
+            revision_id = record.key[-1]
+            content, _ = knit._parse_record(revision_id, raw_data)
+            if record.storage_kind == 'knit-delta-gz':
+                line_iterator = factory.get_linedelta_content(content)
+            elif record.storage_kind == 'knit-ft-gz':
+                line_iterator = factory.get_fulltext_content(content)
+            content_text_keys.update(find_text_keys(
+                [(line, revision_id) for line in line_iterator]))
+        revision_keys = [(r,) for r in revision_ids]
+        def _filtered_inv_stream():
+            source_vf = from_repo.inventories
+            stream = source_vf.get_record_stream(revision_keys,
+                                                 'unordered', False)
+            for record in stream:
+                if record.storage_kind == 'absent':
+                    raise errors.NoSuchRevision(from_repo, record.key)
+                find_text_keys_from_content(record)
+                yield record
+            self._text_keys = content_text_keys - parent_text_keys
+        return ('inventories', _filtered_inv_stream())
+
+    def _get_text_stream(self):
+        # Note: We know we don't have to handle adding root keys, because both
+        # the source and target are the identical network name.
+        text_stream = self.from_repository.texts.get_record_stream(
+                        self._text_keys, self._text_fetch_order, False)
+        return ('texts', text_stream)
+
+    def get_stream(self, search):
+        revision_ids = search.get_keys()
+        for stream_info in self._fetch_revision_texts(revision_ids):
+            yield stream_info
+        self._revision_keys = [(rev_id,) for rev_id in revision_ids]
+        yield self._get_filtered_inv_stream(revision_ids)
+        yield self._get_text_stream()
+
+
+class KnitPacker(Packer):
+    """Packer that works with knit packs."""
+
+    def __init__(self, pack_collection, packs, suffix, revision_ids=None,
+                 reload_func=None):
+        super(KnitPacker, self).__init__(pack_collection, packs, suffix,
+                                          revision_ids=revision_ids,
+                                          reload_func=reload_func)
+
+    def _pack_map_and_index_list(self, index_attribute):
+        """Convert a list of packs to an index pack map and index list.
+
+        :param index_attribute: The attribute that the desired index is found
+            on.
+        :return: A tuple (map, list) where map contains the dict from
+            index:pack_tuple, and list contains the indices in the preferred
+            access order.
+        """
+        indices = []
+        pack_map = {}
+        for pack_obj in self.packs:
+            index = getattr(pack_obj, index_attribute)
+            indices.append(index)
+            pack_map[index] = pack_obj
+        return pack_map, indices
+
+    def _index_contents(self, indices, key_filter=None):
+        """Get an iterable of the index contents from a pack_map.
+
+        :param indices: The list of indices to query
+        :param key_filter: An optional filter to limit the keys returned.
+        """
+        all_index = CombinedGraphIndex(indices)
+        if key_filter is None:
+            return all_index.iter_all_entries()
+        else:
+            return all_index.iter_entries(key_filter)
+
+    def _copy_nodes(self, nodes, index_map, writer, write_index,
+        output_lines=None):
+        """Copy knit nodes between packs with no graph references.
+
+        :param output_lines: Output full texts of copied items.
+        """
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            return self._do_copy_nodes(nodes, index_map, writer,
+                write_index, pb, output_lines=output_lines)
+        finally:
+            pb.finished()
+
+    def _do_copy_nodes(self, nodes, index_map, writer, write_index, pb,
+        output_lines=None):
+        # for record verification
+        knit = KnitVersionedFiles(None, None)
+        # plan a readv on each source pack:
+        # group by pack
+        nodes = sorted(nodes)
+        # how to map this into knit.py - or knit.py into this?
+        # we don't want the typical knit logic, we want grouping by pack
+        # at this point - perhaps a helper library for the following code
+        # duplication points?
+        request_groups = {}
+        for index, key, value in nodes:
+            if index not in request_groups:
+                request_groups[index] = []
+            request_groups[index].append((key, value))
+        record_index = 0
+        pb.update("Copied record", record_index, len(nodes))
+        for index, items in request_groups.iteritems():
+            pack_readv_requests = []
+            for key, value in items:
+                # ---- KnitGraphIndex.get_position
+                bits = value[1:].split(' ')
+                offset, length = int(bits[0]), int(bits[1])
+                pack_readv_requests.append((offset, length, (key, value[0])))
+            # linear scan up the pack
+            pack_readv_requests.sort()
+            # copy the data
+            pack_obj = index_map[index]
+            transport, path = pack_obj.access_tuple()
+            try:
+                reader = pack.make_readv_reader(transport, path,
+                    [offset[0:2] for offset in pack_readv_requests])
+            except errors.NoSuchFile:
+                if self._reload_func is not None:
+                    self._reload_func()
+                raise
+            for (names, read_func), (_1, _2, (key, eol_flag)) in \
+                izip(reader.iter_records(), pack_readv_requests):
+                raw_data = read_func(None)
+                # check the header only
+                if output_lines is not None:
+                    output_lines(knit._parse_record(key[-1], raw_data)[0])
+                else:
+                    df, _ = knit._parse_record_header(key, raw_data)
+                    df.close()
+                pos, size = writer.add_bytes_record(raw_data, names)
+                write_index.add_node(key, eol_flag + "%d %d" % (pos, size))
+                pb.update("Copied record", record_index)
+                record_index += 1
+
+    def _copy_nodes_graph(self, index_map, writer, write_index,
+        readv_group_iter, total_items, output_lines=False):
+        """Copy knit nodes between packs.
+
+        :param output_lines: Return lines present in the copied data as
+            an iterator of line,version_id.
+        """
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            for result in self._do_copy_nodes_graph(index_map, writer,
+                write_index, output_lines, pb, readv_group_iter, total_items):
+                yield result
+        except Exception:
+            # Python 2.4 does not permit try:finally: in a generator.
+            pb.finished()
+            raise
+        else:
+            pb.finished()
+
+    def _do_copy_nodes_graph(self, index_map, writer, write_index,
+        output_lines, pb, readv_group_iter, total_items):
+        # for record verification
+        knit = KnitVersionedFiles(None, None)
+        # for line extraction when requested (inventories only)
+        if output_lines:
+            factory = KnitPlainFactory()
+        record_index = 0
+        pb.update("Copied record", record_index, total_items)
+        for index, readv_vector, node_vector in readv_group_iter:
+            # copy the data
+            pack_obj = index_map[index]
+            transport, path = pack_obj.access_tuple()
+            try:
+                reader = pack.make_readv_reader(transport, path, readv_vector)
+            except errors.NoSuchFile:
+                if self._reload_func is not None:
+                    self._reload_func()
+                raise
+            for (names, read_func), (key, eol_flag, references) in \
+                izip(reader.iter_records(), node_vector):
+                raw_data = read_func(None)
+                if output_lines:
+                    # read the entire thing
+                    content, _ = knit._parse_record(key[-1], raw_data)
+                    if len(references[-1]) == 0:
+                        line_iterator = factory.get_fulltext_content(content)
+                    else:
+                        line_iterator = factory.get_linedelta_content(content)
+                    for line in line_iterator:
+                        yield line, key
+                else:
+                    # check the header only
+                    df, _ = knit._parse_record_header(key, raw_data)
+                    df.close()
+                pos, size = writer.add_bytes_record(raw_data, names)
+                write_index.add_node(key, eol_flag + "%d %d" % (pos, size), references)
+                pb.update("Copied record", record_index)
+                record_index += 1
+
+    def _process_inventory_lines(self, inv_lines):
+        """Use up the inv_lines generator and setup a text key filter."""
+        repo = self._pack_collection.repo
+        fileid_revisions = repo._find_file_ids_from_xml_inventory_lines(
+            inv_lines, self.revision_keys)
+        text_filter = []
+        for fileid, file_revids in fileid_revisions.iteritems():
+            text_filter.extend([(fileid, file_revid) for file_revid in file_revids])
+        self._text_filter = text_filter
+
+    def _copy_inventory_texts(self):
+        # select inventory keys
+        inv_keys = self._revision_keys # currently the same keyspace, and note that
+        # querying for keys here could introduce a bug where an inventory item
+        # is missed, so do not change it to query separately without cross
+        # checking like the text key check below.
+        inventory_index_map, inventory_indices = self._pack_map_and_index_list(
+            'inventory_index')
+        inv_nodes = self._index_contents(inventory_indices, inv_keys)
+        # copy inventory keys and adjust values
+        # XXX: Should be a helper function to allow different inv representation
+        # at this point.
+        self.pb.update("Copying inventory texts", 2)
+        total_items, readv_group_iter = self._least_readv_node_readv(inv_nodes)
+        # Only grab the output lines if we will be processing them
+        output_lines = bool(self.revision_ids)
+        inv_lines = self._copy_nodes_graph(inventory_index_map,
+            self.new_pack._writer, self.new_pack.inventory_index,
+            readv_group_iter, total_items, output_lines=output_lines)
+        if self.revision_ids:
+            self._process_inventory_lines(inv_lines)
+        else:
+            # eat the iterator to cause it to execute.
+            list(inv_lines)
+            self._text_filter = None
+        if 'pack' in debug.debug_flags:
+            trace.mutter('%s: create_pack: inventories copied: %s%s %d items t+%6.3fs',
+                time.ctime(), self._pack_collection._upload_transport.base,
+                self.new_pack.random_name,
+                self.new_pack.inventory_index.key_count(),
+                time.time() - self.new_pack.start_time)
+
+    def _update_pack_order(self, entries, index_to_pack_map):
+        """Determine how we want our packs to be ordered.
+
+        This changes the sort order of the self.packs list so that packs unused
+        by 'entries' will be at the end of the list, so that future requests
+        can avoid probing them.  Used packs will be at the front of the
+        self.packs list, in the order of their first use in 'entries'.
+
+        :param entries: A list of (index, ...) tuples
+        :param index_to_pack_map: A mapping from index objects to pack objects.
+        """
+        packs = []
+        seen_indexes = set()
+        for entry in entries:
+            index = entry[0]
+            if index not in seen_indexes:
+                packs.append(index_to_pack_map[index])
+                seen_indexes.add(index)
+        if len(packs) == len(self.packs):
+            if 'pack' in debug.debug_flags:
+                trace.mutter('Not changing pack list, all packs used.')
+            return
+        seen_packs = set(packs)
+        for pack in self.packs:
+            if pack not in seen_packs:
+                packs.append(pack)
+                seen_packs.add(pack)
+        if 'pack' in debug.debug_flags:
+            old_names = [p.access_tuple()[1] for p in self.packs]
+            new_names = [p.access_tuple()[1] for p in packs]
+            trace.mutter('Reordering packs\nfrom: %s\n  to: %s',
+                   old_names, new_names)
+        self.packs = packs
+
+    def _copy_revision_texts(self):
+        # select revisions
+        if self.revision_ids:
+            revision_keys = [(revision_id,) for revision_id in self.revision_ids]
+        else:
+            revision_keys = None
+        # select revision keys
+        revision_index_map, revision_indices = self._pack_map_and_index_list(
+            'revision_index')
+        revision_nodes = self._index_contents(revision_indices, revision_keys)
+        revision_nodes = list(revision_nodes)
+        self._update_pack_order(revision_nodes, revision_index_map)
+        # copy revision keys and adjust values
+        self.pb.update("Copying revision texts", 1)
+        total_items, readv_group_iter = self._revision_node_readv(revision_nodes)
+        list(self._copy_nodes_graph(revision_index_map, self.new_pack._writer,
+            self.new_pack.revision_index, readv_group_iter, total_items))
+        if 'pack' in debug.debug_flags:
+            trace.mutter('%s: create_pack: revisions copied: %s%s %d items t+%6.3fs',
+                time.ctime(), self._pack_collection._upload_transport.base,
+                self.new_pack.random_name,
+                self.new_pack.revision_index.key_count(),
+                time.time() - self.new_pack.start_time)
+        self._revision_keys = revision_keys
+
+    def _get_text_nodes(self):
+        text_index_map, text_indices = self._pack_map_and_index_list(
+            'text_index')
+        return text_index_map, self._index_contents(text_indices,
+            self._text_filter)
+
+    def _copy_text_texts(self):
+        # select text keys
+        text_index_map, text_nodes = self._get_text_nodes()
+        if self._text_filter is not None:
+            # We could return the keys copied as part of the return value from
+            # _copy_nodes_graph but this doesn't work all that well with the
+            # need to get line output too, so we check separately, and as we're
+            # going to buffer everything anyway, we check beforehand, which
+            # saves reading knit data over the wire when we know there are
+            # mising records.
+            text_nodes = set(text_nodes)
+            present_text_keys = set(_node[1] for _node in text_nodes)
+            missing_text_keys = set(self._text_filter) - present_text_keys
+            if missing_text_keys:
+                # TODO: raise a specific error that can handle many missing
+                # keys.
+                trace.mutter("missing keys during fetch: %r", missing_text_keys)
+                a_missing_key = missing_text_keys.pop()
+                raise errors.RevisionNotPresent(a_missing_key[1],
+                    a_missing_key[0])
+        # copy text keys and adjust values
+        self.pb.update("Copying content texts", 3)
+        total_items, readv_group_iter = self._least_readv_node_readv(text_nodes)
+        list(self._copy_nodes_graph(text_index_map, self.new_pack._writer,
+            self.new_pack.text_index, readv_group_iter, total_items))
+        self._log_copied_texts()
+
+    def _create_pack_from_packs(self):
+        self.pb.update("Opening pack", 0, 5)
+        self.new_pack = self.open_pack()
+        new_pack = self.new_pack
+        # buffer data - we won't be reading-back during the pack creation and
+        # this makes a significant difference on sftp pushes.
+        new_pack.set_write_cache_size(1024*1024)
+        if 'pack' in debug.debug_flags:
+            plain_pack_list = ['%s%s' % (a_pack.pack_transport.base, a_pack.name)
+                for a_pack in self.packs]
+            if self.revision_ids is not None:
+                rev_count = len(self.revision_ids)
+            else:
+                rev_count = 'all'
+            trace.mutter('%s: create_pack: creating pack from source packs: '
+                '%s%s %s revisions wanted %s t=0',
+                time.ctime(), self._pack_collection._upload_transport.base, new_pack.random_name,
+                plain_pack_list, rev_count)
+        self._copy_revision_texts()
+        self._copy_inventory_texts()
+        self._copy_text_texts()
+        # select signature keys
+        signature_filter = self._revision_keys # same keyspace
+        signature_index_map, signature_indices = self._pack_map_and_index_list(
+            'signature_index')
+        signature_nodes = self._index_contents(signature_indices,
+            signature_filter)
+        # copy signature keys and adjust values
+        self.pb.update("Copying signature texts", 4)
+        self._copy_nodes(signature_nodes, signature_index_map, new_pack._writer,
+            new_pack.signature_index)
+        if 'pack' in debug.debug_flags:
+            trace.mutter('%s: create_pack: revision signatures copied: %s%s %d items t+%6.3fs',
+                time.ctime(), self._pack_collection._upload_transport.base, new_pack.random_name,
+                new_pack.signature_index.key_count(),
+                time.time() - new_pack.start_time)
+        new_pack._check_references()
+        if not self._use_pack(new_pack):
+            new_pack.abort()
+            return None
+        self.pb.update("Finishing pack", 5)
+        new_pack.finish()
+        self._pack_collection.allocate(new_pack)
+        return new_pack
+
+    def _least_readv_node_readv(self, nodes):
+        """Generate request groups for nodes using the least readv's.
+
+        :param nodes: An iterable of graph index nodes.
+        :return: Total node count and an iterator of the data needed to perform
+            readvs to obtain the data for nodes. Each item yielded by the
+            iterator is a tuple with:
+            index, readv_vector, node_vector. readv_vector is a list ready to
+            hand to the transport readv method, and node_vector is a list of
+            (key, eol_flag, references) for the node retrieved by the
+            matching readv_vector.
+        """
+        # group by pack so we do one readv per pack
+        nodes = sorted(nodes)
+        total = len(nodes)
+        request_groups = {}
+        for index, key, value, references in nodes:
+            if index not in request_groups:
+                request_groups[index] = []
+            request_groups[index].append((key, value, references))
+        result = []
+        for index, items in request_groups.iteritems():
+            pack_readv_requests = []
+            for key, value, references in items:
+                # ---- KnitGraphIndex.get_position
+                bits = value[1:].split(' ')
+                offset, length = int(bits[0]), int(bits[1])
+                pack_readv_requests.append(
+                    ((offset, length), (key, value[0], references)))
+            # linear scan up the pack to maximum range combining.
+            pack_readv_requests.sort()
+            # split out the readv and the node data.
+            pack_readv = [readv for readv, node in pack_readv_requests]
+            node_vector = [node for readv, node in pack_readv_requests]
+            result.append((index, pack_readv, node_vector))
+        return total, result
+
+    def _revision_node_readv(self, revision_nodes):
+        """Return the total revisions and the readv's to issue.
+
+        :param revision_nodes: The revision index contents for the packs being
+            incorporated into the new pack.
+        :return: As per _least_readv_node_readv.
+        """
+        return self._least_readv_node_readv(revision_nodes)
+
+
+class KnitReconcilePacker(KnitPacker):
+    """A packer which regenerates indices etc as it copies.
+
+    This is used by ``bzr reconcile`` to cause parent text pointers to be
+    regenerated.
+    """
+
+    def __init__(self, *args, **kwargs):
+        super(KnitReconcilePacker, self).__init__(*args, **kwargs)
+        self._data_changed = False
+
+    def _process_inventory_lines(self, inv_lines):
+        """Generate a text key reference map rather for reconciling with."""
+        repo = self._pack_collection.repo
+        refs = repo._serializer._find_text_key_references(inv_lines)
+        self._text_refs = refs
+        # during reconcile we:
+        #  - convert unreferenced texts to full texts
+        #  - correct texts which reference a text not copied to be full texts
+        #  - copy all others as-is but with corrected parents.
+        #  - so at this point we don't know enough to decide what becomes a full
+        #    text.
+        self._text_filter = None
+
+    def _copy_text_texts(self):
+        """generate what texts we should have and then copy."""
+        self.pb.update("Copying content texts", 3)
+        # we have three major tasks here:
+        # 1) generate the ideal index
+        repo = self._pack_collection.repo
+        ancestors = dict([(key[0], tuple(ref[0] for ref in refs[0])) for
+            _1, key, _2, refs in
+            self.new_pack.revision_index.iter_all_entries()])
+        ideal_index = repo._generate_text_key_index(self._text_refs, ancestors)
+        # 2) generate a text_nodes list that contains all the deltas that can
+        #    be used as-is, with corrected parents.
+        ok_nodes = []
+        bad_texts = []
+        discarded_nodes = []
+        NULL_REVISION = _mod_revision.NULL_REVISION
+        text_index_map, text_nodes = self._get_text_nodes()
+        for node in text_nodes:
+            # 0 - index
+            # 1 - key
+            # 2 - value
+            # 3 - refs
+            try:
+                ideal_parents = tuple(ideal_index[node[1]])
+            except KeyError:
+                discarded_nodes.append(node)
+                self._data_changed = True
+            else:
+                if ideal_parents == (NULL_REVISION,):
+                    ideal_parents = ()
+                if ideal_parents == node[3][0]:
+                    # no change needed.
+                    ok_nodes.append(node)
+                elif ideal_parents[0:1] == node[3][0][0:1]:
+                    # the left most parent is the same, or there are no parents
+                    # today. Either way, we can preserve the representation as
+                    # long as we change the refs to be inserted.
+                    self._data_changed = True
+                    ok_nodes.append((node[0], node[1], node[2],
+                        (ideal_parents, node[3][1])))
+                    self._data_changed = True
+                else:
+                    # Reinsert this text completely
+                    bad_texts.append((node[1], ideal_parents))
+                    self._data_changed = True
+        # we're finished with some data.
+        del ideal_index
+        del text_nodes
+        # 3) bulk copy the ok data
+        total_items, readv_group_iter = self._least_readv_node_readv(ok_nodes)
+        list(self._copy_nodes_graph(text_index_map, self.new_pack._writer,
+            self.new_pack.text_index, readv_group_iter, total_items))
+        # 4) adhoc copy all the other texts.
+        # We have to topologically insert all texts otherwise we can fail to
+        # reconcile when parts of a single delta chain are preserved intact,
+        # and other parts are not. E.g. Discarded->d1->d2->d3. d1 will be
+        # reinserted, and if d3 has incorrect parents it will also be
+        # reinserted. If we insert d3 first, d2 is present (as it was bulk
+        # copied), so we will try to delta, but d2 is not currently able to be
+        # extracted because its basis d1 is not present. Topologically sorting
+        # addresses this. The following generates a sort for all the texts that
+        # are being inserted without having to reference the entire text key
+        # space (we only topo sort the revisions, which is smaller).
+        topo_order = tsort.topo_sort(ancestors)
+        rev_order = dict(zip(topo_order, range(len(topo_order))))
+        bad_texts.sort(key=lambda key:rev_order.get(key[0][1], 0))
+        transaction = repo.get_transaction()
+        file_id_index = GraphIndexPrefixAdapter(
+            self.new_pack.text_index,
+            ('blank', ), 1,
+            add_nodes_callback=self.new_pack.text_index.add_nodes)
+        data_access = _DirectPackAccess(
+                {self.new_pack.text_index:self.new_pack.access_tuple()})
+        data_access.set_writer(self.new_pack._writer, self.new_pack.text_index,
+            self.new_pack.access_tuple())
+        output_texts = KnitVersionedFiles(
+            _KnitGraphIndex(self.new_pack.text_index,
+                add_callback=self.new_pack.text_index.add_nodes,
+                deltas=True, parents=True, is_locked=repo.is_locked),
+            data_access=data_access, max_delta_chain=200)
+        for key, parent_keys in bad_texts:
+            # We refer to the new pack to delta data being output.
+            # A possible improvement would be to catch errors on short reads
+            # and only flush then.
+            self.new_pack.flush()
+            parents = []
+            for parent_key in parent_keys:
+                if parent_key[0] != key[0]:
+                    # Graph parents must match the fileid
+                    raise errors.BzrError('Mismatched key parent %r:%r' %
+                        (key, parent_keys))
+                parents.append(parent_key[1])
+            text_lines = osutils.split_lines(repo.texts.get_record_stream(
+                [key], 'unordered', True).next().get_bytes_as('fulltext'))
+            output_texts.add_lines(key, parent_keys, text_lines,
+                random_id=True, check_content=False)
+        # 5) check that nothing inserted has a reference outside the keyspace.
+        missing_text_keys = self.new_pack.text_index._external_references()
+        if missing_text_keys:
+            raise errors.BzrCheckError('Reference to missing compression parents %r'
+                % (missing_text_keys,))
+        self._log_copied_texts()
+
+    def _use_pack(self, new_pack):
+        """Override _use_pack to check for reconcile having changed content."""
+        # XXX: we might be better checking this at the copy time.
+        original_inventory_keys = set()
+        inv_index = self._pack_collection.inventory_index.combined_index
+        for entry in inv_index.iter_all_entries():
+            original_inventory_keys.add(entry[1])
+        new_inventory_keys = set()
+        for entry in new_pack.inventory_index.iter_all_entries():
+            new_inventory_keys.add(entry[1])
+        if new_inventory_keys != original_inventory_keys:
+            self._data_changed = True
+        return new_pack.data_inserted() and self._data_changed
+
+
+class OptimisingKnitPacker(KnitPacker):
+    """A packer which spends more time to create better disk layouts."""
+
+    def _revision_node_readv(self, revision_nodes):
+        """Return the total revisions and the readv's to issue.
+
+        This sort places revisions in topological order with the ancestors
+        after the children.
+
+        :param revision_nodes: The revision index contents for the packs being
+            incorporated into the new pack.
+        :return: As per _least_readv_node_readv.
+        """
+        # build an ancestors dict
+        ancestors = {}
+        by_key = {}
+        for index, key, value, references in revision_nodes:
+            ancestors[key] = references[0]
+            by_key[key] = (index, value, references)
+        order = tsort.topo_sort(ancestors)
+        total = len(order)
+        # Single IO is pathological, but it will work as a starting point.
+        requests = []
+        for key in reversed(order):
+            index, value, references = by_key[key]
+            # ---- KnitGraphIndex.get_position
+            bits = value[1:].split(' ')
+            offset, length = int(bits[0]), int(bits[1])
+            requests.append(
+                (index, [(offset, length)], [(key, value[0], references)]))
+        # TODO: combine requests in the same index that are in ascending order.
+        return total, requests
+
+    def open_pack(self):
+        """Open a pack for the pack we are creating."""
+        new_pack = super(OptimisingKnitPacker, self).open_pack()
+        # Turn on the optimization flags for all the index builders.
+        new_pack.revision_index.set_optimize(for_size=True)
+        new_pack.inventory_index.set_optimize(for_size=True)
+        new_pack.text_index.set_optimize(for_size=True)
+        new_pack.signature_index.set_optimize(for_size=True)
+        return new_pack
+
+
+class KnitRepositoryPackCollection(RepositoryPackCollection):
+    """A knit pack collection."""
+
+    pack_factory = NewPack
+    resumed_pack_factory = ResumedPack
+    normal_packer_class = KnitPacker
+    optimising_packer_class = OptimisingKnitPacker
+
+
+

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2010-05-04 02:41:17 +0000
+++ bzrlib/repofmt/knitrepo.py	2011-06-16 14:54:47 +0000
@@ -16,6 +16,8 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import itertools
+
 from bzrlib import (
     bzrdir,
     errors,
@@ -32,18 +34,20 @@
     xml7,
     )
 """)
-from bzrlib import (
-    symbol_versioning,
-    )
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
-    CommitBuilder,
+    InterRepository,
     IsInWriteGroupError,
-    MetaDirRepository,
-    MetaDirRepositoryFormat,
     RepositoryFormat,
-    RootCommitBuilder,
-    )
+    )
+from bzrlib.vf_repository import (
+    InterSameDataRepository,
+    MetaDirVersionedFileRepository,
+    MetaDirVersionedFileRepositoryFormat,
+    VersionedFileCommitBuilder,
+    VersionedFileRootCommitBuilder,
+    )
+from bzrlib import symbol_versioning
 
 
 class _KnitParentsProvider(object):
@@ -103,7 +107,7 @@
         return result
 
 
-class KnitRepository(MetaDirRepository):
+class KnitRepository(MetaDirVersionedFileRepository):
     """Knit format repository."""
 
     # These attributes are inherited from the Repository base class. Setting
@@ -115,7 +119,7 @@
 
     def __init__(self, _format, a_bzrdir, control_files, _commit_builder_class,
         _serializer):
-        MetaDirRepository.__init__(self, _format, a_bzrdir, control_files)
+        super(KnitRepository, self).__init__(_format, a_bzrdir, control_files)
         self._commit_builder_class = _commit_builder_class
         self._serializer = _serializer
         self._reconcile_fixes_text_parents = True
@@ -177,31 +181,6 @@
         result.get_parent_map([('A',)])
         return result
 
-    def fileid_involved_between_revs(self, from_revid, to_revid):
-        """Find file_id(s) which are involved in the changes between revisions.
-
-        This determines the set of revisions which are involved, and then
-        finds all file ids affected by those revisions.
-        """
-        vf = self._get_revision_vf()
-        from_set = set(vf.get_ancestry(from_revid))
-        to_set = set(vf.get_ancestry(to_revid))
-        changed = to_set.difference(from_set)
-        return self._fileid_involved_by_set(changed)
-
-    def fileid_involved(self, last_revid=None):
-        """Find all file_ids modified in the ancestry of last_revid.
-
-        :param last_revid: If None, last_revision() will be used.
-        """
-        if not last_revid:
-            changed = set(self.all_revision_ids())
-        else:
-            changed = set(self.get_ancestry(last_revid))
-        if None in changed:
-            changed.remove(None)
-        return self._fileid_involved_by_set(changed)
-
     @needs_read_lock
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
@@ -232,44 +211,8 @@
     def _make_parents_provider(self):
         return _KnitsParentsProvider(self.revisions)
 
-    def _find_inconsistent_revision_parents(self, revisions_iterator=None):
-        """Find revisions with different parent lists in the revision object
-        and in the index graph.
-
-        :param revisions_iterator: None, or an iterator of (revid,
-            Revision-or-None). This iterator controls the revisions checked.
-        :returns: an iterator yielding tuples of (revison-id, parents-in-index,
-            parents-in-revision).
-        """
-        if not self.is_locked():
-            raise AssertionError()
-        vf = self.revisions
-        if revisions_iterator is None:
-            revisions_iterator = self._iter_revisions(None)
-        for revid, revision in revisions_iterator:
-            if revision is None:
-                pass
-            parent_map = vf.get_parent_map([(revid,)])
-            parents_according_to_index = tuple(parent[-1] for parent in
-                parent_map[(revid,)])
-            parents_according_to_revision = tuple(revision.parent_ids)
-            if parents_according_to_index != parents_according_to_revision:
-                yield (revid, parents_according_to_index,
-                    parents_according_to_revision)
-
-    def _check_for_inconsistent_revision_parents(self):
-        inconsistencies = list(self._find_inconsistent_revision_parents())
-        if inconsistencies:
-            raise errors.BzrCheckError(
-                "Revision knit has inconsistent parents.")
-
-    def revision_graph_can_have_wrong_parents(self):
-        # The revision.kndx could potentially claim a revision has a different
-        # parent to the revision text.
-        return True
-
-
-class RepositoryFormatKnit(MetaDirRepositoryFormat):
+
+class RepositoryFormatKnit(MetaDirVersionedFileRepositoryFormat):
     """Bzr repository knit format (generalized).
 
     This repository format has:
@@ -304,6 +247,10 @@
     _fetch_order = 'topological'
     _fetch_uses_deltas = True
     fast_deltas = False
+    supports_funky_characters = True
+    # The revision.kndx could potentially claim a revision has a different
+    # parent to the revision text.
+    revision_graph_can_have_wrong_parents = True
 
     def _get_inventories(self, repo_transport, repo, name='inventory'):
         mapper = versionedfile.ConstantMapper(name)
@@ -412,7 +359,7 @@
     """
 
     repository_class = KnitRepository
-    _commit_builder_class = CommitBuilder
+    _commit_builder_class = VersionedFileCommitBuilder
     @property
     def _serializer(self):
         return xml5.serializer_v5
@@ -446,7 +393,7 @@
     """
 
     repository_class = KnitRepository
-    _commit_builder_class = RootCommitBuilder
+    _commit_builder_class = VersionedFileRootCommitBuilder
     rich_root_data = True
     experimental = True
     supports_tree_reference = True
@@ -488,7 +435,7 @@
     """
 
     repository_class = KnitRepository
-    _commit_builder_class = RootCommitBuilder
+    _commit_builder_class = VersionedFileRootCommitBuilder
     rich_root_data = True
     supports_tree_reference = False
     @property
@@ -510,3 +457,73 @@
     def get_format_description(self):
         """See RepositoryFormat.get_format_description()."""
         return "Knit repository format 4"
+
+
+class InterKnitRepo(InterSameDataRepository):
+    """Optimised code paths between Knit based repositories."""
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return RepositoryFormatKnit1()
+
+    @staticmethod
+    def is_compatible(source, target):
+        """Be compatible with known Knit formats.
+
+        We don't test for the stores being of specific types because that
+        could lead to confusing results, and there is no need to be
+        overly general.
+        """
+        try:
+            are_knits = (isinstance(source._format, RepositoryFormatKnit) and
+                isinstance(target._format, RepositoryFormatKnit))
+        except AttributeError:
+            return False
+        return are_knits and InterRepository._same_model(source, target)
+
+    @needs_read_lock
+    def search_missing_revision_ids(self,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None,
+            limit=None):
+        """See InterRepository.search_missing_revision_ids()."""
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.4.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        del revision_id
+        source_ids_set = self._present_source_revisions_for(
+            revision_ids, if_present_ids)
+        # source_ids is the worst possible case we may need to pull.
+        # now we want to filter source_ids against what we actually
+        # have in target, but don't try to check for existence where we know
+        # we do not have a revision as that would be pointless.
+        target_ids = set(self.target.all_revision_ids())
+        possibly_present_revisions = target_ids.intersection(source_ids_set)
+        actually_present_revisions = set(
+            self.target._eliminate_revisions_not_present(possibly_present_revisions))
+        required_revisions = source_ids_set.difference(actually_present_revisions)
+        if revision_ids is not None:
+            # we used get_ancestry to determine source_ids then we are assured all
+            # revisions referenced are present as they are installed in topological order.
+            # and the tip revision was validated by get_ancestry.
+            result_set = required_revisions
+        else:
+            # if we just grabbed the possibly available ids, then
+            # we only have an estimate of whats available and need to validate
+            # that against the revision records.
+            result_set = set(
+                self.source._eliminate_revisions_not_present(required_revisions))
+        if limit is not None:
+            topo_ordered = self.source.get_graph().iter_topo_order(result_set)
+            result_set = set(itertools.islice(topo_ordered, limit))
+        return self.source.revision_ids_to_search_result(result_set)
+
+
+InterRepository.register_optimiser(InterKnitRepo)

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2010-05-11 08:36:16 +0000
+++ bzrlib/repofmt/pack_repo.py	2011-07-25 07:51:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,54 +25,44 @@
 from bzrlib import (
     chk_map,
     cleanup,
+    config,
     debug,
     graph,
     osutils,
     pack,
     transactions,
+    tsort,
     ui,
-    xml5,
-    xml6,
-    xml7,
     )
 from bzrlib.index import (
     CombinedGraphIndex,
     GraphIndexPrefixAdapter,
     )
-from bzrlib.knit import (
-    KnitPlainFactory,
-    KnitVersionedFiles,
-    _KnitGraphIndex,
-    _DirectPackAccess,
-    )
-from bzrlib import tsort
 """)
 from bzrlib import (
-    bzrdir,
+    btree_index,
     errors,
     lockable_files,
     lockdir,
-    revision as _mod_revision,
     )
 
-from bzrlib.decorators import needs_write_lock, only_raises
-from bzrlib.btree_index import (
-    BTreeGraphIndex,
-    BTreeBuilder,
-    )
-from bzrlib.index import (
-    GraphIndex,
-    InMemoryGraphIndex,
+from bzrlib.decorators import (
+    needs_read_lock,
+    needs_write_lock,
+    only_raises,
     )
 from bzrlib.lock import LogicalLockResult
-from bzrlib.repofmt.knitrepo import KnitRepository
 from bzrlib.repository import (
-    CommitBuilder,
-    MetaDirRepositoryFormat,
+    _LazyListJoin,
+    MetaDirRepository,
     RepositoryFormat,
     RepositoryWriteLockResult,
-    RootCommitBuilder,
-    StreamSource,
+    )
+from bzrlib.vf_repository import (
+    MetaDirVersionedFileRepository,
+    MetaDirVersionedFileRepositoryFormat,
+    VersionedFileCommitBuilder,
+    VersionedFileRootCommitBuilder,
     )
 from bzrlib.trace import (
     mutter,
@@ -81,8 +71,8 @@
     )
 
 
-class PackCommitBuilder(CommitBuilder):
-    """A subclass of CommitBuilder to add texts with pack semantics.
+class PackCommitBuilder(VersionedFileCommitBuilder):
+    """Subclass of VersionedFileCommitBuilder to add texts with pack semantics.
 
     Specifically this uses one knit object rather than one knit object per
     added text, reducing memory and object pressure.
@@ -90,10 +80,10 @@
 
     def __init__(self, repository, parents, config, timestamp=None,
                  timezone=None, committer=None, revprops=None,
-                 revision_id=None):
-        CommitBuilder.__init__(self, repository, parents, config,
+                 revision_id=None, lossy=False):
+        VersionedFileCommitBuilder.__init__(self, repository, parents, config,
             timestamp=timestamp, timezone=timezone, committer=committer,
-            revprops=revprops, revision_id=revision_id)
+            revprops=revprops, revision_id=revision_id, lossy=lossy)
         self._file_graph = graph.Graph(
             repository._pack_collection.text_index.combined_index)
 
@@ -102,7 +92,7 @@
         return set([key[1] for key in self._file_graph.heads(keys)])
 
 
-class PackRootCommitBuilder(RootCommitBuilder):
+class PackRootCommitBuilder(VersionedFileRootCommitBuilder):
     """A subclass of RootCommitBuilder to add texts with pack semantics.
 
     Specifically this uses one knit object rather than one knit object per
@@ -111,10 +101,11 @@
 
     def __init__(self, repository, parents, config, timestamp=None,
                  timezone=None, committer=None, revprops=None,
-                 revision_id=None):
-        CommitBuilder.__init__(self, repository, parents, config,
-            timestamp=timestamp, timezone=timezone, committer=committer,
-            revprops=revprops, revision_id=revision_id)
+                 revision_id=None, lossy=False):
+        super(PackRootCommitBuilder, self).__init__(repository, parents,
+            config, timestamp=timestamp, timezone=timezone,
+            committer=committer, revprops=revprops, revision_id=revision_id,
+            lossy=lossy)
         self._file_graph = graph.Graph(
             repository._pack_collection.text_index.combined_index)
 
@@ -231,11 +222,13 @@
         unlimited_cache = False
         if index_type == 'chk':
             unlimited_cache = True
-        setattr(self, index_type + '_index',
-            self.index_class(self.index_transport,
-                self.index_name(index_type, self.name),
-                self.index_sizes[self.index_offset(index_type)],
-                unlimited_cache=unlimited_cache))
+        index = self.index_class(self.index_transport,
+                    self.index_name(index_type, self.name),
+                    self.index_sizes[self.index_offset(index_type)],
+                    unlimited_cache=unlimited_cache)
+        if index_type == 'chk':
+            index._leaf_factory = btree_index._gcchk_factory
+        setattr(self, index_type + '_index', index)
 
 
 class ExistingPack(Pack):
@@ -322,10 +315,10 @@
         for index_type in index_types:
             old_name = self.index_name(index_type, self.name)
             new_name = '../indices/' + old_name
-            self.upload_transport.rename(old_name, new_name)
+            self.upload_transport.move(old_name, new_name)
             self._replace_index_with_readonly(index_type)
         new_name = '../packs/' + self.file_name()
-        self.upload_transport.rename(self.file_name(), new_name)
+        self.upload_transport.move(self.file_name(), new_name)
         self._state = 'finished'
 
     def _get_external_refs(self, index):
@@ -486,7 +479,8 @@
         # visible is smaller.  On the other hand none will be seen until
         # they're in the names list.
         self.index_sizes = [None, None, None, None]
-        self._write_index('revision', self.revision_index, 'revision', suspend)
+        self._write_index('revision', self.revision_index, 'revision',
+            suspend)
         self._write_index('inventory', self.inventory_index, 'inventory',
             suspend)
         self._write_index('text', self.text_index, 'file texts', suspend)
@@ -496,7 +490,8 @@
             self.index_sizes.append(None)
             self._write_index('chk', self.chk_index,
                 'content hash bytes', suspend)
-        self.write_stream.close()
+        self.write_stream.close(
+            want_fdatasync=self._pack_collection.config_stack.get('repository.fdatasync'))
         # Note that this will clobber an existing pack with the same name,
         # without checking for hash collisions. While this is undesirable this
         # is something that can be rectified in a subsequent release. One way
@@ -511,7 +506,7 @@
         new_name = self.name + '.pack'
         if not suspend:
             new_name = '../packs/' + new_name
-        self.upload_transport.rename(self.random_name, new_name)
+        self.upload_transport.move(self.random_name, new_name)
         self._state = 'finished'
         if 'pack' in debug.debug_flags:
             # XXX: size might be interesting?
@@ -545,8 +540,14 @@
             transport = self.upload_transport
         else:
             transport = self.index_transport
-        self.index_sizes[self.index_offset(index_type)] = transport.put_file(
-            index_name, index.finish(), mode=self._file_mode)
+        index_tempfile = index.finish()
+        index_bytes = index_tempfile.read()
+        write_stream = transport.open_write_stream(index_name,
+            mode=self._file_mode)
+        write_stream.write(index_bytes)
+        write_stream.close(
+            want_fdatasync=self._pack_collection.config_stack.get('repository.fdatasync'))
+        self.index_sizes[self.index_offset(index_type)] = len(index_bytes)
         if 'pack' in debug.debug_flags:
             # XXX: size might be interesting?
             mutter('%s: create_pack: wrote %s index: %s%s t+%6.3fs',
@@ -674,39 +675,6 @@
         # What text keys to copy. None for 'all texts'. This is set by
         # _copy_inventory_texts
         self._text_filter = None
-        self._extra_init()
-
-    def _extra_init(self):
-        """A template hook to allow extending the constructor trivially."""
-
-    def _pack_map_and_index_list(self, index_attribute):
-        """Convert a list of packs to an index pack map and index list.
-
-        :param index_attribute: The attribute that the desired index is found
-            on.
-        :return: A tuple (map, list) where map contains the dict from
-            index:pack_tuple, and list contains the indices in the preferred
-            access order.
-        """
-        indices = []
-        pack_map = {}
-        for pack_obj in self.packs:
-            index = getattr(pack_obj, index_attribute)
-            indices.append(index)
-            pack_map[index] = pack_obj
-        return pack_map, indices
-
-    def _index_contents(self, indices, key_filter=None):
-        """Get an iterable of the index contents from a pack_map.
-
-        :param indices: The list of indices to query
-        :param key_filter: An optional filter to limit the keys returned.
-        """
-        all_index = CombinedGraphIndex(indices)
-        if key_filter is None:
-            return all_index.iter_all_entries()
-        else:
-            return all_index.iter_entries(key_filter)
 
     def pack(self, pb=None):
         """Create a new pack by reading data from other packs.
@@ -723,7 +691,7 @@
         :return: A Pack object, or None if nothing was copied.
         """
         # open a pack - using the same name as the last temporary file
-        # - which has already been flushed, so its safe.
+        # - which has already been flushed, so it's safe.
         # XXX: - duplicate code warning with start_write_group; fix before
         #      considering 'done'.
         if self._pack_collection._new_pack is not None:
@@ -761,65 +729,9 @@
         new_pack.signature_index.set_optimize(combine_backing_indices=False)
         return new_pack
 
-    def _update_pack_order(self, entries, index_to_pack_map):
-        """Determine how we want our packs to be ordered.
-
-        This changes the sort order of the self.packs list so that packs unused
-        by 'entries' will be at the end of the list, so that future requests
-        can avoid probing them.  Used packs will be at the front of the
-        self.packs list, in the order of their first use in 'entries'.
-
-        :param entries: A list of (index, ...) tuples
-        :param index_to_pack_map: A mapping from index objects to pack objects.
-        """
-        packs = []
-        seen_indexes = set()
-        for entry in entries:
-            index = entry[0]
-            if index not in seen_indexes:
-                packs.append(index_to_pack_map[index])
-                seen_indexes.add(index)
-        if len(packs) == len(self.packs):
-            if 'pack' in debug.debug_flags:
-                mutter('Not changing pack list, all packs used.')
-            return
-        seen_packs = set(packs)
-        for pack in self.packs:
-            if pack not in seen_packs:
-                packs.append(pack)
-                seen_packs.add(pack)
-        if 'pack' in debug.debug_flags:
-            old_names = [p.access_tuple()[1] for p in self.packs]
-            new_names = [p.access_tuple()[1] for p in packs]
-            mutter('Reordering packs\nfrom: %s\n  to: %s',
-                   old_names, new_names)
-        self.packs = packs
-
     def _copy_revision_texts(self):
         """Copy revision data to the new pack."""
-        # select revisions
-        if self.revision_ids:
-            revision_keys = [(revision_id,) for revision_id in self.revision_ids]
-        else:
-            revision_keys = None
-        # select revision keys
-        revision_index_map, revision_indices = self._pack_map_and_index_list(
-            'revision_index')
-        revision_nodes = self._index_contents(revision_indices, revision_keys)
-        revision_nodes = list(revision_nodes)
-        self._update_pack_order(revision_nodes, revision_index_map)
-        # copy revision keys and adjust values
-        self.pb.update("Copying revision texts", 1)
-        total_items, readv_group_iter = self._revision_node_readv(revision_nodes)
-        list(self._copy_nodes_graph(revision_index_map, self.new_pack._writer,
-            self.new_pack.revision_index, readv_group_iter, total_items))
-        if 'pack' in debug.debug_flags:
-            mutter('%s: create_pack: revisions copied: %s%s %d items t+%6.3fs',
-                time.ctime(), self._pack_collection._upload_transport.base,
-                self.new_pack.random_name,
-                self.new_pack.revision_index.key_count(),
-                time.time() - self.new_pack.start_time)
-        self._revision_keys = revision_keys
+        raise NotImplementedError(self._copy_revision_texts)
 
     def _copy_inventory_texts(self):
         """Copy the inventory texts to the new pack.
@@ -828,307 +740,13 @@
 
         Sets self._text_filter appropriately.
         """
-        # select inventory keys
-        inv_keys = self._revision_keys # currently the same keyspace, and note that
-        # querying for keys here could introduce a bug where an inventory item
-        # is missed, so do not change it to query separately without cross
-        # checking like the text key check below.
-        inventory_index_map, inventory_indices = self._pack_map_and_index_list(
-            'inventory_index')
-        inv_nodes = self._index_contents(inventory_indices, inv_keys)
-        # copy inventory keys and adjust values
-        # XXX: Should be a helper function to allow different inv representation
-        # at this point.
-        self.pb.update("Copying inventory texts", 2)
-        total_items, readv_group_iter = self._least_readv_node_readv(inv_nodes)
-        # Only grab the output lines if we will be processing them
-        output_lines = bool(self.revision_ids)
-        inv_lines = self._copy_nodes_graph(inventory_index_map,
-            self.new_pack._writer, self.new_pack.inventory_index,
-            readv_group_iter, total_items, output_lines=output_lines)
-        if self.revision_ids:
-            self._process_inventory_lines(inv_lines)
-        else:
-            # eat the iterator to cause it to execute.
-            list(inv_lines)
-            self._text_filter = None
-        if 'pack' in debug.debug_flags:
-            mutter('%s: create_pack: inventories copied: %s%s %d items t+%6.3fs',
-                time.ctime(), self._pack_collection._upload_transport.base,
-                self.new_pack.random_name,
-                self.new_pack.inventory_index.key_count(),
-                time.time() - self.new_pack.start_time)
+        raise NotImplementedError(self._copy_inventory_texts)
 
     def _copy_text_texts(self):
-        # select text keys
-        text_index_map, text_nodes = self._get_text_nodes()
-        if self._text_filter is not None:
-            # We could return the keys copied as part of the return value from
-            # _copy_nodes_graph but this doesn't work all that well with the
-            # need to get line output too, so we check separately, and as we're
-            # going to buffer everything anyway, we check beforehand, which
-            # saves reading knit data over the wire when we know there are
-            # mising records.
-            text_nodes = set(text_nodes)
-            present_text_keys = set(_node[1] for _node in text_nodes)
-            missing_text_keys = set(self._text_filter) - present_text_keys
-            if missing_text_keys:
-                # TODO: raise a specific error that can handle many missing
-                # keys.
-                mutter("missing keys during fetch: %r", missing_text_keys)
-                a_missing_key = missing_text_keys.pop()
-                raise errors.RevisionNotPresent(a_missing_key[1],
-                    a_missing_key[0])
-        # copy text keys and adjust values
-        self.pb.update("Copying content texts", 3)
-        total_items, readv_group_iter = self._least_readv_node_readv(text_nodes)
-        list(self._copy_nodes_graph(text_index_map, self.new_pack._writer,
-            self.new_pack.text_index, readv_group_iter, total_items))
-        self._log_copied_texts()
+        raise NotImplementedError(self._copy_text_texts)
 
     def _create_pack_from_packs(self):
-        self.pb.update("Opening pack", 0, 5)
-        self.new_pack = self.open_pack()
-        new_pack = self.new_pack
-        # buffer data - we won't be reading-back during the pack creation and
-        # this makes a significant difference on sftp pushes.
-        new_pack.set_write_cache_size(1024*1024)
-        if 'pack' in debug.debug_flags:
-            plain_pack_list = ['%s%s' % (a_pack.pack_transport.base, a_pack.name)
-                for a_pack in self.packs]
-            if self.revision_ids is not None:
-                rev_count = len(self.revision_ids)
-            else:
-                rev_count = 'all'
-            mutter('%s: create_pack: creating pack from source packs: '
-                '%s%s %s revisions wanted %s t=0',
-                time.ctime(), self._pack_collection._upload_transport.base, new_pack.random_name,
-                plain_pack_list, rev_count)
-        self._copy_revision_texts()
-        self._copy_inventory_texts()
-        self._copy_text_texts()
-        # select signature keys
-        signature_filter = self._revision_keys # same keyspace
-        signature_index_map, signature_indices = self._pack_map_and_index_list(
-            'signature_index')
-        signature_nodes = self._index_contents(signature_indices,
-            signature_filter)
-        # copy signature keys and adjust values
-        self.pb.update("Copying signature texts", 4)
-        self._copy_nodes(signature_nodes, signature_index_map, new_pack._writer,
-            new_pack.signature_index)
-        if 'pack' in debug.debug_flags:
-            mutter('%s: create_pack: revision signatures copied: %s%s %d items t+%6.3fs',
-                time.ctime(), self._pack_collection._upload_transport.base, new_pack.random_name,
-                new_pack.signature_index.key_count(),
-                time.time() - new_pack.start_time)
-        # copy chk contents
-        # NB XXX: how to check CHK references are present? perhaps by yielding
-        # the items? How should that interact with stacked repos?
-        if new_pack.chk_index is not None:
-            self._copy_chks()
-            if 'pack' in debug.debug_flags:
-                mutter('%s: create_pack: chk content copied: %s%s %d items t+%6.3fs',
-                    time.ctime(), self._pack_collection._upload_transport.base,
-                    new_pack.random_name,
-                    new_pack.chk_index.key_count(),
-                    time.time() - new_pack.start_time)
-        new_pack._check_references()
-        if not self._use_pack(new_pack):
-            new_pack.abort()
-            return None
-        self.pb.update("Finishing pack", 5)
-        new_pack.finish()
-        self._pack_collection.allocate(new_pack)
-        return new_pack
-
-    def _copy_chks(self, refs=None):
-        # XXX: Todo, recursive follow-pointers facility when fetching some
-        # revisions only.
-        chk_index_map, chk_indices = self._pack_map_and_index_list(
-            'chk_index')
-        chk_nodes = self._index_contents(chk_indices, refs)
-        new_refs = set()
-        # TODO: This isn't strictly tasteful as we are accessing some private
-        #       variables (_serializer). Perhaps a better way would be to have
-        #       Repository._deserialise_chk_node()
-        search_key_func = chk_map.search_key_registry.get(
-            self._pack_collection.repo._serializer.search_key_name)
-        def accumlate_refs(lines):
-            # XXX: move to a generic location
-            # Yay mismatch:
-            bytes = ''.join(lines)
-            node = chk_map._deserialise(bytes, ("unknown",), search_key_func)
-            new_refs.update(node.refs())
-        self._copy_nodes(chk_nodes, chk_index_map, self.new_pack._writer,
-            self.new_pack.chk_index, output_lines=accumlate_refs)
-        return new_refs
-
-    def _copy_nodes(self, nodes, index_map, writer, write_index,
-        output_lines=None):
-        """Copy knit nodes between packs with no graph references.
-
-        :param output_lines: Output full texts of copied items.
-        """
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            return self._do_copy_nodes(nodes, index_map, writer,
-                write_index, pb, output_lines=output_lines)
-        finally:
-            pb.finished()
-
-    def _do_copy_nodes(self, nodes, index_map, writer, write_index, pb,
-        output_lines=None):
-        # for record verification
-        knit = KnitVersionedFiles(None, None)
-        # plan a readv on each source pack:
-        # group by pack
-        nodes = sorted(nodes)
-        # how to map this into knit.py - or knit.py into this?
-        # we don't want the typical knit logic, we want grouping by pack
-        # at this point - perhaps a helper library for the following code
-        # duplication points?
-        request_groups = {}
-        for index, key, value in nodes:
-            if index not in request_groups:
-                request_groups[index] = []
-            request_groups[index].append((key, value))
-        record_index = 0
-        pb.update("Copied record", record_index, len(nodes))
-        for index, items in request_groups.iteritems():
-            pack_readv_requests = []
-            for key, value in items:
-                # ---- KnitGraphIndex.get_position
-                bits = value[1:].split(' ')
-                offset, length = int(bits[0]), int(bits[1])
-                pack_readv_requests.append((offset, length, (key, value[0])))
-            # linear scan up the pack
-            pack_readv_requests.sort()
-            # copy the data
-            pack_obj = index_map[index]
-            transport, path = pack_obj.access_tuple()
-            try:
-                reader = pack.make_readv_reader(transport, path,
-                    [offset[0:2] for offset in pack_readv_requests])
-            except errors.NoSuchFile:
-                if self._reload_func is not None:
-                    self._reload_func()
-                raise
-            for (names, read_func), (_1, _2, (key, eol_flag)) in \
-                izip(reader.iter_records(), pack_readv_requests):
-                raw_data = read_func(None)
-                # check the header only
-                if output_lines is not None:
-                    output_lines(knit._parse_record(key[-1], raw_data)[0])
-                else:
-                    df, _ = knit._parse_record_header(key, raw_data)
-                    df.close()
-                pos, size = writer.add_bytes_record(raw_data, names)
-                write_index.add_node(key, eol_flag + "%d %d" % (pos, size))
-                pb.update("Copied record", record_index)
-                record_index += 1
-
-    def _copy_nodes_graph(self, index_map, writer, write_index,
-        readv_group_iter, total_items, output_lines=False):
-        """Copy knit nodes between packs.
-
-        :param output_lines: Return lines present in the copied data as
-            an iterator of line,version_id.
-        """
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            for result in self._do_copy_nodes_graph(index_map, writer,
-                write_index, output_lines, pb, readv_group_iter, total_items):
-                yield result
-        except Exception:
-            # Python 2.4 does not permit try:finally: in a generator.
-            pb.finished()
-            raise
-        else:
-            pb.finished()
-
-    def _do_copy_nodes_graph(self, index_map, writer, write_index,
-        output_lines, pb, readv_group_iter, total_items):
-        # for record verification
-        knit = KnitVersionedFiles(None, None)
-        # for line extraction when requested (inventories only)
-        if output_lines:
-            factory = KnitPlainFactory()
-        record_index = 0
-        pb.update("Copied record", record_index, total_items)
-        for index, readv_vector, node_vector in readv_group_iter:
-            # copy the data
-            pack_obj = index_map[index]
-            transport, path = pack_obj.access_tuple()
-            try:
-                reader = pack.make_readv_reader(transport, path, readv_vector)
-            except errors.NoSuchFile:
-                if self._reload_func is not None:
-                    self._reload_func()
-                raise
-            for (names, read_func), (key, eol_flag, references) in \
-                izip(reader.iter_records(), node_vector):
-                raw_data = read_func(None)
-                if output_lines:
-                    # read the entire thing
-                    content, _ = knit._parse_record(key[-1], raw_data)
-                    if len(references[-1]) == 0:
-                        line_iterator = factory.get_fulltext_content(content)
-                    else:
-                        line_iterator = factory.get_linedelta_content(content)
-                    for line in line_iterator:
-                        yield line, key
-                else:
-                    # check the header only
-                    df, _ = knit._parse_record_header(key, raw_data)
-                    df.close()
-                pos, size = writer.add_bytes_record(raw_data, names)
-                write_index.add_node(key, eol_flag + "%d %d" % (pos, size), references)
-                pb.update("Copied record", record_index)
-                record_index += 1
-
-    def _get_text_nodes(self):
-        text_index_map, text_indices = self._pack_map_and_index_list(
-            'text_index')
-        return text_index_map, self._index_contents(text_indices,
-            self._text_filter)
-
-    def _least_readv_node_readv(self, nodes):
-        """Generate request groups for nodes using the least readv's.
-
-        :param nodes: An iterable of graph index nodes.
-        :return: Total node count and an iterator of the data needed to perform
-            readvs to obtain the data for nodes. Each item yielded by the
-            iterator is a tuple with:
-            index, readv_vector, node_vector. readv_vector is a list ready to
-            hand to the transport readv method, and node_vector is a list of
-            (key, eol_flag, references) for the node retrieved by the
-            matching readv_vector.
-        """
-        # group by pack so we do one readv per pack
-        nodes = sorted(nodes)
-        total = len(nodes)
-        request_groups = {}
-        for index, key, value, references in nodes:
-            if index not in request_groups:
-                request_groups[index] = []
-            request_groups[index].append((key, value, references))
-        result = []
-        for index, items in request_groups.iteritems():
-            pack_readv_requests = []
-            for key, value, references in items:
-                # ---- KnitGraphIndex.get_position
-                bits = value[1:].split(' ')
-                offset, length = int(bits[0]), int(bits[1])
-                pack_readv_requests.append(
-                    ((offset, length), (key, value[0], references)))
-            # linear scan up the pack to maximum range combining.
-            pack_readv_requests.sort()
-            # split out the readv and the node data.
-            pack_readv = [readv for readv, node in pack_readv_requests]
-            node_vector = [node for readv, node in pack_readv_requests]
-            result.append((index, pack_readv, node_vector))
-        return total, result
+        raise NotImplementedError(self._create_pack_from_packs)
 
     def _log_copied_texts(self):
         if 'pack' in debug.debug_flags:
@@ -1138,25 +756,6 @@
                 self.new_pack.text_index.key_count(),
                 time.time() - self.new_pack.start_time)
 
-    def _process_inventory_lines(self, inv_lines):
-        """Use up the inv_lines generator and setup a text key filter."""
-        repo = self._pack_collection.repo
-        fileid_revisions = repo._find_file_ids_from_xml_inventory_lines(
-            inv_lines, self.revision_keys)
-        text_filter = []
-        for fileid, file_revids in fileid_revisions.iteritems():
-            text_filter.extend([(fileid, file_revid) for file_revid in file_revids])
-        self._text_filter = text_filter
-
-    def _revision_node_readv(self, revision_nodes):
-        """Return the total revisions and the readv's to issue.
-
-        :param revision_nodes: The revision index contents for the packs being
-            incorporated into the new pack.
-        :return: As per _least_readv_node_readv.
-        """
-        return self._least_readv_node_readv(revision_nodes)
-
     def _use_pack(self, new_pack):
         """Return True if new_pack should be used.
 
@@ -1166,200 +765,16 @@
         return new_pack.data_inserted()
 
 
-class OptimisingPacker(Packer):
-    """A packer which spends more time to create better disk layouts."""
-
-    def _revision_node_readv(self, revision_nodes):
-        """Return the total revisions and the readv's to issue.
-
-        This sort places revisions in topological order with the ancestors
-        after the children.
-
-        :param revision_nodes: The revision index contents for the packs being
-            incorporated into the new pack.
-        :return: As per _least_readv_node_readv.
-        """
-        # build an ancestors dict
-        ancestors = {}
-        by_key = {}
-        for index, key, value, references in revision_nodes:
-            ancestors[key] = references[0]
-            by_key[key] = (index, value, references)
-        order = tsort.topo_sort(ancestors)
-        total = len(order)
-        # Single IO is pathological, but it will work as a starting point.
-        requests = []
-        for key in reversed(order):
-            index, value, references = by_key[key]
-            # ---- KnitGraphIndex.get_position
-            bits = value[1:].split(' ')
-            offset, length = int(bits[0]), int(bits[1])
-            requests.append(
-                (index, [(offset, length)], [(key, value[0], references)]))
-        # TODO: combine requests in the same index that are in ascending order.
-        return total, requests
-
-    def open_pack(self):
-        """Open a pack for the pack we are creating."""
-        new_pack = super(OptimisingPacker, self).open_pack()
-        # Turn on the optimization flags for all the index builders.
-        new_pack.revision_index.set_optimize(for_size=True)
-        new_pack.inventory_index.set_optimize(for_size=True)
-        new_pack.text_index.set_optimize(for_size=True)
-        new_pack.signature_index.set_optimize(for_size=True)
-        return new_pack
-
-
-class ReconcilePacker(Packer):
-    """A packer which regenerates indices etc as it copies.
-
-    This is used by ``bzr reconcile`` to cause parent text pointers to be
-    regenerated.
-    """
-
-    def _extra_init(self):
-        self._data_changed = False
-
-    def _process_inventory_lines(self, inv_lines):
-        """Generate a text key reference map rather for reconciling with."""
-        repo = self._pack_collection.repo
-        refs = repo._find_text_key_references_from_xml_inventory_lines(
-            inv_lines)
-        self._text_refs = refs
-        # during reconcile we:
-        #  - convert unreferenced texts to full texts
-        #  - correct texts which reference a text not copied to be full texts
-        #  - copy all others as-is but with corrected parents.
-        #  - so at this point we don't know enough to decide what becomes a full
-        #    text.
-        self._text_filter = None
-
-    def _copy_text_texts(self):
-        """generate what texts we should have and then copy."""
-        self.pb.update("Copying content texts", 3)
-        # we have three major tasks here:
-        # 1) generate the ideal index
-        repo = self._pack_collection.repo
-        ancestors = dict([(key[0], tuple(ref[0] for ref in refs[0])) for
-            _1, key, _2, refs in
-            self.new_pack.revision_index.iter_all_entries()])
-        ideal_index = repo._generate_text_key_index(self._text_refs, ancestors)
-        # 2) generate a text_nodes list that contains all the deltas that can
-        #    be used as-is, with corrected parents.
-        ok_nodes = []
-        bad_texts = []
-        discarded_nodes = []
-        NULL_REVISION = _mod_revision.NULL_REVISION
-        text_index_map, text_nodes = self._get_text_nodes()
-        for node in text_nodes:
-            # 0 - index
-            # 1 - key
-            # 2 - value
-            # 3 - refs
-            try:
-                ideal_parents = tuple(ideal_index[node[1]])
-            except KeyError:
-                discarded_nodes.append(node)
-                self._data_changed = True
-            else:
-                if ideal_parents == (NULL_REVISION,):
-                    ideal_parents = ()
-                if ideal_parents == node[3][0]:
-                    # no change needed.
-                    ok_nodes.append(node)
-                elif ideal_parents[0:1] == node[3][0][0:1]:
-                    # the left most parent is the same, or there are no parents
-                    # today. Either way, we can preserve the representation as
-                    # long as we change the refs to be inserted.
-                    self._data_changed = True
-                    ok_nodes.append((node[0], node[1], node[2],
-                        (ideal_parents, node[3][1])))
-                    self._data_changed = True
-                else:
-                    # Reinsert this text completely
-                    bad_texts.append((node[1], ideal_parents))
-                    self._data_changed = True
-        # we're finished with some data.
-        del ideal_index
-        del text_nodes
-        # 3) bulk copy the ok data
-        total_items, readv_group_iter = self._least_readv_node_readv(ok_nodes)
-        list(self._copy_nodes_graph(text_index_map, self.new_pack._writer,
-            self.new_pack.text_index, readv_group_iter, total_items))
-        # 4) adhoc copy all the other texts.
-        # We have to topologically insert all texts otherwise we can fail to
-        # reconcile when parts of a single delta chain are preserved intact,
-        # and other parts are not. E.g. Discarded->d1->d2->d3. d1 will be
-        # reinserted, and if d3 has incorrect parents it will also be
-        # reinserted. If we insert d3 first, d2 is present (as it was bulk
-        # copied), so we will try to delta, but d2 is not currently able to be
-        # extracted because it's basis d1 is not present. Topologically sorting
-        # addresses this. The following generates a sort for all the texts that
-        # are being inserted without having to reference the entire text key
-        # space (we only topo sort the revisions, which is smaller).
-        topo_order = tsort.topo_sort(ancestors)
-        rev_order = dict(zip(topo_order, range(len(topo_order))))
-        bad_texts.sort(key=lambda key:rev_order.get(key[0][1], 0))
-        transaction = repo.get_transaction()
-        file_id_index = GraphIndexPrefixAdapter(
-            self.new_pack.text_index,
-            ('blank', ), 1,
-            add_nodes_callback=self.new_pack.text_index.add_nodes)
-        data_access = _DirectPackAccess(
-                {self.new_pack.text_index:self.new_pack.access_tuple()})
-        data_access.set_writer(self.new_pack._writer, self.new_pack.text_index,
-            self.new_pack.access_tuple())
-        output_texts = KnitVersionedFiles(
-            _KnitGraphIndex(self.new_pack.text_index,
-                add_callback=self.new_pack.text_index.add_nodes,
-                deltas=True, parents=True, is_locked=repo.is_locked),
-            data_access=data_access, max_delta_chain=200)
-        for key, parent_keys in bad_texts:
-            # We refer to the new pack to delta data being output.
-            # A possible improvement would be to catch errors on short reads
-            # and only flush then.
-            self.new_pack.flush()
-            parents = []
-            for parent_key in parent_keys:
-                if parent_key[0] != key[0]:
-                    # Graph parents must match the fileid
-                    raise errors.BzrError('Mismatched key parent %r:%r' %
-                        (key, parent_keys))
-                parents.append(parent_key[1])
-            text_lines = osutils.split_lines(repo.texts.get_record_stream(
-                [key], 'unordered', True).next().get_bytes_as('fulltext'))
-            output_texts.add_lines(key, parent_keys, text_lines,
-                random_id=True, check_content=False)
-        # 5) check that nothing inserted has a reference outside the keyspace.
-        missing_text_keys = self.new_pack.text_index._external_references()
-        if missing_text_keys:
-            raise errors.BzrCheckError('Reference to missing compression parents %r'
-                % (missing_text_keys,))
-        self._log_copied_texts()
-
-    def _use_pack(self, new_pack):
-        """Override _use_pack to check for reconcile having changed content."""
-        # XXX: we might be better checking this at the copy time.
-        original_inventory_keys = set()
-        inv_index = self._pack_collection.inventory_index.combined_index
-        for entry in inv_index.iter_all_entries():
-            original_inventory_keys.add(entry[1])
-        new_inventory_keys = set()
-        for entry in new_pack.inventory_index.iter_all_entries():
-            new_inventory_keys.add(entry[1])
-        if new_inventory_keys != original_inventory_keys:
-            self._data_changed = True
-        return new_pack.data_inserted() and self._data_changed
-
-
 class RepositoryPackCollection(object):
     """Management of packs within a repository.
 
     :ivar _names: map of {pack_name: (index_size,)}
     """
 
-    pack_factory = NewPack
-    resumed_pack_factory = ResumedPack
+    pack_factory = None
+    resumed_pack_factory = None
+    normal_packer_class = None
+    optimising_packer_class = None
 
     def __init__(self, repo, transport, index_transport, upload_transport,
                  pack_transport, index_builder_class, index_class,
@@ -1416,6 +831,7 @@
                 set(all_combined).difference([combined_idx]))
         # resumed packs
         self._resumed_packs = []
+        self.config_stack = config.LocationStack(self.transport.base)
 
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.repo)
@@ -1506,27 +922,27 @@
             'containing %d revisions. Packing %d files into %d affecting %d'
             ' revisions', self, total_packs, total_revisions, num_old_packs,
             num_new_packs, num_revs_affected)
-        result = self._execute_pack_operations(pack_operations,
+        result = self._execute_pack_operations(pack_operations, packer_class=self.normal_packer_class,
                                       reload_func=self._restart_autopack)
         mutter('Auto-packing repository %s completed', self)
         return result
 
-    def _execute_pack_operations(self, pack_operations, _packer_class=Packer,
-                                 reload_func=None):
+    def _execute_pack_operations(self, pack_operations, packer_class,
+            reload_func=None):
         """Execute a series of pack operations.
 
         :param pack_operations: A list of [revision_count, packs_to_combine].
-        :param _packer_class: The class of packer to use (default: Packer).
+        :param packer_class: The class of packer to use
         :return: The new pack names.
         """
         for revision_count, packs in pack_operations:
             # we may have no-ops from the setup logic
             if len(packs) == 0:
                 continue
-            packer = _packer_class(self, packs, '.autopack',
+            packer = packer_class(self, packs, '.autopack',
                                    reload_func=reload_func)
             try:
-                packer.pack()
+                result = packer.pack()
             except errors.RetryWithNewPacks:
                 # An exception is propagating out of this context, make sure
                 # this packer has cleaned up. Packer() doesn't set its new_pack
@@ -1535,6 +951,8 @@
                 if packer.new_pack is not None:
                     packer.new_pack.abort()
                 raise
+            if result is None:
+                return
             for pack in packs:
                 self._remove_pack_from_memory(pack)
         # record the newly available packs and stop advertising the old
@@ -1574,6 +992,20 @@
         mutter('Packing repository %s, which has %d pack files, '
             'containing %d revisions with hint %r.', self, total_packs,
             total_revisions, hint)
+        while True:
+            try:
+                self._try_pack_operations(hint)
+            except RetryPackOperations:
+                continue
+            break
+
+        if clean_obsolete_packs:
+            self._clear_obsolete_packs()
+
+    def _try_pack_operations(self, hint):
+        """Calculate the pack operations based on the hint (if any), and
+        execute them.
+        """
         # determine which packs need changing
         pack_operations = [[0, []]]
         for pack in self.all_packs():
@@ -1582,10 +1014,9 @@
                 # or this pack was included in the hint.
                 pack_operations[-1][0] += pack.get_revision_count()
                 pack_operations[-1][1].append(pack)
-        self._execute_pack_operations(pack_operations, OptimisingPacker)
-
-        if clean_obsolete_packs:
-            self._clear_obsolete_packs()
+        self._execute_pack_operations(pack_operations,
+            packer_class=self.optimising_packer_class,
+            reload_func=self._restart_pack_operations)
 
     def plan_autopack_combinations(self, existing_packs, pack_distribution):
         """Plan a pack operation.
@@ -1601,9 +1032,9 @@
         pack_operations = [[0, []]]
         # plan out what packs to keep, and what to reorganise
         while len(existing_packs):
-            # take the largest pack, and if its less than the head of the
+            # take the largest pack, and if it's less than the head of the
             # distribution chart we will include its contents in the new pack
-            # for that position. If its larger, we remove its size from the
+            # for that position. If it's larger, we remove its size from the
             # distribution chart
             next_pack_rev_count, next_pack = existing_packs.pop(0)
             if next_pack_rev_count >= pack_distribution[0]:
@@ -1644,7 +1075,7 @@
 
         :return: True if the disk names had not been previously read.
         """
-        # NB: if you see an assertion error here, its probably access against
+        # NB: if you see an assertion error here, it's probably access against
         # an unlocked repo. Naughty.
         if not self.repo.is_locked():
             raise errors.ObjectNotLocked(self.repo)
@@ -1680,7 +1111,7 @@
             txt_index = self._make_index(name, '.tix')
             sig_index = self._make_index(name, '.six')
             if self.chk_index is not None:
-                chk_index = self._make_index(name, '.cix', unlimited_cache=True)
+                chk_index = self._make_index(name, '.cix', is_chk=True)
             else:
                 chk_index = None
             result = ExistingPack(self._pack_transport, name, rev_index,
@@ -1706,7 +1137,7 @@
             sig_index = self._make_index(name, '.six', resume=True)
             if self.chk_index is not None:
                 chk_index = self._make_index(name, '.cix', resume=True,
-                                             unlimited_cache=True)
+                                             is_chk=True)
             else:
                 chk_index = None
             result = self.resumed_pack_factory(name, rev_index, inv_index,
@@ -1742,7 +1173,7 @@
         return self._index_class(self.transport, 'pack-names', None
                 ).iter_all_entries()
 
-    def _make_index(self, name, suffix, resume=False, unlimited_cache=False):
+    def _make_index(self, name, suffix, resume=False, is_chk=False):
         size_offset = self._suffix_offsets[suffix]
         index_name = name + suffix
         if resume:
@@ -1751,8 +1182,11 @@
         else:
             transport = self._index_transport
             index_size = self._names[name][size_offset]
-        return self._index_class(transport, index_name, index_size,
-                                 unlimited_cache=unlimited_cache)
+        index = self._index_class(transport, index_name, index_size,
+                                  unlimited_cache=is_chk)
+        if is_chk and self._index_class is btree_index.BTreeGraphIndex: 
+            index._leaf_factory = btree_index._gcchk_factory
+        return index
 
     def _max_pack_count(self, total_revisions):
         """Return the maximum number of packs to use for total revisions.
@@ -1787,7 +1221,7 @@
         """
         for pack in packs:
             try:
-                pack.pack_transport.rename(pack.file_name(),
+                pack.pack_transport.move(pack.file_name(),
                     '../obsolete_packs/' + pack.file_name())
             except (errors.PathError, errors.TransportError), e:
                 # TODO: Should these be warnings or mutters?
@@ -1801,7 +1235,7 @@
                 suffixes.append('.cix')
             for suffix in suffixes:
                 try:
-                    self._index_transport.rename(pack.name + suffix,
+                    self._index_transport.move(pack.name + suffix,
                         '../obsolete_packs/' + pack.name + suffix)
                 except (errors.PathError, errors.TransportError), e:
                     mutter("couldn't rename obsolete index, skipping it:\n%s"
@@ -1944,7 +1378,7 @@
                     # disk index because the set values are the same, unless
                     # the only index shows up as deleted by the set difference
                     # - which it may. Until there is a specific test for this,
-                    # assume its broken. RBC 20071017.
+                    # assume it's broken. RBC 20071017.
                     self._remove_pack_from_memory(self.get_pack_by_name(name))
                     self._names[name] = sizes
                     self.get_pack_by_name(name)
@@ -2015,9 +1449,9 @@
         """
         # The ensure_loaded call is to handle the case where the first call
         # made involving the collection was to reload_pack_names, where we 
-        # don't have a view of disk contents. Its a bit of a bandaid, and
-        # causes two reads of pack-names, but its a rare corner case not struck
-        # with regular push/pull etc.
+        # don't have a view of disk contents. It's a bit of a bandaid, and
+        # causes two reads of pack-names, but it's a rare corner case not
+        # struck with regular push/pull etc.
         first_read = self.ensure_loaded()
         if first_read:
             return True
@@ -2042,6 +1476,14 @@
             raise
         raise errors.RetryAutopack(self.repo, False, sys.exc_info())
 
+    def _restart_pack_operations(self):
+        """Reload the pack names list, and restart the autopack code."""
+        if not self.reload_pack_names():
+            # Re-raise the original exception, because something went missing
+            # and a restart didn't find it
+            raise
+        raise RetryPackOperations(self.repo, False, sys.exc_info())
+
     def _clear_obsolete_packs(self, preserve=None):
         """Delete everything from the obsolete-packs directory.
 
@@ -2195,7 +1637,7 @@
             self._resume_pack(token)
 
 
-class KnitPackRepository(KnitRepository):
+class PackRepository(MetaDirVersionedFileRepository):
     """Repository with knit objects stored inside pack containers.
 
     The layering for a KnitPackRepository is:
@@ -2204,7 +1646,7 @@
     ===================================================
     Tuple based apis below, string based, and key based apis above
     ---------------------------------------------------
-    KnitVersionedFiles
+    VersionedFiles
       Provides .texts, .revisions etc
       This adapts the N-tuple keys to physical knit records which only have a
       single string identifier (for historical reasons), which in older formats
@@ -2220,89 +1662,47 @@
 
     """
 
+    # These attributes are inherited from the Repository base class. Setting
+    # them to None ensures that if the constructor is changed to not initialize
+    # them, or a subclass fails to call the constructor, that an error will
+    # occur rather than the system working but generating incorrect data.
+    _commit_builder_class = None
+    _serializer = None
+
     def __init__(self, _format, a_bzrdir, control_files, _commit_builder_class,
         _serializer):
-        KnitRepository.__init__(self, _format, a_bzrdir, control_files,
-            _commit_builder_class, _serializer)
-        index_transport = self._transport.clone('indices')
-        self._pack_collection = RepositoryPackCollection(self, self._transport,
-            index_transport,
-            self._transport.clone('upload'),
-            self._transport.clone('packs'),
-            _format.index_builder_class,
-            _format.index_class,
-            use_chk_index=self._format.supports_chks,
-            )
-        self.inventories = KnitVersionedFiles(
-            _KnitGraphIndex(self._pack_collection.inventory_index.combined_index,
-                add_callback=self._pack_collection.inventory_index.add_callback,
-                deltas=True, parents=True, is_locked=self.is_locked),
-            data_access=self._pack_collection.inventory_index.data_access,
-            max_delta_chain=200)
-        self.revisions = KnitVersionedFiles(
-            _KnitGraphIndex(self._pack_collection.revision_index.combined_index,
-                add_callback=self._pack_collection.revision_index.add_callback,
-                deltas=False, parents=True, is_locked=self.is_locked,
-                track_external_parent_refs=True),
-            data_access=self._pack_collection.revision_index.data_access,
-            max_delta_chain=0)
-        self.signatures = KnitVersionedFiles(
-            _KnitGraphIndex(self._pack_collection.signature_index.combined_index,
-                add_callback=self._pack_collection.signature_index.add_callback,
-                deltas=False, parents=False, is_locked=self.is_locked),
-            data_access=self._pack_collection.signature_index.data_access,
-            max_delta_chain=0)
-        self.texts = KnitVersionedFiles(
-            _KnitGraphIndex(self._pack_collection.text_index.combined_index,
-                add_callback=self._pack_collection.text_index.add_callback,
-                deltas=True, parents=True, is_locked=self.is_locked),
-            data_access=self._pack_collection.text_index.data_access,
-            max_delta_chain=200)
-        if _format.supports_chks:
-            # No graph, no compression:- references from chks are between
-            # different objects not temporal versions of the same; and without
-            # some sort of temporal structure knit compression will just fail.
-            self.chk_bytes = KnitVersionedFiles(
-                _KnitGraphIndex(self._pack_collection.chk_index.combined_index,
-                    add_callback=self._pack_collection.chk_index.add_callback,
-                    deltas=False, parents=False, is_locked=self.is_locked),
-                data_access=self._pack_collection.chk_index.data_access,
-                max_delta_chain=0)
-        else:
-            self.chk_bytes = None
-        # True when the repository object is 'write locked' (as opposed to the
-        # physical lock only taken out around changes to the pack-names list.)
-        # Another way to represent this would be a decorator around the control
-        # files object that presents logical locks as physical ones - if this
-        # gets ugly consider that alternative design. RBC 20071011
-        self._write_lock_count = 0
-        self._transaction = None
-        # for tests
-        self._reconcile_does_inventory_gc = True
+        MetaDirRepository.__init__(self, _format, a_bzrdir, control_files)
+        self._commit_builder_class = _commit_builder_class
+        self._serializer = _serializer
         self._reconcile_fixes_text_parents = True
-        self._reconcile_backsup_inventory = False
+        if self._format.supports_external_lookups:
+            self._unstacked_provider = graph.CachingParentsProvider(
+                self._make_parents_provider_unstacked())
+        else:
+            self._unstacked_provider = graph.CachingParentsProvider(self)
+        self._unstacked_provider.disable_cache()
 
-    def _warn_if_deprecated(self, branch=None):
-        # This class isn't deprecated, but one sub-format is
-        if isinstance(self._format, RepositoryFormatKnitPack5RichRootBroken):
-            super(KnitPackRepository, self)._warn_if_deprecated(branch)
+    @needs_read_lock
+    def _all_revision_ids(self):
+        """See Repository.all_revision_ids()."""
+        return [key[0] for key in self.revisions.keys()]
 
     def _abort_write_group(self):
         self.revisions._index._key_dependencies.clear()
         self._pack_collection._abort_write_group()
 
-    def _get_source(self, to_format):
-        if to_format.network_name() == self._format.network_name():
-            return KnitPackStreamSource(self, to_format)
-        return super(KnitPackRepository, self)._get_source(to_format)
-
     def _make_parents_provider(self):
-        return graph.CachingParentsProvider(self)
+        if not self._format.supports_external_lookups:
+            return self._unstacked_provider
+        return graph.StackedParentsProvider(_LazyListJoin(
+            [self._unstacked_provider], self._fallback_repositories))
 
     def _refresh_data(self):
         if not self.is_locked():
             return
         self._pack_collection.reload_pack_names()
+        self._unstacked_provider.disable_cache()
+        self._unstacked_provider.enable_cache()
 
     def _start_write_group(self):
         self._pack_collection._start_write_group()
@@ -2310,6 +1710,10 @@
     def _commit_write_group(self):
         hint = self._pack_collection._commit_write_group()
         self.revisions._index._key_dependencies.clear()
+        # The commit may have added keys that were previously cached as
+        # missing, so reset the cache.
+        self._unstacked_provider.disable_cache()
+        self._unstacked_provider.enable_cache()
         return hint
 
     def suspend_write_group(self):
@@ -2356,6 +1760,7 @@
             if 'relock' in debug.debug_flags and self._prev_lock == 'w':
                 note('%r was write locked again', self)
             self._prev_lock = 'w'
+            self._unstacked_provider.enable_cache()
             for repo in self._fallback_repositories:
                 # Writes don't affect fallback repos
                 repo.lock_read()
@@ -2376,6 +1781,7 @@
             if 'relock' in debug.debug_flags and self._prev_lock == 'r':
                 note('%r was read locked again', self)
             self._prev_lock = 'r'
+            self._unstacked_provider.enable_cache()
             for repo in self._fallback_repositories:
                 repo.lock_read()
             self._refresh_data()
@@ -2407,13 +1813,13 @@
         return reconciler
 
     def _reconcile_pack(self, collection, packs, extension, revs, pb):
-        packer = ReconcilePacker(collection, packs, extension, revs)
-        return packer.pack(pb)
+        raise NotImplementedError(self._reconcile_pack)
 
     @only_raises(errors.LockNotHeld, errors.LockBroken)
     def unlock(self):
         if self._write_lock_count == 1 and self._write_group is not None:
             self.abort_write_group()
+            self._unstacked_provider.disable_cache()
             self._transaction = None
             self._write_lock_count = 0
             raise errors.BzrError(
@@ -2429,84 +1835,12 @@
             self.control_files.unlock()
 
         if not self.is_locked():
+            self._unstacked_provider.disable_cache()
             for repo in self._fallback_repositories:
                 repo.unlock()
 
 
-class KnitPackStreamSource(StreamSource):
-    """A StreamSource used to transfer data between same-format KnitPack repos.
-
-    This source assumes:
-        1) Same serialization format for all objects
-        2) Same root information
-        3) XML format inventories
-        4) Atomic inserts (so we can stream inventory texts before text
-           content)
-        5) No chk_bytes
-    """
-
-    def __init__(self, from_repository, to_format):
-        super(KnitPackStreamSource, self).__init__(from_repository, to_format)
-        self._text_keys = None
-        self._text_fetch_order = 'unordered'
-
-    def _get_filtered_inv_stream(self, revision_ids):
-        from_repo = self.from_repository
-        parent_ids = from_repo._find_parent_ids_of_revisions(revision_ids)
-        parent_keys = [(p,) for p in parent_ids]
-        find_text_keys = from_repo._find_text_key_references_from_xml_inventory_lines
-        parent_text_keys = set(find_text_keys(
-            from_repo._inventory_xml_lines_for_keys(parent_keys)))
-        content_text_keys = set()
-        knit = KnitVersionedFiles(None, None)
-        factory = KnitPlainFactory()
-        def find_text_keys_from_content(record):
-            if record.storage_kind not in ('knit-delta-gz', 'knit-ft-gz'):
-                raise ValueError("Unknown content storage kind for"
-                    " inventory text: %s" % (record.storage_kind,))
-            # It's a knit record, it has a _raw_record field (even if it was
-            # reconstituted from a network stream).
-            raw_data = record._raw_record
-            # read the entire thing
-            revision_id = record.key[-1]
-            content, _ = knit._parse_record(revision_id, raw_data)
-            if record.storage_kind == 'knit-delta-gz':
-                line_iterator = factory.get_linedelta_content(content)
-            elif record.storage_kind == 'knit-ft-gz':
-                line_iterator = factory.get_fulltext_content(content)
-            content_text_keys.update(find_text_keys(
-                [(line, revision_id) for line in line_iterator]))
-        revision_keys = [(r,) for r in revision_ids]
-        def _filtered_inv_stream():
-            source_vf = from_repo.inventories
-            stream = source_vf.get_record_stream(revision_keys,
-                                                 'unordered', False)
-            for record in stream:
-                if record.storage_kind == 'absent':
-                    raise errors.NoSuchRevision(from_repo, record.key)
-                find_text_keys_from_content(record)
-                yield record
-            self._text_keys = content_text_keys - parent_text_keys
-        return ('inventories', _filtered_inv_stream())
-
-    def _get_text_stream(self):
-        # Note: We know we don't have to handle adding root keys, because both
-        # the source and target are the identical network name.
-        text_stream = self.from_repository.texts.get_record_stream(
-                        self._text_keys, self._text_fetch_order, False)
-        return ('texts', text_stream)
-
-    def get_stream(self, search):
-        revision_ids = search.get_keys()
-        for stream_info in self._fetch_revision_texts(revision_ids):
-            yield stream_info
-        self._revision_keys = [(rev_id,) for rev_id in revision_ids]
-        yield self._get_filtered_inv_stream(revision_ids)
-        yield self._get_text_stream()
-
-
-
-class RepositoryFormatPack(MetaDirRepositoryFormat):
+class RepositoryFormatPack(MetaDirVersionedFileRepositoryFormat):
     """Format logic for pack structured repositories.
 
     This repository format has:
@@ -2542,6 +1876,8 @@
     index_class = None
     _fetch_uses_deltas = True
     fast_deltas = False
+    supports_funky_characters = True
+    revision_graph_can_have_wrong_parents = True
 
     def initialize(self, a_bzrdir, shared=False):
         """Create a pack based repository.
@@ -2584,352 +1920,156 @@
                               _serializer=self._serializer)
 
 
-class RepositoryFormatKnitPack1(RepositoryFormatPack):
-    """A no-subtrees parameterized Pack repository.
-
-    This format was introduced in 0.92.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackCommitBuilder
-    @property
-    def _serializer(self):
-        return xml5.serializer_v5
-    # What index classes to use
-    index_builder_class = InMemoryGraphIndex
-    index_class = GraphIndex
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir('pack-0.92')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar pack repository format 1 (needs bzr 0.92)\n"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Packs containing knits without subtree support"
-
-
-class RepositoryFormatKnitPack3(RepositoryFormatPack):
-    """A subtrees parameterized Pack repository.
-
-    This repository format uses the xml7 serializer to get:
-     - support for recording full info about the tree root
-     - support for recording tree-references
-
-    This format was introduced in 0.92.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackRootCommitBuilder
-    rich_root_data = True
-    experimental = True
-    supports_tree_reference = True
-    @property
-    def _serializer(self):
-        return xml7.serializer_v7
-    # What index classes to use
-    index_builder_class = InMemoryGraphIndex
-    index_class = GraphIndex
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir(
-            'pack-0.92-subtree')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar pack repository format 1 with subtree support (needs bzr 0.92)\n"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Packs containing knits with subtree support\n"
-
-
-class RepositoryFormatKnitPack4(RepositoryFormatPack):
-    """A rich-root, no subtrees parameterized Pack repository.
-
-    This repository format uses the xml6 serializer to get:
-     - support for recording full info about the tree root
-
-    This format was introduced in 1.0.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackRootCommitBuilder
-    rich_root_data = True
-    supports_tree_reference = False
-    @property
-    def _serializer(self):
-        return xml6.serializer_v6
-    # What index classes to use
-    index_builder_class = InMemoryGraphIndex
-    index_class = GraphIndex
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir(
-            'rich-root-pack')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return ("Bazaar pack repository format 1 with rich root"
-                " (needs bzr 1.0)\n")
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Packs containing knits with rich root support\n"
-
-
-class RepositoryFormatKnitPack5(RepositoryFormatPack):
-    """Repository that supports external references to allow stacking.
-
-    New in release 1.6.
-
-    Supports external lookups, which results in non-truncated ghosts after
-    reconcile compared to pack-0.92 formats.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackCommitBuilder
-    supports_external_lookups = True
-    # What index classes to use
-    index_builder_class = InMemoryGraphIndex
-    index_class = GraphIndex
-
-    @property
-    def _serializer(self):
-        return xml5.serializer_v5
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir('1.6')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar RepositoryFormatKnitPack5 (bzr 1.6)\n"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Packs 5 (adds stacking support, requires bzr 1.6)"
-
-
-class RepositoryFormatKnitPack5RichRoot(RepositoryFormatPack):
-    """A repository with rich roots and stacking.
-
-    New in release 1.6.1.
-
-    Supports stacking on other repositories, allowing data to be accessed
-    without being stored locally.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackRootCommitBuilder
-    rich_root_data = True
-    supports_tree_reference = False # no subtrees
-    supports_external_lookups = True
-    # What index classes to use
-    index_builder_class = InMemoryGraphIndex
-    index_class = GraphIndex
-
-    @property
-    def _serializer(self):
-        return xml6.serializer_v6
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir(
-            '1.6.1-rich-root')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6.1)\n"
-
-    def get_format_description(self):
-        return "Packs 5 rich-root (adds stacking support, requires bzr 1.6.1)"
-
-
-class RepositoryFormatKnitPack5RichRootBroken(RepositoryFormatPack):
-    """A repository with rich roots and external references.
-
-    New in release 1.6.
-
-    Supports external lookups, which results in non-truncated ghosts after
-    reconcile compared to pack-0.92 formats.
-
-    This format was deprecated because the serializer it uses accidentally
-    supported subtrees, when the format was not intended to. This meant that
-    someone could accidentally fetch from an incorrect repository.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackRootCommitBuilder
-    rich_root_data = True
-    supports_tree_reference = False # no subtrees
-
-    supports_external_lookups = True
-    # What index classes to use
-    index_builder_class = InMemoryGraphIndex
-    index_class = GraphIndex
-
-    @property
-    def _serializer(self):
-        return xml7.serializer_v7
-
-    def _get_matching_bzrdir(self):
-        matching = bzrdir.format_registry.make_bzrdir(
-            '1.6.1-rich-root')
-        matching.repository_format = self
-        return matching
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6)\n"
-
-    def get_format_description(self):
-        return ("Packs 5 rich-root (adds stacking support, requires bzr 1.6)"
-                " (deprecated)")
-
-
-class RepositoryFormatKnitPack6(RepositoryFormatPack):
-    """A repository with stacking and btree indexes,
-    without rich roots or subtrees.
-
-    This is equivalent to pack-1.6 with B+Tree indices.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackCommitBuilder
-    supports_external_lookups = True
-    # What index classes to use
-    index_builder_class = BTreeBuilder
-    index_class = BTreeGraphIndex
-
-    @property
-    def _serializer(self):
-        return xml5.serializer_v5
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir('1.9')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar RepositoryFormatKnitPack6 (bzr 1.9)\n"
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return "Packs 6 (uses btree indexes, requires bzr 1.9)"
-
-
-class RepositoryFormatKnitPack6RichRoot(RepositoryFormatPack):
-    """A repository with rich roots, no subtrees, stacking and btree indexes.
-
-    1.6-rich-root with B+Tree indices.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackRootCommitBuilder
-    rich_root_data = True
-    supports_tree_reference = False # no subtrees
-    supports_external_lookups = True
-    # What index classes to use
-    index_builder_class = BTreeBuilder
-    index_class = BTreeGraphIndex
-
-    @property
-    def _serializer(self):
-        return xml6.serializer_v6
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir(
-            '1.9-rich-root')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return "Bazaar RepositoryFormatKnitPack6RichRoot (bzr 1.9)\n"
-
-    def get_format_description(self):
-        return "Packs 6 rich-root (uses btree indexes, requires bzr 1.9)"
-
-
-class RepositoryFormatPackDevelopment2Subtree(RepositoryFormatPack):
-    """A subtrees development repository.
-
-    This format should be retained until the second release after bzr 1.7.
-
-    1.6.1-subtree[as it might have been] with B+Tree indices.
-
-    This is [now] retained until we have a CHK based subtree format in
-    development.
-    """
-
-    repository_class = KnitPackRepository
-    _commit_builder_class = PackRootCommitBuilder
-    rich_root_data = True
-    experimental = True
-    supports_tree_reference = True
-    supports_external_lookups = True
-    # What index classes to use
-    index_builder_class = BTreeBuilder
-    index_class = BTreeGraphIndex
-
-    @property
-    def _serializer(self):
-        return xml7.serializer_v7
-
-    def _get_matching_bzrdir(self):
-        return bzrdir.format_registry.make_bzrdir(
-            'development-subtree')
-
-    def _ignore_setting_bzrdir(self, format):
-        pass
-
-    _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
-
-    def get_format_string(self):
-        """See RepositoryFormat.get_format_string()."""
-        return ("Bazaar development format 2 with subtree support "
-            "(needs bzr.dev from before 1.8)\n")
-
-    def get_format_description(self):
-        """See RepositoryFormat.get_format_description()."""
-        return ("Development repository format, currently the same as "
-            "1.6.1-subtree with B+Tree indices.\n")
+class RetryPackOperations(errors.RetryWithNewPacks):
+    """Raised when we are packing and we find a missing file.
+
+    Meant as a signaling exception, to tell the RepositoryPackCollection.pack
+    code it should try again.
+    """
+
+    internal_error = True
+
+    _fmt = ("Pack files have changed, reload and try pack again."
+            " context: %(context)s %(orig_error)s")
+
+
+class _DirectPackAccess(object):
+    """Access to data in one or more packs with less translation."""
+
+    def __init__(self, index_to_packs, reload_func=None, flush_func=None):
+        """Create a _DirectPackAccess object.
+
+        :param index_to_packs: A dict mapping index objects to the transport
+            and file names for obtaining data.
+        :param reload_func: A function to call if we determine that the pack
+            files have moved and we need to reload our caches. See
+            bzrlib.repo_fmt.pack_repo.AggregateIndex for more details.
+        """
+        self._container_writer = None
+        self._write_index = None
+        self._indices = index_to_packs
+        self._reload_func = reload_func
+        self._flush_func = flush_func
+
+    def add_raw_records(self, key_sizes, raw_data):
+        """Add raw knit bytes to a storage area.
+
+        The data is spooled to the container writer in one bytes-record per
+        raw data item.
+
+        :param sizes: An iterable of tuples containing the key and size of each
+            raw data segment.
+        :param raw_data: A bytestring containing the data.
+        :return: A list of memos to retrieve the record later. Each memo is an
+            opaque index memo. For _DirectPackAccess the memo is (index, pos,
+            length), where the index field is the write_index object supplied
+            to the PackAccess object.
+        """
+        if type(raw_data) is not str:
+            raise AssertionError(
+                'data must be plain bytes was %s' % type(raw_data))
+        result = []
+        offset = 0
+        for key, size in key_sizes:
+            p_offset, p_length = self._container_writer.add_bytes_record(
+                raw_data[offset:offset+size], [])
+            offset += size
+            result.append((self._write_index, p_offset, p_length))
+        return result
+
+    def flush(self):
+        """Flush pending writes on this access object.
+
+        This will flush any buffered writes to a NewPack.
+        """
+        if self._flush_func is not None:
+            self._flush_func()
+
+    def get_raw_records(self, memos_for_retrieval):
+        """Get the raw bytes for a records.
+
+        :param memos_for_retrieval: An iterable containing the (index, pos,
+            length) memo for retrieving the bytes. The Pack access method
+            looks up the pack to use for a given record in its index_to_pack
+            map.
+        :return: An iterator over the bytes of the records.
+        """
+        # first pass, group into same-index requests
+        request_lists = []
+        current_index = None
+        for (index, offset, length) in memos_for_retrieval:
+            if current_index == index:
+                current_list.append((offset, length))
+            else:
+                if current_index is not None:
+                    request_lists.append((current_index, current_list))
+                current_index = index
+                current_list = [(offset, length)]
+        # handle the last entry
+        if current_index is not None:
+            request_lists.append((current_index, current_list))
+        for index, offsets in request_lists:
+            try:
+                transport, path = self._indices[index]
+            except KeyError:
+                # A KeyError here indicates that someone has triggered an index
+                # reload, and this index has gone missing, we need to start
+                # over.
+                if self._reload_func is None:
+                    # If we don't have a _reload_func there is nothing that can
+                    # be done
+                    raise
+                raise errors.RetryWithNewPacks(index,
+                                               reload_occurred=True,
+                                               exc_info=sys.exc_info())
+            try:
+                reader = pack.make_readv_reader(transport, path, offsets)
+                for names, read_func in reader.iter_records():
+                    yield read_func(None)
+            except errors.NoSuchFile:
+                # A NoSuchFile error indicates that a pack file has gone
+                # missing on disk, we need to trigger a reload, and start over.
+                if self._reload_func is None:
+                    raise
+                raise errors.RetryWithNewPacks(transport.abspath(path),
+                                               reload_occurred=False,
+                                               exc_info=sys.exc_info())
+
+    def set_writer(self, writer, index, transport_packname):
+        """Set a writer to use for adding data."""
+        if index is not None:
+            self._indices[index] = transport_packname
+        self._container_writer = writer
+        self._write_index = index
+
+    def reload_or_raise(self, retry_exc):
+        """Try calling the reload function, or re-raise the original exception.
+
+        This should be called after _DirectPackAccess raises a
+        RetryWithNewPacks exception. This function will handle the common logic
+        of determining when the error is fatal versus being temporary.
+        It will also make sure that the original exception is raised, rather
+        than the RetryWithNewPacks exception.
+
+        If this function returns, then the calling function should retry
+        whatever operation was being performed. Otherwise an exception will
+        be raised.
+
+        :param retry_exc: A RetryWithNewPacks exception.
+        """
+        is_error = False
+        if self._reload_func is None:
+            is_error = True
+        elif not self._reload_func():
+            # The reload claimed that nothing changed
+            if not retry_exc.reload_occurred:
+                # If there wasn't an earlier reload, then we really were
+                # expecting to find changes. We didn't find them, so this is a
+                # hard error
+                is_error = True
+        if is_error:
+            exc_class, exc_value, exc_traceback = retry_exc.exc_info
+            raise exc_class, exc_value, exc_traceback
+
+
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2010-05-25 17:27:52 +0000
+++ bzrlib/repository.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,55 +16,36 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import cStringIO
-import re
+import itertools
 import time
 
 from bzrlib import (
     bzrdir,
-    check,
-    chk_map,
     config,
+    controldir,
     debug,
-    fetch as _mod_fetch,
-    fifo_cache,
     generate_ids,
-    gpg,
     graph,
-    inventory,
-    inventory_delta,
-    lazy_regex,
     lockable_files,
     lockdir,
-    lru_cache,
     osutils,
     revision as _mod_revision,
-    static_tuple,
-    symbol_versioning,
-    trace,
+    testament as _mod_testament,
     tsort,
-    versionedfile,
+    gpg,
     )
 from bzrlib.bundle import serializer
-from bzrlib.revisiontree import RevisionTree
-from bzrlib.store.versioned import VersionedFileStore
-from bzrlib.testament import Testament
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib import (
     errors,
     registry,
+    symbol_versioning,
     ui,
     )
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
 from bzrlib.inter import InterObject
-from bzrlib.inventory import (
-    Inventory,
-    InventoryDirectory,
-    ROOT_ID,
-    entry_factory,
-    )
-from bzrlib.recordcounter import RecordCounter
 from bzrlib.lock import _RelockDebugMixin, LogicalLockResult
 from bzrlib.trace import (
     log_exception_quietly, note, mutter, mutter_callsite, warning)
@@ -91,31 +72,34 @@
 
     # all clients should supply tree roots.
     record_root_entry = True
-    # the default CommitBuilder does not manage trees whose root is versioned.
-    _versioned_root = False
+    # whether this commit builder supports the record_entry_contents interface
+    supports_record_entry_contents = False
 
     def __init__(self, repository, parents, config, timestamp=None,
                  timezone=None, committer=None, revprops=None,
-                 revision_id=None):
+                 revision_id=None, lossy=False):
         """Initiate a CommitBuilder.
 
         :param repository: Repository to commit to.
         :param parents: Revision ids of the parents of the new revision.
-        :param config: Configuration to use.
         :param timestamp: Optional timestamp recorded for commit.
         :param timezone: Optional timezone for timestamp.
         :param committer: Optional committer to set for commit.
         :param revprops: Optional dictionary of revision properties.
         :param revision_id: Optional revision id.
+        :param lossy: Whether to discard data that can not be natively
+            represented, when pushing to a foreign VCS 
         """
         self._config = config
+        self._lossy = lossy
 
         if committer is None:
             self._committer = self._config.username()
+        elif not isinstance(committer, unicode):
+            self._committer = committer.decode() # throw if non-ascii
         else:
             self._committer = committer
 
-        self.new_inventory = Inventory(None)
         self._new_revision_id = revision_id
         self.parents = parents
         self.repository = repository
@@ -136,25 +120,16 @@
             self._timezone = int(timezone)
 
         self._generate_revision_if_needed()
-        self.__heads = graph.HeadsCache(repository.get_graph()).heads
-        self._basis_delta = []
-        # API compatibility, older code that used CommitBuilder did not call
-        # .record_delete(), which means the delta that is computed would not be
-        # valid. Callers that will call record_delete() should call
-        # .will_record_deletes() to indicate that.
-        self._recording_deletes = False
-        # memo'd check for no-op commits.
-        self._any_changes = False
 
     def any_changes(self):
         """Return True if any entries were changed.
-        
+
         This includes merge-only changes. It is the core for the --unchanged
         detection in commit.
 
         :return: True if any changes have occured.
         """
-        return self._any_changes
+        raise NotImplementedError(self.any_changes)
 
     def _validate_unicode_text(self, text, context):
         """Verify things like commit messages don't have bogus characters."""
@@ -176,40 +151,23 @@
 
         :return: The revision id of the recorded revision.
         """
-        self._validate_unicode_text(message, 'commit message')
-        rev = _mod_revision.Revision(
-                       timestamp=self._timestamp,
-                       timezone=self._timezone,
-                       committer=self._committer,
-                       message=message,
-                       inventory_sha1=self.inv_sha1,
-                       revision_id=self._new_revision_id,
-                       properties=self._revprops)
-        rev.parent_ids = self.parents
-        self.repository.add_revision(self._new_revision_id, rev,
-            self.new_inventory, self._config)
-        self.repository.commit_write_group()
-        return self._new_revision_id
+        raise NotImplementedError(self.commit)
 
     def abort(self):
         """Abort the commit that is being built.
         """
-        self.repository.abort_write_group()
+        raise NotImplementedError(self.abort)
 
     def revision_tree(self):
         """Return the tree that was just committed.
 
-        After calling commit() this can be called to get a RevisionTree
-        representing the newly committed tree. This is preferred to
-        calling Repository.revision_tree() because that may require
-        deserializing the inventory, while we already have a copy in
+        After calling commit() this can be called to get a
+        RevisionTree representing the newly committed tree. This is
+        preferred to calling Repository.revision_tree() because that may
+        require deserializing the inventory, while we already have a copy in
         memory.
         """
-        if self.new_inventory is None:
-            self.new_inventory = self.repository.get_inventory(
-                self._new_revision_id)
-        return RevisionTree(self.repository, self.new_inventory,
-            self._new_revision_id)
+        raise NotImplementedError(self.revision_tree)
 
     def finish_inventory(self):
         """Tell the builder that the inventory is finished.
@@ -217,33 +175,11 @@
         :return: The inventory id in the repository, which can be used with
             repository.get_inventory.
         """
-        if self.new_inventory is None:
-            # an inventory delta was accumulated without creating a new
-            # inventory.
-            basis_id = self.basis_delta_revision
-            # We ignore the 'inventory' returned by add_inventory_by_delta
-            # because self.new_inventory is used to hint to the rest of the
-            # system what code path was taken
-            self.inv_sha1, _ = self.repository.add_inventory_by_delta(
-                basis_id, self._basis_delta, self._new_revision_id,
-                self.parents)
-        else:
-            if self.new_inventory.root is None:
-                raise AssertionError('Root entry should be supplied to'
-                    ' record_entry_contents, as of bzr 0.10.')
-                self.new_inventory.add(InventoryDirectory(ROOT_ID, '', None))
-            self.new_inventory.revision_id = self._new_revision_id
-            self.inv_sha1 = self.repository.add_inventory(
-                self._new_revision_id,
-                self.new_inventory,
-                self.parents
-                )
-        return self._new_revision_id
+        raise NotImplementedError(self.finish_inventory)
 
     def _gen_revision_id(self):
         """Return new revision-id."""
-        return generate_ids.gen_revision_id(self._config.username(),
-                                            self._timestamp)
+        return generate_ids.gen_revision_id(self._committer, self._timestamp)
 
     def _generate_revision_if_needed(self):
         """Create a revision id if None was supplied.
@@ -260,92 +196,6 @@
         else:
             self.random_revid = False
 
-    def _heads(self, file_id, revision_ids):
-        """Calculate the graph heads for revision_ids in the graph of file_id.
-
-        This can use either a per-file graph or a global revision graph as we
-        have an identity relationship between the two graphs.
-        """
-        return self.__heads(revision_ids)
-
-    def _check_root(self, ie, parent_invs, tree):
-        """Helper for record_entry_contents.
-
-        :param ie: An entry being added.
-        :param parent_invs: The inventories of the parent revisions of the
-            commit.
-        :param tree: The tree that is being committed.
-        """
-        # In this revision format, root entries have no knit or weave When
-        # serializing out to disk and back in root.revision is always
-        # _new_revision_id
-        ie.revision = self._new_revision_id
-
-    def _require_root_change(self, tree):
-        """Enforce an appropriate root object change.
-
-        This is called once when record_iter_changes is called, if and only if
-        the root was not in the delta calculated by record_iter_changes.
-
-        :param tree: The tree which is being committed.
-        """
-        if len(self.parents) == 0:
-            raise errors.RootMissing()
-        entry = entry_factory['directory'](tree.path2id(''), '',
-            None)
-        entry.revision = self._new_revision_id
-        self._basis_delta.append(('', '', entry.file_id, entry))
-
-    def _get_delta(self, ie, basis_inv, path):
-        """Get a delta against the basis inventory for ie."""
-        if ie.file_id not in basis_inv:
-            # add
-            result = (None, path, ie.file_id, ie)
-            self._basis_delta.append(result)
-            return result
-        elif ie != basis_inv[ie.file_id]:
-            # common but altered
-            # TODO: avoid tis id2path call.
-            result = (basis_inv.id2path(ie.file_id), path, ie.file_id, ie)
-            self._basis_delta.append(result)
-            return result
-        else:
-            # common, unaltered
-            return None
-
-    def get_basis_delta(self):
-        """Return the complete inventory delta versus the basis inventory.
-
-        This has been built up with the calls to record_delete and
-        record_entry_contents. The client must have already called
-        will_record_deletes() to indicate that they will be generating a
-        complete delta.
-
-        :return: An inventory delta, suitable for use with apply_delta, or
-            Repository.add_inventory_by_delta, etc.
-        """
-        if not self._recording_deletes:
-            raise AssertionError("recording deletes not activated.")
-        return self._basis_delta
-
-    def record_delete(self, path, file_id):
-        """Record that a delete occured against a basis tree.
-
-        This is an optional API - when used it adds items to the basis_delta
-        being accumulated by the commit builder. It cannot be called unless the
-        method will_record_deletes() has been called to inform the builder that
-        a delta is being supplied.
-
-        :param path: The path of the thing deleted.
-        :param file_id: The file id that was deleted.
-        """
-        if not self._recording_deletes:
-            raise AssertionError("recording deletes not activated.")
-        delta = (path, None, file_id, None)
-        self._basis_delta.append(delta)
-        self._any_changes = True
-        return delta
-
     def will_record_deletes(self):
         """Tell the commit builder that deletes are being notified.
 
@@ -353,224 +203,9 @@
         commit to be valid, deletes against the basis MUST be recorded via
         builder.record_delete().
         """
-        self._recording_deletes = True
-        try:
-            basis_id = self.parents[0]
-        except IndexError:
-            basis_id = _mod_revision.NULL_REVISION
-        self.basis_delta_revision = basis_id
-
-    def record_entry_contents(self, ie, parent_invs, path, tree,
-        content_summary):
-        """Record the content of ie from tree into the commit if needed.
-
-        Side effect: sets ie.revision when unchanged
-
-        :param ie: An inventory entry present in the commit.
-        :param parent_invs: The inventories of the parent revisions of the
-            commit.
-        :param path: The path the entry is at in the tree.
-        :param tree: The tree which contains this entry and should be used to
-            obtain content.
-        :param content_summary: Summary data from the tree about the paths
-            content - stat, length, exec, sha/link target. This is only
-            accessed when the entry has a revision of None - that is when it is
-            a candidate to commit.
-        :return: A tuple (change_delta, version_recorded, fs_hash).
-            change_delta is an inventory_delta change for this entry against
-            the basis tree of the commit, or None if no change occured against
-            the basis tree.
-            version_recorded is True if a new version of the entry has been
-            recorded. For instance, committing a merge where a file was only
-            changed on the other side will return (delta, False).
-            fs_hash is either None, or the hash details for the path (currently
-            a tuple of the contents sha1 and the statvalue returned by
-            tree.get_file_with_stat()).
-        """
-        if self.new_inventory.root is None:
-            if ie.parent_id is not None:
-                raise errors.RootMissing()
-            self._check_root(ie, parent_invs, tree)
-        if ie.revision is None:
-            kind = content_summary[0]
-        else:
-            # ie is carried over from a prior commit
-            kind = ie.kind
-        # XXX: repository specific check for nested tree support goes here - if
-        # the repo doesn't want nested trees we skip it ?
-        if (kind == 'tree-reference' and
-            not self.repository._format.supports_tree_reference):
-            # mismatch between commit builder logic and repository:
-            # this needs the entry creation pushed down into the builder.
-            raise NotImplementedError('Missing repository subtree support.')
-        self.new_inventory.add(ie)
-
-        # TODO: slow, take it out of the inner loop.
-        try:
-            basis_inv = parent_invs[0]
-        except IndexError:
-            basis_inv = Inventory(root_id=None)
-
-        # ie.revision is always None if the InventoryEntry is considered
-        # for committing. We may record the previous parents revision if the
-        # content is actually unchanged against a sole head.
-        if ie.revision is not None:
-            if not self._versioned_root and path == '':
-                # repositories that do not version the root set the root's
-                # revision to the new commit even when no change occurs (more
-                # specifically, they do not record a revision on the root; and
-                # the rev id is assigned to the root during deserialisation -
-                # this masks when a change may have occurred against the basis.
-                # To match this we always issue a delta, because the revision
-                # of the root will always be changing.
-                if ie.file_id in basis_inv:
-                    delta = (basis_inv.id2path(ie.file_id), path,
-                        ie.file_id, ie)
-                else:
-                    # add
-                    delta = (None, path, ie.file_id, ie)
-                self._basis_delta.append(delta)
-                return delta, False, None
-            else:
-                # we don't need to commit this, because the caller already
-                # determined that an existing revision of this file is
-                # appropriate. If its not being considered for committing then
-                # it and all its parents to the root must be unaltered so
-                # no-change against the basis.
-                if ie.revision == self._new_revision_id:
-                    raise AssertionError("Impossible situation, a skipped "
-                        "inventory entry (%r) claims to be modified in this "
-                        "commit (%r).", (ie, self._new_revision_id))
-                return None, False, None
-        # XXX: Friction: parent_candidates should return a list not a dict
-        #      so that we don't have to walk the inventories again.
-        parent_candiate_entries = ie.parent_candidates(parent_invs)
-        head_set = self._heads(ie.file_id, parent_candiate_entries.keys())
-        heads = []
-        for inv in parent_invs:
-            if ie.file_id in inv:
-                old_rev = inv[ie.file_id].revision
-                if old_rev in head_set:
-                    heads.append(inv[ie.file_id].revision)
-                    head_set.remove(inv[ie.file_id].revision)
-
-        store = False
-        # now we check to see if we need to write a new record to the
-        # file-graph.
-        # We write a new entry unless there is one head to the ancestors, and
-        # the kind-derived content is unchanged.
-
-        # Cheapest check first: no ancestors, or more the one head in the
-        # ancestors, we write a new node.
-        if len(heads) != 1:
-            store = True
-        if not store:
-            # There is a single head, look it up for comparison
-            parent_entry = parent_candiate_entries[heads[0]]
-            # if the non-content specific data has changed, we'll be writing a
-            # node:
-            if (parent_entry.parent_id != ie.parent_id or
-                parent_entry.name != ie.name):
-                store = True
-        # now we need to do content specific checks:
-        if not store:
-            # if the kind changed the content obviously has
-            if kind != parent_entry.kind:
-                store = True
-        # Stat cache fingerprint feedback for the caller - None as we usually
-        # don't generate one.
-        fingerprint = None
-        if kind == 'file':
-            if content_summary[2] is None:
-                raise ValueError("Files must not have executable = None")
-            if not store:
-                # We can't trust a check of the file length because of content
-                # filtering...
-                if (# if the exec bit has changed we have to store:
-                    parent_entry.executable != content_summary[2]):
-                    store = True
-                elif parent_entry.text_sha1 == content_summary[3]:
-                    # all meta and content is unchanged (using a hash cache
-                    # hit to check the sha)
-                    ie.revision = parent_entry.revision
-                    ie.text_size = parent_entry.text_size
-                    ie.text_sha1 = parent_entry.text_sha1
-                    ie.executable = parent_entry.executable
-                    return self._get_delta(ie, basis_inv, path), False, None
-                else:
-                    # Either there is only a hash change(no hash cache entry,
-                    # or same size content change), or there is no change on
-                    # this file at all.
-                    # Provide the parent's hash to the store layer, so that the
-                    # content is unchanged we will not store a new node.
-                    nostore_sha = parent_entry.text_sha1
-            if store:
-                # We want to record a new node regardless of the presence or
-                # absence of a content change in the file.
-                nostore_sha = None
-            ie.executable = content_summary[2]
-            file_obj, stat_value = tree.get_file_with_stat(ie.file_id, path)
-            try:
-                text = file_obj.read()
-            finally:
-                file_obj.close()
-            try:
-                ie.text_sha1, ie.text_size = self._add_text_to_weave(
-                    ie.file_id, text, heads, nostore_sha)
-                # Let the caller know we generated a stat fingerprint.
-                fingerprint = (ie.text_sha1, stat_value)
-            except errors.ExistingContent:
-                # Turns out that the file content was unchanged, and we were
-                # only going to store a new node if it was changed. Carry over
-                # the entry.
-                ie.revision = parent_entry.revision
-                ie.text_size = parent_entry.text_size
-                ie.text_sha1 = parent_entry.text_sha1
-                ie.executable = parent_entry.executable
-                return self._get_delta(ie, basis_inv, path), False, None
-        elif kind == 'directory':
-            if not store:
-                # all data is meta here, nothing specific to directory, so
-                # carry over:
-                ie.revision = parent_entry.revision
-                return self._get_delta(ie, basis_inv, path), False, None
-            self._add_text_to_weave(ie.file_id, '', heads, None)
-        elif kind == 'symlink':
-            current_link_target = content_summary[3]
-            if not store:
-                # symlink target is not generic metadata, check if it has
-                # changed.
-                if current_link_target != parent_entry.symlink_target:
-                    store = True
-            if not store:
-                # unchanged, carry over.
-                ie.revision = parent_entry.revision
-                ie.symlink_target = parent_entry.symlink_target
-                return self._get_delta(ie, basis_inv, path), False, None
-            ie.symlink_target = current_link_target
-            self._add_text_to_weave(ie.file_id, '', heads, None)
-        elif kind == 'tree-reference':
-            if not store:
-                if content_summary[3] != parent_entry.reference_revision:
-                    store = True
-            if not store:
-                # unchanged, carry over.
-                ie.reference_revision = parent_entry.reference_revision
-                ie.revision = parent_entry.revision
-                return self._get_delta(ie, basis_inv, path), False, None
-            ie.reference_revision = content_summary[3]
-            if ie.reference_revision is None:
-                raise AssertionError("invalid content_summary for nested tree: %r"
-                    % (content_summary,))
-            self._add_text_to_weave(ie.file_id, '', heads, None)
-        else:
-            raise NotImplementedError('unknown kind')
-        ie.revision = self._new_revision_id
-        self._any_changes = True
-        return self._get_delta(ie, basis_inv, path), True, fingerprint
-
-    def record_iter_changes(self, tree, basis_revision_id, iter_changes,
-        _entry_factory=entry_factory):
+        raise NotImplementedError(self.will_record_deletes)
+
+    def record_iter_changes(self, tree, basis_revision_id, iter_changes):
         """Record a new tree via iter_changes.
 
         :param tree: The tree to obtain text contents from for changed objects.
@@ -581,294 +216,10 @@
             to basis_revision_id. The iterator must not include any items with
             a current kind of None - missing items must be either filtered out
             or errored-on beefore record_iter_changes sees the item.
-        :param _entry_factory: Private method to bind entry_factory locally for
-            performance.
         :return: A generator of (file_id, relpath, fs_hash) tuples for use with
             tree._observed_sha1.
         """
-        # Create an inventory delta based on deltas between all the parents and
-        # deltas between all the parent inventories. We use inventory delta's 
-        # between the inventory objects because iter_changes masks
-        # last-changed-field only changes.
-        # Working data:
-        # file_id -> change map, change is fileid, paths, changed, versioneds,
-        # parents, names, kinds, executables
-        merged_ids = {}
-        # {file_id -> revision_id -> inventory entry, for entries in parent
-        # trees that are not parents[0]
-        parent_entries = {}
-        ghost_basis = False
-        try:
-            revtrees = list(self.repository.revision_trees(self.parents))
-        except errors.NoSuchRevision:
-            # one or more ghosts, slow path.
-            revtrees = []
-            for revision_id in self.parents:
-                try:
-                    revtrees.append(self.repository.revision_tree(revision_id))
-                except errors.NoSuchRevision:
-                    if not revtrees:
-                        basis_revision_id = _mod_revision.NULL_REVISION
-                        ghost_basis = True
-                    revtrees.append(self.repository.revision_tree(
-                        _mod_revision.NULL_REVISION))
-        # The basis inventory from a repository 
-        if revtrees:
-            basis_inv = revtrees[0].inventory
-        else:
-            basis_inv = self.repository.revision_tree(
-                _mod_revision.NULL_REVISION).inventory
-        if len(self.parents) > 0:
-            if basis_revision_id != self.parents[0] and not ghost_basis:
-                raise Exception(
-                    "arbitrary basis parents not yet supported with merges")
-            for revtree in revtrees[1:]:
-                for change in revtree.inventory._make_delta(basis_inv):
-                    if change[1] is None:
-                        # Not present in this parent.
-                        continue
-                    if change[2] not in merged_ids:
-                        if change[0] is not None:
-                            basis_entry = basis_inv[change[2]]
-                            merged_ids[change[2]] = [
-                                # basis revid
-                                basis_entry.revision,
-                                # new tree revid
-                                change[3].revision]
-                            parent_entries[change[2]] = {
-                                # basis parent
-                                basis_entry.revision:basis_entry,
-                                # this parent 
-                                change[3].revision:change[3],
-                                }
-                        else:
-                            merged_ids[change[2]] = [change[3].revision]
-                            parent_entries[change[2]] = {change[3].revision:change[3]}
-                    else:
-                        merged_ids[change[2]].append(change[3].revision)
-                        parent_entries[change[2]][change[3].revision] = change[3]
-        else:
-            merged_ids = {}
-        # Setup the changes from the tree:
-        # changes maps file_id -> (change, [parent revision_ids])
-        changes= {}
-        for change in iter_changes:
-            # This probably looks up in basis_inv way to much.
-            if change[1][0] is not None:
-                head_candidate = [basis_inv[change[0]].revision]
-            else:
-                head_candidate = []
-            changes[change[0]] = change, merged_ids.get(change[0],
-                head_candidate)
-        unchanged_merged = set(merged_ids) - set(changes)
-        # Extend the changes dict with synthetic changes to record merges of
-        # texts.
-        for file_id in unchanged_merged:
-            # Record a merged version of these items that did not change vs the
-            # basis. This can be either identical parallel changes, or a revert
-            # of a specific file after a merge. The recorded content will be
-            # that of the current tree (which is the same as the basis), but
-            # the per-file graph will reflect a merge.
-            # NB:XXX: We are reconstructing path information we had, this
-            # should be preserved instead.
-            # inv delta  change: (file_id, (path_in_source, path_in_target),
-            #   changed_content, versioned, parent, name, kind,
-            #   executable)
-            try:
-                basis_entry = basis_inv[file_id]
-            except errors.NoSuchId:
-                # a change from basis->some_parents but file_id isn't in basis
-                # so was new in the merge, which means it must have changed
-                # from basis -> current, and as it hasn't the add was reverted
-                # by the user. So we discard this change.
-                pass
-            else:
-                change = (file_id,
-                    (basis_inv.id2path(file_id), tree.id2path(file_id)),
-                    False, (True, True),
-                    (basis_entry.parent_id, basis_entry.parent_id),
-                    (basis_entry.name, basis_entry.name),
-                    (basis_entry.kind, basis_entry.kind),
-                    (basis_entry.executable, basis_entry.executable))
-                changes[file_id] = (change, merged_ids[file_id])
-        # changes contains tuples with the change and a set of inventory
-        # candidates for the file.
-        # inv delta is:
-        # old_path, new_path, file_id, new_inventory_entry
-        seen_root = False # Is the root in the basis delta?
-        inv_delta = self._basis_delta
-        modified_rev = self._new_revision_id
-        for change, head_candidates in changes.values():
-            if change[3][1]: # versioned in target.
-                # Several things may be happening here:
-                # We may have a fork in the per-file graph
-                #  - record a change with the content from tree
-                # We may have a change against < all trees  
-                #  - carry over the tree that hasn't changed
-                # We may have a change against all trees
-                #  - record the change with the content from tree
-                kind = change[6][1]
-                file_id = change[0]
-                entry = _entry_factory[kind](file_id, change[5][1],
-                    change[4][1])
-                head_set = self._heads(change[0], set(head_candidates))
-                heads = []
-                # Preserve ordering.
-                for head_candidate in head_candidates:
-                    if head_candidate in head_set:
-                        heads.append(head_candidate)
-                        head_set.remove(head_candidate)
-                carried_over = False
-                if len(heads) == 1:
-                    # Could be a carry-over situation:
-                    parent_entry_revs = parent_entries.get(file_id, None)
-                    if parent_entry_revs:
-                        parent_entry = parent_entry_revs.get(heads[0], None)
-                    else:
-                        parent_entry = None
-                    if parent_entry is None:
-                        # The parent iter_changes was called against is the one
-                        # that is the per-file head, so any change is relevant
-                        # iter_changes is valid.
-                        carry_over_possible = False
-                    else:
-                        # could be a carry over situation
-                        # A change against the basis may just indicate a merge,
-                        # we need to check the content against the source of the
-                        # merge to determine if it was changed after the merge
-                        # or carried over.
-                        if (parent_entry.kind != entry.kind or
-                            parent_entry.parent_id != entry.parent_id or
-                            parent_entry.name != entry.name):
-                            # Metadata common to all entries has changed
-                            # against per-file parent
-                            carry_over_possible = False
-                        else:
-                            carry_over_possible = True
-                        # per-type checks for changes against the parent_entry
-                        # are done below.
-                else:
-                    # Cannot be a carry-over situation
-                    carry_over_possible = False
-                # Populate the entry in the delta
-                if kind == 'file':
-                    # XXX: There is still a small race here: If someone reverts the content of a file
-                    # after iter_changes examines and decides it has changed,
-                    # we will unconditionally record a new version even if some
-                    # other process reverts it while commit is running (with
-                    # the revert happening after iter_changes did it's
-                    # examination).
-                    if change[7][1]:
-                        entry.executable = True
-                    else:
-                        entry.executable = False
-                    if (carry_over_possible and
-                        parent_entry.executable == entry.executable):
-                            # Check the file length, content hash after reading
-                            # the file.
-                            nostore_sha = parent_entry.text_sha1
-                    else:
-                        nostore_sha = None
-                    file_obj, stat_value = tree.get_file_with_stat(file_id, change[1][1])
-                    try:
-                        text = file_obj.read()
-                    finally:
-                        file_obj.close()
-                    try:
-                        entry.text_sha1, entry.text_size = self._add_text_to_weave(
-                            file_id, text, heads, nostore_sha)
-                        yield file_id, change[1][1], (entry.text_sha1, stat_value)
-                    except errors.ExistingContent:
-                        # No content change against a carry_over parent
-                        # Perhaps this should also yield a fs hash update?
-                        carried_over = True
-                        entry.text_size = parent_entry.text_size
-                        entry.text_sha1 = parent_entry.text_sha1
-                elif kind == 'symlink':
-                    # Wants a path hint?
-                    entry.symlink_target = tree.get_symlink_target(file_id)
-                    if (carry_over_possible and
-                        parent_entry.symlink_target == entry.symlink_target):
-                        carried_over = True
-                    else:
-                        self._add_text_to_weave(change[0], '', heads, None)
-                elif kind == 'directory':
-                    if carry_over_possible:
-                        carried_over = True
-                    else:
-                        # Nothing to set on the entry.
-                        # XXX: split into the Root and nonRoot versions.
-                        if change[1][1] != '' or self.repository.supports_rich_root():
-                            self._add_text_to_weave(change[0], '', heads, None)
-                elif kind == 'tree-reference':
-                    if not self.repository._format.supports_tree_reference:
-                        # This isn't quite sane as an error, but we shouldn't
-                        # ever see this code path in practice: tree's don't
-                        # permit references when the repo doesn't support tree
-                        # references.
-                        raise errors.UnsupportedOperation(tree.add_reference,
-                            self.repository)
-                    reference_revision = tree.get_reference_revision(change[0])
-                    entry.reference_revision = reference_revision
-                    if (carry_over_possible and
-                        parent_entry.reference_revision == reference_revision):
-                        carried_over = True
-                    else:
-                        self._add_text_to_weave(change[0], '', heads, None)
-                else:
-                    raise AssertionError('unknown kind %r' % kind)
-                if not carried_over:
-                    entry.revision = modified_rev
-                else:
-                    entry.revision = parent_entry.revision
-            else:
-                entry = None
-            new_path = change[1][1]
-            inv_delta.append((change[1][0], new_path, change[0], entry))
-            if new_path == '':
-                seen_root = True
-        self.new_inventory = None
-        if len(inv_delta):
-            # This should perhaps be guarded by a check that the basis we
-            # commit against is the basis for the commit and if not do a delta
-            # against the basis.
-            self._any_changes = True
-        if not seen_root:
-            # housekeeping root entry changes do not affect no-change commits.
-            self._require_root_change(tree)
-        self.basis_delta_revision = basis_revision_id
-
-    def _add_text_to_weave(self, file_id, new_text, parents, nostore_sha):
-        parent_keys = tuple([(file_id, parent) for parent in parents])
-        return self.repository.texts._add_text(
-            (file_id, self._new_revision_id), parent_keys, new_text,
-            nostore_sha=nostore_sha, random_id=self.random_revid)[0:2]
-
-
-class RootCommitBuilder(CommitBuilder):
-    """This commitbuilder actually records the root id"""
-
-    # the root entry gets versioned properly by this builder.
-    _versioned_root = True
-
-    def _check_root(self, ie, parent_invs, tree):
-        """Helper for record_entry_contents.
-
-        :param ie: An entry being added.
-        :param parent_invs: The inventories of the parent revisions of the
-            commit.
-        :param tree: The tree that is being committed.
-        """
-
-    def _require_root_change(self, tree):
-        """Enforce an appropriate root object change.
-
-        This is called once when record_iter_changes is called, if and only if
-        the root was not in the delta calculated by record_iter_changes.
-
-        :param tree: The tree which is being committed.
-        """
-        # versioned roots do not change unless the tree found a change.
+        raise NotImplementedError(self.record_iter_changes)
 
 
 class RepositoryWriteLockResult(LogicalLockResult):
@@ -892,70 +243,17 @@
 # Repositories
 
 
-class Repository(_RelockDebugMixin, bzrdir.ControlComponent):
+class Repository(_RelockDebugMixin, controldir.ControlComponent):
     """Repository holding history for one or more branches.
 
     The repository holds and retrieves historical information including
     revisions and file history.  It's normally accessed only by the Branch,
     which views a particular line of development through that history.
 
-    The Repository builds on top of some byte storage facilies (the revisions,
-    signatures, inventories, texts and chk_bytes attributes) and a Transport,
-    which respectively provide byte storage and a means to access the (possibly
-    remote) disk.
-
-    The byte storage facilities are addressed via tuples, which we refer to
-    as 'keys' throughout the code base. Revision_keys, inventory_keys and
-    signature_keys are all 1-tuples: (revision_id,). text_keys are two-tuples:
-    (file_id, revision_id). chk_bytes uses CHK keys - a 1-tuple with a single
-    byte string made up of a hash identifier and a hash value.
-    We use this interface because it allows low friction with the underlying
-    code that implements disk indices, network encoding and other parts of
-    bzrlib.
-
-    :ivar revisions: A bzrlib.versionedfile.VersionedFiles instance containing
-        the serialised revisions for the repository. This can be used to obtain
-        revision graph information or to access raw serialised revisions.
-        The result of trying to insert data into the repository via this store
-        is undefined: it should be considered read-only except for implementors
-        of repositories.
-    :ivar signatures: A bzrlib.versionedfile.VersionedFiles instance containing
-        the serialised signatures for the repository. This can be used to
-        obtain access to raw serialised signatures.  The result of trying to
-        insert data into the repository via this store is undefined: it should
-        be considered read-only except for implementors of repositories.
-    :ivar inventories: A bzrlib.versionedfile.VersionedFiles instance containing
-        the serialised inventories for the repository. This can be used to
-        obtain unserialised inventories.  The result of trying to insert data
-        into the repository via this store is undefined: it should be
-        considered read-only except for implementors of repositories.
-    :ivar texts: A bzrlib.versionedfile.VersionedFiles instance containing the
-        texts of files and directories for the repository. This can be used to
-        obtain file texts or file graphs. Note that Repository.iter_file_bytes
-        is usually a better interface for accessing file texts.
-        The result of trying to insert data into the repository via this store
-        is undefined: it should be considered read-only except for implementors
-        of repositories.
-    :ivar chk_bytes: A bzrlib.versionedfile.VersionedFiles instance containing
-        any data the repository chooses to store or have indexed by its hash.
-        The result of trying to insert data into the repository via this store
-        is undefined: it should be considered read-only except for implementors
-        of repositories.
-    :ivar _transport: Transport for file access to repository, typically
-        pointing to .bzr/repository.
+    See VersionedFileRepository in bzrlib.vf_repository for the
+    base class for most Bazaar repositories.
     """
 
-    # What class to use for a CommitBuilder. Often its simpler to change this
-    # in a Repository class subclass rather than to override
-    # get_commit_builder.
-    _commit_builder_class = CommitBuilder
-    # The search regex used by xml based repositories to determine what things
-    # where changed in a single commit.
-    _file_ids_altered_regex = lazy_regex.lazy_compile(
-        r'file_id="(?P<file_id>[^"]+)"'
-        r'.* revision="(?P<revision_id>[^"]+)"'
-        )
-
     def abort_write_group(self, suppress_errors=False):
         """Commit the contents accrued within the current write group.
 
@@ -983,7 +281,7 @@
                 raise
             mutter('abort_write_group failed')
             log_exception_quietly()
-            note('bzr: ERROR (ignored): %s', exc)
+            note(gettext('bzr: ERROR (ignored): %s'), exc)
         self._write_group = None
 
     def _abort_write_group(self):
@@ -1004,20 +302,7 @@
 
         :param repository: A repository.
         """
-        if not self._format.supports_external_lookups:
-            raise errors.UnstackableRepositoryFormat(self._format, self.base)
-        if self.is_locked():
-            # This repository will call fallback.unlock() when we transition to
-            # the unlocked state, so we make sure to increment the lock count
-            repository.lock_read()
-        self._check_fallback_repository(repository)
-        self._fallback_repositories.append(repository)
-        self.texts.add_fallback_versioned_files(repository.texts)
-        self.inventories.add_fallback_versioned_files(repository.inventories)
-        self.revisions.add_fallback_versioned_files(repository.revisions)
-        self.signatures.add_fallback_versioned_files(repository.signatures)
-        if self.chk_bytes is not None:
-            self.chk_bytes.add_fallback_versioned_files(repository.chk_bytes)
+        raise NotImplementedError(self.add_fallback_repository)
 
     def _check_fallback_repository(self, repository):
         """Check that this repository can fallback to repository safely.
@@ -1028,134 +313,6 @@
         """
         return InterRepository._assert_same_model(self, repository)
 
-    def add_inventory(self, revision_id, inv, parents):
-        """Add the inventory inv to the repository as revision_id.
-
-        :param parents: The revision ids of the parents that revision_id
-                        is known to have and are in the repository already.
-
-        :returns: The validator(which is a sha1 digest, though what is sha'd is
-            repository format specific) of the serialized inventory.
-        """
-        if not self.is_in_write_group():
-            raise AssertionError("%r not in write group" % (self,))
-        _mod_revision.check_not_reserved_id(revision_id)
-        if not (inv.revision_id is None or inv.revision_id == revision_id):
-            raise AssertionError(
-                "Mismatch between inventory revision"
-                " id and insertion revid (%r, %r)"
-                % (inv.revision_id, revision_id))
-        if inv.root is None:
-            raise errors.RootMissing()
-        return self._add_inventory_checked(revision_id, inv, parents)
-
-    def _add_inventory_checked(self, revision_id, inv, parents):
-        """Add inv to the repository after checking the inputs.
-
-        This function can be overridden to allow different inventory styles.
-
-        :seealso: add_inventory, for the contract.
-        """
-        inv_lines = self._serializer.write_inventory_to_lines(inv)
-        return self._inventory_add_lines(revision_id, parents,
-            inv_lines, check_content=False)
-
-    def add_inventory_by_delta(self, basis_revision_id, delta, new_revision_id,
-                               parents, basis_inv=None, propagate_caches=False):
-        """Add a new inventory expressed as a delta against another revision.
-
-        See the inventory developers documentation for the theory behind
-        inventory deltas.
-
-        :param basis_revision_id: The inventory id the delta was created
-            against. (This does not have to be a direct parent.)
-        :param delta: The inventory delta (see Inventory.apply_delta for
-            details).
-        :param new_revision_id: The revision id that the inventory is being
-            added for.
-        :param parents: The revision ids of the parents that revision_id is
-            known to have and are in the repository already. These are supplied
-            for repositories that depend on the inventory graph for revision
-            graph access, as well as for those that pun ancestry with delta
-            compression.
-        :param basis_inv: The basis inventory if it is already known,
-            otherwise None.
-        :param propagate_caches: If True, the caches for this inventory are
-          copied to and updated for the result if possible.
-
-        :returns: (validator, new_inv)
-            The validator(which is a sha1 digest, though what is sha'd is
-            repository format specific) of the serialized inventory, and the
-            resulting inventory.
-        """
-        if not self.is_in_write_group():
-            raise AssertionError("%r not in write group" % (self,))
-        _mod_revision.check_not_reserved_id(new_revision_id)
-        basis_tree = self.revision_tree(basis_revision_id)
-        basis_tree.lock_read()
-        try:
-            # Note that this mutates the inventory of basis_tree, which not all
-            # inventory implementations may support: A better idiom would be to
-            # return a new inventory, but as there is no revision tree cache in
-            # repository this is safe for now - RBC 20081013
-            if basis_inv is None:
-                basis_inv = basis_tree.inventory
-            basis_inv.apply_delta(delta)
-            basis_inv.revision_id = new_revision_id
-            return (self.add_inventory(new_revision_id, basis_inv, parents),
-                    basis_inv)
-        finally:
-            basis_tree.unlock()
-
-    def _inventory_add_lines(self, revision_id, parents, lines,
-        check_content=True):
-        """Store lines in inv_vf and return the sha1 of the inventory."""
-        parents = [(parent,) for parent in parents]
-        result = self.inventories.add_lines((revision_id,), parents, lines,
-            check_content=check_content)[0]
-        self.inventories._access.flush()
-        return result
-
-    def add_revision(self, revision_id, rev, inv=None, config=None):
-        """Add rev to the revision store as revision_id.
-
-        :param revision_id: the revision id to use.
-        :param rev: The revision object.
-        :param inv: The inventory for the revision. if None, it will be looked
-                    up in the inventory storer
-        :param config: If None no digital signature will be created.
-                       If supplied its signature_needed method will be used
-                       to determine if a signature should be made.
-        """
-        # TODO: jam 20070210 Shouldn't we check rev.revision_id and
-        #       rev.parent_ids?
-        _mod_revision.check_not_reserved_id(revision_id)
-        if config is not None and config.signature_needed():
-            if inv is None:
-                inv = self.get_inventory(revision_id)
-            plaintext = Testament(rev, inv).as_short_text()
-            self.store_revision_signature(
-                gpg.GPGStrategy(config), plaintext, revision_id)
-        # check inventory present
-        if not self.inventories.get_parent_map([(revision_id,)]):
-            if inv is None:
-                raise errors.WeaveRevisionNotPresent(revision_id,
-                                                     self.inventories)
-            else:
-                # yes, this is not suitable for adding with ghosts.
-                rev.inventory_sha1 = self.add_inventory(revision_id, inv,
-                                                        rev.parent_ids)
-        else:
-            key = (revision_id,)
-            rev.inventory_sha1 = self.inventories.get_sha1s([key])[key]
-        self._add_revision(rev)
-
-    def _add_revision(self, revision):
-        text = self._serializer.write_revision_to_string(revision)
-        key = (revision.revision_id,)
-        parents = tuple((parent,) for parent in revision.parent_ids)
-        self.revisions.add_lines(key, parents, osutils.split_lines(text))
-
     def all_revision_ids(self):
         """Returns a list of all the revision ids in the repository.
 
@@ -1184,141 +341,17 @@
         """
         self.control_files.break_lock()
 
-    @needs_read_lock
-    def _eliminate_revisions_not_present(self, revision_ids):
-        """Check every revision id in revision_ids to see if we have it.
-
-        Returns a set of the present revisions.
-        """
-        result = []
-        graph = self.get_graph()
-        parent_map = graph.get_parent_map(revision_ids)
-        # The old API returned a list, should this actually be a set?
-        return parent_map.keys()
-
-    def _check_inventories(self, checker):
-        """Check the inventories found from the revision scan.
-        
-        This is responsible for verifying the sha1 of inventories and
-        creating a pending_keys set that covers data referenced by inventories.
-        """
-        bar = ui.ui_factory.nested_progress_bar()
-        try:
-            self._do_check_inventories(checker, bar)
-        finally:
-            bar.finished()
-
-    def _do_check_inventories(self, checker, bar):
-        """Helper for _check_inventories."""
-        revno = 0
-        keys = {'chk_bytes':set(), 'inventories':set(), 'texts':set()}
-        kinds = ['chk_bytes', 'texts']
-        count = len(checker.pending_keys)
-        bar.update("inventories", 0, 2)
-        current_keys = checker.pending_keys
-        checker.pending_keys = {}
-        # Accumulate current checks.
-        for key in current_keys:
-            if key[0] != 'inventories' and key[0] not in kinds:
-                checker._report_items.append('unknown key type %r' % (key,))
-            keys[key[0]].add(key[1:])
-        if keys['inventories']:
-            # NB: output order *should* be roughly sorted - topo or
-            # inverse topo depending on repository - either way decent
-            # to just delta against. However, pre-CHK formats didn't
-            # try to optimise inventory layout on disk. As such the
-            # pre-CHK code path does not use inventory deltas.
-            last_object = None
-            for record in self.inventories.check(keys=keys['inventories']):
-                if record.storage_kind == 'absent':
-                    checker._report_items.append(
-                        'Missing inventory {%s}' % (record.key,))
-                else:
-                    last_object = self._check_record('inventories', record,
-                        checker, last_object,
-                        current_keys[('inventories',) + record.key])
-            del keys['inventories']
-        else:
-            return
-        bar.update("texts", 1)
-        while (checker.pending_keys or keys['chk_bytes']
-            or keys['texts']):
-            # Something to check.
-            current_keys = checker.pending_keys
-            checker.pending_keys = {}
-            # Accumulate current checks.
-            for key in current_keys:
-                if key[0] not in kinds:
-                    checker._report_items.append('unknown key type %r' % (key,))
-                keys[key[0]].add(key[1:])
-            # Check the outermost kind only - inventories || chk_bytes || texts
-            for kind in kinds:
-                if keys[kind]:
-                    last_object = None
-                    for record in getattr(self, kind).check(keys=keys[kind]):
-                        if record.storage_kind == 'absent':
-                            checker._report_items.append(
-                                'Missing %s {%s}' % (kind, record.key,))
-                        else:
-                            last_object = self._check_record(kind, record,
-                                checker, last_object, current_keys[(kind,) + record.key])
-                    keys[kind] = set()
-                    break
-
-    def _check_record(self, kind, record, checker, last_object, item_data):
-        """Check a single text from this repository."""
-        if kind == 'inventories':
-            rev_id = record.key[0]
-            inv = self._deserialise_inventory(rev_id,
-                record.get_bytes_as('fulltext'))
-            if last_object is not None:
-                delta = inv._make_delta(last_object)
-                for old_path, path, file_id, ie in delta:
-                    if ie is None:
-                        continue
-                    ie.check(checker, rev_id, inv)
-            else:
-                for path, ie in inv.iter_entries():
-                    ie.check(checker, rev_id, inv)
-            if self._format.fast_deltas:
-                return inv
-        elif kind == 'chk_bytes':
-            # No code written to check chk_bytes for this repo format.
-            checker._report_items.append(
-                'unsupported key type chk_bytes for %s' % (record.key,))
-        elif kind == 'texts':
-            self._check_text(record, checker, item_data)
-        else:
-            checker._report_items.append(
-                'unknown key type %s for %s' % (kind, record.key))
-
-    def _check_text(self, record, checker, item_data):
-        """Check a single text."""
-        # Check it is extractable.
-        # TODO: check length.
-        if record.storage_kind == 'chunked':
-            chunks = record.get_bytes_as(record.storage_kind)
-            sha1 = osutils.sha_strings(chunks)
-            length = sum(map(len, chunks))
-        else:
-            content = record.get_bytes_as('fulltext')
-            sha1 = osutils.sha_string(content)
-            length = len(content)
-        if item_data and sha1 != item_data[1]:
-            checker._report_items.append(
-                'sha1 mismatch: %s has sha1 %s expected %s referenced by %s' %
-                (record.key, sha1, item_data[1], item_data[2]))
-
     @staticmethod
-    def create(a_bzrdir):
-        """Construct the current default format repository in a_bzrdir."""
-        return RepositoryFormat.get_default_format().initialize(a_bzrdir)
+    def create(controldir):
+        """Construct the current default format repository in controldir."""
+        return RepositoryFormat.get_default_format().initialize(controldir)
 
-    def __init__(self, _format, a_bzrdir, control_files):
+    def __init__(self, _format, controldir, control_files):
         """instantiate a Repository.
 
         :param _format: The format of the repository on disk.
-        :param a_bzrdir: The BzrDir of the repository.
+        :param controldir: The ControlDir of the repository.
+        :param control_files: Control files to use for locking, etc.
         """
         # In the future we will have a single api for all stores for
         # getting file texts, inventories and revisions, then
@@ -1326,22 +359,12 @@
         super(Repository, self).__init__()
         self._format = _format
         # the following are part of the public API for Repository:
-        self.bzrdir = a_bzrdir
+        self.bzrdir = controldir
         self.control_files = control_files
-        self._transport = control_files._transport
-        self.base = self._transport.base
         # for tests
-        self._reconcile_does_inventory_gc = True
-        self._reconcile_fixes_text_parents = False
-        self._reconcile_backsup_inventory = True
         self._write_group = None
         # Additional places to query for data.
         self._fallback_repositories = []
-        # An InventoryEntry cache, used during deserialization
-        self._inventory_entry_cache = fifo_cache.FIFOCache(10*1024)
-        # Is it safe to return inventory entries directly from the entry cache,
-        # rather copying them?
-        self._safe_to_return_from_cache = False
 
     @property
     def user_transport(self):
@@ -1381,7 +404,7 @@
         """
         if self.__class__ is not other.__class__:
             return False
-        return (self._transport.base == other._transport.base)
+        return (self.control_url == other.control_url)
 
     def is_in_write_group(self):
         """Return True if there is an open write group.
@@ -1487,36 +510,27 @@
         if revid and committers:
             result['committers'] = 0
         if revid and revid != _mod_revision.NULL_REVISION:
+            graph = self.get_graph()
             if committers:
                 all_committers = set()
-            revisions = self.get_ancestry(revid)
-            # pop the leading None
-            revisions.pop(0)
-            first_revision = None
+            revisions = [r for (r, p) in graph.iter_ancestry([revid])
+                        if r != _mod_revision.NULL_REVISION]
+            last_revision = None
             if not committers:
                 # ignore the revisions in the middle - just grab first and last
                 revisions = revisions[0], revisions[-1]
             for revision in self.get_revisions(revisions):
-                if not first_revision:
-                    first_revision = revision
+                if not last_revision:
+                    last_revision = revision
                 if committers:
                     all_committers.add(revision.committer)
-            last_revision = revision
+            first_revision = revision
             if committers:
                 result['committers'] = len(all_committers)
             result['firstrev'] = (first_revision.timestamp,
                 first_revision.timezone)
             result['latestrev'] = (last_revision.timestamp,
                 last_revision.timezone)
-
-        # now gather global repository information
-        # XXX: This is available for many repos regardless of listability.
-        if self.user_transport.listable():
-            # XXX: do we want to __define len__() ?
-            # Maybe the versionedfiles object should provide a different
-            # method to get the number of keys.
-            result['revisions'] = len(self.revisions.keys())
-            # result['size'] = t
         return result
 
     def find_branches(self, using=False):
@@ -1533,22 +547,22 @@
             def __init__(self):
                 self.first_call = True
 
-            def __call__(self, bzrdir):
-                # On the first call, the parameter is always the bzrdir
+            def __call__(self, controldir):
+                # On the first call, the parameter is always the controldir
                 # containing the current repo.
                 if not self.first_call:
                     try:
-                        repository = bzrdir.open_repository()
+                        repository = controldir.open_repository()
                     except errors.NoRepositoryPresent:
                         pass
                     else:
                         return False, ([], repository)
                 self.first_call = False
-                value = (bzrdir.list_branches(), None)
+                value = (controldir.list_branches(), None)
                 return True, value
 
         ret = []
-        for branches, repository in bzrdir.BzrDir.find_bzrdirs(
+        for branches, repository in controldir.ControlDir.find_bzrdirs(
                 self.user_transport, evaluate=Evaluator()):
             if branches is not None:
                 ret.extend(branches)
@@ -1557,15 +571,29 @@
         return ret
 
     @needs_read_lock
-    def search_missing_revision_ids(self, other, revision_id=None, find_ghosts=True):
+    def search_missing_revision_ids(self, other,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None,
+            limit=None):
         """Return the revision ids that other has that this does not.
 
         These are returned in topological order.
 
         revision_id: only return revision ids included by revision_id.
         """
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.4.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=3)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
         return InterRepository.get(other, self).search_missing_revision_ids(
-            revision_id, find_ghosts)
+            find_ghosts=find_ghosts, revision_ids=revision_ids,
+            if_present_ids=if_present_ids, limit=limit)
 
     @staticmethod
     def open(base):
@@ -1574,7 +602,7 @@
         For instance, if the repository is at URL/.bzr/repository,
         Repository.open(URL) -> a Repository instance.
         """
-        control = bzrdir.BzrDir.open(base)
+        control = controldir.ControlDir.open(base)
         return control.open_repository()
 
     def copy_content_into(self, destination, revision_id=None):
@@ -1613,61 +641,6 @@
     def suspend_write_group(self):
         raise errors.UnsuspendableWriteGroup(self)
 
-    def get_missing_parent_inventories(self, check_for_missing_texts=True):
-        """Return the keys of missing inventory parents for revisions added in
-        this write group.
-
-        A revision is not complete if the inventory delta for that revision
-        cannot be calculated.  Therefore if the parent inventories of a
-        revision are not present, the revision is incomplete, and e.g. cannot
-        be streamed by a smart server.  This method finds missing inventory
-        parents for revisions added in this write group.
-        """
-        if not self._format.supports_external_lookups:
-            # This is only an issue for stacked repositories
-            return set()
-        if not self.is_in_write_group():
-            raise AssertionError('not in a write group')
-
-        # XXX: We assume that every added revision already has its
-        # corresponding inventory, so we only check for parent inventories that
-        # might be missing, rather than all inventories.
-        parents = set(self.revisions._index.get_missing_parents())
-        parents.discard(_mod_revision.NULL_REVISION)
-        unstacked_inventories = self.inventories._index
-        present_inventories = unstacked_inventories.get_parent_map(
-            key[-1:] for key in parents)
-        parents.difference_update(present_inventories)
-        if len(parents) == 0:
-            # No missing parent inventories.
-            return set()
-        if not check_for_missing_texts:
-            return set(('inventories', rev_id) for (rev_id,) in parents)
-        # Ok, now we have a list of missing inventories.  But these only matter
-        # if the inventories that reference them are missing some texts they
-        # appear to introduce.
-        # XXX: Texts referenced by all added inventories need to be present,
-        # but at the moment we're only checking for texts referenced by
-        # inventories at the graph's edge.
-        key_deps = self.revisions._index._key_dependencies
-        key_deps.satisfy_refs_for_keys(present_inventories)
-        referrers = frozenset(r[0] for r in key_deps.get_referrers())
-        file_ids = self.fileids_altered_by_revision_ids(referrers)
-        missing_texts = set()
-        for file_id, version_ids in file_ids.iteritems():
-            missing_texts.update(
-                (file_id, version_id) for version_id in version_ids)
-        present_texts = self.texts.get_parent_map(missing_texts)
-        missing_texts.difference_update(present_texts)
-        if not missing_texts:
-            # No texts are missing, so all revisions and their deltas are
-            # reconstructable.
-            return set()
-        # Alternatively the text versions could be returned as the missing
-        # keys, but this is likely to be less data.
-        missing_keys = set(('inventories', rev_id) for (rev_id,) in parents)
-        return missing_keys
-
     def refresh_data(self):
         """Re-read any data needed to synchronise with disk.
 
@@ -1693,7 +666,7 @@
     def _resume_write_group(self, tokens):
         raise errors.UnsuspendableWriteGroup(self)
 
-    def fetch(self, source, revision_id=None, pb=None, find_ghosts=False,
+    def fetch(self, source, revision_id=None, find_ghosts=False,
             fetch_spec=None):
         """Fetch the content required to construct revision_id from source.
 
@@ -1733,11 +706,8 @@
                 not _mod_revision.is_null(revision_id)):
                 self.get_revision(revision_id)
             return 0, []
-        # if there is no specific appropriate InterRepository, this will get
-        # the InterRepository base class, which raises an
-        # IncompatibleRepositories when asked to fetch.
         inter = InterRepository.get(source, self)
-        return inter.fetch(revision_id=revision_id, pb=pb,
+        return inter.fetch(revision_id=revision_id,
             find_ghosts=find_ghosts, fetch_spec=fetch_spec)
 
     def create_bundle(self, target, base, fileobj, format=None):
@@ -1745,7 +715,7 @@
 
     def get_commit_builder(self, branch, parents, config, timestamp=None,
                            timezone=None, committer=None, revprops=None,
-                           revision_id=None):
+                           revision_id=None, lossy=False):
         """Obtain a CommitBuilder for this repository.
 
         :param branch: Branch to commit to.
@@ -1756,15 +726,10 @@
         :param committer: Optional committer to set for commit.
         :param revprops: Optional dictionary of revision properties.
         :param revision_id: Optional revision id.
+        :param lossy: Whether to discard data that can not be natively
+            represented, when pushing to a foreign VCS
         """
-        if self._fallback_repositories:
-            raise errors.BzrError("Cannot commit from a lightweight checkout "
-                "to a stacked branch. See "
-                "https://bugs.launchpad.net/bzr/+bug/375013 for details.")
-        result = self._commit_builder_class(self, parents, config,
-            timestamp, timezone, committer, revprops, revision_id)
-        self.start_write_group()
-        return result
+        raise NotImplementedError(self.get_commit_builder)
 
     @only_raises(errors.LockNotHeld, errors.LockBroken)
     def unlock(self):
@@ -1777,13 +742,12 @@
                     'Must end write groups before releasing write locks.')
         self.control_files.unlock()
         if self.control_files._lock_count == 0:
-            self._inventory_entry_cache.clear()
             for repo in self._fallback_repositories:
                 repo.unlock()
 
     @needs_read_lock
-    def clone(self, a_bzrdir, revision_id=None):
-        """Clone this repository into a_bzrdir using the current format.
+    def clone(self, controldir, revision_id=None):
+        """Clone this repository into controldir using the current format.
 
         Currently no check is made that the format of this repository and
         the bzrdir format are compatible. FIXME RBC 20060201.
@@ -1792,7 +756,8 @@
         """
         # TODO: deprecate after 0.16; cloning this with all its settings is
         # probably not very useful -- mbp 20070423
-        dest_repo = self._create_sprouting_repo(a_bzrdir, shared=self.is_shared())
+        dest_repo = self._create_sprouting_repo(
+            controldir, shared=self.is_shared())
         self.copy_content_into(dest_repo, revision_id)
         return dest_repo
 
@@ -1848,14 +813,6 @@
                 dest_repo = a_bzrdir.open_repository()
         return dest_repo
 
-    def _get_sink(self):
-        """Return a sink for streaming into this repository."""
-        return StreamSink(self)
-
-    def _get_source(self, to_format):
-        """Return a source for streaming from this repository."""
-        return StreamSource(self, to_format)
-
     @needs_read_lock
     def has_revision(self, revision_id):
         """True if this repository has a copy of the revision."""
@@ -1868,20 +825,13 @@
         :param revision_ids: An iterable of revision_ids.
         :return: A set of the revision_ids that were present.
         """
-        parent_map = self.revisions.get_parent_map(
-            [(rev_id,) for rev_id in revision_ids])
-        result = set()
-        if _mod_revision.NULL_REVISION in revision_ids:
-            result.add(_mod_revision.NULL_REVISION)
-        result.update([key[0] for key in parent_map])
-        return result
+        raise NotImplementedError(self.has_revisions)
 
     @needs_read_lock
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision."""
         return self.get_revisions([revision_id])[0]
 
-    @needs_read_lock
     def get_revision_reconcile(self, revision_id):
         """'reconcile' helper routine that allows access to a revision always.
 
@@ -1890,53 +840,15 @@
         be used by reconcile, or reconcile-alike commands that are correcting
         or testing the revision graph.
         """
-        return self._get_revisions([revision_id])[0]
+        raise NotImplementedError(self.get_revision_reconcile)
 
-    @needs_read_lock
     def get_revisions(self, revision_ids):
         """Get many revisions at once.
         
         Repositories that need to check data on every revision read should 
         subclass this method.
         """
-        return self._get_revisions(revision_ids)
-
-    @needs_read_lock
-    def _get_revisions(self, revision_ids):
-        """Core work logic to get many revisions without sanity checks."""
-        revs = {}
-        for revid, rev in self._iter_revisions(revision_ids):
-            if rev is None:
-                raise errors.NoSuchRevision(self, revid)
-            revs[revid] = rev
-        return [revs[revid] for revid in revision_ids]
-
-    def _iter_revisions(self, revision_ids):
-        """Iterate over revision objects.
-
-        :param revision_ids: An iterable of revisions to examine. None may be
-            passed to request all revisions known to the repository. Note that
-            not all repositories can find unreferenced revisions; for those
-            repositories only referenced ones will be returned.
-        :return: An iterator of (revid, revision) tuples. Absent revisions (
-            those asked for but not available) are returned as (revid, None).
-        """
-        if revision_ids is None:
-            revision_ids = self.all_revision_ids()
-        else:
-            for rev_id in revision_ids:
-                if not rev_id or not isinstance(rev_id, basestring):
-                    raise errors.InvalidRevisionId(revision_id=rev_id, branch=self)
-        keys = [(key,) for key in revision_ids]
-        stream = self.revisions.get_record_stream(keys, 'unordered', True)
-        for record in stream:
-            revid = record.key[0]
-            if record.storage_kind == 'absent':
-                yield (revid, None)
-            else:
-                text = record.get_bytes_as('fulltext')
-                rev = self._serializer.read_revision_from_string(text)
-                yield (revid, rev)
+        raise NotImplementedError(self.get_revisions)
 
     def get_deltas_for_revisions(self, revisions, specific_fileids=None):
         """Produce a generator of revision deltas.
@@ -1997,154 +909,13 @@
         signature = gpg_strategy.sign(plaintext)
         self.add_signature_text(revision_id, signature)
 
-    @needs_write_lock
     def add_signature_text(self, revision_id, signature):
-        self.signatures.add_lines((revision_id,), (),
-            osutils.split_lines(signature))
-
-    def find_text_key_references(self):
-        """Find the text key references within the repository.
-
-        :return: A dictionary mapping text keys ((fileid, revision_id) tuples)
-            to whether they were referred to by the inventory of the
-            revision_id that they contain. The inventory texts from all present
-            revision ids are assessed to generate this report.
-        """
-        revision_keys = self.revisions.keys()
-        w = self.inventories
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            return self._find_text_key_references_from_xml_inventory_lines(
-                w.iter_lines_added_or_present_in_keys(revision_keys, pb=pb))
-        finally:
-            pb.finished()
-
-    def _find_text_key_references_from_xml_inventory_lines(self,
-        line_iterator):
-        """Core routine for extracting references to texts from inventories.
-
-        This performs the translation of xml lines to revision ids.
-
-        :param line_iterator: An iterator of lines, origin_version_id
-        :return: A dictionary mapping text keys ((fileid, revision_id) tuples)
-            to whether they were referred to by the inventory of the
-            revision_id that they contain. Note that if that revision_id was
-            not part of the line_iterator's output then False will be given -
-            even though it may actually refer to that key.
-        """
-        if not self._serializer.support_altered_by_hack:
-            raise AssertionError(
-                "_find_text_key_references_from_xml_inventory_lines only "
-                "supported for branches which store inventory as unnested xml"
-                ", not on %r" % self)
-        result = {}
-
-        # this code needs to read every new line in every inventory for the
-        # inventories [revision_ids]. Seeing a line twice is ok. Seeing a line
-        # not present in one of those inventories is unnecessary but not
-        # harmful because we are filtering by the revision id marker in the
-        # inventory lines : we only select file ids altered in one of those
-        # revisions. We don't need to see all lines in the inventory because
-        # only those added in an inventory in rev X can contain a revision=X
-        # line.
-        unescape_revid_cache = {}
-        unescape_fileid_cache = {}
-
-        # jam 20061218 In a big fetch, this handles hundreds of thousands
-        # of lines, so it has had a lot of inlining and optimizing done.
-        # Sorry that it is a little bit messy.
-        # Move several functions to be local variables, since this is a long
-        # running loop.
-        search = self._file_ids_altered_regex.search
-        unescape = _unescape_xml
-        setdefault = result.setdefault
-        for line, line_key in line_iterator:
-            match = search(line)
-            if match is None:
-                continue
-            # One call to match.group() returning multiple items is quite a
-            # bit faster than 2 calls to match.group() each returning 1
-            file_id, revision_id = match.group('file_id', 'revision_id')
-
-            # Inlining the cache lookups helps a lot when you make 170,000
-            # lines and 350k ids, versus 8.4 unique ids.
-            # Using a cache helps in 2 ways:
-            #   1) Avoids unnecessary decoding calls
-            #   2) Re-uses cached strings, which helps in future set and
-            #      equality checks.
-            # (2) is enough that removing encoding entirely along with
-            # the cache (so we are using plain strings) results in no
-            # performance improvement.
-            try:
-                revision_id = unescape_revid_cache[revision_id]
-            except KeyError:
-                unescaped = unescape(revision_id)
-                unescape_revid_cache[revision_id] = unescaped
-                revision_id = unescaped
-
-            # Note that unconditionally unescaping means that we deserialise
-            # every fileid, which for general 'pull' is not great, but we don't
-            # really want to have some many fulltexts that this matters anyway.
-            # RBC 20071114.
-            try:
-                file_id = unescape_fileid_cache[file_id]
-            except KeyError:
-                unescaped = unescape(file_id)
-                unescape_fileid_cache[file_id] = unescaped
-                file_id = unescaped
-
-            key = (file_id, revision_id)
-            setdefault(key, False)
-            if revision_id == line_key[-1]:
-                result[key] = True
-        return result
-
-    def _inventory_xml_lines_for_keys(self, keys):
-        """Get a line iterator of the sort needed for findind references.
-
-        Not relevant for non-xml inventory repositories.
-
-        Ghosts in revision_keys are ignored.
-
-        :param revision_keys: The revision keys for the inventories to inspect.
-        :return: An iterator over (inventory line, revid) for the fulltexts of
-            all of the xml inventories specified by revision_keys.
-        """
-        stream = self.inventories.get_record_stream(keys, 'unordered', True)
-        for record in stream:
-            if record.storage_kind != 'absent':
-                chunks = record.get_bytes_as('chunked')
-                revid = record.key[-1]
-                lines = osutils.chunks_to_lines(chunks)
-                for line in lines:
-                    yield line, revid
-
-    def _find_file_ids_from_xml_inventory_lines(self, line_iterator,
-        revision_keys):
-        """Helper routine for fileids_altered_by_revision_ids.
-
-        This performs the translation of xml lines to revision ids.
-
-        :param line_iterator: An iterator of lines, origin_version_id
-        :param revision_keys: The revision ids to filter for. This should be a
-            set or other type which supports efficient __contains__ lookups, as
-            the revision key from each parsed line will be looked up in the
-            revision_keys filter.
-        :return: a dictionary mapping altered file-ids to an iterable of
-        revision_ids. Each altered file-ids has the exact revision_ids that
-        altered it listed explicitly.
-        """
-        seen = set(self._find_text_key_references_from_xml_inventory_lines(
-                line_iterator).iterkeys())
-        parent_keys = self._find_parent_keys_of_revisions(revision_keys)
-        parent_seen = set(self._find_text_key_references_from_xml_inventory_lines(
-            self._inventory_xml_lines_for_keys(parent_keys)))
-        new_keys = seen - parent_seen
-        result = {}
-        setdefault = result.setdefault
-        for key in new_keys:
-            setdefault(key[0], set()).add(key[-1])
-        return result
+        """Store a signature text for a revision.
+
+        :param revision_id: Revision id of the revision
+        :param signature: Signature text.
+        """
+        raise NotImplementedError(self.add_signature_text)
 
     def _find_parent_ids_of_revisions(self, revision_ids):
         """Find all parent ids that are mentioned in the revision graph.
@@ -2159,37 +930,6 @@
         parent_ids.discard(_mod_revision.NULL_REVISION)
         return parent_ids
 
-    def _find_parent_keys_of_revisions(self, revision_keys):
-        """Similar to _find_parent_ids_of_revisions, but used with keys.
-
-        :param revision_keys: An iterable of revision_keys.
-        :return: The parents of all revision_keys that are not already in
-            revision_keys
-        """
-        parent_map = self.revisions.get_parent_map(revision_keys)
-        parent_keys = set()
-        map(parent_keys.update, parent_map.itervalues())
-        parent_keys.difference_update(revision_keys)
-        parent_keys.discard(_mod_revision.NULL_REVISION)
-        return parent_keys
-
-    def fileids_altered_by_revision_ids(self, revision_ids, _inv_weave=None):
-        """Find the file ids and versions affected by revisions.
-
-        :param revisions: an iterable containing revision ids.
-        :param _inv_weave: The inventory weave from this repository or None.
-            If None, the inventory weave will be opened automatically.
-        :return: a dictionary mapping altered file-ids to an iterable of
-        revision_ids. Each altered file-ids has the exact revision_ids that
-        altered it listed explicitly.
-        """
-        selected_keys = set((revid,) for revid in revision_ids)
-        w = _inv_weave or self.inventories
-        return self._find_file_ids_from_xml_inventory_lines(
-            w.iter_lines_added_or_present_in_keys(
-                selected_keys, pb=None),
-            selected_keys)
-
     def iter_files_bytes(self, desired_files):
         """Iterate through file versions.
 
@@ -2201,273 +941,10 @@
         uniquely identify the file version in the caller's context.  (Examples:
         an index number or a TreeTransform trans_id.)
 
-        bytes_iterator is an iterable of bytestrings for the file.  The
-        kind of iterable and length of the bytestrings are unspecified, but for
-        this implementation, it is a list of bytes produced by
-        VersionedFile.get_record_stream().
-
         :param desired_files: a list of (file_id, revision_id, identifier)
             triples
         """
-        text_keys = {}
-        for file_id, revision_id, callable_data in desired_files:
-            text_keys[(file_id, revision_id)] = callable_data
-        for record in self.texts.get_record_stream(text_keys, 'unordered', True):
-            if record.storage_kind == 'absent':
-                raise errors.RevisionNotPresent(record.key, self)
-            yield text_keys[record.key], record.get_bytes_as('chunked')
-
-    def _generate_text_key_index(self, text_key_references=None,
-        ancestors=None):
-        """Generate a new text key index for the repository.
-
-        This is an expensive function that will take considerable time to run.
-
-        :return: A dict mapping text keys ((file_id, revision_id) tuples) to a
-            list of parents, also text keys. When a given key has no parents,
-            the parents list will be [NULL_REVISION].
-        """
-        # All revisions, to find inventory parents.
-        if ancestors is None:
-            graph = self.get_graph()
-            ancestors = graph.get_parent_map(self.all_revision_ids())
-        if text_key_references is None:
-            text_key_references = self.find_text_key_references()
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            return self._do_generate_text_key_index(ancestors,
-                text_key_references, pb)
-        finally:
-            pb.finished()
-
-    def _do_generate_text_key_index(self, ancestors, text_key_references, pb):
-        """Helper for _generate_text_key_index to avoid deep nesting."""
-        revision_order = tsort.topo_sort(ancestors)
-        invalid_keys = set()
-        revision_keys = {}
-        for revision_id in revision_order:
-            revision_keys[revision_id] = set()
-        text_count = len(text_key_references)
-        # a cache of the text keys to allow reuse; costs a dict of all the
-        # keys, but saves a 2-tuple for every child of a given key.
-        text_key_cache = {}
-        for text_key, valid in text_key_references.iteritems():
-            if not valid:
-                invalid_keys.add(text_key)
-            else:
-                revision_keys[text_key[1]].add(text_key)
-            text_key_cache[text_key] = text_key
-        del text_key_references
-        text_index = {}
-        text_graph = graph.Graph(graph.DictParentsProvider(text_index))
-        NULL_REVISION = _mod_revision.NULL_REVISION
-        # Set a cache with a size of 10 - this suffices for bzr.dev but may be
-        # too small for large or very branchy trees. However, for 55K path
-        # trees, it would be easy to use too much memory trivially. Ideally we
-        # could gauge this by looking at available real memory etc, but this is
-        # always a tricky proposition.
-        inventory_cache = lru_cache.LRUCache(10)
-        batch_size = 10 # should be ~150MB on a 55K path tree
-        batch_count = len(revision_order) / batch_size + 1
-        processed_texts = 0
-        pb.update("Calculating text parents", processed_texts, text_count)
-        for offset in xrange(batch_count):
-            to_query = revision_order[offset * batch_size:(offset + 1) *
-                batch_size]
-            if not to_query:
-                break
-            for revision_id in to_query:
-                parent_ids = ancestors[revision_id]
-                for text_key in revision_keys[revision_id]:
-                    pb.update("Calculating text parents", processed_texts)
-                    processed_texts += 1
-                    candidate_parents = []
-                    for parent_id in parent_ids:
-                        parent_text_key = (text_key[0], parent_id)
-                        try:
-                            check_parent = parent_text_key not in \
-                                revision_keys[parent_id]
-                        except KeyError:
-                            # the parent parent_id is a ghost:
-                            check_parent = False
-                            # truncate the derived graph against this ghost.
-                            parent_text_key = None
-                        if check_parent:
-                            # look at the parent commit details inventories to
-                            # determine possible candidates in the per file graph.
-                            # TODO: cache here.
-                            try:
-                                inv = inventory_cache[parent_id]
-                            except KeyError:
-                                inv = self.revision_tree(parent_id).inventory
-                                inventory_cache[parent_id] = inv
-                            try:
-                                parent_entry = inv[text_key[0]]
-                            except (KeyError, errors.NoSuchId):
-                                parent_entry = None
-                            if parent_entry is not None:
-                                parent_text_key = (
-                                    text_key[0], parent_entry.revision)
-                            else:
-                                parent_text_key = None
-                        if parent_text_key is not None:
-                            candidate_parents.append(
-                                text_key_cache[parent_text_key])
-                    parent_heads = text_graph.heads(candidate_parents)
-                    new_parents = list(parent_heads)
-                    new_parents.sort(key=lambda x:candidate_parents.index(x))
-                    if new_parents == []:
-                        new_parents = [NULL_REVISION]
-                    text_index[text_key] = new_parents
-
-        for text_key in invalid_keys:
-            text_index[text_key] = [NULL_REVISION]
-        return text_index
-
-    def item_keys_introduced_by(self, revision_ids, _files_pb=None):
-        """Get an iterable listing the keys of all the data introduced by a set
-        of revision IDs.
-
-        The keys will be ordered so that the corresponding items can be safely
-        fetched and inserted in that order.
-
-        :returns: An iterable producing tuples of (knit-kind, file-id,
-            versions).  knit-kind is one of 'file', 'inventory', 'signatures',
-            'revisions'.  file-id is None unless knit-kind is 'file'.
-        """
-        for result in self._find_file_keys_to_fetch(revision_ids, _files_pb):
-            yield result
-        del _files_pb
-        for result in self._find_non_file_keys_to_fetch(revision_ids):
-            yield result
-
-    def _find_file_keys_to_fetch(self, revision_ids, pb):
-        # XXX: it's a bit weird to control the inventory weave caching in this
-        # generator.  Ideally the caching would be done in fetch.py I think.  Or
-        # maybe this generator should explicitly have the contract that it
-        # should not be iterated until the previously yielded item has been
-        # processed?
-        inv_w = self.inventories
-
-        # file ids that changed
-        file_ids = self.fileids_altered_by_revision_ids(revision_ids, inv_w)
-        count = 0
-        num_file_ids = len(file_ids)
-        for file_id, altered_versions in file_ids.iteritems():
-            if pb is not None:
-                pb.update("Fetch texts", count, num_file_ids)
-            count += 1
-            yield ("file", file_id, altered_versions)
-
-    def _find_non_file_keys_to_fetch(self, revision_ids):
-        # inventory
-        yield ("inventory", None, revision_ids)
-
-        # signatures
-        # XXX: Note ATM no callers actually pay attention to this return
-        #      instead they just use the list of revision ids and ignore
-        #      missing sigs. Consider removing this work entirely
-        revisions_with_signatures = set(self.signatures.get_parent_map(
-            [(r,) for r in revision_ids]))
-        revisions_with_signatures = set(
-            [r for (r,) in revisions_with_signatures])
-        revisions_with_signatures.intersection_update(revision_ids)
-        yield ("signatures", None, revisions_with_signatures)
-
-        # revisions
-        yield ("revisions", None, revision_ids)
-
-    @needs_read_lock
-    def get_inventory(self, revision_id):
-        """Get Inventory object by revision id."""
-        return self.iter_inventories([revision_id]).next()
-
-    def iter_inventories(self, revision_ids, ordering=None):
-        """Get many inventories by revision_ids.
-
-        This will buffer some or all of the texts used in constructing the
-        inventories in memory, but will only parse a single inventory at a
-        time.
-
-        :param revision_ids: The expected revision ids of the inventories.
-        :param ordering: optional ordering, e.g. 'topological'.  If not
-            specified, the order of revision_ids will be preserved (by
-            buffering if necessary).
-        :return: An iterator of inventories.
-        """
-        if ((None in revision_ids)
-            or (_mod_revision.NULL_REVISION in revision_ids)):
-            raise ValueError('cannot get null revision inventory')
-        return self._iter_inventories(revision_ids, ordering)
-
-    def _iter_inventories(self, revision_ids, ordering):
-        """single-document based inventory iteration."""
-        inv_xmls = self._iter_inventory_xmls(revision_ids, ordering)
-        for text, revision_id in inv_xmls:
-            yield self._deserialise_inventory(revision_id, text)
-
-    def _iter_inventory_xmls(self, revision_ids, ordering):
-        if ordering is None:
-            order_as_requested = True
-            ordering = 'unordered'
-        else:
-            order_as_requested = False
-        keys = [(revision_id,) for revision_id in revision_ids]
-        if not keys:
-            return
-        if order_as_requested:
-            key_iter = iter(keys)
-            next_key = key_iter.next()
-        stream = self.inventories.get_record_stream(keys, ordering, True)
-        text_chunks = {}
-        for record in stream:
-            if record.storage_kind != 'absent':
-                chunks = record.get_bytes_as('chunked')
-                if order_as_requested:
-                    text_chunks[record.key] = chunks
-                else:
-                    yield ''.join(chunks), record.key[-1]
-            else:
-                raise errors.NoSuchRevision(self, record.key)
-            if order_as_requested:
-                # Yield as many results as we can while preserving order.
-                while next_key in text_chunks:
-                    chunks = text_chunks.pop(next_key)
-                    yield ''.join(chunks), next_key[-1]
-                    try:
-                        next_key = key_iter.next()
-                    except StopIteration:
-                        # We still want to fully consume the get_record_stream,
-                        # just in case it is not actually finished at this point
-                        next_key = None
-                        break
-
-    def _deserialise_inventory(self, revision_id, xml):
-        """Transform the xml into an inventory object.
-
-        :param revision_id: The expected revision id of the inventory.
-        :param xml: A serialised inventory.
-        """
-        result = self._serializer.read_inventory_from_string(xml, revision_id,
-                    entry_cache=self._inventory_entry_cache,
-                    return_from_cache=self._safe_to_return_from_cache)
-        if result.revision_id != revision_id:
-            raise AssertionError('revision id mismatch %s != %s' % (
-                result.revision_id, revision_id))
-        return result
-
-    def get_serializer_format(self):
-        return self._serializer.format_num
-
-    @needs_read_lock
-    def _get_inventory_xml(self, revision_id):
-        """Get serialized inventory as a string."""
-        texts = self._iter_inventory_xmls([revision_id], 'unordered')
-        try:
-            text, revision_id = texts.next()
-        except StopIteration:
-            raise errors.HistoryMissing(self, 'inventory', revision_id)
-        return text
+        raise NotImplementedError(self.iter_files_bytes)
 
     def get_rev_id_for_revno(self, revno, known_pair):
         """Return the revision id of a revno, given a later (revno, revid)
@@ -2504,6 +981,7 @@
             raise AssertionError('_iter_for_revno returned too much history')
         return (True, partial_history[-1])
 
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def iter_reverse_revision_history(self, revision_id):
         """Iterate backwards through revision ids in the lefthand history
 
@@ -2511,19 +989,8 @@
             ancestors will be traversed.
         """
         graph = self.get_graph()
-        next_id = revision_id
-        while True:
-            if next_id in (None, _mod_revision.NULL_REVISION):
-                return
-            try:
-                parents = graph.get_parent_map([next_id])[next_id]
-            except KeyError:
-                raise errors.RevisionNotPresent(next_id, self)
-            yield next_id
-            if len(parents) == 0:
-                return
-            else:
-                next_id = parents[0]
+        stop_revisions = (None, _mod_revision.NULL_REVISION)
+        return graph.iter_lefthand_ancestry(revision_id, stop_revisions)
 
     def is_shared(self):
         """Return True if this repository is flagged as a shared repository."""
@@ -2557,15 +1024,7 @@
 
         `revision_id` may be NULL_REVISION for the empty tree revision.
         """
-        revision_id = _mod_revision.ensure_null(revision_id)
-        # TODO: refactor this to use an existing revision object
-        # so we don't need to read it in twice.
-        if revision_id == _mod_revision.NULL_REVISION:
-            return RevisionTree(self, Inventory(root_id=None),
-                                _mod_revision.NULL_REVISION)
-        else:
-            inv = self.get_inventory(revision_id)
-            return RevisionTree(self, inv, revision_id)
+        raise NotImplementedError(self.revision_tree)
 
     def revision_trees(self, revision_ids):
         """Return Trees for revisions in this repository.
@@ -2573,27 +1032,11 @@
         :param revision_ids: a sequence of revision-ids;
           a revision-id may not be None or 'null:'
         """
-        inventories = self.iter_inventories(revision_ids)
-        for inv in inventories:
-            yield RevisionTree(self, inv, inv.revision_id)
-
-    def _filtered_revision_trees(self, revision_ids, file_ids):
-        """Return Tree for a revision on this branch with only some files.
-
-        :param revision_ids: a sequence of revision-ids;
-          a revision-id may not be None or 'null:'
-        :param file_ids: if not None, the result is filtered
-          so that only those file-ids, their parents and their
-          children are included.
-        """
-        inventories = self.iter_inventories(revision_ids)
-        for inv in inventories:
-            # Should we introduce a FilteredRevisionTree class rather
-            # than pre-filter the inventory here?
-            filtered_inv = inv.filter(file_ids)
-            yield RevisionTree(self, filtered_inv, filtered_inv.revision_id)
+        raise NotImplementedError(self.revision_trees)
 
     @needs_read_lock
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def get_ancestry(self, revision_id, topo_sorted=True):
         """Return a list of revision-ids integrated by a revision.
 
@@ -2603,6 +1046,8 @@
 
         This is topologically sorted.
         """
+        if 'evil' in debug.debug_flags:
+            mutter_callsite(2, "get_ancestry is linear with history.")
         if _mod_revision.is_null(revision_id):
             return [None]
         if not self.has_revision(revision_id):
@@ -2630,7 +1075,7 @@
         types it should be a no-op that just returns.
 
         This stub method does not require a lock, but subclasses should use
-        @needs_write_lock as this is a long running call its reasonable to
+        @needs_write_lock as this is a long running call it's reasonable to
         implicitly lock for the user.
 
         :param hint: If not supplied, the whole repository is packed.
@@ -2649,6 +1094,10 @@
 
     def get_parent_map(self, revision_ids):
         """See graph.StackedParentsProvider.get_parent_map"""
+        raise NotImplementedError(self.get_parent_map)
+
+    def _get_parent_map_no_fallbacks(self, revision_ids):
+        """Same as Repository.get_parent_map except doesn't query fallbacks."""
         # revisions index works in keys; this just works in revisions
         # therefore wrap and unwrap
         query_keys = []
@@ -2660,8 +1109,9 @@
                 raise ValueError('get_parent_map(None) is not valid')
             else:
                 query_keys.append((revision_id ,))
+        vf = self.revisions.without_fallbacks()
         for ((revision_id,), parent_keys) in \
-                self.revisions.get_parent_map(query_keys).iteritems():
+                vf.get_parent_map(query_keys).iteritems():
             if parent_keys:
                 result[revision_id] = tuple([parent_revid
                     for (parent_revid,) in parent_keys])
@@ -2670,16 +1120,25 @@
         return result
 
     def _make_parents_provider(self):
-        return self
+        if not self._format.supports_external_lookups:
+            return self
+        return graph.StackedParentsProvider(_LazyListJoin(
+            [self._make_parents_provider_unstacked()],
+            self._fallback_repositories))
+
+    def _make_parents_provider_unstacked(self):
+        return graph.CallableToParentsProviderAdapter(
+            self._get_parent_map_no_fallbacks)
 
     @needs_read_lock
     def get_known_graph_ancestry(self, revision_ids):
         """Return the known graph for a set of revision ids and their ancestors.
         """
-        st = static_tuple.StaticTuple
-        revision_keys = [st(r_id).intern() for r_id in revision_ids]
-        known_graph = self.revisions.get_known_graph_ancestry(revision_keys)
-        return graph.GraphThunkIdsToKeys(known_graph)
+        raise NotImplementedError(self.get_known_graph_ancestry)
+
+    def get_file_graph(self):
+        """Return the graph walker for files."""
+        raise NotImplementedError(self.get_file_graph)
 
     def get_graph(self, other_repository=None):
         """Return the graph walker for this repository format"""
@@ -2690,22 +1149,6 @@
                 [parents_provider, other_repository._make_parents_provider()])
         return graph.Graph(parents_provider)
 
-    def _get_versioned_file_checker(self, text_key_references=None,
-        ancestors=None):
-        """Return an object suitable for checking versioned files.
-        
-        :param text_key_references: if non-None, an already built
-            dictionary mapping text keys ((fileid, revision_id) tuples)
-            to whether they were referred to by the inventory of the
-            revision_id that they contain. If None, this will be
-            calculated.
-        :param ancestors: Optional result from
-            self.get_graph().get_parent_map(self.all_revision_ids()) if already
-            available.
-        """
-        return _VersionedFileChecker(self,
-            text_key_references=text_key_references, ancestors=ancestors)
-
     def revision_ids_to_search_result(self, result_set):
         """Convert a set of revision ids to a graph SearchResult."""
         result_parents = set()
@@ -2737,29 +1180,36 @@
 
     @needs_write_lock
     def sign_revision(self, revision_id, gpg_strategy):
-        plaintext = Testament.from_revision(self, revision_id).as_short_text()
+        testament = _mod_testament.Testament.from_revision(self, revision_id)
+        plaintext = testament.as_short_text()
         self.store_revision_signature(gpg_strategy, plaintext, revision_id)
 
     @needs_read_lock
+    def verify_revision(self, revision_id, gpg_strategy):
+        """Verify the signature on a revision.
+        
+        :param revision_id: the revision to verify
+        :gpg_strategy: the GPGStrategy object to used
+        
+        :return: gpg.SIGNATURE_VALID or a failed SIGNATURE_ value
+        """
+        if not self.has_signature_for_revision_id(revision_id):
+            return gpg.SIGNATURE_NOT_SIGNED, None
+        signature = self.get_signature_text(revision_id)
+
+        testament = _mod_testament.Testament.from_revision(self, revision_id)
+        plaintext = testament.as_short_text()
+
+        return gpg_strategy.verify(signature, plaintext)
+
     def has_signature_for_revision_id(self, revision_id):
         """Query for a revision signature for revision_id in the repository."""
-        if not self.has_revision(revision_id):
-            raise errors.NoSuchRevision(self, revision_id)
-        sig_present = (1 == len(
-            self.signatures.get_parent_map([(revision_id,)])))
-        return sig_present
+        raise NotImplementedError(self.has_signature_for_revision_id)
 
-    @needs_read_lock
     def get_signature_text(self, revision_id):
         """Return the text for a signature."""
-        stream = self.signatures.get_record_stream([(revision_id,)],
-            'unordered', True)
-        record = stream.next()
-        if record.storage_kind == 'absent':
-            raise errors.NoSuchRevision(self, revision_id)
-        return record.get_bytes_as('fulltext')
+        raise NotImplementedError(self.get_signature_text)
 
-    @needs_read_lock
     def check(self, revision_ids=None, callback_refs=None, check_repo=True):
         """Check consistency of all history of given revision_ids.
 
@@ -2773,15 +1223,15 @@
         :param check_repo: If False do not check the repository contents, just 
             calculate the data callback_refs requires and call them back.
         """
-        return self._check(revision_ids, callback_refs=callback_refs,
+        return self._check(revision_ids=revision_ids, callback_refs=callback_refs,
             check_repo=check_repo)
 
-    def _check(self, revision_ids, callback_refs, check_repo):
-        result = check.Check(self, check_repo=check_repo)
-        result.check(callback_refs)
-        return result
+    def _check(self, revision_ids=None, callback_refs=None, check_repo=True):
+        raise NotImplementedError(self.check)
 
     def _warn_if_deprecated(self, branch=None):
+        if not self._format.is_deprecated():
+            return
         global _deprecation_warning_done
         if _deprecation_warning_done:
             return
@@ -2817,141 +1267,6 @@
                 except UnicodeDecodeError:
                     raise errors.NonAsciiRevisionId(method, self)
 
-    def revision_graph_can_have_wrong_parents(self):
-        """Is it possible for this repository to have a revision graph with
-        incorrect parents?
-
-        If True, then this repository must also implement
-        _find_inconsistent_revision_parents so that check and reconcile can
-        check for inconsistencies before proceeding with other checks that may
-        depend on the revision index being consistent.
-        """
-        raise NotImplementedError(self.revision_graph_can_have_wrong_parents)
-
-
-# remove these delegates a while after bzr 0.15
-def __make_delegated(name, from_module):
-    def _deprecated_repository_forwarder():
-        symbol_versioning.warn('%s moved to %s in bzr 0.15'
-            % (name, from_module),
-            DeprecationWarning,
-            stacklevel=2)
-        m = __import__(from_module, globals(), locals(), [name])
-        try:
-            return getattr(m, name)
-        except AttributeError:
-            raise AttributeError('module %s has no name %s'
-                    % (m, name))
-    globals()[name] = _deprecated_repository_forwarder
-
-for _name in [
-        'AllInOneRepository',
-        'WeaveMetaDirRepository',
-        'PreSplitOutRepositoryFormat',
-        'RepositoryFormat4',
-        'RepositoryFormat5',
-        'RepositoryFormat6',
-        'RepositoryFormat7',
-        ]:
-    __make_delegated(_name, 'bzrlib.repofmt.weaverepo')
-
-for _name in [
-        'KnitRepository',
-        'RepositoryFormatKnit',
-        'RepositoryFormatKnit1',
-        ]:
-    __make_delegated(_name, 'bzrlib.repofmt.knitrepo')
-
-
-def install_revision(repository, rev, revision_tree):
-    """Install all revision data into a repository."""
-    install_revisions(repository, [(rev, revision_tree, None)])
-
-
-def install_revisions(repository, iterable, num_revisions=None, pb=None):
-    """Install all revision data into a repository.
-
-    Accepts an iterable of revision, tree, signature tuples.  The signature
-    may be None.
-    """
-    repository.start_write_group()
-    try:
-        inventory_cache = lru_cache.LRUCache(10)
-        for n, (revision, revision_tree, signature) in enumerate(iterable):
-            _install_revision(repository, revision, revision_tree, signature,
-                inventory_cache)
-            if pb is not None:
-                pb.update('Transferring revisions', n + 1, num_revisions)
-    except:
-        repository.abort_write_group()
-        raise
-    else:
-        repository.commit_write_group()
-
-
-def _install_revision(repository, rev, revision_tree, signature,
-    inventory_cache):
-    """Install all revision data into a repository."""
-    present_parents = []
-    parent_trees = {}
-    for p_id in rev.parent_ids:
-        if repository.has_revision(p_id):
-            present_parents.append(p_id)
-            parent_trees[p_id] = repository.revision_tree(p_id)
-        else:
-            parent_trees[p_id] = repository.revision_tree(
-                                     _mod_revision.NULL_REVISION)
-
-    inv = revision_tree.inventory
-    entries = inv.iter_entries()
-    # backwards compatibility hack: skip the root id.
-    if not repository.supports_rich_root():
-        path, root = entries.next()
-        if root.revision != rev.revision_id:
-            raise errors.IncompatibleRevision(repr(repository))
-    text_keys = {}
-    for path, ie in entries:
-        text_keys[(ie.file_id, ie.revision)] = ie
-    text_parent_map = repository.texts.get_parent_map(text_keys)
-    missing_texts = set(text_keys) - set(text_parent_map)
-    # Add the texts that are not already present
-    for text_key in missing_texts:
-        ie = text_keys[text_key]
-        text_parents = []
-        # FIXME: TODO: The following loop overlaps/duplicates that done by
-        # commit to determine parents. There is a latent/real bug here where
-        # the parents inserted are not those commit would do - in particular
-        # they are not filtered by heads(). RBC, AB
-        for revision, tree in parent_trees.iteritems():
-            if ie.file_id not in tree:
-                continue
-            parent_id = tree.inventory[ie.file_id].revision
-            if parent_id in text_parents:
-                continue
-            text_parents.append((ie.file_id, parent_id))
-        lines = revision_tree.get_file(ie.file_id).readlines()
-        repository.texts.add_lines(text_key, text_parents, lines)
-    try:
-        # install the inventory
-        if repository._format._commit_inv_deltas and len(rev.parent_ids):
-            # Cache this inventory
-            inventory_cache[rev.revision_id] = inv
-            try:
-                basis_inv = inventory_cache[rev.parent_ids[0]]
-            except KeyError:
-                repository.add_inventory(rev.revision_id, inv, present_parents)
-            else:
-                delta = inv._make_delta(basis_inv)
-                repository.add_inventory_by_delta(rev.parent_ids[0], delta,
-                    rev.revision_id, present_parents)
-        else:
-            repository.add_inventory(rev.revision_id, inv, present_parents)
-    except errors.RevisionAlreadyPresent:
-        pass
-    if signature is not None:
-        repository.add_signature_text(rev.revision_id, signature)
-    repository.add_revision(rev.revision_id, rev, inv)
-
 
 class MetaDirRepository(Repository):
     """Repositories in the new meta-dir layout.
@@ -2992,12 +1307,12 @@
         return not self._transport.has('no-working-trees')
 
 
-class MetaDirVersionedFileRepository(MetaDirRepository):
-    """Repositories in a meta-dir, that work via versioned file objects."""
+class RepositoryFormatRegistry(controldir.ControlComponentFormatRegistry):
+    """Repository format registry."""
 
-    def __init__(self, _format, a_bzrdir, control_files):
-        super(MetaDirVersionedFileRepository, self).__init__(_format, a_bzrdir,
-            control_files)
+    def get_default(self):
+        """Return the current default format."""
+        return controldir.format_registry.make_bzrdir('default').repository_format
 
 
 network_format_registry = registry.FormatRegistry()
@@ -3009,7 +1324,7 @@
 """
 
 
-format_registry = registry.FormatRegistry(network_format_registry)
+format_registry = RepositoryFormatRegistry(network_format_registry)
 """Registry of formats, indexed by their BzrDirMetaFormat format string.
 
 This can contain either format instances themselves, or classes/factories that
@@ -3020,7 +1335,7 @@
 #####################################################################
 # Repository Formats
 
-class RepositoryFormat(object):
+class RepositoryFormat(controldir.ControlComponentFormat):
     """A repository format.
 
     Formats provide four things:
@@ -3046,7 +1361,7 @@
     created.
 
     Common instance attributes:
-    _matchingbzrdir - the bzrdir format that the repository format was
+    _matchingbzrdir - the controldir format that the repository format was
     originally written to work with. This can be used if manually
     constructing a bzrdir and repository, or more commonly for test suite
     parameterization.
@@ -3061,17 +1376,6 @@
     # Does this format support CHK bytestring lookups. Set to True or False in
     # derived classes.
     supports_chks = None
-    # Should commit add an inventory, or an inventory delta to the repository.
-    _commit_inv_deltas = True
-    # What order should fetch operations request streams in?
-    # The default is unordered as that is the cheapest for an origin to
-    # provide.
-    _fetch_order = 'unordered'
-    # Does this repository format use deltas that can be fetched as-deltas ?
-    # (E.g. knits, where the knit deltas can be transplanted intact.
-    # We default to False, which will ensure that enough data to get
-    # a full text out of any fetch stream will be grabbed.
-    _fetch_uses_deltas = False
     # Should fetch trigger a reconcile after the fetch? Only needed for
     # some repository formats that can suffer internal inconsistencies.
     _fetch_reconcile = False
@@ -3083,10 +1387,28 @@
     # help), and for fetching when data won't have come from the same
     # compressor.
     pack_compresses = False
-    # Does the repository inventory storage understand references to trees?
+    # Does the repository storage understand references to trees?
     supports_tree_reference = None
     # Is the format experimental ?
     experimental = False
+    # Does this repository format escape funky characters, or does it create
+    # files with similar names as the versioned files in its contents on disk
+    # ?
+    supports_funky_characters = None
+    # Does this repository format support leaving locks?
+    supports_leaving_lock = None
+    # Does this format support the full VersionedFiles interface?
+    supports_full_versioned_files = None
+    # Does this format support signing revision signatures?
+    supports_revision_signatures = True
+    # Can the revision graph have incorrect parents?
+    revision_graph_can_have_wrong_parents = None
+    # Does this format support rich root data?
+    rich_root_data = None
+    # Does this format support explicitly versioned directories?
+    supports_versioned_directories = None
+    # Can other repositories be nested into one of this format?
+    supports_nesting_repositories = None
 
     def __repr__(self):
         return "%s()" % self.__class__.__name__
@@ -3117,18 +1439,20 @@
                                             kind='repository')
 
     @classmethod
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def register_format(klass, format):
-        format_registry.register(format.get_format_string(), format)
+        format_registry.register(format)
 
     @classmethod
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def unregister_format(klass, format):
-        format_registry.remove(format.get_format_string())
+        format_registry.remove(format)
 
     @classmethod
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def get_default_format(klass):
         """Return the current default format."""
-        from bzrlib import bzrdir
-        return bzrdir.format_registry.make_bzrdir('default').repository_format
+        return format_registry.get_default()
 
     def get_format_string(self):
         """Return the ASCII format string that identifies this format.
@@ -3142,37 +1466,15 @@
         """Return the short description for this format."""
         raise NotImplementedError(self.get_format_description)
 
-    # TODO: this shouldn't be in the base class, it's specific to things that
-    # use weaves or knits -- mbp 20070207
-    def _get_versioned_file_store(self,
-                                  name,
-                                  transport,
-                                  control_files,
-                                  prefixed=True,
-                                  versionedfile_class=None,
-                                  versionedfile_kwargs={},
-                                  escaped=False):
-        if versionedfile_class is None:
-            versionedfile_class = self._versionedfile_class
-        weave_transport = control_files._transport.clone(name)
-        dir_mode = control_files._dir_mode
-        file_mode = control_files._file_mode
-        return VersionedFileStore(weave_transport, prefixed=prefixed,
-                                  dir_mode=dir_mode,
-                                  file_mode=file_mode,
-                                  versionedfile_class=versionedfile_class,
-                                  versionedfile_kwargs=versionedfile_kwargs,
-                                  escaped=escaped)
-
-    def initialize(self, a_bzrdir, shared=False):
-        """Initialize a repository of this format in a_bzrdir.
-
-        :param a_bzrdir: The bzrdir to put the new repository in it.
+    def initialize(self, controldir, shared=False):
+        """Initialize a repository of this format in controldir.
+
+        :param controldir: The controldir to put the new repository in it.
         :param shared: The repository should be initialized as a sharable one.
         :returns: The new repository object.
 
         This may raise UninitializableFormat if shared repository are not
-        compatible the a_bzrdir.
+        compatible the controldir.
         """
         raise NotImplementedError(self.initialize)
 
@@ -3185,6 +1487,14 @@
         """
         return True
 
+    def is_deprecated(self):
+        """Is this format deprecated?
+
+        Deprecated formats may trigger a user-visible warning recommending
+        the user to upgrade. They are still fully supported.
+        """
+        return False
+
     def network_name(self):
         """A simple byte string uniquely identifying this format for RPC calls.
 
@@ -3206,19 +1516,19 @@
                 'Does not support nested trees', target_format,
                 from_format=self)
 
-    def open(self, a_bzrdir, _found=False):
-        """Return an instance of this format for the bzrdir a_bzrdir.
+    def open(self, controldir, _found=False):
+        """Return an instance of this format for a controldir.
 
         _found is a private parameter, do not use it.
         """
         raise NotImplementedError(self.open)
 
-    def _run_post_repo_init_hooks(self, repository, a_bzrdir, shared):
-        from bzrlib.bzrdir import BzrDir, RepoInitHookParams
-        hooks = BzrDir.hooks['post_repo_init']
+    def _run_post_repo_init_hooks(self, repository, controldir, shared):
+        from bzrlib.controldir import ControlDir, RepoInitHookParams
+        hooks = ControlDir.hooks['post_repo_init']
         if not hooks:
             return
-        params = RepoInitHookParams(repository, self, a_bzrdir, shared)
+        params = RepoInitHookParams(repository, self, controldir, shared)
         for hook in hooks:
             hook(params)
 
@@ -3229,6 +1539,8 @@
     rich_root_data = False
     supports_tree_reference = False
     supports_external_lookups = False
+    supports_leaving_lock = True
+    supports_nesting_repositories = True
 
     @property
     def _matchingbzrdir(self):
@@ -3272,34 +1584,13 @@
         return self.get_format_string()
 
 
-# Pre-0.8 formats that don't have a disk format string (because they are
-# versioned by the matching control directory). We use the control directories
-# disk format string as a key for the network_name because they meet the
-# constraints (simple string, unique, immutable).
-network_format_registry.register_lazy(
-    "Bazaar-NG branch, format 5\n",
-    'bzrlib.repofmt.weaverepo',
-    'RepositoryFormat5',
-)
-network_format_registry.register_lazy(
-    "Bazaar-NG branch, format 6\n",
-    'bzrlib.repofmt.weaverepo',
-    'RepositoryFormat6',
-)
-
 # formats which have no format string are not discoverable or independently
 # creatable on disk, so are not registered in format_registry.  They're
-# all in bzrlib.repofmt.weaverepo now.  When an instance of one of these is
-# needed, it's constructed directly by the BzrDir.  Non-native formats where
+# all in bzrlib.repofmt.knitreponow.  When an instance of one of these is
+# needed, it's constructed directly by the ControlDir.  Non-native formats where
 # the repository is not separately opened are similar.
 
 format_registry.register_lazy(
-    'Bazaar-NG Repository format 7',
-    'bzrlib.repofmt.weaverepo',
-    'RepositoryFormat7'
-    )
-
-format_registry.register_lazy(
     'Bazaar-NG Knit Repository Format 1',
     'bzrlib.repofmt.knitrepo',
     'RepositoryFormatKnit1',
@@ -3322,72 +1613,62 @@
 # NOTE: These are experimental in 0.92. Stable in 1.0 and above
 format_registry.register_lazy(
     'Bazaar pack repository format 1 (needs bzr 0.92)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack1',
     )
 format_registry.register_lazy(
     'Bazaar pack repository format 1 with subtree support (needs bzr 0.92)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack3',
     )
 format_registry.register_lazy(
     'Bazaar pack repository format 1 with rich root (needs bzr 1.0)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack4',
     )
 format_registry.register_lazy(
     'Bazaar RepositoryFormatKnitPack5 (bzr 1.6)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack5',
     )
 format_registry.register_lazy(
     'Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6.1)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack5RichRoot',
     )
 format_registry.register_lazy(
     'Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack5RichRootBroken',
     )
 format_registry.register_lazy(
     'Bazaar RepositoryFormatKnitPack6 (bzr 1.9)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack6',
     )
 format_registry.register_lazy(
     'Bazaar RepositoryFormatKnitPack6RichRoot (bzr 1.9)\n',
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatKnitPack6RichRoot',
     )
+format_registry.register_lazy(
+    'Bazaar repository format 2a (needs bzr 1.16 or later)\n',
+    'bzrlib.repofmt.groupcompress_repo',
+    'RepositoryFormat2a',
+    )
 
 # Development formats.
-# Obsolete but kept pending a CHK based subtree format.
+# Check their docstrings to see if/when they are obsolete.
 format_registry.register_lazy(
     ("Bazaar development format 2 with subtree support "
         "(needs bzr.dev from before 1.8)\n"),
-    'bzrlib.repofmt.pack_repo',
+    'bzrlib.repofmt.knitpack_repo',
     'RepositoryFormatPackDevelopment2Subtree',
     )
-
-# 1.14->1.16 go below here
-format_registry.register_lazy(
-    'Bazaar development format - group compression and chk inventory'
-        ' (needs bzr.dev from 1.14)\n',
-    'bzrlib.repofmt.groupcompress_repo',
-    'RepositoryFormatCHK1',
-    )
-
-format_registry.register_lazy(
-    'Bazaar development format - chk repository with bencode revision '
-        'serialization (needs bzr.dev from 1.16)\n',
-    'bzrlib.repofmt.groupcompress_repo',
-    'RepositoryFormatCHK2',
-    )
-format_registry.register_lazy(
-    'Bazaar repository format 2a (needs bzr 1.16 or later)\n',
-    'bzrlib.repofmt.groupcompress_repo',
-    'RepositoryFormat2a',
+format_registry.register_lazy(
+    'Bazaar development format 8\n',
+    'bzrlib.repofmt.groupcompress_repo',
+    'RepositoryFormat2aSubtree',
     )
 
 
@@ -3403,7 +1684,6 @@
     InterRepository.get(other).method_name(parameters).
     """
 
-    _walk_to_common_revisions_batch_size = 50
     _optimisers = []
     """The available optimised InterRepository types."""
 
@@ -3424,7 +1704,7 @@
         self.target.fetch(self.source, revision_id=revision_id)
 
     @needs_write_lock
-    def fetch(self, revision_id=None, pb=None, find_ghosts=False,
+    def fetch(self, revision_id=None, find_ghosts=False,
             fetch_spec=None):
         """Fetch the content required to construct revision_id.
 
@@ -3432,98 +1712,33 @@
 
         :param revision_id: if None all content is copied, if NULL_REVISION no
                             content is copied.
-        :param pb: ignored.
         :return: None.
         """
-        ui.ui_factory.warn_experimental_format_fetch(self)
-        from bzrlib.fetch import RepoFetcher
-        # See <https://launchpad.net/bugs/456077> asking for a warning here
-        if self.source._format.network_name() != self.target._format.network_name():
-            ui.ui_factory.show_user_warning('cross_format_fetch',
-                from_format=self.source._format,
-                to_format=self.target._format)
-        f = RepoFetcher(to_repository=self.target,
-                               from_repository=self.source,
-                               last_revision=revision_id,
-                               fetch_spec=fetch_spec,
-                               find_ghosts=find_ghosts)
-
-    def _walk_to_common_revisions(self, revision_ids):
-        """Walk out from revision_ids in source to revisions target has.
-
-        :param revision_ids: The start point for the search.
-        :return: A set of revision ids.
-        """
-        target_graph = self.target.get_graph()
-        revision_ids = frozenset(revision_ids)
-        missing_revs = set()
-        source_graph = self.source.get_graph()
-        # ensure we don't pay silly lookup costs.
-        searcher = source_graph._make_breadth_first_searcher(revision_ids)
-        null_set = frozenset([_mod_revision.NULL_REVISION])
-        searcher_exhausted = False
-        while True:
-            next_revs = set()
-            ghosts = set()
-            # Iterate the searcher until we have enough next_revs
-            while len(next_revs) < self._walk_to_common_revisions_batch_size:
-                try:
-                    next_revs_part, ghosts_part = searcher.next_with_ghosts()
-                    next_revs.update(next_revs_part)
-                    ghosts.update(ghosts_part)
-                except StopIteration:
-                    searcher_exhausted = True
-                    break
-            # If there are ghosts in the source graph, and the caller asked for
-            # them, make sure that they are present in the target.
-            # We don't care about other ghosts as we can't fetch them and
-            # haven't been asked to.
-            ghosts_to_check = set(revision_ids.intersection(ghosts))
-            revs_to_get = set(next_revs).union(ghosts_to_check)
-            if revs_to_get:
-                have_revs = set(target_graph.get_parent_map(revs_to_get))
-                # we always have NULL_REVISION present.
-                have_revs = have_revs.union(null_set)
-                # Check if the target is missing any ghosts we need.
-                ghosts_to_check.difference_update(have_revs)
-                if ghosts_to_check:
-                    # One of the caller's revision_ids is a ghost in both the
-                    # source and the target.
-                    raise errors.NoSuchRevision(
-                        self.source, ghosts_to_check.pop())
-                missing_revs.update(next_revs - have_revs)
-                # Because we may have walked past the original stop point, make
-                # sure everything is stopped
-                stop_revs = searcher.find_seen_ancestors(have_revs)
-                searcher.stop_searching_any(stop_revs)
-            if searcher_exhausted:
-                break
-        return searcher.get_result()
+        raise NotImplementedError(self.fetch)
 
     @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
+    def search_missing_revision_ids(self,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None,
+            limit=None):
         """Return the revision ids that source has that target does not.
 
         :param revision_id: only return revision ids included by this
-                            revision_id.
+            revision_id.
+        :param revision_ids: return revision ids included by these
+            revision_ids.  NoSuchRevision will be raised if any of these
+            revisions are not present.
+        :param if_present_ids: like revision_ids, but will not cause
+            NoSuchRevision if any of these are absent, instead they will simply
+            not be in the result.  This is useful for e.g. finding revisions
+            to fetch for tags, which may reference absent revisions.
         :param find_ghosts: If True find missing revisions in deep history
             rather than just finding the surface difference.
+        :param limit: Maximum number of revisions to return, topologically
+            ordered
         :return: A bzrlib.graph.SearchResult.
         """
-        # stop searching at found target revisions.
-        if not find_ghosts and revision_id is not None:
-            return self._walk_to_common_revisions([revision_id])
-        # generic, possibly worst case, slow code path.
-        target_ids = set(self.target.all_revision_ids())
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source.all_revision_ids()
-        result_set = set(source_ids).difference(target_ids)
-        return self.source.revision_ids_to_search_result(result_set)
+        raise NotImplementedError(self.search_missing_revision_ids)
 
     @staticmethod
     def _same_model(source, target):
@@ -3550,549 +1765,6 @@
                 "different serializers")
 
 
-class InterSameDataRepository(InterRepository):
-    """Code for converting between repositories that represent the same data.
-
-    Data format and model must match for this to work.
-    """
-
-    @classmethod
-    def _get_repo_format_to_test(self):
-        """Repository format for testing with.
-
-        InterSameData can pull from subtree to subtree and from non-subtree to
-        non-subtree, so we test this with the richest repository format.
-        """
-        from bzrlib.repofmt import knitrepo
-        return knitrepo.RepositoryFormatKnit3()
-
-    @staticmethod
-    def is_compatible(source, target):
-        return InterRepository._same_model(source, target)
-
-
-class InterWeaveRepo(InterSameDataRepository):
-    """Optimised code paths between Weave based repositories.
-
-    This should be in bzrlib/repofmt/weaverepo.py but we have not yet
-    implemented lazy inter-object optimisation.
-    """
-
-    @classmethod
-    def _get_repo_format_to_test(self):
-        from bzrlib.repofmt import weaverepo
-        return weaverepo.RepositoryFormat7()
-
-    @staticmethod
-    def is_compatible(source, target):
-        """Be compatible with known Weave formats.
-
-        We don't test for the stores being of specific types because that
-        could lead to confusing results, and there is no need to be
-        overly general.
-        """
-        from bzrlib.repofmt.weaverepo import (
-                RepositoryFormat5,
-                RepositoryFormat6,
-                RepositoryFormat7,
-                )
-        try:
-            return (isinstance(source._format, (RepositoryFormat5,
-                                                RepositoryFormat6,
-                                                RepositoryFormat7)) and
-                    isinstance(target._format, (RepositoryFormat5,
-                                                RepositoryFormat6,
-                                                RepositoryFormat7)))
-        except AttributeError:
-            return False
-
-    @needs_write_lock
-    def copy_content(self, revision_id=None):
-        """See InterRepository.copy_content()."""
-        # weave specific optimised path:
-        try:
-            self.target.set_make_working_trees(self.source.make_working_trees())
-        except (errors.RepositoryUpgradeRequired, NotImplemented):
-            pass
-        # FIXME do not peek!
-        if self.source._transport.listable():
-            pb = ui.ui_factory.nested_progress_bar()
-            try:
-                self.target.texts.insert_record_stream(
-                    self.source.texts.get_record_stream(
-                        self.source.texts.keys(), 'topological', False))
-                pb.update('Copying inventory', 0, 1)
-                self.target.inventories.insert_record_stream(
-                    self.source.inventories.get_record_stream(
-                        self.source.inventories.keys(), 'topological', False))
-                self.target.signatures.insert_record_stream(
-                    self.source.signatures.get_record_stream(
-                        self.source.signatures.keys(),
-                        'unordered', True))
-                self.target.revisions.insert_record_stream(
-                    self.source.revisions.get_record_stream(
-                        self.source.revisions.keys(),
-                        'topological', True))
-            finally:
-                pb.finished()
-        else:
-            self.target.fetch(self.source, revision_id=revision_id)
-
-    @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
-        """See InterRepository.missing_revision_ids()."""
-        # we want all revisions to satisfy revision_id in source.
-        # but we don't want to stat every file here and there.
-        # we want then, all revisions other needs to satisfy revision_id
-        # checked, but not those that we have locally.
-        # so the first thing is to get a subset of the revisions to
-        # satisfy revision_id in source, and then eliminate those that
-        # we do already have.
-        # this is slow on high latency connection to self, but as this
-        # disk format scales terribly for push anyway due to rewriting
-        # inventory.weave, this is considered acceptable.
-        # - RBC 20060209
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source._all_possible_ids()
-        source_ids_set = set(source_ids)
-        # source_ids is the worst possible case we may need to pull.
-        # now we want to filter source_ids against what we actually
-        # have in target, but don't try to check for existence where we know
-        # we do not have a revision as that would be pointless.
-        target_ids = set(self.target._all_possible_ids())
-        possibly_present_revisions = target_ids.intersection(source_ids_set)
-        actually_present_revisions = set(
-            self.target._eliminate_revisions_not_present(possibly_present_revisions))
-        required_revisions = source_ids_set.difference(actually_present_revisions)
-        if revision_id is not None:
-            # we used get_ancestry to determine source_ids then we are assured all
-            # revisions referenced are present as they are installed in topological order.
-            # and the tip revision was validated by get_ancestry.
-            result_set = required_revisions
-        else:
-            # if we just grabbed the possibly available ids, then
-            # we only have an estimate of whats available and need to validate
-            # that against the revision records.
-            result_set = set(
-                self.source._eliminate_revisions_not_present(required_revisions))
-        return self.source.revision_ids_to_search_result(result_set)
-
-
-class InterKnitRepo(InterSameDataRepository):
-    """Optimised code paths between Knit based repositories."""
-
-    @classmethod
-    def _get_repo_format_to_test(self):
-        from bzrlib.repofmt import knitrepo
-        return knitrepo.RepositoryFormatKnit1()
-
-    @staticmethod
-    def is_compatible(source, target):
-        """Be compatible with known Knit formats.
-
-        We don't test for the stores being of specific types because that
-        could lead to confusing results, and there is no need to be
-        overly general.
-        """
-        from bzrlib.repofmt.knitrepo import RepositoryFormatKnit
-        try:
-            are_knits = (isinstance(source._format, RepositoryFormatKnit) and
-                isinstance(target._format, RepositoryFormatKnit))
-        except AttributeError:
-            return False
-        return are_knits and InterRepository._same_model(source, target)
-
-    @needs_read_lock
-    def search_missing_revision_ids(self, revision_id=None, find_ghosts=True):
-        """See InterRepository.missing_revision_ids()."""
-        if revision_id is not None:
-            source_ids = self.source.get_ancestry(revision_id)
-            if source_ids[0] is not None:
-                raise AssertionError()
-            source_ids.pop(0)
-        else:
-            source_ids = self.source.all_revision_ids()
-        source_ids_set = set(source_ids)
-        # source_ids is the worst possible case we may need to pull.
-        # now we want to filter source_ids against what we actually
-        # have in target, but don't try to check for existence where we know
-        # we do not have a revision as that would be pointless.
-        target_ids = set(self.target.all_revision_ids())
-        possibly_present_revisions = target_ids.intersection(source_ids_set)
-        actually_present_revisions = set(
-            self.target._eliminate_revisions_not_present(possibly_present_revisions))
-        required_revisions = source_ids_set.difference(actually_present_revisions)
-        if revision_id is not None:
-            # we used get_ancestry to determine source_ids then we are assured all
-            # revisions referenced are present as they are installed in topological order.
-            # and the tip revision was validated by get_ancestry.
-            result_set = required_revisions
-        else:
-            # if we just grabbed the possibly available ids, then
-            # we only have an estimate of whats available and need to validate
-            # that against the revision records.
-            result_set = set(
-                self.source._eliminate_revisions_not_present(required_revisions))
-        return self.source.revision_ids_to_search_result(result_set)
-
-
-class InterDifferingSerializer(InterRepository):
-
-    @classmethod
-    def _get_repo_format_to_test(self):
-        return None
-
-    @staticmethod
-    def is_compatible(source, target):
-        """Be compatible with Knit2 source and Knit3 target"""
-        # This is redundant with format.check_conversion_target(), however that
-        # raises an exception, and we just want to say "False" as in we won't
-        # support converting between these formats.
-        if 'IDS_never' in debug.debug_flags:
-            return False
-        if source.supports_rich_root() and not target.supports_rich_root():
-            return False
-        if (source._format.supports_tree_reference
-            and not target._format.supports_tree_reference):
-            return False
-        if target._fallback_repositories and target._format.supports_chks:
-            # IDS doesn't know how to copy CHKs for the parent inventories it
-            # adds to stacked repos.
-            return False
-        if 'IDS_always' in debug.debug_flags:
-            return True
-        # Only use this code path for local source and target.  IDS does far
-        # too much IO (both bandwidth and roundtrips) over a network.
-        if not source.bzrdir.transport.base.startswith('file:///'):
-            return False
-        if not target.bzrdir.transport.base.startswith('file:///'):
-            return False
-        return True
-
-    def _get_trees(self, revision_ids, cache):
-        possible_trees = []
-        for rev_id in revision_ids:
-            if rev_id in cache:
-                possible_trees.append((rev_id, cache[rev_id]))
-            else:
-                # Not cached, but inventory might be present anyway.
-                try:
-                    tree = self.source.revision_tree(rev_id)
-                except errors.NoSuchRevision:
-                    # Nope, parent is ghost.
-                    pass
-                else:
-                    cache[rev_id] = tree
-                    possible_trees.append((rev_id, tree))
-        return possible_trees
-
-    def _get_delta_for_revision(self, tree, parent_ids, possible_trees):
-        """Get the best delta and base for this revision.
-
-        :return: (basis_id, delta)
-        """
-        deltas = []
-        # Generate deltas against each tree, to find the shortest.
-        texts_possibly_new_in_tree = set()
-        for basis_id, basis_tree in possible_trees:
-            delta = tree.inventory._make_delta(basis_tree.inventory)
-            for old_path, new_path, file_id, new_entry in delta:
-                if new_path is None:
-                    # This file_id isn't present in the new rev, so we don't
-                    # care about it.
-                    continue
-                if not new_path:
-                    # Rich roots are handled elsewhere...
-                    continue
-                kind = new_entry.kind
-                if kind != 'directory' and kind != 'file':
-                    # No text record associated with this inventory entry.
-                    continue
-                # This is a directory or file that has changed somehow.
-                texts_possibly_new_in_tree.add((file_id, new_entry.revision))
-            deltas.append((len(delta), basis_id, delta))
-        deltas.sort()
-        return deltas[0][1:]
-
-    def _fetch_parent_invs_for_stacking(self, parent_map, cache):
-        """Find all parent revisions that are absent, but for which the
-        inventory is present, and copy those inventories.
-
-        This is necessary to preserve correctness when the source is stacked
-        without fallbacks configured.  (Note that in cases like upgrade the
-        source may be not have _fallback_repositories even though it is
-        stacked.)
-        """
-        parent_revs = set()
-        for parents in parent_map.values():
-            parent_revs.update(parents)
-        present_parents = self.source.get_parent_map(parent_revs)
-        absent_parents = set(parent_revs).difference(present_parents)
-        parent_invs_keys_for_stacking = self.source.inventories.get_parent_map(
-            (rev_id,) for rev_id in absent_parents)
-        parent_inv_ids = [key[-1] for key in parent_invs_keys_for_stacking]
-        for parent_tree in self.source.revision_trees(parent_inv_ids):
-            current_revision_id = parent_tree.get_revision_id()
-            parents_parents_keys = parent_invs_keys_for_stacking[
-                (current_revision_id,)]
-            parents_parents = [key[-1] for key in parents_parents_keys]
-            basis_id = _mod_revision.NULL_REVISION
-            basis_tree = self.source.revision_tree(basis_id)
-            delta = parent_tree.inventory._make_delta(basis_tree.inventory)
-            self.target.add_inventory_by_delta(
-                basis_id, delta, current_revision_id, parents_parents)
-            cache[current_revision_id] = parent_tree
-
-    def _fetch_batch(self, revision_ids, basis_id, cache, a_graph=None):
-        """Fetch across a few revisions.
-
-        :param revision_ids: The revisions to copy
-        :param basis_id: The revision_id of a tree that must be in cache, used
-            as a basis for delta when no other base is available
-        :param cache: A cache of RevisionTrees that we can use.
-        :param a_graph: A Graph object to determine the heads() of the
-            rich-root data stream.
-        :return: The revision_id of the last converted tree. The RevisionTree
-            for it will be in cache
-        """
-        # Walk though all revisions; get inventory deltas, copy referenced
-        # texts that delta references, insert the delta, revision and
-        # signature.
-        root_keys_to_create = set()
-        text_keys = set()
-        pending_deltas = []
-        pending_revisions = []
-        parent_map = self.source.get_parent_map(revision_ids)
-        self._fetch_parent_invs_for_stacking(parent_map, cache)
-        self.source._safe_to_return_from_cache = True
-        for tree in self.source.revision_trees(revision_ids):
-            # Find a inventory delta for this revision.
-            # Find text entries that need to be copied, too.
-            current_revision_id = tree.get_revision_id()
-            parent_ids = parent_map.get(current_revision_id, ())
-            parent_trees = self._get_trees(parent_ids, cache)
-            possible_trees = list(parent_trees)
-            if len(possible_trees) == 0:
-                # There either aren't any parents, or the parents are ghosts,
-                # so just use the last converted tree.
-                possible_trees.append((basis_id, cache[basis_id]))
-            basis_id, delta = self._get_delta_for_revision(tree, parent_ids,
-                                                           possible_trees)
-            revision = self.source.get_revision(current_revision_id)
-            pending_deltas.append((basis_id, delta,
-                current_revision_id, revision.parent_ids))
-            if self._converting_to_rich_root:
-                self._revision_id_to_root_id[current_revision_id] = \
-                    tree.get_root_id()
-            # Determine which texts are in present in this revision but not in
-            # any of the available parents.
-            texts_possibly_new_in_tree = set()
-            for old_path, new_path, file_id, entry in delta:
-                if new_path is None:
-                    # This file_id isn't present in the new rev
-                    continue
-                if not new_path:
-                    # This is the root
-                    if not self.target.supports_rich_root():
-                        # The target doesn't support rich root, so we don't
-                        # copy
-                        continue
-                    if self._converting_to_rich_root:
-                        # This can't be copied normally, we have to insert
-                        # it specially
-                        root_keys_to_create.add((file_id, entry.revision))
-                        continue
-                kind = entry.kind
-                texts_possibly_new_in_tree.add((file_id, entry.revision))
-            for basis_id, basis_tree in possible_trees:
-                basis_inv = basis_tree.inventory
-                for file_key in list(texts_possibly_new_in_tree):
-                    file_id, file_revision = file_key
-                    try:
-                        entry = basis_inv[file_id]
-                    except errors.NoSuchId:
-                        continue
-                    if entry.revision == file_revision:
-                        texts_possibly_new_in_tree.remove(file_key)
-            text_keys.update(texts_possibly_new_in_tree)
-            pending_revisions.append(revision)
-            cache[current_revision_id] = tree
-            basis_id = current_revision_id
-        self.source._safe_to_return_from_cache = False
-        # Copy file texts
-        from_texts = self.source.texts
-        to_texts = self.target.texts
-        if root_keys_to_create:
-            root_stream = _mod_fetch._new_root_data_stream(
-                root_keys_to_create, self._revision_id_to_root_id, parent_map,
-                self.source, graph=a_graph)
-            to_texts.insert_record_stream(root_stream)
-        to_texts.insert_record_stream(from_texts.get_record_stream(
-            text_keys, self.target._format._fetch_order,
-            not self.target._format._fetch_uses_deltas))
-        # insert inventory deltas
-        for delta in pending_deltas:
-            self.target.add_inventory_by_delta(*delta)
-        if self.target._fallback_repositories:
-            # Make sure this stacked repository has all the parent inventories
-            # for the new revisions that we are about to insert.  We do this
-            # before adding the revisions so that no revision is added until
-            # all the inventories it may depend on are added.
-            # Note that this is overzealous, as we may have fetched these in an
-            # earlier batch.
-            parent_ids = set()
-            revision_ids = set()
-            for revision in pending_revisions:
-                revision_ids.add(revision.revision_id)
-                parent_ids.update(revision.parent_ids)
-            parent_ids.difference_update(revision_ids)
-            parent_ids.discard(_mod_revision.NULL_REVISION)
-            parent_map = self.source.get_parent_map(parent_ids)
-            # we iterate over parent_map and not parent_ids because we don't
-            # want to try copying any revision which is a ghost
-            for parent_tree in self.source.revision_trees(parent_map):
-                current_revision_id = parent_tree.get_revision_id()
-                parents_parents = parent_map[current_revision_id]
-                possible_trees = self._get_trees(parents_parents, cache)
-                if len(possible_trees) == 0:
-                    # There either aren't any parents, or the parents are
-                    # ghosts, so just use the last converted tree.
-                    possible_trees.append((basis_id, cache[basis_id]))
-                basis_id, delta = self._get_delta_for_revision(parent_tree,
-                    parents_parents, possible_trees)
-                self.target.add_inventory_by_delta(
-                    basis_id, delta, current_revision_id, parents_parents)
-        # insert signatures and revisions
-        for revision in pending_revisions:
-            try:
-                signature = self.source.get_signature_text(
-                    revision.revision_id)
-                self.target.add_signature_text(revision.revision_id,
-                    signature)
-            except errors.NoSuchRevision:
-                pass
-            self.target.add_revision(revision.revision_id, revision)
-        return basis_id
-
-    def _fetch_all_revisions(self, revision_ids, pb):
-        """Fetch everything for the list of revisions.
-
-        :param revision_ids: The list of revisions to fetch. Must be in
-            topological order.
-        :param pb: A ProgressTask
-        :return: None
-        """
-        basis_id, basis_tree = self._get_basis(revision_ids[0])
-        batch_size = 100
-        cache = lru_cache.LRUCache(100)
-        cache[basis_id] = basis_tree
-        del basis_tree # We don't want to hang on to it here
-        hints = []
-        if self._converting_to_rich_root and len(revision_ids) > 100:
-            a_graph = _mod_fetch._get_rich_root_heads_graph(self.source,
-                                                            revision_ids)
-        else:
-            a_graph = None
-
-        for offset in range(0, len(revision_ids), batch_size):
-            self.target.start_write_group()
-            try:
-                pb.update('Transferring revisions', offset,
-                          len(revision_ids))
-                batch = revision_ids[offset:offset+batch_size]
-                basis_id = self._fetch_batch(batch, basis_id, cache,
-                                             a_graph=a_graph)
-            except:
-                self.source._safe_to_return_from_cache = False
-                self.target.abort_write_group()
-                raise
-            else:
-                hint = self.target.commit_write_group()
-                if hint:
-                    hints.extend(hint)
-        if hints and self.target._format.pack_compresses:
-            self.target.pack(hint=hints)
-        pb.update('Transferring revisions', len(revision_ids),
-                  len(revision_ids))
-
-    @needs_write_lock
-    def fetch(self, revision_id=None, pb=None, find_ghosts=False,
-            fetch_spec=None):
-        """See InterRepository.fetch()."""
-        if fetch_spec is not None:
-            raise AssertionError("Not implemented yet...")
-        ui.ui_factory.warn_experimental_format_fetch(self)
-        if (not self.source.supports_rich_root()
-            and self.target.supports_rich_root()):
-            self._converting_to_rich_root = True
-            self._revision_id_to_root_id = {}
-        else:
-            self._converting_to_rich_root = False
-        # See <https://launchpad.net/bugs/456077> asking for a warning here
-        if self.source._format.network_name() != self.target._format.network_name():
-            ui.ui_factory.show_user_warning('cross_format_fetch',
-                from_format=self.source._format,
-                to_format=self.target._format)
-        revision_ids = self.target.search_missing_revision_ids(self.source,
-            revision_id, find_ghosts=find_ghosts).get_keys()
-        if not revision_ids:
-            return 0, 0
-        revision_ids = tsort.topo_sort(
-            self.source.get_graph().get_parent_map(revision_ids))
-        if not revision_ids:
-            return 0, 0
-        # Walk though all revisions; get inventory deltas, copy referenced
-        # texts that delta references, insert the delta, revision and
-        # signature.
-        if pb is None:
-            my_pb = ui.ui_factory.nested_progress_bar()
-            pb = my_pb
-        else:
-            symbol_versioning.warn(
-                symbol_versioning.deprecated_in((1, 14, 0))
-                % "pb parameter to fetch()")
-            my_pb = None
-        try:
-            self._fetch_all_revisions(revision_ids, pb)
-        finally:
-            if my_pb is not None:
-                my_pb.finished()
-        return len(revision_ids), 0
-
-    def _get_basis(self, first_revision_id):
-        """Get a revision and tree which exists in the target.
-
-        This assumes that first_revision_id is selected for transmission
-        because all other ancestors are already present. If we can't find an
-        ancestor we fall back to NULL_REVISION since we know that is safe.
-
-        :return: (basis_id, basis_tree)
-        """
-        first_rev = self.source.get_revision(first_revision_id)
-        try:
-            basis_id = first_rev.parent_ids[0]
-            # only valid as a basis if the target has it
-            self.target.get_revision(basis_id)
-            # Try to get a basis tree - if its a ghost it will hit the
-            # NoSuchRevision case.
-            basis_tree = self.source.revision_tree(basis_id)
-        except (IndexError, errors.NoSuchRevision):
-            basis_id = _mod_revision.NULL_REVISION
-            basis_tree = self.source.revision_tree(basis_id)
-        return basis_id, basis_tree
-
-
-InterRepository.register_optimiser(InterDifferingSerializer)
-InterRepository.register_optimiser(InterSameDataRepository)
-InterRepository.register_optimiser(InterWeaveRepo)
-InterRepository.register_optimiser(InterKnitRepo)
-
-
 class CopyConverter(object):
     """A repository conversion tool which just performs a copy of the content.
 
@@ -4119,130 +1791,28 @@
         # trigger an assertion if not such
         repo._format.get_format_string()
         self.repo_dir = repo.bzrdir
-        pb.update('Moving repository to repository.backup')
+        pb.update(gettext('Moving repository to repository.backup'))
         self.repo_dir.transport.move('repository', 'repository.backup')
         backup_transport =  self.repo_dir.transport.clone('repository.backup')
         repo._format.check_conversion_target(self.target_format)
         self.source_repo = repo._format.open(self.repo_dir,
             _found=True,
             _override_transport=backup_transport)
-        pb.update('Creating new repository')
+        pb.update(gettext('Creating new repository'))
         converted = self.target_format.initialize(self.repo_dir,
                                                   self.source_repo.is_shared())
         converted.lock_write()
         try:
-            pb.update('Copying content')
+            pb.update(gettext('Copying content'))
             self.source_repo.copy_content_into(converted)
         finally:
             converted.unlock()
-        pb.update('Deleting old repository content')
+        pb.update(gettext('Deleting old repository content'))
         self.repo_dir.transport.delete_tree('repository.backup')
-        ui.ui_factory.note('repository converted')
+        ui.ui_factory.note(gettext('repository converted'))
         pb.finished()
 
 
-_unescape_map = {
-    'apos':"'",
-    'quot':'"',
-    'amp':'&',
-    'lt':'<',
-    'gt':'>'
-}
-
-
-def _unescaper(match, _map=_unescape_map):
-    code = match.group(1)
-    try:
-        return _map[code]
-    except KeyError:
-        if not code.startswith('#'):
-            raise
-        return unichr(int(code[1:])).encode('utf8')
-
-
-_unescape_re = None
-
-
-def _unescape_xml(data):
-    """Unescape predefined XML entities in a string of data."""
-    global _unescape_re
-    if _unescape_re is None:
-        _unescape_re = re.compile('\&([^;]*);')
-    return _unescape_re.sub(_unescaper, data)
-
-
-class _VersionedFileChecker(object):
-
-    def __init__(self, repository, text_key_references=None, ancestors=None):
-        self.repository = repository
-        self.text_index = self.repository._generate_text_key_index(
-            text_key_references=text_key_references, ancestors=ancestors)
-
-    def calculate_file_version_parents(self, text_key):
-        """Calculate the correct parents for a file version according to
-        the inventories.
-        """
-        parent_keys = self.text_index[text_key]
-        if parent_keys == [_mod_revision.NULL_REVISION]:
-            return ()
-        return tuple(parent_keys)
-
-    def check_file_version_parents(self, texts, progress_bar=None):
-        """Check the parents stored in a versioned file are correct.
-
-        It also detects file versions that are not referenced by their
-        corresponding revision's inventory.
-
-        :returns: A tuple of (wrong_parents, dangling_file_versions).
-            wrong_parents is a dict mapping {revision_id: (stored_parents,
-            correct_parents)} for each revision_id where the stored parents
-            are not correct.  dangling_file_versions is a set of (file_id,
-            revision_id) tuples for versions that are present in this versioned
-            file, but not used by the corresponding inventory.
-        """
-        local_progress = None
-        if progress_bar is None:
-            local_progress = ui.ui_factory.nested_progress_bar()
-            progress_bar = local_progress
-        try:
-            return self._check_file_version_parents(texts, progress_bar)
-        finally:
-            if local_progress:
-                local_progress.finished()
-
-    def _check_file_version_parents(self, texts, progress_bar):
-        """See check_file_version_parents."""
-        wrong_parents = {}
-        self.file_ids = set([file_id for file_id, _ in
-            self.text_index.iterkeys()])
-        # text keys is now grouped by file_id
-        n_versions = len(self.text_index)
-        progress_bar.update('loading text store', 0, n_versions)
-        parent_map = self.repository.texts.get_parent_map(self.text_index)
-        # On unlistable transports this could well be empty/error...
-        text_keys = self.repository.texts.keys()
-        unused_keys = frozenset(text_keys) - set(self.text_index)
-        for num, key in enumerate(self.text_index.iterkeys()):
-            progress_bar.update('checking text graph', num, n_versions)
-            correct_parents = self.calculate_file_version_parents(key)
-            try:
-                knit_parents = parent_map[key]
-            except errors.RevisionNotPresent:
-                # Missing text!
-                knit_parents = None
-            if correct_parents != knit_parents:
-                wrong_parents[key] = (knit_parents, correct_parents)
-        return wrong_parents, unused_keys
-
-
-def _old_get_graph(repository, revision_id):
-    """DO NOT USE. That is all. I'm serious."""
-    graph = repository.get_graph()
-    revision_graph = dict(((key, value) for key, value in
-        graph.iter_ancestry([revision_id]) if value is not None))
-    return _strip_NULL_ghosts(revision_graph)
-
-
 def _strip_NULL_ghosts(revision_graph):
     """Also don't use this. more compatibility code for unmigrated clients."""
     # Filter ghosts, and null:
@@ -4254,465 +1824,6 @@
     return revision_graph
 
 
-class StreamSink(object):
-    """An object that can insert a stream into a repository.
-
-    This interface handles the complexity of reserialising inventories and
-    revisions from different formats, and allows unidirectional insertion into
-    stacked repositories without looking for the missing basis parents
-    beforehand.
-    """
-
-    def __init__(self, target_repo):
-        self.target_repo = target_repo
-
-    def insert_stream(self, stream, src_format, resume_tokens):
-        """Insert a stream's content into the target repository.
-
-        :param src_format: a bzr repository format.
-
-        :return: a list of resume tokens and an  iterable of keys additional
-            items required before the insertion can be completed.
-        """
-        self.target_repo.lock_write()
-        try:
-            if resume_tokens:
-                self.target_repo.resume_write_group(resume_tokens)
-                is_resume = True
-            else:
-                self.target_repo.start_write_group()
-                is_resume = False
-            try:
-                # locked_insert_stream performs a commit|suspend.
-                return self._locked_insert_stream(stream, src_format,
-                    is_resume)
-            except:
-                self.target_repo.abort_write_group(suppress_errors=True)
-                raise
-        finally:
-            self.target_repo.unlock()
-
-    def _locked_insert_stream(self, stream, src_format, is_resume):
-        to_serializer = self.target_repo._format._serializer
-        src_serializer = src_format._serializer
-        new_pack = None
-        if to_serializer == src_serializer:
-            # If serializers match and the target is a pack repository, set the
-            # write cache size on the new pack.  This avoids poor performance
-            # on transports where append is unbuffered (such as
-            # RemoteTransport).  This is safe to do because nothing should read
-            # back from the target repository while a stream with matching
-            # serialization is being inserted.
-            # The exception is that a delta record from the source that should
-            # be a fulltext may need to be expanded by the target (see
-            # test_fetch_revisions_with_deltas_into_pack); but we take care to
-            # explicitly flush any buffered writes first in that rare case.
-            try:
-                new_pack = self.target_repo._pack_collection._new_pack
-            except AttributeError:
-                # Not a pack repository
-                pass
-            else:
-                new_pack.set_write_cache_size(1024*1024)
-        for substream_type, substream in stream:
-            if 'stream' in debug.debug_flags:
-                mutter('inserting substream: %s', substream_type)
-            if substream_type == 'texts':
-                self.target_repo.texts.insert_record_stream(substream)
-            elif substream_type == 'inventories':
-                if src_serializer == to_serializer:
-                    self.target_repo.inventories.insert_record_stream(
-                        substream)
-                else:
-                    self._extract_and_insert_inventories(
-                        substream, src_serializer)
-            elif substream_type == 'inventory-deltas':
-                self._extract_and_insert_inventory_deltas(
-                    substream, src_serializer)
-            elif substream_type == 'chk_bytes':
-                # XXX: This doesn't support conversions, as it assumes the
-                #      conversion was done in the fetch code.
-                self.target_repo.chk_bytes.insert_record_stream(substream)
-            elif substream_type == 'revisions':
-                # This may fallback to extract-and-insert more often than
-                # required if the serializers are different only in terms of
-                # the inventory.
-                if src_serializer == to_serializer:
-                    self.target_repo.revisions.insert_record_stream(substream)
-                else:
-                    self._extract_and_insert_revisions(substream,
-                        src_serializer)
-            elif substream_type == 'signatures':
-                self.target_repo.signatures.insert_record_stream(substream)
-            else:
-                raise AssertionError('kaboom! %s' % (substream_type,))
-        # Done inserting data, and the missing_keys calculations will try to
-        # read back from the inserted data, so flush the writes to the new pack
-        # (if this is pack format).
-        if new_pack is not None:
-            new_pack._write_data('', flush=True)
-        # Find all the new revisions (including ones from resume_tokens)
-        missing_keys = self.target_repo.get_missing_parent_inventories(
-            check_for_missing_texts=is_resume)
-        try:
-            for prefix, versioned_file in (
-                ('texts', self.target_repo.texts),
-                ('inventories', self.target_repo.inventories),
-                ('revisions', self.target_repo.revisions),
-                ('signatures', self.target_repo.signatures),
-                ('chk_bytes', self.target_repo.chk_bytes),
-                ):
-                if versioned_file is None:
-                    continue
-                # TODO: key is often going to be a StaticTuple object
-                #       I don't believe we can define a method by which
-                #       (prefix,) + StaticTuple will work, though we could
-                #       define a StaticTuple.sq_concat that would allow you to
-                #       pass in either a tuple or a StaticTuple as the second
-                #       object, so instead we could have:
-                #       StaticTuple(prefix) + key here...
-                missing_keys.update((prefix,) + key for key in
-                    versioned_file.get_missing_compression_parent_keys())
-        except NotImplementedError:
-            # cannot even attempt suspending, and missing would have failed
-            # during stream insertion.
-            missing_keys = set()
-        else:
-            if missing_keys:
-                # suspend the write group and tell the caller what we is
-                # missing. We know we can suspend or else we would not have
-                # entered this code path. (All repositories that can handle
-                # missing keys can handle suspending a write group).
-                write_group_tokens = self.target_repo.suspend_write_group()
-                return write_group_tokens, missing_keys
-        hint = self.target_repo.commit_write_group()
-        if (to_serializer != src_serializer and
-            self.target_repo._format.pack_compresses):
-            self.target_repo.pack(hint=hint)
-        return [], set()
-
-    def _extract_and_insert_inventory_deltas(self, substream, serializer):
-        target_rich_root = self.target_repo._format.rich_root_data
-        target_tree_refs = self.target_repo._format.supports_tree_reference
-        for record in substream:
-            # Insert the delta directly
-            inventory_delta_bytes = record.get_bytes_as('fulltext')
-            deserialiser = inventory_delta.InventoryDeltaDeserializer()
-            try:
-                parse_result = deserialiser.parse_text_bytes(
-                    inventory_delta_bytes)
-            except inventory_delta.IncompatibleInventoryDelta, err:
-                trace.mutter("Incompatible delta: %s", err.msg)
-                raise errors.IncompatibleRevision(self.target_repo._format)
-            basis_id, new_id, rich_root, tree_refs, inv_delta = parse_result
-            revision_id = new_id
-            parents = [key[0] for key in record.parents]
-            self.target_repo.add_inventory_by_delta(
-                basis_id, inv_delta, revision_id, parents)
-
-    def _extract_and_insert_inventories(self, substream, serializer,
-            parse_delta=None):
-        """Generate a new inventory versionedfile in target, converting data.
-
-        The inventory is retrieved from the source, (deserializing it), and
-        stored in the target (reserializing it in a different format).
-        """
-        target_rich_root = self.target_repo._format.rich_root_data
-        target_tree_refs = self.target_repo._format.supports_tree_reference
-        for record in substream:
-            # It's not a delta, so it must be a fulltext in the source
-            # serializer's format.
-            bytes = record.get_bytes_as('fulltext')
-            revision_id = record.key[0]
-            inv = serializer.read_inventory_from_string(bytes, revision_id)
-            parents = [key[0] for key in record.parents]
-            self.target_repo.add_inventory(revision_id, inv, parents)
-            # No need to keep holding this full inv in memory when the rest of
-            # the substream is likely to be all deltas.
-            del inv
-
-    def _extract_and_insert_revisions(self, substream, serializer):
-        for record in substream:
-            bytes = record.get_bytes_as('fulltext')
-            revision_id = record.key[0]
-            rev = serializer.read_revision_from_string(bytes)
-            if rev.revision_id != revision_id:
-                raise AssertionError('wtf: %s != %s' % (rev, revision_id))
-            self.target_repo.add_revision(revision_id, rev)
-
-    def finished(self):
-        if self.target_repo._format._fetch_reconcile:
-            self.target_repo.reconcile()
-
-
-class StreamSource(object):
-    """A source of a stream for fetching between repositories."""
-
-    def __init__(self, from_repository, to_format):
-        """Create a StreamSource streaming from from_repository."""
-        self.from_repository = from_repository
-        self.to_format = to_format
-        self._record_counter = RecordCounter()
-
-    def delta_on_metadata(self):
-        """Return True if delta's are permitted on metadata streams.
-
-        That is on revisions and signatures.
-        """
-        src_serializer = self.from_repository._format._serializer
-        target_serializer = self.to_format._serializer
-        return (self.to_format._fetch_uses_deltas and
-            src_serializer == target_serializer)
-
-    def _fetch_revision_texts(self, revs):
-        # fetch signatures first and then the revision texts
-        # may need to be a InterRevisionStore call here.
-        from_sf = self.from_repository.signatures
-        # A missing signature is just skipped.
-        keys = [(rev_id,) for rev_id in revs]
-        signatures = versionedfile.filter_absent(from_sf.get_record_stream(
-            keys,
-            self.to_format._fetch_order,
-            not self.to_format._fetch_uses_deltas))
-        # If a revision has a delta, this is actually expanded inside the
-        # insert_record_stream code now, which is an alternate fix for
-        # bug #261339
-        from_rf = self.from_repository.revisions
-        revisions = from_rf.get_record_stream(
-            keys,
-            self.to_format._fetch_order,
-            not self.delta_on_metadata())
-        return [('signatures', signatures), ('revisions', revisions)]
-
-    def _generate_root_texts(self, revs):
-        """This will be called by get_stream between fetching weave texts and
-        fetching the inventory weave.
-        """
-        if self._rich_root_upgrade():
-            return _mod_fetch.Inter1and2Helper(
-                self.from_repository).generate_root_texts(revs)
-        else:
-            return []
-
-    def get_stream(self, search):
-        phase = 'file'
-        revs = search.get_keys()
-        graph = self.from_repository.get_graph()
-        revs = tsort.topo_sort(graph.get_parent_map(revs))
-        data_to_fetch = self.from_repository.item_keys_introduced_by(revs)
-        text_keys = []
-        for knit_kind, file_id, revisions in data_to_fetch:
-            if knit_kind != phase:
-                phase = knit_kind
-                # Make a new progress bar for this phase
-            if knit_kind == "file":
-                # Accumulate file texts
-                text_keys.extend([(file_id, revision) for revision in
-                    revisions])
-            elif knit_kind == "inventory":
-                # Now copy the file texts.
-                from_texts = self.from_repository.texts
-                yield ('texts', from_texts.get_record_stream(
-                    text_keys, self.to_format._fetch_order,
-                    not self.to_format._fetch_uses_deltas))
-                # Cause an error if a text occurs after we have done the
-                # copy.
-                text_keys = None
-                # Before we process the inventory we generate the root
-                # texts (if necessary) so that the inventories references
-                # will be valid.
-                for _ in self._generate_root_texts(revs):
-                    yield _
-                # we fetch only the referenced inventories because we do not
-                # know for unselected inventories whether all their required
-                # texts are present in the other repository - it could be
-                # corrupt.
-                for info in self._get_inventory_stream(revs):
-                    yield info
-            elif knit_kind == "signatures":
-                # Nothing to do here; this will be taken care of when
-                # _fetch_revision_texts happens.
-                pass
-            elif knit_kind == "revisions":
-                for record in self._fetch_revision_texts(revs):
-                    yield record
-            else:
-                raise AssertionError("Unknown knit kind %r" % knit_kind)
-
-    def get_stream_for_missing_keys(self, missing_keys):
-        # missing keys can only occur when we are byte copying and not
-        # translating (because translation means we don't send
-        # unreconstructable deltas ever).
-        keys = {}
-        keys['texts'] = set()
-        keys['revisions'] = set()
-        keys['inventories'] = set()
-        keys['chk_bytes'] = set()
-        keys['signatures'] = set()
-        for key in missing_keys:
-            keys[key[0]].add(key[1:])
-        if len(keys['revisions']):
-            # If we allowed copying revisions at this point, we could end up
-            # copying a revision without copying its required texts: a
-            # violation of the requirements for repository integrity.
-            raise AssertionError(
-                'cannot copy revisions to fill in missing deltas %s' % (
-                    keys['revisions'],))
-        for substream_kind, keys in keys.iteritems():
-            vf = getattr(self.from_repository, substream_kind)
-            if vf is None and keys:
-                    raise AssertionError(
-                        "cannot fill in keys for a versioned file we don't"
-                        " have: %s needs %s" % (substream_kind, keys))
-            if not keys:
-                # No need to stream something we don't have
-                continue
-            if substream_kind == 'inventories':
-                # Some missing keys are genuinely ghosts, filter those out.
-                present = self.from_repository.inventories.get_parent_map(keys)
-                revs = [key[0] for key in present]
-                # Get the inventory stream more-or-less as we do for the
-                # original stream; there's no reason to assume that records
-                # direct from the source will be suitable for the sink.  (Think
-                # e.g. 2a -> 1.9-rich-root).
-                for info in self._get_inventory_stream(revs, missing=True):
-                    yield info
-                continue
-
-            # Ask for full texts always so that we don't need more round trips
-            # after this stream.
-            # Some of the missing keys are genuinely ghosts, so filter absent
-            # records. The Sink is responsible for doing another check to
-            # ensure that ghosts don't introduce missing data for future
-            # fetches.
-            stream = versionedfile.filter_absent(vf.get_record_stream(keys,
-                self.to_format._fetch_order, True))
-            yield substream_kind, stream
-
-    def inventory_fetch_order(self):
-        if self._rich_root_upgrade():
-            return 'topological'
-        else:
-            return self.to_format._fetch_order
-
-    def _rich_root_upgrade(self):
-        return (not self.from_repository._format.rich_root_data and
-            self.to_format.rich_root_data)
-
-    def _get_inventory_stream(self, revision_ids, missing=False):
-        from_format = self.from_repository._format
-        if (from_format.supports_chks and self.to_format.supports_chks and
-            from_format.network_name() == self.to_format.network_name()):
-            raise AssertionError(
-                "this case should be handled by GroupCHKStreamSource")
-        elif 'forceinvdeltas' in debug.debug_flags:
-            return self._get_convertable_inventory_stream(revision_ids,
-                    delta_versus_null=missing)
-        elif from_format.network_name() == self.to_format.network_name():
-            # Same format.
-            return self._get_simple_inventory_stream(revision_ids,
-                    missing=missing)
-        elif (not from_format.supports_chks and not self.to_format.supports_chks
-                and from_format._serializer == self.to_format._serializer):
-            # Essentially the same format.
-            return self._get_simple_inventory_stream(revision_ids,
-                    missing=missing)
-        else:
-            # Any time we switch serializations, we want to use an
-            # inventory-delta based approach.
-            return self._get_convertable_inventory_stream(revision_ids,
-                    delta_versus_null=missing)
-
-    def _get_simple_inventory_stream(self, revision_ids, missing=False):
-        # NB: This currently reopens the inventory weave in source;
-        # using a single stream interface instead would avoid this.
-        from_weave = self.from_repository.inventories
-        if missing:
-            delta_closure = True
-        else:
-            delta_closure = not self.delta_on_metadata()
-        yield ('inventories', from_weave.get_record_stream(
-            [(rev_id,) for rev_id in revision_ids],
-            self.inventory_fetch_order(), delta_closure))
-
-    def _get_convertable_inventory_stream(self, revision_ids,
-                                          delta_versus_null=False):
-        # The two formats are sufficiently different that there is no fast
-        # path, so we need to send just inventorydeltas, which any
-        # sufficiently modern client can insert into any repository.
-        # The StreamSink code expects to be able to
-        # convert on the target, so we need to put bytes-on-the-wire that can
-        # be converted.  That means inventory deltas (if the remote is <1.19,
-        # RemoteStreamSink will fallback to VFS to insert the deltas).
-        yield ('inventory-deltas',
-           self._stream_invs_as_deltas(revision_ids,
-                                       delta_versus_null=delta_versus_null))
-
-    def _stream_invs_as_deltas(self, revision_ids, delta_versus_null=False):
-        """Return a stream of inventory-deltas for the given rev ids.
-
-        :param revision_ids: The list of inventories to transmit
-        :param delta_versus_null: Don't try to find a minimal delta for this
-            entry, instead compute the delta versus the NULL_REVISION. This
-            effectively streams a complete inventory. Used for stuff like
-            filling in missing parents, etc.
-        """
-        from_repo = self.from_repository
-        revision_keys = [(rev_id,) for rev_id in revision_ids]
-        parent_map = from_repo.inventories.get_parent_map(revision_keys)
-        # XXX: possibly repos could implement a more efficient iter_inv_deltas
-        # method...
-        inventories = self.from_repository.iter_inventories(
-            revision_ids, 'topological')
-        format = from_repo._format
-        invs_sent_so_far = set([_mod_revision.NULL_REVISION])
-        inventory_cache = lru_cache.LRUCache(50)
-        null_inventory = from_repo.revision_tree(
-            _mod_revision.NULL_REVISION).inventory
-        # XXX: ideally the rich-root/tree-refs flags would be per-revision, not
-        # per-repo (e.g.  streaming a non-rich-root revision out of a rich-root
-        # repo back into a non-rich-root repo ought to be allowed)
-        serializer = inventory_delta.InventoryDeltaSerializer(
-            versioned_root=format.rich_root_data,
-            tree_references=format.supports_tree_reference)
-        for inv in inventories:
-            key = (inv.revision_id,)
-            parent_keys = parent_map.get(key, ())
-            delta = None
-            if not delta_versus_null and parent_keys:
-                # The caller did not ask for complete inventories and we have
-                # some parents that we can delta against.  Make a delta against
-                # each parent so that we can find the smallest.
-                parent_ids = [parent_key[0] for parent_key in parent_keys]
-                for parent_id in parent_ids:
-                    if parent_id not in invs_sent_so_far:
-                        # We don't know that the remote side has this basis, so
-                        # we can't use it.
-                        continue
-                    if parent_id == _mod_revision.NULL_REVISION:
-                        parent_inv = null_inventory
-                    else:
-                        parent_inv = inventory_cache.get(parent_id, None)
-                        if parent_inv is None:
-                            parent_inv = from_repo.get_inventory(parent_id)
-                    candidate_delta = inv._make_delta(parent_inv)
-                    if (delta is None or
-                        len(delta) > len(candidate_delta)):
-                        delta = candidate_delta
-                        basis_id = parent_id
-            if delta is None:
-                # Either none of the parents ended up being suitable, or we
-                # were asked to delta against NULL
-                basis_id = _mod_revision.NULL_REVISION
-                delta = inv._make_delta(null_inventory)
-            invs_sent_so_far.add(inv.revision_id)
-            inventory_cache[inv.revision_id] = inv
-            delta_serialized = ''.join(
-                serializer.delta_to_lines(basis_id, key[-1], delta))
-            yield versionedfile.FulltextContentFactory(
-                key, parent_keys, None, delta_serialized)
-
-
 def _iter_for_revno(repo, partial_history_cache, stop_index=None,
                     stop_revision=None):
     """Extend the partial history to include a given index
@@ -4728,9 +1839,11 @@
         it is encountered, history extension will stop.
     """
     start_revision = partial_history_cache[-1]
-    iterator = repo.iter_reverse_revision_history(start_revision)
+    graph = repo.get_graph()
+    iterator = graph.iter_lefthand_ancestry(start_revision,
+        (_mod_revision.NULL_REVISION,))
     try:
-        #skip the last revision in the list
+        # skip the last revision in the list
         iterator.next()
         while True:
             if (stop_index is not None and
@@ -4744,3 +1857,31 @@
         # No more history
         return
 
+
+class _LazyListJoin(object):
+    """An iterable yielding the contents of many lists as one list.
+
+    Each iterator made from this will reflect the current contents of the lists
+    at the time the iterator is made.
+    
+    This is used by Repository's _make_parents_provider implementation so that
+    it is safe to do::
+
+      pp = repo._make_parents_provider()      # uses a list of fallback repos
+      pp.add_fallback_repository(other_repo)  # appends to that list
+      result = pp.get_parent_map(...)
+      # The result will include revs from other_repo
+    """
+
+    def __init__(self, *list_parts):
+        self.list_parts = list_parts
+
+    def __iter__(self):
+        full_list = []
+        for list_part in self.list_parts:
+            full_list.extend(list_part)
+        return iter(full_list)
+
+    def __repr__(self):
+        return "%s.%s(%s)" % (self.__module__, self.__class__.__name__,
+                              self.list_parts)

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2010-07-15 12:37:32 +0000
+++ bzrlib/revision.py	2011-05-10 07:46:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib import deprecated_graph
 from bzrlib import bugtracker
 """)
 from bzrlib import (

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2010-06-24 20:51:59 +0000
+++ bzrlib/revisionspec.py	2011-10-05 18:11:19 +0000
@@ -15,20 +15,25 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import re
-
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import bisect
 import datetime
+
+from bzrlib import (
+    branch as _mod_branch,
+    osutils,
+    revision,
+    symbol_versioning,
+    workingtree,
+    )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib import (
     errors,
-    osutils,
+    lazy_regex,
     registry,
-    revision,
-    symbol_versioning,
     trace,
     )
 
@@ -113,9 +118,6 @@
         return RevisionInfo(branch, revno, revision_id)
 
 
-_revno_regex = None
-
-
 class RevisionSpec(object):
     """A parsed revision specification."""
 
@@ -166,11 +168,6 @@
                          spectype.__name__, spec)
             return spectype(spec, _internal=True)
         else:
-            for spectype in SPEC_TYPES:
-                if spec.startswith(spectype.prefix):
-                    trace.mutter('Returning RevisionSpec %s for %s',
-                                 spectype.__name__, spec)
-                    return spectype(spec, _internal=True)
             # Otherwise treat it as a DWIM, build the RevisionSpec object and
             # wait for _match_on to be called.
             return RevisionSpec_dwim(spec, _internal=True)
@@ -212,7 +209,15 @@
     def in_history(self, branch):
         if branch:
             if self.wants_revision_history:
-                revs = branch.revision_history()
+                # TODO: avoid looking at all of history
+                branch.lock_read()
+                try:
+                    graph = branch.repository.get_graph()
+                    revs = list(graph.iter_lefthand_ancestry(
+                        branch.last_revision(), [revision.NULL_REVISION]))
+                finally:
+                    branch.unlock()
+                revs.reverse()
             else:
                 revs = None
         else:
@@ -302,6 +307,11 @@
     # each revspec we try.
     wants_revision_history = False
 
+    _revno_regex = lazy_regex.lazy_compile(r'^(?:(\d+(\.\d+)*)|-\d+)(:.*)?$')
+
+    # The revspecs to try
+    _possible_revspecs = []
+
     def _try_spectype(self, rstype, branch):
         rs = rstype(self.spec, _internal=True)
         # Hit in_history to find out if it exists, or we need to try the
@@ -312,16 +322,21 @@
         """Run the lookup and see what we can get."""
 
         # First, see if it's a revno
-        global _revno_regex
-        if _revno_regex is None:
-            _revno_regex = re.compile(r'^(?:(\d+(\.\d+)*)|-\d+)(:.*)?$')
-        if _revno_regex.match(self.spec) is not None:
+        if self._revno_regex.match(self.spec) is not None:
             try:
                 return self._try_spectype(RevisionSpec_revno, branch)
             except RevisionSpec_revno.dwim_catchable_exceptions:
                 pass
 
         # Next see what has been registered
+        for objgetter in self._possible_revspecs:
+            rs_class = objgetter.get_obj()
+            try:
+                return self._try_spectype(rs_class, branch)
+            except rs_class.dwim_catchable_exceptions:
+                pass
+
+        # Try the old (deprecated) dwim list:
         for rs_class in dwim_revspecs:
             try:
                 return self._try_spectype(rs_class, branch)
@@ -333,6 +348,24 @@
         # really relevant.
         raise errors.InvalidRevisionSpec(self.spec, branch)
 
+    @classmethod
+    def append_possible_revspec(cls, revspec):
+        """Append a possible DWIM revspec.
+
+        :param revspec: Revision spec to try.
+        """
+        cls._possible_revspecs.append(registry._ObjectGetter(revspec))
+
+    @classmethod
+    def append_possible_lazy_revspec(cls, module_name, member_name):
+        """Append a possible lazily loaded DWIM revspec.
+
+        :param module_name: Name of the module with the revspec
+        :param member_name: Name of the revspec within the module
+        """
+        cls._possible_revspecs.append(
+            registry._LazyObjectGetter(module_name, member_name))
+
 
 class RevisionSpec_revno(RevisionSpec):
     """Selects a revision using a number."""
@@ -444,7 +477,14 @@
 
 
 
-class RevisionSpec_revid(RevisionSpec):
+class RevisionIDSpec(RevisionSpec):
+
+    def _match_on(self, branch, revs):
+        revision_id = self.as_revision_id(branch)
+        return RevisionInfo.from_revision_id(branch, revision_id, revs)
+
+
+class RevisionSpec_revid(RevisionIDSpec):
     """Selects a revision using the revision id."""
 
     help_txt = """Selects a revision using the revision id.
@@ -459,14 +499,10 @@
 
     prefix = 'revid:'
 
-    def _match_on(self, branch, revs):
+    def _as_revision_id(self, context_branch):
         # self.spec comes straight from parsing the command line arguments,
         # so we expect it to be a Unicode string. Switch it to the internal
         # representation.
-        revision_id = osutils.safe_revision_id(self.spec, warn=False)
-        return RevisionInfo.from_revision_id(branch, revision_id, revs)
-
-    def _as_revision_id(self, context_branch):
         return osutils.safe_revision_id(self.spec, warn=False)
 
 
@@ -658,7 +694,7 @@
                                    August 14th, 2006 at 5:10pm.
     """
     prefix = 'date:'
-    _date_re = re.compile(
+    _date_regex = lazy_regex.lazy_compile(
             r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
             r'(,|T)?\s*'
             r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
@@ -682,7 +718,7 @@
         elif self.spec.lower() == 'tomorrow':
             dt = today + datetime.timedelta(days=1)
         else:
-            m = self._date_re.match(self.spec)
+            m = self._date_regex.match(self.spec)
             if not m or (not m.group('date') and not m.group('time')):
                 raise errors.InvalidRevisionSpec(self.user_spec,
                                                  branch, 'invalid date')
@@ -883,7 +919,8 @@
             location_type = 'parent branch'
         if submit_location is None:
             raise errors.NoSubmitBranch(branch)
-        trace.note('Using %s %s', location_type, submit_location)
+        trace.note(gettext('Using {0} {1}').format(location_type,
+                                                        submit_location))
         return submit_location
 
     def _match_on(self, branch, revs):
@@ -896,16 +933,83 @@
             self._get_submit_location(context_branch))
 
 
+class RevisionSpec_annotate(RevisionIDSpec):
+
+    prefix = 'annotate:'
+
+    help_txt = """Select the revision that last modified the specified line.
+
+    Select the revision that last modified the specified line.  Line is
+    specified as path:number.  Path is a relative path to the file.  Numbers
+    start at 1, and are relative to the current version, not the last-
+    committed version of the file.
+    """
+
+    def _raise_invalid(self, numstring, context_branch):
+        raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
+            'No such line: %s' % numstring)
+
+    def _as_revision_id(self, context_branch):
+        path, numstring = self.spec.rsplit(':', 1)
+        try:
+            index = int(numstring) - 1
+        except ValueError:
+            self._raise_invalid(numstring, context_branch)
+        tree, file_path = workingtree.WorkingTree.open_containing(path)
+        tree.lock_read()
+        try:
+            file_id = tree.path2id(file_path)
+            if file_id is None:
+                raise errors.InvalidRevisionSpec(self.user_spec,
+                    context_branch, "File '%s' is not versioned." %
+                    file_path)
+            revision_ids = [r for (r, l) in tree.annotate_iter(file_id)]
+        finally:
+            tree.unlock()
+        try:
+            revision_id = revision_ids[index]
+        except IndexError:
+            self._raise_invalid(numstring, context_branch)
+        if revision_id == revision.CURRENT_REVISION:
+            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
+                'Line %s has not been committed.' % numstring)
+        return revision_id
+
+
+class RevisionSpec_mainline(RevisionIDSpec):
+
+    help_txt = """Select mainline revision that merged the specified revision.
+
+    Select the revision that merged the specified revision into mainline.
+    """
+
+    prefix = 'mainline:'
+
+    def _as_revision_id(self, context_branch):
+        revspec = RevisionSpec.from_string(self.spec)
+        if revspec.get_branch() is None:
+            spec_branch = context_branch
+        else:
+            spec_branch = _mod_branch.Branch.open(revspec.get_branch())
+        revision_id = revspec.as_revision_id(spec_branch)
+        graph = context_branch.repository.get_graph()
+        result = graph.find_lefthand_merger(revision_id,
+                                            context_branch.last_revision())
+        if result is None:
+            raise errors.InvalidRevisionSpec(self.user_spec, context_branch)
+        return result
+
+
 # The order in which we want to DWIM a revision spec without any prefix.
 # revno is always tried first and isn't listed here, this is used by
 # RevisionSpec_dwim._match_on
-dwim_revspecs = [
-    RevisionSpec_tag, # Let's try for a tag
-    RevisionSpec_revid, # Maybe it's a revid?
-    RevisionSpec_date, # Perhaps a date?
-    RevisionSpec_branch, # OK, last try, maybe it's a branch
-    ]
+dwim_revspecs = symbol_versioning.deprecated_list(
+    symbol_versioning.deprecated_in((2, 4, 0)), "dwim_revspecs", [])
 
+RevisionSpec_dwim.append_possible_revspec(RevisionSpec_tag)
+RevisionSpec_dwim.append_possible_revspec(RevisionSpec_revid)
+RevisionSpec_dwim.append_possible_revspec(RevisionSpec_date)
+RevisionSpec_dwim.append_possible_revspec(RevisionSpec_branch)
 
 revspec_registry = registry.Registry()
 def _register_revspec(revspec):
@@ -920,8 +1024,5 @@
 _register_revspec(RevisionSpec_ancestor)
 _register_revspec(RevisionSpec_branch)
 _register_revspec(RevisionSpec_submit)
-
-# classes in this list should have a "prefix" attribute, against which
-# string specs are matched
-SPEC_TYPES = symbol_versioning.deprecated_list(
-    symbol_versioning.deprecated_in((1, 12, 0)), "SPEC_TYPES", [])
+_register_revspec(RevisionSpec_annotate)
+_register_revspec(RevisionSpec_mainline)

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2010-05-06 23:41:35 +0000
+++ bzrlib/revisiontree.py	2011-08-29 19:06:57 +0000
@@ -31,17 +31,15 @@
     File text can be retrieved from the text store.
     """
 
-    def __init__(self, branch, inv, revision_id):
-        # for compatability the 'branch' parameter has not been renamed to
-        # repository at this point. However, we should change RevisionTree's
-        # construction to always be via Repository and not via direct
-        # construction - this will mean that we can change the constructor
-        # with much less chance of breaking client code.
-        self._repository = branch
-        self._inventory = inv
+    def __init__(self, repository, revision_id):
+        self._repository = repository
         self._revision_id = revision_id
         self._rules_searcher = None
 
+    def has_versioned_directories(self):
+        """See `Tree.has_versioned_directories`."""
+        return self._repository._format.supports_versioned_directories
+
     def supports_tree_reference(self):
         return getattr(self._repository._format, "supports_tree_reference",
             False)
@@ -62,6 +60,10 @@
         """Return the revision id associated with this tree."""
         return self._revision_id
 
+    def get_file_revision(self, file_id, path=None):
+        """Return the revision id in which a file was last changed."""
+        raise NotImplementedError(self.get_file_revision)
+
     def get_file_text(self, file_id, path=None):
         _, content = list(self.iter_files_bytes([(file_id, None)]))[0]
         return ''.join(content)
@@ -69,28 +71,43 @@
     def get_file(self, file_id, path=None):
         return StringIO(self.get_file_text(file_id))
 
-    def iter_files_bytes(self, desired_files):
-        """See Tree.iter_files_bytes.
-
-        This version is implemented on top of Repository.extract_files_bytes"""
-        repo_desired_files = [(f, self.inventory[f].revision, i)
-                              for f, i in desired_files]
+    def is_locked(self):
+        return self._repository.is_locked()
+
+    def lock_read(self):
+        self._repository.lock_read()
+        return self
+
+    def __repr__(self):
+        return '<%s instance at %x, rev_id=%r>' % (
+            self.__class__.__name__, id(self), self._revision_id)
+
+    def unlock(self):
+        self._repository.unlock()
+
+    def _get_rules_searcher(self, default_searcher):
+        """See Tree._get_rules_searcher."""
+        if self._rules_searcher is None:
+            self._rules_searcher = super(RevisionTree,
+                self)._get_rules_searcher(default_searcher)
+        return self._rules_searcher
+
+
+class InventoryRevisionTree(RevisionTree,tree.InventoryTree):
+
+    def __init__(self, repository, inv, revision_id):
+        RevisionTree.__init__(self, repository, revision_id)
+        self._inventory = inv
+
+    def get_file_mtime(self, file_id, path=None):
+        ie = self._inventory[file_id]
         try:
-            for result in self._repository.iter_files_bytes(repo_desired_files):
-                yield result
-        except errors.RevisionNotPresent, e:
-            raise errors.NoSuchFile(e.revision_id)
-
-    def annotate_iter(self, file_id,
-                      default_revision=revision.CURRENT_REVISION):
-        """See Tree.annotate_iter"""
-        text_key = (file_id, self.inventory[file_id].revision)
-        annotator = self._repository.texts.get_annotator()
-        annotations = annotator.annotate_flat(text_key)
-        return [(key[-1], line) for key, line in annotations]
+            revision = self._repository.get_revision(ie.revision)
+        except errors.NoSuchRevision:
+            raise errors.FileTimestampUnavailable(self.id2path(file_id))
+        return revision.timestamp
 
     def get_file_size(self, file_id):
-        """See Tree.get_file_size"""
         return self._inventory[file_id].text_size
 
     def get_file_sha1(self, file_id, path=None, stat_value=None):
@@ -99,18 +116,14 @@
             return ie.text_sha1
         return None
 
-    def get_file_mtime(self, file_id, path=None):
+    def get_file_revision(self, file_id, path=None):
         ie = self._inventory[file_id]
-        try:
-            revision = self._repository.get_revision(ie.revision)
-        except errors.NoSuchRevision:
-            raise errors.FileTimestampUnavailable(self.id2path(file_id))
-        return revision.timestamp
+        return ie.revision
 
     def is_executable(self, file_id, path=None):
         ie = self._inventory[file_id]
         if ie.kind != "file":
-            return None
+            return False
         return ie.executable
 
     def has_filename(self, filename):
@@ -133,7 +146,7 @@
         for path, entry in entries:
             yield path, 'V', entry.kind, entry.file_id, entry
 
-    def get_symlink_target(self, file_id):
+    def get_symlink_target(self, file_id, path=None):
         ie = self._inventory[file_id]
         # Inventories store symlink targets in unicode
         return ie.symlink_target
@@ -170,24 +183,6 @@
     def _file_size(self, entry, stat_value):
         return entry.text_size
 
-    def _get_ancestors(self, default_revision):
-        return set(self._repository.get_ancestry(self._revision_id,
-                                                 topo_sorted=False))
-
-    def is_locked(self):
-        return self._repository.is_locked()
-
-    def lock_read(self):
-        self._repository.lock_read()
-        return self
-
-    def __repr__(self):
-        return '<%s instance at %x, rev_id=%r>' % (
-            self.__class__.__name__, id(self), self._revision_id)
-
-    def unlock(self):
-        self._repository.unlock()
-
     def walkdirs(self, prefix=""):
         _directory = 'directory'
         inv = self.inventory
@@ -217,12 +212,25 @@
                 if dir[2] == _directory:
                     pending.append(dir)
 
-    def _get_rules_searcher(self, default_searcher):
-        """See Tree._get_rules_searcher."""
-        if self._rules_searcher is None:
-            self._rules_searcher = super(RevisionTree,
-                self)._get_rules_searcher(default_searcher)
-        return self._rules_searcher
+    def iter_files_bytes(self, desired_files):
+        """See Tree.iter_files_bytes.
+
+        This version is implemented on top of Repository.extract_files_bytes"""
+        repo_desired_files = [(f, self.get_file_revision(f), i)
+                              for f, i in desired_files]
+        try:
+            for result in self._repository.iter_files_bytes(repo_desired_files):
+                yield result
+        except errors.RevisionNotPresent, e:
+            raise errors.NoSuchFile(e.revision_id)
+
+    def annotate_iter(self, file_id,
+                      default_revision=revision.CURRENT_REVISION):
+        """See Tree.annotate_iter"""
+        text_key = (file_id, self.get_file_revision(file_id))
+        annotator = self._repository.texts.get_annotator()
+        annotations = annotator.annotate_flat(text_key)
+        return [(key[-1], line) for key, line in annotations]
 
 
 class InterCHKRevisionTree(tree.InterTree):

=== modified file 'bzrlib/rules.py'
--- bzrlib/rules.py	2010-02-02 06:30:43 +0000
+++ bzrlib/rules.py	2011-04-19 07:52:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -64,6 +64,15 @@
         """
         raise NotImplementedError(self.get_selected_items)
 
+    def get_single_value(self, path, preference_name):
+        """Get a single preference for a single file.
+        
+        :returns: The string preference value, or None.
+        """
+        for key, value in self.get_selected_items(path, [preference_name]):
+            return value
+        return None
+
 
 class _IniBasedRulesSearcher(_RulesSearcher):
 
@@ -74,8 +83,7 @@
 
         :param inifile: the name of the file or a sequence of lines.
         """
-        options = {'encoding': 'utf-8'}
-        self._cfg = configobj.ConfigObj(inifile, options=options)
+        self._cfg = configobj.ConfigObj(inifile, encoding='utf-8')
         sections = self._cfg.keys()
         patterns = []
         self.pattern_to_section = {}

=== modified file 'bzrlib/send.py'
--- bzrlib/send.py	2010-04-22 14:18:17 +0000
+++ bzrlib/send.py	2011-10-11 12:01:51 +0000
@@ -19,12 +19,13 @@
 import time
 
 from bzrlib import (
-    bzrdir,
+    controldir,
     errors,
     osutils,
     registry,
     trace,
     )
+from bzrlib.i18n import gettext
 from bzrlib.branch import (
     Branch,
     )
@@ -39,7 +40,8 @@
 def send(submit_branch, revision, public_branch, remember, format,
          no_bundle, no_patch, output, from_, mail_to, message, body,
          to_file, strict=None):
-    tree, branch = bzrdir.BzrDir.open_containing_tree_or_branch(from_)[:2]
+    tree, branch = controldir.ControlDir.open_containing_tree_or_branch(
+        from_)[:2]
     # we may need to write data into branch's repository to calculate
     # the data to send.
     branch.lock_write()
@@ -51,30 +53,31 @@
             mail_client = config.get_mail_client()
             if (not getattr(mail_client, 'supports_body', False)
                 and body is not None):
-                raise errors.BzrCommandError(
-                    'Mail client "%s" does not support specifying body' %
+                raise errors.BzrCommandError(gettext(
+                    'Mail client "%s" does not support specifying body') %
                     mail_client.__class__.__name__)
         if remember and submit_branch is None:
-            raise errors.BzrCommandError(
-                '--remember requires a branch to be specified.')
+            raise errors.BzrCommandError(gettext(
+                '--remember requires a branch to be specified.'))
         stored_submit_branch = branch.get_submit_branch()
         remembered_submit_branch = None
         if submit_branch is None:
             submit_branch = stored_submit_branch
             remembered_submit_branch = "submit"
         else:
-            if stored_submit_branch is None or remember:
+            # Remembers if asked explicitly or no previous location is set
+            if remember or (remember is None and stored_submit_branch is None):
                 branch.set_submit_branch(submit_branch)
         if submit_branch is None:
             submit_branch = branch.get_parent()
             remembered_submit_branch = "parent"
         if submit_branch is None:
-            raise errors.BzrCommandError('No submit branch known or'
-                                         ' specified')
+            raise errors.BzrCommandError(gettext('No submit branch known or'
+                                         ' specified'))
         if remembered_submit_branch is not None:
-            trace.note('Using saved %s location "%s" to determine what '
-                       'changes to submit.', remembered_submit_branch,
-                       submit_branch)
+            trace.note(gettext('Using saved {0} location "{1}" to determine '
+                       'what changes to submit.').format(
+                                    remembered_submit_branch, submit_branch))
 
         if mail_to is None or format is None:
             # TODO: jam 20090716 we open the submit_branch here, but we *don't*
@@ -89,23 +92,25 @@
                 try:
                     format = format_registry.get(formatname)
                 except KeyError:
-                    raise errors.BzrCommandError("No such send format '%s'." % 
+                    raise errors.BzrCommandError(gettext("No such send format '%s'.") % 
                                                  formatname)
 
         stored_public_branch = branch.get_public_branch()
         if public_branch is None:
             public_branch = stored_public_branch
-        elif stored_public_branch is None or remember:
+        # Remembers if asked explicitly or no previous location is set
+        elif (remember
+              or (remember is None and stored_public_branch is None)):
             branch.set_public_branch(public_branch)
         if no_bundle and public_branch is None:
-            raise errors.BzrCommandError('No public branch specified or'
-                                         ' known')
+            raise errors.BzrCommandError(gettext('No public branch specified or'
+                                         ' known'))
         base_revision_id = None
         revision_id = None
         if revision is not None:
             if len(revision) > 2:
-                raise errors.BzrCommandError('bzr send takes '
-                    'at most two one revision identifiers')
+                raise errors.BzrCommandError(gettext('bzr send takes '
+                    'at most two one revision identifiers'))
             revision_id = revision[-1].as_revision_id(branch)
             if len(revision) == 2:
                 base_revision_id = revision[0].as_revision_id(branch)
@@ -117,7 +122,7 @@
                     more_warning='Uncommitted changes will not be sent.')
             revision_id = branch.last_revision()
         if revision_id == NULL_REVISION:
-            raise errors.BzrCommandError('No revisions to submit.')
+            raise errors.BzrCommandError(gettext('No revisions to submit.'))
         if format is None:
             format = format_registry.get()
         directive = format(branch, revision_id, submit_branch,
@@ -128,8 +133,8 @@
         else:
             if directive.multiple_output_files:
                 if output == '-':
-                    raise errors.BzrCommandError('- not supported for '
-                        'merge directives that use more than one output file.')
+                    raise errors.BzrCommandError(gettext('- not supported for '
+                        'merge directives that use more than one output file.'))
                 if not os.path.exists(output):
                     os.mkdir(output, 0755)
                 for (filename, lines) in directive.to_files():
@@ -170,8 +175,8 @@
         if not no_patch:
             patch_type = 'bundle'
         else:
-            raise errors.BzrCommandError('Format 0.9 does not'
-                ' permit bundle with no patch')
+            raise errors.BzrCommandError(gettext('Format 0.9 does not'
+                ' permit bundle with no patch'))
     else:
         if not no_patch:
             patch_type = 'diff'

=== modified file 'bzrlib/serializer.py'
--- bzrlib/serializer.py	2010-02-17 17:11:16 +0000
+++ bzrlib/serializer.py	2011-02-19 15:23:08 +0000
@@ -93,7 +93,6 @@
 
 
 format_registry = SerializerRegistry()
-format_registry.register_lazy('4', 'bzrlib.xml4', 'serializer_v4')
 format_registry.register_lazy('5', 'bzrlib.xml5', 'serializer_v5')
 format_registry.register_lazy('6', 'bzrlib.xml6', 'serializer_v6')
 format_registry.register_lazy('7', 'bzrlib.xml7', 'serializer_v7')

=== modified file 'bzrlib/shelf.py'
--- bzrlib/shelf.py	2010-04-30 11:03:59 +0000
+++ bzrlib/shelf.py	2011-07-06 21:37:16 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 import errno
 import re
 
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
 from bzrlib import (
     bencode,
     errors,
@@ -26,6 +28,7 @@
     pack,
     transform,
 )
+""")
 
 
 class ShelfCreator(object):
@@ -62,7 +65,8 @@
         """Iterable of tuples describing shelvable changes.
 
         As well as generating the tuples, this updates several members.
-        Tuples may be:
+        Tuples may be::
+
            ('add file', file_id, work_kind, work_path)
            ('delete file', file_id, target_kind, target_path)
            ('rename', file_id, target_path, work_path)
@@ -78,6 +82,9 @@
             # when a tree root was deleted / renamed.
             if kind[0] is None and names[1] == '':
                 continue
+            # Also don't shelve deletion of tree root.
+            if kind[1] is None and names[0] == '':
+                continue
             if kind[0] is None or versioned[0] == False:
                 self.creation[file_id] = (kind[1], names[1], parents[1],
                                           versioned)

=== modified file 'bzrlib/shelf_ui.py'
--- bzrlib/shelf_ui.py	2010-07-16 13:29:07 +0000
+++ bzrlib/shelf_ui.py	2011-10-05 18:40:53 +0000
@@ -16,6 +16,7 @@
 
 
 from cStringIO import StringIO
+import os
 import shutil
 import sys
 import tempfile
@@ -34,7 +35,7 @@
     ui,
     workingtree,
 )
-
+from bzrlib.i18n import gettext
 
 class UseEditor(Exception):
     """Use an editor instead of selecting hunks."""
@@ -42,17 +43,17 @@
 
 class ShelfReporter(object):
 
-    vocab = {'add file': 'Shelve adding file "%(path)s"?',
-             'binary': 'Shelve binary changes?',
-             'change kind': 'Shelve changing "%s" from %(other)s'
-             ' to %(this)s?',
-             'delete file': 'Shelve removing file "%(path)s"?',
-             'final': 'Shelve %d change(s)?',
-             'hunk': 'Shelve?',
-             'modify target': 'Shelve changing target of'
-             ' "%(path)s" from "%(other)s" to "%(this)s"?',
-             'rename': 'Shelve renaming "%(other)s" =>'
-                        ' "%(this)s"?'
+    vocab = {'add file': gettext('Shelve adding file "%(path)s"?'),
+             'binary': gettext('Shelve binary changes?'),
+             'change kind': gettext('Shelve changing "%s" from %(other)s'
+             ' to %(this)s?'),
+             'delete file': gettext('Shelve removing file "%(path)s"?'),
+             'final': gettext('Shelve %d change(s)?'),
+             'hunk': gettext('Shelve?'),
+             'modify target': gettext('Shelve changing target of'
+             ' "%(path)s" from "%(other)s" to "%(this)s"?'),
+             'rename': gettext('Shelve renaming "%(other)s" =>'
+                        ' "%(this)s"?')
              }
 
     invert_diff = False
@@ -66,15 +67,15 @@
 
     def shelved_id(self, shelf_id):
         """Report the id changes were shelved to."""
-        trace.note('Changes shelved with id "%d".' % shelf_id)
+        trace.note(gettext('Changes shelved with id "%d".') % shelf_id)
 
     def changes_destroyed(self):
         """Report that changes were made without shelving."""
-        trace.note('Selected changes destroyed.')
+        trace.note(gettext('Selected changes destroyed.'))
 
     def selected_changes(self, transform):
         """Report the changes that were selected."""
-        trace.note("Selected changes:")
+        trace.note(gettext("Selected changes:"))
         changes = transform.iter_changes()
         delta.report_changes(changes, self.delta_reporter)
 
@@ -94,16 +95,16 @@
 
 class ApplyReporter(ShelfReporter):
 
-    vocab = {'add file': 'Delete file "%(path)s"?',
-             'binary': 'Apply binary changes?',
-             'change kind': 'Change "%(path)s" from %(this)s'
-             ' to %(other)s?',
-             'delete file': 'Add file "%(path)s"?',
-             'final': 'Apply %d change(s)?',
-             'hunk': 'Apply change?',
-             'modify target': 'Change target of'
-             ' "%(path)s" from "%(this)s" to "%(other)s"?',
-             'rename': 'Rename "%(this)s" => "%(other)s"?',
+    vocab = {'add file': gettext('Delete file "%(path)s"?'),
+             'binary': gettext('Apply binary changes?'),
+             'change kind': gettext('Change "%(path)s" from %(this)s'
+             ' to %(other)s?'),
+             'delete file': gettext('Add file "%(path)s"?'),
+             'final': gettext('Apply %d change(s)?'),
+             'hunk': gettext('Apply change?'),
+             'modify target': gettext('Change target of'
+             ' "%(path)s" from "%(this)s" to "%(other)s"?'),
+             'rename': gettext('Rename "%(this)s" => "%(other)s"?'),
              }
 
     invert_diff = True
@@ -154,7 +155,7 @@
 
     @classmethod
     def from_args(klass, diff_writer, revision=None, all=False, file_list=None,
-                  message=None, directory='.', destroy=False):
+                  message=None, directory=None, destroy=False):
         """Create a shelver from commandline arguments.
 
         The returned shelver wil have a work_tree that is locked and should
@@ -168,6 +169,10 @@
         :param destroy: Change the working tree without storing the shelved
             changes.
         """
+        if directory is None:
+            directory = u'.'
+        elif file_list:
+            file_list = [osutils.pathjoin(directory, f) for f in file_list]
         tree, path = workingtree.WorkingTree.open_containing(directory)
         # Ensure that tree is locked for the lifetime of target_tree, as
         # target tree may be reading from the same dirstate.
@@ -247,25 +252,10 @@
         diff_file.seek(0)
         return patches.parse_patch(diff_file)
 
-    def prompt(self, message):
-        """Prompt the user for a character.
-
-        :param message: The message to prompt a user with.
-        :return: A character.
-        """
-        if not sys.stdin.isatty():
-            # Since there is no controlling terminal we will hang when trying
-            # to prompt the user, better abort now.  See
-            # https://code.launchpad.net/~bialix/bzr/shelve-no-tty/+merge/14905
-            # for more context.
-            raise errors.BzrError("You need a controlling terminal.")
-        sys.stdout.write(message)
-        char = osutils.getchar()
-        sys.stdout.write("\r" + ' ' * len(message) + '\r')
-        sys.stdout.flush()
-        return char
-
-    def prompt_bool(self, question, long=False, allow_editor=False):
+    def prompt(self, message, choices, default):
+        return ui.ui_factory.choose(message, choices, default=default)
+
+    def prompt_bool(self, question, allow_editor=False):
         """Prompt the user with a yes/no question.
 
         This may be overridden by self.auto.  It may also *set* self.auto.  It
@@ -275,16 +265,19 @@
         """
         if self.auto:
             return True
-        editor_string = ''
-        if long:
-            if allow_editor:
-                editor_string = '(E)dit manually, '
-            prompt = ' [(y)es, (N)o, %s(f)inish, or (q)uit]' % editor_string
+        alternatives_chars = 'yn'
+        alternatives = '&yes\n&No'
+        if allow_editor:
+            alternatives_chars += 'e'
+            alternatives += '\n&edit manually'
+        alternatives_chars += 'fq'
+        alternatives += '\n&finish\n&quit'
+        choice = self.prompt(question, alternatives, 1)
+        if choice is None:
+            # EOF.
+            char = 'n'
         else:
-            if allow_editor:
-                editor_string = 'e'
-            prompt = ' [yN%sfq?]' % editor_string
-        char = self.prompt(question + prompt)
+            char = alternatives_chars[choice]
         if char == 'y':
             return True
         elif char == 'e' and allow_editor:
@@ -292,8 +285,6 @@
         elif char == 'f':
             self.auto = True
             return True
-        elif char == '?':
-            return self.prompt_bool(question, long=True)
         if char == 'q':
             raise errors.UserAbort()
         else:
@@ -408,7 +399,7 @@
             else:
                 shelf_id = manager.last_shelf()
                 if shelf_id is None:
-                    raise errors.BzrCommandError('No changes are shelved.')
+                    raise errors.BzrCommandError(gettext('No changes are shelved.'))
             apply_changes = True
             delete_shelf = True
             read_shelf = True
@@ -466,11 +457,11 @@
         cleanups = [self.tree.unlock]
         try:
             if self.read_shelf:
-                trace.note('Using changes with id "%d".' % self.shelf_id)
+                trace.note(gettext('Using changes with id "%d".') % self.shelf_id)
                 unshelver = self.manager.get_unshelver(self.shelf_id)
                 cleanups.append(unshelver.finalize)
                 if unshelver.message is not None:
-                    trace.note('Message: %s' % unshelver.message)
+                    trace.note(gettext('Message: %s') % unshelver.message)
                 change_reporter = delta._ChangeReporter()
                 merger = unshelver.make_merger(None)
                 merger.change_reporter = change_reporter
@@ -482,7 +473,7 @@
                     self.show_changes(merger)
             if self.delete_shelf:
                 self.manager.delete_shelf(self.shelf_id)
-                trace.note('Deleted changes with id "%d".' % self.shelf_id)
+                trace.note(gettext('Deleted changes with id "%d".') % self.shelf_id)
         finally:
             for cleanup in reversed(cleanups):
                 cleanup()
@@ -493,7 +484,7 @@
         tt = tree_merger.make_preview_transform()
         new_tree = tt.get_preview_tree()
         if self.write_diff_to is None:
-            self.write_diff_to = ui.ui_factory.make_output_stream()
+            self.write_diff_to = ui.ui_factory.make_output_stream(encoding_type='exact')
         path_encoding = osutils.get_diff_header_encoding()
         diff.show_diff_trees(merger.this_tree, new_tree, self.write_diff_to,
             path_encoding=path_encoding)

=== modified file 'bzrlib/smart/__init__.py'
--- bzrlib/smart/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/smart/__init__.py	2011-09-26 07:56:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006,2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,25 +33,11 @@
 
 """
 
-# TODO: _translate_error should be on the client, not the transport because
-#     error coding is wire protocol specific.
-
 # TODO: A plain integer from query_version is too simple; should give some
 # capabilities too?
 
-# TODO: Server should probably catch exceptions within itself and send them
-# back across the network.  (But shouldn't catch KeyboardInterrupt etc)
-# Also needs to somehow report protocol errors like bad requests.  Need to
-# consider how we'll handle error reporting, e.g. if we get halfway through a
-# bulk transfer and then something goes wrong.
-
 # TODO: Make each request and response self-validatable, e.g. with checksums.
 #
-# TODO: get/put objects could be changed to gradually read back the data as it
-# comes across the network
-#
-# TODO: What should the server do if it hits an error and has to terminate?
-#
 # TODO: is it useful to allow multiple chunks in the bulk data?
 #
 # TODO: If we get an exception during transmission of bulk data we can't just
@@ -60,44 +46,6 @@
 #   chunk, that indicates it is another chunk. Then you can send an 'error'
 #   chunk as long as you finish the previous chunk.
 #
-# TODO: Clone method on Transport; should work up towards parent directory;
-# unclear how this should be stored or communicated to the server... maybe
-# just pass it on all relevant requests?
-#
-# TODO: Better name than clone() for changing between directories.  How about
-# open_dir or change_dir or chdir?
-#
-# TODO: Is it really good to have the notion of current directory within the
-# connection?  Perhaps all Transports should factor out a common connection
-# from the thing that has the directory context?
-#
-# TODO: The server that manages a connection should be quite small and retain
-# minimum state because each of the requests are supposed to be stateless.
-# Then we can write another implementation that maps to http.
-#
-# TODO: What to do when a client connection is garbage collected?  Maybe just
-# abruptly drop the connection?
-#
-# TODO: Server in some cases will need to restrict access to files outside of
-# a particular root directory.  LocalTransport doesn't do anything to stop you
-# ascending above the base directory, so we need to prevent paths
-# containing '..' in either the server or transport layers.  (Also need to
-# consider what happens if someone creates a symlink pointing outside the
-# directory tree...)
-#
-# TODO: Server should rebase absolute paths coming across the network to put
-# them under the virtual root, if one is in use.  LocalTransport currently
-# doesn't do that; if you give it an absolute path it just uses it.
-#
-# XXX: Arguments can't contain newlines or ascii; possibly we should e.g.
-# urlescape them instead.  Indeed possibly this should just literally be
-# http-over-ssh.
-#
-# TODO: Probably want some way for server commands to gradually produce body
-# data rather than passing it as a string; they could perhaps pass an
-# iterator-like callback that will gradually yield data; it probably needs a
-# close() method that will always be closed to do any necessary cleanup.
-
 
 # Promote some attributes from submodules into this namespace
 from bzrlib.smart.request import SmartServerRequestHandler

=== modified file 'bzrlib/smart/branch.py'
--- bzrlib/smart/branch.py	2010-05-13 16:17:54 +0000
+++ bzrlib/smart/branch.py	2011-10-14 13:56:45 +0000
@@ -20,8 +20,9 @@
 from bzrlib import (
     bencode,
     errors,
+    revision as _mod_revision,
     )
-from bzrlib.bzrdir import BzrDir
+from bzrlib.controldir import ControlDir
 from bzrlib.smart.request import (
     FailedSmartServerResponse,
     SmartServerRequest,
@@ -45,10 +46,10 @@
         :return: A SmartServerResponse from self.do_with_branch().
         """
         transport = self.transport_from_client_path(path)
-        bzrdir = BzrDir.open_from_transport(transport)
-        if bzrdir.get_branch_reference() is not None:
+        controldir = ControlDir.open_from_transport(transport)
+        if controldir.get_branch_reference() is not None:
             raise errors.NotBranchError(transport.base)
-        branch = bzrdir.open_branch(ignore_fallbacks=True)
+        branch = controldir.open_branch(ignore_fallbacks=True)
         return self.do_with_branch(branch, *args)
 
 
@@ -142,6 +143,20 @@
             self.branch.unlock()
 
 
+class SmartServerBranchHeadsToFetch(SmartServerBranchRequest):
+
+    def do_with_branch(self, branch):
+        """Return the heads-to-fetch for a Branch as two bencoded lists.
+        
+        See Branch.heads_to_fetch.
+
+        New in 2.4.
+        """
+        must_fetch, if_present_fetch = branch.heads_to_fetch()
+        return SuccessfulSmartServerResponse(
+            (list(must_fetch), list(if_present_fetch)))
+
+
 class SmartServerBranchRequestGetStackedOnURL(SmartServerBranchRequest):
 
     def do_with_branch(self, branch):
@@ -157,8 +172,16 @@
         The revision list is returned as the body content,
         with each revision utf8 encoded and \x00 joined.
         """
+        branch.lock_read()
+        try:
+            graph = branch.repository.get_graph()
+            stop_revisions = (None, _mod_revision.NULL_REVISION)
+            history = list(graph.iter_lefthand_ancestry(
+                branch.last_revision(), stop_revisions))
+        finally:
+            branch.unlock()
         return SuccessfulSmartServerResponse(
-            ('ok', ), ('\x00'.join(branch.revision_history())))
+            ('ok', ), ('\x00'.join(reversed(history))))
 
 
 class SmartServerBranchRequestLastRevisionInfo(SmartServerBranchRequest):
@@ -218,12 +241,12 @@
 
     def do_tip_change_with_locked_branch(self, branch, new_last_revision_id):
         if new_last_revision_id == 'null:':
-            branch.set_revision_history([])
+            branch._set_revision_history([])
         else:
             if not branch.repository.has_revision(new_last_revision_id):
                 return FailedSmartServerResponse(
                     ('NoSuchRevision', new_last_revision_id))
-            branch.set_revision_history(branch._lefthand_history(
+            branch._set_revision_history(branch._lefthand_history(
                 new_last_revision_id, None, None))
         return SuccessfulSmartServerResponse(('ok',))
 

=== modified file 'bzrlib/smart/bzrdir.py'
--- bzrlib/smart/bzrdir.py	2010-05-06 23:41:35 +0000
+++ bzrlib/smart/bzrdir.py	2011-09-21 19:45:57 +0000
@@ -22,6 +22,9 @@
     BzrDir,
     BzrDirFormat,
     BzrDirMetaFormat1,
+    BzrProber,
+    )
+from bzrlib.controldir import (
     network_format_registry,
     )
 from bzrlib.smart.request import (
@@ -44,10 +47,9 @@
             # clients that don't anticipate errors from this method.
             answer = 'no'
         else:
-            default_format = BzrDirFormat.get_default_format()
-            real_bzrdir = default_format.open(t, _found=True)
+            bzr_prober = BzrProber()
             try:
-                real_bzrdir._format.probe_transport(t)
+                bzr_prober.probe_transport(t)
             except (errors.NotBranchError, errors.UnknownFormatError):
                 answer = 'no'
             else:
@@ -84,7 +86,7 @@
 class SmartServerRequestBzrDir(SmartServerRequest):
 
     def do(self, path, *args):
-        """Open a BzrDir at path, and return self.do_bzrdir_request(*args)."""
+        """Open a BzrDir at path, and return `self.do_bzrdir_request(*args)`."""
         try:
             self._bzrdir = BzrDir.open_from_transport(
                 self.transport_from_client_path(path))
@@ -148,9 +150,7 @@
         control_format = self._bzrdir.cloning_metadir(
             require_stacking=require_stacking)
         control_name = control_format.network_name()
-        # XXX: There should be a method that tells us that the format does/does
-        # not have subformats.
-        if isinstance(control_format, BzrDirMetaFormat1):
+        if not control_format.fixed_components:
             branch_name = ('branch',
                 control_format.get_branch_format().network_name())
             repository_name = control_format.repository_format.network_name()
@@ -179,7 +179,8 @@
 
         :param path: The path to the bzrdir.
         :param network_name: The network name of the branch type to create.
-        :return: (ok, network_name)
+        :return: ('ok', branch_format, repo_path, rich_root, tree_ref,
+            external_lookup, repo_format)
         """
         bzrdir = BzrDir.open_from_transport(
             self.transport_from_client_path(path))

=== modified file 'bzrlib/smart/client.py'
--- bzrlib/smart/client.py	2010-02-17 17:11:16 +0000
+++ bzrlib/smart/client.py	2011-03-30 11:45:54 +0000
@@ -194,12 +194,12 @@
 class SmartClientHooks(hooks.Hooks):
 
     def __init__(self):
-        hooks.Hooks.__init__(self)
-        self.create_hook(hooks.HookPoint('call',
+        hooks.Hooks.__init__(self, "bzrlib.smart.client", "_SmartClient.hooks")
+        self.add_hook('call',
             "Called when the smart client is submitting a request to the "
             "smart server. Called with a bzrlib.smart.client.CallHookParams "
             "object. Streaming request bodies, and responses, are not "
-            "accessible.", None, None))
+            "accessible.", None)
 
 
 _SmartClient.hooks = SmartClientHooks()

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2010-06-21 08:08:04 +0000
+++ bzrlib/smart/medium.py	2011-10-03 08:15:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,13 +24,16 @@
 bzrlib/transport/smart/__init__.py.
 """
 
+import errno
 import os
 import sys
+import time
 import urllib
 
 import bzrlib
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import select
 import socket
 import thread
 import weakref
@@ -38,12 +41,12 @@
 from bzrlib import (
     debug,
     errors,
-    symbol_versioning,
     trace,
     ui,
     urlutils,
     )
-from bzrlib.smart import client, protocol, request, vfs
+from bzrlib.i18n import gettext
+from bzrlib.smart import client, protocol, request, signals, vfs
 from bzrlib.transport import ssh
 """)
 from bzrlib import osutils
@@ -176,6 +179,14 @@
         ui.ui_factory.report_transport_activity(self, bytes, direction)
 
 
+_bad_file_descriptor = (errno.EBADF,)
+if sys.platform == 'win32':
+    # Given on Windows if you pass a closed socket to select.select. Probably
+    # also given if you pass a file handle to select.
+    WSAENOTSOCK = 10038
+    _bad_file_descriptor += (WSAENOTSOCK,)
+
+
 class SmartServerStreamMedium(SmartMedium):
     """Handles smart commands coming over a stream.
 
@@ -194,7 +205,9 @@
         the stream.  See also the _push_back method.
     """
 
-    def __init__(self, backing_transport, root_client_path='/'):
+    _timer = time.time
+
+    def __init__(self, backing_transport, root_client_path='/', timeout=None):
         """Construct new server.
 
         :param backing_transport: Transport for the directory served.
@@ -203,6 +216,10 @@
         self.backing_transport = backing_transport
         self.root_client_path = root_client_path
         self.finished = False
+        if timeout is None:
+            raise AssertionError('You must supply a timeout.')
+        self._client_timeout = timeout
+        self._client_poll_timeout = min(timeout / 10.0, 1.0)
         SmartMedium.__init__(self)
 
     def serve(self):
@@ -214,9 +231,39 @@
             while not self.finished:
                 server_protocol = self._build_protocol()
                 self._serve_one_request(server_protocol)
+        except errors.ConnectionTimeout, e:
+            trace.note('%s' % (e,))
+            trace.log_exception_quietly()
+            self._disconnect_client()
+            # We reported it, no reason to make a big fuss.
+            return
         except Exception, e:
             stderr.write("%s terminating on exception %s\n" % (self, e))
             raise
+        self._disconnect_client()
+
+    def _stop_gracefully(self):
+        """When we finish this message, stop looking for more."""
+        trace.mutter('Stopping %s' % (self,))
+        self.finished = True
+
+    def _disconnect_client(self):
+        """Close the current connection. We stopped due to a timeout/etc."""
+        # The default implementation is a no-op, because that is all we used to
+        # do when disconnecting from a client. I suppose we never had the
+        # *server* initiate a disconnect, before
+
+    def _wait_for_bytes_with_timeout(self, timeout_seconds):
+        """Wait for more bytes to be read, but timeout if none available.
+
+        This allows us to detect idle connections, and stop trying to read from
+        them, without setting the socket itself to non-blocking. This also
+        allows us to specify when we watch for idle timeouts.
+
+        :return: Did we timeout? (True if we timed out, False if there is data
+            to be read)
+        """
+        raise NotImplementedError(self._wait_for_bytes_with_timeout)
 
     def _build_protocol(self):
         """Identifies the version of the incoming request, and returns an
@@ -227,6 +274,10 @@
 
         :returns: a SmartServerRequestProtocol.
         """
+        self._wait_for_bytes_with_timeout(self._client_timeout)
+        if self.finished:
+            # We're stopping, so don't try to do any more work
+            return None
         bytes = self._get_line()
         protocol_factory, unused_bytes = _get_protocol_factory_for_bytes(bytes)
         protocol = protocol_factory(
@@ -234,11 +285,44 @@
         protocol.accept_bytes(unused_bytes)
         return protocol
 
+    def _wait_on_descriptor(self, fd, timeout_seconds):
+        """select() on a file descriptor, waiting for nonblocking read()
+
+        This will raise a ConnectionTimeout exception if we do not get a
+        readable handle before timeout_seconds.
+        :return: None
+        """
+        t_end = self._timer() + timeout_seconds
+        poll_timeout = min(timeout_seconds, self._client_poll_timeout)
+        rs = xs = None
+        while not rs and not xs and self._timer() < t_end:
+            if self.finished:
+                return
+            try:
+                rs, _, xs = select.select([fd], [], [fd], poll_timeout)
+            except (select.error, socket.error) as e:
+                err = getattr(e, 'errno', None)
+                if err is None and getattr(e, 'args', None) is not None:
+                    # select.error doesn't have 'errno', it just has args[0]
+                    err = e.args[0]
+                if err in _bad_file_descriptor:
+                    return # Not a socket indicates read() will fail
+                elif err == errno.EINTR:
+                    # Interrupted, keep looping.
+                    continue
+                raise
+        if rs or xs:
+            return
+        raise errors.ConnectionTimeout('disconnecting client after %.1f seconds'
+                                       % (timeout_seconds,))
+
     def _serve_one_request(self, protocol):
         """Read one request from input, process, send back a response.
 
         :param protocol: a SmartServerRequestProtocol.
         """
+        if protocol is None:
+            return
         try:
             self._serve_one_request_unguarded(protocol)
         except KeyboardInterrupt:
@@ -260,16 +344,30 @@
 
 class SmartServerSocketStreamMedium(SmartServerStreamMedium):
 
-    def __init__(self, sock, backing_transport, root_client_path='/'):
+    def __init__(self, sock, backing_transport, root_client_path='/',
+                 timeout=None):
         """Constructor.
 
         :param sock: the socket the server will read from.  It will be put
             into blocking mode.
         """
         SmartServerStreamMedium.__init__(
-            self, backing_transport, root_client_path=root_client_path)
+            self, backing_transport, root_client_path=root_client_path,
+            timeout=timeout)
         sock.setblocking(True)
         self.socket = sock
+        # Get the getpeername now, as we might be closed later when we care.
+        try:
+            self._client_info = sock.getpeername()
+        except socket.error:
+            self._client_info = '<unknown>'
+
+    def __str__(self):
+        return '%s(client=%s)' % (self.__class__.__name__, self._client_info)
+
+    def __repr__(self):
+        return '%s.%s(client=%s)' % (self.__module__, self.__class__.__name__,
+            self._client_info)
 
     def _serve_one_request_unguarded(self, protocol):
         while protocol.next_read_size():
@@ -284,6 +382,22 @@
 
         self._push_back(protocol.unused_data)
 
+    def _disconnect_client(self):
+        """Close the current connection. We stopped due to a timeout/etc."""
+        self.socket.close()
+
+    def _wait_for_bytes_with_timeout(self, timeout_seconds):
+        """Wait for more bytes to be read, but timeout if none available.
+
+        This allows us to detect idle connections, and stop trying to read from
+        them, without setting the socket itself to non-blocking. This also
+        allows us to specify when we watch for idle timeouts.
+
+        :return: None, this will raise ConnectionTimeout if we time out before
+            data is available.
+        """
+        return self._wait_on_descriptor(self.socket, timeout_seconds)
+
     def _read_bytes(self, desired_count):
         return osutils.read_bytes_from_socket(
             self.socket, self._report_activity)
@@ -306,14 +420,15 @@
 
 class SmartServerPipeStreamMedium(SmartServerStreamMedium):
 
-    def __init__(self, in_file, out_file, backing_transport):
+    def __init__(self, in_file, out_file, backing_transport, timeout=None):
         """Construct new server.
 
         :param in_file: Python file from which requests can be read.
         :param out_file: Python file to write responses.
         :param backing_transport: Transport for the directory served.
         """
-        SmartServerStreamMedium.__init__(self, backing_transport)
+        SmartServerStreamMedium.__init__(self, backing_transport,
+            timeout=timeout)
         if sys.platform == 'win32':
             # force binary mode for files
             import msvcrt
@@ -324,6 +439,17 @@
         self._in = in_file
         self._out = out_file
 
+    def serve(self):
+        """See SmartServerStreamMedium.serve"""
+        # This is the regular serve, except it adds signal trapping for soft
+        # shutdown.
+        stop_gracefully = self._stop_gracefully
+        signals.register_on_hangup(id(self), stop_gracefully)
+        try:
+            return super(SmartServerPipeStreamMedium, self).serve()
+        finally:
+            signals.unregister_on_hangup(id(self))
+
     def _serve_one_request_unguarded(self, protocol):
         while True:
             # We need to be careful not to read past the end of the current
@@ -342,6 +468,27 @@
                 return
             protocol.accept_bytes(bytes)
 
+    def _disconnect_client(self):
+        self._in.close()
+        self._out.flush()
+        self._out.close()
+
+    def _wait_for_bytes_with_timeout(self, timeout_seconds):
+        """Wait for more bytes to be read, but timeout if none available.
+
+        This allows us to detect idle connections, and stop trying to read from
+        them, without setting the socket itself to non-blocking. This also
+        allows us to specify when we watch for idle timeouts.
+
+        :return: None, this will raise ConnectionTimeout if we time out before
+            data is available.
+        """
+        if (getattr(self._in, 'fileno', None) is None
+            or sys.platform == 'win32'):
+            # You can't select() file descriptors on Windows.
+            return
+        return self._wait_on_descriptor(self._in, timeout_seconds)
+
     def _read_bytes(self, desired_count):
         return self._in.read(desired_count)
 
@@ -491,6 +638,25 @@
         return self._medium._get_line()
 
 
+class _VfsRefuser(object):
+    """An object that refuses all VFS requests.
+
+    """
+
+    def __init__(self):
+        client._SmartClient.hooks.install_named_hook(
+            'call', self.check_vfs, 'vfs refuser')
+
+    def check_vfs(self, params):
+        try:
+            request_method = request.request_handlers.get(params.method)
+        except KeyError:
+            # A method we don't know about doesn't count as a VFS method.
+            return
+        if issubclass(request_method, vfs.VfsRequest):
+            raise errors.HpssVfsRequestNotAllowed(params.method, params.args)
+
+
 class _DebugCounter(object):
     """An object that counts the HPSS calls made to each client medium.
 
@@ -543,14 +709,15 @@
         value['count'] = 0
         value['vfs_count'] = 0
         if count != 0:
-            trace.note('HPSS calls: %d (%d vfs) %s',
-                       count, vfs_count, medium_repr)
+            trace.note(gettext('HPSS calls: {0} ({1} vfs) {2}').format(
+                       count, vfs_count, medium_repr))
 
     def flush_all(self):
         for ref in list(self.counts.keys()):
             self.done(ref)
 
 _debug_counter = None
+_vfs_refuser = None
 
 
 class SmartClientMedium(SmartMedium):
@@ -573,6 +740,10 @@
             if _debug_counter is None:
                 _debug_counter = _DebugCounter()
             _debug_counter.track(self)
+        if 'hpss_client_no_vfs' in debug.debug_flags:
+            global _vfs_refuser
+            if _vfs_refuser is None:
+                _vfs_refuser = _VfsRefuser()
 
     def _is_remote_before(self, version_tuple):
         """Is it possible the remote side supports RPCs for a given version?

=== modified file 'bzrlib/smart/message.py'
--- bzrlib/smart/message.py	2009-09-11 06:07:05 +0000
+++ bzrlib/smart/message.py	2011-03-02 20:16:03 +0000
@@ -303,7 +303,7 @@
             mutter('   result:   %r', self.args)
         if self.status == 'E':
             self._wait_for_response_end()
-            _translate_error(self.args)
+            _raise_smart_server_error(self.args)
         return tuple(self.args)
 
     def read_body_bytes(self, count=-1):
@@ -335,27 +335,17 @@
                 yield bytes_part
             self._read_more()
         if self._body_stream_status == 'E':
-            _translate_error(self._body_error_args)
+            _raise_smart_server_error(self._body_error_args)
 
     def cancel_read_body(self):
         self._wait_for_response_end()
 
 
-def _translate_error(error_tuple):
-    # Many exceptions need some state from the requestor to be properly
-    # translated (e.g. they need a branch object).  So this only translates a
-    # few errors, and the rest are turned into a generic ErrorFromSmartServer.
-    error_name = error_tuple[0]
-    error_args = error_tuple[1:]
-    if error_name == 'UnknownMethod':
-        raise errors.UnknownSmartMethod(error_args[0])
-    if error_name == 'LockContention':
-        raise errors.LockContention('(remote lock)')
-    elif error_name == 'LockFailed':
-        raise errors.LockFailed(*error_args[:2])
-    elif error_name == 'FileExists':
-        raise errors.FileExists(error_args[0])
-    elif error_name == 'NoSuchFile':
-        raise errors.NoSuchFile(error_args[0])
-    else:
-        raise errors.ErrorFromSmartServer(error_tuple)
+def _raise_smart_server_error(error_tuple):
+    """Raise exception based on tuple received from smart server
+
+    Specific error translation is handled by bzrlib.remote._translate_error
+    """
+    if error_tuple[0] == 'UnknownMethod':
+        raise errors.UnknownSmartMethod(error_tuple[1])
+    raise errors.ErrorFromSmartServer(error_tuple)

=== modified file 'bzrlib/smart/protocol.py'
--- bzrlib/smart/protocol.py	2010-06-11 05:57:09 +0000
+++ bzrlib/smart/protocol.py	2011-05-19 09:32:38 +0000
@@ -654,7 +654,7 @@
         """Make a remote call with a readv array.
 
         The body is encoded with one line per readv offset pair. The numbers in
-        each pair are separated by a comma, and no trailing \n is emitted.
+        each pair are separated by a comma, and no trailing \\n is emitted.
         """
         if 'hpss' in debug.debug_flags:
             mutter('hpss call w/readv: %s', repr(args)[1:-1])
@@ -1332,7 +1332,7 @@
         """Make a remote call with a readv array.
 
         The body is encoded with one line per readv offset pair. The numbers in
-        each pair are separated by a comma, and no trailing \n is emitted.
+        each pair are separated by a comma, and no trailing \\n is emitted.
         """
         if 'hpss' in debug.debug_flags:
             mutter('hpss call w/readv: %s', repr(args)[1:-1])

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2010-05-14 13:36:34 +0000
+++ bzrlib/smart/repository.py	2011-09-12 18:40:02 +0000
@@ -25,12 +25,14 @@
 
 from bzrlib import (
     bencode,
+    commands,
     errors,
+    estimate_compressed_size,
     graph,
     osutils,
     pack,
+    trace,
     ui,
-    versionedfile,
     )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart.request import (
@@ -39,7 +41,6 @@
     SuccessfulSmartServerResponse,
     )
 from bzrlib.repository import _strip_NULL_ghosts, network_format_registry
-from bzrlib.recordcounter import RecordCounter
 from bzrlib import revision as _mod_revision
 from bzrlib.versionedfile import (
     NetworkRecordStream,
@@ -83,6 +84,8 @@
             recreate_search trusts that clients will look for missing things
             they expected and get it from elsewhere.
         """
+        if search_bytes == 'everything':
+            return graph.EverythingResult(repository), None
         lines = search_bytes.split('\n')
         if lines[0] == 'ancestry-of':
             heads = lines[1:]
@@ -141,6 +144,7 @@
         finally:
             repository.unlock()
 
+_lsprof_count = 0
 
 class SmartServerRepositoryGetParentMap(SmartServerRepositoryRequest):
     """Bzr 1.2+ - get parent data for revisions during a graph search."""
@@ -180,27 +184,11 @@
         finally:
             repository.unlock()
 
-    def _do_repository_request(self, body_bytes):
-        repository = self._repository
-        revision_ids = set(self._revision_ids)
-        include_missing = 'include-missing:' in revision_ids
-        if include_missing:
-            revision_ids.remove('include-missing:')
-        body_lines = body_bytes.split('\n')
-        search_result, error = self.recreate_search_from_recipe(
-            repository, body_lines)
-        if error is not None:
-            return error
-        # TODO might be nice to start up the search again; but thats not
-        # written or tested yet.
-        client_seen_revs = set(search_result.get_keys())
-        # Always include the requested ids.
-        client_seen_revs.difference_update(revision_ids)
-        lines = []
-        repo_graph = repository.get_graph()
+    def _expand_requested_revs(self, repo_graph, revision_ids, client_seen_revs,
+                               include_missing, max_size=65536):
         result = {}
         queried_revs = set()
-        size_so_far = 0
+        estimator = estimate_compressed_size.ZLibEstimator(max_size)
         next_revs = revision_ids
         first_loop_done = False
         while next_revs:
@@ -228,21 +216,47 @@
                     # add parents to the result
                     result[encoded_id] = parents
                     # Approximate the serialized cost of this revision_id.
-                    size_so_far += 2 + len(encoded_id) + sum(map(len, parents))
+                    line = '%s %s\n' % (encoded_id, ' '.join(parents))
+                    estimator.add_content(line)
             # get all the directly asked for parents, and then flesh out to
             # 64K (compressed) or so. We do one level of depth at a time to
             # stay in sync with the client. The 250000 magic number is
             # estimated compression ratio taken from bzr.dev itself.
-            if self.no_extra_results or (
-                first_loop_done and size_so_far > 250000):
+            if self.no_extra_results or (first_loop_done and estimator.full()):
+                trace.mutter('size: %d, z_size: %d'
+                             % (estimator._uncompressed_size_added,
+                                estimator._compressed_size_added))
                 next_revs = set()
                 break
             # don't query things we've already queried
-            next_revs.difference_update(queried_revs)
+            next_revs = next_revs.difference(queried_revs)
             first_loop_done = True
+        return result
+
+    def _do_repository_request(self, body_bytes):
+        repository = self._repository
+        revision_ids = set(self._revision_ids)
+        include_missing = 'include-missing:' in revision_ids
+        if include_missing:
+            revision_ids.remove('include-missing:')
+        body_lines = body_bytes.split('\n')
+        search_result, error = self.recreate_search_from_recipe(
+            repository, body_lines)
+        if error is not None:
+            return error
+        # TODO might be nice to start up the search again; but thats not
+        # written or tested yet.
+        client_seen_revs = set(search_result.get_keys())
+        # Always include the requested ids.
+        client_seen_revs.difference_update(revision_ids)
+
+        repo_graph = repository.get_graph()
+        result = self._expand_requested_revs(repo_graph, revision_ids,
+                                             client_seen_revs, include_missing)
 
         # sorting trivially puts lexographically similar revision ids together.
         # Compression FTW.
+        lines = []
         for revision, parents in sorted(result.items()):
             lines.append(' '.join((revision, ) + tuple(parents)))
 
@@ -414,6 +428,13 @@
     def do_repository_request(self, repository, to_network_name):
         """Get a stream for inserting into a to_format repository.
 
+        The request body is 'search_bytes', a description of the revisions
+        being requested.
+
+        In 2.3 this verb added support for search_bytes == 'everything'.  Older
+        implementations will respond with a BadSearch error, and clients should
+        catch this and fallback appropriately.
+
         :param repository: The repository to stream from.
         :param to_network_name: The network name of the format of the target
             repository.
@@ -491,6 +512,13 @@
 
 
 class SmartServerRepositoryGetStream_1_19(SmartServerRepositoryGetStream):
+    """The same as Repository.get_stream, but will return stream CHK formats to
+    clients.
+
+    See SmartServerRepositoryGetStream._should_fake_unknown.
+    
+    New in 1.19.
+    """
 
     def _should_fake_unknown(self):
         """Returns False; we don't need to workaround bugs in 1.19+ clients."""
@@ -506,8 +534,6 @@
         for record in substream:
             if record.storage_kind in ('chunked', 'fulltext'):
                 serialised = record_to_fulltext_bytes(record)
-            elif record.storage_kind == 'inventory-delta':
-                serialised = record_to_inventory_delta_bytes(record)
             elif record.storage_kind == 'absent':
                 raise ValueError("Absent factory for %s" % (record.key,))
             else:

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2010-05-13 16:17:54 +0000
+++ bzrlib/smart/request.py	2011-05-19 09:32:38 +0000
@@ -31,8 +31,6 @@
 # of a SmartServerRequest subclass.
 
 
-import tempfile
-import thread
 import threading
 
 from bzrlib import (
@@ -48,6 +46,9 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib.bundle import serializer
+
+import tempfile
+import thread
 """)
 
 
@@ -134,7 +135,7 @@
         It will return a SmartServerResponse if the command does not expect a
         body.
 
-        :param *args: the arguments of the request.
+        :param args: the arguments of the request.
         """
         self._check_enabled()
         return self.do(*args)
@@ -446,9 +447,13 @@
         return ('TokenMismatch', err.given_token, err.lock_token)
     elif isinstance(err, errors.LockContention):
         return ('LockContention',)
+    elif isinstance(err, MemoryError):
+        # GZ 2011-02-24: Copy bzrlib.trace -Dmem_dump functionality here?
+        return ('MemoryError',)
     # Unserialisable error.  Log it, and return a generic error
     trace.log_exception_quietly()
-    return ('error', str(err))
+    return ('error', trace._qualified_exception_name(err.__class__, True),
+        str(err))
 
 
 class HelloRequest(SmartServerRequest):
@@ -501,6 +506,9 @@
     'Branch.set_tags_bytes', 'bzrlib.smart.branch',
     'SmartServerBranchSetTagsBytes')
 request_handlers.register_lazy(
+    'Branch.heads_to_fetch', 'bzrlib.smart.branch',
+    'SmartServerBranchHeadsToFetch')
+request_handlers.register_lazy(
     'Branch.get_stacked_on_url', 'bzrlib.smart.branch', 'SmartServerBranchRequestGetStackedOnURL')
 request_handlers.register_lazy(
     'Branch.last_revision_info', 'bzrlib.smart.branch', 'SmartServerBranchRequestLastRevisionInfo')

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2010-03-05 07:27:58 +0000
+++ bzrlib/smart/server.py	2011-09-26 10:44:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,23 +20,28 @@
 import os.path
 import socket
 import sys
+import time
 import threading
 
-from bzrlib.hooks import HookPoint, Hooks
+from bzrlib.hooks import Hooks
 from bzrlib import (
     errors,
     trace,
-    transport,
+    transport as _mod_transport,
 )
+from bzrlib.i18n import gettext
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib.smart import medium
+from bzrlib.smart import (
+    medium,
+    signals,
+    )
 from bzrlib.transport import (
     chroot,
-    get_transport,
     pathfilter,
     )
 from bzrlib import (
+    config,
     urlutils,
     )
 """)
@@ -51,18 +56,41 @@
     hooks: An instance of SmartServerHooks.
     """
 
-    def __init__(self, backing_transport, host='127.0.0.1', port=0,
-                 root_client_path='/'):
+    # This is the timeout on the socket we use .accept() on. It is exposed here
+    # so the test suite can set it faster. (It thread.interrupt_main() will not
+    # fire a KeyboardInterrupt during socket.accept)
+    _ACCEPT_TIMEOUT = 1.0
+    _SHUTDOWN_POLL_TIMEOUT = 1.0
+    _LOG_WAITING_TIMEOUT = 10.0
+
+    _timer = time.time
+
+    def __init__(self, backing_transport, root_client_path='/',
+                 client_timeout=None):
         """Construct a new server.
 
         To actually start it running, call either start_background_thread or
         serve.
 
         :param backing_transport: The transport to serve.
+        :param root_client_path: The client path that will correspond to root
+            of backing_transport.
+        :param client_timeout: See SmartServerSocketStreamMedium's timeout
+            parameter.
+        """
+        self.backing_transport = backing_transport
+        self.root_client_path = root_client_path
+        self._client_timeout = client_timeout
+        self._active_connections = []
+        # This is set to indicate we want to wait for clients to finish before
+        # we disconnect.
+        self._gracefully_stopping = False
+
+    def start_server(self, host, port):
+        """Create the server listening socket.
+
         :param host: Name of the interface to listen on.
         :param port: TCP port to listen on, or 0 to allocate a transient port.
-        :param root_client_path: The client path that will correspond to root
-            of backing_transport.
         """
         # let connections timeout so that we get a chance to terminate
         # Keep a reference to the exceptions we want to catch because the socket
@@ -88,22 +116,23 @@
         self._sockname = self._server_socket.getsockname()
         self.port = self._sockname[1]
         self._server_socket.listen(1)
-        self._server_socket.settimeout(1)
-        self.backing_transport = backing_transport
+        self._server_socket.settimeout(self._ACCEPT_TIMEOUT)
+        # Once we start accept()ing connections, we set started.
         self._started = threading.Event()
+        # Once we stop accept()ing connections (and are closing the socket) we
+        # set _stopped
         self._stopped = threading.Event()
-        self.root_client_path = root_client_path
+        # Once we have finished waiting for all clients, etc. We set
+        # _fully_stopped
+        self._fully_stopped = threading.Event()
 
-    def serve(self, thread_name_suffix=''):
-        self._should_terminate = False
-        # for hooks we are letting code know that a server has started (and
-        # later stopped).
+    def _backing_urls(self):
         # There are three interesting urls:
         # The URL the server can be contacted on. (e.g. bzr://host/)
         # The URL that a commit done on the same machine as the server will
         # have within the servers space. (e.g. file:///home/user/source)
         # The URL that will be given to other hooks in the same process -
-        # the URL of the backing transport itself. (e.g. chroot+:///)
+        # the URL of the backing transport itself. (e.g. filtered-36195:///)
         # We need all three because:
         #  * other machines see the first
         #  * local commits on this machine should be able to be mapped to
@@ -113,15 +142,63 @@
         # The latter two urls are different aliases to the servers url,
         # so we group those in a list - as there might be more aliases
         # in the future.
-        backing_urls = [self.backing_transport.base]
+        urls = [self.backing_transport.base]
         try:
-            backing_urls.append(self.backing_transport.external_url())
+            urls.append(self.backing_transport.external_url())
         except errors.InProcessTransport:
             pass
+        return urls
+
+    def run_server_started_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
         for hook in SmartTCPServer.hooks['server_started']:
             hook(backing_urls, self.get_url())
         for hook in SmartTCPServer.hooks['server_started_ex']:
             hook(backing_urls, self)
+
+    def run_server_stopped_hooks(self, backing_urls=None):
+        if backing_urls is None:
+            backing_urls = self._backing_urls()
+        for hook in SmartTCPServer.hooks['server_stopped']:
+            hook(backing_urls, self.get_url())
+
+    def _stop_gracefully(self):
+        trace.note(gettext('Requested to stop gracefully'))
+        self._should_terminate = True
+        self._gracefully_stopping = True
+        for handler, _ in self._active_connections:
+            handler._stop_gracefully()
+
+    def _wait_for_clients_to_disconnect(self):
+        self._poll_active_connections()
+        if not self._active_connections:
+            return
+        trace.note(gettext('Waiting for %d client(s) to finish')
+                   % (len(self._active_connections),))
+        t_next_log = self._timer() + self._LOG_WAITING_TIMEOUT
+        while self._active_connections:
+            now = self._timer()
+            if now >= t_next_log:
+                trace.note(gettext('Still waiting for %d client(s) to finish')
+                           % (len(self._active_connections),))
+                t_next_log = now + self._LOG_WAITING_TIMEOUT
+            self._poll_active_connections(self._SHUTDOWN_POLL_TIMEOUT)
+
+    def serve(self, thread_name_suffix=''):
+        # Note: There is a temptation to do
+        #       signals.register_on_hangup(id(self), self._stop_gracefully)
+        #       However, that creates a temporary object which is a bound
+        #       method. signals._on_sighup is a WeakKeyDictionary so it
+        #       immediately gets garbage collected, because nothing else
+        #       references it. Instead, we need to keep a real reference to the
+        #       bound method for the lifetime of the serve() function.
+        stop_gracefully = self._stop_gracefully
+        signals.register_on_hangup(id(self), stop_gracefully)
+        self._should_terminate = False
+        # for hooks we are letting code know that a server has started (and
+        # later stopped).
+        self.run_server_started_hooks()
         self._started.set()
         try:
             try:
@@ -133,14 +210,19 @@
                         pass
                     except self._socket_error, e:
                         # if the socket is closed by stop_background_thread
-                        # we might get a EBADF here, any other socket errors
-                        # should get logged.
-                        if e.args[0] != errno.EBADF:
-                            trace.warning("listening socket error: %s", e)
+                        # we might get a EBADF here, or if we get a signal we
+                        # can get EINTR, any other socket errors should get
+                        # logged.
+                        if e.args[0] not in (errno.EBADF, errno.EINTR):
+                            trace.warning(gettext("listening socket error: %s")
+                                          % (e,))
                     else:
                         if self._should_terminate:
+                            conn.close()
                             break
                         self.serve_conn(conn, thread_name_suffix)
+                    # Cleanout any threads that have finished processing.
+                    self._poll_active_connections()
             except KeyboardInterrupt:
                 # dont log when CTRL-C'd.
                 raise
@@ -148,32 +230,58 @@
                 trace.report_exception(sys.exc_info(), sys.stderr)
                 raise
         finally:
-            self._stopped.set()
             try:
                 # ensure the server socket is closed.
                 self._server_socket.close()
             except self._socket_error:
                 # ignore errors on close
                 pass
-            for hook in SmartTCPServer.hooks['server_stopped']:
-                hook(backing_urls, self.get_url())
+            self._stopped.set()
+            signals.unregister_on_hangup(id(self))
+            self.run_server_stopped_hooks()
+        if self._gracefully_stopping:
+            self._wait_for_clients_to_disconnect()
+        self._fully_stopped.set()
 
     def get_url(self):
         """Return the url of the server"""
-        return "bzr://%s:%d/" % self._sockname
+        return "bzr://%s:%s/" % (self._sockname[0], self._sockname[1])
+
+    def _make_handler(self, conn):
+        return medium.SmartServerSocketStreamMedium(
+            conn, self.backing_transport, self.root_client_path,
+            timeout=self._client_timeout)
+
+    def _poll_active_connections(self, timeout=0.0):
+        """Check to see if any active connections have finished.
+
+        This will iterate through self._active_connections, and update any
+        connections that are finished.
+
+        :param timeout: The timeout to pass to thread.join(). By default, we
+            set it to 0, so that we don't hang if threads are not done yet.
+        :return: None
+        """
+        still_active = []
+        for handler, thread in self._active_connections:
+            thread.join(timeout)
+            if thread.isAlive():
+                still_active.append((handler, thread))
+        self._active_connections = still_active
 
     def serve_conn(self, conn, thread_name_suffix):
         # For WIN32, where the timeout value from the listening socket
         # propagates to the newly accepted socket.
         conn.setblocking(True)
         conn.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-        handler = medium.SmartServerSocketStreamMedium(
-            conn, self.backing_transport, self.root_client_path)
         thread_name = 'smart-server-child' + thread_name_suffix
+        handler = self._make_handler(conn)
         connection_thread = threading.Thread(
             None, handler.serve, name=thread_name)
+        self._active_connections.append((handler, connection_thread))
         connection_thread.setDaemon(True)
         connection_thread.start()
+        return connection_thread
 
     def start_background_thread(self, thread_name_suffix=''):
         self._started.clear()
@@ -219,21 +327,26 @@
         These are all empty initially, because by default nothing should get
         notified.
         """
-        Hooks.__init__(self)
-        self.create_hook(HookPoint('server_started',
+        Hooks.__init__(self, "bzrlib.smart.server", "SmartTCPServer.hooks")
+        self.add_hook('server_started',
             "Called by the bzr server when it starts serving a directory. "
             "server_started is called with (backing urls, public url), "
             "where backing_url is a list of URLs giving the "
             "server-specific directory locations, and public_url is the "
-            "public URL for the directory being served.", (0, 16), None))
-        self.create_hook(HookPoint('server_started_ex',
+            "public URL for the directory being served.", (0, 16))
+        self.add_hook('server_started_ex',
             "Called by the bzr server when it starts serving a directory. "
             "server_started is called with (backing_urls, server_obj).",
-            (1, 17), None))
-        self.create_hook(HookPoint('server_stopped',
+            (1, 17))
+        self.add_hook('server_stopped',
             "Called by the bzr server when it stops serving a directory. "
             "server_stopped is called with the same parameters as the "
-            "server_started hook: (backing_urls, public_url).", (0, 16), None))
+            "server_started hook: (backing_urls, public_url).", (0, 16))
+        self.add_hook('server_exception',
+            "Called by the bzr server when an exception occurs. "
+            "server_exception is called with the sys.exc_info() tuple "
+            "return true for the hook if the exception has been handled, "
+            "in which case the server will exit normally.", (2, 4))
 
 SmartTCPServer.hooks = SmartServerHooks()
 
@@ -305,27 +418,36 @@
         chroot_server = chroot.ChrootServer(transport)
         chroot_server.start_server()
         self.cleanups.append(chroot_server.stop_server)
-        transport = get_transport(chroot_server.get_url())
+        transport = _mod_transport.get_transport_from_url(chroot_server.get_url())
         if self.base_path is not None:
             # Decorate the server's backing transport with a filter that can
             # expand homedirs.
             expand_userdirs = self._make_expand_userdirs_filter(transport)
             expand_userdirs.start_server()
             self.cleanups.append(expand_userdirs.stop_server)
-            transport = get_transport(expand_userdirs.get_url())
+            transport = _mod_transport.get_transport_from_url(expand_userdirs.get_url())
         self.transport = transport
 
-    def _make_smart_server(self, host, port, inet):
+    def _get_stdin_stdout(self):
+        return sys.stdin, sys.stdout
+
+    def _make_smart_server(self, host, port, inet, timeout):
+        if timeout is None:
+            c = config.GlobalStack()
+            timeout = c.get('serve.client_timeout')
         if inet:
+            stdin, stdout = self._get_stdin_stdout()
             smart_server = medium.SmartServerPipeStreamMedium(
-                sys.stdin, sys.stdout, self.transport)
+                stdin, stdout, self.transport, timeout=timeout)
         else:
             if host is None:
                 host = medium.BZR_DEFAULT_INTERFACE
             if port is None:
                 port = medium.BZR_DEFAULT_PORT
-            smart_server = SmartTCPServer(self.transport, host=host, port=port)
-            trace.note('listening on port: %s' % smart_server.port)
+            smart_server = SmartTCPServer(self.transport,
+                                          client_timeout=timeout)
+            smart_server.start_server(host, port)
+            trace.note(gettext('listening on port: %s') % smart_server.port)
         self.smart_server = smart_server
 
     def _change_globals(self):
@@ -342,10 +464,14 @@
         self.cleanups.append(restore_default_ui_factory_and_lockdir_timeout)
         ui.ui_factory = ui.SilentUIFactory()
         lockdir._DEFAULT_TIMEOUT_SECONDS = 0
+        orig = signals.install_sighup_handler()
+        def restore_signals():
+            signals.restore_sighup_handler(orig)
+        self.cleanups.append(restore_signals)
 
-    def set_up(self, transport, host, port, inet):
+    def set_up(self, transport, host, port, inet, timeout):
         self._make_backing_transport(transport)
-        self._make_smart_server(host, port, inet)
+        self._make_smart_server(host, port, inet, timeout)
         self._change_globals()
 
     def tear_down(self):
@@ -353,17 +479,22 @@
             cleanup()
 
 
-def serve_bzr(transport, host=None, port=None, inet=False):
+def serve_bzr(transport, host=None, port=None, inet=False, timeout=None):
     """This is the default implementation of 'bzr serve'.
-    
+
     It creates a TCP or pipe smart server on 'transport, and runs it.  The
     transport will be decorated with a chroot and pathfilter (using
     os.path.expanduser).
     """
     bzr_server = BzrServerFactory()
     try:
-        bzr_server.set_up(transport, host, port, inet)
+        bzr_server.set_up(transport, host, port, inet, timeout)
         bzr_server.smart_server.serve()
+    except:
+        hook_caught_exception = False
+        for hook in SmartTCPServer.hooks['server_exception']:
+            hook_caught_exception = hook(sys.exc_info())
+        if not hook_caught_exception:
+            raise
     finally:
         bzr_server.tear_down()
-

=== added file 'bzrlib/smart/signals.py'
--- bzrlib/smart/signals.py	1970-01-01 00:00:00 +0000
+++ bzrlib/smart/signals.py	2011-09-23 18:11:28 +0000
@@ -0,0 +1,114 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Signal handling for the smart server code."""
+
+import signal
+import weakref
+
+from bzrlib import trace
+
+
+# I'm pretty sure this has to be global, since signal handling is per-process.
+_on_sighup = None
+# TODO: Using a dict means that the order of calls is unordered. We could use a
+#       list and then do something like LIFO ordering. A dict was chosen so
+#       that you could have a key to easily remove your entry. However, you
+#       could just use the callable itself as the indexed part, and even in
+#       large cases, we shouldn't have more than 100 or so callbacks
+#       registered.
+def _sighup_handler(signal_number, interrupted_frame):
+    """This is the actual function that is registered for handling SIGHUP.
+
+    It will call out to all the registered functions, letting them know that a
+    graceful termination has been requested.
+    """
+    if _on_sighup is None:
+        return
+    trace.mutter('Caught SIGHUP, sending graceful shutdown requests.')
+    for ref in _on_sighup.valuerefs():
+        try:
+            cb = ref()
+            if cb is not None:
+                cb()
+        except KeyboardInterrupt:
+            raise
+        except Exception:
+            trace.mutter('Error occurred while running SIGHUP handlers:')
+            trace.log_exception_quietly()
+
+
+def install_sighup_handler():
+    """Setup a handler for the SIGHUP signal."""
+    if getattr(signal, "SIGHUP", None) is None:
+        # If we can't install SIGHUP, there is no reason (yet) to do graceful
+        # shutdown.
+        old_signal = None
+    else:
+        old_signal = signal.signal(signal.SIGHUP, _sighup_handler)
+    old_dict = _setup_on_hangup_dict()
+    return old_signal, old_dict
+
+
+def _setup_on_hangup_dict():
+    """Create something for _on_sighup.
+
+    This is done when we install the sighup handler, and for tests that want to
+    test the functionality. If this hasn'nt been called, then
+    register_on_hangup is a no-op. As is unregister_on_hangup.
+    """
+    global _on_sighup
+    old = _on_sighup
+    _on_sighup = weakref.WeakValueDictionary()
+    return old
+
+
+def restore_sighup_handler(orig):
+    """Pass in the returned value from install_sighup_handler to reset."""
+    global _on_sighup
+    old_signal, old_dict = orig
+    if old_signal is not None:
+        signal.signal(signal.SIGHUP, old_signal)
+    _on_sighup = old_dict
+
+
+# TODO: Should these be single-use callables? Meaning that once we've triggered
+#       SIGHUP and called them, they should auto-remove themselves? I don't
+#       think so. Callers need to clean up during shutdown anyway, so that we
+#       don't end up with lots of garbage in the _on_sighup dict. On the other
+#       hand, we made _on_sighup a WeakValueDictionary in case cleanups didn't
+#       get fired properly. Maybe we just assume we don't have to do it?
+def register_on_hangup(identifier, a_callable):
+    """Register for us to call a_callable as part of a graceful shutdown."""
+    if _on_sighup is None:
+        return
+    _on_sighup[identifier] = a_callable
+
+
+def unregister_on_hangup(identifier):
+    """Remove a callback from being called during sighup."""
+    if _on_sighup is None:
+        return
+    try:
+        del _on_sighup[identifier]
+    except KeyboardInterrupt:
+        raise
+    except Exception:
+        # This usually runs as a tear-down step. So we don't want to propagate
+        # most exceptions.
+        trace.mutter('Error occurred during unregister_on_hangup:')
+        trace.log_exception_quietly()
+

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2010-04-30 11:35:43 +0000
+++ bzrlib/status.py	2011-06-01 12:53:56 +0000
@@ -18,6 +18,7 @@
 
 from bzrlib import (
     delta as _mod_delta,
+    hooks as _mod_hooks,
     log,
     osutils,
     tsort,
@@ -33,7 +34,7 @@
 def report_changes(to_file, old, new, specific_files, 
                    show_short_reporter, show_long_callback, 
                    short=False, want_unchanged=False, 
-                   want_unversioned=False, show_ids=False):
+                   want_unversioned=False, show_ids=False, classify=True):
     """Display summary of changes.
 
     This compares two trees with regards to a list of files, and delegates 
@@ -58,6 +59,7 @@
         files.
     :param show_ids: If set, includes each file's id.
     :param want_unversioned: If False, only shows versioned files.
+    :param classify: Add special symbols to indicate file kind.
     """
 
     if short:
@@ -75,7 +77,8 @@
             delta.unversioned if not new.is_ignored(unversioned[0])]
         show_long_callback(to_file, delta, 
                            show_ids=show_ids,
-                           show_unchanged=want_unchanged)
+                           show_unchanged=want_unchanged,
+                           classify=classify)
 
 
 def show_tree_status(wt, show_unchanged=None,
@@ -87,6 +90,7 @@
                      short=False,
                      verbose=False,
                      versioned=False,
+                     classify=True,
                      show_long_callback=_mod_delta.report_delta):
     """Display summary of changes.
 
@@ -116,6 +120,7 @@
     :param verbose: If True, show all merged revisions, not just
         the merge tips
     :param versioned: If True, only shows versioned files.
+    :param classify: Add special symbols to indicate file kind.
     :param show_long_callback: A callback: message = show_long_callback(to_file, delta, 
         show_ids, show_unchanged, indent, filter), only used with the long output
     """
@@ -150,17 +155,22 @@
         old.lock_read()
         new.lock_read()
         try:
+            for hook in hooks['pre_status']:
+                hook(StatusHookParams(old, new, to_file, versioned,
+                    show_ids, short, verbose))
+
             specific_files, nonexistents \
                 = _filter_nonexistent(specific_files, old, new)
             want_unversioned = not versioned
 
             # Reporter used for short outputs
             reporter = _mod_delta._ChangeReporter(output_file=to_file,
-                unversioned_filter=new.is_ignored)
+                unversioned_filter=new.is_ignored, classify=classify)
             report_changes(to_file, old, new, specific_files, 
                            reporter, show_long_callback, 
                            short=short, want_unchanged=show_unchanged, 
-                           want_unversioned=want_unversioned, show_ids=show_ids)
+                           want_unversioned=want_unversioned, show_ids=show_ids,
+                           classify=classify)
 
             # show the ignored files among specific files (i.e. show the files
             # identified from input that we choose to ignore). 
@@ -189,7 +199,7 @@
                     prefix = 'C  '
                 else:
                     prefix = ' '
-                to_file.write("%s %s\n" % (prefix, conflict))
+                to_file.write("%s %s\n" % (prefix, unicode(conflict)))
             # Show files that were requested but don't exist (and are
             # not versioned).  We don't involve delta in this; these
             # paths are really the province of just the status
@@ -210,6 +220,9 @@
                 show_pending_merges(new, to_file, short, verbose=verbose)
             if nonexistents:
                 raise errors.PathsDoNotExist(nonexistents)
+            for hook in hooks['post_status']:
+                hook(StatusHookParams(old, new, to_file, versioned,
+                    show_ids, short, verbose))
         finally:
             old.unlock()
             new.unlock()
@@ -356,3 +369,105 @@
     # their groups individually.  But for consistency of this
     # function's API, it's better to sort both than just 'nonexistent'.
     return sorted(remaining), sorted(nonexistent)
+
+
+class StatusHooks(_mod_hooks.Hooks):
+    """A dictionary mapping hook name to a list of callables for status hooks.
+
+    e.g. ['post_status'] Is the list of items to be called when the
+    status command has finished printing the status.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+
+        These are all empty initially, because by default nothing should get
+        notified.
+        """
+        _mod_hooks.Hooks.__init__(self, "bzrlib.status", "hooks")
+        self.add_hook('post_status',
+            "Called with argument StatusHookParams after Bazaar has "
+            "displayed the status. StatusHookParams has the attributes "
+            "(old_tree, new_tree, to_file, versioned, show_ids, short, "
+            "verbose). The last four arguments correspond to the command "
+            "line options specified by the user for the status command. "
+            "to_file is the output stream for writing.",
+            (2, 3))
+        self.add_hook('pre_status',
+            "Called with argument StatusHookParams before Bazaar "
+            "displays the status. StatusHookParams has the attributes "
+            "(old_tree, new_tree, to_file, versioned, show_ids, short, "
+            "verbose). The last four arguments correspond to the command "
+            "line options specified by the user for the status command. "
+            "to_file is the output stream for writing.",
+            (2, 3))
+
+
+class StatusHookParams(object):
+    """Object holding parameters passed to post_status hooks.
+
+    :ivar old_tree: Start tree (basis tree) for comparison.
+    :ivar new_tree: Working tree.
+    :ivar to_file: If set, write to this file.
+    :ivar versioned: Show only versioned files.
+    :ivar show_ids: Show internal object ids.
+    :ivar short: Use short status indicators.
+    :ivar verbose: Verbose flag.
+    """
+
+    def __init__(self, old_tree, new_tree, to_file, versioned, show_ids,
+            short, verbose):
+        """Create a group of post_status hook parameters.
+
+        :param old_tree: Start tree (basis tree) for comparison.
+        :param new_tree: Working tree.
+        :param to_file: If set, write to this file.
+        :param versioned: Show only versioned files.
+        :param show_ids: Show internal object ids.
+        :param short: Use short status indicators.
+        :param verbose: Verbose flag.
+        """
+        self.old_tree = old_tree
+        self.new_tree = new_tree
+        self.to_file = to_file
+        self.versioned = versioned
+        self.show_ids = show_ids
+        self.short = short
+        self.verbose = verbose
+
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
+
+    def __repr__(self):
+        return "<%s(%s, %s, %s, %s, %s, %s, %s)>" % (self.__class__.__name__,
+            self.old_tree, self.new_tree, self.to_file, self.versioned,
+            self.show_ids, self.short, self.verbose)
+
+
+def _show_shelve_summary(params):
+    """post_status hook to display a summary of shelves.
+
+    :param params: StatusHookParams.
+    """
+    get_shelf_manager = getattr(params.new_tree, 'get_shelf_manager', None)
+    if get_shelf_manager is None:
+        return
+    manager = get_shelf_manager()
+    shelves = manager.active_shelves()
+    if shelves:
+        singular = '%d shelf exists. '
+        plural = '%d shelves exist. '
+        if len(shelves) == 1:
+            fmt = singular
+        else:
+            fmt = plural
+        params.to_file.write(fmt % len(shelves))
+        params.to_file.write('See "bzr shelve --list" for details.\n')
+
+
+hooks = StatusHooks()
+
+
+hooks.install_named_hook('post_status', _show_shelve_summary,
+    'bzr status')
+

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2010-05-25 16:45:12 +0000
+++ bzrlib/store/text.py	2010-05-31 21:31:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/store/versioned/__init__.py'
--- bzrlib/store/versioned/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/store/versioned/__init__.py	2011-01-19 20:39:41 +0000
@@ -18,21 +18,14 @@
 # files whose id differs only in case.  That should probably be forbidden.
 
 
-import errno
 import os
-from cStringIO import StringIO
 from warnings import warn
 
 from bzrlib import (
     errors,
     osutils,
     )
-from bzrlib.weavefile import read_weave, write_weave_v5
-from bzrlib.weave import WeaveFile, Weave
 from bzrlib.store import TransportStore
-from bzrlib.atomicfile import AtomicFile
-from bzrlib.symbol_versioning import (deprecated_method,
-        )
 from bzrlib.trace import mutter
 import bzrlib.ui
 
@@ -44,7 +37,7 @@
     # transport factory callable?
     def __init__(self, transport, prefixed=False, precious=False,
                  dir_mode=None, file_mode=None,
-                 versionedfile_class=WeaveFile,
+                 versionedfile_class=None,
                  versionedfile_kwargs={},
                  escaped=False):
         super(VersionedFileStore, self).__init__(transport,
@@ -213,14 +206,14 @@
         """
         from bzrlib.transactions import PassThroughTransaction
         if from_transaction is None:
-            warn("WeaveStore.copy_multi without a from_transaction parameter "
+            warn("VersionedFileStore.copy_multi without a from_transaction parameter "
                  "is deprecated. Please provide a from_transaction.",
                  DeprecationWarning,
                  stacklevel=2)
             # we are reading one object - caching is irrelevant.
             from_transaction = PassThroughTransaction()
         if to_transaction is None:
-            warn("WeaveStore.copy_multi without a to_transaction parameter "
+            warn("VersionedFileStore.copy_multi without a to_transaction parameter "
                  "is deprecated. Please provide a to_transaction.",
                  DeprecationWarning,
                  stacklevel=2)
@@ -246,5 +239,3 @@
     def total_size(self):
         count, bytes =  super(VersionedFileStore, self).total_size()
         return (count / len(self._versionedfile_class.get_suffixes())), bytes
-
-WeaveStore = VersionedFileStore

=== modified file 'bzrlib/strace.py'
--- bzrlib/strace.py	2009-03-23 14:59:43 +0000
+++ bzrlib/strace.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,12 @@
 import subprocess
 import tempfile
 
+from bzrlib import errors
+
+
 # this is currently test-focused, so importing bzrlib.tests is ok. We might
 # want to move feature to its own module though.
-from bzrlib.tests import Feature
+from bzrlib.tests.features import Feature
 
 
 def strace(function, *args, **kwargs):
@@ -46,14 +49,17 @@
     # capture strace output to a file
     log_file = tempfile.NamedTemporaryFile()
     log_file_fd = log_file.fileno()
+    err_file = tempfile.NamedTemporaryFile()
     pid = os.getpid()
     # start strace
     strace_cmd = ['strace', '-r', '-tt', '-p', str(pid), '-o', log_file.name]
     if follow_children:
         strace_args.append('-f')
+    # need to catch both stdout and stderr to work around
+    # bug 627208
     proc = subprocess.Popen(strace_cmd,
                             stdout=subprocess.PIPE,
-                            stderr=subprocess.STDOUT)
+                            stderr=err_file.fileno())
     # Wait for strace to attach
     attached_notice = proc.stdout.readline()
     # Run the function to strace
@@ -65,37 +71,30 @@
     log_file.seek(0)
     log = log_file.read()
     log_file.close()
-    return result, StraceResult(log)
+    # and stderr
+    err_file.seek(0)
+    err_messages = err_file.read()
+    err_file.close()
+    # and read any errors
+    if err_messages.startswith("attach: ptrace(PTRACE_ATTACH,"):
+        raise StraceError(err_messages=err_messages)
+    return result, StraceResult(log, err_messages)
+
+
+class StraceError(errors.BzrError):
+    
+    _fmt = "strace failed: %(err_messages)s"
 
 
 class StraceResult(object):
     """The result of stracing a function."""
 
-    def __init__(self, raw_log):
+    def __init__(self, raw_log, err_messages):
         """Create a StraceResult.
 
         :param raw_log: The output that strace created.
         """
         self.raw_log = raw_log
-
-
-class _StraceFeature(Feature):
-
-    def _probe(self):
-        try:
-            proc = subprocess.Popen(['strace'],
-                stderr=subprocess.PIPE,
-                stdout=subprocess.PIPE)
-            proc.communicate()
-            return True
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                # strace is not installed
-                return False
-            else:
-                raise
-
-    def feature_name(self):
-        return 'strace'
-
-StraceFeature = _StraceFeature()
+        self.err_messages = err_messages
+
+

=== modified file 'bzrlib/switch.py'
--- bzrlib/switch.py	2010-05-07 11:50:28 +0000
+++ bzrlib/switch.py	2011-10-11 12:01:51 +0000
@@ -18,9 +18,9 @@
 
 from bzrlib import errors, merge, revision
 from bzrlib.branch import Branch
+from bzrlib.i18n import gettext
 from bzrlib.trace import note
 
-
 def _run_post_switch_hooks(control_dir, to_branch, force, revision_id):
     from bzrlib.branch import SwitchHookParams
     hooks = Branch.hooks['post_switch']
@@ -33,7 +33,7 @@
 def switch(control_dir, to_branch, force=False, quiet=False, revision_id=None):
     """Switch the branch associated with a checkout.
 
-    :param control_dir: BzrDir of the checkout to change
+    :param control_dir: ControlDir of the checkout to change
     :param to_branch: branch that the checkout is to reference
     :param force: skip the check for local commits in a heavy checkout
     :param revision_id: revision ID to switch to.
@@ -51,7 +51,7 @@
 def _check_pending_merges(control, force=False):
     """Check that there are no outstanding pending merges before switching.
 
-    :param control: BzrDir of the branch to check
+    :param control: ControlDir of the branch to check
     """
     try:
         tree = control.open_workingtree()
@@ -64,14 +64,14 @@
     # XXX: Should the tree be locked for get_parent_ids?
     existing_pending_merges = tree.get_parent_ids()[1:]
     if len(existing_pending_merges) > 0:
-        raise errors.BzrCommandError('Pending merges must be '
-            'committed or reverted before using switch.')
+        raise errors.BzrCommandError(gettext('Pending merges must be '
+            'committed or reverted before using switch.'))
 
 
 def _set_branch_location(control, to_branch, force=False):
     """Set location value of a branch reference.
 
-    :param control: BzrDir of the checkout to change
+    :param control: ControlDir of the checkout to change
     :param to_branch: branch that the checkout is to reference
     :param force: skip the check for local commits in a heavy checkout
     """
@@ -90,22 +90,23 @@
             possible_transports = []
             try:
                 if not force and _any_local_commits(b, possible_transports):
-                    raise errors.BzrCommandError(
+                    raise errors.BzrCommandError(gettext(
                         'Cannot switch as local commits found in the checkout. '
                         'Commit these to the bound branch or use --force to '
-                        'throw them away.')
+                        'throw them away.'))
             except errors.BoundBranchConnectionFailure, e:
-                raise errors.BzrCommandError(
+                raise errors.BzrCommandError(gettext(
                         'Unable to connect to current master branch %(target)s: '
-                        '%(error)s To switch anyway, use --force.' %
+                        '%(error)s To switch anyway, use --force.') %
                         e.__dict__)
             b.set_bound_location(None)
             b.pull(to_branch, overwrite=True,
                 possible_transports=possible_transports)
             b.set_bound_location(to_branch.base)
+            b.set_parent(b.get_master_branch().get_parent())
         else:
-            raise errors.BzrCommandError('Cannot switch a branch, '
-                'only a checkout.')
+            raise errors.BzrCommandError(gettext('Cannot switch a branch, '
+                'only a checkout.'))
 
 
 def _any_local_commits(this_branch, possible_transports):
@@ -140,12 +141,12 @@
             revision_id = to_branch.last_revision()
         if tree.last_revision() == revision_id:
             if not quiet:
-                note("Tree is up to date at revision %d.", to_branch.revno())
+                note(gettext("Tree is up to date at revision %d."), to_branch.revno())
             return
         base_tree = source_repository.revision_tree(tree.last_revision())
         merge.Merge3Merger(tree, tree, base_tree, to_branch.repository.revision_tree(revision_id))
         tree.set_last_revision(to_branch.last_revision())
         if not quiet:
-            note('Updated to revision %d.' % to_branch.revno())
+            note(gettext('Updated to revision %d.') % to_branch.revno())
     finally:
         tree.unlock()

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2010-06-29 16:21:13 +0000
+++ bzrlib/symbol_versioning.py	2010-08-28 15:34:24 +0000
@@ -29,6 +29,9 @@
            'warn',
            ]
 
+
+import warnings
+# Import the 'warn' symbol so bzrlib can call it even if we redefine it
 from warnings import warn
 
 import bzrlib
@@ -296,7 +299,6 @@
     :param error_only: Only match an 'error' filter
     :return: True if a filter is found, False otherwise
     """
-    import warnings
     for filter in warnings.filters:
         if issubclass(DeprecationWarning, filter[2]):
             # This filter will effect DeprecationWarning
@@ -305,6 +307,19 @@
     return False
 
 
+def _remove_filter_callable(filter):
+    """Build and returns a callable removing filter from the warnings.
+
+    :param filter: The filter to remove (can be None).
+
+    :return: A callable that will remove filter from warnings.filters.
+    """
+    def cleanup():
+        if filter:
+            warnings.filters.remove(filter)
+    return cleanup
+
+
 def suppress_deprecation_warnings(override=True):
     """Call this function to suppress all deprecation warnings.
 
@@ -314,18 +329,17 @@
 
     :param override: If True, always set the ignore, if False, only set the
         ignore if there isn't already a filter.
+
     :return: A callable to remove the new warnings this added.
     """
-    import warnings
     if not override and _check_for_filter(error_only=False):
         # If there is already a filter effecting suppress_deprecation_warnings,
         # then skip it.
-        return
-    warnings.filterwarnings('ignore', category=DeprecationWarning)
-    filter = warnings.filters[0]
-    def cleanup():
-        warnings.filters.remove(filter)
-    return cleanup
+        filter = None
+    else:
+        warnings.filterwarnings('ignore', category=DeprecationWarning)
+        filter = warnings.filters[0]
+    return _remove_filter_callable(filter)
 
 
 def activate_deprecation_warnings(override=True):
@@ -342,10 +356,14 @@
     :param override: If False, only add a filter if there isn't an error filter
         already. (This slightly differs from suppress_deprecation_warnings, in
         because it always overrides everything but -Werror).
+
+    :return: A callable to remove the new warnings this added.
     """
-    import warnings
     if not override and _check_for_filter(error_only=True):
         # DeprecationWarnings are already turned into errors, don't downgrade
         # them to 'default'.
-        return
-    warnings.filterwarnings('default', category=DeprecationWarning)
+        filter = None
+    else:
+        warnings.filterwarnings('default', category=DeprecationWarning)
+        filter = warnings.filters[0]
+    return _remove_filter_callable(filter)

=== modified file 'bzrlib/tag.py'
--- bzrlib/tag.py	2010-03-25 09:39:03 +0000
+++ bzrlib/tag.py	2011-09-22 13:26:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,12 +25,21 @@
 # NOTE: I was going to call this tags.py, but vim seems to think all files
 # called tags* are ctags files... mbp 20070220.
 
+from bzrlib.registry import Registry
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import itertools
+import re
+import sys
 
 from bzrlib import (
     bencode,
+    cleanup,
     errors,
+    symbol_versioning,
     trace,
     )
+""")
 
 
 class _Tags(object):
@@ -38,6 +47,63 @@
     def __init__(self, branch):
         self.branch = branch
 
+    def get_tag_dict(self):
+        """Return a dictionary mapping tags to revision ids.
+        """
+        raise NotImplementedError(self.get_tag_dict)
+
+    def get_reverse_tag_dict(self):
+        """Return a dictionary mapping revision ids to list of tags.
+        """
+        raise NotImplementedError(self.get_reverse_tag_dict)
+
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
+        """Merge new tags from this tags container into another.
+
+        :param to_tags: Tags container to merge into
+        :param overwrite: Whether to overwrite existing, divergent, tags.
+        :param ignore_master: Do not modify the tags in the target's master
+            branch (if any).  Default is false (so the master will be updated).
+            New in bzr 2.3.
+        :return: Tuple with tag updates as dictionary and tag conflicts
+        """
+        raise NotImplementedError(self.merge_to)
+
+    def set_tag(self, tag_name, revision):
+        """Set a tag.
+
+        :param tag_name: Tag name
+        :param revision: Revision id
+        :raise GhostTagsNotSupported: if revision is not present in
+            the branch repository
+        """
+        raise NotImplementedError(self.set_tag)
+
+    def lookup_tag(self, tag_name):
+        """Look up a tag.
+
+        :param tag_name: Tag to look up
+        :raise NoSuchTag: Raised when tag does not exist
+        :return: Matching revision id
+        """
+        raise NotImplementedError(self.lookup_tag)
+
+    def delete_tag(self, tag_name):
+        """Delete a tag.
+
+        :param tag_name: Tag to delete
+        :raise NoSuchTag: Raised when tag does not exist
+        """
+        raise NotImplementedError(self.delete_tag)
+
+    def rename_revisions(self, rename_map):
+        """Replace revision ids according to a rename map.
+
+        :param rename_map: Dictionary mapping old revision ids to
+            new revision ids.
+        """
+        raise NotImplementedError(self.rename_revisions)
+
     def has_tag(self, tag_name):
         return self.get_tag_dict().has_key(tag_name)
 
@@ -57,9 +123,9 @@
     lookup_tag = _not_supported
     delete_tag = _not_supported
 
-    def merge_to(self, to_tags, overwrite=False):
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
         # we never have anything to copy
-        pass
+        return {}, []
 
     def rename_revisions(self, rename_map):
         # No tags, so nothing to rename
@@ -177,7 +243,7 @@
             raise ValueError("failed to deserialize tag dictionary %r: %s"
                 % (tag_content, e))
 
-    def merge_to(self, to_tags, overwrite=False):
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
         """Copy tags between repositories if necessary and possible.
 
         This method has common command-line behaviour about handling
@@ -188,35 +254,72 @@
 
         :param to_tags: Branch to receive these tags
         :param overwrite: Overwrite conflicting tags in the target branch
+        :param ignore_master: Do not modify the tags in the target's master
+            branch (if any).  Default is false (so the master will be updated).
+            New in bzr 2.3.
 
-        :returns: A list of tags that conflicted, each of which is
+        :returns: Tuple with tag_updates and tag_conflicts.
+            tag_updates is a dictionary with new tags, None is used for
+            removed tags
+            tag_conflicts is a set of tags that conflicted, each of which is
             (tagname, source_target, dest_target), or None if no copying was
             done.
         """
+        operation = cleanup.OperationWithCleanups(self._merge_to_operation)
+        return operation.run(to_tags, overwrite, ignore_master)
+
+    def _merge_to_operation(self, operation, to_tags, overwrite, ignore_master):
+        add_cleanup = operation.add_cleanup
         if self.branch == to_tags.branch:
-            return
+            return {}, []
         if not self.branch.supports_tags():
             # obviously nothing to copy
-            return
+            return {}, []
         source_dict = self.get_tag_dict()
         if not source_dict:
             # no tags in the source, and we don't want to clobber anything
             # that's in the destination
-            return
-        to_tags.branch.lock_write()
-        try:
-            dest_dict = to_tags.get_tag_dict()
-            result, conflicts = self._reconcile_tags(source_dict, dest_dict,
-                                                     overwrite)
-            if result != dest_dict:
-                to_tags._set_tag_dict(result)
-        finally:
-            to_tags.branch.unlock()
-        return conflicts
+            return {}, []
+        # We merge_to both master and child individually.
+        #
+        # It's possible for master and child to have differing sets of
+        # tags, in which case it's possible to have different sets of
+        # conflicts.  We report the union of both conflict sets.  In
+        # that case it's likely the child and master have accepted
+        # different tags from the source, which may be a surprising result, but
+        # the best we can do in the circumstances.
+        #
+        # Ideally we'd improve this API to report the different conflicts
+        # more clearly to the caller, but we don't want to break plugins
+        # such as bzr-builddeb that use this API.
+        add_cleanup(to_tags.branch.lock_write().unlock)
+        if ignore_master:
+            master = None
+        else:
+            master = to_tags.branch.get_master_branch()
+        if master is not None:
+            add_cleanup(master.lock_write().unlock)
+        updates, conflicts = self._merge_to(to_tags, source_dict, overwrite)
+        if master is not None:
+            extra_updates, extra_conflicts = self._merge_to(master.tags,
+                source_dict, overwrite)
+            updates.update(extra_updates)
+            conflicts += extra_conflicts
+        # We use set() to remove any duplicate conflicts from the master
+        # branch.
+        return updates, set(conflicts)
+
+    def _merge_to(self, to_tags, source_dict, overwrite):
+        dest_dict = to_tags.get_tag_dict()
+        result, updates, conflicts = self._reconcile_tags(source_dict,
+            dest_dict, overwrite)
+        if result != dest_dict:
+            to_tags._set_tag_dict(result)
+        return updates, conflicts
 
     def rename_revisions(self, rename_map):
         """Rename revisions in this tags dictionary.
-        
+
         :param rename_map: Dictionary mapping old revids to new revids
         """
         reverse_tags = self.get_reverse_tag_dict()
@@ -228,27 +331,97 @@
     def _reconcile_tags(self, source_dict, dest_dict, overwrite):
         """Do a two-way merge of two tag dictionaries.
 
-        only in source => source value
-        only in destination => destination value
-        same definitions => that
-        different definitions => if overwrite is False, keep destination
-            value and give a warning, otherwise use the source value
+        * only in source => source value
+        * only in destination => destination value
+        * same definitions => that
+        * different definitions => if overwrite is False, keep destination
+          value and give a warning, otherwise use the source value
 
-        :returns: (result_dict,
+        :returns: (result_dict, updates,
             [(conflicting_tag, source_target, dest_target)])
         """
         conflicts = []
+        updates = {}
         result = dict(dest_dict) # copy
         for name, target in source_dict.items():
-            if name not in result or overwrite:
+            if result.get(name) == target:
+                pass
+            elif name not in result or overwrite:
+                updates[name] = target
                 result[name] = target
-            elif result[name] == target:
-                pass
             else:
                 conflicts.append((name, target, result[name]))
-        return result, conflicts
-
-
-def _merge_tags_if_possible(from_branch, to_branch):
-    from_branch.tags.merge_to(to_branch.tags)
-
+        return result, updates, conflicts
+
+
+def _merge_tags_if_possible(from_branch, to_branch, ignore_master=False):
+    # Try hard to support merge_to implementations that don't expect
+    # 'ignore_master' (new in bzr 2.3).  First, if the flag isn't set then we
+    # can safely avoid passing ignore_master at all.
+    if not ignore_master:
+        from_branch.tags.merge_to(to_branch.tags)
+        return
+    # If the flag is set, try to pass it, but be ready to catch TypeError.
+    try:
+        from_branch.tags.merge_to(to_branch.tags, ignore_master=ignore_master)
+    except TypeError:
+        # Probably this implementation of 'merge_to' is from a plugin that
+        # doesn't expect the 'ignore_master' keyword argument (e.g. bzr-svn
+        # 1.0.4).  There's a small risk that the TypeError is actually caused
+        # by a completely different problem (which is why we don't catch it for
+        # the ignore_master=False case), but even then there's probably no harm
+        # in calling a second time.
+        symbol_versioning.warn(
+            symbol_versioning.deprecated_in((2,3)) % (
+                "Tags.merge_to (of %r) that doesn't accept ignore_master kwarg"
+                % (from_branch.tags,),),
+            DeprecationWarning)
+        from_branch.tags.merge_to(to_branch.tags)
+
+
+def sort_natural(branch, tags):
+    """Sort tags, with numeric substrings as numbers.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    def natural_sort_key(tag):
+        return [f(s) for f,s in
+                zip(itertools.cycle((unicode.lower,int)),
+                                    re.split('([0-9]+)', tag[0]))]
+    tags.sort(key=natural_sort_key)
+
+
+def sort_alpha(branch, tags):
+    """Sort tags lexicographically, in place.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    tags.sort()
+
+
+def sort_time(branch, tags):
+    """Sort tags by time inline.
+
+    :param branch: Branch
+    :param tags: List of tuples with tag name and revision id.
+    """
+    timestamps = {}
+    for tag, revid in tags:
+        try:
+            revobj = branch.repository.get_revision(revid)
+        except errors.NoSuchRevision:
+            timestamp = sys.maxint # place them at the end
+        else:
+            timestamp = revobj.timestamp
+        timestamps[revid] = timestamp
+    tags.sort(key=lambda x: timestamps[x[1]])
+
+
+tag_sort_methods = Registry()
+tag_sort_methods.register("natural", sort_natural,
+    'Sort numeric substrings as numbers. (default)')
+tag_sort_methods.register("alpha", sort_alpha, 'Sort tags lexicographically.')
+tag_sort_methods.register("time", sort_time, 'Sort tags chronologically.')
+tag_sort_methods.default_key = "natural"

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2009-03-23 14:59:43 +0000
+++ bzrlib/testament.py	2011-05-11 14:28:36 +0000
@@ -74,8 +74,9 @@
 from bzrlib.osutils import (
     contains_whitespace,
     contains_linebreaks,
-    sha,
+    sha_strings,
     )
+from bzrlib.tree import Tree
 
 
 class Testament(object):
@@ -91,23 +92,33 @@
 
     long_header = 'bazaar-ng testament version 1\n'
     short_header = 'bazaar-ng testament short form 1\n'
+    include_root = False
 
     @classmethod
     def from_revision(cls, repository, revision_id):
-        """Produce a new testament from a historical revision"""
+        """Produce a new testament from a historical revision."""
         rev = repository.get_revision(revision_id)
-        inventory = repository.get_inventory(revision_id)
-        return cls(rev, inventory)
-
-    def __init__(self, rev, inventory):
-        """Create a new testament for rev using inventory."""
+        tree = repository.revision_tree(revision_id)
+        return cls(rev, tree)
+
+    @classmethod
+    def from_revision_tree(cls, tree):
+        """Produce a new testament from a revision tree."""
+        rev = tree._repository.get_revision(tree.get_revision_id())
+        return cls(rev, tree)
+
+    def __init__(self, rev, tree):
+        """Create a new testament for rev using tree."""
         self.revision_id = rev.revision_id
         self.committer = rev.committer
         self.timezone = rev.timezone or 0
         self.timestamp = rev.timestamp
         self.message = rev.message
         self.parent_ids = rev.parent_ids[:]
-        self.inventory = inventory
+        if not isinstance(tree, Tree):
+            raise TypeError("As of bzr 2.4 Testament.__init__() takes a "
+                "Revision and a Tree.")
+        self.tree = tree
         self.revprops = copy(rev.properties)
         if contains_whitespace(self.revision_id):
             raise ValueError(self.revision_id)
@@ -143,9 +154,8 @@
         return [line.encode('utf-8') for line in r]
 
     def _get_entries(self):
-        entries = self.inventory.iter_entries()
-        entries.next()
-        return entries
+        return ((path, ie) for (path, versioned, kind, file_id, ie) in
+                self.tree.list_files(include_root=self.include_root))
 
     def _escape_path(self, path):
         if contains_linebreaks(path):
@@ -199,9 +209,7 @@
         return r
 
     def as_sha1(self):
-        s = sha()
-        map(s.update, self.as_text_lines())
-        return s.hexdigest()
+        return sha_strings(self.as_text_lines())
 
 
 class StrictTestament(Testament):
@@ -209,6 +217,7 @@
 
     long_header = 'bazaar-ng testament version 2.1\n'
     short_header = 'bazaar-ng testament short form 2.1\n'
+    include_root = False
     def _entry_to_line(self, path, ie):
         l = Testament._entry_to_line(self, path, ie)[:-1]
         l += ' ' + ie.revision
@@ -224,8 +233,7 @@
 
     long_header = 'bazaar testament version 3 strict\n'
     short_header = 'bazaar testament short form 3 strict\n'
-    def _get_entries(self):
-        return self.inventory.iter_entries()
+    include_root = True
 
     def _escape_path(self, path):
         if contains_linebreaks(path):

=== modified file 'bzrlib/tests/EncodingAdapter.py'
--- bzrlib/tests/EncodingAdapter.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/EncodingAdapter.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010, 2011 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,11 +17,6 @@
 
 """Adapter for running test cases against multiple encodings."""
 
-from copy import deepcopy
-
-from bzrlib.tests import TestSuite
-
-
 # prefix for micro (1/1000000)
 _mu = u'\xb5'
 

=== modified file 'bzrlib/tests/TestUtil.py'
--- bzrlib/tests/TestUtil.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/TestUtil.py	2011-10-04 16:39:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #       Author: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,9 @@
 import sys
 import logging
 import unittest
+import weakref
+
+from bzrlib import pyutils
 
 # Mark this python module as being part of the implementation
 # of unittest: this gives us better tracebacks where the last
@@ -27,9 +30,11 @@
 
 
 class LogCollector(logging.Handler):
+
     def __init__(self):
         logging.Handler.__init__(self)
         self.records=[]
+
     def emit(self, record):
         self.records.append(record.getMessage())
 
@@ -58,7 +63,26 @@
                 visitor.visitSuite(test)
                 visitTests(test, visitor)
             else:
-                print "unvisitable non-unittest.TestCase element %r (%r)" % (test, test.__class__)
+                print "unvisitable non-unittest.TestCase element %r (%r)" % (
+                    test, test.__class__)
+
+
+class FailedCollectionCase(unittest.TestCase):
+    """Pseudo-test to run and report failure if given case was uncollected"""
+
+    def __init__(self, case):
+        super(FailedCollectionCase, self).__init__("fail_uncollected")
+        # GZ 2011-09-16: Maybe catch errors from id() method as cases may be
+        #                in a bit of a funny state by now.
+        self._problem_case_id = case.id()
+
+    def id(self):
+        if self._problem_case_id[-1:] == ")":
+            return self._problem_case_id[:-1] + ",uncollected)"
+        return self._problem_case_id + "(uncollected)"
+
+    def fail_uncollected(self):
+        self.fail("Uncollected test case: " + self._problem_case_id)
 
 
 class TestSuite(unittest.TestSuite):
@@ -77,14 +101,34 @@
         tests = list(self)
         tests.reverse()
         self._tests = []
+        stored_count = 0
+        count_stored_tests = getattr(result, "_count_stored_tests", int)
+        from bzrlib.tests import selftest_debug_flags
+        notify = "uncollected_cases" in selftest_debug_flags
         while tests:
             if result.shouldStop:
                 self._tests = reversed(tests)
                 break
-            tests.pop().run(result)
+            case = _run_and_collect_case(tests.pop(), result)()
+            new_stored_count = count_stored_tests()
+            if case is not None and isinstance(case, unittest.TestCase):
+                if stored_count == new_stored_count and notify:
+                    # Testcase didn't fail, but somehow is still alive
+                    FailedCollectionCase(case).run(result)
+                    # Adding a new failure so need to reupdate the count
+                    new_stored_count = count_stored_tests()
+                # GZ 2011-09-16: Previously zombied the case at this point by
+                #                clearing the dict as fallback, skip for now.
+            stored_count = new_stored_count
         return result
 
 
+def _run_and_collect_case(case, res):
+    """Run test case against result and use weakref to drop the refcount"""
+    case.run(res)
+    return weakref.ref(case)
+
+
 class TestLoader(unittest.TestLoader):
     """Custom TestLoader to extend the stock python one."""
 
@@ -106,7 +150,7 @@
 
     def loadTestsFromModuleName(self, name):
         result = self.suiteClass()
-        module = _load_module_by_name(name)
+        module = pyutils.get_named_object(name)
 
         result.addTests(self.loadTestsFromModule(module))
         return result
@@ -135,7 +179,13 @@
         >>>         result.addTests([test, test])
         >>>     return result
         """
-        basic_tests = super(TestLoader, self).loadTestsFromModule(module)
+        if sys.version_info < (2, 7):
+            basic_tests = super(TestLoader, self).loadTestsFromModule(module)
+        else:
+            # GZ 2010-07-19: Python 2.7 unittest also uses load_tests but with
+            #                a different and incompatible signature
+            basic_tests = super(TestLoader, self).loadTestsFromModule(module,
+                use_load_tests=False)
         load_tests = getattr(module, "load_tests", None)
         if load_tests is not None:
             return load_tests(basic_tests, module, self)
@@ -173,20 +223,11 @@
             return self.suiteClass()
 
 
-def _load_module_by_name(mod_name):
-    parts = mod_name.split('.')
-    module = __import__(mod_name)
-    del parts[0]
-    # for historical reasons python returns the top-level module even though
-    # it loads the submodule; we need to walk down to get the one we want.
-    while parts:
-        module = getattr(module, parts.pop(0))
-    return module
-
-
 class TestVisitor(object):
     """A visitor for Tests"""
+
     def visitSuite(self, aTestSuite):
         pass
+
     def visitCase(self, aTestCase):
         pass

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2010-07-29 11:17:57 +0000
+++ bzrlib/tests/__init__.py	2011-10-14 01:48:16 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,12 +16,6 @@
 
 """Testing framework extensions"""
 
-# TODO: Perhaps there should be an API to find out if bzr running under the
-# test suite -- some plugins might want to avoid making intrusive changes if
-# this is the case.  However, we want behaviour under to test to diverge as
-# little as possible, so this should be used rarely if it's added at all.
-# (Suggestion from j-a-meinel, 2005-11-24)
-
 # NOTE: Some classes in here use camelCaseNaming() rather than
 # underscore_naming().  That's for consistency with unittest; it's not the
 # general style of bzrlib.  Please continue that consistency when adding e.g.
@@ -34,9 +28,10 @@
 import difflib
 import doctest
 import errno
+import itertools
 import logging
-import math
 import os
+import platform
 import pprint
 import random
 import re
@@ -54,74 +49,57 @@
 import testtools
 # nb: check this before importing anything else from within it
 _testtools_version = getattr(testtools, '__version__', ())
-if _testtools_version < (0, 9, 2):
-    raise ImportError("need at least testtools 0.9.2: %s is %r"
+if _testtools_version < (0, 9, 5):
+    raise ImportError("need at least testtools 0.9.5: %s is %r"
         % (testtools.__file__, _testtools_version))
 from testtools import content
 
+import bzrlib
 from bzrlib import (
     branchbuilder,
     bzrdir,
     chk_map,
+    commands as _mod_commands,
     config,
+    i18n,
     debug,
     errors,
     hooks,
     lock as _mod_lock,
+    lockdir,
     memorytree,
     osutils,
-    progress,
+    plugin as _mod_plugin,
+    pyutils,
     ui,
     urlutils,
     registry,
+    symbol_versioning,
+    trace,
     transport as _mod_transport,
     workingtree,
     )
-import bzrlib.branch
-import bzrlib.commands
-import bzrlib.timestamp
-import bzrlib.export
-import bzrlib.inventory
-import bzrlib.iterablefile
-import bzrlib.lockdir
 try:
     import bzrlib.lsprof
 except ImportError:
     # lsprof not available
     pass
-from bzrlib.merge import merge_inner
-import bzrlib.merge3
-import bzrlib.plugin
-from bzrlib.smart import client, request, server
-import bzrlib.store
-from bzrlib import symbol_versioning
+from bzrlib.smart import client, request
+from bzrlib.transport import (
+    memory,
+    pathfilter,
+    )
 from bzrlib.symbol_versioning import (
-    DEPRECATED_PARAMETER,
     deprecated_function,
     deprecated_in,
-    deprecated_method,
-    deprecated_passed,
-    )
-import bzrlib.trace
-from bzrlib.transport import (
-    memory,
-    pathfilter,
-    )
-from bzrlib.trace import mutter, note
+    )
 from bzrlib.tests import (
     test_server,
     TestUtil,
     treeshape,
     )
-from bzrlib.tests.http_server import HttpServer
-from bzrlib.tests.TestUtil import (
-                          TestSuite,
-                          TestLoader,
-                          )
 from bzrlib.ui import NullProgressView
 from bzrlib.ui.text import TextUIFactory
-import bzrlib.version_info_formats.format_custom
-from bzrlib.workingtree import WorkingTree, WorkingTreeFormat2
 
 # Mark this python module as being part of the implementation
 # of unittest: this gives us better tracebacks where the last
@@ -139,8 +117,123 @@
 SUBUNIT_SEEK_SET = 0
 SUBUNIT_SEEK_CUR = 1
 
-
-class ExtendedTestResult(unittest._TextTestResult):
+# These are intentionally brought into this namespace. That way plugins, etc
+# can just "from bzrlib.tests import TestCase, TestLoader, etc"
+TestSuite = TestUtil.TestSuite
+TestLoader = TestUtil.TestLoader
+
+# Tests should run in a clean and clearly defined environment. The goal is to
+# keep them isolated from the running environment as mush as possible. The test
+# framework ensures the variables defined below are set (or deleted if the
+# value is None) before a test is run and reset to their original value after
+# the test is run. Generally if some code depends on an environment variable,
+# the tests should start without this variable in the environment. There are a
+# few exceptions but you shouldn't violate this rule lightly.
+isolated_environ = {
+    'BZR_HOME': None,
+    'HOME': None,
+    # bzr now uses the Win32 API and doesn't rely on APPDATA, but the
+    # tests do check our impls match APPDATA
+    'BZR_EDITOR': None, # test_msgeditor manipulates this variable
+    'VISUAL': None,
+    'EDITOR': None,
+    'BZR_EMAIL': None,
+    'BZREMAIL': None, # may still be present in the environment
+    'EMAIL': 'jrandom@example.com', # set EMAIL as bzr does not guess
+    'BZR_PROGRESS_BAR': None,
+    # This should trap leaks to ~/.bzr.log. This occurs when tests use TestCase
+    # as a base class instead of TestCaseInTempDir. Tests inheriting from
+    # TestCase should not use disk resources, BZR_LOG is one.
+    'BZR_LOG': '/you-should-use-TestCaseInTempDir-if-you-need-a-log-file',
+    'BZR_PLUGIN_PATH': None,
+    'BZR_DISABLE_PLUGINS': None,
+    'BZR_PLUGINS_AT': None,
+    'BZR_CONCURRENCY': None,
+    # Make sure that any text ui tests are consistent regardless of
+    # the environment the test case is run in; you may want tests that
+    # test other combinations.  'dumb' is a reasonable guess for tests
+    # going to a pipe or a StringIO.
+    'TERM': 'dumb',
+    'LINES': '25',
+    'COLUMNS': '80',
+    'BZR_COLUMNS': '80',
+    # Disable SSH Agent
+    'SSH_AUTH_SOCK': None,
+    # Proxies
+    'http_proxy': None,
+    'HTTP_PROXY': None,
+    'https_proxy': None,
+    'HTTPS_PROXY': None,
+    'no_proxy': None,
+    'NO_PROXY': None,
+    'all_proxy': None,
+    'ALL_PROXY': None,
+    # Nobody cares about ftp_proxy, FTP_PROXY AFAIK. So far at
+    # least. If you do (care), please update this comment
+    # -- vila 20080401
+    'ftp_proxy': None,
+    'FTP_PROXY': None,
+    'BZR_REMOTE_PATH': None,
+    # Generally speaking, we don't want apport reporting on crashes in
+    # the test envirnoment unless we're specifically testing apport,
+    # so that it doesn't leak into the real system environment.  We
+    # use an env var so it propagates to subprocesses.
+    'APPORT_DISABLE': '1',
+    }
+
+
+def override_os_environ(test, env=None):
+    """Modify os.environ keeping a copy.
+    
+    :param test: A test instance
+
+    :param env: A dict containing variable definitions to be installed
+    """
+    if env is None:
+        env = isolated_environ
+    test._original_os_environ = dict([(var, value)
+                                      for var, value in os.environ.iteritems()])
+    for var, value in env.iteritems():
+        osutils.set_or_unset_env(var, value)
+        if var not in test._original_os_environ:
+            # The var is new, add it with a value of None, so
+            # restore_os_environ will delete it
+            test._original_os_environ[var] = None
+
+
+def restore_os_environ(test):
+    """Restore os.environ to its original state.
+
+    :param test: A test instance previously passed to override_os_environ.
+    """
+    for var, value in test._original_os_environ.iteritems():
+        # Restore the original value (or delete it if the value has been set to
+        # None in override_os_environ).
+        osutils.set_or_unset_env(var, value)
+
+
+def _clear__type_equality_funcs(test):
+    """Cleanup bound methods stored on TestCase instances
+
+    Clear the dict breaking a few (mostly) harmless cycles in the affected
+    unittests released with Python 2.6 and initial Python 2.7 versions.
+
+    For a few revisions between Python 2.7.1 and Python 2.7.2 that annoyingly
+    shipped in Oneiric, an object with no clear method was used, hence the
+    extra complications, see bug 809048 for details.
+    """
+    type_equality_funcs = getattr(test, "_type_equality_funcs", None)
+    if type_equality_funcs is not None:
+        tef_clear = getattr(type_equality_funcs, "clear", None)
+        if tef_clear is None:
+            tef_instance_dict = getattr(type_equality_funcs, "__dict__", None)
+            if tef_instance_dict is not None:
+                tef_clear = tef_instance_dict.clear
+        if tef_clear is not None:
+            tef_clear()
+
+
+class ExtendedTestResult(testtools.TextTestResult):
     """Accepts, reports and accumulates the results of running tests.
 
     Compared to the unittest version this class adds support for
@@ -167,7 +260,7 @@
         :param bench_history: Optionally, a writable file object to accumulate
             benchmark results.
         """
-        unittest._TextTestResult.__init__(self, stream, descriptions, verbosity)
+        testtools.TextTestResult.__init__(self, stream)
         if bench_history is not None:
             from bzrlib.version import _get_bzr_source_tree
             src_tree = _get_bzr_source_tree()
@@ -194,17 +287,22 @@
         self.count = 0
         self._overall_start_time = time.time()
         self._strict = strict
+        self._first_thread_leaker_id = None
+        self._tests_leaking_threads_count = 0
+        self._traceback_from_test = None
 
     def stopTestRun(self):
         run = self.testsRun
         actionTaken = "Ran"
         stopTime = time.time()
         timeTaken = stopTime - self.startTime
-        self.printErrors()
-        self.stream.writeln(self.separator2)
-        self.stream.writeln("%s %d test%s in %.3fs" % (actionTaken,
+        # GZ 2010-07-19: Seems testtools has no printErrors method, and though
+        #                the parent class method is similar have to duplicate
+        self._show_list('ERROR', self.errors)
+        self._show_list('FAIL', self.failures)
+        self.stream.write(self.sep2)
+        self.stream.write("%s %d test%s in %.3fs\n\n" % (actionTaken,
                             run, run != 1 and "s" or "", timeTaken))
-        self.stream.writeln()
         if not self.wasSuccessful():
             self.stream.write("FAILED (")
             failed, errored = map(len, (self.failures, self.errors))
@@ -217,34 +315,34 @@
                 if failed or errored: self.stream.write(", ")
                 self.stream.write("known_failure_count=%d" %
                     self.known_failure_count)
-            self.stream.writeln(")")
+            self.stream.write(")\n")
         else:
             if self.known_failure_count:
-                self.stream.writeln("OK (known_failures=%d)" %
+                self.stream.write("OK (known_failures=%d)\n" %
                     self.known_failure_count)
             else:
-                self.stream.writeln("OK")
+                self.stream.write("OK\n")
         if self.skip_count > 0:
             skipped = self.skip_count
-            self.stream.writeln('%d test%s skipped' %
+            self.stream.write('%d test%s skipped\n' %
                                 (skipped, skipped != 1 and "s" or ""))
         if self.unsupported:
             for feature, count in sorted(self.unsupported.items()):
-                self.stream.writeln("Missing feature '%s' skipped %d tests." %
+                self.stream.write("Missing feature '%s' skipped %d tests.\n" %
                     (feature, count))
         if self._strict:
             ok = self.wasStrictlySuccessful()
         else:
             ok = self.wasSuccessful()
-        if TestCase._first_thread_leaker_id:
+        if self._first_thread_leaker_id:
             self.stream.write(
                 '%s is leaking threads among %d leaking tests.\n' % (
-                TestCase._first_thread_leaker_id,
-                TestCase._leaking_threads_tests))
+                self._first_thread_leaker_id,
+                self._tests_leaking_threads_count))
             # We don't report the main thread as an active one.
             self.stream.write(
                 '%d non-main threads were left active in the end.\n'
-                % (TestCase._active_threads - 1))
+                % (len(self._active_threads) - 1))
 
     def getDescription(self, test):
         return test.id()
@@ -257,7 +355,8 @@
 
     def _elapsedTestTimeString(self):
         """Return a time string for the overall time the current test has taken."""
-        return self._formatTime(time.time() - self._start_time)
+        return self._formatTime(self._delta_to_float(
+            self._now() - self._start_datetime))
 
     def _testTimeString(self, testCase):
         benchmark_time = self._extractBenchmarkTime(testCase)
@@ -277,43 +376,62 @@
         what = re.sub(r'^bzrlib\.tests\.', '', what)
         return what
 
+    # GZ 2010-10-04: Cloned tests may end up harmlessly calling this method
+    #                multiple times in a row, because the handler is added for
+    #                each test but the container list is shared between cases.
+    #                See lp:498869 lp:625574 and lp:637725 for background.
+    def _record_traceback_from_test(self, exc_info):
+        """Store the traceback from passed exc_info tuple till"""
+        self._traceback_from_test = exc_info[2]
+
     def startTest(self, test):
-        unittest.TestResult.startTest(self, test)
+        super(ExtendedTestResult, self).startTest(test)
         if self.count == 0:
             self.startTests()
+        self.count += 1
         self.report_test_start(test)
         test.number = self.count
         self._recordTestStartTime()
+        # Make testtools cases give us the real traceback on failure
+        addOnException = getattr(test, "addOnException", None)
+        if addOnException is not None:
+            addOnException(self._record_traceback_from_test)
+        # Only check for thread leaks on bzrlib derived test cases
+        if isinstance(test, TestCase):
+            test.addCleanup(self._check_leaked_threads, test)
+
+    def stopTest(self, test):
+        super(ExtendedTestResult, self).stopTest(test)
+        # Manually break cycles, means touching various private things but hey
+        getDetails = getattr(test, "getDetails", None)
+        if getDetails is not None:
+            getDetails().clear()
+        _clear__type_equality_funcs(test)
+        self._traceback_from_test = None
 
     def startTests(self):
-        import platform
-        if getattr(sys, 'frozen', None) is None:
-            bzr_path = osutils.realpath(sys.argv[0])
-        else:
-            bzr_path = sys.executable
-        self.stream.write(
-            'bzr selftest: %s\n' % (bzr_path,))
-        self.stream.write(
-            '   %s\n' % (
-                    bzrlib.__path__[0],))
-        self.stream.write(
-            '   bzr-%s python-%s %s\n' % (
-                    bzrlib.version_string,
-                    bzrlib._format_version_tuple(sys.version_info),
-                    platform.platform(aliased=1),
-                    ))
-        self.stream.write('\n')
+        self.report_tests_starting()
+        self._active_threads = threading.enumerate()
+
+    def _check_leaked_threads(self, test):
+        """See if any threads have leaked since last call
+
+        A sample of live threads is stored in the _active_threads attribute,
+        when this method runs it compares the current live threads and any not
+        in the previous sample are treated as having leaked.
+        """
+        now_active_threads = set(threading.enumerate())
+        threads_leaked = now_active_threads.difference(self._active_threads)
+        if threads_leaked:
+            self._report_thread_leak(test, threads_leaked, now_active_threads)
+            self._tests_leaking_threads_count += 1
+            if self._first_thread_leaker_id is None:
+                self._first_thread_leaker_id = test.id()
+            self._active_threads = now_active_threads
 
     def _recordTestStartTime(self):
         """Record that a test has started."""
-        self._start_time = time.time()
-
-    def _cleanupLogFile(self, test):
-        # We can only do this if we have one of our TestCases, not if
-        # we have a doctest.
-        setKeepLogfile = getattr(test, 'setKeepLogfile', None)
-        if setKeepLogfile is not None:
-            setKeepLogfile()
+        self._start_datetime = self._now()
 
     def addError(self, test, err):
         """Tell result that test finished with an error.
@@ -321,13 +439,12 @@
         Called from the TestCase run() method when the test
         fails with an unexpected error.
         """
-        self._post_mortem()
-        unittest.TestResult.addError(self, test, err)
+        self._post_mortem(self._traceback_from_test)
+        super(ExtendedTestResult, self).addError(test, err)
         self.error_count += 1
         self.report_error(test, err)
         if self.stop_early:
             self.stop()
-        self._cleanupLogFile(test)
 
     def addFailure(self, test, err):
         """Tell result that test failed.
@@ -335,13 +452,12 @@
         Called from the TestCase run() method when the test
         fails because e.g. an assert() method failed.
         """
-        self._post_mortem()
-        unittest.TestResult.addFailure(self, test, err)
+        self._post_mortem(self._traceback_from_test)
+        super(ExtendedTestResult, self).addFailure(test, err)
         self.failure_count += 1
         self.report_failure(test, err)
         if self.stop_early:
             self.stop()
-        self._cleanupLogFile(test)
 
     def addSuccess(self, test, details=None):
         """Tell result that test completed successfully.
@@ -355,14 +471,26 @@
                     self._formatTime(benchmark_time),
                     test.id()))
         self.report_success(test)
-        self._cleanupLogFile(test)
-        unittest.TestResult.addSuccess(self, test)
+        super(ExtendedTestResult, self).addSuccess(test)
         test._log_contents = ''
 
     def addExpectedFailure(self, test, err):
         self.known_failure_count += 1
         self.report_known_failure(test, err)
 
+    def addUnexpectedSuccess(self, test, details=None):
+        """Tell result the test unexpectedly passed, counting as a failure
+
+        When the minimum version of testtools required becomes 0.9.8 this
+        can be updated to use the new handling there.
+        """
+        super(ExtendedTestResult, self).addFailure(test, details=details)
+        self.failure_count += 1
+        self.report_unexpected_success(test,
+            "".join(details["reason"].iter_text()))
+        if self.stop_early:
+            self.stop()
+
     def addNotSupported(self, test, feature):
         """The test will not be run because of a missing feature.
         """
@@ -385,10 +513,15 @@
         self.not_applicable_count += 1
         self.report_not_applicable(test, reason)
 
-    def _post_mortem(self):
+    def _count_stored_tests(self):
+        """Count of tests instances kept alive due to not succeeding"""
+        return self.error_count + self.failure_count + self.known_failure_count
+
+    def _post_mortem(self, tb=None):
         """Start a PDB post mortem session."""
         if os.environ.get('BZR_TEST_PDB', None):
-            import pdb;pdb.post_mortem()
+            import pdb
+            pdb.post_mortem(tb)
 
     def progress(self, offset, whence):
         """The test is adjusting the count of tests to run."""
@@ -399,8 +532,37 @@
         else:
             raise errors.BzrError("Unknown whence %r" % whence)
 
-    def report_cleaning_up(self):
-        pass
+    def report_tests_starting(self):
+        """Display information before the test run begins"""
+        if getattr(sys, 'frozen', None) is None:
+            bzr_path = osutils.realpath(sys.argv[0])
+        else:
+            bzr_path = sys.executable
+        self.stream.write(
+            'bzr selftest: %s\n' % (bzr_path,))
+        self.stream.write(
+            '   %s\n' % (
+                    bzrlib.__path__[0],))
+        self.stream.write(
+            '   bzr-%s python-%s %s\n' % (
+                    bzrlib.version_string,
+                    bzrlib._format_version_tuple(sys.version_info),
+                    platform.platform(aliased=1),
+                    ))
+        self.stream.write('\n')
+
+    def report_test_start(self, test):
+        """Display information on the test just about to be run"""
+
+    def _report_thread_leak(self, test, leaked_threads, active_threads):
+        """Display information on a test that leaked one or more threads"""
+        # GZ 2010-09-09: A leak summary reported separately from the general
+        #                thread debugging would be nice. Tests under subunit
+        #                need something not using stream, perhaps adding a
+        #                testtools details object would be fitting.
+        if 'threads' in selftest_debug_flags:
+            self.stream.write('%s is leaking, active is now %d\n' %
+                (test.id(), len(active_threads)))
 
     def startTestRun(self):
         self.startTime = time.time()
@@ -443,15 +605,10 @@
         self.pb.finished()
         super(TextTestResult, self).stopTestRun()
 
-    def startTestRun(self):
-        super(TextTestResult, self).startTestRun()
+    def report_tests_starting(self):
+        super(TextTestResult, self).report_tests_starting()
         self.pb.update('[test 0/%d] Starting' % (self.num_tests))
 
-    def printErrors(self):
-        # clear the pb to make room for the error listing
-        self.pb.clear()
-        super(TextTestResult, self).printErrors()
-
     def _progress_prefix_text(self):
         # the longer this text, the less space we have to show the test
         # name...
@@ -479,7 +636,6 @@
         return a
 
     def report_test_start(self, test):
-        self.count += 1
         self.pb.update(
                 self._progress_prefix_text()
                 + ' '
@@ -503,6 +659,13 @@
     def report_known_failure(self, test, err):
         pass
 
+    def report_unexpected_success(self, test, reason):
+        self.stream.write('FAIL: %s\n    %s: %s\n' % (
+            self._test_description(test),
+            "Unexpected success. Should have failed",
+            reason,
+            ))
+
     def report_skip(self, test, reason):
         pass
 
@@ -512,9 +675,6 @@
     def report_unsupported(self, test, feature):
         """test cannot be run because feature is missing."""
 
-    def report_cleaning_up(self):
-        self.pb.update('Cleaning up')
-
 
 class VerboseTestResult(ExtendedTestResult):
     """Produce long output, with one line per test run plus times"""
@@ -527,12 +687,11 @@
             result = a_string
         return result.ljust(final_width)
 
-    def startTestRun(self):
-        super(VerboseTestResult, self).startTestRun()
+    def report_tests_starting(self):
         self.stream.write('running %d tests...\n' % self.num_tests)
+        super(VerboseTestResult, self).report_tests_starting()
 
     def report_test_start(self, test):
-        self.count += 1
         name = self._shortened_test_description(test)
         width = osutils.terminal_width()
         if width is not None:
@@ -550,40 +709,46 @@
         return '%s%s' % (indent, err[1])
 
     def report_error(self, test, err):
-        self.stream.writeln('ERROR %s\n%s'
+        self.stream.write('ERROR %s\n%s\n'
                 % (self._testTimeString(test),
                    self._error_summary(err)))
 
     def report_failure(self, test, err):
-        self.stream.writeln(' FAIL %s\n%s'
+        self.stream.write(' FAIL %s\n%s\n'
                 % (self._testTimeString(test),
                    self._error_summary(err)))
 
     def report_known_failure(self, test, err):
-        self.stream.writeln('XFAIL %s\n%s'
+        self.stream.write('XFAIL %s\n%s\n'
                 % (self._testTimeString(test),
                    self._error_summary(err)))
 
+    def report_unexpected_success(self, test, reason):
+        self.stream.write(' FAIL %s\n%s: %s\n'
+                % (self._testTimeString(test),
+                   "Unexpected success. Should have failed",
+                   reason))
+
     def report_success(self, test):
-        self.stream.writeln('   OK %s' % self._testTimeString(test))
+        self.stream.write('   OK %s\n' % self._testTimeString(test))
         for bench_called, stats in getattr(test, '_benchcalls', []):
-            self.stream.writeln('LSProf output for %s(%s, %s)' % bench_called)
+            self.stream.write('LSProf output for %s(%s, %s)\n' % bench_called)
             stats.pprint(file=self.stream)
         # flush the stream so that we get smooth output. This verbose mode is
         # used to show the output in PQM.
         self.stream.flush()
 
     def report_skip(self, test, reason):
-        self.stream.writeln(' SKIP %s\n%s'
+        self.stream.write(' SKIP %s\n%s\n'
                 % (self._testTimeString(test), reason))
 
     def report_not_applicable(self, test, reason):
-        self.stream.writeln('  N/A %s\n    %s'
+        self.stream.write('  N/A %s\n    %s\n'
                 % (self._testTimeString(test), reason))
 
     def report_unsupported(self, test, feature):
         """test cannot be run because feature is missing."""
-        self.stream.writeln("NODEP %s\n    The feature '%s' is not available."
+        self.stream.write("NODEP %s\n    The feature '%s' is not available.\n"
                 %(self._testTimeString(test), feature))
 
 
@@ -616,9 +781,12 @@
             encode = codec[0]
         else:
             encode = codec.encode
-        stream = osutils.UnicodeOrBytesToBytesWriter(encode, stream)
+        # GZ 2010-09-08: Really we don't want to be writing arbitrary bytes,
+        #                so should swap to the plain codecs.StreamWriter
+        stream = osutils.UnicodeOrBytesToBytesWriter(encode, stream,
+            "backslashreplace")
         stream.encoding = new_encoding
-        self.stream = unittest._WritelnDecorator(stream)
+        self.stream = stream
         self.descriptions = descriptions
         self.verbosity = verbosity
         self._bench_history = bench_history
@@ -772,6 +940,25 @@
         return NullProgressView()
 
 
+def isolated_doctest_setUp(test):
+    override_os_environ(test)
+
+
+def isolated_doctest_tearDown(test):
+    restore_os_environ(test)
+
+
+def IsolatedDocTestSuite(*args, **kwargs):
+    """Overrides doctest.DocTestSuite to handle isolation.
+
+    The method is really a factory and users are expected to use it as such.
+    """
+
+    kwargs['setUp'] = isolated_doctest_setUp
+    kwargs['tearDown'] = isolated_doctest_tearDown
+    return doctest.DocTestSuite(*args, **kwargs)
+
+
 class TestCase(testtools.TestCase):
     """Base class for bzr unit tests.
 
@@ -788,21 +975,17 @@
     routine, and to build and check bzr trees.
 
     In addition to the usual method of overriding tearDown(), this class also
-    allows subclasses to register functions into the _cleanups list, which is
+    allows subclasses to register cleanup functions via addCleanup, which are
     run in order as the object is torn down.  It's less likely this will be
     accidentally overlooked.
     """
 
-    _active_threads = None
-    _leaking_threads_tests = 0
-    _first_thread_leaker_id = None
-    _log_file_name = None
+    _log_file = None
     # record lsprof data when performing benchmark calls.
     _gather_lsprof_in_benchmarks = False
 
     def __init__(self, methodName='testMethod'):
         super(TestCase, self).__init__(methodName)
-        self._cleanups = []
         self._directory_isolation = True
         self.exception_handlers.insert(0,
             (UnavailableFeature, self._do_unsupported_or_skip))
@@ -813,11 +996,10 @@
         super(TestCase, self).setUp()
         for feature in getattr(self, '_test_needs_features', []):
             self.requireFeature(feature)
-        self._log_contents = None
-        self.addDetail("log", content.Content(content.ContentType("text",
-            "plain", {"charset": "utf8"}),
-            lambda:[self._get_log(keep_log_file=True)]))
         self._cleanEnvironment()
+        if bzrlib.global_state is not None:
+            self.overrideAttr(bzrlib.global_state, 'cmdline_overrides',
+                              config.CommandLineSection())
         self._silenceUI()
         self._startLogFile()
         self._benchcalls = []
@@ -826,28 +1008,90 @@
         self._track_transports()
         self._track_locks()
         self._clear_debug_flags()
-        TestCase._active_threads = threading.activeCount()
-        self.addCleanup(self._check_leaked_threads)
+        # Isolate global verbosity level, to make sure it's reproducible
+        # between tests.  We should get rid of this altogether: bug 656694. --
+        # mbp 20101008
+        self.overrideAttr(bzrlib.trace, '_verbosity_level', 0)
+        # Isolate config option expansion until its default value for bzrlib is
+        # settled on or a the FIXME associated with _get_expand_default_value
+        # is addressed -- vila 20110219
+        self.overrideAttr(config, '_expand_default_value', None)
+        self._log_files = set()
+        # Each key in the ``_counters`` dict holds a value for a different
+        # counter. When the test ends, addDetail() should be used to output the
+        # counter values. This happens in install_counter_hook().
+        self._counters = {}
+        if 'config_stats' in selftest_debug_flags:
+            self._install_config_stats_hooks()
+        # Do not use i18n for tests (unless the test reverses this)
+        i18n.disable_i18n()
 
     def debug(self):
         # debug a frame up.
         import pdb
-        pdb.Pdb().set_trace(sys._getframe().f_back)
-
-    def _check_leaked_threads(self):
-        active = threading.activeCount()
-        leaked_threads = active - TestCase._active_threads
-        TestCase._active_threads = active
-        # If some tests make the number of threads *decrease*, we'll consider
-        # that they are just observing old threads dieing, not agressively kill
-        # random threads. So we don't report these tests as leaking. The risk
-        # is that we have false positives that way (the test see 2 threads
-        # going away but leak one) but it seems less likely than the actual
-        # false positives (the test see threads going away and does not leak).
-        if leaked_threads > 0:
-            TestCase._leaking_threads_tests += 1
-            if TestCase._first_thread_leaker_id is None:
-                TestCase._first_thread_leaker_id = self.id()
+        # The sys preserved stdin/stdout should allow blackbox tests debugging
+        pdb.Pdb(stdin=sys.__stdin__, stdout=sys.__stdout__
+                ).set_trace(sys._getframe().f_back)
+
+    def discardDetail(self, name):
+        """Extend the addDetail, getDetails api so we can remove a detail.
+
+        eg. bzr always adds the 'log' detail at startup, but we don't want to
+        include it for skipped, xfail, etc tests.
+
+        It is safe to call this for a detail that doesn't exist, in case this
+        gets called multiple times.
+        """
+        # We cheat. details is stored in __details which means we shouldn't
+        # touch it. but getDetails() returns the dict directly, so we can
+        # mutate it.
+        details = self.getDetails()
+        if name in details:
+            del details[name]
+
+    def install_counter_hook(self, hooks, name, counter_name=None):
+        """Install a counting hook.
+
+        Any hook can be counted as long as it doesn't need to return a value.
+
+        :param hooks: Where the hook should be installed.
+
+        :param name: The hook name that will be counted.
+
+        :param counter_name: The counter identifier in ``_counters``, defaults
+            to ``name``.
+        """
+        _counters = self._counters # Avoid closing over self
+        if counter_name is None:
+            counter_name = name
+        if _counters.has_key(counter_name):
+            raise AssertionError('%s is already used as a counter name'
+                                  % (counter_name,))
+        _counters[counter_name] = 0
+        self.addDetail(counter_name, content.Content(content.UTF8_TEXT,
+            lambda: ['%d' % (_counters[counter_name],)]))
+        def increment_counter(*args, **kwargs):
+            _counters[counter_name] += 1
+        label = 'count %s calls' % (counter_name,)
+        hooks.install_named_hook(name, increment_counter, label)
+        self.addCleanup(hooks.uninstall_named_hook, name, label)
+
+    def _install_config_stats_hooks(self):
+        """Install config hooks to count hook calls.
+
+        """
+        for hook_name in ('get', 'set', 'remove', 'load', 'save'):
+            self.install_counter_hook(config.ConfigHooks, hook_name,
+                                       'config.%s' % (hook_name,))
+
+        # The OldConfigHooks are private and need special handling to protect
+        # against recursive tests (tests that run other tests), so we just do
+        # manually what registering them into _builtin_known_hooks will provide
+        # us.
+        self.overrideAttr(config, 'OldConfigHooks', config._OldConfigHooks())
+        for hook_name in ('get', 'set', 'remove', 'load', 'save'):
+            self.install_counter_hook(config.OldConfigHooks, hook_name,
+                                      'old_config.%s' % (hook_name,))
 
     def _clear_debug_flags(self):
         """Prevent externally set debug flags affecting tests.
@@ -864,14 +1108,16 @@
 
     def _clear_hooks(self):
         # prevent hooks affecting tests
+        known_hooks = hooks.known_hooks
         self._preserved_hooks = {}
-        for key, factory in hooks.known_hooks.items():
-            parent, name = hooks.known_hooks_key_to_parent_and_attribute(key)
-            current_hooks = hooks.known_hooks_key_to_object(key)
+        for key, (parent, name) in known_hooks.iter_parent_objects():
+            current_hooks = getattr(parent, name)
             self._preserved_hooks[parent] = (name, current_hooks)
+        self._preserved_lazy_hooks = hooks._lazy_hooks
+        hooks._lazy_hooks = {}
         self.addCleanup(self._restoreHooks)
-        for key, factory in hooks.known_hooks.items():
-            parent, name = hooks.known_hooks_key_to_parent_and_attribute(key)
+        for key, (parent, name) in known_hooks.iter_parent_objects():
+            factory = known_hooks.get(key)
             setattr(parent, name, factory())
         # this hook should always be installed
         request._install_hook()
@@ -906,9 +1152,13 @@
         # break some locks on purpose and should be taken into account by
         # considering that breaking a lock is just a dirty way of releasing it.
         if len(acquired_locks) != (len(released_locks) + len(broken_locks)):
-            message = ('Different number of acquired and '
-                       'released or broken locks. (%s, %s + %s)' %
-                       (acquired_locks, released_locks, broken_locks))
+            message = (
+                'Different number of acquired and '
+                'released or broken locks.\n'
+                'acquired=%s\n'
+                'released=%s\n'
+                'broken=%s\n' %
+                (acquired_locks, released_locks, broken_locks))
             if not self._lock_check_thorough:
                 # Rather than fail, just warn
                 print "Broken test %s: %s" % (self, message)
@@ -942,7 +1192,7 @@
 
     def permit_dir(self, name):
         """Permit a directory to be used by this test. See permit_url."""
-        name_transport = _mod_transport.get_transport(name)
+        name_transport = _mod_transport.get_transport_from_path(name)
         self.permit_url(name)
         self.permit_url(name_transport.base)
 
@@ -971,7 +1221,7 @@
             try:
                 workingtree.WorkingTree.open(path)
             except (errors.NotBranchError, errors.NoWorkingTree):
-                return
+                raise TestSkipped('Needs a working tree of bzr sources')
         finally:
             self.enable_directory_isolation()
 
@@ -1027,7 +1277,7 @@
         self.addCleanup(transport_server.stop_server)
         # Obtain a real transport because if the server supplies a password, it
         # will be hidden from the base on the client side.
-        t = _mod_transport.get_transport(transport_server.get_url())
+        t = _mod_transport.get_transport_from_url(transport_server.get_url())
         # Some transport servers effectively chroot the backing transport;
         # others like SFTPServer don't - users of the transport can walk up the
         # transport to read the entire backing transport. This wouldn't matter
@@ -1089,7 +1339,7 @@
         except UnicodeError, e:
             # If we can't compare without getting a UnicodeError, then
             # obviously they are different
-            mutter('UnicodeError: %s', e)
+            trace.mutter('UnicodeError: %s', e)
         if message:
             message += '\n'
         raise AssertionError("%snot equal:\na = %s\nb = %s\n"
@@ -1134,11 +1384,15 @@
                          'st_mtime did not match')
         self.assertEqual(expected.st_ctime, actual.st_ctime,
                          'st_ctime did not match')
-        if sys.platform != 'win32':
+        if sys.platform == 'win32':
             # On Win32 both 'dev' and 'ino' cannot be trusted. In python2.4 it
             # is 'dev' that varies, in python 2.5 (6?) it is st_ino that is
-            # odd. Regardless we shouldn't actually try to assert anything
-            # about their values
+            # odd. We just force it to always be 0 to avoid any problems.
+            self.assertEqual(0, expected.st_dev)
+            self.assertEqual(0, actual.st_dev)
+            self.assertEqual(0, expected.st_ino)
+            self.assertEqual(0, actual.st_ino)
+        else:
             self.assertEqual(expected.st_dev, actual.st_dev,
                              'st_dev did not match')
             self.assertEqual(expected.st_ino, actual.st_ino,
@@ -1153,21 +1407,20 @@
                 length, len(obj_with_len), obj_with_len))
 
     def assertLogsError(self, exception_class, func, *args, **kwargs):
-        """Assert that func(*args, **kwargs) quietly logs a specific exception.
+        """Assert that `func(*args, **kwargs)` quietly logs a specific error.
         """
-        from bzrlib import trace
         captured = []
         orig_log_exception_quietly = trace.log_exception_quietly
         try:
             def capture():
                 orig_log_exception_quietly()
-                captured.append(sys.exc_info())
+                captured.append(sys.exc_info()[1])
             trace.log_exception_quietly = capture
             func(*args, **kwargs)
         finally:
             trace.log_exception_quietly = orig_log_exception_quietly
         self.assertLength(1, captured)
-        err = captured[0][1]
+        err = captured[0]
         self.assertIsInstance(err, exception_class)
         return err
 
@@ -1210,6 +1463,10 @@
         if haystack.find(needle) == -1:
             self.fail("string %r not found in '''%s'''" % (needle, haystack))
 
+    def assertNotContainsString(self, haystack, needle):
+        if haystack.find(needle) != -1:
+            self.fail("string %r found in '''%s'''" % (needle, haystack))
+
     def assertSubset(self, sublist, superlist):
         """Assert that every entry in sublist is present in superlist."""
         missing = set(sublist) - set(superlist)
@@ -1304,7 +1561,7 @@
 
     def assertFileEqual(self, content, path):
         """Fail if path does not contain 'content'."""
-        self.failUnlessExists(path)
+        self.assertPathExists(path)
         f = file(path, 'rb')
         try:
             s = f.read()
@@ -1320,21 +1577,31 @@
         else:
             self.assertEqual(expected_docstring, obj.__doc__)
 
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4)))
     def failUnlessExists(self, path):
+        return self.assertPathExists(path)
+
+    def assertPathExists(self, path):
         """Fail unless path or paths, which may be abs or relative, exist."""
         if not isinstance(path, basestring):
             for p in path:
-                self.failUnlessExists(p)
+                self.assertPathExists(p)
         else:
-            self.failUnless(osutils.lexists(path),path+" does not exist")
+            self.assertTrue(osutils.lexists(path),
+                path + " does not exist")
 
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4)))
     def failIfExists(self, path):
+        return self.assertPathDoesNotExist(path)
+
+    def assertPathDoesNotExist(self, path):
         """Fail if path or paths, which may be abs or relative, exist."""
         if not isinstance(path, basestring):
             for p in path:
-                self.failIfExists(p)
+                self.assertPathDoesNotExist(p)
         else:
-            self.failIf(osutils.lexists(path),path+" exists")
+            self.assertFalse(osutils.lexists(path),
+                path + " exists")
 
     def _capture_deprecation_warnings(self, a_callable, *args, **kwargs):
         """A helper for callDeprecated and applyDeprecated.
@@ -1366,7 +1633,8 @@
         not other callers that go direct to the warning module.
 
         To test that a deprecated method raises an error, do something like
-        this::
+        this (remember that both assertRaises and applyDeprecated delays *args
+        and **kwargs passing)::
 
             self.assertRaises(errors.ReservedId,
                 self.applyDeprecated,
@@ -1454,23 +1722,26 @@
 
         The file is removed as the test is torn down.
         """
-        fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
-        self._log_file = os.fdopen(fileno, 'w+')
-        self._log_memento = bzrlib.trace.push_log_file(self._log_file)
-        self._log_file_name = name
+        pseudo_log_file = StringIO()
+        def _get_log_contents_for_weird_testtools_api():
+            return [pseudo_log_file.getvalue().decode(
+                "utf-8", "replace").encode("utf-8")]
+        self.addDetail("log", content.Content(content.ContentType("text",
+            "plain", {"charset": "utf8"}),
+            _get_log_contents_for_weird_testtools_api))
+        self._log_file = pseudo_log_file
+        self._log_memento = trace.push_log_file(self._log_file)
         self.addCleanup(self._finishLogFile)
 
     def _finishLogFile(self):
         """Finished with the log file.
 
-        Close the file and delete it, unless setKeepLogfile was called.
+        Close the file and delete it.
         """
-        if bzrlib.trace._trace_file:
+        if trace._trace_file:
             # flush the log file, to get all content
-            bzrlib.trace._trace_file.flush()
-        bzrlib.trace.pop_log_file(self._log_memento)
-        # Cache the log result and delete the file on disk
-        self._get_log(False)
+            trace._trace_file.flush()
+        trace.pop_log_file(self._log_memento)
 
     def thisFailsStrictLockCheck(self):
         """It is known that this test would fail with -Dstrict_locks.
@@ -1485,17 +1756,12 @@
         """
         debug.debug_flags.discard('strict_locks')
 
-    def addCleanup(self, callable, *args, **kwargs):
-        """Arrange to run a callable when this case is torn down.
-
-        Callables are run in the reverse of the order they are registered,
-        ie last-in first-out.
-        """
-        self._cleanups.append((callable, args, kwargs))
-
     def overrideAttr(self, obj, attr_name, new=_unitialized_attr):
         """Overrides an object attribute restoring it after the test.
 
+        :note: This should be used with discretion; you should think about
+        whether it's better to make the code testable without monkey-patching.
+
         :param obj: The object that will be mutated.
 
         :param attr_name: The attribute name we want to preserve/override in
@@ -1512,77 +1778,84 @@
             setattr(obj, attr_name, new)
         return value
 
+    def overrideEnv(self, name, new):
+        """Set an environment variable, and reset it after the test.
+
+        :param name: The environment variable name.
+
+        :param new: The value to set the variable to. If None, the 
+            variable is deleted from the environment.
+
+        :returns: The actual variable value.
+        """
+        value = osutils.set_or_unset_env(name, new)
+        self.addCleanup(osutils.set_or_unset_env, name, value)
+        return value
+
+    def recordCalls(self, obj, attr_name):
+        """Monkeypatch in a wrapper that will record calls.
+
+        The monkeypatch is automatically removed when the test concludes.
+
+        :param obj: The namespace holding the reference to be replaced;
+            typically a module, class, or object.
+        :param attr_name: A string for the name of the attribute to 
+            patch.
+        :returns: A list that will be extended with one item every time the
+            function is called, with a tuple of (args, kwargs).
+        """
+        calls = []
+
+        def decorator(*args, **kwargs):
+            calls.append((args, kwargs))
+            return orig(*args, **kwargs)
+        orig = self.overrideAttr(obj, attr_name, decorator)
+        return calls
+
     def _cleanEnvironment(self):
-        new_env = {
-            'BZR_HOME': None, # Don't inherit BZR_HOME to all the tests.
-            'HOME': os.getcwd(),
-            # bzr now uses the Win32 API and doesn't rely on APPDATA, but the
-            # tests do check our impls match APPDATA
-            'BZR_EDITOR': None, # test_msgeditor manipulates this variable
-            'VISUAL': None,
-            'EDITOR': None,
-            'BZR_EMAIL': None,
-            'BZREMAIL': None, # may still be present in the environment
-            'EMAIL': 'jrandom@example.com', # set EMAIL as bzr does not guess
-            'BZR_PROGRESS_BAR': None,
-            'BZR_LOG': None,
-            'BZR_PLUGIN_PATH': None,
-            'BZR_DISABLE_PLUGINS': None,
-            'BZR_PLUGINS_AT': None,
-            'BZR_CONCURRENCY': None,
-            # Make sure that any text ui tests are consistent regardless of
-            # the environment the test case is run in; you may want tests that
-            # test other combinations.  'dumb' is a reasonable guess for tests
-            # going to a pipe or a StringIO.
-            'TERM': 'dumb',
-            'LINES': '25',
-            'COLUMNS': '80',
-            'BZR_COLUMNS': '80',
-            # SSH Agent
-            'SSH_AUTH_SOCK': None,
-            # Proxies
-            'http_proxy': None,
-            'HTTP_PROXY': None,
-            'https_proxy': None,
-            'HTTPS_PROXY': None,
-            'no_proxy': None,
-            'NO_PROXY': None,
-            'all_proxy': None,
-            'ALL_PROXY': None,
-            # Nobody cares about ftp_proxy, FTP_PROXY AFAIK. So far at
-            # least. If you do (care), please update this comment
-            # -- vila 20080401
-            'ftp_proxy': None,
-            'FTP_PROXY': None,
-            'BZR_REMOTE_PATH': None,
-            # Generally speaking, we don't want apport reporting on crashes in
-            # the test envirnoment unless we're specifically testing apport,
-            # so that it doesn't leak into the real system environment.  We
-            # use an env var so it propagates to subprocesses.
-            'APPORT_DISABLE': '1',
-        }
-        self._old_env = {}
-        self.addCleanup(self._restoreEnvironment)
-        for name, value in new_env.iteritems():
-            self._captureVar(name, value)
-
-    def _captureVar(self, name, newvalue):
-        """Set an environment variable, and reset it when finished."""
-        self._old_env[name] = osutils.set_or_unset_env(name, newvalue)
-
-    def _restoreEnvironment(self):
-        for name, value in self._old_env.iteritems():
-            osutils.set_or_unset_env(name, value)
+        for name, value in isolated_environ.iteritems():
+            self.overrideEnv(name, value)
 
     def _restoreHooks(self):
         for klass, (name, hooks) in self._preserved_hooks.items():
             setattr(klass, name, hooks)
+        self._preserved_hooks.clear()
+        bzrlib.hooks._lazy_hooks = self._preserved_lazy_hooks
+        self._preserved_lazy_hooks.clear()
 
     def knownFailure(self, reason):
-        """This test has failed for some known reason."""
-        raise KnownFailure(reason)
+        """Declare that this test fails for a known reason
+
+        Tests that are known to fail should generally be using expectedFailure
+        with an appropriate reverse assertion if a change could cause the test
+        to start passing. Conversely if the test has no immediate prospect of
+        succeeding then using skip is more suitable.
+
+        When this method is called while an exception is being handled, that
+        traceback will be used, otherwise a new exception will be thrown to
+        provide one but won't be reported.
+        """
+        self._add_reason(reason)
+        try:
+            exc_info = sys.exc_info()
+            if exc_info != (None, None, None):
+                self._report_traceback(exc_info)
+            else:
+                try:
+                    raise self.failureException(reason)
+                except self.failureException:
+                    exc_info = sys.exc_info()
+            # GZ 02-08-2011: Maybe cleanup this err.exc_info attribute too?
+            raise testtools.testcase._ExpectedFailure(exc_info)
+        finally:
+            del exc_info
+
+    def _suppress_log(self):
+        """Remove the log info from details."""
+        self.discardDetail('log')
 
     def _do_skip(self, result, reason):
+        self._suppress_log()
         addSkip = getattr(result, 'addSkip', None)
         if not callable(addSkip):
             result.addSuccess(result)
@@ -1591,6 +1864,7 @@
 
     @staticmethod
     def _do_known_failure(self, result, e):
+        self._suppress_log()
         err = sys.exc_info()
         addExpectedFailure = getattr(result, 'addExpectedFailure', None)
         if addExpectedFailure is not None:
@@ -1604,6 +1878,7 @@
             reason = 'No reason given'
         else:
             reason = e.args[0]
+        self._suppress_log ()
         addNotApplicable = getattr(result, 'addNotApplicable', None)
         if addNotApplicable is not None:
             result.addNotApplicable(self, reason)
@@ -1611,8 +1886,29 @@
             self._do_skip(result, reason)
 
     @staticmethod
+    def _report_skip(self, result, err):
+        """Override the default _report_skip.
+
+        We want to strip the 'log' detail. If we waint until _do_skip, it has
+        already been formatted into the 'reason' string, and we can't pull it
+        out again.
+        """
+        self._suppress_log()
+        super(TestCase, self)._report_skip(self, result, err)
+
+    @staticmethod
+    def _report_expected_failure(self, result, err):
+        """Strip the log.
+
+        See _report_skip for motivation.
+        """
+        self._suppress_log()
+        super(TestCase, self)._report_expected_failure(self, result, err)
+
+    @staticmethod
     def _do_unsupported_or_skip(self, result, e):
         reason = e.args[0]
+        self._suppress_log()
         addNotSupported = getattr(result, 'addNotSupported', None)
         if addNotSupported is not None:
             result.addNotSupported(self, reason)
@@ -1644,86 +1940,7 @@
             self._benchtime += time.time() - start
 
     def log(self, *args):
-        mutter(*args)
-
-    def _get_log(self, keep_log_file=False):
-        """Internal helper to get the log from bzrlib.trace for this test.
-
-        Please use self.getDetails, or self.get_log to access this in test case
-        code.
-
-        :param keep_log_file: When True, if the log is still a file on disk
-            leave it as a file on disk. When False, if the log is still a file
-            on disk, the log file is deleted and the log preserved as
-            self._log_contents.
-        :return: A string containing the log.
-        """
-        if self._log_contents is not None:
-            try:
-                self._log_contents.decode('utf8')
-            except UnicodeDecodeError:
-                unicodestr = self._log_contents.decode('utf8', 'replace')
-                self._log_contents = unicodestr.encode('utf8')
-            return self._log_contents
-        import bzrlib.trace
-        if bzrlib.trace._trace_file:
-            # flush the log file, to get all content
-            bzrlib.trace._trace_file.flush()
-        if self._log_file_name is not None:
-            logfile = open(self._log_file_name)
-            try:
-                log_contents = logfile.read()
-            finally:
-                logfile.close()
-            try:
-                log_contents.decode('utf8')
-            except UnicodeDecodeError:
-                unicodestr = log_contents.decode('utf8', 'replace')
-                log_contents = unicodestr.encode('utf8')
-            if not keep_log_file:
-                close_attempts = 0
-                max_close_attempts = 100
-                first_close_error = None
-                while close_attempts < max_close_attempts:
-                    close_attempts += 1
-                    try:
-                        self._log_file.close()
-                    except IOError, ioe:
-                        if ioe.errno is None:
-                            # No errno implies 'close() called during
-                            # concurrent operation on the same file object', so
-                            # retry.  Probably a thread is trying to write to
-                            # the log file.
-                            if first_close_error is None:
-                                first_close_error = ioe
-                            continue
-                        raise
-                    else:
-                        break
-                if close_attempts > 1:
-                    sys.stderr.write(
-                        'Unable to close log file on first attempt, '
-                        'will retry: %s\n' % (first_close_error,))
-                    if close_attempts == max_close_attempts:
-                        sys.stderr.write(
-                            'Unable to close log file after %d attempts.\n'
-                            % (max_close_attempts,))
-                self._log_file = None
-                # Permit multiple calls to get_log until we clean it up in
-                # finishLogFile
-                self._log_contents = log_contents
-                try:
-                    os.remove(self._log_file_name)
-                except OSError, e:
-                    if sys.platform == 'win32' and e.errno == errno.EACCES:
-                        sys.stderr.write(('Unable to delete log file '
-                                             ' %r\n' % self._log_file_name))
-                    else:
-                        raise
-                self._log_file_name = None
-            return log_contents
-        else:
-            return "No log file content and no log file name."
+        trace.mutter(*args)
 
     def get_log(self):
         """Get a unicode string containing the log from bzrlib.trace.
@@ -1779,9 +1996,10 @@
 
         try:
             try:
-                result = self.apply_redirected(ui.ui_factory.stdin,
+                result = self.apply_redirected(
+                    ui.ui_factory.stdin,
                     stdout, stderr,
-                    bzrlib.commands.run_bzr_catch_user_errors,
+                    _mod_commands.run_bzr_catch_user_errors,
                     args)
             except KeyboardInterrupt:
                 # Reraise KeyboardInterrupt with contents of redirected stdout
@@ -1929,7 +2147,7 @@
     def start_bzr_subprocess(self, process_args, env_changes=None,
                              skip_if_plan_to_signal=False,
                              working_dir=None,
-                             allow_plugins=False):
+                             allow_plugins=False, stderr=subprocess.PIPE):
         """Start bzr in a subprocess for testing.
 
         This starts a new Python interpreter and runs bzr in there.
@@ -1944,15 +2162,18 @@
             variables. A value of None will unset the env variable.
             The values must be strings. The change will only occur in the
             child, so you don't need to fix the environment after running.
-        :param skip_if_plan_to_signal: raise TestSkipped when true and os.kill
-            is not available.
+        :param skip_if_plan_to_signal: raise TestSkipped when true and system
+            doesn't support signalling subprocesses.
         :param allow_plugins: If False (default) pass --no-plugins to bzr.
+        :param stderr: file to use for the subprocess's stderr.  Valid values
+            are those valid for the stderr argument of `subprocess.Popen`.
+            Default value is ``subprocess.PIPE``.
 
         :returns: Popen object for the started process.
         """
         if skip_if_plan_to_signal:
-            if not getattr(os, 'kill', None):
-                raise TestSkipped("os.kill not available.")
+            if os.name != "posix":
+                raise TestSkipped("Sending signals not supported")
 
         if env_changes is None:
             env_changes = {}
@@ -1978,6 +2199,9 @@
             # so we will avoid using it on all platforms, just to
             # make sure the code path is used, and we don't break on win32
             cleanup_environment()
+            # Include the subprocess's log file in the test details, in case
+            # the test fails due to an error in the subprocess.
+            self._add_subprocess_log(trace._get_bzr_log_filename())
             command = [sys.executable]
             # frozen executables don't need the path to bzr
             if getattr(sys, "frozen", None) is None:
@@ -1987,7 +2211,7 @@
             command.extend(process_args)
             process = self._popen(command, stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
-                                  stderr=subprocess.PIPE)
+                                  stderr=stderr)
         finally:
             restore_environment()
             if cwd is not None:
@@ -1995,6 +2219,33 @@
 
         return process
 
+    def _add_subprocess_log(self, log_file_path):
+        if len(self._log_files) == 0:
+            # Register an addCleanup func.  We do this on the first call to
+            # _add_subprocess_log rather than in TestCase.setUp so that this
+            # addCleanup is registered after any cleanups for tempdirs that
+            # subclasses might create, which will probably remove the log file
+            # we want to read.
+            self.addCleanup(self._subprocess_log_cleanup)
+        # self._log_files is a set, so if a log file is reused we won't grab it
+        # twice.
+        self._log_files.add(log_file_path)
+
+    def _subprocess_log_cleanup(self):
+        for count, log_file_path in enumerate(self._log_files):
+            # We use buffer_now=True to avoid holding the file open beyond
+            # the life of this function, which might interfere with e.g.
+            # cleaning tempdirs on Windows.
+            # XXX: Testtools 0.9.5 doesn't have the content_from_file helper
+            #detail_content = content.content_from_file(
+            #    log_file_path, buffer_now=True)
+            with open(log_file_path, 'rb') as log_file:
+                log_file_bytes = log_file.read()
+            detail_content = content.Content(content.ContentType("text",
+                "plain", {"charset": "utf8"}), lambda: [log_file_bytes])
+            self.addDetail("start_bzr_subprocess-log-%d" % (count,),
+                detail_content)
+
     def _popen(self, *args, **kwargs):
         """Place a call to Popen.
 
@@ -2037,24 +2288,26 @@
         if retcode is not None and retcode != process.returncode:
             if process_args is None:
                 process_args = "(unknown args)"
-            mutter('Output of bzr %s:\n%s', process_args, out)
-            mutter('Error for bzr %s:\n%s', process_args, err)
+            trace.mutter('Output of bzr %s:\n%s', process_args, out)
+            trace.mutter('Error for bzr %s:\n%s', process_args, err)
             self.fail('Command bzr %s failed with retcode %s != %s'
                       % (process_args, retcode, process.returncode))
         return [out, err]
 
-    def check_inventory_shape(self, inv, shape):
-        """Compare an inventory to a list of expected names.
+    def check_tree_shape(self, tree, shape):
+        """Compare a tree to a list of expected names.
 
         Fail if they are not precisely equal.
         """
         extras = []
         shape = list(shape)             # copy
-        for path, ie in inv.entries():
+        for path, ie in tree.iter_entries_by_dir():
             name = path.replace('\\', '/')
             if ie.kind == 'directory':
                 name = name + '/'
-            if name in shape:
+            if name == "/":
+                pass # ignore root entry
+            elif name in shape:
                 shape.remove(name)
             else:
                 extras.append(name)
@@ -2101,7 +2354,7 @@
 
         Tests that expect to provoke LockContention errors should call this.
         """
-        self.overrideAttr(bzrlib.lockdir, '_DEFAULT_TIMEOUT_SECONDS', 0)
+        self.overrideAttr(lockdir, '_DEFAULT_TIMEOUT_SECONDS', 0)
 
     def make_utf8_encoded_stringio(self, encoding_type=None):
         """Return a StringIOWrapper instance, that will encode Unicode
@@ -2150,20 +2403,21 @@
 class TestCaseWithMemoryTransport(TestCase):
     """Common test class for tests that do not need disk resources.
 
-    Tests that need disk resources should derive from TestCaseWithTransport.
+    Tests that need disk resources should derive from TestCaseInTempDir
+    orTestCaseWithTransport.
 
     TestCaseWithMemoryTransport sets the TEST_ROOT variable for all bzr tests.
 
-    For TestCaseWithMemoryTransport the test_home_dir is set to the name of
+    For TestCaseWithMemoryTransport the ``test_home_dir`` is set to the name of
     a directory which does not exist. This serves to help ensure test isolation
-    is preserved. test_dir is set to the TEST_ROOT, as is cwd, because they
-    must exist. However, TestCaseWithMemoryTransport does not offer local
-    file defaults for the transport in tests, nor does it obey the command line
+    is preserved. ``test_dir`` is set to the TEST_ROOT, as is cwd, because they
+    must exist. However, TestCaseWithMemoryTransport does not offer local file
+    defaults for the transport in tests, nor does it obey the command line
     override, so tests that accidentally write to the common directory should
     be rare.
 
-    :cvar TEST_ROOT: Directory containing all temporary directories, plus
-    a .bzr directory that stops us ascending higher into the filesystem.
+    :cvar TEST_ROOT: Directory containing all temporary directories, plus a
+        ``.bzr`` directory that stops us ascending higher into the filesystem.
     """
 
     TEST_ROOT = None
@@ -2187,7 +2441,7 @@
 
         :param relpath: a path relative to the base url.
         """
-        t = _mod_transport.get_transport(self.get_url(relpath))
+        t = _mod_transport.get_transport_from_url(self.get_url(relpath))
         self.assertFalse(t.is_readonly())
         return t
 
@@ -2199,7 +2453,8 @@
 
         :param relpath: a path relative to the base url.
         """
-        t = _mod_transport.get_transport(self.get_readonly_url(relpath))
+        t = _mod_transport.get_transport_from_url(
+            self.get_readonly_url(relpath))
         self.assertTrue(t.is_readonly())
         return t
 
@@ -2326,7 +2581,17 @@
         real branch.
         """
         root = TestCaseWithMemoryTransport.TEST_ROOT
-        bzrdir.BzrDir.create_standalone_workingtree(root)
+        # Make sure we get a readable and accessible home for .bzr.log
+        # and/or config files, and not fallback to weird defaults (see
+        # http://pad.lv/825027).
+        self.assertIs(None, os.environ.get('BZR_HOME', None))
+        os.environ['BZR_HOME'] = root
+        wt = bzrdir.BzrDir.create_standalone_workingtree(root)
+        del os.environ['BZR_HOME']
+        # Hack for speed: remember the raw bytes of the dirstate file so that
+        # we don't need to re-open the wt to check it hasn't changed.
+        TestCaseWithMemoryTransport._SAFETY_NET_PRISTINE_DIRSTATE = (
+            wt.control_transport.get_bytes('dirstate'))
 
     def _check_safety_net(self):
         """Check that the safety .bzr directory have not been touched.
@@ -2335,10 +2600,10 @@
         propagating. This method ensures than a test did not leaked.
         """
         root = TestCaseWithMemoryTransport.TEST_ROOT
-        self.permit_url(_mod_transport.get_transport(root).base)
-        wt = workingtree.WorkingTree.open(root)
-        last_rev = wt.last_revision()
-        if last_rev != 'null:':
+        t = _mod_transport.get_transport_from_path(root)
+        self.permit_url(t.base)
+        if (t.get_bytes('.bzr/checkout/dirstate') != 
+                TestCaseWithMemoryTransport._SAFETY_NET_PRISTINE_DIRSTATE):
             # The current test have modified the /bzr directory, we need to
             # recreate a new one or all the followng tests will fail.
             # If you need to inspect its content uncomment the following line
@@ -2379,7 +2644,26 @@
     def make_branch(self, relpath, format=None):
         """Create a branch on the transport at relpath."""
         repo = self.make_repository(relpath, format=format)
-        return repo.bzrdir.create_branch()
+        return repo.bzrdir.create_branch(append_revisions_only=False)
+
+    def get_default_format(self):
+        return 'default'
+
+    def resolve_format(self, format):
+        """Resolve an object to a ControlDir format object.
+
+        The initial format object can either already be
+        a ControlDirFormat, None (for the default format),
+        or a string with the name of the control dir format.
+
+        :param format: Object to resolve
+        :return A ControlDirFormat instance
+        """
+        if format is None:
+            format = self.get_default_format()
+        if isinstance(format, basestring):
+            format = bzrdir.format_registry.make_bzrdir(format)
+        return format
 
     def make_bzrdir(self, relpath, format=None):
         try:
@@ -2389,15 +2673,12 @@
             t = _mod_transport.get_transport(maybe_a_url)
             if len(segments) > 1 and segments[-1] not in ('', '.'):
                 t.ensure_base()
-            if format is None:
-                format = 'default'
-            if isinstance(format, basestring):
-                format = bzrdir.format_registry.make_bzrdir(format)
+            format = self.resolve_format(format)
             return format.initialize_on_transport(t)
         except errors.UninitializableFormat:
             raise TestSkipped("Format %s is not initializable." % format)
 
-    def make_repository(self, relpath, shared=False, format=None):
+    def make_repository(self, relpath, shared=None, format=None):
         """Create a repository on our default transport at relpath.
 
         Note that relpath must be a relative path, not a full url.
@@ -2414,7 +2695,7 @@
             backing_server = self.get_server()
         smart_server = test_server.SmartTCPServer_for_testing()
         self.start_server(smart_server, backing_server)
-        remote_transport = _mod_transport.get_transport(smart_server.get_url()
+        remote_transport = _mod_transport.get_transport_from_url(smart_server.get_url()
                                                    ).clone(path)
         return remote_transport
 
@@ -2431,11 +2712,21 @@
         test_home_dir = self.test_home_dir
         if isinstance(test_home_dir, unicode):
             test_home_dir = test_home_dir.encode(sys.getfilesystemencoding())
-        os.environ['HOME'] = test_home_dir
-        os.environ['BZR_HOME'] = test_home_dir
+        self.overrideEnv('HOME', test_home_dir)
+        self.overrideEnv('BZR_HOME', test_home_dir)
 
     def setUp(self):
         super(TestCaseWithMemoryTransport, self).setUp()
+        # Ensure that ConnectedTransport doesn't leak sockets
+        def get_transport_from_url_with_cleanup(*args, **kwargs):
+            t = orig_get_transport_from_url(*args, **kwargs)
+            if isinstance(t, _mod_transport.ConnectedTransport):
+                self.addCleanup(t.disconnect)
+            return t
+
+        orig_get_transport_from_url = self.overrideAttr(
+            _mod_transport, 'get_transport_from_url',
+            get_transport_from_url_with_cleanup)
         self._make_test_root()
         self.addCleanup(os.chdir, os.getcwdu())
         self.makeAndChdirToTestDir()
@@ -2484,6 +2775,12 @@
 
     OVERRIDE_PYTHON = 'python'
 
+    def setUp(self):
+        super(TestCaseInTempDir, self).setUp()
+        # Remove the protection set in isolated_environ, we have a proper
+        # access to disk resources now.
+        self.overrideEnv('BZR_LOG', None)
+
     def check_file_contents(self, filename, expect):
         self.log("check contents of file %s" % filename)
         f = file(filename)
@@ -2570,7 +2867,7 @@
                 "a list or a tuple. Got %r instead" % (shape,))
         # It's OK to just create them using forward slashes on windows.
         if transport is None or transport.is_readonly():
-            transport = _mod_transport.get_transport(".")
+            transport = _mod_transport.get_transport_from_path(".")
         for name in shape:
             self.assertIsInstance(name, basestring)
             if name[-1] == '/':
@@ -2661,6 +2958,10 @@
         # this obviously requires a format that supports branch references
         # so check for that by checking bzrdir.BzrDirFormat.get_default_format()
         # RBC 20060208
+        format = self.resolve_format(format=format)
+        if not format.supports_workingtrees:
+            b = self.make_branch(relpath+'.branch', format=format)
+            return b.create_checkout(relpath, lightweight=True)
         b = self.make_branch(relpath, format=format)
         try:
             return b.bzrdir.create_workingtree()
@@ -2733,9 +3034,10 @@
     """
 
     def setUp(self):
+        from bzrlib.tests import http_server
         super(ChrootedTestCase, self).setUp()
         if not self.vfs_transport_factory == memory.MemoryServer:
-            self.transport_readonly_server = HttpServer
+            self.transport_readonly_server = http_server.HttpServer
 
 
 def condition_id_re(pattern):
@@ -2964,6 +3266,9 @@
                             result_decorators=result_decorators,
                             )
     runner.stop_on_failure=stop_on_failure
+    if isinstance(suite, unittest.TestSuite):
+        # Empty out _tests list of passed suite and populate new TestSuite
+        suite._tests[:], suite = [], TestSuite(suite)
     # built in decorator factories:
     decorators = [
         random_order(random_seed, runner),
@@ -3002,6 +3307,9 @@
 
 
 def fork_decorator(suite):
+    if getattr(os, "fork", None) is None:
+        raise errors.BzrCommandError("platform does not support fork,"
+            " try --parallel=subprocess instead.")
     concurrency = osutils.local_concurrency()
     if concurrency == 1:
         return suite
@@ -3062,36 +3370,19 @@
     return suite
 
 
-class TestDecorator(TestSuite):
+class TestDecorator(TestUtil.TestSuite):
     """A decorator for TestCase/TestSuite objects.
-    
-    Usually, subclasses should override __iter__(used when flattening test
-    suites), which we do to filter, reorder, parallelise and so on, run() and
-    debug().
+
+    Contains rather than flattening suite passed on construction
     """
 
-    def __init__(self, suite):
-        TestSuite.__init__(self)
-        self.addTest(suite)
-
-    def countTestCases(self):
-        cases = 0
-        for test in self:
-            cases += test.countTestCases()
-        return cases
-
-    def debug(self):
-        for test in self:
-            test.debug()
-
-    def run(self, result):
-        # Use iteration on self, not self._tests, to allow subclasses to hook
-        # into __iter__.
-        for test in self:
-            if result.shouldStop:
-                break
-            test.run(result)
-        return result
+    def __init__(self, suite=None):
+        super(TestDecorator, self).__init__()
+        if suite is not None:
+            self.addTest(suite)
+
+    # Don't need subclass run method with suite emptying
+    run = unittest.TestSuite.run
 
 
 class CountingDecorator(TestDecorator):
@@ -3108,103 +3399,64 @@
     """A decorator which excludes test matching an exclude pattern."""
 
     def __init__(self, suite, exclude_pattern):
-        TestDecorator.__init__(self, suite)
-        self.exclude_pattern = exclude_pattern
-        self.excluded = False
-
-    def __iter__(self):
-        if self.excluded:
-            return iter(self._tests)
-        self.excluded = True
-        suite = exclude_tests_by_re(self, self.exclude_pattern)
-        del self._tests[:]
-        self.addTests(suite)
-        return iter(self._tests)
+        super(ExcludeDecorator, self).__init__(
+            exclude_tests_by_re(suite, exclude_pattern))
 
 
 class FilterTestsDecorator(TestDecorator):
     """A decorator which filters tests to those matching a pattern."""
 
     def __init__(self, suite, pattern):
-        TestDecorator.__init__(self, suite)
-        self.pattern = pattern
-        self.filtered = False
-
-    def __iter__(self):
-        if self.filtered:
-            return iter(self._tests)
-        self.filtered = True
-        suite = filter_suite_by_re(self, self.pattern)
-        del self._tests[:]
-        self.addTests(suite)
-        return iter(self._tests)
+        super(FilterTestsDecorator, self).__init__(
+            filter_suite_by_re(suite, pattern))
 
 
 class RandomDecorator(TestDecorator):
     """A decorator which randomises the order of its tests."""
 
     def __init__(self, suite, random_seed, stream):
-        TestDecorator.__init__(self, suite)
-        self.random_seed = random_seed
-        self.randomised = False
-        self.stream = stream
-
-    def __iter__(self):
-        if self.randomised:
-            return iter(self._tests)
-        self.randomised = True
-        self.stream.write("Randomizing test order using seed %s\n\n" %
-            (self.actual_seed()))
+        random_seed = self.actual_seed(random_seed)
+        stream.write("Randomizing test order using seed %s\n\n" %
+            (random_seed,))
         # Initialise the random number generator.
-        random.seed(self.actual_seed())
-        suite = randomize_suite(self)
-        del self._tests[:]
-        self.addTests(suite)
-        return iter(self._tests)
+        random.seed(random_seed)
+        super(RandomDecorator, self).__init__(randomize_suite(suite))
 
-    def actual_seed(self):
-        if self.random_seed == "now":
+    @staticmethod
+    def actual_seed(seed):
+        if seed == "now":
             # We convert the seed to a long to make it reuseable across
             # invocations (because the user can reenter it).
-            self.random_seed = long(time.time())
+            return long(time.time())
         else:
             # Convert the seed to a long if we can
             try:
-                self.random_seed = long(self.random_seed)
-            except:
+                return long(seed)
+            except (TypeError, ValueError):
                 pass
-        return self.random_seed
+        return seed
 
 
 class TestFirstDecorator(TestDecorator):
     """A decorator which moves named tests to the front."""
 
     def __init__(self, suite, pattern):
-        TestDecorator.__init__(self, suite)
-        self.pattern = pattern
-        self.filtered = False
-
-    def __iter__(self):
-        if self.filtered:
-            return iter(self._tests)
-        self.filtered = True
-        suites = split_suite_by_re(self, self.pattern)
-        del self._tests[:]
-        self.addTests(suites)
-        return iter(self._tests)
+        super(TestFirstDecorator, self).__init__()
+        self.addTests(split_suite_by_re(suite, pattern))
 
 
 def partition_tests(suite, count):
     """Partition suite into count lists of tests."""
-    result = []
-    tests = list(iter_suite_tests(suite))
-    tests_per_process = int(math.ceil(float(len(tests)) / count))
-    for block in range(count):
-        low_test = block * tests_per_process
-        high_test = low_test + tests_per_process
-        process_tests = tests[low_test:high_test]
-        result.append(process_tests)
-    return result
+    # This just assigns tests in a round-robin fashion.  On one hand this
+    # splits up blocks of related tests that might run faster if they shared
+    # resources, but on the other it avoids assigning blocks of slow tests to
+    # just one partition.  So the slowest partition shouldn't be much slower
+    # than the fastest.
+    partitions = [list() for i in range(count)]
+    tests = iter_suite_tests(suite)
+    for partition, test in itertools.izip(itertools.cycle(partitions), tests):
+        partition.append(test)
+    return partitions
 
 
 def workaround_zealous_crypto_random():
@@ -3228,7 +3480,7 @@
     """
     concurrency = osutils.local_concurrency()
     result = []
-    from subunit import TestProtocolClient, ProtocolTestCase
+    from subunit import ProtocolTestCase
     from subunit.test_results import AutoTimingTestResultDecorator
     class TestInOtherProcess(ProtocolTestCase):
         # Should be in subunit, I think. RBC.
@@ -3243,9 +3495,12 @@
                 os.waitpid(self.pid, 0)
 
     test_blocks = partition_tests(suite, concurrency)
+    # Clear the tests from the original suite so it doesn't keep them alive
+    suite._tests[:] = []
     for process_tests in test_blocks:
-        process_suite = TestSuite()
-        process_suite.addTests(process_tests)
+        process_suite = TestUtil.TestSuite(process_tests)
+        # Also clear each split list so new suite has only reference
+        process_tests[:] = []
         c2pread, c2pwrite = os.pipe()
         pid = os.fork()
         if pid == 0:
@@ -3257,12 +3512,15 @@
                 # read from stdin (otherwise its a roulette to see what
                 # child actually gets keystrokes for pdb etc).
                 sys.stdin.close()
-                sys.stdin = None
+                # GZ 2011-06-16: Why set stdin to None? Breaks multi fork.
+                #sys.stdin = None
                 stream = os.fdopen(c2pwrite, 'wb', 1)
                 subunit_result = AutoTimingTestResultDecorator(
-                    TestProtocolClient(stream))
+                    SubUnitBzrProtocolClient(stream))
                 process_suite.run(subunit_result)
             finally:
+                # GZ 2011-06-16: Is always exiting with silent success
+                #                really the right thing? Hurts debugging.
                 os._exit(0)
         else:
             os.close(c2pwrite)
@@ -3330,39 +3588,7 @@
     return result
 
 
-class ForwardingResult(unittest.TestResult):
-
-    def __init__(self, target):
-        unittest.TestResult.__init__(self)
-        self.result = target
-
-    def startTest(self, test):
-        self.result.startTest(test)
-
-    def stopTest(self, test):
-        self.result.stopTest(test)
-
-    def startTestRun(self):
-        self.result.startTestRun()
-
-    def stopTestRun(self):
-        self.result.stopTestRun()
-
-    def addSkip(self, test, reason):
-        self.result.addSkip(test, reason)
-
-    def addSuccess(self, test):
-        self.result.addSuccess(test)
-
-    def addError(self, test, err):
-        self.result.addError(test, err)
-
-    def addFailure(self, test, err):
-        self.result.addFailure(test, err)
-ForwardingResult = testtools.ExtendedToOriginalDecorator
-
-
-class ProfileResult(ForwardingResult):
+class ProfileResult(testtools.ExtendedToOriginalDecorator):
     """Generate profiling data for all activity between start and success.
     
     The profile data is appended to the test's _benchcalls attribute and can
@@ -3380,7 +3606,7 @@
         # unavoidably fail.
         bzrlib.lsprof.BzrProfiler.profiler_block = 0
         self.profiler.start()
-        ForwardingResult.startTest(self, test)
+        testtools.ExtendedToOriginalDecorator.startTest(self, test)
 
     def addSuccess(self, test):
         stats = self.profiler.stop()
@@ -3390,10 +3616,10 @@
             test._benchcalls = []
             calls = test._benchcalls
         calls.append(((test.id(), "", ""), stats))
-        ForwardingResult.addSuccess(self, test)
+        testtools.ExtendedToOriginalDecorator.addSuccess(self, test)
 
     def stopTest(self, test):
-        ForwardingResult.stopTest(self, test)
+        testtools.ExtendedToOriginalDecorator.stopTest(self, test)
         self.profiler = None
 
 
@@ -3405,6 +3631,11 @@
 #                           rather than failing tests. And no longer raise
 #                           LockContention when fctnl locks are not being used
 #                           with proper exclusion rules.
+#   -Ethreads               Will display thread ident at creation/join time to
+#                           help track thread leaks
+#   -Euncollected_cases     Display the identity of any test cases that weren't
+#                           deallocated after being completed.
+#   -Econfig_stats          Will collect statistics using addDetail
 selftest_debug_flags = set()
 
 
@@ -3604,8 +3835,9 @@
                 key, obj, help=help, info=info, override_existing=False)
         except KeyError:
             actual = self.get(key)
-            note('Test prefix alias %s is already used for %s, ignoring %s'
-                 % (key, actual, obj))
+            trace.note(
+                'Test prefix alias %s is already used for %s, ignoring %s'
+                % (key, actual, obj))
 
     def resolve_alias(self, id_start):
         """Replace the alias by the prefix in the given string.
@@ -3646,7 +3878,8 @@
         'bzrlib.tests.doc_generate',
         'bzrlib.tests.per_branch',
         'bzrlib.tests.per_bzrdir',
-        'bzrlib.tests.per_bzrdir_colo',
+        'bzrlib.tests.per_controldir',
+        'bzrlib.tests.per_controldir_colo',
         'bzrlib.tests.per_foreign_vcs',
         'bzrlib.tests.per_interrepository',
         'bzrlib.tests.per_intertree',
@@ -3660,11 +3893,13 @@
         'bzrlib.tests.per_repository',
         'bzrlib.tests.per_repository_chk',
         'bzrlib.tests.per_repository_reference',
+        'bzrlib.tests.per_repository_vf',
         'bzrlib.tests.per_uifactory',
         'bzrlib.tests.per_versionedfile',
         'bzrlib.tests.per_workingtree',
         'bzrlib.tests.test__annotator',
         'bzrlib.tests.test__bencode',
+        'bzrlib.tests.test__btree_serializer',
         'bzrlib.tests.test__chk_map',
         'bzrlib.tests.test__dirstate_helpers',
         'bzrlib.tests.test__groupcompress',
@@ -3698,24 +3933,28 @@
         'bzrlib.tests.test_commit_merge',
         'bzrlib.tests.test_config',
         'bzrlib.tests.test_conflicts',
+        'bzrlib.tests.test_controldir',
         'bzrlib.tests.test_counted_lock',
         'bzrlib.tests.test_crash',
         'bzrlib.tests.test_decorators',
         'bzrlib.tests.test_delta',
         'bzrlib.tests.test_debug',
-        'bzrlib.tests.test_deprecated_graph',
         'bzrlib.tests.test_diff',
         'bzrlib.tests.test_directory_service',
         'bzrlib.tests.test_dirstate',
         'bzrlib.tests.test_email_message',
         'bzrlib.tests.test_eol_filters',
         'bzrlib.tests.test_errors',
+        'bzrlib.tests.test_estimate_compressed_size',
         'bzrlib.tests.test_export',
+        'bzrlib.tests.test_export_pot',
         'bzrlib.tests.test_extract',
+        'bzrlib.tests.test_features',
         'bzrlib.tests.test_fetch',
         'bzrlib.tests.test_fixtures',
         'bzrlib.tests.test_fifo_cache',
         'bzrlib.tests.test_filters',
+        'bzrlib.tests.test_filter_tree',
         'bzrlib.tests.test_ftp_transport',
         'bzrlib.tests.test_foreign',
         'bzrlib.tests.test_generate_docs',
@@ -3730,6 +3969,7 @@
         'bzrlib.tests.test_http',
         'bzrlib.tests.test_http_response',
         'bzrlib.tests.test_https_ca_bundle',
+        'bzrlib.tests.test_i18n',
         'bzrlib.tests.test_identitymap',
         'bzrlib.tests.test_ignores',
         'bzrlib.tests.test_index',
@@ -3754,6 +3994,7 @@
         'bzrlib.tests.test_merge3',
         'bzrlib.tests.test_merge_core',
         'bzrlib.tests.test_merge_directive',
+        'bzrlib.tests.test_mergetools',
         'bzrlib.tests.test_missing',
         'bzrlib.tests.test_msgeditor',
         'bzrlib.tests.test_multiparent',
@@ -3768,6 +4009,7 @@
         'bzrlib.tests.test_permissions',
         'bzrlib.tests.test_plugins',
         'bzrlib.tests.test_progress',
+        'bzrlib.tests.test_pyutils',
         'bzrlib.tests.test_read_bundle',
         'bzrlib.tests.test_reconcile',
         'bzrlib.tests.test_reconfigure',
@@ -3782,6 +4024,7 @@
         'bzrlib.tests.test_rio',
         'bzrlib.tests.test_rules',
         'bzrlib.tests.test_sampler',
+        'bzrlib.tests.test_scenarios',
         'bzrlib.tests.test_script',
         'bzrlib.tests.test_selftest',
         'bzrlib.tests.test_serializer',
@@ -3792,6 +4035,7 @@
         'bzrlib.tests.test_smart',
         'bzrlib.tests.test_smart_add',
         'bzrlib.tests.test_smart_request',
+        'bzrlib.tests.test_smart_signals',
         'bzrlib.tests.test_smart_transport',
         'bzrlib.tests.test_smtp_connection',
         'bzrlib.tests.test_source',
@@ -3803,9 +4047,11 @@
         'bzrlib.tests.test_switch',
         'bzrlib.tests.test_symbol_versioning',
         'bzrlib.tests.test_tag',
+        'bzrlib.tests.test_test_server',
         'bzrlib.tests.test_testament',
         'bzrlib.tests.test_textfile',
         'bzrlib.tests.test_textmerge',
+        'bzrlib.tests.test_cethread',
         'bzrlib.tests.test_timestamp',
         'bzrlib.tests.test_trace',
         'bzrlib.tests.test_transactions',
@@ -3822,8 +4068,10 @@
         'bzrlib.tests.test_upgrade',
         'bzrlib.tests.test_upgrade_stacked',
         'bzrlib.tests.test_urlutils',
+        'bzrlib.tests.test_utextwrap',
         'bzrlib.tests.test_version',
         'bzrlib.tests.test_version_info',
+        'bzrlib.tests.test_versionedfile',
         'bzrlib.tests.test_weave',
         'bzrlib.tests.test_whitebox',
         'bzrlib.tests.test_win32utils',
@@ -3843,16 +4091,17 @@
         'bzrlib',
         'bzrlib.branchbuilder',
         'bzrlib.decorators',
-        'bzrlib.export',
         'bzrlib.inventory',
         'bzrlib.iterablefile',
         'bzrlib.lockdir',
         'bzrlib.merge3',
         'bzrlib.option',
+        'bzrlib.pyutils',
         'bzrlib.symbol_versioning',
         'bzrlib.tests',
         'bzrlib.tests.fixtures',
         'bzrlib.timestamp',
+        'bzrlib.transport.http',
         'bzrlib.version_info_formats.format_custom',
         ]
 
@@ -3911,8 +4160,8 @@
         try:
             # note that this really does mean "report only" -- doctest
             # still runs the rest of the examples
-            doc_suite = doctest.DocTestSuite(mod,
-                optionflags=doctest.REPORT_ONLY_FIRST_FAILURE)
+            doc_suite = IsolatedDocTestSuite(
+                mod, optionflags=doctest.REPORT_ONLY_FIRST_FAILURE)
         except ValueError, e:
             print '**failed to get doctest for: %s\n%s' % (mod, e)
             raise
@@ -3921,7 +4170,7 @@
         suite.addTest(doc_suite)
 
     default_encoding = sys.getdefaultencoding()
-    for name, plugin in bzrlib.plugin.plugins().items():
+    for name, plugin in _mod_plugin.plugins().items():
         if not interesting_module(plugin.module.__name__):
             continue
         plugin_suite = plugin.test_suite()
@@ -3933,7 +4182,7 @@
         if plugin_suite is not None:
             suite.addTest(plugin_suite)
         if default_encoding != sys.getdefaultencoding():
-            bzrlib.trace.warning(
+            trace.warning(
                 'Plugin "%s" tried to reset default encoding to: %s', name,
                 sys.getdefaultencoding())
             reload(sys)
@@ -3954,14 +4203,26 @@
             # Some tests mentioned in the list are not in the test suite. The
             # list may be out of date, report to the tester.
             for id in not_found:
-                bzrlib.trace.warning('"%s" not found in the test suite', id)
+                trace.warning('"%s" not found in the test suite', id)
         for id in duplicates:
-            bzrlib.trace.warning('"%s" is used as an id by several tests', id)
+            trace.warning('"%s" is used as an id by several tests', id)
 
     return suite
 
 
-def multiply_scenarios(scenarios_left, scenarios_right):
+def multiply_scenarios(*scenarios):
+    """Multiply two or more iterables of scenarios.
+
+    It is safe to pass scenario generators or iterators.
+
+    :returns: A list of compound scenarios: the cross-product of all 
+        scenarios, with the names concatenated and the parameters
+        merged together.
+    """
+    return reduce(_multiply_two_scenarios, map(list, scenarios))
+
+
+def _multiply_two_scenarios(scenarios_left, scenarios_right):
     """Multiply two sets of scenarios.
 
     :returns: the cartesian product of the two sets of scenarios, that is
@@ -3998,7 +4259,7 @@
     ...     bzrlib.tests.test_sampler.DemoTest('test_nothing'),
     ...     [('one', dict(param=1)),
     ...      ('two', dict(param=2))],
-    ...     TestSuite())
+    ...     TestUtil.TestSuite())
     >>> tests = list(iter_suite_tests(r))
     >>> len(tests)
     2
@@ -4053,6 +4314,18 @@
     """
     new_test = copy.copy(test)
     new_test.id = lambda: new_id
+    # XXX: Workaround <https://bugs.launchpad.net/testtools/+bug/637725>, which
+    # causes cloned tests to share the 'details' dict.  This makes it hard to
+    # read the test output for parameterized tests, because tracebacks will be
+    # associated with irrelevant tests.
+    try:
+        details = new_test._TestCase__details
+    except AttributeError:
+        # must be a different version of testtools than expected.  Do nothing.
+        pass
+    else:
+        # Reset the '__details' dict.
+        new_test._TestCase__details = {}
     return new_test
 
 
@@ -4079,7 +4352,8 @@
         the module is available.
     """
 
-    py_module = __import__(py_module_name, {}, {}, ['NO_SUCH_ATTRIB'])
+    from bzrlib.tests.features import ModuleAvailableFeature
+    py_module = pyutils.get_named_object(py_module_name)
     scenarios = [
         ('python', {'module': py_module}),
     ]
@@ -4125,165 +4399,6 @@
                          % (os.path.basename(dirname), printable_e))
 
 
-class Feature(object):
-    """An operating system Feature."""
-
-    def __init__(self):
-        self._available = None
-
-    def available(self):
-        """Is the feature available?
-
-        :return: True if the feature is available.
-        """
-        if self._available is None:
-            self._available = self._probe()
-        return self._available
-
-    def _probe(self):
-        """Implement this method in concrete features.
-
-        :return: True if the feature is available.
-        """
-        raise NotImplementedError
-
-    def __str__(self):
-        if getattr(self, 'feature_name', None):
-            return self.feature_name()
-        return self.__class__.__name__
-
-
-class _SymlinkFeature(Feature):
-
-    def _probe(self):
-        return osutils.has_symlinks()
-
-    def feature_name(self):
-        return 'symlinks'
-
-SymlinkFeature = _SymlinkFeature()
-
-
-class _HardlinkFeature(Feature):
-
-    def _probe(self):
-        return osutils.has_hardlinks()
-
-    def feature_name(self):
-        return 'hardlinks'
-
-HardlinkFeature = _HardlinkFeature()
-
-
-class _OsFifoFeature(Feature):
-
-    def _probe(self):
-        return getattr(os, 'mkfifo', None)
-
-    def feature_name(self):
-        return 'filesystem fifos'
-
-OsFifoFeature = _OsFifoFeature()
-
-
-class _UnicodeFilenameFeature(Feature):
-    """Does the filesystem support Unicode filenames?"""
-
-    def _probe(self):
-        try:
-            # Check for character combinations unlikely to be covered by any
-            # single non-unicode encoding. We use the characters
-            # - greek small letter alpha (U+03B1) and
-            # - braille pattern dots-123456 (U+283F).
-            os.stat(u'\u03b1\u283f')
-        except UnicodeEncodeError:
-            return False
-        except (IOError, OSError):
-            # The filesystem allows the Unicode filename but the file doesn't
-            # exist.
-            return True
-        else:
-            # The filesystem allows the Unicode filename and the file exists,
-            # for some reason.
-            return True
-
-UnicodeFilenameFeature = _UnicodeFilenameFeature()
-
-
-class _CompatabilityThunkFeature(Feature):
-    """This feature is just a thunk to another feature.
-
-    It issues a deprecation warning if it is accessed, to let you know that you
-    should really use a different feature.
-    """
-
-    def __init__(self, dep_version, module, name,
-                 replacement_name, replacement_module=None):
-        super(_CompatabilityThunkFeature, self).__init__()
-        self._module = module
-        if replacement_module is None:
-            replacement_module = module
-        self._replacement_module = replacement_module
-        self._name = name
-        self._replacement_name = replacement_name
-        self._dep_version = dep_version
-        self._feature = None
-
-    def _ensure(self):
-        if self._feature is None:
-            depr_msg = self._dep_version % ('%s.%s'
-                                            % (self._module, self._name))
-            use_msg = ' Use %s.%s instead.' % (self._replacement_module,
-                                               self._replacement_name)
-            symbol_versioning.warn(depr_msg + use_msg, DeprecationWarning)
-            # Import the new feature and use it as a replacement for the
-            # deprecated one.
-            mod = __import__(self._replacement_module, {}, {},
-                             [self._replacement_name])
-            self._feature = getattr(mod, self._replacement_name)
-
-    def _probe(self):
-        self._ensure()
-        return self._feature._probe()
-
-
-class ModuleAvailableFeature(Feature):
-    """This is a feature than describes a module we want to be available.
-
-    Declare the name of the module in __init__(), and then after probing, the
-    module will be available as 'self.module'.
-
-    :ivar module: The module if it is available, else None.
-    """
-
-    def __init__(self, module_name):
-        super(ModuleAvailableFeature, self).__init__()
-        self.module_name = module_name
-
-    def _probe(self):
-        try:
-            self._module = __import__(self.module_name, {}, {}, [''])
-            return True
-        except ImportError:
-            return False
-
-    @property
-    def module(self):
-        if self.available(): # Make sure the probe has been done
-            return self._module
-        return None
-
-    def feature_name(self):
-        return self.module_name
-
-
-# This is kept here for compatibility, it is recommended to use
-# 'bzrlib.tests.feature.paramiko' instead
-ParamikoFeature = _CompatabilityThunkFeature(
-    deprecated_in((2,1,0)),
-    'bzrlib.tests.features', 'ParamikoFeature', 'paramiko')
-
-
 def probe_unicode_in_user_encoding():
     """Try to encode several unicode strings to use in unicode-aware tests.
     Return first successfull match.
@@ -4317,202 +4432,36 @@
     return None
 
 
-class _HTTPSServerFeature(Feature):
-    """Some tests want an https Server, check if one is available.
-
-    Right now, the only way this is available is under python2.6 which provides
-    an ssl module.
-    """
-
-    def _probe(self):
-        try:
-            import ssl
-            return True
-        except ImportError:
-            return False
-
-    def feature_name(self):
-        return 'HTTPSServer'
-
-
-HTTPSServerFeature = _HTTPSServerFeature()
-
-
-class _UnicodeFilename(Feature):
-    """Does the filesystem support Unicode filenames?"""
-
-    def _probe(self):
-        try:
-            os.stat(u'\u03b1')
-        except UnicodeEncodeError:
-            return False
-        except (IOError, OSError):
-            # The filesystem allows the Unicode filename but the file doesn't
-            # exist.
-            return True
-        else:
-            # The filesystem allows the Unicode filename and the file exists,
-            # for some reason.
-            return True
-
-UnicodeFilename = _UnicodeFilename()
-
-
-class _ByteStringNamedFilesystem(Feature):
-    """Is the filesystem based on bytes?"""
-
-    def _probe(self):
-        if os.name == "posix":
-            return True
-        return False
-
-ByteStringNamedFilesystem = _ByteStringNamedFilesystem()
-
-
-class _UTF8Filesystem(Feature):
-    """Is the filesystem UTF-8?"""
-
-    def _probe(self):
-        if osutils._fs_enc.upper() in ('UTF-8', 'UTF8'):
-            return True
-        return False
-
-UTF8Filesystem = _UTF8Filesystem()
-
-
-class _BreakinFeature(Feature):
-    """Does this platform support the breakin feature?"""
-
-    def _probe(self):
-        from bzrlib import breakin
-        if breakin.determine_signal() is None:
-            return False
-        if sys.platform == 'win32':
-            # Windows doesn't have os.kill, and we catch the SIGBREAK signal.
-            # We trigger SIGBREAK via a Console api so we need ctypes to
-            # access the function
-            try:
-                import ctypes
-            except OSError:
-                return False
-        return True
-
-    def feature_name(self):
-        return "SIGQUIT or SIGBREAK w/ctypes on win32"
-
-
-BreakinFeature = _BreakinFeature()
-
-
-class _CaseInsCasePresFilenameFeature(Feature):
-    """Is the file-system case insensitive, but case-preserving?"""
-
-    def _probe(self):
-        fileno, name = tempfile.mkstemp(prefix='MixedCase')
-        try:
-            # first check truly case-preserving for created files, then check
-            # case insensitive when opening existing files.
-            name = osutils.normpath(name)
-            base, rel = osutils.split(name)
-            found_rel = osutils.canonical_relpath(base, name)
-            return (found_rel == rel
-                    and os.path.isfile(name.upper())
-                    and os.path.isfile(name.lower()))
-        finally:
-            os.close(fileno)
-            os.remove(name)
-
-    def feature_name(self):
-        return "case-insensitive case-preserving filesystem"
-
-CaseInsCasePresFilenameFeature = _CaseInsCasePresFilenameFeature()
-
-
-class _CaseInsensitiveFilesystemFeature(Feature):
-    """Check if underlying filesystem is case-insensitive but *not* case
-    preserving.
-    """
-    # Note that on Windows, Cygwin, MacOS etc, the file-systems are far
-    # more likely to be case preserving, so this case is rare.
-
-    def _probe(self):
-        if CaseInsCasePresFilenameFeature.available():
-            return False
-
-        if TestCaseWithMemoryTransport.TEST_ROOT is None:
-            root = osutils.mkdtemp(prefix='testbzr-', suffix='.tmp')
-            TestCaseWithMemoryTransport.TEST_ROOT = root
-        else:
-            root = TestCaseWithMemoryTransport.TEST_ROOT
-        tdir = osutils.mkdtemp(prefix='case-sensitive-probe-', suffix='',
-            dir=root)
-        name_a = osutils.pathjoin(tdir, 'a')
-        name_A = osutils.pathjoin(tdir, 'A')
-        os.mkdir(name_a)
-        result = osutils.isdir(name_A)
-        _rmtree_temp_dir(tdir)
-        return result
-
-    def feature_name(self):
-        return 'case-insensitive filesystem'
-
-CaseInsensitiveFilesystemFeature = _CaseInsensitiveFilesystemFeature()
-
-
-class _CaseSensitiveFilesystemFeature(Feature):
-
-    def _probe(self):
-        if CaseInsCasePresFilenameFeature.available():
-            return False
-        elif CaseInsensitiveFilesystemFeature.available():
-            return False
-        else:
-            return True
-
-    def feature_name(self):
-        return 'case-sensitive filesystem'
-
-# new coding style is for feature instances to be lowercase
-case_sensitive_filesystem_feature = _CaseSensitiveFilesystemFeature()
-
-
-# Kept for compatibility, use bzrlib.tests.features.subunit instead
-SubUnitFeature = _CompatabilityThunkFeature(
-    deprecated_in((2,1,0)),
-    'bzrlib.tests.features', 'SubUnitFeature', 'subunit')
 # Only define SubUnitBzrRunner if subunit is available.
 try:
     from subunit import TestProtocolClient
     from subunit.test_results import AutoTimingTestResultDecorator
+    class SubUnitBzrProtocolClient(TestProtocolClient):
+
+        def stopTest(self, test):
+            super(SubUnitBzrProtocolClient, self).stopTest(test)
+            _clear__type_equality_funcs(test)
+
+        def addSuccess(self, test, details=None):
+            # The subunit client always includes the details in the subunit
+            # stream, but we don't want to include it in ours.
+            if details is not None and 'log' in details:
+                del details['log']
+            return super(SubUnitBzrProtocolClient, self).addSuccess(
+                test, details)
+
     class SubUnitBzrRunner(TextTestRunner):
         def run(self, test):
             result = AutoTimingTestResultDecorator(
-                TestProtocolClient(self.stream))
+                SubUnitBzrProtocolClient(self.stream))
             test.run(result)
             return result
 except ImportError:
     pass
 
-class _PosixPermissionsFeature(Feature):
-
-    def _probe(self):
-        def has_perms():
-            # create temporary file and check if specified perms are maintained.
-            import tempfile
-
-            write_perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
-            f = tempfile.mkstemp(prefix='bzr_perms_chk_')
-            fd, name = f
-            os.close(fd)
-            os.chmod(name, write_perms)
-
-            read_perms = os.stat(name).st_mode & 0777
-            os.unlink(name)
-            return (write_perms == read_perms)
-
-        return (os.name == 'posix') and has_perms()
-
-    def feature_name(self):
-        return 'POSIX permissions support'
-
-posix_permissions_feature = _PosixPermissionsFeature()
+
+@deprecated_function(deprecated_in((2, 5, 0)))
+def ModuleAvailableFeature(name):
+    from bzrlib.tests import features
+    return features.ModuleAvailableFeature(name)
+    

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2010-07-28 07:05:19 +0000
+++ bzrlib/tests/blackbox/__init__.py	2011-09-27 12:13:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,97 +35,105 @@
     # add the tests for this module
     suite.addTests(basic_tests)
 
+    prefix = 'bzrlib.tests.blackbox.'
     testmod_names = [
-                     'bzrlib.tests.blackbox.test_add',
-                     'bzrlib.tests.blackbox.test_added',
-                     'bzrlib.tests.blackbox.test_alias',
-                     'bzrlib.tests.blackbox.test_aliases',
-                     'bzrlib.tests.blackbox.test_ancestry',
-                     'bzrlib.tests.blackbox.test_annotate',
-                     'bzrlib.tests.blackbox.test_branch',
-                     'bzrlib.tests.blackbox.test_break_lock',
-                     'bzrlib.tests.blackbox.test_breakin',
-                     'bzrlib.tests.blackbox.test_bound_branches',
-                     'bzrlib.tests.blackbox.test_bundle_info',
-                     'bzrlib.tests.blackbox.test_cat',
-                     'bzrlib.tests.blackbox.test_cat_revision',
-                     'bzrlib.tests.blackbox.test_check',
-                     'bzrlib.tests.blackbox.test_checkout',
-                     'bzrlib.tests.blackbox.test_clean_tree',
-                     'bzrlib.tests.blackbox.test_command_encoding',
-                     'bzrlib.tests.blackbox.test_commit',
-                     'bzrlib.tests.blackbox.test_conflicts',
-                     'bzrlib.tests.blackbox.test_debug',
-                     'bzrlib.tests.blackbox.test_deleted',
-                     'bzrlib.tests.blackbox.test_diff',
-                     'bzrlib.tests.blackbox.test_dump_btree',
-                     'bzrlib.tests.blackbox.test_dpush',
-                     'bzrlib.tests.blackbox.test_exceptions',
-                     'bzrlib.tests.blackbox.test_export',
-                     'bzrlib.tests.blackbox.test_filesystem_cicp',
-                     'bzrlib.tests.blackbox.test_filtered_view_ops',
-                     'bzrlib.tests.blackbox.test_find_merge_base',
-                     'bzrlib.tests.blackbox.test_help',
-                     'bzrlib.tests.blackbox.test_hooks',
-                     'bzrlib.tests.blackbox.test_ignore',
-                     'bzrlib.tests.blackbox.test_ignored',
-                     'bzrlib.tests.blackbox.test_info',
-                     'bzrlib.tests.blackbox.test_init',
-                     'bzrlib.tests.blackbox.test_inventory',
-                     'bzrlib.tests.blackbox.test_join',
-                     'bzrlib.tests.blackbox.test_locale',
-                     'bzrlib.tests.blackbox.test_log',
-                     'bzrlib.tests.blackbox.test_logformats',
-                     'bzrlib.tests.blackbox.test_lookup_revision',
-                     'bzrlib.tests.blackbox.test_ls',
-                     'bzrlib.tests.blackbox.test_lsprof',
-                     'bzrlib.tests.blackbox.test_merge',
-                     'bzrlib.tests.blackbox.test_merge_directive',
-                     'bzrlib.tests.blackbox.test_missing',
-                     'bzrlib.tests.blackbox.test_modified',
-                     'bzrlib.tests.blackbox.test_mv',
-                     'bzrlib.tests.blackbox.test_nick',
-                     'bzrlib.tests.blackbox.test_non_ascii',
-                     'bzrlib.tests.blackbox.test_outside_wt',
-                     'bzrlib.tests.blackbox.test_pack',
-                     'bzrlib.tests.blackbox.test_pull',
-                     'bzrlib.tests.blackbox.test_push',
-                     'bzrlib.tests.blackbox.test_reconcile',
-                     'bzrlib.tests.blackbox.test_reconfigure',
-                     'bzrlib.tests.blackbox.test_reference',
-                     'bzrlib.tests.blackbox.test_remerge',
-                     'bzrlib.tests.blackbox.test_remove',
-                     'bzrlib.tests.blackbox.test_re_sign',
-                     'bzrlib.tests.blackbox.test_remove_tree',
-                     'bzrlib.tests.blackbox.test_revert',
-                     'bzrlib.tests.blackbox.test_revno',
-                     'bzrlib.tests.blackbox.test_revision_history',
-                     'bzrlib.tests.blackbox.test_revision_info',
-                     'bzrlib.tests.blackbox.test_rmbranch',
-                     'bzrlib.tests.blackbox.test_selftest',
-                     'bzrlib.tests.blackbox.test_send',
-                     'bzrlib.tests.blackbox.test_serve',
-                     'bzrlib.tests.blackbox.test_shared_repository',
-                     'bzrlib.tests.blackbox.test_shelve',
-                     'bzrlib.tests.blackbox.test_sign_my_commits',
-                     'bzrlib.tests.blackbox.test_split',
-                     'bzrlib.tests.blackbox.test_status',
-                     'bzrlib.tests.blackbox.test_switch',
-                     'bzrlib.tests.blackbox.test_tags',
-                     'bzrlib.tests.blackbox.test_testament',
-                     'bzrlib.tests.blackbox.test_too_much',
-                     'bzrlib.tests.blackbox.test_uncommit',
-                     'bzrlib.tests.blackbox.test_unknowns',
-                     'bzrlib.tests.blackbox.test_update',
-                     'bzrlib.tests.blackbox.test_upgrade',
-                     'bzrlib.tests.blackbox.test_version',
-                     'bzrlib.tests.blackbox.test_version_info',
-                     'bzrlib.tests.blackbox.test_versioning',
-                     'bzrlib.tests.blackbox.test_view',
-                     'bzrlib.tests.blackbox.test_whoami',
+                     'test_add',
+                     'test_added',
+                     'test_alias',
+                     'test_aliases',
+                     'test_ancestry',
+                     'test_annotate',
+                     'test_branch',
+                     'test_branches',
+                     'test_break_lock',
+                     'test_bound_branches',
+                     'test_bundle_info',
+                     'test_cat',
+                     'test_cat_revision',
+                     'test_check',
+                     'test_checkout',
+                     'test_clean_tree',
+                     'test_command_encoding',
+                     'test_commit',
+                     'test_config',
+                     'test_conflicts',
+                     'test_debug',
+                     'test_deleted',
+                     'test_diff',
+                     'test_dump_btree',
+                     'test_dpush',
+                     'test_exceptions',
+                     'test_export',
+                     'test_export_pot',
+                     'test_filesystem_cicp',
+                     'test_filtered_view_ops',
+                     'test_find_merge_base',
+                     'test_help',
+                     'test_hooks',
+                     'test_ignore',
+                     'test_ignored',
+                     'test_info',
+                     'test_init',
+                     'test_inventory',
+                     'test_join',
+                     'test_locale',
+                     'test_log',
+                     'test_logformats',
+                     'test_lookup_revision',
+                     'test_ls',
+                     'test_lsprof',
+                     'test_merge',
+                     'test_merge_directive',
+                     'test_missing',
+                     'test_modified',
+                     'test_mv',
+                     'test_nick',
+                     'test_non_ascii',
+                     'test_outside_wt',
+                     'test_pack',
+                     'test_pull',
+                     'test_push',
+                     'test_reconcile',
+                     'test_reconfigure',
+                     'test_reference',
+                     'test_remerge',
+                     'test_remove',
+                     'test_re_sign',
+                     'test_remember_option',
+                     'test_remove_tree',
+                     'test_repair_workingtree',
+                     'test_resolve',
+                     'test_revert',
+                     'test_revno',
+                     'test_revision_history',
+                     'test_revision_info',
+                     'test_rmbranch',
+                     'test_script',
+                     'test_selftest',
+                     'test_send',
+                     'test_serve',
+                     'test_shared_repository',
+                     'test_shelve',
+                     'test_sign_my_commits',
+                     'test_split',
+                     'test_status',
+                     'test_switch',
+                     'test_tags',
+                     'test_testament',
+                     'test_too_much',
+                     'test_uncommit',
+                     'test_unknowns',
+                     'test_update',
+                     'test_upgrade',
+                     'test_version',
+                     'test_version_info',
+                     'test_versioning',
+                     'test_view',
+                     'test_whoami',
                      ]
     # add the tests for the sub modules
-    suite.addTests(loader.loadTestsFromModuleNames(testmod_names))
+    suite.addTests(loader.loadTestsFromModuleNames(
+            [prefix + module_name for module_name in testmod_names]))
     return suite
 
 

=== modified file 'bzrlib/tests/blackbox/test_add.py'
--- bzrlib/tests/blackbox/test_add.py	2010-07-16 13:53:26 +0000
+++ bzrlib/tests/blackbox/test_add.py	2011-08-19 21:01:49 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,22 +24,21 @@
     tests,
     )
 from bzrlib.tests import (
+    features,
     script,
     )
-
-
-def load_tests(standard_tests, module, loader):
-    """Parameterize tests for view-aware vs not."""
-    to_adapt, result = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance(TestAdd))
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class TestAdd(tests.TestCaseWithTransport):
+
     scenarios = [
         ('pre-views', {'branch_tree_format': 'pack-0.92'}),
-        ('view-aware', {'branch_tree_format': 'development6-rich-root'}),
+        ('view-aware', {'branch_tree_format': '2a'}),
         ]
-    return tests.multiply_tests(to_adapt, scenarios, result)
-
-
-class TestAdd(tests.TestCaseWithTransport):
 
     def make_branch_and_tree(self, dir):
         return super(TestAdd, self).make_branch_and_tree(
@@ -207,7 +206,7 @@
         self.assertContainsRe(err, r'ERROR:.*\.bzr.*control file')
 
     def test_add_via_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.make_branch_and_tree('source')
         self.build_tree(['source/top.txt'])
         os.symlink('source', 'link')
@@ -215,7 +214,7 @@
         self.assertEquals(out, 'adding top.txt\n')
 
     def test_add_symlink_to_abspath(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.make_branch_and_tree('tree')
         os.symlink(osutils.abspath('target'), 'tree/link')
         out = self.run_bzr(['add', 'tree/link'])[0]
@@ -231,3 +230,36 @@
         $ bzr add tree1/a tree2/b
         2>bzr: ERROR: Path "...tree2/b" is not a child of path "...tree1"
         ''')
+
+    def test_add_multiple_files_in_unicode_cwd(self):
+        """Adding multiple files in a non-ascii cwd, see lp:686611"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        self.make_branch_and_tree(u"\xA7")
+        self.build_tree([u"\xA7/a", u"\xA7/b"])
+        out, err = self.run_bzr(["add", "a", "b"], working_dir=u"\xA7")
+        self.assertEquals(out, "adding a\n" "adding b\n")
+        self.assertEquals(err, "")
+        
+    def test_add_skip_large_files(self):
+        """Test skipping files larger than add.maximum_file_size"""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['small.txt', 'big.txt', 'big2.txt'])
+        self.build_tree_contents([('small.txt', '0\n')])
+        self.build_tree_contents([('big.txt', '01234567890123456789\n')])
+        self.build_tree_contents([('big2.txt', '01234567890123456789\n')])
+        tree.branch.get_config().set_user_option('add.maximum_file_size', 5)
+        out = self.run_bzr('add')[0]
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['adding small.txt'], 
+                          results)
+        # named items never skipped, even if over max
+        out, err = self.run_bzr(["add", "big2.txt"])
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['adding big2.txt'], 
+                          results)
+        self.assertEquals(err, "")
+        tree.branch.get_config().set_user_option('add.maximum_file_size', 30)
+        out = self.run_bzr('add')[0]
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['adding big.txt'], 
+                          results)

=== modified file 'bzrlib/tests/blackbox/test_alias.py'
--- bzrlib/tests/blackbox/test_alias.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_alias.py	2011-06-14 01:26:41 +0000
@@ -16,14 +16,17 @@
 #
 
 """Tests of the 'bzr alias' command."""
-import os
-import codecs
-
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
-
-
-class TestAlias(TestCaseWithTransport):
+
+from bzrlib import (
+    config,
+    tests,
+    )
+from bzrlib.tests import (
+    features,
+    )
+
+
+class TestAlias(tests.TestCaseWithTransport):
 
     def test_list_alias_with_none(self):
         """Calling alias with no parameters lists existing aliases."""
@@ -46,7 +49,9 @@
 
     def test_unicode_alias(self):
         """Unicode aliases should work (Bug #529930)"""
-        config_enc = 'utf-8'
+        # XXX: strictly speaking, lack of unicode filenames doesn't imply that
+        # unicode command lines aren't available.
+        self.requireFeature(features.UnicodeFilenameFeature)
         file_name = u'foo\xb6'
 
         tree = self.make_branch_and_tree('.')
@@ -54,11 +59,8 @@
         tree.add(file_name)
         tree.commit('added')
 
-        ensure_config_dir_exists()
-        CONFIG=(u'[ALIASES]\n'
-                u'ust=st foo\xb6\n')
-
-        codecs.open(config_filename(),'w', config_enc).write(CONFIG)
+        config.GlobalConfig.from_string(
+            u'[ALIASES]\nust=st %s\n' % (file_name,), save=True)
 
         out, err = self.run_bzr('ust')
         self.assertEquals(err, '')

=== modified file 'bzrlib/tests/blackbox/test_aliases.py'
--- bzrlib/tests/blackbox/test_aliases.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_aliases.py	2010-08-29 14:32:45 +0000
@@ -18,10 +18,10 @@
 
 import os
 
+from bzrlib import config
 from bzrlib.branch import Branch
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.trace import mutter
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
 
 
 class TestAliases(TestCaseWithTransport):
@@ -35,18 +35,11 @@
             return self.run_bzr(args, **kwargs)[1]
 
 
-        if os.path.isfile(config_filename()):
-            # Something is wrong in environment,
-            # we risk overwriting users config
-            self.assert_(config_filename() + "exists, abort")
-
-        ensure_config_dir_exists()
-        CONFIG=("[ALIASES]\n"
-                "c=cat\n"
-                "c1=cat -r 1\n"
-                "c2=cat -r 1 -r2\n")
-
-        open(config_filename(),'wb').write(CONFIG)
+        conf = config.GlobalConfig.from_string('''[ALIASES]
+c=cat
+c1=cat -r 1
+c2=cat -r 1 -r2
+''', save=True)
 
         str1 = 'foo\n'
         str2 = 'bar\n'

=== modified file 'bzrlib/tests/blackbox/test_ancestry.py'
--- bzrlib/tests/blackbox/test_ancestry.py	2009-08-18 16:53:50 +0000
+++ bzrlib/tests/blackbox/test_ancestry.py	2011-06-13 15:16:17 +0000
@@ -17,9 +17,6 @@
 import os
 
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.workingtree import WorkingTree
-from bzrlib.branch import Branch
-
 
 class TestAncestry(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2010-05-02 20:10:25 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2011-06-13 15:18:27 +0000
@@ -24,15 +24,15 @@
 """
 
 
-import os
+from bzrlib import (
+    config,
+    tests,
+    )
 
-from bzrlib.branch import Branch
-from bzrlib.config import extract_email_address
-from bzrlib.tests import TestCaseWithTransport
 from bzrlib.urlutils import joinpath
 
 
-class TestAnnotate(TestCaseWithTransport):
+class TestAnnotate(tests.TestCaseWithTransport):
 
     def setUp(self):
         super(TestAnnotate, self).setUp()
@@ -151,7 +151,7 @@
                          err)
 
 
-class TestSimpleAnnotate(TestCaseWithTransport):
+class TestSimpleAnnotate(tests.TestCaseWithTransport):
     """Annotate tests with no complex setup."""
 
     def _setup_edited_file(self, relpath='.'):
@@ -162,14 +162,13 @@
         tree.add('file')
         tree.commit('add file', committer="test@host", rev_id="rev1")
         self.build_tree_contents([(file_relpath, 'foo\nbar\ngam\n')])
-        tree.branch.get_config().set_user_option('email', 'current@host2')
         return tree
 
     def test_annotate_cmd_revspec_branch(self):
         tree = self._setup_edited_file('trunk')
         tree.branch.create_checkout(self.get_url('work'), lightweight=True)
-        os.chdir('work')
-        out, err = self.run_bzr('annotate file -r branch:../trunk')
+        out, err = self.run_bzr(['annotate', 'file', '-r', 'branch:../trunk'],
+                                working_dir='work')
         self.assertEqual('', err)
         self.assertEqual(
             '1   test@ho | foo\n'
@@ -178,6 +177,7 @@
 
     def test_annotate_edited_file(self):
         tree = self._setup_edited_file()
+        tree.branch.get_config().set_user_option('email', 'current@host2')
         out, err = self.run_bzr('annotate file')
         self.assertEqual(
             '1   test@ho | foo\n'
@@ -185,8 +185,24 @@
             '1   test@ho | gam\n',
             out)
 
+    def test_annotate_edited_file_no_default(self):
+        # Ensure that when no username is available annotate still works.
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
+        # Also, make sure that it's not inferred from mailname.
+        self.overrideAttr(config, '_auto_user_id',
+            lambda: (None, None))
+        tree = self._setup_edited_file()
+        out, err = self.run_bzr('annotate file')
+        self.assertEqual(
+            '1   test@ho | foo\n'
+            '2?  local u | bar\n'
+            '1   test@ho | gam\n',
+            out)
+
     def test_annotate_edited_file_show_ids(self):
         tree = self._setup_edited_file()
+        tree.branch.get_config().set_user_option('email', 'current@host2')
         out, err = self.run_bzr('annotate file --show-ids')
         self.assertEqual(
             '    rev1 | foo\n'
@@ -211,11 +227,12 @@
         tree.merge_from_branch(tree.branch, "rev1.1.1")
         # edit the file to be 'resolved' and have a further local edit
         self.build_tree_contents([('file', 'local\nfoo\nbar\nbaz\ngam\n')])
+        return tree
 
     def test_annotated_edited_merged_file_revnos(self):
-        self._create_merged_file()
-        out, err = self.run_bzr('annotate file')
-        email = extract_email_address(Branch.open('.').get_config().username())
+        wt = self._create_merged_file()
+        out, err = self.run_bzr(['annotate', 'file'])
+        email = config.extract_email_address(wt.branch.get_config().username())
         self.assertEqual(
             '3?    %-7s | local\n'
             '1     test@ho | foo\n'
@@ -226,7 +243,7 @@
 
     def test_annotated_edited_merged_file_ids(self):
         self._create_merged_file()
-        out, err = self.run_bzr('annotate file --show-ids')
+        out, err = self.run_bzr(['annotate', 'file', '--show-ids'])
         self.assertEqual(
             'current: | local\n'
             '    rev1 | foo\n'
@@ -236,47 +253,50 @@
             out)
 
     def test_annotate_empty_file(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/empty', '')])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
         tree.add('empty')
         tree.commit('add empty file')
+        out, err = self.run_bzr(['annotate', 'empty'])
+        self.assertEqual('', out)
 
-        os.chdir('tree')
-        out, err = self.run_bzr('annotate empty')
+    def test_annotate_removed_file(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
+        tree.add('empty')
+        tree.commit('add empty file')
+        # delete the file.
+        tree.remove('empty')
+        tree.commit('remove empty file')
+        out, err = self.run_bzr(['annotate', '-r1', 'empty'])
         self.assertEqual('', out)
 
     def test_annotate_empty_file_show_ids(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/empty', '')])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
         tree.add('empty')
         tree.commit('add empty file')
-
-        os.chdir('tree')
         out, err = self.run_bzr(['annotate', '--show-ids', 'empty'])
         self.assertEqual('', out)
 
     def test_annotate_nonexistant_file(self):
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/file'])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['file'])
         tree.add(['file'])
         tree.commit('add a file')
-
-        os.chdir('tree')
-        out, err = self.run_bzr("annotate doesnotexist", retcode=3)
+        out, err = self.run_bzr(['annotate', 'doesnotexist'], retcode=3)
         self.assertEqual('', out)
         self.assertEqual("bzr: ERROR: doesnotexist is not versioned.\n", err)
 
     def test_annotate_without_workingtree(self):
-        tree = self.make_branch_and_tree('branch')
-        self.build_tree_contents([('branch/empty', '')])
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('empty', '')])
         tree.add('empty')
         tree.commit('add empty file')
         bzrdir = tree.branch.bzrdir
         bzrdir.destroy_workingtree()
         self.assertFalse(bzrdir.has_workingtree())
-
-        os.chdir('branch')
-        out, err = self.run_bzr('annotate empty')
+        out, err = self.run_bzr(['annotate', 'empty'])
         self.assertEqual('', out)
 
     def test_annotate_directory(self):
@@ -285,5 +305,5 @@
         self.build_tree_contents([('a/hello.txt', 'my helicopter\n')])
         wt.add(['hello.txt'])
         wt.commit('commit', committer='test@user')
-        out, err = self.run_bzr('annotate -d a hello.txt')
+        out, err = self.run_bzr(['annotate', '-d', 'a', 'hello.txt'])
         self.assertEqualDiff('1   test@us | my helicopter\n', out)

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2010-05-02 20:10:25 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2011-09-26 15:21:01 +0000
@@ -18,48 +18,14 @@
 """Tests of bound branches (binding, unbinding, commit, etc) command."""
 
 import os
-from cStringIO import StringIO
 
 from bzrlib import (
-    bzrdir,
     errors,
     tests,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import (BzrDir, BzrDirFormat, BzrDirMetaFormat1)
-from bzrlib.osutils import getcwd
+from bzrlib.bzrdir import BzrDir
 from bzrlib.tests import script
-import bzrlib.urlutils as urlutils
-from bzrlib.workingtree import WorkingTree
-
-
-class TestLegacyFormats(tests.TestCaseWithTransport):
-
-    def setUp(self):
-        super(TestLegacyFormats, self).setUp()
-        self.build_tree(['master/', 'child/'])
-        self.make_branch_and_tree('master')
-        self.make_branch_and_tree('child',
-                        format=bzrdir.format_registry.make_bzrdir('weave'))
-        os.chdir('child')
-
-    def test_bind_format_6_bzrdir(self):
-        # bind on a format 6 bzrdir should error
-        out,err = self.run_bzr('bind ../master', retcode=3)
-        self.assertEqual('', out)
-        # TODO: jam 20060427 Probably something like this really should
-        #       print out the actual path, rather than the URL
-        cwd = urlutils.local_path_to_url(getcwd())
-        self.assertEqual('bzr: ERROR: To use this feature you must '
-                         'upgrade your branch at %s/.\n' % cwd, err)
-
-    def test_unbind_format_6_bzrdir(self):
-        # bind on a format 6 bzrdir should error
-        out,err = self.run_bzr('unbind', retcode=3)
-        self.assertEqual('', out)
-        cwd = urlutils.local_path_to_url(getcwd())
-        self.assertEqual('bzr: ERROR: To use this feature you must '
-                         'upgrade your branch at %s/.\n' % cwd, err)
 
 
 class TestBoundBranches(tests.TestCaseWithTransport):
@@ -83,7 +49,7 @@
 
     def check_revno(self, val, loc='.'):
         self.assertEqual(
-            val, len(BzrDir.open(loc).open_branch().revision_history()))
+            val, BzrDir.open(loc).open_branch().last_revision_info()[0])
 
     def test_simple_binding(self):
         tree = self.make_branch_and_tree('base')
@@ -263,10 +229,9 @@
         child_tree.commit(message='merged')
         self.check_revno(3)
 
-        # After binding, the revision history should be unaltered
-        # take a copy before
-        base_history = base_branch.revision_history()
-        child_history = child_branch.revision_history()
+        self.assertEquals(
+            child_tree.branch.last_revision(),
+            base_tree.branch.last_revision())
 
     def test_bind_parent_ahead(self):
         base_tree = self.create_branches()[0]
@@ -367,11 +332,11 @@
         self.build_tree_contents([('other/c', 'file c\n')])
         other_tree.add('c')
         other_tree.commit(message='adding c')
-        new_rev_id = other_branch.revision_history()[-1]
+        new_rev_id = other_branch.last_revision()
 
         child_tree.merge_from_branch(other_branch)
 
-        self.failUnlessExists('child/c')
+        self.assertPathExists('child/c')
         self.assertEqual([new_rev_id], child_tree.get_parent_ids()[1:])
 
         # Make sure the local branch has the installed revision
@@ -443,7 +408,9 @@
     def test_bind_when_bound(self):
         self.run_script("""
 $ bzr init trunk
+...
 $ bzr init copy
+...
 $ cd copy
 $ bzr bind ../trunk
 $ bzr bind
@@ -453,6 +420,7 @@
     def test_bind_before_bound(self):
         self.run_script("""
 $ bzr init trunk
+...
 $ cd trunk
 $ bzr bind
 2>bzr: ERROR: No location supplied and no previous location known

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2010-07-21 20:09:34 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2011-10-13 14:12:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,18 +22,23 @@
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
-    repository,
     revision as _mod_revision,
     )
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit1
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests import (
-    KnownFailure,
+    fixtures,
+    script,
+    test_server,
+    )
+from bzrlib.tests.features import (
     HardlinkFeature,
-    test_server,
     )
+from bzrlib.tests.blackbox import test_switch
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.tests.script import run_script
 from bzrlib.urlutils import local_path_to_url, strip_trailing_slash
 from bzrlib.workingtree import WorkingTree
 
@@ -59,6 +64,44 @@
         self.assertFalse(b._transport.has('branch-name'))
         b.bzrdir.open_workingtree().commit(message='foo', allow_pointless=True)
 
+    def test_into_colocated(self):
+        """Branch from a branch into a colocated branch."""
+        self.example_branch('a')
+        out, err = self.run_bzr(
+            'init --format=development-colo file:b,branch=orig')
+        self.assertEqual(
+            """Created a standalone tree (format: development-colo)\n""",
+            out)
+        self.assertEqual('', err)
+        out, err = self.run_bzr(
+            'branch --use-existing-dir a file:b,branch=thiswasa')
+        self.assertEqual('', out)
+        self.assertEqual('Branched 2 revisions.\n', err)
+        out, err = self.run_bzr('branches b')
+        self.assertEqual(" orig\n thiswasa\n", out)
+        self.assertEqual('', err)
+
+    def test_branch_broken_pack(self):
+        """branching with a corrupted pack file."""
+        self.example_branch('a')
+        # add some corruption
+        packs_dir = 'a/.bzr/repository/packs/'
+        fname = packs_dir + os.listdir(packs_dir)[0]
+        with open(fname, 'rb+') as f:
+            # Start from the end of the file to avoid choosing a place bigger
+            # than the file itself.
+            f.seek(-5, os.SEEK_END)
+            c = f.read(1)
+            f.seek(-5, os.SEEK_END)
+            # Make sure we inject a value different than the one we just read
+            if c == '\xFF':
+                corrupt = '\x00'
+            else:
+                corrupt = '\xFF'
+            f.write(corrupt) # make sure we corrupt something
+        self.run_bzr_error(['Corruption while decompressing repository file'], 
+                            'branch a b', retcode=3)
+
     def test_branch_switch_no_branch(self):
         # No branch in the current directory:
         #  => new branch will be created, but switch fails
@@ -134,7 +177,7 @@
 
         def make_shared_tree(path):
             shared_repo.bzrdir.root_transport.mkdir(path)
-            shared_repo.bzrdir.create_branch_convenience('repo/' + path)
+            controldir.ControlDir.create_branch_convenience('repo/' + path)
             return WorkingTree.open('repo/' + path)
         tree_a = make_shared_tree('a')
         self.build_tree(['repo/a/file'])
@@ -180,7 +223,7 @@
         source.add('file1')
         source.commit('added file')
         out, err = self.run_bzr('branch source target --files-from source')
-        self.failUnlessExists('target/file1')
+        self.assertPathExists('target/file1')
 
     def test_branch_files_from_hardlink(self):
         self.requireFeature(HardlinkFeature)
@@ -209,8 +252,8 @@
     def test_branch_no_tree(self):
         self.example_branch('source')
         self.run_bzr('branch --no-tree source target')
-        self.failIfExists('target/hello')
-        self.failIfExists('target/goodbye')
+        self.assertPathDoesNotExist('target/hello')
+        self.assertPathDoesNotExist('target/goodbye')
 
     def test_branch_into_existing_dir(self):
         self.example_branch('a')
@@ -226,8 +269,8 @@
         # force operation
         self.run_bzr('branch a b --use-existing-dir')
         # check conflicts
-        self.failUnlessExists('b/hello.moved')
-        self.failIfExists('b/godbye.moved')
+        self.assertPathExists('b/hello.moved')
+        self.assertPathDoesNotExist('b/godbye.moved')
         # we can't branch into branch
         out,err = self.run_bzr('branch a b --use-existing-dir', retcode=3)
         self.assertEqual('', out)
@@ -270,6 +313,18 @@
         self.run_bzr('checkout --lightweight a b')
         self.assertLength(2, calls)
 
+    def test_branch_fetches_all_tags(self):
+        builder = self.make_branch_builder('source')
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        source.tags.set_tag('tag-a', 'rev-2')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        # Now source has a tag not in its ancestry.  Make a branch from it.
+        self.run_bzr('branch source new-branch')
+        new_branch = branch.Branch.open('new-branch')
+        # The tag is present, and so is its revision.
+        self.assertEqual('rev-2', new_branch.tags.lookup_tag('tag-a'))
+        new_branch.repository.get_revision('rev-2')
+
 
 class TestBranchStacked(TestCaseWithTransport):
     """Tests for branch --stacked"""
@@ -307,7 +362,7 @@
         # mainline.
         out, err = self.run_bzr(['branch', 'branch', 'newbranch'])
         self.assertEqual('', out)
-        self.assertEqual('Branched 2 revision(s).\n',
+        self.assertEqual('Branched 2 revisions.\n',
             err)
         # it should have preserved the branch format, and so it should be
         # capable of supporting stacking, but not actually have a stacked_on
@@ -414,7 +469,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(38, self.hpss_calls)
+        self.assertLength(39, self.hpss_calls)
 
     def test_branch_from_trivial_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -451,6 +506,42 @@
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(15, self.hpss_calls)
 
+    def test_branch_from_branch_with_tags(self):
+        self.setup_smart_server_with_call_log()
+        builder = self.make_branch_builder('source')
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.tags.set_tag('tag-a', 'rev-2')
+        source.tags.set_tag('tag-missing', 'missing-rev')
+        # Now source has a tag not in its ancestry.  Make a branch from it.
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['branch', self.get_url('source'), 'target'])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(10, self.hpss_calls)
+
+    def test_branch_to_stacked_from_trivial_branch_streaming_acceptance(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('from')
+        for count in range(9):
+            t.commit(message='commit %d' % count)
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['branch', '--stacked', self.get_url('from'),
+            'local-target'])
+        # XXX: the number of hpss calls for this case isn't deterministic yet,
+        # so we can't easily assert about the number of calls.
+        #self.assertLength(XXX, self.hpss_calls)
+        # We can assert that none of the calls were readv requests for rix
+        # files, though (demonstrating that at least get_parent_map calls are
+        # not using VFS RPCs).
+        readvs_of_rix_files = [
+            c for c in self.hpss_calls
+            if c.call.method == 'readv' and c.call.args[-1].endswith('.rix')]
+        self.assertLength(0, readvs_of_rix_files)
+
 
 class TestRemoteBranch(TestCaseWithSFTPServer):
 
@@ -475,3 +566,46 @@
         # Ensure that no working tree what created remotely
         self.assertFalse(t.has('remote/file'))
 
+
+class TestDeprecatedAliases(TestCaseWithTransport):
+
+    def test_deprecated_aliases(self):
+        """bzr branch can be called clone or get, but those names are deprecated.
+
+        See bug 506265.
+        """
+        for command in ['clone', 'get']:
+            run_script(self, """
+            $ bzr %(command)s A B
+            2>The command 'bzr %(command)s' has been deprecated in bzr 2.4. Please use 'bzr branch' instead.
+            2>bzr: ERROR: Not a branch...
+            """ % locals())
+
+
+class TestBranchParentLocation(test_switch.TestSwitchParentLocationBase):
+
+    def _checkout_and_branch(self, option=''):
+        self.script_runner.run_script(self, '''
+                $ bzr checkout %(option)s repo/trunk checkout
+                $ cd checkout
+                $ bzr branch --switch ../repo/trunk ../repo/branched
+                2>Branched 0 revisions.
+                2>Tree is up to date at revision 0.
+                2>Switched to branch:...branched...
+                $ cd ..
+                ''' % locals())
+        bound_branch = branch.Branch.open_containing('checkout')[0]
+        master_branch = branch.Branch.open_containing('repo/branched')[0]
+        return (bound_branch, master_branch)
+
+    def test_branch_switch_parent_lightweight(self):
+        """Lightweight checkout using bzr branch --switch."""
+        bb, mb = self._checkout_and_branch(option='--lightweight')
+        self.assertParent('repo/trunk', bb)
+        self.assertParent('repo/trunk', mb)
+
+    def test_branch_switch_parent_heavyweight(self):
+        """Heavyweight checkout using bzr branch --switch."""
+        bb, mb = self._checkout_and_branch()
+        self.assertParent('repo/trunk', bb)
+        self.assertParent('repo/trunk', mb)

=== added file 'bzrlib/tests/blackbox/test_branches.py'
--- bzrlib/tests/blackbox/test_branches.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_branches.py	2011-08-30 09:03:33 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for bzr branches."""
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestBranches(TestCaseWithTransport):
+
+    def test_no_colocated_support(self):
+        # Listing the branches in a control directory without colocated branch
+        # support.
+        self.run_bzr('init a')
+        out, err = self.run_bzr('branches a')
+        self.assertEquals(out, " (default)\n")
+
+    def test_no_branch(self):
+        # Listing the branches in a control directory without branches.
+        self.run_bzr('init-repo a')
+        out, err = self.run_bzr('branches a')
+        self.assertEquals(out, "")
+
+    def test_default_current_dir(self):
+        # "bzr branches" list the branches in the current directory
+        # if no location was specified.
+        self.run_bzr('init-repo a')
+        out, err = self.run_bzr('branches', working_dir='a')
+        self.assertEquals(out, "")
+
+    def test_recursive_current(self):
+        self.run_bzr('init .')
+        self.assertEquals(".\n", self.run_bzr('branches --recursive')[0])
+
+    def test_recursive(self):
+        self.run_bzr('init source')
+        self.run_bzr('init source/subsource')
+        self.run_bzr('checkout --lightweight source checkout')
+        self.run_bzr('init checkout/subcheckout')
+        self.run_bzr('init checkout/.bzr/subcheckout')
+        out = self.run_bzr('branches --recursive')[0]
+        lines = out.split('\n')
+        self.assertIs(True, 'source' in lines, lines)
+        self.assertIs(True, 'source/subsource' in lines, lines)
+        self.assertIs(True, 'checkout/subcheckout' in lines, lines)
+        self.assertIs(True, 'checkout' not in lines, lines)

=== modified file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	2010-07-21 13:32:27 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2011-03-10 13:29:54 +0000
@@ -16,16 +16,17 @@
 
 """Tests for lock-breaking user interface"""
 
-import os
-
-import bzrlib
 from bzrlib import (
     branch,
     bzrdir,
+    config,
     errors,
-    lockdir,
+    osutils,
     tests,
     )
+from bzrlib.tests.script import (
+    run_script,
+    )
 
 
 class TestBreakLock(tests.TestCaseWithTransport):
@@ -68,12 +69,22 @@
         # shouldn't fail and should not produce error output
         self.assertEqual('', err)
 
+    def test_break_lock_no_interaction(self):
+        """With --force, the user isn't asked for confirmation"""
+        self.master_branch.lock_write()
+        run_script(self, """
+        $ bzr break-lock --force master-repo/master-branch
+        Broke lock ...master-branch/.bzr/...
+        """)
+        # lock should now be dead
+        self.assertRaises(errors.LockBroken, self.master_branch.unlock)
+
     def test_break_lock_everything_locked(self):
         ### if everything is locked, we should be able to unlock the lot.
         # however, we dont test breaking the working tree because we
         # cannot accurately do so right now: the dirstate lock is held
         # by an os lock, and we need to spawn a separate process to lock it
-        # thne kill -9 it.
+        # then kill -9 it.
         # sketch of test:
         # lock most of the dir:
         self.wt.branch.lock_write()
@@ -93,11 +104,23 @@
         self.assertRaises(errors.LockBroken, self.master_branch.unlock)
 
 
-class TestBreakLockOldBranch(tests.TestCaseWithTransport):
-
-    def test_break_lock_format_5_bzrdir(self):
-        # break lock on a format 5 bzrdir should just return
-        self.make_branch_and_tree('foo', format=bzrdir.BzrDirFormat5())
-        out, err = self.run_bzr('break-lock foo')
-        self.assertEqual('', out)
-        self.assertEqual('', err)
+class TestConfigBreakLock(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigBreakLock, self).setUp()
+        self.config_file_name = './my.conf'
+        self.build_tree_contents([(self.config_file_name,
+                                   '[DEFAULT]\none=1\n')])
+        self.config = config.LockableConfig(file_name=self.config_file_name)
+        self.config.lock_write()
+
+    def test_create_pending_lock(self):
+        self.addCleanup(self.config.unlock)
+        self.assertTrue(self.config._lock.is_held)
+
+    def test_break_lock(self):
+        self.run_bzr('break-lock --config %s'
+                     % osutils.dirname(self.config_file_name),
+                     stdin="y\n")
+        self.assertRaises(errors.LockBroken, self.config.unlock)
+

=== removed file 'bzrlib/tests/blackbox/test_breakin.py'
--- bzrlib/tests/blackbox/test_breakin.py	2009-09-28 03:36:45 +0000
+++ bzrlib/tests/blackbox/test_breakin.py	1970-01-01 00:00:00 +0000
@@ -1,204 +0,0 @@
-# Copyright (C) 2006, 2007, 2009 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Blackbox tests for debugger breakin"""
-
-try:
-    import ctypes
-    have_ctypes = True
-except ImportError:
-    have_ctypes = False
-import errno
-import os
-import signal
-import subprocess
-import sys
-import time
-
-from bzrlib import (
-    breakin,
-    errors,
-    tests,
-    )
-
-
-class TestBreakin(tests.TestCase):
-    # FIXME: If something is broken, these tests may just hang indefinitely in
-    # wait() waiting for the child to exit when it's not going to.
-
-    def setUp(self):
-        super(TestBreakin, self).setUp()
-        self.requireFeature(tests.BreakinFeature)
-        if sys.platform == 'win32':
-            self._send_signal = self._send_signal_win32
-        else:
-            self._send_signal = self._send_signal_via_kill
-
-    def _send_signal_via_kill(self, pid, sig_type):
-        if sig_type == 'break':
-            sig_num = signal.SIGQUIT
-        elif sig_type == 'kill':
-            sig_num = signal.SIGKILL
-        else:
-            raise ValueError("unknown signal type: %s" % (sig_type,))
-        try:
-            os.kill(pid, sig_num)
-        except OSError, e:
-            if e.errno != errno.ESRCH:
-                raise
-
-    def _send_signal_win32(self, pid, sig_type):
-        """Send a 'signal' on Windows.
-
-        Windows doesn't really have signals in the same way. All it really
-        supports is:
-            1) Sending SIGINT to the *current* process group (so self, and all
-                children of self)
-            2) Sending SIGBREAK to a process that shares the current console,
-                which can be in its own process group.
-        So we have start_bzr_subprocess create a new process group for the
-        spawned process (via a flag to Popen), and then we map
-            SIGQUIT to GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT)
-            SIGKILL to TerminateProcess
-        """
-        if sig_type == 'break':
-            CTRL_BREAK_EVENT = 1
-            # CTRL_C_EVENT = 0
-            ret = ctypes.windll.kernel32.GenerateConsoleCtrlEvent(
-                    CTRL_BREAK_EVENT, pid)
-            if ret == 0: #error
-                err = ctypes.FormatError()
-                raise RuntimeError('failed to send CTRL_BREAK: %s'
-                                   % (err,))
-        elif sig_type == 'kill':
-            # Does the exit code matter? For now we are just setting it to
-            # something other than 0
-            exit_code = breakin.determine_signal()
-            ctypes.windll.kernel32.TerminateProcess(pid, exit_code)
-
-    def _popen(self, *args, **kwargs):
-        if sys.platform == 'win32':
-            CREATE_NEW_PROCESS_GROUP = 512
-            # This allows us to send a signal to the child, *without* also
-            # sending it to ourselves
-            kwargs['creationflags'] = CREATE_NEW_PROCESS_GROUP
-        return super(TestBreakin, self)._popen(*args, **kwargs)
-
-    def _dont_SIGQUIT_on_darwin(self):
-        if sys.platform == 'darwin':
-            # At least on Leopard and with python 2.6, this test will raise a
-            # popup window asking if the python failure should be reported to
-            # Apple... That's not the point of the test :) Marking the test as
-            # not applicable Until we find a way to disable that intrusive
-            # behavior... --vila20080611
-            raise tests.TestNotApplicable(
-                '%s raises a popup on OSX' % self.id())
-
-    def _wait_for_process(self, pid, sig=None, count=100):
-        # We don't know quite how long waiting for the process 'pid' will take,
-        # but if it's more than 10s then it's probably not going to work.
-        for i in range(count):
-            if sig is not None:
-                self._send_signal(pid, sig)
-            # Use WNOHANG to ensure we don't get blocked, doing so, we may
-            # leave the process continue after *we* die...
-            # Win32 doesn't support WNOHANG, so we just pass 0
-            opts = getattr(os, 'WNOHANG', 0)
-            try:
-                # TODO: waitpid doesn't work well on windows, we might consider
-                #       using WaitForSingleObject(proc._handle, TIMEOUT)
-                #       instead. Most notably, the WNOHANG isn't allowed, so
-                #       this can hang indefinitely.
-                pid_killed, returncode = os.waitpid(pid, opts)
-                if pid_killed != 0 and returncode != 0:
-                    if sig is not None:
-                        # high bit in low byte says if core was dumped; we
-                        # don't care
-                        status, sig = (returncode >> 8, returncode & 0x7f)
-                        return True, sig
-            except OSError, e:
-                if e.errno in (errno.ECHILD, errno.ESRCH):
-                    # The process doesn't exist anymore
-                    return True, None
-                else:
-                    raise
-            if i + 1 != count:
-                time.sleep(0.1)
-
-        return False, None
-
-    # port 0 means to allocate any port
-    _test_process_args = ['serve', '--port', 'localhost:0']
-
-    def test_breakin(self):
-        # Break in to a debugger while bzr is running
-        # we need to test against a command that will wait for
-        # a while -- bzr serve should do
-        proc = self.start_bzr_subprocess(self._test_process_args,
-                env_changes=dict(BZR_SIGQUIT_PDB=None))
-        # wait for it to get started, and print the 'listening' line
-        proc.stderr.readline()
-        # first sigquit pops into debugger
-        self._send_signal(proc.pid, 'break')
-        # Wait for the debugger to acknowledge the signal reception
-        # Note that it is possible for this to deadlock if the child doesn't
-        # acknowlege the signal and write to stderr. Perhaps we should try
-        # os.read(proc.stderr.fileno())?
-        err = proc.stderr.readline()
-        self.assertContainsRe(err, r'entering debugger')
-        # Try to shutdown cleanly;
-        # Now that the debugger is entered, we can ask him to quit
-        proc.stdin.write("q\n")
-        # But we don't really care if it doesn't.
-        dead, sig = self._wait_for_process(proc.pid, count=3)
-        if not dead:
-            # The process didn't finish, let's kill it.
-            dead, sig = self._wait_for_process(proc.pid, 'kill', count=10)
-            if not dead:
-                # process isn't gone, user will have to hunt it down and kill
-                # it.
-                self.fail("subprocess %d wasn't terminated by repeated SIGKILL" %
-                          proc.pid)
-
-    def test_breakin_harder(self):
-        """SIGQUITting twice ends the process."""
-        self._dont_SIGQUIT_on_darwin()
-        proc = self.start_bzr_subprocess(self._test_process_args,
-                env_changes=dict(BZR_SIGQUIT_PDB=None))
-        # wait for it to get started, and print the 'listening' line
-        proc.stderr.readline()
-        # break into the debugger
-        self._send_signal(proc.pid, 'break')
-        # Wait for the debugger to acknowledge the signal reception (since we
-        # want to send a second signal, we ensure it doesn't get lost by
-        # validating the first get received and produce its effect).
-        err = proc.stderr.readline()
-        self.assertContainsRe(err, r'entering debugger')
-        dead, sig = self._wait_for_process(proc.pid, 'break')
-        self.assertTrue(dead)
-        # Either the child was dead before we could read its status, or the
-        # child was dead from the signal we sent it.
-        self.assertTrue(sig in (None, breakin.determine_signal()))
-
-    def test_breakin_disabled(self):
-        self._dont_SIGQUIT_on_darwin()
-        proc = self.start_bzr_subprocess(self._test_process_args,
-                env_changes=dict(BZR_SIGQUIT_PDB='0'))
-        # wait for it to get started, and print the 'listening' line
-        proc.stderr.readline()
-        # first hit should just kill it
-        self._send_signal(proc.pid, 'break')
-        proc.wait()

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2010-05-02 18:16:37 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2011-06-30 17:42:04 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
-# -*- coding: utf-8 -*-
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,28 +37,32 @@
         tree.commit(message='1')
         self.build_tree_contents([('a', 'baz\n')])
 
-        # We use run_bzr_subprocess rather than run_bzr here so that we can
-        # test mangling of line-endings on Windows.
+        self.assertEquals(self.run_bzr(['cat', 'a'])[0], 'foo\n')
+
+        # On Windows, we used to have a bug where newlines got changed into
+        # crlf, whereas cat ought to write out the file exactly as it's
+        # recorded (by default.)  That problem can't be reproduced in-process,
+        # so we need just one test here that 
         self.assertEquals(self.run_bzr_subprocess(['cat', 'a'])[0], 'foo\n')
 
         tree.commit(message='2')
-        self.assertEquals(self.run_bzr_subprocess(['cat', 'a'])[0], 'baz\n')
-        self.assertEquals(self.run_bzr_subprocess(
+        self.assertEquals(self.run_bzr(['cat', 'a'])[0], 'baz\n')
+        self.assertEquals(self.run_bzr(
             ['cat', 'a', '-r', '1'])[0],
             'foo\n')
-        self.assertEquals(self.run_bzr_subprocess(
+        self.assertEquals(self.run_bzr(
             ['cat', 'a', '-r', '-1'])[0],
             'baz\n')
 
         rev_id = tree.branch.last_revision()
 
-        self.assertEquals(self.run_bzr_subprocess(
+        self.assertEquals(self.run_bzr(
             ['cat', 'a', '-r', 'revid:%s' % rev_id])[0],
             'baz\n')
 
         os.chdir('..')
 
-        self.assertEquals(self.run_bzr_subprocess(
+        self.assertEquals(self.run_bzr(
             ['cat', 'branch/a', '-r', 'revno:1:branch'])[0],
             'foo\n')
         self.run_bzr(['cat', 'a'], retcode=3)
@@ -100,21 +103,21 @@
                            'cat b-tree --name-from-revision')
 
         # get to the old file automatically
-        out, err = self.run_bzr_subprocess('cat d-rev')
+        out, err = self.run_bzr('cat d-rev')
         self.assertEqual('bar\n', out)
         self.assertEqual('', err)
 
         out, err = \
-                self.run_bzr_subprocess('cat a-rev-tree --name-from-revision')
+                self.run_bzr('cat a-rev-tree --name-from-revision')
         self.assertEqual('foo\n', out)
         self.assertEqual('', err)
 
-        out, err = self.run_bzr_subprocess('cat a-rev-tree')
+        out, err = self.run_bzr('cat a-rev-tree')
         self.assertEqual('baz\n', out)
         self.assertEqual('', err)
 
         # the actual file-id for e-rev doesn't exist in the old tree
-        out, err = self.run_bzr_subprocess('cat e-rev -rrevid:first')
+        out, err = self.run_bzr('cat e-rev -rrevid:first')
         self.assertEqual('qux\n', out)
         self.assertEqual('', err)
 
@@ -125,7 +128,7 @@
         wt.commit('Making sure there is a basis_tree available')
 
         url = self.get_readonly_url() + '/README'
-        out, err = self.run_bzr_subprocess(['cat', url])
+        out, err = self.run_bzr(['cat', url])
         self.assertEqual('contents of README\n', out)
 
     def test_cat_branch_revspec(self):
@@ -136,7 +139,7 @@
         wt = self.make_branch_and_tree('b')
         os.chdir('b')
 
-        out, err = self.run_bzr_subprocess(
+        out, err = self.run_bzr(
             ['cat', '-r', 'branch:../a', 'README'])
         self.assertEqual('contents of a/README\n', out)
 
@@ -148,11 +151,11 @@
         url = self.get_readonly_url() + '/README'
 
         # Test unfiltered output
-        out, err = self.run_bzr_subprocess(['cat', url])
+        out, err = self.run_bzr(['cat', url])
         self.assertEqual('contents of README\n', out)
 
         # Test --filters option is legal but has no impact if no filters
-        out, err = self.run_bzr_subprocess(['cat', '--filters', url])
+        out, err = self.run_bzr(['cat', '--filters', url])
         self.assertEqual('contents of README\n', out)
 
     def test_cat_filters_applied(self):
@@ -192,7 +195,7 @@
         wt.branch.bzrdir.destroy_workingtree()
 
         url = self.get_readonly_url() + '/README'
-        out, err = self.run_bzr_subprocess(['cat', url])
+        out, err = self.run_bzr(['cat', url])
         self.assertEqual('contents of README\n', out)
 
     def test_cat_nonexistent_branch(self):
@@ -206,7 +209,7 @@
         wt.add('README')
         wt.commit('Making sure there is a basis_tree available')
 
-        out, err = self.run_bzr_subprocess(['cat', '--directory=a', 'README'])
+        out, err = self.run_bzr(['cat', '--directory=a', 'README'])
         self.assertEqual('contents of a/README\n', out)
 
     def test_cat_remote_directory(self):
@@ -216,5 +219,5 @@
         wt.commit('Making sure there is a basis_tree available')
 
         url = self.get_readonly_url() + '/a'
-        out, err = self.run_bzr_subprocess(['cat', '-d', url, 'README'])
+        out, err = self.run_bzr(['cat', '-d', url, 'README'])
         self.assertEqual('contents of a/README\n', out)

=== modified file 'bzrlib/tests/blackbox/test_cat_revision.py'
--- bzrlib/tests/blackbox/test_cat_revision.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_cat_revision.py	2011-01-18 23:54:51 +0000
@@ -68,3 +68,11 @@
         output, errors = self.run_bzr(['cat-revision', '-d', 'a', u'abcd'])
         self.assertContainsRe(output, 'This revision')
         self.assertEqual('', errors)
+
+    def test_cat_tree_less_branch(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('This revision', rev_id='abcd')
+        tree.bzrdir.destroy_workingtree()
+        output, errors = self.run_bzr(['cat-revision', '-d', 'a', u'abcd'])
+        self.assertContainsRe(output, 'This revision')
+        self.assertEqual('', errors)

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2010-07-20 10:18:05 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2011-06-14 01:26:41 +0000
@@ -32,8 +32,10 @@
     TestCaseWithTransport,
     )
 from bzrlib.tests import (
+    KnownFailure,
+    )
+from bzrlib.tests.features import (
     HardlinkFeature,
-    KnownFailure,
     )
 
 
@@ -69,7 +71,7 @@
         # from 1.
         result = bzrdir.BzrDir.open('checkout')
         self.assertEqual(['1'], result.open_workingtree().get_parent_ids())
-        self.failIfExists('checkout/added_in_2')
+        self.assertPathDoesNotExist('checkout/added_in_2')
 
     def test_checkout_light_dash_r(self):
         out, err = self.run_bzr(['checkout','--lightweight', '-r', '-2',
@@ -78,7 +80,7 @@
         # from 1.
         result = bzrdir.BzrDir.open('checkout')
         self.assertEqual(['1'], result.open_workingtree().get_parent_ids())
-        self.failIfExists('checkout/added_in_2')
+        self.assertPathDoesNotExist('checkout/added_in_2')
 
     def test_checkout_reconstitutes_working_trees(self):
         # doing a 'bzr checkout' in the directory of a branch with no tree
@@ -124,9 +126,9 @@
             cmd.append('--lightweight')
         self.run_bzr('checkout source target')
         # files with unique content should be moved
-        self.failUnlessExists('target/file2.moved')
+        self.assertPathExists('target/file2.moved')
         # files with content matching tree should not be moved
-        self.failIfExists('target/file1.moved')
+        self.assertPathDoesNotExist('target/file1.moved')
 
     def test_checkout_existing_dir_heavy(self):
         self._test_checkout_existing_dir(False)

=== modified file 'bzrlib/tests/blackbox/test_clean_tree.py'
--- bzrlib/tests/blackbox/test_clean_tree.py	2010-05-20 02:57:52 +0000
+++ bzrlib/tests/blackbox/test_clean_tree.py	2011-05-13 12:51:05 +0000
@@ -41,28 +41,28 @@
         self.run_bzr('ignore *.pyc')
         self.touch('name')
         self.touch('name~')
-        self.failUnlessExists('name~')
+        self.assertPathExists('name~')
         self.touch('name.pyc')
         self.run_bzr('clean-tree --force')
-        self.failUnlessExists('name~')
-        self.failIfExists('name')
+        self.assertPathExists('name~')
+        self.assertPathDoesNotExist('name')
         self.touch('name')
         self.run_bzr('clean-tree --detritus --force')
-        self.failUnlessExists('name')
-        self.failIfExists('name~')
-        self.failUnlessExists('name.pyc')
+        self.assertPathExists('name')
+        self.assertPathDoesNotExist('name~')
+        self.assertPathExists('name.pyc')
         self.run_bzr('clean-tree --ignored --force')
-        self.failUnlessExists('name')
-        self.failIfExists('name.pyc')
+        self.assertPathExists('name')
+        self.assertPathDoesNotExist('name.pyc')
         self.run_bzr('clean-tree --unknown --force')
-        self.failIfExists('name')
+        self.assertPathDoesNotExist('name')
         self.touch('name')
         self.touch('name~')
         self.touch('name.pyc')
         self.run_bzr('clean-tree --unknown --ignored --force')
-        self.failIfExists('name')
-        self.failIfExists('name~')
-        self.failIfExists('name.pyc')
+        self.assertPathDoesNotExist('name')
+        self.assertPathDoesNotExist('name~')
+        self.assertPathDoesNotExist('name.pyc')
 
     def test_clean_tree_nested_bzrdir(self):
         # clean-tree should not blindly delete nested bzrdirs (branches)
@@ -73,11 +73,11 @@
         wt3 = self.make_branch_and_tree('bar')
         ignores.tree_ignores_add_patterns(wt1, ['./foo'])
         self.run_bzr(['clean-tree', '--unknown', '--force'])
-        self.failUnlessExists('foo')
-        self.failUnlessExists('bar')
+        self.assertPathExists('foo')
+        self.assertPathExists('bar')
         self.run_bzr(['clean-tree', '--ignored', '--force'])
-        self.failUnlessExists('foo')
-        self.failUnlessExists('bar')
+        self.assertPathExists('foo')
+        self.assertPathExists('bar')
 
     def test_clean_tree_directory(self):
         """Test --directory option"""
@@ -85,6 +85,6 @@
         self.build_tree(['a/added', 'a/unknown', 'a/ignored'])
         tree.add('added')
         self.run_bzr('clean-tree -d a --unknown --ignored --force')
-        self.failIfExists('a/unknown')
-        self.failIfExists('a/ignored')
-        self.failUnlessExists('a/added')
+        self.assertPathDoesNotExist('a/unknown')
+        self.assertPathDoesNotExist('a/ignored')
+        self.assertPathExists('a/added')

=== modified file 'bzrlib/tests/blackbox/test_command_encoding.py'
--- bzrlib/tests/blackbox/test_command_encoding.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_command_encoding.py	2011-04-15 21:13:48 +0000
@@ -54,13 +54,12 @@
         register_command(cmd_echo_exact)
         try:
             self.assertEqual('foo', bzr('echo-exact foo'))
-            # This is cheating a little bit, because 'foo\xb5' shouldn't
-            # get past main()
-            self.assertEqual('foo\xb5', bzr('echo-exact foo\xb5'))
             # Exact should fail to decode the string
             self.assertRaises(UnicodeEncodeError,
                 bzr,
                 ['echo-exact', u'foo\xb5'])
+            # Previously a non-ascii bytestring was also tested, as 'exact'
+            # outputs bytes untouched, but needed buggy argv parsing to work
         finally:
             plugin_cmds.remove('echo-exact')
 

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2010-07-14 08:53:58 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2011-09-02 07:38:52 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,22 +17,23 @@
 
 """Tests for the commit CLI of bzr."""
 
+import doctest
 import os
 import re
 import sys
 
+from testtools.matchers import DocTestMatches
+
 from bzrlib import (
-    bzrdir,
+    config,
     osutils,
     ignores,
     msgeditor,
-    osutils,
-    tests,
     )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.tests import (
-    probe_bad_non_ascii,
-    TestSkipped,
+    test_foreign,
+    features,
     )
 from bzrlib.tests import TestCaseWithTransport
 
@@ -46,8 +47,17 @@
         self.build_tree(['hello.txt'])
         out,err = self.run_bzr('commit -m empty', retcode=3)
         self.assertEqual('', out)
-        self.assertContainsRe(err, 'bzr: ERROR: No changes to commit\.'
-                                  ' Use --unchanged to commit anyhow.\n')
+        # Two ugly bits here.
+        # 1) We really don't want 'aborting commit write group' anymore.
+        # 2) bzr: ERROR: is a really long line, so we wrap it with '\'
+        self.assertThat(
+            err,
+            DocTestMatches("""\
+Committing to: ...
+bzr: ERROR: No changes to commit.\
+ Please 'bzr add' the files you want to commit,\
+ or use --unchanged to force an empty commit.
+""", flags=doctest.ELLIPSIS|doctest.REPORT_UDIFF))
 
     def test_commit_success(self):
         """Successful commit should not leave behind a bzr-commit-* file"""
@@ -59,6 +69,20 @@
         self.run_bzr(["commit", "--unchanged", "-m", u'foo\xb5'])
         self.assertEqual('', self.run_bzr('unknowns')[0])
 
+    def test_commit_lossy_native(self):
+        """A --lossy option to commit is supported."""
+        self.make_branch_and_tree('.')
+        self.run_bzr('commit --lossy --unchanged -m message')
+        self.assertEqual('', self.run_bzr('unknowns')[0])
+
+    def test_commit_lossy_foreign(self):
+        test_foreign.register_dummy_foreign_for_test(self)
+        self.make_branch_and_tree('.',
+            format=test_foreign.DummyForeignVcsDirFormat())
+        self.run_bzr('commit --lossy --unchanged -m message')
+        output = self.run_bzr('revision-info')[0]
+        self.assertTrue(output.startswith('1 dummy-'))
+
     def test_commit_with_path(self):
         """Commit tree with path of root specified"""
         a_tree = self.make_branch_and_tree('a')
@@ -78,7 +102,6 @@
         self.run_bzr('resolved b/a_file')
         self.run_bzr(['commit', '-m', 'merge into b', 'b'])
 
-
     def test_10_verbose_commit(self):
         """Add one file and examine verbose commit output"""
         tree = self.make_branch_and_tree('.')
@@ -112,6 +135,7 @@
     def test_unicode_commit_message_is_filename(self):
         """Unicode commit message same as a filename (Bug #563646).
         """
+        self.requireFeature(features.UnicodeFilenameFeature)
         file_name = u'\N{euro sign}'
         self.run_bzr(['init'])
         open(file_name, 'w').write('hello world')
@@ -305,27 +329,7 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents([('foo.c', 'int main() {}')])
         tree.add('foo.c')
-        self.run_bzr('commit -m ""', retcode=3)
-
-    def test_unsupported_encoding_commit_message(self):
-        if sys.platform == 'win32':
-            raise tests.TestNotApplicable('Win32 parses arguments directly'
-                ' as Unicode, so we can\'t pass invalid non-ascii')
-        tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([('foo.c', 'int main() {}')])
-        tree.add('foo.c')
-        # LANG env variable has no effect on Windows
-        # but some characters anyway cannot be represented
-        # in default user encoding
-        char = probe_bad_non_ascii(osutils.get_user_encoding())
-        if char is None:
-            raise TestSkipped('Cannot find suitable non-ascii character'
-                'for user_encoding (%s)' % osutils.get_user_encoding())
-        out,err = self.run_bzr_subprocess('commit -m "%s"' % char,
-                                          retcode=1,
-                                          env_changes={'LANG': 'C'})
-        self.assertContainsRe(err, r'bzrlib.errors.BzrError: Parameter.*is '
-                                    'unsupported by the current encoding.')
+        self.run_bzr('commit -m ""')
 
     def test_other_branch_commit(self):
         # this branch is to ensure consistent behaviour, whether we're run
@@ -592,6 +596,25 @@
             'commit -m add-b --fixes=xxx:123',
             working_dir='tree')
 
+    def test_fixes_bug_with_default_tracker(self):
+        """commit --fixes=234 uses the default bug tracker."""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/hello.txt'])
+        tree.add('hello.txt')
+        self.run_bzr_error(
+            ["bzr: ERROR: No tracker specified for bug 123. Use the form "
+            "'tracker:id' or specify a default bug tracker using the "
+            "`bugtracker` option.\n"
+            "See \"bzr help bugs\" for more information on this feature. "
+            "Commit refused."],
+            'commit -m add-b --fixes=123',
+            working_dir='tree')
+        tree.branch.get_config().set_user_option("bugtracker", "lp")
+        self.run_bzr('commit -m hello --fixes=234 tree/hello.txt')
+        last_rev = tree.branch.repository.get_revision(tree.last_revision())
+        self.assertEqual('https://launchpad.net/bugs/234 fixed',
+                         last_rev.properties['bugs'])
+
     def test_fixes_invalid_bug_number(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/hello.txt'])
@@ -609,10 +632,10 @@
         self.build_tree(['tree/hello.txt'])
         tree.add('hello.txt')
         self.run_bzr_error(
-            [r"Invalid bug orange. Must be in the form of 'tracker:id'\. "
-             r"See \"bzr help bugs\" for more information on this feature.\n"
-             r"Commit refused\."],
-            'commit -m add-b --fixes=orange',
+            [r"Invalid bug orange:apples:bananas. Must be in the form of "
+             r"'tracker:id'\. See \"bzr help bugs\" for more information on "
+             r"this feature.\nCommit refused\."],
+            'commit -m add-b --fixes=orange:apples:bananas',
             working_dir='tree')
 
     def test_no_author(self):
@@ -717,13 +740,13 @@
             f = file('fed.bat', 'w')
             f.write('@rem dummy fed')
             f.close()
-            osutils.set_or_unset_env('BZR_EDITOR', "fed.bat")
+            self.overrideEnv('BZR_EDITOR', "fed.bat")
         else:
             f = file('fed.sh', 'wb')
             f.write('#!/bin/sh\n')
             f.close()
             os.chmod('fed.sh', 0755)
-            osutils.set_or_unset_env('BZR_EDITOR', "./fed.sh")
+            self.overrideEnv('BZR_EDITOR', "./fed.sh")
 
     def setup_commit_with_template(self):
         self.setup_editor()
@@ -734,6 +757,16 @@
         tree.add('hello.txt')
         return tree
 
+    def test_edit_empty_message(self):
+        tree = self.make_branch_and_tree('tree')
+        self.setup_editor()
+        self.build_tree(['tree/hello.txt'])
+        tree.add('hello.txt')
+        out, err = self.run_bzr("commit tree/hello.txt", retcode=3,
+            stdin="y\n")
+        self.assertContainsRe(err,
+            "bzr: ERROR: Empty commit message specified")
+
     def test_commit_hook_template_accepted(self):
         tree = self.setup_commit_with_template()
         out, err = self.run_bzr("commit tree/hello.txt", stdin="y\n")
@@ -743,10 +776,23 @@
     def test_commit_hook_template_rejected(self):
         tree = self.setup_commit_with_template()
         expected = tree.last_revision()
-        out, err = self.run_bzr_error(["empty commit message"],
+        out, err = self.run_bzr_error(["Empty commit message specified."
+                  " Please specify a commit message with either"
+                  " --message or --file or leave a blank message"
+                  " with --message \"\"."],
             "commit tree/hello.txt", stdin="n\n")
         self.assertEqual(expected, tree.last_revision())
 
+    def test_set_commit_message(self):
+        msgeditor.hooks.install_named_hook("set_commit_message",
+                lambda commit_obj, msg: "save me some typing\n", None)
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/hello.txt'])
+        tree.add('hello.txt')
+        out, err = self.run_bzr("commit tree/hello.txt")
+        last_rev = tree.branch.repository.get_revision(tree.last_revision())
+        self.assertEqual('save me some typing\n', last_rev.message)
+
     def test_commit_without_username(self):
         """Ensure commit error if username is not set.
         """
@@ -754,8 +800,11 @@
         os.chdir('foo')
         open('foo.txt', 'w').write('hello')
         self.run_bzr(['add'])
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
+        # Also, make sure that it's not inferred from mailname.
+        self.overrideAttr(config, '_auto_user_id',
+            lambda: (None, None))
         out, err = self.run_bzr(['commit', '-m', 'initial'], 3)
         self.assertContainsRe(err, 'Unable to determine your name')
 
@@ -772,4 +821,3 @@
         self.assertEqual(out, '')
         self.assertContainsRe(err,
             'Branch.*test_checkout.*appears to be bound to itself')
-

=== added file 'bzrlib/tests/blackbox/test_config.py'
--- bzrlib/tests/blackbox/test_config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_config.py	2011-02-22 12:59:14 +0000
@@ -0,0 +1,323 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for bzr config."""
+
+import os
+
+from bzrlib import (
+    config,
+    errors,
+    tests,
+    )
+from bzrlib.tests import (
+    script,
+    test_config as _t_config,
+    )
+
+class TestWithoutConfig(tests.TestCaseWithTransport):
+
+    def test_config_all(self):
+        out, err = self.run_bzr(['config'])
+        self.assertEquals('', out)
+        self.assertEquals('', err)
+
+    def test_remove_unknown_option(self):
+        self.run_bzr_error(['The "file" configuration option does not exist',],
+                           ['config', '--remove', 'file'])
+
+    def test_all_remove_exclusive(self):
+        self.run_bzr_error(['--all and --remove are mutually exclusive.',],
+                           ['config', '--remove', '--all'])
+
+    def test_all_set_exclusive(self):
+        self.run_bzr_error(['Only one option can be set.',],
+                           ['config', '--all', 'hello=world'])
+
+    def test_remove_no_option(self):
+        self.run_bzr_error(['--remove expects an option to remove.',],
+                           ['config', '--remove'])
+
+    def test_unknown_option(self):
+        self.run_bzr_error(['The "file" configuration option does not exist',],
+                           ['config', 'file'])
+
+    def test_unexpected_regexp(self):
+        self.run_bzr_error(
+            ['The "\*file" configuration option does not exist',],
+            ['config', '*file'])
+
+    def test_wrong_regexp(self):
+        self.run_bzr_error(
+            ['Invalid pattern\(s\) found. "\*file" nothing to repeat',],
+            ['config', '--all', '*file'])
+
+
+
+class TestConfigDisplay(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigDisplay, self).setUp()
+        _t_config.create_configs(self)
+
+    def test_multiline_all_values(self):
+        self.bazaar_config.set_user_option('multiline', '1\n2\n')
+        # Fallout from bug 710410, the triple quotes have been toggled
+        script.run_script(self, '''\
+            $ bzr config -d tree
+            bazaar:
+              multiline = """1
+            2
+            """
+            ''')
+
+    def test_multiline_value_only(self):
+        self.bazaar_config.set_user_option('multiline', '1\n2\n')
+        # Fallout from bug 710410, the triple quotes have been toggled
+        script.run_script(self, '''\
+            $ bzr config -d tree multiline
+            """1
+            2
+            """
+            ''')
+
+    def test_list_all_values(self):
+        self.bazaar_config.set_user_option('list', [1, 'a', 'with, a comma'])
+        script.run_script(self, '''\
+            $ bzr config -d tree
+            bazaar:
+              list = 1, a, "with, a comma"
+            ''')
+
+    def test_list_value_only(self):
+        self.bazaar_config.set_user_option('list', [1, 'a', 'with, a comma'])
+        script.run_script(self, '''\
+            $ bzr config -d tree list
+            1, a, "with, a comma"
+            ''')
+
+    def test_bazaar_config(self):
+        self.bazaar_config.set_user_option('hello', 'world')
+        script.run_script(self, '''\
+            $ bzr config -d tree
+            bazaar:
+              hello = world
+            ''')
+
+    def test_locations_config_for_branch(self):
+        self.locations_config.set_user_option('hello', 'world')
+        self.branch_config.set_user_option('hello', 'you')
+        script.run_script(self, '''\
+            $ bzr config -d tree
+            locations:
+              [.../tree]
+              hello = world
+            branch:
+              hello = you
+            ''')
+
+    def test_locations_config_outside_branch(self):
+        self.bazaar_config.set_user_option('hello', 'world')
+        self.locations_config.set_user_option('hello', 'world')
+        script.run_script(self, '''\
+            $ bzr config
+            bazaar:
+              hello = world
+            ''')
+
+class TestConfigDisplayWithPolicy(tests.TestCaseWithTransport):
+
+    def test_location_with_policy(self):
+        # LocationConfig is the only one dealing with policies so far.
+        self.make_branch_and_tree('tree')
+        config_text = """\
+[%(dir)s]
+url = dir
+url:policy = appendpath
+[%(dir)s/tree]
+url = tree
+""" % {'dir': self.test_dir}
+        # We don't use the config directly so we save it to disk
+        config.LocationConfig.from_string(config_text, 'tree', save=True)
+        # policies are displayed with their options since they are part of
+        # their definition, likewise the path is not appended, we are just
+        # presenting the relevant portions of the config files
+        script.run_script(self, '''\
+            $ bzr config -d tree --all url
+            locations:
+              [.../work/tree]
+              url = tree
+              [.../work]
+              url = dir
+              url:policy = appendpath
+            ''')
+
+
+class TestConfigActive(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigActive, self).setUp()
+        _t_config.create_configs_with_file_option(self)
+
+    def test_active_in_locations(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree file
+            locations
+            ''')
+
+    def test_active_in_bazaar(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --scope bazaar file
+            bazaar
+            ''')
+
+    def test_active_in_branch(self):
+        # We need to delete the locations definition that overrides the branch
+        # one
+        script.run_script(self, '''\
+            $ bzr config -d tree --remove file
+            $ bzr config -d tree file
+            branch
+            ''')
+
+
+class TestConfigSetOption(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigSetOption, self).setUp()
+        _t_config.create_configs(self)
+
+    def test_unknown_config(self):
+        self.run_bzr_error(['The "moon" configuration does not exist'],
+                           ['config', '--scope', 'moon', 'hello=world'])
+
+    def test_bazaar_config_outside_branch(self):
+        script.run_script(self, '''\
+            $ bzr config --scope bazaar hello=world
+            $ bzr config -d tree --all hello
+            bazaar:
+              hello = world
+            ''')
+
+    def test_bazaar_config_inside_branch(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --scope bazaar hello=world
+            $ bzr config -d tree --all hello
+            bazaar:
+              hello = world
+            ''')
+
+    def test_locations_config_inside_branch(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --scope locations hello=world
+            $ bzr config -d tree --all hello
+            locations:
+              [.../work/tree]
+              hello = world
+            ''')
+
+    def test_branch_config_default(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree hello=world
+            $ bzr config -d tree --all hello
+            branch:
+              hello = world
+            ''')
+
+    def test_branch_config_forcing_branch(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --scope branch hello=world
+            $ bzr config -d tree --all hello
+            branch:
+              hello = world
+            ''')
+
+
+class TestConfigRemoveOption(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigRemoveOption, self).setUp()
+        _t_config.create_configs_with_file_option(self)
+
+    def test_unknown_config(self):
+        self.run_bzr_error(['The "moon" configuration does not exist'],
+                           ['config', '--scope', 'moon', '--remove', 'file'])
+
+    def test_bazaar_config_outside_branch(self):
+        script.run_script(self, '''\
+            $ bzr config --scope bazaar --remove file
+            $ bzr config -d tree --all file
+            locations:
+              [.../work/tree]
+              file = locations
+            branch:
+              file = branch
+            ''')
+
+    def test_bazaar_config_inside_branch(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --scope bazaar --remove file
+            $ bzr config -d tree --all file
+            locations:
+              [.../work/tree]
+              file = locations
+            branch:
+              file = branch
+            ''')
+
+    def test_locations_config_inside_branch(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --scope locations --remove file
+            $ bzr config -d tree --all file
+            branch:
+              file = branch
+            bazaar:
+              file = bazaar
+            ''')
+
+    def test_branch_config_default(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --remove file
+            $ bzr config -d tree --all file
+            branch:
+              file = branch
+            bazaar:
+              file = bazaar
+            ''')
+        script.run_script(self, '''\
+            $ bzr config -d tree --remove file
+            $ bzr config -d tree --all file
+            bazaar:
+              file = bazaar
+            ''')
+
+    def test_branch_config_forcing_branch(self):
+        script.run_script(self, '''\
+            $ bzr config -d tree --scope branch --remove file
+            $ bzr config -d tree --all file
+            locations:
+              [.../work/tree]
+              file = locations
+            bazaar:
+              file = bazaar
+            ''')
+        script.run_script(self, '''\
+            $ bzr config -d tree --remove file
+            $ bzr config -d tree --all file
+            bazaar:
+              file = bazaar
+            ''')

=== modified file 'bzrlib/tests/blackbox/test_conflicts.py'
--- bzrlib/tests/blackbox/test_conflicts.py	2010-05-28 14:15:28 +0000
+++ bzrlib/tests/blackbox/test_conflicts.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,110 +19,108 @@
     tests,
     workingtree,
     )
-
-# FIXME: These don't really look at the output of the conflict commands, just
-# the number of lines - there should be more examination.
-
-class TestConflictsBase(tests.TestCaseWithTransport):
+from bzrlib.tests import script, features
+
+
+def make_tree_with_conflicts(test, this_path='this', other_path='other',
+        prefix='my'):
+    this_tree = test.make_branch_and_tree(this_path)
+    test.build_tree_contents([
+        ('%s/%sfile' % (this_path, prefix), 'this content\n'),
+        ('%s/%s_other_file' % (this_path, prefix), 'this content\n'),
+        ('%s/%sdir/' % (this_path, prefix),),
+        ])
+    this_tree.add(prefix+'file')
+    this_tree.add(prefix+'_other_file')
+    this_tree.add(prefix+'dir')
+    this_tree.commit(message="new")
+    other_tree = this_tree.bzrdir.sprout(other_path).open_workingtree()
+    test.build_tree_contents([
+        ('%s/%sfile' % (other_path, prefix), 'contentsb\n'),
+        ('%s/%s_other_file' % (other_path, prefix), 'contentsb\n'),
+        ])
+    other_tree.rename_one(prefix+'dir', prefix+'dir2')
+    other_tree.commit(message="change")
+    test.build_tree_contents([
+        ('%s/%sfile' % (this_path, prefix), 'contentsa2\n'),
+        ('%s/%s_other_file' % (this_path, prefix), 'contentsa2\n'),
+        ])
+    this_tree.rename_one(prefix+'dir', prefix+'dir3')
+    this_tree.commit(message='change')
+    this_tree.merge_from_branch(other_tree.branch)
+    return this_tree, other_tree
+
+
+class TestConflicts(script.TestCaseWithTransportAndScript):
 
     def setUp(self):
-        super(TestConflictsBase, self).setUp()
-        self.make_tree_with_conflicts()
-
-    def make_tree_with_conflicts(self):
-        a_tree = self.make_branch_and_tree('a')
-        self.build_tree_contents([
-            ('a/myfile', 'contentsa\n'),
-            ('a/my_other_file', 'contentsa\n'),
-            ('a/mydir/',),
-            ])
-        a_tree.add('myfile')
-        a_tree.add('my_other_file')
-        a_tree.add('mydir')
-        a_tree.commit(message="new")
-        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
-        self.build_tree_contents([
-            ('b/myfile', 'contentsb\n'),
-            ('b/my_other_file', 'contentsb\n'),
-            ])
-        b_tree.rename_one('mydir', 'mydir2')
-        b_tree.commit(message="change")
-        self.build_tree_contents([
-            ('a/myfile', 'contentsa2\n'),
-            ('a/my_other_file', 'contentsa2\n'),
-            ])
-        a_tree.rename_one('mydir', 'mydir3')
-        a_tree.commit(message='change')
-        a_tree.merge_from_branch(b_tree.branch)
-
-    def run_bzr(self, cmd, working_dir='a', **kwargs):
-        return super(TestConflictsBase, self).run_bzr(
-            cmd, working_dir=working_dir, **kwargs)
-
-
-class TestConflicts(TestConflictsBase):
+        super(TestConflicts, self).setUp()
+        make_tree_with_conflicts(self, 'branch', 'other')
 
     def test_conflicts(self):
-        out, err = self.run_bzr('conflicts')
-        self.assertEqual(3, len(out.splitlines()))
+        self.run_script("""\
+$ cd branch
+$ bzr conflicts
+Text conflict in my_other_file
+Path conflict: mydir3 / mydir2
+Text conflict in myfile
+""")
 
     def test_conflicts_text(self):
-        out, err = self.run_bzr('conflicts --text')
-        self.assertEqual(['my_other_file', 'myfile'], out.splitlines())
+        self.run_script("""\
+$ cd branch
+$ bzr conflicts --text
+my_other_file
+myfile
+""")
 
     def test_conflicts_directory(self):
-        """Test --directory option"""
-        out, err = self.run_bzr('conflicts --directory a', working_dir='.')
-        self.assertEqual(3, len(out.splitlines()))
-        self.assertEqual('', err)
-
-
-class TestResolve(TestConflictsBase):
-
-    def test_resolve(self):
-        self.run_bzr('resolve myfile')
-        out, err = self.run_bzr('conflicts')
-        self.assertEqual(2, len(out.splitlines()))
-        self.run_bzr('resolve my_other_file')
-        self.run_bzr('resolve mydir2')
-        out, err = self.run_bzr('conflicts')
-        self.assertEqual(0, len(out.splitlines()))
-
-    def test_resolve_all(self):
-        self.run_bzr('resolve --all')
-        out, err = self.run_bzr('conflicts')
-        self.assertEqual(0, len(out.splitlines()))
-
-    def test_resolve_in_subdir(self):
-        """resolve when run from subdirectory should handle relative paths"""
-        self.build_tree(["a/subdir/"])
-        self.run_bzr("resolve ../myfile", working_dir='a/subdir')
-        self.run_bzr("resolve ../a/myfile", working_dir='b')
-        wt = workingtree.WorkingTree.open_containing('b')[0]
-        conflicts = wt.conflicts()
-        self.assertEqual(True, conflicts.is_empty(),
-                         "tree still contains conflicts: %r" % conflicts)
-
-    def test_auto_resolve(self):
-        """Text conflicts can be resolved automatically"""
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree_contents([('tree/file',
-            '<<<<<<<\na\n=======\n>>>>>>>\n')])
-        tree.add('file', 'file_id')
-        self.assertEqual(tree.kind('file_id'), 'file')
-        file_conflict = conflicts.TextConflict('file', file_id='file_id')
-        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
-        note = self.run_bzr('resolve', retcode=1, working_dir='tree')[1]
-        self.assertContainsRe(note, '0 conflict\\(s\\) auto-resolved.')
-        self.assertContainsRe(note,
-            'Remaining conflicts:\nText conflict in file')
-        self.build_tree_contents([('tree/file', 'a\n')])
-        note = self.run_bzr('resolve', working_dir='tree')[1]
-        self.assertContainsRe(note, 'All conflicts resolved.')
-
-    def test_resolve_all_directory(self):
-        """Test --directory option"""
-        out, err = self.run_bzr('resolve --all -d a', working_dir='.')
-        self.assertEqual('', err)
-        out, err = self.run_bzr('conflicts')
-        self.assertEqual(0, len(out.splitlines()))
+        self.run_script("""\
+$ bzr conflicts  -d branch
+Text conflict in my_other_file
+Path conflict: mydir3 / mydir2
+Text conflict in myfile
+""")
+
+
+class TestUnicodePaths(tests.TestCaseWithTransport):
+    """Unicode characters in conflicts should be displayed properly"""
+
+    _test_needs_features = [features.UnicodeFilenameFeature]
+    encoding = "UTF-8"
+
+    def _as_output(self, text):
+        return text
+
+    def test_messages(self):
+        """Conflict messages involving non-ascii paths are displayed okay"""
+        make_tree_with_conflicts(self, "branch", prefix=u"\xA7")
+        out, err = self.run_bzr(["conflicts", "-d", "branch"],
+            encoding=self.encoding)
+        self.assertEqual(out.decode(self.encoding),
+            u"Text conflict in \xA7_other_file\n"
+            u"Path conflict: \xA7dir3 / \xA7dir2\n"
+            u"Text conflict in \xA7file\n")
+        self.assertEqual(err, "")
+
+    def test_text_conflict_paths(self):
+        """Text conflicts on non-ascii paths are displayed okay"""
+        make_tree_with_conflicts(self, "branch", prefix=u"\xA7")
+        out, err = self.run_bzr(["conflicts", "-d", "branch", "--text"],
+            encoding=self.encoding)
+        self.assertEqual(out.decode(self.encoding),
+            u"\xA7_other_file\n"
+            u"\xA7file\n")
+        self.assertEqual(err, "")
+
+
+class TestUnicodePathsOnAsciiTerminal(TestUnicodePaths):
+    """Undisplayable unicode characters in conflicts should be escaped"""
+
+    encoding = "ascii"
+
+    def setUp(self):
+        self.skip("Need to decide if replacing is the desired behaviour")
+
+    def _as_output(self, text):
+        return text.encode(self.encoding, "replace")

=== modified file 'bzrlib/tests/blackbox/test_debug.py'
--- bzrlib/tests/blackbox/test_debug.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_debug.py	2011-02-04 22:25:59 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,10 +21,10 @@
 import sys
 import time
 
-from bzrlib.tests import TestCaseInTempDir
-
-
-class TestDebugOption(TestCaseInTempDir):
+from bzrlib import debug, tests
+
+
+class TestDebugOption(tests.TestCaseInTempDir):
 
     def test_dash_derror(self):
         """With -Derror, tracebacks are shown even for user errors"""
@@ -39,3 +39,22 @@
         # With -Dlock, locking and unlocking is recorded into the log
         self.run_bzr("-Dlock init foo")
         self.assertContainsRe(self.get_log(), "lock_write")
+
+
+class TestDebugBytes(tests.TestCaseWithTransport):
+
+    def test_bytes_reports_activity(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/one'])
+        tree.add('one')
+        rev_id = tree.commit('first')
+        remote_trans = self.make_smart_server('.')
+        # I would like to avoid run_bzr_subprocess here, but we need it to be
+        # connected to a real TextUIFactory. The NullProgressView always
+        # ignores transport activity.
+        env = {'BZR_PROGRESS_BAR': 'text'}
+        out, err = self.run_bzr_subprocess('branch -Dbytes %s/tree target'
+                                           % (remote_trans.base,),
+                                           env_changes=env)
+        self.assertContainsRe(err, 'Branched 1 revision')
+        self.assertContainsRe(err, 'Transferred:.*kB')

=== modified file 'bzrlib/tests/blackbox/test_diff.py'
--- bzrlib/tests/blackbox/test_diff.py	2010-07-18 06:54:01 +0000
+++ bzrlib/tests/blackbox/test_diff.py	2011-01-19 18:44:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +29,9 @@
     DiffTree,
     format_registry as diff_format_registry,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 def subst_dates(string):
@@ -155,10 +158,6 @@
         self.assertContainsRe(err,
             "Requested revision: '1.1' does not exist in branch:")
 
-    def test_diff_diff_options_and_using(self):
-        out, err = self.run_bzr('diff --diff-options -wu --using /usr/bin/diff', retcode=3,
-          error_regexes=('are mutually exclusive.',))
-
     def test_diff_unversioned(self):
         # Get an error when diffing a non-versioned file.
         # (Malone #3619)
@@ -324,6 +323,9 @@
         self.build_tree_contents([('hello', 'hello world!\n')])
         output = self.run_bzr('diff --format=boo', retcode=1)
         self.assertTrue("BOO!" in output[0])
+        output = self.run_bzr('diff -Fboo', retcode=1)
+        self.assertTrue("BOO!" in output[0])
+
 
 class TestCheckoutDiff(TestDiff):
 
@@ -387,8 +389,7 @@
         # subprocess.py that we had to workaround).
         # However, if 'diff' may not be available
         self.make_example_branch()
-        # this will be automatically restored by the base bzr test class
-        os.environ['BZR_PROGRESS_BAR'] = 'none'
+        self.overrideEnv('BZR_PROGRESS_BAR', 'none')
         out, err = self.run_bzr_subprocess('diff -r 1 --diff-options -ub',
                                            universal_newlines=True,
                                            retcode=None)
@@ -403,6 +404,16 @@
         self.assertEndsWith(out, "\n@@ -0,0 +1 @@\n"
                                  "+baz\n\n")
 
+    def test_external_diff_options_and_using(self):
+        """Test that the options are passed correctly to an external diff process"""
+        self.requireFeature(features.diff_feature)
+        self.make_example_branch()
+        self.build_tree_contents([('hello', 'Foo\n')])
+        out, err = self.run_bzr('diff --diff-options -i --using diff',
+                                    retcode=1)
+        self.assertEquals("=== modified file 'hello'\n", out)
+        self.assertEquals('', err)
+
 
 class TestDiffOutput(DiffBase):
 

=== modified file 'bzrlib/tests/blackbox/test_dpush.py'
--- bzrlib/tests/blackbox/test_dpush.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_dpush.py	2011-09-22 12:13:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,45 +18,20 @@
 """Black-box tests for bzr dpush."""
 
 
-import os
-
 from bzrlib import (
-    branch,
-    bzrdir,
-    foreign,
     tests,
-    workingtree,
     )
 from bzrlib.tests import (
-    blackbox,
     script,
     test_foreign,
     )
 from bzrlib.tests.blackbox import test_push
-
-
-def load_tests(standard_tests, module, loader):
-    """Multiply tests for the dpush command."""
-    result = loader.suiteClass()
-
-    # one for each king of change
-    changes_tests, remaining_tests = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance((
-                TestDpushStrictWithChanges,
-                )))
-    changes_scenarios = [
-        ('uncommitted',
-         dict(_changes_type= '_uncommitted_changes')),
-        ('pending-merges',
-         dict(_changes_type= '_pending_merges')),
-        ('out-of-sync-trees',
-         dict(_changes_type= '_out_of_sync_trees')),
-        ]
-    tests.multiply_tests(changes_tests, changes_scenarios, result)
-    # No parametrization for the remaining tests
-    result.addTests(remaining_tests)
-
-    return result
+from bzrlib.tests.scenarios import (
+    load_tests_apply_scenarios,
+    )
+
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestDpush(tests.TestCaseWithTransport):
@@ -78,7 +53,9 @@
         source_tree = self.make_branch_and_tree("dc")
         output, error = self.run_bzr("dpush -d dc dp", retcode=3)
         self.assertEquals("", output)
-        self.assertContainsRe(error, 'in the same VCS, lossy push not necessary. Please use regular push.')
+        self.assertContainsRe(error,
+            'in the same VCS, lossy push not necessary. Please use regular '
+            'push.')
 
     def test_dpush(self):
         branch = self.make_dummy_builder('d').get_branch()
@@ -89,6 +66,8 @@
 
         script.run_script(self, """
             $ bzr dpush -d dc d
+            2>Doing on-the-fly conversion from DummyForeignVcsRepositoryFormat() to RepositoryFormat2a().
+            2>This may take some time. Upgrade the repositories to the same format for better performance.
             2>Pushed up to revision 2.
             $ bzr status dc
             """)
@@ -104,6 +83,9 @@
 
         script.run_script(self, '''
             $ bzr dpush -d dc d
+            2>Doing on-the-fly conversion from DummyForeignVcsRepositoryFormat() to RepositoryFormat2a().
+            2>This may take some time. Upgrade the repositories to the same format for better performance.
+            2>Pushed up to revision 2.
             $ bzr revno dc
             2
             $ bzr status dc
@@ -121,6 +103,9 @@
         self.build_tree_contents([("dc/foofile", "blaaaal")])
         script.run_script(self, '''
             $ bzr dpush -d dc d --no-strict
+            2>Doing on-the-fly conversion from DummyForeignVcsRepositoryFormat() to RepositoryFormat2a().
+            2>This may take some time. Upgrade the repositories to the same format for better performance.
+            2>Pushed up to revision 2.
             ''')
         self.assertFileEqual("blaaaal", "dc/foofile")
         # if the dummy vcs wasn't that dummy we could uncomment the line below
@@ -161,8 +146,8 @@
     def set_config_push_strict(self, value):
         # set config var (any of bazaar.conf, locations.conf, branch.conf
         # should do)
-        conf = self.tree.branch.get_config()
-        conf.set_user_option('dpush_strict', value)
+        conf = self.tree.branch.get_config_stack()
+        conf.set('dpush_strict', value)
 
     _default_command = ['dpush', '../to']
 
@@ -178,6 +163,8 @@
 class TestDpushStrictWithChanges(TestDpushStrictMixin,
                                  test_push.TestPushStrictWithChanges):
 
+    scenarios = test_push.strict_push_change_scenarios
+
     _changes_type = None # Set by load_tests
 
     def setUp(self):
@@ -186,4 +173,3 @@
 
     def test_push_with_revision(self):
         raise tests.TestNotApplicable('dpush does not handle --revision')
-

=== modified file 'bzrlib/tests/blackbox/test_dump_btree.py'
--- bzrlib/tests/blackbox/test_dump_btree.py	2010-03-25 09:39:03 +0000
+++ bzrlib/tests/blackbox/test_dump_btree.py	2011-02-08 23:06:34 +0000
@@ -91,3 +91,40 @@
         finally:
             out_f.close()
         out, err = self.run_bzr('dump-btree test.btree')
+
+    def create_sample_empty_btree_index(self):
+        builder = btree_index.BTreeBuilder(
+            reference_lists=1, key_elements=2)
+        out_f = builder.finish()
+        try:
+            self.build_tree_contents([('test.btree', out_f.read())])
+        finally:
+            out_f.close()
+
+    def test_dump_empty_btree_smoke(self):
+        self.create_sample_empty_btree_index()
+        out, err = self.run_bzr('dump-btree test.btree')
+        self.assertEqualDiff("", out)
+
+    def test_dump_empty_btree_http_smoke(self):
+        self.transport_readonly_server = http_server.HttpServer
+        self.create_sample_empty_btree_index()
+        url = self.get_readonly_url('test.btree')
+        out, err = self.run_bzr(['dump-btree', url])
+        self.assertEqualDiff("", out)
+
+    def test_dump_empty_btree_raw_smoke(self):
+        self.create_sample_empty_btree_index()
+        out, err = self.run_bzr('dump-btree test.btree --raw')
+        self.assertEqualDiff(
+            'Root node:\n'
+            'B+Tree Graph Index 2\n'
+            'node_ref_lists=1\n'
+            'key_elements=2\n'
+            'len=0\n'
+            'row_lengths=\n'
+            '\n'
+            'Page 0\n'
+            '(empty)\n',
+            out)
+

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2011-09-16 20:42:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,25 +16,22 @@
 
 """Tests for display of exceptions."""
 
-from cStringIO import StringIO
 import os
-import sys
+import re
 
 from bzrlib import (
     bzrdir,
     config,
+    controldir,
     errors,
     osutils,
     repository,
     tests,
-    trace,
     )
-
-from bzrlib.tests import TestCaseInTempDir, TestCase
-from bzrlib.errors import NotBranchError
-
-
-class TestExceptionReporting(TestCase):
+from bzrlib.repofmt.groupcompress_repo import RepositoryFormat2a
+
+
+class TestExceptionReporting(tests.TestCaseInTempDir):
 
     def test_exception_exitcode(self):
         # we must use a subprocess, because the normal in-memory mechanism
@@ -47,6 +44,41 @@
                 r'exceptions\.AssertionError: always fails\n')
         self.assertContainsRe(err, r'Bazaar has encountered an internal error')
 
+    def test_undecodable_argv(self):
+        """A user error must be reported if argv is not in the locale encoding
+
+        A subprocess with an environment ascii-only setting is used so the test
+        can run without worrying about the locale the test suite is using.
+        """
+        if os.name != "posix":
+            raise tests.TestNotApplicable("Needs system beholden to C locales")
+        out, err = self.run_bzr_subprocess(["\xa0"],
+            env_changes={"LANG": "C", "LC_ALL": "C"},
+            universal_newlines=True,
+            retcode=errors.EXIT_ERROR)
+        self.assertContainsRe(err, r"^bzr: ERROR: .*'\\xa0'.* unsupported",
+            flags=re.MULTILINE)
+        self.assertEquals(out, "")
+
+
+class TestOptParseBugHandling(tests.TestCase):
+    "Test that we handle http://bugs.python.org/issue2931"
+
+    def test_nonascii_optparse(self):
+        """Reasonable error raised when non-ascii in option name"""
+        error_re = 'Only ASCII permitted in option names'
+        out = self.run_bzr_error([error_re], ['st',u'-\xe4'])
+
+
+class TestObsoleteRepoFormat(RepositoryFormat2a):
+
+    @classmethod
+    def get_format_string(cls):
+        return "Test Obsolete Repository Format"
+
+    def is_deprecated(self):
+        return True
+
 
 class TestDeprecationWarning(tests.TestCaseWithTransport):
     """The deprecation warning is controlled via a global variable:
@@ -59,6 +91,16 @@
 
     def setUp(self):
         super(TestDeprecationWarning, self).setUp()
+        self.addCleanup(repository.format_registry.remove,
+            TestObsoleteRepoFormat)
+        repository.format_registry.register(TestObsoleteRepoFormat)
+        self.addCleanup(controldir.format_registry.remove, "testobsolete")
+        bzrdir.register_metadir(controldir.format_registry, "testobsolete",
+            "bzrlib.tests.blackbox.test_exceptions.TestObsoleteRepoFormat",
+            branch_format='bzrlib.branch.BzrBranchFormat7',
+            tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
+            deprecated=True,
+            help='Same as 2a, but with an obsolete repo format.')
         self.disable_deprecation_warning()
 
     def enable_deprecation_warning(self, repo=None):
@@ -71,7 +113,8 @@
 
     def make_obsolete_repo(self, path):
         # We don't want the deprecation raising during the repo creation
-        tree = self.make_branch_and_tree(path, format=bzrdir.BzrDirFormat5())
+        format = controldir.format_registry.make_bzrdir("testobsolete")
+        tree = self.make_branch_and_tree(path, format=format)
         return tree
 
     def check_warning(self, present):
@@ -79,7 +122,7 @@
             check = self.assertContainsRe
         else:
             check = self.assertNotContainsRe
-        check(self._get_log(keep_log_file=True), 'WARNING.*bzr upgrade')
+        check(self.get_log(), 'WARNING.*bzr upgrade')
 
     def test_repository_deprecation_warning(self):
         """Old formats give a warning"""

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_export.py	2011-07-08 23:01:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,63 +23,95 @@
 import stat
 import sys
 import tarfile
+import time
 import zipfile
 
 
 from bzrlib import (
     export,
+    osutils,
     tests,
     )
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import (
+    features,
+    TestCaseWithTransport,
+    )
 
 
 class TestExport(TestCaseWithTransport):
 
-    def test_tar_export(self):
-        tree = self.make_branch_and_tree('tar')
-        self.build_tree(['tar/a'])
+    # On Windows, if we fail to set the binary bit, and a '\r' or '\n'
+    # ends up in the data stream, we will get corruption. Add a fair amount
+    # of random data, to help ensure there is at least one.
+    _file_content = ('!\r\n\t\n \r'
+        + 'r29trp9i1r8k0e24c2o7mcx2isrlaw7toh1hv2mtst3o1udkl36v9xn2z8kt\n'
+          'tvjn7e3i9cj1qs1rw9gcye9w72cbdueiufw8nky7bs08lwggir59d62knecp\n'
+          '7s0537r8sg3e8hdnidji49rswo47c3j8190nh8emef2b6j1mf5kdq45nt3f5\n'
+          '1sz9u7fuzrm4w8bebre7p62sh5os2nkj2iiyuk9n0w0pjpdulu9k2aajejah\n'
+          'ini90ny40qzs12ajuy0ua6l178n93lvy2atqngnntsmtlmqx7yhp0q9a1xr4\n'
+          '1n69kgbo6qu9osjpqq83446r00jijtcstzybfqwm1lnt9spnri2j07bt7bbh\n'
+          'rf3ejatdxta83te2s0pt9rc4hidgy3d2pc53p4wscdt2b1dfxdj9utf5m17f\n'
+          'f03oofcau950o090vyx6m72vfkywo7gp3ajzi6uk02dwqwtumq4r44xx6ho7\n'
+          'nhynborjdjep5j53f9548msb7gd3x9a1xveb4s8zfo6cbdw2kdngcrbakwu8\n'
+          'ql5a8l94gplkwr7oypw5nt1gj5i3xwadyjfr3lb61tfkz31ba7uda9knb294\n'
+          '1azhfta0q3ry9x36lxyanvhp0g5z0t5a0i4wnoc8p4htexi915y1cnw4nznn\n'
+          'aj70dvp88ifiblv2bsp98hz570teinj8g472ddxni9ydmazfzwtznbf3hrg6\n'
+          '84gigirjt6n2yagf70036m8d73cz0jpcighpjtxsmbgzbxx7nb4ewq6jbgnc\n'
+          'hux1b0qtsdi0zfhj6g1otf5jcldmtdvuon8y1ttszkqw3ograwi25yl921hy\n'
+          'izgscmfha9xdhxxabs07b40secpw22ah9iwpbmsns6qz0yr6fswto3ft2ez5\n'
+          'ngn48pdfxj1pw246drmj1y2ll5af5w7cz849rapzd9ih7qvalw358co0yzrs\n'
+          'xan9291d1ivjku4o5gjrsnmllrqwxwy86pcivinbmlnzasa9v3o22lgv4uyd\n'
+          'q8kw77bge3hr5rr5kzwjxk223bkmo3z9oju0954undsz8axr3kb3730otrcr\n'
+          '9cwhu37htnizdwxmpoc5qmobycfm7ubbykfumv6zgkl6b8zlslwl7a8b81vz\n'
+          '3weqkvv5csfza9xvwypr6lo0t03fwp0ihmci3m1muh0lf2u30ze0hjag691j\n'
+          '27fjtd3e3zbiin5n2hq21iuo09ukbs73r5rt7vaw6axvoilvdciir9ugjh2c\n'
+          'na2b8dr0ptftoyhyxv1iwg661y338e28fhz4xxwgv3hnoe98ydfa1oou45vj\n'
+          'ln74oac2keqt0agbylrqhfscin7ireae2bql7z2le823ksy47ud57z8ctomp\n'
+          '31s1vwbczdjwqp0o2jc7mkrurvzg8mj2zwcn2iily4gcl4sy4fsh4rignlyz\n')
+
+    def make_basic_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/a', self._file_content)])
         tree.add('a')
-        self.build_tree_contents([('tar/.bzrrules', '')])
-        tree.add('.bzrrules')
-        self.build_tree(['tar/.bzr-adir/', 'tar/.bzr-adir/afile'])
-        tree.add(['.bzr-adir/', '.bzr-adir/afile'])
-
-        os.chdir('tar')
-        self.run_bzr('ignore something')
         tree.commit('1')
-
-        self.failUnless(tree.has_filename('.bzrignore'))
-        self.failUnless(tree.has_filename('.bzrrules'))
-        self.failUnless(tree.has_filename('.bzr-adir'))
-        self.failUnless(tree.has_filename('.bzr-adir/afile'))
-        self.run_bzr('export test.tar.gz')
+        return tree
+
+    def make_tree_with_extra_bzr_files(self):
+        tree = self.make_basic_tree()
+        self.build_tree_contents([('tree/.bzrrules', '')])
+        self.build_tree(['tree/.bzr-adir/', 'tree/.bzr-adir/afile'])
+        tree.add(['.bzrrules', '.bzr-adir/', '.bzr-adir/afile'])
+
+        self.run_bzr('ignore something -d tree')
+        tree.commit('2')
+        return tree
+
+    def test_tar_export_ignores_bzr(self):
+        tree = self.make_tree_with_extra_bzr_files()
+
+        self.assertTrue(tree.has_filename('.bzrignore'))
+        self.assertTrue(tree.has_filename('.bzrrules'))
+        self.assertTrue(tree.has_filename('.bzr-adir'))
+        self.assertTrue(tree.has_filename('.bzr-adir/afile'))
+        self.run_bzr('export test.tar.gz -d tree')
         ball = tarfile.open('test.tar.gz')
         # Make sure the tarball contains 'a', but does not contain
         # '.bzrignore'.
-        self.assertEqual(['test/a'], sorted(ball.getnames()))
-
-        if sys.version_info < (2, 5, 2) and sys.platform == 'darwin':
-            raise tests.KnownFailure('python %r has a tar related bug, upgrade'
-                                     % (sys.version_info,))
-        out, err = self.run_bzr('export --format=tgz --root=test -')
-        ball = tarfile.open('', fileobj=StringIO(out))
-        self.assertEqual(['test/a'], sorted(ball.getnames()))
-
-    def test_tar_export_unicode(self):
+        self.assertEqual(['test/a'],
+                         sorted(ball.getnames()))
+
+    def test_tar_export_unicode_filename(self):
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tar')
         # FIXME: using fname = u'\xe5.txt' below triggers a bug revealed since
         # bzr.dev revno 4216 but more related to OSX/working trees/unicode than
         # export itself --vila 20090406
         fname = u'\N{Euro Sign}.txt'
-        try:
-            self.build_tree(['tar/' + fname])
-        except UnicodeError:
-            raise tests.TestSkipped('Unable to represent path %r' % (fname,))
+        self.build_tree(['tar/' + fname])
         tree.add([fname])
         tree.commit('first')
 
-        os.chdir('tar')
-        self.run_bzr('export test.tar')
+        self.run_bzr('export test.tar -d tar')
         ball = tarfile.open('test.tar')
         # all paths are prefixed with the base name of the tarball
         self.assertEqual(['test/' + fname.encode('utf8')],
@@ -87,44 +119,88 @@
 
     def test_tar_export_unicode_basedir(self):
         """Test for bug #413406"""
+        self.requireFeature(features.UnicodeFilenameFeature)
         basedir = u'\N{euro sign}'
         os.mkdir(basedir)
-        os.chdir(basedir)
-        self.run_bzr(['init', 'branch'])
-        os.chdir('branch')
-        self.run_bzr(['export', '--format', 'tgz', u'test.tar.gz'])
-
-    def test_zip_export(self):
-        tree = self.make_branch_and_tree('zip')
-        self.build_tree(['zip/a'])
-        tree.add('a')
-        self.build_tree_contents([('zip/.bzrrules', '')])
-        tree.add('.bzrrules')
-        self.build_tree(['zip/.bzr-adir/', 'zip/.bzr-adir/afile'])
-        tree.add(['.bzr-adir/', '.bzr-adir/afile'])
-
-        os.chdir('zip')
-        self.run_bzr('ignore something')
-        tree.commit('1')
-
-        self.failUnless(tree.has_filename('.bzrignore'))
-        self.failUnless(tree.has_filename('.bzrrules'))
-        self.failUnless(tree.has_filename('.bzr-adir'))
-        self.failUnless(tree.has_filename('.bzr-adir/afile'))
-        self.run_bzr('export test.zip')
+        self.run_bzr(['init', basedir])
+        self.run_bzr(['export', '--format', 'tgz', u'test.tar.gz',
+                      '-d', basedir])
+
+    def test_zip_export_ignores_bzr(self):
+        tree = self.make_tree_with_extra_bzr_files()
+
+        self.assertTrue(tree.has_filename('.bzrignore'))
+        self.assertTrue(tree.has_filename('.bzrrules'))
+        self.assertTrue(tree.has_filename('.bzr-adir'))
+        self.assertTrue(tree.has_filename('.bzr-adir/afile'))
+        self.run_bzr('export test.zip -d tree')
 
         zfile = zipfile.ZipFile('test.zip')
         # Make sure the zipfile contains 'a', but does not contain
         # '.bzrignore'.
         self.assertEqual(['test/a'], sorted(zfile.namelist()))
 
+    # TODO: This really looks like something that should be using permutation
+    #       testing. Though the actual setup and teardown functions are pretty
+    #       different for each
+    def assertZipANameAndContent(self, zfile, root=''):
+        """The file should only contain name 'a' and _file_content"""
+        fname = root + 'a'
+        self.assertEqual([fname], sorted(zfile.namelist()))
+        zfile.testzip()
+        self.assertEqualDiff(self._file_content, zfile.read(fname))
+
+    def test_zip_export_stdout(self):
+        tree = self.make_basic_tree()
+        contents = self.run_bzr('export -d tree --format=zip -')[0]
+        self.assertZipANameAndContent(zipfile.ZipFile(StringIO(contents)))
+
+    def test_zip_export_file(self):
+        tree = self.make_basic_tree()
+        self.run_bzr('export -d tree test.zip')
+        self.assertZipANameAndContent(zipfile.ZipFile('test.zip'),
+                                      root='test/')
+
+    def assertTarANameAndContent(self, ball, root=''):
+        fname = root + 'a'
+        tar_info = ball.next()
+        self.assertEqual(fname, tar_info.name)
+        self.assertEqual(tarfile.REGTYPE, tar_info.type)
+        self.assertEqual(len(self._file_content), tar_info.size)
+        f = ball.extractfile(tar_info)
+        if self._file_content != f.read():
+            self.fail('File content has been corrupted.'
+                      ' Check that all streams are handled in binary mode.')
+        # There should be no other files in the tarball
+        self.assertIs(None, ball.next())
+
+    def run_tar_export_disk_and_stdout(self, extension, tarfile_flags):
+        tree = self.make_basic_tree()
+        fname = 'test.%s' % (extension,)
+        self.run_bzr('export -d tree %s' % (fname,))
+        mode = 'r|%s' % (tarfile_flags,)
+        ball = tarfile.open(fname, mode=mode)
+        self.assertTarANameAndContent(ball, root='test/')
+        content = self.run_bzr('export -d tree --format=%s -' % (extension,))[0]
+        ball = tarfile.open(mode=mode, fileobj=StringIO(content))
+        self.assertTarANameAndContent(ball, root='')
+
+    def test_tar_export(self):
+        self.run_tar_export_disk_and_stdout('tar', '')
+
+    def test_tgz_export(self):
+        self.run_tar_export_disk_and_stdout('tgz', 'gz')
+
+    def test_tbz2_export(self):
+        self.run_tar_export_disk_and_stdout('tbz2', 'bz2')
+
+    # TODO: test_xz_export, I don't have pylzma working here to test it.
+
     def test_zip_export_unicode(self):
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('zip')
         fname = u'\N{Euro Sign}.txt'
-        try:
-            self.build_tree(['zip/' + fname])
-        except UnicodeError:
-            raise tests.TestSkipped('Unable to represent path %r' % (fname,))
+        self.build_tree(['zip/' + fname])
         tree.add([fname])
         tree.commit('first')
 
@@ -151,7 +227,8 @@
         self.assertEqual(['test/a', 'test/b/', 'test/b/c', 'test/d/'], names)
 
         file_attr = stat.S_IFREG | export.zip_exporter.FILE_PERMISSIONS
-        dir_attr = stat.S_IFDIR | export.zip_exporter.ZIP_DIRECTORY_BIT
+        dir_attr = (stat.S_IFDIR | export.zip_exporter.ZIP_DIRECTORY_BIT |
+                    export.zip_exporter.DIR_PERMISSIONS)
 
         a_info = zfile.getinfo(names[0])
         self.assertEqual(file_attr, a_info.external_attr)
@@ -178,10 +255,10 @@
         self.run_bzr('ignore something')
         tree.commit('1')
 
-        self.failUnless(tree.has_filename('.bzrignore'))
-        self.failUnless(tree.has_filename('.bzrrules'))
-        self.failUnless(tree.has_filename('.bzr-adir'))
-        self.failUnless(tree.has_filename('.bzr-adir/afile'))
+        self.assertTrue(tree.has_filename('.bzrignore'))
+        self.assertTrue(tree.has_filename('.bzrrules'))
+        self.assertTrue(tree.has_filename('.bzr-adir'))
+        self.assertTrue(tree.has_filename('.bzr-adir/afile'))
         self.run_bzr('export direxport')
 
         files = sorted(os.listdir('direxport'))
@@ -224,7 +301,7 @@
         os.chdir('branch')
 
         self.run_bzr('export ../first.tar -r 1')
-        self.failUnless(os.path.isfile('../first.tar'))
+        self.assertTrue(os.path.isfile('../first.tar'))
         tf = tarfile.open('../first.tar')
         try:
             self.assertEqual(['first/hello'], sorted(tf.getnames()))
@@ -233,13 +310,13 @@
             tf.close()
 
         self.run_bzr('export ../first.tar.gz -r 1')
-        self.failUnless(os.path.isfile('../first.tar.gz'))
+        self.assertTrue(os.path.isfile('../first.tar.gz'))
         self.run_bzr('export ../first.tbz2 -r 1')
-        self.failUnless(os.path.isfile('../first.tbz2'))
+        self.assertTrue(os.path.isfile('../first.tbz2'))
         self.run_bzr('export ../first.tar.bz2 -r 1')
-        self.failUnless(os.path.isfile('../first.tar.bz2'))
+        self.assertTrue(os.path.isfile('../first.tar.bz2'))
         self.run_bzr('export ../first.tar.tbz2 -r 1')
-        self.failUnless(os.path.isfile('../first.tar.tbz2'))
+        self.assertTrue(os.path.isfile('../first.tar.tbz2'))
 
         tf = tarfile.open('../first.tar.tbz2', 'r:bz2')
         try:
@@ -260,7 +337,7 @@
         os.chdir('branch')
 
         self.run_bzr('export ../first.zip -r 1')
-        self.failUnlessExists('../first.zip')
+        self.assertPathExists('../first.zip')
         zf = zipfile.ZipFile('../first.zip')
         try:
             self.assertEqual(['first/hello'], sorted(zf.namelist()))
@@ -313,9 +390,31 @@
         har_st = os.stat('t/har')
         self.assertEquals(315532800, har_st.st_mtime)
 
+    def test_dir_export_partial_tree_per_file_timestamps(self):
+        tree = self.example_branch()
+        self.build_tree(['branch/subdir/', 'branch/subdir/foo.txt'])
+        tree.smart_add(['branch'])
+        # Earliest allowable date on FAT32 filesystems is 1980-01-01
+        tree.commit('setup', timestamp=315532800)
+        self.run_bzr('export --per-file-timestamps tpart branch/subdir')
+        foo_st = os.stat('tpart/foo.txt')
+        self.assertEquals(315532800, foo_st.st_mtime)
+
     def test_export_directory(self):
         """Test --directory option"""
         self.example_branch()
         self.run_bzr(['export', '--directory=branch', 'latest'])
         self.assertEqual(['goodbye', 'hello'], sorted(os.listdir('latest')))
         self.check_file_contents('latest/goodbye', 'baz')
+
+    def test_zip_export_per_file_timestamps(self):
+        tree = self.example_branch()
+        self.build_tree_contents([('branch/har', 'foo')])
+        tree.add('har')
+        # Earliest allowable date on FAT32 filesystems is 1980-01-01
+        timestamp = 347151600
+        tree.commit('setup', timestamp=timestamp)
+        self.run_bzr('export --per-file-timestamps test.zip branch')
+        zfile = zipfile.ZipFile('test.zip')
+        info = zfile.getinfo("test/har")
+        self.assertEquals(time.localtime(timestamp)[:6], info.date_time)

=== added file 'bzrlib/tests/blackbox/test_export_pot.py'
--- bzrlib/tests/blackbox/test_export_pot.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_export_pot.py	2011-09-30 10:44:29 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""External tests of 'bzr export-pot'"""
+import os
+
+from bzrlib import ignores, osutils
+from bzrlib.tests import TestCaseWithMemoryTransport
+from bzrlib.tests.features import ModuleAvailableFeature
+
+class TestExportPot(TestCaseWithMemoryTransport):
+
+    def test_export_pot(self):
+        out, err = self.run_bzr("export-pot")
+        self.assertContainsRe(err, 'Exporting messages from builtin command: add')
+        self.assertContainsRe(out, "help of 'change' option\n"\
+                                   "msgid \"Select changes introduced by the specified revision.")
+
+    def test_export_pot_plugin(self):
+        self.requireFeature(ModuleAvailableFeature('bzrlib.plugins.launchpad'))
+        out, err = self.run_bzr("export-pot --plugin=launchpad")
+        self.assertContainsRe(err, 'Exporting messages from plugin command: launchpad-login in launchpad')
+        self.assertContainsRe(out, 'msgid "Show or set the Launchpad user ID."')

=== modified file 'bzrlib/tests/blackbox/test_filesystem_cicp.py'
--- bzrlib/tests/blackbox/test_filesystem_cicp.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_filesystem_cicp.py	2011-06-14 01:26:41 +0000
@@ -23,9 +23,13 @@
     osutils,
     tests,
     )
-from bzrlib.tests import CaseInsCasePresFilenameFeature, KnownFailure
+from bzrlib.tests import KnownFailure
 from bzrlib.osutils import canonical_relpath, pathjoin
 from bzrlib.tests.script import run_script
+from bzrlib.tests.features import (
+    CaseInsCasePresFilenameFeature,
+    )
+
 
 
 class TestCICPBase(tests.TestCaseWithTransport):
@@ -131,8 +135,8 @@
     def test_mv_newname(self):
         wt = self._make_mixed_case_tree()
         run_script(self, """
-            $ bzr add
-            $ bzr ci -m message
+            $ bzr add -q
+            $ bzr ci -qm message
             $ bzr mv camelcaseparent/camelcase camelcaseparent/NewCamelCase
             CamelCaseParent/CamelCase => CamelCaseParent/NewCamelCase
             """)
@@ -142,8 +146,8 @@
         # In this case we can specify the incorrect case for the destination,
         # as we use --after, so the file-system is sniffed.
         run_script(self, """
-            $ bzr add 
-            $ bzr ci -m message
+            $ bzr add -q
+            $ bzr ci -qm message
             $ mv CamelCaseParent/CamelCase CamelCaseParent/NewCamelCase
             $ bzr mv --after camelcaseparent/camelcase camelcaseparent/newcamelcase
             CamelCaseParent/CamelCase => CamelCaseParent/NewCamelCase

=== modified file 'bzrlib/tests/blackbox/test_filtered_view_ops.py'
--- bzrlib/tests/blackbox/test_filtered_view_ops.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_filtered_view_ops.py	2010-11-22 03:35:24 +0000
@@ -28,10 +28,7 @@
 class TestViewFileOperations(tests.TestCaseWithTransport):
 
     def make_abc_tree_with_ab_view(self):
-        # we need to use a specific format because the default format
-        # doesn't support views yet
-        format = bzrdir.format_registry.make_bzrdir('development6-rich-root')
-        wt = self.make_branch_and_tree( '.', format=format)
+        wt = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b', 'c'])
         wt.views.set_view('my', ['a', 'b'])
         return wt
@@ -156,11 +153,8 @@
 class TestViewTreeOperations(tests.TestCaseWithTransport):
 
     def make_abc_tree_and_clone_with_ab_view(self):
-        # we need to use a specific format because the default format
-        # doesn't support views yet
-        format = bzrdir.format_registry.make_bzrdir('development6-rich-root')
         # Build the first tree
-        wt1 = self.make_branch_and_tree('tree_1', format=format)
+        wt1 = self.make_branch_and_tree('tree_1')
         self.build_tree(['tree_1/a', 'tree_1/b', 'tree_1/c'])
         wt1.add(['a', 'b', 'c'])
         wt1.commit("adding a b c")

=== modified file 'bzrlib/tests/blackbox/test_help.py'
--- bzrlib/tests/blackbox/test_help.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_help.py	2011-01-12 00:58:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 
 
 import bzrlib
+from bzrlib import config
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.config import (ensure_config_dir_exists, config_filename)
 
 
 class TestHelp(TestCaseWithTransport):
@@ -104,6 +104,10 @@
         self.assertEquals(dash_help, qmark_long)
         self.assertEquals(dash_help, qmark_cmds)
 
+    def test_help_width_zero(self):
+        self.overrideEnv('BZR_COLUMNS', '0')
+        self.run_bzr('help commands')
+
     def test_hidden(self):
         help_commands = self.run_bzr('help commands')[0]
         help_hidden = self.run_bzr('help hidden-commands')[0]
@@ -161,12 +165,10 @@
     def test_help_with_aliases(self):
         original = self.run_bzr('help cat')[0]
 
-        ensure_config_dir_exists()
-        CONFIG=("[ALIASES]\n"
-        "c=cat\n"
-        "cat=cat\n")
-
-        open(config_filename(),'wb').write(CONFIG)
+        conf = config.GlobalConfig.from_string('''[ALIASES]
+c=cat
+cat=cat
+''', save=True)
 
         expected = original + "'bzr cat' is an alias for 'bzr cat'.\n"
         self.assertEqual(expected, self.run_bzr('help cat')[0])

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2010-04-14 09:57:01 +0000
+++ bzrlib/tests/blackbox/test_info.py	2011-10-14 13:56:45 +0000
@@ -22,6 +22,7 @@
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     info,
     osutils,
@@ -49,14 +50,14 @@
         transport = self.get_transport()
 
         # Create initial standalone branch
-        tree1 = self.make_branch_and_tree('standalone', 'weave')
+        tree1 = self.make_branch_and_tree('standalone', 'knit')
         self.build_tree(['standalone/a'])
         tree1.add('a')
         branch1 = tree1.branch
 
         out, err = self.run_bzr('info standalone')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: standalone
 """, out)
@@ -65,15 +66,15 @@
         # Standalone branch - verbose mode
         out, err = self.run_bzr('info standalone -v')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          0 unchanged
@@ -96,15 +97,15 @@
         # Standalone branch - really verbose mode
         out, err = self.run_bzr('info standalone -vv')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          0 unchanged
@@ -125,7 +126,7 @@
 """, out)
         self.assertEqual('', err)
         tree1.commit('commit one')
-        rev = branch1.repository.get_revision(branch1.revision_history()[0])
+        rev = branch1.repository.get_revision(branch1.last_revision())
         datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
 
         # Branch standalone with push location
@@ -134,7 +135,7 @@
 
         out, err = self.run_bzr('info branch')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: branch
 
@@ -146,7 +147,7 @@
 
         out, err = self.run_bzr('info branch --verbose')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: branch
 
@@ -155,10 +156,10 @@
   parent branch: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          1 unchanged
@@ -187,7 +188,7 @@
         branch1.bzrdir.sprout('bound')
         knit1_format = bzrdir.format_registry.make_bzrdir('knit')
         upgrade.upgrade('bound', knit1_format)
-        branch3 = bzrdir.BzrDir.open('bound').open_branch()
+        branch3 = controldir.ControlDir.open('bound').open_branch()
         branch3.bind(branch1)
         bound_tree = branch3.bzrdir.open_workingtree()
         out, err = self.run_bzr('info -v bound')
@@ -213,7 +214,7 @@
          0 removed
          0 renamed
          0 unknown
-         1 ignored
+         0 ignored
          0 versioned subdirectories
 
 Branch history:
@@ -232,7 +233,7 @@
         self.assertEqual('', err)
 
         # Checkout standalone (same as above, but does not have parent set)
-        branch4 = bzrdir.BzrDir.create_branch_convenience('checkout',
+        branch4 = controldir.ControlDir.create_branch_convenience('checkout',
             format=knit1_format)
         branch4.bind(branch1)
         branch4.bzrdir.open_workingtree().update()
@@ -276,6 +277,10 @@
         tree5 = branch1.create_checkout('lightcheckout', lightweight=True)
         branch5 = tree5.branch
         out, err = self.run_bzr('info -v lightcheckout')
+        if "metaweave" in bzrdir.format_registry:
+            format_description = "knit or metaweave"
+        else:
+            format_description = "knit"
         self.assertEqualDiff(
 """Lightweight checkout (format: %s)
 Location:
@@ -284,9 +289,9 @@
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 6
-        branch: Branch format 4
-    repository: Weave repository format 6
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          1 unchanged
@@ -306,20 +311,20 @@
 
 Repository:
          1 revision
-""" % (self._repo_strings, datestring_first, datestring_first,), out)
+""" % (format_description, datestring_first, datestring_first,), out)
         self.assertEqual('', err)
 
         # Update initial standalone branch
         self.build_tree(['standalone/b'])
         tree1.add('b')
         tree1.commit('commit two')
-        rev = branch1.repository.get_revision(branch1.revision_history()[-1])
+        rev = branch1.repository.get_revision(branch1.last_revision())
         datestring_last = osutils.format_date(rev.timestamp, rev.timezone)
 
         # Out of date branched standalone branch will not be detected
         out, err = self.run_bzr('info -v branch')
         self.assertEqualDiff(
-"""Standalone tree (format: weave)
+"""Standalone tree (format: knit)
 Location:
   branch root: branch
 
@@ -328,10 +333,10 @@
   parent branch: standalone
 
 Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: Weave repository format 6
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 In the working tree:
          1 unchanged
@@ -381,7 +386,7 @@
          0 removed
          0 renamed
          0 unknown
-         1 ignored
+         0 ignored
          0 versioned subdirectories
 
 Branch history:
@@ -446,9 +451,9 @@
 
 Format:
        control: Meta directory format 1
-  working tree: Working tree format 6
-        branch: Branch format 4
-    repository: Weave repository format 6
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: Knit repository format 1
 
 Working tree is out of date: missing 1 revision.
 
@@ -470,7 +475,7 @@
 
 Repository:
          2 revisions
-""" % (self._repo_strings, datestring_first, datestring_last,), out)
+""" % (format_description, datestring_first, datestring_last,), out)
         self.assertEqual('', err)
 
     def test_info_standalone_no_tree(self):
@@ -525,8 +530,8 @@
 
         # Create branch inside shared repository
         repo.bzrdir.root_transport.mkdir('branch')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch',
-            format=format)
+        branch1 = controldir.ControlDir.create_branch_convenience(
+            'repo/branch', format=format)
         out, err = self.run_bzr('info -v repo/branch')
         self.assertEqualDiff(
 """Repository branch (format: dirstate or knit)
@@ -567,7 +572,7 @@
         self.build_tree(['tree/lightcheckout/a'])
         tree2.add('a')
         tree2.commit('commit one')
-        rev = repo.get_revision(branch2.revision_history()[0])
+        rev = repo.get_revision(branch2.last_revision())
         datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
@@ -686,7 +691,7 @@
         tree3.commit('commit two')
 
         # Out of date lightweight checkout
-        rev = repo.get_revision(branch1.revision_history()[-1])
+        rev = repo.get_revision(branch1.last_revision())
         datestring_last = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
@@ -799,7 +804,7 @@
 
         # Create two branches
         repo.bzrdir.root_transport.mkdir('branch1')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch1',
+        branch1 = controldir.ControlDir.create_branch_convenience('repo/branch1',
             format=format)
         branch2 = branch1.bzrdir.sprout('repo/branch2').open_branch()
 
@@ -842,7 +847,7 @@
         tree1 = branch1.bzrdir.open_workingtree()
         tree1.add('a')
         tree1.commit('commit one')
-        rev = repo.get_revision(branch1.revision_history()[0])
+        rev = repo.get_revision(branch1.last_revision())
         datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info -v repo/branch1')
         self.assertEqualDiff(
@@ -1217,13 +1222,13 @@
                                     format=bzrdir.BzrDirMetaFormat1())
         repo.set_make_working_trees(False)
         repo.bzrdir.root_transport.mkdir('branch')
-        repo_branch = repo.bzrdir.create_branch_convenience('repo/branch',
-                                    format=bzrdir.BzrDirMetaFormat1())
+        repo_branch = controldir.ControlDir.create_branch_convenience(
+            'repo/branch', format=bzrdir.BzrDirMetaFormat1())
         # Do a heavy checkout
         transport.mkdir('tree')
         transport.mkdir('tree/checkout')
-        co_branch = bzrdir.BzrDir.create_branch_convenience('tree/checkout',
-            format=bzrdir.BzrDirMetaFormat1())
+        co_branch = controldir.ControlDir.create_branch_convenience(
+            'tree/checkout', format=bzrdir.BzrDirMetaFormat1())
         co_branch.bind(repo_branch)
         # Do a light checkout of the heavy one
         transport.mkdir('tree/lightcheckout')
@@ -1323,89 +1328,6 @@
             self.knownFailure('Win32 cannot run "bzr info"'
                               ' when the tree is locked.')
 
-    def test_info_locking_oslocks(self):
-        if sys.platform == "win32":
-            self.skip("don't use oslocks on win32 in unix manner")
-        # This test tests old (all-in-one, OS lock using) behaviour which
-        # simply cannot work on windows (and is indeed why we changed our
-        # design. As such, don't try to remove the thisFailsStrictLockCheck
-        # call here.
-        self.thisFailsStrictLockCheck()
-
-        tree = self.make_branch_and_tree('branch',
-                                         format=bzrdir.BzrDirFormat6())
-
-        # Test all permutations of locking the working tree, branch and repository
-        # XXX: Well not yet, as we can't query oslocks yet. Currently, it's
-        # implemented by raising NotImplementedError and get_physical_lock_status()
-        # always returns false. This makes bzr info hide the lock status.  (Olaf)
-        # W B R
-
-        # U U U
-        out, err = self.run_bzr('info -v branch')
-        self.assertEqualDiff(
-"""Standalone tree (format: weave)
-Location:
-  branch root: %s
-
-Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Repository:
-         0 revisions
-""" % ('branch', tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        # L L L
-        tree.lock_write()
-        out, err = self.run_bzr('info -v branch')
-        self.assertEqualDiff(
-"""Standalone tree (format: weave)
-Location:
-  branch root: %s
-
-Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Repository:
-         0 revisions
-""" % ('branch', tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        tree.unlock()
-
     def test_info_stacked(self):
         # We have a mainline
         trunk_tree = self.make_branch_and_tree('mainline',
@@ -1424,3 +1346,33 @@
      stacked on: mainline
 """, out)
         self.assertEqual("", err)
+
+    def test_info_revinfo_optional(self):
+        tree = self.make_branch_and_tree('.')
+        def last_revision_info(self):
+            raise errors.UnsupportedOperation(last_revision_info, self)
+        self.overrideAttr(
+            branch.Branch, "last_revision_info", last_revision_info)
+        out, err = self.run_bzr('info -v .')
+        self.assertEqual(
+"""Standalone tree (format: 2a)
+Location:
+  branch root: .
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 6
+        branch: Branch format 7
+    repository: Repository format 2a - rich roots, group compression and chk inventories
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+""", out)
+        self.assertEqual("", err)

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_init.py	2011-10-05 14:12:34 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,13 +15,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-"""Test "bzr init"""
+"""Test 'bzr init'"""
 
 import os
 import re
 
 from bzrlib import (
     branch as _mod_branch,
+    config as _mod_config,
     osutils,
     urlutils,
     )
@@ -46,21 +47,23 @@
         self.assertIsDirectory('.bzr/checkout', t)
         self.assertIsDirectory('.bzr/checkout/lock', t)
 
-    def test_init_weave(self):
-        # --format=weave should be accepted to allow interoperation with
-        # old releases when desired.
-        out, err = self.run_bzr('init --format=weave')
-        self.assertEqual("""Created a standalone tree (format: weave)\n""",
-            out)
-        self.assertEqual('', err)
-
     def test_init_format_2a(self):
-        """Smoke test for constructing a format 2a repoistory."""
+        """Smoke test for constructing a format 2a repository."""
         out, err = self.run_bzr('init --format=2a')
         self.assertEqual("""Created a standalone tree (format: 2a)\n""",
             out)
         self.assertEqual('', err)
 
+    def test_init_colocated(self):
+        """Smoke test for constructing a colocated branch."""
+        out, err = self.run_bzr('init --format=development-colo file:,branch=abranch')
+        self.assertEqual("""Created a standalone tree (format: development-colo)\n""",
+            out)
+        self.assertEqual('', err)
+        out, err = self.run_bzr('branches')
+        self.assertEqual(" abranch\n", out)
+        self.assertEqual('', err)
+
     def test_init_at_repository_root(self):
         # bzr init at the root of a repository should create a branch
         # and working tree even when creation of working trees is disabled.
@@ -107,7 +110,7 @@
         # init an existing branch.
         out, err = self.run_bzr('init subdir2', retcode=3)
         self.assertEqual('', out)
-        self.failUnless(err.startswith('bzr: ERROR: Already a branch:'))
+        self.assertTrue(err.startswith('bzr: ERROR: Already a branch:'))
 
     def test_init_branch_quiet(self):
         out, err = self.run_bzr('init -q')
@@ -161,7 +164,21 @@
         self.run_bzr_error(['Parent directory of ../new/tree does not exist'],
                             'init ../new/tree', working_dir='tree')
         self.run_bzr('init ../new/tree --create-prefix', working_dir='tree')
-        self.failUnlessExists('new/tree/.bzr')
+        self.assertPathExists('new/tree/.bzr')
+
+    def test_init_default_format_option(self):
+        """bzr init should read default format from option default_format"""
+        conf = _mod_config.GlobalConfig.from_string('''
+[DEFAULT]
+default_format = 1.9
+''', save=True)
+        out, err = self.run_bzr_subprocess('init')
+        self.assertContainsRe(out, '1.9')
+
+    def test_init_no_tree(self):
+        """'bzr init --no-tree' creates a branch with no working tree."""
+        out, err = self.run_bzr('init --no-tree')
+        self.assertStartsWith(out, 'Created a standalone branch')
 
 
 class TestSFTPInit(TestCaseWithSFTPServer):
@@ -196,10 +213,10 @@
     def test_init_append_revisions_only(self):
         self.run_bzr('init --dirstate-tags normal_branch6')
         branch = _mod_branch.Branch.open('normal_branch6')
-        self.assertEqual(None, branch._get_append_revisions_only())
+        self.assertEqual(None, branch.get_append_revisions_only())
         self.run_bzr('init --append-revisions-only --dirstate-tags branch6')
         branch = _mod_branch.Branch.open('branch6')
-        self.assertEqual(True, branch._get_append_revisions_only())
+        self.assertEqual(True, branch.get_append_revisions_only())
         self.run_bzr_error(['cannot be set to append-revisions-only'],
                            'init --append-revisions-only --knit knit')
 
@@ -211,8 +228,9 @@
         # and uses whoami only in a lock file. Without whoami the login name
         # is used. This test is to ensure that init passes even when whoami
         # is not available.
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
         out, err = self.run_bzr(['init', 'foo'])
         self.assertEqual(err, '')
         self.assertTrue(os.path.exists('foo'))
+

=== modified file 'bzrlib/tests/blackbox/test_join.py'
--- bzrlib/tests/blackbox/test_join.py	2010-04-28 07:53:34 +0000
+++ bzrlib/tests/blackbox/test_join.py	2011-06-14 02:21:41 +0000
@@ -74,13 +74,13 @@
         sub_tree.lock_read()
         self.addCleanup(sub_tree.unlock)
         self.assertEqual('file1-id', sub_tree.path2id('file1'))
-        self.assertTrue('file1-id' in sub_tree)
+        self.assertTrue(sub_tree.has_id('file1-id'))
         self.assertEqual('subtree-root-id', sub_tree.path2id(''))
         self.assertEqual('', sub_tree.id2path('subtree-root-id'))
         self.assertIs(None, base_tree.path2id('subtree/file1'))
         base_tree.lock_read()
         self.addCleanup(base_tree.unlock)
-        self.assertTrue('file1-id' not in base_tree)
+        self.assertFalse(base_tree.has_id('file1-id'))
         self.assertEqual('subtree-root-id', base_tree.path2id('subtree'))
         self.assertEqual('subtree', base_tree.id2path('subtree-root-id'))
 
@@ -95,5 +95,3 @@
                                 retcode=3)
         self.assertContainsRe(err, r"Can't join trees")
         self.assertContainsRe(err, r"use bzr upgrade")
-
-

=== modified file 'bzrlib/tests/blackbox/test_locale.py'
--- bzrlib/tests/blackbox/test_locale.py	2009-09-18 08:50:05 +0000
+++ bzrlib/tests/blackbox/test_locale.py	2011-05-15 14:35:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 """Test that bzr handles locales in a reasonable way"""
 
-import os
 import sys
 
 from bzrlib import (
@@ -44,10 +43,13 @@
 
     def test_log_C(self):
         self.disable_missing_extensions_warning()
+        # C is not necessarily the default locale, so set both LANG and LC_ALL
+        # explicitly because LC_ALL is preferred on (some?) Linux systems but
+        # only LANG is respected on Windows.
         out, err = self.run_bzr_subprocess(
             '--no-aliases --no-plugins log -q --log-format=long tree',
-               env_changes={'LANG':'C', 'BZR_PROGRESS_BAR':'none',
-                            'LC_ALL':None, 'LC_CTYPE':None, 'LANGUAGE':None})
+               env_changes={'LANG': 'C', 'BZR_PROGRESS_BAR':'none',
+                            'LC_ALL': 'C', 'LC_CTYPE':None, 'LANGUAGE':None})
         self.assertEqual('', err)
         self.assertEqualDiff("""\
 ------------------------------------------------------------
@@ -74,3 +76,14 @@
 message:
   Unicode ? commit
 """, out)
+
+
+class TestMultibyteCodecs(tests.TestCaseWithTransport):
+    """Tests for quirks of multibyte encodings and their python codecs"""
+
+    def test_plugins_mbcs(self):
+        """Ensure the plugins command works with cjkcodecs, see lp:754082"""
+        self.disable_missing_extensions_warning()
+        out, err = self.run_bzr(["plugins"], encoding="EUC-JP")
+        # The output is tested in bt.test_plugins rather than here
+        self.assertEqual("", err)

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2010-07-09 16:16:11 +0000
+++ bzrlib/tests/blackbox/test_log.py	2011-10-12 16:00:13 +0000
@@ -19,7 +19,6 @@
 
 from itertools import izip
 import os
-import re
 
 from bzrlib import (
     branchbuilder,
@@ -29,8 +28,8 @@
     tests,
     )
 from bzrlib.tests import (
-    script,
     test_log,
+    features,
     )
 
 
@@ -77,6 +76,8 @@
                 self.log_catcher = test_log.LogCatcher(*args, **kwargs)
                 # Always return our own log formatter
                 return self.log_catcher
+        # Break cycle with closure over self on cleanup by removing method
+        self.addCleanup(setattr, MyLogFormatter, "__new__", None)
 
         def getme(branch):
                 # Always return our own log formatter class hijacking the
@@ -88,8 +89,12 @@
     def get_captured_revisions(self):
         return self.log_catcher.revisions
 
-    def assertLogRevnos(self, args, expected_revnos, working_dir='.'):
-        self.run_bzr(['log'] + args, working_dir=working_dir)
+    def assertLogRevnos(self, args, expected_revnos, working_dir='.',
+                        out='', err=''):
+        actual_out, actual_err = self.run_bzr(['log'] + args,
+                                              working_dir=working_dir)
+        self.assertEqual(out, actual_out)
+        self.assertEqual(err, actual_err)
         self.assertEqual(expected_revnos,
                          [r.revno for r in self.get_captured_revisions()])
 
@@ -442,6 +447,25 @@
         self.assertContainsRe(log, r'tags: tag1')
 
 
+class TestLogSignatures(TestLog):
+
+    def test_log_with_signatures(self):
+        self.requireFeature(features.gpgme)
+
+        tree = self.make_linear_branch(format='dirstate-tags')
+
+        log = self.run_bzr("log --signatures")[0]
+        self.assertTrue('signature: no signature' in log)
+
+    def test_log_without_signatures(self):
+        self.requireFeature(features.gpgme)
+
+        tree = self.make_linear_branch(format='dirstate-tags')
+
+        log = self.run_bzr("log")[0]
+        self.assertFalse('signature: no signature' in log)
+
+
 class TestLogVerbose(TestLog):
 
     def setUp(self):
@@ -463,6 +487,9 @@
     def test_log_short_verbose(self):
         self.assertUseShortDeltaFormat(['log', '--short', '-v'])
 
+    def test_log_s_verbose(self):
+        self.assertUseShortDeltaFormat(['log', '-S', '-v'])
+
     def test_log_short_verbose_verbose(self):
         self.assertUseLongDeltaFormat(['log', '--short', '-vv'])
 
@@ -513,19 +540,30 @@
 
     def test_include_merges(self):
         # Confirm --include-merges gives the same output as -n0
-        self.assertLogRevnos(['--include-merges'],
-                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
-                             working_dir='level0')
-        self.assertLogRevnos(['--include-merges'],
-                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
-                             working_dir='level0')
+        msg = ("The option '--include-merges' to 'bzr log' "
+               "has been deprecated in bzr 2.5. "
+               "Please use '--include-merged' instead.\n")
+        self.assertLogRevnos(['--include-merges'],
+                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
+                             working_dir='level0', err=msg)
+        self.assertLogRevnos(['--include-merges'],
+                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
+                             working_dir='level0', err=msg)
         out_im, err_im = self.run_bzr('log --include-merges',
                                       working_dir='level0')
         out_n0, err_n0 = self.run_bzr('log -n0', working_dir='level0')
-        self.assertEqual('', err_im)
+        self.assertEqual(msg, err_im)
         self.assertEqual('', err_n0)
         self.assertEqual(out_im, out_n0)
 
+    def test_include_merged(self):
+        # Confirm --include-merged gives the same output as -n0
+        expected = ['2', '1.1.2', '1.2.1', '1.1.1', '1']
+        self.assertLogRevnos(['--include-merged'],
+                             expected, working_dir='level0')
+        self.assertLogRevnos(['--include-merged'],
+                             expected, working_dir='level0')
+
     def test_force_merge_revisions_N(self):
         self.assertLogRevnos(['-n2'],
                              ['2', '1.1.2', '1.1.1', '1'],
@@ -549,6 +587,14 @@
                 [('2', 0), ('1.1.2', 1), ('1.2.1', 2)],
                 working_dir='level0')
 
+    def test_omit_merges_with_sidelines(self):
+        self.assertLogRevnos(['--omit-merges', '-n0'], ['1.2.1', '1.1.1', '1'],
+                             working_dir='level0')
+
+    def test_omit_merges_without_sidelines(self):
+        self.assertLogRevnos(['--omit-merges', '-n1'], ['1'],
+                             working_dir='level0')
+
 
 class TestLogDiff(TestLogWithLogCatcher):
 
@@ -924,3 +970,85 @@
         self.prepare_tree()
         os.chdir("dir1")
         self.assertLogRevnos(['dir2', 'file5'], ['5', '3'])
+
+
+class MainlineGhostTests(TestLogWithLogCatcher):
+
+    def setUp(self):
+        super(MainlineGhostTests, self).setUp()
+        tree = self.make_branch_and_tree('')
+        tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
+        tree.add('')
+        tree.commit('msg1', rev_id='rev1')
+        tree.commit('msg2', rev_id='rev2')
+
+    def test_log_range(self):
+        self.assertLogRevnos(["-r1..2"], ["2", "1"])
+
+    def test_log_norange(self):
+        self.assertLogRevnos([], ["2", "1"])
+
+    def test_log_range_open_begin(self):
+        self.knownFailure("log with ghosts fails. bug #726466")
+        (stdout, stderr) = self.run_bzr(['log', '-r..2'], retcode=3)
+        self.assertEqual(["2", "1"],
+                         [r.revno for r in self.get_captured_revisions()])
+        self.assertEquals("bzr: ERROR: Further revision history missing.", stderr)
+
+    def test_log_range_open_end(self):
+        self.assertLogRevnos(["-r1.."], ["2", "1"])
+
+class TestLogMatch(TestLogWithLogCatcher):
+    def prepare_tree(self):
+        tree = self.make_branch_and_tree('')
+        self.build_tree(
+            ['/hello.txt', '/goodbye.txt'])
+        tree.add('hello.txt')
+        tree.commit(message='message1', committer='committer1', authors=['author1'])
+        tree.add('goodbye.txt')
+        tree.commit(message='message2', committer='committer2', authors=['author2'])
+    
+    def test_message(self):
+        self.prepare_tree()
+        self.assertLogRevnos(["-m", "message1"], ["1"])
+        self.assertLogRevnos(["-m", "message2"], ["2"])
+        self.assertLogRevnos(["-m", "message"], ["2", "1"])
+        self.assertLogRevnos(["-m", "message1", "-m", "message2"], ["2", "1"])
+        self.assertLogRevnos(["--match-message", "message1"], ["1"])
+        self.assertLogRevnos(["--match-message", "message2"], ["2"])
+        self.assertLogRevnos(["--match-message", "message"], ["2", "1"])
+        self.assertLogRevnos(["--match-message", "message1", 
+                              "--match-message", "message2"], ["2", "1"])
+        self.assertLogRevnos(["--message", "message1"], ["1"])
+        self.assertLogRevnos(["--message", "message2"], ["2"])
+        self.assertLogRevnos(["--message", "message"], ["2", "1"])
+        self.assertLogRevnos(["--match-message", "message1", 
+                              "--message", "message2"], ["2", "1"])
+        self.assertLogRevnos(["--message", "message1", 
+                              "--match-message", "message2"], ["2", "1"])
+
+    def test_committer(self):
+        self.prepare_tree()
+        self.assertLogRevnos(["-m", "committer1"], ["1"])
+        self.assertLogRevnos(["-m", "committer2"], ["2"])
+        self.assertLogRevnos(["-m", "committer"], ["2", "1"])
+        self.assertLogRevnos(["-m", "committer1", "-m", "committer2"], 
+                             ["2", "1"])
+        self.assertLogRevnos(["--match-committer", "committer1"], ["1"])
+        self.assertLogRevnos(["--match-committer", "committer2"], ["2"])
+        self.assertLogRevnos(["--match-committer", "committer"], ["2", "1"])
+        self.assertLogRevnos(["--match-committer", "committer1", 
+                              "--match-committer", "committer2"], ["2", "1"])
+
+    def test_author(self):
+        self.prepare_tree()
+        self.assertLogRevnos(["-m", "author1"], ["1"])
+        self.assertLogRevnos(["-m", "author2"], ["2"])
+        self.assertLogRevnos(["-m", "author"], ["2", "1"])
+        self.assertLogRevnos(["-m", "author1", "-m", "author2"], 
+                             ["2", "1"])
+        self.assertLogRevnos(["--match-author", "author1"], ["1"])
+        self.assertLogRevnos(["--match-author", "author2"], ["2"])
+        self.assertLogRevnos(["--match-author", "author"], ["2", "1"])
+        self.assertLogRevnos(["--match-author", "author1", 
+                              "--match-author", "author2"], ["2", "1"])

=== modified file 'bzrlib/tests/blackbox/test_logformats.py'
--- bzrlib/tests/blackbox/test_logformats.py	2009-05-05 15:31:34 +0000
+++ bzrlib/tests/blackbox/test_logformats.py	2011-05-19 01:37:00 +0000
@@ -107,3 +107,18 @@
 
 """
         self.assertEqualDiff(expected, log)
+
+    def test_logformat_line_wide(self):
+        """Author field should get larger for column widths over 80"""
+        wt = self.make_branch_and_tree('.')
+        wt.commit('revision with a long author', committer='Person with' 
+                  ' long name SENTINEL')
+        log, err = self.run_bzr('log --line')
+        self.assertNotContainsString(log, 'SENTINEL')
+        self.overrideEnv('BZR_COLUMNS', '116')
+        log, err = self.run_bzr('log --line')
+        self.assertContainsString(log, 'SENT...')
+        self.overrideEnv('BZR_COLUMNS', '0')
+        log, err = self.run_bzr('log --line')
+        self.assertContainsString(log, 'SENTINEL')
+

=== modified file 'bzrlib/tests/blackbox/test_lsprof.py'
--- bzrlib/tests/blackbox/test_lsprof.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_lsprof.py	2011-06-14 01:26:41 +0000
@@ -15,12 +15,12 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from bzrlib import tests
-from bzrlib.tests import test_lsprof
+from bzrlib.tests import features
 
 
 class TestLSProf(tests.TestCaseInTempDir):
 
-    _test_needs_features = [test_lsprof.LSProfFeature]
+    _test_needs_features = [features.lsprof_feature]
 
     def test_file(self):
         out, err = self.run_bzr('--lsprof-file output.callgrind rocks')

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2011-08-09 13:30:21 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,19 +19,28 @@
 """Black-box tests for bzr merge.
 """
 
+import doctest
 import os
 
+from testtools import matchers
+
 from bzrlib import (
     branch,
     bzrdir,
     conflicts,
-    errors,
     merge_directive,
     osutils,
     tests,
     urlutils,
     workingtree,
     )
+from bzrlib.tests import (
+    scenarios,
+    script,
+    )
+
+
+load_tests = scenarios.load_tests_apply_scenarios
 
 
 class TestMerge(tests.TestCaseWithTransport):
@@ -102,7 +111,7 @@
         # Merging a branch pulls its revision into the tree
         b = branch.Branch.open('../b')
         b_tip = b.last_revision()
-        self.failUnless(a.branch.repository.has_revision(b_tip))
+        self.assertTrue(a.branch.repository.has_revision(b_tip))
         self.assertEqual([a_tip, b_tip], a.get_parent_ids())
         a_tree.revert(backups=False)
         out, err = self.run_bzr('merge -r revno:1:./hello', retcode=3)
@@ -204,12 +213,12 @@
         b_tree.commit(message='Modified a.txt')
         os.chdir('b')
         self.run_bzr('merge ../a/', retcode=1)
-        self.failUnlessExists('sub/a.txt.THIS')
-        self.failUnlessExists('sub/a.txt.BASE')
+        self.assertPathExists('sub/a.txt.THIS')
+        self.assertPathExists('sub/a.txt.BASE')
         os.chdir('../a')
         self.run_bzr('merge ../b/', retcode=1)
-        self.failUnlessExists('sub/a.txt.OTHER')
-        self.failUnlessExists('sub/a.txt.BASE')
+        self.assertPathExists('sub/a.txt.OTHER')
+        self.assertPathExists('sub/a.txt.BASE')
 
     def test_conflict_leaves_base_this_other_files(self):
         tree, other = self.create_conflicting_branches()
@@ -326,14 +335,14 @@
         tree_a.add(['file_1', 'file_2'])
         tree_a.commit('commit 1')
         tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
-        self.failUnlessExists('b/file_1')
+        self.assertPathExists('b/file_1')
         tree_a.rename_one('file_1', 'file_i')
         tree_a.commit('commit 2')
         tree_a.rename_one('file_2', 'file_ii')
         ## os.chdir('b')
         self.run_bzr('merge a --uncommitted -d b')
-        self.failUnlessExists('b/file_1')
-        self.failUnlessExists('b/file_ii')
+        self.assertPathExists('b/file_1')
+        self.assertPathExists('b/file_ii')
         tree_b.revert()
         self.run_bzr_error(('Cannot use --uncommitted and --revision',),
                            'merge /a --uncommitted -r1 -d b')
@@ -347,18 +356,29 @@
         tree_a.add(['file1', 'file2'])
         os.chdir('tree_b')
         self.run_bzr(['merge', '--uncommitted', '../tree_a/file1'])
-        self.failUnlessExists('file1')
-        self.failIfExists('file2')
+        self.assertPathExists('file1')
+        self.assertPathDoesNotExist('file2')
+
+    def test_merge_nonexistent_file(self):
+        """It should not be possible to merge changes from a file which
+        does not exist."""
+        tree_a = self.make_branch_and_tree('tree_a')
+        self.build_tree_contents([('tree_a/file', 'bar\n')])
+        tree_a.add(['file'])
+        tree_a.commit('commit 1')
+        os.chdir('tree_a')
+        self.run_bzr_error(('Path\(s\) do not exist: non/existing',),
+                           ['merge', 'non/existing'])
 
     def pullable_branch(self):
         tree_a = self.make_branch_and_tree('a')
-        self.build_tree(['a/file'])
+        self.build_tree_contents([('a/file', 'bar\n')])
         tree_a.add(['file'])
         self.id1 = tree_a.commit('commit 1')
 
         tree_b = self.make_branch_and_tree('b')
         tree_b.pull(tree_a.branch)
-        file('b/file', 'wb').write('foo')
+        self.build_tree_contents([('b/file', 'foo\n')])
         self.id2 = tree_b.commit('commit 2')
 
     def test_merge_pull(self):
@@ -369,6 +389,21 @@
         tree_a = workingtree.WorkingTree.open('.')
         self.assertEqual([self.id2], tree_a.get_parent_ids())
 
+    def test_merge_pull_preview(self):
+        self.pullable_branch()
+        (out, err) = self.run_bzr('merge --pull --preview -d a b')
+        self.assertThat(out, matchers.DocTestMatches(
+"""=== modified file 'file'
+--- file\t...
++++ file\t...
+@@ -1,1 +1,1 @@
+-bar
++foo
+
+""", doctest.ELLIPSIS | doctest.REPORT_UDIFF))
+        tree_a = workingtree.WorkingTree.open('a')
+        self.assertEqual([self.id1], tree_a.get_parent_ids())
+
     def test_merge_kind_change(self):
         tree_a = self.make_branch_and_tree('tree_a')
         self.build_tree_contents([('tree_a/file', 'content_1')])
@@ -408,8 +443,8 @@
         self.write_directive('directive', source.branch, 'target', 'rev2',
                              'rev1')
         out, err = self.run_bzr('merge -d target directive')
-        self.failIfExists('target/a')
-        self.failUnlessExists('target/b')
+        self.assertPathDoesNotExist('target/a')
+        self.assertPathExists('target/b')
         self.assertContainsRe(err, 'Performing cherrypick')
 
     def write_directive(self, filename, source, target, revision_id,
@@ -450,19 +485,19 @@
         branch_b.add('file2')
         branch_b.commit('added file2', rev_id='rev2b')
         branch_b.merge_from_branch(branch_a.branch)
-        self.failUnlessExists('branch_b/file1')
+        self.assertPathExists('branch_b/file1')
         branch_b.commit('merged branch_a', rev_id='rev3b')
 
         # It works if the revid has an interger revno
         self.run_bzr('merge -d target -r revid:rev2a branch_a')
-        self.failUnlessExists('target/file1')
-        self.failIfExists('target/file2')
+        self.assertPathExists('target/file1')
+        self.assertPathDoesNotExist('target/file2')
         target.revert()
 
         # It should work if the revid has no integer revno
         self.run_bzr('merge -d target -r revid:rev2a branch_b')
-        self.failUnlessExists('target/file1')
-        self.failIfExists('target/file2')
+        self.assertPathExists('target/file1')
+        self.assertPathDoesNotExist('target/file2')
 
     def assertDirectoryContent(self, directory, entries, message=''):
         """Assert whether entries (file or directories) exist in a directory.
@@ -516,6 +551,7 @@
 
     def test_merge_from_submit(self):
         tree_a = self.make_branch_and_tree('a')
+        tree_a.commit('test')
         tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
         tree_c = tree_a.bzrdir.sprout('c').open_workingtree()
         out, err = self.run_bzr(['merge', '-d', 'c'])
@@ -526,6 +562,7 @@
 
     def test_remember_sets_submit(self):
         tree_a = self.make_branch_and_tree('a')
+        tree_a.commit('rev1')
         tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
         self.assertIs(tree_b.branch.get_submit_branch(), None)
 
@@ -538,6 +575,23 @@
         self.assertEqual(tree_b.branch.get_submit_branch(),
                          tree_a.bzrdir.root_transport.base)
 
+    def test_no_remember_dont_set_submit(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree_contents([('a/file', "a\n")])
+        tree_a.add('file')
+        tree_a.commit('rev1')
+        tree_b = tree_a.bzrdir.sprout('b').open_workingtree()
+        self.assertIs(tree_b.branch.get_submit_branch(), None)
+
+        # Remember should not happen if using default from parent
+        out, err = self.run_bzr(['merge', '-d', 'b', '--no-remember'])
+        self.assertEquals(None, tree_b.branch.get_submit_branch())
+
+        # Remember should not happen if user supplies location but ask for not
+        # remembering it
+        out, err = self.run_bzr(['merge', '-d', 'b', '--no-remember', 'a'])
+        self.assertEqual(None, tree_b.branch.get_submit_branch())
+
     def test_weave_cherrypick(self):
         this_tree = self.make_branch_and_tree('this')
         self.build_tree_contents([('this/file', "a\n")])
@@ -603,28 +657,71 @@
         other.add('other_file')
         other.commit('rev1b')
         self.run_bzr('merge -d this other -r0..')
-        self.failUnlessExists('this/other_file')
+        self.assertPathExists('this/other_file')
 
     def test_merge_interactive_unlocks_branch(self):
         this = self.make_branch_and_tree('this')
-        other = self.make_branch_and_tree('other')
-        other.commit('empty commit')
+        this.commit('empty commit')
+        other = this.bzrdir.sprout('other').open_workingtree()
+        other.commit('empty commit 2')
         self.run_bzr('merge -i -d this other')
         this.lock_write()
         this.unlock()
 
-    def test_merge_reversed_revision_range(self):
-        tree = self.make_branch_and_tree(".")
+    def test_merge_fetches_tags(self):
+        """Tags are updated by merge, and revisions named in those tags are
+        fetched.
+        """
+        # Make a source, sprout a target off it
+        builder = self.make_branch_builder('source')
+        builder.build_commit(message="Rev 1", rev_id='rev-1')
+        source = builder.get_branch()
+        target_bzrdir = source.bzrdir.sprout('target')
+        # Add a non-ancestry tag to source
+        builder.build_commit(message="Rev 2a", rev_id='rev-2a')
+        source.tags.set_tag('tag-a', 'rev-2a')
+        source.set_last_revision_info(1, 'rev-1')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        builder.build_commit(message="Rev 2b", rev_id='rev-2b')
+        # Merge from source
+        self.run_bzr('merge -d target source')
+        target = target_bzrdir.open_branch()
+        # The tag is present, and so is its revision.
+        self.assertEqual('rev-2a', target.tags.lookup_tag('tag-a'))
+        target.repository.get_revision('rev-2a')
+
+
+class TestMergeRevisionRange(tests.TestCaseWithTransport):
+
+    scenarios = (('whole-tree', dict(context='.')),
+                 ('file-only', dict(context='a')))
+
+    def setUp(self):
+        super(TestMergeRevisionRange, self).setUp()
+        self.tree = self.make_branch_and_tree(".")
+        self.tree.commit('initial commit')
         for f in ("a", "b"):
             self.build_tree([f])
-            tree.add(f)
-            tree.commit("added "+f)
-        for context in (".", "", "a"):
-            self.run_bzr("merge -r 1..0 " + context)
-            self.failIfExists("a")
-            tree.revert()
-            self.failUnlessExists("a")
-
+            self.tree.add(f)
+            self.tree.commit("added " + f)
+
+    def test_merge_reversed_revision_range(self):
+        self.run_bzr("merge -r 2..1 " + self.context)
+        self.assertPathDoesNotExist("a")
+        self.assertPathExists("b")
+
+
+class TestMergeScript(script.TestCaseWithTransportAndScript):
+    def test_merge_empty_branch(self):
+        source = self.make_branch_and_tree('source')
+        self.build_tree(['source/a'])
+        source.add('a')
+        source.commit('Added a', rev_id='rev1')
+        target = self.make_branch_and_tree('target')
+        self.run_script("""\
+$ bzr merge -d target source
+2>bzr: ERROR: Merging into empty branches not currently supported, https://bugs.launchpad.net/bzr/+bug/308562
+""")
 
 class TestMergeForce(tests.TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	2010-06-17 09:23:19 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2011-09-16 15:37:58 +0000
@@ -49,7 +49,7 @@
         self.assertEqual('', err)
 
     def test_missing(self):
-        missing = "You are missing 1 revision(s):"
+        missing = "You are missing 1 revision:"
 
         # create a source branch
         a_tree = self.make_branch_and_tree('a')
@@ -92,13 +92,13 @@
 
         # compare again, but now we have the 'merge' commit extra
         lines = self.run_bzr('missing ../b', retcode=1)[0].splitlines()
-        self.assertEqual("You have 1 extra revision(s):", lines[0])
+        self.assertEqual("You have 1 extra revision:", lines[0])
         self.assertEqual(8, len(lines))
         lines2 = self.run_bzr('missing ../b --mine-only', retcode=1)[0]
         lines2 = lines2.splitlines()
         self.assertEqual(lines, lines2)
         lines3 = self.run_bzr('missing ../b --theirs-only', retcode=0)[0]
-        self.assertEqualDiff('Other branch is up to date.\n', lines3)
+        self.assertEqualDiff('Other branch has no new revisions.\n', lines3)
 
         # relative to a, missing the 'merge' commit
         os.chdir('../b')
@@ -109,10 +109,13 @@
         lines2 = lines2.splitlines()
         self.assertEqual(lines, lines2)
         lines3 = self.run_bzr('missing ../a --mine-only', retcode=0)[0]
-        self.assertEqualDiff('This branch is up to date.\n', lines3)
+        self.assertEqualDiff('This branch has no new revisions.\n', lines3)
         lines4 = self.run_bzr('missing ../a --short', retcode=1)[0]
         lines4 = lines4.splitlines()
         self.assertEqual(4, len(lines4))
+        lines4a = self.run_bzr('missing ../a -S', retcode=1)[0]
+        lines4a = lines4a.splitlines()
+        self.assertEqual(lines4, lines4a)
         lines5 = self.run_bzr('missing ../a --line', retcode=1)[0]
         lines5 = lines5.splitlines()
         self.assertEqual(2, len(lines5))
@@ -128,7 +131,7 @@
         self.assertEqual("  a", lines8[-1])
 
         os.chdir('../a')
-        self.assertEqualDiff('Other branch is up to date.\n',
+        self.assertEqualDiff('Other branch has no new revisions.\n',
                              self.run_bzr('missing ../b --theirs-only')[0])
 
         # after a pull we're back on track
@@ -139,9 +142,9 @@
         self.assertEqualDiff('Branches are up to date.\n',
                              self.run_bzr('missing ../a')[0])
         # If you supply mine or theirs you only know one side is up to date
-        self.assertEqualDiff('This branch is up to date.\n',
+        self.assertEqualDiff('This branch has no new revisions.\n',
                              self.run_bzr('missing ../a --mine-only')[0])
-        self.assertEqualDiff('Other branch is up to date.\n',
+        self.assertEqualDiff('Other branch has no new revisions.\n',
                              self.run_bzr('missing ../a --theirs-only')[0])
 
     def test_missing_filtered(self):
@@ -214,7 +217,7 @@
         b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
         self.build_tree_contents([('b/a', 'initial\nmore\n')])
         b_tree.commit(message='more')
-        
+
         out2, err2 = self.run_bzr('missing --directory a b', retcode=1)
         os.chdir('a')
         out1, err1 = self.run_bzr('missing ../b', retcode=1)

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2011-07-11 06:47:32 +0000
@@ -25,9 +25,12 @@
     )
 
 from bzrlib.tests import (
+    TestCaseWithTransport,
+    )
+from bzrlib.tests.features import (
     CaseInsensitiveFilesystemFeature,
     SymlinkFeature,
-    TestCaseWithTransport,
+    UnicodeFilenameFeature,
     )
 
 
@@ -35,10 +38,10 @@
 
     def assertMoved(self,from_path,to_path):
         """Assert that to_path is existing and versioned but from_path not. """
-        self.failIfExists(from_path)
+        self.assertPathDoesNotExist(from_path)
         self.assertNotInWorkingTree(from_path)
 
-        self.failUnlessExists(to_path)
+        self.assertPathExists(to_path)
         self.assertInWorkingTree(to_path)
 
     def test_mv_modes(self):
@@ -119,7 +122,7 @@
 
         os.chdir('sub1/sub2')
         self.run_bzr('mv ../hello.txt .')
-        self.failUnlessExists('./hello.txt')
+        self.assertPathExists('./hello.txt')
 
         os.chdir('..')
         self.run_bzr('mv sub2/hello.txt .')
@@ -229,8 +232,8 @@
             ["^bzr: ERROR: Could not move a => b. b is already versioned\.$"],
             'mv a b')
         #check that nothing changed
-        self.failIfExists('a')
-        self.failUnlessExists('b')
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists('b')
 
     def test_mv_already_moved_file_into_subdir(self):
         """Test bzr mv original_file to versioned_directory/file.
@@ -264,8 +267,8 @@
         self.run_bzr_error(
             ["^bzr: ERROR: Could not move a => a: sub is not versioned\.$"],
             'mv a sub/a')
-        self.failIfExists('a')
-        self.failUnlessExists('sub/a')
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists('sub/a')
 
     def test_mv_already_moved_files_into_subdir(self):
         """Test bzr mv original_files to versioned_directory.
@@ -300,10 +303,10 @@
         self.run_bzr_error(
             ["^bzr: ERROR: Could not move to sub. sub is not versioned\.$"],
             'mv a1 a2 sub')
-        self.failIfExists('a1')
-        self.failUnlessExists('sub/a1')
-        self.failUnlessExists('a2')
-        self.failIfExists('sub/a2')
+        self.assertPathDoesNotExist('a1')
+        self.assertPathExists('sub/a1')
+        self.assertPathExists('a2')
+        self.assertPathDoesNotExist('sub/a2')
 
     def test_mv_already_moved_file_forcing_after(self):
         """Test bzr mv versioned_file to unversioned_file.
@@ -325,8 +328,8 @@
              " \(Use --after to tell bzr about a rename that has already"
              " happened\)$"],
             'mv a b')
-        self.failUnlessExists('a')
-        self.failUnlessExists('b')
+        self.assertPathExists('a')
+        self.assertPathExists('b')
 
     def test_mv_already_moved_file_using_after(self):
         """Test bzr mv --after versioned_file to unversioned_file.
@@ -346,9 +349,9 @@
         self.build_tree(['a']) #touch a
 
         self.run_bzr('mv a b --after')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.assertNotInWorkingTree('a')#a should be unknown now.
-        self.failUnlessExists('b')
+        self.assertPathExists('b')
         self.assertInWorkingTree('b')
 
     def test_mv_already_moved_files_forcing_after(self):
@@ -375,10 +378,10 @@
              " exist. \(Use --after to tell bzr about a rename that has already"
              " happened\)$"],
             'mv a1 a2 sub')
-        self.failUnlessExists('a1')
-        self.failUnlessExists('a2')
-        self.failUnlessExists('sub/a1')
-        self.failUnlessExists('sub/a2')
+        self.assertPathExists('a1')
+        self.assertPathExists('a2')
+        self.assertPathExists('sub/a1')
+        self.assertPathExists('sub/a2')
 
     def test_mv_already_moved_files_using_after(self):
         """Test bzr mv --after versioned_file to directory/unversioned_file.
@@ -402,10 +405,10 @@
         self.build_tree(['a2']) #touch a2
 
         self.run_bzr('mv a1 a2 sub --after')
-        self.failUnlessExists('a1')
-        self.failUnlessExists('a2')
-        self.failUnlessExists('sub/a1')
-        self.failUnlessExists('sub/a2')
+        self.assertPathExists('a1')
+        self.assertPathExists('a2')
+        self.assertPathExists('sub/a1')
+        self.assertPathExists('sub/a2')
         self.assertInWorkingTree('sub/a1')
         self.assertInWorkingTree('sub/a2')
 
@@ -421,15 +424,15 @@
         osutils.rename('c', 'd')
         # mv a b should work just like it does for already renamed files
         self.run_bzr('mv a b')
-        self.failIfExists('a')
+        self.assertPathDoesNotExist('a')
         self.assertNotInWorkingTree('a')
-        self.failUnlessExists('b')
+        self.assertPathExists('b')
         self.assertInWorkingTree('b')
         # and --after should work, too (technically it's ignored)
         self.run_bzr('mv --after c d')
-        self.failIfExists('c')
+        self.assertPathDoesNotExist('c')
         self.assertNotInWorkingTree('c')
-        self.failUnlessExists('d')
+        self.assertPathExists('d')
         self.assertInWorkingTree('d')
 
     def make_abcd_tree(self):
@@ -504,3 +507,11 @@
         # If this fails, the tree is trying to acquire a branch lock, which it
         # shouldn't.
         self.run_bzr(['mv', 'tree/path', 'tree/path2'])
+
+    def test_mv_unversioned_non_ascii(self):
+        """Clear error on mv of an unversioned non-ascii file, see lp:707954"""
+        self.requireFeature(UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree([u"\xA7"])
+        out, err = self.run_bzr_error(["Could not rename", "not versioned"],
+            ["mv", u"\xA7", "b"])

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2010-04-20 14:44:01 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2011-09-02 00:53:06 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,24 +25,24 @@
     urlutils,
     )
 from bzrlib.tests import EncodingAdapter
-
-
-def load_tests(standard_tests, module, loader):
-    return tests.multiply_tests(standard_tests,
-                                EncodingAdapter.encoding_scenarios,
-                                loader.suiteClass())
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestNonAscii(tests.TestCaseWithTransport):
     """Test that bzr handles files/committers/etc which are non-ascii."""
 
+    scenarios = EncodingAdapter.encoding_scenarios
+
     def setUp(self):
         super(TestNonAscii, self).setUp()
         self._check_can_encode_paths()
 
         self.overrideAttr(osutils, '_cached_user_encoding', self.encoding)
         email = self.info['committer'] + ' <joe@foo.com>'
-        os.environ['BZR_EMAIL'] = email.encode(osutils.get_user_encoding())
+        self.overrideEnv('BZR_EMAIL', email.encode(osutils.get_user_encoding()))
         self.create_base()
 
     def run_bzr_decode(self, args, encoding=None, fail=False, retcode=None,
@@ -89,7 +89,7 @@
             normal_thing = unicodedata.normalize('NFD', path)
             mac_encoded = normal_thing.encode(fs_enc)
             if mac_encoded != encoded:
-                raise tests.KnownFailure(
+                self.knownFailure(
                     'Unable to roundtrip path %r on OSX filesystem'
                     ' using encoding "%s"'
                     % (path, fs_enc))
@@ -161,12 +161,12 @@
     def test_cat_revision(self):
         committer = self.info['committer']
         txt = self.run_bzr_decode('cat-revision -r 1')
-        self.failUnless(committer in txt,
+        self.assertTrue(committer in txt,
                         'failed to find %r in %r' % (committer, txt))
 
         msg = self.info['message']
         txt = self.run_bzr_decode('cat-revision -r 2')
-        self.failUnless(msg in txt, 'failed to find %r in %r' % (msg, txt))
+        self.assertTrue(msg in txt, 'failed to find %r in %r' % (msg, txt))
 
     def test_mkdir(self):
         txt = self.run_bzr_decode(['mkdir', self.info['directory']])
@@ -222,8 +222,8 @@
 
         txt = self.run_bzr_decode(['mv', 'a', fname2])
         self.assertEqual(u'a => %s\n' % fname2, txt)
-        self.failIfExists('a')
-        self.failUnlessExists(fname2)
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists(fname2)
 
         # After 'mv' we need to re-open the working tree
         self.wt = self.wt.bzrdir.open_workingtree()
@@ -240,7 +240,7 @@
         # The rename should still succeed
         newpath = u'%s/%s' % (dirname, fname2)
         txt = self.run_bzr_decode(['mv', newpath, 'a'], encoding='ascii')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.assertEqual(newpath.encode('ascii', 'replace') + ' => a\n', txt)
 
     def test_branch(self):
@@ -267,7 +267,7 @@
 
         expected = osutils.pathjoin(osutils.getcwd(), dirname1)
         self.assertEqual(u'Using saved parent location: %s/\n'
-                'No revisions to pull.\n' % (expected,), txt)
+                'No revisions or tags to pull.\n' % (expected,), txt)
 
         self.build_tree_contents(
             [(osutils.pathjoin(dirname1, 'a'), 'and yet more\n')])
@@ -386,7 +386,7 @@
         self.assertEqual(fname+'\n', txt)
 
         txt = self.run_bzr_decode('deleted --show-ids')
-        self.failUnless(txt.startswith(fname))
+        self.assertTrue(txt.startswith(fname))
 
         # Deleted should fail if cannot decode
         # Because it is giving the exact paths
@@ -421,7 +421,7 @@
         self.wt.bzrdir.sprout(url)
 
         txt = self.run_bzr_decode('root', working_dir=dirname)
-        self.failUnless(txt.endswith(dirname+'\n'))
+        self.assertTrue(txt.endswith(dirname+'\n'))
 
         txt = self.run_bzr_decode('root', encoding='ascii', fail=True,
                                   working_dir=dirname)

=== modified file 'bzrlib/tests/blackbox/test_pack.py'
--- bzrlib/tests/blackbox/test_pack.py	2010-04-26 13:51:08 +0000
+++ bzrlib/tests/blackbox/test_pack.py	2011-05-13 20:44:45 +0000
@@ -38,7 +38,9 @@
 
     def _update_file(self, path, text, checkin=True):
         """append text to file 'path' and check it in"""
-        open(path, 'a').write(text)
+        with open(path, 'a') as f:
+            f.write(text)
+
         if checkin:
             self.run_bzr(['ci', path, '-m', '"' + path + '"'])
 

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2011-10-14 13:56:45 +0000
@@ -23,18 +23,22 @@
 from bzrlib import (
     debug,
     remote,
+    tests,
     urlutils,
     )
 
 from bzrlib.branch import Branch
 from bzrlib.directory_service import directories
 from bzrlib.osutils import pathjoin
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import (
+    fixtures,
+    script,
+    )
 from bzrlib.uncommit import uncommit
 from bzrlib.workingtree import WorkingTree
 
 
-class TestPull(TestCaseWithTransport):
+class TestPull(tests.TestCaseWithTransport):
 
     def example_branch(self, path='.'):
         tree = self.make_branch_and_tree(path)
@@ -50,6 +54,7 @@
     def test_pull(self):
         """Pull changes from one branch to another."""
         a_tree = self.example_branch('a')
+        base_rev = a_tree.branch.last_revision()
         os.chdir('a')
         self.run_bzr('pull', retcode=3)
         self.run_bzr('missing', retcode=3)
@@ -68,16 +73,17 @@
         self.run_bzr('pull')
         os.mkdir('subdir')
         b_tree.add('subdir')
-        b_tree.commit(message='blah', allow_pointless=True)
+        new_rev = b_tree.commit(message='blah', allow_pointless=True)
 
         os.chdir('..')
         a = Branch.open('a')
         b = Branch.open('b')
-        self.assertEqual(a.revision_history(), b.revision_history()[:-1])
+        self.assertEqual(a.last_revision(), base_rev)
+        self.assertEqual(b.last_revision(), new_rev)
 
         os.chdir('a')
         self.run_bzr('pull ../b')
-        self.assertEqual(a.revision_history(), b.revision_history())
+        self.assertEqual(a.last_revision(), b.last_revision())
         a_tree.commit(message='blah2', allow_pointless=True)
         b_tree.commit(message='blah3', allow_pointless=True)
         # no overwrite
@@ -88,13 +94,13 @@
         os.chdir('overwriteme')
         self.run_bzr('pull --overwrite ../a')
         overwritten = Branch.open('.')
-        self.assertEqual(overwritten.revision_history(),
-                         a.revision_history())
+        self.assertEqual(overwritten.last_revision(),
+                         a.last_revision())
         a_tree.merge_from_branch(b_tree.branch)
         a_tree.commit(message="blah4", allow_pointless=True)
         os.chdir('../b/subdir')
         self.run_bzr('pull ../../a')
-        self.assertEqual(a.revision_history()[-1], b.revision_history()[-1])
+        self.assertEqual(a.last_revision(), b.last_revision())
         sub_tree = WorkingTree.open_containing('.')[0]
         sub_tree.commit(message="blah5", allow_pointless=True)
         sub_tree.commit(message="blah6", allow_pointless=True)
@@ -140,8 +146,24 @@
         self.run_bzr('pull -r 3')
         self.assertEqual(b.revno(),3)
         self.run_bzr('pull -r 4')
-        self.assertEqual(a.revision_history(), b.revision_history())
+        self.assertEqual(a.last_revision(), b.last_revision())
 
+    def test_pull_tags(self):
+        """Tags are updated by pull, and revisions named in those tags are
+        fetched.
+        """
+        # Make a source, sprout a target off it
+        builder = self.make_branch_builder('source')
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        target_bzrdir = source.bzrdir.sprout('target')
+        source.tags.set_tag('tag-a', 'rev-2')
+        # Pull from source
+        self.run_bzr('pull -d target source')
+        target = target_bzrdir.open_branch()
+        # The tag is present, and so is its revision.
+        self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))
+        target.repository.get_revision('rev-2')
 
     def test_overwrite_uptodate(self):
         # Make sure pull --overwrite overwrites
@@ -160,20 +182,18 @@
         self.build_tree_contents([('a/foo', 'a third change')])
         a_tree.commit(message='a third change')
 
-        rev_history_a = a_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_a), 3)
+        self.assertEqual(a_tree.branch.last_revision_info()[0], 3)
 
         b_tree.merge_from_branch(a_tree.branch)
         b_tree.commit(message='merge')
 
-        self.assertEqual(len(b_tree.branch.revision_history()), 2)
+        self.assertEqual(b_tree.branch.last_revision_info()[0], 2)
 
         os.chdir('b')
         self.run_bzr('pull --overwrite ../a')
-        rev_history_b = b_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_b), 3)
-
-        self.assertEqual(rev_history_b, rev_history_a)
+        (last_revinfo_b) = b_tree.branch.last_revision_info()
+        self.assertEqual(last_revinfo_b[0], 3)
+        self.assertEqual(last_revinfo_b[1], a_tree.branch.last_revision())
 
     def test_overwrite_children(self):
         # Make sure pull --overwrite sets the revision-history
@@ -191,27 +211,26 @@
         self.build_tree_contents([('a/foo', 'a third change')])
         a_tree.commit(message='a third change')
 
-        self.assertEqual(len(a_tree.branch.revision_history()), 3)
+        self.assertEqual(a_tree.branch.last_revision_info()[0], 3)
 
         b_tree.merge_from_branch(a_tree.branch)
         b_tree.commit(message='merge')
 
-        self.assertEqual(len(b_tree.branch.revision_history()), 2)
+        self.assertEqual(b_tree.branch.last_revision_info()[0], 2)
 
         self.build_tree_contents([('a/foo', 'a fourth change\n')])
         a_tree.commit(message='a fourth change')
 
-        rev_history_a = a_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_a), 4)
+        rev_info_a = a_tree.branch.last_revision_info()
+        self.assertEqual(rev_info_a[0], 4)
 
         # With convergence, we could just pull over the
         # new change, but with --overwrite, we want to switch our history
         os.chdir('b')
         self.run_bzr('pull --overwrite ../a')
-        rev_history_b = b_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_b), 4)
-
-        self.assertEqual(rev_history_b, rev_history_a)
+        rev_info_b = b_tree.branch.last_revision_info()
+        self.assertEqual(rev_info_b[0], 4)
+        self.assertEqual(rev_info_b, rev_info_a)
 
     def test_pull_remember(self):
         """Pull changes from one branch to another and test parent location."""
@@ -286,8 +305,8 @@
         self.assertEqual(err,
                 ' M  a\nAll changes applied successfully.\n')
 
-        self.assertEqualDiff(tree_a.branch.revision_history(),
-                             tree_b.branch.revision_history())
+        self.assertEqualDiff(tree_a.branch.last_revision(),
+                             tree_b.branch.last_revision())
 
         testament_a = Testament.from_revision(tree_a.branch.repository,
                                               tree_a.get_parent_ids()[0])
@@ -299,7 +318,7 @@
         # it is legal to attempt to pull an already-merged bundle
         out, err = self.run_bzr('pull ../bundle')
         self.assertEqual(err, '')
-        self.assertEqual(out, 'No revisions to pull.\n')
+        self.assertEqual(out, 'No revisions or tags to pull.\n')
 
     def test_pull_verbose_no_files(self):
         """Pull --verbose should not list modified files"""
@@ -365,6 +384,15 @@
         self.assertNotContainsRe(
             out, r'revno: 1\ncommitter: .*\nbranch nick: source')
 
+    def test_pull_smart_bound_branch(self):
+        self.setup_smart_server_with_call_log()
+        parent = self.make_branch_and_tree('parent')
+        parent.commit(message='first commit')
+        child = parent.bzrdir.sprout('child').open_workingtree()
+        child.commit(message='second commit')
+        checkout = parent.branch.create_checkout('checkout')
+        self.run_bzr(['pull', self.get_url('child')], working_dir='checkout')
+
     def test_pull_smart_stacked_streaming_acceptance(self):
         """'bzr pull -r 123' works on stacked, smart branches, even when the
         revision specified by the revno is only present in the fallback
@@ -392,10 +420,10 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(18, self.hpss_calls)
+        self.assertLength(19, self.hpss_calls)
         remote = Branch.open('stacked')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
-    
+
     def test_pull_cross_format_warning(self):
         """You get a warning for probably slow cross-format pulls.
         """
@@ -453,3 +481,103 @@
         out, err = self.run_bzr(['pull', '-d', 'to', 'from'])
         self.assertContainsRe(err,
             "(?m)Fetching into experimental format")
+
+    def test_pull_show_base(self):
+        """bzr pull supports --show-base
+
+        see https://bugs.launchpad.net/bzr/+bug/202374"""
+        # create two trees with conflicts, setup conflict, check that
+        # conflicted file looks correct
+        a_tree = self.example_branch('a')
+        b_tree = a_tree.bzrdir.sprout('b').open_workingtree()
+
+        f = open(pathjoin('a', 'hello'),'wt')
+        f.write('fee')
+        f.close()
+        a_tree.commit('fee')
+
+        f = open(pathjoin('b', 'hello'),'wt')
+        f.write('fie')
+        f.close()
+
+        out,err=self.run_bzr(['pull','-d','b','a','--show-base'])
+
+        # check for message here
+        self.assertEqual(err,
+                         ' M  hello\nText conflict in hello\n1 conflicts encountered.\n')
+
+        self.assertEqualDiff('<<<<<<< TREE\n'
+                             'fie||||||| BASE-REVISION\n'
+                             'foo=======\n'
+                             'fee>>>>>>> MERGE-SOURCE\n',
+                             open(pathjoin('b', 'hello')).read())
+
+    def test_pull_show_base_working_tree_only(self):
+        """--show-base only allowed if there's a working tree
+
+        see https://bugs.launchpad.net/bzr/+bug/202374"""
+        # create a branch, see that --show-base fails
+        self.make_branch('from')
+        self.make_branch('to')
+        out=self.run_bzr(['pull','-d','to','from','--show-base'],retcode=3)
+        self.assertEqual(out,
+                         ('','bzr: ERROR: Need working tree for --show-base.\n'))
+
+    def test_pull_tag_conflicts(self):
+        """pulling tags with conflicts will change the exit code"""
+        # create a branch, see that --show-base fails
+        from_tree = self.make_branch_and_tree('from')
+        from_tree.branch.tags.set_tag("mytag", "somerevid")
+        to_tree = self.make_branch_and_tree('to')
+        to_tree.branch.tags.set_tag("mytag", "anotherrevid")
+        out = self.run_bzr(['pull','-d','to','from'],retcode=1)
+        self.assertEqual(out,
+            ('No revisions to pull.\nConflicting tags:\n    mytag\n', ''))
+
+    def test_pull_tag_notification(self):
+        """pulling tags with conflicts will change the exit code"""
+        # create a branch, see that --show-base fails
+        from_tree = self.make_branch_and_tree('from')
+        from_tree.branch.tags.set_tag("mytag", "somerevid")
+        to_tree = self.make_branch_and_tree('to')
+        out = self.run_bzr(['pull', '-d', 'to', 'from'])
+        self.assertEqual(out,
+            ('1 tag(s) updated.\n', ''))
+
+    def test_pull_tag_overwrite(self):
+        """pulling tags with --overwrite only reports changed tags."""
+        # create a branch, see that --show-base fails
+        from_tree = self.make_branch_and_tree('from')
+        from_tree.branch.tags.set_tag("mytag", "somerevid")
+        to_tree = self.make_branch_and_tree('to')
+        to_tree.branch.tags.set_tag("mytag", "somerevid")
+        out = self.run_bzr(['pull', '--overwrite', '-d', 'to', 'from'])
+        self.assertEqual(out,
+            ('No revisions or tags to pull.\n', ''))
+
+
+class TestPullOutput(script.TestCaseWithTransportAndScript):
+
+    def test_pull_log_format(self):
+        self.run_script("""
+            $ bzr init trunk
+            Created a standalone tree (format: 2a)
+            $ cd trunk
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'we need some foo'
+            2>Committing to:...trunk/
+            2>added file
+            2>Committed revision 1.
+            $ cd ..
+            $ bzr init feature
+            Created a standalone tree (format: 2a)
+            $ cd feature
+            $ bzr pull -v ../trunk -Olog_format=line
+            Now on revision 1.
+            Added Revisions:
+            1: jrandom@example.com ...we need some foo
+            2>+N  file
+            2>All changes applied successfully.
+            """)

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_push.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     osutils,
     tests,
@@ -34,34 +35,15 @@
 from bzrlib.tests import (
     blackbox,
     http_server,
+    scenarios,
+    script,
     test_foreign,
     test_server,
     )
 from bzrlib.transport import memory
 
 
-def load_tests(standard_tests, module, loader):
-    """Multiply tests for the push command."""
-    result = loader.suiteClass()
-
-    # one for each king of change
-    changes_tests, remaining_tests = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance((
-                TestPushStrictWithChanges,
-                )))
-    changes_scenarios = [
-        ('uncommitted',
-         dict(_changes_type= '_uncommitted_changes')),
-        ('pending-merges',
-         dict(_changes_type= '_pending_merges')),
-        ('out-of-sync-trees',
-         dict(_changes_type= '_out_of_sync_trees')),
-        ]
-    tests.multiply_tests(changes_tests, changes_scenarios, result)
-    # No parametrization for the remaining tests
-    result.addTests(remaining_tests)
-
-    return result
+load_tests = scenarios.load_tests_apply_scenarios
 
 
 class TestPush(tests.TestCaseWithTransport):
@@ -124,12 +106,11 @@
         transport.delete('branch_b/c')
         out, err = self.run_bzr('push', working_dir='branch_a')
         path = branch_a.get_push_location()
-        self.assertEquals(out,
-                          'Using saved push location: %s\n'
-                          % urlutils.local_path_from_url(path))
         self.assertEqual(err,
+                         'Using saved push location: %s\n'
                          'All changes applied successfully.\n'
-                         'Pushed up to revision 2.\n')
+                         'Pushed up to revision 2.\n'
+                         % urlutils.local_path_from_url(path))
         self.assertEqual(path,
                          branch_b.bzrdir.root_transport.base)
         # test explicit --remember
@@ -146,6 +127,17 @@
         b2 = branch.Branch.open('pushed-location')
         self.assertEndsWith(b2.base, 'pushed-location/')
 
+    def test_push_no_tree(self):
+        # bzr push --no-tree of a branch with working trees
+        b = self.make_branch_and_tree('push-from')
+        self.build_tree(['push-from/file'])
+        b.add('file')
+        b.commit('commit 1')
+        out, err = self.run_bzr('push --no-tree -d push-from push-to')
+        self.assertEqual('', out)
+        self.assertEqual('Created new branch.\n', err)
+        self.assertPathDoesNotExist('push-to/file')
+
     def test_push_new_branch_revision_count(self):
         # bzr push of a branch with revisions to a new location
         # should print the number of revisions equal to the length of the
@@ -158,6 +150,22 @@
         self.assertEqual('', out)
         self.assertEqual('Created new branch.\n', err)
 
+    def test_push_quiet(self):
+        # test that using -q makes output quiet
+        t = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        t.add('file')
+        t.commit('commit 1')
+        self.run_bzr('push -d tree pushed-to')
+        path = t.branch.get_push_location()
+        out, err = self.run_bzr('push', working_dir="tree")
+        self.assertEqual('Using saved push location: %s\n'
+                         'No new revisions or tags to push.\n' %
+                         urlutils.local_path_from_url(path), err)
+        out, err = self.run_bzr('push -q', working_dir="tree")
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
     def test_push_only_pushes_history(self):
         # Knit branches should only push the history for the current revision.
         format = bzrdir.BzrDirMetaFormat1()
@@ -167,7 +175,7 @@
 
         def make_shared_tree(path):
             shared_repo.bzrdir.root_transport.mkdir(path)
-            shared_repo.bzrdir.create_branch_convenience('repo/' + path)
+            controldir.ControlDir.create_branch_convenience('repo/' + path)
             return workingtree.WorkingTree.open('repo/' + path)
         tree_a = make_shared_tree('a')
         self.build_tree(['repo/a/file'])
@@ -208,10 +216,26 @@
         t.commit(allow_pointless=True,
                 message='first commit')
         self.run_bzr('push -d from to-one')
-        self.failUnlessExists('to-one')
+        self.assertPathExists('to-one')
         self.run_bzr('push -d %s %s'
             % tuple(map(urlutils.local_path_to_url, ['from', 'to-two'])))
-        self.failUnlessExists('to-two')
+        self.assertPathExists('to-two')
+
+    def test_push_repository_no_branch_doesnt_fetch_all_revs(self):
+        # See https://bugs.launchpad.net/bzr/+bug/465517
+        target_repo = self.make_repository('target')
+        source = self.make_branch_builder('source')
+        source.start_series()
+        source.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        source.build_snapshot('B', ['A'], [])
+        source.build_snapshot('C', ['A'], [])
+        source.finish_series()
+        self.run_bzr('push target -d source')
+        self.addCleanup(target_repo.lock_read().unlock)
+        # We should have pushed 'C', but not 'B', since it isn't in the
+        # ancestry
+        self.assertEqual([('A',), ('C',)], sorted(target_repo.revisions.keys()))
 
     def test_push_smart_non_stacked_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -240,7 +264,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(14, self.hpss_calls)
+        self.assertLength(13, self.hpss_calls)
         remote = branch.Branch.open('public')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
 
@@ -317,7 +341,7 @@
                      working_dir='tree')
         new_tree = workingtree.WorkingTree.open('new/tree')
         self.assertEqual(tree.last_revision(), new_tree.last_revision())
-        self.failUnlessExists('new/tree/a')
+        self.assertPathExists('new/tree/a')
 
     def test_push_use_existing(self):
         """'bzr push --use-existing-dir' can push into an existing dir.
@@ -338,7 +362,7 @@
         new_tree = workingtree.WorkingTree.open('target')
         self.assertEqual(tree.last_revision(), new_tree.last_revision())
         # The push should have created target/a
-        self.failUnlessExists('target/a')
+        self.assertPathExists('target/a')
 
     def test_push_use_existing_into_empty_bzrdir(self):
         """'bzr push --use-existing-dir' into a dir with an empty .bzr dir
@@ -654,8 +678,8 @@
     def set_config_push_strict(self, value):
         # set config var (any of bazaar.conf, locations.conf, branch.conf
         # should do)
-        conf = self.tree.branch.get_config()
-        conf.set_user_option('push_strict', value)
+        conf = self.tree.branch.get_config_stack()
+        conf.set('push_strict', value)
 
     _default_command = ['push', '../to']
     _default_wd = 'local'
@@ -718,9 +742,20 @@
         self.assertPushSucceeds([])
 
 
+strict_push_change_scenarios = [
+    ('uncommitted',
+        dict(_changes_type= '_uncommitted_changes')),
+    ('pending-merges',
+        dict(_changes_type= '_pending_merges')),
+    ('out-of-sync-trees',
+        dict(_changes_type= '_out_of_sync_trees')),
+    ]
+
+
 class TestPushStrictWithChanges(tests.TestCaseWithTransport,
                                 TestPushStrictMixin):
 
+    scenarios = strict_push_change_scenarios 
     _changes_type = None # Set by load_tests
 
     def setUp(self):
@@ -809,3 +844,27 @@
         self.assertEquals("", output)
         self.assertEquals(error, "bzr: ERROR: It is not possible to losslessly"
             " push to dummy. You may want to use dpush instead.\n")
+
+
+class TestPushOutput(script.TestCaseWithTransportAndScript):
+
+    def test_push_log_format(self):
+        self.run_script("""
+            $ bzr init trunk
+            Created a standalone tree (format: 2a)
+            $ cd trunk
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'we need some foo'
+            2>Committing to:...trunk/
+            2>added file
+            2>Committed revision 1.
+            $ bzr init ../feature
+            Created a standalone tree (format: 2a)
+            $ bzr push -v ../feature -Olog_format=line
+            Added Revisions:
+            1: jrandom@example.com ...we need some foo
+            2>All changes applied successfully.
+            2>Pushed up to revision 1.
+            """)

=== modified file 'bzrlib/tests/blackbox/test_reconfigure.py'
--- bzrlib/tests/blackbox/test_reconfigure.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/blackbox/test_reconfigure.py	2011-09-06 18:37:06 +0000
@@ -21,9 +21,10 @@
     workingtree,
     )
 from bzrlib.branchbuilder import BranchBuilder
-
-
-class TestReconfigure(tests.TestCaseWithTransport):
+from bzrlib.tests.script import TestCaseWithTransportAndScript
+
+
+class TestReconfigure(TestCaseWithTransportAndScript):
 
     def test_no_type(self):
         branch = self.make_branch('branch')
@@ -131,7 +132,7 @@
         tree.add('foo')
         self.run_bzr('reconfigure --with-no-trees --force',
             working_dir='repo/branch')
-        self.failUnlessExists('repo/branch/foo')
+        self.assertPathExists('repo/branch/foo')
         tree = workingtree.WorkingTree.open('repo/branch')
 
     def test_shared_format_to_standalone(self, format=None):
@@ -175,15 +176,45 @@
         self.run_bzr('revert', working_dir='checkout')
         self.check_file_contents('checkout/file', 'foo\n')
 
-    def test_lightweight_knit_checkout_to_tree(self, format=None):
+    def test_lightweight_knit_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('knit')
 
-    def test_lightweight_pack092_checkout_to_tree(self, format=None):
+    def test_lightweight_pack092_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('pack-0.92')
 
-    def test_lightweight_rich_root_pack_checkout_to_tree(self, format=None):
+    def test_lightweight_rich_root_pack_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('rich-root-pack')
 
+    def test_branch_and_use_shared(self):
+        self.run_script("""\
+$ bzr init -q branch
+$ echo foo > branch/foo
+$ bzr add -q branch/foo
+$ bzr commit -q -m msg branch
+$ bzr init-repo -q .
+$ bzr reconfigure --branch --use-shared branch
+$ bzr info branch
+Repository branch (format: ...)
+Location:
+  shared repository: .
+  repository branch: branch
+""")
+
+    def test_use_shared_and_branch(self):
+        self.run_script("""\
+$ bzr init -q branch
+$ echo foo > branch/foo
+$ bzr add -q branch/foo
+$ bzr commit -q -m msg branch
+$ bzr init-repo -q .
+$ bzr reconfigure --use-shared --branch branch
+$ bzr info branch
+Repository branch (format: ...)
+Location:
+  shared repository: .
+  repository branch: branch
+""")
+
 
 class TestReconfigureStacking(tests.TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/blackbox/test_reference.py'
--- bzrlib/tests/blackbox/test_reference.py	2009-04-30 14:04:38 +0000
+++ bzrlib/tests/blackbox/test_reference.py	2011-09-21 20:29:48 +0000
@@ -24,11 +24,10 @@
 
 class TestReference(TestCaseWithTransport):
 
-    def make_branch(self, location, format=None):
-        if format is None:
-            format = bzrdir.format_registry.make_bzrdir('1.9')
-            format.set_branch_format(_mod_branch.BzrBranchFormat8())
-        return TestCaseWithTransport.make_branch(self, location, format=format)
+    def get_default_format(self):
+        format = bzrdir.format_registry.make_bzrdir('1.9')
+        format.set_branch_format(_mod_branch.BzrBranchFormat8())
+        return format
 
     def test_no_args_lists(self):
         branch = self.make_branch('branch')

=== added file 'bzrlib/tests/blackbox/test_remember_option.py'
--- bzrlib/tests/blackbox/test_remember_option.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_remember_option.py	2011-05-17 15:01:58 +0000
@@ -0,0 +1,208 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+from bzrlib import (
+    branch,
+    urlutils,
+    )
+from bzrlib.tests import (
+    script,
+    )
+
+
+class TestRememberMixin(object):
+    """--remember and --no-remember set locations or not."""
+
+    # the command to run (expecting additional arguments from the tests
+    command = []
+    # the dir where the command should be run (it should contain a branch for
+    # which the tested locations are/will be set)
+    working_dir = None
+    # argument list for the first command invocation
+    first_use_args = []
+    # argument list for the next command invocation
+    next_uses_args = []
+
+    def do_command(self, *args):
+        # We always expect the same result here and care only about the
+        # arguments used and their consequences on the remembered locations
+        out, err = self.run_bzr(self.command + list(args),
+                                working_dir=self.working_dir)
+
+    def test_first_use_no_option(self):
+        self.do_command(*self.first_use_args)
+        self.assertLocations(self.first_use_args)
+
+    def test_first_use_remember(self):
+        self.do_command('--remember', *self.first_use_args)
+        self.assertLocations(self.first_use_args)
+
+    def test_first_use_no_remember(self):
+        self.do_command('--no-remember', *self.first_use_args)
+        self.assertLocations([])
+
+    def test_next_uses_no_option(self):
+        self.setup_next_uses()
+        self.do_command(*self.next_uses_args)
+        self.assertLocations(self.first_use_args)
+
+    def test_next_uses_remember(self):
+        self.setup_next_uses()
+        self.do_command('--remember', *self.next_uses_args)
+        self.assertLocations(self.next_uses_args)
+
+    def test_next_uses_no_remember(self):
+        self.setup_next_uses()
+        self.do_command('--no-remember', *self.next_uses_args)
+        self.assertLocations(self.first_use_args)
+
+
+class TestSendRemember(script.TestCaseWithTransportAndScript,
+                       TestRememberMixin):
+
+    working_dir = 'work'
+    command = ['send', '-o-',]
+    first_use_args = ['../parent', '../grand_parent',]
+    next_uses_args = ['../new_parent', '../new_grand_parent']
+
+    def setUp(self):
+        super(TestSendRemember, self).setUp()
+        self.run_script('''
+            $ bzr init grand_parent
+            $ cd grand_parent
+            $ echo grand_parent > file
+            $ bzr add
+            $ bzr commit -m 'initial commit'
+            $ cd ..
+            $ bzr branch grand_parent parent
+            $ cd parent
+            $ echo parent > file
+            $ bzr commit -m 'parent'
+            $ cd ..
+            $ bzr branch parent %(working_dir)s
+            $ cd %(working_dir)s
+            $ echo %(working_dir)s > file
+            $ bzr commit -m '%(working_dir)s'
+            $ cd ..
+            ''' % {'working_dir': self.working_dir},
+                        null_output_matches_anything=True)
+
+    def setup_next_uses(self):
+        # Do a first send that remembers the locations
+        self.do_command(*self.first_use_args)
+        # Now create some new targets
+        self.run_script('''
+            $ bzr branch grand_parent new_grand_parent
+            $ bzr branch parent new_parent
+            ''',
+                        null_output_matches_anything=True)
+
+    def assertLocations(self, expected_locations):
+        if not expected_locations:
+            expected_submit_branch, expected_public_branch = None, None
+        else:
+            expected_submit_branch, expected_public_branch = expected_locations
+        br, _ = branch.Branch.open_containing(self.working_dir)
+        self.assertEquals(expected_submit_branch, br.get_submit_branch())
+        self.assertEquals(expected_public_branch, br.get_public_branch())
+
+
+class TestPushRemember(script.TestCaseWithTransportAndScript,
+                       TestRememberMixin):
+
+    working_dir = 'work'
+    command = ['push',]
+    first_use_args = ['../target',]
+    next_uses_args = ['../new_target']
+
+    def setUp(self):
+        super(TestPushRemember, self).setUp()
+        self.run_script('''
+            $ bzr init %(working_dir)s
+            $ cd %(working_dir)s
+            $ echo some content > file
+            $ bzr add
+            $ bzr commit -m 'initial commit'
+            $ cd ..
+            ''' % {'working_dir': self.working_dir},
+                        null_output_matches_anything=True)
+
+    def setup_next_uses(self):
+        # Do a first push that remembers the location
+        self.do_command(*self.first_use_args)
+        # Now create some new content
+        self.run_script('''
+            $ cd %(working_dir)s
+            $ echo new content > file
+            $ bzr commit -m 'new content'
+            $ cd ..
+            ''' % {'working_dir': self.working_dir},
+                        null_output_matches_anything=True)
+
+    def assertLocations(self, expected_locations):
+        br, _ = branch.Branch.open_containing(self.working_dir)
+        if not expected_locations:
+            self.assertEquals(None, br.get_push_location())
+        else:
+            expected_push_location = expected_locations[0]
+            push_location = urlutils.relative_url(br.base,
+                                                  br.get_push_location())
+            self.assertIsSameRealPath(expected_push_location, push_location)
+
+
+class TestPullRemember(script.TestCaseWithTransportAndScript,
+                       TestRememberMixin):
+
+    working_dir = 'work'
+    command = ['pull',]
+    first_use_args = ['../parent',]
+    next_uses_args = ['../new_parent']
+
+    def setUp(self):
+        super(TestPullRemember, self).setUp()
+        self.run_script('''
+            $ bzr init parent
+            $ cd parent
+            $ echo parent > file
+            $ bzr add
+            $ bzr commit -m 'initial commit'
+            $ cd ..
+            $ bzr init %(working_dir)s
+            ''' % {'working_dir': self.working_dir},
+                        null_output_matches_anything=True)
+
+    def setup_next_uses(self):
+        # Do a first push that remembers the location
+        self.do_command(*self.first_use_args)
+        # Now create some new content
+        self.run_script('''
+            $ bzr branch parent new_parent
+            $ cd new_parent
+            $ echo new parent > file
+            $ bzr commit -m 'new parent'
+            $ cd ..
+            ''' % {'working_dir': self.working_dir},
+                        null_output_matches_anything=True)
+
+    def assertLocations(self, expected_locations):
+        br, _ = branch.Branch.open_containing(self.working_dir)
+        if not expected_locations:
+            self.assertEquals(None, br.get_parent())
+        else:
+            expected_pull_location = expected_locations[0]
+            pull_location = urlutils.relative_url(br.base, br.get_parent())
+            self.assertIsSameRealPath(expected_pull_location, pull_location)

=== modified file 'bzrlib/tests/blackbox/test_remerge.py'
--- bzrlib/tests/blackbox/test_remerge.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_remerge.py	2011-05-13 12:51:05 +0000
@@ -74,8 +74,8 @@
                            'remerge hello --merge-type weave',
                            retcode=1)
 
-        self.failUnlessExists('hello.OTHER')
-        self.failIfExists('question.OTHER')
+        self.assertPathExists('hello.OTHER')
+        self.assertPathDoesNotExist('question.OTHER')
 
         file_id = self.run_bzr('file-id hello')[0]
         self.run_bzr_error(['hello.THIS is not versioned'],
@@ -84,8 +84,8 @@
         self.run_bzr_error(['conflicts encountered'],
                            'remerge --merge-type weave', retcode=1)
 
-        self.failUnlessExists('hello.OTHER')
-        self.failUnless('hello.BASE')
+        self.assertPathExists('hello.OTHER')
+        self.assertTrue('hello.BASE')
         self.assertFalse('|||||||' in conflict_text)
         self.assertFalse('hi world' in conflict_text)
 

=== modified file 'bzrlib/tests/blackbox/test_remove.py'
--- bzrlib/tests/blackbox/test_remove.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_remove.py	2011-06-14 01:26:41 +0000
@@ -18,8 +18,12 @@
 import os
 import sys
 
-from bzrlib.tests import SymlinkFeature, TestSkipped
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import (
+    script,
+    features,
+    TestCaseWithTransport,
+    TestSkipped,
+    )
 from bzrlib.workingtree import WorkingTree
 from bzrlib import osutils
 
@@ -49,30 +53,21 @@
         for f in files:
             id=f+_id
             self.assertNotInWorkingTree(f)
-            self.failIfExists(f)
+            self.assertPathDoesNotExist(f)
 
     def assertFilesUnversioned(self, files):
         for f in files:
             self.assertNotInWorkingTree(f)
-            self.failUnlessExists(f)
+            self.assertPathExists(f)
 
     def changeFile(self, file_name):
         f = file(file_name, 'ab')
         f.write("\nsome other new content!")
         f.close()
 
-    def run_bzr_remove_changed_files(self, error_regexes, files_to_remove,
-                                     working_dir=None):
-        error_regexes.extend(["Can't safely remove modified or unknown files:",
-            'Use --keep to not delete them,'
-            ' or --force to delete them regardless.'
-            ])
-        self.run_bzr_error(error_regexes,
-                           ['remove'] + list(files_to_remove),
-                           working_dir=working_dir)
-        #see if we can force it now
-        self.run_bzr(['remove', '--force'] + list(files_to_remove),
-                     working_dir=working_dir)
+    def run_bzr_remove_changed_files(self, files_to_remove, working_dir=None):
+        self.run_bzr(['remove'] + list(files_to_remove),
+           working_dir=working_dir)
 
     def test_remove_new_no_files_specified(self):
         tree = self.make_branch_and_tree('.')
@@ -85,7 +80,7 @@
         self.assertEqual('', err)
         self.assertEqual('', out)
         self.assertInWorkingTree('foo', tree=tree)
-        self.failUnlessExists('foo')
+        self.assertPathExists('foo')
 
     def test_remove_no_files_specified_missing_dir_and_contents(self):
         tree = self._make_tree_and_add(
@@ -99,9 +94,9 @@
             err)
         # non-missing paths not touched:
         self.assertInWorkingTree('foo', tree=tree)
-        self.failUnlessExists('foo')
+        self.assertPathExists('foo')
         self.assertInWorkingTree('dir', tree=tree)
-        self.failUnlessExists('dir')
+        self.assertPathExists('dir')
         # missing files unversioned
         self.assertNotInWorkingTree('dir/missing', tree=tree)
         self.assertNotInWorkingTree('dir/missing/child', tree=tree)
@@ -125,12 +120,12 @@
         self.assertEqual('removed bar\n', err)
         # non-missing files not touched:
         self.assertInWorkingTree('foo', tree=tree)
-        self.failUnlessExists('foo')
+        self.assertPathExists('foo')
         # missing files unversioned
         self.assertNotInWorkingTree('bar', tree=tree)
 
     def test_remove_no_files_specified_missing_link(self):
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self._make_tree_and_add(['foo'])
         os.symlink('foo', 'linkname')
         tree.add(['linkname'])
@@ -140,7 +135,7 @@
         self.assertEqual('removed linkname\n', err)
         # non-missing files not touched:
         self.assertInWorkingTree('foo', tree=tree)
-        self.failUnlessExists('foo')
+        self.assertPathExists('foo')
         # missing files unversioned
         self.assertNotInWorkingTree('linkname', tree=tree)
 
@@ -177,20 +172,19 @@
     def test_remove_unversioned_files(self):
         self.build_tree(files)
         tree = self.make_branch_and_tree('.')
-        self.run_bzr_remove_changed_files(
-            ['unknown:[.\s]*d/[.\s]*b/c[.\s]*b/[.\s]*a'], files)
+        self.run_bzr_remove_changed_files(files)
 
     def test_remove_changed_files(self):
         tree = self._make_tree_and_add(files)
         self.run_bzr("commit -m 'added files'")
         self.changeFile(a)
         self.changeFile(c)
-        self.run_bzr_remove_changed_files(['modified:[.\s]*a[.\s]*b/c'], files)
+        self.run_bzr_remove_changed_files(files)
 
     def test_remove_changed_ignored_files(self):
         tree = self._make_tree_and_add(['a'])
         self.run_bzr(['ignore', 'a'])
-        self.run_bzr_remove_changed_files(['added:[.\s]*a'], ['a'])
+        self.run_bzr_remove_changed_files(['a'])
 
     def test_remove_changed_files_from_child_dir(self):
         if sys.platform == 'win32':
@@ -200,10 +194,9 @@
         self.changeFile(a)
         self.changeFile(c)
         self.run_bzr_remove_changed_files(
-            ['modified:[.\s]*a[.\s]*b/c'],
             ['../a', 'c', '.', '../d'], working_dir='b')
         self.assertNotInWorkingTree(files)
-        self.failIfExists(files)
+        self.assertPathDoesNotExist(files)
 
     def test_remove_keep_unversioned_files(self):
         self.build_tree(files)
@@ -211,12 +204,29 @@
         self.run_bzr('remove --keep a', error_regexes=["a is not versioned."])
         self.assertFilesUnversioned(files)
 
+    def test_remove_no_backup_unversioned_files(self):
+        self.build_tree(files)
+        tree = self.make_branch_and_tree('.')
+        script.ScriptRunner().run_script(self, '''
+        $ bzr remove --no-backup a b/ b/c d/
+        2>deleted d
+        2>removed b/c (but kept a copy: b/c.~1~)
+        2>deleted b
+        2>deleted a
+        ''')
+        self.assertFilesDeleted(files)
+
     def test_remove_force_unversioned_files(self):
         self.build_tree(files)
         tree = self.make_branch_and_tree('.')
-        self.run_bzr(['remove', '--force'] + list(files),
-                     error_regexes=["deleted a", "deleted b",
-                                    "deleted b/c", "deleted d"])
+        script.ScriptRunner().run_script(self, '''
+        $ bzr remove --force a b/ b/c d/
+        2>(The --force option is deprecated, rather use --no-backup in future.)
+        2>deleted d
+        2>removed b/c (but kept a copy: b/c.~1~)
+        2>deleted b
+        2>deleted a
+        ''')
         self.assertFilesDeleted(files)
 
     def test_remove_deleted_files(self):
@@ -227,10 +237,10 @@
         for f in my_files:
             osutils.delete_any(f)
         self.assertInWorkingTree(files)
-        self.failIfExists(files)
+        self.assertPathDoesNotExist(files)
         self.run_bzr('remove ' + ' '.join(files))
         self.assertNotInWorkingTree(a)
-        self.failIfExists(files)
+        self.assertPathDoesNotExist(files)
 
     def test_remove_non_existing_files(self):
         tree = self._make_tree_and_add([])

=== modified file 'bzrlib/tests/blackbox/test_remove_tree.py'
--- bzrlib/tests/blackbox/test_remove_tree.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_remove_tree.py	2011-05-13 12:51:05 +0000
@@ -31,75 +31,75 @@
         self.build_tree(['branch1/foo'])
         self.tree.add('foo')
         self.tree.commit('1')
-        self.failUnlessExists('branch1/foo')
+        self.assertPathExists('branch1/foo')
 
     # Success modes
 
     def test_remove_tree_original_branch(self):
         os.chdir('branch1')
         self.run_bzr('remove-tree')
-        self.failIfExists('foo')
+        self.assertPathDoesNotExist('foo')
 
     def test_remove_tree_original_branch_explicit(self):
         self.run_bzr('remove-tree branch1')
-        self.failIfExists('branch1/foo')
+        self.assertPathDoesNotExist('branch1/foo')
 
     def test_remove_tree_multiple_branch_explicit(self):
         self.tree.bzrdir.sprout('branch2')
         self.run_bzr('remove-tree branch1 branch2')
-        self.failIfExists('branch1/foo')
-        self.failIfExists('branch2/foo')
+        self.assertPathDoesNotExist('branch1/foo')
+        self.assertPathDoesNotExist('branch2/foo')
 
     def test_remove_tree_sprouted_branch(self):
         self.tree.bzrdir.sprout('branch2')
-        self.failUnlessExists('branch2/foo')
+        self.assertPathExists('branch2/foo')
         os.chdir('branch2')
         self.run_bzr('remove-tree')
-        self.failIfExists('foo')
+        self.assertPathDoesNotExist('foo')
 
     def test_remove_tree_sprouted_branch_explicit(self):
         self.tree.bzrdir.sprout('branch2')
-        self.failUnlessExists('branch2/foo')
+        self.assertPathExists('branch2/foo')
         self.run_bzr('remove-tree branch2')
-        self.failIfExists('branch2/foo')
+        self.assertPathDoesNotExist('branch2/foo')
 
     def test_remove_tree_checkout(self):
         self.tree.branch.create_checkout('branch2', lightweight=False)
-        self.failUnlessExists('branch2/foo')
+        self.assertPathExists('branch2/foo')
         os.chdir('branch2')
         self.run_bzr('remove-tree')
-        self.failIfExists('foo')
+        self.assertPathDoesNotExist('foo')
         os.chdir('..')
-        self.failUnlessExists('branch1/foo')
+        self.assertPathExists('branch1/foo')
 
     def test_remove_tree_checkout_explicit(self):
         self.tree.branch.create_checkout('branch2', lightweight=False)
-        self.failUnlessExists('branch2/foo')
+        self.assertPathExists('branch2/foo')
         self.run_bzr('remove-tree branch2')
-        self.failIfExists('branch2/foo')
-        self.failUnlessExists('branch1/foo')
+        self.assertPathDoesNotExist('branch2/foo')
+        self.assertPathExists('branch1/foo')
 
     # Failure modes
 
     def test_remove_tree_lightweight_checkout(self):
         self.tree.branch.create_checkout('branch2', lightweight=True)
-        self.failUnlessExists('branch2/foo')
+        self.assertPathExists('branch2/foo')
         os.chdir('branch2')
         output = self.run_bzr_error(
             ["You cannot remove the working tree from a lightweight checkout"],
             'remove-tree', retcode=3)
-        self.failUnlessExists('foo')
+        self.assertPathExists('foo')
         os.chdir('..')
-        self.failUnlessExists('branch1/foo')
+        self.assertPathExists('branch1/foo')
 
     def test_remove_tree_lightweight_checkout_explicit(self):
         self.tree.branch.create_checkout('branch2', lightweight=True)
-        self.failUnlessExists('branch2/foo')
+        self.assertPathExists('branch2/foo')
         output = self.run_bzr_error(
             ["You cannot remove the working tree from a lightweight checkout"],
             'remove-tree branch2', retcode=3)
-        self.failUnlessExists('branch2/foo')
-        self.failUnlessExists('branch1/foo')
+        self.assertPathExists('branch2/foo')
+        self.assertPathExists('branch1/foo')
 
     def test_remove_tree_empty_dir(self):
         os.mkdir('branch2')
@@ -109,7 +109,7 @@
 
     def test_remove_tree_repeatedly(self):
         self.run_bzr('remove-tree branch1')
-        self.failIfExists('branch1/foo')
+        self.assertPathDoesNotExist('branch1/foo')
         output = self.run_bzr_error(["No working tree to remove"],
                                     'remove-tree branch1', retcode=3)
 
@@ -127,19 +127,19 @@
         self.build_tree(['branch1/bar'])
         self.tree.add('bar')
         self.run_bzr('remove-tree branch1 --force')
-        self.failIfExists('branch1/foo')
-        self.failUnlessExists('branch1/bar')
+        self.assertPathDoesNotExist('branch1/foo')
+        self.assertPathExists('branch1/bar')
 
     def test_remove_tree_pending_merges(self):
         self.run_bzr(['branch', 'branch1', 'branch2'])
         self.build_tree(['branch1/bar'])
         self.tree.add('bar')
         self.tree.commit('2')
-        self.failUnlessExists('branch1/bar')
+        self.assertPathExists('branch1/bar')
         self.run_bzr(['merge', '../branch1'], working_dir='branch2')
-        self.failUnlessExists('branch2/bar')
+        self.assertPathExists('branch2/bar')
         self.run_bzr(['revert', '.'], working_dir='branch2')
-        self.failIfExists('branch2/bar')
+        self.assertPathDoesNotExist('branch2/bar')
         output = self.run_bzr_error(["Working tree .* has uncommitted changes"],
                                     'remove-tree branch2', retcode=3)
 
@@ -148,14 +148,14 @@
         self.build_tree(['branch1/bar'])
         self.tree.add('bar')
         self.tree.commit('2')
-        self.failUnlessExists('branch1/bar')
+        self.assertPathExists('branch1/bar')
         self.run_bzr(['merge', '../branch1'], working_dir='branch2')
-        self.failUnlessExists('branch2/bar')
+        self.assertPathExists('branch2/bar')
         self.run_bzr(['revert', '.'], working_dir='branch2')
-        self.failIfExists('branch2/bar')
+        self.assertPathDoesNotExist('branch2/bar')
         self.run_bzr('remove-tree branch2 --force')
-        self.failIfExists('branch2/foo')
-        self.failIfExists('branch2/bar')
+        self.assertPathDoesNotExist('branch2/foo')
+        self.assertPathDoesNotExist('branch2/bar')
 
     def test_remove_tree_shelved_changes(self):
         # https://bugs.launchpad.net/bzr/+bug/586639

=== added file 'bzrlib/tests/blackbox/test_repair_workingtree.py'
--- bzrlib/tests/blackbox/test_repair_workingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_repair_workingtree.py	2011-04-15 07:01:22 +0000
@@ -0,0 +1,97 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+from bzrlib import (
+    workingtree,
+    )
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestRepairWorkingTree(TestCaseWithTransport):
+
+    def break_dirstate(self, tree, completely=False):
+        """Write garbage into the dirstate file."""
+        # This test assumes that the format uses a DirState file, which we then
+        # manually corrupt. If we change the way to get at that dirstate file,
+        # then we can update how this is done
+        self.assertIsNot(None, getattr(tree, 'current_dirstate', None))
+        tree.lock_read()
+        try:
+            dirstate = tree.current_dirstate()
+            dirstate_path = dirstate._filename
+            self.assertPathExists(dirstate_path)
+        finally:
+            tree.unlock()
+        # We have to have the tree unlocked at this point, so we can safely
+        # mutate the state file on all platforms.
+        if completely:
+            f = open(dirstate_path, 'wb')
+        else:
+            f = open(dirstate_path, 'ab')
+        try:
+            f.write('garbage-at-end-of-file\n')
+        finally:
+            f.close()
+
+    def make_initial_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo', 'tree/dir/', 'tree/dir/bar'])
+        tree.add(['foo', 'dir', 'dir/bar'])
+        tree.commit('first')
+        return tree
+
+    def test_repair_refuses_uncorrupted(self):
+        tree = self.make_initial_tree()
+        # If the tree doesn't appear to be corrupt, we refuse, but prompt the
+        # user to let them know that:
+        # a) they may want to use 'bzr revert' instead of repair-workingtree
+        # b) they can use --force if they really want to do this
+        self.run_bzr_error(['The tree does not appear to be corrupt',
+                            '"bzr revert"',
+                            '--force'],
+                           'repair-workingtree -d tree')
+
+    def test_repair_forced(self):
+        tree = self.make_initial_tree()
+        tree.rename_one('dir', 'alt_dir')
+        self.assertIsNot(None, tree.path2id('alt_dir'))
+        self.run_bzr('repair-workingtree -d tree --force')
+        # This requires the tree has reloaded the working state
+        self.assertIs(None, tree.path2id('alt_dir'))
+        self.assertPathExists('tree/alt_dir')
+
+    def test_repair_corrupted_dirstate(self):
+        tree = self.make_initial_tree()
+        self.break_dirstate(tree)
+        self.run_bzr('repair-workingtree -d tree')
+        tree = workingtree.WorkingTree.open('tree')
+        # At this point, check should be happy
+        tree.check_state()
+
+    def test_repair_naive_destroyed_fails(self):
+        tree = self.make_initial_tree()
+        self.break_dirstate(tree, completely=True)
+        self.run_bzr_error(['the header appears corrupt, try passing'],
+                           'repair-workingtree -d tree')
+
+    def test_repair_destroyed_with_revs_passes(self):
+        tree = self.make_initial_tree()
+        self.break_dirstate(tree, completely=True)
+        self.run_bzr('repair-workingtree -d tree -r -1')
+        tree = workingtree.WorkingTree.open('tree')
+        # At this point, check should be happy
+        tree.check_state()

=== added file 'bzrlib/tests/blackbox/test_resolve.py'
--- bzrlib/tests/blackbox/test_resolve.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_resolve.py	2011-09-16 15:37:58 +0000
@@ -0,0 +1,131 @@
+# Copyright (C) 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from bzrlib import (
+    conflicts,
+    tests,
+    )
+from bzrlib.tests import script
+from bzrlib.tests.blackbox import test_conflicts
+
+
+class TestResolve(script.TestCaseWithTransportAndScript):
+
+    def setUp(self):
+        super(TestResolve, self).setUp()
+        test_conflicts.make_tree_with_conflicts(self, 'branch', 'other')
+
+    def test_resolve_one_by_one(self):
+        self.run_script("""\
+$ cd branch
+$ bzr conflicts
+Text conflict in my_other_file
+Path conflict: mydir3 / mydir2
+Text conflict in myfile
+$ bzr resolve myfile
+2>1 conflict resolved, 2 remaining
+$ bzr resolve my_other_file
+2>1 conflict resolved, 1 remaining
+$ bzr resolve mydir2
+2>1 conflict resolved, 0 remaining
+""")
+
+    def test_resolve_all(self):
+        self.run_script("""\
+$ cd branch
+$ bzr resolve --all
+2>3 conflicts resolved, 0 remaining
+$ bzr conflicts
+""")
+
+    def test_resolve_from_subdir(self):
+        self.run_script("""\
+$ mkdir branch/subdir
+$ cd branch/subdir
+$ bzr resolve ../myfile
+2>1 conflict resolved, 2 remaining
+""")
+
+    def test_resolve_via_directory_option(self):
+        self.run_script("""\
+$ bzr resolve -d branch myfile
+2>1 conflict resolved, 2 remaining
+""")
+
+    def test_resolve_all_via_directory_option(self):
+        self.run_script("""\
+$ bzr resolve -d branch --all
+2>3 conflicts resolved, 0 remaining
+$ bzr conflicts -d branch
+""")
+
+
+class TestBug788000(script.TestCaseWithTransportAndScript):
+
+    def test_bug_788000(self):
+        self.run_script('''\
+$ bzr init a
+$ mkdir a/dir
+$ echo foo > a/dir/file
+$ bzr add a/dir
+$ cd a
+$ bzr commit -m one
+$ cd ..
+$ bzr clone a b
+$ echo bar > b/dir/file
+$ cd a
+$ rm -r dir
+$ bzr commit -m two
+$ cd ../b
+''',
+                        null_output_matches_anything=True)
+
+        self.run_script('''\
+$ bzr pull
+Using saved parent location:...
+Now on revision 2.
+2>RM  dir/file => dir/file.THIS
+2>Conflict: can't delete dir because it is not empty.  Not deleting.
+2>Conflict because dir is not versioned, but has versioned children...
+2>Contents conflict in dir/file
+2>3 conflicts encountered.
+''')
+        self.run_script('''\
+$ bzr resolve --take-other
+2>deleted dir/file.THIS
+2>deleted dir
+2>3 conflicts resolved, 0 remaining
+''')
+
+
+class TestResolveAuto(tests.TestCaseWithTransport):
+
+    def test_auto_resolve(self):
+        """Text conflicts can be resolved automatically"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/file',
+            '<<<<<<<\na\n=======\n>>>>>>>\n')])
+        tree.add('file', 'file_id')
+        self.assertEqual(tree.kind('file_id'), 'file')
+        file_conflict = conflicts.TextConflict('file', file_id='file_id')
+        tree.set_conflicts(conflicts.ConflictList([file_conflict]))
+        note = self.run_bzr('resolve', retcode=1, working_dir='tree')[1]
+        self.assertContainsRe(note, '0 conflicts auto-resolved.')
+        self.assertContainsRe(note,
+            'Remaining conflicts:\nText conflict in file')
+        self.build_tree_contents([('tree/file', 'a\n')])
+        note = self.run_bzr('resolve', working_dir='tree')[1]
+        self.assertContainsRe(note, 'All conflicts resolved.')

=== modified file 'bzrlib/tests/blackbox/test_revert.py'
--- bzrlib/tests/blackbox/test_revert.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_revert.py	2011-05-18 10:03:10 +0000
@@ -93,17 +93,17 @@
         """Test that revert DIRECTORY does what's expected"""
         self._prepare_rename_mod_tree()
         self.run_bzr('revert a')
-        self.failUnlessExists('a/b')
-        self.failUnlessExists('a/d')
-        self.failIfExists('a/g')
+        self.assertPathExists('a/b')
+        self.assertPathExists('a/d')
+        self.assertPathDoesNotExist('a/g')
         self.expectFailure(
             "j is in the delta revert applies because j was renamed too",
-            self.failUnlessExists, 'j')
-        self.failUnlessExists('h')
+            self.assertPathExists, 'j')
+        self.assertPathExists('h')
         self.run_bzr('revert f')
-        self.failIfExists('j')
-        self.failIfExists('h')
-        self.failUnlessExists('a/d/e')
+        self.assertPathDoesNotExist('j')
+        self.assertPathDoesNotExist('h')
+        self.assertPathExists('a/d/e')
 
     def test_revert_chatter(self):
         self._prepare_rename_mod_tree()
@@ -120,16 +120,16 @@
     def test_revert(self):
         self.run_bzr('init')
 
-        file('hello', 'wt').write('foo')
+        with file('hello', 'wt') as f: f.write('foo')
         self.run_bzr('add hello')
         self.run_bzr('commit -m setup hello')
 
-        file('goodbye', 'wt').write('baz')
+        with file('goodbye', 'wt') as f: f.write('baz')
         self.run_bzr('add goodbye')
         self.run_bzr('commit -m setup goodbye')
 
-        file('hello', 'wt').write('bar')
-        file('goodbye', 'wt').write('qux')
+        with file('hello', 'wt') as f: f.write('bar')
+        with file('goodbye', 'wt') as f: f.write('qux')
         self.run_bzr('revert hello')
         self.check_file_contents('hello', 'foo')
         self.check_file_contents('goodbye', 'qux')
@@ -148,7 +148,7 @@
             self.run_bzr('commit -m f')
             os.unlink('symlink')
             self.run_bzr('revert')
-            self.failUnlessExists('symlink')
+            self.assertPathExists('symlink')
             os.unlink('symlink')
             os.symlink('a-different-path', 'symlink')
             self.run_bzr('revert')
@@ -157,7 +157,7 @@
         else:
             self.log("skipping revert symlink tests")
 
-        file('hello', 'wt').write('xyz')
+        with file('hello', 'wt') as f: f.write('xyz')
         self.run_bzr('commit -m xyz hello')
         self.run_bzr('revert -r 1 hello')
         self.check_file_contents('hello', 'foo')

=== modified file 'bzrlib/tests/blackbox/test_revno.py'
--- bzrlib/tests/blackbox/test_revno.py	2009-06-30 16:59:21 +0000
+++ bzrlib/tests/blackbox/test_revno.py	2011-10-08 11:42:32 +0000
@@ -117,3 +117,22 @@
         out, err = self.run_bzr('revno --tree checkout_b')
         self.assertEqual('', err)
         self.assertEqual('???\n', out)
+
+    def test_revno_with_revision(self):
+        wt = self.make_branch_and_tree('.')
+        revid1 = wt.commit('rev1')
+        revid2 = wt.commit('rev2')
+
+        out, err = self.run_bzr('revno -r-2 .')
+        self.assertEqual('1\n', out)
+
+        out, err = self.run_bzr('revno -rrevid:%s .' % revid1)
+        self.assertEqual('1\n', out)
+
+    def test_revno_and_tree_mutually_exclusive(self):
+        wt = self.make_branch_and_tree('.')
+        out, err = self.run_bzr('revno -r-2 --tree .', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual(
+            'bzr: ERROR: --tree and --revision can not be used together\n',
+            err)

=== modified file 'bzrlib/tests/blackbox/test_rmbranch.py'
--- bzrlib/tests/blackbox/test_rmbranch.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_rmbranch.py	2011-05-13 12:51:05 +0000
@@ -42,8 +42,8 @@
         self.run_bzr('rmbranch a')
         dir = bzrdir.BzrDir.open('a')
         self.assertFalse(dir.has_branch())
-        self.failUnlessExists('a/hello')
-        self.failUnlessExists('a/goodbye')
+        self.assertPathExists('a/hello')
+        self.assertPathExists('a/goodbye')
 
     def test_no_branch(self):
         # No branch in the current directory. 

=== added file 'bzrlib/tests/blackbox/test_script.py'
--- bzrlib/tests/blackbox/test_script.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_script.py	2010-11-08 10:53:53 +0000
@@ -0,0 +1,72 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Blacbox tests for the test-script command."""
+
+import os
+
+from bzrlib import (
+    tests,
+    )
+from bzrlib.tests import (
+    script,
+    )
+
+
+class TestTestScript(tests.TestCaseInTempDir):
+
+    def test_unknnown_file(self):
+        self.run_bzr(['test-script', 'I-do-not-exist'], retcode=3)
+
+    def test_empty_file(self):
+        self.build_tree_contents([('script', '')])
+        out, err = self.run_bzr(['test-script', 'script'])
+        out_lines = out.splitlines()
+        self.assertStartsWith(out_lines[-3], 'Ran 1 test in ')
+        self.assertEquals('OK', out_lines[-1])
+        self.assertEquals('', err)
+
+    def test_simple_file(self):
+        self.build_tree_contents([('script', '''
+$ echo hello world
+hello world
+''')])
+        out, err = self.run_bzr(['test-script', 'script'])
+        out_lines = out.splitlines()
+        self.assertStartsWith(out_lines[-3], 'Ran 1 test in ')
+        self.assertEquals('OK', out_lines[-1])
+        self.assertEquals('', err)
+
+    def test_null_output(self):
+        self.build_tree_contents([('script', '''
+$ echo hello world
+''')])
+        out, err = self.run_bzr(['test-script', 'script', '--null-output'])
+        out_lines = out.splitlines()
+        self.assertStartsWith(out_lines[-3], 'Ran 1 test in ')
+        self.assertEquals('OK', out_lines[-1])
+        self.assertEquals('', err)
+
+    def test_failing_script(self):
+        self.build_tree_contents([('script', '''
+$ echo hello foo
+hello bar
+''')])
+        out, err = self.run_bzr(['test-script', 'script'], retcode=1)
+        out_lines = out.splitlines()
+        self.assertStartsWith(out_lines[-3], 'Ran 1 test in ')
+        self.assertEquals('FAILED (failures=1)', out_lines[-1])
+        self.assertEquals('', err)

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2010-07-18 14:28:12 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2010-09-03 20:49:17 +0000
@@ -16,6 +16,8 @@
 
 """UI tests for the test framework."""
 
+import os
+
 from bzrlib import (
     tests,
     )
@@ -148,3 +150,12 @@
     def test_lsprof_tests(self):
         params = self.get_params_passed_to_core('selftest --lsprof-tests')
         self.assertEqual(True, params[1]["lsprof_tests"])
+
+    def test_parallel_fork_unsupported(self):
+        if getattr(os, "fork", None) is not None:
+            self.addCleanup(setattr, os, "fork", os.fork)
+            del os.fork
+        out, err = self.run_bzr(["selftest", "--parallel=fork", "-s", "bt.x"],
+            retcode=3)
+        self.assertIn("platform does not support fork", err)
+        self.assertFalse(out)

=== modified file 'bzrlib/tests/blackbox/test_send.py'
--- bzrlib/tests/blackbox/test_send.py	2010-04-28 10:30:48 +0000
+++ bzrlib/tests/blackbox/test_send.py	2011-09-22 12:13:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # Authors: Aaron Bentley
 #
 # This program is free software; you can redistribute it and/or modify
@@ -27,30 +27,13 @@
     )
 from bzrlib.bundle import serializer
 from bzrlib.transport import memory
-
-
-def load_tests(standard_tests, module, loader):
-    """Multiply tests for the send command."""
-    result = loader.suiteClass()
-
-    # one for each king of change
-    changes_tests, remaining_tests = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance((
-                TestSendStrictWithChanges,
-                )))
-    changes_scenarios = [
-        ('uncommitted',
-         dict(_changes_type='_uncommitted_changes')),
-        ('pending_merges',
-         dict(_changes_type='_pending_merges')),
-        ('out-of-sync-trees',
-         dict(_changes_type='_out_of_sync_trees')),
-        ]
-    tests.multiply_tests(changes_tests, changes_scenarios, result)
-    # No parametrization for the remaining tests
-    result.addTests(remaining_tests)
-
-    return result
+from bzrlib.tests import (
+    scenarios,
+    script,
+    )
+
+
+load_tests = scenarios.load_tests_apply_scenarios
 
 
 class TestSendMixin(object):
@@ -206,7 +189,7 @@
 
     def test_note_revisions(self):
         stderr = self.run_send([])[1]
-        self.assertEndsWith(stderr, '\nBundling 1 revision(s).\n')
+        self.assertEndsWith(stderr, '\nBundling 1 revision.\n')
 
     def test_mailto_option(self):
         b = branch.Branch.open('branch')
@@ -313,8 +296,8 @@
     def set_config_send_strict(self, value):
         # set config var (any of bazaar.conf, locations.conf, branch.conf
         # should do)
-        conf = self.local_tree.branch.get_config()
-        conf.set_user_option('send_strict', value)
+        conf = self.local_tree.branch.get_config_stack()
+        conf.set('send_strict', value)
 
     def assertSendFails(self, args):
         out, err = self.run_send(args, rc=3, err_re=self._default_errors)
@@ -328,7 +311,10 @@
         if revs is None:
             revs = self._default_sent_revs
         out, err = self.run_send(args, err_re=err_re)
-        bundling_revs = 'Bundling %d revision(s).\n' % len(revs)
+        if len(revs) == 1:
+            bundling_revs = 'Bundling %d revision.\n'% len(revs)
+        else:
+            bundling_revs = 'Bundling %d revisions.\n' % len(revs)
         if with_warning:
             self.assertContainsRe(err, self._default_additional_warning)
             self.assertEndsWith(err, bundling_revs)
@@ -366,7 +352,19 @@
 
 
 class TestSendStrictWithChanges(tests.TestCaseWithTransport,
-                                   TestSendStrictMixin):
+                                TestSendStrictMixin):
+
+    # These are textually the same as test_push.strict_push_change_scenarios,
+    # but since the functions are reimplemented here, the definitions are left
+    # here too.
+    scenarios = [
+        ('uncommitted',
+         dict(_changes_type='_uncommitted_changes')),
+        ('pending_merges',
+         dict(_changes_type='_pending_merges')),
+        ('out-of-sync-trees',
+         dict(_changes_type='_out_of_sync_trees')),
+        ]
 
     _changes_type = None # Set by load_tests
 

=== modified file 'bzrlib/tests/blackbox/test_serve.py'
--- bzrlib/tests/blackbox/test_serve.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/blackbox/test_serve.py	2011-10-12 16:00:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,32 +18,33 @@
 """Tests of the bzr serve command."""
 
 import os
-import os.path
 import signal
-import subprocess
 import sys
 import thread
 import threading
+import time
 
 from bzrlib import (
     builtins,
-    debug,
+    config,
     errors,
     osutils,
     revision as _mod_revision,
+    trace,
     transport,
     urlutils,
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart import client, medium
-from bzrlib.smart.server import BzrServerFactory, SmartTCPServer
+from bzrlib.smart.server import (
+    BzrServerFactory,
+    SmartTCPServer,
+    )
 from bzrlib.tests import (
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
-    TestSkipped,
     )
-from bzrlib.trace import mutter
 from bzrlib.transport import remote
 
 
@@ -53,37 +54,49 @@
                                 *func_args, **func_kwargs):
         """Run 'bzr serve', and run the given func in a thread once the server
         has started.
-        
+
         When 'func' terminates, the server will be terminated too.
-        
+
         Returns stdout and stderr.
         """
-        # install hook
-        def on_server_start(backing_urls, tcp_server):
-            t = threading.Thread(
-                target=on_server_start_thread, args=(tcp_server,))
-            t.start()
         def on_server_start_thread(tcp_server):
+            """This runs concurrently with the server thread.
+
+            The server is interrupted as soon as ``func`` finishes, even if an
+            exception is encountered.
+            """
             try:
                 # Run func if set
                 self.tcp_server = tcp_server
-                if not func is None:
+                if func is not None:
                     try:
                         func(*func_args, **func_kwargs)
                     except Exception, e:
                         # Log errors to make some test failures a little less
                         # mysterious.
-                        mutter('func broke: %r', e)
+                        trace.mutter('func broke: %r', e)
             finally:
                 # Then stop the server
-                mutter('interrupting...')
+                trace.mutter('interrupting...')
                 thread.interrupt_main()
+        # When the hook is fired, it just starts ``on_server_start_thread`` and
+        # return
+        def on_server_start(backing_urls, tcp_server):
+            t = threading.Thread(
+                target=on_server_start_thread, args=(tcp_server,))
+            t.start()
+        # install hook
         SmartTCPServer.hooks.install_named_hook(
             'server_started_ex', on_server_start,
             'run_bzr_serve_then_func hook')
+        # It seesm thread.interrupt_main() will not raise KeyboardInterrupt
+        # until after socket.accept returns. So we set the timeout low to make
+        # the test faster.
+        self.overrideAttr(SmartTCPServer, '_ACCEPT_TIMEOUT', 0.1)
         # start a TCP server
         try:
-            out, err = self.run_bzr(['serve'] + list(serve_args))
+            out, err = self.run_bzr(['serve'] + list(serve_args),
+                                    retcode=retcode)
         except KeyboardInterrupt, e:
             out, err = e.args
         return out, err
@@ -95,6 +108,30 @@
         super(TestBzrServe, self).setUp()
         self.disable_missing_extensions_warning()
 
+    def test_server_exception_with_hook(self):
+        """Catch exception from the server in the server_exception hook.
+
+        We use ``run_bzr_serve_then_func`` without a ``func`` so the server
+        will receive a KeyboardInterrupt exception we want to catch.
+        """
+        def hook(exception):
+            if exception[0] is KeyboardInterrupt:
+                sys.stderr.write('catching KeyboardInterrupt\n')
+                return True
+            else:
+                return False
+        SmartTCPServer.hooks.install_named_hook(
+            'server_exception', hook,
+            'test_server_except_hook hook')
+        args = ['--port', 'localhost:0', '--quiet']
+        out, err = self.run_bzr_serve_then_func(args, retcode=0)
+        self.assertEqual('catching KeyboardInterrupt\n', err)
+
+    def test_server_exception_no_hook(self):
+        """test exception without hook returns error"""
+        args = []
+        out, err = self.run_bzr_serve_then_func(args, retcode=3)
+
     def assertInetServerShutsdownCleanly(self, process):
         """Shutdown the server process looking for errors."""
         # Shutdown the server: the server should shut down when it cannot read
@@ -164,7 +201,7 @@
         url = 'bzr://localhost:%d/' % port
         self.permit_url(url)
         return process, url
-    
+
     def test_bzr_serve_quiet(self):
         self.make_branch('.')
         args = ['--port', 'localhost:0', '--quiet']
@@ -193,7 +230,7 @@
     def test_bzr_serve_port_readonly(self):
         """bzr server should provide a read only filesystem by default."""
         process, url = self.start_server_port()
-        t = transport.get_transport(url)
+        t = transport.get_transport_from_url(url)
         self.assertRaises(errors.TransportNotPossible, t.mkdir, 'adir')
         self.assertServerFinishesCleanly(process)
 
@@ -225,7 +262,7 @@
         # -Dhpss, and does drop some hpss logging to the file.
         self.make_branch('.')
         log_fname = os.getcwd() + '/server.log'
-        self._captureVar('BZR_LOG', log_fname)
+        self.overrideEnv('BZR_LOG', log_fname)
         process, transport = self.start_server_inet(['-Dhpss'])
         branch = BzrDir.open_from_transport(transport).open_branch()
         self.make_read_requests(branch)
@@ -235,6 +272,76 @@
         f.close()
         self.assertContainsRe(content, r'hpss request: \[[0-9-]+\]')
 
+    def test_bzr_serve_supports_configurable_timeout(self):
+        gs = config.GlobalStack()
+        gs.set('serve.client_timeout', 0.2)
+        process, url = self.start_server_port()
+        self.build_tree_contents([('a_file', 'contents\n')])
+        # We can connect and issue a request
+        t = transport.get_transport_from_url(url)
+        self.assertEqual('contents\n', t.get_bytes('a_file'))
+        # However, if we just wait for more content from the server, it will
+        # eventually disconnect us.
+        # TODO: Use something like signal.alarm() so that if the server doesn't
+        #       properly handle the timeout, we end up failing the test instead
+        #       of hanging forever.
+        m = t.get_smart_medium()
+        m.read_bytes(1)
+        # Now, we wait for timeout to trigger
+        err = process.stderr.readline()
+        self.assertEqual(
+            'Connection Timeout: disconnecting client after 0.2 seconds\n',
+            err)
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_supports_client_timeout(self):
+        process, url = self.start_server_port(['--client-timeout=0.1'])
+        self.build_tree_contents([('a_file', 'contents\n')])
+        # We can connect and issue a request
+        t = transport.get_transport_from_url(url)
+        self.assertEqual('contents\n', t.get_bytes('a_file'))
+        # However, if we just wait for more content from the server, it will
+        # eventually disconnect us.
+        # TODO: Use something like signal.alarm() so that if the server doesn't
+        #       properly handle the timeout, we end up failing the test instead
+        #       of hanging forever.
+        m = t.get_smart_medium()
+        m.read_bytes(1)
+        # Now, we wait for timeout to trigger
+        err = process.stderr.readline()
+        self.assertEqual(
+            'Connection Timeout: disconnecting client after 0.1 seconds\n',
+            err)
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_graceful_shutdown(self):
+        big_contents = 'a'*64*1024
+        self.build_tree_contents([('bigfile', big_contents)])
+        process, url = self.start_server_port(['--client-timeout=1.0'])
+        t = transport.get_transport_from_url(url)
+        m = t.get_smart_medium()
+        c = client._SmartClient(m)
+        # Start, but don't finish a response
+        resp, response_handler = c.call_expecting_body('get', 'bigfile')
+        self.assertEqual(('ok',), resp)
+        # Note: process.send_signal is a Python 2.6ism
+        process.send_signal(signal.SIGHUP)
+        # Wait for the server to notice the signal, and then read the actual
+        # body of the response. That way we know that it is waiting for the
+        # request to finish
+        self.assertEqual('Requested to stop gracefully\n',
+                         process.stderr.readline())
+        self.assertEqual('Waiting for 1 client(s) to finish\n',
+                         process.stderr.readline())
+        body = response_handler.read_body_bytes()
+        if body != big_contents:
+            self.fail('Failed to properly read the contents of "bigfile"')
+        # Now that our request is finished, the medium should notice it has
+        # been disconnected.
+        self.assertEqual('', m.read_bytes(1))
+        # And the server should be stopping
+        self.assertEqual(0, process.wait())
+
 
 class TestCmdServeChrooting(TestBzrServeBase):
 
@@ -276,7 +383,8 @@
 
 class TestUserdirExpansion(TestCaseWithMemoryTransport):
 
-    def fake_expanduser(self, path):
+    @staticmethod
+    def fake_expanduser(path):
         """A simple, environment-independent, function for the duration of this
         test.
 
@@ -295,7 +403,7 @@
             self.fake_expanduser, lambda t: base_path)
         mem_transport = self.get_transport()
         mem_transport.mkdir_multi(['home', 'home/user'])
-        bzr_server.set_up(mem_transport, None, None, inet=True)
+        bzr_server.set_up(mem_transport, None, None, inet=True, timeout=4.0)
         self.addCleanup(bzr_server.tear_down)
         return bzr_server
 
@@ -317,7 +425,7 @@
         base_url = urlutils.local_path_to_url(base_dir) + '/'
         # Define a fake 'protocol' to capture the transport that cmd_serve
         # passes to serve_bzr.
-        def capture_transport(transport, host, port, inet):
+        def capture_transport(transport, host, port, inet, timeout):
             self.bzr_serve_transport = transport
         cmd = builtins.cmd_serve()
         # Read-only
@@ -334,4 +442,3 @@
         self.assertEqual(base_url, self.bzr_serve_transport.base)
         self.assertEqual(base_dir,
             server_maker.get_base_path(self.bzr_serve_transport))
-

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2010-05-05 14:02:53 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -88,7 +88,7 @@
         cdir = BzrDir.open('a/c')
         cdir.open_branch()
         self.assertRaises(errors.NoRepositoryPresent, cdir.open_repository)
-        self.failUnlessExists('a/c/hello')
+        self.assertPathExists('a/c/hello')
         cdir.open_workingtree()
 
     def test_trees_default(self):
@@ -155,8 +155,8 @@
         # and uses whoami only in a lock file. Without whoami the login name
         # is used. This test is to ensure that init-repo passes even when whoami
         # is not available.
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
         out, err = self.run_bzr(['init-repo', 'foo'])
         self.assertEqual(err, '')
         self.assertTrue(os.path.exists('foo'))

=== modified file 'bzrlib/tests/blackbox/test_shelve.py'
--- bzrlib/tests/blackbox/test_shelve.py	2010-05-28 14:15:28 +0000
+++ bzrlib/tests/blackbox/test_shelve.py	2011-05-13 12:51:05 +0000
@@ -41,6 +41,14 @@
         self.assertEqual('', err)
         self.assertEqual('  1: Foo\n', out)
 
+    def test_shelve_list_via_directory(self):
+        tree = self.make_branch_and_tree('tree')
+        creator = self.make_creator(tree)
+        shelf_id = tree.get_shelf_manager().shelve_changes(creator, 'Foo')
+        out, err = self.run_bzr('shelve -d tree --list', retcode=1)
+        self.assertEqual('', err)
+        self.assertEqual('  1: Foo\n', out)
+
     def test_shelve_no_message(self):
         tree = self.make_branch_and_tree('.')
         creator = self.make_creator(tree)
@@ -64,7 +72,7 @@
         self.build_tree(['file'])
         tree.add('file')
         self.run_bzr('shelve --all --destroy')
-        self.failIfExists('file')
+        self.assertPathDoesNotExist('file')
         self.assertIs(None, tree.get_shelf_manager().last_shelf())
 
     def test_unshelve_keep(self):
@@ -77,16 +85,43 @@
         sr = ScriptRunner()
         sr.run_script(self, '''
 $ bzr add file
+adding file
 $ bzr shelve --all -m Foo
+2>Selected changes:
+2>-D  file
+2>Changes shelved with id "1".
 $ bzr shelve --list
   1: Foo
 $ bzr unshelve --keep
+2>Using changes with id "1".
+2>Message: Foo
+2>+N  file
+2>All changes applied successfully.
 $ bzr shelve --list
   1: Foo
 $ cat file
 contents of file
 ''')
 
+class TestUnshelvePreview(TestCaseWithTransport):
+    
+    def test_non_ascii(self):
+        """Test that we can show a non-ascii diff that would result from unshelving"""
+        
+        init_content = u'Initial: \u0418\u0437\u043d\u0430\u0447\n'.encode('utf-8')
+        more_content = u'More: \u0415\u0449\u0451\n'.encode('utf-8')
+        next_content = init_content + more_content
+        diff_part = '@@ -1,1 +1,2 @@\n %s+%s' % (init_content, more_content)
+        
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('a_file', init_content)])
+        tree.add('a_file')
+        tree.commit(message='committed')
+        self.build_tree_contents([('a_file', next_content)])
+        self.run_bzr(['shelve', '--all'])
+        out, err = self.run_bzr(['unshelve', '--preview'], encoding='latin-1')
+        
+        self.assertContainsString(out, diff_part)
 
 
 class TestShelveRelpath(TestCaseWithTransport):
@@ -98,6 +133,12 @@
         os.chdir('tree/dir')
         self.run_bzr('shelve --all ../file')
 
+    def test_shelve_via_directory(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file', 'tree/dir/'])
+        tree.add('file')
+        self.run_bzr('shelve -d tree/dir --all ../file')
+
 
 class TestShelveUnshelve(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/blackbox/test_sign_my_commits.py'
--- bzrlib/tests/blackbox/test_sign_my_commits.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/blackbox/test_sign_my_commits.py	2011-08-16 11:05:38 +0000
@@ -17,14 +17,10 @@
 
 """Black-box tests for bzr sign-my-commits."""
 
-import os
-
 from bzrlib import (
     gpg,
     tests,
     )
-from bzrlib.testament import Testament
-from bzrlib.workingtree import WorkingTree
 
 
 class SignMyCommits(tests.TestCaseWithTransport):
@@ -45,7 +41,8 @@
         wt.commit("base C", allow_pointless=True, rev_id='C')
         wt.commit("base D", allow_pointless=True, rev_id='D',
                 committer='Alternate <alt@foo.com>')
-
+        wt.add_parent_tree_id("aghost")
+        wt.commit("base E", allow_pointless=True, rev_id='E')
         return wt
 
     def assertUnsigned(self, repo, revision_id):
@@ -109,8 +106,34 @@
 
         out = self.run_bzr('sign-my-commits --dry-run')[0]
 
-        self.assertEquals('A\nB\nC\nSigned 3 revisions\n', out)
+        outlines = out.splitlines()
+        self.assertEquals(5, len(outlines))
+        self.assertEquals('Signed 4 revisions', outlines[-1])
         self.assertUnsigned(repo, 'A')
         self.assertUnsigned(repo, 'B')
         self.assertUnsigned(repo, 'C')
         self.assertUnsigned(repo, 'D')
+        self.assertUnsigned(repo, 'E')
+
+    def test_verify_commits(self):
+        wt = self.setup_tree()
+        self.monkey_patch_gpg()
+        self.run_bzr('sign-my-commits')
+        out = self.run_bzr('verify-signatures', retcode=1)
+        self.assertEquals(('4 commits with valid signatures\n'
+                           '0 commits with key now expired\n'
+                           '0 commits with unknown keys\n'
+                           '0 commits not valid\n'
+                           '1 commit not signed\n', ''), out)
+
+    def test_verify_commits_acceptable_key(self):
+        wt = self.setup_tree()
+        self.monkey_patch_gpg()
+        self.run_bzr('sign-my-commits')
+        out = self.run_bzr(['verify-signatures', '--acceptable-keys=foo,bar'],
+                            retcode=1)
+        self.assertEquals(('4 commits with valid signatures\n'
+                           '0 commits with key now expired\n'
+                           '0 commits with unknown keys\n'
+                           '0 commits not valid\n'
+                           '1 commit not signed\n', ''), out)

=== modified file 'bzrlib/tests/blackbox/test_split.py'
--- bzrlib/tests/blackbox/test_split.py	2009-08-27 21:48:33 +0000
+++ bzrlib/tests/blackbox/test_split.py	2011-04-05 14:47:30 +0000
@@ -15,11 +15,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import os
-
-from bzrlib import bzrdir, tests, workingtree
+from bzrlib import tests, workingtree
 from bzrlib.repofmt.knitrepo import RepositoryFormatKnit4
-from bzrlib.repofmt.pack_repo import RepositoryFormatKnitPack4
+from bzrlib.repofmt.knitpack_repo import RepositoryFormatKnitPack4
 
 
 

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2010-06-24 06:57:31 +0000
+++ bzrlib/tests/blackbox/test_status.py	2011-06-03 08:45:36 +0000
@@ -33,6 +33,7 @@
     conflicts,
     errors,
     osutils,
+    status,
     )
 import bzrlib.branch
 from bzrlib.osutils import pathjoin
@@ -44,6 +45,14 @@
 
 class BranchStatus(TestCaseWithTransport):
 
+    def setUp(self):
+        super(BranchStatus, self).setUp()
+        # As TestCase.setUp clears all hooks, we install this default
+        # post_status hook handler for the test.
+        status.hooks.install_named_hook('post_status',
+            status._show_shelve_summary,
+            'bzr status')
+
     def assertStatus(self, expected_lines, working_tree,
         revision=None, short=False, pending=True, verbose=False):
         """Run status in working_tree and look for output.
@@ -202,12 +211,18 @@
         wt = self.make_branch_and_tree('.')
         b = wt.branch
 
-        self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
+        self.build_tree(['directory/','directory/hello.c',
+                         'bye.c','test.c','dir2/',
+                         'missing.c'])
         wt.add('directory')
         wt.add('test.c')
         wt.commit('testing')
+        wt.add('missing.c')
+        unlink('missing.c')
 
         self.assertStatus([
+                'missing:\n',
+                '  missing.c\n',
                 'unknown:\n',
                 '  bye.c\n',
                 '  dir2/\n',
@@ -218,6 +233,7 @@
         self.assertStatus([
                 '?   bye.c\n',
                 '?   dir2/\n',
+                '+!  missing.c\n',
                 '?   directory/hello.c\n'
                 ],
                 wt, short=True)
@@ -260,6 +276,20 @@
         tof.seek(0)
         self.assertEquals(tof.readlines(), ['+N  test.c\n'])
 
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['missing.c'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['missing:\n',
+                           '  missing.c\n'])
+
+        tof = StringIO()
+        show_tree_status(wt, specific_files=['missing.c'], to_file=tof,
+                         short=True)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['+!  missing.c\n'])
+
     def test_specific_files_conflicts(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir2/'])
@@ -294,10 +324,10 @@
         wt.add('FILE_D')
         wt.add('FILE_E')
         wt.commit('Create five empty files.')
-        open('FILE_B', 'w').write('Modification to file FILE_B.')
-        open('FILE_C', 'w').write('Modification to file FILE_C.')
+        with open('FILE_B', 'w') as f: f.write('Modification to file FILE_B.')
+        with open('FILE_C', 'w') as f: f.write('Modification to file FILE_C.')
         unlink('FILE_E')  # FILE_E will be versioned but missing
-        open('FILE_Q', 'w').write('FILE_Q is added but not committed.')
+        with open('FILE_Q', 'w') as f: f.write('FILE_Q is added but not committed.')
         wt.add('FILE_Q')  # FILE_Q will be added but not committed
         open('UNVERSIONED_BUT_EXISTING', 'w')
         return wt
@@ -447,10 +477,13 @@
           ' M  FILE_B\n',
           'X   NONEXISTENT\n',
           ]
+        expected.sort()
         out, err = self.run_bzr('status --short NONEXISTENT '
                                 'FILE_A FILE_B UNVERSIONED_BUT_EXISTING '
                                 'FILE_C FILE_D FILE_E FILE_Q', retcode=3)
-        self.assertEqual(expected, out.splitlines(True))
+        actual = out.splitlines(True)
+        actual.sort()
+        self.assertEqual(expected, actual)
         self.assertContainsRe(err,
                               r'.*ERROR: Path\(s\) do not exist: '
                               'NONEXISTENT.*')
@@ -520,6 +553,31 @@
         out, err = self.run_bzr('status branch1 -rbranch:branch2')
         self.assertEqual('', out)
 
+    def test_status_with_shelves(self):
+        """Ensure that _show_shelve_summary handler works.
+        """
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['hello.c'])
+        wt.add('hello.c')
+        self.run_bzr(['shelve', '--all', '-m', 'foo'])
+        self.build_tree(['bye.c'])
+        wt.add('bye.c')
+        self.assertStatus([
+                'added:\n',
+                '  bye.c\n',
+                '1 shelf exists. See "bzr shelve --list" for details.\n',
+            ],
+            wt)
+        self.run_bzr(['shelve', '--all', '-m', 'bar'])
+        self.build_tree(['spam.c'])
+        wt.add('spam.c')
+        self.assertStatus([
+                'added:\n',
+                '  spam.c\n',
+                '2 shelves exist. See "bzr shelve --list" for details.\n',
+            ],
+            wt)
+
 
 class CheckoutStatus(BranchStatus):
 
@@ -579,10 +637,10 @@
         self.assertContainsRe(result, "[+]N  hello.txt\n")
 
         self.build_tree(['world.txt'])
-        result = self.run_bzr("status --short -r 0")[0]
+        result = self.run_bzr("status -S -r 0")[0]
         self.assertContainsRe(result, "[+]N  hello.txt\n" \
                                       "[?]   world.txt\n")
-        result2 = self.run_bzr("status --short -r 0..")[0]
+        result2 = self.run_bzr("status -S -r 0..")[0]
         self.assertEquals(result2, result)
 
     def test_status_versioned(self):

=== modified file 'bzrlib/tests/blackbox/test_switch.py'
--- bzrlib/tests/blackbox/test_switch.py	2010-06-20 22:54:30 +0000
+++ bzrlib/tests/blackbox/test_switch.py	2011-07-18 13:58:52 +0000
@@ -20,9 +20,16 @@
 
 import os
 
-from bzrlib import osutils
+from bzrlib import (
+        osutils,
+        urlutils,
+        branch,
+        )
 from bzrlib.workingtree import WorkingTree
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests import (
+        TestCaseWithTransport,
+        script,
+        )
 from bzrlib.directory_service import directories
 
 
@@ -129,7 +136,7 @@
         """Using switch on a heavy checkout should find master sibling
 
         The behaviour of lighweight and heavy checkouts should be
-        consistentwhen using the convenient "switch to sibling" feature
+        consistent when using the convenient "switch to sibling" feature
         Both should switch to a sibling of the branch
         they are bound to, and not a sibling of themself"""
 
@@ -149,17 +156,17 @@
         tree = self._create_sample_tree()
         checkout = tree.branch.create_checkout('checkout', lightweight=True)
         self.run_bzr(['switch', 'branch-1', '-r1'], working_dir='checkout')
-        self.failUnlessExists('checkout/file-1')
-        self.failIfExists('checkout/file-2')
+        self.assertPathExists('checkout/file-1')
+        self.assertPathDoesNotExist('checkout/file-2')
 
     def test_switch_only_revision(self):
         tree = self._create_sample_tree()
         checkout = tree.branch.create_checkout('checkout', lightweight=True)
-        self.failUnlessExists('checkout/file-1')
-        self.failUnlessExists('checkout/file-2')
+        self.assertPathExists('checkout/file-1')
+        self.assertPathExists('checkout/file-2')
         self.run_bzr(['switch', '-r1'], working_dir='checkout')
-        self.failUnlessExists('checkout/file-1')
-        self.failIfExists('checkout/file-2')
+        self.assertPathExists('checkout/file-1')
+        self.assertPathDoesNotExist('checkout/file-2')
         # Check that we don't accept a range
         self.run_bzr_error(
             ['bzr switch --revision takes exactly one revision identifier'],
@@ -270,3 +277,78 @@
         self.run_bzr('checkout --lightweight a checkout')
         self.run_bzr('switch --directory checkout b')
         self.assertFileEqual('initial\nmore\n', 'checkout/a')
+
+
+class TestSwitchParentLocationBase(TestCaseWithTransport):
+
+    def setUp(self):
+        """Set up a repository and branch ready for testing."""
+        super(TestSwitchParentLocationBase, self).setUp()
+        self.script_runner = script.ScriptRunner()
+        self.script_runner.run_script(self, '''
+                $ bzr init-repo --no-trees repo
+                Shared repository...
+                Location:
+                  shared repository: repo
+                $ bzr init repo/trunk
+                Created a repository branch...
+                Using shared repository: ...
+                ''')
+
+    def assertParent(self, expected_parent, branch):
+        """Verify that the parent is not None and is set correctly."""
+        actual_parent = branch.get_parent()
+        self.assertIsSameRealPath(urlutils.local_path_to_url(expected_parent),
+                                  branch.get_parent())
+
+
+class TestSwitchParentLocation(TestSwitchParentLocationBase):
+
+    def _checkout_and_switch(self, option=''):
+        self.script_runner.run_script(self, '''
+                $ bzr checkout %(option)s repo/trunk checkout
+                $ cd checkout
+                $ bzr switch --create-branch switched
+                2>Tree is up to date at revision 0.
+                2>Switched to branch:...switched...
+                $ cd ..
+                ''' % locals())
+        bound_branch = branch.Branch.open_containing('checkout')[0]
+        master_branch = branch.Branch.open_containing('repo/switched')[0]
+        return (bound_branch, master_branch)
+
+    def test_switch_parent_lightweight(self):
+        """Lightweight checkout using bzr switch."""
+        bb, mb = self._checkout_and_switch(option='--lightweight')
+        self.assertParent('repo/trunk', bb)
+        self.assertParent('repo/trunk', mb)
+
+    def test_switch_parent_heavyweight(self):
+        """Heavyweight checkout using bzr switch."""
+        bb, mb = self._checkout_and_switch()
+        self.assertParent('repo/trunk', bb)
+        self.assertParent('repo/trunk', mb)
+
+
+class TestSwitchDoesntOpenMasterBranch(TestCaseWithTransport):
+    # See https://bugs.launchpad.net/bzr/+bug/812285
+    # "bzr switch --create-branch" can point the new branch's parent to the
+    # master branch, but it doesn't have to open it to do so.
+
+    def test_switch_create_doesnt_open_master_branch(self):
+        master = self.make_branch_and_tree('master')
+        master.commit('one')
+        # Note: not a lightweight checkout
+        checkout = master.branch.create_checkout('checkout')
+        opened = []
+        def open_hook(branch):
+            # Just append the final directory of the branch
+            name = branch.base.rstrip('/').rsplit('/', 1)[1]
+            opened.append(name)
+        branch.Branch.hooks.install_named_hook('open', open_hook,
+                                               'open_hook_logger')
+        self.run_bzr('switch --create-branch -d checkout feature')
+        # We only open the master branch 1 time.
+        # This test should be cleaner to write, but see bug:
+        #  https://bugs.launchpad.net/bzr/+bug/812295
+        self.assertEqual(1, opened.count('master'))

=== modified file 'bzrlib/tests/blackbox/test_tags.py'
--- bzrlib/tests/blackbox/test_tags.py	2010-03-24 14:16:15 +0000
+++ bzrlib/tests/blackbox/test_tags.py	2011-09-25 14:36:30 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,30 +17,22 @@
 """Tests for commands related to tags"""
 
 from bzrlib import (
-    bzrdir,
+    errors,
     tag,
+    transform,
     )
 from bzrlib.branch import (
     Branch,
     )
-from bzrlib.bzrdir import BzrDir
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.repository import (
-    Repository,
+from bzrlib.tests import (
+    script,
+    TestCaseWithTransport,
     )
 from bzrlib.workingtree import WorkingTree
 
 
 class TestTagging(TestCaseWithTransport):
 
-    # as of 0.14, the default format doesn't do tags so we need to use a
-    # specific format
-
-    def make_branch_and_tree(self, relpath):
-        format = bzrdir.format_registry.make_bzrdir('dirstate-with-subtree')
-        return TestCaseWithTransport.make_branch_and_tree(self, relpath,
-            format=format)
-
     def test_tag_command_help(self):
         out, err = self.run_bzr('help tag')
         self.assertContainsRe(out, 'Create, remove or modify a tag')
@@ -60,7 +52,7 @@
         Branch.hooks.install_named_hook('automatic_tag_name',
             get_tag_name, 'get tag name')
         out, err = self.run_bzr('tag -d branch')
-        self.assertContainsRe(out, 'Created tag mytag.')
+        self.assertContainsRe(err, 'Created tag mytag.')
 
     def test_tag_current_rev(self):
         t = self.make_branch_and_tree('branch')
@@ -68,7 +60,7 @@
             rev_id='first-revid')
         # make a tag through the command line
         out, err = self.run_bzr('tag -d branch NEWTAG')
-        self.assertContainsRe(out, 'Created tag NEWTAG.')
+        self.assertContainsRe(err, 'Created tag NEWTAG.')
         # tag should be observable through the api
         self.assertEquals(t.branch.tags.get_tag_dict(),
                 dict(NEWTAG='first-revid'))
@@ -83,10 +75,25 @@
         # can also delete an existing tag
         out, err = self.run_bzr('tag --delete -d branch tag2')
         # cannot replace an existing tag normally
-        out, err = self.run_bzr('tag -d branch NEWTAG', retcode=3)
+        out, err = self.run_bzr('tag -d branch NEWTAG -r0', retcode=3)
         self.assertContainsRe(err, 'Tag NEWTAG already exists\\.')
         # ... but can if you use --force
-        out, err = self.run_bzr('tag -d branch NEWTAG --force')
+        out, err = self.run_bzr('tag -d branch NEWTAG --force -r0')
+        self.assertEquals("Updated tag NEWTAG.\n", err)
+
+    def test_tag_same_revision(self):
+        t = self.make_branch_and_tree('branch')
+        t.commit(allow_pointless=True, message='initial commit',
+            rev_id='first-revid')
+        t.commit(allow_pointless=True, message='second commit',
+            rev_id='second-revid')
+        out, err = self.run_bzr('tag -rrevid:first-revid -d branch NEWTAG')
+        out, err = self.run_bzr('tag -rrevid:first-revid -d branch NEWTAG')
+        self.assertContainsRe(err,
+            'Tag NEWTAG already exists for that revision\\.')
+        out, err = self.run_bzr('tag -rrevid:second-revid -d branch NEWTAG',
+            retcode=3)
+        self.assertContainsRe(err, 'Tag NEWTAG already exists\\.')
 
     def test_tag_delete_requires_name(self):
         out, err = self.run_bzr('tag -d branch', retcode=3)
@@ -123,6 +130,80 @@
         b3 = Branch.open('branch3')
         self.assertEquals(b3.tags.lookup_tag('tag1'), 'first-revid')
 
+    def make_master_and_checkout(self):
+        builder = self.make_branch_builder('master')
+        builder.build_commit(message='Initial commit.', rev_id='rev-1')
+        master = builder.get_branch()
+        child = master.create_checkout(self.get_url('child'))
+        return master, child
+
+    def make_fork(self, branch):
+        fork = branch.create_clone_on_transport(self.get_transport('fork'))
+        self.addCleanup(fork.lock_write().unlock)
+        with transform.TransformPreview(fork.basis_tree()) as tt:
+            tt.commit(fork, message='Commit in fork.', revision_id='fork-0')
+        with transform.TransformPreview(fork.basis_tree()) as tt:
+            tt.commit(fork, message='Commit in fork.', revision_id='fork-1')
+        return fork
+
+    def test_merge_without_commit_does_not_propagate_tags_to_master(self):
+        """'bzr merge' alone does not propagate tags to a master branch.
+
+        (If the user runs 'bzr commit', then that is when the tags from the
+        merge are propagated.)
+        """
+        master, child = self.make_master_and_checkout()
+        fork = self.make_fork(master)
+        fork.tags.set_tag('new-tag', fork.last_revision())
+        self.run_bzr(['merge', '../fork'], working_dir='child')
+        self.assertEqual({}, master.tags.get_tag_dict())
+
+    def test_commit_in_heavyweight_checkout_copies_tags_to_master(self):
+        master, child = self.make_master_and_checkout()
+        fork = self.make_fork(master)
+        fork.tags.set_tag('new-tag', fork.last_revision())
+        fork.tags.set_tag('non-ancestry-tag', 'fork-0')
+        fork.tags.set_tag('absent-tag', 'absent-rev')
+        script.run_script(self, """
+            $ cd child
+            $ bzr merge ../fork
+            $ bzr commit -m "Merge fork."
+            2>Committing to: .../master/
+            2>Committed revision 2.
+            """, null_output_matches_anything=True)
+        # Merge copied the tag to child and commit propagated it to master
+        expected_tag_dict = {
+            'new-tag': fork.last_revision(),
+            'non-ancestry-tag': 'fork-0',
+            'absent-tag': 'absent-rev',
+            }
+        self.assertEqual(expected_tag_dict, child.branch.tags.get_tag_dict())
+        self.assertEqual(expected_tag_dict, master.tags.get_tag_dict())
+        # Revisions not in ancestry but named in tags are present
+        child.branch.repository.get_revision('fork-0')
+        master.repository.get_revision('fork-0')
+
+    def test_commit_in_heavyweight_checkout_reports_tag_conflict(self):
+        master, child = self.make_master_and_checkout()
+        fork = self.make_fork(master)
+        fork.tags.set_tag('new-tag', fork.last_revision())
+        master_r1 = master.last_revision()
+        master.tags.set_tag('new-tag', master_r1)
+        script.run_script(self, """
+            $ cd child
+            $ bzr merge ../fork
+            $ bzr commit -m "Merge fork."
+            2>Committing to: .../master/
+            2>Conflicting tags in bound branch:
+            2>    new-tag
+            2>Committed revision 2.
+            """, null_output_matches_anything=True)
+        # Merge copied the tag to child.  master's conflicting tag is unchanged.
+        self.assertEqual(
+            {'new-tag': fork.last_revision()}, child.branch.tags.get_tag_dict())
+        self.assertEqual(
+            {'new-tag': master_r1}, master.tags.get_tag_dict())
+
     def test_list_tags(self):
         tree1 = self.make_branch_and_tree('branch1')
         tree1.commit(allow_pointless=True, message='revision 1',
@@ -132,33 +213,42 @@
 
         b1 = tree1.branch
         # note how the tag for revid-1 sorts after the one for revid-2
-        b1.tags.set_tag(u'tagA\u30d0', 'revid-2')
-        b1.tags.set_tag(u'tagB\u30d0', 'missing') # not present in repository
-        b1.tags.set_tag(u'tagC\u30d0', 'revid-1')
+        b1.tags.set_tag(u'tag1\u30d0', 'revid-2')
+        b1.tags.set_tag(u'tag10\u30d0', 'missing') # not present in repository
+        b1.tags.set_tag(u'tag2\u30d0', 'revid-1')
+
+        # natural order
+        out, err = self.run_bzr('tags -d branch1',
+                                encoding='utf-8')
+        self.assertEquals(err, '')
+        self.assertContainsRe(out, (u'^tag1\u30d0  *2\ntag2\u30d0  *1\n' +
+            u'tag10\u30d0 *\\?\n').encode('utf-8'))
 
         # lexicographical order
-        out, err = self.run_bzr('tags -d branch1', encoding='utf-8')
+        out, err = self.run_bzr('tags --sort=alpha -d branch1',
+                                encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagA\u30d0  *2\ntagB\u30d0  *\\?\n' +
-            u'tagC\u30d0 *1\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag10\u30d0  *\\?\ntag1\u30d0  *2\n' +
+            u'tag2\u30d0 *1\n').encode('utf-8'))
 
-        out, err = self.run_bzr('tags --show-ids -d branch1', encoding='utf-8')
+        out, err = self.run_bzr('tags --sort=alpha --show-ids -d branch1',
+                                encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagA\u30d0  *revid-2\n' +
-            u'tagB\u30d0  *missing\ntagC\u30d0 *revid-1\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag10\u30d0  *missing\n' +
+            u'tag1\u30d0  *revid-2\ntag2\u30d0 *revid-1\n').encode('utf-8'))
 
         # chronological order
         out, err = self.run_bzr('tags --sort=time -d branch1',
                 encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagC\u30d0  *1\ntagA\u30d0  *2\n' +
-            u'tagB\u30d0 *\\?\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag2\u30d0  *1\ntag1\u30d0  *2\n' +
+            u'tag10\u30d0 *\\?\n').encode('utf-8'))
 
         out, err = self.run_bzr('tags --sort=time --show-ids -d branch1',
                 encoding='utf-8')
         self.assertEquals(err, '')
-        self.assertContainsRe(out, (u'^tagC\u30d0  *revid-1\n' +
-            u'tagA\u30d0  *revid-2\ntagB\u30d0 *missing\n').encode('utf-8'))
+        self.assertContainsRe(out, (u'^tag2\u30d0  *revid-1\n' +
+            u'tag1\u30d0  *revid-2\ntag10\u30d0 *missing\n').encode('utf-8'))
 
         # now test dotted revnos
         tree2 = tree1.bzrdir.sprout('branch2').open_workingtree()
@@ -179,6 +269,17 @@
         self.assertEquals(err, '')
         self.assertContainsRe(out, r'tagD  *3\n')
 
+    def test_list_tags_dotted_revnos_unsupported(self):
+        tree = self.make_branch_and_tree('branch')
+        rev1 = tree.commit("rev1")
+        tree.branch.tags.set_tag("mytag", rev1)
+        def revision_id_to_dotted_revno(self, revid):
+            raise errors.UnsupportedOperation(revision_id_to_dotted_revno, self)
+        self.overrideAttr(Branch, "revision_id_to_dotted_revno",
+            revision_id_to_dotted_revno)
+        out, err = self.run_bzr('tags -d branch', encoding='utf-8')
+        self.assertEquals(out, 'mytag                ?\n')
+
     def test_list_tags_revision_filtering(self):
         tree1 = self.make_branch_and_tree('.')
         tree1.commit(allow_pointless=True, message='revision 1',
@@ -210,6 +311,40 @@
             error_regexes=["bzr: ERROR: Requested revision: '123.123' "
                 "does not exist in branch:"])
 
+    def test_sort_tags_custom(self):
+        def sort_by_dots(branch, tags):
+            def sort_key((tag, revid)):
+                return tag.count(".")
+            tags.sort(key=sort_key)
+
+        # Register a custom sort method
+        tag.tag_sort_methods.register("dots", sort_by_dots, "Sort by dots.")
+        self.addCleanup(tag.tag_sort_methods.remove, "dots")
+
+        tree1 = self.make_branch_and_tree('branch1')
+        tree1.commit(allow_pointless=True, message='revision 1',
+                rev_id='revid-1', timestamp=10)
+        tree1.commit(allow_pointless=True, message='revision 2',
+                rev_id='revid-2', timestamp=15)
+
+        b1 = tree1.branch
+
+        b1.tags.set_tag(u'tag..', 'revid-2')
+        b1.tags.set_tag(u'tag....', 'missing') # not present in repository
+        b1.tags.set_tag(u'tag.', 'revid-1')
+        b1.tags.set_tag(u'tag...', 'revid-1')
+        b1.tags.set_tag(u'tag....', 'revid-1')
+
+        # sorted by number of dots
+        out, err = self.run_bzr('tags --sort=dots -d branch1')
+        self.assertEquals(err, '')
+        self.assertEquals([
+            'tag.                 1',
+            'tag..                2',
+            'tag...               1',
+            'tag....              1'],
+            out.splitlines())
+
     def _check_tag_filter(self, argstr, expected_revnos):
         #upper bound of laziness
         out, err = self.run_bzr('tags ' + argstr)
@@ -231,10 +366,43 @@
         self.assertContainsRe(out,
                 'Conflicting tags:\n.*' + tagname.encode('utf-8'))
         # pull should give a warning about the tags
-        out, err = self.run_bzr('pull -d one two', encoding='utf-8')
+        out, err = self.run_bzr('pull -d one two', encoding='utf-8',
+            retcode=1)
         self.assertContainsRe(out,
                 'Conflicting tags:\n.*' + tagname.encode('utf-8'))
         # merge should give a warning about the tags -- not implemented yet
         ## out, err = self.run_bzr('merge -d one two', encoding='utf-8')
         ## self.assertContainsRe(out,
         ##         'Conflicting tags:\n.*' + tagname.encode('utf-8'))
+
+    def test_tag_quiet(self):
+        t1 = self.make_branch_and_tree('')
+        out, err = self.run_bzr('tag --quiet test1')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
+    def test_tag_delete_quiet(self):
+        t1 = self.make_branch_and_tree('')
+        self.run_bzr('tag test1')
+        out, err = self.run_bzr('tag --delete --quiet test1')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
+    def test_tags_with_mainline_ghosts(self):
+        tree = self.make_branch_and_tree('tree1')
+        tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
+        tree.add('')
+        tree.commit('msg1', rev_id='rev1')
+        tree.commit('msg2', rev_id='rev2')
+        tree.branch.tags.set_tag('unknown', 'out-of-mainline')
+        tree.branch.tags.set_tag('ghost', 'spooky')
+        tree.branch.tags.set_tag('tag1', 'rev1')
+        tree.branch.tags.set_tag('tag2', 'rev2')
+
+        out, err = self.run_bzr('tags -d tree1', encoding='utf-8')
+        self.assertEqual(out,
+            'ghost                ?\n'
+            'tag1                 1\n'
+            'tag2                 2\n'
+            'unknown              ?\n')
+        self.assertEqual('', err)

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2010-06-11 07:32:12 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2011-09-26 15:21:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -60,16 +60,16 @@
     def test_revert(self):
         self.run_bzr('init')
 
-        file('hello', 'wt').write('foo')
+        with file('hello', 'wt') as f: f.write('foo')
         self.run_bzr('add hello')
         self.run_bzr('commit -m setup hello')
 
-        file('goodbye', 'wt').write('baz')
+        with file('goodbye', 'wt') as f: f.write('baz')
         self.run_bzr('add goodbye')
         self.run_bzr('commit -m setup goodbye')
 
-        file('hello', 'wt').write('bar')
-        file('goodbye', 'wt').write('qux')
+        with file('hello', 'wt') as f: f.write('bar')
+        with file('goodbye', 'wt') as f: f.write('qux')
         self.run_bzr('revert hello')
         self.check_file_contents('hello', 'foo')
         self.check_file_contents('goodbye', 'qux')
@@ -88,7 +88,7 @@
             self.run_bzr('commit -m f')
             os.unlink('symlink')
             self.run_bzr('revert')
-            self.failUnlessExists('symlink')
+            self.assertPathExists('symlink')
             os.unlink('symlink')
             os.symlink('a-different-path', 'symlink')
             self.run_bzr('revert')
@@ -97,7 +97,7 @@
         else:
             self.log("skipping revert symlink tests")
 
-        file('hello', 'wt').write('xyz')
+        with file('hello', 'wt') as f: f.write('xyz')
         self.run_bzr('commit -m xyz hello')
         self.run_bzr('revert -r 1 hello')
         self.check_file_contents('hello', 'foo')
@@ -109,10 +109,10 @@
 
     def example_branch(test):
         test.run_bzr('init')
-        file('hello', 'wt').write('foo')
+        with file('hello', 'wt') as f: f.write('foo')
         test.run_bzr('add hello')
         test.run_bzr('commit -m setup hello')
-        file('goodbye', 'wt').write('baz')
+        with file('goodbye', 'wt') as f: f.write('baz')
         test.run_bzr('add goodbye')
         test.run_bzr('commit -m setup goodbye')
 
@@ -127,15 +127,15 @@
         os.chdir('..')
         self.run_bzr('branch a b')
         os.chdir('b')
-        open('b', 'wb').write('else\n')
+        with open('b', 'wb') as f: f.write('else\n')
         self.run_bzr('add b')
         self.run_bzr(['commit', '-m', 'added b'])
 
         os.chdir('../a')
         out = self.run_bzr('pull --verbose ../b')[0]
-        self.failIfEqual(out.find('Added Revisions:'), -1)
-        self.failIfEqual(out.find('message:\n  added b'), -1)
-        self.failIfEqual(out.find('added b'), -1)
+        self.assertNotEqual(out.find('Added Revisions:'), -1)
+        self.assertNotEqual(out.find('message:\n  added b'), -1)
+        self.assertNotEqual(out.find('added b'), -1)
 
         # Check that --overwrite --verbose prints out the removed entries
         self.run_bzr('commit -m foo --unchanged')
@@ -145,17 +145,17 @@
         out = self.run_bzr('pull --overwrite --verbose ../a')[0]
 
         remove_loc = out.find('Removed Revisions:')
-        self.failIfEqual(remove_loc, -1)
+        self.assertNotEqual(remove_loc, -1)
         added_loc = out.find('Added Revisions:')
-        self.failIfEqual(added_loc, -1)
+        self.assertNotEqual(added_loc, -1)
 
         removed_message = out.find('message:\n  baz')
-        self.failIfEqual(removed_message, -1)
-        self.failUnless(remove_loc < removed_message < added_loc)
+        self.assertNotEqual(removed_message, -1)
+        self.assertTrue(remove_loc < removed_message < added_loc)
 
         added_message = out.find('message:\n  foo')
-        self.failIfEqual(added_message, -1)
-        self.failUnless(added_loc < added_message)
+        self.assertNotEqual(added_message, -1)
+        self.assertTrue(added_loc < added_message)
 
     def test_locations(self):
         """Using and remembering different locations"""
@@ -195,21 +195,21 @@
         """Create a conflicted tree"""
         os.mkdir('base')
         os.chdir('base')
-        file('hello', 'wb').write("hi world")
-        file('answer', 'wb').write("42")
+        with file('hello', 'wb') as f: f.write("hi world")
+        with file('answer', 'wb') as f: f.write("42")
         self.run_bzr('init')
         self.run_bzr('add')
         self.run_bzr('commit -m base')
         self.run_bzr('branch . ../other')
         self.run_bzr('branch . ../this')
         os.chdir('../other')
-        file('hello', 'wb').write("Hello.")
-        file('answer', 'wb').write("Is anyone there?")
+        with file('hello', 'wb') as f: f.write("Hello.")
+        with file('answer', 'wb') as f: f.write("Is anyone there?")
         self.run_bzr('commit -m other')
         os.chdir('../this')
-        file('hello', 'wb').write("Hello, world")
+        with file('hello', 'wb') as f: f.write("Hello, world")
         self.run_bzr('mv answer question')
-        file('question', 'wb').write("What do you get when you multiply six"
+        with file('question', 'wb') as f: f.write("What do you get when you multiply six"
                                    "times nine?")
         self.run_bzr('commit -m this')
 
@@ -315,37 +315,28 @@
         cmd_name = 'test-command'
         if sys.platform == 'win32':
             cmd_name += '.bat'
-        oldpath = os.environ.get('BZRPATH', None)
-        try:
-            if 'BZRPATH' in os.environ:
-                del os.environ['BZRPATH']
-
-            f = file(cmd_name, 'wb')
-            if sys.platform == 'win32':
-                f.write('@echo off\n')
-            else:
-                f.write('#!/bin/sh\n')
-            # f.write('echo Hello from test-command')
-            f.close()
-            os.chmod(cmd_name, 0755)
-
-            # It should not find the command in the local
-            # directory by default, since it is not in my path
-            self.run_bzr(cmd_name, retcode=3)
-
-            # Now put it into my path
-            os.environ['BZRPATH'] = '.'
-
-            self.run_bzr(cmd_name)
-
-            # Make sure empty path elements are ignored
-            os.environ['BZRPATH'] = os.pathsep
-
-            self.run_bzr(cmd_name, retcode=3)
-
-        finally:
-            if oldpath:
-                os.environ['BZRPATH'] = oldpath
+        self.overrideEnv('BZRPATH', None)
+
+        f = file(cmd_name, 'wb')
+        if sys.platform == 'win32':
+            f.write('@echo off\n')
+        else:
+            f.write('#!/bin/sh\n')
+        # f.write('echo Hello from test-command')
+        f.close()
+        os.chmod(cmd_name, 0755)
+
+        # It should not find the command in the local
+        # directory by default, since it is not in my path
+        self.run_bzr(cmd_name, retcode=3)
+
+        # Now put it into my path
+        self.overrideEnv('BZRPATH', '.')
+        self.run_bzr(cmd_name)
+
+        # Make sure empty path elements are ignored
+        self.overrideEnv('BZRPATH', os.pathsep)
+        self.run_bzr(cmd_name, retcode=3)
 
 
 def listdir_sorted(dir):
@@ -447,7 +438,7 @@
 
         progress("file with spaces in name")
         mkdir('sub directory')
-        file('sub directory/file with spaces ', 'wt').write('see how this works\n')
+        with file('sub directory/file with spaces ', 'wt') as f: f.write('see how this works\n')
         self.run_bzr('add .')
         self.run_bzr('diff', retcode=1)
         self.run_bzr('commit -m add-spaces')
@@ -573,7 +564,7 @@
         url = self.get_readonly_url('from')
         self.run_bzr(['branch', url, 'to'])
         branch = Branch.open('to')
-        self.assertEqual(1, len(branch.revision_history()))
+        self.assertEqual(1, branch.last_revision_info()[0])
         # the branch should be set in to to from
         self.assertEqual(url + '/', branch.get_parent())
 
@@ -599,7 +590,7 @@
         os.mkdir('my-branch')
         os.chdir('my-branch')
         self.run_bzr('init')
-        file('hello', 'wt').write('foo')
+        with file('hello', 'wt') as f: f.write('foo')
         self.run_bzr('add hello')
         self.run_bzr('commit -m setup')
 

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2010-03-01 16:18:43 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2011-10-14 13:56:45 +0000
@@ -18,11 +18,14 @@
 
 import os
 
-from bzrlib import uncommit, workingtree
+from bzrlib import uncommit
 from bzrlib.bzrdir import BzrDirMetaFormat1
-from bzrlib.errors import BzrError, BoundBranchOutOfDate
+from bzrlib.errors import BoundBranchOutOfDate
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.tests.script import ScriptRunner
+from bzrlib.tests.script import (
+    run_script,
+    ScriptRunner,
+    )
 
 
 class TestUncommit(TestCaseWithTransport):
@@ -61,6 +64,20 @@
         out, err = self.run_bzr('status')
         self.assertEquals(out, 'modified:\n  a\n')
 
+    def test_uncommit_interactive(self):
+        """Uncommit seeks confirmation, and doesn't proceed without it."""
+        wt = self.create_simple_tree()
+        os.chdir('tree')
+        run_script(self, """    
+        $ bzr uncommit
+        ...
+        The above revision(s) will be removed.
+        2>Uncommit these revisions? ([y]es, [n]o): no
+        <n
+        Canceled
+        """)
+        self.assertEqual(['a2'], wt.get_parent_ids())
+
     def test_uncommit_no_history(self):
         wt = self.make_branch_and_tree('tree')
         out, err = self.run_bzr('uncommit --force', retcode=1)
@@ -102,8 +119,8 @@
         t_a.commit('commit 3')
         b = t_a.branch.create_checkout('b').branch
         uncommit.uncommit(b)
-        self.assertEqual(len(b.revision_history()), 2)
-        self.assertEqual(len(t_a.branch.revision_history()), 2)
+        self.assertEqual(b.last_revision_info()[0], 2)
+        self.assertEqual(t_a.branch.last_revision_info()[0], 2)
         # update A's tree to not have the uncommitted revision referenced.
         t_a.update()
         t_a.commit('commit 3b')
@@ -263,3 +280,17 @@
         tree.commit(u'\u1234 message')
         out, err = self.run_bzr('uncommit --force tree', encoding='ascii')
         self.assertContainsRe(out, r'\? message')
+
+    def test_uncommit_removes_tags(self):
+        tree = self.make_branch_and_tree('tree')
+        revid = tree.commit('message')
+        tree.branch.tags.set_tag("atag", revid)
+        out, err = self.run_bzr('uncommit --force tree')
+        self.assertEquals({}, tree.branch.tags.get_tag_dict())
+
+    def test_uncommit_keep_tags(self):
+        tree = self.make_branch_and_tree('tree')
+        revid = tree.commit('message')
+        tree.branch.tags.set_tag("atag", revid)
+        out, err = self.run_bzr('uncommit --keep-tags --force tree')
+        self.assertEquals({"atag": revid}, tree.branch.tags.get_tag_dict())

=== modified file 'bzrlib/tests/blackbox/test_update.py'
--- bzrlib/tests/blackbox/test_update.py	2010-04-14 04:48:00 +0000
+++ bzrlib/tests/blackbox/test_update.py	2011-09-29 01:03:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,14 +18,12 @@
 """Tests for the update command of bzr."""
 
 import os
-import re
 
 from bzrlib import (
     branch,
     bzrdir,
     osutils,
     tests,
-    urlutils,
     workingtree,
     )
 from bzrlib.tests.script import ScriptRunner
@@ -90,7 +88,7 @@
 Updated to revision 1 of branch %s
 """ % osutils.pathjoin(self.test_dir, 'branch',),
                          err)
-        self.failUnlessExists('branch/file')
+        self.assertPathExists('branch/file')
 
     def test_update_out_of_date_light_checkout(self):
         self.make_branch_and_tree('branch')
@@ -140,10 +138,9 @@
         # smoke test for doing an update of a checkout of a bound
         # branch with local commits.
         master = self.make_branch_and_tree('master')
+        master.commit('first commit')
         # make a bound branch
         self.run_bzr('checkout master child')
-        # get an object form of child
-        child = workingtree.WorkingTree.open('child')
         # check that out
         self.run_bzr('checkout --lightweight child checkout')
         # get an object form of the checkout to manipulate
@@ -158,6 +155,8 @@
         a_file = file('child/file_b', 'wt')
         a_file.write('Foo')
         a_file.close()
+        # get an object form of child
+        child = workingtree.WorkingTree.open('child')
         child.add(['file_b'])
         child_tip = child.commit('add file_b', local=True)
         # check checkout
@@ -174,14 +173,14 @@
 All changes applied successfully.
 +N  file
 All changes applied successfully.
-Updated to revision 1 of branch %s
+Updated to revision 2 of branch %s
 Your local commits will now show as pending merges with 'bzr status', and can be committed with 'bzr commit'.
 """ % osutils.pathjoin(self.test_dir, 'master',),
                          err)
         self.assertEqual([master_tip, child_tip], wt.get_parent_ids())
-        self.failUnlessExists('checkout/file')
-        self.failUnlessExists('checkout/file_b')
-        self.failUnlessExists('checkout/file_c')
+        self.assertPathExists('checkout/file')
+        self.assertPathExists('checkout/file_b')
+        self.assertPathExists('checkout/file_c')
         self.assertTrue(wt.has_filename('file_c'))
 
     def test_update_with_merges(self):
@@ -299,8 +298,8 @@
 2>All changes applied successfully.
 2>Updated to revision 1 of .../master
 ''')
-        self.failUnlessExists('./file1')
-        self.failIfExists('./file2')
+        self.assertPathExists('./file1')
+        self.assertPathDoesNotExist('./file2')
         self.assertEquals(['m1'], master.get_parent_ids())
 
     def test_update_dash_r_outside_history(self):
@@ -359,6 +358,44 @@
 2>Updated to revision 2 of branch .../master
 ''')
 
+    def test_update_show_base(self):
+        """bzr update support --show-base
+
+        see https://bugs.launchpad.net/bzr/+bug/202374"""
+
+        tree=self.make_branch_and_tree('.')
+
+        f = open('hello','wt')
+        f.write('foo')
+        f.close()
+        tree.add('hello')
+        tree.commit('fie')
+
+        f = open('hello','wt')
+        f.write('fee')
+        f.close()
+        tree.commit('fee')
+
+        #tree.update() gives no such revision, so ...
+        self.run_bzr(['update','-r1'])
+
+        #create conflict
+        f = open('hello','wt')
+        f.write('fie')
+        f.close()
+
+        out, err = self.run_bzr(['update','--show-base'],retcode=1)
+
+        # check for conflict notification
+        self.assertContainsString(err,
+                                  ' M  hello\nText conflict in hello\n1 conflicts encountered.\n')
+        
+        self.assertEqualDiff('<<<<<<< TREE\n'
+                             'fie||||||| BASE-REVISION\n'
+                             'foo=======\n'
+                             'fee>>>>>>> MERGE-SOURCE\n',
+                             open('hello').read())
+
     def test_update_checkout_prevent_double_merge(self):
         """"Launchpad bug 113809 in bzr "update performs two merges"
         https://launchpad.net/bugs/113809"""
@@ -417,3 +454,31 @@
 >>>>>>> MERGE-SOURCE
 ''',
                              'lightweight/file')
+
+
+    def test_no_upgrade_single_file(self):
+        """There's one basis revision per tree.
+
+        Since you can't actually change the basis for a single file at the
+        moment, we don't let you think you can.
+
+        See bug 557886.
+        """
+        self.make_branch_and_tree('.')
+        self.build_tree_contents([('a/',),
+            ('a/file', 'content')])
+        sr = ScriptRunner()
+        sr.run_script(self, '''
+            $ bzr update ./a
+            2>bzr: ERROR: bzr update can only update a whole tree, not a file or subdirectory
+            $ bzr update ./a/file
+            2>bzr: ERROR: bzr update can only update a whole tree, not a file or subdirectory
+            $ bzr update .
+            2>Tree is up to date at revision 0 of branch ...
+            $ cd a
+            $ bzr update .
+            2>bzr: ERROR: bzr update can only update a whole tree, not a file or subdirectory
+            # however, you can update the whole tree from a subdirectory
+            $ bzr update
+            2>Tree is up to date at revision 0 of branch ...
+            ''')

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2011-08-19 19:36:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,26 +20,56 @@
 
 from bzrlib import (
     bzrdir,
-    repository,
-    transport,
+    controldir,
+    lockable_files,
+    ui,
+    urlutils,
     )
 from bzrlib.tests import (
     features,
-    TestCaseInTempDir,
     TestCaseWithTransport,
     )
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.repofmt.knitrepo import (
-    RepositoryFormatKnit1,
-    )
+from bzrlib.repofmt.knitpack_repo import RepositoryFormatKnitPack1
+
+
+class OldBzrDir(bzrdir.BzrDirMeta1):
+    """An test bzr dir implementation"""
+
+    def needs_format_conversion(self, format):
+        return not isinstance(format, self.__class__)
+
+
+class ConvertOldTestToMeta(controldir.Converter):
+    """A trivial converter, used for testing."""
+
+    def convert(self, to_convert, pb):
+        ui.ui_factory.note('starting upgrade from old test format to 2a')
+        to_convert.control_transport.put_bytes(
+            'branch-format',
+            bzrdir.BzrDirMetaFormat1().get_format_string(),
+            mode=to_convert._get_file_mode())
+        return bzrdir.BzrDir.open(to_convert.user_url)
+
+
+class OldBzrDirFormat(bzrdir.BzrDirMetaFormat1):
+
+    _lock_class = lockable_files.TransportLock
+
+    def get_converter(self, format=None):
+        return ConvertOldTestToMeta()
+
+    def get_format_string(self):
+        return "Ancient Test Format"
+
+    def _open(self, transport):
+        return OldBzrDir(transport, self)
 
 
 class TestWithUpgradableBranches(TestCaseWithTransport):
 
     def setUp(self):
         super(TestWithUpgradableBranches, self).setUp()
-        self.addCleanup(bzrdir.BzrDirFormat._set_default_format,
-                        bzrdir.BzrDirFormat.get_default_format())
 
     def make_current_format_branch_and_checkout(self):
         current_tree = self.make_branch_and_tree('current_format_branch',
@@ -47,44 +77,43 @@
         current_tree.branch.create_checkout(
             self.get_url('current_format_checkout'), lightweight=True)
 
-    def make_format_5_branch(self):
-        # setup a format 5 branch we can upgrade from.
-        self.make_branch_and_tree('format_5_branch',
-                                  format=bzrdir.BzrDirFormat5())
-
-    def make_metadir_weave_branch(self):
-        self.make_branch_and_tree('metadir_weave_branch', format='metaweave')
-
     def test_readonly_url_error(self):
-        self.make_format_5_branch()
+        self.make_branch_and_tree("old_format_branch", format="knit")
         (out, err) = self.run_bzr(
-            ['upgrade', self.get_readonly_url('format_5_branch')], retcode=3)
-        self.assertEqual(out, "")
-        self.assertEqual(err, "bzr: ERROR: Upgrade URL cannot work with readonly URLs.\n")
+            ['upgrade', self.get_readonly_url("old_format_branch")], retcode=3)
+        err_msg = 'Upgrade URL cannot work with readonly URLs.'
+        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
+                             % (err_msg, err_msg),
+                             err)
 
     def test_upgrade_up_to_date(self):
         self.make_current_format_branch_and_checkout()
         # when up to date we should get a message to that effect
         (out, err) = self.run_bzr('upgrade current_format_branch', retcode=3)
-        self.assertEqual("", out)
-        self.assertEqualDiff("bzr: ERROR: The branch format Meta "
-                             "directory format 1 is already at the most "
-                             "recent format.\n", err)
+        err_msg = ('The branch format %s is already at the most recent format.'
+                   % ('Meta directory format 1'))
+        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
+                             % (err_msg, err_msg),
+                             err)
 
     def test_upgrade_up_to_date_checkout_warns_branch_left_alone(self):
         self.make_current_format_branch_and_checkout()
         # when upgrading a checkout, the branch location and a suggestion
         # to upgrade it should be emitted even if the checkout is up to
         # date
+        burl = self.get_transport('current_format_branch').local_abspath(".")
+        curl = self.get_transport('current_format_checkout').local_abspath(".")
         (out, err) = self.run_bzr('upgrade current_format_checkout', retcode=3)
-        self.assertEqual("This is a checkout. The branch (%s) needs to be "
-                         "upgraded separately.\n"
-                         % transport.get_transport(
-                self.get_url('current_format_branch')).base,
-                         out)
-        self.assertEqualDiff("bzr: ERROR: The branch format Meta "
-                             "directory format 1 is already at the most "
-                             "recent format.\n", err)
+        self.assertEqual(
+            'Upgrading branch %s/ ...\nThis is a checkout.'
+            ' The branch (%s/) needs to be upgraded separately.\n'
+            % (curl, burl),
+            out)
+        msg = 'The branch format %s is already at the most recent format.' % (
+            'Meta directory format 1')
+        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
+                             % (msg, msg),
+                             err)
 
     def test_upgrade_checkout(self):
         # upgrading a checkout should work
@@ -95,60 +124,95 @@
         # anonymous branch
         pass
 
-    def test_ugrade_branch_in_repo(self):
+    def test_upgrade_branch_in_repo(self):
         # upgrading a branch in a repo should warn about not upgrading the repo
         pass
 
-    def test_upgrade_explicit_metaformat(self):
-        # users can force an upgrade to metadir format.
-        self.make_format_5_branch()
-        url = transport.get_transport(self.get_url('format_5_branch')).base
+    def test_upgrade_control_dir(self):
+        old_format = OldBzrDirFormat()
+        self.addCleanup(bzrdir.BzrProber.formats.remove,
+            old_format.get_format_string())
+        bzrdir.BzrProber.formats.register(old_format.get_format_string(),
+            old_format)
+        self.addCleanup(controldir.ControlDirFormat._set_default_format,
+                        controldir.ControlDirFormat.get_default_format())
+
+        # setup an old format branch we can upgrade from.
+        path = 'old_format_branch'
+        self.make_branch_and_tree(path, format=old_format)
+        transport = self.get_transport(path)
+        url = transport.base
+        display_url = transport.local_abspath('.')
         # check --format takes effect
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
+        controldir.ControlDirFormat._set_default_format(old_format)
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
-            ['upgrade', '--format=metaweave', url])
-        self.assertEqualDiff("""starting upgrade of %s
-making backup of %s.bzr
-  to %s%s
-starting upgrade from format 5 to 6
-adding prefixes to weaves
-adding prefixes to revision-store
-starting upgrade from format 6 to metadir
+            ['upgrade', '--format=2a', url])
+        self.assertEqualDiff("""Upgrading branch %s/ ...
+starting upgrade of %s/
+making backup of %s/.bzr
+  to %s/%s
+starting upgrade from old test format to 2a
 finished
-""" % (url, url, url, backup_dir), out)
+""" % (display_url, display_url, display_url, display_url, backup_dir), out)
         self.assertEqualDiff("", err)
         self.assertTrue(isinstance(
-            bzrdir.BzrDir.open(self.get_url('format_5_branch'))._format,
+            bzrdir.BzrDir.open(self.get_url(path))._format,
             bzrdir.BzrDirMetaFormat1))
 
     def test_upgrade_explicit_knit(self):
-        # users can force an upgrade to knit format from a metadir weave
-        # branch
-        self.make_metadir_weave_branch()
-        url = transport.get_transport(self.get_url('metadir_weave_branch')).base
+        # users can force an upgrade to knit format from a metadir pack 0.92
+        # branch to a 2a branch.
+        self.make_branch_and_tree('branch', format='knit')
+        transport = self.get_transport('branch')
+        url = transport.base
+        display_url = transport.local_abspath('.')
         # check --format takes effect
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
         backup_dir = 'backup.bzr.~1~'
         (out, err) = self.run_bzr(
-            ['upgrade', '--format=knit', url])
-        self.assertEqualDiff("""starting upgrade of %s
-making backup of %s.bzr
-  to %s%s
+            ['upgrade', '--format=pack-0.92', url])
+        self.assertEqualDiff("""Upgrading branch %s/ ...
+starting upgrade of %s/
+making backup of %s/.bzr
+  to %s/%s
 starting repository conversion
 repository converted
 finished
-""" % (url, url, url, backup_dir), out)
+""" % (display_url, display_url, display_url, display_url, backup_dir),
+                             out)
         self.assertEqualDiff("", err)
-        converted_dir = bzrdir.BzrDir.open(self.get_url('metadir_weave_branch'))
+        converted_dir = bzrdir.BzrDir.open(self.get_url('branch'))
         self.assertTrue(isinstance(converted_dir._format,
                                    bzrdir.BzrDirMetaFormat1))
         self.assertTrue(isinstance(converted_dir.open_repository()._format,
-                                   RepositoryFormatKnit1))
+                                   RepositoryFormatKnitPack1))
 
     def test_upgrade_repo(self):
-        self.run_bzr('init-repository --format=metaweave repo')
-        self.run_bzr('upgrade --format=knit repo')
+        self.run_bzr('init-repository --format=pack-0.92 repo')
+        self.run_bzr('upgrade --format=2a repo')
+
+    def assertLegalOption(self, option_str):
+        # Confirm that an option is legal. (Lower level tests are
+        # expected to validate the actual functionality.)
+        self.run_bzr('init --format=pack-0.92 branch-foo')
+        self.run_bzr('upgrade --format=2a branch-foo %s' % (option_str,))
+
+    def assertBranchFormat(self, dir, format):
+        branch = bzrdir.BzrDir.open_tree_or_branch(self.get_url(dir))[1]
+        branch_format = branch._format
+        meta_format = bzrdir.format_registry.make_bzrdir(format)
+        expected_format = meta_format.get_branch_format()
+        self.assertEqual(expected_format, branch_format)
+
+    def test_upgrade_clean_supported(self):
+        self.assertLegalOption('--clean')
+        self.assertBranchFormat('branch-foo', '2a')
+        backup_bzr_dir = os.path.join("branch-foo", "backup.bzr.~1~")
+        self.assertFalse(os.path.exists(backup_bzr_dir))
+
+    def test_upgrade_dry_run_supported(self):
+        self.assertLegalOption('--dry-run')
+        self.assertBranchFormat('branch-foo', 'pack-0.92')
 
     def test_upgrade_permission_check(self):
         """'backup.bzr' should retain permissions of .bzr. Bug #262450"""
@@ -161,51 +225,52 @@
         new_perms = os.stat(backup_dir).st_mode & 0777
         self.assertTrue(new_perms == old_perms)
 
-
     def test_upgrade_with_existing_backup_dir(self):
-        self.make_format_5_branch()
-        t = transport.get_transport(self.get_url('format_5_branch'))
+        self.make_branch_and_tree("old_format_branch", format="knit")
+        t = self.get_transport("old_format_branch")
         url = t.base
-        bzrdir.BzrDirFormat._set_default_format(bzrdir.BzrDirFormat5())
+        display_url = t.local_abspath('.')
         backup_dir1 = 'backup.bzr.~1~'
         backup_dir2 = 'backup.bzr.~2~'
         # explicitly create backup_dir1. bzr should create the .~2~ directory
         # as backup
         t.mkdir(backup_dir1)
         (out, err) = self.run_bzr(
-            ['upgrade', '--format=metaweave', url])
-        self.assertEqualDiff("""starting upgrade of %s
-making backup of %s.bzr
-  to %s%s
-starting upgrade from format 5 to 6
-adding prefixes to weaves
-adding prefixes to revision-store
-starting upgrade from format 6 to metadir
+            ['upgrade', '--format=2a', url])
+        self.assertEqualDiff("""Upgrading branch %s/ ...
+starting upgrade of %s/
+making backup of %s/.bzr
+  to %s/%s
+starting repository conversion
+repository converted
 finished
-""" % (url, url, url, backup_dir2), out)
+""" % (display_url, display_url, display_url, display_url, backup_dir2), out)
         self.assertEqualDiff("", err)
         self.assertTrue(isinstance(
-            bzrdir.BzrDir.open(self.get_url('format_5_branch'))._format,
+            bzrdir.BzrDir.open(self.get_url("old_format_branch"))._format,
             bzrdir.BzrDirMetaFormat1))
         self.assertTrue(t.has(backup_dir2))
 
+
 class SFTPTests(TestCaseWithSFTPServer):
     """Tests for upgrade over sftp."""
 
     def test_upgrade_url(self):
-        self.run_bzr('init --format=weave')
-        t = transport.get_transport(self.get_url())
+        self.run_bzr('init --format=pack-0.92')
+        t = self.get_transport()
         url = t.base
-        out, err = self.run_bzr(['upgrade', '--format=knit', url])
+        display_url = urlutils.unescape_for_display(url,
+            'utf-8')
+        out, err = self.run_bzr(['upgrade', '--format=2a', url])
         backup_dir = 'backup.bzr.~1~'
-        self.assertEqualDiff("""starting upgrade of %s
+        self.assertEqualDiff("""Upgrading branch %s ...
+starting upgrade of %s
 making backup of %s.bzr
   to %s%s
-starting upgrade from format 6 to metadir
 starting repository conversion
 repository converted
 finished
-""" % (url, url, url,backup_dir), out)
+""" % (display_url, display_url, display_url, display_url, backup_dir), out)
         self.assertEqual('', err)
 
 

=== modified file 'bzrlib/tests/blackbox/test_version.py'
--- bzrlib/tests/blackbox/test_version.py	2009-12-02 16:43:09 +0000
+++ bzrlib/tests/blackbox/test_version.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008 Canonical Ltd
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -95,7 +95,7 @@
             raise TestSkipped('Cannot find a unicode character that works in'
                               ' encoding %s' % (osutils.get_user_encoding(),))
 
-        osutils.set_or_unset_env('BZR_HOME', str_val)
+        self.overrideEnv('BZR_HOME', str_val)
         self.permit_source_tree_branch_repo()
         out = self.run_bzr("version")[0]
         self.assertTrue(len(out) > 0)
@@ -106,14 +106,14 @@
 
     def test_simple(self):
         bzr_log = 'my.bzr.log'
-        osutils.set_or_unset_env('BZR_LOG', bzr_log)
+        self.overrideEnv('BZR_LOG', bzr_log)
         default_log = os.path.join(os.environ['BZR_HOME'], '.bzr.log')
-        self.failIfExists([default_log, bzr_log])
+        self.assertPathDoesNotExist([default_log, bzr_log])
         out = self.run_bzr_subprocess('version')[0]
         self.assertTrue(len(out) > 0)
         self.assertContainsRe(out, r"(?m)^  Bazaar log file: " + bzr_log)
-        self.failIfExists(default_log)
-        self.failUnlessExists(bzr_log)
+        self.assertPathDoesNotExist(default_log)
+        self.assertPathExists(bzr_log)
 
     def test_dev_null(self):
         # This test uses a subprocess to cause the log opening logic to
@@ -122,10 +122,10 @@
             bzr_log = 'NUL'
         else:
             bzr_log = '/dev/null'
-        osutils.set_or_unset_env('BZR_LOG', bzr_log)
+        self.overrideEnv('BZR_LOG', bzr_log)
         default_log = os.path.join(os.environ['BZR_HOME'], '.bzr.log')
-        self.failIfExists(default_log)
+        self.assertPathDoesNotExist(default_log)
         out = self.run_bzr_subprocess('version')[0]
         self.assertTrue(len(out) > 0)
         self.assertContainsRe(out, r"(?m)^  Bazaar log file: " + bzr_log)
-        self.failIfExists(default_log)
+        self.assertPathDoesNotExist(default_log)

=== modified file 'bzrlib/tests/blackbox/test_version_info.py'
--- bzrlib/tests/blackbox/test_version_info.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_version_info.py	2011-10-14 13:56:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 import os
 
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -36,29 +37,27 @@
         self.build_tree(['branch/b'])
         wt.add('b')
         wt.commit('adding b', rev_id='r2')
-
-        self.revisions = wt.branch.revision_history()
         return wt
 
     def test_basic(self):
-        self.create_tree()
+        wt = self.create_tree()
 
         txt = self.run_bzr('version-info branch')[0]
         self.assertContainsRe(txt, 'date:')
         self.assertContainsRe(txt, 'build-date:')
         self.assertContainsRe(txt, 'revno: 2')
-        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+        self.assertContainsRe(txt, 'revision-id: ' + wt.branch.last_revision())
 
     def test_all(self):
         """'--all' includes clean, revision history, and file revisions"""
-        self.create_tree()
+        wt = self.create_tree()
         txt = self.run_bzr('version-info branch --all')[0]
         self.assertContainsRe(txt, 'date:')
         self.assertContainsRe(txt, 'revno: 2')
-        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+        self.assertContainsRe(txt, 'revision-id: ' + wt.branch.last_revision())
         self.assertContainsRe(txt, 'clean: True')
         self.assertContainsRe(txt, 'revisions:')
-        for rev_id in self.revisions:
+        for rev_id in wt.branch.repository.all_revision_ids():
             self.assertContainsRe(txt, 'id: ' + rev_id)
         self.assertContainsRe(txt, 'message: adding a')
         self.assertContainsRe(txt, 'message: adding b')
@@ -151,3 +150,25 @@
             '"{revno} {branch_nick} {clean}\n" branch')
         self.assertEqual("2 branch 0\n", out)
         self.assertEqual("", err)
+    
+    def test_non_ascii(self):
+        """Test that we can output non-ascii data"""
+        
+        commit_message = u'Non-ascii message with character not in latin-1: \u1234'
+        
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a_file'])
+        tree.add('a_file')
+        tree.commit(commit_message)
+        out, err = self.run_bzr(
+            ['version-info', '--include-history'], encoding='latin-1')
+        
+        self.assertContainsString(out, commit_message.encode('utf-8'))
+
+    def test_revision(self):
+        tree = self.create_tree()
+        branch = self.make_branch('just_branch')
+        branch.pull(tree.branch)
+
+        txt = self.run_bzr('version-info -r1 just_branch')[0]
+        self.assertStartsWith(txt, 'revision-id: r1\n')

=== modified file 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/tests/blackbox/test_versioning.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2011-10-07 15:09:48 +0000
@@ -39,15 +39,12 @@
         # here locally.
         shared_repo = self.make_repository('.')
         self.run_bzr(['mkdir', 'abc'], retcode=3)
-        self.failIfExists('abc')
-
-
-class TestVersioning(TestCaseInTempDir):
+        self.assertPathDoesNotExist('abc')
 
     def test_mkdir(self):
         """Basic 'bzr mkdir' operation"""
 
-        self.run_bzr('init')
+        self.make_branch_and_tree('.')
         self.run_bzr(['mkdir', 'foo'])
         self.assert_(os.path.isdir('foo'))
 
@@ -61,12 +58,12 @@
 
         self.assertEquals(len(delta.added), 1)
         self.assertEquals(delta.added[0][0], 'foo')
-        self.failIf(delta.modified)
+        self.assertFalse(delta.modified)
 
     def test_mkdir_in_subdir(self):
         """'bzr mkdir' operation in subdirectory"""
 
-        self.run_bzr('init')
+        self.make_branch_and_tree('.')
         self.run_bzr(['mkdir', 'dir'])
         self.assert_(os.path.isdir('dir'))
 
@@ -85,24 +82,19 @@
         self.assertEquals(len(delta.added), 2)
         self.assertEquals(delta.added[0][0], 'dir')
         self.assertEquals(delta.added[1][0], pathjoin('dir','subdir'))
-        self.failIf(delta.modified)
+        self.assertFalse(delta.modified)
 
     def test_mkdir_w_nested_trees(self):
         """'bzr mkdir' with nested trees"""
 
-        self.run_bzr('init')
-        os.mkdir('a')
-        os.chdir('a')
-        self.run_bzr('init')
-        os.mkdir('b')
-        os.chdir('b')
-        self.run_bzr('init')
-        os.chdir('../..')
+        self.make_branch_and_tree('.')
+        self.make_branch_and_tree('a')
+        self.make_branch_and_tree('a/b')
 
         self.run_bzr(['mkdir', 'dir', 'a/dir', 'a/b/dir'])
-        self.failUnless(os.path.isdir('dir'))
-        self.failUnless(os.path.isdir('a/dir'))
-        self.failUnless(os.path.isdir('a/b/dir'))
+        self.assertTrue(os.path.isdir('dir'))
+        self.assertTrue(os.path.isdir('a/dir'))
+        self.assertTrue(os.path.isdir('a/b/dir'))
 
         wt = WorkingTree.open('.')
         wt_a = WorkingTree.open('a')
@@ -111,28 +103,25 @@
         delta = wt.changes_from(wt.basis_tree())
         self.assertEquals(len(delta.added), 1)
         self.assertEquals(delta.added[0][0], 'dir')
-        self.failIf(delta.modified)
+        self.assertFalse(delta.modified)
 
         delta = wt_a.changes_from(wt_a.basis_tree())
         self.assertEquals(len(delta.added), 1)
         self.assertEquals(delta.added[0][0], 'dir')
-        self.failIf(delta.modified)
+        self.assertFalse(delta.modified)
 
         delta = wt_b.changes_from(wt_b.basis_tree())
         self.assertEquals(len(delta.added), 1)
         self.assertEquals(delta.added[0][0], 'dir')
-        self.failIf(delta.modified)
-
-    def check_branch(self):
-        """After all the above changes, run the check and upgrade commands.
-
-        The upgrade should be a no-op."""
-        b = Branch.open(u'.')
-        mutter('branch has %d revisions', b.revno())
-
-        mutter('check branch...')
-        from bzrlib.check import check
-        check(b, False)
+        self.assertFalse(delta.modified)
+
+    def test_mkdir_quiet(self):
+        """'bzr mkdir --quiet' should not print a status message"""
+
+        self.make_branch_and_tree('.')
+        out, err = self.run_bzr(['mkdir', '--quiet', 'foo'])
+        self.assertEquals('', err)
+        self.assertEquals('', out)
 
 
 class SubdirCommit(TestCaseWithTransport):

=== modified file 'bzrlib/tests/blackbox/test_view.py'
--- bzrlib/tests/blackbox/test_view.py	2009-04-08 03:34:31 +0000
+++ bzrlib/tests/blackbox/test_view.py	2010-11-22 03:35:24 +0000
@@ -23,19 +23,12 @@
 
 class TestViewUI(TestCaseWithTransport):
 
-    def make_branch_and_tree(self):
-        # we need to use a specific format because the default format
-        # doesn't support views yet
-        format = bzrdir.format_registry.make_bzrdir('development6-rich-root')
-        return TestCaseWithTransport.make_branch_and_tree(self, '.',
-            format=format)
-
     def test_view_command_help(self):
         out, err = self.run_bzr('help view')
         self.assertContainsRe(out, 'Manage filtered views')
 
     def test_define_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check definition of a new view
         out, err = self.run_bzr('view a b c')
         self.assertEquals(out, "Using 'my' view: a, b, c\n")
@@ -51,7 +44,7 @@
         self.assertContainsRe(err, "Cannot change the 'off' pseudo view")
 
     def test_list_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check list of the current view
         out, err = self.run_bzr('view')
         self.assertEquals(out, "No current view.\n")
@@ -76,7 +69,7 @@
         self.assertContainsRe(err, "No such view")
 
     def test_delete_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check delete of the current view
         out, err = self.run_bzr('view --delete', retcode=3)
         self.assertContainsRe(err, "No current view to delete")
@@ -101,7 +94,7 @@
         self.assertContainsRe(err, "Both --delete and a file list specified")
 
     def test_switch_view(self):
-        wt = self.make_branch_and_tree()
+        wt = self.make_branch_and_tree('.')
         # Check switching to a named view
         self.run_bzr('view a b c')
         self.run_bzr('view e f --name foo')

=== modified file 'bzrlib/tests/blackbox/test_whoami.py'
--- bzrlib/tests/blackbox/test_whoami.py	2010-06-23 08:19:28 +0000
+++ bzrlib/tests/blackbox/test_whoami.py	2011-06-22 09:09:35 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,57 +17,60 @@
 
 """Black-box tests for bzr whoami."""
 
-import os
-
 import bzrlib
 from bzrlib import (
-    osutils,
     config,
+    tests,
     )
-from bzrlib.tests import TestCaseWithTransport
-
-
-class TestWhoami(TestCaseWithTransport):
-
-    def test_whoami(self):
+
+
+class TestWhoami(tests.TestCaseWithTransport):
+
+    def assertWhoAmI(self, expected, *cmd_args, **kwargs):
+        out, err = self.run_bzr(('whoami',) + cmd_args, **kwargs)
+        self.assertEquals('', err)
+        lines = out.splitlines()
+        self.assertLength(1, lines)
+        self.assertEquals(expected, lines[0].rstrip())
+
+    def test_whoami_no_args_no_conf(self):
         # this should always identify something, if only "john@localhost"
         out = self.run_bzr("whoami")[0]
         self.assertTrue(len(out) > 0)
         self.assertEquals(1, out.count('@'))
 
+    def test_whoami_email_no_args(self):
         out = self.run_bzr("whoami --email")[0]
         self.assertTrue(len(out) > 0)
         self.assertEquals(1, out.count('@'))
 
+    def test_whoami_email_arg(self):
+        # whoami --email is mutually exclusive with any arguments
+        out = self.run_bzr("whoami --email 'foo <foo@example.com>'", 3)[0]
+        self.assertEquals("", out)
+
     def test_whoami_branch(self):
         """branch specific user identity works."""
         wt = self.make_branch_and_tree('.')
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email',
                                        'Branch Identity <branch@identi.ty>')
-        whoami = self.run_bzr("whoami")[0]
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email")[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>')
+        self.assertWhoAmI('branch@identi.ty', '--email')
 
         # Verify that the environment variable overrides the value
         # in the file
-        os.environ['BZR_EMAIL'] = 'Different ID <other@environ.ment>'
-        whoami = self.run_bzr("whoami")[0]
-        self.assertEquals('Different ID <other@environ.ment>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email")[0]
-        self.assertEquals('other@environ.ment\n', whoami_email)
+        self.overrideEnv('BZR_EMAIL', 'Different ID <other@environ.ment>')
+        self.assertWhoAmI('Different ID <other@environ.ment>')
+        self.assertWhoAmI('other@environ.ment', '--email')
 
     def test_whoami_utf8(self):
         """verify that an identity can be in utf-8."""
-        wt = self.make_branch_and_tree('.')
         self.run_bzr(['whoami', u'Branch Identity \u20ac <branch@identi.ty>'],
                      encoding='utf-8')
-        whoami = self.run_bzr("whoami", encoding='utf-8')[0]
-        self.assertEquals('Branch Identity \xe2\x82\xac ' +
-                          '<branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email", encoding='utf-8')[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        self.assertWhoAmI('Branch Identity \xe2\x82\xac <branch@identi.ty>',
+                          encoding='utf-8')
+        self.assertWhoAmI('branch@identi.ty', '--email')
 
     def test_whoami_ascii(self):
         """
@@ -78,10 +81,10 @@
         b = bzrlib.branch.Branch.open('.')
         b.get_config().set_user_option('email', u'Branch Identity \u20ac ' +
                                        '<branch@identi.ty>')
-        whoami = self.run_bzr("whoami", encoding='ascii')[0]
-        self.assertEquals('Branch Identity ? <branch@identi.ty>\n', whoami)
-        whoami_email = self.run_bzr("whoami --email", encoding='ascii')[0]
-        self.assertEquals('branch@identi.ty\n', whoami_email)
+        self.assertWhoAmI('Branch Identity ? <branch@identi.ty>',
+                          encoding='ascii')
+        self.assertWhoAmI('branch@identi.ty', '--email',
+                          encoding='ascii')
 
     def test_warning(self):
         """verify that a warning is displayed if no email is given."""
@@ -92,10 +95,13 @@
                           'recommended.\n', display)
 
     def test_whoami_not_set(self):
-        """Ensure whoami error if username is not set.
+        """Ensure whoami error if username is not set and not inferred.
         """
-        osutils.set_or_unset_env('EMAIL', None)
-        osutils.set_or_unset_env('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
+        # Also, make sure that it's not inferred from mailname.
+        self.overrideAttr(config, '_auto_user_id',
+            lambda: (None, None))
         out, err = self.run_bzr(['whoami'], 3)
         self.assertContainsRe(err, 'Unable to determine your name')
 
@@ -104,10 +110,11 @@
         wt = self.make_branch_and_tree('subdir')
         c = wt.branch.get_config()
         c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
-        out, err = self.run_bzr("whoami --directory subdir")
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>',
+                          '--directory', 'subdir')
         self.run_bzr(['whoami', '--directory', 'subdir', '--branch',
                       'Changed Identity <changed@identi.ty>'])
+        c = wt.branch.get_config()
         self.assertEquals('Changed Identity <changed@identi.ty>',
                           c.get_user_option('email'))
 
@@ -117,13 +124,14 @@
         c = wt.branch.get_config()
         c.set_user_option('email', 'Branch Identity <branch@identi.ty>')
         url = self.get_readonly_url() + '/subdir'
-        out, err = self.run_bzr(['whoami', '--directory', url])
-        self.assertEquals('Branch Identity <branch@identi.ty>\n', out)
+        self.assertWhoAmI('Branch Identity <branch@identi.ty>',
+                          '--directory', url)
         url = self.get_url('subdir')
         self.run_bzr(['whoami', '--directory', url, '--branch',
                       'Changed Identity <changed@identi.ty>'])
         # The identity has been set in the branch config (but not the global
         # config)
+        c = wt.branch.get_config()
         self.assertEquals('Changed Identity <changed@identi.ty>',
                           c.get_user_option('email'))
         global_conf = config.GlobalConfig()

=== modified file 'bzrlib/tests/commands/test_pull.py'
--- bzrlib/tests/commands/test_pull.py	2010-04-09 03:58:14 +0000
+++ bzrlib/tests/commands/test_pull.py	2011-08-30 08:46:10 +0000
@@ -54,5 +54,5 @@
         # We don't care about the ouput but 'outf' should be defined
         pull.outf = tests.StringIOWrapper()
         pull.run(self.get_url('remote'), directory='local')
-        self.assertEquals(1, len(self.connections))
+        self.assertEquals(2, len(self.connections))
 

=== modified file 'bzrlib/tests/doc_generate/__init__.py'
--- bzrlib/tests/doc_generate/__init__.py	2010-07-21 08:07:14 +0000
+++ bzrlib/tests/doc_generate/__init__.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,6 +52,11 @@
 
     _test_needs_features = [features.sphinx]
 
+    def sphinx_version(self):
+        # Convert to a tuple to avoid traps in string comparison
+        # ( '1.12' < '1.6' but (1, 12) > (1, 6) )
+        return tuple(map(int, features.sphinx.module.__version__.split('.')))
+
     def make_sphinx(self):
         out = tests.StringIOWrapper()
         err = tests.StringIOWrapper()

=== modified file 'bzrlib/tests/doc_generate/builders/test_texinfo.py'
--- bzrlib/tests/doc_generate/builders/test_texinfo.py	2010-07-06 13:22:32 +0000
+++ bzrlib/tests/doc_generate/builders/test_texinfo.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -67,5 +67,5 @@
              ])
         app, out, err = self.make_sphinx()
         self.build(app)
-        self.failUnlessExists('index.texi')
-        self.failUnlessExists('content.texi')
+        self.assertPathExists('index.texi')
+        self.assertPathExists('content.texi')

=== modified file 'bzrlib/tests/doc_generate/writers/test_texinfo.py'
--- bzrlib/tests/doc_generate/writers/test_texinfo.py	2010-07-21 08:07:14 +0000
+++ bzrlib/tests/doc_generate/writers/test_texinfo.py	2011-03-11 11:33:33 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 """sphinx texinfo writer tests."""
 
+from bzrlib import tests
 from bzrlib.tests import (
     doc_generate as test_dg, # Avoid clash with from bzrlib import doc_generate
     )
@@ -232,6 +233,29 @@
 """,
                              'bzr-0.0.8.texi')
 
+    def test_toctree_empty(self):
+        self.build_tree_contents(
+            [('index.txt', """
+Table of Contents
+=================
+
+.. toctree::
+   :maxdepth: 1
+""")])
+        app, out, err = self.make_sphinx()
+        self.build(app)
+        self.assertFileEqual("""\
+This file has been converted using a beta rst->texinfo converter. 
+Most of the info links are currently bogus, don't report bugs about them,
+this is currently worked on.
+@node Top
+@top Placeholder
+@node table-of-contents
+@chapter Table of Contents
+""",
+                             'index.texi')
+
+
 class TestSections(test_dg.TestSphinx):
 
     def test_sections(self):

=== modified file 'bzrlib/tests/features.py'
--- bzrlib/tests/features.py	2010-07-05 12:40:28 +0000
+++ bzrlib/tests/features.py	2011-08-23 09:29:27 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,24 +14,352 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""A collection of commonly used 'Features' which bzrlib uses to skip tests."""
+"""A collection of commonly used 'Features' to optionally run tests.
+"""
 
 import os
+import subprocess
 import stat
-
-from bzrlib import tests
-from bzrlib.symbol_versioning import deprecated_in
-
-
-apport = tests.ModuleAvailableFeature('apport')
-paramiko = tests.ModuleAvailableFeature('paramiko')
-pycurl = tests.ModuleAvailableFeature('pycurl')
-pywintypes = tests.ModuleAvailableFeature('pywintypes')
-subunit = tests.ModuleAvailableFeature('subunit')
-sphinx = tests.ModuleAvailableFeature('sphinx')
-
-
-class _BackslashDirSeparatorFeature(tests.Feature):
+import sys
+import tempfile
+
+from bzrlib import (
+    osutils,
+    symbol_versioning,
+    tests,
+    )
+
+
+class Feature(object):
+    """An operating system Feature."""
+
+    def __init__(self):
+        self._available = None
+
+    def available(self):
+        """Is the feature available?
+
+        :return: True if the feature is available.
+        """
+        if self._available is None:
+            self._available = self._probe()
+        return self._available
+
+    def _probe(self):
+        """Implement this method in concrete features.
+
+        :return: True if the feature is available.
+        """
+        raise NotImplementedError
+
+    def __str__(self):
+        if getattr(self, 'feature_name', None):
+            return self.feature_name()
+        return self.__class__.__name__
+
+
+class _SymlinkFeature(Feature):
+
+    def _probe(self):
+        return osutils.has_symlinks()
+
+    def feature_name(self):
+        return 'symlinks'
+
+SymlinkFeature = _SymlinkFeature()
+
+
+class _HardlinkFeature(Feature):
+
+    def _probe(self):
+        return osutils.has_hardlinks()
+
+    def feature_name(self):
+        return 'hardlinks'
+
+HardlinkFeature = _HardlinkFeature()
+
+
+class _OsFifoFeature(Feature):
+
+    def _probe(self):
+        return getattr(os, 'mkfifo', None)
+
+    def feature_name(self):
+        return 'filesystem fifos'
+
+OsFifoFeature = _OsFifoFeature()
+
+
+class _UnicodeFilenameFeature(Feature):
+    """Does the filesystem support Unicode filenames?"""
+
+    def _probe(self):
+        try:
+            # Check for character combinations unlikely to be covered by any
+            # single non-unicode encoding. We use the characters
+            # - greek small letter alpha (U+03B1) and
+            # - braille pattern dots-123456 (U+283F).
+            os.stat(u'\u03b1\u283f')
+        except UnicodeEncodeError:
+            return False
+        except (IOError, OSError):
+            # The filesystem allows the Unicode filename but the file doesn't
+            # exist.
+            return True
+        else:
+            # The filesystem allows the Unicode filename and the file exists,
+            # for some reason.
+            return True
+
+UnicodeFilenameFeature = _UnicodeFilenameFeature()
+
+
+class _CompatabilityThunkFeature(Feature):
+    """This feature is just a thunk to another feature.
+
+    It issues a deprecation warning if it is accessed, to let you know that you
+    should really use a different feature.
+    """
+
+    def __init__(self, dep_version, module, name,
+                 replacement_name, replacement_module=None):
+        super(_CompatabilityThunkFeature, self).__init__()
+        self._module = module
+        if replacement_module is None:
+            replacement_module = module
+        self._replacement_module = replacement_module
+        self._name = name
+        self._replacement_name = replacement_name
+        self._dep_version = dep_version
+        self._feature = None
+
+    def _ensure(self):
+        if self._feature is None:
+            from bzrlib import pyutils
+            depr_msg = self._dep_version % ('%s.%s'
+                                            % (self._module, self._name))
+            use_msg = ' Use %s.%s instead.' % (self._replacement_module,
+                                               self._replacement_name)
+            symbol_versioning.warn(depr_msg + use_msg, DeprecationWarning)
+            # Import the new feature and use it as a replacement for the
+            # deprecated one.
+            self._feature = pyutils.get_named_object(
+                self._replacement_module, self._replacement_name)
+
+    def _probe(self):
+        self._ensure()
+        return self._feature._probe()
+
+
+class ModuleAvailableFeature(Feature):
+    """This is a feature than describes a module we want to be available.
+
+    Declare the name of the module in __init__(), and then after probing, the
+    module will be available as 'self.module'.
+
+    :ivar module: The module if it is available, else None.
+    """
+
+    def __init__(self, module_name):
+        super(ModuleAvailableFeature, self).__init__()
+        self.module_name = module_name
+
+    def _probe(self):
+        sentinel = object()
+        module = sys.modules.get(self.module_name, sentinel)
+        if module is sentinel:
+            try:
+                self._module = __import__(self.module_name, {}, {}, [''])
+                return True
+            except ImportError:
+                return False
+        else:
+            self._module = module
+            return True
+
+    @property
+    def module(self):
+        if self.available():
+            return self._module
+        return None
+
+    def feature_name(self):
+        return self.module_name
+
+
+class _HTTPSServerFeature(Feature):
+    """Some tests want an https Server, check if one is available.
+
+    Right now, the only way this is available is under python2.6 which provides
+    an ssl module.
+    """
+
+    def _probe(self):
+        try:
+            import ssl
+            return True
+        except ImportError:
+            return False
+
+    def feature_name(self):
+        return 'HTTPSServer'
+
+
+HTTPSServerFeature = _HTTPSServerFeature()
+
+
+class _ByteStringNamedFilesystem(Feature):
+    """Is the filesystem based on bytes?"""
+
+    def _probe(self):
+        if os.name == "posix":
+            return True
+        return False
+
+ByteStringNamedFilesystem = _ByteStringNamedFilesystem()
+
+
+class _UTF8Filesystem(Feature):
+    """Is the filesystem UTF-8?"""
+
+    def _probe(self):
+        if osutils._fs_enc.upper() in ('UTF-8', 'UTF8'):
+            return True
+        return False
+
+UTF8Filesystem = _UTF8Filesystem()
+
+
+class _BreakinFeature(Feature):
+    """Does this platform support the breakin feature?"""
+
+    def _probe(self):
+        from bzrlib import breakin
+        if breakin.determine_signal() is None:
+            return False
+        if sys.platform == 'win32':
+            # Windows doesn't have os.kill, and we catch the SIGBREAK signal.
+            # We trigger SIGBREAK via a Console api so we need ctypes to
+            # access the function
+            try:
+                import ctypes
+            except OSError:
+                return False
+        return True
+
+    def feature_name(self):
+        return "SIGQUIT or SIGBREAK w/ctypes on win32"
+
+
+BreakinFeature = _BreakinFeature()
+
+
+class _CaseInsCasePresFilenameFeature(Feature):
+    """Is the file-system case insensitive, but case-preserving?"""
+
+    def _probe(self):
+        fileno, name = tempfile.mkstemp(prefix='MixedCase')
+        try:
+            # first check truly case-preserving for created files, then check
+            # case insensitive when opening existing files.
+            name = osutils.normpath(name)
+            base, rel = osutils.split(name)
+            found_rel = osutils.canonical_relpath(base, name)
+            return (found_rel == rel
+                    and os.path.isfile(name.upper())
+                    and os.path.isfile(name.lower()))
+        finally:
+            os.close(fileno)
+            os.remove(name)
+
+    def feature_name(self):
+        return "case-insensitive case-preserving filesystem"
+
+CaseInsCasePresFilenameFeature = _CaseInsCasePresFilenameFeature()
+
+
+class _CaseInsensitiveFilesystemFeature(Feature):
+    """Check if underlying filesystem is case-insensitive but *not* case
+    preserving.
+    """
+    # Note that on Windows, Cygwin, MacOS etc, the file-systems are far
+    # more likely to be case preserving, so this case is rare.
+
+    def _probe(self):
+        if CaseInsCasePresFilenameFeature.available():
+            return False
+
+        if tests.TestCaseWithMemoryTransport.TEST_ROOT is None:
+            root = osutils.mkdtemp(prefix='testbzr-', suffix='.tmp')
+            tests.TestCaseWithMemoryTransport.TEST_ROOT = root
+        else:
+            root = tests.TestCaseWithMemoryTransport.TEST_ROOT
+        tdir = osutils.mkdtemp(prefix='case-sensitive-probe-', suffix='',
+            dir=root)
+        name_a = osutils.pathjoin(tdir, 'a')
+        name_A = osutils.pathjoin(tdir, 'A')
+        os.mkdir(name_a)
+        result = osutils.isdir(name_A)
+        tests._rmtree_temp_dir(tdir)
+        return result
+
+    def feature_name(self):
+        return 'case-insensitive filesystem'
+
+CaseInsensitiveFilesystemFeature = _CaseInsensitiveFilesystemFeature()
+
+
+class _CaseSensitiveFilesystemFeature(Feature):
+
+    def _probe(self):
+        if CaseInsCasePresFilenameFeature.available():
+            return False
+        elif CaseInsensitiveFilesystemFeature.available():
+            return False
+        else:
+            return True
+
+    def feature_name(self):
+        return 'case-sensitive filesystem'
+
+# new coding style is for feature instances to be lowercase
+case_sensitive_filesystem_feature = _CaseSensitiveFilesystemFeature()
+
+
+class _NotRunningAsRoot(Feature):
+
+    def _probe(self):
+        try:
+            uid = os.getuid()
+        except AttributeError:
+            # If there is no uid, chances are there is no root either
+            return True
+        return uid != 0
+
+    def feature_name(self):
+        return 'Not running as root'
+
+
+not_running_as_root = _NotRunningAsRoot()
+
+apport = ModuleAvailableFeature('apport')
+gpgme = ModuleAvailableFeature('gpgme')
+lzma = ModuleAvailableFeature('lzma')
+meliae = ModuleAvailableFeature('meliae.scanner')
+paramiko = ModuleAvailableFeature('paramiko')
+pycurl = ModuleAvailableFeature('pycurl')
+pywintypes = ModuleAvailableFeature('pywintypes')
+sphinx = ModuleAvailableFeature('sphinx')
+subunit = ModuleAvailableFeature('subunit')
+testtools = ModuleAvailableFeature('testtools')
+
+compiled_patiencediff_feature = ModuleAvailableFeature(
+    'bzrlib._patiencediff_c')
+lsprof_feature = ModuleAvailableFeature('bzrlib.lsprof')
+
+
+class _BackslashDirSeparatorFeature(Feature):
 
     def _probe(self):
         try:
@@ -47,33 +375,7 @@
 backslashdir_feature = _BackslashDirSeparatorFeature()
 
 
-class _PosixPermissionsFeature(tests.Feature):
-
-    def _probe(self):
-        def has_perms():
-            # create temporary file and check if specified perms are maintained.
-            import tempfile
-
-            write_perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
-            f = tempfile.mkstemp(prefix='bzr_perms_chk_')
-            fd, name = f
-            os.close(fd)
-            os.chmod(name, write_perms)
-
-            read_perms = os.stat(name).st_mode & 0777
-            os.unlink(name)
-            return (write_perms == read_perms)
-
-        return (os.name == 'posix') and has_perms()
-
-    def feature_name(self):
-        return 'POSIX permissions support'
-
-
-posix_permissions_feature = _PosixPermissionsFeature()
-
-
-class _ChownFeature(tests.Feature):
+class _ChownFeature(Feature):
     """os.chown is supported"""
 
     def _probe(self):
@@ -82,7 +384,7 @@
 chown_feature = _ChownFeature()
 
 
-class ExecutableFeature(tests.Feature):
+class ExecutableFeature(Feature):
     """Feature testing whether an executable of a given name is on the PATH."""
 
     def __init__(self, name):
@@ -97,16 +399,8 @@
         return self._path
 
     def _probe(self):
-        path = os.environ.get('PATH')
-        if path is None:
-            return False
-        for d in path.split(os.pathsep):
-            if d:
-                f = os.path.join(d, self.name)
-                if os.access(f, os.X_OK):
-                    self._path = f
-                    return True
-        return False
+        self._path = osutils.find_executable_on_path(self.name)
+        return self._path is not None
 
     def feature_name(self):
         return '%s executable' % self.name
@@ -114,3 +408,108 @@
 
 bash_feature = ExecutableFeature('bash')
 sed_feature = ExecutableFeature('sed')
+diff_feature = ExecutableFeature('diff')
+
+
+class _PosixPermissionsFeature(Feature):
+
+    def _probe(self):
+        def has_perms():
+            # Create temporary file and check if specified perms are
+            # maintained.
+            write_perms = stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR
+            f = tempfile.mkstemp(prefix='bzr_perms_chk_')
+            fd, name = f
+            os.close(fd)
+            os.chmod(name, write_perms)
+
+            read_perms = os.stat(name).st_mode & 0777
+            os.unlink(name)
+            return (write_perms == read_perms)
+
+        return (os.name == 'posix') and has_perms()
+
+    def feature_name(self):
+        return 'POSIX permissions support'
+
+
+posix_permissions_feature = _PosixPermissionsFeature()
+
+
+class _StraceFeature(Feature):
+
+    def _probe(self):
+        try:
+            proc = subprocess.Popen(['strace'],
+                stderr=subprocess.PIPE,
+                stdout=subprocess.PIPE)
+            proc.communicate()
+            return True
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                # strace is not installed
+                return False
+            else:
+                raise
+
+    def feature_name(self):
+        return 'strace'
+
+
+strace_feature = _StraceFeature()
+
+
+class _AttribFeature(Feature):
+
+    def _probe(self):
+        if (sys.platform not in ('cygwin', 'win32')):
+            return False
+        try:
+            proc = subprocess.Popen(['attrib', '.'], stdout=subprocess.PIPE)
+        except OSError, e:
+            return False
+        return (0 == proc.wait())
+
+    def feature_name(self):
+        return 'attrib Windows command-line tool'
+
+
+AttribFeature = _AttribFeature()
+
+
+class Win32Feature(Feature):
+    """Feature testing whether we're running selftest on Windows
+    or Windows-like platform.
+    """
+
+    def _probe(self):
+        return sys.platform == 'win32'
+
+    def feature_name(self):
+        return "win32 platform"
+
+
+win32_feature = Win32Feature()
+
+
+for name in ['HTTPServerFeature', 
+    'HTTPSServerFeature', 'SymlinkFeature', 'HardlinkFeature',
+    'OsFifoFeature', 'UnicodeFilenameFeature',
+    'ByteStringNamedFilesystem', 'UTF8Filesystem',
+    'BreakinFeature', 'CaseInsCasePresFilenameFeature',
+    'CaseInsensitiveFilesystemFeature', 'case_sensitive_filesystem_feature',
+    'posix_permissions_feature',
+    ]:
+    setattr(tests, name, _CompatabilityThunkFeature(
+        symbol_versioning.deprecated_in((2, 5, 0)),
+        'bzrlib.tests', name,
+        name, 'bzrlib.tests.features'))
+
+
+for (old_name, new_name) in [
+    ('UnicodeFilename', 'UnicodeFilenameFeature'),
+    ]:
+    setattr(tests, name, _CompatabilityThunkFeature(
+        symbol_versioning.deprecated_in((2, 5, 0)),
+        'bzrlib.tests', old_name,
+        new_name, 'bzrlib.tests.features'))

=== modified file 'bzrlib/tests/fixtures.py'
--- bzrlib/tests/fixtures.py	2010-06-26 01:07:16 +0000
+++ bzrlib/tests/fixtures.py	2011-07-04 20:17:44 +0000
@@ -97,3 +97,44 @@
     def __exit__(self, exc_type, exc_val, exc_tb):
         self._calls.append('__exit__')
         return False # propogate exceptions.
+
+
+def build_branch_with_non_ancestral_rev(branch_builder):
+    """Builds a branch with a rev not in the ancestry of the tip.
+
+    This is the revision graph::
+
+      rev-2
+        |
+      rev-1
+        |
+      (null)
+
+    The branch tip is 'rev-1'.  'rev-2' is present in the branch's repository,
+    but is not part of rev-1's ancestry.
+
+    :param branch_builder: A BranchBuilder (e.g. from
+        TestCaseWithMemoryTransport.make_branch_builder).
+    :returns: the new branch
+    """
+    # Make a sequence of two commits
+    branch_builder.build_commit(message="Rev 1", rev_id='rev-1')
+    branch_builder.build_commit(message="Rev 2", rev_id='rev-2')
+    # Move the branch tip back to the first commit
+    source = branch_builder.get_branch()
+    source.set_last_revision_info(1, 'rev-1')
+    return source
+
+
+def make_branch_and_populated_tree(testcase):
+    """Make a simple branch and tree.
+
+    The tree holds some added but uncommitted files.
+    """
+    # TODO: Either accept or return the names of the files, so the caller
+    # doesn't need to be bound to the particular files created? -- mbp
+    # 20110705
+    tree = testcase.make_branch_and_tree('t')
+    testcase.build_tree_contents([('t/hello', 'hello world')])
+    tree.add(['hello'], ['hello-id'])
+    return tree

=== modified file 'bzrlib/tests/ftp_server/__init__.py'
--- bzrlib/tests/ftp_server/__init__.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/ftp_server/__init__.py	2011-06-14 01:26:41 +0000
@@ -13,6 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
 """
 Facilities to use ftp test servers.
 """
@@ -20,12 +21,20 @@
 import sys
 
 from bzrlib import tests
+from bzrlib.tests import (
+    features,
+    )
 
 
 try:
     from bzrlib.tests.ftp_server import medusa_based
-    # medusa is bogus under python2.6
-    medusa_available = sys.version_info < (2, 6)
+    # medusa is bogus starting with python2.6, since we don't support earlier
+    # pythons anymore, it's currently useless. There is hope though that the
+    # unicode bugs get fixed in the future so we leave it disabled until
+    # then. Keeping the framework in place means that only the following line
+    # will need to be changed.  The last tests were conducted with medusa-2.0
+    # -- vila 20110607
+    medusa_available = False
 except ImportError:
     medusa_available = False
 
@@ -37,7 +46,7 @@
     pyftpdlib_available = False
 
 
-class _FTPServerFeature(tests.Feature):
+class _FTPServerFeature(features.Feature):
     """Some tests want an FTP Server, check if one is available.
 
     Right now, the only way this is available is if one of the following is

=== modified file 'bzrlib/tests/ftp_server/medusa_based.py'
--- bzrlib/tests/ftp_server/medusa_based.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/ftp_server/medusa_based.py	2010-06-30 15:19:36 +0000
@@ -254,6 +254,9 @@
         self._async_thread = threading.Thread(
                 target=FTPTestServer._asyncore_loop_ignore_EBADF,
                 kwargs={'timeout':0.1, 'count':10000})
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n'
+                             % (self._async_thread.ident,))
         self._async_thread.setDaemon(True)
         self._async_thread.start()
 
@@ -261,6 +264,9 @@
         self._ftp_server.close()
         asyncore.close_all()
         self._async_thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n'
+                             % (self._async_thread.ident,))
 
     @staticmethod
     def _asyncore_loop_ignore_EBADF(*args, **kwargs):

=== modified file 'bzrlib/tests/ftp_server/pyftpdlib_based.py'
--- bzrlib/tests/ftp_server/pyftpdlib_based.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/ftp_server/pyftpdlib_based.py	2011-06-07 07:00:23 +0000
@@ -28,11 +28,17 @@
 
 from bzrlib import (
     osutils,
+    tests,
     trace,
     )
 from bzrlib.tests import test_server
 
 
+# Convert the pyftplib string version into a tuple to avoid traps in string
+# comparison.
+pyftplib_version = tuple(map(int, ftpserver.__ver__.split('.')))
+
+
 class AnonymousWithWriteAccessAuthorizer(ftpserver.DummyAuthorizer):
 
     def _check_permissions(self, username, perm):
@@ -112,16 +118,34 @@
             self.log('OK SITE CHMOD 0%03o "%s".' % (mode, ftp_path))
             self.respond('200 SITE CHMOD succesful.')
 
-
-# pyftpdlib says to define SITE commands by declaring ftp_SITE_<CMD> methods,
-# but fails to recognize them.
-ftpserver.proto_cmds['SITE CHMOD'] = ftpserver._CommandProperty(
-    perm='w', # Best fit choice even if not exactly right (can be d, f or m too)
-    auth_needed=True, arg_needed=True, check_path=False,
-    help='Syntax: SITE CHMOD <SP>  octal_mode_bits file-name (chmod file)',
-    )
-# An empty password is valid, hence the arg is neither mandatory not forbidden
-ftpserver.proto_cmds['PASS'].arg_needed = None
+    if pyftplib_version >= (0, 6, 0):
+        def log_cmd(self, cmd, arg, respcode, respstr):
+            # base class version choke on unicode, the alternative is to just
+            # provide an empty implementation and relies on the client to do
+            # the logging for debugging purposes. Not worth the trouble so far
+            # -- vila 20110607
+            if cmd in ("DELE", "RMD", "RNFR", "RNTO", "MKD"):
+                line = '"%s" %s' % (' '.join([cmd, unicode(arg)]).strip(),
+                                    respcode)
+                self.log(line)
+
+
+if pyftplib_version < (0, 6,0):
+    # pyftpdlib says to define SITE commands by declaring ftp_SITE_<CMD>
+    # methods, but fails to recognize them.
+    ftpserver.proto_cmds['SITE CHMOD'] = ftpserver._CommandProperty(
+        # Best fit choice even if not exactly right (can be d, f or m too)
+        perm='w',
+        auth_needed=True, arg_needed=True, check_path=False,
+        help='Syntax: SITE CHMOD <SP>  octal_mode_bits file-name (chmod file)',
+        )
+    # An empty password is valid, hence the arg is neither mandatory not
+    # forbidden
+    ftpserver.proto_cmds['PASS'].arg_needed = None
+else:
+    # Same rationale as above (the password should be optional), but the hole
+    # in pyftplib-0.6.0 is narrower
+    ftpserver.proto_cmds['PASS']['arg'] = None
 
 
 class ftp_server(ftpserver.FTPServer):
@@ -183,6 +207,9 @@
         self._ftpd_starting.acquire() # So it can be released by the server
         self._ftpd_thread = threading.Thread(target=self._run_server,)
         self._ftpd_thread.start()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n'
+                             % (self._ftpd_thread.ident,))
         # Wait for the server thread to start (i.e release the lock)
         self._ftpd_starting.acquire()
         self._ftpd_starting.release()
@@ -195,6 +222,9 @@
         self._ftp_server.close()
         self._ftpd_running = False
         self._ftpd_thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n'
+                             % (self._ftpd_thread.ident,))
 
     def _run_server(self):
         """Run the server until stop_server is called, shut it down properly then.

=== modified file 'bzrlib/tests/http_server.py'
--- bzrlib/tests/http_server.py	2010-07-29 11:17:57 +0000
+++ bzrlib/tests/http_server.py	2011-03-08 16:00:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,19 +20,12 @@
 import posixpath
 import random
 import re
-import select
 import SimpleHTTPServer
 import socket
-import SocketServer
-import sys
-import threading
-import time
 import urllib
 import urlparse
 
-from bzrlib import transport
 from bzrlib.tests import test_server
-from bzrlib.transport import local
 
 
 class BadWebserverPath(ValueError):
@@ -71,6 +64,14 @@
                 self.headers.get('referer', '-'),
                 self.headers.get('user-agent', '-'))
 
+    def handle(self):
+        SimpleHTTPServer.SimpleHTTPRequestHandler.handle(self)
+        # Some client (pycurl, I'm looking at you) are more picky than others
+        # and require that the socket itself is closed
+        # (SocketServer.StreamRequestHandler only close the two associated
+        # 'makefile' objects)
+        self.connection.close()
+
     def handle_one_request(self):
         """Handle a single HTTP request.
 
@@ -78,7 +79,7 @@
         connection early to avoid polluting the test results.
         """
         try:
-            SimpleHTTPServer.SimpleHTTPRequestHandler.handle_one_request(self)
+            self._handle_one_request()
         except socket.error, e:
             # Any socket error should close the connection, but some errors are
             # due to the client closing early and we don't want to pollute test
@@ -124,32 +125,45 @@
         if self.command != 'HEAD' and code >= 200 and code not in (204, 304):
             self.wfile.write(content)
 
-    _range_regexp = re.compile(r'^(?P<start>\d+)-(?P<end>\d+)$')
+    def _handle_one_request(self):
+        SimpleHTTPServer.SimpleHTTPRequestHandler.handle_one_request(self)
+
+    _range_regexp = re.compile(r'^(?P<start>\d+)-(?P<end>\d+)?$')
     _tail_regexp = re.compile(r'^-(?P<tail>\d+)$')
 
-    def parse_ranges(self, ranges_header):
-        """Parse the range header value and returns ranges and tail.
-
-        RFC2616 14.35 says that syntactically invalid range
-        specifiers MUST be ignored. In that case, we return 0 for
-        tail and [] for ranges.
+    def _parse_ranges(self, ranges_header, file_size):
+        """Parse the range header value and returns ranges.
+
+        RFC2616 14.35 says that syntactically invalid range specifiers MUST be
+        ignored. In that case, we return None instead of a range list.
+
+        :param ranges_header: The 'Range' header value.
+
+        :param file_size: The size of the requested file.
+
+        :return: A list of (start, end) tuples or None if some invalid range
+            specifier is encountered.
         """
-        tail = 0
-        ranges = []
         if not ranges_header.startswith('bytes='):
             # Syntactically invalid header
-            return 0, []
+            return None
 
+        tail = None
+        ranges = []
         ranges_header = ranges_header[len('bytes='):]
         for range_str in ranges_header.split(','):
-            # FIXME: RFC2616 says end is optional and default to file_size
             range_match = self._range_regexp.match(range_str)
             if range_match is not None:
                 start = int(range_match.group('start'))
-                end = int(range_match.group('end'))
+                end_match = range_match.group('end')
+                if end_match is None:
+                    # RFC2616 says end is optional and default to file_size
+                    end = file_size
+                else:
+                    end = int(end_match)
                 if start > end:
                     # Syntactically invalid range
-                    return 0, []
+                    return None
                 ranges.append((start, end))
             else:
                 tail_match = self._tail_regexp.match(range_str)
@@ -157,8 +171,21 @@
                     tail = int(tail_match.group('tail'))
                 else:
                     # Syntactically invalid range
-                    return 0, []
-        return tail, ranges
+                    return None
+        if tail is not None:
+            # Normalize tail into ranges
+            ranges.append((max(0, file_size - tail), file_size))
+
+        checked_ranges = []
+        for start, end in ranges:
+            if start >= file_size:
+                # RFC2616 14.35, ranges are invalid if start >= file_size
+                return None
+            # RFC2616 14.35, end values should be truncated
+            # to file_size -1 if they exceed it
+            end = min(end, file_size - 1)
+            checked_ranges.append((start, end))
+        return checked_ranges
 
     def _header_line_length(self, keyword, value):
         header_line = '%s: %s\r\n' % (keyword, value)
@@ -248,39 +275,18 @@
             # mode may cause newline translations, making the
             # actual size of the content transmitted *less* than
             # the content-length!
-            file = open(path, 'rb')
+            f = open(path, 'rb')
         except IOError:
             self.send_error(404, "File not found")
             return
 
-        file_size = os.fstat(file.fileno())[6]
-        tail, ranges = self.parse_ranges(ranges_header_value)
-        # Normalize tail into ranges
-        if tail != 0:
-            ranges.append((file_size - tail, file_size))
-
-        self._satisfiable_ranges = True
-        if len(ranges) == 0:
-            self._satisfiable_ranges = False
-        else:
-            def check_range(range_specifier):
-                start, end = range_specifier
-                # RFC2616 14.35, ranges are invalid if start >= file_size
-                if start >= file_size:
-                    self._satisfiable_ranges = False # Side-effect !
-                    return 0, 0
-                # RFC2616 14.35, end values should be truncated
-                # to file_size -1 if they exceed it
-                end = min(end, file_size - 1)
-                return start, end
-
-            ranges = map(check_range, ranges)
-
-        if not self._satisfiable_ranges:
+        file_size = os.fstat(f.fileno())[6]
+        ranges = self._parse_ranges(ranges_header_value, file_size)
+        if not ranges:
             # RFC2616 14.16 and 14.35 says that when a server
             # encounters unsatisfiable range specifiers, it
             # SHOULD return a 416.
-            file.close()
+            f.close()
             # FIXME: We SHOULD send a Content-Range header too,
             # but the implementation of send_error does not
             # allows that. So far.
@@ -289,10 +295,10 @@
 
         if len(ranges) == 1:
             (start, end) = ranges[0]
-            self.get_single_range(file, file_size, start, end)
+            self.get_single_range(f, file_size, start, end)
         else:
-            self.get_multiple_ranges(file, file_size, ranges)
-        file.close()
+            self.get_multiple_ranges(f, file_size, ranges)
+        f.close()
 
     def translate_path(self, path):
         """Translate a /-separated PATH to the local filename syntax.
@@ -354,52 +360,17 @@
         self.test_case_server = test_case_server
         self._home_dir = test_case_server._home_dir
 
-    def stop_server(self):
-         """Called to clean-up the server.
-
-         Since the server may be (surely is, even) in a blocking listen, we
-         shutdown its socket before closing it.
-         """
-         # Note that is this executed as part of the implicit tear down in the
-         # main thread while the server runs in its own thread. The clean way
-         # to tear down the server is to instruct him to stop accepting
-         # connections and wait for the current connection(s) to end
-         # naturally. To end the connection naturally, the http transports
-         # should close their socket when they do not need to talk to the
-         # server anymore. This happens naturally during the garbage collection
-         # phase of the test transport objetcs (the server clients), so we
-         # don't have to worry about them.  So, for the server, we must tear
-         # down here, from the main thread, when the test have ended.  Note
-         # that since the server is in a blocking operation and since python
-         # use select internally, shutting down the socket is reliable and
-         # relatively clean.
-         try:
-             self.socket.shutdown(socket.SHUT_RDWR)
-         except socket.error, e:
-             # WSAENOTCONN (10057) 'Socket is not connected' is harmless on
-             # windows (occurs before the first connection attempt
-             # vila--20071230)
-
-             # 'Socket is not connected' can also occur on OSX, with a
-             # "regular" ENOTCONN (when something went wrong during test case
-             # setup leading to self.setUp() *not* being called but
-             # self.stop_server() still being called -- vila20081106
-             if not len(e.args) or e.args[0] not in (errno.ENOTCONN, 10057):
-                 raise
-         # Let the server properly close the socket
-         self.server_close()
-
-
-class TestingHTTPServer(SocketServer.TCPServer, TestingHTTPServerMixin):
+
+class TestingHTTPServer(test_server.TestingTCPServer, TestingHTTPServerMixin):
 
     def __init__(self, server_address, request_handler_class,
                  test_case_server):
+        test_server.TestingTCPServer.__init__(self, server_address,
+                                              request_handler_class)
         TestingHTTPServerMixin.__init__(self, test_case_server)
-        SocketServer.TCPServer.__init__(self, server_address,
-                                        request_handler_class)
-
-
-class TestingThreadingHTTPServer(SocketServer.ThreadingTCPServer,
+
+
+class TestingThreadingHTTPServer(test_server.TestingThreadingTCPServer,
                                  TestingHTTPServerMixin):
     """A threading HTTP test server for HTTP 1.1.
 
@@ -407,36 +378,14 @@
     server, we need an independent connection for each of them. We achieve that
     by spawning a new thread for each connection.
     """
-
     def __init__(self, server_address, request_handler_class,
                  test_case_server):
+        test_server.TestingThreadingTCPServer.__init__(self, server_address,
+                                                       request_handler_class)
         TestingHTTPServerMixin.__init__(self, test_case_server)
-        SocketServer.ThreadingTCPServer.__init__(self, server_address,
-                                                 request_handler_class)
-        # Decides how threads will act upon termination of the main
-        # process. This is prophylactic as we should not leave the threads
-        # lying around.
-        self.daemon_threads = True
-
-    def process_request_thread(self, request, client_address):
-        SocketServer.ThreadingTCPServer.process_request_thread(
-            self, request, client_address)
-        # Under some circumstances (as in bug #383920), we need to force the
-        # shutdown as python delays it until gc occur otherwise and the client
-        # may hang.
-        try:
-            # The request process has been completed, the thread is about to
-            # die, let's shutdown the socket if we can.
-            request.shutdown(socket.SHUT_RDWR)
-        except (socket.error, select.error), e:
-            if e[0] in (errno.EBADF, errno.ENOTCONN):
-                # Right, the socket is already down
-                pass
-            else:
-                raise
-
-
-class HttpServer(transport.Server):
+
+
+class HttpServer(test_server.TestingTCPServerInAThread):
     """A test server for http transports.
 
     Subclasses can provide a specific request handler.
@@ -464,82 +413,33 @@
         :param protocol_version: if specified, will override the protocol
             version of the request handler.
         """
-        transport.Server.__init__(self)
-        self.request_handler = request_handler
+        # Depending on the protocol version, we will create the approriate
+        # server
+        if protocol_version is None:
+            # Use the request handler one
+            proto_vers = request_handler.protocol_version
+        else:
+            # Use our own, it will be used to override the request handler
+            # one too.
+            proto_vers = protocol_version
+        # Get the appropriate server class for the required protocol
+        serv_cls = self.http_server_class.get(proto_vers, None)
+        if serv_cls is None:
+            raise httplib.UnknownProtocol(proto_vers)
         self.host = 'localhost'
         self.port = 0
-        self._httpd = None
-        self.protocol_version = protocol_version
+        super(HttpServer, self).__init__((self.host, self.port),
+                                         serv_cls,
+                                         request_handler)
+        self.protocol_version = proto_vers
         # Allows tests to verify number of GET requests issued
         self.GET_request_nb = 0
-
-    def create_httpd(self, serv_cls, rhandler_cls):
-        return serv_cls((self.host, self.port), self.request_handler, self)
-
-    def __repr__(self):
-        return "%s(%s:%s)" % \
-            (self.__class__.__name__, self.host, self.port)
-
-    def _get_httpd(self):
-        if self._httpd is None:
-            rhandler = self.request_handler
-            # Depending on the protocol version, we will create the approriate
-            # server
-            if self.protocol_version is None:
-                # Use the request handler one
-                proto_vers = rhandler.protocol_version
-            else:
-                # Use our own, it will be used to override the request handler
-                # one too.
-                proto_vers = self.protocol_version
-            # Create the appropriate server for the required protocol
-            serv_cls = self.http_server_class.get(proto_vers, None)
-            if serv_cls is None:
-                raise httplib.UnknownProtocol(proto_vers)
-            else:
-                self._httpd = self.create_httpd(serv_cls, rhandler)
-            self.host, self.port = self._httpd.socket.getsockname()
-        return self._httpd
-
-    def _http_start(self):
-        """Server thread main entry point. """
-        self._http_running = False
-        try:
-            try:
-                httpd = self._get_httpd()
-                self._http_base_url = '%s://%s:%s/' % (self._url_protocol,
-                                                       self.host, self.port)
-                self._http_running = True
-            except:
-                # Whatever goes wrong, we save the exception for the main
-                # thread. Note that since we are running in a thread, no signal
-                # can be received, so we don't care about KeyboardInterrupt.
-                self._http_exception = sys.exc_info()
-        finally:
-            # Release the lock or the main thread will block and the whole
-            # process will hang.
-            self._http_starting.release()
-
-        # From now on, exceptions are taken care of by the
-        # SocketServer.BaseServer or the request handler.
-        while self._http_running:
-            try:
-                # Really an HTTP connection but the python framework is generic
-                # and call them requests
-                httpd.handle_request()
-            except socket.timeout:
-                pass
-            except (socket.error, select.error), e:
-                if (e[0] == errno.EBADF
-                    or (sys.platform == 'win32' and e[0] == 10038)):
-                    # Starting with python-2.6, handle_request may raise socket
-                    # or select exceptions when the server is shut down (as we
-                    # do).
-                    # 10038 = WSAENOTSOCK
-                    # http://msdn.microsoft.com/en-us/library/ms740668%28VS.85%29.aspx
-                    pass
-                else:
-                    raise
+        self._http_base_url = None
+        self.logs = []
+
+    def create_server(self):
+        return self.server_class(
+            (self.host, self.port), self.request_handler_class, self)
 
     def _get_remote_url(self, path):
         path_parts = path.split(os.path.sep)
@@ -570,36 +470,15 @@
                 or isinstance(backing_transport_server,
                               test_server.LocalURLServer)):
             raise AssertionError(
-                "HTTPServer currently assumes local transport, got %s" % \
+                "HTTPServer currently assumes local transport, got %s" %
                 backing_transport_server)
         self._home_dir = os.getcwdu()
         self._local_path_parts = self._home_dir.split(os.path.sep)
-        self._http_base_url = None
-
-        # Create the server thread
-        self._http_starting = threading.Lock()
-        self._http_starting.acquire()
-        self._http_thread = threading.Thread(target=self._http_start)
-        self._http_thread.setDaemon(True)
-        self._http_exception = None
-        self._http_thread.start()
-
-        # Wait for the server thread to start (i.e release the lock)
-        self._http_starting.acquire()
-
-        if self._http_exception is not None:
-            # Something went wrong during server start
-            exc_class, exc_value, exc_tb = self._http_exception
-            raise exc_class, exc_value, exc_tb
-        self._http_starting.release()
         self.logs = []
 
-    def stop_server(self):
-        self._httpd.stop_server()
-        self._http_running = False
-        # We don't need to 'self._http_thread.join()' here since the thread is
-        # a daemonic one and will be garbage collected anyway. Joining just
-        # slows us down for no added benefit.
+        super(HttpServer, self).start_server()
+        self._http_base_url = '%s://%s:%s/' % (
+            self._url_protocol, self.host, self.port)
 
     def get_url(self):
         """See bzrlib.transport.Server.get_url."""

=== modified file 'bzrlib/tests/http_utils.py'
--- bzrlib/tests/http_utils.py	2010-06-11 10:59:23 +0000
+++ bzrlib/tests/http_utils.py	2011-07-23 16:33:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,26 +15,21 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from cStringIO import StringIO
-import errno
 import re
-import socket
-import threading
-import time
 import urllib2
-import urlparse
 
 
 from bzrlib import (
     errors,
     osutils,
     tests,
-    )
-from bzrlib.smart import medium, protocol
+    transport,
+    )
+from bzrlib.smart import (
+    medium,
+    )
 from bzrlib.tests import http_server
-from bzrlib.transport import (
-    chroot,
-    get_transport,
-    )
+from bzrlib.transport import chroot
 
 
 class HTTPServerWithSmarts(http_server.HttpServer):
@@ -56,11 +51,12 @@
 
     def do_POST(self):
         """Hand the request off to a smart server instance."""
-        backing = get_transport(self.server.test_case_server._home_dir)
+        backing = transport.get_transport_from_path(
+            self.server.test_case_server._home_dir)
         chroot_server = chroot.ChrootServer(backing)
         chroot_server.start_server()
         try:
-            t = get_transport(chroot_server.get_url())
+            t = transport.get_transport_from_url(chroot_server.get_url())
             self.do_POST_inner(t)
         finally:
             chroot_server.stop_server()
@@ -107,18 +103,21 @@
     backed by regular disk files.
     """
 
-    # This can be overriden or parametrized by daughter clasess if needed, but
-    # it must exist so that the create_transport_readonly_server() method can
-    # propagate it.
+    # These attributes can be overriden or parametrized by daughter clasess if
+    # needed, but must exist so that the create_transport_readonly_server()
+    # method (or any method creating an http(s) server) can propagate it.
     _protocol_version = None
+    _url_protocol = 'http'
 
     def setUp(self):
         super(TestCaseWithWebserver, self).setUp()
         self.transport_readonly_server = http_server.HttpServer
 
     def create_transport_readonly_server(self):
-        return self.transport_readonly_server(
+        server = self.transport_readonly_server(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
 
 class TestCaseWithTwoWebservers(TestCaseWithWebserver):
@@ -137,8 +136,10 @@
 
         This is mostly a hook for daughter classes.
         """
-        return self.transport_secondary_server(
+        server = self.transport_secondary_server(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def get_secondary_server(self):
         """Get the server instance for the secondary transport."""
@@ -147,6 +148,15 @@
             self.start_server(self.__secondary_server)
         return self.__secondary_server
 
+    def get_secondary_url(self, relpath=None):
+        base = self.get_secondary_server().get_url()
+        return self._adjust_url(base, relpath)
+
+    def get_secondary_transport(self, relpath=None):
+        t = transport.get_transport_from_url(self.get_secondary_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
 
 class ProxyServer(http_server.HttpServer):
     """A proxy test server for http transports."""
@@ -226,20 +236,39 @@
    The 'old' server is redirected to the 'new' server.
    """
 
+   def setUp(self):
+       super(TestCaseWithRedirectedWebserver, self).setUp()
+       # The redirections will point to the new server
+       self.new_server = self.get_readonly_server()
+       # The requests to the old server will be redirected to the new server
+       self.old_server = self.get_secondary_server()
+
    def create_transport_secondary_server(self):
        """Create the secondary server redirecting to the primary server"""
        new = self.get_readonly_server()
        redirecting = HTTPServerRedirecting(
            protocol_version=self._protocol_version)
        redirecting.redirect_to(new.host, new.port)
+       redirecting._url_protocol = self._url_protocol
        return redirecting
 
-   def setUp(self):
-       super(TestCaseWithRedirectedWebserver, self).setUp()
-       # The redirections will point to the new server
-       self.new_server = self.get_readonly_server()
-       # The requests to the old server will be redirected
-       self.old_server = self.get_secondary_server()
+   def get_old_url(self, relpath=None):
+        base = self.old_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_old_transport(self, relpath=None):
+        t = transport.get_transport_from_url(self.get_old_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
+
+   def get_new_url(self, relpath=None):
+        base = self.new_server.get_url()
+        return self._adjust_url(base, relpath)
+
+   def get_new_transport(self, relpath=None):
+        t = transport.get_transport_from_url(self.get_new_url(relpath))
+        self.assertTrue(t.is_readonly())
+        return t
 
 
 class AuthRequestHandler(http_server.TestingHTTPRequestHandler):
@@ -255,21 +284,30 @@
     # - auth_header_recv: the header received containing auth
     # - auth_error_code: the error code to indicate auth required
 
+    def _require_authentication(self):
+        # Note that we must update test_case_server *before*
+        # sending the error or the client may try to read it
+        # before we have sent the whole error back.
+        tcs = self.server.test_case_server
+        tcs.auth_required_errors += 1
+        self.send_response(tcs.auth_error_code)
+        self.send_header_auth_reqed()
+        # We do not send a body
+        self.send_header('Content-Length', '0')
+        self.end_headers()
+        return
+
     def do_GET(self):
         if self.authorized():
             return http_server.TestingHTTPRequestHandler.do_GET(self)
         else:
-            # Note that we must update test_case_server *before*
-            # sending the error or the client may try to read it
-            # before we have sent the whole error back.
-            tcs = self.server.test_case_server
-            tcs.auth_required_errors += 1
-            self.send_response(tcs.auth_error_code)
-            self.send_header_auth_reqed()
-            # We do not send a body
-            self.send_header('Content-Length', '0')
-            self.end_headers()
-            return
+            return self._require_authentication()
+
+    def do_HEAD(self):
+        if self.authorized():
+            return http_server.TestingHTTPRequestHandler.do_HEAD(self)
+        else:
+            return self._require_authentication()
 
 
 class BasicAuthRequestHandler(AuthRequestHandler):

=== modified file 'bzrlib/tests/https_server.py'
--- bzrlib/tests/https_server.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/https_server.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,10 +17,12 @@
 """HTTPS test server, available when ssl python module is available"""
 
 import ssl
+import sys
 
 from bzrlib.tests import (
     http_server,
     ssl_certs,
+    test_server,
     )
 
 
@@ -30,17 +32,36 @@
         self.key_file = key_file
         self.cert_file = cert_file
 
-    def get_request (self):
-        """Get the request and client address from the socket.
-
-        This is called in response to a connection issued to the server, we
-        wrap the socket with SSL.
+    def _get_ssl_request (self, sock, addr):
+        """Wrap the socket with SSL"""
+        ssl_sock = ssl.wrap_socket(sock, server_side=True,
+                                   keyfile=self.key_file,
+                                   certfile=self.cert_file,
+                                   do_handshake_on_connect=False)
+        return ssl_sock, addr
+
+    def verify_request(self, request, client_address):
+        """Verify the request.
+
+        Return True if we should proceed with this request, False if we should
+        not even touch a single byte in the socket !
         """
-        sock, addr = self.socket.accept()
-        sslconn = ssl.wrap_socket(sock, server_side=True,
-                                  keyfile=self.key_file,
-                                  certfile=self.cert_file)
-        return sslconn, addr
+        serving = test_server.TestingTCPServerMixin.verify_request(
+            self, request, client_address)
+        if serving:
+            request.do_handshake()
+        return serving
+
+    def ignored_exceptions_during_shutdown(self, e):
+        if (sys.version < (2, 7) and isinstance(e, TypeError)
+            and e.args[0] == "'member_descriptor' object is not callable"):
+            # Fixed in python-2.7 (and some Ubuntu 2.6) there is a bug where
+            # the ssl socket fail to raise a socket.error when trying to read
+            # from a closed socket. This is rarely observed in practice but
+            # still make valid selftest runs fail if not caught.
+            return True
+        base = test_server.TestingTCPServerMixin
+        return base.ignored_exceptions_during_shutdown(self, e)
 
 
 class TestingHTTPSServer(TestingHTTPSServerMixin,
@@ -52,6 +73,10 @@
         http_server.TestingHTTPServer.__init__(
             self, server_address, request_handler_class, test_case_server)
 
+    def get_request(self):
+        sock, addr = http_server.TestingHTTPServer.get_request(self)
+        return self._get_ssl_request(sock, addr)
+
 
 class TestingThreadingHTTPSServer(TestingHTTPSServerMixin,
                                   http_server.TestingThreadingHTTPServer):
@@ -62,6 +87,10 @@
         http_server.TestingThreadingHTTPServer.__init__(
             self, server_address, request_handler_class, test_case_server)
 
+    def get_request(self):
+        sock, addr = http_server.TestingThreadingHTTPServer.get_request(self)
+        return self._get_ssl_request(sock, addr)
+
 
 class HTTPSServer(http_server.HttpServer):
 
@@ -73,7 +102,7 @@
                          }
 
     # Provides usable defaults since an https server requires both a
-    # private key and certificate to work.
+    # private key and a certificate to work.
     def __init__(self, request_handler=http_server.TestingHTTPRequestHandler,
                  protocol_version=None,
                  key_file=ssl_certs.build_path('server_without_pass.key'),
@@ -84,9 +113,10 @@
         self.cert_file = cert_file
         self.temp_files = []
 
-    def create_httpd(self, serv_cls, rhandler_cls):
-        return serv_cls((self.host, self.port), self.request_handler,
-                        self, self.key_file, self.cert_file)
+    def create_server(self):
+        return self.server_class(
+            (self.host, self.port), self.request_handler_class, self,
+            self.key_file, self.cert_file)
 
 
 class HTTPSServer_urllib(HTTPSServer):

=== modified file 'bzrlib/tests/matchers.py'
--- bzrlib/tests/matchers.py	2010-05-06 11:08:10 +0000
+++ bzrlib/tests/matchers.py	2011-08-31 23:17:41 +0000
@@ -27,10 +27,17 @@
 """
 
 __all__ = [
+    'HasLayout',
+    'MatchesAncestry',
     'ReturnsUnlockable',
     ]
 
-from testtools.matchers import Mismatch, Matcher
+from bzrlib import (
+    osutils,
+    revision as _mod_revision,
+    )
+
+from testtools.matchers import Equals, Mismatch, Matcher
 
 
 class ReturnsUnlockable(Matcher):
@@ -48,7 +55,7 @@
         self.lockable_thing = lockable_thing
 
     def __str__(self):
-        return ('ReturnsUnlockable(lockable_thing=%s)' % 
+        return ('ReturnsUnlockable(lockable_thing=%s)' %
             self.lockable_thing)
 
     def match(self, lock_method):
@@ -66,3 +73,105 @@
 
     def describe(self):
         return "%s is locked" % self.lockable_thing
+
+
+class _AncestryMismatch(Mismatch):
+    """Ancestry matching mismatch."""
+
+    def __init__(self, tip_revision, got, expected):
+        self.tip_revision = tip_revision
+        self.got = got
+        self.expected = expected
+
+    def describe(self):
+        return "mismatched ancestry for revision %r was %r, expected %r" % (
+            self.tip_revision, self.got, self.expected)
+
+
+class MatchesAncestry(Matcher):
+    """A matcher that checks the ancestry of a particular revision.
+
+    :ivar graph: Graph in which to check the ancestry
+    :ivar revision_id: Revision id of the revision
+    """
+
+    def __init__(self, repository, revision_id):
+        Matcher.__init__(self)
+        self.repository = repository
+        self.revision_id = revision_id
+
+    def __str__(self):
+        return ('MatchesAncestry(repository=%r, revision_id=%r)' % (
+            self.repository, self.revision_id))
+
+    def match(self, expected):
+        self.repository.lock_read()
+        try:
+            graph = self.repository.get_graph()
+            got = [r for r, p in graph.iter_ancestry([self.revision_id])]
+            if _mod_revision.NULL_REVISION in got:
+                got.remove(_mod_revision.NULL_REVISION)
+        finally:
+            self.repository.unlock()
+        if sorted(got) != sorted(expected):
+            return _AncestryMismatch(self.revision_id, sorted(got),
+                sorted(expected))
+
+
+class HasLayout(Matcher):
+    """A matcher that checks if a tree has a specific layout.
+
+    :ivar entries: List of expected entries, as (path, file_id) pairs.
+    """
+
+    def __init__(self, entries):
+        Matcher.__init__(self)
+        self.entries = entries
+
+    def get_tree_layout(self, tree):
+        """Get the (path, file_id) pairs for the current tree."""
+        tree.lock_read()
+        try:
+            for path, ie in tree.iter_entries_by_dir():
+                if ie.parent_id is None:
+                    yield (u"", ie.file_id)
+                else:
+                    yield (path+ie.kind_character(), ie.file_id)
+        finally:
+            tree.unlock()
+
+    @staticmethod
+    def _strip_unreferenced_directories(entries):
+        """Strip all directories that don't (in)directly contain any files.
+
+        :param entries: List of path strings or (path, ie) tuples to process
+        """
+        directories = []
+        for entry in entries:
+            if isinstance(entry, basestring):
+                path = entry
+            else:
+                path = entry[0]
+            if not path or path[-1] == "/":
+                # directory
+                directories.append((path, entry))
+            else:
+                # Yield the referenced parent directories
+                for dirpath, direntry in directories:
+                    if osutils.is_inside(dirpath, path):
+                        yield direntry
+                directories = []
+                yield entry
+
+    def __str__(self):
+        return 'HasLayout(%r)' % self.entries
+
+    def match(self, tree):
+        actual = list(self.get_tree_layout(tree))
+        if self.entries and isinstance(self.entries[0], basestring):
+            actual = [path for (path, fileid) in actual]
+        if not tree.has_versioned_directories():
+            entries = list(self._strip_unreferenced_directories(self.entries))
+        else:
+            entries = self.entries
+        return Equals(entries).match(actual)

=== modified file 'bzrlib/tests/per_branch/__init__.py'
--- bzrlib/tests/per_branch/__init__.py	2010-06-18 04:37:50 +0000
+++ bzrlib/tests/per_branch/__init__.py	2011-10-05 12:51:41 +0000
@@ -20,20 +20,18 @@
 """Branch implementation tests for bzr.
 
 These test the conformance of all the branch variations to the expected API.
-Specific tests for individual formats are in the tests/test_branch file
-rather than in tests/per_branch/*.py.
+Specific tests for individual formats are in the `tests/test_branch` file
+rather than in `tests/per_branch/*.py`.
 """
 
 from bzrlib import (
     errors,
     tests,
     )
-from bzrlib.branch import (BranchFormat,
-                           _legacy_formats,
-                           )
-from bzrlib.remote import RemoteBranchFormat, RemoteBzrDirFormat
+from bzrlib.branch import format_registry
+from bzrlib.remote import RemoteBranchFormat
 from bzrlib.tests import test_server
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
 from bzrlib.transport import memory
 
 
@@ -60,7 +58,7 @@
     return result
 
 
-class TestCaseWithBranch(TestCaseWithBzrDir):
+class TestCaseWithBranch(TestCaseWithControlDir):
     """This helper will be parameterised in each per_branch test."""
 
     def setUp(self):
@@ -69,29 +67,19 @@
 
     def get_branch(self):
         if self.branch is None:
-            self.branch = self.make_branch('')
+            self.branch = self.make_branch('abranch')
         return self.branch
 
+    def get_default_format(self):
+        format = self.bzrdir_format
+        self.assertEquals(format.get_branch_format(), self.branch_format)
+        return format
+
     def make_branch(self, relpath, format=None):
-        if format is not None:
-            return TestCaseWithBzrDir.make_branch(self, relpath, format)
-        repo = self.make_repository(relpath)
-        # fixme RBC 20060210 this isnt necessarily a fixable thing,
-        # Skipped is the wrong exception to raise.
         try:
-            return self.branch_format.initialize(repo.bzrdir)
+            return super(TestCaseWithBranch, self).make_branch(relpath, format)
         except errors.UninitializableFormat:
-            raise tests.TestSkipped('Uninitializable branch format')
-
-    def make_branch_builder(self, relpath, format=None):
-        if format is None:
-            format = self.branch_format._matchingbzrdir
-        return super(TestCaseWithBranch, self).make_branch_builder(
-            relpath, format=format)
-
-    def make_repository(self, relpath, shared=False, format=None):
-        made_control = self.make_bzrdir(relpath, format=format)
-        return made_control.create_repository(shared=shared)
+            raise tests.TestNotApplicable('Uninitializable branch format')
 
     def create_tree_with_merge(self):
         """Create a branch with a simple ancestry.
@@ -132,7 +120,7 @@
     # Generate a list of branch formats and their associated bzrdir formats to
     # use.
     combinations = [(format, format._matchingbzrdir) for format in
-         BranchFormat.get_formats() + _legacy_formats]
+         format_registry._get_all()]
     scenarios = make_scenarios(
         # None here will cause the default vfs transport server to be used.
         None,

=== modified file 'bzrlib/tests/per_branch/test_bound_sftp.py'
--- bzrlib/tests/per_branch/test_bound_sftp.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_branch/test_bound_sftp.py	2011-05-13 12:51:05 +0000
@@ -94,8 +94,8 @@
         # this line is more of a working tree test line, but - what the hey,
         # it has work to do.
         b_child.bzrdir.open_workingtree().update()
-        self.failUnlessExists('child/a')
-        self.failUnlessExists('child/b')
+        self.assertPathExists('child/a')
+        self.assertPathExists('child/b')
 
         b_child.unbind()
         self.assertEqual(None, b_child.get_bound_location())
@@ -150,7 +150,7 @@
         # TestCaseWithSFTPServer only allows you to connect one time
         # to the SFTP server. So we have to create a connection and
         # keep it around, so that it can be reused
-        __unused_t = transport.get_transport(self.get_url('.'))
+        __unused_t = self.get_transport()
 
         wt_base = b_base.bzrdir.open_workingtree()
         open('base/a', 'wb').write('new base contents\n')
@@ -292,22 +292,22 @@
         wt_other.add('c')
         wt_other.commit('adding c', rev_id='r@d-2')
 
-        self.failIf(wt_child.branch.repository.has_revision('r@d-2'))
-        self.failIf(b_base.repository.has_revision('r@d-2'))
+        self.assertFalse(wt_child.branch.repository.has_revision('r@d-2'))
+        self.assertFalse(b_base.repository.has_revision('r@d-2'))
 
         wt_child.merge_from_branch(wt_other.branch)
 
-        self.failUnlessExists('child/c')
+        self.assertPathExists('child/c')
         self.assertEqual(['r@d-2'], wt_child.get_parent_ids()[1:])
-        self.failUnless(wt_child.branch.repository.has_revision('r@d-2'))
-        self.failIf(b_base.repository.has_revision('r@d-2'))
+        self.assertTrue(wt_child.branch.repository.has_revision('r@d-2'))
+        self.assertFalse(b_base.repository.has_revision('r@d-2'))
 
         # Commit should succeed, and cause merged revisions to
         # be pushed into base
         wt_child.commit('merge other', rev_id='r@c-2')
         self.assertEqual(['r@b-1', 'r@c-2'], wt_child.branch.revision_history())
         self.assertEqual(['r@b-1', 'r@c-2'], b_base.revision_history())
-        self.failUnless(b_base.repository.has_revision('r@d-2'))
+        self.assertTrue(b_base.repository.has_revision('r@d-2'))
 
     def test_commit_fails(self):
         b_base, wt_child = self.create_branches()

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2010-04-29 08:59:13 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2011-10-14 13:56:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,21 +19,21 @@
 from bzrlib import (
     branch as _mod_branch,
     bzrdir,
+    config,
     delta as _mod_delta,
     errors,
     gpg,
     merge,
+    osutils,
     urlutils,
-    transactions,
     transport,
     remote,
     repository,
     revision,
+    symbol_versioning,
     tests,
     )
-from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests import (
-    http_server,
     per_branch,
     )
 from bzrlib.tests.http_server import HttpServer
@@ -76,10 +76,8 @@
 
         br = self.get_branch()
         br.fetch(wt.branch)
-        br.set_revision_history(['rev1', 'rev2', 'rev3'])
-        rh = br.revision_history()
-        self.assertEqual(['rev1', 'rev2', 'rev3'], rh)
-        for revision_id in rh:
+        br.generate_revision_history('rev3')
+        for revision_id in ['rev3', 'rev2', 'rev1']:
             self.assertIsInstance(revision_id, str)
         last = br.last_revision()
         self.assertEqual('rev3', last)
@@ -115,10 +113,12 @@
         tree_a.add('vla', 'file2')
         tree_a.commit('rev2', rev_id='rev2')
 
-        delta = tree_a.branch.get_revision_delta(1)
+        delta = self.applyDeprecated(symbol_versioning.deprecated_in(
+            (2, 5, 0)), tree_a.branch.get_revision_delta, 1)
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([('foo', 'file1', 'file')], delta.added)
-        delta = tree_a.branch.get_revision_delta(2)
+        delta = self.applyDeprecated(symbol_versioning.deprecated_in(
+            (2, 5, 0)), tree_a.branch.get_revision_delta, 2)
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([('vla', 'file2', 'file')], delta.added)
 
@@ -186,7 +186,7 @@
         self.assertEqual(branch_b.get_parent(), branch_c.get_parent())
 
         # We can also set a specific parent, and it should be honored
-        random_parent = 'http://bazaar-vcs.org/path/to/branch'
+        random_parent = 'http://example.com/path/to/branch'
         branch_b.set_parent(random_parent)
         repo_d = self.make_repository('d')
         branch_b.repository.copy_content_into(repo_d)
@@ -216,6 +216,9 @@
     def test_record_initial_ghost(self):
         """Branches should support having ghosts."""
         wt = self.make_branch_and_tree('.')
+        if not wt.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable("repository format does not "
+                "support ghosts")
         wt.set_parent_ids(['non:existent@rev--ision--0--2'],
             allow_leftmost_as_ghost=True)
         self.assertEqual(['non:existent@rev--ision--0--2'],
@@ -229,6 +232,9 @@
     def test_record_two_ghosts(self):
         """Recording with all ghosts works."""
         wt = self.make_branch_and_tree('.')
+        if not wt.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable("repository format does not "
+                "support ghosts")
         wt.set_parent_ids([
                 'foo@azkhazan-123123-abcabc',
                 'wibble@fofof--20050401--1928390812',
@@ -246,108 +252,18 @@
                           self.get_branch().repository.get_revision,
                           None)
 
-# TODO 20051003 RBC:
-# compare the gpg-to-sign info for a commit with a ghost and
-#     an identical tree without a ghost
-# fetch missing should rewrite the TOC of weaves to list newly available parents.
-
-    def test_sign_existing_revision(self):
-        wt = self.make_branch_and_tree('.')
-        branch = wt.branch
-        wt.commit("base", allow_pointless=True, rev_id='A')
-        from bzrlib.testament import Testament
-        strategy = gpg.LoopbackGPGStrategy(None)
-        branch.repository.lock_write()
-        branch.repository.start_write_group()
-        branch.repository.sign_revision('A', strategy)
-        branch.repository.commit_write_group()
-        branch.repository.unlock()
-        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
-                         Testament.from_revision(branch.repository,
-                         'A').as_short_text() +
-                         '-----END PSEUDO-SIGNED CONTENT-----\n',
-                         branch.repository.get_signature_text('A'))
-
-    def test_store_signature(self):
-        wt = self.make_branch_and_tree('.')
-        branch = wt.branch
-        branch.lock_write()
-        try:
-            branch.repository.start_write_group()
-            try:
-                branch.repository.store_revision_signature(
-                    gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
-            except:
-                branch.repository.abort_write_group()
-                raise
-            else:
-                branch.repository.commit_write_group()
-        finally:
-            branch.unlock()
-        # A signature without a revision should not be accessible.
-        self.assertRaises(errors.NoSuchRevision,
-                          branch.repository.has_signature_for_revision_id,
-                          'A')
-        wt.commit("base", allow_pointless=True, rev_id='A')
-        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n'
-                         'FOO-----END PSEUDO-SIGNED CONTENT-----\n',
-                         branch.repository.get_signature_text('A'))
-
-    def test_branch_keeps_signatures(self):
-        wt = self.make_branch_and_tree('source')
-        wt.commit('A', allow_pointless=True, rev_id='A')
-        repo = wt.branch.repository
-        repo.lock_write()
-        repo.start_write_group()
-        repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
-        repo.commit_write_group()
-        repo.unlock()
-        #FIXME: clone should work to urls,
-        # wt.clone should work to disks.
-        self.build_tree(['target/'])
-        d2 = repo.bzrdir.clone(urlutils.local_path_to_url('target'))
-        self.assertEqual(repo.get_signature_text('A'),
-                         d2.open_repository().get_signature_text('A'))
-
-    def test_missing_revisions(self):
-        t1 = self.make_branch_and_tree('b1')
-        rev1 = t1.commit('one')
-        t2 = t1.bzrdir.sprout('b2').open_workingtree()
-        rev2 = t1.commit('two')
-        rev3 = t1.commit('three')
-
-        self.assertEqual([rev2, rev3],
-            self.applyDeprecated(deprecated_in((1, 6, 0)),
-            t2.branch.missing_revisions, t1.branch))
-
-        self.assertEqual([],
-            self.applyDeprecated(deprecated_in((1, 6, 0)),
-            t2.branch.missing_revisions, t1.branch, stop_revision=1))
-        self.assertEqual([rev2],
-            self.applyDeprecated(deprecated_in((1, 6, 0)),
-            t2.branch.missing_revisions, t1.branch, stop_revision=2))
-        self.assertEqual([rev2, rev3],
-            self.applyDeprecated(deprecated_in((1, 6, 0)),
-            t2.branch.missing_revisions, t1.branch, stop_revision=3))
-
-        self.assertRaises(errors.NoSuchRevision,
-            self.applyDeprecated, deprecated_in((1, 6, 0)),
-            t2.branch.missing_revisions, t1.branch, stop_revision=4)
-
-        rev4 = t2.commit('four')
-        self.assertRaises(errors.DivergedBranches,
-            self.applyDeprecated, deprecated_in((1, 6, 0)),
-            t2.branch.missing_revisions, t1.branch)
-
-    def test_nicks(self):
-        """Test explicit and implicit branch nicknames.
+    def test_nicks_bzr(self):
+        """Test the behaviour of branch nicks specific to bzr branches.
 
         Nicknames are implicitly the name of the branch's directory, unless an
         explicit nickname is set.  That is, an explicit nickname always
         overrides the implicit one.
+
         """
-        t = transport.get_transport(self.get_url())
+        t = self.get_transport()
         branch = self.make_branch('bzr.dev')
+        if not isinstance(branch, _mod_branch.BzrBranch):
+            raise tests.TestNotApplicable("not a bzr branch format")
         # The nick will be 'bzr.dev', because there is no explicit nick set.
         self.assertEqual(branch.nick, 'bzr.dev')
         # Move the branch to a different directory, 'bzr.ab'.  Now that branch
@@ -359,7 +275,7 @@
         # config file in the branch.
         branch.nick = "Aaron's branch"
         if not isinstance(branch, remote.RemoteBranch):
-            self.failUnless(branch._transport.has("branch.conf"))
+            self.assertTrue(branch._transport.has("branch.conf"))
         # Because the nick has been set explicitly, the nick is now always
         # "Aaron's branch", regardless of directory name.
         self.assertEqual(branch.nick, "Aaron's branch")
@@ -369,6 +285,24 @@
         branch.nick = u"\u1234"
         self.assertEqual(branch.nick, u"\u1234")
 
+    def test_nicks(self):
+        """Test explicit and implicit branch nicknames.
+
+        A nickname is always available, whether set explicitly or not.
+        """
+        t = self.get_transport()
+        branch = self.make_branch('bzr.dev')
+        # An implicit nick name is set; what it is exactly depends on the
+        # format.
+        self.assertIsInstance(branch.nick, basestring)
+        # Set the branch nick explicitly.
+        branch.nick = "Aaron's branch"
+        # Because the nick has been set explicitly, the nick is now always
+        # "Aaron's branch".
+        self.assertEqual(branch.nick, "Aaron's branch")
+        branch.nick = u"\u1234"
+        self.assertEqual(branch.nick, u"\u1234")
+
     def test_commit_nicks(self):
         """Nicknames are committed to the revision"""
         wt = self.make_branch_and_tree('bzr.dev')
@@ -396,14 +330,45 @@
         self.branch_format.initialize(repo.bzrdir, name='branch2')
         self.assertEquals(2, len(repo.bzrdir.list_branches()))
 
+    def test_create_append_revisions_only(self):
+        try:
+            repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        for val in (True, False):
+            try:
+                branch = self.branch_format.initialize(repo.bzrdir,
+                    append_revisions_only=True)
+            except (errors.UninitializableFormat, errors.UpgradeRequired):
+                # branch references are not default init'able and
+                # not all branches support append_revisions_only
+                return
+            self.assertEquals(True, branch.get_append_revisions_only())
+            repo.bzrdir.destroy_branch()
+
+    def test_get_set_append_revisions_only(self):
+        branch = self.make_branch('.')
+        if branch._format.supports_set_append_revisions_only():
+            branch.set_append_revisions_only(True)
+            self.assertTrue(branch.get_append_revisions_only())
+            branch.set_append_revisions_only(False)
+            self.assertFalse(branch.get_append_revisions_only())
+        else:
+            self.assertRaises(errors.UpgradeRequired,
+                branch.set_append_revisions_only, True)
+            self.assertFalse(branch.get_append_revisions_only())
+
     def test_create_open_branch_uses_repository(self):
         try:
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise tests.TestNotApplicable("requires shared repository support")
         child_transport = repo.bzrdir.root_transport.clone('child')
         child_transport.mkdir('.')
-        child_dir = self.bzrdir_format.initialize_on_transport(child_transport)
+        try:
+            child_dir = self.bzrdir_format.initialize_on_transport(child_transport)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable("control dir format not initializable")
         try:
             child_branch = self.branch_format.initialize(child_dir)
         except errors.UninitializableFormat:
@@ -418,7 +383,7 @@
     def test_format_description(self):
         tree = self.make_branch_and_tree('tree')
         text = tree.branch._format.get_format_description()
-        self.failUnless(len(text))
+        self.assertTrue(len(text))
 
     def test_get_commit_builder(self):
         branch = self.make_branch(".")
@@ -432,16 +397,25 @@
         """Create a fake revision history easily."""
         tree = self.make_branch_and_tree('.')
         rev1 = tree.commit('foo')
-        orig_history = tree.branch.revision_history()
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        graph = tree.branch.repository.get_graph()
+        orig_history = list(
+            graph.iter_lefthand_ancestry(
+                tree.branch.last_revision(), [revision.NULL_REVISION]))
         rev2 = tree.commit('bar', allow_pointless=True)
         tree.branch.generate_revision_history(rev1)
-        self.assertEqual(orig_history, tree.branch.revision_history())
+        self.assertEqual(orig_history, list(
+            graph.iter_lefthand_ancestry(
+                tree.branch.last_revision(), [revision.NULL_REVISION])))
 
     def test_generate_revision_history_NULL_REVISION(self):
         tree = self.make_branch_and_tree('.')
         rev1 = tree.commit('foo')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
         tree.branch.generate_revision_history(revision.NULL_REVISION)
-        self.assertEqual([], tree.branch.revision_history())
+        self.assertEqual(revision.NULL_REVISION, tree.branch.last_revision())
 
     def test_create_checkout(self):
         tree_a = self.make_branch_and_tree('a')
@@ -468,7 +442,8 @@
         tree_a = self.make_branch_and_tree('a')
         rev_id = tree_a.commit('put some content in the branch')
         # open the branch via a readonly transport
-        source_branch = _mod_branch.Branch.open(self.get_readonly_url('a'))
+        source_branch = _mod_branch.Branch.open(self.get_readonly_url(
+            urlutils.basename(tree_a.branch.base)))
         # sanity check that the test will be valid
         self.assertRaises((errors.LockError, errors.TransportNotPossible),
             source_branch.lock_write)
@@ -480,7 +455,12 @@
         tree_a = self.make_branch_and_tree('a')
         rev_id = tree_a.commit('put some content in the branch')
         # open the branch via a readonly transport
-        source_branch = _mod_branch.Branch.open(self.get_readonly_url('a'))
+        url = self.get_readonly_url(
+            osutils.basename(tree_a.branch.base.rstrip('/')))
+        t = transport.get_transport_from_url(url)
+        if not tree_a.branch.bzrdir._format.supports_transport(t):
+            raise tests.TestNotApplicable("format does not support transport")
+        source_branch = _mod_branch.Branch.open(url)
         # sanity check that the test will be valid
         self.assertRaises((errors.LockError, errors.TransportNotPossible),
             source_branch.lock_write)
@@ -491,10 +471,32 @@
         tree = self.make_branch_and_tree('a')
         tree.commit('a commit', rev_id='rev1')
         br = tree.branch
-        br.set_revision_history(["rev1"])
-        self.assertEquals(br.revision_history(), ["rev1"])
-        br.set_revision_history([])
-        self.assertEquals(br.revision_history(), [])
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            br.set_revision_history, ["rev1"])
+        self.assertEquals(br.last_revision(), "rev1")
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            br.set_revision_history, [])
+        self.assertEquals(br.last_revision(), 'null:')
+
+    def test_heads_to_fetch(self):
+        # heads_to_fetch is a method that returns a collection of revids that
+        # need to be fetched to copy this branch into another repo.  At a
+        # minimum this will include the tip.
+        # (In native formats, this is the tip + tags, but other formats may
+        # have other revs needed)
+        tree = self.make_branch_and_tree('a')
+        tree.commit('first commit', rev_id='rev1')
+        tree.commit('second commit', rev_id='rev2')
+        must_fetch, should_fetch = tree.branch.heads_to_fetch()
+        self.assertTrue('rev2' in must_fetch)
+
+    def test_heads_to_fetch_not_null_revision(self):
+        # NULL_REVISION does not appear in the result of heads_to_fetch, even
+        # for an empty branch.
+        tree = self.make_branch_and_tree('a')
+        must_fetch, should_fetch = tree.branch.heads_to_fetch()
+        self.assertFalse(revision.NULL_REVISION in must_fetch)
+        self.assertFalse(revision.NULL_REVISION in should_fetch)
 
 
 class TestBranchFormat(per_branch.TestCaseWithBranch):
@@ -518,6 +520,12 @@
             looked_up_format = registry.get(network_name)
             self.assertEqual(format.__class__, looked_up_format.__class__)
 
+    def get_get_config_calls(self):
+        # Smoke test that all branch succeed getting a config
+        br = self.make_branch('.')
+        br.get_config()
+        br.get_config_stack()
+
 
 class ChrootedTests(per_branch.TestCaseWithBranch):
     """A support class that provides readonly urls outside the local namespace.
@@ -540,6 +548,9 @@
                           _mod_branch.Branch.open_containing,
                           self.get_readonly_url('g/p/q'))
         branch = self.make_branch('.')
+        if not branch.bzrdir._format.supports_transport(
+            transport.get_transport_from_url(self.get_readonly_url('.'))):
+            raise tests.TestNotApplicable("format does not support transport")
         branch, relpath = _mod_branch.Branch.open_containing(
             self.get_readonly_url(''))
         self.assertEqual('', relpath)
@@ -617,13 +628,9 @@
         self.assertEqual(None, self.get_branch().get_push_location())
 
     def test_get_push_location_exact(self):
-        from bzrlib.config import (locations_config_filename,
-                                   ensure_config_dir_exists)
-        ensure_config_dir_exists()
-        fn = locations_config_filename()
-        open(fn, 'wt').write(("[%s]\n"
-                                  "push_location=foo\n" %
-                                  self.get_branch().base[:-1]))
+        b = self.get_branch()
+        config.LocationConfig.from_string(
+            '[%s]\npush_location=foo\n' % (b.base,), b.base, save=True)
         self.assertEqual("foo", self.get_branch().get_push_location())
 
     def test_set_push_location(self):
@@ -635,7 +642,9 @@
 class TestChildSubmitFormats(per_branch.TestCaseWithBranch):
 
     def test_get_child_submit_format_default(self):
-        self.assertEqual(None, self.get_branch().get_child_submit_format())
+        submit_format = self.get_branch().get_child_submit_format()
+        self.assertTrue(submit_format is None or
+                        isinstance(submit_format, str))
 
     def test_get_child_submit_format(self):
         branch = self.get_branch()
@@ -685,22 +694,22 @@
             # they may not be initializable.
             return
         # supported formats must be able to init and open
-        t = transport.get_transport(self.get_url())
-        readonly_t = transport.get_transport(self.get_readonly_url())
+        t = self.get_transport()
+        readonly_t = transport.get_transport_from_url(self.get_readonly_url())
         made_branch = self.make_branch('.')
-        self.failUnless(isinstance(made_branch, _mod_branch.Branch))
+        self.assertIsInstance(made_branch, _mod_branch.Branch)
 
         # find it via bzrdir opening:
         opened_control = bzrdir.BzrDir.open(readonly_t.base)
         direct_opened_branch = opened_control.open_branch()
         self.assertEqual(direct_opened_branch.__class__, made_branch.__class__)
         self.assertEqual(opened_control, direct_opened_branch.bzrdir)
-        self.failUnless(isinstance(direct_opened_branch._format,
-                        self.branch_format.__class__))
+        self.assertIsInstance(direct_opened_branch._format,
+            self.branch_format.__class__)
 
         # find it via Branch.open
         opened_branch = _mod_branch.Branch.open(readonly_t.base)
-        self.failUnless(isinstance(opened_branch, made_branch.__class__))
+        self.assertIsInstance(opened_branch, made_branch.__class__)
         self.assertEqual(made_branch._format.__class__,
                          opened_branch._format.__class__)
         # if it has a unique id string, can we probe for it ?
@@ -749,6 +758,66 @@
         except errors.UpgradeRequired:
             raise tests.TestNotApplicable('Format does not support binding')
 
+    def test_unbind_clears_cached_master_branch(self):
+        """b.unbind clears any cached value of b.get_master_branch."""
+        master = self.make_branch('master')
+        branch = self.make_branch('branch')
+        try:
+            branch.bind(master)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable('Format does not support binding')
+        self.addCleanup(branch.lock_write().unlock)
+        self.assertNotEqual(None, branch.get_master_branch())
+        branch.unbind()
+        self.assertEqual(None, branch.get_master_branch())
+
+    def test_unlocked_does_not_cache_master_branch(self):
+        """Unlocked branches do not cache the result of get_master_branch."""
+        master = self.make_branch('master')
+        branch1 = self.make_branch('branch')
+        try:
+            branch1.bind(master)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable('Format does not support binding')
+        # Open branch1 again
+        branch2 = branch1.bzrdir.open_branch()
+        self.assertNotEqual(None, branch1.get_master_branch())
+        # Unbind the branch via branch2.  branch1 isn't locked so will
+        # immediately return the new value for get_master_branch.
+        branch2.unbind()
+        self.assertEqual(None, branch1.get_master_branch())
+
+    def test_bind_clears_cached_master_branch(self):
+        """b.bind clears any cached value of b.get_master_branch."""
+        master1 = self.make_branch('master1')
+        master2 = self.make_branch('master2')
+        branch = self.make_branch('branch')
+        try:
+            branch.bind(master1)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable('Format does not support binding')
+        self.addCleanup(branch.lock_write().unlock)
+        self.assertNotEqual(None, branch.get_master_branch())
+        branch.bind(master2)
+        self.assertEqual('.', urlutils.relative_url(self.get_url('master2'),
+                branch.get_master_branch().base))
+
+    def test_set_bound_location_clears_cached_master_branch(self):
+        """b.set_bound_location clears any cached value of b.get_master_branch.
+        """
+        master1 = self.make_branch('master1')
+        master2 = self.make_branch('master2')
+        branch = self.make_branch('branch')
+        try:
+            branch.bind(master1)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable('Format does not support binding')
+        self.addCleanup(branch.lock_write().unlock)
+        self.assertNotEqual(None, branch.get_master_branch())
+        branch.set_bound_location(self.get_url('master2'))
+        self.assertEqual('.', urlutils.relative_url(self.get_url('master2'),
+                branch.get_master_branch().base))
+
 
 class TestStrict(per_branch.TestCaseWithBranch):
 
@@ -788,12 +857,14 @@
     def test_fallbacks_not_opened(self):
         stacked = self.make_branch_with_fallback()
         self.get_transport('').rename('fallback', 'moved')
-        reopened = stacked.bzrdir.open_branch(ignore_fallbacks=True)
+        reopened_dir = bzrdir.BzrDir.open(stacked.base)
+        reopened = reopened_dir.open_branch(ignore_fallbacks=True)
         self.assertEqual([], reopened.repository._fallback_repositories)
 
     def test_fallbacks_are_opened(self):
         stacked = self.make_branch_with_fallback()
-        reopened = stacked.bzrdir.open_branch(ignore_fallbacks=False)
+        reopened_dir = bzrdir.BzrDir.open(stacked.base)
+        reopened = reopened_dir.open_branch(ignore_fallbacks=False)
         self.assertLength(1, reopened.repository._fallback_repositories)
 
 
@@ -807,8 +878,9 @@
             tree.add_reference(subtree)
         except errors.UnsupportedOperation:
             raise tests.TestNotApplicable('Tree cannot hold references.')
-        reference_parent = tree.branch.reference_parent('subtree-id',
-                                                        'subtree')
+        reference_parent = tree.branch.reference_parent(
+            'subtree-id',
+            urlutils.relative_url(tree.branch.user_url, subtree.branch.user_url))
         self.assertEqual(subtree.branch.base, reference_parent.base)
 
     def test_reference_parent_accepts_possible_transports(self):
@@ -820,7 +892,9 @@
         except errors.UnsupportedOperation:
             raise tests.TestNotApplicable('Tree cannot hold references.')
         reference_parent = tree.branch.reference_parent('subtree-id',
-            'subtree', possible_transports=[subtree.bzrdir.root_transport])
+            urlutils.relative_url(
+                tree.branch.user_url, subtree.branch.user_url),
+            possible_transports=[subtree.bzrdir.root_transport])
 
     def test_get_reference_info(self):
         branch = self.make_branch('branch')
@@ -993,7 +1067,7 @@
 
 class TestBranchControlComponent(per_branch.TestCaseWithBranch):
     """Branch implementations adequately implement ControlComponent."""
-    
+
     def test_urls(self):
         br = self.make_branch('branch')
         self.assertIsInstance(br.user_url, str)

=== modified file 'bzrlib/tests/per_branch/test_break_lock.py'
--- bzrlib/tests/per_branch/test_break_lock.py	2010-05-06 23:41:35 +0000
+++ bzrlib/tests/per_branch/test_break_lock.py	2011-09-30 13:35:45 +0000
@@ -16,7 +16,8 @@
 
 """Tests for branch break-lock behaviour."""
 
-from  bzrlib import (
+from bzrlib import (
+    branch as _mod_branch,
     errors,
     ui,
     tests,
@@ -29,7 +30,7 @@
     def setUp(self):
         super(TestBreakLock, self).setUp()
         self.unused_branch = self.make_branch('branch')
-        self.branch = self.unused_branch.bzrdir.open_branch()
+        self.branch = _mod_branch.Branch.open(self.unused_branch.base)
 
     def test_unlocked(self):
         # break lock when nothing is locked should just return

=== modified file 'bzrlib/tests/per_branch/test_check.py'
--- bzrlib/tests/per_branch/test_check.py	2009-08-04 04:36:34 +0000
+++ bzrlib/tests/per_branch/test_check.py	2011-04-19 14:17:24 +0000
@@ -19,6 +19,7 @@
 from StringIO import StringIO
 
 from bzrlib import errors, tests, ui
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
 
@@ -44,7 +45,8 @@
         r5 = tree.commit('five')
         # Now, try to set an invalid history
         try:
-            tree.branch.set_revision_history([r1, r2b, r5])
+            self.applyDeprecated(deprecated_in((2, 4, 0)),
+                tree.branch.set_revision_history, [r1, r2b, r5])
             if tree.branch.last_revision_info() != (3, r5):
                 # RemoteBranch silently corrects an impossible revision
                 # history given to set_revision_history.  It can be tricked

=== modified file 'bzrlib/tests/per_branch/test_commit.py'
--- bzrlib/tests/per_branch/test_commit.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_commit.py	2011-01-27 14:27:18 +0000
@@ -30,7 +30,7 @@
 
     def test_commit_nicks(self):
         """Nicknames are committed to the revision"""
-        transport.get_transport(self.get_url()).mkdir('bzr.dev')
+        self.get_transport().mkdir('bzr.dev')
         wt = self.make_branch_and_tree('bzr.dev')
         branch = wt.branch
         branch.nick = "My happy branch"

=== modified file 'bzrlib/tests/per_branch/test_create_checkout.py'
--- bzrlib/tests/per_branch/test_create_checkout.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_create_checkout.py	2011-09-09 12:20:33 +0000
@@ -16,22 +16,29 @@
 
 """Tests for the Branch.create_checkout"""
 
-from bzrlib import (
-    branch,
-    )
 from bzrlib.tests import per_branch
 
 
 class TestCreateCheckout(per_branch.TestCaseWithBranch):
 
-    def test_checkout_format(self):
-        """Make sure the new checkout uses the desired branch format."""
-        a_branch = self.make_branch('branch')
-        tree = a_branch.create_checkout('checkout')
-        # All branches can define the format they want checkouts made in.
-        # This checks it is honoured.
-        expected_format = a_branch._get_checkout_format().get_branch_format()
-        self.assertEqual(expected_format.__class__,
+    def test_checkout_format_lightweight(self):
+        """Make sure the new light checkout uses the desired branch format."""
+        a_branch = self.make_branch('branch')
+        tree = a_branch.create_checkout('checkout', lightweight=True)
+        # All branches can define the format they want checkouts made in.
+        # This checks it is honoured.
+        expected_format = a_branch._get_checkout_format(lightweight=True)
+        self.assertEqual(expected_format.get_branch_format().__class__,
+                         tree.branch._format.__class__)
+
+    def test_checkout_format_heavyweight(self):
+        """Make sure the new heavy checkout uses the desired branch format."""
+        a_branch = self.make_branch('branch')
+        tree = a_branch.create_checkout('checkout', lightweight=False)
+        # All branches can define the format they want checkouts made in.
+        # This checks it is honoured.
+        expected_format = a_branch._get_checkout_format(lightweight=False)
+        self.assertEqual(expected_format.get_branch_format().__class__,
                          tree.branch._format.__class__)
 
     def test_create_revision_checkout(self):
@@ -46,8 +53,8 @@
 
         tree2 = tree1.branch.create_checkout('checkout', revision_id='rev-1')
         self.assertEqual('rev-1', tree2.last_revision())
-        self.failUnlessExists('checkout/a')
-        self.failIfExists('checkout/b')
+        self.assertPathExists('checkout/a')
+        self.assertPathDoesNotExist('checkout/b')
 
     def test_create_lightweight_checkout(self):
         """We should be able to make a lightweight checkout."""

=== modified file 'bzrlib/tests/per_branch/test_create_clone.py'
--- bzrlib/tests/per_branch/test_create_clone.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_create_clone.py	2011-10-04 13:29:25 +0000
@@ -96,8 +96,13 @@
         revid = tree.commit('a second commit')
         source = tree.branch
         target_transport = self.get_transport('target')
-        result = tree.branch.create_clone_on_transport(target_transport,
-            stacked_on=trunk.base)
+        try:
+            result = tree.branch.create_clone_on_transport(target_transport,
+                stacked_on=trunk.base)
+        except errors.UnstackableBranchFormat:
+            if not trunk.repository._format.supports_full_versioned_files:
+                raise tests.TestNotApplicable("can not stack on format")
+            raise
         self.assertEqual(revid, result.last_revision())
         self.assertEqual(trunk.base, result.get_stacked_on_url())
 
@@ -133,8 +138,13 @@
         self.hook_calls = []
         branch.Branch.hooks.install_named_hook(
             'pre_change_branch_tip', self.assertBranchHookBranchIsStacked, None)
-        result = tree.branch.create_clone_on_transport(target_transport,
-            stacked_on=trunk.base)
+        try:
+            result = tree.branch.create_clone_on_transport(target_transport,
+                stacked_on=trunk.base)
+        except errors.UnstackableBranchFormat:
+            if not trunk.repository._format.supports_full_versioned_files:
+                raise tests.TestNotApplicable("can not stack on format")
+            raise
         self.assertEqual(revid, result.last_revision())
         self.assertEqual(trunk.base, result.get_stacked_on_url())
         # Smart servers invoke hooks on both sides

=== modified file 'bzrlib/tests/per_branch/test_get_revision_id_to_revno_map.py'
--- bzrlib/tests/per_branch/test_get_revision_id_to_revno_map.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_branch/test_get_revision_id_to_revno_map.py	2011-04-19 14:17:24 +0000
@@ -16,11 +16,7 @@
 
 """Tests for Branch.get_revision_id_to_revno_map()"""
 
-from bzrlib import (
-    errors,
-    revision,
-    )
-
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
 
@@ -93,7 +89,8 @@
             self.assertEqual({'rev-1':(1,), 'rev-2':(2,), 'rev-3':(3,),
                               'rev-1.1.1':(1,1,1)
                              }, branch.get_revision_id_to_revno_map())
-            branch.set_revision_history(['rev-1', 'rev-2'])
+            self.applyDeprecated(deprecated_in((2, 4, 0)),
+                branch.set_revision_history, ['rev-1', 'rev-2'])
             self.assertEqual({'rev-1':(1,), 'rev-2':(2,)},
                              branch.get_revision_id_to_revno_map())
             self.assertEqual({'rev-1':(1,), 'rev-2':(2,)},

=== modified file 'bzrlib/tests/per_branch/test_hooks.py'
--- bzrlib/tests/per_branch/test_hooks.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_branch/test_hooks.py	2011-05-08 13:46:49 +0000
@@ -23,6 +23,7 @@
     revision,
     tests,
     )
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests import test_server
 
 class ChangeBranchTipTestCase(tests.TestCaseWithMemoryTransport):
@@ -87,7 +88,8 @@
         branch = self.make_branch('source')
         _mod_branch.Branch.hooks.install_named_hook(
             'set_rh', self.capture_set_rh_hook, None)
-        branch.set_revision_history([])
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch.set_revision_history, [])
         expected_params = ('set_rh', branch, [], True)
         self.assertHookCalls(expected_params, branch)
 
@@ -103,7 +105,8 @@
             'set_rh', self.capture_set_rh_hook, None)
         # some branches require that their history be set to a revision in the
         # repository
-        branch.set_revision_history(['f\xc2\xb5'])
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch.set_revision_history, ['f\xc2\xb5'])
         expected_params =('set_rh', branch, ['f\xc2\xb5'], True)
         self.assertHookCalls(expected_params, branch)
 
@@ -111,7 +114,8 @@
         branch = self.make_branch('source')
         _mod_branch.Branch.hooks.install_named_hook(
             'set_rh', self.capture_set_rh_hook, None)
-        branch.set_revision_history([])
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch.set_revision_history, [])
         expected_params = ('set_rh', branch, [], True)
         self.assertHookCalls(expected_params, branch)
 
@@ -121,7 +125,8 @@
             'set_rh', self.capture_set_rh_hook, None)
         _mod_branch.Branch.hooks.install_named_hook(
             'set_rh', self.capture_set_rh_hook, None)
-        branch.set_revision_history([])
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch.set_revision_history, [])
         expected_calls = [('set_rh', branch, [], True),
             ('set_rh', branch, [], True),
             ]
@@ -386,7 +391,8 @@
 
     def test_set_revision_history(self):
         branch = self.make_branch('')
-        branch.set_revision_history([])
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch.set_revision_history, [])
         self.assertPreAndPostHooksWereInvoked(branch, True)
 
     def test_set_last_revision_info(self):

=== modified file 'bzrlib/tests/per_branch/test_last_revision_info.py'
--- bzrlib/tests/per_branch/test_last_revision_info.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_branch/test_last_revision_info.py	2011-04-12 00:44:43 +0000
@@ -17,6 +17,7 @@
 """Tests for branch.last_revision_info."""
 
 from bzrlib.revision import NULL_REVISION
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -34,21 +35,55 @@
         revid = tree.commit('2st post', allow_pointless=True)
         self.assertEqual((2, revid), tree.branch.last_revision_info())
 
+    def test_import_deprecated(self):
+        # importing and setting last revision
+        tree1 = self.make_branch_and_tree('branch1')
+        tree1.commit('1st post')
+        revid = tree1.commit('2st post', allow_pointless=True)
+        branch2 = self.make_branch('branch2')
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch2.import_last_revision_info, tree1.branch.repository, 2, revid)
+        self.assertEqual((2, revid), branch2.last_revision_info())
+        self.assertTrue(branch2.repository.has_revision(revid))
+
+    def test_same_repo_deprecated(self):
+        # importing and setting last revision within the same repo
+        tree = self.make_branch_and_tree('branch1')
+        tree.commit('1st post')
+        revid = tree.commit('2st post', allow_pointless=True)
+        tree.branch.set_last_revision_info(0, NULL_REVISION)
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            tree.branch.import_last_revision_info, tree.branch.repository, 2, revid)
+        self.assertEqual((2, revid), tree.branch.last_revision_info())
+
     def test_import(self):
         # importing and setting last revision
         tree1 = self.make_branch_and_tree('branch1')
         tree1.commit('1st post')
         revid = tree1.commit('2st post', allow_pointless=True)
         branch2 = self.make_branch('branch2')
-        branch2.import_last_revision_info(tree1.branch.repository, 2, revid)
+        self.assertEquals((2, revid),
+            branch2.import_last_revision_info_and_tags(tree1.branch, 2, revid))
         self.assertEqual((2, revid), branch2.last_revision_info())
         self.assertTrue(branch2.repository.has_revision(revid))
 
+    def test_import_lossy(self):
+        # importing with lossy=True works
+        tree1 = self.make_branch_and_tree('branch1')
+        tree1.commit('1st post')
+        revid = tree1.commit('2st post', allow_pointless=True)
+        branch2 = self.make_branch('branch2')
+        ret = branch2.import_last_revision_info_and_tags(tree1.branch, 2,
+            revid, lossy=True)
+        self.assertIsInstance(ret, tuple)
+        self.assertIsInstance(ret[0], int)
+        self.assertIsInstance(ret[1], str)
+
     def test_same_repo(self):
         # importing and setting last revision within the same repo
         tree = self.make_branch_and_tree('branch1')
         tree.commit('1st post')
         revid = tree.commit('2st post', allow_pointless=True)
         tree.branch.set_last_revision_info(0, NULL_REVISION)
-        tree.branch.import_last_revision_info(tree.branch.repository, 2, revid)
+        tree.branch.import_last_revision_info_and_tags(tree.branch, 2, revid)
         self.assertEqual((2, revid), tree.branch.last_revision_info())

=== modified file 'bzrlib/tests/per_branch/test_locking.py'
--- bzrlib/tests/per_branch/test_locking.py	2010-06-11 07:59:43 +0000
+++ bzrlib/tests/per_branch/test_locking.py	2011-09-19 18:25:53 +0000
@@ -17,7 +17,6 @@
 """Test locks across all branch implemenations"""
 
 from bzrlib import (
-    branch as _mod_branch,
     errors,
     tests,
     )
@@ -43,13 +42,13 @@
         self.locks = []
         b = lock_helpers.LockWrapper(self.locks, self.get_branch(), 'b')
         b.repository = lock_helpers.LockWrapper(self.locks, b.repository, 'r')
-        bcf = b.control_files
+        bcf = getattr(b, "control_files", None)
         rcf = getattr(b.repository, 'control_files', None)
         if rcf is None:
             self.combined_branch = False
         else:
             # Look out for branch types that reuse their control files
-            self.combined_control = bcf is rcf
+            self.combined_control = bcf is rcf and bcf is not None
         try:
             b.control_files = lock_helpers.LockWrapper(
                 self.locks, b.control_files, 'bc')
@@ -501,10 +500,6 @@
         self.assertThat(branch.lock_read, ReturnsUnlockable(branch))
 
     def test_lock_write_locks_repo_too(self):
-        if isinstance(self.branch_format, _mod_branch.BzrBranchFormat4):
-            # Branch format 4 is combined with the repository, so this test
-            # doesn't apply.
-            return
         branch = self.make_branch('b')
         branch = branch.bzrdir.open_branch()
         branch.lock_write()

=== modified file 'bzrlib/tests/per_branch/test_parent.py'
--- bzrlib/tests/per_branch/test_parent.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_branch/test_parent.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
     def test_set_get_parent(self):
         """Set, re-get and reset the parent"""
         b = self.make_branch('subdir')
-        url = 'http://bazaar-vcs.org/bzr/bzr.dev'
+        url = 'http://example.com/bzr/bzr.dev'
         b.set_parent(url)
         self.assertEqual(url, b.get_parent())
         self.assertEqual(url, b._get_parent_location())

=== modified file 'bzrlib/tests/per_branch/test_permissions.py'
--- bzrlib/tests/per_branch/test_permissions.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_branch/test_permissions.py	2011-09-12 09:51:52 +0000
@@ -29,16 +29,12 @@
 import os
 import sys
 import stat
-from StringIO import StringIO
 
 from bzrlib import tests
-from bzrlib.branch import Branch
+from bzrlib.branch import BzrBranch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.lockable_files import LockableFiles
 from bzrlib.remote import RemoteBranchFormat
-from bzrlib.tests.test_permissions import chmod_r, check_mode_r
-from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.workingtree import WorkingTree
+from bzrlib.tests.test_permissions import check_mode_r
 
 
 class _NullPermsStat(object):
@@ -63,10 +59,13 @@
                                           ' permission logic')
         if sys.platform == 'win32':
             raise tests.TestNotApplicable('chmod has no effect on win32')
-        # also, these are BzrBranch format specific things..
         os.mkdir('a')
         mode = stat.S_IMODE(os.stat('a').st_mode)
         t = self.make_branch_and_tree('.')
+        # also, these are BzrBranch format specific things..
+        if not isinstance(t.branch, BzrBranch):
+            raise tests.TestNotApplicable(
+                "Only applicable to bzr branches")
         b = t.branch
         self.assertEqualMode(mode, b.bzrdir._get_dir_mode())
         self.assertEqualMode(mode & ~07111, b.bzrdir._get_file_mode())
@@ -90,14 +89,17 @@
                                           ' permission logic')
         if sys.platform == 'win32':
             raise tests.TestNotApplicable('chmod has no effect on win32')
-        elif sys.platform == 'darwin' or sys.platform.startswith('freebsd'):
-            # OS X (and FreeBSD) create temp dirs with the 'wheel' group, which
-            # users are not likely to be in, and this prevents us from setting
-            # the sgid bit
+        elif sys.platform == 'darwin' or 'freebsd' in sys.platform:
+            # FreeBSD-based platforms create temp dirs with the 'wheel' group,
+            # which users are not likely to be in, and this prevents us
+            # from setting the sgid bit
             os.chown(self.test_dir, os.getuid(), os.getgid())
-        # also, these are BzrBranch format specific things..
         t = self.make_branch_and_tree('.')
         b = t.branch
+        # also, these are BzrBranch format specific things..
+        if not isinstance(b, BzrBranch):
+            raise tests.TestNotApplicable(
+                "Only applicable to bzr branches")
         os.mkdir('b')
         os.chmod('b', 02777)
         b = self.make_branch('b')

=== modified file 'bzrlib/tests/per_branch/test_pull.py'
--- bzrlib/tests/per_branch/test_pull.py	2010-06-14 06:37:11 +0000
+++ bzrlib/tests/per_branch/test_pull.py	2011-09-26 15:21:01 +0000
@@ -16,8 +16,6 @@
 
 """Tests for branch.pull behaviour."""
 
-import os
-
 from bzrlib import (
     branch,
     bzrdir,
@@ -25,7 +23,11 @@
     memorytree,
     revision,
     )
-from bzrlib.tests import per_branch
+from bzrlib.tests import (
+    fixtures,
+    per_branch,
+    TestNotApplicable,
+    )
 
 
 class TestPull(per_branch.TestCaseWithBranch):
@@ -40,7 +42,7 @@
         parent.merge_from_branch(mine.branch)
         parent.commit('merge my change', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_merged_indirect(self):
         # it should be possible to do a pull from one branch into another
@@ -57,7 +59,7 @@
         parent.merge_from_branch(other.branch)
         parent.commit('merge other', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_updates_checkout_and_master(self):
         """Pulling into a checkout updates the checkout and the master branch"""
@@ -69,8 +71,8 @@
         rev2 = other.commit('other commit')
         # now pull, which should update both checkout and master.
         checkout.branch.pull(other.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_pull_local_updates_checkout_only(self):
         """Pulling --local into a checkout updates the checkout and not the
@@ -83,8 +85,8 @@
         rev2 = other.commit('other commit')
         # now pull local, which should update checkout but not master.
         checkout.branch.pull(other.branch, local = True)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev1, master_tree.branch.last_revision())
 
     def test_pull_local_raises_LocalRequiresBoundBranch_on_unbound(self):
         """Pulling --local into a branch that is not bound should fail."""
@@ -96,7 +98,7 @@
         # now pull --local, which should raise LocalRequiresBoundBranch error.
         self.assertRaises(errors.LocalRequiresBoundBranch,
                           master_tree.branch.pull, other.branch, local = True)
-        self.assertEqual([rev1], master_tree.branch.revision_history())
+        self.assertEqual(rev1, master_tree.branch.last_revision())
 
     def test_pull_returns_result(self):
         parent = self.make_branch_and_tree('parent')
@@ -113,7 +115,7 @@
         self.assertEqual('P1', result.old_revid)
         self.assertEqual(2, result.new_revno)
         self.assertEqual('M1', result.new_revid)
-        self.assertEqual(None, result.tag_conflicts)
+        self.assertEqual([], result.tag_conflicts)
 
     def test_pull_overwrite(self):
         tree_a = self.make_branch_and_tree('tree_a')
@@ -132,8 +134,53 @@
         tree_a.branch.pull(tree_b.branch, overwrite=True,
                            stop_revision='rev2b')
         self.assertEqual('rev2b', tree_a.branch.last_revision())
-        self.assertEqual(tree_b.branch.revision_history(),
-                         tree_a.branch.revision_history())
+        self.assertEqual(tree_b.branch.last_revision(),
+                         tree_a.branch.last_revision())
+
+    def test_pull_merges_and_fetches_tags(self):
+        """Tags are updated by br.pull(source), and revisions named in those
+        tags are fetched.
+        """
+        # Make a source, sprout a target off it
+        try:
+            builder = self.make_branch_builder('source')
+        except errors.UninitializableFormat:
+            raise TestNotApplicable('uninitializeable format')
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        target = source.bzrdir.sprout('target').open_branch()
+        # Add a tag to the source, then pull from source
+        try:
+            source.tags.set_tag('tag-a', 'rev-2')
+        except errors.TagsNotSupported:
+            raise TestNotApplicable('format does not support tags.')
+        source.tags.set_tag('tag-a', 'rev-2')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        target.pull(source)
+        # The tag is present, and so is its revision.
+        self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))
+        target.repository.get_revision('rev-2')
+
+    def test_pull_stop_revision_merges_and_fetches_tags(self):
+        """br.pull(source, stop_revision=REV) updates and fetches tags."""
+        # Make a source, sprout a target off it
+        try:
+            builder = self.make_branch_builder('source')
+        except errors.UninitializableFormat:
+            raise TestNotApplicable('uninitializeable format')
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        target = source.bzrdir.sprout('target').open_branch()
+        # Add a new commit to the ancestry
+        builder.build_commit(message="Rev 2 again", rev_id='rev-2-again')
+        # Add a tag to the source, then pull rev-2-again from source
+        try:
+            source.tags.set_tag('tag-a', 'rev-2')
+        except errors.TagsNotSupported:
+            raise TestNotApplicable('format does not support tags.')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        target.pull(source, 'rev-2-again')
+        # The tag is present, and so is its revision.
+        self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))
+        target.repository.get_revision('rev-2')
 
 
 class TestPullHook(per_branch.TestCaseWithBranch):

=== modified file 'bzrlib/tests/per_branch/test_push.py'
--- bzrlib/tests/per_branch/test_push.py	2010-07-04 22:39:45 +0000
+++ bzrlib/tests/per_branch/test_push.py	2011-09-26 15:21:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,19 +24,16 @@
     builtins,
     bzrdir,
     check,
-    debug,
     errors,
     memorytree,
     push,
-    repository,
     revision,
+    symbol_versioning,
     tests,
     transport,
     )
 from bzrlib.smart import (
     client,
-    server,
-    repository as _mod_smart_repo,
     )
 from bzrlib.tests import (
     per_branch,
@@ -56,12 +53,15 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge my change', rev_id='P2')
         result = mine.branch.push(other.branch)
-        self.assertEqual(['P1', 'P2'], other.branch.revision_history())
+        self.assertEqual('P2', other.branch.last_revision())
         # result object contains some structured data
         self.assertEqual(result.old_revid, 'M1')
         self.assertEqual(result.new_revid, 'P2')
         # and it can be treated as an integer for compatibility
-        self.assertEqual(int(result), 0)
+        self.assertEqual(self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            result.__int__),
+            0)
 
     def test_push_merged_indirect(self):
         # it should be possible to do a push from one branch into another
@@ -78,7 +78,7 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge other', rev_id='P2')
         mine.branch.push(target.branch)
-        self.assertEqual(['P1', 'P2'], target.branch.revision_history())
+        self.assertEqual('P2', target.branch.last_revision())
 
     def test_push_to_checkout_updates_master(self):
         """Pushing into a checkout updates the checkout and the master branch"""
@@ -95,8 +95,8 @@
         rev2 = other.commit('other commit')
         # now push, which should update both checkout and master.
         other.branch.push(checkout.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_push_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_branch_and_tree('master')
@@ -114,6 +114,23 @@
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 other.branch.push, checkout.branch)
 
+    def test_push_new_tag_to_bound_branch(self):
+        master = self.make_branch('master')
+        bound = self.make_branch('bound')
+        try:
+            bound.bind(master)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable(
+                'Format does not support bound branches')
+        other = bound.bzrdir.sprout('other').open_branch()
+        try:
+            other.tags.set_tag('new-tag', 'some-rev')
+        except errors.TagsNotSupported:
+            raise tests.TestNotApplicable('Format does not support tags')
+        other.push(bound)
+        self.assertEqual({'new-tag': 'some-rev'}, bound.tags.get_tag_dict())
+        self.assertEqual({'new-tag': 'some-rev'}, master.tags.get_tag_dict())
+
     def test_push_uses_read_lock(self):
         """Push should only need a read lock on the source side."""
         source = self.make_branch_and_tree('source')
@@ -140,6 +157,8 @@
         except (errors.IncompatibleFormat, errors.UninitializableFormat):
             # This Branch format cannot create shared repositories
             return
+        if not repo._format.supports_nesting_repositories:
+            return
         # This is a little bit trickier because make_branch_and_tree will not
         # re-use a shared repository.
         a_bzrdir = self.make_bzrdir('repo/tree')
@@ -170,6 +189,21 @@
         self.assertEqual(tree.branch.last_revision(),
                          to_branch.last_revision())
 
+    def test_push_overwrite_with_older_mainline_rev(self):
+        """Pushing an older mainline revision with overwrite.
+
+        This was <https://bugs.launchpad.net/bzr/+bug/386576>.
+        """
+        source = self.make_branch_and_tree('source')
+        target = self.make_branch('target')
+
+        source.commit('1st commit')
+        source.commit('2nd commit', rev_id='rev-2')
+        source.commit('3rd commit')
+        source.branch.push(target)
+        source.branch.push(target, stop_revision='rev-2', overwrite=True)
+        self.assertEqual('rev-2', target.last_revision())
+
     def test_push_overwrite_of_non_tip_with_stop_revision(self):
         """Combining the stop_revision and overwrite options works.
 
@@ -186,6 +220,41 @@
         source.branch.push(target, stop_revision='rev-2', overwrite=True)
         self.assertEqual('rev-2', target.last_revision())
 
+    def test_push_repository_no_branch_doesnt_fetch_all_revs(self):
+        # See https://bugs.launchpad.net/bzr/+bug/465517
+        t = self.get_transport('target')
+        t.ensure_base()
+        bzrdir = self.bzrdir_format.initialize_on_transport(t)
+        try:
+            bzrdir.open_branch()
+        except errors.NotBranchError:
+            pass
+        else:
+            raise tests.TestNotApplicable('older formats can\'t have a repo'
+                                          ' without a branch')
+        try:
+            source = self.make_branch_builder('source',
+                                              format=self.bzrdir_format)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable('cannot initialize this format')
+        source.start_series()
+        source.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        source.build_snapshot('B', ['A'], [])
+        source.build_snapshot('C', ['A'], [])
+        source.finish_series()
+        b = source.get_branch()
+        # Note: We can't read lock the source branch. Some formats take a write
+        # lock to 'set_push_location', which breaks
+        self.addCleanup(b.lock_write().unlock)
+        repo = bzrdir.create_repository()
+        # This means 'push the source branch into this dir'
+        bzrdir.push_branch(b)
+        self.addCleanup(repo.lock_read().unlock)
+        # We should have pushed 'C', but not 'B', since it isn't in the
+        # ancestry
+        self.assertEqual(['A', 'C'], sorted(repo.all_revision_ids()))
+
     def test_push_with_default_stacking_does_not_create_broken_branch(self):
         """Pushing a new standalone branch works even when there's a default
         stacking policy at the destination.
@@ -194,7 +263,7 @@
         default for the branch), and will be stacked when the repo format
         allows (which means that the branch format isn't necessarly preserved).
         """
-        if isinstance(self.branch_format, branch.BzrBranchFormat4):
+        if self.bzrdir_format.fixed_components:
             raise tests.TestNotApplicable('Not a metadir format.')
         if isinstance(self.branch_format, branch.BranchReferenceFormat):
             # This test could in principle apply to BranchReferenceFormat, but
@@ -329,14 +398,15 @@
 
     def setUp(self):
         # Skip some scenarios that don't apply to these tests.
-        if (self.transport_server is not None and
-            issubclass(self.transport_server, server.SmartTCPServer)):
+        if (self.transport_server is not None
+            and issubclass(self.transport_server,
+                           test_server.SmartTCPServer_for_testing)):
             raise tests.TestNotApplicable(
                 'Does not apply when remote backing branch is also '
                 'a smart branch')
-        if isinstance(self.branch_format, branch.BzrBranchFormat4):
+        if not self.branch_format.supports_leaving_lock():
             raise tests.TestNotApplicable(
-                'Branch format 4 is not usable via HPSS.')
+                'Branch format is not usable via HPSS.')
         super(EmptyPushSmartEffortTests, self).setUp()
         # Create a smart server that publishes whatever the backing VFS server
         # does.
@@ -356,7 +426,7 @@
     def test_empty_branch_api(self):
         """The branch_obj.push API should make a limited number of HPSS calls.
         """
-        t = transport.get_transport(self.smart_server.get_url()).clone('target')
+        t = transport.get_transport_from_url(self.smart_server.get_url()).clone('target')
         target = branch.Branch.open_from_transport(t)
         self.empty_branch.push(target)
         self.assertEqual(
@@ -393,4 +463,4 @@
     def test_lossy_push_raises_same_vcs(self):
         target = self.make_branch('target')
         source = self.make_branch('source')
-        self.assertRaises(errors.LossyPushToSameVCS, source.lossy_push, target)
+        self.assertRaises(errors.LossyPushToSameVCS, source.push, target, lossy=True)

=== modified file 'bzrlib/tests/per_branch/test_reconcile.py'
--- bzrlib/tests/per_branch/test_reconcile.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_branch/test_reconcile.py	2011-10-05 12:45:41 +0000
@@ -17,7 +17,9 @@
 """Tests for branch implementations - test reconcile() functionality"""
 
 from bzrlib import errors, reconcile
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests.per_branch import TestCaseWithBranch
+from bzrlib.tests import TestNotApplicable
 
 
 class TestBranchReconcile(TestCaseWithBranch):
@@ -42,7 +44,8 @@
         r5 = tree.commit('five')
         # Now, try to set an invalid history
         try:
-            tree.branch.set_revision_history([r1, r2b, r5])
+            self.applyDeprecated(deprecated_in((2, 4, 0)),
+                tree.branch.set_revision_history, [r1, r2b, r5])
             if tree.branch.last_revision_info() != (3, r5):
                 # RemoteBranch silently corrects an impossible revision
                 # history given to set_revision_history.  It can be tricked
@@ -73,10 +76,12 @@
 
     def test_reconcile_handles_ghosts_in_revhistory(self):
         tree = self.make_branch_and_tree('test')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support ghosts")
         tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
         r1 = tree.commit('one')
         r2 = tree.commit('two')
         tree.branch.set_last_revision_info(2, r2)
 
         reconciler = tree.branch.reconcile()
-        self.assertEquals([r1, r2], tree.branch.revision_history())
+        self.assertEquals(r2, tree.branch.last_revision())

=== modified file 'bzrlib/tests/per_branch/test_revision_history.py'
--- bzrlib/tests/per_branch/test_revision_history.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_revision_history.py	2011-10-05 12:45:41 +0000
@@ -21,7 +21,11 @@
     errors,
     revision as _mod_revision,
     )
-from bzrlib.tests import per_branch
+from bzrlib.symbol_versioning import deprecated_in
+from bzrlib.tests import (
+    per_branch,
+    TestNotApplicable,
+    )
 
 
 class TestLastRevision(per_branch.TestCaseWithBranch):
@@ -38,9 +42,13 @@
         br = self.get_branch()
         br.fetch(wt.branch)
         br.set_last_revision_info(1, 'rev1')
-        self.assertEquals(br.revision_history(), ["rev1",])
+        self.assertEquals(
+            self.applyDeprecated(deprecated_in((2, 5, 0)), br.revision_history),
+            ["rev1",])
         br.set_last_revision_info(3, 'rev3')
-        self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
+        self.assertEquals(
+            self.applyDeprecated(deprecated_in((2, 5, 0)), br.revision_history),
+            ["rev1", "rev2", "rev3"])
         # append_revision specifically prohibits some ids;
         # set_last_revision_info currently does not
         ## self.assertRaises(errors.ReservedId,
@@ -84,8 +92,8 @@
         """
         branch, calls = self.get_instrumented_branch()
         # Repeatedly call revision_history.
-        branch.revision_history()
-        branch.revision_history()
+        self.applyDeprecated(deprecated_in((2, 5, 0)), branch.revision_history)
+        self.applyDeprecated(deprecated_in((2, 5, 0)), branch.revision_history)
         self.assertEqual(
             ['_gen_revision_history', '_gen_revision_history'], calls)
 
@@ -97,8 +105,10 @@
         # Lock the branch, then repeatedly call revision_history.
         branch.lock_read()
         try:
-            branch.revision_history()
-            branch.revision_history()
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             self.assertEqual(['_gen_revision_history'], calls)
         finally:
             branch.unlock()
@@ -112,9 +122,11 @@
         # Lock the branch, set the revision history, then repeatedly call
         # revision_history.
         branch.lock_write()
-        branch.set_revision_history([])
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch.set_revision_history, [])
         try:
-            branch.revision_history()
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             self.assertEqual([], calls)
         finally:
             branch.unlock()
@@ -124,8 +136,10 @@
         cause the revision history to be cached.
         """
         branch, calls = self.get_instrumented_branch()
-        branch.set_revision_history([])
-        branch.revision_history()
+        self.applyDeprecated(deprecated_in((2, 4, 0)),
+            branch.set_revision_history, [])
+        self.applyDeprecated(deprecated_in((2, 5, 0)),
+            branch.revision_history)
         self.assertEqual(['_gen_revision_history'], calls)
 
     def test_set_last_revision_info_when_locked(self):
@@ -147,17 +161,14 @@
             a_branch.unlock()
 
     def test_set_last_revision_info_none(self):
-        """Passing None to revision_info to None sets it to NULL_REVISION."""
+        """Passing None to set_last_revision_info raises an exception."""
         a_branch = self.get_branch()
         # Lock the branch, set the last revision info, then call
         # last_revision_info.
         a_branch.lock_write()
         self.addCleanup(a_branch.unlock)
-        self.callDeprecated(['NULL_REVISION should be used for the null'
-            ' revision instead of None, as of bzr 0.91.'],
+        self.assertRaises(errors.InvalidRevisionId,
             a_branch.set_last_revision_info, 0, None)
-        self.assertEqual((0, _mod_revision.NULL_REVISION),
-                         a_branch.last_revision_info())
 
     def test_set_last_revision_info_uncaches_revision_history_for_format6(self):
         """On format 6 branches, set_last_revision_info invalidates the revision
@@ -168,12 +179,14 @@
         a_branch, calls = self.get_instrumented_branch()
         # Lock the branch, cache the revision history.
         a_branch.lock_write()
-        a_branch.revision_history()
+        self.applyDeprecated(deprecated_in((2, 5, 0)),
+            a_branch.revision_history)
         # Set the last revision info, clearing the cache.
         a_branch.set_last_revision_info(0, _mod_revision.NULL_REVISION)
         del calls[:]
         try:
-            a_branch.revision_history()
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                a_branch.revision_history)
             self.assertEqual(['_gen_revision_history'], calls)
         finally:
             a_branch.unlock()
@@ -189,14 +202,17 @@
         branch.lock_read()
         try:
             # The first time the data returned will not be in the cache.
-            history = branch.revision_history()
+            history = self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             history.append('one')
             # The second time the data comes from the cache.
-            history = branch.revision_history()
+            history = self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             history.append('two')
             # The revision_history() should still be unchanged, even though
             # we've mutated the return values from earlier calls.
-            self.assertEqual([], branch.revision_history())
+            self.assertEqual([], self.applyDeprecated(
+                deprecated_in((2, 5, 0)), branch.revision_history))
         finally:
             branch.unlock()
 
@@ -205,8 +221,11 @@
 
     def test_parent_ghost(self):
         tree = self.make_branch_and_tree('tree')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support ghosts")
         tree.add_parent_tree_id('ghost-revision',
                                 allow_leftmost_as_ghost=True)
         tree.commit('first non-ghost commit', rev_id='non-ghost-revision')
         self.assertEqual(['non-ghost-revision'],
-                         tree.branch.revision_history())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                            tree.branch.revision_history))

=== modified file 'bzrlib/tests/per_branch/test_revision_id_to_revno.py'
--- bzrlib/tests/per_branch/test_revision_id_to_revno.py	2009-07-10 05:49:34 +0000
+++ bzrlib/tests/per_branch/test_revision_id_to_revno.py	2011-09-24 13:00:12 +0000
@@ -17,6 +17,7 @@
 """Tests for Branch.revision_id_to_revno()"""
 
 from bzrlib import errors
+from bzrlib.tests import TestNotApplicable
 
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
@@ -39,3 +40,19 @@
         self.assertRaises(errors.NoSuchRevision,
                           the_branch.revision_id_to_revno, 'rev-1.1.1')
 
+    def test_mainline_ghost(self):
+        tree = self.make_branch_and_tree('tree1')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support ghosts")
+        tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
+        tree.add('')
+        tree.commit('msg1', rev_id='rev1')
+        tree.commit('msg2', rev_id='rev2')
+        # Some older branch formats store the full known revision history
+        # and thus can't distinguish between not being able to find a revno because of
+        # a ghost and the revision not being on the mainline. As such,
+        # allow both NoSuchRevision and GhostRevisionsHaveNoRevno here.
+        self.assertRaises((errors.NoSuchRevision, errors.GhostRevisionsHaveNoRevno),
+            tree.branch.revision_id_to_revno, "unknown")
+        self.assertEquals(1, tree.branch.revision_id_to_revno("rev1"))
+        self.assertEquals(2, tree.branch.revision_id_to_revno("rev2"))

=== modified file 'bzrlib/tests/per_branch/test_sprout.py'
--- bzrlib/tests/per_branch/test_sprout.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_branch/test_sprout.py	2011-10-05 12:45:41 +0000
@@ -25,6 +25,9 @@
     revision as _mod_revision,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
 
@@ -53,7 +56,7 @@
         # Start with a format that is unlikely to be the target format
         # We call the super class to allow overriding the format of creation)
         source = tests.TestCaseWithTransport.make_branch(self, 'old-branch',
-                                                         format='metaweave')
+                                                         format='knit')
         target_bzrdir = self.make_bzrdir('target')
         target_bzrdir.create_repository()
         result_format = self.branch_format
@@ -109,7 +112,26 @@
         repo.fetch(wt.branch.repository)
         branch2 = wt.branch.sprout(repo.bzrdir, revision_id='rev2-alt')
         self.assertEqual((2, 'rev2-alt'), branch2.last_revision_info())
-        self.assertEqual(['rev1', 'rev2-alt'], branch2.revision_history())
+        self.assertEqual('rev2-alt', branch2.last_revision())
+
+    def test_sprout_preserves_tags(self):
+        """Sprout preserves tags, even tags of absent revisions."""
+        try:
+            builder = self.make_branch_builder('source')
+        except errors.UninitializableFormat:
+            raise tests.TestSkipped('Uninitializable branch format')
+        builder.build_commit(message="Rev 1", rev_id='rev-1')
+        source = builder.get_branch()
+        try:
+            source.tags.set_tag('tag-a', 'missing-rev')
+        except (errors.TagsNotSupported, errors.GhostTagsNotSupported):
+            raise tests.TestNotApplicable(
+                'Branch format does not support tags or tags to ghosts.')
+        # Now source has a tag pointing to an absent revision.  Sprout it.
+        target_bzrdir = self.make_repository('target').bzrdir
+        new_branch = source.sprout(target_bzrdir)
+        # The tag is present in the target
+        self.assertEqual('missing-rev', new_branch.tags.lookup_tag('tag-a'))
 
     def test_sprout_from_any_repo_revision(self):
         """We should be able to sprout from any revision."""
@@ -125,15 +147,15 @@
         wt2 = wt.bzrdir.sprout('target',
             revision_id='rev1a').open_workingtree()
         self.assertEqual('rev1a', wt2.last_revision())
-        self.failUnlessExists('target/a')
+        self.assertPathExists('target/a')
 
     def test_sprout_with_unicode_symlink(self):
         # this tests bug #272444
         # Since the trigger function seems to be set_parent_trees, there exists
         # also a similar test, with name test_unicode_symlink, in class
         # TestSetParents at file per_workingtree/test_parents.py
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         tree = self.make_branch_and_tree('tree1')
 
@@ -157,6 +179,9 @@
 
     def test_sprout_with_ghost_in_mainline(self):
         tree = self.make_branch_and_tree('tree1')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable(
+                "repository format does not support ghosts in mainline")
         tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
         tree.add('')
         tree.commit('msg1', rev_id='rev1')
@@ -182,9 +207,9 @@
                 source.last_revision(), possible_transports=[target_transport],
                 source_branch=source, stacked=True)
         except errors.UnstackableBranchFormat:
-            if isinstance(self.branch_format, _mod_branch.BzrBranchFormat4):
-                raise tests.KnownFailure(
-                    "Format 4 doesn't auto stack successfully.")
+            if not self.branch_format.supports_stacking():
+                raise tests.TestNotApplicable(
+                    "Format doesn't auto stack successfully.")
             else:
                 raise
         result = dir.open_branch()

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2010-07-01 03:41:20 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2011-10-14 13:56:45 +0000
@@ -23,7 +23,7 @@
     errors,
     )
 from bzrlib.revision import NULL_REVISION
-from bzrlib.tests import TestNotApplicable, transport_util
+from bzrlib.tests import fixtures, TestNotApplicable, transport_util
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
 
@@ -150,12 +150,11 @@
         self.assertRevisionNotInRepository('newbranch', trunk_revid)
         tree = new_dir.open_branch().create_checkout('local')
         new_branch_revid = tree.commit('something local')
-        self.assertRevisionNotInRepository('mainline', new_branch_revid)
+        self.assertRevisionNotInRepository(
+            trunk_tree.branch.base, new_branch_revid)
         self.assertRevisionInRepository('newbranch', new_branch_revid)
 
     def test_sprout_stacked_from_smart_server(self):
-        if isinstance(self.branch_format, branch.BzrBranchFormat4):
-            raise TestNotApplicable('Branch format 4 is not usable via HPSS.')
         # We have a mainline
         trunk_tree = self.make_branch_and_tree('mainline')
         trunk_revid = trunk_tree.commit('mainline')
@@ -173,22 +172,26 @@
         self.assertRevisionNotInRepository('newbranch', trunk_revid)
         tree = new_dir.open_branch().create_checkout('local')
         new_branch_revid = tree.commit('something local')
-        self.assertRevisionNotInRepository('mainline', new_branch_revid)
+        self.assertRevisionNotInRepository(trunk_tree.branch.user_url,
+            new_branch_revid)
         self.assertRevisionInRepository('newbranch', new_branch_revid)
 
     def test_unstack_fetches(self):
         """Removing the stacked-on branch pulls across all data"""
+        try:
+            builder = self.make_branch_builder('trunk')
+        except errors.UninitializableFormat:
+            raise TestNotApplicable('uninitializeable format')
         # We have a mainline
-        trunk_tree = self.make_branch_and_tree('mainline')
-        trunk_revid = trunk_tree.commit('revision on mainline')
-        # and make branch from it which is stacked
+        trunk = fixtures.build_branch_with_non_ancestral_rev(builder)
+        mainline_revid = 'rev-1'
+        # and make branch from it which is stacked (with no tags)
         try:
-            new_dir = trunk_tree.bzrdir.sprout(self.get_url('newbranch'),
-                stacked=True)
+            new_dir = trunk.bzrdir.sprout(self.get_url('newbranch'), stacked=True)
         except unstackable_format_errors, e:
             raise TestNotApplicable(e)
         # stacked repository
-        self.assertRevisionNotInRepository('newbranch', trunk_revid)
+        self.assertRevisionNotInRepository('newbranch', mainline_revid)
         # TODO: we'd like to commit in the stacked repository; that requires
         # some care (maybe a BranchBuilder) if it's remote and has no
         # workingtree
@@ -197,13 +200,21 @@
         # now when we unstack that should implicitly fetch, to make sure that
         # the branch will still work
         new_branch = new_dir.open_branch()
+        try:
+            new_branch.tags.set_tag('tag-a', 'rev-2')
+        except errors.TagsNotSupported:
+            tags_supported = False
+        else:
+            tags_supported = True
         new_branch.set_stacked_on_url(None)
-        self.assertRevisionInRepository('newbranch', trunk_revid)
+        self.assertRevisionInRepository('newbranch', mainline_revid)
         # of course it's still in the mainline
-        self.assertRevisionInRepository('mainline', trunk_revid)
+        self.assertRevisionInRepository('trunk', mainline_revid)
+        if tags_supported:
+            # the tagged revision in trunk is now in newbranch too
+            self.assertRevisionInRepository('newbranch', 'rev-2')
         # and now we're no longer stacked
-        self.assertRaises(errors.NotStacked,
-            new_branch.get_stacked_on_url)
+        self.assertRaises(errors.NotStacked, new_branch.get_stacked_on_url)
 
     def test_unstack_already_locked(self):
         """Removing the stacked-on branch with an already write-locked branch
@@ -320,7 +331,7 @@
 
     def test_sprout_stacking_policy_handling(self):
         """Obey policy where possible, ignore otherwise."""
-        if isinstance(self.branch_format, branch.BzrBranchFormat4):
+        if self.bzrdir_format.fixed_components:
             raise TestNotApplicable('Branch format 4 does not autoupgrade.')
         source = self.make_branch('source')
         stack_on = self.make_stacked_on_matching(source)
@@ -337,7 +348,7 @@
 
     def test_clone_stacking_policy_handling(self):
         """Obey policy where possible, ignore otherwise."""
-        if isinstance(self.branch_format, branch.BzrBranchFormat4):
+        if self.bzrdir_format.fixed_components:
             raise TestNotApplicable('Branch format 4 does not autoupgrade.')
         source = self.make_branch('source')
         stack_on = self.make_stacked_on_matching(source)
@@ -354,8 +365,8 @@
 
     def test_sprout_to_smart_server_stacking_policy_handling(self):
         """Obey policy where possible, ignore otherwise."""
-        if isinstance(self.branch_format, branch.BzrBranchFormat4):
-            raise TestNotApplicable('Branch format 4 is not usable via HPSS.')
+        if not self.branch_format.supports_leaving_lock():
+            raise TestNotApplicable('Branch format is not usable via HPSS.')
         source = self.make_branch('source')
         stack_on = self.make_stacked_on_matching(source)
         parent_bzrdir = self.make_bzrdir('.', format='default')
@@ -502,6 +513,8 @@
             repo = self.make_repository('repo', shared=True)
         except errors.IncompatibleFormat:
             raise TestNotApplicable()
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable()
         # Avoid make_branch, which produces standalone branches.
         bzrdir = self.make_bzrdir('repo/stack-on')
         try:
@@ -537,7 +550,7 @@
         self.assertEqual({}, repo.get_parent_map(['rev1']))
         # revision_history should work, even though the history is spread over
         # multiple repositories.
-        self.assertLength(2, stacked.branch.revision_history())
+        self.assertEquals((2, 'rev2'), stacked.branch.last_revision_info())
 
 
 class TestStackingConnections(
@@ -559,7 +572,7 @@
         stacked.set_last_revision_info(1, 'rev-base')
         stacked_relative = self.make_branch('stacked_relative',
                                             format=self.bzrdir_format)
-        stacked_relative.set_stacked_on_url('../base')
+        stacked_relative.set_stacked_on_url(base_tree.branch.user_url)
         stacked.set_last_revision_info(1, 'rev-base')
         self.start_logging_connections()
 

=== modified file 'bzrlib/tests/per_branch/test_tags.py'
--- bzrlib/tests/per_branch/test_tags.py	2010-03-12 02:14:56 +0000
+++ bzrlib/tests/per_branch/test_tags.py	2011-09-12 09:51:52 +0000
@@ -22,9 +22,8 @@
 
 from bzrlib import (
     branch,
-    bzrdir,
+    bzrdir as _mod_bzrdir,
     errors,
-    repository,
     tests,
     )
 from bzrlib.tests import per_branch
@@ -41,13 +40,22 @@
             raise tests.TestSkipped(
                 "format %s doesn't support tags" % branch._format)
 
+    def make_branch_with_revisions(self, relpath, revisions):
+        builder = self.make_branch_builder(relpath)
+        builder.start_series()
+        for revid in revisions:
+            builder.build_commit(rev_id=revid)
+        builder.finish_series()
+        return builder.get_branch()
+
     def test_tags_initially_empty(self):
         b = self.make_branch('b')
         tags = b.tags.get_tag_dict()
         self.assertEqual(tags, {})
 
     def test_make_and_lookup_tag(self):
-        b = self.make_branch('b')
+        b = self.make_branch_with_revisions('b',
+            ['target-revid-1', 'target-revid-2'])
         b.tags.set_tag('tag-name', 'target-revid-1')
         b.tags.set_tag('other-name', 'target-revid-2')
         # then reopen the branch and see they're still there
@@ -64,16 +72,26 @@
         self.assertFalse(b.tags.has_tag('imaginary'))
 
     def test_reverse_tag_dict(self):
-        b = self.make_branch('b')
+        b = self.make_branch_with_revisions('b',
+            ['target-revid-1', 'target-revid-2'])
         b.tags.set_tag('tag-name', 'target-revid-1')
         b.tags.set_tag('other-name', 'target-revid-2')
         # then reopen the branch and check reverse map id->tags list
         b = branch.Branch.open('b')
-        self.assertEqual(b.tags.get_reverse_tag_dict(),
+        self.assertEqual(dict(b.tags.get_reverse_tag_dict()),
             {'target-revid-1': ['tag-name'],
              'target-revid-2': ['other-name'],
             })
 
+    def test_ghost_tag(self):
+        b = self.make_branch('b')
+        if not b._format.supports_tags_referencing_ghosts():
+            self.assertRaises(errors.GhostTagsNotSupported,
+                b.tags.set_tag, "ghost", "idontexist")
+        else:
+            b.tags.set_tag("ghost", "idontexist")
+            self.assertEquals("idontexist", b.tags.lookup_tag("ghost"))
+
     def test_no_such_tag(self):
         b = self.make_branch('b')
         try:
@@ -85,8 +103,8 @@
             self.fail("didn't get expected exception")
 
     def test_merge_tags(self):
-        b1 = self.make_branch('b1')
-        b2 = self.make_branch('b2')
+        b1 = self.make_branch_with_revisions('b1', ['revid', 'revid-1'])
+        b2 = self.make_branch_with_revisions('b2', ['revid', 'revid-2'])
         # if there are tags in the source and not the destination, then they
         # just go across
         b1.tags.set_tag('tagname', 'revid')
@@ -95,33 +113,34 @@
         # if a tag is in the destination and not in the source, it is not
         # removed when we merge them
         b2.tags.set_tag('in-destination', 'revid')
-        result = b1.tags.merge_to(b2.tags)
-        self.assertEquals(result, [])
+        updates, conflicts = b1.tags.merge_to(b2.tags)
+        self.assertEquals(list(conflicts), [])
+        self.assertEquals(updates, {})
         self.assertEquals(b2.tags.lookup_tag('in-destination'), 'revid')
         # if there's a conflicting tag, it's reported -- the command line
         # interface will say "these tags couldn't be copied"
         b1.tags.set_tag('conflicts', 'revid-1')
         b2.tags.set_tag('conflicts', 'revid-2')
-        result = b1.tags.merge_to(b2.tags)
-        self.assertEquals(result,
+        updates, conflicts = b1.tags.merge_to(b2.tags)
+        self.assertEquals(list(conflicts),
             [('conflicts', 'revid-1', 'revid-2')])
         # and it keeps the same value
+        self.assertEquals(updates, {})
         self.assertEquals(b2.tags.lookup_tag('conflicts'), 'revid-2')
 
-
     def test_unicode_tag(self):
-        b1 = self.make_branch('b')
         tag_name = u'\u3070'
         # in anticipation of the planned change to treating revision ids as
         # just 8bit strings
         revid = ('revid' + tag_name).encode('utf-8')
+        b1 = self.make_branch_with_revisions('b', [revid])
         b1.tags.set_tag(tag_name, revid)
         self.assertEquals(b1.tags.lookup_tag(tag_name), revid)
 
     def test_delete_tag(self):
-        b = self.make_branch('b')
         tag_name = u'\N{GREEK SMALL LETTER ALPHA}'
         revid = ('revid' + tag_name).encode('utf-8')
+        b = self.make_branch_with_revisions('b', [revid])
         b.tags.set_tag(tag_name, revid)
         # now try to delete it
         b.tags.delete_tag(tag_name)
@@ -143,6 +162,271 @@
         b2 = self.make_branch('b2')
         b1.tags.merge_to(b2.tags)
 
+    def test_read_lock_caches_tags(self):
+        """Tags are read from a branch only once during a read-lock."""
+        # Open the same branch twice.  Read-lock one, and then mutate the tags
+        # in the second.  The read-locked branch never re-reads the tags, so it
+        # never observes the changed/new tags.
+        b1 = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-1-changed', 'rev-2'])
+        b1.tags.set_tag('one', 'rev-1')
+        b2 = _mod_bzrdir.BzrDir.open('b').open_branch()
+        b1.lock_read()
+        self.assertEqual({'one': 'rev-1'}, b1.tags.get_tag_dict())
+        # Add a tag and modify a tag in b2.  b1 is read-locked and has already
+        # read the tags, so it is unaffected.
+        b2.tags.set_tag('one', 'rev-1-changed')
+        b2.tags.set_tag('two', 'rev-2')
+        self.assertEqual({'one': 'rev-1'}, b1.tags.get_tag_dict())
+        b1.unlock()
+        # Once unlocked the cached value is forgotten, so now the latest tags
+        # will be retrieved.
+        self.assertEqual(
+            {'one': 'rev-1-changed', 'two': 'rev-2'}, b1.tags.get_tag_dict())
+
+    def test_unlocked_does_not_cache_tags(self):
+        """Unlocked branches do not cache tags."""
+        # Open the same branch twice.
+        b1 = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-1-changed', 'rev-2'])
+        b1.tags.set_tag('one', 'rev-1')
+        b2 = b1.bzrdir.open_branch()
+        self.assertEqual({'one': 'rev-1'}, b1.tags.get_tag_dict())
+        # Add a tag and modify a tag in b2.  b1 isn't locked, so it will
+        # immediately return the new tags too.
+        b2.tags.set_tag('one', 'rev-1-changed')
+        b2.tags.set_tag('two', 'rev-2')
+        self.assertEqual(
+            {'one': 'rev-1-changed', 'two': 'rev-2'}, b1.tags.get_tag_dict())
+
+    def test_cached_tag_dict_not_accidentally_mutable(self):
+        """When there's a cached version of the tags, b.tags.get_tag_dict
+        returns a copy of the cached data so that callers cannot accidentally
+        corrupt the cache.
+        """
+        b = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-2', 'rev-3'])
+        b.tags.set_tag('one', 'rev-1')
+        self.addCleanup(b.lock_read().unlock)
+        # The first time the data returned will not be in the cache
+        tags_dict = b.tags.get_tag_dict()
+        tags_dict['two'] = 'rev-2'
+        # The second time the data comes from the cache
+        tags_dict = b.tags.get_tag_dict()
+        tags_dict['three'] = 'rev-3'
+        # The get_tag_dict() result should still be unchanged, even though we
+        # mutated its earlier return values.
+        self.assertEqual({'one': 'rev-1'}, b.tags.get_tag_dict())
+
+    def make_write_locked_branch_with_one_tag(self):
+        b = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-1-changed', 'rev-2'])
+        b.tags.set_tag('one', 'rev-1')
+        self.addCleanup(b.lock_write().unlock)
+        # Populate the cache
+        b.tags.get_tag_dict()
+        return b
+
+    def test_set_tag_invalides_cache(self):
+        b = self.make_write_locked_branch_with_one_tag()
+        b.tags.set_tag('one', 'rev-1-changed')
+        self.assertEqual({'one': 'rev-1-changed'}, b.tags.get_tag_dict())
+
+    def test_delete_tag_invalides_cache(self):
+        b = self.make_write_locked_branch_with_one_tag()
+        b.tags.delete_tag('one')
+        self.assertEqual({}, b.tags.get_tag_dict())
+
+    def test_merge_to_invalides_cache(self):
+        b1 = self.make_write_locked_branch_with_one_tag()
+        b2 = self.make_branch_with_revisions('b2', ['rev-2', 'rev-1'])
+        b2.tags.set_tag('two', 'rev-2')
+        b2.tags.merge_to(b1.tags)
+        self.assertEqual(
+            {'one': 'rev-1', 'two': 'rev-2'}, b1.tags.get_tag_dict())
+
+    def test_rename_revisions_invalides_cache(self):
+        b = self.make_write_locked_branch_with_one_tag()
+        b.tags.rename_revisions({'rev-1': 'rev-1-changed'})
+        self.assertEqual({'one': 'rev-1-changed'}, b.tags.get_tag_dict())
+
+
+class TestTagsMergeToInCheckouts(per_branch.TestCaseWithBranch):
+    """Tests for checkout.branch.tags.merge_to.
+    
+    In particular this exercises variations in tag conflicts in the master
+    branch and/or the checkout (child).  It may seem strange to have different
+    tags in the child and master, but 'bzr merge' intentionally updates the
+    child and not the master (instead the next 'bzr commit', if the user
+    decides to commit, will update the master).  Also, merge_to in bzr < 2.3
+    didn't propagate changes to the master, and current bzr versions may find
+    themselves operating on checkouts touched by older bzrs
+    
+    So we need to make sure bzr copes gracefully with differing tags in the
+    master versus the child.
+
+    See also <https://bugs.launchpad.net/bzr/+bug/603395>.
+    """
+
+    def setUp(self):
+        super(TestTagsMergeToInCheckouts, self).setUp()
+        branch1 = self.make_branch('tags-probe')
+        if not branch1._format.supports_tags():
+            raise tests.TestSkipped(
+                "format %s doesn't support tags" % branch1._format)
+        branch2 = self.make_branch('bind-probe')
+        try:
+            branch2.bind(branch1)
+        except errors.UpgradeRequired:
+            raise tests.TestNotApplicable(
+                "format %s doesn't support bound branches" % branch2._format)
+
+    def test_merge_to_propagates_tags(self):
+        """merge_to(child) also merges tags to the master."""
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        other.tags.merge_to(child.tags)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+
+    def test_ignore_master_disables_tag_propagation(self):
+        """merge_to(child, ignore_master=True) does not merge tags to the
+        master.
+        """
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        other.tags.merge_to(child.tags, ignore_master=True)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertRaises(errors.NoSuchTag, master.tags.lookup_tag, 'foo')
+
+    def test_merge_to_overwrite_conflict_in_master(self):
+        """merge_to(child, overwrite=True) overwrites any conflicting tags in
+        the master.
+        """
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        master.tags.set_tag('foo', 'rev-2')
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags, overwrite=True)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        self.assertEquals({"foo": "rev-1"}, tag_updates)
+        self.assertLength(0, tag_conflicts)
+
+    def test_merge_to_overwrite_conflict_in_child_and_master(self):
+        """merge_to(child, overwrite=True) overwrites any conflicting tags in
+        both the child and the master.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        tag_updates, tag_conflicts = other.tags.merge_to(
+            child.tags, overwrite=True)
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        self.assertEquals({u'foo': 'rev-1'}, tag_updates)
+        self.assertLength(0, tag_conflicts)
+
+    def test_merge_to_conflict_in_child_only(self):
+        """When new_tags.merge_to(child.tags) conflicts with the child but not
+        the master, a conflict is reported and the child receives the new tag.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        master.tags.delete_tag('foo')
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
+        # Conflict in child, so it is unchanged.
+        self.assertEquals('rev-2', child.tags.lookup_tag('foo'))
+        # No conflict in the master, so the 'foo' tag equals other's value here.
+        self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        # The conflict is reported.
+        self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
+        self.assertEquals({u'foo': 'rev-1'}, tag_updates)
+
+    def test_merge_to_conflict_in_master_only(self):
+        """When new_tags.merge_to(child.tags) conflicts with the master but not
+        the child, a conflict is reported and the child receives the new tag.
+        """
+        master = self.make_branch('master')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        master.tags.set_tag('foo', 'rev-2')
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
+        # No conflict in the child, so the 'foo' tag equals other's value here.
+        self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
+        # Conflict in master, so it is unchanged.
+        self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
+        # The conflict is reported.
+        self.assertEquals({u'foo': 'rev-1'}, tag_updates)
+        self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
+
+    def test_merge_to_same_conflict_in_master_and_child(self):
+        """When new_tags.merge_to(child.tags) conflicts the same way with the
+        master and the child a single conflict is reported.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
+        # Both master and child conflict, so both stay as rev-2
+        self.assertEquals('rev-2', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
+        # The conflict is reported exactly once, even though it occurs in both
+        # master and child.
+        self.assertEquals({}, tag_updates)
+        self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
+
+    def test_merge_to_different_conflict_in_master_and_child(self):
+        """When new_tags.merge_to(child.tags) conflicts differently in the
+        master and the child both conflicts are reported.
+        """
+        master = self.make_branch('master')
+        master.tags.set_tag('foo', 'rev-2')
+        other = self.make_branch('other')
+        other.tags.set_tag('foo', 'rev-1')
+        child = self.make_branch('child')
+        child.bind(master)
+        child.update()
+        # We use the private method _set_tag_dict because normally bzr tries to
+        # avoid this scenario.
+        child.tags._set_tag_dict({'foo': 'rev-3'})
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
+        # Both master and child conflict, so both stay as they were.
+        self.assertEquals('rev-3', child.tags.lookup_tag('foo'))
+        self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
+        # Both conflicts are reported.
+        self.assertEquals({}, tag_updates)
+        self.assertEquals(
+            [(u'foo', 'rev-1', 'rev-2'), (u'foo', 'rev-1', 'rev-3')],
+            sorted(tag_conflicts))
+
 
 class TestUnsupportedTags(per_branch.TestCaseWithBranch):
     """Formats that don't support tags should give reasonable errors."""
@@ -202,7 +486,7 @@
             get_tag_name, 'get tag name foo')
         self.assertEquals("foo", self.branch.automatic_tag_name(
             self.branch.last_revision()))
-    
+
     def test_uses_first_return(self):
         get_tag_name_1 = lambda br, revid: "foo1"
         get_tag_name_2 = lambda br, revid: "foo2"

=== modified file 'bzrlib/tests/per_branch/test_update.py'
--- bzrlib/tests/per_branch/test_update.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_update.py	2011-09-26 15:21:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +16,10 @@
 
 
 from bzrlib import (
+    branch,
     errors,
     revision as _mod_revision,
+    tests,
     )
 from bzrlib.tests import per_branch
 
@@ -66,52 +68,24 @@
         # commit to the master making the child tree out of date and not a prefix.
         master_tree.commit('bar', rev_id='bar', allow_pointless=True)
         self.assertEqual('foo', child_tree.branch.update())
-        self.assertEqual(['bar'], child_tree.branch.revision_history())
-
-
-class TestUpdateRevisions(per_branch.TestCaseWithBranch):
-
-    def test_accepts_graph(self):
-        # An implementation may not use it, but it should allow a 'graph' to be
-        # supplied
-        tree1 = self.make_branch_and_tree('tree1')
-        rev1 = tree1.commit('one')
-        tree2 = tree1.bzrdir.sprout('tree2').open_workingtree()
-        rev2 = tree2.commit('two')
-
-        tree1.lock_write()
-        self.addCleanup(tree1.unlock)
-        tree2.lock_read()
-        self.addCleanup(tree2.unlock)
-        graph = tree2.branch.repository.get_graph(tree1.branch.repository)
-
-        tree1.branch.update_revisions(tree2.branch, graph=graph)
-        self.assertEqual((2, rev2), tree1.branch.last_revision_info())
-
-    def test_overwrite_ignores_diverged(self):
-        tree1 = self.make_branch_and_tree('tree1')
-        rev1 = tree1.commit('one')
-        tree2 = tree1.bzrdir.sprout('tree2').open_workingtree()
-        rev2 = tree1.commit('two')
-        rev2b = tree2.commit('alt two')
-
-        self.assertRaises(errors.DivergedBranches,
-                          tree1.branch.update_revisions,
-                          tree2.branch, overwrite=False)
-        # However, the revision should be copied into the repository
-        self.assertTrue(tree1.branch.repository.has_revision(rev2b))
-
-        tree1.branch.update_revisions(tree2.branch, overwrite=True)
-        self.assertEqual((2, rev2b), tree1.branch.last_revision_info())
-
-    def test_ignores_older_unless_overwrite(self):
-        tree1 = self.make_branch_and_tree('tree1')
-        rev1 = tree1.commit('one')
-        tree2 = tree1.bzrdir.sprout('tree2').open_workingtree()
-        rev2 = tree1.commit('two')
-
-        tree1.branch.update_revisions(tree2.branch)
-        self.assertEqual((2, rev2), tree1.branch.last_revision_info())
-
-        tree1.branch.update_revisions(tree2.branch, overwrite=True)
-        self.assertEqual((1, rev1), tree1.branch.last_revision_info())
+        self.assertEqual('bar', child_tree.branch.last_revision())
+
+    def test_update_in_checkout_of_readonly(self):
+        tree1 = self.make_branch_and_tree('tree1')
+        rev1 = tree1.commit('one')
+        try:
+            tree1.branch.tags.set_tag('test-tag', rev1)
+        except errors.TagsNotSupported:
+            # Tags not supported
+            raise tests.TestNotApplicable("only triggered from branches with"
+                " tags")
+        readonly_branch1 = branch.Branch.open('readonly+' + tree1.branch.base)
+        tree2 = tree1.bzrdir.sprout('tree2').open_workingtree()
+        try:
+            tree2.branch.bind(readonly_branch1)
+        except errors.UpgradeRequired:
+            # old branch, cant test.
+            raise tests.TestNotApplicable("only triggered in bound branches")
+        rev2 = tree1.commit('two')
+        tree2.update()
+        self.assertEqual(rev2, tree2.branch.last_revision())

=== added directory 'bzrlib/tests/per_bzrdir'
=== added file 'bzrlib/tests/per_bzrdir/__init__.py'
--- bzrlib/tests/per_bzrdir/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_bzrdir/__init__.py	2011-09-30 13:23:16 +0000
@@ -0,0 +1,88 @@
+# Copyright (C) 2010 Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+#          Jelmer Vernooij <jelmer.vernooij@canonical.com>
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""BzrDir implementation tests for bzr.
+
+These test the conformance of all the bzrdir variations to the expected API.
+Specific tests for individual formats are in the tests/test_bzrdir.py file
+rather than in tests/per_branch/*.py. Generic control directory tests not
+specific to BzrDir are in tests/per_controldir/*.py.
+"""
+
+from bzrlib.bzrdir import BzrDirFormat
+from bzrlib.controldir import ControlDirFormat
+from bzrlib.tests import (
+    default_transport,
+    multiply_tests,
+    test_server,
+    TestCaseWithTransport,
+    )
+from bzrlib.tests.per_controldir import make_scenarios
+from bzrlib.transport import memory
+
+
+class TestCaseWithBzrDir(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestCaseWithBzrDir, self).setUp()
+        self.bzrdir = None
+
+    def get_bzrdir(self):
+        if self.bzrdir is None:
+            self.bzrdir = self.make_bzrdir(None)
+        return self.bzrdir
+
+    def get_default_format(self):
+        return self.bzrdir_format
+
+
+def load_tests(standard_tests, module, loader):
+    test_per_bzrdir = [
+        'bzrlib.tests.per_bzrdir.test_bzrdir',
+        ]
+    submod_tests = loader.loadTestsFromModuleNames(test_per_bzrdir)
+    formats = [format for format in ControlDirFormat.known_formats()
+               if isinstance(format, BzrDirFormat)]
+    scenarios = make_scenarios(
+        default_transport,
+        None,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        formats)
+    # This will always add scenarios using the smart server.
+    from bzrlib.remote import RemoteBzrDirFormat
+    # test the remote server behaviour when backed with a MemoryTransport
+    # Once for the current version
+    scenarios.extend(make_scenarios(
+        memory.MemoryServer,
+        test_server.SmartTCPServer_for_testing,
+        test_server.ReadonlySmartTCPServer_for_testing,
+        [(RemoteBzrDirFormat())],
+        name_suffix='-default'))
+    # And once with < 1.6 - the 'v2' protocol.
+    scenarios.extend(make_scenarios(
+        memory.MemoryServer,
+        test_server.SmartTCPServer_for_testing_v2_only,
+        test_server.ReadonlySmartTCPServer_for_testing_v2_only,
+        [(RemoteBzrDirFormat())],
+        name_suffix='-v2'))
+    # add the tests for the sub modules
+    return multiply_tests(submod_tests, scenarios, standard_tests)

=== added file 'bzrlib/tests/per_bzrdir/test_bzrdir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_bzrdir/test_bzrdir.py	2011-10-06 23:14:32 +0000
@@ -0,0 +1,671 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrdir implementations - tests a bzrdir format."""
+
+import errno
+from stat import S_ISDIR
+
+import bzrlib.branch
+from bzrlib import (
+    bzrdir,
+    errors,
+    repository,
+    revision as _mod_revision,
+    transport,
+    workingtree,
+    )
+from bzrlib.remote import RemoteBzrDirFormat
+from bzrlib.tests import (
+    TestNotApplicable,
+    TestSkipped,
+    )
+from bzrlib.tests.per_bzrdir import TestCaseWithBzrDir
+from bzrlib.transport.local import (
+    LocalTransport,
+    )
+
+
+class AnonymousTestBranchFormat(bzrlib.branch.BranchFormat):
+    """An anonymous branch format (does not have a format string)"""
+
+    def get_format_string(self):
+        raise NotImplementedError(self.get_format_string)
+
+
+class IdentifiableTestBranchFormat(bzrlib.branch.BranchFormat):
+    """An identifable branch format (has a format string)"""
+
+    def get_format_string(self):
+        return "I have an identity"
+
+
+class AnonymousTestRepositoryFormat(repository.RepositoryFormat):
+    """An anonymous branch format (does not have a format string)"""
+
+    def get_format_string(self):
+        raise NotImplementedError(self.get_format_string)
+
+
+class IdentifiableTestRepositoryFormat(repository.RepositoryFormat):
+    """An identifable branch format (has a format string)"""
+
+    def get_format_string(self):
+        return "I have an identity"
+
+
+class AnonymousTestWorkingTreeFormat(workingtree.WorkingTreeFormat):
+    """An anonymous branch format (does not have a format string)"""
+
+    def get_format_string(self):
+        raise NotImplementedError(self.get_format_string)
+
+
+class IdentifiableTestWorkingTreeFormat(workingtree.WorkingTreeFormat):
+    """An identifable branch format (has a format string)"""
+
+    def get_format_string(self):
+        return "I have an identity"
+
+
+class TestBzrDir(TestCaseWithBzrDir):
+
+    # Many of these tests test for disk equality rather than checking
+    # for semantic equivalence. This works well for some tests but
+    # is not good at handling changes in representation or the addition
+    # or removal of control data. It would be nice to for instance:
+    # sprout a new branch, check that the nickname has been reset by hand
+    # and then set the nickname to match the source branch, at which point
+    # a semantic equivalence should pass
+
+    def assertDirectoriesEqual(self, source, target, ignore_list=[]):
+        """Assert that the content of source and target are identical.
+
+        paths in ignore list will be completely ignored.
+
+        We ignore paths that represent data which is allowed to change during
+        a clone or sprout: for instance, inventory.knit contains gzip fragements
+        which have timestamps in them, and as we have read the inventory from
+        the source knit, the already-read data is recompressed rather than
+        reading it again, which leads to changed timestamps. This is ok though,
+        because the inventory.kndx file is not ignored, and the integrity of
+        knit joins is tested by test_knit and test_versionedfile.
+
+        :seealso: Additionally, assertRepositoryHasSameItems provides value
+            rather than representation checking of repositories for
+            equivalence.
+        """
+        files = []
+        directories = ['.']
+        while directories:
+            dir = directories.pop()
+            for path in set(source.list_dir(dir) + target.list_dir(dir)):
+                path = dir + '/' + path
+                if path in ignore_list:
+                    continue
+                try:
+                    stat = source.stat(path)
+                except errors.NoSuchFile:
+                    self.fail('%s not in source' % path)
+                if S_ISDIR(stat.st_mode):
+                    self.assertTrue(S_ISDIR(target.stat(path).st_mode))
+                    directories.append(path)
+                else:
+                    self.assertEqualDiff(source.get(path).read(),
+                                         target.get(path).read(),
+                                         "text for file %r differs:\n" % path)
+
+    def assertRepositoryHasSameItems(self, left_repo, right_repo):
+        """require left_repo and right_repo to contain the same data."""
+        # XXX: TODO: Doesn't work yet, because we need to be able to compare
+        # local repositories to remote ones...  but this is an as-yet unsolved
+        # aspect of format management and the Remote protocols...
+        # self.assertEqual(left_repo._format.__class__,
+        #     right_repo._format.__class__)
+        left_repo.lock_read()
+        try:
+            right_repo.lock_read()
+            try:
+                # revs
+                all_revs = left_repo.all_revision_ids()
+                self.assertEqual(left_repo.all_revision_ids(),
+                    right_repo.all_revision_ids())
+                for rev_id in left_repo.all_revision_ids():
+                    self.assertEqual(left_repo.get_revision(rev_id),
+                        right_repo.get_revision(rev_id))
+                # Assert the revision trees (and thus the inventories) are equal
+                sort_key = lambda rev_tree: rev_tree.get_revision_id()
+                rev_trees_a = sorted(
+                    left_repo.revision_trees(all_revs), key=sort_key)
+                rev_trees_b = sorted(
+                    right_repo.revision_trees(all_revs), key=sort_key)
+                for tree_a, tree_b in zip(rev_trees_a, rev_trees_b):
+                    self.assertEqual([], list(tree_a.iter_changes(tree_b)))
+                # texts
+                text_index = left_repo._generate_text_key_index()
+                self.assertEqual(text_index,
+                    right_repo._generate_text_key_index())
+                desired_files = []
+                for file_id, revision_id in text_index.iterkeys():
+                    desired_files.append(
+                        (file_id, revision_id, (file_id, revision_id)))
+                left_texts = list(left_repo.iter_files_bytes(desired_files))
+                right_texts = list(right_repo.iter_files_bytes(desired_files))
+                left_texts.sort()
+                right_texts.sort()
+                self.assertEqual(left_texts, right_texts)
+                # signatures
+                for rev_id in all_revs:
+                    try:
+                        left_text = left_repo.get_signature_text(rev_id)
+                    except errors.NoSuchRevision:
+                        continue
+                    right_text = right_repo.get_signature_text(rev_id)
+                    self.assertEqual(left_text, right_text)
+            finally:
+                right_repo.unlock()
+        finally:
+            left_repo.unlock()
+
+    def sproutOrSkip(self, from_bzrdir, to_url, revision_id=None,
+                     force_new_repo=False, accelerator_tree=None,
+                     create_tree_if_local=True):
+        """Sprout from_bzrdir into to_url, or raise TestSkipped.
+
+        A simple wrapper for from_bzrdir.sprout that translates NotLocalUrl into
+        TestSkipped.  Returns the newly sprouted bzrdir.
+        """
+        to_transport = transport.get_transport(to_url)
+        if not isinstance(to_transport, LocalTransport):
+            raise TestSkipped('Cannot sprout to remote bzrdirs.')
+        target = from_bzrdir.sprout(to_url, revision_id=revision_id,
+                                    force_new_repo=force_new_repo,
+                                    possible_transports=[to_transport],
+                                    accelerator_tree=accelerator_tree,
+                                    create_tree_if_local=create_tree_if_local)
+        return target
+
+    def skipIfNoWorkingTree(self, a_bzrdir):
+        """Raises TestSkipped if a_bzrdir doesn't have a working tree.
+
+        If the bzrdir does have a workingtree, this is a no-op.
+        """
+        try:
+            a_bzrdir.open_workingtree()
+        except (errors.NotLocalUrl, errors.NoWorkingTree):
+            raise TestSkipped("bzrdir on transport %r has no working tree"
+                              % a_bzrdir.transport)
+
+    def createWorkingTreeOrSkip(self, a_bzrdir):
+        """Create a working tree on a_bzrdir, or raise TestSkipped.
+
+        A simple wrapper for create_workingtree that translates NotLocalUrl into
+        TestSkipped.  Returns the newly created working tree.
+        """
+        try:
+            # This passes in many named options to make sure they're
+            # understood by subclasses: see
+            # <https://bugs.launchpad.net/bzr/+bug/524627>.
+            return a_bzrdir.create_workingtree(
+                revision_id=None,
+                from_branch=None,
+                accelerator_tree=None,
+                hardlink=False)
+        except errors.NotLocalUrl:
+            raise TestSkipped("cannot make working tree with transport %r"
+                              % a_bzrdir.transport)
+
+    def test_clone_bzrdir_repository_under_shared_force_new_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
+
+    def test_clone_bzrdir_branch_and_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.branch.repository.copy_content_into(source.repository)
+        tree.branch.copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/basis-inventory-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     './.bzr/stat-cache',
+                                    ])
+        self.assertRepositoryHasSameItems(
+            tree.branch.repository, target.open_repository())
+
+    def test_clone_on_transport(self):
+        a_dir = self.make_bzrdir('source')
+        target_transport = a_dir.root_transport.clone('..').clone('target')
+        target = a_dir.clone_on_transport(target_transport)
+        self.assertNotEqual(a_dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(a_dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
+
+    def test_clone_bzrdir_empty(self):
+        dir = self.make_bzrdir('source')
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
+
+    def test_clone_bzrdir_empty_force_new_ignored(self):
+        # the force_new_repo parameter should have no effect on an empty
+        # bzrdir's clone logic
+        dir = self.make_bzrdir('source')
+        target = dir.clone(self.get_url('target'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/merge-hashes'])
+
+    def test_clone_bzrdir_repository(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_repository())
+
+    def test_clone_bzrdir_tree_branch_repo(self):
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['source/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_branch().repository)
+        target.open_workingtree().revert()
+
+    def test_revert_inventory(self):
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['source/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_branch().repository)
+
+        target.open_workingtree().revert()
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository,
+            target.open_branch().repository)
+
+    def test_clone_bzrdir_tree_branch_reference(self):
+        # a tree with a branch reference (aka a checkout)
+        # should stay a checkout on clone.
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                target_branch=referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        self.createWorkingTreeOrSkip(dir)
+        target = dir.clone(self.get_url('target'))
+        self.skipIfNoWorkingTree(target)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/stat-cache',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/merge-hashes',
+                                     './.bzr/merge-hashes',
+                                     './.bzr/repository/inventory.knit',
+                                     ])
+
+    def test_clone_bzrdir_branch_and_repo_into_shared_repo_force_new_repo(self):
+        # by default cloning into a shared repo uses the shared repo.
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.branch.repository.copy_content_into(source.repository)
+        tree.branch.copy_content_into(source)
+        try:
+            self.make_repository('target', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        dir = source.bzrdir
+        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        repo = target.open_repository()
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    ['./.bzr/repository',
+                                     ])
+        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
+
+    def test_clone_bzrdir_branch_reference(self):
+        # cloning should preserve the reference status of the branch in a bzrdir
+        referenced_branch = self.make_branch('referencced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
+                target_branch=referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            return
+        target = dir.clone(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+
+    def test_sprout_bzrdir_repository(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1', rev_id='1')
+        dir = self.make_bzrdir('source')
+        repo = dir.create_repository()
+        repo.fetch(tree.branch.repository)
+        self.assertTrue(repo.has_revision('1'))
+        try:
+            self.assertTrue(
+                _mod_revision.is_null(_mod_revision.ensure_null(
+                dir.open_branch().last_revision())))
+        except errors.NotBranchError:
+            pass
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        # testing inventory isn't reasonable for repositories
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/branch',
+                                     './.bzr/checkout',
+                                     './.bzr/inventory',
+                                     './.bzr/parent',
+                                     './.bzr/repository/inventory.knit',
+                                     ])
+        try:
+            local_inventory = dir.transport.local_abspath('inventory')
+        except errors.NotLocalUrl:
+            return
+        try:
+            # If we happen to have a tree, we'll guarantee everything
+            # except for the tree root is the same.
+            inventory_f = file(local_inventory, 'rb')
+            self.addCleanup(inventory_f.close)
+            self.assertContainsRe(inventory_f.read(),
+                                  '<inventory format="5">\n</inventory>\n')
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+
+    def test_sprout_bzrdir_branch_and_repo(self):
+        tree = self.make_branch_and_tree('commit_tree')
+        self.build_tree(['commit_tree/foo'])
+        tree.add('foo')
+        tree.commit('revision 1')
+        source = self.make_branch('source')
+        tree.branch.repository.copy_content_into(source.repository)
+        tree.bzrdir.open_branch().copy_content_into(source)
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        target_repo = target.open_repository()
+        self.assertRepositoryHasSameItems(source.repository, target_repo)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/basis-inventory-cache',
+                                     './.bzr/branch/branch.conf',
+                                     './.bzr/branch/parent',
+                                     './.bzr/checkout',
+                                     './.bzr/checkout/inventory',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/inventory',
+                                     './.bzr/parent',
+                                     './.bzr/repository',
+                                     './.bzr/stat-cache',
+                                     './foo',
+                                     ])
+
+    def test_sprout_bzrdir_tree_branch_repo(self):
+        tree = self.make_branch_and_tree('source')
+        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
+        tree.add('foo')
+        tree.commit('revision 1')
+        dir = tree.bzrdir
+        target = self.sproutOrSkip(dir, self.get_url('target'))
+        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
+                                    [
+                                     './.bzr/branch',
+                                     './.bzr/checkout/dirstate',
+                                     './.bzr/checkout/stat-cache',
+                                     './.bzr/checkout/inventory',
+                                     './.bzr/inventory',
+                                     './.bzr/parent',
+                                     './.bzr/repository',
+                                     './.bzr/stat-cache',
+                                     ])
+        self.assertRepositoryHasSameItems(
+            tree.branch.repository, target.open_repository())
+
+
+    def test_retire_bzrdir(self):
+        bd = self.make_bzrdir('.')
+        transport = bd.root_transport
+        # must not overwrite existing directories
+        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
+            transport=transport)
+        self.assertTrue(transport.has('.bzr'))
+        bd.retire_bzrdir()
+        self.assertFalse(transport.has('.bzr'))
+        self.assertTrue(transport.has('.bzr.retired.1'))
+
+    def test_retire_bzrdir_limited(self):
+        bd = self.make_bzrdir('.')
+        transport = bd.root_transport
+        # must not overwrite existing directories
+        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
+            transport=transport)
+        self.assertTrue(transport.has('.bzr'))
+        self.assertRaises((errors.FileExists, errors.DirectoryNotEmpty),
+            bd.retire_bzrdir, limit=0)
+
+    def test_get_branch_transport(self):
+        dir = self.make_bzrdir('.')
+        # without a format, get_branch_transport gives use a transport
+        # which -may- point to an existing dir.
+        self.assertTrue(isinstance(dir.get_branch_transport(None),
+                                   transport.Transport))
+        # with a given format, either the bzr dir supports identifiable
+        # branches, or it supports anonymous branch formats, but not both.
+        anonymous_format = AnonymousTestBranchFormat()
+        identifiable_format = IdentifiableTestBranchFormat()
+        try:
+            found_transport = dir.get_branch_transport(anonymous_format)
+            self.assertRaises(errors.IncompatibleFormat,
+                              dir.get_branch_transport,
+                              identifiable_format)
+        except errors.IncompatibleFormat:
+            found_transport = dir.get_branch_transport(identifiable_format)
+        self.assertTrue(isinstance(found_transport, transport.Transport))
+        # and the dir which has been initialized for us must exist.
+        found_transport.list_dir('.')
+
+    def test_get_repository_transport(self):
+        dir = self.make_bzrdir('.')
+        # without a format, get_repository_transport gives use a transport
+        # which -may- point to an existing dir.
+        self.assertTrue(isinstance(dir.get_repository_transport(None),
+                                   transport.Transport))
+        # with a given format, either the bzr dir supports identifiable
+        # repositories, or it supports anonymous repository formats, but not both.
+        anonymous_format = AnonymousTestRepositoryFormat()
+        identifiable_format = IdentifiableTestRepositoryFormat()
+        try:
+            found_transport = dir.get_repository_transport(anonymous_format)
+            self.assertRaises(errors.IncompatibleFormat,
+                              dir.get_repository_transport,
+                              identifiable_format)
+        except errors.IncompatibleFormat:
+            found_transport = dir.get_repository_transport(identifiable_format)
+        self.assertTrue(isinstance(found_transport, transport.Transport))
+        # and the dir which has been initialized for us must exist.
+        found_transport.list_dir('.')
+
+    def test_get_workingtree_transport(self):
+        dir = self.make_bzrdir('.')
+        # without a format, get_workingtree_transport gives use a transport
+        # which -may- point to an existing dir.
+        self.assertTrue(isinstance(dir.get_workingtree_transport(None),
+                                   transport.Transport))
+        # with a given format, either the bzr dir supports identifiable
+        # trees, or it supports anonymous tree formats, but not both.
+        anonymous_format = AnonymousTestWorkingTreeFormat()
+        identifiable_format = IdentifiableTestWorkingTreeFormat()
+        try:
+            found_transport = dir.get_workingtree_transport(anonymous_format)
+            self.assertRaises(errors.IncompatibleFormat,
+                              dir.get_workingtree_transport,
+                              identifiable_format)
+        except errors.IncompatibleFormat:
+            found_transport = dir.get_workingtree_transport(identifiable_format)
+        self.assertTrue(isinstance(found_transport, transport.Transport))
+        # and the dir which has been initialized for us must exist.
+        found_transport.list_dir('.')
+
+    def assertInitializeEx(self, t, need_meta=False, **kwargs):
+        """Execute initialize_on_transport_ex and check it succeeded correctly.
+
+        This involves checking that the disk objects were created, open with
+        the same format returned, and had the expected disk format.
+
+        :param t: The transport to initialize on.
+        :param **kwargs: Additional arguments to pass to
+            initialize_on_transport_ex.
+        :return: the resulting repo, control dir tuple.
+        """
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("control dir format is not "
+                "initializable")
+        repo, control, require_stacking, repo_policy = \
+            self.bzrdir_format.initialize_on_transport_ex(t, **kwargs)
+        if repo is not None:
+            # Repositories are open write-locked
+            self.assertTrue(repo.is_write_locked())
+            self.addCleanup(repo.unlock)
+        self.assertIsInstance(control, bzrdir.BzrDir)
+        opened = bzrdir.BzrDir.open(t.base)
+        expected_format = self.bzrdir_format
+        if need_meta and expected_format.fixed_components:
+            # Pre-metadir formats change when we are making something that
+            # needs a metaformat, because clone is used for push.
+            expected_format = bzrdir.BzrDirMetaFormat1()
+        if not isinstance(expected_format, RemoteBzrDirFormat):
+            self.assertEqual(control._format.network_name(),
+                expected_format.network_name())
+            self.assertEqual(control._format.network_name(),
+                opened._format.network_name())
+        self.assertEqual(control.__class__, opened.__class__)
+        return repo, control
+
+    def test_format_initialize_on_transport_ex_default_stack_on(self):
+        # When initialize_on_transport_ex uses a stacked-on branch because of
+        # a stacking policy on the target, the location of the fallback
+        # repository is the same as the external location of the stacked-on
+        # branch.
+        balloon = self.make_bzrdir('balloon')
+        if isinstance(balloon._format, bzrdir.BzrDirMetaFormat1):
+            stack_on = self.make_branch('stack-on', format='1.9')
+        else:
+            stack_on = self.make_branch('stack-on')
+        if not stack_on.repository._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires nesting repositories")
+        config = self.make_bzrdir('.').get_config()
+        try:
+            config.set_default_stack_on('stack-on')
+        except errors.BzrError:
+            raise TestNotApplicable('Only relevant for stackable formats.')
+        # Initialize a bzrdir subject to the policy.
+        t = self.get_transport('stacked')
+        repo_fmt = bzrdir.format_registry.make_bzrdir('1.9')
+        repo_name = repo_fmt.repository_format.network_name()
+        repo, control = self.assertInitializeEx(
+            t, need_meta=True, repo_format_name=repo_name, stacked_on=None)
+        # self.addCleanup(repo.unlock)
+        # There's one fallback repo, with a public location.
+        self.assertLength(1, repo._fallback_repositories)
+        fallback_repo = repo._fallback_repositories[0]
+        self.assertEqual(
+            stack_on.base, fallback_repo.bzrdir.root_transport.base)
+        # The bzrdir creates a branch in stacking-capable format.
+        new_branch = control.create_branch()
+        self.assertTrue(new_branch._format.supports_stacking())
+
+

=== renamed directory 'bzrlib/tests/per_bzrdir' => 'bzrlib/tests/per_controldir'
=== modified file 'bzrlib/tests/per_controldir/__init__.py'
--- bzrlib/tests/per_bzrdir/__init__.py	2010-02-11 09:21:45 +0000
+++ bzrlib/tests/per_controldir/__init__.py	2011-09-30 13:23:16 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
 # -*- coding: utf-8 -*-
 #
@@ -17,14 +17,14 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-"""BzrDir implementation tests for bzr.
+"""ControlDir implementation tests for bzr.
 
-These test the conformance of all the bzrdir variations to the expected API.
+These test the conformance of all the controldir variations to the expected API.
 Specific tests for individual formats are in the tests/test_bzrdir.py file
 rather than in tests/per_branch/*.py.
 """
 
-from bzrlib.bzrdir import BzrDirFormat
+from bzrlib.controldir import ControlDirFormat
 from bzrlib.tests import (
     default_transport,
     multiply_tests,
@@ -35,7 +35,7 @@
 
 
 def make_scenarios(vfs_factory, transport_server, transport_readonly_server,
-    formats, name_suffix=''):
+                   formats, name_suffix=''):
     """Transform the input to a list of scenarios.
 
     :param formats: A list of bzrdir_format objects.
@@ -56,10 +56,10 @@
     return result
 
 
-class TestCaseWithBzrDir(TestCaseWithTransport):
+class TestCaseWithControlDir(TestCaseWithTransport):
 
     def setUp(self):
-        super(TestCaseWithBzrDir, self).setUp()
+        super(TestCaseWithControlDir, self).setUp()
         self.bzrdir = None
 
     def get_bzrdir(self):
@@ -67,20 +67,18 @@
             self.bzrdir = self.make_bzrdir(None)
         return self.bzrdir
 
-    def make_bzrdir(self, relpath, format=None):
-        if format is None:
-            format = self.bzrdir_format
-        return super(TestCaseWithBzrDir, self).make_bzrdir(
-            relpath, format=format)
+    def get_default_format(self):
+        return self.bzrdir_format
 
 
 def load_tests(standard_tests, module, loader):
-    test_per_bzrdir = [
-        'bzrlib.tests.per_bzrdir.test_bzrdir',
-        'bzrlib.tests.per_bzrdir.test_push',
+    test_per_controldir = [
+        'bzrlib.tests.per_controldir.test_controldir',
+        'bzrlib.tests.per_controldir.test_format',
+        'bzrlib.tests.per_controldir.test_push',
         ]
-    submod_tests = loader.loadTestsFromModuleNames(test_per_bzrdir)
-    formats = BzrDirFormat.known_formats()
+    submod_tests = loader.loadTestsFromModuleNames(test_per_controldir)
+    formats = ControlDirFormat.known_formats()
     scenarios = make_scenarios(
         default_transport,
         None,

=== renamed file 'bzrlib/tests/per_bzrdir/test_bzrdir.py' => 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	2010-06-29 16:21:13 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2011-10-14 13:56:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,153 +14,43 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Tests for bzrdir implementations - tests a bzrdir format."""
+"""Tests for control directory implementations - tests a controldir format."""
 
-from cStringIO import StringIO
-import errno
 from itertools import izip
-import os
-from stat import S_ISDIR
 
 import bzrlib.branch
 from bzrlib import (
     bzrdir,
     check,
+    controldir,
     errors,
     gpg,
-    lockdir,
     osutils,
-    repository,
-    revision as _mod_revision,
-    transactions,
     transport,
     ui,
     urlutils,
     workingtree,
     )
-from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
-from bzrlib.errors import (FileExists,
-                           NoSuchRevision,
-                           NoSuchFile,
-                           UninitializableFormat,
-                           NotBranchError,
-                           )
 import bzrlib.revision
 from bzrlib.tests import (
-                          ChrootedTestCase,
-                          TestCase,
-                          TestCaseWithTransport,
-                          TestNotApplicable,
-                          TestSkipped,
-                          )
-from bzrlib.tests.per_bzrdir import TestCaseWithBzrDir
-from bzrlib.trace import mutter
+    fixtures,
+    ChrootedTestCase,
+    TestNotApplicable,
+    TestSkipped,
+    )
+from bzrlib.tests.per_controldir import TestCaseWithControlDir
 from bzrlib.transport.local import LocalTransport
 from bzrlib.ui import (
     CannedInputUIFactory,
     )
-from bzrlib.upgrade import upgrade
-from bzrlib.remote import RemoteBzrDir, RemoteRepository
-from bzrlib.repofmt import weaverepo
-
-
-class TestBzrDir(TestCaseWithBzrDir):
-    # Many of these tests test for disk equality rather than checking
-    # for semantic equivalence. This works well for some tests but
-    # is not good at handling changes in representation or the addition
-    # or removal of control data. It would be nice to for instance:
-    # sprout a new branch, check that the nickname has been reset by hand
-    # and then set the nickname to match the source branch, at which point
-    # a semantic equivalence should pass
-
-    def assertDirectoriesEqual(self, source, target, ignore_list=[]):
-        """Assert that the content of source and target are identical.
-
-        paths in ignore list will be completely ignored.
-
-        We ignore paths that represent data which is allowed to change during
-        a clone or sprout: for instance, inventory.knit contains gzip fragements
-        which have timestamps in them, and as we have read the inventory from
-        the source knit, the already-read data is recompressed rather than
-        reading it again, which leads to changed timestamps. This is ok though,
-        because the inventory.kndx file is not ignored, and the integrity of
-        knit joins is tested by test_knit and test_versionedfile.
-
-        :seealso: Additionally, assertRepositoryHasSameItems provides value
-            rather than representation checking of repositories for
-            equivalence.
-        """
-        files = []
-        directories = ['.']
-        while directories:
-            dir = directories.pop()
-            for path in set(source.list_dir(dir) + target.list_dir(dir)):
-                path = dir + '/' + path
-                if path in ignore_list:
-                    continue
-                try:
-                    stat = source.stat(path)
-                except errors.NoSuchFile:
-                    self.fail('%s not in source' % path)
-                if S_ISDIR(stat.st_mode):
-                    self.assertTrue(S_ISDIR(target.stat(path).st_mode))
-                    directories.append(path)
-                else:
-                    self.assertEqualDiff(source.get(path).read(),
-                                         target.get(path).read(),
-                                         "text for file %r differs:\n" % path)
-
-    def assertRepositoryHasSameItems(self, left_repo, right_repo):
-        """require left_repo and right_repo to contain the same data."""
-        # XXX: TODO: Doesn't work yet, because we need to be able to compare
-        # local repositories to remote ones...  but this is an as-yet unsolved
-        # aspect of format management and the Remote protocols...
-        # self.assertEqual(left_repo._format.__class__,
-        #     right_repo._format.__class__)
-        left_repo.lock_read()
-        try:
-            right_repo.lock_read()
-            try:
-                # revs
-                all_revs = left_repo.all_revision_ids()
-                self.assertEqual(left_repo.all_revision_ids(),
-                    right_repo.all_revision_ids())
-                for rev_id in left_repo.all_revision_ids():
-                    self.assertEqual(left_repo.get_revision(rev_id),
-                        right_repo.get_revision(rev_id))
-                # Assert the revision trees (and thus the inventories) are equal
-                sort_key = lambda rev_tree: rev_tree.get_revision_id()
-                rev_trees_a = sorted(
-                    left_repo.revision_trees(all_revs), key=sort_key)
-                rev_trees_b = sorted(
-                    right_repo.revision_trees(all_revs), key=sort_key)
-                for tree_a, tree_b in zip(rev_trees_a, rev_trees_b):
-                    self.assertEqual([], list(tree_a.iter_changes(tree_b)))
-                # texts
-                text_index = left_repo._generate_text_key_index()
-                self.assertEqual(text_index,
-                    right_repo._generate_text_key_index())
-                desired_files = []
-                for file_id, revision_id in text_index.iterkeys():
-                    desired_files.append(
-                        (file_id, revision_id, (file_id, revision_id)))
-                left_texts = list(left_repo.iter_files_bytes(desired_files))
-                right_texts = list(right_repo.iter_files_bytes(desired_files))
-                left_texts.sort()
-                right_texts.sort()
-                self.assertEqual(left_texts, right_texts)
-                # signatures
-                for rev_id in all_revs:
-                    try:
-                        left_text = left_repo.get_signature_text(rev_id)
-                    except NoSuchRevision:
-                        continue
-                    right_text = right_repo.get_signature_text(rev_id)
-                    self.assertEqual(left_text, right_text)
-            finally:
-                right_repo.unlock()
-        finally:
-            left_repo.unlock()
+from bzrlib.remote import (
+    RemoteBzrDir,
+    RemoteBzrDirFormat,
+    RemoteRepository,
+    )
+
+
+class TestControlDir(TestCaseWithControlDir):
 
     def skipIfNoWorkingTree(self, a_bzrdir):
         """Raises TestSkipped if a_bzrdir doesn't have a working tree.
@@ -189,7 +79,7 @@
         """
         try:
             return a_bzrdir.create_workingtree()
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("cannot make working tree with transport %r"
                               % a_bzrdir.transport)
 
@@ -211,13 +101,20 @@
                                     create_tree_if_local=create_tree_if_local)
         return target
 
+    def test_uninitializable(self):
+        if self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is initializable")
+        t = self.get_transport()
+        self.assertRaises(errors.UninitializableFormat,
+            self.bzrdir_format.initialize, t.base)
+
     def test_create_null_workingtree(self):
         dir = self.make_bzrdir('dir1')
         dir.create_repository()
         dir.create_branch()
         try:
             wt = dir.create_workingtree(revision_id=bzrlib.revision.NULL_REVISION)
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("cannot make working tree with transport %r"
                               % dir.transport)
         self.assertEqual([], wt.get_parent_ids())
@@ -232,12 +129,12 @@
             bzrdir.destroy_workingtree()
         except errors.UnsupportedOperation:
             raise TestSkipped('Format does not support destroying tree')
-        self.failIfExists('tree/file')
+        self.assertPathDoesNotExist('tree/file')
         self.assertRaises(errors.NoWorkingTree, bzrdir.open_workingtree)
         bzrdir.create_workingtree()
-        self.failUnlessExists('tree/file')
+        self.assertPathExists('tree/file')
         bzrdir.destroy_workingtree_metadata()
-        self.failUnlessExists('tree/file')
+        self.assertPathExists('tree/file')
         self.assertRaises(errors.NoWorkingTree, bzrdir.open_workingtree)
 
     def test_destroy_branch(self):
@@ -264,60 +161,18 @@
         bzrdir.open_repository()
 
     def test_open_workingtree_raises_no_working_tree(self):
-        """BzrDir.open_workingtree() should raise NoWorkingTree (rather than
+        """ControlDir.open_workingtree() should raise NoWorkingTree (rather than
         e.g. NotLocalUrl) if there is no working tree.
         """
         dir = self.make_bzrdir('source')
         vfs_dir = bzrdir.BzrDir.open(self.get_vfs_only_url('source'))
         if vfs_dir.has_workingtree():
-            # This BzrDir format doesn't support BzrDirs without working trees,
-            # so this test is irrelevant.
-            return
+            # This ControlDir format doesn't support ControlDirs without
+            # working trees, so this test is irrelevant.
+            raise TestNotApplicable("format does not support "
+                "control directories without working tree")
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
 
-    def test_clone_on_transport(self):
-        a_dir = self.make_bzrdir('source')
-        target_transport = a_dir.root_transport.clone('..').clone('target')
-        target = a_dir.clone_on_transport(target_transport)
-        self.assertNotEqual(a_dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(a_dir.root_transport, target.root_transport,
-                                    ['./.bzr/merge-hashes'])
-
-    def test_clone_bzrdir_empty(self):
-        dir = self.make_bzrdir('source')
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/merge-hashes'])
-
-    def test_clone_bzrdir_empty_force_new_ignored(self):
-        # the force_new_repo parameter should have no effect on an empty
-        # bzrdir's clone logic
-        dir = self.make_bzrdir('source')
-        target = dir.clone(self.get_url('target'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/merge-hashes'])
-
-    def test_clone_bzrdir_repository(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
-        tree.add('foo')
-        tree.commit('revision 1', rev_id='1')
-        dir = self.make_bzrdir('source')
-        repo = dir.create_repository()
-        repo.fetch(tree.branch.repository)
-        self.assertTrue(repo.has_revision('1'))
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-
     def test_clone_bzrdir_repository_under_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
@@ -325,12 +180,16 @@
         tree.commit('revision 1', rev_id='1')
         dir = self.make_bzrdir('source')
         repo = dir.create_repository()
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("repository format does not support "
+                "nesting")
         repo.fetch(tree.branch.repository)
         self.assertTrue(repo.has_revision('1'))
         try:
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
         target = dir.clone(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
@@ -340,14 +199,19 @@
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         # Make a branch, 'commit_tree', and working tree outside of the shared
         # repository, and commit some revisions to it.
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['foo'], transport=tree.bzrdir.root_transport)
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.bzrdir.open_branch().set_revision_history([])
+        tree.bzrdir.open_branch().generate_revision_history(
+            bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         # Copy the content (i.e. revisions) from the 'commit_tree' branch's
@@ -370,12 +234,17 @@
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.branch.bzrdir.open_branch().set_revision_history([])
+        tree.branch.bzrdir.open_branch().generate_revision_history(
+            bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         tree.branch.repository.copy_content_into(shared_repo)
@@ -393,26 +262,6 @@
         self.assertFalse(branch.repository.make_working_trees())
         self.assertTrue(branch.repository.is_shared())
 
-    def test_clone_bzrdir_repository_under_shared_force_new_repo(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1', rev_id='1')
-        dir = self.make_bzrdir('source')
-        repo = dir.create_repository()
-        repo.fetch(tree.branch.repository)
-        self.assertTrue(repo.has_revision('1'))
-        try:
-            self.make_repository('target', shared=True)
-        except errors.IncompatibleFormat:
-            return
-        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
-
     def test_clone_bzrdir_repository_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a repository with some revisions,
@@ -422,7 +271,8 @@
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.branch.bzrdir.open_branch().set_revision_history([])
+        tree.branch.bzrdir.open_branch().generate_revision_history(
+            bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         source = self.make_repository('source')
@@ -433,7 +283,7 @@
 
     def test_clone_bzrdir_branch_and_repo_fixed_user_id(self):
         # Bug #430868 is about an email containing '.sig'
-        os.environ['BZR_EMAIL'] = 'murphy@host.sighup.org'
+        self.overrideEnv('BZR_EMAIL', 'murphy@host.sighup.org')
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -452,28 +302,6 @@
             tree_repo.get_signature_text(rev1),
             target.repository.get_signature_text(rev1))
 
-    def test_clone_bzrdir_branch_and_repo(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        source = self.make_branch('source')
-        tree.branch.repository.copy_content_into(source.repository)
-        tree.branch.copy_content_into(source)
-        dir = source.bzrdir
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/basis-inventory-cache',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     './.bzr/stat-cache',
-                                    ])
-        self.assertRepositoryHasSameItems(
-            tree.branch.repository, target.open_repository())
-
     def test_clone_bzrdir_branch_and_repo_into_shared_repo(self):
         # by default cloning into a shared repo uses the shared repo.
         tree = self.make_branch_and_tree('commit_tree')
@@ -484,51 +312,19 @@
         tree.branch.repository.copy_content_into(source.repository)
         tree.branch.copy_content_into(source)
         try:
-            self.make_repository('target', shared=True)
+            shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         dir = source.bzrdir
         target = dir.clone(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
-        self.assertEqual(source.revision_history(),
-                         target.open_branch().revision_history())
-
-    def test_clone_bzrdir_branch_and_repo_into_shared_repo_force_new_repo(self):
-        # by default cloning into a shared repo uses the shared repo.
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        source = self.make_branch('source')
-        tree.branch.repository.copy_content_into(source.repository)
-        tree.branch.copy_content_into(source)
-        try:
-            self.make_repository('target', shared=True)
-        except errors.IncompatibleFormat:
-            return
-        dir = source.bzrdir
-        target = dir.clone(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        repo = target.open_repository()
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository, repo)
-
-    def test_clone_bzrdir_branch_reference(self):
-        # cloning should preserve the reference status of the branch in a bzrdir
-        referenced_branch = self.make_branch('referencced')
-        dir = self.make_bzrdir('source')
-        try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
-        except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
-        target = dir.clone(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport)
+        self.assertEqual(source.last_revision(),
+                         target.open_branch().last_revision())
 
     def test_clone_bzrdir_branch_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
@@ -547,27 +343,6 @@
         target = dir.clone(self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_branch().last_revision())
 
-    def test_clone_bzrdir_tree_branch_repo(self):
-        tree = self.make_branch_and_tree('source')
-        self.build_tree(['source/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        dir = tree.bzrdir
-        target = dir.clone(self.get_url('target'))
-        self.skipIfNoWorkingTree(target)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-        target.open_workingtree().revert()
-
     def test_clone_on_transport_preserves_repo_format(self):
         if self.bzrdir_format == bzrdir.format_registry.make_bzrdir('default'):
             format = 'knit'
@@ -587,60 +362,6 @@
             target_repo = target_repo._real_repository
         self.assertEqual(target_repo._format, source_branch.repository._format)
 
-    def test_revert_inventory(self):
-        tree = self.make_branch_and_tree('source')
-        self.build_tree(['source/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        dir = tree.bzrdir
-        target = dir.clone(self.get_url('target'))
-        self.skipIfNoWorkingTree(target)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-
-        target.open_workingtree().revert()
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository',
-                                     ])
-        self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
-
-    def test_clone_bzrdir_tree_branch_reference(self):
-        # a tree with a branch reference (aka a checkout)
-        # should stay a checkout on clone.
-        referenced_branch = self.make_branch('referencced')
-        dir = self.make_bzrdir('source')
-        try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
-        except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
-        self.createWorkingTreeOrSkip(dir)
-        target = dir.clone(self.get_url('target'))
-        self.skipIfNoWorkingTree(target)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    ['./.bzr/stat-cache',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/merge-hashes',
-                                     './.bzr/merge-hashes',
-                                     './.bzr/repository/inventory.knit',
-                                     ])
-
     def test_clone_bzrdir_tree_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
         # make a tree with a revision with a last-revision
@@ -672,16 +393,24 @@
             repo.set_make_working_trees(False)
             self.assertFalse(repo.make_working_trees())
 
-        dir = tree.bzrdir
-        a_dir = dir.clone(self.get_url('repo/a'))
-        a_dir.open_branch()
+        a_dir = tree.bzrdir.clone(self.get_url('repo/a'))
+        a_branch = a_dir.open_branch()
+        # If the new control dir actually uses the repository, it should
+        # not have a working tree.
+        if not a_branch.repository.has_same_location(repo):
+            raise TestNotApplicable('new control dir does not use repository')
         self.assertRaises(errors.NoWorkingTree, a_dir.open_workingtree)
 
     def test_clone_respects_stacked(self):
         branch = self.make_branch('parent')
         child_transport = self.get_transport('child')
-        child = branch.bzrdir.clone_on_transport(child_transport,
-                                                 stacked_on=branch.base)
+        try:
+            child = branch.bzrdir.clone_on_transport(child_transport,
+                                                     stacked_on=branch.base)
+        except (errors.UnstackableBranchFormat,
+                errors.UnstackableRepositoryFormat):
+            raise TestNotApplicable("branch or repository format does "
+                "not support stacking")
         self.assertEqual(child.open_branch().get_stacked_on_url(), branch.base)
 
     def test_get_branch_reference_on_reference(self):
@@ -693,7 +422,8 @@
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("control directory does not "
+                "support branch references")
         self.assertEqual(referenced_branch.bzrdir.root_transport.abspath('') + '/',
             dir.get_branch_reference())
 
@@ -707,13 +437,14 @@
         dir = self.make_bzrdir('source')
         if dir.has_branch():
             # this format does not support branchless bzrdirs.
-            return
+            raise TestNotApplicable("format does not support "
+                "branchless control directories")
         self.assertRaises(errors.NotBranchError, dir.get_branch_reference)
 
     def test_sprout_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
         target = dir.sprout(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.control_transport.base, target.control_transport.base)
         # creates a new repository branch and tree
         target.open_repository()
         target.open_branch()
@@ -725,16 +456,17 @@
         try:
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'))
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
         target.open_branch()
         try:
             target.open_workingtree()
         except errors.NoWorkingTree:
-            # bzrdir's that never have working trees are allowed to pass;
-            # whitelist them for now.
-            self.assertIsInstance(target, RemoteBzrDir)
+            # Some bzrdirs can never have working trees.
+            repo = target.find_repository()
+            self.assertFalse(repo.bzrdir._format.supports_workingtrees)
 
     def test_sprout_bzrdir_empty_under_shared_repo_force_new(self):
         # the force_new_repo parameter should force use of a new repo in an empty
@@ -743,59 +475,20 @@
         try:
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         target.open_repository()
         target.open_branch()
         self.openWorkingTreeIfLocal(target)
 
-    def test_sprout_bzrdir_repository(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
-        tree.add('foo')
-        tree.commit('revision 1', rev_id='1')
-        dir = self.make_bzrdir('source')
-        repo = dir.create_repository()
-        repo.fetch(tree.branch.repository)
-        self.assertTrue(repo.has_revision('1'))
-        try:
-            self.assertTrue(
-                _mod_revision.is_null(_mod_revision.ensure_null(
-                dir.open_branch().last_revision())))
-        except errors.NotBranchError:
-            pass
-        target = dir.sprout(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        # testing inventory isn't reasonable for repositories
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/branch',
-                                     './.bzr/checkout',
-                                     './.bzr/inventory',
-                                     './.bzr/parent',
-                                     './.bzr/repository/inventory.knit',
-                                     ])
-        try:
-            local_inventory = dir.transport.local_abspath('inventory')
-        except errors.NotLocalUrl:
-            return
-        try:
-            # If we happen to have a tree, we'll guarantee everything
-            # except for the tree root is the same.
-            inventory_f = file(local_inventory, 'rb')
-            self.addCleanup(inventory_f.close)
-            self.assertContainsRe(inventory_f.read(),
-                                  '<inventory format="5">\n</inventory>\n')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-
     def test_sprout_bzrdir_with_repository_to_shared(self):
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.bzrdir.open_branch().set_revision_history([])
+        tree.bzrdir.open_branch().generate_revision_history(
+            bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         source = self.make_repository('source')
@@ -804,21 +497,27 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support "
+                "shared repositories")
         target = dir.sprout(self.get_url('target/child'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.user_transport.base, target.user_transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_repository_branch_both_under_shared(self):
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.bzrdir.open_branch().set_revision_history([])
+        tree.bzrdir.open_branch().generate_revision_history(
+            bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         tree.branch.repository.copy_content_into(shared_repo)
@@ -833,12 +532,17 @@
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.bzrdir.open_branch().set_revision_history([])
+        tree.bzrdir.open_branch().generate_revision_history(
+            bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         tree.branch.repository.copy_content_into(shared_repo)
@@ -852,8 +556,11 @@
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertNotEqual(dir.transport.base, shared_repo.bzrdir.transport.base)
         branch = target.open_branch()
-        self.assertTrue(branch.repository.has_revision('1'))
-        if not isinstance(branch.bzrdir, RemoteBzrDir):
+        # The sprouted bzrdir has a branch, so only revisions referenced by
+        # that branch are copied, rather than the whole repository.  It's an
+        # empty branch, so none are copied.
+        self.assertEqual([], branch.repository.all_revision_ids())
+        if branch.bzrdir._format.supports_workingtrees:
             self.assertTrue(branch.repository.make_working_trees())
         self.assertFalse(branch.repository.is_shared())
 
@@ -862,7 +569,8 @@
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.bzrdir.open_branch().set_revision_history([])
+        tree.bzrdir.open_branch().generate_revision_history(
+            bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         source = self.make_repository('source')
@@ -871,9 +579,12 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(
+            dir.control_transport.base,
+            target.control_transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_repository_revision(self):
@@ -885,7 +596,8 @@
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
         tree.commit('revision 1', rev_id='1')
-        tree.bzrdir.open_branch().set_revision_history([])
+        br = tree.bzrdir.open_branch()
+        br.set_last_revision_info(0, bzrlib.revision.NULL_REVISION)
         tree.set_parent_trees([])
         tree.commit('revision 2', rev_id='2')
         source = self.make_repository('source')
@@ -894,34 +606,6 @@
         target = self.sproutOrSkip(dir, self.get_url('target'), revision_id='2')
         raise TestSkipped('revision limiting not strict yet')
 
-    def test_sprout_bzrdir_branch_and_repo(self):
-        tree = self.make_branch_and_tree('commit_tree')
-        self.build_tree(['commit_tree/foo'])
-        tree.add('foo')
-        tree.commit('revision 1')
-        source = self.make_branch('source')
-        tree.branch.repository.copy_content_into(source.repository)
-        tree.bzrdir.open_branch().copy_content_into(source)
-        dir = source.bzrdir
-        target = dir.sprout(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        target_repo = target.open_repository()
-        self.assertRepositoryHasSameItems(source.repository, target_repo)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/basis-inventory-cache',
-                                     './.bzr/branch/branch.conf',
-                                     './.bzr/branch/parent',
-                                     './.bzr/checkout',
-                                     './.bzr/checkout/inventory',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/inventory',
-                                     './.bzr/parent',
-                                     './.bzr/repository',
-                                     './.bzr/stat-cache',
-                                     './foo',
-                                     ])
-
     def test_sprout_bzrdir_branch_and_repo_shared(self):
         # sprouting a branch with a repo into a shared repo uses the shared
         # repo
@@ -936,7 +620,8 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'))
         self.assertTrue(shared_repo.has_revision('1'))
 
@@ -954,9 +639,10 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.control_transport.base, target.control_transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_branch_reference(self):
@@ -967,8 +653,8 @@
             reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support branch "
+                "references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
@@ -987,13 +673,14 @@
             reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
                 target_branch=referenced_tree.branch)
         except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support branch "
+                "references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support "
+                "shared repositories")
         target = dir.sprout(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
@@ -1014,12 +701,14 @@
                 target_branch=referenced_tree.branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
@@ -1046,28 +735,113 @@
         target = dir.sprout(self.get_url('target'), revision_id='1')
         self.assertEqual('1', target.open_branch().last_revision())
 
-    def test_sprout_bzrdir_tree_branch_repo(self):
-        tree = self.make_branch_and_tree('source')
-        self.build_tree(['foo'], transport=tree.bzrdir.transport.clone('..'))
-        tree.add('foo')
-        tree.commit('revision 1')
-        dir = tree.bzrdir
-        target = self.sproutOrSkip(dir, self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
-        self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
-                                    [
-                                     './.bzr/branch/branch.conf',
-                                     './.bzr/branch/parent',
-                                     './.bzr/checkout/dirstate',
-                                     './.bzr/checkout/stat-cache',
-                                     './.bzr/checkout/inventory',
-                                     './.bzr/inventory',
-                                     './.bzr/parent',
-                                     './.bzr/repository',
-                                     './.bzr/stat-cache',
-                                     ])
-        self.assertRepositoryHasSameItems(
-            tree.branch.repository, target.open_repository())
+    def test_sprout_bzrdir_branch_with_tags(self):
+        # when sprouting a branch all revisions named in the tags are copied
+        # too.
+        builder = self.make_branch_builder('source')
+        source = fixtures.build_branch_with_non_ancestral_rev(builder)
+        try:
+            source.tags.set_tag('tag-a', 'rev-2')
+        except errors.TagsNotSupported:
+            raise TestNotApplicable('Branch format does not support tags.')
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        # Now source has a tag not in its ancestry.  Sprout its controldir.
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'))
+        # The tag is present, and so is its revision.
+        new_branch = target.open_branch()
+        self.assertEqual('rev-2', new_branch.tags.lookup_tag('tag-a'))
+        new_branch.repository.get_revision('rev-2')
+
+    def test_sprout_bzrdir_branch_with_absent_tag(self):
+        # tags referencing absent revisions are copied (and those absent
+        # revisions do not prevent the sprout.)
+        builder = self.make_branch_builder('source')
+        builder.build_commit(message="Rev 1", rev_id='rev-1')
+        source = builder.get_branch()
+        try:
+            source.tags.set_tag('tag-a', 'missing-rev')
+        except (errors.TagsNotSupported, errors.GhostTagsNotSupported):
+            raise TestNotApplicable('Branch format does not support tags '
+                'or tags referencing ghost revisions.')
+        # Now source has a tag pointing to an absent revision.  Sprout its
+        # controldir.
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'))
+        # The tag is present in the target
+        new_branch = target.open_branch()
+        self.assertEqual('missing-rev', new_branch.tags.lookup_tag('tag-a'))
+
+    def test_sprout_bzrdir_passing_source_branch_with_absent_tag(self):
+        # tags referencing absent revisions are copied (and those absent
+        # revisions do not prevent the sprout.)
+        builder = self.make_branch_builder('source')
+        builder.build_commit(message="Rev 1", rev_id='rev-1')
+        source = builder.get_branch()
+        try:
+            source.tags.set_tag('tag-a', 'missing-rev')
+        except (errors.TagsNotSupported, errors.GhostTagsNotSupported):
+            raise TestNotApplicable('Branch format does not support tags '
+                'or tags referencing missing revisions.')
+        # Now source has a tag pointing to an absent revision.  Sprout its
+        # controldir.
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'), source_branch=source)
+        # The tag is present in the target
+        new_branch = target.open_branch()
+        self.assertEqual('missing-rev', new_branch.tags.lookup_tag('tag-a'))
+
+    def test_sprout_bzrdir_passing_rev_not_source_branch_copies_tags(self):
+        # dir.sprout(..., revision_id='rev1') copies rev1, and all the tags of
+        # the branch at that bzrdir, the ancestry of all of those, but no other
+        # revs (not even the tip of the source branch).
+        builder = self.make_branch_builder('source')
+        builder.build_commit(message="Base", rev_id='base-rev')
+        # Make three parallel lines of ancestry off this base.
+        source = builder.get_branch()
+        builder.build_commit(message="Rev A1", rev_id='rev-a1')
+        builder.build_commit(message="Rev A2", rev_id='rev-a2')
+        builder.build_commit(message="Rev A3", rev_id='rev-a3')
+        source.set_last_revision_info(1, 'base-rev')
+        builder.build_commit(message="Rev B1", rev_id='rev-b1')
+        builder.build_commit(message="Rev B2", rev_id='rev-b2')
+        builder.build_commit(message="Rev B3", rev_id='rev-b3')
+        source.set_last_revision_info(1, 'base-rev')
+        builder.build_commit(message="Rev C1", rev_id='rev-c1')
+        builder.build_commit(message="Rev C2", rev_id='rev-c2')
+        builder.build_commit(message="Rev C3", rev_id='rev-c3')
+        # Set the branch tip to A2
+        source.set_last_revision_info(3, 'rev-a2')
+        try:
+            # Create a tag for B2, and for an absent rev
+            source.tags.set_tag('tag-non-ancestry', 'rev-b2')
+        except errors.TagsNotSupported:
+            raise TestNotApplicable('Branch format does not support tags ')
+        try:
+            source.tags.set_tag('tag-absent', 'absent-rev')
+        except errors.GhostTagsNotSupported:
+            has_ghost_tag = False
+        else:
+            has_ghost_tag = True
+        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        # And ask sprout for C2
+        dir = source.bzrdir
+        target = dir.sprout(self.get_url('target'), revision_id='rev-c2')
+        # The tags are present
+        new_branch = target.open_branch()
+        if has_ghost_tag:
+            self.assertEqual(
+                {'tag-absent': 'absent-rev', 'tag-non-ancestry': 'rev-b2'},
+                new_branch.tags.get_tag_dict())
+        else:
+            self.assertEqual(
+                {'tag-non-ancestry': 'rev-b2'},
+                new_branch.tags.get_tag_dict())
+        # And the revs for A2, B2 and C2's ancestries are present, but no
+        # others.
+        self.assertEqual(
+            ['base-rev', 'rev-b1', 'rev-b2', 'rev-c1', 'rev-c2'],
+            sorted(new_branch.repository.all_revision_ids()))
 
     def test_sprout_bzrdir_tree_branch_reference(self):
         # sprouting should create a repository if needed and a sprouted branch.
@@ -1079,7 +853,8 @@
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         tree = self.createWorkingTreeOrSkip(dir)
         self.build_tree(['source/subdir/'])
@@ -1105,7 +880,8 @@
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         tree = self.createWorkingTreeOrSkip(dir)
         self.build_tree(['source/foo'])
@@ -1157,13 +933,12 @@
         tree.commit('revision 1', rev_id='1')
         tree.commit('revision 2', rev_id='2', allow_pointless=True)
         dir = tree.bzrdir
-        if isinstance(dir, (bzrdir.BzrDirPreSplitOut,)):
-            self.assertRaises(errors.MustHaveWorkingTree, dir.sprout,
-                              self.get_url('target'),
-                              create_tree_if_local=False)
-            return
-        target = dir.sprout(self.get_url('target'), create_tree_if_local=False)
-        self.failIfExists('target/foo')
+        try:
+            target = dir.sprout(self.get_url('target'),
+                create_tree_if_local=False)
+        except errors.MustHaveWorkingTree:
+            raise TestNotApplicable("control dir format requires working tree")
+        self.assertPathDoesNotExist('target/foo')
         self.assertEqual(tree.branch.last_revision(),
                          target.open_branch().last_revision())
 
@@ -1192,11 +967,11 @@
 
     def test_format_initialize_find_open(self):
         # loopback test to check the current format initializes to itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         # for remote formats, there must be no prior assumption about the
         # network name to use - it's possible that this may somehow have got
         # in through an unisolated test though - see
@@ -1205,19 +980,21 @@
             '_network_name', None),
             None)
         # supported formats must be able to init and open
-        t = transport.get_transport(self.get_url())
-        readonly_t = transport.get_transport(self.get_readonly_url())
+        t = self.get_transport()
+        readonly_t = self.get_readonly_transport()
         made_control = self.bzrdir_format.initialize(t.base)
-        self.failUnless(isinstance(made_control, bzrdir.BzrDir))
+        self.assertIsInstance(made_control, controldir.ControlDir)
+        if isinstance(self.bzrdir_format, RemoteBzrDirFormat):
+            return
         self.assertEqual(self.bzrdir_format,
-                         bzrdir.BzrDirFormat.find_format(readonly_t))
+                         controldir.ControlDirFormat.find_format(readonly_t))
         direct_opened_dir = self.bzrdir_format.open(readonly_t)
         opened_dir = bzrdir.BzrDir.open(t.base)
         self.assertEqual(made_control._format,
                          opened_dir._format)
         self.assertEqual(direct_opened_dir._format,
                          opened_dir._format)
-        self.failUnless(isinstance(opened_dir, bzrdir.BzrDir))
+        self.assertIsInstance(opened_dir, controldir.ControlDir)
 
     def test_format_initialize_on_transport_ex(self):
         t = self.get_transport('dir')
@@ -1229,9 +1006,8 @@
         self.assertInitializeEx(t, use_existing_dir=True)
 
     def test_format_initialize_on_transport_ex_use_existing_dir_False(self):
-        if not self.bzrdir_format.is_supported():
-            # Not initializable - not a failure either.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport('dir')
         t.ensure_base()
         self.assertRaises(errors.FileExists,
@@ -1243,9 +1019,8 @@
         self.assertInitializeEx(t, create_prefix=True)
 
     def test_format_initialize_on_transport_ex_create_prefix_False(self):
-        if not self.bzrdir_format.is_supported():
-            # Not initializable - not a failure either.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport('missing/dir')
         self.assertRaises(errors.NoSuchFile, self.assertInitializeEx, t,
             create_prefix=False)
@@ -1258,9 +1033,6 @@
             repo_format_name=repo_name, shared_repo=True)[0]
         made_repo, control = self.assertInitializeEx(t.clone('branch'),
             force_new_repo=True, repo_format_name=repo_name)
-        if control is None:
-            # uninitialisable format
-            return
         self.assertNotEqual(repo.bzrdir.root_transport.base,
             made_repo.bzrdir.root_transport.base)
 
@@ -1272,64 +1044,10 @@
             repo_format_name=repo_name, shared_repo=True)[0]
         made_repo, control = self.assertInitializeEx(t.clone('branch'),
             force_new_repo=False, repo_format_name=repo_name)
-        if control is None:
-            # uninitialisable format
-            return
-        if not isinstance(control._format, (bzrdir.BzrDirFormat5,
-            bzrdir.BzrDirFormat6,)):
+        if not control._format.fixed_components:
             self.assertEqual(repo.bzrdir.root_transport.base,
                 made_repo.bzrdir.root_transport.base)
 
-    def test_format_initialize_on_transport_ex_stacked_on(self):
-        # trunk is a stackable format.  Note that its in the same server area
-        # which is what launchpad does, but not sufficient to exercise the
-        # general case.
-        trunk = self.make_branch('trunk', format='1.9')
-        t = self.get_transport('stacked')
-        old_fmt = bzrdir.format_registry.make_bzrdir('pack-0.92')
-        repo_name = old_fmt.repository_format.network_name()
-        # Should end up with a 1.9 format (stackable)
-        repo, control = self.assertInitializeEx(t, need_meta=True,
-            repo_format_name=repo_name, stacked_on='../trunk', stack_on_pwd=t.base)
-        if control is None:
-            # uninitialisable format
-            return
-        self.assertLength(1, repo._fallback_repositories)
-
-    def test_format_initialize_on_transport_ex_default_stack_on(self):
-        # When initialize_on_transport_ex uses a stacked-on branch because of
-        # a stacking policy on the target, the location of the fallback
-        # repository is the same as the external location of the stacked-on
-        # branch.
-        balloon = self.make_bzrdir('balloon')
-        if isinstance(balloon._format, bzrdir.BzrDirMetaFormat1):
-            stack_on = self.make_branch('stack-on', format='1.9')
-        else:
-            stack_on = self.make_branch('stack-on')
-        config = self.make_bzrdir('.').get_config()
-        try:
-            config.set_default_stack_on('stack-on')
-        except errors.BzrError:
-            raise TestNotApplicable('Only relevant for stackable formats.')
-        # Initialize a bzrdir subject to the policy.
-        t = self.get_transport('stacked')
-        repo_fmt = bzrdir.format_registry.make_bzrdir('1.9')
-        repo_name = repo_fmt.repository_format.network_name()
-        repo, control = self.assertInitializeEx(
-            t, need_meta=True, repo_format_name=repo_name, stacked_on=None)
-        # self.addCleanup(repo.unlock)
-        if control is None:
-            # uninitialisable format
-            return
-        # There's one fallback repo, with a public location.
-        self.assertLength(1, repo._fallback_repositories)
-        fallback_repo = repo._fallback_repositories[0]
-        self.assertEqual(
-            stack_on.base, fallback_repo.bzrdir.root_transport.base)
-        # The bzrdir creates a branch in stacking-capable format.
-        new_branch = control.create_branch()
-        self.assertTrue(new_branch._format.supports_stacking())
-
     def test_format_initialize_on_transport_ex_repo_fmt_name_None(self):
         t = self.get_transport('dir')
         repo, control = self.assertInitializeEx(t)
@@ -1341,16 +1059,12 @@
         fmt = bzrdir.format_registry.make_bzrdir('1.6')
         repo_name = fmt.repository_format.network_name()
         repo, control = self.assertInitializeEx(t, repo_format_name=repo_name)
-        if control is None:
-            # uninitialisable format
-            return
-        if isinstance(self.bzrdir_format, (bzrdir.BzrDirFormat5,
-            bzrdir.BzrDirFormat6)):
+        if self.bzrdir_format.fixed_components:
             # must stay with the all-in-one-format.
             repo_name = self.bzrdir_format.network_name()
         self.assertEqual(repo_name, repo._format.network_name())
 
-    def assertInitializeEx(self, t, need_meta=False, **kwargs):
+    def assertInitializeEx(self, t, **kwargs):
         """Execute initialize_on_transport_ex and check it succeeded correctly.
 
         This involves checking that the disk objects were created, open with
@@ -1361,28 +1075,19 @@
             initialize_on_transport_ex.
         :return: the resulting repo, control dir tuple.
         """
-        if not self.bzrdir_format.is_supported():
-            # Not initializable - not a failure either.
-            return None, None
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("control dir format is not "
+                "initializable")
         repo, control, require_stacking, repo_policy = \
             self.bzrdir_format.initialize_on_transport_ex(t, **kwargs)
         if repo is not None:
             # Repositories are open write-locked
             self.assertTrue(repo.is_write_locked())
             self.addCleanup(repo.unlock)
-        self.assertIsInstance(control, bzrdir.BzrDir)
+        self.assertIsInstance(control, controldir.ControlDir)
         opened = bzrdir.BzrDir.open(t.base)
         expected_format = self.bzrdir_format
-        if isinstance(expected_format, bzrdir.RemoteBzrDirFormat):
-            # Current RemoteBzrDirFormat's do not reliably get network_name
-            # set, so we skip a number of tests for RemoteBzrDirFormat's.
-            self.assertIsInstance(control, RemoteBzrDir)
-        else:
-            if need_meta and isinstance(expected_format, (bzrdir.BzrDirFormat5,
-                bzrdir.BzrDirFormat6)):
-                # Pre-metadir formats change when we are making something that
-                # needs a metaformat, because clone is used for push.
-                expected_format = bzrdir.BzrDirMetaFormat1()
+        if not isinstance(expected_format, RemoteBzrDirFormat):
             self.assertEqual(control._format.network_name(),
                 expected_format.network_name())
             self.assertEqual(control._format.network_name(),
@@ -1399,61 +1104,79 @@
         # key in the registry gives back the same format. For remote obects
         # we check that the network_name of the RemoteBzrDirFormat we have
         # locally matches the actual format present on disk.
-        if isinstance(format, bzrdir.RemoteBzrDirFormat):
+        if isinstance(format, RemoteBzrDirFormat):
             dir._ensure_real()
             real_dir = dir._real_bzrdir
             network_name = format.network_name()
             self.assertEqual(real_dir._format.network_name(), network_name)
         else:
-            registry = bzrdir.network_format_registry
+            registry = controldir.network_format_registry
             network_name = format.network_name()
             looked_up_format = registry.get(network_name)
-            self.assertEqual(format.__class__, looked_up_format.__class__)
+            self.assertTrue(
+                issubclass(format.__class__, looked_up_format.__class__))
         # The network name must be a byte string.
         self.assertIsInstance(network_name, str)
 
     def test_open_not_bzrdir(self):
         # test the formats specific behaviour for no-content or similar dirs.
-        self.assertRaises(NotBranchError,
+        self.assertRaises(errors.NotBranchError,
                           self.bzrdir_format.open,
-                          transport.get_transport(self.get_readonly_url()))
+                          transport.get_transport_from_url(self.get_readonly_url()))
 
     def test_create_branch(self):
         # a bzrdir can construct a branch and repository for itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
-        t = transport.get_transport(self.get_url())
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
-        self.failUnless(isinstance(made_branch, bzrlib.branch.Branch))
+        self.assertIsInstance(made_branch, bzrlib.branch.Branch)
+        self.assertEqual(made_control, made_branch.bzrdir)
+
+    def test_create_branch_append_revisions_only(self):
+        # a bzrdir can construct a branch and repository for itself.
+        if not self.bzrdir_format.is_initializable():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        try:
+            made_branch = made_control.create_branch(
+                append_revisions_only=True)
+        except errors.UpgradeRequired:
+            raise TestNotApplicable("format does not support "
+                "append_revisions_only setting")
+        self.assertIsInstance(made_branch, bzrlib.branch.Branch)
+        self.assertEquals(True, made_branch.get_append_revisions_only())
         self.assertEqual(made_control, made_branch.bzrdir)
 
     def test_open_branch(self):
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
-        t = transport.get_transport(self.get_url())
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
         opened_branch = made_control.open_branch()
         self.assertEqual(made_control, opened_branch.bzrdir)
-        self.failUnless(isinstance(opened_branch, made_branch.__class__))
-        self.failUnless(isinstance(opened_branch._format, made_branch._format.__class__))
+        self.assertIsInstance(opened_branch, made_branch.__class__)
+        self.assertIsInstance(opened_branch._format, made_branch._format.__class__)
 
     def test_list_branches(self):
-        if not self.bzrdir_format.is_supported():
-            # unsupported formats are not loopback testable
-            # because the default open will not open them and
-            # they may not be initializable.
-            return
-        t = transport.get_transport(self.get_url())
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
@@ -1469,12 +1192,12 @@
 
     def test_create_repository(self):
         # a bzrdir can construct a repository for itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
-        t = transport.get_transport(self.get_url())
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         # Check that we have a repository object.
@@ -1484,64 +1207,73 @@
     def test_create_repository_shared(self):
         # a bzrdir can create a shared repository or
         # fail appropriately
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
-        t = transport.get_transport(self.get_url())
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         try:
             made_repo = made_control.create_repository(shared=True)
         except errors.IncompatibleFormat:
             # Old bzrdir formats don't support shared repositories
             # and should raise IncompatibleFormat
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         self.assertTrue(made_repo.is_shared())
 
     def test_create_repository_nonshared(self):
         # a bzrdir can create a non-shared repository
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
-        t = transport.get_transport(self.get_url())
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
-        made_repo = made_control.create_repository(shared=False)
+        try:
+            made_repo = made_control.create_repository(shared=False)
+        except errors.IncompatibleFormat:
+            # Some control dir formats don't support non-shared repositories
+            # and should raise IncompatibleFormat
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         self.assertFalse(made_repo.is_shared())
 
     def test_open_repository(self):
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
-        t = transport.get_transport(self.get_url())
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         opened_repo = made_control.open_repository()
         self.assertEqual(made_control, opened_repo.bzrdir)
-        self.failUnless(isinstance(opened_repo, made_repo.__class__))
-        self.failUnless(isinstance(opened_repo._format, made_repo._format.__class__))
+        self.assertIsInstance(opened_repo, made_repo.__class__)
+        self.assertIsInstance(opened_repo._format, made_repo._format.__class__)
 
     def test_create_workingtree(self):
         # a bzrdir can construct a working tree for itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch()
         made_tree = self.createWorkingTreeOrSkip(made_control)
-        self.failUnless(isinstance(made_tree, workingtree.WorkingTree))
+        self.assertIsInstance(made_tree, workingtree.WorkingTree)
         self.assertEqual(made_control, made_tree.bzrdir)
 
     def test_create_workingtree_revision(self):
         # a bzrdir can construct a working tree for itself @ a specific revision.
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         source = self.make_branch_and_tree('source')
         source.commit('a', rev_id='a', allow_pointless=True)
@@ -1553,16 +1285,13 @@
         source.branch.clone(made_control)
         try:
             made_tree = made_control.create_workingtree(revision_id='a')
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("Can't make working tree on transport %r" % t)
         self.assertEqual(['a'], made_tree.get_parent_ids())
 
     def test_open_workingtree(self):
-        if not self.bzrdir_format.is_supported():
-            # unsupported formats are not loopback testable
-            # because the default open will not open them and
-            # they may not be initializable.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         # this has to be tested with local access as we still support creating
         # format 6 bzrdirs
         t = self.get_transport()
@@ -1571,81 +1300,47 @@
             made_repo = made_control.create_repository()
             made_branch = made_control.create_branch()
             made_tree = made_control.create_workingtree()
-        except errors.NotLocalUrl:
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
             raise TestSkipped("Can't initialize %r on transport %r"
                               % (self.bzrdir_format, t))
         opened_tree = made_control.open_workingtree()
         self.assertEqual(made_control, opened_tree.bzrdir)
-        self.failUnless(isinstance(opened_tree, made_tree.__class__))
-        self.failUnless(isinstance(opened_tree._format, made_tree._format.__class__))
-
-    def test_get_branch_transport(self):
-        dir = self.make_bzrdir('.')
-        # without a format, get_branch_transport gives use a transport
-        # which -may- point to an existing dir.
-        self.assertTrue(isinstance(dir.get_branch_transport(None),
-                                   transport.Transport))
-        # with a given format, either the bzr dir supports identifiable
-        # branches, or it supports anonymous  branch formats, but not both.
-        anonymous_format = bzrlib.branch.BzrBranchFormat4()
-        identifiable_format = bzrlib.branch.BzrBranchFormat5()
-        try:
-            found_transport = dir.get_branch_transport(anonymous_format)
-            self.assertRaises(errors.IncompatibleFormat,
-                              dir.get_branch_transport,
-                              identifiable_format)
-        except errors.IncompatibleFormat:
-            found_transport = dir.get_branch_transport(identifiable_format)
-        self.assertTrue(isinstance(found_transport, transport.Transport))
-        # and the dir which has been initialized for us must exist.
-        found_transport.list_dir('.')
-
-    def test_get_repository_transport(self):
-        dir = self.make_bzrdir('.')
-        # without a format, get_repository_transport gives use a transport
-        # which -may- point to an existing dir.
-        self.assertTrue(isinstance(dir.get_repository_transport(None),
-                                   transport.Transport))
-        # with a given format, either the bzr dir supports identifiable
-        # repositories, or it supports anonymous  repository formats, but not both.
-        anonymous_format = weaverepo.RepositoryFormat6()
-        identifiable_format = weaverepo.RepositoryFormat7()
-        try:
-            found_transport = dir.get_repository_transport(anonymous_format)
-            self.assertRaises(errors.IncompatibleFormat,
-                              dir.get_repository_transport,
-                              identifiable_format)
-        except errors.IncompatibleFormat:
-            found_transport = dir.get_repository_transport(identifiable_format)
-        self.assertTrue(isinstance(found_transport, transport.Transport))
-        # and the dir which has been initialized for us must exist.
-        found_transport.list_dir('.')
-
-    def test_get_workingtree_transport(self):
-        dir = self.make_bzrdir('.')
-        # without a format, get_workingtree_transport gives use a transport
-        # which -may- point to an existing dir.
-        self.assertTrue(isinstance(dir.get_workingtree_transport(None),
-                                   transport.Transport))
-        # with a given format, either the bzr dir supports identifiable
-        # trees, or it supports anonymous tree formats, but not both.
-        anonymous_format = workingtree.WorkingTreeFormat2()
-        identifiable_format = workingtree.WorkingTreeFormat3()
-        try:
-            found_transport = dir.get_workingtree_transport(anonymous_format)
-            self.assertRaises(errors.IncompatibleFormat,
-                              dir.get_workingtree_transport,
-                              identifiable_format)
-        except errors.IncompatibleFormat:
-            found_transport = dir.get_workingtree_transport(identifiable_format)
-        self.assertTrue(isinstance(found_transport, transport.Transport))
-        # and the dir which has been initialized for us must exist.
-        found_transport.list_dir('.')
+        self.assertIsInstance(opened_tree, made_tree.__class__)
+        self.assertIsInstance(opened_tree._format, made_tree._format.__class__)
+
+    def test_get_selected_branch(self):
+        # The segment parameters are accessible from the root transport
+        # if a URL with segment parameters is opened.
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
+        try:
+            made_control = self.bzrdir_format.initialize(t.base)
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
+            raise TestSkipped("Can't initialize %r on transport %r"
+                              % (self.bzrdir_format, t))
+        dir = bzrdir.BzrDir.open(t.base+",branch=foo")
+        self.assertEquals({"branch": "foo"},
+            dir.user_transport.get_segment_parameters())
+        self.assertEquals("foo", dir._get_selected_branch())
+
+    def test_get_selected_branch_none_selected(self):
+        # _get_selected_branch defaults to None
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
+        try:
+            made_control = self.bzrdir_format.initialize(t.base)
+        except (errors.NotLocalUrl, errors.UnsupportedOperation):
+            raise TestSkipped("Can't initialize %r on transport %r"
+                              % (self.bzrdir_format, t))
+        dir = bzrdir.BzrDir.open(t.base)
+        self.assertIs(None, dir._get_selected_branch())
 
     def test_root_transport(self):
         dir = self.make_bzrdir('.')
         self.assertEqual(dir.root_transport.base,
-                         transport.get_transport(self.get_url('.')).base)
+                         self.get_transport().base)
 
     def test_find_repository_no_repo_under_standalone_branch(self):
         # finding a repo stops at standalone branches even if there is a
@@ -1654,10 +1349,13 @@
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires shared repository support")
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires nesting repositories")
         url = self.get_url('intermediate')
-        transport.get_transport(self.get_url()).mkdir('intermediate')
-        transport.get_transport(self.get_url()).mkdir('intermediate/child')
+        t = self.get_transport()
+        t.mkdir('intermediate')
+        t.mkdir('intermediate/child')
         made_control = self.bzrdir_format.initialize(url)
         made_control.create_repository()
         innermost_control = self.bzrdir_format.initialize(
@@ -1679,9 +1377,13 @@
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires format with shared repository "
+                "support")
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires support for nesting "
+                "repositories")
         url = self.get_url('childbzrdir')
-        transport.get_transport(self.get_url()).mkdir('childbzrdir')
+        self.get_transport().mkdir('childbzrdir')
         made_control = self.bzrdir_format.initialize(url)
         try:
             child_repo = made_control.open_repository()
@@ -1695,12 +1397,17 @@
                          found_repo.bzrdir.root_transport.base)
 
     def test_find_repository_standalone_with_containing_shared_repository(self):
-        # find repo inside a standalone repo inside a shared repo finds the standalone repo
+        # find repo inside a standalone repo inside a shared repo finds the
+        # standalone repo
         try:
             containing_repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires support for shared "
+                "repositories")
+        if not containing_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support "
+                "nesting repositories")
         child_repo = self.make_repository('childrepo')
         opened_control = bzrdir.BzrDir.open(self.get_url('childrepo'))
         found_repo = opened_control.find_repository()
@@ -1713,9 +1420,13 @@
             containing_repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires support for shared "
+                "repositories")
+        if not containing_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires support for nesting "
+                "repositories")
         url = self.get_url('childrepo')
-        transport.get_transport(self.get_url()).mkdir('childrepo')
+        self.get_transport().mkdir('childrepo')
         child_control = self.bzrdir_format.initialize(url)
         child_repo = child_control.create_repository(shared=True)
         opened_control = bzrdir.BzrDir.open(self.get_url('childrepo'))
@@ -1732,10 +1443,15 @@
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires support for shared "
+                "repositories")
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires support for nesting "
+                "repositories")
         url = self.get_url('intermediate')
-        transport.get_transport(self.get_url()).mkdir('intermediate')
-        transport.get_transport(self.get_url()).mkdir('intermediate/child')
+        t = self.get_transport()
+        t.mkdir('intermediate')
+        t.mkdir('intermediate/child')
         made_control = self.bzrdir_format.initialize(url)
         try:
             child_repo = made_control.open_repository()
@@ -1765,9 +1481,9 @@
             # (we force the latest known format as downgrades may not be
             # available
             self.assertTrue(isinstance(dir._format.get_converter(
-                format=dir._format), bzrdir.Converter))
+                format=dir._format), controldir.Converter))
         dir.needs_format_conversion(
-            bzrdir.BzrDirFormat.get_default_format())
+            controldir.ControlDirFormat.get_default_format())
 
     def test_backup_copies_existing(self):
         tree = self.make_branch_and_tree('test')
@@ -1777,8 +1493,8 @@
         old_url, new_url = tree.bzrdir.backup_bzrdir()
         old_path = urlutils.local_path_from_url(old_url)
         new_path = urlutils.local_path_from_url(new_url)
-        self.failUnlessExists(old_path)
-        self.failUnlessExists(new_path)
+        self.assertPathExists(old_path)
+        self.assertPathExists(new_path)
         for (((dir_relpath1, _), entries1),
              ((dir_relpath2, _), entries2)) in izip(
                 osutils.walkdirs(old_path),
@@ -1812,31 +1528,10 @@
     def test_format_description(self):
         dir = self.make_bzrdir('.')
         text = dir._format.get_format_description()
-        self.failUnless(len(text))
-
-    def test_retire_bzrdir(self):
-        bd = self.make_bzrdir('.')
-        transport = bd.root_transport
-        # must not overwrite existing directories
-        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
-            transport=transport)
-        self.failUnless(transport.has('.bzr'))
-        bd.retire_bzrdir()
-        self.failIf(transport.has('.bzr'))
-        self.failUnless(transport.has('.bzr.retired.1'))
-
-    def test_retire_bzrdir_limited(self):
-        bd = self.make_bzrdir('.')
-        transport = bd.root_transport
-        # must not overwrite existing directories
-        self.build_tree(['.bzr.retired.0/', '.bzr.retired.0/junk',],
-            transport=transport)
-        self.failUnless(transport.has('.bzr'))
-        self.assertRaises((errors.FileExists, errors.DirectoryNotEmpty),
-            bd.retire_bzrdir, limit=0)
-
-
-class TestBreakLock(TestCaseWithBzrDir):
+        self.assertTrue(len(text))
+
+
+class TestBreakLock(TestCaseWithControlDir):
 
     def test_break_lock_empty(self):
         # break lock on an empty bzrdir should work silently.
@@ -1856,7 +1551,7 @@
             # and thus this interaction cannot be tested at the interface
             # level.
             repo.unlock()
-            return
+            raise TestNotApplicable("format does not physically lock")
         # only one yes needed here: it should only be unlocking
         # the repo
         bzrlib.ui.ui_factory = CannedInputUIFactory([True])
@@ -1865,7 +1560,7 @@
         except NotImplementedError:
             # this bzrdir does not implement break_lock - so we cant test it.
             repo.unlock()
-            return
+            raise TestNotApplicable("format does not support breaking locks")
         lock_repo.lock_write()
         lock_repo.unlock()
         self.assertRaises(errors.LockBroken, repo.unlock)
@@ -1882,7 +1577,8 @@
             bzrlib.branch.BranchReferenceFormat().initialize(
                 thisdir, target_branch=master)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         unused_repo = thisdir.create_repository()
         master.lock_write()
         unused_repo.lock_write()
@@ -1935,7 +1631,7 @@
             # raised a LockActive because we do still have a live locked
             # object.
             tree.unlock()
-            return
+            raise TestNotApplicable("format does not support breaking locks")
         self.assertEqual([True],
                 bzrlib.ui.ui_factory.responses)
         lock_tree = tree.bzrdir.open_workingtree()
@@ -1944,7 +1640,7 @@
         self.assertRaises(errors.LockBroken, tree.unlock)
 
 
-class TestTransportConfig(TestCaseWithBzrDir):
+class TestTransportConfig(TestCaseWithControlDir):
 
     def test_get_config(self):
         my_dir = self.make_bzrdir('.')
@@ -1966,21 +1662,21 @@
         self.assertEqual('http://example.com', config2.get_default_stack_on())
 
 
-class ChrootedBzrDirTests(ChrootedTestCase):
+class ChrootedControlDirTests(ChrootedTestCase):
 
     def test_find_repository_no_repository(self):
         # loopback test to check the current format fails to find a
         # share repository correctly.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         # supported formats must be able to init and open
         # - do the vfs initialisation over the basic vfs transport
         # XXX: TODO this should become a 'bzrdirlocation' api call.
         url = self.get_vfs_only_url('subdir')
-        transport.get_transport(self.get_vfs_only_url()).mkdir('subdir')
+        transport.get_transport_from_url(self.get_vfs_only_url()).mkdir('subdir')
         made_control = self.bzrdir_format.initialize(self.get_url('subdir'))
         try:
             repo = made_control.open_repository()
@@ -1994,8 +1690,8 @@
                           made_control.find_repository)
 
 
-class TestBzrDirControlComponent(TestCaseWithBzrDir):
-    """BzrDir implementations adequately implement ControlComponent."""
+class TestControlDirControlComponent(TestCaseWithControlDir):
+    """ControlDir implementations adequately implement ControlComponent."""
 
     def test_urls(self):
         bd = self.make_bzrdir('bd')

=== added file 'bzrlib/tests/per_controldir/test_format.py'
--- bzrlib/tests/per_controldir/test_format.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_controldir/test_format.py	2011-10-10 12:47:25 +0000
@@ -0,0 +1,48 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for control directory formats."""
+
+from bzrlib import (
+    errors,
+    )
+
+from bzrlib.tests.per_controldir import TestCaseWithControlDir
+
+
+class TestControlDir(TestCaseWithControlDir):
+
+    def test_get_format_description(self):
+        self.assertIsInstance(self.bzrdir_format.get_format_description(),
+                str)
+
+    def test_is_supported(self):
+        self.assertIsInstance(self.bzrdir_format.is_supported(), bool)
+
+    def test_upgrade_recommended(self):
+        self.assertIsInstance(self.bzrdir_format.upgrade_recommended, bool)
+
+    def test_supports_transport(self):
+        self.assertIsInstance(
+            self.bzrdir_format.supports_transport(self.get_transport()), bool)
+
+    def test_check_support_status(self):
+        if not self.bzrdir_format.is_supported():
+            self.assertRaises(errors.UnsupportedFormatError,
+                self.bzrdir_format.check_support_status, False)
+        else:
+            self.bzrdir_format.check_support_status(True)
+            self.bzrdir_format.check_support_status(False)

=== modified file 'bzrlib/tests/per_controldir/test_push.py'
--- bzrlib/tests/per_bzrdir/test_push.py	2009-07-10 06:45:04 +0000
+++ bzrlib/tests/per_controldir/test_push.py	2010-08-27 17:53:08 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,15 +16,12 @@
 
 """Tests for bzrdir implementations - push."""
 
-from bzrlib import (
-    errors,
-    )
-from bzrlib.tests.per_bzrdir import (
-    TestCaseWithBzrDir,
-    )
-
-
-class TestPush(TestCaseWithBzrDir):
+from bzrlib.tests.per_controldir import (
+    TestCaseWithControlDir,
+    )
+
+
+class TestPush(TestCaseWithControlDir):
 
     def create_simple_tree(self):
         tree = self.make_branch_and_tree('tree')

=== renamed directory 'bzrlib/tests/per_bzrdir_colo' => 'bzrlib/tests/per_controldir_colo'
=== modified file 'bzrlib/tests/per_controldir_colo/__init__.py'
--- bzrlib/tests/per_bzrdir_colo/__init__.py	2010-04-12 00:34:15 +0000
+++ bzrlib/tests/per_controldir_colo/__init__.py	2010-08-21 16:06:24 +0000
@@ -23,13 +23,13 @@
 run.
 """
 
-from bzrlib.bzrdir import BzrDirFormat
+from bzrlib.controldir import ControlDirFormat
 from bzrlib.tests import (
     default_transport,
     multiply_tests,
     )
-from bzrlib.tests.per_bzrdir import (
-    TestCaseWithBzrDir,
+from bzrlib.tests.per_controldir import (
+    TestCaseWithControlDir,
     make_scenarios,
     )
 
@@ -37,7 +37,7 @@
 def load_tests(standard_tests, module, loader):
     colo_supported_formats = []
     colo_unsupported_formats = []
-    for format in BzrDirFormat.known_formats():
+    for format in ControlDirFormat.known_formats():
         if format.colocated_branches:
             colo_supported_formats.append(format)
         else:
@@ -48,9 +48,9 @@
         colo_unsupported_formats)
     result = loader.suiteClass()
     supported_tests = loader.loadTestsFromModuleNames([
-        'bzrlib.tests.per_bzrdir_colo.test_supported'])
+        'bzrlib.tests.per_controldir_colo.test_supported'])
     unsupported_tests = loader.loadTestsFromModuleNames([
-        'bzrlib.tests.per_bzrdir_colo.test_unsupported'])
+        'bzrlib.tests.per_controldir_colo.test_unsupported'])
     multiply_tests(supported_tests, supported_scenarios, result)
     multiply_tests(unsupported_tests, unsupported_scenarios, result)
     return result

=== modified file 'bzrlib/tests/per_controldir_colo/test_supported.py'
--- bzrlib/tests/per_bzrdir_colo/test_supported.py	2010-04-11 19:45:48 +0000
+++ bzrlib/tests/per_controldir_colo/test_supported.py	2011-05-13 12:51:05 +0000
@@ -17,27 +17,27 @@
 """Tests for bzr directories that support colocated branches."""
 
 import bzrlib.branch
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    tests,
+    transport,
+    )
 from bzrlib.tests import (
-    TestNotApplicable,
-    )
-from bzrlib.transport import (
-    get_transport,
-    )
-
-from bzrlib.tests.per_bzrdir_colo import (
-    TestCaseWithBzrDir,
-    )
-
-
-class TestColocatedBranchSupport(TestCaseWithBzrDir):
+    per_controldir,
+    )
+
+
+class TestColocatedBranchSupport(per_controldir.TestCaseWithControlDir):
 
     def test_destroy_colocated_branch(self):
         branch = self.make_branch('branch')
         bzrdir = branch.bzrdir
         colo_branch = bzrdir.create_branch('colo')
-        bzrdir.destroy_branch("colo")
-        self.assertRaises(errors.NotBranchError, bzrdir.open_branch, 
+        try:
+            bzrdir.destroy_branch("colo")
+        except (errors.UnsupportedOperation, errors.TransportNotPossible):
+            raise tests.TestNotApplicable('Format does not support destroying branch')
+        self.assertRaises(errors.NotBranchError, bzrdir.open_branch,
                           "colo")
 
     def test_create_colo_branch(self):
@@ -46,14 +46,14 @@
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            raise TestNotApplicable('Control dir format not supported')
-        t = get_transport(self.get_url())
+            raise tests.TestNotApplicable('Control dir format not supported')
+        t = self.get_transport()
         try:
             made_control = self.bzrdir_format.initialize(t.base)
         except errors.UninitializableFormat:
-            raise TestNotApplicable('Control dir does not support creating '
-                'new branches.')
+            raise tests.TestNotApplicable(
+                'Control dir does not support creating new branches.')
         made_repo = made_control.create_repository()
         made_branch = made_control.create_branch("colo")
-        self.failUnless(isinstance(made_branch, bzrlib.branch.Branch))
+        self.assertIsInstance(made_branch, bzrlib.branch.Branch)
         self.assertEqual(made_control, made_branch.bzrdir)

=== modified file 'bzrlib/tests/per_controldir_colo/test_unsupported.py'
--- bzrlib/tests/per_bzrdir_colo/test_unsupported.py	2010-04-16 22:28:06 +0000
+++ bzrlib/tests/per_controldir_colo/test_unsupported.py	2011-09-23 12:32:16 +0000
@@ -20,20 +20,16 @@
 and attributes colocated branch support added should fail in known ways.
 """
 
-from bzrlib import errors
+from bzrlib import (
+    errors,
+    tests,
+    )
 from bzrlib.tests import (
-    TestNotApplicable,
-    )
-from bzrlib.transport import (
-    get_transport,
-    )
-
-from bzrlib.tests.per_bzrdir_colo import (
-    TestCaseWithBzrDir,
-    )
-
-
-class TestNoColocatedSupport(TestCaseWithBzrDir):
+    per_controldir,
+    )
+
+
+class TestNoColocatedSupport(per_controldir.TestCaseWithControlDir):
 
     def make_bzrdir_with_repo(self):
         # a bzrdir can construct a branch and repository for itself.
@@ -41,9 +37,12 @@
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            raise TestNotApplicable('Control dir format not supported')
-        t = get_transport(self.get_url())
-        made_control = self.bzrdir_format.initialize(t.base)
+            raise tests.TestNotApplicable('Control dir format not supported')
+        t = self.get_transport()
+        try:
+            made_control = self.bzrdir_format.initialize(t.base)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable('Control dir format not initializable')
         made_repo = made_control.create_repository()
         return made_control
 
@@ -60,10 +59,10 @@
         self.assertRaises(errors.NoColocatedBranchSupport, 
             made_control.create_branch, "colo")
 
-    def test_branch_transport(self):
+    def test_open_branch(self):
         made_control = self.make_bzrdir_with_repo()
         self.assertRaises(errors.NoColocatedBranchSupport,
-            made_control.get_branch_transport, None, "colo")
+            made_control.open_branch, name="colo")
 
     def test_get_branch_reference(self):
         made_control = self.make_bzrdir_with_repo()

=== modified file 'bzrlib/tests/per_foreign_vcs/test_branch.py'
--- bzrlib/tests/per_foreign_vcs/test_branch.py	2009-10-11 01:20:37 +0000
+++ bzrlib/tests/per_foreign_vcs/test_branch.py	2011-05-13 08:37:15 +0000
@@ -19,13 +19,13 @@
 
 
 from bzrlib.errors import (
+    IncompatibleFormat,
     UnstackableBranchFormat,
     )
 from bzrlib.revision import (
     NULL_REVISION,
     )
 from bzrlib.tests import (
-    TestCase,
     TestCaseWithTransport,
     )
 
@@ -61,11 +61,6 @@
         branch = self.make_branch()
         branch.set_parent("foobar")
 
-    def test_break_lock(self):
-        """Test that break_lock() works, even if it is a no-op."""
-        branch = self.make_branch()
-        branch.break_lock()
-
     def test_set_push_location(self):
         """Test that setting the push location works."""
         branch = self.make_branch()
@@ -129,7 +124,7 @@
         self.assertEquals((0, NULL_REVISION), branch.last_revision_info())
 
 
-class ForeignBranchFormatTests(TestCase):
+class ForeignBranchFormatTests(TestCaseWithTransport):
     """Basic tests for foreign branch format objects."""
 
     branch_format = None # Set to a BranchFormat instance by adapter
@@ -140,12 +135,11 @@
         Remote branches may be initializable on their own, but none currently
         support living in .bzr/branch.
         """
-        self.assertRaises(NotImplementedError, self.branch_format.initialize, None)
+        bzrdir = self.make_bzrdir('dir')
+        self.assertRaises(IncompatibleFormat, self.branch_format.initialize, bzrdir)
 
     def test_get_format_description_type(self):
         self.assertIsInstance(self.branch_format.get_format_description(), str)
 
     def test_network_name(self):
         self.assertIsInstance(self.branch_format.network_name(), str)
-
-

=== modified file 'bzrlib/tests/per_foreign_vcs/test_repository.py'
--- bzrlib/tests/per_foreign_vcs/test_repository.py	2010-03-03 22:59:21 +0000
+++ bzrlib/tests/per_foreign_vcs/test_repository.py	2011-05-17 13:58:46 +0000
@@ -85,4 +85,3 @@
         repo = self.make_repository()
         stats = repo.gather_stats()
         self.assertIsInstance(stats, dict)
-        self.assertTrue(stats.has_key("revisions"))

=== modified file 'bzrlib/tests/per_interbranch/__init__.py'
--- bzrlib/tests/per_interbranch/__init__.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_interbranch/__init__.py	2011-10-03 23:12:01 +0000
@@ -27,21 +27,11 @@
 
 from bzrlib import (
     branchbuilder,
-    memorytree,
     )
 from bzrlib.branch import (
                            GenericInterBranch,
                            InterBranch,
                            )
-from bzrlib.bzrdir import (
-    BzrDirFormat,
-    BzrDirMetaFormat1,
-    )
-from bzrlib.errors import (
-    FileExists,
-    NotBranchError,
-    UninitializableFormat,
-    )
 from bzrlib.tests import (
     TestCaseWithTransport,
     multiply_tests,
@@ -86,59 +76,79 @@
 class TestCaseWithInterBranch(TestCaseWithTransport):
 
     def make_from_branch(self, relpath):
-        repo = self.make_repository(relpath)
-        return self.branch_format_from.initialize(repo.bzrdir)
+        return self.make_branch(relpath, format=self.branch_format_from._matchingbzrdir)
 
     def make_from_branch_and_memory_tree(self, relpath):
         """Create a branch on the default transport and a MemoryTree for it."""
-        b = self.make_from_branch(relpath)
-        return memorytree.MemoryTree.create_on_branch(b)
+        self.assertEquals(
+            self.branch_format_from._matchingbzrdir.get_branch_format(),
+            self.branch_format_from)
+        return self.make_branch_and_memory_tree(
+            relpath, format=self.branch_format_from._matchingbzrdir)
 
     def make_from_branch_and_tree(self, relpath):
         """Create a branch on the default transport and a working tree for it."""
-        b = self.make_from_branch(relpath)
-        return b.bzrdir.create_workingtree()
+        self.assertEquals(
+            self.branch_format_from._matchingbzrdir.get_branch_format(),
+            self.branch_format_from)
+        return self.make_branch_and_tree(relpath,
+            format=self.branch_format_from._matchingbzrdir)
 
     def make_from_branch_builder(self, relpath):
-        default_format = BzrDirFormat.get_default_format()
-        format = BzrDirMetaFormat1()
-        format.set_branch_format(self.branch_format_from)
-        format.repository_format = default_format.repository_format
-        format.workingtree_format = default_format.workingtree_format
+        self.assertEquals(
+            self.branch_format_from._matchingbzrdir.get_branch_format(),
+            self.branch_format_from)
         return branchbuilder.BranchBuilder(self.get_transport(relpath),
-            format=format)
+            format=self.branch_format_from._matchingbzrdir)
 
     def make_to_branch(self, relpath):
-        repo = self.make_repository(relpath)
-        return self.branch_format_to.initialize(repo.bzrdir)
+        self.assertEquals(
+            self.branch_format_to._matchingbzrdir.get_branch_format(),
+            self.branch_format_to)
+        return self.make_branch(relpath, format=self.branch_format_to._matchingbzrdir)
 
     def make_to_branch_and_memory_tree(self, relpath):
         """Create a branch on the default transport and a MemoryTree for it."""
-        b = self.make_to_branch(relpath)
-        return memorytree.MemoryTree.create_on_branch(b)
+        self.assertEquals(
+            self.branch_format_to._matchingbzrdir.get_branch_format(),
+            self.branch_format_to)
+        return self.make_branch_and_memory_tree(
+            relpath, format=self.branch_format_to._matchingbzrdir)
 
     def make_to_branch_and_tree(self, relpath):
         """Create a branch on the default transport and a working tree for it."""
-        b = self.make_to_branch(relpath)
-        return b.bzrdir.create_workingtree()
+        self.assertEquals(
+            self.branch_format_to._matchingbzrdir.get_branch_format(),
+            self.branch_format_to)
+        return self.make_branch_and_tree(relpath,
+            format=self.branch_format_to._matchingbzrdir)
+
+    def _sprout(self, origdir, to_url, format):
+        if format.supports_workingtrees:
+            newbranch = self.make_branch(to_url, format=format)
+        else:
+            newbranch = self.make_branch(to_url+".branch", format=format)
+        origbranch = origdir.open_branch()
+        newbranch.repository.fetch(origbranch.repository)
+        origbranch.copy_content_into(newbranch)
+        if format.supports_workingtrees:
+            wt = newbranch.bzrdir.create_workingtree()
+        else:
+            wt = newbranch.create_checkout(to_url, lightweight=True)
+        return wt
 
     def sprout_to(self, origdir, to_url):
         """Sprout a bzrdir, using to_format for the new branch."""
-        newbranch = self.make_to_branch(to_url)
-        origbranch = origdir.open_branch()
-        newbranch.repository.fetch(origbranch.repository)
-        origbranch.copy_content_into(newbranch)
-        newbranch.bzrdir.create_workingtree()
-        return newbranch.bzrdir
+        wt = self._sprout(origdir, to_url, self.branch_format_to._matchingbzrdir)
+        self.assertEquals(wt.branch._format, self.branch_format_to)
+        return wt.bzrdir
 
     def sprout_from(self, origdir, to_url):
         """Sprout a bzrdir, using from_format for the new bzrdir."""
-        newbranch = self.make_from_branch(to_url)
-        origbranch = origdir.open_branch()
-        newbranch.repository.fetch(origbranch.repository)
-        origbranch.copy_content_into(newbranch)
-        newbranch.bzrdir.create_workingtree()
-        return newbranch.bzrdir
+        wt = self._sprout(origdir, to_url,
+            self.branch_format_from._matchingbzrdir)
+        self.assertEquals(wt.branch._format, self.branch_format_from)
+        return wt.bzrdir
 
 
 class StubWithFormat(object):
@@ -171,11 +181,11 @@
 
 def load_tests(standard_tests, module, loader):
     submod_tests = loader.loadTestsFromModuleNames([
+        'bzrlib.tests.per_interbranch.test_fetch',
         'bzrlib.tests.per_interbranch.test_get',
         'bzrlib.tests.per_interbranch.test_copy_content_into',
         'bzrlib.tests.per_interbranch.test_pull',
         'bzrlib.tests.per_interbranch.test_push',
-        'bzrlib.tests.per_interbranch.test_update_revisions',
         ])
     scenarios = make_scenarios(default_test_list())
     return multiply_tests(submod_tests, scenarios, standard_tests)

=== added file 'bzrlib/tests/per_interbranch/test_fetch.py'
--- bzrlib/tests/per_interbranch/test_fetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_interbranch/test_fetch.py	2011-05-20 08:25:15 +0000
@@ -0,0 +1,95 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for InterBranch.fetch."""
+
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests.per_interbranch import (
+    TestCaseWithInterBranch,
+    )
+
+
+class TestInterBranchFetch(TestCaseWithInterBranch):
+
+    def test_fetch_revisions(self):
+        """Test fetch-revision operation."""
+        wt = self.make_from_branch_and_tree('b1')
+        b1 = wt.branch
+        self.build_tree_contents([('b1/foo', 'hello')])
+        wt.add(['foo'], ['foo-id'])
+        wt.commit('lala!', rev_id='revision-1', allow_pointless=False)
+
+        b2 = self.make_to_branch('b2')
+        b2.fetch(b1)
+
+        # fetch does not update the last revision
+        self.assertEquals(NULL_REVISION, b2.last_revision())
+
+        rev = b2.repository.get_revision('revision-1')
+        tree = b2.repository.revision_tree('revision-1')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEqual(tree.get_file_text('foo-id'), 'hello')
+
+    def test_fetch_revisions_limit(self):
+        """Test fetch-revision operation."""
+        builder = self.make_branch_builder('b1',
+            format=self.branch_format_from._matchingbzrdir)
+        builder.start_series()
+        builder.build_commit(rev_id='revision-1')
+        builder.build_commit(rev_id='revision-2')
+        builder.build_commit(rev_id='revision-3')
+        builder.finish_series()
+        b1 = builder.get_branch()
+        b2 = self.make_to_branch('b2')
+        b2.fetch(b1, limit=1)
+
+        # fetch does not update the last revision
+        self.assertEquals(NULL_REVISION, b2.last_revision())
+
+        self.assertEquals(
+            set(['revision-1']),
+            b2.repository.has_revisions(
+                ['revision-1', 'revision-2', 'revision-3']))
+
+    def test_fetch_revisions_limit_incremental(self):
+        """Test incremental fetch-revision operation with limit."""
+        wt = self.make_from_branch_and_tree('b1')
+        b1 = wt.branch
+        self.build_tree_contents([('b1/foo', 'hello')])
+        wt.add(['foo'], ['foo-id'])
+        wt.commit('lala!', rev_id='revision-1', allow_pointless=False)
+
+        b2 = self.make_to_branch('b2')
+        b2.fetch(b1, limit=1)
+
+        self.assertEquals(
+            set(['revision-1']),
+            b2.repository.has_revisions(
+                ['revision-1', 'revision-2', 'revision-3']))
+
+        wt.commit('hmm', rev_id='revision-2')
+        wt.commit('hmmm', rev_id='revision-3')
+
+        b2.fetch(b1, limit=1)
+
+        # fetch does not update the last revision
+        self.assertEquals(NULL_REVISION, b2.last_revision())
+
+        self.assertEquals(
+            set(['revision-1', 'revision-2']),
+            b2.repository.has_revisions(
+                ['revision-1', 'revision-2', 'revision-3']))

=== modified file 'bzrlib/tests/per_interbranch/test_pull.py'
--- bzrlib/tests/per_interbranch/test_pull.py	2010-06-17 09:23:19 +0000
+++ bzrlib/tests/per_interbranch/test_pull.py	2011-09-27 11:35:00 +0000
@@ -16,8 +16,6 @@
 
 """Tests for InterBranch.pull behaviour."""
 
-import os
-
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib import errors
@@ -42,7 +40,7 @@
         parent.merge_from_branch(mine.branch)
         parent.commit('merge my change', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_merged_indirect(self):
         # it should be possible to do a pull from one branch into another
@@ -59,7 +57,7 @@
         parent.merge_from_branch(other.branch)
         parent.commit('merge other', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_updates_checkout_and_master(self):
         """Pulling into a checkout updates the checkout and the master branch"""
@@ -70,8 +68,8 @@
         rev2 = other.commit('other commit')
         # now pull, which should update both checkout and master.
         checkout.branch.pull(other.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_pull_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_from_branch_and_tree('master')
@@ -79,7 +77,7 @@
         other = self.sprout_to(master_tree.branch.bzrdir, 'other').open_branch()
         # move the branch out of the way on disk to cause a connection
         # error.
-        os.rename('master', 'master_gone')
+        master_tree.branch.bzrdir.destroy_branch()
         # try to pull, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 checkout.branch.pull, other)
@@ -99,7 +97,7 @@
         self.assertEqual('P1', result.old_revid)
         self.assertEqual(2, result.new_revno)
         self.assertEqual('M1', result.new_revid)
-        self.assertEqual(None, result.tag_conflicts)
+        self.assertEqual([], result.tag_conflicts)
 
     def test_pull_overwrite(self):
         tree_a = self.make_from_branch_and_tree('tree_a')
@@ -118,8 +116,8 @@
         tree_a.branch.pull(tree_b.branch, overwrite=True,
                            stop_revision='rev2b')
         self.assertEqual('rev2b', tree_a.branch.last_revision())
-        self.assertEqual(tree_b.branch.revision_history(),
-                         tree_a.branch.revision_history())
+        self.assertEqual(tree_b.branch.last_revision(),
+                         tree_a.branch.last_revision())
 
 
 class TestPullHook(TestCaseWithInterBranch):

=== modified file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	2010-06-29 16:21:13 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2011-10-05 12:45:41 +0000
@@ -17,7 +17,6 @@
 """Tests for branch.push behaviour."""
 
 from cStringIO import StringIO
-import os
 
 from testtools.matchers import (
     Equals,
@@ -26,20 +25,18 @@
 
 from bzrlib import (
     branch,
-    builtins,
     bzrdir,
     check,
-    debug,
     errors,
     push,
-    repository,
+    symbol_versioning,
     tests,
+    vf_repository,
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.memorytree import MemoryTree
 from bzrlib.revision import NULL_REVISION
-from bzrlib.smart import client, server
 from bzrlib.smart.repository import SmartServerRepositoryGetParentMap
 from bzrlib.tests.per_interbranch import (
     TestCaseWithInterBranch,
@@ -63,12 +60,15 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge my change', rev_id='P2')
         result = mine.branch.push(other.branch)
-        self.assertEqual(['P1', 'P2'], other.branch.revision_history())
+        self.assertEqual('P2', other.branch.last_revision())
         # result object contains some structured data
         self.assertEqual(result.old_revid, 'M1')
         self.assertEqual(result.new_revid, 'P2')
         # and it can be treated as an integer for compatibility
-        self.assertEqual(int(result), 0)
+        self.assertEqual(self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            result.__int__),
+            0)
 
     def test_push_merged_indirect(self):
         # it should be possible to do a push from one branch into another
@@ -85,7 +85,7 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge other', rev_id='P2')
         mine.branch.push(target.branch)
-        self.assertEqual(['P1', 'P2'], target.branch.revision_history())
+        self.assertEqual('P2', target.branch.last_revision())
 
     def test_push_to_checkout_updates_master(self):
         """Pushing into a checkout updates the checkout and the master branch"""
@@ -103,8 +103,8 @@
         rev2 = other.commit('other commit')
         # now push, which should update both checkout and master.
         other.branch.push(checkout.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_push_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_to_branch_and_tree('master')
@@ -118,7 +118,7 @@
         other = other_bzrdir.open_workingtree()
         # move the branch out of the way on disk to cause a connection
         # error.
-        os.rename('master', 'master_gone')
+        master_tree.bzrdir.destroy_branch()
         # try to push, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 other.branch.push, checkout.branch)
@@ -158,6 +158,9 @@
             return
         try:
             tree = a_branch.bzrdir.create_workingtree()
+        except errors.UnsupportedOperation:
+            self.assertFalse(a_branch.bzrdir._format.supports_workingtrees)
+            tree = a_branch.create_checkout('repo/tree', lightweight=True)
         except errors.NotLocalUrl:
             if self.vfs_transport_factory is test_server.LocalURLServer:
                 # the branch is colocated on disk, we cannot create a checkout.
@@ -200,8 +203,6 @@
         default for the branch), and will be stacked when the repo format
         allows (which means that the branch format isn't necessarly preserved).
         """
-        if isinstance(self.branch_format_from, branch.BzrBranchFormat4):
-            raise tests.TestNotApplicable('Not a metadir format.')
         if isinstance(self.branch_format_from, branch.BranchReferenceFormat):
             # This test could in principle apply to BranchReferenceFormat, but
             # make_branch_builder doesn't support it.
@@ -288,7 +289,7 @@
     def disableOptimisticGetParentMap(self):
         # Tweak some class variables to stop remote get_parent_map calls asking
         # for or receiving more data than the caller asked for.
-        self.overrideAttr(repository.InterRepository,
+        self.overrideAttr(vf_repository.InterVersionedFileRepository,
                           '_walk_to_common_revisions_batch_size', 1)
         self.overrideAttr(SmartServerRepositoryGetParentMap,
                             'no_extra_results', True)

=== removed file 'bzrlib/tests/per_interbranch/test_update_revisions.py'
--- bzrlib/tests/per_interbranch/test_update_revisions.py	2010-06-17 09:23:19 +0000
+++ bzrlib/tests/per_interbranch/test_update_revisions.py	1970-01-01 00:00:00 +0000
@@ -1,70 +0,0 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-from bzrlib import (
-    errors,
-    )
-from bzrlib.tests.per_interbranch import (
-    TestCaseWithInterBranch,
-    )
-
-
-class TestUpdateRevisions(TestCaseWithInterBranch):
-
-    def setUp(self):
-        super(TestUpdateRevisions, self).setUp()
-        self.tree1 = self.make_from_branch_and_tree('tree1')
-        rev1 = self.tree1.commit('one')
-        branch2 = self.make_to_branch('tree2')
-        branch2.repository.fetch(self.tree1.branch.repository)
-        self.tree1.branch.copy_content_into(branch2)
-        self.tree2 = branch2.bzrdir.create_workingtree()
-
-    def test_accepts_graph(self):
-        # An implementation may not use it, but it should allow a 'graph' to be
-        # supplied
-        rev2 = self.tree2.commit('two')
-
-        self.tree1.lock_write()
-        self.addCleanup(self.tree1.unlock)
-        self.tree2.lock_read()
-        self.addCleanup(self.tree2.unlock)
-        graph = self.tree2.branch.repository.get_graph(
-            self.tree1.branch.repository)
-
-        self.tree1.branch.update_revisions(self.tree2.branch, graph=graph)
-        self.assertEqual((2, rev2), self.tree1.branch.last_revision_info())
-
-    def test_overwrite_ignores_diverged(self):
-        rev2 = self.tree1.commit('two')
-        rev2b = self.tree2.commit('alt two')
-
-        self.assertRaises(errors.DivergedBranches,
-                          self.tree1.branch.update_revisions,
-                          self.tree2.branch, overwrite=False)
-        # However, the revision should be copied into the repository
-        self.assertTrue(self.tree1.branch.repository.has_revision(rev2b))
-
-        self.tree1.branch.update_revisions(self.tree2.branch, overwrite=True)
-        self.assertEqual((2, rev2b), self.tree1.branch.last_revision_info())
-
-    def test_ignores_older_unless_overwrite(self):
-        rev2 = self.tree1.commit('two')
-
-        self.tree1.branch.update_revisions(self.tree2.branch)
-        self.assertEqual((2, rev2), self.tree1.branch.last_revision_info())
-
-        self.tree1.branch.update_revisions(self.tree2.branch, overwrite=True)

=== modified file 'bzrlib/tests/per_interrepository/__init__.py'
--- bzrlib/tests/per_interrepository/__init__.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_interrepository/__init__.py	2011-09-30 12:31:07 +0000
@@ -26,20 +26,28 @@
 """
 
 
-from bzrlib import transport
+from bzrlib import (
+    pyutils,
+    transport,
+    )
 from bzrlib.errors import (
     FileExists,
     UninitializableFormat,
     )
 
 from bzrlib.repository import (
+    format_registry,
     InterRepository,
     )
 from bzrlib.tests import (
-                          default_transport,
-                          multiply_tests,
-                          )
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
+    TestSkipped,
+    default_transport,
+    multiply_tests,
+    )
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
+from bzrlib.vf_repository import (
+    InterDifferingSerializer,
+    )
 
 
 def make_scenarios(transport_server, transport_readonly_server, formats):
@@ -49,7 +57,7 @@
         (label, repository_format, repository_format_to).
     """
     result = []
-    for label, repository_format, repository_format_to in formats:
+    for label, repository_format, repository_format_to, extra_setup in formats:
         id = '%s,%s,%s' % (label, repository_format.__class__.__name__,
                            repository_format_to.__class__.__name__)
         scenario = (id,
@@ -57,6 +65,7 @@
              "transport_readonly_server": transport_readonly_server,
              "repository_format": repository_format,
              "repository_format_to": repository_format_to,
+             "extra_setup": extra_setup,
              })
         result.append(scenario)
     return result
@@ -67,12 +76,14 @@
     from bzrlib.repofmt import (
         groupcompress_repo,
         knitrepo,
-        pack_repo,
-        weaverepo,
+        knitpack_repo,
         )
     result = []
-    def add_combo(label, from_format, to_format):
-        result.append((label, from_format, to_format))
+    def add_combo(interrepo_cls, from_format, to_format, extra_setup=None,
+                  label=None):
+        if label is None:
+            label = interrepo_cls.__name__
+        result.append((label, from_format, to_format, extra_setup))
     # test the default InterRepository between format 6 and the current
     # default format.
     # XXX: robertc 20060220 reinstate this when there are two supported
@@ -83,55 +94,76 @@
     for optimiser_class in InterRepository._optimisers:
         format_to_test = optimiser_class._get_repo_format_to_test()
         if format_to_test is not None:
-            add_combo(optimiser_class.__name__, format_to_test, format_to_test)
+            add_combo(optimiser_class, format_to_test, format_to_test)
     # if there are specific combinations we want to use, we can add them
     # here. We want to test rich root upgrading.
     # XXX: although we attach InterRepository class names to these scenarios,
     # there's nothing asserting that these labels correspond to what is
     # actually used.
-    add_combo('InterRepository',
-              weaverepo.RepositoryFormat5(),
-              knitrepo.RepositoryFormatKnit3())
-    add_combo('InterRepository',
-              knitrepo.RepositoryFormatKnit1(),
-              knitrepo.RepositoryFormatKnit3())
-    add_combo('InterKnitRepo',
-              knitrepo.RepositoryFormatKnit1(),
-              pack_repo.RepositoryFormatKnitPack1())
-    add_combo('InterKnitRepo',
-              pack_repo.RepositoryFormatKnitPack1(),
+    def force_known_graph(testcase):
+        from bzrlib.fetch import Inter1and2Helper
+        testcase.overrideAttr(Inter1and2Helper, 'known_graph_threshold', -1)
+    # Gather extra scenarios from the repository implementations,
+    # as InterRepositories can be used by Repository implementations
+    # they aren't aware of.
+    for module_name in format_registry._get_all_modules():
+        module = pyutils.get_named_object(module_name)
+        try:
+            get_extra_interrepo_test_combinations = getattr(
+                module,
+                "get_extra_interrepo_test_combinations")
+        except AttributeError:
+            continue
+        for (interrepo_cls, from_format, to_format) in (
+            get_extra_interrepo_test_combinations()):
+            add_combo(interrepo_cls, from_format, to_format)
+    add_combo(InterRepository,
+              knitrepo.RepositoryFormatKnit1(),
+              knitrepo.RepositoryFormatKnit3())
+    add_combo(knitrepo.InterKnitRepo,
+              knitrepo.RepositoryFormatKnit1(),
+              knitpack_repo.RepositoryFormatKnitPack1())
+    add_combo(knitrepo.InterKnitRepo,
+              knitpack_repo.RepositoryFormatKnitPack1(),
               knitrepo.RepositoryFormatKnit1())
-    add_combo('InterKnitRepo',
+    add_combo(knitrepo.InterKnitRepo,
               knitrepo.RepositoryFormatKnit3(),
-              pack_repo.RepositoryFormatKnitPack3())
-    add_combo('InterKnitRepo',
-              pack_repo.RepositoryFormatKnitPack3(),
+              knitpack_repo.RepositoryFormatKnitPack3())
+    add_combo(knitrepo.InterKnitRepo,
+              knitpack_repo.RepositoryFormatKnitPack3(),
               knitrepo.RepositoryFormatKnit3())
-    add_combo('InterKnitRepo',
-              pack_repo.RepositoryFormatKnitPack3(),
-              pack_repo.RepositoryFormatKnitPack4())
-    add_combo('InterDifferingSerializer',
-              pack_repo.RepositoryFormatKnitPack1(),
-              pack_repo.RepositoryFormatKnitPack6RichRoot())
-    add_combo('InterDifferingSerializer',
-              pack_repo.RepositoryFormatKnitPack6RichRoot(),
+    add_combo(knitrepo.InterKnitRepo,
+              knitpack_repo.RepositoryFormatKnitPack3(),
+              knitpack_repo.RepositoryFormatKnitPack4())
+    add_combo(InterDifferingSerializer,
+              knitpack_repo.RepositoryFormatKnitPack1(),
+              knitpack_repo.RepositoryFormatKnitPack6RichRoot())
+    add_combo(InterDifferingSerializer,
+              knitpack_repo.RepositoryFormatKnitPack1(),
+              knitpack_repo.RepositoryFormatKnitPack6RichRoot(),
+              force_known_graph,
+              label='InterDifferingSerializer+get_known_graph_ancestry')
+    add_combo(InterDifferingSerializer,
+              knitpack_repo.RepositoryFormatKnitPack6RichRoot(),
               groupcompress_repo.RepositoryFormat2a())
-    add_combo('InterDifferingSerializer',
+    add_combo(InterDifferingSerializer,
               groupcompress_repo.RepositoryFormat2a(),
-              pack_repo.RepositoryFormatKnitPack6RichRoot())
-    add_combo('InterRepository',
-              groupcompress_repo.RepositoryFormatCHK2(),
-              groupcompress_repo.RepositoryFormat2a())
-    add_combo('InterDifferingSerializer',
-              groupcompress_repo.RepositoryFormatCHK1(),
-              groupcompress_repo.RepositoryFormat2a())
+              knitpack_repo.RepositoryFormatKnitPack6RichRoot())
     return result
 
 
-class TestCaseWithInterRepository(TestCaseWithBzrDir):
+class TestCaseWithInterRepository(TestCaseWithControlDir):
 
     def setUp(self):
         super(TestCaseWithInterRepository, self).setUp()
+        if self.extra_setup:
+            self.extra_setup(self)
+
+    def get_default_format(self):
+        self.assertEquals(
+            self.repository_format._matchingbzrdir.repository_format,
+            self.repository_format)
+        return self.repository_format._matchingbzrdir
 
     def make_branch(self, relpath, format=None):
         repo = self.make_repository(relpath, format=format)

=== modified file 'bzrlib/tests/per_interrepository/test_fetch.py'
--- bzrlib/tests/per_interrepository/test_fetch.py	2010-02-04 16:06:36 +0000
+++ bzrlib/tests/per_interrepository/test_fetch.py	2011-05-08 13:07:11 +0000
@@ -76,7 +76,7 @@
             tree.lock_read()
             self.addCleanup(tree.unlock)
             tree.get_file_text('file1')
-            for file_id in tree:
+            for file_id in tree.all_file_ids():
                 if tree.inventory[file_id].kind == "file":
                     tree.get_file(file_id).read()
 
@@ -451,18 +451,13 @@
     def test_fetch_missing_revision_same_location_fails(self):
         repo_a = self.make_repository('.')
         repo_b = repository.Repository.open('.')
-        try:
-            self.assertRaises(errors.NoSuchRevision, repo_b.fetch, repo_a, revision_id='XXX')
-        except errors.LockError, e:
-            check_old_format_lock_error(self.repository_format)
+        self.assertRaises(errors.NoSuchRevision,
+            repo_b.fetch, repo_a, revision_id='XXX')
 
     def test_fetch_same_location_trivial_works(self):
         repo_a = self.make_repository('.')
         repo_b = repository.Repository.open('.')
-        try:
-            repo_a.fetch(repo_b)
-        except errors.LockError, e:
-            check_old_format_lock_error(self.repository_format)
+        repo_a.fetch(repo_b)
 
     def test_fetch_missing_text_other_location_fails(self):
         source_tree = self.make_branch_and_tree('source')

=== modified file 'bzrlib/tests/per_interrepository/test_interrepository.py'
--- bzrlib/tests/per_interrepository/test_interrepository.py	2009-07-10 06:46:10 +0000
+++ bzrlib/tests/per_interrepository/test_interrepository.py	2011-06-16 15:29:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,57 +19,24 @@
 import sys
 
 import bzrlib
-import bzrlib.bzrdir as bzrdir
-from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 import bzrlib.errors as errors
 import bzrlib.gpg
 from bzrlib.inventory import Inventory
-import bzrlib.repofmt.weaverepo as weaverepo
-import bzrlib.repository as repository
-from bzrlib.revision import NULL_REVISION, Revision
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import (
-    TestCase,
-    TestCaseWithTransport,
     TestNotApplicable,
     TestSkipped,
     )
+from bzrlib.tests.matchers import MatchesAncestry
 from bzrlib.tests.per_interrepository import (
     TestCaseWithInterRepository,
     )
 
 
-def check_old_format_lock_error(repository_format):
-    """Potentially ignore LockError on old formats.
-
-    On win32, with the old OS locks, we get a failure of double-lock when
-    we open a object in 2 objects and try to lock both.
-
-    On new formats, LockError would be invalid, but for old formats
-    this was not supported on Win32.
-    """
-    if sys.platform != 'win32':
-        raise
-
-    description = repository_format.get_format_description()
-    if description in ("Repository format 4",
-                       "Weave repository format 5",
-                       "Weave repository format 6"):
-        # jam 20060701
-        # win32 OS locks are not re-entrant. So one process cannot
-        # open the same repository twice and lock them both.
-        raise TestSkipped('%s on win32 cannot open the same'
-                          ' repository twice in different objects'
-                          % description)
-    raise
-
-
 def check_repo_format_for_funky_id_on_win32(repo):
-    if (isinstance(repo, (weaverepo.AllInOneRepository,
-                          weaverepo.WeaveMetaDirRepository))
-        and sys.platform == 'win32'):
-            raise TestSkipped("funky chars does not permitted"
-                              " on this platform in repository"
-                              " %s" % repo.__class__.__name__)
+    if not repo._format.supports_funky_characters and sys.platform == 'win32':
+        raise TestSkipped("funky chars not allowed on this platform in repository"
+                          " %s" % repo.__class__.__name__)
 
 
 class TestInterRepository(TestCaseWithInterRepository):
@@ -110,7 +77,8 @@
         # and sign 'rev2'
         tree_a.branch.repository.lock_write()
         tree_a.branch.repository.start_write_group()
-        tree_a.branch.repository.sign_revision('rev2', bzrlib.gpg.LoopbackGPGStrategy(None))
+        tree_a.branch.repository.sign_revision('rev2',
+            bzrlib.gpg.LoopbackGPGStrategy(None))
         tree_a.branch.repository.commit_write_group()
         tree_a.branch.repository.unlock()
 
@@ -139,9 +107,15 @@
         self.assertFalse(repo_b.has_revision('pizza'))
         # Asking specifically for an absent revision errors.
         self.assertRaises(errors.NoSuchRevision,
-            repo_b.search_missing_revision_ids, repo_a, revision_id='pizza',
+            repo_b.search_missing_revision_ids, repo_a, revision_ids=['pizza'],
             find_ghosts=True)
         self.assertRaises(errors.NoSuchRevision,
+            repo_b.search_missing_revision_ids, repo_a, revision_ids=['pizza'],
+            find_ghosts=False)
+        self.callDeprecated(
+            ['search_missing_revision_ids(revision_id=...) was deprecated in '
+             '2.4.  Use revision_ids=[...] instead.'],
+            self.assertRaises, errors.NoSuchRevision,
             repo_b.search_missing_revision_ids, repo_a, revision_id='pizza',
             find_ghosts=False)
 
@@ -151,11 +125,23 @@
         # make a repository to compare against that is empty
         repo_b = self.make_to_repository('empty')
         repo_a = self.bzrdir.open_repository()
-        result = repo_b.search_missing_revision_ids(repo_a, revision_id='rev1')
+        result = repo_b.search_missing_revision_ids(
+            repo_a, revision_ids=['rev1'])
         self.assertEqual(set(['rev1']), result.get_keys())
         self.assertEqual(('search', set(['rev1']), set([NULL_REVISION]), 1),
             result.get_recipe())
 
+    def test_search_missing_revision_ids_limit(self):
+        # The limit= argument makes fetch() limit
+        # the results to the first X topo-sorted revisions.
+        repo_b = self.make_to_repository('rev1_only')
+        repo_a = self.bzrdir.open_repository()
+        # check the test will be valid
+        self.assertFalse(repo_b.has_revision('rev2'))
+        result = repo_b.search_missing_revision_ids(repo_a, limit=1)
+        self.assertEqual(('search', set(['rev1']), set(['null:']), 1),
+            result.get_recipe())
+
     def test_fetch_fetches_signatures_too(self):
         from_repo = self.bzrdir.open_repository()
         from_signature = from_repo.get_signature_text('rev2')
@@ -215,5 +201,5 @@
         rev = missing_ghost.get_revision('ghost')
         inv = missing_ghost.get_inventory('ghost')
         # rev must not be corrupt now
-        self.assertEqual([None, 'ghost', 'references', 'tip'],
-            missing_ghost.get_ancestry('tip'))
+        self.assertThat(['ghost', 'references', 'tip'],
+            MatchesAncestry(missing_ghost, 'tip'))

=== modified file 'bzrlib/tests/per_intertree/__init__.py'
--- bzrlib/tests/per_intertree/__init__.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_intertree/__init__.py	2011-08-04 14:01:47 +0000
@@ -24,7 +24,6 @@
 
 import bzrlib
 from bzrlib import (
-    errors,
     revisiontree,
     tests,
     )
@@ -39,9 +38,7 @@
     TestCaseWithTree,
     )
 from bzrlib.tree import InterTree
-from bzrlib.workingtree import (
-    WorkingTreeFormat3,
-    )
+from bzrlib.workingtree_3 import WorkingTreeFormat3
 from bzrlib.workingtree_4 import WorkingTreeFormat4
 
 
@@ -125,6 +122,7 @@
     default_tree_format = WorkingTreeFormat3()
     submod_tests = loader.loadTestsFromModuleNames([
         'bzrlib.tests.per_intertree.test_compare',
+        'bzrlib.tests.per_intertree.test_file_content_matches',
         ])
     test_intertree_permutations = [
         # test InterTree with two default-format working trees.
@@ -137,8 +135,7 @@
             # -- vila 20090311
             chk_tree_format = WorkingTreeFormat4()
             chk_tree_format._get_matchingbzrdir = \
-                lambda:bzrlib.bzrdir.format_registry.make_bzrdir(
-                    'development6-rich-root')
+                lambda:bzrlib.bzrdir.format_registry.make_bzrdir('2a')
             test_intertree_permutations.append(
                 (InterTree.__name__ + "(CHKInventory)",
                  InterTree,

=== modified file 'bzrlib/tests/per_intertree/test_compare.py'
--- bzrlib/tests/per_intertree/test_compare.py	2009-10-02 05:43:41 +0000
+++ bzrlib/tests/per_intertree/test_compare.py	2011-08-04 14:01:47 +0000
@@ -23,11 +23,12 @@
     errors,
     mutabletree,
     tests,
-    workingtree_4,
     )
-from bzrlib.osutils import file_kind, has_symlinks
-from bzrlib.tests import TestNotApplicable
+from bzrlib.osutils import has_symlinks
 from bzrlib.tests.per_intertree import TestCaseWithTwoTrees
+from bzrlib.tests import (
+    features,
+    )
 
 # TODO: test the include_root option.
 # TODO: test that renaming a directory x->y does not emit a rename for the
@@ -963,7 +964,7 @@
         # To add confusion, the totally generic code path works - but it
         # doesn't update persistent metadata. So this test permits InterTrees
         # to either work, or fail with BadFileKindError.
-        self.requireFeature(tests.OsFifoFeature)
+        self.requireFeature(features.OsFifoFeature)
         tree1 = self.make_branch_and_tree('1')
         self.build_tree(['1/a'])
         tree1.set_root_id('root-id')
@@ -1368,7 +1369,7 @@
         return self.mutable_trees_to_locked_test_trees(tree1, tree2)
 
     def test_versioned_symlinks(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1, tree2 = self.make_trees_with_symlinks()
         self.not_applicable_if_cannot_represent_unversioned(tree2)
         root_id = tree1.path2id('')
@@ -1391,7 +1392,7 @@
         self.check_has_changes(True, tree1, tree2)
 
     def test_versioned_symlinks_specific_files(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1, tree2 = self.make_trees_with_symlinks()
         root_id = tree1.path2id('')
         expected = [

=== added file 'bzrlib/tests/per_intertree/test_file_content_matches.py'
--- bzrlib/tests/per_intertree/test_file_content_matches.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_intertree/test_file_content_matches.py	2011-08-04 14:01:47 +0000
@@ -0,0 +1,48 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for the InterTree.file_content_matches() function."""
+
+from bzrlib.tests.per_intertree import TestCaseWithTwoTrees
+
+
+class TestFileContentMatches(TestCaseWithTwoTrees):
+
+    def test_same_contents_and_verifier(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        self.build_tree_contents([
+            ('1/file', 'apples'),
+            ('2/file', 'apples'),
+            ])
+        tree1.add('file', 'file-id-1')
+        tree2.add('file', 'file-id-2')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertTrue(inter.file_content_matches('file-id-1', 'file-id-2'))
+
+    def test_different_contents_and_same_verifier(self):
+        tree1 = self.make_branch_and_tree('1')
+        tree2 = self.make_to_branch_and_tree('2')
+        self.build_tree_contents([
+            ('1/file', 'apples'),
+            ('2/file', 'oranges'),
+            ])
+        tree1.add('file', 'file-id-1')
+        tree2.add('file', 'file-id-2')
+        tree1, tree2 = self.mutable_trees_to_test_trees(self, tree1, tree2)
+        inter = self.intertree_class(tree1, tree2)
+        self.assertFalse(inter.file_content_matches('file-id-1', 'file-id-2'))

=== modified file 'bzrlib/tests/per_inventory/basics.py'
--- bzrlib/tests/per_inventory/basics.py	2009-09-24 20:09:36 +0000
+++ bzrlib/tests/per_inventory/basics.py	2011-06-14 02:21:41 +0000
@@ -35,6 +35,9 @@
 
 from bzrlib.tests.per_inventory import TestCaseWithInventory
 
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    )
 
 
 class TestInventory(TestCaseWithInventory):
@@ -149,7 +152,10 @@
         inv = self.inv_to_test_inv(inv)
         self.assertEqual(inv.path2id('src'), 'src-id')
         self.assertEqual(inv.path2id('src/bye.c'), 'bye-id')
-        self.assertTrue('src-id' in inv)
+        self.assertTrue(
+            self.applyDeprecated(
+                deprecated_in((2, 4, 0)),
+                inv.__contains__, 'src-id'))
 
     def test_non_directory_children(self):
         """Test path2id when a parent directory has no children"""

=== modified file 'bzrlib/tests/per_lock/test_lock.py'
--- bzrlib/tests/per_lock/test_lock.py	2009-07-07 09:00:59 +0000
+++ bzrlib/tests/per_lock/test_lock.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,9 @@
     osutils,
     )
 
-from bzrlib.tests import UnicodeFilenameFeature
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.tests.per_lock import TestCaseWithLock
 
 
@@ -63,6 +65,7 @@
 
         But we shouldn't be able to take a write lock.
         """
+        self.requireFeature(features.not_running_as_root)
         osutils.make_readonly('a-file')
         # Make sure the file is read-only (on all platforms)
         self.assertRaises(IOError, open, 'a-file', 'rb+')
@@ -165,7 +168,7 @@
 
 class TestLockUnicodePath(TestCaseWithLock):
 
-    _test_needs_features = [UnicodeFilenameFeature]
+    _test_needs_features = [features.UnicodeFilenameFeature]
 
     def test_read_lock(self):
         self.build_tree([u'\u1234'])

=== modified file 'bzrlib/tests/per_merger.py'
--- bzrlib/tests/per_merger.py	2010-03-05 08:55:12 +0000
+++ bzrlib/tests/per_merger.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
     errors,
     merge as _mod_merge,
     option,
-    progress,
     )
 from bzrlib.tests import (
     multiply_tests,

=== modified file 'bzrlib/tests/per_pack_repository.py'
--- bzrlib/tests/per_pack_repository.py	2010-05-06 23:41:35 +0000
+++ bzrlib/tests/per_pack_repository.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 These tests are repeated for all pack-based repository formats.
 """
 
-from cStringIO import StringIO
 from stat import S_ISDIR
 
 from bzrlib.btree_index import BTreeGraphIndex
@@ -29,35 +28,24 @@
     errors,
     inventory,
     osutils,
-    progress,
     repository,
     revision as _mod_revision,
-    symbol_versioning,
     tests,
+    transport,
     ui,
-    upgrade,
-    workingtree,
-    )
-from bzrlib.repofmt import (
-    pack_repo,
-    groupcompress_repo,
     )
 from bzrlib.repofmt.groupcompress_repo import RepositoryFormat2a
 from bzrlib.smart import (
     client,
     )
 from bzrlib.tests import (
-    TestCase,
     TestCaseWithTransport,
     TestNotApplicable,
-    TestSkipped,
     )
 from bzrlib.transport import (
-    get_transport,
     memory,
     )
 from bzrlib.tests import test_server
-from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
 class TestPackRepository(TestCaseWithTransport):
@@ -278,8 +266,8 @@
         format = self.get_format()
         server = test_server.FakeNFSServer()
         self.start_server(server)
-        transport = get_transport(server.get_url())
-        bzrdir = self.get_format().initialize_on_transport(transport)
+        t = transport.get_transport_from_url(server.get_url())
+        bzrdir = self.get_format().initialize_on_transport(t)
         repo = bzrdir.create_repository()
         repo_transport = bzrdir.get_repository_transport(None)
         self.assertTrue(repo_transport.has('obsolete_packs'))

=== modified file 'bzrlib/tests/per_repository/__init__.py'
--- bzrlib/tests/per_repository/__init__.py	2010-02-11 09:21:45 +0000
+++ bzrlib/tests/per_repository/__init__.py	2011-10-10 14:03:32 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2008, 2009 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
 #          and others
 #
@@ -25,20 +25,16 @@
 """
 
 from bzrlib import (
+    bzrdir,
     repository,
     )
-from bzrlib.revision import NULL_REVISION
-from bzrlib.repofmt import (
-    weaverepo,
-    )
-from bzrlib.remote import RemoteBzrDirFormat, RemoteRepositoryFormat
+from bzrlib.remote import RemoteRepositoryFormat
 from bzrlib.tests import (
     default_transport,
-    multiply_scenarios,
     multiply_tests,
     test_server,
     )
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
 from bzrlib.transport import memory
 
 
@@ -73,9 +69,7 @@
 def all_repository_format_scenarios():
     """Return a list of test scenarios for parameterising repository tests.
     """
-    registry = repository.format_registry
-    all_formats = [registry.get(k) for k in registry.keys()]
-    all_formats.extend(weaverepo._legacy_formats)
+    all_formats = repository.format_registry._get_all()
     # format_scenarios is all the implementations of Repository; i.e. all disk
     # formats plus RemoteRepository.
     format_scenarios = formats_to_scenarios(
@@ -97,777 +91,44 @@
     return format_scenarios
 
 
-class TestCaseWithRepository(TestCaseWithBzrDir):
-
-    def make_repository(self, relpath, shared=False, format=None):
-        if format is None:
+class TestCaseWithRepository(TestCaseWithControlDir):
+
+    def get_default_format(self):
+        format = self.repository_format._matchingbzrdir
+        self.assertEquals(format.repository_format, self.repository_format)
+        return format
+
+    def make_repository(self, relpath, shared=None, format=None):
+        format = self.resolve_format(format)
+        repo = super(TestCaseWithRepository, self).make_repository(
+            relpath, shared=shared, format=format)
+        if format is None or format.repository_format is self.repository_format:
             # Create a repository of the type we are trying to test.
-            made_control = self.make_bzrdir(relpath)
-            repo = self.repository_format.initialize(made_control,
-                    shared=shared)
             if getattr(self, "repository_to_test_repository", None):
                 repo = self.repository_to_test_repository(repo)
-            return repo
-        else:
-            return super(TestCaseWithRepository, self).make_repository(
-                relpath, shared=shared, format=format)
-
-
-class BrokenRepoScenario(object):
-    """Base class for defining scenarios for testing check and reconcile.
-
-    A subclass needs to define the following methods:
-        :populate_repository: a method to use to populate a repository with
-            sample revisions, inventories and file versions.
-        :all_versions_after_reconcile: all the versions in repository after
-            reconcile.  run_test verifies that the text of each of these
-            versions of the file is unchanged by the reconcile.
-        :populated_parents: a list of (parents list, revision).  Each version
-            of the file is verified to have the given parents before running
-            the reconcile.  i.e. this is used to assert that the repo from the
-            factory is what we expect.
-        :corrected_parents: a list of (parents list, revision).  Each version
-            of the file is verified to have the given parents after the
-            reconcile.  i.e. this is used to assert that reconcile made the
-            changes we expect it to make.
-
-    A subclass may define the following optional method as well:
-        :corrected_fulltexts: a list of file versions that should be stored as
-            fulltexts (not deltas) after reconcile.  run_test will verify that
-            this occurs.
-    """
-
-    def __init__(self, test_case):
-        self.test_case = test_case
-
-    def make_one_file_inventory(self, repo, revision, parents,
-                                inv_revision=None, root_revision=None,
-                                file_contents=None, make_file_version=True):
-        return self.test_case.make_one_file_inventory(
-            repo, revision, parents, inv_revision=inv_revision,
-            root_revision=root_revision, file_contents=file_contents,
-            make_file_version=make_file_version)
-
-    def add_revision(self, repo, revision_id, inv, parent_ids):
-        return self.test_case.add_revision(repo, revision_id, inv, parent_ids)
-
-    def corrected_fulltexts(self):
-        return []
-
-    def repository_text_key_index(self):
-        result = {}
-        if self.versioned_root:
-            result.update(self.versioned_repository_text_keys())
-        result.update(self.repository_text_keys())
-        return result
-
-
-class UndamagedRepositoryScenario(BrokenRepoScenario):
-    """A scenario where the repository has no damage.
-
-    It has a single revision, 'rev1a', with a single file.
-    """
-
-    def all_versions_after_reconcile(self):
-        return ('rev1a', )
-
-    def populated_parents(self):
-        return (((), 'rev1a'), )
-
-    def corrected_parents(self):
-        # Same as the populated parents, because there was nothing wrong.
-        return self.populated_parents()
-
-    def check_regexes(self, repo):
-        return ["0 unreferenced text versions"]
-
-    def populate_repository(self, repo):
-        # make rev1a: A well-formed revision, containing 'a-file'
-        inv = self.make_one_file_inventory(
-            repo, 'rev1a', [], root_revision='rev1a')
-        self.add_revision(repo, 'rev1a', inv, [])
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'rev1a'): True})
-        result.update({('a-file-id', 'rev1a'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'rev1a'):[NULL_REVISION]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'rev1a'):[NULL_REVISION]}
-
-
-class FileParentIsNotInRevisionAncestryScenario(BrokenRepoScenario):
-    """A scenario where a revision 'rev2' has 'a-file' with a
-    parent 'rev1b' that is not in the revision ancestry.
-
-    Reconcile should remove 'rev1b' from the parents list of 'a-file' in
-    'rev2', preserving 'rev1a' as a parent.
-    """
-
-    def all_versions_after_reconcile(self):
-        return ('rev1a', 'rev2')
-
-    def populated_parents(self):
-        return (
-            ((), 'rev1a'),
-            ((), 'rev1b'), # Will be gc'd
-            (('rev1a', 'rev1b'), 'rev2')) # Will have parents trimmed
-
-    def corrected_parents(self):
-        return (
-            ((), 'rev1a'),
-            (None, 'rev1b'),
-            (('rev1a',), 'rev2'))
-
-    def check_regexes(self, repo):
-        return [r"\* a-file-id version rev2 has parents \('rev1a', 'rev1b'\) "
-                r"but should have \('rev1a',\)",
-                "1 unreferenced text versions",
-                ]
-
-    def populate_repository(self, repo):
-        # make rev1a: A well-formed revision, containing 'a-file'
-        inv = self.make_one_file_inventory(
-            repo, 'rev1a', [], root_revision='rev1a')
-        self.add_revision(repo, 'rev1a', inv, [])
-
-        # make rev1b, which has no Revision, but has an Inventory, and
-        # a-file
-        inv = self.make_one_file_inventory(
-            repo, 'rev1b', [], root_revision='rev1b')
-        repo.add_inventory('rev1b', inv, [])
-
-        # make rev2, with a-file.
-        # a-file has 'rev1b' as an ancestor, even though this is not
-        # mentioned by 'rev1a', making it an unreferenced ancestor
-        inv = self.make_one_file_inventory(
-            repo, 'rev2', ['rev1a', 'rev1b'])
-        self.add_revision(repo, 'rev2', inv, ['rev1a'])
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'rev1a'): True,
-                           ('TREE_ROOT', 'rev2'): True})
-        result.update({('a-file-id', 'rev1a'): True,
-                       ('a-file-id', 'rev2'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'rev1a'):[NULL_REVISION],
-                ('a-file-id', 'rev2'):[('a-file-id', 'rev1a')]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'rev1a'):[NULL_REVISION],
-                ('TREE_ROOT', 'rev2'):[('TREE_ROOT', 'rev1a')]}
-
-
-class FileParentHasInaccessibleInventoryScenario(BrokenRepoScenario):
-    """A scenario where a revision 'rev3' containing 'a-file' modified in
-    'rev3', and with a parent which is in the revision ancestory, but whose
-    inventory cannot be accessed at all.
-
-    Reconcile should remove the file version parent whose inventory is
-    inaccessbile (i.e. remove 'rev1c' from the parents of a-file's rev3).
-    """
-
-    def all_versions_after_reconcile(self):
-        return ('rev2', 'rev3')
-
-    def populated_parents(self):
-        return (
-            ((), 'rev2'),
-            (('rev1c',), 'rev3'))
-
-    def corrected_parents(self):
-        return (
-            ((), 'rev2'),
-            ((), 'rev3'))
-
-    def check_regexes(self, repo):
-        return [r"\* a-file-id version rev3 has parents "
-                r"\('rev1c',\) but should have \(\)",
-                ]
-
-    def populate_repository(self, repo):
-        # make rev2, with a-file
-        # a-file is sane
-        inv = self.make_one_file_inventory(repo, 'rev2', [])
-        self.add_revision(repo, 'rev2', inv, [])
-
-        # make ghost revision rev1c, with a version of a-file present so
-        # that we generate a knit delta against this version.  In real life
-        # the ghost might never have been present or rev3 might have been
-        # generated against a revision that was present at the time.  So
-        # currently we have the full history of a-file present even though
-        # the inventory and revision objects are not.
-        self.make_one_file_inventory(repo, 'rev1c', [])
-
-        # make rev3 with a-file
-        # a-file refers to 'rev1c', which is a ghost in this repository, so
-        # a-file cannot have rev1c as its ancestor.
-        inv = self.make_one_file_inventory(repo, 'rev3', ['rev1c'])
-        self.add_revision(repo, 'rev3', inv, ['rev1c', 'rev1a'])
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'rev2'): True,
-                           ('TREE_ROOT', 'rev3'): True})
-        result.update({('a-file-id', 'rev2'): True,
-                       ('a-file-id', 'rev3'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'rev2'):[NULL_REVISION],
-                ('a-file-id', 'rev3'):[NULL_REVISION]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'rev2'):[NULL_REVISION],
-                ('TREE_ROOT', 'rev3'):[NULL_REVISION]}
-
-
-class FileParentsNotReferencedByAnyInventoryScenario(BrokenRepoScenario):
-    """A scenario where a repository with file 'a-file' which has extra
-    per-file versions that are not referenced by any inventory (even though
-    they have the same ID as actual revisions).  The inventory of 'rev2'
-    references 'rev1a' of 'a-file', but there is a 'rev2' of 'some-file' stored
-    and erroneously referenced by later per-file versions (revisions 'rev4' and
-    'rev5').
-
-    Reconcile should remove the file parents that are not referenced by any
-    inventory.
-    """
-
-    def all_versions_after_reconcile(self):
-        return ('rev1a', 'rev2c', 'rev4', 'rev5')
-
-    def populated_parents(self):
-        return [
-            (('rev1a',), 'rev2'),
-            (('rev1a',), 'rev2b'),
-            (('rev2',), 'rev3'),
-            (('rev2',), 'rev4'),
-            (('rev2', 'rev2c'), 'rev5')]
-
-    def corrected_parents(self):
-        return (
-            # rev2 and rev2b have been removed.
-            (None, 'rev2'),
-            (None, 'rev2b'),
-            # rev3's accessible parent inventories all have rev1a as the last
-            # modifier.
-            (('rev1a',), 'rev3'),
-            # rev1a features in both rev4's parents but should only appear once
-            # in the result
-            (('rev1a',), 'rev4'),
-            # rev2c is the head of rev1a and rev2c, the inventory provided
-            # per-file last-modified revisions.
-            (('rev2c',), 'rev5'))
-
-    def check_regexes(self, repo):
-        if repo.supports_rich_root():
-            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
-            # the expected count of errors.
-            count = 9
-        else:
-            count = 3
-        return [
-            # will be gc'd
-            r"unreferenced version: {rev2} in a-file-id",
-            r"unreferenced version: {rev2b} in a-file-id",
-            # will be corrected
-            r"a-file-id version rev3 has parents \('rev2',\) "
-            r"but should have \('rev1a',\)",
-            r"a-file-id version rev5 has parents \('rev2', 'rev2c'\) "
-            r"but should have \('rev2c',\)",
-            r"a-file-id version rev4 has parents \('rev2',\) "
-            r"but should have \('rev1a',\)",
-            "%d inconsistent parents" % count,
-            ]
-
-    def populate_repository(self, repo):
-        # make rev1a: A well-formed revision, containing 'a-file'
-        inv = self.make_one_file_inventory(
-            repo, 'rev1a', [], root_revision='rev1a')
-        self.add_revision(repo, 'rev1a', inv, [])
-
-        # make rev2, with a-file.
-        # a-file is unmodified from rev1a, and an unreferenced rev2 file
-        # version is present in the repository.
-        self.make_one_file_inventory(
-            repo, 'rev2', ['rev1a'], inv_revision='rev1a')
-        self.add_revision(repo, 'rev2', inv, ['rev1a'])
-
-        # make rev3 with a-file
-        # a-file has 'rev2' as its ancestor, but the revision in 'rev2' was
-        # rev1a so this is inconsistent with rev2's inventory - it should
-        # be rev1a, and at the revision level 1c is not present - it is a
-        # ghost, so only the details from rev1a are available for
-        # determining whether a delta is acceptable, or a full is needed,
-        # and what the correct parents are.
-        inv = self.make_one_file_inventory(repo, 'rev3', ['rev2'])
-        self.add_revision(repo, 'rev3', inv, ['rev1c', 'rev1a'])
-
-        # In rev2b, the true last-modifying-revision of a-file is rev1a,
-        # inherited from rev2, but there is a version rev2b of the file, which
-        # reconcile could remove, leaving no rev2b.  Most importantly,
-        # revisions descending from rev2b should not have per-file parents of
-        # a-file-rev2b.
-        # ??? This is to test deduplication in fixing rev4
-        inv = self.make_one_file_inventory(
-            repo, 'rev2b', ['rev1a'], inv_revision='rev1a')
-        self.add_revision(repo, 'rev2b', inv, ['rev1a'])
-
-        # rev4 is for testing that when the last modified of a file in
-        # multiple parent revisions is the same, that it only appears once
-        # in the generated per file parents list: rev2 and rev2b both
-        # descend from 1a and do not change the file a-file, so there should
-        # be no version of a-file 'rev2' or 'rev2b', but rev4 does change
-        # a-file, and is a merge of rev2 and rev2b, so it should end up with
-        # a parent of just rev1a - the starting file parents list is simply
-        # completely wrong.
-        inv = self.make_one_file_inventory(repo, 'rev4', ['rev2'])
-        self.add_revision(repo, 'rev4', inv, ['rev2', 'rev2b'])
-
-        # rev2c changes a-file from rev1a, so the version it of a-file it
-        # introduces is a head revision when rev5 is checked.
-        inv = self.make_one_file_inventory(repo, 'rev2c', ['rev1a'])
-        self.add_revision(repo, 'rev2c', inv, ['rev1a'])
-
-        # rev5 descends from rev2 and rev2c; as rev2 does not alter a-file,
-        # but rev2c does, this should use rev2c as the parent for the per
-        # file history, even though more than one per-file parent is
-        # available, because we use the heads of the revision parents for
-        # the inventory modification revisions of the file to determine the
-        # parents for the per file graph.
-        inv = self.make_one_file_inventory(repo, 'rev5', ['rev2', 'rev2c'])
-        self.add_revision(repo, 'rev5', inv, ['rev2', 'rev2c'])
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'rev1a'): True,
-                           ('TREE_ROOT', 'rev2'): True,
-                           ('TREE_ROOT', 'rev2b'): True,
-                           ('TREE_ROOT', 'rev2c'): True,
-                           ('TREE_ROOT', 'rev3'): True,
-                           ('TREE_ROOT', 'rev4'): True,
-                           ('TREE_ROOT', 'rev5'): True})
-        result.update({('a-file-id', 'rev1a'): True,
-                       ('a-file-id', 'rev2c'): True,
-                       ('a-file-id', 'rev3'): True,
-                       ('a-file-id', 'rev4'): True,
-                       ('a-file-id', 'rev5'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'rev1a'): [NULL_REVISION],
-                 ('a-file-id', 'rev2c'): [('a-file-id', 'rev1a')],
-                 ('a-file-id', 'rev3'): [('a-file-id', 'rev1a')],
-                 ('a-file-id', 'rev4'): [('a-file-id', 'rev1a')],
-                 ('a-file-id', 'rev5'): [('a-file-id', 'rev2c')]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'rev1a'): [NULL_REVISION],
-                ('TREE_ROOT', 'rev2'): [('TREE_ROOT', 'rev1a')],
-                ('TREE_ROOT', 'rev2b'): [('TREE_ROOT', 'rev1a')],
-                ('TREE_ROOT', 'rev2c'): [('TREE_ROOT', 'rev1a')],
-                ('TREE_ROOT', 'rev3'): [('TREE_ROOT', 'rev1a')],
-                ('TREE_ROOT', 'rev4'):
-                    [('TREE_ROOT', 'rev2'), ('TREE_ROOT', 'rev2b')],
-                ('TREE_ROOT', 'rev5'):
-                    [('TREE_ROOT', 'rev2'), ('TREE_ROOT', 'rev2c')]}
-
-
-class UnreferencedFileParentsFromNoOpMergeScenario(BrokenRepoScenario):
-    """
-    rev1a and rev1b with identical contents
-    rev2 revision has parents of [rev1a, rev1b]
-    There is a a-file:rev2 file version, not referenced by the inventory.
-    """
-
-    def all_versions_after_reconcile(self):
-        return ('rev1a', 'rev1b', 'rev2', 'rev4')
-
-    def populated_parents(self):
-        return (
-            ((), 'rev1a'),
-            ((), 'rev1b'),
-            (('rev1a', 'rev1b'), 'rev2'),
-            (None, 'rev3'),
-            (('rev2',), 'rev4'),
-            )
-
-    def corrected_parents(self):
-        return (
-            ((), 'rev1a'),
-            ((), 'rev1b'),
-            ((), 'rev2'),
-            (None, 'rev3'),
-            (('rev2',), 'rev4'),
-            )
-
-    def corrected_fulltexts(self):
-        return ['rev2']
-
-    def check_regexes(self, repo):
-        return []
-
-    def populate_repository(self, repo):
-        # make rev1a: A well-formed revision, containing 'a-file'
-        inv1a = self.make_one_file_inventory(
-            repo, 'rev1a', [], root_revision='rev1a')
-        self.add_revision(repo, 'rev1a', inv1a, [])
-
-        # make rev1b: A well-formed revision, containing 'a-file'
-        # rev1b of a-file has the exact same contents as rev1a.
-        file_contents = repo.revision_tree('rev1a').get_file_text('a-file-id')
-        inv = self.make_one_file_inventory(
-            repo, 'rev1b', [], root_revision='rev1b',
-            file_contents=file_contents)
-        self.add_revision(repo, 'rev1b', inv, [])
-
-        # make rev2, a merge of rev1a and rev1b, with a-file.
-        # a-file is unmodified from rev1a and rev1b, but a new version is
-        # wrongly present anyway.
-        inv = self.make_one_file_inventory(
-            repo, 'rev2', ['rev1a', 'rev1b'], inv_revision='rev1a',
-            file_contents=file_contents)
-        self.add_revision(repo, 'rev2', inv, ['rev1a', 'rev1b'])
-
-        # rev3: a-file unchanged from rev2, but wrongly referencing rev2 of the
-        # file in its inventory.
-        inv = self.make_one_file_inventory(
-            repo, 'rev3', ['rev2'], inv_revision='rev2',
-            file_contents=file_contents, make_file_version=False)
-        self.add_revision(repo, 'rev3', inv, ['rev2'])
-
-        # rev4: a modification of a-file on top of rev3.
-        inv = self.make_one_file_inventory(repo, 'rev4', ['rev2'])
-        self.add_revision(repo, 'rev4', inv, ['rev3'])
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'rev1a'): True,
-                           ('TREE_ROOT', 'rev1b'): True,
-                           ('TREE_ROOT', 'rev2'): True,
-                           ('TREE_ROOT', 'rev3'): True,
-                           ('TREE_ROOT', 'rev4'): True})
-        result.update({('a-file-id', 'rev1a'): True,
-                       ('a-file-id', 'rev1b'): True,
-                       ('a-file-id', 'rev2'): False,
-                       ('a-file-id', 'rev4'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'rev1a'): [NULL_REVISION],
-                ('a-file-id', 'rev1b'): [NULL_REVISION],
-                ('a-file-id', 'rev2'): [NULL_REVISION],
-                ('a-file-id', 'rev4'): [('a-file-id', 'rev2')]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'rev1a'): [NULL_REVISION],
-                ('TREE_ROOT', 'rev1b'): [NULL_REVISION],
-                ('TREE_ROOT', 'rev2'):
-                    [('TREE_ROOT', 'rev1a'), ('TREE_ROOT', 'rev1b')],
-                ('TREE_ROOT', 'rev3'): [('TREE_ROOT', 'rev2')],
-                ('TREE_ROOT', 'rev4'): [('TREE_ROOT', 'rev3')]}
-
-
-class TooManyParentsScenario(BrokenRepoScenario):
-    """A scenario where 'broken-revision' of 'a-file' claims to have parents
-    ['good-parent', 'bad-parent'].  However 'bad-parent' is in the ancestry of
-    'good-parent', so the correct parent list for that file version are is just
-    ['good-parent'].
-    """
-
-    def all_versions_after_reconcile(self):
-        return ('bad-parent', 'good-parent', 'broken-revision')
-
-    def populated_parents(self):
-        return (
-            ((), 'bad-parent'),
-            (('bad-parent',), 'good-parent'),
-            (('good-parent', 'bad-parent'), 'broken-revision'))
-
-    def corrected_parents(self):
-        return (
-            ((), 'bad-parent'),
-            (('bad-parent',), 'good-parent'),
-            (('good-parent',), 'broken-revision'))
-
-    def check_regexes(self, repo):
-        if repo.supports_rich_root():
-            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
-            # the expected count of errors.
-            count = 3
-        else:
-            count = 1
-        return (
-            '     %d inconsistent parents' % count,
-            (r"      \* a-file-id version broken-revision has parents "
-             r"\('good-parent', 'bad-parent'\) but "
-             r"should have \('good-parent',\)"))
-
-    def populate_repository(self, repo):
-        inv = self.make_one_file_inventory(
-            repo, 'bad-parent', (), root_revision='bad-parent')
-        self.add_revision(repo, 'bad-parent', inv, ())
-
-        inv = self.make_one_file_inventory(
-            repo, 'good-parent', ('bad-parent',))
-        self.add_revision(repo, 'good-parent', inv, ('bad-parent',))
-
-        inv = self.make_one_file_inventory(
-            repo, 'broken-revision', ('good-parent', 'bad-parent'))
-        self.add_revision(repo, 'broken-revision', inv, ('good-parent',))
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'bad-parent'): True,
-                           ('TREE_ROOT', 'broken-revision'): True,
-                           ('TREE_ROOT', 'good-parent'): True})
-        result.update({('a-file-id', 'bad-parent'): True,
-                       ('a-file-id', 'broken-revision'): True,
-                       ('a-file-id', 'good-parent'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'bad-parent'): [NULL_REVISION],
-                ('a-file-id', 'broken-revision'):
-                    [('a-file-id', 'good-parent')],
-                ('a-file-id', 'good-parent'): [('a-file-id', 'bad-parent')]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'bad-parent'): [NULL_REVISION],
-                ('TREE_ROOT', 'broken-revision'):
-                    [('TREE_ROOT', 'good-parent')],
-                ('TREE_ROOT', 'good-parent'): [('TREE_ROOT', 'bad-parent')]}
-
-
-class ClaimedFileParentDidNotModifyFileScenario(BrokenRepoScenario):
-    """A scenario where the file parent is the same as the revision parent, but
-    should not be because that revision did not modify the file.
-
-    Specifically, the parent revision of 'current' is
-    'modified-something-else', which does not modify 'a-file', but the
-    'current' version of 'a-file' erroneously claims that
-    'modified-something-else' is the parent file version.
-    """
-
-    def all_versions_after_reconcile(self):
-        return ('basis', 'current')
-
-    def populated_parents(self):
-        return (
-            ((), 'basis'),
-            (('basis',), 'modified-something-else'),
-            (('modified-something-else',), 'current'))
-
-    def corrected_parents(self):
-        return (
-            ((), 'basis'),
-            (None, 'modified-something-else'),
-            (('basis',), 'current'))
-
-    def check_regexes(self, repo):
-        if repo.supports_rich_root():
-            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
-            # the expected count of errors.
-            count = 3
-        else:
-            count = 1
-        return (
-            "%d inconsistent parents" % count,
-            r"\* a-file-id version current has parents "
-            r"\('modified-something-else',\) but should have \('basis',\)",
-            )
-
-    def populate_repository(self, repo):
-        inv = self.make_one_file_inventory(repo, 'basis', ())
-        self.add_revision(repo, 'basis', inv, ())
-
-        # 'modified-something-else' is a correctly recorded revision, but it
-        # does not modify the file we are looking at, so the inventory for that
-        # file in this revision points to 'basis'.
-        inv = self.make_one_file_inventory(
-            repo, 'modified-something-else', ('basis',), inv_revision='basis')
-        self.add_revision(repo, 'modified-something-else', inv, ('basis',))
-
-        # The 'current' revision has 'modified-something-else' as its parent,
-        # but the 'current' version of 'a-file' should have 'basis' as its
-        # parent.
-        inv = self.make_one_file_inventory(
-            repo, 'current', ('modified-something-else',))
-        self.add_revision(repo, 'current', inv, ('modified-something-else',))
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'basis'): True,
-                           ('TREE_ROOT', 'current'): True,
-                           ('TREE_ROOT', 'modified-something-else'): True})
-        result.update({('a-file-id', 'basis'): True,
-                       ('a-file-id', 'current'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'basis'): [NULL_REVISION],
-                ('a-file-id', 'current'): [('a-file-id', 'basis')]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'basis'): ['null:'],
-                ('TREE_ROOT', 'current'):
-                    [('TREE_ROOT', 'modified-something-else')],
-                ('TREE_ROOT', 'modified-something-else'):
-                    [('TREE_ROOT', 'basis')]}
-
-
-class IncorrectlyOrderedParentsScenario(BrokenRepoScenario):
-    """A scenario where the set parents of a version of a file are correct, but
-    the order of those parents is incorrect.
-
-    This defines a 'broken-revision-1-2' and a 'broken-revision-2-1' which both
-    have their file version parents reversed compared to the revision parents,
-    which is invalid.  (We use two revisions with opposite orderings of the
-    same parents to make sure that accidentally relying on dictionary/set
-    ordering cannot make the test pass; the assumption is that while dict/set
-    iteration order is arbitrary, it is also consistent within a single test).
-    """
-
-    def all_versions_after_reconcile(self):
-        return ['parent-1', 'parent-2', 'broken-revision-1-2',
-                'broken-revision-2-1']
-
-    def populated_parents(self):
-        return (
-            ((), 'parent-1'),
-            ((), 'parent-2'),
-            (('parent-2', 'parent-1'), 'broken-revision-1-2'),
-            (('parent-1', 'parent-2'), 'broken-revision-2-1'))
-
-    def corrected_parents(self):
-        return (
-            ((), 'parent-1'),
-            ((), 'parent-2'),
-            (('parent-1', 'parent-2'), 'broken-revision-1-2'),
-            (('parent-2', 'parent-1'), 'broken-revision-2-1'))
-
-    def check_regexes(self, repo):
-        if repo.supports_rich_root():
-            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
-            # the expected count of errors.
-            count = 4
-        else:
-            count = 2
-        return (
-            "%d inconsistent parents" % count,
-            r"\* a-file-id version broken-revision-1-2 has parents "
-            r"\('parent-2', 'parent-1'\) but should have "
-            r"\('parent-1', 'parent-2'\)",
-            r"\* a-file-id version broken-revision-2-1 has parents "
-            r"\('parent-1', 'parent-2'\) but should have "
-            r"\('parent-2', 'parent-1'\)")
-
-    def populate_repository(self, repo):
-        inv = self.make_one_file_inventory(repo, 'parent-1', [])
-        self.add_revision(repo, 'parent-1', inv, [])
-
-        inv = self.make_one_file_inventory(repo, 'parent-2', [])
-        self.add_revision(repo, 'parent-2', inv, [])
-
-        inv = self.make_one_file_inventory(
-            repo, 'broken-revision-1-2', ['parent-2', 'parent-1'])
-        self.add_revision(
-            repo, 'broken-revision-1-2', inv, ['parent-1', 'parent-2'])
-
-        inv = self.make_one_file_inventory(
-            repo, 'broken-revision-2-1', ['parent-1', 'parent-2'])
-        self.add_revision(
-            repo, 'broken-revision-2-1', inv, ['parent-2', 'parent-1'])
-        self.versioned_root = repo.supports_rich_root()
-
-    def repository_text_key_references(self):
-        result = {}
-        if self.versioned_root:
-            result.update({('TREE_ROOT', 'broken-revision-1-2'): True,
-                           ('TREE_ROOT', 'broken-revision-2-1'): True,
-                           ('TREE_ROOT', 'parent-1'): True,
-                           ('TREE_ROOT', 'parent-2'): True})
-        result.update({('a-file-id', 'broken-revision-1-2'): True,
-                       ('a-file-id', 'broken-revision-2-1'): True,
-                       ('a-file-id', 'parent-1'): True,
-                       ('a-file-id', 'parent-2'): True})
-        return result
-
-    def repository_text_keys(self):
-        return {('a-file-id', 'broken-revision-1-2'):
-                    [('a-file-id', 'parent-1'), ('a-file-id', 'parent-2')],
-                ('a-file-id', 'broken-revision-2-1'):
-                    [('a-file-id', 'parent-2'), ('a-file-id', 'parent-1')],
-                ('a-file-id', 'parent-1'): [NULL_REVISION],
-                ('a-file-id', 'parent-2'): [NULL_REVISION]}
-
-    def versioned_repository_text_keys(self):
-        return {('TREE_ROOT', 'broken-revision-1-2'):
-                    [('TREE_ROOT', 'parent-1'), ('TREE_ROOT', 'parent-2')],
-                ('TREE_ROOT', 'broken-revision-2-1'):
-                    [('TREE_ROOT', 'parent-2'), ('TREE_ROOT', 'parent-1')],
-                ('TREE_ROOT', 'parent-1'): [NULL_REVISION],
-                ('TREE_ROOT', 'parent-2'): [NULL_REVISION]}
-
-
-all_broken_scenario_classes = [
-    UndamagedRepositoryScenario,
-    FileParentIsNotInRevisionAncestryScenario,
-    FileParentHasInaccessibleInventoryScenario,
-    FileParentsNotReferencedByAnyInventoryScenario,
-    TooManyParentsScenario,
-    ClaimedFileParentDidNotModifyFileScenario,
-    IncorrectlyOrderedParentsScenario,
-    UnreferencedFileParentsFromNoOpMergeScenario,
-    ]
+        return repo
 
 
 def load_tests(standard_tests, module, loader):
     prefix = 'bzrlib.tests.per_repository.'
     test_repository_modules = [
         'test_add_fallback_repository',
-        'test_add_inventory_by_delta',
         'test_break_lock',
         'test_check',
-        # test_check_reconcile is intentionally omitted, see below.
         'test_commit_builder',
         'test_fetch',
-        'test_fileid_involved',
-        'test_find_text_key_references',
-        'test__generate_text_key_index',
+        'test_file_graph',
         'test_get_parent_map',
         'test_has_same_location',
         'test_has_revisions',
-        'test_is_write_locked',
         'test_iter_reverse_revision_history',
-        'test_merge_directive',
+        'test_locking',
         'test_pack',
         'test_reconcile',
         'test_refresh_data',
         'test_repository',
         'test_revision',
+        'test_signatures',
         'test_statistics',
         'test_write_group',
         ]
@@ -875,14 +136,4 @@
     submod_tests = loader.loadTestsFromModuleNames(
         [prefix + module_name for module_name in test_repository_modules])
     format_scenarios = all_repository_format_scenarios()
-    multiply_tests(submod_tests, format_scenarios, standard_tests)
-
-    # test_check_reconcile needs to be parameterized by format *and* by broken
-    # repository scenario.
-    broken_scenarios = [(s.__name__, {'scenario_class': s})
-                        for s in all_broken_scenario_classes]
-    broken_scenarios_for_all_formats = multiply_scenarios(
-        format_scenarios, broken_scenarios)
-    return multiply_tests(
-        loader.loadTestsFromModuleNames([prefix + 'test_check_reconcile']),
-        broken_scenarios_for_all_formats, standard_tests)
+    return multiply_tests(submod_tests, format_scenarios, standard_tests)

=== modified file 'bzrlib/tests/per_repository/test_add_fallback_repository.py'
--- bzrlib/tests/per_repository/test_add_fallback_repository.py	2009-08-13 05:17:12 +0000
+++ bzrlib/tests/per_repository/test_add_fallback_repository.py	2011-08-30 10:54:28 +0000
@@ -17,9 +17,7 @@
 """Tests for Repository.add_fallback_repository."""
 
 from bzrlib import (
-    bzrdir,
     errors,
-    remote,
     )
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestNotApplicable

=== modified file 'bzrlib/tests/per_repository/test_check.py'
--- bzrlib/tests/per_repository/test_check.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_repository/test_check.py	2011-09-09 12:16:16 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,20 +17,10 @@
 
 """Test operations that check the repository for corruption"""
 
-import os
-
 from bzrlib import (
-    check,
-    config as _mod_config,
-    errors,
-    inventory,
     revision as _mod_revision,
     )
-from bzrlib.tests import TestNotApplicable
 from bzrlib.tests.per_repository import TestCaseWithRepository
-from bzrlib.tests.per_repository.helpers import (
-    TestCaseWithBrokenRevisionIndex,
-    )
 
 
 class TestNoSpuriousInconsistentAncestors(TestCaseWithRepository):
@@ -49,100 +39,20 @@
         self.assertContainsRe(self.get_log(), "0 unreferenced text versions")
 
 
-class TestFindInconsistentRevisionParents(TestCaseWithBrokenRevisionIndex):
-
-    def test__find_inconsistent_revision_parents(self):
-        """_find_inconsistent_revision_parents finds revisions with broken
-        parents.
-        """
-        repo = self.make_repo_with_extra_ghost_index()
-        self.assertEqual(
-            [('revision-id', ('incorrect-parent',), ())],
-            list(repo._find_inconsistent_revision_parents()))
-
-    def test__check_for_inconsistent_revision_parents(self):
-        """_check_for_inconsistent_revision_parents raises BzrCheckError if
-        there are any revisions with inconsistent parents.
-        """
-        repo = self.make_repo_with_extra_ghost_index()
-        self.assertRaises(
-            errors.BzrCheckError,
-            repo._check_for_inconsistent_revision_parents)
-
-    def test__check_for_inconsistent_revision_parents_on_clean_repo(self):
-        """_check_for_inconsistent_revision_parents does nothing if there are
-        no broken revisions.
-        """
-        repo = self.make_repository('empty-repo')
-        if not repo.revision_graph_can_have_wrong_parents():
-            raise TestNotApplicable(
-                '%r cannot have corrupt revision index.' % repo)
-        repo.lock_read()
-        try:
-            repo._check_for_inconsistent_revision_parents()  # nothing happens
-        finally:
-            repo.unlock()
-
-    def test_check_reports_bad_ancestor(self):
-        repo = self.make_repo_with_extra_ghost_index()
-        # XXX: check requires a non-empty revision IDs list, but it ignores the
-        # contents of it!
-        check_object = repo.check(['ignored'])
-        check_object.report_results(verbose=False)
-        self.assertContainsRe(self.get_log(),
-            '1 revisions have incorrect parents in the revision index')
-        check_object.report_results(verbose=True)
-        self.assertContainsRe(
-            self.get_log(),
-            "revision-id has wrong parents in index: "
-            r"\('incorrect-parent',\) should be \(\)")
-
-
-class TestCallbacks(TestCaseWithRepository):
-
-    def test_callback_tree_and_branch(self):
-        # use a real tree to get actual refs that will work
-        tree = self.make_branch_and_tree('foo')
-        revid = tree.commit('foo')
-        tree.lock_read()
-        self.addCleanup(tree.unlock)
-        needed_refs = {}
-        for ref in tree._get_check_refs():
-            needed_refs.setdefault(ref, []).append(tree)
-        for ref in tree.branch._get_check_refs():
-            needed_refs.setdefault(ref, []).append(tree.branch)
-        self.tree_check = tree._check
-        self.branch_check = tree.branch.check
-        tree._check = self.tree_callback
-        tree.branch.check = self.branch_callback
-        self.callbacks = []
-        tree.branch.repository.check([revid], callback_refs=needed_refs)
-        self.assertNotEqual([], self.callbacks)
-
-    def tree_callback(self, refs):
-        self.callbacks.append(('tree', refs))
-        return self.tree_check(refs)
-
-    def branch_callback(self, refs):
-        self.callbacks.append(('branch', refs))
-        return self.branch_check(refs)
-
-
 class TestCleanRepository(TestCaseWithRepository):
 
     def test_new_repo(self):
-        repo = self.make_repository('foo')
-        repo.lock_write()
-        self.addCleanup(repo.unlock)
-        config = _mod_config.Config()
-        os.environ['BZR_EMAIL'] = 'foo@sample.com'
-        builder = repo.get_commit_builder(None, [], config)
+        branch = self.make_branch('foo')
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
+        self.overrideEnv('BZR_EMAIL', 'foo@sample.com')
+        builder = branch.get_commit_builder([], branch.get_config())
         list(builder.record_iter_changes(None, _mod_revision.NULL_REVISION, [
             ('TREE_ROOT', (None, ''), True, (False, True), (None, None),
             (None, ''), (None, 'directory'), (None, False))]))
         builder.finish_inventory()
         rev_id = builder.commit('first post')
-        result = repo.check(None, check_repo=True)
+        result = branch.repository.check(None, check_repo=True)
         result.report_results(True)
         log = self.get_log()
         self.assertFalse('Missing' in log, "Something was missing in %r" % log)

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2010-05-11 13:31:31 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2011-10-09 13:56:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
 import os
 
 from bzrlib import (
+    config,
     errors,
-    graph,
     inventory,
     osutils,
     repository,
@@ -28,6 +28,9 @@
     tests,
     )
 from bzrlib.tests import per_repository
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestCommitBuilder(per_repository.TestCaseWithRepository):
@@ -60,6 +63,9 @@
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             repo = tree.branch.repository
             self.record_root(builder, tree)
             builder.finish_inventory()
@@ -84,11 +90,14 @@
         finally:
             tree.unlock()
 
-    def test_abort(self):
+    def test_abort_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             self.record_root(builder, tree)
             builder.finish_inventory()
             builder.abort()
@@ -111,12 +120,27 @@
         finally:
             tree.unlock()
 
+    def test_commit_lossy(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            builder = tree.branch.get_commit_builder([], lossy=True)
+            list(builder.record_iter_changes(tree, tree.last_revision(),
+                tree.iter_changes(tree.basis_tree())))
+            builder.finish_inventory()
+            rev_id = builder.commit('foo bar blah')
+        finally:
+            tree.unlock()
+        rev = tree.branch.repository.get_revision(rev_id)
+        self.assertEqual('foo bar blah', rev.message)
+
     def test_commit_message(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
-            self.record_root(builder, tree)
+            list(builder.record_iter_changes(tree, tree.last_revision(),
+                tree.iter_changes(tree.basis_tree())))
             builder.finish_inventory()
             rev_id = builder.commit('foo bar blah')
         finally:
@@ -124,7 +148,7 @@
         rev = tree.branch.repository.get_revision(rev_id)
         self.assertEqual('foo bar blah', rev.message)
 
-    def test_commit_with_revision_id(self):
+    def test_commit_with_revision_id_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
@@ -142,6 +166,9 @@
             except errors.CannotSetRevisionId:
                 # This format doesn't support supplied revision ids
                 return
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             self.assertFalse(builder.random_revid)
             self.record_root(builder, tree)
             builder.finish_inventory()
@@ -191,7 +218,7 @@
         # on deserialisation, but thats all the current contract guarantees
         # anyway.
         self.assertEqual(revision_id,
-            tree.branch.repository.get_inventory(revision_id).revision_id)
+            tree.branch.repository.revision_tree(revision_id).get_revision_id())
 
     def test_commit_without_root_errors(self):
         tree = self.make_branch_and_tree(".")
@@ -206,6 +233,8 @@
                 except:
                     builder.abort()
                     raise
+                else:
+                    builder.commit("msg")
             self.assertRaises(errors.RootMissing, do_commit)
         finally:
             tree.unlock()
@@ -216,8 +245,11 @@
         try:
             self.build_tree(['foo'])
             tree.add('foo', 'foo-id')
+            builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             entry = tree.inventory['foo-id']
-            builder = tree.branch.get_commit_builder([])
             self.assertRaises(errors.RootMissing,
                 builder.record_entry_contents, entry, [], 'foo', tree,
                     tree.path_content_summary('foo'))
@@ -225,7 +257,7 @@
         finally:
             tree.unlock()
 
-    def test_commit_unchanged_root(self):
+    def test_commit_unchanged_root_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         old_revision_id = tree.commit('')
         tree.lock_write()
@@ -234,6 +266,9 @@
         self.addCleanup(parent_tree.unlock)
         builder = tree.branch.get_commit_builder([old_revision_id])
         try:
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't support "
+                    "record_entry_contents")
             ie = inventory.make_entry('directory', '', None,
                     tree.get_root_id())
             delta, version_recorded, fs_hash = builder.record_entry_contents(
@@ -273,23 +308,27 @@
             # pointless commit.
             self.assertFalse(builder.any_changes())
             builder.finish_inventory()
-            new_root = tree.branch.repository.get_inventory(
-                builder._new_revision_id).root
+            builder_tree = builder.revision_tree()
+            new_root_id = builder_tree.get_root_id()
+            new_root_revision = builder_tree.get_file_revision(new_root_id)
             if tree.branch.repository.supports_rich_root():
                 # We should not have seen a new root revision
-                self.assertEqual(old_revision_id, new_root.revision)
+                self.assertEqual(old_revision_id, new_root_revision)
             else:
                 # We should see a new root revision
-                self.assertNotEqual(old_revision_id, new_root.revision)
+                self.assertNotEqual(old_revision_id, new_root_revision)
         finally:
             builder.abort()
             tree.unlock()
 
-    def test_commit(self):
+    def test_commit_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't "
+                    "support record_entry_contents")
             self.record_root(builder, tree)
             builder.finish_inventory()
             rev_id = builder.commit('foo bar')
@@ -317,6 +356,9 @@
             builder = tree.branch.get_commit_builder([old_revision_id])
             total_delta = []
             try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
                 parent_invs = [basis.inventory]
                 builder.will_record_deletes()
                 if builder.record_root_entry:
@@ -372,6 +414,9 @@
             basis = tree.branch.repository.revision_tree(rev_id)
             builder = tree.branch.get_commit_builder([rev_id])
             try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
                 builder.will_record_deletes()
                 if builder.record_root_entry is True:
                     parent_invs = [basis.inventory]
@@ -432,6 +477,9 @@
         try:
             builder = tree.branch.get_commit_builder([rev_id])
             try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
                 self.record_root(builder, tree)
                 self.assertRaises(AssertionError,
                     builder.record_delete, "foo", "foo-id")
@@ -440,11 +488,14 @@
         finally:
             tree.unlock()
 
-    def test_revision_tree(self):
+    def test_revision_tree_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
         try:
             builder = tree.branch.get_commit_builder([])
+            if not builder.supports_record_entry_contents:
+                raise tests.TestNotApplicable("CommitBuilder doesn't "
+                    "support record_entry_contents")
             self.record_root(builder, tree)
             builder.finish_inventory()
             rev_id = builder.commit('foo bar')
@@ -474,7 +525,7 @@
             # Just a couple simple tests to ensure that it actually follows
             # the RevisionTree api.
             self.assertEqual(rev_id, rev_tree.get_revision_id())
-            self.assertEqual([], rev_tree.get_parent_ids())
+            self.assertEqual((), tuple(rev_tree.get_parent_ids()))
         finally:
             tree.unlock()
 
@@ -489,7 +540,8 @@
         basis_tree = tree.basis_tree()
         basis_tree.lock_read()
         self.addCleanup(basis_tree.unlock)
-        self.assertEqual(rev_id, basis_tree.inventory.root.revision)
+        self.assertEqual(rev_id,
+            basis_tree.get_file_revision(basis_tree.get_root_id()))
 
     def _get_revtrees(self, tree, revision_ids):
         tree.lock_read()
@@ -509,11 +561,13 @@
         rev1 = tree.commit('')
         rev2 = tree.commit('')
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory.root.revision)
+        self.assertEqual(rev1, tree1.get_file_revision(tree1.get_root_id()))
         if tree.branch.repository.supports_rich_root():
-            self.assertEqual(rev1, tree2.inventory.root.revision)
+            self.assertEqual(rev1,
+                tree2.get_file_revision(tree2.get_root_id()))
         else:
-            self.assertEqual(rev2, tree2.inventory.root.revision)
+            self.assertEqual(rev2,
+                tree2.get_file_revision(tree2.get_root_id()))
 
     def _add_commit_check_unchanged(self, tree, name, mini_commit=None):
         tree.add([name], [name + 'id'])
@@ -526,8 +580,8 @@
             mini_commit = self.mini_commit
         rev2 = mini_commit(tree, name, name, False, False)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory[file_id].revision)
-        self.assertEqual(rev1, tree2.inventory[file_id].revision)
+        self.assertEqual(rev1, tree1.get_file_revision(file_id))
+        self.assertEqual(rev1, tree2.get_file_revision(file_id))
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
         self.assertFileGraph(expected_graph, tree, (file_id, rev1))
@@ -556,8 +610,8 @@
         tree.add(['dir/content'], ['contentid'])
         rev2 = tree.commit('')
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory['dirid'].revision)
-        self.assertEqual(rev1, tree2.inventory['dirid'].revision)
+        self.assertEqual(rev1, tree1.get_file_revision('dirid'))
+        self.assertEqual(rev1, tree2.get_file_revision('dirid'))
         file_id = 'dirid'
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
@@ -578,14 +632,14 @@
 
     def test_last_modified_revision_after_commit_link_unchanged(self):
         # committing without changing a link does not change the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_check_unchanged(tree, 'link')
 
     def test_last_modified_revision_after_commit_link_unchanged_ric(self):
         # committing without changing a link does not change the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_check_unchanged(tree, 'link',
@@ -660,14 +714,14 @@
 
     def test_last_modified_revision_after_rename_link_changes(self):
         # renaming a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_renamed_check_changed(tree, 'link')
 
     def test_last_modified_revision_after_rename_link_changes_ric(self):
         # renaming a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_renamed_check_changed(tree, 'link',
@@ -735,14 +789,14 @@
 
     def test_last_modified_revision_after_reparent_link_changes(self):
         # reparenting a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_reparent_check_changed(tree, 'link')
 
     def test_last_modified_revision_after_reparent_link_changes_ric(self):
         # reparenting a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         self._add_commit_reparent_check_changed(tree, 'link',
@@ -766,8 +820,8 @@
         rev2 = mini_commit(tree, name, tree.id2path(file_id),
             expect_fs_hash=expect_fs_hash)
         tree1, tree2 = self._get_revtrees(tree, [rev1, rev2])
-        self.assertEqual(rev1, tree1.inventory[file_id].revision)
-        self.assertEqual(rev2, tree2.inventory[file_id].revision)
+        self.assertEqual(rev1, tree1.get_file_revision(file_id))
+        self.assertEqual(rev2, tree2.get_file_revision(file_id))
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
         expected_graph[(file_id, rev2)] = ((file_id, rev1),)
@@ -793,61 +847,68 @@
             # record_entry_contents.
             parent_ids = tree.get_parent_ids()
             builder = tree.branch.get_commit_builder(parent_ids)
-            parent_tree = tree.basis_tree()
-            parent_tree.lock_read()
-            self.addCleanup(parent_tree.unlock)
-            parent_invs = [parent_tree.inventory]
-            for parent_id in parent_ids[1:]:
-                parent_invs.append(tree.branch.repository.revision_tree(
-                    parent_id).inventory)
-            # root
-            builder.record_entry_contents(
-                inventory.make_entry('directory', '', None,
-                    tree.get_root_id()), parent_invs, '', tree,
-                    tree.path_content_summary(''))
-            def commit_id(file_id):
-                old_ie = tree.inventory[file_id]
-                path = tree.id2path(file_id)
-                ie = inventory.make_entry(tree.kind(file_id), old_ie.name,
-                    old_ie.parent_id, file_id)
-                content_summary = tree.path_content_summary(path)
-                if content_summary[0] == 'tree-reference':
-                    content_summary = content_summary[:3] + (
-                        tree.get_reference_revision(file_id),)
-                return builder.record_entry_contents(ie, parent_invs, path,
-                    tree, content_summary)
+            try:
+                if not builder.supports_record_entry_contents:
+                    raise tests.TestNotApplicable("CommitBuilder doesn't "
+                        "support record_entry_contents")
+                parent_tree = tree.basis_tree()
+                parent_tree.lock_read()
+                self.addCleanup(parent_tree.unlock)
+                parent_invs = [parent_tree.inventory]
+                for parent_id in parent_ids[1:]:
+                    parent_invs.append(tree.branch.repository.revision_tree(
+                        parent_id).inventory)
+                # root
+                builder.record_entry_contents(
+                    inventory.make_entry('directory', '', None,
+                        tree.get_root_id()), parent_invs, '', tree,
+                        tree.path_content_summary(''))
+                def commit_id(file_id):
+                    old_ie = tree.inventory[file_id]
+                    path = tree.id2path(file_id)
+                    ie = inventory.make_entry(tree.kind(file_id), old_ie.name,
+                        old_ie.parent_id, file_id)
+                    content_summary = tree.path_content_summary(path)
+                    if content_summary[0] == 'tree-reference':
+                        content_summary = content_summary[:3] + (
+                            tree.get_reference_revision(file_id),)
+                    return builder.record_entry_contents(ie, parent_invs, path,
+                        tree, content_summary)
 
-            file_id = tree.path2id(new_name)
-            parent_id = tree.inventory[file_id].parent_id
-            if parent_id != tree.get_root_id():
-                commit_id(parent_id)
-            # because a change of some sort is meant to have occurred,
-            # recording the entry must return True.
-            delta, version_recorded, fs_hash = commit_id(file_id)
-            if records_version:
-                self.assertTrue(version_recorded)
-            else:
-                self.assertFalse(version_recorded)
-            if expect_fs_hash:
-                tree_file_stat = tree.get_file_with_stat(file_id)
-                tree_file_stat[0].close()
-                self.assertEqual(2, len(fs_hash))
-                self.assertEqual(tree.get_file_sha1(file_id), fs_hash[0])
-                self.assertEqualStat(tree_file_stat[1], fs_hash[1])
-            else:
-                self.assertEqual(None, fs_hash)
-            new_entry = builder.new_inventory[file_id]
-            if delta_against_basis:
-                expected_delta = (name, new_name, file_id, new_entry)
-                # The delta should be recorded
-                self.assertEqual(expected_delta, builder._basis_delta[-1])
-            else:
-                expected_delta = None
-            self.assertEqual(expected_delta, delta)
-            builder.finish_inventory()
-            rev2 = builder.commit('')
+                file_id = tree.path2id(new_name)
+                parent_id = tree.inventory[file_id].parent_id
+                if parent_id != tree.get_root_id():
+                    commit_id(parent_id)
+                # because a change of some sort is meant to have occurred,
+                # recording the entry must return True.
+                delta, version_recorded, fs_hash = commit_id(file_id)
+                if records_version:
+                    self.assertTrue(version_recorded)
+                else:
+                    self.assertFalse(version_recorded)
+                if expect_fs_hash:
+                    tree_file_stat = tree.get_file_with_stat(file_id)
+                    tree_file_stat[0].close()
+                    self.assertEqual(2, len(fs_hash))
+                    self.assertEqual(tree.get_file_sha1(file_id), fs_hash[0])
+                    self.assertEqualStat(tree_file_stat[1], fs_hash[1])
+                else:
+                    self.assertEqual(None, fs_hash)
+                new_entry = builder.new_inventory[file_id]
+                if delta_against_basis:
+                    expected_delta = (name, new_name, file_id, new_entry)
+                    # The delta should be recorded
+                    self.assertEqual(expected_delta, builder._basis_delta[-1])
+                else:
+                    expected_delta = None
+                self.assertEqual(expected_delta, delta)
+                builder.finish_inventory()
+            except:
+                builder.abort()
+                raise
+            else:
+                rev2 = builder.commit('')
         except:
-            builder.abort()
             tree.unlock()
             raise
         try:
@@ -881,10 +942,10 @@
             parent_tree = tree.basis_tree()
             parent_tree.lock_read()
             self.addCleanup(parent_tree.unlock)
-            parent_invs = [parent_tree.inventory]
+            parent_trees = [parent_tree]
             for parent_id in parent_ids[1:]:
-                parent_invs.append(tree.branch.repository.revision_tree(
-                    parent_id).inventory)
+                parent_trees.append(tree.branch.repository.revision_tree(
+                    parent_id))
             changes = list(tree.iter_changes(parent_tree))
             result = list(builder.record_iter_changes(tree, parent_ids[0],
                 changes))
@@ -910,10 +971,11 @@
                 self.assertFalse(version_recorded)
             self.assertIs(None, builder.new_inventory)
             builder.finish_inventory()
-            inv_key = (builder._new_revision_id,)
-            inv_sha1 = tree.branch.repository.inventories.get_sha1s(
-                            [inv_key])[inv_key]
-            self.assertEqual(inv_sha1, builder.inv_sha1)
+            if tree.branch.repository._format.supports_full_versioned_files:
+                inv_key = (builder._new_revision_id,)
+                inv_sha1 = tree.branch.repository.inventories.get_sha1s(
+                                [inv_key])[inv_key]
+                self.assertEqual(inv_sha1, builder.inv_sha1)
             self.assertIs(None, builder.new_inventory)
             new_inventory = builder.revision_tree().inventory
             new_entry = new_inventory[file_id]
@@ -938,7 +1000,7 @@
         # (closest to a public per-file graph API we have today)
         tree.lock_read()
         self.addCleanup(tree.unlock)
-        g = dict(graph.Graph(tree.branch.repository.texts).iter_ancestry([tip]))
+        g = dict(tree.branch.repository.get_file_graph().iter_ancestry([tip]))
         self.assertEqual(expected_graph, g)
 
     def test_last_modified_revision_after_content_file_changes(self):
@@ -962,7 +1024,7 @@
 
     def test_last_modified_revision_after_content_link_changes(self):
         # changing a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink('target', 'link')
         def change_link():
@@ -975,7 +1037,7 @@
         if file_id is None:
             file_id = link
         # changing a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         os.symlink(target, link)
         def change_link():
@@ -991,7 +1053,7 @@
             'link', 'target', 'newtarget')
 
     def test_last_modified_rev_after_content_unicode_link_changes_ric(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_last_mod_rev_after_content_link_changes_ric(
             u'li\u1234nk', u'targ\N{Euro Sign}t', u'n\N{Euro Sign}wtarget',
 
@@ -1019,7 +1081,7 @@
         rev4 = mini_commit(tree1, 'new_' + name, 'new_' + name,
             expect_fs_hash=expect_fs_hash)
         tree3, = self._get_revtrees(tree1, [rev4])
-        self.assertEqual(rev4, tree3.inventory[name + 'id'].revision)
+        self.assertEqual(rev4, tree3.get_file_revision(name + 'id'))
         file_id = name + 'id'
         expected_graph = {}
         expected_graph[(file_id, rev1)] = ()
@@ -1056,14 +1118,14 @@
 
     def test_last_modified_revision_after_merge_link_changes(self):
         # merge a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge(tree1, 'link')
 
     def test_last_modified_revision_after_merge_link_changes_ric(self):
         # merge a link changes the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge(tree1, 'link',
@@ -1085,7 +1147,7 @@
             rev3 = mini_commit(in_tree, name, 'new_' + name, False,
                 delta_against_basis=changed_in_tree)
             tree3, = self._get_revtrees(in_tree, [rev2])
-            self.assertEqual(rev2, tree3.inventory[name + 'id'].revision)
+            self.assertEqual(rev2, tree3.get_file_revision(name + 'id'))
             file_id = name + 'id'
             expected_graph = {}
             expected_graph[(file_id, rev1)] = ()
@@ -1117,7 +1179,7 @@
         rev3 = mini_commit(tree1, None, 'name', False)
         tree3, = self._get_revtrees(tree1, [rev2])
         # in rev2, name should be only changed in rev2
-        self.assertEqual(rev2, tree3.inventory[file_id].revision)
+        self.assertEqual(rev2, tree3.get_file_revision(file_id))
         expected_graph = {}
         expected_graph[(file_id, rev2)] = ()
         self.assertFileGraph(expected_graph, tree1, (file_id, rev2))
@@ -1150,14 +1212,14 @@
 
     def test_last_modified_revision_after_converged_merge_link_unchanged(self):
         # merge a link that changed preserves the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge_converged(tree1, 'link')
 
     def test_last_modified_revision_after_converged_merge_link_unchanged_ric(self):
         # merge a link that changed preserves the last modified.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree1 = self.make_branch_and_tree('t1')
         os.symlink('target', 't1/link')
         self._commit_sprout_rename_merge_converged(tree1, 'link',
@@ -1203,7 +1265,7 @@
         self.build_tree([name])
 
     def make_link(self, name):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         os.symlink('target', name)
 
     def make_reference(self, name):
@@ -1218,7 +1280,10 @@
         make_before(path)
 
         def change_kind():
-            osutils.delete_any(path)
+            if osutils.file_kind(path) == "directory":
+                osutils.rmtree(path)
+            else:
+                osutils.delete_any(path)
             make_after(path)
 
         self._add_commit_change_check_changed(tree, path, change_kind,
@@ -1288,10 +1353,20 @@
         self.assertRaises(ValueError, builder.commit,
             u'Invalid\r\ncommit message\r\n')
 
+    def test_non_ascii_str_committer_rejected(self):
+        """Ensure an error is raised on a non-ascii byte string committer"""
+        branch = self.make_branch('.')
+        branch.repository.lock_write()
+        self.addCleanup(branch.repository.unlock)
+        self.assertRaises(UnicodeDecodeError,
+            branch.repository.get_commit_builder,
+            branch, [], branch.get_config(),
+            committer="Erik B\xe5gfors <erik@example.com>")
+
     def test_stacked_repositories_reject_commit_builder(self):
         # As per bug 375013, committing to stacked repositories is currently
-        # broken, so repositories with fallbacks refuse to hand out a commit
-        # builder.
+        # broken if we aren't in a chk repository. So old repositories with
+        # fallbacks refuse to hand out a commit builder.
         repo_basis = self.make_repository('basis')
         branch = self.make_branch('local')
         repo_local = branch.repository
@@ -1299,7 +1374,39 @@
             repo_local.add_fallback_repository(repo_basis)
         except errors.UnstackableRepositoryFormat:
             raise tests.TestNotApplicable("not a stackable format.")
-        repo_local.lock_write()
-        self.addCleanup(repo_local.unlock)
-        self.assertRaises(errors.BzrError, repo_local.get_commit_builder,
-            branch, [], branch.get_config())
+        self.addCleanup(repo_local.lock_write().unlock)
+        if not repo_local._format.supports_chks:
+            self.assertRaises(errors.BzrError, repo_local.get_commit_builder,
+                branch, [], branch.get_config())
+        else:
+            builder = repo_local.get_commit_builder(branch, [],
+                                                    branch.get_config())
+            builder.abort()
+
+    def test_committer_no_username(self):
+        # Ensure that when no username is available but a committer is
+        # supplied, commit works.
+        self.overrideEnv('EMAIL', None)
+        self.overrideEnv('BZR_EMAIL', None)
+        # Also, make sure that it's not inferred from mailname.
+        self.overrideAttr(config, '_auto_user_id',
+            lambda: (None, None))
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            # Make sure no username is available.
+            self.assertRaises(errors.NoWhoami, tree.branch.get_commit_builder,
+                              [])
+            builder = tree.branch.get_commit_builder(
+                [], committer='me@example.com')
+            try:
+                list(builder.record_iter_changes(tree, tree.last_revision(),
+                    tree.iter_changes(tree.basis_tree())))
+                builder.finish_inventory()
+            except:
+                builder.abort()
+                raise
+            repo = tree.branch.repository
+            repo.commit_write_group()
+        finally:
+            tree.unlock()

=== modified file 'bzrlib/tests/per_repository/test_fetch.py'
--- bzrlib/tests/per_repository/test_fetch.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_repository/test_fetch.py	2011-10-10 12:47:25 +0000
@@ -20,13 +20,14 @@
     bzrdir,
     errors,
     gpg,
-    graph,
     remote,
     repository,
-    tests,
     )
 from bzrlib.inventory import ROOT_ID
-from bzrlib.tests import TestSkipped
+from bzrlib.tests import (
+    TestNotApplicable,
+    TestSkipped,
+    )
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
@@ -135,8 +136,9 @@
         repo.lock_write()
         self.addCleanup(repo.unlock)
         repo.fetch(source.repository)
+        graph = repo.get_file_graph()
         self.assertEqual(result,
-            repo.texts.get_parent_map([(root_id, 'tip')])[(root_id, 'tip')])
+            graph.get_parent_map([(root_id, 'tip')])[(root_id, 'tip')])
 
     def test_fetch_to_rich_root_set_parent_no_parents(self):
         # No parents rev -> No parents
@@ -154,6 +156,9 @@
 
     def test_fetch_to_rich_root_set_parent_1_ghost_parent(self):
         # 1 ghost parent -> No parents
+        if not self.repository_format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support "
+                 "ghosts")
         self.do_test_fetch_to_rich_root_sets_parents_correctly((),
             [('tip', ['ghost'], [('add', ('', ROOT_ID, 'directory', ''))]),
             ], allow_lefthand_ghost=True)
@@ -197,6 +202,7 @@
              ('base', None, []),
              ('tip', None, [('unversion', 'my-root'),
                             ('unversion', ROOT_ID),
+                            ('flush', None),
                             ('add', ('', 'my-root', 'directory', '')),
                             ]),
             ], root_id='my-root')
@@ -225,9 +231,11 @@
             # 'my-root' at root
              ('right', None, [('unversion', 'my-root'),
                               ('unversion', ROOT_ID),
+                              ('flush', None),
                               ('add', ('', 'my-root', 'directory', ''))]),
              ('tip', ['base', 'right'], [('unversion', 'my-root'),
                             ('unversion', ROOT_ID),
+                            ('flush', None),
                             ('add', ('', 'my-root', 'directory', '')),
                             ]),
             ], root_id='my-root')
@@ -263,7 +271,11 @@
         repo = wt.branch.repository
         repo.lock_write()
         repo.start_write_group()
-        repo.sign_revision('rev1', gpg.LoopbackGPGStrategy(None))
+        try:
+            repo.sign_revision('rev1', gpg.LoopbackGPGStrategy(None))
+        except errors.UnsupportedOperation:
+            self.assertFalse(repo._format.supports_revision_signatures)
+            raise TestNotApplicable("repository format does not support signatures")
         repo.commit_write_group()
         repo.unlock()
         return repo
@@ -323,6 +335,8 @@
     def test_fetch_into_smart_with_ghost(self):
         trans = self.make_smart_server('target')
         source_b = self.make_simple_branch_with_ghost()
+        if not source_b.bzrdir._format.supports_transport(trans):
+            raise TestNotApplicable("format does not support transport")
         target = self.make_repository('target')
         # Re-open the repository over the smart protocol
         target = repository.Repository.open(trans.base)
@@ -334,12 +348,14 @@
             # The code inside fetch() that tries to lock and then fails, also
             # causes weird problems with 'lock_not_held' later on...
             target.lock_read()
-            raise tests.KnownFailure('some repositories fail to fetch'
+            self.knownFailure('some repositories fail to fetch'
                 ' via the smart server because of locking issues.')
 
     def test_fetch_from_smart_with_ghost(self):
         trans = self.make_smart_server('source')
         source_b = self.make_simple_branch_with_ghost()
+        if not source_b.bzrdir._format.supports_transport(trans):
+            raise TestNotApplicable("format does not support transport")
         target = self.make_repository('target')
         target.lock_write()
         self.addCleanup(target.unlock)
@@ -349,29 +365,3 @@
         self.addCleanup(source.unlock)
         target.fetch(source, revision_id='B-id')
 
-
-class TestSource(TestCaseWithRepository):
-    """Tests for/about the results of Repository._get_source."""
-
-    def test_no_absent_records_in_stream_with_ghosts(self):
-        # XXX: Arguably should be in per_interrepository but
-        # doesn't actually gain coverage there; need a specific set of
-        # permutations to cover it.
-        # bug lp:376255 was reported about this.
-        builder = self.make_branch_builder('repo')
-        builder.start_series()
-        builder.build_snapshot('tip', ['ghost'],
-            [('add', ('', 'ROOT_ID', 'directory', ''))],
-            allow_leftmost_as_ghost=True)
-        builder.finish_series()
-        b = builder.get_branch()
-        b.lock_read()
-        self.addCleanup(b.unlock)
-        repo = b.repository
-        source = repo._get_source(repo._format)
-        search = graph.PendingAncestryResult(['tip'], repo)
-        stream = source.get_stream(search)
-        for substream_type, substream in stream:
-            for record in substream:
-                self.assertNotEqual('absent', record.storage_kind,
-                    "Absent record for %s" % (((substream_type,) + record.key),))

=== added file 'bzrlib/tests/per_repository/test_file_graph.py'
--- bzrlib/tests/per_repository/test_file_graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository/test_file_graph.py	2011-05-13 09:30:57 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Tests for the per file graph API."""
+
+
+from bzrlib.tests.per_repository import TestCaseWithRepository
+
+
+class TestPerFileGraph(TestCaseWithRepository):
+
+    def test_file_graph(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([("a", "contents")])
+        tree.add(["a"], ["fileid"])
+        revid1 = tree.commit("msg")
+        self.build_tree_contents([("a", "new contents")])
+        revid2 = tree.commit("msg")
+        self.addCleanup(tree.lock_read().unlock)
+        graph = tree.branch.repository.get_file_graph()
+        self.assertEquals({
+            ("fileid", revid2): (("fileid", revid1),), ("fileid", revid1):()},
+            graph.get_parent_map([("fileid", revid2), ("fileid", revid1)]))

=== removed file 'bzrlib/tests/per_repository/test_fileid_involved.py'
--- bzrlib/tests/per_repository/test_fileid_involved.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_repository/test_fileid_involved.py	1970-01-01 00:00:00 +0000
@@ -1,412 +0,0 @@
-# Copyright (C) 2005-2010 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-import sys
-import time
-
-from bzrlib import (
-    errors,
-    remote,
-    revision as _mod_revision,
-    tests,
-    transform,
-    )
-from bzrlib.tests import per_repository
-
-
-class FileIdInvolvedBase(per_repository.TestCaseWithRepository):
-
-    def touch(self, tree, filename):
-        # use the trees transport to not depend on the tree's location or type.
-        tree.bzrdir.root_transport.append_bytes(filename, "appended line\n")
-
-    def compare_tree_fileids(self, branch, old_rev, new_rev):
-        old_tree = self.branch.repository.revision_tree(old_rev)
-        new_tree = self.branch.repository.revision_tree(new_rev)
-        delta = new_tree.changes_from(old_tree)
-
-        l2 = [id for path, id, kind in delta.added] + \
-             [id for oldpath, newpath, id, kind, text_modified, \
-                meta_modified in delta.renamed] + \
-             [id for path, id, kind, text_modified, meta_modified in \
-                delta.modified]
-        return set(l2)
-
-
-class TestFileIdInvolved(FileIdInvolvedBase):
-
-    def setUp(self):
-        super(TestFileIdInvolved, self).setUp()
-        # create three branches, and merge it
-        #
-        #          ,-->J------>K                (branch2)
-        #         /             \
-        #  A --->B --->C---->D-->G              (main)
-        #  \          /     /
-        #   '--->E---+---->F                    (branch1)
-
-        # A changes:
-        # B changes: 'a-file-id-2006-01-01-abcd'
-        # C changes:  Nothing (perfect merge)
-        # D changes: 'b-file-id-2006-01-01-defg'
-        # E changes: 'file-d'
-        # F changes: 'file-d'
-        # G changes: 'b-file-id-2006-01-01-defg'
-        # J changes: 'b-file-id-2006-01-01-defg'
-        # K changes: 'c-funky<file-id>quiji%bo'
-
-        main_wt = self.make_branch_and_tree('main')
-        main_branch = main_wt.branch
-        self.build_tree(["main/a","main/b","main/c"])
-
-        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
-                                 'b-file-id-2006-01-01-defg',
-                                 'c-funky<file-id>quiji%bo'])
-        try:
-            main_wt.commit("Commit one", rev_id="rev-A")
-        except errors.IllegalPath:
-            # TODO: jam 20060701 Consider raising a different exception
-            #       newer formats do support this, and nothin can done to
-            #       correct this test - its not a bug.
-            if sys.platform == 'win32':
-                raise tests.TestSkipped('Old repository formats do not'
-                                        ' support file ids with <> on win32')
-            # This is not a known error condition
-            raise
-
-        #-------- end A -----------
-
-        bt1 = self.make_branch_and_tree('branch1')
-        bt1.pull(main_branch)
-        b1 = bt1.branch
-        self.build_tree(["branch1/d"])
-        bt1.add(['d'], ['file-d'])
-        bt1.commit("branch1, Commit one", rev_id="rev-E")
-
-        #-------- end E -----------
-
-        self.touch(main_wt, "a")
-        main_wt.commit("Commit two", rev_id="rev-B")
-
-        #-------- end B -----------
-
-        bt2 = self.make_branch_and_tree('branch2')
-        bt2.pull(main_branch)
-        branch2_branch = bt2.branch
-        set_executability(bt2, 'b', True)
-        bt2.commit("branch2, Commit one", rev_id="rev-J")
-
-        #-------- end J -----------
-
-        main_wt.merge_from_branch(b1)
-        main_wt.commit("merge branch1, rev-11", rev_id="rev-C")
-
-        #-------- end C -----------
-
-        bt1.rename_one("d","e")
-        bt1.commit("branch1, commit two", rev_id="rev-F")
-
-        #-------- end F -----------
-
-        self.touch(bt2, "c")
-        bt2.commit("branch2, commit two", rev_id="rev-K")
-
-        #-------- end K -----------
-
-        main_wt.merge_from_branch(b1)
-        self.touch(main_wt, "b")
-        # D gets some funky characters to make sure the unescaping works
-        main_wt.commit("merge branch1, rev-12", rev_id="rev-<D>")
-
-        # end D
-
-        main_wt.merge_from_branch(branch2_branch)
-        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
-
-        # end G
-        self.branch = main_branch
-
-    def test_fileids_altered_between_two_revs(self):
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"])
-        self.assertEqual(
-            {'b-file-id-2006-01-01-defg':set(['rev-J']),
-             'c-funky<file-id>quiji%bo':set(['rev-K'])
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"]))
-
-        self.assertEqual(
-            {'b-file-id-2006-01-01-defg': set(['rev-<D>']),
-             'file-d': set(['rev-F']),
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(['rev-<D>', 'rev-F']))
-
-        self.assertEqual(
-            {
-             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
-             'c-funky<file-id>quiji%bo': set(['rev-K']),
-             'file-d': set(['rev-F']),
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(
-                ['rev-<D>', 'rev-G', 'rev-F', 'rev-K', 'rev-J']))
-
-        self.assertEqual(
-            {'a-file-id-2006-01-01-abcd': set(['rev-B']),
-             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
-             'c-funky<file-id>quiji%bo': set(['rev-K']),
-             'file-d': set(['rev-F']),
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(
-                ['rev-G', 'rev-F', 'rev-C', 'rev-B', 'rev-<D>', 'rev-K', 'rev-J']))
-
-    def fileids_altered_by_revision_ids(self, revision_ids):
-        """This is a wrapper to strip TREE_ROOT if it occurs"""
-        repo = self.branch.repository
-        root_id = self.branch.basis_tree().get_root_id()
-        result = repo.fileids_altered_by_revision_ids(revision_ids)
-        if root_id in result:
-            del result[root_id]
-        return result
-
-    def test_fileids_altered_by_revision_ids(self):
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        self.assertEqual(
-            {'a-file-id-2006-01-01-abcd':set(['rev-A']),
-             'b-file-id-2006-01-01-defg': set(['rev-A']),
-             'c-funky<file-id>quiji%bo': set(['rev-A']),
-             },
-            self.fileids_altered_by_revision_ids(["rev-A"]))
-        self.assertEqual(
-            {'a-file-id-2006-01-01-abcd':set(['rev-B'])
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-B"]))
-        self.assertEqual(
-            {'b-file-id-2006-01-01-defg':set(['rev-<D>'])
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-<D>"]))
-
-    def test_fileids_involved_full_compare(self):
-        # this tests that the result of each fileid_involved calculation
-        # along a revision history selects only the fileids selected by
-        # comparing the trees - no less, and no more. This is correct
-        # because in our sample data we do not revert any file ids along
-        # the revision history.
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        pp=[]
-        history = self.branch.revision_history( )
-
-        if len(history) < 2: return
-
-        for start in range(0,len(history)-1):
-            start_id = history[start]
-            for end in range(start+1,len(history)):
-                end_id = history[end]
-                old_revs = set(self.branch.repository.get_ancestry(start_id))
-                new_revs = set(self.branch.repository.get_ancestry(end_id))
-                l1 = self.branch.repository.fileids_altered_by_revision_ids(
-                    new_revs.difference(old_revs))
-                l1 = set(l1.keys())
-
-                l2 = self.compare_tree_fileids(self.branch, start_id, end_id)
-                self.assertEquals(l1, l2)
-
-
-class TestFileIdInvolvedNonAscii(FileIdInvolvedBase):
-
-    def test_utf8_file_ids_and_revision_ids(self):
-        main_wt = self.make_branch_and_tree('main')
-        main_branch = main_wt.branch
-        self.build_tree(["main/a"])
-
-        file_id = u'a-f\xedle-id'.encode('utf8')
-        main_wt.add(['a'], [file_id])
-        revision_id = u'r\xe9v-a'.encode('utf8')
-        try:
-            main_wt.commit('a', rev_id=revision_id)
-        except errors.NonAsciiRevisionId:
-            raise tests.TestSkipped('non-ascii revision ids not supported by %s'
-                                    % self.repository_format)
-
-        repo = main_wt.branch.repository
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        file_ids = repo.fileids_altered_by_revision_ids([revision_id])
-        root_id = main_wt.basis_tree().get_root_id()
-        if root_id in file_ids:
-            self.assertEqual({file_id:set([revision_id]),
-                              root_id:set([revision_id])
-                             }, file_ids)
-        else:
-            self.assertEqual({file_id:set([revision_id])}, file_ids)
-
-
-class TestFileIdInvolvedSuperset(FileIdInvolvedBase):
-
-    def setUp(self):
-        super(TestFileIdInvolvedSuperset, self).setUp()
-
-        self.branch = None
-        main_wt = self.make_branch_and_tree('main')
-        main_branch = main_wt.branch
-        self.build_tree(["main/a","main/b","main/c"])
-
-        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
-                                 'b-file-id-2006-01-01-defg',
-                                 'c-funky<file-id>quiji\'"%bo'])
-        try:
-            main_wt.commit("Commit one", rev_id="rev-A")
-        except errors.IllegalPath:
-            # TODO: jam 20060701 Consider raising a different exception
-            #       newer formats do support this, and nothin can done to
-            #       correct this test - its not a bug.
-            if sys.platform == 'win32':
-                raise tests.TestSkipped('Old repository formats do not'
-                                        ' support file ids with <> on win32')
-            # This is not a known error condition
-            raise
-
-        branch2_wt = self.make_branch_and_tree('branch2')
-        branch2_wt.pull(main_branch)
-        branch2_bzrdir = branch2_wt.bzrdir
-        branch2_branch = branch2_bzrdir.open_branch()
-        set_executability(branch2_wt, 'b', True)
-        branch2_wt.commit("branch2, Commit one", rev_id="rev-J")
-
-        main_wt.merge_from_branch(branch2_branch)
-        set_executability(main_wt, 'b', False)
-        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
-
-        # end G
-        self.branch = main_branch
-
-    def test_fileid_involved_full_compare2(self):
-        # this tests that fileids_altered_by_revision_ids returns
-        # more information than compare_tree can, because it
-        # sees each change rather than the aggregate delta.
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        history = self.branch.revision_history()
-        old_rev = history[0]
-        new_rev = history[1]
-        old_revs = set(self.branch.repository.get_ancestry(old_rev))
-        new_revs = set(self.branch.repository.get_ancestry(new_rev))
-
-        l1 = self.branch.repository.fileids_altered_by_revision_ids(
-            new_revs.difference(old_revs))
-        l1 = set(l1.keys())
-
-        l2 = self.compare_tree_fileids(self.branch, old_rev, new_rev)
-        self.assertNotEqual(l2, l1)
-        self.assertSubset(l2, l1)
-
-
-class FileIdInvolvedWGhosts(per_repository.TestCaseWithRepository):
-
-    def create_branch_with_ghost_text(self):
-        builder = self.make_branch_builder('ghost')
-        builder.build_snapshot('A-id', None, [
-            ('add', ('', 'root-id', 'directory', None)),
-            ('add', ('a', 'a-file-id', 'file', 'some content\n'))])
-        b = builder.get_branch()
-        old_rt = b.repository.revision_tree('A-id')
-        new_inv = old_rt.inventory._get_mutable_inventory()
-        new_inv.revision_id = 'B-id'
-        new_inv['a-file-id'].revision = 'ghost-id'
-        new_rev = _mod_revision.Revision('B-id',
-            timestamp=time.time(),
-            timezone=0,
-            message='Committing against a ghost',
-            committer='Joe Foo <joe@foo.com>',
-            properties={},
-            parent_ids=('A-id', 'ghost-id'),
-            )
-        b.lock_write()
-        self.addCleanup(b.unlock)
-        b.repository.start_write_group()
-        b.repository.add_revision('B-id', new_rev, new_inv)
-        self.disable_commit_write_group_paranoia(b.repository)
-        b.repository.commit_write_group()
-        return b
-
-    def disable_commit_write_group_paranoia(self, repo):
-        if isinstance(repo, remote.RemoteRepository):
-            # We can't easily disable the checks in a remote repo.
-            repo.abort_write_group()
-            raise tests.TestSkipped(
-                "repository format does not support storing revisions with "
-                "missing texts.")
-        pack_coll = getattr(repo, '_pack_collection', None)
-        if pack_coll is not None:
-            # Monkey-patch the pack collection instance to allow storing
-            # incomplete revisions.
-            pack_coll._check_new_inventories = lambda: []
-
-    def test_file_ids_include_ghosts(self):
-        b = self.create_branch_with_ghost_text()
-        repo = b.repository
-        self.assertEqual(
-            {'a-file-id':set(['ghost-id'])},
-            repo.fileids_altered_by_revision_ids(['B-id']))
-
-    def test_file_ids_uses_fallbacks(self):
-        builder = self.make_branch_builder('source',
-                                           format=self.bzrdir_format)
-        repo = builder.get_branch().repository
-        if not repo._format.supports_external_lookups:
-            raise tests.TestNotApplicable('format does not support stacking')
-        builder.start_series()
-        builder.build_snapshot('A-id', None, [
-            ('add', ('', 'root-id', 'directory', None)),
-            ('add', ('file', 'file-id', 'file', 'contents\n'))])
-        builder.build_snapshot('B-id', ['A-id'], [
-            ('modify', ('file-id', 'new-content\n'))])
-        builder.build_snapshot('C-id', ['B-id'], [
-            ('modify', ('file-id', 'yet more content\n'))])
-        builder.finish_series()
-        source_b = builder.get_branch()
-        source_b.lock_read()
-        self.addCleanup(source_b.unlock)
-        base = self.make_branch('base')
-        base.pull(source_b, stop_revision='B-id')
-        stacked = self.make_branch('stacked')
-        stacked.set_stacked_on_url('../base')
-        stacked.pull(source_b, stop_revision='C-id')
-
-        stacked.lock_read()
-        self.addCleanup(stacked.unlock)
-        repo = stacked.repository
-        keys = {'file-id': set(['A-id'])}
-        if stacked.repository.supports_rich_root():
-            keys['root-id'] = set(['A-id'])
-        self.assertEqual(keys, repo.fileids_altered_by_revision_ids(['A-id']))
-
-
-def set_executability(wt, path, executable=True):
-    """Set the executable bit for the file at path in the working tree
-
-    os.chmod() doesn't work on windows. But TreeTransform can mark or
-    unmark a file as executable.
-    """
-    file_id = wt.path2id(path)
-    tt = transform.TreeTransform(wt)
-    try:
-        tt.set_executability(executable, tt.trans_id_tree_file_id(file_id))
-        tt.apply()
-    finally:
-        tt.finalize()

=== modified file 'bzrlib/tests/per_repository/test_has_same_location.py'
--- bzrlib/tests/per_repository/test_has_same_location.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_repository/test_has_same_location.py	2011-08-19 22:34:02 +0000
@@ -20,6 +20,9 @@
     bzrdir,
     transport,
     )
+from bzrlib.remote import (
+    RemoteRepositoryFormat,
+    )
 from bzrlib.tests import (
     TestNotApplicable,
     )
@@ -64,7 +67,7 @@
         """Different repository objects for the same location are the same."""
         repo = self.make_repository('.')
         reopened_repo = repo.bzrdir.open_repository()
-        self.failIf(
+        self.assertFalse(
             repo is reopened_repo,
             "This test depends on reopened_repo being a different instance of "
             "the same repo.")
@@ -84,21 +87,19 @@
         CopyConverter creates a second repository in one bzrdir.
         """
         repo = self.make_repository('repo')
-        try:
-            control_transport = repo._transport
-        except AttributeError:
-            raise TestNotApplicable(
-                "%r has no transport" % (repo,))
-        if control_transport.base == repo.bzrdir.transport.base:
+        if repo.control_transport.base == repo.bzrdir.control_transport.base:
             raise TestNotApplicable(
                 "%r has repository files directly in the bzrdir"
                 % (repo,))
             # This test only applies to repository formats where the repo
             # control_files are separate from other bzrdir files, i.e. metadir
             # formats.
-        control_transport.copy_tree('.', '../repository.backup')
-        backup_transport = control_transport.clone('../repository.backup')
-        backup_repo = repo._format.open(repo.bzrdir, _found=True,
+        repo.control_transport.copy_tree('.', '../repository.backup')
+        backup_transport = repo.control_transport.clone('../repository.backup')
+        if isinstance(repo._format, RemoteRepositoryFormat):
+            raise TestNotApplicable("remote repositories don't support overriding "
+                                    "transport")
+        backup_repo = repo._format.open(repo.bzrdir,
                                         _override_transport=backup_transport)
         self.assertDifferentRepo(repo, backup_repo)
 
@@ -114,9 +115,9 @@
         if repo._format == other_repo._format:
             # We're testing the default format!  So we have to use a non-default
             # format for other_repo.
-            transport.get_transport(self.get_vfs_only_url()
-                                    ).delete_tree('other')
-            other_repo = self.make_repository('other', format='metaweave')
+            transport.get_transport_from_url(
+                self.get_vfs_only_url()).delete_tree('other')
+            other_repo = self.make_repository('other', format='knit')
         # Make sure the other_repo is not a RemoteRepository.
         other_bzrdir = bzrdir.BzrDir.open(self.get_vfs_only_url('other'))
         other_repo = other_bzrdir.open_repository()

=== modified file 'bzrlib/tests/per_repository/test_iter_reverse_revision_history.py'
--- bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2009-06-10 03:56:49 +0000
+++ bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2011-09-09 12:17:12 +0000
@@ -18,9 +18,9 @@
 
 from bzrlib import (
     errors,
-    osutils,
     tests,
     )
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
@@ -92,7 +92,8 @@
         repo = tree.branch.repository
         repo.lock_read()
         self.addCleanup(repo.unlock)
-        rev_history = repo.iter_reverse_revision_history('rev4')
+        rev_history = self.applyDeprecated(deprecated_in((2, 4, 0)),
+            repo.iter_reverse_revision_history, 'rev4')
         self.assertEqual('rev4', rev_history.next())
         self.assertEqual('rev3', rev_history.next())
         self.assertEqual('rev2', rev_history.next())
@@ -103,7 +104,8 @@
         """Assert the return values of iter_reverse_revision_history."""
         repo.lock_read()
         try:
-            actual = list(repo.iter_reverse_revision_history(revision_id))
+            actual = list(self.applyDeprecated(deprecated_in((2, 4, 0)),
+                repo.iter_reverse_revision_history, revision_id))
         finally:
             repo.unlock()
         self.assertEqual(expected, actual)
@@ -148,6 +150,9 @@
 
     def test_ghost(self):
         tree = self.make_branch_and_memory_tree('tree')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable(
+                "repository format does not support ghosts")
         tree.lock_write()
         try:
             tree.add('')
@@ -156,7 +161,8 @@
             tree.commit('2', rev_id='rev2')
         finally:
             tree.unlock()
-        iter = tree.branch.repository.iter_reverse_revision_history('rev2')
+        iter = self.applyDeprecated(deprecated_in((2, 4, 0)),
+            tree.branch.repository.iter_reverse_revision_history, 'rev2')
         tree.branch.repository.lock_read()
         try:
             self.assertEquals('rev2', iter.next())

=== renamed file 'bzrlib/tests/per_repository/test_is_write_locked.py' => 'bzrlib/tests/per_repository/test_locking.py'
--- bzrlib/tests/per_repository/test_is_write_locked.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_locking.py	2011-05-16 12:49:26 +0000
@@ -36,3 +36,22 @@
         repo.lock_write()
         self.addCleanup(repo.unlock)
         self.assertTrue(repo.is_write_locked())
+
+
+class TestIsLocked(TestCaseWithRepository):
+
+    def test_not_locked(self):
+        repo = self.make_repository('.')
+        self.assertFalse(repo.is_locked())
+
+    def test_read_locked(self):
+        repo = self.make_repository('.')
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        self.assertTrue(repo.is_locked())
+
+    def test_write_locked(self):
+        repo = self.make_repository('.')
+        repo.lock_write()
+        self.addCleanup(repo.unlock)
+        self.assertTrue(repo.is_locked())

=== modified file 'bzrlib/tests/per_repository/test_reconcile.py'
--- bzrlib/tests/per_repository/test_reconcile.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_repository/test_reconcile.py	2011-04-19 12:41:46 +0000
@@ -17,430 +17,12 @@
 """Tests for reconciliation of repositories."""
 
 
-import bzrlib
-from bzrlib import (
-    errors,
-    transport,
-    )
-from bzrlib.inventory import Inventory
-from bzrlib.reconcile import reconcile, Reconciler
-from bzrlib.repofmt.knitrepo import RepositoryFormatKnit
-from bzrlib.revision import Revision
-from bzrlib.tests import TestSkipped, TestNotApplicable
-from bzrlib.tests.per_repository.helpers import (
-    TestCaseWithBrokenRevisionIndex,
-    )
 from bzrlib.tests.per_repository import (
     TestCaseWithRepository,
     )
-from bzrlib.uncommit import uncommit
-
-
-class TestReconcile(TestCaseWithRepository):
-
-    def checkUnreconciled(self, d, reconciler):
-        """Check that d did not get reconciled."""
-        # nothing should have been fixed yet:
-        self.assertEqual(0, reconciler.inconsistent_parents)
-        # and no garbage inventories
-        self.assertEqual(0, reconciler.garbage_inventories)
-        self.checkNoBackupInventory(d)
-
-    def checkNoBackupInventory(self, aBzrDir):
-        """Check that there is no backup inventory in aBzrDir."""
-        repo = aBzrDir.open_repository()
-        # Remote repository, and possibly others, do not have
-        # _transport.
-        if getattr(repo, '_transport', None) is not None:
-            for path in repo._transport.list_dir('.'):
-                self.assertFalse('inventory.backup' in path)
-
-
-class TestsNeedingReweave(TestReconcile):
-
-    def setUp(self):
-        super(TestsNeedingReweave, self).setUp()
-
-        t = transport.get_transport(self.get_url())
-        # an empty inventory with no revision for testing with.
-        repo = self.make_repository('inventory_without_revision')
-        repo.lock_write()
-        repo.start_write_group()
-        inv = Inventory(revision_id='missing')
-        inv.root.revision = 'missing'
-        repo.add_inventory('missing', inv, [])
-        repo.commit_write_group()
-        repo.unlock()
-
-        def add_commit(repo, revision_id, parent_ids):
-            repo.lock_write()
-            repo.start_write_group()
-            inv = Inventory(revision_id=revision_id)
-            inv.root.revision = revision_id
-            root_id = inv.root.file_id
-            sha1 = repo.add_inventory(revision_id, inv, parent_ids)
-            repo.texts.add_lines((root_id, revision_id), [], [])
-            rev = bzrlib.revision.Revision(timestamp=0,
-                                           timezone=None,
-                                           committer="Foo Bar <foo@example.com>",
-                                           message="Message",
-                                           inventory_sha1=sha1,
-                                           revision_id=revision_id)
-            rev.parent_ids = parent_ids
-            repo.add_revision(revision_id, rev)
-            repo.commit_write_group()
-            repo.unlock()
-        # an empty inventory with no revision for testing with.
-        # this is referenced by 'references_missing' to let us test
-        # that all the cached data is correctly converted into ghost links
-        # and the referenced inventory still cleaned.
-        repo = self.make_repository('inventory_without_revision_and_ghost')
-        repo.lock_write()
-        repo.start_write_group()
-        repo.add_inventory('missing', inv, [])
-        repo.commit_write_group()
-        repo.unlock()
-        add_commit(repo, 'references_missing', ['missing'])
-
-        # a inventory with no parents and the revision has parents..
-        # i.e. a ghost.
-        repo = self.make_repository('inventory_one_ghost')
-        add_commit(repo, 'ghost', ['the_ghost'])
-
-        # a inventory with a ghost that can be corrected now.
-        t.copy_tree('inventory_one_ghost', 'inventory_ghost_present')
-        bzrdir_url = self.get_url('inventory_ghost_present')
-        bzrdir = bzrlib.bzrdir.BzrDir.open(bzrdir_url)
-        repo = bzrdir.open_repository()
-        add_commit(repo, 'the_ghost', [])
-
-    def checkEmptyReconcile(self, **kwargs):
-        """Check a reconcile on an empty repository."""
-        self.make_repository('empty')
-        d = bzrlib.bzrdir.BzrDir.open(self.get_url('empty'))
-        # calling on a empty repository should do nothing
-        reconciler = d.find_repository().reconcile(**kwargs)
-        # no inconsistent parents should have been found
-        self.assertEqual(0, reconciler.inconsistent_parents)
-        # and no garbage inventories
-        self.assertEqual(0, reconciler.garbage_inventories)
-        # and no backup weave should have been needed/made.
-        self.checkNoBackupInventory(d)
-
-    def test_reconcile_empty(self):
-        # in an empty repo, theres nothing to do.
-        self.checkEmptyReconcile()
-
-    def test_repo_has_reconcile_does_inventory_gc_attribute(self):
-        repo = self.make_repository('repo')
-        self.assertNotEqual(None, repo._reconcile_does_inventory_gc)
-
-    def test_reconcile_empty_thorough(self):
-        # reconcile should accept thorough=True
-        self.checkEmptyReconcile(thorough=True)
-
-    def test_convenience_reconcile_inventory_without_revision_reconcile(self):
-        # smoke test for the all in one ui tool
-        bzrdir_url = self.get_url('inventory_without_revision')
-        bzrdir = bzrlib.bzrdir.BzrDir.open(bzrdir_url)
-        repo = bzrdir.open_repository()
-        if not repo._reconcile_does_inventory_gc:
-            raise TestSkipped('Irrelevant test')
-        reconcile(bzrdir)
-        # now the backup should have it but not the current inventory
-        repo = bzrdir.open_repository()
-        self.check_missing_was_removed(repo)
-
-    def test_reweave_inventory_without_revision(self):
-        # an excess inventory on its own is only reconciled by using thorough
-        d_url = self.get_url('inventory_without_revision')
-        d = bzrlib.bzrdir.BzrDir.open(d_url)
-        repo = d.open_repository()
-        if not repo._reconcile_does_inventory_gc:
-            raise TestSkipped('Irrelevant test')
-        self.checkUnreconciled(d, repo.reconcile())
-        reconciler = repo.reconcile(thorough=True)
-        # no bad parents
-        self.assertEqual(0, reconciler.inconsistent_parents)
-        # and one garbage inventory
-        self.assertEqual(1, reconciler.garbage_inventories)
-        self.check_missing_was_removed(repo)
-
-    def check_thorough_reweave_missing_revision(self, aBzrDir, reconcile,
-            **kwargs):
-        # actual low level test.
-        repo = aBzrDir.open_repository()
-        if ([None, 'missing', 'references_missing']
-            != repo.get_ancestry('references_missing')):
-            # the repo handles ghosts without corruption, so reconcile has
-            # nothing to do here. Specifically, this test has the inventory
-            # 'missing' present and the revision 'missing' missing, so clearly
-            # 'missing' cannot be reported in the present ancestry -> missing
-            # is something that can be filled as a ghost.
-            expected_inconsistent_parents = 0
-        else:
-            expected_inconsistent_parents = 1
-        reconciler = reconcile(**kwargs)
-        # some number of inconsistent parents should have been found
-        self.assertEqual(expected_inconsistent_parents,
-                         reconciler.inconsistent_parents)
-        # and one garbage inventories
-        self.assertEqual(1, reconciler.garbage_inventories)
-        # now the backup should have it but not the current inventory
-        repo = aBzrDir.open_repository()
-        self.check_missing_was_removed(repo)
-        # and the parent list for 'references_missing' should have that
-        # revision a ghost now.
-        self.assertEqual([None, 'references_missing'],
-                         repo.get_ancestry('references_missing'))
-
-    def check_missing_was_removed(self, repo):
-        if repo._reconcile_backsup_inventory:
-            backed_up = False
-            for path in repo._transport.list_dir('.'):
-                if 'inventory.backup' in path:
-                    backed_up = True
-            self.assertTrue(backed_up)
-            # Not clear how to do this at an interface level:
-            # self.assertTrue('missing' in backup.versions())
-        self.assertRaises(errors.NoSuchRevision, repo.get_inventory, 'missing')
-
-    def test_reweave_inventory_without_revision_reconciler(self):
-        # smoke test for the all in one Reconciler class,
-        # other tests use the lower level repo.reconcile()
-        d_url = self.get_url('inventory_without_revision_and_ghost')
-        d = bzrlib.bzrdir.BzrDir.open(d_url)
-        if not d.open_repository()._reconcile_does_inventory_gc:
-            raise TestSkipped('Irrelevant test')
-        def reconcile():
-            reconciler = Reconciler(d)
-            reconciler.reconcile()
-            return reconciler
-        self.check_thorough_reweave_missing_revision(d, reconcile)
-
-    def test_reweave_inventory_without_revision_and_ghost(self):
-        # actual low level test.
-        d_url = self.get_url('inventory_without_revision_and_ghost')
-        d = bzrlib.bzrdir.BzrDir.open(d_url)
-        repo = d.open_repository()
-        if not repo._reconcile_does_inventory_gc:
-            raise TestSkipped('Irrelevant test')
-        # nothing should have been altered yet : inventories without
-        # revisions are not data loss incurring for current format
-        self.check_thorough_reweave_missing_revision(d, repo.reconcile,
-            thorough=True)
-
-    def test_reweave_inventory_preserves_a_revision_with_ghosts(self):
-        d = bzrlib.bzrdir.BzrDir.open(self.get_url('inventory_one_ghost'))
-        reconciler = d.open_repository().reconcile(thorough=True)
-        # no inconsistent parents should have been found:
-        # the lack of a parent for ghost is normal
-        self.assertEqual(0, reconciler.inconsistent_parents)
-        # and one garbage inventories
-        self.assertEqual(0, reconciler.garbage_inventories)
-        # now the current inventory should still have 'ghost'
-        repo = d.open_repository()
-        repo.get_inventory('ghost')
-        self.assertEqual([None, 'ghost'], repo.get_ancestry('ghost'))
-
-    def test_reweave_inventory_fixes_ancestryfor_a_present_ghost(self):
-        d = bzrlib.bzrdir.BzrDir.open(self.get_url('inventory_ghost_present'))
-        repo = d.open_repository()
-        ghost_ancestry = repo.get_ancestry('ghost')
-        if ghost_ancestry == [None, 'the_ghost', 'ghost']:
-            # the repo handles ghosts without corruption, so reconcile has
-            # nothing to do
-            return
-        self.assertEqual([None, 'ghost'], ghost_ancestry)
-        reconciler = repo.reconcile()
-        # this is a data corrupting error, so a normal reconcile should fix it.
-        # one inconsistent parents should have been found : the
-        # available but not reference parent for ghost.
-        self.assertEqual(1, reconciler.inconsistent_parents)
-        # and no garbage inventories
-        self.assertEqual(0, reconciler.garbage_inventories)
-        # now the current inventory should still have 'ghost'
-        repo = d.open_repository()
-        repo.get_inventory('ghost')
-        repo.get_inventory('the_ghost')
-        self.assertEqual([None, 'the_ghost', 'ghost'], repo.get_ancestry('ghost'))
-        self.assertEqual([None, 'the_ghost'], repo.get_ancestry('the_ghost'))
-
-    def test_text_from_ghost_revision(self):
-        repo = self.make_repository('text-from-ghost')
-        inv = Inventory(revision_id='final-revid')
-        inv.root.revision = 'root-revid'
-        ie = inv.add_path('bla', 'file', 'myfileid')
-        ie.revision = 'ghostrevid'
-        ie.text_size = 42
-        ie.text_sha1 = "bee68c8acd989f5f1765b4660695275948bf5c00"
-        rev = bzrlib.revision.Revision(timestamp=0,
-                                       timezone=None,
-                                       committer="Foo Bar <foo@example.com>",
-                                       message="Message",
-                                       revision_id='final-revid')
-        repo.lock_write()
-        try:
-            repo.start_write_group()
-            try:
-                repo.add_revision('final-revid', rev, inv)
-                try:
-                    repo.texts.add_lines(('myfileid', 'ghostrevid'),
-                        (('myfileid', 'ghost-text-parent'),),
-                        ["line1\n", "line2\n"])
-                except errors.RevisionNotPresent:
-                    raise TestSkipped("text ghost parents not supported")
-                if repo.supports_rich_root():
-                    root_id = inv.root.file_id
-                    repo.texts.add_lines((inv.root.file_id, inv.root.revision),
-                        [], [])
-            finally:
-                repo.commit_write_group()
-        finally:
-            repo.unlock()
-        repo.reconcile(thorough=True)
-
-
-class TestReconcileWithIncorrectRevisionCache(TestReconcile):
-    """Ancestry data gets cached in knits and weaves should be reconcilable.
-
-    This class tests that reconcile can correct invalid caches (such as after
-    a reconcile).
-    """
-
-    def setUp(self):
-        self.reduceLockdirTimeout()
-        super(TestReconcileWithIncorrectRevisionCache, self).setUp()
-
-        t = transport.get_transport(self.get_url())
-        # we need a revision with two parents in the wrong order
-        # which should trigger reinsertion.
-        # and another with the first one correct but the other two not
-        # which should not trigger reinsertion.
-        # these need to be in different repositories so that we don't
-        # trigger a reconcile based on the other case.
-        # there is no api to construct a broken knit repository at
-        # this point. if we ever encounter a bad graph in a knit repo
-        # we should add a lower level api to allow constructing such cases.
-
-        # first off the common logic:
-        tree = self.make_branch_and_tree('wrong-first-parent')
-        second_tree = self.make_branch_and_tree('reversed-secondary-parents')
-        for t in [tree, second_tree]:
-            t.commit('1', rev_id='1')
-            uncommit(t.branch, tree=t)
-            t.commit('2', rev_id='2')
-            uncommit(t.branch, tree=t)
-            t.commit('3', rev_id='3')
-            uncommit(t.branch, tree=t)
-        #second_tree = self.make_branch_and_tree('reversed-secondary-parents')
-        #second_tree.pull(tree) # XXX won't copy the repo?
-        repo_secondary = second_tree.branch.repository
-
-        # now setup the wrong-first parent case
-        repo = tree.branch.repository
-        repo.lock_write()
-        repo.start_write_group()
-        inv = Inventory(revision_id='wrong-first-parent')
-        inv.root.revision = 'wrong-first-parent'
-        if repo.supports_rich_root():
-            root_id = inv.root.file_id
-            repo.texts.add_lines((root_id, 'wrong-first-parent'), [], [])
-        sha1 = repo.add_inventory('wrong-first-parent', inv, ['2', '1'])
-        rev = Revision(timestamp=0,
-                       timezone=None,
-                       committer="Foo Bar <foo@example.com>",
-                       message="Message",
-                       inventory_sha1=sha1,
-                       revision_id='wrong-first-parent')
-        rev.parent_ids = ['1', '2']
-        repo.add_revision('wrong-first-parent', rev)
-        repo.commit_write_group()
-        repo.unlock()
-
-        # now setup the wrong-secondary parent case
-        repo = repo_secondary
-        repo.lock_write()
-        repo.start_write_group()
-        inv = Inventory(revision_id='wrong-secondary-parent')
-        inv.root.revision = 'wrong-secondary-parent'
-        if repo.supports_rich_root():
-            root_id = inv.root.file_id
-            repo.texts.add_lines((root_id, 'wrong-secondary-parent'), [], [])
-        sha1 = repo.add_inventory('wrong-secondary-parent', inv, ['1', '3', '2'])
-        rev = Revision(timestamp=0,
-                       timezone=None,
-                       committer="Foo Bar <foo@example.com>",
-                       message="Message",
-                       inventory_sha1=sha1,
-                       revision_id='wrong-secondary-parent')
-        rev.parent_ids = ['1', '2', '3']
-        repo.add_revision('wrong-secondary-parent', rev)
-        repo.commit_write_group()
-        repo.unlock()
-
-    def test_reconcile_wrong_order(self):
-        # a wrong order in primary parents is optionally correctable
-        t = transport.get_transport(self.get_url()).clone('wrong-first-parent')
-        d = bzrlib.bzrdir.BzrDir.open_from_transport(t)
-        repo = d.open_repository()
-        repo.lock_read()
-        try:
-            g = repo.get_graph()
-            if g.get_parent_map(['wrong-first-parent'])['wrong-first-parent'] \
-                == ('1', '2'):
-                raise TestSkipped('wrong-first-parent is not setup for testing')
-        finally:
-            repo.unlock()
-        self.checkUnreconciled(d, repo.reconcile())
-        # nothing should have been altered yet : inventories without
-        # revisions are not data loss incurring for current format
-        reconciler = repo.reconcile(thorough=True)
-        # these show up as inconsistent parents
-        self.assertEqual(1, reconciler.inconsistent_parents)
-        # and no garbage inventories
-        self.assertEqual(0, reconciler.garbage_inventories)
-        # and should have been fixed:
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        g = repo.get_graph()
-        self.assertEqual(
-            {'wrong-first-parent':('1', '2')},
-            g.get_parent_map(['wrong-first-parent']))
-
-    def test_reconcile_wrong_order_secondary_inventory(self):
-        # a wrong order in the parents for inventories is ignored.
-        t = transport.get_transport(self.get_url()
-                                    ).clone('reversed-secondary-parents')
-        d = bzrlib.bzrdir.BzrDir.open_from_transport(t)
-        repo = d.open_repository()
-        self.checkUnreconciled(d, repo.reconcile())
-        self.checkUnreconciled(d, repo.reconcile(thorough=True))
-
-
-class TestBadRevisionParents(TestCaseWithBrokenRevisionIndex):
-
-    def test_aborts_if_bad_parents_in_index(self):
-        """Reconcile refuses to proceed if the revision index is wrong when
-        checked against the revision texts, so that it does not generate broken
-        data.
-
-        Ideally reconcile would fix this, but until we implement that we just
-        make sure we safely detect this problem.
-        """
-        repo = self.make_repo_with_extra_ghost_index()
-        reconciler = repo.reconcile(thorough=True)
-        self.assertTrue(reconciler.aborted,
-            "reconcile should have aborted due to bad parents.")
-
-    def test_does_not_abort_on_clean_repo(self):
-        repo = self.make_repository('.')
-        reconciler = repo.reconcile(thorough=True)
-        self.assertFalse(reconciler.aborted,
-            "reconcile should not have aborted on an unbroken repository.")
-
-
-class TestRepeatedReconcile(TestReconcile):
+
+
+class TestRepeatedReconcile(TestCaseWithRepository):
 
     def test_trivial_two_reconciles_no_error(self):
         tree = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/per_repository/test_refresh_data.py'
--- bzrlib/tests/per_repository/test_refresh_data.py	2010-05-11 14:14:55 +0000
+++ bzrlib/tests/per_repository/test_refresh_data.py	2011-08-16 20:29:23 +0000
@@ -17,10 +17,8 @@
 """Tests for Repository.refresh_data."""
 
 from bzrlib import (
-    errors,
     repository,
     )
-from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
@@ -61,60 +59,3 @@
         else:
             # This is ok too.
             pass
-
-    def fetch_new_revision_into_concurrent_instance(self, repo, token):
-        """Create a new revision (revid 'new-rev') and fetch it into a
-        concurrent instance of repo.
-        """
-        source = self.make_branch_and_memory_tree('source')
-        source.lock_write()
-        self.addCleanup(source.unlock)
-        source.add([''], ['root-id'])
-        revid = source.commit('foo', rev_id='new-rev')
-        # Force data reading on weaves/knits
-        repo.all_revision_ids()
-        repo.revisions.keys()
-        repo.inventories.keys()
-        # server repo is the instance a smart server might hold for this
-        # repository.
-        server_repo = repo.bzrdir.open_repository()
-        try:
-            server_repo.lock_write(token)
-        except errors.TokenLockingNotSupported:
-            raise TestSkipped('Cannot concurrently insert into repo format %r'
-                % self.repository_format)
-        try:
-            server_repo.fetch(source.branch.repository, revid)
-        finally:
-            server_repo.unlock()
-
-    def test_refresh_data_after_fetch_new_data_visible(self):
-        repo = self.make_repository('target')
-        token = repo.lock_write().repository_token
-        self.addCleanup(repo.unlock)
-        self.fetch_new_revision_into_concurrent_instance(repo, token)
-        repo.refresh_data()
-        self.assertNotEqual({}, repo.get_graph().get_parent_map(['new-rev']))
-
-    def test_refresh_data_after_fetch_new_data_visible_in_write_group(self):
-        tree = self.make_branch_and_memory_tree('target')
-        tree.lock_write()
-        self.addCleanup(tree.unlock)
-        tree.add([''], ['root-id'])
-        tree.commit('foo', rev_id='commit-in-target')
-        repo = tree.branch.repository
-        token = repo.lock_write().repository_token
-        self.addCleanup(repo.unlock)
-        repo.start_write_group()
-        self.addCleanup(repo.abort_write_group)
-        self.fetch_new_revision_into_concurrent_instance(repo, token)
-        # Call refresh_data.  It either fails with IsInWriteGroupError, or it
-        # succeeds and the new revisions are visible.
-        try:
-            repo.refresh_data()
-        except repository.IsInWriteGroupError:
-            pass
-        else:
-            self.assertEqual(
-                ['commit-in-target', 'new-rev'],
-                sorted(repo.all_revision_ids()))

=== modified file 'bzrlib/tests/per_repository/test_repository.py'
--- bzrlib/tests/per_repository/test_repository.py	2010-05-06 23:41:35 +0000
+++ bzrlib/tests/per_repository/test_repository.py	2011-10-13 16:52:17 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,27 +21,22 @@
 
 from bzrlib import (
     branch as _mod_branch,
-    bzrdir,
+    controldir,
     delta as _mod_delta,
     errors,
     gpg,
-    graph,
     info,
     inventory,
-    osutils,
     remote,
     repository,
     revision as _mod_revision,
     tests,
     transport,
     upgrade,
-    versionedfile,
     workingtree,
-    xml_serializer,
     )
 from bzrlib.repofmt import (
-    pack_repo,
-    weaverepo,
+    knitpack_repo,
     )
 from bzrlib.tests import (
     per_repository,
@@ -69,18 +64,15 @@
         repo = self.make_repository('repo')
         self.assertSubset([getattr(repo._format, attribute)], allowed_values)
 
-    def test_attribute__fetch_order(self):
-        """Test the _fetch_order attribute."""
-        self.assertFormatAttribute('_fetch_order', ('topological', 'unordered'))
-
-    def test_attribute__fetch_uses_deltas(self):
-        """Test the _fetch_uses_deltas attribute."""
-        self.assertFormatAttribute('_fetch_uses_deltas', (True, False))
-
     def test_attribute_fast_deltas(self):
         """Test the format.fast_deltas attribute."""
         self.assertFormatAttribute('fast_deltas', (True, False))
 
+    def test_attribute_supports_nesting_repositories(self):
+        """Test the format.supports_nesting_repositories."""
+        self.assertFormatAttribute('supports_nesting_repositories',
+            (True, False))
+
     def test_attribute__fetch_reconcile(self):
         """Test the _fetch_reconcile attribute."""
         self.assertFormatAttribute('_fetch_reconcile', (True, False))
@@ -91,144 +83,32 @@
     def test_attribute_format_pack_compresses(self):
         self.assertFormatAttribute('pack_compresses', (True, False))
 
-    def test_attribute_inventories_store(self):
-        """Test the existence of the inventories attribute."""
-        tree = self.make_branch_and_tree('tree')
-        repo = tree.branch.repository
-        self.assertIsInstance(repo.inventories, versionedfile.VersionedFiles)
-
-    def test_attribute_inventories_basics(self):
-        """Test basic aspects of the inventories attribute."""
-        tree = self.make_branch_and_tree('tree')
-        repo = tree.branch.repository
-        rev_id = (tree.commit('a'),)
-        tree.lock_read()
-        self.addCleanup(tree.unlock)
-        self.assertEqual(set([rev_id]), set(repo.inventories.keys()))
-
-    def test_attribute_revision_store(self):
-        """Test the existence of the revisions attribute."""
-        tree = self.make_branch_and_tree('tree')
-        repo = tree.branch.repository
-        self.assertIsInstance(repo.revisions,
-            versionedfile.VersionedFiles)
-
-    def test_attribute_revision_store_basics(self):
-        """Test the basic behaviour of the revisions attribute."""
-        tree = self.make_branch_and_tree('tree')
-        repo = tree.branch.repository
-        repo.lock_write()
-        try:
-            self.assertEqual(set(), set(repo.revisions.keys()))
-            revid = (tree.commit("foo"),)
-            self.assertEqual(set([revid]), set(repo.revisions.keys()))
-            self.assertEqual({revid:()},
-                repo.revisions.get_parent_map([revid]))
-        finally:
-            repo.unlock()
-        tree2 = self.make_branch_and_tree('tree2')
-        tree2.pull(tree.branch)
-        left_id = (tree2.commit('left'),)
-        right_id = (tree.commit('right'),)
-        tree.merge_from_branch(tree2.branch)
-        merge_id = (tree.commit('merged'),)
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        self.assertEqual(set([revid, left_id, right_id, merge_id]),
-            set(repo.revisions.keys()))
-        self.assertEqual({revid:(), left_id:(revid,), right_id:(revid,),
-             merge_id:(right_id, left_id)},
-            repo.revisions.get_parent_map(repo.revisions.keys()))
-
-    def test_attribute_signature_store(self):
-        """Test the existence of the signatures attribute."""
-        tree = self.make_branch_and_tree('tree')
-        repo = tree.branch.repository
-        self.assertIsInstance(repo.signatures,
-            versionedfile.VersionedFiles)
-
-    def test_attribute_text_store_basics(self):
-        """Test the basic behaviour of the text store."""
-        tree = self.make_branch_and_tree('tree')
-        repo = tree.branch.repository
-        file_id = "Foo:Bar"
-        file_key = (file_id,)
-        tree.lock_write()
-        try:
-            self.assertEqual(set(), set(repo.texts.keys()))
-            tree.add(['foo'], [file_id], ['file'])
-            tree.put_file_bytes_non_atomic(file_id, 'content\n')
-            try:
-                rev_key = (tree.commit("foo"),)
-            except errors.IllegalPath:
-                raise tests.TestNotApplicable(
-                    'file_id %r cannot be stored on this'
-                    ' platform for this repo format' % (file_id,))
-            if repo._format.rich_root_data:
-                root_commit = (tree.get_root_id(),) + rev_key
-                keys = set([root_commit])
-                parents = {root_commit:()}
-            else:
-                keys = set()
-                parents = {}
-            keys.add(file_key + rev_key)
-            parents[file_key + rev_key] = ()
-            self.assertEqual(keys, set(repo.texts.keys()))
-            self.assertEqual(parents,
-                repo.texts.get_parent_map(repo.texts.keys()))
-        finally:
-            tree.unlock()
-        tree2 = self.make_branch_and_tree('tree2')
-        tree2.pull(tree.branch)
-        tree2.put_file_bytes_non_atomic('Foo:Bar', 'right\n')
-        right_key = (tree2.commit('right'),)
-        keys.add(file_key + right_key)
-        parents[file_key + right_key] = (file_key + rev_key,)
-        tree.put_file_bytes_non_atomic('Foo:Bar', 'left\n')
-        left_key = (tree.commit('left'),)
-        keys.add(file_key + left_key)
-        parents[file_key + left_key] = (file_key + rev_key,)
-        tree.merge_from_branch(tree2.branch)
-        tree.put_file_bytes_non_atomic('Foo:Bar', 'merged\n')
-        try:
-            tree.auto_resolve()
-        except errors.UnsupportedOperation:
-            pass
-        merge_key = (tree.commit('merged'),)
-        keys.add(file_key + merge_key)
-        parents[file_key + merge_key] = (file_key + left_key,
-                                         file_key + right_key)
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        self.assertEqual(keys, set(repo.texts.keys()))
-        self.assertEqual(parents, repo.texts.get_parent_map(repo.texts.keys()))
-
-    def test_attribute_text_store(self):
-        """Test the existence of the texts attribute."""
-        tree = self.make_branch_and_tree('tree')
-        repo = tree.branch.repository
-        self.assertIsInstance(repo.texts,
-            versionedfile.VersionedFiles)
-
-    def test_exposed_versioned_files_are_marked_dirty(self):
-        repo = self.make_repository('.')
-        repo.lock_write()
-        signatures = repo.signatures
-        revisions = repo.revisions
-        inventories = repo.inventories
-        repo.unlock()
-        self.assertRaises(errors.ObjectNotLocked,
-            signatures.keys)
-        self.assertRaises(errors.ObjectNotLocked,
-            revisions.keys)
-        self.assertRaises(errors.ObjectNotLocked,
-            inventories.keys)
-        self.assertRaises(errors.ObjectNotLocked,
-            signatures.add_lines, ('foo',), [], [])
-        self.assertRaises(errors.ObjectNotLocked,
-            revisions.add_lines, ('foo',), [], [])
-        self.assertRaises(errors.ObjectNotLocked,
-            inventories.add_lines, ('foo',), [], [])
+    def test_attribute_format_supports_full_versioned_files(self):
+        self.assertFormatAttribute('supports_full_versioned_files',
+            (True, False))
+
+    def test_attribute_format_supports_funky_characters(self):
+        self.assertFormatAttribute('supports_funky_characters',
+            (True, False))
+
+    def test_attribute_format_supports_leaving_lock(self):
+        self.assertFormatAttribute('supports_leaving_lock',
+            (True, False))
+
+    def test_attribute_format_versioned_directories(self):
+        self.assertFormatAttribute('supports_versioned_directories', (True, False))
+
+    def test_attribute_format_revision_graph_can_have_wrong_parents(self):
+        self.assertFormatAttribute('revision_graph_can_have_wrong_parents',
+            (True, False))
+
+    def test_format_is_deprecated(self):
+        repo = self.make_repository('repo')
+        self.assertSubset([repo._format.is_deprecated()], (True, False))
+
+    def test_format_is_supported(self):
+        repo = self.make_repository('repo')
+        self.assertSubset([repo._format.is_supported()], (True, False))
 
     def test_clone_to_default_format(self):
         #TODO: Test that cloning a repository preserves all the information
@@ -246,19 +126,6 @@
         tree_b.get_file_text('file1')
         rev1 = repo_b.get_revision('rev1')
 
-    def test_iter_inventories_is_ordered(self):
-        # just a smoke test
-        tree = self.make_branch_and_tree('a')
-        first_revision = tree.commit('')
-        second_revision = tree.commit('')
-        tree.lock_read()
-        self.addCleanup(tree.unlock)
-        revs = (first_revision, second_revision)
-        invs = tree.branch.repository.iter_inventories(revs)
-        for rev_id, inv in zip(revs, invs):
-            self.assertEqual(rev_id, inv.revision_id)
-            self.assertIsInstance(inv, inventory.CommonInventory)
-
     def test_supports_rich_root(self):
         tree = self.make_branch_and_tree('a')
         tree.commit('')
@@ -266,8 +133,8 @@
         rev_tree = tree.branch.repository.revision_tree(second_revision)
         rev_tree.lock_read()
         self.addCleanup(rev_tree.unlock)
-        inv = rev_tree.inventory
-        rich_root = (inv.root.revision != second_revision)
+        root_revision = rev_tree.get_file_revision(rev_tree.get_root_id())
+        rich_root = (root_revision != second_revision)
         self.assertEqual(rich_root,
                          tree.branch.repository.supports_rich_root())
 
@@ -282,14 +149,14 @@
             # they may not be initializable.
             return
         # supported formats must be able to init and open
-        t = transport.get_transport(self.get_url())
-        readonly_t = transport.get_transport(self.get_readonly_url())
+        t = self.get_transport()
+        readonly_t = self.get_readonly_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = self.repository_format.initialize(made_control)
         self.assertEqual(made_control, made_repo.bzrdir)
 
-        # find it via bzrdir opening:
-        opened_control = bzrdir.BzrDir.open(readonly_t.base)
+        # find it via controldir opening:
+        opened_control = controldir.ControlDir.open(readonly_t.base)
         direct_opened_repo = opened_control.open_repository()
         self.assertEqual(direct_opened_repo.__class__, made_repo.__class__)
         self.assertEqual(opened_control, direct_opened_repo.bzrdir)
@@ -298,7 +165,7 @@
                               self.repository_format.__class__)
         # find it via Repository.open
         opened_repo = repository.Repository.open(readonly_t.base)
-        self.failUnless(isinstance(opened_repo, made_repo.__class__))
+        self.assertIsInstance(opened_repo, made_repo.__class__)
         self.assertEqual(made_repo._format.__class__,
                          opened_repo._format.__class__)
         # if it has a unique id string, can we probe for it ?
@@ -342,7 +209,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = transport.get_transport(self.get_url())
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
         # Check that we have a repository object.
@@ -356,7 +223,7 @@
             # because the default open will not open them and
             # they may not be initializable.
             return
-        t = transport.get_transport(self.get_url())
+        t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         try:
             made_repo = made_control.create_repository(shared=True)
@@ -376,7 +243,8 @@
         tree = wt.branch.repository.revision_tree('revision-1')
         tree.lock_read()
         try:
-            self.assertEqual('revision-1', tree.inventory.root.revision)
+            self.assertEqual('revision-1',
+                tree.get_file_revision(tree.get_root_id()))
             expected = inventory.InventoryDirectory('fixed-root', '', None)
             expected.revision = 'revision-1'
             self.assertEqual([('', 'V', 'directory', 'fixed-root', expected)],
@@ -484,7 +352,7 @@
             return
         try:
             made_repo.set_make_working_trees(False)
-        except NotImplementedError:
+        except errors.UnsupportedOperation:
             # the repository does not support having its tree-making flag
             # toggled.
             return
@@ -502,15 +370,20 @@
         repo = wt.branch.repository
         repo.lock_write()
         repo.start_write_group()
-        repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
+        try:
+            repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
+        except errors.UnsupportedOperation:
+            self.assertFalse(repo._format.supports_revision_signatures)
+            raise tests.TestNotApplicable("signatures not supported by repository format")
         repo.commit_write_group()
         repo.unlock()
         old_signature = repo.get_signature_text('A')
         try:
-            old_format = bzrdir.BzrDirFormat.get_default_format()
+            old_format = controldir.ControlDirFormat.get_default_format()
             # This gives metadir branches something they can convert to.
             # it would be nice to have a 'latest' vs 'default' concept.
-            format = bzrdir.format_registry.make_bzrdir('dirstate-with-subtree')
+            format = controldir.format_registry.make_bzrdir(
+                'dirstate-with-subtree')
             upgrade.upgrade(repo.bzrdir.root_transport.base, format=format)
         except errors.UpToDateFormat:
             # this is in the most current format already.
@@ -524,7 +397,7 @@
     def test_format_description(self):
         repo = self.make_repository('.')
         text = repo._format.get_format_description()
-        self.failUnless(len(text))
+        self.assertTrue(len(text))
 
     def test_format_supports_external_lookups(self):
         repo = self.make_repository('.')
@@ -536,7 +409,8 @@
         tree = self.make_branch_and_tree('.')
         tree.commit(message, rev_id='a', allow_pointless=True)
         rev = tree.branch.repository.get_revision('a')
-        if tree.branch.repository._serializer.squashes_xml_invalid_characters:
+        serializer = getattr(tree.branch.repository, "_serializer", None)
+        if serializer is not None and serializer.squashes_xml_invalid_characters:
             # we have to manually escape this as we dont try to
             # roundtrip xml invalid characters in the xml-based serializers.
             escaped_message, escape_count = re.subn(
@@ -597,24 +471,8 @@
         rev_tree = tree.branch.repository.revision_tree(tree.last_revision())
         rev_tree.lock_read()
         self.addCleanup(rev_tree.unlock)
-        self.assertEqual('rev_id', rev_tree.inventory.root.revision)
-
-    def test_upgrade_from_format4(self):
-        from bzrlib.tests.test_upgrade import _upgrade_dir_template
-        if isinstance(self.repository_format, remote.RemoteRepositoryFormat):
-            return # local conversion to/from RemoteObjects is irrelevant.
-        if self.repository_format.get_format_description() \
-            == "Repository format 4":
-            raise tests.TestSkipped('Cannot convert format-4 to itself')
-        self.build_tree_contents(_upgrade_dir_template)
-        old_repodir = bzrdir.BzrDir.open_unsupported('.')
-        old_repo_format = old_repodir.open_repository()._format
-        format = self.repository_format._matchingbzrdir
-        try:
-            format.repository_format = self.repository_format
-        except AttributeError:
-            pass
-        upgrade.upgrade('.', format)
+        root_id = rev_tree.get_root_id()
+        self.assertEqual('rev_id', rev_tree.get_file_revision(root_id))
 
     def test_pointless_commit(self):
         tree = self.make_branch_and_tree('.')
@@ -630,11 +488,6 @@
         repo._format.rich_root_data
         repo._format.supports_tree_reference
 
-    def test_get_serializer_format(self):
-        repo = self.make_repository('.')
-        format = repo.get_serializer_format()
-        self.assertEqual(repo._serializer.format_num, format)
-
     def test_iter_files_bytes(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree_contents([('tree/file1', 'foo'),
@@ -662,50 +515,6 @@
                           repository.iter_files_bytes(
                           [('file3-id', 'rev3', 'file1-notpresent')]))
 
-    def test_item_keys_introduced_by(self):
-        # Make a repo with one revision and one versioned file.
-        tree = self.make_branch_and_tree('t')
-        self.build_tree(['t/foo'])
-        tree.add('foo', 'file1')
-        tree.commit('message', rev_id='rev_id')
-        repo = tree.branch.repository
-        repo.lock_write()
-        repo.start_write_group()
-        repo.sign_revision('rev_id', gpg.LoopbackGPGStrategy(None))
-        repo.commit_write_group()
-        repo.unlock()
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-
-        # Item keys will be in this order, for maximum convenience for
-        # generating data to insert into knit repository:
-        #   * files
-        #   * inventory
-        #   * signatures
-        #   * revisions
-        expected_item_keys = [
-            ('file', 'file1', ['rev_id']),
-            ('inventory', None, ['rev_id']),
-            ('signatures', None, ['rev_id']),
-            ('revisions', None, ['rev_id'])]
-        item_keys = list(repo.item_keys_introduced_by(['rev_id']))
-        item_keys = [
-            (kind, file_id, list(versions))
-            for (kind, file_id, versions) in item_keys]
-
-        if repo.supports_rich_root():
-            # Check for the root versioned file in the item_keys, then remove
-            # it from streamed_names so we can compare that with
-            # expected_record_names.
-            # Note that the file keys can be in any order, so this test is
-            # written to allow that.
-            inv = repo.get_inventory('rev_id')
-            root_item_key = ('file', inv.root.file_id, ['rev_id'])
-            self.assertTrue(root_item_key in item_keys)
-            item_keys.remove(root_item_key)
-
-        self.assertEqual(expected_item_keys, item_keys)
-
     def test_get_graph(self):
         """Bare-bones smoketest that all repositories implement get_graph."""
         repo = self.make_repository('repo')
@@ -763,7 +572,7 @@
         """
         repo = self.make_repository('.')
         # This should work, not raise NotImplementedError:
-        if not repo.revision_graph_can_have_wrong_parents():
+        if not repo._format.revision_graph_can_have_wrong_parents:
             return
         repo.lock_read()
         self.addCleanup(repo.unlock)
@@ -775,88 +584,44 @@
         repo._check_for_inconsistent_revision_parents()
 
     def test_add_signature_text(self):
-        repo = self.make_repository('repo')
-        repo.lock_write()
-        self.addCleanup(repo.unlock)
-        repo.start_write_group()
-        self.addCleanup(repo.abort_write_group)
-        inv = inventory.Inventory(revision_id='A')
-        inv.root.revision = 'A'
-        repo.add_inventory('A', inv, [])
-        repo.add_revision('A', _mod_revision.Revision(
-                'A', committer='A', timestamp=0,
-                inventory_sha1='', timezone=0, message='A'))
-        repo.add_signature_text('A', 'This might be a signature')
-        self.assertEqual('This might be a signature',
-                         repo.get_signature_text('A'))
-
-    def test_add_revision_inventory_sha1(self):
-        inv = inventory.Inventory(revision_id='A')
-        inv.root.revision = 'A'
-        inv.root.file_id = 'fixed-root'
-        # Insert the inventory on its own to an identical repository, to get
-        # its sha1.
-        reference_repo = self.make_repository('reference_repo')
-        reference_repo.lock_write()
-        reference_repo.start_write_group()
-        inv_sha1 = reference_repo.add_inventory('A', inv, [])
-        reference_repo.abort_write_group()
-        reference_repo.unlock()
-        # Now insert a revision with this inventory, and it should get the same
-        # sha1.
-        repo = self.make_repository('repo')
-        repo.lock_write()
-        repo.start_write_group()
-        root_id = inv.root.file_id
-        repo.texts.add_lines(('fixed-root', 'A'), [], [])
-        repo.add_revision('A', _mod_revision.Revision(
-                'A', committer='B', timestamp=0,
-                timezone=0, message='C'), inv=inv)
-        repo.commit_write_group()
-        repo.unlock()
-        repo.lock_read()
-        self.assertEquals(inv_sha1, repo.get_revision('A').inventory_sha1)
-        repo.unlock()
-
-    def test_install_revisions(self):
-        wt = self.make_branch_and_tree('source')
-        wt.commit('A', allow_pointless=True, rev_id='A')
-        repo = wt.branch.repository
-        repo.lock_write()
-        repo.start_write_group()
-        repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
-        repo.commit_write_group()
-        repo.unlock()
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        repo2 = self.make_repository('repo2')
-        revision = repo.get_revision('A')
-        tree = repo.revision_tree('A')
-        signature = repo.get_signature_text('A')
-        repo2.lock_write()
-        self.addCleanup(repo2.unlock)
-        repository.install_revisions(repo2, [(revision, tree, signature)])
-        self.assertEqual(revision, repo2.get_revision('A'))
-        self.assertEqual(signature, repo2.get_signature_text('A'))
+        builder = self.make_branch_builder('.')
+        builder.start_series()
+        builder.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None))])
+        builder.finish_series()
+        b = builder.get_branch()
+        b.lock_write()
+        self.addCleanup(b.unlock)
+        b.repository.start_write_group()
+        self.addCleanup(b.repository.abort_write_group)
+        if b.repository._format.supports_revision_signatures:
+            b.repository.add_signature_text('A', 'This might be a signature')
+            self.assertEqual('This might be a signature',
+                             b.repository.get_signature_text('A'))
+        else:
+            self.assertRaises(errors.UnsupportedOperation,
+                b.repository.add_signature_text, 'A',
+                'This might be a signature')
 
     # XXX: this helper duplicated from tests.test_repository
-    def make_remote_repository(self, path, shared=False):
+    def make_remote_repository(self, path, shared=None):
         """Make a RemoteRepository object backed by a real repository that will
         be created at the given path."""
         repo = self.make_repository(path, shared=shared)
         smart_server = test_server.SmartTCPServer_for_testing()
         self.start_server(smart_server, self.get_server())
-        remote_transport = transport.get_transport(
+        remote_transport = transport.get_transport_from_url(
             smart_server.get_url()).clone(path)
-        remote_bzrdir = bzrdir.BzrDir.open_from_transport(remote_transport)
+        if not repo.bzrdir._format.supports_transport(remote_transport):
+            raise tests.TestNotApplicable(
+                "format does not support transport")
+        remote_bzrdir = controldir.ControlDir.open_from_transport(
+            remote_transport)
         remote_repo = remote_bzrdir.open_repository()
         return remote_repo
 
     def test_sprout_from_hpss_preserves_format(self):
         """repo.sprout from a smart server preserves the repository format."""
-        if self.repository_format == weaverepo.RepositoryFormat7():
-            raise tests.TestNotApplicable(
-                "Cannot fetch weaves over smart protocol.")
         remote_repo = self.make_remote_repository('remote')
         local_bzrdir = self.make_bzrdir('local')
         try:
@@ -864,19 +629,16 @@
         except errors.TransportNotPossible:
             raise tests.TestNotApplicable(
                 "Cannot lock_read old formats like AllInOne over HPSS.")
-        remote_backing_repo = bzrdir.BzrDir.open(
+        remote_backing_repo = controldir.ControlDir.open(
             self.get_vfs_only_url('remote')).open_repository()
         self.assertEqual(remote_backing_repo._format, local_repo._format)
 
     def test_sprout_branch_from_hpss_preserves_repo_format(self):
         """branch.sprout from a smart server preserves the repository format.
         """
-        weave_formats = [weaverepo.RepositoryFormat5(),
-                         weaverepo.RepositoryFormat6(),
-                         weaverepo.RepositoryFormat7()]
-        if self.repository_format in weave_formats:
+        if not self.repository_format.supports_leaving_lock:
             raise tests.TestNotApplicable(
-                "Cannot fetch weaves over smart protocol.")
+                "Format can not be used over HPSS")
         remote_repo = self.make_remote_repository('remote')
         remote_branch = remote_repo.bzrdir.create_branch()
         try:
@@ -885,7 +647,7 @@
             raise tests.TestNotApplicable(
                 "Cannot lock_read old formats like AllInOne over HPSS.")
         local_repo = local_bzrdir.open_repository()
-        remote_backing_repo = bzrdir.BzrDir.open(
+        remote_backing_repo = controldir.ControlDir.open(
             self.get_vfs_only_url('remote')).open_repository()
         self.assertEqual(remote_backing_repo._format, local_repo._format)
 
@@ -893,15 +655,12 @@
         """branch.sprout from a smart server preserves the repository format of
         a branch from a shared repository.
         """
-        weave_formats = [weaverepo.RepositoryFormat5(),
-                         weaverepo.RepositoryFormat6(),
-                         weaverepo.RepositoryFormat7()]
-        if self.repository_format in weave_formats:
+        if not self.repository_format.supports_leaving_lock:
             raise tests.TestNotApplicable(
-                "Cannot fetch weaves over smart protocol.")
+                "Format can not be used over HPSS")
         # Make a shared repo
         remote_repo = self.make_remote_repository('remote', shared=True)
-        remote_backing_repo = bzrdir.BzrDir.open(
+        remote_backing_repo = controldir.ControlDir.open(
             self.get_vfs_only_url('remote')).open_repository()
         # Make a branch in that repo in an old format that isn't the default
         # branch format for the repo.
@@ -909,9 +668,9 @@
         format = remote_backing_repo.bzrdir.cloning_metadir()
         format._branch_format = BzrBranchFormat5()
         remote_transport = remote_repo.bzrdir.root_transport.clone('branch')
-        remote_backing_repo.bzrdir.create_branch_convenience(
+        controldir.ControlDir.create_branch_convenience(
             remote_transport.base, force_new_repo=False, format=format)
-        remote_branch = bzrdir.BzrDir.open_from_transport(
+        remote_branch = controldir.ControlDir.open_from_transport(
             remote_transport).open_branch()
         try:
             local_bzrdir = remote_branch.bzrdir.sprout('local')
@@ -922,9 +681,7 @@
         self.assertEqual(remote_backing_repo._format, local_repo._format)
 
     def test_clone_to_hpss(self):
-        pre_metadir_formats = [weaverepo.RepositoryFormat5(),
-                               weaverepo.RepositoryFormat6()]
-        if self.repository_format in pre_metadir_formats:
+        if not self.repository_format.supports_leaving_lock:
             raise tests.TestNotApplicable(
                 "Cannot lock pre_metadir_formats remotely.")
         remote_transport = self.make_smart_server('remote')
@@ -943,12 +700,12 @@
             repo = self.make_repository('repo', shared=True)
         except errors.IncompatibleFormat:
             raise tests.TestNotApplicable('Cannot make a shared repository')
-        if isinstance(repo.bzrdir, bzrdir.BzrDirPreSplitOut):
-            raise tests.KnownFailure(
+        if repo.bzrdir._format.fixed_components:
+            self.knownFailure(
                 "pre metadir branches do not upgrade on push "
                 "with stacking policy")
         if isinstance(repo._format,
-                      pack_repo.RepositoryFormatKnitPack5RichRootBroken):
+                      knitpack_repo.RepositoryFormatKnitPack5RichRootBroken):
             raise tests.TestNotApplicable("unsupported format")
         # Make a source branch in 'repo' in an unstackable branch format
         bzrdir_format = self.repository_format._matchingbzrdir
@@ -993,11 +750,6 @@
         else:
             self.assertEqual(stack_on.repository._format, target_repo._format)
 
-    def test__get_sink(self):
-        repo = self.make_repository('repo')
-        sink = repo._get_sink()
-        self.assertIsInstance(sink, repository.StreamSink)
-
     def test__make_parents_provider(self):
         """Repositories must have a _make_parents_provider method that returns
         an object with a get_parent_map method.
@@ -1005,20 +757,23 @@
         repo = self.make_repository('repo')
         repo._make_parents_provider().get_parent_map
 
-    def make_repository_and_foo_bar(self, shared):
+    def make_repository_and_foo_bar(self, shared=None):
         made_control = self.make_bzrdir('repository')
         repo = made_control.create_repository(shared=shared)
-        bzrdir.BzrDir.create_branch_convenience(self.get_url('repository/foo'),
-                                                force_new_repo=False)
-        bzrdir.BzrDir.create_branch_convenience(self.get_url('repository/bar'),
-                                                force_new_repo=True)
+        if not repo._format.supports_nesting_repositories:
+            raise tests.TestNotApplicable("repository does not support "
+                "nesting repositories")
+        controldir.ControlDir.create_branch_convenience(
+            self.get_url('repository/foo'), force_new_repo=False)
+        controldir.ControlDir.create_branch_convenience(
+            self.get_url('repository/bar'), force_new_repo=True)
         baz = self.make_bzrdir('repository/baz')
         qux = self.make_branch('repository/baz/qux')
         quxx = self.make_branch('repository/baz/qux/quxx')
         return repo
 
     def test_find_branches(self):
-        repo = self.make_repository_and_foo_bar(shared=False)
+        repo = self.make_repository_and_foo_bar()
         branches = repo.find_branches()
         self.assertContainsRe(branches[-1].base, 'repository/foo/$')
         self.assertContainsRe(branches[-3].base, 'repository/baz/qux/$')
@@ -1047,6 +802,9 @@
 
     def test_find_branches_using_standalone(self):
         branch = self.make_branch('branch')
+        if not branch.repository._format.supports_nesting_repositories:
+            raise tests.TestNotApplicable("format does not support nesting "
+                "repositories")
         contained = self.make_branch('branch/contained')
         branches = branch.repository.find_branches(using=True)
         self.assertEqual([branch.base], [b.base for b in branches])
@@ -1055,8 +813,11 @@
                          [b.base for b in branches])
 
     def test_find_branches_using_empty_standalone_repo(self):
-        repo = self.make_repository('repo')
-        self.assertFalse(repo.is_shared())
+        try:
+            repo = self.make_repository('repo', shared=False)
+        except errors.IncompatibleFormat:
+            raise tests.TestNotApplicable("format does not support standalone "
+                "repositories")
         try:
             repo.bzrdir.open_branch()
         except errors.NotBranchError:
@@ -1069,7 +830,7 @@
         repo = self.make_repository('repo')
         try:
             repo.set_make_working_trees(True)
-        except errors.RepositoryUpgradeRequired, e:
+        except (errors.RepositoryUpgradeRequired, errors.UnsupportedOperation), e:
             raise tests.TestNotApplicable('Format does not support this flag.')
         self.assertTrue(repo.make_working_trees())
 
@@ -1077,7 +838,7 @@
         repo = self.make_repository('repo')
         try:
             repo.set_make_working_trees(False)
-        except errors.RepositoryUpgradeRequired, e:
+        except (errors.RepositoryUpgradeRequired, errors.UnsupportedOperation), e:
             raise tests.TestNotApplicable('Format does not support this flag.')
         self.assertFalse(repo.make_working_trees())
 
@@ -1151,161 +912,6 @@
         self.assertThat(repo.lock_write, ReturnsUnlockable(repo))
 
 
-class TestCaseWithComplexRepository(per_repository.TestCaseWithRepository):
-
-    def setUp(self):
-        super(TestCaseWithComplexRepository, self).setUp()
-        tree_a = self.make_branch_and_tree('a')
-        self.bzrdir = tree_a.branch.bzrdir
-        # add a corrupt inventory 'orphan'
-        # this may need some generalising for knits.
-        tree_a.lock_write()
-        try:
-            tree_a.branch.repository.start_write_group()
-            try:
-                inv_file = tree_a.branch.repository.inventories
-                inv_file.add_lines(('orphan',), [], [])
-            except:
-                tree_a.branch.repository.commit_write_group()
-                raise
-            else:
-                tree_a.branch.repository.abort_write_group()
-        finally:
-            tree_a.unlock()
-        # add a real revision 'rev1'
-        tree_a.commit('rev1', rev_id='rev1', allow_pointless=True)
-        # add a real revision 'rev2' based on rev1
-        tree_a.commit('rev2', rev_id='rev2', allow_pointless=True)
-        # add a reference to a ghost
-        tree_a.add_parent_tree_id('ghost1')
-        try:
-            tree_a.commit('rev3', rev_id='rev3', allow_pointless=True)
-        except errors.RevisionNotPresent:
-            raise tests.TestNotApplicable(
-                "Cannot test with ghosts for this format.")
-        # add another reference to a ghost, and a second ghost.
-        tree_a.add_parent_tree_id('ghost1')
-        tree_a.add_parent_tree_id('ghost2')
-        tree_a.commit('rev4', rev_id='rev4', allow_pointless=True)
-
-    def test_revision_trees(self):
-        revision_ids = ['rev1', 'rev2', 'rev3', 'rev4']
-        repository = self.bzrdir.open_repository()
-        repository.lock_read()
-        self.addCleanup(repository.unlock)
-        trees1 = list(repository.revision_trees(revision_ids))
-        trees2 = [repository.revision_tree(t) for t in revision_ids]
-        self.assertEqual(len(trees1), len(trees2))
-        for tree1, tree2 in zip(trees1, trees2):
-            self.assertFalse(tree2.changes_from(tree1).has_changed())
-
-    def test_get_deltas_for_revisions(self):
-        repository = self.bzrdir.open_repository()
-        repository.lock_read()
-        self.addCleanup(repository.unlock)
-        revisions = [repository.get_revision(r) for r in
-                     ['rev1', 'rev2', 'rev3', 'rev4']]
-        deltas1 = list(repository.get_deltas_for_revisions(revisions))
-        deltas2 = [repository.get_revision_delta(r.revision_id) for r in
-                   revisions]
-        self.assertEqual(deltas1, deltas2)
-
-    def test_all_revision_ids(self):
-        # all_revision_ids -> all revisions
-        self.assertEqual(set(['rev1', 'rev2', 'rev3', 'rev4']),
-            set(self.bzrdir.open_repository().all_revision_ids()))
-
-    def test_get_ancestry_missing_revision(self):
-        # get_ancestry(revision that is in some data but not fully installed
-        # -> NoSuchRevision
-        self.assertRaises(errors.NoSuchRevision,
-                          self.bzrdir.open_repository().get_ancestry, 'orphan')
-
-    def test_get_unordered_ancestry(self):
-        repo = self.bzrdir.open_repository()
-        self.assertEqual(set(repo.get_ancestry('rev3')),
-                         set(repo.get_ancestry('rev3', topo_sorted=False)))
-
-    def test_reserved_id(self):
-        repo = self.make_repository('repository')
-        repo.lock_write()
-        repo.start_write_group()
-        try:
-            self.assertRaises(errors.ReservedId, repo.add_inventory, 'reserved:',
-                              None, None)
-            self.assertRaises(errors.ReservedId, repo.add_inventory_by_delta,
-                "foo", [], 'reserved:', None)
-            self.assertRaises(errors.ReservedId, repo.add_revision, 'reserved:',
-                              None)
-        finally:
-            repo.abort_write_group()
-            repo.unlock()
-
-
-class TestCaseWithCorruptRepository(per_repository.TestCaseWithRepository):
-
-    def setUp(self):
-        super(TestCaseWithCorruptRepository, self).setUp()
-        # a inventory with no parents and the revision has parents..
-        # i.e. a ghost.
-        repo = self.make_repository('inventory_with_unnecessary_ghost')
-        repo.lock_write()
-        repo.start_write_group()
-        inv = inventory.Inventory(revision_id = 'ghost')
-        inv.root.revision = 'ghost'
-        if repo.supports_rich_root():
-            root_id = inv.root.file_id
-            repo.texts.add_lines((root_id, 'ghost'), [], [])
-        sha1 = repo.add_inventory('ghost', inv, [])
-        rev = _mod_revision.Revision(
-            timestamp=0, timezone=None, committer="Foo Bar <foo@example.com>",
-            message="Message", inventory_sha1=sha1, revision_id='ghost')
-        rev.parent_ids = ['the_ghost']
-        try:
-            repo.add_revision('ghost', rev)
-        except (errors.NoSuchRevision, errors.RevisionNotPresent):
-            raise tests.TestNotApplicable(
-                "Cannot test with ghosts for this format.")
-
-        inv = inventory.Inventory(revision_id = 'the_ghost')
-        inv.root.revision = 'the_ghost'
-        if repo.supports_rich_root():
-            root_id = inv.root.file_id
-            repo.texts.add_lines((root_id, 'the_ghost'), [], [])
-        sha1 = repo.add_inventory('the_ghost', inv, [])
-        rev = _mod_revision.Revision(
-            timestamp=0, timezone=None, committer="Foo Bar <foo@example.com>",
-            message="Message", inventory_sha1=sha1, revision_id='the_ghost')
-        rev.parent_ids = []
-        repo.add_revision('the_ghost', rev)
-        # check its setup usefully
-        inv_weave = repo.inventories
-        possible_parents = (None, (('ghost',),))
-        self.assertSubset(inv_weave.get_parent_map([('ghost',)])[('ghost',)],
-            possible_parents)
-        repo.commit_write_group()
-        repo.unlock()
-
-    def test_corrupt_revision_access_asserts_if_reported_wrong(self):
-        repo_url = self.get_url('inventory_with_unnecessary_ghost')
-        repo = repository.Repository.open(repo_url)
-        reported_wrong = False
-        try:
-            if repo.get_ancestry('ghost') != [None, 'the_ghost', 'ghost']:
-                reported_wrong = True
-        except errors.CorruptRepository:
-            # caught the bad data:
-            return
-        if not reported_wrong:
-            return
-        self.assertRaises(errors.CorruptRepository, repo.get_revision, 'ghost')
-
-    def test_corrupt_revision_get_revision_reconcile(self):
-        repo_url = self.get_url('inventory_with_unnecessary_ghost')
-        repo = repository.Repository.open(repo_url)
-        repo.get_revision_reconcile('ghost')
-
-
 # FIXME: document why this is a TestCaseWithTransport rather than a
 #        TestCaseWithRepository
 class TestEscaping(tests.TestCaseWithTransport):
@@ -1336,7 +942,7 @@
         wt.add(['foo'], [FOO_ID])
         wt.commit('this is my new commit', rev_id=REV_ID)
         # now access over vfat; should be safe
-        branch = bzrdir.BzrDir.open(self.get_url('repo')).open_branch()
+        branch = controldir.ControlDir.open(self.get_url('repo')).open_branch()
         revtree = branch.repository.revision_tree(REV_ID)
         revtree.lock_read()
         self.addCleanup(revtree.unlock)
@@ -1353,11 +959,9 @@
             'rev1', _mod_revision.NULL_REVISION, fileobj)
 
 
-
-
 class TestRepositoryControlComponent(per_repository.TestCaseWithRepository):
     """Repository implementations adequately implement ControlComponent."""
-    
+
     def test_urls(self):
         repo = self.make_repository('repo')
         self.assertIsInstance(repo.user_url, str)

=== modified file 'bzrlib/tests/per_repository/test_revision.py'
--- bzrlib/tests/per_repository/test_revision.py	2009-06-10 03:56:49 +0000
+++ bzrlib/tests/per_repository/test_revision.py	2011-09-26 15:21:01 +0000
@@ -69,13 +69,14 @@
         tree1 = self.make_branch_and_tree("br1")
 
         # create a revision
-        tree1.commit(message="quux", allow_pointless=True, committer="jaq",
-                     revprops={'empty':'',
-                               'value':'one',
-                               'unicode':u'\xb5',
-                               'multiline':'foo\nbar\n\n'
-                              })
-        self.assertTrue(len(tree1.branch.revision_history()) > 0)
+        rev1 = tree1.commit(message="quux", allow_pointless=True,
+            committer="jaq",
+            revprops={'empty':'',
+                      'value':'one',
+                      'unicode':u'\xb5',
+                      'multiline':'foo\nbar\n\n'
+                      })
+        self.assertEquals(tree1.branch.last_revision(), rev1)
         rev_a = tree1.branch.repository.get_revision(
                             tree1.branch.last_revision())
 

=== added file 'bzrlib/tests/per_repository/test_signatures.py'
--- bzrlib/tests/per_repository/test_signatures.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository/test_signatures.py	2011-10-10 14:03:32 +0000
@@ -0,0 +1,95 @@
+# Copyright (C) 2005-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for repository revision signatures."""
+
+from bzrlib import (
+    errors,
+    gpg,
+    tests,
+    urlutils,
+    )
+
+from bzrlib.testament import Testament
+from bzrlib.tests import per_repository
+
+class TestSignatures(per_repository.TestCaseWithRepository):
+
+# TODO 20051003 RBC:
+# compare the gpg-to-sign info for a commit with a ghost and
+#     an identical tree without a ghost
+# fetch missing should rewrite the TOC of weaves to list newly available parents.
+
+    def test_sign_existing_revision(self):
+        wt = self.make_branch_and_tree('.')
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        strategy = gpg.LoopbackGPGStrategy(None)
+        repo = wt.branch.repository
+        self.addCleanup(repo.lock_write().unlock)
+        repo.start_write_group()
+        repo.sign_revision('A', strategy)
+        repo.commit_write_group()
+        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
+                         Testament.from_revision(repo,
+                         'A').as_short_text() +
+                         '-----END PSEUDO-SIGNED CONTENT-----\n',
+                         repo.get_signature_text('A'))
+
+    def test_store_signature(self):
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        branch.lock_write()
+        try:
+            branch.repository.start_write_group()
+            try:
+                branch.repository.store_revision_signature(
+                    gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
+            except errors.NoSuchRevision:
+                branch.repository.abort_write_group()
+                raise tests.TestNotApplicable(
+                    "repository does not support signing non-present"
+                    "revisions")
+            except:
+                branch.repository.abort_write_group()
+                raise
+            else:
+                branch.repository.commit_write_group()
+        finally:
+            branch.unlock()
+        # A signature without a revision should not be accessible.
+        self.assertRaises(errors.NoSuchRevision,
+                          branch.repository.has_signature_for_revision_id,
+                          'A')
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n'
+                         'FOO-----END PSEUDO-SIGNED CONTENT-----\n',
+                         branch.repository.get_signature_text('A'))
+
+    def test_clone_preserves_signatures(self):
+        wt = self.make_branch_and_tree('source')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        repo = wt.branch.repository
+        repo.lock_write()
+        repo.start_write_group()
+        repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
+        repo.commit_write_group()
+        repo.unlock()
+        #FIXME: clone should work to urls,
+        # wt.clone should work to disks.
+        self.build_tree(['target/'])
+        d2 = repo.bzrdir.clone(urlutils.local_path_to_url('target'))
+        self.assertEqual(repo.get_signature_text('A'),
+                         d2.open_repository().get_signature_text('A'))

=== modified file 'bzrlib/tests/per_repository/test_statistics.py'
--- bzrlib/tests/per_repository/test_statistics.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository/test_statistics.py	2011-09-09 12:19:35 +0000
@@ -38,27 +38,24 @@
         # now, in the same repository, asking for stats with/without the
         # committers flag generates the same date information.
         stats = tree.branch.repository.gather_stats(rev2, committers=False)
-        self.assertEqual({
-            'firstrev': (1170491381.0, 0),
-            'latestrev': (1171491381.0, 0),
-            'revisions': 3,
-            },
-            stats)
+        # this test explicitly only checks for certain keys
+        # in the dictionary, as implementations are allowed to
+        # provide arbitrary data in other keys.
+        self.assertEqual(stats['firstrev'], (1170491381.0, 0))
+        self.assertEqual(stats['latestrev'], (1171491381.0, 0))
+        self.assertEqual(stats['revisions'], 3)
         stats = tree.branch.repository.gather_stats(rev2, committers=True)
-        self.assertEqual({
-            'committers': 2,
-            'firstrev': (1170491381.0, 0),
-            'latestrev': (1171491381.0, 0),
-            'revisions': 3,
-            },
-            stats)
+        self.assertEquals(2, stats["committers"])
+        self.assertEquals((1170491381.0, 0), stats["firstrev"])
+        self.assertEquals((1171491381.0, 0), stats["latestrev"])
+        self.assertEquals(3, stats["revisions"])
 
     def test_gather_stats_empty_repo(self):
         """An empty repository still has revisions."""
         tree = self.make_branch_and_memory_tree('.')
         # now ask for global repository stats.
         stats = tree.branch.repository.gather_stats()
-        self.assertEqual({
-            'revisions': 0
-            },
-            stats)
+        self.assertEquals(0, stats['revisions'])
+        self.assertFalse("committers" in stats)
+        self.assertFalse("firstrev" in stats)
+        self.assertFalse("latestrev" in stats)

=== modified file 'bzrlib/tests/per_repository/test_write_group.py'
--- bzrlib/tests/per_repository/test_write_group.py	2010-02-11 09:27:55 +0000
+++ bzrlib/tests/per_repository/test_write_group.py	2011-05-09 20:08:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,18 +16,8 @@
 
 """Tests for repository write groups."""
 
-import sys
-
 from bzrlib import (
-    branch,
-    bzrdir,
     errors,
-    graph,
-    memorytree,
-    osutils,
-    remote,
-    tests,
-    versionedfile,
     )
 from bzrlib.tests import (
     per_repository,
@@ -132,591 +122,3 @@
         # exception was not generated, or the exception was caught and
         # suppressed.  See also test_pack_repository's test of the same name.
         self.assertEqual(None, repo.abort_write_group(suppress_errors=True))
-
-
-class TestGetMissingParentInventories(per_repository.TestCaseWithRepository):
-
-    def test_empty_get_missing_parent_inventories(self):
-        """A new write group has no missing parent inventories."""
-        repo = self.make_repository('.')
-        repo.lock_write()
-        repo.start_write_group()
-        try:
-            self.assertEqual(set(), set(repo.get_missing_parent_inventories()))
-        finally:
-            repo.commit_write_group()
-            repo.unlock()
-
-    def branch_trunk_and_make_tree(self, trunk_repo, relpath):
-        tree = self.make_branch_and_memory_tree('branch')
-        trunk_repo.lock_read()
-        self.addCleanup(trunk_repo.unlock)
-        tree.branch.repository.fetch(trunk_repo, revision_id='rev-1')
-        tree.set_parent_ids(['rev-1'])
-        return tree 
-
-    def make_first_commit(self, repo):
-        trunk = repo.bzrdir.create_branch()
-        tree = memorytree.MemoryTree.create_on_branch(trunk)
-        tree.lock_write()
-        tree.add([''], ['TREE_ROOT'], ['directory'])
-        tree.add(['dir'], ['dir-id'], ['directory'])
-        tree.add(['filename'], ['file-id'], ['file'])
-        tree.put_file_bytes_non_atomic('file-id', 'content\n')
-        tree.commit('Trunk commit', rev_id='rev-0')
-        tree.commit('Trunk commit', rev_id='rev-1')
-        tree.unlock()
-
-    def make_new_commit_in_new_repo(self, trunk_repo, parents=None):
-        tree = self.branch_trunk_and_make_tree(trunk_repo, 'branch')
-        tree.set_parent_ids(parents)
-        tree.commit('Branch commit', rev_id='rev-2')
-        branch_repo = tree.branch.repository
-        branch_repo.lock_read()
-        self.addCleanup(branch_repo.unlock)
-        return branch_repo
-
-    def make_stackable_repo(self, relpath='trunk'):
-        if isinstance(self.repository_format, remote.RemoteRepositoryFormat):
-            # RemoteRepository by default builds a default format real
-            # repository, but the default format is unstackble.  So explicitly
-            # make a stackable real repository and use that.
-            repo = self.make_repository(relpath, format='1.9')
-            repo = bzrdir.BzrDir.open(self.get_url(relpath)).open_repository()
-        else:
-            repo = self.make_repository(relpath)
-        if not repo._format.supports_external_lookups:
-            raise tests.TestNotApplicable('format not stackable')
-        repo.bzrdir._format.set_branch_format(branch.BzrBranchFormat7())
-        return repo
-
-    def reopen_repo_and_resume_write_group(self, repo):
-        try:
-            resume_tokens = repo.suspend_write_group()
-        except errors.UnsuspendableWriteGroup:
-            # If we got this far, and this repo does not support resuming write
-            # groups, then get_missing_parent_inventories works in all
-            # cases this repo supports.
-            repo.unlock()
-            return
-        repo.unlock()
-        reopened_repo = repo.bzrdir.open_repository()
-        reopened_repo.lock_write()
-        self.addCleanup(reopened_repo.unlock)
-        reopened_repo.resume_write_group(resume_tokens)
-        return reopened_repo
-
-    def test_ghost_revision(self):
-        """A parent inventory may be absent if all the needed texts are present.
-        i.e., a ghost revision isn't (necessarily) considered to be a missing
-        parent inventory.
-        """
-        # Make a trunk with one commit.
-        trunk_repo = self.make_stackable_repo()
-        self.make_first_commit(trunk_repo)
-        trunk_repo.lock_read()
-        self.addCleanup(trunk_repo.unlock)
-        # Branch the trunk, add a new commit.
-        branch_repo = self.make_new_commit_in_new_repo(
-            trunk_repo, parents=['rev-1', 'ghost-rev'])
-        inv = branch_repo.get_inventory('rev-2')
-        # Make a new repo stacked on trunk, and then copy into it:
-        #  - all texts in rev-2
-        #  - the new inventory (rev-2)
-        #  - the new revision (rev-2)
-        repo = self.make_stackable_repo('stacked')
-        repo.lock_write()
-        repo.start_write_group()
-        # Add all texts from in rev-2 inventory.  Note that this has to exclude
-        # the root if the repo format does not support rich roots.
-        rich_root = branch_repo._format.rich_root_data
-        all_texts = [
-            (ie.file_id, ie.revision) for ie in inv.iter_just_entries()
-             if rich_root or inv.id2path(ie.file_id) != '']
-        repo.texts.insert_record_stream(
-            branch_repo.texts.get_record_stream(all_texts, 'unordered', False))
-        # Add inventory and revision for rev-2.
-        repo.add_inventory('rev-2', inv, ['rev-1', 'ghost-rev'])
-        repo.revisions.insert_record_stream(
-            branch_repo.revisions.get_record_stream(
-                [('rev-2',)], 'unordered', False))
-        # Now, no inventories are reported as missing, even though there is a
-        # ghost.
-        self.assertEqual(set(), repo.get_missing_parent_inventories())
-        # Resuming the write group does not affect
-        # get_missing_parent_inventories.
-        reopened_repo = self.reopen_repo_and_resume_write_group(repo)
-        self.assertEqual(set(), reopened_repo.get_missing_parent_inventories())
-        reopened_repo.abort_write_group()
-
-    def test_get_missing_parent_inventories(self):
-        """A stacked repo with a single revision and inventory (no parent
-        inventory) in it must have all the texts in its inventory (even if not
-        changed w.r.t. to the absent parent), otherwise it will report missing
-        texts/parent inventory.
-
-        The core of this test is that a file was changed in rev-1, but in a
-        stacked repo that only has rev-2
-        """
-        # Make a trunk with one commit.
-        trunk_repo = self.make_stackable_repo()
-        self.make_first_commit(trunk_repo)
-        trunk_repo.lock_read()
-        self.addCleanup(trunk_repo.unlock)
-        # Branch the trunk, add a new commit.
-        branch_repo = self.make_new_commit_in_new_repo(
-            trunk_repo, parents=['rev-1'])
-        inv = branch_repo.get_inventory('rev-2')
-        # Make a new repo stacked on trunk, and copy the new commit's revision
-        # and inventory records to it.
-        repo = self.make_stackable_repo('stacked')
-        repo.lock_write()
-        repo.start_write_group()
-        # Insert a single fulltext inv (using add_inventory because it's
-        # simpler than insert_record_stream)
-        repo.add_inventory('rev-2', inv, ['rev-1'])
-        repo.revisions.insert_record_stream(
-            branch_repo.revisions.get_record_stream(
-                [('rev-2',)], 'unordered', False))
-        # There should be no missing compression parents
-        self.assertEqual(set(),
-                repo.inventories.get_missing_compression_parent_keys())
-        self.assertEqual(
-            set([('inventories', 'rev-1')]),
-            repo.get_missing_parent_inventories())
-        # Resuming the write group does not affect
-        # get_missing_parent_inventories.
-        reopened_repo = self.reopen_repo_and_resume_write_group(repo)
-        self.assertEqual(
-            set([('inventories', 'rev-1')]),
-            reopened_repo.get_missing_parent_inventories())
-        # Adding the parent inventory satisfies get_missing_parent_inventories.
-        reopened_repo.inventories.insert_record_stream(
-            branch_repo.inventories.get_record_stream(
-                [('rev-1',)], 'unordered', False))
-        self.assertEqual(
-            set(), reopened_repo.get_missing_parent_inventories())
-        reopened_repo.abort_write_group()
-
-    def test_get_missing_parent_inventories_check(self):
-        builder = self.make_branch_builder('test')
-        builder.build_snapshot('A-id', ['ghost-parent-id'], [
-            ('add', ('', 'root-id', 'directory', None)),
-            ('add', ('file', 'file-id', 'file', 'content\n'))],
-            allow_leftmost_as_ghost=True)
-        b = builder.get_branch()
-        b.lock_read()
-        self.addCleanup(b.unlock)
-        repo = self.make_repository('test-repo')
-        repo.lock_write()
-        self.addCleanup(repo.unlock)
-        repo.start_write_group()
-        self.addCleanup(repo.abort_write_group)
-        # Now, add the objects manually
-        text_keys = [('file-id', 'A-id')]
-        if repo.supports_rich_root():
-            text_keys.append(('root-id', 'A-id'))
-        # Directly add the texts, inventory, and revision object for 'A-id'
-        repo.texts.insert_record_stream(b.repository.texts.get_record_stream(
-            text_keys, 'unordered', True))
-        repo.add_revision('A-id', b.repository.get_revision('A-id'),
-                          b.repository.get_inventory('A-id'))
-        get_missing = repo.get_missing_parent_inventories
-        if repo._format.supports_external_lookups:
-            self.assertEqual(set([('inventories', 'ghost-parent-id')]),
-                get_missing(check_for_missing_texts=False))
-            self.assertEqual(set(), get_missing(check_for_missing_texts=True))
-            self.assertEqual(set(), get_missing())
-        else:
-            # If we don't support external lookups, we always return empty
-            self.assertEqual(set(), get_missing(check_for_missing_texts=False))
-            self.assertEqual(set(), get_missing(check_for_missing_texts=True))
-            self.assertEqual(set(), get_missing())
-
-    def test_insert_stream_passes_resume_info(self):
-        repo = self.make_repository('test-repo')
-        if (not repo._format.supports_external_lookups or
-            isinstance(repo, remote.RemoteRepository)):
-            raise tests.TestNotApplicable(
-                'only valid for direct connections to resumable repos')
-        # log calls to get_missing_parent_inventories, so that we can assert it
-        # is called with the correct parameters
-        call_log = []
-        orig = repo.get_missing_parent_inventories
-        def get_missing(check_for_missing_texts=True):
-            call_log.append(check_for_missing_texts)
-            return orig(check_for_missing_texts=check_for_missing_texts)
-        repo.get_missing_parent_inventories = get_missing
-        repo.lock_write()
-        self.addCleanup(repo.unlock)
-        sink = repo._get_sink()
-        sink.insert_stream((), repo._format, [])
-        self.assertEqual([False], call_log)
-        del call_log[:]
-        repo.start_write_group()
-        # We need to insert something, or suspend_write_group won't actually
-        # create a token
-        repo.texts.insert_record_stream([versionedfile.FulltextContentFactory(
-            ('file-id', 'rev-id'), (), None, 'lines\n')])
-        tokens = repo.suspend_write_group()
-        self.assertNotEqual([], tokens)
-        sink.insert_stream((), repo._format, tokens)
-        self.assertEqual([True], call_log)
-
-
-class TestResumeableWriteGroup(per_repository.TestCaseWithRepository):
-
-    def make_write_locked_repo(self, relpath='repo'):
-        repo = self.make_repository(relpath)
-        repo.lock_write()
-        self.addCleanup(repo.unlock)
-        return repo
-
-    def reopen_repo(self, repo):
-        same_repo = repo.bzrdir.open_repository()
-        same_repo.lock_write()
-        self.addCleanup(same_repo.unlock)
-        return same_repo
-
-    def require_suspendable_write_groups(self, reason):
-        repo = self.make_repository('__suspend_test')
-        repo.lock_write()
-        self.addCleanup(repo.unlock)
-        repo.start_write_group()
-        try:
-            wg_tokens = repo.suspend_write_group()
-        except errors.UnsuspendableWriteGroup:
-            repo.abort_write_group()
-            raise tests.TestNotApplicable(reason)
-
-    def test_suspend_write_group(self):
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        repo.texts.add_lines(('file-id', 'revid'), (), ['lines'])
-        try:
-            wg_tokens = repo.suspend_write_group()
-        except errors.UnsuspendableWriteGroup:
-            # The contract for repos that don't support suspending write groups
-            # is that suspend_write_group raises UnsuspendableWriteGroup, but
-            # is otherwise a no-op.  So we can still e.g. abort the write group
-            # as usual.
-            self.assertTrue(repo.is_in_write_group())
-            repo.abort_write_group()
-        else:
-            # After suspending a write group we are no longer in a write group
-            self.assertFalse(repo.is_in_write_group())
-            # suspend_write_group returns a list of tokens, which are strs.  If
-            # no other write groups were resumed, there will only be one token.
-            self.assertEqual(1, len(wg_tokens))
-            self.assertIsInstance(wg_tokens[0], str)
-            # See also test_pack_repository's test of the same name.
-
-    def test_resume_write_group_then_abort(self):
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        try:
-            wg_tokens = repo.suspend_write_group()
-        except errors.UnsuspendableWriteGroup:
-            # If the repo does not support suspending write groups, it doesn't
-            # support resuming them either.
-            repo.abort_write_group()
-            self.assertRaises(
-                errors.UnsuspendableWriteGroup, repo.resume_write_group, [])
-        else:
-            #self.assertEqual([], list(repo.texts.keys()))
-            same_repo = self.reopen_repo(repo)
-            same_repo.resume_write_group(wg_tokens)
-            self.assertEqual([text_key], list(same_repo.texts.keys()))
-            self.assertTrue(same_repo.is_in_write_group())
-            same_repo.abort_write_group()
-            self.assertEqual([], list(repo.texts.keys()))
-            # See also test_pack_repository's test of the same name.
-
-    def test_multiple_resume_write_group(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        first_key = ('file-id', 'revid')
-        repo.texts.add_lines(first_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        self.assertTrue(same_repo.is_in_write_group())
-        second_key = ('file-id', 'second-revid')
-        same_repo.texts.add_lines(second_key, (first_key,), ['more lines'])
-        try:
-            new_wg_tokens = same_repo.suspend_write_group()
-        except:
-            e = sys.exc_info()
-            same_repo.abort_write_group(suppress_errors=True)
-            raise e[0], e[1], e[2]
-        self.assertEqual(2, len(new_wg_tokens))
-        self.assertSubset(wg_tokens, new_wg_tokens)
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(new_wg_tokens)
-        both_keys = set([first_key, second_key])
-        self.assertEqual(both_keys, same_repo.texts.keys())
-        same_repo.abort_write_group()
-
-    def test_no_op_suspend_resume(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        new_wg_tokens = same_repo.suspend_write_group()
-        self.assertEqual(wg_tokens, new_wg_tokens)
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        self.assertEqual([text_key], list(same_repo.texts.keys()))
-        same_repo.abort_write_group()
-
-    def test_read_after_suspend_fails(self):
-        self.require_suspendable_write_groups(
-            'Cannot test suspend on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        self.assertEqual([], list(repo.texts.keys()))
-
-    def test_read_after_second_suspend_fails(self):
-        self.require_suspendable_write_groups(
-            'Cannot test suspend on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        same_repo.suspend_write_group()
-        self.assertEqual([], list(same_repo.texts.keys()))
-
-    def test_read_after_resume_abort_fails(self):
-        self.require_suspendable_write_groups(
-            'Cannot test suspend on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        same_repo.abort_write_group()
-        self.assertEqual([], list(same_repo.texts.keys()))
-
-    def test_cannot_resume_aborted_write_group(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        same_repo.abort_write_group()
-        same_repo = self.reopen_repo(repo)
-        self.assertRaises(
-            errors.UnresumableWriteGroup, same_repo.resume_write_group,
-            wg_tokens)
-
-    def test_commit_resumed_write_group_no_new_data(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        same_repo.commit_write_group()
-        self.assertEqual([text_key], list(same_repo.texts.keys()))
-        self.assertEqual(
-            'lines', same_repo.texts.get_record_stream([text_key],
-                'unordered', True).next().get_bytes_as('fulltext'))
-        self.assertRaises(
-            errors.UnresumableWriteGroup, same_repo.resume_write_group,
-            wg_tokens)
-
-    def test_commit_resumed_write_group_plus_new_data(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        first_key = ('file-id', 'revid')
-        repo.texts.add_lines(first_key, (), ['lines'])
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        second_key = ('file-id', 'second-revid')
-        same_repo.texts.add_lines(second_key, (first_key,), ['more lines'])
-        same_repo.commit_write_group()
-        self.assertEqual(
-            set([first_key, second_key]), set(same_repo.texts.keys()))
-        self.assertEqual(
-            'lines', same_repo.texts.get_record_stream([first_key],
-                'unordered', True).next().get_bytes_as('fulltext'))
-        self.assertEqual(
-            'more lines', same_repo.texts.get_record_stream([second_key],
-                'unordered', True).next().get_bytes_as('fulltext'))
-
-    def make_source_with_delta_record(self):
-        # Make a source repository with a delta record in it.
-        source_repo = self.make_write_locked_repo('source')
-        source_repo.start_write_group()
-        key_base = ('file-id', 'base')
-        key_delta = ('file-id', 'delta')
-        def text_stream():
-            yield versionedfile.FulltextContentFactory(
-                key_base, (), None, 'lines\n')
-            yield versionedfile.FulltextContentFactory(
-                key_delta, (key_base,), None, 'more\nlines\n')
-        source_repo.texts.insert_record_stream(text_stream())
-        source_repo.commit_write_group()
-        return source_repo
-
-    def test_commit_resumed_write_group_with_missing_parents(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        source_repo = self.make_source_with_delta_record()
-        key_base = ('file-id', 'base')
-        key_delta = ('file-id', 'delta')
-        # Start a write group, insert just a delta.
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        stream = source_repo.texts.get_record_stream(
-            [key_delta], 'unordered', False)
-        repo.texts.insert_record_stream(stream)
-        # It's either not commitable due to the missing compression parent, or
-        # the stacked location has already filled in the fulltext.
-        try:
-            repo.commit_write_group()
-        except errors.BzrCheckError:
-            # It refused to commit because we have a missing parent
-            pass
-        else:
-            same_repo = self.reopen_repo(repo)
-            same_repo.lock_read()
-            record = same_repo.texts.get_record_stream([key_delta],
-                                                       'unordered', True).next()
-            self.assertEqual('more\nlines\n', record.get_bytes_as('fulltext'))
-            return
-        # Merely suspending and resuming doesn't make it commitable either.
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        self.assertRaises(
-            errors.BzrCheckError, same_repo.commit_write_group)
-        same_repo.abort_write_group()
-
-    def test_commit_resumed_write_group_adding_missing_parents(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        source_repo = self.make_source_with_delta_record()
-        key_base = ('file-id', 'base')
-        key_delta = ('file-id', 'delta')
-        # Start a write group.
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        # Add some content so this isn't an empty write group (which may return
-        # 0 tokens)
-        text_key = ('file-id', 'revid')
-        repo.texts.add_lines(text_key, (), ['lines'])
-        # Suspend it, then resume it.
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        # Add a record with a missing compression parent
-        stream = source_repo.texts.get_record_stream(
-            [key_delta], 'unordered', False)
-        same_repo.texts.insert_record_stream(stream)
-        # Just like if we'd added that record without a suspend/resume cycle,
-        # commit_write_group fails.
-        try:
-            same_repo.commit_write_group()
-        except errors.BzrCheckError:
-            pass
-        else:
-            # If the commit_write_group didn't fail, that is because the
-            # insert_record_stream already gave it a fulltext.
-            same_repo = self.reopen_repo(repo)
-            same_repo.lock_read()
-            record = same_repo.texts.get_record_stream([key_delta],
-                                                       'unordered', True).next()
-            self.assertEqual('more\nlines\n', record.get_bytes_as('fulltext'))
-            return
-        same_repo.abort_write_group()
-
-    def test_add_missing_parent_after_resume(self):
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        source_repo = self.make_source_with_delta_record()
-        key_base = ('file-id', 'base')
-        key_delta = ('file-id', 'delta')
-        # Start a write group, insert just a delta.
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        stream = source_repo.texts.get_record_stream(
-            [key_delta], 'unordered', False)
-        repo.texts.insert_record_stream(stream)
-        # Suspend it, then resume it.
-        wg_tokens = repo.suspend_write_group()
-        same_repo = self.reopen_repo(repo)
-        same_repo.resume_write_group(wg_tokens)
-        # Fill in the missing compression parent.
-        stream = source_repo.texts.get_record_stream(
-            [key_base], 'unordered', False)
-        same_repo.texts.insert_record_stream(stream)
-        same_repo.commit_write_group()
-
-    def test_suspend_empty_initial_write_group(self):
-        """Suspending a write group with no writes returns an empty token
-        list.
-        """
-        self.require_suspendable_write_groups(
-            'Cannot test suspend on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.start_write_group()
-        wg_tokens = repo.suspend_write_group()
-        self.assertEqual([], wg_tokens)
-
-    def test_suspend_empty_initial_write_group(self):
-        """Resuming an empty token list is equivalent to start_write_group."""
-        self.require_suspendable_write_groups(
-            'Cannot test resume on repo that does not support suspending')
-        repo = self.make_write_locked_repo()
-        repo.resume_write_group([])
-        repo.abort_write_group()
-
-

=== modified file 'bzrlib/tests/per_repository_chk/__init__.py'
--- bzrlib/tests/per_repository_chk/__init__.py	2010-03-31 16:29:25 +0000
+++ bzrlib/tests/per_repository_chk/__init__.py	2011-04-05 14:47:30 +0000
@@ -27,11 +27,11 @@
     repository,
     remote,
     )
-from bzrlib.repofmt.pack_repo import (
+from bzrlib.repofmt.knitpack_repo import (
     RepositoryFormatKnitPack5,
     )
 from bzrlib.repofmt.groupcompress_repo import (
-    RepositoryFormatCHK1,
+    RepositoryFormat2a,
     )
 from bzrlib.tests import (
     multiply_tests,
@@ -61,7 +61,7 @@
         # with the chosen backing format.
         if isinstance(format, remote.RemoteRepositoryFormat):
             with_support = dict(scenario_info)
-            with_support['repository_format'] = RepositoryFormatCHK1()
+            with_support['repository_format'] = RepositoryFormat2a()
             supported_scenarios.append((test_name + "(Supported)", with_support))
             no_support = dict(scenario_info)
             no_support['repository_format'] = RepositoryFormatKnitPack5()

=== modified file 'bzrlib/tests/per_repository_reference/__init__.py'
--- bzrlib/tests/per_repository_reference/__init__.py	2010-05-20 02:57:52 +0000
+++ bzrlib/tests/per_repository_reference/__init__.py	2011-10-01 11:54:46 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,12 +24,10 @@
 
 from bzrlib import (
     errors,
-    repository,
     remote,
+    urlutils,
     )
-from bzrlib.branch import BzrBranchFormat7
 from bzrlib.bzrdir import BzrDir
-from bzrlib.repofmt.pack_repo import RepositoryFormatKnitPack6
 from bzrlib.tests import multiply_tests
 from bzrlib.tests.per_repository import (
     all_repository_format_scenarios,
@@ -39,17 +37,18 @@
 
 class TestCaseWithExternalReferenceRepository(TestCaseWithRepository):
 
-    def make_referring(self, relpath, target_path):
+    def make_referring(self, relpath, a_repository):
         """Get a new repository that refers to a_repository.
 
         :param relpath: The path to create the repository at.
         :param a_repository: A repository to refer to.
         """
         repo = self.make_repository(relpath)
-        repo.add_fallback_repository(self.readonly_repository(target_path))
+        repo.add_fallback_repository(self.readonly_repository(a_repository))
         return repo
 
-    def readonly_repository(self, relpath):
+    def readonly_repository(self, repo):
+        relpath = urlutils.basename(repo.bzrdir.user_url.rstrip('/'))
         return BzrDir.open_from_transport(
             self.get_readonly_transport(relpath)).open_repository()
 
@@ -61,26 +60,45 @@
         # because developers use this api to setup the tree, branch and
         # repository for their tests: having it not give the right repository
         # type would invalidate the tests.
-        self.make_branch_and_tree('repo')
-        repo = self.make_referring('referring', 'repo')
+        tree = self.make_branch_and_tree('repo')
+        repo = self.make_referring('referring', tree.branch.repository)
         self.assertIsInstance(repo._format,
             self.repository_format.__class__)
 
 
 class TestIncompatibleStacking(TestCaseWithRepository):
 
-    def test_add_fallback_repository_rejects_incompatible(self):
-        # Repository.add_fallback_repository raises IncompatibleRepositories if
-        # you take two repositories in different serializations and try to
-        # stack them.
-        if self.make_repository('test')._format.supports_chks:
+    def make_repo_and_incompatible_fallback(self):
+        referring = self.make_repository('referring')
+        if referring._format.supports_chks:
             different_fmt = '1.9'
         else:
             different_fmt = '2a'
-        repo = self.make_repository('repo', format=different_fmt)
-        referring = self.make_repository('referring')
-        self.assertRaises(errors.IncompatibleRepositories,
-                referring.add_fallback_repository, repo)
+        fallback = self.make_repository('fallback', format=different_fmt)
+        return referring, fallback
+
+    def test_add_fallback_repository_rejects_incompatible(self):
+        # Repository.add_fallback_repository raises IncompatibleRepositories
+        # if you take two repositories in different serializations and try to
+        # stack them.
+        referring, fallback = self.make_repo_and_incompatible_fallback()
+        self.assertRaises(errors.IncompatibleRepositories,
+                referring.add_fallback_repository, fallback)
+
+    def test_add_fallback_doesnt_leave_fallback_locked(self):
+        # Bug #835035. If the referring repository is locked, it wants to lock
+        # the fallback repository. But if they are incompatible, the referring
+        # repository won't take ownership of the fallback, and thus should not
+        # leave the repository in a locked state.
+        referring, fallback = self.make_repo_and_incompatible_fallback()
+        self.addCleanup(referring.lock_read().unlock)
+        # Assert precondition.
+        self.assertFalse(fallback.is_locked())
+        # Assert action.
+        self.assertRaises(errors.IncompatibleRepositories,
+                referring.add_fallback_repository, fallback)
+        # Assert postcondition.
+        self.assertFalse(fallback.is_locked())
 
 
 def external_reference_test_scenarios():
@@ -89,18 +107,8 @@
     result = []
     for test_name, scenario_info in all_repository_format_scenarios():
         format = scenario_info['repository_format']
-        if isinstance(format, remote.RemoteRepositoryFormat):
-            # This is a RemoteRepositoryFormat scenario.  Force the scenario to
-            # use real branch and repository formats that support references.
-            scenario_info = dict(scenario_info)
-            format = remote.RemoteRepositoryFormat()
-            format._custom_format = RepositoryFormatKnitPack6()
-            scenario_info['repository_format'] = format
-            bzrdir_format = remote.RemoteBzrDirFormat()
-            bzrdir_format.repository_format = format
-            bzrdir_format.set_branch_format(BzrBranchFormat7())
-            scenario_info['bzrdir_format'] = bzrdir_format
-        if format.supports_external_lookups:
+        if (isinstance(format, remote.RemoteRepositoryFormat)
+            or format.supports_external_lookups):
             result.append((test_name, scenario_info))
     return result
 
@@ -113,11 +121,14 @@
         'bzrlib.tests.per_repository_reference.test_all_revision_ids',
         'bzrlib.tests.per_repository_reference.test_break_lock',
         'bzrlib.tests.per_repository_reference.test_check',
+        'bzrlib.tests.per_repository_reference.test_commit_with_stacking',
         'bzrlib.tests.per_repository_reference.test_default_stacking',
         'bzrlib.tests.per_repository_reference.test_fetch',
         'bzrlib.tests.per_repository_reference.test_get_record_stream',
         'bzrlib.tests.per_repository_reference.test_get_rev_id_for_revno',
+        'bzrlib.tests.per_repository_reference.test_graph',
         'bzrlib.tests.per_repository_reference.test_initialize',
+        'bzrlib.tests.per_repository_reference.test__make_parents_provider',
         'bzrlib.tests.per_repository_reference.test_unlock',
         ]
     # Parameterize per_repository_reference test modules by format.

=== added file 'bzrlib/tests/per_repository_reference/test__make_parents_provider.py'
--- bzrlib/tests/per_repository_reference/test__make_parents_provider.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_reference/test__make_parents_provider.py	2011-05-04 03:43:03 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Tests for _make_parents_provider on stacked repositories."""
+
+
+from bzrlib.tests.per_repository import TestCaseWithRepository
+
+
+class Test_MakeParentsProvider(TestCaseWithRepository):
+
+    def test_add_fallback_after_make_pp(self):
+        """Fallbacks added after _make_parents_provider are used by that
+        provider.
+        """
+        referring_repo = self.make_repository('repo')
+        pp = referring_repo._make_parents_provider()
+        # Initially referring_repo has no revisions and no fallbacks
+        self.addCleanup(referring_repo.lock_read().unlock)
+        self.assertEqual({}, pp.get_parent_map(['revid2']))
+        # Add a fallback repo with a commit
+        wt_a = self.make_branch_and_tree('fallback')
+        wt_a.commit('first commit', rev_id='revid1')
+        wt_a.commit('second commit', rev_id='revid2')
+        fallback_repo = wt_a.branch.repository
+        referring_repo.add_fallback_repository(fallback_repo)
+        # Now revid1 appears in pp's results.
+        self.assertEqual(('revid1',), pp.get_parent_map(['revid2'])['revid2'])
+
+

=== modified file 'bzrlib/tests/per_repository_reference/test_add_inventory.py'
--- bzrlib/tests/per_repository_reference/test_add_inventory.py	2009-04-09 20:23:07 +0000
+++ bzrlib/tests/per_repository_reference/test_add_inventory.py	2011-10-01 11:54:46 +0000
@@ -33,7 +33,7 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.lock_write()
         try:
             repo.start_write_group()

=== modified file 'bzrlib/tests/per_repository_reference/test_add_revision.py'
--- bzrlib/tests/per_repository_reference/test_add_revision.py	2009-09-08 14:51:23 +0000
+++ bzrlib/tests/per_repository_reference/test_add_revision.py	2011-10-01 11:54:46 +0000
@@ -34,7 +34,7 @@
         self.addCleanup(tree.unlock)
         rev = tree.branch.repository.get_revision(revid)
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.lock_write()
         try:
             repo.start_write_group()

=== modified file 'bzrlib/tests/per_repository_reference/test_add_signature_text.py'
--- bzrlib/tests/per_repository_reference/test_add_signature_text.py	2009-09-08 14:51:23 +0000
+++ bzrlib/tests/per_repository_reference/test_add_signature_text.py	2011-10-01 11:54:46 +0000
@@ -33,7 +33,7 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.lock_write()
         try:
             repo.start_write_group()

=== modified file 'bzrlib/tests/per_repository_reference/test_all_revision_ids.py'
--- bzrlib/tests/per_repository_reference/test_all_revision_ids.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository_reference/test_all_revision_ids.py	2011-10-01 11:54:46 +0000
@@ -16,7 +16,6 @@
 
 """Tests for all_revision_ids on a repository with external references."""
 
-from bzrlib import errors
 from bzrlib.tests.per_repository_reference import (
     TestCaseWithExternalReferenceRepository,
     )
@@ -26,20 +25,20 @@
 
     def test_all_revision_ids_empty(self):
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         self.assertEqual(set([]), set(repo.all_revision_ids()))
 
     def test_all_revision_ids_from_base(self):
         tree = self.make_branch_and_tree('base')
         revid = tree.commit('one')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', tree.branch.repository)
         self.assertEqual(set([revid]), set(repo.all_revision_ids()))
 
     def test_all_revision_ids_from_repo(self):
         tree = self.make_branch_and_tree('spare')
         revid = tree.commit('one')
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.fetch(tree.branch.repository, revid)
         self.assertEqual(set([revid]), set(repo.all_revision_ids()))
 
@@ -48,7 +47,7 @@
         revid = tree.commit('one')
         base_tree = self.make_branch_and_tree('base')
         revid2 = base_tree.commit('two')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base_tree.branch.repository)
         repo.fetch(tree.branch.repository, revid)
         self.assertEqual(set([revid, revid2]), set(repo.all_revision_ids()))
 
@@ -56,7 +55,7 @@
         tree = self.make_branch_and_tree('spare')
         revid = tree.commit('one')
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.fetch(tree.branch.repository, revid)
         base.fetch(tree.branch.repository, revid)
         self.assertEqual(set([revid]), set(repo.all_revision_ids()))

=== modified file 'bzrlib/tests/per_repository_reference/test_break_lock.py'
--- bzrlib/tests/per_repository_reference/test_break_lock.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/per_repository_reference/test_break_lock.py	2011-10-01 11:54:46 +0000
@@ -29,7 +29,7 @@
 
     def test_break_lock(self):
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         unused_repo = repo.bzrdir.open_repository()
         base.lock_write()
         self.addCleanup(base.unlock)

=== modified file 'bzrlib/tests/per_repository_reference/test_check.py'
--- bzrlib/tests/per_repository_reference/test_check.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_repository_reference/test_check.py	2011-10-01 11:54:46 +0000
@@ -17,7 +17,6 @@
 """Tests for check on a repository with external references."""
 
 import bzrlib.ui
-from bzrlib import errors
 from bzrlib.tests.per_repository_reference import (
     TestCaseWithExternalReferenceRepository,
     )
@@ -31,7 +30,7 @@
         tree.add(['file'], ['file-id'])
         rev1_id = tree.commit('one')
         referring = self.make_branch_and_tree('referring')
-        readonly_base = self.readonly_repository('base')
+        readonly_base = self.readonly_repository(tree.branch.repository)
         referring.branch.repository.add_fallback_repository(readonly_base)
         local_tree = referring.branch.create_checkout('local')
         self.build_tree_contents([('local/file', 'change')])

=== added file 'bzrlib/tests/per_repository_reference/test_commit_with_stacking.py'
--- bzrlib/tests/per_repository_reference/test_commit_with_stacking.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_reference/test_commit_with_stacking.py	2011-01-11 00:36:10 +0000
@@ -0,0 +1,220 @@
+# Copyright (C) 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+from bzrlib import (
+    errors,
+    remote,
+    tests,
+    urlutils,
+    )
+from bzrlib.tests.per_repository import TestCaseWithRepository
+
+
+class TestCaseWithStackedTarget(TestCaseWithRepository):
+
+    r1_key = ('rev1-id',)
+    r2_key = ('rev2-id',)
+
+    def make_stacked_target(self):
+        base_tree = self.make_branch_and_tree('base')
+        self.build_tree(['base/f1.txt'])
+        base_tree.add(['f1.txt'], ['f1.txt-id'])
+        base_tree.commit('initial', rev_id=self.r1_key[0])
+        self.build_tree(['base/f2.txt'])
+        base_tree.add(['f2.txt'], ['f2.txt-id'])
+        base_tree.commit('base adds f2', rev_id=self.r2_key[0])
+        stacked_url = urlutils.join(base_tree.branch.base, '../stacked')
+        stacked_bzrdir = base_tree.bzrdir.sprout(stacked_url,
+            stacked=True)
+        if isinstance(stacked_bzrdir, remote.RemoteBzrDir):
+            stacked_branch = stacked_bzrdir.open_branch()
+            stacked_tree = stacked_branch.create_checkout('stacked',
+                lightweight=True)
+        else:
+            stacked_tree = stacked_bzrdir.open_workingtree()
+        return base_tree, stacked_tree
+
+
+class TestCommitWithStacking(TestCaseWithStackedTarget):
+
+    def setUp(self):
+        super(TestCommitWithStacking, self).setUp()
+        format = self.repository_format
+        if (not (isinstance(format, remote.RemoteRepositoryFormat)
+                 or format.supports_chks)):
+            raise tests.TestNotApplicable('stacked commit only supported'
+                ' for chk repositories')
+
+    def get_only_repo(self, tree):
+        """Open just the repository used by this tree.
+
+        This returns a read locked Repository object without any stacking
+        fallbacks.
+        """
+        repo = tree.branch.repository.bzrdir.open_repository()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        return repo
+
+    def assertPresent(self, expected, vf, keys):
+        """Check which of the supplied keys are present."""
+        parent_map = vf.get_parent_map(keys)
+        self.assertEqual(sorted(expected), sorted(parent_map))
+
+    def test_simple_commit(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.assertEqual(1,
+                len(stacked_tree.branch.repository._fallback_repositories))
+        self.build_tree_contents([('stacked/f1.txt', 'new content\n')])
+        stacked_tree.commit('new content', rev_id='new-rev-id')
+        # We open the repository without fallbacks to ensure the data is
+        # locally true
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        # We should have the immediate parent inventory available, but not the
+        # grandparent's
+        self.assertPresent([self.r2_key],
+            stacked_only_repo.inventories, [self.r1_key, self.r2_key])
+        # And we should be able to pull this revision into another stacked
+        # branch
+        stacked2_branch = base_tree.bzrdir.sprout('stacked2',
+                                                  stacked=True).open_branch()
+        stacked2_branch.repository.fetch(stacked_only_repo,
+                                         revision_id='new-rev-id')
+
+    def test_merge_commit(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree_contents([('base/f1.txt', 'new content\n')])
+        r3_key = ('rev3-id',)
+        base_tree.commit('second base', rev_id=r3_key[0])
+        to_be_merged_tree = base_tree.bzrdir.sprout('merged'
+            ).open_workingtree()
+        self.build_tree(['merged/f2.txt'])
+        to_be_merged_tree.add(['f2.txt'], ['f2.txt-id'])
+        to_merge_key = ('to-merge-rev-id',)
+        to_be_merged_tree.commit('new-to-be-merged', rev_id=to_merge_key[0])
+        stacked_tree.merge_from_branch(to_be_merged_tree.branch)
+        merged_key = ('merged-rev-id',)
+        stacked_tree.commit('merge', rev_id=merged_key[0])
+        # to-merge isn't in base, so it should be in stacked.
+        # rev3-id is a parent of a revision we have, so we should have the
+        # inventory, but not the revision.
+        # merged has a parent of r2, so we should also have r2's
+        # inventory-but-not-revision.
+        # Nothing has r1 directly, so we shouldn't have anything present for it
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        all_keys = [self.r1_key, self.r2_key, r3_key, to_merge_key, merged_key]
+        self.assertPresent([to_merge_key, merged_key],
+                           stacked_only_repo.revisions, all_keys)
+        self.assertPresent([self.r2_key, r3_key, to_merge_key, merged_key],
+                           stacked_only_repo.inventories, all_keys)
+
+    def test_merge_from_master(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree_contents([('base/f1.txt', 'new content\n')])
+        r3_key = ('rev3-id',)
+        base_tree.commit('second base', rev_id=r3_key[0])
+        stacked_tree.merge_from_branch(base_tree.branch)
+        merged_key = ('merged-rev-id',)
+        stacked_tree.commit('merge', rev_id=merged_key[0])
+        all_keys = [self.r1_key, self.r2_key, r3_key, merged_key]
+        # We shouldn't have any of the base revisions in the local repo, but we
+        # should have both base inventories.
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        self.assertPresent([merged_key],
+                           stacked_only_repo.revisions, all_keys)
+        self.assertPresent([self.r2_key, r3_key, merged_key],
+                           stacked_only_repo.inventories, all_keys)
+
+    def test_multi_stack(self):
+        """base + stacked + stacked-on-stacked"""
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree(['stacked/f3.txt'])
+        stacked_tree.add(['f3.txt'], ['f3.txt-id'])
+        stacked_key = ('stacked-rev-id',)
+        stacked_tree.commit('add f3', rev_id=stacked_key[0])
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        self.assertPresent([self.r2_key], stacked_only_repo.inventories,
+                           [self.r1_key, self.r2_key])
+        # This ensures we get a Remote URL, rather than a local one.
+        stacked2_url = urlutils.join(base_tree.branch.base, '../stacked2')
+        stacked2_bzrdir = stacked_tree.bzrdir.sprout(stacked2_url,
+                            revision_id=self.r1_key[0],
+                            stacked=True)
+        if isinstance(stacked2_bzrdir, remote.RemoteBzrDir):
+            stacked2_branch = stacked2_bzrdir.open_branch()
+            stacked2_tree = stacked2_branch.create_checkout('stacked2',
+                lightweight=True)
+        else:
+            stacked2_tree = stacked2_bzrdir.open_workingtree()
+        # stacked2 is stacked on stacked, but its content is rev1, so
+        # it needs to pull the basis information from a fallback-of-fallback.
+        self.build_tree(['stacked2/f3.txt'])
+        stacked2_only_repo = self.get_only_repo(stacked2_tree)
+        self.assertPresent([], stacked2_only_repo.inventories,
+                           [self.r1_key, self.r2_key])
+        stacked2_tree.add(['f3.txt'], ['f3.txt-id'])
+        stacked2_tree.commit('add f3', rev_id='stacked2-rev-id')
+        # We added data to this read-locked repo, so refresh it
+        stacked2_only_repo.refresh_data()
+        self.assertPresent([self.r1_key], stacked2_only_repo.inventories,
+                           [self.r1_key, self.r2_key])
+
+    def test_commit_with_ghosts_fails(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        stacked_tree.set_parent_ids([stacked_tree.last_revision(),
+                                     'ghost-rev-id'])
+        self.assertRaises(errors.BzrError,
+            stacked_tree.commit, 'failed_commit')
+
+    def test_commit_with_ghost_in_ancestry(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        self.build_tree_contents([('base/f1.txt', 'new content\n')])
+        r3_key = ('rev3-id',)
+        base_tree.commit('second base', rev_id=r3_key[0])
+        to_be_merged_tree = base_tree.bzrdir.sprout('merged'
+            ).open_workingtree()
+        self.build_tree(['merged/f2.txt'])
+        to_be_merged_tree.add(['f2.txt'], ['f2.txt-id'])
+        ghost_key = ('ghost-rev-id',)
+        to_be_merged_tree.set_parent_ids([r3_key[0], ghost_key[0]])
+        to_merge_key = ('to-merge-rev-id',)
+        to_be_merged_tree.commit('new-to-be-merged', rev_id=to_merge_key[0])
+        stacked_tree.merge_from_branch(to_be_merged_tree.branch)
+        merged_key = ('merged-rev-id',)
+        stacked_tree.commit('merge', rev_id=merged_key[0])
+        # vs test_merge_commit, the fetch for 'merge_from_branch' should
+        # already have handled that 'ghost-rev-id' is a ghost, and commit
+        # should not try to fill it in at this point.
+        stacked_only_repo = self.get_only_repo(stacked_tree)
+        all_keys = [self.r1_key, self.r2_key, r3_key, to_merge_key, merged_key,
+                    ghost_key]
+        self.assertPresent([to_merge_key, merged_key],
+                           stacked_only_repo.revisions, all_keys)
+        self.assertPresent([self.r2_key, r3_key, to_merge_key, merged_key],
+                           stacked_only_repo.inventories, all_keys)
+
+
+class TestCommitStackedFailsAppropriately(TestCaseWithStackedTarget):
+
+    def test_stacked_commit_fails_on_old_formats(self):
+        base_tree, stacked_tree = self.make_stacked_target()
+        format = stacked_tree.branch.repository._format
+        if format.supports_chks:
+            stacked_tree.commit('should succeed')
+        else:
+            self.assertRaises(errors.BzrError,
+                stacked_tree.commit, 'unsupported format')

=== modified file 'bzrlib/tests/per_repository_reference/test_fetch.py'
--- bzrlib/tests/per_repository_reference/test_fetch.py	2009-06-01 18:13:46 +0000
+++ bzrlib/tests/per_repository_reference/test_fetch.py	2010-12-06 04:59:38 +0000
@@ -18,6 +18,7 @@
 from bzrlib import (
     branch,
     errors,
+    graph,
     )
 from bzrlib.smart import (
     server,
@@ -25,7 +26,7 @@
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
-class TestFetch(TestCaseWithRepository):
+class TestFetchBase(TestCaseWithRepository):
 
     def make_source_branch(self):
         # It would be nice if there was a way to force this to be memory-only
@@ -51,6 +52,9 @@
         self.addCleanup(source_b.unlock)
         return content, source_b
 
+
+class TestFetch(TestFetchBase):
+
     def test_sprout_from_stacked_with_short_history(self):
         content, source_b = self.make_source_branch()
         # Split the generated content into a base branch, and a stacked branch
@@ -149,3 +153,38 @@
         source_b.lock_read()
         self.addCleanup(source_b.unlock)
         stacked.pull(source_b, stop_revision='B-id')
+
+
+class TestFetchFromRepoWithUnconfiguredFallbacks(TestFetchBase):
+
+    def make_stacked_source_repo(self):
+        _, source_b = self.make_source_branch()
+        # Use 'make_branch' which gives us a bzr:// branch when appropriate,
+        # rather than creating a branch-on-disk
+        stack_b = self.make_branch('stack-on')
+        stack_b.pull(source_b, stop_revision='B-id')
+        stacked_b = self.make_branch('stacked')
+        stacked_b.set_stacked_on_url('../stack-on')
+        stacked_b.pull(source_b, stop_revision='C-id')
+        return stacked_b.repository
+
+    def test_fetch_everything_includes_parent_invs(self):
+        stacked = self.make_stacked_source_repo()
+        repo_missing_fallbacks = stacked.bzrdir.open_repository()
+        self.addCleanup(repo_missing_fallbacks.lock_read().unlock)
+        target = self.make_repository('target')
+        self.addCleanup(target.lock_write().unlock)
+        target.fetch(
+            repo_missing_fallbacks,
+            fetch_spec=graph.EverythingResult(repo_missing_fallbacks))
+        self.assertEqual(repo_missing_fallbacks.revisions.keys(),
+            target.revisions.keys())
+        self.assertEqual(repo_missing_fallbacks.inventories.keys(),
+            target.inventories.keys())
+        self.assertEqual(['C-id'],
+            sorted(k[-1] for k in target.revisions.keys()))
+        self.assertEqual(['B-id', 'C-id'],
+            sorted(k[-1] for k in target.inventories.keys()))
+
+
+

=== modified file 'bzrlib/tests/per_repository_reference/test_get_record_stream.py'
--- bzrlib/tests/per_repository_reference/test_get_record_stream.py	2009-07-15 18:22:38 +0000
+++ bzrlib/tests/per_repository_reference/test_get_record_stream.py	2011-10-01 11:54:46 +0000
@@ -82,7 +82,7 @@
         self.all_repo.lock_read()
         self.addCleanup(self.all_repo.unlock)
         self.base_repo = self.make_repository('base')
-        self.stacked_repo = self.make_referring('referring', 'base')
+        self.stacked_repo = self.make_referring('referring', self.base_repo)
 
     def make_simple_split(self):
         """Set up the repositories so that everything is in base except F"""

=== modified file 'bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py'
--- bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py	2009-08-20 04:09:58 +0000
+++ bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py	2011-10-01 11:54:46 +0000
@@ -31,7 +31,8 @@
         spare_tree = tree.bzrdir.sprout('spare').open_workingtree()
         revid3 = spare_tree.commit('three')
         branch = spare_tree.branch.create_clone_on_transport(
-            self.get_transport('referring'), stacked_on=self.get_url('base'))
+            self.get_transport('referring'),
+            stacked_on=tree.branch.base)
         repo = branch.repository
         # Sanity check: now repo has 'revid3', and base has 'revid' + 'revid2'
         self.assertEqual(set([revid3]),

=== added file 'bzrlib/tests/per_repository_reference/test_graph.py'
--- bzrlib/tests/per_repository_reference/test_graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_reference/test_graph.py	2011-10-01 11:39:29 +0000
@@ -0,0 +1,116 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Tests for graph operations on stacked repositories."""
+
+
+from bzrlib import (
+    remote,
+    tests,
+    urlutils,
+    )
+from bzrlib.tests.per_repository import TestCaseWithRepository
+
+
+class TestGraph(TestCaseWithRepository):
+
+    def test_get_known_graph_ancestry_stacked(self):
+        """get_known_graph_ancestry works correctly on stacking.
+
+        See <https://bugs.launchpad.net/bugs/715000>.
+        """
+        branch_a, branch_b, branch_c, revid_1 = self.make_double_stacked_branches()
+        for br in [branch_a, branch_b, branch_c]:
+            self.assertEquals(
+                [revid_1],
+                br.repository.get_known_graph_ancestry([revid_1]).topo_sort())
+
+    def make_double_stacked_branches(self):
+        wt_a = self.make_branch_and_tree('a')
+        branch_a = wt_a.branch
+        branch_b = self.make_branch('b')
+        branch_b.set_stacked_on_url(
+            urlutils.relative_url(branch_b.base, branch_a.base))
+        branch_c = self.make_branch('c')
+        branch_c.set_stacked_on_url(
+            urlutils.relative_url(branch_c.base, branch_b.base))
+        revid_1 = wt_a.commit('first commit')
+        return branch_a, branch_b, branch_c, revid_1
+
+    def make_stacked_branch_with_long_history(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        builder.build_snapshot('A', None, [
+            ('add', ('', 'directory', 'root-id', None))])
+        builder.build_snapshot('B', ['A'], [])
+        builder.build_snapshot('C', ['B'], [])
+        builder.build_snapshot('D', ['C'], [])
+        builder.build_snapshot('E', ['D'], [])
+        builder.build_snapshot('F', ['E'], [])
+        source_b = builder.get_branch()
+        master_b = self.make_branch('master')
+        master_b.pull(source_b, stop_revision='E')
+        stacked_b = self.make_branch('stacked')
+        stacked_b.set_stacked_on_url('../master')
+        stacked_b.pull(source_b, stop_revision='F')
+        builder.finish_series()
+        return master_b, stacked_b
+
+    def assertParentMapCalls(self, expected):
+        """Check that self.hpss_calls has the expected get_parent_map calls."""
+        get_parent_map_calls = []
+        for c in self.hpss_calls:
+            # Right now, the only RPCs that get called are get_parent_map. If
+            # this changes in the future, we can change this to:
+            # if c.call.method != 'Repository.get_parent_map':
+            #    continue
+            self.assertEqual('Repository.get_parent_map', c.call.method)
+            args = c.call.args
+            location = args[0]
+            self.assertEqual('include-missing:', args[1])
+            revisions = sorted(args[2:])
+            get_parent_map_calls.append((location, revisions))
+        self.assertEqual(expected, get_parent_map_calls)
+
+    def test_doesnt_call_get_parent_map_on_all_fallback_revs(self):
+        if not isinstance(self.repository_format,
+                          remote.RemoteRepositoryFormat):
+            raise tests.TestNotApplicable('only for RemoteRepository')
+        # bug #388269
+        master_b, stacked_b = self.make_stacked_branch_with_long_history()
+        self.addCleanup(stacked_b.lock_read().unlock)
+        self.make_repository('target_repo', shared=True)
+        target_b = self.make_branch('target_repo/branch')
+        self.addCleanup(target_b.lock_write().unlock)
+        self.setup_smart_server_with_call_log()
+        res = target_b.repository.search_missing_revision_ids(
+                stacked_b.repository, revision_ids=['F'],
+                find_ghosts=False)
+        self.assertParentMapCalls([
+            # One call to stacked to start, which returns F=>E, and that E
+            # itself is missing, so when we step, we won't look for it.
+            ('extra/stacked/', ['F']),
+            # One fallback call to extra/master, which will return the rest of
+            # the history.
+            ('extra/master/', ['E']),
+            # And then one get_parent_map call to the target, to see if it
+            # already has any of these revisions.
+            ('extra/target_repo/branch/', ['A', 'B', 'C', 'D', 'E', 'F']),
+            ])
+        # Before bug #388269 was fixed, there would be a bunch of extra calls
+        # to 'extra/stacked', ['D'] then ['C'], then ['B'], then ['A'].
+        # One-at-a-time for the rest of the ancestry.

=== added directory 'bzrlib/tests/per_repository_vf'
=== added file 'bzrlib/tests/per_repository_vf/__init__.py'
--- bzrlib/tests/per_repository_vf/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/__init__.py	2011-09-12 09:51:52 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Implementation tests for VersionedFile-based repositories.
+
+For more generic per-repository tests, see bzrlib.tests.per_repository.
+"""
+
+from bzrlib.tests.per_repository import (
+    all_repository_format_scenarios,
+    TestCaseWithRepository,
+    )
+
+
+def all_repository_vf_format_scenarios():
+    scenarios = []
+    for test_name, scenario_info in all_repository_format_scenarios():
+        format = scenario_info['repository_format']
+        if format.supports_full_versioned_files:
+            scenarios.append((test_name, scenario_info))
+    return scenarios
+
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'test_add_inventory_by_delta',
+        'test_check',
+        'test_check_reconcile',
+        'test_find_text_key_references',
+        'test__generate_text_key_index',
+        'test_fetch',
+        'test_fileid_involved',
+        'test_merge_directive',
+        'test_reconcile',
+        'test_refresh_data',
+        'test_repository',
+        'test_write_group',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== renamed file 'bzrlib/tests/per_repository/helpers.py' => 'bzrlib/tests/per_repository_vf/helpers.py'
=== renamed file 'bzrlib/tests/per_repository/test__generate_text_key_index.py' => 'bzrlib/tests/per_repository_vf/test__generate_text_key_index.py'
--- bzrlib/tests/per_repository/test__generate_text_key_index.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository_vf/test__generate_text_key_index.py	2011-05-09 17:52:54 +0000
@@ -18,11 +18,20 @@
 """Tests for the _generate_text_key_index API."""
 
 
-from bzrlib.tests.per_repository import TestCaseWithRepository
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestGenerateTextKeyIndex(TestCaseWithRepository):
 
+    scenarios = all_repository_vf_format_scenarios()
+
     def test_empty(self):
         repo = self.make_repository('.')
         repo.lock_read()

=== renamed file 'bzrlib/tests/per_repository/test_add_inventory_by_delta.py' => 'bzrlib/tests/per_repository_vf/test_add_inventory_by_delta.py'
--- bzrlib/tests/per_repository/test_add_inventory_by_delta.py	2009-06-18 18:18:36 +0000
+++ bzrlib/tests/per_repository_vf/test_add_inventory_by_delta.py	2011-05-02 22:39:15 +0000
@@ -17,11 +17,19 @@
 """Tests for Repository.add_inventory_by_delta."""
 
 from bzrlib import errors, revision
-from bzrlib.tests.per_repository import TestCaseWithRepository
+from bzrlib.tests.per_repository_vf import (
+    all_repository_vf_format_scenarios,
+    TestCaseWithRepository
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestAddInventoryByDelta(TestCaseWithRepository):
 
+    scenarios = all_repository_vf_format_scenarios()
+
     def _get_repo_in_write_group(self, path='repository'):
         repo = self.make_repository(path)
         repo.lock_write()

=== added file 'bzrlib/tests/per_repository_vf/test_check.py'
--- bzrlib/tests/per_repository_vf/test_check.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_check.py	2011-05-26 21:02:47 +0000
@@ -0,0 +1,119 @@
+# Copyright (C) 2007-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Test operations that check the repository for corruption"""
+
+
+from bzrlib import (
+    errors,
+    )
+from bzrlib.tests import (
+    TestNotApplicable,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.per_repository_vf.helpers import (
+    TestCaseWithBrokenRevisionIndex,
+    )
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class TestFindInconsistentRevisionParents(TestCaseWithBrokenRevisionIndex):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test__find_inconsistent_revision_parents(self):
+        """_find_inconsistent_revision_parents finds revisions with broken
+        parents.
+        """
+        repo = self.make_repo_with_extra_ghost_index()
+        self.assertEqual(
+            [('revision-id', ('incorrect-parent',), ())],
+            list(repo._find_inconsistent_revision_parents()))
+
+    def test__check_for_inconsistent_revision_parents(self):
+        """_check_for_inconsistent_revision_parents raises BzrCheckError if
+        there are any revisions with inconsistent parents.
+        """
+        repo = self.make_repo_with_extra_ghost_index()
+        self.assertRaises(
+            errors.BzrCheckError,
+            repo._check_for_inconsistent_revision_parents)
+
+    def test__check_for_inconsistent_revision_parents_on_clean_repo(self):
+        """_check_for_inconsistent_revision_parents does nothing if there are
+        no broken revisions.
+        """
+        repo = self.make_repository('empty-repo')
+        if not repo._format.revision_graph_can_have_wrong_parents:
+            raise TestNotApplicable(
+                '%r cannot have corrupt revision index.' % repo)
+        repo.lock_read()
+        try:
+            repo._check_for_inconsistent_revision_parents()  # nothing happens
+        finally:
+            repo.unlock()
+
+    def test_check_reports_bad_ancestor(self):
+        repo = self.make_repo_with_extra_ghost_index()
+        # XXX: check requires a non-empty revision IDs list, but it ignores the
+        # contents of it!
+        check_object = repo.check(['ignored'])
+        check_object.report_results(verbose=False)
+        self.assertContainsRe(self.get_log(),
+            '1 revisions have incorrect parents in the revision index')
+        check_object.report_results(verbose=True)
+        self.assertContainsRe(
+            self.get_log(),
+            "revision-id has wrong parents in index: "
+            r"\('incorrect-parent',\) should be \(\)")
+
+
+class TestCallbacks(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test_callback_tree_and_branch(self):
+        # use a real tree to get actual refs that will work
+        tree = self.make_branch_and_tree('foo')
+        revid = tree.commit('foo')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        needed_refs = {}
+        for ref in tree._get_check_refs():
+            needed_refs.setdefault(ref, []).append(tree)
+        for ref in tree.branch._get_check_refs():
+            needed_refs.setdefault(ref, []).append(tree.branch)
+        self.tree_check = tree._check
+        self.branch_check = tree.branch.check
+        self.overrideAttr(tree, "_check", self.tree_callback)
+        self.overrideAttr(tree.branch, "check", self.branch_callback)
+        self.callbacks = []
+        tree.branch.repository.check([revid], callback_refs=needed_refs)
+        self.assertNotEqual([], self.callbacks)
+
+    def tree_callback(self, refs):
+        self.callbacks.append(('tree', refs))
+        return self.tree_check(refs)
+
+    def branch_callback(self, refs):
+        self.callbacks.append(('branch', refs))
+        return self.branch_check(refs)

=== renamed file 'bzrlib/tests/per_repository/test_check_reconcile.py' => 'bzrlib/tests/per_repository_vf/test_check_reconcile.py'
--- bzrlib/tests/per_repository/test_check_reconcile.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_repository_vf/test_check_reconcile.py	2011-05-11 14:26:41 +0000
@@ -21,15 +21,773 @@
 
 from bzrlib import osutils
 
-from bzrlib.inventory import Inventory, InventoryFile
-from bzrlib.revision import Revision
-from bzrlib.tests import TestNotApplicable
-from bzrlib.tests.per_repository import TestCaseWithRepository
+from bzrlib.inventory import (
+    Inventory,
+    InventoryFile,
+    )
+from bzrlib.revision import (
+    NULL_REVISION,
+    Revision,
+    )
+from bzrlib.tests import (
+    TestNotApplicable,
+    multiply_scenarios,
+    )
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class BrokenRepoScenario(object):
+    """Base class for defining scenarios for testing check and reconcile.
+
+    A subclass needs to define the following methods:
+        :populate_repository: a method to use to populate a repository with
+            sample revisions, inventories and file versions.
+        :all_versions_after_reconcile: all the versions in repository after
+            reconcile.  run_test verifies that the text of each of these
+            versions of the file is unchanged by the reconcile.
+        :populated_parents: a list of (parents list, revision).  Each version
+            of the file is verified to have the given parents before running
+            the reconcile.  i.e. this is used to assert that the repo from the
+            factory is what we expect.
+        :corrected_parents: a list of (parents list, revision).  Each version
+            of the file is verified to have the given parents after the
+            reconcile.  i.e. this is used to assert that reconcile made the
+            changes we expect it to make.
+
+    A subclass may define the following optional method as well:
+        :corrected_fulltexts: a list of file versions that should be stored as
+            fulltexts (not deltas) after reconcile.  run_test will verify that
+            this occurs.
+    """
+
+    def __init__(self, test_case):
+        self.test_case = test_case
+
+    def make_one_file_inventory(self, repo, revision, parents,
+                                inv_revision=None, root_revision=None,
+                                file_contents=None, make_file_version=True):
+        return self.test_case.make_one_file_inventory(
+            repo, revision, parents, inv_revision=inv_revision,
+            root_revision=root_revision, file_contents=file_contents,
+            make_file_version=make_file_version)
+
+    def add_revision(self, repo, revision_id, inv, parent_ids):
+        return self.test_case.add_revision(repo, revision_id, inv, parent_ids)
+
+    def corrected_fulltexts(self):
+        return []
+
+    def repository_text_key_index(self):
+        result = {}
+        if self.versioned_root:
+            result.update(self.versioned_repository_text_keys())
+        result.update(self.repository_text_keys())
+        return result
+
+
+class UndamagedRepositoryScenario(BrokenRepoScenario):
+    """A scenario where the repository has no damage.
+
+    It has a single revision, 'rev1a', with a single file.
+    """
+
+    def all_versions_after_reconcile(self):
+        return ('rev1a', )
+
+    def populated_parents(self):
+        return (((), 'rev1a'), )
+
+    def corrected_parents(self):
+        # Same as the populated parents, because there was nothing wrong.
+        return self.populated_parents()
+
+    def check_regexes(self, repo):
+        return ["0 unreferenced text versions"]
+
+    def populate_repository(self, repo):
+        # make rev1a: A well-formed revision, containing 'a-file'
+        inv = self.make_one_file_inventory(
+            repo, 'rev1a', [], root_revision='rev1a')
+        self.add_revision(repo, 'rev1a', inv, [])
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'rev1a'): True})
+        result.update({('a-file-id', 'rev1a'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'rev1a'):[NULL_REVISION]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'rev1a'):[NULL_REVISION]}
+
+
+class FileParentIsNotInRevisionAncestryScenario(BrokenRepoScenario):
+    """A scenario where a revision 'rev2' has 'a-file' with a
+    parent 'rev1b' that is not in the revision ancestry.
+
+    Reconcile should remove 'rev1b' from the parents list of 'a-file' in
+    'rev2', preserving 'rev1a' as a parent.
+    """
+
+    def all_versions_after_reconcile(self):
+        return ('rev1a', 'rev2')
+
+    def populated_parents(self):
+        return (
+            ((), 'rev1a'),
+            ((), 'rev1b'), # Will be gc'd
+            (('rev1a', 'rev1b'), 'rev2')) # Will have parents trimmed
+
+    def corrected_parents(self):
+        return (
+            ((), 'rev1a'),
+            (None, 'rev1b'),
+            (('rev1a',), 'rev2'))
+
+    def check_regexes(self, repo):
+        return [r"\* a-file-id version rev2 has parents \('rev1a', 'rev1b'\) "
+                r"but should have \('rev1a',\)",
+                "1 unreferenced text versions",
+                ]
+
+    def populate_repository(self, repo):
+        # make rev1a: A well-formed revision, containing 'a-file'
+        inv = self.make_one_file_inventory(
+            repo, 'rev1a', [], root_revision='rev1a')
+        self.add_revision(repo, 'rev1a', inv, [])
+
+        # make rev1b, which has no Revision, but has an Inventory, and
+        # a-file
+        inv = self.make_one_file_inventory(
+            repo, 'rev1b', [], root_revision='rev1b')
+        repo.add_inventory('rev1b', inv, [])
+
+        # make rev2, with a-file.
+        # a-file has 'rev1b' as an ancestor, even though this is not
+        # mentioned by 'rev1a', making it an unreferenced ancestor
+        inv = self.make_one_file_inventory(
+            repo, 'rev2', ['rev1a', 'rev1b'])
+        self.add_revision(repo, 'rev2', inv, ['rev1a'])
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'rev1a'): True,
+                           ('TREE_ROOT', 'rev2'): True})
+        result.update({('a-file-id', 'rev1a'): True,
+                       ('a-file-id', 'rev2'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'rev1a'):[NULL_REVISION],
+                ('a-file-id', 'rev2'):[('a-file-id', 'rev1a')]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'rev1a'):[NULL_REVISION],
+                ('TREE_ROOT', 'rev2'):[('TREE_ROOT', 'rev1a')]}
+
+
+class FileParentHasInaccessibleInventoryScenario(BrokenRepoScenario):
+    """A scenario where a revision 'rev3' containing 'a-file' modified in
+    'rev3', and with a parent which is in the revision ancestory, but whose
+    inventory cannot be accessed at all.
+
+    Reconcile should remove the file version parent whose inventory is
+    inaccessbile (i.e. remove 'rev1c' from the parents of a-file's rev3).
+    """
+
+    def all_versions_after_reconcile(self):
+        return ('rev2', 'rev3')
+
+    def populated_parents(self):
+        return (
+            ((), 'rev2'),
+            (('rev1c',), 'rev3'))
+
+    def corrected_parents(self):
+        return (
+            ((), 'rev2'),
+            ((), 'rev3'))
+
+    def check_regexes(self, repo):
+        return [r"\* a-file-id version rev3 has parents "
+                r"\('rev1c',\) but should have \(\)",
+                ]
+
+    def populate_repository(self, repo):
+        # make rev2, with a-file
+        # a-file is sane
+        inv = self.make_one_file_inventory(repo, 'rev2', [])
+        self.add_revision(repo, 'rev2', inv, [])
+
+        # make ghost revision rev1c, with a version of a-file present so
+        # that we generate a knit delta against this version.  In real life
+        # the ghost might never have been present or rev3 might have been
+        # generated against a revision that was present at the time.  So
+        # currently we have the full history of a-file present even though
+        # the inventory and revision objects are not.
+        self.make_one_file_inventory(repo, 'rev1c', [])
+
+        # make rev3 with a-file
+        # a-file refers to 'rev1c', which is a ghost in this repository, so
+        # a-file cannot have rev1c as its ancestor.
+        inv = self.make_one_file_inventory(repo, 'rev3', ['rev1c'])
+        self.add_revision(repo, 'rev3', inv, ['rev1c', 'rev1a'])
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'rev2'): True,
+                           ('TREE_ROOT', 'rev3'): True})
+        result.update({('a-file-id', 'rev2'): True,
+                       ('a-file-id', 'rev3'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'rev2'):[NULL_REVISION],
+                ('a-file-id', 'rev3'):[NULL_REVISION]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'rev2'):[NULL_REVISION],
+                ('TREE_ROOT', 'rev3'):[NULL_REVISION]}
+
+
+class FileParentsNotReferencedByAnyInventoryScenario(BrokenRepoScenario):
+    """A scenario where a repository with file 'a-file' which has extra
+    per-file versions that are not referenced by any inventory (even though
+    they have the same ID as actual revisions).  The inventory of 'rev2'
+    references 'rev1a' of 'a-file', but there is a 'rev2' of 'some-file' stored
+    and erroneously referenced by later per-file versions (revisions 'rev4' and
+    'rev5').
+
+    Reconcile should remove the file parents that are not referenced by any
+    inventory.
+    """
+
+    def all_versions_after_reconcile(self):
+        return ('rev1a', 'rev2c', 'rev4', 'rev5')
+
+    def populated_parents(self):
+        return [
+            (('rev1a',), 'rev2'),
+            (('rev1a',), 'rev2b'),
+            (('rev2',), 'rev3'),
+            (('rev2',), 'rev4'),
+            (('rev2', 'rev2c'), 'rev5')]
+
+    def corrected_parents(self):
+        return (
+            # rev2 and rev2b have been removed.
+            (None, 'rev2'),
+            (None, 'rev2b'),
+            # rev3's accessible parent inventories all have rev1a as the last
+            # modifier.
+            (('rev1a',), 'rev3'),
+            # rev1a features in both rev4's parents but should only appear once
+            # in the result
+            (('rev1a',), 'rev4'),
+            # rev2c is the head of rev1a and rev2c, the inventory provided
+            # per-file last-modified revisions.
+            (('rev2c',), 'rev5'))
+
+    def check_regexes(self, repo):
+        if repo.supports_rich_root():
+            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
+            # the expected count of errors.
+            count = 9
+        else:
+            count = 3
+        return [
+            # will be gc'd
+            r"unreferenced version: {rev2} in a-file-id",
+            r"unreferenced version: {rev2b} in a-file-id",
+            # will be corrected
+            r"a-file-id version rev3 has parents \('rev2',\) "
+            r"but should have \('rev1a',\)",
+            r"a-file-id version rev5 has parents \('rev2', 'rev2c'\) "
+            r"but should have \('rev2c',\)",
+            r"a-file-id version rev4 has parents \('rev2',\) "
+            r"but should have \('rev1a',\)",
+            "%d inconsistent parents" % count,
+            ]
+
+    def populate_repository(self, repo):
+        # make rev1a: A well-formed revision, containing 'a-file'
+        inv = self.make_one_file_inventory(
+            repo, 'rev1a', [], root_revision='rev1a')
+        self.add_revision(repo, 'rev1a', inv, [])
+
+        # make rev2, with a-file.
+        # a-file is unmodified from rev1a, and an unreferenced rev2 file
+        # version is present in the repository.
+        self.make_one_file_inventory(
+            repo, 'rev2', ['rev1a'], inv_revision='rev1a')
+        self.add_revision(repo, 'rev2', inv, ['rev1a'])
+
+        # make rev3 with a-file
+        # a-file has 'rev2' as its ancestor, but the revision in 'rev2' was
+        # rev1a so this is inconsistent with rev2's inventory - it should
+        # be rev1a, and at the revision level 1c is not present - it is a
+        # ghost, so only the details from rev1a are available for
+        # determining whether a delta is acceptable, or a full is needed,
+        # and what the correct parents are.
+        inv = self.make_one_file_inventory(repo, 'rev3', ['rev2'])
+        self.add_revision(repo, 'rev3', inv, ['rev1c', 'rev1a'])
+
+        # In rev2b, the true last-modifying-revision of a-file is rev1a,
+        # inherited from rev2, but there is a version rev2b of the file, which
+        # reconcile could remove, leaving no rev2b.  Most importantly,
+        # revisions descending from rev2b should not have per-file parents of
+        # a-file-rev2b.
+        # ??? This is to test deduplication in fixing rev4
+        inv = self.make_one_file_inventory(
+            repo, 'rev2b', ['rev1a'], inv_revision='rev1a')
+        self.add_revision(repo, 'rev2b', inv, ['rev1a'])
+
+        # rev4 is for testing that when the last modified of a file in
+        # multiple parent revisions is the same, that it only appears once
+        # in the generated per file parents list: rev2 and rev2b both
+        # descend from 1a and do not change the file a-file, so there should
+        # be no version of a-file 'rev2' or 'rev2b', but rev4 does change
+        # a-file, and is a merge of rev2 and rev2b, so it should end up with
+        # a parent of just rev1a - the starting file parents list is simply
+        # completely wrong.
+        inv = self.make_one_file_inventory(repo, 'rev4', ['rev2'])
+        self.add_revision(repo, 'rev4', inv, ['rev2', 'rev2b'])
+
+        # rev2c changes a-file from rev1a, so the version it of a-file it
+        # introduces is a head revision when rev5 is checked.
+        inv = self.make_one_file_inventory(repo, 'rev2c', ['rev1a'])
+        self.add_revision(repo, 'rev2c', inv, ['rev1a'])
+
+        # rev5 descends from rev2 and rev2c; as rev2 does not alter a-file,
+        # but rev2c does, this should use rev2c as the parent for the per
+        # file history, even though more than one per-file parent is
+        # available, because we use the heads of the revision parents for
+        # the inventory modification revisions of the file to determine the
+        # parents for the per file graph.
+        inv = self.make_one_file_inventory(repo, 'rev5', ['rev2', 'rev2c'])
+        self.add_revision(repo, 'rev5', inv, ['rev2', 'rev2c'])
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'rev1a'): True,
+                           ('TREE_ROOT', 'rev2'): True,
+                           ('TREE_ROOT', 'rev2b'): True,
+                           ('TREE_ROOT', 'rev2c'): True,
+                           ('TREE_ROOT', 'rev3'): True,
+                           ('TREE_ROOT', 'rev4'): True,
+                           ('TREE_ROOT', 'rev5'): True})
+        result.update({('a-file-id', 'rev1a'): True,
+                       ('a-file-id', 'rev2c'): True,
+                       ('a-file-id', 'rev3'): True,
+                       ('a-file-id', 'rev4'): True,
+                       ('a-file-id', 'rev5'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'rev1a'): [NULL_REVISION],
+                 ('a-file-id', 'rev2c'): [('a-file-id', 'rev1a')],
+                 ('a-file-id', 'rev3'): [('a-file-id', 'rev1a')],
+                 ('a-file-id', 'rev4'): [('a-file-id', 'rev1a')],
+                 ('a-file-id', 'rev5'): [('a-file-id', 'rev2c')]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'rev1a'): [NULL_REVISION],
+                ('TREE_ROOT', 'rev2'): [('TREE_ROOT', 'rev1a')],
+                ('TREE_ROOT', 'rev2b'): [('TREE_ROOT', 'rev1a')],
+                ('TREE_ROOT', 'rev2c'): [('TREE_ROOT', 'rev1a')],
+                ('TREE_ROOT', 'rev3'): [('TREE_ROOT', 'rev1a')],
+                ('TREE_ROOT', 'rev4'):
+                    [('TREE_ROOT', 'rev2'), ('TREE_ROOT', 'rev2b')],
+                ('TREE_ROOT', 'rev5'):
+                    [('TREE_ROOT', 'rev2'), ('TREE_ROOT', 'rev2c')]}
+
+
+class UnreferencedFileParentsFromNoOpMergeScenario(BrokenRepoScenario):
+    """
+    rev1a and rev1b with identical contents
+    rev2 revision has parents of [rev1a, rev1b]
+    There is a a-file:rev2 file version, not referenced by the inventory.
+    """
+
+    def all_versions_after_reconcile(self):
+        return ('rev1a', 'rev1b', 'rev2', 'rev4')
+
+    def populated_parents(self):
+        return (
+            ((), 'rev1a'),
+            ((), 'rev1b'),
+            (('rev1a', 'rev1b'), 'rev2'),
+            (None, 'rev3'),
+            (('rev2',), 'rev4'),
+            )
+
+    def corrected_parents(self):
+        return (
+            ((), 'rev1a'),
+            ((), 'rev1b'),
+            ((), 'rev2'),
+            (None, 'rev3'),
+            (('rev2',), 'rev4'),
+            )
+
+    def corrected_fulltexts(self):
+        return ['rev2']
+
+    def check_regexes(self, repo):
+        return []
+
+    def populate_repository(self, repo):
+        # make rev1a: A well-formed revision, containing 'a-file'
+        inv1a = self.make_one_file_inventory(
+            repo, 'rev1a', [], root_revision='rev1a')
+        self.add_revision(repo, 'rev1a', inv1a, [])
+
+        # make rev1b: A well-formed revision, containing 'a-file'
+        # rev1b of a-file has the exact same contents as rev1a.
+        file_contents = repo.revision_tree('rev1a').get_file_text('a-file-id')
+        inv = self.make_one_file_inventory(
+            repo, 'rev1b', [], root_revision='rev1b',
+            file_contents=file_contents)
+        self.add_revision(repo, 'rev1b', inv, [])
+
+        # make rev2, a merge of rev1a and rev1b, with a-file.
+        # a-file is unmodified from rev1a and rev1b, but a new version is
+        # wrongly present anyway.
+        inv = self.make_one_file_inventory(
+            repo, 'rev2', ['rev1a', 'rev1b'], inv_revision='rev1a',
+            file_contents=file_contents)
+        self.add_revision(repo, 'rev2', inv, ['rev1a', 'rev1b'])
+
+        # rev3: a-file unchanged from rev2, but wrongly referencing rev2 of the
+        # file in its inventory.
+        inv = self.make_one_file_inventory(
+            repo, 'rev3', ['rev2'], inv_revision='rev2',
+            file_contents=file_contents, make_file_version=False)
+        self.add_revision(repo, 'rev3', inv, ['rev2'])
+
+        # rev4: a modification of a-file on top of rev3.
+        inv = self.make_one_file_inventory(repo, 'rev4', ['rev2'])
+        self.add_revision(repo, 'rev4', inv, ['rev3'])
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'rev1a'): True,
+                           ('TREE_ROOT', 'rev1b'): True,
+                           ('TREE_ROOT', 'rev2'): True,
+                           ('TREE_ROOT', 'rev3'): True,
+                           ('TREE_ROOT', 'rev4'): True})
+        result.update({('a-file-id', 'rev1a'): True,
+                       ('a-file-id', 'rev1b'): True,
+                       ('a-file-id', 'rev2'): False,
+                       ('a-file-id', 'rev4'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'rev1a'): [NULL_REVISION],
+                ('a-file-id', 'rev1b'): [NULL_REVISION],
+                ('a-file-id', 'rev2'): [NULL_REVISION],
+                ('a-file-id', 'rev4'): [('a-file-id', 'rev2')]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'rev1a'): [NULL_REVISION],
+                ('TREE_ROOT', 'rev1b'): [NULL_REVISION],
+                ('TREE_ROOT', 'rev2'):
+                    [('TREE_ROOT', 'rev1a'), ('TREE_ROOT', 'rev1b')],
+                ('TREE_ROOT', 'rev3'): [('TREE_ROOT', 'rev2')],
+                ('TREE_ROOT', 'rev4'): [('TREE_ROOT', 'rev3')]}
+
+
+class TooManyParentsScenario(BrokenRepoScenario):
+    """A scenario where 'broken-revision' of 'a-file' claims to have parents
+    ['good-parent', 'bad-parent'].  However 'bad-parent' is in the ancestry of
+    'good-parent', so the correct parent list for that file version are is just
+    ['good-parent'].
+    """
+
+    def all_versions_after_reconcile(self):
+        return ('bad-parent', 'good-parent', 'broken-revision')
+
+    def populated_parents(self):
+        return (
+            ((), 'bad-parent'),
+            (('bad-parent',), 'good-parent'),
+            (('good-parent', 'bad-parent'), 'broken-revision'))
+
+    def corrected_parents(self):
+        return (
+            ((), 'bad-parent'),
+            (('bad-parent',), 'good-parent'),
+            (('good-parent',), 'broken-revision'))
+
+    def check_regexes(self, repo):
+        if repo.supports_rich_root():
+            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
+            # the expected count of errors.
+            count = 3
+        else:
+            count = 1
+        return (
+            '     %d inconsistent parents' % count,
+            (r"      \* a-file-id version broken-revision has parents "
+             r"\('good-parent', 'bad-parent'\) but "
+             r"should have \('good-parent',\)"))
+
+    def populate_repository(self, repo):
+        inv = self.make_one_file_inventory(
+            repo, 'bad-parent', (), root_revision='bad-parent')
+        self.add_revision(repo, 'bad-parent', inv, ())
+
+        inv = self.make_one_file_inventory(
+            repo, 'good-parent', ('bad-parent',))
+        self.add_revision(repo, 'good-parent', inv, ('bad-parent',))
+
+        inv = self.make_one_file_inventory(
+            repo, 'broken-revision', ('good-parent', 'bad-parent'))
+        self.add_revision(repo, 'broken-revision', inv, ('good-parent',))
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'bad-parent'): True,
+                           ('TREE_ROOT', 'broken-revision'): True,
+                           ('TREE_ROOT', 'good-parent'): True})
+        result.update({('a-file-id', 'bad-parent'): True,
+                       ('a-file-id', 'broken-revision'): True,
+                       ('a-file-id', 'good-parent'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'bad-parent'): [NULL_REVISION],
+                ('a-file-id', 'broken-revision'):
+                    [('a-file-id', 'good-parent')],
+                ('a-file-id', 'good-parent'): [('a-file-id', 'bad-parent')]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'bad-parent'): [NULL_REVISION],
+                ('TREE_ROOT', 'broken-revision'):
+                    [('TREE_ROOT', 'good-parent')],
+                ('TREE_ROOT', 'good-parent'): [('TREE_ROOT', 'bad-parent')]}
+
+
+class ClaimedFileParentDidNotModifyFileScenario(BrokenRepoScenario):
+    """A scenario where the file parent is the same as the revision parent, but
+    should not be because that revision did not modify the file.
+
+    Specifically, the parent revision of 'current' is
+    'modified-something-else', which does not modify 'a-file', but the
+    'current' version of 'a-file' erroneously claims that
+    'modified-something-else' is the parent file version.
+    """
+
+    def all_versions_after_reconcile(self):
+        return ('basis', 'current')
+
+    def populated_parents(self):
+        return (
+            ((), 'basis'),
+            (('basis',), 'modified-something-else'),
+            (('modified-something-else',), 'current'))
+
+    def corrected_parents(self):
+        return (
+            ((), 'basis'),
+            (None, 'modified-something-else'),
+            (('basis',), 'current'))
+
+    def check_regexes(self, repo):
+        if repo.supports_rich_root():
+            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
+            # the expected count of errors.
+            count = 3
+        else:
+            count = 1
+        return (
+            "%d inconsistent parents" % count,
+            r"\* a-file-id version current has parents "
+            r"\('modified-something-else',\) but should have \('basis',\)",
+            )
+
+    def populate_repository(self, repo):
+        inv = self.make_one_file_inventory(repo, 'basis', ())
+        self.add_revision(repo, 'basis', inv, ())
+
+        # 'modified-something-else' is a correctly recorded revision, but it
+        # does not modify the file we are looking at, so the inventory for that
+        # file in this revision points to 'basis'.
+        inv = self.make_one_file_inventory(
+            repo, 'modified-something-else', ('basis',), inv_revision='basis')
+        self.add_revision(repo, 'modified-something-else', inv, ('basis',))
+
+        # The 'current' revision has 'modified-something-else' as its parent,
+        # but the 'current' version of 'a-file' should have 'basis' as its
+        # parent.
+        inv = self.make_one_file_inventory(
+            repo, 'current', ('modified-something-else',))
+        self.add_revision(repo, 'current', inv, ('modified-something-else',))
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'basis'): True,
+                           ('TREE_ROOT', 'current'): True,
+                           ('TREE_ROOT', 'modified-something-else'): True})
+        result.update({('a-file-id', 'basis'): True,
+                       ('a-file-id', 'current'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'basis'): [NULL_REVISION],
+                ('a-file-id', 'current'): [('a-file-id', 'basis')]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'basis'): ['null:'],
+                ('TREE_ROOT', 'current'):
+                    [('TREE_ROOT', 'modified-something-else')],
+                ('TREE_ROOT', 'modified-something-else'):
+                    [('TREE_ROOT', 'basis')]}
+
+
+class IncorrectlyOrderedParentsScenario(BrokenRepoScenario):
+    """A scenario where the set parents of a version of a file are correct, but
+    the order of those parents is incorrect.
+
+    This defines a 'broken-revision-1-2' and a 'broken-revision-2-1' which both
+    have their file version parents reversed compared to the revision parents,
+    which is invalid.  (We use two revisions with opposite orderings of the
+    same parents to make sure that accidentally relying on dictionary/set
+    ordering cannot make the test pass; the assumption is that while dict/set
+    iteration order is arbitrary, it is also consistent within a single test).
+    """
+
+    def all_versions_after_reconcile(self):
+        return ['parent-1', 'parent-2', 'broken-revision-1-2',
+                'broken-revision-2-1']
+
+    def populated_parents(self):
+        return (
+            ((), 'parent-1'),
+            ((), 'parent-2'),
+            (('parent-2', 'parent-1'), 'broken-revision-1-2'),
+            (('parent-1', 'parent-2'), 'broken-revision-2-1'))
+
+    def corrected_parents(self):
+        return (
+            ((), 'parent-1'),
+            ((), 'parent-2'),
+            (('parent-1', 'parent-2'), 'broken-revision-1-2'),
+            (('parent-2', 'parent-1'), 'broken-revision-2-1'))
+
+    def check_regexes(self, repo):
+        if repo.supports_rich_root():
+            # TREE_ROOT will be wrong; but we're not testing it. so just adjust
+            # the expected count of errors.
+            count = 4
+        else:
+            count = 2
+        return (
+            "%d inconsistent parents" % count,
+            r"\* a-file-id version broken-revision-1-2 has parents "
+            r"\('parent-2', 'parent-1'\) but should have "
+            r"\('parent-1', 'parent-2'\)",
+            r"\* a-file-id version broken-revision-2-1 has parents "
+            r"\('parent-1', 'parent-2'\) but should have "
+            r"\('parent-2', 'parent-1'\)")
+
+    def populate_repository(self, repo):
+        inv = self.make_one_file_inventory(repo, 'parent-1', [])
+        self.add_revision(repo, 'parent-1', inv, [])
+
+        inv = self.make_one_file_inventory(repo, 'parent-2', [])
+        self.add_revision(repo, 'parent-2', inv, [])
+
+        inv = self.make_one_file_inventory(
+            repo, 'broken-revision-1-2', ['parent-2', 'parent-1'])
+        self.add_revision(
+            repo, 'broken-revision-1-2', inv, ['parent-1', 'parent-2'])
+
+        inv = self.make_one_file_inventory(
+            repo, 'broken-revision-2-1', ['parent-1', 'parent-2'])
+        self.add_revision(
+            repo, 'broken-revision-2-1', inv, ['parent-2', 'parent-1'])
+        self.versioned_root = repo.supports_rich_root()
+
+    def repository_text_key_references(self):
+        result = {}
+        if self.versioned_root:
+            result.update({('TREE_ROOT', 'broken-revision-1-2'): True,
+                           ('TREE_ROOT', 'broken-revision-2-1'): True,
+                           ('TREE_ROOT', 'parent-1'): True,
+                           ('TREE_ROOT', 'parent-2'): True})
+        result.update({('a-file-id', 'broken-revision-1-2'): True,
+                       ('a-file-id', 'broken-revision-2-1'): True,
+                       ('a-file-id', 'parent-1'): True,
+                       ('a-file-id', 'parent-2'): True})
+        return result
+
+    def repository_text_keys(self):
+        return {('a-file-id', 'broken-revision-1-2'):
+                    [('a-file-id', 'parent-1'), ('a-file-id', 'parent-2')],
+                ('a-file-id', 'broken-revision-2-1'):
+                    [('a-file-id', 'parent-2'), ('a-file-id', 'parent-1')],
+                ('a-file-id', 'parent-1'): [NULL_REVISION],
+                ('a-file-id', 'parent-2'): [NULL_REVISION]}
+
+    def versioned_repository_text_keys(self):
+        return {('TREE_ROOT', 'broken-revision-1-2'):
+                    [('TREE_ROOT', 'parent-1'), ('TREE_ROOT', 'parent-2')],
+                ('TREE_ROOT', 'broken-revision-2-1'):
+                    [('TREE_ROOT', 'parent-2'), ('TREE_ROOT', 'parent-1')],
+                ('TREE_ROOT', 'parent-1'): [NULL_REVISION],
+                ('TREE_ROOT', 'parent-2'): [NULL_REVISION]}
+
+
+all_broken_scenario_classes = [
+    UndamagedRepositoryScenario,
+    FileParentIsNotInRevisionAncestryScenario,
+    FileParentHasInaccessibleInventoryScenario,
+    FileParentsNotReferencedByAnyInventoryScenario,
+    TooManyParentsScenario,
+    ClaimedFileParentDidNotModifyFileScenario,
+    IncorrectlyOrderedParentsScenario,
+    UnreferencedFileParentsFromNoOpMergeScenario,
+    ]
+
+
+def broken_scenarios_for_all_formats():
+    format_scenarios = all_repository_vf_format_scenarios()
+    # test_check_reconcile needs to be parameterized by format *and* by broken
+    # repository scenario.
+    broken_scenarios = [(s.__name__, {'scenario_class': s})
+                        for s in all_broken_scenario_classes]
+    return multiply_scenarios(format_scenarios, broken_scenarios)
 
 
 class TestFileParentReconciliation(TestCaseWithRepository):
     """Tests for how reconcile corrects errors in parents of file versions."""
 
+    scenarios = broken_scenarios_for_all_formats()
+
     def make_populated_repository(self, factory):
         """Create a new repository populated by the given factory."""
         repo = self.make_repository('broken-repo')
@@ -97,7 +855,7 @@
         entry.text_size = 0
         if file_contents is None:
             file_contents = '%sline\n' % entry.revision
-        entry.text_sha1 = osutils.sha(file_contents).hexdigest()
+        entry.text_sha1 = osutils.sha_string(file_contents)
         inv.add(entry)
         if make_file_version:
             repo.texts.add_lines((file_id, revision),

=== added file 'bzrlib/tests/per_repository_vf/test_fetch.py'
--- bzrlib/tests/per_repository_vf/test_fetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_fetch.py	2011-05-02 22:44:40 +0000
@@ -0,0 +1,57 @@
+# Copyright (C) 2007-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for fetch between repositories of the same type."""
+
+from bzrlib import (
+    graph,
+    )
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+load_tests = load_tests_apply_scenarios
+
+
+class TestSource(TestCaseWithRepository):
+    """Tests for/about the results of Repository._get_source."""
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test_no_absent_records_in_stream_with_ghosts(self):
+        # XXX: Arguably should be in per_interrepository but
+        # doesn't actually gain coverage there; need a specific set of
+        # permutations to cover it.
+        # bug lp:376255 was reported about this.
+        builder = self.make_branch_builder('repo')
+        builder.start_series()
+        builder.build_snapshot('tip', ['ghost'],
+            [('add', ('', 'ROOT_ID', 'directory', ''))],
+            allow_leftmost_as_ghost=True)
+        builder.finish_series()
+        b = builder.get_branch()
+        b.lock_read()
+        self.addCleanup(b.unlock)
+        repo = b.repository
+        source = repo._get_source(repo._format)
+        search = graph.PendingAncestryResult(['tip'], repo)
+        stream = source.get_stream(search)
+        for substream_type, substream in stream:
+            for record in substream:
+                self.assertNotEqual('absent', record.storage_kind,
+                    "Absent record for %s" % (((substream_type,) + record.key),))

=== added file 'bzrlib/tests/per_repository_vf/test_fileid_involved.py'
--- bzrlib/tests/per_repository_vf/test_fileid_involved.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_fileid_involved.py	2011-10-06 00:14:01 +0000
@@ -0,0 +1,432 @@
+# Copyright (C) 2005-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import sys
+import time
+
+from bzrlib import (
+    errors,
+    inventory,
+    remote,
+    revision as _mod_revision,
+    tests,
+    transform,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class FileIdInvolvedWGhosts(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def create_branch_with_ghost_text(self):
+        builder = self.make_branch_builder('ghost')
+        builder.build_snapshot('A-id', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('a', 'a-file-id', 'file', 'some content\n'))])
+        b = builder.get_branch()
+        old_rt = b.repository.revision_tree('A-id')
+        new_inv = inventory.mutable_inventory_from_tree(old_rt)
+        new_inv.revision_id = 'B-id'
+        new_inv['a-file-id'].revision = 'ghost-id'
+        new_rev = _mod_revision.Revision('B-id',
+            timestamp=time.time(),
+            timezone=0,
+            message='Committing against a ghost',
+            committer='Joe Foo <joe@foo.com>',
+            properties={},
+            parent_ids=('A-id', 'ghost-id'),
+            )
+        b.lock_write()
+        self.addCleanup(b.unlock)
+        b.repository.start_write_group()
+        b.repository.add_revision('B-id', new_rev, new_inv)
+        self.disable_commit_write_group_paranoia(b.repository)
+        b.repository.commit_write_group()
+        return b
+
+    def disable_commit_write_group_paranoia(self, repo):
+        if isinstance(repo, remote.RemoteRepository):
+            # We can't easily disable the checks in a remote repo.
+            repo.abort_write_group()
+            raise tests.TestSkipped(
+                "repository format does not support storing revisions with "
+                "missing texts.")
+        pack_coll = getattr(repo, '_pack_collection', None)
+        if pack_coll is not None:
+            # Monkey-patch the pack collection instance to allow storing
+            # incomplete revisions.
+            pack_coll._check_new_inventories = lambda: []
+
+    def test_file_ids_include_ghosts(self):
+        b = self.create_branch_with_ghost_text()
+        repo = b.repository
+        self.assertEqual(
+            {'a-file-id':set(['ghost-id'])},
+            repo.fileids_altered_by_revision_ids(['B-id']))
+
+    def test_file_ids_uses_fallbacks(self):
+        builder = self.make_branch_builder('source',
+                                           format=self.bzrdir_format)
+        repo = builder.get_branch().repository
+        if not repo._format.supports_external_lookups:
+            raise tests.TestNotApplicable('format does not support stacking')
+        builder.start_series()
+        builder.build_snapshot('A-id', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('file', 'file-id', 'file', 'contents\n'))])
+        builder.build_snapshot('B-id', ['A-id'], [
+            ('modify', ('file-id', 'new-content\n'))])
+        builder.build_snapshot('C-id', ['B-id'], [
+            ('modify', ('file-id', 'yet more content\n'))])
+        builder.finish_series()
+        source_b = builder.get_branch()
+        source_b.lock_read()
+        self.addCleanup(source_b.unlock)
+        base = self.make_branch('base')
+        base.pull(source_b, stop_revision='B-id')
+        stacked = self.make_branch('stacked')
+        stacked.set_stacked_on_url('../base')
+        stacked.pull(source_b, stop_revision='C-id')
+
+        stacked.lock_read()
+        self.addCleanup(stacked.unlock)
+        repo = stacked.repository
+        keys = {'file-id': set(['A-id'])}
+        if stacked.repository.supports_rich_root():
+            keys['root-id'] = set(['A-id'])
+        self.assertEqual(keys, repo.fileids_altered_by_revision_ids(['A-id']))
+
+
+class FileIdInvolvedBase(TestCaseWithRepository):
+
+    def touch(self, tree, filename):
+        # use the trees transport to not depend on the tree's location or type.
+        tree.bzrdir.root_transport.append_bytes(filename, "appended line\n")
+
+    def compare_tree_fileids(self, branch, old_rev, new_rev):
+        old_tree = self.branch.repository.revision_tree(old_rev)
+        new_tree = self.branch.repository.revision_tree(new_rev)
+        delta = new_tree.changes_from(old_tree)
+
+        l2 = [id for path, id, kind in delta.added] + \
+             [id for oldpath, newpath, id, kind, text_modified, \
+                meta_modified in delta.renamed] + \
+             [id for path, id, kind, text_modified, meta_modified in \
+                delta.modified]
+        return set(l2)
+
+
+class TestFileIdInvolved(FileIdInvolvedBase):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def setUp(self):
+        super(TestFileIdInvolved, self).setUp()
+        # create three branches, and merge it
+        #
+        #          ,-->J------>K                (branch2)
+        #         /             \
+        #  A --->B --->C---->D-->G              (main)
+        #  \          /     /
+        #   '--->E---+---->F                    (branch1)
+
+        # A changes:
+        # B changes: 'a-file-id-2006-01-01-abcd'
+        # C changes:  Nothing (perfect merge)
+        # D changes: 'b-file-id-2006-01-01-defg'
+        # E changes: 'file-d'
+        # F changes: 'file-d'
+        # G changes: 'b-file-id-2006-01-01-defg'
+        # J changes: 'b-file-id-2006-01-01-defg'
+        # K changes: 'c-funky<file-id>quiji%bo'
+
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a","main/b","main/c"])
+
+        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
+                                 'b-file-id-2006-01-01-defg',
+                                 'c-funky<file-id>quiji%bo'])
+        try:
+            main_wt.commit("Commit one", rev_id="rev-A")
+        except errors.IllegalPath:
+            # TODO: jam 20060701 Consider raising a different exception
+            #       newer formats do support this, and nothin can done to
+            #       correct this test - its not a bug.
+            if sys.platform == 'win32':
+                raise tests.TestSkipped('Old repository formats do not'
+                                        ' support file ids with <> on win32')
+            # This is not a known error condition
+            raise
+
+        #-------- end A -----------
+
+        bt1 = self.make_branch_and_tree('branch1')
+        bt1.pull(main_branch)
+        b1 = bt1.branch
+        self.build_tree(["branch1/d"])
+        bt1.add(['d'], ['file-d'])
+        bt1.commit("branch1, Commit one", rev_id="rev-E")
+
+        #-------- end E -----------
+
+        self.touch(main_wt, "a")
+        main_wt.commit("Commit two", rev_id="rev-B")
+
+        #-------- end B -----------
+
+        bt2 = self.make_branch_and_tree('branch2')
+        bt2.pull(main_branch)
+        branch2_branch = bt2.branch
+        set_executability(bt2, 'b', True)
+        bt2.commit("branch2, Commit one", rev_id="rev-J")
+
+        #-------- end J -----------
+
+        main_wt.merge_from_branch(b1)
+        main_wt.commit("merge branch1, rev-11", rev_id="rev-C")
+
+        #-------- end C -----------
+
+        bt1.rename_one("d","e")
+        bt1.commit("branch1, commit two", rev_id="rev-F")
+
+        #-------- end F -----------
+
+        self.touch(bt2, "c")
+        bt2.commit("branch2, commit two", rev_id="rev-K")
+
+        #-------- end K -----------
+
+        main_wt.merge_from_branch(b1)
+        self.touch(main_wt, "b")
+        # D gets some funky characters to make sure the unescaping works
+        main_wt.commit("merge branch1, rev-12", rev_id="rev-<D>")
+
+        # end D
+
+        main_wt.merge_from_branch(branch2_branch)
+        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
+
+        # end G
+        self.branch = main_branch
+
+    def test_fileids_altered_between_two_revs(self):
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"])
+        self.assertEqual(
+            {'b-file-id-2006-01-01-defg':set(['rev-J']),
+             'c-funky<file-id>quiji%bo':set(['rev-K'])
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"]))
+
+        self.assertEqual(
+            {'b-file-id-2006-01-01-defg': set(['rev-<D>']),
+             'file-d': set(['rev-F']),
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(['rev-<D>', 'rev-F']))
+
+        self.assertEqual(
+            {
+             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
+             'c-funky<file-id>quiji%bo': set(['rev-K']),
+             'file-d': set(['rev-F']),
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(
+                ['rev-<D>', 'rev-G', 'rev-F', 'rev-K', 'rev-J']))
+
+        self.assertEqual(
+            {'a-file-id-2006-01-01-abcd': set(['rev-B']),
+             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
+             'c-funky<file-id>quiji%bo': set(['rev-K']),
+             'file-d': set(['rev-F']),
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(
+                ['rev-G', 'rev-F', 'rev-C', 'rev-B', 'rev-<D>', 'rev-K', 'rev-J']))
+
+    def fileids_altered_by_revision_ids(self, revision_ids):
+        """This is a wrapper to strip TREE_ROOT if it occurs"""
+        repo = self.branch.repository
+        root_id = self.branch.basis_tree().get_root_id()
+        result = repo.fileids_altered_by_revision_ids(revision_ids)
+        if root_id in result:
+            del result[root_id]
+        return result
+
+    def test_fileids_altered_by_revision_ids(self):
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        self.assertEqual(
+            {'a-file-id-2006-01-01-abcd':set(['rev-A']),
+             'b-file-id-2006-01-01-defg': set(['rev-A']),
+             'c-funky<file-id>quiji%bo': set(['rev-A']),
+             },
+            self.fileids_altered_by_revision_ids(["rev-A"]))
+        self.assertEqual(
+            {'a-file-id-2006-01-01-abcd':set(['rev-B'])
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(["rev-B"]))
+        self.assertEqual(
+            {'b-file-id-2006-01-01-defg':set(['rev-<D>'])
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(["rev-<D>"]))
+
+    def test_fileids_involved_full_compare(self):
+        # this tests that the result of each fileid_involved calculation
+        # along a revision history selects only the fileids selected by
+        # comparing the trees - no less, and no more. This is correct
+        # because in our sample data we do not revert any file ids along
+        # the revision history.
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        pp=[]
+        graph = self.branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(self.branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+        history.reverse()
+
+        if len(history) < 2:
+            return
+
+        for start in range(0,len(history)-1):
+            start_id = history[start]
+            for end in range(start+1,len(history)):
+                end_id = history[end]
+                unique_revs = graph.find_unique_ancestors(end_id, [start_id])
+                l1 = self.branch.repository.fileids_altered_by_revision_ids(
+                    unique_revs)
+                l1 = set(l1.keys())
+                l2 = self.compare_tree_fileids(self.branch, start_id, end_id)
+                self.assertEquals(l1, l2)
+
+
+class TestFileIdInvolvedNonAscii(FileIdInvolvedBase):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test_utf8_file_ids_and_revision_ids(self):
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a"])
+
+        file_id = u'a-f\xedle-id'.encode('utf8')
+        main_wt.add(['a'], [file_id])
+        revision_id = u'r\xe9v-a'.encode('utf8')
+        try:
+            main_wt.commit('a', rev_id=revision_id)
+        except errors.NonAsciiRevisionId:
+            raise tests.TestSkipped('non-ascii revision ids not supported by %s'
+                                    % self.repository_format)
+
+        repo = main_wt.branch.repository
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        file_ids = repo.fileids_altered_by_revision_ids([revision_id])
+        root_id = main_wt.basis_tree().get_root_id()
+        if root_id in file_ids:
+            self.assertEqual({file_id:set([revision_id]),
+                              root_id:set([revision_id])
+                             }, file_ids)
+        else:
+            self.assertEqual({file_id:set([revision_id])}, file_ids)
+
+
+class TestFileIdInvolvedSuperset(FileIdInvolvedBase):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def setUp(self):
+        super(TestFileIdInvolvedSuperset, self).setUp()
+
+        self.branch = None
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a","main/b","main/c"])
+
+        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
+                                 'b-file-id-2006-01-01-defg',
+                                 'c-funky<file-id>quiji\'"%bo'])
+        try:
+            main_wt.commit("Commit one", rev_id="rev-A")
+        except errors.IllegalPath:
+            # TODO: jam 20060701 Consider raising a different exception
+            #       newer formats do support this, and nothin can done to
+            #       correct this test - its not a bug.
+            if sys.platform == 'win32':
+                raise tests.TestSkipped('Old repository formats do not'
+                                        ' support file ids with <> on win32')
+            # This is not a known error condition
+            raise
+
+        branch2_wt = self.make_branch_and_tree('branch2')
+        branch2_wt.pull(main_branch)
+        branch2_bzrdir = branch2_wt.bzrdir
+        branch2_branch = branch2_bzrdir.open_branch()
+        set_executability(branch2_wt, 'b', True)
+        branch2_wt.commit("branch2, Commit one", rev_id="rev-J")
+
+        main_wt.merge_from_branch(branch2_branch)
+        set_executability(main_wt, 'b', False)
+        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
+
+        # end G
+        self.branch = main_branch
+
+    def test_fileid_involved_full_compare2(self):
+        # this tests that fileids_altered_by_revision_ids returns
+        # more information than compare_tree can, because it
+        # sees each change rather than the aggregate delta.
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        graph = self.branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(self.branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+        history.reverse()
+        old_rev = history[0]
+        new_rev = history[1]
+        unique_revs = graph.find_unique_ancestors(new_rev, [old_rev])
+
+        l1 = self.branch.repository.fileids_altered_by_revision_ids(
+            unique_revs)
+        l1 = set(l1.keys())
+
+        l2 = self.compare_tree_fileids(self.branch, old_rev, new_rev)
+        self.assertNotEqual(l2, l1)
+        self.assertSubset(l2, l1)
+
+
+def set_executability(wt, path, executable=True):
+    """Set the executable bit for the file at path in the working tree
+
+    os.chmod() doesn't work on windows. But TreeTransform can mark or
+    unmark a file as executable.
+    """
+    file_id = wt.path2id(path)
+    tt = transform.TreeTransform(wt)
+    try:
+        tt.set_executability(executable, tt.trans_id_tree_file_id(file_id))
+        tt.apply()
+    finally:
+        tt.finalize()

=== renamed file 'bzrlib/tests/per_repository/test_find_text_key_references.py' => 'bzrlib/tests/per_repository_vf/test_find_text_key_references.py'
--- bzrlib/tests/per_repository/test_find_text_key_references.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository_vf/test_find_text_key_references.py	2011-05-13 10:22:42 +0000
@@ -18,11 +18,19 @@
 """Test the find_text_key_references API."""
 
 
-from bzrlib.tests.per_repository import TestCaseWithRepository
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestFindTextKeyReferences(TestCaseWithRepository):
 
+    scenarios = all_repository_vf_format_scenarios()
+
     def test_empty(self):
         repo = self.make_repository('.')
         repo.lock_read()

=== renamed file 'bzrlib/tests/per_repository/test_merge_directive.py' => 'bzrlib/tests/per_repository_vf/test_merge_directive.py'
--- bzrlib/tests/per_repository/test_merge_directive.py	2009-07-22 17:22:06 +0000
+++ bzrlib/tests/per_repository_vf/test_merge_directive.py	2011-09-09 12:19:12 +0000
@@ -22,16 +22,22 @@
 
 from bzrlib import (
     chk_map,
-    errors,
     merge_directive,
-    tests,
-    )
-
-from bzrlib.tests.per_repository import TestCaseWithRepository
+    )
+
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestMergeDirective(TestCaseWithRepository):
 
+    scenarios = all_repository_vf_format_scenarios()
+
     def make_two_branches(self):
         builder = self.make_branch_builder('source')
         builder.start_series()

=== added file 'bzrlib/tests/per_repository_vf/test_reconcile.py'
--- bzrlib/tests/per_repository_vf/test_reconcile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_reconcile.py	2011-10-01 11:36:04 +0000
@@ -0,0 +1,439 @@
+# Copyright (C) 2006-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for reconciliation of repositories."""
+
+import bzrlib
+from bzrlib import errors
+from bzrlib.inventory import Inventory
+from bzrlib.reconcile import reconcile, Reconciler
+from bzrlib.revision import Revision
+from bzrlib.tests import TestSkipped
+
+from bzrlib.tests.per_repository_vf.helpers import (
+    TestCaseWithBrokenRevisionIndex,
+    )
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.matchers import MatchesAncestry
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.uncommit import uncommit
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class TestReconcile(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def checkUnreconciled(self, d, reconciler):
+        """Check that d did not get reconciled."""
+        # nothing should have been fixed yet:
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        self.checkNoBackupInventory(d)
+
+    def checkNoBackupInventory(self, aBzrDir):
+        """Check that there is no backup inventory in aBzrDir."""
+        repo = aBzrDir.open_repository()
+        for path in repo.control_transport.list_dir('.'):
+            self.assertFalse('inventory.backup' in path)
+
+
+class TestBadRevisionParents(TestCaseWithBrokenRevisionIndex):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test_aborts_if_bad_parents_in_index(self):
+        """Reconcile refuses to proceed if the revision index is wrong when
+        checked against the revision texts, so that it does not generate broken
+        data.
+
+        Ideally reconcile would fix this, but until we implement that we just
+        make sure we safely detect this problem.
+        """
+        repo = self.make_repo_with_extra_ghost_index()
+        reconciler = repo.reconcile(thorough=True)
+        self.assertTrue(reconciler.aborted,
+            "reconcile should have aborted due to bad parents.")
+
+    def test_does_not_abort_on_clean_repo(self):
+        repo = self.make_repository('.')
+        reconciler = repo.reconcile(thorough=True)
+        self.assertFalse(reconciler.aborted,
+            "reconcile should not have aborted on an unbroken repository.")
+
+
+class TestsNeedingReweave(TestReconcile):
+
+    def setUp(self):
+        super(TestsNeedingReweave, self).setUp()
+
+        t = self.get_transport()
+        # an empty inventory with no revision for testing with.
+        repo = self.make_repository('inventory_without_revision')
+        repo.lock_write()
+        repo.start_write_group()
+        inv = Inventory(revision_id='missing')
+        inv.root.revision = 'missing'
+        repo.add_inventory('missing', inv, [])
+        repo.commit_write_group()
+        repo.unlock()
+
+        def add_commit(repo, revision_id, parent_ids):
+            repo.lock_write()
+            repo.start_write_group()
+            inv = Inventory(revision_id=revision_id)
+            inv.root.revision = revision_id
+            root_id = inv.root.file_id
+            sha1 = repo.add_inventory(revision_id, inv, parent_ids)
+            repo.texts.add_lines((root_id, revision_id), [], [])
+            rev = bzrlib.revision.Revision(timestamp=0,
+                                           timezone=None,
+                                           committer="Foo Bar <foo@example.com>",
+                                           message="Message",
+                                           inventory_sha1=sha1,
+                                           revision_id=revision_id)
+            rev.parent_ids = parent_ids
+            repo.add_revision(revision_id, rev)
+            repo.commit_write_group()
+            repo.unlock()
+        # an empty inventory with no revision for testing with.
+        # this is referenced by 'references_missing' to let us test
+        # that all the cached data is correctly converted into ghost links
+        # and the referenced inventory still cleaned.
+        repo = self.make_repository('inventory_without_revision_and_ghost')
+        repo.lock_write()
+        repo.start_write_group()
+        repo.add_inventory('missing', inv, [])
+        repo.commit_write_group()
+        repo.unlock()
+        add_commit(repo, 'references_missing', ['missing'])
+
+        # a inventory with no parents and the revision has parents..
+        # i.e. a ghost.
+        repo = self.make_repository('inventory_one_ghost')
+        add_commit(repo, 'ghost', ['the_ghost'])
+
+        # a inventory with a ghost that can be corrected now.
+        t.copy_tree('inventory_one_ghost', 'inventory_ghost_present')
+        bzrdir_url = self.get_url('inventory_ghost_present')
+        bzrdir = bzrlib.bzrdir.BzrDir.open(bzrdir_url)
+        repo = bzrdir.open_repository()
+        add_commit(repo, 'the_ghost', [])
+
+    def checkEmptyReconcile(self, **kwargs):
+        """Check a reconcile on an empty repository."""
+        self.make_repository('empty')
+        d = bzrlib.bzrdir.BzrDir.open(self.get_url('empty'))
+        # calling on a empty repository should do nothing
+        reconciler = d.find_repository().reconcile(**kwargs)
+        # no inconsistent parents should have been found
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # and no backup weave should have been needed/made.
+        self.checkNoBackupInventory(d)
+
+    def test_reconcile_empty(self):
+        # in an empty repo, theres nothing to do.
+        self.checkEmptyReconcile()
+
+    def test_repo_has_reconcile_does_inventory_gc_attribute(self):
+        repo = self.make_repository('repo')
+        self.assertNotEqual(None, repo._reconcile_does_inventory_gc)
+
+    def test_reconcile_empty_thorough(self):
+        # reconcile should accept thorough=True
+        self.checkEmptyReconcile(thorough=True)
+
+    def test_convenience_reconcile_inventory_without_revision_reconcile(self):
+        # smoke test for the all in one ui tool
+        bzrdir_url = self.get_url('inventory_without_revision')
+        bzrdir = bzrlib.bzrdir.BzrDir.open(bzrdir_url)
+        repo = bzrdir.open_repository()
+        if not repo._reconcile_does_inventory_gc:
+            raise TestSkipped('Irrelevant test')
+        reconcile(bzrdir)
+        # now the backup should have it but not the current inventory
+        repo = bzrdir.open_repository()
+        self.check_missing_was_removed(repo)
+
+    def test_reweave_inventory_without_revision(self):
+        # an excess inventory on its own is only reconciled by using thorough
+        d_url = self.get_url('inventory_without_revision')
+        d = bzrlib.bzrdir.BzrDir.open(d_url)
+        repo = d.open_repository()
+        if not repo._reconcile_does_inventory_gc:
+            raise TestSkipped('Irrelevant test')
+        self.checkUnreconciled(d, repo.reconcile())
+        reconciler = repo.reconcile(thorough=True)
+        # no bad parents
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and one garbage inventory
+        self.assertEqual(1, reconciler.garbage_inventories)
+        self.check_missing_was_removed(repo)
+
+    def check_thorough_reweave_missing_revision(self, aBzrDir, reconcile,
+            **kwargs):
+        # actual low level test.
+        repo = aBzrDir.open_repository()
+        if not repo.has_revision('missing'):
+            # the repo handles ghosts without corruption, so reconcile has
+            # nothing to do here. Specifically, this test has the inventory
+            # 'missing' present and the revision 'missing' missing, so clearly
+            # 'missing' cannot be reported in the present ancestry -> missing
+            # is something that can be filled as a ghost.
+            expected_inconsistent_parents = 0
+        else:
+            expected_inconsistent_parents = 1
+        reconciler = reconcile(**kwargs)
+        # some number of inconsistent parents should have been found
+        self.assertEqual(expected_inconsistent_parents,
+                         reconciler.inconsistent_parents)
+        # and one garbage inventories
+        self.assertEqual(1, reconciler.garbage_inventories)
+        # now the backup should have it but not the current inventory
+        repo = aBzrDir.open_repository()
+        self.check_missing_was_removed(repo)
+        # and the parent list for 'references_missing' should have that
+        # revision a ghost now.
+        self.assertFalse(repo.has_revision('missing'))
+
+    def check_missing_was_removed(self, repo):
+        if repo._reconcile_backsup_inventory:
+            backed_up = False
+            for path in repo.control_transport.list_dir('.'):
+                if 'inventory.backup' in path:
+                    backed_up = True
+            self.assertTrue(backed_up)
+            # Not clear how to do this at an interface level:
+            # self.assertTrue('missing' in backup.versions())
+        self.assertRaises(errors.NoSuchRevision, repo.get_inventory, 'missing')
+
+    def test_reweave_inventory_without_revision_reconciler(self):
+        # smoke test for the all in one Reconciler class,
+        # other tests use the lower level repo.reconcile()
+        d_url = self.get_url('inventory_without_revision_and_ghost')
+        d = bzrlib.bzrdir.BzrDir.open(d_url)
+        if not d.open_repository()._reconcile_does_inventory_gc:
+            raise TestSkipped('Irrelevant test')
+        def reconcile():
+            reconciler = Reconciler(d)
+            reconciler.reconcile()
+            return reconciler
+        self.check_thorough_reweave_missing_revision(d, reconcile)
+
+    def test_reweave_inventory_without_revision_and_ghost(self):
+        # actual low level test.
+        d_url = self.get_url('inventory_without_revision_and_ghost')
+        d = bzrlib.bzrdir.BzrDir.open(d_url)
+        repo = d.open_repository()
+        if not repo._reconcile_does_inventory_gc:
+            raise TestSkipped('Irrelevant test')
+        # nothing should have been altered yet : inventories without
+        # revisions are not data loss incurring for current format
+        self.check_thorough_reweave_missing_revision(d, repo.reconcile,
+            thorough=True)
+
+    def test_reweave_inventory_preserves_a_revision_with_ghosts(self):
+        d = bzrlib.bzrdir.BzrDir.open(self.get_url('inventory_one_ghost'))
+        reconciler = d.open_repository().reconcile(thorough=True)
+        # no inconsistent parents should have been found:
+        # the lack of a parent for ghost is normal
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and one garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # now the current inventory should still have 'ghost'
+        repo = d.open_repository()
+        repo.get_inventory('ghost')
+        self.assertThat(['ghost', 'the_ghost'], MatchesAncestry(repo, 'ghost'))
+
+    def test_reweave_inventory_fixes_ancestryfor_a_present_ghost(self):
+        d = bzrlib.bzrdir.BzrDir.open(self.get_url('inventory_ghost_present'))
+        repo = d.open_repository()
+        m = MatchesAncestry(repo, 'ghost')
+        if m.match(['the_ghost', 'ghost']) is None:
+            # the repo handles ghosts without corruption, so reconcile has
+            # nothing to do
+            return
+        self.assertThat(['ghost'], m)
+        reconciler = repo.reconcile()
+        # this is a data corrupting error, so a normal reconcile should fix it.
+        # one inconsistent parents should have been found : the
+        # available but not reference parent for ghost.
+        self.assertEqual(1, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # now the current inventory should still have 'ghost'
+        repo = d.open_repository()
+        repo.get_inventory('ghost')
+        repo.get_inventory('the_ghost')
+        self.assertThat(['the_ghost', 'ghost'],
+            MatchesAncestry(repo, 'ghost'))
+        self.assertThat(['the_ghost'],
+            MatchesAncestry(repo, 'the_ghost'))
+
+    def test_text_from_ghost_revision(self):
+        repo = self.make_repository('text-from-ghost')
+        inv = Inventory(revision_id='final-revid')
+        inv.root.revision = 'root-revid'
+        ie = inv.add_path('bla', 'file', 'myfileid')
+        ie.revision = 'ghostrevid'
+        ie.text_size = 42
+        ie.text_sha1 = "bee68c8acd989f5f1765b4660695275948bf5c00"
+        rev = bzrlib.revision.Revision(timestamp=0,
+                                       timezone=None,
+                                       committer="Foo Bar <foo@example.com>",
+                                       message="Message",
+                                       revision_id='final-revid')
+        repo.lock_write()
+        try:
+            repo.start_write_group()
+            try:
+                repo.add_revision('final-revid', rev, inv)
+                try:
+                    repo.texts.add_lines(('myfileid', 'ghostrevid'),
+                        (('myfileid', 'ghost-text-parent'),),
+                        ["line1\n", "line2\n"])
+                except errors.RevisionNotPresent:
+                    raise TestSkipped("text ghost parents not supported")
+                if repo.supports_rich_root():
+                    root_id = inv.root.file_id
+                    repo.texts.add_lines((inv.root.file_id, inv.root.revision),
+                        [], [])
+            finally:
+                repo.commit_write_group()
+        finally:
+            repo.unlock()
+        repo.reconcile(thorough=True)
+
+
+class TestReconcileWithIncorrectRevisionCache(TestReconcile):
+    """Ancestry data gets cached in knits and weaves should be reconcilable.
+
+    This class tests that reconcile can correct invalid caches (such as after
+    a reconcile).
+    """
+
+    def setUp(self):
+        self.reduceLockdirTimeout()
+        super(TestReconcileWithIncorrectRevisionCache, self).setUp()
+
+        t = self.get_transport()
+        # we need a revision with two parents in the wrong order
+        # which should trigger reinsertion.
+        # and another with the first one correct but the other two not
+        # which should not trigger reinsertion.
+        # these need to be in different repositories so that we don't
+        # trigger a reconcile based on the other case.
+        # there is no api to construct a broken knit repository at
+        # this point. if we ever encounter a bad graph in a knit repo
+        # we should add a lower level api to allow constructing such cases.
+
+        # first off the common logic:
+        self.first_tree = self.make_branch_and_tree('wrong-first-parent')
+        self.second_tree = self.make_branch_and_tree(
+            'reversed-secondary-parents')
+        for t in [self.first_tree, self.second_tree]:
+            t.commit('1', rev_id='1')
+            uncommit(t.branch, tree=t)
+            t.commit('2', rev_id='2')
+            uncommit(t.branch, tree=t)
+            t.commit('3', rev_id='3')
+            uncommit(t.branch, tree=t)
+        #second_tree = self.make_branch_and_tree('reversed-secondary-parents')
+        #second_tree.pull(tree) # XXX won't copy the repo?
+        repo_secondary = self.second_tree.branch.repository
+
+        # now setup the wrong-first parent case
+        repo = self.first_tree.branch.repository
+        repo.lock_write()
+        repo.start_write_group()
+        inv = Inventory(revision_id='wrong-first-parent')
+        inv.root.revision = 'wrong-first-parent'
+        if repo.supports_rich_root():
+            root_id = inv.root.file_id
+            repo.texts.add_lines((root_id, 'wrong-first-parent'), [], [])
+        sha1 = repo.add_inventory('wrong-first-parent', inv, ['2', '1'])
+        rev = Revision(timestamp=0,
+                       timezone=None,
+                       committer="Foo Bar <foo@example.com>",
+                       message="Message",
+                       inventory_sha1=sha1,
+                       revision_id='wrong-first-parent')
+        rev.parent_ids = ['1', '2']
+        repo.add_revision('wrong-first-parent', rev)
+        repo.commit_write_group()
+        repo.unlock()
+
+        # now setup the wrong-secondary parent case
+        repo = repo_secondary
+        repo.lock_write()
+        repo.start_write_group()
+        inv = Inventory(revision_id='wrong-secondary-parent')
+        inv.root.revision = 'wrong-secondary-parent'
+        if repo.supports_rich_root():
+            root_id = inv.root.file_id
+            repo.texts.add_lines((root_id, 'wrong-secondary-parent'), [], [])
+        sha1 = repo.add_inventory('wrong-secondary-parent', inv, ['1', '3', '2'])
+        rev = Revision(timestamp=0,
+                       timezone=None,
+                       committer="Foo Bar <foo@example.com>",
+                       message="Message",
+                       inventory_sha1=sha1,
+                       revision_id='wrong-secondary-parent')
+        rev.parent_ids = ['1', '2', '3']
+        repo.add_revision('wrong-secondary-parent', rev)
+        repo.commit_write_group()
+        repo.unlock()
+
+    def test_reconcile_wrong_order(self):
+        # a wrong order in primary parents is optionally correctable
+        repo = self.first_tree.branch.repository
+        repo.lock_read()
+        try:
+            g = repo.get_graph()
+            if g.get_parent_map(['wrong-first-parent'])['wrong-first-parent'] \
+                == ('1', '2'):
+                raise TestSkipped('wrong-first-parent is not setup for testing')
+        finally:
+            repo.unlock()
+        self.checkUnreconciled(repo.bzrdir, repo.reconcile())
+        # nothing should have been altered yet : inventories without
+        # revisions are not data loss incurring for current format
+        reconciler = repo.reconcile(thorough=True)
+        # these show up as inconsistent parents
+        self.assertEqual(1, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        # and should have been fixed:
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        g = repo.get_graph()
+        self.assertEqual(
+            {'wrong-first-parent':('1', '2')},
+            g.get_parent_map(['wrong-first-parent']))
+
+    def test_reconcile_wrong_order_secondary_inventory(self):
+        # a wrong order in the parents for inventories is ignored.
+        repo = self.second_tree.branch.repository
+        self.checkUnreconciled(repo.bzrdir, repo.reconcile())
+        self.checkUnreconciled(repo.bzrdir, repo.reconcile(thorough=True))

=== added file 'bzrlib/tests/per_repository_vf/test_refresh_data.py'
--- bzrlib/tests/per_repository_vf/test_refresh_data.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_refresh_data.py	2011-08-16 20:29:23 +0000
@@ -0,0 +1,91 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for VersionedFileRepository.refresh_data."""
+
+from bzrlib import repository
+
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+load_tests = load_tests_apply_scenarios
+
+
+class TestRefreshData(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def fetch_new_revision_into_concurrent_instance(self, repo, token):
+        """Create a new revision (revid 'new-rev') and fetch it into a
+        concurrent instance of repo.
+        """
+        source = self.make_branch_and_memory_tree('source')
+        source.lock_write()
+        self.addCleanup(source.unlock)
+        source.add([''], ['root-id'])
+        revid = source.commit('foo', rev_id='new-rev')
+        # Force data reading on weaves/knits
+        repo.all_revision_ids()
+        repo.revisions.keys()
+        repo.inventories.keys()
+        # server repo is the instance a smart server might hold for this
+        # repository.
+        server_repo = repo.bzrdir.open_repository()
+        try:
+            server_repo.lock_write(token)
+        except errors.TokenLockingNotSupported:
+            raise TestSkipped('Cannot concurrently insert into repo format %r'
+                % self.repository_format)
+        try:
+            server_repo.fetch(source.branch.repository, revid)
+        finally:
+            server_repo.unlock()
+
+    def test_refresh_data_after_fetch_new_data_visible_in_write_group(self):
+        tree = self.make_branch_and_memory_tree('target')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        tree.add([''], ['root-id'])
+        tree.commit('foo', rev_id='commit-in-target')
+        repo = tree.branch.repository
+        token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        repo.start_write_group()
+        self.addCleanup(repo.abort_write_group)
+        self.fetch_new_revision_into_concurrent_instance(repo, token)
+        # Call refresh_data.  It either fails with IsInWriteGroupError, or it
+        # succeeds and the new revisions are visible.
+        try:
+            repo.refresh_data()
+        except repository.IsInWriteGroupError:
+            pass
+        else:
+            self.assertEqual(
+                ['commit-in-target', 'new-rev'],
+                sorted(repo.all_revision_ids()))
+
+    def test_refresh_data_after_fetch_new_data_visible(self):
+        repo = self.make_repository('target')
+        token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        self.fetch_new_revision_into_concurrent_instance(repo, token)
+        repo.refresh_data()
+        self.assertNotEqual({}, repo.get_graph().get_parent_map(['new-rev']))
+
+

=== added file 'bzrlib/tests/per_repository_vf/test_repository.py'
--- bzrlib/tests/per_repository_vf/test_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_repository.py	2011-06-19 15:23:02 +0000
@@ -0,0 +1,482 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for repository implementations - tests a repository format."""
+
+from bzrlib import (
+    errors,
+    gpg,
+    inventory,
+    repository as _mod_repository,
+    revision as _mod_revision,
+    tests,
+    versionedfile,
+    vf_repository,
+    )
+
+from bzrlib.symbol_versioning import deprecated_in
+from bzrlib.tests.matchers import MatchesAncestry
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class TestRepository(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def assertFormatAttribute(self, attribute, allowed_values):
+        """Assert that the format has an attribute 'attribute'."""
+        repo = self.make_repository('repo')
+        self.assertSubset([getattr(repo._format, attribute)], allowed_values)
+
+    def test_attribute__fetch_order(self):
+        """Test the _fetch_order attribute."""
+        self.assertFormatAttribute('_fetch_order', ('topological', 'unordered'))
+
+    def test_attribute__fetch_uses_deltas(self):
+        """Test the _fetch_uses_deltas attribute."""
+        self.assertFormatAttribute('_fetch_uses_deltas', (True, False))
+
+    def test_attribute_inventories_store(self):
+        """Test the existence of the inventories attribute."""
+        tree = self.make_branch_and_tree('tree')
+        repo = tree.branch.repository
+        self.assertIsInstance(repo.inventories, versionedfile.VersionedFiles)
+
+    def test_attribute_inventories_basics(self):
+        """Test basic aspects of the inventories attribute."""
+        tree = self.make_branch_and_tree('tree')
+        repo = tree.branch.repository
+        rev_id = (tree.commit('a'),)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEqual(set([rev_id]), set(repo.inventories.keys()))
+
+    def test_attribute_revision_store(self):
+        """Test the existence of the revisions attribute."""
+        tree = self.make_branch_and_tree('tree')
+        repo = tree.branch.repository
+        self.assertIsInstance(repo.revisions,
+            versionedfile.VersionedFiles)
+
+    def test_attribute_revision_store_basics(self):
+        """Test the basic behaviour of the revisions attribute."""
+        tree = self.make_branch_and_tree('tree')
+        repo = tree.branch.repository
+        repo.lock_write()
+        try:
+            self.assertEqual(set(), set(repo.revisions.keys()))
+            revid = (tree.commit("foo"),)
+            self.assertEqual(set([revid]), set(repo.revisions.keys()))
+            self.assertEqual({revid:()},
+                repo.revisions.get_parent_map([revid]))
+        finally:
+            repo.unlock()
+        tree2 = self.make_branch_and_tree('tree2')
+        tree2.pull(tree.branch)
+        left_id = (tree2.commit('left'),)
+        right_id = (tree.commit('right'),)
+        tree.merge_from_branch(tree2.branch)
+        merge_id = (tree.commit('merged'),)
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        self.assertEqual(set([revid, left_id, right_id, merge_id]),
+            set(repo.revisions.keys()))
+        self.assertEqual({revid:(), left_id:(revid,), right_id:(revid,),
+             merge_id:(right_id, left_id)},
+            repo.revisions.get_parent_map(repo.revisions.keys()))
+
+    def test_attribute_signature_store(self):
+        """Test the existence of the signatures attribute."""
+        tree = self.make_branch_and_tree('tree')
+        repo = tree.branch.repository
+        self.assertIsInstance(repo.signatures,
+            versionedfile.VersionedFiles)
+
+    def test_exposed_versioned_files_are_marked_dirty(self):
+        repo = self.make_repository('.')
+        repo.lock_write()
+        signatures = repo.signatures
+        revisions = repo.revisions
+        inventories = repo.inventories
+        repo.unlock()
+        self.assertRaises(errors.ObjectNotLocked,
+            signatures.keys)
+        self.assertRaises(errors.ObjectNotLocked,
+            revisions.keys)
+        self.assertRaises(errors.ObjectNotLocked,
+            inventories.keys)
+        self.assertRaises(errors.ObjectNotLocked,
+            signatures.add_lines, ('foo',), [], [])
+        self.assertRaises(errors.ObjectNotLocked,
+            revisions.add_lines, ('foo',), [], [])
+        self.assertRaises(errors.ObjectNotLocked,
+            inventories.add_lines, ('foo',), [], [])
+
+    def test__get_sink(self):
+        repo = self.make_repository('repo')
+        sink = repo._get_sink()
+        self.assertIsInstance(sink, vf_repository.StreamSink)
+
+    def test_get_serializer_format(self):
+        repo = self.make_repository('.')
+        format = repo.get_serializer_format()
+        self.assertEqual(repo._serializer.format_num, format)
+
+    def test_add_revision_inventory_sha1(self):
+        inv = inventory.Inventory(revision_id='A')
+        inv.root.revision = 'A'
+        inv.root.file_id = 'fixed-root'
+        # Insert the inventory on its own to an identical repository, to get
+        # its sha1.
+        reference_repo = self.make_repository('reference_repo')
+        reference_repo.lock_write()
+        reference_repo.start_write_group()
+        inv_sha1 = reference_repo.add_inventory('A', inv, [])
+        reference_repo.abort_write_group()
+        reference_repo.unlock()
+        # Now insert a revision with this inventory, and it should get the same
+        # sha1.
+        repo = self.make_repository('repo')
+        repo.lock_write()
+        repo.start_write_group()
+        root_id = inv.root.file_id
+        repo.texts.add_lines(('fixed-root', 'A'), [], [])
+        repo.add_revision('A', _mod_revision.Revision(
+                'A', committer='B', timestamp=0,
+                timezone=0, message='C'), inv=inv)
+        repo.commit_write_group()
+        repo.unlock()
+        repo.lock_read()
+        self.assertEquals(inv_sha1, repo.get_revision('A').inventory_sha1)
+        repo.unlock()
+
+    def test_install_revisions(self):
+        wt = self.make_branch_and_tree('source')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        repo = wt.branch.repository
+        repo.lock_write()
+        repo.start_write_group()
+        repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
+        repo.commit_write_group()
+        repo.unlock()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        repo2 = self.make_repository('repo2')
+        revision = repo.get_revision('A')
+        tree = repo.revision_tree('A')
+        signature = repo.get_signature_text('A')
+        repo2.lock_write()
+        self.addCleanup(repo2.unlock)
+        vf_repository.install_revisions(repo2, [(revision, tree, signature)])
+        self.assertEqual(revision, repo2.get_revision('A'))
+        self.assertEqual(signature, repo2.get_signature_text('A'))
+
+    def test_attribute_text_store(self):
+        """Test the existence of the texts attribute."""
+        tree = self.make_branch_and_tree('tree')
+        repo = tree.branch.repository
+        self.assertIsInstance(repo.texts,
+            versionedfile.VersionedFiles)
+
+    def test_iter_inventories_is_ordered(self):
+        # just a smoke test
+        tree = self.make_branch_and_tree('a')
+        first_revision = tree.commit('')
+        second_revision = tree.commit('')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        revs = (first_revision, second_revision)
+        invs = tree.branch.repository.iter_inventories(revs)
+        for rev_id, inv in zip(revs, invs):
+            self.assertEqual(rev_id, inv.revision_id)
+            self.assertIsInstance(inv, inventory.CommonInventory)
+
+    def test_item_keys_introduced_by(self):
+        # Make a repo with one revision and one versioned file.
+        tree = self.make_branch_and_tree('t')
+        self.build_tree(['t/foo'])
+        tree.add('foo', 'file1')
+        tree.commit('message', rev_id='rev_id')
+        repo = tree.branch.repository
+        repo.lock_write()
+        repo.start_write_group()
+        try:
+            repo.sign_revision('rev_id', gpg.LoopbackGPGStrategy(None))
+        except errors.UnsupportedOperation:
+            signature_texts = []
+        else:
+            signature_texts = ['rev_id']
+        repo.commit_write_group()
+        repo.unlock()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+
+        # Item keys will be in this order, for maximum convenience for
+        # generating data to insert into knit repository:
+        #   * files
+        #   * inventory
+        #   * signatures
+        #   * revisions
+        expected_item_keys = [
+            ('file', 'file1', ['rev_id']),
+            ('inventory', None, ['rev_id']),
+            ('signatures', None, signature_texts),
+            ('revisions', None, ['rev_id'])]
+        item_keys = list(repo.item_keys_introduced_by(['rev_id']))
+        item_keys = [
+            (kind, file_id, list(versions))
+            for (kind, file_id, versions) in item_keys]
+
+        if repo.supports_rich_root():
+            # Check for the root versioned file in the item_keys, then remove
+            # it from streamed_names so we can compare that with
+            # expected_record_names.
+            # Note that the file keys can be in any order, so this test is
+            # written to allow that.
+            inv = repo.get_inventory('rev_id')
+            root_item_key = ('file', inv.root.file_id, ['rev_id'])
+            self.assertTrue(root_item_key in item_keys)
+            item_keys.remove(root_item_key)
+
+        self.assertEqual(expected_item_keys, item_keys)
+
+    def test_attribute_text_store_basics(self):
+        """Test the basic behaviour of the text store."""
+        tree = self.make_branch_and_tree('tree')
+        repo = tree.branch.repository
+        file_id = "Foo:Bar"
+        file_key = (file_id,)
+        tree.lock_write()
+        try:
+            self.assertEqual(set(), set(repo.texts.keys()))
+            tree.add(['foo'], [file_id], ['file'])
+            tree.put_file_bytes_non_atomic(file_id, 'content\n')
+            try:
+                rev_key = (tree.commit("foo"),)
+            except errors.IllegalPath:
+                raise tests.TestNotApplicable(
+                    'file_id %r cannot be stored on this'
+                    ' platform for this repo format' % (file_id,))
+            if repo._format.rich_root_data:
+                root_commit = (tree.get_root_id(),) + rev_key
+                keys = set([root_commit])
+                parents = {root_commit:()}
+            else:
+                keys = set()
+                parents = {}
+            keys.add(file_key + rev_key)
+            parents[file_key + rev_key] = ()
+            self.assertEqual(keys, set(repo.texts.keys()))
+            self.assertEqual(parents,
+                repo.texts.get_parent_map(repo.texts.keys()))
+        finally:
+            tree.unlock()
+        tree2 = self.make_branch_and_tree('tree2')
+        tree2.pull(tree.branch)
+        tree2.put_file_bytes_non_atomic('Foo:Bar', 'right\n')
+        right_key = (tree2.commit('right'),)
+        keys.add(file_key + right_key)
+        parents[file_key + right_key] = (file_key + rev_key,)
+        tree.put_file_bytes_non_atomic('Foo:Bar', 'left\n')
+        left_key = (tree.commit('left'),)
+        keys.add(file_key + left_key)
+        parents[file_key + left_key] = (file_key + rev_key,)
+        tree.merge_from_branch(tree2.branch)
+        tree.put_file_bytes_non_atomic('Foo:Bar', 'merged\n')
+        try:
+            tree.auto_resolve()
+        except errors.UnsupportedOperation:
+            pass
+        merge_key = (tree.commit('merged'),)
+        keys.add(file_key + merge_key)
+        parents[file_key + merge_key] = (file_key + left_key,
+                                         file_key + right_key)
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        self.assertEqual(keys, set(repo.texts.keys()))
+        self.assertEqual(parents, repo.texts.get_parent_map(repo.texts.keys()))
+
+
+class TestCaseWithComplexRepository(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def setUp(self):
+        super(TestCaseWithComplexRepository, self).setUp()
+        tree_a = self.make_branch_and_tree('a')
+        self.bzrdir = tree_a.branch.bzrdir
+        # add a corrupt inventory 'orphan'
+        # this may need some generalising for knits.
+        tree_a.lock_write()
+        try:
+            tree_a.branch.repository.start_write_group()
+            try:
+                inv_file = tree_a.branch.repository.inventories
+                inv_file.add_lines(('orphan',), [], [])
+            except:
+                tree_a.branch.repository.commit_write_group()
+                raise
+            else:
+                tree_a.branch.repository.abort_write_group()
+        finally:
+            tree_a.unlock()
+        # add a real revision 'rev1'
+        tree_a.commit('rev1', rev_id='rev1', allow_pointless=True)
+        # add a real revision 'rev2' based on rev1
+        tree_a.commit('rev2', rev_id='rev2', allow_pointless=True)
+        # add a reference to a ghost
+        tree_a.add_parent_tree_id('ghost1')
+        try:
+            tree_a.commit('rev3', rev_id='rev3', allow_pointless=True)
+        except errors.RevisionNotPresent:
+            raise tests.TestNotApplicable(
+                "Cannot test with ghosts for this format.")
+        # add another reference to a ghost, and a second ghost.
+        tree_a.add_parent_tree_id('ghost1')
+        tree_a.add_parent_tree_id('ghost2')
+        tree_a.commit('rev4', rev_id='rev4', allow_pointless=True)
+
+    def test_revision_trees(self):
+        revision_ids = ['rev1', 'rev2', 'rev3', 'rev4']
+        repository = self.bzrdir.open_repository()
+        repository.lock_read()
+        self.addCleanup(repository.unlock)
+        trees1 = list(repository.revision_trees(revision_ids))
+        trees2 = [repository.revision_tree(t) for t in revision_ids]
+        self.assertEqual(len(trees1), len(trees2))
+        for tree1, tree2 in zip(trees1, trees2):
+            self.assertFalse(tree2.changes_from(tree1).has_changed())
+
+    def test_get_deltas_for_revisions(self):
+        repository = self.bzrdir.open_repository()
+        repository.lock_read()
+        self.addCleanup(repository.unlock)
+        revisions = [repository.get_revision(r) for r in
+                     ['rev1', 'rev2', 'rev3', 'rev4']]
+        deltas1 = list(repository.get_deltas_for_revisions(revisions))
+        deltas2 = [repository.get_revision_delta(r.revision_id) for r in
+                   revisions]
+        self.assertEqual(deltas1, deltas2)
+
+    def test_all_revision_ids(self):
+        # all_revision_ids -> all revisions
+        self.assertEqual(set(['rev1', 'rev2', 'rev3', 'rev4']),
+            set(self.bzrdir.open_repository().all_revision_ids()))
+
+    def test_get_ancestry_missing_revision(self):
+        # get_ancestry(revision that is in some data but not fully installed
+        # -> NoSuchRevision
+        repo = self.bzrdir.open_repository()
+        self.assertRaises(errors.NoSuchRevision,
+            self.applyDeprecated, deprecated_in((2, 4, 0)),
+            repo.get_ancestry, 'orphan')
+
+    def test_get_unordered_ancestry(self):
+        repo = self.bzrdir.open_repository()
+        self.assertEqual(
+            set(self.applyDeprecated(deprecated_in((2, 4, 0)),
+                repo.get_ancestry, 'rev3')),
+            set(self.applyDeprecated(deprecated_in((2, 4, 0)),
+                repo.get_ancestry, 'rev3', topo_sorted=False)))
+
+    def test_reserved_id(self):
+        repo = self.make_repository('repository')
+        repo.lock_write()
+        repo.start_write_group()
+        try:
+            self.assertRaises(errors.ReservedId, repo.add_inventory,
+                'reserved:', None, None)
+            self.assertRaises(errors.ReservedId, repo.add_inventory_by_delta,
+                "foo", [], 'reserved:', None)
+            self.assertRaises(errors.ReservedId, repo.add_revision,
+                'reserved:', None)
+        finally:
+            repo.abort_write_group()
+            repo.unlock()
+
+
+class TestCaseWithCorruptRepository(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def setUp(self):
+        super(TestCaseWithCorruptRepository, self).setUp()
+        # a inventory with no parents and the revision has parents..
+        # i.e. a ghost.
+        repo = self.make_repository('inventory_with_unnecessary_ghost')
+        repo.lock_write()
+        repo.start_write_group()
+        inv = inventory.Inventory(revision_id = 'ghost')
+        inv.root.revision = 'ghost'
+        if repo.supports_rich_root():
+            root_id = inv.root.file_id
+            repo.texts.add_lines((root_id, 'ghost'), [], [])
+        sha1 = repo.add_inventory('ghost', inv, [])
+        rev = _mod_revision.Revision(
+            timestamp=0, timezone=None, committer="Foo Bar <foo@example.com>",
+            message="Message", inventory_sha1=sha1, revision_id='ghost')
+        rev.parent_ids = ['the_ghost']
+        try:
+            repo.add_revision('ghost', rev)
+        except (errors.NoSuchRevision, errors.RevisionNotPresent):
+            raise tests.TestNotApplicable(
+                "Cannot test with ghosts for this format.")
+
+        inv = inventory.Inventory(revision_id = 'the_ghost')
+        inv.root.revision = 'the_ghost'
+        if repo.supports_rich_root():
+            root_id = inv.root.file_id
+            repo.texts.add_lines((root_id, 'the_ghost'), [], [])
+        sha1 = repo.add_inventory('the_ghost', inv, [])
+        rev = _mod_revision.Revision(
+            timestamp=0, timezone=None, committer="Foo Bar <foo@example.com>",
+            message="Message", inventory_sha1=sha1, revision_id='the_ghost')
+        rev.parent_ids = []
+        repo.add_revision('the_ghost', rev)
+        # check its setup usefully
+        inv_weave = repo.inventories
+        possible_parents = (None, (('ghost',),))
+        self.assertSubset(inv_weave.get_parent_map([('ghost',)])[('ghost',)],
+            possible_parents)
+        repo.commit_write_group()
+        repo.unlock()
+
+    def test_corrupt_revision_access_asserts_if_reported_wrong(self):
+        repo_url = self.get_url('inventory_with_unnecessary_ghost')
+        repo = _mod_repository.Repository.open(repo_url)
+        m = MatchesAncestry(repo, 'ghost')
+        reported_wrong = False
+        try:
+            if m.match(['the_ghost', 'ghost']) is not None:
+                reported_wrong = True
+        except errors.CorruptRepository:
+            # caught the bad data:
+            return
+        if not reported_wrong:
+            return
+        self.assertRaises(errors.CorruptRepository, repo.get_revision, 'ghost')
+
+    def test_corrupt_revision_get_revision_reconcile(self):
+        repo_url = self.get_url('inventory_with_unnecessary_ghost')
+        repo = _mod_repository.Repository.open(repo_url)
+        repo.get_revision_reconcile('ghost')

=== added file 'bzrlib/tests/per_repository_vf/test_write_group.py'
--- bzrlib/tests/per_repository_vf/test_write_group.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_write_group.py	2011-05-09 20:08:47 +0000
@@ -0,0 +1,655 @@
+# Copyright (C) 2007-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for repository write groups."""
+
+import sys
+
+from bzrlib import (
+    branch,
+    bzrdir,
+    errors,
+    memorytree,
+    remote,
+    tests,
+    versionedfile,
+    )
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class TestGetMissingParentInventories(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test_empty_get_missing_parent_inventories(self):
+        """A new write group has no missing parent inventories."""
+        repo = self.make_repository('.')
+        repo.lock_write()
+        repo.start_write_group()
+        try:
+            self.assertEqual(set(), set(repo.get_missing_parent_inventories()))
+        finally:
+            repo.commit_write_group()
+            repo.unlock()
+
+    def branch_trunk_and_make_tree(self, trunk_repo, relpath):
+        tree = self.make_branch_and_memory_tree('branch')
+        trunk_repo.lock_read()
+        self.addCleanup(trunk_repo.unlock)
+        tree.branch.repository.fetch(trunk_repo, revision_id='rev-1')
+        tree.set_parent_ids(['rev-1'])
+        return tree 
+
+    def make_first_commit(self, repo):
+        trunk = repo.bzrdir.create_branch()
+        tree = memorytree.MemoryTree.create_on_branch(trunk)
+        tree.lock_write()
+        tree.add([''], ['TREE_ROOT'], ['directory'])
+        tree.add(['dir'], ['dir-id'], ['directory'])
+        tree.add(['filename'], ['file-id'], ['file'])
+        tree.put_file_bytes_non_atomic('file-id', 'content\n')
+        tree.commit('Trunk commit', rev_id='rev-0')
+        tree.commit('Trunk commit', rev_id='rev-1')
+        tree.unlock()
+
+    def make_new_commit_in_new_repo(self, trunk_repo, parents=None):
+        tree = self.branch_trunk_and_make_tree(trunk_repo, 'branch')
+        tree.set_parent_ids(parents)
+        tree.commit('Branch commit', rev_id='rev-2')
+        branch_repo = tree.branch.repository
+        branch_repo.lock_read()
+        self.addCleanup(branch_repo.unlock)
+        return branch_repo
+
+    def make_stackable_repo(self, relpath='trunk'):
+        if isinstance(self.repository_format, remote.RemoteRepositoryFormat):
+            # RemoteRepository by default builds a default format real
+            # repository, but the default format is unstackble.  So explicitly
+            # make a stackable real repository and use that.
+            repo = self.make_repository(relpath, format='1.9')
+            repo = bzrdir.BzrDir.open(self.get_url(relpath)).open_repository()
+        else:
+            repo = self.make_repository(relpath)
+        if not repo._format.supports_external_lookups:
+            raise tests.TestNotApplicable('format not stackable')
+        repo.bzrdir._format.set_branch_format(branch.BzrBranchFormat7())
+        return repo
+
+    def reopen_repo_and_resume_write_group(self, repo):
+        try:
+            resume_tokens = repo.suspend_write_group()
+        except errors.UnsuspendableWriteGroup:
+            # If we got this far, and this repo does not support resuming write
+            # groups, then get_missing_parent_inventories works in all
+            # cases this repo supports.
+            repo.unlock()
+            return
+        repo.unlock()
+        reopened_repo = repo.bzrdir.open_repository()
+        reopened_repo.lock_write()
+        self.addCleanup(reopened_repo.unlock)
+        reopened_repo.resume_write_group(resume_tokens)
+        return reopened_repo
+
+    def test_ghost_revision(self):
+        """A parent inventory may be absent if all the needed texts are present.
+        i.e., a ghost revision isn't (necessarily) considered to be a missing
+        parent inventory.
+        """
+        # Make a trunk with one commit.
+        trunk_repo = self.make_stackable_repo()
+        self.make_first_commit(trunk_repo)
+        trunk_repo.lock_read()
+        self.addCleanup(trunk_repo.unlock)
+        # Branch the trunk, add a new commit.
+        branch_repo = self.make_new_commit_in_new_repo(
+            trunk_repo, parents=['rev-1', 'ghost-rev'])
+        inv = branch_repo.get_inventory('rev-2')
+        # Make a new repo stacked on trunk, and then copy into it:
+        #  - all texts in rev-2
+        #  - the new inventory (rev-2)
+        #  - the new revision (rev-2)
+        repo = self.make_stackable_repo('stacked')
+        repo.lock_write()
+        repo.start_write_group()
+        # Add all texts from in rev-2 inventory.  Note that this has to exclude
+        # the root if the repo format does not support rich roots.
+        rich_root = branch_repo._format.rich_root_data
+        all_texts = [
+            (ie.file_id, ie.revision) for ie in inv.iter_just_entries()
+             if rich_root or inv.id2path(ie.file_id) != '']
+        repo.texts.insert_record_stream(
+            branch_repo.texts.get_record_stream(all_texts, 'unordered', False))
+        # Add inventory and revision for rev-2.
+        repo.add_inventory('rev-2', inv, ['rev-1', 'ghost-rev'])
+        repo.revisions.insert_record_stream(
+            branch_repo.revisions.get_record_stream(
+                [('rev-2',)], 'unordered', False))
+        # Now, no inventories are reported as missing, even though there is a
+        # ghost.
+        self.assertEqual(set(), repo.get_missing_parent_inventories())
+        # Resuming the write group does not affect
+        # get_missing_parent_inventories.
+        reopened_repo = self.reopen_repo_and_resume_write_group(repo)
+        self.assertEqual(set(), reopened_repo.get_missing_parent_inventories())
+        reopened_repo.abort_write_group()
+
+    def test_get_missing_parent_inventories(self):
+        """A stacked repo with a single revision and inventory (no parent
+        inventory) in it must have all the texts in its inventory (even if not
+        changed w.r.t. to the absent parent), otherwise it will report missing
+        texts/parent inventory.
+
+        The core of this test is that a file was changed in rev-1, but in a
+        stacked repo that only has rev-2
+        """
+        # Make a trunk with one commit.
+        trunk_repo = self.make_stackable_repo()
+        self.make_first_commit(trunk_repo)
+        trunk_repo.lock_read()
+        self.addCleanup(trunk_repo.unlock)
+        # Branch the trunk, add a new commit.
+        branch_repo = self.make_new_commit_in_new_repo(
+            trunk_repo, parents=['rev-1'])
+        inv = branch_repo.get_inventory('rev-2')
+        # Make a new repo stacked on trunk, and copy the new commit's revision
+        # and inventory records to it.
+        repo = self.make_stackable_repo('stacked')
+        repo.lock_write()
+        repo.start_write_group()
+        # Insert a single fulltext inv (using add_inventory because it's
+        # simpler than insert_record_stream)
+        repo.add_inventory('rev-2', inv, ['rev-1'])
+        repo.revisions.insert_record_stream(
+            branch_repo.revisions.get_record_stream(
+                [('rev-2',)], 'unordered', False))
+        # There should be no missing compression parents
+        self.assertEqual(set(),
+                repo.inventories.get_missing_compression_parent_keys())
+        self.assertEqual(
+            set([('inventories', 'rev-1')]),
+            repo.get_missing_parent_inventories())
+        # Resuming the write group does not affect
+        # get_missing_parent_inventories.
+        reopened_repo = self.reopen_repo_and_resume_write_group(repo)
+        self.assertEqual(
+            set([('inventories', 'rev-1')]),
+            reopened_repo.get_missing_parent_inventories())
+        # Adding the parent inventory satisfies get_missing_parent_inventories.
+        reopened_repo.inventories.insert_record_stream(
+            branch_repo.inventories.get_record_stream(
+                [('rev-1',)], 'unordered', False))
+        self.assertEqual(
+            set(), reopened_repo.get_missing_parent_inventories())
+        reopened_repo.abort_write_group()
+
+    def test_get_missing_parent_inventories_check(self):
+        builder = self.make_branch_builder('test')
+        builder.build_snapshot('A-id', ['ghost-parent-id'], [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('file', 'file-id', 'file', 'content\n'))],
+            allow_leftmost_as_ghost=True)
+        b = builder.get_branch()
+        b.lock_read()
+        self.addCleanup(b.unlock)
+        repo = self.make_repository('test-repo')
+        repo.lock_write()
+        self.addCleanup(repo.unlock)
+        repo.start_write_group()
+        self.addCleanup(repo.abort_write_group)
+        # Now, add the objects manually
+        text_keys = [('file-id', 'A-id')]
+        if repo.supports_rich_root():
+            text_keys.append(('root-id', 'A-id'))
+        # Directly add the texts, inventory, and revision object for 'A-id'
+        repo.texts.insert_record_stream(b.repository.texts.get_record_stream(
+            text_keys, 'unordered', True))
+        repo.add_revision('A-id', b.repository.get_revision('A-id'),
+                          b.repository.get_inventory('A-id'))
+        get_missing = repo.get_missing_parent_inventories
+        if repo._format.supports_external_lookups:
+            self.assertEqual(set([('inventories', 'ghost-parent-id')]),
+                get_missing(check_for_missing_texts=False))
+            self.assertEqual(set(), get_missing(check_for_missing_texts=True))
+            self.assertEqual(set(), get_missing())
+        else:
+            # If we don't support external lookups, we always return empty
+            self.assertEqual(set(), get_missing(check_for_missing_texts=False))
+            self.assertEqual(set(), get_missing(check_for_missing_texts=True))
+            self.assertEqual(set(), get_missing())
+
+    def test_insert_stream_passes_resume_info(self):
+        repo = self.make_repository('test-repo')
+        if (not repo._format.supports_external_lookups or
+            isinstance(repo, remote.RemoteRepository)):
+            raise tests.TestNotApplicable(
+                'only valid for direct connections to resumable repos')
+        # log calls to get_missing_parent_inventories, so that we can assert it
+        # is called with the correct parameters
+        call_log = []
+        orig = repo.get_missing_parent_inventories
+        def get_missing(check_for_missing_texts=True):
+            call_log.append(check_for_missing_texts)
+            return orig(check_for_missing_texts=check_for_missing_texts)
+        repo.get_missing_parent_inventories = get_missing
+        repo.lock_write()
+        self.addCleanup(repo.unlock)
+        sink = repo._get_sink()
+        sink.insert_stream((), repo._format, [])
+        self.assertEqual([False], call_log)
+        del call_log[:]
+        repo.start_write_group()
+        # We need to insert something, or suspend_write_group won't actually
+        # create a token
+        repo.texts.insert_record_stream([versionedfile.FulltextContentFactory(
+            ('file-id', 'rev-id'), (), None, 'lines\n')])
+        tokens = repo.suspend_write_group()
+        self.assertNotEqual([], tokens)
+        sink.insert_stream((), repo._format, tokens)
+        self.assertEqual([True], call_log)
+
+    def test_insert_stream_without_locking_fails_without_lock(self):
+        repo = self.make_repository('test-repo')
+        sink = repo._get_sink()
+        stream = [('texts', [versionedfile.FulltextContentFactory(
+            ('file-id', 'rev-id'), (), None, 'lines\n')])]
+        self.assertRaises(errors.ObjectNotLocked,
+            sink.insert_stream_without_locking, stream, repo._format)
+
+    def test_insert_stream_without_locking_fails_without_write_group(self):
+        repo = self.make_repository('test-repo')
+        self.addCleanup(repo.lock_write().unlock)
+        sink = repo._get_sink()
+        stream = [('texts', [versionedfile.FulltextContentFactory(
+            ('file-id', 'rev-id'), (), None, 'lines\n')])]
+        self.assertRaises(errors.BzrError,
+            sink.insert_stream_without_locking, stream, repo._format)
+
+    def test_insert_stream_without_locking(self):
+        repo = self.make_repository('test-repo')
+        self.addCleanup(repo.lock_write().unlock)
+        repo.start_write_group()
+        sink = repo._get_sink()
+        stream = [('texts', [versionedfile.FulltextContentFactory(
+            ('file-id', 'rev-id'), (), None, 'lines\n')])]
+        missing_keys = sink.insert_stream_without_locking(stream, repo._format)
+        repo.commit_write_group()
+        self.assertEqual(set(), missing_keys)
+
+
+class TestResumeableWriteGroup(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def make_write_locked_repo(self, relpath='repo'):
+        repo = self.make_repository(relpath)
+        repo.lock_write()
+        self.addCleanup(repo.unlock)
+        return repo
+
+    def reopen_repo(self, repo):
+        same_repo = repo.bzrdir.open_repository()
+        same_repo.lock_write()
+        self.addCleanup(same_repo.unlock)
+        return same_repo
+
+    def require_suspendable_write_groups(self, reason):
+        repo = self.make_repository('__suspend_test')
+        repo.lock_write()
+        self.addCleanup(repo.unlock)
+        repo.start_write_group()
+        try:
+            wg_tokens = repo.suspend_write_group()
+        except errors.UnsuspendableWriteGroup:
+            repo.abort_write_group()
+            raise tests.TestNotApplicable(reason)
+
+    def test_suspend_write_group(self):
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        repo.texts.add_lines(('file-id', 'revid'), (), ['lines'])
+        try:
+            wg_tokens = repo.suspend_write_group()
+        except errors.UnsuspendableWriteGroup:
+            # The contract for repos that don't support suspending write groups
+            # is that suspend_write_group raises UnsuspendableWriteGroup, but
+            # is otherwise a no-op.  So we can still e.g. abort the write group
+            # as usual.
+            self.assertTrue(repo.is_in_write_group())
+            repo.abort_write_group()
+        else:
+            # After suspending a write group we are no longer in a write group
+            self.assertFalse(repo.is_in_write_group())
+            # suspend_write_group returns a list of tokens, which are strs.  If
+            # no other write groups were resumed, there will only be one token.
+            self.assertEqual(1, len(wg_tokens))
+            self.assertIsInstance(wg_tokens[0], str)
+            # See also test_pack_repository's test of the same name.
+
+    def test_resume_write_group_then_abort(self):
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        try:
+            wg_tokens = repo.suspend_write_group()
+        except errors.UnsuspendableWriteGroup:
+            # If the repo does not support suspending write groups, it doesn't
+            # support resuming them either.
+            repo.abort_write_group()
+            self.assertRaises(
+                errors.UnsuspendableWriteGroup, repo.resume_write_group, [])
+        else:
+            #self.assertEqual([], list(repo.texts.keys()))
+            same_repo = self.reopen_repo(repo)
+            same_repo.resume_write_group(wg_tokens)
+            self.assertEqual([text_key], list(same_repo.texts.keys()))
+            self.assertTrue(same_repo.is_in_write_group())
+            same_repo.abort_write_group()
+            self.assertEqual([], list(repo.texts.keys()))
+            # See also test_pack_repository's test of the same name.
+
+    def test_multiple_resume_write_group(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        first_key = ('file-id', 'revid')
+        repo.texts.add_lines(first_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        self.assertTrue(same_repo.is_in_write_group())
+        second_key = ('file-id', 'second-revid')
+        same_repo.texts.add_lines(second_key, (first_key,), ['more lines'])
+        try:
+            new_wg_tokens = same_repo.suspend_write_group()
+        except:
+            e = sys.exc_info()
+            same_repo.abort_write_group(suppress_errors=True)
+            raise e[0], e[1], e[2]
+        self.assertEqual(2, len(new_wg_tokens))
+        self.assertSubset(wg_tokens, new_wg_tokens)
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(new_wg_tokens)
+        both_keys = set([first_key, second_key])
+        self.assertEqual(both_keys, same_repo.texts.keys())
+        same_repo.abort_write_group()
+
+    def test_no_op_suspend_resume(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        new_wg_tokens = same_repo.suspend_write_group()
+        self.assertEqual(wg_tokens, new_wg_tokens)
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        self.assertEqual([text_key], list(same_repo.texts.keys()))
+        same_repo.abort_write_group()
+
+    def test_read_after_suspend_fails(self):
+        self.require_suspendable_write_groups(
+            'Cannot test suspend on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        self.assertEqual([], list(repo.texts.keys()))
+
+    def test_read_after_second_suspend_fails(self):
+        self.require_suspendable_write_groups(
+            'Cannot test suspend on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        same_repo.suspend_write_group()
+        self.assertEqual([], list(same_repo.texts.keys()))
+
+    def test_read_after_resume_abort_fails(self):
+        self.require_suspendable_write_groups(
+            'Cannot test suspend on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        same_repo.abort_write_group()
+        self.assertEqual([], list(same_repo.texts.keys()))
+
+    def test_cannot_resume_aborted_write_group(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        same_repo.abort_write_group()
+        same_repo = self.reopen_repo(repo)
+        self.assertRaises(
+            errors.UnresumableWriteGroup, same_repo.resume_write_group,
+            wg_tokens)
+
+    def test_commit_resumed_write_group_no_new_data(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        same_repo.commit_write_group()
+        self.assertEqual([text_key], list(same_repo.texts.keys()))
+        self.assertEqual(
+            'lines', same_repo.texts.get_record_stream([text_key],
+                'unordered', True).next().get_bytes_as('fulltext'))
+        self.assertRaises(
+            errors.UnresumableWriteGroup, same_repo.resume_write_group,
+            wg_tokens)
+
+    def test_commit_resumed_write_group_plus_new_data(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        first_key = ('file-id', 'revid')
+        repo.texts.add_lines(first_key, (), ['lines'])
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        second_key = ('file-id', 'second-revid')
+        same_repo.texts.add_lines(second_key, (first_key,), ['more lines'])
+        same_repo.commit_write_group()
+        self.assertEqual(
+            set([first_key, second_key]), set(same_repo.texts.keys()))
+        self.assertEqual(
+            'lines', same_repo.texts.get_record_stream([first_key],
+                'unordered', True).next().get_bytes_as('fulltext'))
+        self.assertEqual(
+            'more lines', same_repo.texts.get_record_stream([second_key],
+                'unordered', True).next().get_bytes_as('fulltext'))
+
+    def make_source_with_delta_record(self):
+        # Make a source repository with a delta record in it.
+        source_repo = self.make_write_locked_repo('source')
+        source_repo.start_write_group()
+        key_base = ('file-id', 'base')
+        key_delta = ('file-id', 'delta')
+        def text_stream():
+            yield versionedfile.FulltextContentFactory(
+                key_base, (), None, 'lines\n')
+            yield versionedfile.FulltextContentFactory(
+                key_delta, (key_base,), None, 'more\nlines\n')
+        source_repo.texts.insert_record_stream(text_stream())
+        source_repo.commit_write_group()
+        return source_repo
+
+    def test_commit_resumed_write_group_with_missing_parents(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        source_repo = self.make_source_with_delta_record()
+        key_base = ('file-id', 'base')
+        key_delta = ('file-id', 'delta')
+        # Start a write group, insert just a delta.
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        stream = source_repo.texts.get_record_stream(
+            [key_delta], 'unordered', False)
+        repo.texts.insert_record_stream(stream)
+        # It's either not commitable due to the missing compression parent, or
+        # the stacked location has already filled in the fulltext.
+        try:
+            repo.commit_write_group()
+        except errors.BzrCheckError:
+            # It refused to commit because we have a missing parent
+            pass
+        else:
+            same_repo = self.reopen_repo(repo)
+            same_repo.lock_read()
+            record = same_repo.texts.get_record_stream([key_delta],
+                                                       'unordered', True).next()
+            self.assertEqual('more\nlines\n', record.get_bytes_as('fulltext'))
+            return
+        # Merely suspending and resuming doesn't make it commitable either.
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        self.assertRaises(
+            errors.BzrCheckError, same_repo.commit_write_group)
+        same_repo.abort_write_group()
+
+    def test_commit_resumed_write_group_adding_missing_parents(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        source_repo = self.make_source_with_delta_record()
+        key_base = ('file-id', 'base')
+        key_delta = ('file-id', 'delta')
+        # Start a write group.
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        # Add some content so this isn't an empty write group (which may return
+        # 0 tokens)
+        text_key = ('file-id', 'revid')
+        repo.texts.add_lines(text_key, (), ['lines'])
+        # Suspend it, then resume it.
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        # Add a record with a missing compression parent
+        stream = source_repo.texts.get_record_stream(
+            [key_delta], 'unordered', False)
+        same_repo.texts.insert_record_stream(stream)
+        # Just like if we'd added that record without a suspend/resume cycle,
+        # commit_write_group fails.
+        try:
+            same_repo.commit_write_group()
+        except errors.BzrCheckError:
+            pass
+        else:
+            # If the commit_write_group didn't fail, that is because the
+            # insert_record_stream already gave it a fulltext.
+            same_repo = self.reopen_repo(repo)
+            same_repo.lock_read()
+            record = same_repo.texts.get_record_stream([key_delta],
+                                                       'unordered', True).next()
+            self.assertEqual('more\nlines\n', record.get_bytes_as('fulltext'))
+            return
+        same_repo.abort_write_group()
+
+    def test_add_missing_parent_after_resume(self):
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        source_repo = self.make_source_with_delta_record()
+        key_base = ('file-id', 'base')
+        key_delta = ('file-id', 'delta')
+        # Start a write group, insert just a delta.
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        stream = source_repo.texts.get_record_stream(
+            [key_delta], 'unordered', False)
+        repo.texts.insert_record_stream(stream)
+        # Suspend it, then resume it.
+        wg_tokens = repo.suspend_write_group()
+        same_repo = self.reopen_repo(repo)
+        same_repo.resume_write_group(wg_tokens)
+        # Fill in the missing compression parent.
+        stream = source_repo.texts.get_record_stream(
+            [key_base], 'unordered', False)
+        same_repo.texts.insert_record_stream(stream)
+        same_repo.commit_write_group()
+
+    def test_suspend_empty_initial_write_group(self):
+        """Suspending a write group with no writes returns an empty token
+        list.
+        """
+        self.require_suspendable_write_groups(
+            'Cannot test suspend on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.start_write_group()
+        wg_tokens = repo.suspend_write_group()
+        self.assertEqual([], wg_tokens)
+
+    def test_resume_empty_initial_write_group(self):
+        """Resuming an empty token list is equivalent to start_write_group."""
+        self.require_suspendable_write_groups(
+            'Cannot test resume on repo that does not support suspending')
+        repo = self.make_write_locked_repo()
+        repo.resume_write_group([])
+        repo.abort_write_group()

=== modified file 'bzrlib/tests/per_transport.py'
--- bzrlib/tests/per_transport.py	2010-07-19 11:06:36 +0000
+++ bzrlib/tests/per_transport.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,28 +26,25 @@
 from StringIO import StringIO as pyStringIO
 import stat
 import sys
-import unittest
 
 from bzrlib import (
     errors,
     osutils,
+    pyutils,
     tests,
+    transport as _mod_transport,
     urlutils,
     )
 from bzrlib.errors import (ConnectionError,
-                           DirectoryNotEmpty,
                            FileExists,
                            InvalidURL,
-                           LockError,
                            NoSuchFile,
-                           NotLocalUrl,
                            PathError,
                            TransportNotPossible,
                            )
 from bzrlib.osutils import getcwd
 from bzrlib.smart import medium
 from bzrlib.tests import (
-    TestCaseInTempDir,
     TestSkipped,
     TestNotApplicable,
     multiply_tests,
@@ -56,7 +53,6 @@
 from bzrlib.tests.test_transport import TestTransportImplementation
 from bzrlib.transport import (
     ConnectedTransport,
-    get_transport,
     _get_transport_modules,
     )
 from bzrlib.transport.memory import MemoryTransport
@@ -78,7 +74,7 @@
     for module in _get_transport_modules():
         try:
             permutations = get_transport_test_permutations(
-                reduce(getattr, (module).split('.')[1:], __import__(module)))
+                pyutils.get_named_object(module))
             for (klass, server_factory) in permutations:
                 scenario = ('%s,%s' % (klass.__name__, server_factory.__name__),
                     {"transport_class":klass,
@@ -102,11 +98,11 @@
 
     def setUp(self):
         super(TransportTests, self).setUp()
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def check_transport_contents(self, content, transport, relpath):
-        """Check that transport.get(relpath).read() == content."""
-        self.assertEqualDiff(content, transport.get(relpath).read())
+        """Check that transport.get_bytes(relpath) == content."""
+        self.assertEqualDiff(content, transport.get_bytes(relpath))
 
     def test_ensure_base_missing(self):
         """.ensure_base() should create the directory if it doesn't exist"""
@@ -211,8 +207,17 @@
                     ]
         self.build_tree(files, transport=t, line_endings='binary')
         self.assertRaises(NoSuchFile, t.get, 'c')
-        self.assertListRaises(NoSuchFile, t.get_multi, ['a', 'b', 'c'])
-        self.assertListRaises(NoSuchFile, t.get_multi, iter(['a', 'b', 'c']))
+        def iterate_and_close(func, *args):
+            for f in func(*args):
+                # We call f.read() here because things like paramiko actually
+                # spawn a thread to prefetch the content, which we want to
+                # consume before we close the handle.
+                content = f.read()
+                f.close()
+        self.assertRaises(NoSuchFile, iterate_and_close,
+                          t.get_multi, ['a', 'b', 'c'])
+        self.assertRaises(NoSuchFile, iterate_and_close,
+                          t.get_multi, iter(['a', 'b', 'c']))
 
     def test_get_directory_read_gives_ReadError(self):
         """consistent errors for read() on a file returned by get()."""
@@ -251,7 +256,6 @@
 
     def test_get_bytes_unknown_file(self):
         t = self.get_transport()
-
         self.assertRaises(NoSuchFile, t.get_bytes, 'c')
 
     def test_get_with_open_write_stream_sees_all_content(self):
@@ -261,7 +265,7 @@
         handle = t.open_write_stream('foo')
         try:
             handle.write('b')
-            self.assertEqual('b', t.get('foo').read())
+            self.assertEqual('b', t.get_bytes('foo'))
         finally:
             handle.close()
 
@@ -273,7 +277,11 @@
         try:
             handle.write('b')
             self.assertEqual('b', t.get_bytes('foo'))
-            self.assertEqual('b', t.get('foo').read())
+            f = t.get('foo')
+            try:
+                self.assertEqual('b', f.read())
+            finally:
+                f.close()
         finally:
             handle.close()
 
@@ -286,7 +294,7 @@
             return
 
         t.put_bytes('a', 'some text for a\n')
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
 
         # The contents should be overwritten
@@ -304,9 +312,9 @@
                     t.put_bytes_non_atomic, 'a', 'some text for a\n')
             return
 
-        self.failIf(t.has('a'))
+        self.assertFalse(t.has('a'))
         t.put_bytes_non_atomic('a', 'some text for a\n')
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
         # Put also replaces contents
         t.put_bytes_non_atomic('a', 'new\ncontents for\na\n')
@@ -324,7 +332,7 @@
         # Now test the create_parent flag
         self.assertRaises(NoSuchFile, t.put_bytes_non_atomic, 'dir/a',
                                        'contents\n')
-        self.failIf(t.has('dir/a'))
+        self.assertFalse(t.has('dir/a'))
         t.put_bytes_non_atomic('dir/a', 'contents for dir/a\n',
                                create_parent_dir=True)
         self.check_transport_contents('contents for dir/a\n', t, 'dir/a')
@@ -402,7 +410,7 @@
         result = t.put_file('a', StringIO('some text for a\n'))
         # put_file returns the length of the data written
         self.assertEqual(16, result)
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
         # Put also replaces contents
         result = t.put_file('a', StringIO('new\ncontents for\na\n'))
@@ -420,9 +428,9 @@
                     t.put_file_non_atomic, 'a', StringIO('some text for a\n'))
             return
 
-        self.failIf(t.has('a'))
+        self.assertFalse(t.has('a'))
         t.put_file_non_atomic('a', StringIO('some text for a\n'))
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         self.check_transport_contents('some text for a\n', t, 'a')
         # Put also replaces contents
         t.put_file_non_atomic('a', StringIO('new\ncontents for\na\n'))
@@ -440,7 +448,7 @@
         # Now test the create_parent flag
         self.assertRaises(NoSuchFile, t.put_file_non_atomic, 'dir/a',
                                        StringIO('contents\n'))
-        self.failIf(t.has('dir/a'))
+        self.assertFalse(t.has('dir/a'))
         t.put_file_non_atomic('dir/a', StringIO('contents for dir/a\n'),
                               create_parent_dir=True)
         self.check_transport_contents('contents for dir/a\n', t, 'dir/a')
@@ -645,7 +653,7 @@
             self.build_tree(files, transport=transport_from)
             self.assertEqual(4, transport_from.copy_to(files, transport_to))
             for f in files:
-                self.check_transport_contents(transport_to.get(f).read(),
+                self.check_transport_contents(transport_to.get_bytes(f),
                                               transport_from, f)
 
         t = self.get_transport()
@@ -674,7 +682,7 @@
         files = ['a', 'b', 'c', 'd']
         t.copy_to(iter(files), temp_transport)
         for f in files:
-            self.check_transport_contents(temp_transport.get(f).read(),
+            self.check_transport_contents(temp_transport.get_bytes(f),
                                           t, f)
         del temp_transport
 
@@ -823,9 +831,9 @@
             return
 
         t.put_bytes('a', 'a little bit of text\n')
-        self.failUnless(t.has('a'))
+        self.assertTrue(t.has('a'))
         t.delete('a')
-        self.failIf(t.has('a'))
+        self.assertFalse(t.has('a'))
 
         self.assertRaises(NoSuchFile, t.delete, 'a')
 
@@ -837,9 +845,9 @@
         t.delete_multi(['a', 'c'])
         self.assertEqual([False, True, False],
                 list(t.has_multi(['a', 'b', 'c'])))
-        self.failIf(t.has('a'))
-        self.failUnless(t.has('b'))
-        self.failIf(t.has('c'))
+        self.assertFalse(t.has('a'))
+        self.assertTrue(t.has('b'))
+        self.assertFalse(t.has('c'))
 
         self.assertRaises(NoSuchFile,
                 t.delete_multi, ['a', 'b', 'c'])
@@ -906,7 +914,7 @@
         t.mkdir('foo-baz')
         t.rmdir('foo')
         self.assertRaises((NoSuchFile, PathError), t.rmdir, 'foo')
-        self.failUnless(t.has('foo-bar'))
+        self.assertTrue(t.has('foo-bar'))
 
     def test_rename_dir_succeeds(self):
         t = self.get_transport()
@@ -995,8 +1003,8 @@
         self.assertEquals([True, False], list(t.has_multi(['a', 'b'])))
 
         t.move('a', 'b')
-        self.failUnless(t.has('b'))
-        self.failIf(t.has('a'))
+        self.assertTrue(t.has('b'))
+        self.assertFalse(t.has('a'))
 
         self.check_transport_contents('a first file\n', t, 'b')
         self.assertEquals([False, True], list(t.has_multi(['a', 'b'])))
@@ -1004,7 +1012,7 @@
         # Overwrite a file
         t.put_bytes('c', 'c this file\n')
         t.move('c', 'b')
-        self.failIf(t.has('c'))
+        self.assertFalse(t.has('c'))
         self.check_transport_contents('c this file\n', t, 'b')
 
         # TODO: Try to write a test for atomicity
@@ -1042,7 +1050,7 @@
         except NotImplementedError:
             raise TestSkipped("Transport %s has no bogus URL support." %
                               self._server.__class__)
-        t = get_transport(url)
+        t = _mod_transport.get_transport_from_url(url)
         self.assertRaises((ConnectionError, NoSuchFile), t.get, '.bzr/branch')
 
     def test_stat(self):
@@ -1064,10 +1072,10 @@
         for path, size in zip(paths, sizes):
             st = t.stat(path)
             if path.endswith('/'):
-                self.failUnless(S_ISDIR(st.st_mode))
+                self.assertTrue(S_ISDIR(st.st_mode))
                 # directory sizes are meaningless
             else:
-                self.failUnless(S_ISREG(st.st_mode))
+                self.assertTrue(S_ISREG(st.st_mode))
                 self.assertEqual(size, st.st_size)
 
         remote_stats = list(t.stat_multi(paths))
@@ -1080,8 +1088,8 @@
         self.assertListRaises(NoSuchFile, t.stat_multi, iter(['a', 'c', 'd']))
         self.build_tree(['subdir/', 'subdir/file'], transport=t)
         subdir = t.clone('subdir')
-        subdir.stat('./file')
-        subdir.stat('.')
+        st = subdir.stat('./file')
+        st = subdir.stat('.')
 
     def test_hardlink(self):
         from stat import ST_NLINK
@@ -1096,11 +1104,11 @@
         try:
             t.hardlink(source_name, link_name)
 
-            self.failUnless(t.has(source_name))
-            self.failUnless(t.has(link_name))
+            self.assertTrue(t.has(source_name))
+            self.assertTrue(t.has(link_name))
 
             st = t.stat(link_name)
-            self.failUnlessEqual(st[ST_NLINK], 2)
+            self.assertEqual(st[ST_NLINK], 2)
         except TransportNotPossible:
             raise TestSkipped("Transport %s does not support hardlinks." %
                               self._server.__class__)
@@ -1118,17 +1126,17 @@
         try:
             t.symlink(source_name, link_name)
 
-            self.failUnless(t.has(source_name))
-            self.failUnless(t.has(link_name))
+            self.assertTrue(t.has(source_name))
+            self.assertTrue(t.has(link_name))
 
             st = t.stat(link_name)
-            self.failUnless(S_ISLNK(st.st_mode),
+            self.assertTrue(S_ISLNK(st.st_mode),
                 "expected symlink, got mode %o" % st.st_mode)
         except TransportNotPossible:
             raise TestSkipped("Transport %s does not support symlinks." %
                               self._server.__class__)
         except IOError:
-            raise tests.KnownFailure("Paramiko fails to create symlinks during tests")
+            self.knownFailure("Paramiko fails to create symlinks during tests")
 
     def test_list_dir(self):
         # TODO: Test list_dir, just try once, and if it throws, stop testing
@@ -1198,11 +1206,11 @@
             raise TestSkipped("not a connected transport")
 
         t2 = t1.clone('subdir')
-        self.assertEquals(t1._scheme, t2._scheme)
-        self.assertEquals(t1._user, t2._user)
-        self.assertEquals(t1._password, t2._password)
-        self.assertEquals(t1._host, t2._host)
-        self.assertEquals(t1._port, t2._port)
+        self.assertEquals(t1._parsed_url.scheme, t2._parsed_url.scheme)
+        self.assertEquals(t1._parsed_url.user, t2._parsed_url.user)
+        self.assertEquals(t1._parsed_url.password, t2._parsed_url.password)
+        self.assertEquals(t1._parsed_url.host, t2._parsed_url.host)
+        self.assertEquals(t1._parsed_url.port, t2._parsed_url.port)
 
     def test__reuse_for(self):
         t = self.get_transport()
@@ -1215,21 +1223,21 @@
 
             Only the parameters different from None will be changed.
             """
-            if scheme   is None: scheme   = t._scheme
-            if user     is None: user     = t._user
-            if password is None: password = t._password
-            if user     is None: user     = t._user
-            if host     is None: host     = t._host
-            if port     is None: port     = t._port
-            if path     is None: path     = t._path
-            return t._unsplit_url(scheme, user, password, host, port, path)
+            if scheme   is None: scheme   = t._parsed_url.scheme
+            if user     is None: user     = t._parsed_url.user
+            if password is None: password = t._parsed_url.password
+            if user     is None: user     = t._parsed_url.user
+            if host     is None: host     = t._parsed_url.host
+            if port     is None: port     = t._parsed_url.port
+            if path     is None: path     = t._parsed_url.path
+            return str(urlutils.URL(scheme, user, password, host, port, path))
 
-        if t._scheme == 'ftp':
+        if t._parsed_url.scheme == 'ftp':
             scheme = 'sftp'
         else:
             scheme = 'ftp'
         self.assertIsNot(t, t._reuse_for(new_url(scheme=scheme)))
-        if t._user == 'me':
+        if t._parsed_url.user == 'me':
             user = 'you'
         else:
             user = 'me'
@@ -1246,8 +1254,8 @@
         #   (they may be typed by the user when prompted for example)
         self.assertIs(t, t._reuse_for(new_url(password='from space')))
         # We will not connect, we can use a invalid host
-        self.assertIsNot(t, t._reuse_for(new_url(host=t._host + 'bar')))
-        if t._port == 1234:
+        self.assertIsNot(t, t._reuse_for(new_url(host=t._parsed_url.host + 'bar')))
+        if t._parsed_url.port == 1234:
             port = 4321
         else:
             port = 1234
@@ -1266,7 +1274,7 @@
         self.assertIs(t._get_connection(), c._get_connection())
 
         # Temporary failure, we need to create a new dummy connection
-        new_connection = object()
+        new_connection = None
         t._set_connection(new_connection)
         # Check that both transports use the same connection
         self.assertIs(new_connection, t._get_connection())
@@ -1294,32 +1302,32 @@
 
         self.build_tree(['a', 'b/', 'b/c'], transport=t1)
 
-        self.failUnless(t1.has('a'))
-        self.failUnless(t1.has('b/c'))
-        self.failIf(t1.has('c'))
+        self.assertTrue(t1.has('a'))
+        self.assertTrue(t1.has('b/c'))
+        self.assertFalse(t1.has('c'))
 
         t2 = t1.clone('b')
         self.assertEqual(t1.base + 'b/', t2.base)
 
-        self.failUnless(t2.has('c'))
-        self.failIf(t2.has('a'))
+        self.assertTrue(t2.has('c'))
+        self.assertFalse(t2.has('a'))
 
         t3 = t2.clone('..')
-        self.failUnless(t3.has('a'))
-        self.failIf(t3.has('c'))
+        self.assertTrue(t3.has('a'))
+        self.assertFalse(t3.has('c'))
 
-        self.failIf(t1.has('b/d'))
-        self.failIf(t2.has('d'))
-        self.failIf(t3.has('b/d'))
+        self.assertFalse(t1.has('b/d'))
+        self.assertFalse(t2.has('d'))
+        self.assertFalse(t3.has('b/d'))
 
         if t1.is_readonly():
             self.build_tree_contents([('b/d', 'newfile\n')])
         else:
             t2.put_bytes('d', 'newfile\n')
 
-        self.failUnless(t1.has('b/d'))
-        self.failUnless(t2.has('d'))
-        self.failUnless(t3.has('b/d'))
+        self.assertTrue(t1.has('b/d'))
+        self.assertTrue(t2.has('d'))
+        self.assertTrue(t3.has('b/d'))
 
     def test_clone_to_root(self):
         orig_transport = self.get_transport()
@@ -1399,6 +1407,7 @@
         self.assertEqual(transport.clone("/").abspath('foo'),
                          transport.abspath("/foo"))
 
+    # GZ 2011-01-26: Test in per_transport but not using self.get_transport?
     def test_win32_abspath(self):
         # Note: we tried to set sys.platform='win32' so we could test on
         # other platforms too, but then osutils does platform specific
@@ -1409,12 +1418,12 @@
 
         # smoke test for abspath on win32.
         # a transport based on 'file:///' never fully qualifies the drive.
-        transport = get_transport("file:///")
-        self.failUnlessEqual(transport.abspath("/"), "file:///")
+        transport = _mod_transport.get_transport_from_url("file:///")
+        self.assertEqual(transport.abspath("/"), "file:///")
 
         # but a transport that starts with a drive spec must keep it.
-        transport = get_transport("file:///C:/")
-        self.failUnlessEqual(transport.abspath("/"), "file:///C:/")
+        transport = _mod_transport.get_transport_from_url("file:///C:/")
+        self.assertEqual(transport.abspath("/"), "file:///C:/")
 
     def test_local_abspath(self):
         transport = self.get_transport()
@@ -1546,7 +1555,7 @@
 
         no_unicode_support = getattr(self._server, 'no_unicode_support', False)
         if no_unicode_support:
-            raise tests.KnownFailure("test server cannot handle unicode paths")
+            self.knownFailure("test server cannot handle unicode paths")
 
         try:
             self.build_tree(files, transport=t, line_endings='binary')
@@ -1617,7 +1626,7 @@
     def test_readv(self):
         transport = self.get_transport()
         if transport.is_readonly():
-            file('a', 'w').write('0123456789')
+            with file('a', 'w') as f: f.write('0123456789')
         else:
             transport.put_bytes('a', '0123456789')
 
@@ -1633,7 +1642,7 @@
     def test_readv_out_of_order(self):
         transport = self.get_transport()
         if transport.is_readonly():
-            file('a', 'w').write('0123456789')
+            with file('a', 'w') as f: f.write('0123456789')
         else:
             transport.put_bytes('a', '01234567890')
 
@@ -1711,7 +1720,7 @@
         transport = self.get_transport()
         # test from observed failure case.
         if transport.is_readonly():
-            file('a', 'w').write('a'*1024*1024)
+            with file('a', 'w') as f: f.write('a'*1024*1024)
         else:
             transport.put_bytes('a', 'a'*1024*1024)
         broken_vector = [(465219, 800), (225221, 800), (445548, 800),
@@ -1751,7 +1760,7 @@
     def test_readv_short_read(self):
         transport = self.get_transport()
         if transport.is_readonly():
-            file('a', 'w').write('0123456789')
+            with file('a', 'w') as f: f.write('0123456789')
         else:
             transport.put_bytes('a', '01234567890')
 
@@ -1767,6 +1776,21 @@
         self.assertListRaises((errors.ShortReadvError, errors.InvalidRange),
                               transport.readv, 'a', [(12,2)])
 
+    def test_no_segment_parameters(self):
+        """Segment parameters should be stripped and stored in
+        transport.segment_parameters."""
+        transport = self.get_transport("foo")
+        self.assertEquals({}, transport.get_segment_parameters())
+
+    def test_segment_parameters(self):
+        """Segment parameters should be stripped and stored in
+        transport.get_segment_parameters()."""
+        base_url = self._server.get_url()
+        parameters = {"key1": "val1", "key2": "val2"}
+        url = urlutils.join_segment_parameters(base_url, parameters)
+        transport = _mod_transport.get_transport_from_url(url)
+        self.assertEquals(parameters, transport.get_segment_parameters())
+
     def test_stat_symlink(self):
         # if a transport points directly to a symlink (and supports symlinks
         # at all) you can tell this.  helps with bug 32669.

=== modified file 'bzrlib/tests/per_tree/__init__.py'
--- bzrlib/tests/per_tree/__init__.py	2010-05-06 23:41:35 +0000
+++ bzrlib/tests/per_tree/__init__.py	2011-06-14 01:26:41 +0000
@@ -27,23 +27,21 @@
 
 from bzrlib import (
     errors,
-    osutils,
-    progress,
     tests,
     transform,
     )
-from bzrlib.tests.per_bzrdir.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.tests.per_controldir.test_controldir import TestCaseWithControlDir
 from bzrlib.tests.per_workingtree import (
     make_scenarios as wt_make_scenarios,
     make_scenario as wt_make_scenario,
     )
-from bzrlib.revision import NULL_REVISION
 from bzrlib.revisiontree import RevisionTree
 from bzrlib.transform import TransformPreview
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.workingtree import (
-    WorkingTreeFormat,
-    WorkingTreeFormat3,
-    _legacy_formats,
+    format_registry,
     )
 from bzrlib.workingtree_4 import (
     DirStateRevisionTree,
@@ -98,7 +96,7 @@
         self.assertIsInstance(tree, RevisionTree)
 
 
-class TestCaseWithTree(TestCaseWithBzrDir):
+class TestCaseWithTree(TestCaseWithControlDir):
 
     def make_branch_and_tree(self, relpath):
         made_control = self.make_bzrdir(relpath, format=
@@ -247,7 +245,7 @@
         note that the order of the paths and fileids is deliberately
         mismatched to ensure that the result order is path based.
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('.')
         paths = ['0file',
             '1top-dir/',
@@ -279,7 +277,7 @@
 
     def _create_tree_with_utf8(self, tree):
         """Generate a tree with a utf8 revision and unicode paths."""
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # We avoid combining characters in file names here, normalization
         # checks (as performed by some file systems (OSX) are outside the scope
         # of these tests).  We use the euro sign \N{Euro Sign} or \u20ac in
@@ -337,7 +335,7 @@
         # for working tree format tests, preserve the tree
         scenario[1]["_workingtree_to_test_tree"] = return_parameter
     # add RevisionTree scenario
-    workingtree_format = WorkingTreeFormat._default_format
+    workingtree_format = format_registry.get_default()
     scenarios.append((RevisionTree.__name__,
         create_tree_scenario(transport_server, transport_readonly_server,
         workingtree_format, revision_tree_from_workingtree,)))
@@ -379,12 +377,14 @@
 def load_tests(standard_tests, module, loader):
     per_tree_mod_names = [
         'annotate_iter',
+        'export',
         'get_file_mtime',
         'get_file_with_stat',
         'get_root_id',
         'get_symlink_target',
         'inv',
         'iter_search_rules',
+        'is_executable',
         'list_files',
         'locking',
         'path_content_summary',
@@ -401,6 +401,6 @@
         # None here will cause a readonly decorator to be created
         # by the TestCaseWithTransport.get_readonly_transport method.
         None,
-        WorkingTreeFormat._formats.values() + _legacy_formats)
+        format_registry._get_all())
     # add the tests for the sub modules
     return tests.multiply_tests(submod_tests, scenarios, standard_tests)

=== added file 'bzrlib/tests/per_tree/test_export.py'
--- bzrlib/tests/per_tree/test_export.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_tree/test_export.py	2011-06-14 01:26:41 +0000
@@ -0,0 +1,98 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import os
+import tarfile
+import zipfile
+
+from bzrlib.export import export
+from bzrlib import osutils
+from bzrlib import tests
+from bzrlib.tests.per_tree import TestCaseWithTree
+from bzrlib.tests import (
+    features,
+    )
+
+
+class ExportTest(object):
+
+    def prepare_export(self):
+        work_a = self.make_branch_and_tree('wta')
+        self.build_tree_contents(
+            [('wta/file', 'a\nb\nc\nd\n'), ('wta/dir', '')])
+        work_a.add('file', 'file-id')
+        work_a.add('dir', 'dir-id')
+        work_a.commit('add file')
+        tree_a = self.workingtree_to_test_tree(work_a)
+        export(tree_a, 'output', self.exporter)
+
+    def prepare_symlink_export(self):
+        self.requireFeature(features.SymlinkFeature)
+        work_a = self.make_branch_and_tree('wta')
+        os.symlink('target', 'wta/link')
+        work_a.add('link', 'link-id')
+        work_a.commit('add link')
+        tree_a = self.workingtree_to_test_tree(work_a)
+        export(tree_a, 'output', self.exporter)
+
+    def test_export(self):
+        self.prepare_export()
+        names = self.get_export_names()
+        self.assertIn('output/file', names)
+        self.assertIn('output/dir', names)
+
+    def test_export_symlink(self):
+        self.prepare_symlink_export()
+        names = self.get_export_names()
+        self.assertIn('output/link', names)
+
+
+class TestTar(ExportTest, TestCaseWithTree):
+
+    exporter = 'tar'
+
+    def get_export_names(self):
+        tf = tarfile.open('output')
+        try:
+            return tf.getnames()
+        finally:
+            tf.close()
+
+
+class TestZip(ExportTest, TestCaseWithTree):
+
+    exporter = 'zip'
+
+    def get_export_names(self):
+        zf = zipfile.ZipFile('output')
+        try:
+            return zf.namelist()
+        finally:
+            zf.close()
+
+    def test_export_symlink(self):
+        self.prepare_symlink_export()
+        names = self.get_export_names()
+        self.assertIn('output/link.lnk', names)
+
+
+class TestDir(ExportTest, TestCaseWithTree):
+
+    exporter = 'dir'
+
+    def get_export_names(self):
+        return [osutils.pathjoin('output', name)
+                for name in os.listdir('output')]

=== modified file 'bzrlib/tests/per_tree/test_get_file_mtime.py'
--- bzrlib/tests/per_tree/test_get_file_mtime.py	2010-03-17 13:03:38 +0000
+++ bzrlib/tests/per_tree/test_get_file_mtime.py	2011-09-21 00:30:44 +0000
@@ -18,6 +18,8 @@
 
 import time
 
+from bzrlib import errors
+
 from bzrlib.tests.per_tree import TestCaseWithTree
 
 
@@ -38,7 +40,14 @@
         # file, working trees return the on-disk time.
         mtime_file_id = tree.get_file_mtime(file_id='one-id')
         self.assertIsInstance(mtime_file_id, (float, int))
-        self.failUnless(now - 5 < mtime_file_id < now + 5,
+        self.assertTrue(now - 5 < mtime_file_id < now + 5,
                         'now: %f, mtime_file_id: %f' % (now, mtime_file_id ))
         mtime_path = tree.get_file_mtime(file_id='one-id', path='one')
         self.assertEqual(mtime_file_id, mtime_path)
+
+    def test_nonexistant(self):
+        tree = self.get_basic_tree()
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertRaises(errors.NoSuchId,
+            tree.get_file_mtime, file_id='unexistant')

=== modified file 'bzrlib/tests/per_tree/test_get_symlink_target.py'
--- bzrlib/tests/per_tree/test_get_symlink_target.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_tree/test_get_symlink_target.py	2011-06-14 01:26:41 +0000
@@ -19,17 +19,19 @@
 import os
 
 from bzrlib import (
-    errors,
     osutils,
     tests,
     )
 from bzrlib.tests import per_tree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestGetSymlinkTarget(per_tree.TestCaseWithTree):
 
     def get_tree_with_symlinks(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink('foo', 'tree/link')
         os.symlink('../bar', 'tree/rel_link')
@@ -46,10 +48,11 @@
         self.assertEqual('foo', tree.get_symlink_target('link-id'))
         self.assertEqual('../bar', tree.get_symlink_target('rel-link-id'))
         self.assertEqual('/baz/bing', tree.get_symlink_target('abs-link-id'))
+        self.assertEqual('foo', tree.get_symlink_target('link-id', 'link'))
 
     def test_get_unicode_symlink_target(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tree')
         target = u'targ\N{Euro Sign}t'
         os.symlink(target,  u'tree/\u03b2_link'.encode(osutils._fs_enc))

=== modified file 'bzrlib/tests/per_tree/test_inv.py'
--- bzrlib/tests/per_tree/test_inv.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/per_tree/test_inv.py	2011-06-14 01:26:41 +0000
@@ -17,64 +17,29 @@
 """Tests for interface conformance of inventories of trees."""
 
 
-from cStringIO import StringIO
-import os
-
 from bzrlib import (
     tests,
     )
 from bzrlib.tests import (
-    features,
     per_tree,
     )
 from bzrlib.mutabletree import MutableTree
-from bzrlib.tests import SymlinkFeature, TestSkipped
+from bzrlib.tests import TestSkipped
+from bzrlib.tree import InventoryTree
 from bzrlib.transform import _PreviewTree
 from bzrlib.uncommit import uncommit
+from bzrlib.tests import (
+    features,
+    )
 
 
 def get_entry(tree, file_id):
     return tree.iter_entries_by_dir([file_id]).next()[1]
 
 
-class TestPreviousHeads(per_tree.TestCaseWithTree):
-
-    def setUp(self):
-        # we want several inventories, that respectively
-        # give use the following scenarios:
-        # A) fileid not in any inventory (A),
-        # B) fileid present in one inventory (B) and (A,B)
-        # C) fileid present in two inventories, and they
-        #   are not mutual descendents (B, C)
-        # D) fileid present in two inventories and one is
-        #   a descendent of the other. (B, D)
-        super(TestPreviousHeads, self).setUp()
-        self.wt = self.make_branch_and_tree('.')
-        self.branch = self.wt.branch
-        self.build_tree(['file'])
-        self.wt.commit('new branch', allow_pointless=True, rev_id='A')
-        self.inv_A = self.branch.repository.get_inventory('A')
-        self.wt.add(['file'], ['fileid'])
-        self.wt.commit('add file', rev_id='B')
-        self.inv_B = self.branch.repository.get_inventory('B')
-        uncommit(self.branch, tree=self.wt)
-        self.assertEqual(self.branch.revision_history(), ['A'])
-        self.wt.commit('another add of file', rev_id='C')
-        self.inv_C = self.branch.repository.get_inventory('C')
-        self.wt.add_parent_tree_id('B')
-        self.wt.commit('merge in B', rev_id='D')
-        self.inv_D = self.branch.repository.get_inventory('D')
-        self.tree = self.workingtree_to_test_tree(self.wt)
-        self.tree.lock_read()
-        self.addCleanup(self.tree.unlock)
-        self.file_active = get_entry(self.tree, 'fileid')
-
-    # TODO: test two inventories with the same file revision
-
-
 class TestInventoryWithSymlinks(per_tree.TestCaseWithTree):
 
-    _test_needs_features = [tests.SymlinkFeature]
+    _test_needs_features = [features.SymlinkFeature]
 
     def setUp(self):
         per_tree.TestCaseWithTree.setUp(self)
@@ -140,32 +105,49 @@
 
     def test_canonical_path(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('dir/file',
                          work_tree.get_canonical_inventory_path('Dir/File'))
 
     def test_canonical_path_before_commit(self):
         work_tree = self._make_canonical_test_tree(False)
-        # note: not committed.
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")        # note: not committed.
         self.assertEqual('dir/file',
                          work_tree.get_canonical_inventory_path('Dir/File'))
 
     def test_canonical_path_dir(self):
         # check it works when asked for just the directory portion.
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('dir', work_tree.get_canonical_inventory_path('Dir'))
 
     def test_canonical_path_root(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('', work_tree.get_canonical_inventory_path(''))
         self.assertEqual('/', work_tree.get_canonical_inventory_path('/'))
 
     def test_canonical_path_invalid_all(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('foo/bar',
                          work_tree.get_canonical_inventory_path('foo/bar'))
 
     def test_canonical_invalid_child(self):
         work_tree = self._make_canonical_test_tree()
+        if not isinstance(work_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         self.assertEqual('dir/None',
                          work_tree.get_canonical_inventory_path('Dir/None'))
 
@@ -174,12 +156,15 @@
         # some of the trees we want to use can only exist on a disk, not in
         # memory - therefore we can only test this if the filesystem is
         # case-sensitive.
-        self.requireFeature(tests.case_sensitive_filesystem_feature)
+        self.requireFeature(features.case_sensitive_filesystem_feature)
         work_tree = self.make_branch_and_tree('.')
         self.build_tree(['test/', 'test/file', 'Test'])
         work_tree.add(['test/', 'test/file', 'Test'])
 
         test_tree = self._convert_tree(work_tree)
+        if not isinstance(test_tree, InventoryTree):
+            raise tests.TestNotApplicable(
+                "test not applicable on non-inventory tests")
         test_tree.lock_read()
         self.addCleanup(test_tree.unlock)
 

=== added file 'bzrlib/tests/per_tree/test_is_executable.py'
--- bzrlib/tests/per_tree/test_is_executable.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_tree/test_is_executable.py	2011-06-14 01:26:41 +0000
@@ -0,0 +1,39 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from bzrlib.tests import (
+    per_tree,
+    )
+from bzrlib.tests.features import (
+    SymlinkFeature,
+    )
+
+
+class TestIsExecutable(per_tree.TestCaseWithTree):
+
+    def test_is_executable_dir(self):
+        tree = self.get_tree_with_subdirs_and_all_supported_content_types(
+            False)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEqual(False, tree.is_executable('1top-dir'))
+
+    def test_is_executable_symlink(self):
+        self.requireFeature(SymlinkFeature)
+        tree = self.get_tree_with_subdirs_and_all_content_types()
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEqual(False, tree.is_executable('symlink'))

=== modified file 'bzrlib/tests/per_tree/test_path_content_summary.py'
--- bzrlib/tests/per_tree/test_path_content_summary.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_tree/test_path_content_summary.py	2011-06-14 01:26:41 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Test that all Tree's implement path_content_summary."""
+"""Test that all Trees implement path_content_summary."""
 
 import os
 
@@ -24,7 +24,13 @@
     transform,
     )
 
-from bzrlib.tests import per_tree
+from bzrlib.tests import (
+    features,
+    per_tree,
+    )
+from bzrlib.tests.features import (
+    SymlinkFeature,
+    )
 
 
 class TestPathContentSummary(per_tree.TestCaseWithTree):
@@ -48,7 +54,7 @@
             self.fail("invalid size in summary: %r" % (returned_size,))
 
     def test_symlink_content_summary(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(SymlinkFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink('target', 'tree/path')
         tree.add(['path'])
@@ -56,8 +62,8 @@
         self.assertEqual(('symlink', None, None, 'target'), summary)
 
     def test_unicode_symlink_content_summary(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink('target', u'tree/\u03b2-path'.encode(osutils._fs_enc))
         tree.add([u'\u03b2-path'])
@@ -65,8 +71,8 @@
         self.assertEqual(('symlink', None, None, 'target'), summary)
 
     def test_unicode_symlink_target_summary(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('tree')
         os.symlink(u'tree/\u03b2-path'.encode(osutils._fs_enc), 'tree/link')
         tree.add(['link'])
@@ -144,10 +150,10 @@
 
     def test_tree_content_summary(self):
         tree = self.make_branch_and_tree('tree')
-        subtree = self.make_branch_and_tree('tree/path')
-        tree.add(['path'])
         if not tree.branch.repository._format.supports_tree_reference:
             raise tests.TestNotApplicable("Tree references not supported.")
+        subtree = self.make_branch_and_tree('tree/path')
+        tree.add(['path'])
         summary = self._convert_tree(tree).path_content_summary('path')
         self.assertEqual(4, len(summary))
         self.assertEqual('tree-reference', summary[0])

=== modified file 'bzrlib/tests/per_tree/test_test_trees.py'
--- bzrlib/tests/per_tree/test_test_trees.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_tree/test_test_trees.py	2011-09-04 21:41:05 +0000
@@ -16,8 +16,10 @@
 
 """Tests for the test trees used by the per_tree tests."""
 
-from bzrlib import tests
 from bzrlib.tests import per_tree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestTreeShapes(per_tree.TestCaseWithTree):
@@ -30,7 +32,7 @@
         self.assertEqual([], tree.get_parent_ids())
         self.assertEqual([], tree.conflicts())
         self.assertEqual([], list(tree.unknowns()))
-        self.assertEqual(['empty-root-id'], list(iter(tree)))
+        self.assertEqual(['empty-root-id'], list(tree.all_file_ids()))
         self.assertEqual(
             [('', 'empty-root-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
@@ -46,7 +48,7 @@
         # __iter__ has no strongly defined order
         self.assertEqual(
             set(['root-id', 'a-id', 'b-id', 'c-id']),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         self.assertEqual(
             [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
@@ -64,7 +66,7 @@
         # __iter__ has no strongly defined order
         self.assertEqual(
             set(['root-id', 'a-id', 'b-id', 'c-id']),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         self.assertEqual(
             [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
@@ -82,7 +84,7 @@
         # __iter__ has no strongly defined order
         self.assertEqual(
             set(['root-id', 'a-id', 'b-id', 'c-id']),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         self.assertEqual(
             [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
@@ -100,7 +102,7 @@
         # __iter__ has no strongly defined order
         self.assertEqual(
             set(['root-id', 'a-id', 'b-id', 'c-id']),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         self.assertEqual(
             [('', 'root-id'), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
@@ -118,7 +120,7 @@
         # __iter__ has no strongly defined order
         self.assertEqual(
             set(['root-id', 'a-id', 'b-id', 'c-id']),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         self.assertEqual(
             [('', 'root-id'), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
@@ -136,7 +138,7 @@
         # __iter__ has no strongly defined order
         self.assertEqual(
             set(['root-id', 'a-id', 'b-id', 'c-id']),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         self.assertEqual(
             [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('e', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
@@ -147,7 +149,7 @@
         # currently this test tree requires unicode. It might be good
         # to have it simply stop having the single unicode file in it
         # when dealing with a non-unicode filesystem.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.get_tree_with_subdirs_and_all_content_types()
         tree.lock_read()
         self.addCleanup(tree.unlock)
@@ -165,7 +167,7 @@
                  u'0utf\u1234file'.encode('utf8'),
                 'symlink',
                  ]),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         # note that the order of the paths and fileids is deliberately
         # mismatched to ensure that the result order is path based.
         self.assertEqual(
@@ -198,7 +200,7 @@
                 '0dir-in-1topdir',
                  u'0utf\u1234file'.encode('utf8'),
                  ]),
-            set(iter(tree)))
+            set(tree.all_file_ids()))
         # note that the order of the paths and fileids is deliberately
         # mismatched to ensure that the result order is path based.
         self.assertEqual(

=== modified file 'bzrlib/tests/per_tree/test_tree.py'
--- bzrlib/tests/per_tree/test_tree.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_tree/test_tree.py	2011-08-29 19:06:57 +0000
@@ -24,6 +24,10 @@
     )
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_tree import TestCaseWithTree
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    )
+
 
 class TestAnnotate(TestCaseWithTree):
 
@@ -94,7 +98,8 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         path = tree.id2path('sub-root')
-        self.assertEqual('sub-1', tree.get_reference_revision('sub-root', path))
+        self.assertEqual('sub-1',
+            tree.get_reference_revision('sub-root', path))
 
     def test_iter_references(self):
         tree = self.create_nested()
@@ -257,8 +262,14 @@
         tree = self._convert_tree(work_tree)
         tree.lock_read()
         self.addCleanup(tree.unlock)
-        self.assertTrue('file-id' in tree)
-        self.assertFalse('dir-id' in tree)
+        self.assertTrue(
+            self.applyDeprecated(
+                deprecated_in((2, 4, 0)),
+                tree.__contains__, 'file-id'))
+        self.assertFalse(
+            self.applyDeprecated(
+                deprecated_in((2, 4, 0)),
+                tree.__contains__, 'dir-id'))
 
 
 class TestExtras(TestCaseWithTree):
@@ -292,3 +303,31 @@
         self.addCleanup(tree.unlock)
         expected = osutils.sha_strings('file content')
         self.assertEqual(expected, tree.get_file_sha1('file-id'))
+
+
+class TestGetFileVerifier(TestCaseWithTree):
+
+    def test_get_file_verifier(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/file1', 'file content'),
+            ('tree/file2', 'file content')])
+        work_tree.add(['file1', 'file2'], ['file-id-1', 'file-id-2'])
+        tree = self._convert_tree(work_tree)
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        (kind, data) = tree.get_file_verifier('file-id-1')
+        self.assertEquals(
+            tree.get_file_verifier('file-id-1'),
+            tree.get_file_verifier('file-id-2'))
+        if kind == "SHA1":
+            expected = osutils.sha_strings('file content')
+            self.assertEqual(expected, data)
+
+
+class TestHasVersionedDirectories(TestCaseWithTree):
+
+    def test_has_versioned_directories(self):
+        work_tree = self.make_branch_and_tree('tree')
+        tree = self._convert_tree(work_tree)
+        self.assertSubset([tree.has_versioned_directories()], (True, False))

=== modified file 'bzrlib/tests/per_uifactory/__init__.py'
--- bzrlib/tests/per_uifactory/__init__.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_uifactory/__init__.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -62,6 +62,17 @@
         self.factory.be_quiet(False)
         self.assertEquals(False, self.factory.is_quiet())
 
+    def test_confirm_action(self):
+        # confirm_action should be answered by every ui factory; even
+        # noninteractive ones should have a reasonable default
+        self._load_responses([True])
+        result = self.factory.confirm_action(u'Break a lock?',
+             'bzr.lock.break.confirm',
+            {})
+        # will be true either because we read it from the input or because
+        # that's the default
+        self.assertEquals(result, True)
+
     def test_note(self):
         self.factory.note("a note to the user")
         self._check_note("a note to the user")
@@ -90,7 +101,7 @@
     def test_transport_activity(self):
         # It doesn't matter what the implementation does, we just want to make
         # sure the interface is there
-        t = transport.get_transport('memory:///')
+        t = transport.get_transport_from_url('memory:///')
         self.factory.report_transport_activity(t, 1000, 'write')
         self.factory.report_transport_activity(t, 2000, 'read')
         self.factory.report_transport_activity(t, 4000, None)
@@ -101,7 +112,7 @@
 
     def test_no_transport_activity(self):
         # No activity to report
-        t = transport.get_transport('memory:///')
+        t = transport.get_transport_from_url('memory:///')
         self.factory.log_transport_activity(display=True)
         self._check_log_transport_activity_display_no_bytes()
 
@@ -150,6 +161,11 @@
         # Without a TTY, we shouldn't display anything
         self.assertEqual('', self.stderr.getvalue())
 
+    def _load_responses(self, responses):
+        self.factory.stdin.seek(0)
+        self.factory.stdin.writelines([(r and "y\n" or "n\n") for r in responses])
+        self.factory.stdin.seek(0)
+
 
 class TestTTYTextUIFactory(TestTextUIFactory):
 
@@ -173,7 +189,7 @@
 
         # Remove 'TERM' == 'dumb' which causes us to *not* treat output as a
         # real terminal, even though isatty returns True
-        self._captureVar('TERM', None)
+        self.overrideEnv('TERM', None)
         self.stderr = TTYStringIO()
         self.stdout = TTYStringIO()
         self.factory = ui.text.TextUIFactory(self.stdin, self.stdout,
@@ -222,6 +238,9 @@
     def _check_log_transport_activity_display_no_bytes(self):
         pass
 
+    def _load_responses(self, responses):
+        pass
+
 
 class TestCannedInputUIFactory(tests.TestCase, UIFactoryTestMixin):
     # discards output, reads input from variables
@@ -250,3 +269,6 @@
 
     def _check_log_transport_activity_display_no_bytes(self):
         pass
+
+    def _load_responses(self, responses):
+        self.factory.responses.extend(responses)

=== modified file 'bzrlib/tests/per_versionedfile.py'
--- bzrlib/tests/per_versionedfile.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_versionedfile.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # Authors:
 #   Johan Rydberg <jrydberg@gnu.org>
@@ -21,6 +21,7 @@
 # TODO: might be nice to create a versionedfile with some type of corruption
 # considered typical and check that it can be detected/corrected.
 
+from gzip import GzipFile
 from itertools import chain, izip
 from StringIO import StringIO
 
@@ -37,29 +38,20 @@
 from bzrlib.errors import (
                            RevisionNotPresent,
                            RevisionAlreadyPresent,
-                           WeaveParentMismatch
                            )
 from bzrlib.knit import (
     cleanup_pack_knit,
     make_file_factory,
     make_pack_factory,
-    KnitAnnotateFactory,
-    KnitPlainFactory,
     )
 from bzrlib.tests import (
     TestCase,
     TestCaseWithMemoryTransport,
     TestNotApplicable,
     TestSkipped,
-    condition_isinstance,
-    split_suite_by_condition,
-    multiply_tests,
     )
 from bzrlib.tests.http_utils import TestCaseWithWebserver
-from bzrlib.trace import mutter
 from bzrlib.transport.memory import MemoryTransport
-from bzrlib.tsort import topo_sort
-from bzrlib.tuned_gzip import GzipFile
 import bzrlib.versionedfile as versionedfile
 from bzrlib.versionedfile import (
     ConstantMapper,
@@ -69,101 +61,11 @@
     make_versioned_files_factory,
     )
 from bzrlib.weave import WeaveFile
-from bzrlib.weavefile import read_weave, write_weave
-
-
-def load_tests(standard_tests, module, loader):
-    """Parameterize VersionedFiles tests for different implementations."""
-    to_adapt, result = split_suite_by_condition(
-        standard_tests, condition_isinstance(TestVersionedFiles))
-    # We want to be sure of behaviour for:
-    # weaves prefix layout (weave texts)
-    # individually named weaves (weave inventories)
-    # annotated knits - prefix|hash|hash-escape layout, we test the third only
-    #                   as it is the most complex mapper.
-    # individually named knits
-    # individual no-graph knits in packs (signatures)
-    # individual graph knits in packs (inventories)
-    # individual graph nocompression knits in packs (revisions)
-    # plain text knits in packs (texts)
-    len_one_scenarios = [
-        ('weave-named', {
-            'cleanup':None,
-            'factory':make_versioned_files_factory(WeaveFile,
-                ConstantMapper('inventory')),
-            'graph':True,
-            'key_length':1,
-            'support_partial_insertion': False,
-            }),
-        ('named-knit', {
-            'cleanup':None,
-            'factory':make_file_factory(False, ConstantMapper('revisions')),
-            'graph':True,
-            'key_length':1,
-            'support_partial_insertion': False,
-            }),
-        ('named-nograph-nodelta-knit-pack', {
-            'cleanup':cleanup_pack_knit,
-            'factory':make_pack_factory(False, False, 1),
-            'graph':False,
-            'key_length':1,
-            'support_partial_insertion': False,
-            }),
-        ('named-graph-knit-pack', {
-            'cleanup':cleanup_pack_knit,
-            'factory':make_pack_factory(True, True, 1),
-            'graph':True,
-            'key_length':1,
-            'support_partial_insertion': True,
-            }),
-        ('named-graph-nodelta-knit-pack', {
-            'cleanup':cleanup_pack_knit,
-            'factory':make_pack_factory(True, False, 1),
-            'graph':True,
-            'key_length':1,
-            'support_partial_insertion': False,
-            }),
-        ('groupcompress-nograph', {
-            'cleanup':groupcompress.cleanup_pack_group,
-            'factory':groupcompress.make_pack_factory(False, False, 1),
-            'graph': False,
-            'key_length':1,
-            'support_partial_insertion':False,
-            }),
-        ]
-    len_two_scenarios = [
-        ('weave-prefix', {
-            'cleanup':None,
-            'factory':make_versioned_files_factory(WeaveFile,
-                PrefixMapper()),
-            'graph':True,
-            'key_length':2,
-            'support_partial_insertion': False,
-            }),
-        ('annotated-knit-escape', {
-            'cleanup':None,
-            'factory':make_file_factory(True, HashEscapedPrefixMapper()),
-            'graph':True,
-            'key_length':2,
-            'support_partial_insertion': False,
-            }),
-        ('plain-knit-pack', {
-            'cleanup':cleanup_pack_knit,
-            'factory':make_pack_factory(True, True, 2),
-            'graph':True,
-            'key_length':2,
-            'support_partial_insertion': True,
-            }),
-        ('groupcompress', {
-            'cleanup':groupcompress.cleanup_pack_group,
-            'factory':groupcompress.make_pack_factory(True, False, 1),
-            'graph': True,
-            'key_length':1,
-            'support_partial_insertion':False,
-            }),
-        ]
-    scenarios = len_one_scenarios + len_two_scenarios
-    return multiply_tests(to_adapt, scenarios, result)
+from bzrlib.weavefile import write_weave
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
 
 
 def get_diamond_vf(f, trailing_eol=True, left_only=False):
@@ -849,7 +751,7 @@
         self.assertEquals(('references_ghost', 'line_c\n'), origins[2])
 
     def test_readonly_mode(self):
-        t = transport.get_transport(self.get_url('.'))
+        t = self.get_transport()
         factory = self.get_factory()
         vf = factory('id', t, 0777, create=True, access_mode='w')
         vf = factory('id', t, access_mode='r')
@@ -880,12 +782,12 @@
 class TestWeave(TestCaseWithMemoryTransport, VersionedFileTestMixIn):
 
     def get_file(self, name='foo'):
-        return WeaveFile(name, transport.get_transport(self.get_url('.')),
+        return WeaveFile(name, self.get_transport(),
                          create=True,
                          get_scope=self.get_transaction)
 
     def get_file_corrupted_text(self):
-        w = WeaveFile('foo', transport.get_transport(self.get_url('.')),
+        w = WeaveFile('foo', self.get_transport(),
                       create=True,
                       get_scope=self.get_transaction)
         w.add_lines('v1', [], ['hello\n'])
@@ -921,7 +823,7 @@
         return w
 
     def reopen_file(self, name='foo', create=False):
-        return WeaveFile(name, transport.get_transport(self.get_url('.')),
+        return WeaveFile(name, self.get_transport(),
                          create=create,
                          get_scope=self.get_transaction)
 
@@ -929,7 +831,7 @@
         self.assertRaises(errors.NoSuchFile,
                           WeaveFile,
                           'foo',
-                          transport.get_transport(self.get_url('.')),
+                          self.get_transport(),
                           get_scope=self.get_transaction)
 
     def get_factory(self):
@@ -1002,8 +904,8 @@
         # we should be able to read from http with a versioned file.
         vf = self.get_file()
         # try an empty file access
-        readonly_vf = self.get_factory()('foo', transport.get_transport(
-                self.get_readonly_url('.')))
+        readonly_vf = self.get_factory()('foo',
+            transport.get_transport_from_url(self.get_readonly_url('.')))
         self.assertEqual([], readonly_vf.versions())
 
     def test_readonly_http_works_with_feeling(self):
@@ -1012,8 +914,8 @@
         # now with feeling.
         vf.add_lines('1', [], ['a\n'])
         vf.add_lines('2', ['1'], ['b\n', 'a\n'])
-        readonly_vf = self.get_factory()('foo', transport.get_transport(
-                self.get_readonly_url('.')))
+        readonly_vf = self.get_factory()('foo',
+            transport.get_transport_from_url(self.get_readonly_url('.')))
         self.assertEqual(['1', '2'], vf.versions())
         self.assertEqual(['1', '2'], readonly_vf.versions())
         for version in readonly_vf.versions():
@@ -1023,7 +925,7 @@
 class TestWeaveHTTP(TestCaseWithWebserver, TestReadonlyHttpMixin):
 
     def get_file(self):
-        return WeaveFile('foo', transport.get_transport(self.get_url('.')),
+        return WeaveFile('foo', self.get_transport(),
                          create=True,
                          get_scope=self.get_transaction)
 
@@ -1275,7 +1177,7 @@
 class TestWeaveMerge(TestCaseWithMemoryTransport, MergeCasesMixin):
 
     def get_file(self, name='foo'):
-        return WeaveFile(name, transport.get_transport(self.get_url('.')),
+        return WeaveFile(name, self.get_transport(),
                          create=True)
 
     def log_contents(self, w):
@@ -1475,6 +1377,95 @@
 class TestVersionedFiles(TestCaseWithMemoryTransport):
     """Tests for the multiple-file variant of VersionedFile."""
 
+    # We want to be sure of behaviour for:
+    # weaves prefix layout (weave texts)
+    # individually named weaves (weave inventories)
+    # annotated knits - prefix|hash|hash-escape layout, we test the third only
+    #                   as it is the most complex mapper.
+    # individually named knits
+    # individual no-graph knits in packs (signatures)
+    # individual graph knits in packs (inventories)
+    # individual graph nocompression knits in packs (revisions)
+    # plain text knits in packs (texts)
+    len_one_scenarios = [
+        ('weave-named', {
+            'cleanup':None,
+            'factory':make_versioned_files_factory(WeaveFile,
+                ConstantMapper('inventory')),
+            'graph':True,
+            'key_length':1,
+            'support_partial_insertion': False,
+            }),
+        ('named-knit', {
+            'cleanup':None,
+            'factory':make_file_factory(False, ConstantMapper('revisions')),
+            'graph':True,
+            'key_length':1,
+            'support_partial_insertion': False,
+            }),
+        ('named-nograph-nodelta-knit-pack', {
+            'cleanup':cleanup_pack_knit,
+            'factory':make_pack_factory(False, False, 1),
+            'graph':False,
+            'key_length':1,
+            'support_partial_insertion': False,
+            }),
+        ('named-graph-knit-pack', {
+            'cleanup':cleanup_pack_knit,
+            'factory':make_pack_factory(True, True, 1),
+            'graph':True,
+            'key_length':1,
+            'support_partial_insertion': True,
+            }),
+        ('named-graph-nodelta-knit-pack', {
+            'cleanup':cleanup_pack_knit,
+            'factory':make_pack_factory(True, False, 1),
+            'graph':True,
+            'key_length':1,
+            'support_partial_insertion': False,
+            }),
+        ('groupcompress-nograph', {
+            'cleanup':groupcompress.cleanup_pack_group,
+            'factory':groupcompress.make_pack_factory(False, False, 1),
+            'graph': False,
+            'key_length':1,
+            'support_partial_insertion':False,
+            }),
+        ]
+    len_two_scenarios = [
+        ('weave-prefix', {
+            'cleanup':None,
+            'factory':make_versioned_files_factory(WeaveFile,
+                PrefixMapper()),
+            'graph':True,
+            'key_length':2,
+            'support_partial_insertion': False,
+            }),
+        ('annotated-knit-escape', {
+            'cleanup':None,
+            'factory':make_file_factory(True, HashEscapedPrefixMapper()),
+            'graph':True,
+            'key_length':2,
+            'support_partial_insertion': False,
+            }),
+        ('plain-knit-pack', {
+            'cleanup':cleanup_pack_knit,
+            'factory':make_pack_factory(True, True, 2),
+            'graph':True,
+            'key_length':2,
+            'support_partial_insertion': True,
+            }),
+        ('groupcompress', {
+            'cleanup':groupcompress.cleanup_pack_group,
+            'factory':groupcompress.make_pack_factory(True, False, 1),
+            'graph': True,
+            'key_length':1,
+            'support_partial_insertion':False,
+            }),
+        ]
+
+    scenarios = len_one_scenarios + len_two_scenarios
+
     def get_versionedfiles(self, relpath='files'):
         transport = self.get_transport(relpath)
         if relpath != '.':
@@ -1491,6 +1482,18 @@
         else:
             return ('FileA',) + (suffix,)
 
+    def test_add_fallback_implies_without_fallbacks(self):
+        f = self.get_versionedfiles('files')
+        if getattr(f, 'add_fallback_versioned_files', None) is None:
+            raise TestNotApplicable("%s doesn't support fallbacks"
+                                    % (f.__class__.__name__,))
+        g = self.get_versionedfiles('fallback')
+        key_a = self.get_simple_key('a')
+        g.add_lines(key_a, [], ['\n'])
+        f.add_fallback_versioned_files(g)
+        self.assertTrue(key_a in f.get_parent_map([key_a]))
+        self.assertFalse(key_a in f.without_fallbacks().get_parent_map([key_a]))
+
     def test_add_lines(self):
         f = self.get_versionedfiles()
         key0 = self.get_simple_key('r0')

=== modified file 'bzrlib/tests/per_workingtree/__init__.py'
--- bzrlib/tests/per_workingtree/__init__.py	2010-07-29 11:17:57 +0000
+++ bzrlib/tests/per_workingtree/__init__.py	2011-09-23 12:32:30 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,11 +23,11 @@
 """
 
 from bzrlib import (
-    errors,
+    branchbuilder,
     tests,
     workingtree,
     )
-from bzrlib.tests import per_bzrdir
+from bzrlib.tests import per_controldir
 
 
 def make_scenarios(transport_server, transport_readonly_server, formats):
@@ -50,7 +50,23 @@
         }
 
 
-class TestCaseWithWorkingTree(per_bzrdir.TestCaseWithBzrDir):
+def wt_scenarios():
+    """Returns the scenarios for all registered working trees.
+
+    This can used by plugins that want to define tests against these working
+    trees.
+    """
+    scenarios = make_scenarios(
+        tests.default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        workingtree.format_registry._get_all()
+        )
+    return scenarios
+
+
+class TestCaseWithWorkingTree(per_controldir.TestCaseWithControlDir):
 
     def make_branch_and_tree(self, relpath, format=None):
         made_control = self.make_bzrdir(relpath, format=format)
@@ -58,11 +74,12 @@
         made_control.create_branch()
         return self.workingtree_format.initialize(made_control)
 
-
-def workingtree_formats():
-    """The known working tree formats."""
-    return (workingtree.WorkingTreeFormat._formats.values() +
-        workingtree._legacy_formats)
+    def make_branch_builder(self, relpath, format=None):
+        if format is None:
+            format = self.workingtree_format.get_controldir_for_branch()
+        builder = branchbuilder.BranchBuilder(self.get_transport(relpath),
+                                              format=format)
+        return builder
 
 
 def load_tests(standard_tests, module, loader):
@@ -75,6 +92,7 @@
         'break_lock',
         'changes_from',
         'check',
+        'check_state',
         'content_filters',
         'commit',
         'eol_conversion',
@@ -112,15 +130,16 @@
         'bzrlib.tests.per_workingtree.test_' + name for
         name in test_names]
 
-    scenarios = make_scenarios(
-        tests.default_transport,
-        # None here will cause a readonly decorator to be created
-        # by the TestCaseWithTransport.get_readonly_transport method.
-        None,
-        workingtree_formats()
-        )
+    scenarios = wt_scenarios()
 
     # add the tests for the sub modules
     return tests.multiply_tests(
         loader.loadTestsFromModuleNames(test_workingtree_implementations),
         scenarios, standard_tests)
+
+
+class TestWtScenarios(tests.TestCase):
+
+    def test_protect_wt_scenarios(self):
+        # Just make sure we don't accidentally delete the helper again
+        scenarios = wt_scenarios()

=== modified file 'bzrlib/tests/per_workingtree/test_add.py'
--- bzrlib/tests/per_workingtree/test_add.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_add.py	2011-08-29 21:53:33 +0000
@@ -16,32 +16,20 @@
 
 """Tests for interface conformance of 'WorkingTree.add'"""
 
-import os
-
 from bzrlib import (
     errors,
     inventory,
-    osutils,
     tests,
     )
+from bzrlib.tests.matchers import HasLayout
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
 class TestAdd(TestCaseWithWorkingTree):
 
-    def get_tree_layout(self, tree):
-        """Get the (path, file_id) pairs for the current tree."""
-        tree.lock_read()
-        try:
-            return [(path, ie.file_id) for path, ie
-                    in tree.iter_entries_by_dir()]
-        finally:
-            tree.unlock()
-
     def assertTreeLayout(self, expected, tree):
         """Check that the tree has the correct layout."""
-        actual = self.get_tree_layout(tree)
-        self.assertEqual(expected, actual)
+        self.assertThat(tree, HasLayout(expected))
 
     def test_add_one(self):
         tree = self.make_branch_and_tree('.')
@@ -113,8 +101,8 @@
         tree.add(['dir/subdir/foo'], ['foo-id'])
         root_id = tree.get_root_id()
 
-        self.assertTreeLayout([('', root_id), ('dir', 'dir-id'),
-                               ('dir/subdir', 'subdir-id'),
+        self.assertTreeLayout([('', root_id), ('dir/', 'dir-id'),
+                               ('dir/subdir/', 'subdir-id'),
                                ('dir/subdir/foo', 'foo-id')], tree)
 
     def test_add_multiple(self):
@@ -125,7 +113,7 @@
         root_id = tree.get_root_id()
 
         self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
-                               ('dir', 'dir-id'), ('dir/subdir', 'subdir-id'),
+                               ('dir/', 'dir-id'), ('dir/subdir/', 'subdir-id'),
                                ('dir/subdir/foo', 'foo-id')], tree)
 
     def test_add_invalid(self):
@@ -153,7 +141,7 @@
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         tree.add('')
-        self.assertEqual([tree.path2id('')], list(tree))
+        self.assertEqual([tree.path2id('')], list(tree.all_file_ids()))
         # the root should have been changed to be a new unique root.
         self.assertNotEqual(inventory.ROOT_ID, tree.path2id(''))
         tree.unlock()

=== modified file 'bzrlib/tests/per_workingtree/test_add_reference.py'
--- bzrlib/tests/per_workingtree/test_add_reference.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_add_reference.py	2011-05-06 14:53:40 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,13 +16,9 @@
 
 import os
 
-from bzrlib import errors, tests, workingtree, workingtree_4
+from bzrlib import errors, tests, workingtree
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
-TREES_NOT_SUPPORTING_REFERENCES = (workingtree.WorkingTree2,
-                                   workingtree.WorkingTree3,
-                                   workingtree_4.WorkingTree4)
-
 
 class TestBasisInventory(TestCaseWithWorkingTree):
 
@@ -37,8 +33,9 @@
         return tree, sub_tree
 
     def _references_unsupported(self, tree):
-        if tree.__class__ in TREES_NOT_SUPPORTING_REFERENCES:
-            raise tests.TestSkipped('Tree format does not support references')
+        if not tree.supports_tree_reference():
+            raise tests.TestNotApplicable(
+                'Tree format does not support references')
         else:
             self.fail('%r does not support references but should'
                 % (tree, ))
@@ -57,8 +54,7 @@
         tree.lock_write()
         try:
             self.assertEqual(tree.path2id('sub-tree'), 'sub-tree-root-id')
-            self.assertEqual(tree.inventory['sub-tree-root-id'].kind,
-                             'tree-reference')
+            self.assertEqual(tree.kind('sub-tree-root-id'), 'tree-reference')
             tree.commit('commit reference')
             basis = tree.basis_tree()
             basis.lock_read()

=== modified file 'bzrlib/tests/per_workingtree/test_annotate_iter.py'
--- bzrlib/tests/per_workingtree/test_annotate_iter.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_annotate_iter.py	2011-05-04 21:10:36 +0000
@@ -16,14 +16,6 @@
 
 """Tests for interface conformance of 'WorkingTree.annotate_iter'"""
 
-import os
-
-from bzrlib import (
-    errors,
-    inventory,
-    osutils,
-    tests,
-    )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 

=== modified file 'bzrlib/tests/per_workingtree/test_basis_inventory.py'
--- bzrlib/tests/per_workingtree/test_basis_inventory.py	2009-08-17 06:58:22 +0000
+++ bzrlib/tests/per_workingtree/test_basis_inventory.py	2011-05-04 21:10:36 +0000
@@ -14,13 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-
 from bzrlib.tests import TestNotApplicable
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib.branch import Branch
-from bzrlib import inventory
-from bzrlib.revision import Revision
 import bzrlib.xml6
 
 

=== modified file 'bzrlib/tests/per_workingtree/test_break_lock.py'
--- bzrlib/tests/per_workingtree/test_break_lock.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/per_workingtree/test_break_lock.py	2011-05-04 11:15:37 +0000
@@ -15,9 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
-import os
-
 from bzrlib import (
     errors,
     ui,

=== modified file 'bzrlib/tests/per_workingtree/test_check.py'
--- bzrlib/tests/per_workingtree/test_check.py	2009-08-04 04:47:35 +0000
+++ bzrlib/tests/per_workingtree/test_check.py	2011-09-04 21:47:01 +0000
@@ -16,16 +16,19 @@
 
 """Tests for checking of trees."""
 
-from bzrlib import (
-    tests,
-    )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
+from bzrlib.workingtree import InventoryWorkingTree
+from bzrlib.tests import TestNotApplicable
+
 
 class TestCheck(TestCaseWithWorkingTree):
 
     def test__get_check_refs_new(self):
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable(
+                "_get_check_refs only relevant for inventory working trees")
         self.assertEqual(set([('trees', 'null:')]),
             set(tree._get_check_refs()))
 
@@ -33,6 +36,9 @@
         # with a basis, all current bzr trees cache it and so need the
         # inventory to cross-check.
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable(
+                "_get_check_refs only relevant for inventory working trees")
         revid = tree.commit('first post')
         self.assertEqual(set([('trees', revid)]),
             set(tree._get_check_refs()))
@@ -40,6 +46,9 @@
     def test__check_with_refs(self):
         # _check can be called with a dict of the things required.
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable(
+                "_get_check_refs only relevant for inventory working trees")
         tree.lock_write()
         self.addCleanup(tree.unlock)
         revid = tree.commit('first post')

=== added file 'bzrlib/tests/per_workingtree/test_check_state.py'
--- bzrlib/tests/per_workingtree/test_check_state.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_workingtree/test_check_state.py	2011-04-15 07:01:22 +0000
@@ -0,0 +1,110 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for WorkingTree.check_state."""
+
+from bzrlib import (
+    errors,
+    tests,
+    )
+from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+
+
+
+class TestCaseWithState(TestCaseWithWorkingTree):
+
+    def make_tree_with_broken_dirstate(self, path):
+        tree = self.make_branch_and_tree(path)
+        self.break_dirstate(tree)
+        return tree
+
+    def break_dirstate(self, tree, completely=False):
+        """Write garbage into the dirstate file."""
+        if getattr(tree, 'current_dirstate', None) is None:
+            raise tests.TestNotApplicable(
+                'Only applies to dirstate-based trees')
+        tree.lock_read()
+        try:
+            dirstate = tree.current_dirstate()
+            dirstate_path = dirstate._filename
+            self.assertPathExists(dirstate_path)
+        finally:
+            tree.unlock()
+        # We have to have the tree unlocked at this point, so we can safely
+        # mutate the state file on all platforms.
+        if completely:
+            f = open(dirstate_path, 'wb')
+        else:
+            f = open(dirstate_path, 'ab')
+        try:
+            f.write('garbage-at-end-of-file\n')
+        finally:
+            f.close()
+
+
+class TestCheckState(TestCaseWithState):
+
+    def test_check_state(self):
+        tree = self.make_branch_and_tree('tree')
+        # Everything should be fine with an unmodified tree, no exception
+        # should be raised.
+        tree.check_state()
+
+    def test_check_broken_dirstate(self):
+        tree = self.make_tree_with_broken_dirstate('tree')
+        self.assertRaises(errors.BzrError, tree.check_state)
+
+
+class TestResetState(TestCaseWithState):
+
+    def make_initial_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo', 'tree/dir/', 'tree/dir/bar'])
+        tree.add(['foo', 'dir', 'dir/bar'])
+        tree.commit('initial')
+        return tree
+
+    def test_reset_state_forgets_changes(self):
+        tree = self.make_initial_tree()
+        foo_id = tree.path2id('foo')
+        tree.rename_one('foo', 'baz')
+        self.assertEqual(None, tree.path2id('foo'))
+        self.assertEqual(foo_id, tree.path2id('baz'))
+        tree.reset_state()
+        # After reset, we should have forgotten about the rename, but we won't
+        # have
+        self.assertEqual(foo_id, tree.path2id('foo'))
+        self.assertEqual(None, tree.path2id('baz'))
+        self.assertPathDoesNotExist('tree/foo')
+        self.assertPathExists('tree/baz')
+
+    def test_reset_state_handles_corrupted_dirstate(self):
+        tree = self.make_initial_tree()
+        rev_id = tree.last_revision()
+        self.break_dirstate(tree)
+        tree.reset_state()
+        tree.check_state()
+        self.assertEqual(rev_id, tree.last_revision())
+
+    def test_reset_state_handles_destroyed_dirstate(self):
+        # If you pass the revision_id, we can handle a completely destroyed
+        # dirstate file.
+        tree = self.make_initial_tree()
+        rev_id = tree.last_revision()
+        self.break_dirstate(tree, completely=True)
+        tree.reset_state(revision_ids=[rev_id])
+        tree.check_state()
+        self.assertEqual(rev_id, tree.last_revision())

=== modified file 'bzrlib/tests/per_workingtree/test_commit.py'
--- bzrlib/tests/per_workingtree/test_commit.py	2010-04-01 12:53:53 +0000
+++ bzrlib/tests/per_workingtree/test_commit.py	2011-05-13 12:51:05 +0000
@@ -15,7 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 import os
 
 from bzrlib import (
@@ -26,67 +25,11 @@
     mutabletree,
     osutils,
     revision as _mod_revision,
+    tests,
     ui,
-    uncommit,
-    workingtree,
     )
-from bzrlib.errors import (NotBranchError, NotVersionedError,
-                           UnsupportedOperation)
-from bzrlib.osutils import pathjoin, getcwd
-from bzrlib.tests import TestCase
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib.trace import mutter
-from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
-                                WorkingTree)
-
-
-class CapturingUIFactory(ui.UIFactory):
-    """A UI Factory for testing - capture the updates made through it."""
-
-    def __init__(self):
-        super(CapturingUIFactory, self).__init__()
-        self._calls = []
-        self.depth = 0
-
-    def clear(self):
-        """See progress.ProgressTask.clear()."""
-
-    def clear_term(self):
-        """See progress.ProgressTask.clear_term()."""
-
-    def finished(self):
-        """See progress.ProgressTask.finished()."""
-        self.depth -= 1
-
-    def note(self, fmt_string, *args, **kwargs):
-        """See progress.ProgressTask.note()."""
-
-    def progress_bar(self):
-        return self
-
-    def nested_progress_bar(self):
-        self.depth += 1
-        return self
-
-    def update(self, message, count=None, total=None):
-        """See progress.ProgressTask.update()."""
-        if self.depth == 1:
-            self._calls.append(("update", count, total, message))
-
-
-class TestCapturingUI(TestCase):
-
-    def test_nested_ignore_depth_beyond_one(self):
-        # we only want to capture the first level out progress, not
-        # want sub-components might do. So we have nested bars ignored.
-        factory = CapturingUIFactory()
-        pb1 = factory.nested_progress_bar()
-        pb1.update('foo', 0, 1)
-        pb2 = factory.nested_progress_bar()
-        pb2.update('foo', 0, 1)
-        pb2.finished()
-        pb1.finished()
-        self.assertEqual([("update", 0, 1, 'foo')], factory._calls)
+from bzrlib.tests.testui import ProgressRecordingUIFactory
 
 
 class TestCommit(TestCaseWithWorkingTree):
@@ -204,8 +147,12 @@
         wt2 = wt.bzrdir.sprout('to').open_workingtree()
         wt2.commit('change_right')
         wt.merge_from_branch(wt2.branch)
-        self.assertRaises(errors.CannotCommitSelectedFileMerge,
-            wt.commit, 'test', exclude=['foo'])
+        try:
+            self.assertRaises(errors.CannotCommitSelectedFileMerge,
+                wt.commit, 'test', exclude=['foo'])
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
 
     def test_commit_exclude_exclude_changed_is_pointless(self):
         tree = self.make_branch_and_tree('.')
@@ -213,14 +160,22 @@
         tree.smart_add(['.'])
         tree.commit('setup test')
         self.build_tree_contents([('a', 'new contents for "a"\n')])
-        self.assertRaises(errors.PointlessCommit, tree.commit, 'test',
-            exclude=['a'], allow_pointless=False)
+        try:
+            self.assertRaises(errors.PointlessCommit, tree.commit, 'test',
+                exclude=['a'], allow_pointless=False)
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
 
     def test_commit_exclude_excludes_modified_files(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b', 'c'])
         tree.smart_add(['.'])
-        tree.commit('test', exclude=['b', 'c'])
+        try:
+            tree.commit('test', exclude=['b', 'c'])
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
         # If b was excluded it will still be 'added' in status.
         tree.lock_read()
         self.addCleanup(tree.unlock)
@@ -233,7 +188,11 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a/', 'a/b'])
         tree.smart_add(['.'])
-        tree.commit('test', specific_files=['a'], exclude=['a/b'])
+        try:
+            tree.commit('test', specific_files=['a'], exclude=['a/b'])
+        except errors.ExcludesUnsupported:
+            raise tests.TestNotApplicable("excludes not supported by this "
+                "repository format")
         # If a/b was excluded it will still be 'added' in status.
         tree.lock_read()
         self.addCleanup(tree.unlock)
@@ -321,7 +280,7 @@
             # older format.
             return
         tree.commit('foo', rev_id='foo', local=True)
-        self.failIf(master.repository.has_revision('foo'))
+        self.assertFalse(master.repository.has_revision('foo'))
         self.assertEqual(_mod_revision.NULL_REVISION,
                          (_mod_revision.ensure_null(master.last_revision())))
 
@@ -507,7 +466,7 @@
 
     def setUp(self):
         super(TestCommitProgress, self).setUp()
-        ui.ui_factory = CapturingUIFactory()
+        ui.ui_factory = ProgressRecordingUIFactory()
 
     def test_commit_progress_steps(self):
         # during commit we one progress update for every entry in the
@@ -526,7 +485,7 @@
         f.close()
         # set a progress bar that captures the calls so we can see what is
         # emitted
-        factory = CapturingUIFactory()
+        factory = ProgressRecordingUIFactory()
         ui.ui_factory = factory
         # TODO RBC 20060421 it would be nice to merge the reporter output
         # into the factory for this test - just make the test ui factory
@@ -547,7 +506,7 @@
         tree = self.make_branch_and_tree('.')
         # set a progress bar that captures the calls so we can see what is
         # emitted
-        factory = CapturingUIFactory()
+        factory = ProgressRecordingUIFactory()
         ui.ui_factory = factory
         def a_hook(_, _2, _3, _4, _5, _6):
             pass
@@ -570,7 +529,7 @@
         tree = self.make_branch_and_tree('.')
         # set a progress bar that captures the calls so we can see what is
         # emitted
-        factory = CapturingUIFactory()
+        factory = ProgressRecordingUIFactory()
         ui.ui_factory = factory
         def a_hook(_, _2, _3, _4, _5, _6, _7, _8):
             pass

=== modified file 'bzrlib/tests/per_workingtree/test_executable.py'
--- bzrlib/tests/per_workingtree/test_executable.py	2009-09-11 07:55:48 +0000
+++ bzrlib/tests/per_workingtree/test_executable.py	2011-05-13 12:51:05 +0000
@@ -21,7 +21,6 @@
 from bzrlib import (
     osutils,
     )
-from bzrlib.inventory import InventoryFile
 from bzrlib.transform import TreeTransform
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
@@ -44,9 +43,9 @@
     def check_exist(self, tree):
         """Just check that both files have the right executable bits set"""
         tree.lock_read()
-        self.failUnless(tree.is_executable(self.a_id),
+        self.assertTrue(tree.is_executable(self.a_id),
                         "'a' lost the execute bit")
-        self.failIf(tree.is_executable(self.b_id),
+        self.assertFalse(tree.is_executable(self.b_id),
                     "'b' gained an execute bit")
         tree.unlock()
 
@@ -61,10 +60,10 @@
             self.assertEqual(
                 [('', tree.inventory.root)],
                 list(tree.inventory.iter_entries()))
-        self.failIf(tree.has_id(self.a_id))
-        self.failIf(tree.has_filename('a'))
-        self.failIf(tree.has_id(self.b_id))
-        self.failIf(tree.has_filename('b'))
+        self.assertFalse(tree.has_id(self.a_id))
+        self.assertFalse(tree.has_filename('a'))
+        self.assertFalse(tree.has_id(self.b_id))
+        self.assertFalse(tree.has_filename('b'))
         tree.unlock()
 
     def commit_and_branch(self):

=== modified file 'bzrlib/tests/per_workingtree/test_flush.py'
--- bzrlib/tests/per_workingtree/test_flush.py	2009-08-25 23:38:10 +0000
+++ bzrlib/tests/per_workingtree/test_flush.py	2011-05-04 21:10:36 +0000
@@ -17,7 +17,7 @@
 """Tests for WorkingTree.flush."""
 
 import sys
-from bzrlib import errors, inventory
+from bzrlib import errors
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 

=== modified file 'bzrlib/tests/per_workingtree/test_get_file_mtime.py'
--- bzrlib/tests/per_workingtree/test_get_file_mtime.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_get_file_mtime.py	2011-09-21 00:30:34 +0000
@@ -18,6 +18,7 @@
 
 import os
 
+from bzrlib import errors
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -98,3 +99,15 @@
             self.assertAlmostEqual(st.st_mtime, mtime)
         finally:
             tree.unlock()
+
+    def test_missing(self):
+        tree = self.make_basic_tree()
+
+        os.remove('tree/one')
+        tree.lock_read()
+        try:
+            self.assertRaises(errors.FileTimestampUnavailable,
+                tree.get_file_mtime, file_id='one-id')
+        finally:
+            tree.unlock()
+

=== modified file 'bzrlib/tests/per_workingtree/test_inv.py'
--- bzrlib/tests/per_workingtree/test_inv.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_inv.py	2011-05-17 12:13:59 +0000
@@ -18,12 +18,8 @@
 
 
 import os
-import time
 
-from bzrlib import (
-    errors,
-    inventory,
-    )
+from bzrlib import inventory, tests
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -33,18 +29,24 @@
         wt = self.make_branch_and_tree('b1')
         wt.lock_tree_write()
         self.addCleanup(wt.unlock)
-        self.assertEqual(len(wt.inventory), 1)
+        self.assertEqual(len(wt.all_file_ids()), 1)
         open('b1/a', 'wb').write('a test\n')
         wt.add('a')
-        self.assertEqual(len(wt.inventory), 2)
+        self.assertEqual(len(wt.all_file_ids()), 2)
         wt.flush() # workaround revert doing wt._write_inventory for now.
         os.unlink('b1/a')
         wt.revert()
-        self.assertEqual(len(wt.inventory), 1)
+        self.assertEqual(len(wt.all_file_ids()), 1)
 
 
 class TestApplyInventoryDelta(TestCaseWithWorkingTree):
 
+    def setUp(self):
+        super(TestApplyInventoryDelta, self).setUp()
+        if not self.bzrdir_format.repository_format.supports_full_versioned_files:
+            raise tests.TestNotApplicable(
+                "format does not support inventory deltas")
+
     def test_add(self):
         wt = self.make_branch_and_tree('.')
         wt.lock_write()
@@ -155,3 +157,26 @@
         wt.apply_inventory_delta([('', None, root_id, None),
             (None, '', 'root-id',
              inventory.InventoryDirectory('root-id', '', None))])
+
+
+class TestTreeReference(TestCaseWithWorkingTree):
+
+    def test_tree_reference_matches_inv(self):
+        base = self.make_branch_and_tree('base')
+        if base.branch.repository._format.supports_full_versioned_files:
+            raise tests.TestNotApplicable(
+                "format does not support inventory deltas")
+        if not base.supports_tree_reference():
+            raise tests.TestNotApplicable("wt doesn't support nested trees")
+        # We add it as a directory, but it becomes a tree-reference
+        base.add(['subdir'], ['subdir-id'], ['directory'])
+        subdir = self.make_branch_and_tree('base/subdir')
+        self.addCleanup(base.lock_read().unlock)
+        # Note: we aren't strict about ie.kind being 'directory' here, what we
+        # are strict about is that wt.inventory should match
+        # wt.current_dirstate()'s idea about what files are where.
+        ie = base.inventory['subdir-id']
+        self.assertEqual('directory', ie.kind)
+        path, ie = base.iter_entries_by_dir(['subdir-id']).next()
+        self.assertEqual('subdir', path)
+        self.assertEqual('tree-reference', ie.kind)

=== modified file 'bzrlib/tests/per_workingtree/test_is_control_filename.py'
--- bzrlib/tests/per_workingtree/test_is_control_filename.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_is_control_filename.py	2011-05-04 11:15:37 +0000
@@ -15,8 +15,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-
 from bzrlib.osutils import basename
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 

=== modified file 'bzrlib/tests/per_workingtree/test_is_ignored.py'
--- bzrlib/tests/per_workingtree/test_is_ignored.py	2010-07-08 13:00:19 +0000
+++ bzrlib/tests/per_workingtree/test_is_ignored.py	2011-05-04 11:15:37 +0000
@@ -15,8 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import bzrlib
-from bzrlib import config, ignores, osutils
+from bzrlib import config, ignores
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 

=== modified file 'bzrlib/tests/per_workingtree/test_locking.py'
--- bzrlib/tests/per_workingtree/test_locking.py	2010-05-11 14:14:55 +0000
+++ bzrlib/tests/per_workingtree/test_locking.py	2011-05-04 11:15:37 +0000
@@ -21,7 +21,6 @@
 from bzrlib import (
     branch,
     errors,
-    lockdir,
     )
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.matchers import *

=== modified file 'bzrlib/tests/per_workingtree/test_merge_from_branch.py'
--- bzrlib/tests/per_workingtree/test_merge_from_branch.py	2010-04-01 14:06:48 +0000
+++ bzrlib/tests/per_workingtree/test_merge_from_branch.py	2011-08-16 13:40:43 +0000
@@ -20,9 +20,9 @@
 import os
 
 from bzrlib import (
-    branchbuilder,
+    conflicts,
     errors,
-    merge
+    merge,
     )
 from bzrlib.tests import per_workingtree
 
@@ -119,13 +119,6 @@
 
 class TestMergedBranch(per_workingtree.TestCaseWithWorkingTree):
 
-    def make_branch_builder(self, relpath, format=None):
-        if format is None:
-            format = self.bzrdir_format
-        builder = branchbuilder.BranchBuilder(self.get_transport(),
-                                              format=format)
-        return builder
-
     def make_inner_branch(self):
         bld_inner = self.make_branch_builder('inner')
         bld_inner.start_series()
@@ -137,6 +130,12 @@
              ('add', ('file3', 'file3-id', 'file', 'file3 content\n')),
              ])
         bld_inner.build_snapshot(
+            '4', ['1'],
+            [('add', ('file4', 'file4-id', 'file', 'file4 content\n'))
+             ])
+        bld_inner.build_snapshot(
+            '5', ['4'], [('rename', ('file4', 'dir/file4'))])
+        bld_inner.build_snapshot(
             '3', ['1'], [('modify', ('file3-id', 'new file3 contents\n')),])
         bld_inner.build_snapshot(
             '2', ['1'],
@@ -163,6 +162,8 @@
         outer.commit('added foo')
         inner = self.make_inner_branch()
         outer.merge_from_branch(inner, to_revision='1', from_revision='null:')
+        #retain original root id.
+        outer.set_root_id(outer.basis_tree().get_root_id())
         outer.commit('merge inner branch')
         outer.mkdir('dir-outer', 'dir-outer-id')
         outer.move(['dir', 'file3'], to_dir='dir-outer')
@@ -215,3 +216,39 @@
                                'foo'],
                               outer)
 
+    def test_file4_added_in_root(self):
+        outer, inner = self.make_outer_tree()
+        nb_conflicts = outer.merge_from_branch(inner, to_revision='4')
+        # file4 could not be added to its original root, so it gets added to
+        # the new root with a conflict.
+        self.assertEqual(1, nb_conflicts)
+        self.assertTreeLayout(['dir-outer',
+                               'dir-outer/dir',
+                               'dir-outer/dir/file1',
+                               'dir-outer/file3',
+                               'file4',
+                               'foo'],
+                              outer)
+
+    def test_file4_added_then_renamed(self):
+        outer, inner = self.make_outer_tree()
+        # 1 conflict, because file4 can't be put into the old root
+        self.assertEqual(1, outer.merge_from_branch(inner, to_revision='4'))
+        try:
+            outer.set_conflicts(conflicts.ConflictList())
+        except errors.UnsupportedOperation:
+            # WT2 doesn't have a separate list of conflicts to clear. It
+            # actually says there is a conflict, but happily forgets all about
+            # it.
+            pass
+        outer.commit('added file4')
+        # And now file4 gets renamed into an existing dir
+        nb_conflicts = outer.merge_from_branch(inner, to_revision='5')
+        self.assertEqual(1, nb_conflicts)
+        self.assertTreeLayout(['dir-outer',
+                               'dir-outer/dir',
+                               'dir-outer/dir/file1',
+                               'dir-outer/dir/file4',
+                               'dir-outer/file3',
+                               'foo'],
+                              outer)

=== modified file 'bzrlib/tests/per_workingtree/test_move.py'
--- bzrlib/tests/per_workingtree/test_move.py	2010-07-15 13:33:30 +0000
+++ bzrlib/tests/per_workingtree/test_move.py	2011-08-29 21:53:33 +0000
@@ -21,27 +21,21 @@
 from bzrlib import (
     errors,
     osutils,
+    tests,
     )
 
-from bzrlib.workingtree_4 import DirStateWorkingTreeFormat
+from bzrlib.tests.matchers import HasLayout
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestMove(TestCaseWithWorkingTree):
 
-    def get_tree_layout(self, tree):
-        """Get the (path, file_id) pairs for the current tree."""
-        tree.lock_read()
-        try:
-            return [(path, ie.file_id) for path, ie
-                    in tree.iter_entries_by_dir()]
-        finally:
-            tree.unlock()
-
     def assertTreeLayout(self, expected, tree):
         """Check that the tree has the correct layout."""
-        actual = self.get_tree_layout(tree)
-        self.assertEqual(expected, actual)
+        self.assertThat(tree, HasLayout(expected))
 
     def test_move_via_rm_and_add(self):
         """Move by remove and add-with-id"""
@@ -132,12 +126,12 @@
                           tree.move, ['d', 'c', 'b'], 'a')
         if osutils.lexists('a/c'):
             # If 'c' was actually moved, then 'd' should have also been moved
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'),
+            self.assertTreeLayout([('', root_id), ('a/', 'a-id'),
                                    ('a/c', 'c-id'),  ('a/d', 'd-id')], tree)
         else:
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+            self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c', 'c-id'),
                                    ('d', 'd-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c', 'c-id'),
                                ('d', 'd-id')], tree.basis_tree())
         tree._validate()
 
@@ -151,7 +145,7 @@
         tree.remove(['a/b'], keep_files=False)
         self.assertEqual([('b', 'a/b')], tree.move(['b'], 'a'))
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
+                               ('a/', 'a-id'),
                                ('a/b', 'b-id'),
                               ], tree)
         tree._validate()
@@ -162,18 +156,18 @@
         tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
         tree.commit('initial', rev_id='rev-1')
         root_id = tree.get_root_id()
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         a_contents = tree.get_file_text('a-id')
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id'),
                                ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
-        self.failIfExists('a')
+        self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'b/a')
         tree._validate()
 
@@ -186,11 +180,11 @@
         c_contents = tree.get_file_text('c-id')
         self.assertEqual([('b/c', 'c')],
             tree.move(['b/c'], ''))
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
-        self.failIfExists('b/c')
+        self.assertPathDoesNotExist('b/c')
         self.assertFileEqual(c_contents, 'c')
         tree._validate()
 
@@ -206,13 +200,13 @@
         # 'c' may or may not have been moved, but either way the tree should
         # maintain a consistent state.
         if osutils.lexists('c'):
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                    ('c', 'c-id')], tree)
         else:
-            self.failUnlessExists('b/c')
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+            self.assertPathExists('b/c')
+            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                    ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree.basis_tree())
         tree._validate()
 
@@ -244,15 +238,15 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/a')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # We don't need after=True as long as source is missing and target
         # exists.
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         tree._validate()
 
@@ -264,14 +258,14 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/a')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # Passing after=True should work as well
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b', after=True))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         tree._validate()
 
@@ -285,7 +279,7 @@
         # Passing after when the file hasn't been move raises an exception
         self.assertRaises(errors.BzrMoveFailedError,
                           tree.move, ['a'], 'b', after=True)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         tree._validate()
 
@@ -309,20 +303,20 @@
         finally:
             ba_file.close()
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.move, ['a'], 'b', after=False)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(ba_text, 'b/a')
         # But you can pass after=True
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b', after=True))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         # But it shouldn't actually move anything
         self.assertFileEqual(a_text, 'a')
@@ -339,15 +333,18 @@
 
         self.assertEqual([('a', 'e/a')],
             tree.move(['a'], 'e'))
-        self.assertTreeLayout([('', root_id), ('e', 'e-id'), ('e/a', 'a-id'),
-                               ('e/a/b', 'b-id'), ('e/a/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('e/', 'e-id'), ('e/a/', 'a-id'),
+                               ('e/a/b', 'b-id'), ('e/a/c/', 'c-id'),
                                ('e/a/c/d', 'd-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('e', 'e-id'),
-                               ('a/b', 'b-id'), ('a/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('e/', 'e-id'),
+                               ('a/b', 'b-id'), ('a/c/', 'c-id'),
                                ('a/c/d', 'd-id')], tree.basis_tree())
         tree._validate()
 
     def test_move_directory_into_parent(self):
+        if not self.workingtree_format.supports_versioned_directories:
+            raise tests.TestNotApplicable(
+                "test requires versioned directories")
         tree = self.make_branch_and_tree('.')
         self.build_tree(['c/', 'c/b/', 'c/b/d/'])
         tree.add(['c', 'c/b', 'c/b/d'],
@@ -358,9 +355,9 @@
         self.assertEqual([('c/b', 'b')],
                          tree.move(['c/b'], ''))
         self.assertTreeLayout([('', root_id),
-                               ('b', 'b-id'),
-                               ('c', 'c-id'),
-                               ('b/d', 'd-id'),
+                               ('b/', 'b-id'),
+                               ('c/', 'c-id'),
+                               ('b/d/', 'd-id'),
                               ], tree)
         tree._validate()
 
@@ -372,20 +369,19 @@
         tree.commit('initial', rev_id='rev-1')
         root_id = tree.get_root_id()
 
-
         tree.rename_one('a/b', 'a/c/b')
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
-                               ('d', 'd-id'),
-                               ('a/c', 'c-id'),
+                               ('a/', 'a-id'),
+                               ('d/', 'd-id'),
+                               ('a/c/', 'c-id'),
                                ('a/c/b', 'b-id'),
                               ], tree)
         self.assertEqual([('a', 'd/a')],
                          tree.move(['a'], 'd'))
         self.assertTreeLayout([('', root_id),
-                               ('d', 'd-id'),
-                               ('d/a', 'a-id'),
-                               ('d/a/c', 'c-id'),
+                               ('d/', 'd-id'),
+                               ('d/a/', 'a-id'),
+                               ('d/a/c/', 'c-id'),
                                ('d/a/c/b', 'b-id'),
                               ], tree)
         tree._validate()
@@ -403,8 +399,8 @@
         self.assertEqual([('a', 'b/a')],
                          tree.move(['a'], 'b'))
         self.assertTreeLayout([('', root_id),
-                               ('b', 'b-id'),
-                               ('b/a', 'a-id'),
+                               ('b/', 'b-id'),
+                               ('b/a/', 'a-id'),
                                ('b/a/c', 'ac-id'),
                               ], tree)
         tree._validate()
@@ -422,8 +418,8 @@
         self.assertEqual([('a', 'b/a')],
                          tree.move(['a'], 'b'))
         self.assertTreeLayout([('', root_id),
-                               ('b', 'b-id'),
-                               ('b/a', 'a-id'),
+                               ('b/', 'b-id'),
+                               ('b/a/', 'a-id'),
                                ('b/a/b', 'ab-id'),
                                ('b/a/c', 'ac-id'),
                                ('b/a/d', 'ad-id'),
@@ -441,18 +437,18 @@
         self.assertEqual([('a/b', 'b')],
                          tree.move(['a/b'], ''))
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
+                               ('a/', 'a-id'),
                                ('b', 'b-id'),
                                ('d', 'd-id'),
-                               ('e', 'e-id'),
+                               ('e/', 'e-id'),
                                ('a/c', 'c-id'),
                               ], tree)
         self.assertEqual([('d', 'a/d')],
                          tree.move(['d'], 'a'))
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
+                               ('a/', 'a-id'),
                                ('b', 'b-id'),
-                               ('e', 'e-id'),
+                               ('e/', 'e-id'),
                                ('a/c', 'c-id'),
                                ('a/d', 'd-id'),
                               ], tree)
@@ -460,8 +456,8 @@
                          tree.move(['a'], 'e'))
         self.assertTreeLayout([('', root_id),
                                ('b', 'b-id'),
-                               ('e', 'e-id'),
-                               ('e/a', 'a-id'),
+                               ('e/', 'e-id'),
+                               ('e/a/', 'a-id'),
                                ('e/a/c', 'c-id'),
                                ('e/a/d', 'd-id'),
                               ], tree)
@@ -477,16 +473,16 @@
 
         tree.rename_one('a/b', 'a/d')
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
-                               ('d', 'd-id'),
+                               ('a/', 'a-id'),
+                               ('d/', 'd-id'),
                                ('a/c', 'c-id'),
                                ('a/d', 'b-id'),
                               ], tree)
         self.assertEqual([('a', 'd/a')],
                          tree.move(['a'], 'd'))
         self.assertTreeLayout([('', root_id),
-                               ('d', 'd-id'),
-                               ('d/a', 'a-id'),
+                               ('d/', 'd-id'),
+                               ('d/a/', 'a-id'),
                                ('d/a/c', 'c-id'),
                                ('d/a/d', 'b-id'),
                               ], tree)
@@ -504,8 +500,8 @@
         tree.rename_one('a/d', 'a/b')
         tree.rename_one('a/bb', 'a/d')
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
-                               ('e', 'e-id'),
+                               ('a/', 'a-id'),
+                               ('e/', 'e-id'),
                                ('a/b', 'd-id'),
                                ('a/c', 'c-id'),
                                ('a/d', 'b-id'),
@@ -513,8 +509,8 @@
         self.assertEqual([('a', 'e/a')],
                          tree.move(['a'], 'e'))
         self.assertTreeLayout([('', root_id),
-                               ('e', 'e-id'),
-                               ('e/a', 'a-id'),
+                               ('e/', 'e-id'),
+                               ('e/a/', 'a-id'),
                                ('e/a/b', 'd-id'),
                                ('e/a/c', 'c-id'),
                                ('e/a/d', 'b-id'),
@@ -531,15 +527,37 @@
 
         self.assertEqual([('a/b', 'c/b')],
             tree.move(['a/b'], 'c'))
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('c/b', 'b-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
 
         self.assertEqual([('c/b', 'b')],
             tree.move(['c/b'], ''))
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
-                               ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('b', 'b-id'),
+                               ('c/', 'c-id')], tree)
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
         tree._validate()
+
+    def test_move_to_unversioned_non_ascii_dir(self):
+        """Check error when moving to unversioned non-ascii directory"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["a", u"\xA7/"])
+        tree.add(["a"])
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.move, ["a"], u"\xA7")
+        self.assertIsInstance(e.extra, errors.NotVersionedError)
+        self.assertEqual(e.extra.path, u"\xA7")
+
+    def test_move_unversioned_non_ascii(self):
+        """Check error when moving an unversioned non-ascii file"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree([u"\xA7", "dir/"])
+        tree.add("dir")
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.move, [u"\xA7"], "dir")
+        self.assertIsInstance(e.extra, errors.NotVersionedError)
+        self.assertEqual(e.extra.path, u"\xA7")

=== modified file 'bzrlib/tests/per_workingtree/test_nested_specifics.py'
--- bzrlib/tests/per_workingtree/test_nested_specifics.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_nested_specifics.py	2011-05-09 14:09:52 +0000
@@ -16,6 +16,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+from bzrlib import (
+    inventory,
+    )
 from bzrlib.tests import TestNotApplicable
 from bzrlib.transform import TreeTransform
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
@@ -65,6 +68,17 @@
 
     def test_comparison_data_does_not_autodetect_subtree(self):
         tree = self.prepare_with_subtree()
-        entry = tree.iter_entries_by_dir(['subtree-id']).next()[1]
+        ie = inventory.InventoryDirectory('subtree-id', 'subtree',
+                                          tree.path2id(''))
         self.assertEqual('directory',
-                         tree._comparison_data(entry, 'subtree')[0])
+                         tree._comparison_data(ie, 'subtree')[0])
+
+    def test_inventory_does_not_autodetect_subtree(self):
+        tree = self.prepare_with_subtree()
+        ie = tree.inventory['subtree-id']
+        self.assertEqual('directory', ie.kind)
+
+    def test_iter_entries_by_dir_autodetects_subtree(self):
+        tree = self.prepare_with_subtree()
+        path, ie = tree.iter_entries_by_dir(['subtree-id']).next()
+        self.assertEqual('tree-reference', ie.kind)

=== modified file 'bzrlib/tests/per_workingtree/test_parents.py'
--- bzrlib/tests/per_workingtree/test_parents.py	2009-09-09 01:05:31 +0000
+++ bzrlib/tests/per_workingtree/test_parents.py	2011-06-14 01:26:41 +0000
@@ -16,14 +16,13 @@
 
 """Tests of the parent related functions of WorkingTrees."""
 
-from errno import EEXIST
+from cStringIO import StringIO
 import os
 
 from bzrlib import (
     errors,
     osutils,
     revision as _mod_revision,
-    symbol_versioning,
     tests,
     )
 from bzrlib.inventory import (
@@ -32,8 +31,11 @@
     InventoryDirectory,
     InventoryLink,
     )
-from bzrlib.revision import Revision
+from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+from bzrlib.tests import (
+    features,
+    )
 from bzrlib.uncommit import uncommit
 
 
@@ -229,8 +231,8 @@
 
     def test_unicode_symlink(self):
         # this tests bug #272444
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         tree = self.make_branch_and_tree('tree1')
 
@@ -240,7 +242,7 @@
         target = u'\u03a9'
         link_name = u'\N{Euro Sign}link'
         os.symlink(target, 'tree1/' + link_name)
-        tree.add([link_name],['link-id'])
+        tree.add([link_name], ['link-id'])
 
         revision1 = tree.commit('added a link to a Unicode target')
         revision2 = tree.commit('this revision will be discarded')
@@ -386,32 +388,41 @@
         return delta
 
     def fake_up_revision(self, tree, revid, shape):
+
+        class ShapeTree(InventoryRevisionTree):
+
+            def __init__(self, shape):
+                self._repository = tree.branch.repository
+                self._inventory = shape
+
+            def get_file_text(self, file_id, path=None):
+                ie = self.inventory[file_id]
+                if ie.kind != "file":
+                    return ""
+                return 'a' * ie.text_size
+
+            def get_file(self, file_id, path=None):
+                return StringIO(self.get_file_text(file_id))
+
         tree.lock_write()
         try:
-            tree.branch.repository.start_write_group()
-            try:
-                if shape.root.revision is None:
-                    shape.root.revision = revid
-                # Create the text records for this inventory.
-                for path, ie in shape.iter_entries():
-                    if ie.text_size:
-                        lines = ['a' * ie.text_size]
-                    else:
-                        lines = []
-                    tree.branch.repository.texts.add_lines(
-                        (ie.file_id, ie.revision), [], lines)
-                sha1 = tree.branch.repository.add_inventory(revid, shape, [])
-                rev = Revision(timestamp=0,
-                               timezone=None,
-                               committer="Foo Bar <foo@example.com>",
-                               message="Message",
-                               inventory_sha1=sha1,
-                               revision_id=revid)
-                tree.branch.repository.add_revision(revid, rev)
-                tree.branch.repository.commit_write_group()
-            except:
-                tree.branch.repository.abort_write_group()
-                raise
+            if shape.root.revision is None:
+                shape.root.revision = revid
+            builder = tree.branch.get_commit_builder(
+                    parents=[],
+                    timestamp=0,
+                    timezone=None,
+                    committer="Foo Bar <foo@example.com>",
+                    revision_id=revid)
+            shape_tree = ShapeTree(shape)
+            base_tree = tree.branch.repository.revision_tree(
+                    _mod_revision.NULL_REVISION)
+            changes = shape_tree.iter_changes(
+                base_tree)
+            list(builder.record_iter_changes(shape_tree,
+                base_tree.get_revision_id(), changes))
+            builder.finish_inventory()
+            builder.commit("Message")
         finally:
             tree.unlock()
 
@@ -468,8 +479,8 @@
 
     def test_no_parents_just_root(self):
         """Test doing an empty commit - no parent, set a root only."""
-        basis_shape = Inventory(root_id=None) # empty tree
-        new_shape = Inventory() # tree with a root
+        basis_shape = Inventory(root_id=None)  # empty tree
+        new_shape = Inventory()  # tree with a root
         self.assertTransitionFromBasisToShape(basis_shape, None, new_shape,
             'new_parent')
 
@@ -520,10 +531,13 @@
         def do_file(inv, revid):
             self.add_file(inv, revid, 'path-id', 'root-id', 'path', '1' * 32,
                 12)
+
         def do_link(inv, revid):
             self.add_link(inv, revid, 'path-id', 'root-id', 'path', 'target')
+
         def do_dir(inv, revid):
             self.add_dir(inv, revid, 'path-id', 'root-id', 'path')
+
         for old_factory in (do_file, do_link, do_dir):
             for new_factory in (do_file, do_link, do_dir):
                 if old_factory == new_factory:

=== modified file 'bzrlib/tests/per_workingtree/test_pull.py'
--- bzrlib/tests/per_workingtree/test_pull.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/per_workingtree/test_pull.py	2011-09-26 15:21:01 +0000
@@ -15,17 +15,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
-import os
-
-from bzrlib import errors
-from bzrlib.errors import NotBranchError, NotVersionedError
-from bzrlib.osutils import basename
-from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib.trace import mutter
-
-
-class TestPull(TestCaseWithWorkingTree):
+
+from bzrlib import tests
+from bzrlib.tests import per_workingtree
+
+
+class TestPull(per_workingtree.TestCaseWithWorkingTree):
 
     def get_pullable_trees(self):
         self.build_tree(['from/', 'from/file', 'to/'])
@@ -38,16 +33,16 @@
     def test_pull(self):
         tree_a, tree_b = self.get_pullable_trees()
         tree_b.pull(tree_a.branch)
-        self.failUnless(tree_b.branch.repository.has_revision('A'))
+        self.assertTrue(tree_b.branch.repository.has_revision('A'))
         self.assertEqual(['A'], tree_b.get_parent_ids())
 
     def test_pull_overwrites(self):
         tree_a, tree_b = self.get_pullable_trees()
         tree_b.commit('foo', rev_id='B')
-        self.assertEqual(['B'], tree_b.branch.revision_history())
+        self.assertEqual('B', tree_b.branch.last_revision())
         tree_b.pull(tree_a.branch, overwrite=True)
-        self.failUnless(tree_b.branch.repository.has_revision('A'))
-        self.failUnless(tree_b.branch.repository.has_revision('B'))
+        self.assertTrue(tree_b.branch.repository.has_revision('A'))
+        self.assertTrue(tree_b.branch.repository.has_revision('B'))
         self.assertEqual(['A'], tree_b.get_parent_ids())
 
     def test_pull_merges_tree_content(self):
@@ -67,3 +62,41 @@
         tree.commit('second')
         to_tree.pull(tree.branch)
         self.assertEqual('second_root_id', to_tree.get_root_id())
+
+
+class TestPullWithOrphans(per_workingtree.TestCaseWithWorkingTree):
+
+    def make_branch_deleting_dir(self, relpath=None):
+        if relpath is None:
+            relpath = 'trunk'
+        builder = self.make_branch_builder(relpath)
+        builder.start_series()
+
+        # Create an empty trunk
+        builder.build_snapshot('1', None, [
+                ('add', ('', 'root-id', 'directory', ''))])
+        builder.build_snapshot('2', ['1'], [
+                ('add', ('dir', 'dir-id', 'directory', '')),
+                ('add', ('file', 'file-id', 'file', 'trunk content\n')),])
+        builder.build_snapshot('3', ['2'], [
+                ('unversion', 'dir-id'),])
+        builder.finish_series()
+        return builder.get_branch()
+
+    def test_pull_orphans(self):
+        if not self.workingtree_format.missing_parent_conflicts:
+            raise tests.TestSkipped(
+                '%r does not support missing parent conflicts' %
+                    self.workingtree_format)
+        trunk = self.make_branch_deleting_dir('trunk')
+        work = trunk.bzrdir.sprout('work', revision_id='2').open_workingtree()
+        work.branch.get_config().set_user_option(
+            'bzr.transform.orphan_policy', 'move')
+        # Add some unversioned files in dir
+        self.build_tree(['work/dir/foo',
+                         'work/dir/subdir/',
+                         'work/dir/subdir/foo'])
+        work.pull(trunk)
+        self.assertLength(0, work.conflicts())
+        # The directory removal should succeed
+        self.assertPathDoesNotExist('work/dir')

=== modified file 'bzrlib/tests/per_workingtree/test_read_working_inventory.py'
--- bzrlib/tests/per_workingtree/test_read_working_inventory.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_read_working_inventory.py	2011-04-21 20:32:16 +0000
@@ -17,19 +17,27 @@
 """Tests for WorkingTree.read_working_inventory."""
 
 from bzrlib import errors, inventory
+from bzrlib.tests import TestNotApplicable
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+from bzrlib.workingtree import InventoryWorkingTree
 
 
 class TestReadWorkingInventory(TestCaseWithWorkingTree):
 
     def test_trivial_read(self):
         tree = self.make_branch_and_tree('t1')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable("read_working_inventory not usable on "
+                "non-inventory working trees")
         tree.lock_read()
         self.assertIsInstance(tree.read_working_inventory(), inventory.Inventory)
         tree.unlock()
 
     def test_read_after_inventory_modification(self):
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable("read_working_inventory not usable on "
+                "non-inventory working trees")
         # prepare for a series of changes that will modify the
         # inventory
         tree.lock_write()

=== modified file 'bzrlib/tests/per_workingtree/test_readonly.py'
--- bzrlib/tests/per_workingtree/test_readonly.py	2010-01-25 15:55:48 +0000
+++ bzrlib/tests/per_workingtree/test_readonly.py	2011-08-31 22:52:43 +0000
@@ -21,12 +21,13 @@
 import time
 
 from bzrlib import (
-    errors,
     hashcache,
     tests,
     )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
+from bzrlib.workingtree import InventoryWorkingTree
+
 
 class TestReadonly(TestCaseWithWorkingTree):
 
@@ -78,6 +79,8 @@
         """Even if the tree is unclean, we should still handle readonly dirs."""
         # First create a tree
         tree = self.create_basic_tree()
+        if not isinstance(tree, InventoryWorkingTree):
+            raise tests.TestNotApplicable("requires inventory working tree")
 
         # XXX: *Ugly* *ugly* hack, we need the hashcache to think it is out of
         # date, but we don't want to actually wait 3 seconds doing nothing.
@@ -106,7 +109,7 @@
             if hack_dirstate:
                 tree._dirstate._sha_cutoff_time = self._custom_cutoff_time
             # Make sure we check all the files
-            for file_id in tree:
+            for file_id in tree.all_file_ids():
                 size = tree.get_file_size(file_id)
                 sha1 = tree.get_file_sha1(file_id)
         finally:

=== modified file 'bzrlib/tests/per_workingtree/test_remove.py'
--- bzrlib/tests/per_workingtree/test_remove.py	2010-04-30 10:28:36 +0000
+++ bzrlib/tests/per_workingtree/test_remove.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,18 +17,19 @@
 """Tests for interface conformance of 'WorkingTree.remove'"""
 
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib import errors, ignores, osutils
+from bzrlib import ignores, osutils
 
 class TestRemove(TestCaseWithWorkingTree):
     """Tests WorkingTree.remove"""
 
     files = ['a', 'b/', 'b/c', 'd/']
     rfiles = ['b/c', 'b', 'a', 'd']
+    backup_files = ['a.~1~', 'b.~1~/', 'b.~1~/c.~1~', 'd.~1~/']
 
     def get_tree(self, files):
         tree = self.make_branch_and_tree('.')
         self.build_tree(files)
-        self.failUnlessExists(files)
+        self.assertPathExists(files)
         return tree
 
     def get_committed_tree(self, files, message="Committing"):
@@ -40,11 +41,11 @@
 
     def assertRemovedAndDeleted(self, files):
         self.assertNotInWorkingTree(files)
-        self.failIfExists(files)
+        self.assertPathDoesNotExist(files)
 
     def assertRemovedAndNotDeleted(self, files):
         self.assertNotInWorkingTree(files)
-        self.failUnlessExists(files)
+        self.assertPathExists(files)
 
     def test_remove_keep(self):
         """Check that files and directories are unversioned but not deleted."""
@@ -72,28 +73,23 @@
         tree._validate()
 
     def test_remove_added_files(self):
-        """Removal of newly added files must fail."""
+        """Removal of newly added files must back them up."""
         tree = self.get_tree(TestRemove.files)
         tree.add(TestRemove.files)
         self.assertInWorkingTree(TestRemove.files)
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            TestRemove.files, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)added:.*a.*b/.*b/c.*d/')
-        self.assertInWorkingTree(TestRemove.files)
-        self.failUnlessExists(TestRemove.files)
+        tree.remove(TestRemove.files, keep_files=False)
+        self.assertNotInWorkingTree(TestRemove.files)
+        self.assertPathExists(TestRemove.backup_files)
         tree._validate()
 
     def test_remove_changed_file(self):
-        """Removal of a changed files must fail."""
+        """Removal of changed files must back it up."""
         tree = self.get_committed_tree(['a'])
         self.build_tree_contents([('a', "some other new content!")])
         self.assertInWorkingTree('a')
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'a', keep_files=False)
-        self.assertContainsRe(err.changes_as_text, '(?s)modified:.*a')
-        self.assertInWorkingTree('a')
-        self.failUnlessExists('a')
+        tree.remove('a', keep_files=False)
+        self.assertNotInWorkingTree(TestRemove.files)
+        self.assertPathExists('a.~1~')
         tree._validate()
 
     def test_remove_deleted_files(self):
@@ -102,7 +98,7 @@
         for f in TestRemove.rfiles:
             osutils.delete_any(f)
         self.assertInWorkingTree(TestRemove.files)
-        self.failIfExists(TestRemove.files)
+        self.assertPathDoesNotExist(TestRemove.files)
         tree.remove(TestRemove.files, keep_files=False)
         self.assertRemovedAndDeleted(TestRemove.files)
         tree._validate()
@@ -115,14 +111,14 @@
             tree.rename_one(f,f+'x')
         rfilesx = ['bx/cx', 'bx', 'ax', 'dx']
         self.assertInWorkingTree(rfilesx)
-        self.failUnlessExists(rfilesx)
+        self.assertPathExists(rfilesx)
 
         tree.remove(rfilesx, keep_files=False)
         self.assertRemovedAndDeleted(rfilesx)
         tree._validate()
 
     def test_remove_renamed_changed_files(self):
-        """Check that files are not removed if they are renamed and changed."""
+        """Check that files that are renamed and changed are backed up."""
         tree = self.get_committed_tree(TestRemove.files)
 
         for f in TestRemove.rfiles:
@@ -131,14 +127,12 @@
         self.build_tree_contents([('ax','changed and renamed!'),
                                   ('bx/cx','changed and renamed!')])
         self.assertInWorkingTree(rfilesx)
-        self.failUnlessExists(rfilesx)
+        self.assertPathExists(rfilesx)
 
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            rfilesx, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)modified:.*ax.*bx/cx')
-        self.assertInWorkingTree(rfilesx)
-        self.failUnlessExists(rfilesx)
+        tree.remove(rfilesx, keep_files=False)
+        self.assertNotInWorkingTree(rfilesx)
+        self.assertPathExists(['bx.~1~/cx.~1~', 'bx.~1~', 'ax.~1~'])
+        self.assertPathDoesNotExist('dx.~1~') # unchanged file
         tree._validate()
 
     def test_force_remove_changed_files(self):
@@ -149,15 +143,15 @@
 
         tree.remove(TestRemove.files, keep_files=False, force=True)
         self.assertRemovedAndDeleted(TestRemove.files)
+        self.assertPathDoesNotExist(['a.~1~', 'b.~1~/', 'b.~1~/c', 'd.~1~/'])
         tree._validate()
 
     def test_remove_unknown_files(self):
-        """Try to delete unknown files."""
+        """Unknown files shuld be backed up"""
         tree = self.get_tree(TestRemove.files)
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            TestRemove.files, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*d/.*b/c.*b/.*a.*')
+        tree.remove(TestRemove.files, keep_files=False)
+        self.assertRemovedAndDeleted(TestRemove.files)
+        self.assertPathExists(TestRemove.backup_files)
         tree._validate()
 
     def test_remove_nonexisting_files(self):
@@ -202,45 +196,40 @@
         tree._validate()
 
     def test_remove_changed_ignored_files(self):
-        """Changed ignored files should not be deleted."""
+        """Changed ignored files should be backed up."""
         files = ['an_ignored_file']
         tree = self.get_tree(files)
         tree.add(files)
         ignores.add_runtime_ignores(["*ignored*"])
         self.assertInWorkingTree(files)
         self.assertNotEquals(None, tree.is_ignored(files[0]))
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            files, keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)added:.*' + files[0])
-        self.assertInWorkingTree(files)
+
+        tree.remove(files, keep_files=False)
+        self.assertNotInWorkingTree(files)
+        self.assertPathExists('an_ignored_file.~1~')
         tree._validate()
 
     def test_dont_remove_directory_with_unknowns(self):
-        """Directories with unknowns should not be deleted."""
+        """Directories with unknowns should be backed up."""
         directories = ['a/', 'b/', 'c/', 'c/c/']
         tree = self.get_committed_tree(directories)
 
         self.build_tree(['a/unknown_file'])
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'a', keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*a/unknown_file')
+        tree.remove('a', keep_files=False)
+        self.assertPathExists('a.~1~/unknown_file')
 
         self.build_tree(['b/unknown_directory'])
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'b', keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*b/unknown_directory')
+        tree.remove('b', keep_files=False)
+        self.assertPathExists('b.~1~/unknown_directory')
 
         self.build_tree(['c/c/unknown_file'])
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'c/c', keep_files=False)
-        self.assertContainsRe(err.changes_as_text,
-            '(?s)unknown:.*c/c/unknown_file')
-
-        self.assertInWorkingTree(directories)
-        self.failUnlessExists(directories)
+        tree.remove('c/c', keep_files=False)
+        self.assertPathExists('c/c.~1~/unknown_file')
+
+        tree.remove('c', keep_files=False)
+        self.assertPathExists('c.~1~/')
+
+        self.assertNotInWorkingTree(directories)
         tree._validate()
 
     def test_force_remove_directory_with_unknowns(self):
@@ -253,7 +242,7 @@
         self.build_tree(other_files)
 
         self.assertInWorkingTree(files)
-        self.failUnlessExists(files)
+        self.assertPathExists(files)
 
         tree.remove('b', keep_files=False, force=True)
 
@@ -262,16 +251,20 @@
         tree._validate()
 
     def test_remove_directory_with_changed_file(self):
-        """Refuse to delete directories with changed files."""
-        files = ['b/', 'b/c']
-        tree = self.get_committed_tree(files)
-        self.build_tree_contents([('b/c', "some other new content!")])
-
-        err = self.assertRaises(errors.BzrRemoveChangedFilesError, tree.remove,
-            'b', keep_files=False)
-        self.assertContainsRe(err.changes_as_text, '(?s)modified:.*b/c')
-        self.assertInWorkingTree(files)
-        self.failUnlessExists(files)
+        """Backup directories with changed files."""
+        files = ['b/', 'b/c']
+        tree = self.get_committed_tree(files)
+        self.build_tree_contents([('b/c', "some other new content!")])
+
+        tree.remove('b', keep_files=False)
+        self.assertPathExists('b.~1~/c.~1~')
+        self.assertNotInWorkingTree(files)
+
+    def test_remove_force_directory_with_changed_file(self):
+        """Delete directories with changed files when forced."""
+        files = ['b/', 'b/c']
+        tree = self.get_committed_tree(files)
+        self.build_tree_contents([('b/c', "some other new content!")])
 
         # see if we can force it now..
         tree.remove('b', keep_files=False, force=True)
@@ -288,9 +281,9 @@
         tree.rename_one('somedir/file', 'moved-file')
         tree.remove('somedir', keep_files=False)
         self.assertNotInWorkingTree('somedir')
-        self.failIfExists('somedir')
+        self.assertPathDoesNotExist('somedir')
         self.assertInWorkingTree('moved-file')
-        self.failUnlessExists('moved-file')
+        self.assertPathExists('moved-file')
 
     def test_remove_directory_with_renames(self):
         """Delete directory with renames in or out."""
@@ -305,7 +298,7 @@
         moved_files = ['b/file', 'b/directory/']
         self.assertRemovedAndDeleted(files_to_move)
         self.assertInWorkingTree(moved_files)
-        self.failUnlessExists(moved_files)
+        self.assertPathExists(moved_files)
 
         # check if it works with renames out
         tree.remove('a', keep_files=False)
@@ -322,7 +315,7 @@
         tree.add(['dir', 'dir/file'])
         tree.commit('add file')
         tree.remove('dir/', keep_files=False)
-        self.failIfExists('tree/dir/file')
+        self.assertPathDoesNotExist('tree/dir/file')
         self.assertNotInWorkingTree('tree/dir/file', 'tree')
         tree._validate()
 
@@ -331,15 +324,15 @@
         tree = self.get_committed_tree(['a'])
         tree.remove('a', keep_files=False)
         tree.remove('a', keep_files=False)
-        self.failIfExists('a')
+        self.assertPathDoesNotExist('a')
         tree._validate()
 
     def test_remove_file_and_containing_dir(self):
         tree = self.get_committed_tree(['config/', 'config/file'])
         tree.remove('config/file', keep_files=False)
         tree.remove('config', keep_files=False)
-        self.failIfExists('config/file')
-        self.failIfExists('config')
+        self.assertPathDoesNotExist('config/file')
+        self.assertPathDoesNotExist('config')
         tree._validate()
 
     def test_remove_dir_before_bzr(self):
@@ -347,6 +340,6 @@
         # or the bug doesn't manifest itself.
         tree = self.get_committed_tree(['.aaa/', '.aaa/file'])
         tree.remove('.aaa/', keep_files=False)
-        self.failIfExists('.aaa/file')
-        self.failIfExists('.aaa')
+        self.assertPathDoesNotExist('.aaa/file')
+        self.assertPathDoesNotExist('.aaa')
         tree._validate()

=== modified file 'bzrlib/tests/per_workingtree/test_rename_one.py'
--- bzrlib/tests/per_workingtree/test_rename_one.py	2009-07-23 00:24:30 +0000
+++ bzrlib/tests/per_workingtree/test_rename_one.py	2011-08-29 21:53:33 +0000
@@ -23,25 +23,19 @@
     osutils,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
+from bzrlib.tests.matchers import HasLayout
 
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
 class TestRenameOne(TestCaseWithWorkingTree):
 
-    def get_tree_layout(self, tree):
-        """Get the (path, file_id) pairs for the current tree."""
-        tree.lock_read()
-        try:
-            return [(path, ie.file_id) for path, ie
-                    in tree.iter_entries_by_dir()]
-        finally:
-            tree.unlock()
-
     def assertTreeLayout(self, expected, tree):
         """Check that the tree has the correct layout."""
-        actual = self.get_tree_layout(tree)
-        self.assertEqual(expected, actual)
+        self.assertThat(tree, HasLayout(expected))
 
     def test_rename_one_target_not_dir(self):
         tree = self.make_branch_and_tree('.')
@@ -87,11 +81,11 @@
 
         a_contents = tree.get_file_text('a-id')
         tree.rename_one('a', 'foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('foo', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('foo', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
-        self.failIfExists('a')
+        self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'foo')
 
     def test_rename_one_not_localdir(self):
@@ -103,11 +97,11 @@
 
         a_contents = tree.get_file_text('a-id')
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/foo', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/foo', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
-        self.failIfExists('tree/a')
+        self.assertPathDoesNotExist('tree/a')
         self.assertFileEqual(a_contents, 'tree/b/foo')
 
     def test_rename_one_subdir(self):
@@ -116,17 +110,17 @@
         tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
         tree.commit('initial', rev_id='rev-1')
         root_id = tree.get_root_id()
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         a_contents = tree.get_file_text('a-id')
         tree.rename_one('a', 'b/d')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/c', 'c-id'),
                                ('b/d', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
-        self.failIfExists('a')
+        self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'b/d')
 
     def test_rename_one_parent_dir(self):
@@ -137,11 +131,11 @@
         root_id = tree.get_root_id()
         c_contents = tree.get_file_text('c-id')
         tree.rename_one('b/c', 'd')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('d', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
-        self.failIfExists('b/c')
+        self.assertPathDoesNotExist('b/c')
         self.assertFileEqual(c_contents, 'd')
 
     def test_rename_one_fail_consistent(self):
@@ -153,9 +147,9 @@
         # Target already exists
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.rename_one, 'a', 'b/a')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree.basis_tree())
 
     def test_rename_one_onto_existing(self):
@@ -193,14 +187,14 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/foo')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # We don't need after=True as long as source is missing and target
         # exists.
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
     def test_rename_one_after_with_after(self):
@@ -211,17 +205,62 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/foo')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # Passing after=True should work as well
         tree.rename_one('a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
+    def test_rename_one_after_dest_versioned(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+        os.rename('a', 'b')
+        tree.add(['b'], ['b-id'])
+
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+                               tree)
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.rename_one, 'a', 'b')
+        self.assertIsInstance(e.extra, errors.AlreadyVersionedError)
+
+    def test_rename_one_after_with_after_dest_versioned(self):
+        ''' using after with an already versioned file should fail '''
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+        os.unlink('a')
+
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+                               tree)
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.rename_one, 'a', 'b', after=True)
+        self.assertIsInstance(e.extra, errors.AlreadyVersionedError)
+
+    def test_rename_one_after_with_after_dest_added(self):
+        ''' using after with a newly added file should work '''
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('initial', rev_id='rev-1')
+        root_id = tree.get_root_id()
+        os.rename('a', 'b')
+        tree.add(['b'], ['b-id'])
+
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+                               tree)
+        tree.rename_one('a', 'b', after=True)
+        self.assertTreeLayout([('', root_id), ('b', 'a-id')], tree)
+
     def test_rename_one_after_source_removed(self):
-        """Rename even if the source was removed from the inventory already"""
+        """Rename even if the source was already unversioned."""
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/'])
         tree.add(['a', 'b'], ['a-id', 'b-id'])
@@ -230,13 +269,13 @@
         os.rename('a', 'b/foo')
         tree.remove(['a'])
 
-        self.assertTreeLayout([('', root_id), ('b', 'b-id')], tree)
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id')], tree)
         # We don't need after=True as long as source is missing and target
         # exists.
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
     def test_rename_one_after_no_target(self):
@@ -249,9 +288,9 @@
         # Passing after when the file hasn't been rename_one raises an exception
         self.assertRaises(errors.BzrMoveFailedError,
                           tree.rename_one, 'a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
     def test_rename_one_after_source_and_dest(self):
@@ -274,19 +313,19 @@
         finally:
             foo_file.close()
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.rename_one, 'a', 'b/foo', after=False)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(foo_text, 'b/foo')
         # But you can pass after=True
         tree.rename_one('a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         # But it shouldn't actually move anything
         self.assertFileEqual(a_text, 'a')
@@ -301,11 +340,11 @@
         root_id = tree.get_root_id()
 
         tree.rename_one('a', 'e/f')
-        self.assertTreeLayout([('', root_id), ('e', 'e-id'), ('e/f', 'a-id'),
-                               ('e/f/b', 'b-id'), ('e/f/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('e/', 'e-id'), ('e/f/', 'a-id'),
+                               ('e/f/b', 'b-id'), ('e/f/c/', 'c-id'),
                                ('e/f/c/d', 'd-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('e', 'e-id'),
-                               ('a/b', 'b-id'), ('a/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('e/', 'e-id'),
+                               ('a/b', 'b-id'), ('a/c/', 'c-id'),
                                ('a/c/d', 'd-id')], tree.basis_tree())
 
     def test_rename_one_moved(self):
@@ -317,15 +356,15 @@
         root_id = tree.get_root_id()
 
         tree.rename_one('a/b', 'c/foo')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('c/foo', 'b-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
 
         tree.rename_one('c/foo', 'bar')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('bar', 'b-id'),
-                               ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('bar', 'b-id'),
+                               ('c/', 'c-id')], tree)
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
 
     def test_rename_to_denormalised_fails(self):
@@ -336,3 +375,46 @@
         tree.add(['a'])
         self.assertRaises((errors.InvalidNormalization, UnicodeEncodeError),
             tree.rename_one, 'a', u'ba\u030arry')
+
+    def test_rename_unversioned_non_ascii(self):
+        """Check error when renaming an unversioned non-ascii file"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree([u"\xA7"])
+        e = self.assertRaises(errors.BzrRenameFailedError,
+            tree.rename_one, u"\xA7", "b")
+        self.assertIsInstance(e.extra, errors.NotVersionedError)
+        self.assertEqual(e.extra.path, u"\xA7")
+
+    def test_rename_into_unversioned_non_ascii_dir(self):
+        """Check error when renaming into unversioned non-ascii directory"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["a", u"\xA7/"])
+        tree.add(["a"])
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.rename_one, "a", u"\xA7/a")
+        self.assertIsInstance(e.extra, errors.NotVersionedError)
+        self.assertEqual(e.extra.path, u"\xA7")
+
+    def test_rename_over_already_versioned_non_ascii(self):
+        """Check error renaming over an already versioned non-ascii file"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["a", u"\xA7"])
+        tree.add(["a", u"\xA7"])
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.rename_one, "a", u"\xA7")
+        self.assertIsInstance(e.extra, errors.AlreadyVersionedError)
+        self.assertEqual(e.extra.path, u"\xA7")
+
+    def test_rename_after_non_existant_non_ascii(self):
+        """Check error renaming after move with missing non-ascii file"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["a"])
+        tree.add(["a"])
+        e = self.assertRaises(errors.BzrMoveFailedError,
+            tree.rename_one, "a", u"\xA7", after=True)
+        self.assertIsInstance(e.extra, errors.NoSuchFile)
+        self.assertEqual(e.extra.path, u"\xA7")

=== modified file 'bzrlib/tests/per_workingtree/test_revision_tree.py'
--- bzrlib/tests/per_workingtree/test_revision_tree.py	2010-03-10 09:33:04 +0000
+++ bzrlib/tests/per_workingtree/test_revision_tree.py	2011-05-04 11:15:37 +0000
@@ -24,7 +24,6 @@
 """
 
 from bzrlib import (
-    branchbuilder,
     errors,
     tests,
     )

=== modified file 'bzrlib/tests/per_workingtree/test_set_root_id.py'
--- bzrlib/tests/per_workingtree/test_set_root_id.py	2010-01-13 23:06:42 +0000
+++ bzrlib/tests/per_workingtree/test_set_root_id.py	2011-05-04 21:10:36 +0000
@@ -18,7 +18,7 @@
 
 import sys
 
-from bzrlib import errors, inventory
+from bzrlib import errors
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 

=== modified file 'bzrlib/tests/per_workingtree/test_smart_add.py'
--- bzrlib/tests/per_workingtree/test_smart_add.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/per_workingtree/test_smart_add.py	2011-09-06 09:51:45 +0000
@@ -17,20 +17,20 @@
 """Test that we can use smart_add on all Tree implementations."""
 
 from cStringIO import StringIO
+import os
 import sys
 
 from bzrlib import (
-    add,
     errors,
     ignores,
     osutils,
     tests,
-    workingtree,
+    trace,
     )
 from bzrlib.tests import (
     features,
+    per_workingtree,
     test_smart_add,
-    per_workingtree,
     )
 
 
@@ -92,6 +92,19 @@
         for path in paths:
             self.assertNotEqual(wt.path2id(path), None)
 
+    def test_skip_nested_trees(self):
+        """Test smart-adding a nested tree ignors it and warns."""
+        wt = self.make_branch_and_tree('.')
+        nested_wt = self.make_branch_and_tree('nested')
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        wt.smart_add((u".",))
+        self.assertIs(wt.path2id("nested"), None)
+        self.assertEquals(
+            ['skipping nested tree %r' % nested_wt.basedir], warnings)
+
     def test_add_dot_from_subdir(self):
         """Test adding . from a subdir of the tree."""
         paths = ("original/", "original/file1", "original/file2")
@@ -151,7 +164,8 @@
         self.build_tree(tree_shape)
         wt.smart_add(add_paths)
         for path in expected_paths:
-            self.assertNotEqual(wt.path2id(path), None, "No id added for %s" % path)
+            self.assertNotEqual(wt.path2id(path), None,
+                "No id added for %s" % path)
 
     def test_add_non_existant(self):
         """Test smart-adding a file that does not exist."""
@@ -202,6 +216,37 @@
         self.assertEqual(['', 'dir', 'dir/subdir', 'dir/subdir/foo'],
             [path for path, ie in tree.iter_entries_by_dir()])
 
+    def test_add_dir_bug_251864(self):
+        """Added file turning into a dir should be detected on add dir
+
+        Similar to bug 205636 but with automatic adding of directory contents.
+        """
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["dir"]) # whoops, make a file called dir
+        tree.smart_add(["dir"])
+        os.remove("dir")
+        self.build_tree(["dir/", "dir/file"])
+        tree.smart_add(["dir"])
+        tree.commit("Add dir contents")
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([(u"dir", "directory"), (u"dir/file", "file")],
+            [(t[0], t[2]) for t in tree.list_files()])
+        self.assertFalse(list(tree.iter_changes(tree.basis_tree())))
+
+    def test_add_subdir_file_bug_205636(self):
+        """Added file turning into a dir should be detected on add dir/file"""
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["dir"]) # whoops, make a file called dir
+        tree.smart_add(["dir"])
+        os.remove("dir")
+        self.build_tree(["dir/", "dir/file"])
+        tree.smart_add(["dir/file"])
+        tree.commit("Add file in dir")
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([(u"dir", "directory"), (u"dir/file", "file")],
+            [(t[0], t[2]) for t in tree.list_files()])
+        self.assertFalse(list(tree.iter_changes(tree.basis_tree())))
+
     def test_custom_ids(self):
         sio = StringIO()
         action = test_smart_add.AddCustomIDAction(to_file=sio,
@@ -221,10 +266,10 @@
         self.addCleanup(wt.unlock)
         self.assertEqual([('', wt.path2id('')),
                           ('dir1', 'directory-dir1'),
+                          ('file1', 'file-file1'),
                           ('dir1/file2', 'file-dir1%file2'),
-                          ('file1', 'file-file1'),
                          ], [(path, ie.file_id) for path, ie
-                                in wt.inventory.iter_entries()])
+                                in wt.iter_entries_by_dir()])
 
 
 class TestSmartAddConflictRelatedFiles(per_workingtree.TestCaseWithWorkingTree):
@@ -259,7 +304,7 @@
 
 class TestSmartAddTreeUnicode(per_workingtree.TestCaseWithWorkingTree):
 
-    _test_needs_features = [tests.UnicodeFilenameFeature]
+    _test_needs_features = [features.UnicodeFilenameFeature]
 
     def setUp(self):
         super(TestSmartAddTreeUnicode, self).setUp()
@@ -267,37 +312,44 @@
         self.wt = self.make_branch_and_tree('.')
         self.overrideAttr(osutils, 'normalized_filename')
 
+    def test_requires_normalized_unicode_filenames_fails_on_unnormalized(self):
+        """Adding unnormalized unicode filenames fail if and only if the
+        workingtree format has the requires_normalized_unicode_filenames flag
+        set and the underlying filesystem doesn't normalize.
+        """
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        if (self.workingtree_format.requires_normalized_unicode_filenames
+            and sys.platform != 'darwin'):
+            self.assertRaises(
+                errors.NoSuchFile, self.wt.smart_add, [u'a\u030a'])
+        else:
+            self.wt.smart_add([u'a\u030a'])
+
     def test_accessible_explicit(self):
         osutils.normalized_filename = osutils._accessible_normalized_filename
-        if isinstance(self.workingtree_format, workingtree.WorkingTreeFormat2):
-            self.expectFailure(
-                'With WorkingTreeFormat2, smart_add requires'
-                ' normalized unicode filenames',
-                self.assertRaises, errors.NoSuchFile,
-                self.wt.smart_add, [u'a\u030a'])
-        else:
-            self.wt.smart_add([u'a\u030a'])
+        if self.workingtree_format.requires_normalized_unicode_filenames:
+            raise tests.TestNotApplicable(
+                'Working tree format smart_add requires normalized unicode '
+                'filenames')
+        self.wt.smart_add([u'a\u030a'])
         self.wt.lock_read()
         self.addCleanup(self.wt.unlock)
         self.assertEqual([('', 'directory'), (u'\xe5', 'file')],
                          [(path, ie.kind) for path,ie in
-                          self.wt.inventory.iter_entries()])
+                          self.wt.iter_entries_by_dir()])
 
     def test_accessible_implicit(self):
         osutils.normalized_filename = osutils._accessible_normalized_filename
-        if isinstance(self.workingtree_format, workingtree.WorkingTreeFormat2):
-            self.expectFailure(
-                'With WorkingTreeFormat2, smart_add requires'
-                ' normalized unicode filenames',
-                self.assertRaises, errors.NoSuchFile,
-                self.wt.smart_add, [])
-        else:
-            self.wt.smart_add([])
+        if self.workingtree_format.requires_normalized_unicode_filenames:
+            raise tests.TestNotApplicable(
+                'Working tree format smart_add requires normalized unicode '
+                'filenames')
+        self.wt.smart_add([])
         self.wt.lock_read()
         self.addCleanup(self.wt.unlock)
         self.assertEqual([('', 'directory'), (u'\xe5', 'file')],
                          [(path, ie.kind) for path,ie
-                          in self.wt.inventory.iter_entries()])
+                          in self.wt.iter_entries_by_dir()])
 
     def test_inaccessible_explicit(self):
         osutils.normalized_filename = osutils._inaccessible_normalized_filename

=== modified file 'bzrlib/tests/per_workingtree/test_symlinks.py'
--- bzrlib/tests/per_workingtree/test_symlinks.py	2010-07-16 15:49:06 +0000
+++ bzrlib/tests/per_workingtree/test_symlinks.py	2011-07-25 02:51:30 +0000
@@ -15,21 +15,26 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Test symlink support.
-
-See eg <https://bugs.launchpad.net/bzr/+bug/192859>
 """
 
+import os
+
 from bzrlib import (
-    builtins,
+    osutils,
     tests,
     workingtree,
     )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestSmartAddTree(TestCaseWithWorkingTree):
 
-    _test_needs_features = [tests.SymlinkFeature]
+    # See eg <https://bugs.launchpad.net/bzr/+bug/192859>
+
+    _test_needs_features = [features.SymlinkFeature]
 
     def test_smart_add_symlink(self):
         tree = self.make_branch_and_tree('tree')
@@ -53,6 +58,83 @@
         self.assertEqual('symlink',
             tree.kind(tree.path2id('link')))
 
+    def test_add_file_under_symlink(self):
+        # similar to 
+        # https://bugs.launchpad.net/bzr/+bug/192859/comments/3
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/link@', 'dir'),
+            ('tree/dir/',),
+            ('tree/dir/file', 'content'),
+            ])
+        self.assertEquals(
+            tree.smart_add(['tree/link/file']),
+            ([u'dir', u'dir/file'], {}))
+        # should add the actual parent directory, not the apparent parent
+        # (which is actually a symlink)
+        self.assertTrue(tree.path2id('dir/file'))
+        self.assertTrue(tree.path2id('dir'))
+        self.assertIs(None, tree.path2id('link'))
+        self.assertIs(None, tree.path2id('link/file'))
+
+
+class TestKindChanges(TestCaseWithWorkingTree):
+
+    _test_needs_features = [features.SymlinkFeature]
+
+    def test_symlink_changes_to_dir(self):
+        # <https://bugs.launchpad.net/bzr/+bug/192859>:
+        # we had some past problems with the workingtree remembering for too
+        # long what kind of object was at a particular name; we really
+        # shouldn't do that.  Operating on the dirstate through passing
+        # inventory deltas rather than mutating the inventory largely avoids
+        # that.
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/a@', 'target')])
+        tree.smart_add(['tree/a'])
+        tree.commit('add symlink')
+        os.unlink('tree/a')
+        self.build_tree_contents([
+            ('tree/a/',),
+            ('tree/a/f', 'content'),
+            ])
+        tree.smart_add(['tree/a/f'])
+        tree.commit('change to dir')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertEquals([], list(tree.iter_changes(tree.basis_tree())))
+        if tree._format.supports_versioned_directories:
+            self.assertEquals(
+                ['a', 'a/f'], sorted(info[0] for info in tree.list_files()))
+        else:
+            self.assertEquals([], list(tree.list_files()))
+
+    def test_dir_changes_to_symlink(self):
+        # <https://bugs.launchpad.net/bzr/+bug/192859>:
+        # we had some past problems with the workingtree remembering for too
+        # long what kind of object was at a particular name; we really
+        # shouldn't do that.  Operating on the dirstate through passing
+        # inventory deltas rather than mutating the inventory largely avoids
+        # that.
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([
+            ('tree/a/',),
+            ('tree/a/file', 'content'),
+            ])
+        tree.smart_add(['tree/a'])
+        tree.commit('add dir')
+        osutils.rmtree('tree/a')
+        self.build_tree_contents([
+            ('tree/a@', 'target'),
+            ])
+        tree.commit('change to symlink')
+
+
+class TestOpenTree(TestCaseWithWorkingTree):
+
+    _test_needs_features = [features.SymlinkFeature]
+
     def test_open_containing_through_symlink(self):
         self.make_test_tree()
         self.check_open_containing('link/content', 'tree', 'content')

=== modified file 'bzrlib/tests/per_workingtree/test_uncommit.py'
--- bzrlib/tests/per_workingtree/test_uncommit.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_uncommit.py	2011-05-04 11:15:37 +0000
@@ -16,12 +16,7 @@
 
 """Tests of the parent related functions of WorkingTrees."""
 
-import os
-
 from bzrlib import (
-    errors,
-    revision as _mod_revision,
-    symbol_versioning,
     uncommit,
     )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree

=== modified file 'bzrlib/tests/per_workingtree/test_unversion.py'
--- bzrlib/tests/per_workingtree/test_unversion.py	2009-07-15 04:33:14 +0000
+++ bzrlib/tests/per_workingtree/test_unversion.py	2011-05-04 11:15:37 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     errors,
-    osutils,
     )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
@@ -42,7 +41,7 @@
         # unversioning children of reparented children of unversioned
         # paths when relocation entries are present and the relocation
         # points later into the dirstate.
-        tree = self.make_branch_and_tree(['.'])
+        tree = self.make_branch_and_tree('.')
         self.build_tree(['del/', 'del/sub/', 'del/sub/b'])
         tree.add(['del', 'del/sub', 'del/sub/b'], ['del', 'sub', 'b'])
         tree.commit('setup')

=== modified file 'bzrlib/tests/per_workingtree/test_walkdirs.py'
--- bzrlib/tests/per_workingtree/test_walkdirs.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_walkdirs.py	2011-06-14 01:26:41 +0000
@@ -19,7 +19,7 @@
 import os
 
 from bzrlib import transform
-from bzrlib.tests import SymlinkFeature
+from bzrlib.tests.features import SymlinkFeature
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 # tests to write:

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2010-05-20 18:23:17 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2011-09-27 11:35:00 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #           and others
 #
@@ -19,32 +19,48 @@
 from cStringIO import StringIO
 import errno
 import os
-import sys
 
 from bzrlib import (
     branch,
     branchbuilder,
     bzrdir,
+    config,
     errors,
     osutils,
+    symbol_versioning,
     tests,
+    trace,
     urlutils,
-    workingtree,
-    )
-from bzrlib.errors import (NotBranchError, NotVersionedError,
-                           UnsupportedOperation, PathsNotVersionedError)
+    )
+from bzrlib.errors import (
+    UnsupportedOperation,
+    PathsNotVersionedError,
+    )
 from bzrlib.inventory import Inventory
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
-from bzrlib.tests import TestSkipped, TestNotApplicable
+from bzrlib.tests import (
+    features,
+    TestSkipped,
+    TestNotApplicable,
+    )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
-from bzrlib.trace import mutter
-from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
-                                WorkingTree, WorkingTree2)
+from bzrlib.workingtree import (
+    TreeDirectory,
+    TreeFile,
+    TreeLink,
+    InventoryWorkingTree,
+    WorkingTree,
+    )
 from bzrlib.conflicts import ConflictList, TextConflict, ContentsConflict
 
 
 class TestWorkingTree(TestCaseWithWorkingTree):
 
+    def test_branch_builder(self):
+        # Just a smoke test that we get a branch at the specified relpath
+        builder = self.make_branch_builder('foobar')
+        br = branch.Branch.open('foobar')
+
     def test_list_files(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/', 'file'])
@@ -195,7 +211,7 @@
         os.unlink('hello.txt')
         tree.remove('hello.txt')
         tree.revert(['hello.txt'])
-        self.failUnlessExists('hello.txt')
+        self.assertPathExists('hello.txt')
 
     def test_versioned_files_not_unknown(self):
         tree = self.make_branch_and_tree('.')
@@ -233,24 +249,22 @@
 
         wt.commit('create initial state')
 
-        revid = b.revision_history()[0]
+        revid = b.last_revision()
         self.log('first revision_id is {%s}' % revid)
 
-        inv = b.repository.get_inventory(revid)
-        self.log('contents of inventory: %r' % inv.entries())
+        tree = b.repository.revision_tree(revid)
+        self.log('contents of tree: %r' % list(tree.iter_entries_by_dir()))
 
-        self.check_inventory_shape(inv,
-                                   ['dir/', 'dir/sub/', 'dir/sub/file'])
+        self.check_tree_shape(tree, ['dir/', 'dir/sub/', 'dir/sub/file'])
         wt.rename_one('dir', 'newdir')
 
         wt.lock_read()
-        self.check_inventory_shape(wt.inventory,
+        self.check_tree_shape(wt,
                                    ['newdir/', 'newdir/sub/', 'newdir/sub/file'])
         wt.unlock()
         wt.rename_one('newdir/sub', 'newdir/newsub')
         wt.lock_read()
-        self.check_inventory_shape(wt.inventory,
-                                   ['newdir/', 'newdir/newsub/',
+        self.check_tree_shape(wt, ['newdir/', 'newdir/newsub/',
                                     'newdir/newsub/file'])
         wt.unlock()
 
@@ -264,9 +278,12 @@
         wt = self.make_branch_and_tree('.')
         self.build_tree(['foo/',
                          'foo/hello'])
-        self.assertRaises(NotVersionedError,
-                          wt.add,
-                          'foo/hello')
+        if not wt._format.supports_versioned_directories:
+            wt.add('foo/hello')
+        else:
+            self.assertRaises(NotVersionedError,
+                              wt.add,
+                              'foo/hello')
 
     def test_add_missing(self):
         # adding a msising file -> NoSuchFile
@@ -319,7 +336,8 @@
         # because some formats mutate the branch to set it on the tree
         # we need to alter the branch to let this pass.
         try:
-            wt.branch.set_revision_history(['A', 'B'])
+            self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+                wt.branch.set_revision_history, ['A', 'B'])
         except errors.NoSuchRevision, e:
             self.assertEqual('B', e.revision)
             raise TestSkipped("Branch format does not permit arbitrary"
@@ -390,12 +408,12 @@
         wt.set_parent_ids(['B'])
         tree = wt.basis_tree()
         tree.lock_read()
-        self.failUnless(tree.has_filename('bar'))
+        self.assertTrue(tree.has_filename('bar'))
         tree.unlock()
         wt.set_parent_ids(['A'])
         tree = wt.basis_tree()
         tree.lock_read()
-        self.failUnless(tree.has_filename('foo'))
+        self.assertTrue(tree.has_filename('foo'))
         tree.unlock()
 
     def test_clone_tree_revision(self):
@@ -456,7 +474,7 @@
         wt.commit('A', rev_id='A')
         # and update old_tree
         self.assertEqual(0, old_tree.update())
-        self.failUnlessExists('checkout/file')
+        self.assertPathExists('checkout/file')
         self.assertEqual(['A'], old_tree.get_parent_ids())
 
     def test_update_sets_root_id(self):
@@ -477,7 +495,7 @@
         wt.commit('A', rev_id='A')
         # and update checkout
         self.assertEqual(0, checkout.update())
-        self.failUnlessExists('checkout/file')
+        self.assertPathExists('checkout/file')
         self.assertEqual(wt.get_root_id(), checkout.get_root_id())
         self.assertNotEqual(None, wt.get_root_id())
 
@@ -558,10 +576,10 @@
             a.close()
         this.revert()
         self.assertFileEqual('a test\n', 'b1/a')
-        self.failUnlessExists('b1/b.~1~')
-        self.failIfExists('b1/c')
-        self.failIfExists('b1/a.~1~')
-        self.failUnlessExists('b1/d')
+        self.assertPathExists('b1/b.~1~')
+        self.assertPathDoesNotExist('b1/c')
+        self.assertPathDoesNotExist('b1/a.~1~')
+        self.assertPathExists('b1/d')
 
     def test_update_updates_bound_branch_no_local_commits(self):
         # doing an update in a tree updates the branch its bound to too.
@@ -605,8 +623,8 @@
         # which should have pivoted the local tip into a merge
         self.assertEqual([master_tip, 'bar'], tree.get_parent_ids())
         # and the local branch history should match the masters now.
-        self.assertEqual(master_tree.branch.revision_history(),
-            tree.branch.revision_history())
+        self.assertEqual(master_tree.branch.last_revision(),
+            tree.branch.last_revision())
 
     def test_update_takes_revision_parameter(self):
         wt = self.make_branch_and_tree('wt')
@@ -626,6 +644,9 @@
         # FIXME: This doesn't really test that it works; also this is not
         # implementation-independent. mbp 20070226
         tree = self.make_branch_and_tree('master')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable("merge-hashes is specific to bzr "
+                "working trees")
         tree._transport.put_bytes('merge-hashes', 'asdfasdf')
         self.assertRaises(errors.MergeModifiedFormatError, tree.merge_modified)
 
@@ -730,7 +751,7 @@
     def test_format_description(self):
         tree = self.make_branch_and_tree('tree')
         text = tree._format.get_format_description()
-        self.failUnless(len(text))
+        self.assertTrue(len(text))
 
     def test_branch_attribute_is_not_settable(self):
         # the branch attribute is an aspect of the working tree, not a
@@ -767,7 +788,7 @@
             tree.lock_read()
             self.assertEqual([('', 'directory'), (u'\xe5', 'file')],
                     [(path, ie.kind) for path,ie in
-                                tree.inventory.iter_entries()])
+                                tree.iter_entries_by_dir()])
             tree.unlock()
         finally:
             osutils.normalized_filename = orig
@@ -789,6 +810,9 @@
     def test__write_inventory(self):
         # The private interface _write_inventory is currently used by transform.
         tree = self.make_branch_and_tree('.')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable("_write_inventory does not exist on "
+                "non-inventory working trees")
         # if we write write an inventory then do a walkdirs we should get back
         # missing entries, and actual, and unknowns as appropriate.
         self.build_tree(['present', 'unknown'])
@@ -915,9 +939,20 @@
         else:
             case_sensitive = True
         tree = self.make_branch_and_tree('test')
-        if tree.__class__ == WorkingTree2:
-            raise TestSkipped('WorkingTree2 is not supported')
         self.assertEqual(case_sensitive, tree.case_sensitive)
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable("get_format_string is only available "
+                                    "on bzr working trees")
+        # now we cheat, and make a file that matches the case-sensitive name
+        t = tree.bzrdir.get_workingtree_transport(None)
+        try:
+            content = tree._format.get_format_string()
+        except NotImplementedError:
+            # All-in-one formats didn't have a separate format string.
+            content = tree.bzrdir._format.get_format_string()
+        t.put_bytes(tree._format.case_sensitive_filename, content)
+        tree = tree.bzrdir.open_workingtree()
+        self.assertFalse(tree.case_sensitive)
 
     def test_all_file_ids_with_missing(self):
         tree = self.make_branch_and_tree('tree')
@@ -969,10 +1004,9 @@
             4 5-M
             |
             W
-         """
-        builder = branchbuilder.BranchBuilder(
-            self.get_transport(),
-            format=self.workingtree_format._matchingbzrdir)
+        """
+        format = self.workingtree_format.get_controldir_for_branch()
+        builder = self.make_branch_builder(".", format=format)
         builder.start_series()
         # mainline
         builder.build_snapshot(
@@ -1054,9 +1088,9 @@
         self.assertEqual(0, wt.update(revision='1'))
         self.assertEqual('1', wt.last_revision())
         self.assertEqual(tip, wt.branch.last_revision())
-        self.failUnlessExists('checkout/file1')
-        self.failIfExists('checkout/file4')
-        self.failIfExists('checkout/file5')
+        self.assertPathExists('checkout/file1')
+        self.assertPathDoesNotExist('checkout/file4')
+        self.assertPathDoesNotExist('checkout/file5')
 
 
 class TestIllegalPaths(TestCaseWithWorkingTree):
@@ -1065,7 +1099,7 @@
         if osutils.normalizes_filenames():
             # You *can't* create an illegal filename on OSX.
             raise tests.TestNotApplicable('OSX normalizes filenames')
-        self.requireFeature(tests.UTF8Filesystem)
+        self.requireFeature(features.UTF8Filesystem)
         # We require a UTF8 filesystem, because otherwise we would need to get
         # tricky to figure out how to create an illegal filename.
         # \xb5 is an illegal path because it should be \xc2\xb5 for UTF-8
@@ -1095,7 +1129,7 @@
 
 class TestControlComponent(TestCaseWithWorkingTree):
     """WorkingTree implementations adequately implement ControlComponent."""
-    
+
     def test_urls(self):
         wt = self.make_branch_and_tree('wt')
         self.assertIsInstance(wt.user_url, str)
@@ -1104,3 +1138,53 @@
         # above the control dir but we might need to relax that?
         self.assertEqual(wt.control_url.find(wt.user_url), 0)
         self.assertEqual(wt.control_url, wt.control_transport.base)
+
+
+class TestWorthSavingLimit(TestCaseWithWorkingTree):
+
+    def make_wt_with_worth_saving_limit(self):
+        wt = self.make_branch_and_tree('wt')
+        if getattr(wt, '_worth_saving_limit', None) is None:
+            raise tests.TestNotApplicable('no _worth_saving_limit for'
+                                          ' this tree type')
+        wt.lock_write()
+        self.addCleanup(wt.unlock)
+        return wt
+
+    def test_not_set(self):
+        # Default should be 10
+        wt = self.make_wt_with_worth_saving_limit()
+        self.assertEqual(10, wt._worth_saving_limit())
+        ds = wt.current_dirstate()
+        self.assertEqual(10, ds._worth_saving_limit)
+
+    def test_set_in_branch(self):
+        wt = self.make_wt_with_worth_saving_limit()
+        conf = config.BranchStack(wt.branch)
+        conf.set('bzr.workingtree.worth_saving_limit', '20')
+        self.assertEqual(20, wt._worth_saving_limit())
+        ds = wt.current_dirstate()
+        self.assertEqual(10, ds._worth_saving_limit)
+
+    def test_invalid(self):
+        wt = self.make_wt_with_worth_saving_limit()
+        conf = config.BranchStack(wt.branch)
+        conf.set('bzr.workingtree.worth_saving_limit', 'a')
+        # If the config entry is invalid, default to 10
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        self.assertEqual(10, wt._worth_saving_limit())
+        self.assertLength(1, warnings)
+        self.assertEquals('Value "a" is not valid for'
+                          ' "bzr.workingtree.worth_saving_limit"',
+                          warnings[0])
+
+
+class TestFormatAttributes(TestCaseWithWorkingTree):
+
+    def test_versioned_directories(self):
+        self.assertSubset(
+            [self.workingtree_format.supports_versioned_directories],
+            (True, False))

=== added file 'bzrlib/tests/scenarios.py'
--- bzrlib/tests/scenarios.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/scenarios.py	2010-11-05 20:54:32 +0000
@@ -0,0 +1,61 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Generate multiple variations in different scenarios.
+
+For a class whose tests should be repeated in varying scenarios, set a
+`scenarios` member to a list of scenarios where it should be repeated.
+
+This is similar to the interface provided by
+<http://launchpad.net/testscenarios/>.
+"""
+
+
+from bzrlib.tests import (
+    iter_suite_tests,
+    multiply_scenarios,
+    multiply_tests,
+    )
+
+
+def load_tests_apply_scenarios(standard_tests, module, loader):
+    """Multiply tests depending on their 'scenarios' attribute.
+
+    This can be assigned to 'load_tests' in any test module to make this
+    automatically work across tests in the module.
+    """
+    result = loader.suiteClass()
+    multiply_tests_by_their_scenarios(standard_tests, result)
+    return result
+
+
+def multiply_tests_by_their_scenarios(some_tests, into_suite):
+    """Multiply the tests in the given suite by their declared scenarios.
+
+    Each test must have a 'scenarios' attribute which is a list of 
+    (name, params) pairs.
+
+    :param some_tests: TestSuite or Test.
+    :param into_suite: A TestSuite into which the resulting tests will be
+        inserted.
+    """
+    for test in iter_suite_tests(some_tests):
+        scenarios = getattr(test, 'scenarios', None)
+        if scenarios is None:
+            into_suite.addTest(test)
+        else:
+            multiply_tests(test, test.scenarios, into_suite)

=== modified file 'bzrlib/tests/script.py'
--- bzrlib/tests/script.py	2010-06-11 06:33:15 +0000
+++ bzrlib/tests/script.py	2011-09-28 15:06:42 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 import os
 import shlex
 import textwrap
-from cStringIO import StringIO
 
 from bzrlib import (
     osutils,
@@ -56,6 +55,10 @@
     Input lines start with '<'.
     Output lines start with nothing.
     Error lines start with '2>'.
+
+    :return: A sequence of ([args], input, output, errors), where the args are
+        split in to words, and the input, output, and errors are just strings,
+        typically containing newlines.
     """
 
     commands = []
@@ -75,18 +78,26 @@
     lineno = 0
     input, output, error = None, None, None
     text = textwrap.dedent(text)
-    for line in text.split('\n'):
+    lines = text.split('\n')
+    # to make use of triple-quoted strings easier, we ignore a blank line
+    # right at the start and right at the end; the rest are meaningful
+    if lines and lines[0] == '':
+        del lines[0]
+    if lines and lines[-1] == '':
+        del lines[-1]
+    for line in lines:
         lineno += 1
         # Keep a copy for error reporting
         orig = line
         comment =  line.find('#')
         if comment >= 0:
             # Delete comments
+            # NB: this syntax means comments are allowed inside output, which
+            # may be confusing...
             line = line[0:comment]
             line = line.rstrip()
-        if line == '':
-            # Ignore empty lines
-            continue
+            if line == '':
+                continue
         if line.startswith('$'):
             # Time to output the current command
             add_command(cmd_cur, input, output, error)
@@ -183,7 +194,7 @@
         self.output_checker = doctest.OutputChecker()
         self.check_options = doctest.ELLIPSIS
 
-    def run_script(self, test_case, text):
+    def run_script(self, test_case, text, null_output_matches_anything=False):
         """Run a shell-like script as a test.
 
         :param test_case: A TestCase instance that should provide the fail(),
@@ -191,7 +202,12 @@
             attribute used as a jail root.
 
         :param text: A shell-like script (see _script_to_commands for syntax).
+
+        :param null_output_matches_anything: For commands with no specified
+            output, ignore any output that does happen, including output on
+            standard error.
         """
+        self.null_output_matches_anything = null_output_matches_anything
         for cmd, input, output, error in _script_to_commands(text):
             self.run_command(test_case, cmd, input, output, error)
 
@@ -200,7 +216,7 @@
         method = getattr(self, mname, None)
         if method is None:
             raise SyntaxError('Command not found "%s"' % (cmd[0],),
-                              None, 1, ' '.join(cmd))
+                              (None, 1, 1, ' '.join(cmd)))
         if input is None:
             str_input = ''
         else:
@@ -209,20 +225,36 @@
         retcode, actual_output, actual_error = method(test_case,
                                                       str_input, args)
 
-        self._check_output(output, actual_output, test_case)
-        self._check_output(error, actual_error, test_case)
+        try:
+            self._check_output(output, actual_output, test_case)
+        except AssertionError, e:
+            raise AssertionError(str(e) + " in stdout of command %s" % cmd)
+        try:
+            self._check_output(error, actual_error, test_case)
+        except AssertionError, e:
+            raise AssertionError(str(e) +
+                " in stderr of running command %s" % cmd)
         if retcode and not error and actual_error:
             test_case.fail('In \n\t%s\nUnexpected error: %s'
                            % (' '.join(cmd), actual_error))
         return retcode, actual_output, actual_error
 
     def _check_output(self, expected, actual, test_case):
-        if expected is None:
-            # Specifying None means: any output is accepted
+        if not actual:
+            if expected is None:
+                return
+            elif expected == '...\n':
+                return
+            else:
+                test_case.fail('expected output: %r, but found nothing'
+                            % (expected,))
+
+        null_output_matches_anything = getattr(
+            self, 'null_output_matches_anything', False)
+        if null_output_matches_anything and expected is None:
             return
-        if actual is None:
-            test_case.fail('We expected output: %r, but found None'
-                           % (expected,))
+
+        expected = expected or ''
         matching = self.output_checker.check_output(
             expected, actual, self.check_options)
         if not matching:
@@ -232,7 +264,15 @@
             # 'expected' parameter. So we just fallback to our good old
             # assertEqualDiff since we know there *are* differences and the
             # output should be decently readable.
-            test_case.assertEqualDiff(expected, actual)
+            #
+            # As a special case, we allow output that's missing a final
+            # newline to match an expected string that does have one, so that
+            # we can match a prompt printed on one line, then input given on
+            # the next line.
+            if expected == actual + '\n':
+                pass
+            else:
+                test_case.assertEqualDiff(expected, actual)
 
     def _pre_process_args(self, args):
         new_args = []
@@ -441,9 +481,14 @@
     def setUp(self):
         super(TestCaseWithMemoryTransportAndScript, self).setUp()
         self.script_runner = ScriptRunner()
+        # FIXME: See shelf_ui.Shelver._char_based. This allow using shelve in
+        # scripts while providing a line-based input (better solution in
+        # progress). -- vila 2011-09-28
+        self.overrideEnv('INSIDE_EMACS', '1')
 
-    def run_script(self, script):
-        return self.script_runner.run_script(self, script)
+    def run_script(self, script, null_output_matches_anything=False):
+        return self.script_runner.run_script(self, script, 
+                   null_output_matches_anything=null_output_matches_anything)
 
     def run_command(self, cmd, input, output, error):
         return self.script_runner.run_command(self, cmd, input, output, error)
@@ -470,14 +515,21 @@
     def setUp(self):
         super(TestCaseWithTransportAndScript, self).setUp()
         self.script_runner = ScriptRunner()
+        # FIXME: See shelf_ui.Shelver._char_based. This allow using shelve in
+        # scripts while providing a line-based input (better solution in
+        # progress). -- vila 2011-09-28
+        self.overrideEnv('INSIDE_EMACS', '1')
 
-    def run_script(self, script):
-        return self.script_runner.run_script(self, script)
+    def run_script(self, script, null_output_matches_anything=False):
+        return self.script_runner.run_script(self, script,
+                   null_output_matches_anything=null_output_matches_anything)
 
     def run_command(self, cmd, input, output, error):
         return self.script_runner.run_command(self, cmd, input, output, error)
 
 
-def run_script(test_case, script_string):
+def run_script(test_case, script_string, null_output_matches_anything=False):
     """Run the given script within a testcase"""
-    return ScriptRunner().run_script(test_case, script_string)
+    return ScriptRunner().run_script(test_case, script_string,
+               null_output_matches_anything=null_output_matches_anything)
+

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2010-06-02 04:50:35 +0000
+++ bzrlib/tests/stub_sftp.py	2011-10-06 07:43:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Robey Pointer <robey@lag.net>, Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2011 Robey Pointer <robey@lag.net>, Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,10 +21,9 @@
 
 import os
 import paramiko
-import select
 import socket
+import SocketServer
 import sys
-import threading
 import time
 
 from bzrlib import (
@@ -38,24 +37,24 @@
 from bzrlib.tests import test_server
 
 
-class StubServer (paramiko.ServerInterface):
+class StubServer(paramiko.ServerInterface):
 
-    def __init__(self, test_case):
+    def __init__(self, test_case_server):
         paramiko.ServerInterface.__init__(self)
-        self._test_case = test_case
+        self.log = test_case_server.log
 
     def check_auth_password(self, username, password):
         # all are allowed
-        self._test_case.log('sftpserver - authorizing: %s' % (username,))
+        self.log('sftpserver - authorizing: %s' % (username,))
         return paramiko.AUTH_SUCCESSFUL
 
     def check_channel_request(self, kind, chanid):
-        self._test_case.log(
-            'sftpserver - channel request: %s, %s' % (kind, chanid))
+        self.log('sftpserver - channel request: %s, %s' % (kind, chanid))
         return paramiko.OPEN_SUCCEEDED
 
 
-class StubSFTPHandle (paramiko.SFTPHandle):
+class StubSFTPHandle(paramiko.SFTPHandle):
+
     def stat(self):
         try:
             return paramiko.SFTPAttributes.from_stat(
@@ -73,7 +72,7 @@
             return paramiko.SFTPServer.convert_errno(e.errno)
 
 
-class StubSFTPServer (paramiko.SFTPServerInterface):
+class StubSFTPServer(paramiko.SFTPServerInterface):
 
     def __init__(self, server, root, home=None):
         paramiko.SFTPServerInterface.__init__(self, server)
@@ -90,7 +89,7 @@
             self.home = home[len(self.root):]
         if self.home.startswith('/'):
             self.home = self.home[1:]
-        server._test_case.log('sftpserver - new connection')
+        server.log('sftpserver - new connection')
 
     def _realpath(self, path):
         # paths returned from self.canonicalize() always start with
@@ -119,9 +118,9 @@
     else:
         def canonicalize(self, path):
             if os.path.isabs(path):
-                return os.path.normpath(path)
+                return osutils.normpath(path)
             else:
-                return os.path.normpath('/' + os.path.join(self.home, path))
+                return osutils.normpath('/' + os.path.join(self.home, path))
 
     def chattr(self, path, attr):
         try:
@@ -241,6 +240,7 @@
     # removed: chattr, symlink, readlink
     # (nothing in bzr's sftp transport uses those)
 
+
 # ------------- server test implementation --------------
 
 STUB_SERVER_KEY = """
@@ -262,55 +262,6 @@
 """
 
 
-class SocketListener(threading.Thread):
-
-    def __init__(self, callback):
-        threading.Thread.__init__(self)
-        self._callback = callback
-        self._socket = socket.socket()
-        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        self._socket.bind(('localhost', 0))
-        self._socket.listen(1)
-        self.host, self.port = self._socket.getsockname()[:2]
-        self._stop_event = threading.Event()
-
-    def stop(self):
-        # called from outside this thread
-        self._stop_event.set()
-        # use a timeout here, because if the test fails, the server thread may
-        # never notice the stop_event.
-        self.join(5.0)
-        self._socket.close()
-
-    def run(self):
-        trace.mutter('SocketListener %r has started', self)
-        while True:
-            readable, writable_unused, exception_unused = \
-                select.select([self._socket], [], [], 0.1)
-            if self._stop_event.isSet():
-                trace.mutter('SocketListener %r has stopped', self)
-                return
-            if len(readable) == 0:
-                continue
-            try:
-                s, addr_unused = self._socket.accept()
-                trace.mutter('SocketListener %r has accepted connection %r',
-                    self, s)
-                # because the loopback socket is inline, and transports are
-                # never explicitly closed, best to launch a new thread.
-                threading.Thread(target=self._callback, args=(s,)).start()
-            except socket.error, x:
-                sys.excepthook(*sys.exc_info())
-                trace.warning('Socket error during accept() '
-                              'within unit test server thread: %r' % x)
-            except Exception, x:
-                # probably a failed test; unit test thread will log the
-                # failure/error
-                sys.excepthook(*sys.exc_info())
-                trace.warning(
-                    'Exception from within unit test server thread: %r' % x)
-
-
 class SocketDelay(object):
     """A socket decorator to make TCP appear slower.
 
@@ -386,55 +337,139 @@
         return bytes_sent
 
 
-class SFTPServer(test_server.TestServer):
+class TestingSFTPConnectionHandler(SocketServer.BaseRequestHandler):
+
+    def setup(self):
+        self.wrap_for_latency()
+        tcs = self.server.test_case_server
+        ptrans = paramiko.Transport(self.request)
+        self.paramiko_transport = ptrans
+        # Set it to a channel under 'bzr' so that we get debug info
+        ptrans.set_log_channel('bzr.paramiko.transport')
+        ptrans.add_server_key(tcs.get_host_key())
+        ptrans.set_subsystem_handler('sftp', paramiko.SFTPServer,
+                                     StubSFTPServer, root=tcs._root,
+                                     home=tcs._server_homedir)
+        server = tcs._server_interface(tcs)
+        # This blocks until the key exchange has been done
+        ptrans.start_server(None, server)
+
+    def finish(self):
+        # Wait for the conversation to finish, when the paramiko.Transport
+        # thread finishes
+        # TODO: Consider timing out after XX seconds rather than hanging.
+        #       Also we could check paramiko_transport.active and possibly
+        #       paramiko_transport.getException().
+        self.paramiko_transport.join()
+
+    def wrap_for_latency(self):
+        tcs = self.server.test_case_server
+        if tcs.add_latency:
+            # Give the socket (which the request really is) a latency adding
+            # decorator.
+            self.request = SocketDelay(self.request, tcs.add_latency)
+
+
+class TestingSFTPWithoutSSHConnectionHandler(TestingSFTPConnectionHandler):
+
+    def setup(self):
+        self.wrap_for_latency()
+        # Re-import these as locals, so that they're still accessible during
+        # interpreter shutdown (when all module globals get set to None, leading
+        # to confusing errors like "'NoneType' object has no attribute 'error'".
+        class FakeChannel(object):
+            def get_transport(self):
+                return self
+            def get_log_channel(self):
+                return 'bzr.paramiko'
+            def get_name(self):
+                return '1'
+            def get_hexdump(self):
+                return False
+            def close(self):
+                pass
+
+        tcs = self.server.test_case_server
+        sftp_server = paramiko.SFTPServer(
+            FakeChannel(), 'sftp', StubServer(tcs), StubSFTPServer,
+            root=tcs._root, home=tcs._server_homedir)
+        self.sftp_server = sftp_server
+        sys_stderr = sys.stderr # Used in error reporting during shutdown
+        try:
+            sftp_server.start_subsystem(
+                'sftp', None, ssh.SocketAsChannelAdapter(self.request))
+        except socket.error, e:
+            if (len(e.args) > 0) and (e.args[0] == errno.EPIPE):
+                # it's okay for the client to disconnect abruptly
+                # (bug in paramiko 1.6: it should absorb this exception)
+                pass
+            else:
+                raise
+        except Exception, e:
+            # This typically seems to happen during interpreter shutdown, so
+            # most of the useful ways to report this error won't work.
+            # Writing the exception type, and then the text of the exception,
+            # seems to be the best we can do.
+            # FIXME: All interpreter shutdown errors should have been related
+            # to daemon threads, cleanup needed -- vila 20100623
+            sys_stderr.write('\nEXCEPTION %r: ' % (e.__class__,))
+            sys_stderr.write('%s\n\n' % (e,))
+
+    def finish(self):
+        self.sftp_server.finish_subsystem()
+
+
+class TestingSFTPServer(test_server.TestingThreadingTCPServer):
+
+    def __init__(self, server_address, request_handler_class, test_case_server):
+        test_server.TestingThreadingTCPServer.__init__(
+            self, server_address, request_handler_class)
+        self.test_case_server = test_case_server
+
+
+class SFTPServer(test_server.TestingTCPServerInAThread):
     """Common code for SFTP server facilities."""
 
     def __init__(self, server_interface=StubServer):
+        self.host = '127.0.0.1'
+        self.port = 0
+        super(SFTPServer, self).__init__((self.host, self.port),
+                                         TestingSFTPServer,
+                                         TestingSFTPConnectionHandler)
         self._original_vendor = None
-        self._homedir = None
-        self._server_homedir = None
-        self._listener = None
-        self._root = None
         self._vendor = ssh.ParamikoVendor()
         self._server_interface = server_interface
-        # sftp server logs
+        self._host_key = None
         self.logs = []
         self.add_latency = 0
+        self._homedir = None
+        self._server_homedir = None
+        self._root = None
 
     def _get_sftp_url(self, path):
         """Calculate an sftp url to this server for path."""
-        return 'sftp://foo:bar@%s:%d/%s' % (self._listener.host,
-                                            self._listener.port, path)
+        return "sftp://foo:bar@%s:%s/%s" % (self.host, self.port, path)
 
     def log(self, message):
         """StubServer uses this to log when a new server is created."""
         self.logs.append(message)
 
-    def _run_server_entry(self, sock):
-        """Entry point for all implementations of _run_server.
-
-        If self.add_latency is > 0.000001 then sock is given a latency adding
-        decorator.
-        """
-        if self.add_latency > 0.000001:
-            sock = SocketDelay(sock, self.add_latency)
-        return self._run_server(sock)
-
-    def _run_server(self, s):
-        ssh_server = paramiko.Transport(s)
-        key_file = osutils.pathjoin(self._homedir, 'test_rsa.key')
-        f = open(key_file, 'w')
-        f.write(STUB_SERVER_KEY)
-        f.close()
-        host_key = paramiko.RSAKey.from_private_key_file(key_file)
-        ssh_server.add_server_key(host_key)
-        server = self._server_interface(self)
-        ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer,
-                                         StubSFTPServer, root=self._root,
-                                         home=self._server_homedir)
-        event = threading.Event()
-        ssh_server.start_server(event, server)
-        event.wait(5.0)
+    def create_server(self):
+        server = self.server_class((self.host, self.port),
+                                   self.request_handler_class,
+                                   self)
+        return server
+
+    def get_host_key(self):
+        if self._host_key is None:
+            key_file = osutils.pathjoin(self._homedir, 'test_rsa.key')
+            f = open(key_file, 'w')
+            try:
+                f.write(STUB_SERVER_KEY)
+            finally:
+                f.close()
+            self._host_key = paramiko.RSAKey.from_private_key_file(key_file)
+        return self._host_key
 
     def start_server(self, backing_server=None):
         # XXX: TODO: make sftpserver back onto backing_server rather than local
@@ -459,17 +494,17 @@
         self._root = '/'
         if sys.platform == 'win32':
             self._root = ''
-        self._listener = SocketListener(self._run_server_entry)
-        self._listener.setDaemon(True)
-        self._listener.start()
+        super(SFTPServer, self).start_server()
 
     def stop_server(self):
-        self._listener.stop()
-        ssh._ssh_vendor_manager._cached_ssh_vendor = self._original_vendor
+        try:
+            super(SFTPServer, self).stop_server()
+        finally:
+            ssh._ssh_vendor_manager._cached_ssh_vendor = self._original_vendor
 
     def get_bogus_url(self):
         """See bzrlib.transport.Server.get_bogus_url."""
-        # this is chosen to try to prevent trouble with proxies, wierd dns, etc
+        # this is chosen to try to prevent trouble with proxies, weird dns, etc
         # we bind a random socket, so that we get a guaranteed unused port
         # we just never listen on that port
         s = socket.socket()
@@ -495,45 +530,10 @@
     def __init__(self):
         super(SFTPServerWithoutSSH, self).__init__()
         self._vendor = ssh.LoopbackVendor()
-
-    def _run_server(self, sock):
-        # Re-import these as locals, so that they're still accessible during
-        # interpreter shutdown (when all module globals get set to None, leading
-        # to confusing errors like "'NoneType' object has no attribute 'error'".
-        class FakeChannel(object):
-            def get_transport(self):
-                return self
-            def get_log_channel(self):
-                return 'paramiko'
-            def get_name(self):
-                return '1'
-            def get_hexdump(self):
-                return False
-            def close(self):
-                pass
-
-        server = paramiko.SFTPServer(
-            FakeChannel(), 'sftp', StubServer(self), StubSFTPServer,
-            root=self._root, home=self._server_homedir)
-        try:
-            server.start_subsystem(
-                'sftp', None, ssh.SocketAsChannelAdapter(sock))
-        except socket.error, e:
-            if (len(e.args) > 0) and (e.args[0] == errno.EPIPE):
-                # it's okay for the client to disconnect abruptly
-                # (bug in paramiko 1.6: it should absorb this exception)
-                pass
-            else:
-                raise
-        except Exception, e:
-            # This typically seems to happen during interpreter shutdown, so
-            # most of the useful ways to report this error are won't work.
-            # Writing the exception type, and then the text of the exception,
-            # seems to be the best we can do.
-            import sys
-            sys.stderr.write('\nEXCEPTION %r: ' % (e.__class__,))
-            sys.stderr.write('%s\n\n' % (e,))
-        server.finish_subsystem()
+        self.request_handler_class = TestingSFTPWithoutSSHConnectionHandler
+
+    def get_host_key():
+        return None
 
 
 class SFTPAbsoluteServer(SFTPServerWithoutSSH):
@@ -553,7 +553,7 @@
 
     def get_url(self):
         """See bzrlib.transport.Server.get_url."""
-        return self._get_sftp_url("~/")
+        return self._get_sftp_url("%7E/")
 
 
 class SFTPSiblingAbsoluteServer(SFTPAbsoluteServer):
@@ -562,7 +562,9 @@
     It does this by serving from a deeply-nested directory that doesn't exist.
     """
 
-    def start_server(self, backing_server=None):
-        self._server_homedir = '/dev/noone/runs/tests/here'
-        super(SFTPSiblingAbsoluteServer, self).start_server(backing_server)
+    def create_server(self):
+        # FIXME: Can't we do that in a cleaner way ? -- vila 20100623
+        server = super(SFTPSiblingAbsoluteServer, self).create_server()
+        server._server_homedir = '/dev/noone/runs/tests/here'
+        return server
 

=== modified file 'bzrlib/tests/test__annotator.py'
--- bzrlib/tests/test__annotator.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__annotator.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     annotate,
-    _annotator_py,
     errors,
     knit,
     revision,

=== modified file 'bzrlib/tests/test__bencode.py'
--- bzrlib/tests/test__bencode.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__bencode.py	2010-09-17 04:32:50 +0000
@@ -16,6 +16,8 @@
 
 """Tests for bencode structured encoding"""
 
+import sys
+
 from bzrlib import tests
 
 def load_tests(standard_tests, module, loader):
@@ -105,7 +107,14 @@
                             'd8:spam.mp3d6:author5:Alice6:lengthi100000eee')
 
     def test_dict_deepnested(self):
-        self._run_check_error(RuntimeError, ("d0:" * 10000) + 'i1e' + ("e" * 10000))
+        # The recursion here provokes CPython into emitting a warning on
+        # stderr, "maximum recursion depth exceeded in __subclasscheck__", due
+        # to running out of stack space while evaluating "except (...):" in
+        # _bencode_py.  This is harmless, so we temporarily override stderr to
+        # avoid distracting noise in the test output.
+        self.overrideAttr(sys, 'stderr', self._log_file)
+        self._run_check_error(
+            RuntimeError, ("d0:" * 10000) + 'i1e' + ("e" * 10000))
 
     def test_malformed_dict(self):
         self._run_check_error(ValueError, 'd')

=== added file 'bzrlib/tests/test__btree_serializer.py'
--- bzrlib/tests/test__btree_serializer.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test__btree_serializer.py	2010-08-23 17:51:17 +0000
@@ -0,0 +1,305 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+#
+
+"""Direct tests of the btree serializer extension"""
+
+import binascii
+import bisect
+
+from bzrlib import tests
+
+from bzrlib.tests.test_btree_index import compiled_btreeparser_feature
+
+
+class TestBtreeSerializer(tests.TestCase):
+
+    _test_needs_features = [compiled_btreeparser_feature]
+
+    def setUp(self):
+        super(TestBtreeSerializer, self).setUp()
+        self.module = compiled_btreeparser_feature.module
+
+
+class TestHexAndUnhex(TestBtreeSerializer):
+
+    def assertHexlify(self, as_binary):
+        self.assertEqual(binascii.hexlify(as_binary),
+                         self.module._py_hexlify(as_binary))
+
+    def assertUnhexlify(self, as_hex):
+        ba_unhex = binascii.unhexlify(as_hex)
+        mod_unhex = self.module._py_unhexlify(as_hex)
+        if ba_unhex != mod_unhex:
+            if mod_unhex is None:
+                mod_hex = '<None>'
+            else:
+                mod_hex = binascii.hexlify(mod_unhex)
+            self.fail('_py_unhexlify returned a different answer'
+                      ' from binascii:\n    %s\n != %s'
+                      % (binascii.hexlify(ba_unhex), mod_hex))
+
+    def assertFailUnhexlify(self, as_hex):
+        # Invalid hex content
+        self.assertIs(None, self.module._py_unhexlify(as_hex))
+
+    def test_to_hex(self):
+        raw_bytes = ''.join(map(chr, range(256)))
+        for i in range(0, 240, 20):
+            self.assertHexlify(raw_bytes[i:i+20])
+        self.assertHexlify(raw_bytes[240:]+raw_bytes[0:4])
+
+    def test_from_hex(self):
+        self.assertUnhexlify('0123456789abcdef0123456789abcdef01234567')
+        self.assertUnhexlify('123456789abcdef0123456789abcdef012345678')
+        self.assertUnhexlify('0123456789ABCDEF0123456789ABCDEF01234567')
+        self.assertUnhexlify('123456789ABCDEF0123456789ABCDEF012345678')
+        hex_chars = binascii.hexlify(''.join(map(chr, range(256))))
+        for i in range(0, 480, 40):
+            self.assertUnhexlify(hex_chars[i:i+40])
+        self.assertUnhexlify(hex_chars[480:]+hex_chars[0:8])
+
+    def test_from_invalid_hex(self):
+        self.assertFailUnhexlify('123456789012345678901234567890123456789X')
+        self.assertFailUnhexlify('12345678901234567890123456789012345678X9')
+
+
+_hex_form = '123456789012345678901234567890abcdefabcd'
+
+class Test_KeyToSha1(TestBtreeSerializer):
+
+    def assertKeyToSha1(self, expected, key):
+        if expected is None:
+            expected_bin = None
+        else:
+            expected_bin = binascii.unhexlify(expected)
+        actual_sha1 = self.module._py_key_to_sha1(key)
+        if expected_bin != actual_sha1:
+            actual_hex_sha1 = None
+            if actual_sha1 is not None:
+                actual_hex_sha1 = binascii.hexlify(actual_sha1)
+            self.fail('_key_to_sha1 returned:\n    %s\n != %s'
+                      % (actual_sha1, expected))
+
+    def test_simple(self):
+        self.assertKeyToSha1(_hex_form, ('sha1:' + _hex_form,))
+
+    def test_invalid_not_tuple(self):
+        self.assertKeyToSha1(None, _hex_form)
+        self.assertKeyToSha1(None, 'sha1:' + _hex_form)
+
+    def test_invalid_empty(self):
+        self.assertKeyToSha1(None, ())
+
+    def test_invalid_not_string(self):
+        self.assertKeyToSha1(None, (None,))
+        self.assertKeyToSha1(None, (list(_hex_form),))
+
+    def test_invalid_not_sha1(self):
+        self.assertKeyToSha1(None, (_hex_form,))
+        self.assertKeyToSha1(None, ('sha2:' + _hex_form,))
+
+    def test_invalid_not_hex(self):
+        self.assertKeyToSha1(None,
+            ('sha1:abcdefghijklmnopqrstuvwxyz12345678901234',))
+
+
+class Test_Sha1ToKey(TestBtreeSerializer):
+
+    def assertSha1ToKey(self, hex_sha1):
+        bin_sha1 = binascii.unhexlify(hex_sha1)
+        key = self.module._py_sha1_to_key(bin_sha1)
+        self.assertEqual(('sha1:' + hex_sha1,), key)
+
+    def test_simple(self):
+        self.assertSha1ToKey(_hex_form)
+
+
+_one_key_content = """type=leaf
+sha1:123456789012345678901234567890abcdefabcd\x00\x001 2 3 4
+"""
+
+_large_offsets = """type=leaf
+sha1:123456789012345678901234567890abcdefabcd\x00\x0012345678901 1234567890 0 1
+sha1:abcd123456789012345678901234567890abcdef\x00\x002147483648 2147483647 0 1
+sha1:abcdefabcd123456789012345678901234567890\x00\x004294967296 4294967295 4294967294 1
+"""
+
+_multi_key_content = """type=leaf
+sha1:c80c881d4a26984ddce795f6f71817c9cf4480e7\x00\x000 0 0 0
+sha1:c86f7e437faa5a7fce15d1ddcb9eaeaea377667b\x00\x001 1 1 1
+sha1:c8e240de74fb1ed08fa08d38063f6a6a91462a81\x00\x002 2 2 2
+sha1:cda39a3ee5e6b4b0d3255bfef95601890afd8070\x00\x003 3 3 3
+sha1:cdf51e37c269aa94d38f93e537bf6e2020b21406\x00\x004 4 4 4
+sha1:ce0c9035898dd52fc65c41454cec9c4d2611bfb3\x00\x005 5 5 5
+sha1:ce93b4e3c464ffd51732fbd6ded717e9efda28aa\x00\x006 6 6 6
+sha1:cf7a9e24777ec23212c54d7a350bc5bea5477fdb\x00\x007 7 7 7
+"""
+
+_multi_key_same_offset = """type=leaf
+sha1:080c881d4a26984ddce795f6f71817c9cf4480e7\x00\x000 0 0 0
+sha1:c86f7e437faa5a7fce15d1ddcb9eaeaea377667b\x00\x001 1 1 1
+sha1:cd0c9035898dd52fc65c41454cec9c4d2611bfb3\x00\x002 2 2 2
+sha1:cda39a3ee5e6b4b0d3255bfef95601890afd8070\x00\x003 3 3 3
+sha1:cde240de74fb1ed08fa08d38063f6a6a91462a81\x00\x004 4 4 4
+sha1:cdf51e37c269aa94d38f93e537bf6e2020b21406\x00\x005 5 5 5
+sha1:ce7a9e24777ec23212c54d7a350bc5bea5477fdb\x00\x006 6 6 6
+sha1:ce93b4e3c464ffd51732fbd6ded717e9efda28aa\x00\x007 7 7 7
+"""
+
+_common_32_bits = """type=leaf
+sha1:123456784a26984ddce795f6f71817c9cf4480e7\x00\x000 0 0 0
+sha1:1234567874fb1ed08fa08d38063f6a6a91462a81\x00\x001 1 1 1
+sha1:12345678777ec23212c54d7a350bc5bea5477fdb\x00\x002 2 2 2
+sha1:123456787faa5a7fce15d1ddcb9eaeaea377667b\x00\x003 3 3 3
+sha1:12345678898dd52fc65c41454cec9c4d2611bfb3\x00\x004 4 4 4
+sha1:12345678c269aa94d38f93e537bf6e2020b21406\x00\x005 5 5 5
+sha1:12345678c464ffd51732fbd6ded717e9efda28aa\x00\x006 6 6 6
+sha1:12345678e5e6b4b0d3255bfef95601890afd8070\x00\x007 7 7 7
+"""
+
+
+class TestGCCKHSHA1LeafNode(TestBtreeSerializer):
+
+    def assertInvalid(self, bytes):
+        """Ensure that we get a proper error when trying to parse invalid bytes.
+
+        (mostly this is testing that bad input doesn't cause us to segfault)
+        """
+        self.assertRaises((ValueError, TypeError), 
+                          self.module._parse_into_chk, bytes, 1, 0)
+
+    def test_non_str(self):
+        self.assertInvalid(u'type=leaf\n')
+
+    def test_not_leaf(self):
+        self.assertInvalid('type=internal\n')
+
+    def test_empty_leaf(self):
+        leaf = self.module._parse_into_chk('type=leaf\n', 1, 0)
+        self.assertEqual(0, len(leaf))
+        self.assertEqual([], leaf.all_items())
+        self.assertEqual([], leaf.all_keys())
+        # It should allow any key to be queried
+        self.assertFalse(('key',) in leaf)
+
+    def test_one_key_leaf(self):
+        leaf = self.module._parse_into_chk(_one_key_content, 1, 0)
+        self.assertEqual(1, len(leaf))
+        sha_key = ('sha1:' + _hex_form,)
+        self.assertEqual([sha_key], leaf.all_keys())
+        self.assertEqual([(sha_key, ('1 2 3 4', ()))], leaf.all_items())
+        self.assertTrue(sha_key in leaf)
+
+    def test_large_offsets(self):
+        leaf = self.module._parse_into_chk(_large_offsets, 1, 0)
+        self.assertEqual(['12345678901 1234567890 0 1',
+                          '2147483648 2147483647 0 1',
+                          '4294967296 4294967295 4294967294 1',
+                         ], [x[1][0] for x in leaf.all_items()])
+
+    def test_many_key_leaf(self):
+        leaf = self.module._parse_into_chk(_multi_key_content, 1, 0)
+        self.assertEqual(8, len(leaf))
+        all_keys = leaf.all_keys()
+        self.assertEqual(8, len(leaf.all_keys()))
+        for idx, key in enumerate(all_keys):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_common_shift(self):
+        # The keys were deliberately chosen so that the first 5 bits all
+        # overlapped, it also happens that a later bit overlaps
+        # Note that by 'overlap' we mean that given bit is either on in all
+        # keys, or off in all keys
+        leaf = self.module._parse_into_chk(_multi_key_content, 1, 0)
+        self.assertEqual(19, leaf.common_shift)
+        # The interesting byte for each key is
+        # (defined as the 8-bits that come after the common prefix)
+        lst = [1, 13, 28, 180, 190, 193, 210, 239]
+        offsets = leaf._get_offsets()
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for idx, val in enumerate(lst):
+            self.assertEqual(idx, offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_multi_key_same_offset(self):
+        # there is no common prefix, though there are some common bits
+        leaf = self.module._parse_into_chk(_multi_key_same_offset, 1, 0)
+        self.assertEqual(24, leaf.common_shift)
+        offsets = leaf._get_offsets()
+        # The interesting byte is just the first 8-bits of the key
+        lst = [8, 200, 205, 205, 205, 205, 206, 206]
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for val in lst:
+            self.assertEqual(lst.index(val), offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_all_common_prefix(self):
+        # The first 32 bits of all hashes are the same. This is going to be
+        # pretty much impossible, but I don't want to fail because of this
+        leaf = self.module._parse_into_chk(_common_32_bits, 1, 0)
+        self.assertEqual(0, leaf.common_shift)
+        lst = [0x78] * 8
+        offsets = leaf._get_offsets()
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for val in lst:
+            self.assertEqual(lst.index(val), offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test_many_entries(self):
+        # Again, this is almost impossible, but we should still work
+        # It would be hard to fit more that 120 entries in a 4k page, much less
+        # more than 256 of them. but hey, weird stuff happens sometimes
+        lines = ['type=leaf\n']
+        for i in range(500):
+            key_str = 'sha1:%04x%s' % (i, _hex_form[:36])
+            key = (key_str,)
+            lines.append('%s\0\0%d %d %d %d\n' % (key_str, i, i, i, i))
+        bytes = ''.join(lines)
+        leaf = self.module._parse_into_chk(bytes, 1, 0)
+        self.assertEqual(24-7, leaf.common_shift)
+        offsets = leaf._get_offsets()
+        # This is the interesting bits for each entry
+        lst = [x // 2 for x in range(500)]
+        expected_offsets = [x * 2 for x in range(128)] + [255]*129
+        self.assertEqual(expected_offsets, offsets)
+        # We truncate because offsets is an unsigned char. So the bisection
+        # will just say 'greater than the last one' for all the rest
+        lst = lst[:255]
+        self.assertEqual([bisect.bisect_left(lst, x) for x in range(0, 257)],
+                         offsets)
+        for val in lst:
+            self.assertEqual(lst.index(val), offsets[val])
+        for idx, key in enumerate(leaf.all_keys()):
+            self.assertEqual(str(idx), leaf[key][0].split()[0])
+
+    def test__sizeof__(self):
+        # We can't use the exact numbers because of platform variations, etc.
+        # But what we really care about is that it does get bigger with more
+        # content.
+        leaf0 = self.module._parse_into_chk('type=leaf\n', 1, 0)
+        leaf1 = self.module._parse_into_chk(_one_key_content, 1, 0)
+        leafN = self.module._parse_into_chk(_multi_key_content, 1, 0)
+        sizeof_1 = leaf1.__sizeof__() - leaf0.__sizeof__()
+        self.assertTrue(sizeof_1 > 0)
+        sizeof_N = leafN.__sizeof__() - leaf0.__sizeof__()
+        self.assertEqual(sizeof_1 * len(leafN), sizeof_N)

=== modified file 'bzrlib/tests/test__chk_map.py'
--- bzrlib/tests/test__chk_map.py	2010-05-11 10:45:26 +0000
+++ bzrlib/tests/test__chk_map.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     chk_map,
-    inventory,
     tests,
     )
 from bzrlib.static_tuple import StaticTuple

=== modified file 'bzrlib/tests/test__chunks_to_lines.py'
--- bzrlib/tests/test__chunks_to_lines.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__chunks_to_lines.py	2011-06-14 01:26:41 +0000
@@ -18,6 +18,9 @@
 """Tests for chunks_to_lines."""
 
 from bzrlib import tests
+from bzrlib.tests import (
+    features,
+    )
 
 
 def load_tests(standard_tests, module, loader):
@@ -29,8 +32,8 @@
 # test_osutils depends on this feature being around. We can't just use the one
 # generated by load_tests, because if we only load osutils but not this module,
 # then that code never gets run
-compiled_chunkstolines_feature = tests.ModuleAvailableFeature(
-                                    'bzrlib._chunks_to_lines_pyx')
+compiled_chunkstolines_feature = features.ModuleAvailableFeature(
+    'bzrlib._chunks_to_lines_pyx')
 
 
 class TestChunksToLines(tests.TestCase):

=== modified file 'bzrlib/tests/test__dirstate_helpers.py'
--- bzrlib/tests/test__dirstate_helpers.py	2010-06-02 04:50:35 +0000
+++ bzrlib/tests/test__dirstate_helpers.py	2011-09-28 16:53:43 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,63 +25,46 @@
     errors,
     osutils,
     tests,
+    _dirstate_helpers_py,
     )
 from bzrlib.tests import (
     test_dirstate,
-    test_osutils,
-    )
-
-try:
-    from bzrlib import _dirstate_helpers_pyx
-    has_dirstate_helpers_pyx = True
-except ImportError:
-    has_dirstate_helpers_pyx = False
-
-
-compiled_dirstate_helpers_feature = tests.ModuleAvailableFeature(
-                                'bzrlib._dirstate_helpers_pyx')
-
-
-def load_tests(basic_tests, module, loader):
-    # FIXME: we should also parametrize against SHA1Provider !
-    suite = loader.suiteClass()
-    remaining_tests = basic_tests
-
-    dir_reader_scenarios = test_osutils.dir_reader_scenarios()
-
-    ue_scenarios = [('dirstate_Python',
-                     {'update_entry': dirstate.py_update_entry})]
-    if compiled_dirstate_helpers_feature.available():
-        update_entry = compiled_dirstate_helpers_feature.module.update_entry
-        pyrex_scenario = ('dirstate_Pyrex', {'update_entry': update_entry})
-        ue_scenarios.append(pyrex_scenario)
-    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance(TestUpdateEntry))
-    tests.multiply_tests(process_entry_tests,
-                         tests.multiply_scenarios(dir_reader_scenarios,
-                                                  ue_scenarios),
-                         suite)
-
-    pe_scenarios = [('dirstate_Python',
-                     {'_process_entry': dirstate.ProcessEntryPython})]
-    if compiled_dirstate_helpers_feature.available():
-        process_entry = compiled_dirstate_helpers_feature.module.ProcessEntryC
-        pyrex_scenario = ('dirstate_Pyrex', {'_process_entry': process_entry})
-        pe_scenarios.append(pyrex_scenario)
-    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance(TestProcessEntry))
-    tests.multiply_tests(process_entry_tests,
-                         tests.multiply_scenarios(dir_reader_scenarios,
-                                                  pe_scenarios),
-                         suite)
-
-    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance(
-            test_dirstate.TestCaseWithDirState))
-    tests.multiply_tests(dir_reader_tests, dir_reader_scenarios, suite)
-    suite.addTest(remaining_tests)
-
-    return suite
+    )
+from bzrlib.tests.test_osutils import dir_reader_scenarios
+from bzrlib.tests.scenarios import (
+    load_tests_apply_scenarios,
+    multiply_scenarios,
+    )
+from bzrlib.tests import (
+    features,
+    )
+
+
+load_tests = load_tests_apply_scenarios
+
+
+compiled_dirstate_helpers_feature = features.ModuleAvailableFeature(
+    'bzrlib._dirstate_helpers_pyx')
+
+
+# FIXME: we should also parametrize against SHA1Provider !
+
+ue_scenarios = [('dirstate_Python',
+    {'update_entry': dirstate.py_update_entry})]
+if compiled_dirstate_helpers_feature.available():
+    update_entry = compiled_dirstate_helpers_feature.module.update_entry
+    ue_scenarios.append(('dirstate_Pyrex', {'update_entry': update_entry}))
+
+pe_scenarios = [('dirstate_Python',
+    {'_process_entry': dirstate.ProcessEntryPython})]
+if compiled_dirstate_helpers_feature.available():
+    process_entry = compiled_dirstate_helpers_feature.module.ProcessEntryC
+    pe_scenarios.append(('dirstate_Pyrex', {'_process_entry': process_entry}))
+
+helper_scenarios = [('dirstate_Python', {'helpers': _dirstate_helpers_py})]
+if compiled_dirstate_helpers_feature.available():
+    helper_scenarios.append(('dirstate_Pyrex',
+        {'helpers': compiled_dirstate_helpers_feature.module}))
 
 
 class TestBisectPathMixin(object):
@@ -719,6 +702,8 @@
     implementation.
     """
 
+    # inherits scenarios from test_dirstate
+
     def get_read_dirblocks(self):
         from bzrlib._dirstate_helpers_py import _read_dirblocks
         return _read_dirblocks
@@ -826,6 +811,9 @@
 class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
     """Test the DirState.update_entry functions"""
 
+    scenarios = multiply_scenarios(
+        dir_reader_scenarios(), ue_scenarios)
+
     # Set by load_tests
     update_entry = None
 
@@ -843,25 +831,34 @@
 
     def test_observed_sha1_cachable(self):
         state, entry = self.get_state_with_a()
+        state.save()
         atime = time.time() - 10
         self.build_tree(['a'])
-        statvalue = os.lstat('a')
-        statvalue = test_dirstate._FakeStat(statvalue.st_size, atime, atime,
-            statvalue.st_dev, statvalue.st_ino, statvalue.st_mode)
+        statvalue = test_dirstate._FakeStat.from_stat(os.lstat('a'))
+        statvalue.st_mtime = statvalue.st_ctime = atime
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
         state._observed_sha1(entry, "foo", statvalue)
         self.assertEqual('foo', entry[1][0][1])
         packed_stat = dirstate.pack_stat(statvalue)
         self.assertEqual(packed_stat, entry[1][0][4])
+        self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
+                         state._dirblock_state)
 
     def test_observed_sha1_not_cachable(self):
         state, entry = self.get_state_with_a()
+        state.save()
         oldval = entry[1][0][1]
         oldstat = entry[1][0][4]
         self.build_tree(['a'])
         statvalue = os.lstat('a')
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
         state._observed_sha1(entry, "foo", statvalue)
         self.assertEqual(oldval, entry[1][0][1])
         self.assertEqual(oldstat, entry[1][0][4])
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
 
     def test_update_entry(self):
         state, _ = self.get_state_with_a()
@@ -892,10 +889,12 @@
                                           stat_value=stat_value)
         self.assertEqual(None, link_or_sha1)
 
-        # The dirblock entry should not have cached the file's sha1 (too new)
+        # The dirblock entry should not have computed or cached the file's
+        # sha1, but it did update the files' st_size. However, this is not
+        # worth writing a dirstate file for, so we leave the state UNMODIFIED
         self.assertEqual(('f', '', 14, False, dirstate.DirState.NULLSTAT),
                          entry[1][0])
-        self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
         mode = stat_value.st_mode
         self.assertEqual([('is_exec', mode, False)], state._log)
@@ -904,9 +903,8 @@
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
 
-        # If we do it again right away, we don't know if the file has changed
-        # so we will re-read the file. Roll the clock back so the file is
-        # guaranteed to look too new.
+        # Roll the clock back so the file is guaranteed to look too new. We
+        # should still not compute the sha1.
         state.adjust_time(-10)
         del state._log[:]
 
@@ -914,7 +912,7 @@
                                           stat_value=stat_value)
         self.assertEqual([('is_exec', mode, False)], state._log)
         self.assertEqual(None, link_or_sha1)
-        self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
         self.assertEqual(('f', '', 14, False, dirstate.DirState.NULLSTAT),
                          entry[1][0])
@@ -930,6 +928,8 @@
         self.assertEqual([('is_exec', mode, False)], state._log)
         self.assertEqual(('f', '', 14, False, dirstate.DirState.NULLSTAT),
                          entry[1][0])
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
 
         # If the file is no longer new, and the clock has been moved forward
         # sufficiently, it will cache the sha.
@@ -960,7 +960,7 @@
 
     def test_update_entry_symlink(self):
         """Update entry should read symlinks."""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         state.save()
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
@@ -977,35 +977,38 @@
         # Dirblock is not updated (the link is too new)
         self.assertEqual([('l', '', 6, False, dirstate.DirState.NULLSTAT)],
                          entry[1])
-        self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+        # The file entry turned into a symlink, that is considered
+        # HASH modified worthy.
+        self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
                          state._dirblock_state)
 
         # Because the stat_value looks new, we should re-read the target
+        del state._log[:]
         link_or_sha1 = self.update_entry(state, entry, abspath='a',
                                           stat_value=stat_value)
         self.assertEqual('target', link_or_sha1)
-        self.assertEqual([('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                         ], state._log)
+        self.assertEqual([('read_link', 'a', '')], state._log)
         self.assertEqual([('l', '', 6, False, dirstate.DirState.NULLSTAT)],
                          entry[1])
+        state.save()
         state.adjust_time(+20) # Skip into the future, all files look old
+        del state._log[:]
         link_or_sha1 = self.update_entry(state, entry, abspath='a',
                                           stat_value=stat_value)
+        # The symlink stayed a symlink. So while it is new enough to cache, we
+        # don't bother setting the flag, because it is not really worth saving
+        # (when we stat the symlink, we'll have paged in the target.)
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
         self.assertEqual('target', link_or_sha1)
         # We need to re-read the link because only now can we cache it
-        self.assertEqual([('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                         ], state._log)
+        self.assertEqual([('read_link', 'a', '')], state._log)
         self.assertEqual([('l', 'target', 6, False, packed_stat)],
                          entry[1])
 
+        del state._log[:]
         # Another call won't re-read the link
-        self.assertEqual([('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                          ('read_link', 'a', ''),
-                         ], state._log)
+        self.assertEqual([], state._log)
         link_or_sha1 = self.update_entry(state, entry, abspath='a',
                                           stat_value=stat_value)
         self.assertEqual('target', link_or_sha1)
@@ -1026,12 +1029,29 @@
         self.build_tree(['a/'])
         state.adjust_time(+20)
         self.assertIs(None, self.do_update_entry(state, entry, 'a'))
+        # a/ used to be a file, but is now a directory, worth saving
         self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
                          state._dirblock_state)
         state.save()
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
-        self.assertIs(None, self.do_update_entry(state, entry, 'a'))
+        # No changes to a/ means not worth saving.
+        self.assertIs(None, self.do_update_entry(state, entry, 'a'))
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
+        # Change the last-modified time for the directory
+        t = time.time() - 100.0
+        try:
+            os.utime('a', (t, t))
+        except OSError:
+            # It looks like Win32 + FAT doesn't allow to change times on a dir.
+            raise tests.TestSkipped("can't update mtime of a dir on FAT")
+        saved_packed_stat = entry[1][0][-1]
+        self.assertIs(None, self.do_update_entry(state, entry, 'a'))
+        # We *do* go ahead and update the information in the dirblocks, but we
+        # don't bother setting IN_MEMORY_MODIFIED because it is trivial to
+        # recompute.
+        self.assertNotEqual(saved_packed_stat, entry[1][0][-1])
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
 
@@ -1137,7 +1157,7 @@
 
     def test_update_file_to_symlink(self):
         """File becomes a symlink"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The file sha1 won't be cached unless the file is old
         state.adjust_time(+10)
@@ -1156,7 +1176,7 @@
 
     def test_update_dir_to_symlink(self):
         """Directory becomes a symlink"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)
@@ -1166,7 +1186,7 @@
 
     def test_update_symlink_to_file(self):
         """Symlink becomes a file"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink and file info won't be cached unless old
         state.adjust_time(+10)
@@ -1176,7 +1196,7 @@
 
     def test_update_symlink_to_dir(self):
         """Symlink becomes a directory"""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)
@@ -1269,6 +1289,8 @@
 
 class TestProcessEntry(test_dirstate.TestCaseWithDirState):
 
+    scenarios = multiply_scenarios(dir_reader_scenarios(), pe_scenarios)
+
     # Set by load_tests
     _process_entry = None
 
@@ -1321,3 +1343,55 @@
         state._sha1_provider = UppercaseSHA1Provider()
         self.assertChangedFileIds(['file-id'], tree)
 
+
+class TestPackStat(tests.TestCase):
+    """Check packed representaton of stat values is robust on all inputs"""
+
+    scenarios = helper_scenarios
+
+    def pack(self, statlike_tuple):
+        return self.helpers.pack_stat(os.stat_result(statlike_tuple))
+
+    @staticmethod
+    def unpack_field(packed_string, stat_field):
+        return _dirstate_helpers_py._unpack_stat(packed_string)[stat_field]
+
+    def test_result(self):
+        self.assertEqual("AAAQAAAAABAAAAARAAAAAgAAAAEAAIHk",
+            self.pack((33252, 1, 2, 0, 0, 0, 4096, 15.5, 16.5, 17.5)))
+
+    def test_giant_inode(self):
+        packed = self.pack((33252, 0xF80000ABC, 0, 0, 0, 0, 0, 0, 0, 0))
+        self.assertEqual(0x80000ABC, self.unpack_field(packed, "st_ino"))
+
+    def test_giant_size(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, (1 << 33) + 4096, 0, 0, 0))
+        self.assertEqual(4096, self.unpack_field(packed, "st_size"))
+
+    def test_fractional_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 16.9375, 0))
+        self.assertEqual(16, self.unpack_field(packed, "st_mtime"))
+
+    def test_ancient_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, -11644473600.0, 0))
+        self.assertEqual(1240428288, self.unpack_field(packed, "st_mtime"))
+
+    def test_distant_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 64060588800.0, 0))
+        self.assertEqual(3931046656, self.unpack_field(packed, "st_mtime"))
+
+    def test_fractional_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, 17.5625))
+        self.assertEqual(17, self.unpack_field(packed, "st_ctime"))
+
+    def test_ancient_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, -11644473600.0))
+        self.assertEqual(1240428288, self.unpack_field(packed, "st_ctime"))
+
+    def test_distant_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, 64060588800.0))
+        self.assertEqual(3931046656, self.unpack_field(packed, "st_ctime"))
+
+    def test_negative_dev(self):
+        packed = self.pack((33252, 0, -0xFFFFFCDE, 0, 0, 0, 0, 0, 0, 0))
+        self.assertEqual(0x322, self.unpack_field(packed, "st_dev"))

=== modified file 'bzrlib/tests/test__groupcompress.py'
--- bzrlib/tests/test__groupcompress.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__groupcompress.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,18 +17,18 @@
 """Tests for the python and pyrex extensions of groupcompress"""
 
 from bzrlib import (
-    groupcompress,
     _groupcompress_py,
     tests,
     )
-
-
-def load_tests(standard_tests, module, loader):
-    """Parameterize tests for all versions of groupcompress."""
-    two_way_scenarios = [
-        ('PP', {'make_delta': _groupcompress_py.make_delta,
-                'apply_delta': _groupcompress_py.apply_delta})
-        ]
+from bzrlib.tests.scenarios import (
+    load_tests_apply_scenarios,
+    )
+from bzrlib.tests import (
+    features,
+    )
+
+
+def module_scenarios():
     scenarios = [
         ('python', {'_gc_module': _groupcompress_py}),
         ]
@@ -36,7 +36,17 @@
         gc_module = compiled_groupcompress_feature.module
         scenarios.append(('C',
             {'_gc_module': gc_module}))
-        two_way_scenarios.extend([
+    return scenarios
+
+
+def two_way_scenarios():
+    scenarios = [
+        ('PP', {'make_delta': _groupcompress_py.make_delta,
+                'apply_delta': _groupcompress_py.apply_delta})
+        ]
+    if compiled_groupcompress_feature.available():
+        gc_module = compiled_groupcompress_feature.module
+        scenarios.extend([
             ('CC', {'make_delta': gc_module.make_delta,
                     'apply_delta': gc_module.apply_delta}),
             ('PC', {'make_delta': _groupcompress_py.make_delta,
@@ -44,18 +54,14 @@
             ('CP', {'make_delta': gc_module.make_delta,
                     'apply_delta': _groupcompress_py.apply_delta}),
             ])
-    to_adapt, result = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance((TestMakeAndApplyDelta,
-                                                    TestBase128Int)))
-    result = tests.multiply_tests(to_adapt, scenarios, result)
-    to_adapt, result = tests.split_suite_by_condition(result,
-        tests.condition_isinstance(TestMakeAndApplyCompatible))
-    result = tests.multiply_tests(to_adapt, two_way_scenarios, result)
-    return result
-
-
-compiled_groupcompress_feature = tests.ModuleAvailableFeature(
-                                    'bzrlib._groupcompress_pyx')
+    return scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+compiled_groupcompress_feature = features.ModuleAvailableFeature(
+    'bzrlib._groupcompress_pyx')
 
 _text1 = """\
 This is a bit
@@ -116,6 +122,7 @@
 
 class TestMakeAndApplyDelta(tests.TestCase):
 
+    scenarios = module_scenarios()
     _gc_module = None # Set by load_tests
 
     def setUp(self):
@@ -226,6 +233,8 @@
 
 class TestMakeAndApplyCompatible(tests.TestCase):
 
+    scenarios = two_way_scenarios()
+
     make_delta = None # Set by load_tests
     apply_delta = None # Set by load_tests
 
@@ -258,6 +267,63 @@
         di = self._gc_module.DeltaIndex('test text\n')
         self.assertEqual('DeltaIndex(1, 10)', repr(di))
 
+    def test__dump_no_index(self):
+        di = self._gc_module.DeltaIndex()
+        self.assertEqual(None, di._dump_index())
+
+    def test__dump_index_simple(self):
+        di = self._gc_module.DeltaIndex()
+        di.add_source(_text1, 0)
+        self.assertFalse(di._has_index())
+        self.assertEqual(None, di._dump_index())
+        _ = di.make_delta(_text1)
+        self.assertTrue(di._has_index())
+        hash_list, entry_list = di._dump_index()
+        self.assertEqual(16, len(hash_list))
+        self.assertEqual(68, len(entry_list))
+        just_entries = [(idx, text_offset, hash_val)
+                        for idx, (text_offset, hash_val)
+                         in enumerate(entry_list)
+                         if text_offset != 0 or hash_val != 0]
+        rabin_hash = self._gc_module._rabin_hash
+        self.assertEqual([(8, 16, rabin_hash(_text1[1:17])),
+                          (25, 48, rabin_hash(_text1[33:49])),
+                          (34, 32, rabin_hash(_text1[17:33])),
+                          (47, 64, rabin_hash(_text1[49:65])),
+                         ], just_entries)
+        # This ensures that the hash map points to the location we expect it to
+        for entry_idx, text_offset, hash_val in just_entries:
+            self.assertEqual(entry_idx, hash_list[hash_val & 0xf])
+
+    def test__dump_index_two_sources(self):
+        di = self._gc_module.DeltaIndex()
+        di.add_source(_text1, 0)
+        di.add_source(_text2, 2)
+        start2 = len(_text1) + 2
+        self.assertTrue(di._has_index())
+        hash_list, entry_list = di._dump_index()
+        self.assertEqual(16, len(hash_list))
+        self.assertEqual(68, len(entry_list))
+        just_entries = [(idx, text_offset, hash_val)
+                        for idx, (text_offset, hash_val)
+                         in enumerate(entry_list)
+                         if text_offset != 0 or hash_val != 0]
+        rabin_hash = self._gc_module._rabin_hash
+        self.assertEqual([(8, 16, rabin_hash(_text1[1:17])),
+                          (9, start2+16, rabin_hash(_text2[1:17])),
+                          (25, 48, rabin_hash(_text1[33:49])),
+                          (30, start2+64, rabin_hash(_text2[49:65])),
+                          (34, 32, rabin_hash(_text1[17:33])),
+                          (35, start2+32, rabin_hash(_text2[17:33])),
+                          (43, start2+48, rabin_hash(_text2[33:49])),
+                          (47, 64, rabin_hash(_text1[49:65])),
+                         ], just_entries)
+        # Each entry should be in the appropriate hash bucket.
+        for entry_idx, text_offset, hash_val in just_entries:
+            hash_idx = hash_val & 0xf
+            self.assertTrue(
+                hash_list[hash_idx] <= entry_idx < hash_list[hash_idx+1])
+
     def test_first_add_source_doesnt_index_until_make_delta(self):
         di = self._gc_module.DeltaIndex()
         self.assertFalse(di._has_index())
@@ -269,6 +335,27 @@
         self.assertTrue(di._has_index())
         self.assertEqual('N\x90/\x1fdiffer from\nagainst other text\n', delta)
 
+    def test_add_source_max_bytes_to_index(self):
+        di = self._gc_module.DeltaIndex()
+        di._max_bytes_to_index = 3*16
+        di.add_source(_text1, 0) # (77 bytes -1) // 3 = 25 byte stride
+        di.add_source(_text3, 3) # (135 bytes -1) // 3 = 44 byte stride
+        start2 = len(_text1) + 3
+        hash_list, entry_list = di._dump_index()
+        self.assertEqual(16, len(hash_list))
+        self.assertEqual(67, len(entry_list))
+        just_entries = sorted([(text_offset, hash_val)
+                               for text_offset, hash_val in entry_list
+                                if text_offset != 0 or hash_val != 0])
+        rabin_hash = self._gc_module._rabin_hash
+        self.assertEqual([(25, rabin_hash(_text1[10:26])),
+                          (50, rabin_hash(_text1[35:51])),
+                          (75, rabin_hash(_text1[60:76])),
+                          (start2+44, rabin_hash(_text3[29:45])),
+                          (start2+88, rabin_hash(_text3[73:89])),
+                          (start2+132, rabin_hash(_text3[117:133])),
+                         ], just_entries)
+
     def test_second_add_source_triggers_make_index(self):
         di = self._gc_module.DeltaIndex()
         self.assertFalse(di._has_index())
@@ -457,6 +544,8 @@
 
 class TestBase128Int(tests.TestCase):
 
+    scenarios = module_scenarios()
+
     _gc_module = None # Set by load_tests
 
     def assertEqualEncode(self, bytes, val):

=== modified file 'bzrlib/tests/test__known_graph.py'
--- bzrlib/tests/test__known_graph.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__known_graph.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,47 +20,43 @@
 
 from bzrlib import (
     errors,
-    graph as _mod_graph,
     _known_graph_py,
     tests,
     )
 from bzrlib.tests import test_graph
 from bzrlib.revision import NULL_REVISION
-
-
-def load_tests(standard_tests, module, loader):
-    """Parameterize tests for all versions of groupcompress."""
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests import (
+    features,
+    )
+
+
+def caching_scenarios():
     scenarios = [
         ('python', {'module': _known_graph_py, 'do_cache': True}),
     ]
-    caching_scenarios = [
-        ('python-nocache', {'module': _known_graph_py, 'do_cache': False}),
-    ]
-    suite = loader.suiteClass()
     if compiled_known_graph_feature.available():
         scenarios.append(('C', {'module': compiled_known_graph_feature.module,
                                 'do_cache': True}))
-        caching_scenarios.append(
+    return scenarios
+
+
+def non_caching_scenarios():
+    scenarios = [
+        ('python-nocache', {'module': _known_graph_py, 'do_cache': False}),
+    ]
+    if compiled_known_graph_feature.available():
+        scenarios.append(
             ('C-nocache', {'module': compiled_known_graph_feature.module,
                            'do_cache': False}))
-    else:
-        # the compiled module isn't available, so we add a failing test
-        class FailWithoutFeature(tests.TestCase):
-            def test_fail(self):
-                self.requireFeature(compiled_known_graph_feature)
-        suite.addTest(loader.loadTestsFromTestCase(FailWithoutFeature))
-    # TestKnownGraphHeads needs to be permutated with and without caching.
-    # All other TestKnownGraph tests only need to be tested across module
-    heads_suite, other_suite = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance(TestKnownGraphHeads))
-    suite = tests.multiply_tests(other_suite, scenarios, suite)
-    suite = tests.multiply_tests(heads_suite, scenarios + caching_scenarios,
-                                 suite)
-    return suite
-
-
-compiled_known_graph_feature = tests.ModuleAvailableFeature(
-                                    'bzrlib._known_graph_pyx')
+    return scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+compiled_known_graph_feature = features.ModuleAvailableFeature(
+    'bzrlib._known_graph_pyx')
 
 
 #  a
@@ -75,6 +71,7 @@
 
 class TestCaseWithKnownGraph(tests.TestCase):
 
+    scenarios = caching_scenarios()
     module = None # Set by load_tests
 
     def make_known_graph(self, ancestry):
@@ -214,6 +211,7 @@
 
 class TestKnownGraphHeads(TestCaseWithKnownGraph):
 
+    scenarios = caching_scenarios() + non_caching_scenarios()
     do_cache = None # Set by load_tests
 
     def test_heads_null(self):

=== modified file 'bzrlib/tests/test__simple_set.py'
--- bzrlib/tests/test__simple_set.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test__simple_set.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,10 +19,11 @@
 import sys
 
 from bzrlib import (
-    errors,
-    osutils,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 try:
     from bzrlib import _simple_set_pyx
@@ -77,7 +78,7 @@
 
 # Even though this is an extension, we don't permute the tests for a python
 # version. As the plain python version is just a dict or set
-compiled_simpleset_feature = tests.ModuleAvailableFeature(
+compiled_simpleset_feature = features.ModuleAvailableFeature(
                                 'bzrlib._simple_set_pyx')
 
 
@@ -379,3 +380,13 @@
         # And even removing an item still causes it to fail
         obj.discard(k2)
         self.assertRaises(RuntimeError, iterator.next)
+
+    def test__sizeof__(self):
+        # SimpleSet needs a custom sizeof implementation, because it allocates
+        # memory that Python cannot directly see (_table).
+        # Too much variability in platform sizes for us to give a fixed size
+        # here. However without a custom implementation, __sizeof__ would give
+        # us only the size of the object, and not its table. We know the table
+        # is at least 4bytes*1024entries in size.
+        obj = self.module.SimpleSet()
+        self.assertTrue(obj.__sizeof__() > 4096)

=== modified file 'bzrlib/tests/test__static_tuple.py'
--- bzrlib/tests/test__static_tuple.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test__static_tuple.py	2011-08-23 09:29:27 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,17 +17,18 @@
 """Tests for the StaticTuple type."""
 
 import cPickle
-import gc
 import sys
 
 from bzrlib import (
     _static_tuple_py,
     debug,
-    errors,
     osutils,
     static_tuple,
     tests,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 def load_tests(standard_tests, module, loader):
@@ -39,21 +40,6 @@
     return suite
 
 
-class _Meliae(tests.Feature):
-
-    def _probe(self):
-        try:
-            from meliae import scanner
-        except ImportError:
-            return False
-        return True
-
-    def feature_name(self):
-        return "Meliae - python memory debugger"
-
-Meliae = _Meliae()
-
-
 class TestStaticTuple(tests.TestCase):
 
     def assertRefcount(self, count, obj):
@@ -446,7 +432,7 @@
         # amount of referenced memory. Unfortunately gc.get_referents() first
         # checks the IS_GC flag before it traverses anything. We could write a
         # helper func, but that won't work for the generic implementation...
-        self.requireFeature(Meliae)
+        self.requireFeature(features.meliae)
         from meliae import scanner
         strs = ['foo', 'bar', 'baz', 'bing']
         k = self.module.StaticTuple(*strs)
@@ -457,7 +443,7 @@
         self.assertEqual(sorted(refs), sorted(scanner.get_referents(k)))
 
     def test_nested_referents(self):
-        self.requireFeature(Meliae)
+        self.requireFeature(features.meliae)
         from meliae import scanner
         strs = ['foo', 'bar', 'baz', 'bing']
         k1 = self.module.StaticTuple(*strs[:2])

=== modified file 'bzrlib/tests/test__walkdirs_win32.py'
--- bzrlib/tests/test__walkdirs_win32.py	2010-05-16 12:21:31 +0000
+++ bzrlib/tests/test__walkdirs_win32.py	2011-06-14 01:26:41 +0000
@@ -22,9 +22,12 @@
     osutils,
     tests,
     )
-
-
-win32_readdir_feature = tests.ModuleAvailableFeature('bzrlib._walkdirs_win32')
+from bzrlib.tests import (
+    features,
+    )
+
+
+win32_readdir_feature = features.ModuleAvailableFeature('bzrlib._walkdirs_win32')
 
 
 class TestWin32Finder(tests.TestCaseInTempDir):

=== modified file 'bzrlib/tests/test_ancestry.py'
--- bzrlib/tests/test_ancestry.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_ancestry.py	2011-06-16 15:29:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,30 +15,22 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-from cStringIO import StringIO
-import os
-import sys
-
+from bzrlib.branchbuilder import BranchBuilder
 from bzrlib.tests import TestCaseWithMemoryTransport
-from bzrlib.branch import Branch
-from bzrlib.branchbuilder import BranchBuilder
+from bzrlib.tests.matchers import MatchesAncestry
 
 
 class TestAncestry(TestCaseWithMemoryTransport):
 
-    def assertAncestryEqual(self, expected, revision_id, branch):
-        """Assert that the ancestry of revision_id in branch is as expected."""
-        ancestry = branch.repository.get_ancestry(revision_id)
-        self.assertEqual(expected, ancestry)
-
     def test_straightline_ancestry(self):
         """Test ancestry file when just committing."""
         builder = BranchBuilder(self.get_transport())
         rev_id_one = builder.build_commit()
         rev_id_two = builder.build_commit()
         branch = builder.get_branch()
-        self.assertAncestryEqual([None, rev_id_one, rev_id_two],
-            rev_id_two, branch)
-        self.assertAncestryEqual([None, rev_id_one], rev_id_one, branch)
+        self.assertThat([rev_id_one, rev_id_two],
+            MatchesAncestry(branch.repository, rev_id_two))
+        self.assertThat([rev_id_one],
+            MatchesAncestry(branch.repository, rev_id_one))
 
 # TODO: check that ancestry is updated to include indirectly merged revisions

=== modified file 'bzrlib/tests/test_annotate.py'
--- bzrlib/tests/test_annotate.py	2009-07-10 07:43:15 +0000
+++ bzrlib/tests/test_annotate.py	2011-05-03 13:53:46 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,10 +21,8 @@
 
 from bzrlib import (
     annotate,
-    conflicts,
-    errors,
+    symbol_versioning,
     tests,
-    trace,
     )
 
 
@@ -263,15 +261,26 @@
             ('modify', ('file-id', e_text))])
         return builder
 
-    def assertRepoAnnotate(self, expected, repo, file_id, revision_id):
-        """Assert that the revision is properly annotated."""
-        actual = list(repo.revision_tree(revision_id).annotate_iter(file_id))
+    def assertAnnotateEqualDiff(self, actual, expected):
         if actual != expected:
             # Create an easier to understand diff when the lines don't actually
             # match
             self.assertEqualDiff(''.join('\t'.join(l) for l in expected),
                                  ''.join('\t'.join(l) for l in actual))
 
+    def assertBranchAnnotate(self, expected, branch, file_id, revision_id,
+            verbose=False, full=False, show_ids=False):
+        tree = branch.repository.revision_tree(revision_id)
+        to_file = StringIO()
+        annotate.annotate_file_tree(tree, file_id, to_file,
+            verbose=verbose, full=full, show_ids=show_ids, branch=branch)
+        self.assertAnnotateEqualDiff(to_file.getvalue(), expected)
+
+    def assertRepoAnnotate(self, expected, repo, file_id, revision_id):
+        """Assert that the revision is properly annotated."""
+        actual = list(repo.revision_tree(revision_id).annotate_iter(file_id))
+        self.assertAnnotateEqualDiff(actual, expected)
+
     def test_annotate_duplicate_lines(self):
         # XXX: Should this be a per_repository test?
         builder = self.create_duplicate_lines_tree()
@@ -288,62 +297,64 @@
     def test_annotate_shows_dotted_revnos(self):
         builder = self.create_merged_trees()
 
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-3', 'a-id',
-                               to_file=sio)
-        self.assertEqualDiff('1     joe@foo | first\n'
-                             '2     joe@foo | second\n'
-                             '1.1.1 barry@f | third\n',
-                             sio.getvalue())
+        self.assertBranchAnnotate('1     joe@foo | first\n'
+                                  '2     joe@foo | second\n'
+                                  '1.1.1 barry@f | third\n',
+                                  builder.get_branch(), 'a-id', 'rev-3')
+
+    def test_annotate_file(self):
+        builder = self.create_merged_trees()
+
+        to_file = StringIO()
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            annotate.annotate_file, builder.get_branch(),
+                'rev-3', 'a-id', to_file=to_file)
+
+        self.assertAnnotateEqualDiff('1     joe@foo | first\n'
+                                     '2     joe@foo | second\n'
+                                     '1.1.1 barry@f | third\n',
+                                     to_file.getvalue())
 
     def test_annotate_limits_dotted_revnos(self):
         """Annotate should limit dotted revnos to a depth of 12"""
         builder = self.create_deeply_merged_trees()
 
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-6', 'a-id',
-                               to_file=sio, verbose=False, full=False)
-        self.assertEqualDiff('1     joe@foo | first\n'
-                             '2     joe@foo | second\n'
-                             '1.1.1 barry@f | third\n'
-                             '1.2.1 jerry@f | fourth\n'
-                             '1.3.1 george@ | fifth\n'
-                             '              | sixth\n',
-                             sio.getvalue())
+        self.assertBranchAnnotate('1     joe@foo | first\n'
+                                  '2     joe@foo | second\n'
+                                  '1.1.1 barry@f | third\n'
+                                  '1.2.1 jerry@f | fourth\n'
+                                  '1.3.1 george@ | fifth\n'
+                                  '              | sixth\n',
+                                  builder.get_branch(), 'a-id', 'rev-6',
+                                  verbose=False, full=False)
 
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-6', 'a-id',
-                               to_file=sio, verbose=False, full=True)
-        self.assertEqualDiff('1     joe@foo | first\n'
-                             '2     joe@foo | second\n'
-                             '1.1.1 barry@f | third\n'
-                             '1.2.1 jerry@f | fourth\n'
-                             '1.3.1 george@ | fifth\n'
-                             '1.3.1 george@ | sixth\n',
-                             sio.getvalue())
+        self.assertBranchAnnotate('1     joe@foo | first\n'
+                                  '2     joe@foo | second\n'
+                                  '1.1.1 barry@f | third\n'
+                                  '1.2.1 jerry@f | fourth\n'
+                                  '1.3.1 george@ | fifth\n'
+                                  '1.3.1 george@ | sixth\n',
+                                  builder.get_branch(), 'a-id', 'rev-6',
+                                  verbose=False, full=True)
 
         # verbose=True shows everything, the full revno, user id, and date
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-6', 'a-id',
-                               to_file=sio, verbose=True, full=False)
-        self.assertEqualDiff('1     joe@foo.com    20061213 | first\n'
-                             '2     joe@foo.com    20061213 | second\n'
-                             '1.1.1 barry@foo.com  20061213 | third\n'
-                             '1.2.1 jerry@foo.com  20061213 | fourth\n'
-                             '1.3.1 george@foo.com 20061213 | fifth\n'
-                             '                              | sixth\n',
-                             sio.getvalue())
+        self.assertBranchAnnotate('1     joe@foo.com    20061213 | first\n'
+                                  '2     joe@foo.com    20061213 | second\n'
+                                  '1.1.1 barry@foo.com  20061213 | third\n'
+                                  '1.2.1 jerry@foo.com  20061213 | fourth\n'
+                                  '1.3.1 george@foo.com 20061213 | fifth\n'
+                                  '                              | sixth\n',
+                                  builder.get_branch(), 'a-id', 'rev-6',
+                                  verbose=True, full=False)
 
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-6', 'a-id',
-                               to_file=sio, verbose=True, full=True)
-        self.assertEqualDiff('1     joe@foo.com    20061213 | first\n'
-                             '2     joe@foo.com    20061213 | second\n'
-                             '1.1.1 barry@foo.com  20061213 | third\n'
-                             '1.2.1 jerry@foo.com  20061213 | fourth\n'
-                             '1.3.1 george@foo.com 20061213 | fifth\n'
-                             '1.3.1 george@foo.com 20061213 | sixth\n',
-                             sio.getvalue())
+        self.assertBranchAnnotate('1     joe@foo.com    20061213 | first\n'
+                                  '2     joe@foo.com    20061213 | second\n'
+                                  '1.1.1 barry@foo.com  20061213 | third\n'
+                                  '1.2.1 jerry@foo.com  20061213 | fourth\n'
+                                  '1.3.1 george@foo.com 20061213 | fifth\n'
+                                  '1.3.1 george@foo.com 20061213 | sixth\n',
+                                  builder.get_branch(), 'a-id', 'rev-6',
+                                  verbose=True, full=True)
 
     def test_annotate_uses_branch_context(self):
         """Dotted revnos should use the Branch context.
@@ -353,43 +364,35 @@
         """
         builder = self.create_deeply_merged_trees()
 
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-1_3_1', 'a-id',
-                               to_file=sio, verbose=False, full=False)
-        self.assertEqualDiff('1     joe@foo | first\n'
-                             '1.1.1 barry@f | third\n'
-                             '1.2.1 jerry@f | fourth\n'
-                             '1.3.1 george@ | fifth\n'
-                             '              | sixth\n',
-                             sio.getvalue())
+        self.assertBranchAnnotate('1     joe@foo | first\n'
+                                  '1.1.1 barry@f | third\n'
+                                  '1.2.1 jerry@f | fourth\n'
+                                  '1.3.1 george@ | fifth\n'
+                                  '              | sixth\n',
+                                  builder.get_branch(), 'a-id', 'rev-1_3_1',
+                                  verbose=False, full=False)
 
     def test_annotate_show_ids(self):
         builder = self.create_deeply_merged_trees()
 
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-6', 'a-id',
-                               to_file=sio, show_ids=True, full=False)
-
         # It looks better with real revision ids :)
-        self.assertEqualDiff('    rev-1 | first\n'
-                             '    rev-2 | second\n'
-                             'rev-1_1_1 | third\n'
-                             'rev-1_2_1 | fourth\n'
-                             'rev-1_3_1 | fifth\n'
-                             '          | sixth\n',
-                             sio.getvalue())
-
-        sio = StringIO()
-        annotate.annotate_file(builder.get_branch(), 'rev-6', 'a-id',
-                               to_file=sio, show_ids=True, full=True)
-
-        self.assertEqualDiff('    rev-1 | first\n'
-                             '    rev-2 | second\n'
-                             'rev-1_1_1 | third\n'
-                             'rev-1_2_1 | fourth\n'
-                             'rev-1_3_1 | fifth\n'
-                             'rev-1_3_1 | sixth\n',
-                             sio.getvalue())
+        self.assertBranchAnnotate('    rev-1 | first\n'
+                                  '    rev-2 | second\n'
+                                  'rev-1_1_1 | third\n'
+                                  'rev-1_2_1 | fourth\n'
+                                  'rev-1_3_1 | fifth\n'
+                                  '          | sixth\n',
+                                  builder.get_branch(), 'a-id', 'rev-6',
+                                  show_ids=True, full=False)
+
+        self.assertBranchAnnotate('    rev-1 | first\n'
+                                  '    rev-2 | second\n'
+                                  'rev-1_1_1 | third\n'
+                                  'rev-1_2_1 | fourth\n'
+                                  'rev-1_3_1 | fifth\n'
+                                  'rev-1_3_1 | sixth\n',
+                                  builder.get_branch(), 'a-id', 'rev-6',
+                                  show_ids=True, full=True)
 
     def test_annotate_unicode_author(self):
         tree1 = self.make_branch_and_tree('tree1')
@@ -408,25 +411,33 @@
 
         tree1.lock_read()
         self.addCleanup(tree1.unlock)
+
+        revtree_1 = tree1.branch.repository.revision_tree('rev-1')
+        revtree_2 = tree1.branch.repository.revision_tree('rev-2')
+
         # this passes if no exception is raised
         to_file = StringIO()
-        annotate.annotate_file(tree1.branch, 'rev-1', 'a-id', to_file=to_file)
+        annotate.annotate_file_tree(revtree_1, 'a-id',
+            to_file=to_file, branch=tree1.branch)
 
         sio = StringIO()
         to_file = codecs.getwriter('ascii')(sio)
         to_file.encoding = 'ascii' # codecs does not set it
-        annotate.annotate_file(tree1.branch, 'rev-2', 'b-id', to_file=to_file)
+        annotate.annotate_file_tree(revtree_2, 'b-id',
+            to_file=to_file, branch=tree1.branch)
         self.assertEqualDiff('2   p?rez   | bye\n', sio.getvalue())
 
         # test now with to_file.encoding = None
         to_file = tests.StringIOWrapper()
         to_file.encoding = None
-        annotate.annotate_file(tree1.branch, 'rev-2', 'b-id', to_file=to_file)
+        annotate.annotate_file_tree(revtree_2, 'b-id',
+            to_file=to_file, branch=tree1.branch)
         self.assertContainsRe('2   p.rez   | bye\n', to_file.getvalue())
 
         # and when it does not exist
         to_file = StringIO()
-        annotate.annotate_file(tree1.branch, 'rev-2', 'b-id', to_file=to_file)
+        annotate.annotate_file_tree(revtree_2, 'b-id',
+            to_file=to_file, branch=tree1.branch)
         self.assertContainsRe('2   p.rez   | bye\n', to_file.getvalue())
 
     def test_annotate_author_or_committer(self):
@@ -447,13 +458,13 @@
 
         tree1.lock_read()
         self.addCleanup(tree1.unlock)
-        to_file = StringIO()
-        annotate.annotate_file(tree1.branch, 'rev-1', 'a-id', to_file=to_file)
-        self.assertEqual('1   committ | hello\n', to_file.getvalue())
-
-        to_file = StringIO()
-        annotate.annotate_file(tree1.branch, 'rev-2', 'b-id', to_file=to_file)
-        self.assertEqual('2   author@ | bye\n', to_file.getvalue())
+
+        self.assertBranchAnnotate('1   committ | hello\n', tree1.branch,
+            'a-id', 'rev-1')
+
+        to_file = StringIO()
+        self.assertBranchAnnotate('2   author@ | bye\n', tree1.branch,
+            'b-id', 'rev-2')
 
 
 class TestReannotate(tests.TestCase):

=== modified file 'bzrlib/tests/test_atomicfile.py'
--- bzrlib/tests/test_atomicfile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_atomicfile.py	2011-07-15 09:04:54 +0000
@@ -33,7 +33,7 @@
 
     def test_commit(self):
         f = atomicfile.AtomicFile('test')
-        self.failIfExists('test')
+        self.assertPathDoesNotExist('test')
         f.write('foo\n')
         f.commit()
 
@@ -124,30 +124,3 @@
         f.commit()
         st = os.lstat('test')
         self.assertEqualMode(0666 & ~umask, stat.S_IMODE(st.st_mode))
-
-    def test_closed(self):
-        local_warnings = []
-        def capture_warnings(msg, cls, stacklevel=None):
-            self.assertEqual(cls, DeprecationWarning)
-            local_warnings.append(msg)
-
-        method = symbol_versioning.warn
-        try:
-            symbol_versioning.set_warning_method(capture_warnings)
-            f = atomicfile.AtomicFile('test', mode='wb')
-            self.assertEqual(False, f.closed)
-            f.abort()
-            self.assertEqual(True, f.closed)
-
-            f = atomicfile.AtomicFile('test', mode='wb')
-            f.close()
-            self.assertEqual(True, f.closed)
-
-            f = atomicfile.AtomicFile('test', mode='wb')
-            f.commit()
-            self.assertEqual(True, f.closed)
-        finally:
-            symbol_versioning.set_warning_method(method)
-
-        txt = 'AtomicFile.closed deprecated in bzr 0.10'
-        self.assertEqual([txt]*4, local_warnings)

=== modified file 'bzrlib/tests/test_bad_files.py'
--- bzrlib/tests/test_bad_files.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_bad_files.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,11 +21,13 @@
 import os
 
 from bzrlib import (
-    add,
     errors,
     )
 from bzrlib.status import show_tree_status
-from bzrlib.tests import OsFifoFeature, TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.features import (
+    OsFifoFeature,
+    )
 
 
 def verify_status(tester, tree, value):

=== modified file 'bzrlib/tests/test_bisect_multi.py'
--- bzrlib/tests/test_bisect_multi.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_bisect_multi.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 """Tests for bisect_multi."""
 
-from bzrlib import errors
 from bzrlib.bisect_multi import bisect_multi_bytes
 from bzrlib.tests import TestCase
 

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2010-06-20 21:14:49 +0000
+++ bzrlib/tests/test_branch.py	2011-09-27 11:34:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 """Tests for the Branch facility that are not interface  tests.
 
-For interface tests see tests/per_branch/*.py.
+For interface tests see `tests/per_branch/*.py`.
 
 For concrete class tests see this file, and for meta-branch tests
 also see this file.
@@ -29,9 +29,9 @@
     bzrdir,
     config,
     errors,
+    symbol_versioning,
     tests,
     trace,
-    transport,
     urlutils,
     )
 
@@ -40,7 +40,7 @@
 
     def test_default_format(self):
         # update this if you change the default branch format
-        self.assertIsInstance(_mod_branch.BranchFormat.get_default_format(),
+        self.assertIsInstance(_mod_branch.format_registry.get_default(),
                 _mod_branch.BzrBranchFormat7)
 
     def test_default_format_is_same_as_bzrdir_default(self):
@@ -48,13 +48,13 @@
         # set, but at the moment that's not true -- mbp 20070814 --
         # https://bugs.launchpad.net/bzr/+bug/132376
         self.assertEqual(
-            _mod_branch.BranchFormat.get_default_format(),
+            _mod_branch.format_registry.get_default(),
             bzrdir.BzrDirFormat.get_default_format().get_branch_format())
 
     def test_get_set_default_format(self):
         # set the format and then set it back again
-        old_format = _mod_branch.BranchFormat.get_default_format()
-        _mod_branch.BranchFormat.set_default_format(SampleBranchFormat())
+        old_format = _mod_branch.format_registry.get_default()
+        _mod_branch.format_registry.set_default(SampleBranchFormat())
         try:
             # the default branch format is used by the meta dir format
             # which is not the default bzrdir format at this point
@@ -62,9 +62,9 @@
             result = dir.create_branch()
             self.assertEqual(result, 'A branch')
         finally:
-            _mod_branch.BranchFormat.set_default_format(old_format)
+            _mod_branch.format_registry.set_default(old_format)
         self.assertEqual(old_format,
-                         _mod_branch.BranchFormat.get_default_format())
+                         _mod_branch.format_registry.get_default())
 
 
 class TestBranchFormat5(tests.TestCaseWithTransport):
@@ -74,7 +74,7 @@
         url = self.get_url()
         bdir = bzrdir.BzrDirMetaFormat1().initialize(url)
         bdir.create_repository()
-        branch = bdir.create_branch()
+        branch = _mod_branch.BzrBranchFormat5().initialize(bdir)
         t = self.get_transport()
         self.log("branch instance is %r" % branch)
         self.assert_(isinstance(branch, _mod_branch.BzrBranch5))
@@ -86,18 +86,7 @@
         self.assertIsDirectory('.bzr/branch/lock/held', t)
 
     def test_set_push_location(self):
-        from bzrlib.config import (locations_config_filename,
-                                   ensure_config_dir_exists)
-        ensure_config_dir_exists()
-        fn = locations_config_filename()
-        # write correct newlines to locations.conf
-        # by default ConfigObj uses native line-endings for new files
-        # but uses already existing line-endings if file is not empty
-        f = open(fn, 'wb')
-        try:
-            f.write('# comment\n')
-        finally:
-            f.close()
+        conf = config.LocationConfig.from_string('# comment\n', '.', save=True)
 
         branch = self.make_branch('.', format='knit')
         branch.set_push_location('foo')
@@ -106,7 +95,7 @@
                              "[%s]\n"
                              "push_location = foo\n"
                              "push_location:policy = norecurse\n" % local_path,
-                             fn)
+                             config.locations_config_filename())
 
     # TODO RBC 20051029 test getting a push location from a branch in a
     # recursive section - that is, it appends the branch name.
@@ -123,7 +112,8 @@
         """See BzrBranchFormat.get_format_string()."""
         return "Sample branch format."
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Format 4 branches cannot be created."""
         t = a_bzrdir.get_branch_transport(self, name=name)
         t.put_bytes('format', self.get_format_string())
@@ -148,7 +138,7 @@
         """See BzrBranchFormat.get_format_string()."""
         return SampleSupportedBranchFormatString
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, append_revisions_only=None):
         t = a_bzrdir.get_branch_transport(self, name=name)
         t.put_bytes('format', self.get_format_string())
         return 'A branch'
@@ -157,6 +147,24 @@
         return "opened supported branch."
 
 
+class SampleExtraBranchFormat(_mod_branch.BranchFormat):
+    """A sample format that is not usable in a metadir."""
+
+    def get_format_string(self):
+        # This format is not usable in a metadir.
+        return None
+
+    def network_name(self):
+        # Network name always has to be provided.
+        return "extra"
+
+    def initialize(self, a_bzrdir, name=None):
+        raise NotImplementedError(self.initialize)
+
+    def open(self, transport, name=None, _found=False, ignore_fallbacks=False):
+        raise NotImplementedError(self.open)
+
+
 class TestBzrBranchFormat(tests.TestCaseWithTransport):
     """Tests for the BzrBranchFormat facility."""
 
@@ -170,7 +178,7 @@
             dir.create_repository()
             format.initialize(dir)
             found_format = _mod_branch.BranchFormat.find_format(dir)
-            self.failUnless(isinstance(found_format, format.__class__))
+            self.assertIsInstance(found_format, format.__class__)
         check_format(_mod_branch.BzrBranchFormat5(), "bar")
 
     def test_find_format_factory(self):
@@ -179,8 +187,8 @@
         factory = _mod_branch.MetaDirBranchFormatFactory(
             SampleSupportedBranchFormatString,
             "bzrlib.tests.test_branch", "SampleSupportedBranchFormat")
-        _mod_branch.BranchFormat.register_format(factory)
-        self.addCleanup(_mod_branch.BranchFormat.unregister_format, factory)
+        _mod_branch.format_registry.register(factory)
+        self.addCleanup(_mod_branch.format_registry.remove, factory)
         b = _mod_branch.Branch.open(self.get_url())
         self.assertEqual(b, "opened supported branch.")
 
@@ -198,13 +206,15 @@
                           dir)
 
     def test_register_unregister_format(self):
+        # Test the deprecated format registration functions
         format = SampleBranchFormat()
         # make a control dir
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         # make a branch
         format.initialize(dir)
         # register a format for it.
-        _mod_branch.BranchFormat.register_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            _mod_branch.BranchFormat.register_format, format)
         # which branch.Open will refuse (not supported)
         self.assertRaises(errors.UnsupportedFormatError,
                           _mod_branch.Branch.open, self.get_url())
@@ -214,10 +224,53 @@
             format.open(dir),
             bzrdir.BzrDir.open(self.get_url()).open_branch(unsupported=True))
         # unregister the format
-        _mod_branch.BranchFormat.unregister_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            _mod_branch.BranchFormat.unregister_format, format)
         self.make_branch_and_tree('bar')
 
 
+class TestBranchFormatRegistry(tests.TestCase):
+
+    def setUp(self):
+        super(TestBranchFormatRegistry, self).setUp()
+        self.registry = _mod_branch.BranchFormatRegistry()
+
+    def test_default(self):
+        self.assertIs(None, self.registry.get_default())
+        format = SampleBranchFormat()
+        self.registry.set_default(format)
+        self.assertEquals(format, self.registry.get_default())
+
+    def test_register_unregister_format(self):
+        format = SampleBranchFormat()
+        self.registry.register(format)
+        self.assertEquals(format,
+            self.registry.get("Sample branch format."))
+        self.registry.remove(format)
+        self.assertRaises(KeyError, self.registry.get,
+            "Sample branch format.")
+
+    def test_get_all(self):
+        format = SampleBranchFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra(self):
+        format = SampleExtraBranchFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra_lazy(self):
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra_lazy("bzrlib.tests.test_branch",
+            "SampleExtraBranchFormat")
+        formats = self.registry._get_all()
+        self.assertEquals(1, len(formats))
+        self.assertIsInstance(formats[0], SampleExtraBranchFormat)
+
+
 #Used by TestMetaDirBranchFormatFactory 
 FakeLazyFormat = None
 
@@ -270,23 +323,23 @@
 
     def test_layout(self):
         branch = self.make_branch('a', format=self.get_format_name())
-        self.failUnlessExists('a/.bzr/branch/last-revision')
-        self.failIfExists('a/.bzr/branch/revision-history')
-        self.failIfExists('a/.bzr/branch/references')
+        self.assertPathExists('a/.bzr/branch/last-revision')
+        self.assertPathDoesNotExist('a/.bzr/branch/revision-history')
+        self.assertPathDoesNotExist('a/.bzr/branch/references')
 
     def test_config(self):
         """Ensure that all configuration data is stored in the branch"""
         branch = self.make_branch('a', format=self.get_format_name())
-        branch.set_parent('http://bazaar-vcs.org')
-        self.failIfExists('a/.bzr/branch/parent')
-        self.assertEqual('http://bazaar-vcs.org', branch.get_parent())
-        branch.set_push_location('sftp://bazaar-vcs.org')
+        branch.set_parent('http://example.com')
+        self.assertPathDoesNotExist('a/.bzr/branch/parent')
+        self.assertEqual('http://example.com', branch.get_parent())
+        branch.set_push_location('sftp://example.com')
         config = branch.get_config()._get_branch_data_config()
-        self.assertEqual('sftp://bazaar-vcs.org',
+        self.assertEqual('sftp://example.com',
                          config.get_user_option('push_location'))
-        branch.set_bound_location('ftp://bazaar-vcs.org')
-        self.failIfExists('a/.bzr/branch/bound')
-        self.assertEqual('ftp://bazaar-vcs.org', branch.get_bound_location())
+        branch.set_bound_location('ftp://example.com')
+        self.assertPathDoesNotExist('a/.bzr/branch/bound')
+        self.assertEqual('ftp://example.com', branch.get_bound_location())
 
     def test_set_revision_history(self):
         builder = self.make_branch_builder('.', format=self.get_format_name())
@@ -297,10 +350,13 @@
         branch = builder.get_branch()
         branch.lock_write()
         self.addCleanup(branch.unlock)
-        branch.set_revision_history(['foo', 'bar'])
-        branch.set_revision_history(['foo'])
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            branch.set_revision_history, ['foo', 'bar'])
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+                branch.set_revision_history, ['foo'])
         self.assertRaises(errors.NotLefthandHistory,
-                          branch.set_revision_history, ['bar'])
+            self.applyDeprecated, symbol_versioning.deprecated_in((2, 4, 0)),
+            branch.set_revision_history, ['bar'])
 
     def do_checkout_test(self, lightweight=False):
         tree = self.make_branch_and_tree('source',
@@ -319,10 +375,10 @@
         subtree.commit('a subtree file')
         subsubtree.commit('a subsubtree file')
         tree.branch.create_checkout('target', lightweight=lightweight)
-        self.failUnlessExists('target')
-        self.failUnlessExists('target/subtree')
-        self.failUnlessExists('target/subtree/file')
-        self.failUnlessExists('target/subtree/subsubtree/file')
+        self.assertPathExists('target')
+        self.assertPathExists('target/subtree')
+        self.assertPathExists('target/subtree/file')
+        self.assertPathExists('target/subtree/subsubtree/file')
         subbranch = _mod_branch.Branch.open('target/subtree/subsubtree')
         if lightweight:
             self.assertEndsWith(subbranch.base, 'source/subtree/subsubtree/')
@@ -485,12 +541,22 @@
         self.assertEqual(('path3', 'location3'),
                          branch.get_reference_info('file-id'))
 
+    def _recordParentMapCalls(self, repo):
+        self._parent_map_calls = []
+        orig_get_parent_map = repo.revisions.get_parent_map
+        def get_parent_map(q):
+            q = list(q)
+            self._parent_map_calls.extend([e[0] for e in q])
+            return orig_get_parent_map(q)
+        repo.revisions.get_parent_map = get_parent_map
+
+
 class TestBranchReference(tests.TestCaseWithTransport):
     """Tests for the branch reference facility."""
 
     def test_create_open_reference(self):
         bzrdirformat = bzrdir.BzrDirMetaFormat1()
-        t = transport.get_transport(self.get_url('.'))
+        t = self.get_transport()
         t.mkdir('repo')
         dir = bzrdirformat.initialize(self.get_url('repo'))
         dir.create_repository()
@@ -552,6 +618,15 @@
         self.assertTrue(hasattr(params, 'bzrdir'))
         self.assertTrue(hasattr(params, 'branch'))
 
+    def test_post_branch_init_hook_repr(self):
+        param_reprs = []
+        _mod_branch.Branch.hooks.install_named_hook('post_branch_init',
+            lambda params: param_reprs.append(repr(params)), None)
+        branch = self.make_branch('a')
+        self.assertLength(1, param_reprs)
+        param_repr = param_reprs[0]
+        self.assertStartsWith(param_repr, '<BranchInitHookParams of ')
+
     def test_post_switch_hook(self):
         from bzrlib import switch
         calls = []
@@ -588,7 +663,7 @@
         if value is not None:
             self.config.set_user_option('append_revisions_only', value)
         self.assertEqual(expected_value,
-                         self.branch._get_append_revisions_only())
+                         self.branch.get_append_revisions_only())
 
     def test_valid_append_revisions_only(self):
         self.assertEquals(None,
@@ -624,8 +699,10 @@
         # this usage of results is not recommended for new code (because it
         # doesn't describe very well what happened), but for api stability
         # it's still supported
-        a = "%d revisions pulled" % r
-        self.assertEqual(a, "10 revisions pulled")
+        self.assertEqual(self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            r.__int__),
+            10)
 
     def test_report_changed(self):
         r = _mod_branch.PullResult()
@@ -643,74 +720,5 @@
         r.new_revid = "same-revid"
         f = StringIO()
         r.report(f)
-        self.assertEqual("No revisions to pull.\n", f.getvalue())
-
-
-class _StubLockable(object):
-    """Helper for TestRunWithWriteLockedTarget."""
-
-    def __init__(self, calls, unlock_exc=None):
-        self.calls = calls
-        self.unlock_exc = unlock_exc
-
-    def lock_write(self):
-        self.calls.append('lock_write')
-
-    def unlock(self):
-        self.calls.append('unlock')
-        if self.unlock_exc is not None:
-            raise self.unlock_exc
-
-
-class _ErrorFromCallable(Exception):
-    """Helper for TestRunWithWriteLockedTarget."""
-
-
-class _ErrorFromUnlock(Exception):
-    """Helper for TestRunWithWriteLockedTarget."""
-
-
-class TestRunWithWriteLockedTarget(tests.TestCase):
-    """Tests for _run_with_write_locked_target."""
-
-    def setUp(self):
-        tests.TestCase.setUp(self)
-        self._calls = []
-
-    def func_that_returns_ok(self):
-        self._calls.append('func called')
-        return 'ok'
-
-    def func_that_raises(self):
-        self._calls.append('func called')
-        raise _ErrorFromCallable()
-
-    def test_success_unlocks(self):
-        lockable = _StubLockable(self._calls)
-        result = _mod_branch._run_with_write_locked_target(
-            lockable, self.func_that_returns_ok)
-        self.assertEqual('ok', result)
-        self.assertEqual(['lock_write', 'func called', 'unlock'], self._calls)
-
-    def test_exception_unlocks_and_propagates(self):
-        lockable = _StubLockable(self._calls)
-        self.assertRaises(_ErrorFromCallable,
-                          _mod_branch._run_with_write_locked_target,
-                          lockable, self.func_that_raises)
-        self.assertEqual(['lock_write', 'func called', 'unlock'], self._calls)
-
-    def test_callable_succeeds_but_error_during_unlock(self):
-        lockable = _StubLockable(self._calls, unlock_exc=_ErrorFromUnlock())
-        self.assertRaises(_ErrorFromUnlock,
-                          _mod_branch._run_with_write_locked_target,
-                          lockable, self.func_that_returns_ok)
-        self.assertEqual(['lock_write', 'func called', 'unlock'], self._calls)
-
-    def test_error_during_unlock_does_not_mask_original_error(self):
-        lockable = _StubLockable(self._calls, unlock_exc=_ErrorFromUnlock())
-        self.assertRaises(_ErrorFromCallable,
-                          _mod_branch._run_with_write_locked_target,
-                          lockable, self.func_that_raises)
-        self.assertEqual(['lock_write', 'func called', 'unlock'], self._calls)
-
+        self.assertEqual("No revisions or tags to pull.\n", f.getvalue())
 

=== modified file 'bzrlib/tests/test_branchbuilder.py'
--- bzrlib/tests/test_branchbuilder.py	2010-02-27 12:27:33 +0000
+++ bzrlib/tests/test_branchbuilder.py	2011-07-15 09:22:16 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2009 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 from bzrlib import (
     branch as _mod_branch,
-    errors,
     revision as _mod_revision,
     tests,
     )
@@ -248,6 +247,18 @@
                               (u'dir', 'dir-id', 'directory'),
                               (u'dir/a', 'a-id', 'file')], rev_tree)
 
+    def test_rename_out_of_unversioned_subdir(self):
+        builder = self.build_a_rev()
+        builder.build_snapshot('B-id', None,
+            [('add', ('dir', 'dir-id', 'directory', None)),
+             ('rename', ('a', 'dir/a'))])
+        builder.build_snapshot('C-id', None,
+            [('rename', ('dir/a', 'a')),
+             ('unversion', 'dir-id')])
+        rev_tree = builder.get_branch().repository.revision_tree('C-id')
+        self.assertTreeShape([(u'', 'a-root-id', 'directory'),
+                              (u'a', 'a-id', 'file')], rev_tree)
+
     def test_set_parent(self):
         builder = self.build_a_rev()
         builder.start_series()
@@ -322,8 +333,23 @@
                              ], d_tree)
         # Because we copied the exact text into *this* tree, the 'c' file
         # should look like it was not modified in the merge
-        self.assertEqual('C-id', d_tree.inventory['c-id'].revision)
-
+        self.assertEqual('C-id', d_tree.get_file_revision('c-id'))
+
+    def test_set_parent_to_null(self):
+        builder = self.build_a_rev()
+        builder.start_series()
+        self.addCleanup(builder.finish_series)
+        builder.build_snapshot('B-id', [],
+            [('add', ('', None, 'directory', None))])
+        # We should now have a graph:
+        #   A B
+        # And not A => B
+        repo = builder.get_branch().repository
+        self.assertEqual({'A-id': (_mod_revision.NULL_REVISION,),
+                          'B-id': (_mod_revision.NULL_REVISION,),},
+                         repo.get_parent_map(['A-id', 'B-id']))
+
+    
     def test_start_finish_series(self):
         builder = BranchBuilder(self.get_transport().clone('foo'))
         builder.start_series()
@@ -350,3 +376,63 @@
         self.addCleanup(b.unlock)
         self.assertEqual(('ghost',),
             b.repository.get_graph().get_parent_map(['tip'])['tip'])
+
+    def test_unversion_root_add_new_root(self):
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('rev-1', None,
+            [('add', ('', 'TREE_ROOT', 'directory', ''))])
+        builder.build_snapshot('rev-2', None,
+            [('unversion', 'TREE_ROOT'),
+             ('add', ('', 'my-root', 'directory', ''))])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('rev-2')
+        self.assertTreeShape([(u'', 'my-root', 'directory')], rev_tree)
+
+    def test_empty_flush(self):
+        """A flush with no actions before it is a no-op."""
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('rev-1', None,
+            [('add', ('', 'TREE_ROOT', 'directory', ''))])
+        builder.build_snapshot('rev-2', None, [('flush', None)])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('rev-2')
+        self.assertTreeShape([(u'', 'TREE_ROOT', 'directory')], rev_tree)
+
+    def test_kind_change(self):
+        """It's possible to change the kind of an entry in a single snapshot
+        with a bit of help from the 'flush' action.
+        """
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('A-id', None,
+            [('add', (u'', 'a-root-id', 'directory', None)),
+             ('add', (u'a', 'a-id', 'file', 'content\n'))])
+        builder.build_snapshot('B-id', None,
+            [('unversion', 'a-id'),
+             ('flush', None),
+             ('add', (u'a', 'a-id', 'directory', None))])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('B-id')
+        self.assertTreeShape(
+            [(u'', 'a-root-id', 'directory'), (u'a', 'a-id', 'directory')],
+            rev_tree)
+
+    def test_pivot_root(self):
+        """It's possible (albeit awkward) to move an existing dir to the root
+        in a single snapshot by using unversion then flush then add.
+        """
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        builder.start_series()
+        builder.build_snapshot('A-id', None,
+            [('add', (u'', 'orig-root', 'directory', None)),
+             ('add', (u'dir', 'dir-id', 'directory', None))])
+        builder.build_snapshot('B-id', None,
+            [('unversion', 'orig-root'),  # implicitly unversions all children
+             ('flush', None),
+             ('add', (u'', 'dir-id', 'directory', None))])
+        builder.finish_series()
+        rev_tree = builder.get_branch().repository.revision_tree('B-id')
+        self.assertTreeShape([(u'', 'dir-id', 'directory')], rev_tree)
+

=== modified file 'bzrlib/tests/test_btree_index.py'
--- bzrlib/tests/test_btree_index.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_btree_index.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,26 +31,27 @@
     )
 from bzrlib.tests import (
     TestCaseWithTransport,
-    condition_isinstance,
-    multiply_tests,
-    split_suite_by_condition,
-    )
-
-
-def load_tests(standard_tests, module, loader):
-    # parameterise the TestBTreeNodes tests
-    node_tests, others = split_suite_by_condition(standard_tests,
-        condition_isinstance(TestBTreeNodes))
+    scenarios,
+    )
+from bzrlib.tests import (
+    features,
+    )
+
+
+load_tests = scenarios.load_tests_apply_scenarios
+
+
+def btreeparser_scenarios():
     import bzrlib._btree_serializer_py as py_module
     scenarios = [('python', {'parse_btree': py_module})]
     if compiled_btreeparser_feature.available():
-        scenarios.append(('C', {'parse_btree':
-                                compiled_btreeparser_feature.module}))
-    return multiply_tests(node_tests, scenarios, others)
-
-
-compiled_btreeparser_feature = tests.ModuleAvailableFeature(
-                                'bzrlib._btree_serializer_pyx')
+        scenarios.append(('C', 
+            {'parse_btree': compiled_btreeparser_feature.module}))
+    return scenarios
+
+
+compiled_btreeparser_feature = features.ModuleAvailableFeature(
+    'bzrlib._btree_serializer_pyx')
 
 
 class BTreeTestCase(TestCaseWithTransport):
@@ -216,12 +217,12 @@
         leaf1_bytes = zlib.decompress(leaf1)
         sorted_node_keys = sorted(node[0] for node in nodes)
         node = btree_index._LeafNode(leaf1_bytes, 1, 0)
-        self.assertEqual(231, len(node.keys))
-        self.assertEqual(sorted_node_keys[:231], sorted(node.keys))
+        self.assertEqual(231, len(node))
+        self.assertEqual(sorted_node_keys[:231], node.all_keys())
         leaf2_bytes = zlib.decompress(leaf2)
         node = btree_index._LeafNode(leaf2_bytes, 1, 0)
-        self.assertEqual(400 - 231, len(node.keys))
-        self.assertEqual(sorted_node_keys[231:], sorted(node.keys))
+        self.assertEqual(400 - 231, len(node))
+        self.assertEqual(sorted_node_keys[231:], node.all_keys())
 
     def test_last_page_rounded_1_layer(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
@@ -241,9 +242,9 @@
         leaf2 = content[74:]
         leaf2_bytes = zlib.decompress(leaf2)
         node = btree_index._LeafNode(leaf2_bytes, 1, 0)
-        self.assertEqual(10, len(node.keys))
+        self.assertEqual(10, len(node))
         sorted_node_keys = sorted(node[0] for node in nodes)
-        self.assertEqual(sorted_node_keys, sorted(node.keys))
+        self.assertEqual(sorted_node_keys, node.all_keys())
 
     def test_last_page_not_rounded_2_layer(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
@@ -263,9 +264,9 @@
         leaf2 = content[8192:]
         leaf2_bytes = zlib.decompress(leaf2)
         node = btree_index._LeafNode(leaf2_bytes, 1, 0)
-        self.assertEqual(400 - 231, len(node.keys))
+        self.assertEqual(400 - 231, len(node))
         sorted_node_keys = sorted(node[0] for node in nodes)
-        self.assertEqual(sorted_node_keys[231:], sorted(node.keys))
+        self.assertEqual(sorted_node_keys[231:], node.all_keys())
 
     def test_three_level_tree_details(self):
         # The left most pointer in the second internal node in a row should
@@ -280,7 +281,7 @@
 
         for node in nodes:
             builder.add_node(*node)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', self.time(builder.finish))
         del builder
         index = btree_index.BTreeGraphIndex(t, 'index', size)
@@ -302,7 +303,7 @@
         # in the second node it points at
         pos = index._row_offsets[2] + internal_node2.offset + 1
         leaf = index._get_leaf_nodes([pos])[pos]
-        self.assertTrue(internal_node2.keys[0] in leaf.keys)
+        self.assertTrue(internal_node2.keys[0] in leaf)
 
     def test_2_leaves_2_2(self):
         builder = btree_index.BTreeBuilder(key_elements=2, reference_lists=2)
@@ -607,7 +608,7 @@
         for key, value, references in nodes:
             builder.add_node(key, value, references)
         stream = builder.finish()
-        trans = transport.get_transport('trace+' + self.get_url())
+        trans = transport.get_transport_from_url('trace+' + self.get_url())
         size = trans.put_file('index', stream)
         return btree_index.BTreeGraphIndex(trans, 'index', size)
 
@@ -648,20 +649,20 @@
         self.assertEqual(0, len(index._leaf_node_cache))
 
     def test_trivial_constructor(self):
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         index = btree_index.BTreeGraphIndex(t, 'index', None)
         # Checks the page size at load, but that isn't logged yet.
         self.assertEqual([], t._activity)
 
     def test_with_size_constructor(self):
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         index = btree_index.BTreeGraphIndex(t, 'index', 1)
         # Checks the page size at load, but that isn't logged yet.
         self.assertEqual([], t._activity)
 
     def test_empty_key_count_no_size(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         t.put_file('index', builder.finish())
         index = btree_index.BTreeGraphIndex(t, 'index', None)
         del t._activity[:]
@@ -674,7 +675,7 @@
 
     def test_empty_key_count(self):
         builder = btree_index.BTreeBuilder(key_elements=1, reference_lists=0)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', builder.finish())
         self.assertEqual(72, size)
         index = btree_index.BTreeGraphIndex(t, 'index', size)
@@ -690,7 +691,7 @@
         nodes = self.make_nodes(35, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', builder.finish())
         index = btree_index.BTreeGraphIndex(t, 'index', size)
         del t._activity[:]
@@ -722,13 +723,13 @@
 
     def test__read_nodes_no_size_one_page_reads_once(self):
         self.make_index(nodes=[(('key',), 'value', ())])
-        trans = transport.get_transport('trace+' + self.get_url())
+        trans = transport.get_transport_from_url('trace+' + self.get_url())
         index = btree_index.BTreeGraphIndex(trans, 'index', None)
         del trans._activity[:]
         nodes = dict(index._read_nodes([0]))
         self.assertEqual([0], nodes.keys())
         node = nodes[0]
-        self.assertEqual([('key',)], node.keys.keys())
+        self.assertEqual([('key',)], node.all_keys())
         self.assertEqual([('get', 'index')], trans._activity)
 
     def test__read_nodes_no_size_multiple_pages(self):
@@ -736,7 +737,7 @@
         index.key_count()
         num_pages = index._row_offsets[-1]
         # Reopen with a traced transport and no size
-        trans = transport.get_transport('trace+' + self.get_url())
+        trans = transport.get_transport_from_url('trace+' + self.get_url())
         index = btree_index.BTreeGraphIndex(trans, 'index', None)
         del trans._activity[:]
         nodes = dict(index._read_nodes([0]))
@@ -747,7 +748,7 @@
         nodes = self.make_nodes(160, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', builder.finish())
         self.assertEqual(17692, size)
         index = btree_index.BTreeGraphIndex(t, 'index', size)
@@ -763,7 +764,7 @@
         nodes = self.make_nodes(45, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', builder.finish())
         index = btree_index.BTreeGraphIndex(t, 'index', size)
         del t._activity[:]
@@ -779,7 +780,7 @@
         nodes = self.make_nodes(80, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', builder.finish())
         # Root page, 2 leaf pages
         self.assertEqual(9339, size)
@@ -797,8 +798,8 @@
 
     def test_eq_ne(self):
         # two indices are equal when constructed with the same parameters:
-        t1 = transport.get_transport('trace+' + self.get_url(''))
-        t2 = transport.get_transport(self.get_url(''))
+        t1 = transport.get_transport_from_url('trace+' + self.get_url(''))
+        t2 = self.get_transport()
         self.assertTrue(
             btree_index.BTreeGraphIndex(t1, 'index', None) ==
             btree_index.BTreeGraphIndex(t1, 'index', None))
@@ -832,7 +833,7 @@
 
     def test_iter_all_only_root_no_size(self):
         self.make_index(nodes=[(('key',), 'value', ())])
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         index = btree_index.BTreeGraphIndex(t, 'index', None)
         del t._activity[:]
         self.assertEqual([(('key',), 'value')],
@@ -849,7 +850,7 @@
         nodes = self.make_nodes(10000, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', builder.finish())
         self.assertEqual(1303220, size, 'number of expected bytes in the'
                                         ' output changed')
@@ -904,7 +905,7 @@
         nodes = self.make_nodes(160, 2, 2)
         for node in nodes:
             builder.add_node(*node)
-        t = transport.get_transport('trace+' + self.get_url(''))
+        t = transport.get_transport_from_url('trace+' + self.get_url(''))
         size = t.put_file('index', builder.finish())
         del builder
         index = btree_index.BTreeGraphIndex(t, 'index', size)
@@ -1112,7 +1113,7 @@
         min_l2_key = l2.min_key
         max_l1_key = l1.max_key
         self.assertTrue(max_l1_key < min_l2_key)
-        parents_min_l2_key = l2.keys[min_l2_key][1][0]
+        parents_min_l2_key = l2[min_l2_key][1][0]
         self.assertEqual((l1.max_key,), parents_min_l2_key)
         # Now, whatever key we select that would fall on the second page,
         # should give us all the parents until the page break
@@ -1131,7 +1132,7 @@
         parent_map = {}
         search_keys = index._find_ancestors([max_l1_key], 0, parent_map,
                                             missing_keys)
-        self.assertEqual(sorted(l1.keys), sorted(parent_map))
+        self.assertEqual(l1.all_keys(), sorted(parent_map))
         self.assertEqual(set(), missing_keys)
         self.assertEqual(set(), search_keys)
 
@@ -1153,7 +1154,7 @@
         for node in nodes:
             builder.add_node(*node)
         stream = builder.finish()
-        trans = transport.get_transport(self.get_url())
+        trans = self.get_transport()
         size = trans.put_file('index', stream)
         index = btree_index.BTreeGraphIndex(trans, 'index', size)
         self.assertEqual(500, index.key_count())
@@ -1185,6 +1186,8 @@
 
 class TestBTreeNodes(BTreeTestCase):
 
+    scenarios = btreeparser_scenarios()
+
     def setUp(self):
         BTreeTestCase.setUp(self)
         self.overrideAttr(btree_index, '_btree_serializer', self.parse_btree)
@@ -1205,7 +1208,7 @@
             ("2222222222222222222222222222222222222222",): ("value:2", ()),
             ("3333333333333333333333333333333333333333",): ("value:3", ()),
             ("4444444444444444444444444444444444444444",): ("value:4", ()),
-            }, node.keys)
+            }, dict(node.all_items()))
 
     def test_LeafNode_2_2(self):
         node_bytes = ("type=leaf\n"
@@ -1225,7 +1228,7 @@
             ('11', '33'): ('value:3',
                 ((('11', 'ref22'),), (('11', 'ref22'), ('11', 'ref22')))),
             ('11', '44'): ('value:4', ((), (('11', 'ref00'),)))
-            }, node.keys)
+            }, dict(node.all_items()))
 
     def test_InternalNode_1(self):
         node_bytes = ("type=internal\n"
@@ -1266,7 +1269,7 @@
             ('11', '33'): ('value:3',
                 ((('11', 'ref22'),), (('11', 'ref22'), ('11', 'ref22')))),
             ('11', '44'): ('value:4', ((), (('11', 'ref00'),)))
-            }, node.keys)
+            }, dict(node.all_items()))
 
     def assertFlattened(self, expected, key, value, refs):
         flat_key, flat_line = self.parse_btree._flatten_node(
@@ -1351,7 +1354,8 @@
         BTreeGraphIndex with the recommended information.
         """
         index = btree_index.BTreeGraphIndex(
-            transport.get_transport('memory:///'), 'test-index', size=size)
+            transport.get_transport_from_url('memory:///'),
+            'test-index', size=size)
         if recommended_pages is not None:
             index._recommended_pages = recommended_pages
         return index

=== modified file 'bzrlib/tests/test_bugtracker.py'
--- bzrlib/tests/test_bugtracker.py	2010-01-25 15:55:48 +0000
+++ bzrlib/tests/test_bugtracker.py	2010-09-25 20:08:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -117,6 +117,14 @@
         self.assertEqual('http://bugs.com/1234/view.html',
                          tracker.get_bug_url('1234'))
 
+    def test_generic_registered_non_integer(self):
+        branch = self.make_branch('some_branch')
+        config = branch.get_config()
+        config.set_user_option('bugtracker_foo_url', 'http://bugs.com/{id}/view.html')
+        tracker = bugtracker.tracker_registry.get_tracker('foo', branch)
+        self.assertEqual('http://bugs.com/ABC-1234/view.html',
+                         tracker.get_bug_url('ABC-1234'))
+
     def test_generic_incorrect_url(self):
         branch = self.make_branch('some_branch')
         config = branch.get_config()
@@ -173,15 +181,13 @@
         self.assertRaises(
             errors.MalformedBugIdentifier, tracker.check_bug_id, 'red')
 
-
-class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
-    """Tests for TracTracker."""
+class TestURLParametrizedBugTracker(TestCaseWithMemoryTransport):
+    """Tests for URLParametrizedBugTracker."""
 
     def setUp(self):
         TestCaseWithMemoryTransport.setUp(self)
         self.url = 'http://twistedmatrix.com/trac'
-        self.tracker = bugtracker.URLParametrizedIntegerBugTracker('some',
-                                                                   'ticket/')
+        self.tracker = bugtracker.URLParametrizedBugTracker('some', 'ticket/')
 
     def test_get_with_unsupported_tag(self):
         """If asked for an unrecognized or unconfigured tag, return None."""
@@ -203,6 +209,22 @@
             urlutils.join(self.url, 'ticket/') + '1234',
             tracker.get_bug_url('1234'))
 
+    def test_get_bug_url_for_integer_id(self):
+        self.tracker.check_bug_id('1234')
+
+    def test_get_bug_url_for_non_integer_id(self):
+        self.tracker.check_bug_id('ABC-1234')
+
+
+class TestURLParametrizedIntegerBugTracker(TestCaseWithMemoryTransport):
+    """Tests for URLParametrizedIntegerBugTracker."""
+
+    def setUp(self):
+        TestCaseWithMemoryTransport.setUp(self)
+        self.url = 'http://twistedmatrix.com/trac'
+        self.tracker = bugtracker.URLParametrizedIntegerBugTracker('some',
+                                                                   'ticket/')
+
     def test_get_bug_url_for_bad_bug(self):
         """When given a bug identifier that is invalid for Trac, get_bug_url
         should raise an error.

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_bundle.py	2011-10-05 12:16:17 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
     inventory,
     merge,
     osutils,
-    repository,
     revision as _mod_revision,
     tests,
     treebuilder,
@@ -35,19 +34,20 @@
 from bzrlib.bundle import read_mergeable_from_url
 from bzrlib.bundle.apply_bundle import install_bundle, merge_bundle
 from bzrlib.bundle.bundle_data import BundleTree
-from bzrlib.bzrdir import BzrDir
 from bzrlib.directory_service import directories
 from bzrlib.bundle.serializer import write_bundle, read_bundle, v09, v4
 from bzrlib.bundle.serializer.v08 import BundleSerializerV08
 from bzrlib.bundle.serializer.v09 import BundleSerializerV09
 from bzrlib.bundle.serializer.v4 import BundleSerializerV4
-from bzrlib.branch import Branch
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import (
     test_read_bundle,
     test_commit,
     )
 from bzrlib.transform import TreeTransform
+from bzrlib.tests import (
+    features,
+    )
 
 
 def get_text(vf, key):
@@ -67,6 +67,7 @@
 
 
 class MockTree(object):
+
     def __init__(self):
         from bzrlib.inventory import InventoryDirectory, ROOT_ID
         object.__init__(self)
@@ -77,8 +78,8 @@
 
     inventory = property(lambda x:x)
 
-    def __iter__(self):
-        return self.paths.iterkeys()
+    def all_file_ids(self):
+        return set(self.paths.keys())
 
     def __getitem__(self, file_id):
         if file_id == self.root.file_id:
@@ -114,12 +115,12 @@
             ie = InventoryDirectory(file_id, name, parent_id)
         elif kind == 'file':
             ie = InventoryFile(file_id, name, parent_id)
+            ie.text_sha1 = text_sha_1
+            ie.text_size = text_size
         elif kind == 'symlink':
             ie = InventoryLink(file_id, name, parent_id)
         else:
             raise errors.BzrError('unknown kind %r' % kind)
-        ie.text_sha1 = text_sha_1
-        ie.text_size = text_size
         return ie
 
     def add_dir(self, file_id, path):
@@ -145,6 +146,9 @@
         result.seek(0,0)
         return result
 
+    def get_file_revision(self, file_id):
+        return self.inventory[file_id].revision
+
     def contents_stats(self, file_id):
         if file_id not in self.contents:
             return None, None
@@ -492,7 +496,7 @@
                                  % (ancestor,))
 
                 # Now check that the file contents are all correct
-                for inventory_id in old:
+                for inventory_id in old.all_file_ids():
                     try:
                         old_file = old.get_file(inventory_id)
                     except errors.NoSuchFile:
@@ -505,8 +509,7 @@
                 new.unlock()
                 old.unlock()
         if not _mod_revision.is_null(rev_id):
-            rh = self.b1.revision_history()
-            tree.branch.set_revision_history(rh[:rh.index(rev_id)+1])
+            tree.branch.generate_revision_history(rev_id)
             tree.update()
             delta = tree.changes_from(self.b1.repository.revision_tree(rev_id))
             self.assertFalse(delta.has_changed(),
@@ -679,7 +682,7 @@
     def _test_symlink_bundle(self, link_name, link_target, new_link_target):
         link_id = 'link-1'
 
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.tree1 = self.make_branch_and_tree('b1')
         self.b1 = self.tree1.branch
 
@@ -726,7 +729,7 @@
         self._test_symlink_bundle('link', 'bar/foo', 'mars')
 
     def test_unicode_symlink_bundle(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_symlink_bundle(u'\N{Euro Sign}link',
                                   u'bar/\N{Euro Sign}foo',
                                   u'mars\N{Euro Sign}')
@@ -833,7 +836,7 @@
         return bundle_file.getvalue()
 
     def test_unicode_bundle(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # Handle international characters
         os.mkdir('b1')
         f = open(u'b1/with Dod\N{Euro Sign}', 'wb')
@@ -1412,7 +1415,7 @@
         branch = tree_a.branch
         repo_a = branch.repository
         tree_a.commit("base", allow_pointless=True, rev_id='A')
-        self.failIf(branch.repository.has_signature_for_revision_id('A'))
+        self.assertFalse(branch.repository.has_signature_for_revision_id('A'))
         try:
             from bzrlib.testament import Testament
             # monkey patch gpg signing mechanism
@@ -1442,12 +1445,6 @@
         install_bundle(repo_b, serializer.read(s))
 
 
-class V4WeaveBundleTester(V4BundleTester):
-
-    def bzrdir_format(self):
-        return 'metaweave'
-
-
 class V4_2aBundleTester(V4BundleTester):
 
     def bzrdir_format(self):

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2010-06-29 16:21:13 +0000
+++ bzrlib/tests/test_bzrdir.py	2011-10-13 16:52:17 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,21 +26,29 @@
 from bzrlib import (
     branch,
     bzrdir,
+    config,
+    controldir,
     errors,
     help_topics,
+    lock,
     repository,
+    revision as _mod_revision,
     osutils,
     remote,
+    symbol_versioning,
+    transport as _mod_transport,
     urlutils,
     win32utils,
-    workingtree,
+    workingtree_3,
+    workingtree_4,
     )
 import bzrlib.branch
-from bzrlib.errors import (NotBranchError,
-                           NoColocatedBranchSupport,
-                           UnknownFormatError,
-                           UnsupportedFormatError,
-                           )
+from bzrlib.errors import (
+    NotBranchError,
+    NoColocatedBranchSupport,
+    UnknownFormatError,
+    UnsupportedFormatError,
+    )
 from bzrlib.tests import (
     TestCase,
     TestCaseWithMemoryTransport,
@@ -53,71 +61,74 @@
     )
 from bzrlib.tests.test_http import TestWithTransport_pycurl
 from bzrlib.transport import (
-    get_transport,
     memory,
     pathfilter,
     )
 from bzrlib.transport.http._urllib import HttpTransport_urllib
 from bzrlib.transport.nosmart import NoSmartTransportDecorator
 from bzrlib.transport.readonly import ReadonlyTransportDecorator
-from bzrlib.repofmt import knitrepo, weaverepo, pack_repo
+from bzrlib.repofmt import knitrepo, knitpack_repo
 
 
 class TestDefaultFormat(TestCase):
 
     def test_get_set_default_format(self):
         old_format = bzrdir.BzrDirFormat.get_default_format()
-        # default is BzrDirFormat6
-        self.failUnless(isinstance(old_format, bzrdir.BzrDirMetaFormat1))
-        bzrdir.BzrDirFormat._set_default_format(SampleBzrDirFormat())
+        # default is BzrDirMetaFormat1
+        self.assertIsInstance(old_format, bzrdir.BzrDirMetaFormat1)
+        controldir.ControlDirFormat._set_default_format(SampleBzrDirFormat())
         # creating a bzr dir should now create an instrumented dir.
         try:
             result = bzrdir.BzrDir.create('memory:///')
-            self.failUnless(isinstance(result, SampleBzrDir))
+            self.assertIsInstance(result, SampleBzrDir)
         finally:
-            bzrdir.BzrDirFormat._set_default_format(old_format)
+            controldir.ControlDirFormat._set_default_format(old_format)
         self.assertEqual(old_format, bzrdir.BzrDirFormat.get_default_format())
 
 
+class DeprecatedBzrDirFormat(bzrdir.BzrDirFormat):
+    """A deprecated bzr dir format."""
+
+
 class TestFormatRegistry(TestCase):
 
     def make_format_registry(self):
-        my_format_registry = bzrdir.BzrDirFormatRegistry()
-        my_format_registry.register('weave', bzrdir.BzrDirFormat6,
-            'Pre-0.8 format.  Slower and does not support checkouts or shared'
-            ' repositories', deprecated=True)
-        my_format_registry.register_lazy('lazy', 'bzrlib.bzrdir',
-            'BzrDirFormat6', 'Format registered lazily', deprecated=True)
-        my_format_registry.register_metadir('knit',
+        my_format_registry = controldir.ControlDirFormatRegistry()
+        my_format_registry.register('deprecated', DeprecatedBzrDirFormat,
+            'Some format.  Slower and unawesome and deprecated.',
+            deprecated=True)
+        my_format_registry.register_lazy('lazy', 'bzrlib.tests.test_bzrdir',
+            'DeprecatedBzrDirFormat', 'Format registered lazily',
+            deprecated=True)
+        bzrdir.register_metadir(my_format_registry, 'knit',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
             'Format using knits',
             )
         my_format_registry.set_default('knit')
-        my_format_registry.register_metadir(
+        bzrdir.register_metadir(my_format_registry,
             'branch6',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit3',
             'Experimental successor to knit.  Use at your own risk.',
             branch_format='bzrlib.branch.BzrBranchFormat6',
             experimental=True)
-        my_format_registry.register_metadir(
+        bzrdir.register_metadir(my_format_registry,
             'hidden format',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit3',
             'Experimental successor to knit.  Use at your own risk.',
             branch_format='bzrlib.branch.BzrBranchFormat6', hidden=True)
-        my_format_registry.register('hiddenweave', bzrdir.BzrDirFormat6,
-            'Pre-0.8 format.  Slower and does not support checkouts or shared'
-            ' repositories', hidden=True)
-        my_format_registry.register_lazy('hiddenlazy', 'bzrlib.bzrdir',
-            'BzrDirFormat6', 'Format registered lazily', deprecated=True,
-            hidden=True)
+        my_format_registry.register('hiddendeprecated', DeprecatedBzrDirFormat,
+            'Old format.  Slower and does not support things. ', hidden=True)
+        my_format_registry.register_lazy('hiddenlazy', 'bzrlib.tests.test_bzrdir',
+            'DeprecatedBzrDirFormat', 'Format registered lazily',
+            deprecated=True, hidden=True)
         return my_format_registry
 
     def test_format_registry(self):
         my_format_registry = self.make_format_registry()
         my_bzrdir = my_format_registry.make_bzrdir('lazy')
-        self.assertIsInstance(my_bzrdir, bzrdir.BzrDirFormat6)
-        my_bzrdir = my_format_registry.make_bzrdir('weave')
-        self.assertIsInstance(my_bzrdir, bzrdir.BzrDirFormat6)
+        self.assertIsInstance(my_bzrdir, DeprecatedBzrDirFormat)
+        my_bzrdir = my_format_registry.make_bzrdir('deprecated')
+        self.assertIsInstance(my_bzrdir, DeprecatedBzrDirFormat)
         my_bzrdir = my_format_registry.make_bzrdir('default')
         self.assertIsInstance(my_bzrdir.repository_format,
             knitrepo.RepositoryFormatKnit1)
@@ -136,9 +147,8 @@
                          my_format_registry.get_help('knit'))
         self.assertEqual('Format using knits',
                          my_format_registry.get_help('default'))
-        self.assertEqual('Pre-0.8 format.  Slower and does not support'
-                         ' checkouts or shared repositories',
-                         my_format_registry.get_help('weave'))
+        self.assertEqual('Some format.  Slower and unawesome and deprecated.',
+                         my_format_registry.get_help('deprecated'))
 
     def test_help_topic(self):
         topics = help_topics.HelpTopicRegistry()
@@ -168,20 +178,20 @@
             self.assertIs(bzrdir.format_registry.get('dirstate-with-subtree'),
                           bzrdir.format_registry.get('default'))
             self.assertIs(
-                repository.RepositoryFormat.get_default_format().__class__,
+                repository.format_registry.get_default().__class__,
                 knitrepo.RepositoryFormatKnit3)
         finally:
             bzrdir.format_registry.set_default_repository(old_default)
 
     def test_aliases(self):
-        a_registry = bzrdir.BzrDirFormatRegistry()
-        a_registry.register('weave', bzrdir.BzrDirFormat6,
-            'Pre-0.8 format.  Slower and does not support checkouts or shared'
-            ' repositories', deprecated=True)
-        a_registry.register('weavealias', bzrdir.BzrDirFormat6,
-            'Pre-0.8 format.  Slower and does not support checkouts or shared'
-            ' repositories', deprecated=True, alias=True)
-        self.assertEqual(frozenset(['weavealias']), a_registry.aliases())
+        a_registry = controldir.ControlDirFormatRegistry()
+        a_registry.register('deprecated', DeprecatedBzrDirFormat,
+            'Old format.  Slower and does not support stuff',
+            deprecated=True)
+        a_registry.register('deprecatedalias', DeprecatedBzrDirFormat,
+            'Old format.  Slower and does not support stuff',
+            deprecated=True, alias=True)
+        self.assertEqual(frozenset(['deprecatedalias']), a_registry.aliases())
 
 
 class SampleBranch(bzrlib.branch.Branch):
@@ -244,35 +254,56 @@
         return "opened branch."
 
 
+class BzrDirFormatTest1(bzrdir.BzrDirMetaFormat1):
+
+    @staticmethod
+    def get_format_string():
+        return "Test format 1"
+
+
+class BzrDirFormatTest2(bzrdir.BzrDirMetaFormat1):
+
+    @staticmethod
+    def get_format_string():
+        return "Test format 2"
+
+
 class TestBzrDirFormat(TestCaseWithTransport):
     """Tests for the BzrDirFormat facility."""
 
     def test_find_format(self):
         # is the right format object found for a branch?
         # create a branch with a few known format objects.
-        # this is not quite the same as
-        t = get_transport(self.get_url())
+        bzrdir.BzrProber.formats.register(BzrDirFormatTest1.get_format_string(),
+            BzrDirFormatTest1())
+        self.addCleanup(bzrdir.BzrProber.formats.remove,
+            BzrDirFormatTest1.get_format_string())
+        bzrdir.BzrProber.formats.register(BzrDirFormatTest2.get_format_string(),
+            BzrDirFormatTest2())
+        self.addCleanup(bzrdir.BzrProber.formats.remove,
+            BzrDirFormatTest2.get_format_string())
+        t = self.get_transport()
         self.build_tree(["foo/", "bar/"], transport=t)
         def check_format(format, url):
             format.initialize(url)
-            t = get_transport(url)
+            t = _mod_transport.get_transport_from_path(url)
             found_format = bzrdir.BzrDirFormat.find_format(t)
-            self.failUnless(isinstance(found_format, format.__class__))
-        check_format(bzrdir.BzrDirFormat5(), "foo")
-        check_format(bzrdir.BzrDirFormat6(), "bar")
+            self.assertIsInstance(found_format, format.__class__)
+        check_format(BzrDirFormatTest1(), "foo")
+        check_format(BzrDirFormatTest2(), "bar")
 
     def test_find_format_nothing_there(self):
         self.assertRaises(NotBranchError,
                           bzrdir.BzrDirFormat.find_format,
-                          get_transport('.'))
+                          _mod_transport.get_transport_from_path('.'))
 
     def test_find_format_unknown_format(self):
-        t = get_transport(self.get_url())
+        t = self.get_transport()
         t.mkdir('.bzr')
         t.put_bytes('.bzr/branch-format', '')
         self.assertRaises(UnknownFormatError,
                           bzrdir.BzrDirFormat.find_format,
-                          get_transport('.'))
+                          _mod_transport.get_transport_from_path('.'))
 
     def test_register_unregister_format(self):
         format = SampleBzrDirFormat()
@@ -280,16 +311,16 @@
         # make a bzrdir
         format.initialize(url)
         # register a format for it.
-        bzrdir.BzrDirFormat.register_format(format)
+        bzrdir.BzrProber.formats.register(format.get_format_string(), format)
         # which bzrdir.Open will refuse (not supported)
         self.assertRaises(UnsupportedFormatError, bzrdir.BzrDir.open, url)
         # which bzrdir.open_containing will refuse (not supported)
         self.assertRaises(UnsupportedFormatError, bzrdir.BzrDir.open_containing, url)
         # but open_downlevel will work
-        t = get_transport(url)
+        t = _mod_transport.get_transport_from_url(url)
         self.assertEqual(format.open(t), bzrdir.BzrDir.open_unsupported(url))
         # unregister the format
-        bzrdir.BzrDirFormat.unregister_format(format)
+        bzrdir.BzrProber.formats.remove(format.get_format_string())
         # now open_downlevel should fail too.
         self.assertRaises(UnknownFormatError, bzrdir.BzrDir.open_unsupported, url)
 
@@ -472,7 +503,7 @@
     def test_default_stacking_with_stackable_branch_unstackable_repo(self):
         # Make stackable source branch with an unstackable repo format.
         source_bzrdir = self.make_bzrdir('source')
-        pack_repo.RepositoryFormatKnitPack1().initialize(source_bzrdir)
+        knitpack_repo.RepositoryFormatKnitPack1().initialize(source_bzrdir)
         source_branch = bzrlib.branch.BzrBranchFormat7().initialize(
             source_bzrdir)
         # Make a directory with a default stacking policy
@@ -482,6 +513,35 @@
         # Clone source into directory
         target = source_bzrdir.clone(self.get_url('parent/target'))
 
+    def test_format_initialize_on_transport_ex_stacked_on(self):
+        # trunk is a stackable format.  Note that its in the same server area
+        # which is what launchpad does, but not sufficient to exercise the
+        # general case.
+        trunk = self.make_branch('trunk', format='1.9')
+        t = self.get_transport('stacked')
+        old_fmt = bzrdir.format_registry.make_bzrdir('pack-0.92')
+        repo_name = old_fmt.repository_format.network_name()
+        # Should end up with a 1.9 format (stackable)
+        repo, control, require_stacking, repo_policy = \
+            old_fmt.initialize_on_transport_ex(t,
+                    repo_format_name=repo_name, stacked_on='../trunk',
+                    stack_on_pwd=t.base)
+        if repo is not None:
+            # Repositories are open write-locked
+            self.assertTrue(repo.is_write_locked())
+            self.addCleanup(repo.unlock)
+        else:
+            repo = control.open_repository()
+        self.assertIsInstance(control, bzrdir.BzrDir)
+        opened = bzrdir.BzrDir.open(t.base)
+        if not isinstance(old_fmt, remote.RemoteBzrDirFormat):
+            self.assertEqual(control._format.network_name(),
+                old_fmt.network_name())
+            self.assertEqual(control._format.network_name(),
+                opened._format.network_name())
+        self.assertEqual(control.__class__, opened.__class__)
+        self.assertLength(1, repo._fallback_repositories)
+
     def test_sprout_obeys_stacking_policy(self):
         child_branch, new_child_transport = self.prepare_default_stacking()
         new_child = child_branch.bzrdir.sprout(new_child_transport.base)
@@ -678,16 +738,21 @@
         self.assertEqual(relpath, 'baz')
 
     def test_open_containing_from_transport(self):
-        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing_from_transport,
-                          get_transport(self.get_readonly_url('')))
-        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing_from_transport,
-                          get_transport(self.get_readonly_url('g/p/q')))
+        self.assertRaises(NotBranchError,
+            bzrdir.BzrDir.open_containing_from_transport,
+            _mod_transport.get_transport_from_url(self.get_readonly_url('')))
+        self.assertRaises(NotBranchError,
+            bzrdir.BzrDir.open_containing_from_transport,
+            _mod_transport.get_transport_from_url(
+                self.get_readonly_url('g/p/q')))
         control = bzrdir.BzrDir.create(self.get_url())
         branch, relpath = bzrdir.BzrDir.open_containing_from_transport(
-            get_transport(self.get_readonly_url('')))
+            _mod_transport.get_transport_from_url(
+                self.get_readonly_url('')))
         self.assertEqual('', relpath)
         branch, relpath = bzrdir.BzrDir.open_containing_from_transport(
-            get_transport(self.get_readonly_url('g/p/q')))
+            _mod_transport.get_transport_from_url(
+                self.get_readonly_url('g/p/q')))
         self.assertEqual('g/p/q', relpath)
 
     def test_open_containing_tree_or_branch(self):
@@ -737,23 +802,21 @@
         # transport pointing at bzrdir should give a bzrdir with root transport
         # set to the given transport
         control = bzrdir.BzrDir.create(self.get_url())
-        transport = get_transport(self.get_url())
-        opened_bzrdir = bzrdir.BzrDir.open_from_transport(transport)
-        self.assertEqual(transport.base, opened_bzrdir.root_transport.base)
+        t = self.get_transport()
+        opened_bzrdir = bzrdir.BzrDir.open_from_transport(t)
+        self.assertEqual(t.base, opened_bzrdir.root_transport.base)
         self.assertIsInstance(opened_bzrdir, bzrdir.BzrDir)
 
     def test_open_from_transport_no_bzrdir(self):
-        transport = get_transport(self.get_url())
-        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport,
-                          transport)
+        t = self.get_transport()
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport, t)
 
     def test_open_from_transport_bzrdir_in_parent(self):
         control = bzrdir.BzrDir.create(self.get_url())
-        transport = get_transport(self.get_url())
-        transport.mkdir('subdir')
-        transport = transport.clone('subdir')
-        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport,
-                          transport)
+        t = self.get_transport()
+        t.mkdir('subdir')
+        t = t.clone('subdir')
+        self.assertRaises(NotBranchError, bzrdir.BzrDir.open_from_transport, t)
 
     def test_sprout_recursive(self):
         tree = self.make_branch_and_tree('tree1',
@@ -768,7 +831,7 @@
         tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
         tree2.lock_read()
         self.addCleanup(tree2.unlock)
-        self.failUnlessExists('tree2/subtree/file')
+        self.assertPathExists('tree2/subtree/file')
         self.assertEqual('tree-reference', tree2.kind('subtree-root'))
 
     def test_cloning_metadir(self):
@@ -778,7 +841,7 @@
         branch = self.make_branch('branch', format='knit')
         format = branch.bzrdir.cloning_metadir()
         self.assertIsInstance(format.workingtree_format,
-            workingtree.WorkingTreeFormat3)
+            workingtree_4.WorkingTreeFormat6)
 
     def test_sprout_recursive_treeless(self):
         tree = self.make_branch_and_tree('tree1',
@@ -789,13 +852,27 @@
         self.build_tree(['tree1/subtree/file'])
         sub_tree.add('file')
         tree.commit('Initial commit')
+        # The following line force the orhaning to reveal bug #634470
+        tree.branch.get_config().set_user_option(
+            'bzr.transform.orphan_policy', 'move')
         tree.bzrdir.destroy_workingtree()
+        # FIXME: subtree/.bzr is left here which allows the test to pass (or
+        # fail :-( ) -- vila 20100909
         repo = self.make_repository('repo', shared=True,
             format='dirstate-with-subtree')
         repo.set_make_working_trees(False)
-        tree.bzrdir.sprout('repo/tree2')
-        self.failUnlessExists('repo/tree2/subtree')
-        self.failIfExists('repo/tree2/subtree/file')
+        # FIXME: we just deleted the workingtree and now we want to use it ????
+        # At a minimum, we should use tree.branch below (but this fails too
+        # currently) or stop calling this test 'treeless'. Specifically, I've
+        # turn the line below into an assertRaises when 'subtree/.bzr' is
+        # orphaned and sprout tries to access the branch there (which is left
+        # by bzrdir.BzrDirMeta1.destroy_workingtree when it ignores the
+        # [DeletingParent('Not deleting', u'subtree', None)] conflict). See bug
+        # #634470.  -- vila 20100909
+        self.assertRaises(errors.NotBranchError,
+                          tree.bzrdir.sprout, 'repo/tree2')
+#        self.assertPathExists('repo/tree2/subtree')
+#        self.assertPathDoesNotExist('repo/tree2/subtree/file')
 
     def make_foo_bar_baz(self):
         foo = bzrdir.BzrDir.create_branch_convenience('foo').bzrdir
@@ -805,9 +882,8 @@
 
     def test_find_bzrdirs(self):
         foo, bar, baz = self.make_foo_bar_baz()
-        transport = get_transport(self.get_url())
-        self.assertEqualBzrdirs([baz, foo, bar],
-                                bzrdir.BzrDir.find_bzrdirs(transport))
+        t = self.get_transport()
+        self.assertEqualBzrdirs([baz, foo, bar], bzrdir.BzrDir.find_bzrdirs(t))
 
     def make_fake_permission_denied_transport(self, transport, paths):
         """Create a transport that raises PermissionDenied for some paths."""
@@ -829,9 +905,9 @@
 
     def test_find_bzrdirs_permission_denied(self):
         foo, bar, baz = self.make_foo_bar_baz()
-        transport = get_transport(self.get_url())
+        t = self.get_transport()
         path_filter_server, path_filter_transport = \
-            self.make_fake_permission_denied_transport(transport, ['foo'])
+            self.make_fake_permission_denied_transport(t, ['foo'])
         # local transport
         self.assertBranchUrlsEndWith('/baz/',
             bzrdir.BzrDir.find_bzrdirs(path_filter_transport))
@@ -846,25 +922,24 @@
             return [s for s in transport.list_dir('') if s != 'baz']
 
         foo, bar, baz = self.make_foo_bar_baz()
-        transport = get_transport(self.get_url())
-        self.assertEqualBzrdirs([foo, bar],
-                                bzrdir.BzrDir.find_bzrdirs(transport,
-                                    list_current=list_current))
+        t = self.get_transport()
+        self.assertEqualBzrdirs(
+            [foo, bar],
+            bzrdir.BzrDir.find_bzrdirs(t, list_current=list_current))
 
     def test_find_bzrdirs_evaluate(self):
         def evaluate(bzrdir):
             try:
                 repo = bzrdir.open_repository()
-            except NoRepositoryPresent:
+            except errors.NoRepositoryPresent:
                 return True, bzrdir.root_transport.base
             else:
                 return False, bzrdir.root_transport.base
 
         foo, bar, baz = self.make_foo_bar_baz()
-        transport = get_transport(self.get_url())
+        t = self.get_transport()
         self.assertEqual([baz.root_transport.base, foo.root_transport.base],
-                         list(bzrdir.BzrDir.find_bzrdirs(transport,
-                                                         evaluate=evaluate)))
+                         list(bzrdir.BzrDir.find_bzrdirs(t, evaluate=evaluate)))
 
     def assertEqualBzrdirs(self, first, second):
         first = list(first)
@@ -877,14 +952,14 @@
         root = self.make_repository('', shared=True)
         foo, bar, baz = self.make_foo_bar_baz()
         qux = self.make_bzrdir('foo/qux')
-        transport = get_transport(self.get_url())
-        branches = bzrdir.BzrDir.find_branches(transport)
+        t = self.get_transport()
+        branches = bzrdir.BzrDir.find_branches(t)
         self.assertEqual(baz.root_transport.base, branches[0].base)
         self.assertEqual(foo.root_transport.base, branches[1].base)
         self.assertEqual(bar.root_transport.base, branches[2].base)
 
         # ensure this works without a top-level repo
-        branches = bzrdir.BzrDir.find_branches(transport.clone('foo'))
+        branches = bzrdir.BzrDir.find_branches(t.clone('foo'))
         self.assertEqual(foo.root_transport.base, branches[0].base)
         self.assertEqual(bar.root_transport.base, branches[1].base)
 
@@ -892,15 +967,15 @@
 class TestMissingRepoBranchesSkipped(TestCaseWithMemoryTransport):
 
     def test_find_bzrdirs_missing_repo(self):
-        transport = get_transport(self.get_url())
+        t = self.get_transport()
         arepo = self.make_repository('arepo', shared=True)
         abranch_url = arepo.user_url + '/abranch'
         abranch = bzrdir.BzrDir.create(abranch_url).create_branch()
-        transport.delete_tree('arepo/.bzr')
+        t.delete_tree('arepo/.bzr')
         self.assertRaises(errors.NoRepositoryPresent,
             branch.Branch.open, abranch_url)
         self.make_branch('baz')
-        for actual_bzrdir in bzrdir.BzrDir.find_branches(transport):
+        for actual_bzrdir in bzrdir.BzrDir.find_branches(t):
             self.assertEndsWith(actual_bzrdir.user_url, '/baz/')
 
 
@@ -916,12 +991,13 @@
                          dir.get_branch_transport(bzrlib.branch.BzrBranchFormat5()).base)
         repository_base = t.clone('repository').base
         self.assertEqual(repository_base, dir.get_repository_transport(None).base)
+        repository_format = repository.format_registry.get_default()
         self.assertEqual(repository_base,
-                         dir.get_repository_transport(weaverepo.RepositoryFormat7()).base)
+                         dir.get_repository_transport(repository_format).base)
         checkout_base = t.clone('checkout').base
         self.assertEqual(checkout_base, dir.get_workingtree_transport(None).base)
         self.assertEqual(checkout_base,
-                         dir.get_workingtree_transport(workingtree.WorkingTreeFormat3()).base)
+                         dir.get_workingtree_transport(workingtree_3.WorkingTreeFormat3()).base)
 
     def test_meta1dir_uses_lockdir(self):
         """Meta1 format uses a LockDir to guard the whole directory, not a file."""
@@ -969,135 +1045,6 @@
         self.assertEqual(2, rpc_count)
 
 
-class TestFormat5(TestCaseWithTransport):
-    """Tests specific to the version 5 bzrdir format."""
-
-    def test_same_lockfiles_between_tree_repo_branch(self):
-        # this checks that only a single lockfiles instance is created
-        # for format 5 objects
-        dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
-        def check_dir_components_use_same_lock(dir):
-            ctrl_1 = dir.open_repository().control_files
-            ctrl_2 = dir.open_branch().control_files
-            ctrl_3 = dir.open_workingtree()._control_files
-            self.assertTrue(ctrl_1 is ctrl_2)
-            self.assertTrue(ctrl_2 is ctrl_3)
-        check_dir_components_use_same_lock(dir)
-        # and if we open it normally.
-        dir = bzrdir.BzrDir.open(self.get_url())
-        check_dir_components_use_same_lock(dir)
-
-    def test_can_convert(self):
-        # format 5 dirs are convertable
-        dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
-        self.assertTrue(dir.can_convert_format())
-
-    def test_needs_conversion(self):
-        # format 5 dirs need a conversion if they are not the default,
-        # and they aren't
-        dir = bzrdir.BzrDirFormat5().initialize(self.get_url())
-        # don't need to convert it to itself
-        self.assertFalse(dir.needs_format_conversion(bzrdir.BzrDirFormat5()))
-        # do need to convert it to the current default
-        self.assertTrue(dir.needs_format_conversion(
-            bzrdir.BzrDirFormat.get_default_format()))
-
-
-class TestFormat6(TestCaseWithTransport):
-    """Tests specific to the version 6 bzrdir format."""
-
-    def test_same_lockfiles_between_tree_repo_branch(self):
-        # this checks that only a single lockfiles instance is created
-        # for format 6 objects
-        dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        def check_dir_components_use_same_lock(dir):
-            ctrl_1 = dir.open_repository().control_files
-            ctrl_2 = dir.open_branch().control_files
-            ctrl_3 = dir.open_workingtree()._control_files
-            self.assertTrue(ctrl_1 is ctrl_2)
-            self.assertTrue(ctrl_2 is ctrl_3)
-        check_dir_components_use_same_lock(dir)
-        # and if we open it normally.
-        dir = bzrdir.BzrDir.open(self.get_url())
-        check_dir_components_use_same_lock(dir)
-
-    def test_can_convert(self):
-        # format 6 dirs are convertable
-        dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        self.assertTrue(dir.can_convert_format())
-
-    def test_needs_conversion(self):
-        # format 6 dirs need an conversion if they are not the default.
-        dir = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        self.assertTrue(dir.needs_format_conversion(
-            bzrdir.BzrDirFormat.get_default_format()))
-
-
-class NotBzrDir(bzrlib.bzrdir.BzrDir):
-    """A non .bzr based control directory."""
-
-    def __init__(self, transport, format):
-        self._format = format
-        self.root_transport = transport
-        self.transport = transport.clone('.not')
-
-
-class NotBzrDirFormat(bzrlib.bzrdir.BzrDirFormat):
-    """A test class representing any non-.bzr based disk format."""
-
-    def initialize_on_transport(self, transport):
-        """Initialize a new .not dir in the base directory of a Transport."""
-        transport.mkdir('.not')
-        return self.open(transport)
-
-    def open(self, transport):
-        """Open this directory."""
-        return NotBzrDir(transport, self)
-
-    @classmethod
-    def _known_formats(self):
-        return set([NotBzrDirFormat()])
-
-    @classmethod
-    def probe_transport(self, transport):
-        """Our format is present if the transport ends in '.not/'."""
-        if transport.has('.not'):
-            return NotBzrDirFormat()
-
-
-class TestNotBzrDir(TestCaseWithTransport):
-    """Tests for using the bzrdir api with a non .bzr based disk format.
-
-    If/when one of these is in the core, we can let the implementation tests
-    verify this works.
-    """
-
-    def test_create_and_find_format(self):
-        # create a .notbzr dir
-        format = NotBzrDirFormat()
-        dir = format.initialize(self.get_url())
-        self.assertIsInstance(dir, NotBzrDir)
-        # now probe for it.
-        bzrlib.bzrdir.BzrDirFormat.register_control_format(format)
-        try:
-            found = bzrlib.bzrdir.BzrDirFormat.find_format(
-                get_transport(self.get_url()))
-            self.assertIsInstance(found, NotBzrDirFormat)
-        finally:
-            bzrlib.bzrdir.BzrDirFormat.unregister_control_format(format)
-
-    def test_included_in_known_formats(self):
-        bzrlib.bzrdir.BzrDirFormat.register_control_format(NotBzrDirFormat)
-        try:
-            formats = bzrlib.bzrdir.BzrDirFormat.known_formats()
-            for format in formats:
-                if isinstance(format, NotBzrDirFormat):
-                    return
-            self.fail("No NotBzrDirFormat in %s" % formats)
-        finally:
-            bzrlib.bzrdir.BzrDirFormat.unregister_control_format(NotBzrDirFormat)
-
-
 class NonLocalTests(TestCaseWithTransport):
     """Tests for bzrdir static behaviour on non local paths."""
 
@@ -1122,7 +1069,7 @@
             self.get_url('foo'),
             force_new_tree=True,
             format=format)
-        t = get_transport(self.get_url('.'))
+        t = self.get_transport()
         self.assertFalse(t.has('foo'))
 
     def test_clone(self):
@@ -1144,7 +1091,7 @@
         my_bzrdir = bzrdir.BzrDir.open(self.get_url('branch-knit2'))
         checkout_format = my_bzrdir.checkout_metadir()
         self.assertIsInstance(checkout_format.workingtree_format,
-                              workingtree.WorkingTreeFormat3)
+                              workingtree_4.WorkingTreeFormat4)
 
 
 class TestHTTPRedirections(object):
@@ -1294,7 +1241,7 @@
 
     def __init__(self, *args, **kwargs):
         super(_TestBzrDir, self).__init__(*args, **kwargs)
-        self.test_branch = _TestBranch()
+        self.test_branch = _TestBranch(self.transport)
         self.test_branch.repository = self.create_repository()
 
     def open_branch(self, unsupported=False):
@@ -1311,25 +1258,39 @@
 class _TestBranch(bzrlib.branch.Branch):
     """Test Branch implementation for TestBzrDirSprout."""
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, transport, *args, **kwargs):
         self._format = _TestBranchFormat()
+        self._transport = transport
+        self.base = transport.base
         super(_TestBranch, self).__init__(*args, **kwargs)
         self.calls = []
         self._parent = None
 
     def sprout(self, *args, **kwargs):
         self.calls.append('sprout')
-        return _TestBranch()
+        return _TestBranch(self._transport)
 
     def copy_content_into(self, destination, revision_id=None):
         self.calls.append('copy_content_into')
 
+    def last_revision(self):
+        return _mod_revision.NULL_REVISION
+
     def get_parent(self):
         return self._parent
 
+    def _get_config(self):
+        return config.TransportConfig(self._transport, 'branch.conf')
+
     def set_parent(self, parent):
         self._parent = parent
 
+    def lock_read(self):
+        return lock.LogicalLockResult(self.unlock)
+
+    def unlock(self):
+        return
+
 
 class TestBzrDirSprout(TestCaseWithMemoryTransport):
 
@@ -1391,7 +1352,7 @@
         self.assertEqual('fail', err._preformatted_string)
 
     def test_post_repo_init(self):
-        from bzrlib.bzrdir import RepoInitHookParams
+        from bzrlib.controldir import RepoInitHookParams
         calls = []
         bzrdir.BzrDir.hooks.install_named_hook('post_repo_init',
             calls.append, None)
@@ -1401,3 +1362,71 @@
         self.assertIsInstance(params, RepoInitHookParams)
         self.assertTrue(hasattr(params, 'bzrdir'))
         self.assertTrue(hasattr(params, 'repository'))
+
+    def test_post_repo_init_hook_repr(self):
+        param_reprs = []
+        bzrdir.BzrDir.hooks.install_named_hook('post_repo_init',
+            lambda params: param_reprs.append(repr(params)), None)
+        self.make_repository('foo')
+        self.assertLength(1, param_reprs)
+        param_repr = param_reprs[0]
+        self.assertStartsWith(param_repr, '<RepoInitHookParams for ')
+
+
+class TestGenerateBackupName(TestCaseWithMemoryTransport):
+    # FIXME: This may need to be unified with test_osutils.TestBackupNames or
+    # moved to per_bzrdir or per_transport for better coverage ?
+    # -- vila 20100909
+
+    def setUp(self):
+        super(TestGenerateBackupName, self).setUp()
+        self._transport = self.get_transport()
+        bzrdir.BzrDir.create(self.get_url(),
+            possible_transports=[self._transport])
+        self._bzrdir = bzrdir.BzrDir.open_from_transport(self._transport)
+
+    def test_deprecated_generate_backup_name(self):
+        res = self.applyDeprecated(
+                symbol_versioning.deprecated_in((2, 3, 0)),
+                self._bzrdir.generate_backup_name, 'whatever')
+
+    def test_new(self):
+        self.assertEqual("a.~1~", self._bzrdir._available_backup_name("a"))
+
+    def test_exiting(self):
+        self._transport.put_bytes("a.~1~", "some content")
+        self.assertEqual("a.~2~", self._bzrdir._available_backup_name("a"))
+
+
+class TestMeta1DirColoFormat(TestCaseWithTransport):
+    """Tests specific to the meta1 dir with colocated branches format."""
+
+    def test_supports_colo(self):
+        format = bzrdir.BzrDirMetaFormat1Colo()
+        self.assertTrue(format.colocated_branches)
+
+    def test_upgrade_from_2a(self):
+        tree = self.make_branch_and_tree('.', format='2a')
+        format = bzrdir.BzrDirMetaFormat1Colo()
+        self.assertTrue(tree.bzrdir.needs_format_conversion(format))
+        converter = tree.bzrdir._format.get_converter(format)
+        result = converter.convert(tree.bzrdir, None)
+        self.assertIsInstance(result._format, bzrdir.BzrDirMetaFormat1Colo)
+        self.assertFalse(result.needs_format_conversion(format))
+
+    def test_downgrade_to_2a(self):
+        tree = self.make_branch_and_tree('.', format='development-colo')
+        format = bzrdir.BzrDirMetaFormat1()
+        self.assertTrue(tree.bzrdir.needs_format_conversion(format))
+        converter = tree.bzrdir._format.get_converter(format)
+        result = converter.convert(tree.bzrdir, None)
+        self.assertIsInstance(result._format, bzrdir.BzrDirMetaFormat1)
+        self.assertFalse(result.needs_format_conversion(format))
+
+    def test_downgrade_to_2a_too_many_branches(self):
+        tree = self.make_branch_and_tree('.', format='development-colo')
+        tree.bzrdir.create_branch(name="another-colocated-branch")
+        converter = tree.bzrdir._format.get_converter(
+            bzrdir.BzrDirMetaFormat1())
+        self.assertRaises(errors.BzrError, converter.convert, tree.bzrdir,
+            None)

=== modified file 'bzrlib/tests/test_cache_utf8.py'
--- bzrlib/tests/test_cache_utf8.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_cache_utf8.py	2011-05-13 12:51:05 +0000
@@ -31,37 +31,37 @@
 
     def check_encode(self, rev_id):
         rev_id_utf8 = rev_id.encode('utf-8')
-        self.failIf(rev_id in cache_utf8._unicode_to_utf8_map)
-        self.failIf(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
+        self.assertFalse(rev_id in cache_utf8._unicode_to_utf8_map)
+        self.assertFalse(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
 
         # After a single encode, the mapping should exist for
         # both directions
         self.assertEqual(rev_id_utf8, cache_utf8.encode(rev_id))
-        self.failUnless(rev_id in cache_utf8._unicode_to_utf8_map)
-        self.failUnless(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
+        self.assertTrue(rev_id in cache_utf8._unicode_to_utf8_map)
+        self.assertTrue(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
 
         self.assertEqual(rev_id, cache_utf8.decode(rev_id_utf8))
 
         cache_utf8.clear_encoding_cache()
-        self.failIf(rev_id in cache_utf8._unicode_to_utf8_map)
-        self.failIf(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
+        self.assertFalse(rev_id in cache_utf8._unicode_to_utf8_map)
+        self.assertFalse(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
 
     def check_decode(self, rev_id):
         rev_id_utf8 = rev_id.encode('utf-8')
-        self.failIf(rev_id in cache_utf8._unicode_to_utf8_map)
-        self.failIf(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
+        self.assertFalse(rev_id in cache_utf8._unicode_to_utf8_map)
+        self.assertFalse(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
 
         # After a single decode, the mapping should exist for
         # both directions
         self.assertEqual(rev_id, cache_utf8.decode(rev_id_utf8))
-        self.failUnless(rev_id in cache_utf8._unicode_to_utf8_map)
-        self.failUnless(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
+        self.assertTrue(rev_id in cache_utf8._unicode_to_utf8_map)
+        self.assertTrue(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
 
         self.assertEqual(rev_id_utf8, cache_utf8.encode(rev_id))
         cache_utf8.clear_encoding_cache()
 
-        self.failIf(rev_id in cache_utf8._unicode_to_utf8_map)
-        self.failIf(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
+        self.assertFalse(rev_id in cache_utf8._unicode_to_utf8_map)
+        self.assertFalse(rev_id_utf8 in cache_utf8._utf8_to_unicode_map)
 
     def test_ascii(self):
         self.check_decode(u'all_ascii_characters123123123')
@@ -74,7 +74,7 @@
     def test_cached_unicode(self):
         x = u'\xb5yy' + u'\xe5zz'
         y = u'\xb5yy' + u'\xe5zz'
-        self.failIf(x is y)
+        self.assertFalse(x is y)
         xp = cache_utf8.get_cached_unicode(x)
         yp = cache_utf8.get_cached_unicode(y)
 
@@ -84,7 +84,7 @@
     def test_cached_utf8(self):
         x = u'\xb5yy\xe5zz'.encode('utf8')
         y = u'\xb5yy\xe5zz'.encode('utf8')
-        self.failIf(x is y)
+        self.assertFalse(x is y)
         xp = cache_utf8.get_cached_utf8(x)
         yp = cache_utf8.get_cached_utf8(y)
 
@@ -94,7 +94,7 @@
     def test_cached_ascii(self):
         x = '%s %s' % ('simple', 'text')
         y = '%s %s' % ('simple', 'text')
-        self.failIf(x is y)
+        self.assertFalse(x is y)
         xp = cache_utf8.get_cached_ascii(x)
         yp = cache_utf8.get_cached_ascii(y)
 

=== added file 'bzrlib/tests/test_cethread.py'
--- bzrlib/tests/test_cethread.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_cethread.py	2011-02-11 16:15:39 +0000
@@ -0,0 +1,161 @@
+# Copyright (C) 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import threading
+
+from bzrlib import (
+    cethread,
+    tests,
+    )
+
+
+class TestCatchingExceptionThread(tests.TestCase):
+
+    def test_start_and_join_smoke_test(self):
+        def do_nothing():
+            pass
+
+        tt = cethread.CatchingExceptionThread(target=do_nothing)
+        tt.start()
+        tt.join()
+
+    def test_exception_is_re_raised(self):
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            raise MyException()
+
+        tt = cethread.CatchingExceptionThread(target=raise_my_exception)
+        tt.start()
+        self.assertRaises(MyException, tt.join)
+
+    def test_join_around_exception(self):
+        resume = threading.Event()
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            # Wait for the test to tell us to resume
+            resume.wait()
+            # Now we can raise
+            raise MyException()
+
+        tt = cethread.CatchingExceptionThread(target=raise_my_exception)
+        tt.start()
+        tt.join(timeout=0)
+        self.assertIs(None, tt.exception)
+        resume.set()
+        self.assertRaises(MyException, tt.join)
+
+    def test_sync_event(self):
+        control = threading.Event()
+        in_thread = threading.Event()
+        class MyException(Exception):
+            pass
+
+        def raise_my_exception():
+            # Wait for the test to tell us to resume
+            control.wait()
+            # Now we can raise
+            raise MyException()
+
+        tt = cethread.CatchingExceptionThread(target=raise_my_exception,
+                                            sync_event=in_thread)
+        tt.start()
+        tt.join(timeout=0)
+        self.assertIs(None, tt.exception)
+        self.assertIs(in_thread, tt.sync_event)
+        control.set()
+        self.assertRaises(MyException, tt.join)
+        self.assertEquals(True, tt.sync_event.isSet())
+
+    def test_switch_and_set(self):
+        """Caller can precisely control a thread."""
+        control1 = threading.Event()
+        control2 = threading.Event()
+        control3 = threading.Event()
+
+        class TestThread(cethread.CatchingExceptionThread):
+
+            def __init__(self):
+                super(TestThread, self).__init__(target=self.step_by_step)
+                self.current_step = 'starting'
+                self.step1 = threading.Event()
+                self.set_sync_event(self.step1)
+                self.step2 = threading.Event()
+                self.final = threading.Event()
+
+            def step_by_step(self):
+                control1.wait()
+                self.current_step = 'step1'
+                self.switch_and_set(self.step2)
+                control2.wait()
+                self.current_step = 'step2'
+                self.switch_and_set(self.final)
+                control3.wait()
+                self.current_step = 'done'
+
+        tt = TestThread()
+        tt.start()
+        self.assertEquals('starting', tt.current_step)
+        control1.set()
+        tt.step1.wait()
+        self.assertEquals('step1', tt.current_step)
+        control2.set()
+        tt.step2.wait()
+        self.assertEquals('step2', tt.current_step)
+        control3.set()
+        # We don't wait on tt.final
+        tt.join()
+        self.assertEquals('done', tt.current_step)
+
+    def test_exception_while_switch_and_set(self):
+        control1 = threading.Event()
+
+        class MyException(Exception):
+            pass
+
+        class TestThread(cethread.CatchingExceptionThread):
+
+            def __init__(self, *args, **kwargs):
+                self.step1 = threading.Event()
+                self.step2 = threading.Event()
+                super(TestThread, self).__init__(target=self.step_by_step,
+                                                 sync_event=self.step1)
+                self.current_step = 'starting'
+                self.set_sync_event(self.step1)
+
+            def step_by_step(self):
+                control1.wait()
+                self.current_step = 'step1'
+                self.switch_and_set(self.step2)
+
+            def set_sync_event(self, event):
+                # We force an exception while trying to set step2
+                if event is self.step2:
+                    raise MyException()
+                super(TestThread, self).set_sync_event(event)
+
+        tt = TestThread()
+        tt.start()
+        self.assertEquals('starting', tt.current_step)
+        control1.set()
+        # We now wait on step1 which will be set when catching the exception
+        tt.step1.wait()
+        self.assertRaises(MyException, tt.pending_exception)
+        self.assertIs(tt.step1, tt.sync_event)
+        self.assertTrue(tt.step1.isSet())

=== modified file 'bzrlib/tests/test_chk_map.py'
--- bzrlib/tests/test_chk_map.py	2010-07-21 09:58:42 +0000
+++ bzrlib/tests/test_chk_map.py	2011-09-16 20:42:31 +0000
@@ -16,8 +16,6 @@
 
 """Tests for maps built on a CHK versionedfiles facility."""
 
-from itertools import izip
-
 from bzrlib import (
     chk_map,
     errors,
@@ -226,7 +224,7 @@
             "      ('ddd',) 'initial ddd content'\n",
             c_map._dump_tree())
 
-    def test_one_deep_map_16(self):
+    def test_root_only_aaa_ddd_16(self):
         c_map = self.make_root_only_aaa_ddd_map(
                 search_key_func=chk_map._search_key_16)
         # We use 'aaa' and 'ddd' because they happen to map to 'F' when using
@@ -1110,7 +1108,7 @@
         basis_get = basis._store.get_record_stream
         def get_record_stream(keys, order, fulltext):
             if ('sha1:1adf7c0d1b9140ab5f33bb64c6275fa78b1580b7',) in keys:
-                self.fail("'aaa' pointer was followed %r" % keys)
+                raise AssertionError("'aaa' pointer was followed %r" % keys)
             return basis_get(keys, order, fulltext)
         basis._store.get_record_stream = get_record_stream
         result = sorted(list(target.iter_changes(basis)))

=== modified file 'bzrlib/tests/test_chk_serializer.py'
--- bzrlib/tests/test_chk_serializer.py	2009-06-04 21:50:04 +0000
+++ bzrlib/tests/test_chk_serializer.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,13 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
-
-from bzrlib import (
-    errors,
-    revision,
-    serializer,
-    )
 from bzrlib.chk_serializer import (
     chk_bencode_serializer,
     )

=== modified file 'bzrlib/tests/test_clean_tree.py'
--- bzrlib/tests/test_clean_tree.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_clean_tree.py	2011-05-26 21:02:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2009 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,9 +15,13 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+import errno
 import os
-from StringIO import StringIO
+import shutil
+import sys
+import types
 
+from bzrlib import tests, ui
 from bzrlib.bzrdir import (
     BzrDir,
     )
@@ -42,12 +46,12 @@
         BzrDir.create_standalone_workingtree('branch')
         os.symlink(os.path.realpath('no-die-please'), 'branch/die-please')
         os.mkdir('no-die-please')
-        self.failUnlessExists('branch/die-please')
+        self.assertPathExists('branch/die-please')
         os.mkdir('no-die-please/child')
 
         clean_tree('branch', unknown=True, no_prompt=True)
-        self.failUnlessExists('no-die-please')
-        self.failUnlessExists('no-die-please/child')
+        self.assertPathExists('no-die-please')
+        self.assertPathExists('no-die-please/child')
 
     def test_iter_deletable(self):
         """Files are selected for deletion appropriately"""
@@ -76,3 +80,59 @@
             self.assertEqual([], dels)
         finally:
             tree.unlock()
+
+    def test_delete_items_warnings(self):
+        """Ensure delete_items issues warnings on EACCES. (bug #430785)
+        """
+        def _dummy_unlink(path):
+            """unlink() files other than files named '0foo'.
+            """
+            if path.endswith('0foo'):
+                # Simulate 'permission denied' error.
+                # This should show up as a warning for the
+                # user.
+                e = OSError()
+                e.errno = errno.EACCES
+                raise e
+
+        def _dummy_rmtree(path, ignore_errors=False, onerror=None):
+            """Call user supplied error handler onerror.
+            """
+            # Indicate failure in removing 'path' if path is subdir0
+            # We later check to ensure that this is indicated
+            # to the user as a warning. We raise OSError to construct
+            # proper excinfo that needs to be passed to onerror
+            try:
+                raise OSError
+            except OSError, e:
+                e.errno = errno.EACCES
+                excinfo = sys.exc_info()
+                function = os.remove
+                if 'subdir0' not in path:
+                    # onerror should show warning only for os.remove
+                    # error. For any other failures the error should
+                    # be shown to the user.
+                    function = os.listdir
+                onerror(function=function,
+                    path=path, excinfo=excinfo)
+
+        self.overrideAttr(os, 'unlink', _dummy_unlink)
+        self.overrideAttr(shutil, 'rmtree', _dummy_rmtree)
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stdout=stdout, stderr=stderr)
+
+        BzrDir.create_standalone_workingtree('.')
+        self.build_tree(['0foo', '1bar', '2baz', 'subdir0/'])
+        clean_tree('.', unknown=True, no_prompt=True)
+        self.assertContainsRe(stderr.getvalue(),
+            'bzr: warning: unable to remove.*0foo')
+        self.assertContainsRe(stderr.getvalue(),
+            'bzr: warning: unable to remove.*subdir0')
+
+        # Ensure that error other than EACCES during os.remove are
+        # not turned into warnings.
+        self.build_tree(['subdir1/'])
+        self.assertRaises(OSError, clean_tree, '.',
+            unknown=True, no_prompt=True)
+

=== modified file 'bzrlib/tests/test_cmdline.py'
--- bzrlib/tests/test_cmdline.py	2010-05-20 02:57:52 +0000
+++ bzrlib/tests/test_cmdline.py	2010-12-06 14:01:44 +0000
@@ -113,4 +113,3 @@
         self.assertAsTokens([(False, r'\\"'), (False, r'*.py')],
                             r'\\\\\" *.py')
         self.assertAsTokens([(True, u'\\\\')], u'"\\\\')
-

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2010-05-27 21:16:48 +0000
+++ bzrlib/tests/test_commands.py	2011-09-29 13:54:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 import errno
 import inspect
 import sys
@@ -92,12 +91,39 @@
         self.assertContainsRe(c.get_help_text(), '--foo')
 
 
+class TestInsideCommand(tests.TestCaseInTempDir):
+
+    def test_command_see_config_overrides(self):
+        def run(cmd):
+            # We override the run() command method so we can observe the
+            # overrides from inside.
+            c = config.GlobalStack()
+            self.assertEquals('12', c.get('xx'))
+            self.assertEquals('foo', c.get('yy'))
+        self.overrideAttr(builtins.cmd_rocks, 'run', run)
+        self.run_bzr(['rocks', '-Oxx=12', '-Oyy=foo'])
+        c = config.GlobalStack()
+        # Ensure that we don't leak outside of the command
+        self.assertEquals(None, c.get('xx'))
+        self.assertEquals(None, c.get('yy'))
+
+
+class TestInvokedAs(tests.TestCase):
+
+    def test_invoked_as(self):
+        """The command object knows the actual name used to invoke it."""
+        commands.install_bzr_command_hooks()
+        commands._register_builtin_commands()
+        # get one from the real get_cmd_object.
+        c = commands.get_cmd_object('ci')
+        self.assertIsInstance(c, builtins.cmd_commit)
+        self.assertEquals(c.invoked_as, 'ci')
+
+
 class TestGetAlias(tests.TestCase):
 
     def _get_config(self, config_text):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(config_text.encode('utf-8'))
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(config_text)
         return my_config
 
     def test_simple(self):
@@ -345,10 +371,3 @@
         cmds = list(commands.all_command_names())
         self.assertEqual(['called'], hook_calls)
         self.assertSubset(['foo', 'bar'], cmds)
-
-class TestDeprecations(tests.TestCase):
-
-    def test_shlex_split_unicode_deprecation(self):
-        res = self.applyDeprecated(
-                symbol_versioning.deprecated_in((2, 2, 0)),
-                commands.shlex_split_unicode, 'whatever')

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_commit.py	2011-10-05 12:16:17 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,18 +21,25 @@
 from bzrlib import (
     bzrdir,
     errors,
-    lockdir,
-    osutils,
-    tests,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
+from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.commit import Commit, NullCommitReporter
 from bzrlib.config import BranchConfig
-from bzrlib.errors import (PointlessCommit, BzrError, SigningFailed,
-                           LockContention)
-from bzrlib.tests import SymlinkFeature, TestCaseWithTransport
-from bzrlib.workingtree import WorkingTree
+from bzrlib.errors import (
+    PointlessCommit,
+    BzrError,
+    SigningFailed,
+    LockContention,
+    )
+from bzrlib.tests import (
+    TestCaseWithTransport,
+    test_foreign,
+    )
+from bzrlib.tests.features import (
+    SymlinkFeature,
+    )
+from bzrlib.tests.matchers import MatchesAncestry
 
 
 # TODO: Test commit with some added, and added-but-missing files
@@ -83,30 +90,67 @@
         b = wt.branch
         file('hello', 'w').write('hello world')
         wt.add('hello')
-        wt.commit(message='add hello')
+        rev1 = wt.commit(message='add hello')
         file_id = wt.path2id('hello')
 
         file('hello', 'w').write('version 2')
-        wt.commit(message='commit 2')
+        rev2 = wt.commit(message='commit 2')
 
         eq = self.assertEquals
         eq(b.revno(), 2)
-        rh = b.revision_history()
-        rev = b.repository.get_revision(rh[0])
+        rev = b.repository.get_revision(rev1)
         eq(rev.message, 'add hello')
 
-        tree1 = b.repository.revision_tree(rh[0])
+        tree1 = b.repository.revision_tree(rev1)
         tree1.lock_read()
         text = tree1.get_file_text(file_id)
         tree1.unlock()
         self.assertEqual('hello world', text)
 
-        tree2 = b.repository.revision_tree(rh[1])
+        tree2 = b.repository.revision_tree(rev2)
         tree2.lock_read()
         text = tree2.get_file_text(file_id)
         tree2.unlock()
         self.assertEqual('version 2', text)
 
+    def test_commit_lossy_native(self):
+        """Attempt a lossy commit to a native branch."""
+        wt = self.make_branch_and_tree('.')
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add('hello')
+        revid = wt.commit(message='add hello', rev_id='revid', lossy=True)
+        self.assertEquals('revid', revid)
+
+    def test_commit_lossy_foreign(self):
+        """Attempt a lossy commit to a foreign branch."""
+        test_foreign.register_dummy_foreign_for_test(self)
+        wt = self.make_branch_and_tree('.',
+            format=test_foreign.DummyForeignVcsDirFormat())
+        b = wt.branch
+        file('hello', 'w').write('hello world')
+        wt.add('hello')
+        revid = wt.commit(message='add hello', lossy=True,
+            timestamp=1302659388, timezone=0)
+        self.assertEquals('dummy-v1:1302659388.0-0-UNKNOWN', revid)
+
+    def test_commit_bound_lossy_foreign(self):
+        """Attempt a lossy commit to a bzr branch bound to a foreign branch."""
+        test_foreign.register_dummy_foreign_for_test(self)
+        foreign_branch = self.make_branch('foreign',
+            format=test_foreign.DummyForeignVcsDirFormat())
+        wt = foreign_branch.create_checkout("local")
+        b = wt.branch
+        file('local/hello', 'w').write('hello world')
+        wt.add('hello')
+        revid = wt.commit(message='add hello', lossy=True,
+            timestamp=1302659388, timezone=0)
+        self.assertEquals('dummy-v1:1302659388.0-0-0', revid)
+        self.assertEquals('dummy-v1:1302659388.0-0-0',
+            foreign_branch.last_revision())
+        self.assertEquals('dummy-v1:1302659388.0-0-0',
+            wt.branch.last_revision())
+
     def test_missing_commit(self):
         """Test a commit with a missing file"""
         wt = self.make_branch_and_tree('.')
@@ -116,7 +160,11 @@
         wt.commit(message='add hello')
 
         os.remove('hello')
-        wt.commit('removed hello', rev_id='rev2')
+        reporter = CapturingReporter()
+        wt.commit('removed hello', rev_id='rev2', reporter=reporter)
+        self.assertEquals(
+            [('missing', u'hello'), ('deleted', u'hello')],
+            reporter.calls)
 
         tree = b.repository.revision_tree('rev2')
         self.assertFalse(tree.has_id('hello-id'))
@@ -224,18 +272,16 @@
         eq(tree1.id2path('hello-id'), 'hello')
         eq(tree1.get_file_text('hello-id'), 'contents of hello\n')
         self.assertFalse(tree1.has_filename('fruity'))
-        self.check_inventory_shape(tree1.inventory, ['hello'])
-        ie = tree1.inventory['hello-id']
-        eq(ie.revision, 'test@rev-1')
+        self.check_tree_shape(tree1, ['hello'])
+        eq(tree1.get_file_revision('hello-id'), 'test@rev-1')
 
         tree2 = b.repository.revision_tree('test@rev-2')
         tree2.lock_read()
         self.addCleanup(tree2.unlock)
         eq(tree2.id2path('hello-id'), 'fruity')
         eq(tree2.get_file_text('hello-id'), 'contents of hello\n')
-        self.check_inventory_shape(tree2.inventory, ['fruity'])
-        ie = tree2.inventory['hello-id']
-        eq(ie.revision, 'test@rev-2')
+        self.check_tree_shape(tree2, ['fruity'])
+        eq(tree2.get_file_revision('hello-id'), 'test@rev-2')
 
     def test_reused_rev_id(self):
         """Test that a revision id cannot be reused in a branch"""
@@ -262,8 +308,7 @@
         wt.commit('two', rev_id=r2, allow_pointless=False)
         wt.lock_read()
         try:
-            self.check_inventory_shape(wt.read_working_inventory(),
-                                       ['a/', 'a/hello', 'b/'])
+            self.check_tree_shape(wt, ['a/', 'a/hello', 'b/'])
         finally:
             wt.unlock()
 
@@ -272,9 +317,9 @@
         wt.commit('three', rev_id=r3, allow_pointless=False)
         wt.lock_read()
         try:
-            self.check_inventory_shape(wt.read_working_inventory(),
+            self.check_tree_shape(wt,
                                        ['a/', 'a/hello', 'a/b/'])
-            self.check_inventory_shape(b.repository.get_inventory(r3),
+            self.check_tree_shape(b.repository.revision_tree(r3),
                                        ['a/', 'a/hello', 'a/b/'])
         finally:
             wt.unlock()
@@ -284,8 +329,7 @@
         wt.commit('four', rev_id=r4, allow_pointless=False)
         wt.lock_read()
         try:
-            self.check_inventory_shape(wt.read_working_inventory(),
-                                       ['a/', 'a/b/hello', 'a/b/'])
+            self.check_tree_shape(wt, ['a/', 'a/b/hello', 'a/b/'])
         finally:
             wt.unlock()
 
@@ -320,11 +364,9 @@
             rev_ids.append(rev_id)
             wt.commit(message='rev %d' % (i+1),
                      rev_id=rev_id)
-        eq = self.assertEquals
-        eq(b.revision_history(), rev_ids)
         for i in range(4):
-            anc = b.repository.get_ancestry(rev_ids[i])
-            eq(anc, [None] + rev_ids[:i+1])
+            self.assertThat(rev_ids[:i+1],
+                MatchesAncestry(b.repository, rev_ids[i]))
 
     def test_commit_new_subdir_child_selective(self):
         wt = self.make_branch_and_tree('.')
@@ -353,7 +395,6 @@
     def test_strict_commit_without_unknowns(self):
         """Try and commit with no unknown files and strict = True,
         should work."""
-        from bzrlib.errors import StrictCommitFailed
         wt = self.make_branch_and_tree('.')
         b = wt.branch
         file('hello', 'w').write('hello world')
@@ -385,7 +426,7 @@
         wt = self.make_branch_and_tree('.')
         branch = wt.branch
         wt.commit("base", allow_pointless=True, rev_id='A')
-        self.failIf(branch.repository.has_signature_for_revision_id('A'))
+        self.assertFalse(branch.repository.has_signature_for_revision_id('A'))
         try:
             from bzrlib.testament import Testament
             # monkey patch gpg signing mechanism
@@ -409,9 +450,8 @@
         wt = self.make_branch_and_tree('.')
         branch = wt.branch
         wt.commit("base", allow_pointless=True, rev_id='A')
-        self.failIf(branch.repository.has_signature_for_revision_id('A'))
+        self.assertFalse(branch.repository.has_signature_for_revision_id('A'))
         try:
-            from bzrlib.testament import Testament
             # monkey patch gpg signing mechanism
             bzrlib.gpg.GPGStrategy = bzrlib.gpg.DisabledGPGStrategy
             config = MustSignConfig(branch)
@@ -422,8 +462,8 @@
                               rev_id='B',
                               working_tree=wt)
             branch = Branch.open(self.get_url('.'))
-            self.assertEqual(branch.revision_history(), ['A'])
-            self.failIf(branch.repository.has_revision('B'))
+            self.assertEqual(branch.last_revision(), 'A')
+            self.assertFalse(branch.repository.has_revision('B'))
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
 

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2009-08-05 02:30:59 +0000
+++ bzrlib/tests/test_commit_merge.py	2011-10-05 18:11:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,16 +16,19 @@
 
 
 import os
-import shutil
 
-from bzrlib import check, osutils
-from bzrlib.branch import Branch
-from bzrlib.errors import PointlessCommit, BzrError
+from bzrlib import (
+    check,
+    osutils,
+    revision as _mod_revision,
+    )
+from bzrlib.errors import PointlessCommit
 from bzrlib.tests import (
+    TestCaseWithTransport,
+    )
+from bzrlib.tests.features import (
     SymlinkFeature,
-    TestCaseWithTransport,
     )
-from bzrlib.tests.test_revision import make_branches
 
 
 class TestCommitMerge(TestCaseWithTransport):
@@ -56,8 +59,12 @@
         wty.commit('merge from x', rev_id='y@u-0-2', allow_pointless=False)
 
         self.assertEquals(by.revno(), 3)
-        self.assertEquals(list(by.revision_history()),
-                          [base_rev, 'y@u-0-1', 'y@u-0-2'])
+        graph = wty.branch.repository.get_graph()
+        self.addCleanup(wty.lock_read().unlock)
+        self.assertEquals(list(
+            graph.iter_lefthand_ancestry(by.last_revision(),
+                [_mod_revision.NULL_REVISION])),
+            ['y@u-0-2', 'y@u-0-1', base_rev])
         rev = by.repository.get_revision('y@u-0-2')
         self.assertEquals(rev.parent_ids,
                           ['y@u-0-1', 'x@u-0-1'])
@@ -92,9 +99,8 @@
 
         wty.commit('merge from x', rev_id='y@u-0-2', allow_pointless=False)
         tree = by.repository.revision_tree('y@u-0-2')
-        inv = tree.inventory
-        self.assertEquals(inv['ecks-id'].revision, 'x@u-0-1')
-        self.assertEquals(inv['why-id'].revision, 'y@u-0-1')
+        self.assertEquals(tree.get_file_revision('ecks-id'), 'x@u-0-1')
+        self.assertEquals(tree.get_file_revision('why-id'), 'y@u-0-1')
 
         check.check_dwim(bx.base, False, True, True)
         check.check_dwim(by.base, False, True, True)

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2010-07-21 03:07:02 +0000
+++ bzrlib/tests/test_config.py	2011-10-06 07:08:29 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 from cStringIO import StringIO
 import os
 import sys
+import threading
+
+
+from testtools import matchers
 
 #import bzrlib specific imports here
 from bzrlib import (
@@ -31,14 +35,140 @@
     mail_client,
     ui,
     urlutils,
+    remote,
     tests,
     trace,
-    transport,
-    )
-from bzrlib.tests import features
+    )
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    )
+from bzrlib.transport import remote as transport_remote
+from bzrlib.tests import (
+    features,
+    scenarios,
+    test_server,
+    )
 from bzrlib.util.configobj import configobj
 
 
+def lockable_config_scenarios():
+    return [
+        ('global',
+         {'config_class': config.GlobalConfig,
+          'config_args': [],
+          'config_section': 'DEFAULT'}),
+        ('locations',
+         {'config_class': config.LocationConfig,
+          'config_args': ['.'],
+          'config_section': '.'}),]
+
+
+load_tests = scenarios.load_tests_apply_scenarios
+
+# Register helpers to build stores
+config.test_store_builder_registry.register(
+    'configobj', lambda test: config.IniFileStore(test.get_transport(),
+                                                  'configobj.conf'))
+config.test_store_builder_registry.register(
+    'bazaar', lambda test: config.GlobalStore())
+config.test_store_builder_registry.register(
+    'location', lambda test: config.LocationStore())
+
+
+def build_backing_branch(test, relpath,
+                         transport_class=None, server_class=None):
+    """Test helper to create a backing branch only once.
+
+    Some tests needs multiple stores/stacks to check concurrent update
+    behaviours. As such, they need to build different branch *objects* even if
+    they share the branch on disk.
+
+    :param relpath: The relative path to the branch. (Note that the helper
+        should always specify the same relpath).
+
+    :param transport_class: The Transport class the test needs to use.
+
+    :param server_class: The server associated with the ``transport_class``
+        above.
+
+    Either both or neither of ``transport_class`` and ``server_class`` should
+    be specified.
+    """
+    if transport_class is not None and server_class is not None:
+        test.transport_class = transport_class
+        test.transport_server = server_class
+    elif not (transport_class is None and server_class is None):
+        raise AssertionError('Specify both ``transport_class`` and '
+                             '``server_class`` or neither of them')
+    if getattr(test, 'backing_branch', None) is None:
+        # First call, let's build the branch on disk
+        test.backing_branch = test.make_branch(relpath)
+
+
+def build_branch_store(test):
+    build_backing_branch(test, 'branch')
+    b = branch.Branch.open('branch')
+    return config.BranchStore(b)
+config.test_store_builder_registry.register('branch', build_branch_store)
+
+
+def build_control_store(test):
+    build_backing_branch(test, 'branch')
+    b = bzrdir.BzrDir.open('branch')
+    return config.ControlStore(b)
+config.test_store_builder_registry.register('control', build_control_store)
+
+
+def build_remote_branch_store(test):
+    # There is only one permutation (but we won't be able to handle more with
+    # this design anyway)
+    (transport_class,
+     server_class) = transport_remote.get_test_permutations()[0]
+    build_backing_branch(test, 'branch', transport_class, server_class)
+    b = branch.Branch.open(test.get_url('branch'))
+    return config.BranchStore(b)
+config.test_store_builder_registry.register('remote_branch',
+                                            build_remote_branch_store)
+
+
+config.test_stack_builder_registry.register(
+    'bazaar', lambda test: config.GlobalStack())
+config.test_stack_builder_registry.register(
+    'location', lambda test: config.LocationStack('.'))
+
+
+def build_branch_stack(test):
+    build_backing_branch(test, 'branch')
+    b = branch.Branch.open('branch')
+    return config.BranchStack(b)
+config.test_stack_builder_registry.register('branch', build_branch_stack)
+
+
+def build_remote_branch_stack(test):
+    # There is only one permutation (but we won't be able to handle more with
+    # this design anyway)
+    (transport_class,
+     server_class) = transport_remote.get_test_permutations()[0]
+    build_backing_branch(test, 'branch', transport_class, server_class)
+    b = branch.Branch.open(test.get_url('branch'))
+    return config.RemoteBranchStack(b)
+config.test_stack_builder_registry.register('remote_branch',
+                                            build_remote_branch_stack)
+
+def build_remote_control_stack(test):
+    # There is only one permutation (but we won't be able to handle more with
+    # this design anyway)
+    (transport_class,
+     server_class) = transport_remote.get_test_permutations()[0]
+    # We need only a bzrdir for this, not a full branch, but it's not worth
+    # creating a dedicated helper to create only the bzrdir
+    build_backing_branch(test, 'branch', transport_class, server_class)
+    b = branch.Branch.open(test.get_url('branch'))
+    return config.RemoteControlStack(b.bzrdir)
+config.test_stack_builder_registry.register('remote_control',
+                                            build_remote_control_stack)
+
+
 sample_long_alias="log -r-15..-1 --line"
 sample_config_text = u"""
 [DEFAULT]
@@ -46,8 +176,15 @@
 editor=vim
 change_editor=vimdiff -of @new_path @old_path
 gpg_signing_command=gnome-gpg
+gpg_signing_key=DD4D5088
 log_format=short
+validate_signatures_in_log=true
+acceptable_keys=amy
 user_global_option=something
+bzr.mergetool.sometool=sometool {base} {this} {other} -o {result}
+bzr.mergetool.funkytool=funkytool "arg with spaces" {this_temp}
+bzr.mergetool.newtool='"newtool with spaces" {this_temp}'
+bzr.default_mergetool=sometool
 [ALIASES]
 h=help
 ll=""" + sample_long_alias + "\n"
@@ -95,6 +232,7 @@
 [/a/]
 check_signatures=check-available
 gpg_signing_command=false
+gpg_signing_key=default
 user_local_option=local
 # test trailing / matching
 [/a/*]
@@ -106,6 +244,49 @@
 """
 
 
+def create_configs(test):
+    """Create configuration files for a given test.
+
+    This requires creating a tree (and populate the ``test.tree`` attribute)
+    and its associated branch and will populate the following attributes:
+
+    - branch_config: A BranchConfig for the associated branch.
+
+    - locations_config : A LocationConfig for the associated branch
+
+    - bazaar_config: A GlobalConfig.
+
+    The tree and branch are created in a 'tree' subdirectory so the tests can
+    still use the test directory to stay outside of the branch.
+    """
+    tree = test.make_branch_and_tree('tree')
+    test.tree = tree
+    test.branch_config = config.BranchConfig(tree.branch)
+    test.locations_config = config.LocationConfig(tree.basedir)
+    test.bazaar_config = config.GlobalConfig()
+
+
+def create_configs_with_file_option(test):
+    """Create configuration files with a ``file`` option set in each.
+
+    This builds on ``create_configs`` and add one ``file`` option in each
+    configuration with a value which allows identifying the configuration file.
+    """
+    create_configs(test)
+    test.bazaar_config.set_user_option('file', 'bazaar')
+    test.locations_config.set_user_option('file', 'locations')
+    test.branch_config.set_user_option('file', 'branch')
+
+
+class TestOptionsMixin:
+
+    def assertOptions(self, expected, conf):
+        # We don't care about the parser (as it will make tests hard to write
+        # and error-prone anyway)
+        self.assertThat([opt[:4] for opt in conf._get_options()],
+                        matchers.Equals(expected))
+
+
 class InstrumentedConfigObj(object):
     """A config obj look-enough-alike to record calls made to it."""
 
@@ -130,6 +311,9 @@
         self._calls.append(('keys',))
         return []
 
+    def reload(self):
+        self._calls.append(('reload',))
+
     def write(self, arg):
         self._calls.append(('write',))
 
@@ -241,11 +425,37 @@
         """
         co = config.ConfigObj()
         co['test'] = 'foo#bar'
-        lines = co.write()
+        outfile = StringIO()
+        co.write(outfile=outfile)
+        lines = outfile.getvalue().splitlines()
         self.assertEqual(lines, ['test = "foo#bar"'])
         co2 = config.ConfigObj(lines)
         self.assertEqual(co2['test'], 'foo#bar')
 
+    def test_triple_quotes(self):
+        # Bug #710410: if the value string has triple quotes
+        # then ConfigObj versions up to 4.7.2 will quote them wrong
+        # and won't able to read them back
+        triple_quotes_value = '''spam
+""" that's my spam """
+eggs'''
+        co = config.ConfigObj()
+        co['test'] = triple_quotes_value
+        # While writing this test another bug in ConfigObj has been found:
+        # method co.write() without arguments produces list of lines
+        # one option per line, and multiline values are not split
+        # across multiple lines,
+        # and that breaks the parsing these lines back by ConfigObj.
+        # This issue only affects test, but it's better to avoid
+        # `co.write()` construct at all.
+        # [bialix 20110222] bug report sent to ConfigObj's author
+        outfile = StringIO()
+        co.write(outfile=outfile)
+        output = outfile.getvalue()
+        # now we're trying to read it back
+        co2 = config.ConfigObj(StringIO(output))
+        self.assertEquals(triple_quotes_value, co2['test'])
+
 
 erroneous_config = """[section] # line 1
 good=good # line 2
@@ -272,7 +482,10 @@
         config.Config()
 
     def test_no_default_editor(self):
-        self.assertRaises(NotImplementedError, config.Config().get_editor)
+        self.assertRaises(
+            NotImplementedError,
+            self.applyDeprecated, deprecated_in((2, 4, 0)),
+            config.Config().get_editor)
 
     def test_user_email(self):
         my_config = InstrumentedConfig()
@@ -321,6 +534,14 @@
         my_config = config.Config()
         self.assertEqual('long', my_config.log_format())
 
+    def test_acceptable_keys_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.acceptable_keys())
+
+    def test_validate_signatures_in_log_default(self):
+        my_config = config.Config()
+        self.assertEqual(False, my_config.validate_signatures_in_log())
+
     def test_get_change_editor(self):
         my_config = InstrumentedConfig()
         change_editor = my_config.get_change_editor('old_tree', 'new_tree')
@@ -334,11 +555,11 @@
 
     def setUp(self):
         super(TestConfigPath, self).setUp()
-        os.environ['HOME'] = '/home/bogus'
-        os.environ['XDG_CACHE_DIR'] = ''
+        self.overrideEnv('HOME', '/home/bogus')
+        self.overrideEnv('XDG_CACHE_DIR', '')
         if sys.platform == 'win32':
-            os.environ['BZR_HOME'] = \
-                r'C:\Documents and Settings\bogus\Application Data'
+            self.overrideEnv(
+                'BZR_HOME', r'C:\Documents and Settings\bogus\Application Data')
             self.bzr_home = \
                 'C:/Documents and Settings/bogus/Application Data/bazaar/2.0'
         else:
@@ -351,10 +572,6 @@
         self.assertEqual(config.config_filename(),
                          self.bzr_home + '/bazaar.conf')
 
-    def test_branches_config_filename(self):
-        self.assertEqual(config.branches_config_filename(),
-                         self.bzr_home + '/branches.conf')
-
     def test_locations_config_filename(self):
         self.assertEqual(config.locations_config_filename(),
                          self.bzr_home + '/locations.conf')
@@ -368,49 +585,438 @@
             '/home/bogus/.cache')
 
 
+class TestXDGConfigDir(tests.TestCaseInTempDir):
+    # must be in temp dir because config tests for the existence of the bazaar
+    # subdirectory of $XDG_CONFIG_HOME
+
+    def setUp(self):
+        if sys.platform in ('darwin', 'win32'):
+            raise tests.TestNotApplicable(
+                'XDG config dir not used on this platform')
+        super(TestXDGConfigDir, self).setUp()
+        self.overrideEnv('HOME', self.test_home_dir)
+        # BZR_HOME overrides everything we want to test so unset it.
+        self.overrideEnv('BZR_HOME', None)
+
+    def test_xdg_config_dir_exists(self):
+        """When ~/.config/bazaar exists, use it as the config dir."""
+        newdir = osutils.pathjoin(self.test_home_dir, '.config', 'bazaar')
+        os.makedirs(newdir)
+        self.assertEqual(config.config_dir(), newdir)
+
+    def test_xdg_config_home(self):
+        """When XDG_CONFIG_HOME is set, use it."""
+        xdgconfigdir = osutils.pathjoin(self.test_home_dir, 'xdgconfig')
+        self.overrideEnv('XDG_CONFIG_HOME', xdgconfigdir)
+        newdir = osutils.pathjoin(xdgconfigdir, 'bazaar')
+        os.makedirs(newdir)
+        self.assertEqual(config.config_dir(), newdir)
+
+
 class TestIniConfig(tests.TestCaseInTempDir):
 
     def make_config_parser(self, s):
-        conf = config.IniBasedConfig(None)
-        parser = conf._get_parser(file=StringIO(s.encode('utf-8')))
-        return conf, parser
+        conf = config.IniBasedConfig.from_string(s)
+        return conf, conf._get_parser()
 
 
 class TestIniConfigBuilding(TestIniConfig):
 
     def test_contructs(self):
-        my_config = config.IniBasedConfig("nothing")
+        my_config = config.IniBasedConfig()
 
     def test_from_fp(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.IniBasedConfig(None)
-        self.failUnless(
-            isinstance(my_config._get_parser(file=config_file),
-                        configobj.ConfigObj))
+        my_config = config.IniBasedConfig.from_string(sample_config_text)
+        self.assertIsInstance(my_config._get_parser(), configobj.ConfigObj)
 
     def test_cached(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.IniBasedConfig(None)
-        parser = my_config._get_parser(file=config_file)
-        self.failUnless(my_config._get_parser() is parser)
+        my_config = config.IniBasedConfig.from_string(sample_config_text)
+        parser = my_config._get_parser()
+        self.assertTrue(my_config._get_parser() is parser)
 
     def _dummy_chown(self, path, uid, gid):
         self.path, self.uid, self.gid = path, uid, gid
 
     def test_ini_config_ownership(self):
-        """Ensure that chown is happening during _write_config_file.
-        """
+        """Ensure that chown is happening during _write_config_file"""
         self.requireFeature(features.chown_feature)
         self.overrideAttr(os, 'chown', self._dummy_chown)
         self.path = self.uid = self.gid = None
-        def get_filename():
-            return 'foo.conf'
-        conf = config.IniBasedConfig(get_filename)
+        conf = config.IniBasedConfig(file_name='./foo.conf')
         conf._write_config_file()
-        self.assertEquals(self.path, 'foo.conf')
+        self.assertEquals(self.path, './foo.conf')
         self.assertTrue(isinstance(self.uid, int))
         self.assertTrue(isinstance(self.gid, int))
 
+    def test_get_filename_parameter_is_deprecated_(self):
+        conf = self.callDeprecated([
+            'IniBasedConfig.__init__(get_filename) was deprecated in 2.3.'
+            ' Use file_name instead.'],
+            config.IniBasedConfig, lambda: 'ini.conf')
+        self.assertEqual('ini.conf', conf.file_name)
+
+    def test_get_parser_file_parameter_is_deprecated_(self):
+        config_file = StringIO(sample_config_text.encode('utf-8'))
+        conf = config.IniBasedConfig.from_string(sample_config_text)
+        conf = self.callDeprecated([
+            'IniBasedConfig._get_parser(file=xxx) was deprecated in 2.3.'
+            ' Use IniBasedConfig(_content=xxx) instead.'],
+            conf._get_parser, file=config_file)
+
+
+class TestIniConfigSaving(tests.TestCaseInTempDir):
+
+    def test_cant_save_without_a_file_name(self):
+        conf = config.IniBasedConfig()
+        self.assertRaises(AssertionError, conf._write_config_file)
+
+    def test_saved_with_content(self):
+        content = 'foo = bar\n'
+        conf = config.IniBasedConfig.from_string(
+            content, file_name='./test.conf', save=True)
+        self.assertFileEqual(content, 'test.conf')
+
+
+class TestIniConfigOptionExpansionDefaultValue(tests.TestCaseInTempDir):
+    """What is the default value of expand for config options.
+
+    This is an opt-in beta feature used to evaluate whether or not option
+    references can appear in dangerous place raising exceptions, disapearing
+    (and as such corrupting data) or if it's safe to activate the option by
+    default.
+
+    Note that these tests relies on config._expand_default_value being already
+    overwritten in the parent class setUp.
+    """
+
+    def setUp(self):
+        super(TestIniConfigOptionExpansionDefaultValue, self).setUp()
+        self.config = None
+        self.warnings = []
+        def warning(*args):
+            self.warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+
+    def get_config(self, expand):
+        c = config.GlobalConfig.from_string('bzr.config.expand=%s' % (expand,),
+                                            save=True)
+        return c
+
+    def assertExpandIs(self, expected):
+        actual = config._get_expand_default_value()
+        #self.config.get_user_option_as_bool('bzr.config.expand')
+        self.assertEquals(expected, actual)
+
+    def test_default_is_None(self):
+        self.assertEquals(None, config._expand_default_value)
+
+    def test_default_is_False_even_if_None(self):
+        self.config = self.get_config(None)
+        self.assertExpandIs(False)
+
+    def test_default_is_False_even_if_invalid(self):
+        self.config = self.get_config('<your choice>')
+        self.assertExpandIs(False)
+        # ...
+        # Huh ? My choice is False ? Thanks, always happy to hear that :D
+        # Wait, you've been warned !
+        self.assertLength(1, self.warnings)
+        self.assertEquals(
+            'Value "<your choice>" is not a boolean for "bzr.config.expand"',
+            self.warnings[0])
+
+    def test_default_is_True(self):
+        self.config = self.get_config(True)
+        self.assertExpandIs(True)
+
+    def test_default_is_False(self):
+        self.config = self.get_config(False)
+        self.assertExpandIs(False)
+
+
+class TestIniConfigOptionExpansion(tests.TestCase):
+    """Test option expansion from the IniConfig level.
+
+    What we really want here is to test the Config level, but the class being
+    abstract as far as storing values is concerned, this can't be done
+    properly (yet).
+    """
+    # FIXME: This should be rewritten when all configs share a storage
+    # implementation -- vila 2011-02-18
+
+    def get_config(self, string=None):
+        if string is None:
+            string = ''
+        c = config.IniBasedConfig.from_string(string)
+        return c
+
+    def assertExpansion(self, expected, conf, string, env=None):
+        self.assertEquals(expected, conf.expand_options(string, env))
+
+    def test_no_expansion(self):
+        c = self.get_config('')
+        self.assertExpansion('foo', c, 'foo')
+
+    def test_env_adding_options(self):
+        c = self.get_config('')
+        self.assertExpansion('bar', c, '{foo}', {'foo': 'bar'})
+
+    def test_env_overriding_options(self):
+        c = self.get_config('foo=baz')
+        self.assertExpansion('bar', c, '{foo}', {'foo': 'bar'})
+
+    def test_simple_ref(self):
+        c = self.get_config('foo=xxx')
+        self.assertExpansion('xxx', c, '{foo}')
+
+    def test_unknown_ref(self):
+        c = self.get_config('')
+        self.assertRaises(errors.ExpandingUnknownOption,
+                          c.expand_options, '{foo}')
+
+    def test_indirect_ref(self):
+        c = self.get_config('''
+foo=xxx
+bar={foo}
+''')
+        self.assertExpansion('xxx', c, '{bar}')
+
+    def test_embedded_ref(self):
+        c = self.get_config('''
+foo=xxx
+bar=foo
+''')
+        self.assertExpansion('xxx', c, '{{bar}}')
+
+    def test_simple_loop(self):
+        c = self.get_config('foo={foo}')
+        self.assertRaises(errors.OptionExpansionLoop, c.expand_options, '{foo}')
+
+    def test_indirect_loop(self):
+        c = self.get_config('''
+foo={bar}
+bar={baz}
+baz={foo}''')
+        e = self.assertRaises(errors.OptionExpansionLoop,
+                              c.expand_options, '{foo}')
+        self.assertEquals('foo->bar->baz', e.refs)
+        self.assertEquals('{foo}', e.string)
+
+    def test_list(self):
+        conf = self.get_config('''
+foo=start
+bar=middle
+baz=end
+list={foo},{bar},{baz}
+''')
+        self.assertEquals(['start', 'middle', 'end'],
+                           conf.get_user_option('list', expand=True))
+
+    def test_cascading_list(self):
+        conf = self.get_config('''
+foo=start,{bar}
+bar=middle,{baz}
+baz=end
+list={foo}
+''')
+        self.assertEquals(['start', 'middle', 'end'],
+                           conf.get_user_option('list', expand=True))
+
+    def test_pathological_hidden_list(self):
+        conf = self.get_config('''
+foo=bin
+bar=go
+start={foo
+middle=},{
+end=bar}
+hidden={start}{middle}{end}
+''')
+        # Nope, it's either a string or a list, and the list wins as soon as a
+        # ',' appears, so the string concatenation never occur.
+        self.assertEquals(['{foo', '}', '{', 'bar}'],
+                          conf.get_user_option('hidden', expand=True))
+
+
+class TestLocationConfigOptionExpansion(tests.TestCaseInTempDir):
+
+    def get_config(self, location, string=None):
+        if string is None:
+            string = ''
+        # Since we don't save the config we won't strictly require to inherit
+        # from TestCaseInTempDir, but an error occurs so quickly...
+        c = config.LocationConfig.from_string(string, location)
+        return c
+
+    def test_dont_cross_unrelated_section(self):
+        c = self.get_config('/another/branch/path','''
+[/one/branch/path]
+foo = hello
+bar = {foo}/2
+
+[/another/branch/path]
+bar = {foo}/2
+''')
+        self.assertRaises(errors.ExpandingUnknownOption,
+                          c.get_user_option, 'bar', expand=True)
+
+    def test_cross_related_sections(self):
+        c = self.get_config('/project/branch/path','''
+[/project]
+foo = qu
+
+[/project/branch/path]
+bar = {foo}ux
+''')
+        self.assertEquals('quux', c.get_user_option('bar', expand=True))
+
+
+class TestIniBaseConfigOnDisk(tests.TestCaseInTempDir):
+
+    def test_cannot_reload_without_name(self):
+        conf = config.IniBasedConfig.from_string(sample_config_text)
+        self.assertRaises(AssertionError, conf.reload)
+
+    def test_reload_see_new_value(self):
+        c1 = config.IniBasedConfig.from_string('editor=vim\n',
+                                               file_name='./test/conf')
+        c1._write_config_file()
+        c2 = config.IniBasedConfig.from_string('editor=emacs\n',
+                                               file_name='./test/conf')
+        c2._write_config_file()
+        self.assertEqual('vim', c1.get_user_option('editor'))
+        self.assertEqual('emacs', c2.get_user_option('editor'))
+        # Make sure we get the Right value
+        c1.reload()
+        self.assertEqual('emacs', c1.get_user_option('editor'))
+
+
+class TestLockableConfig(tests.TestCaseInTempDir):
+
+    scenarios = lockable_config_scenarios()
+
+    # Set by load_tests
+    config_class = None
+    config_args = None
+    config_section = None
+
+    def setUp(self):
+        super(TestLockableConfig, self).setUp()
+        self._content = '[%s]\none=1\ntwo=2\n' % (self.config_section,)
+        self.config = self.create_config(self._content)
+
+    def get_existing_config(self):
+        return self.config_class(*self.config_args)
+
+    def create_config(self, content):
+        kwargs = dict(save=True)
+        c = self.config_class.from_string(content, *self.config_args, **kwargs)
+        return c
+
+    def test_simple_read_access(self):
+        self.assertEquals('1', self.config.get_user_option('one'))
+
+    def test_simple_write_access(self):
+        self.config.set_user_option('one', 'one')
+        self.assertEquals('one', self.config.get_user_option('one'))
+
+    def test_listen_to_the_last_speaker(self):
+        c1 = self.config
+        c2 = self.get_existing_config()
+        c1.set_user_option('one', 'ONE')
+        c2.set_user_option('two', 'TWO')
+        self.assertEquals('ONE', c1.get_user_option('one'))
+        self.assertEquals('TWO', c2.get_user_option('two'))
+        # The second update respect the first one
+        self.assertEquals('ONE', c2.get_user_option('one'))
+
+    def test_last_speaker_wins(self):
+        # If the same config is not shared, the same variable modified twice
+        # can only see a single result.
+        c1 = self.config
+        c2 = self.get_existing_config()
+        c1.set_user_option('one', 'c1')
+        c2.set_user_option('one', 'c2')
+        self.assertEquals('c2', c2._get_user_option('one'))
+        # The first modification is still available until another refresh
+        # occur
+        self.assertEquals('c1', c1._get_user_option('one'))
+        c1.set_user_option('two', 'done')
+        self.assertEquals('c2', c1._get_user_option('one'))
+
+    def test_writes_are_serialized(self):
+        c1 = self.config
+        c2 = self.get_existing_config()
+
+        # We spawn a thread that will pause *during* the write
+        before_writing = threading.Event()
+        after_writing = threading.Event()
+        writing_done = threading.Event()
+        c1_orig = c1._write_config_file
+        def c1_write_config_file():
+            before_writing.set()
+            c1_orig()
+            # The lock is held. We wait for the main thread to decide when to
+            # continue
+            after_writing.wait()
+        c1._write_config_file = c1_write_config_file
+        def c1_set_option():
+            c1.set_user_option('one', 'c1')
+            writing_done.set()
+        t1 = threading.Thread(target=c1_set_option)
+        # Collect the thread after the test
+        self.addCleanup(t1.join)
+        # Be ready to unblock the thread if the test goes wrong
+        self.addCleanup(after_writing.set)
+        t1.start()
+        before_writing.wait()
+        self.assertTrue(c1._lock.is_held)
+        self.assertRaises(errors.LockContention,
+                          c2.set_user_option, 'one', 'c2')
+        self.assertEquals('c1', c1.get_user_option('one'))
+        # Let the lock be released
+        after_writing.set()
+        writing_done.wait()
+        c2.set_user_option('one', 'c2')
+        self.assertEquals('c2', c2.get_user_option('one'))
+
+    def test_read_while_writing(self):
+       c1 = self.config
+       # We spawn a thread that will pause *during* the write
+       ready_to_write = threading.Event()
+       do_writing = threading.Event()
+       writing_done = threading.Event()
+       c1_orig = c1._write_config_file
+       def c1_write_config_file():
+           ready_to_write.set()
+           # The lock is held. We wait for the main thread to decide when to
+           # continue
+           do_writing.wait()
+           c1_orig()
+           writing_done.set()
+       c1._write_config_file = c1_write_config_file
+       def c1_set_option():
+           c1.set_user_option('one', 'c1')
+       t1 = threading.Thread(target=c1_set_option)
+       # Collect the thread after the test
+       self.addCleanup(t1.join)
+       # Be ready to unblock the thread if the test goes wrong
+       self.addCleanup(do_writing.set)
+       t1.start()
+       # Ensure the thread is ready to write
+       ready_to_write.wait()
+       self.assertTrue(c1._lock.is_held)
+       self.assertEquals('c1', c1.get_user_option('one'))
+       # If we read during the write, we get the old value
+       c2 = self.get_existing_config()
+       self.assertEquals('1', c2.get_user_option('one'))
+       # Let the writing occur and ensure it occurred
+       do_writing.set()
+       writing_done.wait()
+       # Now we get the updated value
+       c3 = self.get_existing_config()
+       self.assertEquals('c1', c3.get_user_option('one'))
+
+
 class TestGetUserOptionAs(TestIniConfig):
 
     def test_get_user_option_as_bool(self):
@@ -447,6 +1053,26 @@
         # automatically cast to list
         self.assertEqual(['x'], get_list('one_item'))
 
+    def test_get_user_option_as_int_from_SI(self):
+        conf, parser = self.make_config_parser("""
+plain = 100
+si_k = 5k,
+si_kb = 5kb,
+si_m = 5M,
+si_mb = 5MB,
+si_g = 5g,
+si_gb = 5gB,
+""")
+        get_si = conf.get_user_option_as_int_from_SI
+        self.assertEqual(100, get_si('plain'))
+        self.assertEqual(5000, get_si('si_k'))
+        self.assertEqual(5000, get_si('si_kb'))
+        self.assertEqual(5000000, get_si('si_m'))
+        self.assertEqual(5000000, get_si('si_mb'))
+        self.assertEqual(5000000000, get_si('si_g'))
+        self.assertEqual(5000000000, get_si('si_gb'))
+        self.assertEqual(None, get_si('non-exist'))
+        self.assertEqual(42, get_si('non-exist-with-default',  42))
 
 class TestSupressWarning(TestIniConfig):
 
@@ -479,7 +1105,7 @@
             parser = my_config._get_parser()
         finally:
             config.ConfigObj = oldparserclass
-        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertIsInstance(parser, InstrumentedConfigObj)
         self.assertEqual(parser._calls, [('__init__', config.config_filename(),
                                           'utf-8')])
 
@@ -496,7 +1122,7 @@
         my_config = config.BranchConfig(branch)
         location_config = my_config._get_location_config()
         self.assertEqual(branch.base, location_config.location)
-        self.failUnless(location_config is my_config._get_location_config())
+        self.assertIs(location_config, my_config._get_location_config())
 
     def test_get_config(self):
         """The Branch.get_config method works properly"""
@@ -522,11 +1148,10 @@
         branch = self.make_branch('branch')
         self.assertEqual('branch', branch.nick)
 
-        locations = config.locations_config_filename()
-        config.ensure_config_dir_exists()
         local_url = urlutils.local_path_to_url('branch')
-        open(locations, 'wb').write('[%s]\nnickname = foobar'
-                                    % (local_url,))
+        conf = config.LocationConfig.from_string(
+            '[%s]\nnickname = foobar' % (local_url,),
+            local_url, save=True)
         self.assertEqual('foobar', branch.nick)
 
     def test_config_local_path(self):
@@ -534,20 +1159,19 @@
         branch = self.make_branch('branch')
         self.assertEqual('branch', branch.nick)
 
-        locations = config.locations_config_filename()
-        config.ensure_config_dir_exists()
-        open(locations, 'wb').write('[%s/branch]\nnickname = barry'
-                                    % (osutils.getcwd().encode('utf8'),))
+        local_path = osutils.getcwd().encode('utf8')
+        conf = config.LocationConfig.from_string(
+            '[%s/branch]\nnickname = barry' % (local_path,),
+            'branch',  save=True)
         self.assertEqual('barry', branch.nick)
 
     def test_config_creates_local(self):
         """Creating a new entry in config uses a local path."""
         branch = self.make_branch('branch', format='knit')
         branch.set_push_location('http://foobar')
-        locations = config.locations_config_filename()
         local_path = osutils.getcwd().encode('utf8')
         # Surprisingly ConfigObj doesn't create a trailing newline
-        self.check_file_contents(locations,
+        self.check_file_contents(config.locations_config_filename(),
                                  '[%s/branch]\n'
                                  'push_location = http://foobar\n'
                                  'push_location:policy = norecurse\n'
@@ -558,10 +1182,10 @@
         self.assertEqual('!repo', b.get_config().get_nickname())
 
     def test_warn_if_masked(self):
-        _warning = trace.warning
         warnings = []
         def warning(*args):
             warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
 
         def set_option(store, warn_masked=True):
             warnings[:] = []
@@ -573,53 +1197,43 @@
             else:
                 self.assertEqual(1, len(warnings))
                 self.assertEqual(warning, warnings[0])
-        trace.warning = warning
-        try:
-            branch = self.make_branch('.')
-            conf = branch.get_config()
-            set_option(config.STORE_GLOBAL)
-            assertWarning(None)
-            set_option(config.STORE_BRANCH)
-            assertWarning(None)
-            set_option(config.STORE_GLOBAL)
-            assertWarning('Value "4" is masked by "3" from branch.conf')
-            set_option(config.STORE_GLOBAL, warn_masked=False)
-            assertWarning(None)
-            set_option(config.STORE_LOCATION)
-            assertWarning(None)
-            set_option(config.STORE_BRANCH)
-            assertWarning('Value "3" is masked by "0" from locations.conf')
-            set_option(config.STORE_BRANCH, warn_masked=False)
-            assertWarning(None)
-        finally:
-            trace.warning = _warning
-
-
-class TestGlobalConfigItems(tests.TestCase):
+        branch = self.make_branch('.')
+        conf = branch.get_config()
+        set_option(config.STORE_GLOBAL)
+        assertWarning(None)
+        set_option(config.STORE_BRANCH)
+        assertWarning(None)
+        set_option(config.STORE_GLOBAL)
+        assertWarning('Value "4" is masked by "3" from branch.conf')
+        set_option(config.STORE_GLOBAL, warn_masked=False)
+        assertWarning(None)
+        set_option(config.STORE_LOCATION)
+        assertWarning(None)
+        set_option(config.STORE_BRANCH)
+        assertWarning('Value "3" is masked by "0" from locations.conf')
+        set_option(config.STORE_BRANCH, warn_masked=False)
+        assertWarning(None)
+
+
+class TestGlobalConfigItems(tests.TestCaseInTempDir):
 
     def test_user_id(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
                          my_config._get_user_id())
 
     def test_absent_user_id(self):
-        config_file = StringIO("")
         my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
         self.assertEqual(None, my_config._get_user_id())
 
     def test_configured_editor(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
-        self.assertEqual("vim", my_config.get_editor())
+        my_config = config.GlobalConfig.from_string(sample_config_text)
+        editor = self.applyDeprecated(
+            deprecated_in((2, 4, 0)), my_config.get_editor)
+        self.assertEqual('vim', editor)
 
     def test_signatures_always(self):
-        config_file = StringIO(sample_always_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_always_signatures)
         self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_ALWAYS,
@@ -627,9 +1241,7 @@
         self.assertEqual(True, my_config.signature_needed())
 
     def test_signatures_if_possible(self):
-        config_file = StringIO(sample_maybe_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_maybe_signatures)
         self.assertEqual(config.CHECK_NEVER,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_WHEN_REQUIRED,
@@ -637,9 +1249,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def test_signatures_ignore(self):
-        config_file = StringIO(sample_ignore_signatures)
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_ignore_signatures)
         self.assertEqual(config.CHECK_ALWAYS,
                          my_config.signature_checking())
         self.assertEqual(config.SIGN_NEVER,
@@ -647,9 +1257,7 @@
         self.assertEqual(False, my_config.signature_needed())
 
     def _get_sample_config(self):
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
+        my_config = config.GlobalConfig.from_string(sample_config_text)
         return my_config
 
     def test_gpg_signing_command(self):
@@ -657,10 +1265,12 @@
         self.assertEqual("gnome-gpg", my_config.gpg_signing_command())
         self.assertEqual(False, my_config.signature_needed())
 
+    def test_gpg_signing_key(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("DD4D5088", my_config.gpg_signing_key())
+
     def _get_empty_config(self):
-        config_file = StringIO("")
         my_config = config.GlobalConfig()
-        my_config._parser = my_config._get_parser(file=config_file)
         return my_config
 
     def test_gpg_signing_command_unset(self):
@@ -684,6 +1294,14 @@
         my_config = self._get_sample_config()
         self.assertEqual("short", my_config.log_format())
 
+    def test_configured_acceptable_keys(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("amy", my_config.acceptable_keys())
+
+    def test_configured_validate_signatures_in_log(self):
+        my_config = self._get_sample_config()
+        self.assertEqual(True, my_config.validate_signatures_in_log())
+
     def test_get_alias(self):
         my_config = self._get_sample_config()
         self.assertEqual('help', my_config.get_alias('h'))
@@ -716,6 +1334,42 @@
         change_editor = my_config.get_change_editor('old', 'new')
         self.assertIs(None, change_editor)
 
+    def test_get_merge_tools(self):
+        conf = self._get_sample_config()
+        tools = conf.get_merge_tools()
+        self.log(repr(tools))
+        self.assertEqual(
+            {u'funkytool' : u'funkytool "arg with spaces" {this_temp}',
+            u'sometool' : u'sometool {base} {this} {other} -o {result}',
+            u'newtool' : u'"newtool with spaces" {this_temp}'},
+            tools)
+
+    def test_get_merge_tools_empty(self):
+        conf = self._get_empty_config()
+        tools = conf.get_merge_tools()
+        self.assertEqual({}, tools)
+
+    def test_find_merge_tool(self):
+        conf = self._get_sample_config()
+        cmdline = conf.find_merge_tool('sometool')
+        self.assertEqual('sometool {base} {this} {other} -o {result}', cmdline)
+
+    def test_find_merge_tool_not_found(self):
+        conf = self._get_sample_config()
+        cmdline = conf.find_merge_tool('DOES NOT EXIST')
+        self.assertIs(cmdline, None)
+
+    def test_find_merge_tool_known(self):
+        conf = self._get_empty_config()
+        cmdline = conf.find_merge_tool('kdiff3')
+        self.assertEquals('kdiff3 {base} {this} {other} -o {result}', cmdline)
+
+    def test_find_merge_tool_override_known(self):
+        conf = self._get_empty_config()
+        conf.set_user_option('bzr.mergetool.kdiff3', 'kdiff3 blah')
+        cmdline = conf.find_merge_tool('kdiff3')
+        self.assertEqual('kdiff3 blah', cmdline)
+
 
 class TestGlobalConfigSavingOptions(tests.TestCaseInTempDir):
 
@@ -739,7 +1393,7 @@
         self.assertIs(None, new_config.get_alias('commit'))
 
 
-class TestLocationConfig(tests.TestCaseInTempDir):
+class TestLocationConfig(tests.TestCaseInTempDir, TestOptionsMixin):
 
     def test_constructs(self):
         my_config = config.LocationConfig('http://example.com')
@@ -757,78 +1411,63 @@
             parser = my_config._get_parser()
         finally:
             config.ConfigObj = oldparserclass
-        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertIsInstance(parser, InstrumentedConfigObj)
         self.assertEqual(parser._calls,
                          [('__init__', config.locations_config_filename(),
                            'utf-8')])
-        config.ensure_config_dir_exists()
-        #os.mkdir(config.config_dir())
-        f = file(config.branches_config_filename(), 'wb')
-        f.write('')
-        f.close()
-        oldparserclass = config.ConfigObj
-        config.ConfigObj = InstrumentedConfigObj
-        try:
-            my_config = config.LocationConfig('http://www.example.com')
-            parser = my_config._get_parser()
-        finally:
-            config.ConfigObj = oldparserclass
 
     def test_get_global_config(self):
         my_config = config.BranchConfig(FakeBranch('http://example.com'))
         global_config = my_config._get_global_config()
-        self.failUnless(isinstance(global_config, config.GlobalConfig))
-        self.failUnless(global_config is my_config._get_global_config())
+        self.assertIsInstance(global_config, config.GlobalConfig)
+        self.assertIs(global_config, my_config._get_global_config())
+
+    def assertLocationMatching(self, expected):
+        self.assertEqual(expected,
+                         list(self.my_location_config._get_matching_sections()))
 
     def test__get_matching_sections_no_match(self):
         self.get_branch_config('/')
-        self.assertEqual([], self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([])
 
     def test__get_matching_sections_exact(self):
         self.get_branch_config('http://www.example.com')
-        self.assertEqual([('http://www.example.com', '')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('http://www.example.com', '')])
 
     def test__get_matching_sections_suffix_does_not(self):
         self.get_branch_config('http://www.example.com-com')
-        self.assertEqual([], self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([])
 
     def test__get_matching_sections_subdir_recursive(self):
         self.get_branch_config('http://www.example.com/com')
-        self.assertEqual([('http://www.example.com', 'com')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('http://www.example.com', 'com')])
 
     def test__get_matching_sections_ignoreparent(self):
         self.get_branch_config('http://www.example.com/ignoreparent')
-        self.assertEqual([('http://www.example.com/ignoreparent', '')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('http://www.example.com/ignoreparent',
+                                      '')])
 
     def test__get_matching_sections_ignoreparent_subdir(self):
         self.get_branch_config(
             'http://www.example.com/ignoreparent/childbranch')
-        self.assertEqual([('http://www.example.com/ignoreparent',
-                           'childbranch')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('http://www.example.com/ignoreparent',
+                                      'childbranch')])
 
     def test__get_matching_sections_subdir_trailing_slash(self):
         self.get_branch_config('/b')
-        self.assertEqual([('/b/', '')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('/b/', '')])
 
     def test__get_matching_sections_subdir_child(self):
         self.get_branch_config('/a/foo')
-        self.assertEqual([('/a/*', ''), ('/a/', 'foo')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('/a/*', ''), ('/a/', 'foo')])
 
     def test__get_matching_sections_subdir_child_child(self):
         self.get_branch_config('/a/foo/bar')
-        self.assertEqual([('/a/*', 'bar'), ('/a/', 'foo/bar')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('/a/*', 'bar'), ('/a/', 'foo/bar')])
 
     def test__get_matching_sections_trailing_slash_with_children(self):
         self.get_branch_config('/a/')
-        self.assertEqual([('/a/', '')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('/a/', '')])
 
     def test__get_matching_sections_explicit_over_glob(self):
         # XXX: 2006-09-08 jamesh
@@ -836,8 +1475,7 @@
         # was a config section for '/a/?', it would get precedence
         # over '/a/c'.
         self.get_branch_config('/a/c')
-        self.assertEqual([('/a/c', ''), ('/a/*', ''), ('/a/', 'c')],
-                         self.my_location_config._get_matching_sections())
+        self.assertLocationMatching([('/a/c', ''), ('/a/*', ''), ('/a/', 'c')])
 
     def test__get_option_policy_normal(self):
         self.get_branch_config('http://www.example.com')
@@ -865,6 +1503,21 @@
             'http://www.example.com', 'appendpath_option'),
             config.POLICY_APPENDPATH)
 
+    def test__get_options_with_policy(self):
+        self.get_branch_config('/dir/subdir',
+                               location_config="""\
+[/dir]
+other_url = /other-dir
+other_url:policy = appendpath
+[/dir/subdir]
+other_url = /other-subdir
+""")
+        self.assertOptions(
+            [(u'other_url', u'/other-subdir', u'/dir/subdir', 'locations'),
+             (u'other_url', u'/other-dir', u'/dir', 'locations'),
+             (u'other_url:policy', u'appendpath', u'/dir', 'locations')],
+            self.my_location_config)
+
     def test_location_without_username(self):
         self.get_branch_config('http://www.example.com/ignoreparent')
         self.assertEqual(u'Erik B\u00e5gfors <erik@bagfors.nu>',
@@ -912,6 +1565,14 @@
         self.get_branch_config('/a')
         self.assertEqual("false", self.my_config.gpg_signing_command())
 
+    def test_gpg_signing_key(self):
+        self.get_branch_config('/b')
+        self.assertEqual("DD4D5088", self.my_config.gpg_signing_key())
+
+    def test_gpg_signing_key_default(self):
+        self.get_branch_config('/a')
+        self.assertEqual("erik@bagfors.nu", self.my_config.gpg_signing_key())
+
     def test_get_user_option_global(self):
         self.get_branch_config('/a')
         self.assertEqual('something',
@@ -1006,43 +1667,34 @@
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          self.my_config.post_commit())
 
-    def get_branch_config(self, location, global_config=None):
+    def get_branch_config(self, location, global_config=None,
+                          location_config=None):
+        my_branch = FakeBranch(location)
         if global_config is None:
-            global_file = StringIO(sample_config_text.encode('utf-8'))
-        else:
-            global_file = StringIO(global_config.encode('utf-8'))
-        branches_file = StringIO(sample_branches_text.encode('utf-8'))
-        self.my_config = config.BranchConfig(FakeBranch(location))
-        # Force location config to use specified file
-        self.my_location_config = self.my_config._get_location_config()
-        self.my_location_config._get_parser(branches_file)
-        # Force global config to use specified file
-        self.my_config._get_global_config()._get_parser(global_file)
+            global_config = sample_config_text
+        if location_config is None:
+            location_config = sample_branches_text
+
+        my_global_config = config.GlobalConfig.from_string(global_config,
+                                                           save=True)
+        my_location_config = config.LocationConfig.from_string(
+            location_config, my_branch.base, save=True)
+        my_config = config.BranchConfig(my_branch)
+        self.my_config = my_config
+        self.my_location_config = my_config._get_location_config()
 
     def test_set_user_setting_sets_and_saves(self):
         self.get_branch_config('/a/c')
         record = InstrumentedConfigObj("foo")
         self.my_location_config._parser = record
 
-        real_mkdir = os.mkdir
-        self.created = False
-        def checked_mkdir(path, mode=0777):
-            self.log('making directory: %s', path)
-            real_mkdir(path, mode)
-            self.created = True
-
-        os.mkdir = checked_mkdir
-        try:
-            self.callDeprecated(['The recurse option is deprecated as of '
-                                 '0.14.  The section "/a/c" has been '
-                                 'converted to use policies.'],
-                                self.my_config.set_user_option,
-                                'foo', 'bar', store=config.STORE_LOCATION)
-        finally:
-            os.mkdir = real_mkdir
-
-        self.failUnless(self.created, 'Failed to create ~/.bazaar')
-        self.assertEqual([('__contains__', '/a/c'),
+        self.callDeprecated(['The recurse option is deprecated as of '
+                             '0.14.  The section "/a/c" has been '
+                             'converted to use policies.'],
+                            self.my_config.set_user_option,
+                            'foo', 'bar', store=config.STORE_LOCATION)
+        self.assertEqual([('reload',),
+                          ('__contains__', '/a/c'),
                           ('__contains__', '/a/c/'),
                           ('__setitem__', '/a/c', {}),
                           ('__getitem__', '/a/c'),
@@ -1077,7 +1729,7 @@
         self.assertEqual('bzr', my_config.get_bzr_remote_path())
         my_config.set_user_option('bzr_remote_path', '/path-bzr')
         self.assertEqual('/path-bzr', my_config.get_bzr_remote_path())
-        os.environ['BZR_REMOTE_PATH'] = '/environ-bzr'
+        self.overrideEnv('BZR_REMOTE_PATH', '/environ-bzr')
         self.assertEqual('/environ-bzr', my_config.get_bzr_remote_path())
 
 
@@ -1091,19 +1743,18 @@
 option = exact
 """
 
-
 class TestBranchConfigItems(tests.TestCaseInTempDir):
 
     def get_branch_config(self, global_config=None, location=None,
                           location_config=None, branch_data_config=None):
-        my_config = config.BranchConfig(FakeBranch(location))
+        my_branch = FakeBranch(location)
         if global_config is not None:
-            global_file = StringIO(global_config.encode('utf-8'))
-            my_config._get_global_config()._get_parser(global_file)
-        self.my_location_config = my_config._get_location_config()
+            my_global_config = config.GlobalConfig.from_string(global_config,
+                                                               save=True)
         if location_config is not None:
-            location_file = StringIO(location_config.encode('utf-8'))
-            self.my_location_config._get_parser(location_file)
+            my_location_config = config.LocationConfig.from_string(
+                location_config, my_branch.base, save=True)
+        my_config = config.BranchConfig(my_branch)
         if branch_data_config is not None:
             my_config.branch.control_files.files['branch.conf'] = \
                 branch_data_config
@@ -1123,14 +1774,14 @@
                          my_config.username())
 
     def test_not_set_in_branch(self):
-        my_config = self.get_branch_config(sample_config_text)
+        my_config = self.get_branch_config(global_config=sample_config_text)
         self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
                          my_config._get_user_id())
         my_config.branch.control_files.files['email'] = "John"
         self.assertEqual("John", my_config._get_user_id())
 
     def test_BZR_EMAIL_OVERRIDES(self):
-        os.environ['BZR_EMAIL'] = "Robert Collins <robertc@example.org>"
+        self.overrideEnv('BZR_EMAIL', "Robert Collins <robertc@example.org>")
         branch = FakeBranch()
         my_config = config.BranchConfig(branch)
         self.assertEqual("Robert Collins <robertc@example.org>",
@@ -1153,29 +1804,25 @@
 
     def test_gpg_signing_command(self):
         my_config = self.get_branch_config(
+            global_config=sample_config_text,
             # branch data cannot set gpg_signing_command
             branch_data_config="gpg_signing_command=pgp")
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        my_config._get_global_config()._get_parser(config_file)
         self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
 
     def test_get_user_option_global(self):
-        branch = FakeBranch()
-        my_config = config.BranchConfig(branch)
-        config_file = StringIO(sample_config_text.encode('utf-8'))
-        (my_config._get_global_config()._get_parser(config_file))
+        my_config = self.get_branch_config(global_config=sample_config_text)
         self.assertEqual('something',
                          my_config.get_user_option('user_global_option'))
 
     def test_post_commit_default(self):
-        branch = FakeBranch()
-        my_config = self.get_branch_config(sample_config_text, '/a/c',
-                                           sample_branches_text)
+        my_config = self.get_branch_config(global_config=sample_config_text,
+                                      location='/a/c',
+                                      location_config=sample_branches_text)
         self.assertEqual(my_config.branch.base, '/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          my_config.post_commit())
         my_config.set_user_option('post_commit', 'rmtree_root')
-        # post-commit is ignored when bresent in branch data
+        # post-commit is ignored when present in branch data
         self.assertEqual('bzrlib.tests.test_config.post_commit',
                          my_config.post_commit())
         my_config.set_user_option('post_commit', 'rmtree_root',
@@ -1183,19 +1830,23 @@
         self.assertEqual('rmtree_root', my_config.post_commit())
 
     def test_config_precedence(self):
+        # FIXME: eager test, luckily no persitent config file makes it fail
+        # -- vila 20100716
         my_config = self.get_branch_config(global_config=precedence_global)
         self.assertEqual(my_config.get_user_option('option'), 'global')
         my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch)
+                                           branch_data_config=precedence_branch)
         self.assertEqual(my_config.get_user_option('option'), 'branch')
-        my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch,
-                                      location_config=precedence_location)
+        my_config = self.get_branch_config(
+            global_config=precedence_global,
+            branch_data_config=precedence_branch,
+            location_config=precedence_location)
         self.assertEqual(my_config.get_user_option('option'), 'recurse')
-        my_config = self.get_branch_config(global_config=precedence_global,
-                                      branch_data_config=precedence_branch,
-                                      location_config=precedence_location,
-                                      location='http://example.com/specific')
+        my_config = self.get_branch_config(
+            global_config=precedence_global,
+            branch_data_config=precedence_branch,
+            location_config=precedence_location,
+            location='http://example.com/specific')
         self.assertEqual(my_config.get_user_option('option'), 'exact')
 
     def test_get_mail_client(self):
@@ -1291,9 +1942,57 @@
 
 class TestTransportConfig(tests.TestCaseWithTransport):
 
+    def test_load_utf8(self):
+        """Ensure we can load an utf8-encoded file."""
+        t = self.get_transport()
+        unicode_user = u'b\N{Euro Sign}ar'
+        unicode_content = u'user=%s' % (unicode_user,)
+        utf8_content = unicode_content.encode('utf8')
+        # Store the raw content in the config file
+        t.put_bytes('foo.conf', utf8_content)
+        conf = config.TransportConfig(t, 'foo.conf')
+        self.assertEquals(unicode_user, conf.get_option('user'))
+
+    def test_load_non_ascii(self):
+        """Ensure we display a proper error on non-ascii, non utf-8 content."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', 'user=foo\n#\xff\n')
+        conf = config.TransportConfig(t, 'foo.conf')
+        self.assertRaises(errors.ConfigContentError, conf._get_configobj)
+
+    def test_load_erroneous_content(self):
+        """Ensure we display a proper error on content that can't be parsed."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', '[open_section\n')
+        conf = config.TransportConfig(t, 'foo.conf')
+        self.assertRaises(errors.ParseConfigError, conf._get_configobj)
+
+    def test_load_permission_denied(self):
+        """Ensure we get an empty config file if the file is inaccessible."""
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+
+        class DenyingTransport(object):
+
+            def __init__(self, base):
+                self.base = base
+
+            def get_bytes(self, relpath):
+                raise errors.PermissionDenied(relpath, "")
+
+        cfg = config.TransportConfig(
+            DenyingTransport("nonexisting://"), 'control.conf')
+        self.assertIs(None, cfg.get_option('non-existant', 'SECTION'))
+        self.assertEquals(
+            warnings,
+            [u'Permission denied while trying to open configuration file '
+             u'nonexisting:///control.conf.'])
+
     def test_get_value(self):
         """Test that retreiving a value from a section is possible"""
-        bzrdir_config = config.TransportConfig(transport.get_transport('.'),
+        bzrdir_config = config.TransportConfig(self.get_transport('.'),
                                                'control.conf')
         bzrdir_config.set_option('value', 'key', 'SECTION')
         bzrdir_config.set_option('value2', 'key2')
@@ -1329,6 +2028,1790 @@
         self.assertIs(None, bzrdir_config.get_default_stack_on())
 
 
+class TestOldConfigHooks(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestOldConfigHooks, self).setUp()
+        create_configs_with_file_option(self)
+
+    def assertGetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('get', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'get', None)
+        self.assertLength(0, calls)
+        actual_value = conf.get_user_option(name)
+        self.assertEquals(value, actual_value)
+        self.assertLength(1, calls)
+        self.assertEquals((conf, name, value), calls[0])
+
+    def test_get_hook_bazaar(self):
+        self.assertGetHook(self.bazaar_config, 'file', 'bazaar')
+
+    def test_get_hook_locations(self):
+        self.assertGetHook(self.locations_config, 'file', 'locations')
+
+    def test_get_hook_branch(self):
+        # Since locations masks branch, we define a different option
+        self.branch_config.set_user_option('file2', 'branch')
+        self.assertGetHook(self.branch_config, 'file2', 'branch')
+
+    def assertSetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('set', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'set', None)
+        self.assertLength(0, calls)
+        conf.set_user_option(name, value)
+        self.assertLength(1, calls)
+        # We can't assert the conf object below as different configs use
+        # different means to implement set_user_option and we care only about
+        # coverage here.
+        self.assertEquals((name, value), calls[0][1:])
+
+    def test_set_hook_bazaar(self):
+        self.assertSetHook(self.bazaar_config, 'foo', 'bazaar')
+
+    def test_set_hook_locations(self):
+        self.assertSetHook(self.locations_config, 'foo', 'locations')
+
+    def test_set_hook_branch(self):
+        self.assertSetHook(self.branch_config, 'foo', 'branch')
+
+    def assertRemoveHook(self, conf, name, section_name=None):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('remove', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'remove', None)
+        self.assertLength(0, calls)
+        conf.remove_user_option(name, section_name)
+        self.assertLength(1, calls)
+        # We can't assert the conf object below as different configs use
+        # different means to implement remove_user_option and we care only about
+        # coverage here.
+        self.assertEquals((name,), calls[0][1:])
+
+    def test_remove_hook_bazaar(self):
+        self.assertRemoveHook(self.bazaar_config, 'file')
+
+    def test_remove_hook_locations(self):
+        self.assertRemoveHook(self.locations_config, 'file',
+                              self.locations_config.location)
+
+    def test_remove_hook_branch(self):
+        self.assertRemoveHook(self.branch_config, 'file')
+
+    def assertLoadHook(self, name, conf_class, *conf_args):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('load', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'load', None)
+        self.assertLength(0, calls)
+        # Build a config
+        conf = conf_class(*conf_args)
+        # Access an option to trigger a load
+        conf.get_user_option(name)
+        self.assertLength(1, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_load_hook_bazaar(self):
+        self.assertLoadHook('file', config.GlobalConfig)
+
+    def test_load_hook_locations(self):
+        self.assertLoadHook('file', config.LocationConfig, self.tree.basedir)
+
+    def test_load_hook_branch(self):
+        self.assertLoadHook('file', config.BranchConfig, self.tree.branch)
+
+    def assertSaveHook(self, conf):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('save', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'save', None)
+        self.assertLength(0, calls)
+        # Setting an option triggers a save
+        conf.set_user_option('foo', 'bar')
+        self.assertLength(1, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_save_hook_bazaar(self):
+        self.assertSaveHook(self.bazaar_config)
+
+    def test_save_hook_locations(self):
+        self.assertSaveHook(self.locations_config)
+
+    def test_save_hook_branch(self):
+        self.assertSaveHook(self.branch_config)
+
+
+class TestOldConfigHooksForRemote(tests.TestCaseWithTransport):
+    """Tests config hooks for remote configs.
+
+    No tests for the remove hook as this is not implemented there.
+    """
+
+    def setUp(self):
+        super(TestOldConfigHooksForRemote, self).setUp()
+        self.transport_server = test_server.SmartTCPServer_for_testing
+        create_configs_with_file_option(self)
+
+    def assertGetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('get', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'get', None)
+        self.assertLength(0, calls)
+        actual_value = conf.get_option(name)
+        self.assertEquals(value, actual_value)
+        self.assertLength(1, calls)
+        self.assertEquals((conf, name, value), calls[0])
+
+    def test_get_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.assertGetHook(remote_branch._get_config(), 'file', 'branch')
+
+    def test_get_hook_remote_bzrdir(self):
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        conf = remote_bzrdir._get_config()
+        conf.set_option('remotedir', 'file')
+        self.assertGetHook(conf, 'file', 'remotedir')
+
+    def assertSetHook(self, conf, name, value):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('set', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'set', None)
+        self.assertLength(0, calls)
+        conf.set_option(value, name)
+        self.assertLength(1, calls)
+        # We can't assert the conf object below as different configs use
+        # different means to implement set_user_option and we care only about
+        # coverage here.
+        self.assertEquals((name, value), calls[0][1:])
+
+    def test_set_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        self.assertSetHook(remote_branch._get_config(), 'file', 'remote')
+
+    def test_set_hook_remote_bzrdir(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        self.assertSetHook(remote_bzrdir._get_config(), 'file', 'remotedir')
+
+    def assertLoadHook(self, expected_nb_calls, name, conf_class, *conf_args):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('load', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'load', None)
+        self.assertLength(0, calls)
+        # Build a config
+        conf = conf_class(*conf_args)
+        # Access an option to trigger a load
+        conf.get_option(name)
+        self.assertLength(expected_nb_calls, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_load_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.assertLoadHook(1, 'file', remote.RemoteBranchConfig, remote_branch)
+
+    def test_load_hook_remote_bzrdir(self):
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        # The config file doesn't exist, set an option to force its creation
+        conf = remote_bzrdir._get_config()
+        conf.set_option('remotedir', 'file')
+        # We get one call for the server and one call for the client, this is
+        # caused by the differences in implementations betwen
+        # SmartServerBzrDirRequestConfigFile (in smart/bzrdir.py) and
+        # SmartServerBranchGetConfigFile (in smart/branch.py)
+        self.assertLoadHook(2 ,'file', remote.RemoteBzrDirConfig, remote_bzrdir)
+
+    def assertSaveHook(self, conf):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.OldConfigHooks.install_named_hook('save', hook, None)
+        self.addCleanup(
+            config.OldConfigHooks.uninstall_named_hook, 'save', None)
+        self.assertLength(0, calls)
+        # Setting an option triggers a save
+        conf.set_option('foo', 'bar')
+        self.assertLength(1, calls)
+        # Since we can't assert about conf, we just use the number of calls ;-/
+
+    def test_save_hook_remote_branch(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        self.assertSaveHook(remote_branch._get_config())
+
+    def test_save_hook_remote_bzrdir(self):
+        remote_branch = branch.Branch.open(self.get_url('tree'))
+        self.addCleanup(remote_branch.lock_write().unlock)
+        remote_bzrdir = bzrdir.BzrDir.open(self.get_url('tree'))
+        self.assertSaveHook(remote_bzrdir._get_config())
+
+
+class TestOption(tests.TestCase):
+
+    def test_default_value(self):
+        opt = config.Option('foo', default='bar')
+        self.assertEquals('bar', opt.get_default())
+
+    def test_default_value_from_env(self):
+        opt = config.Option('foo', default='bar', default_from_env=['FOO'])
+        self.overrideEnv('FOO', 'quux')
+        # Env variable provides a default taking over the option one
+        self.assertEquals('quux', opt.get_default())
+
+    def test_first_default_value_from_env_wins(self):
+        opt = config.Option('foo', default='bar',
+                            default_from_env=['NO_VALUE', 'FOO', 'BAZ'])
+        self.overrideEnv('FOO', 'foo')
+        self.overrideEnv('BAZ', 'baz')
+        # The first env var set wins
+        self.assertEquals('foo', opt.get_default())
+
+    def test_not_supported_list_default_value(self):
+        self.assertRaises(AssertionError, config.Option, 'foo', default=[1])
+
+    def test_not_supported_object_default_value(self):
+        self.assertRaises(AssertionError, config.Option, 'foo',
+                          default=object())
+
+
+class TestOptionConverterMixin(object):
+
+    def assertConverted(self, expected, opt, value):
+        self.assertEquals(expected, opt.convert_from_unicode(value))
+
+    def assertWarns(self, opt, value):
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        self.assertEquals(None, opt.convert_from_unicode(value))
+        self.assertLength(1, warnings)
+        self.assertEquals(
+            'Value "%s" is not valid for "%s"' % (value, opt.name),
+            warnings[0])
+
+    def assertErrors(self, opt, value):
+        self.assertRaises(errors.ConfigOptionValueError,
+                          opt.convert_from_unicode, value)
+
+    def assertConvertInvalid(self, opt, invalid_value):
+        opt.invalid = None
+        self.assertEquals(None, opt.convert_from_unicode(invalid_value))
+        opt.invalid = 'warning'
+        self.assertWarns(opt, invalid_value)
+        opt.invalid = 'error'
+        self.assertErrors(opt, invalid_value)
+
+
+class TestOptionWithBooleanConverter(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.Option('foo', help='A boolean.',
+                             from_unicode=config.bool_from_store)
+
+    def test_convert_invalid(self):
+        opt = self.get_option()
+        # A string that is not recognized as a boolean
+        self.assertConvertInvalid(opt, u'invalid-boolean')
+        # A list of strings is never recognized as a boolean
+        self.assertConvertInvalid(opt, [u'not', u'a', u'boolean'])
+
+    def test_convert_valid(self):
+        opt = self.get_option()
+        self.assertConverted(True, opt, u'True')
+        self.assertConverted(True, opt, u'1')
+        self.assertConverted(False, opt, u'False')
+
+
+class TestOptionWithIntegerConverter(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.Option('foo', help='An integer.',
+                             from_unicode=config.int_from_store)
+
+    def test_convert_invalid(self):
+        opt = self.get_option()
+        # A string that is not recognized as an integer
+        self.assertConvertInvalid(opt, u'forty-two')
+        # A list of strings is never recognized as an integer
+        self.assertConvertInvalid(opt, [u'a', u'list'])
+
+    def test_convert_valid(self):
+        opt = self.get_option()
+        self.assertConverted(16, opt, u'16')
+
+class TestOptionWithListConverter(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.Option('foo', help='A list.',
+                             from_unicode=config.list_from_store)
+
+    def test_convert_invalid(self):
+        # No string is invalid as all forms can be converted to a list
+        pass
+
+    def test_convert_valid(self):
+        opt = self.get_option()
+        # An empty string is an empty list
+        self.assertConverted([], opt, '') # Using a bare str() just in case
+        self.assertConverted([], opt, u'')
+        # A boolean
+        self.assertConverted([u'True'], opt, u'True')
+        # An integer
+        self.assertConverted([u'42'], opt, u'42')
+        # A single string
+        self.assertConverted([u'bar'], opt, u'bar')
+        # A list remains a list (configObj will turn a string containing commas
+        # into a list, but that's not what we're testing here)
+        self.assertConverted([u'foo', u'1', u'True'],
+                             opt, [u'foo', u'1', u'True'])
+
+
+class TestOptionConverterMixin(object):
+
+    def assertConverted(self, expected, opt, value):
+        self.assertEquals(expected, opt.convert_from_unicode(value))
+
+    def assertWarns(self, opt, value):
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        self.assertEquals(None, opt.convert_from_unicode(value))
+        self.assertLength(1, warnings)
+        self.assertEquals(
+            'Value "%s" is not valid for "%s"' % (value, opt.name),
+            warnings[0])
+
+    def assertErrors(self, opt, value):
+        self.assertRaises(errors.ConfigOptionValueError,
+                          opt.convert_from_unicode, value)
+
+    def assertConvertInvalid(self, opt, invalid_value):
+        opt.invalid = None
+        self.assertEquals(None, opt.convert_from_unicode(invalid_value))
+        opt.invalid = 'warning'
+        self.assertWarns(opt, invalid_value)
+        opt.invalid = 'error'
+        self.assertErrors(opt, invalid_value)
+
+
+class TestOptionWithBooleanConverter(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.Option('foo', help='A boolean.',
+                             from_unicode=config.bool_from_store)
+
+    def test_convert_invalid(self):
+        opt = self.get_option()
+        # A string that is not recognized as a boolean
+        self.assertConvertInvalid(opt, u'invalid-boolean')
+        # A list of strings is never recognized as a boolean
+        self.assertConvertInvalid(opt, [u'not', u'a', u'boolean'])
+
+    def test_convert_valid(self):
+        opt = self.get_option()
+        self.assertConverted(True, opt, u'True')
+        self.assertConverted(True, opt, u'1')
+        self.assertConverted(False, opt, u'False')
+
+
+class TestOptionWithIntegerConverter(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.Option('foo', help='An integer.',
+                             from_unicode=config.int_from_store)
+
+    def test_convert_invalid(self):
+        opt = self.get_option()
+        # A string that is not recognized as an integer
+        self.assertConvertInvalid(opt, u'forty-two')
+        # A list of strings is never recognized as an integer
+        self.assertConvertInvalid(opt, [u'a', u'list'])
+
+    def test_convert_valid(self):
+        opt = self.get_option()
+        self.assertConverted(16, opt, u'16')
+
+
+class TestOptionWithListConverter(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.Option('foo', help='A list.',
+                             from_unicode=config.list_from_store)
+
+    def test_convert_invalid(self):
+        opt = self.get_option()
+        # We don't even try to convert a list into a list, we only expect
+        # strings
+        self.assertConvertInvalid(opt, [1])
+        # No string is invalid as all forms can be converted to a list
+
+    def test_convert_valid(self):
+        opt = self.get_option()
+        # An empty string is an empty list
+        self.assertConverted([], opt, '') # Using a bare str() just in case
+        self.assertConverted([], opt, u'')
+        # A boolean
+        self.assertConverted([u'True'], opt, u'True')
+        # An integer
+        self.assertConverted([u'42'], opt, u'42')
+        # A single string
+        self.assertConverted([u'bar'], opt, u'bar')
+
+
+class TestOptionRegistry(tests.TestCase):
+
+    def setUp(self):
+        super(TestOptionRegistry, self).setUp()
+        # Always start with an empty registry
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+        self.registry = config.option_registry
+
+    def test_register(self):
+        opt = config.Option('foo')
+        self.registry.register(opt)
+        self.assertIs(opt, self.registry.get('foo'))
+
+    def test_registered_help(self):
+        opt = config.Option('foo', help='A simple option')
+        self.registry.register(opt)
+        self.assertEquals('A simple option', self.registry.get_help('foo'))
+
+    lazy_option = config.Option('lazy_foo', help='Lazy help')
+
+    def test_register_lazy(self):
+        self.registry.register_lazy('lazy_foo', self.__module__,
+                                    'TestOptionRegistry.lazy_option')
+        self.assertIs(self.lazy_option, self.registry.get('lazy_foo'))
+
+    def test_registered_lazy_help(self):
+        self.registry.register_lazy('lazy_foo', self.__module__,
+                                    'TestOptionRegistry.lazy_option')
+        self.assertEquals('Lazy help', self.registry.get_help('lazy_foo'))
+
+
+class TestRegisteredOptions(tests.TestCase):
+    """All registered options should verify some constraints."""
+
+    scenarios = [(key, {'option_name': key, 'option': option}) for key, option
+                 in config.option_registry.iteritems()]
+
+    def setUp(self):
+        super(TestRegisteredOptions, self).setUp()
+        self.registry = config.option_registry
+
+    def test_proper_name(self):
+        # An option should be registered under its own name, this can't be
+        # checked at registration time for the lazy ones.
+        self.assertEquals(self.option_name, self.option.name)
+
+    def test_help_is_set(self):
+        option_help = self.registry.get_help(self.option_name)
+        self.assertNotEquals(None, option_help)
+        # Come on, think about the user, he really wants to know what the
+        # option is about
+        self.assertIsNot(None, option_help)
+        self.assertNotEquals('', option_help)
+
+
+class TestSection(tests.TestCase):
+
+    # FIXME: Parametrize so that all sections produced by Stores run these
+    # tests -- vila 2011-04-01
+
+    def test_get_a_value(self):
+        a_dict = dict(foo='bar')
+        section = config.Section('myID', a_dict)
+        self.assertEquals('bar', section.get('foo'))
+
+    def test_get_unknown_option(self):
+        a_dict = dict()
+        section = config.Section(None, a_dict)
+        self.assertEquals('out of thin air',
+                          section.get('foo', 'out of thin air'))
+
+    def test_options_is_shared(self):
+        a_dict = dict()
+        section = config.Section(None, a_dict)
+        self.assertIs(a_dict, section.options)
+
+
+class TestMutableSection(tests.TestCase):
+
+    scenarios = [('mutable',
+                  {'get_section':
+                       lambda opts: config.MutableSection('myID', opts)},),
+                 ('cmdline',
+                  {'get_section':
+                       lambda opts: config.CommandLineSection(opts)},),
+        ]
+
+    def test_set(self):
+        a_dict = dict(foo='bar')
+        section = self.get_section(a_dict)
+        section.set('foo', 'new_value')
+        self.assertEquals('new_value', section.get('foo'))
+        # The change appears in the shared section
+        self.assertEquals('new_value', a_dict.get('foo'))
+        # We keep track of the change
+        self.assertTrue('foo' in section.orig)
+        self.assertEquals('bar', section.orig.get('foo'))
+
+    def test_set_preserve_original_once(self):
+        a_dict = dict(foo='bar')
+        section = self.get_section(a_dict)
+        section.set('foo', 'first_value')
+        section.set('foo', 'second_value')
+        # We keep track of the original value
+        self.assertTrue('foo' in section.orig)
+        self.assertEquals('bar', section.orig.get('foo'))
+
+    def test_remove(self):
+        a_dict = dict(foo='bar')
+        section = self.get_section(a_dict)
+        section.remove('foo')
+        # We get None for unknown options via the default value
+        self.assertEquals(None, section.get('foo'))
+        # Or we just get the default value
+        self.assertEquals('unknown', section.get('foo', 'unknown'))
+        self.assertFalse('foo' in section.options)
+        # We keep track of the deletion
+        self.assertTrue('foo' in section.orig)
+        self.assertEquals('bar', section.orig.get('foo'))
+
+    def test_remove_new_option(self):
+        a_dict = dict()
+        section = self.get_section(a_dict)
+        section.set('foo', 'bar')
+        section.remove('foo')
+        self.assertFalse('foo' in section.options)
+        # The option didn't exist initially so it we need to keep track of it
+        # with a special value
+        self.assertTrue('foo' in section.orig)
+        self.assertEquals(config._NewlyCreatedOption, section.orig['foo'])
+
+
+class TestCommandLineSection(tests.TestCase):
+
+    def setUp(self):
+        super(TestCommandLineSection, self).setUp()
+        self.section = config.CommandLineSection()
+
+    def test_no_override(self):
+        self.section._from_cmdline([])
+        # FIXME: we want some iterator over all options, failing that, we peek
+        # under the cover -- vila 2011-09026
+        self.assertLength(0, self.section.options)
+
+    def test_simple_override(self):
+        self.section._from_cmdline(['a=b'])
+        self.assertEqual('b', self.section.get('a'))
+
+    def test_list_override(self):
+        self.section._from_cmdline(['l=1,2,3'])
+        val = self.section.get('l')
+        self.assertEqual('1,2,3', val)
+        # Reminder: lists should registered as such explicitely, otherwise the
+        # conversion needs to be done afterwards.
+        self.assertEqual(['1', '2', '3'], config.list_from_store(val))
+
+    def test_multiple_overrides(self):
+        self.section._from_cmdline(['a=b', 'x=y'])
+        self.assertEquals('b', self.section.get('a'))
+        self.assertEquals('y', self.section.get('x'))
+
+    def test_wrong_syntax(self):
+        self.assertRaises(errors.BzrCommandError,
+                          self.section._from_cmdline, ['a=b', 'c'])
+
+
+class TestStore(tests.TestCaseWithTransport):
+
+    def assertSectionContent(self, expected, section):
+        """Assert that some options have the proper values in a section."""
+        expected_name, expected_options = expected
+        self.assertEquals(expected_name, section.id)
+        self.assertEquals(
+            expected_options,
+            dict([(k, section.get(k)) for k in expected_options.keys()]))
+
+
+class TestReadonlyStore(TestStore):
+
+    scenarios = [(key, {'get_store': builder}) for key, builder
+                 in config.test_store_builder_registry.iteritems()]
+
+    def test_building_delays_load(self):
+        store = self.get_store(self)
+        self.assertEquals(False, store.is_loaded())
+        store._load_from_string('')
+        self.assertEquals(True, store.is_loaded())
+
+    def test_get_no_sections_for_empty(self):
+        store = self.get_store(self)
+        store._load_from_string('')
+        self.assertEquals([], list(store.get_sections()))
+
+    def test_get_default_section(self):
+        store = self.get_store(self)
+        store._load_from_string('foo=bar')
+        sections = list(store.get_sections())
+        self.assertLength(1, sections)
+        self.assertSectionContent((None, {'foo': 'bar'}), sections[0])
+
+    def test_get_named_section(self):
+        store = self.get_store(self)
+        store._load_from_string('[baz]\nfoo=bar')
+        sections = list(store.get_sections())
+        self.assertLength(1, sections)
+        self.assertSectionContent(('baz', {'foo': 'bar'}), sections[0])
+
+    def test_load_from_string_fails_for_non_empty_store(self):
+        store = self.get_store(self)
+        store._load_from_string('foo=bar')
+        self.assertRaises(AssertionError, store._load_from_string, 'bar=baz')
+
+
+class TestIniFileStoreContent(tests.TestCaseWithTransport):
+    """Simulate loading a config store with content of various encodings.
+
+    All files produced by bzr are in utf8 content.
+
+    Users may modify them manually and end up with a file that can't be
+    loaded. We need to issue proper error messages in this case.
+    """
+
+    invalid_utf8_char = '\xff'
+
+    def test_load_utf8(self):
+        """Ensure we can load an utf8-encoded file."""
+        t = self.get_transport()
+        # From http://pad.lv/799212
+        unicode_user = u'b\N{Euro Sign}ar'
+        unicode_content = u'user=%s' % (unicode_user,)
+        utf8_content = unicode_content.encode('utf8')
+        # Store the raw content in the config file
+        t.put_bytes('foo.conf', utf8_content)
+        store = config.IniFileStore(t, 'foo.conf')
+        store.load()
+        stack = config.Stack([store.get_sections], store)
+        self.assertEquals(unicode_user, stack.get('user'))
+
+    def test_load_non_ascii(self):
+        """Ensure we display a proper error on non-ascii, non utf-8 content."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', 'user=foo\n#%s\n' % (self.invalid_utf8_char,))
+        store = config.IniFileStore(t, 'foo.conf')
+        self.assertRaises(errors.ConfigContentError, store.load)
+
+    def test_load_erroneous_content(self):
+        """Ensure we display a proper error on content that can't be parsed."""
+        t = self.get_transport()
+        t.put_bytes('foo.conf', '[open_section\n')
+        store = config.IniFileStore(t, 'foo.conf')
+        self.assertRaises(errors.ParseConfigError, store.load)
+
+    def test_load_permission_denied(self):
+        """Ensure we get warned when trying to load an inaccessible file."""
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+
+        t = self.get_transport()
+
+        def get_bytes(relpath):
+            raise errors.PermissionDenied(relpath, "")
+        t.get_bytes = get_bytes
+        store = config.IniFileStore(t, 'foo.conf')
+        self.assertRaises(errors.PermissionDenied, store.load)
+        self.assertEquals(
+            warnings,
+            [u'Permission denied while trying to load configuration store %s.'
+             % store.external_url()])
+
+
+class TestIniConfigContent(tests.TestCaseWithTransport):
+    """Simulate loading a IniBasedConfig with content of various encodings.
+
+    All files produced by bzr are in utf8 content.
+
+    Users may modify them manually and end up with a file that can't be
+    loaded. We need to issue proper error messages in this case.
+    """
+
+    invalid_utf8_char = '\xff'
+
+    def test_load_utf8(self):
+        """Ensure we can load an utf8-encoded file."""
+        # From http://pad.lv/799212
+        unicode_user = u'b\N{Euro Sign}ar'
+        unicode_content = u'user=%s' % (unicode_user,)
+        utf8_content = unicode_content.encode('utf8')
+        # Store the raw content in the config file
+        with open('foo.conf', 'wb') as f:
+            f.write(utf8_content)
+        conf = config.IniBasedConfig(file_name='foo.conf')
+        self.assertEquals(unicode_user, conf.get_user_option('user'))
+
+    def test_load_badly_encoded_content(self):
+        """Ensure we display a proper error on non-ascii, non utf-8 content."""
+        with open('foo.conf', 'wb') as f:
+            f.write('user=foo\n#%s\n' % (self.invalid_utf8_char,))
+        conf = config.IniBasedConfig(file_name='foo.conf')
+        self.assertRaises(errors.ConfigContentError, conf._get_parser)
+
+    def test_load_erroneous_content(self):
+        """Ensure we display a proper error on content that can't be parsed."""
+        with open('foo.conf', 'wb') as f:
+            f.write('[open_section\n')
+        conf = config.IniBasedConfig(file_name='foo.conf')
+        self.assertRaises(errors.ParseConfigError, conf._get_parser)
+
+
+class TestMutableStore(TestStore):
+
+    scenarios = [(key, {'store_id': key, 'get_store': builder}) for key, builder
+                 in config.test_store_builder_registry.iteritems()]
+
+    def setUp(self):
+        super(TestMutableStore, self).setUp()
+        self.transport = self.get_transport()
+
+    def has_store(self, store):
+        store_basename = urlutils.relative_url(self.transport.external_url(),
+                                               store.external_url())
+        return self.transport.has(store_basename)
+
+    def test_save_empty_creates_no_file(self):
+        # FIXME: There should be a better way than relying on the test
+        # parametrization to identify branch.conf -- vila 2011-0526
+        if self.store_id in ('branch', 'remote_branch'):
+            raise tests.TestNotApplicable(
+                'branch.conf is *always* created when a branch is initialized')
+        store = self.get_store(self)
+        store.save()
+        self.assertEquals(False, self.has_store(store))
+
+    def test_save_emptied_succeeds(self):
+        store = self.get_store(self)
+        store._load_from_string('foo=bar\n')
+        section = store.get_mutable_section(None)
+        section.remove('foo')
+        store.save()
+        self.assertEquals(True, self.has_store(store))
+        modified_store = self.get_store(self)
+        sections = list(modified_store.get_sections())
+        self.assertLength(0, sections)
+
+    def test_save_with_content_succeeds(self):
+        # FIXME: There should be a better way than relying on the test
+        # parametrization to identify branch.conf -- vila 2011-0526
+        if self.store_id in ('branch', 'remote_branch'):
+            raise tests.TestNotApplicable(
+                'branch.conf is *always* created when a branch is initialized')
+        store = self.get_store(self)
+        store._load_from_string('foo=bar\n')
+        self.assertEquals(False, self.has_store(store))
+        store.save()
+        self.assertEquals(True, self.has_store(store))
+        modified_store = self.get_store(self)
+        sections = list(modified_store.get_sections())
+        self.assertLength(1, sections)
+        self.assertSectionContent((None, {'foo': 'bar'}), sections[0])
+
+    def test_set_option_in_empty_store(self):
+        store = self.get_store(self)
+        section = store.get_mutable_section(None)
+        section.set('foo', 'bar')
+        store.save()
+        modified_store = self.get_store(self)
+        sections = list(modified_store.get_sections())
+        self.assertLength(1, sections)
+        self.assertSectionContent((None, {'foo': 'bar'}), sections[0])
+
+    def test_set_option_in_default_section(self):
+        store = self.get_store(self)
+        store._load_from_string('')
+        section = store.get_mutable_section(None)
+        section.set('foo', 'bar')
+        store.save()
+        modified_store = self.get_store(self)
+        sections = list(modified_store.get_sections())
+        self.assertLength(1, sections)
+        self.assertSectionContent((None, {'foo': 'bar'}), sections[0])
+
+    def test_set_option_in_named_section(self):
+        store = self.get_store(self)
+        store._load_from_string('')
+        section = store.get_mutable_section('baz')
+        section.set('foo', 'bar')
+        store.save()
+        modified_store = self.get_store(self)
+        sections = list(modified_store.get_sections())
+        self.assertLength(1, sections)
+        self.assertSectionContent(('baz', {'foo': 'bar'}), sections[0])
+
+    def test_load_hook(self):
+        # We first needs to ensure that the store exists
+        store = self.get_store(self)
+        section = store.get_mutable_section('baz')
+        section.set('foo', 'bar')
+        store.save()
+        # Now we can try to load it
+        store = self.get_store(self)
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('load', hook, None)
+        self.assertLength(0, calls)
+        store.load()
+        self.assertLength(1, calls)
+        self.assertEquals((store,), calls[0])
+
+    def test_save_hook(self):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('save', hook, None)
+        self.assertLength(0, calls)
+        store = self.get_store(self)
+        section = store.get_mutable_section('baz')
+        section.set('foo', 'bar')
+        store.save()
+        self.assertLength(1, calls)
+        self.assertEquals((store,), calls[0])
+
+
+class TestIniFileStore(TestStore):
+
+    def test_loading_unknown_file_fails(self):
+        store = config.IniFileStore(self.get_transport(), 'I-do-not-exist')
+        self.assertRaises(errors.NoSuchFile, store.load)
+
+    def test_invalid_content(self):
+        store = config.IniFileStore(self.get_transport(), 'foo.conf', )
+        self.assertEquals(False, store.is_loaded())
+        exc = self.assertRaises(
+            errors.ParseConfigError, store._load_from_string,
+            'this is invalid !')
+        self.assertEndsWith(exc.filename, 'foo.conf')
+        # And the load failed
+        self.assertEquals(False, store.is_loaded())
+
+    def test_get_embedded_sections(self):
+        # A more complicated example (which also shows that section names and
+        # option names share the same name space...)
+        # FIXME: This should be fixed by forbidding dicts as values ?
+        # -- vila 2011-04-05
+        store = config.IniFileStore(self.get_transport(), 'foo.conf', )
+        store._load_from_string('''
+foo=bar
+l=1,2
+[DEFAULT]
+foo_in_DEFAULT=foo_DEFAULT
+[bar]
+foo_in_bar=barbar
+[baz]
+foo_in_baz=barbaz
+[[qux]]
+foo_in_qux=quux
+''')
+        sections = list(store.get_sections())
+        self.assertLength(4, sections)
+        # The default section has no name.
+        # List values are provided as strings and need to be explicitly
+        # converted by specifying from_unicode=list_from_store at option
+        # registration
+        self.assertSectionContent((None, {'foo': 'bar', 'l': u'1,2'}),
+                                  sections[0])
+        self.assertSectionContent(
+            ('DEFAULT', {'foo_in_DEFAULT': 'foo_DEFAULT'}), sections[1])
+        self.assertSectionContent(
+            ('bar', {'foo_in_bar': 'barbar'}), sections[2])
+        # sub sections are provided as embedded dicts.
+        self.assertSectionContent(
+            ('baz', {'foo_in_baz': 'barbaz', 'qux': {'foo_in_qux': 'quux'}}),
+            sections[3])
+
+
+class TestLockableIniFileStore(TestStore):
+
+    def test_create_store_in_created_dir(self):
+        self.assertPathDoesNotExist('dir')
+        t = self.get_transport('dir/subdir')
+        store = config.LockableIniFileStore(t, 'foo.conf')
+        store.get_mutable_section(None).set('foo', 'bar')
+        store.save()
+        self.assertPathExists('dir/subdir')
+
+
+class TestConcurrentStoreUpdates(TestStore):
+    """Test that Stores properly handle conccurent updates.
+
+    New Store implementation may fail some of these tests but until such
+    implementations exist it's hard to properly filter them from the scenarios
+    applied here. If you encounter such a case, contact the bzr devs.
+    """
+
+    scenarios = [(key, {'get_stack': builder}) for key, builder
+                 in config.test_stack_builder_registry.iteritems()]
+
+    def setUp(self):
+        super(TestConcurrentStoreUpdates, self).setUp()
+        self._content = 'one=1\ntwo=2\n'
+        self.stack = self.get_stack(self)
+        if not isinstance(self.stack, config._CompatibleStack):
+            raise tests.TestNotApplicable(
+                '%s is not meant to be compatible with the old config design'
+                % (self.stack,))
+        self.stack.store._load_from_string(self._content)
+        # Flush the store
+        self.stack.store.save()
+
+    def test_simple_read_access(self):
+        self.assertEquals('1', self.stack.get('one'))
+
+    def test_simple_write_access(self):
+        self.stack.set('one', 'one')
+        self.assertEquals('one', self.stack.get('one'))
+
+    def test_listen_to_the_last_speaker(self):
+        c1 = self.stack
+        c2 = self.get_stack(self)
+        c1.set('one', 'ONE')
+        c2.set('two', 'TWO')
+        self.assertEquals('ONE', c1.get('one'))
+        self.assertEquals('TWO', c2.get('two'))
+        # The second update respect the first one
+        self.assertEquals('ONE', c2.get('one'))
+
+    def test_last_speaker_wins(self):
+        # If the same config is not shared, the same variable modified twice
+        # can only see a single result.
+        c1 = self.stack
+        c2 = self.get_stack(self)
+        c1.set('one', 'c1')
+        c2.set('one', 'c2')
+        self.assertEquals('c2', c2.get('one'))
+        # The first modification is still available until another refresh
+        # occur
+        self.assertEquals('c1', c1.get('one'))
+        c1.set('two', 'done')
+        self.assertEquals('c2', c1.get('one'))
+
+    def test_writes_are_serialized(self):
+        c1 = self.stack
+        c2 = self.get_stack(self)
+
+        # We spawn a thread that will pause *during* the config saving.
+        before_writing = threading.Event()
+        after_writing = threading.Event()
+        writing_done = threading.Event()
+        c1_save_without_locking_orig = c1.store.save_without_locking
+        def c1_save_without_locking():
+            before_writing.set()
+            c1_save_without_locking_orig()
+            # The lock is held. We wait for the main thread to decide when to
+            # continue
+            after_writing.wait()
+        c1.store.save_without_locking = c1_save_without_locking
+        def c1_set():
+            c1.set('one', 'c1')
+            writing_done.set()
+        t1 = threading.Thread(target=c1_set)
+        # Collect the thread after the test
+        self.addCleanup(t1.join)
+        # Be ready to unblock the thread if the test goes wrong
+        self.addCleanup(after_writing.set)
+        t1.start()
+        before_writing.wait()
+        self.assertRaises(errors.LockContention,
+                          c2.set, 'one', 'c2')
+        self.assertEquals('c1', c1.get('one'))
+        # Let the lock be released
+        after_writing.set()
+        writing_done.wait()
+        c2.set('one', 'c2')
+        self.assertEquals('c2', c2.get('one'))
+
+    def test_read_while_writing(self):
+       c1 = self.stack
+       # We spawn a thread that will pause *during* the write
+       ready_to_write = threading.Event()
+       do_writing = threading.Event()
+       writing_done = threading.Event()
+       # We override the _save implementation so we know the store is locked
+       c1_save_without_locking_orig = c1.store.save_without_locking
+       def c1_save_without_locking():
+           ready_to_write.set()
+           # The lock is held. We wait for the main thread to decide when to
+           # continue
+           do_writing.wait()
+           c1_save_without_locking_orig()
+           writing_done.set()
+       c1.store.save_without_locking = c1_save_without_locking
+       def c1_set():
+           c1.set('one', 'c1')
+       t1 = threading.Thread(target=c1_set)
+       # Collect the thread after the test
+       self.addCleanup(t1.join)
+       # Be ready to unblock the thread if the test goes wrong
+       self.addCleanup(do_writing.set)
+       t1.start()
+       # Ensure the thread is ready to write
+       ready_to_write.wait()
+       self.assertEquals('c1', c1.get('one'))
+       # If we read during the write, we get the old value
+       c2 = self.get_stack(self)
+       self.assertEquals('1', c2.get('one'))
+       # Let the writing occur and ensure it occurred
+       do_writing.set()
+       writing_done.wait()
+       # Now we get the updated value
+       c3 = self.get_stack(self)
+       self.assertEquals('c1', c3.get('one'))
+
+    # FIXME: It may be worth looking into removing the lock dir when it's not
+    # needed anymore and look at possible fallouts for concurrent lockers. This
+    # will matter if/when we use config files outside of bazaar directories
+    # (.bazaar or .bzr) -- vila 20110-04-111
+
+
+class TestSectionMatcher(TestStore):
+
+    scenarios = [('location', {'matcher': config.LocationMatcher}),
+                 ('id', {'matcher': config.NameMatcher}),]
+
+    def setUp(self):
+        super(TestSectionMatcher, self).setUp()
+        # Any simple store is good enough
+        self.get_store = config.test_store_builder_registry.get('configobj')
+
+    def test_no_matches_for_empty_stores(self):
+        store = self.get_store(self)
+        store._load_from_string('')
+        matcher = self.matcher(store, '/bar')
+        self.assertEquals([], list(matcher.get_sections()))
+
+    def test_build_doesnt_load_store(self):
+        store = self.get_store(self)
+        matcher = self.matcher(store, '/bar')
+        self.assertFalse(store.is_loaded())
+
+
+class TestLocationSection(tests.TestCase):
+
+    def get_section(self, options, extra_path):
+        section = config.Section('foo', options)
+        # We don't care about the length so we use '0'
+        return config.LocationSection(section, 0, extra_path)
+
+    def test_simple_option(self):
+        section = self.get_section({'foo': 'bar'}, '')
+        self.assertEquals('bar', section.get('foo'))
+
+    def test_option_with_extra_path(self):
+        section = self.get_section({'foo': 'bar', 'foo:policy': 'appendpath'},
+                                   'baz')
+        self.assertEquals('bar/baz', section.get('foo'))
+
+    def test_invalid_policy(self):
+        section = self.get_section({'foo': 'bar', 'foo:policy': 'die'},
+                                   'baz')
+        # invalid policies are ignored
+        self.assertEquals('bar', section.get('foo'))
+
+
+class TestLocationMatcher(TestStore):
+
+    def setUp(self):
+        super(TestLocationMatcher, self).setUp()
+        # Any simple store is good enough
+        self.get_store = config.test_store_builder_registry.get('configobj')
+
+    def test_unrelated_section_excluded(self):
+        store = self.get_store(self)
+        store._load_from_string('''
+[/foo]
+section=/foo
+[/foo/baz]
+section=/foo/baz
+[/foo/bar]
+section=/foo/bar
+[/foo/bar/baz]
+section=/foo/bar/baz
+[/quux/quux]
+section=/quux/quux
+''')
+        self.assertEquals(['/foo', '/foo/baz', '/foo/bar', '/foo/bar/baz',
+                           '/quux/quux'],
+                          [section.id for section in store.get_sections()])
+        matcher = config.LocationMatcher(store, '/foo/bar/quux')
+        sections = list(matcher.get_sections())
+        self.assertEquals([3, 2],
+                          [section.length for section in sections])
+        self.assertEquals(['/foo/bar', '/foo'],
+                          [section.id for section in sections])
+        self.assertEquals(['quux', 'bar/quux'],
+                          [section.extra_path for section in sections])
+
+    def test_more_specific_sections_first(self):
+        store = self.get_store(self)
+        store._load_from_string('''
+[/foo]
+section=/foo
+[/foo/bar]
+section=/foo/bar
+''')
+        self.assertEquals(['/foo', '/foo/bar'],
+                          [section.id for section in store.get_sections()])
+        matcher = config.LocationMatcher(store, '/foo/bar/baz')
+        sections = list(matcher.get_sections())
+        self.assertEquals([3, 2],
+                          [section.length for section in sections])
+        self.assertEquals(['/foo/bar', '/foo'],
+                          [section.id for section in sections])
+        self.assertEquals(['baz', 'bar/baz'],
+                          [section.extra_path for section in sections])
+
+    def test_appendpath_in_no_name_section(self):
+        # It's a bit weird to allow appendpath in a no-name section, but
+        # someone may found a use for it
+        store = self.get_store(self)
+        store._load_from_string('''
+foo=bar
+foo:policy = appendpath
+''')
+        matcher = config.LocationMatcher(store, 'dir/subdir')
+        sections = list(matcher.get_sections())
+        self.assertLength(1, sections)
+        self.assertEquals('bar/dir/subdir', sections[0].get('foo'))
+
+    def test_file_urls_are_normalized(self):
+        store = self.get_store(self)
+        if sys.platform == 'win32':
+            expected_url = 'file:///C:/dir/subdir'
+            expected_location = 'C:/dir/subdir'
+        else:
+            expected_url = 'file:///dir/subdir'
+            expected_location = '/dir/subdir'
+        matcher = config.LocationMatcher(store, expected_url)
+        self.assertEquals(expected_location, matcher.location)
+
+
+class TestNameMatcher(TestStore):
+
+    def setUp(self):
+        super(TestNameMatcher, self).setUp()
+        self.matcher = config.NameMatcher
+        # Any simple store is good enough
+        self.get_store = config.test_store_builder_registry.get('configobj')
+
+    def get_matching_sections(self, name):
+        store = self.get_store(self)
+        store._load_from_string('''
+[foo]
+option=foo
+[foo/baz]
+option=foo/baz
+[bar]
+option=bar
+''')
+        matcher = self.matcher(store, name)
+        return list(matcher.get_sections())
+
+    def test_matching(self):
+        sections = self.get_matching_sections('foo')
+        self.assertLength(1, sections)
+        self.assertSectionContent(('foo', {'option': 'foo'}), sections[0])
+
+    def test_not_matching(self):
+        sections = self.get_matching_sections('baz')
+        self.assertLength(0, sections)
+
+
+class TestStackGet(tests.TestCase):
+
+    # FIXME: This should be parametrized for all known Stack or dedicated
+    # paramerized tests created to avoid bloating -- vila 2011-03-31
+
+    def overrideOptionRegistry(self):
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+
+    def test_single_config_get(self):
+        conf = dict(foo='bar')
+        conf_stack = config.Stack([conf])
+        self.assertEquals('bar', conf_stack.get('foo'))
+
+    def test_get_with_registered_default_value(self):
+        conf_stack = config.Stack([dict()])
+        opt = config.Option('foo', default='bar')
+        self.overrideOptionRegistry()
+        config.option_registry.register('foo', opt)
+        self.assertEquals('bar', conf_stack.get('foo'))
+
+    def test_get_without_registered_default_value(self):
+        conf_stack = config.Stack([dict()])
+        opt = config.Option('foo')
+        self.overrideOptionRegistry()
+        config.option_registry.register('foo', opt)
+        self.assertEquals(None, conf_stack.get('foo'))
+
+    def test_get_without_default_value_for_not_registered(self):
+        conf_stack = config.Stack([dict()])
+        opt = config.Option('foo')
+        self.overrideOptionRegistry()
+        self.assertEquals(None, conf_stack.get('foo'))
+
+    def test_get_first_definition(self):
+        conf1 = dict(foo='bar')
+        conf2 = dict(foo='baz')
+        conf_stack = config.Stack([conf1, conf2])
+        self.assertEquals('bar', conf_stack.get('foo'))
+
+    def test_get_embedded_definition(self):
+        conf1 = dict(yy='12')
+        conf2 = config.Stack([dict(xx='42'), dict(foo='baz')])
+        conf_stack = config.Stack([conf1, conf2])
+        self.assertEquals('baz', conf_stack.get('foo'))
+
+    def test_get_for_empty_section_callable(self):
+        conf_stack = config.Stack([lambda : []])
+        self.assertEquals(None, conf_stack.get('foo'))
+
+    def test_get_for_broken_callable(self):
+        # Trying to use and invalid callable raises an exception on first use
+        conf_stack = config.Stack([lambda : object()])
+        self.assertRaises(TypeError, conf_stack.get, 'foo')
+
+
+class TestStackWithTransport(tests.TestCaseWithTransport):
+
+    scenarios = [(key, {'get_stack': builder}) for key, builder
+                 in config.test_stack_builder_registry.iteritems()]
+
+
+class TestConcreteStacks(TestStackWithTransport):
+
+    def test_build_stack(self):
+        # Just a smoke test to help debug builders
+        stack = self.get_stack(self)
+
+
+class TestStackGet(TestStackWithTransport):
+
+    def setUp(self):
+        super(TestStackGet, self).setUp()
+        self.conf = self.get_stack(self)
+
+    def test_get_for_empty_stack(self):
+        self.assertEquals(None, self.conf.get('foo'))
+
+    def test_get_hook(self):
+        self.conf.store._load_from_string('foo=bar')
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('get', hook, None)
+        self.assertLength(0, calls)
+        value = self.conf.get('foo')
+        self.assertEquals('bar', value)
+        self.assertLength(1, calls)
+        self.assertEquals((self.conf, 'foo', 'bar'), calls[0])
+
+
+class TestStackGetWithConverter(TestStackGet):
+
+    def setUp(self):
+        super(TestStackGetWithConverter, self).setUp()
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+        self.registry = config.option_registry
+
+    def register_bool_option(self, name, default=None, default_from_env=None):
+        b = config.Option(name, help='A boolean.',
+                          default=default, default_from_env=default_from_env,
+                          from_unicode=config.bool_from_store)
+        self.registry.register(b)
+
+    def test_get_default_bool_None(self):
+        self.register_bool_option('foo')
+        self.assertEquals(None, self.conf.get('foo'))
+
+    def test_get_default_bool_True(self):
+        self.register_bool_option('foo', u'True')
+        self.assertEquals(True, self.conf.get('foo'))
+
+    def test_get_default_bool_False(self):
+        self.register_bool_option('foo', False)
+        self.assertEquals(False, self.conf.get('foo'))
+
+    def test_get_default_bool_False_as_string(self):
+        self.register_bool_option('foo', u'False')
+        self.assertEquals(False, self.conf.get('foo'))
+
+    def test_get_default_bool_from_env_converted(self):
+        self.register_bool_option('foo', u'True', default_from_env=['FOO'])
+        self.overrideEnv('FOO', 'False')
+        self.assertEquals(False, self.conf.get('foo'))
+
+    def test_get_default_bool_when_conversion_fails(self):
+        self.register_bool_option('foo', default='True')
+        self.conf.store._load_from_string('foo=invalid boolean')
+        self.assertEquals(True, self.conf.get('foo'))
+
+    def register_integer_option(self, name,
+                                default=None, default_from_env=None):
+        i = config.Option(name, help='An integer.',
+                          default=default, default_from_env=default_from_env,
+                          from_unicode=config.int_from_store)
+        self.registry.register(i)
+
+    def test_get_default_integer_None(self):
+        self.register_integer_option('foo')
+        self.assertEquals(None, self.conf.get('foo'))
+
+    def test_get_default_integer(self):
+        self.register_integer_option('foo', 42)
+        self.assertEquals(42, self.conf.get('foo'))
+
+    def test_get_default_integer_as_string(self):
+        self.register_integer_option('foo', u'42')
+        self.assertEquals(42, self.conf.get('foo'))
+
+    def test_get_default_integer_from_env(self):
+        self.register_integer_option('foo', default_from_env=['FOO'])
+        self.overrideEnv('FOO', '18')
+        self.assertEquals(18, self.conf.get('foo'))
+
+    def test_get_default_integer_when_conversion_fails(self):
+        self.register_integer_option('foo', default='12')
+        self.conf.store._load_from_string('foo=invalid integer')
+        self.assertEquals(12, self.conf.get('foo'))
+
+    def register_list_option(self, name, default=None, default_from_env=None):
+        l = config.Option(name, help='A list.',
+                          default=default, default_from_env=default_from_env,
+                          from_unicode=config.list_from_store)
+        self.registry.register(l)
+
+    def test_get_default_list_None(self):
+        self.register_list_option('foo')
+        self.assertEquals(None, self.conf.get('foo'))
+
+    def test_get_default_list_empty(self):
+        self.register_list_option('foo', '')
+        self.assertEquals([], self.conf.get('foo'))
+
+    def test_get_default_list_from_env(self):
+        self.register_list_option('foo', default_from_env=['FOO'])
+        self.overrideEnv('FOO', '')
+        self.assertEquals([], self.conf.get('foo'))
+
+    def test_get_with_list_converter_no_item(self):
+        self.register_list_option('foo', None)
+        self.conf.store._load_from_string('foo=,')
+        self.assertEquals([], self.conf.get('foo'))
+
+    def test_get_with_list_converter_many_items(self):
+        self.register_list_option('foo', None)
+        self.conf.store._load_from_string('foo=m,o,r,e')
+        self.assertEquals(['m', 'o', 'r', 'e'], self.conf.get('foo'))
+
+    def test_get_with_list_converter_embedded_spaces_many_items(self):
+        self.register_list_option('foo', None)
+        self.conf.store._load_from_string('foo=" bar", "baz "')
+        self.assertEquals([' bar', 'baz '], self.conf.get('foo'))
+
+    def test_get_with_list_converter_stripped_spaces_many_items(self):
+        self.register_list_option('foo', None)
+        self.conf.store._load_from_string('foo= bar ,  baz ')
+        self.assertEquals(['bar', 'baz'], self.conf.get('foo'))
+
+
+class TestStackExpandOptions(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestStackExpandOptions, self).setUp()
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+        self.registry = config.option_registry
+        self.conf = build_branch_stack(self)
+
+    def assertExpansion(self, expected, string, env=None):
+        self.assertEquals(expected, self.conf.expand_options(string, env))
+
+    def test_no_expansion(self):
+        self.assertExpansion('foo', 'foo')
+
+    def test_expand_default_value(self):
+        self.conf.store._load_from_string('bar=baz')
+        self.registry.register(config.Option('foo', default=u'{bar}'))
+        self.assertEquals('baz', self.conf.get('foo', expand=True))
+
+    def test_expand_default_from_env(self):
+        self.conf.store._load_from_string('bar=baz')
+        self.registry.register(config.Option('foo', default_from_env=['FOO']))
+        self.overrideEnv('FOO', '{bar}')
+        self.assertEquals('baz', self.conf.get('foo', expand=True))
+
+    def test_expand_default_on_failed_conversion(self):
+        self.conf.store._load_from_string('baz=bogus\nbar=42\nfoo={baz}')
+        self.registry.register(
+            config.Option('foo', default=u'{bar}',
+                          from_unicode=config.int_from_store))
+        self.assertEquals(42, self.conf.get('foo', expand=True))
+
+    def test_env_adding_options(self):
+        self.assertExpansion('bar', '{foo}', {'foo': 'bar'})
+
+    def test_env_overriding_options(self):
+        self.conf.store._load_from_string('foo=baz')
+        self.assertExpansion('bar', '{foo}', {'foo': 'bar'})
+
+    def test_simple_ref(self):
+        self.conf.store._load_from_string('foo=xxx')
+        self.assertExpansion('xxx', '{foo}')
+
+    def test_unknown_ref(self):
+        self.assertRaises(errors.ExpandingUnknownOption,
+                          self.conf.expand_options, '{foo}')
+
+    def test_indirect_ref(self):
+        self.conf.store._load_from_string('''
+foo=xxx
+bar={foo}
+''')
+        self.assertExpansion('xxx', '{bar}')
+
+    def test_embedded_ref(self):
+        self.conf.store._load_from_string('''
+foo=xxx
+bar=foo
+''')
+        self.assertExpansion('xxx', '{{bar}}')
+
+    def test_simple_loop(self):
+        self.conf.store._load_from_string('foo={foo}')
+        self.assertRaises(errors.OptionExpansionLoop,
+                          self.conf.expand_options, '{foo}')
+
+    def test_indirect_loop(self):
+        self.conf.store._load_from_string('''
+foo={bar}
+bar={baz}
+baz={foo}''')
+        e = self.assertRaises(errors.OptionExpansionLoop,
+                              self.conf.expand_options, '{foo}')
+        self.assertEquals('foo->bar->baz', e.refs)
+        self.assertEquals('{foo}', e.string)
+
+    def test_list(self):
+        self.conf.store._load_from_string('''
+foo=start
+bar=middle
+baz=end
+list={foo},{bar},{baz}
+''')
+        self.registry.register(
+            config.Option('list', from_unicode=config.list_from_store))
+        self.assertEquals(['start', 'middle', 'end'],
+                           self.conf.get('list', expand=True))
+
+    def test_cascading_list(self):
+        self.conf.store._load_from_string('''
+foo=start,{bar}
+bar=middle,{baz}
+baz=end
+list={foo}
+''')
+        self.registry.register(
+            config.Option('list', from_unicode=config.list_from_store))
+        self.assertEquals(['start', 'middle', 'end'],
+                           self.conf.get('list', expand=True))
+
+    def test_pathologically_hidden_list(self):
+        self.conf.store._load_from_string('''
+foo=bin
+bar=go
+start={foo
+middle=},{
+end=bar}
+hidden={start}{middle}{end}
+''')
+        # What matters is what the registration says, the conversion happens
+        # only after all expansions have been performed
+        self.registry.register(
+            config.Option('hidden', from_unicode=config.list_from_store))
+        self.assertEquals(['bin', 'go'],
+                          self.conf.get('hidden', expand=True))
+
+
+class TestStackCrossSectionsExpand(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestStackCrossSectionsExpand, self).setUp()
+
+    def get_config(self, location, string):
+        if string is None:
+            string = ''
+        # Since we don't save the config we won't strictly require to inherit
+        # from TestCaseInTempDir, but an error occurs so quickly...
+        c = config.LocationStack(location)
+        c.store._load_from_string(string)
+        return c
+
+    def test_dont_cross_unrelated_section(self):
+        c = self.get_config('/another/branch/path','''
+[/one/branch/path]
+foo = hello
+bar = {foo}/2
+
+[/another/branch/path]
+bar = {foo}/2
+''')
+        self.assertRaises(errors.ExpandingUnknownOption,
+                          c.get, 'bar', expand=True)
+
+    def test_cross_related_sections(self):
+        c = self.get_config('/project/branch/path','''
+[/project]
+foo = qu
+
+[/project/branch/path]
+bar = {foo}ux
+''')
+        self.assertEquals('quux', c.get('bar', expand=True))
+
+
+class TestStackSet(TestStackWithTransport):
+
+    def test_simple_set(self):
+        conf = self.get_stack(self)
+        conf.store._load_from_string('foo=bar')
+        self.assertEquals('bar', conf.get('foo'))
+        conf.set('foo', 'baz')
+        # Did we get it back ?
+        self.assertEquals('baz', conf.get('foo'))
+
+    def test_set_creates_a_new_section(self):
+        conf = self.get_stack(self)
+        conf.set('foo', 'baz')
+        self.assertEquals, 'baz', conf.get('foo')
+
+    def test_set_hook(self):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('set', hook, None)
+        self.assertLength(0, calls)
+        conf = self.get_stack(self)
+        conf.set('foo', 'bar')
+        self.assertLength(1, calls)
+        self.assertEquals((conf, 'foo', 'bar'), calls[0])
+
+
+class TestStackRemove(TestStackWithTransport):
+
+    def test_remove_existing(self):
+        conf = self.get_stack(self)
+        conf.set('foo', 'bar')
+        self.assertEquals('bar', conf.get('foo'))
+        conf.remove('foo')
+        # Did we get it back ?
+        self.assertEquals(None, conf.get('foo'))
+
+    def test_remove_unknown(self):
+        conf = self.get_stack(self)
+        self.assertRaises(KeyError, conf.remove, 'I_do_not_exist')
+
+    def test_remove_hook(self):
+        calls = []
+        def hook(*args):
+            calls.append(args)
+        config.ConfigHooks.install_named_hook('remove', hook, None)
+        self.assertLength(0, calls)
+        conf = self.get_stack(self)
+        conf.set('foo', 'bar')
+        conf.remove('foo')
+        self.assertLength(1, calls)
+        self.assertEquals((conf, 'foo'), calls[0])
+
+
+class TestConfigGetOptions(tests.TestCaseWithTransport, TestOptionsMixin):
+
+    def setUp(self):
+        super(TestConfigGetOptions, self).setUp()
+        create_configs(self)
+
+    def test_no_variable(self):
+        # Using branch should query branch, locations and bazaar
+        self.assertOptions([], self.branch_config)
+
+    def test_option_in_bazaar(self):
+        self.bazaar_config.set_user_option('file', 'bazaar')
+        self.assertOptions([('file', 'bazaar', 'DEFAULT', 'bazaar')],
+                           self.bazaar_config)
+
+    def test_option_in_locations(self):
+        self.locations_config.set_user_option('file', 'locations')
+        self.assertOptions(
+            [('file', 'locations', self.tree.basedir, 'locations')],
+            self.locations_config)
+
+    def test_option_in_branch(self):
+        self.branch_config.set_user_option('file', 'branch')
+        self.assertOptions([('file', 'branch', 'DEFAULT', 'branch')],
+                           self.branch_config)
+
+    def test_option_in_bazaar_and_branch(self):
+        self.bazaar_config.set_user_option('file', 'bazaar')
+        self.branch_config.set_user_option('file', 'branch')
+        self.assertOptions([('file', 'branch', 'DEFAULT', 'branch'),
+                            ('file', 'bazaar', 'DEFAULT', 'bazaar'),],
+                           self.branch_config)
+
+    def test_option_in_branch_and_locations(self):
+        # Hmm, locations override branch :-/
+        self.locations_config.set_user_option('file', 'locations')
+        self.branch_config.set_user_option('file', 'branch')
+        self.assertOptions(
+            [('file', 'locations', self.tree.basedir, 'locations'),
+             ('file', 'branch', 'DEFAULT', 'branch'),],
+            self.branch_config)
+
+    def test_option_in_bazaar_locations_and_branch(self):
+        self.bazaar_config.set_user_option('file', 'bazaar')
+        self.locations_config.set_user_option('file', 'locations')
+        self.branch_config.set_user_option('file', 'branch')
+        self.assertOptions(
+            [('file', 'locations', self.tree.basedir, 'locations'),
+             ('file', 'branch', 'DEFAULT', 'branch'),
+             ('file', 'bazaar', 'DEFAULT', 'bazaar'),],
+            self.branch_config)
+
+
+class TestConfigRemoveOption(tests.TestCaseWithTransport, TestOptionsMixin):
+
+    def setUp(self):
+        super(TestConfigRemoveOption, self).setUp()
+        create_configs_with_file_option(self)
+
+    def test_remove_in_locations(self):
+        self.locations_config.remove_user_option('file', self.tree.basedir)
+        self.assertOptions(
+            [('file', 'branch', 'DEFAULT', 'branch'),
+             ('file', 'bazaar', 'DEFAULT', 'bazaar'),],
+            self.branch_config)
+
+    def test_remove_in_branch(self):
+        self.branch_config.remove_user_option('file')
+        self.assertOptions(
+            [('file', 'locations', self.tree.basedir, 'locations'),
+             ('file', 'bazaar', 'DEFAULT', 'bazaar'),],
+            self.branch_config)
+
+    def test_remove_in_bazaar(self):
+        self.bazaar_config.remove_user_option('file')
+        self.assertOptions(
+            [('file', 'locations', self.tree.basedir, 'locations'),
+             ('file', 'branch', 'DEFAULT', 'branch'),],
+            self.branch_config)
+
+
+class TestConfigGetSections(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestConfigGetSections, self).setUp()
+        create_configs(self)
+
+    def assertSectionNames(self, expected, conf, name=None):
+        """Check which sections are returned for a given config.
+
+        If fallback configurations exist their sections can be included.
+
+        :param expected: A list of section names.
+
+        :param conf: The configuration that will be queried.
+
+        :param name: An optional section name that will be passed to
+            get_sections().
+        """
+        sections = list(conf._get_sections(name))
+        self.assertLength(len(expected), sections)
+        self.assertEqual(expected, [name for name, _, _ in sections])
+
+    def test_bazaar_default_section(self):
+        self.assertSectionNames(['DEFAULT'], self.bazaar_config)
+
+    def test_locations_default_section(self):
+        # No sections are defined in an empty file
+        self.assertSectionNames([], self.locations_config)
+
+    def test_locations_named_section(self):
+        self.locations_config.set_user_option('file', 'locations')
+        self.assertSectionNames([self.tree.basedir], self.locations_config)
+
+    def test_locations_matching_sections(self):
+        loc_config = self.locations_config
+        loc_config.set_user_option('file', 'locations')
+        # We need to cheat a bit here to create an option in sections above and
+        # below the 'location' one.
+        parser = loc_config._get_parser()
+        # locations.cong deals with '/' ignoring native os.sep
+        location_names = self.tree.basedir.split('/')
+        parent = '/'.join(location_names[:-1])
+        child = '/'.join(location_names + ['child'])
+        parser[parent] = {}
+        parser[parent]['file'] = 'parent'
+        parser[child] = {}
+        parser[child]['file'] = 'child'
+        self.assertSectionNames([self.tree.basedir, parent], loc_config)
+
+    def test_branch_data_default_section(self):
+        self.assertSectionNames([None],
+                                self.branch_config._get_branch_data_config())
+
+    def test_branch_default_sections(self):
+        # No sections are defined in an empty locations file
+        self.assertSectionNames([None, 'DEFAULT'],
+                                self.branch_config)
+        # Unless we define an option
+        self.branch_config._get_location_config().set_user_option(
+            'file', 'locations')
+        self.assertSectionNames([self.tree.basedir, None, 'DEFAULT'],
+                                self.branch_config)
+
+    def test_bazaar_named_section(self):
+        # We need to cheat as the API doesn't give direct access to sections
+        # other than DEFAULT.
+        self.bazaar_config.set_alias('bazaar', 'bzr')
+        self.assertSectionNames(['ALIASES'], self.bazaar_config, 'ALIASES')
+
+
 class TestAuthenticationConfigFile(tests.TestCase):
     """Test the authentication.conf file matching"""
 
@@ -1349,6 +3832,11 @@
         self.assertEquals({}, conf._get_config())
         self._got_user_passwd(None, None, conf, 'http', 'foo.net')
 
+    def test_non_utf8_config(self):
+        conf = config.AuthenticationConfig(_file=StringIO(
+                'foo = bar\xff'))
+        self.assertRaises(errors.ConfigContentError, conf._get_config)
+
     def test_missing_auth_section_header(self):
         conf = config.AuthenticationConfig(_file=StringIO('foo = bar'))
         self.assertRaises(ValueError, conf.get_credentials, 'ftp', 'foo.net')
@@ -1612,11 +4100,11 @@
 
     def test_username_defaults_prompts(self):
         # HTTP prompts can't be tested here, see test_http.py
-        self._check_default_username_prompt('FTP %(host)s username: ', 'ftp')
-        self._check_default_username_prompt(
-            'FTP %(host)s:%(port)d username: ', 'ftp', port=10020)
-        self._check_default_username_prompt(
-            'SSH %(host)s:%(port)d username: ', 'ssh', port=12345)
+        self._check_default_username_prompt(u'FTP %(host)s username: ', 'ftp')
+        self._check_default_username_prompt(
+            u'FTP %(host)s:%(port)d username: ', 'ftp', port=10020)
+        self._check_default_username_prompt(
+            u'SSH %(host)s:%(port)d username: ', 'ssh', port=12345)
 
     def test_username_default_no_prompt(self):
         conf = config.AuthenticationConfig()
@@ -1628,22 +4116,21 @@
     def test_password_default_prompts(self):
         # HTTP prompts can't be tested here, see test_http.py
         self._check_default_password_prompt(
-            'FTP %(user)s@%(host)s password: ', 'ftp')
-        self._check_default_password_prompt(
-            'FTP %(user)s@%(host)s:%(port)d password: ', 'ftp', port=10020)
-        self._check_default_password_prompt(
-            'SSH %(user)s@%(host)s:%(port)d password: ', 'ssh', port=12345)
+            u'FTP %(user)s@%(host)s password: ', 'ftp')
+        self._check_default_password_prompt(
+            u'FTP %(user)s@%(host)s:%(port)d password: ', 'ftp', port=10020)
+        self._check_default_password_prompt(
+            u'SSH %(user)s@%(host)s:%(port)d password: ', 'ssh', port=12345)
         # SMTP port handling is a bit special (it's handled if embedded in the
         # host too)
         # FIXME: should we: forbid that, extend it to other schemes, leave
         # things as they are that's fine thank you ?
-        self._check_default_password_prompt('SMTP %(user)s@%(host)s password: ',
-                                            'smtp')
-        self._check_default_password_prompt('SMTP %(user)s@%(host)s password: ',
-                                            'smtp', host='bar.org:10025')
-        self._check_default_password_prompt(
-            'SMTP %(user)s@%(host)s:%(port)d password: ',
-            'smtp', port=10025)
+        self._check_default_password_prompt(
+            u'SMTP %(user)s@%(host)s password: ', 'smtp')
+        self._check_default_password_prompt(
+            u'SMTP %(user)s@%(host)s password: ', 'smtp', host='bar.org:10025')
+        self._check_default_password_prompt(
+            u'SMTP %(user)s@%(host)s:%(port)d password: ', 'smtp', port=10025)
 
     def test_ssh_password_emits_warning(self):
         conf = config.AuthenticationConfig(_file=StringIO(
@@ -1829,3 +4316,26 @@
 # test_user_prompted ?
 class TestAuthenticationRing(tests.TestCaseWithTransport):
     pass
+
+
+class TestAutoUserId(tests.TestCase):
+    """Test inferring an automatic user name."""
+
+    def test_auto_user_id(self):
+        """Automatic inference of user name.
+        
+        This is a bit hard to test in an isolated way, because it depends on
+        system functions that go direct to /etc or perhaps somewhere else.
+        But it's reasonable to say that on Unix, with an /etc/mailname, we ought
+        to be able to choose a user name with no configuration.
+        """
+        if sys.platform == 'win32':
+            raise tests.TestSkipped(
+                "User name inference not implemented on win32")
+        realname, address = config._auto_user_id()
+        if os.path.exists('/etc/mailname'):
+            self.assertIsNot(None, realname)
+            self.assertIsNot(None, address)
+        else:
+            self.assertEquals((None, None), (realname, address))
+

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	2010-04-06 10:12:42 +0000
+++ bzrlib/tests/test_conflicts.py	2011-09-16 15:37:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,34 +18,20 @@
 import os
 
 from bzrlib import (
-    branchbuilder,
     bzrdir,
     conflicts,
     errors,
     option,
+    osutils,
     tests,
-    workingtree,
-    )
-from bzrlib.tests import script
-
-
-def load_tests(standard_tests, module, loader):
-    result = loader.suiteClass()
-
-    sp_tests, remaining_tests = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance((
-                TestParametrizedResolveConflicts,
-                )))
-    # Each test class defines its own scenarios. This is needed for
-    # TestResolvePathConflictBefore531967 that verifies that the same tests as
-    # TestResolvePathConflict still pass.
-    for test in tests.iter_suite_tests(sp_tests):
-        tests.apply_scenarios(test, test.scenarios(), result)
-
-    # No parametrization for the remaining tests
-    result.addTests(remaining_tests)
-
-    return result
+    )
+from bzrlib.tests import (
+    script,
+    scenarios,
+    )
+
+
+load_tests = scenarios.load_tests_apply_scenarios
 
 
 # TODO: Test commit with some added, and added-but-missing files
@@ -76,35 +62,13 @@
 ])
 
 
+def vary_by_conflicts():
+    for conflict in example_conflicts:
+        yield (conflict.__class__.__name__, {"conflict": conflict})
+
+
 class TestConflicts(tests.TestCaseWithTransport):
 
-    def test_conflicts(self):
-        """Conflicts are detected properly"""
-        # Use BzrDirFormat6 so we can fake conflicts
-        tree = self.make_branch_and_tree('.', format=bzrdir.BzrDirFormat6())
-        self.build_tree_contents([('hello', 'hello world4'),
-                                  ('hello.THIS', 'hello world2'),
-                                  ('hello.BASE', 'hello world1'),
-                                  ('hello.OTHER', 'hello world3'),
-                                  ('hello.sploo.BASE', 'yellowworld'),
-                                  ('hello.sploo.OTHER', 'yellowworld2'),
-                                  ])
-        tree.lock_read()
-        self.assertLength(6, list(tree.list_files()))
-        tree.unlock()
-        tree_conflicts = tree.conflicts()
-        self.assertLength(2, tree_conflicts)
-        self.assertTrue('hello' in tree_conflicts[0].path)
-        self.assertTrue('hello.sploo' in tree_conflicts[1].path)
-        conflicts.restore('hello')
-        conflicts.restore('hello.sploo')
-        self.assertLength(0, tree.conflicts())
-        self.assertFileEqual('hello world2', 'hello')
-        self.assertFalse(os.path.lexists('hello.sploo'))
-        self.assertRaises(errors.NotConflicted, conflicts.restore, 'hello')
-        self.assertRaises(errors.NotConflicted,
-                          conflicts.restore, 'hello.sploo')
-
     def test_resolve_conflict_dir(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents([('hello', 'hello world4'),
@@ -161,38 +125,57 @@
         self.assertEqual(conflicts.ConflictList([]), tree.conflicts())
 
 
-class TestConflictStanzas(tests.TestCase):
+class TestPerConflict(tests.TestCase):
+
+    scenarios = scenarios.multiply_scenarios(vary_by_conflicts())
+
+    def test_stringification(self):
+        text = unicode(self.conflict)
+        self.assertContainsString(text, self.conflict.path)
+        self.assertContainsString(text.lower(), "conflict")
+        self.assertContainsString(repr(self.conflict),
+            self.conflict.__class__.__name__)
 
     def test_stanza_roundtrip(self):
-        # write and read our example stanza.
-        stanza_iter = example_conflicts.to_stanzas()
-        processed = conflicts.ConflictList.from_stanzas(stanza_iter)
-        for o, p in zip(processed, example_conflicts):
-            self.assertEqual(o, p)
-
-            self.assertIsInstance(o.path, unicode)
-
-            if o.file_id is not None:
-                self.assertIsInstance(o.file_id, str)
-
-            conflict_path = getattr(o, 'conflict_path', None)
-            if conflict_path is not None:
-                self.assertIsInstance(conflict_path, unicode)
-
-            conflict_file_id = getattr(o, 'conflict_file_id', None)
-            if conflict_file_id is not None:
-                self.assertIsInstance(conflict_file_id, str)
+        p = self.conflict
+        o = conflicts.Conflict.factory(**p.as_stanza().as_dict())
+        self.assertEqual(o, p)
+
+        self.assertIsInstance(o.path, unicode)
+
+        if o.file_id is not None:
+            self.assertIsInstance(o.file_id, str)
+
+        conflict_path = getattr(o, 'conflict_path', None)
+        if conflict_path is not None:
+            self.assertIsInstance(conflict_path, unicode)
+
+        conflict_file_id = getattr(o, 'conflict_file_id', None)
+        if conflict_file_id is not None:
+            self.assertIsInstance(conflict_file_id, str)
 
     def test_stanzification(self):
-        for stanza in example_conflicts.to_stanzas():
-            if 'file_id' in stanza:
-                # In Stanza form, the file_id has to be unicode.
-                self.assertStartsWith(stanza['file_id'], u'\xeed')
-            self.assertStartsWith(stanza['path'], u'p\xe5th')
-            if 'conflict_path' in stanza:
-                self.assertStartsWith(stanza['conflict_path'], u'p\xe5th')
-            if 'conflict_file_id' in stanza:
-                self.assertStartsWith(stanza['conflict_file_id'], u'\xeed')
+        stanza = self.conflict.as_stanza()
+        if 'file_id' in stanza:
+            # In Stanza form, the file_id has to be unicode.
+            self.assertStartsWith(stanza['file_id'], u'\xeed')
+        self.assertStartsWith(stanza['path'], u'p\xe5th')
+        if 'conflict_path' in stanza:
+            self.assertStartsWith(stanza['conflict_path'], u'p\xe5th')
+        if 'conflict_file_id' in stanza:
+            self.assertStartsWith(stanza['conflict_file_id'], u'\xeed')
+
+
+class TestConflictList(tests.TestCase):
+
+    def test_stanzas_roundtrip(self):
+        stanzas_iter = example_conflicts.to_stanzas()
+        processed = conflicts.ConflictList.from_stanzas(stanzas_iter)
+        self.assertEqual(example_conflicts, processed)
+
+    def test_stringification(self):
+        for text, o in zip(example_conflicts.to_strings(), example_conflicts):
+            self.assertEqual(text, unicode(o))
 
 
 # FIXME: The shell-like tests should be converted to real whitebox tests... or
@@ -210,11 +193,6 @@
         self.run_script(self.preamble)
 
 
-class TestResolveTextConflicts(TestResolveConflicts):
-    # TBC
-    pass
-
-
 def mirror_scenarios(base_scenarios):
     """Return a list of mirrored scenarios.
 
@@ -293,34 +271,31 @@
     _this = None
     _other = None
 
-    @staticmethod
-    def scenarios():
-        """Return the scenario list for the conflict type defined by the class.
-
-        Each scenario is of the form:
-        (common, (left_name, left_dict), (right_name, right_dict))
-
-        * common is a dict
-
-        * left_name and right_name are the scenario names that will be combined
-
-        * left_dict and right_dict are the attributes specific to each half of
-          the scenario. They should include at least 'actions' and 'check' and
-          will be available as '_this' and '_other' test instance attributes.
-
-        Daughters classes are free to add their specific attributes as they see
-        fit in any of the three dicts.
-
-        This is a class method so that load_tests can find it.
-
-        '_base_actions' in the common dict, 'actions' and 'check' in the left
-        and right dicts use names that map to methods in the test classes. Some
-        prefixes are added to these names to get the correspong methods (see
-        _get_actions() and _get_check()). The motivation here is to avoid
-        collisions in the class namespace.
-        """
-        # Only concrete classes return actual scenarios
-        return []
+    scenarios = []
+    """The scenario list for the conflict type defined by the class.
+
+    Each scenario is of the form:
+    (common, (left_name, left_dict), (right_name, right_dict))
+
+    * common is a dict
+
+    * left_name and right_name are the scenario names that will be combined
+
+    * left_dict and right_dict are the attributes specific to each half of
+      the scenario. They should include at least 'actions' and 'check' and
+      will be available as '_this' and '_other' test instance attributes.
+
+    Daughters classes are free to add their specific attributes as they see
+    fit in any of the three dicts.
+
+    This is a class method so that load_tests can find it.
+
+    '_base_actions' in the common dict, 'actions' and 'check' in the left
+    and right dicts use names that map to methods in the test classes. Some
+    prefixes are added to these names to get the correspong methods (see
+    _get_actions() and _get_check()). The motivation here is to avoid
+    collisions in the class namespace.
+    """
 
     def setUp(self):
         super(TestParametrizedResolveConflicts, self).setUp()
@@ -388,18 +363,82 @@
         check_other()
 
 
+class TestResolveTextConflicts(TestParametrizedResolveConflicts):
+
+    _conflict_type = conflicts.TextConflict
+
+    # Set by the scenarios
+    # path and file-id for the file involved in the conflict
+    _path = None
+    _file_id = None
+
+    scenarios = mirror_scenarios(
+        [
+            # File modified on both sides
+            (dict(_base_actions='create_file',
+                  _path='file', _file_id='file-id'),
+             ('filed_modified_A',
+              dict(actions='modify_file_A', check='file_has_content_A')),
+             ('file_modified_B',
+              dict(actions='modify_file_B', check='file_has_content_B')),),
+            # File modified on both sides in dir
+            (dict(_base_actions='create_file_in_dir',
+                  _path='dir/file', _file_id='file-id'),
+             ('filed_modified_A_in_dir',
+              dict(actions='modify_file_A',
+                   check='file_in_dir_has_content_A')),
+             ('file_modified_B',
+              dict(actions='modify_file_B',
+                   check='file_in_dir_has_content_B')),),
+            ])
+
+    def do_create_file(self, path='file'):
+        return [('add', (path, 'file-id', 'file', 'trunk content\n'))]
+
+    def do_modify_file_A(self):
+        return [('modify', ('file-id', 'trunk content\nfeature A\n'))]
+
+    def do_modify_file_B(self):
+        return [('modify', ('file-id', 'trunk content\nfeature B\n'))]
+
+    def check_file_has_content_A(self, path='file'):
+        self.assertFileEqual('trunk content\nfeature A\n',
+                             osutils.pathjoin('branch', path))
+
+    def check_file_has_content_B(self, path='file'):
+        self.assertFileEqual('trunk content\nfeature B\n',
+                             osutils.pathjoin('branch', path))
+
+    def do_create_file_in_dir(self):
+        return [('add', ('dir', 'dir-id', 'directory', '')),
+            ] + self.do_create_file('dir/file')
+
+    def check_file_in_dir_has_content_A(self):
+        self.check_file_has_content_A('dir/file')
+
+    def check_file_in_dir_has_content_B(self):
+        self.check_file_has_content_B('dir/file')
+
+    def _get_resolve_path_arg(self, wt, action):
+        return self._path
+
+    def assertTextConflict(self, wt, c):
+        self.assertEqual(self._file_id, c.file_id)
+        self.assertEqual(self._path, c.path)
+    _assert_conflict = assertTextConflict
+
+
 class TestResolveContentsConflict(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.ContentsConflict,
+    _conflict_type = conflicts.ContentsConflict
 
-    # Set by load_tests from scenarios()
+    # Set by the scenarios
     # path and file-id for the file involved in the conflict
     _path = None
     _file_id = None
 
-    @staticmethod
-    def scenarios():
-        base_scenarios = [
+    scenarios = mirror_scenarios(
+        [
             # File modified/deleted
             (dict(_base_actions='create_file',
                   _path='file', _file_id='file-id'),
@@ -407,8 +446,24 @@
               dict(actions='modify_file', check='file_has_more_content')),
              ('file_deleted',
               dict(actions='delete_file', check='file_doesnt_exist')),),
-            ]
-        return mirror_scenarios(base_scenarios)
+            # File renamed-modified/deleted
+            (dict(_base_actions='create_file',
+                  _path='new-file', _file_id='file-id'),
+             ('file_renamed_and_modified',
+              dict(actions='modify_and_rename_file',
+                   check='file_renamed_and_more_content')),
+             ('file_deleted',
+              dict(actions='delete_file', check='file_doesnt_exist')),),
+            # File modified/deleted in dir
+            (dict(_base_actions='create_file_in_dir',
+                  _path='dir/file', _file_id='file-id'),
+             ('file_modified_in_dir',
+              dict(actions='modify_file_in_dir',
+                   check='file_in_dir_has_more_content')),
+             ('file_deleted_in_dir',
+              dict(actions='delete_file',
+                   check='file_in_dir_doesnt_exist')),),
+            ])
 
     def do_create_file(self):
         return [('add', ('file', 'file-id', 'file', 'trunk content\n'))]
@@ -416,14 +471,34 @@
     def do_modify_file(self):
         return [('modify', ('file-id', 'trunk content\nmore content\n'))]
 
+    def do_modify_and_rename_file(self):
+        return [('modify', ('file-id', 'trunk content\nmore content\n')),
+                ('rename', ('file', 'new-file'))]
+
     def check_file_has_more_content(self):
         self.assertFileEqual('trunk content\nmore content\n', 'branch/file')
 
+    def check_file_renamed_and_more_content(self):
+        self.assertFileEqual('trunk content\nmore content\n', 'branch/new-file')
+
     def do_delete_file(self):
         return [('unversion', 'file-id')]
 
     def check_file_doesnt_exist(self):
-        self.failIfExists('branch/file')
+        self.assertPathDoesNotExist('branch/file')
+
+    def do_create_file_in_dir(self):
+        return [('add', ('dir', 'dir-id', 'directory', '')),
+                ('add', ('dir/file', 'file-id', 'file', 'trunk content\n'))]
+
+    def do_modify_file_in_dir(self):
+        return [('modify', ('file-id', 'trunk content\nmore content\n'))]
+
+    def check_file_in_dir_has_more_content(self):
+        self.assertFileEqual('trunk content\nmore content\n', 'branch/dir/file')
+
+    def check_file_in_dir_doesnt_exist(self):
+        self.assertPathDoesNotExist('branch/dir/file')
 
     def _get_resolve_path_arg(self, wt, action):
         return self._path
@@ -436,17 +511,16 @@
 
 class TestResolvePathConflict(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.PathConflict,
+    _conflict_type = conflicts.PathConflict
 
     def do_nothing(self):
         return []
 
-    @staticmethod
-    def scenarios():
-        # Each side dict additionally defines:
-        # - path path involved (can be '<deleted>')
-        # - file-id involved
-        base_scenarios = [
+    # Each side dict additionally defines:
+    # - path path involved (can be '<deleted>')
+    # - file-id involved
+    scenarios = mirror_scenarios(
+        [
             # File renamed/deleted
             (dict(_base_actions='create_file'),
              ('file_renamed',
@@ -457,6 +531,16 @@
                    # PathConflicts deletion handling requires a special
                    # hard-coded value
                    path='<deleted>', file_id='file-id')),),
+            # File renamed/deleted in dir
+            (dict(_base_actions='create_file_in_dir'),
+             ('file_renamed_in_dir',
+              dict(actions='rename_file_in_dir', check='file_in_dir_renamed',
+                   path='dir/new-file', file_id='file-id')),
+             ('file_deleted',
+              dict(actions='delete_file', check='file_in_dir_doesnt_exist',
+                   # PathConflicts deletion handling requires a special
+                   # hard-coded value
+                   path='<deleted>', file_id='file-id')),),
             # File renamed/renamed differently
             (dict(_base_actions='create_file'),
              ('file_renamed',
@@ -483,8 +567,7 @@
              ('dir_renamed2',
               dict(actions='rename_dir2', check='dir_renamed2',
                    path='new-dir2', file_id='dir-id')),),
-        ]
-        return mirror_scenarios(base_scenarios)
+            ])
 
     def do_create_file(self):
         return [('add', ('file', 'file-id', 'file', 'trunk content\n'))]
@@ -496,41 +579,55 @@
         return [('rename', ('file', 'new-file'))]
 
     def check_file_renamed(self):
-        self.failIfExists('branch/file')
-        self.failUnlessExists('branch/new-file')
+        self.assertPathDoesNotExist('branch/file')
+        self.assertPathExists('branch/new-file')
 
     def do_rename_file2(self):
         return [('rename', ('file', 'new-file2'))]
 
     def check_file_renamed2(self):
-        self.failIfExists('branch/file')
-        self.failUnlessExists('branch/new-file2')
+        self.assertPathDoesNotExist('branch/file')
+        self.assertPathExists('branch/new-file2')
 
     def do_rename_dir(self):
         return [('rename', ('dir', 'new-dir'))]
 
     def check_dir_renamed(self):
-        self.failIfExists('branch/dir')
-        self.failUnlessExists('branch/new-dir')
+        self.assertPathDoesNotExist('branch/dir')
+        self.assertPathExists('branch/new-dir')
 
     def do_rename_dir2(self):
         return [('rename', ('dir', 'new-dir2'))]
 
     def check_dir_renamed2(self):
-        self.failIfExists('branch/dir')
-        self.failUnlessExists('branch/new-dir2')
+        self.assertPathDoesNotExist('branch/dir')
+        self.assertPathExists('branch/new-dir2')
 
     def do_delete_file(self):
         return [('unversion', 'file-id')]
 
     def check_file_doesnt_exist(self):
-        self.failIfExists('branch/file')
+        self.assertPathDoesNotExist('branch/file')
 
     def do_delete_dir(self):
         return [('unversion', 'dir-id')]
 
     def check_dir_doesnt_exist(self):
-        self.failIfExists('branch/dir')
+        self.assertPathDoesNotExist('branch/dir')
+
+    def do_create_file_in_dir(self):
+        return [('add', ('dir', 'dir-id', 'directory', '')),
+                ('add', ('dir/file', 'file-id', 'file', 'trunk content\n'))]
+
+    def do_rename_file_in_dir(self):
+        return [('rename', ('dir/file', 'dir/new-file'))]
+
+    def check_file_in_dir_renamed(self):
+        self.assertPathDoesNotExist('branch/dir/file')
+        self.assertPathExists('branch/dir/new-file')
+
+    def check_file_in_dir_doesnt_exist(self):
+        self.assertPathDoesNotExist('branch/dir/file')
 
     def _get_resolve_path_arg(self, wt, action):
         tpath = self._this['path']
@@ -568,14 +665,10 @@
 
 class TestResolveDuplicateEntry(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.DuplicateEntry,
+    _conflict_type = conflicts.DuplicateEntry
 
-    @staticmethod
-    def scenarios():
-        # Each side dict additionally defines:
-        # - path involved
-        # - file-id involved
-        base_scenarios = [
+    scenarios = mirror_scenarios(
+        [
             # File created with different file-ids
             (dict(_base_actions='nothing'),
              ('filea_created',
@@ -584,8 +677,7 @@
              ('fileb_created',
               dict(actions='create_file_b', check='file_content_b',
                    path='file', file_id='file-b-id')),),
-            ]
-        return mirror_scenarios(base_scenarios)
+            ])
 
     def do_nothing(self):
         return []
@@ -625,20 +717,18 @@
     # tests MissingParent resolution :-/
     preamble = """
 $ bzr init trunk
+...
 $ cd trunk
 $ mkdir dir
-$ bzr add dir
-$ bzr commit -m 'Create trunk'
-
+$ bzr add -q dir
+$ bzr commit -m 'Create trunk' -q
 $ echo 'trunk content' >dir/file
-$ bzr add dir/file
-$ bzr commit -m 'Add dir/file in trunk'
-
-$ bzr branch . -r 1 ../branch
+$ bzr add -q dir/file
+$ bzr commit -q -m 'Add dir/file in trunk'
+$ bzr branch -q . -r 1 ../branch
 $ cd ../branch
-$ bzr rm dir
-$ bzr commit -m 'Remove dir in branch'
-
+$ bzr rm dir -q
+$ bzr commit -q -m 'Remove dir in branch'
 $ bzr merge ../trunk
 2>+N  dir/
 2>+N  dir/file
@@ -649,15 +739,17 @@
 
     def test_take_this(self):
         self.run_script("""
-$ bzr rm dir  --force
+$ bzr rm -q dir  --force
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_take_other(self):
         self.run_script("""
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
 
@@ -665,22 +757,20 @@
 
     preamble = """
 $ bzr init trunk
+...
 $ cd trunk
 $ mkdir dir
 $ echo 'trunk content' >dir/file
-$ bzr add
-$ bzr commit -m 'Create trunk'
-
+$ bzr add -q
+$ bzr commit -m 'Create trunk' -q
 $ echo 'trunk content' >dir/file2
-$ bzr add dir/file2
-$ bzr commit -m 'Add dir/file2 in branch'
-
-$ bzr branch . -r 1 ../branch
+$ bzr add -q dir/file2
+$ bzr commit -q -m 'Add dir/file2 in branch'
+$ bzr branch -q . -r 1 ../branch
 $ cd ../branch
-$ bzr rm dir/file --force
-$ bzr rm dir
-$ bzr commit -m 'Remove dir/file'
-
+$ bzr rm -q dir/file --force
+$ bzr rm -q dir
+$ bzr commit -q -m 'Remove dir/file'
 $ bzr merge ../trunk
 2>+N  dir/
 2>+N  dir/file2
@@ -692,34 +782,39 @@
     def test_keep_them_all(self):
         self.run_script("""
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_adopt_child(self):
         self.run_script("""
-$ bzr mv dir/file2 file2
-$ bzr rm dir --force
+$ bzr mv -q dir/file2 file2
+$ bzr rm -q dir --force
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_kill_them_all(self):
         self.run_script("""
-$ bzr rm dir --force
+$ bzr rm -q dir --force
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_this(self):
         self.run_script("""
 $ bzr resolve --take-this dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>...
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_other(self):
         self.run_script("""
 $ bzr resolve --take-other dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>...
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
 
@@ -727,22 +822,20 @@
 
     preamble = """
 $ bzr init trunk
+...
 $ cd trunk
 $ mkdir dir
 $ echo 'trunk content' >dir/file
-$ bzr add
-$ bzr commit -m 'Create trunk'
-
-$ bzr rm dir/file --force
-$ bzr rm dir --force
-$ bzr commit -m 'Remove dir/file'
-
-$ bzr branch . -r 1 ../branch
+$ bzr add -q
+$ bzr commit -m 'Create trunk' -q
+$ bzr rm -q dir/file --force
+$ bzr rm -q dir --force
+$ bzr commit -q -m 'Remove dir/file'
+$ bzr branch -q . -r 1 ../branch
 $ cd ../branch
 $ echo 'branch content' >dir/file2
-$ bzr add dir/file2
-$ bzr commit -m 'Add dir/file2 in branch'
-
+$ bzr add -q dir/file2
+$ bzr commit -q -m 'Add dir/file2 in branch'
 $ bzr merge ../trunk
 2>-D  dir/file
 2>Conflict: can't delete dir because it is not empty.  Not deleting.
@@ -753,52 +846,58 @@
     def test_keep_them_all(self):
         self.run_script("""
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_adopt_child(self):
         self.run_script("""
-$ bzr mv dir/file2 file2
-$ bzr rm dir --force
+$ bzr mv -q dir/file2 file2
+$ bzr rm -q dir --force
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_kill_them_all(self):
         self.run_script("""
-$ bzr rm dir --force
+$ bzr rm -q dir --force
 $ bzr resolve dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_this(self):
         self.run_script("""
 $ bzr resolve --take-this dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_other(self):
         self.run_script("""
 $ bzr resolve --take-other dir
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>deleted dir/file2
+2>deleted dir
+2>2 conflicts resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
 
 class TestResolveParentLoop(TestParametrizedResolveConflicts):
 
-    _conflict_type = conflicts.ParentLoop,
+    _conflict_type = conflicts.ParentLoop
 
     _this_args = None
     _other_args = None
 
-    @staticmethod
-    def scenarios():
-        # Each side dict additionally defines:
-        # - dir_id: the directory being moved
-        # - target_id: The target directory
-        # - xfail: whether the test is expected to fail if the action is
-        #     involved as 'other'
-        base_scenarios = [
+    # Each side dict additionally defines:
+    # - dir_id: the directory being moved
+    # - target_id: The target directory
+    # - xfail: whether the test is expected to fail if the action is
+    #   involved as 'other'
+    scenarios = mirror_scenarios(
+        [
             # Dirs moved into each other
             (dict(_base_actions='create_dir1_dir2'),
              ('dir1_into_dir2',
@@ -815,8 +914,7 @@
              ('dir3_into_dir2',
               dict(actions='move_dir3_into_dir2', check='dir3_4_moved',
                    dir_id='dir3-id', target_id='dir2-id', xfail=True))),
-            ]
-        return mirror_scenarios(base_scenarios)
+            ])
 
     def do_create_dir1_dir2(self):
         return [('add', ('dir1', 'dir1-id', 'directory', '')),
@@ -826,15 +924,15 @@
         return [('rename', ('dir1', 'dir2/dir1'))]
 
     def check_dir1_moved(self):
-        self.failIfExists('branch/dir1')
-        self.failUnlessExists('branch/dir2/dir1')
+        self.assertPathDoesNotExist('branch/dir1')
+        self.assertPathExists('branch/dir2/dir1')
 
     def do_move_dir2_into_dir1(self):
         return [('rename', ('dir2', 'dir1/dir2'))]
 
     def check_dir2_moved(self):
-        self.failIfExists('branch/dir2')
-        self.failUnlessExists('branch/dir1/dir2')
+        self.assertPathDoesNotExist('branch/dir2')
+        self.assertPathExists('branch/dir1/dir2')
 
     def do_create_dir1_4(self):
         return [('add', ('dir1', 'dir1-id', 'directory', '')),
@@ -846,17 +944,17 @@
         return [('rename', ('dir1', 'dir3/dir4/dir1'))]
 
     def check_dir1_2_moved(self):
-        self.failIfExists('branch/dir1')
-        self.failUnlessExists('branch/dir3/dir4/dir1')
-        self.failUnlessExists('branch/dir3/dir4/dir1/dir2')
+        self.assertPathDoesNotExist('branch/dir1')
+        self.assertPathExists('branch/dir3/dir4/dir1')
+        self.assertPathExists('branch/dir3/dir4/dir1/dir2')
 
     def do_move_dir3_into_dir2(self):
         return [('rename', ('dir3', 'dir1/dir2/dir3'))]
 
     def check_dir3_4_moved(self):
-        self.failIfExists('branch/dir3')
-        self.failUnlessExists('branch/dir1/dir2/dir3')
-        self.failUnlessExists('branch/dir1/dir2/dir3/dir4')
+        self.assertPathDoesNotExist('branch/dir3')
+        self.assertPathExists('branch/dir1/dir2/dir3')
+        self.assertPathExists('branch/dir1/dir2/dir3/dir4')
 
     def _get_resolve_path_arg(self, wt, action):
         # ParentLoop says: moving <conflict_path> into <path>. Cancelled move.
@@ -873,7 +971,7 @@
         if self._other['xfail']:
             # It's a bit hackish to raise from here relying on being called for
             # both tests but this avoid overriding test_resolve_taking_other
-            raise tests.KnownFailure(
+            self.knownFailure(
                 "ParentLoop doesn't carry enough info to resolve --take-other")
     _assert_conflict = assertParentLoop
 
@@ -882,19 +980,19 @@
 
     preamble = """
 $ bzr init trunk
+...
 $ cd trunk
 $ bzr mkdir foo
-$ bzr commit -m 'Create trunk'
+...
+$ bzr commit -m 'Create trunk' -q
 $ echo "Boing" >foo/bar
-$ bzr add foo/bar
-$ bzr commit -m 'Add foo/bar'
-
-$ bzr branch . -r 1 ../branch
+$ bzr add -q foo/bar
+$ bzr commit -q -m 'Add foo/bar'
+$ bzr branch -q . -r 1 ../branch
 $ cd ../branch
 $ rm -r foo
 $ echo "Boo!" >foo
-$ bzr commit -m 'foo is now a file'
-
+$ bzr commit -q -m 'foo is now a file'
 $ bzr merge ../trunk
 2>+N  foo.new/bar
 2>RK  foo => foo.new/
@@ -906,32 +1004,36 @@
 
     def test_take_this(self):
         self.run_script("""
-$ bzr rm foo.new --force
+$ bzr rm -q foo.new --force
 # FIXME: Isn't it weird that foo is now unkown even if foo.new has been put
 # aside ? -- vila 090916
-$ bzr add foo
+$ bzr add -q foo
 $ bzr resolve foo.new
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>1 conflict resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_take_other(self):
         self.run_script("""
-$ bzr rm foo --force
-$ bzr mv foo.new foo
+$ bzr rm -q foo --force
+$ bzr mv -q foo.new foo
 $ bzr resolve foo
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>1 conflict resolved, 0 remaining
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_this(self):
         self.run_script("""
 $ bzr resolve --take-this foo.new
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>...
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_other(self):
         self.run_script("""
 $ bzr resolve --take-other foo.new
-$ bzr commit --strict -m 'No more conflicts nor unknown files'
+2>...
+$ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
 
@@ -943,24 +1045,109 @@
         # conflict.
         self.run_script("""
 $ bzr init trunk
+...
 $ cd trunk
 $ bzr mkdir foo
-$ bzr commit -m 'Create trunk'
+...
+$ bzr commit -m 'Create trunk' -q
 $ rm -r foo
 $ echo "Boo!" >foo
-$ bzr commit -m 'foo is now a file'
-
-$ bzr branch . -r 1 ../branch
+$ bzr commit -m 'foo is now a file' -q
+$ bzr branch -q . -r 1 ../branch -q
 $ cd ../branch
 $ echo "Boing" >foo/bar
-$ bzr add foo/bar
-$ bzr commit -m 'Add foo/bar'
-
+$ bzr add -q foo/bar -q
+$ bzr commit -m 'Add foo/bar' -q
 $ bzr merge ../trunk
 2>bzr: ERROR: Tree transform is malformed [('unversioned executability', 'new-1')]
 """)
 
 
+class TestNoFinalPath(script.TestCaseWithTransportAndScript):
+
+    def test_bug_805809(self):
+        self.run_script("""
+$ bzr init trunk
+Created a standalone tree (format: 2a)
+$ cd trunk
+$ echo trunk >file
+$ bzr add
+adding file
+$ bzr commit -m 'create file on trunk'
+2>Committing to: .../trunk/
+2>added file
+2>Committed revision 1.
+# Create a debian branch based on trunk
+$ cd ..
+$ bzr branch trunk -r 1 debian
+2>Branched 1 revision.
+$ cd debian
+$ mkdir dir
+$ bzr add
+adding dir
+$ bzr mv file dir
+file => dir/file
+$ bzr commit -m 'rename file to dir/file for debian'
+2>Committing to: .../debian/
+2>added dir
+2>renamed file => dir/file
+2>Committed revision 2.
+# Create an experimental branch with a new root-id
+$ cd ..
+$ bzr init experimental
+Created a standalone tree (format: 2a)
+$ cd experimental
+# Work around merging into empty branch not being supported
+# (http://pad.lv/308562)
+$ echo something >not-empty
+$ bzr add
+adding not-empty
+$ bzr commit -m 'Add some content in experimental'
+2>Committing to: .../experimental/
+2>added not-empty
+2>Committed revision 1.
+# merge debian even without a common ancestor
+$ bzr merge ../debian -r0..2
+2>+N  dir/
+2>+N  dir/file
+2>All changes applied successfully.
+$ bzr commit -m 'merging debian into experimental'
+2>Committing to: .../experimental/
+2>added dir
+2>added dir/file
+2>Committed revision 2.
+# Create an ubuntu branch with yet another root-id
+$ cd ..
+$ bzr init ubuntu
+Created a standalone tree (format: 2a)
+$ cd ubuntu
+# Work around merging into empty branch not being supported
+# (http://pad.lv/308562)
+$ echo something >not-empty-ubuntu
+$ bzr add
+adding not-empty-ubuntu
+$ bzr commit -m 'Add some content in experimental'
+2>Committing to: .../ubuntu/
+2>added not-empty-ubuntu
+2>Committed revision 1.
+# Also merge debian
+$ bzr merge ../debian -r0..2
+2>+N  dir/
+2>+N  dir/file
+2>All changes applied successfully.
+$ bzr commit -m 'merging debian'
+2>Committing to: .../ubuntu/
+2>added dir
+2>added dir/file
+2>Committed revision 2.
+# Now try to merge experimental
+$ bzr merge ../experimental
+2>+N  not-empty
+2>Path conflict: dir / dir
+2>1 conflicts encountered.
+""")
+
+
 class TestResolveActionOption(tests.TestCase):
 
     def setUp(self):

=== added file 'bzrlib/tests/test_controldir.py'
--- bzrlib/tests/test_controldir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_controldir.py	2011-03-22 12:10:34 +0000
@@ -0,0 +1,237 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for the ControlDir facility.
+
+For interface contract tests, see tests/per_control_dir.
+"""
+
+from bzrlib import (
+    controldir,
+    errors,
+    tests,
+    ui,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class SampleComponentFormat(controldir.ControlComponentFormat):
+
+    def get_format_string(self):
+        return "Example component format."
+
+
+class SampleExtraComponentFormat(controldir.ControlComponentFormat):
+    """Extra format, no format string."""
+
+
+class TestMetaComponentFormatRegistry(tests.TestCase):
+
+    def setUp(self):
+        super(TestMetaComponentFormatRegistry, self).setUp()
+        self.registry = controldir.ControlComponentFormatRegistry()
+
+    def test_register_unregister_format(self):
+        format = SampleComponentFormat()
+        self.registry.register(format)
+        self.assertEquals(format,
+            self.registry.get("Example component format."))
+        self.registry.remove(format)
+        self.assertRaises(KeyError, self.registry.get,
+            "Example component format.")
+
+    def test_get_all(self):
+        format = SampleComponentFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_get_all_modules(self):
+        format = SampleComponentFormat()
+        self.assertEquals(set(), self.registry._get_all_modules())
+        self.registry.register(format)
+        self.assertEquals(
+            set(["bzrlib.tests.test_controldir"]),
+            self.registry._get_all_modules())
+
+    def test_register_extra(self):
+        format = SampleExtraComponentFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra_lazy(self):
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra_lazy("bzrlib.tests.test_controldir",
+            "SampleExtraComponentFormat")
+        formats = self.registry._get_all()
+        self.assertEquals(1, len(formats))
+        self.assertIsInstance(formats[0], SampleExtraComponentFormat)
+
+
+class TestControlDirFormatDeprecated(tests.TestCaseWithTransport):
+    """Tests for removed registration method in the ControlDirFormat facility."""
+
+    def test_register_format(self):
+        self.assertRaises(errors.BzrError,
+            controldir.ControlDirFormat.register_format, object())
+
+
+class TestProber(tests.TestCaseWithTransport):
+    """Per-prober tests."""
+
+    scenarios = [
+        (prober_cls.__name__, {'prober_cls': prober_cls})
+        for prober_cls in controldir.ControlDirFormat._probers]
+
+    def setUp(self):
+        super(TestProber, self).setUp()
+        self.prober = self.prober_cls()
+
+    def test_probe_transport_empty(self):
+        transport = self.get_transport(".")
+        self.assertRaises(errors.NotBranchError,
+            self.prober.probe_transport, transport)
+
+    def test_known_formats(self):
+        known_formats = self.prober_cls.known_formats()
+        self.assertIsInstance(known_formats, set)
+        for format in known_formats:
+            self.assertIsInstance(format, controldir.ControlDirFormat,
+                repr(format))
+
+
+class NotBzrDir(controldir.ControlDir):
+    """A non .bzr based control directory."""
+
+    def __init__(self, transport, format):
+        self._format = format
+        self.root_transport = transport
+        self.transport = transport.clone('.not')
+
+
+class NotBzrDirFormat(controldir.ControlDirFormat):
+    """A test class representing any non-.bzr based disk format."""
+
+    def initialize_on_transport(self, transport):
+        """Initialize a new .not dir in the base directory of a Transport."""
+        transport.mkdir('.not')
+        return self.open(transport)
+
+    def open(self, transport):
+        """Open this directory."""
+        return NotBzrDir(transport, self)
+
+
+class NotBzrDirProber(controldir.Prober):
+
+    def probe_transport(self, transport):
+        """Our format is present if the transport ends in '.not/'."""
+        if transport.has('.not'):
+            return NotBzrDirFormat()
+
+    @classmethod
+    def known_formats(cls):
+        return set([NotBzrDirFormat()])
+
+
+class TestNotBzrDir(tests.TestCaseWithTransport):
+    """Tests for using the controldir api with a non .bzr based disk format.
+
+    If/when one of these is in the core, we can let the implementation tests
+    verify this works.
+    """
+
+    def test_create_and_find_format(self):
+        # create a .notbzr dir
+        format = NotBzrDirFormat()
+        dir = format.initialize(self.get_url())
+        self.assertIsInstance(dir, NotBzrDir)
+        # now probe for it.
+        controldir.ControlDirFormat.register_prober(NotBzrDirProber)
+        try:
+            found = controldir.ControlDirFormat.find_format(self.get_transport())
+            self.assertIsInstance(found, NotBzrDirFormat)
+        finally:
+            controldir.ControlDirFormat.unregister_prober(NotBzrDirProber)
+
+    def test_included_in_known_formats(self):
+        controldir.ControlDirFormat.register_prober(NotBzrDirProber)
+        self.addCleanup(controldir.ControlDirFormat.unregister_prober, NotBzrDirProber)
+        formats = controldir.ControlDirFormat.known_formats()
+        self.assertIsInstance(formats, set)
+        for format in formats:
+            if isinstance(format, NotBzrDirFormat):
+                break
+        else:
+            self.fail("No NotBzrDirFormat in %s" % formats)
+
+
+class UnsupportedControlComponentFormat(controldir.ControlComponentFormat):
+
+    def is_supported(self):
+        return False
+
+
+class OldControlComponentFormat(controldir.ControlComponentFormat):
+
+    def get_format_description(self):
+        return "An old format that is slow"
+
+    upgrade_recommended = True
+
+
+class DefaultControlComponentFormatTests(tests.TestCase):
+    """Tests for default ControlComponentFormat implementation."""
+
+    def test_check_support_status_unsupported(self):
+        self.assertRaises(errors.UnsupportedFormatError,
+            UnsupportedControlComponentFormat().check_support_status,
+            allow_unsupported=False)
+        UnsupportedControlComponentFormat().check_support_status(
+            allow_unsupported=True)
+
+    def test_check_support_status_supported(self):
+        controldir.ControlComponentFormat().check_support_status(
+            allow_unsupported=False)
+        controldir.ControlComponentFormat().check_support_status(
+            allow_unsupported=True)
+
+    def test_recommend_upgrade_current_format(self):
+        stderr = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stderr=stderr)
+        format = controldir.ControlComponentFormat()
+        format.check_support_status(allow_unsupported=False,
+            recommend_upgrade=True)
+        self.assertEquals("", stderr.getvalue())
+
+    def test_recommend_upgrade_old_format(self):
+        stderr = tests.StringIOWrapper()
+        ui.ui_factory = tests.TestUIFactory(stderr=stderr)
+        format = OldControlComponentFormat()
+        format.check_support_status(allow_unsupported=False,
+            recommend_upgrade=False)
+        self.assertEquals("", stderr.getvalue())
+        format.check_support_status(allow_unsupported=False,
+            recommend_upgrade=True, basedir='apath')
+        self.assertEquals(
+            'An old format that is slow is deprecated and a better format '
+            'is available.\nIt is recommended that you upgrade by running '
+            'the command\n  bzr upgrade apath\n',
+            stderr.getvalue())

=== modified file 'bzrlib/tests/test_crash.py'
--- bzrlib/tests/test_crash.py	2010-04-08 04:34:03 +0000
+++ bzrlib/tests/test_crash.py	2011-04-18 03:45:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,18 +15,16 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
+import doctest
+import os
 from StringIO import StringIO
 import sys
 
-
-import os
-
-
 from bzrlib import (
     config,
     crash,
     osutils,
-    symbol_versioning,
+    plugin,
     tests,
     )
 
@@ -40,9 +38,14 @@
     def test_apport_report(self):
         crash_dir = osutils.joinpath((self.test_base_dir, 'crash'))
         os.mkdir(crash_dir)
-        os.environ['APPORT_CRASH_DIR'] = crash_dir
+        self.overrideEnv('APPORT_CRASH_DIR', crash_dir)
         self.assertEquals(crash_dir, config.crash_dir())
 
+        self.overrideAttr(
+            plugin,
+            'plugin_warnings',
+            {'example': ['Failed to load plugin foo']})
+
         stderr = StringIO()
 
         try:
@@ -72,3 +75,38 @@
         self.assertContainsRe(report, 'test_apport_report')
         # should also be in there
         self.assertContainsRe(report, '(?m)^CommandLine:')
+        self.assertContainsRe(
+            report,
+            'Failed to load plugin foo')
+
+
+class TestNonApportReporting(tests.TestCase):
+    """Reporting of crash-type bugs without apport.
+    
+    This should work in all environments.
+    """
+
+    def setup_fake_plugins(self):
+        def fake_plugins():
+            fake = plugin.PlugIn('fake_plugin', plugin)
+            fake.version_info = lambda: (1, 2, 3)
+            return {"fake_plugin": fake}
+        self.overrideAttr(plugin, 'plugins', fake_plugins)
+
+    def test_report_bug_legacy(self):
+        self.setup_fake_plugins()
+        err_file = StringIO()
+        try:
+            raise AssertionError("my error")
+        except AssertionError, e:
+            pass
+        crash.report_bug_legacy(sys.exc_info(), err_file)
+        report = err_file.getvalue()
+        for needle in [
+            "bzr: ERROR: exceptions.AssertionError: my error",
+            r"Traceback \(most recent call last\):",
+            r"plugins: fake_plugin\[1\.2\.3\]",
+            ]:
+            self.assertContainsRe(
+                    report,
+                    needle)

=== modified file 'bzrlib/tests/test_debug.py'
--- bzrlib/tests/test_debug.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_debug.py	2011-08-12 16:25:56 +0000
@@ -17,14 +17,11 @@
 """Tests for bzrlib.debug"""
 
 
-import os
-
-
 from bzrlib import (
+    config,
     debug,
     tests,
     )
-from bzrlib.config import config_filename, ensure_config_dir_exists
 
 
 class TestDebugFlags(tests.TestCaseInTempDir):
@@ -32,33 +29,13 @@
     def test_set_debug_flags_from_config(self):
         # test both combinations because configobject automatically splits up
         # comma-separated lists
-        if os.path.isfile(config_filename()):
-            # Something is wrong in environment,
-            # we risk overwriting users config
-            self.assert_(config_filename() + "exists, abort")
-
-        self.try_debug_flags(
-            """debug_flags = hpss, error\n""",
-            set(['hpss', 'error']))
-
-        self.try_debug_flags(
-            """debug_flags = hpss\n""",
-            set(['hpss']))
-
-    def try_debug_flags(self, conf_bytes, expected_flags):
-        ensure_config_dir_exists()
-        f = open(config_filename(), 'wb')
-        try:
-            f.write(conf_bytes)
-        finally:
-            f.close()
-        saved_debug = set(debug.debug_flags)
-        debug.debug_flags.clear()
-        try:
-            debug.set_debug_flags_from_config()
-            self.assertEqual(expected_flags,
-                debug.debug_flags)
-        finally:
-            # restore without rebinding the variable
-            debug.debug_flags.clear()
-            debug.debug_flags.update(saved_debug)
+        self.assertDebugFlags(['hpss', 'error'], 'debug_flags = hpss, error\n')
+        self.assertDebugFlags(['hpss'], 'debug_flags = hpss\n')
+
+    def assertDebugFlags(self, expected_flags, conf_bytes):
+        conf = config.GlobalStack()
+        conf.store._load_from_string(conf_bytes)
+        conf.store.save()
+        self.overrideAttr(debug, 'debug_flags', set())
+        debug.set_debug_flags_from_config()
+        self.assertEqual(set(expected_flags), debug.debug_flags)

=== modified file 'bzrlib/tests/test_decorators.py'
--- bzrlib/tests/test_decorators.py	2010-04-23 08:51:52 +0000
+++ bzrlib/tests/test_decorators.py	2011-02-14 11:57:45 +0000
@@ -27,11 +27,13 @@
     pass
 
 
-def create_decorator_sample(style, unlock_error=None):
+def create_decorator_sample(style, unlock_error=None, meth=None):
     """Create a DecoratorSample object, using specific lock operators.
 
     :param style: The type of lock decorators to use (fast/pretty/None)
     :param unlock_error: If specified, an error to raise from unlock.
+    :param meth: a function to be decorated and added as a 'meth_read' and
+        'meth_write' to the object.
     :return: An instantiated DecoratorSample object.
     """
 
@@ -92,6 +94,10 @@
             self.actions.append('fail_during_write')
             raise TypeError('during write')
 
+        if meth is not None:
+            meth_read = needs_read_lock(meth)
+            meth_write = needs_write_lock(meth)
+
     return DecoratorSample()
 
 
@@ -149,6 +155,20 @@
         self.assertEqual(['lock_write', ('bank', 'bar', 'bing'),
                           'unlock_fail'], sam.actions)
 
+    def test_read_lock_preserves_default_str_kwarg_identity(self):
+        a_constant = 'A str used as a constant'
+        def meth(self, param=a_constant):
+            return param
+        sam = create_decorator_sample(self._decorator_style, meth=meth)
+        self.assertIs(a_constant, sam.meth_read())
+
+    def test_write_lock_preserves_default_str_kwarg_identity(self):
+        a_constant = 'A str used as a constant'
+        def meth(self, param=a_constant):
+            return param
+        sam = create_decorator_sample(self._decorator_style, meth=meth)
+        self.assertIs(a_constant, sam.meth_write())
+
 
 class TestFastDecoratorActions(TestDecoratorActions):
 

=== modified file 'bzrlib/tests/test_delta.py'
--- bzrlib/tests/test_delta.py	2010-04-26 13:51:08 +0000
+++ bzrlib/tests/test_delta.py	2010-10-20 14:38:53 +0000
@@ -65,8 +65,7 @@
         reporter.report(file_id, (old_path, path), versioned_change, renamed,
             modified, exe_change, kind)
         if expected_lines is not None:
-            for i in range(len(expected_lines)):
-                self.assertEqualDiff(expected_lines[i], result[i])
+            self.assertEqualDiff('\n'.join(expected_lines), '\n'.join(result))
         else:
             self.assertEqual([], result)
 
@@ -123,6 +122,12 @@
             renamed=False, modified='created', exe_change=False,
             kind=(None, 'file'), unversioned_filter=lambda x:True)
 
+    def test_missing(self):
+        self.assertReport('+!  missing.c', file_id=None, path='missing.c',
+             old_path=None, versioned_change='added',
+             renamed=False, modified='missing', exe_change=False,
+             kind=(None, None))
+
     def test_view_filtering(self):
         # If a file in within the view, it should appear in the output
         expected_lines = [
@@ -280,24 +285,32 @@
                                   ('branch/f2', '2\n'),
                                   ('branch/f3', '3\n'),
                                   ('branch/f4', '4\n'),
+                                  ('branch/f5', '5\n'),
                                   ('branch/dir/',),
                                  ])
         wt.add(['f1', 'f2', 'f3', 'f4', 'dir'],
                ['f1-id', 'f2-id', 'f3-id', 'f4-id', 'dir-id'])
         wt.commit('commit one', rev_id='1')
 
+        # TODO add rename,removed,etc. here?
+        wt.add('f5')
+        os.unlink('branch/f5')
+
         long_status = """added:
   dir/
   f1
   f2
   f3
   f4
+missing:
+  f5
 """
         short_status = """A  dir/
 A  f1
 A  f2
 A  f3
 A  f4
+!  f5
 """
 
         repo = wt.branch.repository

=== removed file 'bzrlib/tests/test_deprecated_graph.py'
--- bzrlib/tests/test_deprecated_graph.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_deprecated_graph.py	1970-01-01 00:00:00 +0000
@@ -1,80 +0,0 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-from bzrlib.tests import TestCase
-from bzrlib.deprecated_graph import node_distances, nodes_by_distance, Graph
-
-
-class TestBase(TestCase):
-
-    def edge_add(self, *args):
-        for start, end in zip(args[:-1], args[1:]):
-            if start not in self.graph:
-                self.graph[start] = {}
-            if end not in self.graph:
-                self.graph[end] = {}
-            self.graph[start][end] = 1
-
-    def setUp(self):
-        TestCase.setUp(self)
-        self.graph = {}
-        self.edge_add('A', 'B', 'C', 'D')
-        self.edge_add('A', 'E', 'F', 'C')
-        self.edge_add('A', 'G', 'H', 'I', 'B')
-        self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
-        self.edge_add('O', 'N')
-
-    def node_descendants(self):
-        descendants = {'A':set()}
-        for node in self.graph:
-            for ancestor in self.graph[node]:
-                if ancestor not in descendants:
-                    descendants[ancestor] = set()
-                descendants[ancestor].add(node)
-        return descendants
-
-    def test_distances(self):
-        descendants = self.node_descendants()
-        distances = node_distances(self.graph, descendants, 'A')
-        nodes = nodes_by_distance(distances)
-        self.assertEqual(nodes[0], 'D')
-        self.assert_(nodes[1] in ('N', 'C'))
-        self.assert_(nodes[2] in ('N', 'C'))
-        self.assert_(nodes[3] in ('B', 'M'))
-        self.assert_(nodes[4] in ('B', 'M'))
-
-        #Ensure we don't shortcut through B when there's only a difference of
-        # 1 in distance
-        self.graph = {}
-        self.edge_add('A', 'B', 'C')
-        self.edge_add('A', 'D', 'E', 'C')
-        descendants = self.node_descendants()
-        distances = node_distances(self.graph, descendants, 'A')
-        self.assertEqual(distances['C'], 3)
-
-
-class TestGraph(TestCase):
-
-    def test_get_descendants(self):
-        # Graph objects let you get a descendants graph in
-        # node: {direct-children:distance} which contains
-        # known children, including ghost children
-        graph = Graph()
-        graph.add_ghost('ghost')
-        graph.add_node('rev1', ['ghost'])
-        # check the result contains ghosts:
-        self.assertEqual({'ghost': {'rev1': 1}, 'rev1': {}},
-                         graph.get_descendants())

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2010-05-26 15:58:08 +0000
+++ bzrlib/tests/test_diff.py	2011-07-21 06:46:34 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,29 +33,11 @@
     transform,
     )
 from bzrlib.symbol_versioning import deprecated_in
-from bzrlib.tests import features
+from bzrlib.tests import features, EncodingAdapter
 from bzrlib.tests.blackbox.test_diff import subst_dates
-
-
-class _AttribFeature(tests.Feature):
-
-    def _probe(self):
-        if (sys.platform not in ('cygwin', 'win32')):
-            return False
-        try:
-            proc = subprocess.Popen(['attrib', '.'], stdout=subprocess.PIPE)
-        except OSError, e:
-            return False
-        return (0 == proc.wait())
-
-    def feature_name(self):
-        return 'attrib Windows command-line tool'
-
-AttribFeature = _AttribFeature()
-
-
-compiled_patiencediff_feature = tests.ModuleAvailableFeature(
-                                    'bzrlib._patiencediff_c')
+from bzrlib.tests import (
+    features,
+    )
 
 
 def udiff_lines(old, new, allow_binary=False):
@@ -144,31 +126,22 @@
         self.check_patch(lines)
 
     def test_external_diff_binary_lang_c(self):
-        old_env = {}
         for lang in ('LANG', 'LC_ALL', 'LANGUAGE'):
-            old_env[lang] = osutils.set_or_unset_env(lang, 'C')
-        try:
-            lines = external_udiff_lines(['\x00foobar\n'], ['foo\x00bar\n'])
-            # Older versions of diffutils say "Binary files", newer
-            # versions just say "Files".
-            self.assertContainsRe(lines[0],
-                                  '(Binary f|F)iles old and new differ\n')
-            self.assertEquals(lines[1:], ['\n'])
-        finally:
-            for lang, old_val in old_env.iteritems():
-                osutils.set_or_unset_env(lang, old_val)
+            self.overrideEnv(lang, 'C')
+        lines = external_udiff_lines(['\x00foobar\n'], ['foo\x00bar\n'])
+        # Older versions of diffutils say "Binary files", newer
+        # versions just say "Files".
+        self.assertContainsRe(lines[0], '(Binary f|F)iles old and new differ\n')
+        self.assertEquals(lines[1:], ['\n'])
 
     def test_no_external_diff(self):
         """Check that NoDiff is raised when diff is not available"""
-        # Use os.environ['PATH'] to make sure no 'diff' command is available
-        orig_path = os.environ['PATH']
-        try:
-            os.environ['PATH'] = ''
-            self.assertRaises(errors.NoDiff, diff.external_diff,
-                              'old', ['boo\n'], 'new', ['goo\n'],
-                              StringIO(), diff_opts=['-u'])
-        finally:
-            os.environ['PATH'] = orig_path
+        # Make sure no 'diff' command is available
+        # XXX: Weird, using None instead of '' breaks the test -- vila 20101216
+        self.overrideEnv('PATH', '')
+        self.assertRaises(errors.NoDiff, diff.external_diff,
+                          'old', ['boo\n'], 'new', ['goo\n'],
+                          StringIO(), diff_opts=['-u'])
 
     def test_internal_diff_default(self):
         # Default internal diff encoding is utf8
@@ -235,7 +208,7 @@
         output = StringIO.StringIO()
         diff.internal_diff(u'old_\xb5', ['old_text\n'],
                             u'new_\xe5', ['new_text\n'], output)
-        self.failUnless(isinstance(output.getvalue(), str),
+        self.assertIsInstance(output.getvalue(), str,
             'internal_diff should return bytestrings')
 
 
@@ -258,23 +231,20 @@
         self.assertEqual(out.splitlines(True) + ['\n'], lines)
 
 
-class TestShowDiffTreesHelper(tests.TestCaseWithTransport):
-    """Has a helper for running show_diff_trees"""
-
-    def get_diff(self, tree1, tree2, specific_files=None, working_tree=None):
-        output = StringIO()
-        if working_tree is not None:
-            extra_trees = (working_tree,)
-        else:
-            extra_trees = ()
-        diff.show_diff_trees(tree1, tree2, output,
-                             specific_files=specific_files,
-                             extra_trees=extra_trees, old_label='old/',
-                             new_label='new/')
-        return output.getvalue()
-
-
-class TestDiffDates(TestShowDiffTreesHelper):
+def get_diff_as_string(tree1, tree2, specific_files=None, working_tree=None):
+    output = StringIO()
+    if working_tree is not None:
+        extra_trees = (working_tree,)
+    else:
+        extra_trees = ()
+    diff.show_diff_trees(tree1, tree2, output,
+        specific_files=specific_files,
+        extra_trees=extra_trees, old_label='old/',
+        new_label='new/')
+    return output.getvalue()
+
+
+class TestDiffDates(tests.TestCaseWithTransport):
 
     def setUp(self):
         super(TestDiffDates, self).setUp()
@@ -315,7 +285,7 @@
         os.utime('file1', (1144195200, 1144195200)) # 2006-04-05 00:00:00 UTC
 
     def test_diff_rev_tree_working_tree(self):
-        output = self.get_diff(self.wt.basis_tree(), self.wt)
+        output = get_diff_as_string(self.wt.basis_tree(), self.wt)
         # note that the date for old/file1 is from rev 2 rather than from
         # the basis revision (rev 4)
         self.assertEqualDiff(output, '''\
@@ -331,7 +301,7 @@
     def test_diff_rev_tree_rev_tree(self):
         tree1 = self.b.repository.revision_tree('rev-2')
         tree2 = self.b.repository.revision_tree('rev-3')
-        output = self.get_diff(tree1, tree2)
+        output = get_diff_as_string(tree1, tree2)
         self.assertEqualDiff(output, '''\
 === modified file 'file2'
 --- old/file2\t2006-04-01 00:00:00 +0000
@@ -345,7 +315,7 @@
     def test_diff_add_files(self):
         tree1 = self.b.repository.revision_tree(_mod_revision.NULL_REVISION)
         tree2 = self.b.repository.revision_tree('rev-1')
-        output = self.get_diff(tree1, tree2)
+        output = get_diff_as_string(tree1, tree2)
         # the files have the epoch time stamp for the tree in which
         # they don't exist.
         self.assertEqualDiff(output, '''\
@@ -366,7 +336,7 @@
     def test_diff_remove_files(self):
         tree1 = self.b.repository.revision_tree('rev-3')
         tree2 = self.b.repository.revision_tree('rev-4')
-        output = self.get_diff(tree1, tree2)
+        output = get_diff_as_string(tree1, tree2)
         # the file has the epoch time stamp for the tree in which
         # it doesn't exist.
         self.assertEqualDiff(output, '''\
@@ -383,7 +353,7 @@
         self.wt.rename_one('file1', 'file1b')
         old_tree = self.b.repository.revision_tree('rev-1')
         new_tree = self.b.repository.revision_tree('rev-4')
-        out = self.get_diff(old_tree, new_tree, specific_files=['file1b'],
+        out = get_diff_as_string(old_tree, new_tree, specific_files=['file1b'],
                             working_tree=self.wt)
         self.assertContainsRe(out, 'file1\t')
 
@@ -395,16 +365,15 @@
         self.wt.rename_one('file1', 'dir1/file1')
         old_tree = self.b.repository.revision_tree('rev-1')
         new_tree = self.b.repository.revision_tree('rev-4')
-        out = self.get_diff(old_tree, new_tree, specific_files=['dir1'],
+        out = get_diff_as_string(old_tree, new_tree, specific_files=['dir1'],
                             working_tree=self.wt)
         self.assertContainsRe(out, 'file1\t')
-        out = self.get_diff(old_tree, new_tree, specific_files=['dir2'],
+        out = get_diff_as_string(old_tree, new_tree, specific_files=['dir2'],
                             working_tree=self.wt)
         self.assertNotContainsRe(out, 'file1\t')
 
 
-
-class TestShowDiffTrees(TestShowDiffTreesHelper):
+class TestShowDiffTrees(tests.TestCaseWithTransport):
     """Direct tests for show_diff_trees"""
 
     def test_modified_file(self):
@@ -415,7 +384,7 @@
         tree.commit('one', rev_id='rev-1')
 
         self.build_tree_contents([('tree/file', 'new contents\n')])
-        d = self.get_diff(tree.basis_tree(), tree)
+        d = get_diff_as_string(tree.basis_tree(), tree)
         self.assertContainsRe(d, "=== modified file 'file'\n")
         self.assertContainsRe(d, '--- old/file\t')
         self.assertContainsRe(d, '\\+\\+\\+ new/file\t')
@@ -432,7 +401,7 @@
 
         tree.rename_one('dir', 'other')
         self.build_tree_contents([('tree/other/file', 'new contents\n')])
-        d = self.get_diff(tree.basis_tree(), tree)
+        d = get_diff_as_string(tree.basis_tree(), tree)
         self.assertContainsRe(d, "=== renamed directory 'dir' => 'other'\n")
         self.assertContainsRe(d, "=== modified file 'other/file'\n")
         # XXX: This is technically incorrect, because it used to be at another
@@ -451,7 +420,7 @@
         tree.commit('one', rev_id='rev-1')
 
         tree.rename_one('dir', 'newdir')
-        d = self.get_diff(tree.basis_tree(), tree)
+        d = get_diff_as_string(tree.basis_tree(), tree)
         # Renaming a directory should be a single "you renamed this dir" even
         # when there are files inside.
         self.assertEqual(d, "=== renamed directory 'dir' => 'newdir'\n")
@@ -464,7 +433,7 @@
         tree.commit('one', rev_id='rev-1')
 
         tree.rename_one('file', 'newname')
-        d = self.get_diff(tree.basis_tree(), tree)
+        d = get_diff_as_string(tree.basis_tree(), tree)
         self.assertContainsRe(d, "=== renamed file 'file' => 'newname'\n")
         # We shouldn't have a --- or +++ line, because there is no content
         # change
@@ -479,7 +448,7 @@
 
         tree.rename_one('file', 'newname')
         self.build_tree_contents([('tree/newname', 'new contents\n')])
-        d = self.get_diff(tree.basis_tree(), tree)
+        d = get_diff_as_string(tree.basis_tree(), tree)
         self.assertContainsRe(d, "=== renamed file 'file' => 'newname'\n")
         self.assertContainsRe(d, '--- old/file\t')
         self.assertContainsRe(d, '\\+\\+\\+ new/newname\t')
@@ -509,7 +478,7 @@
         tree.rename_one('c', 'new-c')
         tree.rename_one('d', 'new-d')
 
-        d = self.get_diff(tree.basis_tree(), tree)
+        d = get_diff_as_string(tree.basis_tree(), tree)
 
         self.assertContainsRe(d, r"file 'a'.*\(properties changed:"
                                   ".*\+x to -x.*\)")
@@ -527,7 +496,7 @@
         is a binary file in the diff.
         """
         # See https://bugs.launchpad.net/bugs/110092.
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         # This bug isn't triggered with cStringIO.
         from StringIO import StringIO
@@ -552,7 +521,7 @@
 
     def test_unicode_filename(self):
         """Test when the filename are unicode."""
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         alpha, omega = u'\u03b1', u'\u03c9'
         autf8, outf8 = alpha.encode('utf8'), omega.encode('utf8')
@@ -573,7 +542,7 @@
         tree.add(['add_'+alpha], ['file-id'])
         self.build_tree_contents([('tree/mod_'+alpha, 'contents_mod\n')])
 
-        d = self.get_diff(tree.basis_tree(), tree)
+        d = get_diff_as_string(tree.basis_tree(), tree)
         self.assertContainsRe(d,
                 "=== renamed file 'ren_%s' => 'ren_%s'\n"%(autf8, outf8))
         self.assertContainsRe(d, "=== added file 'add_%s'"%autf8)
@@ -584,7 +553,7 @@
         """Test for bug #382699: unicode filenames on Windows should be shown
         in user encoding.
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # The word 'test' in Russian
         _russian_test = u'\u0422\u0435\u0441\u0442'
         directory = _russian_test + u'/'
@@ -721,7 +690,7 @@
              ' \@\@\n-old\n\+new\n\n')
 
     def test_diff_kind_change(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.build_tree_contents([('old-tree/olddir/',),
                                   ('old-tree/olddir/oldfile', 'old\n')])
         self.old_tree.add('olddir')
@@ -1185,7 +1154,7 @@
 
 class TestPatienceDiffLib_c(TestPatienceDiffLib):
 
-    _test_needs_features = [compiled_patiencediff_feature]
+    _test_needs_features = [features.compiled_patiencediff_feature]
 
     def setUp(self):
         super(TestPatienceDiffLib_c, self).setUp()
@@ -1281,7 +1250,7 @@
 
 class TestPatienceDiffLibFiles_c(TestPatienceDiffLibFiles):
 
-    _test_needs_features = [compiled_patiencediff_feature]
+    _test_needs_features = [features.compiled_patiencediff_feature]
 
     def setUp(self):
         super(TestPatienceDiffLibFiles_c, self).setUp()
@@ -1293,7 +1262,7 @@
 class TestUsingCompiledIfAvailable(tests.TestCase):
 
     def test_PatienceSequenceMatcher(self):
-        if compiled_patiencediff_feature.available():
+        if features.compiled_patiencediff_feature.available():
             from bzrlib._patiencediff_c import PatienceSequenceMatcher_c
             self.assertIs(PatienceSequenceMatcher_c,
                           patiencediff.PatienceSequenceMatcher)
@@ -1303,7 +1272,7 @@
                           patiencediff.PatienceSequenceMatcher)
 
     def test_unique_lcs(self):
-        if compiled_patiencediff_feature.available():
+        if features.compiled_patiencediff_feature.available():
             from bzrlib._patiencediff_c import unique_lcs_c
             self.assertIs(unique_lcs_c,
                           patiencediff.unique_lcs)
@@ -1313,7 +1282,7 @@
                           patiencediff.unique_lcs)
 
     def test_recurse_matches(self):
-        if compiled_patiencediff_feature.available():
+        if features.compiled_patiencediff_feature.available():
             from bzrlib._patiencediff_c import recurse_matches_c
             self.assertIs(recurse_matches_c,
                           patiencediff.recurse_matches)
@@ -1369,7 +1338,7 @@
                          ' on this machine', str(e))
 
     def test_prepare_files_creates_paths_readable_by_windows_tool(self):
-        self.requireFeature(AttribFeature)
+        self.requireFeature(features.AttribFeature)
         output = StringIO()
         tree = self.make_branch_and_tree('tree')
         self.build_tree_contents([('tree/file', 'content')])
@@ -1434,12 +1403,56 @@
         diff_obj._prepare_files('file2-id', 'oldname2', 'newname2')
 
 
+class TestDiffFromToolEncodedFilename(tests.TestCaseWithTransport):
+
+    def test_encodable_filename(self):
+        # Just checks file path for external diff tool.
+        # We cannot change CPython's internal encoding used by os.exec*.
+        import sys
+        diffobj = diff.DiffFromTool(['dummy', '@old_path', '@new_path'],
+                                    None, None, None)
+        for _, scenario in EncodingAdapter.encoding_scenarios:
+            encoding = scenario['encoding']
+            dirname  = scenario['info']['directory']
+            filename = scenario['info']['filename']
+
+            self.overrideAttr(diffobj, '_fenc', lambda: encoding)
+            relpath = dirname + u'/' + filename
+            fullpath = diffobj._safe_filename('safe', relpath)
+            self.assertEqual(
+                    fullpath,
+                    fullpath.encode(encoding).decode(encoding)
+                    )
+            self.assert_(fullpath.startswith(diffobj._root + '/safe'))
+
+    def test_unencodable_filename(self):
+        import sys
+        diffobj = diff.DiffFromTool(['dummy', '@old_path', '@new_path'],
+                                    None, None, None)
+        for _, scenario in EncodingAdapter.encoding_scenarios:
+            encoding = scenario['encoding']
+            dirname  = scenario['info']['directory']
+            filename = scenario['info']['filename']
+
+            if encoding == 'iso-8859-1':
+                encoding = 'iso-8859-2'
+            else:
+                encoding = 'iso-8859-1'
+
+            self.overrideAttr(diffobj, '_fenc', lambda: encoding)
+            relpath = dirname + u'/' + filename
+            fullpath = diffobj._safe_filename('safe', relpath)
+            self.assertEqual(
+                    fullpath,
+                    fullpath.encode(encoding).decode(encoding)
+                    )
+            self.assert_(fullpath.startswith(diffobj._root + '/safe'))
+
+
 class TestGetTreesAndBranchesToDiffLocked(tests.TestCaseWithTransport):
 
     def call_gtabtd(self, path_list, revision_specs, old_url, new_url):
-        """Call get_trees_and_branches_to_diff_locked.  Overridden by
-        TestGetTreesAndBranchesToDiff.
-        """
+        """Call get_trees_and_branches_to_diff_locked."""
         return diff.get_trees_and_branches_to_diff_locked(
             path_list, revision_specs, old_url, new_url, self.addCleanup)
 
@@ -1482,15 +1495,3 @@
         self.assertEqual(tree.branch.base, new_branch.base)
         self.assertIs(None, specific_files)
         self.assertEqual(tree.basedir, extra_trees[0].basedir)
-
-
-class TestGetTreesAndBranchesToDiff(TestGetTreesAndBranchesToDiffLocked):
-    """Apply the tests for get_trees_and_branches_to_diff_locked to the
-    deprecated get_trees_and_branches_to_diff function.
-    """
-
-    def call_gtabtd(self, path_list, revision_specs, old_url, new_url):
-        return self.applyDeprecated(
-            deprecated_in((2, 2, 0)), diff.get_trees_and_branches_to_diff,
-            path_list, revision_specs, old_url, new_url)
-

=== modified file 'bzrlib/tests/test_dirstate.py'
--- bzrlib/tests/test_dirstate.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/test_dirstate.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,19 +16,27 @@
 
 """Tests of the dirstate functionality being built for WorkingTreeFormat4."""
 
-import bisect
 import os
+import tempfile
 
 from bzrlib import (
+    bzrdir,
     dirstate,
     errors,
     inventory,
     memorytree,
     osutils,
     revision as _mod_revision,
+    revisiontree,
     tests,
-    )
-from bzrlib.tests import test_osutils
+    workingtree_4,
+    )
+from bzrlib.transport import memory
+from bzrlib.tests import (
+    features,
+    test_osutils,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
 
 
 # TODO:
@@ -44,19 +52,14 @@
 # set_path_id  setting id when state is in memory modified
 
 
-def load_tests(basic_tests, module, loader):
-    suite = loader.suiteClass()
-    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
-        basic_tests, tests.condition_isinstance(TestCaseWithDirState))
-    tests.multiply_tests(dir_reader_tests,
-                         test_osutils.dir_reader_scenarios(), suite)
-    suite.addTest(remaining_tests)
-    return suite
+load_tests = load_tests_apply_scenarios
 
 
 class TestCaseWithDirState(tests.TestCaseWithTransport):
     """Helper functions for creating DirState objects with various content."""
 
+    scenarios = test_osutils.dir_reader_scenarios()
+
     # Set by load_tests
     _dir_reader_class = None
     _native_to_unicode = None # Not used yet
@@ -532,6 +535,19 @@
 
 class TestDirStateOnFile(TestCaseWithDirState):
 
+    def create_updated_dirstate(self):
+        self.build_tree(['a-file'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a-file'], ['a-id'])
+        tree.commit('add a-file')
+        # Save and unlock the state, re-open it in readonly mode
+        state = dirstate.DirState.from_tree(tree, 'dirstate')
+        state.save()
+        state.unlock()
+        state = dirstate.DirState.on_file('dirstate')
+        state.lock_read()
+        return state
+
     def test_construct_with_path(self):
         tree = self.make_branch_and_tree('tree')
         state = dirstate.DirState.from_tree(tree, 'dirstate.from_tree')
@@ -566,37 +582,25 @@
             state.unlock()
 
     def test_can_save_in_read_lock(self):
-        self.build_tree(['a-file'])
-        state = dirstate.DirState.initialize('dirstate')
-        try:
-            # No stat and no sha1 sum.
-            state.add('a-file', 'a-file-id', 'file', None, '')
-            state.save()
-        finally:
-            state.unlock()
-
-        # Now open in readonly mode
-        state = dirstate.DirState.on_file('dirstate')
-        state.lock_read()
+        state = self.create_updated_dirstate()
         try:
             entry = state._get_entry(0, path_utf8='a-file')
             # The current size should be 0 (default)
             self.assertEqual(0, entry[1][0][2])
             # We should have a real entry.
             self.assertNotEqual((None, None), entry)
-            # Make sure everything is old enough
+            # Set the cutoff-time into the future, so things look cacheable
             state._sha_cutoff_time()
-            state._cutoff_time += 10
-            # Change the file length
-            self.build_tree_contents([('a-file', 'shorter')])
-            sha1sum = dirstate.update_entry(state, entry, 'a-file',
-                os.lstat('a-file'))
-            # new file, no cached sha:
-            self.assertEqual(None, sha1sum)
+            state._cutoff_time += 10.0
+            st = os.lstat('a-file')
+            sha1sum = dirstate.update_entry(state, entry, 'a-file', st)
+            # We updated the current sha1sum because the file is cacheable
+            self.assertEqual('ecc5374e9ed82ad3ea3b4d452ea995a5fd3e70e3',
+                             sha1sum)
 
             # The dirblock has been updated
-            self.assertEqual(7, entry[1][0][2])
-            self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+            self.assertEqual(st.st_size, entry[1][0][2])
+            self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
                              state._dirblock_state)
 
             del entry
@@ -611,30 +615,25 @@
         state.lock_read()
         try:
             entry = state._get_entry(0, path_utf8='a-file')
-            self.assertEqual(7, entry[1][0][2])
+            self.assertEqual(st.st_size, entry[1][0][2])
         finally:
             state.unlock()
 
     def test_save_fails_quietly_if_locked(self):
         """If dirstate is locked, save will fail without complaining."""
-        self.build_tree(['a-file'])
-        state = dirstate.DirState.initialize('dirstate')
-        try:
-            # No stat and no sha1 sum.
-            state.add('a-file', 'a-file-id', 'file', None, '')
-            state.save()
-        finally:
-            state.unlock()
-
-        state = dirstate.DirState.on_file('dirstate')
-        state.lock_read()
+        state = self.create_updated_dirstate()
         try:
             entry = state._get_entry(0, path_utf8='a-file')
-            sha1sum = dirstate.update_entry(state, entry, 'a-file',
-                os.lstat('a-file'))
-            # No sha - too new
-            self.assertEqual(None, sha1sum)
-            self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+            # No cached sha1 yet.
+            self.assertEqual('', entry[1][0][1])
+            # Set the cutoff-time into the future, so things look cacheable
+            state._sha_cutoff_time()
+            state._cutoff_time += 10.0
+            st = os.lstat('a-file')
+            sha1sum = dirstate.update_entry(state, entry, 'a-file', st)
+            self.assertEqual('ecc5374e9ed82ad3ea3b4d452ea995a5fd3e70e3',
+                             sha1sum)
+            self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
                              state._dirblock_state)
 
             # Now, before we try to save, grab another dirstate, and take out a
@@ -730,17 +729,35 @@
 
 class TestDirStateManipulations(TestCaseWithDirState):
 
+    def make_minimal_tree(self):
+        tree1 = self.make_branch_and_memory_tree('tree1')
+        tree1.lock_write()
+        self.addCleanup(tree1.unlock)
+        tree1.add('')
+        revid1 = tree1.commit('foo')
+        return tree1, revid1
+
+    def test_update_minimal_updates_id_index(self):
+        state = self.create_dirstate_with_root_and_subdir()
+        self.addCleanup(state.unlock)
+        id_index = state._get_id_index()
+        self.assertEqual(['a-root-value', 'subdir-id'], sorted(id_index))
+        state.add('file-name', 'file-id', 'file', None, '')
+        self.assertEqual(['a-root-value', 'file-id', 'subdir-id'],
+                         sorted(id_index))
+        state.update_minimal(('', 'new-name', 'file-id'), 'f',
+                             path_utf8='new-name')
+        self.assertEqual(['a-root-value', 'file-id', 'subdir-id'],
+                         sorted(id_index))
+        self.assertEqual([('', 'new-name', 'file-id')],
+                         sorted(id_index['file-id']))
+        state._validate()
+
     def test_set_state_from_inventory_no_content_no_parents(self):
         # setting the current inventory is a slow but important api to support.
-        tree1 = self.make_branch_and_memory_tree('tree1')
-        tree1.lock_write()
-        try:
-            tree1.add('')
-            revid1 = tree1.commit('foo').encode('utf8')
-            root_id = tree1.get_root_id()
-            inv = tree1.inventory
-        finally:
-            tree1.unlock()
+        tree1, revid1 = self.make_minimal_tree()
+        inv = tree1.inventory
+        root_id = inv.path2id('')
         expected_result = [], [
             (('', '', root_id), [
              ('d', '', 0, False, dirstate.DirState.NULLSTAT)])]
@@ -758,6 +775,50 @@
             # This will unlock it
             self.check_state_with_reopen(expected_result, state)
 
+    def test_set_state_from_scratch_no_parents(self):
+        tree1, revid1 = self.make_minimal_tree()
+        inv = tree1.inventory
+        root_id = inv.path2id('')
+        expected_result = [], [
+            (('', '', root_id), [
+             ('d', '', 0, False, dirstate.DirState.NULLSTAT)])]
+        state = dirstate.DirState.initialize('dirstate')
+        try:
+            state.set_state_from_scratch(inv, [], [])
+            self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+                             state._header_state)
+            self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+                             state._dirblock_state)
+        except:
+            state.unlock()
+            raise
+        else:
+            # This will unlock it
+            self.check_state_with_reopen(expected_result, state)
+
+    def test_set_state_from_scratch_identical_parent(self):
+        tree1, revid1 = self.make_minimal_tree()
+        inv = tree1.inventory
+        root_id = inv.path2id('')
+        rev_tree1 = tree1.branch.repository.revision_tree(revid1)
+        d_entry = ('d', '', 0, False, dirstate.DirState.NULLSTAT)
+        parent_entry = ('d', '', 0, False, revid1)
+        expected_result = [revid1], [
+            (('', '', root_id), [d_entry, parent_entry])]
+        state = dirstate.DirState.initialize('dirstate')
+        try:
+            state.set_state_from_scratch(inv, [(revid1, rev_tree1)], [])
+            self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+                             state._header_state)
+            self.assertEqual(dirstate.DirState.IN_MEMORY_MODIFIED,
+                             state._dirblock_state)
+        except:
+            state.unlock()
+            raise
+        else:
+            # This will unlock it
+            self.check_state_with_reopen(expected_result, state)
+
     def test_set_state_from_inventory_preserves_hashcache(self):
         # https://bugs.launchpad.net/bzr/+bug/146176
         # set_state_from_inventory should preserve the stat and hash value for
@@ -1151,7 +1212,7 @@
         # The most trivial addition of a symlink when there are no parents and
         # its in the root and all data about the file is supplied
         # bzr doesn't support fake symlinks on windows, yet.
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         os.symlink(target, link_name)
         stat = os.lstat(link_name)
         expected_entries = [
@@ -1182,7 +1243,7 @@
         self._test_add_symlink_to_root_no_parents_all_data('a link', 'target')
 
     def test_add_symlink_unicode_to_root_no_parents_all_data(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_add_symlink_to_root_no_parents_all_data(
             u'\N{Euro Sign}link', u'targ\N{Euro Sign}et')
 
@@ -1286,6 +1347,62 @@
             tree1.unlock()
 
 
+class TestDirStateHashUpdates(TestCaseWithDirState):
+
+    def do_update_entry(self, state, path):
+        entry = state._get_entry(0, path_utf8=path)
+        stat = os.lstat(path)
+        return dirstate.update_entry(state, entry, os.path.abspath(path), stat)
+
+    def _read_state_content(self, state):
+        """Read the content of the dirstate file.
+
+        On Windows when one process locks a file, you can't even open() the
+        file in another process (to read it). So we go directly to
+        state._state_file. This should always be the exact disk representation,
+        so it is reasonable to do so.
+        DirState also always seeks before reading, so it doesn't matter if we
+        bump the file pointer.
+        """
+        state._state_file.seek(0)
+        return state._state_file.read()
+
+    def test_worth_saving_limit_avoids_writing(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['c', 'd'])
+        tree.lock_write()
+        tree.add(['c', 'd'], ['c-id', 'd-id'])
+        tree.commit('add c and d')
+        state = InstrumentedDirState.on_file(tree.current_dirstate()._filename,
+                                             worth_saving_limit=2)
+        tree.unlock()
+        state.lock_write()
+        self.addCleanup(state.unlock)
+        state._read_dirblocks_if_needed()
+        state.adjust_time(+20) # Allow things to be cached
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
+        content = self._read_state_content(state)
+        self.do_update_entry(state, 'c')
+        self.assertEqual(1, len(state._known_hash_changes))
+        self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
+                         state._dirblock_state)
+        state.save()
+        # It should not have set the state to IN_MEMORY_UNMODIFIED because the
+        # hash values haven't been written out.
+        self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
+                         state._dirblock_state)
+        self.assertEqual(content, self._read_state_content(state))
+        self.assertEqual(dirstate.DirState.IN_MEMORY_HASH_MODIFIED,
+                         state._dirblock_state)
+        self.do_update_entry(state, 'd')
+        self.assertEqual(2, len(state._known_hash_changes))
+        state.save()
+        self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
+                         state._dirblock_state)
+        self.assertEqual(0, len(state._known_hash_changes))
+
+
 class TestGetLines(TestCaseWithDirState):
 
     def test_get_line_with_2_rows(self):
@@ -1684,8 +1801,9 @@
 class InstrumentedDirState(dirstate.DirState):
     """An DirState with instrumented sha1 functionality."""
 
-    def __init__(self, path, sha1_provider):
-        super(InstrumentedDirState, self).__init__(path, sha1_provider)
+    def __init__(self, path, sha1_provider, worth_saving_limit=0):
+        super(InstrumentedDirState, self).__init__(path, sha1_provider,
+            worth_saving_limit=worth_saving_limit)
         self._time_offset = 0
         self._log = []
         # member is dynamically set in DirState.__init__ to turn on trace
@@ -2320,3 +2438,444 @@
         self.assertTrue(len(statvalue) >= 10)
         self.assertEqual(len(text), statvalue.st_size)
         self.assertEqual(expected_sha, sha1)
+
+
+class _Repo(object):
+    """A minimal api to get InventoryRevisionTree to work."""
+
+    def __init__(self):
+        default_format = bzrdir.format_registry.make_bzrdir('default')
+        self._format = default_format.repository_format
+
+    def lock_read(self):
+        pass
+
+    def unlock(self):
+        pass
+
+
+class TestUpdateBasisByDelta(tests.TestCase):
+
+    def path_to_ie(self, path, file_id, rev_id, dir_ids):
+        if path.endswith('/'):
+            is_dir = True
+            path = path[:-1]
+        else:
+            is_dir = False
+        dirname, basename = osutils.split(path)
+        try:
+            dir_id = dir_ids[dirname]
+        except KeyError:
+            dir_id = osutils.basename(dirname) + '-id'
+        if is_dir:
+            ie = inventory.InventoryDirectory(file_id, basename, dir_id)
+            dir_ids[path] = file_id
+        else:
+            ie = inventory.InventoryFile(file_id, basename, dir_id)
+            ie.text_size = 0
+            ie.text_sha1 = ''
+        ie.revision = rev_id
+        return ie
+
+    def create_tree_from_shape(self, rev_id, shape):
+        dir_ids = {'': 'root-id'}
+        inv = inventory.Inventory('root-id', rev_id)
+        for path, file_id in shape:
+            if path == '':
+                # Replace the root entry
+                del inv._byid[inv.root.file_id]
+                inv.root.file_id = file_id
+                inv._byid[file_id] = inv.root
+                dir_ids[''] = file_id
+                continue
+            inv.add(self.path_to_ie(path, file_id, rev_id, dir_ids))
+        return revisiontree.InventoryRevisionTree(_Repo(), inv, rev_id)
+
+    def create_empty_dirstate(self):
+        fd, path = tempfile.mkstemp(prefix='bzr-dirstate')
+        self.addCleanup(os.remove, path)
+        os.close(fd)
+        state = dirstate.DirState.initialize(path)
+        self.addCleanup(state.unlock)
+        return state
+
+    def create_inv_delta(self, delta, rev_id):
+        """Translate a 'delta shape' into an actual InventoryDelta"""
+        dir_ids = {'': 'root-id'}
+        inv_delta = []
+        for old_path, new_path, file_id in delta:
+            if old_path is not None and old_path.endswith('/'):
+                # Don't have to actually do anything for this, because only
+                # new_path creates InventoryEntries
+                old_path = old_path[:-1]
+            if new_path is None: # Delete
+                inv_delta.append((old_path, None, file_id, None))
+                continue
+            ie = self.path_to_ie(new_path, file_id, rev_id, dir_ids)
+            inv_delta.append((old_path, new_path, file_id, ie))
+        return inv_delta
+
+    def assertUpdate(self, active, basis, target):
+        """Assert that update_basis_by_delta works how we want.
+
+        Set up a DirState object with active_shape for tree 0, basis_shape for
+        tree 1. Then apply the delta from basis_shape to target_shape,
+        and assert that the DirState is still valid, and that its stored
+        content matches the target_shape.
+        """
+        active_tree = self.create_tree_from_shape('active', active)
+        basis_tree = self.create_tree_from_shape('basis', basis)
+        target_tree = self.create_tree_from_shape('target', target)
+        state = self.create_empty_dirstate()
+        state.set_state_from_scratch(active_tree.inventory,
+            [('basis', basis_tree)], [])
+        delta = target_tree.inventory._make_delta(basis_tree.inventory)
+        state.update_basis_by_delta(delta, 'target')
+        state._validate()
+        dirstate_tree = workingtree_4.DirStateRevisionTree(state,
+            'target', _Repo())
+        # The target now that delta has been applied should match the
+        # RevisionTree
+        self.assertEqual([], list(dirstate_tree.iter_changes(target_tree)))
+        # And the dirblock state should be identical to the state if we created
+        # it from scratch.
+        state2 = self.create_empty_dirstate()
+        state2.set_state_from_scratch(active_tree.inventory,
+            [('target', target_tree)], [])
+        self.assertEqual(state2._dirblocks, state._dirblocks)
+        return state
+
+    def assertBadDelta(self, active, basis, delta):
+        """Test that we raise InconsistentDelta when appropriate.
+
+        :param active: The active tree shape
+        :param basis: The basis tree shape
+        :param delta: A description of the delta to apply. Similar to the form
+            for regular inventory deltas, but omitting the InventoryEntry.
+            So adding a file is: (None, 'path', 'file-id')
+            Adding a directory is: (None, 'path/', 'dir-id')
+            Renaming a dir is: ('old/', 'new/', 'dir-id')
+            etc.
+        """
+        active_tree = self.create_tree_from_shape('active', active)
+        basis_tree = self.create_tree_from_shape('basis', basis)
+        inv_delta = self.create_inv_delta(delta, 'target')
+        state = self.create_empty_dirstate()
+        state.set_state_from_scratch(active_tree.inventory,
+            [('basis', basis_tree)], [])
+        self.assertRaises(errors.InconsistentDelta,
+            state.update_basis_by_delta, inv_delta, 'target')
+        ## try:
+        ##     state.update_basis_by_delta(inv_delta, 'target')
+        ## except errors.InconsistentDelta, e:
+        ##     import pdb; pdb.set_trace()
+        ## else:
+        ##     import pdb; pdb.set_trace()
+        self.assertTrue(state._changes_aborted)
+
+    def test_remove_file_matching_active_state(self):
+        state = self.assertUpdate(
+            active=[],
+            basis =[('file', 'file-id')],
+            target=[],
+            )
+
+    def test_remove_file_present_in_active_state(self):
+        state = self.assertUpdate(
+            active=[('file', 'file-id')],
+            basis =[('file', 'file-id')],
+            target=[],
+            )
+
+    def test_remove_file_present_elsewhere_in_active_state(self):
+        state = self.assertUpdate(
+            active=[('other-file', 'file-id')],
+            basis =[('file', 'file-id')],
+            target=[],
+            )
+
+    def test_remove_file_active_state_has_diff_file(self):
+        state = self.assertUpdate(
+            active=[('file', 'file-id-2')],
+            basis =[('file', 'file-id')],
+            target=[],
+            )
+
+    def test_remove_file_active_state_has_diff_file_and_file_elsewhere(self):
+        state = self.assertUpdate(
+            active=[('file', 'file-id-2'),
+                    ('other-file', 'file-id')],
+            basis =[('file', 'file-id')],
+            target=[],
+            )
+
+    def test_add_file_matching_active_state(self):
+        state = self.assertUpdate(
+            active=[('file', 'file-id')],
+            basis =[],
+            target=[('file', 'file-id')],
+            )
+
+    def test_add_file_missing_in_active_state(self):
+        state = self.assertUpdate(
+            active=[],
+            basis =[],
+            target=[('file', 'file-id')],
+            )
+
+    def test_add_file_elsewhere_in_active_state(self):
+        state = self.assertUpdate(
+            active=[('other-file', 'file-id')],
+            basis =[],
+            target=[('file', 'file-id')],
+            )
+
+    def test_add_file_active_state_has_diff_file_and_file_elsewhere(self):
+        state = self.assertUpdate(
+            active=[('other-file', 'file-id'),
+                    ('file', 'file-id-2')],
+            basis =[],
+            target=[('file', 'file-id')],
+            )
+
+    def test_rename_file_matching_active_state(self):
+        state = self.assertUpdate(
+            active=[('other-file', 'file-id')],
+            basis =[('file', 'file-id')],
+            target=[('other-file', 'file-id')],
+            )
+
+    def test_rename_file_missing_in_active_state(self):
+        state = self.assertUpdate(
+            active=[],
+            basis =[('file', 'file-id')],
+            target=[('other-file', 'file-id')],
+            )
+
+    def test_rename_file_present_elsewhere_in_active_state(self):
+        state = self.assertUpdate(
+            active=[('third', 'file-id')],
+            basis =[('file', 'file-id')],
+            target=[('other-file', 'file-id')],
+            )
+
+    def test_rename_file_active_state_has_diff_source_file(self):
+        state = self.assertUpdate(
+            active=[('file', 'file-id-2')],
+            basis =[('file', 'file-id')],
+            target=[('other-file', 'file-id')],
+            )
+
+    def test_rename_file_active_state_has_diff_target_file(self):
+        state = self.assertUpdate(
+            active=[('other-file', 'file-id-2')],
+            basis =[('file', 'file-id')],
+            target=[('other-file', 'file-id')],
+            )
+
+    def test_rename_file_active_has_swapped_files(self):
+        state = self.assertUpdate(
+            active=[('file', 'file-id'),
+                    ('other-file', 'file-id-2')],
+            basis= [('file', 'file-id'),
+                    ('other-file', 'file-id-2')],
+            target=[('file', 'file-id-2'),
+                    ('other-file', 'file-id')])
+
+    def test_rename_file_basis_has_swapped_files(self):
+        state = self.assertUpdate(
+            active=[('file', 'file-id'),
+                    ('other-file', 'file-id-2')],
+            basis= [('file', 'file-id-2'),
+                    ('other-file', 'file-id')],
+            target=[('file', 'file-id'),
+                    ('other-file', 'file-id-2')])
+
+    def test_rename_directory_with_contents(self):
+        state = self.assertUpdate( # active matches basis
+            active=[('dir1/', 'dir-id'),
+                    ('dir1/file', 'file-id')],
+            basis= [('dir1/', 'dir-id'),
+                    ('dir1/file', 'file-id')],
+            target=[('dir2/', 'dir-id'),
+                    ('dir2/file', 'file-id')])
+        state = self.assertUpdate( # active matches target
+            active=[('dir2/', 'dir-id'),
+                    ('dir2/file', 'file-id')],
+            basis= [('dir1/', 'dir-id'),
+                    ('dir1/file', 'file-id')],
+            target=[('dir2/', 'dir-id'),
+                    ('dir2/file', 'file-id')])
+        state = self.assertUpdate( # active empty
+            active=[],
+            basis= [('dir1/', 'dir-id'),
+                    ('dir1/file', 'file-id')],
+            target=[('dir2/', 'dir-id'),
+                    ('dir2/file', 'file-id')])
+        state = self.assertUpdate( # active present at other location
+            active=[('dir3/', 'dir-id'),
+                    ('dir3/file', 'file-id')],
+            basis= [('dir1/', 'dir-id'),
+                    ('dir1/file', 'file-id')],
+            target=[('dir2/', 'dir-id'),
+                    ('dir2/file', 'file-id')])
+        state = self.assertUpdate( # active has different ids
+            active=[('dir1/', 'dir1-id'),
+                    ('dir1/file', 'file1-id'),
+                    ('dir2/', 'dir2-id'),
+                    ('dir2/file', 'file2-id')],
+            basis= [('dir1/', 'dir-id'),
+                    ('dir1/file', 'file-id')],
+            target=[('dir2/', 'dir-id'),
+                    ('dir2/file', 'file-id')])
+
+    def test_invalid_file_not_present(self):
+        state = self.assertBadDelta(
+            active=[('file', 'file-id')],
+            basis= [('file', 'file-id')],
+            delta=[('other-file', 'file', 'file-id')])
+
+    def test_invalid_new_id_same_path(self):
+        # The bad entry comes after
+        state = self.assertBadDelta(
+            active=[('file', 'file-id')],
+            basis= [('file', 'file-id')],
+            delta=[(None, 'file', 'file-id-2')])
+        # The bad entry comes first
+        state = self.assertBadDelta(
+            active=[('file', 'file-id-2')],
+            basis=[('file', 'file-id-2')],
+            delta=[(None, 'file', 'file-id')])
+
+    def test_invalid_existing_id(self):
+        state = self.assertBadDelta(
+            active=[('file', 'file-id')],
+            basis= [('file', 'file-id')],
+            delta=[(None, 'file', 'file-id')])
+
+    def test_invalid_parent_missing(self):
+        state = self.assertBadDelta(
+            active=[],
+            basis= [],
+            delta=[(None, 'path/path2', 'file-id')])
+        # Note: we force the active tree to have the directory, by knowing how
+        #       path_to_ie handles entries with missing parents
+        state = self.assertBadDelta(
+            active=[('path/', 'path-id')],
+            basis= [],
+            delta=[(None, 'path/path2', 'file-id')])
+        state = self.assertBadDelta(
+            active=[('path/', 'path-id'),
+                    ('path/path2', 'file-id')],
+            basis= [],
+            delta=[(None, 'path/path2', 'file-id')])
+
+    def test_renamed_dir_same_path(self):
+        # We replace the parent directory, with another parent dir. But the C
+        # file doesn't look like it has been moved.
+        state = self.assertUpdate(# Same as basis
+            active=[('dir/', 'A-id'),
+                    ('dir/B', 'B-id')],
+            basis= [('dir/', 'A-id'),
+                    ('dir/B', 'B-id')],
+            target=[('dir/', 'C-id'),
+                    ('dir/B', 'B-id')])
+        state = self.assertUpdate(# Same as target
+            active=[('dir/', 'C-id'),
+                    ('dir/B', 'B-id')],
+            basis= [('dir/', 'A-id'),
+                    ('dir/B', 'B-id')],
+            target=[('dir/', 'C-id'),
+                    ('dir/B', 'B-id')])
+        state = self.assertUpdate(# empty active
+            active=[],
+            basis= [('dir/', 'A-id'),
+                    ('dir/B', 'B-id')],
+            target=[('dir/', 'C-id'),
+                    ('dir/B', 'B-id')])
+        state = self.assertUpdate(# different active
+            active=[('dir/', 'D-id'),
+                    ('dir/B', 'B-id')],
+            basis= [('dir/', 'A-id'),
+                    ('dir/B', 'B-id')],
+            target=[('dir/', 'C-id'),
+                    ('dir/B', 'B-id')])
+
+    def test_parent_child_swap(self):
+        state = self.assertUpdate(# Same as basis
+            active=[('A/', 'A-id'),
+                    ('A/B/', 'B-id'),
+                    ('A/B/C', 'C-id')],
+            basis= [('A/', 'A-id'),
+                    ('A/B/', 'B-id'),
+                    ('A/B/C', 'C-id')],
+            target=[('A/', 'B-id'),
+                    ('A/B/', 'A-id'),
+                    ('A/B/C', 'C-id')])
+        state = self.assertUpdate(# Same as target
+            active=[('A/', 'B-id'),
+                    ('A/B/', 'A-id'),
+                    ('A/B/C', 'C-id')],
+            basis= [('A/', 'A-id'),
+                    ('A/B/', 'B-id'),
+                    ('A/B/C', 'C-id')],
+            target=[('A/', 'B-id'),
+                    ('A/B/', 'A-id'),
+                    ('A/B/C', 'C-id')])
+        state = self.assertUpdate(# empty active
+            active=[],
+            basis= [('A/', 'A-id'),
+                    ('A/B/', 'B-id'),
+                    ('A/B/C', 'C-id')],
+            target=[('A/', 'B-id'),
+                    ('A/B/', 'A-id'),
+                    ('A/B/C', 'C-id')])
+        state = self.assertUpdate(# different active
+            active=[('D/', 'A-id'),
+                    ('D/E/', 'B-id'),
+                    ('F', 'C-id')],
+            basis= [('A/', 'A-id'),
+                    ('A/B/', 'B-id'),
+                    ('A/B/C', 'C-id')],
+            target=[('A/', 'B-id'),
+                    ('A/B/', 'A-id'),
+                    ('A/B/C', 'C-id')])
+
+    def test_change_root_id(self):
+        state = self.assertUpdate( # same as basis
+            active=[('', 'root-id'),
+                    ('file', 'file-id')],
+            basis= [('', 'root-id'),
+                    ('file', 'file-id')],
+            target=[('', 'target-root-id'),
+                    ('file', 'file-id')])
+        state = self.assertUpdate( # same as target
+            active=[('', 'target-root-id'),
+                    ('file', 'file-id')],
+            basis= [('', 'root-id'),
+                    ('file', 'file-id')],
+            target=[('', 'target-root-id'),
+                    ('file', 'root-id')])
+        state = self.assertUpdate( # all different
+            active=[('', 'active-root-id'),
+                    ('file', 'file-id')],
+            basis= [('', 'root-id'),
+                    ('file', 'file-id')],
+            target=[('', 'target-root-id'),
+                    ('file', 'root-id')])
+
+    def test_change_file_absent_in_active(self):
+        state = self.assertUpdate(
+            active=[],
+            basis= [('file', 'file-id')],
+            target=[('file', 'file-id')])
+
+    def test_invalid_changed_file(self):
+        state = self.assertBadDelta( # Not present in basis
+            active=[('file', 'file-id')],
+            basis= [],
+            delta=[('file', 'file', 'file-id')])
+        state = self.assertBadDelta( # present at another location in basis
+            active=[('file', 'file-id')],
+            basis= [('other-file', 'file-id')],
+            delta=[('file', 'file', 'file-id')])

=== modified file 'bzrlib/tests/test_eol_filters.py'
--- bzrlib/tests/test_eol_filters.py	2009-06-10 23:29:48 +0000
+++ bzrlib/tests/test_eol_filters.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 
 from bzrlib import (
     errors,
-    rules,
     )
 from bzrlib.filters import _get_filter_stack_for
 from bzrlib.filters.eol import (

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2010-07-14 14:22:40 +0000
+++ bzrlib/tests/test_errors.py	2011-05-26 21:02:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,10 +25,13 @@
     bzrdir,
     errors,
     osutils,
-    symbol_versioning,
     urlutils,
     )
-from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.tests import (
+    TestCase,
+    TestCaseWithTransport,
+    TestSkipped,
+    )
 
 
 class TestErrors(TestCaseWithTransport):
@@ -159,6 +162,13 @@
             "cannot be broken.",
             str(error))
 
+    def test_lock_corrupt(self):
+        error = errors.LockCorrupt("corruption info")
+        self.assertEqualDiff("Lock is apparently held, but corrupted: "
+            "corruption info\n"
+            "Use 'bzr break-lock' to clear it",
+            str(error))
+
     def test_knit_data_stream_incompatible(self):
         error = errors.KnitDataStreamIncompatible(
             'stream format', 'target format')
@@ -290,11 +300,10 @@
             str(error))
 
     def test_up_to_date(self):
-        error = errors.UpToDateFormat(bzrdir.BzrDirFormat4())
-        self.assertEqualDiff("The branch format All-in-one "
-                             "format 4 is already at the most "
-                             "recent format.",
-                             str(error))
+        error = errors.UpToDateFormat("someformat")
+        self.assertEqualDiff(
+            "The branch format someformat is already at the most "
+            "recent format.", str(error))
 
     def test_corrupt_repository(self):
         repo = self.make_repository('.')
@@ -568,12 +577,17 @@
         try:
             1/0
         except ZeroDivisionError:
-            exc_info = sys.exc_info()
-        err = errors.HookFailed('hook stage', 'hook name', exc_info, warn=False)
-        self.assertStartsWith(
-            str(err), 'Hook \'hook name\' during hook stage failed:\n')
-        self.assertEndsWith(
-            str(err), 'integer division or modulo by zero')
+            err = errors.HookFailed('hook stage', 'hook name', sys.exc_info(),
+                warn=False)
+        # GZ 2010-11-08: Should not store exc_info in exception instances, but
+        #                HookFailed is deprecated anyway and could maybe go.
+        try:
+            self.assertStartsWith(
+                str(err), 'Hook \'hook name\' during hook stage failed:\n')
+            self.assertEndsWith(
+                str(err), 'integer division or modulo by zero')
+        finally:
+            del err
 
     def test_tip_change_rejected(self):
         err = errors.TipChangeRejected(u'Unicode message\N{INTERROBANG}')
@@ -602,11 +616,14 @@
         try:
             raise Exception("example error")
         except Exception:
-            exc_info = sys.exc_info()
-        err = errors.SmartMessageHandlerError(exc_info)
-        self.assertStartsWith(
-            str(err), "The message handler raised an exception:\n")
-        self.assertEndsWith(str(err), "Exception: example error\n")
+            err = errors.SmartMessageHandlerError(sys.exc_info())
+        # GZ 2010-11-08: Should not store exc_info in exception instances.
+        try:
+            self.assertStartsWith(
+                str(err), "The message handler raised an exception:\n")
+            self.assertEndsWith(str(err), "Exception: example error\n")
+        finally:
+            del err
 
     def test_must_have_working_tree(self):
         err = errors.MustHaveWorkingTree('foo', 'bar')
@@ -665,6 +682,15 @@
         err = errors.NotBranchError('path', bzrdir=bzrdir)
         self.assertEqual('Not a branch: "path".', str(err))
 
+    def test_not_branch_bzrdir_with_recursive_not_branch_error(self):
+        class FakeBzrDir(object):
+            def open_repository(self):
+                # str() on the NotBranchError will trigger a call to this,
+                # which in turn will another, identical NotBranchError.
+                raise errors.NotBranchError('path', bzrdir=FakeBzrDir())
+        err = errors.NotBranchError('path', bzrdir=FakeBzrDir())
+        self.assertEqual('Not a branch: "path".', str(err))
+
     def test_not_branch_laziness(self):
         real_bzrdir = self.make_bzrdir('path')
         class FakeBzrDir(object):

=== added file 'bzrlib/tests/test_estimate_compressed_size.py'
--- bzrlib/tests/test_estimate_compressed_size.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_estimate_compressed_size.py	2011-09-01 16:54:46 +0000
@@ -0,0 +1,79 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for our estimation of compressed content."""
+
+import zlib
+import hashlib
+
+from bzrlib import (
+    estimate_compressed_size,
+    tests,
+    )
+
+
+class TestZLibEstimator(tests.TestCase):
+
+    def get_slightly_random_content(self, length, seed=''):
+        """We generate some hex-data that can be seeded.
+
+        The output should be deterministic, but the data stream is effectively
+        random.
+        """
+        h = hashlib.md5(seed)
+        hex_content = []
+        count = 0
+        while count < length:
+            b = h.hexdigest()
+            hex_content.append(b)
+            h.update(b)
+            count += len(b)
+        return ''.join(hex_content)[:length]
+
+    def test_adding_content(self):
+        ze = estimate_compressed_size.ZLibEstimator(32000)
+        raw_data = self.get_slightly_random_content(60000)
+        block_size = 1000
+        for start in xrange(0, len(raw_data), block_size):
+            ze.add_content(raw_data[start:start+block_size])
+            if ze.full():
+                break
+        # Practise showed that 'start' was 56000. However, zlib is a bit
+        # platform dependent, so give it +/- 5%.
+        self.assertTrue(54000 <= start <= 58000,
+            "Unexpected amount of raw data added: %d bytes" % (start,))
+        # The real compression should be 'close' to 32000, real measurement was
+        # 32401
+        raw_comp = zlib.compress(raw_data[:start])
+        self.assertTrue(31000 < len(raw_comp) < 33000,
+            "Unexpected compressed size: %d bytes" % (len(raw_comp),))
+
+    def test_adding_more_content(self):
+        ze = estimate_compressed_size.ZLibEstimator(64000)
+        raw_data = self.get_slightly_random_content(150000)
+        block_size = 1000
+        for start in xrange(0, len(raw_data), block_size):
+            ze.add_content(raw_data[start:start+block_size])
+            if ze.full():
+                break
+        # Practise showed that 'start' was 112000
+        self.assertTrue(110000 <= start <= 114000,
+            "Unexpected amount of raw data added: %d bytes" % (start,))
+        # The real compression should be 'close' to 32000, real measurement was
+        # 32401
+        raw_comp = zlib.compress(raw_data[:start])
+        self.assertTrue(63000 < len(raw_comp) < 65000,
+            "Unexpected compressed size: %d bytes" % (len(raw_comp),))

=== modified file 'bzrlib/tests/test_export.py'
--- bzrlib/tests/test_export.py	2010-04-14 00:11:32 +0000
+++ bzrlib/tests/test_export.py	2011-07-08 23:01:39 +0000
@@ -14,65 +14,91 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+"""Tests for bzrlib.export."""
+
+from cStringIO import StringIO
 import os
+import tarfile
 import time
+import zipfile
 
 from bzrlib import (
     errors,
     export,
     tests,
     )
-
-
-class TestExport(tests.TestCaseWithTransport):
-
-    def test_dir_export_missing_file(self):
+from bzrlib.export import get_root_name
+from bzrlib.export.tar_exporter import export_tarball_generator
+from bzrlib.tests import features
+
+
+class TestDirExport(tests.TestCaseWithTransport):
+
+    def test_missing_file(self):
         self.build_tree(['a/', 'a/b', 'a/c'])
         wt = self.make_branch_and_tree('.')
         wt.add(['a', 'a/b', 'a/c'])
         os.unlink('a/c')
         export.export(wt, 'target', format="dir")
-        self.failUnlessExists('target/a/b')
-        self.failIfExists('target/a/c')
-
-    def test_dir_export_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.assertPathExists('target/a/b')
+        self.assertPathDoesNotExist('target/a/c')
+
+    def test_empty(self):
+        wt = self.make_branch_and_tree('.')
+        export.export(wt, 'target', format="dir")
+        self.assertEquals([], os.listdir("target"))
+
+    def test_symlink(self):
+        self.requireFeature(features.SymlinkFeature)
         wt = self.make_branch_and_tree('.')
         os.symlink('source', 'link')
         wt.add(['link'])
         export.export(wt, 'target', format="dir")
-        self.failUnlessExists('target/link')
+        self.assertPathExists('target/link')
 
-    def test_dir_export_to_existing_empty_dir_success(self):
+    def test_to_existing_empty_dir_success(self):
         self.build_tree(['source/', 'source/a', 'source/b/', 'source/b/c'])
         wt = self.make_branch_and_tree('source')
         wt.add(['a', 'b', 'b/c'])
         wt.commit('1')
         self.build_tree(['target/'])
         export.export(wt, 'target', format="dir")
-        self.failUnlessExists('target/a')
-        self.failUnlessExists('target/b')
-        self.failUnlessExists('target/b/c')
-
-    def test_dir_export_to_existing_nonempty_dir_fail(self):
+        self.assertPathExists('target/a')
+        self.assertPathExists('target/b')
+        self.assertPathExists('target/b/c')
+
+    def test_empty_subdir(self):
+        self.build_tree(['source/', 'source/a', 'source/b/', 'source/b/c'])
+        wt = self.make_branch_and_tree('source')
+        wt.add(['a', 'b', 'b/c'])
+        wt.commit('1')
+        self.build_tree(['target/'])
+        export.export(wt, 'target', format="dir", subdir='')
+        self.assertPathExists('target/a')
+        self.assertPathExists('target/b')
+        self.assertPathExists('target/b/c')
+
+    def test_to_existing_nonempty_dir_fail(self):
         self.build_tree(['source/', 'source/a', 'source/b/', 'source/b/c'])
         wt = self.make_branch_and_tree('source')
         wt.add(['a', 'b', 'b/c'])
         wt.commit('1')
         self.build_tree(['target/', 'target/foo'])
-        self.assertRaises(errors.BzrError, export.export, wt, 'target', format="dir")
+        self.assertRaises(errors.BzrError,
+            export.export, wt, 'target', format="dir")
 
-    def test_dir_export_existing_single_file(self):
-        self.build_tree(['dir1/', 'dir1/dir2/', 'dir1/first', 'dir1/dir2/second'])
+    def test_existing_single_file(self):
+        self.build_tree([
+            'dir1/', 'dir1/dir2/', 'dir1/first', 'dir1/dir2/second'])
         wtree = self.make_branch_and_tree('dir1')
         wtree.add(['dir2', 'first', 'dir2/second'])
         wtree.commit('1')
         export.export(wtree, 'target1', format='dir', subdir='first')
-        self.failUnlessExists('target1/first')
+        self.assertPathExists('target1/first')
         export.export(wtree, 'target2', format='dir', subdir='dir2/second')
-        self.failUnlessExists('target2/second')
-        
-    def test_dir_export_files_same_timestamp(self):
+        self.assertPathExists('target2/second')
+
+    def test_files_same_timestamp(self):
         builder = self.make_branch_builder('source')
         builder.start_series()
         builder.build_snapshot(None, None, [
@@ -86,6 +112,7 @@
         self.addCleanup(b.unlock)
         tree = b.basis_tree()
         orig_iter_files_bytes = tree.iter_files_bytes
+
         # Make iter_files_bytes slower, so we provoke mtime skew
         def iter_files_bytes(to_fetch):
             for thing in orig_iter_files_bytes(to_fetch):
@@ -99,7 +126,7 @@
         # All files must be given the same mtime.
         self.assertEqual(st_a.st_mtime, st_b.st_mtime)
 
-    def test_dir_export_files_per_file_timestamps(self):
+    def test_files_per_file_timestamps(self):
         builder = self.make_branch_builder('source')
         builder.start_series()
         # Earliest allowable date on FAT32 filesystems is 1980-01-01
@@ -121,3 +148,147 @@
         t = self.get_transport('target')
         self.assertEqual(a_time, t.stat('a').st_mtime)
         self.assertEqual(b_time, t.stat('b').st_mtime)
+
+    def test_subdir_files_per_timestamps(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        foo_time = time.mktime((1999, 12, 12, 0, 0, 0, 0, 0, 0))
+        builder.build_snapshot(None, None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('subdir', 'subdir-id', 'directory', '')),
+            ('add', ('subdir/foo.txt', 'foo-id', 'file', 'content\n'))],
+            timestamp=foo_time)
+        builder.finish_series()
+        b = builder.get_branch()
+        b.lock_read()
+        self.addCleanup(b.unlock)
+        tree = b.basis_tree()
+        export.export(tree, 'target', format='dir', subdir='subdir',
+            per_file_timestamps=True)
+        t = self.get_transport('target')
+        self.assertEquals(foo_time, t.stat('foo.txt').st_mtime)
+
+
+class TarExporterTests(tests.TestCaseWithTransport):
+
+    def test_xz(self):
+        self.requireFeature(features.lzma)
+        import lzma
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        wt.add(["a"])
+        wt.commit("1")
+        export.export(wt, 'target.tar.xz', format="txz")
+        tf = tarfile.open(fileobj=lzma.LZMAFile('target.tar.xz'))
+        self.assertEquals(["target/a"], tf.getnames())
+
+    def test_lzma(self):
+        self.requireFeature(features.lzma)
+        import lzma
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        wt.add(["a"])
+        wt.commit("1")
+        export.export(wt, 'target.tar.lzma', format="tlzma")
+        tf = tarfile.open(fileobj=lzma.LZMAFile('target.tar.lzma'))
+        self.assertEquals(["target/a"], tf.getnames())
+
+    def test_tgz(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        wt.add(["a"])
+        wt.commit("1")
+        export.export(wt, 'target.tar.gz', format="tgz")
+        tf = tarfile.open('target.tar.gz')
+        self.assertEquals(["target/a"], tf.getnames())
+
+    def test_tgz_ignores_dest_path(self):
+        # The target path should not be a part of the target file.
+        # (bug #102234)
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        wt.add(["a"])
+        wt.commit("1")
+        os.mkdir("testdir1")
+        os.mkdir("testdir2")
+        export.export(wt, 'testdir1/target.tar.gz', format="tgz",
+            per_file_timestamps=True)
+        export.export(wt, 'testdir2/target.tar.gz', format="tgz",
+            per_file_timestamps=True)
+        file1 = open('testdir1/target.tar.gz', 'r')
+        self.addCleanup(file1.close)
+        file2 = open('testdir1/target.tar.gz', 'r')
+        self.addCleanup(file2.close)
+        content1 = file1.read()
+        content2 = file2.read()
+        self.assertEqualDiff(content1, content2)
+        # the gzip module doesn't have a way to read back to the original
+        # filename, but it's stored as-is in the tarfile.
+        self.assertFalse("testdir1" in content1)
+        self.assertFalse("target.tar.gz" in content1)
+        self.assertTrue("target.tar" in content1)
+
+    def test_tbz2(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        wt.add(["a"])
+        wt.commit("1")
+        export.export(wt, 'target.tar.bz2', format="tbz2")
+        tf = tarfile.open('target.tar.bz2')
+        self.assertEquals(["target/a"], tf.getnames())
+
+    def test_xz_stdout(self):
+        wt = self.make_branch_and_tree('.')
+        self.assertRaises(errors.BzrError, export.export, wt, '-',
+            format="txz")
+
+    def test_export_tarball_generator(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        wt.add(["a"])
+        wt.commit("1", timestamp=42)
+        target = StringIO()
+        ball = tarfile.open(None, "w|", target)
+        wt.lock_read()
+        try:
+            for _ in export_tarball_generator(wt, ball, "bar"):
+                pass
+        finally:
+            wt.unlock()
+        # Ball should now be closed.
+        target.seek(0)
+        ball2 = tarfile.open(None, "r", target)
+        self.addCleanup(ball2.close)
+        self.assertEquals(["bar/a"], ball2.getnames())
+
+
+class ZipExporterTests(tests.TestCaseWithTransport):
+
+    def test_per_file_timestamps(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('har', 'foo')])
+        tree.add('har')
+        # Earliest allowable date on FAT32 filesystems is 1980-01-01
+        timestamp = 347151600
+        tree.commit('setup', timestamp=timestamp)
+        export.export(tree.basis_tree(), 'test.zip', format='zip',
+            per_file_timestamps=True)
+        zfile = zipfile.ZipFile('test.zip')
+        info = zfile.getinfo("test/har")
+        self.assertEquals(time.localtime(timestamp)[:6], info.date_time)
+
+
+class RootNameTests(tests.TestCase):
+
+    def test_root_name(self):
+        self.assertEquals('mytest', get_root_name('../mytest.tar'))
+        self.assertEquals('mytar', get_root_name('mytar.tar'))
+        self.assertEquals('mytar', get_root_name('mytar.tar.bz2'))
+        self.assertEquals('tar.tar.tar', get_root_name('tar.tar.tar.tgz'))
+        self.assertEquals('bzr-0.0.5', get_root_name('bzr-0.0.5.tar.gz'))
+        self.assertEquals('bzr-0.0.5', get_root_name('bzr-0.0.5.zip'))
+        self.assertEquals('bzr-0.0.5', get_root_name('bzr-0.0.5'))
+        self.assertEquals('mytar', get_root_name('a/long/path/mytar.tgz'))
+        self.assertEquals('other',
+            get_root_name('../parent/../dir/other.tbz2'))
+        self.assertEquals('', get_root_name('-'))

=== added file 'bzrlib/tests/test_export_pot.py'
--- bzrlib/tests/test_export_pot.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_export_pot.py	2011-06-28 10:28:29 +0000
@@ -0,0 +1,193 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from cStringIO import StringIO
+import textwrap
+
+from bzrlib import (
+    commands,
+    export_pot,
+    tests,
+    )
+
+import re
+
+
+class TestEscape(tests.TestCase):
+
+    def test_simple_escape(self):
+        self.assertEqual(
+                export_pot._escape('foobar'),
+                'foobar')
+
+        s = '''foo\nbar\r\tbaz\\"spam"'''
+        e = '''foo\\nbar\\r\\tbaz\\\\\\"spam\\"'''
+        self.assertEqual(export_pot._escape(s), e)
+
+    def test_complex_escape(self):
+        s = '''\\r \\\n'''
+        e = '''\\\\r \\\\\\n'''
+        self.assertEqual(export_pot._escape(s), e)
+
+
+class TestNormalize(tests.TestCase):
+
+    def test_single_line(self):
+        s = 'foobar'
+        e = '"foobar"'
+        self.assertEqual(export_pot._normalize(s), e)
+
+        s = 'foo"bar'
+        e = '"foo\\"bar"'
+        self.assertEqual(export_pot._normalize(s), e)
+
+    def test_multi_lines(self):
+        s = 'foo\nbar\n'
+        e = '""\n"foo\\n"\n"bar\\n"'
+        self.assertEqual(export_pot._normalize(s), e)
+
+        s = '\nfoo\nbar\n'
+        e = ('""\n'
+             '"\\n"\n'
+             '"foo\\n"\n'
+             '"bar\\n"')
+        self.assertEqual(export_pot._normalize(s), e)
+
+
+class PoEntryTestCase(tests.TestCase):
+
+    def setUp(self):
+        self.overrideAttr(export_pot, '_FOUND_MSGID', set())
+        self._outf = StringIO()
+        super(PoEntryTestCase, self).setUp()
+
+    def check_output(self, expected):
+        self.assertEqual(
+                self._outf.getvalue(),
+                textwrap.dedent(expected)
+                )
+
+class TestPoEntry(PoEntryTestCase):
+
+    def test_simple(self):
+        export_pot._poentry(self._outf, 'dummy', 1, "spam")
+        export_pot._poentry(self._outf, 'dummy', 2, "ham", 'EGG')
+        self.check_output('''\
+                #: dummy:1
+                msgid "spam"
+                msgstr ""
+
+                #: dummy:2
+                # EGG
+                msgid "ham"
+                msgstr ""
+
+                ''')
+
+    def test_duplicate(self):
+        export_pot._poentry(self._outf, 'dummy', 1, "spam")
+        # This should be ignored.
+        export_pot._poentry(self._outf, 'dummy', 2, "spam", 'EGG')
+
+        self.check_output('''\
+                #: dummy:1
+                msgid "spam"
+                msgstr ""\n
+                ''')
+
+
+class TestPoentryPerPergraph(PoEntryTestCase):
+
+    def test_single(self):
+        export_pot._poentry_per_paragraph(
+                self._outf,
+                'dummy',
+                10,
+                '''foo\nbar\nbaz\n'''
+                )
+        self.check_output('''\
+                #: dummy:10
+                msgid ""
+                "foo\\n"
+                "bar\\n"
+                "baz\\n"
+                msgstr ""\n
+                ''')
+
+    def test_multi(self):
+        export_pot._poentry_per_paragraph(
+                self._outf,
+                'dummy',
+                10,
+                '''spam\nham\negg\n\nSPAM\nHAM\nEGG\n'''
+                )
+        self.check_output('''\
+                #: dummy:10
+                msgid ""
+                "spam\\n"
+                "ham\\n"
+                "egg"
+                msgstr ""
+
+                #: dummy:14
+                msgid ""
+                "SPAM\\n"
+                "HAM\\n"
+                "EGG\\n"
+                msgstr ""\n
+                ''')
+
+
+class TestExportCommandHelp(PoEntryTestCase):
+
+    def test_command_help(self):
+
+        class cmd_Demo(commands.Command):
+            __doc__ = """A sample command.
+
+            :Usage:
+                bzr demo
+
+            :Examples:
+                Example 1::
+
+                    cmd arg1
+
+            Blah Blah Blah
+            """
+
+        export_pot._write_command_help(self._outf, cmd_Demo())
+        result = self._outf.getvalue()
+        # We don't care about filename and lineno here.
+        result = re.sub(r'(?m)^#: [^\n]+\n', '', result)
+
+        self.assertEqualDiff(
+                'msgid "A sample command."\n'
+                'msgstr ""\n'
+                '\n'                # :Usage: should not be translated.
+                'msgid ""\n'
+                '":Examples:\\n"\n'
+                '"    Example 1::"\n'
+                'msgstr ""\n'
+                '\n'
+                'msgid "        cmd arg1"\n'
+                'msgstr ""\n'
+                '\n'
+                'msgid "Blah Blah Blah"\n'
+                'msgstr ""\n'
+                '\n',
+                result
+                )

=== modified file 'bzrlib/tests/test_extract.py'
--- bzrlib/tests/test_extract.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_extract.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 from bzrlib import (
     branch,
-    bzrdir,
     errors,
     )
 from bzrlib.tests import TestCaseWithTransport

=== added file 'bzrlib/tests/test_features.py'
--- bzrlib/tests/test_features.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_features.py	2011-07-11 06:47:32 +0000
@@ -0,0 +1,115 @@
+# Copyright (C) 2005-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for test feature dependencies.
+"""
+
+from bzrlib import (
+    symbol_versioning,
+    tests,
+    )
+from bzrlib.tests import (
+    features,
+    )
+
+
+class TestFeature(tests.TestCase):
+
+    def test_caching(self):
+        """Feature._probe is called by the feature at most once."""
+        class InstrumentedFeature(features.Feature):
+            def __init__(self):
+                super(InstrumentedFeature, self).__init__()
+                self.calls = []
+
+            def _probe(self):
+                self.calls.append('_probe')
+                return False
+        feature = InstrumentedFeature()
+        feature.available()
+        self.assertEqual(['_probe'], feature.calls)
+        feature.available()
+        self.assertEqual(['_probe'], feature.calls)
+
+    def test_named_str(self):
+        """Feature.__str__ should thunk to feature_name()."""
+        class NamedFeature(features.Feature):
+            def feature_name(self):
+                return 'symlinks'
+        feature = NamedFeature()
+        self.assertEqual('symlinks', str(feature))
+
+    def test_default_str(self):
+        """Feature.__str__ should default to __class__.__name__."""
+        class NamedFeature(features.Feature):
+            pass
+        feature = NamedFeature()
+        self.assertEqual('NamedFeature', str(feature))
+
+
+class TestUnavailableFeature(tests.TestCase):
+
+    def test_access_feature(self):
+        feature = features.Feature()
+        exception = tests.UnavailableFeature(feature)
+        self.assertIs(feature, exception.args[0])
+
+
+# Although this was deprecated a long time ago, please keep it here because
+# it's really just a test fixture for test-feature deprecation.
+simple_thunk_feature = features._CompatabilityThunkFeature(
+    symbol_versioning.deprecated_in((2, 1, 0)),
+    'bzrlib.tests.test_features',
+    'simple_thunk_feature',
+    'UnicodeFilenameFeature',
+    replacement_module='bzrlib.tests.features')
+
+
+class Test_CompatibilityFeature(tests.TestCase):
+
+    def test_does_thunk(self):
+        res = self.callDeprecated(
+            ['bzrlib.tests.test_features.simple_thunk_feature '
+             'was deprecated in version 2.1.0. '
+             'Use bzrlib.tests.features.UnicodeFilenameFeature instead.'],
+            simple_thunk_feature.available)
+        self.assertEqual(features.UnicodeFilenameFeature.available(), res)
+
+
+class TestModuleAvailableFeature(tests.TestCase):
+
+    def test_available_module(self):
+        feature = features.ModuleAvailableFeature('bzrlib.tests')
+        self.assertEqual('bzrlib.tests', feature.module_name)
+        self.assertEqual('bzrlib.tests', str(feature))
+        self.assertTrue(feature.available())
+        self.assertIs(tests, feature.module)
+
+    def test_unavailable_module(self):
+        feature = features.ModuleAvailableFeature(
+            'bzrlib.no_such_module_exists')
+        self.assertEqual('bzrlib.no_such_module_exists', str(feature))
+        self.assertFalse(feature.available())
+        self.assertIs(None, feature.module)
+
+
+class TestUnicodeFilenameFeature(tests.TestCase):
+
+    def test_probe_passes(self):
+        """UnicodeFilenameFeature._probe passes."""
+        # We can't test much more than that because the behaviour depends
+        # on the platform.
+        features.UnicodeFilenameFeature._probe()

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2010-02-09 20:18:32 +0000
+++ bzrlib/tests/test_fetch.py	2011-10-14 12:49:33 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007, 2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,25 +14,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-import re
-import sys
-
-import bzrlib
 from bzrlib import (
     bzrdir,
     errors,
     osutils,
-    merge,
-    repository,
+    revision as _mod_revision,
     versionedfile,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests.test_revision import make_branches
-from bzrlib.trace import mutter
 from bzrlib.upgrade import Convert
 from bzrlib.workingtree import WorkingTree
 
@@ -44,24 +36,37 @@
 def has_revision(branch, revision_id):
     return branch.repository.has_revision(revision_id)
 
+
+def revision_history(branch):
+    branch.lock_read()
+    try:
+        graph = branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+    finally:
+        branch.unlock()
+    history.reverse()
+    return history
+
+
 def fetch_steps(self, br_a, br_b, writable_a):
     """A foreign test method for testing fetch locally and remotely."""
 
     # TODO RBC 20060201 make this a repository test.
     repo_b = br_b.repository
-    self.assertFalse(repo_b.has_revision(br_a.revision_history()[3]))
-    self.assertTrue(repo_b.has_revision(br_a.revision_history()[2]))
-    self.assertEquals(len(br_b.revision_history()), 7)
-    br_b.fetch(br_a, br_a.revision_history()[2])
+    self.assertFalse(repo_b.has_revision(revision_history(br_a)[3]))
+    self.assertTrue(repo_b.has_revision(revision_history(br_a)[2]))
+    self.assertEquals(len(revision_history(br_b)), 7)
+    br_b.fetch(br_a, revision_history(br_a)[2])
     # branch.fetch is not supposed to alter the revision history
-    self.assertEquals(len(br_b.revision_history()), 7)
-    self.assertFalse(repo_b.has_revision(br_a.revision_history()[3]))
+    self.assertEquals(len(revision_history(br_b)), 7)
+    self.assertFalse(repo_b.has_revision(revision_history(br_a)[3]))
 
     # fetching the next revision up in sample data copies one revision
-    br_b.fetch(br_a, br_a.revision_history()[3])
-    self.assertTrue(repo_b.has_revision(br_a.revision_history()[3]))
-    self.assertFalse(has_revision(br_a, br_b.revision_history()[6]))
-    self.assertTrue(br_a.repository.has_revision(br_b.revision_history()[5]))
+    br_b.fetch(br_a, revision_history(br_a)[3])
+    self.assertTrue(repo_b.has_revision(revision_history(br_a)[3]))
+    self.assertFalse(has_revision(br_a, revision_history(br_b)[6]))
+    self.assertTrue(br_a.repository.has_revision(revision_history(br_b)[5]))
 
     # When a non-branch ancestor is missing, it should be unlisted...
     # as its not reference from the inventory weave.
@@ -69,20 +74,20 @@
     br_b4.fetch(br_b)
 
     writable_a.fetch(br_b)
-    self.assertTrue(has_revision(br_a, br_b.revision_history()[3]))
-    self.assertTrue(has_revision(br_a, br_b.revision_history()[4]))
+    self.assertTrue(has_revision(br_a, revision_history(br_b)[3]))
+    self.assertTrue(has_revision(br_a, revision_history(br_b)[4]))
 
     br_b2 = self.make_branch('br_b2')
     br_b2.fetch(br_b)
-    self.assertTrue(has_revision(br_b2, br_b.revision_history()[4]))
-    self.assertTrue(has_revision(br_b2, br_a.revision_history()[2]))
-    self.assertFalse(has_revision(br_b2, br_a.revision_history()[3]))
+    self.assertTrue(has_revision(br_b2, revision_history(br_b)[4]))
+    self.assertTrue(has_revision(br_b2, revision_history(br_a)[2]))
+    self.assertFalse(has_revision(br_b2, revision_history(br_a)[3]))
 
     br_a2 = self.make_branch('br_a2')
     br_a2.fetch(br_a)
-    self.assertTrue(has_revision(br_a2, br_b.revision_history()[4]))
-    self.assertTrue(has_revision(br_a2, br_a.revision_history()[3]))
-    self.assertTrue(has_revision(br_a2, br_a.revision_history()[2]))
+    self.assertTrue(has_revision(br_a2, revision_history(br_b)[4]))
+    self.assertTrue(has_revision(br_a2, revision_history(br_a)[3]))
+    self.assertTrue(has_revision(br_a2, revision_history(br_a)[2]))
 
     br_a3 = self.make_branch('br_a3')
     # pulling a branch with no revisions grabs nothing, regardless of
@@ -90,10 +95,10 @@
     br_a3.fetch(br_a2)
     for revno in range(4):
         self.assertFalse(
-            br_a3.repository.has_revision(br_a.revision_history()[revno]))
-    br_a3.fetch(br_a2, br_a.revision_history()[2])
+            br_a3.repository.has_revision(revision_history(br_a)[revno]))
+    br_a3.fetch(br_a2, revision_history(br_a)[2])
     # pull the 3 revisions introduced by a@u-0-3
-    br_a3.fetch(br_a2, br_a.revision_history()[3])
+    br_a3.fetch(br_a2, revision_history(br_a)[3])
     # NoSuchRevision should be raised if the branch is missing the revision
     # that was requested.
     self.assertRaises(errors.NoSuchRevision, br_a3.fetch, br_a2, 'pizza')

=== modified file 'bzrlib/tests/test_fifo_cache.py'
--- bzrlib/tests/test_fifo_cache.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_fifo_cache.py	2011-06-14 01:43:10 +0000
@@ -229,11 +229,11 @@
         c = fifo_cache.FIFOCache()
         c.add(1, 2, cleanup=logging_cleanup)
         del c
-        # TODO: We currently don't support calling the cleanup() funcs during
-        #       __del__. We might want to consider implementing this.
-        self.expectFailure("we don't call cleanups during __del__",
-                           self.assertEqual, [(1, 2)], log)
-        self.assertEqual([(1, 2)], log)
+        # As a matter of design, bzr does not (can not) count on anything
+        # being run from Python __del__ methods, because they may not run for
+        # a long time, and because in cPython merely having them defined
+        # interferes with garbage collection.
+        self.assertEqual([], log)
 
 
 class TestFIFOSizeCache(tests.TestCase):

=== added file 'bzrlib/tests/test_filter_tree.py'
--- bzrlib/tests/test_filter_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_filter_tree.py	2011-07-04 21:15:24 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for ContentFilterTree"""
+
+import tarfile
+import zipfile
+
+from bzrlib import (
+    export,
+    filter_tree,
+    tests,
+    )
+from bzrlib.tests import (
+    fixtures,
+    )
+from bzrlib.tests.test_filters import _stack_1
+
+
+class TestFilterTree(tests.TestCaseWithTransport):
+
+    def make_tree(self):
+        self.underlying_tree = fixtures.make_branch_and_populated_tree(
+            self)
+        def stack_callback(path):
+            return _stack_1
+        self.filter_tree = filter_tree.ContentFilterTree(
+            self.underlying_tree, stack_callback)
+        return self.filter_tree
+
+    def test_get_file_text(self):
+        self.make_tree()
+        self.assertEquals(
+            self.underlying_tree.get_file_text('hello-id'),
+            'hello world')
+        self.assertEquals(
+            self.filter_tree.get_file_text('hello-id'),
+            'HELLO WORLD')
+
+    def test_tar_export_content_filter_tree(self):
+        # TODO: this could usefully be run generically across all exporters.
+        self.make_tree()
+        export.export(self.filter_tree, "out.tgz")
+        ball = tarfile.open("out.tgz", "r:gz")
+        self.assertEquals(
+            'HELLO WORLD',
+            ball.extractfile('out/hello').read())
+
+    def test_zip_export_content_filter_tree(self):
+        self.make_tree()
+        export.export(self.filter_tree, 'out.zip')
+        zipf = zipfile.ZipFile('out.zip', 'r')
+        self.assertEquals(
+            'HELLO WORLD',
+            zipf.read('out/hello'))

=== modified file 'bzrlib/tests/test_foreign.py'
--- bzrlib/tests/test_foreign.py	2010-04-23 00:44:15 +0000
+++ bzrlib/tests/test_foreign.py	2011-10-14 15:45:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,15 +21,20 @@
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     foreign,
     lockable_files,
     lockdir,
+    repository,
     revision,
     tests,
     trace,
+    vf_repository,
     )
 
+from bzrlib.repofmt import groupcompress_repo
+
 # This is the dummy foreign revision control system, used 
 # mainly here in the testsuite to test the foreign VCS infrastructure.
 # It is basically standard Bazaar with some minor modifications to 
@@ -91,32 +96,87 @@
         self._base = a_bzrdir.transport.base
         self._ignore_fallbacks = False
         self.bzrdir = a_bzrdir
-        foreign.ForeignBranch.__init__(self, 
+        foreign.ForeignBranch.__init__(self,
             DummyForeignVcsMapping(DummyForeignVcs()))
-        branch.BzrBranch6.__init__(self, _format, _control_files, a_bzrdir, 
+        branch.BzrBranch6.__init__(self, _format, _control_files, a_bzrdir,
             *args, **kwargs)
 
-
-class InterToDummyVcsBranch(branch.GenericInterBranch,
-                            foreign.InterToForeignBranch):
+    def _get_checkout_format(self, lightweight=False):
+        """Return the most suitable metadir for a checkout of this branch.
+        Weaves are used if this branch's repository uses weaves.
+        """
+        return self.bzrdir.checkout_metadir()
+
+    def import_last_revision_info_and_tags(self, source, revno, revid,
+                                           lossy=False):
+        interbranch = InterToDummyVcsBranch(source, self)
+        result = interbranch.push(stop_revision=revid, lossy=True)
+        if lossy:
+            revid = result.revidmap[revid]
+        return (revno, revid)
+
+
+class DummyForeignCommitBuilder(vf_repository.VersionedFileRootCommitBuilder):
+
+    def _generate_revision_if_needed(self):
+        mapping = DummyForeignVcsMapping(DummyForeignVcs())
+        if self._lossy:
+            self._new_revision_id = mapping.revision_id_foreign_to_bzr(
+                (str(self._timestamp), str(self._timezone), "UNKNOWN"))
+            self.random_revid = False
+        elif self._new_revision_id is not None:
+            self.random_revid = False
+        else:
+            self._new_revision_id = self._gen_revision_id()
+            self.random_revid = True
+
+
+class DummyForeignVcsRepository(groupcompress_repo.CHKInventoryRepository,
+    foreign.ForeignRepository):
+    """Dummy foreign vcs repository."""
+
+
+class DummyForeignVcsRepositoryFormat(groupcompress_repo.RepositoryFormat2a):
+
+    repository_class = DummyForeignVcsRepository
+    _commit_builder_class = DummyForeignCommitBuilder
+
+    def get_format_string(self):
+        return "Dummy Foreign Vcs Repository"
+
+    def get_format_description(self):
+        return "Dummy Foreign Vcs Repository"
+
+
+def branch_history(graph, revid):
+    ret = list(graph.iter_lefthand_ancestry(revid,
+        (revision.NULL_REVISION,)))
+    ret.reverse()
+    return ret
+
+
+class InterToDummyVcsBranch(branch.GenericInterBranch):
 
     @staticmethod
     def is_compatible(source, target):
         return isinstance(target, DummyForeignVcsBranch)
 
-    def push(self, overwrite=False, stop_revision=None):
-        raise errors.NoRoundtrippingSupport(self.source, self.target)
-
-    def lossy_push(self, stop_revision=None):
+    def push(self, overwrite=False, stop_revision=None, lossy=False):
+        if not lossy:
+            raise errors.NoRoundtrippingSupport(self.source, self.target)
         result = branch.BranchPushResult()
         result.source_branch = self.source
         result.target_branch = self.target
         result.old_revno, result.old_revid = self.target.last_revision_info()
         self.source.lock_read()
         try:
+            graph = self.source.repository.get_graph()
             # This just handles simple cases, but that's good enough for tests
-            my_history = self.target.revision_history()
-            their_history = self.source.revision_history()
+            my_history = branch_history(self.target.repository.get_graph(),
+                result.old_revid)
+            if stop_revision is None:
+                stop_revision = self.source.last_revision()
+            their_history = branch_history(graph, stop_revision)
             if their_history[:min(len(my_history), len(their_history))] != my_history:
                 raise errors.DivergedBranches(self.target, self.source)
             todo = their_history[len(my_history):]
@@ -128,7 +188,7 @@
                     return (tree.get_file(file_id), None)
                 tree.get_file_with_stat = get_file_with_stat
                 new_revid = self.target.mapping.revision_id_foreign_to_bzr(
-                    (str(rev.timestamp), str(rev.timezone), 
+                    (str(rev.timestamp), str(rev.timezone),
                         str(self.target.revno())))
                 parent_revno, parent_revid= self.target.last_revision_info()
                 if parent_revid == revision.NULL_REVISION:
@@ -168,21 +228,24 @@
     def get_format_string(self):
         return "Branch for Testing"
 
-    def __init__(self):
-        super(DummyForeignVcsBranchFormat, self).__init__()
-        self._matchingbzrdir = DummyForeignVcsDirFormat()
+    @property
+    def _matchingbzrdir(self):
+        return DummyForeignVcsDirFormat()
 
-    def open(self, a_bzrdir, name=None, _found=False):
+    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
+            found_repository=None):
         if not _found:
             raise NotImplementedError
         try:
             transport = a_bzrdir.get_branch_transport(None, name=name)
             control_files = lockable_files.LockableFiles(transport, 'lock',
                                                          lockdir.LockDir)
+            if found_repository is None:
+                found_repository = a_bzrdir.find_repository()
             return DummyForeignVcsBranch(_format=self,
                               _control_files=control_files,
                               a_bzrdir=a_bzrdir,
-                              _repository=a_bzrdir.find_repository())
+                              _repository=found_repository)
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base)
 
@@ -205,12 +268,9 @@
     def get_branch_format(self):
         return DummyForeignVcsBranchFormat()
 
-    @classmethod
-    def probe_transport(klass, transport):
-        """Return the .bzrdir style format present in a directory."""
-        if not transport.has('.dummy'):
-            raise errors.NotBranchError(path=transport.base)
-        return klass()
+    @property
+    def repository_format(self):
+        return DummyForeignVcsRepositoryFormat()
 
     def initialize_on_transport(self, transport):
         """Initialize a new bzrdir in the base directory of a Transport."""
@@ -244,6 +304,11 @@
         self._control_files = lockable_files.LockableFiles(self.transport,
             "lock", lockable_files.TransportLock)
 
+    def create_workingtree(self):
+        # dirstate requires a ".bzr" entry to exist
+        self.root_transport.put_bytes(".bzr", "foo")
+        return super(DummyForeignVcsDir, self).create_workingtree()
+
     def open_branch(self, name=None, unsupported=False, ignore_fallbacks=True):
         if name is not None:
             raise errors.NoColocatedBranchSupport(self)
@@ -253,6 +318,9 @@
         """Produce a metadir suitable for cloning with."""
         return bzrdir.format_registry.make_bzrdir("default")
 
+    def checkout_metadir(self):
+        return self.cloning_metadir()
+
     def sprout(self, url, revision_id=None, force_new_repo=False,
                recurse='down', possible_transports=None,
                accelerator_tree=None, hardlink=False, stacked=False,
@@ -266,9 +334,15 @@
 
 
 def register_dummy_foreign_for_test(testcase):
-    bzrdir.BzrDirFormat.register_control_format(DummyForeignVcsDirFormat)
-    testcase.addCleanup(bzrdir.BzrDirFormat.unregister_control_format,
-                        DummyForeignVcsDirFormat)
+    controldir.ControlDirFormat.register_prober(DummyForeignProber)
+    testcase.addCleanup(controldir.ControlDirFormat.unregister_prober,
+        DummyForeignProber)
+    repository.format_registry.register(DummyForeignVcsRepositoryFormat())
+    testcase.addCleanup(repository.format_registry.remove,
+            DummyForeignVcsRepositoryFormat())
+    branch.format_registry.register(DummyForeignVcsBranchFormat())
+    testcase.addCleanup(branch.format_registry.remove,
+            DummyForeignVcsBranchFormat())
     # We need to register the optimiser to make the dummy appears really
     # different from a regular bzr repository.
     branch.InterBranch.register_optimiser(InterToDummyVcsBranch)
@@ -276,6 +350,20 @@
                         InterToDummyVcsBranch)
 
 
+class DummyForeignProber(controldir.Prober):
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return the .bzrdir style format present in a directory."""
+        if not transport.has('.dummy'):
+            raise errors.NotBranchError(path=transport.base)
+        return DummyForeignVcsDirFormat()
+
+    @classmethod
+    def known_formats(cls):
+        return set([DummyForeignVcsDirFormat()])
+
+
 class ForeignVcsRegistryTests(tests.TestCase):
     """Tests for the ForeignVcsRegistry class."""
 
@@ -293,8 +381,9 @@
         reg = foreign.ForeignVcsRegistry()
         vcs = DummyForeignVcs()
         reg.register("dummy", vcs, "Dummy VCS")
-        self.assertEquals((("some", "foreign", "revid"), DummyForeignVcsMapping(vcs)),
-                          reg.parse_revision_id("dummy-v1:some-foreign-revid"))
+        self.assertEquals((
+            ("some", "foreign", "revid"), DummyForeignVcsMapping(vcs)),
+            reg.parse_revision_id("dummy-v1:some-foreign-revid"))
 
 
 class ForeignRevisionTests(tests.TestCase):
@@ -368,7 +457,7 @@
         source_tree = self.make_branch_and_tree("source")
         target_tree = self.make_branch_and_tree("target", 
             format=DummyForeignVcsDirFormat())
-        pushresult = source_tree.branch.lossy_push(target_tree.branch)
+        pushresult = source_tree.branch.push(target_tree.branch, lossy=True)
         self.assertEquals(revision.NULL_REVISION, pushresult.old_revid)
         self.assertEquals(revision.NULL_REVISION, pushresult.new_revid)
         self.assertEquals({}, pushresult.revidmap)
@@ -382,7 +471,8 @@
             format=DummyForeignVcsDirFormat())
         target_tree.branch.lock_write()
         try:
-            pushresult = source_tree.branch.lossy_push(target_tree.branch)
+            pushresult = source_tree.branch.push(
+                target_tree.branch, lossy=True)
         finally:
             target_tree.branch.unlock()
         self.assertEquals(revision.NULL_REVISION, pushresult.old_revid)

=== modified file 'bzrlib/tests/test_ftp_transport.py'
--- bzrlib/tests/test_ftp_transport.py	2010-02-27 01:34:49 +0000
+++ bzrlib/tests/test_ftp_transport.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2010 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,10 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 import ftplib
 import getpass
-import sys
+import urllib
 
 from bzrlib import (
     config,
@@ -45,10 +44,10 @@
     """Test aftp transport."""
 
     def test_aftp_degrade(self):
-        t = transport.get_transport('aftp://host/path')
-        self.failUnless(t.is_active)
+        t = transport.get_transport_from_url('aftp://host/path')
+        self.assertTrue(t.is_active)
         parent = t.clone('..')
-        self.failUnless(parent.is_active)
+        self.assertTrue(parent.is_active)
 
         self.assertEqual('aftp://host/path', t.abspath(''))
 
@@ -83,11 +82,13 @@
     def get_url(self, relpath=None):
         """Overrides get_url to inject our user."""
         base = super(TestFTPTestServerUI, self).get_url(relpath)
-        (scheme, user, password,
-         host, port, path) = transport.ConnectedTransport._split_url(base)
-        url = transport.ConnectedTransport._unsplit_url(
-            scheme, self.user, self.password, host, port, path)
-        return url
+        parsed_url = transport.ConnectedTransport._split_url(base)
+        new_url = parsed_url.clone()
+        new_url.user = self.user
+        new_url.quoted_user = urllib.quote(self.user)
+        new_url.password = self.password
+        new_url.quoted_password = urllib.quote(self.password)
+        return str(new_url)
 
     def test_no_prompt_for_username(self):
         """ensure getpass.getuser() is used if there's no username in the 

=== modified file 'bzrlib/tests/test_generate_docs.py'
--- bzrlib/tests/test_generate_docs.py	2009-06-19 09:06:56 +0000
+++ bzrlib/tests/test_generate_docs.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 from cStringIO import StringIO
 
 import bzrlib.commands
-from bzrlib.tests import TestCase, TestSkipped
+from bzrlib.tests import TestCase
 
 
 class Options:

=== modified file 'bzrlib/tests/test_generate_ids.py'
--- bzrlib/tests/test_generate_ids.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_generate_ids.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,8 +16,6 @@
 
 """Tests for bzrlib/generate_ids.py"""
 
-import re
-
 from bzrlib import (
     generate_ids,
     tests,
@@ -64,13 +62,13 @@
         # Test both case squashing and length restriction
         fid = gen_file_id('A'*50 + '.txt')
         self.assertStartsWith(fid, 'a'*20 + '-')
-        self.failUnless(len(fid) < 60)
+        self.assertTrue(len(fid) < 60)
 
         # restricting length happens after the other actions, so
         # we preserve as much as possible
         fid = gen_file_id('\xe5\xb5..aBcd\tefGhijKLMnop\tqrstuvwxyz')
         self.assertStartsWith(fid, 'abcdefghijklmnopqrst-')
-        self.failUnless(len(fid) < 60)
+        self.assertTrue(len(fid) < 60)
 
     def test_file_ids_are_ascii(self):
         tail = r'-\d{14}-[a-z0-9]{16}-\d+'

=== modified file 'bzrlib/tests/test_globbing.py'
--- bzrlib/tests/test_globbing.py	2010-07-19 14:25:37 +0000
+++ bzrlib/tests/test_globbing.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,6 @@
     )
 from bzrlib.tests import (
     TestCase,
-    TestCaseInTempDir,
     )
 
 
@@ -38,11 +37,11 @@
                 glob = glob_prefix + glob
             globster = Globster([glob])
             for name in positive:
-                self.failUnless(globster.match(name), repr(
+                self.assertTrue(globster.match(name), repr(
                     u'name "%s" does not match glob "%s" (re=%s)' %
                     (name, glob, globster._regex_patterns[0][0].pattern)))
             for name in negative:
-                self.failIf(globster.match(name), repr(
+                self.assertFalse(globster.match(name), repr(
                     u'name "%s" does match glob "%s" (re=%s)' %
                     (name, glob, globster._regex_patterns[0][0].pattern)))
 
@@ -55,14 +54,16 @@
     def test_char_group_digit(self):
         self.assertMatchBasenameAndFullpath([
             # The definition of digit this uses includes arabic digits from
-            # non-latin scripts (arabic, indic, etc.) and subscript/superscript
-            # digits, but neither roman numerals nor vulgar fractions.
+            # non-latin scripts (arabic, indic, etc.) but neither roman
+            # numerals nor vulgar fractions. Some characters such as
+            # subscript/superscript digits may or may not match depending on
+            # the Python version used, see: <http://bugs.python.org/issue6561>
             (u'[[:digit:]]',
-             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21', u'\xb9'],
+             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21'],
              [u'T', u'q', u' ', u'\u8336', u'.']),
             (u'[^[:digit:]]',
              [u'T', u'q', u' ', u'\u8336', u'.'],
-             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21', u'\xb9']),
+             [u'0', u'5', u'\u0663', u'\u06f9', u'\u0f21']),
             ])
 
     def test_char_group_space(self):

=== modified file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_gpg.py	2011-09-07 23:01:19 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2011 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,24 +18,36 @@
 """Tests for signing and verifying blobs of data via gpg."""
 
 # import system imports here
-import os
 import sys
 
-from bzrlib import errors, ui
-import bzrlib.gpg as gpg
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib import (
+    errors,
+    gpg,
+    trace,
+    ui,
+    )
+from bzrlib.tests import (
+    TestCase,
+    features,
+    )
 
 class FakeConfig(object):
 
+    def gpg_signing_key(self):
+        return "amy@example.com"
+
     def gpg_signing_command(self):
         return "false"
 
+    def acceptable_keys(self):
+        return None
+
 
 class TestCommandLine(TestCase):
 
     def test_signing_command_line(self):
         my_gpg = gpg.GPGStrategy(FakeConfig())
-        self.assertEqual(['false',  '--clearsign'],
+        self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
                          my_gpg._command_line())
 
     def test_checks_return_code(self):
@@ -85,9 +97,408 @@
         self.assertRaises(errors.BzrBadParameterUnicode,
                           self.assertProduces, u'foo')
 
+class TestVerify(TestCase):
+
+    def import_keys(self):
+        from StringIO import StringIO
+        import gpgme
+        context = gpgme.Context()
+
+        key = StringIO("""-----BEGIN PGP PUBLIC KEY BLOCK-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+mQENBE343IgBCADwzPW7kmKb2bjB+UU+1ER/ABMZspvtoZMPusUw7bk6coXHF/0W
+u1K/hSYeX9xaGOfOQw41r/g13MoR9dsL6L84RLiisf38rRoBZt+d5bCbZA5Xo801
+2PeoBoGo6u5oOYKAFLMvrUitPiiE0IT/oQTfC4YUrLN4A+9W0QZruPGIpIXwmZXr
+L0zsqYfNqIN0ompeJenVpKpvm3loJ/zfK7R3EJ3hsv6nkUmWCFsP1Pw3UV1YuCmw
+Mkdn1U7DaOql1WjXgj9ABQDJrun2TGsqrSRzBODtHKA/uOX0K3VfKBU8VZo3dXUm
+1Q4ZeZC39L9qJGTH8TQYlwBLe1yAOp+vx7QJABEBAAG0JEJhemFhciBUZXN0IEtl
+eSA8YmF6YWFyQGV4YW1wbGUuY29tPokBOAQTAQIAIgUCTfjciAIbAwYLCQgHAwIG
+FQgCCQoLBBYCAwECHgECF4AACgkQh2gbHuMIDkWJUggAwj537fH6WW+GGLA5onys
+2hZmXUq/tU+L92bjQoRY4fmsQpk/FUVPUf+NQ0v1gkxx4BTfyYewaj5G6L8cvqW2
+jj7UiJd8z9gTRxWTnYwfR/w5PGmxfJsBfEUKWsccrPQdOXAhwu0fjYIVk4nqgswa
+IOAZIwe5Vsfs36uSS7p8RQHAZXLXtTOn3KcXHaxu83w6nc4zkWRovGJ9isBN3haO
+2qEa0mYiAfDpz40CGtb8N/TQHF3Xcw8rJcxpg6RF3jMtWQnzbVJFp13it00R3LqW
+o/r3RII3Ii3z2yARlg6D+5hVOrFBV8jFLkff1R2ZnVu+7WOrnbpmt3OiMkSeZrtB
+OrkBDQRN+NyIAQgArRZ2YGzUj5dXOVIWgZ1/QFpyfx/cG/293WjRE4Wt2e4SxMf2
+V0dcVCqWwT0+a79Wbausv4bStD4SkwDmu0Jf3z5ERzrr7oZwP0PMsIlM5zT6XSsr
+6UUneB3UXX7MrEqVogVhRM0ORIaK/oRwMXr7K6xVT+bCBP3/p66kHtY1ZpfEzTEX
+imBsN3GqoewBHYIneJKBtHE7uzdzw3O5p5dXqoj5foxGi9R1J15vAmt5pI68HJeX
+P6ktvXbX2Iu7VDNoCvRXM9+ntyJtsXCjNXg4pTGHS/XO4nm2db4FUZOBcVMb1vCc
+VtFjLTcbCqJqpoJWUtsLcNqDqMHOQDpe6KTNTQARAQABiQEfBBgBAgAJBQJN+NyI
+AhsMAAoJEIdoGx7jCA5FrR8IANnOF3PUj1TbRcwV6RoWmHsFQHrPmM8ogXia1Lsv
+jE1iEWoC+muvKh6Oydf90k6ZslS7rdDnp2qzYY8W/TiDkxP+fvsZ4mMi1Y0F+3ty
+1jzWhcsnB2VrJSiavxEXk0tKPrNv4EUGWG6wHsC9TBj37If+nrMyim94VHvI0eHm
+X8yMlN4O3HfmgD9CbJdUxueP3e31OIYuwh/6F7GII8TNEVHU/8vh/mQcCxppNbc+
+boff+kIsoa/TAMLwtJoSrX1nXm0K3vZePRLnIgmwVzdkOIkaRJUG2tSQFvkfhvtE
+LhnkL5l4MO0wrUds0UWRwa3d7j/P2ExrqXdlLmEzrifWyEQ=
+=hUJn
+-----END PGP PUBLIC KEY BLOCK-----
+""")
+
+        secret_key = StringIO("""-----BEGIN PGP PRIVATE KEY BLOCK-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+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+=LXn0
+-----END PGP PRIVATE KEY BLOCK-----
+""")
+
+        revoked_key = StringIO("""-----BEGIN PGP PUBLIC KEY BLOCK-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+mI0ETjlW5gEEAOb/6P+TVM59E897wRtatxys2BhsHCXM4T7xjIiANfDwejDdifqh
+tluTfSJLLxPembtrrEjux1C0AJgc+f0MIfsc3Pr3eFJzKB2ot/1IVG1/1KnA0zt3
+W2xPT3lRib27WJ9Fag+dMtQaIzgJ7/n2DFxsFZ33FD2kxrEXB2exGg6FABEBAAGI
+pgQgAQIAEAUCTjlXkAkdAHJldm9rZWQACgkQjs6dvEpb0cQPHAP/Wi9rbx0e+1Sf
+ziGgyVdr3m3A6uvze5oXKVgFRbGRUYSH4/I8GW0W9x4TcRg9h+YaQ8NUdADr9kNE
+tKAljLqYA5qdqSfYuaij1M++Xj+KUZ359R74sHuQqwnRy1XXQNfRs/QpXA7vLdds
+rjg+pbWuXO92TZJUdnqtWW+VEyZBsPy0G3Rlc3Qga2V5IDx0ZXN0QGV4YW1wbGUu
+Y29tPoi4BBMBAgAiBQJOOVbmAhsDBgsJCAcDAgYVCAIJCgsEFgIDAQIeAQIXgAAK
+CRCOzp28SlvRxNWzA/42WVmI0b+6mF/imEOlY1TiyvrcpK250rkSDsCtL4lOwy7G
+antZhpgNfnXRd/ySfsS3EB6dpOWgOSxGRvWQhA+vxBT9BYNk49qd3JIrSaSWpR12
+rET8qO1rEQQFWsw03CxTGujxGlmEO+a1yguRXp2UWaY7FngcQmD+8q7BUIVm7riN
+BE45VuYBBADTEH2jHTjNCc5CMOhea6EJTrkx3upcEqB2oyhWeSWJiBGOxlcddsjo
+3J3/EmBB8kK1hM9TidD3SG64x1N287lg8ELJBlKv+pQVyxohGJ1u/THgpTDMMQcL
+luG5rAHQGSfyzKTiOnaTyBYg3M/nzgUOU9dKEFB0EA3tjUXFOT+r3wARAQABiJ8E
+GAECAAkFAk45VuYCGwwACgkQjs6dvEpb0cRSLQP/fzCWX2lXwlwWiVF8BOPF7o9z
+icHErc7/X17RGb4qj1kVf+UkRdUWJrbEVh4h6MncBIuA70WsYogiw+Kz/0LCtQAR
+YUJsPy/EL++OKPH1aFasOdTxwkTka85+RdYqhP1+z/aYLFMWq6mRFI+o6x2k5mGi
+7dMv2kKTJPoXUpiXJbg=
+=hLYO
+-----END PGP PUBLIC KEY BLOCK-----
+""")
+
+        expired_key = StringIO("""-----BEGIN PGP PUBLIC KEY BLOCK-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+mI0ETjZ6PAEEALkR4GcFQidCCxV7pgQwQd5MZua0YO2l92fVqHX+PhnZ6egCLKdD
+2bWlMUd6MLPF3FlRL7BBAxvW/DazkBOp7ljsnpMpptEzY49Uem1irYLYiVb9zK96
+0sQZzFxFkfEYetQEXC68mIck8tbySOX5NAOw++3jFm3J7dsU1R3XtYzRABEBAAG0
+G3Rlc3Qga2V5IDx0ZXN0QGV4YW1wbGUuY29tPoi+BBMBAgAoBQJONno8AhsDBQkA
+AVGABgsJCAcDAgYVCAIJCgsEFgIDAQIeAQIXgAAKCRAc4m97T40VEz+DA/9PBphG
+Yp9cHVaHSfTUKGTGgIbvRe60sFNpDCYZeAGDrygOMuI8MNzbVpwefRBFHVPx7jWd
+rrYMsLkcsNUS9D0baU+0D/qp7JVg7ZSQtG0O6IG4eTZhibteY1fu0+unlXmg9NHx
+5VvhwzBiJDYji00M2p/CZEMiYFUuy76CsxUpN7iNBE42ejwBBACkv2/mX7IPQg0C
+A3KSrJsJv+sdvKm4b4xuI4OwagwTIVz4KlTqV4IBrVjSBfwyMXucXz0bTW85qjgA
++n67td8vyjYYZUEz1uY9lSquQQDnAN0txL3cLHZXWiWOkmzZVddQtlflK2a/J9o0
+QkHPVUm+hc4l64dIzStrNl2S66fAvQARAQABiKUEGAECAA8FAk42ejwCGwwFCQAB
+UYAACgkQHOJve0+NFROEYQP/epg+o8iBs31hkSERyZjrRR66LpywezWj30Rn/3mX
+Fzi9HkF4xLemWOzdNt9C5PYrOep85PQg8haEjknxVjZFS0ikT1h3OWk/TF1ZrLVm
+WzyX8DaHQEjKpLJJjXcAbTiZBNMk0QaVC9RvIeHpCf3n3DC49DdjsPJRMKOn8KDi
+kRk=
+=p0gt
+-----END PGP PUBLIC KEY BLOCK-----
+""")
+        context.import_(key)
+        context.import_(secret_key)
+        context.import_(revoked_key)
+        context.import_(expired_key)
+
+    def test_verify_untrusted_but_accepted(self):
+        #untrusted by gpg but listed as acceptable_keys by user
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+            
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+
+bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+iQEcBAEBAgAGBQJN+ekFAAoJEIdoGx7jCA5FGtEH/i+XxJRvqU6wdBtLVrGBMAGk
+FZ5VP+KyXYtymSbgSstj/vM12NeMIeFs3xGnNnYuX1MIcY6We5TKtCH0epY6ym5+
+6g2Q2QpQ5/sT2d0mWzR0K4uVngmxVQaXTdk5PdZ40O7ULeDLW6CxzxMHyUL1rsIx
+7UBUTBh1O/1n3ZfD99hUkm3hVcnsN90uTKH59zV9NWwArU0cug60+5eDKJhSJDbG
+rIwlqbFAjDZ7L/48e+IaYIJwBZFzMBpJKdCxzALLtauMf+KK8hGiL2hrRbWm7ty6
+NgxfkMYOB4rDPdSstT35N+5uBG3n/UzjxHssi0svMfVETYYX40y57dm2eZQXFp8=
+=iwsn
+-----END PGP SIGNATURE-----
+"""
+        plain = """bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+"""
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        my_gpg.set_acceptable_keys("bazaar@example.com")
+        self.assertEqual((gpg.SIGNATURE_VALID, None), my_gpg.verify(content,
+                            plain))
+
+    def test_verify_unacceptable_key(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+            
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+
+bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+iQEcBAEBAgAGBQJN+ekFAAoJEIdoGx7jCA5FGtEH/i+XxJRvqU6wdBtLVrGBMAGk
+FZ5VP+KyXYtymSbgSstj/vM12NeMIeFs3xGnNnYuX1MIcY6We5TKtCH0epY6ym5+
+6g2Q2QpQ5/sT2d0mWzR0K4uVngmxVQaXTdk5PdZ40O7ULeDLW6CxzxMHyUL1rsIx
+7UBUTBh1O/1n3ZfD99hUkm3hVcnsN90uTKH59zV9NWwArU0cug60+5eDKJhSJDbG
+rIwlqbFAjDZ7L/48e+IaYIJwBZFzMBpJKdCxzALLtauMf+KK8hGiL2hrRbWm7ty6
+NgxfkMYOB4rDPdSstT35N+5uBG3n/UzjxHssi0svMfVETYYX40y57dm2eZQXFp8=
+=iwsn
+-----END PGP SIGNATURE-----
+"""
+        plain = """bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+"""
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        my_gpg.set_acceptable_keys("foo@example.com")
+        self.assertEqual((gpg.SIGNATURE_KEY_MISSING, u'E3080E45'),
+                         my_gpg.verify(content, plain))
+
+    def test_verify_valid_but_untrusted(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+            
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+
+bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+iQEcBAEBAgAGBQJN+ekFAAoJEIdoGx7jCA5FGtEH/i+XxJRvqU6wdBtLVrGBMAGk
+FZ5VP+KyXYtymSbgSstj/vM12NeMIeFs3xGnNnYuX1MIcY6We5TKtCH0epY6ym5+
+6g2Q2QpQ5/sT2d0mWzR0K4uVngmxVQaXTdk5PdZ40O7ULeDLW6CxzxMHyUL1rsIx
+7UBUTBh1O/1n3ZfD99hUkm3hVcnsN90uTKH59zV9NWwArU0cug60+5eDKJhSJDbG
+rIwlqbFAjDZ7L/48e+IaYIJwBZFzMBpJKdCxzALLtauMf+KK8hGiL2hrRbWm7ty6
+NgxfkMYOB4rDPdSstT35N+5uBG3n/UzjxHssi0svMfVETYYX40y57dm2eZQXFp8=
+=iwsn
+-----END PGP SIGNATURE-----
+"""
+        plain = """bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+"""
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None), my_gpg.verify(content,
+                            plain))
+
+    def test_verify_bad_testament(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+            
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+
+bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+iQEcBAEBAgAGBQJN+ekFAAoJEIdoGx7jCA5FGtEH/i+XxJRvqU6wdBtLVrGBMAGk
+FZ5VP+KyXYtymSbgSstj/vM12NeMIeFs3xGnNnYuX1MIcY6We5TKtCH0epY6ym5+
+6g2Q2QpQ5/sT2d0mWzR0K4uVngmxVQaXTdk5PdZ40O7ULeDLW6CxzxMHyUL1rsIx
+7UBUTBh1O/1n3ZfD99hUkm3hVcnsN90uTKH59zV9NWwArU0cug60+5eDKJhSJDbG
+rIwlqbFAjDZ7L/48e+IaYIJwBZFzMBpJKdCxzALLtauMf+KK8hGiL2hrRbWm7ty6
+NgxfkMYOB4rDPdSstT35N+5uBG3n/UzjxHssi0svMfVETYYX40y57dm2eZQXFp8=
+=iwsn
+-----END PGP SIGNATURE-----
+"""
+        plain = """bazaar-ng testament short form 1
+revision-id: doctor@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+"""
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        my_gpg.set_acceptable_keys("bazaar@example.com")
+        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None), my_gpg.verify(content,
+                            plain))
+
+
+    def test_verify_revoked_signature(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+            
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+
+asdf
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+iJwEAQECAAYFAk45V18ACgkQjs6dvEpb0cSIZQP/eOGTXGPlrNwvDkcX2d8O///I
+ecB4sUIUEpv1XAk1MkNu58lsjjK72lRaLusEGqd7HwrFmpxVeVs0oWLg23PNPCFs
+yJBID9ma+VxFVPtkEFnrc1R72sBJLfBcTxMkwVTC8eeznjdtn+cg+aLkxbPdrGnr
+JFA6kUIJU2w9LU/b88Y=
+=UuRX
+-----END PGP SIGNATURE-----
+"""
+        plain = """asdf\n"""
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        my_gpg.set_acceptable_keys("test@example.com")
+        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None), my_gpg.verify(content,
+                            plain))
+
+    def test_verify_invalid(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+
+bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+iEYEARECAAYFAk33gYsACgkQpQbm1N1NUIhiDACglOuQDlnSF4NxfHSkN/zrmFy8
+nswAoNGXAVuR9ONasAKIGBNUE0b+lols
+=SOuC
+-----END PGP SIGNATURE-----
+"""
+        plain = """bazaar-ng testament short form 1
+revision-id: amy@example.com-20110527185938-hluafawphszb8dl1
+sha1: 6411f9bdf6571200357140c9ce7c0f50106ac9a4
+"""
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual((gpg.SIGNATURE_NOT_VALID, None),
+                            my_gpg.verify(content, plain))
+
+    def test_verify_expired_but_valid(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+ 
+bazaar-ng testament short form 1
+revision-id: test@example.com-20110801100657-f1dr1nompeex723z
+sha1: 59ab434be4c2d5d646dee84f514aa09e1b72feeb
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.10 (GNU/Linux)
+ 
+iJwEAQECAAYFAk42esUACgkQHOJve0+NFRPc5wP7BoZkzBU8JaHMLv/LmqLr0sUz
+zuE51ofZZ19L7KVtQWsOi4jFy0fi4A5TFwO8u9SOfoREGvkw292Uty9subSouK5/
+mFmDOYPQ+O83zWgYZsBmMJWYDZ+X9I6XXZSbPtV/7XyTjaxtl5uRnDVJjg+AzKvD
+dTp8VatVVrwuvzOPDVc=
+=uHen
+-----END PGP SIGNATURE-----
+"""
+        plain = """bazaar-ng testament short form 1
+revision-id: test@example.com-20110801100657-f1dr1nompeex723z
+sha1: 59ab434be4c2d5d646dee84f514aa09e1b72feeb
+"""
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual((gpg.SIGNATURE_EXPIRED, u'4F8D1513'),
+                            my_gpg.verify(content, plain))
+
+    def test_verify_unknown_key(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+        content = """-----BEGIN PGP SIGNED MESSAGE-----
+Hash: SHA1
+
+asdf
+-----BEGIN PGP SIGNATURE-----
+Version: GnuPG v1.4.11 (GNU/Linux)
+
+iQEcBAEBAgAGBQJOORKwAAoJENf6AkFdUeVvJDYH/1Cz+AJn1Jvy5n64o+0fZ5Ow
+Y7UQb4QQTIOV7jI7n4hv/yBzuHrtImFzYvQl/o2Ezzi8B8L5gZtQy+xCUF+Q8iWs
+gytZ5JUtSze7hDZo1NUl4etjoRGYqRfrUcvE2LkVH2dFbDGyyQfVmoeSHa5akuuP
+QZmyg2F983rACVIpGvsqTH6RcBdvE9vx68lugeKQA8ArDn39/74FBFipFzrXSPij
+eKFpl+yZmIb3g6HkPIC8o4j/tMvc37xF1OG5sBu8FT0+FC+VgY7vAblneDftAbyP
+sIODx4WcfJtjLG/qkRYqJ4gDHo0eMpTJSk2CWebajdm4b+JBrM1F9mgKuZFLruE=
+=RNR5
+-----END PGP SIGNATURE-----
+"""
+        plain = "asdf\n"
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual((gpg.SIGNATURE_KEY_MISSING, u'5D51E56F'),
+                            my_gpg.verify(content, plain))
+
+    def test_set_acceptable_keys(self):
+        self.requireFeature(features.gpgme)
+        self.import_keys()
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        my_gpg.set_acceptable_keys("bazaar@example.com")
+        self.assertEqual(my_gpg.acceptable_keys,
+                         [u'B5DEED5FCB15DAE6ECEF919587681B1EE3080E45'])
+
+    def test_set_acceptable_keys_unknown(self):
+        self.requireFeature(features.gpgme)
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.notes = []
+        def note(*args):
+            self.notes.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'note', note)
+        my_gpg.set_acceptable_keys("unknown")
+        self.assertEqual(my_gpg.acceptable_keys, [])
+        self.assertEqual(self.notes,
+            ['No GnuPG key results for pattern: unknown'])
+
 
 class TestDisabled(TestCase):
 
     def test_sign(self):
         self.assertRaises(errors.SigningFailed,
                           gpg.DisabledGPGStrategy(None).sign, 'content')
+
+    def test_verify(self):
+        self.assertRaises(errors.SignatureVerificationFailed,
+                          gpg.DisabledGPGStrategy(None).verify, 'content',
+                          'testament')

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_graph.py	2011-08-26 11:35:50 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,12 +17,10 @@
 from bzrlib import (
     errors,
     graph as _mod_graph,
-    symbol_versioning,
     tests,
     )
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCaseWithMemoryTransport
-from bzrlib.symbol_versioning import deprecated_in
 
 
 # Ancestry 1:
@@ -426,11 +424,41 @@
     def __init__(self, parents_provider):
         self.calls = []
         self._real_parents_provider = parents_provider
+        get_cached = getattr(parents_provider, 'get_cached_parent_map', None)
+        if get_cached is not None:
+            # Only expose the underlying 'get_cached_parent_map' function if
+            # the wrapped provider has it.
+            self.get_cached_parent_map = self._get_cached_parent_map
 
     def get_parent_map(self, nodes):
         self.calls.extend(nodes)
         return self._real_parents_provider.get_parent_map(nodes)
 
+    def _get_cached_parent_map(self, nodes):
+        self.calls.append(('cached', sorted(nodes)))
+        return self._real_parents_provider.get_cached_parent_map(nodes)
+
+
+class SharedInstrumentedParentsProvider(object):
+
+    def __init__(self, parents_provider, calls, info):
+        self.calls = calls
+        self.info = info
+        self._real_parents_provider = parents_provider
+        get_cached = getattr(parents_provider, 'get_cached_parent_map', None)
+        if get_cached is not None:
+            # Only expose the underlying 'get_cached_parent_map' function if
+            # the wrapped provider has it.
+            self.get_cached_parent_map = self._get_cached_parent_map
+
+    def get_parent_map(self, nodes):
+        self.calls.append((self.info, sorted(nodes)))
+        return self._real_parents_provider.get_parent_map(nodes)
+
+    def _get_cached_parent_map(self, nodes):
+        self.calls.append((self.info, 'cached', sorted(nodes)))
+        return self._real_parents_provider.get_cached_parent_map(nodes)
+
 
 class TestGraphBase(tests.TestCase):
 
@@ -675,45 +703,6 @@
         self.assertEqual((set(['e']), set(['f', 'g'])),
                          graph.find_difference('e', 'f'))
 
-
-    def test_stacked_parents_provider(self):
-        parents1 = _mod_graph.DictParentsProvider({'rev2': ['rev3']})
-        parents2 = _mod_graph.DictParentsProvider({'rev1': ['rev4']})
-        stacked = _mod_graph.StackedParentsProvider([parents1, parents2])
-        self.assertEqual({'rev1':['rev4'], 'rev2':['rev3']},
-                         stacked.get_parent_map(['rev1', 'rev2']))
-        self.assertEqual({'rev2':['rev3'], 'rev1':['rev4']},
-                         stacked.get_parent_map(['rev2', 'rev1']))
-        self.assertEqual({'rev2':['rev3']},
-                         stacked.get_parent_map(['rev2', 'rev2']))
-        self.assertEqual({'rev1':['rev4']},
-                         stacked.get_parent_map(['rev1', 'rev1']))
-    
-    def test_stacked_parents_provider_overlapping(self):
-        # rev2 is availible in both providers.
-        # 1
-        # |
-        # 2
-        parents1 = _mod_graph.DictParentsProvider({'rev2': ['rev1']})
-        parents2 = _mod_graph.DictParentsProvider({'rev2': ['rev1']})
-        stacked = _mod_graph.StackedParentsProvider([parents1, parents2])
-        self.assertEqual({'rev2': ['rev1']},
-                         stacked.get_parent_map(['rev2']))
-
-    def test__stacked_parents_provider_deprecated(self):
-        parents1 = _mod_graph.DictParentsProvider({'rev2': ['rev3']})
-        parents2 = _mod_graph.DictParentsProvider({'rev1': ['rev4']})
-        stacked = self.applyDeprecated(deprecated_in((1, 16, 0)),
-                    _mod_graph._StackedParentsProvider, [parents1, parents2])
-        self.assertEqual({'rev1':['rev4'], 'rev2':['rev3']},
-                         stacked.get_parent_map(['rev1', 'rev2']))
-        self.assertEqual({'rev2':['rev3'], 'rev1':['rev4']},
-                         stacked.get_parent_map(['rev2', 'rev1']))
-        self.assertEqual({'rev2':['rev3']},
-                         stacked.get_parent_map(['rev2', 'rev2']))
-        self.assertEqual({'rev1':['rev4']},
-                         stacked.get_parent_map(['rev1', 'rev1']))
-
     def test_iter_topo_order(self):
         graph = self.make_graph(ancestry_1)
         args = ['rev2a', 'rev3', 'rev1']
@@ -1424,6 +1413,61 @@
         self.assertMergeOrder(['rev3', 'rev1'], graph, 'rev4', ['rev1', 'rev3'])
 
 
+class TestFindDescendants(TestGraphBase):
+
+    def test_find_descendants_rev1_rev3(self):
+        graph = self.make_graph(ancestry_1)
+        descendants = graph.find_descendants('rev1', 'rev3')
+        self.assertEqual(set(['rev1', 'rev2a', 'rev3']), descendants)
+
+    def test_find_descendants_rev1_rev4(self):
+        graph = self.make_graph(ancestry_1)
+        descendants = graph.find_descendants('rev1', 'rev4')
+        self.assertEqual(set(['rev1', 'rev2a', 'rev2b', 'rev3', 'rev4']),
+                         descendants)
+
+    def test_find_descendants_rev2a_rev4(self):
+        graph = self.make_graph(ancestry_1)
+        descendants = graph.find_descendants('rev2a', 'rev4')
+        self.assertEqual(set(['rev2a', 'rev3', 'rev4']), descendants)
+
+class TestFindLefthandMerger(TestGraphBase):
+
+    def check_merger(self, result, ancestry, merged, tip):
+        graph = self.make_graph(ancestry)
+        self.assertEqual(result, graph.find_lefthand_merger(merged, tip))
+
+    def test_find_lefthand_merger_rev2b(self):
+        self.check_merger('rev4', ancestry_1, 'rev2b', 'rev4')
+
+    def test_find_lefthand_merger_rev2a(self):
+        self.check_merger('rev2a', ancestry_1, 'rev2a', 'rev4')
+
+    def test_find_lefthand_merger_rev4(self):
+        self.check_merger(None, ancestry_1, 'rev4', 'rev2a')
+
+    def test_find_lefthand_merger_f(self):
+        self.check_merger('i', complex_shortcut, 'f', 'm')
+
+    def test_find_lefthand_merger_g(self):
+        self.check_merger('i', complex_shortcut, 'g', 'm')
+
+    def test_find_lefthand_merger_h(self):
+        self.check_merger('n', complex_shortcut, 'h', 'n')
+
+
+class TestGetChildMap(TestGraphBase):
+
+    def test_get_child_map(self):
+        graph = self.make_graph(ancestry_1)
+        child_map = graph.get_child_map(['rev4', 'rev3', 'rev2a', 'rev2b'])
+        self.assertEqual({'rev1': ['rev2a', 'rev2b'],
+                          'rev2a': ['rev3'],
+                          'rev2b': ['rev4'],
+                          'rev3': ['rev4']},
+                          child_map)
+
+
 class TestCachingParentsProvider(tests.TestCase):
     """These tests run with:
 
@@ -1434,7 +1478,7 @@
 
     def setUp(self):
         super(TestCachingParentsProvider, self).setUp()
-        dict_pp = _mod_graph.DictParentsProvider({'a':('b',)})
+        dict_pp = _mod_graph.DictParentsProvider({'a': ('b',)})
         self.inst_pp = InstrumentedParentsProvider(dict_pp)
         self.caching_pp = _mod_graph.CachingParentsProvider(self.inst_pp)
 
@@ -1479,6 +1523,14 @@
         self.assertEqual([], self.inst_pp.calls)
         self.assertEqual(set(['b']), self.caching_pp.missing_keys)
 
+    def test_get_cached_parent_map(self):
+        self.assertEqual({}, self.caching_pp.get_cached_parent_map(['a']))
+        self.assertEqual([], self.inst_pp.calls)
+        self.assertEqual({'a': ('b',)}, self.caching_pp.get_parent_map(['a']))
+        self.assertEqual(['a'], self.inst_pp.calls)
+        self.assertEqual({'a': ('b',)},
+                         self.caching_pp.get_cached_parent_map(['a']))
+
 
 class TestCachingParentsProviderExtras(tests.TestCaseWithTransport):
     """Test the behaviour when parents are provided that were not requested."""
@@ -1543,6 +1595,14 @@
                          self.caching_pp.get_parent_map(['rev2']))
         self.assertEqual(['rev3'], self.inst_pp.calls)
 
+    def test_extras_using_cached(self):
+        self.assertEqual({}, self.caching_pp.get_cached_parent_map(['rev3']))
+        self.assertEqual({}, self.caching_pp.get_parent_map(['rev3']))
+        self.assertEqual({'rev2': ['rev1']},
+                         self.caching_pp.get_cached_parent_map(['rev2']))
+        self.assertEqual(['rev3'], self.inst_pp.calls)
+
+
 
 class TestCollapseLinearRegions(tests.TestCase):
 
@@ -1599,6 +1659,23 @@
         self.assertEqual(['D'], sorted(graph_thunk.heads(['D', 'C'])))
         self.assertEqual(['B', 'C'], sorted(graph_thunk.heads(['B', 'C'])))
 
+    def test_add_node(self):
+        d = {('C',):[('A',)], ('B',): [('A',)], ('A',): []}
+        g = _mod_graph.KnownGraph(d)
+        graph_thunk = _mod_graph.GraphThunkIdsToKeys(g)
+        graph_thunk.add_node("D", ["A", "C"])
+        self.assertEqual(['B', 'D'],
+            sorted(graph_thunk.heads(['D', 'B', 'A'])))
+
+    def test_merge_sort(self):
+        d = {('C',):[('A',)], ('B',): [('A',)], ('A',): []}
+        g = _mod_graph.KnownGraph(d)
+        graph_thunk = _mod_graph.GraphThunkIdsToKeys(g)
+        graph_thunk.add_node("D", ["A", "C"])
+        self.assertEqual([('C', 0, (2,), False), ('A', 0, (1,), True)],
+            [(n.key, n.merge_depth, n.revno, n.end_of_merge)
+                 for n in graph_thunk.merge_sort('C')])
+
 
 class TestPendingAncestryResultGetKeys(TestCaseWithMemoryTransport):
     """Tests for bzrlib.graph.PendingAncestryResult."""
@@ -1688,3 +1765,141 @@
         self.assertEqual(set([NULL_REVISION, 'tip', 'tag', 'mid']), recipe[2])
         self.assertEqual(0, recipe[3])
         self.assertTrue(result.is_empty())
+
+
+class TestSearchResultFromParentMap(TestGraphBase):
+
+    def assertSearchResult(self, start_keys, stop_keys, key_count, parent_map,
+                           missing_keys=()):
+        (start, stop, count) = _mod_graph.search_result_from_parent_map(
+            parent_map, missing_keys)
+        self.assertEqual((sorted(start_keys), sorted(stop_keys), key_count),
+                         (sorted(start), sorted(stop), count))
+
+    def test_no_parents(self):
+        self.assertSearchResult([], [], 0, {})
+        self.assertSearchResult([], [], 0, None)
+
+    def test_ancestry_1(self):
+        self.assertSearchResult(['rev4'], [NULL_REVISION], len(ancestry_1),
+                                ancestry_1)
+
+    def test_ancestry_2(self):
+        self.assertSearchResult(['rev1b', 'rev4a'], [NULL_REVISION],
+                                len(ancestry_2), ancestry_2)
+        self.assertSearchResult(['rev1b', 'rev4a'], [],
+                                len(ancestry_2)+1, ancestry_2,
+                                missing_keys=[NULL_REVISION])
+
+    def test_partial_search(self):
+        parent_map = dict((k,extended_history_shortcut[k])
+                          for k in ['e', 'f'])
+        self.assertSearchResult(['e', 'f'], ['d', 'a'], 2,
+                                parent_map)
+        parent_map.update((k,extended_history_shortcut[k])
+                          for k in ['d', 'a'])
+        self.assertSearchResult(['e', 'f'], ['c', NULL_REVISION], 4,
+                                parent_map)
+        parent_map['c'] = extended_history_shortcut['c']
+        self.assertSearchResult(['e', 'f'], ['b'], 6,
+                                parent_map, missing_keys=[NULL_REVISION])
+        parent_map['b'] = extended_history_shortcut['b']
+        self.assertSearchResult(['e', 'f'], [], 7,
+                                parent_map, missing_keys=[NULL_REVISION])
+
+
+class TestLimitedSearchResultFromParentMap(TestGraphBase):
+
+    def assertSearchResult(self, start_keys, stop_keys, key_count, parent_map,
+                           missing_keys, tip_keys, depth):
+        (start, stop, count) = _mod_graph.limited_search_result_from_parent_map(
+            parent_map, missing_keys, tip_keys, depth)
+        self.assertEqual((sorted(start_keys), sorted(stop_keys), key_count),
+                         (sorted(start), sorted(stop), count))
+
+    def test_empty_ancestry(self):
+        self.assertSearchResult([], [], 0, {}, (), ['tip-rev-id'], 10)
+
+    def test_ancestry_1(self):
+        self.assertSearchResult(['rev4'], ['rev1'], 4,
+                                ancestry_1, (), ['rev1'], 10)
+        self.assertSearchResult(['rev2a', 'rev2b'], ['rev1'], 2,
+                                ancestry_1, (), ['rev1'], 1)
+
+
+    def test_multiple_heads(self):
+        self.assertSearchResult(['e', 'f'], ['a'], 5,
+                                extended_history_shortcut, (), ['a'], 10)
+        # Note that even though we only take 1 step back, we find 'f', which
+        # means the described search will still find d and c.
+        self.assertSearchResult(['f'], ['a'], 4,
+                                extended_history_shortcut, (), ['a'], 1)
+        self.assertSearchResult(['f'], ['a'], 4,
+                                extended_history_shortcut, (), ['a'], 2)
+
+
+class TestStackedParentsProvider(tests.TestCase):
+
+    def setUp(self):
+        super(TestStackedParentsProvider, self).setUp()
+        self.calls = []
+
+    def get_shared_provider(self, info, ancestry, has_cached):
+        pp = _mod_graph.DictParentsProvider(ancestry)
+        if has_cached:
+            pp.get_cached_parent_map = pp.get_parent_map
+        return SharedInstrumentedParentsProvider(pp, self.calls, info)
+
+    def test_stacked_parents_provider(self):
+        parents1 = _mod_graph.DictParentsProvider({'rev2': ['rev3']})
+        parents2 = _mod_graph.DictParentsProvider({'rev1': ['rev4']})
+        stacked = _mod_graph.StackedParentsProvider([parents1, parents2])
+        self.assertEqual({'rev1':['rev4'], 'rev2':['rev3']},
+                         stacked.get_parent_map(['rev1', 'rev2']))
+        self.assertEqual({'rev2':['rev3'], 'rev1':['rev4']},
+                         stacked.get_parent_map(['rev2', 'rev1']))
+        self.assertEqual({'rev2':['rev3']},
+                         stacked.get_parent_map(['rev2', 'rev2']))
+        self.assertEqual({'rev1':['rev4']},
+                         stacked.get_parent_map(['rev1', 'rev1']))
+
+    def test_stacked_parents_provider_overlapping(self):
+        # rev2 is availible in both providers.
+        # 1
+        # |
+        # 2
+        parents1 = _mod_graph.DictParentsProvider({'rev2': ['rev1']})
+        parents2 = _mod_graph.DictParentsProvider({'rev2': ['rev1']})
+        stacked = _mod_graph.StackedParentsProvider([parents1, parents2])
+        self.assertEqual({'rev2': ['rev1']},
+                         stacked.get_parent_map(['rev2']))
+
+    def test_handles_no_get_cached_parent_map(self):
+        # this shows that we both handle when a provider doesn't implement
+        # get_cached_parent_map
+        pp1 = self.get_shared_provider('pp1', {'rev2': ('rev1',)},
+                                       has_cached=False)
+        pp2 = self.get_shared_provider('pp2', {'rev2': ('rev1',)},
+                                       has_cached=True)
+        stacked = _mod_graph.StackedParentsProvider([pp1, pp2])
+        self.assertEqual({'rev2': ('rev1',)}, stacked.get_parent_map(['rev2']))
+        # No call on 'pp1' because it doesn't provide get_cached_parent_map
+        self.assertEqual([('pp2', 'cached', ['rev2'])], self.calls)
+
+    def test_query_order(self):
+        # We should call get_cached_parent_map on all providers before we call
+        # get_parent_map. Further, we should track what entries we have found,
+        # and not re-try them.
+        pp1 = self.get_shared_provider('pp1', {'a': ()}, has_cached=True)
+        pp2 = self.get_shared_provider('pp2', {'c': ('b',)}, has_cached=False)
+        pp3 = self.get_shared_provider('pp3', {'b': ('a',)}, has_cached=True)
+        stacked = _mod_graph.StackedParentsProvider([pp1, pp2, pp3])
+        self.assertEqual({'a': (), 'b': ('a',), 'c': ('b',)},
+                         stacked.get_parent_map(['a', 'b', 'c', 'd']))
+        self.assertEqual([('pp1', 'cached', ['a', 'b', 'c', 'd']),
+                          # No call to pp2, because it doesn't have cached
+                          ('pp3', 'cached', ['b', 'c', 'd']),
+                          ('pp1', ['c', 'd']),
+                          ('pp2', ['c', 'd']),
+                          ('pp3', ['d']),
+                         ], self.calls)

=== modified file 'bzrlib/tests/test_groupcompress.py'
--- bzrlib/tests/test_groupcompress.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_groupcompress.py	2011-05-14 10:33:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 from bzrlib import (
     btree_index,
+    config,
     groupcompress,
     errors,
     index as _mod_index,
@@ -30,19 +31,20 @@
     )
 from bzrlib.osutils import sha_string
 from bzrlib.tests.test__groupcompress import compiled_groupcompress_feature
-
-
-def load_tests(standard_tests, module, loader):
-    """Parameterize tests for all versions of groupcompress."""
-    to_adapt, result = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance(TestAllGroupCompressors))
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+def group_compress_implementation_scenarios():
     scenarios = [
         ('python', {'compressor': groupcompress.PythonGroupCompressor}),
         ]
     if compiled_groupcompress_feature.available():
         scenarios.append(('C',
             {'compressor': groupcompress.PyrexGroupCompressor}))
-    return tests.multiply_tests(to_adapt, scenarios, result)
+    return scenarios
+
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestGroupCompressor(tests.TestCase):
@@ -66,7 +68,8 @@
 class TestAllGroupCompressors(TestGroupCompressor):
     """Tests for GroupCompressor"""
 
-    compressor = None # Set by multiply_tests
+    scenarios = group_compress_implementation_scenarios()
+    compressor = None # Set by scenario
 
     def test_empty_delta(self):
         compressor = self.compressor()
@@ -347,6 +350,30 @@
         self.assertEqual(z_content, block._z_content)
         self.assertEqual(content, block._content)
 
+    def test_to_chunks(self):
+        content_chunks = ['this is some content\n',
+                          'this content will be compressed\n']
+        content_len = sum(map(len, content_chunks))
+        content = ''.join(content_chunks)
+        gcb = groupcompress.GroupCompressBlock()
+        gcb.set_chunked_content(content_chunks, content_len)
+        total_len, block_chunks = gcb.to_chunks()
+        block_bytes = ''.join(block_chunks)
+        self.assertEqual(gcb._z_content_length, len(gcb._z_content))
+        self.assertEqual(total_len, len(block_bytes))
+        self.assertEqual(gcb._content_length, content_len)
+        expected_header =('gcb1z\n' # group compress block v1 zlib
+                          '%d\n' # Length of compressed content
+                          '%d\n' # Length of uncompressed content
+                         ) % (gcb._z_content_length, gcb._content_length)
+        # The first chunk should be the header chunk. It is small, fixed size,
+        # and there is no compelling reason to split it up
+        self.assertEqual(expected_header, block_chunks[0])
+        self.assertStartsWith(block_bytes, expected_header)
+        remaining_bytes = block_bytes[len(expected_header):]
+        raw_bytes = zlib.decompress(remaining_bytes)
+        self.assertEqual(content, raw_bytes)
+
     def test_to_bytes(self):
         content = ('this is some content\n'
                    'this content will be compressed\n')
@@ -389,7 +416,7 @@
         z_content = zlib.compress(content)
         self.assertEqual(57182, len(z_content))
         block = groupcompress.GroupCompressBlock()
-        block._z_content = z_content
+        block._z_content_chunks = (z_content,)
         block._z_content_length = len(z_content)
         block._compressor_name = 'zlib'
         block._content_length = 158634
@@ -434,7 +461,7 @@
         z_content = zlib.compress(content)
         self.assertEqual(57182, len(z_content))
         block = groupcompress.GroupCompressBlock()
-        block._z_content = z_content
+        block._z_content_chunks = (z_content,)
         block._z_content_length = len(z_content)
         block._compressor_name = 'zlib'
         block._content_length = 158634
@@ -526,6 +553,23 @@
                     'as-requested', False)]
         self.assertEqual([('b',), ('a',), ('d',), ('c',)], keys)
 
+    def test_get_record_stream_max_bytes_to_index_default(self):
+        vf = self.make_test_vf(True, dir='source')
+        vf.add_lines(('a',), (), ['lines\n'])
+        vf.writer.end()
+        record = vf.get_record_stream([('a',)], 'unordered', True).next()
+        self.assertEqual(vf._DEFAULT_COMPRESSOR_SETTINGS,
+                         record._manager._get_compressor_settings())
+
+    def test_get_record_stream_accesses_compressor_settings(self):
+        vf = self.make_test_vf(True, dir='source')
+        vf.add_lines(('a',), (), ['lines\n'])
+        vf.writer.end()
+        vf._max_bytes_to_index = 1234
+        record = vf.get_record_stream([('a',)], 'unordered', True).next()
+        self.assertEqual(dict(max_bytes_to_index=1234),
+                         record._manager._get_compressor_settings())
+
     def test_insert_record_stream_reuses_blocks(self):
         vf = self.make_test_vf(True, dir='source')
         def grouped_stream(revision_ids, first_parents=()):
@@ -744,6 +788,48 @@
         self.assertEqual(0, len(vf._group_cache))
 
 
+class TestGroupCompressConfig(tests.TestCaseWithTransport):
+
+    def make_test_vf(self):
+        t = self.get_transport('.')
+        t.ensure_base()
+        factory = groupcompress.make_pack_factory(graph=True,
+            delta=False, keylength=1, inconsistency_fatal=True)
+        vf = factory(t)
+        self.addCleanup(groupcompress.cleanup_pack_group, vf)
+        return vf
+
+    def test_max_bytes_to_index_default(self):
+        vf = self.make_test_vf()
+        gc = vf._make_group_compressor()
+        self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                         vf._max_bytes_to_index)
+        if isinstance(gc, groupcompress.PyrexGroupCompressor):
+            self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                             gc._delta_index._max_bytes_to_index)
+
+    def test_max_bytes_to_index_in_config(self):
+        c = config.GlobalConfig()
+        c.set_user_option('bzr.groupcompress.max_bytes_to_index', '10000')
+        vf = self.make_test_vf()
+        gc = vf._make_group_compressor()
+        self.assertEqual(10000, vf._max_bytes_to_index)
+        if isinstance(gc, groupcompress.PyrexGroupCompressor):
+            self.assertEqual(10000, gc._delta_index._max_bytes_to_index)
+
+    def test_max_bytes_to_index_bad_config(self):
+        c = config.GlobalConfig()
+        c.set_user_option('bzr.groupcompress.max_bytes_to_index', 'boogah')
+        vf = self.make_test_vf()
+        # TODO: This is triggering a warning, we might want to trap and make
+        #       sure it is readable.
+        gc = vf._make_group_compressor()
+        self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                         vf._max_bytes_to_index)
+        if isinstance(gc, groupcompress.PyrexGroupCompressor):
+            self.assertEqual(vf._DEFAULT_MAX_BYTES_TO_INDEX,
+                             gc._delta_index._max_bytes_to_index)
+
 
 class StubGCVF(object):
     def __init__(self, canned_get_blocks=None):
@@ -1020,6 +1106,54 @@
             self.assertEqual(self._texts[record.key],
                              record.get_bytes_as('fulltext'))
 
+    def test_manager_default_compressor_settings(self):
+        locations, old_block = self.make_block(self._texts)
+        manager = groupcompress._LazyGroupContentManager(old_block)
+        gcvf = groupcompress.GroupCompressVersionedFiles
+        # It doesn't greedily evaluate _max_bytes_to_index
+        self.assertIs(None, manager._compressor_settings)
+        self.assertEqual(gcvf._DEFAULT_COMPRESSOR_SETTINGS,
+                         manager._get_compressor_settings())
+
+    def test_manager_custom_compressor_settings(self):
+        locations, old_block = self.make_block(self._texts)
+        called = []
+        def compressor_settings():
+            called.append('called')
+            return (10,)
+        manager = groupcompress._LazyGroupContentManager(old_block,
+            get_compressor_settings=compressor_settings)
+        gcvf = groupcompress.GroupCompressVersionedFiles
+        # It doesn't greedily evaluate compressor_settings
+        self.assertIs(None, manager._compressor_settings)
+        self.assertEqual((10,), manager._get_compressor_settings())
+        self.assertEqual((10,), manager._get_compressor_settings())
+        self.assertEqual((10,), manager._compressor_settings)
+        # Only called 1 time
+        self.assertEqual(['called'], called)
+
+    def test__rebuild_handles_compressor_settings(self):
+        if not isinstance(groupcompress.GroupCompressor,
+                          groupcompress.PyrexGroupCompressor):
+            raise tests.TestNotApplicable('pure-python compressor'
+                ' does not handle compressor_settings')
+        locations, old_block = self.make_block(self._texts)
+        manager = groupcompress._LazyGroupContentManager(old_block,
+            get_compressor_settings=lambda: dict(max_bytes_to_index=32))
+        gc = manager._make_group_compressor()
+        self.assertEqual(32, gc._delta_index._max_bytes_to_index)
+        self.add_key_to_manager(('key3',), locations, old_block, manager)
+        self.add_key_to_manager(('key4',), locations, old_block, manager)
+        action, last_byte, total_bytes = manager._check_rebuild_action()
+        self.assertEqual('rebuild', action)
+        manager._rebuild_block()
+        new_block = manager._block
+        self.assertIsNot(old_block, new_block)
+        # Because of the new max_bytes_to_index, we do a poor job of
+        # rebuilding. This is a side-effect of the change, but at least it does
+        # show the setting had an effect.
+        self.assertTrue(old_block._content_length < new_block._content_length)
+
     def test_check_is_well_utilized_all_keys(self):
         block, manager = self.make_block_and_full_manager(self._texts)
         self.assertFalse(manager.check_is_well_utilized())
@@ -1066,3 +1200,25 @@
         # consumption
         self.add_key_to_manager(('key4',), locations, block, manager)
         self.assertTrue(manager.check_is_well_utilized())
+
+
+class Test_GCBuildDetails(tests.TestCase):
+
+    def test_acts_like_tuple(self):
+        # _GCBuildDetails inlines some of the data that used to be spread out
+        # across a bunch of tuples
+        bd = groupcompress._GCBuildDetails((('parent1',), ('parent2',)),
+            ('INDEX', 10, 20, 0, 5))
+        self.assertEqual(4, len(bd))
+        self.assertEqual(('INDEX', 10, 20, 0, 5), bd[0])
+        self.assertEqual(None, bd[1]) # Compression Parent is always None
+        self.assertEqual((('parent1',), ('parent2',)), bd[2])
+        self.assertEqual(('group', None), bd[3]) # Record details
+
+    def test__repr__(self):
+        bd = groupcompress._GCBuildDetails((('parent1',), ('parent2',)),
+            ('INDEX', 10, 20, 0, 5))
+        self.assertEqual("_GCBuildDetails(('INDEX', 10, 20, 0, 5),"
+                         " (('parent1',), ('parent2',)))",
+                         repr(bd))
+                    

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_hashcache.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,17 +16,20 @@
 
 import os
 import stat
-import sys
 import time
 
 from bzrlib import osutils
 from bzrlib.errors import BzrError
 from bzrlib.hashcache import HashCache
-from bzrlib.tests import OsFifoFeature, TestCaseInTempDir, TestCase
-
-
-def sha1(t):
-    return osutils.sha(t).hexdigest()
+from bzrlib.tests import (
+    TestCaseInTempDir,
+    )
+from bzrlib.tests.features import (
+    OsFifoFeature,
+    )
+
+
+sha1 = osutils.sha_string
 
 
 def pause():

=== modified file 'bzrlib/tests/test_help.py'
--- bzrlib/tests/test_help.py	2010-05-24 05:50:53 +0000
+++ bzrlib/tests/test_help.py	2011-09-27 13:18:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,47 +16,64 @@
 
 """Unit tests for the bzrlib.help module."""
 
-from cStringIO import StringIO
+import textwrap
 
 from bzrlib import (
     builtins,
     commands,
+    config,
     errors,
     help,
     help_topics,
+    i18n,
     plugin,
     tests,
     )
 
-
-class TestHelp(tests.TestCase):
-
-    def setUp(self):
-        tests.TestCase.setUp(self)
-        commands.install_bzr_command_hooks()
+from bzrlib.tests.test_i18n import ZzzTranslations
+import re
 
 
 class TestCommandHelp(tests.TestCase):
     """Tests for help on commands."""
 
+    def assertCmdHelp(self, expected, cmd):
+        self.assertEqualDiff(textwrap.dedent(expected), cmd.get_help_text())
+
     def test_command_help_includes_see_also(self):
         class cmd_WithSeeAlso(commands.Command):
             __doc__ = """A sample command."""
             _see_also = ['foo', 'bar']
-        cmd = cmd_WithSeeAlso()
-        helptext = cmd.get_help_text()
-        self.assertEndsWith(
-            helptext,
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'See also: bar, foo\n')
+        self.assertCmdHelp('''\
+Purpose: A sample command.
+Usage:   bzr WithSeeAlso
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+See also: bar, foo
+''',
+                           cmd_WithSeeAlso())
 
     def test_get_help_text(self):
         """Commands have a get_help_text method which returns their help."""
         class cmd_Demo(commands.Command):
             __doc__ = """A sample command."""
+        self.assertCmdHelp('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+''',
+                           cmd_Demo())
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
         self.assertStartsWith(helptext,
@@ -73,8 +90,8 @@
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
             helptext,
+            '  -q, --quiet    Only display errors and warnings.\n'
             '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
             '\n'
             'See also: bar, foo, gam\n')
@@ -86,8 +103,8 @@
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
             helptext,
+            '  -q, --quiet    Only display errors and warnings.\n'
             '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
             '\n'
             'See also: gam\n')
@@ -121,56 +138,59 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEquals(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Examples:\n'
-            '    Example 1:\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n'
-            '    Example 2:\n'
-            '\n'
-            '        cmd arg2\n'
-            '\n'
-            '    A code block follows.\n'
-            '\n'
-            '        bzr Demo something\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Examples:
+    Example 1:
+
+        cmd arg1
+
+    Example 2:
+
+        cmd arg2
+
+    A code block follows.
+
+        bzr Demo something
+
+''',
+                                         helptext)
         helptext = cmd.get_help_text(plain=False)
-        self.assertEquals(helptext,
-            ':Purpose: A sample command.\n'
-            ':Usage:   bzr Demo\n'
-            '\n'
-            ':Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            ':Examples:\n'
-            '    Example 1::\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n'
-            '    Example 2::\n'
-            '\n'
-            '        cmd arg2\n'
-            '\n'
-            '    A code block follows.\n'
-            '\n'
-            '    ::\n'
-            '\n'
-            '        bzr Demo something\n'
-            '\n')
+        self.assertEqualDiff('''\
+:Purpose: A sample command.
+:Usage:   bzr Demo
+
+:Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+:Examples:
+    Example 1::
+
+        cmd arg1
+
+    Example 2::
+
+        cmd arg2
+
+    A code block follows.
+
+    ::
+
+        bzr Demo something
+
+''',
+                             helptext)
 
     def test_concise_help_text(self):
         """Concise help text excludes the descriptive sections."""
@@ -186,41 +206,219 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEqualDiff(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n'
-            '\n'
-            'Examples:\n'
-            '    Example 1:\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+Examples:
+    Example 1:
+
+        cmd arg1
+
+''',
+                             helptext)
         helptext = cmd.get_help_text(verbose=False)
-        self.assertEquals(helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'See bzr help Demo for more details and examples.\n'
-            '\n')
-
-    def test_help_custom_section_ordering(self):
-        """Custom descriptive sections should remain in the order given."""
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+See bzr help Demo for more details and examples.
+
+''',
+                             helptext)
+
+    def test_help_custom_section_ordering(self):
+        """Custom descriptive sections should remain in the order given."""
+        class cmd_Demo(commands.Command):
+            __doc__ = """\
+A sample command.
+
+Blah blah blah.
+
+:Formats:
+  Interesting stuff about formats.
+
+:Examples:
+  Example 1::
+
+    cmd arg1
+
+:Tips:
+  Clever things to keep in mind.
+"""
+        cmd = cmd_Demo()
+        helptext = cmd.get_help_text()
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+Formats:
+  Interesting stuff about formats.
+
+Examples:
+  Example 1:
+
+    cmd arg1
+
+Tips:
+  Clever things to keep in mind.
+
+''',
+                             helptext)
+
+    def test_help_text_custom_usage(self):
+        """Help text may contain a custom usage section."""
+        class cmd_Demo(commands.Command):
+            __doc__ = """A sample command.
+
+            :Usage:
+                cmd Demo [opts] args
+
+                cmd Demo -h
+
+            Blah blah blah.
+            """
+        cmd = cmd_Demo()
+        helptext = cmd.get_help_text()
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:
+    cmd Demo [opts] args
+
+    cmd Demo -h
+
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+''',
+                             helptext)
+
+
+class ZzzTranslationsForDoc(ZzzTranslations):
+
+    _section_pat = re.compile(':\w+:\\n\\s+')
+    _indent_pat = re.compile('\\s+')
+
+    def zzz(self, s):
+        m = self._section_pat.match(s)
+        if m is None:
+            m = self._indent_pat.match(s)
+        if m:
+            return u'%szz{{%s}}' % (m.group(0), s[m.end():])
+        return u'zz{{%s}}' % s
+
+
+class TestCommandHelpI18n(tests.TestCase):
+    """Tests for help on translated commands."""
+
+    def setUp(self):
+        super(TestCommandHelpI18n, self).setUp()
+        self.overrideAttr(i18n, '_translations', ZzzTranslationsForDoc())
+
+    def assertCmdHelp(self, expected, cmd):
+        self.assertEqualDiff(textwrap.dedent(expected), cmd.get_help_text())
+
+    def test_command_help_includes_see_also(self):
+        class cmd_WithSeeAlso(commands.Command):
+            __doc__ = """A sample command."""
+            _see_also = ['foo', 'bar']
+        self.assertCmdHelp('''\
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr WithSeeAlso
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: bar, foo}}
+''',
+                           cmd_WithSeeAlso())
+
+    def test_get_help_text(self):
+        """Commands have a get_help_text method which returns their help."""
+        class cmd_Demo(commands.Command):
+            __doc__ = """A sample command."""
+        self.assertCmdHelp('''\
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr Demo
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+''',
+                           cmd_Demo())
+
+    def test_command_with_additional_see_also(self):
+        class cmd_WithSeeAlso(commands.Command):
+            __doc__ = """A sample command."""
+            _see_also = ['foo', 'bar']
+        cmd = cmd_WithSeeAlso()
+        helptext = cmd.get_help_text(['gam'])
+        self.assertEndsWith(
+            helptext,'''\
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: bar, foo, gam}}
+''')
+
+    def test_command_only_additional_see_also(self):
+        class cmd_WithSeeAlso(commands.Command):
+            __doc__ = """A sample command."""
+        cmd = cmd_WithSeeAlso()
+        helptext = cmd.get_help_text(['gam'])
+        self.assertEndsWith(
+            helptext, '''\
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: gam}}
+''')
+
+
+    def test_help_custom_section_ordering(self):
+        """Custom descriptive sections should remain in the order given."""
+        # The help formatter expect the class name to start with 'cmd_'
         class cmd_Demo(commands.Command):
             __doc__ = """A sample command.
  
@@ -237,33 +435,32 @@
             :Tips:
               Clever things to keep in mind.
             """
-        cmd = cmd_Demo()
-        helptext = cmd.get_help_text()
-        self.assertEqualDiff(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n'
-            '\n'
-            'Formats:\n'
-            '  Interesting stuff about formats.\n'
-            '\n'
-            'Examples:\n'
-            '  Example 1:\n'
-            '\n'
-            '    cmd arg1\n'
-            '\n'
-            'Tips:\n'
-            '  Clever things to keep in mind.\n'
-            '\n')
+        self.assertCmdHelp('''\
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr Demo
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+Description:
+  zz{{zz{{Blah blah blah.}}
+ 
+}}:Formats:
+  zz{{Interesting stuff about formats.}}
+ 
+Examples:
+  zz{{Example 1::}}
+ 
+    zz{{cmd arg1}}
+ 
+Tips:
+  zz{{Clever things to keep in mind.}}
+ 
+''',
+                           cmd_Demo())
 
     def test_help_text_custom_usage(self):
         """Help text may contain a custom usage section."""
@@ -277,24 +474,33 @@
 
             Blah blah blah.
             """
-        cmd = cmd_Demo()
-        helptext = cmd.get_help_text()
-        self.assertEquals(helptext,
-            'Purpose: A sample command.\n'
-            'Usage:\n'
-            '    cmd Demo [opts] args\n'
-            '\n'
-            '    cmd Demo -h\n'
-            '\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n\n')
+        self.assertCmdHelp('''\
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:
+    zz{{cmd Demo [opts] args}}
+
+    zz{{cmd Demo -h}}
+
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+Description:
+  zz{{zz{{Blah blah blah.}}
+
+}}
+''',
+                           cmd_Demo())
+
+
+class TestHelp(tests.TestCase):
+
+    def setUp(self):
+        tests.TestCase.setUp(self)
+        commands.install_bzr_command_hooks()
 
 
 class TestRegisteredTopic(TestHelp):
@@ -308,10 +514,10 @@
         self.assertEqual('basic', topic.topic)
 
     def test_get_help_text(self):
-        """A RegisteredTopic returns the get_detail results for get_help_text."""
+        """RegisteredTopic returns the get_detail results for get_help_text."""
         topic = help_topics.RegisteredTopic('commands')
         self.assertEqual(help_topics.topic_registry.get_detail('commands'),
-            topic.get_help_text())
+                         topic.get_help_text())
 
     def test_get_help_text_with_additional_see_also(self):
         topic = help_topics.RegisteredTopic('commands')
@@ -329,7 +535,7 @@
             '\n')
 
     def test_get_help_topic(self):
-        """The help topic for a RegisteredTopic is its topic from construction."""
+        """The help topic for RegisteredTopic is its topic from construction."""
         topic = help_topics.RegisteredTopic('foobar')
         self.assertEqual('foobar', topic.get_help_topic())
         topic = help_topics.RegisteredTopic('baz')
@@ -369,6 +575,31 @@
         self.assertEqual('', index.prefix)
 
 
+class TestConfigOptionIndex(TestHelp):
+    """Tests for the HelpCommandIndex class."""
+
+    def setUp(self):
+        super(TestConfigOptionIndex, self).setUp()
+        self.index = help_topics.ConfigOptionHelpIndex()
+
+    def test_get_topics_None(self):
+        """Searching for None returns an empty list."""
+        self.assertEqual([], self.index.get_topics(None))
+
+    def test_get_topics_no_topic(self):
+        self.assertEqual([], self.index.get_topics('nothing by this name'))
+
+    def test_prefix(self):
+        self.assertEqual('configuration/', self.index.prefix)
+
+    def test_get_topic_with_prefix(self):
+        topics = self.index.get_topics('configuration/default_format')
+        self.assertLength(1, topics)
+        opt = topics[0]
+        self.assertIsInstance(opt, config.Option)
+        self.assertEquals('default_format', opt.name)
+
+
 class TestCommandIndex(TestHelp):
     """Tests for the HelpCommandIndex class."""
 
@@ -411,16 +642,19 @@
     def test_default_search_path(self):
         """The default search path should include internal indexs."""
         indices = help.HelpIndices()
-        self.assertEqual(3, len(indices.search_path))
+        self.assertEqual(4, len(indices.search_path))
         # help topics should be searched in first.
         self.assertIsInstance(indices.search_path[0],
-            help_topics.HelpTopicIndex)
+                              help_topics.HelpTopicIndex)
         # with commands being search second.
         self.assertIsInstance(indices.search_path[1],
-            commands.HelpCommandIndex)
-        # and plugins are a third index.
+                              commands.HelpCommandIndex)
+        # plugins are a third index.
         self.assertIsInstance(indices.search_path[2],
-            plugin.PluginsHelpIndex)
+                              plugin.PluginsHelpIndex)
+        # config options are a fourth index
+        self.assertIsInstance(indices.search_path[3],
+                              help_topics.ConfigOptionHelpIndex)
 
     def test_search_for_unknown_topic_raises(self):
         """Searching for an unknown topic should raise NoHelpTopic."""

=== modified file 'bzrlib/tests/test_hooks.py'
--- bzrlib/tests/test_hooks.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_hooks.py	2011-04-02 00:08:39 +0000
@@ -19,53 +19,58 @@
 from bzrlib import (
     branch,
     errors,
+    hooks as _mod_hooks,
+    pyutils,
     tests,
     )
 from bzrlib.hooks import (
     HookPoint,
     Hooks,
+    install_lazy_named_hook,
     known_hooks,
     known_hooks_key_to_object,
     known_hooks_key_to_parent_and_attribute,
     )
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    )
 
 
 class TestHooks(tests.TestCase):
 
     def test_create_hook_first(self):
-        hooks = Hooks()
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
         doc = ("Invoked after changing the tip of a branch object. Called with"
             "a bzrlib.branch.PostChangeBranchTipParams object")
         hook = HookPoint("post_tip_change", doc, (0, 15), None)
-        hooks.create_hook(hook)
+        self.applyDeprecated(deprecated_in((2, 4)), hooks.create_hook, hook)
         self.assertEqual(hook, hooks['post_tip_change'])
 
     def test_create_hook_name_collision_errors(self):
-        hooks = Hooks()
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
         doc = ("Invoked after changing the tip of a branch object. Called with"
             "a bzrlib.branch.PostChangeBranchTipParams object")
         hook = HookPoint("post_tip_change", doc, (0, 15), None)
         hook2 = HookPoint("post_tip_change", None, None, None)
-        hooks.create_hook(hook)
-        self.assertRaises(errors.DuplicateKey, hooks.create_hook, hook2)
+        self.applyDeprecated(deprecated_in((2, 4)), hooks.create_hook, hook)
+        self.assertRaises(errors.DuplicateKey, self.applyDeprecated,
+            deprecated_in((2, 4, 0)), hooks.create_hook, hook2)
         self.assertEqual(hook, hooks['post_tip_change'])
 
     def test_docs(self):
         """docs() should return something reasonable about the Hooks."""
         class MyHooks(Hooks):
             pass
-        hooks = MyHooks()
+        hooks = MyHooks("bzrlib.tests.test_hooks", "some_hooks")
         hooks['legacy'] = []
-        hook1 = HookPoint('post_tip_change',
+        hooks.add_hook('post_tip_change',
             "Invoked after the tip of a branch changes. Called with "
-            "a ChangeBranchTipParams object.", (1, 4), None)
-        hook2 = HookPoint('pre_tip_change',
+            "a ChangeBranchTipParams object.", (1, 4))
+        hooks.add_hook('pre_tip_change',
             "Invoked before the tip of a branch changes. Called with "
             "a ChangeBranchTipParams object. Hooks should raise "
             "TipChangeRejected to signal that a tip change is not permitted.",
             (1, 6), None)
-        hooks.create_hook(hook1)
-        hooks.create_hook(hook2)
         self.assertEqualDiff(
             "MyHooks\n"
             "-------\n"
@@ -93,22 +98,105 @@
             "signal that a tip change is not permitted.\n", hooks.docs())
 
     def test_install_named_hook_raises_unknown_hook(self):
-        hooks = Hooks()
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
         self.assertRaises(errors.UnknownHook, hooks.install_named_hook, 'silly',
                           None, "")
 
     def test_install_named_hook_appends_known_hook(self):
-        hooks = Hooks()
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
         hooks['set_rh'] = []
         hooks.install_named_hook('set_rh', None, "demo")
         self.assertEqual(hooks['set_rh'], [None])
 
     def test_install_named_hook_and_retrieve_name(self):
-        hooks = Hooks()
+        hooks = Hooks("bzrlib.tests.test_hooks", "somehooks")
         hooks['set_rh'] = []
         hooks.install_named_hook('set_rh', None, "demo")
         self.assertEqual("demo", hooks.get_hook_name(None))
 
+    def test_uninstall_named_hook(self):
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
+        hooks.add_hook('set_rh', "Set revision history", (2, 0))
+        hooks.install_named_hook('set_rh', None, "demo")
+        self.assertEqual(1, len(hooks["set_rh"]))
+        hooks.uninstall_named_hook("set_rh", "demo")
+        self.assertEqual(0, len(hooks["set_rh"]))
+
+    def test_uninstall_multiple_named_hooks(self):
+        # Multiple callbacks with the same label all get removed
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
+        hooks.add_hook('set_rh', "Set revision history", (2, 0))
+        hooks.install_named_hook('set_rh', 1, "demo")
+        hooks.install_named_hook('set_rh', 2, "demo")
+        hooks.install_named_hook('set_rh', 3, "othername")
+        self.assertEqual(3, len(hooks["set_rh"]))
+        hooks.uninstall_named_hook("set_rh", "demo")
+        self.assertEqual(1, len(hooks["set_rh"]))
+
+    def test_uninstall_named_hook_unknown_callable(self):
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
+        hooks.add_hook('set_rh', "Set revision hsitory", (2, 0))
+        self.assertRaises(KeyError, hooks.uninstall_named_hook, "set_rh",
+            "demo")
+
+    def test_uninstall_named_hook_raises_unknown_hook(self):
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
+        self.assertRaises(errors.UnknownHook, hooks.uninstall_named_hook,
+            'silly', "")
+
+    def test_uninstall_named_hook_old_style(self):
+        hooks = Hooks("bzrlib.tests.test_hooks", "some_hooks")
+        hooks["set_rh"] = []
+        hooks.install_named_hook('set_rh', None, "demo")
+        self.assertRaises(errors.UnsupportedOperation,
+            hooks.uninstall_named_hook, "set_rh", "demo")
+
+    hooks = Hooks("bzrlib.tests.test_hooks", "TestHooks.hooks")
+
+    def test_install_lazy_named_hook(self):
+        # When the hook points are not yet registered the hook is
+        # added to the _lazy_hooks dictionary in bzrlib.hooks.
+        self.hooks.add_hook('set_rh', "doc", (0, 15))
+        set_rh = lambda: None
+        install_lazy_named_hook('bzrlib.tests.test_hooks',
+            'TestHooks.hooks', 'set_rh', set_rh, "demo")
+        set_rh_lazy_hooks = _mod_hooks._lazy_hooks[
+            ('bzrlib.tests.test_hooks', 'TestHooks.hooks', 'set_rh')]
+        self.assertEquals(1, len(set_rh_lazy_hooks))
+        self.assertEquals(set_rh, set_rh_lazy_hooks[0][0].get_obj())
+        self.assertEquals("demo", set_rh_lazy_hooks[0][1])
+        self.assertEqual(list(TestHooks.hooks['set_rh']), [set_rh])
+
+    set_rh = lambda: None
+
+    def test_install_named_hook_lazy(self):
+        hooks = Hooks("bzrlib.tests.hooks", "some_hooks")
+        hooks['set_rh'] = HookPoint("set_rh", "doc", (0, 15), None)
+        hooks.install_named_hook_lazy('set_rh', 'bzrlib.tests.test_hooks',
+            'TestHooks.set_rh', "demo")
+        self.assertEqual(list(hooks['set_rh']), [TestHooks.set_rh])
+
+    def test_install_named_hook_lazy_old(self):
+        # An exception is raised if a lazy hook is raised for
+        # an old style hook point.
+        hooks = Hooks("bzrlib.tests.hooks", "some_hooks")
+        hooks['set_rh'] = []
+        self.assertRaises(errors.UnsupportedOperation,
+            hooks.install_named_hook_lazy,
+            'set_rh', 'bzrlib.tests.test_hooks', 'TestHooks.set_rh',
+            "demo")
+
+    def test_valid_lazy_hooks(self):
+        # Make sure that all the registered lazy hooks are referring to existing
+        # hook points which allow lazy registration.
+        for key, callbacks in _mod_hooks._lazy_hooks.iteritems():
+            (module_name, member_name, hook_name) = key
+            obj = pyutils.get_named_object(module_name, member_name)
+            self.assertEquals(obj._module, module_name)
+            self.assertEquals(obj._member_name, member_name)
+            self.assertTrue(hook_name in obj)
+            self.assertIs(callbacks, obj[hook_name]._callbacks)
+
 
 class TestHook(tests.TestCase):
 
@@ -141,6 +229,29 @@
         hook.hook(callback, "my callback")
         self.assertEqual([callback], list(hook))
 
+    def lazy_callback():
+        pass
+
+    def test_lazy_hook(self):
+        hook = HookPoint("foo", "no docs", None, None)
+        hook.hook_lazy(
+            "bzrlib.tests.test_hooks", "TestHook.lazy_callback",
+            "my callback")
+        self.assertEqual([TestHook.lazy_callback], list(hook))
+
+    def test_uninstall(self):
+        hook = HookPoint("foo", "no docs", None, None)
+        hook.hook_lazy(
+            "bzrlib.tests.test_hooks", "TestHook.lazy_callback",
+            "my callback")
+        self.assertEqual([TestHook.lazy_callback], list(hook))
+        hook.uninstall("my callback")
+        self.assertEqual([], list(hook))
+
+    def test_uninstall_unknown(self):
+        hook = HookPoint("foo", "no docs", None, None)
+        self.assertRaises(KeyError, hook.uninstall, "my callback")
+
     def test___repr(self):
         # The repr should list all the callbacks, with names.
         hook = HookPoint("foo", "no docs", None, None)
@@ -175,10 +286,20 @@
         self.assertIs(branch.Branch.hooks,
             known_hooks_key_to_object(('bzrlib.branch', 'Branch.hooks')))
 
+    def test_known_hooks_key_to_parent_and_attribute_deprecated(self):
+        self.assertEqual((branch.Branch, 'hooks'),
+            self.applyDeprecated(deprecated_in((2,3)),
+                known_hooks_key_to_parent_and_attribute,
+                ('bzrlib.branch', 'Branch.hooks')))
+        self.assertEqual((branch, 'Branch'),
+            self.applyDeprecated(deprecated_in((2,3)),
+                known_hooks_key_to_parent_and_attribute,
+                ('bzrlib.branch', 'Branch')))
+
     def test_known_hooks_key_to_parent_and_attribute(self):
         self.assertEqual((branch.Branch, 'hooks'),
-            known_hooks_key_to_parent_and_attribute(
+            known_hooks.key_to_parent_and_attribute(
             ('bzrlib.branch', 'Branch.hooks')))
         self.assertEqual((branch, 'Branch'),
-            known_hooks_key_to_parent_and_attribute(
+            known_hooks.key_to_parent_and_attribute(
             ('bzrlib.branch', 'Branch')))

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2010-07-15 12:53:44 +0000
+++ bzrlib/tests/test_http.py	2011-10-12 16:00:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,10 +23,7 @@
 # TODO: Should be renamed to bzrlib.transport.http.tests?
 # TODO: What about renaming to bzrlib.tests.transport.http ?
 
-from cStringIO import StringIO
 import httplib
-import os
-import select
 import SimpleHTTPServer
 import socket
 import sys
@@ -35,19 +32,26 @@
 import bzrlib
 from bzrlib import (
     bzrdir,
+    cethread,
     config,
+    debug,
     errors,
     osutils,
     remote as _mod_remote,
     tests,
+    trace,
     transport,
     ui,
-    urlutils,
     )
 from bzrlib.tests import (
     features,
     http_server,
     http_utils,
+    test_server,
+    )
+from bzrlib.tests.scenarios import (
+    load_tests_apply_scenarios,
+    multiply_scenarios,
     )
 from bzrlib.transport import (
     http,
@@ -63,107 +67,76 @@
     from bzrlib.transport.http._pycurl import PyCurlTransport
 
 
-def load_tests(standard_tests, module, loader):
-    """Multiply tests for http clients and protocol versions."""
-    result = loader.suiteClass()
-
-    # one for each transport implementation
-    t_tests, remaining_tests = tests.split_suite_by_condition(
-        standard_tests, tests.condition_isinstance((
-                TestHttpTransportRegistration,
-                TestHttpTransportUrls,
-                Test_redirected_to,
-                )))
+load_tests = load_tests_apply_scenarios
+
+
+def vary_by_http_client_implementation():
+    """Test the two libraries we can use, pycurl and urllib."""
     transport_scenarios = [
         ('urllib', dict(_transport=_urllib.HttpTransport_urllib,
                         _server=http_server.HttpServer_urllib,
-                        _qualified_prefix='http+urllib',)),
+                        _url_protocol='http+urllib',)),
         ]
     if features.pycurl.available():
         transport_scenarios.append(
             ('pycurl', dict(_transport=PyCurlTransport,
                             _server=http_server.HttpServer_PyCurl,
-                            _qualified_prefix='http+pycurl',)))
-    tests.multiply_tests(t_tests, transport_scenarios, result)
-
-    protocol_scenarios = [
-            ('HTTP/1.0',  dict(_protocol_version='HTTP/1.0')),
-            ('HTTP/1.1',  dict(_protocol_version='HTTP/1.1')),
-            ]
-
-    # some tests are parametrized by the protocol version only
-    p_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance((
-                TestAuthOnRedirected,
-                )))
-    tests.multiply_tests(p_tests, protocol_scenarios, result)
-
-    # each implementation tested with each HTTP version
-    tp_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance((
-                SmartHTTPTunnellingTest,
-                TestDoCatchRedirections,
-                TestHTTPConnections,
-                TestHTTPRedirections,
-                TestHTTPSilentRedirections,
-                TestLimitedRangeRequestServer,
-                TestPost,
-                TestProxyHttpServer,
-                TestRanges,
-                TestSpecificRequestHandler,
-                )))
-    tp_scenarios = tests.multiply_scenarios(transport_scenarios,
-                                            protocol_scenarios)
-    tests.multiply_tests(tp_tests, tp_scenarios, result)
-
-    # proxy auth: each auth scheme on all http versions on all implementations.
-    tppa_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance((
-                TestProxyAuth,
-                )))
-    proxy_auth_scheme_scenarios = [
-        ('basic', dict(_auth_server=http_utils.ProxyBasicAuthServer)),
-        ('digest', dict(_auth_server=http_utils.ProxyDigestAuthServer)),
-        ('basicdigest',
-         dict(_auth_server=http_utils.ProxyBasicAndDigestAuthServer)),
+                            _url_protocol='http+pycurl',)))
+    return transport_scenarios
+
+
+def vary_by_http_protocol_version():
+    """Test on http/1.0 and 1.1"""
+    return [
+        ('HTTP/1.0',  dict(_protocol_version='HTTP/1.0')),
+        ('HTTP/1.1',  dict(_protocol_version='HTTP/1.1')),
         ]
-    tppa_scenarios = tests.multiply_scenarios(tp_scenarios,
-                                              proxy_auth_scheme_scenarios)
-    tests.multiply_tests(tppa_tests, tppa_scenarios, result)
-
-    # auth: each auth scheme on all http versions on all implementations.
-    tpa_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance((
-                TestAuth,
-                )))
-    auth_scheme_scenarios = [
+
+
+def vary_by_http_auth_scheme():
+    scenarios = [
         ('basic', dict(_auth_server=http_utils.HTTPBasicAuthServer)),
         ('digest', dict(_auth_server=http_utils.HTTPDigestAuthServer)),
         ('basicdigest',
-         dict(_auth_server=http_utils.HTTPBasicAndDigestAuthServer)),
-        ]
-    tpa_scenarios = tests.multiply_scenarios(tp_scenarios,
-                                             auth_scheme_scenarios)
-    tests.multiply_tests(tpa_tests, tpa_scenarios, result)
-
-    # activity: on all http[s] versions on all implementations
-    tpact_tests, remaining_tests = tests.split_suite_by_condition(
-        remaining_tests, tests.condition_isinstance((
-                TestActivity,
-                )))
+            dict(_auth_server=http_utils.HTTPBasicAndDigestAuthServer)),
+        ]
+    # Add some attributes common to all scenarios
+    for scenario_id, scenario_dict in scenarios:
+        scenario_dict.update(_auth_header='Authorization',
+                             _username_prompt_prefix='',
+                             _password_prompt_prefix='')
+    return scenarios
+
+
+def vary_by_http_proxy_auth_scheme():
+    scenarios = [
+        ('proxy-basic', dict(_auth_server=http_utils.ProxyBasicAuthServer)),
+        ('proxy-digest', dict(_auth_server=http_utils.ProxyDigestAuthServer)),
+        ('proxy-basicdigest',
+            dict(_auth_server=http_utils.ProxyBasicAndDigestAuthServer)),
+        ]
+    # Add some attributes common to all scenarios
+    for scenario_id, scenario_dict in scenarios:
+        scenario_dict.update(_auth_header='Proxy-Authorization',
+                             _username_prompt_prefix='Proxy ',
+                             _password_prompt_prefix='Proxy ')
+    return scenarios
+
+
+def vary_by_http_activity():
     activity_scenarios = [
         ('urllib,http', dict(_activity_server=ActivityHTTPServer,
-                             _transport=_urllib.HttpTransport_urllib,)),
+                            _transport=_urllib.HttpTransport_urllib,)),
         ]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         activity_scenarios.append(
             ('urllib,https', dict(_activity_server=ActivityHTTPSServer,
-                                  _transport=_urllib.HttpTransport_urllib,)),)
+                                _transport=_urllib.HttpTransport_urllib,)),)
     if features.pycurl.available():
         activity_scenarios.append(
             ('pycurl,http', dict(_activity_server=ActivityHTTPServer,
-                                 _transport=PyCurlTransport,)),)
-        if tests.HTTPSServerFeature.available():
+                                _transport=PyCurlTransport,)),)
+        if features.HTTPSServerFeature.available():
             from bzrlib.tests import (
                 ssl_certs,
                 )
@@ -180,16 +153,8 @@
 
             activity_scenarios.append(
                 ('pycurl,https', dict(_activity_server=ActivityHTTPSServer,
-                                      _transport=HTTPS_pycurl_transport,)),)
-
-    tpact_scenarios = tests.multiply_scenarios(activity_scenarios,
-                                               protocol_scenarios)
-    tests.multiply_tests(tpact_tests, tpact_scenarios, result)
-
-    # No parametrization for the remaining tests
-    result.addTests(remaining_tests)
-
-    return result
+                                    _transport=HTTPS_pycurl_transport,)),)
+    return activity_scenarios
 
 
 class FakeManager(object):
@@ -228,25 +193,22 @@
         self._sock.bind(('127.0.0.1', 0))
         self.host, self.port = self._sock.getsockname()
         self._ready = threading.Event()
-        self._thread = threading.Thread(target=self._accept_read_and_reply)
-        self._thread.setDaemon(True)
+        self._thread = test_server.TestThread(
+            sync_event=self._ready, target=self._accept_read_and_reply)
         self._thread.start()
-        self._ready.wait(5)
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread started: %s\n' % (self._thread.ident,))
+        self._ready.wait()
 
     def _accept_read_and_reply(self):
         self._sock.listen(1)
         self._ready.set()
-        self._sock.settimeout(5)
-        try:
-            conn, address = self._sock.accept()
-            # On win32, the accepted connection will be non-blocking to start
-            # with because we're using settimeout.
-            conn.setblocking(True)
+        conn, address = self._sock.accept()
+        if self._expect_body_tail is not None:
             while not self.received_bytes.endswith(self._expect_body_tail):
                 self.received_bytes += conn.recv(4096)
             conn.sendall('HTTP/1.1 200 OK\r\n')
-        except socket.timeout:
-            # Make sure the client isn't stuck waiting for us to e.g. accept.
+        try:
             self._sock.close()
         except socket.error:
             # The client may have already closed the socket.
@@ -254,12 +216,18 @@
 
     def stop_server(self):
         try:
-            self._sock.close()
+            # Issue a fake connection to wake up the server and allow it to
+            # finish quickly
+            fake_conn = osutils.connect_socket((self.host, self.port))
+            fake_conn.close()
         except socket.error:
             # We might have already closed it.  We don't care.
             pass
         self.host = None
         self.port = None
+        self._thread.join()
+        if 'threads' in tests.selftest_debug_flags:
+            sys.stderr.write('Thread  joined: %s\n' % (self._thread.ident,))
 
 
 class TestAuthHeader(tests.TestCase):
@@ -301,6 +269,40 @@
         self.assertEqual('realm="Thou should not pass"', remainder)
 
 
+class TestHTTPRangeParsing(tests.TestCase):
+
+    def setUp(self):
+        super(TestHTTPRangeParsing, self).setUp()
+        # We focus on range  parsing here and ignore everything else
+        class RequestHandler(http_server.TestingHTTPRequestHandler):
+            def setup(self): pass
+            def handle(self): pass
+            def finish(self): pass
+
+        self.req_handler = RequestHandler(None, None, None)
+
+    def assertRanges(self, ranges, header, file_size):
+        self.assertEquals(ranges,
+                          self.req_handler._parse_ranges(header, file_size))
+
+    def test_simple_range(self):
+        self.assertRanges([(0,2)], 'bytes=0-2', 12)
+
+    def test_tail(self):
+        self.assertRanges([(8, 11)], 'bytes=-4', 12)
+
+    def test_tail_bigger_than_file(self):
+        self.assertRanges([(0, 11)], 'bytes=-99', 12)
+
+    def test_range_without_end(self):
+        self.assertRanges([(4, 11)], 'bytes=4-', 12)
+
+    def test_invalid_ranges(self):
+        self.assertRanges(None, 'bytes=12-22', 12)
+        self.assertRanges(None, 'bytes=1-3,12-22', 12)
+        self.assertRanges(None, 'bytes=-', 12)
+
+
 class TestHTTPServer(tests.TestCase):
     """Test the HTTP servers implementations."""
 
@@ -309,29 +311,20 @@
 
             protocol_version = 'HTTP/0.1'
 
-        server = http_server.HttpServer(BogusRequestHandler)
-        try:
-            self.assertRaises(httplib.UnknownProtocol, server.start_server)
-        except:
-            server.stop_server()
-            self.fail('HTTP Server creation did not raise UnknownProtocol')
+        self.assertRaises(httplib.UnknownProtocol,
+                          http_server.HttpServer, BogusRequestHandler)
 
     def test_force_invalid_protocol(self):
-        server = http_server.HttpServer(protocol_version='HTTP/0.1')
-        try:
-            self.assertRaises(httplib.UnknownProtocol, server.start_server)
-        except:
-            server.stop_server()
-            self.fail('HTTP Server creation did not raise UnknownProtocol')
+        self.assertRaises(httplib.UnknownProtocol,
+                          http_server.HttpServer, protocol_version='HTTP/0.1')
 
     def test_server_start_and_stop(self):
         server = http_server.HttpServer()
+        self.addCleanup(server.stop_server)
         server.start_server()
-        try:
-            self.assertTrue(server._http_running)
-        finally:
-            server.stop_server()
-        self.assertFalse(server._http_running)
+        self.assertTrue(server.server is not None)
+        self.assertTrue(server.server.serving is not None)
+        self.assertTrue(server.server.serving)
 
     def test_create_http_server_one_zero(self):
         class RequestHandlerOneZero(http_server.TestingHTTPRequestHandler):
@@ -340,7 +333,7 @@
 
         server = http_server.HttpServer(RequestHandlerOneZero)
         self.start_server(server)
-        self.assertIsInstance(server._httpd, http_server.TestingHTTPServer)
+        self.assertIsInstance(server.server, http_server.TestingHTTPServer)
 
     def test_create_http_server_one_one(self):
         class RequestHandlerOneOne(http_server.TestingHTTPRequestHandler):
@@ -349,7 +342,7 @@
 
         server = http_server.HttpServer(RequestHandlerOneOne)
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingThreadingHTTPServer)
 
     def test_create_http_server_force_one_one(self):
@@ -360,7 +353,7 @@
         server = http_server.HttpServer(RequestHandlerOneZero,
                                         protocol_version='HTTP/1.1')
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingThreadingHTTPServer)
 
     def test_create_http_server_force_one_zero(self):
@@ -371,7 +364,7 @@
         server = http_server.HttpServer(RequestHandlerOneOne,
                                         protocol_version='HTTP/1.0')
         self.start_server(server)
-        self.assertIsInstance(server._httpd,
+        self.assertIsInstance(server.server,
                               http_server.TestingHTTPServer)
 
 
@@ -406,29 +399,31 @@
 class TestHttpTransportUrls(tests.TestCase):
     """Test the http urls."""
 
+    scenarios = vary_by_http_client_implementation()
+
     def test_abs_url(self):
         """Construction of absolute http URLs"""
-        t = self._transport('http://bazaar-vcs.org/bzr/bzr.dev/')
+        t = self._transport('http://example.com/bzr/bzr.dev/')
         eq = self.assertEqualDiff
-        eq(t.abspath('.'), 'http://bazaar-vcs.org/bzr/bzr.dev')
-        eq(t.abspath('foo/bar'), 'http://bazaar-vcs.org/bzr/bzr.dev/foo/bar')
-        eq(t.abspath('.bzr'), 'http://bazaar-vcs.org/bzr/bzr.dev/.bzr')
+        eq(t.abspath('.'), 'http://example.com/bzr/bzr.dev')
+        eq(t.abspath('foo/bar'), 'http://example.com/bzr/bzr.dev/foo/bar')
+        eq(t.abspath('.bzr'), 'http://example.com/bzr/bzr.dev/.bzr')
         eq(t.abspath('.bzr/1//2/./3'),
-           'http://bazaar-vcs.org/bzr/bzr.dev/.bzr/1/2/3')
+           'http://example.com/bzr/bzr.dev/.bzr/1/2/3')
 
     def test_invalid_http_urls(self):
         """Trap invalid construction of urls"""
-        t = self._transport('http://bazaar-vcs.org/bzr/bzr.dev/')
+        self._transport('http://example.com/bzr/bzr.dev/')
         self.assertRaises(errors.InvalidURL,
                           self._transport,
-                          'http://http://bazaar-vcs.org/bzr/bzr.dev/')
+                          'http://http://example.com/bzr/bzr.dev/')
 
     def test_http_root_urls(self):
         """Construction of URLs from server root"""
-        t = self._transport('http://bzr.ozlabs.org/')
+        t = self._transport('http://example.com/')
         eq = self.assertEqualDiff
         eq(t.abspath('.bzr/tree-version'),
-           'http://bzr.ozlabs.org/.bzr/tree-version')
+           'http://example.com/.bzr/tree-version')
 
     def test_http_impl_urls(self):
         """There are servers which ask for particular clients to connect"""
@@ -436,7 +431,7 @@
         server.start_server()
         try:
             url = server.get_url()
-            self.assertTrue(url.startswith('%s://' % self._qualified_prefix))
+            self.assertTrue(url.startswith('%s://' % self._url_protocol))
         finally:
             server.stop_server()
 
@@ -480,6 +475,11 @@
 class TestHTTPConnections(http_utils.TestCaseWithWebserver):
     """Test the http connections."""
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     def setUp(self):
         http_utils.TestCaseWithWebserver.setUp(self)
         self.build_tree(['foo/', 'foo/bar'], line_endings='binary',
@@ -487,7 +487,7 @@
 
     def test_http_has(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertEqual(t.has('foo/bar'), True)
         self.assertEqual(len(server.logs), 1)
         self.assertContainsRe(server.logs[0],
@@ -495,14 +495,14 @@
 
     def test_http_has_not_found(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertEqual(t.has('not-found'), False)
         self.assertContainsRe(server.logs[1],
             r'"HEAD /not-found HTTP/1.." 404 - "-" "bzr/')
 
     def test_http_get(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         fp = t.get('foo/bar')
         self.assertEqualDiff(
             fp.read(),
@@ -530,24 +530,35 @@
 class TestHttpTransportRegistration(tests.TestCase):
     """Test registrations of various http implementations"""
 
+    scenarios = vary_by_http_client_implementation()
+
     def test_http_registered(self):
-        t = transport.get_transport('%s://foo.com/' % self._qualified_prefix)
+        t = transport.get_transport_from_url(
+            '%s://foo.com/' % self._url_protocol)
         self.assertIsInstance(t, transport.Transport)
         self.assertIsInstance(t, self._transport)
 
 
 class TestPost(tests.TestCase):
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     def test_post_body_is_received(self):
         server = RecordingServer(expect_body_tail='end-of-body',
-            scheme=self._qualified_prefix)
+                                 scheme=self._url_protocol)
         self.start_server(server)
         url = server.get_url()
-        http_transport = self._transport(url)
+        # FIXME: needs a cleanup -- vila 20100611
+        http_transport = transport.get_transport_from_url(url)
         code, response = http_transport._post('abc def end-of-body')
         self.assertTrue(
             server.received_bytes.startswith('POST /.bzr/smart HTTP/1.'))
         self.assertTrue('content-length: 19\r' in server.received_bytes.lower())
+        self.assertTrue('content-type: application/octet-stream\r'
+                        in server.received_bytes.lower())
         # The transport should not be assuming that the server can accept
         # chunked encoding the first time it connects, because HTTP/1.1, so we
         # check for the literal string.
@@ -589,12 +600,19 @@
     Daughter classes are expected to override _req_handler_class
     """
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     # Provide a useful default
     _req_handler_class = http_server.TestingHTTPRequestHandler
 
     def create_transport_readonly_server(self):
-        return http_server.HttpServer(self._req_handler_class,
-                                      protocol_version=self._protocol_version)
+        server = http_server.HttpServer(self._req_handler_class,
+                                        protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
@@ -605,7 +623,7 @@
 class WallRequestHandler(http_server.TestingHTTPRequestHandler):
     """Whatever request comes in, close the connection"""
 
-    def handle_one_request(self):
+    def _handle_one_request(self):
         """Handle a single HTTP request, by abruptly closing the connection"""
         self.close_connection = 1
 
@@ -616,8 +634,7 @@
     _req_handler_class = WallRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # Unfortunately httplib (see HTTPResponse._read_status
         # for details) make no distinction between a closed
         # socket and badly formatted status line, so we can't
@@ -629,8 +646,7 @@
                           t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises((errors.ConnectionError, errors.ConnectionReset,
                            errors.InvalidHttpResponse),
                           t.get, 'foo/bar')
@@ -653,13 +669,11 @@
     _req_handler_class = BadStatusRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.get, 'foo/bar')
 
 
@@ -670,6 +684,10 @@
         """Fakes handling a single HTTP request, returns a bad status"""
         ignored = http_server.TestingHTTPRequestHandler.parse_request(self)
         self.wfile.write("Invalid status line\r\n")
+        # If we don't close the connection pycurl will hang. Since this is a
+        # stress test we don't *have* to respect the protocol, but we don't
+        # have to sabotage it too much either.
+        self.close_connection = True
         return False
 
 
@@ -681,18 +699,6 @@
 
     _req_handler_class = InvalidStatusRequestHandler
 
-    def test_http_has(self):
-        if self._testing_pycurl() and self._protocol_version == 'HTTP/1.1':
-            raise tests.KnownFailure(
-                'pycurl hangs if the server send back garbage')
-        super(TestInvalidStatusServer, self).test_http_has()
-
-    def test_http_get(self):
-        if self._testing_pycurl() and self._protocol_version == 'HTTP/1.1':
-            raise tests.KnownFailure(
-                'pycurl hangs if the server send back garbage')
-        super(TestInvalidStatusServer, self).test_http_get()
-
 
 class BadProtocolRequestHandler(http_server.TestingHTTPRequestHandler):
     """Whatever request comes in, returns a bad protocol version"""
@@ -720,13 +726,11 @@
         super(TestBadProtocolServer, self).setUp()
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.InvalidHttpResponse, t.get, 'foo/bar')
 
 
@@ -746,13 +750,11 @@
     _req_handler_class = ForbiddenRequestHandler
 
     def test_http_has(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.TransportError, t.has, 'foo/bar')
 
     def test_http_get(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         self.assertRaises(errors.TransportError, t.get, 'foo/bar')
 
 
@@ -797,8 +799,7 @@
         self.build_tree_contents([('a', '0123456789')],)
 
     def test_readv(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 1), (1, 1), (3, 2), (9, 1))))
         self.assertEqual(l[0], (0, '0'))
         self.assertEqual(l[1], (1, '1'))
@@ -806,8 +807,7 @@
         self.assertEqual(l[3], (9, '9'))
 
     def test_readv_out_of_order(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((1, 1), (9, 1), (0, 1), (3, 2))))
         self.assertEqual(l[0], (1, '1'))
         self.assertEqual(l[1], (9, '9'))
@@ -815,8 +815,7 @@
         self.assertEqual(l[3], (3, '34'))
 
     def test_readv_invalid_ranges(self):
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
 
         # This is intentionally reading off the end of the file
         # since we are sure that it cannot get there
@@ -830,7 +829,7 @@
 
     def test_readv_multiple_get_requests(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._max_readv_combine = 1
         t._max_get_ranges = 1
@@ -844,7 +843,7 @@
 
     def test_readv_get_max_size(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests by limiting the number of
         # bytes by request. Note that this apply to coalesced offsets only, a
         # single range will keep its size even if bigger than the limit.
@@ -859,10 +858,10 @@
 
     def test_complete_readv_leave_pipe_clean(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._get_max_size = 2
-        l = list(t.readv('a', ((0, 1), (1, 1), (2, 4), (6, 4))))
+        list(t.readv('a', ((0, 1), (1, 1), (2, 4), (6, 4))))
         # The server should have issued 3 requests
         self.assertEqual(3, server.GET_request_nb)
         self.assertEqual('0123456789', t.get_bytes('a'))
@@ -870,7 +869,7 @@
 
     def test_incomplete_readv_leave_pipe_clean(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # force transport to issue multiple requests
         t._get_max_size = 2
         # Don't collapse readv results into a list so that we leave unread
@@ -945,6 +944,8 @@
     def get_multiple_ranges(self, file, file_size, ranges):
         self.send_response(206)
         self.send_header('Accept-Ranges', 'bytes')
+        # XXX: this is strange; the 'random' name below seems undefined and
+        # yet the tests pass -- mbp 2010-10-11 bug 658773
         boundary = "%d" % random.randint(0,0x7FFFFFFF)
         self.send_header("Content-Type",
                          "multipart/byteranges; boundary=%s" % boundary)
@@ -1012,7 +1013,7 @@
                 return
             self.send_range_content(file, start, end - start + 1)
             cur += 1
-        # No final boundary
+        # Final boundary
         self.wfile.write(boundary_line)
 
 
@@ -1026,7 +1027,7 @@
 
     def test_readv_with_short_reads(self):
         server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # Force separate ranges for each offset
         t._bytes_to_read_before_seek = 0
         ireadv = iter(t.readv('a', ((0, 1), (2, 1), (4, 2), (9, 1))))
@@ -1047,6 +1048,73 @@
         # that mode
         self.assertEqual('single', t._range_hint)
 
+
+class TruncatedBeforeBoundaryRequestHandler(
+    http_server.TestingHTTPRequestHandler):
+    """Truncation before a boundary, like in bug 198646"""
+
+    _truncated_ranges = 1
+
+    def get_multiple_ranges(self, file, file_size, ranges):
+        self.send_response(206)
+        self.send_header('Accept-Ranges', 'bytes')
+        boundary = 'tagada'
+        self.send_header('Content-Type',
+                         'multipart/byteranges; boundary=%s' % boundary)
+        boundary_line = '--%s\r\n' % boundary
+        # Calculate the Content-Length
+        content_length = 0
+        for (start, end) in ranges:
+            content_length += len(boundary_line)
+            content_length += self._header_line_length(
+                'Content-type', 'application/octet-stream')
+            content_length += self._header_line_length(
+                'Content-Range', 'bytes %d-%d/%d' % (start, end, file_size))
+            content_length += len('\r\n') # end headers
+            content_length += end - start # + 1
+        content_length += len(boundary_line)
+        self.send_header('Content-length', content_length)
+        self.end_headers()
+
+        # Send the multipart body
+        cur = 0
+        for (start, end) in ranges:
+            if cur + self._truncated_ranges >= len(ranges):
+                # Abruptly ends the response and close the connection
+                self.close_connection = 1
+                return
+            self.wfile.write(boundary_line)
+            self.send_header('Content-type', 'application/octet-stream')
+            self.send_header('Content-Range', 'bytes %d-%d/%d'
+                             % (start, end, file_size))
+            self.end_headers()
+            self.send_range_content(file, start, end - start + 1)
+            cur += 1
+        # Final boundary
+        self.wfile.write(boundary_line)
+
+
+class TestTruncatedBeforeBoundary(TestSpecificRequestHandler):
+    """Tests the case of bug 198646, disconnecting before a boundary."""
+
+    _req_handler_class = TruncatedBeforeBoundaryRequestHandler
+
+    def setUp(self):
+        super(TestTruncatedBeforeBoundary, self).setUp()
+        self.build_tree_contents([('a', '0123456789')],)
+
+    def test_readv_with_short_reads(self):
+        server = self.get_readonly_server()
+        t = self.get_readonly_transport()
+        # Force separate ranges for each offset
+        t._bytes_to_read_before_seek = 0
+        ireadv = iter(t.readv('a', ((0, 1), (2, 1), (4, 2), (9, 1))))
+        self.assertEqual((0, '0'), ireadv.next())
+        self.assertEqual((2, '2'), ireadv.next())
+        self.assertEqual((4, '45'), ireadv.next())
+        self.assertEqual((9, '9'), ireadv.next())
+
+
 class LimitedRangeRequestHandler(http_server.TestingHTTPRequestHandler):
     """Errors out when range specifiers exceed the limit"""
 
@@ -1076,6 +1144,11 @@
 class TestLimitedRangeRequestServer(http_utils.TestCaseWithWebserver):
     """Tests readv requests against a server erroring out on too much ranges."""
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     # Requests with more range specifiers will error out
     range_limit = 3
 
@@ -1083,9 +1156,6 @@
         return LimitedRangeHTTPServer(range_limit=self.range_limit,
                                       protocol_version=self._protocol_version)
 
-    def get_transport(self):
-        return self._transport(self.get_readonly_server().get_url())
-
     def setUp(self):
         http_utils.TestCaseWithWebserver.setUp(self)
         # We need to manipulate ranges that correspond to real chunks in the
@@ -1095,14 +1165,14 @@
         self.build_tree_contents([('a', content)],)
 
     def test_few_ranges(self):
-        t = self.get_transport()
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 4), (1024, 4), )))
         self.assertEqual(l[0], (0, '0000'))
         self.assertEqual(l[1], (1024, '0001'))
         self.assertEqual(1, self.get_readonly_server().GET_request_nb)
 
     def test_more_ranges(self):
-        t = self.get_transport()
+        t = self.get_readonly_transport()
         l = list(t.readv('a', ((0, 4), (1024, 4), (4096, 4), (8192, 4))))
         self.assertEqual(l[0], (0, '0000'))
         self.assertEqual(l[1], (1024, '0001'))
@@ -1119,35 +1189,53 @@
     Only the urllib implementation is tested here.
     """
 
-    def setUp(self):
-        tests.TestCase.setUp(self)
-        self._old_env = {}
-        self.addCleanup(self._restore_env)
-
-    def _install_env(self, env):
-        for name, value in env.iteritems():
-            self._old_env[name] = osutils.set_or_unset_env(name, value)
-
-    def _restore_env(self):
-        for name, value in self._old_env.iteritems():
-            osutils.set_or_unset_env(name, value)
-
     def _proxied_request(self):
         handler = _urllib2_wrappers.ProxyHandler()
-        request = _urllib2_wrappers.Request('GET','http://baz/buzzle')
+        request = _urllib2_wrappers.Request('GET', 'http://baz/buzzle')
         handler.set_proxy(request, 'http')
         return request
 
+    def assertEvaluateProxyBypass(self, expected, host, no_proxy):
+        handler = _urllib2_wrappers.ProxyHandler()
+        self.assertEquals(expected,
+                          handler.evaluate_proxy_bypass(host, no_proxy))
+
     def test_empty_user(self):
-        self._install_env({'http_proxy': 'http://bar.com'})
+        self.overrideEnv('http_proxy', 'http://bar.com')
+        request = self._proxied_request()
+        self.assertFalse(request.headers.has_key('Proxy-authorization'))
+
+    def test_user_with_at(self):
+        self.overrideEnv('http_proxy',
+                         'http://username@domain:password@proxy_host:1234')
         request = self._proxied_request()
         self.assertFalse(request.headers.has_key('Proxy-authorization'))
 
     def test_invalid_proxy(self):
         """A proxy env variable without scheme"""
-        self._install_env({'http_proxy': 'host:1234'})
+        self.overrideEnv('http_proxy', 'host:1234')
         self.assertRaises(errors.InvalidURL, self._proxied_request)
 
+    def test_evaluate_proxy_bypass_true(self):
+        """The host is not proxied"""
+        self.assertEvaluateProxyBypass(True, 'example.com', 'example.com')
+        self.assertEvaluateProxyBypass(True, 'bzr.example.com', '*example.com')
+
+    def test_evaluate_proxy_bypass_false(self):
+        """The host is proxied"""
+        self.assertEvaluateProxyBypass(False, 'bzr.example.com', None)
+
+    def test_evaluate_proxy_bypass_unknown(self):
+        """The host is not explicitly proxied"""
+        self.assertEvaluateProxyBypass(None, 'example.com', 'not.example.com')
+        self.assertEvaluateProxyBypass(None, 'bzr.example.com', 'example.com')
+
+    def test_evaluate_proxy_bypass_empty_entries(self):
+        """Ignore empty entries"""
+        self.assertEvaluateProxyBypass(None, 'example.com', '')
+        self.assertEvaluateProxyBypass(None, 'example.com', ',')
+        self.assertEvaluateProxyBypass(None, 'example.com', 'foo,,bar')
+
 
 class TestProxyHttpServer(http_utils.TestCaseWithTwoWebservers):
     """Tests proxy server.
@@ -1158,66 +1246,47 @@
     to the file names).
     """
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     # FIXME: We don't have an https server available, so we don't
-    # test https connections.
+    # test https connections. --vila toolongago
 
     def setUp(self):
         super(TestProxyHttpServer, self).setUp()
+        self.transport_secondary_server = http_utils.ProxyServer
         self.build_tree_contents([('foo', 'contents of foo\n'),
                                   ('foo-proxied', 'proxied contents of foo\n')])
         # Let's setup some attributes for tests
-        self.server = self.get_readonly_server()
-        self.proxy_address = '%s:%d' % (self.server.host, self.server.port)
+        server = self.get_readonly_server()
+        self.server_host_port = '%s:%d' % (server.host, server.port)
         if self._testing_pycurl():
             # Oh my ! pycurl does not check for the port as part of
             # no_proxy :-( So we just test the host part
-            self.no_proxy_host = self.server.host
+            self.no_proxy_host = server.host
         else:
-            self.no_proxy_host = self.proxy_address
+            self.no_proxy_host = self.server_host_port
         # The secondary server is the proxy
-        self.proxy = self.get_secondary_server()
-        self.proxy_url = self.proxy.get_url()
-        self._old_env = {}
+        self.proxy_url = self.get_secondary_url()
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
         return (features.pycurl.available()
                 and self._transport == PyCurlTransport)
 
-    def create_transport_secondary_server(self):
-        """Creates an http server that will serve files with
-        '-proxied' appended to their names.
-        """
-        return http_utils.ProxyServer(protocol_version=self._protocol_version)
-
-    def _install_env(self, env):
-        for name, value in env.iteritems():
-            self._old_env[name] = osutils.set_or_unset_env(name, value)
-
-    def _restore_env(self):
-        for name, value in self._old_env.iteritems():
-            osutils.set_or_unset_env(name, value)
-
-    def proxied_in_env(self, env):
-        self._install_env(env)
-        url = self.server.get_url()
-        t = self._transport(url)
-        try:
-            self.assertEqual('proxied contents of foo\n', t.get('foo').read())
-        finally:
-            self._restore_env()
-
-    def not_proxied_in_env(self, env):
-        self._install_env(env)
-        url = self.server.get_url()
-        t = self._transport(url)
-        try:
-            self.assertEqual('contents of foo\n', t.get('foo').read())
-        finally:
-            self._restore_env()
+    def assertProxied(self):
+        t = self.get_readonly_transport()
+        self.assertEqual('proxied contents of foo\n', t.get('foo').read())
+
+    def assertNotProxied(self):
+        t = self.get_readonly_transport()
+        self.assertEqual('contents of foo\n', t.get('foo').read())
 
     def test_http_proxy(self):
-        self.proxied_in_env({'http_proxy': self.proxy_url})
+        self.overrideEnv('http_proxy', self.proxy_url)
+        self.assertProxied()
 
     def test_HTTP_PROXY(self):
         if self._testing_pycurl():
@@ -1226,69 +1295,80 @@
             # about. Should we ?)
             raise tests.TestNotApplicable(
                 'pycurl does not check HTTP_PROXY for security reasons')
-        self.proxied_in_env({'HTTP_PROXY': self.proxy_url})
+        self.overrideEnv('HTTP_PROXY', self.proxy_url)
+        self.assertProxied()
 
     def test_all_proxy(self):
-        self.proxied_in_env({'all_proxy': self.proxy_url})
+        self.overrideEnv('all_proxy', self.proxy_url)
+        self.assertProxied()
 
     def test_ALL_PROXY(self):
-        self.proxied_in_env({'ALL_PROXY': self.proxy_url})
+        self.overrideEnv('ALL_PROXY', self.proxy_url)
+        self.assertProxied()
 
     def test_http_proxy_with_no_proxy(self):
-        self.not_proxied_in_env({'http_proxy': self.proxy_url,
-                                 'no_proxy': self.no_proxy_host})
+        self.overrideEnv('no_proxy', self.no_proxy_host)
+        self.overrideEnv('http_proxy', self.proxy_url)
+        self.assertNotProxied()
 
     def test_HTTP_PROXY_with_NO_PROXY(self):
         if self._testing_pycurl():
             raise tests.TestNotApplicable(
                 'pycurl does not check HTTP_PROXY for security reasons')
-        self.not_proxied_in_env({'HTTP_PROXY': self.proxy_url,
-                                 'NO_PROXY': self.no_proxy_host})
+        self.overrideEnv('NO_PROXY', self.no_proxy_host)
+        self.overrideEnv('HTTP_PROXY', self.proxy_url)
+        self.assertNotProxied()
 
     def test_all_proxy_with_no_proxy(self):
-        self.not_proxied_in_env({'all_proxy': self.proxy_url,
-                                 'no_proxy': self.no_proxy_host})
+        self.overrideEnv('no_proxy', self.no_proxy_host)
+        self.overrideEnv('all_proxy', self.proxy_url)
+        self.assertNotProxied()
 
     def test_ALL_PROXY_with_NO_PROXY(self):
-        self.not_proxied_in_env({'ALL_PROXY': self.proxy_url,
-                                 'NO_PROXY': self.no_proxy_host})
+        self.overrideEnv('NO_PROXY', self.no_proxy_host)
+        self.overrideEnv('ALL_PROXY', self.proxy_url)
+        self.assertNotProxied()
 
     def test_http_proxy_without_scheme(self):
+        self.overrideEnv('http_proxy', self.server_host_port)
         if self._testing_pycurl():
             # pycurl *ignores* invalid proxy env variables. If that ever change
             # in the future, this test will fail indicating that pycurl do not
             # ignore anymore such variables.
-            self.not_proxied_in_env({'http_proxy': self.proxy_address})
+            self.assertNotProxied()
         else:
-            self.assertRaises(errors.InvalidURL,
-                              self.proxied_in_env,
-                              {'http_proxy': self.proxy_address})
+            self.assertRaises(errors.InvalidURL, self.assertProxied)
 
 
 class TestRanges(http_utils.TestCaseWithWebserver):
     """Test the Range header in GET methods."""
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     def setUp(self):
         http_utils.TestCaseWithWebserver.setUp(self)
         self.build_tree_contents([('a', '0123456789')],)
-        server = self.get_readonly_server()
-        self.transport = self._transport(server.get_url())
 
     def create_transport_readonly_server(self):
         return http_server.HttpServer(protocol_version=self._protocol_version)
 
     def _file_contents(self, relpath, ranges):
+        t = self.get_readonly_transport()
         offsets = [ (start, end - start + 1) for start, end in ranges]
-        coalesce = self.transport._coalesce_offsets
+        coalesce = t._coalesce_offsets
         coalesced = list(coalesce(offsets, limit=0, fudge_factor=0))
-        code, data = self.transport._get(relpath, coalesced)
+        code, data = t._get(relpath, coalesced)
         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
         for start, end in ranges:
             data.seek(start)
             yield data.read(end - start + 1)
 
     def _file_tail(self, relpath, tail_amount):
-        code, data = self.transport._get(relpath, [], tail_amount)
+        t = self.get_readonly_transport()
+        code, data = t._get(relpath, [], tail_amount)
         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
         data.seek(-tail_amount, 2)
         return data.read(tail_amount)
@@ -1313,14 +1393,10 @@
 class TestHTTPRedirections(http_utils.TestCaseWithRedirectedWebserver):
     """Test redirection between http servers."""
 
-    def create_transport_secondary_server(self):
-        """Create the secondary server redirecting to the primary server"""
-        new = self.get_readonly_server()
-
-        redirecting = http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-        redirecting.redirect_to(new.host, new.port)
-        return redirecting
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
 
     def setUp(self):
         super(TestHTTPRedirections, self).setUp()
@@ -1328,13 +1404,11 @@
                                   ('bundle',
                                   '# Bazaar revision bundle v0.9\n#\n')
                                   ],)
-        # The requests to the old server will be redirected to the new server
-        self.old_transport = self._transport(self.old_server.get_url())
 
     def test_redirected(self):
-        self.assertRaises(errors.RedirectRequested, self.old_transport.get, 'a')
-        t = self._transport(self.new_server.get_url())
-        self.assertEqual('0123456789', t.get('a').read())
+        self.assertRaises(errors.RedirectRequested,
+                          self.get_old_transport().get, 'a')
+        self.assertEqual('0123456789', self.get_new_transport().get('a').read())
 
 
 class RedirectedRequest(_urllib2_wrappers.Request):
@@ -1357,6 +1431,27 @@
     test.overrideAttr(_urllib2_wrappers, 'Request', RedirectedRequest)
 
 
+def cleanup_http_redirection_connections(test):
+    # Some sockets are opened but never seen by _urllib, so we trap them at
+    # the _urllib2_wrappers level to be able to clean them up.
+    def socket_disconnect(sock):
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+            sock.close()
+        except socket.error:
+            pass
+    def connect(connection):
+        test.http_connect_orig(connection)
+        test.addCleanup(socket_disconnect, connection.sock)
+    test.http_connect_orig = test.overrideAttr(
+        _urllib2_wrappers.HTTPConnection, 'connect', connect)
+    def connect(connection):
+        test.https_connect_orig(connection)
+        test.addCleanup(socket_disconnect, connection.sock)
+    test.https_connect_orig = test.overrideAttr(
+        _urllib2_wrappers.HTTPSConnection, 'connect', connect)
+
+
 class TestHTTPSilentRedirections(http_utils.TestCaseWithRedirectedWebserver):
     """Test redirections.
 
@@ -1371,13 +1466,19 @@
     -- vila 20070212
     """
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     def setUp(self):
         if (features.pycurl.available()
             and self._transport == PyCurlTransport):
             raise tests.TestNotApplicable(
-                "pycurl doesn't redirect silently annymore")
+                "pycurl doesn't redirect silently anymore")
         super(TestHTTPSilentRedirections, self).setUp()
         install_redirected_request(self)
+        cleanup_http_redirection_connections(self)
         self.build_tree_contents([('a','a'),
                                   ('1/',),
                                   ('1/a', 'redirected once'),
@@ -1391,27 +1492,18 @@
                                   ('5/a', 'redirected 5 times'),
                                   ],)
 
-        self.old_transport = self._transport(self.old_server.get_url())
-
-    def create_transport_secondary_server(self):
-        """Create the secondary server, redirections are defined in the tests"""
-        return http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-
     def test_one_redirection(self):
-        t = self.old_transport
-
-        req = RedirectedRequest('GET', t.abspath('a'))
+        t = self.get_old_transport()
+        req = RedirectedRequest('GET', t._remote_path('a'))
         new_prefix = 'http://%s:%s' % (self.new_server.host,
                                        self.new_server.port)
         self.old_server.redirections = \
             [('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.assertEqual('redirected once',t._perform(req).read())
+        self.assertEqual('redirected once', t._perform(req).read())
 
     def test_five_redirections(self):
-        t = self.old_transport
-
-        req = RedirectedRequest('GET', t.abspath('a'))
+        t = self.get_old_transport()
+        req = RedirectedRequest('GET', t._remote_path('a'))
         old_prefix = 'http://%s:%s' % (self.old_server.host,
                                        self.old_server.port)
         new_prefix = 'http://%s:%s' % (self.new_server.host,
@@ -1423,23 +1515,29 @@
             ('/4(.*)', r'%s/5\1' % (new_prefix), 301),
             ('(/[^/]+)', r'%s/1\1' % (old_prefix), 301),
             ]
-        self.assertEqual('redirected 5 times',t._perform(req).read())
+        self.assertEqual('redirected 5 times', t._perform(req).read())
 
 
 class TestDoCatchRedirections(http_utils.TestCaseWithRedirectedWebserver):
     """Test transport.do_catching_redirections."""
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     def setUp(self):
         super(TestDoCatchRedirections, self).setUp()
         self.build_tree_contents([('a', '0123456789'),],)
-
-        self.old_transport = self._transport(self.old_server.get_url())
-
-    def get_a(self, transport):
-        return transport.get('a')
+        cleanup_http_redirection_connections(self)
+
+        self.old_transport = self.get_old_transport()
+
+    def get_a(self, t):
+        return t.get('a')
 
     def test_no_redirection(self):
-        t = self._transport(self.new_server.get_url())
+        t = self.get_new_transport()
 
         # We use None for redirected so that we fail if redirected
         self.assertEqual('0123456789',
@@ -1449,10 +1547,10 @@
     def test_one_redirection(self):
         self.redirections = 0
 
-        def redirected(transport, exception, redirection_notice):
+        def redirected(t, exception, redirection_notice):
             self.redirections += 1
-            dir, file = urlutils.split(exception.target)
-            return self._transport(dir)
+            redirected_t = t._redirected_to(exception.source, exception.target)
+            return redirected_t
 
         self.assertEqual('0123456789',
                          transport.do_catching_redirections(
@@ -1472,14 +1570,43 @@
                           self.get_a, self.old_transport, redirected)
 
 
+def _setup_authentication_config(**kwargs):
+    conf = config.AuthenticationConfig()
+    conf._get_config().update({'httptest': kwargs})
+    conf._save()
+
+
+class TestUrllib2AuthHandler(tests.TestCaseWithTransport):
+    """Unit tests for glue by which urllib2 asks us for authentication"""
+
+    def test_get_user_password_without_port(self):
+        """We cope if urllib2 doesn't tell us the port.
+
+        See https://bugs.launchpad.net/bzr/+bug/654684
+        """
+        user = 'joe'
+        password = 'foo'
+        _setup_authentication_config(scheme='http', host='localhost',
+                                     user=user, password=password)
+        handler = _urllib2_wrappers.HTTPAuthHandler()
+        got_pass = handler.get_user_password(dict(
+            user='joe',
+            protocol='http',
+            host='localhost',
+            path='/',
+            realm='Realm',
+            ))
+        self.assertEquals((user, password), got_pass)
+
+
 class TestAuth(http_utils.TestCaseWithWebserver):
     """Test authentication scheme"""
 
-    _auth_header = 'Authorization'
-    _password_prompt_prefix = ''
-    _username_prompt_prefix = ''
-    # Set by load_tests
-    _auth_server = None
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        vary_by_http_auth_scheme(),
+        )
 
     def setUp(self):
         super(TestAuth, self).setUp()
@@ -1488,7 +1615,9 @@
                                   ('b', 'contents of b\n'),])
 
     def create_transport_readonly_server(self):
-        return self._auth_server(protocol_version=self._protocol_version)
+        server = self._auth_server(protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def _testing_pycurl(self):
         # TODO: This is duplicated for lots of the classes in this file
@@ -1507,7 +1636,9 @@
         return url
 
     def get_user_transport(self, user, password):
-        return self._transport(self.get_user_url(user, password))
+        t = transport.get_transport_from_url(
+            self.get_user_url(user, password))
+        return t
 
     def test_no_user(self):
         self.server.add_user('joe', 'foo')
@@ -1624,11 +1755,8 @@
         ui.ui_factory = tests.TestUIFactory(stdin=stdin_content,
                                             stderr=tests.StringIOWrapper())
         # Create a minimal config file with the right password
-        conf = config.AuthenticationConfig()
-        conf._get_config().update(
-            {'httptest': {'scheme': 'http', 'port': self.server.port,
-                          'user': user, 'password': password}})
-        conf._save()
+        _setup_authentication_config(scheme='http', port=self.server.port,
+                                     user=user, password=password)
         # Issue a request to the server to connect
         self.assertEqual('contents of a\n',t.get('a').read())
         # stdin should have  been left untouched
@@ -1636,31 +1764,12 @@
         # Only one 'Authentication Required' error should occur
         self.assertEqual(1, self.server.auth_required_errors)
 
-    def test_user_from_auth_conf(self):
-        if self._testing_pycurl():
-            raise tests.TestNotApplicable(
-                'pycurl does not support authentication.conf')
-        user = 'joe'
-        password = 'foo'
-        self.server.add_user(user, password)
-        # Create a minimal config file with the right password
-        conf = config.AuthenticationConfig()
-        conf._get_config().update(
-            {'httptest': {'scheme': 'http', 'port': self.server.port,
-                          'user': user, 'password': password}})
-        conf._save()
-        t = self.get_user_transport(None, None)
-        # Issue a request to the server to connect
-        self.assertEqual('contents of a\n', t.get('a').read())
-        # Only one 'Authentication Required' error should occur
-        self.assertEqual(1, self.server.auth_required_errors)
-
     def test_changing_nonce(self):
         if self._auth_server not in (http_utils.HTTPDigestAuthServer,
                                      http_utils.ProxyDigestAuthServer):
             raise tests.TestNotApplicable('HTTP/proxy auth digest only test')
         if self._testing_pycurl():
-            raise tests.KnownFailure(
+            self.knownFailure(
                 'pycurl does not handle a nonce change')
         self.server.add_user('joe', 'foo')
         t = self.get_user_transport('joe', 'foo')
@@ -1676,19 +1785,63 @@
         # initial 'who are you' and a second 'who are you' with the new nonce)
         self.assertEqual(2, self.server.auth_required_errors)
 
+    def test_user_from_auth_conf(self):
+        if self._testing_pycurl():
+            raise tests.TestNotApplicable(
+                'pycurl does not support authentication.conf')
+        user = 'joe'
+        password = 'foo'
+        self.server.add_user(user, password)
+        _setup_authentication_config(scheme='http', port=self.server.port,
+                                     user=user, password=password)
+        t = self.get_user_transport(None, None)
+        # Issue a request to the server to connect
+        self.assertEqual('contents of a\n', t.get('a').read())
+        # Only one 'Authentication Required' error should occur
+        self.assertEqual(1, self.server.auth_required_errors)
+
+    def test_no_credential_leaks_in_log(self):
+        self.overrideAttr(debug, 'debug_flags', set(['http']))
+        user = 'joe'
+        password = 'very-sensitive-password'
+        self.server.add_user(user, password)
+        t = self.get_user_transport(user, password)
+        # Capture the debug calls to mutter
+        self.mutters = []
+        def mutter(*args):
+            lines = args[0] % args[1:]
+            # Some calls output multiple lines, just split them now since we
+            # care about a single one later.
+            self.mutters.extend(lines.splitlines())
+        self.overrideAttr(trace, 'mutter', mutter)
+        # Issue a request to the server to connect
+        self.assertEqual(True, t.has('a'))
+        # Only one 'Authentication Required' error should occur
+        self.assertEqual(1, self.server.auth_required_errors)
+        # Since the authentification succeeded, there should be a corresponding
+        # debug line
+        sent_auth_headers = [line for line in self.mutters
+                             if line.startswith('> %s' % (self._auth_header,))]
+        self.assertLength(1, sent_auth_headers)
+        self.assertStartsWith(sent_auth_headers[0],
+                              '> %s: <masked>' % (self._auth_header,))
 
 
 class TestProxyAuth(TestAuth):
-    """Test proxy authentication schemes."""
-
-    _auth_header = 'Proxy-authorization'
-    _password_prompt_prefix = 'Proxy '
-    _username_prompt_prefix = 'Proxy '
+    """Test proxy authentication schemes.
+
+    This inherits from TestAuth to tweak the setUp and filter some failing
+    tests.
+    """
+
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        vary_by_http_proxy_auth_scheme(),
+        )
 
     def setUp(self):
         super(TestProxyAuth, self).setUp()
-        self._old_env = {}
-        self.addCleanup(self._restore_env)
         # Override the contents to avoid false positives
         self.build_tree_contents([('a', 'not proxied contents of a\n'),
                                   ('b', 'not proxied contents of b\n'),
@@ -1697,22 +1850,14 @@
                                   ])
 
     def get_user_transport(self, user, password):
-        self._install_env({'all_proxy': self.get_user_url(user, password)})
-        return self._transport(self.server.get_url())
-
-    def _install_env(self, env):
-        for name, value in env.iteritems():
-            self._old_env[name] = osutils.set_or_unset_env(name, value)
-
-    def _restore_env(self):
-        for name, value in self._old_env.iteritems():
-            osutils.set_or_unset_env(name, value)
+        self.overrideEnv('all_proxy', self.get_user_url(user, password))
+        return TestAuth.get_user_transport(self, user, password)
 
     def test_empty_pass(self):
         if self._testing_pycurl():
             import pycurl
             if pycurl.version_info()[1] < '7.16.0':
-                raise tests.KnownFailure(
+                self.knownFailure(
                     'pycurl < 7.16.0 does not handle empty proxy passwords')
         super(TestProxyAuth, self).test_empty_pass()
 
@@ -1731,6 +1876,7 @@
         self.readfile = StringIO(socket_read_content)
         self.writefile = StringIO()
         self.writefile.close = lambda: None
+        self.close = lambda: None
 
     def makefile(self, mode='r', bufsize=None):
         if 'r' in mode:
@@ -1741,21 +1887,29 @@
 
 class SmartHTTPTunnellingTest(tests.TestCaseWithTransport):
 
+    scenarios = multiply_scenarios(
+        vary_by_http_client_implementation(),
+        vary_by_http_protocol_version(),
+        )
+
     def setUp(self):
         super(SmartHTTPTunnellingTest, self).setUp()
         # We use the VFS layer as part of HTTP tunnelling tests.
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.transport_readonly_server = http_utils.HTTPServerWithSmarts
+        self.http_server = self.get_readonly_server()
 
     def create_transport_readonly_server(self):
-        return http_utils.HTTPServerWithSmarts(
+        server = http_utils.HTTPServerWithSmarts(
             protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
 
     def test_open_bzrdir(self):
         branch = self.make_branch('relpath')
-        http_server = self.get_readonly_server()
-        url = http_server.get_url() + 'relpath'
+        url = self.http_server.get_url() + 'relpath'
         bd = bzrdir.BzrDir.open(url)
+        self.addCleanup(bd.transport.disconnect)
         self.assertIsInstance(bd, _mod_remote.RemoteBzrDir)
 
     def test_bulk_data(self):
@@ -1763,8 +1917,8 @@
         # The 'readv' command in the smart protocol both sends and receives
         # bulk data, so we use that.
         self.build_tree(['data-file'])
-        http_server = self.get_readonly_server()
-        http_transport = self._transport(http_server.get_url())
+        http_transport = transport.get_transport_from_url(
+            self.http_server.get_url())
         medium = http_transport.get_smart_medium()
         # Since we provide the medium, the url below will be mostly ignored
         # during the test, as long as the path is '/'.
@@ -1778,15 +1932,15 @@
         post_body = 'hello\n'
         expected_reply_body = 'ok\x012\n'
 
-        http_server = self.get_readonly_server()
-        http_transport = self._transport(http_server.get_url())
+        http_transport = transport.get_transport_from_url(
+            self.http_server.get_url())
         medium = http_transport.get_smart_medium()
         response = medium.send_http_smart_request(post_body)
         reply_body = response.read()
         self.assertEqual(expected_reply_body, reply_body)
 
     def test_smart_http_server_post_request_handler(self):
-        httpd = self.get_readonly_server()._get_httpd()
+        httpd = self.http_server.server
 
         socket = SampleSocket(
             'POST /.bzr/smart %s \r\n' % self._protocol_version
@@ -1824,16 +1978,18 @@
 
     def test_probe_smart_server(self):
         """Test error handling against server refusing smart requests."""
-        server = self.get_readonly_server()
-        t = self._transport(server.get_url())
+        t = self.get_readonly_transport()
         # No need to build a valid smart request here, the server will not even
         # try to interpret it.
         self.assertRaises(errors.SmartProtocolError,
                           t.get_smart_medium().send_http_smart_request,
                           'whatever')
 
+
 class Test_redirected_to(tests.TestCase):
 
+    scenarios = vary_by_http_client_implementation()
+
     def test_redirected_to_subdir(self):
         t = self._transport('http://www.example.com/foo')
         r = t._redirected_to('http://www.example.com/foo',
@@ -1841,6 +1997,7 @@
         self.assertIsInstance(r, type(t))
         # Both transports share the some connection
         self.assertEqual(t._get_connection(), r._get_connection())
+        self.assertEquals('http://www.example.com/foo/subdir/', r.base)
 
     def test_redirected_to_self_with_slash(self):
         t = self._transport('http://www.example.com/foo')
@@ -1857,25 +2014,37 @@
         r = t._redirected_to('http://www.example.com/foo',
                              'http://foo.example.com/foo/subdir')
         self.assertIsInstance(r, type(t))
+        self.assertEquals('http://foo.example.com/foo/subdir/',
+            r.external_url())
 
     def test_redirected_to_same_host_sibling_protocol(self):
         t = self._transport('http://www.example.com/foo')
         r = t._redirected_to('http://www.example.com/foo',
                              'https://www.example.com/foo')
         self.assertIsInstance(r, type(t))
+        self.assertEquals('https://www.example.com/foo/',
+            r.external_url())
 
     def test_redirected_to_same_host_different_protocol(self):
         t = self._transport('http://www.example.com/foo')
         r = t._redirected_to('http://www.example.com/foo',
                              'ftp://www.example.com/foo')
         self.assertNotEquals(type(r), type(t))
+        self.assertEquals('ftp://www.example.com/foo/', r.external_url())
+
+    def test_redirected_to_same_host_specific_implementation(self):
+        t = self._transport('http://www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'https+urllib://www.example.com/foo')
+        self.assertEquals('https://www.example.com/foo/', r.external_url())
 
     def test_redirected_to_different_host_same_user(self):
         t = self._transport('http://joe@www.example.com/foo')
         r = t._redirected_to('http://www.example.com/foo',
                              'https://foo.example.com/foo')
         self.assertIsInstance(r, type(t))
-        self.assertEqual(t._user, r._user)
+        self.assertEqual(t._parsed_url.user, r._parsed_url.user)
+        self.assertEquals('https://joe@foo.example.com/foo/', r.external_url())
 
 
 class PredefinedRequestHandler(http_server.TestingHTTPRequestHandler):
@@ -1890,7 +2059,7 @@
     line.
     """
 
-    def handle_one_request(self):
+    def _handle_one_request(self):
         tcs = self.server.test_case_server
         requestline = self.rfile.readline()
         headers = self.MessageClass(self.rfile, 0)
@@ -1934,7 +2103,7 @@
     pass
 
 
-if tests.HTTPSServerFeature.available():
+if features.HTTPSServerFeature.available():
     from bzrlib.tests import https_server
     class ActivityHTTPSServer(ActivityServerMixin, https_server.HTTPSServer):
         pass
@@ -1951,11 +2120,12 @@
         tests.TestCase.setUp(self)
         self.server = self._activity_server(self._protocol_version)
         self.server.start_server()
-        self.activities = {}
+        _activities = {} # Don't close over self and create a cycle
         def report_activity(t, bytes, direction):
-            count = self.activities.get(direction, 0)
+            count = _activities.get(direction, 0)
             count += bytes
-            self.activities[direction] = count
+            _activities[direction] = count
+        self.activities = _activities
 
         # We override at class level because constructors may propagate the
         # bound method and render instance overriding ineffective (an
@@ -1964,7 +2134,9 @@
         self.addCleanup(self.server.stop_server)
 
     def get_transport(self):
-        return self._transport(self.server.get_url())
+        t = self._transport(self.server.get_url())
+        # FIXME: Needs cleanup -- vila 20100611
+        return t
 
     def assertActivitiesMatch(self):
         self.assertEqual(self.server.bytes_read,
@@ -2075,7 +2247,7 @@
 '''
         t = self.get_transport()
         # We must send a single line of body bytes, see
-        # PredefinedRequestHandler.handle_one_request
+        # PredefinedRequestHandler._handle_one_request
         code, f = t._post('abc def end-of-body\n')
         self.assertEqual('lalala whatever as long as itsssss\n', f.read())
         self.assertActivitiesMatch()
@@ -2083,6 +2255,11 @@
 
 class TestActivity(tests.TestCase, TestActivityMixin):
 
+    scenarios = multiply_scenarios(
+        vary_by_http_activity(),
+        vary_by_http_protocol_version(),
+        )
+
     def setUp(self):
         TestActivityMixin.setUp(self)
 
@@ -2109,24 +2286,14 @@
 class TestAuthOnRedirected(http_utils.TestCaseWithRedirectedWebserver):
     """Test authentication on the redirected http server."""
 
+    scenarios = vary_by_http_protocol_version()
+
     _auth_header = 'Authorization'
     _password_prompt_prefix = ''
     _username_prompt_prefix = ''
     _auth_server = http_utils.HTTPBasicAuthServer
     _transport = _urllib.HttpTransport_urllib
 
-    def create_transport_readonly_server(self):
-        return self._auth_server(protocol_version=self._protocol_version)
-
-    def create_transport_secondary_server(self):
-        """Create the secondary server redirecting to the primary server"""
-        new = self.get_readonly_server()
-
-        redirecting = http_utils.HTTPServerRedirecting(
-            protocol_version=self._protocol_version)
-        redirecting.redirect_to(new.host, new.port)
-        return redirecting
-
     def setUp(self):
         super(TestAuthOnRedirected, self).setUp()
         self.build_tree_contents([('a','a'),
@@ -2137,19 +2304,26 @@
                                        self.new_server.port)
         self.old_server.redirections = [
             ('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.old_transport = self._transport(self.old_server.get_url())
+        self.old_transport = self.get_old_transport()
         self.new_server.add_user('joe', 'foo')
-
-    def get_a(self, transport):
-        return transport.get('a')
+        cleanup_http_redirection_connections(self)
+
+    def create_transport_readonly_server(self):
+        server = self._auth_server(protocol_version=self._protocol_version)
+        server._url_protocol = self._url_protocol
+        return server
+
+    def get_a(self, t):
+        return t.get('a')
 
     def test_auth_on_redirected_via_do_catching_redirections(self):
         self.redirections = 0
 
-        def redirected(transport, exception, redirection_notice):
+        def redirected(t, exception, redirection_notice):
             self.redirections += 1
-            dir, file = urlutils.split(exception.target)
-            return self._transport(dir)
+            redirected_t = t._redirected_to(exception.source, exception.target)
+            self.addCleanup(redirected_t.disconnect)
+            return redirected_t
 
         stdout = tests.StringIOWrapper()
         stderr = tests.StringIOWrapper()
@@ -2176,10 +2350,9 @@
                                        self.new_server.port)
         self.old_server.redirections = [
             ('(.*)', r'%s/1\1' % (new_prefix), 301),]
-        self.assertEqual('redirected once',t._perform(req).read())
+        self.assertEqual('redirected once', t._perform(req).read())
         # stdin should be empty
         self.assertEqual('', ui.ui_factory.stdin.readline())
         # stdout should be empty, stderr will contains the prompts
         self.assertEqual('', stdout.getvalue())
 
-

=== modified file 'bzrlib/tests/test_https_ca_bundle.py'
--- bzrlib/tests/test_https_ca_bundle.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_https_ca_bundle.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 import os
 import sys
 
-import bzrlib
-from bzrlib import osutils
 from bzrlib.tests import (
     TestCaseInTempDir,
     TestSkipped,
@@ -31,19 +29,9 @@
 class TestGetCAPath(TestCaseInTempDir):
 
     def setUp(self):
-        TestCaseInTempDir.setUp(self)
-        new_env = {
-            'CURL_CA_BUNDLE': None,
-            'PATH': None,
-            }
-        self._old_env = {}
-        self.addCleanup(self._restore)
-        for name, value in new_env.iteritems():
-            self._old_env[name] = osutils.set_or_unset_env(name, None)
-
-    def _restore(self):
-        for name, value in self._old_env.iteritems():
-            osutils.set_or_unset_env(name, value)
+        super(TestGetCAPath, self).setUp()
+        self.overrideEnv('CURL_CA_BUNDLE', None)
+        self.overrideEnv('PATH', None)
 
     def _make_file(self, in_dir='.'):
         fname = os.path.join(in_dir, 'curl-ca-bundle.crt')
@@ -55,7 +43,7 @@
         self.assertEqual('', ca_bundle.get_ca_path(use_cache=False))
 
     def test_env_var(self):
-        osutils.set_or_unset_env('CURL_CA_BUNDLE', 'foo.bar')
+        self.overrideEnv('CURL_CA_BUNDLE', 'foo.bar')
         self._make_file()
         self.assertEqual('foo.bar', ca_bundle.get_ca_path(use_cache=False))
 
@@ -65,6 +53,6 @@
         os.mkdir('foo')
         in_dir = os.path.join(os.getcwd(), 'foo')
         self._make_file(in_dir=in_dir)
-        osutils.set_or_unset_env('PATH', in_dir)
+        self.overrideEnv('PATH', in_dir)
         shouldbe = os.path.join(in_dir, 'curl-ca-bundle.crt')
         self.assertEqual(shouldbe, ca_bundle.get_ca_path(use_cache=False))

=== added file 'bzrlib/tests/test_i18n.py'
--- bzrlib/tests/test_i18n.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_i18n.py	2011-10-04 22:20:49 +0000
@@ -0,0 +1,162 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrlib.i18n"""
+
+from bzrlib import (
+    i18n,
+    tests,
+    errors,
+    workingtree,
+    )
+
+
+class ZzzTranslations(object):
+    """Special Zzz translation for debugging i18n stuff.
+
+    This class can be used to confirm that the message is properly translated
+    during black box tests.
+    """
+    _null_translation = i18n._gettext.NullTranslations()
+
+    def zzz(self, s):
+        return u'zz\xe5{{%s}}' % s
+
+    def ugettext(self, s):
+        return self.zzz(self._null_translation.ugettext(s))
+
+    def ungettext(self, s, p, n):
+        return self.zzz(self._null_translation.ungettext(s, p, n))
+
+
+class TestZzzTranslation(tests.TestCase):
+
+    def _check_exact(self, expected, source):
+        self.assertEqual(expected, source)
+        self.assertEqual(type(expected), type(source))
+
+    def test_translation(self):
+        trans = ZzzTranslations()
+
+        t = trans.zzz('msg')
+        self._check_exact(u'zz\xe5{{msg}}', t)
+
+        t = trans.ugettext('msg')
+        self._check_exact(u'zz\xe5{{msg}}', t)
+
+        t = trans.ungettext('msg1', 'msg2', 0)
+        self._check_exact(u'zz\xe5{{msg2}}', t)
+        t = trans.ungettext('msg1', 'msg2', 2)
+        self._check_exact(u'zz\xe5{{msg2}}', t)
+
+        t = trans.ungettext('msg1', 'msg2', 1)
+        self._check_exact(u'zz\xe5{{msg1}}', t)
+
+
+class TestGetText(tests.TestCase):
+
+    def setUp(self):
+        super(TestGetText, self).setUp()
+        self.overrideAttr(i18n, '_translations', ZzzTranslations())
+
+    def test_oneline(self):
+        self.assertEqual(u"zz\xe5{{spam ham eggs}}",
+                         i18n.gettext("spam ham eggs"))
+
+    def test_multiline(self):
+        self.assertEqual(u"zz\xe5{{spam\nham\n\neggs\n}}",
+                         i18n.gettext("spam\nham\n\neggs\n"))
+
+
+class TestGetTextPerParagraph(tests.TestCase):
+
+    def setUp(self):
+        super(TestGetTextPerParagraph, self).setUp()
+        self.overrideAttr(i18n, '_translations', ZzzTranslations())
+
+    def test_oneline(self):
+        self.assertEqual(u"zz\xe5{{spam ham eggs}}",
+                         i18n.gettext_per_paragraph("spam ham eggs"))
+
+    def test_multiline(self):
+        self.assertEqual(u"zz\xe5{{spam\nham}}\n\nzz\xe5{{eggs\n}}",
+                         i18n.gettext_per_paragraph("spam\nham\n\neggs\n"))
+
+
+class TestInstall(tests.TestCase):
+
+    def setUp(self):
+        super(TestInstall, self).setUp()
+        # Restore a proper env to test translation installation
+        self.overrideAttr(i18n, '_translations', None)
+
+    def test_custom_languages(self):
+        i18n.install('nl:fy')
+        # Whether we found a valid tranlsation or not doesn't matter, we got
+        # one and _translations is not None anymore.
+        self.assertIsInstance(i18n._translations,
+                              i18n._gettext.NullTranslations)
+
+    def test_no_env_variables(self):
+        self.overrideEnv('LANGUAGE', None)
+        self.overrideEnv('LC_ALL', None)
+        self.overrideEnv('LC_MESSAGES', None)
+        self.overrideEnv('LANG', None)
+        i18n.install()
+        # Whether we found a valid tranlsation or not doesn't matter, we got
+        # one and _translations is not None anymore.
+        self.assertIsInstance(i18n._translations,
+                              i18n._gettext.NullTranslations)
+
+    def test_disable_i18n(self):
+        i18n.disable_i18n()
+        i18n.install()
+        # It's disabled, you can't install anything and we fallback to null
+        self.assertIsInstance(i18n._translations,
+                              i18n._gettext.NullTranslations)
+
+
+class TestTranslate(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestTranslate, self).setUp()
+        self.overrideAttr(i18n, '_translations', ZzzTranslations())
+
+    def test_error_message_translation(self):
+        """do errors get translated?"""
+        err = None
+        tree = self.make_branch_and_tree('.')
+        try:
+            workingtree.WorkingTree.open('./foo')
+        except errors.NotBranchError,e:
+            err = str(e)
+        self.assertContainsRe(err, 
+                              u"zz\xe5{{Not a branch: .*}}".encode("utf-8"))
+
+    def test_topic_help_translation(self):
+        """does topic help get translated?"""
+        from bzrlib import help
+        from StringIO import StringIO
+        out = StringIO()
+        help.help("authentication", out)
+        self.assertContainsRe(out.getvalue(), "zz\xe5{{Authentication Settings")
+
+
+class LoadPluginTranslations(tests.TestCase):
+
+    def test_does_not_exist(self):
+        translation = i18n.load_plugin_translations("doesnotexist")
+        self.assertEquals("foo", translation.gettext("foo"))

=== modified file 'bzrlib/tests/test_ignores.py'
--- bzrlib/tests/test_ignores.py	2010-05-03 04:51:58 +0000
+++ bzrlib/tests/test_ignores.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,15 @@
 
 from cStringIO import StringIO
 
-from bzrlib import config, errors, ignores
-from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
+from bzrlib import (
+    config,
+    ignores,
+    )
+from bzrlib.tests import (
+    TestCase,
+    TestCaseInTempDir,
+    TestCaseWithTransport,
+    )
 
 
 class TestParseIgnoreFile(TestCase):
@@ -69,11 +76,11 @@
     def test_create_if_missing(self):
         # $HOME should be set to '.'
         ignore_path = config.user_ignore_config_filename()
-        self.failIfExists(ignore_path)
+        self.assertPathDoesNotExist(ignore_path)
         user_ignores = ignores.get_user_ignores()
         self.assertEqual(set(ignores.USER_DEFAULTS), user_ignores)
 
-        self.failUnlessExists(ignore_path)
+        self.assertPathExists(ignore_path)
         f = open(ignore_path, 'rb')
         try:
             entries = ignores.parse_ignore_file(f)

=== modified file 'bzrlib/tests/test_import_tariff.py'
--- bzrlib/tests/test_import_tariff.py	2010-06-17 16:54:26 +0000
+++ bzrlib/tests/test_import_tariff.py	2011-08-30 09:30:27 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,8 +17,18 @@
 
 """Tests for how many modules are loaded in executing various commands."""
 
+import os
+
 from testtools import content
 
+from bzrlib import (
+    plugins as _mod_plugins,
+    trace,
+    )
+from bzrlib.bzrdir import BzrDir
+from bzrlib.smart import medium
+from bzrlib.transport import remote
+
 from bzrlib.plugin import (
     are_plugins_disabled,
     )
@@ -27,42 +37,71 @@
     TestCaseWithTransport,
     )
 
-
-class TestImportTariffs(TestCaseWithTransport):
-
+old_format_modules = [
+    'bzrlib.repofmt.knitrepo',
+    'bzrlib.repofmt.knitpack_repo',
+    'bzrlib.plugins.weave_fmt.branch',
+    'bzrlib.plugins.weave_fmt.bzrdir',
+    'bzrlib.plugins.weave_fmt.repository',
+    'bzrlib.plugins.weave_fmt.workingtree',
+    'bzrlib.weave',
+    'bzrlib.weavefile',
+    'bzrlib.xml4',
+    'bzrlib.xml5',
+    'bzrlib.xml6',
+    'bzrlib.xml7',
+    ]
+
+
+class ImportTariffTestCase(TestCaseWithTransport):
     """Check how many modules are loaded for some representative scenarios.
 
     See the Testing Guide in the developer documentation for more explanation.
+
+
+    We must respect the setup used by the selftest command regarding
+    plugins. This allows the user to control which plugins are in effect while
+    running these tests and respect the import policies defined here.
+
+    When failures are encountered for a given plugin, they can generally be
+    addressed by using lazy import or lazy hook registration.
     """
 
-    def run_command_check_imports(self, args, forbidden_imports):
-        """Run bzr ARGS in a subprocess and check its imports.
+    def setUp(self):
+        self.preserved_env_vars = {}
+        for name in ('BZR_PLUGIN_PATH', 'BZR_DISABLE_PLUGINS', 'BZR_PLUGINS_AT'
+                     ):
+            self.preserved_env_vars[name] = os.environ.get(name)
+        super(ImportTariffTestCase, self).setUp()
+
+    def start_bzr_subprocess_with_import_check(self, args, stderr_file=None):
+        """Run a bzr process and capture the imports.
 
         This is fairly expensive because we start a subprocess, so we aim to
         cover representative rather than exhaustive cases.
-
-        :param forbidden_imports: List of fully-qualified Python module names
-            that should not be loaded while running this command.
         """
-        # We use PYTHON_VERBOSE rather than --profile-importts because in
+        # We use PYTHON_VERBOSE rather than --profile-imports because in
         # experimentation the profile-imports output seems to not always show
         # the modules you'd expect; this can be debugged but python -v seems
         # more likely to always show everything.  And we use the environment
         # variable rather than 'python -v' in the hope it will work even if
         # bzr is frozen and python is not explicitly specified. -- mbp 20100208
-        #
-        # Normally we want test isolation from the real $HOME but here we
-        # explicitly do want to test against things installed there, therefore
-        # we pass it through.
-        env_changes = dict(PYTHONVERBOSE='1')
-        for name in ['BZR_HOME', 'BZR_PLUGIN_PATH',
-                     'BZR_DISABLE_PLUGINS', 'BZR_PLUGINS_AT',
-                     'HOME',]:
-            env_changes[name] = self._old_env.get(name)
-        out, err = self.run_bzr_subprocess(args,
-            allow_plugins=(not are_plugins_disabled()),
-            env_changes=env_changes)
-
+        env_changes = dict(PYTHONVERBOSE='1', **self.preserved_env_vars)
+        trace.mutter('Setting env for bzr subprocess: %r', env_changes)
+        kwargs = dict(env_changes=env_changes,
+                      allow_plugins=(not are_plugins_disabled()))
+        if stderr_file:
+            # We don't want to update the whole call chain so we insert stderr
+            # *iff* we need to
+            kwargs['stderr'] = stderr_file
+        return self.start_bzr_subprocess(args, **kwargs)
+
+    def check_forbidden_modules(self, err, forbidden_imports):
+        """Check for forbidden modules in stderr.
+
+        :param err: Standard error
+        :param forbidden_imports: List of forbidden modules
+        """
         self.addDetail('subprocess_stderr',
             content.Content(content.ContentType("text", "plain"),
                 lambda:[err]))
@@ -73,10 +112,39 @@
                 bad_modules.append(module_name)
 
         if bad_modules:
-            self.fail("command %r loaded forbidden modules %r"
-                % (args, bad_modules))
+            self.fail("command loaded forbidden modules %r"
+                % (bad_modules,))
+
+    def finish_bzr_subprocess_with_import_check(self, process,
+            args, forbidden_imports):
+        """Finish subprocess and check specific modules have not been
+        imported.
+
+        :param forbidden_imports: List of fully-qualified Python module names
+            that should not be loaded while running this command.
+        """
+        (out, err) = self.finish_bzr_subprocess(process,
+            universal_newlines=False, process_args=args)
+        self.check_forbidden_modules(err, forbidden_imports)
         return out, err
 
+    def run_command_check_imports(self, args, forbidden_imports):
+        """Run bzr ARGS in a subprocess and check its imports.
+
+        This is fairly expensive because we start a subprocess, so we aim to
+        cover representative rather than exhaustive cases.
+
+        :param forbidden_imports: List of fully-qualified Python module names
+            that should not be loaded while running this command.
+        """
+        process = self.start_bzr_subprocess_with_import_check(args)
+        self.finish_bzr_subprocess_with_import_check(process, args,
+            forbidden_imports)
+
+
+class TestImportTariffs(ImportTariffTestCase):
+    """Basic import tariff tests for some common bzr commands"""
+
     def test_import_tariffs_working(self):
         # check some guaranteed-true and false imports to be sure we're
         # measuring correctly
@@ -89,23 +157,105 @@
             ['bzrlib.tree'])
 
     def test_simple_local(self):
-        # 'st' in a working tree shouldn't need many modules
+        # 'st' in a default format working tree shouldn't need many modules
         self.make_branch_and_tree('.')
         self.run_command_check_imports(['st'], [
+            'bzrlib.annotate',
+            'bzrlib.atomicfile',
+            'bzrlib.bugtracker',
             'bzrlib.bundle.commands',
             'bzrlib.cmd_version_info',
-            'bzrlib.foreign',
+            'bzrlib.externalcommand',
+            'bzrlib.filters',
+            'bzrlib.hashcache',
+            # foreign branch plugins import the foreign_vcs_registry from 
+            # bzrlib.foreign so it can't be blacklisted
+            'bzrlib.gpg',
+            'bzrlib.info',
+            'bzrlib.knit',
             'bzrlib.merge3',
+            'bzrlib.merge_directive',
+            'bzrlib.msgeditor',
             'bzrlib.patiencediff',
             'bzrlib.remote',
+            'bzrlib.rules',
             'bzrlib.sign_my_commits',
             'bzrlib.smart',
+            'bzrlib.smart.client',
+            'bzrlib.smart.medium',
+            'bzrlib.smart.server',
             'bzrlib.transform',
+            'bzrlib.version_info_formats.format_rio',
+            'getpass',
             'kerberos',
             'smtplib',
             'tarfile',
-            ])
+            'tempfile',
+            ] + old_format_modules)
         # TODO: similar test for repository-only operations, checking we avoid
         # loading wt-specific stuff
         #
         # See https://bugs.launchpad.net/bzr/+bug/553017
+
+    def test_help_commands(self):
+        # See https://bugs.launchpad.net/bzr/+bug/663773
+        self.run_command_check_imports(['help', 'commands'], [
+            'testtools',
+            ])
+
+    def test_simple_serve(self):
+        # 'serve' in a default format working tree shouldn't need many modules
+        tree = self.make_branch_and_tree('.')
+        # Capture the bzr serve process' stderr in a file to avoid deadlocks
+        # while the smart client interacts with it.
+        stderr_file = open('bzr-serve.stderr', 'w')
+        process = self.start_bzr_subprocess_with_import_check(['serve',
+            '--inet', '-d', tree.basedir], stderr_file=stderr_file)
+        url = 'bzr://localhost/'
+        self.permit_url(url)
+        client_medium = medium.SmartSimplePipesClientMedium(
+            process.stdout, process.stdin, url)
+        transport = remote.RemoteTransport(url, medium=client_medium)
+        branch = BzrDir.open_from_transport(transport).open_branch()
+        process.stdin.close()
+        # Hide stdin from the subprocess module, so it won't fail to close it.
+        process.stdin = None
+        (out, err) = self.finish_bzr_subprocess(process,
+            universal_newlines=False)
+        stderr_file.close()
+        with open('bzr-serve.stderr', 'r') as stderr_file:
+            err = stderr_file.read()
+        self.check_forbidden_modules(err,
+            ['bzrlib.annotate',
+            'bzrlib.atomicfile',
+            'bzrlib.bugtracker',
+            'bzrlib.bundle.commands',
+            'bzrlib.cmd_version_info',
+            'bzrlib.dirstate',
+            'bzrlib._dirstate_helpers_py',
+            'bzrlib._dirstate_helpers_pyx',
+            'bzrlib.externalcommand',
+            'bzrlib.filters',
+            'bzrlib.hashcache',
+            # foreign branch plugins import the foreign_vcs_registry from 
+            # bzrlib.foreign so it can't be blacklisted
+            'bzrlib.gpg',
+            'bzrlib.info',
+            'bzrlib.knit',
+            'bzrlib.merge3',
+            'bzrlib.merge_directive',
+            'bzrlib.msgeditor',
+            'bzrlib.patiencediff',
+            'bzrlib.remote',
+            'bzrlib.rules',
+            'bzrlib.sign_my_commits',
+            'bzrlib.smart.client',
+            'bzrlib.transform',
+            'bzrlib.version_info_formats.format_rio',
+            'bzrlib.workingtree_4',
+            'getpass',
+            'kerberos',
+            'smtplib',
+            'tarfile',
+            'tempfile',
+            ] + old_format_modules)

=== modified file 'bzrlib/tests/test_index.py'
--- bzrlib/tests/test_index.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_index.py	2011-08-19 22:34:02 +0000
@@ -389,7 +389,7 @@
         for key, value, references in nodes:
             builder.add_node(key, value, references)
         stream = builder.finish()
-        trans = transport.get_transport('trace+' + self.get_url())
+        trans = transport.get_transport_from_url('trace+' + self.get_url())
         size = trans.put_file('index', stream)
         return index.GraphIndex(trans, 'index', size)
 
@@ -1056,7 +1056,7 @@
     def test_supports_unlimited_cache(self):
         builder = index.GraphIndexBuilder(0, key_elements=1)
         stream = builder.finish()
-        trans = transport.get_transport(self.get_url())
+        trans = self.get_transport()
         size = trans.put_file('index', stream)
         # It doesn't matter what unlimited_cache does here, just that it can be
         # passed

=== modified file 'bzrlib/tests/test_info.py'
--- bzrlib/tests/test_info.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_info.py	2011-03-11 21:06:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import sys
-from urllib import quote
 
 from bzrlib import (
     branch as _mod_branch,
@@ -149,7 +148,10 @@
             if key in ('pack-0.92',):
                 expected = 'pack-0.92'
             elif key in ('knit', 'metaweave'):
-                expected = 'knit or metaweave'
+                if 'metaweave' in bzrdir.format_registry:
+                    expected = 'knit or metaweave'
+                else:
+                    expected = 'knit'
             elif key in ('1.14', '1.14-rich-root'):
                 expected = '1.14 or 1.14-rich-root'
             self.assertCheckoutDescription(key, expected)
@@ -184,7 +186,7 @@
                 expected = '1.14-rich-root'
             self.assertRepoDescription(key, expected)
 
-        format = bzrdir.format_registry.make_bzrdir('metaweave')
+        format = bzrdir.format_registry.make_bzrdir('knit')
         format.set_branch_format(_mod_branch.BzrBranchFormat6())
         tree = self.make_branch_and_tree('unknown', format=format)
         self.assertEqual('unnamed', info.describe_format(tree.bzrdir,

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2009-09-24 19:26:45 +0000
+++ bzrlib/tests/test_inv.py	2011-08-12 09:49:24 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008, 2009 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,30 +18,35 @@
 from bzrlib import (
     chk_map,
     groupcompress,
-    bzrdir,
     errors,
     inventory,
     osutils,
     repository,
     revision,
     tests,
-    )
-from bzrlib.inventory import (CHKInventory, Inventory, ROOT_ID, InventoryFile,
-    InventoryDirectory, InventoryEntry, TreeReference)
+    workingtree,
+    )
+from bzrlib.inventory import (
+    CHKInventory,
+    Inventory,
+    ROOT_ID,
+    InventoryFile,
+    InventoryDirectory,
+    InventoryEntry,
+    TreeReference,
+    mutable_inventory_from_tree,
+    )
 from bzrlib.tests import (
     TestCase,
     TestCaseWithTransport,
-    condition_isinstance,
-    multiply_tests,
-    split_suite_by_condition,
     )
-from bzrlib.tests.per_workingtree import workingtree_formats
-
-
-def load_tests(standard_tests, module, loader):
-    """Parameterise some inventory tests."""
-    to_adapt, result = split_suite_by_condition(standard_tests,
-        condition_isinstance(TestDeltaApplication))
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+def delta_application_scenarios():
     scenarios = [
         ('Inventory', {'apply_delta':apply_inventory_Inventory}),
         ]
@@ -52,10 +57,14 @@
     # just creating trees.
     formats = set()
     for _, format in repository.format_registry.iteritems():
-        scenarios.append((str(format.__name__), {
-            'apply_delta':apply_inventory_Repository_add_inventory_by_delta,
-            'format':format}))
-    for format in workingtree_formats():
+        if format.supports_full_versioned_files:
+            scenarios.append((str(format.__name__), {
+                'apply_delta':apply_inventory_Repository_add_inventory_by_delta,
+                'format':format}))
+    for format in workingtree.format_registry._get_all():
+        repo_fmt = format._matchingbzrdir.repository_format
+        if not repo_fmt.supports_full_versioned_files:
+            continue
         scenarios.append(
             (str(format.__class__.__name__) + ".update_basis_by_delta", {
             'apply_delta':apply_inventory_WT_basis,
@@ -64,7 +73,7 @@
             (str(format.__class__.__name__) + ".apply_inventory_delta", {
             'apply_delta':apply_inventory_WT,
             'format':format}))
-    return multiply_tests(to_adapt, scenarios, result)
+    return scenarios
 
 
 def create_texts_for_inv(repo, inv):
@@ -74,10 +83,11 @@
         else:
             lines = []
         repo.texts.add_lines((ie.file_id, ie.revision), [], lines)
-    
-def apply_inventory_Inventory(self, basis, delta):
+
+
+def apply_inventory_Inventory(self, basis, delta, invalid_delta=True):
     """Apply delta to basis and return the result.
-    
+
     :param basis: An inventory to be used as the basis.
     :param delta: The inventory delta to apply:
     :return: An inventory resulting from the application.
@@ -86,11 +96,11 @@
     return basis
 
 
-def apply_inventory_WT(self, basis, delta):
+def apply_inventory_WT(self, basis, delta, invalid_delta=True):
     """Apply delta to basis and return the result.
 
     This sets the tree state to be basis, and then calls apply_inventory_delta.
-    
+
     :param basis: An inventory to be used as the basis.
     :param delta: The inventory delta to apply:
     :return: An inventory resulting from the application.
@@ -115,14 +125,65 @@
     tree = tree.bzrdir.open_workingtree()
     tree.lock_read()
     self.addCleanup(tree.unlock)
-    # One could add 'tree._validate' here but that would cause 'early' failues 
-    # as far as higher level code is concerned. Possibly adding an
-    # expect_fail parameter to this function and if that is False then do a
-    # validate call.
+    if not invalid_delta:
+        tree._validate()
     return tree.inventory
 
 
-def apply_inventory_WT_basis(self, basis, delta):
+def _create_repo_revisions(repo, basis, delta, invalid_delta):
+    repo.start_write_group()
+    try:
+        rev = revision.Revision('basis', timestamp=0, timezone=None,
+            message="", committer="foo@example.com")
+        basis.revision_id = 'basis'
+        create_texts_for_inv(repo, basis)
+        repo.add_revision('basis', rev, basis)
+        if invalid_delta:
+            # We don't want to apply the delta to the basis, because we expect
+            # the delta is invalid.
+            result_inv = basis
+            result_inv.revision_id = 'result'
+            target_entries = None
+        else:
+            result_inv = basis.create_by_apply_delta(delta, 'result')
+            create_texts_for_inv(repo, result_inv)
+            target_entries = list(result_inv.iter_entries_by_dir())
+        rev = revision.Revision('result', timestamp=0, timezone=None,
+            message="", committer="foo@example.com")
+        repo.add_revision('result', rev, result_inv)
+        repo.commit_write_group()
+    except:
+        repo.abort_write_group()
+        raise
+    return target_entries
+
+
+def _get_basis_entries(tree):
+    basis_tree = tree.basis_tree()
+    basis_tree.lock_read()
+    basis_tree_entries = list(basis_tree.inventory.iter_entries_by_dir())
+    basis_tree.unlock()
+    return basis_tree_entries
+
+
+def _populate_different_tree(tree, basis, delta):
+    """Put all entries into tree, but at a unique location."""
+    added_ids = set()
+    added_paths = set()
+    tree.add(['unique-dir'], ['unique-dir-id'], ['directory'])
+    for path, ie in basis.iter_entries_by_dir():
+        if ie.file_id in added_ids:
+            continue
+        # We want a unique path for each of these, we use the file-id
+        tree.add(['unique-dir/' + ie.file_id], [ie.file_id], [ie.kind])
+        added_ids.add(ie.file_id)
+    for old_path, new_path, file_id, ie in delta:
+        if file_id in added_ids:
+            continue
+        tree.add(['unique-dir/' + file_id], [file_id], [ie.kind])
+
+
+def apply_inventory_WT_basis(test, basis, delta, invalid_delta=True):
     """Apply delta to basis and return the result.
 
     This sets the parent and then calls update_basis_by_delta.
@@ -130,88 +191,48 @@
     allow safety checks made by the WT to succeed, and finally ensures that all
     items in the delta with a new path are present in the WT before calling
     update_basis_by_delta.
-    
+
     :param basis: An inventory to be used as the basis.
     :param delta: The inventory delta to apply:
     :return: An inventory resulting from the application.
     """
-    control = self.make_bzrdir('tree', format=self.format._matchingbzrdir)
+    control = test.make_bzrdir('tree', format=test.format._matchingbzrdir)
     control.create_repository()
     control.create_branch()
-    tree = self.format.initialize(control)
+    tree = test.format.initialize(control)
     tree.lock_write()
     try:
-        repo = tree.branch.repository
-        repo.start_write_group()
-        try:
-            rev = revision.Revision('basis', timestamp=0, timezone=None,
-                message="", committer="foo@example.com")
-            basis.revision_id = 'basis'
-            create_texts_for_inv(tree.branch.repository, basis)
-            repo.add_revision('basis', rev, basis)
-            # Add a revision for the result, with the basis content - 
-            # update_basis_by_delta doesn't check that the delta results in
-            # result, and we want inconsistent deltas to get called on the
-            # tree, or else the code isn't actually checked.
-            rev = revision.Revision('result', timestamp=0, timezone=None,
-                message="", committer="foo@example.com")
-            basis.revision_id = 'result'
-            repo.add_revision('result', rev, basis)
-            repo.commit_write_group()
-        except:
-            repo.abort_write_group()
-            raise
+        target_entries = _create_repo_revisions(tree.branch.repository, basis,
+                                                delta, invalid_delta)
         # Set the basis state as the trees current state
         tree._write_inventory(basis)
         # This reads basis from the repo and puts it into the tree's local
         # cache, if it has one.
         tree.set_parent_ids(['basis'])
-        paths = {}
-        parents = set()
-        for old, new, id, entry in delta:
-            if None in (new, entry):
-                continue
-            paths[new] = (entry.file_id, entry.kind)
-            parents.add(osutils.dirname(new))
-        parents = osutils.minimum_path_selection(parents)
-        parents.discard('')
-        # Put place holders in the tree to permit adding the other entries.
-        for pos, parent in enumerate(parents):
-            if not tree.path2id(parent):
-                # add a synthetic directory in the tree so we can can put the
-                # tree0 entries in place for dirstate.
-                tree.add([parent], ["id%d" % pos], ["directory"])
-        if paths:
-            # Many deltas may cause this mini-apply to fail, but we want to see what
-            # the delta application code says, not the prep that we do to deal with 
-            # limitations of dirstate's update_basis code.
-            for path, (file_id, kind) in sorted(paths.items()):
-                try:
-                    tree.add([path], [file_id], [kind])
-                except (KeyboardInterrupt, SystemExit):
-                    raise
-                except:
-                    pass
     finally:
         tree.unlock()
     # Fresh lock, reads disk again.
     tree.lock_write()
     try:
         tree.update_basis_by_delta('result', delta)
+        if not invalid_delta:
+            tree._validate()
     finally:
         tree.unlock()
     # reload tree - ensure we get what was written.
     tree = tree.bzrdir.open_workingtree()
     basis_tree = tree.basis_tree()
     basis_tree.lock_read()
-    self.addCleanup(basis_tree.unlock)
-    # Note, that if the tree does not have a local cache, the trick above of
-    # setting the result as the basis, will come back to bite us. That said,
-    # all the implementations in bzr do have a local cache.
-    return basis_tree.inventory
-
-
-def apply_inventory_Repository_add_inventory_by_delta(self, basis, delta):
+    test.addCleanup(basis_tree.unlock)
+    basis_inv = basis_tree.inventory
+    if target_entries:
+        basis_entries = list(basis_inv.iter_entries_by_dir())
+        test.assertEqual(target_entries, basis_entries)
+    return basis_inv
+
+
+def apply_inventory_Repository_add_inventory_by_delta(self, basis, delta,
+                                                      invalid_delta=True):
     """Apply delta to basis and return the result.
     
     This inserts basis as a whole inventory and then uses
@@ -330,6 +351,8 @@
 
 
 class TestDeltaApplication(TestCaseWithTransport):
+
+    scenarios = delta_application_scenarios()
  
     def get_empty_inventory(self, reference_inv=None):
         """Get an empty inventory.
@@ -350,6 +373,13 @@
             inv.root.revision = 'basis'
         return inv
 
+    def make_file_ie(self, file_id='file-id', name='name', parent_id=None):
+        ie_file = inventory.InventoryFile(file_id, name, parent_id)
+        ie_file.revision = 'result'
+        ie_file.text_size = 0
+        ie_file.text_sha1 = ''
+        return ie_file
+
     def test_empty_delta(self):
         inv = self.get_empty_inventory()
         delta = []
@@ -379,10 +409,8 @@
         file1.revision = 'result'
         file1.text_size = 0
         file1.text_sha1 = ""
-        file2 = inventory.InventoryFile('id', 'path2', inv.root.file_id)
-        file2.revision = 'result'
-        file2.text_size = 0
-        file2.text_sha1 = ""
+        file2 = file1.copy()
+        file2.name = 'path2'
         delta = [(None, u'path1', 'id', file1), (None, u'path2', 'id', file2)]
         self.assertRaises(errors.InconsistentDelta, self.apply_delta, self,
             inv, delta)
@@ -393,10 +421,8 @@
         file1.revision = 'result'
         file1.text_size = 0
         file1.text_sha1 = ""
-        file2 = inventory.InventoryFile('id2', 'path', inv.root.file_id)
-        file2.revision = 'result'
-        file2.text_size = 0
-        file2.text_sha1 = ""
+        file2 = file1.copy()
+        file2.file_id = 'id2'
         delta = [(None, u'path', 'id1', file1), (None, u'path', 'id2', file2)]
         self.assertRaises(errors.InconsistentDelta, self.apply_delta, self,
             inv, delta)
@@ -574,8 +600,81 @@
         self.assertRaises(errors.InconsistentDelta, self.apply_delta, self,
             inv, delta)
 
-
-class TestInventory(TestCase):
+    def test_add_file(self):
+        inv = self.get_empty_inventory()
+        file1 = inventory.InventoryFile('file-id', 'path', inv.root.file_id)
+        file1.revision = 'result'
+        file1.text_size = 0
+        file1.text_sha1 = ''
+        delta = [(None, u'path', 'file-id', file1)]
+        res_inv = self.apply_delta(self, inv, delta, invalid_delta=False)
+        self.assertEqual('file-id', res_inv['file-id'].file_id)
+
+    def test_remove_file(self):
+        inv = self.get_empty_inventory()
+        file1 = inventory.InventoryFile('file-id', 'path', inv.root.file_id)
+        file1.revision = 'result'
+        file1.text_size = 0
+        file1.text_sha1 = ''
+        inv.add(file1)
+        delta = [(u'path', None, 'file-id', None)]
+        res_inv = self.apply_delta(self, inv, delta, invalid_delta=False)
+        self.assertEqual(None, res_inv.path2id('path'))
+        self.assertRaises(errors.NoSuchId, res_inv.id2path, 'file-id')
+
+    def test_rename_file(self):
+        inv = self.get_empty_inventory()
+        file1 = self.make_file_ie(name='path', parent_id=inv.root.file_id)
+        inv.add(file1)
+        file2 = self.make_file_ie(name='path2', parent_id=inv.root.file_id)
+        delta = [(u'path', 'path2', 'file-id', file2)]
+        res_inv = self.apply_delta(self, inv, delta, invalid_delta=False)
+        self.assertEqual(None, res_inv.path2id('path'))
+        self.assertEqual('file-id', res_inv.path2id('path2'))
+
+    def test_replaced_at_new_path(self):
+        inv = self.get_empty_inventory()
+        file1 = self.make_file_ie(file_id='id1', parent_id=inv.root.file_id)
+        inv.add(file1)
+        file2 = self.make_file_ie(file_id='id2', parent_id=inv.root.file_id)
+        delta = [(u'name', None, 'id1', None),
+                 (None, u'name', 'id2', file2)]
+        res_inv = self.apply_delta(self, inv, delta, invalid_delta=False)
+        self.assertEqual('id2', res_inv.path2id('name'))
+
+    def test_rename_dir(self):
+        inv = self.get_empty_inventory()
+        dir1 = inventory.InventoryDirectory('dir-id', 'dir1', inv.root.file_id)
+        dir1.revision = 'basis'
+        file1 = self.make_file_ie(parent_id='dir-id')
+        inv.add(dir1)
+        inv.add(file1)
+        dir2 = inventory.InventoryDirectory('dir-id', 'dir2', inv.root.file_id)
+        dir2.revision = 'result'
+        delta = [('dir1', 'dir2', 'dir-id', dir2)]
+        res_inv = self.apply_delta(self, inv, delta, invalid_delta=False)
+        # The file should be accessible under the new path
+        self.assertEqual('file-id', res_inv.path2id('dir2/name'))
+
+    def test_renamed_dir_with_renamed_child(self):
+        inv = self.get_empty_inventory()
+        dir1 = inventory.InventoryDirectory('dir-id', 'dir1', inv.root.file_id)
+        dir1.revision = 'basis'
+        file1 = self.make_file_ie('file-id-1', 'name1', parent_id='dir-id')
+        file2 = self.make_file_ie('file-id-2', 'name2', parent_id='dir-id')
+        inv.add(dir1)
+        inv.add(file1)
+        inv.add(file2)
+        dir2 = inventory.InventoryDirectory('dir-id', 'dir2', inv.root.file_id)
+        dir2.revision = 'result'
+        file2b = self.make_file_ie('file-id-2', 'name2', inv.root.file_id)
+        delta = [('dir1', 'dir2', 'dir-id', dir2),
+                 ('dir1/name2', 'name2', 'file-id-2', file2b)]
+        res_inv = self.apply_delta(self, inv, delta, invalid_delta=False)
+        # The file should be accessible under the new path
+        self.assertEqual('file-id-1', res_inv.path2id('dir2/name1'))
+        self.assertEqual(None, res_inv.path2id('dir2/name2'))
+        self.assertEqual('file-id-2', res_inv.path2id('name2'))
 
     def test_is_root(self):
         """Ensure our root-checking code is accurate."""
@@ -590,6 +689,11 @@
         self.assertFalse(inv.is_root('TREE_ROOT'))
         self.assertFalse(inv.is_root('booga'))
 
+    def test_entries_for_empty_inventory(self):
+        """Test that entries() will not fail for an empty inventory"""
+        inv = Inventory(root_id=None)
+        self.assertEqual([], inv.entries())
+
 
 class TestInventoryEntry(TestCase):
 
@@ -607,12 +711,7 @@
 
     def test_dir_detect_changes(self):
         left = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
-        left.text_sha1 = 123
-        left.executable = True
-        left.symlink_target='foo'
         right = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
-        right.text_sha1 = 321
-        right.symlink_target='bar'
         self.assertEqual((False, False), left.detect_changes(right))
         self.assertEqual((False, False), right.detect_changes(left))
 
@@ -632,11 +731,8 @@
 
     def test_symlink_detect_changes(self):
         left = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
-        left.text_sha1 = 123
-        left.executable = True
         left.symlink_target='foo'
         right = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
-        right.text_sha1 = 321
         right.symlink_target='foo'
         self.assertEqual((False, False), left.detect_changes(right))
         self.assertEqual((False, False), right.detect_changes(left))
@@ -646,15 +742,15 @@
 
     def test_file_has_text(self):
         file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
-        self.failUnless(file.has_text())
+        self.assertTrue(file.has_text())
 
     def test_directory_has_text(self):
         dir = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
-        self.failIf(dir.has_text())
+        self.assertFalse(dir.has_text())
 
     def test_link_has_text(self):
         link = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
-        self.failIf(link.has_text())
+        self.assertFalse(link.has_text())
 
     def test_make_entry(self):
         self.assertIsInstance(inventory.make_entry("file", "name", ROOT_ID),
@@ -1215,6 +1311,108 @@
         self.assertEqual(('tree\xce\xa9name', 'tree-root-id', 'tree-rev-id'),
                          inv._bytes_to_utf8name_key(bytes))
 
+    def make_basic_utf8_inventory(self):
+        inv = Inventory()
+        inv.revision_id = "revid"
+        inv.root.revision = "rootrev"
+        root_id = inv.root.file_id
+        inv.add(InventoryFile("fileid", u'f\xefle', root_id))
+        inv["fileid"].revision = "filerev"
+        inv["fileid"].text_sha1 = "ffff"
+        inv["fileid"].text_size = 0
+        inv.add(InventoryDirectory("dirid", u'dir-\N{EURO SIGN}', root_id))
+        inv.add(InventoryFile("childid", u'ch\xefld', "dirid"))
+        inv["childid"].revision = "filerev"
+        inv["childid"].text_sha1 = "ffff"
+        inv["childid"].text_size = 0
+        chk_bytes = self.get_chk_bytes()
+        chk_inv = CHKInventory.from_inventory(chk_bytes, inv)
+        bytes = ''.join(chk_inv.to_lines())
+        return CHKInventory.deserialise(chk_bytes, bytes, ("revid",))
+
+    def test__preload_handles_utf8(self):
+        new_inv = self.make_basic_utf8_inventory()
+        self.assertEqual({}, new_inv._fileid_to_entry_cache)
+        self.assertFalse(new_inv._fully_cached)
+        new_inv._preload_cache()
+        self.assertEqual(
+            sorted([new_inv.root_id, "fileid", "dirid", "childid"]),
+            sorted(new_inv._fileid_to_entry_cache.keys()))
+        ie_root = new_inv._fileid_to_entry_cache[new_inv.root_id]
+        self.assertEqual([u'dir-\N{EURO SIGN}', u'f\xefle'],
+                         sorted(ie_root._children.keys()))
+        ie_dir = new_inv._fileid_to_entry_cache['dirid']
+        self.assertEqual([u'ch\xefld'], sorted(ie_dir._children.keys()))
+
+    def test__preload_populates_cache(self):
+        inv = Inventory()
+        inv.revision_id = "revid"
+        inv.root.revision = "rootrev"
+        root_id = inv.root.file_id
+        inv.add(InventoryFile("fileid", "file", root_id))
+        inv["fileid"].revision = "filerev"
+        inv["fileid"].executable = True
+        inv["fileid"].text_sha1 = "ffff"
+        inv["fileid"].text_size = 1
+        inv.add(InventoryDirectory("dirid", "dir", root_id))
+        inv.add(InventoryFile("childid", "child", "dirid"))
+        inv["childid"].revision = "filerev"
+        inv["childid"].executable = False
+        inv["childid"].text_sha1 = "dddd"
+        inv["childid"].text_size = 1
+        chk_bytes = self.get_chk_bytes()
+        chk_inv = CHKInventory.from_inventory(chk_bytes, inv)
+        bytes = ''.join(chk_inv.to_lines())
+        new_inv = CHKInventory.deserialise(chk_bytes, bytes, ("revid",))
+        self.assertEqual({}, new_inv._fileid_to_entry_cache)
+        self.assertFalse(new_inv._fully_cached)
+        new_inv._preload_cache()
+        self.assertEqual(
+            sorted([root_id, "fileid", "dirid", "childid"]),
+            sorted(new_inv._fileid_to_entry_cache.keys()))
+        self.assertTrue(new_inv._fully_cached)
+        ie_root = new_inv._fileid_to_entry_cache[root_id]
+        self.assertEqual(['dir', 'file'], sorted(ie_root._children.keys()))
+        ie_dir = new_inv._fileid_to_entry_cache['dirid']
+        self.assertEqual(['child'], sorted(ie_dir._children.keys()))
+
+    def test__preload_handles_partially_evaluated_inventory(self):
+        new_inv = self.make_basic_utf8_inventory()
+        ie = new_inv[new_inv.root_id]
+        self.assertIs(None, ie._children)
+        self.assertEqual([u'dir-\N{EURO SIGN}', u'f\xefle'],
+                         sorted(ie.children.keys()))
+        # Accessing .children loads _children
+        self.assertEqual([u'dir-\N{EURO SIGN}', u'f\xefle'],
+                         sorted(ie._children.keys()))
+        new_inv._preload_cache()
+        # No change
+        self.assertEqual([u'dir-\N{EURO SIGN}', u'f\xefle'],
+                         sorted(ie._children.keys()))
+        ie_dir = new_inv["dirid"]
+        self.assertEqual([u'ch\xefld'],
+                         sorted(ie_dir._children.keys()))
+
+    def test_filter_change_in_renamed_subfolder(self):
+        inv = Inventory('tree-root')
+        src_ie = inv.add_path('src', 'directory', 'src-id')
+        inv.add_path('src/sub/', 'directory', 'sub-id')
+        a_ie = inv.add_path('src/sub/a', 'file', 'a-id')
+        a_ie.text_sha1 = osutils.sha_string('content\n')
+        a_ie.text_size = len('content\n')
+        chk_bytes = self.get_chk_bytes()
+        inv = CHKInventory.from_inventory(chk_bytes, inv)
+        inv = inv.create_by_apply_delta([
+            ("src/sub/a", "src/sub/a", "a-id", a_ie),
+            ("src", "src2", "src-id", src_ie),
+            ], 'new-rev-2')
+        new_inv = inv.filter(['a-id', 'src-id'])
+        self.assertEqual([
+            ('', 'tree-root'),
+            ('src', 'src-id'),
+            ('src/sub', 'sub-id'),
+            ('src/sub/a', 'a-id'),
+            ], [(path, ie.file_id) for path, ie in new_inv.iter_entries()])
 
 class TestCHKInventoryExpand(tests.TestCaseWithMemoryTransport):
 
@@ -1346,3 +1544,29 @@
         inv = self.make_simple_inventory()
         self.assertExpand(['TREE_ROOT', 'dir1-id', 'sub-dir1-id', 'top-id',
                            'subsub-file1-id'], inv, ['top-id', 'subsub-file1-id'])
+
+
+class TestMutableInventoryFromTree(TestCaseWithTransport):
+
+    def test_empty(self):
+        repository = self.make_repository('.')
+        tree = repository.revision_tree(revision.NULL_REVISION)
+        inv = mutable_inventory_from_tree(tree)
+        self.assertEquals(revision.NULL_REVISION, inv.revision_id)
+        self.assertEquals(0, len(inv))
+
+    def test_some_files(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['a'])
+        wt.add(['a'], ['thefileid'])
+        revid = wt.commit("commit")
+        tree = wt.branch.repository.revision_tree(revid)
+        inv = mutable_inventory_from_tree(tree)
+        self.assertEquals(revid, inv.revision_id)
+        self.assertEquals(2, len(inv))
+        self.assertEquals("a", inv['thefileid'].name)
+        # The inventory should be mutable and independent of
+        # the original tree
+        self.assertFalse(tree.inventory['thefileid'].executable)
+        inv['thefileid'].executable = True
+        self.assertFalse(tree.inventory['thefileid'].executable)

=== modified file 'bzrlib/tests/test_inventory_delta.py'
--- bzrlib/tests/test_inventory_delta.py	2009-08-13 00:20:29 +0000
+++ bzrlib/tests/test_inventory_delta.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009 Canonical Ltd
+# Copyright (C) 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 from cStringIO import StringIO
 
 from bzrlib import (
-    errors,
     inventory,
     inventory_delta,
     )
@@ -387,9 +386,10 @@
         root = new_inv.make_entry('directory', '', None, 'my-rich-root-id')
         root.revision = 'changed'
         new_inv.add(root)
-        non_root = new_inv.make_entry('directory', 'foo', root.file_id, 'id')
+        class StrangeInventoryEntry(inventory.InventoryEntry):
+            kind = 'strange'
+        non_root = StrangeInventoryEntry('id', 'foo', root.file_id)
         non_root.revision = 'changed'
-        non_root.kind = 'strangelove'
         new_inv.add(non_root)
         delta = new_inv._make_delta(old_inv)
         serializer = inventory_delta.InventoryDeltaSerializer(
@@ -398,7 +398,7 @@
         # This test aims to prove that it errors more than how it errors.
         err = self.assertRaises(KeyError,
             serializer.delta_to_lines, NULL_REVISION, 'entry-version', delta)
-        self.assertEqual(('strangelove',), err.args)
+        self.assertEqual(('strange',), err.args)
 
     def test_tree_reference_disabled(self):
         old_inv = Inventory(None)

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_knit.py	2011-09-16 20:42:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 """Tests for Knit data structure"""
 
 from cStringIO import StringIO
+import gzip
 import sys
 
 from bzrlib import (
@@ -27,7 +28,6 @@
     pack,
     tests,
     transport,
-    tuned_gzip,
     )
 from bzrlib.errors import (
     KnitHeaderError,
@@ -40,14 +40,16 @@
     KnitVersionedFiles,
     PlainKnitContent,
     _VFContentMapGenerator,
-    _DirectPackAccess,
     _KndxIndex,
     _KnitGraphIndex,
     _KnitKeyAccess,
     make_file_factory,
     )
 from bzrlib.patiencediff import PatienceSequenceMatcher
-from bzrlib.repofmt import pack_repo
+from bzrlib.repofmt import (
+    knitpack_repo,
+    pack_repo,
+    )
 from bzrlib.tests import (
     TestCase,
     TestCaseWithMemoryTransport,
@@ -60,10 +62,13 @@
     network_bytes_to_kind_and_offset,
     RecordingVersionedFilesDecorator,
     )
-
-
-compiled_knit_feature = tests.ModuleAvailableFeature(
-                            'bzrlib._knit_load_data_pyx')
+from bzrlib.tests import (
+    features,
+    )
+
+
+compiled_knit_feature = features.ModuleAvailableFeature(
+    'bzrlib._knit_load_data_pyx')
 
 
 class KnitContentTestsMixin(object):
@@ -325,7 +330,7 @@
             transport.append_bytes(packname, bytes)
         writer = pack.ContainerWriter(write_data)
         writer.begin()
-        access = _DirectPackAccess({})
+        access = pack_repo._DirectPackAccess({})
         access.set_writer(writer, index, (transport, packname))
         return access, writer
 
@@ -338,6 +343,33 @@
         writer.end()
         return memos
 
+    def test_pack_collection_pack_retries(self):
+        """An explicit pack of a pack collection succeeds even when a
+        concurrent pack happens.
+        """
+        builder = self.make_branch_builder('.')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('file', 'file-id', 'file', 'content\nrev 1\n')),
+            ])
+        builder.build_snapshot('rev-2', ['rev-1'], [
+            ('modify', ('file-id', 'content\nrev 2\n')),
+            ])
+        builder.build_snapshot('rev-3', ['rev-2'], [
+            ('modify', ('file-id', 'content\nrev 3\n')),
+            ])
+        self.addCleanup(builder.finish_series)
+        b = builder.get_branch()
+        self.addCleanup(b.lock_write().unlock)
+        repo = b.repository
+        collection = repo._pack_collection
+        # Concurrently repack the repo.
+        reopened_repo = repo.bzrdir.open_repository()
+        reopened_repo.pack()
+        # Pack the new pack.
+        collection.pack()
+
     def make_vf_for_retrying(self):
         """Create 3 packs and a reload function.
 
@@ -370,7 +402,7 @@
         collection = repo._pack_collection
         collection.ensure_loaded()
         orig_packs = collection.packs
-        packer = pack_repo.Packer(collection, orig_packs, '.testpack')
+        packer = knitpack_repo.KnitPacker(collection, orig_packs, '.testpack')
         new_pack = packer.pack()
         # forget about the new pack
         collection.reset()
@@ -415,6 +447,7 @@
         except _TestException, e:
             retry_exc = errors.RetryWithNewPacks(None, reload_occurred=False,
                                                  exc_info=sys.exc_info())
+        # GZ 2010-08-10: Cycle with exc_info affects 3 tests
         return retry_exc
 
     def test_read_from_several_packs(self):
@@ -429,7 +462,7 @@
         memos.extend(access.add_raw_records([('key', 5)], 'alpha'))
         writer.end()
         transport = self.get_transport()
-        access = _DirectPackAccess({"FOO":(transport, 'packfile'),
+        access = pack_repo._DirectPackAccess({"FOO":(transport, 'packfile'),
             "FOOBAR":(transport, 'pack2'),
             "BAZ":(transport, 'pack3')})
         self.assertEqual(['1234567890', '12345', 'alpha'],
@@ -445,7 +478,7 @@
 
     def test_set_writer(self):
         """The writer should be settable post construction."""
-        access = _DirectPackAccess({})
+        access = pack_repo._DirectPackAccess({})
         transport = self.get_transport()
         packname = 'packfile'
         index = 'foo'
@@ -463,7 +496,7 @@
         transport = self.get_transport()
         reload_called, reload_func = self.make_reload_func()
         # Note that the index key has changed from 'foo' to 'bar'
-        access = _DirectPackAccess({'bar':(transport, 'packname')},
+        access = pack_repo._DirectPackAccess({'bar':(transport, 'packname')},
                                    reload_func=reload_func)
         e = self.assertListRaises(errors.RetryWithNewPacks,
                                   access.get_raw_records, memos)
@@ -478,7 +511,7 @@
         memos = self.make_pack_file()
         transport = self.get_transport()
         # Note that the index key has changed from 'foo' to 'bar'
-        access = _DirectPackAccess({'bar':(transport, 'packname')})
+        access = pack_repo._DirectPackAccess({'bar':(transport, 'packname')})
         e = self.assertListRaises(KeyError, access.get_raw_records, memos)
 
     def test_missing_file_raises_retry(self):
@@ -486,8 +519,9 @@
         transport = self.get_transport()
         reload_called, reload_func = self.make_reload_func()
         # Note that the 'filename' has been changed to 'different-packname'
-        access = _DirectPackAccess({'foo':(transport, 'different-packname')},
-                                   reload_func=reload_func)
+        access = pack_repo._DirectPackAccess(
+            {'foo':(transport, 'different-packname')},
+            reload_func=reload_func)
         e = self.assertListRaises(errors.RetryWithNewPacks,
                                   access.get_raw_records, memos)
         # The file has gone missing, so we assume we need to reload
@@ -501,7 +535,8 @@
         memos = self.make_pack_file()
         transport = self.get_transport()
         # Note that the 'filename' has been changed to 'different-packname'
-        access = _DirectPackAccess({'foo':(transport, 'different-packname')})
+        access = pack_repo._DirectPackAccess(
+            {'foo': (transport, 'different-packname')})
         e = self.assertListRaises(errors.NoSuchFile,
                                   access.get_raw_records, memos)
 
@@ -511,8 +546,9 @@
         failing_transport = MockReadvFailingTransport(
                                 [transport.get_bytes('packname')])
         reload_called, reload_func = self.make_reload_func()
-        access = _DirectPackAccess({'foo':(failing_transport, 'packname')},
-                                   reload_func=reload_func)
+        access = pack_repo._DirectPackAccess(
+            {'foo': (failing_transport, 'packname')},
+            reload_func=reload_func)
         # Asking for a single record will not trigger the Mock failure
         self.assertEqual(['1234567890'],
             list(access.get_raw_records(memos[:1])))
@@ -534,7 +570,8 @@
         failing_transport = MockReadvFailingTransport(
                                 [transport.get_bytes('packname')])
         reload_called, reload_func = self.make_reload_func()
-        access = _DirectPackAccess({'foo':(failing_transport, 'packname')})
+        access = pack_repo._DirectPackAccess(
+            {'foo':(failing_transport, 'packname')})
         # Asking for a single record will not trigger the Mock failure
         self.assertEqual(['1234567890'],
             list(access.get_raw_records(memos[:1])))
@@ -545,14 +582,14 @@
                                   access.get_raw_records, memos)
 
     def test_reload_or_raise_no_reload(self):
-        access = _DirectPackAccess({}, reload_func=None)
+        access = pack_repo._DirectPackAccess({}, reload_func=None)
         retry_exc = self.make_retry_exception()
         # Without a reload_func, we will just re-raise the original exception
         self.assertRaises(_TestException, access.reload_or_raise, retry_exc)
 
     def test_reload_or_raise_reload_changed(self):
         reload_called, reload_func = self.make_reload_func(return_val=True)
-        access = _DirectPackAccess({}, reload_func=reload_func)
+        access = pack_repo._DirectPackAccess({}, reload_func=reload_func)
         retry_exc = self.make_retry_exception()
         access.reload_or_raise(retry_exc)
         self.assertEqual([1], reload_called)
@@ -562,7 +599,7 @@
 
     def test_reload_or_raise_reload_no_change(self):
         reload_called, reload_func = self.make_reload_func(return_val=False)
-        access = _DirectPackAccess({}, reload_func=reload_func)
+        access = pack_repo._DirectPackAccess({}, reload_func=reload_func)
         retry_exc = self.make_retry_exception()
         # If reload_occurred is False, then we consider it an error to have
         # reload_func() return False (no changes).
@@ -692,14 +729,14 @@
 
     def create_gz_content(self, text):
         sio = StringIO()
-        gz_file = tuned_gzip.GzipFile(mode='wb', fileobj=sio)
+        gz_file = gzip.GzipFile(mode='wb', fileobj=sio)
         gz_file.write(text)
         gz_file.close()
         return sio.getvalue()
 
     def make_multiple_records(self):
         """Create the content for multiple records."""
-        sha1sum = osutils.sha('foo\nbar\n').hexdigest()
+        sha1sum = osutils.sha_string('foo\nbar\n')
         total_txt = []
         gz_txt = self.create_gz_content('version rev-id-1 2 %s\n'
                                         'foo\n'
@@ -708,7 +745,7 @@
                                         % (sha1sum,))
         record_1 = (0, len(gz_txt), sha1sum)
         total_txt.append(gz_txt)
-        sha1sum = osutils.sha('baz\n').hexdigest()
+        sha1sum = osutils.sha_string('baz\n')
         gz_txt = self.create_gz_content('version rev-id-2 1 %s\n'
                                         'baz\n'
                                         'end rev-id-2\n'
@@ -718,7 +755,7 @@
         return total_txt, record_1, record_2
 
     def test_valid_knit_data(self):
-        sha1sum = osutils.sha('foo\nbar\n').hexdigest()
+        sha1sum = osutils.sha_string('foo\nbar\n')
         gz_txt = self.create_gz_content('version rev-id-1 2 %s\n'
                                         'foo\n'
                                         'bar\n'
@@ -755,7 +792,7 @@
                          raw_contents)
 
     def test_not_enough_lines(self):
-        sha1sum = osutils.sha('foo\n').hexdigest()
+        sha1sum = osutils.sha_string('foo\n')
         # record says 2 lines data says 1
         gz_txt = self.create_gz_content('version rev-id-1 2 %s\n'
                                         'foo\n'
@@ -773,7 +810,7 @@
         self.assertEqual([(('rev-id-1',),  gz_txt, sha1sum)], raw_contents)
 
     def test_too_many_lines(self):
-        sha1sum = osutils.sha('foo\nbar\n').hexdigest()
+        sha1sum = osutils.sha_string('foo\nbar\n')
         # record says 1 lines data says 2
         gz_txt = self.create_gz_content('version rev-id-1 1 %s\n'
                                         'foo\n'
@@ -792,7 +829,7 @@
         self.assertEqual([(('rev-id-1',), gz_txt, sha1sum)], raw_contents)
 
     def test_mismatched_version_id(self):
-        sha1sum = osutils.sha('foo\nbar\n').hexdigest()
+        sha1sum = osutils.sha_string('foo\nbar\n')
         gz_txt = self.create_gz_content('version rev-id-1 2 %s\n'
                                         'foo\n'
                                         'bar\n'
@@ -811,7 +848,7 @@
             knit._read_records_iter_raw(records))
 
     def test_uncompressed_data(self):
-        sha1sum = osutils.sha('foo\nbar\n').hexdigest()
+        sha1sum = osutils.sha_string('foo\nbar\n')
         txt = ('version rev-id-1 2 %s\n'
                'foo\n'
                'bar\n'
@@ -831,7 +868,7 @@
             knit._read_records_iter_raw(records))
 
     def test_corrupted_data(self):
-        sha1sum = osutils.sha('foo\nbar\n').hexdigest()
+        sha1sum = osutils.sha_string('foo\nbar\n')
         gz_txt = self.create_gz_content('version rev-id-1 2 %s\n'
                                         'foo\n'
                                         'bar\n'
@@ -1159,8 +1196,7 @@
             self.assertRaises(errors.KnitCorrupt, index.keys)
         except TypeError, e:
             if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.IndexError')
-                and sys.version_info[0:2] >= (2,5)):
+                           ' not exceptions.IndexError')):
                 self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
                                   ' raising new style exceptions with python'
                                   ' >=2.5')
@@ -1179,8 +1215,7 @@
             self.assertRaises(errors.KnitCorrupt, index.keys)
         except TypeError, e:
             if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')
-                and sys.version_info[0:2] >= (2,5)):
+                           ' not exceptions.ValueError')):
                 self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
                                   ' raising new style exceptions with python'
                                   ' >=2.5')
@@ -1199,8 +1234,7 @@
             self.assertRaises(errors.KnitCorrupt, index.keys)
         except TypeError, e:
             if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')
-                and sys.version_info[0:2] >= (2,5)):
+                           ' not exceptions.ValueError')):
                 self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
                                   ' raising new style exceptions with python'
                                   ' >=2.5')
@@ -1217,8 +1251,7 @@
             self.assertRaises(errors.KnitCorrupt, index.keys)
         except TypeError, e:
             if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')
-                and sys.version_info[0:2] >= (2,5)):
+                           ' not exceptions.ValueError')):
                 self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
                                   ' raising new style exceptions with python'
                                   ' >=2.5')
@@ -1235,8 +1268,7 @@
             self.assertRaises(errors.KnitCorrupt, index.keys)
         except TypeError, e:
             if (str(e) == ('exceptions must be strings, classes, or instances,'
-                           ' not exceptions.ValueError')
-                and sys.version_info[0:2] >= (2,5)):
+                           ' not exceptions.ValueError')):
                 self.knownFailure('Pyrex <0.9.5 fails with TypeError when'
                                   ' raising new style exceptions with python'
                                   ' >=2.5')
@@ -1571,13 +1603,13 @@
         # could leave an empty .kndx file, which bzr would later claim was a
         # corrupted file since the header was not present. In reality, the file
         # just wasn't created, so it should be ignored.
-        t = transport.get_transport('.')
+        t = transport.get_transport_from_path('.')
         t.put_bytes('test.kndx', '')
 
         knit = self.make_test_knit()
 
     def test_knit_index_checks_header(self):
-        t = transport.get_transport('.')
+        t = transport.get_transport_from_path('.')
         t.put_bytes('test.kndx', '# not really a knit header\n\n')
         k = self.make_test_knit()
         self.assertRaises(KnitHeaderError, k.keys)
@@ -2411,7 +2443,7 @@
         key_basis = ('bar',)
         key_missing = ('missing',)
         test.add_lines(key, (), ['foo\n'])
-        key_sha1sum = osutils.sha('foo\n').hexdigest()
+        key_sha1sum = osutils.sha_string('foo\n')
         sha1s = test.get_sha1s([key])
         self.assertEqual({key: key_sha1sum}, sha1s)
         self.assertEqual([], basis.calls)
@@ -2419,7 +2451,7 @@
         # directly (rather than via text reconstruction) so that remote servers
         # etc don't have to answer with full content.
         basis.add_lines(key_basis, (), ['foo\n', 'bar\n'])
-        basis_sha1sum = osutils.sha('foo\nbar\n').hexdigest()
+        basis_sha1sum = osutils.sha_string('foo\nbar\n')
         basis.calls = []
         sha1s = test.get_sha1s([key, key_missing, key_basis])
         self.assertEqual({key: key_sha1sum,

=== modified file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	2010-01-25 15:55:48 +0000
+++ bzrlib/tests/test_lazy_import.py	2011-08-19 18:19:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,9 @@
 
 """Test the lazy_import functionality."""
 
+import linecache
 import os
+import re
 import sys
 
 from bzrlib import (
@@ -24,7 +26,10 @@
     lazy_import,
     osutils,
     )
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests import (
+    TestCase,
+    TestCaseInTempDir,
+    )
 
 
 class InstrumentedReplacer(lazy_import.ScopeReplacer):
@@ -127,13 +132,9 @@
         else:
             self.fail('test_obj1 was not supposed to exist yet')
 
-        orig_globals = set(globals().keys())
-
         InstrumentedReplacer(scope=globals(), name='test_obj1',
                              factory=factory)
 
-        new_globals = set(globals().keys())
-
         # We can't use isinstance() because that uses test_obj1.__class__
         # and that goes through __getattribute__ which would activate
         # the replacement
@@ -168,13 +169,9 @@
         else:
             self.fail('test_obj6 was not supposed to exist yet')
 
-        orig_globals = set(globals().keys())
-
         lazy_import.ScopeReplacer(scope=globals(), name='test_obj6',
                                   factory=factory)
 
-        new_globals = set(globals().keys())
-
         # We can't use isinstance() because that uses test_obj6.__class__
         # and that goes through __getattribute__ which would activate
         # the replacement
@@ -270,7 +267,7 @@
         InstrumentedReplacer(scope=globals(), name='test_class2',
                              factory=factory)
 
-        self.failIf(test_class2 is TestClass)
+        self.assertFalse(test_class2 is TestClass)
         obj = test_class2()
         self.assertIs(test_class2, TestClass)
         self.assertIsInstance(obj, TestClass)
@@ -303,7 +300,7 @@
         InstrumentedReplacer(scope=globals(), name='test_func1',
                              factory=factory)
 
-        self.failIf(test_func1 is func)
+        self.assertFalse(test_func1 is func)
         val = test_func1(1, 2, c='3')
         self.assertIs(test_func1, func)
 
@@ -440,6 +437,36 @@
                           ('foo', 4),
                          ], actions)
 
+    def test_replacing_from_own_scope_fails(self):
+        """If a ScopeReplacer tries to replace itself a nice error is given"""
+        actions = []
+        InstrumentedReplacer.use_actions(actions)
+        TestClass.use_actions(actions)
+
+        def factory(replacer, scope, name):
+            actions.append('factory')
+            # return the name in given scope, which is currently the replacer
+            return scope[name]
+
+        try:
+            test_obj7
+        except NameError:
+            # test_obj7 shouldn't exist yet
+            pass
+        else:
+            self.fail('test_obj7 was not supposed to exist yet')
+
+        InstrumentedReplacer(scope=globals(), name='test_obj7',
+                             factory=factory)
+
+        self.assertEqual(InstrumentedReplacer,
+                         object.__getattribute__(test_obj7, '__class__'))
+        e = self.assertRaises(errors.IllegalUseOfScopeReplacer, test_obj7)
+        self.assertIn("replace itself", e.msg)
+        self.assertEqual([('__call__', (), {}),
+                          '_replace',
+                          'factory'], actions)
+
 
 class ImportReplacerHelper(TestCaseInTempDir):
     """Test the ability to have a lazily imported module or object"""
@@ -1142,3 +1169,79 @@
                           ('_import', 'root8'),
                           ('import', self.root_name, []),
                          ], self.actions)
+
+
+class TestScopeReplacerReentrance(TestCase):
+    """The ScopeReplacer should be reentrant.
+
+    Invoking a replacer while an invocation was already on-going leads to a
+    race to see which invocation will be the first to delete the _factory and
+    _scope attributes.  The loosing caller used to see AttributeErrors (bug
+    702914).
+
+    These tests set up a tracer that stops at the moment just before one of
+    the attributes is being deleted and starts another call to the
+    functionality in question (__call__, __getattribute__, __setattr_) in
+    order win the race, setting up the originall caller to loose.
+    """
+
+    def tracer(self, frame, event, arg):
+        # Grab the name of the file that contains the code being executed.
+        filename = frame.f_globals["__file__"]
+        # Convert ".pyc" and ".pyo" file names to their ".py" equivalent.
+        filename = re.sub(r'\.py[co]$', '.py', filename)
+        # If we're executing a line of code from the right module...
+        if event == 'line' and 'lazy_import.py' in filename:
+            line = linecache.getline(filename, frame.f_lineno)
+            # ...and the line of code is the one we're looking for...
+            if 'del self._factory' in line:
+                # We don't need to trace any more.
+                sys.settrace(None)
+                # Run another racer.  This one will "win" the race, deleting
+                # the attributes.  When the first racer resumes it will loose
+                # the race, generating an AttributeError.
+                self.racer()
+        return self.tracer
+
+    def run_race(self, racer):
+        self.racer = racer
+        sys.settrace(self.tracer)
+        self.racer() # Should not raise an AttributeError
+        # Make sure the tracer actually found the code it was looking for.  If
+        # not, maybe the code was refactored in such a way that these tests
+        # aren't needed any more.
+        self.assertEqual(None, sys.gettrace())
+
+    def test_call(self):
+        def factory(*args):
+            return factory
+        replacer = lazy_import.ScopeReplacer({}, factory, 'name')
+        self.run_race(replacer)
+
+    def test_setattr(self):
+        class Replaced:
+            pass
+
+        def factory(*args):
+            return Replaced()
+
+        replacer = lazy_import.ScopeReplacer({}, factory, 'name')
+
+        def racer():
+            replacer.foo = 42
+
+        self.run_race(racer)
+
+    def test_getattribute(self):
+        class Replaced:
+            foo = 'bar'
+
+        def factory(*args):
+            return Replaced()
+
+        replacer = lazy_import.ScopeReplacer({}, factory, 'name')
+
+        def racer():
+            replacer.foo
+
+        self.run_race(racer)

=== modified file 'bzrlib/tests/test_lazy_regex.py'
--- bzrlib/tests/test_lazy_regex.py	2010-07-09 16:16:11 +0000
+++ bzrlib/tests/test_lazy_regex.py	2011-06-28 22:39:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,7 +41,8 @@
     def _real_re_compile(self, *args, **kwargs):
         self._actions.append(('_real_re_compile',
                                                args, kwargs))
-        return super(InstrumentedLazyRegex, self)._real_re_compile(*args, **kwargs)
+        return super(InstrumentedLazyRegex, self)._real_re_compile(
+            *args, **kwargs)
 
 
 class TestLazyRegex(tests.TestCase):
@@ -116,23 +117,26 @@
 
 
 class TestInstallLazyCompile(tests.TestCase):
+    """Tests for lazy compiled regexps.
 
-    def setUp(self):
-        super(TestInstallLazyCompile, self).setUp()
-        self.addCleanup(lazy_regex.reset_compile)
+    Other tests, and bzrlib in general, count on the lazy regexp compiler
+    being installed, and this is done by loading bzrlib.  So these tests
+    assume it is installed, and leave it installed when they're done.
+    """
 
     def test_install(self):
+        # Don't count on it being present
         lazy_regex.install_lazy_compile()
         pattern = re.compile('foo')
         self.assertIsInstance(pattern, lazy_regex.LazyRegex)
 
     def test_reset(self):
-        lazy_regex.install_lazy_compile()
         lazy_regex.reset_compile()
+        self.addCleanup(lazy_regex.install_lazy_compile)
         pattern = re.compile('foo')
-        self.failIf(isinstance(pattern, lazy_regex.LazyRegex),
-                    'lazy_regex.reset_compile() did not restore the original'
-                    ' compile() function %s' % (type(pattern),))
+        self.assertFalse(isinstance(pattern, lazy_regex.LazyRegex),
+            'lazy_regex.reset_compile() did not restore the original'
+            ' compile() function %s' % (type(pattern),))
         # but the returned object should still support regex operations
         m = pattern.match('foo')
         self.assertEqual('foo', m.group())

=== modified file 'bzrlib/tests/test_library_state.py'
--- bzrlib/tests/test_library_state.py	2010-06-26 01:07:16 +0000
+++ bzrlib/tests/test_library_state.py	2011-10-03 14:34:45 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 
 """Tests for BzrLibraryState."""
 
-import bzrlib
 from bzrlib import (
     library_state,
     tests,

=== modified file 'bzrlib/tests/test_lock.py'
--- bzrlib/tests/test_lock.py	2009-07-31 16:51:48 +0000
+++ bzrlib/tests/test_lock.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Canonical Ltd
+# Copyright (C) 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,29 +17,26 @@
 """Tests for OS Locks."""
 
 
-
 from bzrlib import (
     debug,
     errors,
     lock,
     tests,
     )
-
-
-def load_tests(standard_tests, module, loader):
-    """Parameterize tests for all versions of groupcompress."""
-    scenarios = []
-    for name, write_lock, read_lock in lock._lock_classes:
-        scenarios.append((name, {'write_lock': write_lock,
-                                 'read_lock': read_lock}))
-    suite = loader.suiteClass()
-    result = tests.multiply_tests(standard_tests, scenarios, suite)
-    return result
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestOSLock(tests.TestCaseInTempDir):
 
-    # Set by load_tests
+    scenarios = [(
+        name, {
+            'write_lock': write_lock,
+            'read_lock': read_lock})
+        for name, write_lock, read_lock in lock._lock_classes]
+
     read_lock = None
     write_lock = None
 

=== modified file 'bzrlib/tests/test_lockable_files.py'
--- bzrlib/tests/test_lockable_files.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_lockable_files.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,8 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from StringIO import StringIO
-
 import bzrlib
 from bzrlib import (
     errors,
@@ -23,11 +21,7 @@
     osutils,
     transport,
     )
-from bzrlib.errors import BzrBadParameterNotString, NoSuchFile, ReadOnlyError
 from bzrlib.lockable_files import LockableFiles, TransportLock
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import (
     TestCaseInTempDir,
     TestNotApplicable,
@@ -279,7 +273,7 @@
 
     def setUp(self):
         TestCaseInTempDir.setUp(self)
-        t = transport.get_transport('.')
+        t = transport.get_transport_from_path('.')
         t.mkdir('.bzr')
         self.sub_transport = t.clone('.bzr')
         self.lockable = self.get_lockable()
@@ -304,7 +298,7 @@
 
     def setUp(self):
         TestCaseInTempDir.setUp(self)
-        self.transport = transport.get_transport('.')
+        self.transport = transport.get_transport_from_path('.')
         self.lockable = self.get_lockable()
         # the lock creation here sets mode - test_permissions on branch
         # tests that implicitly, but it might be a good idea to factor
@@ -347,7 +341,7 @@
         # in test_remote and test_smart as usual.
         b = self.make_branch('foo')
         self.addCleanup(b.bzrdir.transport.disconnect)
-        self.transport = transport.get_transport('.')
+        self.transport = transport.get_transport_from_path('.')
         self.lockable = self.get_lockable()
 
     def get_lockable(self):

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2010-06-15 09:53:42 +0000
+++ bzrlib/tests/test_lockdir.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,9 +16,7 @@
 
 """Tests for LockDir"""
 
-from cStringIO import StringIO
 import os
-from threading import Thread, Lock
 import time
 
 import bzrlib
@@ -26,6 +24,7 @@
     config,
     errors,
     lock,
+    lockdir,
     osutils,
     tests,
     transport,
@@ -34,17 +33,18 @@
     LockBreakMismatch,
     LockBroken,
     LockContention,
-    LockError,
     LockFailed,
     LockNotHeld,
     )
-from bzrlib.lockdir import LockDir
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.trace import note
-
-# These tests sometimes use threads to test the behaviour of lock files with
-# concurrent actors.  This is not a typical (or necessarily supported) use;
-# they're really meant for guarding between processes.
+from bzrlib.lockdir import (
+    LockDir,
+    LockHeldInfo,
+    )
+from bzrlib.tests import (
+    features,
+    TestCase,
+    TestCaseWithTransport,
+    )
 
 # These tests are run on the default transport provided by the test framework
 # (typically a local disk transport).  That can be changed by the --transport
@@ -52,6 +52,7 @@
 # implementation are tested separately.  (The main requirement is just that
 # they don't allow overwriting nonempty directories.)
 
+
 class TestLockDir(TestCaseWithTransport):
     """Test LockDir operations"""
 
@@ -145,8 +146,8 @@
         self.addCleanup(lf1.unlock)
         # lock is held, should get some info on it
         info1 = lf1.peek()
-        self.assertEqual(set(info1.keys()),
-                         set(['user', 'nonce', 'hostname', 'pid', 'start_time']))
+        self.assertEqual(set(info1.info_dict.keys()),
+            set(['user', 'nonce', 'hostname', 'pid', 'start_time']))
         # should get the same info if we look at it through a different
         # instance
         info2 = LockDir(t, 'test_lock').peek()
@@ -165,7 +166,7 @@
         self.addCleanup(lf1.unlock)
         info2 = lf2.peek()
         self.assertTrue(info2)
-        self.assertEqual(info2['nonce'], lf1.nonce)
+        self.assertEqual(info2.get('nonce'), lf1.nonce)
 
     def test_30_lock_wait_fail(self):
         """Wait on a lock, then fail
@@ -188,24 +189,16 @@
             # it should only take about 0.4 seconds, but we allow more time in
             # case the machine is heavily loaded
             self.assertTrue(after - before <= 8.0,
-                    "took %f seconds to detect lock contention" % (after - before))
+                "took %f seconds to detect lock contention" % (after - before))
         finally:
             lf1.unlock()
         self.assertEqual(1, len(self._logged_reports))
-        self.assertEqual(self._logged_reports[0][0],
-            '%s lock %s held by %s\n'
-            'at %s [process #%s], acquired %s.\n'
-            'Will continue to try until %s, unless '
-            'you press Ctrl-C.\n'
-            'See "bzr help break-lock" for more.')
-        start, lock_url, user, hostname, pid, time_ago, deadline_str = \
-            self._logged_reports[0][1]
-        self.assertEqual(start, u'Unable to obtain')
-        self.assertEqual(user, u'jrandom@example.com')
-        # skip hostname
-        self.assertContainsRe(pid, r'\d+')
-        self.assertContainsRe(time_ago, r'.* ago')
-        self.assertContainsRe(deadline_str, r'\d{2}:\d{2}:\d{2}')
+        self.assertContainsRe(self._logged_reports[0][0],
+            r'Unable to obtain lock .* held by jrandom@example\.com on .*'
+            r' \(process #\d+\), acquired .* ago\.\n'
+            r'Will continue to try until \d{2}:\d{2}:\d{2}, unless '
+            r'you press Ctrl-C.\n'
+            r'See "bzr help break-lock" for more.')
 
     def test_31_lock_wait_easy(self):
         """Succeed when waiting on a lock with no contention.
@@ -223,226 +216,6 @@
             lf1.unlock()
         self.assertEqual([], self._logged_reports)
 
-    def test_32_lock_wait_succeed(self):
-        """Succeed when trying to acquire a lock that gets released
-
-        One thread holds on a lock and then releases it; another
-        tries to lock it.
-        """
-        # This test sometimes fails like this:
-        # Traceback (most recent call last):
-
-        #   File "/home/pqm/bzr-pqm-workdir/home/+trunk/bzrlib/tests/
-        # test_lockdir.py", line 247, in test_32_lock_wait_succeed
-        #     self.assertEqual(1, len(self._logged_reports))
-        # AssertionError: not equal:
-        # a = 1
-        # b = 0
-        raise tests.TestSkipped("Test fails intermittently")
-        t = self.get_transport()
-        lf1 = LockDir(t, 'test_lock')
-        lf1.create()
-        lf1.attempt_lock()
-
-        def wait_and_unlock():
-            time.sleep(0.1)
-            lf1.unlock()
-        unlocker = Thread(target=wait_and_unlock)
-        unlocker.start()
-        try:
-            lf2 = LockDir(t, 'test_lock')
-            self.setup_log_reporter(lf2)
-            before = time.time()
-            # wait and then lock
-            lf2.wait_lock(timeout=0.4, poll=0.1)
-            after = time.time()
-            self.assertTrue(after - before <= 1.0)
-        finally:
-            unlocker.join()
-
-        # There should be only 1 report, even though it should have to
-        # wait for a while
-        lock_base = lf2.transport.abspath(lf2.path)
-        self.assertEqual(1, len(self._logged_reports))
-        self.assertEqual('%s %s\n'
-                         '%s\n%s\n'
-                         'Will continue to try until %s\n',
-                         self._logged_reports[0][0])
-        args = self._logged_reports[0][1]
-        self.assertEqual('Unable to obtain', args[0])
-        self.assertEqual('lock %s' % (lock_base,), args[1])
-        self.assertStartsWith(args[2], 'held by ')
-        self.assertStartsWith(args[3], 'locked ')
-        self.assertEndsWith(args[3], ' ago')
-        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
-
-    def test_34_lock_write_waits(self):
-        """LockDir.lock_write() will wait for the lock."""
-        # the test suite sets the default to 0 to make deadlocks fail fast.
-        # change it for this test, as we want to try a manual deadlock.
-        raise tests.TestSkipped('Timing-sensitive test')
-        bzrlib.lockdir._DEFAULT_TIMEOUT_SECONDS = 300
-        t = self.get_transport()
-        lf1 = LockDir(t, 'test_lock')
-        lf1.create()
-        lf1.attempt_lock()
-
-        def wait_and_unlock():
-            time.sleep(0.1)
-            lf1.unlock()
-        unlocker = Thread(target=wait_and_unlock)
-        unlocker.start()
-        try:
-            lf2 = LockDir(t, 'test_lock')
-            self.setup_log_reporter(lf2)
-            before = time.time()
-            # wait and then lock
-            lf2.lock_write()
-            after = time.time()
-        finally:
-            unlocker.join()
-
-        # There should be only 1 report, even though it should have to
-        # wait for a while
-        lock_base = lf2.transport.abspath(lf2.path)
-        self.assertEqual(1, len(self._logged_reports))
-        self.assertEqual('%s %s\n'
-                         '%s\n%s\n'
-                         'Will continue to try until %s\n',
-                         self._logged_reports[0][0])
-        args = self._logged_reports[0][1]
-        self.assertEqual('Unable to obtain', args[0])
-        self.assertEqual('lock %s' % (lock_base,), args[1])
-        self.assertStartsWith(args[2], 'held by ')
-        self.assertStartsWith(args[3], 'locked ')
-        self.assertEndsWith(args[3], ' ago')
-        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
-
-    def test_35_wait_lock_changing(self):
-        """LockDir.wait_lock() will report if the lock changes underneath.
-
-        This is the stages we want to happen:
-
-        0) Synchronization locks are created and locked.
-        1) Lock1 obtains the lockdir, and releases the 'check' lock.
-        2) Lock2 grabs the 'check' lock, and checks the lockdir.
-           It sees the lockdir is already acquired, reports the fact,
-           and unsets the 'checked' lock.
-        3) Thread1 blocks on acquiring the 'checked' lock, and then tells
-           Lock1 to release and acquire the lockdir. This resets the 'check'
-           lock.
-        4) Lock2 acquires the 'check' lock, and checks again. It notices
-           that the holder of the lock has changed, and so reports a new
-           lock holder.
-        5) Thread1 blocks on the 'checked' lock, this time, it completely
-           unlocks the lockdir, allowing Lock2 to acquire the lock.
-        """
-
-        raise tests.KnownFailure(
-            "timing dependency in lock tests (#213182)")
-
-        wait_to_check_lock = Lock()
-        wait_until_checked_lock = Lock()
-
-        wait_to_check_lock.acquire()
-        wait_until_checked_lock.acquire()
-        note('locked check and checked locks')
-
-        class LockDir1(LockDir):
-            """Use the synchronization points for the first lock."""
-
-            def attempt_lock(self):
-                # Once we have acquired the lock, it is okay for
-                # the other lock to check it
-                try:
-                    return super(LockDir1, self).attempt_lock()
-                finally:
-                    note('lock1: releasing check lock')
-                    wait_to_check_lock.release()
-
-        class LockDir2(LockDir):
-            """Use the synchronization points for the second lock."""
-
-            def attempt_lock(self):
-                note('lock2: waiting for check lock')
-                wait_to_check_lock.acquire()
-                note('lock2: acquired check lock')
-                try:
-                    return super(LockDir2, self).attempt_lock()
-                finally:
-                    note('lock2: releasing checked lock')
-                    wait_until_checked_lock.release()
-
-        t = self.get_transport()
-        lf1 = LockDir1(t, 'test_lock')
-        lf1.create()
-
-        lf2 = LockDir2(t, 'test_lock')
-        self.setup_log_reporter(lf2)
-
-        def wait_and_switch():
-            lf1.attempt_lock()
-            # Block until lock2 has had a chance to check
-            note('lock1: waiting 1 for checked lock')
-            wait_until_checked_lock.acquire()
-            note('lock1: acquired for checked lock')
-            note('lock1: released lockdir')
-            lf1.unlock()
-            note('lock1: acquiring lockdir')
-            # Create a new nonce, so the lock looks different.
-            lf1.nonce = osutils.rand_chars(20)
-            lf1.lock_write()
-            note('lock1: acquired lockdir')
-
-            # Block until lock2 has peeked again
-            note('lock1: waiting 2 for checked lock')
-            wait_until_checked_lock.acquire()
-            note('lock1: acquired for checked lock')
-            # Now unlock, and let lock 2 grab the lock
-            lf1.unlock()
-            wait_to_check_lock.release()
-
-        unlocker = Thread(target=wait_and_switch)
-        unlocker.start()
-        try:
-            # Wait and play against the other thread
-            lf2.wait_lock(timeout=20.0, poll=0.01)
-        finally:
-            unlocker.join()
-        lf2.unlock()
-
-        # There should be 2 reports, because the lock changed
-        lock_base = lf2.transport.abspath(lf2.path)
-        self.assertEqual(2, len(self._logged_reports))
-        lock_url = lf2.transport.abspath(lf2.path)
-        self.assertEqual('%s %s\n'
-                         '%s\n%s\n'
-                         'Will continue to try until %s, unless '
-                         'you press Ctrl-C.\n'
-                         'See "bzr help break-lock" for more.',
-                         self._logged_reports[0][0])
-        args = self._logged_reports[0][1]
-        self.assertEqual('Unable to obtain', args[0])
-        self.assertEqual('lock %s' % (lock_base,), args[1])
-        self.assertStartsWith(args[2], 'held by ')
-        self.assertStartsWith(args[3], 'locked ')
-        self.assertEndsWith(args[3], ' ago')
-        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
-
-        self.assertEqual('%s %s\n'
-                         '%s\n%s\n'
-                         'Will continue to try until %s, unless '
-                         'you press Ctrl-C.\n'
-                         'See "bzr help break-lock" for more.',
-                         self._logged_reports[1][0])
-        args = self._logged_reports[1][1]
-        self.assertEqual('Lock owner changed for', args[0])
-        self.assertEqual('lock %s' % (lock_base,), args[1])
-        self.assertStartsWith(args[2], 'held by ')
-        self.assertStartsWith(args[3], 'locked ')
-        self.assertEndsWith(args[3], ' ago')
-        self.assertContainsRe(args[4], r'\d\d:\d\d:\d\d')
-
     def test_40_confirm_easy(self):
         """Confirm a lock that's already held"""
         t = self.get_transport()
@@ -564,6 +337,64 @@
         finally:
             bzrlib.ui.ui_factory = orig_factory
 
+    def test_break_lock_corrupt_info(self):
+        """break_lock works even if the info file is corrupt (and tells the UI
+        that it is corrupt).
+        """
+        ld = self.get_lock()
+        ld2 = self.get_lock()
+        ld.create()
+        ld.lock_write()
+        ld.transport.put_bytes_non_atomic('test_lock/held/info', '\0')
+
+        class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
+            def __init__(self):
+                self.prompts = []
+
+            def get_boolean(self, prompt):
+                self.prompts.append(('boolean', prompt))
+                return True
+
+        ui = LoggingUIFactory()
+        self.overrideAttr(bzrlib.ui, 'ui_factory', ui)
+        ld2.break_lock()
+        self.assertLength(1, ui.prompts)
+        self.assertEqual('boolean', ui.prompts[0][0])
+        self.assertStartsWith(ui.prompts[0][1], 'Break (corrupt LockDir')
+        self.assertRaises(LockBroken, ld.unlock)
+
+    def test_break_lock_missing_info(self):
+        """break_lock works even if the info file is missing (and tells the UI
+        that it is corrupt).
+        """
+        ld = self.get_lock()
+        ld2 = self.get_lock()
+        ld.create()
+        ld.lock_write()
+        ld.transport.delete('test_lock/held/info')
+
+        class LoggingUIFactory(bzrlib.ui.SilentUIFactory):
+            def __init__(self):
+                self.prompts = []
+
+            def get_boolean(self, prompt):
+                self.prompts.append(('boolean', prompt))
+                return True
+
+        ui = LoggingUIFactory()
+        orig_factory = bzrlib.ui.ui_factory
+        bzrlib.ui.ui_factory = ui
+        try:
+            ld2.break_lock()
+            self.assertRaises(LockBroken, ld.unlock)
+            self.assertLength(0, ui.prompts)
+        finally:
+            bzrlib.ui.ui_factory = orig_factory
+        # Suppress warnings due to ld not being unlocked
+        # XXX: if lock_broken hook was invoked in this case, this hack would
+        # not be necessary.  - Andrew Bennetts, 2010-09-06.
+        del self._lock_actions[:]
+
     def test_create_missing_base_directory(self):
         """If LockDir.path doesn't exist, it can be created
 
@@ -575,31 +406,30 @@
         lf1 = LockDir(t, 'test_lock')
 
         lf1.create()
-        self.failUnless(t.has('test_lock'))
+        self.assertTrue(t.has('test_lock'))
 
         t.rmdir('test_lock')
-        self.failIf(t.has('test_lock'))
+        self.assertFalse(t.has('test_lock'))
 
         # This will create 'test_lock' if it needs to
         lf1.lock_write()
-        self.failUnless(t.has('test_lock'))
-        self.failUnless(t.has('test_lock/held/info'))
+        self.assertTrue(t.has('test_lock'))
+        self.assertTrue(t.has('test_lock/held/info'))
 
         lf1.unlock()
-        self.failIf(t.has('test_lock/held/info'))
+        self.assertFalse(t.has('test_lock/held/info'))
 
-    def test__format_lock_info(self):
+    def test_display_form(self):
         ld1 = self.get_lock()
         ld1.create()
         ld1.lock_write()
         try:
-            info_list = ld1._format_lock_info(ld1.peek())
+            info_list = ld1.peek().to_readable_dict()
         finally:
             ld1.unlock()
-        self.assertEqual(info_list[0], u'jrandom@example.com')
-        # info_list[1] is hostname. we skip this.
-        self.assertContainsRe(info_list[2], '^\d+$') # pid
-        self.assertContainsRe(info_list[3], r'^\d+ seconds? ago$') # time_ago
+        self.assertEqual(info_list['user'], u'jrandom@example.com')
+        self.assertContainsRe(info_list['pid'], '^\d+$')
+        self.assertContainsRe(info_list['time_ago'], r'^\d+ seconds? ago$')
 
     def test_lock_without_email(self):
         global_config = config.GlobalConfig()
@@ -611,6 +441,7 @@
         ld1.unlock()
 
     def test_lock_permission(self):
+        self.requireFeature(features.not_running_as_root)
         if not osutils.supports_posix_readonly():
             raise tests.TestSkipped('Cannot induce a permission failure')
         ld1 = self.get_lock()
@@ -632,7 +463,8 @@
     def test_lock_with_buggy_rename(self):
         # test that lock acquisition handles servers which pretend they
         # renamed correctly but that actually fail
-        t = transport.get_transport('brokenrename+' + self.get_url())
+        t = transport.get_transport_from_url(
+            'brokenrename+' + self.get_url())
         ld1 = LockDir(t, 'test_lock')
         ld1.create()
         ld1.attempt_lock()
@@ -652,8 +484,10 @@
         # should be nothing before we start
         ld1.create()
         t = self.get_transport().clone('test_lock')
+
         def check_dir(a):
             self.assertEquals(a, t.list_dir('.'))
+
         check_dir([])
         # when held, that's all we see
         ld1.attempt_lock()
@@ -676,11 +510,57 @@
         t.put_bytes('test_lock/held/info', '')
         lf = LockDir(t, 'test_lock')
         info = lf.peek()
-        formatted_info = lf._format_lock_info(info)
+        formatted_info = info.to_readable_dict()
         self.assertEquals(
-            ['<unknown>', '<unknown>', '<unknown>', '(unknown)'],
+            dict(user='<unknown>', hostname='<unknown>', pid='<unknown>',
+                time_ago='(unknown)'),
             formatted_info)
 
+    def test_corrupt_lockdir_info(self):
+        """We can cope with corrupt (and thus unparseable) info files."""
+        # This seems like a fairly common failure case too - see
+        # <https://bugs.launchpad.net/bzr/+bug/619872> for instance.
+        # In particular some systems tend to fill recently created files with
+        # nul bytes after recovering from a system crash.
+        t = self.get_transport()
+        t.mkdir('test_lock')
+        t.mkdir('test_lock/held')
+        t.put_bytes('test_lock/held/info', '\0')
+        lf = LockDir(t, 'test_lock')
+        self.assertRaises(errors.LockCorrupt, lf.peek)
+        # Currently attempt_lock gives LockContention, but LockCorrupt would be
+        # a reasonable result too.
+        self.assertRaises(
+            (errors.LockCorrupt, errors.LockContention), lf.attempt_lock)
+        self.assertRaises(errors.LockCorrupt, lf.validate_token, 'fake token')
+
+    def test_missing_lockdir_info(self):
+        """We can cope with absent info files."""
+        t = self.get_transport()
+        t.mkdir('test_lock')
+        t.mkdir('test_lock/held')
+        lf = LockDir(t, 'test_lock')
+        # In this case we expect the 'not held' result from peek, because peek
+        # cannot be expected to notice that there is a 'held' directory with no
+        # 'info' file.
+        self.assertEqual(None, lf.peek())
+        # And lock/unlock may work or give LockContention (but not any other
+        # error).
+        try:
+            lf.attempt_lock()
+        except LockContention:
+            # LockContention is ok, and expected on Windows
+            pass
+        else:
+            # no error is ok, and expected on POSIX (because POSIX allows
+            # os.rename over an empty directory).
+            lf.unlock()
+        # Currently raises TokenMismatch, but LockCorrupt would be reasonable
+        # too.
+        self.assertRaises(
+            (errors.TokenMismatch, errors.LockCorrupt),
+            lf.validate_token, 'fake token')
+
 
 class TestLockDirHooks(TestCaseWithTransport):
 
@@ -772,3 +652,109 @@
         ld2.force_break(holder_info)
         lock_path = ld.transport.abspath(ld.path)
         self.assertEqual([], self._calls)
+
+
+class TestLockHeldInfo(TestCase):
+    """Can get information about the lock holder, and detect whether they're
+    still alive."""
+
+    def test_repr(self):
+        info = LockHeldInfo.for_this_process(None)
+        self.assertContainsRe(repr(info), r"LockHeldInfo\(.*\)")
+
+    def test_unicode(self):
+        info = LockHeldInfo.for_this_process(None)
+        self.assertContainsRe(unicode(info),
+            r'held by .* on .* \(process #\d+\), acquired .* ago')
+
+    def test_is_locked_by_this_process(self):
+        info = LockHeldInfo.for_this_process(None)
+        self.assertTrue(info.is_locked_by_this_process())
+
+    def test_is_not_locked_by_this_process(self):
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['pid'] = '123123123123123'
+        self.assertFalse(info.is_locked_by_this_process())
+
+    def test_lock_holder_live_process(self):
+        """Detect that the holder (this process) is still running."""
+        info = LockHeldInfo.for_this_process(None)
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+    def test_lock_holder_dead_process(self):
+        """Detect that the holder (this process) is still running."""
+        self.overrideAttr(lockdir, 'get_host_name',
+            lambda: 'aproperhostname')
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['pid'] = '123123123'
+        self.assertTrue(info.is_lock_holder_known_dead())
+
+    def test_lock_holder_other_machine(self):
+        """The lock holder isn't here so we don't know if they're alive."""
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['hostname'] = 'egg.example.com'
+        info.info_dict['pid'] = '123123123'
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+    def test_lock_holder_other_user(self):
+        """Only auto-break locks held by this user."""
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['user'] = 'notme@example.com'
+        info.info_dict['pid'] = '123123123'
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+    def test_no_good_hostname(self):
+        """Correctly handle ambiguous hostnames.
+
+        If the lock's recorded with just 'localhost' we can't really trust
+        it's the same 'localhost'.  (There are quite a few of them. :-)
+        So even if the process is known not to be alive, we can't say that's
+        known for sure.
+        """
+        self.overrideAttr(lockdir, 'get_host_name',
+            lambda: 'localhost')
+        info = LockHeldInfo.for_this_process(None)
+        info.info_dict['pid'] = '123123123'
+        self.assertFalse(info.is_lock_holder_known_dead())
+
+
+class TestStaleLockDir(TestCaseWithTransport):
+    """Can automatically break stale locks.
+
+    :see: https://bugs.launchpad.net/bzr/+bug/220464
+    """
+
+    def test_auto_break_stale_lock(self):
+        """Locks safely known to be stale are just cleaned up.
+
+        This generates a warning but no other user interaction.
+        """
+        self.overrideAttr(lockdir, 'get_host_name',
+            lambda: 'aproperhostname')
+        # This is off by default at present; see the discussion in the bug.
+        # If you change the default, don't forget to update the docs.
+        config.GlobalConfig().set_user_option('locks.steal_dead', True)
+        # Create a lock pretending to come from a different nonexistent
+        # process on the same machine.
+        l1 = LockDir(self.get_transport(), 'a',
+            extra_holder_info={'pid': '12312313'})
+        token_1 = l1.attempt_lock()
+        l2 = LockDir(self.get_transport(), 'a')
+        token_2 = l2.attempt_lock()
+        # l1 will notice its lock was stolen.
+        self.assertRaises(errors.LockBroken,
+            l1.unlock)
+        l2.unlock()
+
+    def test_auto_break_stale_lock_configured_off(self):
+        """Automatic breaking can be turned off"""
+        l1 = LockDir(self.get_transport(), 'a',
+            extra_holder_info={'pid': '12312313'})
+        token_1 = l1.attempt_lock()
+        self.addCleanup(l1.unlock)
+        l2 = LockDir(self.get_transport(), 'a')
+        # This fails now, because dead lock breaking is off by default.
+        self.assertRaises(LockContention,
+            l2.attempt_lock)
+        # and it's in fact not broken
+        l1.confirm()

=== modified file 'bzrlib/tests/test_log.py'
--- bzrlib/tests/test_log.py	2010-06-08 09:50:27 +0000
+++ bzrlib/tests/test_log.py	2011-09-19 15:22:23 +0000
@@ -117,6 +117,7 @@
             branch.tags.set_tag('v1.0', 'rev-3')
         return wt
 
+
 class LogCatcher(log.LogFormatter):
     """Pull log messages into a list rather than displaying them.
 
@@ -322,7 +323,7 @@
     1 Joe Foo\t2005-11-22
       rev-1
 
-Use --include-merges or -n0 to see merged revisions.
+Use --include-merged or -n0 to see merged revisions.
 """,
             wt.branch, log.ShortLogFormatter,
             formatter_kwargs=dict(show_advice=True))
@@ -371,6 +372,32 @@
             wt.branch, log.ShortLogFormatter,
             show_log_kwargs=dict(start_revision=rev, end_revision=rev))
 
+    def test_show_ids(self):
+        wt = self.make_branch_and_tree('parent')
+        self.build_tree(['parent/f1', 'parent/f2'])
+        wt.add(['f1','f2'])
+        self.wt_commit(wt, 'first post', rev_id='a')
+        child_wt = wt.bzrdir.sprout('child').open_workingtree()
+        self.wt_commit(child_wt, 'branch 1 changes', rev_id='b')
+        wt.merge_from_branch(child_wt.branch)
+        self.wt_commit(wt, 'merge branch 1', rev_id='c')
+        self.assertFormatterResult("""\
+    2 Joe Foo\t2005-11-22 [merge]
+      revision-id:c
+      merge branch 1
+
+          1.1.1 Joe Foo\t2005-11-22
+                revision-id:b
+                branch 1 changes
+
+    1 Joe Foo\t2005-11-22
+      revision-id:a
+      first post
+
+""",
+            wt.branch, log.ShortLogFormatter,
+            formatter_kwargs=dict(levels=0,show_ids=True))
+
 
 class TestShortLogFormatterWithMergeRevisions(TestCaseForLogFormatter):
 
@@ -659,6 +686,47 @@
         self.assertEqualDiff('''custom_prop_name: test_value\n''',
                              sio.getvalue())
 
+    def test_show_ids(self):
+        wt = self.make_branch_and_tree('parent')
+        self.build_tree(['parent/f1', 'parent/f2'])
+        wt.add(['f1','f2'])
+        self.wt_commit(wt, 'first post', rev_id='a')
+        child_wt = wt.bzrdir.sprout('child').open_workingtree()
+        self.wt_commit(child_wt, 'branch 1 changes', rev_id='b')
+        wt.merge_from_branch(child_wt.branch)
+        self.wt_commit(wt, 'merge branch 1', rev_id='c')
+        self.assertFormatterResult("""\
+------------------------------------------------------------
+revno: 2 [merge]
+revision-id: c
+parent: a
+parent: b
+committer: Joe Foo <joe@foo.com>
+branch nick: parent
+timestamp: Tue 2005-11-22 00:00:02 +0000
+message:
+  merge branch 1
+    ------------------------------------------------------------
+    revno: 1.1.1
+    revision-id: b
+    parent: a
+    committer: Joe Foo <joe@foo.com>
+    branch nick: child
+    timestamp: Tue 2005-11-22 00:00:01 +0000
+    message:
+      branch 1 changes
+------------------------------------------------------------
+revno: 1
+revision-id: a
+committer: Joe Foo <joe@foo.com>
+branch nick: parent
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  first post
+""",
+            wt.branch, log.LongLogFormatter,
+            formatter_kwargs=dict(levels=0,show_ids=True))
+
 
 class TestLongLogFormatterWithoutMergeRevisions(TestCaseForLogFormatter):
 
@@ -915,349 +983,6 @@
             wt.branch, log.GnuChangelogLogFormatter,
             show_log_kwargs=dict(verbose=True))
 
-class TestGetViewRevisions(tests.TestCaseWithTransport, TestLogMixin):
-
-    def _get_view_revisions(self, *args, **kwargs):
-        return self.applyDeprecated(symbol_versioning.deprecated_in((2, 2, 0)),
-                                    log.get_view_revisions, *args, **kwargs)
-
-    def make_tree_with_commits(self):
-        """Create a tree with well-known revision ids"""
-        wt = self.make_branch_and_tree('tree1')
-        self.wt_commit(wt, 'commit one', rev_id='1')
-        self.wt_commit(wt, 'commit two', rev_id='2')
-        self.wt_commit(wt, 'commit three', rev_id='3')
-        mainline_revs = [None, '1', '2', '3']
-        rev_nos = {'1': 1, '2': 2, '3': 3}
-        return mainline_revs, rev_nos, wt
-
-    def make_tree_with_merges(self):
-        """Create a tree with well-known revision ids and a merge"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_commits()
-        tree2 = wt.bzrdir.sprout('tree2').open_workingtree()
-        self.wt_commit(tree2, 'four-a', rev_id='4a')
-        wt.merge_from_branch(tree2.branch)
-        self.wt_commit(wt, 'four-b', rev_id='4b')
-        mainline_revs.append('4b')
-        rev_nos['4b'] = 4
-        # 4a: 3.1.1
-        return mainline_revs, rev_nos, wt
-
-    def make_branch_with_many_merges(self):
-        """Create a tree with well-known revision ids"""
-        builder = self.make_branch_builder('tree1')
-        builder.start_series()
-        builder.build_snapshot('1', None, [
-            ('add', ('', 'TREE_ROOT', 'directory', '')),
-            ('add', ('f', 'f-id', 'file', '1\n'))])
-        builder.build_snapshot('2', ['1'], [])
-        builder.build_snapshot('3a', ['2'], [
-            ('modify', ('f-id', '1\n2\n3a\n'))])
-        builder.build_snapshot('3b', ['2', '3a'], [
-            ('modify', ('f-id', '1\n2\n3a\n'))])
-        builder.build_snapshot('3c', ['2', '3b'], [
-            ('modify', ('f-id', '1\n2\n3a\n'))])
-        builder.build_snapshot('4a', ['3b'], [])
-        builder.build_snapshot('4b', ['3c', '4a'], [])
-        builder.finish_series()
-
-        # 1
-        # |
-        # 2-.
-        # |\ \
-        # | | 3a
-        # | |/
-        # | 3b
-        # |/|
-        # 3c4a
-        # |/
-        # 4b
-
-        mainline_revs = [None, '1', '2', '3c', '4b']
-        rev_nos = {'1':1, '2':2, '3c': 3, '4b':4}
-        full_rev_nos_for_reference = {
-            '1': '1',
-            '2': '2',
-            '3a': '2.1.1', #first commit tree 3
-            '3b': '2.2.1', # first commit tree 2
-            '3c': '3', #merges 3b to main
-            '4a': '2.2.2', # second commit tree 2
-            '4b': '4', # merges 4a to main
-            }
-        return mainline_revs, rev_nos, builder.get_branch()
-
-    def test_get_view_revisions_forward(self):
-        """Test the get_view_revisions method"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_commits()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward'))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3', '3', 0)],
-                         revisions)
-        revisions2 = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward',
-                include_merges=False))
-        self.assertEqual(revisions, revisions2)
-
-    def test_get_view_revisions_reverse(self):
-        """Test the get_view_revisions with reverse"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_commits()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse'))
-        self.assertEqual([('3', '3', 0), ('2', '2', 0), ('1', '1', 0), ],
-                         revisions)
-        revisions2 = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse',
-                include_merges=False))
-        self.assertEqual(revisions, revisions2)
-
-    def test_get_view_revisions_merge(self):
-        """Test get_view_revisions when there are merges"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_merges()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward'))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3', '3', 0),
-                          ('4b', '4', 0), ('4a', '3.1.1', 1)],
-                         revisions)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'forward',
-                include_merges=False))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3', '3', 0),
-                          ('4b', '4', 0)],
-                         revisions)
-
-    def test_get_view_revisions_merge_reverse(self):
-        """Test get_view_revisions in reverse when there are merges"""
-        mainline_revs, rev_nos, wt = self.make_tree_with_merges()
-        wt.lock_read()
-        self.addCleanup(wt.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse'))
-        self.assertEqual([('4b', '4', 0), ('4a', '3.1.1', 1),
-                          ('3', '3', 0), ('2', '2', 0), ('1', '1', 0)],
-                         revisions)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, wt.branch, 'reverse',
-                include_merges=False))
-        self.assertEqual([('4b', '4', 0), ('3', '3', 0), ('2', '2', 0),
-                          ('1', '1', 0)],
-                         revisions)
-
-    def test_get_view_revisions_merge2(self):
-        """Test get_view_revisions when there are merges"""
-        mainline_revs, rev_nos, b = self.make_branch_with_many_merges()
-        b.lock_read()
-        self.addCleanup(b.unlock)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, b, 'forward'))
-        expected = [('1', '1', 0), ('2', '2', 0), ('3c', '3', 0),
-                    ('3b', '2.2.1', 1), ('3a', '2.1.1', 2), ('4b', '4', 0),
-                    ('4a', '2.2.2', 1)]
-        self.assertEqual(expected, revisions)
-        revisions = list(self._get_view_revisions(
-                mainline_revs, rev_nos, b, 'forward',
-                include_merges=False))
-        self.assertEqual([('1', '1', 0), ('2', '2', 0), ('3c', '3', 0),
-                          ('4b', '4', 0)],
-                         revisions)
-
-    def test_file_id_for_range(self):
-        mainline_revs, rev_nos, b = self.make_branch_with_many_merges()
-        b.lock_read()
-        self.addCleanup(b.unlock)
-
-        def rev_from_rev_id(revid, branch):
-            revspec = revisionspec.RevisionSpec.from_string('revid:%s' % revid)
-            return revspec.in_history(branch)
-
-        def view_revs(start_rev, end_rev, file_id, direction):
-            revs = self.applyDeprecated(
-                symbol_versioning.deprecated_in((2, 2, 0)),
-                log.calculate_view_revisions,
-                b,
-                start_rev, # start_revision
-                end_rev, # end_revision
-                direction, # direction
-                file_id, # specific_fileid
-                True, # generate_merge_revisions
-                )
-            return revs
-
-        rev_3a = rev_from_rev_id('3a', b)
-        rev_4b = rev_from_rev_id('4b', b)
-        self.assertEqual([('3c', '3', 0), ('3b', '2.2.1', 1),
-                          ('3a', '2.1.1', 2)],
-                          view_revs(rev_3a, rev_4b, 'f-id', 'reverse'))
-        # Note: 3c still appears before 3a here because of depth-based sorting
-        self.assertEqual([('3c', '3', 0), ('3b', '2.2.1', 1),
-                          ('3a', '2.1.1', 2)],
-                          view_revs(rev_3a, rev_4b, 'f-id', 'forward'))
-
-
-class TestGetRevisionsTouchingFileID(tests.TestCaseWithTransport):
-
-    def get_view_revisions(self, *args):
-        return self.applyDeprecated(symbol_versioning.deprecated_in((2, 2, 0)),
-                                    log.get_view_revisions, *args)
-
-    def create_tree_with_single_merge(self):
-        """Create a branch with a moderate layout.
-
-        The revision graph looks like:
-
-           A
-           |\
-           B C
-           |/
-           D
-
-        In this graph, A introduced files f1 and f2 and f3.
-        B modifies f1 and f3, and C modifies f2 and f3.
-        D merges the changes from B and C and resolves the conflict for f3.
-        """
-        # TODO: jam 20070218 This seems like it could really be done
-        #       with make_branch_and_memory_tree() if we could just
-        #       create the content of those files.
-        # TODO: jam 20070218 Another alternative is that we would really
-        #       like to only create this tree 1 time for all tests that
-        #       use it. Since 'log' only uses the tree in a readonly
-        #       fashion, it seems a shame to regenerate an identical
-        #       tree for each test.
-        # TODO: vila 20100122 One way to address the shame above will be to
-        #       create a memory tree during test parametrization and give a
-        #       *copy* of this tree to each test. Copying a memory tree ought
-        #       to be cheap, at least cheaper than creating them with such
-        #       complex setups.
-        tree = self.make_branch_and_tree('tree')
-        tree.lock_write()
-        self.addCleanup(tree.unlock)
-
-        self.build_tree_contents([('tree/f1', 'A\n'),
-                                  ('tree/f2', 'A\n'),
-                                  ('tree/f3', 'A\n'),
-                                 ])
-        tree.add(['f1', 'f2', 'f3'], ['f1-id', 'f2-id', 'f3-id'])
-        tree.commit('A', rev_id='A')
-
-        self.build_tree_contents([('tree/f2', 'A\nC\n'),
-                                  ('tree/f3', 'A\nC\n'),
-                                 ])
-        tree.commit('C', rev_id='C')
-        # Revert back to A to build the other history.
-        tree.set_last_revision('A')
-        tree.branch.set_last_revision_info(1, 'A')
-        self.build_tree_contents([('tree/f1', 'A\nB\n'),
-                                  ('tree/f2', 'A\n'),
-                                  ('tree/f3', 'A\nB\n'),
-                                 ])
-        tree.commit('B', rev_id='B')
-        tree.set_parent_ids(['B', 'C'])
-        self.build_tree_contents([('tree/f1', 'A\nB\n'),
-                                  ('tree/f2', 'A\nC\n'),
-                                  ('tree/f3', 'A\nB\nC\n'),
-                                 ])
-        tree.commit('D', rev_id='D')
-
-        # Switch to a read lock for this tree.
-        # We still have an addCleanup(tree.unlock) pending
-        tree.unlock()
-        tree.lock_read()
-        return tree
-
-    def check_delta(self, delta, **kw):
-        """Check the filenames touched by a delta are as expected.
-
-        Caller only have to pass in the list of files for each part, all
-        unspecified parts are considered empty (and checked as such).
-        """
-        for n in 'added', 'removed', 'renamed', 'modified', 'unchanged':
-            # By default we expect an empty list
-            expected = kw.get(n, [])
-            # strip out only the path components
-            got = [x[0] for x in getattr(delta, n)]
-            self.assertEqual(expected, got)
-
-    def test_tree_with_single_merge(self):
-        """Make sure the tree layout is correct."""
-        tree = self.create_tree_with_single_merge()
-        rev_A_tree = tree.branch.repository.revision_tree('A')
-        rev_B_tree = tree.branch.repository.revision_tree('B')
-        rev_C_tree = tree.branch.repository.revision_tree('C')
-        rev_D_tree = tree.branch.repository.revision_tree('D')
-
-        self.check_delta(rev_B_tree.changes_from(rev_A_tree),
-                         modified=['f1', 'f3'])
-
-        self.check_delta(rev_C_tree.changes_from(rev_A_tree),
-                         modified=['f2', 'f3'])
-
-        self.check_delta(rev_D_tree.changes_from(rev_B_tree),
-                         modified=['f2', 'f3'])
-
-        self.check_delta(rev_D_tree.changes_from(rev_C_tree),
-                         modified=['f1', 'f3'])
-
-    def assertAllRevisionsForFileID(self, tree, file_id, revisions):
-        """Ensure _filter_revisions_touching_file_id returns the right values.
-
-        Get the return value from _filter_revisions_touching_file_id and make
-        sure they are correct.
-        """
-        # The api for _filter_revisions_touching_file_id is a little crazy.
-        # So we do the setup here.
-        mainline = tree.branch.revision_history()
-        mainline.insert(0, None)
-        revnos = dict((rev, idx+1) for idx, rev in enumerate(mainline))
-        view_revs_iter = self.get_view_revisions(
-            mainline, revnos, tree.branch, 'reverse', True)
-        actual_revs = log._filter_revisions_touching_file_id(
-            tree.branch, file_id, list(view_revs_iter))
-        self.assertEqual(revisions, [r for r, revno, depth in actual_revs])
-
-    def test_file_id_f1(self):
-        tree = self.create_tree_with_single_merge()
-        # f1 should be marked as modified by revisions A and B
-        self.assertAllRevisionsForFileID(tree, 'f1-id', ['B', 'A'])
-
-    def test_file_id_f2(self):
-        tree = self.create_tree_with_single_merge()
-        # f2 should be marked as modified by revisions A, C, and D
-        # because D merged the changes from C.
-        self.assertAllRevisionsForFileID(tree, 'f2-id', ['D', 'C', 'A'])
-
-    def test_file_id_f3(self):
-        tree = self.create_tree_with_single_merge()
-        # f3 should be marked as modified by revisions A, B, C, and D
-        self.assertAllRevisionsForFileID(tree, 'f3-id', ['D', 'C', 'B', 'A'])
-
-    def test_file_id_with_ghosts(self):
-        # This is testing bug #209948, where having a ghost would cause
-        # _filter_revisions_touching_file_id() to fail.
-        tree = self.create_tree_with_single_merge()
-        # We need to add a revision, so switch back to a write-locked tree
-        # (still a single addCleanup(tree.unlock) pending).
-        tree.unlock()
-        tree.lock_write()
-        first_parent = tree.last_revision()
-        tree.set_parent_ids([first_parent, 'ghost-revision-id'])
-        self.build_tree_contents([('tree/f1', 'A\nB\nXX\n')])
-        tree.commit('commit with a ghost', rev_id='XX')
-        self.assertAllRevisionsForFileID(tree, 'f1-id', ['XX', 'B', 'A'])
-        self.assertAllRevisionsForFileID(tree, 'f2-id', ['D', 'C', 'A'])
-
-    def test_unknown_file_id(self):
-        tree = self.create_tree_with_single_merge()
-        self.assertAllRevisionsForFileID(tree, 'unknown', [])
-
-    def test_empty_branch_unknown_file_id(self):
-        tree = self.make_branch_and_tree('tree')
-        self.assertAllRevisionsForFileID(tree, 'unknown', [])
-
 
 class TestShowChangedRevisions(tests.TestCaseWithTransport):
 
@@ -1463,6 +1188,124 @@
         self.assertNotContainsRe(s.getvalue(), 'Added Revisions:')
 
 
+class TestRevisionNotInBranch(TestCaseForLogFormatter):
+
+    def setup_a_tree(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        kwargs = {
+            'committer': 'Joe Foo <joe@foo.com>',
+            'timestamp': 1132617600, # Mon 2005-11-22 00:00:00 +0000
+            'timezone': 0, # UTC
+        }
+        tree.commit('commit 1a', rev_id='1a', **kwargs)
+        tree.commit('commit 2a', rev_id='2a', **kwargs)
+        tree.commit('commit 3a', rev_id='3a', **kwargs)
+        return tree
+
+    def setup_ab_tree(self):
+        tree = self.setup_a_tree()
+        tree.set_last_revision('1a')
+        tree.branch.set_last_revision_info(1, '1a')
+        kwargs = {
+            'committer': 'Joe Foo <joe@foo.com>',
+            'timestamp': 1132617600, # Mon 2005-11-22 00:00:00 +0000
+            'timezone': 0, # UTC
+        }
+        tree.commit('commit 2b', rev_id='2b', **kwargs)
+        tree.commit('commit 3b', rev_id='3b', **kwargs)
+        return tree
+
+    def test_one_revision(self):
+        tree = self.setup_ab_tree()
+        lf = LogCatcher()
+        rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        log.show_log(tree.branch, lf, verbose=True, start_revision=rev,
+                     end_revision=rev)
+        self.assertEqual(1, len(lf.revisions))
+        self.assertEqual(None, lf.revisions[0].revno)   # Out-of-branch
+        self.assertEqual('3a', lf.revisions[0].rev.revision_id)
+
+    def test_many_revisions(self):
+        tree = self.setup_ab_tree()
+        lf = LogCatcher()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        log.show_log(tree.branch, lf, verbose=True, start_revision=start_rev,
+                     end_revision=end_rev)
+        self.assertEqual(3, len(lf.revisions))
+        self.assertEqual(None, lf.revisions[0].revno)   # Out-of-branch
+        self.assertEqual('3a', lf.revisions[0].rev.revision_id)
+        self.assertEqual(None, lf.revisions[1].revno)   # Out-of-branch
+        self.assertEqual('2a', lf.revisions[1].rev.revision_id)
+        self.assertEqual('1', lf.revisions[2].revno)    # In-branch
+
+    def test_long_format(self):
+        tree = self.setup_ab_tree()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        self.assertFormatterResult("""\
+------------------------------------------------------------
+revision-id: 3a
+committer: Joe Foo <joe@foo.com>
+branch nick: tree
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  commit 3a
+------------------------------------------------------------
+revision-id: 2a
+committer: Joe Foo <joe@foo.com>
+branch nick: tree
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  commit 2a
+------------------------------------------------------------
+revno: 1
+committer: Joe Foo <joe@foo.com>
+branch nick: tree
+timestamp: Tue 2005-11-22 00:00:00 +0000
+message:
+  commit 1a
+""",
+            tree.branch, log.LongLogFormatter, show_log_kwargs={
+                'start_revision': start_rev, 'end_revision': end_rev
+            })
+
+    def test_short_format(self):
+        tree = self.setup_ab_tree()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        self.assertFormatterResult("""\
+      Joe Foo\t2005-11-22
+      revision-id:3a
+      commit 3a
+
+      Joe Foo\t2005-11-22
+      revision-id:2a
+      commit 2a
+
+    1 Joe Foo\t2005-11-22
+      commit 1a
+
+""",
+            tree.branch, log.ShortLogFormatter, show_log_kwargs={
+                'start_revision': start_rev, 'end_revision': end_rev
+            })
+
+    def test_line_format(self):
+        tree = self.setup_ab_tree()
+        start_rev = revisionspec.RevisionInfo(tree.branch, None, '1a')
+        end_rev = revisionspec.RevisionInfo(tree.branch, None, '3a')
+        self.assertFormatterResult("""\
+Joe Foo 2005-11-22 commit 3a
+Joe Foo 2005-11-22 commit 2a
+1: Joe Foo 2005-11-22 commit 1a
+""",
+            tree.branch, log.LineLogFormatter, show_log_kwargs={
+                'start_revision': start_rev, 'end_revision': end_rev
+            })
+
 
 class TestLogWithBugs(TestCaseForLogFormatter, TestLogMixin):
 
@@ -1492,7 +1335,7 @@
         self.assertFormatterResult("""\
 ------------------------------------------------------------
 revno: 2
-fixes bug(s): test://bug/id test://bug/2
+fixes bugs: test://bug/id test://bug/2
 author: Joe Bar <joe@bar.com>
 committer: Joe Foo <joe@foo.com>
 branch nick: work
@@ -1503,7 +1346,7 @@
   message
 ------------------------------------------------------------
 revno: 1
-fixes bug(s): test://bug/id
+fixes bug: test://bug/id
 committer: Joe Foo <joe@foo.com>
 branch nick: work
 timestamp: Tue 2005-11-22 00:00:00 +0000
@@ -1516,13 +1359,13 @@
         tree = self.make_commits_with_bugs()
         self.assertFormatterResult("""\
     2 Joe Bar\t2005-11-22
-      fixes bug(s): test://bug/id test://bug/2
+      fixes bugs: test://bug/id test://bug/2
       multiline
       log
       message
 
     1 Joe Foo\t2005-11-22
-      fixes bug(s): test://bug/id
+      fixes bug: test://bug/id
       simple log message
 
 """,
@@ -1688,12 +1531,13 @@
 
 """)
 
+
 class TestLogExcludeAncestry(tests.TestCaseWithTransport):
 
     def make_branch_with_alternate_ancestries(self, relpath='.'):
         # See test_merge_sorted_exclude_ancestry below for the difference with
         # bt.per_branch.test_iter_merge_sorted_revision.
-        # TestIterMergeSortedRevisionsBushyGraph. 
+        # TestIterMergeSortedRevisionsBushyGraph.
         # make_branch_with_alternate_ancestries
         # and test_merge_sorted_exclude_ancestry
         # See the FIXME in assertLogRevnos too.
@@ -1754,3 +1598,41 @@
                              b, '1', '3', exclude_common_ancestry=True,
                              generate_merge_revisions=True)
 
+
+class TestLogDefaults(TestCaseForLogFormatter):
+    def test_default_log_level(self):
+        """
+        Test to ensure that specifying 'levels=1' to make_log_request_dict
+        doesn't get overwritten when using a LogFormatter that supports more
+        detail.
+        Fixes bug #747958.
+        """
+        wt = self._prepare_tree_with_merges()
+        b = wt.branch
+
+        class CustomLogFormatter(log.LogFormatter):
+            def __init__(self, *args, **kwargs):
+                super(CustomLogFormatter, self).__init__(*args, **kwargs)
+                self.revisions = []
+            def get_levels(self):
+                # log formatter supports all levels:
+                return 0
+            def log_revision(self, revision):
+                self.revisions.append(revision)
+
+        log_formatter = LogCatcher()
+        # First request we don't specify number of levels, we should get a
+        # sensible default (whatever the LogFormatter handles - which in this
+        # case is 0/everything):
+        request = log.make_log_request_dict(limit=10)
+        log.Logger(b, request).show(log_formatter)
+        # should have all three revisions:
+        self.assertEquals(len(log_formatter.revisions), 3)
+
+        del log_formatter
+        log_formatter = LogCatcher()
+        # now explicitly request mainline revisions only:
+        request = log.make_log_request_dict(limit=10, levels=1)
+        log.Logger(b, request).show(log_formatter)
+        # should now only have 2 revisions:
+        self.assertEquals(len(log_formatter.revisions), 2)

=== modified file 'bzrlib/tests/test_lru_cache.py'
--- bzrlib/tests/test_lru_cache.py	2010-07-15 13:05:40 +0000
+++ bzrlib/tests/test_lru_cache.py	2011-05-13 12:51:05 +0000
@@ -38,18 +38,18 @@
     def test_missing(self):
         cache = lru_cache.LRUCache(max_cache=10)
 
-        self.failIf('foo' in cache)
+        self.assertFalse('foo' in cache)
         self.assertRaises(KeyError, cache.__getitem__, 'foo')
 
         cache['foo'] = 'bar'
         self.assertEqual('bar', cache['foo'])
-        self.failUnless('foo' in cache)
-        self.failIf('bar' in cache)
+        self.assertTrue('foo' in cache)
+        self.assertFalse('bar' in cache)
 
     def test_map_None(self):
         # Make sure that we can properly map None as a key.
         cache = lru_cache.LRUCache(max_cache=10)
-        self.failIf(None in cache)
+        self.assertFalse(None in cache)
         cache[None] = 1
         self.assertEqual(1, cache[None])
         cache[None] = 2
@@ -75,8 +75,8 @@
         # With a max cache of 1, adding 'baz' should pop out 'foo'
         cache['baz'] = 'biz'
 
-        self.failIf('foo' in cache)
-        self.failUnless('baz' in cache)
+        self.assertFalse('foo' in cache)
+        self.assertTrue('baz' in cache)
 
         self.assertEqual('biz', cache['baz'])
 
@@ -92,7 +92,7 @@
         # This must kick out 'foo' because it was the last accessed
         cache['nub'] = 'in'
 
-        self.failIf('foo' in cache)
+        self.assertFalse('foo' in cache)
 
     def test_cleanup(self):
         """Test that we can use a cleanup function."""

=== modified file 'bzrlib/tests/test_lsprof.py'
--- bzrlib/tests/test_lsprof.py	2010-07-04 06:22:11 +0000
+++ bzrlib/tests/test_lsprof.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,25 +18,13 @@
 
 
 import cPickle
-import os
 import threading
 
 import bzrlib
 from bzrlib import errors, tests
-
-
-class _LSProfFeature(tests.Feature):
-
-    def available(self):
-        try:
-            from bzrlib import lsprof
-        except ImportError:
-            return False
-        else:
-            return True
-
-
-LSProfFeature = _LSProfFeature()
+from bzrlib.tests import (
+    features,
+    )
 
 
 _TXT_HEADER = "   CallCount    Recursive    Total(ms)   " + \
@@ -57,7 +45,7 @@
 
 class TestStatsSave(tests.TestCaseInTempDir):
 
-    _test_needs_features = [LSProfFeature]
+    _test_needs_features = [features.lsprof_feature]
 
     def setUp(self):
         super(tests.TestCaseInTempDir, self).setUp()
@@ -97,7 +85,7 @@
 
 class TestBzrProfiler(tests.TestCase):
 
-    _test_needs_features = [LSProfFeature]
+    _test_needs_features = [features.lsprof_feature]
 
     def test_start_call_stuff_stop(self):
         profiler = bzrlib.lsprof.BzrProfiler()

=== modified file 'bzrlib/tests/test_matchers.py'
--- bzrlib/tests/test_matchers.py	2010-05-06 11:08:10 +0000
+++ bzrlib/tests/test_matchers.py	2011-08-31 23:17:41 +0000
@@ -18,7 +18,10 @@
 
 from testtools.matchers import *
 
-from bzrlib.tests import TestCase
+from bzrlib.tests import (
+    TestCase,
+    TestCaseWithTransport,
+    )
 from bzrlib.tests.matchers import *
 
 
@@ -62,3 +65,84 @@
         self.assertNotEqual(None, mismatch)
         self.assertThat(mismatch.describe(), Equals("I am da tree is locked"))
 
+
+class TestMatchesAncestry(TestCaseWithTransport):
+
+    def test__str__(self):
+        matcher = MatchesAncestry("A repository", "arevid")
+        self.assertEqual(
+            "MatchesAncestry(repository='A repository', "
+            "revision_id='arevid')",
+            str(matcher))
+
+    def test_match(self):
+        b = self.make_branch_builder('.')
+        b.start_series()
+        revid1 = b.build_commit()
+        revid2 = b.build_commit()
+        b.finish_series()
+        branch = b.get_branch()
+        m = MatchesAncestry(branch.repository, revid2)
+        self.assertThat([revid2, revid1], m)
+        self.assertThat([revid1, revid2], m)
+        m = MatchesAncestry(branch.repository, revid1)
+        self.assertThat([revid1], m)
+        m = MatchesAncestry(branch.repository, "unknown")
+        self.assertThat(["unknown"], m)
+
+    def test_mismatch(self):
+        b = self.make_branch_builder('.')
+        b.start_series()
+        revid1 = b.build_commit()
+        revid2 = b.build_commit()
+        b.finish_series()
+        branch = b.get_branch()
+        m = MatchesAncestry(branch.repository, revid1)
+        mismatch = m.match([])
+        self.assertIsNot(None, mismatch)
+        self.assertEquals(
+            "mismatched ancestry for revision '%s' was ['%s'], expected []" % (
+                revid1, revid1),
+            mismatch.describe())
+
+
+class TestHasLayout(TestCaseWithTransport):
+
+    def test__str__(self):
+        matcher = HasLayout([("a", "a-id")])
+        self.assertEqual("HasLayout([('a', 'a-id')])", str(matcher))
+
+    def test_match(self):
+        t = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+        t.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        self.assertThat(t, HasLayout(['', 'a', 'b/', 'b/c']))
+        self.assertThat(t, HasLayout(
+            [('', t.get_root_id()),
+             ('a', 'a-id'),
+             ('b/', 'b-id'),
+             ('b/c', 'c-id')]))
+
+    def test_mismatch(self):
+        t = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+        t.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        mismatch = HasLayout(['a']).match(t)
+        self.assertIsNot(None, mismatch)
+        self.assertEquals(
+            "['a'] != [u'', u'a', u'b/', u'b/c']",
+            mismatch.describe())
+
+    def test_no_dirs(self):
+        # Some tree/repository formats do not support versioned directories
+        t = self.make_branch_and_tree('.')
+        t.has_versioned_directories = lambda: False
+        self.build_tree(['a', 'b/', 'b/c'])
+        t.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        self.assertIs(None, HasLayout(['', 'a', 'b/', 'b/c']).match(t))
+        self.assertIs(None, HasLayout(['', 'a', 'b/', 'b/c', 'd/']).match(t))
+        mismatch = HasLayout([u'', u'a', u'd/']).match(t)
+        self.assertIsNot(None, mismatch)
+        self.assertEquals(
+            "[u'', u'a'] != [u'', u'a', u'b/', u'b/c']",
+            mismatch.describe())

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2010-07-29 04:07:27 +0000
+++ bzrlib/tests/test_merge.py	2011-10-14 13:56:45 +0000
@@ -37,6 +37,7 @@
 from bzrlib.merge import transform_tree, merge_inner, _PlanMerge
 from bzrlib.osutils import basename, pathjoin, file_kind
 from bzrlib.tests import (
+    features,
     TestCaseWithMemoryTransport,
     TestCaseWithTransport,
     test_merge_core,
@@ -90,8 +91,8 @@
         os.chdir('branch2')
         self.run_bzr('merge ../branch1/baz', retcode=3)
         self.run_bzr('merge ../branch1/foo')
-        self.failUnlessExists('foo')
-        self.failIfExists('bar')
+        self.assertPathExists('foo')
+        self.assertPathDoesNotExist('bar')
         wt2 = WorkingTree.open('.') # opens branch2
         self.assertEqual([tip], wt2.get_parent_ids())
 
@@ -121,6 +122,32 @@
         finally:
             wt1.unlock()
 
+    def test_merge_into_null_tree(self):
+        wt = self.make_branch_and_tree('tree')
+        null_tree = wt.basis_tree()
+        self.build_tree(['tree/file'])
+        wt.add('file')
+        wt.commit('tree with root')
+        merger = _mod_merge.Merge3Merger(null_tree, null_tree, null_tree, wt,
+                                         this_branch=wt.branch,
+                                         do_merge=False)
+        with merger.make_preview_transform() as tt:
+            self.assertEqual([], tt.find_conflicts())
+            preview = tt.get_preview_tree()
+            self.assertEqual(wt.get_root_id(), preview.get_root_id())
+
+    def test_merge_unrelated_retains_root(self):
+        wt = self.make_branch_and_tree('tree')
+        other_tree = self.make_branch_and_tree('other')
+        self.addCleanup(other_tree.lock_read().unlock)
+        merger = _mod_merge.Merge3Merger(wt, wt, wt.basis_tree(), other_tree,
+                                         this_branch=wt.branch,
+                                         do_merge=False)
+        with transform.TransformPreview(wt) as merger.tt:
+            merger._compute_transform()
+            new_root_id = merger.tt.final_file_id(merger.tt.root)
+            self.assertEqual(wt.get_root_id(), new_root_id)
+
     def test_create_rename(self):
         """Rename an inventory entry while creating the file"""
         tree =self.make_branch_and_tree('.')
@@ -157,12 +184,12 @@
         log = StringIO()
         merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(),
                     this_tree=tree_b, ignore_zero=True)
-        self.failUnless('All changes applied successfully.\n' not in
+        self.assertTrue('All changes applied successfully.\n' not in
             self.get_log())
         tree_b.revert()
         merge_inner(tree_b.branch, tree_a, tree_b.basis_tree(),
                     this_tree=tree_b, ignore_zero=False)
-        self.failUnless('All changes applied successfully.\n' in self.get_log())
+        self.assertTrue('All changes applied successfully.\n' in self.get_log())
 
     def test_merge_inner_conflicts(self):
         tree_a = self.make_branch_and_tree('a')
@@ -387,6 +414,25 @@
                              '>>>>>>> MERGE-SOURCE\n',
                              'this/file')
 
+    def test_merge_reverse_revision_range(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree(['a'])
+        tree.add('a')
+        first_rev = tree.commit("added a")
+        merger = _mod_merge.Merger.from_revision_ids(None, tree,
+                                          _mod_revision.NULL_REVISION,
+                                          first_rev)
+        merger.merge_type = _mod_merge.Merge3Merger
+        merger.interesting_files = 'a'
+        conflict_count = merger.do_merge()
+        self.assertEqual(0, conflict_count)
+
+        self.assertPathDoesNotExist("a")
+        tree.revert()
+        self.assertPathExists("a")
+
     def test_make_merger(self):
         this_tree = self.make_branch_and_tree('this')
         this_tree.commit('rev1', rev_id='rev1')
@@ -456,6 +502,23 @@
         finally:
             tree_file.close()
 
+    def test_merge_require_tree_root(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree(['a'])
+        tree.add('a')
+        first_rev = tree.commit("added a")
+        old_root_id = tree.get_root_id()
+        merger = _mod_merge.Merger.from_revision_ids(None, tree,
+                                          _mod_revision.NULL_REVISION,
+                                          first_rev)
+        merger.merge_type = _mod_merge.Merge3Merger
+        conflict_count = merger.do_merge()
+        self.assertEqual(0, conflict_count)
+        self.assertEquals(set([old_root_id]), tree.all_file_ids())
+        tree.set_parent_ids([])
+
     def test_merge_add_into_deleted_root(self):
         # Yes, people actually do this.  And report bugs if it breaks.
         source = self.make_branch_and_tree('source', format='rich-root-pack')
@@ -1270,6 +1333,26 @@
         self.assertEqual(['B-id', 'C-id', 'F-id'],
                          [t.get_revision_id() for t in merger._lca_trees])
 
+    def test_find_base_new_root_criss_cross(self):
+        # A   B
+        # |\ /|
+        # | X |
+        # |/ \|
+        # C   D
+        
+        builder = self.get_builder()
+        builder.build_snapshot('A-id', None,
+            [('add', ('', None, 'directory', None))])
+        builder.build_snapshot('B-id', [],
+            [('add', ('', None, 'directory', None))])
+        builder.build_snapshot('D-id', ['A-id', 'B-id'], [])
+        builder.build_snapshot('C-id', ['A-id', 'B-id'], [])
+        merger = self.make_Merger(builder, 'D-id')
+        self.assertEqual('A-id', merger.base_rev_id)
+        self.assertTrue(merger._is_criss_cross)
+        self.assertEqual(['A-id', 'B-id'], [t.get_revision_id()
+                                            for t in merger._lca_trees])
+
     def test_no_criss_cross_passed_to_merge_type(self):
         class LCATreesMerger(LoggingMerger):
             supports_lca_trees = True
@@ -1807,6 +1890,7 @@
         builder.build_snapshot('C-id', ['A-id'], [])
         builder.build_snapshot('E-id', ['C-id', 'B-id'],
             [('unversion', 'a-id'),
+             ('flush', None),
              ('add', (u'a', 'a-id', 'directory', None))])
         builder.build_snapshot('D-id', ['B-id', 'C-id'], [])
         merge_obj = self.make_merge_obj(builder, 'E-id')
@@ -1830,6 +1914,7 @@
         builder.build_snapshot('E-id', ['C-id', 'B-id'], [])
         builder.build_snapshot('D-id', ['B-id', 'C-id'],
             [('unversion', 'a-id'),
+             ('flush', None),
              ('add', (u'a', 'a-id', 'directory', None))])
         merge_obj = self.make_merge_obj(builder, 'E-id')
         entries = list(merge_obj._entries_lca())
@@ -2098,7 +2183,7 @@
         self.assertTrue(wt.is_executable('foo-id'))
 
     def test_create_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A
         #  / \
         # B   C
@@ -2163,7 +2248,7 @@
                              wt.get_file_text('foo-id'))
 
     def test_modified_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Create symlink foo => bar
         #  / \
         # B   C     B relinks foo => baz
@@ -2208,7 +2293,7 @@
         self.assertEqual('bing', wt.get_symlink_target('foo-id'))
 
     def test_renamed_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Create symlink foo => bar
         #  / \
         # B   C     B renames foo => barry
@@ -2264,7 +2349,7 @@
         self.assertEqual('blah', wt.id2path('foo-id'))
 
     def test_symlink_no_content_change(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Create symlink foo => bar
         #  / \
         # B   C     B relinks foo => baz
@@ -2315,7 +2400,7 @@
         self.assertEqual('bing', wt.get_symlink_target('foo-id'))
 
     def test_symlink_this_changed_kind(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         #   A       Nothing
         #  / \
         # B   C     B creates symlink foo => bar
@@ -2368,7 +2453,7 @@
 
     def test_symlink_all_wt(self):
         """Check behavior if all trees are Working Trees."""
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         # The big issue is that entry.symlink_target is None for WorkingTrees.
         # So we need to make sure we handle that case correctly.
         #   A   foo => bar
@@ -2843,14 +2928,14 @@
 
     def get_merger_factory(self):
         # Allows  the inner methods to access the test attributes
-        test = self
+        calls = self.calls
 
         class FooMerger(_mod_merge.ConfigurableFileMerger):
             name_prefix = "foo"
             default_files = ['bar']
 
             def merge_text(self, params):
-                test.calls.append('merge_text')
+                calls.append('merge_text')
                 return ('not_applicable', None)
 
         def factory(merger):

=== modified file 'bzrlib/tests/test_merge3.py'
--- bzrlib/tests/test_merge3.py	2010-04-26 13:51:08 +0000
+++ bzrlib/tests/test_merge3.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,11 +16,9 @@
 
 
 from bzrlib import (
-    errors,
     merge3,
     tests,
     )
-from bzrlib.merge3 import Merge3
 from bzrlib.errors import CantReprocessAndShowBase, BinaryFile
 
 def split_lines(t):

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_merge_core.py	2011-09-26 15:21:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,11 @@
     merge_directive,
     osutils,
     )
-from bzrlib.conflicts import ContentsConflict, TextConflict, PathConflict
+from bzrlib.conflicts import (
+    ContentsConflict,
+    TextConflict,
+    PathConflict,
+    )
 from bzrlib.merge import (
     Merge3Merger,
     Diff3Merger,
@@ -32,7 +36,6 @@
     Merger,
     )
 from bzrlib.osutils import getcwd, pathjoin
-from bzrlib import progress
 from bzrlib.transform import TreeTransform
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
 from bzrlib.workingtree import WorkingTree
@@ -95,7 +98,7 @@
             tt.apply()
             wt.commit('branch commit')
             wt.flush()
-            if len(wt.branch.revision_history()) != 2:
+            if wt.branch.last_revision_info()[0] != 2:
                 raise AssertionError()
         self.this.branch.fetch(self.other.branch)
         other_basis = self.other.branch.basis_tree()
@@ -490,16 +493,16 @@
         wtb = d_b.open_workingtree()
         wtb.commit('this revision', allow_pointless=False)
         self.assertEqual(1, wtb.merge_from_branch(wta.branch))
-        self.failUnlessExists('b/file.THIS')
-        self.failUnlessExists('b/file.BASE')
-        self.failUnlessExists('b/file.OTHER')
+        self.assertPathExists('b/file.THIS')
+        self.assertPathExists('b/file.BASE')
+        self.assertPathExists('b/file.OTHER')
         wtb.revert()
         self.assertEqual(1, wtb.merge_from_branch(wta.branch,
                                                   merge_type=WeaveMerger))
-        self.failUnlessExists('b/file')
-        self.failUnlessExists('b/file.THIS')
-        self.failUnlessExists('b/file.BASE')
-        self.failUnlessExists('b/file.OTHER')
+        self.assertPathExists('b/file')
+        self.assertPathExists('b/file.THIS')
+        self.assertPathExists('b/file.BASE')
+        self.assertPathExists('b/file.OTHER')
 
     def test_weave_conflicts_not_in_base(self):
         builder = self.make_branch_builder('source')
@@ -529,11 +532,11 @@
         self.assertEqual(1, tree.merge_from_branch(tree.branch,
                                                    to_revision='D-id',
                                                    merge_type=WeaveMerger))
-        self.failUnlessExists('tree/foo.THIS')
-        self.failUnlessExists('tree/foo.OTHER')
+        self.assertPathExists('tree/foo.THIS')
+        self.assertPathExists('tree/foo.OTHER')
         self.expectFailure('fail to create .BASE in some criss-cross merges',
-            self.failUnlessExists, 'tree/foo.BASE')
-        self.failUnlessExists('tree/foo.BASE')
+            self.assertPathExists, 'tree/foo.BASE')
+        self.assertPathExists('tree/foo.BASE')
 
     def test_merge_unrelated(self):
         """Sucessfully merges unrelated branches with no common names"""
@@ -581,7 +584,7 @@
         wtb.commit('changed file', allow_pointless=False)
         wtb.merge_from_branch(wta.branch, wta.branch.last_revision(),
                               wta.branch.get_rev_id(1))
-        self.failIf(os.path.lexists('b/file'))
+        self.assertFalse(os.path.lexists('b/file'))
 
     def test_merge_metadata_vs_deletion(self):
         """Conflict deletion vs metadata change"""
@@ -615,8 +618,8 @@
         b_wt.commit('r1', rev_id='r1')
         self.assertEqual(0, a_wt.merge_from_branch(b_wt.branch,
             b_wt.branch.last_revision(), b_wt.branch.get_rev_id(1)))
-        self.failUnlessExists('a/un')
-        self.failUnless('a/deux')
+        self.assertPathExists('a/un')
+        self.assertTrue('a/deux')
         self.assertFalse(os.path.exists('a/tmp'))
         self.assertEqual(file('a/un').read(),'DEUX')
         self.assertEqual(file('a/deux').read(),'UN')

=== added file 'bzrlib/tests/test_mergetools.py'
--- bzrlib/tests/test_mergetools.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_mergetools.py	2011-06-30 21:00:38 +0000
@@ -0,0 +1,170 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import os
+import re
+import sys
+import tempfile
+
+from bzrlib import (
+    mergetools,
+    tests
+)
+from bzrlib.tests.features import backslashdir_feature
+
+
+class TestFilenameSubstitution(tests.TestCaseInTempDir):
+
+    def test_simple_filename(self):
+        cmd_list = ['kdiff3', '{base}', '{this}', '{other}', '-o', '{result}']
+        args, tmpfile = mergetools._subst_filename(cmd_list, 'test.txt')
+        self.assertEqual(
+            ['kdiff3',
+             'test.txt.BASE',
+             'test.txt.THIS',
+             'test.txt.OTHER',
+             '-o',
+             'test.txt'],
+            args)
+        
+    def test_spaces(self):
+        cmd_list = ['kdiff3', '{base}', '{this}', '{other}', '-o', '{result}']
+        args, tmpfile = mergetools._subst_filename(cmd_list,
+                                                   'file with space.txt')
+        self.assertEqual(
+            ['kdiff3',
+             'file with space.txt.BASE',
+             'file with space.txt.THIS',
+             'file with space.txt.OTHER',
+             '-o',
+             'file with space.txt'],
+            args)
+
+    def test_spaces_and_quotes(self):
+        cmd_list = ['kdiff3', '{base}', '{this}', '{other}', '-o', '{result}']
+        args, tmpfile = mergetools._subst_filename(cmd_list,
+            'file with "space and quotes".txt')
+        self.assertEqual(
+            ['kdiff3',
+             'file with "space and quotes".txt.BASE',
+             'file with "space and quotes".txt.THIS',
+             'file with "space and quotes".txt.OTHER',
+             '-o',
+             'file with "space and quotes".txt'],
+            args)
+
+    def test_tempfile(self):
+        self.build_tree(('test.txt', 'test.txt.BASE', 'test.txt.THIS',
+                         'test.txt.OTHER'))
+        cmd_list = ['some_tool', '{this_temp}']
+        args, tmpfile = mergetools._subst_filename(cmd_list, 'test.txt')
+        self.assertPathExists(tmpfile)
+        os.remove(tmpfile)
+
+
+class TestCheckAvailability(tests.TestCaseInTempDir):
+
+    def test_full_path(self):
+        self.assertTrue(mergetools.check_availability(sys.executable))
+
+    def test_exe_on_path(self):
+        if sys.platform == 'win32':
+            exe = 'cmd.exe'
+        else:
+            exe = 'sh'
+        self.assertTrue(mergetools.check_availability(exe))
+
+    def test_nonexistent(self):
+        self.assertFalse(mergetools.check_availability('DOES NOT EXIST'))
+
+    def test_non_executable(self):
+        f, name = tempfile.mkstemp()
+        try:
+            self.log('temp filename: %s', name)
+            self.assertFalse(mergetools.check_availability(name))
+        finally:
+            os.close(f)
+            os.unlink(name)
+
+
+class TestInvoke(tests.TestCaseInTempDir):
+    def setUp(self):
+        super(tests.TestCaseInTempDir, self).setUp()
+        self._exe = None
+        self._args = None
+        self.build_tree_contents((
+            ('test.txt', 'stuff'),
+            ('test.txt.BASE', 'base stuff'),
+            ('test.txt.THIS', 'this stuff'),
+            ('test.txt.OTHER', 'other stuff'),
+        ))
+        
+    def test_success(self):
+        def dummy_invoker(exe, args, cleanup):
+            self._exe = exe
+            self._args = args
+            cleanup(0)
+            return 0
+        retcode = mergetools.invoke('tool {result}', 'test.txt', dummy_invoker)
+        self.assertEqual(0, retcode)
+        self.assertEqual('tool', self._exe)
+        self.assertEqual(['test.txt'], self._args)
+    
+    def test_failure(self):
+        def dummy_invoker(exe, args, cleanup):
+            self._exe = exe
+            self._args = args
+            cleanup(1)
+            return 1
+        retcode = mergetools.invoke('tool {result}', 'test.txt', dummy_invoker)
+        self.assertEqual(1, retcode)
+        self.assertEqual('tool', self._exe)
+        self.assertEqual(['test.txt'], self._args)
+
+    def test_success_tempfile(self):
+        def dummy_invoker(exe, args, cleanup):
+            self._exe = exe
+            self._args = args
+            self.assertPathExists(args[0])
+            f = open(args[0], 'wt')
+            f.write('temp stuff')
+            f.close()
+            cleanup(0)
+            return 0
+        retcode = mergetools.invoke('tool {this_temp}', 'test.txt',
+                                    dummy_invoker)
+        self.assertEqual(0, retcode)
+        self.assertEqual('tool', self._exe)
+        self.assertPathDoesNotExist(self._args[0])
+        self.assertFileEqual('temp stuff', 'test.txt')
+    
+    def test_failure_tempfile(self):
+        def dummy_invoker(exe, args, cleanup):
+            self._exe = exe
+            self._args = args
+            self.assertPathExists(args[0])
+            self.log(repr(args))
+            f = open(args[0], 'wt')
+            self.log(repr(f))
+            f.write('temp stuff')
+            f.close()
+            cleanup(1)
+            return 1
+        retcode = mergetools.invoke('tool {this_temp}', 'test.txt',
+                                    dummy_invoker)
+        self.assertEqual(1, retcode)
+        self.assertEqual('tool', self._exe)
+        self.assertFileEqual('stuff', 'test.txt')

=== modified file 'bzrlib/tests/test_missing.py'
--- bzrlib/tests/test_missing.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_missing.py	2011-09-19 08:49:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,9 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
-
-
 from bzrlib import (
     missing,
     tests,
@@ -25,7 +22,6 @@
     iter_log_revisions,
     )
 from bzrlib.tests import TestCaseWithTransport
-from bzrlib.workingtree import WorkingTree
 
 
 class TestMissing(TestCaseWithTransport):
@@ -138,12 +134,12 @@
 class TestFindUnmerged(tests.TestCaseWithTransport):
 
     def assertUnmerged(self, local, remote, local_branch, remote_branch,
-            restrict='all', include_merges=False, backward=False,
+            restrict='all', include_merged=False, backward=False,
             local_revid_range=None, remote_revid_range=None):
         """Check the output of find_unmerged_mainline_revisions"""
         local_extra, remote_extra = missing.find_unmerged(
             local_branch, remote_branch, restrict,
-            include_merges=include_merges, backward=backward,
+            include_merged=include_merged, backward=backward,
             local_revid_range=local_revid_range,
             remote_revid_range=remote_revid_range)
         self.assertEqual(local, local_extra)
@@ -191,7 +187,7 @@
         self.assertUnmerged([], [], tree.branch, tree2.branch,
             local_revid_range=(rev1, rev1))
 
-    def test_include_merges(self):
+    def test_include_merged(self):
         tree = self.make_branch_and_tree('tree')
         rev1 = tree.commit('one', rev_id='rev1')
 
@@ -211,27 +207,27 @@
                                  ('3.1.1', 'rev4', 1), ('3.1.2', 'rev5', 1),
                                  ],
                             tree.branch, tree2.branch,
-                            include_merges=True)
+                            include_merged=True)
 
         self.assertUnmerged([], [('4', 'rev6', 0),
                                  ('3.1.2', 'rev5', 1), ('3.1.1', 'rev4', 1),
                                  ('3', 'rev3', 0), ('2', 'rev2', 0),
                                  ],
                             tree.branch, tree2.branch,
-                            include_merges=True,
+                            include_merged=True,
                             backward=True)
 
         self.assertUnmerged([], [('4', 'rev6', 0)],
             tree.branch, tree2.branch,
-            include_merges=True, remote_revid_range=(rev6, rev6))
+            include_merged=True, remote_revid_range=(rev6, rev6))
 
         self.assertUnmerged([], [('3', 'rev3', 0), ('3.1.1', 'rev4', 1)],
                     tree.branch, tree2.branch,
-                    include_merges=True, remote_revid_range=(rev3, rev4))
+                    include_merged=True, remote_revid_range=(rev3, rev4))
 
         self.assertUnmerged([], [('4', 'rev6', 0), ('3.1.2', 'rev5', 1)],
                     tree.branch, tree2.branch,
-                    include_merges=True, remote_revid_range=(rev5, rev6))
+                    include_merged=True, remote_revid_range=(rev5, rev6))
 
     def test_revision_range(self):
         local = self.make_branch_and_tree('local')

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_msgeditor.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,19 +22,19 @@
 
 from bzrlib import (
     commit,
+    config,
     errors,
     msgeditor,
     osutils,
     tests,
     trace,
     )
-from bzrlib.branch import Branch
-from bzrlib.config import ensure_config_dir_exists, config_filename
 from bzrlib.msgeditor import (
     make_commit_message_template_encoded,
     edit_commit_message_encoded
 )
 from bzrlib.tests import (
+    features,
     TestCaseInTempDir,
     TestCaseWithTransport,
     TestNotApplicable,
@@ -142,24 +142,35 @@
   hell\u00d8
 """.encode('utf8') in template)
 
-    def make_do_nothing_editor(self):
+    def make_do_nothing_editor(self, basename='fed'):
         if sys.platform == "win32":
-            f = file('fed.bat', 'w')
+            name = basename + '.bat'
+            f = file(name, 'w')
             f.write('@rem dummy fed')
             f.close()
-            return 'fed.bat'
+            return name
         else:
-            f = file('fed.sh', 'wb')
+            name = basename + '.sh'
+            f = file(name, 'wb')
             f.write('#!/bin/sh\n')
             f.close()
-            os.chmod('fed.sh', 0755)
-            return './fed.sh'
+            os.chmod(name, 0755)
+            return './' + name
 
     def test_run_editor(self):
-        os.environ['BZR_EDITOR'] = self.make_do_nothing_editor()
+        self.overrideEnv('BZR_EDITOR', self.make_do_nothing_editor())
         self.assertEqual(True, msgeditor._run_editor(''),
                          'Unable to run dummy fake editor')
 
+    def test_parse_editor_name(self):
+        """Correctly interpret names with spaces.
+
+        See <https://bugs.launchpad.net/bzr/+bug/220331>
+        """
+        self.overrideEnv('BZR_EDITOR',
+            '"%s"' % self.make_do_nothing_editor('name with spaces'))
+        self.assertEqual(True, msgeditor._run_editor('a_filename'))    
+
     def make_fake_editor(self, message='test message from fed\\n'):
         """Set up environment so that an editor will be a known script.
 
@@ -190,11 +201,11 @@
 "%s" fed.py %%1
 """ % sys.executable)
             f.close()
-            os.environ['BZR_EDITOR'] = 'fed.bat'
+            self.overrideEnv('BZR_EDITOR', 'fed.bat')
         else:
             # [non-win32] make python script executable and set BZR_EDITOR
             os.chmod('fed.py', 0755)
-            os.environ['BZR_EDITOR'] = './fed.py'
+            self.overrideEnv('BZR_EDITOR', './fed.py')
 
     def test_edit_commit_message(self):
         working_tree = self.make_uncommitted_tree()
@@ -229,57 +240,37 @@
         working_tree = self.make_uncommitted_tree()
 
         if sys.platform == 'win32':
-            os.environ['BZR_EDITOR'] = 'cmd.exe /c del'
+            editor = 'cmd.exe /c del'
         else:
-            os.environ['BZR_EDITOR'] = 'rm'
+            editor = 'rm'
+        self.overrideEnv('BZR_EDITOR', editor)
 
         self.assertRaises((IOError, OSError), msgeditor.edit_commit_message, '')
 
     def test__get_editor(self):
-        # Test that _get_editor can return a decent list of items
-        bzr_editor = os.environ.get('BZR_EDITOR')
-        visual = os.environ.get('VISUAL')
-        editor = os.environ.get('EDITOR')
-        try:
-            os.environ['BZR_EDITOR'] = 'bzr_editor'
-            os.environ['VISUAL'] = 'visual'
-            os.environ['EDITOR'] = 'editor'
-
-            ensure_config_dir_exists()
-            f = open(config_filename(), 'wb')
-            f.write('editor = config_editor\n')
-            f.close()
-
-            editors = list(msgeditor._get_editor())
-            editors = [editor for (editor, cfg_src) in editors]
-
-            self.assertEqual(['bzr_editor', 'config_editor', 'visual',
-                              'editor'], editors[:4])
-
-            if sys.platform == 'win32':
-                self.assertEqual(['wordpad.exe', 'notepad.exe'], editors[4:])
-            else:
-                self.assertEqual(['/usr/bin/editor', 'vi', 'pico', 'nano',
-                                  'joe'], editors[4:])
-
-        finally:
-            # Restore the environment
-            if bzr_editor is None:
-                del os.environ['BZR_EDITOR']
-            else:
-                os.environ['BZR_EDITOR'] = bzr_editor
-            if visual is None:
-                del os.environ['VISUAL']
-            else:
-                os.environ['VISUAL'] = visual
-            if editor is None:
-                del os.environ['EDITOR']
-            else:
-                os.environ['EDITOR'] = editor
+        self.overrideEnv('BZR_EDITOR', 'bzr_editor')
+        self.overrideEnv('VISUAL', 'visual')
+        self.overrideEnv('EDITOR', 'editor')
+
+        conf = config.GlobalConfig.from_string('editor = config_editor\n',
+                                               save=True)
+
+        editors = list(msgeditor._get_editor())
+        editors = [editor for (editor, cfg_src) in editors]
+
+        self.assertEqual(['bzr_editor', 'config_editor', 'visual', 'editor'],
+                         editors[:4])
+
+        if sys.platform == 'win32':
+            self.assertEqual(['wordpad.exe', 'notepad.exe'], editors[4:])
+        else:
+            self.assertEqual(['/usr/bin/editor', 'vi', 'pico', 'nano', 'joe'],
+                             editors[4:])
+
 
     def test__run_editor_EACCES(self):
         """If running a configured editor raises EACESS, the user is warned."""
-        os.environ['BZR_EDITOR'] = 'eacces.py'
+        self.overrideEnv('BZR_EDITOR', 'eacces.py')
         f = file('eacces.py', 'wb')
         f.write('# Not a real editor')
         f.close()
@@ -287,7 +278,7 @@
         os.chmod('eacces.py', 0)
         # Set $EDITOR so that _run_editor will terminate before trying real
         # editors.
-        os.environ['EDITOR'] = self.make_do_nothing_editor()
+        self.overrideEnv('EDITOR', self.make_do_nothing_editor())
         # Call _run_editor, capturing mutter.warning calls.
         warnings = []
         def warning(*args):
@@ -319,11 +310,14 @@
         self.assertFileEqual(expected, msgfilename)
 
     def test__create_temp_file_with_commit_template_in_unicode_dir(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         if hasattr(self, 'info'):
-            os.mkdir(self.info['directory'])
-            os.chdir(self.info['directory'])
-            msgeditor._create_temp_file_with_commit_template('infotext')
+            tmpdir = self.info['directory']
+            os.mkdir(tmpdir)
+            # Force the creation of temp file in a directory whose name
+            # requires some encoding support
+            msgeditor._create_temp_file_with_commit_template('infotext',
+                                                             tmpdir=tmpdir)
         else:
             raise TestNotApplicable('Test run elsewhere with non-ascii data.')
 
@@ -336,23 +330,32 @@
         self.assertFileEqual('', msgfilename)
 
     def test_unsupported_encoding_commit_message(self):
-        old_env = osutils.set_or_unset_env('LANG', 'C')
-        try:
-            # LANG env variable has no effect on Windows
-            # but some characters anyway cannot be represented
-            # in default user encoding
-            char = probe_bad_non_ascii(osutils.get_user_encoding())
-            if char is None:
-                raise TestSkipped('Cannot find suitable non-ascii character '
-                    'for user_encoding (%s)' % osutils.get_user_encoding())
-
-            self.make_fake_editor(message=char)
-
-            working_tree = self.make_uncommitted_tree()
-            self.assertRaises(errors.BadCommitMessageEncoding,
-                              msgeditor.edit_commit_message, '')
-        finally:
-            osutils.set_or_unset_env('LANG', old_env)
+        self.overrideEnv('LANG', 'C')
+        # LANG env variable has no effect on Windows
+        # but some characters anyway cannot be represented
+        # in default user encoding
+        char = probe_bad_non_ascii(osutils.get_user_encoding())
+        if char is None:
+            raise TestSkipped('Cannot find suitable non-ascii character '
+                'for user_encoding (%s)' % osutils.get_user_encoding())
+
+        self.make_fake_editor(message=char)
+
+        working_tree = self.make_uncommitted_tree()
+        self.assertRaises(errors.BadCommitMessageEncoding,
+                          msgeditor.edit_commit_message, '')
+
+    def test_set_commit_message_no_hooks(self):
+        commit_obj = commit.Commit()
+        self.assertIs(None,
+            msgeditor.set_commit_message(commit_obj))
+
+    def test_set_commit_message_hook(self):
+        msgeditor.hooks.install_named_hook("set_commit_message",
+                lambda commit_obj, existing_message: "save me some typing\n", None)
+        commit_obj = commit.Commit()
+        self.assertEquals("save me some typing\n",
+            msgeditor.set_commit_message(commit_obj))
 
     def test_generate_commit_message_template_no_hooks(self):
         commit_obj = commit.Commit()
@@ -378,11 +381,5 @@
         ERROR_BAD_EXE_FORMAT = 193
         file("textfile.txt", "w").close()
         e = self.assertRaises(WindowsError, subprocess.call, "textfile.txt")
-        # Python2.4 used the 'winerror' as the errno, which confuses a lot of
-        # our error trapping code. Make sure that we understand the mapping
-        # correctly.
-        if sys.version_info >= (2, 5):
-            self.assertEqual(e.errno, errno.ENOEXEC)
-            self.assertEqual(e.winerror, ERROR_BAD_EXE_FORMAT)
-        else:
-            self.assertEqual(e.errno, ERROR_BAD_EXE_FORMAT)
+        self.assertEqual(e.errno, errno.ENOEXEC)
+        self.assertEqual(e.winerror, ERROR_BAD_EXE_FORMAT)

=== modified file 'bzrlib/tests/test_multiparent.py'
--- bzrlib/tests/test_multiparent.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_multiparent.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 from unittest import TestCase
 
 from bzrlib import (
-    knit,
     multiparent,
     patiencediff,
     tests,
@@ -266,10 +265,10 @@
     def test_filenames(self):
         vf = multiparent.MultiVersionedFile('foop')
         vf.add_version('a\nb\nc\nd'.splitlines(True), 'a', [])
-        self.failUnlessExists('foop.mpknit')
-        self.failIfExists('foop.mpidx')
+        self.assertPathExists('foop.mpknit')
+        self.assertPathDoesNotExist('foop.mpidx')
         vf.save()
-        self.failUnlessExists('foop.mpidx')
+        self.assertPathExists('foop.mpidx')
         vf.destroy()
-        self.failIfExists('foop.mpknit')
-        self.failIfExists('foop.mpidx')
+        self.assertPathDoesNotExist('foop.mpknit')
+        self.assertPathDoesNotExist('foop.mpidx')

=== modified file 'bzrlib/tests/test_mutabletree.py'
--- bzrlib/tests/test_mutabletree.py	2009-10-06 16:34:48 +0000
+++ bzrlib/tests/test_mutabletree.py	2011-06-09 18:46:24 +0000
@@ -55,9 +55,8 @@
         self.assertTrue(self.tree.has_changes())
 
     def test_with_pending_merges(self):
+        self.tree.commit('first commit')
         other_tree = self.tree.bzrdir.sprout('other').open_workingtree()
-        self.build_tree(['other/file'])
-        other_tree.add('file')
-        other_tree.commit('added file')
+        other_tree.commit('mergeable commit')
         self.tree.merge_from_branch(other_tree.branch)
         self.assertTrue(self.tree.has_changes())

=== modified file 'bzrlib/tests/test_nonascii.py'
--- bzrlib/tests/test_nonascii.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_nonascii.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +23,6 @@
 from bzrlib import osutils
 from bzrlib.osutils import pathjoin
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
-from bzrlib.workingtree import WorkingTree
 
 
 class NonAsciiTest(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2010-04-20 08:47:38 +0000
+++ bzrlib/tests/test_options.py	2011-09-29 15:50:58 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,11 +19,13 @@
 from bzrlib import (
     bzrdir,
     commands,
+    controldir,
     errors,
     option,
+    registry,
     )
 from bzrlib.builtins import cmd_commit
-from bzrlib.commands import Command, parse_args
+from bzrlib.commands import parse_args
 from bzrlib.tests import TestCase
 from bzrlib.repofmt import knitrepo
 
@@ -41,15 +43,18 @@
         # XXX: Using cmd_commit makes these tests overly sensitive to changes
         # to cmd_commit, when they are meant to be about option parsing in
         # general.
-        self.assertEqual(parse_args(cmd_commit(), ['--help']),
-           ([], {'author': [], 'exclude': [], 'fixes': [], 'help': True}))
-        self.assertEqual(parse_args(cmd_commit(), ['--message=biter']),
-           ([], {'author': [], 'exclude': [], 'fixes': [], 'message': 'biter'}))
+        self.assertEqual(
+           ([], {'author': [], 'exclude': [], 'fixes': [], 'help': True}),
+           parse_args(cmd_commit(), ['--help']))
+        self.assertEqual(
+           ([], {'author': [], 'exclude': [], 'fixes': [], 'message': 'biter'}),
+           parse_args(cmd_commit(), ['--message=biter']))
 
     def test_no_more_opts(self):
         """Terminated options"""
-        self.assertEqual(parse_args(cmd_commit(), ['--', '-file-with-dashes']),
-                          (['-file-with-dashes'], {'author': [], 'exclude': [], 'fixes': []}))
+        self.assertEqual(
+            (['-file-with-dashes'], {'author': [], 'exclude': [], 'fixes': []}),
+            parse_args(cmd_commit(), ['--', '-file-with-dashes']))
 
     def test_option_help(self):
         """Options have help strings."""
@@ -63,6 +68,11 @@
         out, err = self.run_bzr('help status')
         self.assertContainsRe(out, r'--show-ids.*Show internal object.')
 
+    def test_option_help_global_hidden(self):
+        """Hidden global options have no help strings."""
+        out, err = self.run_bzr('help log')
+        self.assertNotContainsRe(out, r'--message')
+
     def test_option_arg_help(self):
         """Help message shows option arguments."""
         out, err = self.run_bzr('help commit')
@@ -107,10 +117,10 @@
         self.assertFalse(option.Option('foo', hidden=False).is_hidden('foo'))
 
     def test_registry_conversion(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two', 'RepositoryFormatKnit1', 'two help')
-        registry.register_metadir('hidden', 'RepositoryFormatKnit1',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two', 'RepositoryFormatKnit1', 'two help')
+        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormatKnit1',
             'two help', hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', '', registry, str)]
@@ -177,13 +187,13 @@
         self.assertEqual('test option', my_option.help)
 
     def test_help(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two',
             'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
             'two help',
             )
-        registry.register_metadir('hidden', 'RepositoryFormat7', 'hidden help',
+        bzrdir.register_metadir(registry, 'hidden', 'RepositoryFormat7', 'hidden help',
             hidden=True)
         registry.set_default('one')
         options = [option.RegistryOption('format', 'format help', registry,
@@ -205,9 +215,9 @@
         opt = option.Option('hello', help='fg', type=int, argname='gar')
         self.assertEqual(list(opt.iter_switches()),
                          [('hello', None, 'GAR', 'fg')])
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('one', 'RepositoryFormat7', 'one help')
-        registry.register_metadir('two',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'one', 'RepositoryFormat7', 'one help')
+        bzrdir.register_metadir(registry, 'two',
                 'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
                 'two help',
                 )
@@ -386,15 +396,23 @@
                     + '\n'.join(msgs))
 
     def test_is_hidden(self):
-        registry = bzrdir.BzrDirFormatRegistry()
-        registry.register_metadir('hidden', 'HiddenFormat',
+        registry = controldir.ControlDirFormatRegistry()
+        bzrdir.register_metadir(registry, 'hidden', 'HiddenFormat',
             'hidden help text', hidden=True)
-        registry.register_metadir('visible', 'VisibleFormat',
+        bzrdir.register_metadir(registry, 'visible', 'VisibleFormat',
             'visible help text', hidden=False)
         format = option.RegistryOption('format', '', registry, str)
         self.assertTrue(format.is_hidden('hidden'))
         self.assertFalse(format.is_hidden('visible'))
 
+    def test_short_name(self):
+        registry = controldir.ControlDirFormatRegistry()
+        opt = option.RegistryOption('format', help='', registry=registry)
+        self.assertEquals(None, opt.short_name())
+        opt = option.RegistryOption('format', short_name='F', help='',
+            registry=registry)
+        self.assertEquals('F', opt.short_name())
+
     def test_option_custom_help(self):
         the_opt = option.Option.OPTIONS['help']
         orig_help = the_opt.help[:]
@@ -403,6 +421,17 @@
         self.assertEqual('suggest lottery numbers', my_opt.help)
         self.assertEqual(orig_help, the_opt.help)
 
+    def test_short_value_switches(self):
+        reg = registry.Registry()
+        reg.register('short', 'ShortChoice')
+        reg.register('long', 'LongChoice')
+        ropt = option.RegistryOption('choice', '', reg, value_switches=True,
+            short_value_switches={'short': 's'})
+        opts, args = parse([ropt], ['--short'])
+        self.assertEqual('ShortChoice', opts.choice)
+        opts, args = parse([ropt], ['-s'])
+        self.assertEqual('ShortChoice', opts.choice)
+
 
 class TestVerboseQuietLinkage(TestCase):
 

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2010-07-08 09:11:44 +0000
+++ bzrlib/tests/test_osutils.py	2011-10-06 07:43:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 import os
 import re
 import socket
-import stat
 import sys
 import time
 
@@ -39,9 +38,10 @@
     file_utils,
     test__walkdirs_win32,
     )
-
-
-class _UTF8DirReaderFeature(tests.Feature):
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+class _UTF8DirReaderFeature(features.Feature):
 
     def _probe(self):
         try:
@@ -54,9 +54,9 @@
     def feature_name(self):
         return 'bzrlib._readdir_pyx'
 
-UTF8DirReaderFeature = _UTF8DirReaderFeature()
+UTF8DirReaderFeature = features.ModuleAvailableFeature('bzrlib._readdir_pyx')
 
-term_ios_feature = tests.ModuleAvailableFeature('termios')
+term_ios_feature = features.ModuleAvailableFeature('termios')
 
 
 def _already_unicode(s):
@@ -97,30 +97,24 @@
     return scenarios
 
 
-def load_tests(basic_tests, module, loader):
-    suite = loader.suiteClass()
-    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
-        basic_tests, tests.condition_isinstance(TestDirReader))
-    tests.multiply_tests(dir_reader_tests, dir_reader_scenarios(), suite)
-    suite.addTest(remaining_tests)
-    return suite
+load_tests = load_tests_apply_scenarios
 
 
 class TestContainsWhitespace(tests.TestCase):
 
     def test_contains_whitespace(self):
-        self.failUnless(osutils.contains_whitespace(u' '))
-        self.failUnless(osutils.contains_whitespace(u'hello there'))
-        self.failUnless(osutils.contains_whitespace(u'hellothere\n'))
-        self.failUnless(osutils.contains_whitespace(u'hello\nthere'))
-        self.failUnless(osutils.contains_whitespace(u'hello\rthere'))
-        self.failUnless(osutils.contains_whitespace(u'hello\tthere'))
+        self.assertTrue(osutils.contains_whitespace(u' '))
+        self.assertTrue(osutils.contains_whitespace(u'hello there'))
+        self.assertTrue(osutils.contains_whitespace(u'hellothere\n'))
+        self.assertTrue(osutils.contains_whitespace(u'hello\nthere'))
+        self.assertTrue(osutils.contains_whitespace(u'hello\rthere'))
+        self.assertTrue(osutils.contains_whitespace(u'hello\tthere'))
 
         # \xa0 is "Non-breaking-space" which on some python locales thinks it
         # is whitespace, but we do not.
-        self.failIf(osutils.contains_whitespace(u''))
-        self.failIf(osutils.contains_whitespace(u'hellothere'))
-        self.failIf(osutils.contains_whitespace(u'hello\xa0there'))
+        self.assertFalse(osutils.contains_whitespace(u''))
+        self.assertFalse(osutils.contains_whitespace(u'hellothere'))
+        self.assertFalse(osutils.contains_whitespace(u'hello\xa0there'))
 
 
 class TestRename(tests.TestCaseInTempDir):
@@ -140,8 +134,8 @@
         # This should work everywhere
         self.create_file('a', 'something in a\n')
         self._fancy_rename('a', 'b')
-        self.failIfExists('a')
-        self.failUnlessExists('b')
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists('b')
         self.check_file_contents('b', 'something in a\n')
 
         self.create_file('a', 'new something in a\n')
@@ -154,7 +148,7 @@
         self.create_file('target', 'data in target\n')
         self.assertRaises((IOError, OSError), self._fancy_rename,
                           'missingsource', 'target')
-        self.failUnlessExists('target')
+        self.assertPathExists('target')
         self.check_file_contents('target', 'data in target\n')
 
     def test_fancy_rename_fails_if_source_and_target_missing(self):
@@ -165,8 +159,8 @@
         # Rename should be semi-atomic on all platforms
         self.create_file('a', 'something in a\n')
         osutils.rename('a', 'b')
-        self.failIfExists('a')
-        self.failUnlessExists('b')
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists('b')
         self.check_file_contents('b', 'something in a\n')
 
         self.create_file('a', 'new something in a\n')
@@ -237,6 +231,25 @@
             self.assertFalse(osutils.is_inside_or_parent_of_any(dirs, fn))
 
 
+class TestLstat(tests.TestCaseInTempDir):
+
+    def test_lstat_matches_fstat(self):
+        # On Windows, lstat and fstat don't always agree, primarily in the
+        # 'st_ino' and 'st_dev' fields. So we force them to be '0' in our
+        # custom implementation.
+        if sys.platform == 'win32':
+            # We only have special lstat/fstat if we have the extension.
+            # Without it, we may end up re-reading content when we don't have
+            # to, but otherwise it doesn't effect correctness.
+            self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
+        f = open('test-file.txt', 'wb')
+        self.addCleanup(f.close)
+        f.write('some content\n')
+        f.flush()
+        self.assertEqualStat(osutils.fstat(f.fileno()),
+                             osutils.lstat('test-file.txt'))
+
+
 class TestRmTree(tests.TestCaseInTempDir):
 
     def test_rmtree(self):
@@ -254,8 +267,8 @@
 
         osutils.rmtree('dir')
 
-        self.failIfExists('dir/file')
-        self.failIfExists('dir')
+        self.assertPathDoesNotExist('dir/file')
+        self.assertPathDoesNotExist('dir')
 
 
 class TestDeleteAny(tests.TestCaseInTempDir):
@@ -416,7 +429,7 @@
 class TestLinks(tests.TestCaseInTempDir):
 
     def test_dereference_path(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         cwd = osutils.realpath('.')
         os.mkdir('bar')
         bar_path = osutils.pathjoin(cwd, 'bar')
@@ -469,19 +482,19 @@
 
 class TestCanonicalRelPath(tests.TestCaseInTempDir):
 
-    _test_needs_features = [tests.CaseInsCasePresFilenameFeature]
+    _test_needs_features = [features.CaseInsCasePresFilenameFeature]
 
     def test_canonical_relpath_simple(self):
         f = file('MixedCaseName', 'w')
         f.close()
         actual = osutils.canonical_relpath(self.test_base_dir, 'mixedcasename')
-        self.failUnlessEqual('work/MixedCaseName', actual)
+        self.assertEqual('work/MixedCaseName', actual)
 
     def test_canonical_relpath_missing_tail(self):
         os.mkdir('MixedCaseParent')
         actual = osutils.canonical_relpath(self.test_base_dir,
                                            'mixedcaseparent/nochild')
-        self.failUnlessEqual('work/MixedCaseParent/nochild', actual)
+        self.assertEqual('work/MixedCaseParent/nochild', actual)
 
 
 class Test_CICPCanonicalRelpath(tests.TestCaseWithTransport):
@@ -805,6 +818,16 @@
         self.assertEqual(None, osutils.safe_file_id(None))
 
 
+class TestPosixFuncs(tests.TestCase):
+    """Test that the posix version of normpath returns an appropriate path
+       when used with 2 leading slashes."""
+
+    def test_normpath(self):
+        self.assertEqual('/etc/shadow', osutils._posix_normpath('/etc/shadow'))
+        self.assertEqual('/etc/shadow', osutils._posix_normpath('//etc/shadow'))
+        self.assertEqual('/etc/shadow', osutils._posix_normpath('///etc/shadow'))
+
+
 class TestWin32Funcs(tests.TestCase):
     """Test that _win32 versions of os utilities return appropriate paths."""
 
@@ -875,7 +898,7 @@
     """Test win32 functions that create files."""
 
     def test_getcwd(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.mkdir(u'mu-\xb5')
         os.chdir(u'mu-\xb5')
         # TODO: jam 20060427 This will probably fail on Mac OSX because
@@ -911,8 +934,8 @@
         b.close()
 
         osutils._win32_rename('b', 'a')
-        self.failUnlessExists('a')
-        self.failIfExists('b')
+        self.assertPathExists('a')
+        self.assertPathDoesNotExist('b')
         self.assertFileEqual('baz\n', 'a')
 
     def test_rename_missing_file(self):
@@ -971,13 +994,13 @@
     """Test mac special functions that require directories."""
 
     def test_getcwd(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.mkdir(u'B\xe5gfors')
         os.chdir(u'B\xe5gfors')
         self.assertEndsWith(osutils._mac_getcwd(), u'B\xe5gfors')
 
     def test_getcwd_nonnorm(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # Test that _mac_getcwd() will normalize this path
         os.mkdir(u'Ba\u030agfors')
         os.chdir(u'Ba\u030agfors')
@@ -1072,6 +1095,7 @@
         if sys.platform == 'win32':
             raise tests.TestNotApplicable(
                 "readdir IOError not tested on win32")
+        self.requireFeature(features.not_running_as_root)
         os.mkdir("test-unreadable")
         os.chmod("test-unreadable", 0000)
         # must chmod it back so that it can be removed
@@ -1092,7 +1116,7 @@
         # are not using the filesystem's encoding
 
         # require a bytestring based filesystem
-        self.requireFeature(tests.ByteStringNamedFilesystem)
+        self.requireFeature(features.ByteStringNamedFilesystem)
 
         tree = [
             '.bzr',
@@ -1107,6 +1131,8 @@
 
         # rename the 1file to a latin-1 filename
         os.rename("./1file", "\xe8file")
+        if "\xe8file" not in os.listdir("."):
+            self.skip("Lack filesystem that preserves arbitrary bytes")
 
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
@@ -1191,21 +1217,24 @@
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'UTF-8'
-        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(
+            UTF8DirReaderFeature.module.UTF8DirReader)
 
     def test_force_walkdirs_utf8_fs_ascii(self):
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'US-ASCII'
-        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(
+            UTF8DirReaderFeature.module.UTF8DirReader)
 
     def test_force_walkdirs_utf8_fs_ANSI(self):
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
         osutils._fs_enc = 'ANSI_X3.4-1968'
-        self.assertDirReaderIs(UTF8DirReaderFeature.reader)
+        self.assertDirReaderIs(
+            UTF8DirReaderFeature.module.UTF8DirReader)
 
     def test_force_walkdirs_utf8_fs_latin1(self):
         self._save_platform_info()
@@ -1229,7 +1258,7 @@
 
     def test_unicode_walkdirs(self):
         """Walkdirs should always return unicode paths."""
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         name0 = u'0file-\xb6'
         name1 = u'1dir-\u062c\u0648'
         name2 = u'2file-\u0633'
@@ -1272,7 +1301,7 @@
 
         The abspath portion might be in unicode or utf-8
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         name0 = u'0file-\xb6'
         name1 = u'1dir-\u062c\u0648'
         name2 = u'2file-\u0633'
@@ -1333,7 +1362,7 @@
 
         The abspath portion should be in unicode
         """
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         # Use the unicode reader. TODO: split into driver-and-driven unit
         # tests.
         self._save_platform_info()
@@ -1380,7 +1409,7 @@
 
     def test__walkdirs_utf8_win32readdir(self):
         self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         from bzrlib._walkdirs_win32 import Win32ReadDir
         self._save_platform_info()
         osutils._selected_dir_reader = Win32ReadDir()
@@ -1437,7 +1466,7 @@
     def test__walkdirs_utf_win32_find_file_stat_file(self):
         """make sure our Stat values are valid"""
         self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         from bzrlib._walkdirs_win32 import Win32ReadDir
         name0u = u'0file-\xb6'
         name0 = name0u.encode('utf8')
@@ -1461,7 +1490,7 @@
     def test__walkdirs_utf_win32_find_file_stat_directory(self):
         """make sure our Stat values are valid"""
         self.requireFeature(test__walkdirs_win32.win32_readdir_feature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         from bzrlib._walkdirs_win32 import Win32ReadDir
         name0u = u'0dir-\u062c\u0648'
         name0 = name0u.encode('utf8')
@@ -1567,7 +1596,7 @@
         self.assertEqual(['c'], os.listdir('target/b'))
 
     def test_copy_tree_symlinks(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.build_tree(['source/'])
         os.symlink('a/generic/path', 'source/lnk')
         osutils.copy_tree('source', 'target')
@@ -1598,7 +1627,7 @@
                           ('d', 'source/b', 'target/b'),
                           ('f', 'source/b/c', 'target/b/c'),
                          ], processed_files)
-        self.failIfExists('target')
+        self.assertPathDoesNotExist('target')
         if osutils.has_symlinks():
             self.assertEqual([('source/lnk', 'target/lnk')], processed_links)
 
@@ -1650,7 +1679,7 @@
         old = osutils.set_or_unset_env('BZR_TEST_ENV_VAR', None)
         self.assertEqual('foo', old)
         self.assertEqual(None, os.environ.get('BZR_TEST_ENV_VAR'))
-        self.failIf('BZR_TEST_ENV_VAR' in os.environ)
+        self.assertFalse('BZR_TEST_ENV_VAR' in os.environ)
 
 
 class TestSizeShaFile(tests.TestCaseInTempDir):
@@ -1733,6 +1762,8 @@
 
 class TestDirReader(tests.TestCaseInTempDir):
 
+    scenarios = dir_reader_scenarios()
+
     # Set by load_tests
     _dir_reader_class = None
     _native_to_unicode = None
@@ -1838,15 +1869,15 @@
         return filtered_dirblocks
 
     def test_walk_unicode_tree(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         tree, expected_dirblocks = self._get_unicode_tree()
         self.build_tree(tree)
         result = list(osutils._walkdirs_utf8('.'))
         self.assertEqual(expected_dirblocks, self._filter_out(result))
 
     def test_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.SymlinkFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         target = u'target\N{Euro Sign}'
         link_name = u'l\N{Euro Sign}nk'
         os.symlink(target, link_name)
@@ -1870,7 +1901,7 @@
     But prior python versions failed to properly encode the passed unicode
     string.
     """
-    _test_needs_features = [tests.SymlinkFeature, tests.UnicodeFilenameFeature]
+    _test_needs_features = [features.SymlinkFeature, features.UnicodeFilenameFeature]
 
     def setUp(self):
         super(tests.TestCaseInTempDir, self).setUp()
@@ -1879,10 +1910,7 @@
         os.symlink(self.target, self.link)
 
     def test_os_readlink_link_encoding(self):
-        if sys.version_info < (2, 6):
-            self.assertRaises(UnicodeEncodeError, os.readlink, self.link)
-        else:
-            self.assertEquals(self.target,  os.readlink(self.link))
+        self.assertEquals(self.target,  os.readlink(self.link))
 
     def test_os_readlink_link_decoding(self):
         self.assertEquals(self.target.encode(osutils._fs_enc),
@@ -1900,17 +1928,17 @@
         self.assertIsInstance(concurrency, int)
 
     def test_local_concurrency_environment_variable(self):
-        os.environ['BZR_CONCURRENCY'] = '2'
+        self.overrideEnv('BZR_CONCURRENCY', '2')
         self.assertEqual(2, osutils.local_concurrency(use_cache=False))
-        os.environ['BZR_CONCURRENCY'] = '3'
+        self.overrideEnv('BZR_CONCURRENCY', '3')
         self.assertEqual(3, osutils.local_concurrency(use_cache=False))
-        os.environ['BZR_CONCURRENCY'] = 'foo'
+        self.overrideEnv('BZR_CONCURRENCY', 'foo')
         self.assertEqual(1, osutils.local_concurrency(use_cache=False))
 
     def test_option_concurrency(self):
-        os.environ['BZR_CONCURRENCY'] = '1'
+        self.overrideEnv('BZR_CONCURRENCY', '1')
         self.run_bzr('rocks --concurrency 42')
-        # Command line overrides envrionment variable
+        # Command line overrides environment variable
         self.assertEquals('42', os.environ['BZR_CONCURRENCY'])
         self.assertEquals(42, osutils.local_concurrency(use_cache=False))
 
@@ -1986,19 +2014,23 @@
     def test_defaults_to_BZR_COLUMNS(self):
         # BZR_COLUMNS is set by the test framework
         self.assertNotEqual('12', os.environ['BZR_COLUMNS'])
-        os.environ['BZR_COLUMNS'] = '12'
+        self.overrideEnv('BZR_COLUMNS', '12')
         self.assertEqual(12, osutils.terminal_width())
 
+    def test_BZR_COLUMNS_0_no_limit(self):
+        self.overrideEnv('BZR_COLUMNS', '0')
+        self.assertEqual(None, osutils.terminal_width())
+
     def test_falls_back_to_COLUMNS(self):
-        del os.environ['BZR_COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
         self.assertNotEqual('42', os.environ['COLUMNS'])
         self.set_fake_tty()
-        os.environ['COLUMNS'] = '42'
+        self.overrideEnv('COLUMNS', '42')
         self.assertEqual(42, osutils.terminal_width())
 
     def test_tty_default_without_columns(self):
-        del os.environ['BZR_COLUMNS']
-        del os.environ['COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
+        self.overrideEnv('COLUMNS', None)
 
         def terminal_size(w, h):
             return 42, 42
@@ -2011,8 +2043,8 @@
         self.assertEqual(42, osutils.terminal_width())
 
     def test_non_tty_default_without_columns(self):
-        del os.environ['BZR_COLUMNS']
-        del os.environ['COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
+        self.overrideEnv('COLUMNS', None)
         self.replace_stdout(None)
         self.assertEqual(None, osutils.terminal_width())
 
@@ -2028,11 +2060,12 @@
         else:
             self.overrideAttr(termios, 'TIOCGWINSZ')
             del termios.TIOCGWINSZ
-        del os.environ['BZR_COLUMNS']
-        del os.environ['COLUMNS']
+        self.overrideEnv('BZR_COLUMNS', None)
+        self.overrideEnv('COLUMNS', None)
         # Whatever the result is, if we don't raise an exception, it's ok.
         osutils.terminal_width()
 
+
 class TestCreationOps(tests.TestCaseInTempDir):
     _test_needs_features = [features.chown_feature]
 
@@ -2068,13 +2101,81 @@
         self.assertEquals(self.uid, s.st_uid)
         self.assertEquals(self.gid, s.st_gid)
 
+
 class TestGetuserUnicode(tests.TestCase):
 
     def test_ascii_user(self):
-        osutils.set_or_unset_env('LOGNAME', 'jrandom')
+        self.overrideEnv('LOGNAME', 'jrandom')
         self.assertEqual(u'jrandom', osutils.getuser_unicode())
 
     def test_unicode_user(self):
         ue = osutils.get_user_encoding()
-        osutils.set_or_unset_env('LOGNAME', u'jrandom\xb6'.encode(ue))
+        uni_val, env_val = tests.probe_unicode_in_user_encoding()
+        if uni_val is None:
+            raise tests.TestSkipped(
+                'Cannot find a unicode character that works in encoding %s'
+                % (osutils.get_user_encoding(),))
+        uni_username = u'jrandom' + uni_val
+        encoded_username = uni_username.encode(ue)
+        self.overrideEnv('LOGNAME', encoded_username)
+        self.assertEqual(uni_username, osutils.getuser_unicode())
+        self.overrideEnv('LOGNAME', u'jrandom\xb6'.encode(ue))
         self.assertEqual(u'jrandom\xb6', osutils.getuser_unicode())
+
+    def test_no_username_bug_660174(self):
+        self.requireFeature(features.win32_feature)
+        for name in ('LOGNAME', 'USER', 'LNAME', 'USERNAME'):
+            self.overrideEnv(name, None)
+        self.assertEqual(u'UNKNOWN', osutils.getuser_unicode())
+
+
+class TestBackupNames(tests.TestCase):
+
+    def setUp(self):
+        super(TestBackupNames, self).setUp()
+        self.backups = []
+
+    def backup_exists(self, name):
+        return name in self.backups
+
+    def available_backup_name(self, name):
+        backup_name = osutils.available_backup_name(name, self.backup_exists)
+        self.backups.append(backup_name)
+        return backup_name
+
+    def assertBackupName(self, expected, name):
+        self.assertEqual(expected, self.available_backup_name(name))
+
+    def test_empty(self):
+        self.assertBackupName('file.~1~', 'file')
+
+    def test_existing(self):
+        self.available_backup_name('file')
+        self.available_backup_name('file')
+        self.assertBackupName('file.~3~', 'file')
+        # Empty slots are found, this is not a strict requirement and may be
+        # revisited if we test against all implementations.
+        self.backups.remove('file.~2~')
+        self.assertBackupName('file.~2~', 'file')
+
+
+class TestFindExecutableInPath(tests.TestCase):
+
+    def test_windows(self):
+        if sys.platform != 'win32':
+            raise tests.TestSkipped('test requires win32')
+        self.assertTrue(osutils.find_executable_on_path('explorer') is not None)
+        self.assertTrue(
+            osutils.find_executable_on_path('explorer.exe') is not None)
+        self.assertTrue(
+            osutils.find_executable_on_path('EXPLORER.EXE') is not None)
+        self.assertTrue(
+            osutils.find_executable_on_path('THIS SHOULD NOT EXIST') is None)
+        self.assertTrue(osutils.find_executable_on_path('file.txt') is None)
+
+    def test_other(self):
+        if sys.platform == 'win32':
+            raise tests.TestSkipped('test requires non-win32')
+        self.assertTrue(osutils.find_executable_on_path('sh') is not None)
+        self.assertTrue(
+            osutils.find_executable_on_path('THIS SHOULD NOT EXIST') is None)

=== modified file 'bzrlib/tests/test_osutils_encodings.py'
--- bzrlib/tests/test_osutils_encodings.py	2010-06-29 16:21:13 +0000
+++ bzrlib/tests/test_osutils_encodings.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,11 +18,9 @@
 
 import codecs
 import locale
-import os
 import sys
 
 from bzrlib import (
-    errors,
     osutils,
     )
 from bzrlib.tests import (
@@ -171,8 +169,6 @@
     def setUp(self):
         TestCase.setUp(self)
         self.overrideAttr(locale, 'getpreferredencoding')
-        self.addCleanup(osutils.set_or_unset_env,
-                        'LANG', os.environ.get('LANG'))
         self.overrideAttr(sys, 'stderr', StringIOWrapper())
 
     def test_get_user_encoding(self):
@@ -181,7 +177,8 @@
 
         locale.getpreferredencoding = f
         fake_codec.add('user_encoding')
-        self.assertEquals('user_encoding', osutils.get_user_encoding(use_cache=False))
+        self.assertEquals('user_encoding',
+                          osutils.get_user_encoding(use_cache=False))
         self.assertEquals('', sys.stderr.getvalue())
 
     def test_user_cp0(self):
@@ -216,7 +213,7 @@
             raise locale.Error, 'unsupported locale'
 
         locale.getpreferredencoding = f
-        os.environ['LANG'] = 'BOGUS'
+        self.overrideEnv('LANG', 'BOGUS')
         self.assertEquals('ascii', osutils.get_user_encoding(use_cache=False))
         self.assertEquals('bzr: warning: unsupported locale\n'
                           '  Could not determine what text encoding to use.\n'

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_permissions.py	2011-02-25 15:24:22 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -32,8 +32,6 @@
 
 import os
 import sys
-import stat
-from cStringIO import StringIO
 import urllib
 
 from bzrlib import transport
@@ -65,7 +63,7 @@
     :param dir_mode: The mode for all directories
     :param include_base: If false, only check the subdirectories
     """
-    t = transport.get_transport(".")
+    t = test.get_transport()
     if include_base:
         test.assertTransportMode(t, base, dir_mode)
     for root, dirs, files in os.walk(base):
@@ -140,10 +138,10 @@
     def test_new_files_group_sticky_bit(self):
         if sys.platform == 'win32':
             raise TestSkipped('chmod has no effect on win32')
-        elif sys.platform == 'darwin' or sys.platform.startswith('freebsd'):
-            # OS X (and FreeBSD) create temp dirs with the 'wheel' group, which
-            # users are not likely to be in, and this prevents us from setting
-            # the sgid bit
+        elif sys.platform == 'darwin' or 'freebsd' in sys.platform:
+            # FreeBSD-based platforms create temp dirs with the 'wheel' group,
+            # which users are not likely to be in, and this prevents us from
+            # setting the sgid bit
             os.chown(self.test_dir, os.getuid(), os.getgid())
 
         t = self.make_branch_and_tree('.')
@@ -180,7 +178,7 @@
 
         # bodge around for stubsftpserver not letting use connect
         # more than once
-        _t = transport.get_transport(self.get_url())
+        _t = self.get_transport()
 
         os.mkdir('local')
         t_local = self.make_branch_and_tree('local')
@@ -257,7 +255,7 @@
         original_umask = os.umask(umask)
 
         try:
-            t = transport.get_transport(self.get_url())
+            t = self.get_transport()
             # Direct access should be masked by umask
             t._sftp_open_exclusive('a', mode=0666).write('foo\n')
             self.assertTransportMode(t, 'a', 0666 &~umask)

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2010-06-11 08:02:42 +0000
+++ bzrlib/tests/test_plugins.py	2011-08-16 07:33:29 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@
 
 # TODO: Write a test for plugin decoration of commands.
 
-class TestPluginMixin(object):
+class BaseTestPlugins(tests.TestCaseInTempDir):
 
     def create_plugin(self, name, source=None, dir='.', file_name=None):
         if source is None:
@@ -91,15 +91,15 @@
                 delattr(plugin, submodule_name)
 
     def assertPluginUnknown(self, name):
-        self.failIf(getattr(bzrlib.plugins, name, None) is not None)
-        self.failIf('bzrlib.plugins.%s' % name in sys.modules)
+        self.assertFalse(getattr(bzrlib.plugins, name, None) is not None)
+        self.assertFalse('bzrlib.plugins.%s' % name in sys.modules)
 
     def assertPluginKnown(self, name):
-        self.failUnless(getattr(bzrlib.plugins, name, None) is not None)
-        self.failUnless('bzrlib.plugins.%s' % name in sys.modules)
-
-
-class TestLoadingPlugins(tests.TestCaseInTempDir, TestPluginMixin):
+        self.assertTrue(getattr(bzrlib.plugins, name, None) is not None)
+        self.assertTrue('bzrlib.plugins.%s' % name in sys.modules)
+
+
+class TestLoadingPlugins(BaseTestPlugins):
 
     activeattributes = {}
 
@@ -109,12 +109,12 @@
         # file name we can use which is also a valid attribute for accessing in
         # activeattributes. - we cannot give import parameters.
         tempattribute = "0"
-        self.failIf(tempattribute in self.activeattributes)
+        self.assertFalse(tempattribute in self.activeattributes)
         # set a place for the plugins to record their loading, and at the same
         # time validate that the location the plugins should record to is
         # valid and correct.
         self.__class__.activeattributes [tempattribute] = []
-        self.failUnless(tempattribute in self.activeattributes)
+        self.assertTrue(tempattribute in self.activeattributes)
         # create two plugin directories
         os.mkdir('first')
         os.mkdir('second')
@@ -147,21 +147,21 @@
         self.assertPluginUnknown('plugin')
 
     def test_plugins_from_different_dirs_can_demand_load(self):
-        self.failIf('bzrlib.plugins.pluginone' in sys.modules)
-        self.failIf('bzrlib.plugins.plugintwo' in sys.modules)
+        self.assertFalse('bzrlib.plugins.pluginone' in sys.modules)
+        self.assertFalse('bzrlib.plugins.plugintwo' in sys.modules)
         # This test tests that having two plugins in different
         # directories with different names allows them both to be loaded, when
         # we do a direct import statement.
         # Determine a file name we can use which is also a valid attribute
         # for accessing in activeattributes. - we cannot give import parameters.
         tempattribute = "different-dirs"
-        self.failIf(tempattribute in self.activeattributes)
+        self.assertFalse(tempattribute in self.activeattributes)
         # set a place for the plugins to record their loading, and at the same
         # time validate that the location the plugins should record to is
         # valid and correct.
         bzrlib.tests.test_plugins.TestLoadingPlugins.activeattributes \
             [tempattribute] = []
-        self.failUnless(tempattribute in self.activeattributes)
+        self.assertTrue(tempattribute in self.activeattributes)
         # create two plugin directories
         os.mkdir('first')
         os.mkdir('second')
@@ -186,8 +186,8 @@
 
         oldpath = bzrlib.plugins.__path__
         try:
-            self.failIf('bzrlib.plugins.pluginone' in sys.modules)
-            self.failIf('bzrlib.plugins.plugintwo' in sys.modules)
+            self.assertFalse('bzrlib.plugins.pluginone' in sys.modules)
+            self.assertFalse('bzrlib.plugins.plugintwo' in sys.modules)
             bzrlib.plugins.__path__ = ['first', 'second']
             exec "import bzrlib.plugins.pluginone"
             self.assertEqual(['first'], self.activeattributes[tempattribute])
@@ -208,13 +208,13 @@
         # check the plugin is not loaded already
         self.assertPluginUnknown('ts_plugin')
         tempattribute = "trailing-slash"
-        self.failIf(tempattribute in self.activeattributes)
+        self.assertFalse(tempattribute in self.activeattributes)
         # set a place for the plugin to record its loading, and at the same
         # time validate that the location the plugin should record to is
         # valid and correct.
         bzrlib.tests.test_plugins.TestLoadingPlugins.activeattributes \
             [tempattribute] = []
-        self.failUnless(tempattribute in self.activeattributes)
+        self.assertTrue(tempattribute in self.activeattributes)
         # create a directory for the plugin
         os.mkdir('plugin_test')
         # write a plugin that will record when its loaded in the
@@ -267,8 +267,13 @@
             stream.close()
 
     def test_plugin_with_bad_api_version_reports(self):
-        # This plugin asks for bzrlib api version 1.0.0, which is not supported
-        # anymore.
+        """Try loading a plugin that requests an unsupported api.
+        
+        Observe that it records the problem but doesn't complain on stderr.
+
+        See https://bugs.launchpad.net/bzr/+bug/704195
+        """
+        self.overrideAttr(plugin, 'plugin_warnings', {})
         name = 'wants100.py'
         f = file(name, 'w')
         try:
@@ -276,9 +281,14 @@
                 "bzrlib.api.require_any_api(bzrlib, [(1, 0, 0)])\n")
         finally:
             f.close()
-
         log = self.load_and_capture(name)
-        self.assertContainsRe(log,
+        self.assertNotContainsRe(log,
+            r"It requested API version")
+        self.assertEquals(
+            ['wants100'],
+            plugin.plugin_warnings.keys())
+        self.assertContainsRe(
+            plugin.plugin_warnings['wants100'][0],
             r"It requested API version")
 
     def test_plugin_with_bad_name_does_not_load(self):
@@ -292,7 +302,7 @@
             "it to 'bad_plugin_name_'\.")
 
 
-class TestPlugins(tests.TestCaseInTempDir, TestPluginMixin):
+class TestPlugins(BaseTestPlugins):
 
     def setup_plugin(self, source=""):
         # This test tests a new plugin appears in bzrlib.plugin.plugins().
@@ -442,7 +452,7 @@
     def test_final_fallback__version__with_version_info(self):
         self.setup_plugin("version_info = (1, 2, 3, 'final', 2)")
         plugin = bzrlib.plugin.plugins()['plugin']
-        self.assertEqual("1.2.3.final.2", plugin.__version__)
+        self.assertEqual("1.2.3.2", plugin.__version__)
 
 
 class TestPluginHelp(tests.TestCaseInTempDir):
@@ -605,15 +615,16 @@
     def test_get_help_text_with_additional_see_also(self):
         mod = FakeModule('two lines of help\nand more', 'demo')
         topic = plugin.ModuleHelpTopic(mod)
-        self.assertEqual("two lines of help\nand more\nSee also: bar, foo\n",
-            topic.get_help_text(['foo', 'bar']))
+        self.assertEqual("two lines of help\nand more\n\n:See also: bar, foo\n",
+                         topic.get_help_text(['foo', 'bar']))
 
     def test_get_help_topic(self):
         """The help topic for a plugin is its module name."""
         mod = FakeModule('two lines of help\nand more', 'bzrlib.plugins.demo')
         topic = plugin.ModuleHelpTopic(mod)
         self.assertEqual('demo', topic.get_help_topic())
-        mod = FakeModule('two lines of help\nand more', 'bzrlib.plugins.foo_bar')
+        mod = FakeModule('two lines of help\nand more',
+                         'bzrlib.plugins.foo_bar')
         topic = plugin.ModuleHelpTopic(mod)
         self.assertEqual('foo_bar', topic.get_help_topic())
 
@@ -656,7 +667,7 @@
                     self.fail('No path to global plugins')
 
     def test_get_standard_plugins_path_env(self):
-        os.environ['BZR_PLUGIN_PATH'] = 'foo/'
+        self.overrideEnv('BZR_PLUGIN_PATH', 'foo/')
         path = plugin.get_standard_plugins_path()
         for directory in path:
             self.assertNotContainsRe(directory, r'\\/$')
@@ -692,7 +703,7 @@
 
     def _set_path(self, *args):
         path = os.pathsep.join(self._list2paths(*args))
-        osutils.set_or_unset_env('BZR_PLUGIN_PATH', path)
+        self.overrideEnv('BZR_PLUGIN_PATH', path)
 
     def check_path(self, expected_dirs, setting_dirs):
         if setting_dirs:
@@ -768,7 +779,7 @@
                         ['+foo', '-bar'])
 
 
-class TestDisablePlugin(tests.TestCaseInTempDir, TestPluginMixin):
+class TestDisablePlugin(BaseTestPlugins):
 
     def setUp(self):
         super(TestDisablePlugin, self).setUp()
@@ -779,7 +790,7 @@
         self.addCleanup(self._unregister_plugin, 'test_foo')
 
     def test_cannot_import(self):
-        osutils.set_or_unset_env('BZR_DISABLE_PLUGINS', 'test_foo')
+        self.overrideEnv('BZR_DISABLE_PLUGINS', 'test_foo')
         plugin.set_plugins_path(['.'])
         try:
             import bzrlib.plugins.test_foo
@@ -801,7 +812,7 @@
         self.overrideAttr(trace, 'warning', captured_warning)
         # Reset the flag that protect against double loading
         self.overrideAttr(plugin, '_loaded', False)
-        osutils.set_or_unset_env('BZR_DISABLE_PLUGINS', 'test_foo')
+        self.overrideEnv('BZR_DISABLE_PLUGINS', 'test_foo')
         plugin.load_plugins(['.'])
         self.assertPluginUnknown('test_foo')
         # Make sure we don't warn about the plugin ImportError since this has
@@ -809,6 +820,7 @@
         self.assertLength(0, self.warnings)
 
 
+
 class TestLoadPluginAtSyntax(tests.TestCase):
 
     def _get_paths(self, paths):
@@ -832,7 +844,7 @@
                           os.pathsep.join(['batman@cave', '', 'robin@mobile']))
 
 
-class TestLoadPluginAt(tests.TestCaseInTempDir, TestPluginMixin):
+class TestLoadPluginAt(BaseTestPlugins):
 
     def setUp(self):
         super(TestLoadPluginAt, self).setUp()
@@ -847,6 +859,9 @@
         self.create_plugin_package('test_foo', dir='standard/test_foo')
         # All the tests will load the 'test_foo' plugin from various locations
         self.addCleanup(self._unregister_plugin, 'test_foo')
+        # Unfortunately there's global cached state for the specific
+        # registered paths.
+        self.addCleanup(plugin.PluginImporter.reset)
 
     def assertTestFooLoadedFrom(self, path):
         self.assertPluginKnown('test_foo')
@@ -859,7 +874,7 @@
         self.assertTestFooLoadedFrom('standard/test_foo')
 
     def test_import(self):
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.set_plugins_path(['standard'])
         try:
             import bzrlib.plugins.test_foo
@@ -868,12 +883,12 @@
         self.assertTestFooLoadedFrom('non-standard-dir')
 
     def test_loading(self):
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom('non-standard-dir')
 
     def test_compiled_loaded(self):
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom('non-standard-dir')
         self.assertIsSameRealPath('non-standard-dir/__init__.py',
@@ -896,7 +911,7 @@
         self.create_plugin_package('test_bar', dir='non-standard-dir/test_bar')
         self.addCleanup(self._unregister_plugin_submodule,
                         'test_foo', 'test_bar')
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.set_plugins_path(['standard'])
         import bzrlib.plugins.test_foo
         self.assertEqual('bzrlib.plugins.test_foo',
@@ -913,7 +928,7 @@
         self.create_plugin_package('test_bar', dir='another-dir/test_bar')
         self.addCleanup(self._unregister_plugin_submodule,
                         'test_foo', 'test_bar')
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@another-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@another-dir')
         plugin.set_plugins_path(['standard'])
         import bzrlib.plugins.test_foo
         self.assertEqual('bzrlib.plugins.test_foo',
@@ -928,7 +943,7 @@
         random = 'non-standard-dir/setup.py'
         os.rename(init, random)
         self.addCleanup(os.rename, random, init)
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@non-standard-dir')
         plugin.load_plugins(['standard'])
         self.assertPluginUnknown('test_foo')
 
@@ -942,6 +957,29 @@
 ''' % ('test_foo', plugin_path)
         self.create_plugin('test_foo', source=source,
                            dir=plugin_dir, file_name=plugin_file_name)
-        osutils.set_or_unset_env('BZR_PLUGINS_AT', 'test_foo@%s' % plugin_path)
+        self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@%s' % plugin_path)
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom(plugin_path)
+
+
+class TestDescribePlugins(BaseTestPlugins):
+
+    def test_describe_plugins(self):
+        class DummyModule(object):
+            __doc__ = 'Hi there'
+        class DummyPlugin(object):
+            __version__ = '0.1.0'
+            module = DummyModule()
+        def dummy_plugins():
+            return { 'good': DummyPlugin() }
+        self.overrideAttr(plugin, 'plugin_warnings',
+            {'bad': ['Failed to load (just testing)']})
+        self.overrideAttr(plugin, 'plugins', dummy_plugins)
+        self.assertEquals("""\
+bad (failed to load)
+  ** Failed to load (just testing)
+
+good 0.1.0
+  Hi there
+
+""", ''.join(plugin.describe_plugins()))

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2010-07-09 15:47:01 +0000
+++ bzrlib/tests/test_progress.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,16 +15,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import os
 from StringIO import StringIO
 
-from bzrlib import errors
 from bzrlib.progress import (
     ProgressTask,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import TestCase
 from bzrlib.ui.text import (
     TextProgressView,
@@ -70,6 +65,21 @@
         task.total_cnt = total
         return task
 
+    def test_clear(self):
+        # <https://bugs.launchpad.net/bzr/+bug/611127> clear must actually
+        # send spaces to clear the line
+        out, view = self.make_view()
+        task = self.make_task(None, view, 'reticulating splines', 5, 20)
+        view.show_progress(task)
+        self.assertEqual(
+'\r/ reticulating splines 5/20                                                    \r'
+            , out.getvalue())
+        view.clear()
+        self.assertEqual(
+'\r/ reticulating splines 5/20                                                    \r'
+            + '\r' + 79 * ' ' + '\r',
+            out.getvalue())
+
     def test_render_progress_no_bar(self):
         """The default view now has a spinner but no bar."""
         out, view = self.make_view()

=== added file 'bzrlib/tests/test_pyutils.py'
--- bzrlib/tests/test_pyutils.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_pyutils.py	2010-09-21 03:20:09 +0000
@@ -0,0 +1,88 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for bzrlib.pyutils."""
+
+from bzrlib import (
+    branch,
+    tests,
+    )
+from bzrlib.pyutils import (
+    calc_parent_name,
+    get_named_object,
+    )
+
+
+class TestGetNamedObject(tests.TestCase):
+    """Tests for get_named_object."""
+
+    def test_module_only(self):
+        import sys
+        self.assertIs(sys, get_named_object('sys'))
+
+    def test_dotted_module(self):
+        self.assertIs(branch, get_named_object('bzrlib.branch'))
+
+    def test_module_attr(self):
+        self.assertIs(
+            branch.Branch, get_named_object('bzrlib.branch', 'Branch'))
+
+    def test_dotted_attr(self):
+        self.assertIs(
+            branch.Branch.hooks,
+            get_named_object('bzrlib.branch', 'Branch.hooks'))
+
+    def test_package(self):
+        # bzrlib.tests is a package, not simply a module
+        self.assertIs(tests, get_named_object('bzrlib.tests'))
+
+    def test_package_attr(self):
+        # bzrlib.tests is a package, not simply a module
+        self.assertIs(
+            tests.TestCase, get_named_object('bzrlib.tests', 'TestCase'))
+
+    def test_import_error(self):
+        self.assertRaises(ImportError, get_named_object, 'NO_SUCH_MODULE')
+
+    def test_attribute_error(self):
+        self.assertRaises(
+            AttributeError, get_named_object, 'sys', 'NO_SUCH_ATTR')
+
+
+
+class TestCalcParent_name(tests.TestCase):
+    """Tests for calc_parent_name."""
+
+    def test_dotted_member(self):
+        self.assertEqual(
+            ('mod_name', 'attr1', 'attr2'),
+            calc_parent_name('mod_name', 'attr1.attr2'))
+
+    def test_undotted_member(self):
+        self.assertEqual(
+            ('mod_name', None, 'attr1'),
+            calc_parent_name('mod_name', 'attr1'))
+
+    def test_dotted_module_no_member(self):
+        self.assertEqual(
+            ('mod', None, 'sub_mod'),
+            calc_parent_name('mod.sub_mod'))
+
+    def test_undotted_module_no_member(self):
+        err = self.assertRaises(AssertionError, calc_parent_name, 'mod_name')
+        self.assertEqual(
+            "No parent object for top-level module 'mod_name'", err.args[0])
+

=== modified file 'bzrlib/tests/test_read_bundle.py'
--- bzrlib/tests/test_read_bundle.py	2010-07-15 12:53:44 +0000
+++ bzrlib/tests/test_read_bundle.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,28 +17,20 @@
 """Test read_bundle works properly across various transports."""
 
 import cStringIO
-import os
 
 import bzrlib.bundle
 from bzrlib.bundle.serializer import write_bundle
 import bzrlib.bzrdir
 import bzrlib.errors as errors
-from bzrlib.symbol_versioning import deprecated_in
 from bzrlib import tests
 from bzrlib.tests.test_transport import TestTransportImplementation
 from bzrlib.tests.per_transport import transport_test_permutations
 import bzrlib.transport
-from bzrlib.transport.memory import MemoryTransport
 import bzrlib.urlutils
-
-
-def load_tests(standard_tests, module, loader):
-    """Multiply tests for tranport implementations."""
-    transport_tests, remaining_tests = tests.split_suite_by_condition(
-        standard_tests,
-        tests.condition_isinstance(TestReadMergeableBundleFromURL))
-    return tests.multiply_tests(transport_tests, transport_test_permutations(),
-        remaining_tests)
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
 
 
 def create_bundle_file(test_case):
@@ -64,6 +56,8 @@
 class TestReadMergeableBundleFromURL(TestTransportImplementation):
     """Test that read_bundle works properly across multiple transports"""
 
+    scenarios = transport_test_permutations()
+
     def setUp(self):
         super(TestReadMergeableBundleFromURL, self).setUp()
         self.bundle_name = 'test_bundle'
@@ -72,7 +66,7 @@
         # from the one we want to test, so we must inject a correct transport
         # into possible_transports first).
         self.possible_transports = [self.get_transport(self.bundle_name)]
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         wt = self.create_test_bundle()
 
     def read_mergeable_from_url(self, url):

=== modified file 'bzrlib/tests/test_reconcile.py'
--- bzrlib/tests/test_reconcile.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_reconcile.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,9 +23,7 @@
     tests,
     )
 from bzrlib.reconcile import reconcile, Reconciler
-from bzrlib.revision import Revision
 from bzrlib.tests import per_repository
-from bzrlib.workingtree import WorkingTree
 
 
 class TestWorksWithSharedRepositories(per_repository.TestCaseWithRepository):

=== modified file 'bzrlib/tests/test_reconfigure.py'
--- bzrlib/tests/test_reconfigure.py	2009-10-06 14:40:37 +0000
+++ bzrlib/tests/test_reconfigure.py	2011-06-10 08:11:44 +0000
@@ -46,10 +46,9 @@
 
     def test_tree_with_pending_merge_to_branch(self):
         tree = self.make_branch_and_tree('tree')
+        tree.commit('unchanged')
         other_tree = tree.bzrdir.sprout('other').open_workingtree()
-        self.build_tree(['other/file'])
-        other_tree.add('file')
-        other_tree.commit('file added')
+        other_tree.commit('mergeable commit')
         tree.merge_from_branch(other_tree.branch)
         reconfiguration = reconfigure.Reconfigure.to_branch(tree.bzrdir)
         self.assertRaises(errors.UncommittedChanges, reconfiguration.apply)

=== modified file 'bzrlib/tests/test_registry.py'
--- bzrlib/tests/test_registry.py	2009-09-16 10:37:29 +0000
+++ bzrlib/tests/test_registry.py	2011-05-26 21:02:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 import sys
 
 from bzrlib import (
-    errors,
+    branch,
     osutils,
     registry,
     tests,
@@ -39,7 +39,7 @@
         a_registry = registry.Registry()
         self.register_stuff(a_registry)
 
-        self.failUnless(a_registry.default_key is None)
+        self.assertTrue(a_registry.default_key is None)
 
         # test get() (self.default_key is None)
         self.assertRaises(KeyError, a_registry.get)
@@ -49,7 +49,7 @@
 
         # test _set_default_key
         a_registry.default_key = 'five'
-        self.failUnless(a_registry.default_key == 'five')
+        self.assertTrue(a_registry.default_key == 'five')
         self.assertEqual(5, a_registry.get())
         self.assertEqual(5, a_registry.get(None))
         # If they ask for a specific entry, they should get KeyError
@@ -64,9 +64,9 @@
         a_registry = registry.Registry()
         self.register_stuff(a_registry)
 
-        self.failUnless('one' in a_registry)
+        self.assertTrue('one' in a_registry)
         a_registry.remove('one')
-        self.failIf('one' in a_registry)
+        self.assertFalse('one' in a_registry)
         self.assertRaises(KeyError, a_registry.get, 'one')
 
         a_registry.register('one', 'one')
@@ -215,11 +215,13 @@
         # We create a registry with "official" objects and "hidden"
         # objects. The later represent the side effects that led to bug #277048
         # and #430510
-        self.registry =  registry.Registry()
+        _registry = registry.Registry()
 
         def register_more():
-            self.registry.register('hidden', None)
+           _registry.register('hidden', None)
 
+        # Avoid closing over self by binding local variable
+        self.registry = _registry
         self.registry.register('passive', None)
         self.registry.register('active', register_more)
         self.registry.register('passive-too', None)
@@ -229,7 +231,7 @@
             def get_obj(inner_self):
                 # Surprise ! Getting a registered object (think lazy loaded
                 # module) register yet another object !
-                self.registry.register('more hidden', None)
+                _registry.register('more hidden', None)
                 return inner_self._obj
 
         self.registry.register('hacky', None)
@@ -286,6 +288,13 @@
             '\n\n'
         )
 
+    def test_lazy_import_registry_foo(self):
+        a_registry = registry.Registry()
+        a_registry.register_lazy('foo', 'bzrlib.branch', 'Branch')
+        a_registry.register_lazy('bar', 'bzrlib.branch', 'Branch.hooks')
+        self.assertEqual(branch.Branch, a_registry.get('foo'))
+        self.assertEqual(branch.Branch.hooks, a_registry.get('bar'))
+
     def test_lazy_import_registry(self):
         plugin_name = self.create_simple_plugin()
         a_registry = registry.Registry()
@@ -297,7 +306,7 @@
         self.assertEqual(['function', 'klass', 'module', 'obj'],
                          sorted(a_registry.keys()))
         # The plugin should not be loaded until we grab the first object
-        self.failIf(plugin_name in sys.modules)
+        self.assertFalse(plugin_name in sys.modules)
 
         # By default the plugin won't be in the search path
         self.assertRaises(ImportError, a_registry.get, 'obj')
@@ -307,7 +316,7 @@
         try:
             obj = a_registry.get('obj')
             self.assertEqual('foo', obj)
-            self.failUnless(plugin_name in sys.modules)
+            self.assertTrue(plugin_name in sys.modules)
 
             # Now grab another object
             func = a_registry.get('function')
@@ -331,3 +340,17 @@
         finally:
             sys.path.remove(plugin_path)
 
+    def test_lazy_import_get_module(self):
+        a_registry = registry.Registry()
+        a_registry.register_lazy('obj', "bzrlib.tests.test_registry",
+            'object1')
+        self.assertEquals("bzrlib.tests.test_registry",
+            a_registry._get_module("obj"))
+
+    def test_normal_get_module(self):
+        class AThing(object):
+            """Something"""
+        a_registry = registry.Registry()
+        a_registry.register("obj", AThing())
+        self.assertEquals("bzrlib.tests.test_registry",
+            a_registry._get_module("obj"))

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2010-07-13 19:02:12 +0000
+++ bzrlib/tests/test_remote.py	2011-09-25 23:58:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,21 +30,24 @@
     branch,
     bzrdir,
     config,
+    controldir,
     errors,
-    graph,
+    graph as _mod_graph,
     inventory,
     inventory_delta,
-    pack,
     remote,
     repository,
     tests,
     transport,
     treebuilder,
-    urlutils,
     versionedfile,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir, BzrDirFormat
+from bzrlib.bzrdir import (
+    BzrDir,
+    BzrDirFormat,
+    RemoteBzrProber,
+    )
 from bzrlib.remote import (
     RemoteBranch,
     RemoteBranchFormat,
@@ -53,36 +56,38 @@
     RemoteRepository,
     RemoteRepositoryFormat,
     )
-from bzrlib.repofmt import groupcompress_repo, pack_repo
+from bzrlib.repofmt import groupcompress_repo, knitpack_repo
 from bzrlib.revision import NULL_REVISION
-from bzrlib.smart import medium
+from bzrlib.smart import medium, request
 from bzrlib.smart.client import _SmartClient
-from bzrlib.smart.repository import SmartServerRepositoryGetParentMap
+from bzrlib.smart.repository import (
+    SmartServerRepositoryGetParentMap,
+    SmartServerRepositoryGetStream_1_19,
+    )
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests import (
-    condition_isinstance,
-    split_suite_by_condition,
-    multiply_tests,
     test_server,
     )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
 from bzrlib.transport.memory import MemoryTransport
 from bzrlib.transport.remote import (
     RemoteTransport,
     RemoteSSHTransport,
     RemoteTCPTransport,
-)
-
-def load_tests(standard_tests, module, loader):
-    to_adapt, result = split_suite_by_condition(
-        standard_tests, condition_isinstance(BasicRemoteObjectTests))
-    smart_server_version_scenarios = [
+    )
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class BasicRemoteObjectTests(tests.TestCaseWithTransport):
+
+    scenarios = [
         ('HPSS-v2',
-         {'transport_server': test_server.SmartTCPServer_for_testing_v2_only}),
+            {'transport_server': test_server.SmartTCPServer_for_testing_v2_only}),
         ('HPSS-v3',
-         {'transport_server': test_server.SmartTCPServer_for_testing})]
-    return multiply_tests(to_adapt, smart_server_version_scenarios, result)
-
-
-class BasicRemoteObjectTests(tests.TestCaseWithTransport):
+            {'transport_server': test_server.SmartTCPServer_for_testing})]
+
 
     def setUp(self):
         super(BasicRemoteObjectTests, self).setUp()
@@ -92,12 +97,12 @@
         self.addCleanup(self.transport.disconnect)
 
     def test_create_remote_bzrdir(self):
-        b = remote.RemoteBzrDir(self.transport, remote.RemoteBzrDirFormat())
+        b = remote.RemoteBzrDir(self.transport, RemoteBzrDirFormat())
         self.assertIsInstance(b, BzrDir)
 
     def test_open_remote_branch(self):
         # open a standalone branch in the working directory
-        b = remote.RemoteBzrDir(self.transport, remote.RemoteBzrDirFormat())
+        b = remote.RemoteBzrDir(self.transport, RemoteBzrDirFormat())
         branch = b.open_branch()
         self.assertIsInstance(branch, Branch)
 
@@ -111,17 +116,19 @@
 
     def test_remote_branch_revision_history(self):
         b = BzrDir.open_from_transport(self.transport).open_branch()
-        self.assertEqual([], b.revision_history())
+        self.assertEqual([],
+            self.applyDeprecated(deprecated_in((2, 5, 0)), b.revision_history))
         r1 = self.local_wt.commit('1st commit')
         r2 = self.local_wt.commit('1st commit', rev_id=u'\xc8'.encode('utf8'))
-        self.assertEqual([r1, r2], b.revision_history())
+        self.assertEqual([r1, r2],
+            self.applyDeprecated(deprecated_in((2, 5, 0)), b.revision_history))
 
     def test_find_correct_format(self):
         """Should open a RemoteBzrDir over a RemoteTransport"""
         fmt = BzrDirFormat.find_format(self.transport)
-        self.assertTrue(RemoteBzrDirFormat
-                        in BzrDirFormat._control_server_formats)
-        self.assertIsInstance(fmt, remote.RemoteBzrDirFormat)
+        self.assertTrue(bzrdir.RemoteBzrProber
+                        in controldir.ControlDirFormat._server_probers)
+        self.assertIsInstance(fmt, RemoteBzrDirFormat)
 
     def test_open_detected_smart_format(self):
         fmt = BzrDirFormat.find_format(self.transport)
@@ -447,7 +454,7 @@
         client.add_expected_call(
             'BzrDir.open_branchV3', ('quack/',),
             'success', ('ref', self.get_url('referenced'))),
-        a_bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         result = a_bzrdir.cloning_metadir()
         # We should have got a control dir matching the referenced branch.
@@ -466,7 +473,7 @@
         client.add_expected_call(
             'BzrDir.cloning_metadir', ('quack/', 'False'),
             'success', (control_name, '', ('branch', ''))),
-        a_bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         result = a_bzrdir.cloning_metadir()
         # We should have got a reference control dir with default branch and
@@ -492,14 +499,14 @@
         client.add_expected_call(
             'BzrDir.open_2.1', ('quack/',), 'success', ('no',))
         self.assertRaises(errors.NotBranchError, RemoteBzrDir, transport,
-                remote.RemoteBzrDirFormat(), _client=client, _force_probe=True)
+                RemoteBzrDirFormat(), _client=client, _force_probe=True)
         self.assertFinished(client)
 
     def test_present_without_workingtree(self):
         client, transport = self.make_fake_client_and_transport()
         client.add_expected_call(
             'BzrDir.open_2.1', ('quack/',), 'success', ('yes', 'no'))
-        bd = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bd = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client, _force_probe=True)
         self.assertIsInstance(bd, RemoteBzrDir)
         self.assertFalse(bd.has_workingtree())
@@ -510,7 +517,7 @@
         client, transport = self.make_fake_client_and_transport()
         client.add_expected_call(
             'BzrDir.open_2.1', ('quack/',), 'success', ('yes', 'yes'))
-        bd = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bd = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client, _force_probe=True)
         self.assertIsInstance(bd, RemoteBzrDir)
         self.assertTrue(bd.has_workingtree())
@@ -523,7 +530,7 @@
             'BzrDir.open_2.1', ('quack/',), 'unknown', ('BzrDir.open_2.1',))
         client.add_expected_call(
             'BzrDir.open', ('quack/',), 'success', ('yes',))
-        bd = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bd = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client, _force_probe=True)
         self.assertIsInstance(bd, RemoteBzrDir)
         self.assertFinished(client)
@@ -545,7 +552,7 @@
             'BzrDir.open_2.1', ('quack/',), 'unknown', ('BzrDir.open_2.1',))
         client.add_expected_call(
             'BzrDir.open', ('quack/',), 'success', ('yes',))
-        bd = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bd = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client, _force_probe=True)
         self.assertIsInstance(bd, RemoteBzrDir)
         self.assertFinished(client)
@@ -582,7 +589,7 @@
         client.add_expected_call(
             'Branch.get_stacked_on_url', ('quack/',),
             'error', ('NotStacked',))
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         result = bzrdir.open_branch()
         self.assertIsInstance(result, RemoteBranch)
@@ -595,7 +602,7 @@
         transport = transport.clone('quack')
         client = FakeClient(transport.base)
         client.add_error_response('nobranch')
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         self.assertRaises(errors.NotBranchError, bzrdir.open_branch)
         self.assertEqual(
@@ -612,7 +619,7 @@
         transport = MemoryTransport()
         # no requests on the network - catches other api calls being made.
         client = FakeClient(transport.base)
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         # patch the open_branch call to record that it was called.
         bzrdir.open_branch = open_branch
@@ -637,7 +644,7 @@
         client.add_expected_call(
             'Branch.get_stacked_on_url', ('~hello/',),
             'error', ('NotStacked',))
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         result = bzrdir.open_branch()
         self.assertFinished(client)
@@ -660,7 +667,7 @@
         client.add_success_response(
             'ok', '', rich_response, subtree_response, external_lookup,
             network_name)
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         result = bzrdir.open_repository()
         self.assertEqual(
@@ -683,7 +690,7 @@
         old.
         """
         self.assertRaises(errors.NotBranchError,
-            RemoteBzrDirFormat.probe_transport, OldServerTransport())
+            RemoteBzrProber.probe_transport, OldServerTransport())
 
 
 class TestBzrDirCreateBranch(TestRemote):
@@ -712,7 +719,7 @@
             'BzrDir.create_branch', ('quack/', network_name),
             'success', ('ok', network_name, '', 'no', 'no', 'yes',
             reference_repo_name))
-        a_bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         branch = a_bzrdir.create_branch()
         # We should have got a remote branch
@@ -721,6 +728,34 @@
         format = branch._format
         self.assertEqual(network_name, format.network_name())
 
+    def test_already_open_repo_and_reused_medium(self):
+        """Bug 726584: create_branch(..., repository=repo) should work
+        regardless of what the smart medium's base URL is.
+        """
+        self.transport_server = test_server.SmartTCPServer_for_testing
+        transport = self.get_transport('.')
+        repo = self.make_repository('quack')
+        # Client's medium rooted a transport root (not at the bzrdir)
+        client = FakeClient(transport.base)
+        transport = transport.clone('quack')
+        reference_bzrdir_format = bzrdir.format_registry.get('default')()
+        reference_format = reference_bzrdir_format.get_branch_format()
+        network_name = reference_format.network_name()
+        reference_repo_fmt = reference_bzrdir_format.repository_format
+        reference_repo_name = reference_repo_fmt.network_name()
+        client.add_expected_call(
+            'BzrDir.create_branch', ('extra/quack/', network_name),
+            'success', ('ok', network_name, '', 'no', 'no', 'yes',
+            reference_repo_name))
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        branch = a_bzrdir.create_branch(repository=repo)
+        # We should have got a remote branch
+        self.assertIsInstance(branch, remote.RemoteBranch)
+        # its format should have the settings from the response
+        format = branch._format
+        self.assertEqual(network_name, format.network_name())
+
 
 class TestBzrDirCreateRepository(TestRemote):
 
@@ -747,7 +782,7 @@
                 'Bazaar repository format 2a (needs bzr 1.16 or later)\n',
                 'False'),
             'success', ('ok', 'yes', 'yes', 'yes', network_name))
-        a_bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         repo = a_bzrdir.create_repository()
         # We should have got a remote repository
@@ -782,7 +817,7 @@
         client.add_success_response('stat', '0', '65535')
         remote_transport = RemoteTransport(server_url + 'quack/', medium=False,
             _client=client)
-        bzrdir = RemoteBzrDir(remote_transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(remote_transport, RemoteBzrDirFormat(),
             _client=client)
         repo = bzrdir.open_repository()
         self.assertEqual(
@@ -815,7 +850,7 @@
         client.add_success_response('stat', '0', '65535')
         remote_transport = RemoteTransport(server_url + 'quack/', medium=False,
             _client=client)
-        bzrdir = RemoteBzrDir(remote_transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(remote_transport, RemoteBzrDirFormat(),
             _client=client)
         repo = bzrdir.open_repository()
         self.assertEqual(
@@ -836,7 +871,7 @@
         transport = transport.clone('quack')
         client = FakeClient(transport.base)
         client.add_success_response('ok', '', 'no', 'no', 'no', network_name)
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         repo = bzrdir.open_repository()
         self.assertEqual(
@@ -849,7 +884,7 @@
 
     def test_success(self):
         """Simple test for typical successful call."""
-        fmt = bzrdir.RemoteBzrDirFormat()
+        fmt = RemoteBzrDirFormat()
         default_format_name = BzrDirFormat.get_default_format().network_name()
         transport = self.get_transport()
         client = FakeClient(transport.base)
@@ -871,7 +906,7 @@
         """Error responses are translated, e.g. 'PermissionDenied' raises the
         corresponding error from the client.
         """
-        fmt = bzrdir.RemoteBzrDirFormat()
+        fmt = RemoteBzrDirFormat()
         default_format_name = BzrDirFormat.get_default_format().network_name()
         transport = self.get_transport()
         client = FakeClient(transport.base)
@@ -895,7 +930,7 @@
         """Integration test for error translation."""
         transport = self.make_smart_server('foo')
         transport = transport.clone('no-such-path')
-        fmt = bzrdir.RemoteBzrDirFormat()
+        fmt = RemoteBzrDirFormat()
         err = self.assertRaises(errors.NoSuchFile,
             fmt.initialize_on_transport_ex, transport, create_prefix=False)
 
@@ -932,7 +967,7 @@
 
     def make_remote_bzrdir(self, transport, client):
         """Make a RemotebzrDir using 'client' as the _client."""
-        return RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        return RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
 
 
@@ -1140,6 +1175,115 @@
             [('set_tags_bytes', 'tags bytes')] * 2, real_branch.calls)
 
 
+class TestBranchHeadsToFetch(RemoteBranchTestCase):
+
+    def test_uses_last_revision_info_and_tags_by_default(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.last_revision_info', ('quack/',),
+            'success', ('ok', '1', 'rev-tip'))
+        client.add_expected_call(
+            'Branch.get_config_file', ('quack/',),
+            'success', ('ok',), '')
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        result = branch.heads_to_fetch()
+        self.assertFinished(client)
+        self.assertEqual((set(['rev-tip']), set()), result)
+
+    def test_uses_last_revision_info_and_tags_when_set(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.last_revision_info', ('quack/',),
+            'success', ('ok', '1', 'rev-tip'))
+        client.add_expected_call(
+            'Branch.get_config_file', ('quack/',),
+            'success', ('ok',), 'branch.fetch_tags = True')
+        # XXX: this will break if the default format's serialization of tags
+        # changes, or if the RPC for fetching tags changes from get_tags_bytes.
+        client.add_expected_call(
+            'Branch.get_tags_bytes', ('quack/',),
+            'success', ('d5:tag-17:rev-foo5:tag-27:rev-bare',))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        result = branch.heads_to_fetch()
+        self.assertFinished(client)
+        self.assertEqual(
+            (set(['rev-tip']), set(['rev-foo', 'rev-bar'])), result)
+
+    def test_uses_rpc_for_formats_with_non_default_heads_to_fetch(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.heads_to_fetch', ('quack/',),
+            'success', (['tip'], ['tagged-1', 'tagged-2']))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        branch._format._use_default_local_heads_to_fetch = lambda: False
+        result = branch.heads_to_fetch()
+        self.assertFinished(client)
+        self.assertEqual((set(['tip']), set(['tagged-1', 'tagged-2'])), result)
+
+    def make_branch_with_tags(self):
+        self.setup_smart_server_with_call_log()
+        # Make a branch with a single revision.
+        builder = self.make_branch_builder('foo')
+        builder.start_series()
+        builder.build_snapshot('tip', None, [
+            ('add', ('', 'root-id', 'directory', ''))])
+        builder.finish_series()
+        branch = builder.get_branch()
+        # Add two tags to that branch
+        branch.tags.set_tag('tag-1', 'rev-1')
+        branch.tags.set_tag('tag-2', 'rev-2')
+        return branch
+
+    def test_backwards_compatible(self):
+        branch = self.make_branch_with_tags()
+        c = branch.get_config()
+        c.set_user_option('branch.fetch_tags', 'True')
+        self.addCleanup(branch.lock_read().unlock)
+        # Disable the heads_to_fetch verb
+        verb = 'Branch.heads_to_fetch'
+        self.disable_verb(verb)
+        self.reset_smart_call_log()
+        result = branch.heads_to_fetch()
+        self.assertEqual((set(['tip']), set(['rev-1', 'rev-2'])), result)
+        self.assertEqual(
+            ['Branch.last_revision_info', 'Branch.get_config_file',
+             'Branch.get_tags_bytes'],
+            [call.call.method for call in self.hpss_calls])
+
+    def test_backwards_compatible_no_tags(self):
+        branch = self.make_branch_with_tags()
+        c = branch.get_config()
+        c.set_user_option('branch.fetch_tags', 'False')
+        self.addCleanup(branch.lock_read().unlock)
+        # Disable the heads_to_fetch verb
+        verb = 'Branch.heads_to_fetch'
+        self.disable_verb(verb)
+        self.reset_smart_call_log()
+        result = branch.heads_to_fetch()
+        self.assertEqual((set(['tip']), set()), result)
+        self.assertEqual(
+            ['Branch.last_revision_info', 'Branch.get_config_file'],
+            [call.call.method for call in self.hpss_calls])
+
+
 class TestBranchLastRevisionInfo(RemoteBranchTestCase):
 
     def test_empty_branch(self):
@@ -1200,7 +1344,7 @@
         client.add_expected_call(
             'Branch.get_stacked_on_url', ('stacked/',),
             'success', ('ok', vfs_url))
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=client)
         repo_fmt = remote.RemoteRepositoryFormat()
         repo_fmt._custom_format = stacked_branch.repository._format
@@ -1233,7 +1377,7 @@
         # this will also do vfs access, but that goes direct to the transport
         # and isn't seen by the FakeClient.
         bzrdir = RemoteBzrDir(self.get_transport('stacked'),
-            remote.RemoteBzrDirFormat(), _client=client)
+            RemoteBzrDirFormat(), _client=client)
         branch = bzrdir.open_branch()
         result = branch.get_stacked_on_url()
         self.assertEqual('../base', result)
@@ -1266,7 +1410,7 @@
             'Branch.get_stacked_on_url', ('stacked/',),
             'success', ('ok', '../base'))
         bzrdir = RemoteBzrDir(self.get_transport('stacked'),
-            remote.RemoteBzrDirFormat(), _client=client)
+            RemoteBzrDirFormat(), _client=client)
         branch = bzrdir.open_branch()
         result = branch.get_stacked_on_url()
         self.assertEqual('../base', result)
@@ -1280,7 +1424,7 @@
 class TestBranchSetLastRevision(RemoteBranchTestCase):
 
     def test_set_empty(self):
-        # set_revision_history([]) is translated to calling
+        # _set_last_revision_info('null:') is translated to calling
         # Branch.set_last_revision(path, '') on the wire.
         transport = MemoryTransport()
         transport.mkdir('branch')
@@ -1308,13 +1452,13 @@
         # unnecessarily invokes _ensure_real upon a call to lock_write.
         branch._ensure_real = lambda: None
         branch.lock_write()
-        result = branch.set_revision_history([])
+        result = branch._set_last_revision(NULL_REVISION)
         branch.unlock()
         self.assertEqual(None, result)
         self.assertFinished(client)
 
     def test_set_nonempty(self):
-        # set_revision_history([rev-id1, ..., rev-idN]) is translated to calling
+        # set_last_revision_info(N, rev-idN) is translated to calling
         # Branch.set_last_revision(path, rev-idN) on the wire.
         transport = MemoryTransport()
         transport.mkdir('branch')
@@ -1346,7 +1490,7 @@
         branch._ensure_real = lambda: None
         # Lock the branch, reset the record of remote calls.
         branch.lock_write()
-        result = branch.set_revision_history(['rev-id1', 'rev-id2'])
+        result = branch._set_last_revision('rev-id2')
         branch.unlock()
         self.assertEqual(None, result)
         self.assertFinished(client)
@@ -1382,7 +1526,7 @@
         branch = self.make_remote_branch(transport, client)
         branch.lock_write()
         self.assertRaises(
-            errors.NoSuchRevision, branch.set_revision_history, ['rev-id'])
+            errors.NoSuchRevision, branch._set_last_revision, 'rev-id')
         branch.unlock()
         self.assertFinished(client)
 
@@ -1419,9 +1563,10 @@
         branch._ensure_real = lambda: None
         branch.lock_write()
         # The 'TipChangeRejected' error response triggered by calling
-        # set_revision_history causes a TipChangeRejected exception.
+        # set_last_revision_info causes a TipChangeRejected exception.
         err = self.assertRaises(
-            errors.TipChangeRejected, branch.set_revision_history, ['rev-id'])
+            errors.TipChangeRejected,
+            branch._set_last_revision, 'rev-id')
         # The UTF-8 message from the response has been decoded into a unicode
         # object.
         self.assertIsInstance(err.msg, unicode)
@@ -1842,7 +1987,7 @@
         client = FakeClient(transport.base)
         transport = transport.clone(transport_path)
         # we do not want bzrdir to make any remote calls
-        bzrdir = RemoteBzrDir(transport, remote.RemoteBzrDirFormat(),
+        bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
             _client=False)
         repo = RemoteRepository(bzrdir, None, _client=client)
         return repo, client
@@ -1856,7 +2001,7 @@
 
     def test_get_format_description(self):
         remote_format = RemoteBranchFormat()
-        real_format = branch.BranchFormat.get_default_format()
+        real_format = branch.format_registry.get_default()
         remote_format._network_name = real_format.network_name()
         self.assertEqual(remoted_description(real_format),
             remote_format.get_format_description())
@@ -1865,18 +2010,18 @@
 class TestRepositoryFormat(TestRemoteRepository):
 
     def test_fast_delta(self):
-        true_name = groupcompress_repo.RepositoryFormatCHK1().network_name()
+        true_name = groupcompress_repo.RepositoryFormat2a().network_name()
         true_format = RemoteRepositoryFormat()
         true_format._network_name = true_name
         self.assertEqual(True, true_format.fast_deltas)
-        false_name = pack_repo.RepositoryFormatKnitPack1().network_name()
+        false_name = knitpack_repo.RepositoryFormatKnitPack1().network_name()
         false_format = RemoteRepositoryFormat()
         false_format._network_name = false_name
         self.assertEqual(False, false_format.fast_deltas)
 
     def test_get_format_description(self):
         remote_repo_format = RemoteRepositoryFormat()
-        real_format = repository.RepositoryFormat.get_default_format()
+        real_format = repository.format_registry.get_default()
         remote_repo_format._network_name = real_format.network_name()
         self.assertEqual(remoted_description(real_format),
             remote_repo_format.get_format_description())
@@ -2005,8 +2150,8 @@
         parents = repo.get_parent_map([rev_id])
         self.assertEqual(
             [('call_with_body_bytes_expecting_body',
-              'Repository.get_parent_map', ('quack/', 'include-missing:',
-              rev_id), '\n\n0'),
+              'Repository.get_parent_map',
+              ('quack/', 'include-missing:', rev_id), '\n\n0'),
              ('disconnect medium',),
              ('call_expecting_body', 'Repository.get_revision_graph',
               ('quack/', ''))],
@@ -2132,6 +2277,32 @@
         self.assertEqual({}, repo.get_parent_map(['non-existant']))
         self.assertLength(0, self.hpss_calls)
 
+    def test_exposes_get_cached_parent_map(self):
+        """RemoteRepository exposes get_cached_parent_map from
+        _unstacked_provider
+        """
+        r1 = u'\u0e33'.encode('utf8')
+        r2 = u'\u0dab'.encode('utf8')
+        lines = [' '.join([r2, r1]), r1]
+        encoded_body = bz2.compress('\n'.join(lines))
+
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response_with_body(encoded_body, 'ok')
+        repo.lock_read()
+        # get_cached_parent_map should *not* trigger an RPC
+        self.assertEqual({}, repo.get_cached_parent_map([r1]))
+        self.assertEqual([], client._calls)
+        self.assertEqual({r2: (r1,)}, repo.get_parent_map([r2]))
+        self.assertEqual({r1: (NULL_REVISION,)},
+            repo.get_cached_parent_map([r1]))
+        self.assertEqual(
+            [('call_with_body_bytes_expecting_body',
+              'Repository.get_parent_map', ('quack/', 'include-missing:', r2),
+              '\n\n0')],
+            client._calls)
+        repo.unlock()
+
 
 class TestGetParentMapAllowsNew(tests.TestCaseWithTransport):
 
@@ -2440,7 +2611,7 @@
         the client is finished.
         """
         sink = repo._get_sink()
-        fmt = repository.RepositoryFormat.get_default_format()
+        fmt = repository.format_registry.get_default()
         resume_tokens, missing_keys = sink.insert_stream([], fmt, [])
         self.assertEqual([], resume_tokens)
         self.assertEqual(set(), missing_keys)
@@ -2546,7 +2717,7 @@
                 return True
         repo._real_repository = FakeRealRepository()
         sink = repo._get_sink()
-        fmt = repository.RepositoryFormat.get_default_format()
+        fmt = repository.format_registry.get_default()
         stream = self.make_stream_with_inv_deltas(fmt)
         resume_tokens, missing_keys = sink.insert_stream(stream, fmt, [])
         # Every record from the first inventory delta should have been sent to
@@ -2772,6 +2943,16 @@
              ('pack collection autopack',)],
             client._calls)
 
+    def test_oom_error_reporting(self):
+        """An out-of-memory condition on the server is reported clearly"""
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'PackRepository.autopack', ('quack/',),
+            'error', ('MemoryError',))
+        err = self.assertRaises(errors.BzrError, repo.autopack)
+        self.assertContainsRe(str(err), "^remote server out of mem")
+
 
 class TestErrorTranslationBase(tests.TestCaseWithMemoryTransport):
     """Base class for unit tests for bzrlib.remote._translate_error."""
@@ -2850,6 +3031,13 @@
             detail='extra detail')
         self.assertEqual(expected_error, translated_error)
 
+    def test_norepository(self):
+        bzrdir = self.make_bzrdir('')
+        translated_error = self.translateTuple(('norepository',),
+            bzrdir=bzrdir)
+        expected_error = errors.NoRepositoryPresent(bzrdir)
+        self.assertEqual(expected_error, translated_error)
+
     def test_LockContention(self):
         translated_error = self.translateTuple(('LockContention',))
         expected_error = errors.LockContention('(remote lock)')
@@ -2883,6 +3071,12 @@
         expected_error = errors.DivergedBranches(branch, other_branch)
         self.assertEqual(expected_error, translated_error)
 
+    def test_NotStacked(self):
+        branch = self.make_branch('')
+        translated_error = self.translateTuple(('NotStacked',), branch=branch)
+        expected_error = errors.NotStacked(branch)
+        self.assertEqual(expected_error, translated_error)
+
     def test_ReadError_no_args(self):
         path = 'a path'
         translated_error = self.translateTuple(('ReadError',), path=path)
@@ -2904,7 +3098,8 @@
 
     def test_PermissionDenied_no_args(self):
         path = 'a path'
-        translated_error = self.translateTuple(('PermissionDenied',), path=path)
+        translated_error = self.translateTuple(('PermissionDenied',),
+            path=path)
         expected_error = errors.PermissionDenied(path)
         self.assertEqual(expected_error, translated_error)
 
@@ -2933,6 +3128,45 @@
         expected_error = errors.PermissionDenied(path, extra)
         self.assertEqual(expected_error, translated_error)
 
+    # GZ 2011-03-02: TODO test for PermissionDenied with non-ascii 'extra'
+
+    def test_NoSuchFile_context_path(self):
+        local_path = "local path"
+        translated_error = self.translateTuple(('ReadError', "remote path"),
+            path=local_path)
+        expected_error = errors.ReadError(local_path)
+        self.assertEqual(expected_error, translated_error)
+
+    def test_NoSuchFile_without_context(self):
+        remote_path = "remote path"
+        translated_error = self.translateTuple(('ReadError', remote_path))
+        expected_error = errors.ReadError(remote_path)
+        self.assertEqual(expected_error, translated_error)
+
+    def test_ReadOnlyError(self):
+        translated_error = self.translateTuple(('ReadOnlyError',))
+        expected_error = errors.TransportNotPossible("readonly transport")
+        self.assertEqual(expected_error, translated_error)
+
+    def test_MemoryError(self):
+        translated_error = self.translateTuple(('MemoryError',))
+        self.assertStartsWith(str(translated_error),
+            "remote server out of memory")
+
+    def test_generic_IndexError_no_classname(self):
+        err = errors.ErrorFromSmartServer(('error', "list index out of range"))
+        translated_error = self.translateErrorFromSmartServer(err)
+        expected_error = errors.UnknownErrorFromSmartServer(err)
+        self.assertEqual(expected_error, translated_error)
+
+    # GZ 2011-03-02: TODO test generic non-ascii error string
+
+    def test_generic_KeyError(self):
+        err = errors.ErrorFromSmartServer(('error', 'KeyError', "1"))
+        translated_error = self.translateErrorFromSmartServer(err)
+        expected_error = errors.UnknownErrorFromSmartServer(err)
+        self.assertEqual(expected_error, translated_error)
+
 
 class TestErrorTranslationRobustness(TestErrorTranslationBase):
     """Unit tests for bzrlib.remote._translate_error's robustness.
@@ -3083,12 +3317,14 @@
         _, stacked = branch_factory()
         source = stacked.repository._get_source(target_repository_format)
         tip = stacked.last_revision()
-        revs = stacked.repository.get_ancestry(tip)
-        search = graph.PendingAncestryResult([tip], stacked.repository)
+        stacked.repository._ensure_real()
+        graph = stacked.repository.get_graph()
+        revs = [r for (r,ps) in graph.iter_ancestry([tip])
+                if r != NULL_REVISION]
+        revs.reverse()
+        search = _mod_graph.PendingAncestryResult([tip], stacked.repository)
         self.reset_smart_call_log()
         stream = source.get_stream(search)
-        if None in revs:
-            revs.remove(None)
         # We trust that if a revision is in the stream the rest of the new
         # content for it is too, as per our main fetch tests; here we are
         # checking that the revisions are actually included at all, and their
@@ -3133,7 +3369,7 @@
         self.assertEqual(expected_revs, rev_ord)
         # Getting topological sort requires VFS calls still - one of which is
         # pushing up from the bound branch.
-        self.assertLength(13, self.hpss_calls)
+        self.assertLength(14, self.hpss_calls)
 
     def test_stacked_get_stream_groupcompress(self):
         # Repository._get_source.get_stream() from a stacked repository with
@@ -3180,11 +3416,111 @@
 
     def test_copy_content_into_avoids_revision_history(self):
         local = self.make_branch('local')
-        remote_backing_tree = self.make_branch_and_tree('remote')
-        remote_backing_tree.commit("Commit.")
+        builder = self.make_branch_builder('remote')
+        builder.build_commit(message="Commit.")
         remote_branch_url = self.smart_server.get_url() + 'remote'
         remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
         local.repository.fetch(remote_branch.repository)
         self.hpss_calls = []
         remote_branch.copy_content_into(local)
         self.assertFalse('Branch.revision_history' in self.hpss_calls)
+
+    def test_fetch_everything_needs_just_one_call(self):
+        local = self.make_branch('local')
+        builder = self.make_branch_builder('remote')
+        builder.build_commit(message="Commit.")
+        remote_branch_url = self.smart_server.get_url() + 'remote'
+        remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
+        self.hpss_calls = []
+        local.repository.fetch(
+            remote_branch.repository,
+            fetch_spec=_mod_graph.EverythingResult(remote_branch.repository))
+        self.assertEqual(['Repository.get_stream_1.19'], self.hpss_calls)
+
+    def override_verb(self, verb_name, verb):
+        request_handlers = request.request_handlers
+        orig_verb = request_handlers.get(verb_name)
+        request_handlers.register(verb_name, verb, override_existing=True)
+        self.addCleanup(request_handlers.register, verb_name, orig_verb,
+                override_existing=True)
+
+    def test_fetch_everything_backwards_compat(self):
+        """Can fetch with EverythingResult even with pre 2.4 servers.
+        
+        Pre-2.4 do not support 'everything' searches with the
+        Repository.get_stream_1.19 verb.
+        """
+        verb_log = []
+        class OldGetStreamVerb(SmartServerRepositoryGetStream_1_19):
+            """A version of the Repository.get_stream_1.19 verb patched to
+            reject 'everything' searches the way 2.3 and earlier do.
+            """
+            def recreate_search(self, repository, search_bytes,
+                                discard_excess=False):
+                verb_log.append(search_bytes.split('\n', 1)[0])
+                if search_bytes == 'everything':
+                    return (None,
+                            request.FailedSmartServerResponse(('BadSearch',)))
+                return super(OldGetStreamVerb,
+                        self).recreate_search(repository, search_bytes,
+                            discard_excess=discard_excess)
+        self.override_verb('Repository.get_stream_1.19', OldGetStreamVerb)
+        local = self.make_branch('local')
+        builder = self.make_branch_builder('remote')
+        builder.build_commit(message="Commit.")
+        remote_branch_url = self.smart_server.get_url() + 'remote'
+        remote_branch = bzrdir.BzrDir.open(remote_branch_url).open_branch()
+        self.hpss_calls = []
+        local.repository.fetch(
+            remote_branch.repository,
+            fetch_spec=_mod_graph.EverythingResult(remote_branch.repository))
+        # make sure the overridden verb was used
+        self.assertLength(1, verb_log)
+        # more than one HPSS call is needed, but because it's a VFS callback
+        # its hard to predict exactly how many.
+        self.assertTrue(len(self.hpss_calls) > 1)
+
+
+class TestUpdateBoundBranchWithModifiedBoundLocation(
+    tests.TestCaseWithTransport):
+    """Ensure correct handling of bound_location modifications.
+
+    This is tested against a smart server as http://pad.lv/786980 was about a
+    ReadOnlyError (write attempt during a read-only transaction) which can only
+    happen in this context.
+    """
+
+    def setUp(self):
+        super(TestUpdateBoundBranchWithModifiedBoundLocation, self).setUp()
+        self.transport_server = test_server.SmartTCPServer_for_testing
+
+    def make_master_and_checkout(self, master_name, checkout_name):
+        # Create the master branch and its associated checkout
+        self.master = self.make_branch_and_tree(master_name)
+        self.checkout = self.master.branch.create_checkout(checkout_name)
+        # Modify the master branch so there is something to update
+        self.master.commit('add stuff')
+        self.last_revid = self.master.commit('even more stuff')
+        self.bound_location = self.checkout.branch.get_bound_location()
+
+    def assertUpdateSucceeds(self, new_location):
+        self.checkout.branch.set_bound_location(new_location)
+        self.checkout.update()
+        self.assertEquals(self.last_revid, self.checkout.last_revision())
+
+    def test_without_final_slash(self):
+        self.make_master_and_checkout('master', 'checkout')
+        # For unclear reasons some users have a bound_location without a final
+        # '/', simulate that by forcing such a value
+        self.assertEndsWith(self.bound_location, '/')
+        self.assertUpdateSucceeds(self.bound_location.rstrip('/'))
+
+    def test_plus_sign(self):
+        self.make_master_and_checkout('+master', 'checkout')
+        self.assertUpdateSucceeds(self.bound_location.replace('%2B', '+', 1))
+
+    def test_tilda(self):
+        # Embed ~ in the middle of the path just to avoid any $HOME
+        # interpretation
+        self.make_master_and_checkout('mas~ter', 'checkout')
+        self.assertUpdateSucceeds(self.bound_location.replace('%2E', '~', 1))

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2010-03-31 16:29:25 +0000
+++ bzrlib/tests/test_repository.py	2011-09-16 20:42:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,16 +23,18 @@
 """
 
 from stat import S_ISDIR
-import sys
 
 import bzrlib
-from bzrlib.errors import (NoSuchFile,
-                           UnknownFormatError,
-                           UnsupportedFormatError,
-                           )
+from bzrlib.errors import (
+    UnknownFormatError,
+    UnsupportedFormatError,
+    )
 from bzrlib import (
+    btree_index,
     graph,
+    symbol_versioning,
     tests,
+    transport,
     )
 from bzrlib.btree_index import BTreeBuilder, BTreeGraphIndex
 from bzrlib.index import GraphIndex
@@ -41,9 +43,6 @@
     TestCase,
     TestCaseWithTransport,
     )
-from bzrlib.transport import (
-    get_transport,
-    )
 from bzrlib import (
     bzrdir,
     errors,
@@ -53,13 +52,14 @@
     revision as _mod_revision,
     upgrade,
     versionedfile,
+    vf_repository,
     workingtree,
     )
 from bzrlib.repofmt import (
     groupcompress_repo,
     knitrepo,
+    knitpack_repo,
     pack_repo,
-    weaverepo,
     )
 
 
@@ -68,7 +68,7 @@
     def test_get_set_default_format(self):
         old_default = bzrdir.format_registry.get('default')
         private_default = old_default().repository_format.__class__
-        old_format = repository.RepositoryFormat.get_default_format()
+        old_format = repository.format_registry.get_default()
         self.assertTrue(isinstance(old_format, private_default))
         def make_sample_bzrdir():
             my_bzrdir = bzrdir.BzrDirMetaFormat1()
@@ -88,7 +88,7 @@
             bzrdir.format_registry.remove('default')
             bzrdir.format_registry.remove('sample')
             bzrdir.format_registry.register('default', old_default, '')
-        self.assertIsInstance(repository.RepositoryFormat.get_default_format(),
+        self.assertIsInstance(repository.format_registry.get_default(),
                               old_format.__class__)
 
 
@@ -116,6 +116,15 @@
         return "opened repository."
 
 
+class SampleExtraRepositoryFormat(repository.RepositoryFormat):
+    """A sample format that can not be used in a metadir
+
+    """
+
+    def get_format_string(self):
+        raise NotImplementedError
+
+
 class TestRepositoryFormat(TestCaseWithTransport):
     """Tests for the Repository format detection used by the bzr meta dir facility.BzrBranchFormat facility."""
 
@@ -127,10 +136,10 @@
         def check_format(format, url):
             dir = format._matchingbzrdir.initialize(url)
             format.initialize(dir)
-            t = get_transport(url)
+            t = transport.get_transport_from_path(url)
             found_format = repository.RepositoryFormat.find_format(dir)
-            self.failUnless(isinstance(found_format, format.__class__))
-        check_format(weaverepo.RepositoryFormat7(), "bar")
+            self.assertIsInstance(found_format, format.__class__)
+        check_format(repository.format_registry.get_default(), "bar")
 
     def test_find_format_no_repository(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
@@ -146,207 +155,57 @@
                           dir)
 
     def test_register_unregister_format(self):
+        # Test deprecated format registration functions
         format = SampleRepositoryFormat()
         # make a control dir
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         # make a repo
         format.initialize(dir)
         # register a format for it.
-        repository.RepositoryFormat.register_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            repository.RepositoryFormat.register_format, format)
         # which repository.Open will refuse (not supported)
-        self.assertRaises(UnsupportedFormatError, repository.Repository.open, self.get_url())
+        self.assertRaises(UnsupportedFormatError, repository.Repository.open,
+            self.get_url())
         # but open(unsupported) will work
         self.assertEqual(format.open(dir), "opened repository.")
         # unregister the format
-        repository.RepositoryFormat.unregister_format(format)
-
-
-class TestFormat6(TestCaseWithTransport):
-
-    def test_attribute__fetch_order(self):
-        """Weaves need topological data insertion."""
-        control = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat6().initialize(control)
-        self.assertEqual('topological', repo._format._fetch_order)
-
-    def test_attribute__fetch_uses_deltas(self):
-        """Weaves do not reuse deltas."""
-        control = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat6().initialize(control)
-        self.assertEqual(False, repo._format._fetch_uses_deltas)
-
-    def test_attribute__fetch_reconcile(self):
-        """Weave repositories need a reconcile after fetch."""
-        control = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat6().initialize(control)
-        self.assertEqual(True, repo._format._fetch_reconcile)
-
-    def test_no_ancestry_weave(self):
-        control = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat6().initialize(control)
-        # We no longer need to create the ancestry.weave file
-        # since it is *never* used.
-        self.assertRaises(NoSuchFile,
-                          control.transport.get,
-                          'ancestry.weave')
-
-    def test_supports_external_lookups(self):
-        control = bzrdir.BzrDirFormat6().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat6().initialize(control)
-        self.assertFalse(repo._format.supports_external_lookups)
-
-
-class TestFormat7(TestCaseWithTransport):
-
-    def test_attribute__fetch_order(self):
-        """Weaves need topological data insertion."""
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control)
-        self.assertEqual('topological', repo._format._fetch_order)
-
-    def test_attribute__fetch_uses_deltas(self):
-        """Weaves do not reuse deltas."""
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control)
-        self.assertEqual(False, repo._format._fetch_uses_deltas)
-
-    def test_attribute__fetch_reconcile(self):
-        """Weave repositories need a reconcile after fetch."""
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control)
-        self.assertEqual(True, repo._format._fetch_reconcile)
-
-    def test_disk_layout(self):
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control)
-        # in case of side effects of locking.
-        repo.lock_write()
-        repo.unlock()
-        # we want:
-        # format 'Bazaar-NG Repository format 7'
-        # lock ''
-        # inventory.weave == empty_weave
-        # empty revision-store directory
-        # empty weaves directory
-        t = control.get_repository_transport(None)
-        self.assertEqualDiff('Bazaar-NG Repository format 7',
-                             t.get('format').read())
-        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
-        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
-        self.assertEqualDiff('# bzr weave file v5\n'
-                             'w\n'
-                             'W\n',
-                             t.get('inventory.weave').read())
-        # Creating a file with id Foo:Bar results in a non-escaped file name on
-        # disk.
-        control.create_branch()
-        tree = control.create_workingtree()
-        tree.add(['foo'], ['Foo:Bar'], ['file'])
-        tree.put_file_bytes_non_atomic('Foo:Bar', 'content\n')
-        try:
-            tree.commit('first post', rev_id='first')
-        except errors.IllegalPath:
-            if sys.platform != 'win32':
-                raise
-            self.knownFailure('Foo:Bar cannot be used as a file-id on windows'
-                              ' in repo format 7')
-            return
-        self.assertEqualDiff(
-            '# bzr weave file v5\n'
-            'i\n'
-            '1 7fe70820e08a1aac0ef224d9c66ab66831cc4ab1\n'
-            'n first\n'
-            '\n'
-            'w\n'
-            '{ 0\n'
-            '. content\n'
-            '}\n'
-            'W\n',
-            t.get('weaves/74/Foo%3ABar.weave').read())
-
-    def test_shared_disk_layout(self):
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
-        # we want:
-        # format 'Bazaar-NG Repository format 7'
-        # inventory.weave == empty_weave
-        # empty revision-store directory
-        # empty weaves directory
-        # a 'shared-storage' marker file.
-        # lock is not present when unlocked
-        t = control.get_repository_transport(None)
-        self.assertEqualDiff('Bazaar-NG Repository format 7',
-                             t.get('format').read())
-        self.assertEqualDiff('', t.get('shared-storage').read())
-        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
-        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
-        self.assertEqualDiff('# bzr weave file v5\n'
-                             'w\n'
-                             'W\n',
-                             t.get('inventory.weave').read())
-        self.assertFalse(t.has('branch-lock'))
-
-    def test_creates_lockdir(self):
-        """Make sure it appears to be controlled by a LockDir existence"""
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
-        t = control.get_repository_transport(None)
-        # TODO: Should check there is a 'lock' toplevel directory,
-        # regardless of contents
-        self.assertFalse(t.has('lock/held/info'))
-        repo.lock_write()
-        try:
-            self.assertTrue(t.has('lock/held/info'))
-        finally:
-            # unlock so we don't get a warning about failing to do so
-            repo.unlock()
-
-    def test_uses_lockdir(self):
-        """repo format 7 actually locks on lockdir"""
-        base_url = self.get_url()
-        control = bzrdir.BzrDirMetaFormat1().initialize(base_url)
-        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
-        t = control.get_repository_transport(None)
-        repo.lock_write()
-        repo.unlock()
-        del repo
-        # make sure the same lock is created by opening it
-        repo = repository.Repository.open(base_url)
-        repo.lock_write()
-        self.assertTrue(t.has('lock/held/info'))
-        repo.unlock()
-        self.assertFalse(t.has('lock/held/info'))
-
-    def test_shared_no_tree_disk_layout(self):
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control, shared=True)
-        repo.set_make_working_trees(False)
-        # we want:
-        # format 'Bazaar-NG Repository format 7'
-        # lock ''
-        # inventory.weave == empty_weave
-        # empty revision-store directory
-        # empty weaves directory
-        # a 'shared-storage' marker file.
-        t = control.get_repository_transport(None)
-        self.assertEqualDiff('Bazaar-NG Repository format 7',
-                             t.get('format').read())
-        ## self.assertEqualDiff('', t.get('lock').read())
-        self.assertEqualDiff('', t.get('shared-storage').read())
-        self.assertEqualDiff('', t.get('no-working-trees').read())
-        repo.set_make_working_trees(True)
-        self.assertFalse(t.has('no-working-trees'))
-        self.assertTrue(S_ISDIR(t.stat('revision-store').st_mode))
-        self.assertTrue(S_ISDIR(t.stat('weaves').st_mode))
-        self.assertEqualDiff('# bzr weave file v5\n'
-                             'w\n'
-                             'W\n',
-                             t.get('inventory.weave').read())
-
-    def test_supports_external_lookups(self):
-        control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        repo = weaverepo.RepositoryFormat7().initialize(control)
-        self.assertFalse(repo._format.supports_external_lookups)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            repository.RepositoryFormat.unregister_format, format)
+
+
+class TestRepositoryFormatRegistry(TestCase):
+
+    def setUp(self):
+        super(TestRepositoryFormatRegistry, self).setUp()
+        self.registry = repository.RepositoryFormatRegistry()
+
+    def test_register_unregister_format(self):
+        format = SampleRepositoryFormat()
+        self.registry.register(format)
+        self.assertEquals(format, self.registry.get("Sample .bzr repository format."))
+        self.registry.remove(format)
+        self.assertRaises(KeyError, self.registry.get, "Sample .bzr repository format.")
+
+    def test_get_all(self):
+        format = SampleRepositoryFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra(self):
+        format = SampleExtraRepositoryFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra_lazy(self):
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra_lazy("bzrlib.tests.test_repository",
+            "SampleExtraRepositoryFormat")
+        formats = self.registry._get_all()
+        self.assertEquals(1, len(formats))
+        self.assertIsInstance(formats[0], SampleExtraRepositoryFormat)
 
 
 class TestFormatKnit1(TestCaseWithTransport):
@@ -520,7 +379,11 @@
         # classes do not barf inappropriately when a surprising repository type
         # is handed to them.
         dummy_a = DummyRepository()
+        dummy_a._format = RepositoryFormat()
+        dummy_a._format.supports_full_versioned_files = True
         dummy_b = DummyRepository()
+        dummy_b._format = RepositoryFormat()
+        dummy_b._format.supports_full_versioned_files = True
         self.assertGetsDefaultInterRepository(dummy_a, dummy_b)
 
     def assertGetsDefaultInterRepository(self, repo_a, repo_b):
@@ -530,7 +393,7 @@
         no actual sane default in the presence of incompatible data models.
         """
         inter_repo = repository.InterRepository.get(repo_a, repo_b)
-        self.assertEqual(repository.InterSameDataRepository,
+        self.assertEqual(vf_repository.InterSameDataRepository,
                          inter_repo.__class__)
         self.assertEqual(repo_a, inter_repo.source)
         self.assertEqual(repo_b, inter_repo.target)
@@ -550,9 +413,11 @@
         dummy_a._serializer = repo._serializer
         dummy_a._format.supports_tree_reference = repo._format.supports_tree_reference
         dummy_a._format.rich_root_data = repo._format.rich_root_data
+        dummy_a._format.supports_full_versioned_files = repo._format.supports_full_versioned_files
         dummy_b._serializer = repo._serializer
         dummy_b._format.supports_tree_reference = repo._format.supports_tree_reference
         dummy_b._format.rich_root_data = repo._format.rich_root_data
+        dummy_b._format.supports_full_versioned_files = repo._format.supports_full_versioned_files
         repository.InterRepository.register_optimiser(InterDummy)
         try:
             # we should get the default for something InterDummy returns False
@@ -571,45 +436,33 @@
         self.assertGetsDefaultInterRepository(dummy_a, dummy_b)
 
 
-class TestInterWeaveRepo(TestCaseWithTransport):
-
-    def test_is_compatible_and_registered(self):
-        # InterWeaveRepo is compatible when either side
-        # is a format 5/6/7 branch
-        from bzrlib.repofmt import knitrepo, weaverepo
-        formats = [weaverepo.RepositoryFormat5(),
-                   weaverepo.RepositoryFormat6(),
-                   weaverepo.RepositoryFormat7()]
-        incompatible_formats = [weaverepo.RepositoryFormat4(),
-                                knitrepo.RepositoryFormatKnit1(),
-                                ]
-        repo_a = self.make_repository('a')
-        repo_b = self.make_repository('b')
-        is_compatible = repository.InterWeaveRepo.is_compatible
-        for source in incompatible_formats:
-            # force incompatible left then right
-            repo_a._format = source
-            repo_b._format = formats[0]
-            self.assertFalse(is_compatible(repo_a, repo_b))
-            self.assertFalse(is_compatible(repo_b, repo_a))
-        for source in formats:
-            repo_a._format = source
-            for target in formats:
-                repo_b._format = target
-                self.assertTrue(is_compatible(repo_a, repo_b))
-        self.assertEqual(repository.InterWeaveRepo,
-                         repository.InterRepository.get(repo_a,
-                                                        repo_b).__class__)
+class TestRepositoryFormat1(knitrepo.RepositoryFormatKnit1):
+
+    def get_format_string(self):
+        return "Test Format 1"
+
+
+class TestRepositoryFormat2(knitrepo.RepositoryFormatKnit1):
+
+    def get_format_string(self):
+        return "Test Format 2"
 
 
 class TestRepositoryConverter(TestCaseWithTransport):
 
     def test_convert_empty(self):
-        t = get_transport(self.get_url('.'))
+        source_format = TestRepositoryFormat1()
+        target_format = TestRepositoryFormat2()
+        repository.format_registry.register(source_format)
+        self.addCleanup(repository.format_registry.remove,
+            source_format)
+        repository.format_registry.register(target_format)
+        self.addCleanup(repository.format_registry.remove,
+            target_format)
+        t = self.get_transport()
         t.mkdir('repository')
         repo_dir = bzrdir.BzrDirMetaFormat1().initialize('repository')
-        repo = weaverepo.RepositoryFormat7().initialize(repo_dir)
-        target_format = knitrepo.RepositoryFormatKnit1()
+        repo = TestRepositoryFormat1().initialize(repo_dir)
         converter = repository.CopyConverter(target_format)
         pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
@@ -620,13 +473,6 @@
         self.assertTrue(isinstance(target_format, repo._format.__class__))
 
 
-class TestMisc(TestCase):
-
-    def test_unescape_xml(self):
-        """We get some kind of error when malformed entities are passed"""
-        self.assertRaises(KeyError, repository._unescape_xml, 'foo&bar;')
-
-
 class TestRepositoryFormatKnit3(TestCaseWithTransport):
 
     def test_attribute__fetch_order(self):
@@ -681,6 +527,21 @@
 
 class Test2a(tests.TestCaseWithMemoryTransport):
 
+    def test_chk_bytes_uses_custom_btree_parser(self):
+        mt = self.make_branch_and_memory_tree('test', format='2a')
+        mt.lock_write()
+        self.addCleanup(mt.unlock)
+        mt.add([''], ['root-id'])
+        mt.commit('first')
+        index = mt.branch.repository.chk_bytes._index._graph_index._indices[0]
+        self.assertEqual(btree_index._gcchk_factory, index._leaf_factory)
+        # It should also work if we re-open the repo
+        repo = mt.branch.repository.bzrdir.open_repository()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        index = repo.chk_bytes._index._graph_index._indices[0]
+        self.assertEqual(btree_index._gcchk_factory, index._leaf_factory)
+
     def test_fetch_combines_groups(self):
         builder = self.make_branch_builder('source', format='2a')
         builder.start_series()
@@ -812,7 +673,7 @@
         target = self.make_repository('target', format='rich-root-pack')
         stream = source._get_source(target._format)
         # We don't want the child GroupCHKStreamSource
-        self.assertIs(type(stream), repository.StreamSource)
+        self.assertIs(type(stream), vf_repository.StreamSource)
 
     def test_get_stream_for_missing_keys_includes_all_chk_refs(self):
         source_builder = self.make_branch_builder('source',
@@ -894,25 +755,25 @@
         source = self.make_repository('source', format='pack-0.92')
         target = self.make_repository('target', format='pack-0.92')
         stream_source = source._get_source(target._format)
-        self.assertIsInstance(stream_source, pack_repo.KnitPackStreamSource)
+        self.assertIsInstance(stream_source, knitpack_repo.KnitPackStreamSource)
 
     def test_source_to_exact_pack_rich_root_pack(self):
         source = self.make_repository('source', format='rich-root-pack')
         target = self.make_repository('target', format='rich-root-pack')
         stream_source = source._get_source(target._format)
-        self.assertIsInstance(stream_source, pack_repo.KnitPackStreamSource)
+        self.assertIsInstance(stream_source, knitpack_repo.KnitPackStreamSource)
 
     def test_source_to_exact_pack_19(self):
         source = self.make_repository('source', format='1.9')
         target = self.make_repository('target', format='1.9')
         stream_source = source._get_source(target._format)
-        self.assertIsInstance(stream_source, pack_repo.KnitPackStreamSource)
+        self.assertIsInstance(stream_source, knitpack_repo.KnitPackStreamSource)
 
     def test_source_to_exact_pack_19_rich_root(self):
         source = self.make_repository('source', format='1.9-rich-root')
         target = self.make_repository('target', format='1.9-rich-root')
         stream_source = source._get_source(target._format)
-        self.assertIsInstance(stream_source, pack_repo.KnitPackStreamSource)
+        self.assertIsInstance(stream_source, knitpack_repo.KnitPackStreamSource)
 
     def test_source_to_remote_exact_pack_19(self):
         trans = self.make_smart_server('target')
@@ -921,19 +782,19 @@
         target = self.make_repository('target', format='1.9')
         target = repository.Repository.open(trans.base)
         stream_source = source._get_source(target._format)
-        self.assertIsInstance(stream_source, pack_repo.KnitPackStreamSource)
+        self.assertIsInstance(stream_source, knitpack_repo.KnitPackStreamSource)
 
     def test_stream_source_to_non_exact(self):
         source = self.make_repository('source', format='pack-0.92')
         target = self.make_repository('target', format='1.9')
         stream = source._get_source(target._format)
-        self.assertIs(type(stream), repository.StreamSource)
+        self.assertIs(type(stream), vf_repository.StreamSource)
 
     def test_stream_source_to_non_exact_rich_root(self):
         source = self.make_repository('source', format='1.9')
         target = self.make_repository('target', format='1.9-rich-root')
         stream = source._get_source(target._format)
-        self.assertIs(type(stream), repository.StreamSource)
+        self.assertIs(type(stream), vf_repository.StreamSource)
 
     def test_source_to_remote_non_exact_pack_19(self):
         trans = self.make_smart_server('target')
@@ -942,13 +803,13 @@
         target = self.make_repository('target', format='1.6')
         target = repository.Repository.open(trans.base)
         stream_source = source._get_source(target._format)
-        self.assertIs(type(stream_source), repository.StreamSource)
+        self.assertIs(type(stream_source), vf_repository.StreamSource)
 
     def test_stream_source_to_knit(self):
         source = self.make_repository('source', format='pack-0.92')
         target = self.make_repository('target', format='dirstate')
         stream = source._get_source(target._format)
-        self.assertIs(type(stream), repository.StreamSource)
+        self.assertIs(type(stream), vf_repository.StreamSource)
 
 
 class TestDevelopment6FindParentIdsOfRevisions(TestCaseWithTransport):
@@ -956,8 +817,7 @@
 
     def setUp(self):
         super(TestDevelopment6FindParentIdsOfRevisions, self).setUp()
-        self.builder = self.make_branch_builder('source',
-            format='development6-rich-root')
+        self.builder = self.make_branch_builder('source')
         self.builder.start_series()
         self.builder.build_snapshot('initial', None,
             [('add', ('', 'tree-root', 'directory', None))])
@@ -1591,7 +1451,7 @@
         # Because of how they were built, they correspond to
         # ['D', 'C', 'B', 'A']
         packs = b.repository._pack_collection.packs
-        packer = pack_repo.Packer(b.repository._pack_collection,
+        packer = knitpack_repo.KnitPacker(b.repository._pack_collection,
                                   packs, 'testing',
                                   revision_ids=['B', 'C'])
         # Now, when we are copying the B & C revisions, their pack files should
@@ -1611,7 +1471,7 @@
         return repo._pack_collection
 
     def test_open_pack_will_optimise(self):
-        packer = pack_repo.OptimisingPacker(self.get_pack_collection(),
+        packer = knitpack_repo.OptimisingKnitPacker(self.get_pack_collection(),
                                             [], '.test')
         new_pack = packer.open_pack()
         self.addCleanup(new_pack.abort) # ensure cleanup
@@ -1622,6 +1482,106 @@
         self.assertTrue(new_pack.signature_index._optimize_for_size)
 
 
+class TestGCCHKPacker(TestCaseWithTransport):
+
+    def make_abc_branch(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        builder.build_snapshot('A', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('file', 'file-id', 'file', 'content\n')),
+            ])
+        builder.build_snapshot('B', ['A'], [
+            ('add', ('dir', 'dir-id', 'directory', None))])
+        builder.build_snapshot('C', ['B'], [
+            ('modify', ('file-id', 'new content\n'))])
+        builder.finish_series()
+        return builder.get_branch()
+
+    def make_branch_with_disjoint_inventory_and_revision(self):
+        """a repo with separate packs for a revisions Revision and Inventory.
+
+        There will be one pack file that holds the Revision content, and one
+        for the Inventory content.
+
+        :return: (repository,
+                  pack_name_with_rev_A_Revision,
+                  pack_name_with_rev_A_Inventory,
+                  pack_name_with_rev_C_content)
+        """
+        b_source = self.make_abc_branch()
+        b_base = b_source.bzrdir.sprout('base', revision_id='A').open_branch()
+        b_stacked = b_base.bzrdir.sprout('stacked', stacked=True).open_branch()
+        b_stacked.lock_write()
+        self.addCleanup(b_stacked.unlock)
+        b_stacked.fetch(b_source, 'B')
+        # Now re-open the stacked repo directly (no fallbacks) so that we can
+        # fill in the A rev.
+        repo_not_stacked = b_stacked.bzrdir.open_repository()
+        repo_not_stacked.lock_write()
+        self.addCleanup(repo_not_stacked.unlock)
+        # Now we should have a pack file with A's inventory, but not its
+        # Revision
+        self.assertEqual([('A',), ('B',)],
+                         sorted(repo_not_stacked.inventories.keys()))
+        self.assertEqual([('B',)],
+                         sorted(repo_not_stacked.revisions.keys()))
+        stacked_pack_names = repo_not_stacked._pack_collection.names()
+        # We have a couple names here, figure out which has A's inventory
+        for name in stacked_pack_names:
+            pack = repo_not_stacked._pack_collection.get_pack_by_name(name)
+            keys = [n[1] for n in pack.inventory_index.iter_all_entries()]
+            if ('A',) in keys:
+                inv_a_pack_name = name
+                break
+        else:
+            self.fail('Could not find pack containing A\'s inventory')
+        repo_not_stacked.fetch(b_source.repository, 'A')
+        self.assertEqual([('A',), ('B',)],
+                         sorted(repo_not_stacked.revisions.keys()))
+        new_pack_names = set(repo_not_stacked._pack_collection.names())
+        rev_a_pack_names = new_pack_names.difference(stacked_pack_names)
+        self.assertEqual(1, len(rev_a_pack_names))
+        rev_a_pack_name = list(rev_a_pack_names)[0]
+        # Now fetch 'C', so we have a couple pack files to join
+        repo_not_stacked.fetch(b_source.repository, 'C')
+        rev_c_pack_names = set(repo_not_stacked._pack_collection.names())
+        rev_c_pack_names = rev_c_pack_names.difference(new_pack_names)
+        self.assertEqual(1, len(rev_c_pack_names))
+        rev_c_pack_name = list(rev_c_pack_names)[0]
+        return (repo_not_stacked, rev_a_pack_name, inv_a_pack_name,
+                rev_c_pack_name)
+
+    def test_pack_with_distant_inventories(self):
+        # See https://bugs.launchpad.net/bzr/+bug/437003
+        # When repacking, it is possible to have an inventory in a different
+        # pack file than the associated revision. An autopack can then come
+        # along, and miss that inventory, and complain.
+        (repo, rev_a_pack_name, inv_a_pack_name, rev_c_pack_name
+         ) = self.make_branch_with_disjoint_inventory_and_revision()
+        a_pack = repo._pack_collection.get_pack_by_name(rev_a_pack_name)
+        c_pack = repo._pack_collection.get_pack_by_name(rev_c_pack_name)
+        packer = groupcompress_repo.GCCHKPacker(repo._pack_collection,
+                    [a_pack, c_pack], '.test-pack')
+        # This would raise ValueError in bug #437003, but should not raise an
+        # error once fixed.
+        packer.pack()
+
+    def test_pack_with_missing_inventory(self):
+        # Similar to test_pack_with_missing_inventory, but this time, we force
+        # the A inventory to actually be gone from the repository.
+        (repo, rev_a_pack_name, inv_a_pack_name, rev_c_pack_name
+         ) = self.make_branch_with_disjoint_inventory_and_revision()
+        inv_a_pack = repo._pack_collection.get_pack_by_name(inv_a_pack_name)
+        repo._pack_collection._remove_pack_from_memory(inv_a_pack)
+        packer = groupcompress_repo.GCCHKPacker(repo._pack_collection,
+            repo._pack_collection.all_packs(), '.test-pack')
+        e = self.assertRaises(ValueError, packer.pack)
+        packer.new_pack.abort()
+        self.assertContainsRe(str(e),
+            r"We are missing inventories for revisions: .*'A'")
+
+
 class TestCrossFormatPacks(TestCaseWithTransport):
 
     def log_pack(self, hint=None):
@@ -1642,9 +1602,9 @@
         self.addCleanup(target.unlock)
         source = source_tree.branch.repository._get_source(target._format)
         self.orig_pack = target.pack
-        target.pack = self.log_pack
+        self.overrideAttr(target, "pack", self.log_pack)
         search = target.search_missing_revision_ids(
-            source_tree.branch.repository, tip)
+            source_tree.branch.repository, revision_ids=[tip])
         stream = source.get_stream(search)
         from_format = source_tree.branch.repository._format
         sink = target._get_sink()
@@ -1666,7 +1626,7 @@
         self.addCleanup(target.unlock)
         source = source_tree.branch.repository
         self.orig_pack = target.pack
-        target.pack = self.log_pack
+        self.overrideAttr(target, "pack", self.log_pack)
         target.fetch(source)
         if expect_pack_called:
             self.assertLength(1, self.calls)
@@ -1700,3 +1660,11 @@
     def test_IDS_format_same_no(self):
         # When the formats are the same, pack is not called.
         self.run_fetch('2a', '2a', False)
+
+
+class Test_LazyListJoin(tests.TestCase):
+
+    def test__repr__(self):
+        lazy = repository._LazyListJoin(['a'], ['b'])
+        self.assertEqual("bzrlib.repository._LazyListJoin((['a'], ['b']))",
+                         repr(lazy))

=== modified file 'bzrlib/tests/test_revert.py'
--- bzrlib/tests/test_revert.py	2010-01-11 22:27:17 +0000
+++ bzrlib/tests/test_revert.py	2011-06-15 11:36:05 +0000
@@ -34,11 +34,11 @@
         self.addCleanup(target_tree.unlock)
         merge.merge_inner(target_tree.branch, source_tree.basis_tree(),
                           target_tree.basis_tree(), this_tree=target_tree)
-        self.failUnlessExists('target/dir')
-        self.failUnlessExists('target/dir/contents')
+        self.assertPathExists('target/dir')
+        self.assertPathExists('target/dir/contents')
         target_tree.revert()
-        self.failIfExists('target/dir/contents')
-        self.failIfExists('target/dir')
+        self.assertPathDoesNotExist('target/dir/contents')
+        self.assertPathDoesNotExist('target/dir')
 
     def test_revert_new(self):
         """Only locally-changed new files should be preserved when reverting
@@ -60,26 +60,26 @@
         tree.add('new_file')
         basis_tree = tree.branch.repository.revision_tree(tree.last_revision())
         tree.revert()
-        self.failUnlessExists('tree/new_file')
+        self.assertPathExists('tree/new_file')
 
         # unchanged files should be deleted
         tree.add('new_file')
         tree.commit('add new_file')
         tree.revert(old_tree=basis_tree)
-        self.failIfExists('tree/new_file')
+        self.assertPathDoesNotExist('tree/new_file')
 
         # files should be deleted if their changes came from merges
         merge_target.merge_from_branch(tree.branch)
-        self.failUnlessExists('merge_target/new_file')
+        self.assertPathExists('merge_target/new_file')
         merge_target.revert()
-        self.failIfExists('merge_target/new_file')
+        self.assertPathDoesNotExist('merge_target/new_file')
 
         # files should not be deleted if changed after a merge
         merge_target.merge_from_branch(tree.branch)
-        self.failUnlessExists('merge_target/new_file')
+        self.assertPathExists('merge_target/new_file')
         self.build_tree_contents([('merge_target/new_file', 'new_contents')])
         merge_target.revert()
-        self.failUnlessExists('merge_target/new_file')
+        self.assertPathExists('merge_target/new_file')
 
     def tree_with_executable(self):
         tree = self.make_branch_and_tree('tree')
@@ -127,22 +127,13 @@
         tree.commit('added file', rev_id='rev1')
         os.unlink('file')
         tree.commit('removed file')
-        self.failIfExists('file')
+        self.assertPathDoesNotExist('file')
         tree.revert(old_tree=tree.branch.repository.revision_tree('rev1'))
-        self.failUnlessExists('file')
+        self.assertPathExists('file')
         tree.revert()
-        self.failIfExists('file')
+        self.assertPathDoesNotExist('file')
         self.assertEqual({}, tree.merge_modified())
 
-    def test_empty_deprecated(self):
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['file'])
-        tree.add('file')
-        self.callDeprecated(['Using [] to revert all files is deprecated'
-            ' as of bzr 0.91.  Please use None (the default) instead.'],
-            tree.revert, [])
-        self.assertIs(None, tree.path2id('file'))
-
     def test_revert_file_in_deleted_dir(self):
         tree = self.make_branch_and_tree('.')
         self.build_tree(['dir/', 'dir/file1', 'dir/file2'])
@@ -154,8 +145,8 @@
         os.rmdir('dir')
         tree.remove(['dir/', 'dir/file1', 'dir/file2'])
         tree.revert(['dir/file1'])
-        self.failUnlessExists('dir/file1')
-        self.failIfExists('dir/file2')
+        self.assertPathExists('dir/file1')
+        self.assertPathDoesNotExist('dir/file2')
         self.assertEqual('dir-id', tree.path2id('dir'))
 
     def test_revert_root_id_change(self):

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2010-07-15 12:37:32 +0000
+++ bzrlib/tests/test_revision.py	2011-10-06 00:14:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,26 +15,19 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import os
 import warnings
 
 from bzrlib import (
     bugtracker,
     revision,
-    symbol_versioning,
     )
-from bzrlib.branch import Branch
 from bzrlib.errors import (
     InvalidBugStatus,
     InvalidLineInBugsProperty,
-    NoSuchRevision,
     )
-from bzrlib.deprecated_graph import Graph
-from bzrlib.revision import (find_present_ancestors,
-                             NULL_REVISION)
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport
-from bzrlib.trace import mutter
-from bzrlib.workingtree import WorkingTree
+from bzrlib.tests.matchers import MatchesAncestry
 
 # We're allowed to test deprecated interfaces
 warnings.filterwarnings('ignore',
@@ -73,19 +66,26 @@
     br2 = tree2.branch
     tree2.commit("Commit four", rev_id="b@u-0-3")
     tree2.commit("Commit five", rev_id="b@u-0-4")
-    revisions_2 = br2.revision_history()
-    self.assertEquals(revisions_2[-1], 'b@u-0-4')
+    self.assertEquals(br2.last_revision(), 'b@u-0-4')
 
     tree1.merge_from_branch(br2)
     tree1.commit("Commit six", rev_id="a@u-0-3")
     tree1.commit("Commit seven", rev_id="a@u-0-4")
     tree2.commit("Commit eight", rev_id="b@u-0-5")
-    self.assertEquals(br2.revision_history()[-1], 'b@u-0-5')
+    self.assertEquals(br2.last_revision(), 'b@u-0-5')
 
     tree1.merge_from_branch(br2)
     tree1.commit("Commit nine", rev_id="a@u-0-5")
     # DO NOT MERGE HERE - we WANT a GHOST.
-    tree2.add_parent_tree_id(br1.revision_history()[4])
+    br1.lock_read()
+    try:
+        graph = br1.repository.get_graph()
+        revhistory = list(graph.iter_lefthand_ancestry(br1.last_revision(),
+            [revision.NULL_REVISION]))
+        revhistory.reverse()
+    finally:
+        br1.unlock()
+    tree2.add_parent_tree_id(revhistory[4])
     tree2.commit("Commit ten - ghost merge", rev_id="b@u-0-6")
 
     return br1, br2
@@ -111,7 +111,7 @@
              ('a@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4',
                           'b@u-0-3', 'b@u-0-4',
                           'b@u-0-5', 'a@u-0-5']),
-             ('b@u-0-6', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2',
+             ('b@u-0-6', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'a@u-0-4',
                           'b@u-0-3', 'b@u-0-4',
                           'b@u-0-5', 'b@u-0-6']),
              ]
@@ -123,10 +123,8 @@
                     continue
                 if rev_id in br2_only and not branch is br2:
                     continue
-                mutter('ancestry of {%s}: %r',
-                       rev_id, branch.repository.get_ancestry(rev_id))
-                result = sorted(branch.repository.get_ancestry(rev_id))
-                self.assertEquals(result, [None] + sorted(anc))
+                self.assertThat(anc,
+                    MatchesAncestry(branch.repository, rev_id))
 
 
 class TestIntermediateRevisions(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_revisionspec.py'
--- bzrlib/tests/test_revisionspec.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_revisionspec.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,20 +15,17 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import datetime
-import os
 import time
 
 from bzrlib import (
-    branch,
-    bzrdir,
     errors,
-    repository,
     revision as _mod_revision,
     )
-from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.revisionspec import (
+    RevisionInfo,
     RevisionSpec,
-    RevisionSpec_revno,
+    RevisionSpec_dwim,
     RevisionSpec_tag,
     )
 
@@ -147,6 +144,17 @@
         self.assertRaises(TypeError, RevisionSpec.from_string, object())
 
 
+class RevisionSpec_bork(RevisionSpec):
+
+    prefix = 'irrelevant:'
+
+    def _match_on(self, branch, revs):
+        if self.spec == "bork":
+            return RevisionInfo.from_revision_id(branch, "r1", revs)
+        else:
+            raise errors.InvalidRevisionSpec(self.spec, branch)
+
+
 class TestRevisionSpec_dwim(TestRevisionSpec):
 
     # Don't need to test revno's explicitly since TRS_revno already
@@ -190,6 +198,23 @@
         self.assertInvalid('1.2..1', invalid_as_revision_id=False)
         self.assertInvalid('1.', invalid_as_revision_id=False)
 
+    def test_append_dwim_revspec(self):
+        original_dwim_revspecs = list(RevisionSpec_dwim._possible_revspecs)
+        def reset_dwim_revspecs():
+            RevisionSpec_dwim._possible_revspecs = original_dwim_revspecs
+        self.addCleanup(reset_dwim_revspecs)
+        RevisionSpec_dwim.append_possible_revspec(RevisionSpec_bork)
+        self.assertAsRevisionId('r1', 'bork')
+
+    def test_append_lazy_dwim_revspec(self):
+        original_dwim_revspecs = list(RevisionSpec_dwim._possible_revspecs)
+        def reset_dwim_revspecs():
+            RevisionSpec_dwim._possible_revspecs = original_dwim_revspecs
+        self.addCleanup(reset_dwim_revspecs)
+        RevisionSpec_dwim.append_possible_lazy_revspec(
+            "bzrlib.tests.test_revisionspec", "RevisionSpec_bork")
+        self.assertAsRevisionId('r1', 'bork')
+
 
 class TestRevisionSpec_revno(TestRevisionSpec):
 
@@ -372,8 +397,7 @@
         """We can get any revision id in the repository"""
         # XXX: This may change in the future, but for now, it is true
         self.tree2.commit('alt third', rev_id='alt_r3')
-        self.tree.branch.repository.fetch(self.tree2.branch.repository,
-                                          revision_id='alt_r3')
+        self.tree.branch.fetch(self.tree2.branch, 'alt_r3')
         self.assertInHistoryIs(None, 'alt_r3', 'revid:alt_r3')
 
     def test_unicode(self):
@@ -450,8 +474,7 @@
     def test_alt_no_parents(self):
         new_tree = self.make_branch_and_tree('new_tree')
         new_tree.commit('first', rev_id='new_r1')
-        self.tree.branch.repository.fetch(new_tree.branch.repository,
-                                          revision_id='new_r1')
+        self.tree.branch.fetch(new_tree.branch, 'new_r1')
         self.assertInHistoryIs(0, 'null:', 'before:revid:new_r1')
 
     def test_as_revision_id(self):
@@ -616,7 +639,7 @@
         # XXX: Right now, we use fetch() to make sure the remote revisions
         # have been pulled into the local branch. We may change that
         # behavior in the future.
-        self.failUnless(self.tree.branch.repository.has_revision('new_r3'))
+        self.assertTrue(self.tree.branch.repository.has_revision('new_r3'))
 
     def test_no_commits(self):
         new_tree = self.make_branch_and_tree('new_tree')
@@ -652,3 +675,79 @@
     def test_as_revision_id(self):
         self.tree.branch.set_submit_branch('tree2')
         self.assertAsRevisionId('alt_r2', 'branch:tree2')
+
+
+class TestRevisionSpec_mainline(TestRevisionSpec):
+
+    def test_as_revision_id(self):
+        self.assertAsRevisionId('r1', 'mainline:1')
+        self.assertAsRevisionId('r2', 'mainline:1.1.1')
+        self.assertAsRevisionId('r2', 'mainline:revid:alt_r2')
+        spec = RevisionSpec.from_string('mainline:revid:alt_r22')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            "Requested revision: 'mainline:revid:alt_r22' does not exist in"
+            " branch: ")
+
+    def test_in_history(self):
+        self.assertInHistoryIs(2, 'r2', 'mainline:revid:alt_r2')
+
+
+class TestRevisionSpec_annotate(TestRevisionSpec):
+
+    def setUp(self):
+        TestRevisionSpec.setUp(self)
+        self.tree = self.make_branch_and_tree('annotate-tree')
+        self.build_tree_contents([('annotate-tree/file1', '1\n')])
+        self.tree.add('file1')
+        self.tree.commit('r1', rev_id='r1')
+        self.build_tree_contents([('annotate-tree/file1', '2\n1\n')])
+        self.tree.commit('r2', rev_id='r2')
+        self.build_tree_contents([('annotate-tree/file1', '2\n1\n3\n')])
+
+    def test_as_revision_id_r1(self):
+        self.assertAsRevisionId('r1', 'annotate:annotate-tree/file1:2')
+
+    def test_as_revision_id_r2(self):
+        self.assertAsRevisionId('r2', 'annotate:annotate-tree/file1:1')
+
+    def test_as_revision_id_uncommitted(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file1:3')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file1:3\' does not"
+            " exist in branch: .*\nLine 3 has not been committed.")
+
+    def test_non_existent_line(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file1:4')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file1:4\' does not"
+            " exist in branch: .*\nNo such line: 4")
+
+    def test_invalid_line(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file1:q')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file1:q\' does not"
+            " exist in branch: .*\nNo such line: q")
+
+    def test_no_such_file(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/file2:1')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/file2:1\' does not"
+            " exist in branch: .*\nFile 'file2' is not versioned")
+
+    def test_no_such_file_with_colon(self):
+        spec = RevisionSpec.from_string('annotate:annotate-tree/fi:le2:1')
+        e = self.assertRaises(errors.InvalidRevisionSpec,
+                              spec.as_revision_id, self.tree.branch)
+        self.assertContainsRe(str(e),
+            r"Requested revision: \'annotate:annotate-tree/fi:le2:1\' does not"
+            " exist in branch: .*\nFile 'fi:le2' is not versioned")

=== modified file 'bzrlib/tests/test_revisiontree.py'
--- bzrlib/tests/test_revisiontree.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_revisiontree.py	2011-05-09 13:55:20 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2008-2011 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -21,8 +21,6 @@
     errors,
     revision,
     )
-import bzrlib
-from bzrlib.inventory import ROOT_ID
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -61,9 +59,23 @@
         null_tree = self.t.branch.repository.revision_tree(
             revision.NULL_REVISION)
         self.assertIs(None, null_tree.inventory.root)
+        self.assertIs(None, null_tree.get_root_id())
+
+    def test_get_file_revision_root(self):
+        self.assertEquals(self.rev_id,
+            self.rev_tree.get_file_revision(self.rev_tree.get_root_id()))
+
+    def test_get_file_revision(self):
+        self.build_tree_contents([('a', 'initial')])
+        self.t.add(['a'])
+        revid1 = self.t.commit('add a')
+        revid2 = self.t.commit('another change', allow_pointless=True)
+        tree = self.t.branch.repository.revision_tree(revid2)
+        self.assertEquals(revid1,
+            tree.get_file_revision(tree.path2id('a')))
 
     def test_get_file_mtime_ghost(self):
-        file_id = iter(self.rev_tree).next()
+        file_id = iter(self.rev_tree.all_file_ids()).next()
         self.rev_tree.inventory[file_id].revision = 'ghostrev'
         self.assertRaises(errors.FileTimestampUnavailable, 
             self.rev_tree.get_file_mtime, file_id)

=== modified file 'bzrlib/tests/test_rio.py'
--- bzrlib/tests/test_rio.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_rio.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,18 +22,20 @@
 but this depends on the transport.
 """
 
-import cStringIO
-import os
 import re
-import sys
 from tempfile import TemporaryFile
 
 from bzrlib import (
     rio,
     )
-from bzrlib.tests import TestCaseInTempDir, TestCase
-from bzrlib.rio import (RioWriter, Stanza, read_stanza, read_stanzas, rio_file,
-                        RioReader)
+from bzrlib.tests import TestCase
+from bzrlib.rio import (
+    RioReader,
+    Stanza,
+    read_stanza,
+    read_stanzas,
+    rio_file,
+    )
 
 
 class TestRio(TestCase):
@@ -188,6 +190,14 @@
         self.assertEqual(s, None)
         self.assertTrue(s is None)
 
+    def test_read_nul_byte(self):
+        """File consisting of a nul byte causes an error."""
+        self.assertRaises(ValueError, read_stanza, ['\0'])
+
+    def test_read_nul_bytes(self):
+        """File consisting of many nul bytes causes an error."""
+        self.assertRaises(ValueError, read_stanza, ['\0' * 100])
+
     def test_read_iter(self):
         """Read several stanzas from file"""
         tmpf = TemporaryFile()

=== modified file 'bzrlib/tests/test_rules.py'
--- bzrlib/tests/test_rules.py	2010-01-14 15:24:04 +0000
+++ bzrlib/tests/test_rules.py	2011-04-19 07:52:11 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,16 +16,13 @@
 
 """Tests for finding, parsing and searching rule-based preferences."""
 
-import os
 import sys
 
 from bzrlib import (
-    config,
     errors,
     rules,
     tests,
     )
-from bzrlib.util.configobj import configobj
 
 
 class TestIniBasedRulesSearcher(tests.TestCase):
@@ -46,11 +43,13 @@
         rs = self.make_searcher(None)
         self.assertEquals((), rs.get_items('a.txt'))
         self.assertEquals((), rs.get_selected_items('a.txt', ['foo']))
+        self.assertEquals(None, rs.get_single_value('a.txt', 'foo'))
 
     def test_get_items_file_empty(self):
         rs = self.make_searcher("")
         self.assertEquals((), rs.get_items('a.txt'))
         self.assertEquals((), rs.get_selected_items('a.txt', ['foo']))
+        self.assertEquals(None, rs.get_single_value('a.txt', 'foo'))
 
     def test_get_items_from_extension_match(self):
         rs = self.make_searcher("[name *.txt]\nfoo=bar\na=True\n")
@@ -61,6 +60,7 @@
             rs.get_items('dir/a.txt'))
         self.assertEquals((('foo', 'bar'),),
             rs.get_selected_items('a.txt', ['foo']))
+        self.assertEquals('bar', rs.get_single_value('a.txt', 'foo'))
 
     def test_get_items_from_multiple_glob_match(self):
         rs = self.make_searcher(
@@ -74,12 +74,15 @@
             rs.get_items('x x'))
         self.assertEquals((('foo', 'bar'), ('a', 'True')),
             rs.get_items('y y'))
+        self.assertEquals('bar', rs.get_single_value('a.txt', 'foo'))
 
     def test_get_items_pathname_match(self):
         rs = self.make_searcher("[name ./a.txt]\nfoo=baz\n")
         self.assertEquals((('foo', 'baz'),),
             rs.get_items('a.txt'))
+        self.assertEquals('baz', rs.get_single_value('a.txt', 'foo'))
         self.assertEquals((), rs.get_items('dir/a.txt'))
+        self.assertEquals(None, rs.get_single_value('dir/a.txt', 'foo'))
 
     def test_get_items_match_first(self):
         rs = self.make_searcher(
@@ -87,8 +90,10 @@
             "[name *.txt]\nfoo=bar\na=True\n")
         self.assertEquals((('foo', 'baz'),),
             rs.get_items('a.txt'))
+        self.assertEquals('baz', rs.get_single_value('a.txt', 'foo'))
         self.assertEquals((('foo', 'bar'), ('a', 'True')),
             rs.get_items('dir/a.txt'))
+        self.assertEquals('bar', rs.get_single_value('dir/a.txt', 'foo'))
 
 
 class TestStackedRulesSearcher(tests.TestCase):
@@ -110,18 +115,22 @@
             "[name *.txt]\nfoo=bar\na=True\n")
         self.assertEquals((('foo', 'baz'),),
             rs.get_items('a.txt'))
+        self.assertEquals('baz', rs.get_single_value('a.txt', 'foo'))
+        self.assertEquals(None, rs.get_single_value('a.txt', 'a'))
         self.assertEquals((('foo', 'bar'), ('a', 'True')),
             rs.get_items('dir/a.txt'))
+        self.assertEquals('bar', rs.get_single_value('dir/a.txt', 'foo'))
+        self.assertEquals('True', rs.get_single_value('dir/a.txt', 'a'))
 
 
 class TestRulesPath(tests.TestCase):
 
     def setUp(self):
         super(TestRulesPath, self).setUp()
-        os.environ['HOME'] = '/home/bogus'
+        self.overrideEnv('HOME', '/home/bogus')
         if sys.platform == 'win32':
-            os.environ['BZR_HOME'] = \
-                r'C:\Documents and Settings\bogus\Application Data'
+            self.overrideEnv(
+                'BZR_HOME', r'C:\Documents and Settings\bogus\Application Data')
             self.bzr_home = \
                 'C:/Documents and Settings/bogus/Application Data/bazaar/2.0'
         else:

=== added file 'bzrlib/tests/test_scenarios.py'
--- bzrlib/tests/test_scenarios.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_scenarios.py	2010-11-05 20:54:32 +0000
@@ -0,0 +1,110 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Tests for generating multiple tests for scenarios."""
+
+from bzrlib.tests import (
+    TestCase,
+    TestLoader,
+    iter_suite_tests,
+    multiply_tests,
+    )
+
+from bzrlib.tests.scenarios import (
+    load_tests_apply_scenarios,
+    multiply_scenarios,
+    multiply_tests_by_their_scenarios,
+    )
+
+
+# There aren't any actually parameterized tests here, but this exists as a
+# demonstration; so that you can interactively observe them being multiplied;
+# and so that we check everything hooks up properly.
+load_tests = load_tests_apply_scenarios
+
+
+def vary_by_color():
+    """Very simple static variation example"""
+    for color in ['red', 'green', 'blue']:
+        yield (color, {'color': color})
+
+
+def vary_named_attribute(attr_name):
+    """More sophisticated: vary a named parameter"""
+    yield ('a', {attr_name: 'a'})
+    yield ('b', {attr_name: 'b'})
+
+
+def get_generated_test_attributes(suite, attr_name):
+    """Return the `attr_name` attribute from all tests in the suite"""
+    return sorted([
+        getattr(t, attr_name) for t in iter_suite_tests(suite)])
+
+
+class TestTestScenarios(TestCase):
+
+    def test_multiply_tests(self):
+        loader = TestLoader()
+        suite = loader.suiteClass()
+        multiply_tests(
+            self,
+            vary_by_color(),
+            suite)
+        self.assertEquals(
+            ['blue', 'green', 'red'],
+            get_generated_test_attributes(suite, 'color'))
+
+    def test_multiply_scenarios_from_generators(self):
+        """It's safe to multiply scenarios that come from generators"""
+        s = multiply_scenarios(
+            vary_named_attribute('one'),
+            vary_named_attribute('two'),
+            )
+        self.assertEquals(
+            2*2,
+            len(s),
+            s)
+
+    def test_multiply_tests_by_their_scenarios(self):
+        loader = TestLoader()
+        suite = loader.suiteClass()
+        test_instance = PretendVaryingTest('test_nothing')
+        multiply_tests_by_their_scenarios(
+            test_instance,
+            suite)
+        self.assertEquals(
+            ['a', 'a', 'b', 'b'],
+            get_generated_test_attributes(suite, 'value'))
+
+    def test_multiply_tests_no_scenarios(self):
+        """Tests with no scenarios attribute aren't multiplied"""
+        suite = TestLoader().suiteClass()
+        multiply_tests_by_their_scenarios(self,
+            suite)
+        self.assertLength(1, list(iter_suite_tests(suite)))
+
+
+class PretendVaryingTest(TestCase):
+    
+    scenarios = multiply_scenarios(
+        vary_named_attribute('value'), 
+        vary_named_attribute('other'),
+        )
+
+    def test_nothing(self):
+        """This test exists just so it can be multiplied"""
+        pass

=== modified file 'bzrlib/tests/test_script.py'
--- bzrlib/tests/test_script.py	2010-06-11 06:33:15 +0000
+++ bzrlib/tests/test_script.py	2011-10-05 22:15:03 +0000
@@ -16,8 +16,11 @@
 
 
 from bzrlib import (
+    commands,
     osutils,
     tests,
+    trace,
+    ui,
     )
 from bzrlib.tests import script
 
@@ -27,8 +30,33 @@
     def test_comment_is_ignored(self):
         self.assertEquals([], script._script_to_commands('#comment\n'))
 
-    def test_empty_line_is_ignored(self):
-        self.assertEquals([], script._script_to_commands('\n'))
+    def test_comment_multiple_lines(self):
+        self.assertEquals([
+            (['bar'], None, None, None),
+            ],
+            script._script_to_commands("""
+            # this comment is ignored
+            # so is this
+            # no we run bar
+            $ bar
+            """))
+
+    def test_trim_blank_lines(self):
+        """Blank lines are respected, but trimmed at the start and end.
+
+        Python triple-quoted syntax is going to give stubby/empty blank lines 
+        right at the start and the end.  These are cut off so that callers don't 
+        need special syntax to avoid them.
+
+        However we do want to be able to match commands that emit blank lines.
+        """
+        self.assertEquals([
+            (['bar'], None, '\n', None),
+            ],
+            script._script_to_commands("""
+            $bar
+
+            """))
 
     def test_simple_command(self):
         self.assertEquals([(['cd', 'trunk'], None, None, None)],
@@ -133,7 +161,51 @@
 class TestExecution(script.TestCaseWithTransportAndScript):
 
     def test_unknown_command(self):
-        self.assertRaises(SyntaxError, self.run_script, 'foo')
+        """A clear error is reported for commands that aren't recognised
+
+        Testing the attributes of the SyntaxError instance is equivalent to
+        using traceback.format_exception_only and comparing with:
+          File "<string>", line 1
+            foo --frob
+            ^
+        SyntaxError: Command not found "foo"
+        """
+        e = self.assertRaises(SyntaxError, self.run_script, "$ foo --frob")
+        self.assertContainsRe(e.msg, "not found.*foo")
+        self.assertEquals(e.text, "foo --frob")
+
+    def test_blank_output_mismatches_output(self):
+        """If you give output, the output must actually be blank.
+        
+        See <https://bugs.launchpad.net/bzr/+bug/637830>: previously blank
+        output was a wildcard.  Now you must say ... if you want that.
+        """
+        self.assertRaises(AssertionError,
+            self.run_script,
+            """
+            $ echo foo
+            """)
+
+    def test_null_output_matches_option(self):
+        """If you want null output to be a wild card, you can pass 
+        null_output_matches_anything to run_script"""
+        self.run_script(
+            """
+            $ echo foo
+            """, null_output_matches_anything=True)
+
+    def test_ellipsis_everything(self):
+        """A simple ellipsis matches everything."""
+        self.run_script("""
+        $ echo foo
+        ...
+        """)
+
+    def test_ellipsis_matches_empty(self):
+        self.run_script("""
+        $ cd .
+        ...
+        """)
 
     def test_stops_on_unexpected_output(self):
         story = """
@@ -143,7 +215,6 @@
 """
         self.assertRaises(AssertionError, self.run_script, story)
 
-
     def test_stops_on_unexpected_error(self):
         story = """
 $ cat
@@ -163,10 +234,13 @@
         # The status matters, not the output
         story = """
 $ bzr init
+...
 $ cat >file
 <Hello
 $ bzr add file
+...
 $ bzr commit -m 'adding file'
+2>...
 """
         self.run_script(story)
 
@@ -218,6 +292,15 @@
 cat dog "chicken" 'dragon'
 """)
 
+    def test_verbosity_isolated(self):
+        """Global verbosity is isolated from commands run in scripts.
+        """
+        # see also 656694; we should get rid of global verbosity
+        self.run_script("""
+        $ bzr init --quiet a
+        """)
+        self.assertEquals(trace.is_quiet(), False)
+
 
 class TestCat(script.TestCaseWithTransportAndScript):
 
@@ -300,8 +383,8 @@
 $ mkdir ../dir2
 $ cd ..
 """)
-        self.failUnlessExists('dir')
-        self.failUnlessExists('dir2')
+        self.assertPathExists('dir')
+        self.assertPathExists('dir2')
 
 
 class TestCd(script.TestCaseWithTransportAndScript):
@@ -329,8 +412,11 @@
 class TestBzr(script.TestCaseWithTransportAndScript):
 
     def test_bzr_smoke(self):
-        self.run_script('$ bzr init branch')
-        self.failUnlessExists('branch')
+        self.run_script("""
+            $ bzr init branch
+            Created a standalone tree (format: ...)
+            """)
+        self.assertPathExists('branch')
 
 
 class TestEcho(script.TestCaseWithMemoryTransportAndScript):
@@ -377,6 +463,14 @@
         self.assertEquals(None, err)
         self.assertFileEqual('hello\nhappy\n', 'file')
 
+    def test_empty_line_in_output_is_respected(self):
+        self.run_script("""
+            $ echo
+
+            $ echo bar
+            bar
+            """)
+
 
 class TestRm(script.TestCaseWithTransportAndScript):
 
@@ -386,39 +480,39 @@
 
     def test_rm_file(self):
         self.run_script('$ echo content >file')
-        self.failUnlessExists('file')
+        self.assertPathExists('file')
         self.run_script('$ rm file')
-        self.failIfExists('file')
+        self.assertPathDoesNotExist('file')
 
     def test_rm_file_force(self):
-        self.failIfExists('file')
+        self.assertPathDoesNotExist('file')
         self.run_script('$ rm -f file')
-        self.failIfExists('file')
+        self.assertPathDoesNotExist('file')
 
     def test_rm_files(self):
         self.run_script("""
 $ echo content >file
 $ echo content >file2
 """)
-        self.failUnlessExists('file2')
+        self.assertPathExists('file2')
         self.run_script('$ rm file file2')
-        self.failIfExists('file2')
+        self.assertPathDoesNotExist('file2')
 
     def test_rm_dir(self):
         self.run_script('$ mkdir dir')
-        self.failUnlessExists('dir')
+        self.assertPathExists('dir')
         self.run_script("""
 $ rm dir
 2>rm: cannot remove 'dir': Is a directory
 """)
-        self.failUnlessExists('dir')
+        self.assertPathExists('dir')
 
     def test_rm_dir_recursive(self):
         self.run_script("""
 $ mkdir dir
 $ rm -r dir
 """)
-        self.failIfExists('dir')
+        self.assertPathDoesNotExist('dir')
 
 
 class TestMv(script.TestCaseWithTransportAndScript):
@@ -430,10 +524,10 @@
 
     def test_move_file(self):
         self.run_script('$ echo content >file')
-        self.failUnlessExists('file')
+        self.assertPathExists('file')
         self.run_script('$ mv file new_name')
-        self.failIfExists('file')
-        self.failUnlessExists('new_name')
+        self.assertPathDoesNotExist('file')
+        self.assertPathExists('new_name')
 
     def test_move_unknown_file(self):
         self.assertRaises(AssertionError,
@@ -445,9 +539,9 @@
 $ echo content >dir/file
 """)
         self.run_script('$ mv dir new_name')
-        self.failIfExists('dir')
-        self.failUnlessExists('new_name')
-        self.failUnlessExists('new_name/file')
+        self.assertPathDoesNotExist('dir')
+        self.assertPathExists('new_name')
+        self.assertPathExists('new_name/file')
 
     def test_move_file_into_dir(self):
         self.run_script("""
@@ -455,7 +549,91 @@
 $ echo content > file
 """)
         self.run_script('$ mv file dir')
-        self.failUnlessExists('dir')
-        self.failIfExists('file')
-        self.failUnlessExists('dir/file')
-
+        self.assertPathExists('dir')
+        self.assertPathDoesNotExist('file')
+        self.assertPathExists('dir/file')
+
+
+class cmd_test_confirm(commands.Command):
+
+    def run(self):
+        if ui.ui_factory.get_boolean(
+            u'Really do it',
+            # 'bzrlib.tests.test_script.confirm',
+            # {}
+            ):
+            self.outf.write('Do it!\n')
+        else:
+            print 'ok, no'
+
+
+class TestUserInteraction(script.TestCaseWithMemoryTransportAndScript):
+
+    def test_confirm_action(self):
+        """You can write tests that demonstrate user confirmation.
+        
+        Specifically, ScriptRunner does't care if the output line for the
+        prompt isn't terminated by a newline from the program; it's implicitly
+        terminated by the input.
+        """
+        commands.builtin_command_registry.register(cmd_test_confirm)
+        self.addCleanup(commands.builtin_command_registry.remove, 'test-confirm')
+        self.run_script("""
+            $ bzr test-confirm
+            2>Really do it? ([y]es, [n]o): yes
+            <y
+            Do it!
+            $ bzr test-confirm
+            2>Really do it? ([y]es, [n]o): no
+            <n
+            ok, no
+            """)
+
+class TestShelve(script.TestCaseWithTransportAndScript):
+
+    def setUp(self):
+        super(TestShelve, self).setUp()
+        self.run_script("""
+            $ bzr init test
+            Created a standalone tree (format: 2a)
+            $ cd test
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'file added'
+            2>Committing to:...test/
+            2>added file
+            2>Committed revision 1.
+            $ echo bar > file
+            """)
+
+    def test_shelve(self):
+        self.run_script("""
+            $ bzr shelve -m 'shelve bar'
+            2>Shelve? ([y]es, [N]o, [f]inish, [q]uit): yes
+            <y
+            2>Selected changes:
+            2> M  file
+            2>Shelve 1 change(s)? ([y]es, [N]o, [f]inish, [q]uit): yes
+            <y
+            2>Changes shelved with id "1".
+            """,
+                        null_output_matches_anything=True)
+        self.run_script("""
+            $ bzr shelve --list
+              1: shelve bar
+            """)
+
+    def test_dont_shelve(self):
+        # We intentionally provide no input here to test EOF
+        self.run_script("""
+            $ bzr shelve -m 'shelve bar'
+            2>Shelve? ([y]es, [N]o, [f]inish, [q]uit): 
+            2>No changes to shelve.
+            """,
+                        null_output_matches_anything=True)
+        self.run_script("""
+            $ bzr st
+            modified:
+              file
+            """)

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2010-07-23 18:45:31 +0000
+++ bzrlib/tests/test_selftest.py	2011-10-05 14:46:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,43 +17,49 @@
 """Tests for the test framework."""
 
 from cStringIO import StringIO
-from doctest import ELLIPSIS
+import gc
+import doctest
 import os
 import signal
 import sys
+import threading
 import time
 import unittest
 import warnings
 
-from testtools import MultiTestResult
+from testtools import (
+    ExtendedToOriginalDecorator,
+    MultiTestResult,
+    __version__ as testtools_version,
+    )
+from testtools.content import Content
 from testtools.content_type import ContentType
 from testtools.matchers import (
     DocTestMatches,
     Equals,
     )
-import testtools.tests.helpers
+import testtools.testresult.doubles
 
 import bzrlib
 from bzrlib import (
     branchbuilder,
     bzrdir,
-    debug,
     errors,
+    hooks,
     lockdir,
     memorytree,
     osutils,
-    progress,
     remote,
     repository,
     symbol_versioning,
     tests,
     transport,
     workingtree,
+    workingtree_3,
+    workingtree_4,
     )
 from bzrlib.repofmt import (
     groupcompress_repo,
-    pack_repo,
-    weaverepo,
     )
 from bzrlib.symbol_versioning import (
     deprecated_function,
@@ -64,12 +70,10 @@
     features,
     test_lsprof,
     test_server,
-    test_sftp_transport,
     TestUtil,
     )
-from bzrlib.trace import note
+from bzrlib.trace import note, mutter
 from bzrlib.transport import memory
-from bzrlib.version import _get_bzr_source_tree
 
 
 def _test_ids(test_suite):
@@ -77,18 +81,6 @@
     return [t.id() for t in tests.iter_suite_tests(test_suite)]
 
 
-class SelftestTests(tests.TestCase):
-
-    def test_import_tests(self):
-        mod = TestUtil._load_module_by_name('bzrlib.tests.test_selftest')
-        self.assertEqual(mod.SelftestTests, SelftestTests)
-
-    def test_import_test_failure(self):
-        self.assertRaises(ImportError,
-                          TestUtil._load_module_by_name,
-                          'bzrlib.no-name-yet')
-
-
 class MetaTestLog(tests.TestCase):
 
     def test_logging(self):
@@ -100,26 +92,30 @@
             "text", "plain", {"charset": "utf8"})))
         self.assertThat(u"".join(log.iter_text()), Equals(self.get_log()))
         self.assertThat(self.get_log(),
-            DocTestMatches(u"...a test message\n", ELLIPSIS))
-
-
-class TestUnicodeFilename(tests.TestCase):
-
-    def test_probe_passes(self):
-        """UnicodeFilename._probe passes."""
-        # We can't test much more than that because the behaviour depends
-        # on the platform.
-        tests.UnicodeFilename._probe()
+            DocTestMatches(u"...a test message\n", doctest.ELLIPSIS))
 
 
 class TestTreeShape(tests.TestCaseInTempDir):
 
     def test_unicode_paths(self):
-        self.requireFeature(tests.UnicodeFilename)
+        self.requireFeature(features.UnicodeFilenameFeature)
 
         filename = u'hell\u00d8'
         self.build_tree_contents([(filename, 'contents of hello')])
-        self.failUnlessExists(filename)
+        self.assertPathExists(filename)
+
+
+class TestClassesAvailable(tests.TestCase):
+    """As a convenience we expose Test* classes from bzrlib.tests"""
+
+    def test_test_case(self):
+        from bzrlib.tests import TestCase
+
+    def test_test_loader(self):
+        from bzrlib.tests import TestLoader
+
+    def test_test_suite(self):
+        from bzrlib.tests import TestSuite
 
 
 class TestTransportScenarios(tests.TestCase):
@@ -208,7 +204,7 @@
     def test_scenarios(self):
         # check that constructor parameters are passed through to the adapted
         # test.
-        from bzrlib.tests.per_bzrdir import make_scenarios
+        from bzrlib.tests.per_controldir import make_scenarios
         vfs_factory = "v"
         server1 = "a"
         server2 = "b"
@@ -312,19 +308,21 @@
         from bzrlib.tests.per_interrepository import make_scenarios
         server1 = "a"
         server2 = "b"
-        formats = [("C0", "C1", "C2"), ("D0", "D1", "D2")]
+        formats = [("C0", "C1", "C2", "C3"), ("D0", "D1", "D2", "D3")]
         scenarios = make_scenarios(server1, server2, formats)
         self.assertEqual([
             ('C0,str,str',
              {'repository_format': 'C1',
               'repository_format_to': 'C2',
               'transport_readonly_server': 'b',
-              'transport_server': 'a'}),
+              'transport_server': 'a',
+              'extra_setup': 'C3'}),
             ('D0,str,str',
              {'repository_format': 'D1',
               'repository_format_to': 'D2',
               'transport_readonly_server': 'b',
-              'transport_server': 'a'})],
+              'transport_server': 'a',
+              'extra_setup': 'D3'})],
             scenarios)
 
 
@@ -336,11 +334,11 @@
         from bzrlib.tests.per_workingtree import make_scenarios
         server1 = "a"
         server2 = "b"
-        formats = [workingtree.WorkingTreeFormat2(),
-                   workingtree.WorkingTreeFormat3(),]
+        formats = [workingtree_4.WorkingTreeFormat4(),
+                   workingtree_3.WorkingTreeFormat3(),]
         scenarios = make_scenarios(server1, server2, formats)
         self.assertEqual([
-            ('WorkingTreeFormat2',
+            ('WorkingTreeFormat4',
              {'bzrdir_format': formats[0]._matchingbzrdir,
               'transport_readonly_server': 'b',
               'transport_server': 'a',
@@ -373,15 +371,15 @@
             )
         server1 = "a"
         server2 = "b"
-        formats = [workingtree.WorkingTreeFormat2(),
-                   workingtree.WorkingTreeFormat3(),]
+        formats = [workingtree_4.WorkingTreeFormat4(),
+                   workingtree_3.WorkingTreeFormat3(),]
         scenarios = make_scenarios(server1, server2, formats)
         self.assertEqual(7, len(scenarios))
-        default_wt_format = workingtree.WorkingTreeFormat4._default_format
-        wt4_format = workingtree.WorkingTreeFormat4()
-        wt5_format = workingtree.WorkingTreeFormat5()
+        default_wt_format = workingtree.format_registry.get_default()
+        wt4_format = workingtree_4.WorkingTreeFormat4()
+        wt5_format = workingtree_4.WorkingTreeFormat5()
         expected_scenarios = [
-            ('WorkingTreeFormat2',
+            ('WorkingTreeFormat4',
              {'bzrdir_format': formats[0]._matchingbzrdir,
               'transport_readonly_server': 'b',
               'transport_server': 'a',
@@ -447,17 +445,17 @@
         # ones to add.
         from bzrlib.tests.per_tree import (
             return_parameter,
-            revision_tree_from_workingtree
             )
         from bzrlib.tests.per_intertree import (
             make_scenarios,
             )
-        from bzrlib.workingtree import WorkingTreeFormat2, WorkingTreeFormat3
+        from bzrlib.workingtree_3 import WorkingTreeFormat3
+        from bzrlib.workingtree_4 import WorkingTreeFormat4
         input_test = TestInterTreeScenarios(
             "test_scenarios")
         server1 = "a"
         server2 = "b"
-        format1 = WorkingTreeFormat2()
+        format1 = WorkingTreeFormat4()
         format2 = WorkingTreeFormat3()
         formats = [("1", str, format1, format2, "converter1"),
             ("2", int, format2, format1, "converter2")]
@@ -509,6 +507,25 @@
         self.assertRaises(AssertionError, self.assertEqualStat,
             os.lstat("foo"), os.lstat("longname"))
 
+    def test_failUnlessExists(self):
+        """Deprecated failUnlessExists and failIfExists"""
+        self.applyDeprecated(
+            deprecated_in((2, 4)),
+            self.failUnlessExists, '.')
+        self.build_tree(['foo/', 'foo/bar'])
+        self.applyDeprecated(
+            deprecated_in((2, 4)),
+            self.failUnlessExists, 'foo/bar')
+        self.applyDeprecated(
+            deprecated_in((2, 4)),
+            self.failIfExists, 'foo/foo')
+
+    def test_assertPathExists(self):
+        self.assertPathExists('.')
+        self.build_tree(['foo/', 'foo/bar'])
+        self.assertPathExists('foo/bar')
+        self.assertPathDoesNotExist('foo/foo')
+
 
 class TestTestCaseWithMemoryTransport(tests.TestCaseWithMemoryTransport):
 
@@ -548,17 +565,17 @@
         tree = self.make_branch_and_memory_tree('dir')
         # Guard against regression into MemoryTransport leaking
         # files to disk instead of keeping them in memory.
-        self.failIf(osutils.lexists('dir'))
+        self.assertFalse(osutils.lexists('dir'))
         self.assertIsInstance(tree, memorytree.MemoryTree)
 
     def test_make_branch_and_memory_tree_with_format(self):
         """make_branch_and_memory_tree should accept a format option."""
         format = bzrdir.BzrDirMetaFormat1()
-        format.repository_format = weaverepo.RepositoryFormat7()
+        format.repository_format = repository.format_registry.get_default()
         tree = self.make_branch_and_memory_tree('dir', format=format)
         # Guard against regression into MemoryTransport leaking
         # files to disk instead of keeping them in memory.
-        self.failIf(osutils.lexists('dir'))
+        self.assertFalse(osutils.lexists('dir'))
         self.assertIsInstance(tree, memorytree.MemoryTree)
         self.assertEqual(format.repository_format.__class__,
             tree.branch.repository._format.__class__)
@@ -568,19 +585,19 @@
         self.assertIsInstance(builder, branchbuilder.BranchBuilder)
         # Guard against regression into MemoryTransport leaking
         # files to disk instead of keeping them in memory.
-        self.failIf(osutils.lexists('dir'))
+        self.assertFalse(osutils.lexists('dir'))
 
     def test_make_branch_builder_with_format(self):
         # Use a repo layout that doesn't conform to a 'named' layout, to ensure
         # that the format objects are used.
         format = bzrdir.BzrDirMetaFormat1()
-        repo_format = weaverepo.RepositoryFormat7()
+        repo_format = repository.format_registry.get_default()
         format.repository_format = repo_format
         builder = self.make_branch_builder('dir', format=format)
         the_branch = builder.get_branch()
         # Guard against regression into MemoryTransport leaking
         # files to disk instead of keeping them in memory.
-        self.failIf(osutils.lexists('dir'))
+        self.assertFalse(osutils.lexists('dir'))
         self.assertEqual(format.repository_format.__class__,
                          the_branch.repository._format.__class__)
         self.assertEqual(repo_format.get_format_string(),
@@ -592,7 +609,7 @@
         the_branch = builder.get_branch()
         # Guard against regression into MemoryTransport leaking
         # files to disk instead of keeping them in memory.
-        self.failIf(osutils.lexists('dir'))
+        self.assertFalse(osutils.lexists('dir'))
         dir_format = bzrdir.format_registry.make_bzrdir('knit')
         self.assertEqual(dir_format.repository_format.__class__,
                          the_branch.repository._format.__class__)
@@ -603,7 +620,7 @@
     def test_dangling_locks_cause_failures(self):
         class TestDanglingLock(tests.TestCaseWithMemoryTransport):
             def test_function(self):
-                t = self.get_transport('.')
+                t = self.get_transport_from_path('.')
                 l = lockdir.LockDir(t, 'lock')
                 l.create()
                 l.attempt_lock()
@@ -611,11 +628,11 @@
         result = test.run()
         total_failures = result.errors + result.failures
         if self._lock_check_thorough:
-            self.assertLength(1, total_failures)
+            self.assertEqual(1, len(total_failures))
         else:
             # When _lock_check_thorough is disabled, then we don't trigger a
             # failure
-            self.assertLength(0, total_failures)
+            self.assertEqual(0, len(total_failures))
 
 
 class TestTestCaseWithTransport(tests.TestCaseWithTransport):
@@ -629,10 +646,10 @@
         # for the server
         url = self.get_readonly_url()
         url2 = self.get_readonly_url('foo/bar')
-        t = transport.get_transport(url)
-        t2 = transport.get_transport(url2)
-        self.failUnless(isinstance(t, ReadonlyTransportDecorator))
-        self.failUnless(isinstance(t2, ReadonlyTransportDecorator))
+        t = transport.get_transport_from_url(url)
+        t2 = transport.get_transport_from_url(url2)
+        self.assertIsInstance(t, ReadonlyTransportDecorator)
+        self.assertIsInstance(t2, ReadonlyTransportDecorator)
         self.assertEqual(t2.base[:-1], t.abspath('foo/bar'))
 
     def test_get_readonly_url_http(self):
@@ -644,10 +661,10 @@
         url = self.get_readonly_url()
         url2 = self.get_readonly_url('foo/bar')
         # the transport returned may be any HttpTransportBase subclass
-        t = transport.get_transport(url)
-        t2 = transport.get_transport(url2)
-        self.failUnless(isinstance(t, HttpTransportBase))
-        self.failUnless(isinstance(t2, HttpTransportBase))
+        t = transport.get_transport_from_url(url)
+        t2 = transport.get_transport_from_url(url2)
+        self.assertIsInstance(t, HttpTransportBase)
+        self.assertIsInstance(t2, HttpTransportBase)
         self.assertEqual(t2.base[:-1], t.abspath('foo/bar'))
 
     def test_is_directory(self):
@@ -661,7 +678,7 @@
     def test_make_branch_builder(self):
         builder = self.make_branch_builder('dir')
         rev_id = builder.build_commit()
-        self.failUnlessExists('dir')
+        self.assertPathExists('dir')
         a_dir = bzrdir.BzrDir.open('dir')
         self.assertRaises(errors.NoWorkingTree, a_dir.open_workingtree)
         a_branch = a_dir.open_branch()
@@ -683,13 +700,13 @@
         self.assertIsInstance(result_bzrdir.transport,
                               memory.MemoryTransport)
         # should not be on disk, should only be in memory
-        self.failIfExists('subdir')
+        self.assertPathDoesNotExist('subdir')
 
 
 class TestChrootedTest(tests.ChrootedTestCase):
 
     def test_root_is_root(self):
-        t = transport.get_transport(self.get_readonly_url())
+        t = transport.get_transport_from_url(self.get_readonly_url())
         url = t.base
         self.assertEqual(url, t.clone('..').base)
 
@@ -697,8 +714,8 @@
 class TestProfileResult(tests.TestCase):
 
     def test_profiles_tests(self):
-        self.requireFeature(test_lsprof.LSProfFeature)
-        terminal = testtools.tests.helpers.ExtendedTestResult()
+        self.requireFeature(features.lsprof_feature)
+        terminal = testtools.testresult.doubles.ExtendedTestResult()
         result = tests.ProfileResult(terminal)
         class Sample(tests.TestCase):
             def a(self):
@@ -721,7 +738,7 @@
                 descriptions=0,
                 verbosity=1,
                 )
-        capture = testtools.tests.helpers.ExtendedTestResult()
+        capture = testtools.testresult.doubles.ExtendedTestResult()
         test_case.run(MultiTestResult(result, capture))
         run_case = capture._events[0][1]
         timed_string = result._testTimeString(run_case)
@@ -748,46 +765,6 @@
         self.check_timing(ShortDelayTestCase('test_short_delay'),
                           r"^ +[0-9]+ms$")
 
-    def _patch_get_bzr_source_tree(self):
-        # Reading from the actual source tree breaks isolation, but we don't
-        # want to assume that thats *all* that would happen.
-        self.overrideAttr(bzrlib.version, '_get_bzr_source_tree', lambda: None)
-
-    def test_assigned_benchmark_file_stores_date(self):
-        self._patch_get_bzr_source_tree()
-        output = StringIO()
-        result = bzrlib.tests.TextTestResult(self._log_file,
-                                        descriptions=0,
-                                        verbosity=1,
-                                        bench_history=output
-                                        )
-        output_string = output.getvalue()
-        # if you are wondering about the regexp please read the comment in
-        # test_bench_history (bzrlib.tests.test_selftest.TestRunner)
-        # XXX: what comment?  -- Andrew Bennetts
-        self.assertContainsRe(output_string, "--date [0-9.]+")
-
-    def test_benchhistory_records_test_times(self):
-        self._patch_get_bzr_source_tree()
-        result_stream = StringIO()
-        result = bzrlib.tests.TextTestResult(
-            self._log_file,
-            descriptions=0,
-            verbosity=1,
-            bench_history=result_stream
-            )
-
-        # we want profile a call and check that its test duration is recorded
-        # make a new test instance that when run will generate a benchmark
-        example_test_case = TestTestResult("_time_hello_world_encoding")
-        # execute the test, which should succeed and record times
-        example_test_case.run(result)
-        lines = result_stream.getvalue().splitlines()
-        self.assertEqual(2, len(lines))
-        self.assertContainsRe(lines[1],
-            " *[0-9]+ms bzrlib.tests.test_selftest.TestTestResult"
-            "._time_hello_world_encoding")
-
     def _time_hello_world_encoding(self):
         """Profile two sleep calls
 
@@ -798,10 +775,10 @@
 
     def test_lsprofiling(self):
         """Verbose test result prints lsprof statistics from test cases."""
-        self.requireFeature(test_lsprof.LSProfFeature)
+        self.requireFeature(features.lsprof_feature)
         result_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(result_stream),
+            result_stream,
             descriptions=0,
             verbosity=2,
             )
@@ -833,18 +810,32 @@
         self.assertContainsRe(output,
             r"LSProf output for <type 'unicode'>\(\('world',\), {'errors': 'replace'}\)\n")
 
+    def test_uses_time_from_testtools(self):
+        """Test case timings in verbose results should use testtools times"""
+        import datetime
+        class TimeAddedVerboseTestResult(tests.VerboseTestResult):
+            def startTest(self, test):
+                self.time(datetime.datetime.utcfromtimestamp(1.145))
+                super(TimeAddedVerboseTestResult, self).startTest(test)
+            def addSuccess(self, test):
+                self.time(datetime.datetime.utcfromtimestamp(51.147))
+                super(TimeAddedVerboseTestResult, self).addSuccess(test)
+            def report_tests_starting(self): pass
+        sio = StringIO()
+        self.get_passing_test().run(TimeAddedVerboseTestResult(sio, 0, 2))
+        self.assertEndsWith(sio.getvalue(), "OK    50002ms\n")
+
     def test_known_failure(self):
-        """A KnownFailure being raised should trigger several result actions."""
+        """Using knownFailure should trigger several result actions."""
         class InstrumentedTestResult(tests.ExtendedTestResult):
             def stopTestRun(self): pass
-            def startTests(self): pass
-            def report_test_start(self, test): pass
+            def report_tests_starting(self): pass
             def report_known_failure(self, test, err=None, details=None):
                 self._call = test, 'known failure'
         result = InstrumentedTestResult(None, None, None, None)
         class Test(tests.TestCase):
             def test_function(self):
-                raise tests.KnownFailure('failed!')
+                self.knownFailure('failed!')
         test = Test("test_function")
         test.run(result)
         # it should invoke 'report_known_failure'.
@@ -862,24 +853,17 @@
         # verbose test output formatting
         result_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(result_stream),
+            result_stream,
             descriptions=0,
             verbosity=2,
             )
-        test = self.get_passing_test()
-        result.startTest(test)
-        prefix = len(result_stream.getvalue())
-        # the err parameter has the shape:
-        # (class, exception object, traceback)
-        # KnownFailures dont get their tracebacks shown though, so we
-        # can skip that.
-        err = (tests.KnownFailure, tests.KnownFailure('foo'), None)
-        result.report_known_failure(test, err)
-        output = result_stream.getvalue()[prefix:]
-        lines = output.splitlines()
-        self.assertContainsRe(lines[0], r'XFAIL *\d+ms$')
-        self.assertEqual(lines[1], '    foo')
-        self.assertEqual(2, len(lines))
+        _get_test("test_xfail").run(result)
+        self.assertContainsRe(result_stream.getvalue(),
+            "\n\\S+\\.test_xfail\\s+XFAIL\\s+\\d+ms\n"
+            "\\s*(?:Text attachment: )?reason"
+            "(?:\n-+\n|: {{{)"
+            "this_fails"
+            "(?:\n-+\n|}}}\n)")
 
     def get_passing_test(self):
         """Return a test object that can't be run usefully."""
@@ -891,13 +875,12 @@
         """Test the behaviour of invoking addNotSupported."""
         class InstrumentedTestResult(tests.ExtendedTestResult):
             def stopTestRun(self): pass
-            def startTests(self): pass
-            def report_test_start(self, test): pass
+            def report_tests_starting(self): pass
             def report_unsupported(self, test, feature):
                 self._call = test, feature
         result = InstrumentedTestResult(None, None, None, None)
         test = SampleTestCase('_test_pass')
-        feature = tests.Feature()
+        feature = features.Feature()
         result.startTest(test)
         result.addNotSupported(test, feature)
         # it should invoke 'report_unsupported'.
@@ -917,12 +900,12 @@
         # verbose test output formatting
         result_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(result_stream),
+            result_stream,
             descriptions=0,
             verbosity=2,
             )
         test = self.get_passing_test()
-        feature = tests.Feature()
+        feature = features.Feature()
         result.startTest(test)
         prefix = len(result_stream.getvalue())
         result.report_unsupported(test, feature)
@@ -937,12 +920,11 @@
         """An UnavailableFeature being raised should invoke addNotSupported."""
         class InstrumentedTestResult(tests.ExtendedTestResult):
             def stopTestRun(self): pass
-            def startTests(self): pass
-            def report_test_start(self, test): pass
+            def report_tests_starting(self): pass
             def addNotSupported(self, test, feature):
                 self._call = test, feature
         result = InstrumentedTestResult(None, None, None, None)
-        feature = tests.Feature()
+        feature = features.Feature()
         class Test(tests.TestCase):
             def test_function(self):
                 raise tests.UnavailableFeature(feature)
@@ -967,9 +949,8 @@
     def test_strict_with_known_failure(self):
         result = bzrlib.tests.TextTestResult(self._log_file, descriptions=0,
                                              verbosity=1)
-        test = self.get_passing_test()
-        err = (tests.KnownFailure, tests.KnownFailure('foo'), None)
-        result.addExpectedFailure(test, err)
+        test = _get_test("test_xfail")
+        test.run(result)
         self.assertFalse(result.wasStrictlySuccessful())
         self.assertEqual(None, result._extractBenchmarkTime(test))
 
@@ -986,7 +967,6 @@
         class InstrumentedTestResult(tests.ExtendedTestResult):
             calls = 0
             def startTests(self): self.calls += 1
-            def report_test_start(self, test): pass
         result = InstrumentedTestResult(None, None, None, None)
         def test_function():
             pass
@@ -994,14 +974,18 @@
         test.run(result)
         self.assertEquals(1, result.calls)
 
-
-class TestUnicodeFilenameFeature(tests.TestCase):
-
-    def test_probe_passes(self):
-        """UnicodeFilenameFeature._probe passes."""
-        # We can't test much more than that because the behaviour depends
-        # on the platform.
-        tests.UnicodeFilenameFeature._probe()
+    def test_startTests_only_once(self):
+        """With multiple tests startTests should still only be called once"""
+        class InstrumentedTestResult(tests.ExtendedTestResult):
+            calls = 0
+            def startTests(self): self.calls += 1
+        result = InstrumentedTestResult(None, None, None, None)
+        suite = unittest.TestSuite([
+            unittest.FunctionTestCase(lambda: None),
+            unittest.FunctionTestCase(lambda: None)])
+        suite.run(result)
+        self.assertEquals(1, result.calls)
+        self.assertEquals(2, result.count)
 
 
 class TestRunner(tests.TestCase):
@@ -1020,14 +1004,11 @@
         because of our use of global state.
         """
         old_root = tests.TestCaseInTempDir.TEST_ROOT
-        old_leak = tests.TestCase._first_thread_leaker_id
         try:
             tests.TestCaseInTempDir.TEST_ROOT = None
-            tests.TestCase._first_thread_leaker_id = None
             return testrunner.run(test)
         finally:
             tests.TestCaseInTempDir.TEST_ROOT = old_root
-            tests.TestCase._first_thread_leaker_id = old_leak
 
     def test_known_failure_failed_run(self):
         # run a test that generates a known failure which should be printed in
@@ -1038,7 +1019,7 @@
         test = unittest.TestSuite()
         test.addTest(Test("known_failure_test"))
         def failing_test():
-            self.fail('foo')
+            raise AssertionError('foo')
         test.addTest(unittest.FunctionTestCase(failing_test))
         stream = StringIO()
         runner = tests.TextTestRunner(stream=stream)
@@ -1052,7 +1033,7 @@
             '^----------------------------------------------------------------------\n'
             'Traceback \\(most recent call last\\):\n'
             '  .*' # File .*, line .*, in failing_test' - but maybe not from .pyc
-            '    self.fail\\(\'foo\'\\)\n'
+            '    raise AssertionError\\(\'foo\'\\)\n'
             '.*'
             '^----------------------------------------------------------------------\n'
             '.*'
@@ -1064,7 +1045,7 @@
         # the final output.
         class Test(tests.TestCase):
             def known_failure_test(self):
-                self.expectFailure('failed', self.assertTrue, False)
+                self.knownFailure("Never works...")
         test = Test("known_failure_test")
         stream = StringIO()
         runner = tests.TextTestRunner(stream=stream)
@@ -1076,12 +1057,44 @@
             '\n'
             'OK \\(known_failures=1\\)\n')
 
+    def test_unexpected_success_bad(self):
+        class Test(tests.TestCase):
+            def test_truth(self):
+                self.expectFailure("No absolute truth", self.assertTrue, True)
+        runner = tests.TextTestRunner(stream=StringIO())
+        result = self.run_test_runner(runner, Test("test_truth"))
+        if testtools_version[:3] <= (0, 9, 11):
+            self.assertContainsRe(runner.stream.getvalue(),
+                "=+\n"
+                "FAIL: \\S+\.test_truth\n"
+                "-+\n"
+                "(?:.*\n)*"
+                "No absolute truth\n"
+                "(?:.*\n)*"
+                "-+\n"
+                "Ran 1 test in .*\n"
+                "\n"
+                "FAILED \\(failures=1\\)\n\\Z")
+        else:
+            self.assertContainsRe(runner.stream.getvalue(),
+                "=+\n"
+                "FAIL: \\S+\.test_truth\n"
+                "-+\n"
+                "Empty attachments:\n"
+                "  log\n"
+                "\n"
+                "reason: {{{No absolute truth}}}\n"
+                "-+\n"
+                "Ran 1 test in .*\n"
+                "\n"
+                "FAILED \\(failures=1\\)\n\\Z")
+
     def test_result_decorator(self):
         # decorate results
         calls = []
-        class LoggingDecorator(tests.ForwardingResult):
+        class LoggingDecorator(ExtendedToOriginalDecorator):
             def startTest(self, test):
-                tests.ForwardingResult.startTest(self, test)
+                ExtendedToOriginalDecorator.startTest(self, test)
                 calls.append('start')
         test = unittest.FunctionTestCase(lambda:None)
         stream = StringIO()
@@ -1165,9 +1178,9 @@
 
     def test_unsupported_features_listed(self):
         """When unsupported features are encountered they are detailed."""
-        class Feature1(tests.Feature):
+        class Feature1(features.Feature):
             def _probe(self): return False
-        class Feature2(tests.Feature):
+        class Feature2(features.Feature):
             def _probe(self): return False
         # create sample tests
         test1 = SampleTestCase('_test_pass')
@@ -1188,35 +1201,24 @@
             ],
             lines[-3:])
 
-    def _patch_get_bzr_source_tree(self):
-        # Reading from the actual source tree breaks isolation, but we don't
-        # want to assume that thats *all* that would happen.
-        self._get_source_tree_calls = []
-        def new_get():
-            self._get_source_tree_calls.append("called")
-            return None
-        self.overrideAttr(bzrlib.version, '_get_bzr_source_tree',  new_get)
-
-    def test_bench_history(self):
-        # tests that the running the benchmark passes bench_history into
-        # the test result object. We can tell that happens if
-        # _get_bzr_source_tree is called.
-        self._patch_get_bzr_source_tree()
-        test = TestRunner('dummy_test')
-        output = StringIO()
-        runner = tests.TextTestRunner(stream=self._log_file,
-                                      bench_history=output)
-        result = self.run_test_runner(runner, test)
-        output_string = output.getvalue()
-        self.assertContainsRe(output_string, "--date [0-9.]+")
-        self.assertLength(1, self._get_source_tree_calls)
+    def test_verbose_test_count(self):
+        """A verbose test run reports the right test count at the start"""
+        suite = TestUtil.TestSuite([
+            unittest.FunctionTestCase(lambda:None),
+            unittest.FunctionTestCase(lambda:None)])
+        self.assertEqual(suite.countTestCases(), 2)
+        stream = StringIO()
+        runner = tests.TextTestRunner(stream=stream, verbosity=2)
+        # Need to use the CountingDecorator as that's what sets num_tests
+        result = self.run_test_runner(runner, tests.CountingDecorator(suite))
+        self.assertStartsWith(stream.getvalue(), "running 2 tests")
 
     def test_startTestRun(self):
         """run should call result.startTestRun()"""
         calls = []
-        class LoggingDecorator(tests.ForwardingResult):
+        class LoggingDecorator(ExtendedToOriginalDecorator):
             def startTestRun(self):
-                tests.ForwardingResult.startTestRun(self)
+                ExtendedToOriginalDecorator.startTestRun(self)
                 calls.append('startTestRun')
         test = unittest.FunctionTestCase(lambda:None)
         stream = StringIO()
@@ -1228,9 +1230,9 @@
     def test_stopTestRun(self):
         """run should call result.stopTestRun()"""
         calls = []
-        class LoggingDecorator(tests.ForwardingResult):
+        class LoggingDecorator(ExtendedToOriginalDecorator):
             def stopTestRun(self):
-                tests.ForwardingResult.stopTestRun(self)
+                ExtendedToOriginalDecorator.stopTestRun(self)
                 calls.append('stopTestRun')
         test = unittest.FunctionTestCase(lambda:None)
         stream = StringIO()
@@ -1239,6 +1241,26 @@
         result = self.run_test_runner(runner, test)
         self.assertLength(1, calls)
 
+    def test_unicode_test_output_on_ascii_stream(self):
+        """Showing results should always succeed even on an ascii console"""
+        class FailureWithUnicode(tests.TestCase):
+            def test_log_unicode(self):
+                self.log(u"\u2606")
+                self.fail("Now print that log!")
+        out = StringIO()
+        self.overrideAttr(osutils, "get_terminal_encoding",
+            lambda trace=False: "ascii")
+        result = self.run_test_runner(tests.TextTestRunner(stream=out),
+            FailureWithUnicode("test_log_unicode"))
+        if testtools_version[:3] > (0, 9, 11):
+            self.assertContainsRe(out.getvalue(), "log: {{{\d+\.\d+  \\\\u2606}}}")
+        else:
+            self.assertContainsRe(out.getvalue(),
+                "Text attachment: log\n"
+                "-+\n"
+                "\d+\.\d+  \\\\u2606\n"
+                "-+\n")
+
 
 class SampleTestCase(tests.TestCase):
 
@@ -1419,7 +1441,7 @@
         sample_test = TestTestCase("method_that_times_a_bit_twice")
         output_stream = StringIO()
         result = bzrlib.tests.VerboseTestResult(
-            unittest._WritelnDecorator(output_stream),
+            output_stream,
             descriptions=0,
             verbosity=2)
         sample_test.run(result)
@@ -1432,19 +1454,19 @@
         # Note this test won't fail with hooks that the core library doesn't
         # use - but it trigger with a plugin that adds hooks, so its still a
         # useful warning in that case.
-        self.assertEqual(bzrlib.branch.BranchHooks(),
-            bzrlib.branch.Branch.hooks)
-        self.assertEqual(bzrlib.smart.server.SmartServerHooks(),
+        self.assertEqual(bzrlib.branch.BranchHooks(), bzrlib.branch.Branch.hooks)
+        self.assertEqual(
+            bzrlib.smart.server.SmartServerHooks(),
             bzrlib.smart.server.SmartTCPServer.hooks)
-        self.assertEqual(bzrlib.commands.CommandHooks(),
-            bzrlib.commands.Command.hooks)
+        self.assertEqual(
+            bzrlib.commands.CommandHooks(), bzrlib.commands.Command.hooks)
 
     def test__gather_lsprof_in_benchmarks(self):
         """When _gather_lsprof_in_benchmarks is on, accumulate profile data.
 
         Each self.time() call is individually and separately profiled.
         """
-        self.requireFeature(test_lsprof.LSProfFeature)
+        self.requireFeature(features.lsprof_feature)
         # overrides the class member with an instance member so no cleanup
         # needed.
         self._gather_lsprof_in_benchmarks = True
@@ -1469,7 +1491,7 @@
         transport_server = memory.MemoryServer()
         transport_server.start_server()
         self.addCleanup(transport_server.stop_server)
-        t = transport.get_transport(transport_server.get_url())
+        t = transport.get_transport_from_url(transport_server.get_url())
         bzrdir.BzrDir.create(t.base)
         self.assertRaises(errors.BzrError,
             bzrdir.BzrDir.open_from_transport, t)
@@ -1480,14 +1502,14 @@
 
     def test_requireFeature_available(self):
         """self.requireFeature(available) is a no-op."""
-        class Available(tests.Feature):
+        class Available(features.Feature):
             def _probe(self):return True
         feature = Available()
         self.requireFeature(feature)
 
     def test_requireFeature_unavailable(self):
         """self.requireFeature(unavailable) raises UnavailableFeature."""
-        class Unavailable(tests.Feature):
+        class Unavailable(features.Feature):
             def _probe(self):return False
         feature = Unavailable()
         self.assertRaises(tests.UnavailableFeature,
@@ -1652,6 +1674,156 @@
         test.run(unittest.TestResult())
         self.assertEqual('original', obj.test_attr)
 
+    def test_recordCalls(self):
+        from bzrlib.tests import test_selftest
+        calls = self.recordCalls(
+            test_selftest, '_add_numbers')
+        self.assertEqual(test_selftest._add_numbers(2, 10),
+            12)
+        self.assertEquals(calls, [((2, 10), {})])
+
+
+def _add_numbers(a, b):
+    return a + b
+
+
+class _MissingFeature(features.Feature):
+    def _probe(self):
+        return False
+missing_feature = _MissingFeature()
+
+
+def _get_test(name):
+    """Get an instance of a specific example test.
+
+    We protect this in a function so that they don't auto-run in the test
+    suite.
+    """
+
+    class ExampleTests(tests.TestCase):
+
+        def test_fail(self):
+            mutter('this was a failing test')
+            self.fail('this test will fail')
+
+        def test_error(self):
+            mutter('this test errored')
+            raise RuntimeError('gotcha')
+
+        def test_missing_feature(self):
+            mutter('missing the feature')
+            self.requireFeature(missing_feature)
+
+        def test_skip(self):
+            mutter('this test will be skipped')
+            raise tests.TestSkipped('reason')
+
+        def test_success(self):
+            mutter('this test succeeds')
+
+        def test_xfail(self):
+            mutter('test with expected failure')
+            self.knownFailure('this_fails')
+
+        def test_unexpected_success(self):
+            mutter('test with unexpected success')
+            self.expectFailure('should_fail', lambda: None)
+
+    return ExampleTests(name)
+
+
+class TestTestCaseLogDetails(tests.TestCase):
+
+    def _run_test(self, test_name):
+        test = _get_test(test_name)
+        result = testtools.TestResult()
+        test.run(result)
+        return result
+
+    def test_fail_has_log(self):
+        result = self._run_test('test_fail')
+        self.assertEqual(1, len(result.failures))
+        result_content = result.failures[0][1]
+        if testtools_version < (0, 9, 12):
+            self.assertContainsRe(result_content, 'Text attachment: log')
+        self.assertContainsRe(result_content, 'this was a failing test')
+
+    def test_error_has_log(self):
+        result = self._run_test('test_error')
+        self.assertEqual(1, len(result.errors))
+        result_content = result.errors[0][1]
+        if testtools_version < (0, 9, 12):
+            self.assertContainsRe(result_content, 'Text attachment: log')
+        self.assertContainsRe(result_content, 'this test errored')
+
+    def test_skip_has_no_log(self):
+        result = self._run_test('test_skip')
+        self.assertEqual(['reason'], result.skip_reasons.keys())
+        skips = result.skip_reasons['reason']
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        self.assertFalse('log' in test.getDetails())
+
+    def test_missing_feature_has_no_log(self):
+        # testtools doesn't know about addNotSupported, so it just gets
+        # considered as a skip
+        result = self._run_test('test_missing_feature')
+        self.assertEqual([missing_feature], result.skip_reasons.keys())
+        skips = result.skip_reasons[missing_feature]
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        self.assertFalse('log' in test.getDetails())
+
+    def test_xfail_has_no_log(self):
+        result = self._run_test('test_xfail')
+        self.assertEqual(1, len(result.expectedFailures))
+        result_content = result.expectedFailures[0][1]
+        self.assertNotContainsRe(result_content, 'Text attachment: log')
+        self.assertNotContainsRe(result_content, 'test with expected failure')
+
+    def test_unexpected_success_has_log(self):
+        result = self._run_test('test_unexpected_success')
+        self.assertEqual(1, len(result.unexpectedSuccesses))
+        # Inconsistency, unexpectedSuccesses is a list of tests,
+        # expectedFailures is a list of reasons?
+        test = result.unexpectedSuccesses[0]
+        details = test.getDetails()
+        self.assertTrue('log' in details)
+
+
+class TestTestCloning(tests.TestCase):
+    """Tests that test cloning of TestCases (as used by multiply_tests)."""
+
+    def test_cloned_testcase_does_not_share_details(self):
+        """A TestCase cloned with clone_test does not share mutable attributes
+        such as details or cleanups.
+        """
+        class Test(tests.TestCase):
+            def test_foo(self):
+                self.addDetail('foo', Content('text/plain', lambda: 'foo'))
+        orig_test = Test('test_foo')
+        cloned_test = tests.clone_test(orig_test, orig_test.id() + '(cloned)')
+        orig_test.run(unittest.TestResult())
+        self.assertEqual('foo', orig_test.getDetails()['foo'].iter_bytes())
+        self.assertEqual(None, cloned_test.getDetails().get('foo'))
+
+    def test_double_apply_scenario_preserves_first_scenario(self):
+        """Applying two levels of scenarios to a test preserves the attributes
+        added by both scenarios.
+        """
+        class Test(tests.TestCase):
+            def test_foo(self):
+                pass
+        test = Test('test_foo')
+        scenarios_x = [('x=1', {'x': 1}), ('x=2', {'x': 2})]
+        scenarios_y = [('y=1', {'y': 1}), ('y=2', {'y': 2})]
+        suite = tests.multiply_tests(test, scenarios_x, unittest.TestSuite())
+        suite = tests.multiply_tests(suite, scenarios_y, unittest.TestSuite())
+        all_tests = list(tests.iter_suite_tests(suite))
+        self.assertLength(4, all_tests)
+        all_xys = sorted((t.x, t.y) for t in all_tests)
+        self.assertEqual([(1, 1), (1, 2), (2, 1), (2, 2)], all_xys)
+
 
 # NB: Don't delete this; it's not actually from 0.11!
 @deprecated_function(deprecated_in((0, 11, 0)))
@@ -1785,11 +1957,8 @@
     def test_make_branch_and_tree_with_format(self):
         # we should be able to supply a format to make_branch_and_tree
         self.make_branch_and_tree('a', format=bzrlib.bzrdir.BzrDirMetaFormat1())
-        self.make_branch_and_tree('b', format=bzrlib.bzrdir.BzrDirFormat6())
         self.assertIsInstance(bzrlib.bzrdir.BzrDir.open('a')._format,
                               bzrlib.bzrdir.BzrDirMetaFormat1)
-        self.assertIsInstance(bzrlib.bzrdir.BzrDir.open('b')._format,
-                              bzrlib.bzrdir.BzrDirFormat6)
 
     def test_make_branch_and_memory_tree(self):
         # we should be able to get a new branch and a mutable tree from
@@ -1812,7 +1981,7 @@
                 tree.branch.repository.bzrdir.root_transport)
 
 
-class SelfTestHelper:
+class SelfTestHelper(object):
 
     def run_selftest(self, **kwargs):
         """Run selftest returning its output."""
@@ -1872,18 +2041,18 @@
         self.assertLength(2, output.readlines())
 
     def test_lsprof_tests(self):
-        self.requireFeature(test_lsprof.LSProfFeature)
-        calls = []
+        self.requireFeature(features.lsprof_feature)
+        results = []
         class Test(object):
             def __call__(test, result):
                 test.run(result)
             def run(test, result):
-                self.assertIsInstance(result, tests.ForwardingResult)
-                calls.append("called")
+                results.append(result)
             def countTestCases(self):
                 return 1
         self.run_selftest(test_suite_factory=Test, lsprof_tests=True)
-        self.assertLength(1, calls)
+        self.assertLength(1, results)
+        self.assertIsInstance(results.pop(), ExtendedToOriginalDecorator)
 
     def test_random(self):
         # test randomising by listing a number of tests.
@@ -1969,6 +2138,87 @@
             load_list='missing file name', list_only=True)
 
 
+class TestSubunitLogDetails(tests.TestCase, SelfTestHelper):
+
+    _test_needs_features = [features.subunit]
+
+    def run_subunit_stream(self, test_name):
+        from subunit import ProtocolTestCase
+        def factory():
+            return TestUtil.TestSuite([_get_test(test_name)])
+        stream = self.run_selftest(runner_class=tests.SubUnitBzrRunner,
+            test_suite_factory=factory)
+        test = ProtocolTestCase(stream)
+        result = testtools.TestResult()
+        test.run(result)
+        content = stream.getvalue()
+        return content, result
+
+    def test_fail_has_log(self):
+        content, result = self.run_subunit_stream('test_fail')
+        self.assertEqual(1, len(result.failures))
+        self.assertContainsRe(content, '(?m)^log$')
+        self.assertContainsRe(content, 'this test will fail')
+
+    def test_error_has_log(self):
+        content, result = self.run_subunit_stream('test_error')
+        self.assertContainsRe(content, '(?m)^log$')
+        self.assertContainsRe(content, 'this test errored')
+
+    def test_skip_has_no_log(self):
+        content, result = self.run_subunit_stream('test_skip')
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'this test will be skipped')
+        self.assertEqual(['reason'], result.skip_reasons.keys())
+        skips = result.skip_reasons['reason']
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        # RemotedTestCase doesn't preserve the "details"
+        ## self.assertFalse('log' in test.getDetails())
+
+    def test_missing_feature_has_no_log(self):
+        content, result = self.run_subunit_stream('test_missing_feature')
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'missing the feature')
+        self.assertEqual(['_MissingFeature\n'], result.skip_reasons.keys())
+        skips = result.skip_reasons['_MissingFeature\n']
+        self.assertEqual(1, len(skips))
+        test = skips[0]
+        # RemotedTestCase doesn't preserve the "details"
+        ## self.assertFalse('log' in test.getDetails())
+
+    def test_xfail_has_no_log(self):
+        content, result = self.run_subunit_stream('test_xfail')
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'test with expected failure')
+        self.assertEqual(1, len(result.expectedFailures))
+        result_content = result.expectedFailures[0][1]
+        self.assertNotContainsRe(result_content, 'Text attachment: log')
+        self.assertNotContainsRe(result_content, 'test with expected failure')
+
+    def test_unexpected_success_has_log(self):
+        content, result = self.run_subunit_stream('test_unexpected_success')
+        self.assertContainsRe(content, '(?m)^log$')
+        self.assertContainsRe(content, 'test with unexpected success')
+        # GZ 2011-05-18: Old versions of subunit treat unexpected success as a
+        #                success, if a min version check is added remove this
+        from subunit import TestProtocolClient as _Client
+        if _Client.addUnexpectedSuccess.im_func is _Client.addSuccess.im_func:
+            self.expectFailure('subunit treats "unexpectedSuccess"'
+                               ' as a plain success',
+                self.assertEqual, 1, len(result.unexpectedSuccesses))
+        self.assertEqual(1, len(result.unexpectedSuccesses))
+        test = result.unexpectedSuccesses[0]
+        # RemotedTestCase doesn't preserve the "details"
+        ## self.assertTrue('log' in test.getDetails())
+
+    def test_success_has_no_log(self):
+        content, result = self.run_subunit_stream('test_success')
+        self.assertEqual(1, result.testsRun)
+        self.assertNotContainsRe(content, '(?m)^log$')
+        self.assertNotContainsRe(content, 'this test succeeds')
+
+
 class TestRunBzr(tests.TestCase):
 
     out = ''
@@ -2097,7 +2347,7 @@
         # stdout and stderr of the invoked run_bzr
         current_factory = bzrlib.ui.ui_factory
         self.run_bzr(['foo'])
-        self.failIf(current_factory is self.factory)
+        self.assertFalse(current_factory is self.factory)
         self.assertNotEqual(sys.stdout, self.factory.stdout)
         self.assertNotEqual(sys.stderr, self.factory.stderr)
         self.assertEqual('foo\n', self.factory.stdout.getvalue())
@@ -2260,17 +2510,24 @@
 
 
 class TestStartBzrSubProcess(tests.TestCase):
+    """Stub test start_bzr_subprocess."""
 
-    def check_popen_state(self):
-        """Replace to make assertions when popen is called."""
+    def _subprocess_log_cleanup(self):
+        """Inhibits the base version as we don't produce a log file."""
 
     def _popen(self, *args, **kwargs):
-        """Record the command that is run, so that we can ensure it is correct"""
+        """Override the base version to record the command that is run.
+
+        From there we can ensure it is correct without spawning a real process.
+        """
         self.check_popen_state()
         self._popen_args = args
         self._popen_kwargs = kwargs
         raise _DontSpawnProcess()
 
+    def check_popen_state(self):
+        """Replace to make assertions when popen is called."""
+
     def test_run_bzr_subprocess_no_plugins(self):
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [])
         command = self._popen_args[0]
@@ -2280,41 +2537,41 @@
 
     def test_allow_plugins(self):
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            allow_plugins=True)
+                          allow_plugins=True)
         command = self._popen_args[0]
         self.assertEqual([], command[2:])
 
     def test_set_env(self):
-        self.failIf('EXISTANT_ENV_VAR' in os.environ)
+        self.assertFalse('EXISTANT_ENV_VAR' in os.environ)
         # set in the child
         def check_environment():
             self.assertEqual('set variable', os.environ['EXISTANT_ENV_VAR'])
         self.check_popen_state = check_environment
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            env_changes={'EXISTANT_ENV_VAR':'set variable'})
+                          env_changes={'EXISTANT_ENV_VAR':'set variable'})
         # not set in theparent
         self.assertFalse('EXISTANT_ENV_VAR' in os.environ)
 
     def test_run_bzr_subprocess_env_del(self):
         """run_bzr_subprocess can remove environment variables too."""
-        self.failIf('EXISTANT_ENV_VAR' in os.environ)
+        self.assertFalse('EXISTANT_ENV_VAR' in os.environ)
         def check_environment():
             self.assertFalse('EXISTANT_ENV_VAR' in os.environ)
         os.environ['EXISTANT_ENV_VAR'] = 'set variable'
         self.check_popen_state = check_environment
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            env_changes={'EXISTANT_ENV_VAR':None})
+                          env_changes={'EXISTANT_ENV_VAR':None})
         # Still set in parent
         self.assertEqual('set variable', os.environ['EXISTANT_ENV_VAR'])
         del os.environ['EXISTANT_ENV_VAR']
 
     def test_env_del_missing(self):
-        self.failIf('NON_EXISTANT_ENV_VAR' in os.environ)
+        self.assertFalse('NON_EXISTANT_ENV_VAR' in os.environ)
         def check_environment():
             self.assertFalse('NON_EXISTANT_ENV_VAR' in os.environ)
         self.check_popen_state = check_environment
         self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-            env_changes={'NON_EXISTANT_ENV_VAR':None})
+                          env_changes={'NON_EXISTANT_ENV_VAR':None})
 
     def test_working_dir(self):
         """Test that we can specify the working dir for the child"""
@@ -2323,18 +2580,12 @@
         chdirs = []
         def chdir(path):
             chdirs.append(path)
-        os.chdir = chdir
-        try:
-            def getcwd():
-                return 'current'
-            osutils.getcwd = getcwd
-            try:
-                self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
-                    working_dir='foo')
-            finally:
-                osutils.getcwd = orig_getcwd
-        finally:
-            os.chdir = orig_chdir
+        self.overrideAttr(os, 'chdir', chdir)
+        def getcwd():
+            return 'current'
+        self.overrideAttr(osutils, 'getcwd', getcwd)
+        self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
+                          working_dir='foo')
         self.assertEqual(['foo', 'current'], chdirs)
 
     def test_get_bzr_path_with_cwd_bzrlib(self):
@@ -2360,80 +2611,6 @@
         self.assertEqual('bzr: interrupted\n', result[1])
 
 
-class TestFeature(tests.TestCase):
-
-    def test_caching(self):
-        """Feature._probe is called by the feature at most once."""
-        class InstrumentedFeature(tests.Feature):
-            def __init__(self):
-                super(InstrumentedFeature, self).__init__()
-                self.calls = []
-            def _probe(self):
-                self.calls.append('_probe')
-                return False
-        feature = InstrumentedFeature()
-        feature.available()
-        self.assertEqual(['_probe'], feature.calls)
-        feature.available()
-        self.assertEqual(['_probe'], feature.calls)
-
-    def test_named_str(self):
-        """Feature.__str__ should thunk to feature_name()."""
-        class NamedFeature(tests.Feature):
-            def feature_name(self):
-                return 'symlinks'
-        feature = NamedFeature()
-        self.assertEqual('symlinks', str(feature))
-
-    def test_default_str(self):
-        """Feature.__str__ should default to __class__.__name__."""
-        class NamedFeature(tests.Feature):
-            pass
-        feature = NamedFeature()
-        self.assertEqual('NamedFeature', str(feature))
-
-
-class TestUnavailableFeature(tests.TestCase):
-
-    def test_access_feature(self):
-        feature = tests.Feature()
-        exception = tests.UnavailableFeature(feature)
-        self.assertIs(feature, exception.args[0])
-
-
-simple_thunk_feature = tests._CompatabilityThunkFeature(
-    deprecated_in((2, 1, 0)),
-    'bzrlib.tests.test_selftest',
-    'simple_thunk_feature','UnicodeFilename',
-    replacement_module='bzrlib.tests'
-    )
-
-class Test_CompatibilityFeature(tests.TestCase):
-
-    def test_does_thunk(self):
-        res = self.callDeprecated(
-            ['bzrlib.tests.test_selftest.simple_thunk_feature was deprecated'
-             ' in version 2.1.0. Use bzrlib.tests.UnicodeFilename instead.'],
-            simple_thunk_feature.available)
-        self.assertEqual(tests.UnicodeFilename.available(), res)
-
-
-class TestModuleAvailableFeature(tests.TestCase):
-
-    def test_available_module(self):
-        feature = tests.ModuleAvailableFeature('bzrlib.tests')
-        self.assertEqual('bzrlib.tests', feature.module_name)
-        self.assertEqual('bzrlib.tests', str(feature))
-        self.assertTrue(feature.available())
-        self.assertIs(tests, feature.module)
-
-    def test_unavailable_module(self):
-        feature = tests.ModuleAvailableFeature('bzrlib.no_such_module_exists')
-        self.assertEqual('bzrlib.no_such_module_exists', str(feature))
-        self.assertFalse(feature.available())
-        self.assertIs(None, feature.module)
-
-
 class TestSelftestFiltering(tests.TestCase):
 
     def setUp(self):
@@ -2590,16 +2767,16 @@
         self.assertEqual(remaining_names, _test_ids(split_suite[1]))
 
 
-class TestCheckInventoryShape(tests.TestCaseWithTransport):
+class TestCheckTreeShape(tests.TestCaseWithTransport):
 
-    def test_check_inventory_shape(self):
+    def test_check_tree_shape(self):
         files = ['a', 'b/', 'b/c']
         tree = self.make_branch_and_tree('.')
         self.build_tree(files)
         tree.add(files)
         tree.lock_read()
         try:
-            self.check_inventory_shape(tree.inventory, files)
+            self.check_tree_shape(tree, files)
         finally:
             tree.unlock()
 
@@ -2937,7 +3114,8 @@
         tpr.register('bar', 'bBB.aAA.rRR')
         self.assertEquals('bbb.aaa.rrr', tpr.get('bar'))
         self.assertThat(self.get_log(),
-            DocTestMatches("...bar...bbb.aaa.rrr...BB.aAA.rRR", ELLIPSIS))
+            DocTestMatches("...bar...bbb.aaa.rrr...BB.aAA.rRR",
+                           doctest.ELLIPSIS))
 
     def test_get_unknown_prefix(self):
         tpr = self._get_registry()
@@ -2963,6 +3141,162 @@
         self.assertEquals('bzrlib.plugins', tpr.resolve_alias('bp'))
 
 
+class TestThreadLeakDetection(tests.TestCase):
+    """Ensure when tests leak threads we detect and report it"""
+
+    class LeakRecordingResult(tests.ExtendedTestResult):
+        def __init__(self):
+            tests.ExtendedTestResult.__init__(self, StringIO(), 0, 1)
+            self.leaks = []
+        def _report_thread_leak(self, test, leaks, alive):
+            self.leaks.append((test, leaks))
+
+    def test_testcase_without_addCleanups(self):
+        """Check old TestCase instances don't break with leak detection"""
+        class Test(unittest.TestCase):
+            def runTest(self):
+                pass
+        result = self.LeakRecordingResult()
+        test = Test()
+        result.startTestRun()
+        test.run(result)
+        result.stopTestRun()
+        self.assertEqual(result._tests_leaking_threads_count, 0)
+        self.assertEqual(result.leaks, [])
+        
+    def test_thread_leak(self):
+        """Ensure a thread that outlives the running of a test is reported
+
+        Uses a thread that blocks on an event, and is started by the inner
+        test case. As the thread outlives the inner case's run, it should be
+        detected as a leak, but the event is then set so that the thread can
+        be safely joined in cleanup so it's not leaked for real.
+        """
+        event = threading.Event()
+        thread = threading.Thread(name="Leaker", target=event.wait)
+        class Test(tests.TestCase):
+            def test_leak(self):
+                thread.start()
+        result = self.LeakRecordingResult()
+        test = Test("test_leak")
+        self.addCleanup(thread.join)
+        self.addCleanup(event.set)
+        result.startTestRun()
+        test.run(result)
+        result.stopTestRun()
+        self.assertEqual(result._tests_leaking_threads_count, 1)
+        self.assertEqual(result._first_thread_leaker_id, test.id())
+        self.assertEqual(result.leaks, [(test, set([thread]))])
+        self.assertContainsString(result.stream.getvalue(), "leaking threads")
+
+    def test_multiple_leaks(self):
+        """Check multiple leaks are blamed on the test cases at fault
+
+        Same concept as the previous test, but has one inner test method that
+        leaks two threads, and one that doesn't leak at all.
+        """
+        event = threading.Event()
+        thread_a = threading.Thread(name="LeakerA", target=event.wait)
+        thread_b = threading.Thread(name="LeakerB", target=event.wait)
+        thread_c = threading.Thread(name="LeakerC", target=event.wait)
+        class Test(tests.TestCase):
+            def test_first_leak(self):
+                thread_b.start()
+            def test_second_no_leak(self):
+                pass
+            def test_third_leak(self):
+                thread_c.start()
+                thread_a.start()
+        result = self.LeakRecordingResult()
+        first_test = Test("test_first_leak")
+        third_test = Test("test_third_leak")
+        self.addCleanup(thread_a.join)
+        self.addCleanup(thread_b.join)
+        self.addCleanup(thread_c.join)
+        self.addCleanup(event.set)
+        result.startTestRun()
+        unittest.TestSuite(
+            [first_test, Test("test_second_no_leak"), third_test]
+            ).run(result)
+        result.stopTestRun()
+        self.assertEqual(result._tests_leaking_threads_count, 2)
+        self.assertEqual(result._first_thread_leaker_id, first_test.id())
+        self.assertEqual(result.leaks, [
+            (first_test, set([thread_b])),
+            (third_test, set([thread_a, thread_c]))])
+        self.assertContainsString(result.stream.getvalue(), "leaking threads")
+
+
+class TestPostMortemDebugging(tests.TestCase):
+    """Check post mortem debugging works when tests fail or error"""
+
+    class TracebackRecordingResult(tests.ExtendedTestResult):
+        def __init__(self):
+            tests.ExtendedTestResult.__init__(self, StringIO(), 0, 1)
+            self.postcode = None
+        def _post_mortem(self, tb=None):
+            """Record the code object at the end of the current traceback"""
+            tb = tb or sys.exc_info()[2]
+            if tb is not None:
+                next = tb.tb_next
+                while next is not None:
+                    tb = next
+                    next = next.tb_next
+                self.postcode = tb.tb_frame.f_code
+        def report_error(self, test, err):
+            pass
+        def report_failure(self, test, err):
+            pass
+
+    def test_location_unittest_error(self):
+        """Needs right post mortem traceback with erroring unittest case"""
+        class Test(unittest.TestCase):
+            def runTest(self):
+                raise RuntimeError
+        result = self.TracebackRecordingResult()
+        Test().run(result)
+        self.assertEqual(result.postcode, Test.runTest.func_code)
+
+    def test_location_unittest_failure(self):
+        """Needs right post mortem traceback with failing unittest case"""
+        class Test(unittest.TestCase):
+            def runTest(self):
+                raise self.failureException
+        result = self.TracebackRecordingResult()
+        Test().run(result)
+        self.assertEqual(result.postcode, Test.runTest.func_code)
+
+    def test_location_bt_error(self):
+        """Needs right post mortem traceback with erroring bzrlib.tests case"""
+        class Test(tests.TestCase):
+            def test_error(self):
+                raise RuntimeError
+        result = self.TracebackRecordingResult()
+        Test("test_error").run(result)
+        self.assertEqual(result.postcode, Test.test_error.func_code)
+
+    def test_location_bt_failure(self):
+        """Needs right post mortem traceback with failing bzrlib.tests case"""
+        class Test(tests.TestCase):
+            def test_failure(self):
+                raise self.failureException
+        result = self.TracebackRecordingResult()
+        Test("test_failure").run(result)
+        self.assertEqual(result.postcode, Test.test_failure.func_code)
+
+    def test_env_var_triggers_post_mortem(self):
+        """Check pdb.post_mortem is called iff BZR_TEST_PDB is set"""
+        import pdb
+        result = tests.ExtendedTestResult(StringIO(), 0, 1)
+        post_mortem_calls = []
+        self.overrideAttr(pdb, "post_mortem", post_mortem_calls.append)
+        self.overrideEnv('BZR_TEST_PDB', None)
+        result._post_mortem(1)
+        self.overrideEnv('BZR_TEST_PDB', 'on')
+        result._post_mortem(2)
+        self.assertEqual([2], post_mortem_calls)
+
+
 class TestRunSuite(tests.TestCase):
 
     def test_runner_class(self):
@@ -2979,3 +3313,329 @@
                                                 self.verbosity)
         tests.run_suite(suite, runner_class=MyRunner, stream=StringIO())
         self.assertLength(1, calls)
+
+
+class TestUncollectedWarnings(tests.TestCase):
+    """Check a test case still alive after being run emits a warning"""
+
+    class Test(tests.TestCase):
+        def test_pass(self):
+            pass
+        def test_self_ref(self):
+            self.also_self = self.test_self_ref
+        def test_skip(self):
+            self.skip("Don't need")
+
+    def _get_suite(self):
+        return TestUtil.TestSuite([
+            self.Test("test_pass"),
+            self.Test("test_self_ref"),
+            self.Test("test_skip"),
+            ])
+
+    def _inject_stream_into_subunit(self, stream):
+        """To be overridden by subclasses that run tests out of process"""
+
+    def _run_selftest_with_suite(self, **kwargs):
+        sio = StringIO()
+        self._inject_stream_into_subunit(sio)
+        old_flags = tests.selftest_debug_flags
+        tests.selftest_debug_flags = old_flags.union(["uncollected_cases"])
+        gc_on = gc.isenabled()
+        if gc_on:
+            gc.disable()
+        try:
+            tests.selftest(test_suite_factory=self._get_suite, stream=sio,
+                stop_on_failure=False, **kwargs)
+        finally:
+            if gc_on:
+                gc.enable()
+            tests.selftest_debug_flags = old_flags
+        output = sio.getvalue()
+        self.assertNotContainsRe(output, "Uncollected test case.*test_pass")
+        self.assertContainsRe(output, "Uncollected test case.*test_self_ref")
+        return output
+
+    def test_testsuite(self):
+        self._run_selftest_with_suite()
+
+    def test_pattern(self):
+        out = self._run_selftest_with_suite(pattern="test_(?:pass|self_ref)$")
+        self.assertNotContainsRe(out, "test_skip")
+
+    def test_exclude_pattern(self):
+        out = self._run_selftest_with_suite(exclude_pattern="test_skip$")
+        self.assertNotContainsRe(out, "test_skip")
+
+    def test_random_seed(self):
+        self._run_selftest_with_suite(random_seed="now")
+
+    def test_matching_tests_first(self):
+        self._run_selftest_with_suite(matching_tests_first=True,
+            pattern="test_self_ref$")
+
+    def test_starting_with_and_exclude(self):
+        out = self._run_selftest_with_suite(starting_with=["bt."],
+            exclude_pattern="test_skip$")
+        self.assertNotContainsRe(out, "test_skip")
+
+    def test_additonal_decorator(self):
+        out = self._run_selftest_with_suite(
+            suite_decorators=[tests.TestDecorator])
+
+
+class TestUncollectedWarningsSubunit(TestUncollectedWarnings):
+    """Check warnings from tests staying alive are emitted with subunit"""
+
+    _test_needs_features = [features.subunit]
+
+    def _run_selftest_with_suite(self, **kwargs):
+        return TestUncollectedWarnings._run_selftest_with_suite(self,
+            runner_class=tests.SubUnitBzrRunner, **kwargs)
+
+
+class TestUncollectedWarningsForking(TestUncollectedWarnings):
+    """Check warnings from tests staying alive are emitted when forking"""
+
+    _test_needs_features = [features.subunit]
+
+    def _inject_stream_into_subunit(self, stream):
+        """Monkey-patch subunit so the extra output goes to stream not stdout
+
+        Some APIs need rewriting so this kind of bogus hackery can be replaced
+        by passing the stream param from run_tests down into ProtocolTestCase.
+        """
+        from subunit import ProtocolTestCase
+        _original_init = ProtocolTestCase.__init__
+        def _init_with_passthrough(self, *args, **kwargs):
+            _original_init(self, *args, **kwargs)
+            self._passthrough = stream
+        self.overrideAttr(ProtocolTestCase, "__init__", _init_with_passthrough)
+
+    def _run_selftest_with_suite(self, **kwargs):
+        # GZ 2011-05-26: Add a PosixSystem feature so this check can go away
+        if getattr(os, "fork", None) is None:
+            raise tests.TestNotApplicable("Platform doesn't support forking")
+        # Make sure the fork code is actually invoked by claiming two cores
+        self.overrideAttr(osutils, "local_concurrency", lambda: 2)
+        kwargs.setdefault("suite_decorators", []).append(tests.fork_decorator)
+        return TestUncollectedWarnings._run_selftest_with_suite(self, **kwargs)
+
+
+class TestEnvironHandling(tests.TestCase):
+
+    def test_overrideEnv_None_called_twice_doesnt_leak(self):
+        self.assertFalse('MYVAR' in os.environ)
+        self.overrideEnv('MYVAR', '42')
+        # We use an embedded test to make sure we fix the _captureVar bug
+        class Test(tests.TestCase):
+            def test_me(self):
+                # The first call save the 42 value
+                self.overrideEnv('MYVAR', None)
+                self.assertEquals(None, os.environ.get('MYVAR'))
+                # Make sure we can call it twice
+                self.overrideEnv('MYVAR', None)
+                self.assertEquals(None, os.environ.get('MYVAR'))
+        output = StringIO()
+        result = tests.TextTestResult(output, 0, 1)
+        Test('test_me').run(result)
+        if not result.wasStrictlySuccessful():
+            self.fail(output.getvalue())
+        # We get our value back
+        self.assertEquals('42', os.environ.get('MYVAR'))
+
+
+class TestIsolatedEnv(tests.TestCase):
+    """Test isolating tests from os.environ.
+
+    Since we use tests that are already isolated from os.environ a bit of care
+    should be taken when designing the tests to avoid bootstrap side-effects.
+    The tests start an already clean os.environ which allow doing valid
+    assertions about which variables are present or not and design tests around
+    these assertions.
+    """
+
+    class ScratchMonkey(tests.TestCase):
+
+        def test_me(self):
+            pass
+
+    def test_basics(self):
+        # Make sure we know the definition of BZR_HOME: not part of os.environ
+        # for tests.TestCase.
+        self.assertTrue('BZR_HOME' in tests.isolated_environ)
+        self.assertEquals(None, tests.isolated_environ['BZR_HOME'])
+        # Being part of isolated_environ, BZR_HOME should not appear here
+        self.assertFalse('BZR_HOME' in os.environ)
+        # Make sure we know the definition of LINES: part of os.environ for
+        # tests.TestCase
+        self.assertTrue('LINES' in tests.isolated_environ)
+        self.assertEquals('25', tests.isolated_environ['LINES'])
+        self.assertEquals('25', os.environ['LINES'])
+
+    def test_injecting_unknown_variable(self):
+        # BZR_HOME is known to be absent from os.environ
+        test = self.ScratchMonkey('test_me')
+        tests.override_os_environ(test, {'BZR_HOME': 'foo'})
+        self.assertEquals('foo', os.environ['BZR_HOME'])
+        tests.restore_os_environ(test)
+        self.assertFalse('BZR_HOME' in os.environ)
+
+    def test_injecting_known_variable(self):
+        test = self.ScratchMonkey('test_me')
+        # LINES is known to be present in os.environ
+        tests.override_os_environ(test, {'LINES': '42'})
+        self.assertEquals('42', os.environ['LINES'])
+        tests.restore_os_environ(test)
+        self.assertEquals('25', os.environ['LINES'])
+
+    def test_deleting_variable(self):
+        test = self.ScratchMonkey('test_me')
+        # LINES is known to be present in os.environ
+        tests.override_os_environ(test, {'LINES': None})
+        self.assertTrue('LINES' not in os.environ)
+        tests.restore_os_environ(test)
+        self.assertEquals('25', os.environ['LINES'])
+
+
+class TestDocTestSuiteIsolation(tests.TestCase):
+    """Test that `tests.DocTestSuite` isolates doc tests from os.environ.
+
+    Since tests.TestCase alreay provides an isolation from os.environ, we use
+    the clean environment as a base for testing. To precisely capture the
+    isolation provided by tests.DocTestSuite, we use doctest.DocTestSuite to
+    compare against.
+
+    We want to make sure `tests.DocTestSuite` respect `tests.isolated_environ`,
+    not `os.environ` so each test overrides it to suit its needs.
+
+    """
+
+    def get_doctest_suite_for_string(self, klass, string):
+        class Finder(doctest.DocTestFinder):
+
+            def find(*args, **kwargs):
+                test = doctest.DocTestParser().get_doctest(
+                    string, {}, 'foo', 'foo.py', 0)
+                return [test]
+
+        suite = klass(test_finder=Finder())
+        return suite
+
+    def run_doctest_suite_for_string(self, klass, string):
+        suite = self.get_doctest_suite_for_string(klass, string)
+        output = StringIO()
+        result = tests.TextTestResult(output, 0, 1)
+        suite.run(result)
+        return result, output
+
+    def assertDocTestStringSucceds(self, klass, string):
+        result, output = self.run_doctest_suite_for_string(klass, string)
+        if not result.wasStrictlySuccessful():
+            self.fail(output.getvalue())
+
+    def assertDocTestStringFails(self, klass, string):
+        result, output = self.run_doctest_suite_for_string(klass, string)
+        if result.wasStrictlySuccessful():
+            self.fail(output.getvalue())
+
+    def test_injected_variable(self):
+        self.overrideAttr(tests, 'isolated_environ', {'LINES': '42'})
+        test = """
+            >>> import os
+            >>> os.environ['LINES']
+            '42'
+            """
+        # doctest.DocTestSuite fails as it sees '25'
+        self.assertDocTestStringFails(doctest.DocTestSuite, test)
+        # tests.DocTestSuite sees '42'
+        self.assertDocTestStringSucceds(tests.IsolatedDocTestSuite, test)
+
+    def test_deleted_variable(self):
+        self.overrideAttr(tests, 'isolated_environ', {'LINES': None})
+        test = """
+            >>> import os
+            >>> os.environ.get('LINES')
+            """
+        # doctest.DocTestSuite fails as it sees '25'
+        self.assertDocTestStringFails(doctest.DocTestSuite, test)
+        # tests.DocTestSuite sees None
+        self.assertDocTestStringSucceds(tests.IsolatedDocTestSuite, test)
+
+
+class TestSelftestExcludePatterns(tests.TestCase):
+
+    def setUp(self):
+        super(TestSelftestExcludePatterns, self).setUp()
+        self.overrideAttr(tests, 'test_suite', self.suite_factory)
+
+    def suite_factory(self, keep_only=None, starting_with=None):
+        """A test suite factory with only a few tests."""
+        class Test(tests.TestCase):
+            def id(self):
+                # We don't need the full class path
+                return self._testMethodName
+            def a(self):
+                pass
+            def b(self):
+                pass
+            def c(self):
+                pass
+        return TestUtil.TestSuite([Test("a"), Test("b"), Test("c")])
+
+    def assertTestList(self, expected, *selftest_args):
+        # We rely on setUp installing the right test suite factory so we can
+        # test at the command level without loading the whole test suite
+        out, err = self.run_bzr(('selftest', '--list') + selftest_args)
+        actual = out.splitlines()
+        self.assertEquals(expected, actual)
+
+    def test_full_list(self):
+        self.assertTestList(['a', 'b', 'c'])
+
+    def test_single_exclude(self):
+        self.assertTestList(['b', 'c'], '-x', 'a')
+
+    def test_mutiple_excludes(self):
+        self.assertTestList(['c'], '-x', 'a', '-x', 'b')
+
+
+class TestCounterHooks(tests.TestCase, SelfTestHelper):
+
+    _test_needs_features = [features.subunit]
+
+    def setUp(self):
+        super(TestCounterHooks, self).setUp()
+        class Test(tests.TestCase):
+
+            def setUp(self):
+                super(Test, self).setUp()
+                self.hooks = hooks.Hooks()
+                self.hooks.add_hook('myhook', 'Foo bar blah', (2,4))
+                self.install_counter_hook(self.hooks, 'myhook')
+
+            def no_hook(self):
+                pass
+
+            def run_hook_once(self):
+                for hook in self.hooks['myhook']:
+                    hook(self)
+
+        self.test_class = Test
+
+    def assertHookCalls(self, expected_calls, test_name):
+        test = self.test_class(test_name)
+        result = unittest.TestResult()
+        test.run(result)
+        self.assertTrue(hasattr(test, '_counters'))
+        self.assertTrue(test._counters.has_key('myhook'))
+        self.assertEquals(expected_calls, test._counters['myhook'])
+
+    def test_no_hook(self):
+        self.assertHookCalls(0, 'no_hook')
+
+    def test_run_hook_once(self):
+        tt = features.testtools
+        if tt.module.__version__ < (0, 9, 8):
+            raise tests.TestSkipped('testtools-0.9.8 required for addDetail')
+        self.assertHookCalls(1, 'run_hook_once')

=== modified file 'bzrlib/tests/test_serializer.py'
--- bzrlib/tests/test_serializer.py	2009-06-10 03:56:49 +0000
+++ bzrlib/tests/test_serializer.py	2011-02-19 15:23:08 +0000
@@ -21,7 +21,6 @@
 from bzrlib import (
     chk_serializer,
     serializer,
-    xml4,
     xml5,
     xml6,
     xml7,
@@ -34,8 +33,6 @@
     """Test serializer"""
 
     def test_registry(self):
-        self.assertIs(xml4.serializer_v4,
-                      serializer.format_registry.get('4'))
         self.assertIs(xml5.serializer_v5,
                       serializer.format_registry.get('5'))
         self.assertIs(xml6.serializer_v6,

=== modified file 'bzrlib/tests/test_server.py'
--- bzrlib/tests/test_server.py	2010-02-11 09:21:45 +0000
+++ bzrlib/tests/test_server.py	2011-10-10 13:59:22 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008, 2010 Canonical Ltd
+# Copyright (C) 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import errno
+import socket
+import SocketServer
+import sys
+import threading
+import traceback
+
+
 from bzrlib import (
+    cethread,
+    errors,
+    osutils,
     transport,
     urlutils,
     )
@@ -22,7 +33,18 @@
     chroot,
     pathfilter,
     )
-from bzrlib.smart import server
+from bzrlib.smart import (
+    medium,
+    server,
+    )
+
+
+def debug_threads():
+    # FIXME: There is a dependency loop between bzrlib.tests and
+    # bzrlib.tests.test_server that needs to be fixed. In the mean time
+    # defining this function is enough for our needs. -- vila 20100611
+    from bzrlib import tests
+    return 'threads' in tests.selftest_debug_flags
 
 
 class TestServer(transport.Server):
@@ -192,10 +214,10 @@
     def start_server(self, backing_server=None):
         """Setup the Chroot on backing_server."""
         if backing_server is not None:
-            self.backing_transport = transport.get_transport(
+            self.backing_transport = transport.get_transport_from_url(
                 backing_server.get_url())
         else:
-            self.backing_transport = transport.get_transport('.')
+            self.backing_transport = transport.get_transport_from_path('.')
         self.backing_transport.clone('added-by-filter').ensure_base()
         self.filter_func = lambda x: 'added-by-filter/' + x
         super(TestingPathFilteringServer, self).start_server()
@@ -213,33 +235,435 @@
     def start_server(self, backing_server=None):
         """Setup the Chroot on backing_server."""
         if backing_server is not None:
-            self.backing_transport = transport.get_transport(
+            self.backing_transport = transport.get_transport_from_url(
                 backing_server.get_url())
         else:
-            self.backing_transport = transport.get_transport('.')
+            self.backing_transport = transport.get_transport_from_path('.')
         super(TestingChrootServer, self).start_server()
 
     def get_bogus_url(self):
         raise NotImplementedError
 
 
-class SmartTCPServer_for_testing(server.SmartTCPServer):
+class TestThread(cethread.CatchingExceptionThread):
+
+    def join(self, timeout=5):
+        """Overrides to use a default timeout.
+
+        The default timeout is set to 5 and should expire only when a thread
+        serving a client connection is hung.
+        """
+        super(TestThread, self).join(timeout)
+        if timeout and self.isAlive():
+            # The timeout expired without joining the thread, the thread is
+            # therefore stucked and that's a failure as far as the test is
+            # concerned. We used to hang here.
+
+            # FIXME: we need to kill the thread, but as far as the test is
+            # concerned, raising an assertion is too strong. On most of the
+            # platforms, this doesn't occur, so just mentioning the problem is
+            # enough for now -- vila 2010824
+            sys.stderr.write('thread %s hung\n' % (self.name,))
+            #raise AssertionError('thread %s hung' % (self.name,))
+
+
+class TestingTCPServerMixin(object):
+    """Mixin to support running SocketServer.TCPServer in a thread.
+
+    Tests are connecting from the main thread, the server has to be run in a
+    separate thread.
+    """
+
+    def __init__(self):
+        self.started = threading.Event()
+        self.serving = None
+        self.stopped = threading.Event()
+        # We collect the resources used by the clients so we can release them
+        # when shutting down
+        self.clients = []
+        self.ignored_exceptions = None
+
+    def server_bind(self):
+        self.socket.bind(self.server_address)
+        self.server_address = self.socket.getsockname()
+
+    def serve(self):
+        self.serving = True
+        # We are listening and ready to accept connections
+        self.started.set()
+        try:
+            while self.serving:
+                # Really a connection but the python framework is generic and
+                # call them requests
+                self.handle_request()
+            # Let's close the listening socket
+            self.server_close()
+        finally:
+            self.stopped.set()
+
+    def handle_request(self):
+        """Handle one request.
+
+        The python version swallows some socket exceptions and we don't use
+        timeout, so we override it to better control the server behavior.
+        """
+        request, client_address = self.get_request()
+        if self.verify_request(request, client_address):
+            try:
+                self.process_request(request, client_address)
+            except:
+                self.handle_error(request, client_address)
+        else:
+            self.close_request(request)
+
+    def get_request(self):
+        return self.socket.accept()
+
+    def verify_request(self, request, client_address):
+        """Verify the request.
+
+        Return True if we should proceed with this request, False if we should
+        not even touch a single byte in the socket ! This is useful when we
+        stop the server with a dummy last connection.
+        """
+        return self.serving
+
+    def handle_error(self, request, client_address):
+        # Stop serving and re-raise the last exception seen
+        self.serving = False
+        # The following can be used for debugging purposes, it will display the
+        # exception and the traceback just when it occurs instead of waiting
+        # for the thread to be joined.
+        # SocketServer.BaseServer.handle_error(self, request, client_address)
+
+        # We call close_request manually, because we are going to raise an
+        # exception. The SocketServer implementation calls:
+        #   handle_error(...)
+        #   close_request(...)
+        # But because we raise the exception, close_request will never be
+        # triggered. This helps client not block waiting for a response when
+        # the server gets an exception.
+        self.close_request(request)
+        raise
+
+    def ignored_exceptions_during_shutdown(self, e):
+        if sys.platform == 'win32':
+            accepted_errnos = [errno.EBADF,
+                               errno.EPIPE,
+                               errno.WSAEBADF,
+                               errno.WSAECONNRESET,
+                               errno.WSAENOTCONN,
+                               errno.WSAESHUTDOWN,
+                               ]
+        else:
+            accepted_errnos = [errno.EBADF,
+                               errno.ECONNRESET,
+                               errno.ENOTCONN,
+                               errno.EPIPE,
+                               ]
+        if isinstance(e, socket.error) and e[0] in accepted_errnos:
+            return True
+        return False
+
+    # The following methods are called by the main thread
+
+    def stop_client_connections(self):
+        while self.clients:
+            c = self.clients.pop()
+            self.shutdown_client(c)
+
+    def shutdown_socket(self, sock):
+        """Properly shutdown a socket.
+
+        This should be called only when no other thread is trying to use the
+        socket.
+        """
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+            sock.close()
+        except Exception, e:
+            if self.ignored_exceptions(e):
+                pass
+            else:
+                raise
+
+    # The following methods are called by the main thread
+
+    def set_ignored_exceptions(self, thread, ignored_exceptions):
+        self.ignored_exceptions = ignored_exceptions
+        thread.set_ignored_exceptions(self.ignored_exceptions)
+
+    def _pending_exception(self, thread):
+        """Raise server uncaught exception.
+
+        Daughter classes can override this if they use daughter threads.
+        """
+        thread.pending_exception()
+
+
+class TestingTCPServer(TestingTCPServerMixin, SocketServer.TCPServer):
+
+    def __init__(self, server_address, request_handler_class):
+        TestingTCPServerMixin.__init__(self)
+        SocketServer.TCPServer.__init__(self, server_address,
+                                        request_handler_class)
+
+    def get_request(self):
+        """Get the request and client address from the socket."""
+        sock, addr = TestingTCPServerMixin.get_request(self)
+        self.clients.append((sock, addr))
+        return sock, addr
+
+    # The following methods are called by the main thread
+
+    def shutdown_client(self, client):
+        sock, addr = client
+        self.shutdown_socket(sock)
+
+
+class TestingThreadingTCPServer(TestingTCPServerMixin,
+                                SocketServer.ThreadingTCPServer):
+
+    def __init__(self, server_address, request_handler_class):
+        TestingTCPServerMixin.__init__(self)
+        SocketServer.ThreadingTCPServer.__init__(self, server_address,
+                                                 request_handler_class)
+
+    def get_request(self):
+        """Get the request and client address from the socket."""
+        sock, addr = TestingTCPServerMixin.get_request(self)
+        # The thread is not create yet, it will be updated in process_request
+        self.clients.append((sock, addr, None))
+        return sock, addr
+
+    def process_request_thread(self, started, stopped, request, client_address):
+        started.set()
+        SocketServer.ThreadingTCPServer.process_request_thread(
+            self, request, client_address)
+        self.close_request(request)
+        stopped.set()
+
+    def process_request(self, request, client_address):
+        """Start a new thread to process the request."""
+        started = threading.Event()
+        stopped = threading.Event()
+        t = TestThread(
+            sync_event=stopped,
+            name='%s -> %s' % (client_address, self.server_address),
+            target=self.process_request_thread,
+            args=(started, stopped, request, client_address))
+        # Update the client description
+        self.clients.pop()
+        self.clients.append((request, client_address, t))
+        # Propagate the exception handler since we must use the same one as
+        # TestingTCPServer for connections running in their own threads.
+        t.set_ignored_exceptions(self.ignored_exceptions)
+        t.start()
+        started.wait()
+        if debug_threads():
+            sys.stderr.write('Client thread %s started\n' % (t.name,))
+        # If an exception occured during the thread start, it will get raised.
+        # In rare cases, an exception raised during the request processing may
+        # also get caught here (see http://pad.lv/869366)
+        t.pending_exception()
+
+    # The following methods are called by the main thread
+
+    def shutdown_client(self, client):
+        sock, addr, connection_thread = client
+        self.shutdown_socket(sock)
+        if connection_thread is not None:
+            # The thread has been created only if the request is processed but
+            # after the connection is inited. This could happen during server
+            # shutdown. If an exception occurred in the thread it will be
+            # re-raised
+            if debug_threads():
+                sys.stderr.write('Client thread %s will be joined\n'
+                                 % (connection_thread.name,))
+            connection_thread.join()
+
+    def set_ignored_exceptions(self, thread, ignored_exceptions):
+        TestingTCPServerMixin.set_ignored_exceptions(self, thread,
+                                                     ignored_exceptions)
+        for sock, addr, connection_thread in self.clients:
+            if connection_thread is not None:
+                connection_thread.set_ignored_exceptions(
+                    self.ignored_exceptions)
+
+    def _pending_exception(self, thread):
+        for sock, addr, connection_thread in self.clients:
+            if connection_thread is not None:
+                connection_thread.pending_exception()
+        TestingTCPServerMixin._pending_exception(self, thread)
+
+
+class TestingTCPServerInAThread(transport.Server):
+    """A server in a thread that re-raise thread exceptions."""
+
+    def __init__(self, server_address, server_class, request_handler_class):
+        self.server_class = server_class
+        self.request_handler_class = request_handler_class
+        self.host, self.port = server_address
+        self.server = None
+        self._server_thread = None
+
+    def __repr__(self):
+        return "%s(%s:%s)" % (self.__class__.__name__, self.host, self.port)
+
+    def create_server(self):
+        return self.server_class((self.host, self.port),
+                                 self.request_handler_class)
+
+    def start_server(self):
+        self.server = self.create_server()
+        self._server_thread = TestThread(
+            sync_event=self.server.started,
+            target=self.run_server)
+        self._server_thread.start()
+        # Wait for the server thread to start (i.e. release the lock)
+        self.server.started.wait()
+        # Get the real address, especially the port
+        self.host, self.port = self.server.server_address
+        self._server_thread.name = self.server.server_address
+        if debug_threads():
+            sys.stderr.write('Server thread %s started\n'
+                             % (self._server_thread.name,))
+        # If an exception occured during the server start, it will get raised,
+        # otherwise, the server is blocked on its accept() call.
+        self._server_thread.pending_exception()
+        # From now on, we'll use a different event to ensure the server can set
+        # its exception
+        self._server_thread.set_sync_event(self.server.stopped)
+
+    def run_server(self):
+        self.server.serve()
+
+    def stop_server(self):
+        if self.server is None:
+            return
+        try:
+            # The server has been started successfully, shut it down now.  As
+            # soon as we stop serving, no more connection are accepted except
+            # one to get out of the blocking listen.
+            self.set_ignored_exceptions(
+                self.server.ignored_exceptions_during_shutdown)
+            self.server.serving = False
+            if debug_threads():
+                sys.stderr.write('Server thread %s will be joined\n'
+                                 % (self._server_thread.name,))
+            # The server is listening for a last connection, let's give it:
+            last_conn = None
+            try:
+                last_conn = osutils.connect_socket((self.host, self.port))
+            except socket.error, e:
+                # But ignore connection errors as the point is to unblock the
+                # server thread, it may happen that it's not blocked or even
+                # not started.
+                pass
+            # We start shutting down the clients while the server itself is
+            # shutting down.
+            self.server.stop_client_connections()
+            # Now we wait for the thread running self.server.serve() to finish
+            self.server.stopped.wait()
+            if last_conn is not None:
+                # Close the last connection without trying to use it. The
+                # server will not process a single byte on that socket to avoid
+                # complications (SSL starts with a handshake for example).
+                last_conn.close()
+            # Check for any exception that could have occurred in the server
+            # thread
+            try:
+                self._server_thread.join()
+            except Exception, e:
+                if self.server.ignored_exceptions(e):
+                    pass
+                else:
+                    raise
+        finally:
+            # Make sure we can be called twice safely, note that this means
+            # that we will raise a single exception even if several occurred in
+            # the various threads involved.
+            self.server = None
+
+    def set_ignored_exceptions(self, ignored_exceptions):
+        """Install an exception handler for the server."""
+        self.server.set_ignored_exceptions(self._server_thread,
+                                           ignored_exceptions)
+
+    def pending_exception(self):
+        """Raise uncaught exception in the server."""
+        self.server._pending_exception(self._server_thread)
+
+
+class TestingSmartConnectionHandler(SocketServer.BaseRequestHandler,
+                                    medium.SmartServerSocketStreamMedium):
+
+    def __init__(self, request, client_address, server):
+        medium.SmartServerSocketStreamMedium.__init__(
+            self, request, server.backing_transport,
+            server.root_client_path,
+            timeout=_DEFAULT_TESTING_CLIENT_TIMEOUT)
+        request.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+        SocketServer.BaseRequestHandler.__init__(self, request, client_address,
+                                                 server)
+
+    def handle(self):
+        try:
+            while not self.finished:
+                server_protocol = self._build_protocol()
+                self._serve_one_request(server_protocol)
+        except errors.ConnectionTimeout:
+            # idle connections aren't considered a failure of the server
+            return
+
+
+_DEFAULT_TESTING_CLIENT_TIMEOUT = 60.0
+
+class TestingSmartServer(TestingThreadingTCPServer, server.SmartTCPServer):
+
+    def __init__(self, server_address, request_handler_class,
+                 backing_transport, root_client_path):
+        TestingThreadingTCPServer.__init__(self, server_address,
+                                           request_handler_class)
+        server.SmartTCPServer.__init__(self, backing_transport,
+            root_client_path, client_timeout=_DEFAULT_TESTING_CLIENT_TIMEOUT)
+
+    def serve(self):
+        self.run_server_started_hooks()
+        try:
+            TestingThreadingTCPServer.serve(self)
+        finally:
+            self.run_server_stopped_hooks()
+
+    def get_url(self):
+        """Return the url of the server"""
+        return "bzr://%s:%d/" % self.server_address
+
+
+class SmartTCPServer_for_testing(TestingTCPServerInAThread):
     """Server suitable for use by transport tests.
 
     This server is backed by the process's cwd.
     """
-
     def __init__(self, thread_name_suffix=''):
-        super(SmartTCPServer_for_testing, self).__init__(None)
         self.client_path_extra = None
         self.thread_name_suffix = thread_name_suffix
-
-    def get_backing_transport(self, backing_transport_server):
-        """Get a backing transport from a server we are decorating."""
-        return transport.get_transport(backing_transport_server.get_url())
+        self.host = '127.0.0.1'
+        self.port = 0
+        super(SmartTCPServer_for_testing, self).__init__(
+                (self.host, self.port),
+                TestingSmartServer,
+                TestingSmartConnectionHandler)
+
+    def create_server(self):
+        return self.server_class((self.host, self.port),
+                                 self.request_handler_class,
+                                 self.backing_transport,
+                                 self.root_client_path)
+
 
     def start_server(self, backing_transport_server=None,
-              client_path_extra='/extra/'):
+                     client_path_extra='/extra/'):
         """Set up server for testing.
 
         :param backing_transport_server: backing server to use.  If not
@@ -254,23 +678,30 @@
         """
         if not client_path_extra.startswith('/'):
             raise ValueError(client_path_extra)
+        self.root_client_path = self.client_path_extra = client_path_extra
         from bzrlib.transport.chroot import ChrootServer
         if backing_transport_server is None:
             backing_transport_server = LocalURLServer()
         self.chroot_server = ChrootServer(
             self.get_backing_transport(backing_transport_server))
         self.chroot_server.start_server()
-        self.backing_transport = transport.get_transport(
+        self.backing_transport = transport.get_transport_from_url(
             self.chroot_server.get_url())
-        self.root_client_path = self.client_path_extra = client_path_extra
-        self.start_background_thread(self.thread_name_suffix)
+        super(SmartTCPServer_for_testing, self).start_server()
 
     def stop_server(self):
-        self.stop_background_thread()
-        self.chroot_server.stop_server()
+        try:
+            super(SmartTCPServer_for_testing, self).stop_server()
+        finally:
+            self.chroot_server.stop_server()
+
+    def get_backing_transport(self, backing_transport_server):
+        """Get a backing transport from a server we are decorating."""
+        return transport.get_transport_from_url(
+            backing_transport_server.get_url())
 
     def get_url(self):
-        url = super(SmartTCPServer_for_testing, self).get_url()
+        url = self.server.get_url()
         return url[:-1] + self.client_path_extra
 
     def get_bogus_url(self):
@@ -284,7 +715,7 @@
     def get_backing_transport(self, backing_transport_server):
         """Get a backing transport from a server we are decorating."""
         url = 'readonly+' + backing_transport_server.get_url()
-        return transport.get_transport(url)
+        return transport.get_transport_from_url(url)
 
 
 class SmartTCPServer_for_testing_v2_only(SmartTCPServer_for_testing):
@@ -305,8 +736,4 @@
     def get_backing_transport(self, backing_transport_server):
         """Get a backing transport from a server we are decorating."""
         url = 'readonly+' + backing_transport_server.get_url()
-        return transport.get_transport(url)
-
-
-
-
+        return transport.get_transport_from_url(url)

=== modified file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_setup.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,8 +19,6 @@
 import os
 import sys
 import subprocess
-import shutil
-from tempfile import TemporaryFile
 
 import bzrlib
 from bzrlib.tests import TestCase, TestSkipped
@@ -43,30 +41,34 @@
         This tests that the build process and man generator run correctly.
         It also can catch new subdirectories that weren't added to setup.py.
         """
-        if not os.path.isfile('setup.py'):
-            raise TestSkipped('There is no setup.py file in current directory')
+        # setup.py must be run from the root source directory, but the tests
+        # are not necessarily invoked from there
+        self.source_dir = os.path.dirname(os.path.dirname(bzrlib.__file__))
+        if not os.path.isfile(os.path.join(self.source_dir, 'setup.py')):
+            raise TestSkipped(
+                'There is no setup.py file adjacent to the bzrlib directory')
         try:
             import distutils.sysconfig
             makefile_path = distutils.sysconfig.get_makefile_filename()
             if not os.path.exists(makefile_path):
-                raise TestSkipped('You must have the python Makefile installed to run this test.'
-                                  ' Usually this can be found by installing "python-dev"')
+                raise TestSkipped(
+                    'You must have the python Makefile installed to run this'
+                    ' test. Usually this can be found by installing'
+                    ' "python-dev"')
         except ImportError:
-            raise TestSkipped('You must have distutils installed to run this test.'
-                              ' Usually this can be found by installing "python-dev"')
+            raise TestSkipped(
+                'You must have distutils installed to run this test.'
+                ' Usually this can be found by installing "python-dev"')
         self.log('test_build running in %s' % os.getcwd())
-        install_dir = osutils.mkdtemp()
-        # setup.py must be run from the root source directory, but the tests
-        # are not necessarily invoked from there
-        self.source_dir = os.path.dirname(os.path.dirname(bzrlib.__file__))
+        root_dir = osutils.mkdtemp()
         try:
             self.run_setup(['clean'])
             # build is implied by install
             ## self.run_setup(['build'])
-            self.run_setup(['install', '--prefix', install_dir])
+            self.run_setup(['install', '--root', root_dir])
             self.run_setup(['clean'])
         finally:
-            osutils.rmtree(install_dir)
+            osutils.rmtree(root_dir)
 
     def run_setup(self, args):
         args = [sys.executable, './setup.py', ] + args
@@ -74,9 +76,11 @@
         self.log('args: %r', args)
         p = subprocess.Popen(args,
                              cwd=self.source_dir,
-                             stdout=self._log_file,
-                             stderr=self._log_file,
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE,
                              )
-        s = p.communicate()
+        stdout, stderr = p.communicate()
+        self.log('stdout: %r', stdout)
+        self.log('stderr: %r', stderr)
         self.assertEqual(0, p.returncode,
                          'invocation of %r failed' % args)

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_sftp_transport.py	2011-09-26 15:21:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Robey Pointer <robey@lag.net>
+# Copyright (C) 2005-2011 Robey Pointer <robey@lag.net>
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,6 @@
 import os
 import socket
 import sys
-import threading
 import time
 
 from bzrlib import (
@@ -30,9 +29,7 @@
     ui,
     )
 from bzrlib.osutils import (
-    pathjoin,
     lexists,
-    set_or_unset_env,
     )
 from bzrlib.tests import (
     features,
@@ -47,8 +44,6 @@
     from bzrlib.transport import sftp as _mod_sftp
     from bzrlib.tests import stub_sftp
 
-from bzrlib.workingtree import WorkingTree
-
 
 def set_test_transport_to_sftp(testcase):
     """A helper to set transports on test case instances."""
@@ -77,14 +72,14 @@
         t = self.get_transport()
 
         l = t.lock_write('bogus')
-        self.failUnlessExists('bogus.write-lock')
+        self.assertPathExists('bogus.write-lock')
 
         # Don't wait for the lock, locking an already locked
         # file should raise an assert
         self.assertRaises(LockError, t.lock_write, 'bogus')
 
         l.unlock()
-        self.failIf(lexists('bogus.write-lock'))
+        self.assertFalse(lexists('bogus.write-lock'))
 
         open('something.write-lock', 'wb').write('fake lock\n')
         self.assertRaises(LockError, t.lock_write, 'something')
@@ -145,7 +140,7 @@
     def test__remote_path_relative_root(self):
         # relative paths are preserved
         t = self.get_transport('')
-        self.assertEqual('/~/', t._path)
+        self.assertEqual('/~/', t._parsed_url.path)
         # the remote path should be relative to home dir
         # (i.e. not begining with a '/')
         self.assertEqual('a', t._remote_path('a'))
@@ -159,11 +154,11 @@
     def test_parse_url_with_home_dir(self):
         s = _mod_sftp.SFTPTransport(
             'sftp://ro%62ey:h%40t@example.com:2222/~/relative')
-        self.assertEquals(s._host, 'example.com')
-        self.assertEquals(s._port, 2222)
-        self.assertEquals(s._user, 'robey')
-        self.assertEquals(s._password, 'h@t')
-        self.assertEquals(s._path, '/~/relative/')
+        self.assertEquals(s._parsed_url.host, 'example.com')
+        self.assertEquals(s._parsed_url.port, 2222)
+        self.assertEquals(s._parsed_url.user, 'robey')
+        self.assertEquals(s._parsed_url.password, 'h@t')
+        self.assertEquals(s._parsed_url.path, '/~/relative/')
 
     def test_relpath(self):
         s = _mod_sftp.SFTPTransport('sftp://user@host.com/abs/path')
@@ -174,46 +169,26 @@
         """Test that if no 'ssh' is available we get builtin paramiko"""
         from bzrlib.transport import ssh
         # set '.' as the only location in the path, forcing no 'ssh' to exist
-        orig_vendor = ssh._ssh_vendor_manager._cached_ssh_vendor
-        orig_path = set_or_unset_env('PATH', '.')
-        try:
-            # No vendor defined yet, query for one
-            ssh._ssh_vendor_manager.clear_cache()
-            vendor = ssh._get_ssh_vendor()
-            self.assertIsInstance(vendor, ssh.ParamikoVendor)
-        finally:
-            set_or_unset_env('PATH', orig_path)
-            ssh._ssh_vendor_manager._cached_ssh_vendor = orig_vendor
+        self.overrideAttr(ssh, '_ssh_vendor_manager')
+        self.overrideEnv('PATH', '.')
+        ssh._ssh_vendor_manager.clear_cache()
+        vendor = ssh._get_ssh_vendor()
+        self.assertIsInstance(vendor, ssh.ParamikoVendor)
 
     def test_abspath_root_sibling_server(self):
         server = stub_sftp.SFTPSiblingAbsoluteServer()
         server.start_server()
-        try:
-            transport = _mod_transport.get_transport(server.get_url())
-            self.assertFalse(transport.abspath('/').endswith('/~/'))
-            self.assertTrue(transport.abspath('/').endswith('/'))
-            del transport
-        finally:
-            server.stop_server()
+        self.addCleanup(server.stop_server)
+
+        transport = _mod_transport.get_transport_from_url(server.get_url())
+        self.assertFalse(transport.abspath('/').endswith('/~/'))
+        self.assertTrue(transport.abspath('/').endswith('/'))
+        del transport
 
 
 class SFTPBranchTest(TestCaseWithSFTPServer):
     """Test some stuff when accessing a bzr Branch over sftp"""
 
-    def test_lock_file(self):
-        # old format branches use a special lock file on sftp.
-        b = self.make_branch('', format=bzrdir.BzrDirFormat6())
-        b = bzrlib.branch.Branch.open(self.get_url())
-        self.failUnlessExists('.bzr/')
-        self.failUnlessExists('.bzr/branch-format')
-        self.failUnlessExists('.bzr/branch-lock')
-
-        self.failIf(lexists('.bzr/branch-lock.write-lock'))
-        b.lock_write()
-        self.failUnlessExists('.bzr/branch-lock.write-lock')
-        b.unlock()
-        self.failIf(lexists('.bzr/branch-lock.write-lock'))
-
     def test_push_support(self):
         self.build_tree(['a/', 'a/foo'])
         t = bzrdir.BzrDir.create_standalone_workingtree('a')
@@ -224,13 +199,13 @@
         b2 = bzrdir.BzrDir.create_branch_and_repo(self.get_url('/b'))
         b2.pull(b)
 
-        self.assertEquals(b2.revision_history(), ['a1'])
+        self.assertEquals(b2.last_revision(), 'a1')
 
         open('a/foo', 'wt').write('something new in foo\n')
         t.commit('new', rev_id='a2')
         b2.pull(b)
 
-        self.assertEquals(b2.revision_history(), ['a1', 'a2'])
+        self.assertEquals(b2.last_revision(), 'a2')
 
 
 class SSHVendorConnection(TestCaseWithSFTPServer):
@@ -312,7 +287,7 @@
         """Test that a real connection attempt raises the right error"""
         from bzrlib.transport import ssh
         self.set_vendor(ssh.ParamikoVendor())
-        t = _mod_transport.get_transport(self.bogus_url)
+        t = _mod_transport.get_transport_from_url(self.bogus_url)
         self.assertRaises(errors.ConnectionError, t.get, 'foobar')
 
     def test_bad_connection_ssh(self):
@@ -442,7 +417,7 @@
 
 
 class ReadvFile(object):
-    """An object that acts like Paramiko's SFTPFile.readv()"""
+    """An object that acts like Paramiko's SFTPFile when readv() is used"""
 
     def __init__(self, data):
         self._data = data
@@ -451,6 +426,9 @@
         for start, length in requests:
             yield self._data[start:start+length]
 
+    def close(self):
+        pass
+
 
 def _null_report_activity(*a, **k):
     pass
@@ -504,13 +482,14 @@
     """Test that AuthenticationConfig can supply default usernames."""
 
     def get_transport_for_connection(self, set_config):
-        port = self.get_server()._listener.port
+        port = self.get_server().port
         if set_config:
             conf = config.AuthenticationConfig()
             conf._get_config().update(
                 {'sftptest': {'scheme': 'ssh', 'port': port, 'user': 'bar'}})
             conf._save()
-        t = _mod_transport.get_transport('sftp://localhost:%d' % port)
+        t = _mod_transport.get_transport_from_url(
+            'sftp://localhost:%d' % port)
         # force a connection to be performed.
         t.has('foo')
         return t

=== modified file 'bzrlib/tests/test_shelf.py'
--- bzrlib/tests/test_shelf.py	2010-01-13 23:06:42 +0000
+++ bzrlib/tests/test_shelf.py	2011-07-08 23:01:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +25,9 @@
     transform,
     workingtree,
     )
+from bzrlib.tests import (
+    features,
+    )
 
 
 EMPTY_SHELF = ("Bazaar pack format 1 (introduced in 0.18)\n"
@@ -191,8 +194,8 @@
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
         self.assertEqual('foo-id',
                          creator.shelf_transform.final_file_id(s_trans_id))
-        self.failIfExists('foo')
-        self.failIfExists('bar')
+        self.assertPathDoesNotExist('foo')
+        self.assertPathDoesNotExist('bar')
         self.assertShelvedFileEqual('a\n', creator, 'foo-id')
         s_bar_trans_id = creator.shelf_transform.trans_id_file_id('bar-id')
         self.assertEqual('directory',
@@ -214,7 +217,7 @@
 
     def _test_shelve_symlink_creation(self, link_name, link_target,
                                       shelve_change=False):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -231,7 +234,7 @@
             creator.shelve_creation('foo-id')
         creator.transform()
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
-        self.failIfExists(link_name)
+        self.assertPathDoesNotExist(link_name)
         limbo_name = creator.shelf_transform._limbo_name(s_trans_id)
         self.assertEqual(link_target, osutils.readlink(limbo_name))
         ptree = creator.shelf_transform.get_preview_tree()
@@ -241,7 +244,7 @@
         self._test_shelve_symlink_creation('foo', 'bar')
 
     def test_shelve_unicode_symlink_creation(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_shelve_symlink_creation(u'fo\N{Euro Sign}o',
                                            u'b\N{Euro Sign}ar')
 
@@ -251,7 +254,7 @@
     def _test_shelve_symlink_target_change(self, link_name,
                                            old_target, new_target,
                                            shelve_change=False):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         tree.lock_write()
         self.addCleanup(tree.unlock)
@@ -282,7 +285,7 @@
         self._test_shelve_symlink_target_change('foo', 'bar', 'baz')
 
     def test_shelve_unicode_symlink_target_change(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_shelve_symlink_target_change(
             u'fo\N{Euro Sign}o', u'b\N{Euro Sign}ar', u'b\N{Euro Sign}az')
 
@@ -310,7 +313,7 @@
         s_trans_id = creator.shelf_transform.trans_id_file_id('foo-id')
         self.assertEqual('foo-id',
                          creator.shelf_transform.final_file_id(s_trans_id))
-        self.failIfExists('foo')
+        self.assertPathDoesNotExist('foo')
 
     def prepare_shelve_deletion(self):
         tree = self.make_branch_and_tree('tree')
@@ -330,8 +333,8 @@
         return creator, tree
 
     def check_shelve_deletion(self, tree):
-        self.assertTrue('foo-id' in tree)
-        self.assertTrue('bar-id' in tree)
+        self.assertTrue(tree.has_id('foo-id'))
+        self.assertTrue(tree.has_id('bar-id'))
         self.assertFileEqual('baz', 'tree/foo/bar')
 
     def test_shelve_deletion(self):
@@ -362,7 +365,7 @@
                          sorted(list(creator.iter_shelvable())))
         creator.shelve_deletion('foo-id')
         creator.transform()
-        self.failUnlessExists('tree/foo')
+        self.assertPathExists('tree/foo')
 
     def prepare_shelve_change_kind(self):
         tree = self.make_branch_and_tree('tree')
@@ -421,7 +424,7 @@
                          sorted(list(creator.iter_shelvable())))
         creator.shelve_deletion('foo-id')
         creator.transform()
-        self.failUnlessExists('tree/foo')
+        self.assertPathExists('tree/foo')
 
     def test_shelve_serialization(self):
         tree = self.make_branch_and_tree('.')
@@ -568,21 +571,20 @@
         list(creator.iter_shelvable())
         creator.shelve_deletion('foo-id')
         creator.shelve_deletion('bar-id')
-        shelf_file = open('shelf', 'w+b')
-        self.addCleanup(shelf_file.close)
-        creator.write_shelf(shelf_file)
-        creator.transform()
-        creator.finalize()
+        with open('shelf', 'w+b') as shelf_file:
+            creator.write_shelf(shelf_file)
+            creator.transform()
+            creator.finalize()
         # validate the test setup
-        self.assertTrue('foo-id' in tree)
-        self.assertTrue('bar-id' in tree)
+        self.assertTrue(tree.has_id('foo-id'))
+        self.assertTrue(tree.has_id('bar-id'))
         self.assertFileEqual('baz', 'tree/foo/bar')
-        shelf_file.seek(0)
-        unshelver = shelf.Unshelver.from_tree_and_shelf(tree, shelf_file)
-        self.addCleanup(unshelver.finalize)
-        unshelver.make_merger().do_merge()
-        self.assertFalse('foo-id' in tree)
-        self.assertFalse('bar-id' in tree)
+        with open('shelf', 'r+b') as shelf_file:
+            unshelver = shelf.Unshelver.from_tree_and_shelf(tree, shelf_file)
+            self.addCleanup(unshelver.finalize)
+            unshelver.make_merger().do_merge()
+        self.assertFalse(tree.has_id('foo-id'))
+        self.assertFalse(tree.has_id('bar-id'))
 
     def test_unshelve_base(self):
         tree = self.make_branch_and_tree('tree')
@@ -623,6 +625,29 @@
                               shelf_file)
         self.assertEqual('Shelf corrupt.', str(e))
 
+    def test_unshelve_subdir_in_now_removed_dir(self):
+        tree = self.make_branch_and_tree('.')
+        self.addCleanup(tree.lock_write().unlock)
+        self.build_tree(['dir/', 'dir/subdir/', 'dir/subdir/foo'])
+        tree.add(['dir'], ['dir-id'])
+        tree.commit('versioned dir')
+        tree.add(['dir/subdir', 'dir/subdir/foo'], ['subdir-id', 'foo-id'])
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        for change in creator.iter_shelvable():
+            creator.shelve_change(change)
+        shelf_manager = tree.get_shelf_manager()
+        shelf_id = shelf_manager.shelve_changes(creator)
+        self.assertPathDoesNotExist('dir/subdir')
+        tree.remove(['dir'])
+        unshelver = shelf_manager.get_unshelver(shelf_id)
+        self.addCleanup(unshelver.finalize)
+        unshelver.make_merger().do_merge()
+        self.assertPathExists('dir/subdir/foo')
+        self.assertEqual('dir-id', tree.path2id('dir'))
+        self.assertEqual('subdir-id', tree.path2id('dir/subdir'))
+        self.assertEqual('foo-id', tree.path2id('dir/subdir/foo'))
+
 
 class TestShelfManager(tests.TestCaseWithTransport):
 
@@ -723,7 +748,7 @@
         creator.shelve_creation('foo-id')
         shelf_manager = tree.get_shelf_manager()
         shelf_id = shelf_manager.shelve_changes(creator)
-        self.failIfExists('tree/foo')
+        self.assertPathDoesNotExist('tree/foo')
         unshelver = shelf_manager.get_unshelver(shelf_id)
         self.addCleanup(unshelver.finalize)
         unshelver.make_merger().do_merge()

=== modified file 'bzrlib/tests/test_shelf_ui.py'
--- bzrlib/tests/test_shelf_ui.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_shelf_ui.py	2011-10-10 20:55:52 +0000
@@ -27,6 +27,9 @@
     tests,
 )
 from bzrlib.tests import script
+from bzrlib.tests import (
+    features,
+    )
 
 
 class ExpectShelver(shelf_ui.Shelver):
@@ -41,16 +44,18 @@
         self.expected = []
         self.diff_writer = StringIO()
 
-    def expect(self, prompt, response):
-        self.expected.append((prompt, response))
+    def expect(self, message, response):
+        self.expected.append((message, response))
 
-    def prompt(self, message):
+    def prompt(self, message, choices, default):
         try:
-            prompt, response = self.expected.pop(0)
+            expected_message, response = self.expected.pop(0)
         except IndexError:
             raise AssertionError('Unexpected prompt: %s' % message)
-        if prompt != message:
+        if message != expected_message:
             raise AssertionError('Wrong prompt: %s' % message)
+        if choices != '&yes\n&No\n&finish\n&quit':
+            raise AssertionError('Wrong choices: %s' % choices)
         return response
 
 
@@ -63,7 +68,7 @@
 LINES_AY = 'a\nb\nc\nd\ne\nf\ng\nh\ni\ny\n'
 
 
-class TestShelver(tests.TestCaseWithTransport):
+class ShelfTestCase(tests.TestCaseWithTransport):
 
     def create_shelvable_tree(self):
         tree = self.make_branch_and_tree('tree')
@@ -73,6 +78,9 @@
         self.build_tree_contents([('tree/foo', LINES_ZY)])
         return tree
 
+
+class TestShelver(ShelfTestCase):
+
     def test_unexpected_prompt_failure(self):
         tree = self.create_shelvable_tree()
         tree.lock_tree_write()
@@ -80,7 +88,7 @@
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
         e = self.assertRaises(AssertionError, shelver.run)
-        self.assertEqual('Unexpected prompt: Shelve? [yNfq?]', str(e))
+        self.assertEqual('Unexpected prompt: Shelve?', str(e))
 
     def test_wrong_prompt_failure(self):
         tree = self.create_shelvable_tree()
@@ -88,9 +96,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('foo', 'y')
+        shelver.expect('foo', 0)
         e = self.assertRaises(AssertionError, shelver.run)
-        self.assertEqual('Wrong prompt: Shelve? [yNfq?]', str(e))
+        self.assertEqual('Wrong prompt: Shelve?', str(e))
 
     def test_shelve_not_diff(self):
         tree = self.create_shelvable_tree()
@@ -98,8 +106,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'n')
-        shelver.expect('Shelve? [yNfq?]', 'n')
+        shelver.expect('Shelve?', 1)
+        shelver.expect('Shelve?', 1)
         # No final shelving prompt because no changes were selected
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
@@ -110,9 +118,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'n')
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve 2 change(s)?', 1)
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
@@ -122,9 +130,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve 2 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -134,9 +142,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'n')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 1)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AY, 'tree/foo')
 
@@ -147,8 +155,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve binary changes? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve binary changes?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -159,10 +167,10 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve renaming "foo" => "bar"? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve 3 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve renaming "foo" => "bar"?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve 3 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -173,8 +181,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve removing file "foo"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve removing file "foo"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -187,10 +195,10 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve adding file "foo"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve adding file "foo"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
-        self.failIfExists('tree/foo')
+        self.assertPathDoesNotExist('tree/foo')
 
     def test_shelve_kind_change(self):
         tree = self.create_shelvable_tree()
@@ -200,12 +208,12 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve changing "foo" from file to directory? [yNfq?]',
-                       'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve changing "foo" from file to directory?',
+                       0)
+        shelver.expect('Shelve 1 change(s)?', 0)
 
     def test_shelve_modify_target(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.create_shelvable_tree()
         os.symlink('bar', 'tree/baz')
         tree.add('baz', 'baz-id')
@@ -217,8 +225,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
         shelver.expect('Shelve changing target of "baz" from "bar" to '
-                '"vax"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+                '"vax"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertEqual('bar', os.readlink('tree/baz'))
 
@@ -228,8 +236,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'f')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve?', 2)
+        shelver.expect('Shelve 2 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -239,7 +247,7 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'q')
+        shelver.expect('Shelve?', 3)
         self.assertRaises(errors.UserAbort, shelver.run)
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
@@ -261,19 +269,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(), file_list=['bar'])
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve adding file "bar"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
-        shelver.run()
-
-    def test_shelve_help(self):
-        tree = self.create_shelvable_tree()
-        tree.lock_tree_write()
-        self.addCleanup(tree.unlock)
-        shelver = ExpectShelver(tree, tree.basis_tree())
-        self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', '?')
-        shelver.expect('Shelve? [(y)es, (N)o, (f)inish, or (q)uit]', 'f')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve adding file "bar"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
 
     def test_shelve_destroy(self):
@@ -299,19 +296,27 @@
         finally:
             tree.unlock()
 
-    def test_shelve_old_root_deleted(self):
+    def test_shelve_old_root_preserved(self):
         tree1 = self.make_branch_and_tree('tree1')
         tree1.commit('add root')
+        tree1_root_id = tree1.get_root_id()
         tree2 = self.make_branch_and_tree('tree2')
         rev2 = tree2.commit('add root')
+        self.assertNotEquals(tree1_root_id, tree2.get_root_id())
         tree1.merge_from_branch(tree2.branch,
                                 from_revision=revision.NULL_REVISION)
-        tree1.commit('Replaced root entry')
+        tree1.commit('merging in tree2')
+        self.assertEquals(tree1_root_id, tree1.get_root_id())
         # This is essentially assertNotRaises(InconsistentDelta)
-        self.expectFailure('Cannot shelve replacing a root entry',
-                           self.assertRaises, AssertionError,
-                           self.assertRaises, errors.InconsistentDelta,
-                           self.shelve_all, tree1, rev2)
+        # With testtools 0.9.9, it can be rewritten as:
+        # with ExpectedException(AssertionError,
+        #                        'InconsistentDelta not raised'):
+        #     with ExpectedException(errors.InconsistentDelta, ''):
+        #         self.shelve_all(tree1, rev2)
+        e = self.assertRaises(AssertionError, self.assertRaises,
+                              errors.InconsistentDelta, self.shelve_all, tree1,
+                              rev2)
+        self.assertContainsRe('InconsistentDelta not raised', str(e))
 
     def test_shelve_split(self):
         outer_tree = self.make_branch_and_tree('outer')
@@ -327,7 +332,7 @@
                            outer_tree, rev2)
 
 
-class TestApplyReporter(TestShelver):
+class TestApplyReporter(ShelfTestCase):
 
     def test_shelve_not_diff(self):
         tree = self.create_shelvable_tree()
@@ -336,8 +341,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply change? [yNfq?]', 'n')
-        shelver.expect('Apply change? [yNfq?]', 'n')
+        shelver.expect('Apply change?', 1)
+        shelver.expect('Apply change?', 1)
         # No final shelving prompt because no changes were selected
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
@@ -349,9 +354,9 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply 2 change(s)? [yNfq?]', 'n')
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply 2 change(s)?', 1)
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
@@ -362,9 +367,9 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply 2 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -376,8 +381,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply binary changes? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Apply binary changes?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -389,10 +394,10 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Rename "bar" => "foo"? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply 3 change(s)? [yNfq?]', 'y')
+        shelver.expect('Rename "bar" => "foo"?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply 3 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -404,8 +409,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Add file "foo"? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Add file "foo"?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -419,10 +424,10 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Delete file "foo"? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Delete file "foo"?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
-        self.failIfExists('tree/foo')
+        self.assertPathDoesNotExist('tree/foo')
 
     def test_shelve_kind_change(self):
         tree = self.create_shelvable_tree()
@@ -433,11 +438,11 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Change "foo" from directory to a file? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Change "foo" from directory to a file?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
 
     def test_shelve_modify_target(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tree = self.create_shelvable_tree()
         os.symlink('bar', 'tree/baz')
         tree.add('baz', 'baz-id')
@@ -449,9 +454,9 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Change target of "baz" from "vax" to "bar"? [yNfq?]',
-                       'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Change target of "baz" from "vax" to "bar"?',
+                       0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
         self.assertEqual('bar', os.readlink('tree/baz'))
 

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2010-05-13 16:17:54 +0000
+++ bzrlib/tests/test_smart.py	2011-09-16 20:42:31 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,15 +25,11 @@
 """
 
 import bz2
-from cStringIO import StringIO
-import tarfile
 
 from bzrlib import (
-    bencode,
     branch as _mod_branch,
     bzrdir,
     errors,
-    pack,
     tests,
     transport,
     urlutils,
@@ -89,7 +85,7 @@
             backing_transport = tests.TestCaseWithTransport.get_transport(self)
             self._chroot_server = chroot.ChrootServer(backing_transport)
             self.start_server(self._chroot_server)
-        t = transport.get_transport(self._chroot_server.get_url())
+        t = transport.get_transport_from_url(self._chroot_server.get_url())
         if relpath is not None:
             t = t.clone(relpath)
         return t
@@ -103,7 +99,7 @@
         # the default or a parameterized class, but rather use the
         # TestCaseWithTransport infrastructure to set up a smart server and
         # transport.
-        self.transport_server = self.make_transport_server
+        self.overrideAttr(self, "transport_server", self.make_transport_server)
 
     def make_transport_server(self):
         return test_server.SmartTCPServer_for_testing('-' + self.id())
@@ -936,7 +932,7 @@
         # its repository.
         self.make_tree_with_two_commits()
         rev_id_utf8 = u'\xc8'.encode('utf-8')
-        self.tree.branch.set_revision_history([])
+        self.tree.branch.set_last_revision_info(0, 'null:')
         self.assertEqual(
             (0, 'null:'), self.tree.branch.last_revision_info())
         # We can update the branch to a revision that is present in the
@@ -1474,7 +1470,7 @@
             request.execute('stacked', 1, (3, r3)))
 
 
-class TestSmartServerRepositoryGetStream(tests.TestCaseWithMemoryTransport):
+class GetStreamTestBase(tests.TestCaseWithMemoryTransport):
 
     def make_two_commit_repo(self):
         tree = self.make_branch_and_memory_tree('.')
@@ -1486,6 +1482,9 @@
         repo = tree.branch.repository
         return repo, r1, r2
 
+
+class TestSmartServerRepositoryGetStream(GetStreamTestBase):
+
     def test_ancestry_of(self):
         """The search argument may be a 'ancestry-of' some heads'."""
         backing = self.get_transport()
@@ -1512,6 +1511,18 @@
         stream_bytes = ''.join(response.body_stream)
         self.assertStartsWith(stream_bytes, 'Bazaar pack format 1')
 
+    def test_search_everything(self):
+        """A search of 'everything' returns a stream."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryGetStream_1_19(backing)
+        repo, r1, r2 = self.make_two_commit_repo()
+        serialised_fetch_spec = 'everything'
+        request.execute('', repo._format.network_name())
+        response = request.do_body(serialised_fetch_spec)
+        self.assertEqual(('ok',), response.args)
+        stream_bytes = ''.join(response.body_stream)
+        self.assertStartsWith(stream_bytes, 'Bazaar pack format 1')
+
 
 class TestSmartServerRequestHasRevision(tests.TestCaseWithMemoryTransport):
 
@@ -1911,6 +1922,8 @@
             smart_repo.SmartServerRepositoryGetRevisionGraph)
         self.assertHandlerEqual('Repository.get_stream',
             smart_repo.SmartServerRepositoryGetStream)
+        self.assertHandlerEqual('Repository.get_stream_1.19',
+            smart_repo.SmartServerRepositoryGetStream_1_19)
         self.assertHandlerEqual('Repository.has_revision',
             smart_repo.SmartServerRequestHasRevision)
         self.assertHandlerEqual('Repository.insert_stream',
@@ -1927,3 +1940,50 @@
             smart_repo.SmartServerRepositoryUnlock)
         self.assertHandlerEqual('Transport.is_readonly',
             smart_req.SmartServerIsReadonly)
+
+
+class SmartTCPServerHookTests(tests.TestCaseWithMemoryTransport):
+    """Tests for SmartTCPServer hooks."""
+
+    def setUp(self):
+        super(SmartTCPServerHookTests, self).setUp()
+        self.server = server.SmartTCPServer(self.get_transport())
+
+    def test_run_server_started_hooks(self):
+        """Test the server started hooks get fired properly."""
+        started_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_started',
+            lambda backing_urls, url: started_calls.append((backing_urls, url)),
+            None)
+        started_ex_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_started_ex',
+            lambda backing_urls, url: started_ex_calls.append((backing_urls, url)),
+            None)
+        self.server._sockname = ('example.com', 42)
+        self.server.run_server_started_hooks()
+        self.assertEquals(started_calls,
+            [([self.get_transport().base], 'bzr://example.com:42/')])
+        self.assertEquals(started_ex_calls,
+            [([self.get_transport().base], self.server)])
+
+    def test_run_server_started_hooks_ipv6(self):
+        """Test that socknames can contain 4-tuples."""
+        self.server._sockname = ('::', 42, 0, 0)
+        started_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_started',
+            lambda backing_urls, url: started_calls.append((backing_urls, url)),
+            None)
+        self.server.run_server_started_hooks()
+        self.assertEquals(started_calls,
+                [([self.get_transport().base], 'bzr://:::42/')])
+
+    def test_run_server_stopped_hooks(self):
+        """Test the server stopped hooks."""
+        self.server._sockname = ('example.com', 42)
+        stopped_calls = []
+        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+            lambda backing_urls, url: stopped_calls.append((backing_urls, url)),
+            None)
+        self.server.run_server_stopped_hooks()
+        self.assertEquals(stopped_calls,
+            [([self.get_transport().base], 'bzr://example.com:42/')])

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/test_smart_add.py	2011-06-14 02:21:41 +0000
@@ -30,11 +30,11 @@
         # The first part just logs if appropriate
         # Now generate a custom id
         file_id = osutils.safe_file_id(kind + '-'
-                                       + path.raw_path.replace('/', '%'),
+                                       + path.replace('/', '%'),
                                        warn=False)
         if self.should_print:
             self._to_file.write('added %s with id %s\n'
-                                % (path.raw_path, file_id))
+                                % (path, file_id))
         return file_id
 
 
@@ -115,11 +115,11 @@
         self.assertNotEqual(None, c_id)
         self.base_tree.lock_read()
         self.addCleanup(self.base_tree.unlock)
-        self.failIf(c_id in self.base_tree)
+        self.assertFalse(self.base_tree.has_id(c_id))
 
         d_id = new_tree.path2id('subdir/d')
         self.assertNotEqual(None, d_id)
-        self.failIf(d_id in self.base_tree)
+        self.assertFalse(self.base_tree.has_id(d_id))
 
     def test_copy_existing_dir(self):
         self.make_base_tree()
@@ -140,7 +140,7 @@
         self.assertNotEqual(None, a_id)
         self.base_tree.lock_read()
         self.addCleanup(self.base_tree.unlock)
-        self.failIf(a_id in self.base_tree)
+        self.assertFalse(self.base_tree.has_id(a_id))
 
 
 class TestAddActions(tests.TestCase):
@@ -152,11 +152,10 @@
         self.run_action("adding path\n")
 
     def run_action(self, output):
-        from bzrlib.mutabletree import _FastPath
         inv = inventory.Inventory()
         stdout = StringIO()
         action = add.AddAction(to_file=stdout, should_print=bool(output))
 
         self.apply_redirected(None, stdout, None, action, inv, None,
-            _FastPath('path'), 'file')
+            'path', 'file')
         self.assertEqual(stdout.getvalue(), output)

=== modified file 'bzrlib/tests/test_smart_request.py'
--- bzrlib/tests/test_smart_request.py	2010-07-13 19:02:12 +0000
+++ bzrlib/tests/test_smart_request.py	2011-08-19 22:34:02 +0000
@@ -41,6 +41,13 @@
         raise errors.NoSuchFile('xyzzy')
 
 
+class DoUnexpectedErrorRequest(request.SmartServerRequest):
+    """A request that encounters a generic error in self.do()"""
+
+    def do(self):
+        dict()[1]
+
+
 class ChunkErrorRequest(request.SmartServerRequest):
     """A request that raises an error from self.do_chunk()."""
     
@@ -149,6 +156,14 @@
         handler.end_received()
         self.assertResponseIsTranslatedError(handler)
 
+    def test_unexpected_error_translation(self):
+        handler = request.SmartServerRequestHandler(
+            None, {'foo': DoUnexpectedErrorRequest}, '/')
+        handler.args_received(('foo',))
+        self.assertEqual(
+            request.FailedSmartServerResponse(('error', 'KeyError', "1")),
+            handler.response)
+
 
 class TestRequestHanderErrorTranslation(TestCase):
     """Tests for bzrlib.smart.request._translate_error."""
@@ -172,6 +187,23 @@
             ('TokenMismatch', 'some-token', 'actual-token'),
             errors.TokenMismatch('some-token', 'actual-token'))
 
+    def test_MemoryError(self):
+        self.assertTranslationEqual(("MemoryError",), MemoryError())
+
+    def test_generic_Exception(self):
+        self.assertTranslationEqual(('error', 'Exception', ""),
+            Exception())
+
+    def test_generic_BzrError(self):
+        self.assertTranslationEqual(('error', 'BzrError', "some text"),
+            errors.BzrError(msg="some text"))
+
+    def test_generic_zlib_error(self):
+        from zlib import error
+        msg = "Error -3 while decompressing data: incorrect data check"
+        self.assertTranslationEqual(('error', 'zlib.error', msg),
+            error(msg))
+
 
 class TestRequestJail(TestCaseWithMemoryTransport):
 
@@ -208,7 +240,7 @@
         self.assertRaises(errors.JailBreak, _pre_open_hook, t.clone('..'))
         # A completely unrelated transport is not allowed
         self.assertRaises(errors.JailBreak, _pre_open_hook,
-                          transport.get_transport('http://host/'))
+                          transport.get_transport_from_url('http://host/'))
 
     def test_open_bzrdir_in_non_main_thread(self):
         """Opening a bzrdir in a non-main thread should work ok.

=== added file 'bzrlib/tests/test_smart_signals.py'
--- bzrlib/tests/test_smart_signals.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_smart_signals.py	2011-09-26 12:12:40 +0000
@@ -0,0 +1,189 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+import os
+import signal
+import threading
+import weakref
+
+from bzrlib import tests, transport
+from bzrlib.smart import client, medium, server, signals
+
+# Windows doesn't define SIGHUP. And while we could just skip a lot of these
+# tests, we often don't actually care about interaction with 'signal', so we
+# can still run the tests for code coverage.
+SIGHUP = getattr(signal, 'SIGHUP', 1)
+
+
+class TestSignalHandlers(tests.TestCase):
+
+    def setUp(self):
+        super(TestSignalHandlers, self).setUp()
+        # This allows us to mutate the signal handler callbacks, but leave it
+        # 'pristine' after the test case.
+        # TODO: Arguably, this could be put into the base test.TestCase, along
+        #       with a tearDown that asserts that all the entries have been
+        #       removed properly. Global state is always a bit messy. A shame
+        #       that we need it for signal handling.
+        orig = signals._setup_on_hangup_dict()
+        self.assertIs(None, orig)
+        def cleanup():
+            signals._on_sighup = None
+        self.addCleanup(cleanup)
+
+    def test_registered_callback_gets_called(self):
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me)
+        signals._sighup_handler(SIGHUP, None)
+        self.assertEqual(['called'], calls)
+        signals.unregister_on_hangup('myid')
+
+    def test_unregister_not_present(self):
+        # We don't want unregister to fail, since it is generally run at times
+        # that shouldn't interrupt other flow.
+        signals.unregister_on_hangup('no-such-id')
+        log = self.get_log()
+        self.assertContainsRe(log, 'Error occurred during unregister_on_hangup:')
+        self.assertContainsRe(log, '(?s)Traceback.*KeyError')
+
+    def test_failing_callback(self):
+        calls = []
+        def call_me():
+            calls.append('called')
+        def fail_me():
+            raise RuntimeError('something bad happened')
+        signals.register_on_hangup('myid', call_me)
+        signals.register_on_hangup('otherid', fail_me)
+        # _sighup_handler should call both, even though it got an exception
+        signals._sighup_handler(SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+        signals.unregister_on_hangup('otherid')
+        log = self.get_log()
+        self.assertContainsRe(log, '(?s)Traceback.*RuntimeError')
+        self.assertEqual(['called'], calls)
+
+    def test_unregister_during_call(self):
+        # _sighup_handler should handle if some callbacks actually remove
+        # themselves while running.
+        calls = []
+        def call_me_and_unregister():
+            signals.unregister_on_hangup('myid')
+            calls.append('called_and_unregistered')
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me_and_unregister)
+        signals.register_on_hangup('other', call_me)
+        signals._sighup_handler(SIGHUP, None)
+
+    def test_keyboard_interrupt_propagated(self):
+        # In case we get 'stuck' while running a hangup function, we should
+        # not suppress KeyboardInterrupt
+        def call_me_and_raise():
+            raise KeyboardInterrupt()
+        signals.register_on_hangup('myid', call_me_and_raise)
+        self.assertRaises(KeyboardInterrupt,
+                          signals._sighup_handler, SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+
+    def test_weak_references(self):
+        # TODO: This is probably a very-CPython-specific test
+        # Adding yourself to the callback should not make you immortal
+        # We overrideAttr during the test suite, so that we don't pollute the
+        # original dict. However, we can test that what we override matches
+        # what we are putting there.
+        self.assertIsInstance(signals._on_sighup,
+                              weakref.WeakValueDictionary)
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me)
+        del call_me
+        # Non-CPython might want to do a gc.collect() here
+        signals._sighup_handler(SIGHUP, None)
+        self.assertEqual([], calls)
+
+    def test_not_installed(self):
+        # If you haven't called bzrlib.smart.signals.install_sighup_handler,
+        # then _on_sighup should be None, and all the calls become no-ops.
+        signals._on_sighup = None
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', calls)
+        signals._sighup_handler(SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+        log = self.get_log()
+        self.assertEqual('', log)
+
+    def test_install_sighup_handler(self):
+        # install_sighup_handler should set up a signal handler for SIGHUP, as
+        # well as the signals._on_sighup dict.
+        signals._on_sighup = None
+        orig = signals.install_sighup_handler()
+        if getattr(signal, 'SIGHUP', None) is not None:
+            cur = signal.getsignal(SIGHUP)
+            self.assertEqual(signals._sighup_handler, cur)
+        self.assertIsNot(None, signals._on_sighup)
+        signals.restore_sighup_handler(orig)
+        self.assertIs(None, signals._on_sighup)
+
+
+class TestInetServer(tests.TestCase):
+
+    def create_file_pipes(self):
+        r, w = os.pipe()
+        rf = os.fdopen(r, 'rb')
+        wf = os.fdopen(w, 'wb')
+        return rf, wf
+
+    def test_inet_server_responds_to_sighup(self):
+        t = transport.get_transport('memory:///')
+        content = 'a'*1024*1024
+        t.put_bytes('bigfile', content)
+        factory = server.BzrServerFactory()
+        # Override stdin/stdout so that we can inject our own handles
+        client_read, server_write = self.create_file_pipes()
+        server_read, client_write = self.create_file_pipes()
+        factory._get_stdin_stdout = lambda: (server_read, server_write)
+        factory.set_up(t, None, None, inet=True, timeout=4.0)
+        self.addCleanup(factory.tear_down)
+        started = threading.Event()
+        stopped = threading.Event()
+        def serving():
+            started.set()
+            factory.smart_server.serve()
+            stopped.set()
+        server_thread = threading.Thread(target=serving)
+        server_thread.start()
+        started.wait()
+        client_medium = medium.SmartSimplePipesClientMedium(client_read,
+                            client_write, 'base')
+        client_client = client._SmartClient(client_medium)
+        resp, response_handler = client_client.call_expecting_body('get',
+            'bigfile')
+        signals._sighup_handler(SIGHUP, None)
+        self.assertTrue(factory.smart_server.finished)
+        # We can still finish reading the file content, but more than that, and
+        # the file is closed.
+        v = response_handler.read_body_bytes()
+        if v != content:
+            self.fail('Got the wrong content back, expected 1M "a"')
+        stopped.wait()
+        server_thread.join()
+

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2010-06-25 09:56:07 +0000
+++ bzrlib/tests/test_smart_transport.py	2011-10-12 16:00:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,9 +18,14 @@
 
 # all of this deals with byte strings so this is safe
 from cStringIO import StringIO
+import doctest
 import os
 import socket
+import sys
 import threading
+import time
+
+from testtools.matchers import DocTestMatches
 
 import bzrlib
 from bzrlib import (
@@ -28,7 +33,7 @@
         errors,
         osutils,
         tests,
-        transport,
+        transport as _mod_transport,
         urlutils,
         )
 from bzrlib.smart import (
@@ -37,10 +42,14 @@
         message,
         protocol,
         request as _mod_request,
-        server,
+        server as _mod_server,
         vfs,
 )
-from bzrlib.tests import test_smart
+from bzrlib.tests import (
+    features,
+    test_smart,
+    test_server,
+    )
 from bzrlib.transport import (
         http,
         local,
@@ -50,6 +59,21 @@
         )
 
 
+def portable_socket_pair():
+    """Return a pair of TCP sockets connected to each other.
+
+    Unlike socket.socketpair, this should work on Windows.
+    """
+    listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    listen_sock.bind(('127.0.0.1', 0))
+    listen_sock.listen(1)
+    client_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    client_sock.connect(listen_sock.getsockname())
+    server_sock, addr = listen_sock.accept()
+    listen_sock.close()
+    return server_sock, client_sock
+
+
 class StringIOSSHVendor(object):
     """A SSH vendor that uses StringIO to buffer writes and answer reads."""
 
@@ -79,7 +103,7 @@
         return 'pipes', (self.vendor.read_from, self.vendor.write_to)
 
 
-class _InvalidHostnameFeature(tests.Feature):
+class _InvalidHostnameFeature(features.Feature):
     """Does 'non_existent.invalid' fail to resolve?
 
     RFC 2606 states that .invalid is reserved for invalid domain names, and
@@ -612,30 +636,48 @@
 
     def setUp(self):
         super(TestSmartServerStreamMedium, self).setUp()
-        self._captureVar('BZR_NO_SMART_VFS', None)
-
-    def portable_socket_pair(self):
-        """Return a pair of TCP sockets connected to each other.
-
-        Unlike socket.socketpair, this should work on Windows.
-        """
-        listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        listen_sock.bind(('127.0.0.1', 0))
-        listen_sock.listen(1)
-        client_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        client_sock.connect(listen_sock.getsockname())
-        server_sock, addr = listen_sock.accept()
-        listen_sock.close()
-        return server_sock, client_sock
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
+
+    def create_pipe_medium(self, to_server, from_server, transport,
+                           timeout=4.0):
+        """Create a new SmartServerPipeStreamMedium."""
+        return medium.SmartServerPipeStreamMedium(to_server, from_server,
+            transport, timeout=timeout)
+
+    def create_pipe_context(self, to_server_bytes, transport):
+        """Create a SmartServerSocketStreamMedium.
+
+        This differes from create_pipe_medium, in that we initialize the
+        request that is sent to the server, and return the StringIO class that
+        will hold the response.
+        """
+        to_server = StringIO(to_server_bytes)
+        from_server = StringIO()
+        m = self.create_pipe_medium(to_server, from_server, transport)
+        return m, from_server
+
+    def create_socket_medium(self, server_sock, transport, timeout=4.0):
+        """Initialize a new medium.SmartServerSocketStreamMedium."""
+        return medium.SmartServerSocketStreamMedium(server_sock, transport,
+            timeout=timeout)
+
+    def create_socket_context(self, transport, timeout=4.0):
+        """Create a new SmartServerSocketStreamMedium with default context.
+
+        This will call portable_socket_pair and pass the server side to
+        create_socket_medium along with transport.
+        It then returns the client_sock and the server.
+        """
+        server_sock, client_sock = portable_socket_pair()
+        server = self.create_socket_medium(server_sock, transport,
+                                           timeout=timeout)
+        return server, client_sock
 
     def test_smart_query_version(self):
         """Feed a canned query version to a server"""
         # wire-to-wire, using the whole stack
-        to_server = StringIO('hello\n')
-        from_server = StringIO()
         transport = local.LocalTransport(urlutils.local_path_to_url('/'))
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+        server, from_server = self.create_pipe_context('hello\n', transport)
         smart_protocol = protocol.SmartServerRequestProtocolOne(transport,
                 from_server.write)
         server._serve_one_request(smart_protocol)
@@ -645,10 +687,8 @@
     def test_response_to_canned_get(self):
         transport = memory.MemoryTransport('memory:///')
         transport.put_bytes('testfile', 'contents\nof\nfile\n')
-        to_server = StringIO('get\001./testfile\n')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+        server, from_server = self.create_pipe_context('get\001./testfile\n',
+            transport)
         smart_protocol = protocol.SmartServerRequestProtocolOne(transport,
                 from_server.write)
         server._serve_one_request(smart_protocol)
@@ -665,10 +705,8 @@
         # VFS requests use filenames, not raw UTF-8.
         hpss_path = urlutils.escape(utf8_filename)
         transport.put_bytes(utf8_filename, 'contents\nof\nfile\n')
-        to_server = StringIO('get\001' + hpss_path + '\n')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+        server, from_server = self.create_pipe_context(
+                'get\001' + hpss_path + '\n', transport)
         smart_protocol = protocol.SmartServerRequestProtocolOne(transport,
                 from_server.write)
         server._serve_one_request(smart_protocol)
@@ -680,10 +718,8 @@
 
     def test_pipe_like_stream_with_bulk_data(self):
         sample_request_bytes = 'command\n9\nbulk datadone\n'
-        to_server = StringIO(sample_request_bytes)
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, from_server = self.create_pipe_context(
+            sample_request_bytes, None)
         sample_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         server._serve_one_request(sample_protocol)
         self.assertEqual('', from_server.getvalue())
@@ -692,29 +728,23 @@
 
     def test_socket_stream_with_bulk_data(self):
         sample_request_bytes = 'command\n9\nbulk datadone\n'
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         sample_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         client_sock.sendall(sample_request_bytes)
         server._serve_one_request(sample_protocol)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual('', client_sock.recv(1))
         self.assertEqual(sample_request_bytes, sample_protocol.accepted_bytes)
         self.assertFalse(server.finished)
 
     def test_pipe_like_stream_shutdown_detection(self):
-        to_server = StringIO('')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(to_server, from_server, None)
+        server, _ = self.create_pipe_context('', None)
         server._serve_one_request(SampleRequest('x'))
         self.assertTrue(server.finished)
 
     def test_socket_stream_shutdown_detection(self):
-        server_sock, client_sock = self.portable_socket_pair()
+        server, client_sock = self.create_socket_context(None)
         client_sock.close()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
         server._serve_one_request(SampleRequest('x'))
         self.assertTrue(server.finished)
 
@@ -731,14 +761,12 @@
         rest_of_request_bytes = 'lo\n'
         expected_response = (
             protocol.RESPONSE_VERSION_TWO + 'success\nok\x012\n')
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         client_sock.sendall(incomplete_request_bytes)
         server_protocol = server._build_protocol()
         client_sock.sendall(rest_of_request_bytes)
         server._serve_one_request(server_protocol)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual(expected_response, osutils.recv_all(client_sock, 50),
                          "Not a version 2 response to 'hello' request.")
         self.assertEqual('', client_sock.recv(1))
@@ -763,8 +791,7 @@
         to_server_w = os.fdopen(to_server_w, 'w', 0)
         from_server_r = os.fdopen(from_server_r, 'r', 0)
         from_server = os.fdopen(from_server, 'w', 0)
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server = self.create_pipe_medium(to_server, from_server, None)
         # Like test_socket_stream_incomplete_request, write an incomplete
         # request (that does not end in '\n') and build a protocol from it.
         to_server_w.write(incomplete_request_bytes)
@@ -785,10 +812,8 @@
         # _serve_one_request should still process both of them as if they had
         # been received separately.
         sample_request_bytes = 'command\n'
-        to_server = StringIO(sample_request_bytes * 2)
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, from_server = self.create_pipe_context(
+            sample_request_bytes * 2, None)
         first_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         server._serve_one_request(first_protocol)
         self.assertEqual(0, first_protocol.next_read_size())
@@ -807,9 +832,7 @@
         # _serve_one_request should still process both of them as if they had
         # been received separately.
         sample_request_bytes = 'command\n'
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         first_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         # Put two whole requests on the wire.
         client_sock.sendall(sample_request_bytes * 2)
@@ -822,7 +845,7 @@
         stream_still_open = server._serve_one_request(second_protocol)
         self.assertEqual(sample_request_bytes, second_protocol.accepted_bytes)
         self.assertFalse(server.finished)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual('', client_sock.recv(1))
 
     def test_pipe_like_stream_error_handling(self):
@@ -835,7 +858,7 @@
         def close():
             self.closed = True
         from_server.close = close
-        server = medium.SmartServerPipeStreamMedium(
+        server = self.create_pipe_medium(
             to_server, from_server, None)
         fake_protocol = ErrorRaisingProtocol(Exception('boom'))
         server._serve_one_request(fake_protocol)
@@ -844,9 +867,7 @@
         self.assertTrue(server.finished)
 
     def test_socket_stream_error_handling(self):
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         fake_protocol = ErrorRaisingProtocol(Exception('boom'))
         server._serve_one_request(fake_protocol)
         # recv should not block, because the other end of the socket has been
@@ -855,36 +876,26 @@
         self.assertTrue(server.finished)
 
     def test_pipe_like_stream_keyboard_interrupt_handling(self):
-        to_server = StringIO('')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, from_server = self.create_pipe_context('', None)
         fake_protocol = ErrorRaisingProtocol(KeyboardInterrupt('boom'))
         self.assertRaises(
             KeyboardInterrupt, server._serve_one_request, fake_protocol)
         self.assertEqual('', from_server.getvalue())
 
     def test_socket_stream_keyboard_interrupt_handling(self):
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         fake_protocol = ErrorRaisingProtocol(KeyboardInterrupt('boom'))
         self.assertRaises(
             KeyboardInterrupt, server._serve_one_request, fake_protocol)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual('', client_sock.recv(1))
 
     def build_protocol_pipe_like(self, bytes):
-        to_server = StringIO(bytes)
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, _ = self.create_pipe_context(bytes, None)
         return server._build_protocol()
 
     def build_protocol_socket(self, bytes):
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         client_sock.sendall(bytes)
         client_sock.close()
         return server._build_protocol()
@@ -930,6 +941,108 @@
         server_protocol = self.build_protocol_socket('bzr request 2\n')
         self.assertProtocolTwo(server_protocol)
 
+    def test__build_protocol_returns_if_stopping(self):
+        # _build_protocol should notice that we are stopping, and return
+        # without waiting for bytes from the client.
+        server, client_sock = self.create_socket_context(None)
+        server._stop_gracefully()
+        self.assertIs(None, server._build_protocol())
+
+    def test_socket_set_timeout(self):
+        server, _ = self.create_socket_context(None, timeout=1.23)
+        self.assertEqual(1.23, server._client_timeout)
+
+    def test_pipe_set_timeout(self):
+        server = self.create_pipe_medium(None, None, None,
+            timeout=1.23)
+        self.assertEqual(1.23, server._client_timeout)
+
+    def test_socket_wait_for_bytes_with_timeout_with_data(self):
+        server, client_sock = self.create_socket_context(None)
+        client_sock.sendall('data\n')
+        # This should not block or consume any actual content
+        self.assertFalse(server._wait_for_bytes_with_timeout(0.1))
+        data = server.read_bytes(5)
+        self.assertEqual('data\n', data)
+
+    def test_socket_wait_for_bytes_with_timeout_no_data(self):
+        server, client_sock = self.create_socket_context(None)
+        # This should timeout quickly, reporting that there wasn't any data
+        self.assertRaises(errors.ConnectionTimeout,
+                          server._wait_for_bytes_with_timeout, 0.01)
+        client_sock.close()
+        data = server.read_bytes(1)
+        self.assertEqual('', data)
+
+    def test_socket_wait_for_bytes_with_timeout_closed(self):
+        server, client_sock = self.create_socket_context(None)
+        # With the socket closed, this should return right away.
+        # It seems select.select() returns that you *can* read on the socket,
+        # even though it closed. Presumably as a way to tell it is closed?
+        # Testing shows that without sock.close() this times-out failing the
+        # test, but with it, it returns False immediately.
+        client_sock.close()
+        self.assertFalse(server._wait_for_bytes_with_timeout(10))
+        data = server.read_bytes(1)
+        self.assertEqual('', data)
+
+    def test_socket_wait_for_bytes_with_shutdown(self):
+        server, client_sock = self.create_socket_context(None)
+        t = time.time()
+        # Override the _timer functionality, so that time never increments,
+        # this way, we can be sure we stopped because of the flag, and not
+        # because of a timeout, etc.
+        server._timer = lambda: t
+        server._client_poll_timeout = 0.1
+        server._stop_gracefully()
+        server._wait_for_bytes_with_timeout(1.0)
+
+    def test_socket_serve_timeout_closes_socket(self):
+        server, client_sock = self.create_socket_context(None, timeout=0.1)
+        # This should timeout quickly, and then close the connection so that
+        # client_sock recv doesn't block.
+        server.serve()
+        self.assertEqual('', client_sock.recv(1))
+
+    def test_pipe_wait_for_bytes_with_timeout_with_data(self):
+        # We intentionally use a real pipe here, so that we can 'select' on it.
+        # You can't select() on a StringIO
+        (r_server, w_client) = os.pipe()
+        self.addCleanup(os.close, w_client)
+        with os.fdopen(r_server, 'rb') as rf_server:
+            server = self.create_pipe_medium(
+                rf_server, None, None)
+            os.write(w_client, 'data\n')
+            # This should not block or consume any actual content
+            server._wait_for_bytes_with_timeout(0.1)
+            data = server.read_bytes(5)
+            self.assertEqual('data\n', data)
+
+    def test_pipe_wait_for_bytes_with_timeout_no_data(self):
+        # We intentionally use a real pipe here, so that we can 'select' on it.
+        # You can't select() on a StringIO
+        (r_server, w_client) = os.pipe()
+        # We can't add an os.close cleanup here, because we need to control
+        # when the file handle gets closed ourselves.
+        with os.fdopen(r_server, 'rb') as rf_server:
+            server = self.create_pipe_medium(
+                rf_server, None, None)
+            if sys.platform == 'win32':
+                # Windows cannot select() on a pipe, so we just always return
+                server._wait_for_bytes_with_timeout(0.01)
+            else:
+                self.assertRaises(errors.ConnectionTimeout,
+                                  server._wait_for_bytes_with_timeout, 0.01)
+            os.close(w_client)
+            data = server.read_bytes(5)
+            self.assertEqual('', data)
+
+    def test_pipe_wait_for_bytes_no_fileno(self):
+        server, _ = self.create_pipe_context('', None)
+        # Our file doesn't support polling, so we should always just return
+        # 'you have data to consume.
+        server._wait_for_bytes_with_timeout(0.01)
+
 
 class TestGetProtocolFactoryForBytes(tests.TestCase):
     """_get_protocol_factory_for_bytes identifies the protocol factory a server
@@ -965,31 +1078,229 @@
 
 class TestSmartTCPServer(tests.TestCase):
 
+    def make_server(self):
+        """Create a SmartTCPServer that we can exercise.
+
+        Note: we don't use SmartTCPServer_for_testing because the testing
+        version overrides lots of functionality like 'serve', and we want to
+        test the raw service.
+
+        This will start the server in another thread, and wait for it to
+        indicate it has finished starting up.
+
+        :return: (server, server_thread)
+        """
+        t = _mod_transport.get_transport_from_url('memory:///')
+        server = _mod_server.SmartTCPServer(t, client_timeout=4.0)
+        server._ACCEPT_TIMEOUT = 0.1
+        # We don't use 'localhost' because that might be an IPv6 address.
+        server.start_server('127.0.0.1', 0)
+        server_thread = threading.Thread(target=server.serve,
+                                         args=(self.id(),))
+        server_thread.start()
+        # Ensure this gets called at some point
+        self.addCleanup(server._stop_gracefully)
+        server._started.wait()
+        return server, server_thread
+
+    def ensure_client_disconnected(self, client_sock):
+        """Ensure that a socket is closed, discarding all errors."""
+        try:
+            client_sock.close()
+        except Exception:
+            pass
+
+    def connect_to_server(self, server):
+        """Create a client socket that can talk to the server."""
+        client_sock = socket.socket()
+        server_info = server._server_socket.getsockname()
+        client_sock.connect(server_info)
+        self.addCleanup(self.ensure_client_disconnected, client_sock)
+        return client_sock
+
+    def connect_to_server_and_hangup(self, server):
+        """Connect to the server, and then hang up.
+        That way it doesn't sit waiting for 'accept()' to timeout.
+        """
+        # If the server has already signaled that the socket is closed, we
+        # don't need to try to connect to it. Not being set, though, the server
+        # might still close the socket while we try to connect to it. So we
+        # still have to catch the exception.
+        if server._stopped.isSet():
+            return
+        try:
+            client_sock = self.connect_to_server(server)
+            client_sock.close()
+        except socket.error, e:
+            # If the server has hung up already, that is fine.
+            pass
+
+    def say_hello(self, client_sock):
+        """Send the 'hello' smart RPC, and expect the response."""
+        client_sock.send('hello\n')
+        self.assertEqual('ok\x012\n', client_sock.recv(5))
+
+    def shutdown_server_cleanly(self, server, server_thread):
+        server._stop_gracefully()
+        self.connect_to_server_and_hangup(server)
+        server._stopped.wait()
+        server._fully_stopped.wait()
+        server_thread.join()
+
     def test_get_error_unexpected(self):
         """Error reported by server with no specific representation"""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         class FlakyTransport(object):
             base = 'a_url'
             def external_url(self):
                 return self.base
-            def get_bytes(self, path):
+            def get(self, path):
                 raise Exception("some random exception from inside server")
-        smart_server = server.SmartTCPServer(backing_transport=FlakyTransport())
-        smart_server.start_background_thread('-' + self.id())
-        try:
-            transport = remote.RemoteTCPTransport(smart_server.get_url())
-            err = self.assertRaises(errors.UnknownErrorFromSmartServer,
-                transport.get, 'something')
-            self.assertContainsRe(str(err), 'some random exception')
-            transport.disconnect()
-        finally:
-            smart_server.stop_background_thread()
+
+        class FlakyServer(test_server.SmartTCPServer_for_testing):
+            def get_backing_transport(self, backing_transport_server):
+                return FlakyTransport()
+
+        smart_server = FlakyServer()
+        smart_server.start_server()
+        self.addCleanup(smart_server.stop_server)
+        t = remote.RemoteTCPTransport(smart_server.get_url())
+        self.addCleanup(t.disconnect)
+        err = self.assertRaises(errors.UnknownErrorFromSmartServer,
+                                t.get, 'something')
+        self.assertContainsRe(str(err), 'some random exception')
+
+    def test_propagates_timeout(self):
+        server = _mod_server.SmartTCPServer(None, client_timeout=1.23)
+        server_sock, client_sock = portable_socket_pair()
+        handler = server._make_handler(server_sock)
+        self.assertEqual(1.23, handler._client_timeout)
+
+    def test_serve_conn_tracks_connections(self):
+        server = _mod_server.SmartTCPServer(None, client_timeout=4.0)
+        server_sock, client_sock = portable_socket_pair()
+        server.serve_conn(server_sock, '-%s' % (self.id(),))
+        self.assertEqual(1, len(server._active_connections))
+        # We still want to talk on the connection. Polling should indicate it
+        # is still active.
+        server._poll_active_connections()
+        self.assertEqual(1, len(server._active_connections))
+        # Closing the socket will end the active thread, and polling will
+        # notice and remove it from the active set.
+        client_sock.close()
+        server._poll_active_connections(0.1)
+        self.assertEqual(0, len(server._active_connections))
+
+    def test_serve_closes_out_finished_connections(self):
+        server, server_thread = self.make_server()
+        # The server is started, connect to it.
+        client_sock = self.connect_to_server(server)
+        # We send and receive on the connection, so that we know the
+        # server-side has seen the connect, and started handling the
+        # results.
+        self.say_hello(client_sock)
+        self.assertEqual(1, len(server._active_connections))
+        # Grab a handle to the thread that is processing our request
+        _, server_side_thread = server._active_connections[0]
+        # Close the connection, ask the server to stop, and wait for the
+        # server to stop, as well as the thread that was servicing the
+        # client request.
+        client_sock.close()
+        # Wait for the server-side request thread to notice we are closed.
+        server_side_thread.join()
+        # Stop the server, it should notice the connection has finished.
+        self.shutdown_server_cleanly(server, server_thread)
+        # The server should have noticed that all clients are gone before
+        # exiting.
+        self.assertEqual(0, len(server._active_connections))
+
+    def test_serve_reaps_finished_connections(self):
+        server, server_thread = self.make_server()
+        client_sock1 = self.connect_to_server(server)
+        # We send and receive on the connection, so that we know the
+        # server-side has seen the connect, and started handling the
+        # results.
+        self.say_hello(client_sock1)
+        server_handler1, server_side_thread1 = server._active_connections[0]
+        client_sock1.close()
+        server_side_thread1.join()
+        # By waiting until the first connection is fully done, the server
+        # should notice after another connection that the first has finished.
+        client_sock2 = self.connect_to_server(server)
+        self.say_hello(client_sock2)
+        server_handler2, server_side_thread2 = server._active_connections[-1]
+        # There is a race condition. We know that client_sock2 has been
+        # registered, but not that _poll_active_connections has been called. We
+        # know that it will be called before the server will accept a new
+        # connection, however. So connect one more time, and assert that we
+        # either have 1 or 2 active connections (never 3), and that the 'first'
+        # connection is not connection 1
+        client_sock3 = self.connect_to_server(server)
+        self.say_hello(client_sock3)
+        # Copy the list, so we don't have it mutating behind our back
+        conns = list(server._active_connections)
+        self.assertEqual(2, len(conns))
+        self.assertNotEqual((server_handler1, server_side_thread1), conns[0])
+        self.assertEqual((server_handler2, server_side_thread2), conns[0])
+        client_sock2.close()
+        client_sock3.close()
+        self.shutdown_server_cleanly(server, server_thread)
+
+    def test_graceful_shutdown_waits_for_clients_to_stop(self):
+        server, server_thread = self.make_server()
+        # We need something big enough that it won't fit in a single recv. So
+        # the server thread gets blocked writing content to the client until we
+        # finish reading on the client.
+        server.backing_transport.put_bytes('bigfile',
+            'a'*1024*1024)
+        client_sock = self.connect_to_server(server)
+        self.say_hello(client_sock)
+        _, server_side_thread = server._active_connections[0]
+        # Start the RPC, but don't finish reading the response
+        client_medium = medium.SmartClientAlreadyConnectedSocketMedium(
+            'base', client_sock)
+        client_client = client._SmartClient(client_medium)
+        resp, response_handler = client_client.call_expecting_body('get',
+            'bigfile')
+        self.assertEqual(('ok',), resp)
+        # Ask the server to stop gracefully, and wait for it.
+        server._stop_gracefully()
+        self.connect_to_server_and_hangup(server)
+        server._stopped.wait()
+        # It should not be accepting another connection.
+        self.assertRaises(socket.error, self.connect_to_server, server)
+        # It should also not be fully stopped
+        server._fully_stopped.wait(0.01)
+        self.assertFalse(server._fully_stopped.isSet())
+        response_handler.read_body_bytes()
+        client_sock.close()
+        server_side_thread.join()
+        server_thread.join()
+        self.assertTrue(server._fully_stopped.isSet())
+        log = self.get_log()
+        self.assertThat(log, DocTestMatches("""\
+    INFO  Requested to stop gracefully
+... Stopping SmartServerSocketStreamMedium(client=('127.0.0.1', ...
+    INFO  Waiting for 1 client(s) to finish
+""", flags=doctest.ELLIPSIS|doctest.REPORT_UDIFF))
+
+    def test_stop_gracefully_tells_handlers_to_stop(self):
+        server, server_thread = self.make_server()
+        client_sock = self.connect_to_server(server)
+        self.say_hello(client_sock)
+        server_handler, server_side_thread = server._active_connections[0]
+        self.assertFalse(server_handler.finished)
+        server._stop_gracefully()
+        self.assertTrue(server_handler.finished)
+        client_sock.close()
+        self.connect_to_server_and_hangup(server)
+        server_thread.join()
 
 
 class SmartTCPTests(tests.TestCase):
     """Tests for connection/end to end behaviour using the TCP server.
 
-    All of these tests are run with a server running on another thread serving
+    All of these tests are run with a server running in another thread serving
     a MemoryTransport, and a connection to it already open.
 
     the server is obtained by calling self.start_server(readonly=False).
@@ -1003,55 +1314,60 @@
         # NB: Tests using this fall into two categories: tests of the server,
         # tests wanting a server. The latter should be updated to use
         # self.vfs_transport_factory etc.
-        if not backing_transport:
+        if backing_transport is None:
             mem_server = memory.MemoryServer()
             mem_server.start_server()
             self.addCleanup(mem_server.stop_server)
             self.permit_url(mem_server.get_url())
-            self.backing_transport = transport.get_transport(
+            self.backing_transport = _mod_transport.get_transport_from_url(
                 mem_server.get_url())
         else:
             self.backing_transport = backing_transport
         if readonly:
             self.real_backing_transport = self.backing_transport
-            self.backing_transport = transport.get_transport(
+            self.backing_transport = _mod_transport.get_transport_from_url(
                 "readonly+" + self.backing_transport.abspath('.'))
-        self.server = server.SmartTCPServer(self.backing_transport)
+        self.server = _mod_server.SmartTCPServer(self.backing_transport,
+                                                 client_timeout=4.0)
+        self.server.start_server('127.0.0.1', 0)
         self.server.start_background_thread('-' + self.id())
         self.transport = remote.RemoteTCPTransport(self.server.get_url())
-        self.addCleanup(self.tearDownServer)
+        self.addCleanup(self.stop_server)
         self.permit_url(self.server.get_url())
 
-    def tearDownServer(self):
+    def stop_server(self):
+        """Disconnect the client and stop the server.
+
+        This must be re-entrant as some tests will call it explicitly in
+        addition to the normal cleanup.
+        """
         if getattr(self, 'transport', None):
             self.transport.disconnect()
             del self.transport
         if getattr(self, 'server', None):
             self.server.stop_background_thread()
-            # XXX: why not .stop_server() -- mbp 20100106
             del self.server
 
 
 class TestServerSocketUsage(SmartTCPTests):
 
-    def test_server_setup_teardown(self):
-        """It should be safe to teardown the server with no requests."""
+    def test_server_start_stop(self):
+        """It should be safe to stop the server with no requests."""
         self.start_server()
-        server = self.server
-        transport = remote.RemoteTCPTransport(self.server.get_url())
-        self.tearDownServer()
-        self.assertRaises(errors.ConnectionError, transport.has, '.')
+        t = remote.RemoteTCPTransport(self.server.get_url())
+        self.stop_server()
+        self.assertRaises(errors.ConnectionError, t.has, '.')
 
     def test_server_closes_listening_sock_on_shutdown_after_request(self):
         """The server should close its listening socket when it's stopped."""
         self.start_server()
-        server = self.server
+        server_url = self.server.get_url()
         self.transport.has('.')
-        self.tearDownServer()
+        self.stop_server()
         # if the listening socket has closed, we should get a BADFD error
         # when connecting, rather than a hang.
-        transport = remote.RemoteTCPTransport(server.get_url())
-        self.assertRaises(errors.ConnectionError, transport.has, '.')
+        t = remote.RemoteTCPTransport(server_url)
+        self.assertRaises(errors.ConnectionError, t.has, '.')
 
 
 class WritableEndToEndTests(SmartTCPTests):
@@ -1067,14 +1383,14 @@
 
     def test_smart_transport_has(self):
         """Checking for file existence over smart."""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.backing_transport.put_bytes("foo", "contents of foo\n")
         self.assertTrue(self.transport.has("foo"))
         self.assertFalse(self.transport.has("non-foo"))
 
     def test_smart_transport_get(self):
         """Read back a file over smart."""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.backing_transport.put_bytes("foo", "contents\nof\nfoo\n")
         fp = self.transport.get("foo")
         self.assertEqual('contents\nof\nfoo\n', fp.read())
@@ -1084,7 +1400,7 @@
         # The path in a raised NoSuchFile exception should be the precise path
         # asked for by the client. This gives meaningful and unsurprising errors
         # for users.
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         err = self.assertRaises(
             errors.NoSuchFile, self.transport.get, 'not%20a%20file')
         self.assertSubset([err.path], ['not%20a%20file', './not%20a%20file'])
@@ -1109,7 +1425,7 @@
 
     def test_open_dir(self):
         """Test changing directory"""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         transport = self.transport
         self.backing_transport.mkdir('toffee')
         self.backing_transport.mkdir('toffee/apple')
@@ -1137,7 +1453,7 @@
 
     def test_mkdir_error_readonly(self):
         """TransportNotPossible should be preserved from the backing transport."""
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         self.start_server(readonly=True)
         self.assertRaises(errors.TransportNotPossible, self.transport.mkdir,
             'foo')
@@ -1152,7 +1468,7 @@
     def test_server_started_hook_memory(self):
         """The server_started hook fires when the server is started."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_started',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_started',
             self.capture_server_call, None)
         self.start_server()
         # at this point, the server will be starting a thread up.
@@ -1166,9 +1482,10 @@
     def test_server_started_hook_file(self):
         """The server_started hook fires when the server is started."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_started',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_started',
             self.capture_server_call, None)
-        self.start_server(backing_transport=transport.get_transport("."))
+        self.start_server(
+            backing_transport=_mod_transport.get_transport_from_path("."))
         # at this point, the server will be starting a thread up.
         # there is no indicator at the moment, so bodge it by doing a request.
         self.transport.has('.')
@@ -1182,7 +1499,7 @@
     def test_server_stopped_hook_simple_memory(self):
         """The server_stopped hook fires when the server is stopped."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_stopped',
             self.capture_server_call, None)
         self.start_server()
         result = [([self.backing_transport.base], self.transport.base)]
@@ -1192,16 +1509,17 @@
         self.transport.has('.')
         self.assertEqual([], self.hook_calls)
         # clean up the server
-        self.tearDownServer()
+        self.stop_server()
         # now it should have fired.
         self.assertEqual(result, self.hook_calls)
 
     def test_server_stopped_hook_simple_file(self):
         """The server_stopped hook fires when the server is stopped."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_stopped',
             self.capture_server_call, None)
-        self.start_server(backing_transport=transport.get_transport("."))
+        self.start_server(
+            backing_transport=_mod_transport.get_transport_from_path("."))
         result = [(
             [self.backing_transport.base, self.backing_transport.external_url()]
             , self.transport.base)]
@@ -1211,7 +1529,7 @@
         self.transport.has('.')
         self.assertEqual([], self.hook_calls)
         # clean up the server
-        self.tearDownServer()
+        self.stop_server()
         # now it should have fired.
         self.assertEqual(result, self.hook_calls)
 
@@ -1251,7 +1569,7 @@
 
     def setUp(self):
         super(SmartServerRequestHandlerTests, self).setUp()
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def build_handler(self, transport):
         """Returns a handler for the commands in protocol version one."""
@@ -1276,10 +1594,7 @@
         handler = vfs.HasRequest(None, '/')
         # set environment variable after construction to make sure it's
         # examined.
-        # Note that we can safely clobber BZR_NO_SMART_VFS here, because setUp
-        # has called _captureVar, so it will be restored to the right state
-        # afterwards.
-        os.environ['BZR_NO_SMART_VFS'] = ''
+        self.overrideEnv('BZR_NO_SMART_VFS', '')
         self.assertRaises(errors.DisabledMethod, handler.execute)
 
     def test_readonly_exception_becomes_transport_not_possible(self):
@@ -1346,13 +1661,13 @@
 class RemoteTransportRegistration(tests.TestCase):
 
     def test_registration(self):
-        t = transport.get_transport('bzr+ssh://example.com/path')
+        t = _mod_transport.get_transport_from_url('bzr+ssh://example.com/path')
         self.assertIsInstance(t, remote.RemoteSSHTransport)
-        self.assertEqual('example.com', t._host)
+        self.assertEqual('example.com', t._parsed_url.host)
 
     def test_bzr_https(self):
         # https://bugs.launchpad.net/bzr/+bug/128456
-        t = transport.get_transport('bzr+https://example.com/path')
+        t = _mod_transport.get_transport_from_url('bzr+https://example.com/path')
         self.assertIsInstance(t, remote.RemoteHTTPTransport)
         self.assertStartsWith(
             t._http_transport.base,
@@ -1486,6 +1801,7 @@
         smart_protocol._has_dispatched = True
         smart_protocol.request = _mod_request.SmartServerRequestHandler(
             None, _mod_request.request_handlers, '/')
+        # GZ 2010-08-10: Cycle with closure affects 4 tests
         class FakeCommand(_mod_request.SmartServerRequest):
             def do_body(self_cmd, body_bytes):
                 self.end_received = True
@@ -1592,7 +1908,7 @@
         self.assertTrue(self.end_received)
 
     def test_accept_request_and_body_all_at_once(self):
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         mem_transport = memory.MemoryTransport()
         mem_transport.put_bytes('foo', 'abcdefghij')
         out_stream = StringIO()
@@ -1858,7 +2174,7 @@
         self.assertTrue(self.end_received)
 
     def test_accept_request_and_body_all_at_once(self):
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
         mem_transport = memory.MemoryTransport()
         mem_transport.put_bytes('foo', 'abcdefghij')
         out_stream = StringIO()
@@ -2405,7 +2721,7 @@
         self.assertEqual('aaa', stream.next())
         self.assertEqual('bbb', stream.next())
         exc = self.assertRaises(errors.ErrorFromSmartServer, stream.next)
-        self.assertEqual(('error', 'Boom!'), exc.error_tuple)
+        self.assertEqual(('error', 'Exception', 'Boom!'), exc.error_tuple)
 
     def test_interrupted_by_connection_lost(self):
         interrupted_body_stream = (
@@ -2534,7 +2850,7 @@
         from_server = StringIO()
         transport = memory.MemoryTransport('memory:///')
         server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+            to_server, from_server, transport, timeout=4.0)
         proto = server._build_protocol()
         message_handler = proto.message_handler
         server._serve_one_request(proto)
@@ -2808,7 +3124,8 @@
     'b\x00\x00\x00\x03aaa' # body part ('aaa')
     'b\x00\x00\x00\x03bbb' # body part ('bbb')
     'oE' # status flag (error)
-    's\x00\x00\x00\x10l5:error5:Boom!e' # err struct ('error', 'Boom!')
+    # err struct ('error', 'Exception', 'Boom!')
+    's\x00\x00\x00\x1bl5:error9:Exception5:Boom!e'
     'e' # EOM
     )
 
@@ -3507,7 +3824,7 @@
     def setUp(self):
         super(HTTPTunnellingSmokeTest, self).setUp()
         # We use the VFS layer as part of HTTP tunnelling tests.
-        self._captureVar('BZR_NO_SMART_VFS', None)
+        self.overrideEnv('BZR_NO_SMART_VFS', None)
 
     def test_smart_http_medium_request_accept_bytes(self):
         medium = FakeHTTPMedium()
@@ -3548,7 +3865,7 @@
         # still work correctly.
         base_transport = remote.RemoteHTTPTransport('bzr+http://host/%7Ea/b')
         new_transport = base_transport.clone('c')
-        self.assertEqual('bzr+http://host/~a/b/c/', new_transport.base)
+        self.assertEqual('bzr+http://host/%7Ea/b/c/', new_transport.base)
         self.assertEqual(
             'c/',
             new_transport._client.remote_path_from_transport(new_transport))
@@ -3571,8 +3888,8 @@
         r = t._redirected_to('http://www.example.com/foo',
                              'http://www.example.com/bar')
         self.assertEquals(type(r), type(t))
-        self.assertEquals('joe', t._user)
-        self.assertEquals(t._user, r._user)
+        self.assertEquals('joe', t._parsed_url.user)
+        self.assertEquals(t._parsed_url.user, r._parsed_url.user)
 
     def test_redirected_to_same_host_different_protocol(self):
         t = remote.RemoteHTTPTransport('bzr+http://joe@www.example.com/foo')

=== modified file 'bzrlib/tests/test_smtp_connection.py'
--- bzrlib/tests/test_smtp_connection.py	2009-07-19 01:36:56 +0000
+++ bzrlib/tests/test_smtp_connection.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007, 2009 Canonical Ltd
+# Copyright (C) 2007, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
 from email.Message import Message
 import errno
 import smtplib
@@ -91,9 +90,7 @@
 class TestSMTPConnection(tests.TestCaseInTempDir):
 
     def get_connection(self, text, smtp_factory=None):
-        my_config = config.GlobalConfig()
-        config_file = StringIO(text)
-        my_config._get_parser(config_file)
+        my_config = config.GlobalConfig.from_string(text)
         return smtp_connection.SMTPConnection(my_config,
                                               _smtp_factory=smtp_factory)
 
@@ -160,17 +157,18 @@
 
     def test_authenticate_with_byte_strings(self):
         user = 'joe'
-        password = 'h\xC3\xACspass'
+        unicode_pass = u'h\xECspass'
+        utf8_pass = unicode_pass.encode('utf-8')
         factory = WideOpenSMTPFactory()
         conn = self.get_connection(
-            '[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
-            % (user, password), smtp_factory=factory)
-        self.assertEqual(u'h\xECspass', conn._smtp_password)
+            u'[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
+            % (user, unicode_pass), smtp_factory=factory)
+        self.assertEqual(unicode_pass, conn._smtp_password)
         conn._connect()
         self.assertEqual([('connect', 'localhost'),
                           ('ehlo',),
                           ('has_extn', 'starttls'),
-                          ('login', user, password)], factory._calls)
+                          ('login', user, utf8_pass)], factory._calls)
         smtp_username, smtp_password = factory._calls[-1][1:]
         self.assertIsInstance(smtp_username, str)
         self.assertIsInstance(smtp_password, str)

=== modified file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	2010-07-05 16:01:33 +0000
+++ bzrlib/tests/test_source.py	2011-06-13 22:10:04 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 They are useful for testing code quality, checking coverage metric etc.
 """
 
-# import system imports here
 import os
 import parser
 import re
@@ -27,7 +26,6 @@
 import sys
 import token
 
-#import bzrlib specific imports here
 from bzrlib import (
     osutils,
     )
@@ -113,8 +111,9 @@
 
         # Avoid the case when bzrlib is packaged in a zip file
         if not os.path.isdir(source_dir):
-            raise TestSkipped('Cannot find bzrlib source directory. Expected %s'
-                              % source_dir)
+            raise TestSkipped(
+                'Cannot find bzrlib source directory. Expected %s'
+                % source_dir)
         return source_dir
 
     def get_source_files(self, extensions=None):
@@ -154,7 +153,7 @@
             yield fname, text
 
     def is_our_code(self, fname):
-        """Return true if it's a "real" part of bzrlib rather than external code"""
+        """True if it's a "real" part of bzrlib rather than external code"""
         if '/util/' in fname or '/plugins/' in fname:
             return False
         else:
@@ -194,10 +193,9 @@
 
         copyright_re = re.compile('#\\s*copyright.*(?=\n)', re.I)
         copyright_canonical_re = re.compile(
-            r'# Copyright \(C\) ' # Opening "# Copyright (C)"
-            r'(\d+)(, \d+)*' # Followed by a series of dates
-            r'.*Canonical Ltd' # And containing 'Canonical Ltd'
-            )
+            r'# Copyright \(C\) '  # Opening "# Copyright (C)"
+            r'(\d+)(, \d+)*'       # followed by a series of dates
+            r'.*Canonical Ltd')    # and containing 'Canonical Ltd'.
 
         for fname, text in self.get_source_file_contents(
                 extensions=('.py', '.pyx')):
@@ -230,7 +228,7 @@
                         ]
             for fname, comment in incorrect:
                 help_text.append(fname)
-                help_text.append((' '*4) + comment)
+                help_text.append((' ' * 4) + comment)
 
             self.fail('\n'.join(help_text))
 
@@ -269,10 +267,9 @@
                          " LICENSE_EXCEPTIONS in"
                          " bzrlib/tests/test_source.py",
                          "Or add the following text to the beginning:",
-                         gpl_txt
-                        ]
+                         gpl_txt]
             for fname in incorrect:
-                help_text.append((' '*4) + fname)
+                help_text.append((' ' * 4) + fname)
 
             self.fail('\n'.join(help_text))
 
@@ -283,7 +280,7 @@
             dict_[fname].append(line_no)
 
     def _format_message(self, dict_, message):
-        files = ["%s: %s" % (f, ', '.join([str(i+1) for i in lines]))
+        files = ["%s: %s" % (f, ', '.join([str(i + 1) for i in lines]))
                 for f, lines in dict_.items()]
         files.sort()
         return message + '\n\n    %s' % ('\n    '.join(files))
@@ -291,19 +288,16 @@
     def test_coding_style(self):
         """Check if bazaar code conforms to some coding style conventions.
 
-        Currently we assert that the following is not present:
-         * any tab characters
-         * non-unix newlines
-         * no newline at end of files
+        Generally we expect PEP8, but we do not generally strictly enforce
+        this, and there are existing files that do not comply.  The 'pep8'
+        tool, available separately, will check for more cases.
 
-        Print how many files have
-         * trailing white space
-         * lines longer than 79 chars
+        This test only enforces conditions that are globally true at the
+        moment, and that should cause a patch to be rejected: spaces rather
+        than tabs, unix newlines, and a newline at the end of the file.
         """
         tabs = {}
-        trailing_ws = {}
         illegal_newlines = {}
-        long_lines = {}
         no_newline_at_eof = []
         for fname, text in self.get_source_file_contents(
                 extensions=('.py', '.pyx')):
@@ -315,12 +309,8 @@
                 if '\t' in line:
                     self._push_file(tabs, fname, line_no)
                 if not line.endswith('\n') or line.endswith('\r\n'):
-                    if line_no != last_line_no: # not no_newline_at_eof
+                    if line_no != last_line_no:  # not no_newline_at_eof
                         self._push_file(illegal_newlines, fname, line_no)
-                if line.endswith(' \n'):
-                    self._push_file(trailing_ws, fname, line_no)
-                if len(line) > 80:
-                    self._push_file(long_lines, fname, line_no)
             if not lines[-1].endswith('\n'):
                 no_newline_at_eof.append(fname)
         problems = []
@@ -328,17 +318,9 @@
             problems.append(self._format_message(tabs,
                 'Tab characters were found in the following source files.'
                 '\nThey should either be replaced by "\\t" or by spaces:'))
-        if trailing_ws:
-            print ("There are %i lines with trailing white space in %i files."
-                % (sum([len(lines) for f, lines in trailing_ws.items()]),
-                    len(trailing_ws)))
         if illegal_newlines:
             problems.append(self._format_message(illegal_newlines,
                 'Non-unix newlines were found in the following source files:'))
-        if long_lines:
-            print ("There are %i lines longer than 79 characters in %i files."
-                % (sum([len(lines) for f, lines in long_lines.items()]),
-                    len(long_lines)))
         if no_newline_at_eof:
             no_newline_at_eof.sort()
             problems.append("The following source files doesn't have a "
@@ -382,24 +364,27 @@
     def test_extension_exceptions(self):
         """Extension functions should propagate exceptions.
 
-        Either they should return an object, have an 'except' clause, or have a
-        "# cannot_raise" to indicate that we've audited them and defined them as not
-        raising exceptions.
+        Either they should return an object, have an 'except' clause, or
+        have a "# cannot_raise" to indicate that we've audited them and
+        defined them as not raising exceptions.
         """
         both_exc_and_no_exc = []
         missing_except = []
-        class_re = re.compile(r'^(cdef\s+)?(public\s+)?(api\s+)?class (\w+).*:',
-                              re.MULTILINE)
-        except_re = re.compile(r'cdef\s+' # start with cdef
-                               r'([\w *]*?)\s*' # this is the return signature
-                               r'(\w+)\s*\(' # the function name
-                               r'[^)]*\)\s*' # parameters
-                               r'(.*)\s*:' # the except clause
-                               r'\s*(#\s*cannot[- _]raise)?' # cannot raise comment
-                              )
+        class_re = re.compile(r'^(cdef\s+)?(public\s+)?'
+                              r'(api\s+)?class (\w+).*:', re.MULTILINE)
+        extern_class_re = re.compile(r'## extern cdef class (\w+)',
+                                     re.MULTILINE)
+        except_re = re.compile(
+            r'cdef\s+'        # start with cdef
+            r'([\w *]*?)\s*'  # this is the return signature
+            r'(\w+)\s*\('     # the function name
+            r'[^)]*\)\s*'     # parameters
+            r'(.*)\s*:'       # the except clause
+            r'\s*(#\s*cannot[- _]raise)?')  # cannot raise comment
         for fname, text in self.get_source_file_contents(
                 extensions=('.pyx',)):
             known_classes = set([m[-1] for m in class_re.findall(text)])
+            known_classes.update(extern_class_re.findall(text))
             cdefs = except_re.findall(text)
             for sig, func, exc_clause, no_exc_comment in cdefs:
                 if sig.startswith('api '):
@@ -414,15 +399,18 @@
                     missing_except.append((fname, func))
         error_msg = []
         if both_exc_and_no_exc:
-            error_msg.append('The following functions had "cannot raise" comments'
-                             ' but did have an except clause set:')
+            error_msg.append(
+                'The following functions had "cannot raise" comments'
+                ' but did have an except clause set:')
             for fname, func in both_exc_and_no_exc:
                 error_msg.append('%s:%s' % (fname, func))
             error_msg.extend(('', ''))
         if missing_except:
-            error_msg.append('The following functions have fixed return types,'
-                             ' but no except clause.')
-            error_msg.append('Either add an except or append "# cannot_raise".')
+            error_msg.append(
+                'The following functions have fixed return types,'
+                ' but no except clause.')
+            error_msg.append(
+                'Either add an except or append "# cannot_raise".')
             for fname, func in missing_except:
                 error_msg.append('%s:%s' % (fname, func))
             error_msg.extend(('', ''))

=== modified file 'bzrlib/tests/test_ssh_transport.py'
--- bzrlib/tests/test_ssh_transport.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_ssh_transport.py	2010-10-07 12:45:51 +0000
@@ -20,6 +20,7 @@
     OpenSSHSubprocessVendor,
     PLinkSubprocessVendor,
     SSHCorpSubprocessVendor,
+    LSHSubprocessVendor,
     SSHVendorManager,
     )
 
@@ -72,6 +73,12 @@
         manager.set_ssh_version_string("SSH Secure Shell")
         self.assertIsInstance(manager.get_vendor({}), SSHCorpSubprocessVendor)
 
+    def test_get_vendor_by_inspection_lsh(self):
+        manager = TestSSHVendorManager()
+        self.assertRaises(SSHVendorNotFound, manager.get_vendor, {})
+        manager.set_ssh_version_string("lsh")
+        self.assertIsInstance(manager.get_vendor({}), LSHSubprocessVendor)
+
     def test_get_vendor_by_inspection_plink(self):
         manager = TestSSHVendorManager()
         self.assertRaises(SSHVendorNotFound, manager.get_vendor, {})
@@ -160,7 +167,7 @@
             vendor._get_vendor_specific_argv(
                 "user", "host", 100, command=["bzr"]),
             ["ssh", "-oForwardX11=no", "-oForwardAgent=no",
-                "-oClearAllForwardings=yes", "-oProtocol=2",
+                "-oClearAllForwardings=yes",
                 "-oNoHostAuthenticationForLocalhost=yes",
                 "-p", "100",
                 "-l", "user",
@@ -173,7 +180,7 @@
             vendor._get_vendor_specific_argv(
                 "user", "host", 100, subsystem="sftp"),
             ["ssh", "-oForwardX11=no", "-oForwardAgent=no",
-                "-oClearAllForwardings=yes", "-oProtocol=2",
+                "-oClearAllForwardings=yes",
                 "-oNoHostAuthenticationForLocalhost=yes",
                 "-p", "100",
                 "-l", "user",
@@ -202,6 +209,28 @@
                 "-s", "sftp", "host"]
             )
 
+    def test_lsh_command_arguments(self):
+        vendor = LSHSubprocessVendor()
+        self.assertEqual(
+            vendor._get_vendor_specific_argv(
+                "user", "host", 100, command=["bzr"]),
+            ["lsh",
+                "-p", "100",
+                "-l", "user",
+                "host", "bzr"]
+            )
+
+    def test_lsh_subsystem_arguments(self):
+        vendor = LSHSubprocessVendor()
+        self.assertEqual(
+            vendor._get_vendor_specific_argv(
+                "user", "host", 100, subsystem="sftp"),
+            ["lsh",
+                "-p", "100",
+                "-l", "user",
+                "--subsystem", "sftp", "host"]
+            )
+
     def test_plink_command_arguments(self):
         vendor = PLinkSubprocessVendor()
         self.assertEqual(

=== modified file 'bzrlib/tests/test_status.py'
--- bzrlib/tests/test_status.py	2009-10-06 14:40:37 +0000
+++ bzrlib/tests/test_status.py	2011-03-30 11:45:54 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,10 @@
 
 from StringIO import StringIO
 
-from bzrlib import config
+from bzrlib import (
+    config,
+    status as _mod_status,
+    )
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.status import show_pending_merges, show_tree_status
 from bzrlib.tests import TestCaseWithTransport
@@ -130,3 +133,67 @@
                      revision=[RevisionSpec.from_string("revid:%s" % r1_id),
                                RevisionSpec.from_string("revid:%s" % r2_id)])
         # return does not matter as long as it did not raise.
+
+
+class TestHooks(TestCaseWithTransport):
+
+    def test_constructor(self):
+        """Check that creating a StatusHooks instance has the right defaults.
+        """
+        hooks = _mod_status.StatusHooks()
+        self.assertTrue("post_status" in hooks, "post_status not in %s" % hooks)
+        self.assertTrue("pre_status" in hooks, "pre_status not in %s" % hooks)
+
+    def test_installed_hooks_are_StatusHooks(self):
+        """The installed hooks object should be a StatusHooks.
+        """
+        # the installed hooks are saved in self._preserved_hooks.
+        self.assertIsInstance(self._preserved_hooks[_mod_status][1],
+            _mod_status.StatusHooks)
+
+    def test_post_status_hook(self):
+        """Ensure that post_status hook is invoked with the right args.
+        """
+        calls = []
+        _mod_status.hooks.install_named_hook('post_status', calls.append, None)
+        self.assertLength(0, calls)
+        tree = self.make_branch_and_tree('.')
+        r1_id = tree.commit('one', allow_pointless=True)
+        r2_id = tree.commit('two', allow_pointless=True)
+        r2_tree = tree.branch.repository.revision_tree(r2_id)
+        output = StringIO()
+        show_tree_status(tree, to_file=output,
+            revision=[RevisionSpec.from_string("revid:%s" % r1_id),
+                RevisionSpec.from_string("revid:%s" % r2_id)])
+        self.assertLength(1, calls)
+        params = calls[0]
+        self.assertIsInstance(params, _mod_status.StatusHookParams)
+        attrs = ['old_tree', 'new_tree', 'to_file', 'versioned',
+            'show_ids', 'short', 'verbose']
+        for a in attrs:
+            self.assertTrue(hasattr(params, a),
+                'Attribute "%s" not found in StatusHookParam' % a)
+
+    def test_pre_status_hook(self):
+        """Ensure that pre_status hook is invoked with the right args.
+        """
+        calls = []
+        _mod_status.hooks.install_named_hook('pre_status', calls.append, None)
+        self.assertLength(0, calls)
+        tree = self.make_branch_and_tree('.')
+        r1_id = tree.commit('one', allow_pointless=True)
+        r2_id = tree.commit('two', allow_pointless=True)
+        r2_tree = tree.branch.repository.revision_tree(r2_id)
+        output = StringIO()
+        show_tree_status(tree, to_file=output,
+            revision=[RevisionSpec.from_string("revid:%s" % r1_id),
+                RevisionSpec.from_string("revid:%s" % r2_id)])
+        self.assertLength(1, calls)
+        params = calls[0]
+        self.assertIsInstance(params, _mod_status.StatusHookParams)
+        attrs = ['old_tree', 'new_tree', 'to_file', 'versioned',
+            'show_ids', 'short', 'verbose']
+        for a in attrs:
+            self.assertTrue(hasattr(params, a),
+                'Attribute "%s" not found in StatusHookParam' % a)
+

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_store.py	2011-08-19 22:34:02 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2007 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,15 +21,15 @@
 import gzip
 
 import bzrlib.errors as errors
-from bzrlib.errors import BzrError, UnlistableStore, NoSuchFile
-from bzrlib.transport.local import LocalTransport
+from bzrlib.errors import BzrError
+from bzrlib.store import TransportStore
 from bzrlib.store.text import TextStore
+from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.tests import TestCase, TestCaseInTempDir, TestCaseWithTransport
-import bzrlib.store as store
-import bzrlib.store.versioned
 import bzrlib.transactions as transactions
 import bzrlib.transport as transport
 from bzrlib.transport.memory import MemoryTransport
+from bzrlib.weave import WeaveFile
 
 
 class TestStores(object):
@@ -82,7 +82,7 @@
 class TestCompressedTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.'):
-        t = transport.get_transport(path)
+        t = transport.get_transport_from_path(path)
         return TextStore(t, compressed=True)
 
     def test_total_size(self):
@@ -116,7 +116,7 @@
 
     def test_missing_is_absent(self):
         store = self.get_store()
-        self.failIf('aa' in store)
+        self.assertFalse('aa' in store)
 
     def test_adding_fails_when_present(self):
         my_store = self.get_store()
@@ -139,7 +139,7 @@
 class TestTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.'):
-        t = transport.get_transport(path)
+        t = transport.get_transport_from_path(path)
         return TextStore(t, compressed=False)
 
     def test_total_size(self):
@@ -157,7 +157,7 @@
 class TestMixedTextStore(TestCaseInTempDir, TestStores):
 
     def get_store(self, path=u'.', compressed=True):
-        t = transport.get_transport(path)
+        t = transport.get_transport_from_path(path)
         return TextStore(t, compressed=compressed)
 
     def test_get_mixed(self):
@@ -165,8 +165,8 @@
         s = self.get_store(u'.', compressed=False)
         cs.add(StringIO('hello there'), 'a')
 
-        self.failUnlessExists('a.gz')
-        self.failIf(os.path.lexists('a'))
+        self.assertPathExists('a.gz')
+        self.assertFalse(os.path.lexists('a'))
 
         self.assertEquals(gzip.GzipFile('a.gz').read(), 'hello there')
 
@@ -178,8 +178,8 @@
         self.assertRaises(BzrError, s.add, StringIO('goodbye'), 'a')
 
         s.add(StringIO('goodbye'), 'b')
-        self.failUnlessExists('b')
-        self.failIf(os.path.lexists('b.gz'))
+        self.assertPathExists('b')
+        self.assertFalse(os.path.lexists('b.gz'))
         self.assertEquals(open('b').read(), 'goodbye')
 
         self.assertEquals(cs.has_id('b'), True)
@@ -204,7 +204,7 @@
         return
 
 
-class InstrumentedTransportStore(store.TransportStore):
+class InstrumentedTransportStore(TransportStore):
     """An instrumented TransportStore.
 
     Here we replace template method worker methods with calls that record the
@@ -230,7 +230,7 @@
 class TestMockTransport(TestCase):
 
     def test_isinstance(self):
-        self.failUnless(isinstance(MockTransport(), transport.Transport))
+        self.assertIsInstance(MockTransport(), transport.Transport)
 
     def test_has(self):
         self.assertEqual(False, MockTransport().has('foo'))
@@ -242,37 +242,37 @@
 class TestTransportStore(TestCase):
 
     def test__relpath_invalid(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertRaises(ValueError, my_store._relpath, '/foo')
         self.assertRaises(ValueError, my_store._relpath, 'foo/')
 
     def test_register_invalid_suffixes(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertRaises(ValueError, my_store.register_suffix, '/')
         self.assertRaises(ValueError, my_store.register_suffix, '.gz/bar')
 
     def test__relpath_unregister_suffixes(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertRaises(ValueError, my_store._relpath, 'foo', ['gz'])
         self.assertRaises(ValueError, my_store._relpath, 'foo', ['dsc', 'gz'])
 
     def test__relpath_simple(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         self.assertEqual("foo", my_store._relpath('foo'))
 
     def test__relpath_prefixed(self):
-        my_store = store.TransportStore(MockTransport(), True)
+        my_store = TransportStore(MockTransport(), True)
         self.assertEqual('45/foo', my_store._relpath('foo'))
 
     def test__relpath_simple_suffixed(self):
-        my_store = store.TransportStore(MockTransport())
+        my_store = TransportStore(MockTransport())
         my_store.register_suffix('bar')
         my_store.register_suffix('baz')
         self.assertEqual('foo.baz', my_store._relpath('foo', ['baz']))
         self.assertEqual('foo.bar.baz', my_store._relpath('foo', ['bar', 'baz']))
 
     def test__relpath_prefixed_suffixed(self):
-        my_store = store.TransportStore(MockTransport(), True)
+        my_store = TransportStore(MockTransport(), True)
         my_store.register_suffix('bar')
         my_store.register_suffix('baz')
         self.assertEqual('45/foo.baz', my_store._relpath('foo', ['baz']))
@@ -394,12 +394,12 @@
         self.assertRaises(KeyError, to_store.get, 'missing', 'sig')
 
     def test_relpath_escaped(self):
-        my_store = store.TransportStore(MemoryTransport())
+        my_store = TransportStore(MemoryTransport())
         self.assertEqual('%25', my_store._relpath('%'))
 
     def test_escaped_uppercase(self):
         """Uppercase letters are escaped for safety on Windows"""
-        my_store = store.TransportStore(MemoryTransport(), prefixed=True,
+        my_store = TransportStore(MemoryTransport(), prefixed=True,
             escaped=True)
         # a particularly perverse file-id! :-)
         self.assertEquals(my_store._relpath('C:<>'), 'be/%2543%253a%253c%253e')
@@ -412,7 +412,8 @@
 
     def setUp(self):
         super(TestVersionFileStore, self).setUp()
-        self.vfstore = store.versioned.VersionedFileStore(MemoryTransport())
+        self.vfstore = VersionedFileStore(MemoryTransport(),
+            versionedfile_class=WeaveFile)
         self.vfstore.get_scope = self.get_scope
         self._transaction = None
 
@@ -437,8 +438,8 @@
         self.assertRaises(errors.ReadOnlyError, vf.add_lines, 'b', [], [])
 
     def test___iter__escaped(self):
-        self.vfstore = store.versioned.VersionedFileStore(MemoryTransport(),
-            prefixed=True, escaped=True)
+        self.vfstore = VersionedFileStore(MemoryTransport(),
+            prefixed=True, escaped=True, versionedfile_class=WeaveFile)
         self.vfstore.get_scope = self.get_scope
         self._transaction = transactions.WriteTransaction()
         vf = self.vfstore.get_weave_or_empty(' ', self._transaction)

=== modified file 'bzrlib/tests/test_strace.py'
--- bzrlib/tests/test_strace.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_strace.py	2011-08-04 00:17:53 +0000
@@ -1,5 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
-#   Authors: Robert Collins <robert.collins@canonical.com>
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,38 +16,31 @@
 
 """Tests for the strace-invoking support."""
 
-import errno
-import subprocess
 import threading
 
 from bzrlib import (
+    strace,
     tests,
     )
-from bzrlib.strace import StraceFeature, strace_detailed, StraceResult
-
-
-class TestStraceFeature(tests.TestCaseWithTransport):
-
-    def test_strace_detection(self):
-        """Strace is available if its runnable."""
-        try:
-            proc = subprocess.Popen(['strace'],
-                stderr=subprocess.PIPE,
-                stdout=subprocess.PIPE)
-            proc.communicate()
-            found_strace = True
-        except OSError, e:
-            if e.errno == errno.ENOENT:
-                # strace is not installed
-                found_strace = False
-            else:
-                raise
-        self.assertEqual(found_strace, StraceFeature.available())
+from bzrlib.strace import strace_detailed, StraceResult
+from bzrlib.tests.features import (
+    strace_feature,
+    )
 
 
 class TestStrace(tests.TestCaseWithTransport):
 
-    _test_needs_features = [StraceFeature]
+    _test_needs_features = [strace_feature]
+
+    def setUp(self):
+        # NB: see http://pad.lv/626679 and
+        # <https://code.launchpad.net/~mbp/bzr/626679-strace/+merge/34157>:
+        # testing strace by connecting to ourselves has repeatedly caused
+        # hangs in running the test suite; these are fixable given enough
+        # determination but given that strace is not used by any other tests
+        # at the moment and that it's only test-support code, we just leave it
+        # untested -- mbp 20100901
+        raise tests.TestSkipped("strace selftests are broken and disabled")
 
     def _check_threads(self):
         # For bug #226769, it was decided that the strace tests should not be
@@ -58,17 +50,29 @@
         # restriction.
         active = threading.activeCount()
         if active > 1: # There is always the main thread at least
-            raise tests.KnownFailure(
+            self.knownFailure(
                 '%d active threads, bug #103133 needs to be fixed.' % active)
 
+    def strace_detailed_or_skip(self, *args, **kwargs):
+        """Run strace, but cope if it's not allowed"""
+        try:
+            return strace_detailed(*args, **kwargs)
+        except strace.StraceError, e:
+            if e.err_messages.startswith(
+                    "attach: ptrace(PTRACE_ATTACH, ...): Operation not permitted"):
+                raise tests.TestSkipped("ptrace not permitted")
+            else:
+                raise
+
     def test_strace_callable_is_called(self):
         self._check_threads()
 
         output = []
         def function(positional, *args, **kwargs):
             output.append((positional, args, kwargs))
-        strace_detailed(function, ["a", "b"], {"c": "c"},
-                        follow_children=False)
+        self.strace_detailed_or_skip(
+            function, ["a", "b"], {"c": "c"},
+            follow_children=False)
         self.assertEqual([("a", ("b",), {"c":"c"})], output)
 
     def test_strace_callable_result(self):
@@ -76,7 +80,7 @@
 
         def function():
             return "foo"
-        result, strace_result = strace_detailed(function,[], {},
+        result, strace_result = self.strace_detailed_or_skip(function,[], {},
                                                 follow_children=False)
         self.assertEqual("foo", result)
         self.assertIsInstance(strace_result, StraceResult)
@@ -87,6 +91,6 @@
 
         def function():
             self.build_tree(['myfile'])
-        unused, result = strace_detailed(function, [], {},
+        unused, result = self.strace_detailed_or_skip(function, [], {},
                                          follow_children=False)
         self.assertContainsRe(result.raw_log, 'myfile')

=== modified file 'bzrlib/tests/test_subsume.py'
--- bzrlib/tests/test_subsume.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_subsume.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from bzrlib import bzrdir, errors, repository, workingtree, tests
+from bzrlib import errors, workingtree, tests
 
 
 class TestWorkingTree(tests.TestCaseWithTransport):
@@ -64,14 +64,14 @@
         # this test checks the subdir is removed, so it needs to know the
         # control directory; that changes rarely so just hardcode (and check)
         # it is correct.
-        self.failUnlessExists('tree/subtree/.bzr')
+        self.assertPathExists('tree/subtree/.bzr')
         base_tree.subsume(sub_tree)
         self.assertEqual(['tree-1', 'subtree-1'], base_tree.get_parent_ids())
         self.assertEqual(sub_root_id, base_tree.path2id('subtree'))
         self.assertEqual('file2-id', base_tree.path2id('subtree/file2'))
         # subsuming the tree removes the control directory, so you can't open
         # it.
-        self.failIfExists('tree/subtree/.bzr')
+        self.assertPathDoesNotExist('tree/subtree/.bzr')
         file2 = open('tree/subtree/file2', 'rb')
         try:
             file2_contents = file2.read()
@@ -91,9 +91,9 @@
         self.assertEqualDiff(file2_contents,
                              basis_tree.get_file_text('file2-id'))
         self.assertEqual('subtree-1',
-                         basis_tree.inventory['file2-id'].revision)
+                         basis_tree.get_file_revision('file2-id'))
         self.assertEqual('combined-1',
-                         basis_tree.inventory[sub_root_id].revision)
+                         basis_tree.get_file_revision(sub_root_id))
 
     def test_subsume_failure(self):
         base_tree, sub_tree = self.make_trees()

=== modified file 'bzrlib/tests/test_switch.py'
--- bzrlib/tests/test_switch.py	2010-05-03 04:08:50 +0000
+++ bzrlib/tests/test_switch.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
     merge as _mod_merge,
     switch,
     tests,
-)
+    )
 
 
 class TestSwitch(tests.TestCaseWithTransport):
@@ -53,12 +53,12 @@
             lightweight=self.lightweight)
         self.build_tree(['checkout/file-3'])
         checkout.add('file-3')
-        self.failIfExists('checkout/file-1')
-        self.failUnlessExists('checkout/file-2')
+        self.assertPathDoesNotExist('checkout/file-1')
+        self.assertPathExists('checkout/file-2')
         switch.switch(checkout.bzrdir, to_branch)
-        self.failUnlessExists('checkout/file-1')
-        self.failIfExists('checkout/file-2')
-        self.failUnlessExists('checkout/file-3')
+        self.assertPathExists('checkout/file-1')
+        self.assertPathDoesNotExist('checkout/file-2')
+        self.assertPathExists('checkout/file-3')
 
     def test_switch_after_branch_moved(self):
         """Test switch after the branch is moved."""
@@ -84,9 +84,9 @@
                 'Unable to connect to current master branch .*'
                 'To switch anyway, use --force.')
         switch.switch(checkout.bzrdir, to_branch, force=True)
-        self.failIfExists('checkout/file-1')
-        self.failUnlessExists('checkout/file-2')
-        self.failUnlessExists('checkout/file-3')
+        self.assertPathDoesNotExist('checkout/file-1')
+        self.assertPathExists('checkout/file-2')
+        self.assertPathExists('checkout/file-3')
 
     def test_switch_when_pending_merges(self):
         """Test graceful failure if pending merges are outstanding."""
@@ -120,8 +120,8 @@
         checkout = tree.branch.create_checkout('checkout',
             lightweight=self.lightweight)
         switch.switch(checkout.bzrdir, tree.branch, revision_id="rev1")
-        self.failUnlessExists('checkout/file-1')
-        self.failIfExists('checkout/file-2')
+        self.assertPathExists('checkout/file-1')
+        self.assertPathDoesNotExist('checkout/file-2')
 
     def test_switch_changing_root_id(self):
         tree = self._setup_tree()
@@ -179,13 +179,13 @@
         self.assertContainsRe(str(err),
             'Cannot switch as local commits found in the checkout.')
         # Check all is ok when force is given
-        self.failIfExists('checkout/file-1')
-        self.failUnlessExists('checkout/file-2')
+        self.assertPathDoesNotExist('checkout/file-1')
+        self.assertPathExists('checkout/file-2')
         switch.switch(checkout.bzrdir, to_branch, force=True)
-        self.failUnlessExists('checkout/file-1')
-        self.failIfExists('checkout/file-2')
-        self.failIfExists('checkout/file-3')
-        self.failUnlessExists('checkout/file-4')
+        self.assertPathExists('checkout/file-1')
+        self.assertPathDoesNotExist('checkout/file-2')
+        self.assertPathDoesNotExist('checkout/file-3')
+        self.assertPathExists('checkout/file-4')
         # Check that the checkout is a true mirror of the bound branch
         self.assertEqual(to_branch.last_revision_info(),
                          checkout.branch.last_revision_info())

=== modified file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	2010-06-25 06:11:21 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 from bzrlib.symbol_versioning import (
     deprecated_function,
     deprecated_in,
-    deprecated_list,
     deprecated_method,
     )
 from bzrlib.tests import TestCase

=== modified file 'bzrlib/tests/test_tag.py'
--- bzrlib/tests/test_tag.py	2010-03-25 09:39:03 +0000
+++ bzrlib/tests/test_tag.py	2011-09-22 13:26:14 +0000
@@ -26,7 +26,6 @@
     DisabledTags,
     )
 from bzrlib.tests import (
-    KnownFailure,
     TestCase,
     TestCaseWithTransport,
     )
@@ -110,16 +109,21 @@
         self.assertRaises(errors.NoSuchTag, a.tags.lookup_tag, 'tag-2')
         # conflicting merge
         a.tags.set_tag('tag-2', 'z')
-        conflicts = a.tags.merge_to(b.tags)
-        self.assertEqual(conflicts, [('tag-2', 'z', 'y')])
+        updates, conflicts = a.tags.merge_to(b.tags)
+        self.assertEqual({}, updates)
+        self.assertEqual(list(conflicts), [('tag-2', 'z', 'y')])
         self.assertEqual('y', b.tags.lookup_tag('tag-2'))
         # overwrite conflicts
-        conflicts = a.tags.merge_to(b.tags, overwrite=True)
-        self.assertEqual(conflicts, [])
+        updates, conflicts = a.tags.merge_to(b.tags, overwrite=True)
+        self.assertEqual(list(conflicts), [])
+        self.assertEqual({u'tag-2': 'z'}, updates)
         self.assertEqual('z', b.tags.lookup_tag('tag-2'))
 
 
 class TestTagsInCheckouts(TestCaseWithTransport):
+    """Tests for how tags are synchronised between the master and child branch
+    of a checkout.
+    """
 
     def test_update_tag_into_checkout(self):
         # checkouts are directly connected to the tags of their master branch:
@@ -165,7 +169,7 @@
         child.update()
         # and deletion of tags should also propagate
         master.tags.delete_tag('foo')
-        raise KnownFailure("tag deletion does not propagate: "
+        self.knownFailure("tag deletion does not propagate: "
             "https://bugs.launchpad.net/bzr/+bug/138802")
         self.assertRaises(errors.NoSuchTag,
             child.tags.lookup_tag, 'foo')

=== added file 'bzrlib/tests/test_test_server.py'
--- bzrlib/tests/test_test_server.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_test_server.py	2011-10-10 13:59:22 +0000
@@ -0,0 +1,339 @@
+# Copyright (C) 2010, 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import errno
+import socket
+import SocketServer
+import threading
+
+
+from bzrlib import (
+    osutils,
+    tests,
+    )
+from bzrlib.tests import test_server
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+load_tests = load_tests_apply_scenarios
+
+
+def portable_socket_pair():
+    """Return a pair of TCP sockets connected to each other.
+
+    Unlike socket.socketpair, this should work on Windows.
+    """
+    listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    listen_sock.bind(('127.0.0.1', 0))
+    listen_sock.listen(1)
+    client_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    client_sock.connect(listen_sock.getsockname())
+    server_sock, addr = listen_sock.accept()
+    listen_sock.close()
+    return server_sock, client_sock
+
+
+class TCPClient(object):
+
+    def __init__(self):
+        self.sock = None
+
+    def connect(self, addr):
+        if self.sock is not None:
+            raise AssertionError('Already connected to %r'
+                                 % (self.sock.getsockname(),))
+        self.sock = osutils.connect_socket(addr)
+
+    def disconnect(self):
+        if self.sock is not None:
+            try:
+                self.sock.shutdown(socket.SHUT_RDWR)
+                self.sock.close()
+            except socket.error, e:
+                if e[0] in (errno.EBADF, errno.ENOTCONN):
+                    # Right, the socket is already down
+                    pass
+                else:
+                    raise
+            self.sock = None
+
+    def write(self, s):
+        return self.sock.sendall(s)
+
+    def read(self, bufsize=4096):
+        return self.sock.recv(bufsize)
+
+
+class TCPConnectionHandler(SocketServer.BaseRequestHandler):
+
+    def handle(self):
+        self.done = False
+        self.handle_connection()
+        while not self.done:
+            self.handle_connection()
+
+    def readline(self):
+        # TODO: We should be buffering any extra data sent, etc. However, in
+        #       practice, we don't send extra content, so we haven't bothered
+        #       to implement it yet.
+        req = self.request.recv(4096)
+        # An empty string is allowed, to indicate the end of the connection
+        if not req or (req.endswith('\n') and req.count('\n') == 1):
+            return req
+        raise ValueError('[%r] not a simple line' % (req,))
+
+    def handle_connection(self):
+        req = self.readline()
+        if not req:
+            self.done = True
+        elif req == 'ping\n':
+            self.request.sendall('pong\n')
+        else:
+            raise ValueError('[%s] not understood' % req)
+
+
+class TestTCPServerInAThread(tests.TestCase):
+
+    scenarios = [
+        (name, {'server_class': getattr(test_server, name)})
+        for name in
+        ('TestingTCPServer', 'TestingThreadingTCPServer')]
+
+    def get_server(self, server_class=None, connection_handler_class=None):
+        if server_class is not None:
+            self.server_class = server_class
+        if connection_handler_class is None:
+            connection_handler_class = TCPConnectionHandler
+        server = test_server.TestingTCPServerInAThread(
+            ('localhost', 0), self.server_class, connection_handler_class)
+        server.start_server()
+        self.addCleanup(server.stop_server)
+        return server
+
+    def get_client(self):
+        client = TCPClient()
+        self.addCleanup(client.disconnect)
+        return client
+
+    def get_server_connection(self, server, conn_rank):
+        return server.server.clients[conn_rank]
+
+    def assertClientAddr(self, client, server, conn_rank):
+        conn = self.get_server_connection(server, conn_rank)
+        self.assertEquals(client.sock.getsockname(), conn[1])
+
+    def test_start_stop(self):
+        server = self.get_server()
+        client = self.get_client()
+        server.stop_server()
+        # since the server doesn't accept connections anymore attempting to
+        # connect should fail
+        client = self.get_client()
+        self.assertRaises(socket.error,
+                          client.connect, (server.host, server.port))
+
+    def test_client_talks_server_respond(self):
+        server = self.get_server()
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        self.assertIs(None, client.write('ping\n'))
+        resp = client.read()
+        self.assertClientAddr(client, server, 0)
+        self.assertEquals('pong\n', resp)
+
+    def test_server_fails_to_start(self):
+        class CantStart(Exception):
+            pass
+
+        class CantStartServer(test_server.TestingTCPServer):
+
+            def server_bind(self):
+                raise CantStart()
+
+        # The exception is raised in the main thread
+        self.assertRaises(CantStart,
+                          self.get_server, server_class=CantStartServer)
+
+    def test_server_fails_while_serving_or_stopping(self):
+        class CantConnect(Exception):
+            pass
+
+        class FailingConnectionHandler(TCPConnectionHandler):
+
+            def handle(self):
+                raise CantConnect()
+
+        server = self.get_server(
+            connection_handler_class=FailingConnectionHandler)
+        # The server won't fail until a client connect
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        # We make sure the server wants to handle a request, but the request is
+        # guaranteed to fail. However, the server should make sure that the
+        # connection gets closed, and stop_server should then raise the
+        # original exception.
+        client.write('ping\n')
+        try:
+            self.assertEqual('', client.read())
+        except socket.error, e:
+            # On Windows, failing during 'handle' means we get
+            # 'forced-close-of-connection'. Possibly because we haven't
+            # processed the write request before we close the socket.
+            WSAECONNRESET = 10054
+            if e.errno in (WSAECONNRESET,):
+                pass
+        # Now the server has raised the exception in its own thread
+        self.assertRaises(CantConnect, server.stop_server)
+
+    def test_server_crash_while_responding(self):
+        # We want to ensure the exception has been caught
+        caught = threading.Event()
+        caught.clear()
+        # The thread that will serve the client, this needs to be an attribute
+        # so the handler below can modify it when it's executed (it's
+        # instantiated when the request is processed)
+        self.connection_thread = None
+
+        class FailToRespond(Exception):
+            pass
+
+        class FailingDuringResponseHandler(TCPConnectionHandler):
+
+            def handle_connection(request):
+                req = request.readline()
+                # Capture the thread and make it use 'caught' so we can wait on
+                # the even that will be set when the exception is caught. We
+                # also capture the thread to know where to look.
+                self.connection_thread = threading.currentThread()
+                self.connection_thread.set_sync_event(caught)
+                raise FailToRespond()
+
+        server = self.get_server(
+            connection_handler_class=FailingDuringResponseHandler)
+        client = self.get_client()
+        client.connect((server.host, server.port))
+        client.write('ping\n')
+        # Wait for the exception to be caught
+        caught.wait()
+        self.assertEqual('', client.read()) # connection closed
+        # Check that the connection thread did catch the exception,
+        # http://pad.lv/869366 was wrongly checking the server thread which
+        # works for TestingTCPServer where the connection is handled in the
+        # same thread than the server one but is racy for
+        # TestingThreadingTCPServer where the server thread may be in a
+        # blocking accept() call (or not).
+        try:
+            self.connection_thread.pending_exception()
+        except FailToRespond:
+            # Great, the test succeeded
+            pass
+        else:
+            # If the exception is not in the connection thread anymore, it's in
+            # the server's one. 
+            server.server.stopped.wait()
+            # The exception is available now
+            self.assertRaises(FailToRespond, server.pending_exception)
+
+    def test_exception_swallowed_while_serving(self):
+        # We need to ensure the exception has been caught
+        caught = threading.Event()
+        caught.clear()
+        # The thread that will serve the client, this needs to be an attribute
+        # so the handler below can access it when it's executed (it's
+        # instantiated when the request is processed)
+        self.connection_thread = None
+        class CantServe(Exception):
+            pass
+
+        class FailingWhileServingConnectionHandler(TCPConnectionHandler):
+
+            def handle(request):
+                # Capture the thread and make it use 'caught' so we can wait on
+                # the even that will be set when the exception is caught. We
+                # also capture the thread to know where to look.
+                self.connection_thread = threading.currentThread()
+                self.connection_thread.set_sync_event(caught)
+                raise CantServe()
+
+        server = self.get_server(
+            connection_handler_class=FailingWhileServingConnectionHandler)
+        # Install the exception swallower
+        server.set_ignored_exceptions(CantServe)
+        client = self.get_client()
+        # Connect to the server so the exception is raised there
+        client.connect((server.host, server.port))
+        # Wait for the exception to be caught
+        caught.wait()
+        self.assertEqual('', client.read()) # connection closed
+        # The connection wasn't served properly but the exception should have
+        # been swallowed (see test_server_crash_while_responding remark about
+        # http://pad.lv/869366 explaining why we can't check the server thread
+        # here). More precisely, the exception *has* been caught and captured
+        # but it is cleared when joining the thread (or trying to acquire the
+        # exception) and as such won't propagate to the server thread.
+        self.connection_thread.pending_exception()
+        server.pending_exception()
+
+    def test_handle_request_closes_if_it_doesnt_process(self):
+        server = self.get_server()
+        client = self.get_client()
+        server.server.serving = False
+        client.connect((server.host, server.port))
+        self.assertEqual('', client.read())
+
+
+class TestTestingSmartServer(tests.TestCase):
+
+    def test_sets_client_timeout(self):
+        server = test_server.TestingSmartServer(('localhost', 0), None, None,
+            root_client_path='/no-such-client/path')
+        self.assertEqual(test_server._DEFAULT_TESTING_CLIENT_TIMEOUT,
+                         server._client_timeout)
+        sock = socket.socket()
+        h = server._make_handler(sock)
+        self.assertEqual(test_server._DEFAULT_TESTING_CLIENT_TIMEOUT,
+                         h._client_timeout)
+
+
+class FakeServer(object):
+    """Minimal implementation to pass to TestingSmartConnectionHandler"""
+    backing_transport = None
+    root_client_path = '/'
+
+
+class TestTestingSmartConnectionHandler(tests.TestCase):
+
+    def test_connection_timeout_suppressed(self):
+        self.overrideAttr(test_server, '_DEFAULT_TESTING_CLIENT_TIMEOUT', 0.01)
+        s = FakeServer()
+        server_sock, client_sock = portable_socket_pair()
+        # This should timeout quickly, but not generate an exception.
+        handler = test_server.TestingSmartConnectionHandler(server_sock,
+            server_sock.getpeername(), s)
+
+    def test_connection_shutdown_while_serving_no_error(self):
+        s = FakeServer()
+        server_sock, client_sock = portable_socket_pair()
+        class ShutdownConnectionHandler(
+            test_server.TestingSmartConnectionHandler):
+
+            def _build_protocol(self):
+                self.finished = True
+                return super(ShutdownConnectionHandler, self)._build_protocol()
+        # This should trigger shutdown after the entering _build_protocol, and
+        # we should exit cleanly, without raising an exception.
+        handler = ShutdownConnectionHandler(server_sock,
+            server_sock.getpeername(), s)

=== modified file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_testament.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,9 +21,16 @@
 import os
 
 from bzrlib import osutils
-from bzrlib.tests import SymlinkFeature, TestCaseWithTransport
-from bzrlib.testament import Testament, StrictTestament, StrictTestament3
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.testament import (
+    Testament,
+    StrictTestament,
+    StrictTestament3,
+    )
 from bzrlib.transform import TreeTransform
+from bzrlib.tests.features import (
+    SymlinkFeature,
+    )
 
 
 class TestamentSetup(TestCaseWithTransport):
@@ -129,17 +136,25 @@
             timestamp=1129025493,
             timezone=36000,
             rev_id='test@user-3',
-            committer='Erik B\xe5gfors <test@user>',
+            committer=u'Erik B\xe5gfors <test@user>',
             revprops={'uni':u'\xb5'}
             )
         t = self.from_revision(self.b.repository, 'test@user-3')
         self.assertEqualDiff(
             self.expected('sample_unicode').encode('utf-8'), t.as_text())
 
+    def test_from_tree(self):
+        tree = self.b.repository.revision_tree('test@user-2')
+        testament = self.testament_class().from_revision_tree(tree)
+        text_1 = testament.as_short_text()
+        text_2 = self.from_revision(self.b.repository,
+                                    'test@user-2').as_short_text()
+        self.assertEqual(text_1, text_2)
+
     def test___init__(self):
         revision = self.b.repository.get_revision('test@user-2')
-        inventory = self.b.repository.get_inventory('test@user-2')
-        testament_1 = self.testament_class()(revision, inventory)
+        tree = self.b.repository.revision_tree('test@user-2')
+        testament_1 = self.testament_class()(revision, tree)
         text_1 = testament_1.as_short_text()
         text_2 = self.from_revision(self.b.repository,
                                     'test@user-2').as_short_text()
@@ -211,21 +226,21 @@
 bazaar-ng testament short form 1
 revision-id: test@user-1
 sha1: %s
-""" % osutils.sha(REV_1_TESTAMENT).hexdigest()
+""" % osutils.sha_string(REV_1_TESTAMENT)
 
 
 REV_1_SHORT_STRICT = """\
 bazaar-ng testament short form 2.1
 revision-id: test@user-1
 sha1: %s
-""" % osutils.sha(REV_1_STRICT_TESTAMENT).hexdigest()
+""" % osutils.sha_string(REV_1_STRICT_TESTAMENT)
 
 
 REV_1_SHORT_STRICT3 = """\
 bazaar testament short form 3 strict
 revision-id: test@user-1
 sha1: %s
-""" % osutils.sha(REV_1_STRICT_TESTAMENT3).hexdigest()
+""" % osutils.sha_string(REV_1_STRICT_TESTAMENT3)
 
 
 REV_2_TESTAMENT = """\
@@ -293,21 +308,21 @@
 bazaar-ng testament short form 1
 revision-id: test@user-2
 sha1: %s
-""" % osutils.sha(REV_2_TESTAMENT).hexdigest()
+""" % osutils.sha_string(REV_2_TESTAMENT)
 
 
 REV_2_SHORT_STRICT = """\
 bazaar-ng testament short form 2.1
 revision-id: test@user-2
 sha1: %s
-""" % osutils.sha(REV_2_STRICT_TESTAMENT).hexdigest()
+""" % osutils.sha_string(REV_2_STRICT_TESTAMENT)
 
 
 REV_2_SHORT_STRICT3 = """\
 bazaar testament short form 3 strict
 revision-id: test@user-2
 sha1: %s
-""" % osutils.sha(REV_2_STRICT_TESTAMENT3).hexdigest()
+""" % osutils.sha_string(REV_2_STRICT_TESTAMENT3)
 
 
 REV_PROPS_TESTAMENT = """\

=== modified file 'bzrlib/tests/test_timestamp.py'
--- bzrlib/tests/test_timestamp.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_timestamp.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,10 +14,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import random
+import time
+
+
 from bzrlib import (
     tests,
     timestamp,
     )
+from bzrlib.osutils import local_time_offset
+
 
 class TestPatchHeader(tests.TestCase):
 
@@ -52,3 +58,38 @@
         # offset of three minutes
         self.assertEqual((1173193459, +3 * 60),
             timestamp.parse_patch_date('2007-03-06 15:07:19 +0003'))
+
+
+class UnpackHighresDateTests(tests.TestCase):
+
+    def test_unpack_highres_date(self):
+        self.assertEquals(
+            (1120153132.3508501, -18000),
+            timestamp.unpack_highres_date('Thu 2005-06-30 12:38:52.350850105 -0500'))
+        self.assertEquals(
+            (1120153132.3508501, 0),
+            timestamp.unpack_highres_date('Thu 2005-06-30 17:38:52.350850105 +0000'))
+        self.assertEquals(
+            (1120153132.3508501, 7200),
+            timestamp.unpack_highres_date('Thu 2005-06-30 19:38:52.350850105 +0200'))
+        self.assertEquals(
+            (1152428738.867522, 19800),
+            timestamp.unpack_highres_date('Sun 2006-07-09 12:35:38.867522001 +0530'))
+
+    def test_random(self):
+        t = time.time()
+        o = local_time_offset()
+        t2, o2 = timestamp.unpack_highres_date(timestamp.format_highres_date(t, o))
+        self.assertEquals(t, t2)
+        self.assertEquals(o, o2)
+        t -= 24*3600*365*2 # Start 2 years ago
+        o = -12*3600
+        for count in xrange(500):
+            t += random.random()*24*3600*30
+            o = ((o/3600 + 13) % 25 - 12)*3600 # Add 1 wrap around from [-12, 12]
+            date = timestamp.format_highres_date(t, o)
+            t2, o2 = timestamp.unpack_highres_date(date)
+            self.assertEquals(t, t2,
+                'Failed on date %r, %s,%s diff:%s' % (date, t, o, t2-t))
+            self.assertEquals(o, o2,
+                'Failed on date %r, %s,%s diff:%s' % (date, t, o, t2-t))

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2010-06-26 01:07:16 +0000
+++ bzrlib/tests/test_trace.py	2011-09-16 12:03:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 import tempfile
 
 from bzrlib import (
+    debug,
     errors,
     trace,
     )
@@ -81,7 +82,18 @@
             pass
         msg = _format_exception()
         self.assertEquals(msg,
-            "bzr: out of memory\n")
+            "bzr: out of memory\nUse -Dmem_dump to dump memory to a file.\n")
+
+    def test_format_mem_dump(self):
+        self.requireFeature(features.meliae)
+        debug.debug_flags.add('mem_dump')
+        try:
+            raise MemoryError()
+        except MemoryError:
+            pass
+        msg = _format_exception()
+        self.assertStartsWith(msg,
+            "bzr: out of memory\nMemory dumped to ")
 
     def test_format_os_error(self):
         try:
@@ -101,8 +113,9 @@
         msg = _format_exception()
         # Even though Windows and Linux differ for 'os.rmdir', they both give
         # 'No such file' for open()
+        # However it now gets translated so we can not test for a specific message
         self.assertContainsRe(msg,
-            r'^bzr: ERROR: \[Errno .*\] No such file.*nosuchfile')
+            r'^bzr: ERROR: \[Errno .*\] .*nosuchfile')
 
     def test_format_pywintypes_error(self):
         self.requireFeature(features.pywintypes)
@@ -116,6 +129,18 @@
         #                with errno, function name, and locale error message
         self.assertContainsRe(msg,
             r"^bzr: ERROR: \(2, 'RemoveDirectory[AW]?', .*\)")
+            
+    def test_format_sockets_error(self):
+        try:
+            import socket
+            sock = socket.socket()
+            sock.send("This should fail.")
+        except socket.error:
+            pass
+        msg = _format_exception()
+        
+        self.assertNotContainsRe(msg,
+            r"Traceback (most recent call last):")
 
     def test_format_unicode_error(self):
         try:
@@ -289,9 +314,7 @@
         # set up.
         self.overrideAttr(sys, 'stderr', StringIO())
         # Set the log file to something that cannot exist
-        # FIXME: A bit dangerous: we are not in an isolated dir here -- vilajam
-        # 20100125
-        os.environ['BZR_LOG'] = os.getcwd() + '/no-dir/bzr.log'
+        self.overrideEnv('BZR_LOG', os.getcwd() + '/no-dir/bzr.log')
         self.overrideAttr(trace, '_bzr_log_filename')
         logf = trace._open_bzr_log()
         self.assertIs(None, logf)

=== modified file 'bzrlib/tests/test_transactions.py'
--- bzrlib/tests/test_transactions.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_transactions.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2009, 2011 Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -17,13 +17,9 @@
 
 """Tests for the behaviour of the Transaction concept in bzr."""
 
-# import system imports here
-import os
-import sys
-
 #import bzrlib specific imports here
 import bzrlib.errors as errors
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests import TestCase
 import bzrlib.transactions as transactions
 
 

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2010-05-13 17:32:55 +0000
+++ bzrlib/tests/test_transform.py	2011-07-21 06:46:34 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+import errno
 import os
 from StringIO import StringIO
 import sys
@@ -27,7 +28,10 @@
     osutils,
     revision as _mod_revision,
     rules,
+    symbol_versioning,
     tests,
+    trace,
+    transform,
     urlutils,
     )
 from bzrlib.bzrdir import BzrDir
@@ -49,7 +53,6 @@
     ImmortalPendingDeletion,
     LockError,
     MalformedTransform,
-    NoSuchFile,
     ReusingTransform,
 )
 from bzrlib.osutils import (
@@ -58,19 +61,20 @@
 )
 from bzrlib.merge import Merge3Merger, Merger
 from bzrlib.tests import (
+    features,
+    TestCaseInTempDir,
+    TestSkipped,
+    )
+from bzrlib.tests.features import (
     HardlinkFeature,
     SymlinkFeature,
-    TestCase,
-    TestCaseInTempDir,
-    TestSkipped,
-)
+    )
 from bzrlib.transform import (
     build_tree,
     create_from_tree,
     cook_conflicts,
     _FileMover,
     FinalPaths,
-    get_backup_name,
     resolve_conflicts,
     resolve_checkout,
     ROOT_PARENT,
@@ -91,6 +95,16 @@
         self.addCleanup(transform.finalize)
         return transform, transform.root
 
+    def get_transform_for_sha1_test(self):
+        trans, root = self.get_transform()
+        self.wt.lock_tree_write()
+        self.addCleanup(self.wt.unlock)
+        contents = ['just some content\n']
+        sha1 = osutils.sha_strings(contents)
+        # Roll back the clock
+        trans._creation_mtime = time.time() - 20.0
+        return trans, root, contents, sha1
+
     def test_existing_limbo(self):
         transform, root = self.get_transform()
         limbo_name = transform._limbodir
@@ -123,12 +137,12 @@
         imaginary_id = transform.trans_id_tree_path('imaginary')
         imaginary_id2 = transform.trans_id_tree_path('imaginary/')
         self.assertEqual(imaginary_id, imaginary_id2)
-        self.assertEqual(transform.get_tree_parent(imaginary_id), root)
-        self.assertEqual(transform.final_kind(root), 'directory')
-        self.assertEqual(transform.final_file_id(root), self.wt.get_root_id())
+        self.assertEqual(root, transform.get_tree_parent(imaginary_id))
+        self.assertEqual('directory', transform.final_kind(root))
+        self.assertEqual(self.wt.get_root_id(), transform.final_file_id(root))
         trans_id = transform.create_path('name', root)
         self.assertIs(transform.final_file_id(trans_id), None)
-        self.assertRaises(NoSuchFile, transform.final_kind, trans_id)
+        self.assertIs(None, transform.final_kind(trans_id))
         transform.create_file('contents', trans_id)
         transform.set_executability(True, trans_id)
         transform.version_file('my_pretties', trans_id)
@@ -159,6 +173,67 @@
         transform.finalize()
         transform.finalize()
 
+    def test_apply_informs_tree_of_observed_sha1(self):
+        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
+        trans_id = trans.new_file('file1', root, contents, file_id='file1-id',
+                                  sha1=sha1)
+        calls = []
+        orig = self.wt._observed_sha1
+        def _observed_sha1(*args):
+            calls.append(args)
+            orig(*args)
+        self.wt._observed_sha1 = _observed_sha1
+        trans.apply()
+        self.assertEqual([(None, 'file1', trans._observed_sha1s[trans_id])],
+                         calls)
+
+    def test_create_file_caches_sha1(self):
+        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
+        trans_id = trans.create_path('file1', root)
+        trans.create_file(contents, trans_id, sha1=sha1)
+        st_val = osutils.lstat(trans._limbo_name(trans_id))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, st_val)
+
+    def test__apply_insertions_updates_sha1(self):
+        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
+        trans_id = trans.create_path('file1', root)
+        trans.create_file(contents, trans_id, sha1=sha1)
+        st_val = osutils.lstat(trans._limbo_name(trans_id))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, st_val)
+        creation_mtime = trans._creation_mtime + 10.0
+        # We fake a time difference from when the file was created until now it
+        # is being renamed by using os.utime. Note that the change we actually
+        # want to see is the real ctime change from 'os.rename()', but as long
+        # as we observe a new stat value, we should be fine.
+        os.utime(trans._limbo_name(trans_id), (creation_mtime, creation_mtime))
+        trans.apply()
+        new_st_val = osutils.lstat(self.wt.abspath('file1'))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, new_st_val)
+        self.assertNotEqual(st_val.st_mtime, new_st_val.st_mtime)
+
+    def test_new_file_caches_sha1(self):
+        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
+        trans_id = trans.new_file('file1', root, contents, file_id='file1-id',
+                                  sha1=sha1)
+        st_val = osutils.lstat(trans._limbo_name(trans_id))
+        o_sha1, o_st_val = trans._observed_sha1s[trans_id]
+        self.assertEqual(o_sha1, sha1)
+        self.assertEqualStat(o_st_val, st_val)
+
+    def test_cancel_creation_removes_observed_sha1(self):
+        trans, root, contents, sha1 = self.get_transform_for_sha1_test()
+        trans_id = trans.new_file('file1', root, contents, file_id='file1-id',
+                                  sha1=sha1)
+        self.assertTrue(trans_id in trans._observed_sha1s)
+        trans.cancel_creation(trans_id)
+        self.assertFalse(trans_id in trans._observed_sha1s)
+
     def test_create_files_same_timestamp(self):
         transform, root = self.get_transform()
         self.wt.lock_tree_write()
@@ -212,6 +287,68 @@
         new_trans_id = transform.new_directory('', ROOT_PARENT, 'alt-root-id')
         self.assertRaises(ValueError, transform.fixup_new_roots)
 
+    def test_retain_existing_root(self):
+        tt, root = self.get_transform()
+        with tt:
+            tt.new_directory('', ROOT_PARENT, 'new-root-id')
+            tt.fixup_new_roots()
+            self.assertNotEqual('new-root-id', tt.final_file_id(tt.root))
+
+    def test_retain_existing_root_added_file(self):
+        tt, root = self.get_transform()
+        new_trans_id = tt.new_directory('', ROOT_PARENT, 'new-root-id')
+        child = tt.new_directory('child', new_trans_id, 'child-id')
+        tt.fixup_new_roots()
+        self.assertEqual(tt.root, tt.final_parent(child))
+
+    def test_add_unversioned_root(self):
+        transform, root = self.get_transform()
+        new_trans_id = transform.new_directory('', ROOT_PARENT, None)
+        transform.delete_contents(transform.root)
+        transform.fixup_new_roots()
+        self.assertNotIn(transform.root, transform._new_id)
+
+    def test_remove_root_fixup(self):
+        transform, root = self.get_transform()
+        old_root_id = self.wt.get_root_id()
+        self.assertNotEqual('new-root-id', old_root_id)
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        transform.apply()
+        self.assertEqual(old_root_id, self.wt.get_root_id())
+
+        transform, root = self.get_transform()
+        new_trans_id = transform.new_directory('', ROOT_PARENT, 'new-root-id')
+        new_trans_id = transform.new_directory('', ROOT_PARENT, 'alt-root-id')
+        self.assertRaises(ValueError, transform.fixup_new_roots)
+
+    def test_fixup_new_roots_permits_empty_tree(self):
+        transform, root = self.get_transform()
+        transform.delete_contents(root)
+        transform.unversion_file(root)
+        transform.fixup_new_roots()
+        self.assertIs(None, transform.final_kind(root))
+        self.assertIs(None, transform.final_file_id(root))
+
+    def test_apply_retains_root_directory(self):
+        # Do not attempt to delete the physical root directory, because that
+        # is impossible.
+        transform, root = self.get_transform()
+        with transform:
+            transform.delete_contents(root)
+            e = self.assertRaises(AssertionError, self.assertRaises,
+                                  errors.TransformRenameFailed,
+                                  transform.apply)
+        self.assertContainsRe('TransformRenameFailed not raised', str(e))
+
+    def test_apply_retains_file_id(self):
+        transform, root = self.get_transform()
+        old_root_id = transform.tree_file_id(root)
+        transform.unversion_file(root)
+        transform.apply()
+        self.assertEqual(old_root_id, self.wt.get_root_id())
+
     def test_hardlink(self):
         self.requireFeature(HardlinkFeature)
         transform, root = self.get_transform()
@@ -222,7 +359,7 @@
         trans_id = target_transform.create_path('file1', target_transform.root)
         target_transform.create_hardlink(self.wt.abspath('file1'), trans_id)
         target_transform.apply()
-        self.failUnlessExists('target/file1')
+        self.assertPathExists('target/file1')
         source_stat = os.stat(self.wt.abspath('file1'))
         target_stat = os.stat('target/file1')
         self.assertEqual(source_stat, target_stat)
@@ -394,8 +531,8 @@
         transform.new_file('FiLe', transform.root, 'content')
         resolve_conflicts(transform)
         transform.apply()
-        self.failUnlessExists('tree/file')
-        self.failUnlessExists('tree/FiLe.moved')
+        self.assertPathExists('tree/file')
+        self.assertPathExists('tree/FiLe.moved')
 
     def test_resolve_checkout_case_conflict(self):
         tree = self.make_branch_and_tree('tree')
@@ -410,8 +547,8 @@
         resolve_conflicts(transform,
                           pass_func=lambda t, c: resolve_checkout(t, c, []))
         transform.apply()
-        self.failUnlessExists('tree/file')
-        self.failUnlessExists('tree/FiLe.moved')
+        self.assertPathExists('tree/file')
+        self.assertPathExists('tree/FiLe.moved')
 
     def test_apply_case_conflict(self):
         """Ensure that a transform with case conflicts can always be applied"""
@@ -425,12 +562,12 @@
         transform.new_file('dirFiLe', dir, 'content')
         resolve_conflicts(transform)
         transform.apply()
-        self.failUnlessExists('tree/file')
+        self.assertPathExists('tree/file')
         if not os.path.exists('tree/FiLe.moved'):
-            self.failUnlessExists('tree/FiLe')
-        self.failUnlessExists('tree/dir/dirfile')
+            self.assertPathExists('tree/FiLe')
+        self.assertPathExists('tree/dir/dirfile')
         if not os.path.exists('tree/dir/dirFiLe.moved'):
-            self.failUnlessExists('tree/dir/dirFiLe')
+            self.assertPathExists('tree/dir/dirFiLe')
 
     def test_case_insensitive_limbo(self):
         tree = self.make_branch_and_tree('tree')
@@ -644,7 +781,7 @@
                             'wizard2', 'behind_curtain')
 
     def test_symlinks_unicode(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self._test_symlinks(u'\N{Euro Sign}wizard',
                             u'wizard-targ\N{Euro Sign}t',
                             u'\N{Euro Sign}wizard2',
@@ -738,7 +875,7 @@
         raw_conflicts = resolve_conflicts(tt)
         cooked_conflicts = cook_conflicts(raw_conflicts, tt)
         tt.finalize()
-        conflicts_s = [str(c) for c in cooked_conflicts]
+        conflicts_s = [unicode(c) for c in cooked_conflicts]
         self.assertEqual(len(cooked_conflicts), len(conflicts_s))
         self.assertEqual(conflicts_s[0], 'Conflict adding file dorothy.  '
                                          'Moved existing file to '
@@ -814,6 +951,21 @@
         self.assertIs(None, self.wt.path2id('parent'))
         self.assertIs(None, self.wt.path2id('parent.new'))
 
+    def test_resolve_conflicts_missing_parent(self):
+        wt = self.make_branch_and_tree('.')
+        tt = TreeTransform(wt)
+        self.addCleanup(tt.finalize)
+        parent = tt.trans_id_file_id('parent-id')
+        tt.new_file('file', parent, 'Contents')
+        raw_conflicts = resolve_conflicts(tt)
+        # Since the directory doesn't exist it's seen as 'missing'.  So
+        # 'resolve_conflicts' create a conflict asking for it to be created.
+        self.assertLength(1, raw_conflicts)
+        self.assertEqual(('missing parent', 'Created directory', 'new-1'),
+                         raw_conflicts.pop())
+        # apply fail since the missing directory doesn't exist
+        self.assertRaises(errors.NoFinalPath, tt.apply)
+
     def test_moving_versioned_directories(self):
         create, root = self.get_transform()
         kansas = create.new_directory('kansas', root, 'kansas-id')
@@ -853,16 +1005,21 @@
         rename.apply()
 
     def test_rename_fails(self):
+        self.requireFeature(features.not_running_as_root)
         # see https://bugs.launchpad.net/bzr/+bug/491763
         create, root_id = self.get_transform()
         first_dir = create.new_directory('first-dir', root_id, 'first-id')
         myfile = create.new_file('myfile', root_id, 'myfile-text',
                                  'myfile-id')
         create.apply()
-        # make the file and directory readonly in the hope this will prevent
-        # renames
-        osutils.make_readonly(self.wt.abspath('first-dir'))
-        osutils.make_readonly(self.wt.abspath('myfile'))
+        if os.name == "posix" and sys.platform != "cygwin":
+            # posix filesystems fail on renaming if the readonly bit is set
+            osutils.make_readonly(self.wt.abspath('first-dir'))
+        elif os.name == "nt":
+            # windows filesystems fail on renaming open files
+            self.addCleanup(file(self.wt.abspath('myfile')).close)
+        else:
+            self.skip("Don't know how to force a permissions error on rename")
         # now transform to rename
         rename_transform, root_id = self.get_transform()
         file_trans_id = rename_transform.trans_id_file_id('myfile-id')
@@ -870,13 +1027,20 @@
         rename_transform.adjust_path('newname', dir_id, file_trans_id)
         e = self.assertRaises(errors.TransformRenameFailed,
             rename_transform.apply)
-        # Looks like: 
+        # On nix looks like: 
         # "Failed to rename .../work/.bzr/checkout/limbo/new-1
         # to .../first-dir/newname: [Errno 13] Permission denied"
-        # so the first filename is not visible in it; we expect a strerror but
-        # it may vary per OS and language so it's not checked here
-        self.assertContainsRe(str(e),
-            "Failed to rename .*first-dir.newname:")
+        # On windows looks like:
+        # "Failed to rename .../work/myfile to 
+        # .../work/.bzr/checkout/limbo/new-1: [Errno 13] Permission denied"
+        # This test isn't concerned with exactly what the error looks like,
+        # and the strerror will vary across OS and locales, but the assert
+        # that the exeception attributes are what we expect
+        self.assertEqual(e.errno, errno.EACCES)
+        if os.name == "posix":
+            self.assertEndsWith(e.to_path, "/first-dir/newname")
+        else:
+            self.assertEqual(os.path.basename(e.from_path), "myfile")
 
     def test_set_executability_order(self):
         """Ensure that executability behaves the same, no matter what order.
@@ -1155,11 +1319,11 @@
         parent2 = transform.new_directory('parent2', root)
         transform.adjust_path('child1', parent2, child1)
         transform.apply()
-        self.failIfExists(self.wt.abspath('parent1/child1'))
-        self.failUnlessExists(self.wt.abspath('parent2/child1'))
+        self.assertPathDoesNotExist(self.wt.abspath('parent1/child1'))
+        self.assertPathExists(self.wt.abspath('parent2/child1'))
         # rename limbo/new-1 => parent1, rename limbo/new-3 => parent2
         # no rename for child1 (counting only renames during apply)
-        self.failUnlessEqual(2, transform.rename_count)
+        self.assertEqual(2, transform.rename_count)
 
     def test_cancel_parent(self):
         """Cancelling a parent doesn't cause deletion of a non-empty directory
@@ -1188,10 +1352,10 @@
         parent2 = transform.new_directory('parent2', root)
         transform.adjust_path('child1', parent2, child1)
         transform.apply()
-        self.failIfExists(self.wt.abspath('parent1'))
-        self.failUnlessExists(self.wt.abspath('parent2/child1'))
+        self.assertPathDoesNotExist(self.wt.abspath('parent1'))
+        self.assertPathExists(self.wt.abspath('parent2/child1'))
         # rename limbo/new-3 => parent2, rename limbo/new-2 => child1
-        self.failUnlessEqual(2, transform.rename_count)
+        self.assertEqual(2, transform.rename_count)
 
     def test_adjust_and_cancel(self):
         """Make sure adjust_path keeps track of limbo children properly"""
@@ -1230,7 +1394,7 @@
         child = transform.new_directory('child', parent)
         transform.adjust_path('parent', root, parent)
         transform.apply()
-        self.failUnlessExists(self.wt.abspath('parent/child'))
+        self.assertPathExists(self.wt.abspath('parent/child'))
         self.assertEqual(1, transform.rename_count)
 
     def test_reuse_name(self):
@@ -1367,7 +1531,7 @@
         tt.create_file(["aa\n"], bar_trans_id)
         tt.version_file("bar-1", bar_trans_id)
         tt.apply()
-        self.failUnlessExists("foo/bar")
+        self.assertPathExists("foo/bar")
         wt.lock_read()
         try:
             self.assertEqual(wt.inventory.get_file_kind(wt.path2id("foo")),
@@ -1390,7 +1554,7 @@
         tt.delete_contents(foo_trans_id)
         tt.create_symlink("bar", foo_trans_id)
         tt.apply()
-        self.failUnlessExists("foo")
+        self.assertPathExists("foo")
         wt.lock_read()
         self.addCleanup(wt.unlock)
         self.assertEqual(wt.inventory.get_file_kind(wt.path2id("foo")),
@@ -1409,7 +1573,7 @@
         tt.delete_versioned(bar_trans_id)
         tt.create_file(["aa\n"], foo_trans_id)
         tt.apply()
-        self.failUnlessExists("foo")
+        self.assertPathExists("foo")
         wt.lock_read()
         self.addCleanup(wt.unlock)
         self.assertEqual(wt.inventory.get_file_kind(wt.path2id("foo")),
@@ -1430,8 +1594,8 @@
         self.build_tree(['baz'])
         tt.create_hardlink("baz", foo_trans_id)
         tt.apply()
-        self.failUnlessExists("foo")
-        self.failUnlessExists("baz")
+        self.assertPathExists("foo")
+        self.assertPathExists("baz")
         wt.lock_read()
         self.addCleanup(wt.unlock)
         self.assertEqual(wt.inventory.get_file_kind(wt.path2id("foo")),
@@ -1499,6 +1663,38 @@
     return template % ('<' * 7, tree, '=' * 7, merge, '>' * 7)
 
 
+class TestInventoryAltered(tests.TestCaseWithTransport):
+
+    def test_inventory_altered_unchanged(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo'])
+        tree.add('foo', 'foo-id')
+        with TransformPreview(tree) as tt:
+            self.assertEqual([], tt._inventory_altered())
+
+    def test_inventory_altered_changed_parent_id(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo'])
+        tree.add('foo', 'foo-id')
+        with TransformPreview(tree) as tt:
+            tt.unversion_file(tt.root)
+            tt.version_file('new-id', tt.root)
+            foo_trans_id = tt.trans_id_tree_file_id('foo-id')
+            foo_tuple = ('foo', foo_trans_id)
+            root_tuple = ('', tt.root)
+            self.assertEqual([root_tuple, foo_tuple], tt._inventory_altered())
+
+    def test_inventory_altered_noop_changed_parent_id(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/foo'])
+        tree.add('foo', 'foo-id')
+        with TransformPreview(tree) as tt:
+            tt.unversion_file(tt.root)
+            tt.version_file(tree.get_root_id(), tt.root)
+            foo_trans_id = tt.trans_id_tree_file_id('foo-id')
+            self.assertEqual([], tt._inventory_altered())
+
+
 class TestTransformMerge(TestCaseInTempDir):
 
     def test_text_merge(self):
@@ -1714,8 +1910,8 @@
         tree.add_reference(subtree)
         tree.commit('a revision')
         tree.branch.create_checkout('target')
-        self.failUnlessExists('target')
-        self.failUnlessExists('target/subtree')
+        self.assertPathExists('target')
+        self.assertPathExists('target/subtree')
 
     def test_file_conflict_handling(self):
         """Ensure that when building trees, conflict handling is done"""
@@ -1768,24 +1964,24 @@
         source.commit('added file')
         build_tree(source.basis_tree(), target)
         self.assertEqual([], target.conflicts())
-        self.failUnlessExists('target/dir1/file')
+        self.assertPathExists('target/dir1/file')
 
         # Ensure contents are merged
         target = self.make_branch_and_tree('target2')
         self.build_tree(['target2/dir1/', 'target2/dir1/file2'])
         build_tree(source.basis_tree(), target)
         self.assertEqual([], target.conflicts())
-        self.failUnlessExists('target2/dir1/file2')
-        self.failUnlessExists('target2/dir1/file')
+        self.assertPathExists('target2/dir1/file2')
+        self.assertPathExists('target2/dir1/file')
 
         # Ensure new contents are suppressed for existing branches
         target = self.make_branch_and_tree('target3')
         self.make_branch('target3/dir1')
         self.build_tree(['target3/dir1/file2'])
         build_tree(source.basis_tree(), target)
-        self.failIfExists('target3/dir1/file')
-        self.failUnlessExists('target3/dir1/file2')
-        self.failUnlessExists('target3/dir1.diverted/file')
+        self.assertPathDoesNotExist('target3/dir1/file')
+        self.assertPathExists('target3/dir1/file2')
+        self.assertPathExists('target3/dir1.diverted/file')
         self.assertEqual([DuplicateEntry('Diverted to',
             'dir1.diverted', 'dir1', 'new-dir1', None)],
             target.conflicts())
@@ -1794,9 +1990,9 @@
         self.build_tree(['target4/dir1/'])
         self.make_branch('target4/dir1/file')
         build_tree(source.basis_tree(), target)
-        self.failUnlessExists('target4/dir1/file')
+        self.assertPathExists('target4/dir1/file')
         self.assertEqual('directory', file_kind('target4/dir1/file'))
-        self.failUnlessExists('target4/dir1/file.diverted')
+        self.assertPathExists('target4/dir1/file.diverted')
         self.assertEqual([DuplicateEntry('Diverted to',
             'dir1/file.diverted', 'dir1/file', 'new-file', None)],
             target.conflicts())
@@ -1870,6 +2066,18 @@
         self.addCleanup(target.unlock)
         self.assertEqual([], list(target.iter_changes(revision_tree)))
 
+    def test_build_tree_accelerator_tree_observes_sha1(self):
+        source = self.create_ab_tree()
+        sha1 = osutils.sha_string('A')
+        target = self.make_branch_and_tree('target')
+        target.lock_write()
+        self.addCleanup(target.unlock)
+        state = target.current_dirstate()
+        state._cutoff_time = time.time() + 60
+        build_tree(source.basis_tree(), target, source)
+        entry = state._get_entry(0, path_utf8='file1')
+        self.assertEqual(sha1, entry[1][0][1])
+
     def test_build_tree_accelerator_tree_missing_file(self):
         source = self.create_ab_tree()
         os.unlink('source/file1')
@@ -2018,8 +2226,8 @@
         self.assertEqualStat(source_stat, target_stat)
 
     def test_case_insensitive_build_tree_inventory(self):
-        if (tests.CaseInsensitiveFilesystemFeature.available()
-            or tests.CaseInsCasePresFilenameFeature.available()):
+        if (features.CaseInsensitiveFilesystemFeature.available()
+            or features.CaseInsCasePresFilenameFeature.available()):
             raise tests.UnavailableFeature('Fully case sensitive filesystem')
         source = self.make_branch_and_tree('source')
         self.build_tree(['source/file', 'source/FILE'])
@@ -2033,6 +2241,42 @@
         self.assertEqual('file.moved', target.id2path('lower-id'))
         self.assertEqual('FILE', target.id2path('upper-id'))
 
+    def test_build_tree_observes_sha(self):
+        source = self.make_branch_and_tree('source')
+        self.build_tree(['source/file1', 'source/dir/', 'source/dir/file2'])
+        source.add(['file1', 'dir', 'dir/file2'],
+                   ['file1-id', 'dir-id', 'file2-id'])
+        source.commit('new files')
+        target = self.make_branch_and_tree('target')
+        target.lock_write()
+        self.addCleanup(target.unlock)
+        # We make use of the fact that DirState caches its cutoff time. So we
+        # set the 'safe' time to one minute in the future.
+        state = target.current_dirstate()
+        state._cutoff_time = time.time() + 60
+        build_tree(source.basis_tree(), target)
+        entry1_sha = osutils.sha_file_by_name('source/file1')
+        entry2_sha = osutils.sha_file_by_name('source/dir/file2')
+        # entry[1] is the state information, entry[1][0] is the state of the
+        # working tree, entry[1][0][1] is the sha value for the current working
+        # tree
+        entry1 = state._get_entry(0, path_utf8='file1')
+        self.assertEqual(entry1_sha, entry1[1][0][1])
+        # The 'size' field must also be set.
+        self.assertEqual(25, entry1[1][0][2])
+        entry1_state = entry1[1][0]
+        entry2 = state._get_entry(0, path_utf8='dir/file2')
+        self.assertEqual(entry2_sha, entry2[1][0][1])
+        self.assertEqual(29, entry2[1][0][2])
+        entry2_state = entry2[1][0]
+        # Now, make sure that we don't have to re-read the content. The
+        # packed_stat should match exactly.
+        self.assertEqual(entry1_sha, target.get_file_sha1('file1-id', 'file1'))
+        self.assertEqual(entry2_sha,
+                         target.get_file_sha1('file2-id', 'dir/file2'))
+        self.assertEqual(entry1_state, entry1[1][0])
+        self.assertEqual(entry2_state, entry2[1][0])
+
 
 class TestCommitTransform(tests.TestCaseWithTransport):
 
@@ -2172,37 +2416,29 @@
         self.assertEqual('tree', revision.properties['branch-nick'])
 
 
-class MockTransform(object):
-
-    def has_named_child(self, by_parent, parent_id, name):
-        for child_id in by_parent[parent_id]:
-            if child_id == '0':
-                if name == "name~":
-                    return True
-            elif name == "name.~%s~" % child_id:
-                return True
-        return False
-
-
-class MockEntry(object):
-    def __init__(self):
-        object.__init__(self)
-        self.name = "name"
-
-
-class TestGetBackupName(TestCase):
-    def test_get_backup_name(self):
+class TestBackupName(tests.TestCase):
+
+    def test_deprecations(self):
+        class MockTransform(object):
+
+            def has_named_child(self, by_parent, parent_id, name):
+                return name in by_parent.get(parent_id, [])
+
+        class MockEntry(object):
+
+            def __init__(self):
+                object.__init__(self)
+                self.name = "name"
+
         tt = MockTransform()
-        name = get_backup_name(MockEntry(), {'a':[]}, 'a', tt)
-        self.assertEqual(name, 'name.~1~')
-        name = get_backup_name(MockEntry(), {'a':['1']}, 'a', tt)
-        self.assertEqual(name, 'name.~2~')
-        name = get_backup_name(MockEntry(), {'a':['2']}, 'a', tt)
-        self.assertEqual(name, 'name.~1~')
-        name = get_backup_name(MockEntry(), {'a':['2'], 'b':[]}, 'b', tt)
-        self.assertEqual(name, 'name.~1~')
-        name = get_backup_name(MockEntry(), {'a':['1', '2', '3']}, 'a', tt)
-        self.assertEqual(name, 'name.~4~')
+        name1 = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            transform.get_backup_name, MockEntry(), {'a':[]}, 'a', tt)
+        self.assertEqual('name.~1~', name1)
+        name2 = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            transform._get_backup_name, 'name', {'a':['name.~1~']}, 'a', tt)
+        self.assertEqual('name.~2~', name2)
 
 
 class TestFileMover(tests.TestCaseWithTransport):
@@ -2211,36 +2447,36 @@
         self.build_tree(['a/', 'a/b', 'c/', 'c/d'])
         mover = _FileMover()
         mover.rename('a', 'q')
-        self.failUnlessExists('q')
-        self.failIfExists('a')
-        self.failUnlessExists('q/b')
-        self.failUnlessExists('c')
-        self.failUnlessExists('c/d')
+        self.assertPathExists('q')
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists('q/b')
+        self.assertPathExists('c')
+        self.assertPathExists('c/d')
 
     def test_pre_delete_rollback(self):
         self.build_tree(['a/'])
         mover = _FileMover()
         mover.pre_delete('a', 'q')
-        self.failUnlessExists('q')
-        self.failIfExists('a')
+        self.assertPathExists('q')
+        self.assertPathDoesNotExist('a')
         mover.rollback()
-        self.failIfExists('q')
-        self.failUnlessExists('a')
+        self.assertPathDoesNotExist('q')
+        self.assertPathExists('a')
 
     def test_apply_deletions(self):
         self.build_tree(['a/', 'b/'])
         mover = _FileMover()
         mover.pre_delete('a', 'q')
         mover.pre_delete('b', 'r')
-        self.failUnlessExists('q')
-        self.failUnlessExists('r')
-        self.failIfExists('a')
-        self.failIfExists('b')
+        self.assertPathExists('q')
+        self.assertPathExists('r')
+        self.assertPathDoesNotExist('a')
+        self.assertPathDoesNotExist('b')
         mover.apply_deletions()
-        self.failIfExists('q')
-        self.failIfExists('r')
-        self.failIfExists('a')
-        self.failIfExists('b')
+        self.assertPathDoesNotExist('q')
+        self.assertPathDoesNotExist('r')
+        self.assertPathDoesNotExist('a')
+        self.assertPathDoesNotExist('b')
 
     def test_file_mover_rollback(self):
         self.build_tree(['a/', 'a/b', 'c/', 'c/d/', 'c/e/'])
@@ -2251,8 +2487,8 @@
             mover.rename('a', 'c')
         except errors.FileExists, e:
             mover.rollback()
-        self.failUnlessExists('a')
-        self.failUnlessExists('c/d')
+        self.assertPathExists('a')
+        self.assertPathExists('c/d')
 
 
 class Bogus(Exception):
@@ -2288,11 +2524,11 @@
         tt.adjust_path('d', a_id, tt.trans_id_tree_path('a/b'))
         self.assertRaises(Bogus, tt.apply,
                           _mover=self.ExceptionFileMover(bad_source='a'))
-        self.failUnlessExists('a')
-        self.failUnlessExists('a/b')
+        self.assertPathExists('a')
+        self.assertPathExists('a/b')
         tt.apply()
-        self.failUnlessExists('c')
-        self.failUnlessExists('c/d')
+        self.assertPathExists('c')
+        self.assertPathExists('c/d')
 
     def test_rollback_rename_into_place(self):
         tree = self.make_branch_and_tree('.')
@@ -2304,11 +2540,11 @@
         tt.adjust_path('d', a_id, tt.trans_id_tree_path('a/b'))
         self.assertRaises(Bogus, tt.apply,
                           _mover=self.ExceptionFileMover(bad_target='c/d'))
-        self.failUnlessExists('a')
-        self.failUnlessExists('a/b')
+        self.assertPathExists('a')
+        self.assertPathExists('a/b')
         tt.apply()
-        self.failUnlessExists('c')
-        self.failUnlessExists('c/d')
+        self.assertPathExists('c')
+        self.assertPathExists('c/d')
 
     def test_rollback_deletion(self):
         tree = self.make_branch_and_tree('.')
@@ -2320,23 +2556,174 @@
         tt.adjust_path('d', tt.root, tt.trans_id_tree_path('a/b'))
         self.assertRaises(Bogus, tt.apply,
                           _mover=self.ExceptionFileMover(bad_target='d'))
-        self.failUnlessExists('a')
-        self.failUnlessExists('a/b')
-
-    def test_resolve_no_parent(self):
+        self.assertPathExists('a')
+        self.assertPathExists('a/b')
+
+
+class TestFinalizeRobustness(tests.TestCaseWithTransport):
+    """Ensure treetransform creation errors can be safely cleaned up after"""
+
+    def _override_globals_in_method(self, instance, method_name, globals):
+        """Replace method on instance with one with updated globals"""
+        import types
+        func = getattr(instance, method_name).im_func
+        new_globals = dict(func.func_globals)
+        new_globals.update(globals)
+        new_func = types.FunctionType(func.func_code, new_globals,
+            func.func_name, func.func_defaults)
+        setattr(instance, method_name,
+            types.MethodType(new_func, instance, instance.__class__))
+        self.addCleanup(delattr, instance, method_name)
+
+    @staticmethod
+    def _fake_open_raises_before(name, mode):
+        """Like open() but raises before doing anything"""
+        raise RuntimeError
+
+    @staticmethod
+    def _fake_open_raises_after(name, mode):
+        """Like open() but raises after creating file without returning"""
+        open(name, mode).close()
+        raise RuntimeError
+
+    def create_transform_and_root_trans_id(self):
+        """Setup a transform creating a file in limbo"""
+        tree = self.make_branch_and_tree('.')
+        tt = TreeTransform(tree)
+        return tt, tt.create_path("a", tt.root)
+
+    def create_transform_and_subdir_trans_id(self):
+        """Setup a transform creating a directory containing a file in limbo"""
+        tree = self.make_branch_and_tree('.')
+        tt = TreeTransform(tree)
+        d_trans_id = tt.create_path("d", tt.root)
+        tt.create_directory(d_trans_id)
+        f_trans_id = tt.create_path("a", d_trans_id)
+        tt.adjust_path("a", d_trans_id, f_trans_id)
+        return tt, f_trans_id
+
+    def test_root_create_file_open_raises_before_creation(self):
+        tt, trans_id = self.create_transform_and_root_trans_id()
+        self._override_globals_in_method(tt, "create_file",
+            {"open": self._fake_open_raises_before})
+        self.assertRaises(RuntimeError, tt.create_file, ["contents"], trans_id)
+        path = tt._limbo_name(trans_id)
+        self.assertPathDoesNotExist(path)
+        tt.finalize()
+        self.assertPathDoesNotExist(tt._limbodir)
+
+    def test_root_create_file_open_raises_after_creation(self):
+        tt, trans_id = self.create_transform_and_root_trans_id()
+        self._override_globals_in_method(tt, "create_file",
+            {"open": self._fake_open_raises_after})
+        self.assertRaises(RuntimeError, tt.create_file, ["contents"], trans_id)
+        path = tt._limbo_name(trans_id)
+        self.assertPathExists(path)
+        tt.finalize()
+        self.assertPathDoesNotExist(path)
+        self.assertPathDoesNotExist(tt._limbodir)
+
+    def test_subdir_create_file_open_raises_before_creation(self):
+        tt, trans_id = self.create_transform_and_subdir_trans_id()
+        self._override_globals_in_method(tt, "create_file",
+            {"open": self._fake_open_raises_before})
+        self.assertRaises(RuntimeError, tt.create_file, ["contents"], trans_id)
+        path = tt._limbo_name(trans_id)
+        self.assertPathDoesNotExist(path)
+        tt.finalize()
+        self.assertPathDoesNotExist(tt._limbodir)
+
+    def test_subdir_create_file_open_raises_after_creation(self):
+        tt, trans_id = self.create_transform_and_subdir_trans_id()
+        self._override_globals_in_method(tt, "create_file",
+            {"open": self._fake_open_raises_after})
+        self.assertRaises(RuntimeError, tt.create_file, ["contents"], trans_id)
+        path = tt._limbo_name(trans_id)
+        self.assertPathExists(path)
+        tt.finalize()
+        self.assertPathDoesNotExist(path)
+        self.assertPathDoesNotExist(tt._limbodir)
+
+    def test_rename_in_limbo_rename_raises_after_rename(self):
+        tt, trans_id = self.create_transform_and_root_trans_id()
+        parent1 = tt.new_directory('parent1', tt.root)
+        child1 = tt.new_file('child1', parent1, 'contents')
+        parent2 = tt.new_directory('parent2', tt.root)
+
+        class FakeOSModule(object):
+            def rename(self, old, new):
+                os.rename(old, new)
+                raise RuntimeError
+        self._override_globals_in_method(tt, "_rename_in_limbo",
+            {"os": FakeOSModule()})
+        self.assertRaises(
+            RuntimeError, tt.adjust_path, "child1", parent2, child1)
+        path = osutils.pathjoin(tt._limbo_name(parent2), "child1")
+        self.assertPathExists(path)
+        tt.finalize()
+        self.assertPathDoesNotExist(path)
+        self.assertPathDoesNotExist(tt._limbodir)
+
+    def test_rename_in_limbo_rename_raises_before_rename(self):
+        tt, trans_id = self.create_transform_and_root_trans_id()
+        parent1 = tt.new_directory('parent1', tt.root)
+        child1 = tt.new_file('child1', parent1, 'contents')
+        parent2 = tt.new_directory('parent2', tt.root)
+
+        class FakeOSModule(object):
+            def rename(self, old, new):
+                raise RuntimeError
+        self._override_globals_in_method(tt, "_rename_in_limbo",
+            {"os": FakeOSModule()})
+        self.assertRaises(
+            RuntimeError, tt.adjust_path, "child1", parent2, child1)
+        path = osutils.pathjoin(tt._limbo_name(parent1), "child1")
+        self.assertPathExists(path)
+        tt.finalize()
+        self.assertPathDoesNotExist(path)
+        self.assertPathDoesNotExist(tt._limbodir)
+
+
+class TestTransformMissingParent(tests.TestCaseWithTransport):
+
+    def make_tt_with_versioned_dir(self):
         wt = self.make_branch_and_tree('.')
+        self.build_tree(['dir/',])
+        wt.add(['dir'], ['dir-id'])
+        wt.commit('Create dir')
         tt = TreeTransform(wt)
         self.addCleanup(tt.finalize)
-        parent = tt.trans_id_file_id('parent-id')
-        tt.new_file('file', parent, 'Contents')
-        resolve_conflicts(tt)
+        return wt, tt
+
+    def test_resolve_create_parent_for_versioned_file(self):
+        wt, tt = self.make_tt_with_versioned_dir()
+        dir_tid = tt.trans_id_tree_file_id('dir-id')
+        file_tid = tt.new_file('file', dir_tid, 'Contents', file_id='file-id')
+        tt.delete_contents(dir_tid)
+        tt.unversion_file(dir_tid)
+        conflicts = resolve_conflicts(tt)
+        # one conflict for the missing directory, one for the unversioned
+        # parent
+        self.assertLength(2, conflicts)
+
+    def test_non_versioned_file_create_conflict(self):
+        wt, tt = self.make_tt_with_versioned_dir()
+        dir_tid = tt.trans_id_tree_file_id('dir-id')
+        tt.new_file('file', dir_tid, 'Contents')
+        tt.delete_contents(dir_tid)
+        tt.unversion_file(dir_tid)
+        conflicts = resolve_conflicts(tt)
+        # no conflicts or rather: orphaning 'file' resolve the 'dir' conflict
+        self.assertLength(1, conflicts)
+        self.assertEqual(('deleting parent', 'Not deleting', 'new-1'),
+                         conflicts.pop())
 
 
 A_ENTRY = ('a-id', ('a', 'a'), True, (True, True),
                   ('TREE_ROOT', 'TREE_ROOT'), ('a', 'a'), ('file', 'file'),
                   (False, False))
 ROOT_ENTRY = ('TREE_ROOT', ('', ''), False, (True, True), (None, None),
-              ('', ''), ('directory', 'directory'), (False, None))
+              ('', ''), ('directory', 'directory'), (False, False))
 
 
 class TestTransformPreview(tests.TestCaseWithTransport):
@@ -2429,13 +2816,13 @@
         revision_tree, preview_tree = self.get_tree_and_preview_tree()
         changes = preview_tree.iter_changes(revision_tree,
                                             specific_files=[''])
-        self.assertEqual([ROOT_ENTRY, A_ENTRY], list(changes))
+        self.assertEqual([A_ENTRY], list(changes))
 
     def test_want_unversioned(self):
         revision_tree, preview_tree = self.get_tree_and_preview_tree()
         changes = preview_tree.iter_changes(revision_tree,
                                             want_unversioned=True)
-        self.assertEqual([ROOT_ENTRY, A_ENTRY], list(changes))
+        self.assertEqual([A_ENTRY], list(changes))
 
     def test_ignore_extra_trees_no_specific_files(self):
         # extra_trees is harmless without specific_files, so we'll silently
@@ -2484,6 +2871,18 @@
         preview_mtime = preview_tree.get_file_mtime('file-id', 'renamed')
         work_mtime = work_tree.get_file_mtime('file-id', 'file')
 
+    def test_get_file_size(self):
+        work_tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/old', 'old')])
+        work_tree.add('old', 'old-id')
+        preview = TransformPreview(work_tree)
+        self.addCleanup(preview.finalize)
+        new_id = preview.new_file('name', preview.root, 'contents', 'new-id',
+                                  'executable')
+        tree = preview.get_preview_tree()
+        self.assertEqual(len('old'), tree.get_file_size('old-id'))
+        self.assertEqual(len('contents'), tree.get_file_size('new-id'))
+
     def test_get_file(self):
         preview = self.get_empty_preview()
         preview.new_file('file', preview.root, 'contents', 'file-id')
@@ -2901,6 +3300,24 @@
         merger.merge_type = Merge3Merger
         merger.do_merge()
 
+    def test_has_filename(self):
+        wt = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/unmodified', 'tree/removed', 'tree/modified'])
+        tt = TransformPreview(wt)
+        removed_id = tt.trans_id_tree_path('removed')
+        tt.delete_contents(removed_id)
+        tt.new_file('new', tt.root, 'contents')
+        modified_id = tt.trans_id_tree_path('modified')
+        tt.delete_contents(modified_id)
+        tt.create_file('modified-contents', modified_id)
+        self.addCleanup(tt.finalize)
+        tree = tt.get_preview_tree()
+        self.assertTrue(tree.has_filename('unmodified'))
+        self.assertFalse(tree.has_filename('not-present'))
+        self.assertFalse(tree.has_filename('removed'))
+        self.assertTrue(tree.has_filename('new'))
+        self.assertTrue(tree.has_filename('modified'))
+
     def test_is_executable(self):
         tree = self.make_branch_and_tree('tree')
         preview = TransformPreview(tree)
@@ -2929,7 +3346,7 @@
         self.assertEqual('contents', rev2_tree.get_file_text('file_id'))
 
     def test_ascii_limbo_paths(self):
-        self.requireFeature(tests.UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         branch = self.make_branch('any')
         tree = branch.repository.revision_tree(_mod_revision.NULL_REVISION)
         tt = TransformPreview(tree)
@@ -2952,7 +3369,7 @@
 
 class TestSerializeTransform(tests.TestCaseWithTransport):
 
-    _test_needs_features = [tests.UnicodeFilenameFeature]
+    _test_needs_features = [features.UnicodeFilenameFeature]
 
     def get_preview(self, tree=None):
         if tree is None:
@@ -3033,13 +3450,13 @@
         return self.make_records(attribs, contents)
 
     def test_serialize_symlink_creation(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tt = self.get_preview()
         tt.new_symlink(u'foo\u1234', tt.root, u'bar\u1234')
         self.assertSerializesTo(self.symlink_creation_records(), tt)
 
     def test_deserialize_symlink_creation(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         tt = self.get_preview()
         tt.deserialize(iter(self.symlink_creation_records()))
         abspath = tt._limbo_name('new-1')
@@ -3212,3 +3629,94 @@
         trans_id = tt.trans_id_tree_path('file')
         self.assertEqual((LINES_ONE,),
             tt._get_parents_texts(trans_id))
+
+
+class TestOrphan(tests.TestCaseWithTransport):
+
+    def test_no_orphan_for_transform_preview(self):
+        tree = self.make_branch_and_tree('tree')
+        tt = transform.TransformPreview(tree)
+        self.addCleanup(tt.finalize)
+        self.assertRaises(NotImplementedError, tt.new_orphan, 'foo', 'bar')
+
+    def _set_orphan_policy(self, wt, policy):
+        wt.branch.get_config().set_user_option('bzr.transform.orphan_policy',
+                                               policy)
+
+    def _prepare_orphan(self, wt):
+        self.build_tree(['dir/', 'dir/file', 'dir/foo'])
+        wt.add(['dir', 'dir/file'], ['dir-id', 'file-id'])
+        wt.commit('add dir and file ignoring foo')
+        tt = transform.TreeTransform(wt)
+        self.addCleanup(tt.finalize)
+        # dir and bar are deleted
+        dir_tid = tt.trans_id_tree_path('dir')
+        file_tid = tt.trans_id_tree_path('dir/file')
+        orphan_tid = tt.trans_id_tree_path('dir/foo')
+        tt.delete_contents(file_tid)
+        tt.unversion_file(file_tid)
+        tt.delete_contents(dir_tid)
+        tt.unversion_file(dir_tid)
+        # There should be a conflict because dir still contain foo
+        raw_conflicts = tt.find_conflicts()
+        self.assertLength(1, raw_conflicts)
+        self.assertEqual(('missing parent', 'new-1'), raw_conflicts[0])
+        return tt, orphan_tid
+
+    def test_new_orphan_created(self):
+        wt = self.make_branch_and_tree('.')
+        self._set_orphan_policy(wt, 'move')
+        tt, orphan_tid = self._prepare_orphan(wt)
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        remaining_conflicts = resolve_conflicts(tt)
+        self.assertEquals(['dir/foo has been orphaned in bzr-orphans'],
+                          warnings)
+        # Yeah for resolved conflicts !
+        self.assertLength(0, remaining_conflicts)
+        # We have a new orphan
+        self.assertEquals('foo.~1~', tt.final_name(orphan_tid))
+        self.assertEquals('bzr-orphans',
+                          tt.final_name(tt.final_parent(orphan_tid)))
+
+    def test_never_orphan(self):
+        wt = self.make_branch_and_tree('.')
+        self._set_orphan_policy(wt, 'conflict')
+        tt, orphan_tid = self._prepare_orphan(wt)
+        remaining_conflicts = resolve_conflicts(tt)
+        self.assertLength(1, remaining_conflicts)
+        self.assertEqual(('deleting parent', 'Not deleting', 'new-1'),
+                         remaining_conflicts.pop())
+
+    def test_orphan_error(self):
+        def bogus_orphan(tt, orphan_id, parent_id):
+            raise transform.OrphaningError(tt.final_name(orphan_id),
+                                           tt.final_name(parent_id))
+        transform.orphaning_registry.register('bogus', bogus_orphan,
+                                              'Raise an error when orphaning')
+        wt = self.make_branch_and_tree('.')
+        self._set_orphan_policy(wt, 'bogus')
+        tt, orphan_tid = self._prepare_orphan(wt)
+        remaining_conflicts = resolve_conflicts(tt)
+        self.assertLength(1, remaining_conflicts)
+        self.assertEqual(('deleting parent', 'Not deleting', 'new-1'),
+                         remaining_conflicts.pop())
+
+    def test_unknown_orphan_policy(self):
+        wt = self.make_branch_and_tree('.')
+        # Set a fictional policy nobody ever implemented
+        self._set_orphan_policy(wt, 'donttouchmypreciouuus')
+        tt, orphan_tid = self._prepare_orphan(wt)
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        remaining_conflicts = resolve_conflicts(tt)
+        # We fallback to the default policy which create a conflict
+        self.assertLength(1, remaining_conflicts)
+        self.assertEqual(('deleting parent', 'Not deleting', 'new-1'),
+                         remaining_conflicts.pop())
+        self.assertLength(1, warnings)
+        self.assertStartsWith(warnings[0], 'donttouchmypreciouuus')

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/test_transport.py	2011-09-12 16:31:46 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,14 +28,18 @@
     transport,
     urlutils,
     )
+from bzrlib.directory_service import directories
 from bzrlib.transport import (
     chroot,
     fakenfs,
+    http,
     local,
+    location_to_url,
     memory,
     pathfilter,
     readonly,
     )
+import bzrlib.transport.trace
 from bzrlib.tests import (
     features,
     test_server,
@@ -48,83 +52,72 @@
 class TestTransport(tests.TestCase):
     """Test the non transport-concrete class functionality."""
 
-    # FIXME: These tests should use addCleanup() and/or overrideAttr() instead
-    # of try/finally -- vila 20100205
-
     def test__get_set_protocol_handlers(self):
         handlers = transport._get_protocol_handlers()
-        self.assertNotEqual([], handlers.keys( ))
-        try:
-            transport._clear_protocol_handlers()
-            self.assertEqual([], transport._get_protocol_handlers().keys())
-        finally:
-            transport._set_protocol_handlers(handlers)
+        self.assertNotEqual([], handlers.keys())
+        transport._clear_protocol_handlers()
+        self.addCleanup(transport._set_protocol_handlers, handlers)
+        self.assertEqual([], transport._get_protocol_handlers().keys())
 
     def test_get_transport_modules(self):
         handlers = transport._get_protocol_handlers()
+        self.addCleanup(transport._set_protocol_handlers, handlers)
         # don't pollute the current handlers
         transport._clear_protocol_handlers()
+
         class SampleHandler(object):
             """I exist, isnt that enough?"""
-        try:
-            transport._clear_protocol_handlers()
-            transport.register_transport_proto('foo')
-            transport.register_lazy_transport('foo',
-                                              'bzrlib.tests.test_transport',
-                                              'TestTransport.SampleHandler')
-            transport.register_transport_proto('bar')
-            transport.register_lazy_transport('bar',
-                                              'bzrlib.tests.test_transport',
-                                              'TestTransport.SampleHandler')
-            self.assertEqual([SampleHandler.__module__,
-                              'bzrlib.transport.chroot',
-                              'bzrlib.transport.pathfilter'],
-                             transport._get_transport_modules())
-        finally:
-            transport._set_protocol_handlers(handlers)
+        transport._clear_protocol_handlers()
+        transport.register_transport_proto('foo')
+        transport.register_lazy_transport('foo',
+                                            'bzrlib.tests.test_transport',
+                                            'TestTransport.SampleHandler')
+        transport.register_transport_proto('bar')
+        transport.register_lazy_transport('bar',
+                                            'bzrlib.tests.test_transport',
+                                            'TestTransport.SampleHandler')
+        self.assertEqual([SampleHandler.__module__,
+                            'bzrlib.transport.chroot',
+                            'bzrlib.transport.pathfilter'],
+                            transport._get_transport_modules())
 
     def test_transport_dependency(self):
         """Transport with missing dependency causes no error"""
         saved_handlers = transport._get_protocol_handlers()
+        self.addCleanup(transport._set_protocol_handlers, saved_handlers)
         # don't pollute the current handlers
         transport._clear_protocol_handlers()
+        transport.register_transport_proto('foo')
+        transport.register_lazy_transport(
+            'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
         try:
-            transport.register_transport_proto('foo')
-            transport.register_lazy_transport(
-                'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
-            try:
-                transport.get_transport('foo://fooserver/foo')
-            except errors.UnsupportedProtocol, e:
-                e_str = str(e)
-                self.assertEquals('Unsupported protocol'
-                                  ' for url "foo://fooserver/foo":'
-                                  ' Unable to import library "some_lib":'
-                                  ' testing missing dependency', str(e))
-            else:
-                self.fail('Did not raise UnsupportedProtocol')
-        finally:
-            # restore original values
-            transport._set_protocol_handlers(saved_handlers)
+            transport.get_transport_from_url('foo://fooserver/foo')
+        except errors.UnsupportedProtocol, e:
+            e_str = str(e)
+            self.assertEquals('Unsupported protocol'
+                                ' for url "foo://fooserver/foo":'
+                                ' Unable to import library "some_lib":'
+                                ' testing missing dependency', str(e))
+        else:
+            self.fail('Did not raise UnsupportedProtocol')
 
     def test_transport_fallback(self):
         """Transport with missing dependency causes no error"""
         saved_handlers = transport._get_protocol_handlers()
-        try:
-            transport._clear_protocol_handlers()
-            transport.register_transport_proto('foo')
-            transport.register_lazy_transport(
-                'foo', 'bzrlib.tests.test_transport', 'BackupTransportHandler')
-            transport.register_lazy_transport(
-                'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
-            t = transport.get_transport('foo://fooserver/foo')
-            self.assertTrue(isinstance(t, BackupTransportHandler))
-        finally:
-            transport._set_protocol_handlers(saved_handlers)
+        self.addCleanup(transport._set_protocol_handlers, saved_handlers)
+        transport._clear_protocol_handlers()
+        transport.register_transport_proto('foo')
+        transport.register_lazy_transport(
+            'foo', 'bzrlib.tests.test_transport', 'BackupTransportHandler')
+        transport.register_lazy_transport(
+            'foo', 'bzrlib.tests.test_transport', 'BadTransportHandler')
+        t = transport.get_transport_from_url('foo://fooserver/foo')
+        self.assertTrue(isinstance(t, BackupTransportHandler))
 
     def test_ssh_hints(self):
         """Transport ssh:// should raise an error pointing out bzr+ssh://"""
         try:
-            transport.get_transport('ssh://fooserver/foo')
+            transport.get_transport_from_url('ssh://fooserver/foo')
         except errors.UnsupportedProtocol, e:
             e_str = str(e)
             self.assertEquals('Unsupported protocol'
@@ -145,17 +138,6 @@
         self.assertRaises(errors.ReadError, a_file.read, 40)
         a_file.close()
 
-    def test__combine_paths(self):
-        t = transport.Transport('/')
-        self.assertEqual('/home/sarah/project/foo',
-                         t._combine_paths('/home/sarah', 'project/foo'))
-        self.assertEqual('/etc',
-                         t._combine_paths('/home/sarah', '../../etc'))
-        self.assertEqual('/etc',
-                         t._combine_paths('/home/sarah', '../../../etc'))
-        self.assertEqual('/etc',
-                         t._combine_paths('/home/sarah', '/etc'))
-
     def test_local_abspath_non_local_transport(self):
         # the base implementation should throw
         t = memory.MemoryTransport()
@@ -218,33 +200,32 @@
 
     def test_coalesce_fudge(self):
         self.check([(10, 30, [(0, 10), (20, 10)]),
-                    (100, 10, [(0, 10),]),
+                    (100, 10, [(0, 10)]),
                    ], [(10, 10), (30, 10), (100, 10)],
-                   fudge=10
-                  )
+                   fudge=10)
+
     def test_coalesce_max_size(self):
         self.check([(10, 20, [(0, 10), (10, 10)]),
                     (30, 50, [(0, 50)]),
                     # If one range is above max_size, it gets its own coalesced
                     # offset
-                    (100, 80, [(0, 80),]),],
+                    (100, 80, [(0, 80)]),],
                    [(10, 10), (20, 10), (30, 50), (100, 80)],
-                   max_size=50
-                  )
+                   max_size=50)
 
     def test_coalesce_no_max_size(self):
-        self.check([(10, 170, [(0, 10), (10, 10), (20, 50), (70, 100)]),],
+        self.check([(10, 170, [(0, 10), (10, 10), (20, 50), (70, 100)])],
                    [(10, 10), (20, 10), (30, 50), (80, 100)],
                   )
 
     def test_coalesce_default_limit(self):
         # By default we use a 100MB max size.
-        ten_mb = 10*1024*1024
-        self.check([(0, 10*ten_mb, [(i*ten_mb, ten_mb) for i in range(10)]),
+        ten_mb = 10 * 1024 * 1024
+        self.check([(0, 10 * ten_mb, [(i * ten_mb, ten_mb) for i in range(10)]),
                     (10*ten_mb, ten_mb, [(0, ten_mb)])],
                    [(i*ten_mb, ten_mb) for i in range(11)])
-        self.check([(0, 11*ten_mb, [(i*ten_mb, ten_mb) for i in range(11)]),],
-                   [(i*ten_mb, ten_mb) for i in range(11)],
+        self.check([(0, 11 * ten_mb, [(i * ten_mb, ten_mb) for i in range(11)])],
+                   [(i * ten_mb, ten_mb) for i in range(11)],
                    max_size=1*1024*1024*1024)
 
 
@@ -255,7 +236,7 @@
         server.start_server()
         url = server.get_url()
         self.assertTrue(url in transport.transport_list_registry)
-        t = transport.get_transport(url)
+        t = transport.get_transport_from_url(url)
         del t
         server.stop_server()
         self.assertFalse(url in transport.transport_list_registry)
@@ -376,9 +357,9 @@
     def test_abspath(self):
         # The abspath is always relative to the chroot_url.
         server = chroot.ChrootServer(
-            transport.get_transport('memory:///foo/bar/'))
+            transport.get_transport_from_url('memory:///foo/bar/'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         self.assertEqual(server.get_url(), t.abspath('/'))
 
         subdir_t = t.clone('subdir')
@@ -386,9 +367,9 @@
 
     def test_clone(self):
         server = chroot.ChrootServer(
-            transport.get_transport('memory:///foo/bar/'))
+            transport.get_transport_from_url('memory:///foo/bar/'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         # relpath from root and root path are the same
         relpath_cloned = t.clone('foo')
         abspath_cloned = t.clone('/foo')
@@ -403,13 +384,13 @@
         This is so that it is not possible to escape a chroot by doing::
             url = chroot_transport.base
             parent_url = urlutils.join(url, '..')
-            new_t = transport.get_transport(parent_url)
+            new_t = transport.get_transport_from_url(parent_url)
         """
         server = chroot.ChrootServer(
-            transport.get_transport('memory:///path/subpath'))
+            transport.get_transport_from_url('memory:///path/subpath'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
-        new_t = transport.get_transport(t.base)
+        t = transport.get_transport_from_url(server.get_url())
+        new_t = transport.get_transport_from_url(t.base)
         self.assertEqual(t.server, new_t.server)
         self.assertEqual(t.base, new_t.base)
 
@@ -420,11 +401,12 @@
         This is so that it is not possible to escape a chroot by doing::
             url = chroot_transport.base
             parent_url = urlutils.join(url, '..')
-            new_t = transport.get_transport(parent_url)
+            new_t = transport.get_transport_from_url(parent_url)
         """
-        server = chroot.ChrootServer(transport.get_transport('memory:///path/'))
+        server = chroot.ChrootServer(
+            transport.get_transport_from_url('memory:///path/'))
         self.start_server(server)
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         self.assertRaises(
             errors.InvalidURLJoin, urlutils.join, t.base, '..')
 
@@ -440,11 +422,9 @@
         backing_transport = memory.MemoryTransport()
         server = chroot.ChrootServer(backing_transport)
         server.start_server()
-        try:
-            self.assertTrue(server.scheme
-                            in transport._get_protocol_handlers().keys())
-        finally:
-            server.stop_server()
+        self.addCleanup(server.stop_server)
+        self.assertTrue(server.scheme
+                        in transport._get_protocol_handlers().keys())
 
     def test_stop_server(self):
         backing_transport = memory.MemoryTransport()
@@ -458,10 +438,8 @@
         backing_transport = memory.MemoryTransport()
         server = chroot.ChrootServer(backing_transport)
         server.start_server()
-        try:
-            self.assertEqual('chroot-%d:///' % id(server), server.get_url())
-        finally:
-            server.stop_server()
+        self.addCleanup(server.stop_server)
+        self.assertEqual('chroot-%d:///' % id(server), server.get_url())
 
 
 class PathFilteringDecoratorTransportTest(tests.TestCase):
@@ -470,10 +448,10 @@
     def test_abspath(self):
         # The abspath is always relative to the base of the backing transport.
         server = pathfilter.PathFilteringServer(
-            transport.get_transport('memory:///foo/bar/'),
+            transport.get_transport_from_url('memory:///foo/bar/'),
             lambda x: x)
         server.start_server()
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         self.assertEqual(server.get_url(), t.abspath('/'))
 
         subdir_t = t.clone('subdir')
@@ -482,16 +460,16 @@
 
     def make_pf_transport(self, filter_func=None):
         """Make a PathFilteringTransport backed by a MemoryTransport.
-        
+
         :param filter_func: by default this will be a no-op function.  Use this
             parameter to override it."""
         if filter_func is None:
             filter_func = lambda x: x
         server = pathfilter.PathFilteringServer(
-            transport.get_transport('memory:///foo/bar/'), filter_func)
+            transport.get_transport_from_url('memory:///foo/bar/'), filter_func)
         server.start_server()
         self.addCleanup(server.stop_server)
-        return transport.get_transport(server.get_url())
+        return transport.get_transport_from_url(server.get_url())
 
     def test__filter(self):
         # _filter (with an identity func as filter_func) always returns
@@ -510,6 +488,7 @@
 
     def test_filter_invocation(self):
         filter_log = []
+
         def filter(path):
             filter_log.append(path)
             return path
@@ -540,10 +519,10 @@
         otherwise) the filtering by doing::
             url = filtered_transport.base
             parent_url = urlutils.join(url, '..')
-            new_t = transport.get_transport(parent_url)
+            new_t = transport.get_transport_from_url(parent_url)
         """
         t = self.make_pf_transport()
-        new_t = transport.get_transport(t.base)
+        new_t = transport.get_transport_from_url(t.base)
         self.assertEqual(t.server, new_t.server)
         self.assertEqual(t.base, new_t.base)
 
@@ -562,8 +541,8 @@
         # connect to '.' via http which is not listable
         server = HttpServer()
         self.start_server(server)
-        t = transport.get_transport('readonly+' + server.get_url())
-        self.failUnless(isinstance(t, readonly.ReadonlyTransportDecorator))
+        t = transport.get_transport_from_url('readonly+' + server.get_url())
+        self.assertIsInstance(t, readonly.ReadonlyTransportDecorator)
         self.assertEqual(False, t.listable())
         self.assertEqual(True, t.is_readonly())
 
@@ -601,7 +580,7 @@
         # the url should be decorated appropriately
         self.assertStartsWith(server.get_url(), 'fakenfs+')
         # and we should be able to get a transport for it
-        t = transport.get_transport(server.get_url())
+        t = transport.get_transport_from_url(server.get_url())
         # which must be a FakeNFSTransportDecorator instance.
         self.assertIsInstance(t, fakenfs.FakeNFSTransportDecorator)
 
@@ -684,7 +663,7 @@
         base_url = self._server.get_url()
         url = self._adjust_url(base_url, relpath)
         # try getting the transport via the regular interface:
-        t = transport.get_transport(url)
+        t = transport.get_transport_from_url(url)
         # vila--20070607 if the following are commented out the test suite
         # still pass. Is this really still needed or was it a forgotten
         # temporary fix ?
@@ -695,6 +674,43 @@
         return t
 
 
+class TestTransportFromPath(tests.TestCaseInTempDir):
+
+    def test_with_path(self):
+        t = transport.get_transport_from_path(self.test_dir)
+        self.assertIsInstance(t, local.LocalTransport)
+        self.assertEquals(t.base.rstrip("/"),
+            urlutils.local_path_to_url(self.test_dir))
+
+    def test_with_url(self):
+        t = transport.get_transport_from_path("file:")
+        self.assertIsInstance(t, local.LocalTransport)
+        self.assertEquals(t.base.rstrip("/"),
+            urlutils.local_path_to_url(os.path.join(self.test_dir, "file:")))
+
+
+class TestTransportFromUrl(tests.TestCaseInTempDir):
+
+    def test_with_path(self):
+        self.assertRaises(errors.InvalidURL, transport.get_transport_from_url,
+            self.test_dir)
+
+    def test_with_url(self):
+        url = urlutils.local_path_to_url(self.test_dir)
+        t = transport.get_transport_from_url(url)
+        self.assertIsInstance(t, local.LocalTransport)
+        self.assertEquals(t.base.rstrip("/"), url)
+
+    def test_with_url_and_segment_parameters(self):
+        url = urlutils.local_path_to_url(self.test_dir)+",branch=foo"
+        t = transport.get_transport_from_url(url)
+        self.assertIsInstance(t, local.LocalTransport)
+        self.assertEquals(t.base.rstrip("/"), url)
+        with open(os.path.join(self.test_dir, "afile"), 'w') as f:
+            f.write("data")
+        self.assertTrue(t.has("afile"))
+
+
 class TestLocalTransports(tests.TestCase):
 
     def test_get_transport_from_abspath(self):
@@ -722,6 +738,33 @@
         self.assertEquals(t.local_abspath(''), here)
 
 
+class TestLocalTransportWriteStream(tests.TestCaseWithTransport):
+
+    def test_local_fdatasync_calls_fdatasync(self):
+        """Check fdatasync on a stream tries to flush the data to the OS.
+        
+        We can't easily observe the external effect but we can at least see
+        it's called.
+        """
+        sentinel = object()
+        fdatasync = getattr(os, 'fdatasync', sentinel)
+        if fdatasync is sentinel:
+            raise tests.TestNotApplicable('fdatasync not supported')
+        t = self.get_transport('.')
+        calls = self.recordCalls(os, 'fdatasync')
+        w = t.open_write_stream('out')
+        w.write('foo')
+        w.fdatasync()
+        with open('out', 'rb') as f:
+            # Should have been flushed.
+            self.assertEquals(f.read(), 'foo')
+        self.assertEquals(len(calls), 1, calls)
+
+    def test_missing_directory(self):
+        t = self.get_transport('.')
+        self.assertRaises(errors.NoSuchFile, t.open_write_stream, 'dir/foo')
+
+
 class TestWin32LocalTransport(tests.TestCase):
 
     def test_unc_clone_to_root(self):
@@ -743,30 +786,30 @@
     def test_parse_url(self):
         t = transport.ConnectedTransport(
             'http://simple.example.com/home/source')
-        self.assertEquals(t._host, 'simple.example.com')
-        self.assertEquals(t._port, None)
-        self.assertEquals(t._path, '/home/source/')
-        self.failUnless(t._user is None)
-        self.failUnless(t._password is None)
+        self.assertEquals(t._parsed_url.host, 'simple.example.com')
+        self.assertEquals(t._parsed_url.port, None)
+        self.assertEquals(t._parsed_url.path, '/home/source/')
+        self.assertTrue(t._parsed_url.user is None)
+        self.assertTrue(t._parsed_url.password is None)
 
         self.assertEquals(t.base, 'http://simple.example.com/home/source/')
 
     def test_parse_url_with_at_in_user(self):
         # Bug 228058
         t = transport.ConnectedTransport('ftp://user@host.com@www.host.com/')
-        self.assertEquals(t._user, 'user@host.com')
+        self.assertEquals(t._parsed_url.user, 'user@host.com')
 
     def test_parse_quoted_url(self):
         t = transport.ConnectedTransport(
             'http://ro%62ey:h%40t@ex%41mple.com:2222/path')
-        self.assertEquals(t._host, 'exAmple.com')
-        self.assertEquals(t._port, 2222)
-        self.assertEquals(t._user, 'robey')
-        self.assertEquals(t._password, 'h@t')
-        self.assertEquals(t._path, '/path/')
+        self.assertEquals(t._parsed_url.host, 'exAmple.com')
+        self.assertEquals(t._parsed_url.port, 2222)
+        self.assertEquals(t._parsed_url.user, 'robey')
+        self.assertEquals(t._parsed_url.password, 'h@t')
+        self.assertEquals(t._parsed_url.path, '/path/')
 
         # Base should not keep track of the password
-        self.assertEquals(t.base, 'http://robey@exAmple.com:2222/path/')
+        self.assertEquals(t.base, 'http://ro%62ey@ex%41mple.com:2222/path/')
 
     def test_parse_invalid_url(self):
         self.assertRaises(errors.InvalidURL,
@@ -776,7 +819,8 @@
     def test_relpath(self):
         t = transport.ConnectedTransport('sftp://user@host.com/abs/path')
 
-        self.assertEquals(t.relpath('sftp://user@host.com/abs/path/sub'), 'sub')
+        self.assertEquals(t.relpath('sftp://user@host.com/abs/path/sub'),
+            'sub')
         self.assertRaises(errors.PathNotChild, t.relpath,
                           'http://user@host.com/abs/path/sub')
         self.assertRaises(errors.PathNotChild, t.relpath,
@@ -795,13 +839,13 @@
 
     def test_connection_sharing_propagate_credentials(self):
         t = transport.ConnectedTransport('ftp://user@host.com/abs/path')
-        self.assertEquals('user', t._user)
-        self.assertEquals('host.com', t._host)
+        self.assertEquals('user', t._parsed_url.user)
+        self.assertEquals('host.com', t._parsed_url.host)
         self.assertIs(None, t._get_connection())
-        self.assertIs(None, t._password)
+        self.assertIs(None, t._parsed_url.password)
         c = t.clone('subdir')
         self.assertIs(None, c._get_connection())
-        self.assertIs(None, t._password)
+        self.assertIs(None, t._parsed_url.password)
 
         # Simulate the user entering a password
         password = 'secret'
@@ -826,39 +870,40 @@
 
     def test_reuse_same_transport(self):
         possible_transports = []
-        t1 = transport.get_transport('http://foo/',
+        t1 = transport.get_transport_from_url('http://foo/',
                                      possible_transports=possible_transports)
         self.assertEqual([t1], possible_transports)
-        t2 = transport.get_transport('http://foo/',
+        t2 = transport.get_transport_from_url('http://foo/',
                                      possible_transports=[t1])
         self.assertIs(t1, t2)
 
         # Also check that final '/' are handled correctly
-        t3 = transport.get_transport('http://foo/path/')
-        t4 = transport.get_transport('http://foo/path',
+        t3 = transport.get_transport_from_url('http://foo/path/')
+        t4 = transport.get_transport_from_url('http://foo/path',
                                      possible_transports=[t3])
         self.assertIs(t3, t4)
 
-        t5 = transport.get_transport('http://foo/path')
-        t6 = transport.get_transport('http://foo/path/',
+        t5 = transport.get_transport_from_url('http://foo/path')
+        t6 = transport.get_transport_from_url('http://foo/path/',
                                      possible_transports=[t5])
         self.assertIs(t5, t6)
 
     def test_don_t_reuse_different_transport(self):
-        t1 = transport.get_transport('http://foo/path')
-        t2 = transport.get_transport('http://bar/path',
+        t1 = transport.get_transport_from_url('http://foo/path')
+        t2 = transport.get_transport_from_url('http://bar/path',
                                      possible_transports=[t1])
         self.assertIsNot(t1, t2)
 
 
 class TestTransportTrace(tests.TestCase):
 
-    def test_get(self):
-        t = transport.get_transport('trace+memory://')
-        self.assertIsInstance(t, bzrlib.transport.trace.TransportTraceDecorator)
+    def test_decorator(self):
+        t = transport.get_transport_from_url('trace+memory://')
+        self.assertIsInstance(
+            t, bzrlib.transport.trace.TransportTraceDecorator)
 
     def test_clone_preserves_activity(self):
-        t = transport.get_transport('trace+memory://')
+        t = transport.get_transport_from_url('trace+memory://')
         t2 = t.clone('.')
         self.assertTrue(t is not t2)
         self.assertTrue(t._activity is t2._activity)
@@ -868,7 +913,7 @@
     # still won't cause a test failure when the top level Transport API
     # changes; so there is little return doing that.
     def test_get(self):
-        t = transport.get_transport('trace+memory:///')
+        t = transport.get_transport_from_url('trace+memory:///')
         t.put_bytes('foo', 'barish')
         t.get('foo')
         expected_result = []
@@ -880,7 +925,7 @@
         self.assertEqual(expected_result, t._activity)
 
     def test_readv(self):
-        t = transport.get_transport('trace+memory:///')
+        t = transport.get_transport_from_url('trace+memory:///')
         t.put_bytes('foo', 'barish')
         list(t.readv('foo', [(0, 1), (3, 2)],
                      adjust_for_latency=True, upper_limit=6))
@@ -896,7 +941,7 @@
 class TestSSHConnections(tests.TestCaseWithTransport):
 
     def test_bzr_connect_to_bzr_ssh(self):
-        """User acceptance that get_transport of a bzr+ssh:// behaves correctly.
+        """get_transport of a bzr+ssh:// behaves correctly.
 
         bzr+ssh:// should cause bzr to run a remote bzr smart server over SSH.
         """
@@ -918,6 +963,7 @@
         # SSH channel ourselves.  Surely this has already been implemented
         # elsewhere?
         started = []
+
         class StubSSHServer(stub_sftp.StubServer):
 
             test = self
@@ -929,8 +975,8 @@
                     stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
                 # XXX: horribly inefficient, not to mention ugly.
-                # Start a thread for each of stdin/out/err, and relay bytes from
-                # the subprocess to channel and vice versa.
+                # Start a thread for each of stdin/out/err, and relay bytes
+                # from the subprocess to channel and vice versa.
                 def ferry_bytes(read, write, close):
                     while True:
                         bytes = read(1)
@@ -955,14 +1001,18 @@
         ssh_server = stub_sftp.SFTPFullAbsoluteServer(StubSSHServer)
         # We *don't* want to override the default SSH vendor: the detected one
         # is the one to use.
+
+        # FIXME: I don't understand the above comment, SFTPFullAbsoluteServer
+        # inherits from SFTPServer which forces the SSH vendor to
+        # ssh.ParamikoVendor(). So it's forced, not detected. --vila 20100623
         self.start_server(ssh_server)
-        port = ssh_server._listener.port
+        port = ssh_server.port
 
         if sys.platform == 'win32':
             bzr_remote_path = sys.executable + ' ' + self.get_bzr_path()
         else:
             bzr_remote_path = self.get_bzr_path()
-        os.environ['BZR_REMOTE_PATH'] = bzr_remote_path
+        self.overrideEnv('BZR_REMOTE_PATH', bzr_remote_path)
 
         # Access the branch via a bzr+ssh URL.  The BZR_REMOTE_PATH environment
         # variable is used to tell bzr what command to run on the remote end.
@@ -989,3 +1039,62 @@
         # And the rest are threads
         for t in started[1:]:
             t.join()
+
+
+class TestUnhtml(tests.TestCase):
+
+    """Tests for unhtml_roughly"""
+
+    def test_truncation(self):
+        fake_html = "<p>something!\n" * 1000
+        result = http.unhtml_roughly(fake_html)
+        self.assertEquals(len(result), 1000)
+        self.assertStartsWith(result, " something!")
+
+
+class SomeDirectory(object):
+
+    def look_up(self, name, url):
+        return "http://bar"
+
+
+class TestLocationToUrl(tests.TestCase):
+
+    def get_base_location(self):
+        path = osutils.abspath('/foo/bar')
+        if path.startswith('/'):
+            url = 'file://%s' % (path,)
+        else:
+            # On Windows, abspaths start with the drive letter, so we have to
+            # add in the extra '/'
+            url = 'file:///%s' % (path,)
+        return path, url
+
+    def test_regular_url(self):
+        self.assertEquals("file://foo", location_to_url("file://foo"))
+
+    def test_directory(self):
+        directories.register("bar:", SomeDirectory, "Dummy directory")
+        self.addCleanup(directories.remove, "bar:")
+        self.assertEquals("http://bar", location_to_url("bar:"))
+
+    def test_unicode_url(self):
+        self.assertRaises(errors.InvalidURL, location_to_url,
+            "http://fo/\xc3\xaf".decode("utf-8"))
+
+    def test_unicode_path(self):
+        path, url = self.get_base_location()
+        location = path + "\xc3\xaf".decode("utf-8")
+        url += '%C3%AF'
+        self.assertEquals(url, location_to_url(location))
+
+    def test_path(self):
+        path, url = self.get_base_location()
+        self.assertEquals(url, location_to_url(path))
+
+    def test_relative_file_url(self):
+        self.assertEquals(urlutils.local_path_to_url(".") + "/bar",
+            location_to_url("file:bar"))
+
+    def test_absolute_file_url(self):
+        self.assertEquals("file:///bar", location_to_url("file:/bar"))

=== modified file 'bzrlib/tests/test_transport_log.py'
--- bzrlib/tests/test_transport_log.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_transport_log.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,8 +18,6 @@
 """Tests for log+ transport decorator."""
 
 
-import types
-
 from bzrlib import transport
 from bzrlib.tests import TestCaseWithMemoryTransport
 from bzrlib.trace import mutter

=== modified file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	2009-08-28 05:00:33 +0000
+++ bzrlib/tests/test_tree.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2008 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
 from bzrlib import (
     errors,
     revision,
-    tests,
     tree as _mod_tree,
     )
 from bzrlib.tests import TestCaseWithTransport

=== modified file 'bzrlib/tests/test_treeshape.py'
--- bzrlib/tests/test_treeshape.py	2010-07-15 13:55:46 +0000
+++ bzrlib/tests/test_treeshape.py	2011-06-14 01:26:41 +0000
@@ -19,6 +19,9 @@
 
 
 from bzrlib import tests
+from bzrlib.tests import (
+    features,
+    )
 
 
 class TestTreeShape(tests.TestCaseWithTransport):
@@ -30,12 +33,12 @@
             ('.bzr/',),
             ('.bzr/README', 'hello'),
             ])
-        self.failUnlessExists('foo')
-        self.failUnlessExists('.bzr/README')
+        self.assertPathExists('foo')
+        self.assertPathExists('.bzr/README')
         self.assertFileEqual('hello', '.bzr/README')
 
     def test_build_tree_symlink(self):
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(features.SymlinkFeature)
         self.build_tree_contents([('link@', 'target')])
         self.assertEqual('target',
             os.readlink('link'))

=== modified file 'bzrlib/tests/test_tuned_gzip.py'
--- bzrlib/tests/test_tuned_gzip.py	2009-06-02 19:56:24 +0000
+++ bzrlib/tests/test_tuned_gzip.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,11 +21,14 @@
 # do not use bzrlib test cases here - this should be suitable for sending
 # upstream.
 from cStringIO import StringIO
-from unittest import TestCase
 import zlib
 
 
-from bzrlib import tuned_gzip
+from bzrlib import (
+    symbol_versioning,
+    tuned_gzip,
+    tests,
+    )
 
 
 class FakeDecompress(object):
@@ -42,7 +45,7 @@
         return ''
 
 
-class TestFakeDecompress(TestCase):
+class TestFakeDecompress(tests.TestCase):
     """We use a fake decompressor to test GzipFile.
 
     This class tests the behaviours we want from it.
@@ -67,14 +70,16 @@
         self.assertEqual('1234567', decompress.unused_data)
 
 
-class TestGzip(TestCase):
+class TestGzip(tests.TestCase):
 
     def test__read_short_remainder(self):
         # a _read call at the end of a compressed hunk should
         # read more bytes if there is less than 8 bytes (the
         # gzip trailer) unread.
         stream = StringIO('\0\0\0\0\0\0\0\0')
-        myfile = tuned_gzip.GzipFile(fileobj=stream)
+        myfile = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            tuned_gzip.GzipFile, fileobj=stream)
         # disable the _new_member check, we are microtesting.
         myfile._new_member = False
         myfile.crc = zlib.crc32('')
@@ -84,17 +89,30 @@
         # all the data should have been read now
         self.assertEqual('', stream.read())
         # and it should be new member time in the stream.
-        self.failUnless(myfile._new_member)
-
-
-class TestToGzip(TestCase):
+        self.assertTrue(myfile._new_member)
+
+    def test_negative_crc(self):
+        """Content with a negative crc should not break when written"""
+        sio = StringIO()
+        gfile = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            tuned_gzip.GzipFile, mode="w", fileobj=sio)
+        gfile.write("\xFF")
+        gfile.close()
+        self.assertEqual(gfile.crc & 0xFFFFFFFFL, 0xFF000000L)
+        self.assertEqual(sio.getvalue()[-8:-4], "\x00\x00\x00\xFF")
+
+
+class TestToGzip(tests.TestCase):
 
     def assertToGzip(self, chunks):
         bytes = ''.join(chunks)
         gzfromchunks = tuned_gzip.chunks_to_gzip(chunks)
         gzfrombytes = tuned_gzip.bytes_to_gzip(bytes)
         self.assertEqual(gzfrombytes, gzfromchunks)
-        decoded = tuned_gzip.GzipFile(fileobj=StringIO(gzfromchunks)).read()
+        decoded = self.applyDeprecated(
+            symbol_versioning.deprecated_in((2, 3, 0)),
+            tuned_gzip.GzipFile, fileobj=StringIO(gzfromchunks)).read()
         self.assertEqual(bytes, decoded)
 
     def test_single_chunk(self):

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2010-07-17 16:46:44 +0000
+++ bzrlib/tests/test_ui.py	2011-10-08 19:01:59 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,28 +17,26 @@
 """Tests for the bzrlib ui
 """
 
-import os
-import re
 import time
 
 from StringIO import StringIO
 
+from testtools.matchers import *
+
 from bzrlib import (
     config,
-    errors,
     remote,
-    repository,
     tests,
     ui as _mod_ui,
     )
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    )
 from bzrlib.tests import (
     fixtures,
     test_progress,
     )
 from bzrlib.ui import text as _mod_ui_text
+from bzrlib.tests.testui import (
+    ProgressRecordingUIFactory,
+    )
 
 
 class TestUIConfiguration(tests.TestCaseWithTransport):
@@ -50,16 +48,28 @@
         ui = tests.TestUIFactory(stdin=None,
             stdout=tests.StringIOWrapper(),
             stderr=tests.StringIOWrapper())
-        os = ui.make_output_stream()
-        self.assertEquals(os.encoding, enc)
+        output = ui.make_output_stream()
+        self.assertEquals(output.encoding, enc)
 
 
 class TestTextUIFactory(tests.TestCase):
 
+    def make_test_ui_factory(self, stdin_contents):
+        ui = tests.TestUIFactory(stdin=stdin_contents,
+                                 stdout=tests.StringIOWrapper(),
+                                 stderr=tests.StringIOWrapper())
+        return ui
+
+    def test_text_factory_confirm(self):
+        # turns into reading a regular boolean
+        ui = self.make_test_ui_factory('n\n')
+        self.assertEquals(ui.confirm_action(u'Should %(thing)s pass?',
+            'bzrlib.tests.test_ui.confirmation',
+            {'thing': 'this'},),
+            False)
+
     def test_text_factory_ascii_password(self):
-        ui = tests.TestUIFactory(stdin='secret\n',
-                                 stdout=tests.StringIOWrapper(),
-                                 stderr=tests.StringIOWrapper())
+        ui = self.make_test_ui_factory('secret\n')
         pb = ui.nested_progress_bar()
         try:
             self.assertEqual('secret',
@@ -80,9 +90,7 @@
         We can't predict what encoding users will have for stdin, so we force
         it to utf8 to test that we transport the password correctly.
         """
-        ui = tests.TestUIFactory(stdin=u'baz\u1234'.encode('utf8'),
-                                 stdout=tests.StringIOWrapper(),
-                                 stderr=tests.StringIOWrapper())
+        ui = self.make_test_ui_factory(u'baz\u1234'.encode('utf8'))
         ui.stderr.encoding = ui.stdout.encoding = ui.stdin.encoding = 'utf8'
         pb = ui.nested_progress_bar()
         try:
@@ -103,6 +111,8 @@
     def test_text_ui_get_boolean(self):
         stdin = tests.StringIOWrapper("y\n" # True
                                       "n\n" # False
+                                      " \n y \n" # True
+                                      " no \n" # False
                                       "yes with garbage\nY\n" # True
                                       "not an answer\nno\n" # False
                                       "I'm sure!\nyes\n" # True
@@ -111,15 +121,83 @@
         stdout = tests.StringIOWrapper()
         stderr = tests.StringIOWrapper()
         factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
-        self.assertEqual(True, factory.get_boolean(""))
-        self.assertEqual(False, factory.get_boolean(""))
-        self.assertEqual(True, factory.get_boolean(""))
-        self.assertEqual(False, factory.get_boolean(""))
-        self.assertEqual(True, factory.get_boolean(""))
-        self.assertEqual(False, factory.get_boolean(""))
-        self.assertEqual("foo\n", factory.stdin.read())
-        # stdin should be empty
-        self.assertEqual('', factory.stdin.readline())
+        self.assertEqual(True, factory.get_boolean(u""))
+        self.assertEqual(False, factory.get_boolean(u""))
+        self.assertEqual(True, factory.get_boolean(u""))
+        self.assertEqual(False, factory.get_boolean(u""))
+        self.assertEqual(True, factory.get_boolean(u""))
+        self.assertEqual(False, factory.get_boolean(u""))
+        self.assertEqual(True, factory.get_boolean(u""))
+        self.assertEqual(False, factory.get_boolean(u""))
+        self.assertEqual("foo\n", factory.stdin.read())
+        # stdin should be empty
+        self.assertEqual('', factory.stdin.readline())
+        # return false on EOF
+        self.assertEqual(False, factory.get_boolean(u""))
+
+    def test_text_ui_choose_bad_parameters(self):
+        stdin = tests.StringIOWrapper()
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        # invalid default index
+        self.assertRaises(ValueError, factory.choose, u"", u"&Yes\n&No", 3)
+        # duplicated choice
+        self.assertRaises(ValueError, factory.choose, u"", u"&choice\n&ChOiCe")
+        # duplicated shortcut
+        self.assertRaises(ValueError, factory.choose, u"", u"&choice1\nchoi&ce2")
+
+    def test_text_ui_choose_prompt(self):
+        stdin = tests.StringIOWrapper()
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        # choices with explicit shortcuts
+        factory.choose(u"prompt", u"&yes\n&No\nmore &info")
+        self.assertEqual("prompt ([y]es, [N]o, more [i]nfo): \n", factory.stderr.getvalue())
+        # automatic shortcuts
+        factory.stderr.truncate(0)
+        factory.choose(u"prompt", u"yes\nNo\nmore info")
+        self.assertEqual("prompt ([y]es, [N]o, [m]ore info): \n", factory.stderr.getvalue())
+
+    def test_text_ui_choose_return_values(self):
+        choose = lambda: factory.choose(u"", u"&Yes\n&No\nMaybe\nmore &info", 3)
+        stdin = tests.StringIOWrapper("y\n" # 0
+                                      "n\n" # 1
+                                      " \n" # default: 3
+                                      " no \n" # 1
+                                      "b\na\nd \n" # bad shortcuts, all ignored
+                                      "yes with garbage\nY\n" # 0
+                                      "not an answer\nno\n" # 1
+                                      "info\nmore info\n" # 3
+                                      "Maybe\n" # 2
+                                      "foo\n")
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        self.assertEqual(0, choose())
+        self.assertEqual(1, choose())
+        self.assertEqual(3, choose())
+        self.assertEqual(1, choose())
+        self.assertEqual(0, choose())
+        self.assertEqual(1, choose())
+        self.assertEqual(3, choose())
+        self.assertEqual(2, choose())
+        self.assertEqual("foo\n", factory.stdin.read())
+        # stdin should be empty
+        self.assertEqual('', factory.stdin.readline())
+        # return None on EOF
+        self.assertEqual(None, choose())
+
+    def test_text_ui_choose_no_default(self):
+        stdin = tests.StringIOWrapper(" \n" # no default, invalid!
+                                      " yes \n" # 0
+                                      "foo\n")
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        self.assertEqual(0, factory.choose(u"", u"&Yes\n&No"))
+        self.assertEqual("foo\n", factory.stdin.read())
 
     def test_text_ui_get_integer(self):
         stdin = tests.StringIOWrapper(
@@ -129,22 +207,22 @@
         stdout = tests.StringIOWrapper()
         stderr = tests.StringIOWrapper()
         factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
-        self.assertEqual(1, factory.get_integer(""))
-        self.assertEqual(-2, factory.get_integer(""))
-        self.assertEqual(42, factory.get_integer(""))
+        self.assertEqual(1, factory.get_integer(u""))
+        self.assertEqual(-2, factory.get_integer(u""))
+        self.assertEqual(42, factory.get_integer(u""))
 
     def test_text_factory_prompt(self):
         # see <https://launchpad.net/bugs/365891>
         StringIO = tests.StringIOWrapper
         factory = _mod_ui_text.TextUIFactory(StringIO(), StringIO(), StringIO())
-        factory.prompt('foo %2e')
+        factory.prompt(u'foo %2e')
         self.assertEqual('', factory.stdout.getvalue())
         self.assertEqual('foo %2e', factory.stderr.getvalue())
 
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         out = test_progress._TTYStringIO()
-        os.environ['TERM'] = 'xterm'
+        self.overrideEnv('TERM', 'xterm')
         factory = _mod_ui_text.TextUIFactory(
             stdin=tests.StringIOWrapper("yada\ny\n"),
             stdout=out, stderr=out)
@@ -158,12 +236,12 @@
                          self.apply_redirected(None, factory.stdout,
                                                factory.stdout,
                                                factory.get_boolean,
-                                               "what do you want"))
+                                               u"what do you want"))
         output = out.getvalue()
         self.assertContainsRe(output,
             "| foo *\r\r  *\r*")
-        self.assertContainsRe(output,
-            r"what do you want\? \[y/n\]: what do you want\? \[y/n\]: ")
+        self.assertContainsString(output,
+            r"what do you want? ([y]es, [n]o): what do you want? ([y]es, [n]o): ")
         # stdin should have been totally consumed
         self.assertEqual('', factory.stdin.readline())
 
@@ -187,10 +265,10 @@
         factory.stdout.encoding = "utf8"
         # there is no output from the base factory
         self.assertEqual("someuser",
-                         factory.get_username('Hello %(host)s', host='some'))
+                         factory.get_username(u'Hello %(host)s', host='some'))
         self.assertEquals("Hello some: ", factory.stderr.getvalue())
         self.assertEquals('', factory.stdout.getvalue())
-        self.assertEqual("", factory.get_username("Gebruiker"))
+        self.assertEqual("", factory.get_username(u"Gebruiker"))
         # stdin should be empty
         self.assertEqual('', factory.stdin.readline())
 
@@ -212,7 +290,7 @@
             pb.finished()
 
     def test_quietness(self):
-        os.environ['BZR_PROGRESS_BAR'] = 'text'
+        self.overrideEnv('BZR_PROGRESS_BAR', 'text')
         ui_factory = _mod_ui_text.TextUIFactory(None,
             test_progress._TTYStringIO(),
             test_progress._TTYStringIO())
@@ -224,7 +302,7 @@
 
     def test_text_ui_show_user_warning(self):
         from bzrlib.repofmt.groupcompress_repo import RepositoryFormat2a
-        from bzrlib.repofmt.pack_repo import RepositoryFormatKnitPack5
+        from bzrlib.repofmt.knitpack_repo import RepositoryFormatKnitPack5
         err = StringIO()
         out = StringIO()
         ui = tests.TextUIFactory(stdin=None, stdout=out, stderr=err)
@@ -299,12 +377,8 @@
             # however, it can still be forced on
             (FileStringIO, 'dumb', 'text', _mod_ui_text.TextProgressView),
             ):
-            os.environ['TERM'] = term
-            if pb is None:
-                if 'BZR_PROGRESS_BAR' in os.environ:
-                    del os.environ['BZR_PROGRESS_BAR']
-            else:
-                os.environ['BZR_PROGRESS_BAR'] = pb
+            self.overrideEnv('TERM', term)
+            self.overrideEnv('BZR_PROGRESS_BAR', pb)
             stdin = file_class('')
             stderr = file_class()
             stdout = file_class()
@@ -321,10 +395,7 @@
         stderr = test_progress._NonTTYStringIO()
         stdout = test_progress._NonTTYStringIO()
         for term_type in ['dumb', None, 'xterm']:
-            if term_type is None:
-                del os.environ['TERM']
-            else:
-                os.environ['TERM'] = term_type
+            self.overrideEnv('TERM', term_type)
             uif = _mod_ui.make_ui_for_terminal(stdin, stdout, stderr)
             self.assertIsInstance(uif, _mod_ui_text.TextUIFactory,
                 'TERM=%r' % (term_type,))
@@ -351,7 +422,7 @@
         self.assertRaises(
             NotImplementedError,
             self.apply_redirected,
-            None, stdout, stdout, factory.get_boolean, "foo")
+            None, stdout, stdout, factory.get_boolean, u"foo")
 
 
 class TestUIFactoryTests(tests.TestCase):
@@ -370,12 +441,12 @@
 
     def test_canned_input_get_input(self):
         uif = _mod_ui.CannedInputUIFactory([True, 'mbp', 'password', 42])
-        self.assertEqual(True, uif.get_boolean('Extra cheese?'))
-        self.assertEqual('mbp', uif.get_username('Enter your user name'))
+        self.assertEqual(True, uif.get_boolean(u'Extra cheese?'))
+        self.assertEqual('mbp', uif.get_username(u'Enter your user name'))
         self.assertEqual('password',
-                         uif.get_password('Password for %(host)s',
+                         uif.get_password(u'Password for %(host)s',
                                           host='example.com'))
-        self.assertEqual(42, uif.get_integer('And all that jazz ?'))
+        self.assertEqual(42, uif.get_integer(u'And all that jazz ?'))
 
 
 class TestBoolFromString(tests.TestCase):
@@ -432,3 +503,52 @@
         self.assertIsNone('0', av)
         self.assertIsNone('on', av)
         self.assertIsNone('off', av)
+
+
+class TestConfirmationUserInterfacePolicy(tests.TestCase):
+
+    def test_confirm_action_default(self):
+        base_ui = _mod_ui.NoninteractiveUIFactory()
+        for answer in [True, False]:
+            self.assertEquals(
+                _mod_ui.ConfirmationUserInterfacePolicy(base_ui, answer, {})
+                .confirm_action("Do something?",
+                    "bzrlib.tests.do_something", {}),
+                answer)
+
+    def test_confirm_action_specific(self):
+        base_ui = _mod_ui.NoninteractiveUIFactory()
+        for default_answer in [True, False]:
+            for specific_answer in [True, False]:
+                for conf_id in ['given_id', 'other_id']:
+                    wrapper = _mod_ui.ConfirmationUserInterfacePolicy(
+                        base_ui, default_answer, dict(given_id=specific_answer))
+                    result = wrapper.confirm_action("Do something?", conf_id, {})
+                    if conf_id == 'given_id':
+                        self.assertEquals(result, specific_answer)
+                    else:
+                        self.assertEquals(result, default_answer)
+
+    def test_repr(self):
+        base_ui = _mod_ui.NoninteractiveUIFactory()
+        wrapper = _mod_ui.ConfirmationUserInterfacePolicy(
+            base_ui, True, dict(a=2))
+        self.assertThat(repr(wrapper),
+            Equals("ConfirmationUserInterfacePolicy("
+                "NoninteractiveUIFactory(), True, {'a': 2})"))
+
+
+class TestProgressRecordingUI(tests.TestCase):
+    """Test test-oriented UIFactory that records progress updates"""
+
+    def test_nested_ignore_depth_beyond_one(self):
+        # we only want to capture the first level out progress, not
+        # want sub-components might do. So we have nested bars ignored.
+        factory = ProgressRecordingUIFactory()
+        pb1 = factory.nested_progress_bar()
+        pb1.update('foo', 0, 1)
+        pb2 = factory.nested_progress_bar()
+        pb2.update('foo', 0, 1)
+        pb2.finished()
+        pb1.finished()
+        self.assertEqual([("update", 0, 1, 'foo')], factory._calls)

=== modified file 'bzrlib/tests/test_uncommit.py'
--- bzrlib/tests/test_uncommit.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_uncommit.py	2011-08-22 10:44:29 +0000
@@ -49,7 +49,7 @@
         self.assertEqual((1, history[0]), tree.branch.last_revision_info())
 
         # The file should not be removed
-        self.failUnlessExists('tree/two')
+        self.assertPathExists('tree/two')
         # And it should still be listed as added
         self.assertIsNot(None, tree.path2id('two'))
 
@@ -96,3 +96,30 @@
         # If this tree isn't bound, local=True raises an exception
         self.assertRaises(errors.LocalRequiresBoundBranch,
             uncommit.uncommit, tree.branch, tree=tree, local=True)
+
+    def test_uncommit_remove_tags(self):
+        tree, history = self.make_linear_tree()
+        self.assertEqual(history[1], tree.last_revision())
+        self.assertEqual((2, history[1]), tree.branch.last_revision_info())
+        tree.branch.tags.set_tag(u"pointsatexisting", history[0])
+        tree.branch.tags.set_tag(u"pointsatremoved", history[1])
+        uncommit.uncommit(tree.branch, tree=tree)
+        self.assertEqual(history[0], tree.last_revision())
+        self.assertEqual((1, history[0]), tree.branch.last_revision_info())
+        self.assertEqual({
+            "pointsatexisting": history[0]
+            }, tree.branch.tags.get_tag_dict())
+
+    def test_uncommit_keep_tags(self):
+        tree, history = self.make_linear_tree()
+        self.assertEqual(history[1], tree.last_revision())
+        self.assertEqual((2, history[1]), tree.branch.last_revision_info())
+        tree.branch.tags.set_tag(u"pointsatexisting", history[0])
+        tree.branch.tags.set_tag(u"pointsatremoved", history[1])
+        uncommit.uncommit(tree.branch, tree=tree, keep_tags=True)
+        self.assertEqual(history[0], tree.last_revision())
+        self.assertEqual((1, history[0]), tree.branch.last_revision_info())
+        self.assertEqual({
+            "pointsatexisting": history[0],
+            "pointsatremoved": history[1],
+            }, tree.branch.tags.get_tag_dict())

=== modified file 'bzrlib/tests/test_upgrade.py'
--- bzrlib/tests/test_upgrade.py	2010-07-29 11:17:57 +0000
+++ bzrlib/tests/test_upgrade.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,167 +22,34 @@
 # TODO queue for upgrade:
 # test the error message when upgrading an unknown BzrDir format.
 
-import base64
-import os
-import sys
-
 from bzrlib import (
-    branch as _mod_branch,
+    branch,
     bzrdir,
-    progress,
-    repository,
-    transport,
+    tests,
+    upgrade,
     workingtree,
     workingtree_4,
     )
-import bzrlib.branch
-from bzrlib.branch import Branch
-from bzrlib.tests import TestCaseWithTransport
-from bzrlib.upgrade import upgrade
-
-
-class TestUpgrade(TestCaseWithTransport):
-
-    def test_upgrade_simple(self):
-        """Upgrade simple v0.0.4 format to latest format"""
-        eq = self.assertEquals
-        self.build_tree_contents(_upgrade1_template)
-        upgrade(u'.')
-        control = bzrdir.BzrDir.open('.')
-        b = control.open_branch()
-        # tsk, peeking under the covers.
-        self.failUnless(
-            isinstance(
-                control._format,
-                bzrdir.BzrDirFormat.get_default_format().__class__))
-        rh = b.revision_history()
-        eq(rh,
-           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
-            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
-        rt = b.repository.revision_tree(rh[0])
-        foo_id = 'foo-20051004035605-91e788d1875603ae'
-        rt.lock_read()
-        try:
-            eq(rt.get_file_text(foo_id), 'initial contents\n')
-        finally:
-            rt.unlock()
-        rt = b.repository.revision_tree(rh[1])
-        rt.lock_read()
-        try:
-            eq(rt.get_file_text(foo_id), 'new contents\n')
-        finally:
-            rt.unlock()
-        # check a backup was made:
-        backup_dir = 'backup.bzr.~1~'
-        t = transport.get_transport(b.base)
-        t.stat(backup_dir)
-        t.stat(backup_dir + '/README')
-        t.stat(backup_dir + '/branch-format')
-        t.stat(backup_dir + '/revision-history')
-        t.stat(backup_dir + '/merged-patches')
-        t.stat(backup_dir + '/pending-merged-patches')
-        t.stat(backup_dir + '/pending-merges')
-        t.stat(backup_dir + '/branch-name')
-        t.stat(backup_dir + '/branch-lock')
-        t.stat(backup_dir + '/inventory')
-        t.stat(backup_dir + '/stat-cache')
-        t.stat(backup_dir + '/text-store')
-        t.stat(backup_dir + '/text-store/foo-20051004035611-1591048e9dc7c2d4.gz')
-        t.stat(backup_dir + '/text-store/foo-20051004035756-4081373d897c3453.gz')
-        t.stat(backup_dir + '/inventory-store/')
-        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
-        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
-        t.stat(backup_dir + '/revision-store/')
-        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
-        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
-
-    def test_upgrade_with_ghosts(self):
-        """Upgrade v0.0.4 tree containing ghost references.
-
-        That is, some of the parents of revisions mentioned in the branch
-        aren't present in the branch's storage.
-
-        This shouldn't normally happen in branches created entirely in
-        bzr, but can happen in branches imported from baz and arch, or from
-        other systems, where the importer knows about a revision but not
-        its contents."""
-        eq = self.assertEquals
-        self.build_tree_contents(_ghost_template)
-        upgrade(u'.')
-        b = Branch.open(u'.')
-        revision_id = b.revision_history()[1]
-        rev = b.repository.get_revision(revision_id)
-        eq(len(rev.parent_ids), 2)
-        eq(rev.parent_ids[1], 'wibble@wobble-2')
-
-    def test_upgrade_makes_dir_weaves(self):
-        self.build_tree_contents(_upgrade_dir_template)
-        old_repodir = bzrlib.bzrdir.BzrDir.open_unsupported('.')
-        old_repo_format = old_repodir.open_repository()._format
-        upgrade('.')
-        # this is the path to the literal file. As format changes
-        # occur it needs to be updated. FIXME: ask the store for the
-        # path.
-        repo = bzrlib.repository.Repository.open('.')
-        # it should have changed the format
-        self.assertNotEqual(old_repo_format.__class__, repo._format.__class__)
-        # and we should be able to read the names for the file id
-        # 'dir-20051005095101-da1441ea3fa6917a'
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        text_keys = repo.texts.keys()
-        dir_keys = [key for key in text_keys if key[0] ==
-                'dir-20051005095101-da1441ea3fa6917a']
-        self.assertNotEqual([], dir_keys)
-
-    def test_upgrade_to_meta_sets_workingtree_last_revision(self):
-        self.build_tree_contents(_upgrade_dir_template)
-        upgrade('.', bzrdir.BzrDirMetaFormat1())
-        tree = workingtree.WorkingTree.open('.')
-        self.assertEqual([tree.branch.revision_history()[-1]],
-            tree.get_parent_ids())
-
-    def test_upgrade_v6_to_meta_no_workingtree(self):
-        # Some format6 branches do not have checkout files. Upgrading
-        # such a branch to metadir must not setup a working tree.
-        self.build_tree_contents(_upgrade1_template)
-        upgrade('.', bzrdir.BzrDirFormat6())
-        t = transport.get_transport('.')
-        t.delete_multi(['.bzr/pending-merges', '.bzr/inventory'])
-        self.assertFalse(t.has('.bzr/stat-cache'))
-        # XXX: upgrade fails if a backup.bzr is already present
-        # -- David Allouche 2006-08-11
-        t.delete_tree('backup.bzr.~1~')
-        # At this point, we have a format6 branch without checkout files.
-        upgrade('.', bzrdir.BzrDirMetaFormat1())
-        # The upgrade should not have set up a working tree.
-        control = bzrdir.BzrDir.open('.')
-        self.assertFalse(control.has_workingtree())
-        # We have covered the scope of this test, we may as well check that
-        # upgrade has not eaten our data, even if it's a bit redundant with
-        # other tests.
-        self.failUnless(isinstance(control._format, bzrdir.BzrDirMetaFormat1))
-        branch = control.open_branch()
-        self.assertEquals(branch.revision_history(),
-           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
-            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+
+
+class TestUpgrade(tests.TestCaseWithTransport):
 
     def test_upgrade_rich_root(self):
         tree = self.make_branch_and_tree('tree', format='rich-root')
         rev_id = tree.commit('first post')
-        upgrade('tree')
+        upgrade.upgrade('tree')
 
     def test_convert_branch5_branch6(self):
-        branch = self.make_branch('branch', format='knit')
-        branch.set_revision_history(['AB', 'CD'])
-        branch.set_parent('file:///EF')
-        branch.set_bound_location('file:///GH')
-        branch.set_push_location('file:///IJ')
+        b = self.make_branch('branch', format='knit')
+        b._set_revision_history(['CD'])
+        b.set_parent('file:///EF')
+        b.set_bound_location('file:///GH')
+        b.set_push_location('file:///IJ')
         target = bzrdir.format_registry.make_bzrdir('dirstate-with-subtree')
-        converter = branch.bzrdir._format.get_converter(target)
-        converter.convert(branch.bzrdir, None)
-        new_branch = _mod_branch.Branch.open(self.get_url('branch'))
-        self.assertIs(new_branch.__class__, _mod_branch.BzrBranch6)
+        converter = b.bzrdir._format.get_converter(target)
+        converter.convert(b.bzrdir, None)
+        new_branch = branch.Branch.open(self.get_url('branch'))
+        self.assertIs(new_branch.__class__, branch.BzrBranch6)
         self.assertEqual('CD', new_branch.last_revision())
         self.assertEqual('file:///EF', new_branch.get_parent())
         self.assertEqual('file:///GH', new_branch.get_bound_location())
@@ -190,21 +57,21 @@
         self.assertEqual('file:///IJ',
             branch_config.get_user_option('push_location'))
 
-        branch2 = self.make_branch('branch2', format='knit')
-        converter = branch2.bzrdir._format.get_converter(target)
-        converter.convert(branch2.bzrdir, None)
-        branch2 = _mod_branch.Branch.open(self.get_url('branch'))
-        self.assertIs(branch2.__class__, _mod_branch.BzrBranch6)
+        b2 = self.make_branch('branch2', format='knit')
+        converter = b2.bzrdir._format.get_converter(target)
+        converter.convert(b2.bzrdir, None)
+        b2 = branch.Branch.open(self.get_url('branch'))
+        self.assertIs(b2.__class__, branch.BzrBranch6)
 
     def test_convert_branch7_branch8(self):
-        branch = self.make_branch('branch', format='1.9')
+        b = self.make_branch('branch', format='1.9')
         target = bzrdir.format_registry.make_bzrdir('1.9')
-        target.set_branch_format(_mod_branch.BzrBranchFormat8())
-        converter = branch.bzrdir._format.get_converter(target)
-        converter.convert(branch.bzrdir, None)
-        branch = _mod_branch.Branch.open(self.get_url('branch'))
-        self.assertIs(branch.__class__, _mod_branch.BzrBranch8)
-        self.assertEqual({}, branch._get_all_reference_info())
+        target.set_branch_format(branch.BzrBranchFormat8())
+        converter = b.bzrdir._format.get_converter(target)
+        converter.convert(b.bzrdir, None)
+        b = branch.Branch.open(self.get_url('branch'))
+        self.assertIs(b.__class__, branch.BzrBranch8)
+        self.assertEqual({}, b._get_all_reference_info())
 
     def test_convert_knit_dirstate_empty(self):
         # test that asking for an upgrade from knit to dirstate works.
@@ -241,7 +108,7 @@
         self.assertEqual(rev_id, new_tree.last_revision())
         for path in ['basis-inventory-cache', 'inventory', 'last-revision',
             'pending-merges', 'stat-cache']:
-            self.failIfExists('tree/.bzr/checkout/' + path)
+            self.assertPathDoesNotExist('tree/.bzr/checkout/' + path)
 
     def test_convert_knit_merges_dirstate(self):
         tree = self.make_branch_and_tree('tree', format='knit')
@@ -259,167 +126,89 @@
         self.assertEqual([rev_id2, rev_id3], new_tree.get_parent_ids())
         for path in ['basis-inventory-cache', 'inventory', 'last-revision',
             'pending-merges', 'stat-cache']:
-            self.failIfExists('tree/.bzr/checkout/' + path)
-
-
-_upgrade1_template = \
-     [
-     ('foo', 'new contents\n'),
-     ('.bzr/',),
-     ('.bzr/README',
-      'This is a Bazaar control directory.\n'
-      'Do not change any files in this directory.\n'
-      'See http://bazaar-vcs.org/ for more information about Bazaar.\n'),
-     ('.bzr/branch-format', 'Bazaar-NG branch, format 0.0.4\n'),
-     ('.bzr/revision-history',
-      'mbp@sourcefrog.net-20051004035611-176b16534b086b3c\n'
-      'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd\n'),
-     ('.bzr/merged-patches', ''),
-     ('.bzr/pending-merged-patches', ''),
-     ('.bzr/branch-name', ''),
-     ('.bzr/branch-lock', ''),
-     ('.bzr/pending-merges', ''),
-     ('.bzr/inventory',
-      '<inventory>\n'
-      '<entry file_id="foo-20051004035605-91e788d1875603ae" kind="file" name="foo" />\n'
-      '</inventory>\n'),
-     ('.bzr/stat-cache',
-      '### bzr hashcache v5\n'
-      'foo// be9f309239729f69a6309e970ef24941d31e042c 13 1128398176 1128398176 303464 770\n'),
-     ('.bzr/text-store/',),
-     ('.bzr/text-store/foo-20051004035611-1591048e9dc7c2d4.gz',
-      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\xcb\xcc\xcb,\xc9L\xccQH\xce\xcf+I\xcd+)\xe6\x02\x00\xdd\xcc\xf90\x11\x00\x00\x00'),
-     ('.bzr/text-store/foo-20051004035756-4081373d897c3453.gz',
-      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\xcbK-WH\xce\xcf+I\xcd+)\xe6\x02\x00g\xc3\xdf\xc9\r\x00\x00\x00'),
-     ('.bzr/inventory-store/',),
-     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
-      '\x1f\x8b\x08\x00[\xfdAC\x02\xffm\x8f\xcd\n\xc20\x10\x84\xef>E\xc8\xbdt7?M\x02\xad\xaf"\xa1\x99`P[\xa8E\xacOo\x14\x05\x0f\xdef\xe1\xfbv\x98\xbeL7L\xeb\xbcl\xfb]_\xc3\xb2\x89\\\xce8\x944\xc8<\xcf\x8d"\xb2LdH\xdb\x8el\x13\x18\xce\xfb\xc4\xde\xd5SGHq*\xd3\x0b\xad\x8e\x14S\xbc\xe0\xadI\xb1\xe2\xbe\xfe}\xc2\xdc\xb0\rL\xc6#\xa4\xd1\x8d*\x99\x0f}=F\x1e$8G\x9d\xa0\x02\xa1rP9\x01c`FV\xda1qg\x98"\x02}\xa5\xf2\xa8\x95\xec\xa4h\xeb\x80\xf6g\xcd\x13\xb3\x01\xcc\x98\xda\x00\x00\x00'),
-     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
-      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xffm\x8f\xc1\n\xc20\x10D\xef~E\xc8\xbd\xb8\x9bM\x9a,\xb4\xfe\x8a\xc4f\x83Am\xa1\x16\xb1~\xbdQ\x14<x\x9b\x81y3LW\xc6\x9b\x8c\xcb4\xaf\xbbMW\xc5\xbc\xaa\\\xce\xb2/\xa9\xd7y\x9a\x1a\x03\xe0\x10\xc0\x02\xb9\x16\\\xc3(>\x84\x84\xc1WKQ\xb4:\x95\xf1\x15\xad\x8cVc\xbc\xc8\x1b\xd3j\x91\xfb\xf2\xaf\xa4r\x8d\x85\x80\xe4)\x05\xf6\x03YG\x9f\xf4\xf5\x18\xb1\xd7\x07\xe1L\xc0\x86\xd8\x1b\xce-\xc7\xb6:a\x0f\x92\x8de\x8b\x89P\xc0\x9a\xe1\x0b\x95G\x9d\xc4\xda\xb1\xad\x07\xb6?o\x9e\xb5\xff\xf0\xf9\xda\x00\x00\x00'),
-     ('.bzr/revision-store/',),
-     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
-      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\x9d\x8eKj\xc30\x14E\xe7^\x85\xd0 \xb3$\xefI\xd1\x8f\xd8\xa6\x1b(t\x07E?\xbb\x82H\n\xb2\x1ahW\xdfB1\x14:\xeb\xf4r\xee\xbdgl\xf1\x91\xb6T\x0b\xf15\xe7\xd4{l\x13}\xb6\xad\xa7B^j\xbd\x91\xc3\xad_\xb3\xbb?m\xf5\xbd\xf9\xb8\xb4\xba\x9eJ\xec\x87\xb5_)I\xe5\x11K\xaf\xed\xe35\x85\x89\xfe\xa5\x8e\x0c@ \xc0\x05\xb8\x90\x88GT\xd2\xa1\x14\xfc\xe2@K\xc7\xfd\xef\x85\xed\xcd\xe2D\x95\x8d\x1a\xa47<\x02c2\xb0 \xbc\xd0\x8ay\xa3\xbcp\x8a\x83\x12A3\xb7XJv\xef\x7f_\xf7\x94\xe3\xd6m\xbeO\x14\x91in4*<\x812\x88\xc60\xfc\x01>k\x89\x13\xe5\x12\x00\xe8<\x8c\xdf\x8d\xcd\xaeq\xb6!\x90\xa5\xd6\xf1\xbc\x07\xc3x\xde\x85\xe6\xe1\x0b\xc8\x8a\x98\x03T\x01\x00\x00'),
-     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
-      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\x9d\x90Kj\x031\x0c\x86\xf79\xc5\xe0Ev\xe9\xc8o\x9b\xcc\x84^\xa0\xd0\x1b\x14\xbf&5d\xec`\xbb\x81\xf6\xf45\x84\xa4\x81\xaeZ\xa1\x85\x84^\xdf\xaf\xa9\x84K\xac1\xa7\xc1\xe5u\x8d\xad\x852\xa3\x17SZL\xc3k\xce\xa7a{j\xfb\xd5\x9e\x9fk\xfe(.,%\x1f\x9fRh\xdbc\xdb\xa3!\xa6KH-\x97\xcf\xb7\xe8g\xf4\xbbkG\x008\x06`@\xb9\xe4bG(_\x88\x95\xde\xf9n\xca\xfb\xc7\r\xf5\xdd\xe0\x19\xa9\x85)\x81\xf5"\xbd\x04j\xb8\x02b\xa8W\\\x0b\xc9\x14\xf4\xbc\xbb\xd7\xd6H4\xdc\xb8\xff}\xba\xc55\xd4f\xd6\xf3\x8c0&\x8ajE\xa4x\xe2@\xa5\xa6\x9a\xf3k\xc3WNaFT\x00\x00:l\xa6>Q\xcd1\x1cjp9\xf9;\xc34\xde\n\x9b\xe9lJWT{t\',a\xf9\x0b\xae\xc0x\x87\xa5\xb0Xp\xca,(a\xa9{\xd0{}\xd4\x12\x04(\xc5\xbb$\xc5$V\xceaI\x19\x01\xa2\x1dh\xed\x82d\x8c.\xccr@\xc3\xd8Q\xc6\x1f\xaa\xf1\xb6\xe8\xb0\xf9\x06QR\r\xf9\xfc\x01\x00\x00')]
-
-
-_ghost_template = [
-    ( './foo',
-        'hello\n'
-    ),
-    ( './.bzr/', ),
-    ( './.bzr/README',
-      'This is a Bazaar control directory.\n'
-      'Do not change any files in this directory.\n'
-      'See http://bazaar-vcs.org/ for more information about Bazaar.\n'
-    ),
-    ( './.bzr/branch-format',
-        'Bazaar-NG branch, format 0.0.4\n'
-    ),
-    ( './.bzr/branch-lock',
-        ''
-    ),
-    ( './.bzr/branch-name',
-        ''
-    ),
-    ( './.bzr/inventory',
-        '<inventory>\n'
-        '<entry file_id="foo-20051004104918-0379cb7c76354cde" kind="file" name="foo" />\n'
-        '</inventory>\n'
-    ),
-    ( './.bzr/merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merges',
-        ''
-    ),
-    ( './.bzr/revision-history',
-        'mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\n'
-        'mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\n'
-    ),
-    ( './.bzr/stat-cache',
-        '### bzr hashcache v5\n'
-        'foo// f572d396fae9206628714fb2ce00f72e94f2258f 6 1128422956 1128422956 306900 770\n'
-    ),
-    ( './.bzr/text-store/', ),
-    ( './.bzr/text-store/foo-20051004104921-8de8118a71be45ba.gz',
-        '\x1f\x8b\x08\x081^BC\x00\x03foo-20051004104921-8de8118a71be45ba\x00\xcbH\xcd\xc9\xc9\xe7\x02\x00 0:6\x06\x00\x00\x00'
-    ),
-    ( './.bzr/inventory-store/', ),
-    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
-        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00m\x8f\xcb\n'
-        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
-        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
-    ),
-    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
-        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00m\x8f\xcb\n'
-        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
-        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
-    ),
-    ( './.bzr/revision-store/', ),
-    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
-        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00\x9d\x8eMj\xc30\x14\x84\xf7>\x85\xd0"\xbb$\xef\xc9\xb6,\x11\xdb\xf4\x02\x85\xde\xa0\xe8\xe7\xd9\x11\xc4R\x90\xd4@{\xfa\x06\x8a\xa1\xd0]\x97\x03\xdf\xcc|c\xa6G(!E\xe6\xd2\xb6\x85Z)O\xfc\xd5\xe4\x1a"{K\xe9\xc6\x0e\xb7z\xd9\xec\xfd\xa5\xa4\x8f\xech\xc9i=E\xaa\x87\xb5^8\x0b\xf1A\xb1\xa6\xfc\xf9\x1e\xfc\xc4\xffRG\x01\xd0#@\x87\xd0i\x81G\xa3\x95%!\x06\xe5}\x0bv\xb0\xbf\x17\xca\xd5\xe0\xc4-\xa0\xb1\x8b\xb6`\xc0I\xa4\xc5\xf4\x9el\xef\x95v [\x94\xcf\x8e\xd5\xcay\xe4l\xf7\xfe\xf7u\r'
-        '\x1b\x95j\xb6\xfb\xc4\x11\x85\xea\x84\xd0\x12O\x03t\x83D\xad\xc4\x0f\xf0\x95"M\xbc\x95\x00\xc0\xe7f|6\x8aYi^B.u<\xef\xb1\x19\xcf\xbb\xce\xdc|\x038=\xc7\xe6R\x01\x00\x00'
-    ),
-    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
-        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00\x9d\x90\xc1j\xc30\x0c\x86\xef}\n'
-        "\xe3Coie'\xb1c\x9a\x94\xbe\xc0`o0,[N\x03M\\\x1c\xafe{\xfae\x94n\x85\xc1`;Y\x88O\xd2\xff\xb9Mt\x19\xe6!N\xcc\xc5q\x1cr\xa6\xd4\xf1'\x9b\xf20\xb1\xe7\x18Ol}\xca\xbb\x11\xcf\x879\xbe&G!\xc5~3Q^\xf7y\xc7\xd90]h\xca1\xbd\xbd\x0c\xbe\xe3?\xa9B\x02\xd4\x02\xa0\x12P\x99R\x17\xce\xa0\xb6\x1a\x83s\x80(\xa5\x7f\xdc0\x1f\xad\xe88\x82\xb0\x18\x0c\x82\x05\xa7\x04\x05[{\xc2\xda7\xc6\x81*\x85B\x8dh\x1a\xe7\x05g\xf7\xdc\xff>\x9d\x87\x91\xe6l\xc7s\xc7\x85\x90M%\xa5\xd1z#\x85\xa8\x9b\x1a\xaa\xfa\x06\xbc\xc7\x89:^*\x00\xe0\xfbU\xbbL\xcc\xb6\xa7\xfdH\xa9'\x16\x03\xeb\x8fq\xce\xed\xf6\xde_\xb5g\x9b\x16\xa1y\xa9\xbe\x02&\n"
-        '\x7fJ+EaM\x83$\xa5n\xbc/a\x91~\xd0\xbd\xfd\x135\n'
-        '\xd0\x9a`\x0c*W\x1aR\xc1\x94du\x08(\t\xb0\x91\xdeZ\xa3\x9cU\x9cm\x7f\x8dr\x1d\x10Ot\xb8\xc6\xcf\xa7\x907|\xfb-\xb1\xbd\xd3\xfb\xd5\x07\xeeD\xee\x08*\x02\x00\x00'
-    ),
-]
-
-_upgrade_dir_template = [
-    ( './.bzr/', ),
-    ( './.bzr/README',
-      'This is a Bazaar control directory.\n'
-      'Do not change any files in this directory.\n'
-      'See http://bazaar-vcs.org/ for more information about Bazaar.\n'
-    ),
-    ( './.bzr/branch-format',
-        'Bazaar-NG branch, format 0.0.4\n'
-    ),
-    ( './.bzr/branch-lock',
-        ''
-    ),
-    ( './.bzr/branch-name',
-        ''
-    ),
-    ( './.bzr/inventory',
-        '<inventory>\n'
-        '<entry file_id="dir-20051005095101-da1441ea3fa6917a" kind="directory" name="dir" />\n'
-        '</inventory>\n'
-    ),
-    ( './.bzr/merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merges',
-        ''
-    ),
-    ( './.bzr/revision-history',
-        'robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e\n'
-    ),
-    ( './.bzr/stat-cache',
-        '### bzr hashcache v5\n'
-    ),
-    ( './.bzr/text-store/', ),
-    ( './.bzr/inventory-store/', ),
-    ( './.bzr/inventory-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
-        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xb3\xc9\xcc+K\xcd+\xc9/\xaa\xb4\xe3\xb2\x012\x8a*\x15\xd22sR\xe33Sl\x95R2\x8bt\x8d\x0c\x0cL\r'
-        "\x81\xd8\xc0\x12H\x19\xea\xa6$\x1a\x9a\x98\x18\xa6&\x1a\xa7%\x9aY\x1a\x9a'*)dg\xe6A\x94\xa6&\x83LQR\xc8K\xccM\x05\x0b()\xe8\x03\xcd\xd4G\xb2\x00\x00\xc2<\x94\xb1m\x00\x00\x00"
-    ),
-    ( './.bzr/revision-store/', ),
-    ( './.bzr/revision-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
-        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xa5OKj\xc30\x14\xdc\xfb\x14B\x8b\xec\x92<I\xd6\xc7\xc42\x85\xde\xa0\x17(\xb6\xf4\x9c\n'
-        'l\xa9H"\x90\x9c\xbe\xa6\xa9\xa1\x9b\xae\xbax\x0c\xcc\xe71\xd3g\xbc\x85\x12R$.\xadk\xa8\x15\xb3\xa5oi\xc2\\\xc9kZ\x96\x10\x0b9,\xf5\x92\xbf)\xf7\xf2\x83O\xe5\x14\xb1\x1e\xae\xf5BI\x887\x8c5\xe5\xfb{\xf0\x96\xfei>r\x00\xc9\xb6\x83n\x03sT\xa0\xe4<y\x83\xda\x1b\xc54\xfe~T>Ff\xe9\xcc:\xdd\x8e\xa6E\xc7@\xa2\x82I\xaaNL\xbas\\313)\x00\xb9\xe6\xe0(\xd9\x87\xfc\xb7A\r'
-        "+\x96:\xae\x9f\x962\xc6\x8d\x04i\x949\x01\x97R\xb7\x1d\x17O\xc3#E\xb4T(\x00\xa0C\xd3o\x892^q\x18\xbd'>\xe4\xfe\xbc\x13M\x7f\xde{\r"
-        '\xcd\x17\x85\xea\xba\x03l\x01\x00\x00'
-    ),
-    ( './dir/', ),
-]
+            self.assertPathDoesNotExist('tree/.bzr/checkout/' + path)
+
+
+class TestSmartUpgrade(tests.TestCaseWithTransport):
+
+    from_format = bzrdir.format_registry.make_bzrdir("pack-0.92")
+    to_format = bzrdir.format_registry.make_bzrdir("2a")
+
+    def make_standalone_branch(self):
+        wt = self.make_branch_and_tree("branch1", format=self.from_format)
+        return wt.bzrdir
+
+    def test_upgrade_standalone_branch(self):
+        control = self.make_standalone_branch()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format)
+        self.assertLength(1, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(1, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.assertPathExists('branch1/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+
+    def test_upgrade_standalone_branch_cleanup(self):
+        control = self.make_standalone_branch()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format, clean_up=True)
+        self.assertLength(1, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(1, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.assertPathExists('branch1')
+        self.assertPathExists('branch1/.bzr')
+        self.assertPathDoesNotExist('branch1/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+
+    def make_repo_with_branches(self):
+        repo = self.make_repository('repo', shared=True,
+            format=self.from_format)
+        # Note: self.make_branch() always creates a new repo at the location
+        # so we need to avoid using that here ...
+        b1 = bzrdir.BzrDir.create_branch_convenience("repo/branch1",
+            format=self.from_format)
+        b2 = bzrdir.BzrDir.create_branch_convenience("repo/branch2",
+            format=self.from_format)
+        return repo.bzrdir
+
+    def test_upgrade_repo_with_branches(self):
+        control = self.make_repo_with_branches()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format)
+        self.assertLength(3, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(3, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.assertPathExists('repo/backup.bzr.~1~')
+        self.assertPathExists('repo/branch1/backup.bzr.~1~')
+        self.assertPathExists('repo/branch2/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+        b1 = branch.Branch.open('repo/branch1')
+        self.assertEqual(b1._format, self.to_format._branch_format)
+
+    def test_upgrade_repo_with_branches_cleanup(self):
+        control = self.make_repo_with_branches()
+        tried, worked, issues = upgrade.smart_upgrade(
+            [control], format=self.to_format, clean_up=True)
+        self.assertLength(3, tried)
+        self.assertEqual(tried[0], control)
+        self.assertLength(3, worked)
+        self.assertEqual(worked[0], control)
+        self.assertLength(0, issues)
+        self.assertPathExists('repo')
+        self.assertPathExists('repo/.bzr')
+        self.assertPathDoesNotExist('repo/backup.bzr.~1~')
+        self.assertPathDoesNotExist('repo/branch1/backup.bzr.~1~')
+        self.assertPathDoesNotExist('repo/branch2/backup.bzr.~1~')
+        self.assertEqual(control.open_repository()._format,
+                         self.to_format._repository_format)
+        b1 = branch.Branch.open('repo/branch1')
+        self.assertEqual(b1._format, self.to_format._branch_format)

=== modified file 'bzrlib/tests/test_upgrade_stacked.py'
--- bzrlib/tests/test_upgrade_stacked.py	2009-08-05 02:30:59 +0000
+++ bzrlib/tests/test_upgrade_stacked.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,12 +24,35 @@
     tests,
     )
 from bzrlib.upgrade import upgrade
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+
+
+def upgrade_scenarios():
+    scenario_pairs = [ # old format, new format, model_change
+#        ('knit', 'rich-root', True),
+        ('knit', '1.6', False),
+#        ('pack-0.92', '1.6', False),
+        ('1.6', '1.6.1-rich-root', True),
+        ]
+    scenarios = []
+    for (old_name, new_name, model_change) in scenario_pairs:
+        name = old_name + ', ' + new_name
+        scenarios.append((name,
+            dict(scenario_old_format=old_name,
+                scenario_new_format=new_name,
+                scenario_model_change=model_change)))
+    return scenarios
+
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestStackUpgrade(tests.TestCaseWithTransport):
     # TODO: This should possibly be repeated for all stacking repositories,
     # pairwise by rich/non-rich format; should possibly also try other kinds
     # of upgrades like knit->pack. -- mbp 20080804
+    
+    scenarios = upgrade_scenarios()
 
     def test_stack_upgrade(self):
         """Correct checks when stacked-on repository is upgraded.
@@ -65,25 +88,3 @@
         stacked = bzrdir.BzrDir.open('stacked')
         # And passes check.
         check.check_dwim('stacked', False, True, True)
-
-
-def load_tests(basic_tests, module, loader):
-    """Generate dynamic scenario tests.
-
-    Called by the bzrlib test framework.
-    """
-    scenario_pairs = [ # old format, new format, model_change
-#        ('knit', 'rich-root', True),
-        ('knit', '1.6', False),
-#        ('pack-0.92', '1.6', False),
-        ('1.6', '1.6.1-rich-root', True),
-        ]
-    scenarios = []
-    for (old_name, new_name, model_change) in scenario_pairs:
-        name = old_name + ', ' + new_name
-        scenarios.append((name,
-            dict(scenario_old_format=old_name,
-                scenario_new_format=new_name,
-                scenario_model_change=model_change)))
-    suite = loader.suiteClass()
-    return tests.multiply_tests(basic_tests, scenarios, suite)

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2010-06-15 03:08:31 +0000
+++ bzrlib/tests/test_urlutils.py	2011-09-04 23:18:43 +0000
@@ -336,17 +336,24 @@
 
     def test_function_type(self):
         if sys.platform == 'win32':
-            self.assertEqual(urlutils._win32_local_path_to_url, urlutils.local_path_to_url)
-            self.assertEqual(urlutils._win32_local_path_from_url, urlutils.local_path_from_url)
+            self.assertEqual(urlutils._win32_local_path_to_url,
+                urlutils.local_path_to_url)
+            self.assertEqual(urlutils._win32_local_path_from_url,
+                urlutils.local_path_from_url)
         else:
-            self.assertEqual(urlutils._posix_local_path_to_url, urlutils.local_path_to_url)
-            self.assertEqual(urlutils._posix_local_path_from_url, urlutils.local_path_from_url)
+            self.assertEqual(urlutils._posix_local_path_to_url,
+                urlutils.local_path_to_url)
+            self.assertEqual(urlutils._posix_local_path_from_url,
+                urlutils.local_path_from_url)
 
     def test_posix_local_path_to_url(self):
         to_url = urlutils._posix_local_path_to_url
         self.assertEqual('file:///path/to/foo',
             to_url('/path/to/foo'))
 
+        self.assertEqual('file:///path/to/foo%2Cbar',
+            to_url('/path/to/foo,bar'))
+
         try:
             result = to_url(u'/path/to/r\xe4ksm\xf6rg\xe5s')
         except UnicodeError:
@@ -359,6 +366,8 @@
         from_url = urlutils._posix_local_path_from_url
         self.assertEqual('/path/to/foo',
             from_url('file:///path/to/foo'))
+        self.assertEqual('/path/to/foo',
+            from_url('file:///path/to/foo,branch=foo'))
         self.assertEqual(u'/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///path/to/r%C3%A4ksm%C3%B6rg%C3%A5s'))
         self.assertEqual(u'/path/to/r\xe4ksm\xf6rg\xe5s',
@@ -386,6 +395,8 @@
 
         self.assertEqual('file:///', to_url('/'))
 
+        self.assertEqual('file:///C:/path/to/foo%2Cbar',
+            to_url('C:/path/to/foo,bar'))
         try:
             result = to_url(u'd:/path/to/r\xe4ksm\xf6rg\xe5s')
         except UnicodeError:
@@ -418,7 +429,11 @@
         self.assertEqual(u'D:/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///d:/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
         self.assertEqual('/', from_url('file:///'))
+        self.assertEqual('C:/path/to/foo',
+            from_url('file:///C|/path/to/foo,branch=foo'))
 
+        self.assertRaises(InvalidURL, from_url, 'file:///C:')
+        self.assertRaises(InvalidURL, from_url, 'file:///c')
         self.assertRaises(InvalidURL, from_url, '/path/to/foo')
         # Not a valid _win32 url, no drive letter
         self.assertRaises(InvalidURL, from_url, 'file:///path/to/foo')
@@ -426,6 +441,8 @@
     def test_win32_unc_path_from_url(self):
         from_url = urlutils._win32_local_path_from_url
         self.assertEqual('//HOST/path', from_url('file://HOST/path'))
+        self.assertEqual('//HOST/path',
+            from_url('file://HOST/path,branch=foo'))
         # despite IE allows 2, 4, 5 and 6 slashes in URL to another machine
         # we want to use only 2 slashes
         # Firefox understand only 5 slashes in URL, but it's ugly
@@ -440,6 +457,12 @@
         self.assertEqual(('file:///C:', '/foo'), extract('file://', '/C:/foo'))
         self.assertEqual(('file:///d|', '/path'), extract('file://', '/d|/path'))
         self.assertRaises(InvalidURL, extract, 'file://', '/path')
+        # Root drives without slash treated as invalid, see bug #841322
+        self.assertEqual(('file:///C:', '/'), extract('file://', '/C:/'))
+        self.assertRaises(InvalidURL, extract, 'file://', '/C:')
+        # Invalid without drive separator or following forward slash
+        self.assertRaises(InvalidURL, extract, 'file://', '/C')
+        self.assertRaises(InvalidURL, extract, 'file://', '/C:ool')
 
     def test_split(self):
         # Test bzrlib.urlutils.split()
@@ -492,6 +515,8 @@
             split_segment_parameters_raw(",key1=val1"))
         self.assertEquals(("foo/", ["key1=val1"]),
             split_segment_parameters_raw("foo/,key1=val1"))
+        self.assertEquals(("/foo", ["key1=val1"]),
+            split_segment_parameters_raw("foo,key1=val1"))
         self.assertEquals(("foo/base,la=bla/other/elements", []),
             split_segment_parameters_raw("foo/base,la=bla/other/elements"))
         self.assertEquals(("foo/base,la=bla/other/elements", ["a=b"]),
@@ -791,10 +816,98 @@
 
 class TestParseURL(TestCase):
 
-    def test_parse_url(self):
-        self.assertEqual(urlutils.parse_url('http://example.com:80/one'),
-            ('http', None, None, 'example.com', 80, '/one'))
-        self.assertEqual(urlutils.parse_url('http://[1:2:3::40]/one'),
-                ('http', None, None, '1:2:3::40', None, '/one'))
-        self.assertEqual(urlutils.parse_url('http://[1:2:3::40]:80/one'),
-                ('http', None, None, '1:2:3::40', 80, '/one'))
+    def test_parse_simple(self):
+        parsed = urlutils.parse_url('http://example.com:80/one')
+        self.assertEquals(('http', None, None, 'example.com', 80, '/one'),
+            parsed)
+
+    def test_ipv6(self):
+        parsed = urlutils.parse_url('http://[1:2:3::40]/one')
+        self.assertEquals(('http', None, None, '1:2:3::40', None, '/one'),
+            parsed)
+
+    def test_ipv6_port(self):
+        parsed = urlutils.parse_url('http://[1:2:3::40]:80/one')
+        self.assertEquals(('http', None, None, '1:2:3::40', 80, '/one'),
+            parsed)
+
+
+class TestURL(TestCase):
+
+    def test_parse_simple(self):
+        parsed = urlutils.URL.from_string('http://example.com:80/one')
+        self.assertEquals('http', parsed.scheme)
+        self.assertIs(None, parsed.user)
+        self.assertIs(None, parsed.password)
+        self.assertEquals('example.com', parsed.host)
+        self.assertEquals(80, parsed.port)
+        self.assertEquals('/one', parsed.path)
+
+    def test_ipv6(self):
+        parsed = urlutils.URL.from_string('http://[1:2:3::40]/one')
+        self.assertEquals('http', parsed.scheme)
+        self.assertIs(None, parsed.port)
+        self.assertIs(None, parsed.user)
+        self.assertIs(None, parsed.password)
+        self.assertEquals('1:2:3::40', parsed.host)
+        self.assertEquals('/one', parsed.path)
+
+    def test_ipv6_port(self):
+        parsed = urlutils.URL.from_string('http://[1:2:3::40]:80/one')
+        self.assertEquals('http', parsed.scheme)
+        self.assertEquals('1:2:3::40', parsed.host)
+        self.assertIs(None, parsed.user)
+        self.assertIs(None, parsed.password)
+        self.assertEquals(80, parsed.port)
+        self.assertEquals('/one', parsed.path)
+
+    def test_quoted(self):
+        parsed = urlutils.URL.from_string(
+            'http://ro%62ey:h%40t@ex%41mple.com:2222/path')
+        self.assertEquals(parsed.quoted_host, 'ex%41mple.com')
+        self.assertEquals(parsed.host, 'exAmple.com')
+        self.assertEquals(parsed.port, 2222)
+        self.assertEquals(parsed.quoted_user, 'ro%62ey')
+        self.assertEquals(parsed.user, 'robey')
+        self.assertEquals(parsed.quoted_password, 'h%40t')
+        self.assertEquals(parsed.password, 'h@t')
+        self.assertEquals(parsed.path, '/path')
+
+    def test_eq(self):
+        parsed1 = urlutils.URL.from_string('http://[1:2:3::40]:80/one')
+        parsed2 = urlutils.URL.from_string('http://[1:2:3::40]:80/one')
+        self.assertEquals(parsed1, parsed2)
+        self.assertEquals(parsed1, parsed1)
+        parsed2.path = '/two'
+        self.assertNotEquals(parsed1, parsed2)
+
+    def test_repr(self):
+        parsed = urlutils.URL.from_string('http://[1:2:3::40]:80/one')
+        self.assertEquals(
+            "<URL('http', None, None, '1:2:3::40', 80, '/one')>",
+            repr(parsed))
+
+    def test_str(self):
+        parsed = urlutils.URL.from_string('http://[1:2:3::40]:80/one')
+        self.assertEquals('http://[1:2:3::40]:80/one', str(parsed))
+
+    def test__combine_paths(self):
+        combine = urlutils.URL._combine_paths
+        self.assertEqual('/home/sarah/project/foo',
+                         combine('/home/sarah', 'project/foo'))
+        self.assertEqual('/etc',
+                         combine('/home/sarah', '../../etc'))
+        self.assertEqual('/etc',
+                         combine('/home/sarah', '../../../etc'))
+        self.assertEqual('/etc',
+                         combine('/home/sarah', '/etc'))
+
+    def test_clone(self):
+        url = urlutils.URL.from_string('http://[1:2:3::40]:80/one')
+        url1 = url.clone("two")
+        self.assertEquals("/one/two", url1.path)
+        url2 = url.clone("/two")
+        self.assertEquals("/two", url2.path)
+        url3 = url.clone()
+        self.assertIsNot(url, url3)
+        self.assertEquals(url, url3)

=== added file 'bzrlib/tests/test_utextwrap.py'
--- bzrlib/tests/test_utextwrap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_utextwrap.py	2011-05-28 06:44:01 +0000
@@ -0,0 +1,221 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+#
+
+"""Tests of the bzrlib.utextwrap."""
+
+from bzrlib import (
+    tests,
+    utextwrap,
+    )
+from bzrlib.tests import features
+
+
+# Japanese "Good morning".
+# Each character have double width. So total 8 width on console.
+_str_D = u'\u304a\u306f\u3088\u3046'
+
+_str_S = u"hello"
+
+# Combine single width characters and double width characters.
+_str_SD = _str_S + _str_D
+_str_DS = _str_D + _str_S
+
+class TestUTextWrap(tests.TestCase):
+
+    def check_width(self, text, expected_width):
+        w = utextwrap.UTextWrapper()
+        self.assertEqual(
+                w._width(text),
+                expected_width,
+                "Width of %r should be %d" % (text, expected_width))
+
+    def test_width(self):
+        self.check_width(_str_D, 8)
+        self.check_width(_str_SD, 13)
+
+    def check_cut(self, text, width, pos):
+        w = utextwrap.UTextWrapper()
+        self.assertEqual((text[:pos], text[pos:]), w._cut(text, width))
+
+    def test_cut(self):
+        s = _str_SD
+        self.check_cut(s, 0, 0)
+        self.check_cut(s, 1, 1)
+        self.check_cut(s, 5, 5)
+        self.check_cut(s, 6, 5)
+        self.check_cut(s, 7, 6)
+        self.check_cut(s, 12, 8)
+        self.check_cut(s, 13, 9)
+        self.check_cut(s, 14, 9)
+        self.check_cut(u'A'*5, 3, 3)
+
+    def test_split(self):
+        w = utextwrap.UTextWrapper()
+        self.assertEqual(list(_str_D), w._split(_str_D))
+        self.assertEqual([_str_S]+list(_str_D), w._split(_str_SD))
+        self.assertEqual(list(_str_D)+[_str_S], w._split(_str_DS))
+
+    def test_wrap(self):
+        self.assertEqual(list(_str_D), utextwrap.wrap(_str_D, 1))
+        self.assertEqual(list(_str_D), utextwrap.wrap(_str_D, 2))
+        self.assertEqual(list(_str_D), utextwrap.wrap(_str_D, 3))
+        self.assertEqual(list(_str_D),
+                         utextwrap.wrap(_str_D, 3, break_long_words=False))
+
+class TestUTextFill(tests.TestCase):
+
+    def test_fill_simple(self):
+        # Test only can call fill() because it's just '\n'.join(wrap(text)).
+        self.assertEqual("%s\n%s" % (_str_D[:2], _str_D[2:]),
+                         utextwrap.fill(_str_D, 4))
+
+    def test_fill_with_breaks(self):
+        # Demonstrate complicated case.
+        text = u"spam ham egg spamhamegg" + _str_D + u" spam" + _str_D*2
+        self.assertEqual(u'\n'.join(["spam ham",
+                                     "egg spam",
+                                     "hamegg" + _str_D[0],
+                                     _str_D[1:],
+                                     "spam" + _str_D[:2],
+                                     _str_D[2:]+_str_D[:2],
+                                     _str_D[2:]]),
+                         utextwrap.fill(text, 8))
+
+    def test_fill_without_breaks(self):
+        text = u"spam ham egg spamhamegg" + _str_D + u" spam" + _str_D*2
+        self.assertEqual(u'\n'.join(["spam ham",
+                                     "egg",
+                                     "spamhamegg", 
+                                     # border between single width and double
+                                     # width.
+                                     _str_D,
+                                     "spam" + _str_D[:2],
+                                     _str_D[2:]+_str_D[:2],
+                                     _str_D[2:]]),
+                         utextwrap.fill(text, 8, break_long_words=False))
+
+    def test_fill_indent_with_breaks(self):
+        w = utextwrap.UTextWrapper(8, initial_indent=' '*4,
+                                   subsequent_indent=' '*4)
+        self.assertEqual(u'\n'.join(["    hell",
+                                     "    o" + _str_D[0],
+                                     "    " + _str_D[1:3],
+                                     "    " + _str_D[3]
+                                     ]),
+                         w.fill(_str_SD))
+
+    def test_fill_indent_without_breaks(self):
+        w = utextwrap.UTextWrapper(8, initial_indent=' '*4,
+                                   subsequent_indent=' '*4)
+        w.break_long_words = False
+        self.assertEqual(u'\n'.join(["    hello",
+                                     "    " + _str_D[:2],
+                                     "    " + _str_D[2:],
+                                     ]),
+                         w.fill(_str_SD))
+
+    def test_fill_indent_without_breaks_with_fixed_width(self):
+        w = utextwrap.UTextWrapper(8, initial_indent=' '*4,
+                                   subsequent_indent=' '*4)
+        w.break_long_words = False
+        w.width = 3
+        self.assertEqual(u'\n'.join(["    hello",
+                                     "    " + _str_D[0],
+                                     "    " + _str_D[1],
+                                     "    " + _str_D[2],
+                                     "    " + _str_D[3],
+                                     ]),
+                         w.fill(_str_SD))
+
+class TestUTextWrapAmbiWidth(tests.TestCase):
+    _cyrill_char = u"\u0410" # east_asian_width() == 'A'
+
+    def test_ambiwidth1(self):
+        w = utextwrap.UTextWrapper(4, ambiguous_width=1)
+        s = self._cyrill_char*8
+        self.assertEqual([self._cyrill_char*4]*2, w.wrap(s))
+
+    def test_ambiwidth2(self):
+        w = utextwrap.UTextWrapper(4, ambiguous_width=2)
+        s = self._cyrill_char*8
+        self.assertEqual([self._cyrill_char*2]*4, w.wrap(s))
+
+
+# Regression test with Python's test_textwrap
+# Note that some distribution including Ubuntu doesn't install
+# Python's test suite.
+try:
+    from test import test_textwrap
+
+    def override_textwrap_symbols(testcase):
+        # Override the symbols imported by test_textwrap so it uses our own
+        # replacements.
+        testcase.overrideAttr(test_textwrap, 'TextWrapper',
+                              utextwrap.UTextWrapper)
+        testcase.overrideAttr(test_textwrap, 'wrap', utextwrap.wrap)
+        testcase.overrideAttr(test_textwrap, 'fill', utextwrap.fill)
+
+
+    def setup_both(testcase, base_class, reused_class):
+
+        if (features.sphinx.available()):
+            # Until https://bitbucket.org/birkenfeld/sphinx/issue/706 is fixed,
+            # we can't run these tests when sphinx <= 1.0.1 as it breaks
+            # textwrap.TextWrapper.wordsep_re
+            version = tuple(map(int,
+                                features.sphinx.module.__version__.split('.')))
+            if version <= (1, 0, 7):
+                raise tests.TestSkipped(
+                    'sphinx textwrap monkeypatch breaks utextwrap')
+        super(base_class, testcase).setUp()
+        override_textwrap_symbols(testcase)
+        reused_class.setUp(testcase)
+
+
+    class TestWrap(tests.TestCase, test_textwrap.WrapTestCase):
+
+        def setUp(self):
+            setup_both(self, TestWrap, test_textwrap.WrapTestCase)
+
+
+    class TestLongWord(tests.TestCase, test_textwrap.LongWordTestCase):
+
+        def setUp(self):
+            setup_both(self, TestLongWord, test_textwrap.LongWordTestCase)
+
+
+    class TestIndent(tests.TestCase, test_textwrap.IndentTestCases):
+
+        def setUp(self):
+            setup_both(self, TestIndent, test_textwrap.IndentTestCases)
+
+except ImportError:
+
+    class TestWrap(tests.TestCase):
+
+        def test_wrap(self):
+            raise tests.TestSkipped("test.test_textwrap is not available.")
+
+    class TestLongWord(tests.TestCase):
+
+        def test_longword(self):
+            raise tests.TestSkipped("test.test_textwrap is not available.")
+
+    class TestIndent(tests.TestCase):
+
+        def test_indent(self):
+            raise tests.TestSkipped("test.test_textwrap is not available.")

=== modified file 'bzrlib/tests/test_version.py'
--- bzrlib/tests/test_version.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_version.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2009, 2010 Canonical Ltd
+# Copyright (C) 2006, 2009, 2010, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 """Tests for versioning of bzrlib."""
 
 from cStringIO import StringIO
-import os
 import re
 
 from bzrlib import version, workingtree
@@ -47,4 +46,4 @@
         out = sio.getvalue()
         m = re.search(r"Python interpreter: (.*) [0-9]", out)
         self.assertIsNot(m, None)
-        self.failUnlessExists(m.group(1))
+        self.assertPathExists(m.group(1))

=== modified file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/test_version_info.py	2011-06-14 17:01:15 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 from bzrlib import (
     errors,
     registry,
-    symbol_versioning,
     tests,
     version_info_formats,
     )
@@ -54,6 +53,25 @@
 
         return wt
 
+    def create_tree_with_dotted_revno(self):
+        wt = self.make_branch_and_tree('branch')
+        self.build_tree(['branch/a'])
+        wt.add('a')
+        wt.commit('a', rev_id='r1')
+
+        other = wt.bzrdir.sprout('other').open_workingtree()
+        self.build_tree(['other/b.a'])
+        other.add(['b.a'])
+        other.commit('b.a', rev_id='o2')
+
+        os.chdir('branch')
+        self.run_bzr('merge ../other')
+        wt.commit('merge', rev_id='merge')
+
+        wt.update(revision='o2')
+
+        return wt
+
     def test_rio_null(self):
         wt = self.make_branch_and_tree('branch')
 
@@ -64,6 +82,15 @@
         self.assertContainsRe(val, 'build-date:')
         self.assertContainsRe(val, 'revno: 0')
 
+    def test_rio_dotted_revno(self):
+        wt = self.create_tree_with_dotted_revno()
+
+        sio = StringIO()
+        builder = RioVersionInfoBuilder(wt.branch, working_tree=wt)
+        builder.generate(sio)
+        val = sio.getvalue()
+        self.assertContainsRe(val, 'revno: 1.1.1')
+
     def test_rio_version_text(self):
         wt = self.create_branch()
 
@@ -137,8 +164,8 @@
             return new_stanzas[0]
 
         stanza = regen()
-        self.failUnless('date' in stanza)
-        self.failUnless('build-date' in stanza)
+        self.assertTrue('date' in stanza)
+        self.assertTrue('build-date' in stanza)
         self.assertEqual(['3'], stanza.get_all('revno'))
         self.assertEqual(['r3'], stanza.get_all('revision-id'))
 
@@ -192,7 +219,17 @@
         builder.generate(sio)
         val = sio.getvalue()
         self.assertContainsRe(val, "'revision_id': None")
-        self.assertContainsRe(val, "'revno': 0")
+        self.assertContainsRe(val, "'revno': '0'")
+        self.assertNotContainsString(val, '\n\n\n\n')
+
+    def test_python_dotted_revno(self):
+        wt = self.create_tree_with_dotted_revno()
+
+        sio = StringIO()
+        builder = PythonVersionInfoBuilder(wt.branch, working_tree=wt)
+        builder.generate(sio)
+        val = sio.getvalue()
+        self.assertContainsRe(val, "'revno': '1.1.1'")
 
     def test_python_version(self):
         wt = self.create_branch()
@@ -223,9 +260,9 @@
             return tvi
 
         tvi = regen()
-        self.assertEqual(3, tvi.version_info['revno'])
+        self.assertEqual('3', tvi.version_info['revno'])
         self.assertEqual('r3', tvi.version_info['revision_id'])
-        self.failUnless(tvi.version_info.has_key('date'))
+        self.assertTrue(tvi.version_info.has_key('date'))
         self.assertEqual(None, tvi.version_info['clean'])
 
         tvi = regen(check_for_clean=True)
@@ -283,6 +320,14 @@
         self.assertRaises(errors.MissingTemplateVariable, 
             builder.generate, sio)
 
+    def test_custom_dotted_revno(self):
+        sio = StringIO()
+        wt = self.create_tree_with_dotted_revno()
+        builder = CustomVersionInfoBuilder(wt.branch, working_tree=wt, 
+            template='{revno} revid: {revision_id}')
+        builder.generate(sio)
+        self.assertEquals("1.1.1 revid: o2", sio.getvalue())
+
     def test_custom_version_text(self):
         wt = self.create_branch()
 

=== added file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_versionedfile.py	2010-08-10 20:03:44 +0000
@@ -0,0 +1,139 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for VersionedFile classes"""
+
+from bzrlib import (
+    errors,
+    groupcompress,
+    multiparent,
+    tests,
+    versionedfile,
+    )
+
+
+class Test_MPDiffGenerator(tests.TestCaseWithMemoryTransport):
+    # Should this be a per vf test?
+
+    def make_vf(self):
+        t = self.get_transport('')
+        factory = groupcompress.make_pack_factory(True, True, 1)
+        return factory(t)
+
+    def make_three_vf(self):
+        vf = self.make_vf()
+        vf.add_lines(('one',), (), ['first\n'])
+        vf.add_lines(('two',), [('one',)], ['first\n', 'second\n'])
+        vf.add_lines(('three',), [('one',), ('two',)],
+                    ['first\n', 'second\n', 'third\n'])
+        return vf
+
+    def test_finds_parents(self):
+        vf = self.make_three_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('three',)])
+        needed_keys, refcount = gen._find_needed_keys()
+        self.assertEqual(sorted([('one',), ('two',), ('three',)]),
+                         sorted(needed_keys))
+        self.assertEqual({('one',): 1, ('two',): 1}, refcount)
+
+    def test_ignores_ghost_parents(self):
+        # If a parent is a ghost, it is just ignored
+        vf = self.make_vf()
+        vf.add_lines(('two',), [('one',)], ['first\n', 'second\n'])
+        gen = versionedfile._MPDiffGenerator(vf, [('two',)])
+        needed_keys, refcount = gen._find_needed_keys()
+        self.assertEqual(sorted([('two',)]), sorted(needed_keys))
+        # It is returned, but we don't really care as we won't extract it
+        self.assertEqual({('one',): 1}, refcount)
+        self.assertEqual([('one',)], sorted(gen.ghost_parents))
+        self.assertEqual([], sorted(gen.present_parents))
+
+    def test_raises_on_ghost_keys(self):
+        # If the requested key is a ghost, then we have a problem
+        vf = self.make_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('one',)])
+        self.assertRaises(errors.RevisionNotPresent,
+                          gen._find_needed_keys)
+
+    def test_refcount_multiple_children(self):
+        vf = self.make_three_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('two',), ('three',)])
+        needed_keys, refcount = gen._find_needed_keys()
+        self.assertEqual(sorted([('one',), ('two',), ('three',)]),
+                         sorted(needed_keys))
+        self.assertEqual({('one',): 2, ('two',): 1}, refcount)
+        self.assertEqual([('one',)], sorted(gen.present_parents))
+
+    def test_process_contents(self):
+        vf = self.make_three_vf()
+        gen = versionedfile._MPDiffGenerator(vf, [('two',), ('three',)])
+        gen._find_needed_keys()
+        self.assertEqual({('two',): (('one',),),
+                          ('three',): (('one',), ('two',))},
+                         gen.parent_map)
+        self.assertEqual({('one',): 2, ('two',): 1}, gen.refcounts)
+        self.assertEqual(sorted([('one',), ('two',), ('three',)]),
+                         sorted(gen.needed_keys))
+        stream = vf.get_record_stream(gen.needed_keys, 'topological', True)
+        record = stream.next()
+        self.assertEqual(('one',), record.key)
+        # one is not needed in the output, but it is needed by children. As
+        # such, it should end up in the various caches
+        gen._process_one_record(record.key, record.get_bytes_as('chunked'))
+        # The chunks should be cached, the refcount untouched
+        self.assertEqual([('one',)], gen.chunks.keys())
+        self.assertEqual({('one',): 2, ('two',): 1}, gen.refcounts)
+        self.assertEqual([], gen.diffs.keys())
+        # Next we get 'two', which is something we output, but also needed for
+        # three
+        record = stream.next()
+        self.assertEqual(('two',), record.key)
+        gen._process_one_record(record.key, record.get_bytes_as('chunked'))
+        # Both are now cached, and the diff for two has been extracted, and
+        # one's refcount has been updated. two has been removed from the
+        # parent_map
+        self.assertEqual(sorted([('one',), ('two',)]),
+                         sorted(gen.chunks.keys()))
+        self.assertEqual({('one',): 1, ('two',): 1}, gen.refcounts)
+        self.assertEqual([('two',)], gen.diffs.keys())
+        self.assertEqual({('three',): (('one',), ('two',))},
+                         gen.parent_map)
+        # Finally 'three', which allows us to remove all parents from the
+        # caches
+        record = stream.next()
+        self.assertEqual(('three',), record.key)
+        gen._process_one_record(record.key, record.get_bytes_as('chunked'))
+        # Both are now cached, and the diff for two has been extracted, and
+        # one's refcount has been updated
+        self.assertEqual([], gen.chunks.keys())
+        self.assertEqual({}, gen.refcounts)
+        self.assertEqual(sorted([('two',), ('three',)]),
+                         sorted(gen.diffs.keys()))
+
+    def test_compute_diffs(self):
+        vf = self.make_three_vf()
+        # The content is in the order requested, even if it isn't topological
+        gen = versionedfile._MPDiffGenerator(vf, [('two',), ('three',),
+                                                  ('one',)])
+        diffs = gen.compute_diffs()
+        expected_diffs = [
+            multiparent.MultiParent([multiparent.ParentText(0, 0, 0, 1),
+                                     multiparent.NewText(['second\n'])]),
+            multiparent.MultiParent([multiparent.ParentText(1, 0, 0, 2),
+                                     multiparent.NewText(['third\n'])]),
+            multiparent.MultiParent([multiparent.NewText(['first\n'])]),
+            ]
+        self.assertEqual(expected_diffs, diffs)

=== modified file 'bzrlib/tests/test_weave.py'
--- bzrlib/tests/test_weave.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_weave.py	2011-02-05 14:24:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
     )
 from bzrlib.osutils import sha_string
 from bzrlib.tests import TestCase, TestCaseInTempDir
-from bzrlib.weave import Weave, WeaveFormatError, WeaveError
+from bzrlib.weave import Weave, WeaveFormatError
 from bzrlib.weavefile import write_weave, read_weave
 
 
@@ -64,6 +64,7 @@
 
 class WeaveContains(TestBase):
     """Weave __contains__ operator"""
+
     def runTest(self):
         k = Weave(get_scope=lambda:None)
         self.assertFalse('foo' in k)
@@ -72,11 +73,13 @@
 
 
 class Easy(TestBase):
+
     def runTest(self):
         k = Weave()
 
 
 class AnnotateOne(TestBase):
+
     def runTest(self):
         k = Weave()
         k.add_lines('text0', [], TEXT_0)
@@ -86,6 +89,7 @@
 
 class InvalidAdd(TestBase):
     """Try to use invalid version number during add."""
+
     def runTest(self):
         k = Weave()
 
@@ -107,6 +111,7 @@
 
 
 class InvalidRepeatedAdd(TestBase):
+
     def runTest(self):
         k = Weave()
         k.add_lines('basis', [], TEXT_0)
@@ -533,6 +538,7 @@
 
 class Merge(TestBase):
     """Storage of versions that merge diverged parents"""
+
     def runTest(self):
         k = Weave()
 
@@ -649,6 +655,7 @@
 
 
 class JoinWeavesTests(TestBase):
+
     def setUp(self):
         super(JoinWeavesTests, self).setUp()
         self.weave1 = Weave()

=== modified file 'bzrlib/tests/test_whitebox.py'
--- bzrlib/tests/test_whitebox.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_whitebox.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,13 +15,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import os
-import unittest
 
 from bzrlib import (
     osutils,
     )
-from bzrlib.tests import TestCaseWithTransport, TestCase
-from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseWithTransport
 from bzrlib.errors import PathNotChild
 from bzrlib.osutils import relpath, pathjoin, abspath, realpath
 

=== modified file 'bzrlib/tests/test_win32utils.py'
--- bzrlib/tests/test_win32utils.py	2010-06-22 23:58:34 +0000
+++ bzrlib/tests/test_win32utils.py	2011-06-14 01:26:41 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,6 @@
 """Tests for win32utils."""
 
 import os
-import sys
 
 from bzrlib import (
     osutils,
@@ -25,36 +24,21 @@
     win32utils,
     )
 from bzrlib.tests import (
-    Feature,
     TestCase,
     TestCaseInTempDir,
     TestSkipped,
-    UnicodeFilenameFeature,
     )
 from bzrlib.tests.features import backslashdir_feature
 from bzrlib.win32utils import glob_expand, get_app_path
-
-
-class _RequiredModuleFeature(Feature):
-
-    def __init__(self, mod_name):
-        self.mod_name = mod_name
-        super(_RequiredModuleFeature, self).__init__()
-
-    def _probe(self):
-        try:
-            __import__(self.mod_name)
-            return True
-        except ImportError:
-            return False
-
-    def feature_name(self):
-        return self.mod_name
-
-Win32RegistryFeature = _RequiredModuleFeature('_winreg')
-CtypesFeature = _RequiredModuleFeature('ctypes')
-Win32comShellFeature = _RequiredModuleFeature('win32com.shell')
-Win32ApiFeature = _RequiredModuleFeature('win32api') 
+from bzrlib.tests import (
+    features,
+    )
+
+
+Win32RegistryFeature = features.ModuleAvailableFeature('_winreg')
+CtypesFeature = features.ModuleAvailableFeature('ctypes')
+Win32comShellFeature = features.ModuleAvailableFeature('win32com.shell')
+Win32ApiFeature = features.ModuleAvailableFeature('win32api') 
 
 
 # Tests
@@ -79,7 +63,7 @@
                          'd/', 'd/d1', 'd/d2', 'd/e/', 'd/e/e1'])
 
     def build_unicode_tree(self):
-        self.requireFeature(UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         self.build_tree([u'\u1234', u'\u1234\u1234', u'\u1235/',
                          u'\u1235/\u1235'])
 
@@ -120,7 +104,8 @@
             ])
 
     def test_case_insensitive_globbing(self):
-        self.requireFeature(tests.CaseInsCasePresFilenameFeature)
+        if os.path.normcase("AbC") == "AbC":
+            self.skip("Test requires case insensitive globbing function")
         self.build_ascii_tree()
         self._run_testset([
             [[u'A'], [u'A']],
@@ -216,7 +201,7 @@
     def test_appdata_not_using_environment(self):
         # Test that we aren't falling back to the environment
         first = win32utils.get_appdata_location()
-        self._captureVar("APPDATA", None)
+        self.overrideEnv("APPDATA", None)
         self.assertPathsEqual(first, win32utils.get_appdata_location())
 
     def test_appdata_matches_environment(self):
@@ -233,7 +218,7 @@
     def test_local_appdata_not_using_environment(self):
         # Test that we aren't falling back to the environment
         first = win32utils.get_local_appdata_location()
-        self._captureVar("LOCALAPPDATA", None)
+        self.overrideEnv("LOCALAPPDATA", None)
         self.assertPathsEqual(first, win32utils.get_local_appdata_location())
 
     def test_local_appdata_matches_environment(self):
@@ -263,14 +248,14 @@
 
     def test_unicode_dir(self):
         # we should handle unicode paths without errors
-        self.requireFeature(UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.mkdir(u'\u1234')
         win32utils.set_file_attr_hidden(u'\u1234')
 
     def test_dot_bzr_in_unicode_dir(self):
         # we should not raise traceback if we try to set hidden attribute
         # on .bzr directory below unicode path
-        self.requireFeature(UnicodeFilenameFeature)
+        self.requireFeature(features.UnicodeFilenameFeature)
         os.makedirs(u'\u1234\\.bzr')
         path = osutils.abspath(u'\u1234\\.bzr')
         win32utils.set_file_attr_hidden(path)
@@ -326,7 +311,8 @@
             single_quotes_allowed=True)
 
     def test_case_insensitive_globs(self):
-        self.requireFeature(tests.CaseInsCasePresFilenameFeature)
+        if os.path.normcase("AbC") == "AbC":
+            self.skip("Test requires case insensitive globbing function")
         self.build_tree(['a/', 'a/b.c', 'a/c.c', 'a/c.h'])
         self.assertCommandLine([u'A/b.c'], 'A/B*')
 

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_workingtree.py	2011-05-13 12:51:05 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -15,18 +15,16 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
-import os
-
 from bzrlib import (
     bzrdir,
     conflicts,
     errors,
+    symbol_versioning,
     transport,
     workingtree,
+    workingtree_3,
+    workingtree_4,
     )
-from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
 from bzrlib.lockdir import LockDir
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
@@ -65,21 +63,42 @@
 class TestDefaultFormat(TestCaseWithTransport):
 
     def test_get_set_default_format(self):
-        old_format = workingtree.WorkingTreeFormat.get_default_format()
-        # default is 3
-        self.assertTrue(isinstance(old_format, workingtree.WorkingTreeFormat3))
-        workingtree.WorkingTreeFormat.set_default_format(SampleTreeFormat())
-        try:
-            # the default branch format is used by the meta dir format
-            # which is not the default bzrdir format at this point
-            dir = bzrdir.BzrDirMetaFormat1().initialize('.')
-            dir.create_repository()
-            dir.create_branch()
-            result = dir.create_workingtree()
-            self.assertEqual(result, 'A tree')
-        finally:
-            workingtree.WorkingTreeFormat.set_default_format(old_format)
-        self.assertEqual(old_format, workingtree.WorkingTreeFormat.get_default_format())
+        old_format = workingtree.format_registry.get_default()
+        # default is 6
+        self.assertTrue(isinstance(old_format, workingtree_4.WorkingTreeFormat6))
+        workingtree.format_registry.set_default(SampleTreeFormat())
+        try:
+            # the default branch format is used by the meta dir format
+            # which is not the default bzrdir format at this point
+            dir = bzrdir.BzrDirMetaFormat1().initialize('.')
+            dir.create_repository()
+            dir.create_branch()
+            result = dir.create_workingtree()
+            self.assertEqual(result, 'A tree')
+        finally:
+            workingtree.format_registry.set_default(old_format)
+        self.assertEqual(old_format, workingtree.format_registry.get_default())
+
+    def test_get_set_default_format_by_key(self):
+        old_format = workingtree.format_registry.get_default()
+        # default is 6
+        format = SampleTreeFormat()
+        workingtree.format_registry.register(format)
+        self.addCleanup(workingtree.format_registry.remove, format)
+        self.assertTrue(isinstance(old_format, workingtree_4.WorkingTreeFormat6))
+        workingtree.format_registry.set_default_key(format.get_format_string())
+        try:
+            # the default branch format is used by the meta dir format
+            # which is not the default bzrdir format at this point
+            dir = bzrdir.BzrDirMetaFormat1().initialize('.')
+            dir.create_repository()
+            dir.create_branch()
+            result = dir.create_workingtree()
+            self.assertEqual(result, 'A tree')
+        finally:
+            workingtree.format_registry.set_default_key(
+                old_format.get_format_string())
+        self.assertEqual(old_format, workingtree.format_registry.get_default())
 
     def test_open(self):
         tree = self.make_branch_and_tree('.')
@@ -126,9 +145,42 @@
         return "opened tree."
 
 
+class SampleExtraTreeFormat(workingtree.WorkingTreeFormat):
+    """A sample format that does not support use in a metadir.
+
+    """
+
+    def get_format_string(self):
+        # Not usable in a metadir, so no format string
+        return None
+
+    def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
+                   accelerator_tree=None, hardlink=False):
+        raise NotImplementedError(self.initialize)
+
+    def is_supported(self):
+        return False
+
+    def open(self, transport, _found=False):
+        raise NotImplementedError(self.open)
+
+
 class TestWorkingTreeFormat(TestCaseWithTransport):
     """Tests for the WorkingTreeFormat facility."""
 
+    def test_find_format_string(self):
+        # is the right format object found for a working tree?
+        branch = self.make_branch('branch')
+        self.assertRaises(errors.NoWorkingTree,
+            workingtree.WorkingTreeFormat.find_format_string, branch.bzrdir)
+        transport = branch.bzrdir.get_workingtree_transport(None)
+        transport.mkdir('.')
+        transport.put_bytes("format", "some format name")
+        # The format does not have to be known by Bazaar,
+        # find_format_string just retrieves the name
+        self.assertEquals("some format name",
+            workingtree.WorkingTreeFormat.find_format_string(branch.bzrdir))
+
     def test_find_format(self):
         # is the right format object found for a working tree?
         # create a branch with a few known format objects.
@@ -140,8 +192,8 @@
             format.initialize(dir)
             t = transport.get_transport(url)
             found_format = workingtree.WorkingTreeFormat.find_format(dir)
-            self.failUnless(isinstance(found_format, format.__class__))
-        check_format(workingtree.WorkingTreeFormat3(), "bar")
+            self.assertIsInstance(found_format, format.__class__)
+        check_format(workingtree_3.WorkingTreeFormat3(), "bar")
 
     def test_find_format_no_tree(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize('.')
@@ -167,13 +219,96 @@
         # make a branch
         format.initialize(dir)
         # register a format for it.
-        workingtree.WorkingTreeFormat.register_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            workingtree.WorkingTreeFormat.register_format, format)
+        self.assertTrue(format in 
+            self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+                workingtree.WorkingTreeFormat.get_formats))
         # which branch.Open will refuse (not supported)
         self.assertRaises(errors.UnsupportedFormatError, workingtree.WorkingTree.open, '.')
         # but open_downlevel will work
         self.assertEqual(format.open(dir), workingtree.WorkingTree.open_downlevel('.'))
         # unregister the format
-        workingtree.WorkingTreeFormat.unregister_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            workingtree.WorkingTreeFormat.unregister_format, format)
+        self.assertFalse(format in
+            self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+                workingtree.WorkingTreeFormat.get_formats))
+
+
+class TestWorkingTreeIterEntriesByDir_wSubtrees(TestCaseWithTransport):
+
+    def make_simple_tree(self):
+        tree = self.make_branch_and_tree('tree', format='development-subtree')
+        self.build_tree(['tree/a/', 'tree/a/b/', 'tree/a/b/c'])
+        tree.set_root_id('root-id')
+        tree.add(['a', 'a/b', 'a/b/c'], ['a-id', 'b-id', 'c-id'])
+        tree.commit('initial')
+        return tree
+
+    def test_just_directory(self):
+        tree = self.make_simple_tree()
+        self.assertEqual([('directory', 'root-id'),
+                          ('directory', 'a-id'),
+                          ('directory', 'b-id'),
+                          ('file', 'c-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir()])
+        subtree = self.make_branch_and_tree('tree/a/b')
+        self.assertEqual([('tree-reference', 'b-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir(['b-id'])])
+
+    def test_direct_subtree(self):
+        tree = self.make_simple_tree()
+        subtree = self.make_branch_and_tree('tree/a/b')
+        self.assertEqual([('directory', 'root-id'),
+                          ('directory', 'a-id'),
+                          ('tree-reference', 'b-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir()])
+
+    def test_indirect_subtree(self):
+        tree = self.make_simple_tree()
+        subtree = self.make_branch_and_tree('tree/a')
+        self.assertEqual([('directory', 'root-id'),
+                          ('tree-reference', 'a-id')],
+                         [(ie.kind, ie.file_id)
+                          for path, ie in tree.iter_entries_by_dir()])
+
+
+class TestWorkingTreeFormatRegistry(TestCase):
+
+    def setUp(self):
+        super(TestWorkingTreeFormatRegistry, self).setUp()
+        self.registry = workingtree.WorkingTreeFormatRegistry()
+
+    def test_register_unregister_format(self):
+        format = SampleTreeFormat()
+        self.registry.register(format)
+        self.assertEquals(format, self.registry.get("Sample tree format."))
+        self.registry.remove(format)
+        self.assertRaises(KeyError, self.registry.get, "Sample tree format.")
+
+    def test_get_all(self):
+        format = SampleTreeFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra(self):
+        format = SampleExtraTreeFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra_lazy(self):
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra_lazy("bzrlib.tests.test_workingtree",
+            "SampleExtraTreeFormat")
+        formats = self.registry._get_all()
+        self.assertEquals(1, len(formats))
+        self.assertIsInstance(formats[0], SampleExtraTreeFormat)
 
 
 class TestWorkingTreeFormat3(TestCaseWithTransport):
@@ -183,7 +318,7 @@
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         control.create_repository()
         control.create_branch()
-        tree = workingtree.WorkingTreeFormat3().initialize(control)
+        tree = workingtree_3.WorkingTreeFormat3().initialize(control)
         # we want:
         # format 'Bazaar-NG Working Tree format 3'
         # inventory = blank inventory
@@ -217,7 +352,7 @@
         repo = dir.create_repository()
         branch = dir.create_branch()
         try:
-            tree = workingtree.WorkingTreeFormat3().initialize(dir)
+            tree = workingtree_3.WorkingTreeFormat3().initialize(dir)
         except errors.NotLocalUrl:
             raise TestSkipped('Not a local URL')
         self.assertIsDirectory('.bzr', t)
@@ -234,41 +369,11 @@
         control = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         control.create_repository()
         control.create_branch()
-        tree = workingtree.WorkingTreeFormat3().initialize(control)
+        tree = workingtree_3.WorkingTreeFormat3().initialize(control)
         tree._transport.delete("pending-merges")
         self.assertEqual([], tree.get_parent_ids())
 
 
-class TestFormat2WorkingTree(TestCaseWithTransport):
-    """Tests that are specific to format 2 trees."""
-
-    def create_format2_tree(self, url):
-        return self.make_branch_and_tree(
-            url, format=bzrdir.BzrDirFormat6())
-
-    def test_conflicts(self):
-        # test backwards compatability
-        tree = self.create_format2_tree('.')
-        self.assertRaises(errors.UnsupportedOperation, tree.set_conflicts,
-                          None)
-        file('lala.BASE', 'wb').write('labase')
-        expected = conflicts.ContentsConflict('lala')
-        self.assertEqual(list(tree.conflicts()), [expected])
-        file('lala', 'wb').write('la')
-        tree.add('lala', 'lala-id')
-        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflicts()), [expected])
-        file('lala.THIS', 'wb').write('lathis')
-        file('lala.OTHER', 'wb').write('laother')
-        # When "text conflict"s happen, stem, THIS and OTHER are text
-        expected = conflicts.TextConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflicts()), [expected])
-        os.unlink('lala.OTHER')
-        os.mkdir('lala.OTHER')
-        expected = conflicts.ContentsConflict('lala', file_id='lala-id')
-        self.assertEqual(list(tree.conflicts()), [expected])
-
-
 class InstrumentedTree(object):
     """A instrumented tree to check the needs_tree_write_lock decorator."""
 
@@ -345,40 +450,40 @@
         self.build_tree_contents([('other/hello', 'hELLO')])
         other.commit('Case switch')
         this = base.bzrdir.sprout('this').open_workingtree()
-        self.failUnlessExists('this/hello')
+        self.assertPathExists('this/hello')
         self.build_tree_contents([('this/hello', 'Hello World')])
         this.commit('Add World')
         this.merge_from_branch(other.branch)
-        self.assertEqual([conflicts.TextConflict('hello', None, 'hello_id')],
+        self.assertEqual([conflicts.TextConflict('hello', 'hello_id')],
                          this.conflicts())
         this.auto_resolve()
-        self.assertEqual([conflicts.TextConflict('hello', None, 'hello_id')],
+        self.assertEqual([conflicts.TextConflict('hello', 'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '<<<<<<<')])
         this.auto_resolve()
-        self.assertEqual([conflicts.TextConflict('hello', None, 'hello_id')],
+        self.assertEqual([conflicts.TextConflict('hello', 'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '=======')])
         this.auto_resolve()
-        self.assertEqual([conflicts.TextConflict('hello', None, 'hello_id')],
+        self.assertEqual([conflicts.TextConflict('hello', 'hello_id')],
                          this.conflicts())
         self.build_tree_contents([('this/hello', '\n>>>>>>>')])
         remaining, resolved = this.auto_resolve()
-        self.assertEqual([conflicts.TextConflict('hello', None, 'hello_id')],
+        self.assertEqual([conflicts.TextConflict('hello', 'hello_id')],
                          this.conflicts())
         self.assertEqual([], resolved)
         self.build_tree_contents([('this/hello', 'hELLO wORLD')])
         remaining, resolved = this.auto_resolve()
         self.assertEqual([], this.conflicts())
-        self.assertEqual([conflicts.TextConflict('hello', None, 'hello_id')],
+        self.assertEqual([conflicts.TextConflict('hello', 'hello_id')],
                          resolved)
-        self.failIfExists('this/hello.BASE')
+        self.assertPathDoesNotExist('this/hello.BASE')
 
     def test_auto_resolve_dir(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/hello/'])
         tree.add('hello', 'hello-id')
-        file_conflict = conflicts.TextConflict('file', None, 'hello-id')
+        file_conflict = conflicts.TextConflict('file', 'hello-id')
         tree.set_conflicts(conflicts.ConflictList([file_conflict]))
         tree.auto_resolve()
 

=== modified file 'bzrlib/tests/test_workingtree_4.py'
--- bzrlib/tests/test_workingtree_4.py	2010-07-21 09:58:42 +0000
+++ bzrlib/tests/test_workingtree_4.py	2011-05-26 21:02:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -57,6 +57,22 @@
         finally:
             state.unlock()
 
+    def test_resets_ignores_on_last_unlock(self):
+        # Only the last unlock call will actually reset the
+        # ignores. (bug #785671)
+        tree = self.make_workingtree()
+        tree.lock_read()
+        try:
+            tree.lock_read()
+            try:
+                tree.is_ignored("foo")
+            finally:
+                tree.unlock()
+            self.assertIsNot(None, tree._ignoreglobster)
+        finally:
+            tree.unlock()
+        self.assertIs(None, tree._ignoreglobster)
+
     def test_uses_lockdir(self):
         """WorkingTreeFormat4 uses its own LockDir:
 
@@ -174,9 +190,9 @@
         # it's given; any calls to forbidden methods will raise an
         # AssertionError
         repo = tree.branch.repository
-        repo.get_revision = self.fail
-        repo.get_inventory = self.fail
-        repo._get_inventory_xml = self.fail
+        self.overrideAttr(repo, "get_revision", self.fail)
+        self.overrideAttr(repo, "get_inventory", self.fail)
+        self.overrideAttr(repo, "_get_inventory_xml", self.fail)
         # try to set the parent trees.
         tree.set_parent_trees([(rev1, rev1_tree)])
 
@@ -214,8 +230,8 @@
         # answer 'get_parent_ids' for the revision tree- dirstate does not
         # cache the parents of a parent tree at this point.
         #repo.get_revision = self.fail
-        repo.get_inventory = self.fail
-        repo._get_inventory_xml = self.fail
+        self.overrideAttr(repo, "get_inventory", self.fail)
+        self.overrideAttr(repo, "_get_inventory_xml", self.fail)
         # set the parent trees.
         tree.set_parent_trees([(rev1, rev1_tree), (rev2, rev2_tree)])
         # read the first tree
@@ -258,6 +274,68 @@
         lock_and_call_current_dirstate(tree, 'lock_tree_write')
         self.assertRaises(errors.ObjectNotLocked, tree.current_dirstate)
 
+    def test_set_parent_trees_uses_update_basis_by_delta(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        self.addCleanup(builder.finish_series)
+        builder.build_snapshot('A', [], [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('a', 'a-id', 'file', 'content\n'))])
+        builder.build_snapshot('B', ['A'], [
+            ('modify', ('a-id', 'new content\nfor a\n')),
+            ('add', ('b', 'b-id', 'file', 'b-content\n'))])
+        tree = self.make_workingtree('tree')
+        source_branch = builder.get_branch()
+        tree.branch.repository.fetch(source_branch.repository, 'B')
+        tree.pull(source_branch, stop_revision='A')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        state = tree.current_dirstate()
+        called = []
+        orig_update = state.update_basis_by_delta
+        def log_update_basis_by_delta(delta, new_revid):
+            called.append(new_revid)
+            return orig_update(delta, new_revid)
+        state.update_basis_by_delta = log_update_basis_by_delta
+        basis = tree.basis_tree()
+        self.assertEqual('a-id', basis.path2id('a'))
+        self.assertEqual(None, basis.path2id('b'))
+        def fail_set_parent_trees(trees, ghosts):
+            raise AssertionError('dirstate.set_parent_trees() was called')
+        state.set_parent_trees = fail_set_parent_trees
+        repo = tree.branch.repository
+        tree.pull(source_branch, stop_revision='B')
+        self.assertEqual(['B'], called)
+        basis = tree.basis_tree()
+        self.assertEqual('a-id', basis.path2id('a'))
+        self.assertEqual('b-id', basis.path2id('b'))
+
+    def test_set_parent_trees_handles_missing_basis(self):
+        builder = self.make_branch_builder('source')
+        builder.start_series()
+        self.addCleanup(builder.finish_series)
+        builder.build_snapshot('A', [], [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('a', 'a-id', 'file', 'content\n'))])
+        builder.build_snapshot('B', ['A'], [
+            ('modify', ('a-id', 'new content\nfor a\n')),
+            ('add', ('b', 'b-id', 'file', 'b-content\n'))])
+        builder.build_snapshot('C', ['A'], [
+            ('add', ('c', 'c-id', 'file', 'c-content\n'))])
+        b_c = self.make_branch('branch_with_c')
+        b_c.pull(builder.get_branch(), stop_revision='C')
+        b_b = self.make_branch('branch_with_b')
+        b_b.pull(builder.get_branch(), stop_revision='B')
+        # This is reproducing some of what 'switch' does, just to isolate the
+        # set_parent_trees() step.
+        wt = b_b.create_checkout('tree', lightweight=True)
+        fmt = wt.bzrdir.find_branch_format()
+        fmt.set_reference(wt.bzrdir, None, b_c)
+        # Re-open with the new reference
+        wt = wt.bzrdir.open_workingtree()
+        wt.set_parent_trees([('C', b_c.repository.revision_tree('C'))])
+        self.assertEqual(None, wt.basis_tree().path2id('b'))
+
     def test_new_dirstate_on_new_lock(self):
         # until we have detection for when a dirstate can be reused, we
         # want to reparse dirstate on every new lock.
@@ -597,38 +675,40 @@
 
     def get_tree_with_cachable_file_foo(self):
         tree = self.make_branch_and_tree('.')
-        self.build_tree(['foo'])
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        self.build_tree_contents([('foo', 'a bit of content for foo\n')])
         tree.add(['foo'], ['foo-id'])
-        # a 4 second old timestamp is always hashable - sucks to delay
-        # the test suite, but not testing this is worse.
-        time.sleep(4)
+        tree.current_dirstate()._cutoff_time = time.time() + 60
         return tree
 
     def test_commit_updates_hash_cache(self):
         tree = self.get_tree_with_cachable_file_foo()
         revid = tree.commit('a commit')
         # tree's dirstate should now have a valid stat entry for foo.
-        tree.lock_read()
-        self.addCleanup(tree.unlock)
         entry = tree._get_entry(path='foo')
         expected_sha1 = osutils.sha_file_by_name('foo')
         self.assertEqual(expected_sha1, entry[1][0][1])
+        self.assertEqual(len('a bit of content for foo\n'), entry[1][0][2])
 
     def test_observed_sha1_cachable(self):
         tree = self.get_tree_with_cachable_file_foo()
         expected_sha1 = osutils.sha_file_by_name('foo')
         statvalue = os.lstat("foo")
-        tree.lock_write()
-        try:
-            tree._observed_sha1("foo-id", "foo", (expected_sha1, statvalue))
-            self.assertEqual(expected_sha1,
-                tree._get_entry(path="foo")[1][0][1])
-        finally:
-            tree.unlock()
+        tree._observed_sha1("foo-id", "foo", (expected_sha1, statvalue))
+        entry = tree._get_entry(path="foo")
+        entry_state = entry[1][0]
+        self.assertEqual(expected_sha1, entry_state[1])
+        self.assertEqual(statvalue.st_size, entry_state[2])
+        tree.unlock()
+        tree.lock_read()
         tree = tree.bzrdir.open_workingtree()
         tree.lock_read()
         self.addCleanup(tree.unlock)
-        self.assertEqual(expected_sha1, tree._get_entry(path="foo")[1][0][1])
+        entry = tree._get_entry(path="foo")
+        entry_state = entry[1][0]
+        self.assertEqual(expected_sha1, entry_state[1])
+        self.assertEqual(statvalue.st_size, entry_state[2])
 
     def test_observed_sha1_new_file(self):
         tree = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/test_wsgi.py'
--- bzrlib/tests/test_wsgi.py	2009-10-21 11:13:40 +0000
+++ bzrlib/tests/test_wsgi.py	2011-01-12 01:01:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 from cStringIO import StringIO
 
 from bzrlib import tests
-from bzrlib.smart import medium, message, protocol
+from bzrlib.smart import medium, protocol
 from bzrlib.transport.http import wsgi
 from bzrlib.transport import chroot, memory
 

=== modified file 'bzrlib/tests/test_xml.py'
--- bzrlib/tests/test_xml.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/test_xml.py	2011-02-21 23:43:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,35 +23,11 @@
     xml6,
     xml7,
     xml8,
-    serializer,
     )
 from bzrlib.tests import TestCase
-from bzrlib.inventory import Inventory, InventoryEntry
-from bzrlib.xml4 import serializer_v4
+from bzrlib.inventory import Inventory
 import bzrlib.xml5
 
-_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
-<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
-<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
-<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
-</inventory>"""
-
-
-_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
-    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
-    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
-    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
-    timestamp="1125907235.212"
-    timezone="36000">
-<message>- start splitting code for xml (de)serialization away from objects
-  preparatory to supporting multiple formats by a single library
-</message>
-<parents>
-<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
-</parents>
-</revision>
-"""
-
 _revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
     inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
     revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
@@ -218,26 +194,6 @@
 class TestSerializer(TestCase):
     """Test XML serialization"""
 
-    def test_canned_inventory(self):
-        """Test unpacked a canned inventory v4 file."""
-        inp = StringIO(_working_inventory_v4)
-        inv = serializer_v4.read_inventory(inp)
-        self.assertEqual(len(inv), 4)
-        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
-
-    def test_unpack_revision(self):
-        """Test unpacking a canned revision v4"""
-        inp = StringIO(_revision_v4)
-        rev = serializer_v4.read_revision(inp)
-        eq = self.assertEqual
-        eq(rev.committer,
-           "Martin Pool <mbp@sourcefrog.net>")
-        eq(rev.inventory_id,
-           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
-        eq(len(rev.parent_ids), 1)
-        eq(rev.parent_ids[0],
-           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
-
     def test_unpack_revision_5(self):
         """Test unpacking a canned revision v5"""
         inp = StringIO(_revision_v5)
@@ -580,3 +536,10 @@
         uni_str = u'\xb5\xe5&\u062c'
         self.assertEqual('&#181;&#229;&amp;&#1580;"',
                          bzrlib.xml8._encode_and_escape(uni_str))
+
+
+class TestMisc(TestCase):
+
+    def test_unescape_xml(self):
+        """We get some kind of error when malformed entities are passed"""
+        self.assertRaises(KeyError, bzrlib.xml8._unescape_xml, 'foo&bar;')

=== added file 'bzrlib/tests/testui.py'
--- bzrlib/tests/testui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/testui.py	2010-09-25 20:08:01 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""UI implementations for use in testing.
+"""
+
+
+from bzrlib import (
+    progress,
+    ui,
+    )
+
+
+class ProgressRecordingUIFactory(ui.UIFactory, progress.DummyProgress):
+    """Captures progress updates made through it.
+    
+    This is overloaded as both the UIFactory and the progress model."""
+
+    def __init__(self):
+        super(ProgressRecordingUIFactory, self).__init__()
+        self._calls = []
+        self.depth = 0
+
+    def nested_progress_bar(self):
+        self.depth += 1
+        return self
+
+    def finished(self):
+        self.depth -= 1
+
+    def update(self, message, count=None, total=None):
+        if self.depth == 1:
+            self._calls.append(("update", count, total, message))

=== modified file 'bzrlib/tests/transport_util.py'
--- bzrlib/tests/transport_util.py	2010-02-23 07:43:11 +0000
+++ bzrlib/tests/transport_util.py	2011-08-19 22:34:02 +0000
@@ -15,6 +15,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import bzrlib.hooks
+from bzrlib import transport
 from bzrlib.tests import features
 
 # SFTPTransport offers better performances but relies on paramiko, if paramiko
@@ -34,7 +35,6 @@
 
 from bzrlib.transport import (
     ConnectedTransport,
-    get_transport,
     register_transport,
     register_urlparse_netloc_protocol,
     unregister_transport,
@@ -47,7 +47,8 @@
     """Dict-mapping hook name to a list of callables for transport hooks"""
 
     def __init__(self):
-        super(TransportHooks, self).__init__()
+        super(TransportHooks, self).__init__("bzrlib.tests.transport_util",
+            "InstrumentedTransport.hooks")
         # Invoked when the transport has just created a new connection.
         # The api signature is (transport, connection, credentials)
         self['_set_connection'] = []
@@ -76,12 +77,8 @@
             fake_base, _from_transport=_from_transport)
         # The following is needed to minimize the effects of our trick above
         # while retaining the best compatibility.
-        self._scheme = _hooked_scheme
-        base = self._unsplit_url(self._scheme,
-                                 self._user, self._password,
-                                 self._host, self._port,
-                                 self._path)
-        super(ConnectedTransport, self).__init__(base)
+        self._parsed_url.scheme = _hooked_scheme
+        super(ConnectedTransport, self).__init__(str(self._parsed_url))
 
 
 class ConnectionHookedTransport(InstrumentedTransport):
@@ -111,7 +108,7 @@
         # standard test support code will work and permit the server url
         # correctly.
         url = self.get_url()
-        t = get_transport(url)
+        t = transport.get_transport_from_url(url)
         if t.base.endswith('work/'):
             t = t.clone('../..')
         self.permit_url(t.base)

=== modified file 'bzrlib/timestamp.py'
--- bzrlib/timestamp.py	2009-03-23 14:59:43 +0000
+++ bzrlib/timestamp.py	2011-01-10 22:20:12 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2007, 2008, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -72,34 +72,6 @@
     :param date: A date formated by format_highres_date
     :type date: string
 
-    >>> import time, random
-    >>> unpack_highres_date('Thu 2005-06-30 12:38:52.350850105 -0500')
-    (1120153132.3508501, -18000)
-    >>> unpack_highres_date('Thu 2005-06-30 17:38:52.350850105 +0000')
-    (1120153132.3508501, 0)
-    >>> unpack_highres_date('Thu 2005-06-30 19:38:52.350850105 +0200')
-    (1120153132.3508501, 7200)
-    >>> unpack_highres_date('Sun 2006-07-09 12:35:38.867522001 +0530')
-    (1152428738.867522, 19800)
-    >>> from bzrlib.osutils import local_time_offset
-    >>> t = time.time()
-    >>> o = local_time_offset()
-    >>> t2, o2 = unpack_highres_date(format_highres_date(t, o))
-    >>> t == t2
-    True
-    >>> o == o2
-    True
-    >>> t -= 24*3600*365*2 # Start 2 years ago
-    >>> o = -12*3600
-    >>> for count in xrange(500):
-    ...   t += random.random()*24*3600*30
-    ...   o = ((o/3600 + 13) % 25 - 12)*3600 # Add 1 wrap around from [-12, 12]
-    ...   date = format_highres_date(t, o)
-    ...   t2, o2 = unpack_highres_date(date)
-    ...   if t != t2 or o != o2:
-    ...      print 'Failed on date %r, %s,%s diff:%s' % (date, t, o, t2-t)
-    ...      break
-
     """
     # Weekday parsing is locale sensitive, so drop the weekday
     space_loc = date.find(' ')

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2010-06-28 02:41:22 +0000
+++ bzrlib/trace.py	2011-09-23 20:24:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,23 +65,17 @@
 from cStringIO import StringIO
 import errno
 import locale
+import tempfile
 import traceback
 """)
 
 import bzrlib
 
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    )
-
 lazy_import(globals(), """
 from bzrlib import (
     debug,
     errors,
     osutils,
-    plugin,
-    symbol_versioning,
     ui,
     )
 """)
@@ -122,35 +116,15 @@
     # FIXME: clearing the ui and then going through the abstract logging
     # framework is whack; we should probably have a logging Handler that
     # deals with terminal output if needed.
-    import bzrlib.ui
-    bzrlib.ui.ui_factory.clear_term()
+    ui.ui_factory.clear_term()
     _bzr_logger.info(*args, **kwargs)
 
 
 def warning(*args, **kwargs):
-    import bzrlib.ui
-    bzrlib.ui.ui_factory.clear_term()
+    ui.ui_factory.clear_term()
     _bzr_logger.warning(*args, **kwargs)
 
 
-@deprecated_function(deprecated_in((2, 1, 0)))
-def info(*args, **kwargs):
-    """Deprecated: use trace.note instead."""
-    note(*args, **kwargs)
-
-
-@deprecated_function(deprecated_in((2, 1, 0)))
-def log_error(*args, **kwargs):
-    """Deprecated: use bzrlib.trace.show_error instead"""
-    _bzr_logger.error(*args, **kwargs)
-
-
-@deprecated_function(deprecated_in((2, 1, 0)))
-def error(*args, **kwargs):
-    """Deprecated: use bzrlib.trace.show_error instead"""
-    _bzr_logger.error(*args, **kwargs)
-
-
 def show_error(*args, **kwargs):
     """Show an error message to the user.
 
@@ -335,7 +309,7 @@
     :param to_file: A file-like object to which messages will be sent.
 
     :returns: A memento that should be passed to _pop_log_file to restore the
-    previously active logging.
+        previously active logging.
     """
     global _trace_file
     # make a new handler
@@ -469,6 +443,40 @@
                     note(line)
                     break
 
+def _dump_memory_usage(err_file):
+    try:
+        try:
+            fd, name = tempfile.mkstemp(prefix="bzr_memdump", suffix=".json")
+            dump_file = os.fdopen(fd, 'w')
+            from meliae import scanner
+            scanner.dump_gc_objects(dump_file)
+            err_file.write("Memory dumped to %s\n" % name)
+        except ImportError:
+            err_file.write("Dumping memory requires meliae module.\n")
+            log_exception_quietly()
+        except:
+            err_file.write("Exception while dumping memory.\n")
+            log_exception_quietly()
+    finally:
+        if dump_file is not None:
+            dump_file.close()
+        elif fd is not None:
+            os.close(fd)
+
+
+def _qualified_exception_name(eclass, unqualified_bzrlib_errors=False):
+    """Give name of error class including module for non-builtin exceptions
+
+    If `unqualified_bzrlib_errors` is True, errors specific to bzrlib will
+    also omit the module prefix.
+    """
+    class_name = eclass.__name__
+    module_name = eclass.__module__
+    if module_name in ("exceptions", "__main__") or (
+            unqualified_bzrlib_errors and module_name == "bzrlib.errors"):
+        return class_name
+    return "%s.%s" % (module_name, class_name)
+
 
 def report_exception(exc_info, err_file):
     """Report an exception to err_file (typically stderr) and to .bzr.log.
@@ -492,6 +500,10 @@
         return errors.EXIT_ERROR
     elif isinstance(exc_object, MemoryError):
         err_file.write("bzr: out of memory\n")
+        if 'mem_dump' in debug.debug_flags:
+            _dump_memory_usage(err_file)
+        else:
+            err_file.write("Use -Dmem_dump to dump memory to a file.\n")
         return errors.EXIT_ERROR
     elif isinstance(exc_object, ImportError) \
         and str(exc_object).startswith("No module named "):
@@ -547,6 +559,10 @@
     try:
         sys.stdout.flush()
         sys.stderr.flush()
+    except ValueError, e:
+        # On Windows, I get ValueError calling stdout.flush() on a closed
+        # handle
+        pass
     except IOError, e:
         import errno
         if e.errno in [errno.EINVAL, errno.EPIPE]:

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2010-07-15 13:39:38 +0000
+++ bzrlib/transform.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,13 +19,20 @@
 from stat import S_ISREG, S_IEXEC
 import time
 
-from bzrlib.lazy_import import lazy_import
-lazy_import(globals(), """
+from bzrlib import (
+    errors,
+    lazy_import,
+    registry,
+    trace,
+    tree,
+    )
+lazy_import.lazy_import(globals(), """
 from bzrlib import (
     annotate,
     bencode,
-    bzrdir,
+    controldir,
     commit,
+    conflicts,
     delta,
     errors,
     inventory,
@@ -33,44 +40,41 @@
     osutils,
     revision as _mod_revision,
     ui,
+    urlutils,
     )
+from bzrlib.i18n import gettext
 """)
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
                            ReusingTransform, CantMoveRoot,
                            ExistingLimbo, ImmortalLimbo, NoFinalPath,
                            UnableCreateSymlink)
 from bzrlib.filters import filtered_output_bytes, ContentFilterContext
-from bzrlib.inventory import InventoryEntry
 from bzrlib.osutils import (
     delete_any,
     file_kind,
     has_symlinks,
-    lexists,
     pathjoin,
     sha_file,
     splitpath,
     supports_executable,
-)
+    )
 from bzrlib.progress import ProgressPhase
 from bzrlib.symbol_versioning import (
-        deprecated_function,
-        deprecated_in,
-        )
-from bzrlib.trace import mutter, warning
-from bzrlib import tree
-import bzrlib.ui
-import bzrlib.urlutils as urlutils
+    deprecated_function,
+    deprecated_in,
+    deprecated_method,
+    )
 
 
 ROOT_PARENT = "root-parent"
 
-
 def unique_add(map, key, value):
     if key in map:
         raise DuplicateKey(key=key)
     map[key] = value
 
 
+
 class _TransformResults(object):
     def __init__(self, modified_paths, rename_count):
         object.__init__(self)
@@ -100,6 +104,8 @@
         self._new_parent = {}
         # mapping of trans_id with new contents -> new file_kind
         self._new_contents = {}
+        # mapping of trans_id => (sha1 of content, stat_value)
+        self._observed_sha1s = {}
         # Set of trans_ids whose contents will be removed
         self._removed_contents = set()
         # Mapping of trans_id -> new execute-bit value
@@ -124,7 +130,7 @@
             self._new_root = self.trans_id_tree_file_id(root_id)
         else:
             self._new_root = None
-        # Indictor of whether the transform has been applied
+        # Indicator of whether the transform has been applied
         self._done = False
         # A progress bar
         self._pb = pb
@@ -133,6 +139,14 @@
         # A counter of how many files have been renamed
         self.rename_count = 0
 
+    def __enter__(self):
+        """Support Context Manager API."""
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """Support Context Manager API."""
+        self.finalize()
+
     def finalize(self):
         """Release the working tree lock, if held.
 
@@ -213,6 +227,7 @@
         This means that the old root trans-id becomes obsolete, so it is
         recommended only to invoke this after the root trans-id has become
         irrelevant.
+
         """
         new_roots = [k for k, v in self._new_parent.iteritems() if v is
                      ROOT_PARENT]
@@ -224,13 +239,11 @@
             self._new_root = new_roots[0]
             return
         old_new_root = new_roots[0]
-        # TODO: What to do if a old_new_root is present, but self._new_root is
-        #       not listed as being removed? This code explicitly unversions
-        #       the old root and versions it with the new file_id. Though that
-        #       seems like an incomplete delta
-
         # unversion the new root's directory.
-        file_id = self.final_file_id(old_new_root)
+        if self.final_kind(self._new_root) is None:
+            file_id = self.final_file_id(old_new_root)
+        else:
+            file_id = self.final_file_id(self._new_root)
         if old_new_root in self._new_id:
             self.cancel_versioning(old_new_root)
         else:
@@ -240,7 +253,8 @@
         if (self.tree_file_id(self._new_root) is not None and
             self._new_root not in self._removed_id):
             self.unversion_file(self._new_root)
-        self.version_file(file_id, self._new_root)
+        if file_id is not None:
+            self.version_file(file_id, self._new_root)
 
         # Now move children of new root into old root directory.
         # Ensure all children are registered with the transaction, but don't
@@ -315,10 +329,9 @@
 
     def delete_contents(self, trans_id):
         """Schedule the contents of a path entry for deletion"""
-        # Ensure that the object exists in the WorkingTree, this will raise an
-        # exception if there is a problem
-        self.tree_kind(trans_id)
-        self._removed_contents.add(trans_id)
+        kind = self.tree_kind(trans_id)
+        if kind is not None:
+            self._removed_contents.add(trans_id)
 
     def cancel_deletion(self, trans_id):
         """Cancel a scheduled deletion"""
@@ -381,30 +394,56 @@
         return sorted(FinalPaths(self).get_paths(new_ids))
 
     def _inventory_altered(self):
-        """Get the trans_ids and paths of files needing new inv entries."""
-        new_ids = set()
-        for id_set in [self._new_name, self._new_parent, self._new_id,
+        """Determine which trans_ids need new Inventory entries.
+
+        An new entry is needed when anything that would be reflected by an
+        inventory entry changes, including file name, file_id, parent file_id,
+        file kind, and the execute bit.
+
+        Some care is taken to return entries with real changes, not cases
+        where the value is deleted and then restored to its original value,
+        but some actually unchanged values may be returned.
+
+        :returns: A list of (path, trans_id) for all items requiring an
+            inventory change. Ordered by path.
+        """
+        changed_ids = set()
+        # Find entries whose file_ids are new (or changed).
+        new_file_id = set(t for t in self._new_id
+                          if self._new_id[t] != self.tree_file_id(t))
+        for id_set in [self._new_name, self._new_parent, new_file_id,
                        self._new_executability]:
-            new_ids.update(id_set)
+            changed_ids.update(id_set)
+        # removing implies a kind change
         changed_kind = set(self._removed_contents)
+        # so does adding
         changed_kind.intersection_update(self._new_contents)
-        changed_kind.difference_update(new_ids)
-        changed_kind = (t for t in changed_kind if self.tree_kind(t) !=
-                        self.final_kind(t))
-        new_ids.update(changed_kind)
-        return sorted(FinalPaths(self).get_paths(new_ids))
+        # Ignore entries that are already known to have changed.
+        changed_kind.difference_update(changed_ids)
+        #  to keep only the truly changed ones
+        changed_kind = (t for t in changed_kind
+                        if self.tree_kind(t) != self.final_kind(t))
+        # all kind changes will alter the inventory
+        changed_ids.update(changed_kind)
+        # To find entries with changed parent_ids, find parents which existed,
+        # but changed file_id.
+        changed_file_id = set(t for t in new_file_id if t in self._removed_id)
+        # Now add all their children to the set.
+        for parent_trans_id in new_file_id:
+            changed_ids.update(self.iter_tree_children(parent_trans_id))
+        return sorted(FinalPaths(self).get_paths(changed_ids))
 
     def final_kind(self, trans_id):
         """Determine the final file kind, after any changes applied.
 
-        Raises NoSuchFile if the file does not exist/has no contents.
-        (It is conceivable that a path would be created without the
-        corresponding contents insertion command)
+        :return: None if the file does not exist/has no contents.  (It is
+            conceivable that a path would be created without the corresponding
+            contents insertion command)
         """
         if trans_id in self._new_contents:
             return self._new_contents[trans_id]
         elif trans_id in self._removed_contents:
-            raise NoSuchFile(None)
+            return None
         else:
             return self.tree_kind(trans_id)
 
@@ -523,6 +562,8 @@
         for trans_id in self._removed_id:
             file_id = self.tree_file_id(trans_id)
             if file_id is not None:
+                # XXX: This seems like something that should go via a different
+                #      indirection.
                 if self._tree.inventory[file_id].kind == 'directory':
                     parents.append(trans_id)
             elif self.tree_kind(trans_id) == 'directory':
@@ -532,30 +573,53 @@
             # ensure that all children are registered with the transaction
             list(self.iter_tree_children(parent_id))
 
+    @deprecated_method(deprecated_in((2, 3, 0)))
     def has_named_child(self, by_parent, parent_id, name):
-        try:
-            children = by_parent[parent_id]
-        except KeyError:
-            children = []
-        for child in children:
+        return self._has_named_child(
+            name, parent_id, known_children=by_parent.get(parent_id, []))
+
+    def _has_named_child(self, name, parent_id, known_children):
+        """Does a parent already have a name child.
+
+        :param name: The searched for name.
+
+        :param parent_id: The parent for which the check is made.
+
+        :param known_children: The already known children. This should have
+            been recently obtained from `self.by_parent.get(parent_id)`
+            (or will be if None is passed).
+        """
+        if known_children is None:
+            known_children = self.by_parent().get(parent_id, [])
+        for child in known_children:
             if self.final_name(child) == name:
                 return True
-        try:
-            path = self._tree_id_paths[parent_id]
-        except KeyError:
+        parent_path = self._tree_id_paths.get(parent_id, None)
+        if parent_path is None:
+            # No parent... no children
             return False
-        childpath = joinpath(path, name)
-        child_id = self._tree_path_ids.get(childpath)
+        child_path = joinpath(parent_path, name)
+        child_id = self._tree_path_ids.get(child_path, None)
         if child_id is None:
-            return lexists(self._tree.abspath(childpath))
+            # Not known by the tree transform yet, check the filesystem
+            return osutils.lexists(self._tree.abspath(child_path))
         else:
-            if self.final_parent(child_id) != parent_id:
-                return False
-            if child_id in self._removed_contents:
-                # XXX What about dangling file-ids?
-                return False
-            else:
-                return True
+            raise AssertionError('child_id is missing: %s, %s, %s'
+                                 % (name, parent_id, child_id))
+
+    def _available_backup_name(self, name, target_id):
+        """Find an available backup name.
+
+        :param name: The basename of the file.
+
+        :param target_id: The directory trans_id where the backup should 
+            be placed.
+        """
+        known_children = self.by_parent().get(target_id, [])
+        return osutils.available_backup_name(
+            name,
+            lambda base: self._has_named_child(
+                base, target_id, known_children))
 
     def _parent_loops(self):
         """No entry should be its own ancestor"""
@@ -596,12 +660,11 @@
         """
         conflicts = []
         for trans_id in self._new_id.iterkeys():
-            try:
-                kind = self.final_kind(trans_id)
-            except NoSuchFile:
+            kind = self.final_kind(trans_id)
+            if kind is None:
                 conflicts.append(('versioning no contents', trans_id))
                 continue
-            if not InventoryEntry.versionable_kind(kind):
+            if not inventory.InventoryEntry.versionable_kind(kind):
                 conflicts.append(('versioning bad kind', trans_id, kind))
         return conflicts
 
@@ -618,11 +681,7 @@
             if self.final_file_id(trans_id) is None:
                 conflicts.append(('unversioned executability', trans_id))
             else:
-                try:
-                    non_file = self.final_kind(trans_id) != "file"
-                except NoSuchFile:
-                    non_file = True
-                if non_file is True:
+                if self.final_kind(trans_id) != "file":
                     conflicts.append(('non-file executability', trans_id))
         return conflicts
 
@@ -630,9 +689,7 @@
         """Check for overwrites (not permitted on Win32)"""
         conflicts = []
         for trans_id in self._new_contents:
-            try:
-                self.tree_kind(trans_id)
-            except NoSuchFile:
+            if self.tree_kind(trans_id) is None:
                 continue
             if trans_id not in self._removed_contents:
                 conflicts.append(('overwrite', trans_id,
@@ -645,17 +702,19 @@
         if (self._new_name, self._new_parent) == ({}, {}):
             return conflicts
         for children in by_parent.itervalues():
-            name_ids = [(self.final_name(t), t) for t in children]
-            if not self._case_sensitive_target:
-                name_ids = [(n.lower(), t) for n, t in name_ids]
+            name_ids = []
+            for child_tid in children:
+                name = self.final_name(child_tid)
+                if name is not None:
+                    # Keep children only if they still exist in the end
+                    if not self._case_sensitive_target:
+                        name = name.lower()
+                    name_ids.append((name, child_tid))
             name_ids.sort()
             last_name = None
             last_trans_id = None
             for name, trans_id in name_ids:
-                try:
-                    kind = self.final_kind(trans_id)
-                except NoSuchFile:
-                    kind = None
+                kind = self.final_kind(trans_id)
                 file_id = self.final_file_id(trans_id)
                 if kind is None and file_id is None:
                     continue
@@ -680,38 +739,29 @@
         return conflicts
 
     def _parent_type_conflicts(self, by_parent):
-        """parents must have directory 'contents'."""
+        """Children must have a directory parent"""
         conflicts = []
         for parent_id, children in by_parent.iteritems():
             if parent_id is ROOT_PARENT:
                 continue
-            if not self._any_contents(children):
+            no_children = True
+            for child_id in children:
+                if self.final_kind(child_id) is not None:
+                    no_children = False
+                    break
+            if no_children:
                 continue
-            for child in children:
-                try:
-                    self.final_kind(child)
-                except NoSuchFile:
-                    continue
-            try:
-                kind = self.final_kind(parent_id)
-            except NoSuchFile:
-                kind = None
+            # There is at least a child, so we need an existing directory to
+            # contain it.
+            kind = self.final_kind(parent_id)
             if kind is None:
+                # The directory will be deleted
                 conflicts.append(('missing parent', parent_id))
             elif kind != "directory":
+                # Meh, we need a *directory* to put something in it
                 conflicts.append(('non-directory parent', parent_id))
         return conflicts
 
-    def _any_contents(self, trans_ids):
-        """Return true if any of the trans_ids, will have contents."""
-        for trans_id in trans_ids:
-            try:
-                kind = self.final_kind(trans_id)
-            except NoSuchFile:
-                continue
-            return True
-        return False
-
     def _set_executability(self, path, trans_id):
         """Set the executability of versioned files """
         if supports_executable():
@@ -739,7 +789,7 @@
         return trans_id
 
     def new_file(self, name, parent_id, contents, file_id=None,
-                 executable=None):
+                 executable=None, sha1=None):
         """Convenience method to create files.
 
         name is the name of the file to create.
@@ -752,7 +802,7 @@
         trans_id = self._new_entry(name, parent_id, file_id)
         # TODO: rather than scheduling a set_executable call,
         # have create_file create the file with the right mode.
-        self.create_file(contents, trans_id)
+        self.create_file(contents, trans_id, sha1=sha1)
         if executable is not None:
             self.set_executability(executable, trans_id)
         return trans_id
@@ -781,6 +831,48 @@
         self.create_symlink(target, trans_id)
         return trans_id
 
+    def new_orphan(self, trans_id, parent_id):
+        """Schedule an item to be orphaned.
+
+        When a directory is about to be removed, its children, if they are not
+        versioned are moved out of the way: they don't have a parent anymore.
+
+        :param trans_id: The trans_id of the existing item.
+        :param parent_id: The parent trans_id of the item.
+        """
+        raise NotImplementedError(self.new_orphan)
+
+    def _get_potential_orphans(self, dir_id):
+        """Find the potential orphans in a directory.
+
+        A directory can't be safely deleted if there are versioned files in it.
+        If all the contained files are unversioned then they can be orphaned.
+
+        The 'None' return value means that the directory contains at least one
+        versioned file and should not be deleted.
+
+        :param dir_id: The directory trans id.
+
+        :return: A list of the orphan trans ids or None if at least one
+             versioned file is present.
+        """
+        orphans = []
+        # Find the potential orphans, stop if one item should be kept
+        for child_tid in self.by_parent()[dir_id]:
+            if child_tid in self._removed_contents:
+                # The child is removed as part of the transform. Since it was
+                # versioned before, it's not an orphan
+                continue
+            elif self.final_file_id(child_tid) is None:
+                # The child is not versioned
+                orphans.append(child_tid)
+            else:
+                # We have a versioned file here, searching for orphans is
+                # meaningless.
+                orphans = None
+                break
+        return orphans
+
     def _affected_ids(self):
         """Return the set of transform ids affected by the transform"""
         trans_ids = set(self._removed_id)
@@ -845,10 +937,7 @@
         Return a (name, parent, kind, executable) tuple
         """
         to_name = self.final_name(to_trans_id)
-        try:
-            to_kind = self.final_kind(to_trans_id)
-        except NoSuchFile:
-            to_kind = None
+        to_kind = self.final_kind(to_trans_id)
         to_parent = self.final_file_id(self.final_parent(to_trans_id))
         if to_trans_id in self._new_executability:
             to_executable = self._new_executability[to_trans_id]
@@ -1101,6 +1190,7 @@
         self._deletiondir = None
         # A mapping of transform ids to their limbo filename
         self._limbo_files = {}
+        self._possibly_stale_limbo_files = set()
         # A mapping of transform ids to a set of the transform ids of children
         # that their limbo directory has
         self._limbo_children = {}
@@ -1119,11 +1209,18 @@
         if self._tree is None:
             return
         try:
-            entries = [(self._limbo_name(t), t, k) for t, k in
-                       self._new_contents.iteritems()]
-            entries.sort(reverse=True)
-            for path, trans_id, kind in entries:
-                delete_any(path)
+            limbo_paths = self._limbo_files.values() + list(
+                self._possibly_stale_limbo_files)
+            limbo_paths = sorted(limbo_paths, reverse=True)
+            for path in limbo_paths:
+                try:
+                    delete_any(path)
+                except OSError, e:
+                    if e.errno != errno.ENOENT:
+                        raise
+                    # XXX: warn? perhaps we just got interrupted at an
+                    # inconvenient moment, but perhaps files are disappearing
+                    # from under us?
             try:
                 delete_any(self._limbodir)
             except OSError:
@@ -1178,11 +1275,14 @@
         entries from _limbo_files, because they are now stale.
         """
         for trans_id in trans_ids:
-            old_path = self._limbo_files.pop(trans_id)
+            old_path = self._limbo_files[trans_id]
+            self._possibly_stale_limbo_files.add(old_path)
+            del self._limbo_files[trans_id]
             if trans_id not in self._new_contents:
                 continue
             new_path = self._limbo_name(trans_id)
-            osutils.rename(old_path, new_path)
+            os.rename(old_path, new_path)
+            self._possibly_stale_limbo_files.remove(old_path)
             for descendant in self._limbo_descendants(trans_id):
                 desc_path = self._limbo_files[descendant]
                 desc_path = new_path + desc_path[len(old_path):]
@@ -1195,34 +1295,34 @@
             descendants.update(self._limbo_descendants(descendant))
         return descendants
 
-    def create_file(self, contents, trans_id, mode_id=None):
+    def create_file(self, contents, trans_id, mode_id=None, sha1=None):
         """Schedule creation of a new file.
 
-        See also new_file.
-
-        Contents is an iterator of strings, all of which will be written
-        to the target destination.
-
-        New file takes the permissions of any existing file with that id,
-        unless mode_id is specified.
+        :seealso: new_file.
+
+        :param contents: an iterator of strings, all of which will be written
+            to the target destination.
+        :param trans_id: TreeTransform handle
+        :param mode_id: If not None, force the mode of the target file to match
+            the mode of the object referenced by mode_id.
+            Otherwise, we will try to preserve mode bits of an existing file.
+        :param sha1: If the sha1 of this content is already known, pass it in.
+            We can use it to prevent future sha1 computations.
         """
         name = self._limbo_name(trans_id)
         f = open(name, 'wb')
         try:
-            try:
-                unique_add(self._new_contents, trans_id, 'file')
-            except:
-                # Clean up the file, it never got registered so
-                # TreeTransform.finalize() won't clean it up.
-                f.close()
-                os.unlink(name)
-                raise
-
+            unique_add(self._new_contents, trans_id, 'file')
             f.writelines(contents)
         finally:
             f.close()
         self._set_mtime(name)
         self._set_mode(trans_id, mode_id, S_ISREG)
+        # It is unfortunate we have to use lstat instead of fstat, but we just
+        # used utime and chmod on the file, so we need the accurate final
+        # details.
+        if sha1 is not None:
+            self._observed_sha1s[trans_id] = (sha1, osutils.lstat(name))
 
     def _read_file_chunks(self, trans_id):
         cur_file = open(self._limbo_name(trans_id), 'rb')
@@ -1287,6 +1387,8 @@
     def cancel_creation(self, trans_id):
         """Cancel the creation of new file contents."""
         del self._new_contents[trans_id]
+        if trans_id in self._observed_sha1s:
+            del self._observed_sha1s[trans_id]
         children = self._limbo_children.get(trans_id)
         # if this is a limbo directory with children, move them before removing
         # the directory
@@ -1296,6 +1398,89 @@
             del self._limbo_children_names[trans_id]
         delete_any(self._limbo_name(trans_id))
 
+    def new_orphan(self, trans_id, parent_id):
+        # FIXME: There is no tree config, so we use the branch one (it's weird
+        # to define it this way as orphaning can only occur in a working tree,
+        # but that's all we have (for now). It will find the option in
+        # locations.conf or bazaar.conf though) -- vila 20100916
+        conf = self._tree.branch.get_config()
+        conf_var_name = 'bzr.transform.orphan_policy'
+        orphan_policy = conf.get_user_option(conf_var_name)
+        default_policy = orphaning_registry.default_key
+        if orphan_policy is None:
+            orphan_policy = default_policy
+        if orphan_policy not in orphaning_registry:
+            trace.warning('%s (from %s) is not a known policy, defaulting '
+                'to %s' % (orphan_policy, conf_var_name, default_policy))
+            orphan_policy = default_policy
+        handle_orphan = orphaning_registry.get(orphan_policy)
+        handle_orphan(self, trans_id, parent_id)
+
+
+class OrphaningError(errors.BzrError):
+
+    # Only bugs could lead to such exception being seen by the user
+    internal_error = True
+    _fmt = "Error while orphaning %s in %s directory"
+
+    def __init__(self, orphan, parent):
+        errors.BzrError.__init__(self)
+        self.orphan = orphan
+        self.parent = parent
+
+
+class OrphaningForbidden(OrphaningError):
+
+    _fmt = "Policy: %s doesn't allow creating orphans."
+
+    def __init__(self, policy):
+        errors.BzrError.__init__(self)
+        self.policy = policy
+
+
+def move_orphan(tt, orphan_id, parent_id):
+    """See TreeTransformBase.new_orphan.
+
+    This creates a new orphan in the `bzr-orphans` dir at the root of the
+    `TreeTransform`.
+
+    :param tt: The TreeTransform orphaning `trans_id`.
+
+    :param orphan_id: The trans id that should be orphaned.
+
+    :param parent_id: The orphan parent trans id.
+    """
+    # Add the orphan dir if it doesn't exist
+    orphan_dir_basename = 'bzr-orphans'
+    od_id = tt.trans_id_tree_path(orphan_dir_basename)
+    if tt.final_kind(od_id) is None:
+        tt.create_directory(od_id)
+    parent_path = tt._tree_id_paths[parent_id]
+    # Find a name that doesn't exist yet in the orphan dir
+    actual_name = tt.final_name(orphan_id)
+    new_name = tt._available_backup_name(actual_name, od_id)
+    tt.adjust_path(new_name, od_id, orphan_id)
+    trace.warning('%s has been orphaned in %s'
+                  % (joinpath(parent_path, actual_name), orphan_dir_basename))
+
+
+def refuse_orphan(tt, orphan_id, parent_id):
+    """See TreeTransformBase.new_orphan.
+
+    This refuses to create orphan, letting the caller handle the conflict.
+    """
+    raise OrphaningForbidden('never')
+
+
+orphaning_registry = registry.Registry()
+orphaning_registry.register(
+    'conflict', refuse_orphan,
+    'Leave orphans in place and create a conflict on the directory.')
+orphaning_registry.register(
+    'move', move_orphan,
+    'Move orphans into the bzr-orphans directory.')
+orphaning_registry._set_default_key('conflict')
+
 
 class TreeTransform(DiskTreeTransform):
     """Represent a tree transformation.
@@ -1419,18 +1604,15 @@
     def tree_kind(self, trans_id):
         """Determine the file kind in the working tree.
 
-        Raises NoSuchFile if the file does not exist
+        :returns: The file kind or None if the file does not exist
         """
         path = self._tree_id_paths.get(trans_id)
         if path is None:
-            raise NoSuchFile(None)
+            return None
         try:
             return file_kind(self._tree.abspath(path))
-        except OSError, e:
-            if e.errno != errno.ENOENT:
-                raise
-            else:
-                raise NoSuchFile(path)
+        except errors.NoSuchFile:
+            return None
 
     def _set_mode(self, trans_id, mode_id, typefunc):
         """Set the mode of new file contents.
@@ -1542,10 +1724,10 @@
         """
         if not no_conflicts:
             self._check_malformed()
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         try:
             if precomputed_delta is None:
-                child_pb.update('Apply phase', 0, 2)
+                child_pb.update(gettext('Apply phase'), 0, 2)
                 inventory_delta = self._generate_inventory_delta()
                 offset = 1
             else:
@@ -1556,9 +1738,9 @@
             else:
                 mover = _mover
             try:
-                child_pb.update('Apply phase', 0 + offset, 2 + offset)
+                child_pb.update(gettext('Apply phase'), 0 + offset, 2 + offset)
                 self._apply_removals(mover)
-                child_pb.update('Apply phase', 1 + offset, 2 + offset)
+                child_pb.update(gettext('Apply phase'), 1 + offset, 2 + offset)
                 modified_paths = self._apply_insertions(mover)
             except:
                 mover.rollback()
@@ -1567,7 +1749,10 @@
                 mover.apply_deletions()
         finally:
             child_pb.finished()
+        if self.final_file_id(self.root) is None:
+            inventory_delta = [e for e in inventory_delta if e[0] != '']
         self._tree.apply_inventory_delta(inventory_delta)
+        self._apply_observed_sha1s()
         self._done = True
         self.finalize()
         return _TransformResults(modified_paths, self.rename_count)
@@ -1575,13 +1760,13 @@
     def _generate_inventory_delta(self):
         """Generate an inventory delta for the current transform."""
         inventory_delta = []
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         new_paths = self._inventory_altered()
         total_entries = len(new_paths) + len(self._removed_id)
         try:
             for num, trans_id in enumerate(self._removed_id):
                 if (num % 10) == 0:
-                    child_pb.update('removing file', num, total_entries)
+                    child_pb.update(gettext('removing file'), num, total_entries)
                 if trans_id == self._new_root:
                     file_id = self._tree.get_root_id()
                 else:
@@ -1599,15 +1784,14 @@
             final_kinds = {}
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
-                    child_pb.update('adding file',
+                    child_pb.update(gettext('adding file'),
                                     num + len(self._removed_id), total_entries)
                 file_id = new_path_file_ids[trans_id]
                 if file_id is None:
                     continue
                 needs_entry = False
-                try:
-                    kind = self.final_kind(trans_id)
-                except NoSuchFile:
+                kind = self.final_kind(trans_id)
+                if kind is None:
                     kind = self._tree.stored_kind(file_id)
                 parent_trans_id = self.final_parent(trans_id)
                 parent_file_id = new_path_file_ids.get(parent_trans_id)
@@ -1644,11 +1828,13 @@
         """
         tree_paths = list(self._tree_path_ids.iteritems())
         tree_paths.sort(reverse=True)
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         try:
-            for num, data in enumerate(tree_paths):
-                path, trans_id = data
-                child_pb.update('removing file', num, len(tree_paths))
+            for num, (path, trans_id) in enumerate(tree_paths):
+                # do not attempt to move root into a subdirectory of itself.
+                if path == '':
+                    continue
+                child_pb.update(gettext('removing file'), num, len(tree_paths))
                 full_path = self._tree.abspath(path)
                 if trans_id in self._removed_contents:
                     delete_path = os.path.join(self._deletiondir, trans_id)
@@ -1679,11 +1865,11 @@
         modified_paths = []
         new_path_file_ids = dict((t, self.final_file_id(t)) for p, t in
                                  new_paths)
-        child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        child_pb = ui.ui_factory.nested_progress_bar()
         try:
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
-                    child_pb.update('adding file', num, len(new_paths))
+                    child_pb.update(gettext('adding file'), num, len(new_paths))
                 full_path = self._tree.abspath(path)
                 if trans_id in self._needs_rename:
                     try:
@@ -1694,17 +1880,52 @@
                             raise
                     else:
                         self.rename_count += 1
+                    # TODO: if trans_id in self._observed_sha1s, we should
+                    #       re-stat the final target, since ctime will be
+                    #       updated by the change.
                 if (trans_id in self._new_contents or
                     self.path_changed(trans_id)):
                     if trans_id in self._new_contents:
                         modified_paths.append(full_path)
                 if trans_id in self._new_executability:
                     self._set_executability(path, trans_id)
+                if trans_id in self._observed_sha1s:
+                    o_sha1, o_st_val = self._observed_sha1s[trans_id]
+                    st = osutils.lstat(full_path)
+                    self._observed_sha1s[trans_id] = (o_sha1, st)
         finally:
             child_pb.finished()
+        for path, trans_id in new_paths:
+            # new_paths includes stuff like workingtree conflicts. Only the
+            # stuff in new_contents actually comes from limbo.
+            if trans_id in self._limbo_files:
+                del self._limbo_files[trans_id]
         self._new_contents.clear()
         return modified_paths
 
+    def _apply_observed_sha1s(self):
+        """After we have finished renaming everything, update observed sha1s
+
+        This has to be done after self._tree.apply_inventory_delta, otherwise
+        it doesn't know anything about the files we are updating. Also, we want
+        to do this as late as possible, so that most entries end up cached.
+        """
+        # TODO: this doesn't update the stat information for directories. So
+        #       the first 'bzr status' will still need to rewrite
+        #       .bzr/checkout/dirstate. However, we at least don't need to
+        #       re-read all of the files.
+        # TODO: If the operation took a while, we could do a time.sleep(3) here
+        #       to allow the clock to tick over and ensure we won't have any
+        #       problems. (we could observe start time, and finish time, and if
+        #       it is less than eg 10% overhead, add a sleep call.)
+        paths = FinalPaths(self)
+        for trans_id, observed in self._observed_sha1s.iteritems():
+            path = paths.get_path(trans_id)
+            # We could get the file_id, but dirstate prefers to use the path
+            # anyway, and it is 'cheaper' to determine.
+            # file_id = self._new_id[trans_id]
+            self._tree._observed_sha1(None, path, observed)
+
 
 class TransformPreview(DiskTreeTransform):
     """A TreeTransform for generating preview trees.
@@ -1725,9 +1946,11 @@
     def tree_kind(self, trans_id):
         path = self._tree_id_paths.get(trans_id)
         if path is None:
-            raise NoSuchFile(None)
-        file_id = self._tree.path2id(path)
-        return self._tree.kind(file_id)
+            return None
+        kind = self._tree.path_content_summary(path)[0]
+        if kind == 'missing':
+            kind = None
+        return kind
 
     def _set_mode(self, trans_id, mode_id, typefunc):
         """Set the mode of new file contents.
@@ -1753,8 +1976,11 @@
             childpath = joinpath(path, child)
             yield self.trans_id_tree_path(childpath)
 
-
-class _PreviewTree(tree.Tree):
+    def new_orphan(self, trans_id, parent_id):
+        raise NotImplementedError(self.new_orphan)
+
+
+class _PreviewTree(tree.InventoryTree):
     """Partial implementation of Tree to support show_diff_trees"""
 
     def __init__(self, transform):
@@ -1789,7 +2015,7 @@
                 yield self._get_repository().revision_tree(revision_id)
 
     def _get_file_revision(self, file_id, vf, tree_revision):
-        parent_keys = [(file_id, self._file_revision(t, file_id)) for t in
+        parent_keys = [(file_id, t.get_file_revision(file_id)) for t in
                        self._iter_parent_trees()]
         vf.add_lines((file_id, tree_revision), parent_keys,
                      self.get_file_lines(file_id))
@@ -1799,8 +2025,9 @@
             vf.fallback_versionedfiles.append(base_vf)
         return tree_revision
 
-    def _stat_limbo_file(self, file_id):
-        trans_id = self._transform.trans_id_file_id(file_id)
+    def _stat_limbo_file(self, file_id=None, trans_id=None):
+        if trans_id is None:
+            trans_id = self._transform.trans_id_file_id(file_id)
         name = self._transform._limbo_name(trans_id)
         return os.lstat(name)
 
@@ -1929,9 +2156,8 @@
             if (specific_file_ids is not None
                 and file_id not in specific_file_ids):
                 continue
-            try:
-                kind = self._transform.final_kind(trans_id)
-            except NoSuchFile:
+            kind = self._transform.final_kind(trans_id)
+            if kind is None:
                 kind = self._transform._tree.stored_kind(file_id)
             new_entry = inventory.make_entry(
                 kind,
@@ -2022,11 +2248,29 @@
 
     def get_file_size(self, file_id):
         """See Tree.get_file_size"""
+        trans_id = self._transform.trans_id_file_id(file_id)
+        kind = self._transform.final_kind(trans_id)
+        if kind != 'file':
+            return None
+        if trans_id in self._transform._new_contents:
+            return self._stat_limbo_file(trans_id=trans_id).st_size
         if self.kind(file_id) == 'file':
             return self._transform._tree.get_file_size(file_id)
         else:
             return None
 
+    def get_file_verifier(self, file_id, path=None, stat_value=None):
+        trans_id = self._transform.trans_id_file_id(file_id)
+        kind = self._transform._new_contents.get(trans_id)
+        if kind is None:
+            return self._transform._tree.get_file_verifier(file_id)
+        if kind == 'file':
+            fileobj = self.get_file(file_id)
+            try:
+                return ("SHA1", sha_file(fileobj))
+            finally:
+                fileobj.close()
+
     def get_file_sha1(self, file_id, path=None, stat_value=None):
         trans_id = self._transform.trans_id_file_id(file_id)
         kind = self._transform._new_contents.get(trans_id)
@@ -2055,6 +2299,15 @@
             except errors.NoSuchId:
                 return False
 
+    def has_filename(self, path):
+        trans_id = self._path2trans_id(path)
+        if trans_id in self._transform._new_contents:
+            return True
+        elif trans_id in self._transform._removed_contents:
+            return False
+        else:
+            return self._transform._tree.has_filename(path)
+
     def path_content_summary(self, path):
         trans_id = self._path2trans_id(path)
         tt = self._transform
@@ -2148,7 +2401,7 @@
                                    self.get_file(file_id).readlines(),
                                    default_revision)
 
-    def get_symlink_target(self, file_id):
+    def get_symlink_target(self, file_id, path=None):
         """See Tree.get_symlink_target"""
         if not self._content_change(file_id):
             return self._transform._tree.get_symlink_target(file_id)
@@ -2169,10 +2422,10 @@
                 path_from_root = self._final_paths.get_path(child_id)
                 basename = self._transform.final_name(child_id)
                 file_id = self._transform.final_file_id(child_id)
-                try:
-                    kind = self._transform.final_kind(child_id)
+                kind  = self._transform.final_kind(child_id)
+                if kind is not None:
                     versioned_kind = kind
-                except NoSuchFile:
+                else:
                     kind = 'unknown'
                     versioned_kind = self._transform._tree.stored_kind(file_id)
                 if versioned_kind == 'directory':
@@ -2292,7 +2545,7 @@
         if num > 0:  # more than just a root
             raise errors.WorkingTreeAlreadyPopulated(base=wt.basedir)
     file_trans_id = {}
-    top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    top_pb = ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
     if tree.inventory.root is not None:
         # This is kind of a hack: we should be altering the root
@@ -2311,7 +2564,7 @@
         pp.next_phase()
         file_trans_id[wt.get_root_id()] = \
             tt.trans_id_tree_file_id(wt.get_root_id())
-        pb = bzrlib.ui.ui_factory.nested_progress_bar()
+        pb = ui.ui_factory.nested_progress_bar()
         try:
             deferred_contents = []
             num = 0
@@ -2331,7 +2584,7 @@
                     existing_files.update(f[0] for f in files)
             for num, (tree_path, entry) in \
                 enumerate(tree.inventory.iter_entries_by_dir()):
-                pb.update("Building tree", num - len(deferred_contents), total)
+                pb.update(gettext("Building tree"), num - len(deferred_contents), total)
                 if entry.parent_id is None:
                     continue
                 reparent = False
@@ -2343,7 +2596,7 @@
                     kind = file_kind(target_path)
                     if kind == "directory":
                         try:
-                            bzrdir.BzrDir.open(target_path)
+                            controldir.ControlDir.open(target_path)
                         except errors.NotBranchError:
                             pass
                         else:
@@ -2364,7 +2617,7 @@
                     executable = tree.is_executable(file_id, tree_path)
                     if executable:
                         tt.set_executability(executable, trans_id)
-                    trans_data = (trans_id, tree_path)
+                    trans_data = (trans_id, tree_path, entry.text_sha1)
                     deferred_contents.append((file_id, trans_data))
                 else:
                     file_trans_id[file_id] = new_by_entry(tt, entry, parent_id,
@@ -2386,7 +2639,7 @@
             precomputed_delta = None
         conflicts = cook_conflicts(raw_conflicts, tt)
         for conflict in conflicts:
-            warning(conflict)
+            trace.warning(unicode(conflict))
         try:
             wt.add_conflicts(conflicts)
         except errors.UnsupportedOperation:
@@ -2415,12 +2668,13 @@
         unchanged = dict(unchanged)
         new_desired_files = []
         count = 0
-        for file_id, (trans_id, tree_path) in desired_files:
+        for file_id, (trans_id, tree_path, text_sha1) in desired_files:
             accelerator_path = unchanged.get(file_id)
             if accelerator_path is None:
-                new_desired_files.append((file_id, (trans_id, tree_path)))
+                new_desired_files.append((file_id,
+                    (trans_id, tree_path, text_sha1)))
                 continue
-            pb.update('Adding file contents', count + offset, total)
+            pb.update(gettext('Adding file contents'), count + offset, total)
             if hardlink:
                 tt.create_hardlink(accelerator_tree.abspath(accelerator_path),
                                    trans_id)
@@ -2431,7 +2685,7 @@
                     contents = filtered_output_bytes(contents, filters,
                         ContentFilterContext(tree_path, tree))
                 try:
-                    tt.create_file(contents, trans_id)
+                    tt.create_file(contents, trans_id, sha1=text_sha1)
                 finally:
                     try:
                         contents.close()
@@ -2440,26 +2694,28 @@
                         pass
             count += 1
         offset += count
-    for count, ((trans_id, tree_path), contents) in enumerate(
+    for count, ((trans_id, tree_path, text_sha1), contents) in enumerate(
             tree.iter_files_bytes(new_desired_files)):
         if wt.supports_content_filtering():
             filters = wt._content_filter_stack(tree_path)
             contents = filtered_output_bytes(contents, filters,
                 ContentFilterContext(tree_path, tree))
-        tt.create_file(contents, trans_id)
-        pb.update('Adding file contents', count + offset, total)
+        tt.create_file(contents, trans_id, sha1=text_sha1)
+        pb.update(gettext('Adding file contents'), count + offset, total)
 
 
 def _reparent_children(tt, old_parent, new_parent):
     for child in tt.iter_tree_children(old_parent):
         tt.adjust_path(tt.final_name(child), new_parent, child)
 
+
 def _reparent_transform_children(tt, old_parent, new_parent):
     by_parent = tt.by_parent()
     for child in by_parent[old_parent]:
         tt.adjust_path(tt.final_name(child), new_parent, child)
     return by_parent[old_parent]
 
+
 def _content_match(tree, entry, file_id, kind, target_path):
     if entry.kind != kind:
         return False
@@ -2565,10 +2821,12 @@
         tt.set_executability(entry.executable, trans_id)
 
 
+@deprecated_function(deprecated_in((2, 3, 0)))
 def get_backup_name(entry, by_parent, parent_trans_id, tt):
     return _get_backup_name(entry.name, by_parent, parent_trans_id, tt)
 
 
+@deprecated_function(deprecated_in((2, 3, 0)))
 def _get_backup_name(name, by_parent, parent_trans_id, tt):
     """Produce a backup-style name that appears to be available"""
     def name_gen():
@@ -2623,7 +2881,7 @@
                 unversioned_filter=working_tree.is_ignored)
             delta.report_changes(tt.iter_changes(), change_reporter)
         for conflict in conflicts:
-            warning(conflict)
+            trace.warning(unicode(conflict))
         pp.next_phase()
         tt.apply()
         working_tree.set_merge_modified(merge_modified)
@@ -2637,7 +2895,7 @@
 def _prepare_revert_transform(working_tree, target_tree, tt, filenames,
                               backups, pp, basis_tree=None,
                               merge_modified=None):
-    child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    child_pb = ui.ui_factory.nested_progress_bar()
     try:
         if merge_modified is None:
             merge_modified = working_tree.merge_modified()
@@ -2646,7 +2904,7 @@
                                       merge_modified, basis_tree)
     finally:
         child_pb.finished()
-    child_pb = bzrlib.ui.ui_factory.nested_progress_bar()
+    child_pb = ui.ui_factory.nested_progress_bar()
     try:
         raw_conflicts = resolve_conflicts(tt, child_pb,
             lambda t, c: conflict_pass(t, c, target_tree))
@@ -2660,7 +2918,11 @@
                  backups, merge_modified, basis_tree=None):
     if basis_tree is not None:
         basis_tree.lock_read()
-    change_list = target_tree.iter_changes(working_tree,
+    # We ask the working_tree for its changes relative to the target, rather
+    # than the target changes relative to the working tree. Because WT4 has an
+    # optimizer to compare itself to a target, but no optimizer for the
+    # reverse.
+    change_list = working_tree.iter_changes(target_tree,
         specific_files=specific_files, pb=pb)
     if target_tree.get_root_id() is None:
         skip_root = True
@@ -2670,13 +2932,19 @@
         deferred_files = []
         for id_num, (file_id, path, changed_content, versioned, parent, name,
                 kind, executable) in enumerate(change_list):
-            if skip_root and file_id[0] is not None and parent[0] is None:
+            target_path, wt_path = path
+            target_versioned, wt_versioned = versioned
+            target_parent, wt_parent = parent
+            target_name, wt_name = name
+            target_kind, wt_kind = kind
+            target_executable, wt_executable = executable
+            if skip_root and wt_parent is None:
                 continue
             trans_id = tt.trans_id_file_id(file_id)
             mode_id = None
             if changed_content:
                 keep_content = False
-                if kind[0] == 'file' and (backups or kind[1] is None):
+                if wt_kind == 'file' and (backups or target_kind is None):
                     wt_sha1 = working_tree.get_file_sha1(file_id)
                     if merge_modified.get(file_id) != wt_sha1:
                         # acquire the basis tree lazily to prevent the
@@ -2685,71 +2953,70 @@
                         if basis_tree is None:
                             basis_tree = working_tree.basis_tree()
                             basis_tree.lock_read()
-                        if file_id in basis_tree:
+                        if basis_tree.has_id(file_id):
                             if wt_sha1 != basis_tree.get_file_sha1(file_id):
                                 keep_content = True
-                        elif kind[1] is None and not versioned[1]:
+                        elif target_kind is None and not target_versioned:
                             keep_content = True
-                if kind[0] is not None:
+                if wt_kind is not None:
                     if not keep_content:
                         tt.delete_contents(trans_id)
-                    elif kind[1] is not None:
-                        parent_trans_id = tt.trans_id_file_id(parent[0])
-                        by_parent = tt.by_parent()
-                        backup_name = _get_backup_name(name[0], by_parent,
-                                                       parent_trans_id, tt)
+                    elif target_kind is not None:
+                        parent_trans_id = tt.trans_id_file_id(wt_parent)
+                        backup_name = tt._available_backup_name(
+                            wt_name, parent_trans_id)
                         tt.adjust_path(backup_name, parent_trans_id, trans_id)
-                        new_trans_id = tt.create_path(name[0], parent_trans_id)
-                        if versioned == (True, True):
+                        new_trans_id = tt.create_path(wt_name, parent_trans_id)
+                        if wt_versioned and target_versioned:
                             tt.unversion_file(trans_id)
                             tt.version_file(file_id, new_trans_id)
                         # New contents should have the same unix perms as old
                         # contents
                         mode_id = trans_id
                         trans_id = new_trans_id
-                if kind[1] in ('directory', 'tree-reference'):
+                if target_kind in ('directory', 'tree-reference'):
                     tt.create_directory(trans_id)
-                    if kind[1] == 'tree-reference':
+                    if target_kind == 'tree-reference':
                         revision = target_tree.get_reference_revision(file_id,
-                                                                      path[1])
+                                                                      target_path)
                         tt.set_tree_reference(revision, trans_id)
-                elif kind[1] == 'symlink':
+                elif target_kind == 'symlink':
                     tt.create_symlink(target_tree.get_symlink_target(file_id),
                                       trans_id)
-                elif kind[1] == 'file':
+                elif target_kind == 'file':
                     deferred_files.append((file_id, (trans_id, mode_id)))
                     if basis_tree is None:
                         basis_tree = working_tree.basis_tree()
                         basis_tree.lock_read()
                     new_sha1 = target_tree.get_file_sha1(file_id)
-                    if (file_id in basis_tree and new_sha1 ==
-                        basis_tree.get_file_sha1(file_id)):
+                    if (basis_tree.has_id(file_id) and
+                        new_sha1 == basis_tree.get_file_sha1(file_id)):
                         if file_id in merge_modified:
                             del merge_modified[file_id]
                     else:
                         merge_modified[file_id] = new_sha1
 
                     # preserve the execute bit when backing up
-                    if keep_content and executable[0] == executable[1]:
-                        tt.set_executability(executable[1], trans_id)
-                elif kind[1] is not None:
-                    raise AssertionError(kind[1])
-            if versioned == (False, True):
+                    if keep_content and wt_executable == target_executable:
+                        tt.set_executability(target_executable, trans_id)
+                elif target_kind is not None:
+                    raise AssertionError(target_kind)
+            if not wt_versioned and target_versioned:
                 tt.version_file(file_id, trans_id)
-            if versioned == (True, False):
+            if wt_versioned and not target_versioned:
                 tt.unversion_file(trans_id)
-            if (name[1] is not None and
-                (name[0] != name[1] or parent[0] != parent[1])):
-                if name[1] == '' and parent[1] is None:
+            if (target_name is not None and
+                (wt_name != target_name or wt_parent != target_parent)):
+                if target_name == '' and target_parent is None:
                     parent_trans = ROOT_PARENT
                 else:
-                    parent_trans = tt.trans_id_file_id(parent[1])
-                if parent[0] is None and versioned[0]:
-                    tt.adjust_root_path(name[1], parent_trans)
+                    parent_trans = tt.trans_id_file_id(target_parent)
+                if wt_parent is None and wt_versioned:
+                    tt.adjust_root_path(target_name, parent_trans)
                 else:
-                    tt.adjust_path(name[1], parent_trans, trans_id)
-            if executable[0] != executable[1] and kind[1] == "file":
-                tt.set_executability(executable[1], trans_id)
+                    tt.adjust_path(target_name, parent_trans, trans_id)
+            if wt_executable != target_executable and target_kind == "file":
+                tt.set_executability(target_executable, trans_id)
         if working_tree.supports_content_filtering():
             for index, ((trans_id, mode_id), bytes) in enumerate(
                 target_tree.iter_files_bytes(deferred_files)):
@@ -2781,7 +3048,7 @@
     pb = ui.ui_factory.nested_progress_bar()
     try:
         for n in range(10):
-            pb.update('Resolution pass', n+1, 10)
+            pb.update(gettext('Resolution pass'), n+1, 10)
             conflicts = tt.find_conflicts()
             if len(conflicts) == 0:
                 return new_conflicts
@@ -2826,11 +3093,30 @@
 
         elif c_type == 'missing parent':
             trans_id = conflict[1]
-            try:
-                tt.cancel_deletion(trans_id)
-                new_conflicts.add(('deleting parent', 'Not deleting',
-                                   trans_id))
-            except KeyError:
+            if trans_id in tt._removed_contents:
+                cancel_deletion = True
+                orphans = tt._get_potential_orphans(trans_id)
+                if orphans:
+                    cancel_deletion = False
+                    # All children are orphans
+                    for o in orphans:
+                        try:
+                            tt.new_orphan(o, trans_id)
+                        except OrphaningError:
+                            # Something bad happened so we cancel the directory
+                            # deletion which will leave it in place with a
+                            # conflict. The user can deal with it from there.
+                            # Note that this also catch the case where we don't
+                            # want to create orphans and leave the directory in
+                            # place.
+                            cancel_deletion = True
+                            break
+                if cancel_deletion:
+                    # Cancel the directory deletion
+                    tt.cancel_deletion(trans_id)
+                    new_conflicts.add(('deleting parent', 'Not deleting',
+                                       trans_id))
+            else:
                 create = True
                 try:
                     tt.final_name(trans_id)
@@ -2839,7 +3125,8 @@
                         file_id = tt.final_file_id(trans_id)
                         if file_id is None:
                             file_id = tt.inactive_file_id(trans_id)
-                        entry = path_tree.inventory[file_id]
+                        _, entry = path_tree.iter_entries_by_dir(
+                            [file_id]).next()
                         # special-case the other tree root (move its
                         # children to current root)
                         if entry.parent_id is None:
@@ -2860,8 +3147,9 @@
         elif c_type == 'unversioned parent':
             file_id = tt.inactive_file_id(conflict[1])
             # special-case the other tree root (move its children instead)
-            if path_tree and file_id in path_tree:
-                if path_tree.inventory[file_id].parent_id is None:
+            if path_tree and path_tree.has_id(file_id):
+                if path_tree.path2id('') == file_id:
+                    # This is the root entry, skip it
                     continue
             tt.version_file(file_id, conflict[1])
             new_conflicts.add((c_type, 'Versioned directory', conflict[1]))
@@ -2883,13 +3171,11 @@
 
 def cook_conflicts(raw_conflicts, tt):
     """Generate a list of cooked conflicts, sorted by file path"""
-    from bzrlib.conflicts import Conflict
     conflict_iter = iter_cook_conflicts(raw_conflicts, tt)
-    return sorted(conflict_iter, key=Conflict.sort_key)
+    return sorted(conflict_iter, key=conflicts.Conflict.sort_key)
 
 
 def iter_cook_conflicts(raw_conflicts, tt):
-    from bzrlib.conflicts import Conflict
     fp = FinalPaths(tt)
     for conflict in raw_conflicts:
         c_type = conflict[0]
@@ -2897,16 +3183,17 @@
         modified_path = fp.get_path(conflict[2])
         modified_id = tt.final_file_id(conflict[2])
         if len(conflict) == 3:
-            yield Conflict.factory(c_type, action=action, path=modified_path,
-                                     file_id=modified_id)
+            yield conflicts.Conflict.factory(
+                c_type, action=action, path=modified_path, file_id=modified_id)
 
         else:
             conflicting_path = fp.get_path(conflict[3])
             conflicting_id = tt.final_file_id(conflict[3])
-            yield Conflict.factory(c_type, action=action, path=modified_path,
-                                   file_id=modified_id,
-                                   conflict_path=conflicting_path,
-                                   conflict_file_id=conflicting_id)
+            yield conflicts.Conflict.factory(
+                c_type, action=action, path=modified_path,
+                file_id=modified_id,
+                conflict_path=conflicting_path,
+                conflict_file_id=conflicting_id)
 
 
 class _FileMover(object):
@@ -2919,8 +3206,8 @@
     def rename(self, from_, to):
         """Rename a file from one path to another."""
         try:
-            osutils.rename(from_, to)
-        except (IOError, OSError), e:
+            os.rename(from_, to)
+        except OSError, e:
             if e.errno in (errno.EEXIST, errno.ENOTEMPTY):
                 raise errors.FileExists(to, str(e))
             # normal OSError doesn't include filenames so it's hard to see where
@@ -2942,9 +3229,9 @@
         """Reverse all renames that have been performed"""
         for from_, to in reversed(self.past_renames):
             try:
-                osutils.rename(to, from_)
-            except (OSError, IOError), e:
-                raise errors.TransformRenameFailed(to, from_, str(e), e.errno)                
+                os.rename(to, from_)
+            except OSError, e:
+                raise errors.TransformRenameFailed(to, from_, str(e), e.errno)
         # after rollback, don't reuse _FileMover
         past_renames = None
         pending_deletions = None

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2010-06-02 01:45:31 +0000
+++ bzrlib/transport/__init__.py	2011-09-04 23:25:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 """
 
 from cStringIO import StringIO
-import re
 import sys
 
 from bzrlib.lazy_import import lazy_import
@@ -47,8 +46,8 @@
 """)
 
 from bzrlib.symbol_versioning import (
-        DEPRECATED_PARAMETER,
-        )
+    DEPRECATED_PARAMETER,
+    )
 from bzrlib.trace import (
     mutter,
     )
@@ -84,10 +83,7 @@
     modules = set()
     for prefix, factory_list in transport_list_registry.items():
         for factory in factory_list:
-            if hasattr(factory, "_module_name"):
-                modules.add(factory._module_name)
-            else:
-                modules.add(factory._obj.__module__)
+            modules.add(factory.get_module())
     # Add chroot and pathfilter directly, because there is no handler
     # registered for it.
     modules.add('bzrlib.transport.chroot')
@@ -122,10 +118,6 @@
     def register_transport(self, key, help=None):
         self.register(key, [], help)
 
-    def set_default_transport(self, key=None):
-        """Return either 'key' or the default key if key is None"""
-        self._default_key = key
-
 
 transport_list_registry = TransportListRegistry()
 
@@ -235,10 +227,24 @@
     def _close(self):
         """A hook point for subclasses that need to take action on close."""
 
-    def close(self):
+    def close(self, want_fdatasync=False):
+        if want_fdatasync:
+            try:
+                self.fdatasync()
+            except errors.TransportNotPossible:
+                pass
         self._close()
         del _file_streams[self.transport.abspath(self.relpath)]
 
+    def fdatasync(self):
+        """Force data out to physical disk if possible.
+
+        :raises TransportNotPossible: If this transport has no way to 
+            flush to disk.
+        """
+        raise errors.TransportNotPossible(
+            "%s cannot fdatasync" % (self.transport,))
+
 
 class FileFileStream(FileStream):
     """A file stream object returned by open_write_stream.
@@ -253,6 +259,15 @@
     def _close(self):
         self.file_handle.close()
 
+    def fdatasync(self):
+        """Force data out to physical disk if possible."""
+        self.file_handle.flush()
+        try:
+            fileno = self.file_handle.fileno()
+        except AttributeError:
+            raise errors.TransportNotPossible()
+        osutils.fdatasync(fileno)
+
     def write(self, bytes):
         osutils.pump_string_file(bytes, self.file_handle)
 
@@ -295,6 +310,7 @@
     def __init__(self, base):
         super(Transport, self).__init__()
         self.base = base
+        self._segment_parameters = urlutils.split_segment_parameters(base)[1]
 
     def _translate_error(self, e, path, raise_generic=True):
         """Translate an IOError or OSError into an appropriate bzr error.
@@ -302,7 +318,10 @@
         This handles things like ENOENT, ENOTDIR, EEXIST, and EACCESS
         """
         if getattr(e, 'errno', None) is not None:
-            if e.errno in (errno.ENOENT, errno.ENOTDIR, errno.EINVAL):
+            if e.errno in (errno.ENOENT, errno.ENOTDIR):
+                raise errors.NoSuchFile(path, extra=e)
+            elif e.errno == errno.EINVAL:
+                mutter("EINVAL returned on path %s: %r" % (path, e))
                 raise errors.NoSuchFile(path, extra=e)
             # I would rather use errno.EFOO, but there doesn't seem to be
             # any matching for 267
@@ -391,6 +410,11 @@
         """
         raise NotImplementedError(self.external_url)
 
+    def get_segment_parameters(self):
+        """Return the segment parameters for the top segment of the URL.
+        """
+        return self._segment_parameters
+
     def _pump(self, from_file, to_file):
         """Most children will need to copy from one file-like
         object or string to another one.
@@ -461,50 +485,6 @@
         # interface ?
         raise NotImplementedError(self.abspath)
 
-    def _combine_paths(self, base_path, relpath):
-        """Transform a Transport-relative path to a remote absolute path.
-
-        This does not handle substitution of ~ but does handle '..' and '.'
-        components.
-
-        Examples::
-
-            t._combine_paths('/home/sarah', 'project/foo')
-                => '/home/sarah/project/foo'
-            t._combine_paths('/home/sarah', '../../etc')
-                => '/etc'
-            t._combine_paths('/home/sarah', '/etc')
-                => '/etc'
-
-        :param base_path: urlencoded path for the transport root; typically a
-             URL but need not contain scheme/host/etc.
-        :param relpath: relative url string for relative part of remote path.
-        :return: urlencoded string for final path.
-        """
-        if not isinstance(relpath, str):
-            raise errors.InvalidURL(relpath)
-        if relpath.startswith('/'):
-            base_parts = []
-        else:
-            base_parts = base_path.split('/')
-        if len(base_parts) > 0 and base_parts[-1] == '':
-            base_parts = base_parts[:-1]
-        for p in relpath.split('/'):
-            if p == '..':
-                if len(base_parts) == 0:
-                    # In most filesystems, a request for the parent
-                    # of root, just returns root.
-                    continue
-                base_parts.pop()
-            elif p == '.':
-                continue # No-op
-            elif p != '':
-                base_parts.append(p)
-        path = '/'.join(base_parts)
-        if not path.startswith('/'):
-            path = '/' + path
-        return path
-
     def recommended_page_size(self):
         """Return the recommended page size for this transport.
 
@@ -676,7 +656,9 @@
 
         This uses _coalesce_offsets to issue larger reads and fewer seeks.
 
-        :param fp: A file-like object that supports seek() and read(size)
+        :param fp: A file-like object that supports seek() and read(size).
+            Note that implementations are allowed to call .close() on this file
+            handle, so don't trust that you can use it for other work.
         :param offsets: A list of offsets to be read from the given file.
         :return: yield (pos, data) tuples for each request
         """
@@ -693,37 +675,32 @@
 
         # Cache the results, but only until they have been fulfilled
         data_map = {}
-        for c_offset in coalesced:
-            # TODO: jam 20060724 it might be faster to not issue seek if
-            #       we are already at the right location. This should be
-            #       benchmarked.
-            fp.seek(c_offset.start)
-            data = fp.read(c_offset.length)
-            if len(data) < c_offset.length:
-                raise errors.ShortReadvError(relpath, c_offset.start,
-                            c_offset.length, actual=len(data))
-            for suboffset, subsize in c_offset.ranges:
-                key = (c_offset.start+suboffset, subsize)
-                data_map[key] = data[suboffset:suboffset+subsize]
+        try:
+            for c_offset in coalesced:
+                # TODO: jam 20060724 it might be faster to not issue seek if
+                #       we are already at the right location. This should be
+                #       benchmarked.
+                fp.seek(c_offset.start)
+                data = fp.read(c_offset.length)
+                if len(data) < c_offset.length:
+                    raise errors.ShortReadvError(relpath, c_offset.start,
+                                c_offset.length, actual=len(data))
+                for suboffset, subsize in c_offset.ranges:
+                    key = (c_offset.start+suboffset, subsize)
+                    data_map[key] = data[suboffset:suboffset+subsize]
 
-            # Now that we've read some data, see if we can yield anything back
-            while cur_offset_and_size in data_map:
-                this_data = data_map.pop(cur_offset_and_size)
-                this_offset = cur_offset_and_size[0]
-                try:
-                    cur_offset_and_size = offset_stack.next()
-                except StopIteration:
-                    # Close the file handle as there will be no more data
-                    # The handle would normally be cleaned up as this code goes
-                    # out of scope, but as we are a generator, not all code
-                    # will re-enter once we have consumed all the expected
-                    # data. For example:
-                    #   zip(range(len(requests)), readv(foo, requests))
-                    # Will stop because the range is done, and not run the
-                    # cleanup code for the readv().
-                    fp.close()
-                    cur_offset_and_size = None
-                yield this_offset, this_data
+                # Now that we've read some data, see if we can yield anything back
+                while cur_offset_and_size in data_map:
+                    this_data = data_map.pop(cur_offset_and_size)
+                    this_offset = cur_offset_and_size[0]
+                    try:
+                        cur_offset_and_size = offset_stack.next()
+                    except StopIteration:
+                        fp.close()
+                        cur_offset_and_size = None
+                    yield this_offset, this_data
+        finally:
+            fp.close()
 
     def _sort_expand_and_combine(self, offsets, upper_limit):
         """Helper for readv.
@@ -1286,6 +1263,12 @@
         # should be asked to ConnectedTransport only.
         return None
 
+    def disconnect(self):
+        # This is really needed for ConnectedTransport only, but it's easier to
+        # have Transport do nothing than testing that the disconnect should be
+        # asked to ConnectedTransport only.
+        pass
+
     def _redirected_to(self, source, target):
         """Returns a transport suitable to re-issue a redirected request.
 
@@ -1342,21 +1325,17 @@
         """
         if not base.endswith('/'):
             base += '/'
-        (self._scheme,
-         self._user, self._password,
-         self._host, self._port,
-         self._path) = self._split_url(base)
+        self._parsed_url = self._split_url(base)
         if _from_transport is not None:
             # Copy the password as it does not appear in base and will be lost
             # otherwise. It can appear in the _split_url above if the user
             # provided it on the command line. Otherwise, daughter classes will
             # prompt the user for one when appropriate.
-            self._password = _from_transport._password
+            self._parsed_url.password = _from_transport._parsed_url.password
+            self._parsed_url.quoted_password = (
+                _from_transport._parsed_url.quoted_password)
 
-        base = self._unsplit_url(self._scheme,
-                                 self._user, self._password,
-                                 self._host, self._port,
-                                 self._path)
+        base = str(self._parsed_url)
 
         super(ConnectedTransport, self).__init__(base)
         if _from_transport is None:
@@ -1364,6 +1343,30 @@
         else:
             self._shared_connection = _from_transport._shared_connection
 
+    @property
+    def _user(self):
+        return self._parsed_url.user
+
+    @property
+    def _password(self):
+        return self._parsed_url.password
+
+    @property
+    def _host(self):
+        return self._parsed_url.host
+
+    @property
+    def _port(self):
+        return self._parsed_url.port
+
+    @property
+    def _path(self):
+        return self._parsed_url.path
+
+    @property
+    def _scheme(self):
+        return self._parsed_url.scheme
+
     def clone(self, offset=None):
         """Return a new transport with root at self.base + offset
 
@@ -1377,26 +1380,20 @@
 
     @staticmethod
     def _split_url(url):
-        return urlutils.parse_url(url)
+        return urlutils.URL.from_string(url)
 
     @staticmethod
     def _unsplit_url(scheme, user, password, host, port, path):
-        """
-        Build the full URL for the given already URL encoded path.
+        """Build the full URL for the given already URL encoded path.
 
         user, password, host and path will be quoted if they contain reserved
         chars.
 
         :param scheme: protocol
-
         :param user: login
-
         :param password: associated password
-
         :param host: the server address
-
         :param port: the associated port
-
         :param path: the absolute path on the server
 
         :return: The corresponding URL.
@@ -1414,23 +1411,24 @@
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path"""
-        scheme, user, password, host, port, path = self._split_url(abspath)
+        parsed_url = self._split_url(abspath)
         error = []
-        if (scheme != self._scheme):
+        if parsed_url.scheme != self._parsed_url.scheme:
             error.append('scheme mismatch')
-        if (user != self._user):
+        if parsed_url.user != self._parsed_url.user:
             error.append('user name mismatch')
-        if (host != self._host):
+        if parsed_url.host != self._parsed_url.host:
             error.append('host mismatch')
-        if (port != self._port):
+        if parsed_url.port != self._parsed_url.port:
             error.append('port mismatch')
-        if not (path == self._path[:-1] or path.startswith(self._path)):
+        if (not (parsed_url.path == self._parsed_url.path[:-1] or
+            parsed_url.path.startswith(self._parsed_url.path))):
             error.append('path mismatch')
         if error:
             extra = ', '.join(error)
             raise errors.PathNotChild(abspath, self.base, extra=extra)
-        pl = len(self._path)
-        return path[pl:].strip('/')
+        pl = len(self._parsed_url.path)
+        return parsed_url.path[pl:].strip('/')
 
     def abspath(self, relpath):
         """Return the full url to the given relative path.
@@ -1439,11 +1437,7 @@
 
         :returns: the Unicode version of the absolute path for relpath.
         """
-        relative = urlutils.unescape(relpath).encode('utf-8')
-        path = self._combine_paths(self._path, relative)
-        return self._unsplit_url(self._scheme, self._user, self._password,
-                                 self._host, self._port,
-                                 path)
+        return str(self._parsed_url.clone(relpath))
 
     def _remote_path(self, relpath):
         """Return the absolute path part of the url to the given relative path.
@@ -1456,9 +1450,7 @@
 
         :return: the absolute Unicode path on the server,
         """
-        relative = urlutils.unescape(relpath).encode('utf-8')
-        remote_path = self._combine_paths(self._path, relative)
-        return remote_path
+        return self._parsed_url.clone(relpath).path
 
     def _get_shared_connection(self):
         """Get the object shared amongst cloned transports.
@@ -1523,8 +1515,7 @@
         :return: A new transport or None if the connection cannot be shared.
         """
         try:
-            (scheme, user, password,
-             host, port, path) = self._split_url(other_base)
+            parsed_url = self._split_url(other_base)
         except errors.InvalidURL:
             # No hope in trying to reuse an existing transport for an invalid
             # URL
@@ -1533,15 +1524,16 @@
         transport = None
         # Don't compare passwords, they may be absent from other_base or from
         # self and they don't carry more information than user anyway.
-        if (scheme == self._scheme
-            and user == self._user
-            and host == self._host
-            and port == self._port):
+        if (parsed_url.scheme == self._parsed_url.scheme
+            and parsed_url.user == self._parsed_url.user
+            and parsed_url.host == self._parsed_url.host
+            and parsed_url.port == self._parsed_url.port):
+            path = parsed_url.path
             if not path.endswith('/'):
                 # This normally occurs at __init__ time, but it's easier to do
                 # it now to avoid creating two transports for the same base.
                 path += '/'
-            if self._path  == path:
+            if self._parsed_url.path  == path:
                 # shortcut, it's really the same transport
                 return self
             # We don't call clone here because the intent is different: we
@@ -1550,72 +1542,106 @@
             transport = self.__class__(other_base, _from_transport=self)
         return transport
 
-
-def get_transport(base, possible_transports=None):
-    """Open a transport to access a URL or directory.
-
-    :param base: either a URL or a directory name.
-
-    :param transports: optional reusable transports list. If not None, created
-        transports will be added to the list.
-
-    :return: A new transport optionally sharing its connection with one of
-        possible_transports.
+    def disconnect(self):
+        """Disconnect the transport.
+
+        If and when required the transport willl reconnect automatically.
+        """
+        raise NotImplementedError(self.disconnect)
+
+
+def location_to_url(location):
+    """Determine a fully qualified URL from a location string.
+
+    This will try to interpret location as both a URL and a directory path. It
+    will also lookup the location in directories.
+
+    :param location: Unicode or byte string object with a location
+    :raise InvalidURL: If the location is already a URL, but not valid.
+    :return: Byte string with resulting URL
     """
-    if base is None:
-        base = '.'
-    last_err = None
+    if not isinstance(location, basestring):
+        raise AssertionError("location not a byte or unicode string")
     from bzrlib.directory_service import directories
-    base = directories.dereference(base)
-
-    def convert_path_to_url(base, error_str):
-        if urlutils.is_url(base):
-            # This looks like a URL, but we weren't able to
-            # instantiate it as such raise an appropriate error
-            # FIXME: we have a 'error_str' unused and we use last_err below
-            raise errors.UnsupportedProtocol(base, last_err)
-        # This doesn't look like a protocol, consider it a local path
-        new_base = urlutils.local_path_to_url(base)
-        # mutter('converting os path %r => url %s', base, new_base)
-        return new_base
+    location = directories.dereference(location)
 
     # Catch any URLs which are passing Unicode rather than ASCII
     try:
-        base = base.encode('ascii')
+        location = location.encode('ascii')
     except UnicodeError:
-        # Only local paths can be Unicode
-        base = convert_path_to_url(base,
-            'URLs must be properly escaped (protocol: %s)')
-
+        if urlutils.is_url(location):
+            raise errors.InvalidURL(path=location,
+                extra='URLs must be properly escaped')
+        location = urlutils.local_path_to_url(location)
+
+    if location.startswith("file:") and not location.startswith("file://"):
+        return urlutils.join(urlutils.local_path_to_url("."), location[5:])
+
+    if not urlutils.is_url(location):
+        return urlutils.local_path_to_url(location)
+
+    return location
+
+
+def get_transport_from_path(path, possible_transports=None):
+    """Open a transport for a local path.
+
+    :param path: Local path as byte or unicode string
+    :return: Transport object for path
+    """
+    return get_transport_from_url(urlutils.local_path_to_url(path),
+        possible_transports)
+
+
+def get_transport_from_url(url, possible_transports=None):
+    """Open a transport to access a URL.
+    
+    :param base: a URL
+    :param transports: optional reusable transports list. If not None, created
+        transports will be added to the list.
+
+    :return: A new transport optionally sharing its connection with one of
+        possible_transports.
+    """
     transport = None
     if possible_transports is not None:
         for t in possible_transports:
-            t_same_connection = t._reuse_for(base)
+            t_same_connection = t._reuse_for(url)
             if t_same_connection is not None:
                 # Add only new transports
                 if t_same_connection not in possible_transports:
                     possible_transports.append(t_same_connection)
                 return t_same_connection
 
+    last_err = None
     for proto, factory_list in transport_list_registry.items():
-        if proto is not None and base.startswith(proto):
-            transport, last_err = _try_transport_factories(base, factory_list)
+        if proto is not None and url.startswith(proto):
+            transport, last_err = _try_transport_factories(url, factory_list)
             if transport:
                 if possible_transports is not None:
                     if transport in possible_transports:
                         raise AssertionError()
                     possible_transports.append(transport)
                 return transport
-
-    # We tried all the different protocols, now try one last time
-    # as a local protocol
-    base = convert_path_to_url(base, 'Unsupported protocol: %s')
-
-    # The default handler is the filesystem handler, stored as protocol None
-    factory_list = transport_list_registry.get(None)
-    transport, last_err = _try_transport_factories(base, factory_list)
-
-    return transport
+    if not urlutils.is_url(url):
+        raise errors.InvalidURL(path=url)
+    raise errors.UnsupportedProtocol(url, last_err)
+
+
+def get_transport(base, possible_transports=None):
+    """Open a transport to access a URL or directory.
+
+    :param base: either a URL or a directory name.
+
+    :param transports: optional reusable transports list. If not None, created
+        transports will be added to the list.
+
+    :return: A new transport optionally sharing its connection with one of
+        possible_transports.
+    """
+    if base is None:
+        base = '.'
+    return get_transport_from_url(location_to_url(base), possible_transports)
 
 
 def _try_transport_factories(base, factory_list):
@@ -1690,7 +1716,6 @@
 register_transport_proto('file://',
             help="Access using the standard filesystem (default)")
 register_lazy_transport('file://', 'bzrlib.transport.local', 'LocalTransport')
-transport_list_registry.set_default_transport("file://")
 
 register_transport_proto('sftp://',
             help="Access using SFTP (most SSH servers provide SFTP).",
@@ -1763,9 +1788,6 @@
 register_lazy_transport('memory://', 'bzrlib.transport.memory',
                         'MemoryTransport')
 
-# chroots cannot be implicitly accessed, they must be explicitly created:
-register_transport_proto('chroot+')
-
 register_transport_proto('readonly+',
 #              help="This modifier converts any transport to be readonly."
             )
@@ -1800,12 +1822,6 @@
 register_lazy_transport('nosmart+', 'bzrlib.transport.nosmart',
                         'NoSmartTransportDecorator')
 
-# These two schemes were registered, but don't seem to have an actual transport
-# protocol registered
-for scheme in ['ssh', 'bzr+loopback']:
-    register_urlparse_netloc_protocol(scheme)
-del scheme
-
 register_transport_proto('bzr://',
             help="Fast access using the Bazaar smart server.",
                          register_netloc=True)

=== modified file 'bzrlib/transport/decorator.py'
--- bzrlib/transport/decorator.py	2010-06-20 11:18:38 +0000
+++ bzrlib/transport/decorator.py	2011-08-19 22:34:02 +0000
@@ -50,7 +50,8 @@
                              (url, prefix))
         not_decorated_url = url[len(prefix):]
         if _decorated is None:
-            self._decorated = transport.get_transport(not_decorated_url)
+            self._decorated = transport.get_transport(
+                not_decorated_url)
         else:
             self._decorated = _decorated
         super(TransportDecorator, self).__init__(prefix + self._decorated.base)
@@ -156,6 +157,15 @@
         """See Transport.rmdir."""
         return self._decorated.rmdir(relpath)
 
+    def _get_segment_parameters(self):
+        return self._decorated.segment_parameters
+
+    def _set_segment_parameters(self, value):
+        self._decorated.segment_parameters = value
+
+    segment_parameters = property(_get_segment_parameters,
+        _set_segment_parameters, "See Transport.segment_parameters")
+
     def stat(self, relpath):
         """See Transport.stat()."""
         return self._decorated.stat(relpath)

=== modified file 'bzrlib/transport/ftp/__init__.py'
--- bzrlib/transport/ftp/__init__.py	2010-02-27 01:37:54 +0000
+++ bzrlib/transport/ftp/__init__.py	2011-08-08 17:52:59 +0000
@@ -40,6 +40,12 @@
     osutils,
     urlutils,
     )
+from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
+    deprecated_in,
+    deprecated_passed,
+    warn,
+    )
 from bzrlib.trace import mutter, warning
 from bzrlib.transport import (
     AppendBasedFileStream,
@@ -90,7 +96,7 @@
         super(FtpTransport, self).__init__(base,
                                            _from_transport=_from_transport)
         self._unqualified_scheme = 'ftp'
-        if self._scheme == 'aftp':
+        if self._parsed_url.scheme == 'aftp':
             self.is_active = True
         else:
             self.is_active = False
@@ -164,6 +170,11 @@
         connection, credentials = self._create_connection(credentials)
         self._set_connection(connection, credentials)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _translate_ftp_error(self, err, path, extra=None,
                               unknown_exc=FtpPathError):
         """Try to translate an ftplib exception to a bzrlib exception.
@@ -191,6 +202,9 @@
             or 'file/directory not found' in s # filezilla server
             # Microsoft FTP-Service RNFR reply if file not found
             or (s.startswith('550 ') and 'unable to rename to' in extra)
+            # if containing directory doesn't exist, suggested by
+            # <https://bugs.launchpad.net/bzr/+bug/224373>
+            or (s.startswith('550 ') and "can't find folder" in s)
             ):
             raise errors.NoSuchFile(path, extra=extra)
         elif ('file exists' in s):
@@ -232,7 +246,7 @@
             mutter("FTP has not: %s: %s", abspath, e)
             return False
 
-    def get(self, relpath, decode=False, retries=0):
+    def get(self, relpath, decode=DEPRECATED_PARAMETER, retries=0):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
@@ -242,7 +256,10 @@
         We're meant to return a file-like object which bzr will
         then read from. For now we do this via the magic of StringIO
         """
-        # TODO: decode should be deprecated
+        if deprecated_passed(decode):
+            warn(deprecated_in((2,3,0)) %
+                 '"decode" parameter to FtpTransport.get()',
+                 DeprecationWarning, stacklevel=2)
         try:
             mutter("FTP get: %s", self._remote_path(relpath))
             f = self._get_FTP()
@@ -314,8 +331,9 @@
                     return len(bytes)
                 else:
                     return fp.counted_bytes
-            except (ftplib.error_temp,EOFError), e:
-                warning("Failure during ftp PUT. Deleting temporary file.")
+            except (ftplib.error_temp, EOFError), e:
+                warning("Failure during ftp PUT of %s: %s. Deleting temporary file."
+                    % (tmp_abspath, e, ))
                 try:
                     f.delete(tmp_abspath)
                 except:
@@ -328,8 +346,9 @@
                                        unknown_exc=errors.NoSuchFile)
         except ftplib.error_temp, e:
             if retries > _number_of_retries:
-                raise errors.TransportError("FTP temporary error during PUT %s. Aborting."
-                                     % self.abspath(relpath), orig_error=e)
+                raise errors.TransportError(
+                    "FTP temporary error during PUT %s: %s. Aborting."
+                    % (self.abspath(relpath), e), orig_error=e)
             else:
                 warning("FTP temporary error: %s. Retrying.", str(e))
                 self._reconnect()
@@ -350,7 +369,17 @@
         try:
             mutter("FTP mkd: %s", abspath)
             f = self._get_FTP()
-            f.mkd(abspath)
+            try:
+                f.mkd(abspath)
+            except ftplib.error_reply, e:
+                # <https://bugs.launchpad.net/bzr/+bug/224373> Microsoft FTP
+                # server returns "250 Directory created." which is kind of
+                # reasonable, 250 meaning "requested file action OK", but not what
+                # Python's ftplib expects.
+                if e[0][:3] == '250':
+                    pass
+                else:
+                    raise
             self._setmode(relpath, mode)
         except ftplib.error_perm, e:
             self._translate_ftp_error(e, abspath,

=== modified file 'bzrlib/transport/ftp/_gssapi.py'
--- bzrlib/transport/ftp/_gssapi.py	2010-05-30 16:26:11 +0000
+++ bzrlib/transport/ftp/_gssapi.py	2011-09-20 10:49:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2008, 2009, 2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 from bzrlib import (
     errors,
     )
+from bzrlib.i18n import gettext
 from bzrlib.trace import mutter
 from bzrlib.transport.ftp import FtpTransport
 
@@ -79,8 +80,8 @@
                         if not ((resp.startswith('235 ') and rc == 1) or
                                 (resp.startswith('335 ') and rc == 0)):
                             raise ftplib.error_reply, resp
-            trace.note("Authenticated as %s" % kerberos.authGSSClientUserName(
-                    self.vc))
+            trace.note(gettext("Authenticated as %s") %
+                            kerberos.authGSSClientUserName(self.vc))
 
             # Monkey patch ftplib
             self.putcmd = self.mic_putcmd

=== modified file 'bzrlib/transport/gio_transport.py'
--- bzrlib/transport/gio_transport.py	2010-05-28 18:52:03 +0000
+++ bzrlib/transport/gio_transport.py	2011-08-11 16:24:09 +0000
@@ -24,15 +24,10 @@
 supported by gio.
 """
 from cStringIO import StringIO
-import getpass
 import os
 import random
-import socket
 import stat
-import urllib
 import time
-import sys
-import getpass
 import urlparse
 
 from bzrlib import (
@@ -43,12 +38,17 @@
     debug,
     ui,
     )
-from bzrlib.trace import mutter, warning
+from bzrlib.symbol_versioning import (
+    DEPRECATED_PARAMETER,
+    deprecated_in,
+    deprecated_passed,
+    warn,
+    )
+from bzrlib.trace import mutter
 from bzrlib.transport import (
     FileStream,
     ConnectedTransport,
     _file_streams,
-    Server,
     )
 
 from bzrlib.tests.test_server import TestServer
@@ -165,35 +165,37 @@
         #really use bzrlib.auth get_password for this
         #or possibly better gnome-keyring?
         auth = config.AuthenticationConfig()
-        (scheme, urluser, urlpassword, host, port, urlpath) = \
-           urlutils.parse_url(self.url)
+        parsed_url = urlutils.URL.from_string(self.url)
         user = None
         if (flags & gio.ASK_PASSWORD_NEED_USERNAME and
                 flags & gio.ASK_PASSWORD_NEED_DOMAIN):
-            prompt = scheme.upper() + ' %(host)s DOMAIN\username'
-            user_and_domain = auth.get_user(scheme, host,
-                    port=port, ask=True, prompt=prompt)
+            prompt = (u'%s' % (parsed_url.scheme.upper(),) +
+                      u' %(host)s DOMAIN\\username')
+            user_and_domain = auth.get_user(parsed_url.scheme,
+                parsed_url.host, port=parsed_url.port, ask=True,
+                prompt=prompt)
             (domain, user) = user_and_domain.split('\\', 1)
             op.set_username(user)
             op.set_domain(domain)
         elif flags & gio.ASK_PASSWORD_NEED_USERNAME:
-            user = auth.get_user(scheme, host,
-                    port=port, ask=True)
+            user = auth.get_user(parsed_url.scheme, parsed_url.host,
+                    port=parsed_url.port, ask=True)
             op.set_username(user)
         elif flags & gio.ASK_PASSWORD_NEED_DOMAIN:
             #Don't know how common this case is, but anyway
             #a DOMAIN and a username prompt should be the
             #same so I will missuse the ui_factory get_username
             #a little bit here.
-            prompt = scheme.upper() + ' %(host)s DOMAIN'
+            prompt = (u'%s' % (parsed_url.scheme.upper(),) +
+                      u' %(host)s DOMAIN')
             domain = ui.ui_factory.get_username(prompt=prompt)
             op.set_domain(domain)
 
         if flags & gio.ASK_PASSWORD_NEED_PASSWORD:
             if user is None:
                 user = op.get_username()
-            password = auth.get_password(scheme, host,
-                    user, port=port)
+            password = auth.get_password(parsed_url.scheme, parsed_url.host,
+                    user, port=parsed_url.port)
             op.set_password(password)
         op.reply(gio.MOUNT_OPERATION_HANDLED)
 
@@ -207,7 +209,7 @@
 
     def _create_connection(self, credentials=None):
         if credentials is None:
-            user, password = self._user, self._password
+            user, password = self._parsed_url.user, self._parsed_url.password
         else:
             user, password = credentials
 
@@ -235,16 +237,20 @@
                                         " %s" % str(e), orig_error=e)
         return connection, (user, password)
 
+    def disconnect(self):
+        # FIXME: Nothing seems to be necessary here, which sounds a bit strange
+        # -- vila 20100601
+        pass
+
     def _reconnect(self):
+        # FIXME: This doesn't seem to be used -- vila 20100601
         """Create a new connection with the previously used credentials"""
         credentials = self._get_credentials()
         connection, credentials = self._create_connection(credentials)
         self._set_connection(connection, credentials)
 
     def _remote_path(self, relpath):
-        relative = urlutils.unescape(relpath).encode('utf-8')
-        remote_path = self._combine_paths(self._path, relative)
-        return remote_path
+        return self._parsed_url.clone(relpath).path
 
     def has(self, relpath):
         """Does the target location exist?"""
@@ -262,7 +268,7 @@
             else:
                 self._translate_gio_error(e, relpath)
 
-    def get(self, relpath, decode=False, retries=0):
+    def get(self, relpath, decode=DEPRECATED_PARAMETER, retries=0):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
@@ -272,6 +278,10 @@
         We're meant to return a file-like object which bzr will
         then read from. For now we do this via the magic of StringIO
         """
+        if deprecated_passed(decode):
+            warn(deprecated_in((2,3,0)) %
+                 '"decode" parameter to GioTransport.get()',
+                 DeprecationWarning, stacklevel=2)
         try:
             if 'gio' in debug.debug_flags:
                 mutter("GIO get: %s" % relpath)

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/__init__.py	2011-09-19 13:02:42 +0000
@@ -20,7 +20,6 @@
 """
 
 from cStringIO import StringIO
-import mimetools
 import re
 import urlparse
 import urllib
@@ -30,19 +29,14 @@
 from bzrlib import (
     debug,
     errors,
+    transport,
     ui,
     urlutils,
     )
 from bzrlib.smart import medium
-from bzrlib.symbol_versioning import (
-        deprecated_method,
-        )
 from bzrlib.trace import mutter
 from bzrlib.transport import (
     ConnectedTransport,
-    _CoalescedOffset,
-    get_transport,
-    Transport,
     )
 
 # TODO: This is not used anymore by HttpTransport_urllib
@@ -74,7 +68,7 @@
             password = urllib.unquote(password)
         else:
             password = ui.ui_factory.get_password(
-                prompt='HTTP %(user)s@%(host)s password',
+                prompt=u'HTTP %(user)s@%(host)s password',
                 user=username, host=host)
         password_manager.add_password(None, host, username, password)
     url = urlparse.urlunsplit((scheme, netloc, path, query, fragment))
@@ -148,17 +142,18 @@
 
         user and passwords are not embedded in the path provided to the server.
         """
-        relative = urlutils.unescape(relpath).encode('utf-8')
-        path = self._combine_paths(self._path, relative)
-        return self._unsplit_url(self._unqualified_scheme,
-                                 None, None, self._host, self._port, path)
+        url = self._parsed_url.clone(relpath)
+        url.user = url.quoted_user = None
+        url.password = url.quoted_password = None
+        url.scheme = self._unqualified_scheme
+        return str(url)
 
     def _create_auth(self):
         """Returns a dict containing the credentials provided at build time."""
-        auth = dict(host=self._host, port=self._port,
-                    user=self._user, password=self._password,
+        auth = dict(host=self._parsed_url.host, port=self._parsed_url.port,
+                    user=self._parsed_url.user, password=self._parsed_url.password,
                     protocol=self._unqualified_scheme,
-                    path=self._path)
+                    path=self._parsed_url.path)
         return auth
 
     def get_smart_medium(self):
@@ -271,7 +266,7 @@
                         cur_offset_and_size = iter_offsets.next()
 
             except (errors.ShortReadvError, errors.InvalidRange,
-                    errors.InvalidHttpRange), e:
+                    errors.InvalidHttpRange, errors.HttpBoundaryMissing), e:
                 mutter('Exception %r: %s during http._readv',e, e)
                 if (not isinstance(e, errors.ShortReadvError)
                     or retried_offset == cur_offset_and_size):
@@ -412,10 +407,9 @@
         """See bzrlib.transport.Transport.external_url."""
         # HTTP URL's are externally usable as long as they don't mention their
         # implementation qualifier
-        return self._unsplit_url(self._unqualified_scheme,
-                                 self._user, self._password,
-                                 self._host, self._port,
-                                 self._path)
+        url = self._parsed_url.clone()
+        url.scheme = self._unqualified_scheme
+        return str(url)
 
     def is_readonly(self):
         """See Transport.is_readonly."""
@@ -517,68 +511,57 @@
 
         :returns: A transport or None.
         """
-        def relpath(abspath):
-            """Returns the path relative to our base.
-
-            The constraints are weaker than the real relpath method because the
-            abspath is coming from the server and may slightly differ from our
-            base. We don't check the scheme, host, port, user, password parts,
-            relying on the caller to give us a proper url (i.e. one returned by
-            the server mirroring the one we sent).
-            """
-            (scheme,
-             user, password,
-             host, port,
-             path) = self._split_url(abspath)
-            pl = len(self._path)
-            return path[pl:].strip('/')
-
-        relpath = relpath(source)
-        if not target.endswith(relpath):
+        parsed_source = self._split_url(source)
+        parsed_target = self._split_url(target)
+        pl = len(self._parsed_url.path)
+        # determine the excess tail - the relative path that was in
+        # the original request but not part of this transports' URL.
+        excess_tail = parsed_source.path[pl:].strip("/")
+        if not target.endswith(excess_tail):
             # The final part of the url has been renamed, we can't handle the
             # redirection.
             return None
-        new_transport = None
-        (scheme,
-         user, password,
-         host, port,
-         path) = self._split_url(target)
-        # Recalculate base path. This is needed to ensure that when the
-        # redirected tranport will be used to re-try whatever request was
-        # redirected, we end up with the same url
-        base_path = path[:-len(relpath)]
-        if scheme in ('http', 'https'):
+
+        target_path = parsed_target.path
+        if excess_tail:
+            # Drop the tail that was in the redirect but not part of
+            # the path of this transport.
+            target_path = target_path[:-len(excess_tail)]
+
+        if parsed_target.scheme in ('http', 'https'):
             # Same protocol family (i.e. http[s]), we will preserve the same
             # http client implementation when a redirection occurs from one to
             # the other (otherwise users may be surprised that bzr switches
             # from one implementation to the other, and devs may suffer
             # debugging it).
-            if (scheme == self._unqualified_scheme
-                and host == self._host
-                and port == self._port
-                and (user is None or user == self._user)):
+            if (parsed_target.scheme == self._unqualified_scheme
+                and parsed_target.host == self._parsed_url.host
+                and parsed_target.port == self._parsed_url.port
+                and (parsed_target.user is None or
+                     parsed_target.user == self._parsed_url.user)):
                 # If a user is specified, it should match, we don't care about
                 # passwords, wrong passwords will be rejected anyway.
-                new_transport = self.clone(base_path)
+                return self.clone(target_path)
             else:
                 # Rebuild the url preserving the scheme qualification and the
                 # credentials (if they don't apply, the redirected to server
                 # will tell us, but if they do apply, we avoid prompting the
                 # user)
-                redir_scheme = scheme + '+' + self._impl_name
+                redir_scheme = parsed_target.scheme + '+' + self._impl_name
                 new_url = self._unsplit_url(redir_scheme,
-                                            self._user, self._password,
-                                            host, port,
-                                            base_path)
-                new_transport = get_transport(new_url)
+                    self._parsed_url.user,
+                    self._parsed_url.password,
+                    parsed_target.host, parsed_target.port,
+                    target_path)
+                return transport.get_transport_from_url(new_url)
         else:
             # Redirected to a different protocol
-            new_url = self._unsplit_url(scheme,
-                                        user, password,
-                                        host, port,
-                                        base_path)
-            new_transport = get_transport(new_url)
-        return new_transport
+            new_url = self._unsplit_url(parsed_target.scheme,
+                    parsed_target.user,
+                    parsed_target.password,
+                    parsed_target.host, parsed_target.port,
+                    target_path)
+            return transport.get_transport_from_url(new_url)
 
 
 # TODO: May be better located in smart/medium.py with the other
@@ -614,7 +597,7 @@
             t = self._http_transport_ref()
             code, body_filelike = t._post(bytes)
             if code != 200:
-                raise InvalidHttpResponse(
+                raise errors.InvalidHttpResponse(
                     t._remote_path('.bzr/smart'),
                     'Expected 200 response code, got %r' % (code,))
         except (errors.InvalidHttpResponse, errors.ConnectionReset), e:
@@ -629,6 +612,11 @@
         """
         pass
 
+    def disconnect(self):
+        """See SmartClientMedium.disconnect()."""
+        t = self._http_transport_ref()
+        t.disconnect()
+
 
 # TODO: May be better located in smart/medium.py with the other
 # SmartMediumRequest classes
@@ -661,3 +649,14 @@
     def _finished_reading(self):
         """See SmartClientMediumRequest._finished_reading."""
         pass
+
+
+def unhtml_roughly(maybe_html, length_limit=1000):
+    """Very approximate html->text translation, for presenting error bodies.
+
+    :param length_limit: Truncate the result to this many characters.
+
+    >>> unhtml_roughly("<b>bad</b> things happened\\n")
+    ' bad  things happened '
+    """
+    return re.subn(r"(<[^>]*>|\n|&nbsp;)", " ", maybe_html)[0][:length_limit]

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2010-05-30 16:22:32 +0000
+++ bzrlib/transport/http/_pycurl.py	2011-08-21 09:24:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,22 +34,23 @@
 from cStringIO import StringIO
 import httplib
 
+import bzrlib
 from bzrlib import (
     debug,
     errors,
+    trace,
     )
-import bzrlib
-from bzrlib.trace import mutter
 from bzrlib.transport.http import (
     ca_bundle,
     HttpTransportBase,
     response,
+    unhtml_roughly,
     )
 
 try:
     import pycurl
 except ImportError, e:
-    mutter("failed to import pycurl: %s", e)
+    trace.mutter("failed to import pycurl: %s", e)
     raise errors.DependencyNotPresent('pycurl', e)
 
 try:
@@ -64,7 +65,7 @@
     # reported by Alexander Belchenko, 2006-04-26
     pycurl.Curl()
 except pycurl.error, e:
-    mutter("failed to initialize pycurl: %s", e)
+    trace.mutter("failed to initialize pycurl: %s", e)
     raise errors.DependencyNotPresent('pycurl', e)
 
 
@@ -128,6 +129,11 @@
             self._set_connection(connection, auth)
         return connection
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def has(self, relpath):
         """See Transport.has()"""
         # We set NO BODY=0 in _get_full, so it should be safe
@@ -262,7 +268,9 @@
         # We override the Expect: header so that pycurl will send the POST
         # body immediately.
         try:
-            self._curl_perform(curl, header, ['Expect: '])
+            self._curl_perform(curl, header,
+                               ['Expect: ',
+                                'Content-Type: application/octet-stream'])
         except pycurl.error, e:
             if e[0] == CURLE_SEND_ERROR:
                 # When talking to an HTTP/1.0 server, getting a 400+ error code
@@ -274,8 +282,8 @@
                 # error code and the headers are known to be available, we just
                 # swallow the exception, leaving the upper levels handle the
                 # 400+ error.
-                mutter('got pycurl error in POST: %s, %s, %s, url: %s ',
-                       e[0], e[1], e, abspath)
+                trace.mutter('got pycurl error in POST: %s, %s, %s, url: %s ',
+                             e[0], e[1], e, abspath)
             else:
                 # Re-raise otherwise
                 raise
@@ -285,38 +293,67 @@
         return code, response.handle_response(abspath, code, msg, data)
 
 
-    def _raise_curl_http_error(self, curl, info=None):
+    def _raise_curl_http_error(self, curl, info=None, body=None):
+        """Common curl->bzrlib error translation.
+
+        Some methods may choose to override this for particular cases.
+
+        The URL and code are automatically included as appropriate.
+
+        :param info: Extra information to include in the message.
+
+        :param body: File-like object from which the body of the page can be
+            read.
+        """
         code = curl.getinfo(pycurl.HTTP_CODE)
         url = curl.getinfo(pycurl.EFFECTIVE_URL)
-        # Some error codes can be handled the same way for all
-        # requests
+        if body is not None:
+            response_body = body.read()
+            plaintext_body = unhtml_roughly(response_body)
+        else:
+            response_body = None
+            plaintext_body = ''
         if code == 403:
             raise errors.TransportError(
                 'Server refuses to fulfill the request (403 Forbidden)'
-                ' for %s' % url)
+                ' for %s: %s' % (url, plaintext_body))
         else:
             if info is None:
                 msg = ''
             else:
                 msg = ': ' + info
             raise errors.InvalidHttpResponse(
-                url, 'Unable to handle http code %d%s' % (code,msg))
+                url, 'Unable to handle http code %d%s: %s'
+                % (code, msg, plaintext_body))
 
     def _debug_cb(self, kind, text):
-        if kind in (pycurl.INFOTYPE_HEADER_IN, pycurl.INFOTYPE_DATA_IN,
-                    pycurl.INFOTYPE_SSL_DATA_IN):
+        if kind in (pycurl.INFOTYPE_HEADER_IN, pycurl.INFOTYPE_DATA_IN):
             self._report_activity(len(text), 'read')
             if (kind == pycurl.INFOTYPE_HEADER_IN
                 and 'http' in debug.debug_flags):
-                mutter('< %s' % text)
-        elif kind in (pycurl.INFOTYPE_HEADER_OUT, pycurl.INFOTYPE_DATA_OUT,
-                      pycurl.INFOTYPE_SSL_DATA_OUT):
+                trace.mutter('< %s' % (text.rstrip(),))
+        elif kind in (pycurl.INFOTYPE_HEADER_OUT, pycurl.INFOTYPE_DATA_OUT):
             self._report_activity(len(text), 'write')
             if (kind == pycurl.INFOTYPE_HEADER_OUT
                 and 'http' in debug.debug_flags):
-                mutter('> %s' % text)
+                lines = []
+                for line in text.rstrip().splitlines():
+                    # People are often told to paste -Dhttp output to help
+                    # debug. Don't compromise credentials.
+                    try:
+                        header, details = line.split(':', 1)
+                    except ValueError:
+                        header = None
+                    if header in ('Authorization', 'Proxy-Authorization'):
+                        line = '%s: <masked>' % (header,)
+                    lines.append(line)
+                trace.mutter('> ' + '\n> '.join(lines))
         elif kind == pycurl.INFOTYPE_TEXT and 'http' in debug.debug_flags:
-            mutter('* %s' % text)
+            trace.mutter('* %s' % text.rstrip())
+        elif (kind in (pycurl.INFOTYPE_TEXT, pycurl.INFOTYPE_SSL_DATA_IN,
+                       pycurl.INFOTYPE_SSL_DATA_OUT)
+              and 'http' in debug.debug_flags):
+            trace.mutter('* %s' % text)
 
     def _set_curl_options(self, curl):
         """Set options for all requests"""
@@ -353,8 +390,8 @@
             curl.perform()
         except pycurl.error, e:
             url = curl.getinfo(pycurl.EFFECTIVE_URL)
-            mutter('got pycurl error: %s, %s, %s, url: %s ',
-                    e[0], e[1], e, url)
+            trace.mutter('got pycurl error: %s, %s, %s, url: %s ',
+                         e[0], e[1], e, url)
             if e[0] in (CURLE_COULDNT_RESOLVE_HOST,
                         CURLE_COULDNT_RESOLVE_PROXY,
                         CURLE_COULDNT_CONNECT,
@@ -388,10 +425,10 @@
 
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    from bzrlib import tests
+    from bzrlib.tests import features
     from bzrlib.tests import http_server
     permutations = [(PyCurlTransport, http_server.HttpServer_PyCurl),]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         from bzrlib.tests import (
             https_server,
             ssl_certs,

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/_urllib.py	2011-09-03 16:30:53 +0000
@@ -14,14 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
-import urllib
-import urlparse
-
 from bzrlib import (
     errors,
     trace,
-    urlutils,
     )
 from bzrlib.transport import http
 # TODO: handle_response should be integrated into the http/__init__.py
@@ -99,6 +94,11 @@
 
         return response
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get(self, relpath, offsets, tail_amount=0):
         """See HttpTransport._get"""
         abspath = self._remote_path(relpath)
@@ -133,8 +133,10 @@
         abspath = self._remote_path('.bzr/smart')
         # We include 403 in accepted_errors so that send_http_smart_request can
         # handle a 403.  Otherwise a 403 causes an unhandled TransportError.
-        response = self._perform(Request('POST', abspath, body_bytes,
-                                         accepted_errors=[200, 403]))
+        response = self._perform(
+            Request('POST', abspath, body_bytes,
+                    {'Content-Type': 'application/octet-stream'},
+                    accepted_errors=[200, 403]))
         code = response.code
         data = handle_response(abspath, code, response.info(), response)
         return code, data
@@ -165,10 +167,12 @@
 
 def get_test_permutations():
     """Return the permutations to be used in testing."""
-    from bzrlib import tests
-    from bzrlib.tests import http_server
+    from bzrlib.tests import (
+        features,
+        http_server,
+        )
     permutations = [(HttpTransport_urllib, http_server.HttpServer_urllib),]
-    if tests.HTTPSServerFeature.available():
+    if features.HTTPSServerFeature.available():
         from bzrlib.tests import https_server
         permutations.append((HttpTransport_urllib,
                              https_server.HTTPSServer_urllib))

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2010-05-08 05:56:28 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2011-09-03 16:30:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -70,11 +70,30 @@
     osutils,
     trace,
     transport,
-    ui,
     urlutils,
     )
 
 
+class addinfourl(urllib2.addinfourl):
+    '''Replacement addinfourl class compatible with python-2.7's xmlrpclib
+
+    In python-2.7, xmlrpclib expects that the response object that it receives
+    has a getheader method.  httplib.HTTPResponse provides this but
+    urllib2.addinfourl does not.  Add the necessary functions here, ported to
+    use the internal data structures of addinfourl.
+    '''
+
+    def getheader(self, name, default=None):
+        if self.headers is None:
+            raise httplib.ResponseNotReady()
+        return self.headers.getheader(name, default)
+
+    def getheaders(self):
+        if self.headers is None:
+            raise httplib.ResponseNotReady()
+        return self.headers.items()
+
+
 class _ReportingFileSocket(object):
 
     def __init__(self, filesock, report_activity=None):
@@ -90,13 +109,8 @@
         self.report_activity(len(s), 'read')
         return s
 
-    def readline(self):
-        # This should be readline(self, size=-1), but httplib in python 2.4 and
-        #  2.5 defines a SSLFile wrapper whose readline method lacks the size
-        #  parameter.  So until we drop support for 2.4 and 2.5 and since we
-        #  don't *need* the size parameter we'll stay with readline(self)
-        #  --  vila 20090209
-        s = self.filesock.readline()
+    def readline(self, size=-1):
+        s = self.filesock.readline(size)
         self.report_activity(len(s), 'read')
         return s
 
@@ -145,7 +159,7 @@
     """
 
     # Some responses have bodies in which we have no interest
-    _body_ignored_responses = [301,302, 303, 307, 401, 403, 404]
+    _body_ignored_responses = [301,302, 303, 307, 400, 401, 403, 404, 501]
 
     # in finish() below, we may have to discard several MB in the worst
     # case. To avoid buffering that much, we read and discard by chunks
@@ -246,17 +260,27 @@
     def cleanup_pipe(self):
         """Read the remaining bytes of the last response if any."""
         if self._response is not None:
-            pending = self._response.finish()
-            # Warn the user (once)
-            if (self._ranges_received_whole_file is None
-                and self._response.status == 200
-                and pending and pending > self._range_warning_thresold
-                ):
-                self._ranges_received_whole_file = True
-                trace.warning(
-                    'Got a 200 response when asking for multiple ranges,'
-                    ' does your server at %s:%s support range requests?',
-                    self.host, self.port)
+            try:
+                pending = self._response.finish()
+                # Warn the user (once)
+                if (self._ranges_received_whole_file is None
+                    and self._response.status == 200
+                    and pending and pending > self._range_warning_thresold
+                    ):
+                    self._ranges_received_whole_file = True
+                    trace.warning(
+                        'Got a 200 response when asking for multiple ranges,'
+                        ' does your server at %s:%s support range requests?',
+                        self.host, self.port)
+            except socket.error, e:
+                # It's conceivable that the socket is in a bad state here
+                # (including some test cases) and in this case, it doesn't need
+                # cleaning anymore, so no need to fail, we just get rid of the
+                # socket and let callers reconnect
+                if (len(e.args) == 0
+                    or e.args[0] not in (errno.ECONNRESET, errno.ECONNABORTED)):
+                    raise
+                self.close()
             self._response = None
         # Preserve our preciousss
         sock = self.sock
@@ -564,7 +588,9 @@
                         'Bad status line received',
                         orig_error=exc_val)
                 elif (isinstance(exc_val, socket.error) and len(exc_val.args)
-                      and exc_val.args[0] in (errno.ECONNRESET, 10054)):
+                      and exc_val.args[0] in (errno.ECONNRESET, 10053, 10054)):
+                      # 10053 == WSAECONNABORTED
+                      # 10054 == WSAECONNRESET
                     raise errors.ConnectionReset(
                         "Connection lost while sending request.")
                 else:
@@ -621,7 +647,13 @@
                                      headers)
             if 'http' in debug.debug_flags:
                 trace.mutter('> %s %s' % (method, url))
-                hdrs = ['%s: %s' % (k, v) for k,v in headers.items()]
+                hdrs = []
+                for k,v in headers.iteritems():
+                    # People are often told to paste -Dhttp output to help
+                    # debug. Don't compromise credentials.
+                    if k in ('Authorization', 'Proxy-Authorization'):
+                        v = '<masked>'
+                    hdrs.append('%s: %s' % (k, v))
                 trace.mutter('> ' + '\n> '.join(hdrs) + '\n')
             if self._debuglevel >= 1:
                 print 'Request sent: [%r] from (%s)' \
@@ -656,7 +688,7 @@
             r = response
             r.recv = r.read
             fp = socket._fileobject(r, bufsize=65536)
-            resp = urllib2.addinfourl(fp, r.msg, req.get_full_url())
+            resp = addinfourl(fp, r.msg, req.get_full_url())
             resp.code = r.status
             resp.msg = r.reason
             resp.version = r.version
@@ -906,9 +938,31 @@
         return None
 
     def proxy_bypass(self, host):
-        """Check if host should be proxied or not"""
+        """Check if host should be proxied or not.
+
+        :returns: True to skip the proxy, False otherwise.
+        """
         no_proxy = self.get_proxy_env_var('no', default_to=None)
+        bypass = self.evaluate_proxy_bypass(host, no_proxy)
+        if bypass is None:
+            # Nevertheless, there are platform-specific ways to
+            # ignore proxies...
+            return urllib.proxy_bypass(host)
+        else:
+            return bypass
+
+    def evaluate_proxy_bypass(self, host, no_proxy):
+        """Check the host against a comma-separated no_proxy list as a string.
+
+        :param host: ``host:port`` being requested
+
+        :param no_proxy: comma-separated list of hosts to access directly.
+
+        :returns: True to skip the proxy, False not to, or None to
+            leave it to urllib.
+        """
         if no_proxy is None:
+            # All hosts are proxied
             return False
         hhost, hport = urllib.splitport(host)
         # Does host match any of the domains mentioned in
@@ -916,6 +970,9 @@
         # are fuzzy (to say the least). We try to allow most
         # commonly seen values.
         for domain in no_proxy.split(','):
+            domain = domain.strip()
+            if domain == '':
+                continue
             dhost, dport = urllib.splitport(domain)
             if hport == dport or dport is None:
                 # Protect glob chars
@@ -924,9 +981,8 @@
                 dhost = dhost.replace("?", r".")
                 if re.match(dhost, hhost, re.IGNORECASE):
                     return True
-        # Nevertheless, there are platform-specific ways to
-        # ignore proxies...
-        return urllib.proxy_bypass(host)
+        # Nothing explicitly avoid the host
+        return None
 
     def set_proxy(self, request, type):
         if self.proxy_bypass(request.get_host()):
@@ -939,9 +995,8 @@
         # grok user:password@host:port as well as
         # http://user:password@host:port
 
-        (scheme, user, password,
-         host, port, path) = transport.ConnectedTransport._split_url(proxy)
-        if not host:
+        parsed_url = transport.ConnectedTransport._split_url(proxy)
+        if not parsed_url.host:
             raise errors.InvalidURL(proxy, 'No host component')
 
         if request.proxy_auth == {}:
@@ -949,15 +1004,17 @@
             # proxied request, intialize.  scheme (the authentication scheme)
             # and realm will be set by the AuthHandler
             request.proxy_auth = {
-                                  'host': host, 'port': port,
-                                  'user': user, 'password': password,
-                                  'protocol': scheme,
+                                  'host': parsed_url.host,
+                                  'port': parsed_url.port,
+                                  'user': parsed_url.user,
+                                  'password': parsed_url.password,
+                                  'protocol': parsed_url.scheme,
                                    # We ignore path since we connect to a proxy
                                   'path': None}
-        if port is None:
-            phost = host
+        if parsed_url.port is None:
+            phost = parsed_url.host
         else:
-            phost = host + ':%d' % port
+            phost = parsed_url.host + ':%d' % parsed_url.port
         request.set_proxy(phost, type)
         if self._debuglevel >= 3:
             print 'set_proxy: proxy set to %s://%s' % (type, phost)
@@ -1072,12 +1129,11 @@
         auth['modified'] = False
         # Put some common info in auth if the caller didn't
         if auth.get('path', None) is None:
-            (protocol, _, _,
-             host, port, path) = urlutils.parse_url(request.get_full_url())
-            self.update_auth(auth, 'protocol', protocol)
-            self.update_auth(auth, 'host', host)
-            self.update_auth(auth, 'port', port)
-            self.update_auth(auth, 'path', path)
+            parsed_url = urlutils.URL.from_string(request.get_full_url())
+            self.update_auth(auth, 'protocol', parsed_url.scheme)
+            self.update_auth(auth, 'host', parsed_url.host)
+            self.update_auth(auth, 'port', parsed_url.port)
+            self.update_auth(auth, 'path', parsed_url.path)
         # FIXME: the auth handler should be selected at a single place instead
         # of letting all handlers try to match all headers, but the current
         # design doesn't allow a simple implementation.
@@ -1182,15 +1238,17 @@
         user = auth.get('user', None)
         password = auth.get('password', None)
         realm = auth['realm']
+        port = auth.get('port', None)
 
         if user is None:
             user = auth_conf.get_user(auth['protocol'], auth['host'],
-                                      port=auth['port'], path=auth['path'],
+                                      port=port, path=auth['path'],
                                       realm=realm, ask=True,
                                       prompt=self.build_username_prompt(auth))
         if user is not None and password is None:
             password = auth_conf.get_password(
-                auth['protocol'], auth['host'], user, port=auth['port'],
+                auth['protocol'], auth['host'], user,
+                port=port,
                 path=auth['path'], realm=realm,
                 prompt=self.build_password_prompt(auth))
 
@@ -1207,11 +1265,11 @@
         user. The daughter classes should implements a public
         build_password_prompt using this method.
         """
-        prompt = '%s' % auth['protocol'].upper() + ' %(user)s@%(host)s'
+        prompt = u'%s' % auth['protocol'].upper() + u' %(user)s@%(host)s'
         realm = auth['realm']
         if realm is not None:
-            prompt += ", Realm: '%s'" % realm
-        prompt += ' password'
+            prompt += u", Realm: '%s'" % realm.decode('utf8')
+        prompt += u' password'
         return prompt
 
     def _build_username_prompt(self, auth):
@@ -1225,11 +1283,11 @@
         user. The daughter classes should implements a public
         build_username_prompt using this method.
         """
-        prompt = '%s' % auth['protocol'].upper() + ' %(host)s'
+        prompt = u'%s' % auth['protocol'].upper() + u' %(host)s'
         realm = auth['realm']
         if realm is not None:
-            prompt += ", Realm: '%s'" % realm
-        prompt += ' username'
+            prompt += u", Realm: '%s'" % realm.decode('utf8')
+        prompt += u' username'
         return prompt
 
     def http_request(self, request):
@@ -1340,7 +1398,7 @@
     if algorithm == 'MD5':
         H = lambda x: osutils.md5(x).hexdigest()
     elif algorithm == 'SHA':
-        H = lambda x: osutils.sha(x).hexdigest()
+        H = osutils.sha_string
     if H is not None:
         KD = lambda secret, data: H("%s:%s" % (secret, data))
     return H, KD
@@ -1349,7 +1407,7 @@
 def get_new_cnonce(nonce, nonce_count):
     raw = '%s:%d:%s:%s' % (nonce, nonce_count, time.ctime(),
                            urllib2.randombytes(8))
-    return osutils.sha(raw).hexdigest()[:16]
+    return osutils.sha_string(raw)[:16]
 
 
 class DigestAuthHandler(AbstractAuthHandler):
@@ -1499,12 +1557,12 @@
 
     def build_password_prompt(self, auth):
         prompt = self._build_password_prompt(auth)
-        prompt = 'Proxy ' + prompt
+        prompt = u'Proxy ' + prompt
         return prompt
 
     def build_username_prompt(self, auth):
         prompt = self._build_username_prompt(auth)
-        prompt = 'Proxy ' + prompt
+        prompt = u'Proxy ' + prompt
         return prompt
 
     def http_error_407(self, req, fp, code, msg, headers):

=== modified file 'bzrlib/transport/http/response.py'
--- bzrlib/transport/http/response.py	2009-03-23 14:59:43 +0000
+++ bzrlib/transport/http/response.py	2011-09-03 16:30:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,6 @@
 
 from bzrlib import (
     errors,
-    trace,
     osutils,
     )
 
@@ -109,6 +108,13 @@
             # To be on the safe side we allow it before any boundary line
             boundary_line = self._file.readline()
 
+        if boundary_line == '':
+            # A timeout in the proxy server caused the response to end early.
+            # See launchpad bug 198646.
+            raise errors.HttpBoundaryMissing(
+                self._path,
+                self._boundary)
+
         if boundary_line != '--' + self._boundary + '\r\n':
             # rfc822.unquote() incorrectly unquotes strings enclosed in <>
             # IIS 6 and 7 incorrectly wrap boundary strings in <>

=== modified file 'bzrlib/transport/http/wsgi.py'
--- bzrlib/transport/http/wsgi.py	2010-02-17 17:11:16 +0000
+++ bzrlib/transport/http/wsgi.py	2011-09-03 16:30:53 +0000
@@ -22,7 +22,7 @@
 
 from cStringIO import StringIO
 
-from bzrlib.smart import protocol, medium
+from bzrlib.smart import medium
 from bzrlib.transport import chroot, get_transport
 from bzrlib.urlutils import local_path_to_url
 
@@ -101,7 +101,7 @@
             backing_transport.  This is used to interpret relpaths received from
             the client.
         """
-        # Use a ChrootTransportDecorator so that this web application won't
+        # Use a ChrootServer so that this web application won't
         # accidentally let people access locations they shouldn't.
         # e.g. consider a smart server request for "get /etc/passwd" or
         # something.

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2010-07-19 20:09:53 +0000
+++ bzrlib/transport/local.py	2011-10-06 07:43:13 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 """
 
 import os
-from stat import ST_MODE, S_ISDIR, ST_SIZE, S_IMODE
+from stat import ST_MODE, S_ISDIR, S_IMODE
 import sys
 
 from bzrlib.lazy_import import lazy_import
@@ -33,9 +33,7 @@
     osutils,
     urlutils,
     symbol_versioning,
-    transport,
     )
-from bzrlib.trace import mutter
 from bzrlib.transport import LateReadError
 """)
 
@@ -74,6 +72,8 @@
 
         super(LocalTransport, self).__init__(base)
         self._local_base = urlutils.local_path_from_url(base)
+        if self._local_base[-1] != '/':
+            self._local_base = self._local_base + '/'
 
     def clone(self, offset=None):
         """Return a new LocalTransport with root at self.base + offset
@@ -329,10 +329,12 @@
 
     def open_write_stream(self, relpath, mode=None):
         """See Transport.open_write_stream."""
-        # initialise the file
-        self.put_bytes_non_atomic(relpath, "", mode=mode)
         abspath = self._abspath(relpath)
-        handle = osutils.open_file(abspath, 'wb')
+        try:
+            handle = osutils.open_file(abspath, 'wb')
+        except (IOError, OSError),e:
+            self._translate_error(e, abspath)
+        handle.truncate()
         if mode is not None:
             self._check_mode_and_size(abspath, handle.fileno(), mode)
         transport._file_streams[self.abspath(relpath)] = handle
@@ -565,7 +567,7 @@
         self._local_base = urlutils._win32_local_path_from_url(base)
 
     def abspath(self, relpath):
-        path = osutils.normpath(osutils.pathjoin(
+        path = osutils._win32_normpath(osutils.pathjoin(
                     self._local_base, urlutils.unescape(relpath)))
         return urlutils._win32_local_path_to_url(path)
 

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2010-02-23 07:43:11 +0000
+++ bzrlib/transport/memory.py	2011-08-11 14:08:05 +0000
@@ -81,7 +81,7 @@
 
     def clone(self, offset=None):
         """See Transport.clone()."""
-        path = self._combine_paths(self._cwd, offset)
+        path = urlutils.URL._combine_paths(self._cwd, offset)
         if len(path) == 0 or path[-1] != '/':
             path += '/'
         url = self._scheme + path
@@ -289,12 +289,6 @@
             raise LockError('File %r already locked' % (self.path,))
         self.transport._locks[self.path] = self
 
-    def __del__(self):
-        # Should this warn, or actually try to cleanup?
-        if self.transport:
-            warnings.warn("MemoryLock %r not explicitly unlocked" % (self.path,))
-            self.unlock()
-
     def unlock(self):
         del self.transport._locks[self.path]
         self.transport = None

=== modified file 'bzrlib/transport/pathfilter.py'
--- bzrlib/transport/pathfilter.py	2010-02-23 07:43:11 +0000
+++ bzrlib/transport/pathfilter.py	2011-08-11 14:08:05 +0000
@@ -16,9 +16,9 @@
 
 """A transport decorator that filters all paths that are passed to it."""
 
+from bzrlib import urlutils
 
 from bzrlib.transport import (
-    get_transport,
     register_transport,
     Server,
     Transport,
@@ -82,7 +82,7 @@
         self.scheme = self.server.scheme
 
     def _relpath_from_server_root(self, relpath):
-        unfiltered_path = self._combine_paths(self.base_path, relpath)
+        unfiltered_path = urlutils.URL._combine_paths(self.base_path, relpath)
         if not unfiltered_path.startswith('/'):
             raise ValueError(unfiltered_path)
         return unfiltered_path[1:]

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2010-06-17 09:23:19 +0000
+++ bzrlib/transport/remote.py	2011-08-19 22:34:02 +0000
@@ -170,7 +170,7 @@
 
     def _remote_path(self, relpath):
         """Returns the Unicode version of the absolute path for relpath."""
-        return self._combine_paths(self._path, relpath)
+        return urlutils.URL._combine_paths(self._parsed_url.path, relpath)
 
     def _call(self, method, *args):
         resp = self._call2(method, *args)
@@ -435,7 +435,9 @@
         remote._translate_error(err, path=relpath)
 
     def disconnect(self):
-        self.get_smart_medium().disconnect()
+        m = self.get_smart_medium()
+        if m is not None:
+            m.disconnect()
 
     def stat(self, relpath):
         resp = self._call2('stat', self._remote_path(relpath))
@@ -481,7 +483,7 @@
 
     def _build_medium(self):
         client_medium = medium.SmartTCPClientMedium(
-            self._host, self._port, self.base)
+            self._parsed_url.host, self._parsed_url.port, self.base)
         return client_medium, None
 
 
@@ -494,7 +496,7 @@
 
     def _build_medium(self):
         client_medium = medium.SmartTCPClientMedium(
-            self._host, self._port, self.base)
+            self._parsed_url.host, self._parsed_url.port, self.base)
         client_medium._protocol_version = 2
         client_medium._remember_remote_is_before((1, 6))
         return client_medium, None
@@ -510,14 +512,16 @@
     def _build_medium(self):
         location_config = config.LocationConfig(self.base)
         bzr_remote_path = location_config.get_bzr_remote_path()
-        user = self._user
+        user = self._parsed_url.user
         if user is None:
             auth = config.AuthenticationConfig()
-            user = auth.get_user('ssh', self._host, self._port)
-        ssh_params = medium.SSHParams(self._host, self._port, user,
-            self._password, bzr_remote_path)
+            user = auth.get_user('ssh', self._parsed_url.host,
+                self._parsed_url.port)
+        ssh_params = medium.SSHParams(self._parsed_url.host,
+                self._parsed_url.port, user, self._parsed_url.password,
+                bzr_remote_path)
         client_medium = medium.SmartSSHClientMedium(self.base, ssh_params)
-        return client_medium, (user, self._password)
+        return client_medium, (user, self._parsed_url.password)
 
 
 class RemoteHTTPTransport(RemoteTransport):
@@ -537,7 +541,7 @@
             # url only for an intial construction (when the url came from the
             # command-line).
             http_url = base[len('bzr+'):]
-            self._http_transport = transport.get_transport(http_url)
+            self._http_transport = transport.get_transport_from_url(http_url)
         else:
             self._http_transport = http_transport
         super(RemoteHTTPTransport, self).__init__(

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2010-07-19 10:55:15 +0000
+++ bzrlib/transport/sftp.py	2011-08-11 16:24:09 +0000
@@ -31,8 +31,6 @@
 import stat
 import sys
 import time
-import urllib
-import urlparse
 import warnings
 
 from bzrlib import (
@@ -42,22 +40,17 @@
     urlutils,
     )
 from bzrlib.errors import (FileExists,
-                           NoSuchFile, PathNotChild,
+                           NoSuchFile,
                            TransportError,
                            LockError,
                            PathError,
                            ParamikoNotPresent,
                            )
-from bzrlib.osutils import pathjoin, fancy_rename, getcwd
-from bzrlib.symbol_versioning import (
-        deprecated_function,
-        )
+from bzrlib.osutils import fancy_rename
 from bzrlib.trace import mutter, warning
 from bzrlib.transport import (
     FileFileStream,
     _file_streams,
-    local,
-    Server,
     ssh,
     ConnectedTransport,
     )
@@ -114,12 +107,6 @@
         except FileExists:
             raise LockError('File %r already locked' % (self.path,))
 
-    def __del__(self):
-        """Should this warn, or actually try to cleanup?"""
-        if self.lock_file:
-            warning("SFTPLock %r not explicitly unlocked" % (self.path,))
-            self.unlock()
-
     def unlock(self):
         if not self.lock_file:
             return
@@ -281,6 +268,8 @@
                     buffered = buffered[buffered_offset:]
                     buffered_data = [buffered]
                     buffered_len = len(buffered)
+        # now that the data stream is done, close the handle
+        fp.close()
         if buffered_len:
             buffered = ''.join(buffered_data)
             del buffered_data[:]
@@ -341,17 +330,12 @@
     # up the request itself, rather than us having to worry about it
     _max_request_size = 32768
 
-    def __init__(self, base, _from_transport=None):
-        super(SFTPTransport, self).__init__(base,
-                                            _from_transport=_from_transport)
-
     def _remote_path(self, relpath):
         """Return the path to be passed along the sftp protocol for relpath.
 
         :param relpath: is a urlencoded string.
         """
-        relative = urlutils.unescape(relpath).encode('utf-8')
-        remote_path = self._combine_paths(self._path, relative)
+        remote_path = self._parsed_url.clone(relpath).path
         # the initial slash should be removed from the path, and treated as a
         # homedir relative path (the path begins with a double slash if it is
         # absolute).  see draft-ietf-secsh-scp-sftp-ssh-uri-03.txt
@@ -376,19 +360,25 @@
         in base url at transport creation time.
         """
         if credentials is None:
-            password = self._password
+            password = self._parsed_url.password
         else:
             password = credentials
 
         vendor = ssh._get_ssh_vendor()
-        user = self._user
+        user = self._parsed_url.user
         if user is None:
             auth = config.AuthenticationConfig()
-            user = auth.get_user('ssh', self._host, self._port)
-        connection = vendor.connect_sftp(self._user, password,
-                                         self._host, self._port)
+            user = auth.get_user('ssh', self._parsed_url.host,
+                self._parsed_url.port)
+        connection = vendor.connect_sftp(self._parsed_url.user, password,
+            self._parsed_url.host, self._parsed_url.port)
         return connection, (user, password)
 
+    def disconnect(self):
+        connection = self._get_connection()
+        if connection is not None:
+            connection.close()
+
     def _get_sftp(self):
         """Ensures that a connection is established"""
         connection = self._get_connection()
@@ -416,11 +406,6 @@
         :param relpath: The relative path to the file
         """
         try:
-            # FIXME: by returning the file directly, we don't pass this
-            # through to report_activity.  We could try wrapping the object
-            # before it's returned.  For readv and get_bytes it's handled in
-            # the higher-level function.
-            # -- mbp 20090126
             path = self._remote_path(relpath)
             f = self._get_sftp().file(path, mode='rb')
             if self._do_prefetch and (getattr(f, 'prefetch', None) is not None):

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2010-06-17 13:16:51 +0000
+++ bzrlib/transport/ssh.py	2011-05-16 13:39:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Robey Pointer <robey@lag.net>
+# Copyright (C) 2006-2011 Robey Pointer <robey@lag.net>
 # Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
@@ -126,12 +126,15 @@
         elif 'SSH Secure Shell' in version:
             trace.mutter('ssh implementation is SSH Corp.')
             vendor = SSHCorpSubprocessVendor()
+        elif 'lsh' in version:
+            trace.mutter('ssh implementation is GNU lsh.')
+            vendor = LSHSubprocessVendor()
         # As plink user prompts are not handled currently, don't auto-detect
         # it by inspection below, but keep this vendor detection for if a path
         # is given in BZR_SSH. See https://bugs.launchpad.net/bugs/414743
         elif 'plink' in version and progname == 'plink':
             # Checking if "plink" was the executed argument as Windows
-            # sometimes reports 'ssh -V' incorrectly with 'plink' in it's
+            # sometimes reports 'ssh -V' incorrectly with 'plink' in its
             # version.  See https://bugs.launchpad.net/bzr/+bug/107155
             trace.mutter("ssh implementation is Putty's plink.")
             vendor = PLinkSubprocessVendor()
@@ -336,15 +339,14 @@
             self._raise_connection_error(host, port=port, orig_error=e,
                                          msg='Unable to invoke remote bzr')
 
+_ssh_connection_errors = (EOFError, OSError, IOError, socket.error)
 if paramiko is not None:
     vendor = ParamikoVendor()
     register_ssh_vendor('paramiko', vendor)
     register_ssh_vendor('none', vendor)
     register_default_ssh_vendor(vendor)
-    _sftp_connection_errors = (EOFError, paramiko.SSHException)
+    _ssh_connection_errors += (paramiko.SSHException,)
     del vendor
-else:
-    _sftp_connection_errors = (EOFError,)
 
 
 class SubprocessVendor(SSHVendor):
@@ -357,17 +359,19 @@
         # whatever) chunks.
         try:
             my_sock, subproc_sock = socket.socketpair()
+            osutils.set_fd_cloexec(my_sock)
         except (AttributeError, socket.error):
             # This platform doesn't support socketpair(), so just use ordinary
             # pipes instead.
             stdin = stdout = subprocess.PIPE
-            sock = None
+            my_sock, subproc_sock = None, None
         else:
             stdin = stdout = subproc_sock
-            sock = my_sock
         proc = subprocess.Popen(argv, stdin=stdin, stdout=stdout,
                                 **os_specific_subprocess_params())
-        return SSHSubprocessConnection(proc, sock=sock)
+        if subproc_sock is not None:
+            subproc_sock.close()
+        return SSHSubprocessConnection(proc, sock=my_sock)
 
     def connect_sftp(self, username, password, host, port):
         try:
@@ -375,14 +379,7 @@
                                                   subsystem='sftp')
             sock = self._connect(argv)
             return SFTPClient(SocketAsChannelAdapter(sock))
-        except _sftp_connection_errors, e:
-            self._raise_connection_error(host, port=port, orig_error=e)
-        except (OSError, IOError), e:
-            # If the machine is fast enough, ssh can actually exit
-            # before we try and send it the sftp request, which
-            # raises a Broken Pipe
-            if e.errno not in (errno.EPIPE,):
-                raise
+        except _ssh_connection_errors, e:
             self._raise_connection_error(host, port=port, orig_error=e)
 
     def connect_ssh(self, username, password, host, port, command):
@@ -390,14 +387,7 @@
             argv = self._get_vendor_specific_argv(username, host, port,
                                                   command=command)
             return self._connect(argv)
-        except (EOFError), e:
-            self._raise_connection_error(host, port=port, orig_error=e)
-        except (OSError, IOError), e:
-            # If the machine is fast enough, ssh can actually exit
-            # before we try and send it the sftp request, which
-            # raises a Broken Pipe
-            if e.errno not in (errno.EPIPE,):
-                raise
+        except _ssh_connection_errors, e:
             self._raise_connection_error(host, port=port, orig_error=e)
 
     def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
@@ -418,7 +408,7 @@
                                   command=None):
         args = [self.executable_path,
                 '-oForwardX11=no', '-oForwardAgent=no',
-                '-oClearAllForwardings=yes', '-oProtocol=2',
+                '-oClearAllForwardings=yes',
                 '-oNoHostAuthenticationForLocalhost=yes']
         if port is not None:
             args.extend(['-p', str(port)])
@@ -454,6 +444,27 @@
 register_ssh_vendor('sshcorp', SSHCorpSubprocessVendor())
 
 
+class LSHSubprocessVendor(SubprocessVendor):
+    """SSH vendor that uses the 'lsh' executable from GNU"""
+
+    executable_path = 'lsh'
+
+    def _get_vendor_specific_argv(self, username, host, port, subsystem=None,
+                                  command=None):
+        args = [self.executable_path]
+        if port is not None:
+            args.extend(['-p', str(port)])
+        if username is not None:
+            args.extend(['-l', username])
+        if subsystem is not None:
+            args.extend(['--subsystem', subsystem, host])
+        else:
+            args.extend([host] + command)
+        return args
+
+register_ssh_vendor('lsh', LSHSubprocessVendor())
+
+
 class PLinkSubprocessVendor(SubprocessVendor):
     """SSH vendor that uses the 'plink' executable from Putty."""
 
@@ -562,7 +573,8 @@
         return True
     except paramiko.PasswordRequiredException:
         password = ui.ui_factory.get_password(
-            prompt='SSH %(filename)s password', filename=filename)
+            prompt=u'SSH %(filename)s password',
+            filename=filename.decode(osutils._fs_enc))
         try:
             key = pkey_class.from_private_key_file(filename, password)
             paramiko_transport.auth_publickey(username, key)
@@ -644,12 +656,28 @@
 import weakref
 _subproc_weakrefs = set()
 
-def _close_ssh_proc(proc):
-    for func in [proc.stdin.close, proc.stdout.close, proc.wait]:
+def _close_ssh_proc(proc, sock):
+    """Carefully close stdin/stdout and reap the SSH process.
+
+    If the pipes are already closed and/or the process has already been
+    wait()ed on, that's ok, and no error is raised.  The goal is to do our best
+    to clean up (whether or not a clean up was already tried).
+    """
+    funcs = []
+    for closeable in (proc.stdin, proc.stdout, sock):
+        # We expect that either proc (a subprocess.Popen) will have stdin and
+        # stdout streams to close, or that we will have been passed a socket to
+        # close, with the option not in use being None.
+        if closeable is not None:
+            funcs.append(closeable.close)
+    funcs.append(proc.wait)
+    for func in funcs:
         try:
             func()
         except OSError:
-            pass
+            # It's ok for the pipe to already be closed, or the process to
+            # already be finished.
+            continue
 
 
 class SSHConnection(object):
@@ -690,7 +718,7 @@
         # to avoid leaving processes lingering indefinitely.
         def terminate(ref):
             _subproc_weakrefs.remove(ref)
-            _close_ssh_proc(proc)
+            _close_ssh_proc(proc, sock)
         _subproc_weakrefs.add(weakref.ref(self, terminate))
 
     def send(self, data):
@@ -706,7 +734,7 @@
             return os.read(self.proc.stdout.fileno(), count)
 
     def close(self):
-        _close_ssh_proc(self.proc)
+        _close_ssh_proc(self.proc, self._sock)
 
     def get_sock_or_pipes(self):
         if self._sock is not None:

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2010-05-25 17:27:52 +0000
+++ bzrlib/tree.py	2011-09-15 11:31:47 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,26 +18,32 @@
 """
 
 import os
-from collections import deque
-
-import bzrlib
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import collections
+
 from bzrlib import (
     conflicts as _mod_conflicts,
     debug,
     delta,
+    errors,
     filters,
+    inventory,
     osutils,
     revision as _mod_revision,
     rules,
+    trace,
     )
+from bzrlib.i18n import gettext
+""")
+
 from bzrlib.decorators import needs_read_lock
-from bzrlib.errors import BzrError, NoSuchId
-from bzrlib import errors
-from bzrlib.inventory import InventoryFile
 from bzrlib.inter import InterObject
-from bzrlib.osutils import fingerprint_file
-from bzrlib.symbol_versioning import deprecated_function, deprecated_in
-from bzrlib.trace import note
+from bzrlib.symbol_versioning import (
+    deprecated_in,
+    deprecated_method,
+    )
 
 
 class Tree(object):
@@ -49,17 +55,18 @@
 
     * `RevisionTree` is a tree as recorded at some point in the past.
 
-    Trees contain an `Inventory` object, and also know how to retrieve
-    file texts mentioned in the inventory, either from a working
-    directory or from a store.
-
-    It is possible for trees to contain files that are not described
-    in their inventory or vice versa; for this use `filenames()`.
-
     Trees can be compared, etc, regardless of whether they are working
     trees or versioned trees.
     """
 
+    def has_versioned_directories(self):
+        """Whether this tree can contain explicitly versioned directories.
+
+        This defaults to True, but some implementations may want to override
+        it.
+        """
+        return True
+
     def changes_from(self, other, want_unchanged=False, specific_files=None,
         extra_trees=None, require_versioned=False, include_root=False,
         want_unversioned=False):
@@ -127,13 +134,14 @@
         raise NotImplementedError(self.has_filename)
 
     def has_id(self, file_id):
-        return self.inventory.has_id(file_id)
+        raise NotImplementedError(self.has_id)
 
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def __contains__(self, file_id):
         return self.has_id(file_id)
 
     def has_or_had_id(self, file_id):
-        return self.inventory.has_id(file_id)
+        raise NotImplementedError(self.has_or_had_id)
 
     def is_ignored(self, filename):
         """Check whether the filename is ignored by this tree.
@@ -143,33 +151,17 @@
         """
         return False
 
-    def __iter__(self):
-        return iter(self.inventory)
-
     def all_file_ids(self):
         """Iterate through all file ids, including ids for missing files."""
-        return set(self.inventory)
+        raise NotImplementedError(self.all_file_ids)
 
     def id2path(self, file_id):
         """Return the path for a file id.
 
         :raises NoSuchId:
         """
-        return self.inventory.id2path(file_id)
-
-    def is_control_filename(self, filename):
-        """True if filename is the name of a control file in this tree.
-
-        :param filename: A filename within the tree. This is a relative path
-        from the root of this tree.
-
-        This is true IF and ONLY IF the filename is part of the meta data
-        that bzr controls in this tree. I.E. a random .bzr directory placed
-        on disk will not be a control file for this tree.
-        """
-        return self.bzrdir.is_control_filename(filename)
-
-    @needs_read_lock
+        raise NotImplementedError(self.id2path)
+
     def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         """Walk the tree in 'by_dir' order.
 
@@ -192,14 +184,25 @@
              g
 
         The yield order (ignoring root) would be::
+
           a, f, a/b, a/d, a/b/c, a/d/e, f/g
 
         :param yield_parents: If True, yield the parents from the root leading
             down to specific_file_ids that have been requested. This has no
             impact if specific_file_ids is None.
         """
-        return self.inventory.iter_entries_by_dir(
-            specific_file_ids=specific_file_ids, yield_parents=yield_parents)
+        raise NotImplementedError(self.iter_entries_by_dir)
+
+    def list_files(self, include_root=False, from_dir=None, recursive=True):
+        """List all files in this tree.
+
+        :param include_root: Whether to include the entry for the tree root
+        :param from_dir: Directory under which to list files
+        :param recursive: Whether to list files recursively
+        :return: iterator over tuples of (path, versioned, kind, file_id,
+            inventory entry)
+        """
+        raise NotImplementedError(self.list_files)
 
     def iter_references(self):
         if self.supports_tree_reference():
@@ -256,9 +259,6 @@
     def _file_size(self, entry, stat_value):
         raise NotImplementedError(self._file_size)
 
-    def _get_inventory(self):
-        return self._inventory
-
     def get_file(self, file_id, path=None):
         """Return a file object for the file file_id in the tree.
 
@@ -286,8 +286,11 @@
 
         :param file_id: The file_id of the file.
         :param path: The path of the file.
+
         If both file_id and path are supplied, an implementation may use
         either one.
+
+        :returns: A single byte string for the whole file.
         """
         my_file = self.get_file(file_id, path)
         try:
@@ -300,11 +303,40 @@
 
         :param file_id: The file_id of the file.
         :param path: The path of the file.
+
         If both file_id and path are supplied, an implementation may use
         either one.
         """
         return osutils.split_lines(self.get_file_text(file_id, path))
 
+    def get_file_verifier(self, file_id, path=None, stat_value=None):
+        """Return a verifier for a file.
+
+        The default implementation returns a sha1.
+
+        :param file_id: The handle for this file.
+        :param path: The path that this file can be found at.
+            These must point to the same object.
+        :param stat_value: Optional stat value for the object
+        :return: Tuple with verifier name and verifier data
+        """
+        return ("SHA1", self.get_file_sha1(file_id, path=path,
+            stat_value=stat_value))
+
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        """Return the SHA1 file for a file.
+
+        :note: callers should use get_file_verifier instead
+            where possible, as the underlying repository implementation may
+            have quicker access to a non-sha1 verifier.
+
+        :param file_id: The handle for this file.
+        :param path: The path that this file can be found at.
+            These must point to the same object.
+        :param stat_value: Optional stat value for the object
+        """
+        raise NotImplementedError(self.get_file_sha1)
+
     def get_file_mtime(self, file_id, path=None):
         """Return the modification time for a file.
 
@@ -324,7 +356,16 @@
         raise NotImplementedError(self.get_file_size)
 
     def get_file_by_path(self, path):
-        return self.get_file(self._inventory.path2id(path), path)
+        raise NotImplementedError(self.get_file_by_path)
+
+    def is_executable(self, file_id, path=None):
+        """Check if a file is executable.
+
+        :param file_id: The handle for this file.
+        :param path: The path that this file can be found at.
+            These must point to the same object.
+        """
+        raise NotImplementedError(self.is_executable)
 
     def iter_files_bytes(self, desired_files):
         """Iterate through file contents.
@@ -352,94 +393,19 @@
             cur_file = (self.get_file_text(file_id),)
             yield identifier, cur_file
 
-    def get_symlink_target(self, file_id):
+    def get_symlink_target(self, file_id, path=None):
         """Get the target for a given file_id.
 
         It is assumed that the caller already knows that file_id is referencing
         a symlink.
         :param file_id: Handle for the symlink entry.
+        :param path: The path of the file.
+        If both file_id and path are supplied, an implementation may use
+        either one.
         :return: The path the symlink points to.
         """
         raise NotImplementedError(self.get_symlink_target)
 
-    def get_canonical_inventory_paths(self, paths):
-        """Like get_canonical_inventory_path() but works on multiple items.
-
-        :param paths: A sequence of paths relative to the root of the tree.
-        :return: A list of paths, with each item the corresponding input path
-        adjusted to account for existing elements that match case
-        insensitively.
-        """
-        return list(self._yield_canonical_inventory_paths(paths))
-
-    def get_canonical_inventory_path(self, path):
-        """Returns the first inventory item that case-insensitively matches path.
-
-        If a path matches exactly, it is returned. If no path matches exactly
-        but more than one path matches case-insensitively, it is implementation
-        defined which is returned.
-
-        If no path matches case-insensitively, the input path is returned, but
-        with as many path entries that do exist changed to their canonical
-        form.
-
-        If you need to resolve many names from the same tree, you should
-        use get_canonical_inventory_paths() to avoid O(N) behaviour.
-
-        :param path: A paths relative to the root of the tree.
-        :return: The input path adjusted to account for existing elements
-        that match case insensitively.
-        """
-        return self._yield_canonical_inventory_paths([path]).next()
-
-    def _yield_canonical_inventory_paths(self, paths):
-        for path in paths:
-            # First, if the path as specified exists exactly, just use it.
-            if self.path2id(path) is not None:
-                yield path
-                continue
-            # go walkin...
-            cur_id = self.get_root_id()
-            cur_path = ''
-            bit_iter = iter(path.split("/"))
-            for elt in bit_iter:
-                lelt = elt.lower()
-                new_path = None
-                for child in self.iter_children(cur_id):
-                    try:
-                        # XXX: it seem like if the child is known to be in the
-                        # tree, we shouldn't need to go from its id back to
-                        # its path -- mbp 2010-02-11
-                        #
-                        # XXX: it seems like we could be more efficient
-                        # by just directly looking up the original name and
-                        # only then searching all children; also by not
-                        # chopping paths so much. -- mbp 2010-02-11
-                        child_base = os.path.basename(self.id2path(child))
-                        if (child_base == elt):
-                            # if we found an exact match, we can stop now; if
-                            # we found an approximate match we need to keep
-                            # searching because there might be an exact match
-                            # later.  
-                            cur_id = child
-                            new_path = osutils.pathjoin(cur_path, child_base)
-                            break
-                        elif child_base.lower() == lelt:
-                            cur_id = child
-                            new_path = osutils.pathjoin(cur_path, child_base)
-                    except NoSuchId:
-                        # before a change is committed we can see this error...
-                        continue
-                if new_path:
-                    cur_path = new_path
-                else:
-                    # got to the end of this directory and no entries matched.
-                    # Return what matched so far, plus the rest as specified.
-                    cur_path = osutils.pathjoin(cur_path, elt, *list(bit_iter))
-                    break
-            yield cur_path
-        # all done.
-
     def get_root_id(self):
         """Return the file_id for the root of this tree."""
         raise NotImplementedError(self.get_root_id)
@@ -503,59 +469,49 @@
             except errors.NoSuchRevisionInTree:
                 yield self.repository.revision_tree(revision_id)
 
-    @staticmethod
-    def _file_revision(revision_tree, file_id):
-        """Determine the revision associated with a file in a given tree."""
-        revision_tree.lock_read()
-        try:
-            return revision_tree.inventory[file_id].revision
-        finally:
-            revision_tree.unlock()
-
     def _get_file_revision(self, file_id, vf, tree_revision):
         """Ensure that file_id, tree_revision is in vf to plan the merge."""
 
         if getattr(self, '_repository', None) is None:
             last_revision = tree_revision
-            parent_keys = [(file_id, self._file_revision(t, file_id)) for t in
+            parent_keys = [(file_id, t.get_file_revision(file_id)) for t in
                 self._iter_parent_trees()]
             vf.add_lines((file_id, last_revision), parent_keys,
                          self.get_file_lines(file_id))
             repo = self.branch.repository
             base_vf = repo.texts
         else:
-            last_revision = self._file_revision(self, file_id)
+            last_revision = self.get_file_revision(file_id)
             base_vf = self._repository.texts
         if base_vf not in vf.fallback_versionedfiles:
             vf.fallback_versionedfiles.append(base_vf)
         return last_revision
 
-    inventory = property(_get_inventory,
-                         doc="Inventory of this Tree")
-
     def _check_retrieved(self, ie, f):
         if not __debug__:
             return
-        fp = fingerprint_file(f)
+        fp = osutils.fingerprint_file(f)
         f.seek(0)
 
         if ie.text_size is not None:
             if ie.text_size != fp['size']:
-                raise BzrError("mismatched size for file %r in %r" % (ie.file_id, self._store),
+                raise errors.BzrError(
+                        "mismatched size for file %r in %r" %
+                        (ie.file_id, self._store),
                         ["inventory expects %d bytes" % ie.text_size,
                          "file is actually %d bytes" % fp['size'],
                          "store is probably damaged/corrupt"])
 
         if ie.text_sha1 != fp['sha1']:
-            raise BzrError("wrong SHA-1 for file %r in %r" % (ie.file_id, self._store),
+            raise errors.BzrError("wrong SHA-1 for file %r in %r" %
+                    (ie.file_id, self._store),
                     ["inventory expects %s" % ie.text_sha1,
                      "file is actually %s" % fp['sha1'],
                      "store is probably damaged/corrupt"])
 
-    @needs_read_lock
     def path2id(self, path):
         """Return the id for path in this tree."""
-        return self._inventory.path2id(path)
+        raise NotImplementedError(self.path2id)
 
     def paths2ids(self, paths, trees=[], require_versioned=True):
         """Return all the ids that can be reached by walking from paths.
@@ -618,11 +574,7 @@
 
         :return: set of paths.
         """
-        # NB: we specifically *don't* call self.has_filename, because for
-        # WorkingTrees that can indicate files that exist on disk but that
-        # are not versioned.
-        pred = self.inventory.has_filename
-        return set((p for p in paths if not pred(p)))
+        raise NotImplementedError(self.filter_unversioned_files)
 
     def walkdirs(self, prefix=""):
         """Walk the contents of this tree from path down.
@@ -680,7 +632,7 @@
         prefs = self.iter_search_rules([path], filter_pref_names).next()
         stk = filters._get_filter_stack_for(prefs)
         if 'filters' in debug.debug_flags:
-            note("*** %s content-filter: %s => %r" % (path,prefs,stk))
+            trace.note(gettext("*** {0} content-filter: {1} => {2!r}").format(path,prefs,stk))
         return stk
 
     def _content_filter_stack_provider(self):
@@ -725,68 +677,158 @@
         return searcher
 
 
-######################################################################
-# diff
-
-# TODO: Merge these two functions into a single one that can operate
-# on either a whole tree or a set of files.
-
-# TODO: Return the diff in order by filename, not by category or in
-# random order.  Can probably be done by lock-stepping through the
-# filenames from both trees.
-
-
-def file_status(filename, old_tree, new_tree):
-    """Return single-letter status, old and new names for a file.
-
-    The complexity here is in deciding how to represent renames;
-    many complex cases are possible.
+class InventoryTree(Tree):
+    """A tree that relies on an inventory for its metadata.
+
+    Trees contain an `Inventory` object, and also know how to retrieve
+    file texts mentioned in the inventory, either from a working
+    directory or from a store.
+
+    It is possible for trees to contain files that are not described
+    in their inventory or vice versa; for this use `filenames()`.
+
+    Subclasses should set the _inventory attribute, which is considered
+    private to external API users.
     """
-    old_inv = old_tree.inventory
-    new_inv = new_tree.inventory
-    new_id = new_inv.path2id(filename)
-    old_id = old_inv.path2id(filename)
-
-    if not new_id and not old_id:
-        # easy: doesn't exist in either; not versioned at all
-        if new_tree.is_ignored(filename):
-            return 'I', None, None
-        else:
-            return '?', None, None
-    elif new_id:
-        # There is now a file of this name, great.
-        pass
-    else:
-        # There is no longer a file of this name, but we can describe
-        # what happened to the file that used to have
-        # this name.  There are two possibilities: either it was
-        # deleted entirely, or renamed.
-        if new_inv.has_id(old_id):
-            return 'X', old_inv.id2path(old_id), new_inv.id2path(old_id)
-        else:
-            return 'D', old_inv.id2path(old_id), None
-
-    # if the file_id is new in this revision, it is added
-    if new_id and not old_inv.has_id(new_id):
-        return 'A'
-
-    # if there used to be a file of this name, but that ID has now
-    # disappeared, it is deleted
-    if old_id and not new_inv.has_id(old_id):
-        return 'D'
-
-    return 'wtf?'
-
-
-@deprecated_function(deprecated_in((1, 9, 0)))
-def find_renames(old_inv, new_inv):
-    for file_id in old_inv:
-        if file_id not in new_inv:
-            continue
-        old_name = old_inv.id2path(file_id)
-        new_name = new_inv.id2path(file_id)
-        if old_name != new_name:
-            yield (old_name, new_name)
+
+    def get_canonical_inventory_paths(self, paths):
+        """Like get_canonical_inventory_path() but works on multiple items.
+
+        :param paths: A sequence of paths relative to the root of the tree.
+        :return: A list of paths, with each item the corresponding input path
+        adjusted to account for existing elements that match case
+        insensitively.
+        """
+        return list(self._yield_canonical_inventory_paths(paths))
+
+    def get_canonical_inventory_path(self, path):
+        """Returns the first inventory item that case-insensitively matches path.
+
+        If a path matches exactly, it is returned. If no path matches exactly
+        but more than one path matches case-insensitively, it is implementation
+        defined which is returned.
+
+        If no path matches case-insensitively, the input path is returned, but
+        with as many path entries that do exist changed to their canonical
+        form.
+
+        If you need to resolve many names from the same tree, you should
+        use get_canonical_inventory_paths() to avoid O(N) behaviour.
+
+        :param path: A paths relative to the root of the tree.
+        :return: The input path adjusted to account for existing elements
+        that match case insensitively.
+        """
+        return self._yield_canonical_inventory_paths([path]).next()
+
+    def _yield_canonical_inventory_paths(self, paths):
+        for path in paths:
+            # First, if the path as specified exists exactly, just use it.
+            if self.path2id(path) is not None:
+                yield path
+                continue
+            # go walkin...
+            cur_id = self.get_root_id()
+            cur_path = ''
+            bit_iter = iter(path.split("/"))
+            for elt in bit_iter:
+                lelt = elt.lower()
+                new_path = None
+                for child in self.iter_children(cur_id):
+                    try:
+                        # XXX: it seem like if the child is known to be in the
+                        # tree, we shouldn't need to go from its id back to
+                        # its path -- mbp 2010-02-11
+                        #
+                        # XXX: it seems like we could be more efficient
+                        # by just directly looking up the original name and
+                        # only then searching all children; also by not
+                        # chopping paths so much. -- mbp 2010-02-11
+                        child_base = os.path.basename(self.id2path(child))
+                        if (child_base == elt):
+                            # if we found an exact match, we can stop now; if
+                            # we found an approximate match we need to keep
+                            # searching because there might be an exact match
+                            # later.  
+                            cur_id = child
+                            new_path = osutils.pathjoin(cur_path, child_base)
+                            break
+                        elif child_base.lower() == lelt:
+                            cur_id = child
+                            new_path = osutils.pathjoin(cur_path, child_base)
+                    except errors.NoSuchId:
+                        # before a change is committed we can see this error...
+                        continue
+                if new_path:
+                    cur_path = new_path
+                else:
+                    # got to the end of this directory and no entries matched.
+                    # Return what matched so far, plus the rest as specified.
+                    cur_path = osutils.pathjoin(cur_path, elt, *list(bit_iter))
+                    break
+            yield cur_path
+        # all done.
+
+    def _get_inventory(self):
+        return self._inventory
+
+    inventory = property(_get_inventory,
+                         doc="Inventory of this Tree")
+
+    @needs_read_lock
+    def path2id(self, path):
+        """Return the id for path in this tree."""
+        return self._inventory.path2id(path)
+
+    def id2path(self, file_id):
+        """Return the path for a file id.
+
+        :raises NoSuchId:
+        """
+        return self.inventory.id2path(file_id)
+
+    def has_id(self, file_id):
+        return self.inventory.has_id(file_id)
+
+    def has_or_had_id(self, file_id):
+        return self.inventory.has_id(file_id)
+
+    def all_file_ids(self):
+        return set(self.inventory)
+
+    @deprecated_method(deprecated_in((2, 4, 0)))
+    def __iter__(self):
+        return iter(self.inventory)
+
+    def filter_unversioned_files(self, paths):
+        """Filter out paths that are versioned.
+
+        :return: set of paths.
+        """
+        # NB: we specifically *don't* call self.has_filename, because for
+        # WorkingTrees that can indicate files that exist on disk but that
+        # are not versioned.
+        pred = self.inventory.has_filename
+        return set((p for p in paths if not pred(p)))
+
+    @needs_read_lock
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+        """Walk the tree in 'by_dir' order.
+
+        This will yield each entry in the tree as a (path, entry) tuple.
+        The order that they are yielded is:
+
+        See Tree.iter_entries_by_dir for details.
+
+        :param yield_parents: If True, yield the parents from the root leading
+            down to specific_file_ids that have been requested. This has no
+            impact if specific_file_ids is None.
+        """
+        return self.inventory.iter_entries_by_dir(
+            specific_file_ids=specific_file_ids, yield_parents=yield_parents)
+
+    def get_file_by_path(self, path):
+        return self.get_file(self._inventory.path2id(path), path)
 
 
 def find_ids_across_trees(filenames, trees, require_versioned=True):
@@ -799,7 +841,7 @@
         None)
     :param trees: The trees to find file_ids within
     :param require_versioned: if true, all specified filenames must occur in
-    at least one tree.
+        at least one tree.
     :return: a set of file ids for the specified filenames and their children.
     """
     if not filenames:
@@ -881,6 +923,12 @@
 
     _optimisers = []
 
+    @classmethod
+    def is_compatible(kls, source, target):
+        # The default implementation is naive and uses the public API, so
+        # it works for all trees.
+        return True
+
     def _changes_from_entries(self, source_entry, target_entry,
         source_path=None, target_path=None):
         """Generate a iter_changes tuple between source_entry and target_entry.
@@ -934,8 +982,8 @@
         if source_kind != target_kind:
             changed_content = True
         elif source_kind == 'file':
-            if (self.source.get_file_sha1(file_id, source_path, source_stat) !=
-                self.target.get_file_sha1(file_id, target_path, target_stat)):
+            if not self.file_content_matches(file_id, file_id, source_path,
+                    target_path, source_stat, target_stat):
                 changed_content = True
         elif source_kind == 'symlink':
             if (self.source.get_symlink_target(file_id) !=
@@ -991,7 +1039,7 @@
             # All files are unversioned, so just return an empty delta
             # _compare_trees would think we want a complete delta
             result = delta.TreeDelta()
-            fake_entry = InventoryFile('unused', 'unused', 'unused')
+            fake_entry = inventory.InventoryFile('unused', 'unused', 'unused')
             result.unversioned = [(path, None,
                 self.target._comparison_data(fake_entry, path)[0]) for path in
                 specific_files]
@@ -1062,9 +1110,9 @@
                                      self.target.extras()
                 if specific_files is None or
                     osutils.is_inside_any(specific_files, p)])
-            all_unversioned = deque(all_unversioned)
+            all_unversioned = collections.deque(all_unversioned)
         else:
-            all_unversioned = deque()
+            all_unversioned = collections.deque()
         to_paths = {}
         from_entries_by_dir = list(self.source.iter_entries_by_dir(
             specific_file_ids=specific_file_ids))
@@ -1076,7 +1124,7 @@
         # the unversioned path lookup only occurs on real trees - where there
         # can be extras. So the fake_entry is solely used to look up
         # executable it values when execute is not supported.
-        fake_entry = InventoryFile('unused', 'unused', 'unused')
+        fake_entry = inventory.InventoryFile('unused', 'unused', 'unused')
         for target_path, target_entry in to_entries_by_dir:
             while (all_unversioned and
                 all_unversioned[0][0] < target_path.split('/')):
@@ -1130,7 +1178,7 @@
             if file_id in to_paths:
                 # already returned
                 continue
-            if file_id not in self.target.all_file_ids():
+            if not self.target.has_id(file_id):
                 # common case - paths we have not emitted are not present in
                 # target.
                 to_path = None
@@ -1254,6 +1302,43 @@
                     changed_file_ids.add(result[0])
                     yield result
 
+    @needs_read_lock
+    def file_content_matches(self, source_file_id, target_file_id,
+            source_path=None, target_path=None, source_stat=None, target_stat=None):
+        """Check if two files are the same in the source and target trees.
+
+        This only checks that the contents of the files are the same,
+        it does not touch anything else.
+
+        :param source_file_id: File id of the file in the source tree
+        :param target_file_id: File id of the file in the target tree
+        :param source_path: Path of the file in the source tree
+        :param target_path: Path of the file in the target tree
+        :param source_stat: Optional stat value of the file in the source tree
+        :param target_stat: Optional stat value of the file in the target tree
+        :return: Boolean indicating whether the files have the same contents
+        """
+        source_verifier_kind, source_verifier_data = self.source.get_file_verifier(
+            source_file_id, source_path, source_stat)
+        target_verifier_kind, target_verifier_data = self.target.get_file_verifier(
+            target_file_id, target_path, target_stat)
+        if source_verifier_kind == target_verifier_kind:
+            return (source_verifier_data == target_verifier_data)
+        # Fall back to SHA1 for now
+        if source_verifier_kind != "SHA1":
+            source_sha1 = self.source.get_file_sha1(source_file_id,
+                    source_path, source_stat)
+        else:
+            source_sha1 = source_verifier_data
+        if target_verifier_kind != "SHA1":
+            target_sha1 = self.target.get_file_sha1(target_file_id,
+                    target_path, target_stat)
+        else:
+            target_sha1 = target_verifier_data
+        return (source_sha1 == target_sha1)
+
+InterTree.register_optimiser(InterTree)
+
 
 class MultiWalker(object):
     """Walk multiple trees simultaneously, getting combined results."""

=== modified file 'bzrlib/tsort.py'
--- bzrlib/tsort.py	2009-08-26 16:03:59 +0000
+++ bzrlib/tsort.py	2011-05-18 16:42:48 +0000
@@ -177,7 +177,8 @@
         revision number sequences in the output. See the output description of
         the MergeSorter docstring for details.
     :result: See the MergeSorter docstring for details.
-    node identifiers can be any hashable object, and are typically strings.
+
+    Node identifiers can be any hashable object, and are typically strings.
     """
     return MergeSorter(graph, branch_tip, mainline_revisions,
         generate_revno).sorted()

=== modified file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	2009-06-02 19:56:24 +0000
+++ bzrlib/tuned_gzip.py	2011-01-11 18:10:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 # Written by Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 import zlib
 
 # we want a \n preserved, break on \n only splitlines.
-import bzrlib
+from bzrlib import symbol_versioning
 
 __all__ = ["GzipFile", "bytes_to_gzip"]
 
@@ -118,6 +118,13 @@
     Yes, its only 1.6 seconds, but they add up.
     """
 
+    def __init__(self, *args, **kwargs):
+        symbol_versioning.warn(
+            symbol_versioning.deprecated_in((2, 3, 0))
+            % 'bzrlib.tuned_gzip.GzipFile',
+            DeprecationWarning, stacklevel=2)
+        gzip.GzipFile.__init__(self, *args, **kwargs)
+
     def _add_read_data(self, data):
         # 4169 calls in 183
         # temp var for len(data) and switch to +='s.
@@ -395,4 +402,12 @@
         # (4 seconds to 1 seconds for the sample upgrades I was testing).
         self.write(''.join(lines))
 
+    if sys.version_info > (2, 7):
+        # As of Python 2.7 the crc32 must be positive when close is called
+        def close(self):
+            if self.fileobj is None:
+                return
+            if self.mode == gzip.WRITE:
+                self.crc &= 0xFFFFFFFFL
+            gzip.GzipFile.close(self)
 

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2010-06-25 20:34:05 +0000
+++ bzrlib/ui/__init__.py	2011-10-05 18:40:53 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,26 +42,17 @@
 """
 
 
-import os
-import sys
 import warnings
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import getpass
-
 from bzrlib import (
-    errors,
+    config,
     osutils,
     progress,
     trace,
     )
 """)
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    deprecated_method,
-    )
 
 
 _valid_boolean_strings = dict(yes=True, no=False,
@@ -100,6 +91,42 @@
     return val
 
 
+class ConfirmationUserInterfacePolicy(object):
+    """Wrapper for a UIFactory that allows or denies all confirmed actions."""
+
+    def __init__(self, wrapped_ui, default_answer, specific_answers):
+        """Generate a proxy UI that does no confirmations.
+
+        :param wrapped_ui: Underlying UIFactory.
+        :param default_answer: Bool for whether requests for
+            confirmation from the user should be noninteractively accepted or
+            denied.
+        :param specific_answers: Map from confirmation_id to bool answer.
+        """
+        self.wrapped_ui = wrapped_ui
+        self.default_answer = default_answer
+        self.specific_answers = specific_answers
+
+    def __getattr__(self, name):
+        return getattr(self.wrapped_ui, name)
+
+    def __repr__(self):
+        return '%s(%r, %r, %r)' % (
+            self.__class__.__name__,
+            self.wrapped_ui,
+            self.default_answer, 
+            self.specific_answers)
+
+    def confirm_action(self, prompt, confirmation_id, prompt_kwargs):
+        if confirmation_id in self.specific_answers:
+            return self.specific_answers[confirmation_id]
+        elif self.default_answer is not None:
+            return self.default_answer
+        else:
+            return self.wrapped_ui.confirm_action(
+                prompt, confirmation_id, prompt_kwargs)
+
+
 class UIFactory(object):
     """UI abstraction.
 
@@ -118,7 +145,26 @@
             "%(from_format)s to %(to_format)s.\n"
             "This may take some time. Upgrade the repositories to the "
             "same format for better performance."
-            )
+            ),
+        experimental_format_fetch=("Fetching into experimental format "
+            "%(to_format)s.\n"
+            "This format may be unreliable or change in the future "
+            "without an upgrade path.\n"),
+        deprecated_command=(
+            "The command 'bzr %(deprecated_name)s' "
+            "has been deprecated in bzr %(deprecated_in_version)s. "
+            "Please use 'bzr %(recommended_name)s' instead."),
+        deprecated_command_option=(
+            "The option '%(deprecated_name)s' to 'bzr %(command)s' "
+            "has been deprecated in bzr %(deprecated_in_version)s. "
+            "Please use '%(recommended_name)s' instead."),
+        recommend_upgrade=("%(current_format_name)s is deprecated "
+            "and a better format is available.\n"
+            "It is recommended that you upgrade by "
+            "running the command\n"
+            "  bzr upgrade %(basedir)s"),
+        locks_steal_dead=(
+            u"Stole dead lock %(lock_url)s %(other_holder_info)s."),
         )
 
     def __init__(self):
@@ -151,10 +197,28 @@
         """
         self._quiet = state
 
-    def get_password(self, prompt='', **kwargs):
+    def confirm_action(self, prompt, confirmation_id, prompt_kwargs):
+        """Seek user confirmation for an action.
+
+        If the UI is noninteractive, or the user does not want to be asked
+        about this action, True is returned, indicating bzr should just
+        proceed.
+
+        The confirmation id allows the user to configure certain actions to
+        always be confirmed or always denied, and for UIs to specialize the
+        display of particular confirmations.
+
+        :param prompt: Suggested text to display to the user.
+        :param prompt_kwargs: A dictionary of arguments that can be
+            string-interpolated into the prompt.
+        :param confirmation_id: Unique string identifier for the confirmation.
+        """
+        return self.get_boolean(prompt % prompt_kwargs)
+
+    def get_password(self, prompt=u'', **kwargs):
         """Prompt the user for a password.
 
-        :param prompt: The prompt to present the user
+        :param prompt: The prompt to present the user (must be unicode)
         :param kwargs: Arguments which will be expanded into the prompt.
                        This lets front ends display different things if
                        they so choose.
@@ -188,9 +252,7 @@
         """
         # XXX: is the caller supposed to close the resulting object?
         if encoding is None:
-            from bzrlib import config
-            encoding = config.GlobalConfig().get_user_option(
-                'output_encoding')
+            encoding = config.GlobalStack().get('output_encoding')
         if encoding is None:
             encoding = osutils.get_terminal_encoding(trace=True)
         if encoding_type is None:
@@ -251,31 +313,52 @@
         try:
             template = self._user_warning_templates[warning_id]
         except KeyError:
-            fail = "failed to format warning %r, %r" % (warning_id, message_args)
-            warnings.warn(fail)   # so tests will fail etc
+            fail = "bzr warning: %r, %r" % (warning_id, message_args)
+            warnings.warn("no template for warning: " + fail)   # so tests will fail etc
             return fail
         try:
             return template % message_args
         except ValueError, e:
-            fail = "failed to format warning %r, %r: %s" % (
+            fail = "bzr unprintable warning: %r, %r, %s" % (
                 warning_id, message_args, e)
             warnings.warn(fail)   # so tests will fail etc
             return fail
 
+    def choose(self, msg, choices, default=None):
+        """Prompt the user for a list of alternatives.
+
+        :param msg: message to be shown as part of the prompt.
+
+        :param choices: list of choices, with the individual choices separated
+            by '\n', e.g.: choose("Save changes?", "&Yes\n&No\n&Cancel"). The
+            letter after the '&' is the shortcut key for that choice. Thus you
+            can type 'c' to select "Cancel".  Shorcuts are case insensitive.
+            The shortcut does not need to be the first letter. If a shorcut key
+            is not provided, the first letter for the choice will be used.
+
+        :param default: default choice (index), returned for example when enter
+            is pressed for the console version.
+
+        :return: the index fo the user choice (so '0', '1' or '2' for
+            respectively yes/no/cancel in the previous example).
+        """
+        raise NotImplementedError(self.choose)
+
     def get_boolean(self, prompt):
         """Get a boolean question answered from the user.
 
         :param prompt: a message to prompt the user with. Should be a single
-        line without terminating \n.
+            line without terminating \\n.
         :return: True or False for y/yes or n/no.
         """
-        raise NotImplementedError(self.get_boolean)
+        choice = self.choose(prompt + '?', '&yes\n&no', default=None)
+        return 0 == choice
 
     def get_integer(self, prompt):
         """Get an integer from the user.
 
         :param prompt: a message to prompt the user with. Could be a multi-line
-            prompt but without a terminating \n.
+            prompt but without a terminating \\n.
 
         :return: A signed integer.
         """
@@ -289,21 +372,14 @@
         """
         return NullProgressView()
 
-    def recommend_upgrade(self,
-        current_format_name,
-        basedir):
-        # XXX: this should perhaps be in the TextUIFactory and the default can do
-        # nothing
-        #
-        # XXX: Change to show_user_warning - that will accomplish the previous
-        # xxx. -- mbp 2010-02-25
-        trace.warning("%s is deprecated "
-            "and a better format is available.\n"
-            "It is recommended that you upgrade by "
-            "running the command\n"
-            "  bzr upgrade %s",
-            current_format_name,
-            basedir)
+    def recommend_upgrade(self, current_format_name, basedir):
+        """Recommend the user upgrade a control directory.
+
+        :param current_format_name: Description of the current format
+        :param basedir: Location of the control dir
+        """
+        self.show_user_warning('recommend_upgrade',
+            current_format_name=current_format_name, basedir=basedir)
 
     def report_transport_activity(self, transport, byte_count, direction):
         """Called by transports as they do IO.
@@ -360,24 +436,18 @@
         """Show a warning to the user."""
         raise NotImplementedError(self.show_warning)
 
-    def warn_cross_format_fetch(self, from_format, to_format):
-        """Warn about a potentially slow cross-format transfer.
-        
-        This is deprecated in favor of show_user_warning, but retained for api
-        compatibility in 2.0 and 2.1.
-        """
-        self.show_user_warning('cross_format_fetch', from_format=from_format,
-            to_format=to_format)
-
-    def warn_experimental_format_fetch(self, inter):
-        """Warn about fetching into experimental repository formats."""
-        if inter.target._format.experimental:
-            trace.warning("Fetching into experimental format %s.\n"
-                "This format may be unreliable or change in the future "
-                "without an upgrade path.\n" % (inter.target._format,))
-
-
-class SilentUIFactory(UIFactory):
+
+class NoninteractiveUIFactory(UIFactory):
+    """Base class for UIs with no user."""
+
+    def confirm_action(self, prompt, confirmation_id, prompt_kwargs):
+        return True
+
+    def __repr__(self):
+        return '%s()' % (self.__class__.__name__, )
+
+
+class SilentUIFactory(NoninteractiveUIFactory):
     """A UI Factory which never prints anything.
 
     This is the default UI, if another one is never registered by a program
@@ -418,13 +488,16 @@
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self.responses)
 
+    def confirm_action(self, prompt, confirmation_id, args):
+        return self.get_boolean(prompt % args)
+
     def get_boolean(self, prompt):
         return self.responses.pop(0)
 
     def get_integer(self, prompt):
         return self.responses.pop(0)
 
-    def get_password(self, prompt='', **kwargs):
+    def get_password(self, prompt=u'', **kwargs):
         return self.responses.pop(0)
 
     def get_username(self, prompt, **kwargs):

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2010-07-09 15:47:01 +0000
+++ bzrlib/ui/text.py	2011-10-10 21:21:50 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,20 +18,20 @@
 """Text UI, write output to the console.
 """
 
-import codecs
-import getpass
 import os
 import sys
 import time
-import warnings
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import codecs
+import getpass
+import warnings
+
 from bzrlib import (
     debug,
     progress,
     osutils,
-    symbol_versioning,
     trace,
     )
 
@@ -43,6 +43,116 @@
     )
 
 
+class _ChooseUI(object):
+
+    """ Helper class for choose implementation.
+    """
+
+    def __init__(self, ui, msg, choices, default):
+        self.ui = ui
+        self._setup_mode()
+        self._build_alternatives(msg, choices, default)
+
+    def _setup_mode(self):
+        """Setup input mode (line-based, char-based) and echo-back.
+
+        Line-based input is used if the BZR_TEXTUI_INPUT environment
+        variable is set to 'line-based', or if there is no controlling
+        terminal.
+        """
+        if os.environ.get('BZR_TEXTUI_INPUT') != 'line-based' and \
+           self.ui.stdin == sys.stdin and self.ui.stdin.isatty():
+            self.line_based = False
+            self.echo_back = True
+        else:
+            self.line_based = True
+            self.echo_back = not self.ui.stdin.isatty()
+
+    def _build_alternatives(self, msg, choices, default):
+        """Parse choices string.
+
+        Setup final prompt and the lists of choices and associated
+        shortcuts.
+        """
+        index = 0
+        help_list = []
+        self.alternatives = {}
+        choices = choices.split('\n')
+        if default is not None and default not in range(0, len(choices)):
+            raise ValueError("invalid default index")
+        for c in choices:
+            name = c.replace('&', '').lower()
+            choice = (name, index)
+            if name in self.alternatives:
+                raise ValueError("duplicated choice: %s" % name)
+            self.alternatives[name] = choice
+            shortcut = c.find('&')
+            if -1 != shortcut and (shortcut + 1) < len(c):
+                help = c[:shortcut]
+                help += '[' + c[shortcut + 1] + ']'
+                help += c[(shortcut + 2):]
+                shortcut = c[shortcut + 1]
+            else:
+                c = c.replace('&', '')
+                shortcut = c[0]
+                help = '[%s]%s' % (shortcut, c[1:])
+            shortcut = shortcut.lower()
+            if shortcut in self.alternatives:
+                raise ValueError("duplicated shortcut: %s" % shortcut)
+            self.alternatives[shortcut] = choice
+            # Add redirections for default.
+            if index == default:
+                self.alternatives[''] = choice
+                self.alternatives['\r'] = choice
+            help_list.append(help)
+            index += 1
+
+        self.prompt = u'%s (%s): ' % (msg, ', '.join(help_list))
+
+    def _getline(self):
+        line = self.ui.stdin.readline()
+        if '' == line:
+            raise EOFError
+        return line.strip()
+
+    def _getchar(self):
+        char = osutils.getchar()
+        if char == chr(3): # INTR
+            raise KeyboardInterrupt
+        if char == chr(4): # EOF (^d, C-d)
+            raise EOFError
+        return char
+
+    def interact(self):
+        """Keep asking the user until a valid choice is made.
+        """
+        if self.line_based:
+            getchoice = self._getline
+        else:
+            getchoice = self._getchar
+        iter = 0
+        while True:
+            iter += 1
+            if 1 == iter or self.line_based:
+                self.ui.prompt(self.prompt)
+            try:
+                choice = getchoice()
+            except EOFError:
+                self.ui.stderr.write('\n')
+                return None
+            except KeyboardInterrupt:
+                self.ui.stderr.write('\n')
+                raise KeyboardInterrupt
+            choice = choice.lower()
+            if choice not in self.alternatives:
+                # Not a valid choice, keep on asking.
+                continue
+            name, index = self.alternatives[choice]
+            if self.echo_back:
+                self.ui.stderr.write(name + '\n')
+            return index
+
+
 class TextUIFactory(UIFactory):
     """A UI factory for Text user interefaces."""
 
@@ -61,6 +171,27 @@
         # paints progress, network activity, etc
         self._progress_view = self.make_progress_view()
 
+    def choose(self, msg, choices, default=None):
+        """Prompt the user for a list of alternatives.
+
+        Support both line-based and char-based editing.
+
+        In line-based mode, both the shortcut and full choice name are valid
+        answers, e.g. for choose('prompt', '&yes\n&no'): 'y', ' Y ', ' yes',
+        'YES ' are all valid input lines for choosing 'yes'.
+
+        An empty line, when in line-based mode, or pressing enter in char-based
+        mode will select the default choice (if any).
+
+        Choice is echoed back if:
+        - input is char-based; which means a controlling terminal is available,
+          and osutils.getchar is used
+        - input is line-based, and no controlling terminal is available
+        """
+
+        choose_ui = _ChooseUI(self, msg, choices, default)
+        return choose_ui.interact()
+
     def be_quiet(self, state):
         if state and not self._quiet:
             self.clear_term()
@@ -78,18 +209,6 @@
         # to clear it.  We might need to separately check for the case of
         self._progress_view.clear()
 
-    def get_boolean(self, prompt):
-        while True:
-            self.prompt(prompt + "? [y/n]: ")
-            line = self.stdin.readline().lower()
-            if line in ('y\n', 'yes\n'):
-                return True
-            elif line in ('n\n', 'no\n'):
-                return False
-            elif line in ('', None):
-                # end-of-file; possibly should raise an error here instead
-                return None
-
     def get_integer(self, prompt):
         while True:
             self.prompt(prompt)
@@ -114,7 +233,7 @@
                 password = password[:-1]
         return password
 
-    def get_password(self, prompt='', **kwargs):
+    def get_password(self, prompt=u'', **kwargs):
         """Prompt the user for a password.
 
         :param prompt: The prompt to present the user
@@ -198,11 +317,14 @@
         :param kwargs: Dictionary of arguments to insert into the prompt,
             to allow UIs to reformat the prompt.
         """
+        if type(prompt) != unicode:
+            raise ValueError("prompt %r not a unicode string" % prompt)
         if kwargs:
             # See <https://launchpad.net/bugs/365891>
             prompt = prompt % kwargs
         prompt = prompt.encode(osutils.get_terminal_encoding(), 'replace')
         self.clear_term()
+        self.stdout.flush()
         self.stderr.write(prompt)
 
     def report_transport_activity(self, transport, byte_count, direction):
@@ -304,11 +426,15 @@
         # we need one extra space for terminals that wrap on last char
         w = osutils.terminal_width() 
         if w is None:
-            return w
+            return None
         else:
             return w - 1
 
     def _show_line(self, s):
+        # sys.stderr.write("progress %r\n" % s)
+        width = self._avail_width()
+        if width is not None:
+            s = '%-*.*s' % (width, width, s)
         self._term_file.write('\r' + s + '\r')
 
     def clear(self):

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2010-04-30 11:03:59 +0000
+++ bzrlib/uncommit.py	2011-08-22 10:53:21 +0000
@@ -26,8 +26,29 @@
 from bzrlib.errors import BoundBranchOutOfDate
 
 
+def remove_tags(branch, graph, old_tip, new_tip):
+    """Remove tags on revisions between old_tip and new_tip.
+
+    :param branch: Branch to remove tags from
+    :param graph: Graph object for branch repository
+    :param old_tip: Old branch tip
+    :param new_tip: New branch tip
+    :return: Names of the removed tags
+    """
+    reverse_tags = branch.tags.get_reverse_tag_dict()
+    ancestors = graph.find_unique_ancestors(old_tip, [new_tip])
+    removed_tags = []
+    for revid, tags in reverse_tags.iteritems():
+        if not revid in ancestors:
+            continue
+        for tag in tags:
+            branch.tags.delete_tag(tag)
+            removed_tags.append(tag)
+    return removed_tags
+
+
 def uncommit(branch, dry_run=False, verbose=False, revno=None, tree=None,
-             local=False):
+             local=False, keep_tags=False):
     """Remove the last revision from the supplied branch.
 
     :param dry_run: Don't actually change anything
@@ -36,6 +57,8 @@
     :param local: If this branch is bound, only remove the revisions from the
         local branch. If this branch is not bound, it is an error to pass
         local=True.
+    :param keep_tags: Whether to keep tags pointing at the removed revisions
+        around.
     """
     unlockable = []
     try:
@@ -66,12 +89,10 @@
             revno = old_revno
         new_revno = revno - 1
 
-        revid_iterator = branch.repository.iter_reverse_revision_history(
-                            old_tip)
         cur_revno = old_revno
         new_revision_id = old_tip
         graph = branch.repository.get_graph()
-        for rev_id in revid_iterator:
+        for rev_id in graph.iter_lefthand_ancestry(old_tip):
             if cur_revno == new_revno:
                 new_revision_id = rev_id
                 break
@@ -107,6 +128,8 @@
                     hook_new_tip = None
                 hook(hook_local, hook_master, old_revno, old_tip, new_revno,
                      hook_new_tip)
+            if branch.supports_tags() and not keep_tags:
+                remove_tags(branch, graph, old_tip, new_revision_id)
             if tree is not None:
                 if not _mod_revision.is_null(new_revision_id):
                     parents = [new_revision_id]

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2010-04-21 11:27:04 +0000
+++ bzrlib/upgrade.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2008, 2009, 2010 Canonical Ltd
+# Copyright (C) 2005, 2006, 2008-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,20 +17,44 @@
 """bzr upgrade logic."""
 
 
-from bzrlib.bzrdir import BzrDir, format_registry
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    trace,
+    ui,
+    urlutils,
+    )
+from bzrlib.controldir import (
+    ControlDir,
+    format_registry,
+    )
+from bzrlib.i18n import gettext
 from bzrlib.remote import RemoteBzrDir
-import bzrlib.ui as ui
 
 
 class Convert(object):
 
-    def __init__(self, url, format=None):
+    def __init__(self, url=None, format=None, control_dir=None):
+        """Convert a Bazaar control directory to a given format.
+
+        Either the url or control_dir parameter must be given.
+
+        :param url: the URL of the control directory or None if the
+          control_dir is explicitly given instead
+        :param format: the format to convert to or None for the default
+        :param control_dir: the control directory or None if it is
+          specified via the URL parameter instead
+        """
         self.format = format
-        self.bzrdir = BzrDir.open_unsupported(url)
         # XXX: Change to cleanup
         warning_id = 'cross_format_fetch'
         saved_warning = warning_id in ui.ui_factory.suppressed_warnings
+        if url is None and control_dir is None:
+            raise AssertionError(
+                "either the url or control_dir parameter must be set.")
+        if control_dir is not None:
+            self.bzrdir = control_dir
+        else:
+            self.bzrdir = ControlDir.open_unsupported(url)
         if isinstance(self.bzrdir, RemoteBzrDir):
             self.bzrdir._ensure_real()
             self.bzrdir = self.bzrdir._real_bzrdir
@@ -48,9 +72,10 @@
         try:
             branch = self.bzrdir.open_branch()
             if branch.user_url != self.bzrdir.user_url:
-                ui.ui_factory.note("This is a checkout. The branch (%s) needs to be "
-                             "upgraded separately." %
-                             branch.user_url)
+                ui.ui_factory.note(gettext(
+                    'This is a checkout. The branch (%s) needs to be upgraded'
+                    ' separately.') % (urlutils.unescape_for_display(
+                        branch.user_url, 'utf-8')))
             del branch
         except (errors.NotBranchError, errors.IncompatibleRepositories):
             # might not be a format we can open without upgrading; see e.g.
@@ -71,18 +96,211 @@
         if not self.bzrdir.needs_format_conversion(format):
             raise errors.UpToDateFormat(self.bzrdir._format)
         if not self.bzrdir.can_convert_format():
-            raise errors.BzrError("cannot upgrade from bzrdir format %s" %
+            raise errors.BzrError(gettext("cannot upgrade from bzrdir format %s") %
                            self.bzrdir._format)
         self.bzrdir.check_conversion_target(format)
-        ui.ui_factory.note('starting upgrade of %s' % self.transport.base)
+        ui.ui_factory.note(gettext('starting upgrade of %s') % 
+            urlutils.unescape_for_display(self.transport.base, 'utf-8'))
 
-        self.bzrdir.backup_bzrdir()
+        self.backup_oldpath, self.backup_newpath = self.bzrdir.backup_bzrdir()
         while self.bzrdir.needs_format_conversion(format):
             converter = self.bzrdir._format.get_converter(format)
             self.bzrdir = converter.convert(self.bzrdir, None)
-        ui.ui_factory.note("finished")
-
-
-def upgrade(url, format=None):
-    """Upgrade to format, or the default bzrdir format if not supplied."""
-    Convert(url, format)
+        ui.ui_factory.note(gettext('finished'))
+
+    def clean_up(self):
+        """Clean-up after a conversion.
+
+        This removes the backup.bzr directory.
+        """
+        transport = self.transport
+        backup_relpath = transport.relpath(self.backup_newpath)
+        child_pb = ui.ui_factory.nested_progress_bar()
+        child_pb.update(gettext('Deleting backup.bzr'))
+        try:
+            transport.delete_tree(backup_relpath)
+        finally:
+            child_pb.finished()
+
+
+def upgrade(url, format=None, clean_up=False, dry_run=False):
+    """Upgrade locations to format.
+ 
+    This routine wraps the smart_upgrade() routine with a nicer UI.
+    In particular, it ensures all URLs can be opened before starting
+    and reports a summary at the end if more than one upgrade was attempted.
+    This routine is useful for command line tools. Other bzrlib clients
+    probably ought to use smart_upgrade() instead.
+
+    :param url: a URL of the locations to upgrade.
+    :param format: the format to convert to or None for the best default
+    :param clean-up: if True, the backup.bzr directory is removed if the
+      upgrade succeeded for a given repo/branch/tree
+    :param dry_run: show what would happen but don't actually do any upgrades
+    :return: the list of exceptions encountered
+    """
+    control_dirs = [ControlDir.open_unsupported(url)]
+    attempted, succeeded, exceptions = smart_upgrade(control_dirs,
+        format, clean_up=clean_up, dry_run=dry_run)
+    if len(attempted) > 1:
+        attempted_count = len(attempted)
+        succeeded_count = len(succeeded)
+        failed_count = attempted_count - succeeded_count
+        ui.ui_factory.note(
+            gettext('\nSUMMARY: {0} upgrades attempted, {1} succeeded,'\
+                    ' {2} failed').format(
+                     attempted_count, succeeded_count, failed_count))
+    return exceptions
+
+
+def smart_upgrade(control_dirs, format, clean_up=False,
+    dry_run=False):
+    """Convert control directories to a new format intelligently.
+
+    If the control directory is a shared repository, dependent branches
+    are also converted provided the repository converted successfully.
+    If the conversion of a branch fails, remaining branches are still tried.
+
+    :param control_dirs: the BzrDirs to upgrade
+    :param format: the format to convert to or None for the best default
+    :param clean_up: if True, the backup.bzr directory is removed if the
+      upgrade succeeded for a given repo/branch/tree
+    :param dry_run: show what would happen but don't actually do any upgrades
+    :return: attempted-control-dirs, succeeded-control-dirs, exceptions
+    """
+    all_attempted = []
+    all_succeeded = []
+    all_exceptions = []
+    for control_dir in control_dirs:
+        attempted, succeeded, exceptions = _smart_upgrade_one(control_dir,
+            format, clean_up=clean_up, dry_run=dry_run)
+        all_attempted.extend(attempted)
+        all_succeeded.extend(succeeded)
+        all_exceptions.extend(exceptions)
+    return all_attempted, all_succeeded, all_exceptions
+
+
+def _smart_upgrade_one(control_dir, format, clean_up=False,
+    dry_run=False):
+    """Convert a control directory to a new format intelligently.
+
+    See smart_upgrade for parameter details.
+    """
+    # If the URL is a shared repository, find the dependent branches
+    dependents = None
+    try:
+        repo = control_dir.open_repository()
+    except errors.NoRepositoryPresent:
+        # A branch or checkout using a shared repository higher up
+        pass
+    else:
+        # The URL is a repository. If it successfully upgrades,
+        # then upgrade the dependent branches as well.
+        if repo.is_shared():
+            dependents = repo.find_branches(using=True)
+
+    # Do the conversions
+    attempted = [control_dir]
+    succeeded, exceptions = _convert_items([control_dir], format, clean_up,
+                                           dry_run)
+    if succeeded and dependents:
+        ui.ui_factory.note(gettext('Found %d dependent branches - upgrading ...')
+                           % (len(dependents),))
+        # Convert dependent branches
+        branch_cdirs = [b.bzrdir for b in dependents]
+        successes, problems = _convert_items(branch_cdirs, format, clean_up,
+            dry_run, label="branch")
+        attempted.extend(branch_cdirs)
+        succeeded.extend(successes)
+        exceptions.extend(problems)
+
+    # Return the result
+    return attempted, succeeded, exceptions
+
+# FIXME: There are several problems below:
+# - RemoteRepository doesn't support _unsupported (really ?)
+# - raising AssertionError is rude and may not be necessary
+# - no tests
+# - the only caller uses only the label
+def _get_object_and_label(control_dir):
+    """Return the primary object and type label for a control directory.
+
+    :return: object, label where:
+      * object is a Branch, Repository or WorkingTree and
+      * label is one of:
+        * branch            - a branch
+        * repository        - a repository
+        * tree              - a lightweight checkout
+    """
+    try:
+        try:
+            br = control_dir.open_branch(unsupported=True,
+                                         ignore_fallbacks=True)
+        except NotImplementedError:
+            # RemoteRepository doesn't support the unsupported parameter
+            br = control_dir.open_branch(ignore_fallbacks=True)
+    except errors.NotBranchError:
+        pass
+    else:
+        return br, "branch"
+    try:
+        repo = control_dir.open_repository()
+    except errors.NoRepositoryPresent:
+        pass
+    else:
+        return repo, "repository"
+    try:
+        wt = control_dir.open_workingtree()
+    except (errors.NoWorkingTree, errors.NotLocalUrl):
+        pass
+    else:
+        return wt, "tree"
+    raise AssertionError("unknown type of control directory %s", control_dir)
+
+
+def _convert_items(items, format, clean_up, dry_run, label=None):
+    """Convert a sequence of control directories to the given format.
+ 
+    :param items: the control directories to upgrade
+    :param format: the format to convert to or None for the best default
+    :param clean-up: if True, the backup.bzr directory is removed if the
+      upgrade succeeded for a given repo/branch/tree
+    :param dry_run: show what would happen but don't actually do any upgrades
+    :param label: the label for these items or None to calculate one
+    :return: items successfully upgraded, exceptions
+    """
+    succeeded = []
+    exceptions = []
+    child_pb = ui.ui_factory.nested_progress_bar()
+    child_pb.update(gettext('Upgrading bzrdirs'), 0, len(items))
+    for i, control_dir in enumerate(items):
+        # Do the conversion
+        location = control_dir.root_transport.base
+        bzr_object, bzr_label = _get_object_and_label(control_dir)
+        type_label = label or bzr_label
+        child_pb.update(gettext("Upgrading %s") % (type_label), i+1, len(items))
+        ui.ui_factory.note(gettext('Upgrading {0} {1} ...').format(type_label, 
+            urlutils.unescape_for_display(location, 'utf-8'),))
+        try:
+            if not dry_run:
+                cv = Convert(control_dir=control_dir, format=format)
+        except Exception, ex:
+            trace.warning('conversion error: %s' % ex)
+            exceptions.append(ex)
+            continue
+
+        # Do any required post processing
+        succeeded.append(control_dir)
+        if clean_up:
+            try:
+                ui.ui_factory.note(gettext('Removing backup ...'))
+                if not dry_run:
+                    cv.clean_up()
+            except Exception, ex:
+                trace.warning(gettext('failed to clean-up {0}: {1}') % (location, ex))
+                exceptions.append(ex)
+
+    child_pb.finished()
+
+    # Return the result
+    return succeeded, exceptions

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2010-06-15 04:32:49 +0000
+++ bzrlib/urlutils.py	2011-10-06 07:43:13 +0000
@@ -22,7 +22,7 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from posixpath import split as _posix_split, normpath as _posix_normpath
+from posixpath import split as _posix_split
 import urllib
 import urlparse
 
@@ -181,6 +181,7 @@
 # jam 20060502 Sorted to 'l' because the final target is 'local_path_from_url'
 def _posix_local_path_from_url(url):
     """Convert a url like file:///path/to/foo into /path/to/foo"""
+    url = split_segment_parameters_raw(url)[0]
     file_localhost_prefix = 'file://localhost/'
     if url.startswith(file_localhost_prefix):
         path = url[len(file_localhost_prefix) - 1:]
@@ -200,8 +201,7 @@
     """
     # importing directly from posixpath allows us to test this
     # on non-posix platforms
-    return 'file://' + escape(_posix_normpath(
-        osutils._posix_abspath(path)))
+    return 'file://' + escape(osutils._posix_abspath(path))
 
 
 def _win32_local_path_from_url(url):
@@ -209,6 +209,7 @@
     if not url.startswith('file://'):
         raise errors.InvalidURL(url, 'local urls must start with file:///, '
                                      'UNC path urls must start with file://')
+    url = split_segment_parameters_raw(url)[0]
     # We strip off all 3 slashes
     win32_url = url[len('file:'):]
     # check for UNC path: //HOST/path
@@ -224,8 +225,9 @@
         return '/'
 
     # usual local path with drive letter
-    if (win32_url[3] not in ('abcdefghijklmnopqrstuvwxyz'
-                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
+    if (len(win32_url) < 6
+        or win32_url[3] not in ('abcdefghijklmnopqrstuvwxyz'
+                                'ABCDEFGHIJKLMNOPQRSTUVWXYZ')
         or win32_url[4] not in  '|:'
         or win32_url[5] != '/'):
         raise errors.InvalidURL(url, 'Win32 file urls start with'
@@ -385,7 +387,7 @@
     """On win32 the drive letter needs to be added to the url base."""
     # Strip off the drive letter
     # path is currently /C:/foo
-    if len(path) < 3 or path[2] not in ':|' or path[3] != '/':
+    if len(path) < 4 or path[2] not in ':|' or path[3] != '/':
         raise errors.InvalidURL(url_base + path,
             'win32 file:/// paths need a drive letter')
     url_base += path[0:3] # file:// + /C:
@@ -729,6 +731,148 @@
     return osutils.pathjoin(*segments)
 
 
+class URL(object):
+    """Parsed URL."""
+
+    def __init__(self, scheme, quoted_user, quoted_password, quoted_host,
+            port, quoted_path):
+        self.scheme = scheme
+        self.quoted_host = quoted_host
+        self.host = urllib.unquote(self.quoted_host)
+        self.quoted_user = quoted_user
+        if self.quoted_user is not None:
+            self.user = urllib.unquote(self.quoted_user)
+        else:
+            self.user = None
+        self.quoted_password = quoted_password
+        if self.quoted_password is not None:
+            self.password = urllib.unquote(self.quoted_password)
+        else:
+            self.password = None
+        self.port = port
+        self.quoted_path = quoted_path
+        self.path = urllib.unquote(self.quoted_path)
+
+    def __eq__(self, other):
+        return (isinstance(other, self.__class__) and
+                self.scheme == other.scheme and
+                self.host == other.host and
+                self.user == other.user and
+                self.password == other.password and
+                self.path == other.path)
+
+    def __repr__(self):
+        return "<%s(%r, %r, %r, %r, %r, %r)>" % (
+            self.__class__.__name__,
+            self.scheme, self.quoted_user, self.quoted_password,
+            self.quoted_host, self.port, self.quoted_path)
+
+    @classmethod
+    def from_string(cls, url):
+        """Create a URL object from a string.
+
+        :param url: URL as bytestring
+        """
+        if isinstance(url, unicode):
+            raise errors.InvalidURL('should be ascii:\n%r' % url)
+        url = url.encode('utf-8')
+        (scheme, netloc, path, params,
+         query, fragment) = urlparse.urlparse(url, allow_fragments=False)
+        user = password = host = port = None
+        if '@' in netloc:
+            user, host = netloc.rsplit('@', 1)
+            if ':' in user:
+                user, password = user.split(':', 1)
+        else:
+            host = netloc
+
+        if ':' in host and not (host[0] == '[' and host[-1] == ']'):
+            # there *is* port
+            host, port = host.rsplit(':',1)
+            try:
+                port = int(port)
+            except ValueError:
+                raise errors.InvalidURL('invalid port number %s in url:\n%s' %
+                                        (port, url))
+        if host != "" and host[0] == '[' and host[-1] == ']': #IPv6
+            host = host[1:-1]
+
+        return cls(scheme, user, password, host, port, path)
+
+    def __str__(self):
+        netloc = self.quoted_host
+        if ":" in netloc:
+            netloc = "[%s]" % netloc
+        if self.quoted_user is not None:
+            # Note that we don't put the password back even if we
+            # have one so that it doesn't get accidentally
+            # exposed.
+            netloc = '%s@%s' % (self.quoted_user, netloc)
+        if self.port is not None:
+            netloc = '%s:%d' % (netloc, self.port)
+        return urlparse.urlunparse(
+            (self.scheme, netloc, self.quoted_path, None, None, None))
+
+    @staticmethod
+    def _combine_paths(base_path, relpath):
+        """Transform a Transport-relative path to a remote absolute path.
+
+        This does not handle substitution of ~ but does handle '..' and '.'
+        components.
+
+        Examples::
+
+            t._combine_paths('/home/sarah', 'project/foo')
+                => '/home/sarah/project/foo'
+            t._combine_paths('/home/sarah', '../../etc')
+                => '/etc'
+            t._combine_paths('/home/sarah', '/etc')
+                => '/etc'
+
+        :param base_path: base path
+        :param relpath: relative url string for relative part of remote path.
+        :return: urlencoded string for final path.
+        """
+        if not isinstance(relpath, str):
+            raise errors.InvalidURL(relpath)
+        if relpath.startswith('/'):
+            base_parts = []
+        else:
+            base_parts = base_path.split('/')
+        if len(base_parts) > 0 and base_parts[-1] == '':
+            base_parts = base_parts[:-1]
+        for p in relpath.split('/'):
+            if p == '..':
+                if len(base_parts) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                base_parts.pop()
+            elif p == '.':
+                continue # No-op
+            elif p != '':
+                base_parts.append(p)
+        path = '/'.join(base_parts)
+        if not path.startswith('/'):
+            path = '/' + path
+        return path
+
+    def clone(self, offset=None):
+        """Return a new URL for a path relative to this URL.
+
+        :param offset: A relative path, already urlencoded
+        :return: `URL` instance
+        """
+        if offset is not None:
+            relative = unescape(offset).encode('utf-8')
+            path = self._combine_paths(self.path, relative)
+            path = urllib.quote(path)
+        else:
+            path = self.quoted_path
+        return self.__class__(self.scheme, self.quoted_user,
+                self.quoted_password, self.quoted_host, self.port,
+                path)
+
 
 def parse_url(url):
     """Extract the server address, the credentials and the path from the url.
@@ -737,36 +881,9 @@
     chars.
 
     :param url: an quoted url
-
     :return: (scheme, user, password, host, port, path) tuple, all fields
         are unquoted.
     """
-    if isinstance(url, unicode):
-        raise errors.InvalidURL('should be ascii:\n%r' % url)
-    url = url.encode('utf-8')
-    (scheme, netloc, path, params,
-     query, fragment) = urlparse.urlparse(url, allow_fragments=False)
-    user = password = host = port = None
-    if '@' in netloc:
-        user, host = netloc.rsplit('@', 1)
-        if ':' in user:
-            user, password = user.split(':', 1)
-            password = urllib.unquote(password)
-        user = urllib.unquote(user)
-    else:
-        host = netloc
-
-    if ':' in host and not (host[0] == '[' and host[-1] == ']'): #there *is* port
-        host, port = host.rsplit(':',1)
-        try:
-            port = int(port)
-        except ValueError:
-            raise errors.InvalidURL('invalid port number %s in url:\n%s' %
-                                    (port, url))
-    if host != "" and host[0] == '[' and host[-1] == ']': #IPv6
-        host = host[1:-1]
-
-    host = urllib.unquote(host)
-    path = urllib.unquote(path)
-
-    return (scheme, user, password, host, port, path)
+    parsed_url = URL.from_string(url)
+    return (parsed_url.scheme, parsed_url.user, parsed_url.password,
+        parsed_url.host, parsed_url.port, parsed_url.path)

=== added file 'bzrlib/utextwrap.py'
--- bzrlib/utextwrap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/utextwrap.py	2011-05-14 14:32:02 +0000
@@ -0,0 +1,264 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# UTextWrapper._handle_long_word, UTextWrapper._wrap_chunks,
+# UTextWrapper._fix_sentence_endings, wrap and fill is copied from Python's
+# textwrap module (under PSF license) and modified for support CJK.
+# Original Copyright for these functions:
+#
+# Copyright (C) 1999-2001 Gregory P. Ward.
+# Copyright (C) 2002, 2003 Python Software Foundation.
+#
+# Written by Greg Ward <gward@python.net>
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import sys
+import textwrap
+from unicodedata import east_asian_width as _eawidth
+
+from bzrlib import osutils
+
+__all__ = ["UTextWrapper", "fill", "wrap"]
+
+class UTextWrapper(textwrap.TextWrapper):
+    """
+    Extend TextWrapper for Unicode.
+
+    This textwrapper handles east asian double width and split word
+    even if !break_long_words when word contains double width
+    characters.
+
+    :param ambiguous_width: (keyword argument) width for character when
+                            unicodedata.east_asian_width(c) == 'A'
+                            (default: 1)
+
+    Limitations:
+    * expand_tabs doesn't fixed. It uses len() for calculating width
+      of string on left of TAB.
+    * Handles one codeunit as a single character having 1 or 2 width.
+      This is not correct when there are surrogate pairs, combined
+      characters or zero-width characters.
+    * Treats all asian character are line breakable. But it is not
+      true because line breaking is prohibited around some characters.
+      (For example, breaking before punctation mark is prohibited.)
+      See UAX # 14 "UNICODE LINE BREAKING ALGORITHM"
+    """
+
+    def __init__(self, width=None, **kwargs):
+        if width is None:
+            width = (osutils.terminal_width() or
+                        osutils.default_terminal_width) - 1
+
+        ambi_width = kwargs.pop('ambiguous_width', 1)
+        if ambi_width == 1:
+            self._east_asian_doublewidth = 'FW'
+        elif ambi_width == 2:
+            self._east_asian_doublewidth = 'FWA'
+        else:
+            raise ValueError("ambiguous_width should be 1 or 2")
+
+        # No drop_whitespace param before Python 2.6 it was always dropped
+        if sys.version_info < (2, 6):
+            self.drop_whitespace = kwargs.pop("drop_whitespace", True)
+            if not self.drop_whitespace:
+                raise ValueError("TextWrapper version must drop whitespace")
+        textwrap.TextWrapper.__init__(self, width, **kwargs)
+
+    def _unicode_char_width(self, uc):
+        """Return width of character `uc`.
+
+        :param:     uc      Single unicode character.
+        """
+        # 'A' means width of the character is not be able to determine.
+        # We assume that it's width is 2 because longer wrap may over
+        # terminal width but shorter wrap may be acceptable.
+        return (_eawidth(uc) in self._east_asian_doublewidth and 2) or 1
+
+    def _width(self, s):
+        """Returns width for s.
+
+        When s is unicode, take care of east asian width.
+        When s is bytes, treat all byte is single width character.
+        """
+        charwidth = self._unicode_char_width
+        return sum(charwidth(c) for c in s)
+
+    def _cut(self, s, width):
+        """Returns head and rest of s. (head+rest == s)
+
+        Head is large as long as _width(head) <= width.
+        """
+        w = 0
+        charwidth = self._unicode_char_width
+        for pos, c in enumerate(s):
+            w += charwidth(c)
+            if w > width:
+                return s[:pos], s[pos:]
+        return s, u''
+
+    def _fix_sentence_endings(self, chunks):
+        """_fix_sentence_endings(chunks : [string])
+
+        Correct for sentence endings buried in 'chunks'.  Eg. when the
+        original text contains "... foo.\nBar ...", munge_whitespace()
+        and split() will convert that to [..., "foo.", " ", "Bar", ...]
+        which has one too few spaces; this method simply changes the one
+        space to two.
+
+        Note: This function is copied from textwrap.TextWrap and modified
+        to use unicode always.
+        """
+        i = 0
+        L = len(chunks)-1
+        patsearch = self.sentence_end_re.search
+        while i < L:
+            if chunks[i+1] == u" " and patsearch(chunks[i]):
+                chunks[i+1] = u"  "
+                i += 2
+            else:
+                i += 1
+
+    def _handle_long_word(self, chunks, cur_line, cur_len, width):
+        # Figure out when indent is larger than the specified width, and make
+        # sure at least one character is stripped off on every pass
+        if width < 2:
+            space_left = chunks[-1] and self._width(chunks[-1][0]) or 1
+        else:
+            space_left = width - cur_len
+
+        # If we're allowed to break long words, then do so: put as much
+        # of the next chunk onto the current line as will fit.
+        if self.break_long_words:
+            head, rest = self._cut(chunks[-1], space_left)
+            cur_line.append(head)
+            if rest:
+                chunks[-1] = rest
+            else:
+                del chunks[-1]
+
+        # Otherwise, we have to preserve the long word intact.  Only add
+        # it to the current line if there's nothing already there --
+        # that minimizes how much we violate the width constraint.
+        elif not cur_line:
+            cur_line.append(chunks.pop())
+
+        # If we're not allowed to break long words, and there's already
+        # text on the current line, do nothing.  Next time through the
+        # main loop of _wrap_chunks(), we'll wind up here again, but
+        # cur_len will be zero, so the next line will be entirely
+        # devoted to the long word that we can't handle right now.
+
+    def _wrap_chunks(self, chunks):
+        lines = []
+        if self.width <= 0:
+            raise ValueError("invalid width %r (must be > 0)" % self.width)
+
+        # Arrange in reverse order so items can be efficiently popped
+        # from a stack of chucks.
+        chunks.reverse()
+
+        while chunks:
+
+            # Start the list of chunks that will make up the current line.
+            # cur_len is just the length of all the chunks in cur_line.
+            cur_line = []
+            cur_len = 0
+
+            # Figure out which static string will prefix this line.
+            if lines:
+                indent = self.subsequent_indent
+            else:
+                indent = self.initial_indent
+
+            # Maximum width for this line.
+            width = self.width - len(indent)
+
+            # First chunk on line is whitespace -- drop it, unless this
+            # is the very beginning of the text (ie. no lines started yet).
+            if self.drop_whitespace and chunks[-1].strip() == '' and lines:
+                del chunks[-1]
+
+            while chunks:
+                # Use _width instead of len for east asian width
+                l = self._width(chunks[-1])
+
+                # Can at least squeeze this chunk onto the current line.
+                if cur_len + l <= width:
+                    cur_line.append(chunks.pop())
+                    cur_len += l
+
+                # Nope, this line is full.
+                else:
+                    break
+
+            # The current line is full, and the next chunk is too big to
+            # fit on *any* line (not just this one).
+            if chunks and self._width(chunks[-1]) > width:
+                self._handle_long_word(chunks, cur_line, cur_len, width)
+
+            # If the last chunk on this line is all whitespace, drop it.
+            if self.drop_whitespace and cur_line and not cur_line[-1].strip():
+                del cur_line[-1]
+
+            # Convert current line back to a string and store it in list
+            # of all lines (return value).
+            if cur_line:
+                lines.append(indent + u''.join(cur_line))
+
+        return lines
+
+    def _split(self, text):
+        chunks = textwrap.TextWrapper._split(self, unicode(text))
+        cjk_split_chunks = []
+        for chunk in chunks:
+            prev_pos = 0
+            for pos, char in enumerate(chunk):
+                if self._unicode_char_width(char) == 2:
+                    if prev_pos < pos:
+                        cjk_split_chunks.append(chunk[prev_pos:pos])
+                    cjk_split_chunks.append(char)
+                    prev_pos = pos+1
+            if prev_pos < len(chunk):
+                cjk_split_chunks.append(chunk[prev_pos:])
+        return cjk_split_chunks
+
+    def wrap(self, text):
+        # ensure text is unicode
+        return textwrap.TextWrapper.wrap(self, unicode(text))
+
+# -- Convenience interface ---------------------------------------------
+
+def wrap(text, width=None, **kwargs):
+    """Wrap a single paragraph of text, returning a list of wrapped lines.
+
+    Reformat the single paragraph in 'text' so it fits in lines of no
+    more than 'width' columns, and return a list of wrapped lines.  By
+    default, tabs in 'text' are expanded with string.expandtabs(), and
+    all other whitespace characters (including newline) are converted to
+    space.  See TextWrapper class for available keyword args to customize
+    wrapping behaviour.
+    """
+    return UTextWrapper(width=width, **kwargs).wrap(text)
+
+def fill(text, width=None, **kwargs):
+    """Fill a single paragraph of text, returning a new string.
+
+    Reformat the single paragraph in 'text' to fit in lines of no more
+    than 'width' columns, and return a new string containing the entire
+    wrapped paragraph.  As with wrap(), tabs are expanded and other
+    whitespace characters converted to space.  See TextWrapper class for
+    available keyword args to customize wrapping behaviour.
+    """
+    return UTextWrapper(width=width, **kwargs).fill(text)
+

=== removed file 'bzrlib/util/bencode.py'
--- bzrlib/util/bencode.py	2009-06-23 22:31:59 +0000
+++ bzrlib/util/bencode.py	1970-01-01 00:00:00 +0000
@@ -1,24 +0,0 @@
-# Copyright (C) 2009 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-from bzrlib import symbol_versioning
-
-dep_warning = symbol_versioning.deprecated_in((1, 16, 0)) % (
-    'bzrlib.util.bencode',) + '\n  Use bzrlib.bencode instead'
-
-symbol_versioning.warn(dep_warning, DeprecationWarning, stacklevel=2)
-
-from bzrlib.bencode import *

=== modified file 'bzrlib/util/configobj/configobj.py'
--- bzrlib/util/configobj/configobj.py	2009-04-17 22:24:54 +0000
+++ bzrlib/util/configobj/configobj.py	2011-02-22 09:17:40 +0000
@@ -1794,10 +1794,12 @@
     def _get_triple_quote(self, value):
         if (value.find('"""') != -1) and (value.find("'''") != -1):
             raise ConfigObjError('Value "%s" cannot be safely quoted.' % value)
+        # upstream version (up to version 4.7.2) has the bug with incorrect quoting;
+        # fixed in our copy based on the suggestion of ConfigObj's author
         if value.find('"""') == -1:
+            quot = tsquot
+        else:
             quot = tdquot
-        else:
-            quot = tsquot
         return quot
 
 

=== modified file 'bzrlib/version.py'
--- bzrlib/version.py	2010-04-30 11:03:59 +0000
+++ bzrlib/version.py	2011-10-11 12:01:51 +0000
@@ -21,8 +21,8 @@
 
 import bzrlib
 from bzrlib import (
-    bzrdir,
     config,
+    controldir,
     errors,
     osutils,
     trace,
@@ -73,7 +73,7 @@
     else:
         to_file.write(bzrlib.__path__[0] + '\n')
     if show_config:
-        config_dir = os.path.normpath(config.config_dir())  # use native slashes
+        config_dir = osutils.normpath(config.config_dir())  # use native slashes
         if not isinstance(config_dir, unicode):
             config_dir = config_dir.decode(osutils.get_user_encoding())
         to_file.write("  Bazaar configuration: %s\n" % (config_dir,))
@@ -98,7 +98,7 @@
     If bzr is not being run from its working tree, returns None.
     """
     try:
-        control = bzrdir.BzrDir.open_containing(__file__)[0]
+        control = controldir.ControlDir.open_containing(__file__)[0]
         return control.open_workingtree(recommend_upgrade=False)
     except (errors.NotBranchError, errors.UnknownFormatError,
             errors.NoWorkingTree):

=== modified file 'bzrlib/version_info_formats/__init__.py'
--- bzrlib/version_info_formats/__init__.py	2010-07-15 13:41:31 +0000
+++ bzrlib/version_info_formats/__init__.py	2011-10-14 15:45:10 +0000
@@ -19,7 +19,10 @@
 import time
 
 from bzrlib.osutils import local_time_offset, format_date
-from bzrlib import registry
+from bzrlib import (
+    registry,
+    revision as _mod_revision,
+    )
 
 
 def create_date_str(timestamp=None, offset=None):
@@ -45,7 +48,8 @@
                 check_for_clean=False,
                 include_revision_history=False,
                 include_file_revisions=False,
-                template=None):
+                template=None,
+                revision_id=None):
         """Build up information about the given branch.
         If working_tree is given, it can be checked for changes.
 
@@ -61,6 +65,7 @@
             include the explicit last-changed revision for each file.
         :param template: Template for the output formatting, not used by
             all builders.
+        :param revision_id: Revision id to print version for (optional)
         """
         self._branch = branch
         self._working_tree = working_tree
@@ -71,7 +76,7 @@
 
         self._clean = None
         self._file_revisions = {}
-        self._revision_history_info= []
+        self._revision_id = revision_id
 
     def _extract_file_revisions(self):
         """Extract the working revisions for all files"""
@@ -130,35 +135,41 @@
             if self._working_tree is not None:
                 self._working_tree.unlock()
 
-    def _extract_revision_history(self):
+    def _iter_revision_history(self):
         """Find the messages for all revisions in history."""
 
         # Unfortunately, there is no WorkingTree.revision_history
-        rev_hist = self._branch.revision_history()
         if self._working_tree is not None:
             last_rev = self._working_tree.last_revision()
-            if last_rev not in rev_hist:
-                raise AssertionError(
-                    "Working Tree's last revision not in branch.revision_history")
-            rev_hist = rev_hist[:rev_hist.index(last_rev)+1]
+        else:
+            last_rev = self._branch.last_revision()
 
         repository =  self._branch.repository
         repository.lock_read()
         try:
-            for revision_id in rev_hist:
+            graph = repository.get_graph()
+            revhistory = list(graph.iter_lefthand_ancestry(
+                last_rev, [_mod_revision.NULL_REVISION]))
+            for revision_id in reversed(revhistory):
                 rev = repository.get_revision(revision_id)
-                self._revision_history_info.append(
-                    (rev.revision_id, rev.message,
-                     rev.timestamp, rev.timezone))
+                yield (rev.revision_id, rev.message,
+                       rev.timestamp, rev.timezone)
         finally:
             repository.unlock()
 
     def _get_revision_id(self):
         """Get the revision id we are working on."""
+        if self._revision_id is not None:
+            return self._revision_id
         if self._working_tree is not None:
             return self._working_tree.last_revision()
         return self._branch.last_revision()
 
+    def _get_revno_str(self, revision_id):
+        numbers = self._branch.revision_id_to_dotted_revno(revision_id)
+        revno_str = '.'.join([str(num) for num in numbers])
+        return revno_str
+
     def generate(self, to_file):
         """Output the version information to the supplied file.
 

=== modified file 'bzrlib/version_info_formats/format_custom.py'
--- bzrlib/version_info_formats/format_custom.py	2009-04-03 21:23:55 +0000
+++ bzrlib/version_info_formats/format_custom.py	2011-06-12 20:09:55 +0000
@@ -93,7 +93,7 @@
         if revision_id == NULL_REVISION:
             info.add('revno', 0)
         else:
-            info.add('revno', self._branch.revision_id_to_revno(revision_id))
+            info.add('revno', self._get_revno_str(revision_id))
             info.add('revision_id', revision_id)
             rev = self._branch.repository.get_revision(revision_id)
             info.add('date', create_date_str(rev.timestamp, rev.timezone))

=== modified file 'bzrlib/version_info_formats/format_python.py'
--- bzrlib/version_info_formats/format_python.py	2009-04-03 21:23:55 +0000
+++ bzrlib/version_info_formats/format_python.py	2011-10-14 15:45:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2009, 2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,9 +38,8 @@
 
 
 _py_version_footer = '''
-
 if __name__ == '__main__':
-    print 'revision: %(revno)d' % version_info
+    print 'revision: %(revno)s' % version_info
     print 'nick: %(branch_nick)s' % version_info
     print 'revision id: %(revision_id)s' % version_info
 '''
@@ -61,9 +60,9 @@
 
         revision_id = self._get_revision_id()
         if revision_id == NULL_REVISION:
-            info['revno'] = 0
+            info['revno'] = '0'
         else:
-            info['revno'] = self._branch.revision_id_to_revno(revision_id)
+            info['revno'] = self._get_revno_str(revision_id)
             info['revision_id'] = revision_id
             rev = self._branch.repository.get_revision(revision_id)
             info['date'] = create_date_str(rev.timestamp, rev.timezone)
@@ -84,8 +83,8 @@
         to_file.write('\n\n')
 
         if self._include_history:
-            self._extract_revision_history()
-            revision_str = pprint.pformat(self._revision_history_info)
+            history = list(self._iter_revision_history())
+            revision_str = pprint.pformat(history)
             to_file.write('revisions = ')
             to_file.write(revision_str)
             to_file.write('\n\n')

=== modified file 'bzrlib/version_info_formats/format_rio.py'
--- bzrlib/version_info_formats/format_rio.py	2010-02-17 17:11:16 +0000
+++ bzrlib/version_info_formats/format_rio.py	2011-10-14 15:45:10 +0000
@@ -38,7 +38,7 @@
             info.add('revision-id', revision_id)
             rev = self._branch.repository.get_revision(revision_id)
             info.add('date', create_date_str(rev.timestamp, rev.timezone))
-            revno = str(self._branch.revision_id_to_revno(revision_id))
+            revno = self._get_revno_str(revision_id)
             for hook in RioVersionInfoBuilder.hooks['revision']:
                 hook(rev, info)
         else:
@@ -60,10 +60,9 @@
                 info.add('clean', 'False')
 
         if self._include_history:
-            self._extract_revision_history()
             log = Stanza()
             for (revision_id, message,
-                 timestamp, timezone) in self._revision_history_info:
+                 timestamp, timezone) in self._iter_revision_history():
                 log.add('id', revision_id)
                 log.add('message', message)
                 log.add('date', create_date_str(timestamp, timezone))
@@ -84,10 +83,12 @@
     """Hooks for rio-formatted version-info output."""
 
     def __init__(self):
-        super(RioVersionInfoBuilderHooks, self).__init__()
-        self.create_hook(hooks.HookPoint('revision',
+        super(RioVersionInfoBuilderHooks, self).__init__(
+            "bzrlib.version_info_formats.format_rio", "RioVersionInfoBuilder.hooks")
+        self.add_hook('revision',
             "Invoked when adding information about a revision to the"
             " RIO stanza that is printed. revision is called with a"
-            " revision object and a RIO stanza.", (1, 15), None))
+            " revision object and a RIO stanza.", (1, 15))
+
 
 RioVersionInfoBuilder.hooks = RioVersionInfoBuilderHooks()

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2010-05-14 12:55:13 +0000
+++ bzrlib/versionedfile.py	2011-05-16 10:08:01 +0000
@@ -1,7 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
-#
-# Authors:
-#   Johan Rydberg <jrydberg@gnu.org>
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,6 +28,7 @@
 
 from bzrlib import (
     annotate,
+    bencode,
     errors,
     graph as _mod_graph,
     groupcompress,
@@ -40,15 +38,10 @@
     multiparent,
     tsort,
     revision,
-    ui,
     )
-from bzrlib.graph import DictParentsProvider, Graph, StackedParentsProvider
-from bzrlib.transport.memory import MemoryTransport
 """)
 from bzrlib.registry import Registry
-from bzrlib.symbol_versioning import *
 from bzrlib.textmerge import TextMerge
-from bzrlib import bencode
 
 
 adapter_registry = Registry()
@@ -206,6 +199,138 @@
             yield record
 
 
+class _MPDiffGenerator(object):
+    """Pull out the functionality for generating mp_diffs."""
+
+    def __init__(self, vf, keys):
+        self.vf = vf
+        # This is the order the keys were requested in
+        self.ordered_keys = tuple(keys)
+        # keys + their parents, what we need to compute the diffs
+        self.needed_keys = ()
+        # Map from key: mp_diff
+        self.diffs = {}
+        # Map from key: parents_needed (may have ghosts)
+        self.parent_map = {}
+        # Parents that aren't present
+        self.ghost_parents = ()
+        # Map from parent_key => number of children for this text
+        self.refcounts = {}
+        # Content chunks that are cached while we still need them
+        self.chunks = {}
+
+    def _find_needed_keys(self):
+        """Find the set of keys we need to request.
+
+        This includes all the original keys passed in, and the non-ghost
+        parents of those keys.
+
+        :return: (needed_keys, refcounts)
+            needed_keys is the set of all texts we need to extract
+            refcounts is a dict of {key: num_children} letting us know when we
+                no longer need to cache a given parent text
+        """
+        # All the keys and their parents
+        needed_keys = set(self.ordered_keys)
+        parent_map = self.vf.get_parent_map(needed_keys)
+        self.parent_map = parent_map
+        # TODO: Should we be using a different construct here? I think this
+        #       uses difference_update internally, and we expect the result to
+        #       be tiny
+        missing_keys = needed_keys.difference(parent_map)
+        if missing_keys:
+            raise errors.RevisionNotPresent(list(missing_keys)[0], self.vf)
+        # Parents that might be missing. They are allowed to be ghosts, but we
+        # should check for them
+        refcounts = {}
+        setdefault = refcounts.setdefault
+        just_parents = set()
+        for child_key, parent_keys in parent_map.iteritems():
+            if not parent_keys:
+                # parent_keys may be None if a given VersionedFile claims to
+                # not support graph operations.
+                continue
+            just_parents.update(parent_keys)
+            needed_keys.update(parent_keys)
+            for p in parent_keys:
+                refcounts[p] = setdefault(p, 0) + 1
+        just_parents.difference_update(parent_map)
+        # Remove any parents that are actually ghosts from the needed set
+        self.present_parents = set(self.vf.get_parent_map(just_parents))
+        self.ghost_parents = just_parents.difference(self.present_parents)
+        needed_keys.difference_update(self.ghost_parents)
+        self.needed_keys = needed_keys
+        self.refcounts = refcounts
+        return needed_keys, refcounts
+
+    def _compute_diff(self, key, parent_lines, lines):
+        """Compute a single mp_diff, and store it in self._diffs"""
+        if len(parent_lines) > 0:
+            # XXX: _extract_blocks is not usefully defined anywhere...
+            #      It was meant to extract the left-parent diff without
+            #      having to recompute it for Knit content (pack-0.92,
+            #      etc). That seems to have regressed somewhere
+            left_parent_blocks = self.vf._extract_blocks(key,
+                parent_lines[0], lines)
+        else:
+            left_parent_blocks = None
+        diff = multiparent.MultiParent.from_lines(lines,
+                    parent_lines, left_parent_blocks)
+        self.diffs[key] = diff
+
+    def _process_one_record(self, key, this_chunks):
+        parent_keys = None
+        if key in self.parent_map:
+            # This record should be ready to diff, since we requested
+            # content in 'topological' order
+            parent_keys = self.parent_map.pop(key)
+            # If a VersionedFile claims 'no-graph' support, then it may return
+            # None for any parent request, so we replace it with an empty tuple
+            if parent_keys is None:
+                parent_keys = ()
+            parent_lines = []
+            for p in parent_keys:
+                # Alternatively we could check p not in self.needed_keys, but
+                # ghost_parents should be tiny versus huge
+                if p in self.ghost_parents:
+                    continue
+                refcount = self.refcounts[p]
+                if refcount == 1: # Last child reference
+                    self.refcounts.pop(p)
+                    parent_chunks = self.chunks.pop(p)
+                else:
+                    self.refcounts[p] = refcount - 1
+                    parent_chunks = self.chunks[p]
+                p_lines = osutils.chunks_to_lines(parent_chunks)
+                # TODO: Should we cache the line form? We did the
+                #       computation to get it, but storing it this way will
+                #       be less memory efficient...
+                parent_lines.append(p_lines)
+                del p_lines
+            lines = osutils.chunks_to_lines(this_chunks)
+            # Since we needed the lines, we'll go ahead and cache them this way
+            this_chunks = lines
+            self._compute_diff(key, parent_lines, lines)
+            del lines
+        # Is this content required for any more children?
+        if key in self.refcounts:
+            self.chunks[key] = this_chunks
+
+    def _extract_diffs(self):
+        needed_keys, refcounts = self._find_needed_keys()
+        for record in self.vf.get_record_stream(needed_keys,
+                                                'topological', True):
+            if record.storage_kind == 'absent':
+                raise errors.RevisionNotPresent(record.key, self.vf)
+            self._process_one_record(record.key,
+                                     record.get_bytes_as('chunked'))
+        
+    def compute_diffs(self):
+        self._extract_diffs()
+        dpop = self.diffs.pop
+        return [dpop(k) for k in self.ordered_keys]
+
+
 class VersionedFile(object):
     """Versioned text file storage.
 
@@ -348,6 +473,10 @@
 
     def make_mpdiffs(self, version_ids):
         """Create multiparent diffs for specified versions."""
+        # XXX: Can't use _MPDiffGenerator just yet. This is because version_ids
+        #      is a list of strings, not keys. And while self.get_record_stream
+        #      is supported, it takes *keys*, while self.get_parent_map() takes
+        #      strings... *sigh*
         knit_versions = set()
         knit_versions.update(version_ids)
         parent_map = self.get_parent_map(version_ids)
@@ -795,6 +924,10 @@
 
     The use of tuples allows a single code base to support several different
     uses with only the mapping logic changing from instance to instance.
+
+    :ivar _immediate_fallback_vfs: For subclasses that support stacking,
+        this is a list of other VersionedFiles immediately underneath this
+        one.  They may in turn each have further fallbacks.
     """
 
     def add_lines(self, key, parents, lines, parent_texts=None,
@@ -839,7 +972,7 @@
     def _add_text(self, key, parents, text, nostore_sha=None, random_id=False):
         """Add a text to the store.
 
-        This is a private function for use by CommitBuilder.
+        This is a private function for use by VersionedFileCommitBuilder.
 
         :param key: The key tuple of the text to add. If the last element is
             None, a CHK string will be generated during the addition.
@@ -1047,49 +1180,30 @@
 
     def make_mpdiffs(self, keys):
         """Create multiparent diffs for specified keys."""
-        keys_order = tuple(keys)
-        keys = frozenset(keys)
-        knit_keys = set(keys)
-        parent_map = self.get_parent_map(keys)
-        for parent_keys in parent_map.itervalues():
-            if parent_keys:
-                knit_keys.update(parent_keys)
-        missing_keys = keys - set(parent_map)
-        if missing_keys:
-            raise errors.RevisionNotPresent(list(missing_keys)[0], self)
-        # We need to filter out ghosts, because we can't diff against them.
-        maybe_ghosts = knit_keys - keys
-        ghosts = maybe_ghosts - set(self.get_parent_map(maybe_ghosts))
-        knit_keys.difference_update(ghosts)
-        lines = {}
-        chunks_to_lines = osutils.chunks_to_lines
-        for record in self.get_record_stream(knit_keys, 'topological', True):
-            lines[record.key] = chunks_to_lines(record.get_bytes_as('chunked'))
-            # line_block_dict = {}
-            # for parent, blocks in record.extract_line_blocks():
-            #   line_blocks[parent] = blocks
-            # line_blocks[record.key] = line_block_dict
-        diffs = []
-        for key in keys_order:
-            target = lines[key]
-            parents = parent_map[key] or []
-            # Note that filtering knit_keys can lead to a parent difference
-            # between the creation and the application of the mpdiff.
-            parent_lines = [lines[p] for p in parents if p in knit_keys]
-            if len(parent_lines) > 0:
-                left_parent_blocks = self._extract_blocks(key, parent_lines[0],
-                    target)
-            else:
-                left_parent_blocks = None
-            diffs.append(multiparent.MultiParent.from_lines(target,
-                parent_lines, left_parent_blocks))
-        return diffs
+        generator = _MPDiffGenerator(self, keys)
+        return generator.compute_diffs()
+
+    def get_annotator(self):
+        return annotate.Annotator(self)
 
     missing_keys = index._missing_keys_from_parent_map
 
     def _extract_blocks(self, version_id, source, target):
         return None
 
+    def _transitive_fallbacks(self):
+        """Return the whole stack of fallback versionedfiles.
+
+        This VersionedFiles may have a list of fallbacks, but it doesn't
+        necessarily know about the whole stack going down, and it can't know
+        at open time because they may change after the objects are opened.
+        """
+        all_fallbacks = []
+        for a_vfs in self._immediate_fallback_vfs:
+            all_fallbacks.append(a_vfs)
+            all_fallbacks.extend(a_vfs._transitive_fallbacks())
+        return all_fallbacks
+
 
 class ThunkedVersionedFiles(VersionedFiles):
     """Storage for many versioned files thunked onto a 'VersionedFile' class.
@@ -1159,9 +1273,6 @@
             result.append((prefix + (origin,), line))
         return result
 
-    def get_annotator(self):
-        return annotate.Annotator(self)
-
     def check(self, progress_bar=None, keys=None):
         """See VersionedFiles.check()."""
         # XXX: This is over-enthusiastic but as we only thunk for Weaves today
@@ -1311,6 +1422,33 @@
         return result
 
 
+class VersionedFilesWithFallbacks(VersionedFiles):
+
+    def without_fallbacks(self):
+        """Return a clone of this object without any fallbacks configured."""
+        raise NotImplementedError(self.without_fallbacks)
+
+    def add_fallback_versioned_files(self, a_versioned_files):
+        """Add a source of texts for texts not present in this knit.
+
+        :param a_versioned_files: A VersionedFiles object.
+        """
+        raise NotImplementedError(self.add_fallback_versioned_files)
+
+    def get_known_graph_ancestry(self, keys):
+        """Get a KnownGraph instance with the ancestry of keys."""
+        parent_map, missing_keys = self._index.find_ancestry(keys)
+        for fallback in self._transitive_fallbacks():
+            if not missing_keys:
+                break
+            (f_parent_map, f_missing_keys) = fallback._index.find_ancestry(
+                                                missing_keys)
+            parent_map.update(f_parent_map)
+            missing_keys = f_missing_keys
+        kg = _mod_graph.KnownGraph(parent_map)
+        return kg
+
+
 class _PlanMergeVersionedFile(VersionedFiles):
     """A VersionedFile for uncommitted and committed texts.
 
@@ -1337,7 +1475,7 @@
         # line data for locally held keys.
         self._lines = {}
         # key lookup providers
-        self._providers = [DictParentsProvider(self._parents)]
+        self._providers = [_mod_graph.DictParentsProvider(self._parents)]
 
     def plan_merge(self, ver_a, ver_b, base=None):
         """See VersionedFile.plan_merge"""
@@ -1350,7 +1488,7 @@
 
     def plan_lca_merge(self, ver_a, ver_b, base=None):
         from bzrlib.merge import _PlanLCAMerge
-        graph = Graph(self)
+        graph = _mod_graph.Graph(self)
         new_plan = _PlanLCAMerge(ver_a, ver_b, self, (self._file_id,), graph).plan_merge()
         if base is None:
             return new_plan
@@ -1408,7 +1546,8 @@
             result[revision.NULL_REVISION] = ()
         self._providers = self._providers[:1] + self.fallback_versionedfiles
         result.update(
-            StackedParentsProvider(self._providers).get_parent_map(keys))
+            _mod_graph.StackedParentsProvider(
+                self._providers).get_parent_map(keys))
         for key, parents in result.iteritems():
             if parents == ():
                 result[key] = (revision.NULL_REVISION,)
@@ -1624,6 +1763,46 @@
                 yield (l, key)
 
 
+class NoDupeAddLinesDecorator(object):
+    """Decorator for a VersionedFiles that skips doing an add_lines if the key
+    is already present.
+    """
+
+    def __init__(self, store):
+        self._store = store
+
+    def add_lines(self, key, parents, lines, parent_texts=None,
+            left_matching_blocks=None, nostore_sha=None, random_id=False,
+            check_content=True):
+        """See VersionedFiles.add_lines.
+        
+        This implementation may return None as the third element of the return
+        value when the original store wouldn't.
+        """
+        if nostore_sha:
+            raise NotImplementedError(
+                "NoDupeAddLinesDecorator.add_lines does not implement the "
+                "nostore_sha behaviour.")
+        if key[-1] is None:
+            sha1 = osutils.sha_strings(lines)
+            key = ("sha1:" + sha1,)
+        else:
+            sha1 = None
+        if key in self._store.get_parent_map([key]):
+            # This key has already been inserted, so don't do it again.
+            if sha1 is None:
+                sha1 = osutils.sha_strings(lines)
+            return sha1, sum(map(len, lines)), None
+        return self._store.add_lines(key, parents, lines,
+                parent_texts=parent_texts,
+                left_matching_blocks=left_matching_blocks,
+                nostore_sha=nostore_sha, random_id=random_id,
+                check_content=check_content)
+
+    def __getattr__(self, name):
+        return getattr(self._store, name)
+
+
 def network_bytes_to_kind_and_offset(network_bytes):
     """Strip of a record kind from the front of network_bytes.
 
@@ -1720,3 +1899,64 @@
     for prefix in sorted(per_prefix_map):
         present_keys.extend(reversed(tsort.topo_sort(per_prefix_map[prefix])))
     return present_keys
+
+
+class _KeyRefs(object):
+
+    def __init__(self, track_new_keys=False):
+        # dict mapping 'key' to 'set of keys referring to that key'
+        self.refs = {}
+        if track_new_keys:
+            # set remembering all new keys
+            self.new_keys = set()
+        else:
+            self.new_keys = None
+
+    def clear(self):
+        if self.refs:
+            self.refs.clear()
+        if self.new_keys:
+            self.new_keys.clear()
+
+    def add_references(self, key, refs):
+        # Record the new references
+        for referenced in refs:
+            try:
+                needed_by = self.refs[referenced]
+            except KeyError:
+                needed_by = self.refs[referenced] = set()
+            needed_by.add(key)
+        # Discard references satisfied by the new key
+        self.add_key(key)
+
+    def get_new_keys(self):
+        return self.new_keys
+    
+    def get_unsatisfied_refs(self):
+        return self.refs.iterkeys()
+
+    def _satisfy_refs_for_key(self, key):
+        try:
+            del self.refs[key]
+        except KeyError:
+            # No keys depended on this key.  That's ok.
+            pass
+
+    def add_key(self, key):
+        # satisfy refs for key, and remember that we've seen this key.
+        self._satisfy_refs_for_key(key)
+        if self.new_keys is not None:
+            self.new_keys.add(key)
+
+    def satisfy_refs_for_keys(self, keys):
+        for key in keys:
+            self._satisfy_refs_for_key(key)
+
+    def get_referrers(self):
+        result = set()
+        for referrers in self.refs.itervalues():
+            result.update(referrers)
+        return result
+
+
+

=== added file 'bzrlib/vf_repository.py'
--- bzrlib/vf_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/vf_repository.py	2011-10-11 12:01:51 +0000
@@ -0,0 +1,3129 @@
+# Copyright (C) 2005-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Repository formats built around versioned files."""
+
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import itertools
+
+from bzrlib import (
+    check,
+    debug,
+    fetch as _mod_fetch,
+    fifo_cache,
+    gpg,
+    graph,
+    inventory_delta,
+    lru_cache,
+    osutils,
+    revision as _mod_revision,
+    serializer as _mod_serializer,
+    static_tuple,
+    symbol_versioning,
+    tsort,
+    ui,
+    versionedfile,
+    )
+
+from bzrlib.recordcounter import RecordCounter
+from bzrlib.revisiontree import InventoryRevisionTree
+from bzrlib.testament import Testament
+from bzrlib.i18n import gettext
+""")
+
+from bzrlib import (
+    errors,
+    )
+from bzrlib.decorators import (
+    needs_read_lock,
+    needs_write_lock,
+    only_raises,
+    )
+from bzrlib.inventory import (
+    Inventory,
+    InventoryDirectory,
+    ROOT_ID,
+    entry_factory,
+    )
+
+from bzrlib.repository import (
+    CommitBuilder,
+    InterRepository,
+    MetaDirRepository,
+    MetaDirRepositoryFormat,
+    Repository,
+    RepositoryFormat,
+    )
+
+from bzrlib.trace import (
+    mutter
+    )
+
+
+class VersionedFileRepositoryFormat(RepositoryFormat):
+    """Base class for all repository formats that are VersionedFiles-based."""
+
+    supports_full_versioned_files = True
+    supports_versioned_directories = True
+
+    # Should commit add an inventory, or an inventory delta to the repository.
+    _commit_inv_deltas = True
+    # What order should fetch operations request streams in?
+    # The default is unordered as that is the cheapest for an origin to
+    # provide.
+    _fetch_order = 'unordered'
+    # Does this repository format use deltas that can be fetched as-deltas ?
+    # (E.g. knits, where the knit deltas can be transplanted intact.
+    # We default to False, which will ensure that enough data to get
+    # a full text out of any fetch stream will be grabbed.
+    _fetch_uses_deltas = False
+
+
+class VersionedFileCommitBuilder(CommitBuilder):
+    """Commit builder implementation for versioned files based repositories.
+    """
+
+    # this commit builder supports the record_entry_contents interface
+    supports_record_entry_contents = True
+
+    # the default CommitBuilder does not manage trees whose root is versioned.
+    _versioned_root = False
+
+    def __init__(self, repository, parents, config, timestamp=None,
+                 timezone=None, committer=None, revprops=None,
+                 revision_id=None, lossy=False):
+        super(VersionedFileCommitBuilder, self).__init__(repository,
+            parents, config, timestamp, timezone, committer, revprops,
+            revision_id, lossy)
+        try:
+            basis_id = self.parents[0]
+        except IndexError:
+            basis_id = _mod_revision.NULL_REVISION
+        self.basis_delta_revision = basis_id
+        self.new_inventory = Inventory(None)
+        self._basis_delta = []
+        self.__heads = graph.HeadsCache(repository.get_graph()).heads
+        # memo'd check for no-op commits.
+        self._any_changes = False
+        # API compatibility, older code that used CommitBuilder did not call
+        # .record_delete(), which means the delta that is computed would not be
+        # valid. Callers that will call record_delete() should call
+        # .will_record_deletes() to indicate that.
+        self._recording_deletes = False
+
+    def will_record_deletes(self):
+        """Tell the commit builder that deletes are being notified.
+
+        This enables the accumulation of an inventory delta; for the resulting
+        commit to be valid, deletes against the basis MUST be recorded via
+        builder.record_delete().
+        """
+        self._recording_deletes = True
+
+    def any_changes(self):
+        """Return True if any entries were changed.
+
+        This includes merge-only changes. It is the core for the --unchanged
+        detection in commit.
+
+        :return: True if any changes have occured.
+        """
+        return self._any_changes
+
+    def _ensure_fallback_inventories(self):
+        """Ensure that appropriate inventories are available.
+
+        This only applies to repositories that are stacked, and is about
+        enusring the stacking invariants. Namely, that for any revision that is
+        present, we either have all of the file content, or we have the parent
+        inventory and the delta file content.
+        """
+        if not self.repository._fallback_repositories:
+            return
+        if not self.repository._format.supports_chks:
+            raise errors.BzrError("Cannot commit directly to a stacked branch"
+                " in pre-2a formats. See "
+                "https://bugs.launchpad.net/bzr/+bug/375013 for details.")
+        # This is a stacked repo, we need to make sure we have the parent
+        # inventories for the parents.
+        parent_keys = [(p,) for p in self.parents]
+        parent_map = self.repository.inventories._index.get_parent_map(parent_keys)
+        missing_parent_keys = set([pk for pk in parent_keys
+                                       if pk not in parent_map])
+        fallback_repos = list(reversed(self.repository._fallback_repositories))
+        missing_keys = [('inventories', pk[0])
+                        for pk in missing_parent_keys]
+        resume_tokens = []
+        while missing_keys and fallback_repos:
+            fallback_repo = fallback_repos.pop()
+            source = fallback_repo._get_source(self.repository._format)
+            sink = self.repository._get_sink()
+            stream = source.get_stream_for_missing_keys(missing_keys)
+            missing_keys = sink.insert_stream_without_locking(stream,
+                self.repository._format)
+        if missing_keys:
+            raise errors.BzrError('Unable to fill in parent inventories for a'
+                                  ' stacked branch')
+
+    def commit(self, message):
+        """Make the actual commit.
+
+        :return: The revision id of the recorded revision.
+        """
+        self._validate_unicode_text(message, 'commit message')
+        rev = _mod_revision.Revision(
+                       timestamp=self._timestamp,
+                       timezone=self._timezone,
+                       committer=self._committer,
+                       message=message,
+                       inventory_sha1=self.inv_sha1,
+                       revision_id=self._new_revision_id,
+                       properties=self._revprops)
+        rev.parent_ids = self.parents
+        self.repository.add_revision(self._new_revision_id, rev,
+            self.new_inventory, self._config)
+        self._ensure_fallback_inventories()
+        self.repository.commit_write_group()
+        return self._new_revision_id
+
+    def abort(self):
+        """Abort the commit that is being built.
+        """
+        self.repository.abort_write_group()
+
+    def revision_tree(self):
+        """Return the tree that was just committed.
+
+        After calling commit() this can be called to get a
+        RevisionTree representing the newly committed tree. This is
+        preferred to calling Repository.revision_tree() because that may
+        require deserializing the inventory, while we already have a copy in
+        memory.
+        """
+        if self.new_inventory is None:
+            self.new_inventory = self.repository.get_inventory(
+                self._new_revision_id)
+        return InventoryRevisionTree(self.repository, self.new_inventory,
+            self._new_revision_id)
+
+    def finish_inventory(self):
+        """Tell the builder that the inventory is finished.
+
+        :return: The inventory id in the repository, which can be used with
+            repository.get_inventory.
+        """
+        if self.new_inventory is None:
+            # an inventory delta was accumulated without creating a new
+            # inventory.
+            basis_id = self.basis_delta_revision
+            # We ignore the 'inventory' returned by add_inventory_by_delta
+            # because self.new_inventory is used to hint to the rest of the
+            # system what code path was taken
+            self.inv_sha1, _ = self.repository.add_inventory_by_delta(
+                basis_id, self._basis_delta, self._new_revision_id,
+                self.parents)
+        else:
+            if self.new_inventory.root is None:
+                raise AssertionError('Root entry should be supplied to'
+                    ' record_entry_contents, as of bzr 0.10.')
+                self.new_inventory.add(InventoryDirectory(ROOT_ID, '', None))
+            self.new_inventory.revision_id = self._new_revision_id
+            self.inv_sha1 = self.repository.add_inventory(
+                self._new_revision_id,
+                self.new_inventory,
+                self.parents
+                )
+        return self._new_revision_id
+
+    def _check_root(self, ie, parent_invs, tree):
+        """Helper for record_entry_contents.
+
+        :param ie: An entry being added.
+        :param parent_invs: The inventories of the parent revisions of the
+            commit.
+        :param tree: The tree that is being committed.
+        """
+        # In this revision format, root entries have no knit or weave When
+        # serializing out to disk and back in root.revision is always
+        # _new_revision_id
+        ie.revision = self._new_revision_id
+
+    def _require_root_change(self, tree):
+        """Enforce an appropriate root object change.
+
+        This is called once when record_iter_changes is called, if and only if
+        the root was not in the delta calculated by record_iter_changes.
+
+        :param tree: The tree which is being committed.
+        """
+        if len(self.parents) == 0:
+            raise errors.RootMissing()
+        entry = entry_factory['directory'](tree.path2id(''), '',
+            None)
+        entry.revision = self._new_revision_id
+        self._basis_delta.append(('', '', entry.file_id, entry))
+
+    def _get_delta(self, ie, basis_inv, path):
+        """Get a delta against the basis inventory for ie."""
+        if not basis_inv.has_id(ie.file_id):
+            # add
+            result = (None, path, ie.file_id, ie)
+            self._basis_delta.append(result)
+            return result
+        elif ie != basis_inv[ie.file_id]:
+            # common but altered
+            # TODO: avoid tis id2path call.
+            result = (basis_inv.id2path(ie.file_id), path, ie.file_id, ie)
+            self._basis_delta.append(result)
+            return result
+        else:
+            # common, unaltered
+            return None
+
+    def _heads(self, file_id, revision_ids):
+        """Calculate the graph heads for revision_ids in the graph of file_id.
+
+        This can use either a per-file graph or a global revision graph as we
+        have an identity relationship between the two graphs.
+        """
+        return self.__heads(revision_ids)
+
+    def get_basis_delta(self):
+        """Return the complete inventory delta versus the basis inventory.
+
+        This has been built up with the calls to record_delete and
+        record_entry_contents. The client must have already called
+        will_record_deletes() to indicate that they will be generating a
+        complete delta.
+
+        :return: An inventory delta, suitable for use with apply_delta, or
+            Repository.add_inventory_by_delta, etc.
+        """
+        if not self._recording_deletes:
+            raise AssertionError("recording deletes not activated.")
+        return self._basis_delta
+
+    def record_delete(self, path, file_id):
+        """Record that a delete occured against a basis tree.
+
+        This is an optional API - when used it adds items to the basis_delta
+        being accumulated by the commit builder. It cannot be called unless the
+        method will_record_deletes() has been called to inform the builder that
+        a delta is being supplied.
+
+        :param path: The path of the thing deleted.
+        :param file_id: The file id that was deleted.
+        """
+        if not self._recording_deletes:
+            raise AssertionError("recording deletes not activated.")
+        delta = (path, None, file_id, None)
+        self._basis_delta.append(delta)
+        self._any_changes = True
+        return delta
+
+    def record_entry_contents(self, ie, parent_invs, path, tree,
+        content_summary):
+        """Record the content of ie from tree into the commit if needed.
+
+        Side effect: sets ie.revision when unchanged
+
+        :param ie: An inventory entry present in the commit.
+        :param parent_invs: The inventories of the parent revisions of the
+            commit.
+        :param path: The path the entry is at in the tree.
+        :param tree: The tree which contains this entry and should be used to
+            obtain content.
+        :param content_summary: Summary data from the tree about the paths
+            content - stat, length, exec, sha/link target. This is only
+            accessed when the entry has a revision of None - that is when it is
+            a candidate to commit.
+        :return: A tuple (change_delta, version_recorded, fs_hash).
+            change_delta is an inventory_delta change for this entry against
+            the basis tree of the commit, or None if no change occured against
+            the basis tree.
+            version_recorded is True if a new version of the entry has been
+            recorded. For instance, committing a merge where a file was only
+            changed on the other side will return (delta, False).
+            fs_hash is either None, or the hash details for the path (currently
+            a tuple of the contents sha1 and the statvalue returned by
+            tree.get_file_with_stat()).
+        """
+        if self.new_inventory.root is None:
+            if ie.parent_id is not None:
+                raise errors.RootMissing()
+            self._check_root(ie, parent_invs, tree)
+        if ie.revision is None:
+            kind = content_summary[0]
+        else:
+            # ie is carried over from a prior commit
+            kind = ie.kind
+        # XXX: repository specific check for nested tree support goes here - if
+        # the repo doesn't want nested trees we skip it ?
+        if (kind == 'tree-reference' and
+            not self.repository._format.supports_tree_reference):
+            # mismatch between commit builder logic and repository:
+            # this needs the entry creation pushed down into the builder.
+            raise NotImplementedError('Missing repository subtree support.')
+        self.new_inventory.add(ie)
+
+        # TODO: slow, take it out of the inner loop.
+        try:
+            basis_inv = parent_invs[0]
+        except IndexError:
+            basis_inv = Inventory(root_id=None)
+
+        # ie.revision is always None if the InventoryEntry is considered
+        # for committing. We may record the previous parents revision if the
+        # content is actually unchanged against a sole head.
+        if ie.revision is not None:
+            if not self._versioned_root and path == '':
+                # repositories that do not version the root set the root's
+                # revision to the new commit even when no change occurs (more
+                # specifically, they do not record a revision on the root; and
+                # the rev id is assigned to the root during deserialisation -
+                # this masks when a change may have occurred against the basis.
+                # To match this we always issue a delta, because the revision
+                # of the root will always be changing.
+                if basis_inv.has_id(ie.file_id):
+                    delta = (basis_inv.id2path(ie.file_id), path,
+                        ie.file_id, ie)
+                else:
+                    # add
+                    delta = (None, path, ie.file_id, ie)
+                self._basis_delta.append(delta)
+                return delta, False, None
+            else:
+                # we don't need to commit this, because the caller already
+                # determined that an existing revision of this file is
+                # appropriate. If it's not being considered for committing then
+                # it and all its parents to the root must be unaltered so
+                # no-change against the basis.
+                if ie.revision == self._new_revision_id:
+                    raise AssertionError("Impossible situation, a skipped "
+                        "inventory entry (%r) claims to be modified in this "
+                        "commit (%r).", (ie, self._new_revision_id))
+                return None, False, None
+        # XXX: Friction: parent_candidates should return a list not a dict
+        #      so that we don't have to walk the inventories again.
+        parent_candidate_entries = ie.parent_candidates(parent_invs)
+        head_set = self._heads(ie.file_id, parent_candidate_entries.keys())
+        heads = []
+        for inv in parent_invs:
+            if inv.has_id(ie.file_id):
+                old_rev = inv[ie.file_id].revision
+                if old_rev in head_set:
+                    heads.append(inv[ie.file_id].revision)
+                    head_set.remove(inv[ie.file_id].revision)
+
+        store = False
+        # now we check to see if we need to write a new record to the
+        # file-graph.
+        # We write a new entry unless there is one head to the ancestors, and
+        # the kind-derived content is unchanged.
+
+        # Cheapest check first: no ancestors, or more the one head in the
+        # ancestors, we write a new node.
+        if len(heads) != 1:
+            store = True
+        if not store:
+            # There is a single head, look it up for comparison
+            parent_entry = parent_candidate_entries[heads[0]]
+            # if the non-content specific data has changed, we'll be writing a
+            # node:
+            if (parent_entry.parent_id != ie.parent_id or
+                parent_entry.name != ie.name):
+                store = True
+        # now we need to do content specific checks:
+        if not store:
+            # if the kind changed the content obviously has
+            if kind != parent_entry.kind:
+                store = True
+        # Stat cache fingerprint feedback for the caller - None as we usually
+        # don't generate one.
+        fingerprint = None
+        if kind == 'file':
+            if content_summary[2] is None:
+                raise ValueError("Files must not have executable = None")
+            if not store:
+                # We can't trust a check of the file length because of content
+                # filtering...
+                if (# if the exec bit has changed we have to store:
+                    parent_entry.executable != content_summary[2]):
+                    store = True
+                elif parent_entry.text_sha1 == content_summary[3]:
+                    # all meta and content is unchanged (using a hash cache
+                    # hit to check the sha)
+                    ie.revision = parent_entry.revision
+                    ie.text_size = parent_entry.text_size
+                    ie.text_sha1 = parent_entry.text_sha1
+                    ie.executable = parent_entry.executable
+                    return self._get_delta(ie, basis_inv, path), False, None
+                else:
+                    # Either there is only a hash change(no hash cache entry,
+                    # or same size content change), or there is no change on
+                    # this file at all.
+                    # Provide the parent's hash to the store layer, so that the
+                    # content is unchanged we will not store a new node.
+                    nostore_sha = parent_entry.text_sha1
+            if store:
+                # We want to record a new node regardless of the presence or
+                # absence of a content change in the file.
+                nostore_sha = None
+            ie.executable = content_summary[2]
+            file_obj, stat_value = tree.get_file_with_stat(ie.file_id, path)
+            try:
+                text = file_obj.read()
+            finally:
+                file_obj.close()
+            try:
+                ie.text_sha1, ie.text_size = self._add_text_to_weave(
+                    ie.file_id, text, heads, nostore_sha)
+                # Let the caller know we generated a stat fingerprint.
+                fingerprint = (ie.text_sha1, stat_value)
+            except errors.ExistingContent:
+                # Turns out that the file content was unchanged, and we were
+                # only going to store a new node if it was changed. Carry over
+                # the entry.
+                ie.revision = parent_entry.revision
+                ie.text_size = parent_entry.text_size
+                ie.text_sha1 = parent_entry.text_sha1
+                ie.executable = parent_entry.executable
+                return self._get_delta(ie, basis_inv, path), False, None
+        elif kind == 'directory':
+            if not store:
+                # all data is meta here, nothing specific to directory, so
+                # carry over:
+                ie.revision = parent_entry.revision
+                return self._get_delta(ie, basis_inv, path), False, None
+            self._add_text_to_weave(ie.file_id, '', heads, None)
+        elif kind == 'symlink':
+            current_link_target = content_summary[3]
+            if not store:
+                # symlink target is not generic metadata, check if it has
+                # changed.
+                if current_link_target != parent_entry.symlink_target:
+                    store = True
+            if not store:
+                # unchanged, carry over.
+                ie.revision = parent_entry.revision
+                ie.symlink_target = parent_entry.symlink_target
+                return self._get_delta(ie, basis_inv, path), False, None
+            ie.symlink_target = current_link_target
+            self._add_text_to_weave(ie.file_id, '', heads, None)
+        elif kind == 'tree-reference':
+            if not store:
+                if content_summary[3] != parent_entry.reference_revision:
+                    store = True
+            if not store:
+                # unchanged, carry over.
+                ie.reference_revision = parent_entry.reference_revision
+                ie.revision = parent_entry.revision
+                return self._get_delta(ie, basis_inv, path), False, None
+            ie.reference_revision = content_summary[3]
+            if ie.reference_revision is None:
+                raise AssertionError("invalid content_summary for nested tree: %r"
+                    % (content_summary,))
+            self._add_text_to_weave(ie.file_id, '', heads, None)
+        else:
+            raise NotImplementedError('unknown kind')
+        ie.revision = self._new_revision_id
+        # The initial commit adds a root directory, but this in itself is not
+        # a worthwhile commit.
+        if (self.basis_delta_revision != _mod_revision.NULL_REVISION or
+            path != ""):
+            self._any_changes = True
+        return self._get_delta(ie, basis_inv, path), True, fingerprint
+
+    def record_iter_changes(self, tree, basis_revision_id, iter_changes,
+        _entry_factory=entry_factory):
+        """Record a new tree via iter_changes.
+
+        :param tree: The tree to obtain text contents from for changed objects.
+        :param basis_revision_id: The revision id of the tree the iter_changes
+            has been generated against. Currently assumed to be the same
+            as self.parents[0] - if it is not, errors may occur.
+        :param iter_changes: An iter_changes iterator with the changes to apply
+            to basis_revision_id. The iterator must not include any items with
+            a current kind of None - missing items must be either filtered out
+            or errored-on before record_iter_changes sees the item.
+        :param _entry_factory: Private method to bind entry_factory locally for
+            performance.
+        :return: A generator of (file_id, relpath, fs_hash) tuples for use with
+            tree._observed_sha1.
+        """
+        # Create an inventory delta based on deltas between all the parents and
+        # deltas between all the parent inventories. We use inventory delta's 
+        # between the inventory objects because iter_changes masks
+        # last-changed-field only changes.
+        # Working data:
+        # file_id -> change map, change is fileid, paths, changed, versioneds,
+        # parents, names, kinds, executables
+        merged_ids = {}
+        # {file_id -> revision_id -> inventory entry, for entries in parent
+        # trees that are not parents[0]
+        parent_entries = {}
+        ghost_basis = False
+        try:
+            revtrees = list(self.repository.revision_trees(self.parents))
+        except errors.NoSuchRevision:
+            # one or more ghosts, slow path.
+            revtrees = []
+            for revision_id in self.parents:
+                try:
+                    revtrees.append(self.repository.revision_tree(revision_id))
+                except errors.NoSuchRevision:
+                    if not revtrees:
+                        basis_revision_id = _mod_revision.NULL_REVISION
+                        ghost_basis = True
+                    revtrees.append(self.repository.revision_tree(
+                        _mod_revision.NULL_REVISION))
+        # The basis inventory from a repository 
+        if revtrees:
+            basis_inv = revtrees[0].inventory
+        else:
+            basis_inv = self.repository.revision_tree(
+                _mod_revision.NULL_REVISION).inventory
+        if len(self.parents) > 0:
+            if basis_revision_id != self.parents[0] and not ghost_basis:
+                raise Exception(
+                    "arbitrary basis parents not yet supported with merges")
+            for revtree in revtrees[1:]:
+                for change in revtree.inventory._make_delta(basis_inv):
+                    if change[1] is None:
+                        # Not present in this parent.
+                        continue
+                    if change[2] not in merged_ids:
+                        if change[0] is not None:
+                            basis_entry = basis_inv[change[2]]
+                            merged_ids[change[2]] = [
+                                # basis revid
+                                basis_entry.revision,
+                                # new tree revid
+                                change[3].revision]
+                            parent_entries[change[2]] = {
+                                # basis parent
+                                basis_entry.revision:basis_entry,
+                                # this parent 
+                                change[3].revision:change[3],
+                                }
+                        else:
+                            merged_ids[change[2]] = [change[3].revision]
+                            parent_entries[change[2]] = {change[3].revision:change[3]}
+                    else:
+                        merged_ids[change[2]].append(change[3].revision)
+                        parent_entries[change[2]][change[3].revision] = change[3]
+        else:
+            merged_ids = {}
+        # Setup the changes from the tree:
+        # changes maps file_id -> (change, [parent revision_ids])
+        changes= {}
+        for change in iter_changes:
+            # This probably looks up in basis_inv way to much.
+            if change[1][0] is not None:
+                head_candidate = [basis_inv[change[0]].revision]
+            else:
+                head_candidate = []
+            changes[change[0]] = change, merged_ids.get(change[0],
+                head_candidate)
+        unchanged_merged = set(merged_ids) - set(changes)
+        # Extend the changes dict with synthetic changes to record merges of
+        # texts.
+        for file_id in unchanged_merged:
+            # Record a merged version of these items that did not change vs the
+            # basis. This can be either identical parallel changes, or a revert
+            # of a specific file after a merge. The recorded content will be
+            # that of the current tree (which is the same as the basis), but
+            # the per-file graph will reflect a merge.
+            # NB:XXX: We are reconstructing path information we had, this
+            # should be preserved instead.
+            # inv delta  change: (file_id, (path_in_source, path_in_target),
+            #   changed_content, versioned, parent, name, kind,
+            #   executable)
+            try:
+                basis_entry = basis_inv[file_id]
+            except errors.NoSuchId:
+                # a change from basis->some_parents but file_id isn't in basis
+                # so was new in the merge, which means it must have changed
+                # from basis -> current, and as it hasn't the add was reverted
+                # by the user. So we discard this change.
+                pass
+            else:
+                change = (file_id,
+                    (basis_inv.id2path(file_id), tree.id2path(file_id)),
+                    False, (True, True),
+                    (basis_entry.parent_id, basis_entry.parent_id),
+                    (basis_entry.name, basis_entry.name),
+                    (basis_entry.kind, basis_entry.kind),
+                    (basis_entry.executable, basis_entry.executable))
+                changes[file_id] = (change, merged_ids[file_id])
+        # changes contains tuples with the change and a set of inventory
+        # candidates for the file.
+        # inv delta is:
+        # old_path, new_path, file_id, new_inventory_entry
+        seen_root = False # Is the root in the basis delta?
+        inv_delta = self._basis_delta
+        modified_rev = self._new_revision_id
+        for change, head_candidates in changes.values():
+            if change[3][1]: # versioned in target.
+                # Several things may be happening here:
+                # We may have a fork in the per-file graph
+                #  - record a change with the content from tree
+                # We may have a change against < all trees  
+                #  - carry over the tree that hasn't changed
+                # We may have a change against all trees
+                #  - record the change with the content from tree
+                kind = change[6][1]
+                file_id = change[0]
+                entry = _entry_factory[kind](file_id, change[5][1],
+                    change[4][1])
+                head_set = self._heads(change[0], set(head_candidates))
+                heads = []
+                # Preserve ordering.
+                for head_candidate in head_candidates:
+                    if head_candidate in head_set:
+                        heads.append(head_candidate)
+                        head_set.remove(head_candidate)
+                carried_over = False
+                if len(heads) == 1:
+                    # Could be a carry-over situation:
+                    parent_entry_revs = parent_entries.get(file_id, None)
+                    if parent_entry_revs:
+                        parent_entry = parent_entry_revs.get(heads[0], None)
+                    else:
+                        parent_entry = None
+                    if parent_entry is None:
+                        # The parent iter_changes was called against is the one
+                        # that is the per-file head, so any change is relevant
+                        # iter_changes is valid.
+                        carry_over_possible = False
+                    else:
+                        # could be a carry over situation
+                        # A change against the basis may just indicate a merge,
+                        # we need to check the content against the source of the
+                        # merge to determine if it was changed after the merge
+                        # or carried over.
+                        if (parent_entry.kind != entry.kind or
+                            parent_entry.parent_id != entry.parent_id or
+                            parent_entry.name != entry.name):
+                            # Metadata common to all entries has changed
+                            # against per-file parent
+                            carry_over_possible = False
+                        else:
+                            carry_over_possible = True
+                        # per-type checks for changes against the parent_entry
+                        # are done below.
+                else:
+                    # Cannot be a carry-over situation
+                    carry_over_possible = False
+                # Populate the entry in the delta
+                if kind == 'file':
+                    # XXX: There is still a small race here: If someone reverts the content of a file
+                    # after iter_changes examines and decides it has changed,
+                    # we will unconditionally record a new version even if some
+                    # other process reverts it while commit is running (with
+                    # the revert happening after iter_changes did its
+                    # examination).
+                    if change[7][1]:
+                        entry.executable = True
+                    else:
+                        entry.executable = False
+                    if (carry_over_possible and
+                        parent_entry.executable == entry.executable):
+                            # Check the file length, content hash after reading
+                            # the file.
+                            nostore_sha = parent_entry.text_sha1
+                    else:
+                        nostore_sha = None
+                    file_obj, stat_value = tree.get_file_with_stat(file_id, change[1][1])
+                    try:
+                        text = file_obj.read()
+                    finally:
+                        file_obj.close()
+                    try:
+                        entry.text_sha1, entry.text_size = self._add_text_to_weave(
+                            file_id, text, heads, nostore_sha)
+                        yield file_id, change[1][1], (entry.text_sha1, stat_value)
+                    except errors.ExistingContent:
+                        # No content change against a carry_over parent
+                        # Perhaps this should also yield a fs hash update?
+                        carried_over = True
+                        entry.text_size = parent_entry.text_size
+                        entry.text_sha1 = parent_entry.text_sha1
+                elif kind == 'symlink':
+                    # Wants a path hint?
+                    entry.symlink_target = tree.get_symlink_target(file_id)
+                    if (carry_over_possible and
+                        parent_entry.symlink_target == entry.symlink_target):
+                        carried_over = True
+                    else:
+                        self._add_text_to_weave(change[0], '', heads, None)
+                elif kind == 'directory':
+                    if carry_over_possible:
+                        carried_over = True
+                    else:
+                        # Nothing to set on the entry.
+                        # XXX: split into the Root and nonRoot versions.
+                        if change[1][1] != '' or self.repository.supports_rich_root():
+                            self._add_text_to_weave(change[0], '', heads, None)
+                elif kind == 'tree-reference':
+                    if not self.repository._format.supports_tree_reference:
+                        # This isn't quite sane as an error, but we shouldn't
+                        # ever see this code path in practice: tree's don't
+                        # permit references when the repo doesn't support tree
+                        # references.
+                        raise errors.UnsupportedOperation(tree.add_reference,
+                            self.repository)
+                    reference_revision = tree.get_reference_revision(change[0])
+                    entry.reference_revision = reference_revision
+                    if (carry_over_possible and
+                        parent_entry.reference_revision == reference_revision):
+                        carried_over = True
+                    else:
+                        self._add_text_to_weave(change[0], '', heads, None)
+                else:
+                    raise AssertionError('unknown kind %r' % kind)
+                if not carried_over:
+                    entry.revision = modified_rev
+                else:
+                    entry.revision = parent_entry.revision
+            else:
+                entry = None
+            new_path = change[1][1]
+            inv_delta.append((change[1][0], new_path, change[0], entry))
+            if new_path == '':
+                seen_root = True
+        self.new_inventory = None
+        # The initial commit adds a root directory, but this in itself is not
+        # a worthwhile commit.
+        if ((len(inv_delta) > 0 and basis_revision_id != _mod_revision.NULL_REVISION) or
+            (len(inv_delta) > 1 and basis_revision_id == _mod_revision.NULL_REVISION)):
+            # This should perhaps be guarded by a check that the basis we
+            # commit against is the basis for the commit and if not do a delta
+            # against the basis.
+            self._any_changes = True
+        if not seen_root:
+            # housekeeping root entry changes do not affect no-change commits.
+            self._require_root_change(tree)
+        self.basis_delta_revision = basis_revision_id
+
+    def _add_text_to_weave(self, file_id, new_text, parents, nostore_sha):
+        parent_keys = tuple([(file_id, parent) for parent in parents])
+        return self.repository.texts._add_text(
+            (file_id, self._new_revision_id), parent_keys, new_text,
+            nostore_sha=nostore_sha, random_id=self.random_revid)[0:2]
+
+
+class VersionedFileRootCommitBuilder(VersionedFileCommitBuilder):
+    """This commitbuilder actually records the root id"""
+
+    # the root entry gets versioned properly by this builder.
+    _versioned_root = True
+
+    def _check_root(self, ie, parent_invs, tree):
+        """Helper for record_entry_contents.
+
+        :param ie: An entry being added.
+        :param parent_invs: The inventories of the parent revisions of the
+            commit.
+        :param tree: The tree that is being committed.
+        """
+
+    def _require_root_change(self, tree):
+        """Enforce an appropriate root object change.
+
+        This is called once when record_iter_changes is called, if and only if
+        the root was not in the delta calculated by record_iter_changes.
+
+        :param tree: The tree which is being committed.
+        """
+        # versioned roots do not change unless the tree found a change.
+
+
+class VersionedFileRepository(Repository):
+    """Repository holding history for one or more branches.
+
+    The repository holds and retrieves historical information including
+    revisions and file history.  It's normally accessed only by the Branch,
+    which views a particular line of development through that history.
+
+    The Repository builds on top of some byte storage facilies (the revisions,
+    signatures, inventories, texts and chk_bytes attributes) and a Transport,
+    which respectively provide byte storage and a means to access the (possibly
+    remote) disk.
+
+    The byte storage facilities are addressed via tuples, which we refer to
+    as 'keys' throughout the code base. Revision_keys, inventory_keys and
+    signature_keys are all 1-tuples: (revision_id,). text_keys are two-tuples:
+    (file_id, revision_id). chk_bytes uses CHK keys - a 1-tuple with a single
+    byte string made up of a hash identifier and a hash value.
+    We use this interface because it allows low friction with the underlying
+    code that implements disk indices, network encoding and other parts of
+    bzrlib.
+
+    :ivar revisions: A bzrlib.versionedfile.VersionedFiles instance containing
+        the serialised revisions for the repository. This can be used to obtain
+        revision graph information or to access raw serialised revisions.
+        The result of trying to insert data into the repository via this store
+        is undefined: it should be considered read-only except for implementors
+        of repositories.
+    :ivar signatures: A bzrlib.versionedfile.VersionedFiles instance containing
+        the serialised signatures for the repository. This can be used to
+        obtain access to raw serialised signatures.  The result of trying to
+        insert data into the repository via this store is undefined: it should
+        be considered read-only except for implementors of repositories.
+    :ivar inventories: A bzrlib.versionedfile.VersionedFiles instance containing
+        the serialised inventories for the repository. This can be used to
+        obtain unserialised inventories.  The result of trying to insert data
+        into the repository via this store is undefined: it should be
+        considered read-only except for implementors of repositories.
+    :ivar texts: A bzrlib.versionedfile.VersionedFiles instance containing the
+        texts of files and directories for the repository. This can be used to
+        obtain file texts or file graphs. Note that Repository.iter_file_bytes
+        is usually a better interface for accessing file texts.
+        The result of trying to insert data into the repository via this store
+        is undefined: it should be considered read-only except for implementors
+        of repositories.
+    :ivar chk_bytes: A bzrlib.versionedfile.VersionedFiles instance containing
+        any data the repository chooses to store or have indexed by its hash.
+        The result of trying to insert data into the repository via this store
+        is undefined: it should be considered read-only except for implementors
+        of repositories.
+    :ivar _transport: Transport for file access to repository, typically
+        pointing to .bzr/repository.
+    """
+
+    # What class to use for a CommitBuilder. Often it's simpler to change this
+    # in a Repository class subclass rather than to override
+    # get_commit_builder.
+    _commit_builder_class = VersionedFileCommitBuilder
+
+    def add_fallback_repository(self, repository):
+        """Add a repository to use for looking up data not held locally.
+
+        :param repository: A repository.
+        """
+        if not self._format.supports_external_lookups:
+            raise errors.UnstackableRepositoryFormat(self._format, self.base)
+        # This can raise an exception, so should be done before we lock the
+        # fallback repository.
+        self._check_fallback_repository(repository)
+        if self.is_locked():
+            # This repository will call fallback.unlock() when we transition to
+            # the unlocked state, so we make sure to increment the lock count
+            repository.lock_read()
+        self._fallback_repositories.append(repository)
+        self.texts.add_fallback_versioned_files(repository.texts)
+        self.inventories.add_fallback_versioned_files(repository.inventories)
+        self.revisions.add_fallback_versioned_files(repository.revisions)
+        self.signatures.add_fallback_versioned_files(repository.signatures)
+        if self.chk_bytes is not None:
+            self.chk_bytes.add_fallback_versioned_files(repository.chk_bytes)
+
+    @only_raises(errors.LockNotHeld, errors.LockBroken)
+    def unlock(self):
+        super(VersionedFileRepository, self).unlock()
+        if self.control_files._lock_count == 0:
+            self._inventory_entry_cache.clear()
+
+    def add_inventory(self, revision_id, inv, parents):
+        """Add the inventory inv to the repository as revision_id.
+
+        :param parents: The revision ids of the parents that revision_id
+                        is known to have and are in the repository already.
+
+        :returns: The validator(which is a sha1 digest, though what is sha'd is
+            repository format specific) of the serialized inventory.
+        """
+        if not self.is_in_write_group():
+            raise AssertionError("%r not in write group" % (self,))
+        _mod_revision.check_not_reserved_id(revision_id)
+        if not (inv.revision_id is None or inv.revision_id == revision_id):
+            raise AssertionError(
+                "Mismatch between inventory revision"
+                " id and insertion revid (%r, %r)"
+                % (inv.revision_id, revision_id))
+        if inv.root is None:
+            raise errors.RootMissing()
+        return self._add_inventory_checked(revision_id, inv, parents)
+
+    def _add_inventory_checked(self, revision_id, inv, parents):
+        """Add inv to the repository after checking the inputs.
+
+        This function can be overridden to allow different inventory styles.
+
+        :seealso: add_inventory, for the contract.
+        """
+        inv_lines = self._serializer.write_inventory_to_lines(inv)
+        return self._inventory_add_lines(revision_id, parents,
+            inv_lines, check_content=False)
+
+    def add_inventory_by_delta(self, basis_revision_id, delta, new_revision_id,
+                               parents, basis_inv=None, propagate_caches=False):
+        """Add a new inventory expressed as a delta against another revision.
+
+        See the inventory developers documentation for the theory behind
+        inventory deltas.
+
+        :param basis_revision_id: The inventory id the delta was created
+            against. (This does not have to be a direct parent.)
+        :param delta: The inventory delta (see Inventory.apply_delta for
+            details).
+        :param new_revision_id: The revision id that the inventory is being
+            added for.
+        :param parents: The revision ids of the parents that revision_id is
+            known to have and are in the repository already. These are supplied
+            for repositories that depend on the inventory graph for revision
+            graph access, as well as for those that pun ancestry with delta
+            compression.
+        :param basis_inv: The basis inventory if it is already known,
+            otherwise None.
+        :param propagate_caches: If True, the caches for this inventory are
+          copied to and updated for the result if possible.
+
+        :returns: (validator, new_inv)
+            The validator(which is a sha1 digest, though what is sha'd is
+            repository format specific) of the serialized inventory, and the
+            resulting inventory.
+        """
+        if not self.is_in_write_group():
+            raise AssertionError("%r not in write group" % (self,))
+        _mod_revision.check_not_reserved_id(new_revision_id)
+        basis_tree = self.revision_tree(basis_revision_id)
+        basis_tree.lock_read()
+        try:
+            # Note that this mutates the inventory of basis_tree, which not all
+            # inventory implementations may support: A better idiom would be to
+            # return a new inventory, but as there is no revision tree cache in
+            # repository this is safe for now - RBC 20081013
+            if basis_inv is None:
+                basis_inv = basis_tree.inventory
+            basis_inv.apply_delta(delta)
+            basis_inv.revision_id = new_revision_id
+            return (self.add_inventory(new_revision_id, basis_inv, parents),
+                    basis_inv)
+        finally:
+            basis_tree.unlock()
+
+    def _inventory_add_lines(self, revision_id, parents, lines,
+        check_content=True):
+        """Store lines in inv_vf and return the sha1 of the inventory."""
+        parents = [(parent,) for parent in parents]
+        result = self.inventories.add_lines((revision_id,), parents, lines,
+            check_content=check_content)[0]
+        self.inventories._access.flush()
+        return result
+
+    def add_revision(self, revision_id, rev, inv=None, config=None):
+        """Add rev to the revision store as revision_id.
+
+        :param revision_id: the revision id to use.
+        :param rev: The revision object.
+        :param inv: The inventory for the revision. if None, it will be looked
+                    up in the inventory storer
+        :param config: If None no digital signature will be created.
+                       If supplied its signature_needed method will be used
+                       to determine if a signature should be made.
+        """
+        # TODO: jam 20070210 Shouldn't we check rev.revision_id and
+        #       rev.parent_ids?
+        _mod_revision.check_not_reserved_id(revision_id)
+        if config is not None and config.signature_needed():
+            if inv is None:
+                inv = self.get_inventory(revision_id)
+            tree = InventoryRevisionTree(self, inv, revision_id)
+            testament = Testament(rev, tree)
+            plaintext = testament.as_short_text()
+            self.store_revision_signature(
+                gpg.GPGStrategy(config), plaintext, revision_id)
+        # check inventory present
+        if not self.inventories.get_parent_map([(revision_id,)]):
+            if inv is None:
+                raise errors.WeaveRevisionNotPresent(revision_id,
+                                                     self.inventories)
+            else:
+                # yes, this is not suitable for adding with ghosts.
+                rev.inventory_sha1 = self.add_inventory(revision_id, inv,
+                                                        rev.parent_ids)
+        else:
+            key = (revision_id,)
+            rev.inventory_sha1 = self.inventories.get_sha1s([key])[key]
+        self._add_revision(rev)
+
+    def _add_revision(self, revision):
+        text = self._serializer.write_revision_to_string(revision)
+        key = (revision.revision_id,)
+        parents = tuple((parent,) for parent in revision.parent_ids)
+        self.revisions.add_lines(key, parents, osutils.split_lines(text))
+
+    def _check_inventories(self, checker):
+        """Check the inventories found from the revision scan.
+        
+        This is responsible for verifying the sha1 of inventories and
+        creating a pending_keys set that covers data referenced by inventories.
+        """
+        bar = ui.ui_factory.nested_progress_bar()
+        try:
+            self._do_check_inventories(checker, bar)
+        finally:
+            bar.finished()
+
+    def _do_check_inventories(self, checker, bar):
+        """Helper for _check_inventories."""
+        revno = 0
+        keys = {'chk_bytes':set(), 'inventories':set(), 'texts':set()}
+        kinds = ['chk_bytes', 'texts']
+        count = len(checker.pending_keys)
+        bar.update(gettext("inventories"), 0, 2)
+        current_keys = checker.pending_keys
+        checker.pending_keys = {}
+        # Accumulate current checks.
+        for key in current_keys:
+            if key[0] != 'inventories' and key[0] not in kinds:
+                checker._report_items.append('unknown key type %r' % (key,))
+            keys[key[0]].add(key[1:])
+        if keys['inventories']:
+            # NB: output order *should* be roughly sorted - topo or
+            # inverse topo depending on repository - either way decent
+            # to just delta against. However, pre-CHK formats didn't
+            # try to optimise inventory layout on disk. As such the
+            # pre-CHK code path does not use inventory deltas.
+            last_object = None
+            for record in self.inventories.check(keys=keys['inventories']):
+                if record.storage_kind == 'absent':
+                    checker._report_items.append(
+                        'Missing inventory {%s}' % (record.key,))
+                else:
+                    last_object = self._check_record('inventories', record,
+                        checker, last_object,
+                        current_keys[('inventories',) + record.key])
+            del keys['inventories']
+        else:
+            return
+        bar.update(gettext("texts"), 1)
+        while (checker.pending_keys or keys['chk_bytes']
+            or keys['texts']):
+            # Something to check.
+            current_keys = checker.pending_keys
+            checker.pending_keys = {}
+            # Accumulate current checks.
+            for key in current_keys:
+                if key[0] not in kinds:
+                    checker._report_items.append('unknown key type %r' % (key,))
+                keys[key[0]].add(key[1:])
+            # Check the outermost kind only - inventories || chk_bytes || texts
+            for kind in kinds:
+                if keys[kind]:
+                    last_object = None
+                    for record in getattr(self, kind).check(keys=keys[kind]):
+                        if record.storage_kind == 'absent':
+                            checker._report_items.append(
+                                'Missing %s {%s}' % (kind, record.key,))
+                        else:
+                            last_object = self._check_record(kind, record,
+                                checker, last_object, current_keys[(kind,) + record.key])
+                    keys[kind] = set()
+                    break
+
+    def _check_record(self, kind, record, checker, last_object, item_data):
+        """Check a single text from this repository."""
+        if kind == 'inventories':
+            rev_id = record.key[0]
+            inv = self._deserialise_inventory(rev_id,
+                record.get_bytes_as('fulltext'))
+            if last_object is not None:
+                delta = inv._make_delta(last_object)
+                for old_path, path, file_id, ie in delta:
+                    if ie is None:
+                        continue
+                    ie.check(checker, rev_id, inv)
+            else:
+                for path, ie in inv.iter_entries():
+                    ie.check(checker, rev_id, inv)
+            if self._format.fast_deltas:
+                return inv
+        elif kind == 'chk_bytes':
+            # No code written to check chk_bytes for this repo format.
+            checker._report_items.append(
+                'unsupported key type chk_bytes for %s' % (record.key,))
+        elif kind == 'texts':
+            self._check_text(record, checker, item_data)
+        else:
+            checker._report_items.append(
+                'unknown key type %s for %s' % (kind, record.key))
+
+    def _check_text(self, record, checker, item_data):
+        """Check a single text."""
+        # Check it is extractable.
+        # TODO: check length.
+        if record.storage_kind == 'chunked':
+            chunks = record.get_bytes_as(record.storage_kind)
+            sha1 = osutils.sha_strings(chunks)
+            length = sum(map(len, chunks))
+        else:
+            content = record.get_bytes_as('fulltext')
+            sha1 = osutils.sha_string(content)
+            length = len(content)
+        if item_data and sha1 != item_data[1]:
+            checker._report_items.append(
+                'sha1 mismatch: %s has sha1 %s expected %s referenced by %s' %
+                (record.key, sha1, item_data[1], item_data[2]))
+
+    @needs_read_lock
+    def _eliminate_revisions_not_present(self, revision_ids):
+        """Check every revision id in revision_ids to see if we have it.
+
+        Returns a set of the present revisions.
+        """
+        result = []
+        graph = self.get_graph()
+        parent_map = graph.get_parent_map(revision_ids)
+        # The old API returned a list, should this actually be a set?
+        return parent_map.keys()
+
+    def __init__(self, _format, a_bzrdir, control_files):
+        """Instantiate a VersionedFileRepository.
+
+        :param _format: The format of the repository on disk.
+        :param controldir: The ControlDir of the repository.
+        :param control_files: Control files to use for locking, etc.
+        """
+        # In the future we will have a single api for all stores for
+        # getting file texts, inventories and revisions, then
+        # this construct will accept instances of those things.
+        super(VersionedFileRepository, self).__init__(_format, a_bzrdir,
+            control_files)
+        self._transport = control_files._transport
+        self.base = self._transport.base
+        # for tests
+        self._reconcile_does_inventory_gc = True
+        self._reconcile_fixes_text_parents = False
+        self._reconcile_backsup_inventory = True
+        # An InventoryEntry cache, used during deserialization
+        self._inventory_entry_cache = fifo_cache.FIFOCache(10*1024)
+        # Is it safe to return inventory entries directly from the entry cache,
+        # rather copying them?
+        self._safe_to_return_from_cache = False
+
+    @needs_read_lock
+    def gather_stats(self, revid=None, committers=None):
+        """See Repository.gather_stats()."""
+        result = super(VersionedFileRepository, self).gather_stats(revid, committers)
+        # now gather global repository information
+        # XXX: This is available for many repos regardless of listability.
+        if self.user_transport.listable():
+            # XXX: do we want to __define len__() ?
+            # Maybe the versionedfiles object should provide a different
+            # method to get the number of keys.
+            result['revisions'] = len(self.revisions.keys())
+            # result['size'] = t
+        return result
+
+    def get_commit_builder(self, branch, parents, config, timestamp=None,
+                           timezone=None, committer=None, revprops=None,
+                           revision_id=None, lossy=False):
+        """Obtain a CommitBuilder for this repository.
+
+        :param branch: Branch to commit to.
+        :param parents: Revision ids of the parents of the new revision.
+        :param config: Configuration to use.
+        :param timestamp: Optional timestamp recorded for commit.
+        :param timezone: Optional timezone for timestamp.
+        :param committer: Optional committer to set for commit.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.
+        :param lossy: Whether to discard data that can not be natively
+            represented, when pushing to a foreign VCS
+        """
+        if self._fallback_repositories and not self._format.supports_chks:
+            raise errors.BzrError("Cannot commit directly to a stacked branch"
+                " in pre-2a formats. See "
+                "https://bugs.launchpad.net/bzr/+bug/375013 for details.")
+        result = self._commit_builder_class(self, parents, config,
+            timestamp, timezone, committer, revprops, revision_id,
+            lossy)
+        self.start_write_group()
+        return result
+
+    def get_missing_parent_inventories(self, check_for_missing_texts=True):
+        """Return the keys of missing inventory parents for revisions added in
+        this write group.
+
+        A revision is not complete if the inventory delta for that revision
+        cannot be calculated.  Therefore if the parent inventories of a
+        revision are not present, the revision is incomplete, and e.g. cannot
+        be streamed by a smart server.  This method finds missing inventory
+        parents for revisions added in this write group.
+        """
+        if not self._format.supports_external_lookups:
+            # This is only an issue for stacked repositories
+            return set()
+        if not self.is_in_write_group():
+            raise AssertionError('not in a write group')
+
+        # XXX: We assume that every added revision already has its
+        # corresponding inventory, so we only check for parent inventories that
+        # might be missing, rather than all inventories.
+        parents = set(self.revisions._index.get_missing_parents())
+        parents.discard(_mod_revision.NULL_REVISION)
+        unstacked_inventories = self.inventories._index
+        present_inventories = unstacked_inventories.get_parent_map(
+            key[-1:] for key in parents)
+        parents.difference_update(present_inventories)
+        if len(parents) == 0:
+            # No missing parent inventories.
+            return set()
+        if not check_for_missing_texts:
+            return set(('inventories', rev_id) for (rev_id,) in parents)
+        # Ok, now we have a list of missing inventories.  But these only matter
+        # if the inventories that reference them are missing some texts they
+        # appear to introduce.
+        # XXX: Texts referenced by all added inventories need to be present,
+        # but at the moment we're only checking for texts referenced by
+        # inventories at the graph's edge.
+        key_deps = self.revisions._index._key_dependencies
+        key_deps.satisfy_refs_for_keys(present_inventories)
+        referrers = frozenset(r[0] for r in key_deps.get_referrers())
+        file_ids = self.fileids_altered_by_revision_ids(referrers)
+        missing_texts = set()
+        for file_id, version_ids in file_ids.iteritems():
+            missing_texts.update(
+                (file_id, version_id) for version_id in version_ids)
+        present_texts = self.texts.get_parent_map(missing_texts)
+        missing_texts.difference_update(present_texts)
+        if not missing_texts:
+            # No texts are missing, so all revisions and their deltas are
+            # reconstructable.
+            return set()
+        # Alternatively the text versions could be returned as the missing
+        # keys, but this is likely to be less data.
+        missing_keys = set(('inventories', rev_id) for (rev_id,) in parents)
+        return missing_keys
+
+    @needs_read_lock
+    def has_revisions(self, revision_ids):
+        """Probe to find out the presence of multiple revisions.
+
+        :param revision_ids: An iterable of revision_ids.
+        :return: A set of the revision_ids that were present.
+        """
+        parent_map = self.revisions.get_parent_map(
+            [(rev_id,) for rev_id in revision_ids])
+        result = set()
+        if _mod_revision.NULL_REVISION in revision_ids:
+            result.add(_mod_revision.NULL_REVISION)
+        result.update([key[0] for key in parent_map])
+        return result
+
+    @needs_read_lock
+    def get_revision_reconcile(self, revision_id):
+        """'reconcile' helper routine that allows access to a revision always.
+
+        This variant of get_revision does not cross check the weave graph
+        against the revision one as get_revision does: but it should only
+        be used by reconcile, or reconcile-alike commands that are correcting
+        or testing the revision graph.
+        """
+        return self._get_revisions([revision_id])[0]
+
+    @needs_read_lock
+    def get_revisions(self, revision_ids):
+        """Get many revisions at once.
+        
+        Repositories that need to check data on every revision read should 
+        subclass this method.
+        """
+        return self._get_revisions(revision_ids)
+
+    @needs_read_lock
+    def _get_revisions(self, revision_ids):
+        """Core work logic to get many revisions without sanity checks."""
+        revs = {}
+        for revid, rev in self._iter_revisions(revision_ids):
+            if rev is None:
+                raise errors.NoSuchRevision(self, revid)
+            revs[revid] = rev
+        return [revs[revid] for revid in revision_ids]
+
+    def _iter_revisions(self, revision_ids):
+        """Iterate over revision objects.
+
+        :param revision_ids: An iterable of revisions to examine. None may be
+            passed to request all revisions known to the repository. Note that
+            not all repositories can find unreferenced revisions; for those
+            repositories only referenced ones will be returned.
+        :return: An iterator of (revid, revision) tuples. Absent revisions (
+            those asked for but not available) are returned as (revid, None).
+        """
+        if revision_ids is None:
+            revision_ids = self.all_revision_ids()
+        else:
+            for rev_id in revision_ids:
+                if not rev_id or not isinstance(rev_id, basestring):
+                    raise errors.InvalidRevisionId(revision_id=rev_id, branch=self)
+        keys = [(key,) for key in revision_ids]
+        stream = self.revisions.get_record_stream(keys, 'unordered', True)
+        for record in stream:
+            revid = record.key[0]
+            if record.storage_kind == 'absent':
+                yield (revid, None)
+            else:
+                text = record.get_bytes_as('fulltext')
+                rev = self._serializer.read_revision_from_string(text)
+                yield (revid, rev)
+
+    @needs_write_lock
+    def add_signature_text(self, revision_id, signature):
+        """Store a signature text for a revision.
+
+        :param revision_id: Revision id of the revision
+        :param signature: Signature text.
+        """
+        self.signatures.add_lines((revision_id,), (),
+            osutils.split_lines(signature))
+
+    def find_text_key_references(self):
+        """Find the text key references within the repository.
+
+        :return: A dictionary mapping text keys ((fileid, revision_id) tuples)
+            to whether they were referred to by the inventory of the
+            revision_id that they contain. The inventory texts from all present
+            revision ids are assessed to generate this report.
+        """
+        revision_keys = self.revisions.keys()
+        w = self.inventories
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            return self._serializer._find_text_key_references(
+                w.iter_lines_added_or_present_in_keys(revision_keys, pb=pb))
+        finally:
+            pb.finished()
+
+    def _inventory_xml_lines_for_keys(self, keys):
+        """Get a line iterator of the sort needed for findind references.
+
+        Not relevant for non-xml inventory repositories.
+
+        Ghosts in revision_keys are ignored.
+
+        :param revision_keys: The revision keys for the inventories to inspect.
+        :return: An iterator over (inventory line, revid) for the fulltexts of
+            all of the xml inventories specified by revision_keys.
+        """
+        stream = self.inventories.get_record_stream(keys, 'unordered', True)
+        for record in stream:
+            if record.storage_kind != 'absent':
+                chunks = record.get_bytes_as('chunked')
+                revid = record.key[-1]
+                lines = osutils.chunks_to_lines(chunks)
+                for line in lines:
+                    yield line, revid
+
+    def _find_file_ids_from_xml_inventory_lines(self, line_iterator,
+        revision_keys):
+        """Helper routine for fileids_altered_by_revision_ids.
+
+        This performs the translation of xml lines to revision ids.
+
+        :param line_iterator: An iterator of lines, origin_version_id
+        :param revision_keys: The revision ids to filter for. This should be a
+            set or other type which supports efficient __contains__ lookups, as
+            the revision key from each parsed line will be looked up in the
+            revision_keys filter.
+        :return: a dictionary mapping altered file-ids to an iterable of
+            revision_ids. Each altered file-ids has the exact revision_ids that
+            altered it listed explicitly.
+        """
+        seen = set(self._serializer._find_text_key_references(
+                line_iterator).iterkeys())
+        parent_keys = self._find_parent_keys_of_revisions(revision_keys)
+        parent_seen = set(self._serializer._find_text_key_references(
+            self._inventory_xml_lines_for_keys(parent_keys)))
+        new_keys = seen - parent_seen
+        result = {}
+        setdefault = result.setdefault
+        for key in new_keys:
+            setdefault(key[0], set()).add(key[-1])
+        return result
+
+    def _find_parent_keys_of_revisions(self, revision_keys):
+        """Similar to _find_parent_ids_of_revisions, but used with keys.
+
+        :param revision_keys: An iterable of revision_keys.
+        :return: The parents of all revision_keys that are not already in
+            revision_keys
+        """
+        parent_map = self.revisions.get_parent_map(revision_keys)
+        parent_keys = set()
+        map(parent_keys.update, parent_map.itervalues())
+        parent_keys.difference_update(revision_keys)
+        parent_keys.discard(_mod_revision.NULL_REVISION)
+        return parent_keys
+
+    def fileids_altered_by_revision_ids(self, revision_ids, _inv_weave=None):
+        """Find the file ids and versions affected by revisions.
+
+        :param revisions: an iterable containing revision ids.
+        :param _inv_weave: The inventory weave from this repository or None.
+            If None, the inventory weave will be opened automatically.
+        :return: a dictionary mapping altered file-ids to an iterable of
+            revision_ids. Each altered file-ids has the exact revision_ids that
+            altered it listed explicitly.
+        """
+        selected_keys = set((revid,) for revid in revision_ids)
+        w = _inv_weave or self.inventories
+        return self._find_file_ids_from_xml_inventory_lines(
+            w.iter_lines_added_or_present_in_keys(
+                selected_keys, pb=None),
+            selected_keys)
+
+    def iter_files_bytes(self, desired_files):
+        """Iterate through file versions.
+
+        Files will not necessarily be returned in the order they occur in
+        desired_files.  No specific order is guaranteed.
+
+        Yields pairs of identifier, bytes_iterator.  identifier is an opaque
+        value supplied by the caller as part of desired_files.  It should
+        uniquely identify the file version in the caller's context.  (Examples:
+        an index number or a TreeTransform trans_id.)
+
+        bytes_iterator is an iterable of bytestrings for the file.  The
+        kind of iterable and length of the bytestrings are unspecified, but for
+        this implementation, it is a list of bytes produced by
+        VersionedFile.get_record_stream().
+
+        :param desired_files: a list of (file_id, revision_id, identifier)
+            triples
+        """
+        text_keys = {}
+        for file_id, revision_id, callable_data in desired_files:
+            text_keys[(file_id, revision_id)] = callable_data
+        for record in self.texts.get_record_stream(text_keys, 'unordered', True):
+            if record.storage_kind == 'absent':
+                raise errors.RevisionNotPresent(record.key, self)
+            yield text_keys[record.key], record.get_bytes_as('chunked')
+
+    def _generate_text_key_index(self, text_key_references=None,
+        ancestors=None):
+        """Generate a new text key index for the repository.
+
+        This is an expensive function that will take considerable time to run.
+
+        :return: A dict mapping text keys ((file_id, revision_id) tuples) to a
+            list of parents, also text keys. When a given key has no parents,
+            the parents list will be [NULL_REVISION].
+        """
+        # All revisions, to find inventory parents.
+        if ancestors is None:
+            graph = self.get_graph()
+            ancestors = graph.get_parent_map(self.all_revision_ids())
+        if text_key_references is None:
+            text_key_references = self.find_text_key_references()
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            return self._do_generate_text_key_index(ancestors,
+                text_key_references, pb)
+        finally:
+            pb.finished()
+
+    def _do_generate_text_key_index(self, ancestors, text_key_references, pb):
+        """Helper for _generate_text_key_index to avoid deep nesting."""
+        revision_order = tsort.topo_sort(ancestors)
+        invalid_keys = set()
+        revision_keys = {}
+        for revision_id in revision_order:
+            revision_keys[revision_id] = set()
+        text_count = len(text_key_references)
+        # a cache of the text keys to allow reuse; costs a dict of all the
+        # keys, but saves a 2-tuple for every child of a given key.
+        text_key_cache = {}
+        for text_key, valid in text_key_references.iteritems():
+            if not valid:
+                invalid_keys.add(text_key)
+            else:
+                revision_keys[text_key[1]].add(text_key)
+            text_key_cache[text_key] = text_key
+        del text_key_references
+        text_index = {}
+        text_graph = graph.Graph(graph.DictParentsProvider(text_index))
+        NULL_REVISION = _mod_revision.NULL_REVISION
+        # Set a cache with a size of 10 - this suffices for bzr.dev but may be
+        # too small for large or very branchy trees. However, for 55K path
+        # trees, it would be easy to use too much memory trivially. Ideally we
+        # could gauge this by looking at available real memory etc, but this is
+        # always a tricky proposition.
+        inventory_cache = lru_cache.LRUCache(10)
+        batch_size = 10 # should be ~150MB on a 55K path tree
+        batch_count = len(revision_order) / batch_size + 1
+        processed_texts = 0
+        pb.update(gettext("Calculating text parents"), processed_texts, text_count)
+        for offset in xrange(batch_count):
+            to_query = revision_order[offset * batch_size:(offset + 1) *
+                batch_size]
+            if not to_query:
+                break
+            for revision_id in to_query:
+                parent_ids = ancestors[revision_id]
+                for text_key in revision_keys[revision_id]:
+                    pb.update(gettext("Calculating text parents"), processed_texts)
+                    processed_texts += 1
+                    candidate_parents = []
+                    for parent_id in parent_ids:
+                        parent_text_key = (text_key[0], parent_id)
+                        try:
+                            check_parent = parent_text_key not in \
+                                revision_keys[parent_id]
+                        except KeyError:
+                            # the parent parent_id is a ghost:
+                            check_parent = False
+                            # truncate the derived graph against this ghost.
+                            parent_text_key = None
+                        if check_parent:
+                            # look at the parent commit details inventories to
+                            # determine possible candidates in the per file graph.
+                            # TODO: cache here.
+                            try:
+                                inv = inventory_cache[parent_id]
+                            except KeyError:
+                                inv = self.revision_tree(parent_id).inventory
+                                inventory_cache[parent_id] = inv
+                            try:
+                                parent_entry = inv[text_key[0]]
+                            except (KeyError, errors.NoSuchId):
+                                parent_entry = None
+                            if parent_entry is not None:
+                                parent_text_key = (
+                                    text_key[0], parent_entry.revision)
+                            else:
+                                parent_text_key = None
+                        if parent_text_key is not None:
+                            candidate_parents.append(
+                                text_key_cache[parent_text_key])
+                    parent_heads = text_graph.heads(candidate_parents)
+                    new_parents = list(parent_heads)
+                    new_parents.sort(key=lambda x:candidate_parents.index(x))
+                    if new_parents == []:
+                        new_parents = [NULL_REVISION]
+                    text_index[text_key] = new_parents
+
+        for text_key in invalid_keys:
+            text_index[text_key] = [NULL_REVISION]
+        return text_index
+
+    def item_keys_introduced_by(self, revision_ids, _files_pb=None):
+        """Get an iterable listing the keys of all the data introduced by a set
+        of revision IDs.
+
+        The keys will be ordered so that the corresponding items can be safely
+        fetched and inserted in that order.
+
+        :returns: An iterable producing tuples of (knit-kind, file-id,
+            versions).  knit-kind is one of 'file', 'inventory', 'signatures',
+            'revisions'.  file-id is None unless knit-kind is 'file'.
+        """
+        for result in self._find_file_keys_to_fetch(revision_ids, _files_pb):
+            yield result
+        del _files_pb
+        for result in self._find_non_file_keys_to_fetch(revision_ids):
+            yield result
+
+    def _find_file_keys_to_fetch(self, revision_ids, pb):
+        # XXX: it's a bit weird to control the inventory weave caching in this
+        # generator.  Ideally the caching would be done in fetch.py I think.  Or
+        # maybe this generator should explicitly have the contract that it
+        # should not be iterated until the previously yielded item has been
+        # processed?
+        inv_w = self.inventories
+
+        # file ids that changed
+        file_ids = self.fileids_altered_by_revision_ids(revision_ids, inv_w)
+        count = 0
+        num_file_ids = len(file_ids)
+        for file_id, altered_versions in file_ids.iteritems():
+            if pb is not None:
+                pb.update(gettext("Fetch texts"), count, num_file_ids)
+            count += 1
+            yield ("file", file_id, altered_versions)
+
+    def _find_non_file_keys_to_fetch(self, revision_ids):
+        # inventory
+        yield ("inventory", None, revision_ids)
+
+        # signatures
+        # XXX: Note ATM no callers actually pay attention to this return
+        #      instead they just use the list of revision ids and ignore
+        #      missing sigs. Consider removing this work entirely
+        revisions_with_signatures = set(self.signatures.get_parent_map(
+            [(r,) for r in revision_ids]))
+        revisions_with_signatures = set(
+            [r for (r,) in revisions_with_signatures])
+        revisions_with_signatures.intersection_update(revision_ids)
+        yield ("signatures", None, revisions_with_signatures)
+
+        # revisions
+        yield ("revisions", None, revision_ids)
+
+    @needs_read_lock
+    def get_inventory(self, revision_id):
+        """Get Inventory object by revision id."""
+        return self.iter_inventories([revision_id]).next()
+
+    def iter_inventories(self, revision_ids, ordering=None):
+        """Get many inventories by revision_ids.
+
+        This will buffer some or all of the texts used in constructing the
+        inventories in memory, but will only parse a single inventory at a
+        time.
+
+        :param revision_ids: The expected revision ids of the inventories.
+        :param ordering: optional ordering, e.g. 'topological'.  If not
+            specified, the order of revision_ids will be preserved (by
+            buffering if necessary).
+        :return: An iterator of inventories.
+        """
+        if ((None in revision_ids)
+            or (_mod_revision.NULL_REVISION in revision_ids)):
+            raise ValueError('cannot get null revision inventory')
+        return self._iter_inventories(revision_ids, ordering)
+
+    def _iter_inventories(self, revision_ids, ordering):
+        """single-document based inventory iteration."""
+        inv_xmls = self._iter_inventory_xmls(revision_ids, ordering)
+        for text, revision_id in inv_xmls:
+            yield self._deserialise_inventory(revision_id, text)
+
+    def _iter_inventory_xmls(self, revision_ids, ordering):
+        if ordering is None:
+            order_as_requested = True
+            ordering = 'unordered'
+        else:
+            order_as_requested = False
+        keys = [(revision_id,) for revision_id in revision_ids]
+        if not keys:
+            return
+        if order_as_requested:
+            key_iter = iter(keys)
+            next_key = key_iter.next()
+        stream = self.inventories.get_record_stream(keys, ordering, True)
+        text_chunks = {}
+        for record in stream:
+            if record.storage_kind != 'absent':
+                chunks = record.get_bytes_as('chunked')
+                if order_as_requested:
+                    text_chunks[record.key] = chunks
+                else:
+                    yield ''.join(chunks), record.key[-1]
+            else:
+                raise errors.NoSuchRevision(self, record.key)
+            if order_as_requested:
+                # Yield as many results as we can while preserving order.
+                while next_key in text_chunks:
+                    chunks = text_chunks.pop(next_key)
+                    yield ''.join(chunks), next_key[-1]
+                    try:
+                        next_key = key_iter.next()
+                    except StopIteration:
+                        # We still want to fully consume the get_record_stream,
+                        # just in case it is not actually finished at this point
+                        next_key = None
+                        break
+
+    def _deserialise_inventory(self, revision_id, xml):
+        """Transform the xml into an inventory object.
+
+        :param revision_id: The expected revision id of the inventory.
+        :param xml: A serialised inventory.
+        """
+        result = self._serializer.read_inventory_from_string(xml, revision_id,
+                    entry_cache=self._inventory_entry_cache,
+                    return_from_cache=self._safe_to_return_from_cache)
+        if result.revision_id != revision_id:
+            raise AssertionError('revision id mismatch %s != %s' % (
+                result.revision_id, revision_id))
+        return result
+
+    def get_serializer_format(self):
+        return self._serializer.format_num
+
+    @needs_read_lock
+    def _get_inventory_xml(self, revision_id):
+        """Get serialized inventory as a string."""
+        texts = self._iter_inventory_xmls([revision_id], 'unordered')
+        try:
+            text, revision_id = texts.next()
+        except StopIteration:
+            raise errors.HistoryMissing(self, 'inventory', revision_id)
+        return text
+
+    @needs_read_lock
+    def revision_tree(self, revision_id):
+        """Return Tree for a revision on this branch.
+
+        `revision_id` may be NULL_REVISION for the empty tree revision.
+        """
+        revision_id = _mod_revision.ensure_null(revision_id)
+        # TODO: refactor this to use an existing revision object
+        # so we don't need to read it in twice.
+        if revision_id == _mod_revision.NULL_REVISION:
+            return InventoryRevisionTree(self,
+                Inventory(root_id=None), _mod_revision.NULL_REVISION)
+        else:
+            inv = self.get_inventory(revision_id)
+            return InventoryRevisionTree(self, inv, revision_id)
+
+    def revision_trees(self, revision_ids):
+        """Return Trees for revisions in this repository.
+
+        :param revision_ids: a sequence of revision-ids;
+          a revision-id may not be None or 'null:'
+        """
+        inventories = self.iter_inventories(revision_ids)
+        for inv in inventories:
+            yield InventoryRevisionTree(self, inv, inv.revision_id)
+
+    def _filtered_revision_trees(self, revision_ids, file_ids):
+        """Return Tree for a revision on this branch with only some files.
+
+        :param revision_ids: a sequence of revision-ids;
+          a revision-id may not be None or 'null:'
+        :param file_ids: if not None, the result is filtered
+          so that only those file-ids, their parents and their
+          children are included.
+        """
+        inventories = self.iter_inventories(revision_ids)
+        for inv in inventories:
+            # Should we introduce a FilteredRevisionTree class rather
+            # than pre-filter the inventory here?
+            filtered_inv = inv.filter(file_ids)
+            yield InventoryRevisionTree(self, filtered_inv, filtered_inv.revision_id)
+
+    def get_parent_map(self, revision_ids):
+        """See graph.StackedParentsProvider.get_parent_map"""
+        # revisions index works in keys; this just works in revisions
+        # therefore wrap and unwrap
+        query_keys = []
+        result = {}
+        for revision_id in revision_ids:
+            if revision_id == _mod_revision.NULL_REVISION:
+                result[revision_id] = ()
+            elif revision_id is None:
+                raise ValueError('get_parent_map(None) is not valid')
+            else:
+                query_keys.append((revision_id ,))
+        for ((revision_id,), parent_keys) in \
+                self.revisions.get_parent_map(query_keys).iteritems():
+            if parent_keys:
+                result[revision_id] = tuple([parent_revid
+                    for (parent_revid,) in parent_keys])
+            else:
+                result[revision_id] = (_mod_revision.NULL_REVISION,)
+        return result
+
+    @needs_read_lock
+    def get_known_graph_ancestry(self, revision_ids):
+        """Return the known graph for a set of revision ids and their ancestors.
+        """
+        st = static_tuple.StaticTuple
+        revision_keys = [st(r_id).intern() for r_id in revision_ids]
+        known_graph = self.revisions.get_known_graph_ancestry(revision_keys)
+        return graph.GraphThunkIdsToKeys(known_graph)
+
+    @needs_read_lock
+    def get_file_graph(self):
+        """Return the graph walker for text revisions."""
+        return graph.Graph(self.texts)
+
+    def _get_versioned_file_checker(self, text_key_references=None,
+        ancestors=None):
+        """Return an object suitable for checking versioned files.
+        
+        :param text_key_references: if non-None, an already built
+            dictionary mapping text keys ((fileid, revision_id) tuples)
+            to whether they were referred to by the inventory of the
+            revision_id that they contain. If None, this will be
+            calculated.
+        :param ancestors: Optional result from
+            self.get_graph().get_parent_map(self.all_revision_ids()) if already
+            available.
+        """
+        return _VersionedFileChecker(self,
+            text_key_references=text_key_references, ancestors=ancestors)
+
+    @needs_read_lock
+    def has_signature_for_revision_id(self, revision_id):
+        """Query for a revision signature for revision_id in the repository."""
+        if not self.has_revision(revision_id):
+            raise errors.NoSuchRevision(self, revision_id)
+        sig_present = (1 == len(
+            self.signatures.get_parent_map([(revision_id,)])))
+        return sig_present
+
+    @needs_read_lock
+    def get_signature_text(self, revision_id):
+        """Return the text for a signature."""
+        stream = self.signatures.get_record_stream([(revision_id,)],
+            'unordered', True)
+        record = stream.next()
+        if record.storage_kind == 'absent':
+            raise errors.NoSuchRevision(self, revision_id)
+        return record.get_bytes_as('fulltext')
+
+    @needs_read_lock
+    def _check(self, revision_ids, callback_refs, check_repo):
+        result = check.VersionedFileCheck(self, check_repo=check_repo)
+        result.check(callback_refs)
+        return result
+
+    def _find_inconsistent_revision_parents(self, revisions_iterator=None):
+        """Find revisions with different parent lists in the revision object
+        and in the index graph.
+
+        :param revisions_iterator: None, or an iterator of (revid,
+            Revision-or-None). This iterator controls the revisions checked.
+        :returns: an iterator yielding tuples of (revison-id, parents-in-index,
+            parents-in-revision).
+        """
+        if not self.is_locked():
+            raise AssertionError()
+        vf = self.revisions
+        if revisions_iterator is None:
+            revisions_iterator = self._iter_revisions(None)
+        for revid, revision in revisions_iterator:
+            if revision is None:
+                pass
+            parent_map = vf.get_parent_map([(revid,)])
+            parents_according_to_index = tuple(parent[-1] for parent in
+                parent_map[(revid,)])
+            parents_according_to_revision = tuple(revision.parent_ids)
+            if parents_according_to_index != parents_according_to_revision:
+                yield (revid, parents_according_to_index,
+                    parents_according_to_revision)
+
+    def _check_for_inconsistent_revision_parents(self):
+        inconsistencies = list(self._find_inconsistent_revision_parents())
+        if inconsistencies:
+            raise errors.BzrCheckError(
+                "Revision knit has inconsistent parents.")
+
+    def _get_sink(self):
+        """Return a sink for streaming into this repository."""
+        return StreamSink(self)
+
+    def _get_source(self, to_format):
+        """Return a source for streaming from this repository."""
+        return StreamSource(self, to_format)
+
+
+class MetaDirVersionedFileRepository(MetaDirRepository,
+                                     VersionedFileRepository):
+    """Repositories in a meta-dir, that work via versioned file objects."""
+
+    def __init__(self, _format, a_bzrdir, control_files):
+        super(MetaDirVersionedFileRepository, self).__init__(_format, a_bzrdir,
+            control_files)
+
+
+class MetaDirVersionedFileRepositoryFormat(MetaDirRepositoryFormat,
+        VersionedFileRepositoryFormat):
+    """Base class for repository formats using versioned files in metadirs."""
+
+
+class StreamSink(object):
+    """An object that can insert a stream into a repository.
+
+    This interface handles the complexity of reserialising inventories and
+    revisions from different formats, and allows unidirectional insertion into
+    stacked repositories without looking for the missing basis parents
+    beforehand.
+    """
+
+    def __init__(self, target_repo):
+        self.target_repo = target_repo
+
+    def insert_stream(self, stream, src_format, resume_tokens):
+        """Insert a stream's content into the target repository.
+
+        :param src_format: a bzr repository format.
+
+        :return: a list of resume tokens and an  iterable of keys additional
+            items required before the insertion can be completed.
+        """
+        self.target_repo.lock_write()
+        try:
+            if resume_tokens:
+                self.target_repo.resume_write_group(resume_tokens)
+                is_resume = True
+            else:
+                self.target_repo.start_write_group()
+                is_resume = False
+            try:
+                # locked_insert_stream performs a commit|suspend.
+                missing_keys = self.insert_stream_without_locking(stream,
+                                    src_format, is_resume)
+                if missing_keys:
+                    # suspend the write group and tell the caller what we is
+                    # missing. We know we can suspend or else we would not have
+                    # entered this code path. (All repositories that can handle
+                    # missing keys can handle suspending a write group).
+                    write_group_tokens = self.target_repo.suspend_write_group()
+                    return write_group_tokens, missing_keys
+                hint = self.target_repo.commit_write_group()
+                to_serializer = self.target_repo._format._serializer
+                src_serializer = src_format._serializer
+                if (to_serializer != src_serializer and
+                    self.target_repo._format.pack_compresses):
+                    self.target_repo.pack(hint=hint)
+                return [], set()
+            except:
+                self.target_repo.abort_write_group(suppress_errors=True)
+                raise
+        finally:
+            self.target_repo.unlock()
+
+    def insert_stream_without_locking(self, stream, src_format,
+                                      is_resume=False):
+        """Insert a stream's content into the target repository.
+
+        This assumes that you already have a locked repository and an active
+        write group.
+
+        :param src_format: a bzr repository format.
+        :param is_resume: Passed down to get_missing_parent_inventories to
+            indicate if we should be checking for missing texts at the same
+            time.
+
+        :return: A set of keys that are missing.
+        """
+        if not self.target_repo.is_write_locked():
+            raise errors.ObjectNotLocked(self)
+        if not self.target_repo.is_in_write_group():
+            raise errors.BzrError('you must already be in a write group')
+        to_serializer = self.target_repo._format._serializer
+        src_serializer = src_format._serializer
+        new_pack = None
+        if to_serializer == src_serializer:
+            # If serializers match and the target is a pack repository, set the
+            # write cache size on the new pack.  This avoids poor performance
+            # on transports where append is unbuffered (such as
+            # RemoteTransport).  This is safe to do because nothing should read
+            # back from the target repository while a stream with matching
+            # serialization is being inserted.
+            # The exception is that a delta record from the source that should
+            # be a fulltext may need to be expanded by the target (see
+            # test_fetch_revisions_with_deltas_into_pack); but we take care to
+            # explicitly flush any buffered writes first in that rare case.
+            try:
+                new_pack = self.target_repo._pack_collection._new_pack
+            except AttributeError:
+                # Not a pack repository
+                pass
+            else:
+                new_pack.set_write_cache_size(1024*1024)
+        for substream_type, substream in stream:
+            if 'stream' in debug.debug_flags:
+                mutter('inserting substream: %s', substream_type)
+            if substream_type == 'texts':
+                self.target_repo.texts.insert_record_stream(substream)
+            elif substream_type == 'inventories':
+                if src_serializer == to_serializer:
+                    self.target_repo.inventories.insert_record_stream(
+                        substream)
+                else:
+                    self._extract_and_insert_inventories(
+                        substream, src_serializer)
+            elif substream_type == 'inventory-deltas':
+                self._extract_and_insert_inventory_deltas(
+                    substream, src_serializer)
+            elif substream_type == 'chk_bytes':
+                # XXX: This doesn't support conversions, as it assumes the
+                #      conversion was done in the fetch code.
+                self.target_repo.chk_bytes.insert_record_stream(substream)
+            elif substream_type == 'revisions':
+                # This may fallback to extract-and-insert more often than
+                # required if the serializers are different only in terms of
+                # the inventory.
+                if src_serializer == to_serializer:
+                    self.target_repo.revisions.insert_record_stream(substream)
+                else:
+                    self._extract_and_insert_revisions(substream,
+                        src_serializer)
+            elif substream_type == 'signatures':
+                self.target_repo.signatures.insert_record_stream(substream)
+            else:
+                raise AssertionError('kaboom! %s' % (substream_type,))
+        # Done inserting data, and the missing_keys calculations will try to
+        # read back from the inserted data, so flush the writes to the new pack
+        # (if this is pack format).
+        if new_pack is not None:
+            new_pack._write_data('', flush=True)
+        # Find all the new revisions (including ones from resume_tokens)
+        missing_keys = self.target_repo.get_missing_parent_inventories(
+            check_for_missing_texts=is_resume)
+        try:
+            for prefix, versioned_file in (
+                ('texts', self.target_repo.texts),
+                ('inventories', self.target_repo.inventories),
+                ('revisions', self.target_repo.revisions),
+                ('signatures', self.target_repo.signatures),
+                ('chk_bytes', self.target_repo.chk_bytes),
+                ):
+                if versioned_file is None:
+                    continue
+                # TODO: key is often going to be a StaticTuple object
+                #       I don't believe we can define a method by which
+                #       (prefix,) + StaticTuple will work, though we could
+                #       define a StaticTuple.sq_concat that would allow you to
+                #       pass in either a tuple or a StaticTuple as the second
+                #       object, so instead we could have:
+                #       StaticTuple(prefix) + key here...
+                missing_keys.update((prefix,) + key for key in
+                    versioned_file.get_missing_compression_parent_keys())
+        except NotImplementedError:
+            # cannot even attempt suspending, and missing would have failed
+            # during stream insertion.
+            missing_keys = set()
+        return missing_keys
+
+    def _extract_and_insert_inventory_deltas(self, substream, serializer):
+        target_rich_root = self.target_repo._format.rich_root_data
+        target_tree_refs = self.target_repo._format.supports_tree_reference
+        for record in substream:
+            # Insert the delta directly
+            inventory_delta_bytes = record.get_bytes_as('fulltext')
+            deserialiser = inventory_delta.InventoryDeltaDeserializer()
+            try:
+                parse_result = deserialiser.parse_text_bytes(
+                    inventory_delta_bytes)
+            except inventory_delta.IncompatibleInventoryDelta, err:
+                mutter("Incompatible delta: %s", err.msg)
+                raise errors.IncompatibleRevision(self.target_repo._format)
+            basis_id, new_id, rich_root, tree_refs, inv_delta = parse_result
+            revision_id = new_id
+            parents = [key[0] for key in record.parents]
+            self.target_repo.add_inventory_by_delta(
+                basis_id, inv_delta, revision_id, parents)
+
+    def _extract_and_insert_inventories(self, substream, serializer,
+            parse_delta=None):
+        """Generate a new inventory versionedfile in target, converting data.
+
+        The inventory is retrieved from the source, (deserializing it), and
+        stored in the target (reserializing it in a different format).
+        """
+        target_rich_root = self.target_repo._format.rich_root_data
+        target_tree_refs = self.target_repo._format.supports_tree_reference
+        for record in substream:
+            # It's not a delta, so it must be a fulltext in the source
+            # serializer's format.
+            bytes = record.get_bytes_as('fulltext')
+            revision_id = record.key[0]
+            inv = serializer.read_inventory_from_string(bytes, revision_id)
+            parents = [key[0] for key in record.parents]
+            self.target_repo.add_inventory(revision_id, inv, parents)
+            # No need to keep holding this full inv in memory when the rest of
+            # the substream is likely to be all deltas.
+            del inv
+
+    def _extract_and_insert_revisions(self, substream, serializer):
+        for record in substream:
+            bytes = record.get_bytes_as('fulltext')
+            revision_id = record.key[0]
+            rev = serializer.read_revision_from_string(bytes)
+            if rev.revision_id != revision_id:
+                raise AssertionError('wtf: %s != %s' % (rev, revision_id))
+            self.target_repo.add_revision(revision_id, rev)
+
+    def finished(self):
+        if self.target_repo._format._fetch_reconcile:
+            self.target_repo.reconcile()
+
+
+class StreamSource(object):
+    """A source of a stream for fetching between repositories."""
+
+    def __init__(self, from_repository, to_format):
+        """Create a StreamSource streaming from from_repository."""
+        self.from_repository = from_repository
+        self.to_format = to_format
+        self._record_counter = RecordCounter()
+
+    def delta_on_metadata(self):
+        """Return True if delta's are permitted on metadata streams.
+
+        That is on revisions and signatures.
+        """
+        src_serializer = self.from_repository._format._serializer
+        target_serializer = self.to_format._serializer
+        return (self.to_format._fetch_uses_deltas and
+            src_serializer == target_serializer)
+
+    def _fetch_revision_texts(self, revs):
+        # fetch signatures first and then the revision texts
+        # may need to be a InterRevisionStore call here.
+        from_sf = self.from_repository.signatures
+        # A missing signature is just skipped.
+        keys = [(rev_id,) for rev_id in revs]
+        signatures = versionedfile.filter_absent(from_sf.get_record_stream(
+            keys,
+            self.to_format._fetch_order,
+            not self.to_format._fetch_uses_deltas))
+        # If a revision has a delta, this is actually expanded inside the
+        # insert_record_stream code now, which is an alternate fix for
+        # bug #261339
+        from_rf = self.from_repository.revisions
+        revisions = from_rf.get_record_stream(
+            keys,
+            self.to_format._fetch_order,
+            not self.delta_on_metadata())
+        return [('signatures', signatures), ('revisions', revisions)]
+
+    def _generate_root_texts(self, revs):
+        """This will be called by get_stream between fetching weave texts and
+        fetching the inventory weave.
+        """
+        if self._rich_root_upgrade():
+            return _mod_fetch.Inter1and2Helper(
+                self.from_repository).generate_root_texts(revs)
+        else:
+            return []
+
+    def get_stream(self, search):
+        phase = 'file'
+        revs = search.get_keys()
+        graph = self.from_repository.get_graph()
+        revs = tsort.topo_sort(graph.get_parent_map(revs))
+        data_to_fetch = self.from_repository.item_keys_introduced_by(revs)
+        text_keys = []
+        for knit_kind, file_id, revisions in data_to_fetch:
+            if knit_kind != phase:
+                phase = knit_kind
+                # Make a new progress bar for this phase
+            if knit_kind == "file":
+                # Accumulate file texts
+                text_keys.extend([(file_id, revision) for revision in
+                    revisions])
+            elif knit_kind == "inventory":
+                # Now copy the file texts.
+                from_texts = self.from_repository.texts
+                yield ('texts', from_texts.get_record_stream(
+                    text_keys, self.to_format._fetch_order,
+                    not self.to_format._fetch_uses_deltas))
+                # Cause an error if a text occurs after we have done the
+                # copy.
+                text_keys = None
+                # Before we process the inventory we generate the root
+                # texts (if necessary) so that the inventories references
+                # will be valid.
+                for _ in self._generate_root_texts(revs):
+                    yield _
+                # we fetch only the referenced inventories because we do not
+                # know for unselected inventories whether all their required
+                # texts are present in the other repository - it could be
+                # corrupt.
+                for info in self._get_inventory_stream(revs):
+                    yield info
+            elif knit_kind == "signatures":
+                # Nothing to do here; this will be taken care of when
+                # _fetch_revision_texts happens.
+                pass
+            elif knit_kind == "revisions":
+                for record in self._fetch_revision_texts(revs):
+                    yield record
+            else:
+                raise AssertionError("Unknown knit kind %r" % knit_kind)
+
+    def get_stream_for_missing_keys(self, missing_keys):
+        # missing keys can only occur when we are byte copying and not
+        # translating (because translation means we don't send
+        # unreconstructable deltas ever).
+        keys = {}
+        keys['texts'] = set()
+        keys['revisions'] = set()
+        keys['inventories'] = set()
+        keys['chk_bytes'] = set()
+        keys['signatures'] = set()
+        for key in missing_keys:
+            keys[key[0]].add(key[1:])
+        if len(keys['revisions']):
+            # If we allowed copying revisions at this point, we could end up
+            # copying a revision without copying its required texts: a
+            # violation of the requirements for repository integrity.
+            raise AssertionError(
+                'cannot copy revisions to fill in missing deltas %s' % (
+                    keys['revisions'],))
+        for substream_kind, keys in keys.iteritems():
+            vf = getattr(self.from_repository, substream_kind)
+            if vf is None and keys:
+                    raise AssertionError(
+                        "cannot fill in keys for a versioned file we don't"
+                        " have: %s needs %s" % (substream_kind, keys))
+            if not keys:
+                # No need to stream something we don't have
+                continue
+            if substream_kind == 'inventories':
+                # Some missing keys are genuinely ghosts, filter those out.
+                present = self.from_repository.inventories.get_parent_map(keys)
+                revs = [key[0] for key in present]
+                # Get the inventory stream more-or-less as we do for the
+                # original stream; there's no reason to assume that records
+                # direct from the source will be suitable for the sink.  (Think
+                # e.g. 2a -> 1.9-rich-root).
+                for info in self._get_inventory_stream(revs, missing=True):
+                    yield info
+                continue
+
+            # Ask for full texts always so that we don't need more round trips
+            # after this stream.
+            # Some of the missing keys are genuinely ghosts, so filter absent
+            # records. The Sink is responsible for doing another check to
+            # ensure that ghosts don't introduce missing data for future
+            # fetches.
+            stream = versionedfile.filter_absent(vf.get_record_stream(keys,
+                self.to_format._fetch_order, True))
+            yield substream_kind, stream
+
+    def inventory_fetch_order(self):
+        if self._rich_root_upgrade():
+            return 'topological'
+        else:
+            return self.to_format._fetch_order
+
+    def _rich_root_upgrade(self):
+        return (not self.from_repository._format.rich_root_data and
+            self.to_format.rich_root_data)
+
+    def _get_inventory_stream(self, revision_ids, missing=False):
+        from_format = self.from_repository._format
+        if (from_format.supports_chks and self.to_format.supports_chks and
+            from_format.network_name() == self.to_format.network_name()):
+            raise AssertionError(
+                "this case should be handled by GroupCHKStreamSource")
+        elif 'forceinvdeltas' in debug.debug_flags:
+            return self._get_convertable_inventory_stream(revision_ids,
+                    delta_versus_null=missing)
+        elif from_format.network_name() == self.to_format.network_name():
+            # Same format.
+            return self._get_simple_inventory_stream(revision_ids,
+                    missing=missing)
+        elif (not from_format.supports_chks and not self.to_format.supports_chks
+                and from_format._serializer == self.to_format._serializer):
+            # Essentially the same format.
+            return self._get_simple_inventory_stream(revision_ids,
+                    missing=missing)
+        else:
+            # Any time we switch serializations, we want to use an
+            # inventory-delta based approach.
+            return self._get_convertable_inventory_stream(revision_ids,
+                    delta_versus_null=missing)
+
+    def _get_simple_inventory_stream(self, revision_ids, missing=False):
+        # NB: This currently reopens the inventory weave in source;
+        # using a single stream interface instead would avoid this.
+        from_weave = self.from_repository.inventories
+        if missing:
+            delta_closure = True
+        else:
+            delta_closure = not self.delta_on_metadata()
+        yield ('inventories', from_weave.get_record_stream(
+            [(rev_id,) for rev_id in revision_ids],
+            self.inventory_fetch_order(), delta_closure))
+
+    def _get_convertable_inventory_stream(self, revision_ids,
+                                          delta_versus_null=False):
+        # The two formats are sufficiently different that there is no fast
+        # path, so we need to send just inventorydeltas, which any
+        # sufficiently modern client can insert into any repository.
+        # The StreamSink code expects to be able to
+        # convert on the target, so we need to put bytes-on-the-wire that can
+        # be converted.  That means inventory deltas (if the remote is <1.19,
+        # RemoteStreamSink will fallback to VFS to insert the deltas).
+        yield ('inventory-deltas',
+           self._stream_invs_as_deltas(revision_ids,
+                                       delta_versus_null=delta_versus_null))
+
+    def _stream_invs_as_deltas(self, revision_ids, delta_versus_null=False):
+        """Return a stream of inventory-deltas for the given rev ids.
+
+        :param revision_ids: The list of inventories to transmit
+        :param delta_versus_null: Don't try to find a minimal delta for this
+            entry, instead compute the delta versus the NULL_REVISION. This
+            effectively streams a complete inventory. Used for stuff like
+            filling in missing parents, etc.
+        """
+        from_repo = self.from_repository
+        revision_keys = [(rev_id,) for rev_id in revision_ids]
+        parent_map = from_repo.inventories.get_parent_map(revision_keys)
+        # XXX: possibly repos could implement a more efficient iter_inv_deltas
+        # method...
+        inventories = self.from_repository.iter_inventories(
+            revision_ids, 'topological')
+        format = from_repo._format
+        invs_sent_so_far = set([_mod_revision.NULL_REVISION])
+        inventory_cache = lru_cache.LRUCache(50)
+        null_inventory = from_repo.revision_tree(
+            _mod_revision.NULL_REVISION).inventory
+        # XXX: ideally the rich-root/tree-refs flags would be per-revision, not
+        # per-repo (e.g.  streaming a non-rich-root revision out of a rich-root
+        # repo back into a non-rich-root repo ought to be allowed)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            versioned_root=format.rich_root_data,
+            tree_references=format.supports_tree_reference)
+        for inv in inventories:
+            key = (inv.revision_id,)
+            parent_keys = parent_map.get(key, ())
+            delta = None
+            if not delta_versus_null and parent_keys:
+                # The caller did not ask for complete inventories and we have
+                # some parents that we can delta against.  Make a delta against
+                # each parent so that we can find the smallest.
+                parent_ids = [parent_key[0] for parent_key in parent_keys]
+                for parent_id in parent_ids:
+                    if parent_id not in invs_sent_so_far:
+                        # We don't know that the remote side has this basis, so
+                        # we can't use it.
+                        continue
+                    if parent_id == _mod_revision.NULL_REVISION:
+                        parent_inv = null_inventory
+                    else:
+                        parent_inv = inventory_cache.get(parent_id, None)
+                        if parent_inv is None:
+                            parent_inv = from_repo.get_inventory(parent_id)
+                    candidate_delta = inv._make_delta(parent_inv)
+                    if (delta is None or
+                        len(delta) > len(candidate_delta)):
+                        delta = candidate_delta
+                        basis_id = parent_id
+            if delta is None:
+                # Either none of the parents ended up being suitable, or we
+                # were asked to delta against NULL
+                basis_id = _mod_revision.NULL_REVISION
+                delta = inv._make_delta(null_inventory)
+            invs_sent_so_far.add(inv.revision_id)
+            inventory_cache[inv.revision_id] = inv
+            delta_serialized = ''.join(
+                serializer.delta_to_lines(basis_id, key[-1], delta))
+            yield versionedfile.FulltextContentFactory(
+                key, parent_keys, None, delta_serialized)
+
+
+class _VersionedFileChecker(object):
+
+    def __init__(self, repository, text_key_references=None, ancestors=None):
+        self.repository = repository
+        self.text_index = self.repository._generate_text_key_index(
+            text_key_references=text_key_references, ancestors=ancestors)
+
+    def calculate_file_version_parents(self, text_key):
+        """Calculate the correct parents for a file version according to
+        the inventories.
+        """
+        parent_keys = self.text_index[text_key]
+        if parent_keys == [_mod_revision.NULL_REVISION]:
+            return ()
+        return tuple(parent_keys)
+
+    def check_file_version_parents(self, texts, progress_bar=None):
+        """Check the parents stored in a versioned file are correct.
+
+        It also detects file versions that are not referenced by their
+        corresponding revision's inventory.
+
+        :returns: A tuple of (wrong_parents, dangling_file_versions).
+            wrong_parents is a dict mapping {revision_id: (stored_parents,
+            correct_parents)} for each revision_id where the stored parents
+            are not correct.  dangling_file_versions is a set of (file_id,
+            revision_id) tuples for versions that are present in this versioned
+            file, but not used by the corresponding inventory.
+        """
+        local_progress = None
+        if progress_bar is None:
+            local_progress = ui.ui_factory.nested_progress_bar()
+            progress_bar = local_progress
+        try:
+            return self._check_file_version_parents(texts, progress_bar)
+        finally:
+            if local_progress:
+                local_progress.finished()
+
+    def _check_file_version_parents(self, texts, progress_bar):
+        """See check_file_version_parents."""
+        wrong_parents = {}
+        self.file_ids = set([file_id for file_id, _ in
+            self.text_index.iterkeys()])
+        # text keys is now grouped by file_id
+        n_versions = len(self.text_index)
+        progress_bar.update(gettext('loading text store'), 0, n_versions)
+        parent_map = self.repository.texts.get_parent_map(self.text_index)
+        # On unlistable transports this could well be empty/error...
+        text_keys = self.repository.texts.keys()
+        unused_keys = frozenset(text_keys) - set(self.text_index)
+        for num, key in enumerate(self.text_index.iterkeys()):
+            progress_bar.update(gettext('checking text graph'), num, n_versions)
+            correct_parents = self.calculate_file_version_parents(key)
+            try:
+                knit_parents = parent_map[key]
+            except errors.RevisionNotPresent:
+                # Missing text!
+                knit_parents = None
+            if correct_parents != knit_parents:
+                wrong_parents[key] = (knit_parents, correct_parents)
+        return wrong_parents, unused_keys
+
+
+class InterVersionedFileRepository(InterRepository):
+
+    _walk_to_common_revisions_batch_size = 50
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, find_ghosts=False,
+            fetch_spec=None):
+        """Fetch the content required to construct revision_id.
+
+        The content is copied from self.source to self.target.
+
+        :param revision_id: if None all content is copied, if NULL_REVISION no
+                            content is copied.
+        :return: None.
+        """
+        if self.target._format.experimental:
+            ui.ui_factory.show_user_warning('experimental_format_fetch',
+                from_format=self.source._format,
+                to_format=self.target._format)
+        from bzrlib.fetch import RepoFetcher
+        # See <https://launchpad.net/bugs/456077> asking for a warning here
+        if self.source._format.network_name() != self.target._format.network_name():
+            ui.ui_factory.show_user_warning('cross_format_fetch',
+                from_format=self.source._format,
+                to_format=self.target._format)
+        f = RepoFetcher(to_repository=self.target,
+                               from_repository=self.source,
+                               last_revision=revision_id,
+                               fetch_spec=fetch_spec,
+                               find_ghosts=find_ghosts)
+
+    def _walk_to_common_revisions(self, revision_ids, if_present_ids=None):
+        """Walk out from revision_ids in source to revisions target has.
+
+        :param revision_ids: The start point for the search.
+        :return: A set of revision ids.
+        """
+        target_graph = self.target.get_graph()
+        revision_ids = frozenset(revision_ids)
+        if if_present_ids:
+            all_wanted_revs = revision_ids.union(if_present_ids)
+        else:
+            all_wanted_revs = revision_ids
+        missing_revs = set()
+        source_graph = self.source.get_graph()
+        # ensure we don't pay silly lookup costs.
+        searcher = source_graph._make_breadth_first_searcher(all_wanted_revs)
+        null_set = frozenset([_mod_revision.NULL_REVISION])
+        searcher_exhausted = False
+        while True:
+            next_revs = set()
+            ghosts = set()
+            # Iterate the searcher until we have enough next_revs
+            while len(next_revs) < self._walk_to_common_revisions_batch_size:
+                try:
+                    next_revs_part, ghosts_part = searcher.next_with_ghosts()
+                    next_revs.update(next_revs_part)
+                    ghosts.update(ghosts_part)
+                except StopIteration:
+                    searcher_exhausted = True
+                    break
+            # If there are ghosts in the source graph, and the caller asked for
+            # them, make sure that they are present in the target.
+            # We don't care about other ghosts as we can't fetch them and
+            # haven't been asked to.
+            ghosts_to_check = set(revision_ids.intersection(ghosts))
+            revs_to_get = set(next_revs).union(ghosts_to_check)
+            if revs_to_get:
+                have_revs = set(target_graph.get_parent_map(revs_to_get))
+                # we always have NULL_REVISION present.
+                have_revs = have_revs.union(null_set)
+                # Check if the target is missing any ghosts we need.
+                ghosts_to_check.difference_update(have_revs)
+                if ghosts_to_check:
+                    # One of the caller's revision_ids is a ghost in both the
+                    # source and the target.
+                    raise errors.NoSuchRevision(
+                        self.source, ghosts_to_check.pop())
+                missing_revs.update(next_revs - have_revs)
+                # Because we may have walked past the original stop point, make
+                # sure everything is stopped
+                stop_revs = searcher.find_seen_ancestors(have_revs)
+                searcher.stop_searching_any(stop_revs)
+            if searcher_exhausted:
+                break
+        return searcher.get_result()
+
+    @needs_read_lock
+    def search_missing_revision_ids(self,
+            revision_id=symbol_versioning.DEPRECATED_PARAMETER,
+            find_ghosts=True, revision_ids=None, if_present_ids=None,
+            limit=None):
+        """Return the revision ids that source has that target does not.
+
+        :param revision_id: only return revision ids included by this
+            revision_id.
+        :param revision_ids: return revision ids included by these
+            revision_ids.  NoSuchRevision will be raised if any of these
+            revisions are not present.
+        :param if_present_ids: like revision_ids, but will not cause
+            NoSuchRevision if any of these are absent, instead they will simply
+            not be in the result.  This is useful for e.g. finding revisions
+            to fetch for tags, which may reference absent revisions.
+        :param find_ghosts: If True find missing revisions in deep history
+            rather than just finding the surface difference.
+        :return: A bzrlib.graph.SearchResult.
+        """
+        if symbol_versioning.deprecated_passed(revision_id):
+            symbol_versioning.warn(
+                'search_missing_revision_ids(revision_id=...) was '
+                'deprecated in 2.4.  Use revision_ids=[...] instead.',
+                DeprecationWarning, stacklevel=2)
+            if revision_ids is not None:
+                raise AssertionError(
+                    'revision_ids is mutually exclusive with revision_id')
+            if revision_id is not None:
+                revision_ids = [revision_id]
+        del revision_id
+        # stop searching at found target revisions.
+        if not find_ghosts and (revision_ids is not None or if_present_ids is
+                not None):
+            result = self._walk_to_common_revisions(revision_ids,
+                    if_present_ids=if_present_ids)
+            if limit is None:
+                return result
+            result_set = result.get_keys()
+        else:
+            # generic, possibly worst case, slow code path.
+            target_ids = set(self.target.all_revision_ids())
+            source_ids = self._present_source_revisions_for(
+                revision_ids, if_present_ids)
+            result_set = set(source_ids).difference(target_ids)
+        if limit is not None:
+            topo_ordered = self.source.get_graph().iter_topo_order(result_set)
+            result_set = set(itertools.islice(topo_ordered, limit))
+        return self.source.revision_ids_to_search_result(result_set)
+
+    def _present_source_revisions_for(self, revision_ids, if_present_ids=None):
+        """Returns set of all revisions in ancestry of revision_ids present in
+        the source repo.
+
+        :param revision_ids: if None, all revisions in source are returned.
+        :param if_present_ids: like revision_ids, but if any/all of these are
+            absent no error is raised.
+        """
+        if revision_ids is not None or if_present_ids is not None:
+            # First, ensure all specified revisions exist.  Callers expect
+            # NoSuchRevision when they pass absent revision_ids here.
+            if revision_ids is None:
+                revision_ids = set()
+            if if_present_ids is None:
+                if_present_ids = set()
+            revision_ids = set(revision_ids)
+            if_present_ids = set(if_present_ids)
+            all_wanted_ids = revision_ids.union(if_present_ids)
+            graph = self.source.get_graph()
+            present_revs = set(graph.get_parent_map(all_wanted_ids))
+            missing = revision_ids.difference(present_revs)
+            if missing:
+                raise errors.NoSuchRevision(self.source, missing.pop())
+            found_ids = all_wanted_ids.intersection(present_revs)
+            source_ids = [rev_id for (rev_id, parents) in
+                          graph.iter_ancestry(found_ids)
+                          if rev_id != _mod_revision.NULL_REVISION
+                          and parents is not None]
+        else:
+            source_ids = self.source.all_revision_ids()
+        return set(source_ids)
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return None
+
+    @classmethod
+    def is_compatible(cls, source, target):
+        # The default implementation is compatible with everything
+        return (source._format.supports_full_versioned_files and
+                target._format.supports_full_versioned_files)
+
+
+class InterDifferingSerializer(InterVersionedFileRepository):
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        return None
+
+    @staticmethod
+    def is_compatible(source, target):
+        if not source._format.supports_full_versioned_files:
+            return False
+        if not target._format.supports_full_versioned_files:
+            return False
+        # This is redundant with format.check_conversion_target(), however that
+        # raises an exception, and we just want to say "False" as in we won't
+        # support converting between these formats.
+        if 'IDS_never' in debug.debug_flags:
+            return False
+        if source.supports_rich_root() and not target.supports_rich_root():
+            return False
+        if (source._format.supports_tree_reference
+            and not target._format.supports_tree_reference):
+            return False
+        if target._fallback_repositories and target._format.supports_chks:
+            # IDS doesn't know how to copy CHKs for the parent inventories it
+            # adds to stacked repos.
+            return False
+        if 'IDS_always' in debug.debug_flags:
+            return True
+        # Only use this code path for local source and target.  IDS does far
+        # too much IO (both bandwidth and roundtrips) over a network.
+        if not source.bzrdir.transport.base.startswith('file:///'):
+            return False
+        if not target.bzrdir.transport.base.startswith('file:///'):
+            return False
+        return True
+
+    def _get_trees(self, revision_ids, cache):
+        possible_trees = []
+        for rev_id in revision_ids:
+            if rev_id in cache:
+                possible_trees.append((rev_id, cache[rev_id]))
+            else:
+                # Not cached, but inventory might be present anyway.
+                try:
+                    tree = self.source.revision_tree(rev_id)
+                except errors.NoSuchRevision:
+                    # Nope, parent is ghost.
+                    pass
+                else:
+                    cache[rev_id] = tree
+                    possible_trees.append((rev_id, tree))
+        return possible_trees
+
+    def _get_delta_for_revision(self, tree, parent_ids, possible_trees):
+        """Get the best delta and base for this revision.
+
+        :return: (basis_id, delta)
+        """
+        deltas = []
+        # Generate deltas against each tree, to find the shortest.
+        texts_possibly_new_in_tree = set()
+        for basis_id, basis_tree in possible_trees:
+            delta = tree.inventory._make_delta(basis_tree.inventory)
+            for old_path, new_path, file_id, new_entry in delta:
+                if new_path is None:
+                    # This file_id isn't present in the new rev, so we don't
+                    # care about it.
+                    continue
+                if not new_path:
+                    # Rich roots are handled elsewhere...
+                    continue
+                kind = new_entry.kind
+                if kind != 'directory' and kind != 'file':
+                    # No text record associated with this inventory entry.
+                    continue
+                # This is a directory or file that has changed somehow.
+                texts_possibly_new_in_tree.add((file_id, new_entry.revision))
+            deltas.append((len(delta), basis_id, delta))
+        deltas.sort()
+        return deltas[0][1:]
+
+    def _fetch_parent_invs_for_stacking(self, parent_map, cache):
+        """Find all parent revisions that are absent, but for which the
+        inventory is present, and copy those inventories.
+
+        This is necessary to preserve correctness when the source is stacked
+        without fallbacks configured.  (Note that in cases like upgrade the
+        source may be not have _fallback_repositories even though it is
+        stacked.)
+        """
+        parent_revs = set()
+        for parents in parent_map.values():
+            parent_revs.update(parents)
+        present_parents = self.source.get_parent_map(parent_revs)
+        absent_parents = set(parent_revs).difference(present_parents)
+        parent_invs_keys_for_stacking = self.source.inventories.get_parent_map(
+            (rev_id,) for rev_id in absent_parents)
+        parent_inv_ids = [key[-1] for key in parent_invs_keys_for_stacking]
+        for parent_tree in self.source.revision_trees(parent_inv_ids):
+            current_revision_id = parent_tree.get_revision_id()
+            parents_parents_keys = parent_invs_keys_for_stacking[
+                (current_revision_id,)]
+            parents_parents = [key[-1] for key in parents_parents_keys]
+            basis_id = _mod_revision.NULL_REVISION
+            basis_tree = self.source.revision_tree(basis_id)
+            delta = parent_tree.inventory._make_delta(basis_tree.inventory)
+            self.target.add_inventory_by_delta(
+                basis_id, delta, current_revision_id, parents_parents)
+            cache[current_revision_id] = parent_tree
+
+    def _fetch_batch(self, revision_ids, basis_id, cache):
+        """Fetch across a few revisions.
+
+        :param revision_ids: The revisions to copy
+        :param basis_id: The revision_id of a tree that must be in cache, used
+            as a basis for delta when no other base is available
+        :param cache: A cache of RevisionTrees that we can use.
+        :return: The revision_id of the last converted tree. The RevisionTree
+            for it will be in cache
+        """
+        # Walk though all revisions; get inventory deltas, copy referenced
+        # texts that delta references, insert the delta, revision and
+        # signature.
+        root_keys_to_create = set()
+        text_keys = set()
+        pending_deltas = []
+        pending_revisions = []
+        parent_map = self.source.get_parent_map(revision_ids)
+        self._fetch_parent_invs_for_stacking(parent_map, cache)
+        self.source._safe_to_return_from_cache = True
+        for tree in self.source.revision_trees(revision_ids):
+            # Find a inventory delta for this revision.
+            # Find text entries that need to be copied, too.
+            current_revision_id = tree.get_revision_id()
+            parent_ids = parent_map.get(current_revision_id, ())
+            parent_trees = self._get_trees(parent_ids, cache)
+            possible_trees = list(parent_trees)
+            if len(possible_trees) == 0:
+                # There either aren't any parents, or the parents are ghosts,
+                # so just use the last converted tree.
+                possible_trees.append((basis_id, cache[basis_id]))
+            basis_id, delta = self._get_delta_for_revision(tree, parent_ids,
+                                                           possible_trees)
+            revision = self.source.get_revision(current_revision_id)
+            pending_deltas.append((basis_id, delta,
+                current_revision_id, revision.parent_ids))
+            if self._converting_to_rich_root:
+                self._revision_id_to_root_id[current_revision_id] = \
+                    tree.get_root_id()
+            # Determine which texts are in present in this revision but not in
+            # any of the available parents.
+            texts_possibly_new_in_tree = set()
+            for old_path, new_path, file_id, entry in delta:
+                if new_path is None:
+                    # This file_id isn't present in the new rev
+                    continue
+                if not new_path:
+                    # This is the root
+                    if not self.target.supports_rich_root():
+                        # The target doesn't support rich root, so we don't
+                        # copy
+                        continue
+                    if self._converting_to_rich_root:
+                        # This can't be copied normally, we have to insert
+                        # it specially
+                        root_keys_to_create.add((file_id, entry.revision))
+                        continue
+                kind = entry.kind
+                texts_possibly_new_in_tree.add((file_id, entry.revision))
+            for basis_id, basis_tree in possible_trees:
+                basis_inv = basis_tree.inventory
+                for file_key in list(texts_possibly_new_in_tree):
+                    file_id, file_revision = file_key
+                    try:
+                        entry = basis_inv[file_id]
+                    except errors.NoSuchId:
+                        continue
+                    if entry.revision == file_revision:
+                        texts_possibly_new_in_tree.remove(file_key)
+            text_keys.update(texts_possibly_new_in_tree)
+            pending_revisions.append(revision)
+            cache[current_revision_id] = tree
+            basis_id = current_revision_id
+        self.source._safe_to_return_from_cache = False
+        # Copy file texts
+        from_texts = self.source.texts
+        to_texts = self.target.texts
+        if root_keys_to_create:
+            root_stream = _mod_fetch._new_root_data_stream(
+                root_keys_to_create, self._revision_id_to_root_id, parent_map,
+                self.source)
+            to_texts.insert_record_stream(root_stream)
+        to_texts.insert_record_stream(from_texts.get_record_stream(
+            text_keys, self.target._format._fetch_order,
+            not self.target._format._fetch_uses_deltas))
+        # insert inventory deltas
+        for delta in pending_deltas:
+            self.target.add_inventory_by_delta(*delta)
+        if self.target._fallback_repositories:
+            # Make sure this stacked repository has all the parent inventories
+            # for the new revisions that we are about to insert.  We do this
+            # before adding the revisions so that no revision is added until
+            # all the inventories it may depend on are added.
+            # Note that this is overzealous, as we may have fetched these in an
+            # earlier batch.
+            parent_ids = set()
+            revision_ids = set()
+            for revision in pending_revisions:
+                revision_ids.add(revision.revision_id)
+                parent_ids.update(revision.parent_ids)
+            parent_ids.difference_update(revision_ids)
+            parent_ids.discard(_mod_revision.NULL_REVISION)
+            parent_map = self.source.get_parent_map(parent_ids)
+            # we iterate over parent_map and not parent_ids because we don't
+            # want to try copying any revision which is a ghost
+            for parent_tree in self.source.revision_trees(parent_map):
+                current_revision_id = parent_tree.get_revision_id()
+                parents_parents = parent_map[current_revision_id]
+                possible_trees = self._get_trees(parents_parents, cache)
+                if len(possible_trees) == 0:
+                    # There either aren't any parents, or the parents are
+                    # ghosts, so just use the last converted tree.
+                    possible_trees.append((basis_id, cache[basis_id]))
+                basis_id, delta = self._get_delta_for_revision(parent_tree,
+                    parents_parents, possible_trees)
+                self.target.add_inventory_by_delta(
+                    basis_id, delta, current_revision_id, parents_parents)
+        # insert signatures and revisions
+        for revision in pending_revisions:
+            try:
+                signature = self.source.get_signature_text(
+                    revision.revision_id)
+                self.target.add_signature_text(revision.revision_id,
+                    signature)
+            except errors.NoSuchRevision:
+                pass
+            self.target.add_revision(revision.revision_id, revision)
+        return basis_id
+
+    def _fetch_all_revisions(self, revision_ids, pb):
+        """Fetch everything for the list of revisions.
+
+        :param revision_ids: The list of revisions to fetch. Must be in
+            topological order.
+        :param pb: A ProgressTask
+        :return: None
+        """
+        basis_id, basis_tree = self._get_basis(revision_ids[0])
+        batch_size = 100
+        cache = lru_cache.LRUCache(100)
+        cache[basis_id] = basis_tree
+        del basis_tree # We don't want to hang on to it here
+        hints = []
+        a_graph = None
+
+        for offset in range(0, len(revision_ids), batch_size):
+            self.target.start_write_group()
+            try:
+                pb.update(gettext('Transferring revisions'), offset,
+                          len(revision_ids))
+                batch = revision_ids[offset:offset+batch_size]
+                basis_id = self._fetch_batch(batch, basis_id, cache)
+            except:
+                self.source._safe_to_return_from_cache = False
+                self.target.abort_write_group()
+                raise
+            else:
+                hint = self.target.commit_write_group()
+                if hint:
+                    hints.extend(hint)
+        if hints and self.target._format.pack_compresses:
+            self.target.pack(hint=hints)
+        pb.update(gettext('Transferring revisions'), len(revision_ids),
+                  len(revision_ids))
+
+    @needs_write_lock
+    def fetch(self, revision_id=None, find_ghosts=False,
+            fetch_spec=None):
+        """See InterRepository.fetch()."""
+        if fetch_spec is not None:
+            revision_ids = fetch_spec.get_keys()
+        else:
+            revision_ids = None
+        if self.source._format.experimental:
+            ui.ui_factory.show_user_warning('experimental_format_fetch',
+                from_format=self.source._format,
+                to_format=self.target._format)
+        if (not self.source.supports_rich_root()
+            and self.target.supports_rich_root()):
+            self._converting_to_rich_root = True
+            self._revision_id_to_root_id = {}
+        else:
+            self._converting_to_rich_root = False
+        # See <https://launchpad.net/bugs/456077> asking for a warning here
+        if self.source._format.network_name() != self.target._format.network_name():
+            ui.ui_factory.show_user_warning('cross_format_fetch',
+                from_format=self.source._format,
+                to_format=self.target._format)
+        if revision_ids is None:
+            if revision_id:
+                search_revision_ids = [revision_id]
+            else:
+                search_revision_ids = None
+            revision_ids = self.target.search_missing_revision_ids(self.source,
+                revision_ids=search_revision_ids,
+                find_ghosts=find_ghosts).get_keys()
+        if not revision_ids:
+            return 0, 0
+        revision_ids = tsort.topo_sort(
+            self.source.get_graph().get_parent_map(revision_ids))
+        if not revision_ids:
+            return 0, 0
+        # Walk though all revisions; get inventory deltas, copy referenced
+        # texts that delta references, insert the delta, revision and
+        # signature.
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            self._fetch_all_revisions(revision_ids, pb)
+        finally:
+            pb.finished()
+        return len(revision_ids), 0
+
+    def _get_basis(self, first_revision_id):
+        """Get a revision and tree which exists in the target.
+
+        This assumes that first_revision_id is selected for transmission
+        because all other ancestors are already present. If we can't find an
+        ancestor we fall back to NULL_REVISION since we know that is safe.
+
+        :return: (basis_id, basis_tree)
+        """
+        first_rev = self.source.get_revision(first_revision_id)
+        try:
+            basis_id = first_rev.parent_ids[0]
+            # only valid as a basis if the target has it
+            self.target.get_revision(basis_id)
+            # Try to get a basis tree - if it's a ghost it will hit the
+            # NoSuchRevision case.
+            basis_tree = self.source.revision_tree(basis_id)
+        except (IndexError, errors.NoSuchRevision):
+            basis_id = _mod_revision.NULL_REVISION
+            basis_tree = self.source.revision_tree(basis_id)
+        return basis_id, basis_tree
+
+
+class InterSameDataRepository(InterVersionedFileRepository):
+    """Code for converting between repositories that represent the same data.
+
+    Data format and model must match for this to work.
+    """
+
+    @classmethod
+    def _get_repo_format_to_test(self):
+        """Repository format for testing with.
+
+        InterSameData can pull from subtree to subtree and from non-subtree to
+        non-subtree, so we test this with the richest repository format.
+        """
+        from bzrlib.repofmt import knitrepo
+        return knitrepo.RepositoryFormatKnit3()
+
+    @staticmethod
+    def is_compatible(source, target):
+        return (
+            InterRepository._same_model(source, target) and
+            source._format.supports_full_versioned_files and
+            target._format.supports_full_versioned_files)
+
+
+InterRepository.register_optimiser(InterVersionedFileRepository)
+InterRepository.register_optimiser(InterDifferingSerializer)
+InterRepository.register_optimiser(InterSameDataRepository)
+
+
+def install_revisions(repository, iterable, num_revisions=None, pb=None):
+    """Install all revision data into a repository.
+
+    Accepts an iterable of revision, tree, signature tuples.  The signature
+    may be None.
+    """
+    repository.start_write_group()
+    try:
+        inventory_cache = lru_cache.LRUCache(10)
+        for n, (revision, revision_tree, signature) in enumerate(iterable):
+            _install_revision(repository, revision, revision_tree, signature,
+                inventory_cache)
+            if pb is not None:
+                pb.update(gettext('Transferring revisions'), n + 1, num_revisions)
+    except:
+        repository.abort_write_group()
+        raise
+    else:
+        repository.commit_write_group()
+
+
+def _install_revision(repository, rev, revision_tree, signature,
+    inventory_cache):
+    """Install all revision data into a repository."""
+    present_parents = []
+    parent_trees = {}
+    for p_id in rev.parent_ids:
+        if repository.has_revision(p_id):
+            present_parents.append(p_id)
+            parent_trees[p_id] = repository.revision_tree(p_id)
+        else:
+            parent_trees[p_id] = repository.revision_tree(
+                                     _mod_revision.NULL_REVISION)
+
+    inv = revision_tree.inventory
+    entries = inv.iter_entries()
+    # backwards compatibility hack: skip the root id.
+    if not repository.supports_rich_root():
+        path, root = entries.next()
+        if root.revision != rev.revision_id:
+            raise errors.IncompatibleRevision(repr(repository))
+    text_keys = {}
+    for path, ie in entries:
+        text_keys[(ie.file_id, ie.revision)] = ie
+    text_parent_map = repository.texts.get_parent_map(text_keys)
+    missing_texts = set(text_keys) - set(text_parent_map)
+    # Add the texts that are not already present
+    for text_key in missing_texts:
+        ie = text_keys[text_key]
+        text_parents = []
+        # FIXME: TODO: The following loop overlaps/duplicates that done by
+        # commit to determine parents. There is a latent/real bug here where
+        # the parents inserted are not those commit would do - in particular
+        # they are not filtered by heads(). RBC, AB
+        for revision, tree in parent_trees.iteritems():
+            if not tree.has_id(ie.file_id):
+                continue
+            parent_id = tree.get_file_revision(ie.file_id)
+            if parent_id in text_parents:
+                continue
+            text_parents.append((ie.file_id, parent_id))
+        lines = revision_tree.get_file(ie.file_id).readlines()
+        repository.texts.add_lines(text_key, text_parents, lines)
+    try:
+        # install the inventory
+        if repository._format._commit_inv_deltas and len(rev.parent_ids):
+            # Cache this inventory
+            inventory_cache[rev.revision_id] = inv
+            try:
+                basis_inv = inventory_cache[rev.parent_ids[0]]
+            except KeyError:
+                repository.add_inventory(rev.revision_id, inv, present_parents)
+            else:
+                delta = inv._make_delta(basis_inv)
+                repository.add_inventory_by_delta(rev.parent_ids[0], delta,
+                    rev.revision_id, present_parents)
+        else:
+            repository.add_inventory(rev.revision_id, inv, present_parents)
+    except errors.RevisionAlreadyPresent:
+        pass
+    if signature is not None:
+        repository.add_signature_text(rev.revision_id, signature)
+    repository.add_revision(rev.revision_id, rev, inv)
+
+
+def install_revision(repository, rev, revision_tree):
+    """Install all revision data into a repository."""
+    install_revisions(repository, [(rev, revision_tree, None)])

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2009-09-20 22:12:36 +0000
+++ bzrlib/weavefile.py	2011-02-05 14:24:01 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005-2010 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by

=== modified file 'bzrlib/win32utils.py'
--- bzrlib/win32utils.py	2010-06-22 00:32:37 +0000
+++ bzrlib/win32utils.py	2011-09-19 15:59:40 +0000
@@ -25,6 +25,7 @@
 import sys
 
 from bzrlib import cmdline
+from bzrlib.i18n import gettext
 
 # Windows version
 if sys.platform == 'win32':
@@ -128,7 +129,7 @@
             ctypes.byref(mem_struct),
             ctypes.sizeof(mem_struct))
         if not ret:
-            trace.note('Failed to GetProcessMemoryInfo()')
+            trace.note(gettext('Failed to GetProcessMemoryInfo()'))
             return
         info = {'PageFaultCount': mem_struct.PageFaultCount,
                 'PeakWorkingSetSize': mem_struct.PeakWorkingSetSize,
@@ -149,26 +150,26 @@
         proc = win32process.GetCurrentProcess()
         info = win32process.GetProcessMemoryInfo(proc)
     else:
-        trace.note('Cannot debug memory on win32 without ctypes'
-                   ' or win32process')
+        trace.note(gettext('Cannot debug memory on win32 without ctypes'
+                   ' or win32process'))
         return
     if short:
         # using base-2 units (see HACKING.txt).
-        trace.note('WorkingSize %7dKiB'
-                   '\tPeakWorking %7dKiB\t%s',
+        trace.note(gettext('WorkingSize {0:>7}KiB'
+                   '\tPeakWorking {1:>7}KiB\t{2}').format(
                    info['WorkingSetSize'] / 1024,
                    info['PeakWorkingSetSize'] / 1024,
-                   message)
+                   message))
         return
     if message:
         trace.note('%s', message)
-    trace.note('WorkingSize       %8d KiB', info['WorkingSetSize'] / 1024)
-    trace.note('PeakWorking       %8d KiB', info['PeakWorkingSetSize'] / 1024)
-    trace.note('PagefileUsage     %8d KiB', info.get('PagefileUsage', 0) / 1024)
-    trace.note('PeakPagefileUsage %8d KiB',
+    trace.note(gettext('WorkingSize       %8d KiB'), info['WorkingSetSize'] / 1024)
+    trace.note(gettext('PeakWorking       %8d KiB'), info['PeakWorkingSetSize'] / 1024)
+    trace.note(gettext('PagefileUsage     %8d KiB'), info.get('PagefileUsage', 0) / 1024)
+    trace.note(gettext('PeakPagefileUsage %8d KiB'),
                info.get('PeakPagefileUsage', 0) / 1024)
-    trace.note('PrivateUsage      %8d KiB', info.get('PrivateUsage', 0) / 1024)
-    trace.note('PageFaultCount    %8d', info.get('PageFaultCount', 0))
+    trace.note(gettext('PrivateUsage      %8d KiB'), info.get('PrivateUsage', 0) / 1024)
+    trace.note(gettext('PageFaultCount    %8d'), info.get('PageFaultCount', 0))
 
 
 def get_console_size(defaultx=80, defaulty=25):
@@ -468,7 +469,7 @@
 
 
 def get_app_path(appname):
-    """Look up in Windows registry for full path to application executable.
+    r"""Look up in Windows registry for full path to application executable.
     Typically, applications create subkey with their basename
     in HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\
 
@@ -576,3 +577,41 @@
         return argv
 else:
     get_unicode_argv = None
+
+
+if has_win32api:
+    def _pywin32_is_local_pid_dead(pid):
+        """True if pid doesn't correspond to live process on this machine"""
+        try:
+            handle = win32api.OpenProcess(1, False, pid) # PROCESS_TERMINATE
+        except pywintypes.error, e:
+            if e[0] == 5: # ERROR_ACCESS_DENIED
+                # Probably something alive we're not allowed to kill
+                return False
+            elif e[0] == 87: # ERROR_INVALID_PARAMETER
+                return True
+            raise
+        handle.close()
+        return False
+    is_local_pid_dead = _pywin32_is_local_pid_dead
+elif has_ctypes and sys.platform == 'win32':
+    from ctypes.wintypes import BOOL, DWORD, HANDLE
+    _kernel32 = ctypes.windll.kernel32
+    _CloseHandle = ctypes.WINFUNCTYPE(BOOL, HANDLE)(
+        ("CloseHandle", _kernel32))
+    _OpenProcess = ctypes.WINFUNCTYPE(HANDLE, DWORD, BOOL, DWORD)(
+        ("OpenProcess", _kernel32))
+    def _ctypes_is_local_pid_dead(pid):
+        """True if pid doesn't correspond to live process on this machine"""
+        handle = _OpenProcess(1, False, pid) # PROCESS_TERMINATE
+        if not handle:
+            errorcode = ctypes.GetLastError()
+            if errorcode == 5: # ERROR_ACCESS_DENIED
+                # Probably something alive we're not allowed to kill
+                return False
+            elif errorcode == 87: # ERROR_INVALID_PARAMETER
+                return True
+            raise ctypes.WinError(errorcode)
+        _CloseHandle(handle)
+        return False
+    is_local_pid_dead = _ctypes_is_local_pid_dead

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2010-07-19 14:26:11 +0000
+++ bzrlib/workingtree.py	2011-10-13 16:30:39 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,22 +44,22 @@
 import stat
 import re
 
-import bzrlib
 from bzrlib import (
     branch,
     bzrdir,
     conflicts as _mod_conflicts,
+    controldir,
     errors,
+    filters as _mod_filters,
     generate_ids,
     globbing,
     graph as _mod_graph,
-    hashcache,
     ignores,
     inventory,
     merge,
     revision as _mod_revision,
     revisiontree,
-    trace,
+    rio as _mod_rio,
     transform,
     transport,
     ui,
@@ -67,18 +67,12 @@
     xml5,
     xml7,
     )
-from bzrlib.workingtree_4 import (
-    WorkingTreeFormat4,
-    WorkingTreeFormat5,
-    WorkingTreeFormat6,
-    )
 """)
 
 from bzrlib import symbol_versioning
 from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.i18n import gettext
 from bzrlib.lock import LogicalLockResult
-from bzrlib.lockable_files import LockableFiles
-from bzrlib.lockdir import LockDir
 import bzrlib.mutabletree
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib import osutils
@@ -92,11 +86,8 @@
     splitpath,
     supports_executable,
     )
-from bzrlib.filters import filtered_input_file
 from bzrlib.trace import mutter, note
-from bzrlib.transport.local import LocalTransport
 from bzrlib.revision import CURRENT_REVISION
-from bzrlib.rio import RioReader, rio_file, Stanza
 from bzrlib.symbol_versioning import (
     deprecated_passed,
     DEPRECATED_PARAMETER,
@@ -168,15 +159,9 @@
 
 
 class WorkingTree(bzrlib.mutabletree.MutableTree,
-    bzrdir.ControlComponent):
+    controldir.ControlComponent):
     """Working copy tree.
 
-    The inventory is held in the `Branch` working-inventory, and the
-    files are in a directory on disk.
-
-    It is possible for a `WorkingTree` to have a filename which is
-    not listed in the Inventory and vice versa.
-
     :ivar basedir: The root of the tree on disk. This is a unicode path object
         (as opposed to a URL).
     """
@@ -187,7 +172,6 @@
 
     def __init__(self, basedir='.',
                  branch=DEPRECATED_PARAMETER,
-                 _inventory=None,
                  _control_files=None,
                  _internal=False,
                  _format=None,
@@ -208,45 +192,8 @@
         else:
             self._branch = self.bzrdir.open_branch()
         self.basedir = realpath(basedir)
-        # if branch is at our basedir and is a format 6 or less
-        if isinstance(self._format, WorkingTreeFormat2):
-            # share control object
-            self._control_files = self.branch.control_files
-        else:
-            # assume all other formats have their own control files.
-            self._control_files = _control_files
+        self._control_files = _control_files
         self._transport = self._control_files._transport
-        # update the whole cache up front and write to disk if anything changed;
-        # in the future we might want to do this more selectively
-        # two possible ways offer themselves : in self._unlock, write the cache
-        # if needed, or, when the cache sees a change, append it to the hash
-        # cache file, and have the parser take the most recent entry for a
-        # given path only.
-        wt_trans = self.bzrdir.get_workingtree_transport(None)
-        cache_filename = wt_trans.local_abspath('stat-cache')
-        self._hashcache = hashcache.HashCache(basedir, cache_filename,
-            self.bzrdir._get_file_mode(),
-            self._content_filter_stack_provider())
-        hc = self._hashcache
-        hc.read()
-        # is this scan needed ? it makes things kinda slow.
-        #hc.scan()
-
-        if hc.needs_write:
-            mutter("write hc")
-            hc.write()
-
-        if _inventory is None:
-            # This will be acquired on lock_read() or lock_write()
-            self._inventory_is_modified = False
-            self._inventory = None
-        else:
-            # the caller of __init__ has provided an inventory,
-            # we assume they know what they are doing - as its only
-            # the Format factory and creation methods that are
-            # permitted to do this.
-            self._set_inventory(_inventory, dirty=False)
-        self._detect_case_handling()
         self._rules_searcher = None
         self.views = self._make_views()
 
@@ -258,16 +205,17 @@
     def control_transport(self):
         return self._transport
 
-    def _detect_case_handling(self):
-        wt_trans = self.bzrdir.get_workingtree_transport(None)
-        try:
-            wt_trans.stat("FoRMaT")
-        except errors.NoSuchFile:
-            self.case_sensitive = True
-        else:
-            self.case_sensitive = False
-
-        self._setup_directory_is_tree_reference()
+    def is_control_filename(self, filename):
+        """True if filename is the name of a control file in this tree.
+
+        :param filename: A filename within the tree. This is a relative path
+            from the root of this tree.
+
+        This is true IF and ONLY IF the filename is part of the meta data
+        that bzr controls in this tree. I.E. a random .bzr directory placed
+        on disk will not be a control file for this tree.
+        """
+        return self.bzrdir.is_control_filename(filename)
 
     branch = property(
         fget=lambda self: self._branch,
@@ -277,6 +225,10 @@
             the working tree has been constructed from.
             """)
 
+    def has_versioned_directories(self):
+        """See `Tree.has_versioned_directories`."""
+        return self._format.supports_versioned_directories
+
     def break_lock(self):
         """Break a lock if one is present from another instance.
 
@@ -288,16 +240,6 @@
         self._control_files.break_lock()
         self.branch.break_lock()
 
-    def _get_check_refs(self):
-        """Return the references needed to perform a check of this tree.
-        
-        The default implementation returns no refs, and is only suitable for
-        trees that have no local caching and can commit on ghosts at any time.
-
-        :seealso: bzrlib.check for details about check_refs.
-        """
-        return []
-
     def requires_rich_root(self):
         return self._format.requires_rich_root
 
@@ -310,19 +252,6 @@
     def supports_views(self):
         return self.views.supports_views()
 
-    def _set_inventory(self, inv, dirty):
-        """Set the internal cached inventory.
-
-        :param inv: The inventory to set.
-        :param dirty: A boolean indicating whether the inventory is the same
-            logical inventory as whats on disk. If True the inventory is not
-            the same and should be written to disk or data will be lost, if
-            False then the inventory is the same as that on disk and any
-            serialisation would be unneeded overhead.
-        """
-        self._inventory = inv
-        self._inventory_is_modified = dirty
-
     @staticmethod
     def open(path=None, _unsupported=False):
         """Open an existing working tree at path.
@@ -330,7 +259,7 @@
         """
         if path is None:
             path = osutils.getcwd()
-        control = bzrdir.BzrDir.open(path, _unsupported)
+        control = controldir.ControlDir.open(path, _unsupported)
         return control.open_workingtree(_unsupported)
 
     @staticmethod
@@ -348,12 +277,12 @@
         """
         if path is None:
             path = osutils.getcwd()
-        control, relpath = bzrdir.BzrDir.open_containing(path)
+        control, relpath = controldir.ControlDir.open_containing(path)
         return control.open_workingtree(), relpath
 
     @staticmethod
-    def open_containing_paths(file_list, default_directory='.',
-        canonicalize=True, apply_view=True):
+    def open_containing_paths(file_list, default_directory=None,
+                              canonicalize=True, apply_view=True):
         """Open the WorkingTree that contains a set of paths.
 
         Fail if the paths given are not all in a single tree.
@@ -361,6 +290,8 @@
         This is used for the many command-line interfaces that take a list of
         any number of files and that require they all be in the same tree.
         """
+        if default_directory is None:
+            default_directory = u'.'
         # recommended replacement for builtins.internal_tree_files
         if file_list is None or len(file_list) == 0:
             tree = WorkingTree.open_containing(default_directory)[0]
@@ -372,11 +303,17 @@
                 if view_files:
                     file_list = view_files
                     view_str = views.view_display_str(view_files)
-                    note("Ignoring files outside view. View is %s" % view_str)
+                    note(gettext("Ignoring files outside view. View is %s") % view_str)
             return tree, file_list
-        tree = WorkingTree.open_containing(file_list[0])[0]
+        if default_directory == u'.':
+            seed = file_list[0]
+        else:
+            seed = default_directory
+            file_list = [osutils.pathjoin(default_directory, f)
+                         for f in file_list]
+        tree = WorkingTree.open_containing(seed)[0]
         return tree, tree.safe_relpath_files(file_list, canonicalize,
-            apply_view=apply_view)
+                                             apply_view=apply_view)
 
     def safe_relpath_files(self, file_list, canonicalize=True, apply_view=True):
         """Convert file_list into a list of relpaths in tree.
@@ -429,27 +366,13 @@
             else:
                 return True, tree
         t = transport.get_transport(location)
-        iterator = bzrdir.BzrDir.find_bzrdirs(t, evaluate=evaluate,
+        iterator = controldir.ControlDir.find_bzrdirs(t, evaluate=evaluate,
                                               list_current=list_current)
         return [tr for tr in iterator if tr is not None]
 
-    # should be deprecated - this is slow and in any case treating them as a
-    # container is (we now know) bad style -- mbp 20070302
-    ## @deprecated_method(zero_fifteen)
-    def __iter__(self):
-        """Iterate through file_ids for this tree.
-
-        file_ids are in a WorkingTree if they are in the working inventory
-        and the working file exists.
-        """
-        inv = self._inventory
-        for path, ie in inv.iter_entries():
-            if osutils.lexists(self.abspath(path)):
-                yield ie.file_id
-
     def all_file_ids(self):
         """See Tree.iter_all_file_ids"""
-        return set(self.inventory)
+        raise NotImplementedError(self.all_file_ids)
 
     def __repr__(self):
         return "<%s of %s>" % (self.__class__.__name__,
@@ -510,7 +433,7 @@
         return self.get_file_with_stat(file_id, path, filtered=filtered)[0]
 
     def get_file_with_stat(self, file_id, path=None, filtered=True,
-        _fstat=os.fstat):
+                           _fstat=osutils.fstat):
         """See Tree.get_file_with_stat."""
         if path is None:
             path = self.id2path(file_id)
@@ -518,7 +441,7 @@
         stat_value = _fstat(file_obj.fileno())
         if filtered and self.supports_content_filtering():
             filters = self._content_filter_stack(path)
-            file_obj = filtered_input_file(file_obj, filters)
+            file_obj = _mod_filters.filtered_input_file(file_obj, filters)
         return (file_obj, stat_value)
 
     def get_file_text(self, file_id, path=None, filtered=True):
@@ -533,7 +456,7 @@
         f = file(path, 'rb')
         if filtered and self.supports_content_filtering():
             filters = self._content_filter_stack(filename)
-            return filtered_input_file(f, filters)
+            return _mod_filters.filtered_input_file(f, filters)
         else:
             return f
 
@@ -545,61 +468,6 @@
         finally:
             file.close()
 
-    @needs_read_lock
-    def annotate_iter(self, file_id, default_revision=CURRENT_REVISION):
-        """See Tree.annotate_iter
-
-        This implementation will use the basis tree implementation if possible.
-        Lines not in the basis are attributed to CURRENT_REVISION
-
-        If there are pending merges, lines added by those merges will be
-        incorrectly attributed to CURRENT_REVISION (but after committing, the
-        attribution will be correct).
-        """
-        maybe_file_parent_keys = []
-        for parent_id in self.get_parent_ids():
-            try:
-                parent_tree = self.revision_tree(parent_id)
-            except errors.NoSuchRevisionInTree:
-                parent_tree = self.branch.repository.revision_tree(parent_id)
-            parent_tree.lock_read()
-            try:
-                if file_id not in parent_tree:
-                    continue
-                ie = parent_tree.inventory[file_id]
-                if ie.kind != 'file':
-                    # Note: this is slightly unnecessary, because symlinks and
-                    # directories have a "text" which is the empty text, and we
-                    # know that won't mess up annotations. But it seems cleaner
-                    continue
-                parent_text_key = (file_id, ie.revision)
-                if parent_text_key not in maybe_file_parent_keys:
-                    maybe_file_parent_keys.append(parent_text_key)
-            finally:
-                parent_tree.unlock()
-        graph = _mod_graph.Graph(self.branch.repository.texts)
-        heads = graph.heads(maybe_file_parent_keys)
-        file_parent_keys = []
-        for key in maybe_file_parent_keys:
-            if key in heads:
-                file_parent_keys.append(key)
-
-        # Now we have the parents of this content
-        annotator = self.branch.repository.texts.get_annotator()
-        text = self.get_file_text(file_id)
-        this_key =(file_id, default_revision)
-        annotator.add_special_text(this_key, file_parent_keys, text)
-        annotations = [(key[-1], line)
-                       for key, line in annotator.annotate_flat(this_key)]
-        return annotations
-
-    def _get_ancestors(self, default_revision):
-        ancestors = set([default_revision])
-        for parent_id in self.get_parent_ids():
-            ancestors.update(self.branch.repository.get_ancestry(
-                             parent_id, topo_sorted=False))
-        return ancestors
-
     def get_parent_ids(self):
         """See Tree.get_parent_ids.
 
@@ -621,23 +489,18 @@
                 parents.append(revision_id)
         return parents
 
-    @needs_read_lock
     def get_root_id(self):
         """Return the id of this trees root"""
-        return self._inventory.root.file_id
-
-    def _get_store_filename(self, file_id):
-        ## XXX: badly named; this is not in the store at all
-        return self.abspath(self.id2path(file_id))
+        raise NotImplementedError(self.get_root_id)
 
     @needs_read_lock
-    def clone(self, to_bzrdir, revision_id=None):
+    def clone(self, to_controldir, revision_id=None):
         """Duplicate this working tree into to_bzr, including all state.
 
         Specifically modified files are kept as modified, but
         ignored and unknown files are discarded.
 
-        If you want to make a new line of development, see bzrdir.sprout()
+        If you want to make a new line of development, see ControlDir.sprout()
 
         revision
             If not None, the cloned tree will have its last revision set to
@@ -645,7 +508,7 @@
             and this one merged in.
         """
         # assumes the target bzr dir format is compatible.
-        result = to_bzrdir.create_workingtree()
+        result = to_controldir.create_workingtree()
         self.copy_content_into(result, revision_id)
         return result
 
@@ -664,20 +527,33 @@
     def id2abspath(self, file_id):
         return self.abspath(self.id2path(file_id))
 
-    def has_id(self, file_id):
-        # files that have been deleted are excluded
-        inv = self.inventory
-        if not inv.has_id(file_id):
-            return False
-        path = inv.id2path(file_id)
-        return osutils.lexists(self.abspath(path))
-
-    def has_or_had_id(self, file_id):
-        if file_id == self.inventory.root.file_id:
-            return True
-        return self.inventory.has_id(file_id)
-
-    __contains__ = has_id
+    def _check_for_tree_references(self, iterator):
+        """See if directories have become tree-references."""
+        blocked_parent_ids = set()
+        for path, ie in iterator:
+            if ie.parent_id in blocked_parent_ids:
+                # This entry was pruned because one of its parents became a
+                # TreeReference. If this is a directory, mark it as blocked.
+                if ie.kind == 'directory':
+                    blocked_parent_ids.add(ie.file_id)
+                continue
+            if ie.kind == 'directory' and self._directory_is_tree_reference(path):
+                # This InventoryDirectory needs to be a TreeReference
+                ie = inventory.TreeReference(ie.file_id, ie.name, ie.parent_id)
+                blocked_parent_ids.add(ie.file_id)
+            yield path, ie
+
+    def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
+        """See Tree.iter_entries_by_dir()"""
+        # The only trick here is that if we supports_tree_reference then we
+        # need to detect if a directory becomes a tree-reference.
+        iterator = super(WorkingTree, self).iter_entries_by_dir(
+                specific_file_ids=specific_file_ids,
+                yield_parents=yield_parents)
+        if not self.supports_tree_reference():
+            return iterator
+        else:
+            return self._check_for_tree_references(iterator)
 
     def get_file_size(self, file_id):
         """See Tree.get_file_size"""
@@ -691,60 +567,6 @@
             else:
                 return None
 
-    @needs_read_lock
-    def get_file_sha1(self, file_id, path=None, stat_value=None):
-        if not path:
-            path = self._inventory.id2path(file_id)
-        return self._hashcache.get_sha1(path, stat_value)
-
-    def get_file_mtime(self, file_id, path=None):
-        if not path:
-            path = self.inventory.id2path(file_id)
-        return os.lstat(self.abspath(path)).st_mtime
-
-    def _is_executable_from_path_and_stat_from_basis(self, path, stat_result):
-        file_id = self.path2id(path)
-        if file_id is None:
-            # For unversioned files on win32, we just assume they are not
-            # executable
-            return False
-        return self._inventory[file_id].executable
-
-    def _is_executable_from_path_and_stat_from_stat(self, path, stat_result):
-        mode = stat_result.st_mode
-        return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
-
-    if not supports_executable():
-        def is_executable(self, file_id, path=None):
-            return self._inventory[file_id].executable
-
-        _is_executable_from_path_and_stat = \
-            _is_executable_from_path_and_stat_from_basis
-    else:
-        def is_executable(self, file_id, path=None):
-            if not path:
-                path = self.id2path(file_id)
-            mode = os.lstat(self.abspath(path)).st_mode
-            return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
-
-        _is_executable_from_path_and_stat = \
-            _is_executable_from_path_and_stat_from_stat
-
-    @needs_tree_write_lock
-    def _add(self, files, ids, kinds):
-        """See MutableTree._add."""
-        # TODO: Re-adding a file that is removed in the working copy
-        # should probably put it back with the previous ID.
-        # the read and write working inventory should not occur in this
-        # function - they should be part of lock_write and unlock.
-        inv = self.inventory
-        for f, file_id, kind in zip(files, ids, kinds):
-            if file_id is None:
-                inv.add_path(f, kind=kind)
-            else:
-                inv.add_path(f, kind=kind, file_id=file_id)
-            self._inventory_is_modified = True
-
     @needs_tree_write_lock
     def _gather_kinds(self, files, kinds):
         """See MutableTree._gather_kinds."""
@@ -765,8 +587,8 @@
         and setting the list to its value plus revision_id.
 
         :param revision_id: The revision id to add to the parent list. It may
-        be a ghost revision as long as its not the first parent to be added,
-        or the allow_leftmost_as_ghost parameter is set True.
+            be a ghost revision as long as its not the first parent to be
+            added, or the allow_leftmost_as_ghost parameter is set True.
         :param allow_leftmost_as_ghost: Allow the first parent to be a ghost.
         """
         parents = self.get_parent_ids() + [revision_id]
@@ -875,7 +697,7 @@
             if revision_id in heads and revision_id not in new_revision_ids:
                 new_revision_ids.append(revision_id)
         if new_revision_ids != revision_ids:
-            trace.mutter('requested to set revision_ids = %s,'
+            mutter('requested to set revision_ids = %s,'
                          ' but filtered to %s', revision_ids, new_revision_ids)
         return new_revision_ids
 
@@ -907,36 +729,6 @@
         self._set_merges_from_parent_ids(revision_ids)
 
     @needs_tree_write_lock
-    def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
-        """See MutableTree.set_parent_trees."""
-        parent_ids = [rev for (rev, tree) in parents_list]
-        for revision_id in parent_ids:
-            _mod_revision.check_not_reserved_id(revision_id)
-
-        self._check_parents_for_ghosts(parent_ids,
-            allow_leftmost_as_ghost=allow_leftmost_as_ghost)
-
-        parent_ids = self._filter_parent_ids_by_ancestry(parent_ids)
-
-        if len(parent_ids) == 0:
-            leftmost_parent_id = _mod_revision.NULL_REVISION
-            leftmost_parent_tree = None
-        else:
-            leftmost_parent_id, leftmost_parent_tree = parents_list[0]
-
-        if self._change_last_revision(leftmost_parent_id):
-            if leftmost_parent_tree is None:
-                # If we don't have a tree, fall back to reading the
-                # parent tree from the repository.
-                self._cache_basis_inventory(leftmost_parent_id)
-            else:
-                inv = leftmost_parent_tree.inventory
-                xml = self._create_basis_xml_from_inventory(
-                                        leftmost_parent_id, inv)
-                self._write_basis_inventory(xml)
-        self._set_merges_from_parent_ids(parent_ids)
-
-    @needs_tree_write_lock
     def set_pending_merges(self, rev_list):
         parents = self.get_parent_ids()
         leftmost = parents[:1]
@@ -947,7 +739,8 @@
     def set_merge_modified(self, modified_hashes):
         def iter_stanzas():
             for file_id, hash in modified_hashes.iteritems():
-                yield Stanza(file_id=file_id.decode('utf8'), hash=hash)
+                yield _mod_rio.Stanza(file_id=file_id.decode('utf8'),
+                    hash=hash)
         self._put_rio('merge-hashes', iter_stanzas(), MERGE_MODIFIED_HEADER_1)
 
     def _sha_from_stat(self, path, stat_result):
@@ -962,7 +755,7 @@
 
     def _put_rio(self, filename, stanzas, header):
         self._must_be_locked()
-        my_file = rio_file(stanzas, header)
+        my_file = _mod_rio.rio_file(stanzas, header)
         self._transport.put_file(filename, my_file,
             mode=self.bzrdir._get_file_mode())
 
@@ -1010,7 +803,6 @@
         merger.set_pending()
         return conflicts
 
-    @needs_read_lock
     def merge_modified(self):
         """Return a dictionary of files modified by a merge.
 
@@ -1021,28 +813,7 @@
         This returns a map of file_id->sha1, containing only files which are
         still in the working inventory and have that text hash.
         """
-        try:
-            hashfile = self._transport.get('merge-hashes')
-        except errors.NoSuchFile:
-            return {}
-        try:
-            merge_hashes = {}
-            try:
-                if hashfile.next() != MERGE_MODIFIED_HEADER_1 + '\n':
-                    raise errors.MergeModifiedFormatError()
-            except StopIteration:
-                raise errors.MergeModifiedFormatError()
-            for s in RioReader(hashfile):
-                # RioReader reads in Unicode, so convert file_ids back to utf8
-                file_id = osutils.safe_file_id(s.get("file_id"), warn=False)
-                if file_id not in self.inventory:
-                    continue
-                text_hash = s.get("hash")
-                if text_hash == self.get_file_sha1(file_id):
-                    merge_hashes[file_id] = text_hash
-            return merge_hashes
-        finally:
-            hashfile.close()
+        raise NotImplementedError(self.merge_modified)
 
     @needs_write_lock
     def mkdir(self, path, file_id=None):
@@ -1053,52 +824,16 @@
         self.add(path, file_id, 'directory')
         return file_id
 
-    def get_symlink_target(self, file_id):
-        abspath = self.id2abspath(file_id)
+    def get_symlink_target(self, file_id, path=None):
+        if path is not None:
+            abspath = self.abspath(path)
+        else:
+            abspath = self.id2abspath(file_id)
         target = osutils.readlink(abspath)
         return target
 
-    @needs_write_lock
     def subsume(self, other_tree):
-        def add_children(inventory, entry):
-            for child_entry in entry.children.values():
-                inventory._byid[child_entry.file_id] = child_entry
-                if child_entry.kind == 'directory':
-                    add_children(inventory, child_entry)
-        if other_tree.get_root_id() == self.get_root_id():
-            raise errors.BadSubsumeSource(self, other_tree,
-                                          'Trees have the same root')
-        try:
-            other_tree_path = self.relpath(other_tree.basedir)
-        except errors.PathNotChild:
-            raise errors.BadSubsumeSource(self, other_tree,
-                'Tree is not contained by the other')
-        new_root_parent = self.path2id(osutils.dirname(other_tree_path))
-        if new_root_parent is None:
-            raise errors.BadSubsumeSource(self, other_tree,
-                'Parent directory is not versioned.')
-        # We need to ensure that the result of a fetch will have a
-        # versionedfile for the other_tree root, and only fetching into
-        # RepositoryKnit2 guarantees that.
-        if not self.branch.repository.supports_rich_root():
-            raise errors.SubsumeTargetNeedsUpgrade(other_tree)
-        other_tree.lock_tree_write()
-        try:
-            new_parents = other_tree.get_parent_ids()
-            other_root = other_tree.inventory.root
-            other_root.parent_id = new_root_parent
-            other_root.name = osutils.basename(other_tree_path)
-            self.inventory.add(other_root)
-            add_children(self.inventory, other_root)
-            self._write_inventory(self.inventory)
-            # normally we don't want to fetch whole repositories, but i think
-            # here we really do want to consolidate the whole thing.
-            for parent_id in other_tree.get_parent_ids():
-                self.branch.fetch(other_tree.branch, parent_id)
-                self.add_parent_tree_id(parent_id)
-        finally:
-            other_tree.unlock()
-        other_tree.bzrdir.retire_bzrdir()
+        raise NotImplementedError(self.subsume)
 
     def _setup_directory_is_tree_reference(self):
         if self._branch.repository._format.supports_tree_reference:
@@ -1126,74 +861,16 @@
         # checkout in a subdirectory.  This can be avoided by not adding
         # it.  mbp 20070306
 
-    @needs_tree_write_lock
     def extract(self, file_id, format=None):
         """Extract a subtree from this tree.
 
         A new branch will be created, relative to the path for this tree.
         """
-        self.flush()
-        def mkdirs(path):
-            segments = osutils.splitpath(path)
-            transport = self.branch.bzrdir.root_transport
-            for name in segments:
-                transport = transport.clone(name)
-                transport.ensure_base()
-            return transport
-
-        sub_path = self.id2path(file_id)
-        branch_transport = mkdirs(sub_path)
-        if format is None:
-            format = self.bzrdir.cloning_metadir()
-        branch_transport.ensure_base()
-        branch_bzrdir = format.initialize_on_transport(branch_transport)
-        try:
-            repo = branch_bzrdir.find_repository()
-        except errors.NoRepositoryPresent:
-            repo = branch_bzrdir.create_repository()
-        if not repo.supports_rich_root():
-            raise errors.RootNotRich()
-        new_branch = branch_bzrdir.create_branch()
-        new_branch.pull(self.branch)
-        for parent_id in self.get_parent_ids():
-            new_branch.fetch(self.branch, parent_id)
-        tree_transport = self.bzrdir.root_transport.clone(sub_path)
-        if tree_transport.base != branch_transport.base:
-            tree_bzrdir = format.initialize_on_transport(tree_transport)
-            branch.BranchReferenceFormat().initialize(tree_bzrdir,
-                target_branch=new_branch)
-        else:
-            tree_bzrdir = branch_bzrdir
-        wt = tree_bzrdir.create_workingtree(_mod_revision.NULL_REVISION)
-        wt.set_parent_ids(self.get_parent_ids())
-        my_inv = self.inventory
-        child_inv = inventory.Inventory(root_id=None)
-        new_root = my_inv[file_id]
-        my_inv.remove_recursive_id(file_id)
-        new_root.parent_id = None
-        child_inv.add(new_root)
-        self._write_inventory(my_inv)
-        wt._write_inventory(child_inv)
-        return wt
-
-    def _serialize(self, inventory, out_file):
-        xml5.serializer_v5.write_inventory(self._inventory, out_file,
-            working=True)
-
-    def _deserialize(selt, in_file):
-        return xml5.serializer_v5.read_inventory(in_file)
+        raise NotImplementedError(self.extract)
 
     def flush(self):
-        """Write the in memory inventory to disk."""
-        # TODO: Maybe this should only write on dirty ?
-        if self._control_files._lock_mode != 'w':
-            raise errors.NotWriteLocked(self)
-        sio = StringIO()
-        self._serialize(self._inventory, sio)
-        sio.seek(0)
-        self._transport.put_file('inventory', sio,
-            mode=self.bzrdir._get_file_mode())
-        self._inventory_is_modified = False
+        """Write the in memory meta data to disk."""
+        raise NotImplementedError(self.flush)
 
     def _kind(self, relpath):
         return osutils.file_kind(self.abspath(relpath))
@@ -1209,122 +886,12 @@
         :param from_dir: start from this directory or None for the root
         :param recursive: whether to recurse into subdirectories or not
         """
-        # list_files is an iterator, so @needs_read_lock doesn't work properly
-        # with it. So callers should be careful to always read_lock the tree.
-        if not self.is_locked():
-            raise errors.ObjectNotLocked(self)
-
-        inv = self.inventory
-        if from_dir is None and include_root is True:
-            yield ('', 'V', 'directory', inv.root.file_id, inv.root)
-        # Convert these into local objects to save lookup times
-        pathjoin = osutils.pathjoin
-        file_kind = self._kind
-
-        # transport.base ends in a slash, we want the piece
-        # between the last two slashes
-        transport_base_dir = self.bzrdir.transport.base.rsplit('/', 2)[1]
-
-        fk_entries = {'directory':TreeDirectory, 'file':TreeFile, 'symlink':TreeLink}
-
-        # directory file_id, relative path, absolute path, reverse sorted children
-        if from_dir is not None:
-            from_dir_id = inv.path2id(from_dir)
-            if from_dir_id is None:
-                # Directory not versioned
-                return
-            from_dir_abspath = pathjoin(self.basedir, from_dir)
-        else:
-            from_dir_id = inv.root.file_id
-            from_dir_abspath = self.basedir
-        children = os.listdir(from_dir_abspath)
-        children.sort()
-        # jam 20060527 The kernel sized tree seems equivalent whether we
-        # use a deque and popleft to keep them sorted, or if we use a plain
-        # list and just reverse() them.
-        children = collections.deque(children)
-        stack = [(from_dir_id, u'', from_dir_abspath, children)]
-        while stack:
-            from_dir_id, from_dir_relpath, from_dir_abspath, children = stack[-1]
-
-            while children:
-                f = children.popleft()
-                ## TODO: If we find a subdirectory with its own .bzr
-                ## directory, then that is a separate tree and we
-                ## should exclude it.
-
-                # the bzrdir for this tree
-                if transport_base_dir == f:
-                    continue
-
-                # we know that from_dir_relpath and from_dir_abspath never end in a slash
-                # and 'f' doesn't begin with one, we can do a string op, rather
-                # than the checks of pathjoin(), all relative paths will have an extra slash
-                # at the beginning
-                fp = from_dir_relpath + '/' + f
-
-                # absolute path
-                fap = from_dir_abspath + '/' + f
-
-                f_ie = inv.get_child(from_dir_id, f)
-                if f_ie:
-                    c = 'V'
-                elif self.is_ignored(fp[1:]):
-                    c = 'I'
-                else:
-                    # we may not have found this file, because of a unicode issue
-                    f_norm, can_access = osutils.normalized_filename(f)
-                    if f == f_norm or not can_access:
-                        # No change, so treat this file normally
-                        c = '?'
-                    else:
-                        # this file can be accessed by a normalized path
-                        # check again if it is versioned
-                        # these lines are repeated here for performance
-                        f = f_norm
-                        fp = from_dir_relpath + '/' + f
-                        fap = from_dir_abspath + '/' + f
-                        f_ie = inv.get_child(from_dir_id, f)
-                        if f_ie:
-                            c = 'V'
-                        elif self.is_ignored(fp[1:]):
-                            c = 'I'
-                        else:
-                            c = '?'
-
-                fk = file_kind(fap)
-
-                # make a last minute entry
-                if f_ie:
-                    yield fp[1:], c, fk, f_ie.file_id, f_ie
-                else:
-                    try:
-                        yield fp[1:], c, fk, None, fk_entries[fk]()
-                    except KeyError:
-                        yield fp[1:], c, fk, None, TreeEntry()
-                    continue
-
-                if fk != 'directory':
-                    continue
-
-                # But do this child first if recursing down
-                if recursive:
-                    new_children = os.listdir(fap)
-                    new_children.sort()
-                    new_children = collections.deque(new_children)
-                    stack.append((f_ie.file_id, fp, fap, new_children))
-                    # Break out of inner loop,
-                    # so that we start outer loop with child
-                    break
-            else:
-                # if we finished all children, pop it off the stack
-                stack.pop()
-
-    @needs_tree_write_lock
+        raise NotImplementedError(self.list_files)
+
     def move(self, from_paths, to_dir=None, after=False):
         """Rename files.
 
-        to_dir must exist in the inventory.
+        to_dir must be known to the working tree.
 
         If to_dir exists and is a directory, the files are moved into
         it, keeping their old names.
@@ -1336,9 +903,8 @@
         independently.
 
         The first mode moves the file in the filesystem and updates the
-        inventory. The second mode only updates the inventory without
-        touching the file on the filesystem. This is the new mode introduced
-        in version 0.15.
+        working tree metadata. The second mode only updates the working tree
+        metadata without touching the file on the filesystem.
 
         move uses the second mode if 'after == True' and the target is not
         versioned but present in the working tree.
@@ -1356,168 +922,7 @@
         This returns a list of (from_path, to_path) pairs for each
         entry that is moved.
         """
-        rename_entries = []
-        rename_tuples = []
-
-        # check for deprecated use of signature
-        if to_dir is None:
-            raise TypeError('You must supply a target directory')
-        # check destination directory
-        if isinstance(from_paths, basestring):
-            raise ValueError()
-        inv = self.inventory
-        to_abs = self.abspath(to_dir)
-        if not isdir(to_abs):
-            raise errors.BzrMoveFailedError('',to_dir,
-                errors.NotADirectory(to_abs))
-        if not self.has_filename(to_dir):
-            raise errors.BzrMoveFailedError('',to_dir,
-                errors.NotInWorkingDirectory(to_dir))
-        to_dir_id = inv.path2id(to_dir)
-        if to_dir_id is None:
-            raise errors.BzrMoveFailedError('',to_dir,
-                errors.NotVersionedError(path=str(to_dir)))
-
-        to_dir_ie = inv[to_dir_id]
-        if to_dir_ie.kind != 'directory':
-            raise errors.BzrMoveFailedError('',to_dir,
-                errors.NotADirectory(to_abs))
-
-        # create rename entries and tuples
-        for from_rel in from_paths:
-            from_tail = splitpath(from_rel)[-1]
-            from_id = inv.path2id(from_rel)
-            if from_id is None:
-                raise errors.BzrMoveFailedError(from_rel,to_dir,
-                    errors.NotVersionedError(path=str(from_rel)))
-
-            from_entry = inv[from_id]
-            from_parent_id = from_entry.parent_id
-            to_rel = pathjoin(to_dir, from_tail)
-            rename_entry = WorkingTree._RenameEntry(from_rel=from_rel,
-                                         from_id=from_id,
-                                         from_tail=from_tail,
-                                         from_parent_id=from_parent_id,
-                                         to_rel=to_rel, to_tail=from_tail,
-                                         to_parent_id=to_dir_id)
-            rename_entries.append(rename_entry)
-            rename_tuples.append((from_rel, to_rel))
-
-        # determine which move mode to use. checks also for movability
-        rename_entries = self._determine_mv_mode(rename_entries, after)
-
-        original_modified = self._inventory_is_modified
-        try:
-            if len(from_paths):
-                self._inventory_is_modified = True
-            self._move(rename_entries)
-        except:
-            # restore the inventory on error
-            self._inventory_is_modified = original_modified
-            raise
-        self._write_inventory(inv)
-        return rename_tuples
-
-    def _determine_mv_mode(self, rename_entries, after=False):
-        """Determines for each from-to pair if both inventory and working tree
-        or only the inventory has to be changed.
-
-        Also does basic plausability tests.
-        """
-        inv = self.inventory
-
-        for rename_entry in rename_entries:
-            # store to local variables for easier reference
-            from_rel = rename_entry.from_rel
-            from_id = rename_entry.from_id
-            to_rel = rename_entry.to_rel
-            to_id = inv.path2id(to_rel)
-            only_change_inv = False
-
-            # check the inventory for source and destination
-            if from_id is None:
-                raise errors.BzrMoveFailedError(from_rel,to_rel,
-                    errors.NotVersionedError(path=str(from_rel)))
-            if to_id is not None:
-                raise errors.BzrMoveFailedError(from_rel,to_rel,
-                    errors.AlreadyVersionedError(path=str(to_rel)))
-
-            # try to determine the mode for rename (only change inv or change
-            # inv and file system)
-            if after:
-                if not self.has_filename(to_rel):
-                    raise errors.BzrMoveFailedError(from_id,to_rel,
-                        errors.NoSuchFile(path=str(to_rel),
-                        extra="New file has not been created yet"))
-                only_change_inv = True
-            elif not self.has_filename(from_rel) and self.has_filename(to_rel):
-                only_change_inv = True
-            elif self.has_filename(from_rel) and not self.has_filename(to_rel):
-                only_change_inv = False
-            elif (not self.case_sensitive
-                  and from_rel.lower() == to_rel.lower()
-                  and self.has_filename(from_rel)):
-                only_change_inv = False
-            else:
-                # something is wrong, so lets determine what exactly
-                if not self.has_filename(from_rel) and \
-                   not self.has_filename(to_rel):
-                    raise errors.BzrRenameFailedError(from_rel,to_rel,
-                        errors.PathsDoNotExist(paths=(str(from_rel),
-                        str(to_rel))))
-                else:
-                    raise errors.RenameFailedFilesExist(from_rel, to_rel)
-            rename_entry.only_change_inv = only_change_inv
-        return rename_entries
-
-    def _move(self, rename_entries):
-        """Moves a list of files.
-
-        Depending on the value of the flag 'only_change_inv', the
-        file will be moved on the file system or not.
-        """
-        inv = self.inventory
-        moved = []
-
-        for entry in rename_entries:
-            try:
-                self._move_entry(entry)
-            except:
-                self._rollback_move(moved)
-                raise
-            moved.append(entry)
-
-    def _rollback_move(self, moved):
-        """Try to rollback a previous move in case of an filesystem error."""
-        inv = self.inventory
-        for entry in moved:
-            try:
-                self._move_entry(WorkingTree._RenameEntry(
-                    entry.to_rel, entry.from_id,
-                    entry.to_tail, entry.to_parent_id, entry.from_rel,
-                    entry.from_tail, entry.from_parent_id,
-                    entry.only_change_inv))
-            except errors.BzrMoveFailedError, e:
-                raise errors.BzrMoveFailedError( '', '', "Rollback failed."
-                        " The working tree is in an inconsistent state."
-                        " Please consider doing a 'bzr revert'."
-                        " Error message is: %s" % e)
-
-    def _move_entry(self, entry):
-        inv = self.inventory
-        from_rel_abs = self.abspath(entry.from_rel)
-        to_rel_abs = self.abspath(entry.to_rel)
-        if from_rel_abs == to_rel_abs:
-            raise errors.BzrMoveFailedError(entry.from_rel, entry.to_rel,
-                "Source and target are identical.")
-
-        if not entry.only_change_inv:
-            try:
-                osutils.rename(from_rel_abs, to_rel_abs)
-            except OSError, e:
-                raise errors.BzrMoveFailedError(entry.from_rel,
-                    entry.to_rel, e[1])
-        inv.rename(entry.from_id, entry.to_parent_id, entry.to_tail)
+        raise NotImplementedError(self.move)
 
     @needs_tree_write_lock
     def rename_one(self, from_rel, to_rel, after=False):
@@ -1527,11 +932,10 @@
 
         rename_one has several 'modes' to work. First, it can rename a physical
         file and change the file_id. That is the normal mode. Second, it can
-        only change the file_id without touching any physical file. This is
-        the new mode introduced in version 0.15.
+        only change the file_id without touching any physical file.
 
-        rename_one uses the second mode if 'after == True' and 'to_rel' is not
-        versioned but present in the working tree.
+        rename_one uses the second mode if 'after == True' and 'to_rel' is
+        either not versioned or newly added, and present in the working tree.
 
         rename_one uses the second mode if 'after == False' and 'from_rel' is
         versioned but no longer in the working tree, and 'to_rel' is not
@@ -1543,67 +947,7 @@
 
         Everything else results in an error.
         """
-        inv = self.inventory
-        rename_entries = []
-
-        # create rename entries and tuples
-        from_tail = splitpath(from_rel)[-1]
-        from_id = inv.path2id(from_rel)
-        if from_id is None:
-            # if file is missing in the inventory maybe it's in the basis_tree
-            basis_tree = self.branch.basis_tree()
-            from_id = basis_tree.path2id(from_rel)
-            if from_id is None:
-                raise errors.BzrRenameFailedError(from_rel,to_rel,
-                    errors.NotVersionedError(path=str(from_rel)))
-            # put entry back in the inventory so we can rename it
-            from_entry = basis_tree.inventory[from_id].copy()
-            inv.add(from_entry)
-        else:
-            from_entry = inv[from_id]
-        from_parent_id = from_entry.parent_id
-        to_dir, to_tail = os.path.split(to_rel)
-        to_dir_id = inv.path2id(to_dir)
-        rename_entry = WorkingTree._RenameEntry(from_rel=from_rel,
-                                     from_id=from_id,
-                                     from_tail=from_tail,
-                                     from_parent_id=from_parent_id,
-                                     to_rel=to_rel, to_tail=to_tail,
-                                     to_parent_id=to_dir_id)
-        rename_entries.append(rename_entry)
-
-        # determine which move mode to use. checks also for movability
-        rename_entries = self._determine_mv_mode(rename_entries, after)
-
-        # check if the target changed directory and if the target directory is
-        # versioned
-        if to_dir_id is None:
-            raise errors.BzrMoveFailedError(from_rel,to_rel,
-                errors.NotVersionedError(path=str(to_dir)))
-
-        # all checks done. now we can continue with our actual work
-        mutter('rename_one:\n'
-               '  from_id   {%s}\n'
-               '  from_rel: %r\n'
-               '  to_rel:   %r\n'
-               '  to_dir    %r\n'
-               '  to_dir_id {%s}\n',
-               from_id, from_rel, to_rel, to_dir, to_dir_id)
-
-        self._move(rename_entries)
-        self._write_inventory(inv)
-
-    class _RenameEntry(object):
-        def __init__(self, from_rel, from_id, from_tail, from_parent_id,
-                     to_rel, to_tail, to_parent_id, only_change_inv=False):
-            self.from_rel = from_rel
-            self.from_id = from_id
-            self.from_tail = from_tail
-            self.from_parent_id = from_parent_id
-            self.to_rel = to_rel
-            self.to_tail = to_tail
-            self.to_parent_id = to_parent_id
-            self.only_change_inv = only_change_inv
+        raise NotImplementedError(self.rename_one)
 
     @needs_read_lock
     def unknowns(self):
@@ -1617,7 +961,6 @@
         return iter(
             [subp for subp in self.extras() if not self.is_ignored(subp)])
 
-    @needs_tree_write_lock
     def unversion(self, file_ids):
         """Remove the file ids in file_ids from the current versioned set.
 
@@ -1627,37 +970,12 @@
         :param file_ids: The file ids to stop versioning.
         :raises: NoSuchId if any fileid is not currently versioned.
         """
-        for file_id in file_ids:
-            if file_id not in self._inventory:
-                raise errors.NoSuchId(self, file_id)
-        for file_id in file_ids:
-            if self._inventory.has_id(file_id):
-                self._inventory.remove_recursive_id(file_id)
-        if len(file_ids):
-            # in the future this should just set a dirty bit to wait for the
-            # final unlock. However, until all methods of workingtree start
-            # with the current in -memory inventory rather than triggering
-            # a read, it is more complex - we need to teach read_inventory
-            # to know when to read, and when to not read first... and possibly
-            # to save first when the in memory one may be corrupted.
-            # so for now, we just only write it if it is indeed dirty.
-            # - RBC 20060907
-            self._write_inventory(self._inventory)
-
-    def _iter_conflicts(self):
-        conflicted = set()
-        for info in self.list_files():
-            path = info[0]
-            stem = get_conflicted_stem(path)
-            if stem is None:
-                continue
-            if stem not in conflicted:
-                conflicted.add(stem)
-                yield stem
+        raise NotImplementedError(self.unversion)
 
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-             change_reporter=None, possible_transports=None, local=False):
+             change_reporter=None, possible_transports=None, local=False,
+             show_base=False):
         source.lock_read()
         try:
             old_revision_info = self.branch.last_revision_info()
@@ -1668,6 +986,11 @@
             new_revision_info = self.branch.last_revision_info()
             if new_revision_info != old_revision_info:
                 repository = self.branch.repository
+                if repository._format.fast_deltas:
+                    parent_ids = self.get_parent_ids()
+                    if parent_ids:
+                        basis_id = parent_ids[0]
+                        basis_tree = repository.revision_tree(basis_id)
                 basis_tree.lock_read()
                 try:
                     new_basis_tree = self.branch.basis_tree()
@@ -1677,7 +1000,8 @@
                                 basis_tree,
                                 this_tree=self,
                                 pb=None,
-                                change_reporter=change_reporter)
+                                change_reporter=change_reporter,
+                                show_base=show_base)
                     basis_root_id = basis_tree.get_root_id()
                     new_root_id = new_basis_tree.get_root_id()
                     if basis_root_id != new_root_id:
@@ -1710,7 +1034,6 @@
             stream.write(bytes)
         finally:
             stream.close()
-        # TODO: update the hashcache here ?
 
     def extras(self):
         """Yield all unversioned files in this WorkingTree.
@@ -1722,37 +1045,7 @@
         Currently returned depth-first, sorted by name within directories.
         This is the same order used by 'osutils.walkdirs'.
         """
-        ## TODO: Work from given directory downwards
-        for path, dir_entry in self.inventory.directories():
-            # mutter("search for unknowns in %r", path)
-            dirabs = self.abspath(path)
-            if not isdir(dirabs):
-                # e.g. directory deleted
-                continue
-
-            fl = []
-            for subf in os.listdir(dirabs):
-                if self.bzrdir.is_control_filename(subf):
-                    continue
-                if subf not in dir_entry.children:
-                    try:
-                        (subf_norm,
-                         can_access) = osutils.normalized_filename(subf)
-                    except UnicodeDecodeError:
-                        path_os_enc = path.encode(osutils._fs_enc)
-                        relpath = path_os_enc + '/' + subf
-                        raise errors.BadFilenameEncoding(relpath,
-                                                         osutils._fs_enc)
-                    if subf_norm != subf and can_access:
-                        if subf_norm not in dir_entry.children:
-                            fl.append(subf_norm)
-                    else:
-                        fl.append(subf)
-
-            fl.sort()
-            for subf in fl:
-                subp = pathjoin(path, subf)
-                yield subp
+        raise NotImplementedError(self.extras)
 
     def ignored_files(self):
         """Yield list of PATH, IGNORE_PATTERN"""
@@ -1807,7 +1100,7 @@
 
     def stored_kind(self, file_id):
         """See Tree.stored_kind"""
-        return self.inventory[file_id].kind
+        raise NotImplementedError(self.stored_kind)
 
     def _comparison_data(self, entry, path):
         abspath = self.abspath(path)
@@ -1904,24 +1197,13 @@
     def get_physical_lock_status(self):
         return self._control_files.get_physical_lock_status()
 
-    def _basis_inventory_name(self):
-        return 'basis-inventory-cache'
-
     def _reset_data(self):
         """Reset transient data that cannot be revalidated."""
-        self._inventory_is_modified = False
-        f = self._transport.get('inventory')
-        try:
-            result = self._deserialize(f)
-        finally:
-            f.close()
-        self._set_inventory(result, dirty=False)
+        raise NotImplementedError(self._reset_data)
 
-    @needs_tree_write_lock
     def set_last_revision(self, new_revision):
         """Change the last revision in the working tree."""
-        if self._change_last_revision(new_revision):
-            self._cache_basis_inventory(new_revision)
+        raise NotImplementedError(self.set_last_revision)
 
     def _change_last_revision(self, new_revision):
         """Template method part of set_last_revision to perform the change.
@@ -1930,84 +1212,20 @@
         when their last revision is set.
         """
         if _mod_revision.is_null(new_revision):
-            self.branch.set_revision_history([])
+            self.branch.set_last_revision_info(0, new_revision)
             return False
+        _mod_revision.check_not_reserved_id(new_revision)
         try:
             self.branch.generate_revision_history(new_revision)
         except errors.NoSuchRevision:
             # not present in the repo - dont try to set it deeper than the tip
-            self.branch.set_revision_history([new_revision])
+            self.branch._set_revision_history([new_revision])
         return True
 
-    def _write_basis_inventory(self, xml):
-        """Write the basis inventory XML to the basis-inventory file"""
-        path = self._basis_inventory_name()
-        sio = StringIO(xml)
-        self._transport.put_file(path, sio,
-            mode=self.bzrdir._get_file_mode())
-
-    def _create_basis_xml_from_inventory(self, revision_id, inventory):
-        """Create the text that will be saved in basis-inventory"""
-        inventory.revision_id = revision_id
-        return xml7.serializer_v7.write_inventory_to_string(inventory)
-
-    def _cache_basis_inventory(self, new_revision):
-        """Cache new_revision as the basis inventory."""
-        # TODO: this should allow the ready-to-use inventory to be passed in,
-        # as commit already has that ready-to-use [while the format is the
-        # same, that is].
-        try:
-            # this double handles the inventory - unpack and repack -
-            # but is easier to understand. We can/should put a conditional
-            # in here based on whether the inventory is in the latest format
-            # - perhaps we should repack all inventories on a repository
-            # upgrade ?
-            # the fast path is to copy the raw xml from the repository. If the
-            # xml contains 'revision_id="', then we assume the right
-            # revision_id is set. We must check for this full string, because a
-            # root node id can legitimately look like 'revision_id' but cannot
-            # contain a '"'.
-            xml = self.branch.repository._get_inventory_xml(new_revision)
-            firstline = xml.split('\n', 1)[0]
-            if (not 'revision_id="' in firstline or
-                'format="7"' not in firstline):
-                inv = self.branch.repository._serializer.read_inventory_from_string(
-                    xml, new_revision)
-                xml = self._create_basis_xml_from_inventory(new_revision, inv)
-            self._write_basis_inventory(xml)
-        except (errors.NoSuchRevision, errors.RevisionNotPresent):
-            pass
-
-    def read_basis_inventory(self):
-        """Read the cached basis inventory."""
-        path = self._basis_inventory_name()
-        return self._transport.get_bytes(path)
-
-    @needs_read_lock
-    def read_working_inventory(self):
-        """Read the working inventory.
-
-        :raises errors.InventoryModified: read_working_inventory will fail
-            when the current in memory inventory has been modified.
-        """
-        # conceptually this should be an implementation detail of the tree.
-        # XXX: Deprecate this.
-        # ElementTree does its own conversion from UTF-8, so open in
-        # binary.
-        if self._inventory_is_modified:
-            raise errors.InventoryModified(self)
-        f = self._transport.get('inventory')
-        try:
-            result = self._deserialize(f)
-        finally:
-            f.close()
-        self._set_inventory(result, dirty=False)
-        return result
-
     @needs_tree_write_lock
     def remove(self, files, verbose=False, to_file=None, keep_files=True,
         force=False):
-        """Remove nominated files from the working inventory.
+        """Remove nominated files from the working tree metadata.
 
         :files: File paths relative to the basedir.
         :keep_files: If true, the files will also be kept.
@@ -2019,34 +1237,36 @@
 
         inv_delta = []
 
-        new_files=set()
+        all_files = set() # specified and nested files 
         unknown_nested_files=set()
         if to_file is None:
             to_file = sys.stdout
 
+        files_to_backup = []
+
         def recurse_directory_to_add_files(directory):
             # Recurse directory and add all files
             # so we can check if they have changed.
             for parent_info, file_infos in self.walkdirs(directory):
                 for relpath, basename, kind, lstat, fileid, kind in file_infos:
                     # Is it versioned or ignored?
-                    if self.path2id(relpath) or self.is_ignored(relpath):
+                    if self.path2id(relpath):
                         # Add nested content for deletion.
-                        new_files.add(relpath)
+                        all_files.add(relpath)
                     else:
-                        # Files which are not versioned and not ignored
+                        # Files which are not versioned
                         # should be treated as unknown.
-                        unknown_nested_files.add((relpath, None, kind))
+                        files_to_backup.append(relpath)
 
         for filename in files:
             # Get file name into canonical form.
             abspath = self.abspath(filename)
             filename = self.relpath(abspath)
             if len(filename) > 0:
-                new_files.add(filename)
+                all_files.add(filename)
                 recurse_directory_to_add_files(filename)
 
-        files = list(new_files)
+        files = list(all_files)
 
         if len(files) == 0:
             return # nothing to do
@@ -2056,37 +1276,27 @@
 
         # Bail out if we are going to delete files we shouldn't
         if not keep_files and not force:
-            has_changed_files = len(unknown_nested_files) > 0
-            if not has_changed_files:
-                for (file_id, path, content_change, versioned, parent_id, name,
-                     kind, executable) in self.iter_changes(self.basis_tree(),
-                         include_unchanged=True, require_versioned=False,
-                         want_unversioned=True, specific_files=files):
-                    if versioned == (False, False):
-                        # The record is unknown ...
-                        if not self.is_ignored(path[1]):
-                            # ... but not ignored
-                            has_changed_files = True
-                            break
-                    elif (content_change and (kind[1] is not None) and
-                            osutils.is_inside_any(files, path[1])):
-                        # Versioned and changed, but not deleted, and still
-                        # in one of the dirs to be deleted.
-                        has_changed_files = True
-                        break
+            for (file_id, path, content_change, versioned, parent_id, name,
+                 kind, executable) in self.iter_changes(self.basis_tree(),
+                     include_unchanged=True, require_versioned=False,
+                     want_unversioned=True, specific_files=files):
+                if versioned[0] == False:
+                    # The record is unknown or newly added
+                    files_to_backup.append(path[1])
+                elif (content_change and (kind[1] is not None) and
+                        osutils.is_inside_any(files, path[1])):
+                    # Versioned and changed, but not deleted, and still
+                    # in one of the dirs to be deleted.
+                    files_to_backup.append(path[1])
 
-            if has_changed_files:
-                # Make delta show ALL applicable changes in error message.
-                tree_delta = self.changes_from(self.basis_tree(),
-                    require_versioned=False, want_unversioned=True,
-                    specific_files=files)
-                for unknown_file in unknown_nested_files:
-                    if unknown_file not in tree_delta.unversioned:
-                        tree_delta.unversioned.extend((unknown_file,))
-                raise errors.BzrRemoveChangedFilesError(tree_delta)
+        def backup(file_to_backup):
+            backup_name = self.bzrdir._available_backup_name(file_to_backup)
+            osutils.rename(abs_path, self.abspath(backup_name))
+            return "removed %s (but kept a copy: %s)" % (file_to_backup,
+                                                         backup_name)
 
         # Build inv_delta and delete files where applicable,
-        # do this before any modifications to inventory.
+        # do this before any modifications to meta data.
         for f in files:
             fid = self.path2id(f)
             message = None
@@ -2113,12 +1323,15 @@
                         len(os.listdir(abs_path)) > 0):
                         if force:
                             osutils.rmtree(abs_path)
+                            message = "deleted %s" % (f,)
                         else:
-                            message = "%s is not an empty directory "\
-                                "and won't be deleted." % (f,)
+                            message = backup(f)
                     else:
-                        osutils.delete_any(abs_path)
-                        message = "deleted %s" % (f,)
+                        if f in files_to_backup:
+                            message = backup(f)
+                        else:
+                            osutils.delete_any(abs_path)
+                            message = "deleted %s" % (f,)
                 elif message is not None:
                     # Only care if we haven't done anything yet.
                     message = "%s does not exist." % (f,)
@@ -2132,11 +1345,6 @@
     def revert(self, filenames=None, old_tree=None, backups=True,
                pb=None, report_changes=False):
         from bzrlib.conflicts import resolve
-        if filenames == []:
-            filenames = None
-            symbol_versioning.warn('Using [] to revert all files is deprecated'
-                ' as of bzr 0.91.  Please use None (the default) instead.',
-                DeprecationWarning, stacklevel=2)
         if old_tree is None:
             basis_tree = self.basis_tree()
             basis_tree.lock_read()
@@ -2169,47 +1377,7 @@
         WorkingTree can supply revision_trees for the basis revision only
         because there is only one cached inventory in the bzr directory.
         """
-        if revision_id == self.last_revision():
-            try:
-                xml = self.read_basis_inventory()
-            except errors.NoSuchFile:
-                pass
-            else:
-                try:
-                    inv = xml7.serializer_v7.read_inventory_from_string(xml)
-                    # dont use the repository revision_tree api because we want
-                    # to supply the inventory.
-                    if inv.revision_id == revision_id:
-                        return revisiontree.RevisionTree(self.branch.repository,
-                            inv, revision_id)
-                except errors.BadInventoryFormat:
-                    pass
-        # raise if there was no inventory, or if we read the wrong inventory.
-        raise errors.NoSuchRevisionInTree(self, revision_id)
-
-    # XXX: This method should be deprecated in favour of taking in a proper
-    # new Inventory object.
-    @needs_tree_write_lock
-    def set_inventory(self, new_inventory_list):
-        from bzrlib.inventory import (Inventory,
-                                      InventoryDirectory,
-                                      InventoryFile,
-                                      InventoryLink)
-        inv = Inventory(self.get_root_id())
-        for path, file_id, parent, kind in new_inventory_list:
-            name = os.path.basename(path)
-            if name == "":
-                continue
-            # fixme, there should be a factory function inv,add_??
-            if kind == 'directory':
-                inv.add(InventoryDirectory(file_id, name, parent))
-            elif kind == 'file':
-                inv.add(InventoryFile(file_id, name, parent))
-            elif kind == 'symlink':
-                inv.add(InventoryLink(file_id, name, parent))
-            else:
-                raise errors.BzrError("unknown kind %r" % kind)
-        self._write_inventory(inv)
+        raise NotImplementedError(self.revision_tree)
 
     @needs_tree_write_lock
     def set_root_id(self, file_id):
@@ -2228,24 +1396,7 @@
             present in the current inventory or an error will occur. It must
             not be None, but rather a valid file id.
         """
-        inv = self._inventory
-        orig_root_id = inv.root.file_id
-        # TODO: it might be nice to exit early if there was nothing
-        # to do, saving us from trigger a sync on unlock.
-        self._inventory_is_modified = True
-        # we preserve the root inventory entry object, but
-        # unlinkit from the byid index
-        del inv._byid[inv.root.file_id]
-        inv.root.file_id = file_id
-        # and link it into the index with the new changed id.
-        inv._byid[inv.root.file_id] = inv.root
-        # and finally update all children to reference the new id.
-        # XXX: this should be safe to just look at the root.children
-        # list, not the WHOLE INVENTORY.
-        for fid in inv:
-            entry = inv[fid]
-            if entry.parent_id == orig_root_id:
-                entry.parent_id = inv.root.file_id
+        raise NotImplementedError(self._set_root_id)
 
     def unlock(self):
         """See Branch.unlock.
@@ -2261,7 +1412,7 @@
     _marker = object()
 
     def update(self, change_reporter=None, possible_transports=None,
-               revision=None, old_tip=_marker):
+               revision=None, old_tip=_marker, show_base=False):
         """Update a working tree along its branch.
 
         This will update the branch if its bound too, which means we have
@@ -2282,6 +1433,7 @@
         - Restore the wt.basis->wt.state changes.
 
         There isn't a single operation at the moment to do that, so we:
+
         - Merge current state -> basis tree of the master w.r.t. the old tree
           basis.
         - Do a 'normal' merge of the old branch basis if it is relevant.
@@ -2304,12 +1456,13 @@
             else:
                 if old_tip is self._marker:
                     old_tip = None
-            return self._update_tree(old_tip, change_reporter, revision)
+            return self._update_tree(old_tip, change_reporter, revision, show_base)
         finally:
             self.unlock()
 
     @needs_tree_write_lock
-    def _update_tree(self, old_tip=None, change_reporter=None, revision=None):
+    def _update_tree(self, old_tip=None, change_reporter=None, revision=None,
+                     show_base=False):
         """Update a tree to the master branch.
 
         :param old_tip: if supplied, the previous tip revision the branch,
@@ -2342,10 +1495,11 @@
             other_tree = self.branch.repository.revision_tree(old_tip)
             nb_conflicts = merge.merge_inner(self.branch, other_tree,
                                              base_tree, this_tree=self,
-                                             change_reporter=change_reporter)
+                                             change_reporter=change_reporter,
+                                             show_base=show_base)
             if nb_conflicts:
                 self.add_parent_tree((old_tip, other_tree))
-                trace.note('Rerun update after fixing the conflicts.')
+                note(gettext('Rerun update after fixing the conflicts.'))
                 return nb_conflicts
 
         if last_rev != _mod_revision.ensure_null(revision):
@@ -2357,8 +1511,7 @@
             basis = self.basis_tree()
             basis.lock_read()
             try:
-                if (basis.inventory.root is None
-                    or basis.inventory.root.file_id != to_root_id):
+                if (basis.get_root_id() is None or basis.get_root_id() != to_root_id):
                     self.set_root_id(to_root_id)
                     self.flush()
             finally:
@@ -2372,7 +1525,8 @@
 
             nb_conflicts = merge.merge_inner(self.branch, to_tree, base_tree,
                                              this_tree=self,
-                                             change_reporter=change_reporter)
+                                             change_reporter=change_reporter,
+                                             show_base=show_base)
             self.set_last_revision(revision)
             # TODO - dedup parents list with things merged by pull ?
             # reuse the tree we've updated to to set the basis:
@@ -2393,57 +1547,14 @@
             last_rev = parent_trees[0][0]
         return nb_conflicts
 
-    def _write_hashcache_if_dirty(self):
-        """Write out the hashcache if it is dirty."""
-        if self._hashcache.needs_write:
-            try:
-                self._hashcache.write()
-            except OSError, e:
-                if e.errno not in (errno.EPERM, errno.EACCES):
-                    raise
-                # TODO: jam 20061219 Should this be a warning? A single line
-                #       warning might be sufficient to let the user know what
-                #       is going on.
-                mutter('Could not write hashcache for %s\nError: %s',
-                       self._hashcache.cache_file_name(), e)
-
-    @needs_tree_write_lock
-    def _write_inventory(self, inv):
-        """Write inventory as the current inventory."""
-        self._set_inventory(inv, dirty=True)
-        self.flush()
-
     def set_conflicts(self, arg):
         raise errors.UnsupportedOperation(self.set_conflicts, self)
 
     def add_conflicts(self, arg):
         raise errors.UnsupportedOperation(self.add_conflicts, self)
 
-    @needs_read_lock
     def conflicts(self):
-        conflicts = _mod_conflicts.ConflictList()
-        for conflicted in self._iter_conflicts():
-            text = True
-            try:
-                if file_kind(self.abspath(conflicted)) != "file":
-                    text = False
-            except errors.NoSuchFile:
-                text = False
-            if text is True:
-                for suffix in ('.THIS', '.OTHER'):
-                    try:
-                        kind = file_kind(self.abspath(conflicted+suffix))
-                        if kind != "file":
-                            text = False
-                    except errors.NoSuchFile:
-                        text = False
-                    if text == False:
-                        break
-            ctype = {True: 'text conflict', False: 'contents conflict'}[text]
-            conflicts.append(_mod_conflicts.Conflict.factory(ctype,
-                             path=conflicted,
-                             file_id=self.path2id(conflicted)))
-        return conflicts
+        raise NotImplementedError(self.conflicts)
 
     def walkdirs(self, prefix=""):
         """Walk the directories of this tree.
@@ -2569,41 +1680,14 @@
     def _walkdirs(self, prefix=""):
         """Walk the directories of this tree.
 
-           :prefix: is used as the directrory to start with.
-           returns a generator which yields items in the form:
-                ((curren_directory_path, fileid),
-                 [(file1_path, file1_name, file1_kind, None, file1_id,
-                   file1_kind), ... ])
+        :param prefix: is used as the directrory to start with.
+        :returns: a generator which yields items in the form::
+
+            ((curren_directory_path, fileid),
+             [(file1_path, file1_name, file1_kind, None, file1_id,
+               file1_kind), ... ])
         """
-        _directory = 'directory'
-        # get the root in the inventory
-        inv = self.inventory
-        top_id = inv.path2id(prefix)
-        if top_id is None:
-            pending = []
-        else:
-            pending = [(prefix, '', _directory, None, top_id, None)]
-        while pending:
-            dirblock = []
-            currentdir = pending.pop()
-            # 0 - relpath, 1- basename, 2- kind, 3- stat, 4-id, 5-kind
-            top_id = currentdir[4]
-            if currentdir[0]:
-                relroot = currentdir[0] + '/'
-            else:
-                relroot = ""
-            # FIXME: stash the node in pending
-            entry = inv[top_id]
-            if entry.kind == 'directory':
-                for name, child in entry.sorted_children():
-                    dirblock.append((relroot + name, name, child.kind, None,
-                        child.file_id, child.kind
-                        ))
-            yield (currentdir[0], entry.file_id), dirblock
-            # push the user specified dirs from dirblock
-            for dir in reversed(dirblock):
-                if dir[2] == _directory:
-                    pending.append(dir)
+        raise NotImplementedError(self._walkdirs)
 
     @needs_tree_write_lock
     def auto_resolve(self):
@@ -2613,6 +1697,7 @@
         are considered 'resolved', because bzr always puts conflict markers
         into files that have text conflicts.  The corresponding .THIS .BASE and
         .OTHER files are deleted, as per 'resolve'.
+
         :return: a tuple of ConflictLists: (un_resolved, resolved).
         """
         un_resolved = _mod_conflicts.ConflictList()
@@ -2637,25 +1722,6 @@
         self.set_conflicts(un_resolved)
         return un_resolved, resolved
 
-    @needs_read_lock
-    def _check(self, references):
-        """Check the tree for consistency.
-
-        :param references: A dict with keys matching the items returned by
-            self._get_check_refs(), and values from looking those keys up in
-            the repository.
-        """
-        tree_basis = self.basis_tree()
-        tree_basis.lock_read()
-        try:
-            repo_basis = references[('trees', self.last_revision())]
-            if len(list(repo_basis.iter_changes(tree_basis))) > 0:
-                raise errors.BzrCheckError(
-                    "Mismatched basis inventory content.")
-            self._validate()
-        finally:
-            tree_basis.unlock()
-
     def _validate(self):
         """Validate internal structures.
 
@@ -2667,6 +1733,18 @@
         """
         return
 
+    def check_state(self):
+        """Check that the working state is/isn't valid."""
+        raise NotImplementedError(self.check_state)
+
+    def reset_state(self, revision_ids=None):
+        """Reset the state of the working tree.
+
+        This does a hard-reset to a last-known-good state. This is a way to
+        fix if something got corrupted (like the .bzr/checkout/dirstate file)
+        """
+        raise NotImplementedError(self.reset_state)
+
     def _get_rules_searcher(self, default_searcher):
         """See Tree._get_rules_searcher."""
         if self._rules_searcher is None:
@@ -2680,98 +1758,217 @@
         return ShelfManager(self, self._transport)
 
 
-class WorkingTree2(WorkingTree):
-    """This is the Format 2 working tree.
-
-    This was the first weave based working tree.
-     - uses os locks for locking.
-     - uses the branch last-revision.
+class InventoryWorkingTree(WorkingTree,
+        bzrlib.mutabletree.MutableInventoryTree):
+    """Base class for working trees that are inventory-oriented.
+
+    The inventory is held in the `Branch` working-inventory, and the
+    files are in a directory on disk.
+
+    It is possible for a `WorkingTree` to have a filename which is
+    not listed in the Inventory and vice versa.
     """
 
-    def __init__(self, *args, **kwargs):
-        super(WorkingTree2, self).__init__(*args, **kwargs)
-        # WorkingTree2 has more of a constraint that self._inventory must
-        # exist. Because this is an older format, we don't mind the overhead
-        # caused by the extra computation here.
-
-        # Newer WorkingTree's should only have self._inventory set when they
-        # have a read lock.
-        if self._inventory is None:
-            self.read_working_inventory()
-
-    def _get_check_refs(self):
-        """Return the references needed to perform a check of this tree."""
-        return [('trees', self.last_revision())]
-
-    def lock_tree_write(self):
-        """See WorkingTree.lock_tree_write().
-
-        In Format2 WorkingTrees we have a single lock for the branch and tree
-        so lock_tree_write() degrades to lock_write().
-
-        :return: An object with an unlock method which will release the lock
-            obtained.
-        """
-        self.branch.lock_write()
-        try:
-            self._control_files.lock_write()
-            return self
-        except:
-            self.branch.unlock()
-            raise
-
-    def unlock(self):
-        # do non-implementation specific cleanup
-        self._cleanup()
-
-        # we share control files:
-        if self._control_files._lock_count == 3:
-            # _inventory_is_modified is always False during a read lock.
-            if self._inventory_is_modified:
-                self.flush()
-            self._write_hashcache_if_dirty()
-
-        # reverse order of locking.
-        try:
-            return self._control_files.unlock()
+    def __init__(self, basedir='.',
+                 branch=DEPRECATED_PARAMETER,
+                 _inventory=None,
+                 _control_files=None,
+                 _internal=False,
+                 _format=None,
+                 _bzrdir=None):
+        """Construct a InventoryWorkingTree instance. This is not a public API.
+
+        :param branch: A branch to override probing for the branch.
+        """
+        super(InventoryWorkingTree, self).__init__(basedir=basedir,
+            branch=branch, _control_files=_control_files, _internal=_internal,
+            _format=_format, _bzrdir=_bzrdir)
+
+        self._detect_case_handling()
+
+        if _inventory is None:
+            # This will be acquired on lock_read() or lock_write()
+            self._inventory_is_modified = False
+            self._inventory = None
+        else:
+            # the caller of __init__ has provided an inventory,
+            # we assume they know what they are doing - as its only
+            # the Format factory and creation methods that are
+            # permitted to do this.
+            self._set_inventory(_inventory, dirty=False)
+
+    def _set_inventory(self, inv, dirty):
+        """Set the internal cached inventory.
+
+        :param inv: The inventory to set.
+        :param dirty: A boolean indicating whether the inventory is the same
+            logical inventory as whats on disk. If True the inventory is not
+            the same and should be written to disk or data will be lost, if
+            False then the inventory is the same as that on disk and any
+            serialisation would be unneeded overhead.
+        """
+        self._inventory = inv
+        self._inventory_is_modified = dirty
+
+    def _detect_case_handling(self):
+        wt_trans = self.bzrdir.get_workingtree_transport(None)
+        try:
+            wt_trans.stat(self._format.case_sensitive_filename)
+        except errors.NoSuchFile:
+            self.case_sensitive = True
+        else:
+            self.case_sensitive = False
+
+        self._setup_directory_is_tree_reference()
+
+    def _serialize(self, inventory, out_file):
+        xml5.serializer_v5.write_inventory(self._inventory, out_file,
+            working=True)
+
+    def _deserialize(selt, in_file):
+        return xml5.serializer_v5.read_inventory(in_file)
+
+    @needs_tree_write_lock
+    def _write_inventory(self, inv):
+        """Write inventory as the current inventory."""
+        self._set_inventory(inv, dirty=True)
+        self.flush()
+
+    # XXX: This method should be deprecated in favour of taking in a proper
+    # new Inventory object.
+    @needs_tree_write_lock
+    def set_inventory(self, new_inventory_list):
+        from bzrlib.inventory import (Inventory,
+                                      InventoryDirectory,
+                                      InventoryFile,
+                                      InventoryLink)
+        inv = Inventory(self.get_root_id())
+        for path, file_id, parent, kind in new_inventory_list:
+            name = os.path.basename(path)
+            if name == "":
+                continue
+            # fixme, there should be a factory function inv,add_??
+            if kind == 'directory':
+                inv.add(InventoryDirectory(file_id, name, parent))
+            elif kind == 'file':
+                inv.add(InventoryFile(file_id, name, parent))
+            elif kind == 'symlink':
+                inv.add(InventoryLink(file_id, name, parent))
+            else:
+                raise errors.BzrError("unknown kind %r" % kind)
+        self._write_inventory(inv)
+
+    def _write_basis_inventory(self, xml):
+        """Write the basis inventory XML to the basis-inventory file"""
+        path = self._basis_inventory_name()
+        sio = StringIO(xml)
+        self._transport.put_file(path, sio,
+            mode=self.bzrdir._get_file_mode())
+
+    def _reset_data(self):
+        """Reset transient data that cannot be revalidated."""
+        self._inventory_is_modified = False
+        f = self._transport.get('inventory')
+        try:
+            result = self._deserialize(f)
         finally:
-            self.branch.unlock()
-
-
-class WorkingTree3(WorkingTree):
-    """This is the Format 3 working tree.
-
-    This differs from the base WorkingTree by:
-     - having its own file lock
-     - having its own last-revision property.
-
-    This is new in bzr 0.8
-    """
-
-    @needs_read_lock
-    def _last_revision(self):
-        """See Mutable.last_revision."""
+            f.close()
+        self._set_inventory(result, dirty=False)
+
+    def _set_root_id(self, file_id):
+        """Set the root id for this tree, in a format specific manner.
+
+        :param file_id: The file id to assign to the root. It must not be
+            present in the current inventory or an error will occur. It must
+            not be None, but rather a valid file id.
+        """
+        inv = self._inventory
+        orig_root_id = inv.root.file_id
+        # TODO: it might be nice to exit early if there was nothing
+        # to do, saving us from trigger a sync on unlock.
+        self._inventory_is_modified = True
+        # we preserve the root inventory entry object, but
+        # unlinkit from the byid index
+        del inv._byid[inv.root.file_id]
+        inv.root.file_id = file_id
+        # and link it into the index with the new changed id.
+        inv._byid[inv.root.file_id] = inv.root
+        # and finally update all children to reference the new id.
+        # XXX: this should be safe to just look at the root.children
+        # list, not the WHOLE INVENTORY.
+        for fid in inv:
+            entry = inv[fid]
+            if entry.parent_id == orig_root_id:
+                entry.parent_id = inv.root.file_id
+
+    def all_file_ids(self):
+        """See Tree.iter_all_file_ids"""
+        return set(self.inventory)
+
+    @needs_tree_write_lock
+    def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
+        """See MutableTree.set_parent_trees."""
+        parent_ids = [rev for (rev, tree) in parents_list]
+        for revision_id in parent_ids:
+            _mod_revision.check_not_reserved_id(revision_id)
+
+        self._check_parents_for_ghosts(parent_ids,
+            allow_leftmost_as_ghost=allow_leftmost_as_ghost)
+
+        parent_ids = self._filter_parent_ids_by_ancestry(parent_ids)
+
+        if len(parent_ids) == 0:
+            leftmost_parent_id = _mod_revision.NULL_REVISION
+            leftmost_parent_tree = None
+        else:
+            leftmost_parent_id, leftmost_parent_tree = parents_list[0]
+
+        if self._change_last_revision(leftmost_parent_id):
+            if leftmost_parent_tree is None:
+                # If we don't have a tree, fall back to reading the
+                # parent tree from the repository.
+                self._cache_basis_inventory(leftmost_parent_id)
+            else:
+                inv = leftmost_parent_tree.inventory
+                xml = self._create_basis_xml_from_inventory(
+                                        leftmost_parent_id, inv)
+                self._write_basis_inventory(xml)
+        self._set_merges_from_parent_ids(parent_ids)
+
+    def _cache_basis_inventory(self, new_revision):
+        """Cache new_revision as the basis inventory."""
+        # TODO: this should allow the ready-to-use inventory to be passed in,
+        # as commit already has that ready-to-use [while the format is the
+        # same, that is].
         try:
-            return self._transport.get_bytes('last-revision')
-        except errors.NoSuchFile:
-            return _mod_revision.NULL_REVISION
-
-    def _change_last_revision(self, revision_id):
-        """See WorkingTree._change_last_revision."""
-        if revision_id is None or revision_id == _mod_revision.NULL_REVISION:
-            try:
-                self._transport.delete('last-revision')
-            except errors.NoSuchFile:
-                pass
-            return False
-        else:
-            self._transport.put_bytes('last-revision', revision_id,
-                mode=self.bzrdir._get_file_mode())
-            return True
-
-    def _get_check_refs(self):
-        """Return the references needed to perform a check of this tree."""
-        return [('trees', self.last_revision())]
+            # this double handles the inventory - unpack and repack -
+            # but is easier to understand. We can/should put a conditional
+            # in here based on whether the inventory is in the latest format
+            # - perhaps we should repack all inventories on a repository
+            # upgrade ?
+            # the fast path is to copy the raw xml from the repository. If the
+            # xml contains 'revision_id="', then we assume the right
+            # revision_id is set. We must check for this full string, because a
+            # root node id can legitimately look like 'revision_id' but cannot
+            # contain a '"'.
+            xml = self.branch.repository._get_inventory_xml(new_revision)
+            firstline = xml.split('\n', 1)[0]
+            if (not 'revision_id="' in firstline or
+                'format="7"' not in firstline):
+                inv = self.branch.repository._serializer.read_inventory_from_string(
+                    xml, new_revision)
+                xml = self._create_basis_xml_from_inventory(new_revision, inv)
+            self._write_basis_inventory(xml)
+        except (errors.NoSuchRevision, errors.RevisionNotPresent):
+            pass
+
+    def _basis_inventory_name(self):
+        return 'basis-inventory-cache'
+
+    def _create_basis_xml_from_inventory(self, revision_id, inventory):
+        """Create the text that will be saved in basis-inventory"""
+        inventory.revision_id = revision_id
+        return xml7.serializer_v7.write_inventory_to_string(inventory)
 
     @needs_tree_write_lock
     def set_conflicts(self, conflicts):
@@ -2797,32 +1994,963 @@
                     raise errors.ConflictFormatError()
             except StopIteration:
                 raise errors.ConflictFormatError()
-            return _mod_conflicts.ConflictList.from_stanzas(RioReader(confile))
+            reader = _mod_rio.RioReader(confile)
+            return _mod_conflicts.ConflictList.from_stanzas(reader)
         finally:
             confile.close()
 
-    def unlock(self):
-        # do non-implementation specific cleanup
-        self._cleanup()
-        if self._control_files._lock_count == 1:
-            # _inventory_is_modified is always False during a read lock.
-            if self._inventory_is_modified:
-                self.flush()
-            self._write_hashcache_if_dirty()
-        # reverse order of locking.
-        try:
-            return self._control_files.unlock()
-        finally:
-            self.branch.unlock()
-
-
-def get_conflicted_stem(path):
-    for suffix in _mod_conflicts.CONFLICT_SUFFIXES:
-        if path.endswith(suffix):
-            return path[:-len(suffix)]
-
-
-class WorkingTreeFormat(object):
+    def read_basis_inventory(self):
+        """Read the cached basis inventory."""
+        path = self._basis_inventory_name()
+        return self._transport.get_bytes(path)
+
+    @needs_read_lock
+    def read_working_inventory(self):
+        """Read the working inventory.
+
+        :raises errors.InventoryModified: read_working_inventory will fail
+            when the current in memory inventory has been modified.
+        """
+        # conceptually this should be an implementation detail of the tree.
+        # XXX: Deprecate this.
+        # ElementTree does its own conversion from UTF-8, so open in
+        # binary.
+        if self._inventory_is_modified:
+            raise errors.InventoryModified(self)
+        f = self._transport.get('inventory')
+        try:
+            result = self._deserialize(f)
+        finally:
+            f.close()
+        self._set_inventory(result, dirty=False)
+        return result
+
+    @needs_read_lock
+    def get_root_id(self):
+        """Return the id of this trees root"""
+        return self._inventory.root.file_id
+
+    def has_id(self, file_id):
+        # files that have been deleted are excluded
+        inv = self.inventory
+        if not inv.has_id(file_id):
+            return False
+        path = inv.id2path(file_id)
+        return osutils.lexists(self.abspath(path))
+
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
+
+    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
+    def __iter__(self):
+        """Iterate through file_ids for this tree.
+
+        file_ids are in a WorkingTree if they are in the working inventory
+        and the working file exists.
+        """
+        inv = self._inventory
+        for path, ie in inv.iter_entries():
+            if osutils.lexists(self.abspath(path)):
+                yield ie.file_id
+
+    @needs_tree_write_lock
+    def set_last_revision(self, new_revision):
+        """Change the last revision in the working tree."""
+        if self._change_last_revision(new_revision):
+            self._cache_basis_inventory(new_revision)
+
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree.
+        
+        The default implementation returns no refs, and is only suitable for
+        trees that have no local caching and can commit on ghosts at any time.
+
+        :seealso: bzrlib.check for details about check_refs.
+        """
+        return []
+
+    @needs_read_lock
+    def _check(self, references):
+        """Check the tree for consistency.
+
+        :param references: A dict with keys matching the items returned by
+            self._get_check_refs(), and values from looking those keys up in
+            the repository.
+        """
+        tree_basis = self.basis_tree()
+        tree_basis.lock_read()
+        try:
+            repo_basis = references[('trees', self.last_revision())]
+            if len(list(repo_basis.iter_changes(tree_basis))) > 0:
+                raise errors.BzrCheckError(
+                    "Mismatched basis inventory content.")
+            self._validate()
+        finally:
+            tree_basis.unlock()
+
+    @needs_read_lock
+    def check_state(self):
+        """Check that the working state is/isn't valid."""
+        check_refs = self._get_check_refs()
+        refs = {}
+        for ref in check_refs:
+            kind, value = ref
+            if kind == 'trees':
+                refs[ref] = self.branch.repository.revision_tree(value)
+        self._check(refs)
+
+    @needs_tree_write_lock
+    def reset_state(self, revision_ids=None):
+        """Reset the state of the working tree.
+
+        This does a hard-reset to a last-known-good state. This is a way to
+        fix if something got corrupted (like the .bzr/checkout/dirstate file)
+        """
+        if revision_ids is None:
+            revision_ids = self.get_parent_ids()
+        if not revision_ids:
+            rt = self.branch.repository.revision_tree(
+                _mod_revision.NULL_REVISION)
+        else:
+            rt = self.branch.repository.revision_tree(revision_ids[0])
+        self._write_inventory(rt.inventory)
+        self.set_parent_ids(revision_ids)
+
+    def flush(self):
+        """Write the in memory inventory to disk."""
+        # TODO: Maybe this should only write on dirty ?
+        if self._control_files._lock_mode != 'w':
+            raise errors.NotWriteLocked(self)
+        sio = StringIO()
+        self._serialize(self._inventory, sio)
+        sio.seek(0)
+        self._transport.put_file('inventory', sio,
+            mode=self.bzrdir._get_file_mode())
+        self._inventory_is_modified = False
+
+    def get_file_mtime(self, file_id, path=None):
+        """See Tree.get_file_mtime."""
+        if not path:
+            path = self.inventory.id2path(file_id)
+        try:
+            return os.lstat(self.abspath(path)).st_mtime
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                raise errors.FileTimestampUnavailable(path)
+            raise
+
+    def _is_executable_from_path_and_stat_from_basis(self, path, stat_result):
+        file_id = self.path2id(path)
+        if file_id is None:
+            # For unversioned files on win32, we just assume they are not
+            # executable
+            return False
+        return self._inventory[file_id].executable
+
+    def _is_executable_from_path_and_stat_from_stat(self, path, stat_result):
+        mode = stat_result.st_mode
+        return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
+
+    if not supports_executable():
+        def is_executable(self, file_id, path=None):
+            return self._inventory[file_id].executable
+
+        _is_executable_from_path_and_stat = \
+            _is_executable_from_path_and_stat_from_basis
+    else:
+        def is_executable(self, file_id, path=None):
+            if not path:
+                path = self.id2path(file_id)
+            mode = os.lstat(self.abspath(path)).st_mode
+            return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
+
+        _is_executable_from_path_and_stat = \
+            _is_executable_from_path_and_stat_from_stat
+
+    @needs_tree_write_lock
+    def _add(self, files, ids, kinds):
+        """See MutableTree._add."""
+        # TODO: Re-adding a file that is removed in the working copy
+        # should probably put it back with the previous ID.
+        # the read and write working inventory should not occur in this
+        # function - they should be part of lock_write and unlock.
+        inv = self.inventory
+        for f, file_id, kind in zip(files, ids, kinds):
+            if file_id is None:
+                inv.add_path(f, kind=kind)
+            else:
+                inv.add_path(f, kind=kind, file_id=file_id)
+            self._inventory_is_modified = True
+
+    def revision_tree(self, revision_id):
+        """See WorkingTree.revision_id."""
+        if revision_id == self.last_revision():
+            try:
+                xml = self.read_basis_inventory()
+            except errors.NoSuchFile:
+                pass
+            else:
+                try:
+                    inv = xml7.serializer_v7.read_inventory_from_string(xml)
+                    # dont use the repository revision_tree api because we want
+                    # to supply the inventory.
+                    if inv.revision_id == revision_id:
+                        return revisiontree.InventoryRevisionTree(
+                            self.branch.repository, inv, revision_id)
+                except errors.BadInventoryFormat:
+                    pass
+        # raise if there was no inventory, or if we read the wrong inventory.
+        raise errors.NoSuchRevisionInTree(self, revision_id)
+
+    @needs_read_lock
+    def annotate_iter(self, file_id, default_revision=CURRENT_REVISION):
+        """See Tree.annotate_iter
+
+        This implementation will use the basis tree implementation if possible.
+        Lines not in the basis are attributed to CURRENT_REVISION
+
+        If there are pending merges, lines added by those merges will be
+        incorrectly attributed to CURRENT_REVISION (but after committing, the
+        attribution will be correct).
+        """
+        maybe_file_parent_keys = []
+        for parent_id in self.get_parent_ids():
+            try:
+                parent_tree = self.revision_tree(parent_id)
+            except errors.NoSuchRevisionInTree:
+                parent_tree = self.branch.repository.revision_tree(parent_id)
+            parent_tree.lock_read()
+            try:
+                if not parent_tree.has_id(file_id):
+                    continue
+                ie = parent_tree.inventory[file_id]
+                if ie.kind != 'file':
+                    # Note: this is slightly unnecessary, because symlinks and
+                    # directories have a "text" which is the empty text, and we
+                    # know that won't mess up annotations. But it seems cleaner
+                    continue
+                parent_text_key = (file_id, ie.revision)
+                if parent_text_key not in maybe_file_parent_keys:
+                    maybe_file_parent_keys.append(parent_text_key)
+            finally:
+                parent_tree.unlock()
+        graph = _mod_graph.Graph(self.branch.repository.texts)
+        heads = graph.heads(maybe_file_parent_keys)
+        file_parent_keys = []
+        for key in maybe_file_parent_keys:
+            if key in heads:
+                file_parent_keys.append(key)
+
+        # Now we have the parents of this content
+        annotator = self.branch.repository.texts.get_annotator()
+        text = self.get_file_text(file_id)
+        this_key =(file_id, default_revision)
+        annotator.add_special_text(this_key, file_parent_keys, text)
+        annotations = [(key[-1], line)
+                       for key, line in annotator.annotate_flat(this_key)]
+        return annotations
+
+    @needs_read_lock
+    def merge_modified(self):
+        """Return a dictionary of files modified by a merge.
+
+        The list is initialized by WorkingTree.set_merge_modified, which is
+        typically called after we make some automatic updates to the tree
+        because of a merge.
+
+        This returns a map of file_id->sha1, containing only files which are
+        still in the working inventory and have that text hash.
+        """
+        try:
+            hashfile = self._transport.get('merge-hashes')
+        except errors.NoSuchFile:
+            return {}
+        try:
+            merge_hashes = {}
+            try:
+                if hashfile.next() != MERGE_MODIFIED_HEADER_1 + '\n':
+                    raise errors.MergeModifiedFormatError()
+            except StopIteration:
+                raise errors.MergeModifiedFormatError()
+            for s in _mod_rio.RioReader(hashfile):
+                # RioReader reads in Unicode, so convert file_ids back to utf8
+                file_id = osutils.safe_file_id(s.get("file_id"), warn=False)
+                if not self.inventory.has_id(file_id):
+                    continue
+                text_hash = s.get("hash")
+                if text_hash == self.get_file_sha1(file_id):
+                    merge_hashes[file_id] = text_hash
+            return merge_hashes
+        finally:
+            hashfile.close()
+
+    @needs_write_lock
+    def subsume(self, other_tree):
+        def add_children(inventory, entry):
+            for child_entry in entry.children.values():
+                inventory._byid[child_entry.file_id] = child_entry
+                if child_entry.kind == 'directory':
+                    add_children(inventory, child_entry)
+        if other_tree.get_root_id() == self.get_root_id():
+            raise errors.BadSubsumeSource(self, other_tree,
+                                          'Trees have the same root')
+        try:
+            other_tree_path = self.relpath(other_tree.basedir)
+        except errors.PathNotChild:
+            raise errors.BadSubsumeSource(self, other_tree,
+                'Tree is not contained by the other')
+        new_root_parent = self.path2id(osutils.dirname(other_tree_path))
+        if new_root_parent is None:
+            raise errors.BadSubsumeSource(self, other_tree,
+                'Parent directory is not versioned.')
+        # We need to ensure that the result of a fetch will have a
+        # versionedfile for the other_tree root, and only fetching into
+        # RepositoryKnit2 guarantees that.
+        if not self.branch.repository.supports_rich_root():
+            raise errors.SubsumeTargetNeedsUpgrade(other_tree)
+        other_tree.lock_tree_write()
+        try:
+            new_parents = other_tree.get_parent_ids()
+            other_root = other_tree.inventory.root
+            other_root.parent_id = new_root_parent
+            other_root.name = osutils.basename(other_tree_path)
+            self.inventory.add(other_root)
+            add_children(self.inventory, other_root)
+            self._write_inventory(self.inventory)
+            # normally we don't want to fetch whole repositories, but i think
+            # here we really do want to consolidate the whole thing.
+            for parent_id in other_tree.get_parent_ids():
+                self.branch.fetch(other_tree.branch, parent_id)
+                self.add_parent_tree_id(parent_id)
+        finally:
+            other_tree.unlock()
+        other_tree.bzrdir.retire_bzrdir()
+
+    @needs_tree_write_lock
+    def extract(self, file_id, format=None):
+        """Extract a subtree from this tree.
+
+        A new branch will be created, relative to the path for this tree.
+        """
+        self.flush()
+        def mkdirs(path):
+            segments = osutils.splitpath(path)
+            transport = self.branch.bzrdir.root_transport
+            for name in segments:
+                transport = transport.clone(name)
+                transport.ensure_base()
+            return transport
+
+        sub_path = self.id2path(file_id)
+        branch_transport = mkdirs(sub_path)
+        if format is None:
+            format = self.bzrdir.cloning_metadir()
+        branch_transport.ensure_base()
+        branch_bzrdir = format.initialize_on_transport(branch_transport)
+        try:
+            repo = branch_bzrdir.find_repository()
+        except errors.NoRepositoryPresent:
+            repo = branch_bzrdir.create_repository()
+        if not repo.supports_rich_root():
+            raise errors.RootNotRich()
+        new_branch = branch_bzrdir.create_branch()
+        new_branch.pull(self.branch)
+        for parent_id in self.get_parent_ids():
+            new_branch.fetch(self.branch, parent_id)
+        tree_transport = self.bzrdir.root_transport.clone(sub_path)
+        if tree_transport.base != branch_transport.base:
+            tree_bzrdir = format.initialize_on_transport(tree_transport)
+            branch.BranchReferenceFormat().initialize(tree_bzrdir,
+                target_branch=new_branch)
+        else:
+            tree_bzrdir = branch_bzrdir
+        wt = tree_bzrdir.create_workingtree(_mod_revision.NULL_REVISION)
+        wt.set_parent_ids(self.get_parent_ids())
+        my_inv = self.inventory
+        child_inv = inventory.Inventory(root_id=None)
+        new_root = my_inv[file_id]
+        my_inv.remove_recursive_id(file_id)
+        new_root.parent_id = None
+        child_inv.add(new_root)
+        self._write_inventory(my_inv)
+        wt._write_inventory(child_inv)
+        return wt
+
+    def list_files(self, include_root=False, from_dir=None, recursive=True):
+        """List all files as (path, class, kind, id, entry).
+
+        Lists, but does not descend into unversioned directories.
+        This does not include files that have been deleted in this
+        tree. Skips the control directory.
+
+        :param include_root: if True, return an entry for the root
+        :param from_dir: start from this directory or None for the root
+        :param recursive: whether to recurse into subdirectories or not
+        """
+        # list_files is an iterator, so @needs_read_lock doesn't work properly
+        # with it. So callers should be careful to always read_lock the tree.
+        if not self.is_locked():
+            raise errors.ObjectNotLocked(self)
+
+        inv = self.inventory
+        if from_dir is None and include_root is True:
+            yield ('', 'V', 'directory', inv.root.file_id, inv.root)
+        # Convert these into local objects to save lookup times
+        pathjoin = osutils.pathjoin
+        file_kind = self._kind
+
+        # transport.base ends in a slash, we want the piece
+        # between the last two slashes
+        transport_base_dir = self.bzrdir.transport.base.rsplit('/', 2)[1]
+
+        fk_entries = {'directory':TreeDirectory, 'file':TreeFile, 'symlink':TreeLink}
+
+        # directory file_id, relative path, absolute path, reverse sorted children
+        if from_dir is not None:
+            from_dir_id = inv.path2id(from_dir)
+            if from_dir_id is None:
+                # Directory not versioned
+                return
+            from_dir_abspath = pathjoin(self.basedir, from_dir)
+        else:
+            from_dir_id = inv.root.file_id
+            from_dir_abspath = self.basedir
+        children = os.listdir(from_dir_abspath)
+        children.sort()
+        # jam 20060527 The kernel sized tree seems equivalent whether we
+        # use a deque and popleft to keep them sorted, or if we use a plain
+        # list and just reverse() them.
+        children = collections.deque(children)
+        stack = [(from_dir_id, u'', from_dir_abspath, children)]
+        while stack:
+            from_dir_id, from_dir_relpath, from_dir_abspath, children = stack[-1]
+
+            while children:
+                f = children.popleft()
+                ## TODO: If we find a subdirectory with its own .bzr
+                ## directory, then that is a separate tree and we
+                ## should exclude it.
+
+                # the bzrdir for this tree
+                if transport_base_dir == f:
+                    continue
+
+                # we know that from_dir_relpath and from_dir_abspath never end in a slash
+                # and 'f' doesn't begin with one, we can do a string op, rather
+                # than the checks of pathjoin(), all relative paths will have an extra slash
+                # at the beginning
+                fp = from_dir_relpath + '/' + f
+
+                # absolute path
+                fap = from_dir_abspath + '/' + f
+
+                dir_ie = inv[from_dir_id]
+                if dir_ie.kind == 'directory':
+                    f_ie = dir_ie.children.get(f)
+                else:
+                    f_ie = None
+                if f_ie:
+                    c = 'V'
+                elif self.is_ignored(fp[1:]):
+                    c = 'I'
+                else:
+                    # we may not have found this file, because of a unicode
+                    # issue, or because the directory was actually a symlink.
+                    f_norm, can_access = osutils.normalized_filename(f)
+                    if f == f_norm or not can_access:
+                        # No change, so treat this file normally
+                        c = '?'
+                    else:
+                        # this file can be accessed by a normalized path
+                        # check again if it is versioned
+                        # these lines are repeated here for performance
+                        f = f_norm
+                        fp = from_dir_relpath + '/' + f
+                        fap = from_dir_abspath + '/' + f
+                        f_ie = inv.get_child(from_dir_id, f)
+                        if f_ie:
+                            c = 'V'
+                        elif self.is_ignored(fp[1:]):
+                            c = 'I'
+                        else:
+                            c = '?'
+
+                fk = file_kind(fap)
+
+                # make a last minute entry
+                if f_ie:
+                    yield fp[1:], c, fk, f_ie.file_id, f_ie
+                else:
+                    try:
+                        yield fp[1:], c, fk, None, fk_entries[fk]()
+                    except KeyError:
+                        yield fp[1:], c, fk, None, TreeEntry()
+                    continue
+
+                if fk != 'directory':
+                    continue
+
+                # But do this child first if recursing down
+                if recursive:
+                    new_children = os.listdir(fap)
+                    new_children.sort()
+                    new_children = collections.deque(new_children)
+                    stack.append((f_ie.file_id, fp, fap, new_children))
+                    # Break out of inner loop,
+                    # so that we start outer loop with child
+                    break
+            else:
+                # if we finished all children, pop it off the stack
+                stack.pop()
+
+    @needs_tree_write_lock
+    def move(self, from_paths, to_dir=None, after=False):
+        """Rename files.
+
+        to_dir must exist in the inventory.
+
+        If to_dir exists and is a directory, the files are moved into
+        it, keeping their old names.
+
+        Note that to_dir is only the last component of the new name;
+        this doesn't change the directory.
+
+        For each entry in from_paths the move mode will be determined
+        independently.
+
+        The first mode moves the file in the filesystem and updates the
+        inventory. The second mode only updates the inventory without
+        touching the file on the filesystem.
+
+        move uses the second mode if 'after == True' and the target is
+        either not versioned or newly added, and present in the working tree.
+
+        move uses the second mode if 'after == False' and the source is
+        versioned but no longer in the working tree, and the target is not
+        versioned but present in the working tree.
+
+        move uses the first mode if 'after == False' and the source is
+        versioned and present in the working tree, and the target is not
+        versioned and not present in the working tree.
+
+        Everything else results in an error.
+
+        This returns a list of (from_path, to_path) pairs for each
+        entry that is moved.
+        """
+        rename_entries = []
+        rename_tuples = []
+
+        # check for deprecated use of signature
+        if to_dir is None:
+            raise TypeError('You must supply a target directory')
+        # check destination directory
+        if isinstance(from_paths, basestring):
+            raise ValueError()
+        inv = self.inventory
+        to_abs = self.abspath(to_dir)
+        if not isdir(to_abs):
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotADirectory(to_abs))
+        if not self.has_filename(to_dir):
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotInWorkingDirectory(to_dir))
+        to_dir_id = inv.path2id(to_dir)
+        if to_dir_id is None:
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotVersionedError(path=to_dir))
+
+        to_dir_ie = inv[to_dir_id]
+        if to_dir_ie.kind != 'directory':
+            raise errors.BzrMoveFailedError('',to_dir,
+                errors.NotADirectory(to_abs))
+
+        # create rename entries and tuples
+        for from_rel in from_paths:
+            from_tail = splitpath(from_rel)[-1]
+            from_id = inv.path2id(from_rel)
+            if from_id is None:
+                raise errors.BzrMoveFailedError(from_rel,to_dir,
+                    errors.NotVersionedError(path=from_rel))
+
+            from_entry = inv[from_id]
+            from_parent_id = from_entry.parent_id
+            to_rel = pathjoin(to_dir, from_tail)
+            rename_entry = InventoryWorkingTree._RenameEntry(
+                from_rel=from_rel,
+                from_id=from_id,
+                from_tail=from_tail,
+                from_parent_id=from_parent_id,
+                to_rel=to_rel, to_tail=from_tail,
+                to_parent_id=to_dir_id)
+            rename_entries.append(rename_entry)
+            rename_tuples.append((from_rel, to_rel))
+
+        # determine which move mode to use. checks also for movability
+        rename_entries = self._determine_mv_mode(rename_entries, after)
+
+        original_modified = self._inventory_is_modified
+        try:
+            if len(from_paths):
+                self._inventory_is_modified = True
+            self._move(rename_entries)
+        except:
+            # restore the inventory on error
+            self._inventory_is_modified = original_modified
+            raise
+        self._write_inventory(inv)
+        return rename_tuples
+
+    @needs_tree_write_lock
+    def rename_one(self, from_rel, to_rel, after=False):
+        """Rename one file.
+
+        This can change the directory or the filename or both.
+
+        rename_one has several 'modes' to work. First, it can rename a physical
+        file and change the file_id. That is the normal mode. Second, it can
+        only change the file_id without touching any physical file.
+
+        rename_one uses the second mode if 'after == True' and 'to_rel' is not
+        versioned but present in the working tree.
+
+        rename_one uses the second mode if 'after == False' and 'from_rel' is
+        versioned but no longer in the working tree, and 'to_rel' is not
+        versioned but present in the working tree.
+
+        rename_one uses the first mode if 'after == False' and 'from_rel' is
+        versioned and present in the working tree, and 'to_rel' is not
+        versioned and not present in the working tree.
+
+        Everything else results in an error.
+        """
+        inv = self.inventory
+        rename_entries = []
+
+        # create rename entries and tuples
+        from_tail = splitpath(from_rel)[-1]
+        from_id = inv.path2id(from_rel)
+        if from_id is None:
+            # if file is missing in the inventory maybe it's in the basis_tree
+            basis_tree = self.branch.basis_tree()
+            from_id = basis_tree.path2id(from_rel)
+            if from_id is None:
+                raise errors.BzrRenameFailedError(from_rel,to_rel,
+                    errors.NotVersionedError(path=from_rel))
+            # put entry back in the inventory so we can rename it
+            from_entry = basis_tree.inventory[from_id].copy()
+            inv.add(from_entry)
+        else:
+            from_entry = inv[from_id]
+        from_parent_id = from_entry.parent_id
+        to_dir, to_tail = os.path.split(to_rel)
+        to_dir_id = inv.path2id(to_dir)
+        rename_entry = InventoryWorkingTree._RenameEntry(from_rel=from_rel,
+                                     from_id=from_id,
+                                     from_tail=from_tail,
+                                     from_parent_id=from_parent_id,
+                                     to_rel=to_rel, to_tail=to_tail,
+                                     to_parent_id=to_dir_id)
+        rename_entries.append(rename_entry)
+
+        # determine which move mode to use. checks also for movability
+        rename_entries = self._determine_mv_mode(rename_entries, after)
+
+        # check if the target changed directory and if the target directory is
+        # versioned
+        if to_dir_id is None:
+            raise errors.BzrMoveFailedError(from_rel,to_rel,
+                errors.NotVersionedError(path=to_dir))
+
+        # all checks done. now we can continue with our actual work
+        mutter('rename_one:\n'
+               '  from_id   {%s}\n'
+               '  from_rel: %r\n'
+               '  to_rel:   %r\n'
+               '  to_dir    %r\n'
+               '  to_dir_id {%s}\n',
+               from_id, from_rel, to_rel, to_dir, to_dir_id)
+
+        self._move(rename_entries)
+        self._write_inventory(inv)
+
+    class _RenameEntry(object):
+        def __init__(self, from_rel, from_id, from_tail, from_parent_id,
+                     to_rel, to_tail, to_parent_id, only_change_inv=False,
+                     change_id=False):
+            self.from_rel = from_rel
+            self.from_id = from_id
+            self.from_tail = from_tail
+            self.from_parent_id = from_parent_id
+            self.to_rel = to_rel
+            self.to_tail = to_tail
+            self.to_parent_id = to_parent_id
+            self.change_id = change_id
+            self.only_change_inv = only_change_inv
+
+    def _determine_mv_mode(self, rename_entries, after=False):
+        """Determines for each from-to pair if both inventory and working tree
+        or only the inventory has to be changed.
+
+        Also does basic plausability tests.
+        """
+        inv = self.inventory
+
+        for rename_entry in rename_entries:
+            # store to local variables for easier reference
+            from_rel = rename_entry.from_rel
+            from_id = rename_entry.from_id
+            to_rel = rename_entry.to_rel
+            to_id = inv.path2id(to_rel)
+            only_change_inv = False
+            change_id = False
+
+            # check the inventory for source and destination
+            if from_id is None:
+                raise errors.BzrMoveFailedError(from_rel,to_rel,
+                    errors.NotVersionedError(path=from_rel))
+            if to_id is not None:
+                allowed = False
+                # allow it with --after but only if dest is newly added
+                if after:
+                    basis = self.basis_tree()
+                    basis.lock_read()
+                    try:
+                        if not basis.has_id(to_id):
+                            rename_entry.change_id = True
+                            allowed = True
+                    finally:
+                        basis.unlock()
+                if not allowed:
+                    raise errors.BzrMoveFailedError(from_rel,to_rel,
+                        errors.AlreadyVersionedError(path=to_rel))
+
+            # try to determine the mode for rename (only change inv or change
+            # inv and file system)
+            if after:
+                if not self.has_filename(to_rel):
+                    raise errors.BzrMoveFailedError(from_id,to_rel,
+                        errors.NoSuchFile(path=to_rel,
+                        extra="New file has not been created yet"))
+                only_change_inv = True
+            elif not self.has_filename(from_rel) and self.has_filename(to_rel):
+                only_change_inv = True
+            elif self.has_filename(from_rel) and not self.has_filename(to_rel):
+                only_change_inv = False
+            elif (not self.case_sensitive
+                  and from_rel.lower() == to_rel.lower()
+                  and self.has_filename(from_rel)):
+                only_change_inv = False
+            else:
+                # something is wrong, so lets determine what exactly
+                if not self.has_filename(from_rel) and \
+                   not self.has_filename(to_rel):
+                    raise errors.BzrRenameFailedError(from_rel,to_rel,
+                        errors.PathsDoNotExist(paths=(str(from_rel),
+                        str(to_rel))))
+                else:
+                    raise errors.RenameFailedFilesExist(from_rel, to_rel)
+            rename_entry.only_change_inv = only_change_inv
+        return rename_entries
+
+    def _move(self, rename_entries):
+        """Moves a list of files.
+
+        Depending on the value of the flag 'only_change_inv', the
+        file will be moved on the file system or not.
+        """
+        inv = self.inventory
+        moved = []
+
+        for entry in rename_entries:
+            try:
+                self._move_entry(entry)
+            except:
+                self._rollback_move(moved)
+                raise
+            moved.append(entry)
+
+    def _rollback_move(self, moved):
+        """Try to rollback a previous move in case of an filesystem error."""
+        inv = self.inventory
+        for entry in moved:
+            try:
+                self._move_entry(WorkingTree._RenameEntry(
+                    entry.to_rel, entry.from_id,
+                    entry.to_tail, entry.to_parent_id, entry.from_rel,
+                    entry.from_tail, entry.from_parent_id,
+                    entry.only_change_inv))
+            except errors.BzrMoveFailedError, e:
+                raise errors.BzrMoveFailedError( '', '', "Rollback failed."
+                        " The working tree is in an inconsistent state."
+                        " Please consider doing a 'bzr revert'."
+                        " Error message is: %s" % e)
+
+    def _move_entry(self, entry):
+        inv = self.inventory
+        from_rel_abs = self.abspath(entry.from_rel)
+        to_rel_abs = self.abspath(entry.to_rel)
+        if from_rel_abs == to_rel_abs:
+            raise errors.BzrMoveFailedError(entry.from_rel, entry.to_rel,
+                "Source and target are identical.")
+
+        if not entry.only_change_inv:
+            try:
+                osutils.rename(from_rel_abs, to_rel_abs)
+            except OSError, e:
+                raise errors.BzrMoveFailedError(entry.from_rel,
+                    entry.to_rel, e[1])
+        if entry.change_id:
+            to_id = inv.path2id(entry.to_rel)
+            inv.remove_recursive_id(to_id)
+        inv.rename(entry.from_id, entry.to_parent_id, entry.to_tail)
+
+    @needs_tree_write_lock
+    def unversion(self, file_ids):
+        """Remove the file ids in file_ids from the current versioned set.
+
+        When a file_id is unversioned, all of its children are automatically
+        unversioned.
+
+        :param file_ids: The file ids to stop versioning.
+        :raises: NoSuchId if any fileid is not currently versioned.
+        """
+        for file_id in file_ids:
+            if not self._inventory.has_id(file_id):
+                raise errors.NoSuchId(self, file_id)
+        for file_id in file_ids:
+            if self._inventory.has_id(file_id):
+                self._inventory.remove_recursive_id(file_id)
+        if len(file_ids):
+            # in the future this should just set a dirty bit to wait for the
+            # final unlock. However, until all methods of workingtree start
+            # with the current in -memory inventory rather than triggering
+            # a read, it is more complex - we need to teach read_inventory
+            # to know when to read, and when to not read first... and possibly
+            # to save first when the in memory one may be corrupted.
+            # so for now, we just only write it if it is indeed dirty.
+            # - RBC 20060907
+            self._write_inventory(self._inventory)
+
+    def stored_kind(self, file_id):
+        """See Tree.stored_kind"""
+        return self.inventory[file_id].kind
+
+    def extras(self):
+        """Yield all unversioned files in this WorkingTree.
+
+        If there are any unversioned directories then only the directory is
+        returned, not all its children.  But if there are unversioned files
+        under a versioned subdirectory, they are returned.
+
+        Currently returned depth-first, sorted by name within directories.
+        This is the same order used by 'osutils.walkdirs'.
+        """
+        ## TODO: Work from given directory downwards
+        for path, dir_entry in self.inventory.directories():
+            # mutter("search for unknowns in %r", path)
+            dirabs = self.abspath(path)
+            if not isdir(dirabs):
+                # e.g. directory deleted
+                continue
+
+            fl = []
+            for subf in os.listdir(dirabs):
+                if self.bzrdir.is_control_filename(subf):
+                    continue
+                if subf not in dir_entry.children:
+                    try:
+                        (subf_norm,
+                         can_access) = osutils.normalized_filename(subf)
+                    except UnicodeDecodeError:
+                        path_os_enc = path.encode(osutils._fs_enc)
+                        relpath = path_os_enc + '/' + subf
+                        raise errors.BadFilenameEncoding(relpath,
+                                                         osutils._fs_enc)
+                    if subf_norm != subf and can_access:
+                        if subf_norm not in dir_entry.children:
+                            fl.append(subf_norm)
+                    else:
+                        fl.append(subf)
+
+            fl.sort()
+            for subf in fl:
+                subp = pathjoin(path, subf)
+                yield subp
+
+    def _walkdirs(self, prefix=""):
+        """Walk the directories of this tree.
+
+        :param prefix: is used as the directrory to start with.
+        :returns: a generator which yields items in the form::
+
+            ((curren_directory_path, fileid),
+             [(file1_path, file1_name, file1_kind, None, file1_id,
+               file1_kind), ... ])
+        """
+        _directory = 'directory'
+        # get the root in the inventory
+        inv = self.inventory
+        top_id = inv.path2id(prefix)
+        if top_id is None:
+            pending = []
+        else:
+            pending = [(prefix, '', _directory, None, top_id, None)]
+        while pending:
+            dirblock = []
+            currentdir = pending.pop()
+            # 0 - relpath, 1- basename, 2- kind, 3- stat, 4-id, 5-kind
+            top_id = currentdir[4]
+            if currentdir[0]:
+                relroot = currentdir[0] + '/'
+            else:
+                relroot = ""
+            # FIXME: stash the node in pending
+            entry = inv[top_id]
+            if entry.kind == 'directory':
+                for name, child in entry.sorted_children():
+                    dirblock.append((relroot + name, name, child.kind, None,
+                        child.file_id, child.kind
+                        ))
+            yield (currentdir[0], entry.file_id), dirblock
+            # push the user specified dirs from dirblock
+            for dir in reversed(dirblock):
+                if dir[2] == _directory:
+                    pending.append(dir)
+
+
+class WorkingTreeFormatRegistry(controldir.ControlComponentFormatRegistry):
+    """Registry for working tree formats."""
+
+    def __init__(self, other_registry=None):
+        super(WorkingTreeFormatRegistry, self).__init__(other_registry)
+        self._default_format = None
+        self._default_format_key = None
+
+    def get_default(self):
+        """Return the current default format."""
+        if (self._default_format_key is not None and
+            self._default_format is None):
+            self._default_format = self.get(self._default_format_key)
+        return self._default_format
+
+    def set_default(self, format):
+        """Set the default format."""
+        self._default_format = format
+        self._default_format_key = None
+
+    def set_default_key(self, format_string):
+        """Set the default format by its format string."""
+        self._default_format_key = format_string
+        self._default_format = None
+
+
+format_registry = WorkingTreeFormatRegistry()
+
+
+class WorkingTreeFormat(controldir.ControlComponentFormat):
     """An encapsulation of the initialization and open routines for a format.
 
     Formats provide three things:
@@ -2840,29 +2968,55 @@
     object will be created every time regardless.
     """
 
-    _default_format = None
-    """The default format used for new trees."""
-
-    _formats = {}
-    """The known formats."""
-
     requires_rich_root = False
 
     upgrade_recommended = False
 
+    requires_normalized_unicode_filenames = False
+
+    case_sensitive_filename = "FoRMaT"
+
+    missing_parent_conflicts = False
+    """If this format supports missing parent conflicts."""
+
+    supports_versioned_directories = None
+
     @classmethod
-    def find_format(klass, a_bzrdir):
-        """Return the format for the working tree object in a_bzrdir."""
+    def find_format_string(klass, controldir):
+        """Return format name for the working tree object in controldir."""
         try:
-            transport = a_bzrdir.get_workingtree_transport(None)
-            format_string = transport.get_bytes("format")
-            return klass._formats[format_string]
+            transport = controldir.get_workingtree_transport(None)
+            return transport.get_bytes("format")
         except errors.NoSuchFile:
             raise errors.NoWorkingTree(base=transport.base)
+
+    @classmethod
+    def find_format(klass, controldir):
+        """Return the format for the working tree object in controldir."""
+        try:
+            format_string = klass.find_format_string(controldir)
+            return format_registry.get(format_string)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string,
                                             kind="working tree")
 
+    def initialize(self, controldir, revision_id=None, from_branch=None,
+                   accelerator_tree=None, hardlink=False):
+        """Initialize a new working tree in controldir.
+
+        :param controldir: ControlDir to initialize the working tree in.
+        :param revision_id: allows creating a working tree at a different
+            revision than the branch is at.
+        :param from_branch: Branch to checkout
+        :param accelerator_tree: A tree which can be used for retrieving file
+            contents more quickly than the revision tree, i.e. a workingtree.
+            The revision tree will be used for cases where accelerator_tree's
+            content is different.
+        :param hardlink: If true, hard-link files from accelerator_tree,
+            where possible.
+        """
+        raise NotImplementedError(self.initialize)
+
     def __eq__(self, other):
         return self.__class__ is other.__class__
 
@@ -2870,9 +3024,11 @@
         return not (self == other)
 
     @classmethod
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def get_default_format(klass):
         """Return the current default format."""
-        return klass._default_format
+        return format_registry.get_default()
 
     def get_format_string(self):
         """Return the ASCII format string that identifies this format."""
@@ -2900,238 +3056,56 @@
         return False
 
     @classmethod
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def register_format(klass, format):
-        klass._formats[format.get_format_string()] = format
-
-    @classmethod
+        format_registry.register(format)
+
+    @classmethod
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
+    def register_extra_format(klass, format):
+        format_registry.register_extra(format)
+
+    @classmethod
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
+    def unregister_extra_format(klass, format):
+        format_registry.unregister_extra(format)
+
+    @classmethod
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
+    def get_formats(klass):
+        return format_registry._get_all()
+
+    @classmethod
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def set_default_format(klass, format):
-        klass._default_format = format
+        format_registry.set_default(format)
 
     @classmethod
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 4, 0)))
     def unregister_format(klass, format):
-        del klass._formats[format.get_format_string()]
-
-
-class WorkingTreeFormat2(WorkingTreeFormat):
-    """The second working tree format.
-
-    This format modified the hash cache from the format 1 hash cache.
-    """
-
-    upgrade_recommended = True
-
-    def get_format_description(self):
-        """See WorkingTreeFormat.get_format_description()."""
-        return "Working tree format 2"
-
-    def _stub_initialize_on_transport(self, transport, file_mode):
-        """Workaround: create control files for a remote working tree.
-
-        This ensures that it can later be updated and dealt with locally,
-        since BzrDirFormat6 and BzrDirFormat5 cannot represent dirs with
-        no working tree.  (See bug #43064).
-        """
-        sio = StringIO()
-        inv = inventory.Inventory()
-        xml5.serializer_v5.write_inventory(inv, sio, working=True)
-        sio.seek(0)
-        transport.put_file('inventory', sio, file_mode)
-        transport.put_bytes('pending-merges', '', file_mode)
-
-    def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
-                   accelerator_tree=None, hardlink=False):
-        """See WorkingTreeFormat.initialize()."""
-        if not isinstance(a_bzrdir.transport, LocalTransport):
-            raise errors.NotLocalUrl(a_bzrdir.transport.base)
-        if from_branch is not None:
-            branch = from_branch
-        else:
-            branch = a_bzrdir.open_branch()
-        if revision_id is None:
-            revision_id = _mod_revision.ensure_null(branch.last_revision())
-        branch.lock_write()
-        try:
-            branch.generate_revision_history(revision_id)
-        finally:
-            branch.unlock()
-        inv = inventory.Inventory()
-        wt = WorkingTree2(a_bzrdir.root_transport.local_abspath('.'),
-                         branch,
-                         inv,
-                         _internal=True,
-                         _format=self,
-                         _bzrdir=a_bzrdir)
-        basis_tree = branch.repository.revision_tree(revision_id)
-        if basis_tree.inventory.root is not None:
-            wt.set_root_id(basis_tree.get_root_id())
-        # set the parent list and cache the basis tree.
-        if _mod_revision.is_null(revision_id):
-            parent_trees = []
-        else:
-            parent_trees = [(revision_id, basis_tree)]
-        wt.set_parent_trees(parent_trees)
-        transform.build_tree(basis_tree, wt)
-        return wt
-
-    def __init__(self):
-        super(WorkingTreeFormat2, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirFormat6()
-
-    def open(self, a_bzrdir, _found=False):
-        """Return the WorkingTree object for a_bzrdir
-
-        _found is a private parameter, do not use it. It is used to indicate
-               if format probing has already been done.
-        """
-        if not _found:
-            # we are being called directly and must probe.
-            raise NotImplementedError
-        if not isinstance(a_bzrdir.transport, LocalTransport):
-            raise errors.NotLocalUrl(a_bzrdir.transport.base)
-        wt = WorkingTree2(a_bzrdir.root_transport.local_abspath('.'),
-                           _internal=True,
-                           _format=self,
-                           _bzrdir=a_bzrdir)
-        return wt
-
-class WorkingTreeFormat3(WorkingTreeFormat):
-    """The second working tree format updated to record a format marker.
-
-    This format:
-        - exists within a metadir controlling .bzr
-        - includes an explicit version marker for the workingtree control
-          files, separate from the BzrDir format
-        - modifies the hash cache format
-        - is new in bzr 0.8
-        - uses a LockDir to guard access for writes.
-    """
-
-    upgrade_recommended = True
-
-    def get_format_string(self):
-        """See WorkingTreeFormat.get_format_string()."""
-        return "Bazaar-NG Working Tree format 3"
-
-    def get_format_description(self):
-        """See WorkingTreeFormat.get_format_description()."""
-        return "Working tree format 3"
-
-    _lock_file_name = 'lock'
-    _lock_class = LockDir
-
-    _tree_class = WorkingTree3
-
-    def __get_matchingbzrdir(self):
-        return bzrdir.BzrDirMetaFormat1()
-
-    _matchingbzrdir = property(__get_matchingbzrdir)
-
-    def _open_control_files(self, a_bzrdir):
-        transport = a_bzrdir.get_workingtree_transport(None)
-        return LockableFiles(transport, self._lock_file_name,
-                             self._lock_class)
-
-    def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
-                   accelerator_tree=None, hardlink=False):
-        """See WorkingTreeFormat.initialize().
-
-        :param revision_id: if supplied, create a working tree at a different
-            revision than the branch is at.
-        :param accelerator_tree: A tree which can be used for retrieving file
-            contents more quickly than the revision tree, i.e. a workingtree.
-            The revision tree will be used for cases where accelerator_tree's
-            content is different.
-        :param hardlink: If true, hard-link files from accelerator_tree,
-            where possible.
-        """
-        if not isinstance(a_bzrdir.transport, LocalTransport):
-            raise errors.NotLocalUrl(a_bzrdir.transport.base)
-        transport = a_bzrdir.get_workingtree_transport(self)
-        control_files = self._open_control_files(a_bzrdir)
-        control_files.create_lock()
-        control_files.lock_write()
-        transport.put_bytes('format', self.get_format_string(),
-            mode=a_bzrdir._get_file_mode())
-        if from_branch is not None:
-            branch = from_branch
-        else:
-            branch = a_bzrdir.open_branch()
-        if revision_id is None:
-            revision_id = _mod_revision.ensure_null(branch.last_revision())
-        # WorkingTree3 can handle an inventory which has a unique root id.
-        # as of bzr 0.12. However, bzr 0.11 and earlier fail to handle
-        # those trees. And because there isn't a format bump inbetween, we
-        # are maintaining compatibility with older clients.
-        # inv = Inventory(root_id=gen_root_id())
-        inv = self._initial_inventory()
-        wt = self._tree_class(a_bzrdir.root_transport.local_abspath('.'),
-                         branch,
-                         inv,
-                         _internal=True,
-                         _format=self,
-                         _bzrdir=a_bzrdir,
-                         _control_files=control_files)
-        wt.lock_tree_write()
-        try:
-            basis_tree = branch.repository.revision_tree(revision_id)
-            # only set an explicit root id if there is one to set.
-            if basis_tree.inventory.root is not None:
-                wt.set_root_id(basis_tree.get_root_id())
-            if revision_id == _mod_revision.NULL_REVISION:
-                wt.set_parent_trees([])
-            else:
-                wt.set_parent_trees([(revision_id, basis_tree)])
-            transform.build_tree(basis_tree, wt)
-        finally:
-            # Unlock in this order so that the unlock-triggers-flush in
-            # WorkingTree is given a chance to fire.
-            control_files.unlock()
-            wt.unlock()
-        return wt
-
-    def _initial_inventory(self):
-        return inventory.Inventory()
-
-    def __init__(self):
-        super(WorkingTreeFormat3, self).__init__()
-
-    def open(self, a_bzrdir, _found=False):
-        """Return the WorkingTree object for a_bzrdir
-
-        _found is a private parameter, do not use it. It is used to indicate
-               if format probing has already been done.
-        """
-        if not _found:
-            # we are being called directly and must probe.
-            raise NotImplementedError
-        if not isinstance(a_bzrdir.transport, LocalTransport):
-            raise errors.NotLocalUrl(a_bzrdir.transport.base)
-        wt = self._open(a_bzrdir, self._open_control_files(a_bzrdir))
-        return wt
-
-    def _open(self, a_bzrdir, control_files):
-        """Open the tree itself.
-
-        :param a_bzrdir: the dir for the tree.
-        :param control_files: the control files for the tree.
-        """
-        return self._tree_class(a_bzrdir.root_transport.local_abspath('.'),
-                                _internal=True,
-                                _format=self,
-                                _bzrdir=a_bzrdir,
-                                _control_files=control_files)
-
-    def __str__(self):
-        return self.get_format_string()
-
-
-__default_format = WorkingTreeFormat6()
-WorkingTreeFormat.register_format(__default_format)
-WorkingTreeFormat.register_format(WorkingTreeFormat5())
-WorkingTreeFormat.register_format(WorkingTreeFormat4())
-WorkingTreeFormat.register_format(WorkingTreeFormat3())
-WorkingTreeFormat.set_default_format(__default_format)
-# formats which have no format string are not discoverable
-# and not independently creatable, so are not registered.
-_legacy_formats = [WorkingTreeFormat2(),
-                   ]
+        format_registry.remove(format)
+
+    def get_controldir_for_branch(self):
+        """Get the control directory format for creating branches.
+
+        This is to support testing of working tree formats that can not exist
+        in the same control directory as a branch.
+        """
+        return self._matchingbzrdir
+
+
+format_registry.register_lazy("Bazaar Working Tree Format 4 (bzr 0.15)\n",
+    "bzrlib.workingtree_4", "WorkingTreeFormat4")
+format_registry.register_lazy("Bazaar Working Tree Format 5 (bzr 1.11)\n",
+    "bzrlib.workingtree_4", "WorkingTreeFormat5")
+format_registry.register_lazy("Bazaar Working Tree Format 6 (bzr 1.14)\n",
+    "bzrlib.workingtree_4", "WorkingTreeFormat6")
+format_registry.register_lazy("Bazaar-NG Working Tree format 3",
+    "bzrlib.workingtree_3", "WorkingTreeFormat3")
+format_registry.set_default_key("Bazaar Working Tree Format 6 (bzr 1.14)\n")

=== added file 'bzrlib/workingtree_3.py'
--- bzrlib/workingtree_3.py	1970-01-01 00:00:00 +0000
+++ bzrlib/workingtree_3.py	2011-10-11 12:01:51 +0000
@@ -0,0 +1,264 @@
+# Copyright (C) 2007-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""WorkingTree3 format and implementation.
+
+"""
+
+import errno
+
+from bzrlib import (
+    bzrdir,
+    errors,
+    hashcache,
+    inventory,
+    revision as _mod_revision,
+    trace,
+    transform,
+    )
+from bzrlib.decorators import (
+    needs_read_lock,
+    )
+from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockdir import LockDir
+from bzrlib.transport.local import LocalTransport
+from bzrlib.workingtree import (
+    InventoryWorkingTree,
+    WorkingTreeFormat,
+    )
+
+
+class PreDirStateWorkingTree(InventoryWorkingTree):
+
+    def __init__(self, basedir='.', *args, **kwargs):
+        super(PreDirStateWorkingTree, self).__init__(basedir, *args, **kwargs)
+        # update the whole cache up front and write to disk if anything changed;
+        # in the future we might want to do this more selectively
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
+        wt_trans = self.bzrdir.get_workingtree_transport(None)
+        cache_filename = wt_trans.local_abspath('stat-cache')
+        self._hashcache = hashcache.HashCache(basedir, cache_filename,
+            self.bzrdir._get_file_mode(),
+            self._content_filter_stack_provider())
+        hc = self._hashcache
+        hc.read()
+        # is this scan needed ? it makes things kinda slow.
+        #hc.scan()
+
+        if hc.needs_write:
+            trace.mutter("write hc")
+            hc.write()
+
+    def _write_hashcache_if_dirty(self):
+        """Write out the hashcache if it is dirty."""
+        if self._hashcache.needs_write:
+            try:
+                self._hashcache.write()
+            except OSError, e:
+                if e.errno not in (errno.EPERM, errno.EACCES):
+                    raise
+                # TODO: jam 20061219 Should this be a warning? A single line
+                #       warning might be sufficient to let the user know what
+                #       is going on.
+                trace.mutter('Could not write hashcache for %s\nError: %s',
+                              self._hashcache.cache_file_name(), e)
+
+    @needs_read_lock
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        if not path:
+            path = self._inventory.id2path(file_id)
+        return self._hashcache.get_sha1(path, stat_value)
+
+
+class WorkingTree3(PreDirStateWorkingTree):
+    """This is the Format 3 working tree.
+
+    This differs from the base WorkingTree by:
+     - having its own file lock
+     - having its own last-revision property.
+
+    This is new in bzr 0.8
+    """
+
+    @needs_read_lock
+    def _last_revision(self):
+        """See Mutable.last_revision."""
+        try:
+            return self._transport.get_bytes('last-revision')
+        except errors.NoSuchFile:
+            return _mod_revision.NULL_REVISION
+
+    def _change_last_revision(self, revision_id):
+        """See WorkingTree._change_last_revision."""
+        if revision_id is None or revision_id == _mod_revision.NULL_REVISION:
+            try:
+                self._transport.delete('last-revision')
+            except errors.NoSuchFile:
+                pass
+            return False
+        else:
+            self._transport.put_bytes('last-revision', revision_id,
+                mode=self.bzrdir._get_file_mode())
+            return True
+
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree."""
+        return [('trees', self.last_revision())]
+
+    def unlock(self):
+        if self._control_files._lock_count == 1:
+           # do non-implementation specific cleanup
+            self._cleanup()
+            # _inventory_is_modified is always False during a read lock.
+            if self._inventory_is_modified:
+                self.flush()
+            self._write_hashcache_if_dirty()
+        # reverse order of locking.
+        try:
+            return self._control_files.unlock()
+        finally:
+            self.branch.unlock()
+
+
+class WorkingTreeFormat3(WorkingTreeFormat):
+    """The second working tree format updated to record a format marker.
+
+    This format:
+        - exists within a metadir controlling .bzr
+        - includes an explicit version marker for the workingtree control
+          files, separate from the ControlDir format
+        - modifies the hash cache format
+        - is new in bzr 0.8
+        - uses a LockDir to guard access for writes.
+    """
+
+    upgrade_recommended = True
+
+    missing_parent_conflicts = True
+
+    supports_versioned_directories = True
+
+    def get_format_string(self):
+        """See WorkingTreeFormat.get_format_string()."""
+        return "Bazaar-NG Working Tree format 3"
+
+    def get_format_description(self):
+        """See WorkingTreeFormat.get_format_description()."""
+        return "Working tree format 3"
+
+    _tree_class = WorkingTree3
+
+    def __get_matchingbzrdir(self):
+        return bzrdir.BzrDirMetaFormat1()
+
+    _matchingbzrdir = property(__get_matchingbzrdir)
+
+    def _open_control_files(self, a_bzrdir):
+        transport = a_bzrdir.get_workingtree_transport(None)
+        return LockableFiles(transport, 'lock', LockDir)
+
+    def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
+                   accelerator_tree=None, hardlink=False):
+        """See WorkingTreeFormat.initialize().
+
+        :param revision_id: if supplied, create a working tree at a different
+            revision than the branch is at.
+        :param accelerator_tree: A tree which can be used for retrieving file
+            contents more quickly than the revision tree, i.e. a workingtree.
+            The revision tree will be used for cases where accelerator_tree's
+            content is different.
+        :param hardlink: If true, hard-link files from accelerator_tree,
+            where possible.
+        """
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        transport = a_bzrdir.get_workingtree_transport(self)
+        control_files = self._open_control_files(a_bzrdir)
+        control_files.create_lock()
+        control_files.lock_write()
+        transport.put_bytes('format', self.get_format_string(),
+            mode=a_bzrdir._get_file_mode())
+        if from_branch is not None:
+            branch = from_branch
+        else:
+            branch = a_bzrdir.open_branch()
+        if revision_id is None:
+            revision_id = _mod_revision.ensure_null(branch.last_revision())
+        # WorkingTree3 can handle an inventory which has a unique root id.
+        # as of bzr 0.12. However, bzr 0.11 and earlier fail to handle
+        # those trees. And because there isn't a format bump inbetween, we
+        # are maintaining compatibility with older clients.
+        # inv = Inventory(root_id=gen_root_id())
+        inv = self._initial_inventory()
+        wt = self._tree_class(a_bzrdir.root_transport.local_abspath('.'),
+                         branch,
+                         inv,
+                         _internal=True,
+                         _format=self,
+                         _bzrdir=a_bzrdir,
+                         _control_files=control_files)
+        wt.lock_tree_write()
+        try:
+            basis_tree = branch.repository.revision_tree(revision_id)
+            # only set an explicit root id if there is one to set.
+            if basis_tree.inventory.root is not None:
+                wt.set_root_id(basis_tree.get_root_id())
+            if revision_id == _mod_revision.NULL_REVISION:
+                wt.set_parent_trees([])
+            else:
+                wt.set_parent_trees([(revision_id, basis_tree)])
+            transform.build_tree(basis_tree, wt)
+        finally:
+            # Unlock in this order so that the unlock-triggers-flush in
+            # WorkingTree is given a chance to fire.
+            control_files.unlock()
+            wt.unlock()
+        return wt
+
+    def _initial_inventory(self):
+        return inventory.Inventory()
+
+    def open(self, a_bzrdir, _found=False):
+        """Return the WorkingTree object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already been done.
+        """
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        wt = self._open(a_bzrdir, self._open_control_files(a_bzrdir))
+        return wt
+
+    def _open(self, a_bzrdir, control_files):
+        """Open the tree itself.
+
+        :param a_bzrdir: the dir for the tree.
+        :param control_files: the control files for the tree.
+        """
+        return self._tree_class(a_bzrdir.root_transport.local_abspath('.'),
+                                _internal=True,
+                                _format=self,
+                                _bzrdir=a_bzrdir,
+                                _control_files=control_files)
+
+    def __str__(self):
+        return self.get_format_string()

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2010-07-21 09:58:42 +0000
+++ bzrlib/workingtree_4.py	2011-10-11 12:01:51 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2010 Canonical Ltd
+# Copyright (C) 2007-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,13 +31,15 @@
 import errno
 import stat
 
-import bzrlib
 from bzrlib import (
     bzrdir,
     cache_utf8,
+    config,
+    conflicts as _mod_conflicts,
     debug,
     dirstate,
     errors,
+    filters as _mod_filters,
     generate_ids,
     osutils,
     revision as _mod_revision,
@@ -46,14 +48,13 @@
     transform,
     views,
     )
-import bzrlib.branch
-import bzrlib.ui
 """)
 
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-from bzrlib.filters import filtered_input_file, internal_size_sha_file_byname
 from bzrlib.inventory import Inventory, ROOT_ID, entry_factory
 from bzrlib.lock import LogicalLockResult
+from bzrlib.lockable_files import LockableFiles
+from bzrlib.lockdir import LockDir
 from bzrlib.mutabletree import needs_tree_write_lock
 from bzrlib.osutils import (
     file_kind,
@@ -62,14 +63,20 @@
     realpath,
     safe_unicode,
     )
-from bzrlib.trace import mutter
 from bzrlib.transport.local import LocalTransport
-from bzrlib.tree import InterTree
-from bzrlib.tree import Tree
-from bzrlib.workingtree import WorkingTree, WorkingTree3, WorkingTreeFormat3
-
-
-class DirStateWorkingTree(WorkingTree3):
+from bzrlib.tree import (
+    InterTree,
+    InventoryTree,
+    )
+from bzrlib.workingtree import (
+    InventoryWorkingTree,
+    WorkingTree,
+    WorkingTreeFormat,
+    )
+
+
+class DirStateWorkingTree(InventoryWorkingTree):
+
     def __init__(self, basedir,
                  branch,
                  _control_files=None,
@@ -85,7 +92,7 @@
         self._format = _format
         self.bzrdir = _bzrdir
         basedir = safe_unicode(basedir)
-        mutter("opening working tree %r", basedir)
+        trace.mutter("opening working tree %r", basedir)
         self._branch = branch
         self.basedir = realpath(basedir)
         # if branch is at our basedir and is a format 6 or less
@@ -125,6 +132,10 @@
             state.add(f, file_id, kind, None, '')
         self._make_dirty(reset_inventory=True)
 
+    def _get_check_refs(self):
+        """Return the references needed to perform a check of this tree."""
+        return [('trees', self.last_revision())]
+
     def _make_dirty(self, reset_inventory):
         """Make the tree state dirty.
 
@@ -182,7 +193,7 @@
 
     def _comparison_data(self, entry, path):
         kind, executable, stat_value = \
-            WorkingTree3._comparison_data(self, entry, path)
+            WorkingTree._comparison_data(self, entry, path)
         # it looks like a plain directory, but it's really a reference -- see
         # also kind()
         if (self._repo_supports_tree_reference and kind == 'directory'
@@ -194,7 +205,7 @@
     def commit(self, message=None, revprops=None, *args, **kwargs):
         # mark the tree as dirty post commit - commit
         # can change the current versioned list by doing deletes.
-        result = WorkingTree3.commit(self, message, revprops, *args, **kwargs)
+        result = WorkingTree.commit(self, message, revprops, *args, **kwargs)
         self._make_dirty(reset_inventory=True)
         return result
 
@@ -219,7 +230,7 @@
         local_path = self.bzrdir.get_workingtree_transport(None
             ).local_abspath('dirstate')
         self._dirstate = dirstate.DirState.on_file(local_path,
-            self._sha1_provider())
+            self._sha1_provider(), self._worth_saving_limit())
         return self._dirstate
 
     def _sha1_provider(self):
@@ -234,6 +245,15 @@
         else:
             return None
 
+    def _worth_saving_limit(self):
+        """How many hash changes are ok before we must save the dirstate.
+
+        :return: an integer. -1 means never save.
+        """
+        # FIXME: We want a WorkingTreeStack here -- vila 20110812
+        conf = config.BranchStack(self.branch)
+        return conf.get('bzr.workingtree.worth_saving_limit')
+
     def filter_unversioned_files(self, paths):
         """Filter out paths that are versioned.
 
@@ -369,7 +389,7 @@
         state = self.current_dirstate()
         if stat_value is None:
             try:
-                stat_value = os.lstat(file_abspath)
+                stat_value = osutils.lstat(file_abspath)
             except OSError, e:
                 if e.errno == errno.ENOENT:
                     return None
@@ -478,7 +498,7 @@
             self._must_be_locked()
             if not path:
                 path = self.id2path(file_id)
-            mode = os.lstat(self.abspath(path)).st_mode
+            mode = osutils.lstat(self.abspath(path)).st_mode
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
     def all_file_ids(self):
@@ -850,7 +870,7 @@
                 rollback_rename()
                 raise
             result.append((from_rel, to_rel))
-            state._dirblock_state = dirstate.DirState.IN_MEMORY_MODIFIED
+            state._mark_modified()
             self._make_dirty(reset_inventory=False)
 
         return result
@@ -1113,7 +1133,29 @@
                         _mod_revision.NULL_REVISION)))
                 ghosts.append(rev_id)
             accepted_revisions.add(rev_id)
-        dirstate.set_parent_trees(real_trees, ghosts=ghosts)
+        updated = False
+        if (len(real_trees) == 1
+            and not ghosts
+            and self.branch.repository._format.fast_deltas
+            and isinstance(real_trees[0][1],
+                revisiontree.InventoryRevisionTree)
+            and self.get_parent_ids()):
+            rev_id, rev_tree = real_trees[0]
+            basis_id = self.get_parent_ids()[0]
+            # There are times when basis_tree won't be in
+            # self.branch.repository, (switch, for example)
+            try:
+                basis_tree = self.branch.repository.revision_tree(basis_id)
+            except errors.NoSuchRevision:
+                # Fall back to the set_parent_trees(), since we can't use
+                # _make_delta if we can't get the RevisionTree
+                pass
+            else:
+                delta = rev_tree.inventory._make_delta(basis_tree.inventory)
+                dirstate.update_basis_by_delta(delta, rev_id)
+                updated = True
+        if not updated:
+            dirstate.set_parent_trees(real_trees, ghosts=ghosts)
         self._make_dirty(reset_inventory=False)
 
     def _set_root_id(self, file_id):
@@ -1139,10 +1181,10 @@
 
     def unlock(self):
         """Unlock in format 4 trees needs to write the entire dirstate."""
-        # do non-implementation specific cleanup
-        self._cleanup()
-
         if self._control_files._lock_count == 1:
+            # do non-implementation specific cleanup
+            self._cleanup()
+
             # eventually we should do signature checking during read locks for
             # dirstate updates.
             if self._control_files._lock_mode == 'w':
@@ -1254,7 +1296,7 @@
     def rename_one(self, from_rel, to_rel, after=False):
         """See WorkingTree.rename_one"""
         self.flush()
-        WorkingTree.rename_one(self, from_rel, to_rel, after)
+        super(DirStateWorkingTree, self).rename_one(from_rel, to_rel, after)
 
     @needs_tree_write_lock
     def apply_inventory_delta(self, changes):
@@ -1293,6 +1335,27 @@
             self._inventory = inv
         self.flush()
 
+    @needs_tree_write_lock
+    def reset_state(self, revision_ids=None):
+        """Reset the state of the working tree.
+
+        This does a hard-reset to a last-known-good state. This is a way to
+        fix if something got corrupted (like the .bzr/checkout/dirstate file)
+        """
+        if revision_ids is None:
+            revision_ids = self.get_parent_ids()
+        if not revision_ids:
+            base_tree = self.branch.repository.revision_tree(
+                _mod_revision.NULL_REVISION)
+            trees = []
+        else:
+            trees = zip(revision_ids,
+                        self.branch.repository.revision_trees(revision_ids))
+            base_tree = trees[0][1]
+        state = self.current_dirstate()
+        # We don't support ghosts yet
+        state.set_state_from_scratch(base_tree.inventory, trees, [])
+
 
 class ContentFilterAwareSHA1Provider(dirstate.SHA1Provider):
 
@@ -1303,7 +1366,7 @@
         """See dirstate.SHA1Provider.sha1()."""
         filters = self.tree._content_filter_stack(
             self.tree.relpath(osutils.safe_unicode(abspath)))
-        return internal_size_sha_file_byname(abspath, filters)[1]
+        return _mod_filters.internal_size_sha_file_byname(abspath, filters)[1]
 
     def stat_and_sha1(self, abspath):
         """See dirstate.SHA1Provider.stat_and_sha1()."""
@@ -1313,7 +1376,7 @@
         try:
             statvalue = os.fstat(file_obj.fileno())
             if filters:
-                file_obj = filtered_input_file(file_obj, filters)
+                file_obj = _mod_filters.filtered_input_file(file_obj, filters)
             sha1 = osutils.size_sha_file(file_obj)[1]
         finally:
             file_obj.close()
@@ -1349,7 +1412,7 @@
 class WorkingTree4(DirStateWorkingTree):
     """This is the Format 4 working tree.
 
-    This differs from WorkingTree3 by:
+    This differs from WorkingTree by:
      - Having a consolidated internal dirstate, stored in a
        randomly-accessible sorted file on disk.
      - Not having a regular inventory attribute.  One can be synthesized
@@ -1383,14 +1446,26 @@
         return views.PathBasedViews(self)
 
 
-class DirStateWorkingTreeFormat(WorkingTreeFormat3):
+class DirStateWorkingTreeFormat(WorkingTreeFormat):
+
+    missing_parent_conflicts = True
+
+    supports_versioned_directories = True
+
+    _lock_class = LockDir
+    _lock_file_name = 'lock'
+
+    def _open_control_files(self, a_bzrdir):
+        transport = a_bzrdir.get_workingtree_transport(None)
+        return LockableFiles(transport, self._lock_file_name,
+                             self._lock_class)
 
     def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
                    accelerator_tree=None, hardlink=False):
         """See WorkingTreeFormat.initialize().
 
         :param revision_id: allows creating a working tree at a different
-        revision than the branch is at.
+            revision than the branch is at.
         :param accelerator_tree: A tree which can be used for retrieving file
             contents more quickly than the revision tree, i.e. a workingtree.
             The revision tree will be used for cases where accelerator_tree's
@@ -1489,6 +1564,20 @@
         :param wt: the WorkingTree object
         """
 
+    def open(self, a_bzrdir, _found=False):
+        """Return the WorkingTree object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already been done.
+        """
+        if not _found:
+            # we are being called directly and must probe.
+            raise NotImplementedError
+        if not isinstance(a_bzrdir.transport, LocalTransport):
+            raise errors.NotLocalUrl(a_bzrdir.transport.base)
+        wt = self._open(a_bzrdir, self._open_control_files(a_bzrdir))
+        return wt
+
     def _open(self, a_bzrdir, control_files):
         """Open the tree itself.
 
@@ -1519,7 +1608,7 @@
     This format:
         - exists within a metadir controlling .bzr
         - includes an explicit version marker for the workingtree control
-          files, separate from the BzrDir format
+          files, separate from the ControlDir format
         - modifies the hash cache format
         - is new in bzr 0.15
         - uses a LockDir to guard access to it.
@@ -1585,7 +1674,7 @@
         return True
 
 
-class DirStateRevisionTree(Tree):
+class DirStateRevisionTree(InventoryTree):
     """A revision tree pulling the inventory from a dirstate.
     
     Note that this is one of the historical (ie revision) trees cached in the
@@ -1610,13 +1699,10 @@
     def annotate_iter(self, file_id,
                       default_revision=_mod_revision.CURRENT_REVISION):
         """See Tree.annotate_iter"""
-        text_key = (file_id, self.inventory[file_id].revision)
+        text_key = (file_id, self.get_file_revision(file_id))
         annotations = self._repository.texts.annotate(text_key)
         return [(key[-1], line) for (key, line) in annotations]
 
-    def _get_ancestors(self, default_revision):
-        return set(self._repository.get_ancestry(self._revision_id,
-                                                 topo_sorted=False))
     def _comparison_data(self, entry, path):
         """See Tree._comparison_data."""
         if entry is None:
@@ -1738,8 +1824,6 @@
                 elif kind == 'directory':
                     parent_ies[(dirname + '/' + name).strip('/')] = inv_entry
                 elif kind == 'symlink':
-                    inv_entry.executable = False
-                    inv_entry.text_size = None
                     inv_entry.symlink_target = utf8_decode(fingerprint)[0]
                 elif kind == 'tree-reference':
                     inv_entry.reference_revision = fingerprint or None
@@ -1765,7 +1849,7 @@
         # Make sure the file exists
         entry = self._get_entry(file_id, path=path)
         if entry == (None, None): # do we raise?
-            return None
+            raise errors.NoSuchId(self, file_id)
         parent_index = self._get_parent_index()
         last_changed_revision = entry[1][parent_index][4]
         try:
@@ -1782,6 +1866,10 @@
             return parent_details[1]
         return None
 
+    @needs_read_lock
+    def get_file_revision(self, file_id):
+        return self.inventory[file_id].revision
+
     def get_file(self, file_id, path=None):
         return StringIO(self.get_file_text(file_id))
 
@@ -1810,7 +1898,7 @@
                                        identifier))
         return self._repository.iter_files_bytes(repo_desired_files)
 
-    def get_symlink_target(self, file_id):
+    def get_symlink_target(self, file_id, path=None):
         entry = self._get_entry(file_id=file_id)
         parent_index = self._get_parent_index()
         if entry[1][parent_index][0] != 'l':
@@ -1869,7 +1957,7 @@
     def is_executable(self, file_id, path=None):
         ie = self.inventory[file_id]
         if ie.kind != "file":
-            return None
+            return False
         return ie.executable
 
     def is_locked(self):
@@ -1988,7 +2076,7 @@
     def make_source_parent_tree(source, target):
         """Change the source tree into a parent of the target."""
         revid = source.commit('record tree')
-        target.branch.repository.fetch(source.branch.repository, revid)
+        target.branch.fetch(source.branch, revid)
         target.set_parent_ids([revid])
         return target.basis_tree(), target
 

=== modified file 'bzrlib/xml8.py'
--- bzrlib/xml8.py	2010-04-30 11:03:59 +0000
+++ bzrlib/xml8.py	2011-02-21 15:09:19 +0000
@@ -21,6 +21,7 @@
     cache_utf8,
     errors,
     inventory,
+    lazy_regex,
     revision as _mod_revision,
     trace,
     )
@@ -45,6 +46,35 @@
     ">":"&gt;",
     }
 
+_xml_unescape_map = {
+    'apos':"'",
+    'quot':'"',
+    'amp':'&',
+    'lt':'<',
+    'gt':'>'
+}
+
+
+def _unescaper(match, _map=_xml_unescape_map):
+    code = match.group(1)
+    try:
+        return _map[code]
+    except KeyError:
+        if not code.startswith('#'):
+            raise
+        return unichr(int(code[1:])).encode('utf8')
+
+
+_unescape_re = None
+
+
+def _unescape_xml(data):
+    """Unescape predefined XML entities in a string of data."""
+    global _unescape_re
+    if _unescape_re is None:
+        _unescape_re = re.compile('\&([^;]*);')
+    return _unescape_re.sub(_unescaper, data)
+
 
 def _ensure_utf8_re():
     """Make sure the _utf8_re and _unicode_re regexes have been compiled."""
@@ -161,6 +191,13 @@
     format_num = '8'
     revision_format_num = None
 
+    # The search regex used by xml based repositories to determine what things
+    # where changed in a single commit.
+    _file_ids_altered_regex = lazy_regex.lazy_compile(
+        r'file_id="(?P<file_id>[^"]+)"'
+        r'.* revision="(?P<revision_id>[^"]+)"'
+        )
+
     def _check_revisions(self, inv):
         """Extension point for subclasses to check during serialisation.
 
@@ -532,5 +569,84 @@
                 raise AssertionError("repeated property %r" % name)
             rev.properties[name] = value
 
+    def _find_text_key_references(self, line_iterator):
+        """Core routine for extracting references to texts from inventories.
+
+        This performs the translation of xml lines to revision ids.
+
+        :param line_iterator: An iterator of lines, origin_version_id
+        :return: A dictionary mapping text keys ((fileid, revision_id) tuples)
+            to whether they were referred to by the inventory of the
+            revision_id that they contain. Note that if that revision_id was
+            not part of the line_iterator's output then False will be given -
+            even though it may actually refer to that key.
+        """
+        if not self.support_altered_by_hack:
+            raise AssertionError(
+                "_find_text_key_references only "
+                "supported for branches which store inventory as unnested xml"
+                ", not on %r" % self)
+        result = {}
+
+        # this code needs to read every new line in every inventory for the
+        # inventories [revision_ids]. Seeing a line twice is ok. Seeing a line
+        # not present in one of those inventories is unnecessary but not
+        # harmful because we are filtering by the revision id marker in the
+        # inventory lines : we only select file ids altered in one of those
+        # revisions. We don't need to see all lines in the inventory because
+        # only those added in an inventory in rev X can contain a revision=X
+        # line.
+        unescape_revid_cache = {}
+        unescape_fileid_cache = {}
+
+        # jam 20061218 In a big fetch, this handles hundreds of thousands
+        # of lines, so it has had a lot of inlining and optimizing done.
+        # Sorry that it is a little bit messy.
+        # Move several functions to be local variables, since this is a long
+        # running loop.
+        search = self._file_ids_altered_regex.search
+        unescape = _unescape_xml
+        setdefault = result.setdefault
+        for line, line_key in line_iterator:
+            match = search(line)
+            if match is None:
+                continue
+            # One call to match.group() returning multiple items is quite a
+            # bit faster than 2 calls to match.group() each returning 1
+            file_id, revision_id = match.group('file_id', 'revision_id')
+
+            # Inlining the cache lookups helps a lot when you make 170,000
+            # lines and 350k ids, versus 8.4 unique ids.
+            # Using a cache helps in 2 ways:
+            #   1) Avoids unnecessary decoding calls
+            #   2) Re-uses cached strings, which helps in future set and
+            #      equality checks.
+            # (2) is enough that removing encoding entirely along with
+            # the cache (so we are using plain strings) results in no
+            # performance improvement.
+            try:
+                revision_id = unescape_revid_cache[revision_id]
+            except KeyError:
+                unescaped = unescape(revision_id)
+                unescape_revid_cache[revision_id] = unescaped
+                revision_id = unescaped
+
+            # Note that unconditionally unescaping means that we deserialise
+            # every fileid, which for general 'pull' is not great, but we don't
+            # really want to have some many fulltexts that this matters anyway.
+            # RBC 20071114.
+            try:
+                file_id = unescape_fileid_cache[file_id]
+            except KeyError:
+                unescaped = unescape(file_id)
+                unescape_fileid_cache[file_id] = unescaped
+                file_id = unescaped
+
+            key = (file_id, revision_id)
+            setdefault(key, False)
+            if revision_id == line_key[-1]:
+                result[key] = True
+        return result
+
 
 serializer_v8 = Serializer_v8()

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2010-05-25 17:27:52 +0000
+++ bzrlib/xml_serializer.py	2010-09-17 04:35:23 +0000
@@ -22,6 +22,8 @@
 # importing this module is fairly slow because it has to load several
 # ElementTree bits
 
+import re
+
 from bzrlib.serializer import Serializer
 from bzrlib.trace import mutter
 
@@ -31,6 +33,8 @@
         from xml.etree.cElementTree import (ElementTree, SubElement, Element,
             XMLTreeBuilder, fromstring, tostring)
         import xml.etree as elementtree
+        # Also import ElementTree module so monkey-patching below always works
+        import xml.etree.ElementTree
     except ImportError:
         from cElementTree import (ElementTree, SubElement, Element,
                                   XMLTreeBuilder, fromstring, tostring)
@@ -111,73 +115,6 @@
         return ElementTree().parse(f)
 
 
-# performance tuning for elementree's serialiser. This should be
-# sent upstream - RBC 20060523.
-# the functions here are patched into elementtree at runtime.
-import re
-escape_re = re.compile("[&'\"<>]")
-escape_map = {
-    "&":'&amp;',
-    "'":"&apos;", # FIXME: overkill
-    "\"":"&quot;",
-    "<":"&lt;",
-    ">":"&gt;",
-    }
-def _escape_replace(match, map=escape_map):
-    return map[match.group()]
-
-def _escape_attrib(text, encoding=None, replace=None):
-    # escape attribute value
-    try:
-        if encoding:
-            try:
-                text = elementtree.ElementTree._encode(text, encoding)
-            except UnicodeError:
-                return elementtree.ElementTree._encode_entity(text)
-        if replace is None:
-            return escape_re.sub(_escape_replace, text)
-        else:
-            text = replace(text, "&", "&amp;")
-            text = replace(text, "'", "&apos;") # FIXME: overkill
-            text = replace(text, "\"", "&quot;")
-            text = replace(text, "<", "&lt;")
-            text = replace(text, ">", "&gt;")
-            return text
-    except (TypeError, AttributeError):
-        elementtree.ElementTree._raise_serialization_error(text)
-
-elementtree.ElementTree._escape_attrib = _escape_attrib
-
-escape_cdata_re = re.compile("[&<>]")
-escape_cdata_map = {
-    "&":'&amp;',
-    "<":"&lt;",
-    ">":"&gt;",
-    }
-def _escape_cdata_replace(match, map=escape_cdata_map):
-    return map[match.group()]
-
-def _escape_cdata(text, encoding=None, replace=None):
-    # escape character data
-    try:
-        if encoding:
-            try:
-                text = elementtree.ElementTree._encode(text, encoding)
-            except UnicodeError:
-                return elementtree.ElementTree._encode_entity(text)
-        if replace is None:
-            return escape_cdata_re.sub(_escape_cdata_replace, text)
-        else:
-            text = replace(text, "&", "&amp;")
-            text = replace(text, "<", "&lt;")
-            text = replace(text, ">", "&gt;")
-            return text
-    except (TypeError, AttributeError):
-        elementtree.ElementTree._raise_serialization_error(text)
-
-elementtree.ElementTree._escape_cdata = _escape_cdata
-
-
 def escape_invalid_chars(message):
     """Escape the XML-invalid characters in a commit message.
 

=== removed file 'contrib/add-bzr-to-baz'
--- contrib/add-bzr-to-baz	2005-09-19 06:05:19 +0000
+++ contrib/add-bzr-to-baz	1970-01-01 00:00:00 +0000
@@ -1,16 +0,0 @@
-#! /bin/sh -e
-
-# Take a file that is versioned by bzr and
-# add it to baz with the same file-id.
-
-if [ $# -lt 1 ]
-then
-    echo "usage: $0 FILE" >&2
-    exit 1
-fi
-
-for f
-do
-    baz add -i "$( bzr file-id "$f" )" "$f"
-done
-

=== removed file 'contrib/newinventory.py'
--- contrib/newinventory.py	2009-03-23 14:59:43 +0000
+++ contrib/newinventory.py	1970-01-01 00:00:00 +0000
@@ -1,144 +0,0 @@
-# (C) 2005 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-from bzrlib.xml import ElementTree, Element
-
-
-def write_inventory(inv, f):
-    el = Element('inventory', {'version': '2'})
-    el.text = '\n'
-
-    root = Element('root_directory', {'id': inv.root.file_id})
-    root.tail = root.text = '\n'
-    el.append(root)
-
-    def descend(parent_el, ie):
-        kind = ie.kind
-        el = Element(kind, {'name': ie.name,
-                            'id': ie.file_id,})
-        
-        if kind == 'file':
-            if ie.text_id:
-                el.set('text_id', ie.text_id)
-            if ie.text_sha1:
-                el.set('text_sha1', ie.text_sha1)
-            if ie.text_size is not None:
-                el.set('text_size', ('%d' % ie.text_size))
-        elif kind != 'directory':
-            raise BzrError('unknown InventoryEntry kind %r' % kind)
-
-        el.tail = '\n'
-        parent_el.append(el)
-
-        if kind == 'directory':
-            el.text = '\n' # break before having children
-            l = ie.children.items()
-            l.sort()
-            for child_name, child_ie in l:
-                descend(el, child_ie)
-                
-        
-    # walk down through inventory, adding all directories
-
-    l = inv.root.children.items()
-    l.sort()
-    for entry_name, ie in l:
-        descend(root, ie)
-    
-    ElementTree(el).write(f, 'utf-8')
-    f.write('\n')
-
-
-
-def escape_attr(text):
-    return text.replace("&", "&amp;") \
-           .replace("'", "&apos;") \
-           .replace('"', "&quot;") \
-           .replace("<", "&lt;") \
-           .replace(">", "&gt;")
-
-
-# This writes out an inventory without building an XML tree first,
-# just to see if it's faster.  Not currently used.
-def write_slacker_inventory(inv, f):
-    def descend(ie):
-        kind = ie.kind
-        f.write('<%s name="%s" id="%s" ' % (kind, escape_attr(ie.name),
-                                            escape_attr(ie.file_id)))
-
-        if kind == 'file':
-            if ie.text_id:
-                f.write('text_id="%s" ' % ie.text_id)
-            if ie.text_sha1:
-                f.write('text_sha1="%s" ' % ie.text_sha1)
-            if ie.text_size is not None:
-                f.write('text_size="%d" ' % ie.text_size)
-            f.write('/>\n')
-        elif kind == 'directory':
-            f.write('>\n')
-            
-            l = ie.children.items()
-            l.sort()
-            for child_name, child_ie in l:
-                descend(child_ie)
-
-            f.write('</directory>\n')
-        else:
-            raise BzrError('unknown InventoryEntry kind %r' % kind)
-
-    f.write('<inventory>\n')
-    f.write('<root_directory id="%s">\n' % escape_attr(inv.root.file_id))
-
-    l = inv.root.children.items()
-    l.sort()
-    for entry_name, ie in l:
-        descend(ie)
-
-    f.write('</root_directory>\n')
-    f.write('</inventory>\n')
-    
-
-
-def read_new_inventory(f):
-    from inventory import Inventory, InventoryEntry
-    
-    def descend(parent_ie, el):
-        kind = el.tag
-        name = el.get('name')
-        file_id = el.get('id')
-        ie = InventoryEntry(file_id, name, el.tag)
-        parent_ie.children[name] = ie
-        inv._byid[file_id] = ie
-        if kind == 'directory':
-            for child_el in el:
-                descend(ie, child_el)
-        elif kind == 'file':
-            assert len(el) == 0
-            ie.text_id = el.get('text_id')
-            v = el.get('text_size')
-            ie.text_size = v and int(v)
-            ie.text_sha1 = el.get('text_sha1')
-        else:
-            raise BzrError("unknown inventory entry %r" % kind)
-
-    inv_el = ElementTree().parse(f)
-    assert inv_el.tag == 'inventory'
-    root_el = inv_el[0]
-    assert root_el.tag == 'root_directory'
-
-    inv = Inventory(inv_el.get('file_id'))
-    for el in root_el:
-        descend(inv.root, el)

=== removed file 'contrib/pwclient.full'
--- contrib/pwclient.full	2009-03-23 14:59:43 +0000
+++ contrib/pwclient.full	1970-01-01 00:00:00 +0000
@@ -1,643 +0,0 @@
-#!/usr/bin/perl -w
-#
-# Patchwork - automated patch tracking system
-# Copyright (C) 2005 Jeremy Kerr <jk@ozlabs.org>
-#
-# This file is part of the Patchwork package.
-#
-# Patchwork is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# Patchwork is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Patchwork; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-use strict;
-use lib '../lib';
-
-use SOAP::Lite;
-use Getopt::Std;
-
-my $uri = 'urn:SOAPInterface';
-# this URI has the address of the soap.pl script, followed by the project name
-my $proxy = 'http://patchwork.ozlabs.org/soap.pl/bazaar-ng';
-my $soap;
-my ($rows, $cols);
-
-my %actions = (
-	list   => 'List all patches (restrict to a state with -s <state>)',
-	view   => 'View a patch',
-	get    => 'Download a patch and save it locally',
-	apply  => 'Apply a patch (in the current dir, using -p1)',
-	search => 'Search for patches (by name)'
-);
-
-sub page($@)
-{
-	my $str = shift;
-	my $lines;
-	if (@_) {
-		($lines) = @_;
-	} else {
-		my @l = split(/\n/, $str);
-		$lines = $#l;
-	}
-	if ($rows && $lines >= $rows) {
-		my $pager = $ENV{PAGER} || 'more';
-		open(FH, "|-", $pager) || die "Couldn't run pager '$pager': $!";
-		print FH $str;
-		close(FH);
-	} else {
-		print $str;
-	}
-}
-
-sub patch_list(@)
-{
-	my @patches = @_;
-	my $states = 
-	return "No patches\n" unless @patches;
-	my $str = list_header();
-	my $max = $cols - 9;
-	$max = 10 if $max < 10;
-	foreach my $patch (@patches) {
-		my $name = $patch->name();
-		if ($cols && length($name) > $max) {
-			$name = substr($name, 0, $max - 1).'$';
-		}
-		$str .= sprintf "%4d %3s %s\n", $patch->id(),
-				substr(states($patch->state()), 0, 3),
-				$name;
-	}
-	return $str;
-}
-
-sub _get_patch($)
-{
-	my ($id) = @_;
-	unless ($id) {
-		print STDERR "No id given to retrieve a patch\n";
-		exit 1;
-	}
-
-	unless ($id =~ m/^[0-9]+$/) {
-		print STDERR "Invalid patch id '$id'\n'";
-		exit 1;
-	}
-
-	my $res = $soap->get_patch($id);
-	die "SOAP fault: ".$res->faultstring if $res->fault;
-	my $patch = $res->result;
-	unless ($patch) {
-		print STDERR "Patch not found\n";
-		exit 1;
-	}
-	return $patch;
-}
-
-sub list()
-{
-	my %opts;
-	my $res;
-	getopts('s:', \%opts);
-	if ($opts{s}) {
-		$res = $soap->get_patches_by_state(state_from_name($opts{s}));
-	} else {
-		$res = $soap->get_patches();
-	}
-	die "SOAP fault: ".$res->faultstring if $res->fault;
-	my $patches = $res->result;
-	page(patch_list(@$patches), $#{$patches} + 2);
-	return 1;
-}
-
-sub search()
-{
-	my $query = join(' ', map { '"'.$_.'"' } @ARGV);
-	my $res = $soap->search($query);
-	die "SOAP fault: ".$res->faultstring if $res->fault;
-	my $patches = $res->result;
-	my $str = '';
-	unless ($patches && @{$patches}) {
-		print "No patches found\n";
-		return 1;
-	}
-	
-	$str .= list_header();
-	page(patch_list(@$patches), $#{$patches});
-	return 1;
-}
-
-sub view()
-{
-	my ($id) = @ARGV;
-	my $patch = _get_patch($id);
-	page($patch->content());
-	return 1;
-}
-
-sub get()
-{
-	my ($id) = @ARGV;
-	my $patch = _get_patch($id);
-	if (-e $patch->filename()) {
-		printf STDERR "Patch file:\n\t%s\nalready exists\n",
-			$patch->filename();
-		exit 1;
-	}
-	open(FH, ">", $patch->filename())
-		or die "Couldn't open ".$patch->filename()." for writing: $!";
-	print FH $patch->content;
-	close(FH);
-	printf "Saved '%s'\n\tto: %s\n", $patch->name, $patch->filename();
-	return 1;
-}
-
-sub apply()
-{
-	my ($id) = @ARGV;
-	my $patch = _get_patch($id);
-	open(FH, "|-", "patch", "-p1")
-		or die "Couldn't execute 'patch -p1'";
-	print FH $patch->content;
-	close(FH);
-	return 1;
-}
-
-sub usage()
-{
-	printf STDERR "Usage: %s <action> [options]\n", $0;
-	printf STDERR "Where <action> is one of:\n";
-	printf STDERR "\t%-6s : %s\n", $_, $actions{$_} for sort keys %actions;
-}
-
-sub list_header()
-{
-	return sprintf "%4s %3s %s\n", 'ID', 'Sta', 'Name';
-}
-
-my %_states;
-sub states(@)
-{
-	my $state = @_ ? shift : undef;
-	unless (%_states) {
-		my $res = $soap->get_states();
-		die "SOAP fault: ".$res->faultstring if $res->fault;
-		my $stateref = $res->result;
-		%_states = %$stateref;
-	}
-	return $state ? $_states{$state} : %_states;
-}
-
-sub state_from_name($)
-{
-	my ($name) = @_;
-	my @matches;
-	my %states = states();
-	foreach my $id (keys(%states)) {
-		push(@matches, $id) if ($states{$id} =~ m/^$name/i);
-	}
-	if ($#matches < 0) {
-		print STDERR "No such state '$name'\n";
-		exit 1;
-	} elsif ($#matches > 0) {
-		printf STDERR "Multiple states match '$name':\n";
-		printf STDERR "\t%s\n", $states{$_} for @matches;
-		exit 1;
-	}
-	return $matches[0];
-}
-
-my $action = shift;
-unless ($action) {
-	usage();
-	exit 1;
-}
-
-if (eval "require Term::Size") {
-	($cols, $rows) = Term::Size::chars(*STDOUT);
-} else {
-	($cols, $rows) = (0,0);
-}
-
-$soap = new SOAP::Lite(uri => $uri, proxy => $proxy);
-
-foreach (sort(keys(%actions))) {
-	if ($_ eq $action) {
-		eval "return &$action()" or die $@;
-		exit 0;
-	}
-}
-printf STDERR "No such action '%s'\n", $action;
-usage();
-exit 1;
-
-# Patchwork - automated patch tracking system
-# Copyright (C) 2005 Jeremy Kerr <jk@ozlabs.org>
-#
-# This file is part of the Patchwork package.
-#
-# Patchwork is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# Patchwork is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Patchwork; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-package PatchWork::Comment;
-
-use strict;
-
-# internal variables
-#  id
-#  msgid
-#  submitter
-#  content
-#  date
-#  @refs
-
-sub new($)
-{
-	my ($cls) = @_;
-	my $obj = {};
-	bless($obj, $cls);
-	return $obj;
-}
-
-sub id(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{id} = shift }
-	return $obj->{id};
-}
-
-sub submitter(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{submitter} = shift }
-	return $obj->{submitter};
-}
-
-sub msgid(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{msgid} = shift }
-	return $obj->{msgid};
-}
-
-sub date(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{date} = shift }
-	return $obj->{date};
-}
-
-sub content(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{content} = shift }
-	return $obj->{content};
-}
-
-sub refs(@)
-{
-	my ($obj) = shift;
-	push(@{$obj->{refs}}, @_) if @_;
-	return $obj->{refs};
-}
-
-1;
-
-# Patchwork - automated patch tracking system
-# Copyright (C) 2005 Jeremy Kerr <jk@ozlabs.org>
-#
-# This file is part of the Patchwork package.
-#
-# Patchwork is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# Patchwork is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Patchwork; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-package PatchWork::Person;
-
-use strict;
-
-# internal variables
-#   email
-#   name
-
-sub new(@)
-{
-	my $cls = shift;
-	my $obj = {};
-	bless($obj, $cls);
-	$obj->{email} = shift;
-	$obj->{name} = shift;
-	return $obj;
-}
-
-sub parse_from($$)
-{
-	my ($obj, $str) = @_;
-
-	if ($str =~ m/"?(.*?)"?\s*<([^>]+)>/) {
-		$obj->{email} = $2;
-		$obj->{name} = $1;
-	
-	} elsif ($str =~ m/"?(.*?)"?\s*\(([^\)]+)\)/) {
-		$obj->{email} = $1;
-		$obj->{name} = $2;
-	
-	} else {
-		$obj->{email} = $str;
-	}
-}
-
-sub id(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{id} = shift }
-	return $obj->{id};
-}
-
-sub email(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{email} = shift }
-	return $obj->{email};
-}
-
-sub name(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{name} = shift }
-	return $obj->{name};
-}
-
-sub username(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{username} = shift }
-	return $obj->{username};
-}
-
-1;
-
-# Patchwork - automated patch tracking system
-# Copyright (C) 2005 Jeremy Kerr <jk@ozlabs.org>
-#
-# This file is part of the Patchwork package.
-#
-# Patchwork is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# Patchwork is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Patchwork; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-package PatchWork::Patch;
-
-use strict;
-
-# internal variables
-#  id
-#  msgid
-#  date
-#  name
-#  content
-#  filename
-#  submitter
-#  comments
-#  @trees
-
-sub new($)
-{
-	my ($cls) = @_;
-	my $obj = {};
-	bless($obj, $cls);
-	$obj->{comments} = [];
-	$obj->{trees} = {};
-	$obj->{archived} = 0;
-	$obj->{state} = 1;
-	return $obj;
-}
-
-sub id(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{id} = shift }
-	return $obj->{id};
-}
-
-sub msgid(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{msgid} = shift }
-	return $obj->{msgid};
-}
-
-sub date(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{date} = shift }
-	return $obj->{date};
-}
-
-sub state(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{state} = shift }
-	return $obj->{state};
-}
-
-sub name(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{name} = shift }
-	return $obj->{name};
-}
-
-sub filename(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{filename} = shift }
-	return $obj->{filename};
-}
-
-sub submitter(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{submitter} = shift }
-	return $obj->{submitter};
-}
-
-sub content(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{content} = shift }
-	return $obj->{content};
-}
-
-sub archived(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{archived} = shift }
-	return $obj->{archived};
-}
-
-sub add_comment($$)
-{
-	my ($obj, $comment) = @_;
-	push(@{$obj->{comments}}, $comment);
-}
-
-sub comments($)
-{
-	my ($obj) = @_;
-	return $obj->{comments};
-}
-
-sub trees(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{trees} = shift }
-	return $obj->{trees};
-}
-
-1;
-
-# Patchwork - automated patch tracking system
-# Copyright (C) 2005 Jeremy Kerr <jk@ozlabs.org>
-#
-# This file is part of the Patchwork package.
-#
-# Patchwork is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# Patchwork is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Patchwork; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-package PatchWork::Tree;
-
-use strict;
-
-# internal variables
-#   id
-#   name
-#   url
-
-sub new($$)
-{
-	my ($cls, $id) = @_;
-	my $obj = {};
-	bless($obj, $cls);
-	$obj->{id} = $id;
-	return $obj;
-}
-
-sub id($)
-{
-	my ($obj) = @_;
-	return $obj->{id};
-
-}
-
-sub name(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{name} = shift }
-	return $obj->{name};
-}
-
-sub url(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{url} = shift }
-	return $obj->{url};
-}
-
-1;
-
-# Patchwork - automated patch tracking system
-# Copyright (C) 2005 Jeremy Kerr <jk@ozlabs.org>
-#
-# This file is part of the Patchwork package.
-#
-# Patchwork is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# Patchwork is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Patchwork; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-package PatchWork::User;
-@PatchWork::User::ISA = ('PatchWork::Person');
-
-use strict;
-
-# internal variables
-#   username
-
-sub new($$)
-{
-	my ($cls, $id) = @_;
-	my $obj = {};
-	bless($obj, $cls);
-	$obj->{id} = $id;
-	return $obj;
-}
-
-sub username(@)
-{
-	my ($obj) = shift;
-	if (@_) { $obj->{username} = shift }
-	return $obj->{username};
-}
-
-1;
-

=== removed file 'contrib/pwk'
--- contrib/pwk	2006-02-28 14:45:51 +0000
+++ contrib/pwk	1970-01-01 00:00:00 +0000
@@ -1,50 +0,0 @@
-#! /bin/sh -pe
-
-# take patches from patchwork into bzr
-
-# authentication must be in ~/.netrc
-
-# TODO: Scan all pending patches and say which ones apply cleanly.
-
-# these should be moved into some kind of per-project configuration
-PWK_ROOT='http://patchwork.ozlabs.org/bazaar-ng'
-PWK_AUTH_ROOT='https://patchwork.ozlabs.org/bazaar-ng'
-
-usage() {
-    cat <<EOF
-usage: 
-   pwk cat PATCH-ID       show the patch text
-   pwk try PATCH-ID        see if the patch applies cleanly
-   pwk apply PATCH-ID      apply patch into current directory
-EOF
-}
-
-catpatch() {
-    curl --silent --show-error --get -d id=$1 $PWK_ROOT/patchcontent
-}
-
-if [ $# -lt 1 ]
-then
-    usage
-    exit 1
-fi
-
-
-case "$1" in
-help|-h|--help)
-    usage
-    exit 0
-    ;;
-cat)
-    catpatch $2 | ${PAGER:-less}
-    ;;
-try)
-    catpatch $2 | patch -p1 --dry-run
-    ;;
-apply)
-    catpatch $2 | patch -p1
-    ;;
-*)
-    usage
-    exit 1
-esac

=== modified file 'doc/developers/HACKING.txt'
--- doc/developers/HACKING.txt	2010-06-11 07:44:09 +0000
+++ doc/developers/HACKING.txt	2011-08-19 16:55:28 +0000
@@ -73,7 +73,7 @@
 Bazaar Development in a Nutshell
 ================================
 
-.. was from bazaar-vcs.org/BzrGivingBack
+.. was from http://wiki.bazaar.canonical.com/BzrGivingBack
 
 One of the fun things about working on a version control system like Bazaar is
 that the users have a high level of proficiency in contributing back into
@@ -89,7 +89,7 @@
 
  $ bzr init-repo ~/bzr
  $ cd ~/bzr
- $ bzr branch http://bazaar-vcs.org/bzr/bzr.dev/ bzr.dev
+ $ bzr branch lp:bzr bzr.dev
 
 Now make your own branch::
 
@@ -117,12 +117,12 @@
 `your_lp_username`.  You can push your branch to Launchpad directly from
 Bazaar::
 
-  $ bzr push lp:~your_lp_username/bzr/meaningful_name_here
+  $ bzr push lp:~<your_lp_username>/bzr/meaningful_name_here
 
 After you have pushed your branch, you will need to propose it for merging to
 the Bazaar trunk.  Go to
-<https://launchpad.net/your_lp_username/bzr/meaningful_name_here> and choose
-"Propose for merging into another branch".  Select "~bzr/bzr/trunk" to hand
+<https://launchpad.net/~<your_lp_username>/bzr/meaningful_name_here> and choose
+"Propose for merging into another branch".  Select "lp:bzr" to hand
 your changes off to the Bazaar developers for review and merging.
 
 Alternatively, after pushing you can use the ``lp-propose`` command to 
@@ -224,7 +224,7 @@
 * create a local copy of the main development branch (bzr.dev) by using
   this command::
 
-    bzr branch http://bazaar-vcs.org/bzr/bzr.dev/ bzr.dev
+    bzr branch lp:bzr bzr.dev
 
 * keep your copy of bzr.dev pristine (by not developing in it) and keep
   it up to date (by using bzr pull)
@@ -242,7 +242,7 @@
 Navigating the Code Base
 ========================
 
-.. Was at <http://bazaar-vcs.org/NewDeveloperIntroduction>
+.. Was at <http://wiki.bazaar.canonical.com/NewDeveloperIntroduction>
 
 Some of the key files in this directory are:
 
@@ -254,10 +254,6 @@
     This file covers a brief introduction to Bazaar and lists some of its
     key features.
 
-NEWS
-    Summary of changes in each Bazaar release that can affect users or
-    plugin developers.
-
 setup.py
     Installs Bazaar system-wide or to your home directory.  To perform
     development work on Bazaar it is not required to run this file - you
@@ -285,6 +281,16 @@
     Documentation specifically targeted at Bazaar and plugin developers.
     (Including this document.)
 
+doc/en/release-notes/
+
+    Detailed changes in each Bazaar release (there is one file by series:
+    bzr-2.3.txt, bzr-2.4.txt, etc) that can affect users or plugin
+    developers.
+
+doc/en/whats-new/
+
+    High-level summaries of changes in each Bazaar release (there is one
+    file by series: whats-new-in-2.3.txt, whats-new-in-2.4.txt, etc).
 
 
 Automatically-generated API reference information is available at
@@ -359,270 +365,6 @@
 can't fix.
 
 
-Getting Input
-=============
-
-Processing Command Lines
-------------------------
-
-bzrlib has a standard framework for parsing command lines and calling
-processing routines associated with various commands. See builtins.py
-for numerous examples.
-
-
-Standard Parameter Types
-------------------------
-
-There are some common requirements in the library: some parameters need to be
-unicode safe, some need byte strings, and so on. At the moment we have
-only codified one specific pattern: Parameters that need to be unicode
-should be checked via ``bzrlib.osutils.safe_unicode``. This will coerce the
-input into unicode in a consistent fashion, allowing trivial strings to be
-used for programmer convenience, but not performing unpredictably in the
-presence of different locales.
-
-
-Writing Output
-==============
-
-(The strategy described here is what we want to get to, but it's not
-consistently followed in the code at the moment.)
-
-bzrlib is intended to be a generically reusable library.  It shouldn't
-write messages to stdout or stderr, because some programs that use it
-might want to display that information through a GUI or some other
-mechanism.
-
-We can distinguish two types of output from the library:
-
- 1. Structured data representing the progress or result of an
-    operation.  For example, for a commit command this will be a list
-    of the modified files and the finally committed revision number
-    and id.
-
-    These should be exposed either through the return code or by calls
-    to a callback parameter.
-
-    A special case of this is progress indicators for long-lived
-    operations, where the caller should pass a ProgressBar object.
-
- 2. Unstructured log/debug messages, mostly for the benefit of the
-    developers or users trying to debug problems.  This should always
-    be sent through ``bzrlib.trace`` and Python ``logging``, so that
-    it can be redirected by the client.
-
-The distinction between the two is a bit subjective, but in general if
-there is any chance that a library would want to see something as
-structured data, we should make it so.
-
-The policy about how output is presented in the text-mode client
-should be only in the command-line tool.
-
-
-Progress and Activity Indications
----------------------------------
-
-bzrlib has a way for code to display to the user that stuff is happening
-during a long operation.  There are two particular types: *activity* which
-means that IO is happening on a Transport, and *progress* which means that
-higher-level application work is occurring.  Both are drawn together by
-the `ui_factory`.
-
-Transport objects are responsible for calling `report_transport_activity`
-when they do IO.
-
-Progress uses a model/view pattern: application code acts on a
-`ProgressTask` object, which notifies the UI when it needs to be
-displayed.  Progress tasks form a stack.  To create a new progress task on
-top of the stack, call `bzrlib.ui.ui_factory.nested_progress_bar()`, then
-call `update()` on the returned ProgressTask.  It can be updated with just
-a text description, with a numeric count, or with a numeric count and
-expected total count.  If an expected total count is provided the view
-can show the progress moving along towards the expected total.
-
-The user should call `finish` on the `ProgressTask` when the logical
-operation has finished, so it can be removed from the stack.
-
-Progress tasks have a complex relationship with generators: it's a very
-good place to use them, but because python2.4 does not allow ``finally``
-blocks in generators it's hard to clean them up properly.  In this case
-it's probably better to have the code calling the generator allocate a
-progress task for its use and then call `finalize` when it's done, which
-will close it if it was not already closed.  The generator should also
-finish the progress task when it exits, because it may otherwise be a long
-time until the finally block runs.
-
-
-Message guidelines
-------------------
-
-When filenames or similar variables are presented inline within a message,
-they should be enclosed in double quotes (ascii 0x22, not chiral unicode
-quotes)::
-
-  bzr: ERROR: No such file "asdf"
-
-When we print just a list of filenames there should not be any quoting:
-see `bug 544297`_.
-
-.. _bug 544297: https://bugs.launchpad.net/bugs/544297
-
-https://wiki.ubuntu.com/UnitsPolicy provides a good explanation about
-which unit should be used when. Roughly speaking, IEC standard applies
-for base-2 units and SI standard applies for base-10 units:
-
-* for network bandwidth and disk sizes, use base-10 (Mbits/s, kB/s, GB)
-
-* for RAM sizes, use base-2 (GiB, TiB)
-
-
-
-Displaying help
-===============
-
-Bazaar has online help for various topics through ``bzr help COMMAND`` or
-equivalently ``bzr command -h``.  We also have help on command options,
-and on other help topics.  (See ``help_topics.py``.)
-
-As for python docstrings, the first paragraph should be a single-sentence
-synopsis of the command. These are user-visible and should be prefixed with
-``__doc__ =`` so help works under ``python -OO`` with docstrings stripped.
-
-The help for options should be one or more proper sentences, starting with
-a capital letter and finishing with a full stop (period).
-
-All help messages and documentation should have two spaces between
-sentences.
-
-
-Handling Errors and Exceptions
-==============================
-
-Commands should return non-zero when they encounter circumstances that
-the user should really pay attention to - which includes trivial shell
-pipelines.
-
-Recommended values are:
-
-    0. OK.
-    1. Conflicts in merge-like operations, or changes are present in
-       diff-like operations.
-    2. Unrepresentable diff changes (i.e. binary files that we cannot show
-       a diff of).
-    3. An error or exception has occurred.
-    4. An internal error occurred (one that shows a traceback.)
-
-Errors are handled through Python exceptions. Exceptions should be defined
-inside bzrlib.errors, so that we can see the whole tree at a glance.
-
-We broadly classify errors as either being either internal or not,
-depending on whether ``internal_error`` is set or not.  If we think it's our
-fault, we show a backtrace, an invitation to report the bug, and possibly
-other details.  This is the default for errors that aren't specifically
-recognized as being caused by a user error.  Otherwise we show a briefer
-message, unless -Derror was given.
-
-Many errors originate as "environmental errors" which are raised by Python
-or builtin libraries -- for example IOError.  These are treated as being
-our fault, unless they're caught in a particular tight scope where we know
-that they indicate a user errors.  For example if the repository format
-is not found, the user probably gave the wrong path or URL.  But if one of
-the files inside the repository is not found, then it's our fault --
-either there's a bug in bzr, or something complicated has gone wrong in
-the environment that means one internal file was deleted.
-
-Many errors are defined in ``bzrlib/errors.py`` but it's OK for new errors
-to be added near the place where they are used.
-
-Exceptions are formatted for the user by conversion to a string
-(eventually calling their ``__str__`` method.)  As a convenience the
-``._fmt`` member can be used as a template which will be mapped to the
-error's instance dict.
-
-New exception classes should be defined when callers might want to catch
-that exception specifically, or when it needs a substantially different
-format string.
-
-#. If it is something that a caller can recover from, a custom exception
-   is reasonable.
-
-#. If it is a data consistency issue, using a builtin like
-   ``ValueError``/``TypeError`` is reasonable.
-
-#. If it is a programmer error (using an api incorrectly)
-   ``AssertionError`` is reasonable.
-
-#. Otherwise, use ``BzrError`` or ``InternalBzrError``.
-
-Exception strings should start with a capital letter and should not have a
-final fullstop.  If long, they may contain newlines to break the text.
-
-
-
-Documenting Changes
-===================
-
-When you change bzrlib, please update the relevant documentation for the
-change you made: Changes to commands should update their help, and
-possibly end user tutorials; changes to the core library should be
-reflected in API documentation.
-
-NEWS File
----------
-
-If you make a user-visible change, please add a note to the NEWS file.
-The description should be written to make sense to someone who's just
-a user of bzr, not a developer: new functions or classes shouldn't be
-mentioned, but new commands, changes in behaviour or fixed nontrivial
-bugs should be listed.  See the existing entries for an idea of what
-should be done.
-
-Within each release, entries in the news file should have the most
-user-visible changes first.  So the order should be approximately:
-
- * changes to existing behaviour - the highest priority because the
-   user's existing knowledge is incorrect
- * new features - should be brought to their attention
- * bug fixes - may be of interest if the bug was affecting them, and
-   should include the bug number if any
- * major documentation changes, including fixed documentation bugs
- * changes to internal interfaces
-
-People who made significant contributions to each change are listed in
-parenthesis.  This can include reporting bugs (particularly with good
-details or reproduction recipes), submitting patches, etc.
-
-To help with merging, NEWS entries should be sorted lexicographically
-within each section.
-
-Commands
---------
-
-The docstring of a command is used by ``bzr help`` to generate help output
-for the command. The list 'takes_options' attribute on a command is used by
-``bzr help`` to document the options for the command - the command
-docstring does not need to document them. Finally, the '_see_also'
-attribute on a command can be used to reference other related help topics.
-
-API Documentation
------------------
-
-Functions, methods, classes and modules should have docstrings
-describing how they are used.
-
-The first line of the docstring should be a self-contained sentence.
-
-For the special case of Command classes, this acts as the user-visible
-documentation shown by the help command.
-
-The docstrings should be formatted as reStructuredText_ (like this
-document), suitable for processing using the epydoc_ tool into HTML
-documentation.
-
-.. _reStructuredText: http://docutils.sourceforge.net/rst.html
-.. _epydoc: http://epydoc.sourceforge.net/
-
-
 General Guidelines
 ==================
 
@@ -694,6 +436,12 @@
 be disabled if necessary by setting the environment variable
 ``BZR_SIGQUIT_PDB=0``.
 
+All tests inheriting from bzrlib.tests.TestCase can use ``self.debug()``
+instead of the longer ``import pdb; pdb.set_trace()``. The former also works
+when ``stdin/stdout`` are redirected (by using the original ``stdin/stdout``
+file handles at the start of the ``bzr`` script) while the later doesn't.
+``bzrlib.debug.set_trace()`` also uses the original ``stdin/stdout`` file
+handles.
 
 Debug Flags
 ===========
@@ -775,7 +523,7 @@
 Because Transports work in URLs (as defined earlier), printing the raw URL
 to the user is usually less than optimal. Characters outside the standard
 set are printed as escapes, rather than the real character, and local
-paths would be printed as ``file://`` urls. The function
+paths would be printed as ``file://`` URLs. The function
 ``unescape_for_display`` attempts to unescape a URL, such that anything
 that cannot be printed in the current encoding stays an escaped URL, but
 valid characters are generated where possible.
@@ -840,7 +588,6 @@
 stuff covered above, "core" developers have responsibility for:
 
 * reviewing changes
-* reviewing blueprints
 * planning releases
 * managing releases (see `Releasing Bazaar <http://doc.bazaar.canonical.com/developers/releasing.html>`_)
 

=== modified file 'doc/developers/authentication-ring.txt'
--- doc/developers/authentication-ring.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/authentication-ring.txt	2010-11-12 22:46:28 +0000
@@ -61,12 +61,12 @@
 This specification describes a way to provide user and passwords to bzr while
 storing them in a relatively safe way.
 
-Note that ssh servers can be configured to use keys instead of (``user``,
+Note that SSH servers can be configured to use keys instead of (``user``,
 ``password``) and, when used with appropriate agents, provide the same kind of
-comfort this specification aims to provide for all other schemes. Since ssh
+comfort this specification aims to provide for all other schemes. Since SSH 
 agents provide a safer way to secure the passwords, this specification is
 restricted to providing ``user`` but does not provide ``password`` when used
-for ssh.
+for SSH.
 
 Authentication definitions
 --------------------------
@@ -77,8 +77,8 @@
 1. user and password
 
 ``FTP`` and ``SFTP`` needs a (``user``, ``password``) to authenticate against a
-``host`` (SFTP can use ssh keys too, but we don't talk about that in this
-specification as ssh agents provide a better solution).
+``host`` (SFTP can use SSH keys too, but we don't talk about that in this
+specification as SSH agents provide a better solution).
 
 2. user, realm and password
 
@@ -115,8 +115,8 @@
 
   * ``path``: can be empty (FTP or SFTP will never use it),
 
-  * ``user``: can be empty (``bzr`` will defaults to python's
-    ``getpass.get_user()`` for FTP, SFTP and ssh),
+  * ``user``: can be empty (``bzr`` will defaults to Python's
+    ``getpass.get_user()`` for FTP, SFTP and SSH),
 
   * ``password``: can be empty (for security reasons, a user may use the
     definitions without storing the passwords but want to be prompted ; or the
@@ -303,7 +303,7 @@
   * source hosting provider declaring sub-domains for each project::
 
         [sfnet domain]
-        # we use sftp, but ssh is the scheme used for authentication
+        # we use SFTP, but SSH is the scheme used for authentication
         scheme=ssh
         # The leading '.' ensures that 'sf.net' alone doesn't match
         host=.sf.net
@@ -347,7 +347,7 @@
 * No assumptions should be made about the clients of this service
   (i.e. Transport is the primary target but plugins must be able to use it as
   well, the definitions used: (``scheme, host, [port,] path``) are general
-  enough to described credentials for ``svn`` servers or LaunchPad xmlrpc
+  enough to described credentials for ``svn`` servers or LaunchPad XML-RPC
   calls).
 
 * Policies regarding default users may be taken into account by the
@@ -361,7 +361,7 @@
   a password.
 
 * As this specification proposes a matching between some credentials
-  definitions and real urls, the implementation provides an optional UI
+  definitions and real URLs, the implementation provides an optional UI
   feedback about which credential definition is used. Using ``-Dauth`` will
   output some traces in the ``.bzr.log`` file metionning the sections
   used. This allows the user to validate his definitions.
@@ -392,7 +392,7 @@
     * plugins can be written and registered to handle the associated
       ``password_encoding``.
 
-  * Could it be possible to encode the whole authentication file with a ssh key
+  * Could it be possible to encode the whole authentication file with an SSH key
     ?
 
     * yes and if the user configure a ssh-agent it will not be queried for

=== modified file 'doc/developers/bug-handling.txt'
--- doc/developers/bug-handling.txt	2010-05-31 19:33:05 +0000
+++ doc/developers/bug-handling.txt	2011-05-04 10:45:00 +0000
@@ -19,14 +19,14 @@
 Links
 *****
 
-* `bzr bugs home page <https://bugs.edge.launchpad.net/bzr>`_.
-
-* `Critical bugs <https://bugs.edge.launchpad.net/bzr/+bugs?search=Search&field.importance=Critical&field.status=New&field.status=Incomplete&field.status=Confirmed&field.status=Triaged&field.status=In+Progress&field.status=Fix+Committed>`_.
-
-* `Open bugs by importance <https://bugs.edge.launchpad.net/bzr/+bugs>`_.
+* `bzr bugs home page <https://bugs.launchpad.net/bzr>`_.
+
+* `Critical bugs <https://bugs.launchpad.net/bzr/+bugs?search=Search&field.importance=Critical&field.status=New&field.status=Incomplete&field.status=Confirmed&field.status=Triaged&field.status=In+Progress&field.status=Fix+Committed>`_.
+
+* `Open bugs by importance <https://bugs.launchpad.net/bzr/+bugs>`_.
 
 * `Open bugs most recently changed first
-  <https://bugs.edge.launchpad.net/bzr/+bugs?field.searchtext=&orderby=-date_last_updated&search=Search&field.status%3Alist=NEW&field.status%3Alist=INCOMPLETE_WITH_RESPONSE&field.status%3Alist=INCOMPLETE_WITHOUT_RESPONSE&field.status%3Alist=CONFIRMED&field.status%3Alist=TRIAGED&field.status%3Alist=INPROGRESS&field.status%3Alist=FIXCOMMITTED&field.assignee=&field.bug_reporter=&field.omit_dupes=on&field.has_patch=&field.has_no_package=>`_.
+  <https://bugs.launchpad.net/bzr/+bugs?field.searchtext=&orderby=-date_last_updated&search=Search&field.status%3Alist=NEW&field.status%3Alist=INCOMPLETE_WITH_RESPONSE&field.status%3Alist=INCOMPLETE_WITHOUT_RESPONSE&field.status%3Alist=CONFIRMED&field.status%3Alist=TRIAGED&field.status%3Alist=INPROGRESS&field.status%3Alist=FIXCOMMITTED&field.assignee=&field.bug_reporter=&field.omit_dupes=on&field.has_patch=&field.has_no_package=>`_.
 
 * `Most commonly duplicated bugs <http://tinyurl.com/bzr-bugs-by-dupes>`_.
 
@@ -43,7 +43,7 @@
 When you file a bug as a Bazaar developer or active user, if you feel
 confident in doing so, make an assessment of status and importance at the
 time you file it, rather than leaving it for someone else.  It's more
-efficient to change the importance if someone else feel's it's higher or
+efficient to change the importance if someone else feels it's higher or
 lower, than to have someone else edit all bugs.
 
 It's more useful to actually ship bug fixes than to garden the bug
@@ -184,7 +184,7 @@
     time updating this if you don't immediately know: its not critical
     that it be set.
 
-.. _`bug 163694`: https://bugs.launchpad.net/malone/+bug/163694
+.. _`bug 163694`: https://bugs.launchpad.net/launchpad/+bug/163694
 
 
 Bug Importance
@@ -228,7 +228,7 @@
 ***************
 
 It's possible to target a bug to a milestone, eg
-<https://bugs.edge.launchpad.net/bzr/+milestone/1.16>.  We use this to help the
+<https://bugs.launchpad.net/bzr/+milestone/1.16>.  We use this to help the
 release manager know what **must** be merged to make the release.
 
 Therefore, we don't target bugs that we'd like to have fixed or that could
@@ -255,11 +255,11 @@
 separate work to get it into that release.
 
 
-The News File
+Release Notes
 *************
 
-Most bugs that are fixed should be mentioned in a `NEWS
-<../en/release-notes/NEWS.html>`_ file entry,
+Most bugs that are fixed should be mentioned in the `Release Notes
+<../en/release-notes/>`_ for the forthcoming version,
 including the bug number.
 (Exceptions might be bugs that are not at all user visible.)
 
@@ -267,7 +267,7 @@
 Tags
 ****
 
-Here are some bug tags we use.  In Malone tags are currently of limited use, so don't feel obliged to tag bugs unless you're finding it useful.
+Here are some bug tags we use.  In Launchpad Bugs tags are currently of limited use, so don't feel obliged to tag bugs unless you're finding it useful.
 
 
 authentication
@@ -310,19 +310,21 @@
     needs changes to the test framework
 
 transport
-    virtual filesystem for http, sftp, etc
+    virtual filesystem for HTTP, SFTP, etc.
 
 trivial
-    should be very easy to fix (10-20 minutes) and easily landed: typically just spelling errors and the like
+    should be very easy to fix (10-20 minutes) and easily landed: typically 
+    just spelling errors and the like
 
 ui
     bugs relating to the bzr user interface, e.g. confusing error messages.
 
 win32
-    bugs that mainly affects Windows. Also there is cygwin and win98 tags for marking specific bugs.
+    bugs that mainly affects Windows. Also there is cygwin and win98 tags for 
+    marking specific bugs.
 
 You can see the full list of tags in use at
-<https://bugs.edge.launchpad.net/bzr/+bugs>.  As of September 2008 the
-list is on the right.
+<https://bugs.launchpad.net/bzr/+bugs>.  As of September 2008 the
+list is on the right. 
 
 .. vim: ft=rst

=== modified file 'doc/developers/case-insensitive-file-systems.txt'
--- doc/developers/case-insensitive-file-systems.txt	2010-06-02 05:03:31 +0000
+++ doc/developers/case-insensitive-file-systems.txt	2010-08-13 19:08:57 +0000
@@ -32,7 +32,7 @@
 relevant to a modern Bazaar.
 
 For more details, including use cases, please see
-http://bazaar-vcs.org/CasePreservingWorkingTreeUseCases
+http://wiki.bazaar.canonical.com/CasePreservingWorkingTreeUseCases
 
 Handling these file-systems
 ---------------------------

=== modified file 'doc/developers/check.txt'
--- doc/developers/check.txt	2009-05-12 03:50:39 +0000
+++ doc/developers/check.txt	2010-11-12 22:43:38 +0000
@@ -35,7 +35,7 @@
 Things that can go wrong:
 * Tag or tip revision ids may be missing from the repo.
 * The revno tip cache may be wrong.
-* Various urls could be problematic (not inaccessible, just invalid)
+* Various URLS could be problematic (not inaccessible, just invalid)
 * Stacked-on branch could be inaccessible.
 
 Tree

=== modified file 'doc/developers/code-review.txt'
--- doc/developers/code-review.txt	2010-05-14 13:34:22 +0000
+++ doc/developers/code-review.txt	2011-06-06 08:24:45 +0000
@@ -9,7 +9,7 @@
 
 Normally changes by core contributors are reviewed by one other core
 developer, and changes from other people are reviewed by two core
-developers.  Use intelligent discretion about whether if the patch is trivial.
+developers.  Use intelligent discretion about whether the patch is trivial.
 
 No one likes their merge requests sitting in a queue going nowhere: this 
 is pure waste.  We prioritize reviewing existing proposals.  
@@ -66,9 +66,9 @@
   blackbox (command-line level) and API-oriented tests?
 
 * If this change will be visible to end users or API users, is it
-  appropriately documented in NEWS?
+  appropriately documented in release notes and/or in whats-new ?
 
-* Does it meet the coding standards below?
+* Does it meet the `coding standards <code-style.html>`_?
 
 * If it changes the user-visible behaviour, does it update the help
   strings and user documentation?

=== modified file 'doc/developers/code-style.txt'
--- doc/developers/code-style.txt	2010-06-02 04:39:08 +0000
+++ doc/developers/code-style.txt	2011-06-28 21:45:01 +0000
@@ -78,9 +78,9 @@
 ===============
 
 Bazaar supports Python from 2.4 through 2.6, and in the future we want to
-support Python 3.0.  Avoid using language features added in 2.5 or 2.6, or
-features deprecated in Python 3.0.  (You can check v3 compatibility using
-the ``-3`` option of Python2.6.)
+support Python 2.7 and 3.0.  Avoid using language features added in 2.5,
+2.6 or 2.7, or features deprecated in Python 3.0.  (You can check v3
+compatibility using the ``-3`` option of Python2.6.)
 
 Specifically:
 
@@ -101,6 +101,38 @@
   if getattr(thing, 'name', None) is None
 
 
+kwargs
+======
+
+``**kwargs`` in the prototype of a function should be used sparingly.
+It can be good on higher-order functions that decorate other functions,
+such as ``addCleanup`` or ``assertRaises``, or on functions that take only
+(or almost only) kwargs, where any kwargs can be passed.  
+
+Otherwise, be careful: if the parameters to a function are a bit complex
+and might vary over time (e.g.  the ``commit`` API) then we prefer to pass an
+object rather than a bag of positional and/or keyword args.  If you have
+an arbitrary set of keys and values that are different with each use (e.g.
+string interpolation inputs) then again that should not be mixed in with
+the regular positional/keyword args, it seems like a different category of
+thing.
+
+
+Imitating standard objects
+==========================
+
+Don't provide methods that imitate built-in classes (eg ``__in__``,
+``__call__``, ``__int__``, ``__getitem__``) unless the class you're
+implementing really does act like the builtin class, in semantics and
+performance.
+
+For example, old code lets you say ``file_id in inv`` but we no longer
+consider this good style.  Instead, say more explicitly
+``inv.has_id(file_id)``.
+
+``__repr__``, ``__cmp__``, ``__str__`` are usually fine.
+
+
 Module Imports
 ==============
 
@@ -157,15 +189,21 @@
    why in a comment.
 
 1. Never rely on a ``__del__`` method running.  If there is code that
-   must run, do it from a ``finally`` block instead.
+   must run, instead have a ``finally`` block or an ``addCleanup`` call an
+   explicit ``close`` method.
 
 2. Never ``import`` from inside a ``__del__`` method, or you may crash the
    interpreter!!
 
-3. In some places we raise a warning from the destructor if the object
-   has not been cleaned up or closed.  This is considered OK: the warning
-   may not catch every case but it's still useful sometimes.
-
+3. Prior to bzr 2.4, we sometimes used to raise warnings from del methods
+   that the object was not cleaned up or closed.  We no longer do this:
+   failure to close the object doesn't cause a test failure; the warning
+   appears an arbitrary long time after the problem occurred (the object
+   being leaked); merely having a del method inhibits Python gc; the
+   warnings appear to users and upset them; they can also break tests that
+   are checking what appears on stderr.
+   
+In short, just don't use ``__del__``.
 
 Cleanup methods
 ===============
@@ -332,6 +370,8 @@
 
 Because repr methods are often called when something has already gone
 wrong, they should be written somewhat more defensively than most code.
+They shouldn't have side effects like doing network or disk
+IO.
 The object may be half-initialized or in some other way in an illegal
 state.  The repr method shouldn't raise an exception, or it may hide the
 (probably more useful) underlying exception.
@@ -353,6 +393,13 @@
 ``Exception`` (which excludes system errors in Python2.5 and later) would
 be better.
 
+The ``__str__`` method on exceptions should be small and have no side
+effects, following the rules given for `Object string representations`_.
+In particular it should not do any network IO, or complicated
+introspection of other objects.  All the state needed to present the
+exception to the user should be gathered before the error is raised.
+In other words, exceptions should basically be value objects.
+
 
 Test coverage
 =============
@@ -453,5 +500,19 @@
 
  * Don't say "open source" when you mean "free software".
 
+
+Dynamic imports
+===============
+
+If you need to import a module (or attribute of a module) named in a
+variable:
+
+ * If importing a module, not an attribute, and the module is a top-level
+   module (i.e. has no dots in the name), then it's ok to use the builtin
+   ``__import__``, e.g. ``__import__(module_name)``.
+ * In all other cases, prefer ``bzrlib.pyutils.get_named_object`` to the
+   built-in ``__import__``.  ``__import__`` has some subtleties and
+   unintuitive behaviours that make it hard to use correctly.
+
 ..
    vim: ft=rst tw=74 ai

=== added file 'doc/developers/configuration.txt'
--- doc/developers/configuration.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/configuration.txt	2011-09-07 15:21:39 +0000
@@ -0,0 +1,142 @@
+Configuring Bazaar
+==================
+
+A configuration option has:
+
+* a name: a valid python identifier (even if it's not used as an
+  identifier in python itself)
+
+* a value: a unicode string or a list of unicode strings.
+
+Option
+------
+
+The Option object is used to define its properties:
+
+* name: a name: a valid python identifier (even if it's not used as an
+  identifier in python itself). This is also used to register the option.
+
+* default: the default value that Stack.get() should return if no
+  value can be found for the option.
+
+* default_from_env: a list of environment variables. The first variable set
+  will provide a default value overriding 'default' which remains the
+  default value if *no* environment variable is set.
+
+* help: a doc string describing the option, the first line should be a
+  summary and can be followed by a blank line and a more detailed
+  explanation.
+
+* from_unicode: a callable accepting a unicode string and returning a
+  suitable value for the option. If the string cannot be coerced it should
+  return None.
+
+* invalid: the action to be taken when an invalid value is encountered in a
+  store (during a Stack.get()).
+
+Sections
+--------
+
+Options are grouped into sections which share some properties with the well
+known dict objects:
+
+* the key is the name,
+* you can get, set and remove an option,
+* the value is a unicode string.
+
+MutableSection is needed to set or remove an option, ReadOnlySection should
+be used otherwise.
+
+Stores
+------
+
+Options can be persistent in which case they are saved into Stores.
+
+``config.Store`` defines the abstract interface that all stores should
+implement.
+
+This object doesn't provide direct access to the options, it only provides
+access to Sections. This is deliberate to ensure that sections can be
+properly shared by reusing the same underlying objects. Accessing options
+should be done via the ``Section`` objects.
+
+A ``Store`` can contain one or more sections, each section is uniquely
+identified by a unicode string.
+
+``config.ConfigObjStore`` is an implementation that use ``ConfigObj``.
+
+Depending on the object it is associated with (or not) a ``Store`` also needs
+to implement a locking mechanism. ``LockableConfigObjStore`` implements such a
+mechanism for ``ConfigObj`` based stores.
+
+Classes are provided for the usual Bazaar configuration files and could be
+used as examples to define new ones if needed. The associated tests provides a
+basis for new classes which only need to register themselves in the right
+places to inherit from the existing basic tests and add their own specific
+ones.
+
+Filtering sections
+------------------
+
+For some contexts, only some sections from a given store will apply. Defining
+which is what the ``SectionMatcher`` are about.
+
+The main constraint here is that a ``SectionMatcher`` should delay the loading
+of the associated store as long as possible. The constructor should collect
+all data needed for the selection and uses it while processing the sections in
+``get_sections``.
+
+Only ``ReadOnlySection`` objects are manipulated here but a
+``SectionMatcher`` can return dedicated ``Section`` objects to provide
+additional context (the ``LocationSection`` add an ``extra_path`` attribute
+to implement the ``appendpath`` policy for example). If no sections match,
+an empty list is returned.
+
+.. FIXME: Replace the appendpath example if/when it's deprecated ;)
+
+Specific section matchers can be implemented by overriding ``get_sections``
+or just ``match``.
+
+``bzrlib`` provides:
+
+* ``LocationMatcher(store, location)``: To select all sections that match
+  ``location``.
+
+* ``NameMatcher(store, unique_id)``: To select a single section matching
+  ``unique_id``.
+
+Stacks
+------
+
+An option can take different values depending on the context it is used. Such
+a context can involve configuration files, options from the command line,
+default values in bzrlib and then some.
+
+Such a context is implemented by creating a list of ``Section`` stacked upon
+each other. A ``Stack`` can then be asked for an option value and returns the
+first definition found.
+
+This provides a great flexibility to decide priorities between sections when
+the stack is defined without to worry about them in the code itself.
+
+A stack also defines a mutable section (which can be None) to handle
+modifications.
+
+Many sections (or even stores) are aimed at providing default values for an
+option but these sections shouldn't be modified lightly as modifying an option
+used for different contexts will indeed be seen by all these contexts.
+
+Default values in configuration files are defined by users. Developers
+shouldn't have to modify them, as such, no mechanism nor heuristics are used
+to find which section (or sections) should be modified.
+
+A ``Stack`` defines a mutable section when there is no ambiguity.  If there
+is one, then the *user* should be able to decide and in this case a new
+``Stack`` can be created cheaply.
+
+Different stacks can be created for different purposes, the existing
+``GlobalStack``, ``LocationStack`` and ``BranchStack`` can be used as basis
+or examples. These classes are the only ones that should be used in code,
+``Stores`` can be used to build them but shouldn't be used otherwise, ditto
+for sections. Again, the associated tests could and should be used against the
+created stacks.

=== modified file 'doc/developers/content-filtering.txt'
--- doc/developers/content-filtering.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/content-filtering.txt	2010-12-02 09:46:15 +0000
@@ -59,7 +59,7 @@
 which that cache is valid.  This implies that the stored size will
 in general be different to the size in the packed stat.  (However, it
 may not always do this correctly - see
-<https://bugs.edge.launchpad.net/bzr/+bug/418439>.)
+<https://bugs.launchpad.net/bzr/+bug/418439>.)
 
 The dirstate is given a SHA1Provider instance by its tree.  This class
 can calculate the (canonical) hash and size given a filename.  This
@@ -136,9 +136,9 @@
 See also
 ********
 
-* http://bazaar-vcs.org/LineEndings
+* http://wiki.bazaar.canonical.com/LineEndings
 
-* http://bazaar-vcs.org/LineEndings/Roadmap
+* http://wiki.bazaar.canonical.com/LineEndings/Roadmap
 
 * `Developer Documentation <index.html>`_
 

=== modified file 'doc/developers/contribution-quickstart.txt'
--- doc/developers/contribution-quickstart.txt	2010-06-11 07:44:09 +0000
+++ doc/developers/contribution-quickstart.txt	2011-08-11 05:48:38 +0000
@@ -26,7 +26,11 @@
 you should set it to "In Progress" and if you wish assign it to yourself.
 
 You might like to start with a bug tagged `easy
-<https://bugs.edge.launchpad.net/bzr/+bugs?field.tag=easy>`_.
+<https://bugs.launchpad.net/bzr/+bugs?field.tag=easy>`_.
+
+If you are wondering if your understanding of the bug is correct, or if the 
+approach you have in mind is likely to work, feel to ask about it on the bug,
+in ``#bzr`` or on the mailing list.
 
 Making a branch
 ---------------
@@ -96,7 +100,7 @@
     $ ./bzr selftest
 
 Normally the tests will skip if some library dependencies are not present.
-On Ubuntu, you can install them with this command (you must have some
+On Ubuntu, you can install them with this command (you must have source
 repositories enabled in Software Sources)::
 
     $ sudo apt-get build-dep bzr
@@ -113,7 +117,7 @@
 
 
 Then propose a merge into bzr; for bzr 2.2 and later you can use the ``bzr
-propose-merge`` command.  In the comment for your merge proposal please
+lp-propose-merge`` command.  In the comment for your merge proposal please
 explain what you're trying to do and why.  For `example
 <https://code.launchpad.net/~ian-clatworthy/bzr/whats-new-in-2.1/+merge/19677>`_:
 

=== modified file 'doc/developers/cycle.txt'
--- doc/developers/cycle.txt	2010-01-08 00:44:26 +0000
+++ doc/developers/cycle.txt	2011-01-03 23:48:17 +0000
@@ -2,7 +2,7 @@
 Bazaar Release Cycles
 *********************
 
-:status: Current policy, as of 2009-08.
+:status: Current policy, as of 2010-10.
 :blueprint: <https://blueprints.launchpad.net/bzr/+spec/6m-cycle>
 
 
@@ -10,7 +10,7 @@
 core product. They also want a Just Works experience across the full
 Bazaar ecosystem. To deliver the first and enable the second, we're
 adopting some standard process patterns: a 6 monthly release cycle and a
-stable branch. These changes will also have other benefits, including
+stable series. These changes will also have other benefits, including
 better availability of bug fixes in OS distributions, more freedom to
 remove old code, and less work for in packaging.
 
@@ -25,11 +25,11 @@
 The Process
 ************
 
-Bazaar will make a major release every six months, which will be supported
-at least until the time of the next major release.  During this support
-period, we'll make incremental releases which fix bugs, but which do not
-change network or disk formats or command syntax, and which do not require
-updates to plugins.
+Bazaar will make a major release every six months, which will be supported at
+least until the time of the next major release and generally 18 months after
+the first final release in a series.  During this support period, we'll make
+incremental releases which fix bugs, but which do not change network or disk
+formats or command syntax, and which do not require updates to plugins.
 
 We will also run a development series, which will become the next major
 release.  We'll make a beta release from this every four weeks.  The
@@ -75,15 +75,6 @@
 often if there are serious bugs, perhaps much less often if no new changes
 have landed.
 
-We will then make a release candidate for the next major release, and at
-this point create a release branch for it.  We will iterate release
-candidates at approximately weekly intervals until there are no bugs
-blocking the final major release.
-
-Compared to the current process this has approximately the same amount of
-release-related work, because the extra releases from the stable branch
-are "paid for" by not doing RCs for the development series.
-
 We will synchronize our major releases with Ubuntu, so that they come out
 in sufficient time for some testing and margin of error before Ubuntu's
 upstream freeze.
@@ -118,32 +109,40 @@
 -----------
 
 Major releases (2.0.0 or 2.1.0)
+
     The big ones, every six months, intended to ship in distributions and
     to be used by stability-oriented users.
 
 Release candidate (2.0.0rc1)
-    A preview of a major release, made one or a few weeks beforehand at
-    the time the release branch is created.  There should be few if any
-    changes from the rc to the stable release.  We should avoid the
-    confusing phrasing "release candidate 2.0.0rc1 is released"; instead
-    use "available."
+
+    A preview of a major release, made one or a few weeks beforehand at the
+    time the release branch is created.  There should be few if any changes
+    from the rc to the stable release.  We should avoid the confusing phrasing
+    "release candidate 2.0.0rc1 is released"; instead use "available."
+    Starting with the 2.3 series we don't plan on making release candidates
+    anymore.
 
 Bugfix releases (2.0.1)
+
     Based on the previous major release or bugfix; contains only bugfixes
     and perhaps documentation or translation corrections.
 
 Stable series
+
     A major release and its descendant bugfix releases.
 
 Stable release
+
     Either a major release or a bugfix release.
 
 Beta release (3.0.0beta1)
+
     Made from trunk every month, except for the month there's a major
     release.  Stable and suitable for users who want the latest code and
     can live with some changes from month to month.
 
 Development series
+
     The development releases leading up to a stable release.
 
 Bug Work
@@ -183,8 +182,8 @@
 Plugins that want to cooperate with this should make a series and a branch
 that matches each bzr stable series, and follow similar rules in making
 releases from their stable branch.  We'd expect that plugins will make a
-release between the last development release of a series and the major
-release candidate.
+release between the first beta release of a series and the final major
+release.
 
 Within a stable series, anything that breaks any known plugin is
 considered an API break and will be avoided.  Before
@@ -274,11 +273,10 @@
 Packaging
 ---------
 
-At present we have three upstream-maintained PPAs containing Ubuntu
-packages of Bazaar: ``~bzr-nightly-ppa``, ``~bzr-beta-ppa`` (rcs and
-releases) and ``~bzr`` (ie stable).  We will keep these PPAs, and reorient
-beta to contain the monthly beta releases, and the stable PPA to contain
-stable releases, their release candidates, and bugfixes to those releases.
+At present we have three upstream-maintained PPAs containing Ubuntu packages
+of Bazaar: ``bzr/daily`` (snapshots), ``bzr-beta-ppa/ppa`` (beta releases) and
+``bzr/ppa`` (ie stable).  Beta contains the monthly beta releases, and the
+stable PPA contains stable releases and bugfixes to those releases.
 
 Some platforms with relatively less active packagers may choose to ship
 only the stable releases.  This is probably better than having them only

=== modified file 'doc/developers/development-repo.txt'
--- doc/developers/development-repo.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/development-repo.txt	2010-08-13 19:08:57 +0000
@@ -171,7 +171,7 @@
 
 If you need any help or encounter any problems, please contact the developers
 via the usual ways, i.e. chat to us on IRC or send a message to our mailing
-list. See http://bazaar-vcs.org/BzrSupport for contact details.
+list. See http://wiki.bazaar.canonical.com/BzrSupport for contact details.
 
 
 Technical notes

=== modified file 'doc/developers/directory-fingerprints.txt'
--- doc/developers/directory-fingerprints.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/directory-fingerprints.txt	2010-11-12 22:08:18 +0000
@@ -258,7 +258,7 @@
      as stored, or change the sort order for the inventory, or synthesize
      per-directory inventories in memory for hashing.
 
-     However, xml is somewhat redundant and slow to parse/generate; and
+     However, XML is somewhat redundant and slow to parse/generate; and
      reading the whole thing before comparing some sections is only a
      partial win.  It may be a smaller change but we'd be preserving
      things we want to change.

=== modified file 'doc/developers/dirstate.txt'
--- doc/developers/dirstate.txt	2009-09-09 15:51:18 +0000
+++ doc/developers/dirstate.txt	2011-04-22 14:12:22 +0000
@@ -4,3 +4,57 @@
 Don't really need the hashes of the current versions - just knowing
 whether they've changed or not will generally be enough - and just the
 mtime and ctime of a point in time may be enough?
+
+
+``_dirblock_state``
+-------------------
+
+There are currently 4 levels that state can have.
+
+ 1. NOT_IN_MEMORY
+    The actual content blocks have not been read at all.
+ 2. IN_MEMORY_UNMODIFIED
+    The content blocks have been read and are available for use. They have
+    not been changed at all versus what was written on disk when we read
+    them.
+ 3. IN_MEMORY_HASH_MODIFIED
+    We have updated the in-memory state, but only to record the
+    sha1/symlink target value and the stat value that means this
+    information is 'fresh'.
+ 4. IN_MEMORY_MODIFIED
+    We have updated an actual record. (Parent lists, added a new file,
+    deleted something, etc.) In this state, we must always write out the
+    dirstate, or some user action will be lost.
+
+
+IN_MEMORY_HASH_MODIFIED
+~~~~~~~~~~~~~~~~~~~~~~~
+
+This state is a bit special, so deserves its own topic.  If we are
+IN_MEMORY_HASH_MODIFIED, we only write out the dirstate if enough records
+have been updated. The idea is that if we would save future I/O by writing
+an updated dirstate, then we should do so. The threshold for this is set
+by "worth_saving_limit". The default is that at least 10 entries must be
+updated in order to consider the dirstate file worth updating.
+
+Going one step further, newly added files, symlinks, and directory entries
+updates are treated specially. We know that we will always stat all
+entries in the tree so that we can observe *if* they have changed. In the
+case of directories, all the information we know about them is just from
+that stat value. There is no extra content to read. So an update directory
+entry doesn't cause us to update to IN_MEMORY_HASH_MODIFIED. However, if
+there are other modifications worth saving, we will go ahead and save the
+directory entry update at the same time.
+
+Similarly, symlink targets are commonly stored in the inode entry
+directly. So once we have stat'ed the symlink, we already have its target
+information in memory. The one caveat is if we used to think an object was
+a file, and it became a directory or symlink, then we will treat it as
+worth saving.
+
+In the case of newly added files, we never have to read their content to
+know that they are different from the basis tree. So saving the updated
+information also won't save a future read.
+
+
+.. vim: ft=rst tw=74 et

=== added file 'doc/developers/documenting-changes.txt'
--- doc/developers/documenting-changes.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/documenting-changes.txt	2010-10-14 21:23:08 +0000
@@ -0,0 +1,100 @@
+===================
+Documenting Changes
+===================
+
+When you change bzr, please update the relevant documentation for the
+change you made:
+
+Changing existing behavior
+  If the change will break existing command lines, or break
+  interoperability with other versions of Bazaar, mention this in 
+  "External Compatibility Breaks" in NEWS, and also in "What's New".
+
+Adding a new feature, function or option
+  Describe this in NEWS, in the user guide, and in the "What's New"
+  document.  Consider whether you should also update command help
+  or any help topics.
+
+A new hook or extension point
+  These are also described in NEWS, in the hook documentation, and in
+  "What's New."  Even though they might be API-only changes, the fact that 
+  they exist may interest users enough to write a new plugin that uses
+  them.
+
+Fixing a bug
+  If the bug has any user-visible impact, describe it in the NEWS file
+  in such a way that users can tell whether the problem they're
+  experiencing is the one that was fixed.
+
+Improving performance
+  Mention this under "improvements" in NEWS, and if it's a notable
+  improvement mention it in "What's New".
+
+Deprecating an API, or adding a new recommended API
+  Mention this in the "API Changes" of NEWS, if it's likely to affect
+  plugin authors.  Consider whether you should also update the developer
+  documentation.
+
+Changing the way the test suite works or adding more tests
+  Mention this under "Testing" in NEWS, and update the developer guide
+  to testing.
+
+Purely internal changes
+  If the change has no user-visible impact and is not interesting to 
+  plugin developers, it doesn't need to be mentioned in NEWS.  If you're 
+  setting a new pattern or adding a new abstraction, update the developer
+  docs.
+
+NEWS File
+---------
+
+If you make a user-visible change, please add a note to the NEWS file.
+The description should be written to make sense to someone who's just
+a user of bzr, not a developer: new functions or classes shouldn't be
+mentioned, but new commands, changes in behaviour or fixed nontrivial
+bugs should be listed.  See the existing entries for an idea of what
+should be done.
+
+Within each release, entries in the news file should have the most
+user-visible changes first.  So the order should be approximately:
+
+* changes to existing behaviour - the highest priority because the
+  user's existing knowledge is incorrect
+* new features - should be brought to their attention
+* bug fixes - may be of interest if the bug was affecting them, and
+  should include the bug number if any
+* major documentation changes, including fixed documentation bugs
+
+People who made significant contributions to each change are listed in
+parenthesis.  This can include reporting bugs (particularly with good
+details or reproduction recipes), submitting patches, etc.
+
+To help with merging, NEWS entries should be sorted lexicographically
+within each section.
+
+Commands
+--------
+
+The docstring of a command is used by ``bzr help`` to generate help output
+for the command. The list 'takes_options' attribute on a command is used by
+``bzr help`` to document the options for the command - the command
+docstring does not need to document them. Finally, the '_see_also'
+attribute on a command can be used to reference other related help topics.
+
+API Documentation
+-----------------
+
+Functions, methods, classes and modules should have docstrings
+describing how they are used.
+
+The first line of the docstring should be a self-contained sentence.
+
+For the special case of Command classes, this acts as the user-visible
+documentation shown by the help command.
+
+The docstrings should be formatted as reStructuredText_ (like this
+document), suitable for processing using the epydoc_ tool into HTML
+documentation.
+
+.. _reStructuredText: http://docutils.sourceforge.net/rst.html
+.. _epydoc: http://epydoc.sourceforge.net/

=== modified file 'doc/developers/ec2.txt'
--- doc/developers/ec2.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/ec2.txt	2010-12-16 06:24:14 +0000
@@ -74,7 +74,7 @@
 password from Amazon, and ``ec2-add-keypair`` creates a named keypair at
 Amazon and returns the private path. One keypair is all that is needed.
 This is distinct from the account identifier - likely due to the different
-toolchains in use (the keypairs are used for unix ssh keys, and I (Robert)
+toolchains in use (the keypairs are used for unix SSH keys, and I (Robert)
 suspect a rather unix friendly core at Amazon).
 Once a custom image is made with a saved password, you can skip using
 ``ec2-get-password`` (which is only needed for Windows anyway).
@@ -125,9 +125,9 @@
 * Install the rdesktop client, to actually access the machine.
 
 * Possibly read some of the `EC2 documentation`_ for background.
-
-.. _`bug 330930`: https://bugs.edge.launchpad.net/ubuntu/+bug/330930
-.. _`Amazon EC2 API tools`:
+   
+.. _`bug 330930`: https://bugs.launchpad.net/ubuntu/+bug/330930 
+.. _`Amazon EC2 API tools`: 
      http://developer.amazonwebservices.com/connect/entry.jspa?externalID=368&categoryID=88
 .. _`EC2 documentation`: http://aws.amazon.com/
 .. _`ami-bdfe19d4`:

=== added file 'doc/developers/fetch.txt'
--- doc/developers/fetch.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/fetch.txt	2011-06-09 07:38:32 +0000
@@ -0,0 +1,171 @@
+=============
+Fetching data
+=============
+
+Overview of a fetch
+===================
+
+Inside bzr, a typical fetch happens like this:
+
+* a user runs a command like ``bzr branch`` or ``bzr pull``
+
+* ``Repository.fetch`` is called (by a higher-level method such as
+  ``ControlDir.sprout``, ``Branch.fetch``, etc).
+
+* An ``InterRepository`` object is created.  The exact implementation of
+  ``InterRepository`` chosen depends on the format/capabilities of the
+  source and target repos.
+
+* The source and target repositories are compared to determine which data
+  needs to be transferred.
+
+* The repository data is copied.  Often this is done by creating a
+  ``StreamSource`` and ``StreamSink`` from the source and target
+  repositories and feeding the stream from the source into the sink, but
+  some ``InterRepository`` implementations do differently.
+
+
+How objects to be transferred are determined
+============================================
+
+See ``InterRepository._walk_to_common_revisions``.  The basic idea is to
+do a breadth-first search in the source repository's revision graph
+(starting from the head or heads the caller asked for), and look in the
+target repository to see if those revisions are already present.
+Eventually this will find the common ancestors in both graphs, and thus
+the set of revisions to be copied has been identified.
+
+All inventories for the copied revisions need to be present (and all
+parent inventories at the stacking boundary too, to support stacking).
+
+All texts versions introduced by those inventories need to be transferred
+(but see also stacking constraints).
+
+Fetch specs
+===========
+
+The most ``fetch`` methods accept a ``fetch_spec`` parameter.  This is how
+the caller controls what is fetched: e.g. all revisions for a given head
+(that aren't already present in the target), the full ancestry for one or
+more heads, or even the full contents of the source repository.
+
+The ``fetch_spec`` parameter is an object that implements the interface
+defined by ``AbstractSearchResult`` in ``bzrlib.graph``.  It describes
+which keys should be fetched.  Current implementations are
+``SearchResult``, ``PendingAncestryResult``, ``EmptySearchResult``, and
+``EverythingResult``.  Some have options controlling if missing revisions
+cause errors or not, etc.
+
+There are also some “search” objects, which can be used to conveniently
+construct a search result for common cases: ``EverythingNotInOther`` and
+``NotInOtherForRevs``.  They provide an ``execute`` method that performs
+the search and returns a search result.
+
+Also, ``Graph._make_breadth_first_searcher`` returns an object with a
+``get_result`` method that returns a search result.
+
+
+Streams
+=======
+
+A **stream** is an iterable of (substream type, substream) pairs.
+The **substream type** is a ``str`` that will be one of ``texts``,
+``inventories``, ``inventory-deltas``, ``chk_bytes``, ``revisions`` or
+``signatures``.  A **substream** is a record stream.  The format of those
+records depends on the repository format being streamed, except for
+``inventory-deltas`` records which are format-independent.
+
+A stream source can be constructed with ``repo._get_source(to_format)``,
+and it provides a ``get_stream(search)`` method (among others).  A stream
+sink can be constructed with ``repo._get_sink()``, and provides an
+``insert_stream(stream, src_format, resume_tokens)`` method (among
+others).
+
+
+Stacking constraints
+====================
+
+**In short the rule is:** "repositories must hold revisions' parent
+inventories and their new texts (or else all texts for those revisions)."
+
+This is sometimes called "the stacking invariant."
+
+Why that rule?
+--------------
+
+A stacked repository needs to be capable of generating a complete stream
+for the revisions it does hold without access to its fallback
+repositories [#]_.  "Complete" here means that the stream for a revision (or
+set of revisions) can be inserted into a repository that already contains
+the parent(s) of that revision, and that repository will have a fully
+usable copy of that revision: a working tree can be built for that
+revision, etc.
+
+Assuming for a moment the stream has the necessary inventory, signature
+and CHK records to have a usable revision, what texts are required to have
+a usable revision?  The simple way to satisfy the requirement is to have
+*every* text for every revision at the stacking boundary.  Thus the
+revisions at the stacking boundary and all their descendants have their
+texts present and so can be fully reconstructed.  But this is expensive:
+it implies each stacked repository much contain *O(tree)* data even for a
+single revision of a 1-line change, and also implies transferring
+*O(tree)* data to fetch that revision.
+
+Because the goal is a usable revision *when added to a repository with the
+parent revision(s)* most of those texts will be redundant.  The minimal
+set that is needed is just those texts that are new in the revisions in
+our repository.  However, we need enough inventory data to be able to
+determine that set of texts.  So to make this possible every revision must
+have its parent inventories present so that the inventory delta between
+revisions can be calculated, and of course the CHK pages associated with
+that delta.  In fact the entire inventory does not need to be present,
+just enough of it to find the delta (assuming a repository format, like
+2a, that allows only part of an inventory to be stored).  Thus the stacked
+repository can contain only *O(changes)* data [#]_ and still deliver
+complete streams of that data.
+
+What about revisions at the stacking boundary with more than one parent?
+All of the parent inventories must be present, as a client may ask for a
+stream up to any parent, not just the left-hand parent.  If any parent is
+absent then all texts must be present instead.  Otherwise there will be
+the strange situation where some fetches of a revision will succeed and
+others fail depending the precise details of the fetch.
+
+Implications for fetching
+-------------------------
+
+Fetches must retrieve the records necessary to satisfy that rule.  The
+stream source will attempt to send the necessary records, and the stream
+sink will check for any missing records and make a second fetch for just
+those missing records before committing the write group.
+
+Our repository implementations check this constraint is satisfied before
+committing a write group, to prevent a bad stream from creating a corrupt
+repository.  So a fetch from a bad source (e.g. a damaged repository, or a
+buggy foreign-format import) may trigger ``BzrCheckError`` during
+``commit_write_group``.
+
+To fetch from a stacked repository via a smart server, the smart client:
+
+* first fetches a stream of as many of the requested revisions as possible
+  from the initial repository,
+* then while there are still missing revisions and untried fallback
+  repositories fetches the outstanding revisions from the next fallback
+  until either all revisions have been found (success) or the list of
+  fallbacks has been exhausted (failure).
+
+
+.. [#] This is not just a theoretical concern.  The smart server always
+   opens repositories without opening fallbacks, as it cannot assume it
+   can access the fallbacks that the client can.
+
+.. [#] Actually *O(changes)* isn't quite right in practice.  In the
+   current implementation the fulltext of a changed file must be
+   transferred, not just a delta, so a 1-line change to a 10MB file will
+   still transfer 10MB of text data.  This is because current formats
+   require records' compression parents to be present in the same
+   repository.
+
+
+..
+   vim: ft=rst tw=74 ai

=== modified file 'doc/developers/groupcompress-design.txt'
--- doc/developers/groupcompress-design.txt	2009-04-08 16:33:19 +0000
+++ doc/developers/groupcompress-design.txt	2010-11-12 14:28:36 +0000
@@ -29,7 +29,7 @@
 
 Reasonable sizes 'amount read' from remote machines to reconstruct an arbitrary
 text: Reading 5MB for a 100K plain text is not a good trade off. Reading (say)
-500K is probably acceptable. Reading ~100K is ideal. However, its likely that
+500K is probably acceptable. Reading ~100K is ideal. However, it's likely that
 some texts (e.g NEWS versions) can be stored for nearly-no space at all if we
 are willing to have unbounded IO. Profiling to set a good heuristic will be
 important. Also allowing users to choose to optimise for a server environment

=== modified file 'doc/developers/incremental-push-pull.txt'
--- doc/developers/incremental-push-pull.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/incremental-push-pull.txt	2010-11-12 14:28:36 +0000
@@ -32,7 +32,7 @@
 1. New data is identified in the source repository.
 2. That data is read from the source repository.
 3. The same data is verified and written to the target repository in such a
-   manner that its not visible to readers until its ready for use.
+   manner that it's not visible to readers until it's ready for use.
 
 New data identification
 ~~~~~~~~~~~~~~~~~~~~~~~
@@ -119,7 +119,7 @@
    ghost points from the target; plus a set difference search is needed on
    signatures.
 
- * Semantic level can probably be tuned, but as its also complex I suggest
+ * Semantic level can probably be tuned, but as it's also complex I suggest
    deferring analysis for optimal behaviour of this use case.
 
 
@@ -180,7 +180,7 @@
 validate the contents of a revision we need the new texts in the inventory for
 the revision - to check a fileid:revisionid we need to know the expected sha1
 of the full text and thus also need to read the delta chain to construct the
-text as we accept it to determine if its valid. Providing separate validators
+text as we accept it to determine if it's valid. Providing separate validators
 for the chosen representation would address this.
 e.g: For an inventory entry FILEID:REVISIONID we store the validator of the
 full text :SHA1:. If we also stored the validator of the chosen disk

=== modified file 'doc/developers/index-plain.txt'
--- doc/developers/index-plain.txt	2010-07-16 20:52:54 +0000
+++ doc/developers/index-plain.txt	2010-10-13 04:13:48 +0000
@@ -15,7 +15,7 @@
   (external link)
   |--| automatically generated API reference information
 
-* `Integrating with Bazaar <http://bazaar-vcs.org/Integrating_with_Bazaar>`_
+* `Integrating with Bazaar <http://wiki.bazaar.canonical.com/Integrating_with_Bazaar>`_
   (wiki) |--| a guide for writing Python programs that work with Bazaar.
 
 * `Revision Properties <revision-properties.html>`_ |--| An application
@@ -31,6 +31,8 @@
 * `Writing plugins <http://doc.bazaar.canonical.com/plugins/en/plugin-development.html>`_
   |--| specific advice on writing Bazaar plugins. (web link)
 
+* `Documenting changes <documenting-changes.html>`_.
+
 Process
 =======
 
@@ -42,7 +44,7 @@
 
 * `Managing the Bazaar PPA <ppa.html>`_ |--| Packaging Bazaar for Ubuntu.
 
-* `Giving back <http://bazaar-vcs.org/BzrGivingBack>`_ (wiki) |--| How to get
+* `Giving back <http://wiki.bazaar.canonical.com/BzrGivingBack>`_ (wiki) |--| How to get
   your changes to Bazaar integrated into a release.
 
 * `Profiling notes <profiling.html>`_ |--| Instructions on how to profile

=== modified file 'doc/developers/index.txt'
--- doc/developers/index.txt	2010-07-16 20:52:54 +0000
+++ doc/developers/index.txt	2011-05-16 10:35:04 +0000
@@ -25,6 +25,7 @@
    testing
    code-review
    code-style
+   documenting-changes
 
 * `Contributing to Bazaar Documentation <http://wiki.bazaar.canonical.com/ContributingToTheDocs>`_ (wiki)
 
@@ -35,7 +36,10 @@
 .. toctree::
    :maxdepth: 1
 
+   configuration
+   fetch
    transports
+   ui
 
 Releasing and Packaging
 =======================
@@ -76,6 +80,14 @@
    miscellaneous-notes
 
 
+Licence
+============
+
+Copyright 2005-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later.  See
+<http://www.gnu.org/licenses/>.
+
 .. |--| unicode:: U+2014
 
 ..

=== modified file 'doc/developers/initial-push-pull.txt'
--- doc/developers/initial-push-pull.txt	2007-06-05 08:02:04 +0000
+++ doc/developers/initial-push-pull.txt	2010-11-12 22:08:18 +0000
@@ -65,5 +65,5 @@
 Wants
 -----
 - Read partial graph
-- Read multiple segments of multiple files on http and sftp
+- Read multiple segments of multiple files on HTTP and SFTP
 - Write multiple files over SFTP

=== modified file 'doc/developers/integration.txt'
--- doc/developers/integration.txt	2010-05-16 09:29:44 +0000
+++ doc/developers/integration.txt	2011-03-29 09:50:04 +0000
@@ -14,8 +14,38 @@
 Starting with bzrlib
 ====================
 
-Before doing anything else with bzrlib, you should run `bzrlib.initialize`
-which sets up some global state.  
+Within bzr
+----------
+
+When using bzrlib within the ``bzr`` program (for instance as a bzr
+plugin), bzrlib's global state is already available for use.
+
+From outside bzr
+----------------
+
+To use bzrlib outside of ``bzr`` some global state needs to be setup.
+bzrlib needs ways to handle user input, passwords, a place to emit
+progress bars, logging setup appropriately for your program. The easiest
+way to set all this up in the same fashion ``bzr`` does is to call
+``bzrlib.initialize``. 
+
+This returns a context manager within which bzrlib functions will work
+correctly. See the pydoc for ``bzrlib.initialize`` for more information. 
+(You can get away without entering the context manager, because the setup
+work happens directly from ``initialize``.)
+
+In Python 2.4 the ``with`` keyword is not supported and
+so you need to use the context manager manually::
+
+  # This sets up your ~/.bzr.log, ui factory and so on and so forth. It is
+  # not safe to use as a doctest.
+  library_state = bzrlib.initialize()
+  library_state.__enter__()
+  try:
+      pass
+      # do stuff here
+  finally:
+      library_state.__exit__(None, None, None)
 
 
 Running bzr commands
@@ -43,7 +73,7 @@
 
 
 This gives us a WorkingTree object, which has various methods spread over
-itself, and its parent classes MutableTree and Tree - its worth having a
+itself, and its parent classes MutableTree and Tree - it's worth having a
 look through these three files (workingtree.py, mutabletree.py and tree.py)
 to see which methods are available.
 

=== modified file 'doc/developers/inventory.txt'
--- doc/developers/inventory.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/inventory.txt	2010-11-12 22:08:18 +0000
@@ -26,7 +26,7 @@
 =============
 
 There are several variants of serialised tree shape in use by bzr. To date
-these have been mostly xml based, though plugins have offered non-xml versions.
+these have been mostly XML-based, though plugins have offered non-XML versions.
 
 dirstate
 --------
@@ -35,10 +35,10 @@
 for the working tree and one for each parent tree, interleaved to allow
 efficient diff and status operations.
 
-xml
+XML
 ---
 
-All the xml serialized forms write to and read from a single byte string, whose
+All the XML serialized forms write to and read from a single byte string, whose
 hash is then the inventory validator for the commit object.
 
 
@@ -82,14 +82,14 @@
 Current situation
 -----------------
 
-The xml based implementation we use today layers the inventory as a bytestring
+The XML-based implementation we use today layers the inventory as a bytestring
 which is stored under a single key; the bytestring is then compressed as a
 delta against the bytestring of its left hand parent by the knit code.
 
 Gap analysis:
 
  1. Succeeds
- 2. Fails - generating a new xml representation needs full tree data.
+ 2. Fails - generating a new XML representation needs full tree data.
  3. Succeeds - the inventory layer accesses the bytestring, which is
     deterministic
  4. Fails - we have to reconstruct both inventories as trees and then delta

=== modified file 'doc/developers/network-protocol.txt'
--- doc/developers/network-protocol.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/network-protocol.txt	2010-11-12 22:46:28 +0000
@@ -317,7 +317,7 @@
 
   (Discussion) We're discussing having the byte segments be not just a
   method for sending a stream across the network, but actually having them
-  be preserved in the rpc from end to end.  This may be useful when
+  be preserved in the RPC from end to end.  This may be useful when
   there's an iterator on one side feeding in to an iterator on the other,
   if it avoids doing chunking and byte-counting at two levels, and if
   those iterators are a natural place to get good granularity.  Also, for
@@ -369,8 +369,8 @@
   sent.
 
   This relies on the client being able to read back from the server while
-  it's writing.  This is technically difficult for http but feasible over
-  a socket or ssh.
+  it's writing.  This is technically difficult for HTTP but feasible over
+  a socket or SSH.
 
   We'd need a clean way to pass this back to the request method, even
   though it's presumably in the middle of doing its body iterator.
@@ -400,8 +400,8 @@
   happening out of order, or mixed requests happening at the same time.
 
   Wonder how our network performance would have turned out now if we'd
-  done full-duplex from the start, and ignored hpss over http.  We have
-  pretty good (readonly) http support just over dumb http, and that may be
+  done full-duplex from the start, and ignored hpss over HTTP.  We have
+  pretty good (read-only) HTTP support just over dumb HTTP, and that may be
   better for many users.
 
 
@@ -422,9 +422,9 @@
 that includes any repository that might need to be referenced, and the
 client needs to know about a root directory beyond which it cannot ascend.
 
-Servers run over ssh will typically want to be able to access any path the
+Servers run over SSH will typically want to be able to access any path the
 user can access.  Public servers on the other hand (which might be over
-http, ssh or tcp) will typically want to restrict access to only a
+HTTP, SSH or TCP) will typically want to restrict access to only a
 particular directory and its children, so will want to do a software
 virtual root at that level.  In other words they'll want to rewrite
 incoming paths to be under that level (and prevent escaping using ../

=== modified file 'doc/developers/overview.txt'
--- doc/developers/overview.txt	2010-06-21 03:55:08 +0000
+++ doc/developers/overview.txt	2011-05-17 13:01:06 +0000
@@ -5,44 +5,45 @@
 This document describes the key classes and concepts within Bazaar.  It is
 intended to be useful to people working on the Bazaar codebase, or to
 people writing plugins.  People writing plugins may also like to read the 
-guide to `Integrating with Bazaar <integrating.html>`_ for some specific
-recipes.
+guide to `Integrating with Bazaar <integration.html>`_ for some specific recipes.
+
+There's some overlap between this and the `Core Concepts`_ section of the
+user guide, but this document is targetted to people interested in the
+internals.  In particular the user guide doesn't go any deeper than
+"revision", because regular users don't care about lower-level details
+like inventories, but this guide does.
 
 If you have any questions, or if something seems to be incorrect, unclear
-or missing, please talk to us in ``irc://irc.freenode.net/#bzr``, or write
-to the Bazaar mailing list.  
-
-
-Using bzrlib
+or missing, please talk to us in ``irc://irc.freenode.net/#bzr``, write to
+the Bazaar mailing list, or simply file a bug report.
+
+
+IDs and keys
 ############
 
-Within bzr
-==========
-
-When using bzrlib within the ``bzr`` program (for instance as a bzr
-plugin), bzrlib's global state is already available for use.
-
-From outside bzr
-================
-
-To use bzrlib outside of ``bzr`` some global state needs to be setup.
-bzrlib needs ways to handle user input, passwords, a place to emit
-progress bars, logging setup appropriately for your program. The easiest
-way to set all this up in the same fashion ``bzr`` does is to call
-``bzrlib.initialize``. This returns a context manager within which bzrlib
-functions will work correctly. See the pydoc for ``bzrlib.initialize`` for
-more information. In Python 2.4 the ``with`` keyword is not supported and
-so you need to use the context manager manually::
-
-  # This sets up your ~/.bzr.log, ui factory and so on and so forth. It is
-  # not safe to use as a doctest.
-  library_state = bzrlib.initialize()
-  library_state.__enter__()
-  try:
-      pass
-      # do stuff here
-  finally:
-      library_state.__exit__(None, None, None)
+IDs
+===
+
+All IDs are globally unique identifiers.  Inside bzrlib they are almost
+always represented as UTF-8 encoded bytestrings (i.e. ``str`` objects).
+
+The main two IDs are:
+
+:Revision IDs: The unique identifier of a single revision, such as
+  ``pqm@pqm.ubuntu.com-20110201161347-ao76mv267gc1b5v2``
+:File IDs: The unique identifier of a single file.  It is allocated when
+  a user does ``bzr add`` and is unchanged by renames.
+
+By convention, in the bzrlib API, parameters of methods that are expected
+to be IDs (as opposed to keys, revision numbers, or some other handle)
+will end in ``id``, e.g.  ``revid`` or ``file_id``.
+
+Keys
+====
+
+A composite of one or more ID elements.  E.g. a (file-id, revision-id)
+pair is the key to the "texts" store, but a single element key of
+(revision-id) is the key to the "revisions" store.
 
 
 Core classes
@@ -98,6 +99,28 @@
 elsewhere.  Information that Transports return, such as from ``list_dir``,
 is also in the form of URL components.
 
+More information
+----------------
+
+See also:
+
+* `Developer guide to bzrlib transports <transports.html>`_ 
+* API docs for ``bzrlib.transport.Transport``
+
+Tree
+====
+
+A representation of a directory of files (and other directories and
+symlinks etc).  The most important kinds of Tree are:
+
+:WorkingTree: the files on disk editable by the user
+:RevisionTree: a tree as recorded at some point in the past
+
+Trees can map file paths to file-ids and vice versa (although trees such
+as WorkingTree may have unversioned files not described in that mapping).
+Trees have an inventory and parents (an ordered list of zero or more
+revision IDs).
+
 
 WorkingTree
 ===========
@@ -107,19 +130,20 @@
 
 Responsibilities:
 
- * Maintaining a WorkingTree on disk at a file path.
- * Maintaining the basis inventory (the inventory of the last commit done)
- * Maintaining the working inventory.
- * Maintaining the pending merges list.
- * Maintaining the stat cache.
- * Maintaining the last revision the working tree was updated to.
- * Knows where its Branch is located.
+* Maintaining a WorkingTree on disk at a file path.
+* Maintaining the basis inventory (the inventory of the last commit done)
+* Maintaining the working inventory.
+* Maintaining the pending merges list.
+* Maintaining the stat cache.
+* Maintaining the last revision the working tree was updated to.
+* Knows where its Branch is located.
 
 Dependencies:
 
- * a Branch
- * an MutableInventory
- * local access to the working tree
+* a Branch
+* an MutableInventory
+* local access to the working tree
+
 
 Branch
 ======
@@ -129,15 +153,18 @@
 
 A Branch is responsible for:
 
- * Holding user preferences that are set in a Branch.
- * Holding the 'tip': the last revision to be committed to this Branch. (And the revno of that revision.)
- * Knowing how to open the Repository that holds its history.
- * Allowing write locks to be taken out to prevent concurrent alterations to the branch.
+* Holding user preferences that are set in a Branch.
+* Holding the 'tip': the last revision to be committed to this Branch.
+  (And the revno of that revision.)
+* Knowing how to open the Repository that holds its history.
+* Allowing write locks to be taken out to prevent concurrent alterations to the branch.
 
 Depends on:
- * URL access to its base directory.
- * A Transport to access its files.
- * A Repository to hold its history.
+
+* URL access to its base directory.
+* A Transport to access its files.
+* A Repository to hold its history.
+
 
 Repository
 ==========
@@ -146,16 +173,17 @@
 and graph relationships between them.  A repository holds a bag of
 revision data that can be pointed to by various branches:
 
- * Maintains storage of various history data at a URL:
-   * Revisions (Must have a matching inventory)
-   * Digital Signatures
-   * Inventories for each Revision. (Must have all the file texts available).
-   * File texts
+* Maintains storage of various history data at a URL:
+  
+  * Revisions (Must have a matching inventory)
+  * Digital Signatures
+  * Inventories for each Revision. (Must have all the file texts available).
+  * File texts
 
- * Synchronizes concurrent access to the repository by different
-   processes.  (Most repository implementations use a physical 
-   mutex only for a short period, and effectively support multiple readers
-   and writers.)
+* Synchronizes concurrent access to the repository by different
+  processes.  (Most repository implementations use a physical mutex only
+  for a short period, and effectively support multiple readers and
+  writers.)
 
 Stacked Repositories
 --------------------
@@ -184,5 +212,140 @@
 server exposes the stacked-on URL and the client can open that.
 
 
+Storage model
+#############
+
+This section describes the model for how bzr stores its data.  The
+representation of that data on disk varies considerable depending on the
+format of the repository (and to a lesser extent the format of the branch
+and working tree), but ultimately the set of objects being represented is
+the same.
+
+Branch
+======
+
+A branch directly contains:
+
+* the ID of the current revision that branch (a.k.a. the “tip”)
+* some settings for that branch (the values in “branch.conf”)
+* the set of tags for that branch (not supported in all formats)
+
+A branch implicitly references:
+
+* A repository.  The repository might be colocated in the same directory
+  as the branch, or it might be somewhere else entirely.
+
+
+Repository
+==========
+
+A repository contains:
+
+* a revision store
+* an inventory store
+* a text store
+* a signature store
+
+A store is a key-value mapping.  This says nothing about the layout on
+disk, just that conceptually there are distinct stores, each with a
+separate namespace for the keys.  Internally the repository may serialize
+stores in the same file, and/or e.g. apply compression algorithms that
+combine records from separate stores in one block, etc.
+
+You can consider the repository as a single key space, with keys that look
+like *(store-name, ...)*.  For example, *('revisions',
+revision-id)* or *('texts', revision-id, file-id)*.
+
+Revision store
+--------------
+
+Stores revision objects.  The keys are GUIDs.  The value is a revision
+object (the exact representation on disk depends on the repository
+format).
+
+As described in `Core Concepts`_ a revision describes a snapshot of the
+tree of files and some metadata about them.
+
+* metadata:
+
+  * parent revisions (an ordered sequence of zero or more revision IDs)
+  * commit message
+  * author(s)
+  * timestamp
+  * (and all other revision properties)
+
+* an inventory ID (that inventory describes the tree contents).  Is often
+  the same as the revision ID, but doesn't have to be (e.g. if no files
+  were changed between two revisions then both revisions will refer to
+  the same inventory).
+
+
+Inventory store
+---------------
+
+Stores inventory objects.  The keys are GUIDs.  (Footnote: there will
+usually be a revision with the same key in the revision store, but there
+are rare cases where this is not true.)
+
+An inventory object contains:
+
+* a set of inventory entries
+
+An inventory entry has the following attributes
+
+* a file-id (a GUID, or the special value TREE_ROOT for the root entry of
+  inventories created by older versions of bzr)
+* a revision-id, a GUID (generally corresponding to the ID of a
+  revision).  The combination of (file-id, revision-id) is a key into the
+  texts store.
+* a kind: one of file, directory, symlink, tree-reference (tree-reference
+  is only supported in unsupported developer formats)
+* parent-id: the file-id of the directory that contains this entry (this
+  value is unset for the root of the tree).
+* name: the name of the file/directory/etc in that parent directory
+* executable: a flag indicating if the executable bit is set for that
+  file.
+
+An inventory entry will have other attributes, depending on the kind:
+
+* file:
+
+  * SHA1
+  * size
+
+* directory
+
+  * children
+
+* symlink
+
+  * symlink_target
+
+* tree-reference
+
+  * reference_revision
+
+For some more details see `Inventories <inventory.html>`_.
+
+
+Texts store
+-----------
+
+Stores the contents of individual versions of files.  The keys are pairs
+of (file-id, revision-id), and the values are the full content (or
+"text") of a version of a file.
+
+For consistency/simplicity text records exist for all inventory entries,
+but in general only entries with of kind "file" have interesting records.
+
+
+Signature store
+---------------
+
+Stores cryptographic signatures of revision contents.  The keys match
+those of the revision store.
+
+.. _Core Concepts: http://doc.bazaar.canonical.com/latest/en/user-guide/core_concepts.html
+
 ..
    vim: ft=rst tw=74 ai

=== modified file 'doc/developers/packrepo.txt'
--- doc/developers/packrepo.txt	2007-12-05 19:55:07 +0000
+++ doc/developers/packrepo.txt	2010-11-12 22:08:18 +0000
@@ -138,7 +138,7 @@
 
 If you need any help or encounter any problems, please contact the developers
 via the usual ways, i.e. chat to us on IRC or send a message to our mailing
-list. See http://bazaar-vcs.org/BzrSupport for contact details.
+list. See http://wiki.bazaar.canonical.com/BzrSupport for contact details.
 
 
 Technical notes
@@ -250,7 +250,7 @@
 
 The ``pack-names`` file gives the list of all finished non-obsolete
 packs.  (This should always be the same as the list of files in the
-``packs/`` directory, but the file is needed for readonly http clients
+``packs/`` directory, but the file is needed for read-only HTTP clients
 that can't easily list directories, and it includes other information.)
 The constraint on the ``pack-names`` list is that every file mentioned
 must exist in the ``packs/`` directory.

=== modified file 'doc/developers/performance-roadmap-rationale.txt'
--- doc/developers/performance-roadmap-rationale.txt	2007-06-05 08:02:04 +0000
+++ doc/developers/performance-roadmap-rationale.txt	2010-11-12 14:28:36 +0000
@@ -109,7 +109,7 @@
 of great assistance I think. We want to help everyone that wishes to
 contribute to performance to do so effectively.
 
-Finally, its important to note that coding is not the only contribution
+Finally, it's important to note that coding is not the only contribution
 - testing, giving feedback on current performance, helping with the
 analysis are all extremely important tasks too and we probably want to
 have clear markers of where that should be done to encourage such

=== modified file 'doc/developers/performance-use-case-analysis.txt'
--- doc/developers/performance-use-case-analysis.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/performance-use-case-analysis.txt	2010-11-12 14:28:36 +0000
@@ -38,7 +38,7 @@
 be constant time. Retrieval of the annotated text should be roughly
 constant for any text of the same size regardless of the number of
 revisions contributing to its content. Mapping of the revision ids to
-dotted revnos could be done as the text is retrieved, but its completely
+dotted revnos could be done as the text is retrieved, but it's completely
 fine to post-process the annotated text to obtain dotted-revnos.'
 
 What factors should be considered?
@@ -79,7 +79,7 @@
 - locality of reference: If an operation requires data that is located
   within a small region at any point, we often get better performance
   than with an implementation of the same operation that requires the
-  same amount of data but with a lower locality of reference. Its
+  same amount of data but with a lower locality of reference. It's
   fairly tricky to add locality of reference after the fact, so I think
   its worth considering up front.
 
@@ -102,8 +102,8 @@
 file-level operation latency considerations.
 
 Many performance problems only become visible when changing the scaling knobs
-upwards to large trees. On small trees its our baseline performance that drives
-incremental improvements; on large trees its the amount of processing per item
+upwards to large trees. On small trees it's our baseline performance that drives
+incremental improvements; on large trees it's the amount of processing per item
 that drives performance. A significant goal therefore is to keep the amount of
 data to be processed under control. Ideally we can scale in a sublinear fashion
 for all operations, but we MUST NOT scale even linearly for operations that

=== modified file 'doc/developers/planned-change-integration.txt'
--- doc/developers/planned-change-integration.txt	2010-06-02 05:03:31 +0000
+++ doc/developers/planned-change-integration.txt	2010-11-12 14:28:36 +0000
@@ -66,7 +66,7 @@
 
  * Network-efficient revision graph API: This influences what questions we will
    want to ask a local repository very quickly; as such it's a driver for the
-   new repository format and should be in place first if possible. Its probably
+   new repository format and should be in place first if possible. It's probably
    not sufficiently different to local operations to make this a hard ordering
    though.
 
@@ -91,14 +91,14 @@
  * Repository stacking API: The key dependency/change required for this is that
    repositories must individually be happy with having partial data - e.g. many
    ghosts. However the way the API needs to be used should be driven from the
-   command layer in, because its unclear at the moment what will work best.
+   command layer in, because it's unclear at the moment what will work best.
 
  * Revision stream API: This API will become clear as we streamline commands.
    On the data insertion side commit will want to generate new data. The
    commands pull, bundle, merge, push, possibly uncommit will want to copy
    existing data in a streaming fashion.
 
- * New container format: Its hard to tell what the right way to structure the
+ * New container format: It's hard to tell what the right way to structure the
    layering is. Probably having smooth layering down to the point that code
    wants to operate on the containers directly will make this more clear. As
    bundles will become a read-only branch & repository, the smart server wants
@@ -135,7 +135,7 @@
    today, it is likely able to be implemented immediately, but we are not sure
    that its needed anymore, so it is being shelved.
 
- * Removing derivable data: Its very hard to do this while the derived data is
+ * Removing derivable data: It's very hard to do this while the derived data is
    exposed in API's but not used by commands. Implemented the targeted API's
    for our core use cases should allow use to remove accidental use of derived
    data, making only explicit uses of it visible, and isolating the impact of

=== modified file 'doc/developers/planned-performance-changes.txt'
--- doc/developers/planned-performance-changes.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/planned-performance-changes.txt	2010-11-12 14:28:36 +0000
@@ -43,7 +43,7 @@
    without changing disk, possibly with a performance hit until the disk
    formats match the validatory logic. It will be hard to tell if we have the
    right routine for that until all the disk changes are complete, so while
-   this is a library only change, its likely one that will be delayed to near
+   this is a library only change, it's likely one that will be delayed to near
    the end of the process.
 
  * Add an explicit API for managing cached annotations. While annotations are
@@ -68,7 +68,7 @@
    through review.
 
  * Stop requiring full memory copies of files. Currently bzr requires that it
-   can hold 3 copies of any file its versioning in memory. Solving this is
+   can hold 3 copies of any file it's versioning in memory. Solving this is
    tricky, particularly without performance regressions on small files, but
    without solving it versioning of .iso and other large objects will continue
    to be extremely painful.
@@ -84,7 +84,7 @@
  * Revision data manipulation API. We need a single streaming API for adding
    data to or getting it from a repository. This will need to allow hints such
    as 'optimise for size', or 'optimise for fast-addition' to meet the various
-   users planned, but it is a core part of the library today, and its not
+   users planned, but it is a core part of the library today, and it's not
    sufficiently clean to let us simplify/remove a lot of related code today.
 
 Interoperable disk changes
@@ -104,7 +104,7 @@
 
  * Repository disk operation ordering. The order that tasks access data within
    the repository and the layout of the data should be harmonised. This will
-   require disk format changes but does not inherently alter the model, so its
+   require disk format changes but does not inherently alter the model, so it's
    straight forward to export from a repository that has been optimised in this
    way to a 0.16 based repository.
 
@@ -120,13 +120,6 @@
    between two historical trees much more efficiently than the current
    repositories.
 
- * Delta storage optimisation. We plan to change the delta storage logic to use
-   a binary delta like xdelta rather than using line based deltas from python.
-   These binary deltas could be done along ancestry ordering, or other
-   arbitrary patterns chosen for their intended use. Line based deltas will
-   still be created for cached annotations. This is still under some discussion.
-   http://bazaar-vcs.org/PerformanceRoadmap/Xdelta
-
  * Greatest distance from origin cache. This is a possible change to introduce,
    but it may be unnecessary - listed here for completeness till it has been
    established as [un]needed.

=== modified file 'doc/developers/ppa.txt'
--- doc/developers/ppa.txt	2010-07-13 17:14:43 +0000
+++ doc/developers/ppa.txt	2011-02-01 01:34:58 +0000
@@ -7,62 +7,99 @@
 Background
 ----------
 
-We build Ubuntu ``.deb`` packages for Bazaar as an important part of the release
-process.  These packages are hosted in a few `Personal Package Archives (PPA)`__ on
-Launchpad.
+We build Ubuntu ``.deb`` packages for Bazaar as an important part of the
+release process.  These packages are hosted in a few `Personal Package
+Archives (PPA)`__ on Launchpad.
 
   __ https://help.launchpad.net/PPAQuickStart
 
-As of June 2010, there are three PPAs:
+As of January 2011, there are the following PPAs:
 
-<https://launchpad.net/~bzr/+archive>
+<https://launchpad.net/~bzr/+archive/ppa>
     Final released versions and updates.
-
-<https://launchpad.net/~bzr-beta-ppa/+archive>
+    Most users who want updates to bzr should add this.
+
+<https://launchpad.net/~bzr/+archive/proposed>
+    Proposed uploads to move into ~bzr/ppa, awaiting testing.
+
+<https://launchpad.net/~bzr/+archive/obsolete>
+    A preserved copy of the final version of packages from ~bzr/ppa for
+    obsolete Ubuntu series.
+
+<https://launchpad.net/~bzr/+archive/beta>
     Beta releases.
 
-<https://launchpad.net/~bzr-nightly-ppa/+archive>
+<https://launchpad.net/~bzr/+archive/beta-obsolete>
+    A preserved copy of the final version of packages from
+    ~bzr/beta for obsolete Ubuntu series.
+
+<https://launchpad.net/~bzr/+archive/daily>
     Automatic nightly builds from trunk.
 
-We build packages for every supported Ubuntu release
-<https://wiki.ubuntu.com/Releases>.  Packages need no longer be updated
-when the release passes end-of-life because all users should
-have upgraded by then.  
-
-We build a distinct package for each distrorelease.  
+We build a distinct package for each distrorelease.
 If you upload a release-specific version, you should add a suffix to the
-package version, e.g. ``bzr.1.3-1~bazaar1~dapper1``.
+package version, e.g. ``1.3-1~bazaar1~dapper1``.
 
 Dapper uses the ``python-support`` framework and later distributions use
 ``python-central``.  This has little effect on everyday packaging but does
 mean that some of the control files are quite different.
 
-Every package is first uploaded into the beta ppa.  For final release
-versions it is also copied to the main PPA.
+Beta releases of bzr and plugins are uploaded into the beta PPA.
+
+Final release versions are first uploaded into the proposed PPA, which
+serves as a staging area to allow for new packages to be tested, and also
+so that a complete set of Bazaar core and plugin updated versions can be
+prepared together, when negotiating an API version transition.
+
+Once ready, packages can be copied from the proposed PPA to the main PPA
+using the lp-promote-ppa script found within the hydrazine project.  This
+procedure reduces the risk of broken packages or dependencies between
+packages in the main PPA from which many people get bzr updates.
 
 The packaging information is kept in branches of bzr on Launchpad, named
 like
-<https://code.launchpad.net/~bzr/bzr/packaging-hardy>.
+<https://code.launchpad.net/~bzr/ubuntu/hardy/bzr/bzr-ppa>.
 or
-<lp:~bzr/bzr/packaging-hardy>.  These branches are intended to be used
+<lp:~bzr/ubuntu/hardy/bzr/bzr-ppa>.  These branches are intended to be used
 with the ``bzr-builddeb`` plugin.
 
-**You should almost always upload to the beta ppa first** and then either 
-upload again or copy the packages into the release ppa.  That reduces the 
-risk of breaking the main archive from which people get bzr updates.
+The page <http://wiki.bazaar.canonical.com/PpaPackagingBranches> is a
+reference to where the PPA packaging branches for each of the source
+packages in the ``~bzr`` PPAs may be found.
+
+
+Supported releases
+------------------
+
+We build packages for every supported Ubuntu release
+<https://wiki.ubuntu.com/Releases>.  Packages need no longer be updated
+when the release passes end-of-life because all users should
+have upgraded by then.  
+
+As of August 2010, the following releases are supported:
+
+* Maverick 
+* Lucid LTS
+* Karmic
+* Jaunty (support ends October 2010)
+* Hardy LTS
+* Dapper LTS (supported but no longer updated for new releases)
+
+The ``rmadison bzr`` command will gives you an up-to-date summary
+of which bzr releases are current in each Ubuntu release.
 
 Preconditions
 -------------
 
-* You must have a Launchpad account and be a member of the teams
-  that own these PPAs (``~bzr``, ``~bzr-beta-ppa``).
+* You must have a Launchpad account and be a member of the team
+  that owns these PPAs (``~bzr``).
 
 * You must have a GPG key registered to your Launchpad account.
 
 On reasonably recent versions of Ubuntu you no longer need special dput
 configuration, because you can just say ::
 
-  dput ppa:bzr/2.1-proposed <source.changes
+  dput ppa:bzr/proposed source.changes
   
 
 However, you may still want to add these lines to ``~/.dput.cf`` prevent 
@@ -72,13 +109,6 @@
     [DEFAULT]
     default_host_main = notspecified
 
-* Configure ``bzr-builddeb`` to sign the package, which is required for
-  Launchpad to build it.  Put this in ``~/.bazaar/builddeb.conf`` ::
-
-      [BUILDDEB]
-      builder = dpkg-buildpackage -rfakeroot
-      source-builder= dpkg-buildpackage -rfakeroot -S -sa
-
 * You need a Ubuntu (or probably Debian) machine, and ::
 
     sudo apt-get install build-essential devscripts dput quilt patch libcrypt-ssleay-perl debhelper cdbs python-docutils
@@ -86,11 +116,9 @@
   Please update this document if you encounter unmet dependencies or find a
   shorter way to express them.
 
-* You will also want to have the `bzr-builddeb`_ plugin installed, which
-  depends on `bzrtools`_.
+* You will also want to have the `bzr-builddeb`_ plugin installed.
 
 .. _`bzr-builddeb`: http://launchpad.net/bzr-builddeb
-.. _`bzrtools`: http://launchpad.net/bzrtools
 
 
 Packaging Bazaar
@@ -105,7 +133,7 @@
   that into the ppa.  
   
   (``-S`` says to make a source-only upload, which is
-  required for Launchpad's builders.  ``--sa`` says to include the
+  required for Launchpad's builders.  ``-sa`` says to include the
   ``.orig.tgz`` even if this doesn't seem to be the first upload for an
   upstream release: this is often needed when rebuilding something that's
   previously been uploaded to Debian or Ubuntu or into a different PPA.)
@@ -113,12 +141,20 @@
 * Now merge across that change into each supported branch with a 
   simple ``bzr merge``.
   
-Locally testing builds
-~~~~~~~~~~~~~~~~~~~~~~
+Locally testing using pbuilder
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 It may be useful to locally test builds inside pbuilder.  You may want to 
 use the script from <http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=255165> 
-to wrap it.
+to wrap it, and to give it sensible defaults for your local machine.
+
+Update all packages in proposed before copying to the main ppa
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+If one updates bzr, and there are plugins that are not compatible with the
+new version of bzr, this can cause pain for users using the ppa. In order to
+avoid this, we first get all packages up to date in the proposed ppa, and then
+copy them to the main ppa.
 
 
 Short form
@@ -132,10 +168,11 @@
   export PACKAGE="bzr"
   export UBUNTU_RELEASES="dapper hardy intrepid jaunty karmic"
   ~/dev/bzr/bzr.dev/tools/packaging/update-packaging-branches.sh
+  * Optionaly merge debian unstable. 
   ~/dev/bzr/bzr.dev/tools/packaging/update-changelogs.sh
   ~/dev/bzr/bzr.dev/tools/packaging/update-control.sh 1.16 1.17 1.18
   ~/dev/bzr/bzr.dev/tools/packaging/build-packages.sh
-  dput bzr-beta-ppa ${PACKAGE}_$VERSION*.changes
+  dput ppa:bzr/proposed ${PACKAGE}_$VERSION*.changes
 
 Rinse and repeat for all the plugins by changing VERSION and PACKAGE.
 
@@ -177,12 +214,16 @@
 
 #. Checkout (or update) the packaging branch for each supported release::
 
-      bzr co lp:~bzr/bzr/packaging-hardy
+      bzr co lp:~bzr/ubuntu/hardy/bzr/bzr-ppa
 
    There is a script available to help::
 
       tools/packaging/update-packaging-branches.sh
 
+#. Optionaly, merge the Debian unstable branch into each of the packaging
+   branches. You can find the Debian unstable branch here:
+   http://bzr.debian.org/pkg-bazaar/
+
 #. The ``bzr-builddeb`` step will download the original tarball if you do
    not already have it, putting it into a ``tarballs`` directory.
 
@@ -238,7 +279,7 @@
 
 #. Build the source packages::
 
-      cd packaging-$DISTRO; bzr builddeb -S
+      cd bzr-$DISTRO; bzr builddeb -S
 
    This will create a ``.changes`` file.  If you didn't configure builddeb
    to automatically sign them, you can use ::
@@ -252,21 +293,14 @@
 
       tools/packaging/build-packages.sh
 
-#. Upload into the beta PPA for each release::
-
-     dput bzr-beta-ppa bzr*1.17-1*.changes
-
-#. For final release versions, also copy it into the ``~bzr`` PPA::
-
-     dput bzr-ppa ../bzr_1.17-1\~bazaar1\~hardy1\_source.changes
-
-   Alternatively, you can use Launchpad's "copy" feature to copy the
-   packages between repositories.
+#. Upload into the PPA for each release::
+
+     dput dput ppa:bzr/proposed bzr*1.17-1*.changes
 
 #. You should soon get an "upload accepted" mail from Launchpad, which
    means that your package is waiting to be built.  You can then track its
-   progress in <https://launchpad.net/~bzr-beta-ppa/+archive> and
-   <https://launchpad.net/~bzr-beta-ppa/+archive/+builds>.
+   progress in <https://launchpad.net/~bzr/+archive/proposed> and
+   <https://launchpad.net/~bzr/+archive/proposed/+builds>.
 
 
 Packaging bzr-svn
@@ -293,7 +327,7 @@
    bzr-builddeb will automatically check out the appropriate tag from the
    main branch of bzr-svn, build, and package it.
 
-#. ``dput bzr-beta-ppa ../bzr-svn_0.4.15-1~bazaar1~hardy1_source.changes``
+#. ``dput ppa:bzr/proposed ../bzr-svn_0.4.15-1~bazaar1~hardy1_source.changes``
 
 
 Monitoring the contents of PPAs
@@ -305,7 +339,20 @@
   apt-cache madison bzr
   
   
+Testing the contents of the PPA
+-------------------------------
+
+A somewhat crude but useful way to test the contents of the PPA is to
+install the relevant packages into an schroot::
+
+ schroot -c hardy-test -u root -- \
+   apt-get install -o 'APT::Install-Suggests="true"' \
+   -o 'APT::Install-Recommends="true"' \
+   bzr
+
+This should make sure everything can be installed; it won't guarantee that 
   
+
 Packaging dependencies
 ----------------------
 
@@ -313,6 +360,7 @@
 dependencies.  Specific branches holding these backports:
 
  * ``lp:~bzr/ubuntu/dapper/configobj/dapper-backport``
+ * ``lp:~bzr/ubuntu/hardy/python-central-debhelper-sequence-addon/bzr-ppa``
 
 
 ..

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2010-05-27 19:54:56 +0000
+++ doc/developers/releasing.txt	2011-10-06 11:28:16 +0000
@@ -3,77 +3,177 @@
 
 This document describes the processes for making and announcing a Bazaar
 release, and managing the release process.  This is just one phase of the
-`overall development cycle <http://doc.bazaar-vcs.org/developers/cycle.html>`_,
-but it's the most complex part.
-This document gives a checklist you can follow from start to end in one
-go.
+`overall development cycle
+<http://doc.bazaar.canonical.com/developers/cycle.html>`_, (go re-read this
+document to ensure it hasn't been updated since you last read it) but it's
+the most complex part.
+
+If you're doing your first release you can follow this document and read
+each step explanation. It's also a good practice to read it for any release
+to ensure you don't miss a step and to update it as the release process
+evolves.
 
 If you're helping the Release Manager (RM) for one reason or another, you
 may notice that he didn't follow that document scrupulously. He may have
 good reasons to do that but he may also have missed some parts.
 
-Follow the document yourself and don't hesitate to create the missing
-milestones for example (we tend to forget these ones a lot).
-
 .. contents::
 
 
 Preconditions
 =============
 
+#. PQM access rights (or you won't be able to land any change)
+
 #. Download the pqm plugin and install it into your ``~/.bazaar/plugins``::
 
      bzr branch lp:bzr-pqm ~/.bazaar/plugins/pqm
 
+#. Alternatively, you can download and install ``lp:hydrazine`` (the main
+   difference is that hydrazine requires the branch to land to be hosted on
+   launchpad).
+
+What do we release
+==================
+
+In this document, we're talking about source releases only, packages and
+installers are built from this but we won't talk about them here.
+
+Every release is part of a series, ``bzr-2.4.1`` is part of series ``2.4``.
+
+We do two different kind of releases: the betas releases and the stable
+releases for a given series.
+
+For a given series, releases will be done to deliver new versions of bzr to
+different kinds of users:
+
+#. beta releases: named ``x.ybn`` where ``x.y`` is the series and ``n``
+   starts at 1 and is incremented. These releases are targeted to beta
+   testers who don't want to run from source but are interested in features
+   or improvements.
+
+#. stable releases: name ``x.y.z`` where ``x.y.`` is the series and ``z``
+   starts at 1 and is incremented. These releases are targeted at people
+   that want bugfixes only and no new features.
+
+
+Differences in the release process between beta and stable release will be
+mentioned when needed.
+
+When do we relase ?
+===================
+
+As of July 2011, we maintain four series (and one that is about to be EOLed).
+Concurrently releasing them all at the same time makes it harder to shorten
+the delay between the source availability and the package building longer
+than necessary (we delay the official announcement until most of our users
+can install the new release).
+
+In order to continue to do time-based releases, we need to plan the
+releases by series to minimize the collisions. In the end, it's the Release
+Manager call to decide whether he prefers to do all releases at once
+though, so the rules presented here are a conservative approach.
+
+We want to respect the following rules:
+
+#. as much as possible releases should not disturb development, and
+   ongoing development should not disturb releases,
+
+#. the most recent development series should release once a month during
+   the beta period (see `Development cycles <cycle.html>`_ for more
+   details),
+
+#. the most recent stable series should release every other month (based
+   on the amount of bug fixes, this can be shorter or longer depending on
+   the bugs importance),
+
+#. previous series should release on a regular basis without interfering
+   with the most recent series with a decreasing order of priority (again
+   this should be based on bugs importance and user feedback),
+
+#. the death of a series should be planned ahead of time. 6 months should
+   give enough time to our users to migrate to a more recent series. This
+   doesn't mean we will make a release at the end of the series, just that
+   before the end date we *could* possibly put out another release if
+   there was a sufficiently important fix.  Beyond that date, we won't
+   even land changes on that branch (unless something causes a miraculous
+   resurrection.)
+
+#. there should not be more than 2 releases in the same week (but the
+   Release Manager is free to ignore this (get in touch with packagers
+   though),
+
+#. the series are aligned with Ubuntu releases for convenience since we
+   create a new series every 6 months. This means that we support the
+   stable series for 18 months. Note that we also propose the most recent
+   stable series via the stable PPA but that the SRU processs allow us to
+   reach a wider audience.
+
+At the start of a series cycle
+==============================
+
+To start a new series cycle:
+
+#. Create a new series ``x.y`` at <https://launchpad.net/bzr/+addseries>.
+
+#. Add milestones at <https://launchpad.net/bzr/x.y/+addmilestone> to that
+   series for the beta releases and the stable series mentioning their
+   expected dates. Only the milestone associated to the next release in
+   this series should be left active to avoid clutter when targeting bugs.
+
+#. If you made a new series, you will need to create a new pqm-controlled
+   branch for this release series. This branch will be used only from the
+   first non-beta release onwards. It needs to be created by a Canonical
+   sysadmin (ask the core devs for instructions or to do it for you).
+
+#. Start a new release-notes file::
+
+       cd doc/en/release-notes
+       cp series-template.txt bzr-x.y.txt  # e.g. bzr-2.3.txt
+       bzr add bzr-x.y.txt
+
+#. Start a new whats-new file::
+
+       cd doc/en/whats-new
+       cp template.txt bzr-x.y.txt  # e.g. bzr-2.6.txt
+       bzr add bzr-x.y.txt
+
 
 At the start of a release cycle
 ===============================
 
 To start a new release cycle:
 
-#. If this is the first release for a given *x.y* then create a new
-   series at <https://launchpad.net/bzr/+addseries>. There is one series
-   for every *x.y* release.
-
-#. If you made a new series, create a new pqm-controlled branch for this
-   release series, by asking a Canonical sysadmin.  This branch means that
-   from the first release beta or candidate onwards, general development
-   continues on the trunk, and only specifically-targeted fixes go into
-   the release branch.
-
-#. If you made a new series, add milestones at
-   <https://edge.launchpad.net/bzr/x.y/+addmilestone> to that series for
-   the beta release, release candidate and the final release, and their
-   expected dates.
-
-#. Create a new milestone <https://edge.launchpad.net/bzr/x.y/+addmilestone>
-   and add information about this release.  We will not use it yet, but it
-   will be available for targeting or nominating bugs.
-
 #. Send mail to the list with the key dates, who will be the release
    manager, and the main themes or targeted bugs.  Ask people to nominate
    objectives, or point out any high-risk things that are best done early,
    or that interact with other changes. This is called the metronome mail
    and is described in `Development cycles <cycle.html>`_.
 
-#. Make a local branch for preparing this release.  (Only for the first
-   release in a series, otherwise you should already have a branch.) ::
-
-     bzr branch trunk prepare-1.14
+#. Make a local branch to prepare the release::
+
+        bzr branch lp:bzr/x.y x.y-dev
+
+   If you're doing your first beta release, branch from trunk::
+
+       bzr branch lp:bzr x.y-dev
+
+   Note that you will generally reuse the same branch for all releases in a
+   given series.
 
 #. Configure pqm-submit for this branch, with a section like this (where
-   x.y is the version to release). **Or use hydrazine for easy use**
-   ``~/.bazaar/locations.conf``::
+   ``x.y`` is the series for your release). **Or use hydrazine for easier
+   setup** ``~/.bazaar/locations.conf``::
 
-	[/home/mbp/bzr/prepare-x.y]
+	[/home/mbp/bzr/x.y-dev]
 	pqm_email = Canonical PQM <pqm@bazaar-vcs.org>
 	submit_branch = http://bazaar.launchpad.net/~bzr-pqm/bzr/x.y
 	parent_branch = http://bazaar.launchpad.net/~bzr-pqm/bzr/x.y
-	public_branch = http://bazaar.example.com/prepare-x.y
+	public_branch = http://bazaar.example.com/x.y-dev
 	submit_to = bazaar@lists.canonical.com
 	smtp_server = mail.example.com:25
 
-    Please see <http://doc.bazaar-vcs.org/developers/HACKING.html#an-overview-of-pqm>
+    Please see <http://doc.bazaar.canonical.com/developers/HACKING.html#an-overview-of-pqm>
     for more details on PQM
 
 #. Update the version number in the ``bzr`` script, and the
@@ -81,9 +181,16 @@
    
        version_info = (x, y, z, 'dev', 0)
    
-#. Add a new section at the top of ``NEWS`` about the new release,
-   including its version number and the headings from
-   ``NEWS-template.txt``.
+#. Add a new section at the top of the current release notes (in
+   ``doc/en/release-notes``) about the new release, including its version
+   number and the headings from ``release-template.txt``.
+
+#. Update the "What's New" documents in ``doc/en/whats-new``.
+
+#. Make sure a milestone exists for your release and that it is active,
+   <https://launchpad.net/bzr/x.y> lists the existing milestones,
+   <https://launchpad.net/bzr/x.y/x.y.z/+edit> allows you to toggle the
+   active flag.
 
 #. Commit this and send it to PQM.
 
@@ -99,10 +206,22 @@
    also mail the list to raise this issue in our process. Milestones are
    found at <https://launchpad.net/bzr/+milestone/x.y.z>.
 
+#. Merge into your branch all previous stable series fixes that haven't been
+   merged yet. For example, if you're releasing 2.5.x, make sure the fixes
+   on 2.4, 2.3, etc have already been merged up::
+
+     bzr merge lp:bzr/2.4
+
+   and commit that merge in its own commit. This should happen only if the
+   devs landing changes in previous releases forgot to merge them up. Since
+   this can slow down the freeze, feel free to gently remind them about
+   their duties ;) If you feel unsafe resolving the conflicts or it's too
+   time consuming, contact the related devs and skip this merge.
+ 
 #. In the release branch, update  ``version_info`` in ``./bzrlib/__init__.py``.
    Make sure the corresponding milestone exists.
    Double check that ./bzr ``_script_version`` matches ``version_info``. Check
-   the output of ``bzr --version``.
+   the output of ``./bzr --version``.
 
    For beta releases use::
 
@@ -112,82 +231,136 @@
 
        version_info = (2, 1, 0, 'beta', 1)
 
-   For release candidates use::
-
-       version_info = (2, 0, 1, 'candidate', SERIAL)
-
    For stable releases use::
 
        version_info = (2, 1, 2, 'final', 0)
 
-#. Check the release number in ``./NEWS``
+#. Update the ``./doc/en/release-notes/`` section for this release.
+
+   Check that all news entries related to this release have been added in
+   the right section. For example, if you're releasing 2.5b2, the following
+   command should display a a single chuk diff for the 2.5b2 release::
+
+     bzr diff -rbzr-2.5b1.. doc/en/release-notes/bzr-2.5.txt
 
    Fill out the date and a description of the release under the existing
-   header. If there isn't one, follow the above for using the NEWS
-   template.
+   header (the diff above will help you summarizing). If there isn't one,
+   follow the instructions above for using the ``release-template.txt`` file
+   and remind people that they should document their changes there ;)
 
    See *2.1.1* or similar for an example of what this looks like.
 
-#. To check that all bugs mentioned in ``./NEWS`` are actually marked as
-   closed in Launchpad, you can run ``tools/check-newsbugs.py``::
-
-     ./tools/check-newsbugs.py NEWS
-
-   (But note there will be many false positives, and this script may be
-   flaky <https://bugs.edge.launchpad.net/bzr/+bug/354985>.  Don't let
-   this slow you down too much.)
+#. Add or check the summary of the release into the "What's New" document.
+
+#. To check that all bugs mentioned in the release notes are actually
+   marked as closed in Launchpad, you can run
+   ``tools/check-newsbugs.py``::
+
+     ./tools/check-newsbugs.py doc/en/release-notes/bzr-x.y.txt
+
+   As of 2011-07-18, all bugs mentioned in the output of the script requires
+   some sort of intervention (either changing the status if it's not 'Fix
+   Released' or setting a different milestone if the bug hasn't been
+   fixed). A few false positives may remain in the older series, don't let
+   this slow you down too much. This script accepts options you may find
+   useful, use ``./tools/check-newsbugs.py`` to display its usage (``-w``
+   will open each bug in your browser for example).
+
+#. For beta releases update the translation template::
+
+     make -f po/bzr.pot
+
+   This is especially important for the final beta release which is
+   when translations are frozen and translators are requested to make
+   the translations.
+
+#. For stable releases update the translations::
+
+     bzr merge lp:~bzr-core/bzr/bzr-translations-export-x.y
 
 #. Commit these changes to the release branch, using a command like::
 
-     bzr commit -m "Release 1.14."
+     bzr commit -m "Release 2.3.1"
 
    The diff before you commit will be something like::
 
-     === modified file 'NEWS'
-     --- NEWS        2008-09-17 23:09:18 +0000
-     +++ NEWS        2008-09-23 16:14:54 +0000
-     @@ -4,6 +4,23 @@
-
-      .. contents::
-
-     +bzr 1.7 2008-09-23
-     +------------------
-     +
-     +This release includes many bug fixes and a few performance and feature
-     +improvements.  ``bzr rm`` will now scan for missing files and remove them,
-     +like how ``bzr add`` scans for unknown files and adds them. A bit more
-     +polish has been applied to the stacking code. The b-tree indexing code has
-     +been brought in, with an eye on using it in a future repository format.
-     +There are only minor installer changes since bzr-1.7rc2.
-     +
-      bzr 1.7rc2 2008-09-17
-      ---------------------
-
-
-     === modified file 'bzrlib/__init__.py'
-     --- bzrlib/__init__.py  2008-09-16 21:39:28 +0000
-     +++ bzrlib/__init__.py  2008-09-23 16:14:54 +0000
-     @@ -41,7 +41,7 @@
-      # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
-      # releaselevel of 'dev' for unreleased under-development code.
-
-     -version_info = (1, 7, 0, 'candidate', 2)
-     +version_info = (1, 7, 0, 'final', 0)
-
-
-      # API compatibility version: bzrlib is currently API compatible with 1.7.
+      === modified file 'bzrlib/__init__.py'
+      --- bzrlib/__init__.py	2011-02-09 06:35:00 +0000
+      +++ bzrlib/__init__.py	2011-03-10 10:24:47 +0000
+      @@ -52,7 +52,7 @@
+       # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
+       # releaselevel of 'dev' for unreleased under-development code.
+       
+      -version_info = (2, 3, 1, 'dev', 0)
+      +version_info = (2, 3, 1, 'final', 0)
+       
+       # API compatibility version
+       api_minimum_version = (2, 3, 0)
+      
+      === modified file 'doc/en/release-notes/bzr-2.3.txt'
+      --- doc/en/release-notes/bzr-2.3.txt	2011-03-09 08:30:16 +0000
+      +++ doc/en/release-notes/bzr-2.3.txt	2011-03-10 10:40:47 +0000
+      @@ -8,23 +8,10 @@
+       bzr 2.3.1
+       #########
+       
+      -:2.3.1: NOT RELEASED YET
+      -
+      -External Compatibility Breaks
+      -*****************************
+      -
+      -.. These may require users to change the way they use Bazaar.
+      -
+      -New Features
+      -************
+      -
+      -.. New commands, options, etc that users may wish to try out.
+      -
+      -Improvements
+      -************
+      -
+      -.. Improvements to existing commands, especially improved performance 
+      -   or memory usage, or better results.
+      +:2.3.1: 2011-03-10
+      +
+      +This is a bugfix release. Upgrading is recommended for all users of earlier
+      +2.3 releases.
+       
+       Bug Fixes
+       *********
+      
+      === modified file 'doc/en/whats-new/whats-new-in-2.3.txt'
+      --- doc/en/whats-new/whats-new-in-2.3.txt	2011-02-03 16:29:18 +0000
+      +++ doc/en/whats-new/whats-new-in-2.3.txt	2011-03-10 11:10:36 +0000
+      @@ -17,8 +17,13 @@
+       improvements made to the core product, it highlights enhancements within the
+       broader Bazaar world of potential interest to those upgrading.
+       
+      -Bazaar 2.3.0 is fully compatible both locally and on the network with 2.0 2.1,
+      -and 2.2, and can read and write repositories generated by all previous
+      +Bazaar 2.3.1 includes all the fixes in the un-released 2.0.7, 2.1.4 and 2.2.5
+      +versions that weren't included in 2.3.0 and fixes some bugs on its own.
+      +
+      +See the :doc:`../release-notes/index` for details.
+      +
+      +Bazaar 2.3 is fully compatible both locally and on the network with 2.0, 2.1,
+      +and 2.2. It can read and write repositories generated by all previous
+       versions.
+       
+       Changed Behaviour
+      
 
 #. Tag the new release::
 
-     bzr tag bzr-1.14
+     bzr tag bzr-2.3.1
 
-#. Push those changes to a bzr repository that is public and accessible on
-   the Internet. PQM will pull from this repository when it attempts to merge
-   your changes. Then submit those changes to PQM for merge into the
-   appropriate release branch::
+#. Push those changes to a bzr branch that is public and accessible on the
+   Internet. PQM will pull from this branch when it attempts to merge your
+   changes. Then submit those changes to PQM for merge into the appropriate
+   release branch::
 
      bzr push
-     bzr pqm-submit -m "(mbp) prepare 1.14"
+     bzr pqm-submit -m "(vila) Release 2.3.1 (Vincent Ladeuil)"
 
    Or with hydrazine::
 
@@ -207,13 +380,23 @@
 #. Now we'll try expanding this tarball and running the test suite
    to check for packaging problems::
 
-     make check-dist-tarball
-
-   You may encounter failures while running the test suite caused
-   by your locally installed plugins. Use your own judgment to
-   decide if you can release with these failures. When in doubt,
-   disable the faulty plugins one by one until you get no more
-   failures.
+     make check-dist-tarball | subunit2pyunit
+
+   You may encounter failures while running the test suite caused by your
+   locally installed plugins. Use your own judgment to decide if you can
+   release with these failures. When in doubt, disable the faulty plugins
+   one by one until you get no more failures. Alternatively, you can use
+   ``BZR_DISABLE_PLUGINS`` or ``BZR_PLUGIN_PATH=-site`` to disable one or
+   all plugins.
+
+   Until <http://pad.lv/839461> is fixed, you may encounter issues if you
+   cut a release for old stable branches (<= 2.2) and use a more recent
+   OS/distro. If that's the case, check the bug status and use the following
+   workaround if no fix is available::
+
+     export TTPATH=<local branch of lp:testtools -r 0.9.2>
+     export SUPATH=<local branch of lp:subunit -r 0.0.6>
+     PYTHONPATH=$TTPATH:$SUPATH/python PATH=$SUPATH/filters:${PATH} BZR_PLUGIN_PATH=-site make check-dist-tarball PYTHON=python2.6 | subunit2pyunit
 
    Remember that PQM has just tested everything too, this step is
    particularly testing that the pyrex extensions, which are updated
@@ -224,7 +407,7 @@
 Publishing the source tarball
 -----------------------------
 
-#. Go to the relevant milestone page in Launchpad.
+#. Go to the relevant <https://launchpad.net/bzr/x.y> series page in Launchpad.
 
 #. Create a release of the milestone, and upload the source tarball and
    the GPG signature.  Or, if you prefer, use the
@@ -232,25 +415,103 @@
    this changes what the download widget on the Launchpad bzr home
    page shows, so don't stop the release process yet, or platform binary
    installers won't be made and the download list will stay very small!
-   <https://bugs.edge.launchpad.net/launchpad/+bug/586445>
-
-
-Announcing the source freeze
-----------------------------
-
-#. Post to the ``bazaar`` list, saying that the source has been frozen.
-   This is the cue for platform maintainers and plugin authors to update
-   their code.  This is done before the general public announcement of the
-   release.
+   <https://bugs.launchpad.net/launchpad/+bug/586445>
 
 
 Kick off the next cycle
 -----------------------
 
-#. To let developers work on the next release, do
-   `At the start of a release cycle` now.
-
-#. Pause for a few days.
+From that point, there is no possible return, the tarball has been uploaded
+so you can relax a bit.
+
+You're still holding a "social" lock on the launchpad branch though. Until
+your start the next cycle, nobody should land anything on this branch. If
+they do, they either targeted the wrong branch or didn't update the news
+file correctly, so the sooner the branch is opened again, the better.
+
+This matters more for ``lp:bzr`` than for ``lp:bzr/x.y``, ``lp:bzr`` should
+always be open for landing, so you should do `At the start of a release
+cycle`_ as soon as possible (i.e. update the version number in ``bzr`` and
+``bzrlib/__init__``, create/update the news files and create/update the
+milestone for the next relase).
+
+You may also need to do `At the start of a series cycle`_ if you're starting
+a new series.
+
+The final beta - branching and translations
+-------------------------------------------
+
+A word of caution: the instructions above works well for all releases but
+there is one special case that requires a bit more care: when you release
+the *last* beta for a given ``x.y`` series (from trunk aka lp:bzr), you need
+to setup *two* branches for the next cycle:
+
+#. ``lp:bzr`` needs to be opened for the next *series* ``x.(y+1)``
+
+#. ``lp:bzr/x.y`` needs to be opened for the next *release* ``x.y.0`` in the
+   series. Since this is first real use of ``lp:bzr/x.y``, this is also the
+   deadline for the PQM branch to be created.
+
+Both are important as ``lp:bzr`` should remain open so any change can be
+landed, ``lp:bzr/x.y`` on the other hand should be ready to receive bug
+fixes.
+
+``lp:bzr`` is generally more important as the bug fixes on ``lp:bzr/x.y``
+won't be released sooner than a month from now whereas people may already
+been waiting to land on ``lp:bzr``.
+
+In a nutshell:
+
+#. Create or update the ``x.y`` PQM branch based on whatever
+   revision you want to release
+
+#. Open ``lp:bzr`` for ``x.(y+1)``
+
+#. Release ``x.y.0`` from ``lp:bzr/x.y``
+
+#. Open ``lp:bzr/x.y`` for bug fixes
+
+You also need to ensure Launchpad is set up to import/export
+translations for the new branch and inform translators.
+
+#. Push a branch::
+
+     bzr push lp:~bzr-core/bzr/bzr-translations-export-x.y
+
+#. On the translations series synchronization settings page
+   <https://translations.launchpad.net/bzr/x.y/+translations-settings>
+   turn on ``Import template files`` then for exports click ``Choose a
+   target branch`` and point it at the branch you just pushed.
+
+#. E-mail translators to announce that the forthcoming stable release
+   of bzr is ready for translations.  Send to
+   ``translators@lists.launchpad.net`` and
+   ``ubuntu-translators@lists.ubuntu.com``.
+
+#. The series is now frozen for strings and API, see below for adding
+   that to the announcement.
+
+Announcing the source freeze
+----------------------------
+
+#. Post to the ``bazaar@lists.canonical.com`` list, saying that the source
+   has been frozen (gone gold). Be extra clear that this is only a *source*
+   release targeted at packagers and installer builders (see
+   <https://bugs.launchpad.net/launchpad/+bug/645084>).  This is the cue for
+   platform maintainers and plugin authors to update their code.  This is
+   done before the general public announcement of the release.
+
+   The freeze announcement generally guess the date of the official public
+   announcement, for the most recent stable series (the one supported by the
+   installers and most of the distributions) it's generally a few days after
+   the freeze. For older series supported only via SRUs for Ubuntu, we don't
+   control the process as tightly so guessing the date is not appropriate.
+
+   For the final beta release include in your announcement a notice of
+   API and translation freezes nothing that public methods should not
+   be removed or changed and strings should not be added or changed.
+
+#. Pause for a few days. 
 
 
 Publishing the release
@@ -263,20 +524,14 @@
 
 #. Go to the release web page at <https://launchpad.net/bzr/x.y/x.y.z>
 
-#. Link from http://bazaar-vcs.org/SourceDownloads to the tarball and
-   signature.
-
-#. Announce on the `Bazaar website <http://bazaar-vcs.org/>`_.
+#. Announce on the `Bazaar website <http://bazaar.canonical.com/>`_.
    This page is edited via the lp:bzr-website branch. (Changes
    pushed to this branch are refreshed by a cron job on escudero.)
 
-#. Announce on the `Bazaar wiki <http://bazaar-vcs.org/Welcome>`_.
-
 #. Check that the documentation for this release is available in
-   <http://doc.bazaar-vcs.org>.  It should be automatically build when the
+   <http://doc.bazaar.canonical.com>.  It should be automatically build when the
    branch is created, by a cron script ``update-bzr-docs`` on
-   ``escudero``. As of today (2009-08-27) ``igc`` manually updates the
-   pretty version of it.
+   ``escudero``.
 
 
 Announcing the release
@@ -286,13 +541,11 @@
 
 #. Make an announcement mail.
 
-   For release candidates or beta releases, this is sent to the ``bazaar``
-   list only to inform plugin authors and package or installer managers.
-
-   Once the installers are available, the mail can be sent to the
-   ``bazaar-announce`` list too.
-
-   For final releases, it should also be cc'd to ``info-gnu@gnu.org``,
+   For beta releases, this is sent to the ``bazaar@lists.canonical.com`` and
+   ``bazaar-announce@lists.canonical.com`` lists.
+
+   For stable releases (excluding SRUs which are for older stable releases),
+   it should also be cc'd to ``info-gnu@gnu.org``,
    ``python-announce-list@python.org``, ``bug-directory@gnu.org``.
 
    In all cases, it is good to set ``Reply-To: bazaar@lists.canonical.com``,
@@ -302,43 +555,69 @@
 
       Subject: bzr x.y.z released!
 
-      <<Summary paragraph from news>>
-
       The Bazaar team is happy to announce availability of a new
       release of the bzr adaptive version control system.
-      Bazaar is part of the GNU system <http://gnu.org/>.
+
+      Bazaar <http://bazaar.canonical.com/> is a Canonical project and part
+      of the GNU project <http://gnu.org/> to produce a free operating
+      system.
+
+      <<Summary paragraph from news>>
 
       Thanks to everyone who contributed patches, suggestions, and
       feedback.
 
       Bazaar is now available for download from
-      http://bazaar-vcs.org/Download as a source tarball; packages
+      https://launchpad.net/bzr/x.y/x.y.z/ as a source tarball; packages
       for various systems will be available soon.
 
-      <<NEWS section from this release back to the last major release>>
+      <<release notes from this release back to the last major release>>
 
    Feel free to tweak this to your taste.
 
 #. Make an announcement through <https://launchpad.net/bzr/+announce>
 
-#. Update the IRC channel topic. Use the ``/topic`` command to do this,
-   ensuring the new topic text keeps the project name, web site link, etc.
-
 #. Announce on http://freshmeat.net/projects/bzr/
 
-   This should be done for beta releases, release candidates and final
-   releases. If you do not have a Freshmeat account yet, ask one of the
-   existing admins.
+   This should be done for beta releases and stable releases. If you do not
+   have a Freshmeat account yet, ask one of the existing admins.
+
+   The purpose here is to point users to the latest stable release
+   (i.e. SRUs are excluded) while still publishing announcements for beta
+   releases.
+
+   There are several kinds of modifications that could be done there via the
+   ``Administration`` box in the lower right area of the page:
+
+   * Edit the project: This is where most of the URLs proposed in the
+     ``Links`` box are edited. This should rarely change except for the URLs
+     related to the latest stable release.
+
+   * New announcement: When doing a release, put the summary of the release
+     (you can't embed URLs there, the moderation staff remove them). Users
+     can still access the releases notes via the ``Release Notes`` URL in
+     the ``Links`` box in the upper right area of the page. When doing the
+     first stable release in a series, delete the ``Unstable installers``
+     <https://launchpad.net/bzr/x.y/x.ybn> and ``Unstable source tarball``
+     <http://launchpad.net/bzr/x.y/x.ybn/+download/bzr-x.ybn.tar.gz>
+     links. Conversely, when creating the first beta in a development
+     series, create these links again. Check all links when doing other
+     kinds of release.
+
+   * Set direct download: When releasing a new stable release, this should
+     point to the corresponding launchpad page:
+     <https://launchpad.net/bzr/x.y/x.y.z/>
 
 #. Update `<http://en.wikipedia.org/wiki/Bazaar_(software)>`_ -- this should
-   be done for final releases but not for beta releases or Release Candidates.
+   be done for the stable and beta releases.
 
 #. Update the python package index: <http://pypi.python.org/pypi/bzr> - best
    done by running ::
 
        python setup.py register
 
-   Remember to check the results afterwards.
+   Remember to check the results afterward -- this should be done for
+   stable releases but not for beta releases nor SRUs.
 
    To be able to register the release you must create an account on
    <http://pypi.python.org/pypi> and have one of the existing owners of
@@ -348,36 +627,87 @@
 Merging the released code back to trunk
 ---------------------------------------
 
-The rule is to keep ``NEWS`` sections sorted by date. You'll need to
-review the merge and make sure that that is respected.
+Merge the release branch back into the trunk.  The ``doc/en/release-notes``
+changes should be merged into the right place because each release series
+has its own release-notes file, but double-check.
 
-Merge the release branch back into the trunk.  Check that changes in NEWS
-were merged into the right sections.  If it's not already done, advance
-the version number in ``bzr`` and ``bzrlib/__init__.py``.  Submit this
-back into pqm for bzr.dev.
+If it's not already done, advance the version number in ``bzr`` and
+``bzrlib/__init__.py``.  Submit this back into pqm for bzr.dev.
 
 As soon as you change the version number in trunk, make sure you have
 created the corresponding milestone to ensure the continuity in bug
 targeting or nominating. Depending on the change, you may even have to
 create a new series (if your change the major or minor release number), in
-that case go to `At the start of a release cycle` and follow the instructions from there.
+that case go to `At the start of a series cycle`_ and follow the
+instructions from there.
 
-You should also merge (not pull) the release branch into
-``lp:~bzr/bzr/current``, so that branch contains the current released code
-at any time.
 
 Releases until the final one
 ----------------------------
 
-Congratulations - you have made your first release.  Have a beer
-or fruit juice - it's on the house! If it was a beta, or
-candidate, you're not finished yet. Another beta or candidate or
-hopefully a final release is still to come.
-
-The process is the same as for the first release. Goto `Doing a
-particular release`_ and follow the instructions again. Some details change
-between beta, candidate and final releases, but they should be
-documented. If the instructions aren't clear enough, please fix them.
+Congratulations - you have made your first release.  Have a beer or fruit
+juice - it's on the house! If it was a beta, you're not finished
+yet. Another beta or hopefully a stable release is still to come.
+
+The process is the same as for the first release. Goto `Doing a particular
+release`_ and follow the instructions again. Some details change between
+beta and stable releases, but they should be documented. If the instructions
+aren't clear enough, please fix them.
+
+
+Getting the release into Ubuntu
+-------------------------------
+
+(Feel free to propose or add new sections here about what we should do to
+get bzr into other places.)
+
+For the currently-under-development release of Ubuntu, no special action
+is needed: the release should be picked by Debian and synced from there into
+Ubuntu.
+
+Releases off stable bzr branches should go in to the ``-updates`` of the
+Ubuntu release that originally contained that branch.  (Ubuntu Lucid had
+bzr 2.2.0, so should get every 2.2.x update.)  This means going through
+the `SRU (Stable Release Updates)
+<https://wiki.ubuntu.com/StableReleaseUpdates>`__ process.   
+
+Since September 2010, bzr has received approval by the technical
+board for the `MicroReleaseExceptions
+<https://wiki.ubuntu.com/StableReleaseUpdates/MicroReleaseExceptions>`__
+category so that whole bugfix releases can more easily be
+approved.
+
+Progress on these realeases is tracked on the `SRU wiki
+<http://wiki.bazaar.canonical.com/UbuntuStableReleaseUpdates>`_
+page.
+
+**After making a bzr stable-release release, nominate the most serious bug
+for the appropriate Ubuntu release and subscribe the `ubuntu-sru` team.**
+
+This requires a couple of tricks (please reconsider and tweak as things
+evolves from one release to the other):
+
+ * create a distro task with the ``Also affects distribution`` button and
+   select ``bzr (Ubuntu)``.
+
+ * change the *URL* to point to ``ubuntu/+source/bzr`` instead of ``bzr``
+   (this is needed if you create the distro task but not if it exists
+   already). You should now be able to click the ``Nominate for release``
+   button and select the right Ubuntu release. As of September 2010, this
+   means:
+
+  * ``oneiric`` for the 2.4 series,
+  * ``natty`` for the 2.3 series,
+  * ``maverick`` for the 2.2 series,
+  * ``lucid`` for the 2.1 series,
+
+ * Subscribe the ``~ubuntu-sru`` team to the bug.
+
+ * Add a comment targeted to ``~ubuntu-sru`` explaining the expectations
+   (we are targeting running the test suite during the build which, as of
+   September 2010, fails for known reasons that are currently addressed).
+   Search for bugs tagged with ``sru`` for examples and don't forget to tag
+   the bug you selected.
 
 
 See also

=== modified file 'doc/developers/repository.txt'
--- doc/developers/repository.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/repository.txt	2010-11-12 14:28:36 +0000
@@ -246,7 +246,7 @@
   offset for the data record in the knit, the byte length for the data
   record in the knit and the no-end-of-line flag.
 
-Its important to note that knit repositories cannot be regenerated by
+It's important to note that knit repositories cannot be regenerated by
 scanning .knits, so a mapped index is still irreplaceable and must be
 transmitted on push/pull.
 

=== modified file 'doc/developers/revert.txt'
--- doc/developers/revert.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/revert.txt	2010-11-12 14:28:36 +0000
@@ -17,7 +17,7 @@
 for the selected scopes, for each element in the wt:
 
  1. get hash tree data for that scope.
- 1. get 'new enough' hash data for the siblings of the scope: it can be out of date as long as its not older than the last move or rename out of that siblings scope.
+ 1. get 'new enough' hash data for the siblings of the scope: it can be out of date as long as it's not older than the last move or rename out of that siblings scope.
  1. Use the hash tree data to tune the work done in finding matching paths/ids which are different in the two trees.
 
 For each thing that needs to change - group by target directory?

=== modified file 'doc/developers/revision-properties.txt'
--- doc/developers/revision-properties.txt	2009-03-25 16:53:14 +0000
+++ doc/developers/revision-properties.txt	2011-05-25 13:07:26 +0000
@@ -28,7 +28,7 @@
 At the moment, three standardized revision properties are recognized and used
 by bzrlib:
 
- * ``authors`` - Authors of the change. This value is a "\n" separated set
+ * ``authors`` - Authors of the change. This value is a "\\n" separated set
    of values in the same format as the committer-id. This property can be
    set by passing a list to the keyword argument ``authors`` of the function
    ``MutableTree.commit``.
@@ -39,7 +39,7 @@
    or manually set by ``bzr nick``. The value is set automatically in
    ``MutableTree.commit``.
  * ``bugs`` - A list of bug URLs and their statuses. The list is separated
-   by the new-line character (\n) and each entry is in format
+   by the new-line character (\\n) and each entry is in format
    '<URL> <status>'. Currently, bzrlib uses only status 'fixed'. See
    `Bug Trackers`_ for more details about using this feature.
 

=== modified file 'doc/developers/testing.txt'
--- doc/developers/testing.txt	2010-07-16 20:52:54 +0000
+++ doc/developers/testing.txt	2011-09-14 16:38:55 +0000
@@ -329,20 +329,28 @@
 We make selective use of doctests__.  In general they should provide
 *examples* within the API documentation which can incidentally be tested.  We
 don't try to test every important case using doctests |--| regular Python
-tests are generally a better solution.  That is, we just use doctests to
-make our documentation testable, rather than as a way to make tests.
+tests are generally a better solution.  That is, we just use doctests to make
+our documentation testable, rather than as a way to make tests. Be aware that
+doctests are not as well isolated as the unit tests, if you need more
+isolation, you're likely want to write unit tests anyway if only to get a
+better control of the test environment.
 
 Most of these are in ``bzrlib/doc/api``.  More additions are welcome.
 
   __ http://docs.python.org/lib/module-doctest.html
 
+There is an `assertDoctestExampleMatches` method in
+`bzrlib.tests.TestCase` that allows you to match against doctest-style
+string templates (including ``...`` to skip sections) from regular Python
+tests.
+
 
 Shell-like tests
 ----------------
 
-``bzrlib/tests/script.py`` allows users to write tests in a syntax very close to a shell session,
-using a restricted and limited set of commands that should be enough to mimic
-most of the behaviours.
+``bzrlib/tests/script.py`` allows users to write tests in a syntax very
+close to a shell session, using a restricted and limited set of commands
+that should be enough to mimic most of the behaviours.
 
 A script is a set of commands, each command is composed of:
 
@@ -367,8 +375,9 @@
 The execution stops as soon as an expected output or an expected error is not
 matched.
 
-When no output is specified, any ouput from the command is accepted
-and execution continue.
+If output occurs and no output is expected, the execution stops and the
+test fails.  If unexpected output occurs on the standard error, then
+execution stops and the test fails.
 
 If an error occurs and no expected error is specified, the execution stops.
 
@@ -388,6 +397,12 @@
 If you want the command to succeed for any output, just use::
 
   $ bzr add file
+  ...
+  2>...
+
+or use the ``--quiet`` option::
+
+  $ bzr add -q file
 
 The following will stop with an error::
 
@@ -427,6 +442,12 @@
 
   $ cat file
 
+You can run files containing shell-like scripts with::
+
+  $ bzr test-script <script>
+
+where ``<script>`` is the path to the file containing the shell-like script.
+
 The actual use of ScriptRunner within a TestCase looks something like
 this::
 
@@ -435,17 +456,42 @@
     def test_unshelve_keep(self):
         # some setup here
         script.run_script(self, '''
-            $ bzr add file
-            $ bzr shelve --all -m Foo
+            $ bzr add -q file
+            $ bzr shelve -q --all -m Foo
             $ bzr shelve --list
             1: Foo
-            $ bzr unshelve --keep
+            $ bzr unshelve -q --keep
             $ bzr shelve --list
             1: Foo
             $ cat file
             contents of file
             ''')
 
+You can also test commands that read user interaction::
+
+    def test_confirm_action(self):
+        """You can write tests that demonstrate user confirmation"""
+        commands.builtin_command_registry.register(cmd_test_confirm)
+        self.addCleanup(commands.builtin_command_registry.remove, 'test-confirm')
+        self.run_script("""
+            $ bzr test-confirm
+            2>Really do it? [y/n]: 
+            <yes
+            yes
+            """)
+
+To avoid having to specify "-q" for all commands whose output is
+irrelevant, the run_script() method may be passed the keyword argument
+``null_output_matches_anything=True``.  For example::
+
+    def test_ignoring_null_output(self):
+        self.run_script("""
+            $ bzr init
+            $ bzr ci -m 'first revision' --unchanged
+            $ bzr log --line
+            1: ...
+            """, null_output_matches_anything=True)
+           
 
 Import tariff tests
 -------------------
@@ -470,9 +516,9 @@
 regress.
 
 This is done by running the command in a subprocess with
-``--profile-imports``.  Starting a whole Python interpreter is pretty
-slow, so we don't want exhaustive testing here, but just enough to guard
-against distinct fixed problems.
+``PYTHON_VERBOSE=1``.  Starting a whole Python interpreter is pretty slow,
+so we don't want exhaustive testing here, but just enough to guard against
+distinct fixed problems.
 
 Assertions about precisely what is loaded tend to be brittle so we instead
 make assertions that particular things aren't loaded.
@@ -699,6 +745,38 @@
         _test_needs_features = [features.apport]
 
 
+Testing translations
+-----------------------
+
+Translations are disabled by default in tests.  If you want to test
+that code is translated you can use the ``ZzzTranslations`` class from
+``test_i18n``::
+
+    self.overrideAttr(i18n, '_translations', ZzzTranslations())
+
+And check the output strings look like ``u"zz\xe5{{output}}"``.
+
+To test the gettext setup and usage you override i18n.installed back
+to self.i18nInstalled and _translations to None, see
+test_i18n.TestInstall.
+
+
+Testing deprecated code
+-----------------------
+
+When code is deprecated, it is still supported for some length of time,
+usually until the next major version. The ``applyDeprecated`` helper
+wraps calls to deprecated code to verify that it is correctly issuing the
+deprecation warning, and also prevents the warnings from being printed
+during test runs.
+
+Typically patches that apply the ``@deprecated_function`` decorator should
+update the accompanying tests to use the ``applyDeprecated`` wrapper.
+
+``applyDeprecated`` is defined in ``bzrlib.tests.TestCase``. See the API
+docs for more details.
+
+
 Testing exceptions and errors
 -----------------------------
 
@@ -777,17 +855,11 @@
 whether a test should be added for that particular implementation,
 or for all implementations of the interface.
 
-The multiplication of tests for different implementations is normally
-accomplished by overriding the ``load_tests`` function used to load tests
-from a module.  This function typically loads all the tests, then applies
-a TestProviderAdapter to them, which generates a longer suite containing
-all the test variations.
-
 See also `Per-implementation tests`_ (above).
 
 
-Test scenarios
---------------
+Test scenarios and variations
+-----------------------------
 
 Some utilities are provided for generating variations of tests.  This can
 be used for per-implementation tests, or other cases where the same test
@@ -798,8 +870,24 @@
 values to which the test should be applied.  The test suite should then
 also provide a list of scenarios in which to run the tests.
 
-Typically ``multiply_tests_from_modules`` should be called from the test
-module's ``load_tests`` function.
+A single *scenario* is defined by a `(name, parameter_dict)` tuple.  The
+short string name is combined with the name of the test method to form the
+test instance name.  The parameter dict is merged into the instance's
+attributes.
+
+For example::
+
+    load_tests = load_tests_apply_scenarios
+
+    class TestCheckout(TestCase):
+
+        scenarios = multiply_scenarios(
+            VaryByRepositoryFormat(), 
+            VaryByTreeFormat(),
+            )
+
+The `load_tests` declaration or definition should be near the top of the
+file so its effect can be seen.
 
 
 Test support
@@ -904,6 +992,39 @@
 
 Please see bzrlib.treebuilder for more details.
 
+PreviewTree
+~~~~~~~~~~~
+
+PreviewTrees are based on TreeTransforms.  This means they can represent
+virtually any state that a WorkingTree can have, including unversioned files.
+They can be used to test the output of anything that produces TreeTransforms,
+such as merge algorithms and revert.  They can also be used to test anything
+that takes arbitrary Trees as its input.
+
+::
+
+  # Get an empty tree to base the transform on.
+  b = self.make_branch('.')
+  empty_tree = b.repository.revision_tree(_mod_revision.NULL_REVISION)
+  tt = TransformPreview(empty_tree)
+  self.addCleanup(tt.finalize)
+  # Empty trees don't have a root, so add it first.
+  root = tt.new_directory('', ROOT_PARENT, 'tree-root')
+  # Set the contents of a file.
+  tt.new_file('new-file', root, 'contents', 'file-id')
+  preview = tt.get_preview_tree()
+  # Test the contents.
+  self.assertEqual('contents', preview.get_file_text('file-id'))
+
+PreviewTrees can stack, with each tree falling back to the previous::
+
+  tt2 = TransformPreview(preview)
+  self.addCleanup(tt2.finalize)
+  tt2.new_file('new-file2', tt2.root, 'contents2', 'file-id2')
+  preview2 = tt2.get_preview_tree()
+  self.assertEqual('contents', preview2.get_file_text('file-id'))
+  self.assertEqual('contents2', preview2.get_file_text('file-id2'))
+
 
 Temporarily changing state
 ~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -913,6 +1034,42 @@
 
     self.overrideAttr(osutils, '_cached_user_encoding', 'latin-1')
 
+This should be used with discretion; sometimes it's better to make the
+underlying code more testable so that you don't need to rely on monkey
+patching.
+
+
+Observing calls to a function
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+Sometimes it's useful to observe how a function is called, typically when
+calling it has side effects but the side effects are not easy to observe
+from a test case.  For instance the function may be expensive and we want
+to assert it is not called too many times, or it has effects on the
+machine that are safe to run during a test but not easy to measure.  In
+these cases, you can use `recordCalls` which will monkey-patch in a
+wrapper that records when the function is called.
+
+
+Temporarily changing environment variables
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+If yout test needs to temporarily change some environment variable value
+(which generally means you want it restored at the end), you can use::
+
+    self.overrideEnv('BZR_ENV_VAR', 'new_value')
+
+If you want to remove a variable from the environment, you should use the
+special ``None`` value::
+
+    self.overrideEnv('PATH', None)
+
+If you add a new feature which depends on a new environment variable, make
+sure it behaves properly when this variable is not defined (if applicable) and
+if you need to enforce a specific default value, check the
+``TestCase._cleanEnvironment`` in ``bzrlib.tests.__init__.py`` which defines a
+proper set of values for all tests.
+
 Cleaning up
 ~~~~~~~~~~~
 
@@ -948,9 +1105,8 @@
 
     tc qdisc add dev lo root handle 1: prio
     tc qdisc add dev lo parent 1:3 handle 30: netem delay 500ms 
-    tc qdisc add dev lo parent 30:1 handle 40: prio
-    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip dport 4155 0xffff flowid 1:3 handle 800::800
-    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip sport 4155 0xffff flowid 1:3 handle 800::801
+    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip dport 4155 0xffff flowid 1:3 
+    tc filter add dev lo protocol ip parent 1:0 prio 3 u32 match ip sport 4155 0xffff flowid 1:3 
 
 and to remove this::
 

=== modified file 'doc/developers/tortoise-strategy.txt'
--- doc/developers/tortoise-strategy.txt	2009-12-02 20:34:07 +0000
+++ doc/developers/tortoise-strategy.txt	2010-12-16 06:24:14 +0000
@@ -306,7 +306,7 @@
 Reuse of TSVNCache?
 ~~~~~~~~~~~~~~~~~~~
 
-The RPC mechanism and the tasks performed by the RPC server (rpc, file system
+The RPC mechanism and the tasks performed by the RPC server (RPC, file system
 crawling and watching, device notifications, caching) are very similar to
 those already implemented for TSVN and analysis of that code shows that
 it is not particularly tied to any VCS model.  As a result, consideration
@@ -437,7 +437,7 @@
 * Implement property pages and context menus in C++. Expand RPC server as
   necessary.
 
-* Create binary for alpha releases, then go round-and-round until its baked.
+* Create binary for alpha releases, then go round-and-round until it's baked.
 
 Alternative Implementation Strategies
 -------------------------------------

=== modified file 'doc/developers/transports.txt'
--- doc/developers/transports.txt	2010-07-16 12:40:07 +0000
+++ doc/developers/transports.txt	2010-11-12 22:08:18 +0000
@@ -88,8 +88,8 @@
 Transports that hide symlinks
 -----------------------------
 
-On local, sftp and bzr+ssh transports, we can directly see symlinks as
-symlinks.  Over http (and ftp?) they're expanded by the server and we
+On local, SFTP and bzr+ssh transports, we can directly see symlinks as
+symlinks.  Over HTTP (and FTP?) they're expanded by the server and we
 cannot detect them.  This can cause problems when bzr follows relative
 paths because typically we will join the paths, and we may do this
 inconsistently with how the server, which can see the symlinks, would do.

=== added file 'doc/developers/ui.txt'
--- doc/developers/ui.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/ui.txt	2010-09-14 06:46:18 +0000
@@ -0,0 +1,235 @@
+=========================
+Interacting with the user
+=========================
+
+Getting Input
+=============
+
+Processing Command Lines
+------------------------
+
+bzrlib has a standard framework for parsing command lines and calling
+processing routines associated with various commands. See builtins.py
+for numerous examples.
+
+
+Standard Parameter Types
+------------------------
+
+There are some common requirements in the library: some parameters need to be
+unicode safe, some need byte strings, and so on. At the moment we have
+only codified one specific pattern: Parameters that need to be unicode
+should be checked via ``bzrlib.osutils.safe_unicode``. This will coerce the
+input into unicode in a consistent fashion, allowing trivial strings to be
+used for programmer convenience, but not performing unpredictably in the
+presence of different locales.
+
+Confirmation
+============
+
+There are some operations, such as uncommitting, or breaking a lock, where
+bzr may want to get confirmation from the user before proceeding.
+However in some circumstances bzr should just go ahead without asking: if
+it's being used from a noninteractive program, or if the user's asked to
+never be asked for this particular confirmation or for any confirmations
+at all.
+
+We provide a special UIFactory method `confirm_action` to do this.  It
+takes a `confirmation_id` parameter that acts as a symbolic name for the
+type of confirmation, so the user can configure them off.  (This is not
+implemented at present.)  GUIs can have a "don't ask me again" option
+keyed by the confirmation id.
+
+Confirmation ids look like Python paths to the logical code that should
+use them.  (Because code may move or the check may for implementation
+reasons be done elsewhere, they need not perfectly correspond to the place
+they're used, and they should stay stable even when the code moves.)
+
+``bzrlib.builtins.uncommit``
+    Before the ``uncommit`` command actually changes the branch.
+
+``bzrlib.lockdir.break``
+    Before breaking a lock.
+
+``bzrlib.msgeditor.unchanged``
+    Proceed even though the user made no changes to the template message.
+
+Interactive confirmations can be overridden by using a
+`ConfirmationUserInterfacePolicy` decorator as the default
+ui_factory.
+
+
+Writing Output
+==============
+
+(The strategy described here is what we want to get to, but it's not
+consistently followed in the code at the moment.)
+
+bzrlib is intended to be a generically reusable library.  It shouldn't
+write messages to stdout or stderr, because some programs that use it
+might want to display that information through a GUI or some other
+mechanism.
+
+We can distinguish two types of output from the library:
+
+ 1. Structured data representing the progress or result of an
+    operation.  For example, for a commit command this will be a list
+    of the modified files and the finally committed revision number
+    and id.
+
+    These should be exposed either through the return code or by calls
+    to a callback parameter.
+
+    A special case of this is progress indicators for long-lived
+    operations, where the caller should pass a ProgressBar object.
+
+ 2. Unstructured log/debug messages, mostly for the benefit of the
+    developers or users trying to debug problems.  This should always
+    be sent through ``bzrlib.trace`` and Python ``logging``, so that
+    it can be redirected by the client.
+
+The distinction between the two is a bit subjective, but in general if
+there is any chance that a library would want to see something as
+structured data, we should make it so.
+
+The policy about how output is presented in the text-mode client
+should be only in the command-line tool.
+
+
+Progress and Activity Indications
+---------------------------------
+
+bzrlib has a way for code to display to the user that stuff is happening
+during a long operation.  There are two particular types: *activity* which
+means that IO is happening on a Transport, and *progress* which means that
+higher-level application work is occurring.  Both are drawn together by
+the `ui_factory`.
+
+Transport objects are responsible for calling `report_transport_activity`
+when they do IO.
+
+Progress uses a model/view pattern: application code acts on a
+`ProgressTask` object, which notifies the UI when it needs to be
+displayed.  Progress tasks form a stack.  To create a new progress task on
+top of the stack, call `bzrlib.ui.ui_factory.nested_progress_bar()`, then
+call `update()` on the returned ProgressTask.  It can be updated with just
+a text description, with a numeric count, or with a numeric count and
+expected total count.  If an expected total count is provided the view
+can show the progress moving along towards the expected total.
+
+The user should call `finish` on the `ProgressTask` when the logical
+operation has finished, so it can be removed from the stack.
+
+Progress tasks have a complex relationship with generators: it's a very
+good place to use them, but because python2.4 does not allow ``finally``
+blocks in generators it's hard to clean them up properly.  In this case
+it's probably better to have the code calling the generator allocate a
+progress task for its use and then call `finalize` when it's done, which
+will close it if it was not already closed.  The generator should also
+finish the progress task when it exits, because it may otherwise be a long
+time until the finally block runs.
+
+
+Message guidelines
+------------------
+
+When filenames or similar variables are presented inline within a message,
+they should be enclosed in double quotes (ascii 0x22, not chiral unicode
+quotes)::
+
+  bzr: ERROR: No such file "asdf"
+
+When we print just a list of filenames there should not be any quoting:
+see `bug 544297`_.
+
+.. _bug 544297: https://bugs.launchpad.net/bugs/544297
+
+https://wiki.ubuntu.com/UnitsPolicy provides a good explanation about
+which unit should be used when. Roughly speaking, IEC standard applies
+for base-2 units and SI standard applies for base-10 units:
+
+* for network bandwidth and disk sizes, use base-10 (Mbits/s, kB/s, GB)
+
+* for RAM sizes, use base-2 (GiB, TiB)
+
+
+
+Displaying help
+===============
+
+Bazaar has online help for various topics through ``bzr help COMMAND`` or
+equivalently ``bzr command -h``.  We also have help on command options,
+and on other help topics.  (See ``help_topics.py``.)
+
+As for python docstrings, the first paragraph should be a single-sentence
+synopsis of the command. These are user-visible and should be prefixed with
+``__doc__ =`` so help works under ``python -OO`` with docstrings stripped.
+
+The help for options should be one or more proper sentences, starting with
+a capital letter and finishing with a full stop (period).
+
+All help messages and documentation should have two spaces between
+sentences.
+
+
+Handling Errors and Exceptions
+==============================
+
+Commands should return non-zero when they encounter circumstances that
+the user should really pay attention to - which includes trivial shell
+pipelines.
+
+Recommended values are:
+
+    0. OK.
+    1. Conflicts in merge-like operations, or changes are present in
+       diff-like operations.
+    2. Unrepresentable diff changes (i.e. binary files that we cannot show
+       a diff of).
+    3. An error or exception has occurred.
+    4. An internal error occurred (one that shows a traceback.)
+
+Errors are handled through Python exceptions. Exceptions should be defined
+inside bzrlib.errors, so that we can see the whole tree at a glance.
+
+We broadly classify errors as either being either internal or not,
+depending on whether ``internal_error`` is set or not.  If we think it's our
+fault, we show a backtrace, an invitation to report the bug, and possibly
+other details.  This is the default for errors that aren't specifically
+recognized as being caused by a user error.  Otherwise we show a briefer
+message, unless -Derror was given.
+
+Many errors originate as "environmental errors" which are raised by Python
+or builtin libraries -- for example IOError.  These are treated as being
+our fault, unless they're caught in a particular tight scope where we know
+that they indicate a user errors.  For example if the repository format
+is not found, the user probably gave the wrong path or URL.  But if one of
+the files inside the repository is not found, then it's our fault --
+either there's a bug in bzr, or something complicated has gone wrong in
+the environment that means one internal file was deleted.
+
+Many errors are defined in ``bzrlib/errors.py`` but it's OK for new errors
+to be added near the place where they are used.
+
+Exceptions are formatted for the user by conversion to a string
+(eventually calling their ``__str__`` method.)  As a convenience the
+``._fmt`` member can be used as a template which will be mapped to the
+error's instance dict.
+
+New exception classes should be defined when callers might want to catch
+that exception specifically, or when it needs a substantially different
+format string.
+
+#. If it is something that a caller can recover from, a custom exception
+   is reasonable.
+
+#. If it is a data consistency issue, using a builtin like
+   ``ValueError``/``TypeError`` is reasonable.
+
+#. If it is a programmer error (using an api incorrectly)
+   ``AssertionError`` is reasonable.
+
+#. Otherwise, use ``BzrError`` or ``InternalBzrError``.
+
+Exception strings should start with a capital letter and should not have a
+final fullstop.  If long, they may contain newlines to break the text.

=== modified file 'doc/developers/win32_build_setup.txt'
--- doc/developers/win32_build_setup.txt	2010-01-25 10:18:24 +0000
+++ doc/developers/win32_build_setup.txt	2010-11-12 22:36:19 +0000
@@ -129,7 +129,7 @@
 
    If you do this, you'll probably also want to install a shortcut to
    ``pageant.exe`` in Start / Programs / Startup so that it always starts when
-   you log in (though you still have to manually add your ssh keys.)
+   you log in (though you still have to manually add your SSH keys.)
 
    Note that on the Amazon EC2 machine, I'm having problems with temp files
    being created without the permission for the current user to actually read

=== added file 'doc/developers/xdg_config_spec.txt'
--- doc/developers/xdg_config_spec.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/xdg_config_spec.txt	2010-11-27 05:38:41 +0000
@@ -0,0 +1,27 @@
+Transitioning Unix installs to the XDG Base Directory Specification
+===================================================================
+
+Currently, Bazaar stores its configuration files and plugins under the
+directory ~/.bazaar on unix installs.  On Windows, this is
+%APPDATA%/Bazaar/2.0 and on Mac OS X, the directory is ~/.bazaar.  With the
+XDG Base Directory specification
+(http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html), many
+Linux and Unix platforms have tried to centralize configuration files under a
+specific directory referred to as $XDG_CONFIG_HOME.  This has a default value
+of ~/.config.
+
+Bazaar would like to be a good Unix citizen by using these standard locations
+for configuration files.  As such, we should support that location, but not
+require it.  Note that the following descriptions do not apply
+to Windows or Mac OS X which should use their own native configuration
+locations.  (On Windows, we currently do this by working under %APPDATA%.  The
+Mac OS X equivalent would be ~/Library/Application Support/Bazaar but there is
+also cultural support for ~/.bazaar on that platform.)
+
+* If $XDG_CONFIG_HOME/bazaar exists, use the files there for configuration,
+  noting in the log that we are doing so.  This allows individuals who would
+  like to use the XDG specification to do so.
+* Due to a lack of consensus on where plugins should live under the XDG Base
+  Directory spec, continue to look for plugins in ~/.bazaar/plugins.  To
+  change this directory to something not under ~/.bazaar, use the environment
+  variable $BZR_PLUGIN_PATH.

=== modified file 'doc/en/_static/en/bzr-en-quick-reference.pdf'
Binary files doc/en/_static/en/bzr-en-quick-reference.pdf and doc/en/_static/en/bzr-en-quick-reference.pdf differ
=== modified file 'doc/en/_static/en/bzr-en-quick-reference.png'
Binary files doc/en/_static/en/bzr-en-quick-reference.png and doc/en/_static/en/bzr-en-quick-reference.png differ
=== modified file 'doc/en/_static/en/bzr-en-quick-reference.svg'
--- doc/en/_static/en/bzr-en-quick-reference.svg	2010-01-29 10:36:23 +0000
+++ doc/en/_static/en/bzr-en-quick-reference.svg	2011-04-20 10:25:40 +0000
@@ -2,7 +2,7 @@
 <!-- Created with Inkscape (http://www.inkscape.org/) -->
 <svg
    xmlns:dc="http://purl.org/dc/elements/1.1/"
-   xmlns:cc="http://web.resource.org/cc/"
+   xmlns:cc="http://creativecommons.org/ns#"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:svg="http://www.w3.org/2000/svg"
    xmlns="http://www.w3.org/2000/svg"
@@ -13,7 +13,7 @@
    height="744.09448"
    id="svg2766"
    sodipodi:version="0.32"
-   inkscape:version="0.45.1"
+   inkscape:version="0.46"
    version="1.0"
    sodipodi:docbase="/home/ian/bzr/repo.packs/bzr.quick-start-tweaks/doc/en/quick-reference"
    sodipodi:docname="quick-start-summary.svg"
@@ -363,7 +363,7 @@
              sodipodi:role="line"
              id="tspan2187"
              x="64.448624"
-             y="277.48495">bzr branch mp myproject</tspan></text>
+             y="277.48495">bzr branch myproject newproject</tspan></text>
         <text
            xml:space="preserve"
            style="font-size:12px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-align:start;line-height:100%;writing-mode:lr-tb;text-anchor:start;fill:#333333;fill-opacity:1;fill-rule:nonzero;stroke:none;stroke-width:30.5;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;font-family:Bitstream Vera Sans"
@@ -389,7 +389,7 @@
              sodipodi:role="line"
              id="tspan3406"
              x="64.448624"
-             y="308.44989">bzr checkout mp myproject</tspan></text>
+             y="308.44989">bzr checkout srcproject myproject</tspan></text>
         <text
            xml:space="preserve"
            style="font-size:12px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-align:start;line-height:100%;writing-mode:lr-tb;text-anchor:start;fill:#333333;fill-opacity:1;fill-rule:nonzero;stroke:none;stroke-width:30.5;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;font-family:Bitstream Vera Sans"
@@ -415,11 +415,11 @@
              id="tspan3424"
              sodipodi:role="line"
              x="64.448624"
-             y="341.88864">bzr checkout --lightweight mp \</tspan><tspan
+             y="341.88864">bzr checkout --lightweight \</tspan><tspan
              sodipodi:role="line"
              x="64.448624"
              y="352.96197"
-             id="tspan2780"> myproject</tspan></text>
+             id="tspan2780"> srcproject myproject</tspan></text>
         <text
            xml:space="preserve"
            style="font-size:12px;font-style:normal;font-variant:normal;font-weight:bold;font-stretch:normal;text-align:start;line-height:100%;writing-mode:lr-tb;text-anchor:start;fill:#333333;fill-opacity:1;fill-rule:nonzero;stroke:none;stroke-width:30.5;stroke-miterlimit:4;stroke-dasharray:none;stroke-dashoffset:0;stroke-opacity:1;font-family:Bitstream Vera Sans"
@@ -1329,7 +1329,7 @@
          sodipodi:role="line"
          id="tspan3411"
          x="146.57143"
-         y="97.808769">Quick Start Card</tspan></text>
+         y="97.808769">Quick Reference Card</tspan></text>
     <g
        id="g12314"
        transform="translate(17.946423,1.7472534)"

=== modified file 'doc/en/_templates/index.html'
--- doc/en/_templates/index.html	2010-03-04 04:15:18 +0000
+++ doc/en/_templates/index.html	2011-08-30 09:27:59 +0000
@@ -7,7 +7,7 @@
 
   <table class="contentstable" align="center" style="margin-left: 30px"><tr>
     <td width="50%">
-        <p class="biglink"><a class="biglink" href="{{ pathto("whats-new/whats-new-in-2.2") }}">What's New in Bazaar 2.2?</a><br/>
+        <p class="biglink"><a class="biglink" href="{{ pathto("whats-new/whats-new-in-2.5") }}">What's New in Bazaar 2.5?</a><br/>
         <span class="linkdescr">what's new in this release</span>
       </p>
         <p class="biglink"><a class="biglink" href="{{ pathto("user-guide/index") }}">User Guide</a><br/>
@@ -45,7 +45,7 @@
         <p class="biglink"><a class="biglink" href="https://answers.launchpad.net/bzr/+faqs">FAQ</a><br/>
       <span class="linkdescr">frequently asked questions</span>
       </p>
-      <p class="biglink"><a class="biglink" href="http://bazaar.canonical.com/BzrGlossary/">Glossary</a><br/>
+      <p class="biglink"><a class="biglink" href="http://wiki.bazaar.canonical.com/BzrGlossary/">Glossary</a><br/>
       <span class="linkdescr">help with terminology</span>
       </p>
     </td>

=== modified file 'doc/en/admin-guide/code-browsing.txt'
--- doc/en/admin-guide/code-browsing.txt	2010-06-02 05:03:31 +0000
+++ doc/en/admin-guide/code-browsing.txt	2010-08-13 19:08:57 +0000
@@ -110,8 +110,8 @@
 --------------------
 
 There are a number of other web interfaces available for Bazaar branches (see
-the list at http://bazaar-vcs.org/WebInterfaces) and we will just mention a
-couple of them here for their advantages in particular situations.
+the list at http://wiki.bazaar.canonical.com/WebInterfaces) and we will just
+mention a couple of them here for their advantages in particular situations.
 
 trac+bzr (http://launchpad.net/trac-bzr)
   Trac is a popular web app that integrates a browser for branches, an issue

=== modified file 'doc/en/admin-guide/hooks-plugins.txt'
--- doc/en/admin-guide/hooks-plugins.txt	2010-03-01 05:15:11 +0000
+++ doc/en/admin-guide/hooks-plugins.txt	2010-08-13 19:08:57 +0000
@@ -59,8 +59,8 @@
 mail should be sent.  If it isn't set, email is sent via ``/usr/bin/mail``.
 It can also be configured to communicate directly with an SMTP server.
 For more details on configuring this plugin, see
-http://doc.bazaar-vcs.org/plugins/en/email-plugin.html.  As examples, consider
-the following two possible configurations.  A minimal one (uses
+http://doc.bazaar.canonical.com/plugins/en/email-plugin.html.  As examples,
+consider the following two possible configurations.  A minimal one (uses
 ``/usr/bin/mail``)
 
 ::
@@ -166,8 +166,8 @@
 ``branch.atom`` files.
 
 In addition, there are other tools that are not plugins for creating news
-feeds from Bazaar branches.  See http://bazaar-vcs.org/FeedGenerators for more
-on those tools.
+feeds from Bazaar branches.  See
+http://wiki.bazaar.canonical.com/FeedGenerators for more on those tools.
 
 Mirroring
 ---------

=== modified file 'doc/en/admin-guide/index-plain.txt'
--- doc/en/admin-guide/index-plain.txt	2009-12-07 18:12:21 +0000
+++ doc/en/admin-guide/index-plain.txt	2011-05-16 10:05:25 +0000
@@ -21,3 +21,4 @@
 .. include:: backup.txt
 .. include:: upgrade.txt
 .. include:: advanced.txt
+.. include:: licence.txt

=== modified file 'doc/en/admin-guide/index.txt'
--- doc/en/admin-guide/index.txt	2009-12-07 18:12:21 +0000
+++ doc/en/admin-guide/index.txt	2011-05-16 10:05:25 +0000
@@ -23,3 +23,4 @@
    backup
    upgrade
    advanced
+   licence

=== added file 'doc/en/admin-guide/licence.txt'
--- doc/en/admin-guide/licence.txt	1970-01-01 00:00:00 +0000
+++ doc/en/admin-guide/licence.txt	2011-05-16 10:05:25 +0000
@@ -0,0 +1,7 @@
+Licence
+===============
+
+Copyright 2008-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later.  See
+<http://www.gnu.org/licenses/>.

=== modified file 'doc/en/admin-guide/migration.txt'
--- doc/en/admin-guide/migration.txt	2009-12-07 18:29:00 +0000
+++ doc/en/admin-guide/migration.txt	2010-08-13 19:08:57 +0000
@@ -3,7 +3,7 @@
 
 Migrating between version control systems can be a complicated process, and
 Bazaar has extensive documentation on the process at 
-http://doc.bazaar-vcs.org/migration/en and we won't attempt to repeat that
+http://doc.bazaar.canonical.com/migration/en and we won't attempt to repeat that
 here.  We will try to give a few motivating examples for conversion from
 Mercurial and Subversion.
 
@@ -47,7 +47,7 @@
 Bazaar's `svn`_ plugin provides tools for interaction with Subversion
 projects.  In fact, Bazaar can be used transparently with projects stored in
 Subversion, but that is beyond the scope of this document.  (See
-http://doc.bazaar-vcs.org/en/migration/foreign/bzr-on-svn-projects.html for
+http://doc.bazaar.canonical.com/en/migration/foreign/bzr-on-svn-projects.html for
 more on that subject.)  What is relevant here is the ``svn-import`` command
 provided by that plugin.  This can import an entire subversion repository
 including tags and branches, particularly if they are stored in Subversion's

=== modified file 'doc/en/admin-guide/other-setups.txt'
--- doc/en/admin-guide/other-setups.txt	2009-12-07 21:51:01 +0000
+++ doc/en/admin-guide/other-setups.txt	2010-09-23 10:56:44 +0000
@@ -6,8 +6,8 @@
 
 Bazaar can also serve branches over protocols that know nothing about Bazaar's
 specific needs.  These are called "dumb servers" to distinguish them from
-Bazaar's native protocol.  Currently HTTP, FTP, SFTP and HTTP+WebDAV can be
-used to read branches remotely.  FTP, SFTP and HTTP+WebDAV can be used for
+Bazaar's native protocol.  Currently HTTP, HTTPS, FTP, SFTP and HTTP+WebDAV can
+be used to read branches remotely.  FTP, SFTP and HTTP+WebDAV can be used for
 writing as well.  To use any of these protocols, it is just necessary to
 provide access to the server's filesystem under ``/srv/bzr``.
 
@@ -33,14 +33,14 @@
 Smart server over HTTP(S)
 -------------------------
 
-Bazaar can use its native protocol with HTTP requests.  Since HTTP is a network
-protocol that is available on many networks, this can be a good option where
-SSH access is not possible.  Another benefit of this setup is that all of the
-authentication and access control methods available to the HTTP server (basic,
-LDAP, ActiveDirectory, etc.) are then available to control access to Bazaar
-branches.  More information about setting up this type of access using Apache
-and FastCGI or mod_python or WSGI is in the `smart server section of the User's
-Guide <../user-guide/http_smart_server.html>`_.
+Bazaar can use its native protocol with HTTP or HTTPS requests.  Since HTTP is
+a network protocol that is available on many networks, this can be a good
+option where SSH access is not possible.  Another benefit of this setup is that
+all of the authentication and access control methods available to the HTTP
+server (basic, LDAP, ActiveDirectory, etc.) are then available to control
+access to Bazaar branches.  More information about setting up this type of
+access using Apache and FastCGI or mod_python or WSGI is in the `smart server
+section of the User's Guide <../user-guide/http_smart_server.html>`_.
 
 Direct Smart Server Access
 --------------------------

=== modified file 'doc/en/admin-guide/simple-setups.txt'
--- doc/en/admin-guide/simple-setups.txt	2009-12-07 21:51:01 +0000
+++ doc/en/admin-guide/simple-setups.txt	2011-03-24 00:13:10 +0000
@@ -3,7 +3,7 @@
 
 Consider the following simple scenario where we will be serving Bazaar branches
 that live on a single server.  Those branches are in the subdirectories of
-``/srv/bzr`` (or ``C:\\bzr``) and they will all be related to a single project
+``/srv/bzr`` (or ``C:\bzr``) and they will all be related to a single project
 called "ProjectX".  ProjectX will have a trunk branch and at least one feature
 branch.  As we get further, we will consider other scenarios, but this will be
 a sufficiently motivating example.
@@ -91,3 +91,52 @@
 
 .. [#] The version of Bazaar installed on the server must be at least 2.1.0b1 
        or newer to support ``/~/`` in bzr+ssh URLs.
+
+Using a restricted SSH account to host multiple users and repositories
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+Once you have a bzr+ssh setup using a shared repository you may want to share
+that repository among a small set of developers.  Using shared SSH access enables
+you to complete this task without any complicated setup or ongoing management.
+
+To allow multiple users to access Bazaar over ssh we can allow ssh access to a common
+account that only allows users to run a specific command.  Using a single account
+simplifies deployment as no permissions management issues exist for the filesystem.
+All users are the same user at the server level.  Bazaar labels the commits with
+each users details so seperate server accounts are not required.
+
+To enable this configuration we update the ``~/.ssh/authorized_keys`` to include 
+command restrictions for connecting users.
+
+In these examples the user will be called ``bzruser``.
+
+The following example shows how a single line is configured::
+
+  command="bzr --serve --inet --allow-writes --directory=/srv/bzr",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= my bzr key
+
+This command allows the user to access only bzr and disables other SSH use.  Write
+access to each repository in the directory ``/srv/bzr`` has been granted with ``--allow-writes``
+and can be removed for individual users that should only require read access.  The root of
+the directory structure can be altered for each user to allow them to see only a subet
+of the repositories available.  The example below assumes two seperate repositories
+for Alice and Bob.  This method will not allow you to restrict access to part
+of a repository, you may only restrict access to a single part of the directory structure::
+
+  command="bzr --serve --inet --allow-writes --directory=/srv/bzr/alice/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Alice's SSH Key
+  command="bzr --serve --inet --allow-writes --directory=/srv/bzr/bob/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Bob's SSH Key
+  command="bzr --serve --inet --allow-writes --directory=/srv/bzr/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Repo Manager SSH Key
+
+Alice and Bob have access to their own repository and Repo Manager
+has access to the each of their repositories.  Users are not allowed access to any part of 
+the system except the directory specified. The bzr+ssh urls are simplified by 
+serving using ``bzr serve`` and the ``--directory`` option.
+
+If Alice logs in she uses the following command for her fix-1023 branch::
+
+  $ bzr log bzr+ssh://bzruser@server.example.com/fix-1023
+
+If Repo Manager logs in he uses the following command to access Alice's
+fix-1023::
+
+  $ bzr log bzr+ssh://bzruser@server.example.com/alice/fix-1023
+

=== modified file 'doc/en/admin-guide/upgrade.txt'
--- doc/en/admin-guide/upgrade.txt	2010-06-02 05:03:31 +0000
+++ doc/en/admin-guide/upgrade.txt	2010-08-13 19:08:57 +0000
@@ -29,8 +29,8 @@
 Upgrading the Bazaar software is as simple as re-installing the Python package
 using either the latest binary package for Windows or Mac OS X, the binary
 package provided by your GNU/Linux distribution, or installing from the source
-release.  See http://bazaar-vcs.org/Downloads for the latest releases for all
-supported platforms.
+release.  See http://wiki.bazaar.canonical.com/Downloads for the latest
+releases for all supported platforms.
 
 Bazaar's later versions support all of the earlier disk formats (back to the
 very first one), so there is no need to upgrade the branches on the disk when

=== modified file 'doc/en/conf.py'
--- doc/en/conf.py	2010-07-05 12:40:28 +0000
+++ doc/en/conf.py	2010-10-13 06:14:37 +0000
@@ -55,7 +55,7 @@
     bzr_title(u'Bazaar User Guide'), bzr_team, 'manual'),
   ('user-reference/index', 'bzr-%s-user-reference.tex' % (bzr_locale,),
     bzr_title(u'Bazaar User Reference'), bzr_team, 'manual'),
-  ('release-notes/NEWS', 'bzr-%s-release-notes.tex' % (bzr_locale,),
+  ('release-notes/index', 'bzr-%s-release-notes.tex' % (bzr_locale,),
     bzr_title(u'Bazaar Release Notes'), bzr_team, 'manual'),
   ('upgrade-guide/index', 'bzr-%s-upgrade-guide.tex' % (bzr_locale,),
     bzr_title(u'Bazaar Upgrade Guide'), bzr_team, 'manual'),
@@ -87,6 +87,9 @@
     'user-reference/bzr_man',
     'user-guide/index-plain',
     'admin-guide/index-plain',
+    # Templates
+    'release-notes/release-template',
+    'release-notes/series-template',
     # Miscellaneous
     'user-reference/readme',
 ]

=== modified file 'doc/en/index.txt'
--- doc/en/index.txt	2010-03-04 04:15:18 +0000
+++ doc/en/index.txt	2011-08-30 09:41:34 +0000
@@ -10,7 +10,7 @@
 .. toctree::
    :maxdepth: 1
 
-   whats-new/whats-new-in-2.2
+   whats-new/whats-new-in-2.5
    user-guide/index
    tutorials/index
    quick-reference/index

=== modified file 'doc/en/mini-tutorial/index.txt'
--- doc/en/mini-tutorial/index.txt	2010-07-05 15:50:19 +0000
+++ doc/en/mini-tutorial/index.txt	2011-07-01 22:01:47 +0000
@@ -12,9 +12,6 @@
 version control, how to record changes to them, examine your work, publish
 it and send your work for merger into a project's trunk.
 
-If you'd prefer a more detailed introduction, take a look at
-`Learning More`_.
-
 
 Installation
 ============
@@ -29,78 +26,71 @@
 For other platforms and to install from source code, see the Download_
 and Installation_ pages.
 
-.. _installation instructions for Windows: http://bazaar-vcs.org/WindowsDownloads
-.. _installation instructions for Mac OS X: http://bazaar-vcs.org/MacOSXBundle
-.. _Download: http://bazaar-vcs.org/Download
-.. _Installation: http://bazaar-vcs.org/InstallationFaq
+.. _installation instructions for Windows: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _installation instructions for Mac OS X: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _Download: http://wiki.bazaar.canonical.com/Download
+.. _Installation: http://wiki.bazaar.canonical.com/InstallationFaq
 
 
 Introducing yourself
 ====================
 
-Before you start working, it is good to tell Bazaar who you are. That
-way your work is properly identified in revision logs.
+Bazaar records changes to source code, and it records who made the change.
+The person is identified by their name and email address.  (If you're
+concerned about spam, you don't need to use a real address that you
+actually read, but the convention is that it looks like an email address.)
 
-Using your name and email address, instead of John Doe's, type::
+Before you start working, let's tell Bazaar who you are.  Using your name
+and email address, instead of John Doe's, type::
 
   $ bzr whoami "John Doe <john.doe@gmail.com>"
 
-Bazaar will now create or modify a configuration file, including your
-name and email address.
-
-Now, check that your name and email address are correctly registered::
+You can check what identity is stored in Bazaar's configuration::
 
   $ bzr whoami
   John Doe <john.doe@gmail.com>
 
 
-Putting files under version control
-===================================
-
-Let's create a directory and some files to use with Bazaar::
-
- $ mkdir myproject
- $ cd myproject
- $ mkdir subdirectory
- $ touch test1.txt test2.txt test3.txt subdirectory/test4.txt
-
-**Note for Windows users**: use Windows Explorer to create your
-directories, then right-click in those directories and select
-``New file`` to create your files.
-
-Now get Bazaar to initialize itself in your project directory::
-
-  $ bzr init
-
-If it looks like nothing happened, don't worry. Bazaar has created a
-branch_ where it will store your files and their revision histories.
-
-.. _branch: http://bazaar-vcs.org/Branch
-
-The next step is to tell Bazaar which files you want to track. Running
-``bzr add`` will recursively add everything in the project::
-
- $ bzr add
- added subdirectory
- added test1.txt
- added test2.txt
- added test3.txt
- added subdirectory/test4.txt
-
-Next, take a snapshot of your files by committing them to your branch. Add
-a message to explain why you made the commit::
-
-  $ bzr commit -m "Initial import"
-
-As Bazaar is a distributed version control system, it doesn't need to
-connect to a central server to make the commit. Instead, Bazaar stores your
-branch and all its commits inside the directory you're working with; look
-for the ``.bzr`` sub-directory.
+Starting a new project
+======================
+
+Let's suppose we want to store a new project under Bazaar.  First, we'll
+make a *repository directory* to hold all our work related to this
+project, where developers can create branches to test development of
+specific features or, more generally, modifications to the working file
+set.  
+
+After creating the repository, change to that directory, and create the
+project's main trunk branch.
+
+::
+
+  bzr init-repo sample
+  cd sample
+  bzr init trunk
+  cd trunk
+
+
+Adding files
+============
+
+Now that we have the trunk, we need to move to that directory and
+create some example files for the first version of that project.  Create
+a file ``test1.txt`` using a text editor (like emacs, nano, or notepad),
+and save it.  Then we'll "add" the file, which tells bzr we want it to
+track changes::
+
+  bzr add test1.txt
+
+and then commit, which saves a snapshot of all versioned files::
+
+  bzr commit -m "Added first line of text"
 
 
 Making changes to your files
 ============================
 
+
 Let's change a file and commit that change to your branch.
 
 Edit ``test1.txt`` in your favourite editor, then check what have you done::
@@ -140,39 +130,12 @@
     Initial import
 
 
-Publishing your branch with sftp
-================================
-
-There are a couple of ways to publish your branch. If you already have
-an SFTP server or are comfortable setting one up, you can publish your
-branch to it.
-
-Otherwise, skip to the next section to publish with Launchpad_, a free
-hosting service for Bazaar.
-
-.. _Launchpad: https://launchpad.net/
-
-Let's assume you want to publish your branch at ``www.example.com/myproject``::
-
- $ bzr push --create-prefix sftp://your.name@example.com/~/public_html/myproject
- 2 revision(s) pushed.
-
-Bazaar will create a ``myproject`` directory on the remote server and
-push your branch to it.
-
-Now anyone can create their own copy of your branch by typing::
-
- $ bzr branch http://www.example.com/myproject
-
-**Note**: to use sftp, you may need to install ``paramiko`` and
-``pyCrypto``. See http://bazaar-vcs.org/InstallationFaq for details.
-
-
-Publishing your branch with Launchpad
-=====================================
-
-Launchpad is a suite of development and hosting tools for free
-software projects. You can use it to publish your branch.
+Publishing your branch on Launchpad
+===================================
+
+Launchpad is a suite of development and hosting tools for
+software projects. You can use it to publish your branch.  (You can 
+also publish branches onto your own server or other hosting services.)
 
 If you don't have a Launchpad account, follow the `account signup guide`_
 and `register an SSH key`_ in your new Launchpad account.
@@ -180,14 +143,14 @@
 .. _account signup guide: https://help.launchpad.net/CreatingYourLaunchpadAccount
 .. _register an SSH key: https://launchpad.net/people/+me/+editsshkeys
 
-Replacing ``john.doe`` with your own Launchpad username, type [#]_::
+Replacing ``john.doe`` with your own Launchpad username, type::
 
  $ bzr push lp:~john.doe/+junk/myproject
 
-.. [#] Use of the ``lp:`` URL scheme requires bzr 0.92 or later.
-
-**Note**: ``+junk`` means that this branch isn't associated with any particular
-project in Launchpad.
+**Note**: ``+junk`` is a place to store experimental branches not
+associated with any particular project.  Normally, you should push a
+project into an existing project, or register a new project through the
+web interface.
 
 Now, anyone can create their own copy of your branch by typing::
 
@@ -203,11 +166,12 @@
 To work with someone else's code, you can make your own copy of their
 branch. Let's take a real-world example, Bazaar's GTK interface::
 
-  $ bzr branch lp:~bzr/bzr-gtk/trunk bzr-gtk.john
+  $ bzr init-repo ~/bzr-gtk
+  $ bzr branch lp:~bzr/bzr-gtk/trunk ~/bzr-gtk/john
   Branched 292 revision(s).
 
 Bazaar will download all the files and complete revision history from the
-bzr-gtk project's trunk branch and create a copy called bzr-gtk.john.
+bzr-gtk project's trunk branch and create a copy called ``john``.
 
 Now, you have your own copy of the branch and can commit changes with
 or without a net connection. You can share your branch at any time by
@@ -232,6 +196,13 @@
 
   $ bzr diff
 
+If different branches have made changes to the same areas of the same
+files, then merging them may generate conflicts.  When this happens,
+Bazaar puts text markers like ``<<<<<<<`` into the files, and records them
+in a list of conflicted files.  You should edit the files to reflect the
+way you want to resolve the conflicts, use ``bzr diff`` to check the
+changes, and then ``bzr resolve`` to mark them as resolved.
+
 If you're happy with the changes, you can commit them to your personal
 branch::
 
@@ -239,27 +210,6 @@
   Committed revision 295.
 
 
-Merging your work into the parent branch
-========================================
-
-After you've worked on your personal branch of bzr-gtk, you may want to
-send your changes back upstream to the project. The easiest way is to
-use a merge directive.
-
-A merge directive is a machine-readable request to perform a
-particular merge.  It usually contains a patch preview of the merge
-and either contains the necessary revisions, or provides a branch
-where they can be found.
-
-Replacing ``mycode.patch``, create your merge directive::
-
- $ bzr send -o mycode.patch
- Using saved parent location: http://bazaar.launchpad.net/~bzr/bzr-gtk/trunk
-
-You can now email the merge directive to the bzr-gtk project who, if
-they choose, can use it merge your work back into the parent branch.
-
-
 Learning more
 =============
 
@@ -270,10 +220,14 @@
 
   $ bzr help
 
-To learn about Bazaar commands::
-
-  $ bzr help commands
-
 To learn about the ''foo'' topic or command::
 
   $ bzr help foo
+
+Licence
+=======
+
+Copyright 2007-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later. See
+<http://www.gnu.org/licenses/>.

=== added file 'doc/en/release-notes/bzr-0.1.txt'
--- doc/en/release-notes/bzr-0.1.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.1.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,753 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.1.1
+#########
+
+:Released: 2005-10-12
+
+Bug Fixes
+*********
+
+* Fix problem in pulling over HTTP from machines that do not
+  allow directories to be listed.
+
+* Avoid harmless warning about invalid hash cache after
+  upgrading branch format.
+
+Performance
+***********
+
+* Avoid some unnecessary HTTP operations in branch and pull.
+
+
+bzr 0.1
+#######
+
+:Released: 2005-10-11
+
+Notes
+*****
+
+* 'bzr branch' over HTTP initially gives a very high estimate
+  of completion time but it should fall as the first few
+  revisions are pulled in.  branch is still slow on
+  high-latency connections.
+
+Bug Fixes
+*********
+
+* bzr-man.py has been updated to work again. Contributed by
+  Rob Weir.
+
+* Locking is now done with fcntl.lockf which works with NFS
+  file systems. Contributed by Harald Meland.
+
+* When a merge encounters a file that has been deleted on
+  one side and modified on the other, the old contents are
+  written out to foo.BASE and foo.SIDE, where SIDE is this
+  or OTHER. Contributed by Aaron Bentley.
+
+* Export was choosing incorrect file paths for the content of
+  the tarball, this has been fixed by Aaron Bentley.
+
+* Commit will no longer commit without a log message, an
+  error is returned instead. Contributed by Jelmer Vernooij.
+
+* If you commit a specific file in a sub directory, any of its
+  parent directories that are added but not listed will be
+  automatically included. Suggested by Michael Ellerman.
+
+* bzr commit and upgrade did not correctly record new revisions
+  for files with only a change to their executable status.
+  bzr will correct this when it encounters it. Fixed by
+  Robert Collins
+
+* HTTP tests now force off the use of ``http_proxy`` for the duration.
+  Contributed by Gustavo Niemeyer.
+
+* Fix problems in merging weave-based branches that have
+  different partial views of history.
+
+* Symlink support: working with symlinks when not in the root of a
+  bzr tree was broken, patch from Scott James Remnant.
+
+Improvements
+************
+
+* 'branch' now accepts a --basis parameter which will take advantage
+  of local history when making a new branch. This allows faster
+  branching of remote branches. Contributed by Aaron Bentley.
+
+* New tree format based on weave files, called version 5.
+  Existing branches can be upgraded to this format using
+  'bzr upgrade'.
+
+* Symlinks are now versionable. Initial patch by
+  Erik Toubro Nielsen, updated to head by Robert Collins.
+
+* Executable bits are tracked on files. Patch from Gustavo
+  Niemeyer.
+
+* 'bzr status' now shows unknown files inside a selected directory.
+  Patch from Heikki Paajanen.
+
+* Merge conflicts are recorded in .bzr. Two new commands 'conflicts'
+  and 'resolve' have needed added, which list and remove those
+  merge conflicts respectively. A conflicted tree cannot be committed
+  in. Contributed by Aaron Bentley.
+
+* 'rm' is now an alias for 'remove'.
+
+* Stores now split out their content in a single byte prefixed hash,
+  dropping the density of files per directory by 256. Contributed by
+  Gustavo Niemeyer.
+
+* 'bzr diff -r branch:URL' will now perform a diff between two branches.
+  Contributed by Robert Collins.
+
+* 'bzr log' with the default formatter will show merged revisions,
+  indented to the right. Initial implementation contributed by Gustavo
+  Niemeyer, made incremental by Robert Collins.
+
+
+Internals
+*********
+
+* Test case failures have the exception printed after the log
+  for your viewing pleasure.
+
+* InventoryEntry is now an abstract base class, use one of the
+  concrete InventoryDirectory etc classes instead.
+
+* Branch raises an UnsupportedFormatError when it detects a
+  bzr branch it cannot understand. This allows for precise
+  handling of such circumstances.
+
+* Remove RevisionReference class; ``Revision.parent_ids`` is now simply a
+  list of their ids and ``parent_sha1s`` is a list of their corresponding
+  sha1s (for old branches only at the moment.)
+
+* New method-object style interface for Commit() and Fetch().
+
+* Renamed ``Branch.last_patch()`` to ``Branch.last_revision()``, since
+  we call them revisions not patches.
+
+* Move ``copy_branch`` to ``bzrlib.clone.copy_branch``.  The destination
+  directory is created if it doesn't exist.
+
+* Inventories now identify the files which were present by
+  giving the revision *of that file*.
+
+* Inventory and Revision XML contains a version identifier.
+  This must be consistent with the overall branch version
+  but allows for more flexibility in future upgrades.
+
+Testing
+*******
+
+* Removed testsweet module so that tests can be run after
+  bzr installed by 'bzr selftest'.
+
+* 'bzr selftest' command-line arguments can now be partial ids
+  of tests to run, e.g. ``bzr selftest test_weave``
+
+bzr 0.0.9
+#########
+
+:Released: 2005-09-23
+
+Bug Fixes
+*********
+
+* Fixed "branch -r" option.
+
+* Fix remote access to branches containing non-compressed history.
+  (Robert Collins).
+
+* Better reliability of HTTP server tests.  (John Arbash-Meinel)
+
+* Merge graph maximum distance calculation fix.  (Aaron Bentley)
+
+* Various minor bug in windows support have been fixed, largely in the
+  test suite. Contributed by Alexander Belchenko.
+
+Improvements
+************
+
+* Status now accepts a -r argument to give status between chosen
+  revisions. Contributed by Heikki Paajanen.
+
+* Revision arguments no longer use +/-/= to control ranges, instead
+  there is a 'before' namespace, which limits the successive namespace.
+  For example '$ bzr log -r date:yesterday..before:date:today' will
+  select everything from yesterday and before today. Contributed by
+  Robey Pointer
+
+* There is now a bzr.bat file created by distutils when building on
+  Windows. Contributed by Alexander Belchenko.
+
+Internals
+*********
+
+* Removed uuid() as it was unused.
+
+* Improved 'fetch' code for pulling revisions from one branch into
+  another (used by pull, merged, etc.)
+
+
+bzr 0.0.8
+#########
+
+:Released: 2005-09-20
+
+
+Improvements
+************
+
+* Adding a file whose parent directory is not versioned will
+  implicitly add the parent, and so on up to the root. This means
+  you should never need to explictly add a directory, they'll just
+  get added when you add a file in the directory.  Contributed by
+  Michael Ellerman.
+
+* Ignore ``.DS_Store`` (contains Mac metadata) by default.
+  (Nir Soffer)
+
+* If you set ``BZR_EDITOR`` in the environment, it is checked in
+  preference to EDITOR and the config file for the interactive commit
+  editing program. Related to this is a bugfix where a missing program
+  set in EDITOR would cause editing to fail, now the fallback program
+  for the operating system is still tried.
+
+* Files that are not directories/symlinks/regular files will no longer
+  cause bzr to fail, it will just ignore them by default. You cannot add
+  them to the tree though - they are not versionable.
+
+
+Internals
+*********
+
+* Refactor XML packing/unpacking.
+
+Bug Fixes
+*********
+
+* Fixed 'bzr mv' by Ollie Rutherfurd.
+
+* Fixed strange error when trying to access a nonexistent HTTP
+  branch.
+
+* Make sure that the hashcache gets written out if it can't be
+  read.
+
+
+Portability
+***********
+
+* Various Windows fixes from Ollie Rutherfurd.
+
+* Quieten warnings about locking; patch from Matt Lavin.
+
+
+bzr-0.0.7
+#########
+
+:Released: 2005-09-02
+
+New Features
+************
+
+* ``bzr shell-complete`` command contributed by Clint Adams to
+  help with intelligent shell completion.
+
+* New expert command ``bzr find-merge-base`` for debugging merges.
+
+
+Enhancements
+************
+
+* Much better merge support.
+
+* merge3 conflicts are now reported with markers like '<<<<<<<'
+  (seven characters) which is the same as CVS and pleases things
+  like emacs smerge.
+
+
+Bug Fixes
+*********
+
+* ``bzr upgrade`` no longer fails when trying to fix trees that
+  mention revisions that are not present.
+
+* Fixed bugs in listing plugins from ``bzr plugins``.
+
+* Fix case of $EDITOR containing options for the editor.
+
+* Fix log -r refusing to show the last revision.
+  (Patch from Goffredo Baroncelli.)
+
+
+Changes
+*******
+
+* ``bzr log --show-ids`` shows the revision ids of all parents.
+
+* Externally provided commands on your $BZRPATH no longer need
+  to recognize --bzr-usage to work properly, and can just handle
+  --help themselves.
+
+
+Library
+*******
+
+* Changed trace messages to go through the standard logging
+  framework, so that they can more easily be redirected by
+  libraries.
+
+
+
+bzr-0.0.6
+#########
+
+:Released: 2005-08-18
+
+New Features
+************
+
+* Python plugins, automatically loaded from the directories on
+  ``BZR_PLUGIN_PATH`` or ``~/.bzr.conf/plugins`` by default.
+
+* New 'bzr mkdir' command.
+
+* Commit mesage is fetched from an editor if not given on the
+  command line; patch from Torsten Marek.
+
+* ``bzr log -m FOO`` displays commits whose message matches regexp
+  FOO.
+
+* ``bzr add`` with no arguments adds everything under the current directory.
+
+* ``bzr mv`` does move or rename depending on its arguments, like
+  the Unix command.
+
+* ``bzr missing`` command shows a summary of the differences
+  between two trees.  (Merged from John Arbash-Meinel.)
+
+* An email address for commits to a particular tree can be
+  specified by putting it into .bzr/email within a branch.  (Based
+  on a patch from Heikki Paajanen.)
+
+
+Enhancements
+************
+
+* Faster working tree operations.
+
+
+Changes
+*******
+
+* 3rd-party modules shipped with bzr are copied within the bzrlib
+  python package, so that they can be installed by the setup
+  script without clashing with anything already existing on the
+  system.  (Contributed by Gustavo Niemeyer.)
+
+* Moved plugins directory to bzrlib/, so that there's a standard
+  plugin directory which is not only installed with bzr itself but
+  is also available when using bzr from the development tree.
+  ``BZR_PLUGIN_PATH`` and ``DEFAULT_PLUGIN_PATH`` are then added to the
+  standard plugins directory.
+
+* When exporting to a tarball with ``bzr export --format tgz``, put
+  everything under a top directory rather than dumping it into the
+  current directory.   This can be overridden with the ``--root``
+  option.  Patch from William Dodé and John Meinel.
+
+* New ``bzr upgrade`` command to upgrade the format of a branch,
+  replacing ``bzr check --update``.
+
+* Files within store directories are no longer marked readonly on
+  disk.
+
+* Changed ``bzr log`` output to a more compact form suggested by
+  John A Meinel.  Old format is available with the ``--long`` or
+  ``-l`` option, patched by William Dodé.
+
+* By default the commit command refuses to record a revision with
+  no changes unless the ``--unchanged`` option is given.
+
+* The ``--no-plugins``, ``--profile`` and ``--builtin`` command
+  line options must come before the command name because they
+  affect what commands are available; all other options must come
+  after the command name because their interpretation depends on
+  it.
+
+* ``branch`` and ``clone`` added as aliases for ``branch``.
+
+* Default log format is back to the long format; the compact one
+  is available with ``--short``.
+
+
+Bug Fixes
+*********
+
+* Fix bugs in committing only selected files or within a subdirectory.
+
+
+bzr-0.0.5
+#########
+
+:Released:  2005-06-15
+
+Changes
+*******
+
+* ``bzr`` with no command now shows help rather than giving an
+  error.  Suggested by Michael Ellerman.
+
+* ``bzr status`` output format changed, because svn-style output
+  doesn't really match the model of bzr.  Now files are grouped by
+  status and can be shown with their IDs.  ``bzr status --all``
+  shows all versioned files and unknown files but not ignored files.
+
+* ``bzr log`` runs from most-recent to least-recent, the reverse
+  of the previous order.  The previous behaviour can be obtained
+  with the ``--forward`` option.
+
+* ``bzr inventory`` by default shows only filenames, and also ids
+  if ``--show-ids`` is given, in which case the id is the second
+  field.
+
+
+Enhancements
+************
+
+* New 'bzr whoami --email' option shows only the email component
+  of the user identification, from Jo Vermeulen.
+
+* New ``bzr ignore PATTERN`` command.
+
+* Nicer error message for broken pipe, interrupt and similar
+  conditions that don't indicate an internal error.
+
+* Add ``.*.sw[nop] .git .*.tmp *,v`` to default ignore patterns.
+
+* Per-branch locks keyed on ``.bzr/branch-lock``, available in
+  either read or write mode.
+
+* New option ``bzr log --show-ids`` shows revision and file ids.
+
+* New usage ``bzr log FILENAME`` shows only revisions that
+  affected that file.
+
+* Changed format for describing changes in ``bzr log -v``.
+
+* New option ``bzr commit --file`` to take a message from a file,
+  suggested by LarstiQ.
+
+* New syntax ``bzr status [FILE...]`` contributed by Bartosz
+  Oler.  File may be in a branch other than the working directory.
+
+* ``bzr log`` and ``bzr root`` can be given an HTTP URL instead of
+  a filename.
+
+* Commands can now be defined by external programs or scripts
+  in a directory on $BZRPATH.
+
+* New "stat cache" avoids reading the contents of files if they
+  haven't changed since the previous time.
+
+* If the Python interpreter is too old, try to find a better one
+  or give an error.  Based on a patch from Fredrik Lundh.
+
+* New optional parameter ``bzr info [BRANCH]``.
+
+* New form ``bzr commit SELECTED`` to commit only selected files.
+
+* New form ``bzr log -r FROM:TO`` shows changes in selected
+  range; contributed by John A Meinel.
+
+* New option ``bzr diff --diff-options 'OPTS'`` allows passing
+  options through to an external GNU diff.
+
+* New option ``bzr add --no-recurse`` to add a directory but not
+  their contents.
+
+* ``bzr --version`` now shows more information if bzr is being run
+  from a branch.
+
+
+Bug Fixes
+*********
+
+* Fixed diff format so that added and removed files will be
+  handled properly by patch.  Fix from Lalo Martins.
+
+* Various fixes for files whose names contain spaces or other
+  metacharacters.
+
+
+Testing
+*******
+
+* Converted black-box test suites from Bourne shell into Python;
+  now run using ``./testbzr``.  Various structural improvements to
+  the tests.
+
+* testbzr by default runs the version of bzr found in the same
+  directory as the tests, or the one given as the first parameter.
+
+* testbzr also runs the internal tests, so the only command
+  required to check is just ``./testbzr``.
+
+* testbzr requires python2.4, but can be used to test bzr running
+  under a different version.
+
+* Tests added for many other changes in this release.
+
+
+Internal
+********
+
+* Included ElementTree library upgraded to 1.2.6 by Fredrik Lundh.
+
+* Refactor command functions into Command objects based on HCT by
+  Scott James Remnant.
+
+* Better help messages for many commands.
+
+* Expose ``bzrlib.open_tracefile()`` to start the tracefile; until
+  this is called trace messages are just discarded.
+
+* New internal function ``find_touching_revisions()`` and hidden
+  command touching-revisions trace the changes to a given file.
+
+* Simpler and faster ``compare_inventories()`` function.
+
+* ``bzrlib.open_tracefile()`` takes a tracefilename parameter.
+
+* New AtomicFile class.
+
+* New developer commands ``added``, ``modified``.
+
+
+Portability
+***********
+
+* Cope on Windows on python2.3 by using the weaker random seed.
+  2.4 is now only recommended.
+
+
+bzr-0.0.4
+#########
+
+:Released:  2005-04-22
+
+Enhancements
+************
+
+* 'bzr diff' optionally takes a list of files to diff.  Still a bit
+  basic.  Patch from QuantumG.
+
+* More default ignore patterns.
+
+* New 'bzr log --verbose' shows a list of files changed in the
+  changeset.  Patch from Sebastian Cote.
+
+* Roll over ~/.bzr.log if it gets too large.
+
+* Command abbreviations 'ci', 'st', 'stat', '?' based on a patch
+  by Jason Diamon.
+
+* New 'bzr help commands' based on a patch from Denys Duchier.
+
+
+Changes
+*******
+
+* User email is determined by looking at $BZREMAIL or ~/.bzr.email
+  or $EMAIL.  All are decoded by the locale preferred encoding.
+  If none of these are present user@hostname is used.  The host's
+  fully-qualified name is not used because that tends to fail when
+  there are DNS problems.
+
+* New 'bzr whoami' command instead of username user-email.
+
+
+Bug Fixes
+*********
+
+* Make commit safe for hardlinked bzr trees.
+
+* Some Unicode/locale fixes.
+
+* Partial workaround for ``difflib.unified_diff`` not handling
+  trailing newlines properly.
+
+
+Internal
+********
+
+* Allow docstrings for help to be in PEP0257 format.  Patch from
+  Matt Brubeck.
+
+* More tests in test.sh.
+
+* Write profile data to a temporary file not into working
+  directory and delete it when done.
+
+* Smaller .bzr.log with process ids.
+
+
+Portability
+***********
+
+* Fix opening of ~/.bzr.log on Windows.  Patch from Andrew
+  Bennetts.
+
+* Some improvements in handling paths on Windows, based on a patch
+  from QuantumG.
+
+
+bzr-0.0.3
+#########
+
+:Released:  2005-04-06
+
+Enhancements
+************
+
+* New "directories" internal command lists versioned directories
+  in the tree.
+
+* Can now say "bzr commit --help".
+
+* New "rename" command to rename one file to a different name
+  and/or directory.
+
+* New "move" command to move one or more files into a different
+  directory.
+
+* New "renames" command lists files renamed since base revision.
+
+* New cat command contributed by janmar.
+
+Changes
+*******
+
+* .bzr.log is placed in $HOME (not pwd) and is always written in
+  UTF-8.  (Probably not a completely good long-term solution, but
+  will do for now.)
+
+Portability
+***********
+
+* Workaround for difflib bug in Python 2.3 that causes an
+  exception when comparing empty files.  Reported by Erik Toubro
+  Nielsen.
+
+Internal
+********
+
+* Refactored inventory storage to insert a root entry at the top.
+
+Testing
+*******
+
+* Start of shell-based black-box testing in test.sh.
+
+
+bzr-0.0.2.1
+###########
+
+Portability
+***********
+
+* Win32 fixes from Steve Brown.
+
+
+bzr-0.0.2
+#########
+
+:Codename: "black cube"
+:Released: 2005-03-31
+
+Enhancements
+************
+
+* Default ignore list extended (see bzrlib/__init__.py).
+
+* Patterns in .bzrignore are now added to the default ignore list,
+  rather than replacing it.
+
+* Ignore list isn't reread for every file.
+
+* More help topics.
+
+* Reinstate the 'bzr check' command to check invariants of the
+  branch.
+
+* New 'ignored' command lists which files are ignored and why;
+  'deleted' lists files deleted in the current working tree.
+
+* Performance improvements.
+
+* New global --profile option.
+
+* Ignore patterns like './config.h' now correctly match files in
+  the root directory only.
+
+
+bzr-0.0.1
+#########
+
+:Released:  2005-03-26
+
+Enhancements
+************
+
+* More information from info command.
+
+* Can now say "bzr help COMMAND" for more detailed help.
+
+* Less file flushing and faster performance when writing logs and
+  committing to stores.
+
+* More useful verbose output from some commands.
+
+Bug Fixes
+*********
+
+* Fix inverted display of 'R' and 'M' during 'commit -v'.
+
+Portability
+***********
+
+* Include a subset of ElementTree-1.2.20040618 to make
+  installation easier.
+
+* Fix time.localtime call to work with Python 2.3 (the minimum
+  supported).
+
+
+bzr-0.0.0.69
+############
+
+:Released:  2005-03-22
+
+Enhancements
+************
+
+* First public release.
+
+* Storage of local versions: init, add, remove, rm, info, log,
+  diff, status, etc.
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.10.txt'
--- doc/en/release-notes/bzr-0.10.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.10.txt	2010-10-08 08:15:14 +0000
@@ -0,0 +1,90 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.10
+########
+
+:Released:  2006-08-29
+
+Improvements
+************
+* 'merge' now takes --uncommitted, to apply uncommitted changes from a
+  tree.  (Aaron Bentley)
+
+* 'bzr add --file-ids-from' can be used to specify another path to use
+  for creating file ids, rather than generating all new ones. Internally,
+  the 'action' passed to ``smart_add_tree()`` can return ``file_ids`` that
+  will be used, rather than having bzrlib generate new ones.
+  (John Arbash Meinel, #55781)
+
+* ``bzr selftest --benchmark`` now allows a ``--cache-dir`` parameter.
+  This will cache some of the intermediate trees, and decrease the
+  setup time for benchmark tests. (John Arbash Meinel)
+
+* Inverse forms are provided for all boolean options.  For example,
+  --strict has --no-strict, --no-recurse has --recurse (Aaron Bentley)
+
+* Serialize out Inventories directly, rather than using ElementTree.
+  Writing out a kernel sized inventory drops from 2s down to ~350ms.
+  (Robert Collins, John Arbash Meinel)
+
+Bug Fixes
+*********
+
+* Help diffutils 2.8.4 get along with binary tests (Marien Zwart: #57614)
+
+* Change LockDir so that if the lock directory doesn't exist when
+  ``lock_write()`` is called, an attempt will be made to create it.
+  (John Arbash Meinel, #56974)
+
+* ``bzr uncommit`` preserves pending merges. (John Arbash Meinel, #57660)
+
+* Active FTP transport now works as intended. (ghozzy, #56472)
+
+* Really fix mutter() so that it won't ever raise a UnicodeError.
+  It means it is possible for ~/.bzr.log to contain non UTF-8 characters.
+  But it is a debugging log, not a real user file.
+  (John Arbash Meinel, #56947, #53880)
+
+* Change Command handle to allow Unicode command and options.
+  At present we cannot register Unicode command names, so we will get
+  BzrCommandError('unknown command'), or BzrCommandError('unknown option')
+  But that is better than a UnicodeError + a traceback.
+  (John Arbash Meinel, #57123)
+
+* Handle TZ=UTC properly when reading/writing revisions.
+  (John Arbash Meinel, #55783, #56290)
+
+* Use ``GPG_TTY`` to allow gpg --cl to work with gpg-agent in a pipeline,
+  (passing text to sign in on stdin). (John Arbash Meinel, #54468)
+
+* External diff does the right thing for binaries even in foreign
+  languages. (John Arbash Meinel, #56307)
+
+* Testament handles more cases when content is unicode. Specific bug was
+  in handling of revision properties.
+  (John Arbash Meinel, Holger Krekel, #54723)
+
+* The bzr selftest was failing on installed versions due to a bug in a new
+  test helper. (John Arbash Meinel, Robert Collins, #58057)
+
+Internals
+*********
+
+* ``bzrlib.cache_utf8`` contains ``encode()`` and ``decode()`` functions
+  which can be used to cache the conversion between utf8 and Unicode.
+  Especially helpful for some of the knit annotation code, which has to
+  convert revision ids to utf8 to annotate lines in storage.
+  (John Arbash Meinel)
+
+* ``setup.py`` now searches the filesystem to find all packages which
+  need to be installed. This should help make the life of packagers
+  easier. (John Arbash Meinel)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.11.txt'
--- doc/en/release-notes/bzr-0.11.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.11.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,226 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.11
+########
+
+:Released:  2006-10-02
+
+* Smart server transport test failures on windows fixed. (Lukáš Lalinský).
+
+bzr 0.11rc2
+###########
+
+:Released:  2006-09-27
+
+Bug Fixes
+*********
+
+* Test suite hangs on windows fixed. (Andrew Bennets, Alexander Belchenko).
+
+* Commit performance regression fixed. (Aaron Bentley, Robert Collins, John
+  Arbash Meinel).
+
+bzr 0.11rc1
+###########
+
+:Released:  2006-09-25
+
+Improvements
+************
+
+* Knit files now wait to create their contents until the first data is
+  added. The old code used to create an empty .knit and a .kndx with just
+  the header. However, this caused a lot of extra round trips over SFTP.
+  This can change the time for ``bzr push`` to create a new remote branch
+  from 160s down to 100s. This also affects ``bzr commit`` performance when
+  adding new files, ``bzr commit`` on a new kernel-like tree drops from 50s
+  down to 40s (John Arbash Meinel, #44692)
+
+* When an entire subtree has been deleted, commit will now report that
+  just the top of the subtree has been deleted, rather than reporting
+  all the individual items. (Robert Collins)
+
+* Commit performs one less XML parse. (Robert Collins)
+
+* ``bzr checkout`` now operates on readonly branches as well
+  as readwrite branches. This fixes bug #39542. (Robert Collins)
+
+* ``bzr bind`` no longer synchronises history with the master branch.
+  Binding should be followed by an update or push to synchronise the
+  two branches. This is closely related to the fix for bug #39542.
+  (Robert Collins)
+
+* ``bzrlib.lazy_import.lazy_import`` function to create on-demand
+  objects.  This allows all imports to stay at the global scope, but
+  modules will not actually be imported if they are not used.
+  (John Arbash Meinel)
+
+* Support ``bzr://`` and ``bzr+ssh://`` URLS to work with the new RPC-based
+  transport which will be used with the upcoming high-performance smart
+  server. The new command ``bzr serve`` will invoke bzr in server mode,
+  which processes these requests. (Andrew Bennetts, Robert Collins, Martin
+  Pool)
+
+* New command ``bzr version-info`` which can be used to get a summary
+  of the current state of the tree. This is especially useful as part
+  of a build commands. See ``doc/version_info.txt`` for more information
+  (John Arbash Meinel)
+
+Bug Fixes
+*********
+
+* ``'bzr inventory [FILE...]'`` allows restricting the file list to a
+  specific set of files. (John Arbash Meinel, #3631)
+
+* Don't abort when annotating empty files (John Arbash Meinel, #56814)
+
+* Add ``Stanza.to_unicode()`` which can be passed to another Stanza
+  when nesting stanzas. Also, add ``read_stanza_unicode`` to handle when
+  reading a nested Stanza. (John Arbash Meinel)
+
+* Transform._set_mode() needs to stat the right file.
+  (John Arbash Meinel, #56549)
+
+* Raise WeaveFormatError rather than StopIteration when trying to read
+  an empty Weave file. (John Arbash Meinel, #46871)
+
+* Don't access e.code for generic URLErrors, only HTTPErrors have .code.
+  (Vincent Ladeuil, #59835)
+
+* Handle boundary="" lines properly to allow access through a Squid proxy.
+  (John Arbash Meinel, #57723)
+
+* revert now removes newly-added directories (Aaron Bentley, #54172)
+
+* ``bzr upgrade sftp://`` shouldn't fail to upgrade v6 branches if there
+  isn't a working tree. (David Allouche, #40679)
+
+* Give nicer error messages when a user supplies an invalid --revision
+  parameter. (John Arbash Meinel, #55420)
+
+* Handle when LANG is not recognized by python. Emit a warning, but
+  just revert to using 'ascii'. (John Arbash Meinel, #35392)
+
+* Don't use ``preexec_fn`` on win32, as it is not supported by subprocess.
+  (John Arbash Meinel)
+
+* Skip specific tests when the dependencies aren't met. This includes
+  some ``setup.py`` tests when ``python-dev`` is not available, and
+  some tests that depend on paramiko. (John Arbash Meinel, Mattheiu Moy)
+
+* Fallback to Paramiko properly, if no ``ssh`` executable exists on
+  the system. (Andrew Bennetts, John Arbash Meinel)
+
+* ``Branch.bind(other_branch)`` no longer takes a write lock on the
+  other branch, and will not push or pull between the two branches.
+  API users will need to perform a push or pull or update operation if they
+  require branch synchronisation to take place. (Robert Collins, #43744)
+
+* When creating a tarball or zipfile export, export unicode names as utf-8
+  paths. This may not work perfectly on all platforms, but has the best
+  chance of working in the common case. (John Arbash Meinel, #56815)
+
+* When committing, only files that exist in working tree or basis tree
+  may be specified (Aaron Bentley, #50793)
+
+Portability
+***********
+
+* Fixes to run on Python 2.5 (Brian M. Carlson, Martin Pool, Marien Zwart)
+
+Internals
+*********
+
+* TestCaseInTempDir now creates a separate directory for HOME, rather
+  than having HOME set to the same location as the working directory.
+  (John Arbash Meinel)
+
+* ``run_bzr_subprocess()`` can take an optional ``env_changes={}`` parameter,
+  which will update os.environ inside the spawned child. It also can
+  take a ``universal_newlines=True``, which helps when checking the output
+  of the command. (John Arbash Meinel)
+
+* Refactor SFTP vendors to allow easier re-use when SSH is used.
+  (Andrew Bennetts)
+
+* ``Transport.list_dir()`` and ``Transport.iter_files_recursive()`` should always
+  return urlescaped paths. This is now tested (there were bugs in a few
+  of the transports) (Andrew Bennetts, David Allouche, John Arbash Meinel)
+
+* New utility function ``symbol_versioning.deprecation_string``. Returns the
+  formatted string for a callable, deprecation format pair. (Robert Collins)
+
+* New TestCase helper applyDeprecated. This allows you to call a callable
+  which is deprecated without it spewing to the screen, just by supplying
+  the deprecation format string issued for it. (Robert Collins)
+
+* Transport.append and Transport.put have been deprecated in favor of
+  ``.append_bytes``, ``.append_file``, ``.put_bytes``, and
+  ``.put_file``. This removes the ambiguity in what type of object the
+  functions take.  ``Transport.non_atomic_put_{bytes,file}`` has also
+  been added. Which works similarly to ``Transport.append()`` except for
+  SFTP, it doesn't have a round trip when opening the file. Also, it
+  provides functionality for creating a parent directory when trying
+  to create a file, rather than raise NoSuchFile and forcing the
+  caller to repeat their request.
+  (John Arbash Meinel)
+
+* WorkingTree has a new api ``unversion`` which allow the unversioning of
+  entries by their file id. (Robert Collins)
+
+* ``WorkingTree.pending_merges`` is deprecated.  Please use the
+  ``get_parent_ids`` (introduced in 0.10) method instead. (Robert Collins)
+
+* WorkingTree has a new ``lock_tree_write`` method which locks the branch for
+  read rather than write. This is appropriate for actions which only need
+  the branch data for reference rather than mutation. A new decorator
+  ``needs_tree_write_lock`` is provided in the workingtree module. Like the
+  ``needs_read_lock`` and ``needs_write_lock`` decorators this allows static
+  declaration of the locking requirements of a function to ensure that
+  a lock is taken out for casual scripts. (Robert Collins, #54107)
+
+* All WorkingTree methods which write to the tree, but not to the branch
+  have been converted to use ``needs_tree_write_lock`` rather than
+  ``needs_write_lock``. Also converted is the revert, conflicts and tree
+  transform modules. This provides a modest performance improvement on
+  metadir style trees, due to the reduce lock-acquisition, and a more
+  significant performance improvement on lightweight checkouts from
+  remote branches, where trivial operations used to pay a significant
+  penalty. It also provides the basis for allowing readonly checkouts.
+  (Robert Collins)
+
+* Special case importing the standard library 'copy' module. This shaves
+  off 40ms of startup time, while retaining compatibility. See:
+  ``bzrlib/inspect_for_copy.py`` for more details. (John Arbash Meinel)
+
+* WorkingTree has a new parent class MutableTree which represents the
+  specialisations of Tree which are able to be altered. (Robert Collins)
+
+* New methods mkdir and ``put_file_bytes_non_atomic`` on MutableTree that
+  mutate the tree and its contents. (Robert Collins)
+
+* Transport behaviour at the root of the URL is now defined and tested.
+  (Andrew Bennetts, Robert Collins)
+
+Testing
+*******
+
+* New test helper classs MemoryTree. This is typically accessed via
+  ``self.make_branch_and_memory_tree()`` in test cases. (Robert Collins)
+
+* Add ``start_bzr_subprocess`` and ``stop_bzr_subprocess`` to allow test
+  code to continue running concurrently with a subprocess of bzr.
+  (Andrew Bennetts, Robert Collins)
+
+* Add a new method ``Transport.get_smart_client()``. This is provided to
+  allow upgrades to a richer interface than the VFS one provided by
+  Transport. (Andrew Bennetts, Martin Pool)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.12.txt'
--- doc/en/release-notes/bzr-0.12.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.12.txt	2010-10-08 08:15:14 +0000
@@ -0,0 +1,146 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.12
+########
+
+:Released:  2006-10-30
+
+Internals
+*********
+
+* Clean up ``bzr selftest --benchmark bundle`` to correct an import,
+  and remove benchmarks that take longer than 10min to run.
+  (John Arbash Meinel)
+
+bzr 0.12rc1
+###########
+
+:Released:  2006-10-23
+
+Improvements
+************
+
+* ``bzr log`` now shows dotted-decimal revision numbers for all revisions,
+  rather than just showing a decimal revision number for revisions on the
+  mainline. These revision numbers are not yet accepted as input into bzr
+  commands such as log, diff etc. (Robert Collins)
+
+* revisions can now be specified using dotted-decimal revision numbers.
+  For instance, ``bzr diff -r 1.2.1..1.2.3``. (Robert Collins)
+
+* ``bzr help commands`` output is now shorter (Aaron Bentley)
+
+* ``bzr`` now uses lazy importing to reduce the startup time. This has
+  a moderate effect on lots of actions, especially ones that have
+  little to do. For example ``bzr rocks`` time is down to 116ms from
+  283ms. (John Arbash Meinel)
+
+* New Registry class to provide name-to-object registry-like support,
+  for example for schemes where plugins can register new classes to
+  do certain tasks (e.g. log formatters). Also provides lazy registration
+  to allow modules to be loaded on request.
+  (John Arbash Meinel, Adeodato Simó)
+
+API Incompatability
+*******************
+
+* LogFormatter subclasses show now expect the 'revno' parameter to
+  show() to be a string rather than an int. (Robert Collins)
+
+Internals
+*********
+
+* ``TestCase.run_bzr``, ``run_bzr_captured``, and ``run_bzr_subprocess``
+  can take a ``working_dir='foo'`` parameter, which will change directory
+  for the command. (John Arbash Meinel)
+
+* ``bzrlib.lazy_regex.lazy_compile`` can be used to create a proxy
+  around a regex, which defers compilation until first use.
+  (John Arbash Meinel)
+
+* ``TestCase.run_bzr_subprocess`` defaults to supplying the
+  ``--no-plugins`` parameter to ensure test reproducability, and avoid
+  problems with system-wide installed plugins. (John Arbash Meinel)
+
+* Unique tree root ids are now supported. Newly created trees still
+  use the common root id for compatibility with bzr versions before 0.12.
+  (Aaron Bentley)
+
+* ``WorkingTree.set_root_id(None)`` is now deprecated. Please
+  pass in ``inventory.ROOT_ID`` if you want the default root id value.
+  (Robert Collins, John Arbash Meinel)
+
+* New method ``WorkingTree.flush()`` which will write the current memory
+  inventory out to disk. At the same time, ``read_working_inventory`` will
+  no longer trash the current tree inventory if it has been modified within
+  the current lock, and the tree will now ``flush()`` automatically on
+  ``unlock()``. ``WorkingTree.set_root_id()`` has been updated to take
+  advantage of this functionality. (Robert Collins, John Arbash Meinel)
+
+* ``bzrlib.tsort.merge_sorted`` now accepts ``generate_revnos``. This
+  parameter will cause it to add another column to its output, which
+  contains the dotted-decimal revno for each revision, as a tuple.
+  (Robert Collins)
+
+* ``LogFormatter.show_merge`` is deprecated in favour of
+  ``LogFormatter.show_merge_revno``. (Robert Collins)
+
+Bug Fixes
+*********
+
+* Avoid circular imports by creating a deprecated function for
+  ``bzrlib.tree.RevisionTree``. Callers should have been using
+  ``bzrlib.revisontree.RevisionTree`` anyway. (John Arbash Meinel,
+  #66349)
+
+* Don't use ``socket.MSG_WAITALL`` as it doesn't exist on all
+  platforms. (Martin Pool, #66356)
+
+* Don't require ``Content-Type`` in range responses. Assume they are a
+  single range if ``Content-Type`` does not exist.
+  (John Arbash Meinel, #62473)
+
+* bzr branch/pull no longer complain about progress bar cleanup when
+  interrupted during fetch.  (Aaron Bentley, #54000)
+
+* ``WorkingTree.set_parent_trees()`` uses the trees to directly write
+  the basis inventory, rather than going through the repository. This
+  allows us to have 1 inventory read, and 2 inventory writes when
+  committing a new tree. (John Arbash Meinel)
+
+* When reverting, files that are not locally modified that do not exist
+  in the target are deleted, not just unversioned (Aaron Bentley)
+
+* When trying to acquire a lock, don't fail immediately. Instead, try
+  a few times (up to 1 hour) before timing out. Also, report why the
+  lock is unavailable (John Arbash Meinel, #43521, #49556)
+
+* Leave HttpTransportBase daughter classes decides how they
+  implement cloning. (Vincent Ladeuil, #61606)
+
+* diff3 does not indicate conflicts on clean merge. (Aaron Bentley)
+
+* If a commit fails, the commit message is stored in a file at the root of
+  the tree for later commit. (Cheuksan Edward Wang, Stefan Metzmacher,
+  #32054)
+
+Testing
+*******
+
+* New test base class TestCaseWithMemoryTransport offers memory-only
+  testing facilities: its not suitable for tests that need to mutate disk
+  state, but most tests should not need that and should be converted to
+  TestCaseWithMemoryTransport. (Robert Collins)
+
+* ``TestCase.make_branch_and_memory_tree`` now takes a format
+  option to set the BzrDir, Repository and Branch formats of the
+  created objects. (Robert Collins, John Arbash Meinel)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.13.txt'
--- doc/en/release-notes/bzr-0.13.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.13.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,145 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.13
+########
+
+:Released:  2006-12-05
+
+No changes from 0.13rc
+
+
+bzr 0.13rc1
+###########
+
+:Released:  2006-11-27
+
+Improvements
+************
+
+* New command ``bzr remove-tree`` allows the removal of the working
+  tree from a branch.
+  (Daniel Silverstone)
+
+* urllib uses shared keep-alive connections, so HTTP  operations are substantially faster.
+  (Vincent Ladeuil, #53654)
+
+* ``bzr export`` allows an optional branch parameter, to export a bzr
+  tree from some other URL. For example:
+  ``bzr export bzr.tar.gz http://bazaar-vcs.org/bzr/bzr.dev``
+  (Daniel Silverstone)
+
+* Added ``bzr help topics`` to the bzr help system. This gives a
+  location for general information, outside of a specific command.
+  This includes updates for ``bzr help revisionspec`` the first topic
+  included. (Goffredo Baroncelli, John Arbash Meinel, #42714)
+
+* WSGI-compatible HTTP smart server.  See ``doc/http_smart_server.txt``.
+  (Andrew Bennetts)
+
+* Knit files will now cache full texts only when the size of the
+  deltas is as large as the size of the fulltext. (Or after 200
+  deltas, whichever comes first). This has the most benefit on large
+  files with small changes, such as the inventory for a large project.
+  (eg For a project with 2500 files, and 7500 revisions, it changes
+  the size of inventory.knit from 11MB to 5.4MB) (John Arbash Meinel)
+
+Internals
+*********
+
+* New -D option given before the command line turns on debugging output
+  for particular areas.  -Derror shows tracebacks on all errors.
+  (Martin Pool)
+
+* Clean up ``bzr selftest --benchmark bundle`` to correct an import,
+  and remove benchmarks that take longer than 10min to run.
+  (John Arbash Meinel)
+
+* Use ``time.time()`` instead of ``time.clock()`` to decide on
+  progress throttling. Because ``time.clock()`` is actually CPU time,
+  so over a high-latency connection, too many updates get throttled.
+  (John Arbash Meinel)
+
+* ``MemoryTransport.list_dir()`` would strip the first character for
+  files or directories in root directory. (John Arbash Meinel)
+
+* New method ``get_branch_reference`` on 'BzrDir' allows the detection of
+  branch references - which the smart server component needs.
+
+* New ``ChrootTransportDecorator``, accessible via the ``chroot+`` URL
+  prefix.  It disallows any access to locations above a set URL.  (Andrew
+  Bennetts)
+
+Bug Fixes
+*********
+
+* Now ``_KnitIndex`` properly decode revision ids when loading index data.
+  And optimize the knit index parsing code.
+  (Dmitry Vasiliev, John Arbash Meinel)
+
+* ``bzrlib/bzrdir.py`` was directly referencing ``bzrlib.workingtree``,
+  without importing it. This prevented ``bzr upgrade`` from working
+  unless a plugin already imported ``bzrlib.workingtree``
+  (John Arbash Meinel, #70716)
+
+* Suppress the traceback on invalid URLs (Vincent Ladeuil, #70803).
+
+* Give nicer error message when an HTTP server returns a 403
+  error code. (Vincent Ladeuil, #57644).
+
+* When a multi-range HTTP GET request fails, try a single
+  range one. If it fails too, forget about ranges. Remember that until
+  the death of the transport and propagates that to the clones.
+  (Vincent Ladeuil, #62276, #62029).
+
+* Handles user/passwords supplied in URL from command
+  line (for the urllib implementation). Don't request already
+  known passwords (Vincent Ladeuil, #42383, #44647, #48527)
+
+* ``_KnitIndex.add_versions()`` dictionary compresses revision ids as they
+  are added. This fixes bug where fetching remote revisions records
+  them as full references rather than integers.
+  (John Arbash Meinel, #64789)
+
+* ``bzr ignore`` strips trailing slashes in patterns.
+  Also ``bzr ignore`` rejects absolute paths. (Kent Gibson, #4559)
+
+* ``bzr ignore`` takes multiple arguments. (Cheuksan Edward Wang, #29488)
+
+* mv correctly handles paths that traverse symlinks.
+  (Aaron Bentley, #66964)
+
+* Give nicer looking error messages when failing to connect over SSH.
+  (John Arbash Meinel, #49172)
+
+* Pushing to a remote branch does not currently update the remote working
+  tree. After a remote push, ``bzr status`` and ``bzr diff`` on the remote
+  machine now show that the working tree is out of date.
+  (Cheuksan Edward Wang #48136)
+
+* Use patiencediff instead of difflib for determining deltas to insert
+  into knits. This avoids the O(N^3) behavior of difflib. Patience
+  diff should be O(N^2). (Cheuksan Edward Wang, #65714)
+
+* Running ``bzr log`` on nonexistent file gives an error instead of the
+  entire log history. (Cheuksan Edward Wang #50793)
+
+* ``bzr cat`` can look up contents of removed or renamed files. If the
+  pathname is ambiguous, i.e. the files in the old and new trees have
+  different id's, the default is the file in the new tree. The user can
+  use "--name-from-revision" to select the file in the old tree.
+  (Cheuksan Edward Wang, #30190)
+
+Testing
+*******
+
+* TestingHTTPRequestHandler really handles the Range header
+  (previously it was ignoring it and returning the whole file,).
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.14.txt'
--- doc/en/release-notes/bzr-0.14.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.14.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,168 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.14
+########
+
+:Released:  2007-01-23
+
+Improvements
+************
+
+* ``bzr help global-options`` describes the global options. (Aaron Bentley)
+
+Bug Fixes
+*********
+
+* Skip documentation generation tests if the tools to do so are not
+  available. Fixes running selftest for installled copies of bzr.
+  (John Arbash Meinel, #80330)
+
+* Fix the code that discovers whether bzr is being run from it's
+  working tree to handle the case when it isn't but the directory
+  it is in is below a repository. (James Westby, #77306)
+
+
+bzr 0.14rc1
+###########
+
+:Released:  2007-01-16
+
+Improvements
+************
+
+* New connection: ``bzr+http://`` which supports tunnelling the smart
+  protocol over an HTTP connection. If writing is enabled on the bzr
+  server, then you can write over the HTTP connection.
+  (Andrew Bennetts, John Arbash Meinel)
+
+* Aliases now support quotation marks, so they can contain whitespace
+  (Marius Kruger)
+
+* PyCurlTransport now use a single curl object. By specifying explicitly
+  the 'Range' header, we avoid the need to use two different curl objects
+  (and two connections to the same server). (Vincent Ladeuil)
+
+* ``bzr commit`` does not prompt for a message until it is very likely to
+  succeed.  (Aaron Bentley)
+
+* ``bzr conflicts`` now takes --text to list pathnames of text conflicts
+  (Aaron Bentley)
+
+* Fix ``iter_lines_added_or_present_in_versions`` to use a set instead
+  of a list while checking if a revision id was requested. Takes 10s
+  off of the ``fileids_affected_by_revision_ids`` time, which is 10s
+  of the ``bzr branch`` time. Also improve ``fileids_...`` time by
+  filtering lines with a regex rather than multiple ``str.find()``
+  calls. (saves another 300ms) (John Arbash Meinel)
+
+* Policy can be set for each configuration key. This allows keys to be
+  inherited properly across configuration entries. For example, this
+  should enable you to do::
+
+    [/home/user/project]
+    push_location = sftp://host/srv/project/
+    push_location:policy = appendpath
+
+  And then a branch like ``/home/user/project/mybranch`` should get an
+  automatic push location of ``sftp://host/srv/project/mybranch``.
+  (James Henstridge)
+
+* Added ``bzr status --short`` to make status report svn style flags
+  for each file.  For example::
+
+    $ bzr status --short
+    A  foo
+    A  bar
+    D  baz
+    ?  wooley
+
+* 'bzr selftest --clean-output' allows easily clean temporary tests
+  directories without running tests. (Alexander Belchenko)
+
+* ``bzr help hidden-commands`` lists all hidden commands. (Aaron Bentley)
+
+* ``bzr merge`` now has an option ``--pull`` to fall back to pull if
+  local is fully merged into remote. (Jan Hudec)
+
+* ``bzr help formats`` describes available directory formats. (Aaron Bentley)
+
+Internals
+*********
+
+* A few tweaks directly to ``fileids_affected_by_revision_ids`` to
+  help speed up processing, as well allowing to extract unannotated
+  lines. Between the two ``fileids_affected_by_revision_ids`` is
+  improved by approx 10%. (John Arbash Meinel)
+
+* Change Revision serialization to only write out millisecond
+  resolution. Rather than expecting floating point serialization to
+  preserve more resolution than we need. (Henri Weichers, Martin Pool)
+
+* Test suite ends cleanly on Windows.  (Vincent Ladeuil)
+
+* When ``encoding_type`` attribute of class Command is equal to 'exact',
+  force sys.stdout to be a binary stream on Windows, and therefore
+  keep exact line-endings (without LF -> CRLF conversion).
+  (Alexander Belchenko)
+
+* Single-letter short options are no longer globally declared.  (Martin
+  Pool)
+
+* Before using detected user/terminal encoding bzr should check
+  that Python has corresponding codec. (Alexander Belchenko)
+
+* Formats for end-user selection are provided via a FormatRegistry (Aaron Bentley)
+
+Bug Fixes
+*********
+
+* ``bzr missing --verbose`` was showing adds/removals in the wrong
+  direction. (John Arbash Meinel)
+
+* ``bzr annotate`` now defaults to showing dotted revnos for merged
+  revisions. It cuts them off at a depth of 12 characters, but you can
+  supply ``--long`` to see the full number. You can also use
+  ``--show-ids`` to display the original revision ids, rather than
+  revision numbers and committer names. (John Arbash Meinel, #75637)
+
+* bzr now supports Win32 UNC path (e.g. ``\HOST\path``.
+  (Alexander Belchenko, #57869)
+
+* Win32-specific: output of cat, bundle and diff commands don't mangle
+  line-endings (Alexander Belchenko, #55276)
+
+* Replace broken fnmatch based ignore pattern matching with custom pattern
+  matcher.
+  (Kent Gibson, Jan Hudec #57637)
+
+* pycurl and urllib can detect short reads at different places. Update
+  the test suite to test more cases. Also detect HTTP error code 416
+  which was raised for that specific bug. Also enhance the urllib
+  robustness by detecting invalid ranges (and pycurl's one by detecting
+  short reads during the initial GET). (Vincent Ladeuil, #73948)
+
+* The urllib connection sharing interacts badly with urllib2
+  proxy setting (the connections didn't go thru the proxy
+  anymore). Defining a proper ProxyHandler solves the
+  problem.  (Vincent Ladeuil, #74759)
+
+* Use urlutils to generate relative URLs, not osutils
+  (Aaron Bentley, #76229)
+
+* ``bzr status`` in a readonly directory should work without giving
+  lots of errors. (John Arbash Meinel, #76299)
+
+* Mention the revisionspec topic for the revision option help.
+  (Wouter van Heyst, #31663)
+
+* Allow plugins import from zip archives.
+  (Alexander Belchenko, #68124)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.15.txt'
--- doc/en/release-notes/bzr-0.15.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.15.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,391 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.15
+########
+
+:Released: 2007-04-01
+
+Bugfixes
+********
+
+* Handle incompatible repositories as a user issue when fetching.
+  (Aaron Bentley)
+
+* Don't give a recommendation to upgrade when branching or
+  checking out a branch that contains an old-format working tree.
+  (Martin Pool)
+
+bzr 0.15rc3
+###########
+
+:Released:  2007-03-26
+
+Changes
+*******
+
+* A warning is now displayed when opening working trees in older
+  formats, to encourage people to upgrade to WorkingTreeFormat4.
+  (Martin Pool)
+
+Improvements
+************
+
+* HTTP redirections are now taken into account when a branch (or a
+  bundle) is accessed for the first time. A message is issued at each
+  redirection to inform the user. In the past, HTTP redirections were
+  silently followed for each request which significantly degraded the
+  performances. The HTTP redirections are not followed anymore by
+  default, instead a RedirectRequested exception is raised. For bzrlib
+  users needing to follow HTTP redirections anyway,
+  ``bzrlib.transport.do_catching_redirections`` provide an easy transition
+  path.  (vila)
+
+Internals
+*********
+
+* Added ``ReadLock.temporary_write_lock()`` to allow upgrading an OS read
+  lock to an OS write lock. Linux can do this without unlocking, Win32
+  needs to unlock in between. (John Arbash Meinel)
+
+* New parameter ``recommend_upgrade`` to ``BzrDir.open_workingtree``
+  to silence (when false) warnings about opening old formats.
+  (Martin Pool)
+
+* Fix minor performance regression with bzr-0.15 on pre-dirstate
+  trees. (We were reading the working inventory too many times).
+  (John Arbash Meinel)
+
+* Remove ``Branch.get_transaction()`` in favour of a simple cache of
+  ``revision_history``.  Branch subclasses should override
+  ``_gen_revision_history`` rather than ``revision_history`` to make use of
+  this cache, and call ``_clear_revision_history_cache`` and
+  ``_cache_revision_history`` at appropriate times. (Andrew Bennetts)
+
+Bugfixes
+********
+
+* Take ``smtp_server`` from user config into account.
+  (vila, #92195)
+
+* Restore Unicode filename handling for versioned and unversioned files.
+  (John Arbash Meinel, #92608)
+
+* Don't fail during ``bzr commit`` if a file is marked removed, and
+  the containing directory is auto-removed.  (John Arbash Meinel, #93681)
+
+* ``bzr status FILENAME`` failed on Windows because of an uncommon
+  errno. (``ERROR_DIRECTORY == 267 != ENOTDIR``).
+  (Wouter van Heyst, John Arbash Meinel, #90819)
+
+* ``bzr checkout source`` should create a local branch in the same
+  format as source. (John Arbash Meinel, #93854)
+
+* ``bzr commit`` with a kind change was failing to update the
+  last-changed-revision for directories.  The
+  InventoryDirectory._unchanged only looked at the ``parent_id`` and name,
+  ignoring the fact that the kind could have changed, too.
+  (John Arbash Meinel, #90111)
+
+* ``bzr mv dir/subdir other`` was incorrectly updating files inside
+  the directory. So that there was a chance it would break commit,
+  etc. (John Arbash Meinel, #94037)
+
+* Correctly handles mutiple permanent HTTP redirections.
+  (vila, #88780)
+
+bzr 0.15rc2
+###########
+
+:Released:  2007-03-14
+
+Notes When Upgrading
+********************
+
+* Release 0.15rc2 of bzr changes the ``bzr init-repo`` command to
+  default to ``--trees`` instead of ``--no-trees``.
+  Existing shared repositories are not affected.
+
+Improvements
+************
+
+* New ``merge-directive`` command to generate machine- and human-readable
+  merge requests.  (Aaron Bentley)
+
+* New ``submit:`` revision specifier makes it easy to diff against the
+  common ancestor with the submit location (Aaron Bentley)
+
+* Added support for Putty's SSH implementation. (Dmitry Vasiliev)
+
+* Added ``bzr status --versioned`` to report only versioned files,
+  not unknowns. (Kent Gibson)
+
+* Merge now autodetects the correct line-ending style for its conflict
+  markers.  (Aaron Bentley)
+
+Internals
+*********
+
+* Refactored SSH vendor registration into SSHVendorManager class.
+  (Dmitry Vasiliev)
+
+Bugfixes
+********
+
+* New ``--numbered-dirs`` option to ``bzr selftest`` to use
+  numbered dirs for TestCaseInTempDir. This is default behavior
+  on Windows. Anyone can force named dirs on Windows
+  with ``--no-numbered-dirs``. (Alexander Belchenko)
+
+* Fix ``RevisionSpec_revid`` to handle the Unicode strings passed in
+  from the command line. (Marien Zwart, #90501)
+
+* Fix ``TreeTransform._iter_changes`` when both the source and
+  destination are missing. (Aaron Bentley, #88842)
+
+* Fix commit of merges with symlinks in dirstate trees.
+  (Marien Zwart)
+
+* Switch the ``bzr init-repo`` default from --no-trees to --trees.
+  (Wouter van Heyst, #53483)
+
+
+bzr 0.15rc1
+###########
+
+:Released:  2007-03-07
+
+Surprises
+*********
+
+* The default disk format has changed. Please run 'bzr upgrade' in your
+  working trees to upgrade. This new default is compatible for network
+  operations, but not for local operations. That is, if you have two
+  versions of bzr installed locally, after upgrading you can only use the
+  bzr 0.15 version. This new default does not enable tags or nested-trees
+  as they are incompatible with bzr versions before 0.15 over the network.
+
+* For users of bzrlib: Two major changes have been made to the working tree
+  api in bzrlib. The first is that many methods and attributes, including
+  the inventory attribute, are no longer valid for use until one of
+  ``lock_read``/``lock_write``/``lock_tree_write`` has been called,
+  and become invalid again after unlock is called. This has been done
+  to improve performance and correctness as part of the dirstate
+  development.
+  (Robert Collins, John A Meinel, Martin Pool, and others).
+
+* For users of bzrlib: The attribute 'tree.inventory' should be considered
+  readonly. Previously it was possible to directly alter this attribute, or
+  its contents, and have the tree notice this. This has been made
+  unsupported - it may work in some tree formats, but in the newer dirstate
+  format such actions will have no effect and will be ignored, or even
+  cause assertions. All operations possible can still be carried out by a
+  combination of the tree API, and the bzrlib.transform API. (Robert
+  Collins, John A Meinel, Martin Pool, and others).
+
+Improvements
+************
+
+* Support for OS Windows 98. Also .bzr.log on any windows system
+  saved in My Documents folder. (Alexander Belchenko)
+
+* ``bzr mv`` enhanced to support already moved files.
+  In the past the mv command would have failed if the source file doesn't
+  exist. In this situation ``bzr mv`` would now detect that the file has
+  already moved and update the repository accordingly, if the target file
+  does exist.
+  A new option ``--after`` has been added so that if two files already
+  exist, you could notify Bazaar that you have moved a (versioned) file
+  and replaced it with another. Thus in this case ``bzr move --after``
+  will only update the Bazaar identifier.
+  (Steffen Eichenberg, Marius Kruger)
+
+* ``ls`` now works on treeless branches and remote branches.
+  (Aaron Bentley)
+
+* ``bzr help global-options`` describes the global options.
+  (Aaron Bentley)
+
+* ``bzr pull --overwrite`` will now correctly overwrite checkouts.
+  (Robert Collins)
+
+* Files are now allowed to change kind (e.g. from file to symlink).
+  Supported by ``commit``, ``revert`` and ``status``
+  (Aaron Bentley)
+
+* ``inventory`` and ``unknowns`` hidden in favour of ``ls``
+  (Aaron Bentley)
+
+* ``bzr help checkouts`` descibes what checkouts are and some possible
+  uses of them. (James Westby, Aaron Bentley)
+
+* A new ``-d`` option to push, pull and merge overrides the default
+  directory.  (Martin Pool)
+
+* Branch format 6: smaller, and potentially faster than format 5.  Supports
+  ``append_history_only`` mode, where the log view and revnos do not change,
+  except by being added to.  Stores policy settings in
+  ".bzr/branch/branch.conf".
+
+* ``append_only`` branches:  Format 6 branches may be configured so that log
+  view and revnos are always consistent.  Either create the branch using
+  "bzr init --append-revisions-only" or edit the config file as descriped
+  in docs/configuration.txt.
+
+* rebind: Format 6 branches retain the last-used bind location, so if you
+  "bzr unbind", you can "bzr bind" to bind to the previously-selected
+  bind location.
+
+* Builtin tags support, created and deleted by the ``tag`` command and
+  stored in the branch.  Tags can be accessed with the revisionspec
+  ``-rtag:``, and listed with ``bzr tags``.  Tags are not versioned
+  at present. Tags require a network incompatible upgrade. To perform this
+  upgrade, run ``bzr upgrade --dirstate-tags`` in your branch and
+  repositories. (Martin Pool)
+
+* The ``bzr://`` transport now has a well-known port number, 4155,
+  which it will use by default.  (Andrew Bennetts, Martin Pool)
+
+* Bazaar now looks for user-installed plugins before looking for site-wide
+  plugins. (Jonathan Lange)
+
+* ``bzr resolve`` now detects and marks resolved text conflicts.
+  (Aaron Bentley)
+
+Internals
+*********
+
+* Internally revision ids and file ids are now passed around as utf-8
+  bytestrings, rather than treating them as Unicode strings. This has
+  performance benefits for Knits, since we no longer need to decode the
+  revision id for each line of content, nor for each entry in the index.
+  This will also help with the future dirstate format.
+  (John Arbash Meinel)
+
+* Reserved ids (any revision-id ending in a colon) are rejected by
+  versionedfiles, repositories, branches, and working trees
+  (Aaron Bentley)
+
+* Minor performance improvement by not creating a ProgressBar for
+  every KnitIndex we create. (about 90ms for a bzr.dev tree)
+  (John Arbash Meinel)
+
+* New easier to use Branch hooks facility. There are five initial hooks,
+  all documented in bzrlib.branch.BranchHooks.__init__ - ``'set_rh'``,
+  ``'post_push'``, ``'post_pull'``, ``'post_commit'``,
+  ``'post_uncommit'``. These hooks fire after the matching operation
+  on a branch has taken place, and were originally added for the
+  branchrss plugin. (Robert Collins)
+
+* New method ``Branch.push()`` which should be used when pushing from a
+  branch as it makes performance and policy decisions to match the UI
+  level command ``push``. (Robert Collins).
+
+* Add a new method ``Tree.revision_tree`` which allows access to cached
+  trees for arbitrary revisions. This allows the in development dirstate
+  tree format to provide access to the callers to cached copies of
+  inventory data which are cheaper to access than inventories from the
+  repository.
+  (Robert Collins, Martin Pool)
+
+* New ``Branch.last_revision_info`` method, this is being done to allow
+  optimization of requests for both the number of revisions and the last
+  revision of a branch with smartservers and potentially future branch
+  formats. (Wouter van Heyst, Robert Collins)
+
+* Allow ``'import bzrlib.plugins.NAME'`` to work when the plugin NAME has not
+  yet been loaded by ``load_plugins()``. This allows plugins to depend on each
+  other for code reuse without requiring users to perform file-renaming
+  gymnastics. (Robert Collins)
+
+* New Repository method ``'gather_stats'`` for statistic data collection.
+  This is expected to grow to cover a number of related uses mainly
+  related to bzr info. (Robert Collins)
+
+* Log formatters are now managed with a registry.
+  ``log.register_formatter`` continues to work, but callers accessing
+  the FORMATTERS dictionary directly will not.
+
+* Allow a start message to be passed to the ``edit_commit_message``
+  function.  This will be placed in the message offered to the user
+  for editing above the separator. It allows a template commit message
+  to be used more easily. (James Westby)
+
+* ``GPGStrategy.sign()`` will now raise ``BzrBadParameterUnicode`` if
+  you pass a Unicode string rather than an 8-bit string. Callers need
+  to be updated to encode first. (John Arbash Meinel)
+
+* Branch.push, pull, merge now return Result objects with information
+  about what happened, rather than a scattering of various methods.  These
+  are also passed to the post hooks.  (Martin Pool)
+
+* File formats and architecture is in place for managing a forest of trees
+  in bzr, and splitting up existing trees into smaller subtrees, and
+  finally joining trees to make a larger tree. This is the first iteration
+  of this support, and the user-facing aspects still require substantial
+  work.  If you wish to experiment with it, use ``bzr upgrade
+  --dirstate-with-subtree`` in your working trees and repositories.
+  You can use the hidden commands ``split`` and ``join`` and to create
+  and manipulate nested trees, but please consider using the nested-trees
+  branch, which contains substantial UI improvements, instead.
+  http://code.aaronbentley.com/bzr/bzrrepo/nested-trees/
+  (Aaron Bentley, Martin Pool, Robert Collins).
+
+Bugfixes
+********
+
+* ``bzr annotate`` now uses dotted revnos from the viewpoint of the
+  branch, rather than the last changed revision of the file.
+  (John Arbash Meinel, #82158)
+
+* Lock operations no longer hang if they encounter a permission problem.
+  (Aaron Bentley)
+
+* ``bzr push`` can resume a push that was canceled before it finished.
+  Also, it can push even if the target directory exists if you supply
+  the ``--use-existing-dir`` flag.
+  (John Arbash Meinel, #30576, #45504)
+
+* Fix HTTP proxy authentication when user and an optional
+  password appears in the ``*_proxy`` vars. (Vincent Ladeuil,
+  #83954).
+
+* ``bzr log branch/file`` works for local treeless branches
+  (Aaron Bentley, #84247)
+
+* Fix problem with UNC paths on Windows 98. (Alexander Belchenko, #84728)
+
+* Searching location of CA bundle for PyCurl in env variable
+  (``CURL_CA_BUNDLE``), and on win32 along the PATH.
+  (Alexander Belchenko, #82086)
+
+* ``bzr init`` works with unicode argument LOCATION.
+  (Alexander Belchenko, #85599)
+
+* Raise ``DependencyNotPresent`` if pycurl do not support https.
+  (Vincent Ladeuil, #85305)
+
+* Invalid proxy env variables should not cause a traceback.
+  (Vincent Ladeuil, #87765)
+
+* Ignore patterns normalised to use '/' path separator.
+  (Kent Gibson, #86451)
+
+* bzr rocks. It sure does! Fix case. (Vincent Ladeuil, #78026)
+
+* Fix bzrtools shelve command for removed lines beginning with "--"
+  (Johan Dahlberg, #75577)
+
+Testing
+*******
+
+* New ``--first`` option to ``bzr selftest`` to run specified tests
+  before the rest of the suite.  (Martin Pool)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.16.txt'
--- doc/en/release-notes/bzr-0.16.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.16.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,379 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.16
+########
+
+:Released:  2007-05-07
+
+Bugfixes
+********
+
+* Handle when you have 2 directories with similar names, but one has a
+  hyphen. (``'abc'`` versus ``'abc-2'``). The WT4._iter_changes
+  iterator was using direct comparison and ``'abc/a'`` sorts after
+  ``'abc-2'``, but ``('abc', 'a')`` sorts before ``('abc-2',)``.
+  (John Arbash Meinel, #111127)
+
+* Handle when someone renames a file on disk without telling bzr.
+  Previously we would report the first file as missing, but not show
+  the new unknown file. (John Arbash Meinel, #111288)
+
+* Avoid error when running hooks after pulling into or pushing from
+  a branch bound to a smartserver branch.  (Martin Pool, #111968)
+
+Improvements
+************
+
+* Move developer documentation to doc/developers/. This reduces clutter in
+  the root of the source tree and allows HACKING to be split into multiple
+  files. (Robert Collins, Alexander Belchenko)
+
+* Clean up the ``WorkingTree4._iter_changes()`` internal loops as well as
+  ``DirState.update_entry()``. This optimizes the core logic for ``bzr
+  diff`` and ``bzr status`` significantly improving the speed of
+  both. (John Arbash Meinel)
+
+bzr 0.16rc2
+###########
+
+:Released:  2007-04-30
+
+Bugfixes
+********
+
+* Handle the case when you delete a file, and then rename another file
+  on top of it. Also handle the case of ``bzr rm --keep foo``. ``bzr
+  status`` should show the removed file and an unknown file in its
+  place. (John Arbash Meinel, #109993)
+
+* Bundles properly read and write revision properties that have an
+  empty value. And when the value is not ASCII.
+  (John Arbash Meinel, #109613)
+
+* Fix the bzr commit message to be in text mode.
+  (Alexander Belchenko, #110901)
+
+* Also handle when you rename a file and create a file where it used
+  to be. (John Arbash Meinel, #110256)
+
+* ``WorkingTree4._iter_changes`` should not descend into unversioned
+  directories. (John Arbash Meinel, #110399)
+
+bzr 0.16rc1
+###########
+
+:Released:  2007-04-26
+
+Notes When Upgrading
+********************
+
+* ``bzr remove`` and ``bzr rm`` will now remove the working file, if
+  it could be recovered again.
+  This has been done for consistency with svn and the unix rm command.
+  The old ``remove`` behaviour has been retained in the new option
+  ``bzr remove --keep``, which will just stop versioning the file,
+  but not delete it.
+  ``bzr remove --force`` have been added which will always delete the
+  files.
+  ``bzr remove`` is also more verbose.
+  (Marius Kruger, #82602)
+
+Improvements
+************
+
+* Merge directives can now be supplied as input to `merge` and `pull`,
+  like bundles can.  (Aaron Bentley)
+
+* Sending the SIGQUIT signal to bzr, which can be done on Unix by
+  pressing Control-Backslash, drops bzr into a debugger.  Type ``'c'``
+  to continue.  This can be disabled by setting the environment variable
+  ``BZR_SIGQUIT_PDB=0``.  (Martin Pool)
+
+* selftest now supports --list-only to list tests instead of running
+  them. (Ian Clatworthy)
+
+* selftest now supports --exclude PATTERN (or -x PATTERN) to exclude
+  tests with names that match that regular expression.
+  (Ian Clatworthy, #102679)
+
+* selftest now supports --randomize SEED to run tests in a random order.
+  SEED is typically the value 'now' meaning 'use the current time'.
+  (Ian Clatworthy, #102686)
+
+* New option ``--fixes`` to commit, which stores bug fixing annotations as
+  revision properties. Built-in support for Launchpad, Debian, Trac and
+  Bugzilla bug trackers. (Jonathan Lange, James Henstridge, Robert Collins)
+
+* New API, ``bzrlib.bugtracker.tracker_registry``, for adding support for
+  other bug trackers to ``fixes``. (Jonathan Lange, James Henstridge,
+  Robert Collins)
+
+* ``selftest`` has new short options ``-f`` and ``-1``.  (Martin
+  Pool)
+
+* ``bzrlib.tsort.MergeSorter`` optimizations. Change the inner loop
+  into using local variables instead of going through ``self._var``.
+  Improves the time to ``merge_sort`` a 10k revision graph by
+  approximately 40% (~700->400ms).  (John Arbash Meinel)
+
+* ``make docs`` now creates a man page at ``man1/bzr.1`` fixing bug 107388.
+  (Robert Collins)
+
+* ``bzr help`` now provides cross references to other help topics using
+  the _see_also facility on command classes. Likewise the bzr_man
+  documentation, and the bzr.1 man page also include this information.
+  (Robert Collins)
+
+* Tags are now included in logs, that use the long log formatter.
+  (Erik Bågfors, Alexander Belchenko)
+
+* ``bzr help`` provides a clearer message when a help topic cannot be
+  found. (Robert Collins, #107656)
+
+* ``bzr help`` now accepts optional prefixes for command help. The help
+  for all commands can now be found at ``bzr help commands/COMMANDNAME``
+  as well as ``bzr help COMMANDNAME`` (which only works for commands
+  where the name is not the same as a more general help topic).
+  (Robert Collins)
+
+* ``bzr help PLUGINNAME`` will now return the module docstring from the
+  plugin PLUGINNAME. (Robert Collins, #50408)
+
+* New help topic ``urlspec`` which lists the availables transports.
+  (Goffredo Baroncelli)
+
+* doc/server.txt updated to document the default bzr:// port
+  and also update the blurb about the hpss' current status.
+  (Robert Collins, #107125).
+
+* ``bzr serve`` now listens on interface 0.0.0.0 by default, making it
+  serve out to the local LAN (and anyone in the world that can reach the
+  machine running ``bzr serve``. (Robert Collins, #98918)
+
+* A new smart server protocol version has been added.  It prefixes requests
+  and responses with an explicit version identifier so that future protocol
+  revisions can be dealt with gracefully.  (Andrew Bennetts, Robert Collins)
+
+* The bzr protocol version 2 indicates success or failure in every response
+  without depending on particular commands encoding that consistently,
+  allowing future client refactorings to be much more robust about error
+  handling. (Robert Collins, Martin Pool, Andrew Bennetts)
+
+* The smart protocol over HTTP client has been changed to always post to the
+  same ``.bzr/smart`` URL under the original location when it can.  This allows
+  HTTP servers to only have to pass URLs ending in .bzr/smart to the smart
+  server handler, and not arbitrary ``.bzr/*/smart`` URLs.  (Andrew Bennetts)
+
+* digest authentication is now supported for proxies and HTTP by the urllib
+  based HTTP implementation. Tested against Apache 2.0.55 and Squid
+  2.6.5. Basic and digest authentication are handled coherently for HTTP
+  and proxy: if the user is provided in the URL (bzr command line for HTTP,
+  proxy environment variables for proxies), the password is prompted for
+  (only once). If the password is provided, it is taken into account. Once
+  the first authentication is successful, all further authentication
+  roundtrips are avoided by preventively setting the right authentication
+  header(s).
+  (Vincent Ladeuil).
+
+Internals
+*********
+
+* bzrlib API compatability with 0.8 has been dropped, cleaning up some
+  code paths. (Robert Collins)
+
+* Change the format of chroot URLS so that they can be safely manipulated
+  by generic URL utilities without causing the resulting URLs to have
+  escaped the chroot. A side effect of this is that creating a chroot
+  requires an explicit action using a ChrootServer.
+  (Robert Collins, Andrew Bennetts)
+
+* Deprecate ``Branch.get_root_id()`` because branches don't have root ids,
+  rather than fixing bug #96847.  (Aaron Bentley)
+
+* ``WorkingTree.apply_inventory_delta`` provides a better alternative to
+  ``WorkingTree._write_inventory``.  (Aaron Bentley)
+
+* Convenience method ``TestCase.expectFailure`` ensures that known failures
+  do not silently pass.  (Aaron Bentley)
+
+* ``Transport.local_abspath`` now raises ``NotLocalUrl`` rather than
+  ``TransportNotPossible``. (Martin Pool, Ian Clatworthy)
+
+* New SmartServer hooks facility. There are two initial hooks documented
+  in ``bzrlib.transport.smart.SmartServerHooks``. The two initial hooks allow
+  plugins to execute code upon server startup and shutdown.
+  (Robert Collins).
+
+* SmartServer in standalone mode will now close its listening socket
+  when it stops, rather than waiting for garbage collection. This primarily
+  fixes test suite hangs when a test tries to connect to a shutdown server.
+  It may also help improve behaviour when dealing with a server running
+  on a specific port (rather than dynamically assigned ports).
+  (Robert Collins)
+
+* Move most SmartServer code into a new package, bzrlib/smart.
+  bzrlib/transport/remote.py contains just the Transport classes that used
+  to be in bzrlib/transport/smart.py.  (Andrew Bennetts)
+
+* urllib HTTP implementation avoid roundtrips associated with
+  401 (and 407) errors once the authentication succeeds.
+  (Vincent Ladeuil).
+
+* urlib HTTP now supports querying the user for a proxy password if
+  needed. Realm is shown in the prompt for both HTTP and proxy
+  authentication when the user is required to type a password.
+  (Vincent Ladeuil).
+
+* Renamed SmartTransport (and subclasses like SmartTCPTransport) to
+  RemoteTransport (and subclasses to RemoteTCPTransport, etc).  This is more
+  consistent with its new home in ``bzrlib/transport/remote.py``, and because
+  it's not really a "smart" transport, just one that does file operations
+  via remote procedure calls.  (Andrew Bennetts)
+
+* The ``lock_write`` method of ``LockableFiles``, ``Repository`` and
+  ``Branch`` now accept a ``token`` keyword argument, so that separate
+  instances of those objects can share a lock if it has the right token.
+  (Andrew Bennetts, Robert Collins)
+
+* New method ``get_branch_reference`` on ``BzrDir`` allows the detection of
+  branch references - which the smart server component needs.
+
+* The Repository API ``make_working_trees`` is now permitted to return
+  False when ``set_make_working_trees`` is not implemented - previously
+  an unimplemented ``set_make_working_trees`` implied the result True
+  from ``make_working_trees``. This has been changed to accomodate the
+  smart server, where it does not make sense (at this point) to ever
+  make working trees by default. (Robert Collins)
+
+* Command objects can now declare related help topics by having _see_also
+  set to a list of related topic. (Robert Collins)
+
+* ``bzrlib.help`` now delegates to the Command class for Command specific
+  help. (Robert Collins)
+
+* New class ``TransportListRegistry``, derived from the Registry class, which
+  simplifies tracking the available Transports. (Goffredo Baroncelli)
+
+* New function ``Branch.get_revision_id_to_revno_map`` which will
+  return a dictionary mapping revision ids to dotted revnos. Since
+  dotted revnos are defined in the context of the branch tip, it makes
+  sense to generate them from a ``Branch`` object.
+  (John Arbash Meinel)
+
+* Fix the 'Unprintable error' message display to use the repr of the
+  exception that prevented printing the error because the str value
+  for it is often not useful in debugging (e.g. KeyError('foo') has a
+  str() of 'foo' but a repr of 'KeyError('foo')' which is much more
+  useful. (Robert Collins)
+
+* ``urlutils.normalize_url`` now unescapes unreserved characters, such as "~".
+  (Andrew Bennetts)
+
+Bugfixes
+********
+
+* Don't fail bundle selftest if email has 'two' embedded.
+  (Ian Clatworthy, #98510)
+
+* Remove ``--verbose`` from ``bzr bundle``. It didn't work anyway.
+  (Robert Widhopf-Fenk, #98591)
+
+* Remove ``--basis`` from the checkout/branch commands - it didn't work
+  properly and is no longer beneficial.
+  (Robert Collins, #53675, #43486)
+
+* Don't produce encoding error when adding duplicate files.
+  (Aaron Bentley)
+
+* Fix ``bzr log <file>`` so it only logs the revisions that changed
+  the file, and does it faster.
+  (Kent Gibson, John Arbash Meinel, #51980, #69477)
+
+* Fix ``InterDirstateTre._iter_changes`` to handle when we come across
+  an empty versioned directory, which now has files in it.
+  (John Arbash Meinel, #104257)
+
+* Teach ``common_ancestor`` to shortcut when the tip of one branch is
+  inside the ancestry of the other. Saves a lot of graph processing
+  (with an ancestry of 16k revisions, ``bzr merge ../already-merged``
+  changes from 2m10s to 13s).  (John Arbash Meinel, #103757)
+
+* Fix ``show_diff_trees`` to handle the case when a file is modified,
+  and the containing directory is renamed. (The file path is different
+  in this versus base, but it isn't marked as a rename).
+  (John Arbash Meinel, #103870)
+
+* FTP now works even when the FTP server does not support atomic rename.
+  (Aaron Bentley, #89436)
+
+* Correct handling in bundles and merge directives of timezones with
+  that are not an integer number of hours offset from UTC.  Always
+  represent the epoch time in UTC to avoid problems with formatting
+  earlier times on win32.  (Martin Pool, Alexander Belchenko, John
+  Arbash Meinel)
+
+* Typo in the help for ``register-branch`` fixed. (Robert Collins, #96770)
+
+* "dirstate" and "dirstate-tags" formats now produce branches compatible
+  with old versions of bzr. (Aaron Bentley, #107168))
+
+* Handle moving a directory when children have been added, removed,
+  and renamed. (John Arbash Meinel, #105479)
+
+* Don't preventively use basic authentication for proxy before receiving a
+  407 error. Otherwise people willing to use other authentication schemes
+  may expose their password in the clear (or nearly). This add one
+  roundtrip in case basic authentication should be used, but plug the
+  security hole.
+  (Vincent Ladeuil)
+
+* Handle HTTP and proxy digest authentication.
+  (Vincent Ladeuil, #94034).
+
+Testing
+*******
+
+* Added ``bzrlib.strace.strace`` which will strace a single callable and
+  return a StraceResult object which contains just the syscalls involved
+  in running it. (Robert Collins)
+
+* New test method ``reduceLockdirTimeout`` to drop the default (ui-centric)
+  default time down to one suitable for tests. (Andrew Bennetts)
+
+* Add new ``vfs_transport_factory`` attribute on tests which provides the
+  common vfs backing for both the readonly and readwrite transports.
+  This allows the RemoteObject tests to back onto local disk or memory,
+  and use the existing ``transport_server`` attribute all tests know about
+  to be the smart server transport. This in turn allows tests to
+  differentiate between 'transport to access the branch', and
+  'transport which is a VFS' - which matters in Remote* tests.
+  (Robert Collins, Andrew Bennetts)
+
+* The ``make_branch_and_tree`` method for tests will now create a
+  lightweight checkout for the tree if the ``vfs_transport_factory`` is not
+  a LocalURLServer. (Robert Collins, Andrew Bennetts)
+
+* Branch implementation tests have been audited to ensure that all URLs
+  passed to Branch APIs use proper URLs, except when local-disk paths
+  are intended. This is so that tests correctly access the test transport
+  which is often not equivalent to local disk in Remote* tests. As part
+  of this many tests were adjusted to remove dependencies on local disk
+  access.
+  (Robert Collins, Andrew Bennetts)
+
+* Mark bzrlib.tests and bzrlib.tests.TestUtil as providing assertFOO helper
+  functions by adding a ``__unittest`` global attribute. (Robert Collins,
+  Andrew Bennetts, Martin Pool, Jonathan Lange)
+
+* Refactored proxy and authentication handling to simplify the
+  implementation of new auth schemes for both HTTP and proxy.
+  (Vincent Ladeuil)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.17.txt'
--- doc/en/release-notes/bzr-0.17.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.17.txt	2010-10-08 08:15:14 +0000
@@ -0,0 +1,129 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.17
+########
+
+:Released:  2007-06-18
+
+Bugfixes
+********
+
+* Fix crash of commit due to wrong lookup of filesystem encoding.
+  (Colin Watson, #120647)
+
+* Revert logging just to stderr in commit as broke unicode filenames.
+  (Aaron Bentley, Ian Clatworthy, #120930)
+
+
+bzr 0.17rc1
+###########
+
+:Released:  2007-06-12
+
+Notes When Upgrading
+********************
+
+* The kind() and is_executable() APIs on the WorkingTree interface no
+  longer implicitly (read) locks and unlocks the tree. This *might*
+  impact some plug-ins and tools using this part of the API. If you find
+  an issue that may be caused by this change, please let us know,
+  particularly the plug-in/tool maintainer. If encountered, the API
+  fix is to surround kind() and is_executable() calls with lock_read()
+  and unlock() like so::
+
+    work_tree.lock_read()
+    try:
+        kind = work_tree.kind(...)
+    finally:
+        work_tree.unlock()
+
+Internals
+*********
+* Rework of LogFormatter API to provide beginning/end of log hooks and to
+  encapsulate the details of the revision to be logged in a LogRevision
+  object.
+  In long log formats, merge revision ids are only shown when --show-ids
+  is specified, and are labelled "revision-id:", as per mainline
+  revisions, instead of "merged:". (Kent Gibson)
+
+* New ``BranchBuilder`` API which allows the construction of particular
+  histories quickly. Useful for testing and potentially other applications
+  too. (Robert Collins)
+
+Improvements
+************
+
+* There are two new help topics, working-trees and repositories that
+  attempt to explain these concepts. (James Westby, John Arbash Meinel,
+  Aaron Bentley)
+
+* Added ``bzr log --limit`` to report a limited number of revisions.
+  (Kent Gibson, #3659)
+
+* Revert does not try to preserve file contents that were originally
+  produced by reverting to a historical revision.  (Aaron Bentley)
+
+* ``bzr log --short`` now includes ``[merge]`` for revisions which
+  have more than one parent. This is a small improvement to help
+  understanding what changes have occurred
+  (John Arbash Meinel, #83887)
+
+* TreeTransform avoids many renames when contructing large trees,
+  improving speed.  3.25x speedups have been observed for construction of
+  kernel-sized-trees, and checkouts are 1.28x faster.  (Aaron Bentley)
+
+* Commit on large trees is now faster. In my environment, a commit of
+  a small change to the Mozilla tree (55k files) has dropped from
+  66 seconds to 32 seconds. For a small tree of 600 files, commit of a
+  small change is 33% faster. (Ian Clatworthy)
+
+* New --create-prefix option to bzr init, like for push.  (Daniel Watkins,
+  #56322)
+
+Bugfixes
+********
+
+* ``bzr push`` should only connect to the remote location one time.
+  We have been connecting 3 times because we forget to pass around
+  the Transport object. This adds ``BzrDir.clone_on_transport()``, so
+  that we can pass in the Transport that we already have.
+  (John Arbash Meinel, #75721)
+
+* ``DirState.set_state_from_inventory()`` needs to properly order
+  based on split paths, not just string paths.
+  (John Arbash Meinel, #115947)
+
+* Let TestUIFactoy encode the password prompt with its own stdout.
+  (Vincent Ladeuil, #110204)
+
+* pycurl should take use the range header that takes the range hint
+  into account.
+  (Vincent Ladeuil, #112719)
+
+* WorkingTree4.get_file_sha1 no longer raises an exception when invoked
+  on a missing file.  (Aaron Bentley, #118186)
+
+* WorkingTree.remove works correctly with tree references, and when pwd is
+  not the tree root. (Aaron Bentley)
+
+* Merge no longer fails when a file is renamed in one tree and deleted
+  in the other. (Aaron Bentley, #110279)
+
+* ``revision-info`` now accepts dotted revnos, doesn't require a tree,
+  and defaults to the last revision (Matthew Fuller, #90048)
+
+* Tests no longer fail when BZR_REMOTE_PATH is set in the environment.
+  (Daniel Watkins, #111958)
+
+* ``bzr branch -r revid:foo`` can be used to branch any revision in
+  your repository. (Previously Branch6 only supported revisions in your
+  mainline). (John Arbash Meinel, #115343)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.18.txt'
--- doc/en/release-notes/bzr-0.18.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.18.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,274 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.18
+########
+
+:Released:  2007-07-17
+
+Bugfixes
+********
+
+* Fix 'bzr add' crash under Win32 (Kuno Meyer)
+
+
+bzr 0.18rc1
+###########
+
+:Released:  2007-07-10
+
+Bugfixes
+********
+
+* Do not suppress pipe errors, etc. in non-display commands
+  (Alexander Belchenko, #87178)
+
+* Display a useful error message when the user requests to annotate
+  a file that is not present in the specified revision.
+  (James Westby, #122656)
+
+* Commands that use status flags now have a reference to 'help
+  status-flags'.  (Daniel Watkins, #113436)
+
+* Work around python-2.4.1 inhability to correctly parse the
+  authentication header.
+  (Vincent Ladeuil, #121889)
+
+* Use exact encoding for merge directives. (Adeodato Simó, #120591)
+
+* Fix tempfile permissions error in smart server tar bundling under
+  Windows. (Martin _, #119330)
+
+* Fix detection of directory entries in the inventory. (James Westby)
+
+* Fix handling of HTTP code 400: Bad Request When issuing too many ranges.
+  (Vincent Ladeuil, #115209)
+
+* Issue a CONNECT request when connecting to an https server
+  via a proxy to enable SSL tunneling.
+  (Vincent Ladeuil, #120678)
+
+* Fix ``bzr log -r`` to support selecting merge revisions, both
+  individually and as part of revision ranges.
+  (Kent Gibson, #4663)
+
+* Don't leave cruft behind when failing to acquire a lockdir.
+  (Martin Pool, #109169)
+
+* Don't use the '-f' strace option during tests.
+  (Vincent Ladeuil, #102019).
+
+* Warn when setting ``push_location`` to a value that will be masked by
+  locations.conf.  (Aaron Bentley, #122286)
+
+* Fix commit ordering in corner case (Aaron Bentley, #94975)
+
+*  Make annotate behave in a non-ASCII world (Adeodato Simó).
+
+Improvements
+************
+
+* The --lsprof-file option now dumps a text rendering of the profiling
+  information if the filename ends in ".txt". It will also convert the
+  profiling information to a format suitable for KCacheGrind if the
+  output filename ends in ".callgrind". Fixes to the lsprofcalltree
+  conversion process by Jean Paul Calderone and Itamar were also merged.
+  See http://ddaa.net/blog/python/lsprof-calltree. (Ian Clatworthy)
+
+* ``info`` now defaults to non-verbose mode, displaying only paths and
+  abbreviated format info.  ``info -v`` displays all the information
+  formerly displayed by ``info``.  (Aaron Bentley, Adeodato Simó)
+
+* ``bzr missing`` now has better option names ``--this`` and ``--other``.
+  (Elliot Murphy)
+
+* The internal ``weave-list`` command has become ``versionedfile-list``,
+  and now lists knits as well as weaves.  (Aaron Bentley)
+
+* Automatic merge base selection uses a faster algorithm that chooses
+  better bases in criss-cross merge situations (Aaron Bentley)
+
+* Progress reporting in ``commit`` has been improved. The various logical
+  stages are now reported on as follows, namely:
+
+  * Collecting changes [Entry x/y] - Stage n/m
+  * Saving data locally - Stage n/m
+  * Uploading data to master branch - Stage n/m
+  * Updating the working tree - Stage n/m
+  * Running post commit hooks - Stage n/m
+
+  If there is no master branch, the 3rd stage is omitted and the total
+  number of stages is adjusted accordingly.
+
+  Each hook that is run after commit is listed with a name (as hooks
+  can be slow it is useful feedback).
+  (Ian Clatworthy, Robert Collins)
+
+* Various operations that are now faster due to avoiding unnecessary
+  topological sorts. (Aaron Bentley)
+
+* Make merge directives robust against broken bundles. (Aaron Bentley)
+
+* The lsprof filename note is emitted via trace.note(), not standard
+  output.  (Aaron Bentley)
+
+* ``bzrlib`` now exports explicit API compatibility information to assist
+  library users and plugins. See the ``bzrlib.api`` module for details.
+  (Robert Collins)
+
+* Remove unnecessary lock probes when acquiring a lockdir.
+  (Martin Pool)
+
+* ``bzr --version`` now shows the location of the bzr log file, which
+  is especially useful on Windows.  (Martin Pool)
+
+* -D now supports hooks to get debug tracing of hooks (though its currently
+  minimal in nature). (Robert Collins)
+
+* Long log format reports deltas on merge revisions.
+  (John Arbash Meinel, Kent Gibson)
+
+* Make initial push over FTP more resilient. (John Arbash Meinel)
+
+* Print a summary of changes for update just like pull does.
+  (Daniel Watkins, #113990)
+
+* Add a -Dhpss option to trace smart protocol requests and responses.
+  (Andrew Bennetts)
+
+Library API Breaks
+******************
+
+* Testing cleanups -
+  ``bzrlib.repository.RepositoryTestProviderAdapter`` has been moved
+  to ``bzrlib.tests.repository_implementations``;
+  ``bzrlib.repository.InterRepositoryTestProviderAdapter`` has been moved
+  to ``bzrlib.tests.interrepository_implementations``;
+  ``bzrlib.transport.TransportTestProviderAdapter`` has moved to
+  ``bzrlib.tests.test_transport_implementations``.
+  ``bzrlib.branch.BranchTestProviderAdapter`` has moved to
+  ``bzrlib.tests.branch_implementations``.
+  ``bzrlib.bzrdir.BzrDirTestProviderAdapter`` has moved to
+  ``bzrlib.tests.bzrdir_implementations``.
+  ``bzrlib.versionedfile.InterVersionedFileTestProviderAdapter`` has moved
+  to ``bzrlib.tests.interversionedfile_implementations``.
+  ``bzrlib.store.revision.RevisionStoreTestProviderAdapter`` has moved to
+  ``bzrlib.tests.revisionstore_implementations``.
+  ``bzrlib.workingtree.WorkingTreeTestProviderAdapter`` has moved to
+  ``bzrlib.tests.workingtree_implementations``.
+  These changes are an API break in the testing infrastructure only.
+  (Robert Collins)
+
+* Relocate TestCaseWithRepository to be more central. (Robert Collins)
+
+* ``bzrlib.add.smart_add_tree`` will no longer perform glob expansion on
+  win32. Callers of the function should do this and use the new
+  ``MutableTree.smart_add`` method instead. (Robert Collins)
+
+* ``bzrlib.add.glob_expand_for_win32`` is now
+  ``bzrlib.win32utils.glob_expand``.  (Robert Collins)
+
+* ``bzrlib.add.FastPath`` is now private and moved to
+  ``bzrlib.mutabletree._FastPath``. (Robert Collins, Martin Pool)
+
+* ``LockDir.wait`` removed.  (Martin Pool)
+
+* The ``SmartServer`` hooks API has changed for the ``server_started`` and
+  ``server_stopped`` hooks. The first parameter is now an iterable of
+  backing URLs rather than a single URL. This is to reflect that many
+  URLs may map to the external URL of the server. E.g. the server interally
+  may have a chrooted URL but also the local file:// URL will be at the
+  same location. (Robert Collins)
+
+Internals
+*********
+
+* New SMTPConnection class to unify email handling.  (Adeodato Simó)
+
+* Fix documentation of BzrError. (Adeodato Simó)
+
+* Make BzrBadParameter an internal error. (Adeodato Simó)
+
+* Remove use of 'assert False' to raise an exception unconditionally.
+  (Martin Pool)
+
+* Give a cleaner error when failing to decode knit index entry.
+  (Martin Pool)
+
+* TreeConfig would mistakenly search the top level when asked for options
+  from a section. It now respects the section argument and only
+  searches the specified section. (James Westby)
+
+* Improve ``make api-docs`` output. (John Arbash Meinel)
+
+* Use os.lstat rather than os.stat for osutils.make_readonly and
+  osutils.make_writeable. This makes the difftools plugin more
+  robust when dangling symlinks are found. (Elliot Murphy)
+
+* New ``-Dlock`` option to log (to ~/.bzr.log) information on when
+  lockdirs are taken or released.  (Martin Pool)
+
+* ``bzrlib`` Hooks are now nameable using ``Hooks.name_hook``. This
+  allows a nicer UI when hooks are running as the current hook can
+  be displayed. (Robert Collins)
+
+* ``Transport.get`` has had its interface made more clear for ease of use.
+  Retrieval of a directory must now fail with either 'PathError' at open
+  time, or raise 'ReadError' on a read. (Robert Collins)
+
+* New method ``_maybe_expand_globs`` on the ``Command`` class for
+  dealing with unexpanded glob lists - e.g. on the win32 platform. This
+  was moved from ``bzrlib.add._prepare_file_list``. (Robert Collins)
+
+* ``bzrlib.add.smart_add`` and ``bzrlib.add.smart_add_tree`` are now
+  deprecated in favour of ``MutableTree.smart_add``. (Robert Collins,
+  Martin Pool)
+
+* New method ``external_url`` on Transport for obtaining the URL to
+  hand to external processes. (Robert Collins)
+
+* Teach windows installers to build pyrex/C extensions.
+  (Alexander Belchenko)
+
+Testing
+*******
+
+* Removed the ``--keep-output`` option from selftest and clean up test
+  directories as they're used.  This reduces the IO load from
+  running the test suite and cuts the time by about half.
+  (Andrew Bennetts, Martin Pool)
+
+* Add scenarios as a public attribute on the TestAdapter classes to allow
+  modification of the generated scenarios before adaption and easier
+  testing. (Robert Collins)
+
+* New testing support class ``TestScenarioApplier`` which multiplies
+  out a single teste by a list of supplied scenarios. (RobertCollins)
+
+* Setting ``repository_to_test_repository`` on a repository_implementations
+  test will cause it to be called during repository creation, allowing the
+  testing of repository classes which are not based around the Format
+  concept. For example a repository adapter can be tested in this manner,
+  by altering the repository scenarios to include a scenario that sets this
+  attribute during the test parameterisation in
+  ``bzrlib.tests.repository.repository_implementations``. (Robert Collins)
+
+* Clean up many of the APIs for blackbox testing of Bazaar.  The standard
+  interface is now self.run_bzr.  The command to run can be passed as
+  either a list of parameters, a string containing the command line, or
+  (deprecated) varargs parameters.  (Martin Pool)
+
+* The base TestCase now isolates tests from -D parameters by clearing
+  ``debug.debug_flags`` and restores it afterwards. (Robert Collins)
+
+* Add a relpath parameter to get_transport methods in test framework to
+  avoid useless cloning.
+  (Vincent Ladeuil, #110448)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.6.txt'
--- doc/en/release-notes/bzr-0.6.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.6.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,227 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.6
+#######
+
+:Released: 2005-10-28
+
+Improvements
+************
+
+* pull now takes --verbose to show you what revisions are added or removed
+  (John A Meinel)
+
+* merge now takes a --show-base option to include the base text in
+  conflicts.
+  (Aaron Bentley)
+
+* The config files are now read using ConfigObj, so '=' should be used as
+  a separator, not ':'.
+  (Aaron Bentley)
+
+* New 'bzr commit --strict' option refuses to commit if there are
+  any unknown files in the tree.  To commit, make sure all files are
+  either ignored, added, or deleted.  (Michael Ellerman)
+
+* The config directory is now ~/.bazaar, and there is a single file
+  ~/.bazaar/bazaar.conf storing email, editor and other preferences.
+  (Robert Collins)
+
+* 'bzr add' no longer takes a --verbose option, and a --quiet option
+  has been added that suppresses all output.
+
+* Improved zsh completion support in contrib/zsh, from Clint
+  Adams.
+
+* Builtin 'bzr annotate' command, by Martin Pool with improvements from
+  Goffredo Baroncelli.
+
+* 'bzr check' now accepts -v for verbose reporting, and checks for
+  ghosts in the branch. (Robert Collins)
+
+* New command 're-sign' which will regenerate the gpg signature for
+  a revision. (Robert Collins)
+
+* If you set ``check_signatures=require`` for a path in
+  ``~/.bazaar/branches.conf`` then bzr will invoke your
+  ``gpg_signing_command`` (defaults to gpg) and record a digital signature
+  of your commit. (Robert Collins)
+
+* New SFTP transport, based on Paramiko.  (Robey Pointer)
+
+* 'bzr pull' now accepts '--clobber' which will discard local changes
+  and make this branch identical to the source branch. (Robert Collins)
+
+* Just give a quieter warning if a plugin can't be loaded, and
+  put the details in .bzr.log.  (Martin Pool)
+
+* 'bzr branch' will now set the branch-name to the last component of the
+  output directory, if one was supplied.
+
+* If the option ``post_commit`` is set to one (or more) python function
+  names (must be in the bzrlib namespace), then they will be invoked
+  after the commit has completed, with the branch and ``revision_id`` as
+  parameters. (Robert Collins)
+
+* Merge now has a retcode of 1 when conflicts occur. (Robert Collins)
+
+* --merge-type weave is now supported for file contents.  Tree-shape
+  changes are still three-way based.  (Martin Pool, Aaron Bentley)
+
+* 'bzr check' allows the first revision on revision-history to have
+  parents - something that is expected for cheap checkouts, and occurs
+  when conversions from baz do not have all history.  (Robert Collins).
+
+* 'bzr merge' can now graft unrelated trees together, if your specify
+  0 as a base. (Aaron Bentley)
+
+* 'bzr commit branch' and 'bzr commit branch/file1 branch/file2' now work
+  (Aaron Bentley)
+
+* Add '.sconsign*' to default ignore list.  (Alexander Belchenko)
+
+* 'bzr merge --reprocess' minimizes conflicts
+
+Testing
+*******
+
+* The 'bzr selftest --pattern' option for has been removed, now
+  test specifiers on the command line can be simple strings, or
+  regexps, or both. (Robert Collins)
+
+* Passing -v to selftest will now show the time each test took to
+  complete, which will aid in analysing performance regressions and
+  related questions. (Robert Collins)
+
+* 'bzr selftest' runs all tests, even if one fails, unless '--one'
+  is given. (Martin Pool)
+
+* There is a new method for TestCaseInTempDir, assertFileEqual, which
+  will check that a given content is equal to the content of the named
+  file. (Robert Collins)
+
+* Fix test suite's habit of leaving many temporary log files in $TMPDIR.
+  (Martin Pool)
+
+Internals
+*********
+
+* New 'testament' command and concept for making gpg-signatures
+  of revisions that are not tied to a particular internal
+  representation.  (Martin Pool).
+
+* Per-revision properties ('revprops') as key-value associated
+  strings on each revision created when the revision is committed.
+  Intended mainly for the use of external tools.  (Martin Pool).
+
+* Config options have moved from bzrlib.osutils to bzrlib.config.
+  (Robert Collins)
+
+* Improved command line option definitions allowing explanations
+  for individual options, among other things.  Contributed by
+  Magnus Therning.
+
+* Config options have moved from bzrlib.osutils to bzrlib.config.
+  Configuration is now done via the config.Config interface:
+  Depending on whether you have a Branch, a Location or no information
+  available, construct a ``*Config``, and use its ``signature_checking``,
+  ``username`` and ``user_email`` methods. (Robert Collins)
+
+* Plugins are now loaded under bzrlib.plugins, not bzrlib.plugin, and
+  they are made available for other plugins to use. You should not
+  import other plugins during the ``__init__`` of your plugin though, as
+  no ordering is guaranteed, and the plugins directory is not on the
+  python path. (Robert Collins)
+
+* Branch.relpath has been moved to WorkingTree.relpath. WorkingTree no
+  no longer takes an inventory, rather it takes an option branch
+  parameter, and if None is given will open the branch at basedir
+  implicitly. (Robert Collins)
+
+* Cleaner exception structure and error reporting.  Suggested by
+  Scott James Remnant.  (Martin Pool)
+
+* Branch.remove has been moved to WorkingTree, which has also gained
+  ``lock_read``, ``lock_write`` and ``unlock`` methods for convenience.
+  (Robert Collins)
+
+* Two decorators, ``needs_read_lock`` and ``needs_write_lock`` have been
+  added to the branch module. Use these to cause a function to run in a
+  read or write lock respectively. (Robert Collins)
+
+* ``Branch.open_containing`` now returns a tuple (Branch, relative-path),
+  which allows direct access to the common case of 'get me this file
+  from its branch'. (Robert Collins)
+
+* Transports can register using ``register_lazy_transport``, and they
+  will be loaded when first used.  (Martin Pool)
+
+* 'pull' has been factored out of the command as ``WorkingTree.pull()``.
+  A new option to WorkingTree.pull has been added, clobber, which will
+  ignore diverged history and pull anyway.
+  (Robert Collins)
+
+* config.Config has a ``get_user_option`` call that accepts an option name.
+  This will be looked up in branches.conf and bazaar.conf as normal.
+  It is intended that this be used by plugins to support options -
+  options of built in programs should have specific methods on the config.
+  (Robert Collins)
+
+* ``merge.merge_inner`` now has tempdir as an optional parameter.
+  (Robert Collins)
+
+* Tree.kind is not recorded at the top level of the hierarchy, as it was
+  missing on EmptyTree, leading to a bug with merge on EmptyTrees.
+  (Robert Collins)
+
+* ``WorkingTree.__del__`` has been removed, it was non deterministic and not
+  doing what it was intended to. See ``WorkingTree.__init__`` for a comment
+  about future directions. (Robert Collins/Martin Pool)
+
+* bzrlib.transport.http has been modified so that only 404 urllib errors
+  are returned as NoSuchFile. Other exceptions will propagate as normal.
+  This allows debuging of actual errors. (Robert Collins)
+
+* bzrlib.transport.Transport now accepts *ONLY* URL-escaped relative paths
+  to apis like 'put', 'get' and 'has'. This is to provide consistent
+  behaviour - it operates on URLs only. (Robert Collins)
+
+* Transports can register using ``register_lazy_transport``, and they
+  will be loaded when first used.  (Martin Pool)
+
+* ``merge_flex`` no longer calls ``conflict_handler.finalize()``, instead that
+  is called by ``merge_inner``. This is so that the conflict count can be
+  retrieved (and potentially manipulated) before returning to the caller
+  of ``merge_inner``. Likewise 'merge' now returns the conflict count to the
+  caller. (Robert Collins)
+
+* ``revision.revision_graph`` can handle having only partial history for
+  a revision - that is no revisions in the graph with no parents.
+  (Robert Collins).
+
+* New ``builtins.branch_files`` uses the standard ``file_list`` rules to
+  produce a branch and a list of paths, relative to that branch
+  (Aaron Bentley)
+
+* New TestCase.addCleanup facility.
+
+* New ``bzrlib.version_info`` tuple (similar to ``sys.version_info``),
+  which can be used by programs importing bzrlib.
+
+Bug Fixes
+*********
+
+* Better handling of branches in directories with non-ascii names.
+  (Joel Rosdahl, Panagiotis Papadakos)
+
+* Upgrades of trees with no commits will not fail due to accessing
+  [-1] in the revision-history. (Andres Salomon)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.7.txt'
--- doc/en/release-notes/bzr-0.7.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.7.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,308 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.7
+#######
+
+:Released: 2006-01-09
+
+Changes
+*******
+
+* .bzrignore is excluded from exports, on the grounds that it's a bzr
+  internal-use file and may not be wanted.  (Jamie Wilkinson)
+
+* The "bzr directories" command were removed in favor of the new
+  --kind option to the "bzr inventory" command.  To list all
+  versioned directories, now use "bzr inventory --kind directory".
+  (Johan Rydberg)
+
+* Under Windows configuration directory is now ``%APPDATA%\bazaar\2.0``
+  by default. (John Arbash Meinel)
+
+* The parent of Bzr configuration directory can be set by ``BZR_HOME``
+  environment variable. Now the path for it is searched in ``BZR_HOME``,
+  then in HOME. Under Windows the order is: ``BZR_HOME``, ``APPDATA``
+  (usually points to ``C:\Documents and Settings\User Name\Application Data``),
+  ``HOME``. (John Arbash Meinel)
+
+* Plugins with the same name in different directories in the bzr plugin
+  path are no longer loaded: only the first successfully loaded one is
+  used. (Robert Collins)
+
+* Use system's external SSH command to open connections if possible.
+  This gives better integration with user settings such as ProxyCommand.
+  (James Henstridge)
+
+* Permissions on files underneath .bzr/ are inherited from the .bzr
+  directory. So for a shared repository, simply doing 'chmod -R g+w .bzr/'
+  will mean that future file will be created with group write permissions.
+
+* configure.in and config.guess are no longer in the builtin default
+  ignore list.
+
+* '.sw[nop]' pattern ignored, to ignore vim swap files for nameless
+  files.  (John Arbash Meinel, Martin Pool)
+
+Improvements
+************
+
+* "bzr INIT dir" now initializes the specified directory, and creates
+  it if it does not exist.  (John Arbash Meinel)
+
+* New remerge command (Aaron Bentley)
+
+* Better zsh completion script.  (Steve Borho)
+
+* 'bzr diff' now returns 1 when there are changes in the working
+  tree. (Robert Collins)
+
+* 'bzr push' now exists and can push changes to a remote location.
+  This uses the transport infrastructure, and can store the remote
+  location in the ~/.bazaar/branches.conf configuration file.
+  (Robert Collins)
+
+* Test directories are only kept if the test fails and the user requests
+  that they be kept.
+
+* Tweaks to short log printing
+
+* Added branch nicks, new nick command, printing them in log output.
+  (Aaron Bentley)
+
+* If ``$BZR_PDB`` is set, pop into the debugger when an uncaught exception
+  occurs.  (Martin Pool)
+
+* Accept 'bzr resolved' (an alias for 'bzr resolve'), as this is
+  the same as Subversion.  (Martin Pool)
+
+* New FTP transport support (on ftplib), for ftp:// and aftp://
+  URLs.  (Daniel Silverstone)
+
+* Commit editor temporary files now start with ``bzr_log.``, to allow
+  text editors to match the file name and set up appropriate modes or
+  settings.  (Magnus Therning)
+
+* Improved performance when integrating changes from a remote weave.
+  (Goffredo Baroncelli)
+
+* Sftp will attempt to cache the connection, so it is more likely that
+  a connection will be reused, rather than requiring multiple password
+  requests.
+
+* bzr revno now takes an optional argument indicating the branch whose
+  revno should be printed.  (Michael Ellerman)
+
+* bzr cat defaults to printing the last version of the file.
+  (Matthieu Moy, #3632)
+
+* New global option 'bzr --lsprof COMMAND' runs bzr under the lsprof
+  profiler.  (Denys Duchier)
+
+* Faster commits by reading only the headers of affected weave files.
+  (Denys Duchier)
+
+* 'bzr add' now takes a --dry-run parameter which shows you what would be
+  added, but doesn't actually add anything. (Michael Ellerman)
+
+* 'bzr add' now lists how many files were ignored per glob.  add --verbose
+  lists the specific files.  (Aaron Bentley)
+
+* 'bzr missing' now supports displaying changes in diverged trees and can
+  be limited to show what either end of the comparison is missing.
+  (Aaron Bently, with a little prompting from Daniel Silverstone)
+
+Bug Fixes
+*********
+
+* SFTP can walk up to the root path without index errors. (Robert Collins)
+
+* Fix bugs in running bzr with 'python -O'.  (Martin Pool)
+
+* Error when run with -OO
+
+* Fix bug in reporting HTTP errors that don't have an HTTP error code.
+  (Martin Pool)
+
+* Handle more cases of pipe errors in display commands
+
+* Change status to 3 for all errors
+
+* Files that are added and unlinked before committing are completely
+  ignored by diff and status
+
+* Stores with some compressed texts and some uncompressed texts are now
+  able to be used. (John A Meinel)
+
+* Fix for bzr pull failing sometimes under windows
+
+* Fix for SFTP transport under windows when using interactive auth
+
+* Show files which are both renamed and modified as such in 'bzr
+  status' output.  (Daniel Silverstone, #4503)
+
+* Make annotate cope better with revisions committed without a valid
+  email address.  (Marien Zwart)
+
+* Fix representation of tab characters in commit messages.
+  (Harald Meland)
+
+* List of plugin directories in ``BZR_PLUGIN_PATH`` environment variable is
+  now parsed properly under Windows. (Alexander Belchenko)
+
+* Show number of revisions pushed/pulled/merged. (Robey Pointer)
+
+* Keep a cached copy of the basis inventory to speed up operations
+  that need to refer to it.  (Johan Rydberg, Martin Pool)
+
+* Fix bugs in bzr status display of non-ascii characters.
+  (Martin Pool)
+
+* Remove Makefile.in from default ignore list.
+  (Tollef Fog Heen, Martin Pool, #6413)
+
+* Fix failure in 'bzr added'.  (Nathan McCallum, Martin Pool)
+
+Testing
+*******
+
+* Fix selftest asking for passwords when there are no SFTP keys.
+  (Robey Pointer, Jelmer Vernooij)
+
+* Fix selftest run with 'python -O'.  (Martin Pool)
+
+* Fix HTTP tests under Windows. (John Arbash Meinel)
+
+* Make tests work even if HOME is not set (Aaron Bentley)
+
+* Updated ``build_tree`` to use fixed line-endings for tests which read
+  the file cotents and compare. Make some tests use this to pass under
+  Windows. (John Arbash Meinel)
+
+* Skip stat and symlink tests under Windows. (Alexander Belchenko)
+
+* Delay in selftest/testhashcash is now issued under win32 and Cygwin.
+  (John Arbash Meinel)
+
+* Use terminal width to align verbose test output.  (Martin Pool)
+
+* Blackbox tests are maintained within the bzrlib.tests.blackbox directory.
+  If adding a new test script please add that to
+  ``bzrlib.tests.blackbox.__init__``. (Robert Collins)
+
+* Much better error message if one of the test suites can't be
+  imported.  (Martin Pool)
+
+* Make check now runs the test suite twice - once with the default locale,
+  and once with all locales forced to C, to expose bugs. This is not
+  trivially done within python, so for now its only triggered by running
+  Make check. Integrators and packagers who wish to check for full
+  platform support should run 'make check' to test the source.
+  (Robert Collins)
+
+* Tests can now run TestSkipped if they can't execute for any reason.
+  (Martin Pool) (NB: TestSkipped should only be raised for correctable
+  reasons - see the wiki spec ImprovingBzrTestSuite).
+
+* Test SFTP with relative, absolute-in-homedir and absolute-not-in-homedir
+  paths for the transport tests. Introduce blackbox remote SFTP tests that
+  test the same permutations. (Robert Collins, Robey Pointer)
+
+* Transport implementation tests are now independent of the local file
+  system, which allows tests for esoteric transports, and for features
+  not available in the local file system. They also repeat for variations
+  on the URL scheme that can introduce issues in the transport code,
+  see bzrlib.transport.TransportTestProviderAdapter() for this.
+  (Robert Collins).
+
+* ``TestCase.build_tree`` uses the transport interface to build trees,
+  pass in a transport parameter to give it an existing connection.
+  (Robert Collins).
+
+Internals
+*********
+
+* WorkingTree.pull has been split across Branch and WorkingTree,
+  to allow Branch only pulls. (Robert Collins)
+
+* ``commands.display_command`` now returns the result of the decorated
+  function. (Robert Collins)
+
+* LocationConfig now has a ``set_user_option(key, value)`` call to save
+  a setting in its matching location section (a new one is created
+  if needed). (Robert Collins)
+
+* Branch has two new methods, ``get_push_location`` and
+  ``set_push_location`` to respectively, get and set the push location.
+  (Robert Collins)
+
+* ``commands.register_command`` now takes an optional flag to signal that
+  the registrant is planning to decorate an existing command. When
+  given multiple plugins registering a command is not an error, and
+  the original command class (whether built in or a plugin based one) is
+  returned to the caller. There is a new error 'MustUseDecorated' for
+  signalling when a wrapping command should switch to the original
+  version. (Robert Collins)
+
+* Some option parsing errors will raise 'BzrOptionError', allowing
+  granular detection for decorating commands. (Robert Collins).
+
+* ``Branch.read_working_inventory`` has moved to
+  ``WorkingTree.read_working_inventory``. This necessitated changes to
+  ``Branch.get_root_id``, and a move of ``Branch.set_inventory`` to
+  WorkingTree as well. To make it clear that a WorkingTree cannot always
+  be obtained ``Branch.working_tree()`` will raise
+  ``errors.NoWorkingTree`` if one cannot be obtained. (Robert Collins)
+
+* All pending merges operations from Branch are now on WorkingTree.
+  (Robert Collins)
+
+* The follow operations from Branch have moved to WorkingTree::
+
+      add()
+      commit()
+      move()
+      rename_one()
+      unknowns()
+
+  (Robert Collins)
+
+* ``bzrlib.add.smart_add_branch`` is now ``smart_add_tree``. (Robert Collins)
+
+* New "rio" serialization format, similar to rfc-822. (Martin Pool)
+
+* Rename selftests to ``bzrlib.tests.test_foo``.  (John A Meinel, Martin
+  Pool)
+
+* ``bzrlib.plugin.all_plugins`` has been changed from an attribute to a
+  query method. (Robert Collins)
+
+* New options to read only the table-of-contents of a weave.
+  (Denys Duchier)
+
+* Raise NoSuchFile when someone tries to add a non-existant file.
+  (Michael Ellerman)
+
+* Simplify handling of DivergedBranches in ``cmd_pull()``.
+  (Michael Ellerman)
+
+* Branch.controlfile* logic has moved to lockablefiles.LockableFiles, which
+  is exposed as ``Branch().control_files``. Also this has been altered with the
+  controlfile pre/suffix replaced by simple method names like 'get' and
+  'put'. (Aaron Bentley, Robert Collins).
+
+* Deprecated functions and methods can now be marked as such using the
+  ``bzrlib.symbol_versioning`` module. Marked method have their docstring
+  updated and will issue a DeprecationWarning using the warnings module
+  when they are used. (Robert Collins)
+
+* ``bzrlib.osutils.safe_unicode`` now exists to provide parameter coercion
+  for functions that need unicode strings. (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.8.txt'
--- doc/en/release-notes/bzr-0.8.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.8.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,340 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.8.2
+#########
+
+:Released:  2006-05-17
+
+Bug Fixes
+*********
+
+* setup.py failed to install launchpad plugin.  (Martin Pool)
+
+bzr 0.8.1
+#########
+
+:Released:  2006-05-16
+
+Bug Fixes
+*********
+
+* Fix failure to commit a merge in a checkout.  (Martin Pool,
+  Robert Collins, Erik Bågfors, #43959)
+
+* Nicer messages from 'commit' in the case of renames, and correct
+  messages when a merge has occured. (Robert Collins, Martin Pool)
+
+* Separate functionality from assert statements as they are skipped in
+  optimized mode of python. Add the same check to pending merges.
+  (Olaf Conradi, #44443)
+
+Changes
+*******
+
+* Do not show the None revision in output of bzr ancestry. (Olaf Conradi)
+
+* Add info on standalone branches without a working tree.
+  (Olaf Conradi, #44155)
+
+* Fix bug in knits when raising InvalidRevisionId. (Olaf Conradi, #44284)
+
+Changes
+*******
+
+* Make editor invocation comply with Debian Policy. First check
+  environment variables VISUAL and EDITOR, then try editor from
+  alternatives system. If that all fails, fall back to the pre-defined
+  list of editors. (Olaf Conradi, #42904)
+
+New Features
+************
+
+* New 'register-branch' command registers a public branch into
+  Launchpad.net, where it can be associated with bugs, etc.
+  (Martin Pool, Bjorn Tillenius, Robert Collins)
+
+Internals
+*********
+
+* New public api in InventoryEntry - ``describe_change(old, new)`` which
+  provides a human description of the changes between two old and
+  new. (Robert Collins, Martin Pool)
+
+Testing
+*******
+
+* Fix test case for bzr info in upgrading a standalone branch to metadir,
+  uses bzrlib api now. (Olaf Conradi)
+
+bzr 0.8
+#######
+
+:Released:  2006-05-08
+
+Notes When Upgrading
+********************
+
+Release 0.8 of bzr introduces a new format for history storage, called
+'knit', as an evolution of to the 'weave' format used in 0.7.  Local
+and remote operations are faster using knits than weaves.  Several
+operations including 'init', 'init-repo', and 'upgrade' take a
+--format option that controls this.  Branching from an existing branch
+will keep the same format.
+
+It is possible to merge, pull and push between branches of different
+formats but this is slower than moving data between homogenous
+branches.  It is therefore recommended (but not required) that you
+upgrade all branches for a project at the same time.  Information on
+formats is shown by 'bzr info'.
+
+bzr 0.8 now allows creation of 'repositories', which hold the history
+of files and revisions for several branches.  Previously bzr kept all
+the history for a branch within the .bzr directory at the root of the
+branch, and this is still the default.  To create a repository, use
+the new 'bzr init-repo' command.  Branches exist as directories under
+the repository and contain just a small amount of information
+indicating the current revision of the branch.
+
+bzr 0.8 also supports 'checkouts', which are similar to in cvs and
+subversion.  Checkouts are associated with a branch (optionally in a
+repository), which contains all the historical information.  The
+result is that a checkout can be deleted without losing any
+already-committed revisions.  A new 'update' command is also available.
+
+Repositories and checkouts are not supported with the 0.7 storage
+format.  To use them you must upgrad to either knits, or to the
+'metaweave' format, which uses weaves but changes the .bzr directory
+arrangement.
+
+
+Improvements
+************
+
+* SFTP paths can now be relative, or local, according to the lftp
+  convention. Paths now take the form::
+
+      sftp://user:pass@host:port/~/relative/path
+      or
+      sftp://user:pass@host:port/absolute/path
+
+* The FTP transport now tries to reconnect after a temporary
+  failure. FTP put is made atomic. (Matthieu Moy)
+
+* The FTP transport now maintains a pool of connections, and
+  reuses them to avoid multiple connections to the same host (like
+  SFTP did). (Daniel Silverstone)
+
+* The ``bzr_man.py`` file has been removed. To create the man page now,
+  use ``./generate_docs.py man``. The new program can also create other files.
+  Run ``python generate_docs.py --help`` for usage information.
+  (Hans Ulrich Niedermann & James Blackwell).
+
+* Man Page now gives full help (James Blackwell).
+  Help also updated to reflect user config now being stored in .bazaar
+  (Hans Ulrich Niedermann)
+
+* It's now possible to set aliases in bazaar.conf (Erik Bågfors)
+
+* Pull now accepts a --revision argument (Erik Bågfors)
+
+* ``bzr re-sign`` now allows multiple revisions to be supplied on the command
+  line. You can now use the following command to sign all of your old
+  commits::
+
+    find .bzr/revision-store// -name my@email-* \
+      | sed 's/.*\/\/..\///' \
+      | xargs bzr re-sign
+
+* Upgrade can now upgrade over the network. (Robert Collins)
+
+* Two new commands 'bzr checkout' and 'bzr update' allow for CVS/SVN-alike
+  behaviour.  By default they will cache history in the checkout, but
+  with --lightweight almost all data is kept in the master branch.
+  (Robert Collins)
+
+* 'revert' unversions newly-versioned files, instead of deleting them.
+
+* 'merge' is more robust.  Conflict messages have changed.
+
+* 'merge' and 'revert' no longer clobber existing files that end in '~' or
+  '.moved'.
+
+* Default log format can be set in configuration and plugins can register
+  their own formatters. (Erik Bågfors)
+
+* New 'reconcile' command will check branch consistency and repair indexes
+  that can become out of sync in pre 0.8 formats. (Robert Collins,
+  Daniel Silverstone)
+
+* New 'bzr init --format' and 'bzr upgrade --format' option to control
+  what storage format is created or produced.  (Robert Collins,
+  Martin Pool)
+
+* Add parent location to 'bzr info', if there is one.  (Olaf Conradi)
+
+* New developer commands 'weave-list' and 'weave-join'.  (Martin Pool)
+
+* New 'init-repository' command, plus support for repositories in 'init'
+  and 'branch' (Aaron Bentley, Erik Bågfors, Robert Collins)
+
+* Improve output of 'info' command. Show all relevant locations related to
+  working tree, branch and repository. Use kibibytes for binary quantities.
+  Fix off-by-one error in missing revisions of working tree.  Make 'info'
+  work on branches, repositories and remote locations.  Show locations
+  relative to the shared repository, if applicable.  Show locking status
+  of locations.  (Olaf Conradi)
+
+* Diff and merge now safely handle binary files. (Aaron Bentley)
+
+* 'pull' and 'push' now normalise the revision history, so that any two
+  branches with the same tip revision will have the same output from 'log'.
+  (Robert Collins)
+
+* 'merge' accepts --remember option to store parent location, like 'push'
+  and 'pull'. (Olaf Conradi)
+
+* bzr status and diff when files given as arguments do not exist
+  in the relevant trees.  (Martin Pool, #3619)
+
+* Add '.hg' to the default ignore list.  (Martin Pool)
+
+* 'knit' is now the default disk format. This improves disk performance and
+  utilization, increases incremental pull performance, robustness with SFTP
+  and allows checkouts over SFTP to perform acceptably.
+  The initial Knit code was contributed by Johan Rydberg based on a
+  specification by Martin Pool.
+  (Robert Collins, Aaron Bentley, Johan Rydberg, Martin Pool).
+
+* New tool to generate all-in-one html version of the manual.  (Alexander
+  Belchenko)
+
+* Hitting CTRL-C while doing an SFTP push will no longer cause stale locks
+  to be left in the SFTP repository. (Robert Collins, Martin Pool).
+
+* New option 'diff --prefix' to control how files are named in diff
+  output, with shortcuts '-p0' and '-p1' corresponding to the options for
+  GNU patch.  (Alexander Belchenko, Goffredo Baroncelli, Martin Pool)
+
+* Add --revision option to 'annotate' command.  (Olaf Conradi)
+
+* If bzr shows an unexpected revision-history after pulling (perhaps due
+  to a reweave) it can now be corrected by 'bzr reconcile'.
+  (Robert Collins)
+
+Changes
+*******
+
+* Commit is now verbose by default, and shows changed filenames and the
+  new revision number.  (Robert Collins, Martin Pool)
+
+* Unify 'mv', 'move', 'rename'.  (Matthew Fuller, #5379)
+
+* 'bzr -h' shows help.  (Martin Pool, Ian Bicking, #35940)
+
+* Make 'pull' and 'push' remember location on failure using --remember.
+  (Olaf Conradi)
+
+* For compatibility, make old format for using weaves inside metadir
+  available as 'metaweave' format.  Rename format 'metadir' to 'default'.
+  Clean up help for option --format in commands 'init', 'init-repo' and
+  'upgrade'.  (Olaf Conradi)
+
+Internals
+*********
+
+* The internal storage of history, and logical branch identity have now
+  been split into Branch, and Repository. The common locking and file
+  management routines are now in bzrlib.lockablefiles.
+  (Aaron Bentley, Robert Collins, Martin Pool)
+
+* Transports can now raise DependencyNotPresent if they need a library
+  which is not installed, and then another implementation will be
+  tried.  (Martin Pool)
+
+* Remove obsolete (and no-op) `decode` parameter to `Transport.get`.
+  (Martin Pool)
+
+* Using Tree Transform for merge, revert, tree-building
+
+* WorkingTree.create, Branch.create, ``WorkingTree.create_standalone``,
+  Branch.initialize are now deprecated. Please see ``BzrDir.create_*`` for
+  replacement API's. (Robert Collins)
+
+* New BzrDir class represents the .bzr control directory and manages
+  formatting issues. (Robert Collins)
+
+* New repository.InterRepository class encapsulates Repository to
+  Repository actions and allows for clean selection of optimised code
+  paths. (Robert Collins)
+
+* ``bzrlib.fetch.fetch`` and ``bzrlib.fetch.greedy_fetch`` are now
+  deprecated, please use ``branch.fetch`` or ``repository.fetch``
+  depending on your needs. (Robert Collins)
+
+* deprecated methods now have a ``is_deprecated`` flag on them that can
+  be checked, if you need to determine whether a given callable is
+  deprecated at runtime. (Robert Collins)
+
+* Progress bars are now nested - see
+  ``bzrlib.ui.ui_factory.nested_progress_bar``.
+  (Robert Collins, Robey Pointer)
+
+* New API call ``get_format_description()`` for each type of format.
+  (Olaf Conradi)
+
+* Changed ``branch.set_parent()`` to accept None to remove parent.
+  (Olaf Conradi)
+
+* Deprecated BzrError AmbiguousBase.  (Olaf Conradi)
+
+* WorkingTree.branch is now a read only property.  (Robert Collins)
+
+* bzrlib.ui.text.TextUIFactory now accepts a ``bar_type`` parameter which
+  can be None or a factory that will create a progress bar. This is
+  useful for testing or for overriding the bzrlib.progress heuristic.
+  (Robert Collins)
+
+* New API method ``get_physical_lock_status()`` to query locks present on a
+  transport.  (Olaf Conradi)
+
+* Repository.reconcile now takes a thorough keyword parameter to allow
+  requesting an indepth reconciliation, rather than just a data-loss
+  check. (Robert Collins)
+
+* ``bzrlib.ui.ui_factory protocol`` now supports ``get_boolean`` to prompt
+  the user for yes/no style input. (Robert Collins)
+
+Testing
+*******
+
+* SFTP tests now shortcut the SSH negotiation, reducing test overhead
+  for testing SFTP protocol support. (Robey Pointer)
+
+* Branch formats are now tested once per implementation (see ``bzrlib.
+  tests.branch_implementations``. This is analagous to the transport
+  interface tests, and has been followed up with working tree,
+  repository and BzrDir tests. (Robert Collins)
+
+* New test base class TestCaseWithTransport provides a transport aware
+  test environment, useful for testing any transport-interface using
+  code. The test suite option --transport controls the transport used
+  by this class (when its not being used as part of implementation
+  contract testing). (Robert Collins)
+
+* Close logging handler on disabling the test log. This will remove the
+  handler from the internal list inside python's logging module,
+  preventing shutdown from closing it twice.  (Olaf Conradi)
+
+* Move test case for uncommit to blackbox tests.  (Olaf Conradi)
+
+* ``run_bzr`` and ``run_bzr_captured`` now accept a 'stdin="foo"'
+  parameter which will provide String("foo") to the command as its stdin.
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.9.txt'
--- doc/en/release-notes/bzr-0.9.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.9.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,280 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.9.0
+#########
+
+:Released:  2006-08-11
+
+Surprises
+*********
+
+* The hard-coded built-in ignore rules have been removed. There are
+  now two rulesets which are enforced. A user global one in
+  ``~/.bazaar/ignore`` which will apply to every tree, and the tree
+  specific one '.bzrignore'.
+  ``~/.bazaar/ignore`` will be created if it does not exist, but with
+  a more conservative list than the old default.
+  This fixes bugs with default rules being enforced no matter what.
+  The old list of ignore rules from bzr is available by
+  running 'bzr ignore --old-default-rules'.
+  (Robert Collins, Martin Pool, John Arbash Meinel)
+
+* 'branches.conf' has been changed to 'locations.conf', since it can apply
+  to more locations than just branch locations.
+  (Aaron Bentley)
+
+Improvements
+************
+
+* The revision specifier "revno:" is extended to accept the syntax
+  revno:N:branch. For example,
+  revno:42:http://bazaar-vcs.org/bzr/bzr.dev/ means revision 42 in
+  bzr.dev.  (Matthieu Moy)
+
+* Tests updates to ensure proper URL handling, UNICODE support, and
+  proper printing when the user's terminal encoding cannot display
+  the path of a file that has been versioned.
+  ``bzr branch`` can take a target URL rather than only a local directory.
+  ``Branch.get_parent()/set_parent()`` now save a relative path if possible,
+  and normalize the parent based on root, allowing access across
+  different transports. (John Arbash Meinel, Wouter van Heyst, Martin Pool)
+  (Malone #48906, #42699, #40675, #5281, #3980, #36363, #43689,
+  #42517, #42514)
+
+* On Unix, detect terminal width using an ioctl not just $COLUMNS.
+  Use terminal width for single-line logs from ``bzr log --line`` and
+  pending-merge display.  (Robert Widhopf-Fenk, Gustavo Niemeyer)
+  (Malone #3507)
+
+* On Windows, detect terminal width using GetConsoleScreenBufferInfo.
+  (Alexander Belchenko)
+
+* Speedup improvement for 'date:'-revision search. (Guillaume Pinot).
+
+* Show the correct number of revisions pushed when pushing a new branch.
+  (Robert Collins).
+
+* 'bzr selftest' now shows a progress bar with the number of tests, and
+  progress made. 'make check' shows tests in -v mode, to be more useful
+  for the PQM status window. (Robert Collins).
+  When using a progress bar, failed tests are printed out, rather than
+  being overwritten by the progress bar until the suite finishes.
+  (John Arbash Meinel)
+
+* 'bzr selftest --benchmark' will run a new benchmarking selftest.
+  'bzr selftest --benchmark --lsprof-timed' will use lsprofile to generate
+  profile data for the individual profiled calls, allowing for fine
+  grained analysis of performance.
+  (Robert Collins, Martin Pool).
+
+* 'bzr commit' shows a progress bar. This is useful for commits over SFTP
+  where commit can take an appreciable time. (Robert Collins)
+
+* 'bzr add' is now less verbose in telling you what ignore globs were
+  matched by files being ignored. Instead it just tells you how many
+  were ignored (because you might reasonably be expecting none to be
+  ignored). 'bzr add -v' is unchanged and will report every ignored
+  file. (Robert Collins).
+
+* FTP now has a test server if medusa is installed. As part of testing,
+  FTP support has been improved, including support for supplying a
+  non-standard port. (John Arbash Meinel).
+
+* 'bzr log --line' shows the revision number, and uses only the
+  first line of the log message (#5162, Alexander Belchenko;
+  Matthieu Moy)
+
+* 'bzr status' has had the --all option removed. The 'bzr ls' command
+  should be used to retrieve all versioned files. (Robert Collins)
+
+* 'bzr bundle OTHER/BRANCH' will create a bundle which can be sent
+  over email, and applied on the other end, while maintaining ancestry.
+  This bundle can be applied with either 'bzr merge' or 'bzr pull',
+  the same way you would apply another branch.
+  (John Arbash Meinel, Aaron Bentley)
+
+* 'bzr whoami' can now be used to set your identity from the command line,
+  for a branch or globally.  (Robey Pointer)
+
+* 'bzr checkout' now aliased to 'bzr co', and 'bzr annotate' to 'bzr ann'.
+  (Michael Ellerman)
+
+* 'bzr revert DIRECTORY' now reverts the contents of the directory as well.
+  (Aaron Bentley)
+
+* 'bzr get sftp://foo' gives a better error when paramiko is not present.
+  Also updates things like 'http+pycurl://' if pycurl is not present.
+  (John Arbash Meinel) (Malone #47821, #52204)
+
+* New env variable ``BZR_PROGRESS_BAR``, sets the default progress bar type.
+  Can be set to 'none' or 'dummy' to disable the progress bar, 'dots' or
+  'tty' to create the respective type. (John Arbash Meinel, #42197, #51107)
+
+* Improve the help text for 'bzr diff' to explain what various options do.
+  (John Arbash Meinel, #6391)
+
+* 'bzr uncommit -r 10' now uncommits revisions 11.. rather than uncommitting
+  revision 10. This makes -r10 more in line with what other commands do.
+  'bzr uncommit' also now saves the pending merges of the revisions that
+  were removed. So it is safe to uncommit after a merge, fix something,
+  and commit again. (John Arbash Meinel, #32526, #31426)
+
+* 'bzr init' now also works on remote locations.
+  (Wouter van Heyst, #48904)
+
+* HTTP support has been updated. When using pycurl we now support
+  connection keep-alive, which reduces dns requests and round trips.
+  And for both urllib and pycurl we support multi-range requests,
+  which decreases the number of round-trips. Performance results for
+  ``bzr branch http://bazaar-vcs.org/bzr/bzr.dev/`` indicate
+  HTTP branching is now 2-3x faster, and ``bzr pull`` in an existing
+  branch is as much as 4x faster.
+  (Michael Ellerman, Johan Rydberg, John Arbash Meinel, #46768)
+
+* Performance improvements for SFTP. Branching and pulling are now up to
+  2x faster. Utilize paramiko.readv() support for async requests if it
+  is available (paramiko > 1.6) (John Arbash Meinel)
+
+Bug Fixes
+*********
+
+* Fix shadowed definition of TestLocationConfig that caused some
+  tests not to run.
+  (Erik Bågfors, Michael Ellerman, Martin Pool, #32587)
+
+* Fix unnecessary requirement of sign-my-commits that it be run from
+  a working directory.  (Martin Pool, Robert Collins)
+
+* 'bzr push location' will only remember the push location if it succeeds
+  in connecting to the remote location. (John Arbash Meinel, #49742)
+
+* 'bzr revert' no longer toggles the executable bit on win32
+  (John Arbash Meinel, #45010)
+
+* Handle broken pipe under win32 correctly. (John Arbash Meinel)
+
+* SFTP tests now work correctly on win32 if you have a newer paramiko
+  (John Arbash Meinel)
+
+* Cleanup win32 test suite, and general cleanup of places where
+  file handles were being held open. (John Arbash Meinel)
+
+* When specifying filenames for 'diff -r x..y', the name of the file in the
+  working directory can be used, even if its name is different in both x
+  and y.
+
+* File-ids containing single- or double-quotes are handled correctly by
+  push. (Aaron Bentley, #52227)
+
+* Normalize unicode filenames to ensure cross-platform consistency.
+  (John Arbash Meinel, #43689)
+
+* The argument parser can now handle '-' as an argument. Currently
+  no code interprets it specially (it is mostly handled as a file named
+  '-'). But plugins, and future operations can use it.
+  (John Arbash meinel, #50984)
+
+* Bundles can properly read binary files with a plain '\r' in them.
+  (John Arbash Meinel, #51927)
+
+* Tuning ``iter_entries()`` to be more efficient (John Arbash Meinel, #5444)
+
+* Lots of win32 fixes (the test suite passes again).
+  (John Arbash Meinel, #50155)
+
+* Handle openbsd returning None for sys.getfilesystemencoding() (#41183)
+
+* Support FTP APPE (append) to allow Knits to be used over FTP (#42592)
+
+* Removals are only committed if they match the filespec (or if there is
+  no filespec).  (#46635, Aaron Bentley)
+
+* smart-add recurses through all supplied directories
+  (John Arbash Meinel, #52578)
+
+* Make the bundle reader extra lines before and after the bundle text.
+  This allows you to parse an email with the bundle inline.
+  (John Arbash Meinel, #49182)
+
+* Change the file id generator to squash a little bit more. Helps when
+  working with long filenames on windows. (Also helps for unicode filenames
+  not generating hidden files). (John Arbash Meinel, #43801)
+
+* Restore terminal mode on C-c while reading SFTP password.  (#48923,
+  Nicholas Allen, Martin Pool)
+
+* Timestamps are rounded to 1ms, and revision entries can be recreated
+  exactly. (John Arbash Meinel, Jamie Wilkinson, #40693)
+
+* Branch.base has changed to a URL, but ~/.bazaar/locations.conf should
+  use local paths, since it is user visible (John Arbash Meinel, #53653)
+
+* ``bzr status foo`` when foo was unversioned used to cause a full delta
+  to be generated (John Arbash Meinel, #53638)
+
+* When reading revision properties, an empty value should be considered
+  the empty string, not None (John Arbash Meinel, #47782)
+
+* ``bzr diff --diff-options`` can now handle binary files being changed.
+  Also, the output is consistent when --diff-options is not supplied.
+  (John Arbash Meinel, #54651, #52930)
+
+* Use the right suffixes for loading plugins (John Arbash Meinel, #51810)
+
+* Fix ``Branch.get_parent()`` to handle the case when the parent is not
+  accessible (John Arbash Meinel, #52976)
+
+Internals
+*********
+
+* Combine the ignore rules into a single regex rather than looping over
+  them to reduce the threshold where  N^2 behaviour occurs in operations
+  like status. (Jan Hudec, Robert Collins).
+
+* Appending to ``bzrlib.DEFAULT_IGNORE`` is now deprecated. Instead, use
+  one of the add functions in bzrlib.ignores. (John Arbash Meinel)
+
+* 'bzr push' should only push the ancestry of the current revision, not
+  all of the history in the repository. This is especially important for
+  shared repositories. (John Arbash Meinel)
+
+* ``bzrlib.delta.compare_trees`` now iterates in alphabetically sorted order,
+  rather than randomly walking the inventories. (John Arbash Meinel)
+
+* Doctests are now run in temporary directories which are cleaned up when
+  they finish, rather than using special ScratchDir/ScratchBranch objects.
+  (Martin Pool)
+
+* Split ``check`` into separate methods on the branch and on the repository,
+  so that it can be specialized in ways that are useful or efficient for
+  different formats.  (Martin Pool, Robert Collins)
+
+* Deprecate ``Repository.all_revision_ids``; most methods don't really need
+  the global revision graph but only that part leading up to a particular
+  revision.  (Martin Pool, Robert Collins)
+
+* Add a BzrDirFormat ``control_formats`` list which allows for control formats
+  that do not use '.bzr' to store their data - i.e. '.svn', '.hg' etc.
+  (Robert Collins, Jelmer Vernooij).
+
+* ``bzrlib.diff.external_diff`` can be redirected to any file-like object.
+  Uses subprocess instead of spawnvp.
+  (James Henstridge, John Arbash Meinel, #4047, #48914)
+
+* New command line option '--profile-imports', which will install a custom
+  importer to log time to import modules and regex compilation time to
+  sys.stderr (John Arbash Meinel)
+
+* 'EmptyTree' is now deprecated, please use ``repository.revision_tree(None)``
+  instead. (Robert Collins)
+
+* "RevisionTree" is now in bzrlib/revisiontree.py. (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.90.txt'
--- doc/en/release-notes/bzr-0.90.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.90.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,311 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.90
+########
+
+:Released: 2007-08-28
+
+Improvements
+************
+
+* Documentation is now organized into multiple directories with a level
+  added for different languages or locales. Added the Mini Tutorial
+  and Quick Start Summary (en) documents from the Wiki, improving the
+  content and readability of the former. Formatted NEWS as Release Notes
+  complete with a Table of Conents, one heading per release. Moved the
+  Developer Guide into the main document catalog and provided a link
+  from the developer document catalog back to the main one.
+  (Ian Clatworthy, Sabin Iacob, Alexander Belchenko)
+
+
+API Changes
+***********
+
+* The static convenience method ``BzrDir.create_repository``
+  is deprecated.  Callers should instead create a ``BzrDir`` instance
+  and call ``create_repository`` on that.  (Martin Pool)
+
+
+bzr 0.90rc1
+###########
+
+:Released: 2007-08-14
+
+Bugfixes
+********
+
+* ``bzr init`` should connect to the remote location one time only.  We
+  have been connecting several times because we forget to pass around the
+  Transport object. This modifies ``BzrDir.create_branch_convenience``,
+  so that we can give it the Transport we already have.
+  (John Arbash Meinel, Vincent Ladeuil, #111702)
+
+* Get rid of SFTP connection cache (get rid of the FTP one too).
+  (Vincent Ladeuil, #43731)
+
+* bzr branch {local|remote} remote don't try to create a working tree
+  anymore.
+  (Vincent Ladeuil, #112173)
+
+* All identified multiple connections for a single bzr command have been
+  fixed. See bzrlib/tests/commands directory.
+  (Vincent Ladeuil)
+
+* ``bzr rm`` now does not insist on ``--force`` to delete files that
+  have been renamed but not otherwise modified.  (Marius Kruger,
+  #111664)
+
+* ``bzr selftest --bench`` no longer emits deprecation warnings
+  (Lukáš Lalinský)
+
+* ``bzr status`` now honours FILE parameters for conflict lists
+  (Aaron Bentley, #127606)
+
+* ``bzr checkout`` now honours -r when reconstituting a working tree.
+  It also honours -r 0.  (Aaron Bentley, #127708)
+
+* ``bzr add *`` no more fails on Windows if working tree contains
+  non-ascii file names. (Kuno Meyer, #127361)
+
+* allow ``easy_install bzr`` runs without fatal errors.
+  (Alexander Belchenko, #125521)
+
+* Graph._filter_candidate_lca does not raise KeyError if a candidate
+  is eliminated just before it would normally be examined.  (Aaron Bentley)
+
+* SMTP connection failures produce a nice message, not a traceback.
+  (Aaron Bentley)
+
+Improvements
+************
+
+* Don't show "dots" progress indicators when run non-interactively, such
+  as from cron.  (Martin Pool)
+
+* ``info`` now formats locations more nicely and lists "submit" and
+  "public" branches (Aaron Bentley)
+
+* New ``pack`` command that will trigger database compression within
+  the repository (Robert Collins)
+
+* Implement ``_KnitIndex._load_data`` in a pyrex extension. The pyrex
+  version is approximately 2-3x faster at parsing a ``.kndx`` file.
+  Which yields a measurable improvement for commands which have to
+  read from the repository, such as a 1s => 0.75s improvement in
+  ``bzr diff`` when there are changes to be shown.  (John Arbash Meinel)
+
+* Merge is now faster.  Depending on the scenario, it can be more than 2x
+  faster. (Aaron Bentley)
+
+* Give a clearer warning, and allow ``python setup.py install`` to
+  succeed even if pyrex is not available.
+  (John Arbash Meinel)
+
+* ``DirState._read_dirblocks`` now has an optional Pyrex
+  implementation. This improves the speed of any command that has to
+  read the entire DirState. (``diff``, ``status``, etc, improve by
+  about 10%).
+  ``bisect_dirblocks`` has also been improved, which helps all
+  ``_get_entry`` type calls (whenever we are searching for a
+  particular entry in the in-memory DirState).
+  (John Arbash Meinel)
+
+* ``bzr pull`` and ``bzr push`` no longer do a complete walk of the
+  branch revision history for ui display unless -v is supplied.
+  (Robert Collins)
+
+* ``bzr log -rA..B`` output shifted to the left margin if the log only
+  contains merge revisions. (Kent Gibson)
+
+* The ``plugins`` command is now public with improved help.
+  (Ian Clatworthy)
+
+* New bundle and merge directive formats are faster to generate, and
+
+* Annotate merge now works when there are local changes. (Aaron Bentley)
+
+* Commit now only shows the progress in terms of directories instead of
+  entries. (Ian Clatworthy)
+
+* Fix ``KnitRepository.get_revision_graph`` to not request the graph 2
+  times. This makes ``get_revision_graph`` 2x faster. (John Arbash
+  Meinel)
+
+* Fix ``VersionedFile.get_graph()`` to avoid using
+  ``set.difference_update(other)``, which has bad scaling when
+  ``other`` is large. This improves ``VF.get_graph([version_id])`` for
+  a 12.5k graph from 2.9s down to 200ms. (John Arbash Meinel)
+
+* The ``--lsprof-file`` option now generates output for KCacheGrind if
+  the file starts with ``callgrind.out``. This matches the default file
+  filtering done by KCacheGrind's Open Dialog. (Ian Clatworthy)
+
+* Fix ``bzr update`` to avoid an unnecessary
+  ``branch.get_master_branch`` call, which avoids 1 extra connection
+  to the remote server. (Partial fix for #128076, John Arbash Meinel)
+
+* Log errors from the smart server in the trace file, to make debugging
+  test failures (and live failures!) easier.  (Andrew Bennetts)
+
+* The HTML version of the man page has been superceded by a more
+  comprehensive manual called the Bazaar User Reference. This manual
+  is completed generated from the online help topics. As part of this
+  change, limited reStructuredText is now explicitly supported in help
+  topics and command help with 'unnatural' markup being removed prior
+  to display by the online help or inclusion in the man page.
+  (Ian Clatworthy)
+
+* HTML documentation now use files extension ``*.html``
+  (Alexander Belchenko)
+
+* The cache of ignore definitions is now cleared in WorkingTree.unlock()
+  so that changes to .bzrignore aren't missed. (#129694, Daniel Watkins)
+
+* ``bzr selftest --strict`` fails if there are any missing features or
+  expected test failures. (Daniel Watkins, #111914)
+
+* Link to registration survey added to README. (Ian Clatworthy)
+
+* Windows standalone installer show link to registration survey
+  when installation finished. (Alexander Belchenko)
+
+Library API Breaks
+******************
+
+* Deprecated dictionary ``bzrlib.option.SHORT_OPTIONS`` removed.
+  Options are now required to provide a help string and it must
+  comply with the style guide by being one or more sentences with an
+  initial capital and final period. (Martin Pool)
+
+* KnitIndex.get_parents now returns tuples. (Robert Collins)
+
+* Ancient unused ``Repository.text_store`` attribute has been removed.
+  (Robert Collins)
+
+* The ``bzrlib.pack`` interface has changed to use tuples of bytestrings
+  rather than just bytestrings, making it easier to represent multiple
+  element names. As this interface was not used by any internal facilities
+  since it was introduced in 0.18 no API compatibility is being preserved.
+  The serialised form of these packs is identical with 0.18 when a single
+  element tuple is in use. (Robert Collins)
+
+Internals
+*********
+
+* merge now uses ``iter_changes`` to calculate changes, which makes room for
+  future performance increases.  It is also more consistent with other
+  operations that perform comparisons, and reduces reliance on
+  Tree.inventory.  (Aaron Bentley)
+
+* Refactoring of transport classes connected to a remote server.
+  ConnectedTransport is a new class that serves as a basis for all
+  transports needing to connect to a remote server.  transport.split_url
+  have been deprecated, use the static method on the object instead. URL
+  tests have been refactored too.
+  (Vincent Ladeuil)
+
+* Better connection sharing for ConnectedTransport objects.
+  transport.get_transport() now accepts a 'possible_transports' parameter.
+  If a newly requested transport can share a connection with one of the
+  list, it will.
+  (Vincent Ladeuil)
+
+* Most functions now accept ``bzrlib.revision.NULL_REVISION`` to indicate
+  the null revision, and consider using ``None`` for this purpose
+  deprecated.  (Aaron Bentley)
+
+* New ``index`` module with abstract index functionality. This will be
+  used during the planned changes in the repository layer. Currently the
+  index layer provides a graph aware immutable index, a builder for the
+  same index type to allow creating them, and finally a composer for
+  such indices to allow the use of many indices in a single query. The
+  index performance is not optimised, however the API is stable to allow
+  development on top of the index. (Robert Collins)
+
+* ``bzrlib.dirstate.cmp_by_dirs`` can be used to compare two paths by
+  their directory sections. This is equivalent to comparing
+  ``path.split('/')``, only without having to split the paths.
+  This has a Pyrex implementation available.
+  (John Arbash Meinel)
+
+* New transport decorator 'unlistable+' which disables the list_dir
+  functionality for testing.
+
+* Deprecated ``change_entry`` in transform.py. (Ian Clatworthy)
+
+* RevisionTree.get_weave is now deprecated.  Tree.plan_merge is now used
+  for performing annotate-merge.  (Aaron Bentley)
+
+* New EmailMessage class to create email messages. (Adeodato Simó)
+
+* Unused functions on the private interface KnitIndex have been removed.
+  (Robert Collins)
+
+* New ``knit.KnitGraphIndex`` which provides a ``KnitIndex`` layered on top
+  of a ``index.GraphIndex``. (Robert Collins)
+
+* New ``knit.KnitVersionedFile.iter_parents`` method that allows querying
+  the parents of many knit nodes at once, reducing round trips to the
+  underlying index. (Robert Collins)
+
+* Graph now has an is_ancestor method, various bits use it.
+  (Aaron Bentley)
+
+* The ``-Dhpss`` flag now includes timing information. As well as
+  logging when a new connection is opened. (John Arbash Meinel)
+
+* ``bzrlib.pack.ContainerWriter`` now returns an offset, length tuple to
+  callers when inserting data, allowing generation of readv style access
+  during pack creation, without needing a separate pass across the output
+  pack to gather such details. (Robert Collins)
+
+* ``bzrlib.pack.make_readv_reader`` allows readv based access to pack
+  files that are stored on a transport. (Robert Collins)
+
+* New ``Repository.has_same_location`` method that reports if two
+  repository objects refer to the same repository (although with some risk
+  of false negatives).  (Andrew Bennetts)
+
+* InterTree.compare now passes require_versioned on correctly.
+  (Marius Kruger)
+
+* New methods on Repository - ``start_write_group``,
+  ``commit_write_group``, ``abort_write_group`` and ``is_in_write_group`` -
+  which provide a clean hook point for transactional Repositories - ones
+  where all the data for a fetch or commit needs to be made atomically
+  available in one step. This allows the write lock to remain while making
+  a series of data insertions.  (e.g. data conversion). (Robert Collins)
+
+* In ``bzrlib.knit`` the internal interface has been altered to use
+  3-tuples (index, pos, length) rather than two-tuples (pos, length) to
+  describe where data in a knit is, allowing knits to be split into
+  many files. (Robert Collins)
+
+* ``bzrlib.knit._KnitData`` split into cache management and physical access
+  with two access classes - ``_PackAccess`` and ``_KnitAccess`` defined.
+  The former provides access into a .pack file, and the latter provides the
+  current production repository form of .knit files. (Robert Collins)
+
+Testing
+*******
+
+* Remove selftest ``--clean-output``, ``--numbered-dirs`` and
+  ``--keep-output`` options, which are obsolete now that tests
+  are done within directories in $TMPDIR.  (Martin Pool)
+
+* The SSH_AUTH_SOCK environment variable is now reset to avoid
+  interaction with any running SSH agents.  (Jelmer Vernooij, #125955)
+
+* run_bzr_subprocess handles parameters the same way as run_bzr:
+  either a string or a list of strings should be passed as the first
+  parameter.  Varargs-style parameters are deprecated. (Aaron Bentley)
+
+
+..
+   vim: tw=74 ft=rst ff=unix
+

=== added file 'doc/en/release-notes/bzr-0.91.txt'
--- doc/en/release-notes/bzr-0.91.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.91.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,372 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.91
+########
+
+:Released: 2007-09-26
+
+Bug Fixes
+*********
+
+* Print a warning instead of aborting the ``python setup.py install``
+  process if building of a C extension is not possible.
+  (Lukáš Lalinský, Alexander Belchenko)
+
+* Fix commit ordering in corner case (Aaron Bentley, #94975)
+
+* Fix ''bzr info bzr://host/'' and other operations on ''bzr://' URLs with
+  an implicit port.  We were incorrectly raising PathNotChild due to
+  inconsistent treatment of the ''_port'' attribute on the Transport object.
+  (Andrew Bennetts, #133965)
+
+* Make RemoteRepository.sprout cope gracefully with servers that don't
+  support the ``Repository.tarball`` request.
+  (Andrew Bennetts)
+
+
+bzr 0.91rc2
+###########
+
+:Released: 2007-09-11
+
+* Replaced incorrect tarball for previous release; a debug statement was left
+  in bzrlib/remote.py.
+
+
+bzr 0.91rc1
+###########
+
+:Released: 2007-09-11
+
+Changes
+*******
+
+* The default branch and repository format has changed to
+  ``dirstate-tags``, so tag commands are active by default.
+  This format is compatible with Bazaar 0.15 and later.
+  This incidentally fixes bug #126141.
+  (Martin Pool)
+
+* ``--quiet`` or ``-q`` is no longer a global option. If present, it
+  must now appear after the command name. Scripts doing things like
+  ``bzr -q missing`` need to be rewritten as ``bzr missing -q``.
+  (Ian Clatworthy)
+
+Features
+********
+
+* New option ``--author`` in ``bzr commit`` to specify the author of the
+  change, if it's different from the committer. ``bzr log`` and
+  ``bzr annotate`` display the author instead of the committer.
+  (Lukáš Lalinský)
+
+* In addition to global options and command specific options, a set of
+  standard options are now supported. Standard options are legal for
+  all commands. The initial set of standard options are:
+
+  * ``--help`` or ``-h`` - display help message
+  * ``--verbose`` or ``-v`` - display additional information
+  * ``--quiet``  or ``-q`` - only output warnings and errors.
+
+  Unlike global options, standard options can be used in aliases and
+  may have command-specific help. (Ian Clatworthy)
+
+* Verbosity level processing has now been unified. If ``--verbose``
+  or ``-v`` is specified on the command line multiple times, the
+  verbosity level is made positive the first time then increased.
+  If ``--quiet`` or ``-q`` is specified on the command line
+  multiple times, the verbosity level is made negative the first
+  time then decreased. To get the default verbosity level of zero,
+  either specify none of the above , ``--no-verbose`` or ``--no-quiet``.
+  Note that most commands currently ignore the magnitude of the
+  verbosity level but do respect *quiet vs normal vs verbose* when
+  generating output. (Ian Clatworthy)
+
+* ``Branch.hooks`` now supports ``pre_commit`` hook. The hook's signature
+  is documented in BranchHooks constructor. (Nam T. Nguyen, #102747)
+
+* New ``Repository.stream_knit_data_for_revisions`` request added to the
+  network protocol for greatly reduced roundtrips when retrieving a set of
+  revisions. (Andrew Bennetts)
+
+Bug Fixes
+*********
+
+* ``bzr plugins`` now lists the version number for each plugin in square
+  brackets after the path. (Robert Collins, #125421)
+
+* Pushing, pulling and branching branches with subtree references was not
+  copying the subtree weave, preventing the file graph from being accessed
+  and causing errors in commits in clones. (Robert Collins)
+
+* Suppress warning "integer argument expected, got float" from Paramiko,
+  which sometimes caused false test failures.  (Martin Pool)
+
+* Fix bug in bundle 4 that could cause attempts to write data to wrong
+  versionedfile.  (Aaron Bentley)
+
+* Diffs generated using "diff -p" no longer break the patch parser.
+  (Aaron Bentley)
+
+* get_transport treats an empty possible_transports list the same as a non-
+  empty one.  (Aaron Bentley)
+
+* patch verification for merge directives is reactivated, and works with
+  CRLF and CR files.  (Aaron Bentley)
+
+* Accept ..\ as a path in revision specifiers. This fixes for example
+  "-r branch:..\other-branch" on Windows.  (Lukáš Lalinský)
+
+* ``BZR_PLUGIN_PATH`` may now contain trailing slashes.
+  (Blake Winton, #129299)
+
+* man page no longer lists hidden options (#131667, Aaron Bentley)
+
+* ``uncommit --help`` now explains the -r option adequately.  (Daniel
+  Watkins, #106726)
+
+* Error messages are now better formatted with parameters (such as
+  filenames) quoted when necessary. This avoids confusion when directory
+  names ending in a '.' at the end of messages were confused with a
+  full stop that may or not have been there. (Daniel Watkins, #129791)
+
+* Fix ``status FILE -r X..Y``. (Lukáš Lalinský)
+
+* If a particular command is an alias, ``help`` will show the alias
+  instead of claiming there is no help for said alias. (Daniel Watkins,
+  #133548)
+
+* TreeTransform-based operations, like pull, merge, revert, and branch,
+  now roll back if they encounter an error.  (Aaron Bentley, #67699)
+
+* ``bzr commit`` now exits cleanly if a character unsupported by the
+  current encoding is used in the commit message.  (Daniel Watkins,
+  #116143)
+
+* bzr send uses default values for ranges when only half of an elipsis
+  is specified ("-r..5" or "-r5..").  (#61685, Aaron Bentley)
+
+* Avoid trouble when Windows SSH calls itself 'plink' but no plink
+  binary is present.  (Martin Albisetti, #107155)
+
+* ``bzr remove`` should remove clean subtrees.  Now it will remove (without
+  needing ``--force``) subtrees that contain no files with text changes or
+  modified files.  With ``--force`` it removes the subtree regardless of
+  text changes or unknown files. Directories with renames in or out (but
+  not changed otherwise) will now be removed without needing ``--force``.
+  Unknown ignored files will be deleted without needing ``--force``.
+  (Marius Kruger, #111665)
+
+* When two plugins conflict, the source of both the losing and now the
+  winning definition is shown.  (Konstantin Mikhaylov, #5454)
+
+* When committing to a branch, the location being committed to is
+  displayed.  (Daniel Watkins, #52479)
+
+* ``bzr --version`` takes care about encoding of stdout, especially
+  when output is redirected. (Alexander Belchenko, #131100)
+
+* Prompt for an FTP password if none is provided.
+  (Vincent Ladeuil, #137044)
+
+* Reuse bound branch associated transport to avoid multiple
+  connections.
+  (Vincent Ladeuil, #128076, #131396)
+
+* Overwrite conflicting tags by ``push`` and ``pull`` if the
+  ``--overwrite`` option is specified.  (Lukáš Lalinský, #93947)
+
+* In checkouts, tags are copied into the master branch when created,
+  changed or deleted, and are copied into the checkout when it is
+  updated.  (Martin Pool, #93856, #93860)
+
+* Print a warning instead of aborting the ``python setup.py install``
+  process if building of a C extension is not possible.
+  (Lukáš Lalinský, Alexander Belchenko)
+
+Improvements
+************
+
+* Add the option "--show-diff" to the commit command in order to display
+  the diff during the commit log creation. (Goffredo Baroncelli)
+
+* ``pull`` and ``merge`` are much faster at installing bundle format 4.
+  (Aaron Bentley)
+
+* ``pull -v`` no longer includes deltas, making it much faster.
+  (Aaron Bentley)
+
+* ``send`` now sends the directive as an attachment by default.
+  (Aaron Bentley, Lukáš Lalinský, Alexander Belchenko)
+
+* Documentation updates (Martin Albisetti)
+
+* Help on debug flags is now included in ``help global-options``.
+  (Daniel Watkins, #124853)
+
+* Parameters passed on the command line are checked to ensure they are
+  supported by the encoding in use. (Daniel Watkins)
+
+* The compression used within the bzr repository has changed from zlib
+  level 9 to the zlib default level. This improves commit performance with
+  only a small increase in space used (and in some cases a reduction in
+  space). (Robert Collins)
+
+* Initial commit no longer SHAs files twice and now reuses the path
+  rather than looking it up again, making it faster.
+  (Ian Clatworthy)
+
+* New option ``-c``/``--change`` for ``diff`` and ``status`` to show
+  changes in one revision.  (Lukáš Lalinský)
+
+* If versioned files match a given ignore pattern, a warning is now
+  given. (Daniel Watkins, #48623)
+
+* ``bzr status`` now has -S as a short name for --short and -V as a
+  short name for --versioned. These have been added to assist users
+  migrating from Subversion: ``bzr status -SV`` is now like
+  ``svn status -q``.  (Daniel Watkins, #115990)
+
+* Added C implementation of  ``PatienceSequenceMatcher``, which is about
+  10x faster than the Python version. This speeds up commands that
+  need file diffing, such as ``bzr commit`` or ``bzr diff``.
+  (Lukáš Lalinský)
+
+* HACKING has been extended with a large section on core developer tasks.
+  (Ian Clatworthy)
+
+* Add ``branches`` and ``standalone-trees`` as online help topics and
+  include them as Concepts within the User Reference.
+  (Paul Moore, Ian Clatworthy)
+
+* ``check`` can detect versionedfile parent references that are
+  inconsistent with revision and inventory info, and ``reconcile`` can fix
+  them.  These faulty references were generated by 0.8-era releases,
+  so repositories which were manipulated by old bzrs should be
+  checked, and possibly reconciled ASAP.  (Aaron Bentley, Andrew Bennetts)
+
+API Breaks
+**********
+
+* ``Branch.append_revision`` is removed altogether; please use
+  ``Branch.set_last_revision_info`` instead.  (Martin Pool)
+
+* CommitBuilder now advertises itself as requiring the root entry to be
+  supplied. This only affects foreign repository implementations which reuse
+  CommitBuilder directly and have changed record_entry_contents to require
+  that the root not be supplied. This should be precisely zero plugins
+  affected. (Robert Collins)
+
+* The ``add_lines`` methods on ``VersionedFile`` implementations has changed
+  its return value to include the sha1 and length of the inserted text. This
+  allows the avoidance of double-sha1 calculations during commit.
+  (Robert Collins)
+
+* ``Transport.should_cache`` has been removed.  It was not called in the
+  previous release.  (Martin Pool)
+
+Testing
+*******
+
+* Tests may now raise TestNotApplicable to indicate they shouldn't be
+  run in a particular scenario.  (Martin Pool)
+
+* New function multiply_tests_from_modules to give a simpler interface
+  to test parameterization.  (Martin Pool, Robert Collins)
+
+* ``Transport.should_cache`` has been removed.  It was not called in the
+  previous release.  (Martin Pool)
+
+* NULL_REVISION is returned to indicate the null revision, not None.
+  (Aaron Bentley)
+
+* Use UTF-8 encoded StringIO for log tests to avoid failures on
+  non-ASCII committer names.  (Lukáš Lalinský)
+
+Internals
+*********
+
+* ``bzrlib.plugin.all_plugins`` has been deprecated in favour of
+  ``bzrlib.plugin.plugins()`` which returns PlugIn objects that provide
+  useful functionality for determining the path of a plugin, its tests, and
+  its version information. (Robert Collins)
+
+* Add the option user_encoding to the function 'show_diff_trees()'
+  in order to move the user encoding at the UI level. (Goffredo Baroncelli)
+
+* Add the function make_commit_message_template_encoded() and the function
+  edit_commit_message_encoded() which handle encoded strings.
+  This is done in order to mix the commit messages (which is a unicode
+  string), and the diff which is a raw string. (Goffredo Baroncelli)
+
+* CommitBuilder now defaults to using add_lines_with_ghosts, reducing
+  overhead on non-weave repositories which don't require all parents to be
+  present. (Robert Collins)
+
+* Deprecated method ``find_previous_heads`` on
+  ``bzrlib.inventory.InventoryEntry``. This has been superseded by the use
+  of ``parent_candidates`` and a separate heads check via the repository
+  API. (Robert Collins)
+
+* New trace function ``mutter_callsite`` will print out a subset of the
+  stack to the log, which can be useful for gathering debug details.
+  (Robert Collins)
+
+* ``bzrlib.pack.ContainerWriter`` now tracks how many records have been
+  added via a public attribute records_written. (Robert Collins)
+
+* New method ``bzrlib.transport.Transport.get_recommended_page_size``.
+  This provides a hint to users of transports as to the reasonable
+  minimum data to read. In principle this can take latency and
+  bandwidth into account on a per-connection basis, but for now it
+  just has hard coded values based on the URL. (E.g., http:// has a large
+  page size, file:// has a small one.) (Robert Collins)
+
+* New method on ``bzrlib.transport.Transport`` ``open_write_stream`` allows
+  incremental addition of data to a file without requiring that all the
+  data be buffered in memory. (Robert Collins)
+
+* New methods on ``bzrlib.knit.KnitVersionedFile``:
+  ``get_data_stream(versions)``, ``insert_data_stream(stream)`` and
+  ``get_format_signature()``.  These provide some infrastructure for
+  efficiently streaming the knit data for a set of versions over the smart
+  protocol.
+
+* Knits with no annotation cache still produce correct annotations.
+  (Aaron Bentley)
+
+* Three new methods have been added to ``bzrlib.trace``:
+  ``set_verbosity_level``, ``get_verbosity_level`` and ``is_verbose``.
+  ``set_verbosity_level`` expects a numeric value: negative for quiet,
+  zero for normal, positive for verbose. The size of the number can be
+  used to determine just how quiet or verbose the application should be.
+  The existing ``be_quiet`` and ``is_quiet`` routines have been
+  integrated into this new scheme. (Ian Clatworthy)
+
+* Options can now be delcared with a ``custom_callback`` parameter. If
+  set, this routine is called after the option is processed. This feature
+  is now used by the standard options ``verbose`` and ``quiet`` so that
+  setting one implicitly resets the other. (Ian Clatworthy)
+
+* Rather than declaring a new option from scratch in order to provide
+  custom help, a centrally registered option can be decorated using the
+  new ``bzrlib.Option.custom_help`` routine. In particular, this routine
+  is useful when declaring better help for the ``verbose`` and ``quiet``
+  standard options as the base definition of these is now more complex
+  than before thanks to their use of a custom callback. (Ian Clatworthy)
+
+* Tree._iter_changes(specific_file=[]) now iterates through no files,
+  instead of iterating through all files.  None is used to iterate through
+  all files.  (Aaron Bentley)
+
+* WorkingTree.revert() now accepts None to revert all files.  The use of
+  [] to revert all files is deprecated.  (Aaron Bentley)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-0.92.txt'
--- doc/en/release-notes/bzr-0.92.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-0.92.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,342 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 0.92
+########
+
+:Released: 2007-11-05
+
+Changes
+*******
+
+  * New uninstaller on Win32.  (Alexander Belchenko)
+
+
+bzr 0.92rc1
+###########
+
+:Released: 2007-10-29
+
+Changes
+*******
+
+* ``bzr`` now returns exit code 4 if an internal error occurred, and
+  3 if a normal error occurred.  (Martin Pool)
+
+* ``pull``, ``merge`` and ``push`` will no longer silently correct some
+  repository index errors that occured as a result of the Weave disk format.
+  Instead the ``reconcile`` command needs to be run to correct those
+  problems if they exist (and it has been able to fix most such problems
+  since bzr 0.8). Some new problems have been identified during this release
+  and you should run ``bzr check`` once on every repository to see if you
+  need to reconcile. If you cannot ``pull`` or ``merge`` from a remote
+  repository due to mismatched parent errors - a symptom of index errors -
+  you should simply take a full copy of that remote repository to a clean
+  directory outside any local repositories, then run reconcile on it, and
+  finally pull from it locally. (And naturally email the repositories owner
+  to ask them to upgrade and run reconcile).
+  (Robert Collins)
+
+Features
+********
+
+* New ``knitpack-experimental`` repository format. This is interoperable with
+  the ``dirstate-tags`` format but uses a smarter storage design that greatly
+  speeds up many operations, both local and remote. This new format can be
+  used as an option to the ``init``, ``init-repository`` and ``upgrade``
+  commands. (Robert Collins)
+
+* For users of bzr-svn (and those testing the prototype subtree support) that
+  wish to try packs, a new ``knitpack-subtree-experimental`` format has also
+  been added. This is interoperable with the ``dirstate-subtrees`` format.
+  (Robert Collins)
+
+* New ``reconfigure`` command. (Aaron Bentley)
+
+* New ``revert --forget-merges`` command, which removes the record of a pending
+  merge without affecting the working tree contents.  (Martin Pool)
+
+* New ``bzr_remote_path`` configuration variable allows finer control of
+  remote bzr locations than BZR_REMOTE_PATH environment variable.
+  (Aaron Bentley)
+
+* New ``launchpad-login`` command to tell Bazaar your Launchpad
+  user ID.  This can then be used by other functions of the
+  Launchpad plugin. (James Henstridge)
+
+Performance
+***********
+
+* Commit in quiet mode is now slightly faster as the information to
+  output is no longer calculated. (Ian Clatworthy)
+
+* Commit no longer checks for new text keys during insertion when the
+  revision id was deterministically unique. (Robert Collins)
+
+* Committing a change which is not a merge and does not change the number of
+  files in the tree is faster by utilising the data about whether files are
+  changed to determine if the tree is unchanged rather than recalculating
+  it at the end of the commit process. (Robert Collins)
+
+* Inventory serialisation no longer double-sha's the content.
+  (Robert Collins)
+
+* Knit text reconstruction now avoids making copies of the lines list for
+  interim texts when building a single text. The new ``apply_delta`` method
+  on ``KnitContent`` aids this by allowing modification of the revision id
+  such objects represent. (Robert Collins)
+
+* Pack indices are now partially parsed for specific key lookup using a
+  bisection approach. (Robert Collins)
+
+* Partial commits are now approximately 40% faster by walking over the
+  unselected current tree more efficiently. (Robert Collins)
+
+* XML inventory serialisation takes 20% less time while being stricter about
+  the contents. (Robert Collins)
+
+* Graph ``heads()`` queries have been fixed to no longer access all history
+  unnecessarily. (Robert Collins)
+
+Improvements
+************
+
+* ``bzr+https://`` smart server across https now supported.
+  (John Ferlito, Martin Pool, #128456)
+
+* Mutt is now a supported mail client; set ``mail_client=mutt`` in your
+  bazaar.conf and ``send`` will use mutt. (Keir Mierle)
+
+* New option ``-c``/``--change`` for ``merge`` command for cherrypicking
+  changes from one revision. (Alexander Belchenko, #141368)
+
+* Show encodings, locale and list of plugins in the traceback message.
+  (Martin Pool, #63894)
+
+* Experimental directory formats can now be marked with
+  ``experimental = True`` during registration. (Ian Clatworthy)
+
+Documentation
+*************
+
+* New *Bazaar in Five Minutes* guide.  (Matthew Revell)
+
+* The hooks reference documentation is now converted to html as expected.
+  (Ian Clatworthy)
+
+Bug Fixes
+*********
+
+* Connection error reporting for the smart server has been fixed to
+  display a user friendly message instead of a traceback.
+  (Ian Clatworthy, #115601)
+
+* Make sure to use ``O_BINARY`` when opening files to check their
+  sha1sum. (Alexander Belchenko, John Arbash Meinel, #153493)
+
+* Fix a problem with Win32 handling of the executable bit.
+  (John Arbash Meinel, #149113)
+
+* ``bzr+ssh://`` and ``sftp://`` URLs that do not specify ports explicitly
+  no longer assume that means port 22.  This allows people using OpenSSH to
+  override the default port in their ``~/.ssh/config`` if they wish.  This
+  fixes a bug introduced in bzr 0.91.  (Andrew Bennetts, #146715)
+
+* Commands reporting exceptions can now be profiled and still have their
+  data correctly dumped to a file. For example, a ``bzr commit`` with
+  no changes still reports the operation as pointless but doing so no
+  longer throws away the profiling data if this command is run with
+  ``--lsprof-file callgrind.out.ci`` say. (Ian Clatworthy)
+
+* Fallback to FTP when paramiko is not installed and SFTP can't be used for
+  ``tests/commands`` so that the test suite is still usable without
+  paramiko.
+  (Vincent Ladeuil, #59150)
+
+* Fix commit ordering in corner case. (Aaron Bentley, #94975)
+
+* Fix long standing bug in partial commit when there are renames
+  left in tree. (Robert Collins, #140419)
+
+* Fix selftest semi-random noise during HTTP related tests.
+  (Vincent Ladeuil, #140614)
+
+* Fix typo in ftp.py making the reconnection fail on temporary errors.
+  (Vincent Ladeuil, #154259)
+
+* Fix failing test by comparing real paths to cover the case where the TMPDIR
+  contains a symbolic link.
+  (Vincent Ladeuil, #141382).
+
+* Fix log against smart server branches that don't support tags.
+  (James Westby, #140615)
+
+* Fix pycurl HTTP implementation by defining error codes from
+  pycurl instead of relying on an old curl definition.
+  (Vincent Ladeuil, #147530)
+
+* Fix 'unprintable error' message when displaying BzrCheckError and
+  some other exceptions on Python 2.5.
+  (Martin Pool, #144633)
+
+* Fix ``Inventory.copy()`` and add test for it. (Jelmer Vernooij)
+
+* Handles default value for ListOption in cmd_commit.
+  (Vincent Ladeuil, #140432)
+
+* HttpServer and FtpServer need to be closed properly or a listening socket
+  will remain opened.
+  (Vincent Ladeuil, #140055)
+
+* Monitor the .bzr directory created in the top level test
+  directory to detect leaking tests.
+  (Vincent Ladeuil, #147986)
+
+* The basename, not the full path, is now used when checking whether
+  the profiling dump file begins with ``callgrind.out`` or not. This
+  fixes a bug reported by Aaron Bentley on IRC. (Ian Clatworthy)
+
+* Trivial fix for invoking command ``reconfigure`` without arguments.
+  (Rob Weir, #141629)
+
+* ``WorkingTree.rename_one`` will now raise an error if normalisation of the
+  new path causes bzr to be unable to access the file. (Robert Collins)
+
+* Correctly detect a NoSuchFile when using a filezilla server. (Gary van der
+  Merwe)
+
+API Breaks
+**********
+
+* ``bzrlib.index.GraphIndex`` now requires a size parameter to the
+  constructor, for enabling bisection searches. (Robert Collins)
+
+* ``CommitBuilder.record_entry_contents`` now requires the root entry of a
+  tree be supplied to it, previously failing to do so would trigger a
+  deprecation warning. (Robert Collins)
+
+* ``KnitVersionedFile.add*`` will no longer cache added records even when
+  enable_cache() has been called - the caching feature is now exclusively for
+  reading existing data. (Robert Collins)
+
+* ``ReadOnlyLockError`` is deprecated; ``LockFailed`` is usually more
+  appropriate.  (Martin Pool)
+
+* Removed ``bzrlib.transport.TransportLogger`` - please see the new
+  ``trace+`` transport instead. (Robert Collins)
+
+* Removed previously deprecated varargs interface to ``TestCase.run_bzr`` and
+  deprecated methods ``TestCase.capture`` and ``TestCase.run_bzr_captured``.
+  (Martin Pool)
+
+* Removed previous deprecated ``basis_knit`` parameter to the
+  ``KnitVersionedFile`` constructor. (Robert Collins)
+
+* Special purpose method ``TestCase.run_bzr_decode`` is moved to the test_non_ascii
+  class that needs it.
+  (Martin Pool)
+
+* The class ``bzrlib.repofmt.knitrepo.KnitRepository3`` has been folded into
+  ``KnitRepository`` by parameters to the constructor. (Robert Collins)
+
+* The ``VersionedFile`` interface now allows content checks to be bypassed
+  by supplying check_content=False.  This saves nearly 30% of the minimum
+  cost to store a version of a file. (Robert Collins)
+
+* Tree's with bad state such as files with no length or sha will no longer
+  be silently accepted by the repository XML serialiser. To serialise
+  inventories without such data, pass working=True to write_inventory.
+  (Robert Collins)
+
+* ``VersionedFile.fix_parents`` has been removed as a harmful API.
+  ``VersionedFile.join`` will no longer accept different parents on either
+  side of a join - it will either ignore them, or error, depending on the
+  implementation. See notes when upgrading for more information.
+  (Robert Collins)
+
+Internals
+*********
+
+* ``bzrlib.transport.Transport.put_file`` now returns the number of bytes
+  put by the method call, to allow avoiding stat-after-write or
+  housekeeping in callers. (Robert Collins)
+
+* ``bzrlib.xml_serializer.Serializer`` is now responsible for checking that
+  mandatory attributes are present on serialisation and deserialisation.
+  This fixes some holes in API usage and allows better separation between
+  physical storage and object serialisation. (Robert Collins)
+
+* New class ``bzrlib.errors.InternalBzrError`` which is just a convenient
+  shorthand for deriving from BzrError and setting internal_error = True.
+  (Robert Collins)
+
+* New method ``bzrlib.mutabletree.update_to_one_parent_via_delta`` for
+  moving the state of a parent tree to a new version via a delta rather than
+  a complete replacement tree. (Robert Collins)
+
+* New method ``bzrlib.osutils.minimum_path_selection`` useful for removing
+  duplication from user input, when a user mentions both a path and an item
+  contained within that path. (Robert Collins)
+
+* New method ``bzrlib.repository.Repository.is_write_locked`` useful for
+  determining if a repository is write locked. (Robert Collins)
+
+* New method on ``bzrlib.tree.Tree`` ``path_content_summary`` provides a
+  tuple containing the key information about a path for commit processing
+  to complete. (Robert Collins)
+
+* New method on XML serialisers, write_inventory_to_lines, which matches the
+  API used by knits for adding content. (Robert Collins)
+
+* New module ``bzrlib.bisect_multi`` with generic multiple-bisection-at-once
+  logic, currently only available for byte-based lookup
+  (``bisect_multi_bytes``). (Robert Collins)
+
+* New helper ``bzrlib.tuned_gzip.bytes_to_gzip`` which takes a byte string
+  and returns a gzipped version of the same. This is used to avoid a bunch
+  of api friction during adding of knit hunks. (Robert Collins)
+
+* New parameter on ``bzrlib.transport.Transport.readv``
+  ``adjust_for_latency`` which changes readv from returning strictly the
+  requested data to inserted return larger ranges and in forward read order
+  to reduce the effect of network latency. (Robert Collins)
+
+* New parameter yield_parents on ``Inventory.iter_entries_by_dir`` which
+  causes the parents of a selected id to be returned recursively, so all the
+  paths from the root down to each element of selected_file_ids are
+  returned. (Robert Collins)
+
+* Knit joining has been enhanced to support plain to annotated conversion
+  and annotated to plain conversion. (Ian Clatworthy)
+
+* The CommitBuilder method ``record_entry_contents`` now returns summary
+  information about the effect of the commit on the repository. This tuple
+  contains an inventory delta item if the entry changed from the basis, and a
+  boolean indicating whether a new file graph node was recorded.
+  (Robert Collins)
+
+* The python path used in the Makefile can now be overridden.
+  (Andrew Bennetts, Ian Clatworthy)
+
+Testing
+*******
+
+* New transport implementation ``trace+`` which is useful for testing,
+  logging activity taken to its _activity attribute. (Robert Collins)
+
+* When running bzr commands within the test suite, internal exceptions are
+  not caught and reported in the usual way, but rather allowed to propagate
+  up and be visible to the test suite.  A new API ``run_bzr_catch_user_errors``
+  makes this behavior available to other users.
+  (Martin Pool)
+
+* New method ``TestCase.call_catch_warnings`` for testing methods that
+  raises a Python warning.  (Martin Pool)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.0.txt'
--- doc/en/release-notes/bzr-1.0.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.0.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,429 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.0
+#######
+
+:Released: 2007-12-14
+
+Documentation
+*************
+
+* More improvements and fixes to the User Guide.  (Ian Clatworthy)
+
+* Add information on cherrypicking/rebasing to the User Guide.
+  (Ian Clatworthy)
+
+* Improve bug tracker integration documentation. (Ian Clatworthy)
+
+* Minor edits to ``Bazaar in five minutes`` from David Roberts and
+  to the rebasing section of the User Guide from Aaron Bentley.
+  (Ian Clatworthy)
+
+
+bzr 1.0rc3
+##########
+
+:Released: 2007-12-11
+
+Changes
+*******
+
+* If a traceback occurs, users are now asked to report the bug
+  through Launchpad (https://bugs.launchpad.net/bzr/), rather than
+  by mail to the mailing list.
+  (Martin Pool)
+
+Bugfixes
+********
+
+* Fix Makefile rules for doc generation. (Ian Clatworthy, #175207)
+
+* Give more feedback during long HTTP downloads by making readv deliver data
+  as it arrives for urllib, and issue more requests for pycurl. High latency
+  networks are better handled by urllib, the pycurl implementation give more
+  feedback but also incur more latency.
+  (Vincent Ladeuil, #173010)
+
+* Implement _make_parents_provider on RemoteRepository, allowing generating
+  bundles against branches on a smart server.  (Andrew Bennetts, #147836)
+
+Documentation
+*************
+
+* Improved user guide.  (Ian Clatworthy)
+
+* The single-page quick reference guide is now available as a PDF.
+  (Ian Clatworthy)
+
+Internals
+*********
+
+* readv urllib HTTP implementation is now a real iterator above the
+  underlying socket and deliver data as soon as it arrives. 'get' still
+  wraps its output in a StringIO.
+  (Vincent Ladeuil)
+
+
+bzr 1.0rc2
+##########
+
+:Released: 2007-12-07
+
+Improvements
+************
+
+* Added a --coverage option to selftest. (Andrew Bennetts)
+
+* Annotate merge (merge-type=weave) now supports cherrypicking.
+  (Aaron Bentley)
+
+* ``bzr commit`` now doesn't print the revision number twice. (Matt
+  Nordhoff, #172612)
+
+* New configuration option ``bugtracker_<tracker_abbrevation>_url`` to
+  define locations of bug trackers that are not directly supported by
+  bzr or a plugin. The URL will be treated as a template and ``{id}``
+  placeholders will be replaced by specific bug IDs.  (Lukáš Lalinský)
+
+* Support logging single merge revisions with short and line log formatters.
+  (Kent Gibson)
+
+* User Guide enhanced with suggested readability improvements from
+  Matt Revell and corrections from John Arbash Meinel. (Ian Clatworthy)
+
+* Quick Start Guide renamed to Quick Start Card, moved down in
+  the catalog, provided in pdf and png format and updated to refer
+  to ``send`` instead of ``bundle``. (Ian Clatworthy, #165080)
+
+* ``switch`` can now be used on heavyweight checkouts as well as
+  lightweight ones. After switching a heavyweight checkout, the
+  local branch is a mirror/cache of the new bound branch and
+  uncommitted changes in the working tree are merged. As a safety
+  check, if there are local commits in a checkout which have not
+  been committed to the previously bound branch, then ``switch``
+  fails unless the ``--force`` option is given. This option is
+  now also required if the branch a lightweight checkout is pointing
+  to has been moved. (Ian Clatworthy)
+
+Internals
+*********
+
+* New -Dhttp debug option reports HTTP connections, requests and responses.
+  (Vincent Ladeuil)
+
+* New -Dmerge debug option, which emits merge plans for merge-type=weave.
+
+Bugfixes
+********
+
+* Better error message when running ``bzr cat`` on a non-existant branch.
+  (Lukáš Lalinský, #133782)
+
+* Catch OSError 17 (file exists) in final phase of tree transform and show
+  filename to user.
+  (Alexander Belchenko, #111758)
+
+* Catch ShortReadvErrors while using pycurl. Also make readv more robust by
+  allowing multiple GET requests to be issued if too many ranges are
+  required.
+  (Vincent Ladeuil, #172701)
+
+* Check for missing basis texts when fetching from packs to packs.
+  (John Arbash Meinel, #165290)
+
+* Fall back to showing e-mail in ``log --short/--line`` if the
+  committer/author has only e-mail. (Lukáš Lalinský, #157026)
+
+API Breaks
+**********
+
+* Deprecate not passing a ``location`` argument to commit reporters'
+  ``started`` methods. (Matt Nordhoff)
+
+
+bzr 1.0rc1
+##########
+
+:Released: 2007-11-30
+
+Notes When Upgrading
+********************
+
+* The default repository format is now ``pack-0.92``.  This
+  default is used when creating new repositories with ``init`` and
+  ``init-repo``, and when branching over bzr+ssh or bzr+hpss.
+  (See https://bugs.launchpad.net/bugs/164626)
+
+  This format can be read and written by Bazaar 0.92 and later, and
+  data can be transferred to and from older formats.
+
+  To upgrade, please reconcile your repository (``bzr reconcile``), and then
+  upgrade (``bzr upgrade``).
+
+  ``pack-0.92`` offers substantially better scaling and performance than the
+  previous knits format. Some operations are slower where the code already
+  had bad scaling characteristics under knits, the pack format makes such
+  operations more visible as part of being more scalable overall. We will
+  correct such operations over the coming releases and encourage the filing
+  of bugs on any operation which you observe to be slower in a packs
+  repository. One particular case that we do not intend to fix is pulling
+  data from a pack repository into a knit repository over a high latency
+  link;  downgrading such data requires reinsertion of the file texts, and
+  this is a classic space/time tradeoff. The current implementation is
+  conservative on memory usage because we need to support converting data
+  from any tree without problems.
+  (Robert Collins, Martin Pool, #164476)
+
+Changes
+*******
+
+* Disable detection of plink.exe as possible SSH vendor. Plink vendor
+  still available if user selects it explicitly with BZR_SSH environment
+  variable. (Alexander Belchenko, workaround for bug #107593)
+
+* The pack format is now accessible as "pack-0.92", or "pack-0.92-subtree"
+  to enable the subtree functions (for example, for bzr-svn).
+  (Martin Pool)
+
+Features
+********
+
+* New ``authentication.conf`` file holding the password or other credentials
+  for remote servers. This can be used for SSH, SFTP, SMTP and other
+  supported transports.
+  (Vincent Ladeuil)
+
+* New rich-root and rich-root-pack formats, recording the same data about
+  tree roots that's recorded for all other directories.
+  (Aaron Bentley, #164639)
+
+* ``pack-0.92`` repositories can now be reconciled.
+  (Robert Collins, #154173)
+
+* ``switch`` command added for changing the branch a lightweight checkout
+  is associated with and updating the tree to reflect the latest content
+  accordingly. This command was previously part of the BzrTools plug-in.
+  (Ian Clatworthy, Aaron Bentley, David Allouche)
+
+* ``reconfigure`` command can now convert branches, trees, or checkouts to
+  lightweight checkouts.  (Aaron Bentley)
+
+Performance
+***********
+
+* Commit updates the state of the working tree via a delta rather than
+  supplying entirely new basis trees. For commit of a single specified file
+  this reduces the wall clock time for commit by roughly a 30%.
+  (Robert Collins, Martin Pool)
+
+* Commit with many automatically found deleted paths no longer performs
+  linear scanning for the children of those paths during inventory
+  iteration. This should fix commit performance blowing out when many such
+  paths occur during commit. (Robert Collins, #156491)
+
+* Fetch with pack repositories will no longer read the entire history graph.
+  (Robert Collins, #88319)
+
+* Revert takes out an appropriate lock when reverting to a basis tree, and
+  does not read the basis inventory twice. (Robert Collins)
+
+* Diff does not require an inventory to be generated on dirstate trees.
+  (Aaron Bentley, #149254)
+
+* New annotate merge (--merge-type=weave) implementation is fast on
+  versionedfiles withough cached annotations, e.g. pack-0.92.
+  (Aaron Bentley)
+
+Improvements
+************
+
+* ``bzr merge`` now warns when it encounters a criss-cross merge.
+  (Aaron Bentley)
+
+* ``bzr send`` now doesn't require the target e-mail address to be
+  specified on the command line if an interactive e-mail client is used.
+  (Lukáš Lalinský)
+
+* ``bzr tags`` now prints the revision number for each tag, instead of
+  the revision id, unless --show-ids is passed. In addition, tags can be
+  sorted chronologically instead of lexicographically with --sort=time.
+  (Adeodato Simó, #120231)
+
+* Windows standalone version of bzr is able to load system-wide plugins from
+  "plugins" subdirectory in installation directory. In addition standalone
+  installer write to the registry (HKLM\SOFTWARE\Bazaar) useful info
+  about paths and bzr version. (Alexander Belchenko, #129298)
+
+Documentation
+*************
+
+Bug Fixes
+*********
+
+* A progress bar has been added for knitpack -> knitpack fetching.
+  (Robert Collins, #157789, #159147)
+
+* Branching from a branch via smart server now preserves the repository
+  format. (Andrew Bennetts,  #164626)
+
+* ``commit`` is now able to invoke an external editor in a non-ascii
+  directory. (Daniel Watkins, #84043)
+
+* Catch connection errors for FTP.
+  (Vincent Ladeuil, #164567)
+
+* ``check`` no longer reports spurious unreferenced text versions.
+  (Robert Collins, John A Meinel, #162931, #165071)
+
+* Conflicts are now resolved recursively by ``revert``.
+  (Aaron Bentley, #102739)
+
+* Detect invalid transport reuse attempts by catching invalid URLs.
+  (Vincent Ladeuil, #161819)
+
+* Deleting a file without removing it shows a correct diff, not a traceback.
+  (Aaron Bentley)
+
+* Do no use timeout in HttpServer anymore.
+  (Vincent Ladeuil, #158972).
+
+* Don't catch the exceptions related to the HTTP pipeline status before
+  retrying an HTTP request or some programming errors may be masked.
+  (Vincent Ladeuil, #160012)
+
+* Fix ``bzr rm`` to not delete modified and ignored files.
+  (Lukáš Lalinský, #172598)
+
+* Fix exception when revisionspec contains merge revisons but log
+  formatter doesn't support merge revisions. (Kent Gibson, #148908)
+
+* Fix exception when ScopeReplacer is assigned to before any members have
+  been retrieved.  (Aaron Bentley)
+
+* Fix multiple connections during checkout --lightweight.
+  (Vincent Ladeuil, #159150)
+
+* Fix possible error in insert_data_stream when copying between
+  pack repositories over bzr+ssh or bzr+http.
+  KnitVersionedFile.get_data_stream now makes sure that requested
+  compression parents are sent before any delta hunks that depend
+  on them.
+  (Martin Pool, #164637)
+
+* Fix typo in limiting offsets coalescing for HTTP, leading to
+  whole files being downloaded instead of parts.
+  (Vincent Ladeuil, #165061)
+
+* FTP server errors don't error in the error handling code.
+  (Robert Collins, #161240)
+
+* Give a clearer message when a pull fails because the source needs
+  to be reconciled.
+  (Martin Pool, #164443)
+
+* It is clearer when a plugin cannot be loaded because of its name, and a
+  suggestion for an acceptable name is given. (Daniel Watkins, #103023)
+
+* Leave port as None in transport objects if user doesn't
+  specify a port in URLs.
+  (vincent Ladeuil, #150860)
+
+* Make sure Repository.fetch(self) is properly a no-op for all
+  Repository implementations. (John Arbash Meinel, #158333)
+
+* Mark .bzr directories as "hidden" on Windows.
+  (Alexander Belchenko, #71147)
+
+* ``merge --uncommitted`` can now operate on a single file.
+  (Aaron Bentley, Lukáš Lalinský, #136890)
+
+* Obsolete packs are now cleaned up by pack and autopack operations.
+  (Robert Collins, #153789)
+
+* Operations pulling data from a smart server where the underlying
+  repositories are not both annotated/both unannotated will now work.
+  (Robert Collins, #165304).
+
+* Reconcile now shows progress bars. (Robert Collins, #159351)
+
+* ``RemoteBranch`` was not initializing ``self._revision_id_to_revno_map``
+  properly. (John Arbash Meinel, #162486)
+
+* Removing an already-removed file reports the file does not exist. (Daniel
+  Watkins, #152811)
+
+* Rename on Windows is able to change filename case.
+  (Alexander Belchenko, #77740)
+
+* Return error instead of a traceback for ``bzr log -r0``.
+  (Kent Gibson, #133751)
+
+* Return error instead of a traceback when bzr is unable to create
+  symlink on some platforms (e.g. on Windows).
+  (Alexander Belchenko, workaround for #81689)
+
+* Revert doesn't crash when restoring a single file from a deleted
+  directory. (Aaron Bentley)
+
+* Stderr output via logging mechanism now goes through encoded wrapper
+  and no more uses utf-8, but terminal encoding instead. So all unicode
+  strings now should be readable in non-utf-8 terminal.
+  (Alexander Belchenko, #54173)
+
+* The error message when ``move --after`` should be used makes how to do so
+  clearer. (Daniel Watkins, #85237)
+
+* Unicode-safe output from ``bzr info``. The output will be encoded
+  using the terminal encoding and unrepresentable characters will be
+  replaced by '?'. (Lukáš Lalinský, #151844)
+
+* Working trees are no longer created when pushing into a local no-trees
+  repo. (Daniel Watkins, #50582)
+
+* Upgrade util/configobj to version 4.4.0.
+  (Vincent Ladeuil, #151208).
+
+* Wrap medusa FTP test server as an FTPServer feature.
+  (Vincent Ladeuil, #157752)
+
+API Breaks
+**********
+
+* ``osutils.backup_file`` is deprecated. Actually it's not used in bzrlib
+  during very long time. (Alexander Belchenko)
+
+* The return value of
+  ``VersionedFile.iter_lines_added_or_present_in_versions`` has been
+  changed. Previously it was an iterator of lines, now it is an iterator of
+  (line, version_id) tuples. This change has been made to aid reconcile and
+  fetch operations. (Robert Collins)
+
+* ``bzrlib.repository.get_versioned_file_checker`` is now private.
+  (Robert Collins)
+
+* The Repository format registry default has been removed; it was previously
+  obsoleted by the bzrdir format default, which implies a default repository
+  format.
+  (Martin Pool)
+
+Internals
+*********
+
+* Added ``ContainerSerialiser`` and ``ContainerPushParser`` to
+  ``bzrlib.pack``.  These classes provide more convenient APIs for generating
+  and parsing containers from streams rather than from files.  (Andrew
+  Bennetts)
+
+* New module ``lru_cache`` providing a cache for use by tasks that need
+  semi-random access to large amounts of data. (John A Meinel)
+
+* InventoryEntry.diff is now deprecated.  Please use diff.DiffTree instead.
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.1.txt'
--- doc/en/release-notes/bzr-1.1.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.1.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,228 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.1
+#######
+
+:Released: 2008-01-15
+
+(no changes from 1.1rc1)
+
+bzr 1.1rc1
+##########
+
+:Released: 2008-01-05
+
+Changes
+*******
+
+* Dotted revision numbers have been revised. Instead of growing longer with
+  nested branches the branch number just increases. (eg instead of 1.1.1.1.1
+  we now report 1.2.1.) This helps scale long lived branches which have many
+  feature branches merged between them. (John Arbash Meinel)
+
+* The syntax ``bzr diff branch1 branch2`` is no longer supported.
+  Use ``bzr diff branch1 --new branch2`` instead. This change has
+  been made to remove the ambiguity where ``branch2`` is in fact a
+  specific file to diff within ``branch1``.
+
+Features
+********
+
+* New option to use custom template-based formats in  ``bzr version-info``.
+  (Lukáš Lalinský)
+
+* diff '--using' allows an external diff tool to be used for files.
+  (Aaron Bentley)
+
+* New "lca" merge-type for fast everyday merging that also supports
+  criss-cross merges.  (Aaron Bentley)
+
+Improvements
+************
+
+* ``annotate`` now doesn't require a working tree. (Lukáš Lalinský,
+  #90049)
+
+* ``branch`` and ``checkout`` can now use files from a working tree to
+  to speed up the process.  For checkout, this requires the new
+  --files-from flag.  (Aaron Bentley)
+
+* ``bzr diff`` now sorts files in alphabetical order.  (Aaron Bentley)
+
+* ``bzr diff`` now works on branches without working trees. Tree-less
+  branches can also be compared to each other and to working trees using
+  the new diff options ``--old`` and ``--new``. Diffing between branches,
+  with or without trees, now supports specific file filtering as well.
+  (Ian Clatworthy, #6700)
+
+* ``bzr pack`` now orders revision texts in topological order, with newest
+  at the start of the file, promoting linear reads for ``bzr log`` and the
+  like. This partially fixes #154129. (Robert Collins)
+
+* Merge directives now fetch prerequisites from the target branch if
+  needed.  (Aaron Bentley)
+
+* pycurl now handles digest authentication.
+  (Vincent Ladeuil)
+
+* ``reconfigure`` can now convert from repositories.  (Aaron Bentley)
+
+* ``-l`` is now a short form for ``--limit`` in ``log``.  (Matt Nordhoff)
+
+* ``merge`` now warns when merge directives cause cherrypicks.
+  (Aaron Bentley)
+
+* ``split`` now supported, to enable splitting large trees into smaller
+  pieces.  (Aaron Bentley)
+
+Bugfixes
+********
+
+* Avoid AttributeError when unlocking a pack repository when an error occurs.
+  (Martin Pool, #180208)
+
+* Better handle short reads when processing multiple range requests.
+  (Vincent Ladeuil, #179368)
+
+* build_tree acceleration uses the correct path when a file has been moved.
+  (Aaron Bentley)
+
+* ``commit`` now succeeds when a checkout and its master branch share a
+  repository.  (Aaron Bentley, #177592)
+
+* Fixed error reporting of unsupported timezone format in
+  ``log --timezone``. (Lukáš Lalinský, #178722)
+
+* Fixed Unicode encoding error in ``ignored`` when the output is
+  redirected to a pipe. (Lukáš Lalinský)
+
+* Fix traceback when sending large response bodies over the smart protocol
+  on Windows. (Andrew Bennetts, #115781)
+
+* Fix ``urlutils.relative_url`` for the case of two ``file:///`` URLs
+  pointed to different logical drives on Windows.
+  (Alexander Belchenko, #90847)
+
+* HTTP test servers are now compatible with the HTTP protocol version 1.1.
+  (Vincent Ladeuil, #175524)
+
+* _KnitParentsProvider.get_parent_map now handles requests for ghosts
+  correctly, instead of erroring or attributing incorrect parents to ghosts.
+  (Aaron Bentley)
+
+* ``merge --weave --uncommitted`` now works.  (Aaron Bentley)
+
+* pycurl authentication handling was broken and incomplete. Fix handling of
+  user:pass embedded in the URLs.
+  (Vincent Ladeuil, #177643)
+
+* Files inside non-directories are now handled like other conflict types.
+  (Aaron Bentley, #177390)
+
+* ``reconfigure`` is able to convert trees into lightweight checkouts.
+  (Aaron Bentley)
+
+* Reduce lockdir timeout to 0 when running ``bzr serve``.  (Andrew Bennetts,
+  #148087)
+
+* Test that the old ``version_info_format`` functions still work, even
+  though they are deprecated. (John Arbash Meinel, ShenMaq, #177872)
+
+* Transform failures no longer cause ImmortalLimbo errors (Aaron Bentley,
+  #137681)
+
+* ``uncommit`` works even when the commit messages of revisions to be
+  removed use characters not supported in the terminal encoding.
+  (Aaron Bentley)
+
+* When dumb HTTP servers return whole files instead of the requested ranges,
+  read the remaining bytes by chunks to avoid overflowing network buffers.
+  (Vincent Ladeuil, #175886)
+
+Documentation
+*************
+
+* Minor tweaks made to the bug tracker integration documentation.
+  (Ian Clatworthy)
+
+* Reference material has now be moved out of the User Guide and added
+  to the User Reference. The User Reference has gained 4 sections as
+  a result: Authenication Settings, Configuration Settings, Conflicts
+  and Hooks. All help topics are now dumped into text format in the
+  doc/en/user-reference directory for those who like browsing that
+  information in their editor. (Ian Clatworthy)
+
+* *Using Bazaar with Launchpad* tutorial added. (Ian Clatworthy)
+
+Internals
+*********
+
+* find_* methods available for BzrDirs, Branches and WorkingTrees.
+  (Aaron Bentley)
+
+* Help topics can now be loaded from files.
+  (Ian Clatworthy, Alexander Belchenko)
+
+* get_parent_map now always provides tuples as its output.  (Aaron Bentley)
+
+* Parent Providers should now implement ``get_parent_map`` returning a
+  dictionary instead of ``get_parents`` returning a list.
+  ``Graph.get_parents`` is now deprecated. (John Arbash Meinel,
+  Robert Collins)
+
+* Patience Diff now supports arbitrary python objects, as long as they
+  support ``hash()``. (John Arbash Meinel)
+
+* Reduce selftest overhead to establish test names by memoization.
+  (Vincent Ladeuil)
+
+API Breaks
+**********
+
+Testing
+*******
+
+* Modules can now customise their tests by defining a ``load_tests``
+  attribute. ``pydoc bzrlib.tests.TestUtil.TestLoader.loadTestsFromModule``
+  for the documentation on this attribute. (Robert Collins)
+
+* New helper function ``bzrlib.tests.condition_id_re`` which helps
+  filter tests based on a regular expression search on the tests id.
+  (Robert Collins)
+
+* New helper function ``bzrlib.tests.condition_isinstance`` which helps
+  filter tests based on class. (Robert Collins)
+
+* New helper function ``bzrlib.tests.exclude_suite_by_condition`` which
+  generalises the ``exclude_suite_by_re`` function. (Robert Collins)
+
+* New helper function ``bzrlib.tests.filter_suite_by_condition`` which
+  generalises the ``filter_suite_by_re`` function. (Robert Collins)
+
+* New helper method ``bzrlib.tests.exclude_tests_by_re`` which gives a new
+  TestSuite that does not contain tests from the input that matched a
+  regular expression. (Robert Collins)
+
+* New helper method ``bzrlib.tests.randomize_suite`` which returns a
+  randomized copy of the input suite. (Robert Collins)
+
+* New helper method ``bzrlib.tests.split_suite_by_re`` which splits a test
+  suite into two according to a regular expression. (Robert Collins)
+
+* Parametrize all HTTP tests for the transport implementations, the HTTP protocol versions (1.0 and 1.1) and the authentication schemes.
+  (Vincent Ladeuil)
+
+* The ``exclude_pattern`` and ``random_order`` parameters to the function
+  ``bzrlib.tests.filter_suite_by_re`` have been deprecated. (Robert Collins)
+
+* The method ``bzrlib.tests.sort_suite_by_re`` has been deprecated. It is
+  replaced by the new helper methods added in this release. (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.10.txt'
--- doc/en/release-notes/bzr-1.10.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.10.txt	2010-10-08 08:15:14 +0000
@@ -0,0 +1,160 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.10
+########
+
+:Released: 2008-12-05
+
+Bazaar 1.10 has several performance improvements for copying revisions
+(especially for small updates to large projects).  There has also been a
+significant amount of effort in polishing stacked branches.  The commands
+``shelve`` and ``unshelve`` have become core commands, with an improved
+implementation.
+
+The only changes versus bzr-1.10rc1 are bugfixes for stacked branches.
+
+bug Fixes
+*********
+
+* Don't set a pack write cache size from RepoFetcher, because the
+  cache is not coherent with reads and causes ShortReadvErrors.
+  This reverses the change that fixed #294479.
+  (Martin Pool, #303856)
+
+* Properly handle when a revision can be inserted as a delta versus
+  when it needs to be expanded to a fulltext for stacked branches.
+  There was a bug involving merge revisions. As a method to help
+  prevent future difficulties, also make stacked fetches sort
+  topologically. (John Arbash Meinel, #304841)
+
+
+bzr 1.10rc1
+###########
+
+:Released: 2008-11-28
+
+This release of Bazaar focuses on performance improvements when pushing
+and pulling revisions, both locally and to remote networks.  The popular
+``shelve`` and ``unshelve`` commands, used to interactively revert and
+restore work in progress, have been merged from bzrtools into the bzr
+core.  There are also bug fixes for portability, and for stacked branches.
+
+New Features
+************
+
+* New ``commit_message_template`` hook that is called by the commit
+  code to generate a template commit message. (Jelmer Vernooij)
+
+* New `shelve` and `unshelve` commands allow undoing and redoing changes.
+  (Aaron Bentley)
+
+Improvements
+************
+
+* ``(Remote)Branch.copy_content_into`` no longer generates the full revision
+  history just to set the last revision info.
+  (Andrew Bennetts, John Arbash Meinel)
+
+* Fetches between formats with different serializers (such as
+  pack-0.92-subtree and 1.9-rich-root) are faster now.  This is due to
+  operating on batches of 100 revisions at time rather than
+  one-by-one.  (Andrew Bennetts, John Arbash Meinel)
+
+* Search index files corresponding to pack files we've already used
+  before searching others, because they are more likely to have the
+  keys we're looking for.  This reduces the number of iix and tix
+  files accessed when pushing 1 new revision, for instance.
+  (John Arbash Meinel)
+
+* Signatures to transfer are calculated more efficiently in
+  ``item_keys_introduced_by``.  (Andrew Bennetts, John Arbash Meinel)
+
+* The generic fetch code can once again copy revisions and signatures
+  without extracting them completely to fulltexts and then serializing
+  them back down into byte strings. This is a significant performance
+  improvement when fetching from a stacked branch.
+  (John Arbash Meinel, #300289)
+
+* When making a large readv() request over ``bzr+ssh``, break up the
+  request into more manageable chunks. Because the RPC is not yet able
+  to stream, this helps keep us from buffering too much information at
+  once. (John Arbash Meinel)
+
+Bug Fixes
+*********
+
+* Better message when the user needs to set their Launchpad ID.
+  (Martin Pool, #289148)
+
+* ``bzr commit --local`` doesn't access the master branch anymore.
+  This fixes a regression introduced in 1.9.  (Marius Kruger, #299313)
+
+* Don't call the system ``chdir()`` with an empty path. Sun OS seems
+  to give an error in that case.  Also, don't count on ``getcwd()``
+  being able to allocate a new buffer, which is a gnu extension.
+  (John Arbash Meinel, Martin Pool, Harry Hirsch, #297831)
+
+* Don't crash when requesting log --forward <file> for a revision range
+  starting with a dotted revno.
+  (Vincent Ladeuil, #300055)
+
+* Don't create text deltas spanning stacked repositories; this could
+  cause "Revision X not present in Y" when later accessing them.
+  (Martin Pool, #288751)
+
+* Pack repositories are now able to reload the pack listing and retry
+  the current operation if another action causes the data to be
+  repacked.  (John Arbash Meinel, #153786)
+
+* PermissionDenied errors from smart servers no longer cause
+  "PermissionDenied: "None"" on the client.
+  (Andrew Bennetts, #299254)
+
+* Pushing to a stacked pack repository now batches writes, the same
+  way writes are batched to ordinary pack repository.  This makes
+  pushing to a stacked branch over the network much faster.
+  (Andrew Bennetts, #294479)
+
+* TooManyConcurrentRequests no longer occur when a fetch fails and
+  tries to abort a write group.  This allows the root cause (e.g. a
+  network interruption) to be reported.  (Andrew Bennetts, #297014)
+
+* RemoteRepository.get_parent_map now uses fallback repositories.
+  (Aaron Bentley, #297991?, #293679?)
+
+API Changes
+***********
+
+* ``CommitBuilder`` now validates the strings it will be committing,
+  to ensure that they do not have characters that will not be properly
+  round-tripped. For now, it just checks for characters that are
+  invalid in the XML form. (John Arbash Meinel, #295161)
+
+* Constructor parameters for NewPack (internal to pack repositories)
+  have changed incompatibly.
+
+* ``Repository.abort_write_group`` now accepts an optional
+  ``suppress_errors`` flag.  Repository implementations that override
+  ``abort_write_group`` will need to be updated to accept the new
+  argument.  Subclasses that only override ``_abort_write_group``
+  don't need to change.
+
+* Transport implementations must provide copy_tree_to_transport.  A default
+  implementation is provided for Transport subclasses.
+
+Testing
+*******
+
+* ``bzr selftest`` now fails if no doctests are found in a module
+  that's expected to have them.  (Martin Pool)
+
+* Doctests now only report the first failure.  (Martin Pool)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.11.txt'
--- doc/en/release-notes/bzr-1.11.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.11.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,278 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.11
+########
+
+:Codename: "Eyes up!"
+:Released: 2009-01-19
+
+This first monthly release of Bazaar for 2009 improves Bazaar's operation
+in Windows, Mac OS X, and other situations where file names are matched
+without regard to capitalization: Bazaar tries to match the case of an
+existing file.  This release of Bazaar also improves the efficiency of
+Tortoise Windows Shell integration and lets it work on 64-bit platforms.
+
+The UI through which Bazaar supports historic formats has been improved,
+so 'bzr help formats' now gives a simpler and shorter list, with clear
+advice.
+
+This release also fixes a number of bugs, particularly a glitch that can
+occur when there are concurrent writes to a pack repository.
+
+Bug Fixes
+*********
+
+* Fix failing test when CompiledChunksToLines is not available.
+  (Vincent Ladeuil)
+
+* Stacked branches don't repeatedly open their transport connection.
+  (John Arbash Meinel)
+
+
+
+bzr 1.11rc1
+###########
+
+:Codename: "Eyes up!"
+:Released: 2009-01-09
+
+Changes
+*******
+
+* Formats using Knit-based repository formats are now explicitly
+  marked as deprecated. (Ian Clatworthy)
+
+New Features
+************
+
+* Add support for `bzr tags -r 1..2`, that is we now support showing
+  tags applicable for a specified revision range. (Marius Kruger)
+
+* ``authentication.conf`` now accepts pluggable read-only credential
+  stores. Such a plugin (``netrc_credential_store``) is now included,
+  handles the ``$HOME/.netrc`` file and can server as an example to
+  implement other plugins.
+  (Vincent Ladeuil)
+
+* ``shelve --list`` can now be used to list shelved changes.
+  (Aaron Bentley)
+
+Improvements
+************
+
+* Add trailing slash to directories in all output of ``bzr ls``, except
+  ``bzr ls --null``. (Gordon P. Hemsley, #306424)
+
+* ``bzr revision-info`` now supports a -d option to specify an
+  alternative branch. (Michael Hudson)
+
+* Add connection to a C++ implementation of the Windows Shell Extension
+  which is able to fully replace the current Python implemented one.
+  Advantages include 64bit support and reduction in overhead for
+  processes which drag in shell extensions.
+  (Mark Hammond)
+
+* Support the Claws mail client directly, rather than via
+  xdg-email. This prevents the display of an unnecessary modal
+  dialog in Claws, informing the user that a file has been
+  attached to the message, and works around bug #291847 in
+  xdg-utils which corrupts the destination address.
+
+* When working on a case-insensitive case-preserving file-system, as
+  commonly found with Windows, bzr will often ignore the case of the
+  arguments specified by the user in preference to the case of an existing
+  item on the file-system or in the inventory to help prevent
+  counter-intuitive behaviour on Windows. (Mark Hammond)
+
+Bug Fixes
+*********
+  
+* Allow BzrDir implementation to implement backing up of 
+  control directory. (#139691)
+
+* ``bzr push`` creating a new stacked branch will now only open a
+  single connection to the target machine. (John Arbash Meinel)
+
+* Don't call iteritems on transport_list_registry, because it may
+  change during iteration.  (Martin Pool, #277048)
+
+* Don't make a broken branch when pushing an unstackable-format branch
+  that's in a stackable shared repository to a location with default
+  stack-on location.  (Andrew Bennetts, #291046)
+
+* Don't require embedding user in HTTP(S) URLs do use authentication.conf.
+  (Ben Jansen, Vincent Ladeuil, #300347)
+
+* Fix a problem with CIFS client/server lag on windows colliding with
+  an invariant-per-process algorithm for generating AtomicFile names
+  (Adrian Wilkins, #304023)
+
+* Fix bogus setUp signature in UnavailableFTPServer.
+  (Gary van der Merwe, #313498)
+
+* Fix compilation error in ``_dirstate_helpers_c`` on SunOS/Solaris.
+  (Jari Aalto)
+
+* Fix SystemError in ``_patiencediff_c`` module by calling
+  PyErr_NoMemory() before returning NULL in PatienceSequenceMatcher_new.
+  (Andrew Bennetts, #303206)
+
+* Give proper error message for diff with non-existent dotted revno.
+  (Marius Kruger, #301969)
+
+* Handle EACCES (permission denied) errors when launching a message
+  editor, and emit warnings when a configured editor cannot be
+  started. (Andrew Bennetts)
+
+* ``$HOME/.netrc`` file is now recognized as a read-only credential store
+  if configured in ``authentication.conf`` with 'password_encoding=netrc'
+  in the appropriate sections.
+  (Vincent Ladeuil, #103029)
+
+* Opening a stacked branch now properly shares the connection, rather
+  than opening a new connection for the stacked-on branch.
+  (John Arbash meinel)
+
+* Preserve transport decorators while following redirections.
+  (Vincent Ladeuil, #245964, #270863)
+
+* Provides a finer and more robust filter for accepted redirections.
+  (Vincent Ladeuil, #303959, #265070)
+
+* ``shelve`` paths are now interpreted relative to the current working
+  tree.  (Aaron Bentley)
+
+* ``Transport.readv()`` defaults to not reading more than 100MB in a
+  single array. Further ``RemoteTransport.readv`` sets this to 5MB to
+  work better with how it splits its requests.
+  (John Arbash Meinel, #303538)
+
+* Pack repositories are now able to reload the pack listing and retry
+  the current operation if another action causes the data to be
+  repacked.  (John Arbash Meinel, #153786)
+
+* ``pull -v`` now respects the log_format configuration variable.
+  (Aaron Bentley)
+
+* ``push -v`` now works on non-initial pushes.  (Aaron Bentley)
+
+* Use the short status format when the short format is used for log.
+  (Vincent Ladeuil, #87179)
+
+* Allow files to be renamed or moved via remove + add-by-id. (Charles
+  Duffy, #314251)
+
+Documentation
+*************
+
+* Improved the formats help topic to explain why multiple formats
+  exist and to provide guidelines in selecting one. Introduced
+  two new supporting help topics: current-formats and other-formats.
+  (Ian Clatworthy)
+
+API Changes
+***********
+
+* ``LRUCache(after_cleanup_size)`` was renamed to
+  ``after_cleanup_count`` and the old name deprecated. The new name is
+  used for clarity, and to avoid confusion with
+  ``LRUSizeCache(after_cleanup_size)``. (John Arbash Meinel)
+
+* New ``ForeignRepository`` base class, to help with foreign branch 
+  support (e.g. svn).  (Jelmer Vernooij)
+
+* ``node_distances`` and ``select_farthest`` can no longer be imported
+  from ``bzrlib.graph``.  They can still be imported from
+  ``bzrlib.deprecated_graph``, which has been the preferred way to
+  import them since before 1.0.  (Andrew Bennetts)
+  
+* The logic in commit now delegates inventory basis calculations to
+  the ``CommitBuilder`` object; this requires that the commit builder
+  in use has been updated to support the new ``recording_deletes`` and
+  ``record_delete`` methods. (Robert Collins)
+
+Testing
+*******
+
+* An HTTPS server is now available (it requires python-2.6). Future bzr
+  versions will allow the use of the python-2.6 ssl module that can be
+  installed for 2.5 and 2.4.
+
+* ``bzr selftest`` now fails if new trailing white space is added to
+  the bazaar sources. It only checks changes not committed yet. This
+  means that PQM will now reject changes that introduce new trailing
+  whitespace. (Marius Kruger)
+
+* Introduced new experimental formats called ``1.12-preview`` and
+  ``1.12-preview-rich-root`` to enable testing of related pending
+  features, namely content filtering and filtered views.
+  (Ian Clatworthy)
+
+Internals
+*********
+
+* Added an ``InventoryEntry`` cache when deserializing inventories.
+  Can cut the time to iterate over multiple RevisionsTrees in half.
+  (John Arbash Meinel)
+
+* Added ``bzrlib.fifo_cache.FIFOCache`` which is designed to have
+  minimal overhead versus using a plain dict for cache hits, at the
+  cost of not preserving the 'active' set as well as an ``LRUCache``.
+  (John Arbash Meinel)
+
+* ``bzrlib.patience_diff.unified_diff`` now properly uses a tab
+  character to separate the filename from the date stamp, and doesn't
+  add trailing whitespace when a date stamp is not supplied.
+  (Adeodato Simó, John Arbash Meinel)
+
+* ``DirStateWorkingTree`` and ``DirStateWorkingTreeFormat`` added
+  as base classes of ``WorkingTree4`` and ``WorkingTreeFormat4``
+  respectively. (Ian Clatworthy)
+
+* ``KnitVersionedFiles._check_should_delta()`` now uses the
+  ``get_build_details`` api to avoid multiple hits to the index, and
+  to properly follow the ``compression_parent`` rather than assuming
+  it is the left-hand parent. (John Arbash Meinel)
+
+* ``KnitVersionedFiles.get_record_stream()`` will now chose a
+  more optimal ordering when the keys are requested 'unordered'.
+  Previously the order was fully random, now the records should be
+  returned from each pack in turn, in forward I/O order.
+  (John Arbash Meinel)
+    
+* ``mutter()`` will now flush the ``~/.bzr.log`` if it has been more
+  than 2s since the last time it flushed. (John Arbash Meinel)
+
+* New method ``bzrlib.repository.Repository.add_inventory_by_delta``
+  allows adding an inventory via an inventory delta, which can be
+  more efficient for some repository types. (Robert Collins)
+
+* Repository ``CommitBuilder`` objects can now accumulate an inventory
+  delta. To enable this functionality call ``builder.recording_deletes``
+  and additionally call ``builder.record_delete`` when a delete
+  against the basis occurs. (Robert Collins)
+
+* The default HTTP handler has been changed from pycurl to urllib.
+  The default is still pycurl for https connections. (The only
+  advantage of pycurl is that it checks ssl certificates.)
+  (John Arbash Meinel)
+
+* ``VersionedFiles.get_record_stream()`` can now return objects with a
+  storage_kind of ``chunked``. This is a collection (list/tuple) of
+  strings. You can use ``osutils.chunks_to_lines()`` to turn them into
+  guaranteed 'lines' or you can use ``''.join(chunks)`` to turn it
+  into a fulltext. This allows for some very good memory savings when
+  asking for many texts that share ancestry, as the individual chunks
+  can be shared between versions of the file. (John Arbash Meinel)
+
+* ``pull -v`` and ``push -v`` use new function
+  ``bzrlib.log.show_branch_change`` (Aaron Bentley)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.12.txt'
--- doc/en/release-notes/bzr-1.12.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.12.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,227 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.12
+########
+
+:Codename: 1234567890
+:1.12: 2009-02-13
+:1.12rc1: 2009-02-10
+
+This release of Bazaar contains many improvements to the speed,
+documentation and functionality of ``bzr log`` and the display of logged
+revisions by ``bzr status``.  bzr now also gives a better indication of
+progress, both in the way operations are drawn onto a text terminal, and
+by showing the rate of network IO.
+
+Changes from RC1 to Final
+*************************
+
+* ``bzr init --development-wt5[-rich-root]`` would fail because of
+  circular import errors. (John Arbash Meinel, #328135)
+
+* Expanded the help for log and added a new help topic called
+  ``log-formats``.  (Ian Clatworthy)
+
+Compatibility Breaks
+********************
+
+* By default, ``bzr status`` after a merge now shows just the pending
+  merge tip revisions. This improves the signal-to-noise ratio after
+  merging from trunk and completes much faster. To see all merged
+  revisions, use the new ``-v`` flag.  (Ian Clatworthy)
+
+* ``bzr log --line`` now shows any tags after the date and before
+  the commit message. If you have scripts which parse the output
+  from this command, you may need to adjust them accordingly.
+  (Ian Clatworthy)
+
+* ``bzr log --short`` now shows any additional revision properties
+  after the date and before the commit message.  Scripts that parse 
+  output of the log command in this situation may need to adjust.
+  (Neil Martinsen-Burrell)
+
+* The experimental formats ``1.12-preview`` and ``1.12-preview-rich-root``
+  have been renamed ``development-wt5`` and ``development-wt5-rich-root``
+  respectively, given they are not ready for release in 1.12.
+  (Ian Clatworthy)
+
+* ``read_bundle_from_url`` has been deprecated. (Vincent Ladeuil)
+
+New Features
+************
+
+* Add support for filtering ``bzr missing`` on revisions.  Remote revisions
+  can be filtered using ``bzr missing -r -20..-10`` and local revisions can
+  be filtered using ``bzr missing --my-revision -20..-10``.
+  (Marius Kruger)
+
+* ``bzr log -p`` displays the patch diff for each revision.
+  When logging a file, the diff only includes changes to that file.
+  (Ian Clatworthy, #202331, #227335)
+
+* ``bzr log`` supports a new option called ``-n N`` or ``--level N``.
+  A value of 0 (zero) means "show all nested merge revisions" while
+  a value of 1 (one) means "show just the top level". Values above
+  1 can be used to see a limited amount of nesting. That can be
+  useful for seeing the level or two below PQM submits for example.
+  To force the ``--short`` and ``--line`` formats to display all nested
+  merge revisions just like ``--long`` does by default, use a command
+  like ``bzr log --short -n0``. To display just the mainline using
+  ``--long`` format, ``bzr log --long -n1``.
+  (Ian Clatworthy)
+
+Improvements
+************
+
+* ``bzr add`` more clearly communicates success vs failure.
+  (Daniel Watkins)
+
+* ``bzr init`` will now print a little less verbose output.
+  (Marius Kruger)
+
+* ``bzr log`` is now much faster in many use cases, particularly
+  at incrementally displaying results and filtering by a
+  revision range. (Ian Clatworthy)
+
+* ``bzr log --short`` and ``bzr log --line`` now show tags, if any,
+  for each revision. The tags are shown comma-separated inside
+  ``{}``. For short format, the tags appear at the end of line
+  before the optional ``[merge]`` indicator. For line format,
+  the tags appear after the date. (Ian Clatworthy)
+
+* Progress bars now show the rate of activity for some SFTP 
+  operations, and they are drawn different.  (Martin Pool, #172741)
+
+* Progress bars now show the rate of activity for urllib and pycurl based
+  HTTP client implementations. The operations are tracked at the socket
+  level for better precision.
+  (Vincent Ladeuil)
+
+* Rule-based preferences can now accept multiple patterns for a set of
+  rules.  (Marius Kruger)
+
+* The ``ancestor:`` revision spec will now default to referring to the
+  parent of the branch if no other location is given.
+  (Daniel Watkins, #198417)
+
+* The debugger started as a result of setting ``$BZR_PDB`` works
+  around a bug in ``pdb``, http://bugs.python.org/issue4150.  The bug
+  can cause truncated tracebacks in Python versions before 2.6.
+  (Andrew Bennetts)
+
+* VirtualVersionedFiles now implements
+  ``iter_lines_added_or_present_in_keys``. This allows the creation of 
+  new branches based on stacked bzr-svn branches. (#311997)
+
+Bug Fixes
+*********
+
+* ``bzr annotate --show-ids`` doesn't give a backtrace on empty files
+  anymore.
+  (Anne Mohsen, Vincent Ladeuil, #314525)
+
+* ``bzr log FILE`` now correctly shows mainline revisions merging
+  a change to FILE when the ``--short`` and ``--line`` log formats
+  are used. (Ian Clatworthy, #317417)
+
+* ``bzr log -rX..Y FILE`` now shows the history of FILE provided
+  it existed in Y or X, even if the file has since been deleted or
+  renamed. If no range is given, the current/basis tree and
+  initial tree are searched in that order. More generally, log
+  now interprets filenames in their historical context.
+  (Ian Clatworthy, #175520)
+
+* ``bzr status`` now reports nonexistent files and continues, then
+  errors (with code 3) at the end.  (Karl Fogel, #306394)
+
+* Don't require the present compression base in knits to be the same
+  when adding records in knits. (Jelmer Vernooij, #307394)
+
+* Fix a problem with CIFS client/server lag on Windows colliding with
+  an invariant-per-process algorithm for generating AtomicFile names
+  (Adrian Wilkins, #304023)
+
+* Many socket operations now handle EINTR by retrying the operation.
+  Previously EINTR was treated as an unrecoverable failure.  There is
+  a new ``until_no_eintr`` helper function in ``bzrlib.osutils``.
+  (Andrew Bennetts)
+
+* Support symlinks with non-ascii characters in the symlink filename.
+  (Jelmer Vernooij, #319323)
+
+* There was a bug in how we handled resolving when a file is deleted
+  in one branch, and modified in the other. If there was a criss-cross
+  merge, we would cause the deletion to conflict a second time.
+  (Vincent Ladeuil, John Arbash Meinel)
+
+* There was another bug in how we chose the correct intermediate LCA in
+  criss-cross merges leading to several kind of changes be incorrectly
+  handled.
+  (John Arbash Meinel, Vincent Ladeuil)
+
+* Unshelve now handles deleted paths without crashing. (Robert Collins)
+
+Documentation
+*************
+
+* Improved plugin developer documentation.  (Martin Pool)
+
+API Changes
+***********
+
+* ``ProgressBarStack`` is deprecated; instead use
+  ``ui_factory.nested_progress_bar`` to create new progress bars.
+  (Martin Pool)
+
+* ForeignVcsMapping() now requires a ForeignVcs object as first
+  argument. (Jelmer Vernooij)
+
+* ForeignVcsMapping.show_foreign_revid() has been moved to
+  ForeignVcs. (Jelmer Vernooij)
+
+* ``read_bundle_from_url`` is deprecated in favor of
+  ``read_mergeable_from_url``.  (Vincent Ladeuil)
+
+* Revision specifiers are now registered in
+  ``bzrlib.revisionspec.revspec_registry``, and the old list of 
+  revisionspec classes (``bzrlib.revisionspec.SPEC_TYPES``) has been
+  deprecated. (Jelmer Vernooij, #321183)
+
+* The progress and UI classes have changed; the main APIs remain the
+  same but code that provides a new UI or progress bar class may
+  need to be updated.  (Martin Pool)
+
+Internals
+*********
+
+* Default User Interface (UI) is CLIUIFactory when bzr runs in a dumb
+  terminal. It is sometimes desirable do override this default by forcing
+  bzr to use TextUIFactory. This can be achieved by setting the
+  BZR_USE_TEXT_UI environment variable (emacs shells, as opposed to
+  compile buffers, are such an example).
+  (Vincent Ladeuil)
+
+* New API ``Branch.iter_merge_sorted_revisions()`` that iterates over
+  ``(revision_id, depth, revno, end_of_merge)`` tuples.
+  (Ian Clatworthy)
+
+* New ``Branch.dotted_revno_to_revision_id()`` and
+  ``Branch.revision_id_to_dotted_revno()`` APIs that pick the most
+  efficient way of doing the mapping.
+  (Ian Clatworthy)
+
+* Refactor cmd_serve so that it's a little easier to build commands that
+  extend it, and perhaps even a bit easier to read.  (Jonathan Lange)
+
+* ``TreeDelta.show()`` now accepts a ``filter`` parameter allowing log
+  formatters to retrict the output.
+  (Vincent Ladeuil)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.13.txt'
--- doc/en/release-notes/bzr-1.13.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.13.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,400 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+
+bzr 1.13
+########
+
+:Codename: paraskavedekatriaphobia
+:1.13: 2009-03-14
+:1.13rc1: 2009-03-10
+:1.13.1: 2009-03-23
+:1.13.2: 2009-04-27
+
+GNU Changelog output can now be produced by ``bzr log --gnu-changelog``.  Debug
+flags can now be set in ``~/.bazaar/bazaar.conf``.  Lightweight checkouts and
+stacked branches should both be much faster over remote connections.  
+
+Changes From 1.13.1 to 1.13.2
+*****************************
+
+A regression was found in the 1.13.1 release. When bzr 1.13.1 and earlier push
+a stacked branch they do not take care to push all the parent inventories for
+the transferred revisions. This means that a smart server serving that branch
+often cannot calculate inventory deltas for the branch (because smart server
+does not/cannot open fallback repositories). Prior to 1.13 the server did not
+have a verb to stream revisions out of a repository, so that's why this bug has
+appeared now.
+
+Bug Fixes
+*********
+
+* Fix for bug 354036 ErrorFromSmartServer - AbsentContentFactory object has no
+  attribute 'get_bytes_as' exception while pulling from Launchpad 
+  (Jean-Francois Roy, Andrew Bennetts, Robert Collins)
+
+Changes From 1.13final to 1.13.1
+********************************
+
+A couple regessions where found in the 1.13 release. The pyrex-generated C
+extensions are missing from the .tar.gz and .zip files.  Documentation on how
+to generate GNU ChangeLogs is wrong.
+
+Bug Fixes
+*********
+
+* Change ``./bzr``'s ``_script_version`` to match ./bzrlib/__init__.py
+  version_info. (Bob Tanner, Martin Pool, #345232)
+
+* Distribution archives for 1.13 do not contain generated C extension modules
+  (Jean-Francois Roy, Bob Tanner, #344465)
+
+* GNU ChangeLog output can now be produced by bzr log --format gnu-changelog is
+  incorrect (Deejay, Bob Tanner, Martin Pool, Robert Collins, #343928)
+
+* ``merge --force`` works again. (Robert Collins, #342105)
+
+Changes From 1.13rc1 to 1.13final
+*********************************
+
+* Fix "is not a stackable format" error when pushing a
+  stackable-format branch with an unstackable-format repository to a
+  destination with a default stacking policy.  (Andrew Bennetts)
+
+* Progress bars now show the rate of network activity for
+  ``bzr+ssh://`` and ``bzr://`` connections.  (Andrew Bennetts)
+
+Compatibility Breaks
+********************
+
+* ``bzr log --line`` now indicates which revisions are merges with
+  `[merge]` after the date.  Scripts which parse the output of this
+  command may need to be adjusted.
+  (Neil Martinsen-Burrell)
+
+New Features
+************
+
+* ``bzr reconfigure`` now supports --with-trees and --with-no-trees
+  options to change the default tree-creation policy of shared
+  repositories.  (Matthew Fuller, Marius Kruger, #145033)
+
+* Debug flags can now be set in ``~/.bazaar/bazaar.conf``.
+  (Martin Pool)
+
+* Filtered views provide a mask over the tree so that users can focus
+  on a subset of a tree when doing their work. See ``Filtered views``
+  in chapter 7 of the User Guide and ``bzr help view`` for details.
+  (Ian Clatworthy)
+
+* GNU Changelog output can now be produced by ``bzr log --gnu-changelog``.
+  (Andrea Bolognani, Martin Pool)
+
+* The ``-Dmemory`` flag now gives memory information on Windows.
+  (John Arbash Meinel)
+
+* Multiple authors for a commit can now be recorded by using the "--author"
+  option multiple times. (James Westby, #185772)
+
+* New clean-tree command, from bzrtools.  (Aaron Bentley, Jelmer Vernooij)
+
+* New command ``bzr launchpad-open`` opens a Launchpad web page for that
+  branch in your web browser, as long as the branch is on Launchpad at all.
+  (Jonathan Lange)
+
+* New API for getting bugs fixed by a revision: Revision.iter_bugs().
+  (Jonathan Lange)
+
+Improvements
+************
+
+* All bzr ``Hooks`` classes are now registered in
+  ``bzrlib.hooks.known_hooks``. This removes the separate list from
+  ``bzrlib.tests`` and ensures that all hooks registered there are
+  correctly isolated by the test suite (previously
+  ``MutableTreeHooks`` were not being isolated correctly). Further, 
+  documentation for hooks is now dynamically generated from the
+  present HookPoints. ``bzr hooks`` will now also report on all the
+  hooks present in the ``bzrlib.hooks.known_hooks`` registry.
+  (Robert Collins)
+
+* ``bzr add`` no longer prints ``add completed`` on success. Failure
+  still prints an error message. (Robert Collins)
+
+* ``bzr branch`` now has a ``--no-tree`` option which turns off the
+  generation of a working tree in the new branch.
+  (Daniel Watkins, John Klinger, #273993)
+
+* Bazaar will now point out ``bzr+ssh://`` to the user when they 
+  use ssh://. (Jelmer Vernooij, #330535)
+
+* ``bzr -v info`` now omits the number of committers branch statistic,
+  making it many times faster for large projects. To include that
+  statistic in the output, use ``bzr -vv info``.
+  (Ian Clatworthy)
+
+* ``bzr push`` to a ``bzr`` url (``bzr://``, ``bzr+ssh://`` etc) will
+  stream if the server is version 1.13 or greater, reducing roundtrips
+  significantly. (Andrew Bennetts, Robert Collins)
+
+* Lightweight Checkouts and Stacked Branches should both be much
+  faster over remote connections. Building the working tree now
+  batches up requests into approx 5MB requests, rather than a separate
+  request for each file. (John Arbash Meinel)
+
+* Support for GSSAPI authentication when using HTTP or HTTPS. 
+  (Jelmer Vernooij)
+
+* The ``bzr shelve`` prompt now includes a '?' help option to explain the
+  short options better. (Daniel Watkins, #327429)
+
+* ``bzr lp-open`` now falls back to the push location if it cannot find a
+  public location. (Jonathan Lange, #332372)
+
+* ``bzr lp-open`` will try to find the Launchpad URL for the location
+  passed on the command line. This makes ``bzr lp-open lp:foo`` work as
+  expected. (Jonathan Lange, #332705)
+
+* ``bzr send`` now supports MH-E via ``emacsclient``. (Eric Gillespie)
+
+Bug Fixes
+*********
+
+* Allows ``bzr log <FILE>`` to be called in an empty branch without
+  backtracing. (Vincent Ladeuil, #346431)
+
+* Bazaar now gives a better message including the filename if it's
+  unable to read a file in the working directory, for example because
+  of a permission error.  (Martin Pool, #338653)
+
+* ``bzr cat -r<old> <path>`` doesn't traceback anymore when <path> has a
+  file id in the working tree different from the one in revision <old>.
+  (Vincent Ladeuil, #341517, #253806)
+
+* ``bzr send`` help is more specific about how to apply merge
+  directives.  (Neil Martinsen-Burrell, #253470)
+
+* ``bzr missing`` now uses ``Repository.get_revision_delta()`` rather
+  than fetching trees and determining a delta itself. (Jelmer
+  Vernooij, #315048)
+
+* ``bzr push`` to a smart server no longer causes "Revision
+  {set([('null:',)])} not present ..." errors when the branch has
+  multiple root revisions. (Andrew Bennetts, #317654)
+
+* ``bzr shelve`` now properly handle patches with no terminating newline.
+  (Benoît PIERRE, #303569)
+
+* ``bzr unshelve`` gives a more palatable error if passed a non-integer
+  shelf id. (Daniel Watkins)
+
+* Export now handles files that are not present in the tree.
+  (James Westby, #174539)
+
+* Fixed incorrect "Source format does not support stacking" warning
+  when pushing to a smart server.  (Andrew Bennetts, #334114)
+  
+* Fixed "sprout() got an unexpected keyword argument 'source_branch'"
+  error branching from old repositories.
+  (Martin Pool, #321695)
+
+* Make ``bzr push --quiet <non-local location>`` less chatty.
+  (Kent Gibson, #221461)
+
+* Many Branch hooks would not fire with ``bzr://`` and ``bzr+ssh://``
+  branches, and this was not noticed due to a bug in the test logic
+  for branches. This is now fixed and a test added to prevent it
+  reoccuring. (Robert Collins, Andrew Bennetts)
+
+* Restore the progress bar on Windows. We were disabling it when TERM
+  wasn't set, but Windows doesn't set TERM. (Alexander Belchenko,
+  #334808)
+
+* ``setup.py build_ext`` now gives a proper error when an extension
+  fails to build. (John Arbash Meinel)
+
+* Symlinks to non ascii file names are now supported.
+  (Robert Collins, Vincent Ladeuil, #339055, #272444)    
+
+* Under rare circumstances (aka nobody reported a bug about it), the FTP
+  transport could revert to ascii mode. It now stays in binary mode except
+  when needed.  (Vincent Ladeuil)
+
+* Unshelve does not generate warnings about progress bars.
+  (Aaron Bentley, #328148)
+
+* shelve cleans up properly when unversioned files are specified.
+  (Benoît Pierre, Aaron Bentley)
+
+Documentation
+*************
+
+* Added ``Organizing your workspace`` to the User Guide appendices,
+  summarizing some common ways of organizing trees, branches and
+  repositories and the processes/workflows implied/enabled by each.
+  (Ian Clatworthy)
+
+* Hooks can now be self documenting. ``bzrlib.hooks.Hooks.create_hook``
+  is the entry point for this feature. (Robert Collins)
+
+* The documentation for ``shelve`` and ``unshelve`` has been clarified.
+  (Daniel Watkins, #327421, #327425)
+
+API Changes
+***********
+
+* ``bzr selftest`` now fails if the bazaar sources contain trailing
+  whitespace, non-unix style line endings and files not ending in a
+  newline. About 372 files and 3243 lines with trailing whitespace was
+  updated to comply with this. The code already complied with the other
+  criteria, but now it is enforced. (Marius Kruger)
+
+* ``bzrlib.branch.PushResult`` was renamed to 
+  ``bzrlib.branch.BranchPushResult``. (Jelmer Vernooij)
+
+* ``Branch.fetch`` and ``Repository.fetch`` now return None rather
+  than a count of copied revisions and failed revisions. A while back
+  we stopped ever reporting failed revisions because we started
+  erroring instead, and the copied revisions count is not used in the
+  UI at all - indeed it only reflects the repository status not
+  changes to the branch itself. (Robert Collins)
+
+* ``Inventory.apply_delta`` now raises an AssertionError if a file-id
+  appears multiple times within the delta. (Ian Clatworthy)
+
+* MutableTree.commit now favours the "authors" argument, with the old
+  "author" argument being deprecated.
+
+* Remove deprecated EmptyTree.  (Martin Pool)
+
+* ``Repository.fetch`` now accepts an optional ``fetch_spec``
+  parameter.  A ``SearchResult`` or ``MiniSearchResult`` may be passed
+  to ``fetch_spec`` instead of a ``last_revision`` to specify exactly
+  which revisions to fetch. (Andrew Bennetts)
+
+* ``RepositoryAcquisitionPolicy.acquire_repository`` now returns a
+  tuple of ``(repository, is_new_flag)``, rather than just the
+  repository.  (Andrew Bennetts)
+
+* Revision.get_apparent_author() is now deprecated, replaced by
+  Revision.get_apparent_authors(), which returns a list. The former
+  now returns the first item that would be returned from the second.
+
+* The ``BranchBuilder`` test helper now accepts a ``timestamp``
+  parameter to ``build_commit`` and ``build_snapshot``.  (Martin Pool)
+
+* The ``_fetch_*`` attributes on ``Repository`` are now on
+  ``RepositoryFormat``, more accurately reflecting their intent (they
+  describe a disk format capability, not state of a particular
+  repository of that format). (Robert Collins)
+
+Internals
+*********
+
+* Branching from a non-stacked branch on a smart protocol is now
+  free of virtual file system methods.
+  (Robert Collins, Andrew Bennetts)
+
+* Branch and Repository creation on a bzr+ssh://server are now done
+  via RPC calls rather than VFS calls, reducing round trips for
+  pushing new branches substantially. (Robert Collins)
+
+* ``Branch.clone`` now takes the ``repository_policy`` formerly used
+  inside ``BzrDir.clone_on_transport``, allowing stacking to be
+  configured before the branch tags and revision tip are set. This
+  fixes a race condition cloning stacked branches that would cause
+  plugins to have hooks called on non-stacked instances.
+  (Robert Collins, #334187)
+
+* ``BzrDir.cloning_metadir`` now has a RPC call. (Robert Collins)
+
+* ``BzrDirFormat.__str__`` now uses the human readable description
+  rather than the sometimes-absent disk label. (Robert Collins)
+
+* ``bzrlib.fetch`` is now composed of a sender and a sink component
+  allowing for decoupling over a network connection. Fetching from
+  or into a RemoteRepository with a 1.13 server will use this to
+  stream the operation.
+  (Andrew Bennetts, Robert Collins)
+
+* ``bzrlib.tests.run_suite`` accepts a runner_class parameter
+  supporting the use of different runners. (Robert Collins)
+
+* Change how file_ids and revision_ids are interned as part of
+  inventory deserialization. Now we use the real ``intern()``, rather
+  than our own workaround that would also cache a Unicode copy of the
+  string, and never emptied the cache. This should slightly reduce
+  memory consumption. (John Arbash Meinel)
+
+* New branch method ``create_clone_on_transport`` that returns a
+  branch object. (Robert Collins)
+
+* New hook Commands['extend_command'] to allow plugins to access a
+  command object before the command is run (or help generated from
+  it), without overriding the command. (Robert Collins)
+
+* New version of the ``BzrDir.find_repository`` verb supporting
+  ``_network_name`` to support removing more _ensure_real calls.
+  (Robert Collins)
+
+* ``RemoteBranchFormat`` no longer claims to have a disk format string.
+  (Robert Collins)
+
+* ``Repository`` objects now have ``suspend_write_group`` and
+  ``resume_write_group`` methods.  These are currently only useful
+  with pack repositories. (Andrew Bennetts, Robert Collins)
+
+* ``BzrDirFormat``, ``BranchFormat`` and ``RepositoryFormat`` objects
+  now have a ``network_name`` for passing the format across RPC calls.
+  (Robert Collins, Andrew Bennetts)
+
+* ``RepositoryFormat`` objects now all have a new attribute
+  ``_serializer`` used by fetch when reserialising is required.
+  (Robert Collins, Andrew Bennetts)
+
+* Some methods have been pulled up from ``BzrBranch`` to ``Branch``
+  to aid branch types that are not bzr branch objects (like
+  RemoteBranch). (Robert Collins, Andrew Bennetts)
+
+* Test adaptation has been made consistent throughout the built in
+  tests. ``TestScenarioApplier``, ``multiply_tests_from_modules``,
+  ``adapt_tests``, ``adapt_modules`` have all been deleted. Please
+  use ``multiply_tests``, or for lower level needs ``apply_scenarios``
+  and ``apply_scenario``. (Robert Collins)
+
+* ``TestSkipped`` is now detected by TestCase and passed to the
+  ``TestResult`` by calling ``addSkip``. For older TestResult objects,
+  where ``addSkip`` is not available, ``addError`` is still called.
+  This permits test filtering in subunit to strip out skipped tests
+  resulting in a faster fix-shrink-list-run cycle. This is compatible
+  with the testtools protocol for skips. (Robert Collins)
+
+* The ``_index`` of ``KnitVersionedFiles`` now supports the ability
+  to scan an underlying index that is going to be incorporated into
+  the ``KnitVersionedFiles`` object, to determine if it has missing
+  delta references. The method is ``scan_unvalidated_index``.
+  (Andrew Bennetts, Robert Collins)
+
+* There is a RemoteSink object which handles pushing to smart servers.
+  (Andrew Bennetts, Robert Collins)
+
+* ``TransportTraceDecorator`` now logs ``put_bytes_non_atomic`` and
+  ``rmdir`` calls. (Robert Collins)
+
+* ``VersionedFiles`` record adapters have had their signature change
+  from ``(record, record.get_bytes_as(record.storage_kind))`` to
+  ``(record)`` reducing excess duplication and allowing adapters
+  to access private data in record to obtain content more
+  efficiently. (Robert Collins)
+
+* We no longer probe to see if we should create a working tree during
+  clone if we cannot get a local_abspath for the new bzrdir.
+  (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.14.txt'
--- doc/en/release-notes/bzr-1.14.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.14.txt	2010-10-08 08:15:14 +0000
@@ -0,0 +1,459 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.14
+########
+:Codename: brisbane-core
+:1.14rc1: 2009-04-06
+:1.14rc2: 2009-04-19
+:1.14: 2009-04-28
+:1.14.1: 2009-05-01
+
+New formats 1.14 and 1.14-rich-root supporting End-Of-Line (EOL) conversions,
+keyword templating (via the bzr-keywords plugin) and generic content filtering.
+End-of-line conversion is now supported for formats supporting content
+filtering.
+
+Changes from 1.14final to 1.14.1
+********************************
+
+* Change api_minimum_version back to api_minimum_version = (1, 13, 0)
+
+Changes from 1.14rc2 to 1.14final
+*********************************
+
+* Fix a bug in the pure-python ``GroupCompress`` code when handling copies
+  longer than 64KiB. (John Arbash Meinel, #364900)
+
+Changes from 1.14rc1 to 1.14rc2
+*******************************
+
+* Fix for bug 358037 Revision not in
+  bzrlib.groupcompress.GroupCompressVersionedFiles (Brian de Alwis, 
+  John A Meinel)
+
+* Fix for bug 354036 ErrorFromSmartServer - AbsentContentFactory object has no
+  attribute 'get_bytes_as' exception while pulling from Launchpad 
+  (Jean-Francois Roy, Andrew Bennetts, Robert Collins)
+
+* Fix for bug 355280 eol content filters are never loaded and thus never
+  applied (Brian de Alwis, Ian Clatworthy)
+ 
+* bzr.dev -r4280  Change _fetch_uses_deltas = False for CHK repos until we can
+  write a better fix. (John Arbash Meinel, Robert Collins)
+
+* Fix for bug 361574 uncommit recommends undefined --levels and -n options
+  (Marius Kruger, Ian Clatworthy)
+
+* bzr.dev r4289 as cherrypicked at lp:~spiv/bzr/stacking-cherrypick-1.14 
+  (Andrew Bennetts, Robert Collins)
+
+Compatibility Breaks
+********************
+
+* A previously disabled code path to accelerate getting configuration
+  settings from a smart server has been reinstated. We think this *may*
+  cause a incompatibility with servers older than bzr 0.15. We intend
+  to issue a point release to address this if it turns out to be a
+  problem. (Robert Collins, Andrew Bennetts)
+
+* bzr no longer autodetects nested trees as 'tree-references'.  They
+  must now be explicitly added tree references.  At the commandline, use
+  join --reference instead of add.  (Aaron Bentley)
+
+* The ``--long`` log format (the default) no longer shows merged
+  revisions implicitly, making it consistent with the ``short`` and
+  ``line`` log formats.  To see merged revisions for just a given
+  revision, use ``bzr log -n0 -rX``. To see every merged revision,
+  use ``bzr log -n0``.  (Ian Clatworthy)
+
+New Features
+************
+
+* New formats ``1.14`` and ``1.14-rich-root`` supporting End-Of-Line
+  (EOL) conversions, keyword templating (via the bzr-keywords plugin)
+  and generic content filtering. These formats replace the experimental
+  ``development-wt5`` and ``development-wt5-rich-root`` formats
+  respectively, but have support for filtered views disabled.
+  (Ian Clatworthy)
+
+* New ``mv --auto`` option recognizes renames after they occur.
+  (Aaron Bentley)
+
+* ``bzr`` can now get passwords from stdin without requiring a controlling
+  terminal (i.e. by redirecting stdin). (Vincent Ladeuil)
+
+* ``bzr log`` now supports filtering of multiple files and directories
+  and will show changes that touch any of them. Furthermore,
+  directory filtering now shows the changes to any children of that
+  directory, not just the directory object itself.
+  (Ian Clatworthy, #97715)
+
+* ``bzr shelve`` can now apply changes without storing anything on the
+  shelf, via the new --destroy option.  (Aaron Bentley)
+
+* ``bzr send`` now accepts --body to specify an initial message body.
+  (Aaron bentley)
+
+* ``bzr xxx --usage`` where xxx is a command now shows a usage
+  message and the options without the descriptive help sections
+  (like Description and Examples). A message is also given
+  explaining how to see the complete help, i.e. ``bzr help xxx``.
+  (Ian Clatworthy)
+
+* Content filters can now be used to provide custom conversion
+  between the canonical format of content (i.e. as stored) and
+  the convenience format of content (i.e. as created in working
+  trees). See ``bzr help content-filters`` for further details.
+  (Ian Clatworthy, Alexander Belchenko)
+
+* End-of-line conversion is now supported for formats supporting
+  content filtering. See ``bzr help eol`` for details.
+  (Ian Clatworthy)
+
+* Newly-blessed `join` command allows combining two trees into one.
+  (Aaron Bentley)
+
+Improvements
+************
+
+* A new format name alias ``default-rich-root`` has been added and
+  points at the closest relative of the default format that supports 
+  rich roots. (Jelmer Vernooij, #338061)
+
+* Branching from a stacked branch using ``bzr*://`` will now stream
+  the data when the target repository does not need topological
+  ordering, reducing round trips and network overhead. This uses the
+  existing smart server methods added in 1.13, so will work on any
+  1.13 or newer server. (Robert Collins, Andrew Bennetts)
+
+* ``bzr cat`` and ``bzr export`` now supports a ``--filters`` option
+  that displays/saves the content after content filters are applied.
+  (Ian Clatworthy)
+
+* ``bzr ignore`` gives a more informative message when existing
+  version controlled files match the ignore pattern. (Neil
+  Martinsen-Burrell, #248895)
+
+* ``bzr log`` now has ``--include-merges`` as an alias for ``--levels 0``.
+  (Ian Clatworthy)
+
+* ``bzr send`` is faster on repositories with deep histories.
+  (Ian Clatworthy)
+
+* IPv6 literals are accepted in URLs.
+  (stlman, Martin Pool, Jelmer Vernooij, #165014)
+
+* Progress bars now show the rate of network activity for
+  ``bzr+ssh://`` and ``bzr://`` connections.  (Andrew Bennetts)
+
+* Prompt for user names if they are not in the configuration. 
+  (Jelmer Vernooij, #256612)
+
+* Pushing to a stacked pack-format branch on a 1.12 or older smart server
+  now takes many less round trips.  (Andrew Bennetts, Robert Collins,
+  #294479)
+  
+* Streaming push can be done to older repository formats.  This is
+  implemented using a new ``Repository.insert_stream_locked`` RPC.
+  (Andrew Bennetts, Robert Collins)
+
+* The "ignoring files outside view: .." message has been re-worded
+  to "Ignoring files outside view. View is .." to reduce confusion
+  about what was being considered and what was being ignored.
+  (Ian Clatworthy)
+
+* The ``long`` log formatter now shows [merge] indicators. If
+  only one level of revisions is displayed and merges are found,
+  the ``long`` and ``short`` log formatters now tell the user
+  how to see the hidden merged revisions.  (Ian Clatworthy)
+
+* The ``brisbane-core`` project has delivered its beta format
+  ``development6-rich-root``. This format is suitable for judicious
+  testing by early adopters. In particular if you are benchmarking bzr
+  performance please be sure to test using this format. At this stage
+  more information is best obtained by contacting the Bazaar mailing list
+  or IRC channel if you are interested in using this format. We will make
+  end user documentation available closer to blessing the format as
+  production ready. (Robert Collins, John Arbash Meinel, Ian Clatworthy,
+  Vincent Ladeuil, Andrew Bennetts, Martin Pool)
+
+* Tildes are no longer escaped. No more %7Euser/project/branch!
+  (Jonathan Lange)
+
+Bug Fixes
+*********
+
+* Pushing a new stacked branch will also push the parent inventories for
+  revisions at the stacking boundary.  This makes sure that the stacked
+  branch has enough data to calculate inventory deltas for all of its
+  revisions (without requiring the fallback branch).  This avoids
+  "'AbsentContentFactory' object has no attribute 'get_bytes_as'" errors
+  when fetching the stacked branch from a 1.13 (or later) smart server.
+  This partially fixes #354036.  (Andrew Bennetts, Robert Collins)
+
+* End-Of-Line content filters are now loaded correctly.
+  (Ian Clatworthy, Brian de Alwis, #355280)
+
+* Authentication plugins now receive all the parameters from the request
+  itself (aka host, port, realm, path, etc). Previously, only the 
+  authentication section name, username and encoded password were 
+  provided. (Jean-Francois Roy)
+
+* bzr gives a better message if an invalid regexp is passed to ``bzr log
+  -m``.  (Anne Mohsen, Martin Pool)
+
+* ``bzr split`` now says "See also: join" (Aaron Bentley, #335015)
+
+* ``bzr version-info`` now works in empty branches. (Jelmer Vernooij,
+  #313028)
+
+* Fix "is not a stackable format" error when pushing a
+  stackable-format branch with an unstackable-format repository to a
+  destination with a default stacking policy.  (Andrew Bennetts)
+
+* Fixed incorrect "Source format does not support stacking" warning
+  when pushing to a smart server.  (Andrew Bennetts, #334114)
+
+* Fix 'make check-dist-tarball' failure by converting paths to unicode when
+  needed. (Vincent Ladeuil, #355454)
+
+* Fixed "Specified file 'x/y/z' is outside current view: " occurring
+  on ``bzr add x/y/z`` in formats supporting views when no view is
+  defined.  (Ian Clatworthy, #344708)
+
+* It is no longer possible to fetch between repositories while the
+  target repository is in a write group. This prevents race conditions
+  that prevent the use of RPC's to perform fetch, and thus allows
+  optimising more operations. (Robert Collins, Andrew Bennetts)
+
+* ``merge --force`` works again. (Robert Collins, #342105)
+
+* No more warnings are issued about ``sha`` being deprecated under python-2.6.
+  (Vincent Ladeuil, #346593)
+
+* Pushing a new branch to a server that has a stacking policy will now
+  upgrade from the local branch format when the stacking policy points at
+  a branch which is itself stackable, because we know the client can read
+  both branches, we know that the trunk for the project can be read too,
+  so the upgrade will not inconvenience users. (Robert Collins, #345169)
+
+* Pushing a new stacked branch will also push the parent inventories for
+  revisions at the stacking boundary.  This makes sure that the stacked
+  branch has enough data to calculate inventory deltas for all of its
+  revisions (without requiring the fallback branch).  This avoids
+  "'AbsentContentFactory' object has no attribute 'get_bytes_as'" errors
+  when fetching the stacked branch from a 1.13 (or later) smart server.
+  This partially fixes #354036.  (Andrew Bennetts, Robert Collins)
+
+* The full test suite is passing again on OSX. Several minor issues (mostly
+  test related) have been fixed. (Vincent Ladeuil, #355273).
+
+* The GNU Changelog formatter is slightly improved in the case where
+  the delta is empty, and now correctly claims not to support tags.
+  (Andrea Bolognani)
+
+* Shelve can now shelve changes to a symlink target.
+  (James Westby, #341558)
+
+* The help for the ``info`` command has been corrected.
+  (Ian Clatworthy, #351931)
+
+* Upgrade will now use a sensible default format if the source repository
+  uses rich roots.  (Jelmer Vernooij, #252908)
+
+Documentation
+*************
+
+* Expanded the index of the developer documentation. (Eric Siegerman)
+
+* New topic `bzr help debug-flags`.  (Martin Pool)
+
+* The generated manpage now explicitly lists aliases as commands.
+  (James Westby, #336998)
+
+API Changes
+***********
+
+* APIs deprecated in 1.6 and previous versions of bzr are now removed.
+  (Martin Pool)
+
+* ``CommitReporter`` is no longer called with ``unchanged`` status during
+  commit - this was a full-tree overhead that bzr no longer performs.
+  (Robert Collins)
+
+* New abstract ``UIFactory`` method ``get_username`` which will be called to 
+  obtain the username to use when connecting to remote machines. 
+  (Jelmer Vernooij)
+
+* New API ``Inventory.filter()`` added that filters an inventory by
+  a set of file-ids so that only those fileids, their parents and
+  their children are included.  (Ian Clatworthy)
+
+* New sort order for ``get_record_stream`` ``groupcompress`` which
+  sorts optimally for use with groupcompress compressors. (John Arbash
+  Meinel, Robert Collins)
+
+* Repository APIs ``get_deltas_for_revisions()`` and
+  ``get_revision_delta()`` now support an optional ``specific_fileids``
+  parameter. If provided, the deltas are filtered so that only those
+  file-ids, their parents and their children are included.
+  (Ian Clatworthy)
+
+* The ``get_credentials`` and ``set_credentials`` methods of 
+  ``AuthenticationConfig`` now accept an optional realm argument.
+  (Jean-Francois Roy)
+
+* The ``pb`` argument to ``fetch()`` is deprecated.
+  (Martin Pool)
+
+* The ``Serializer`` class and the serializer ``format registry`` have moved
+  from ``bzrlib.xml_serializer`` to ``bzrlib.serializer``. (Jelmer Vernooij)
+
+* The smart server jail now hooks into BzrDir.open to prevent any BzrDir
+  that is not inside the backing transport from being opened.  See the
+  module documentation for ``bzrlib.smart.request`` for details.
+  (Andrew Bennetts, Robert Collins)
+
+* ``Tree.get_symlink_target`` now always returns a unicode string result
+  or None. Previously it would return the bytes from reading the link
+  which could be in any arbitrary encoding. (Robert Collins)
+
+Testing
+*******
+
+* ``bzrlib.tests.TestCase`` now fails the test if its own ``setUp``
+  and ``tearDown`` weren't called.  This catches faulty tests that
+  forget to upcall when overriding ``setUp`` and ``tearDown``.  Those
+  faulty tests were not properly isolated.
+  (Andrew Bennetts, Robert Collins)
+
+* Fix test_msgeditor.MsgEditorTest test isolation.
+  (Vincent Ladeuil, #347130)
+
+* ``medusa`` is not used anymore as an FTP test server starting with
+  python2.6. A new FTP test server based on ``pyftplib`` can be used
+  instead. This new server is a soft dependency as medusa which is still
+  preferred if both are available (modulo python version).
+  (Vincent Ladeuil)
+
+Internals
+*********
+
+* Added ``chk_map`` for fast, trie-based storage of tuple to string maps.
+  (Robert Collins, John Arbash Meinel, Vincent Ladeuil)
+
+* Added ``bzrlib.chk_map`` for fast, trie-based storage of tuple to string
+  maps.  (Robert Collins, John Arbash Meinel, Vincent Ladeuil)
+
+* Added ``bzrlib.inventory_delta`` module.  This will be used for
+  serializing and deserializing inventory deltas for more efficient
+  streaming on the network.  (Robert Collins, Andrew Bennetts)
+
+* ``Branch._get_config`` has been added, which splits out access to the
+  specific config file from the branch. This is used to let RemoteBranch
+  avoid constructing real branch objects to access configuration settings.
+  (Robert Collins, Andrew Bennetts)
+
+* ``Branch`` now implements ``set_stacked_on_url`` in the base class as
+  the implementation is generic and should impact foreign formats. This
+  helps performance for ``RemoteBranch`` push operations to new stacked
+  branches. (Robert Collins, Andrew Bennetts)
+
+* ``BtreeIndex._spill_mem_keys_to_disk()`` now generates disk index with
+  optmizations turned off. This only has effect when processing > 100,000
+  keys during something like ``bzr pack``. (John Arbash Meinel)
+
+* ``bzr selftest`` now accepts ``--subunit`` to run in subunit output
+  mode. Requires ``lp:subunit`` installed to work, but is not a hard
+  dependency. (Robert Collins)
+
+* ``BzrDir.open_branch`` now takes an optional ``ignore_fallbacks``
+  parameter for controlling opening of stacked branches.
+  (Andrew Bennetts, Robert Collins)
+  
+* ``CommitBuilder`` has a new method, ``record_iter_changes`` which works
+  in terms of an iter_changes iterator rather than full tree scanning.
+  (Robert Collins)
+
+* ``DirState`` can now be passed a custom ``SHA1Provider`` object
+  enabling it to store the SHA1 and stat of the canonical (post
+  content filtered) form. (Ian Clatworthy)
+
+* New ``assertLength`` method based on one Martin has squirreled away
+  somewhere. (Robert Collins, Martin Pool)
+
+* New hook ``BzrDir.pre_open`` which runs before opening ``BzrDir``
+  objects, allowing better enforcement of the smart server jail when
+  dealing with stacked branches. (Robert Collins, Andrew Bennetts)
+
+* New hook ``RioVersionInfoBuilder.revision``, allowing extra entries 
+  to be added to the stanza that is printed for a particular revision.
+  (Jelmer Vernooij)
+
+* New repository method ``refresh_data`` to cause any repository to
+  make visible data inserted into the repository by a smart server
+  fetch operation. (Robert Collins, Andrew Bennetts)
+
+* ``register_filter_stack_map`` now takes an optional fallback parameter,
+  a callable to invoke if a preference has a value not in the map
+  of filter stacks. This enhancement allows, for example,  bzr-svn to
+  handle existing svn properties that define a list of keywords to be
+  expanded.  (Ian Clatworthy)
+
+* ``RemoteBranchConfig`` will use a new verb ``Branch.set_config_option``
+  to write config settings to smart servers that support this, saving
+  5 round trips on the stacked streaming acceptance test.
+  (Robert Collins, Andrew Bennetts)
+
+* ``RemoteBranch`` now provides ``_get_config`` for access to just the
+  branch specific configuration from a remote server, which uses the 
+  already existing ``Branch.get_config_file`` smart verb.
+  (Robert Collins, Andrew Bennetts)
+
+* ``RemoteRepository`` will now negatively cache missing revisions during
+  ``get_parent_map`` while read-locked. Write-locks are unaffected.
+  (Robert Collins, Andrew Bennetts)
+
+* Removed ``InterRemoteToOther``, ``InterOtherToRemote`` and
+  ``InterPackToRemotePack`` classes, as they are now unnecessary.
+  (Andrew Bennetts)
+
+* ``RepositoryFormat`` as a new attribute ``fast_deltas`` to indicate
+  whether the repository can efficiently generate deltas between trees
+  regardless of tree size. (Robert Collins)
+
+* ``Repository.iter_files_bytes()`` now properly returns an "iterable of
+  byte strings" (aka 'chunked') for the content. It previously was
+  returning a plain string, which worked, but performed very poorly when
+  building a working tree (file.writelines(str) is very inefficient). This
+  can have a large effect on ``bzr checkout`` times. (John Arbash Meinel)
+
+* selftest now supports a --parallel option, with values of 'fork' or
+  'subprocess' to run the test suite in parallel. Currently only Linux
+  machines work, other platforms need patches submitted. (Robert Collins,
+  Vincent Ladeuil)
+
+* ``tests.run_suite`` has a new parameter ``suite_decorators``, a list of 
+  callables to use to decorate the test suite. Such decorators can add or
+  remove tests, or even remote the test suite to another machine if
+  desired. (Robert Collins)
+
+* The smart server verb ``Repository.get_parent_map`` can now include
+  information about ghosts when the special revision ``include-missing:``
+  is in the requested parents map list. With this flag, ghosts are
+  included as ``missing:REVISION_ID``. (Robert Collins, Andrew Bennetts)
+
+* ``_walk_to_common_revisions`` will now batch up at least 50
+  revisions before calling ``get_parent_map`` on the target,
+  regardless of ``InterRepository``.
+  (Andrew Bennetts, Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix
+

=== added file 'doc/en/release-notes/bzr-1.15.txt'
--- doc/en/release-notes/bzr-1.15.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.15.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,230 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.15
+########
+:1.15rc1: 2009-05-16
+:1.15: 2009-05-22
+:1.15.1: 2009-06-09
+
+The smart server will no longer raise 'NoSuchRevision' when streaming content
+with a size mismatch in a reconstructed graph search. New command ``bzr
+dpush``. Plugins can now define their own annotation tie-breaker when two
+revisions introduce the exact same line.
+
+Changes from 1.15.1 to 1.15.2
+*****************************
+
+* Use zdll on Windows to build ``_chk_map_pyx`` extension.
+  (Alexander Belchenko)
+
+Changes from 1.15final to 1.15.1
+*********************************
+
+* Translate errors received from a smart server in response to a
+  ``BzrDirFormat.initialize`` or ``BzrDirFormat.initialize_ex`` request.
+  This was causing tracebacks even for mundane errors like
+  ``PermissionDenied``.  (Andrew Bennetts, #381329)
+
+Changes from 1.15rc1 to 1.15final
+*********************************
+
+* No changes
+
+Compatibility Breaks
+********************
+
+* ``bzr ls`` is no longer recursive by default. To recurse, use the
+  new ``-R`` option. The old ``--non-recursive`` option has been removed.
+  If you alias ``ls`` to ``ls -R``, you can disable recursion using
+  ``--no-recursive`` instead.  (Ian Clatworthy)
+
+New Features
+************
+
+* New command ``bzr dpush`` that can push changes to foreign 
+  branches (svn, git) without setting custom bzr-specific metadata.
+  (Jelmer Vernooij)
+
+* The new development format ``--development6-rich-root`` now supports
+  stacking. We chose not to use a new format marker, since old clients
+  will just fail to open stacked branches, the same as if we used a new
+  format flag. (John Arbash Meinel, #373455)
+
+* Plugins can now define their own annotation tie-breaker when two revisions
+  introduce the exact same line. See ``bzrlib.annotate._break_annotation_tie``
+  Be aware though that this is temporary, private (as indicated by the leading
+  '_') and a first step to address the problem. (Vincent Ladeuil, #348459)
+
+* New command ``bzr dpush`` that can push changes to foreign 
+  branches (svn, git) without setting custom bzr-specific metadata.
+  (Jelmer Vernooij)
+
+* ``bzr send`` will now check the ``child_submit_format`` setting in
+  the submit branch to determine what format to use, if none was 
+  specified on the command-line.  (Jelmer Vernooij)
+
+Improvements
+************
+
+* -Dhpss output now includes the number of VFS calls made to the remote
+  server. (Jonathan Lange)
+
+* ``--coverage`` works for code running in threads too.
+  (Andrew Bennets, Vincent Ladeuil)
+
+* ``bzr pull`` now has a ``--local`` option to only make changes to the
+  local branch, and not the bound master branch.
+  (Gary van der Merwe, #194716)
+
+* ``bzr rm *`` is now as fast as ``bzr rm * --keep``. (Johan Walles, #180116)
+
+Bug Fixes
+*********
+
+* Adding now works properly when path contains a symbolic link.
+  (Geoff Bache, #183831)
+
+* An error is now raised for unknown eol values. (Brian de Alwis, #358199)
+
+* ``bzr merge --weave`` will now generate a conflict if one side deletes a
+  line, and the other side modifies the line. (John Arbash Meinel, #328171)
+
+* ``bzr reconfigure --standalone`` no longer raises IncompatibleRepositories.
+  (Martin von Gagern, #248932)
+
+* ``bzr send`` works to send emails again using MAPI.
+  (Neil Martinsen-Burrell, #346998)
+
+* Check for missing parent inventories in StreamSink.  This prevents
+  incomplete stacked branches being created by 1.13 bzr:// and
+  bzr+ssh:// clients (which have bug #354036).  Instead, the server now
+  causes those clients to send the missing records.  (Andrew Bennetts)
+
+* Correctly handle HTTP servers proposing multiple authentication schemes.
+  (Vincent Ladeuil, #366107)
+
+* End-Of-Line content filters are now loaded correctly.
+  (Ian Clatworthy, Brian de Alwis, #355280)
+
+* Fix a bug in the pure-python ``GroupCompress`` code when handling copies
+  longer than 64KiB. (John Arbash Meinel, #364900)
+
+* Fix TypeError in running ``bzr break-lock`` on some URLs.
+  (Alexander Belchenko, Martin Pool, #365891)
+
+* Non-recursive ``bzr ls`` now works properly when a path is specified.
+  (Jelmer Vernooij, #357863)
+
+* SSH usernames (defined in ~/.ssh/config) are honoured for bzr+ssh connections.
+  (Vincent Ladeuil, #367726)
+
+* Several bugs related to unicode symlinks have been fixed and the test suite
+  enhanced to better catch regressions for them. (Vincent Ladeuil)
+
+* The smart server will no longer raise 'NoSuchRevision' when streaming
+  content with a size mismatch in a reconstructed graph search: it assumes
+  that the client will make sure it is happy with what it got, and this
+  sort of mismatch is normal for stacked environments.
+  bzr 1.13.0/1 will stream from unstacked branches only - in that case not
+  getting all the content expected would be a bug. However the graph
+  search is how we figured out what we wanted, so a mismatch is both odd
+  and unrecoverable without starting over, and starting over will end up
+  with the same data as if we just permitted the mismatch. If data is
+  gc'd, doing a new search will find only the truncated data, so sending
+  only the truncated data seems reasonable. bzr versions newer than this
+  will stream from stacked branches and check the stream to find missing
+  content in the stacked-on branch, and thus will handle the situation
+  implicitly.  (Robert Collins, #360791)
+
+* Upgrading to, or fetching into a 'rich-root' format will now correctly
+  set the root data the same way that reconcile does.
+  (Robert Collins, #368921)
+
+* Using unicode Windows API to obtain command-line arguments.
+  (Alexander Belchenko, #375934)
+
+Documentation
+*************
+
+API Changes
+***********
+
+* ``InterPackRepo.fetch`` and ``RepoFetcher`` now raise ``NoSuchRevision``
+  instead of ``InstallFailed`` when they detect a missing revision.
+  ``InstallFailed`` itself has been deleted. (Jonathan Lange)
+
+* Not passing arguments to ``bzrlib.commands.main()`` will now grab the
+  arguments from ``osutils.get_unicode_argv()`` which has proper support
+  for unicode arguments on windows. Further, the supplied arguments are now 
+  required to be unicode strings, rather than user_encoded strings.
+  (Alexander Belchenko)
+
+Internals
+*********
+
+* ``bzrlib.branch.Branch.set_parent`` is now present on the base branch
+  class and will call ``_set_parent_location`` after doing unicode 
+  encoding. (Robert Collins)
+
+* ``bzrlib.remote.RemoteBranch._set_parent_location`` will use a new verb
+  ``Branch.set_parent_location`` removing further VFS operations.
+  (Robert Collins)
+
+* ``bzrlib.bzrdir.BzrDir._get_config`` now returns a ``TransportConfig``
+  or similar when the dir supports configuration settings. The base class
+  defaults to None. There is a matching new server verb
+  ``BzrDir.get-config_file`` to reduce roundtrips for getting BzrDir
+  configuration. (Robert Collins)
+
+* ``bzrlib.tests.ExtendedTestResult`` has new methods ``startTests``
+  called before the first test is started, ``done`` called after the last
+  test completes, and a new parameter ``strict``. (Robert Collins)
+
+* ``-Dhpss`` when passed to bzr will cause a backtrace to be printed when
+  VFS operations are started on a smart server repository. This should not
+  occur on regular push and pull operations, and is a key indicator for
+  performance regressions. (Robert Collins)
+
+* ``-Dlock`` when passed to the selftest (e.g. ``bzr -Dlock selftest``) will
+  cause mismatched physical locks to cause test errors rather than just
+  reporting to the screen. (Robert Collins)
+
+* -Dprogress will cause pdb to start up if a progress view jumps
+  backwards. (Robert Collins)
+
+* Fallback ``CredentialStore`` instances registered with ``fallback=True``
+  are now be able to provide credentials if obtaining credentials 
+  via ~/.bazaar/authentication.conf fails. (Jelmer Vernooij, 
+  Vincent Ladeuil, #321918)
+
+* New hook ``Lock.lock_broken`` which runs when a lock is
+  broken. This is mainly for testing that lock/unlock are
+  balanced in tests. (Vincent Ladeuil)
+
+* New MergeDirective hook 'merge_request_body' allows hooks to supply or
+  alter a body for the message produced by ``bzr send``.
+
+* New smart server verb ``BzrDir.initialize_ex`` which implements a
+  refactoring to the core of clone allowing less round trips on new
+  branches. (Robert Collins)
+
+* New method ``Tags.rename_revisions`` that can rename revision ids tags
+  are pointing at. (Jelmer Vernooij)
+
+* Updated the bundled ``ConfigObj`` library to 4.6.0 (Matt Nordhoff)
+
+Testing
+*******
+
+* ``bzr selftest`` will now fail if lock/unlock are not correctly balanced in
+  tests. Using ``-Dlock`` will turn the related failures into warnings.
+  (Vincent Ladeuil, Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.16.txt'
--- doc/en/release-notes/bzr-1.16.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.16.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,269 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.16.1
+##########
+
+:Released: 2009-06-26
+
+End user testing of the 2a format revealed two serious bugs. The first,
+#365615, caused bzr to raise AbsentContentFactory errors when autopacking.
+This meant that commits or pushes to 2a-format repositories failed
+intermittently.
+
+The second bug, #390563, caused the smart server to raise AbsentContentFactory
+when streaming 2a stacked 2a-format branches. This particularly affected
+branches stored on Launchpad in the 2a format.
+
+Both of these bugs cause command failures only, neither of them cause data
+corruption or data loss. And, of course, both of these bugs are now fixed.
+
+Bug Fixes
+*********
+
+* We now properly request a more minimal set of file texts when fetching
+  multiple revisions. (Robert Collins, John Arbash Meinel, #390563)
+
+* Repositories using CHK pages (which includes the new 2a format) will no
+  longer error during commit or push operations when an autopack operation
+  is triggered. (Robert Collins, #365615)
+
+* ``chk_map.iter_interesting_nodes`` now properly uses the *intersection*
+  of referenced nodes rather than the *union* to determine what
+  uninteresting pages we still need to look at. Prior to this,
+  incrementally pushing to stacked branch would push the minimal data, but
+  fetching everything would request extra texts. There are some unhandled
+  cases wrt trees of different depths, but this fixes the common cases.
+  (Robert Collins, John Arbash Meinel, #390563)
+
+* ``GroupCompress`` repositories now take advantage of the pack hints
+  parameter to permit cross-format fetching to incrementally pack the
+  converted data. (Robert Collins)
+
+* ``Repository.commit_write_group`` now returns opaque data about what
+  was committed, for passing to the ``Repository.pack``. Repositories
+  without atomic commits will still return None. (Robert Collins)
+
+* ``Repository.pack`` now takes an optional ``hint`` parameter
+  which will support doing partial packs for repositories that can do
+  that. (Robert Collins)
+
+* RepositoryFormat has a new attribute 'pack_compresses' which is True
+  when doing a pack operation changes the compression of content in the
+  repository. (Robert Collins)
+
+* ``StreamSink`` and ``InterDifferingSerialiser`` will call
+  ``Repository.pack`` with the hint returned by
+  ``Repository.commit_write_group`` if the formats were different and the
+  repository can increase compression by doing a pack operation.
+  (Robert Collins, #376748)
+
+
+bzr 1.16
+########
+:Codename: yesterday-in-california
+:1.16rc1: 2009-06-11
+:1.16: 2009-06-18
+
+This version of Bazaar contains the beta release of the new ``2a`` repository
+format, suitable for testing by fearless, advanced users. This format or an
+updated version of it will become the default format in Bazaar 2.0. Please
+read the NEWS entry before even thinking about upgrading to the new format.
+
+Also included are speedups for many operations on huge projects, a bug fix for
+pushing stacked new stacked branches to smart servers and the usual bevy of
+bug fixes and improvements.
+
+
+Changes from 1.16rc1 to 1.16final
+*********************************
+
+* Fix the nested tree flag check so that upgrade from development formats to
+  2a can work correctly.
+  (Jelmer Vernooij, #388727)
+
+* Automatic format upgrades triggered by default stacking policies on a
+  1.16rc1 (or later) smart server work again.
+  (Andrew Bennetts, #388675)
+
+
+Compatibility Breaks
+********************
+
+* Display prompt on stderr (instead of stdout) when querying users so
+  that the output of commands can be safely redirected.
+  (Vincent Ladeuil, #376582)
+
+
+New Features
+************
+
+* A new repository format ``2a`` has been added.  This is a beta release
+  of the brisbane-core (aka group-compress) project.  This format now
+  suitable for wider testing by advanced users willing to deal with some
+  bugs.  We would appreciate test reports, either positive or negative.
+  Format 2a is substantially smaller and faster for many operations on
+  many trees.  This format or an updated version will become the default
+  in bzr 2.0.
+
+  This is a rich-root format, so this repository format can be used with
+  bzr-svn.  Bazaar branches in previous non-rich-root formats can be
+  converted (including by merge, push and pull) to format 2a, but not vice
+  versa.  We recommend upgrading previous development formats to 2a.
+
+  Upgrading to this format can take considerable time because it expands
+  and more concisely repacks the full history.
+
+  If you use stacked branches, you must upgrade the stacked branches
+  before the stacked-on branches.  (See <https://bugs.launchpad.net/bugs/374735>)
+
+* ``--development7-rich-root`` is a new dev format, similar to ``--dev6``
+  but using a Revision serializer using bencode rather than XML.
+  (Jelmer Vernooij, John Arbash Meinel)
+
+* mail_client=claws now supports --body (and message body hooks).  Also uses
+  configured from address.  (Barry Warsaw)
+
+Improvements
+************
+
+
+* ``--development6-rich-root`` can now stack. (Modulo some smart-server
+  bugs with stacking and non default formats.)
+  (John Arbash Meinel, #373455)
+
+* ``--development6-rich-root`` delays generating a delta index for the
+  first object inserted into a group. This has a beneficial impact on
+  ``bzr commit`` since each committed texts goes to its own group. For
+  committing a 90MB file, it drops peak memory by about 200MB, and speeds
+  up commit from 7s => 4s. (John Arbash Meinel)
+
+* Numerous operations are now faster for huge projects, i.e. those
+  with a large number of files and/or a large number of revisions,
+  particularly when the latest development format is used. These
+  operations (and improvements on OpenOffice.org) include:
+
+  * branch in a shared repository (2X faster)
+  * branch --no-tree (100X faster)
+  * diff (2X faster)
+  * tags (70X faster)
+
+  (Ian Clatworthy)
+
+* Pyrex version of ``bencode`` support. This provides optimized support
+  for both encoding and decoding, and is now found at ``bzrlib.bencode``.
+  ``bzrlib.utils.bencode`` is now deprecated.
+  (Alexander Belchenko, Jelmer Vernooij, John Arbash Meinel)
+
+
+Bug Fixes
+*********
+
+* Bazaar can now pass attachment files to the mutt email client.
+  (Edwin Grubbs, #384158)
+
+* Better message in ``bzr add`` output suggesting using ``bzr ignored`` to
+  see which files can also be added.  (Jason Spashett, #76616)
+
+* ``bzr pull -r 123`` from a stacked branch on a smart server no longer fails.
+  Also, the ``Branch.revision_history()`` API now works in the same
+  situation.  (Andrew Bennetts, #380314)
+  
+* ``bzr serve`` on Windows no longer displays a traceback simply because a
+  TCP client disconnected. (Andrew Bennetts)
+
+* Clarify the rules for locking and fallback repositories. Fix bugs in how
+  ``RemoteRepository`` was handling fallbacks along with the
+  ``_real_repository``. (Andrew Bennetts, John Arbash Meinel, #375496)
+
+* Fix a small bug with fetching revisions w/ ghosts into a new stacked
+  branch. Not often triggered, because it required ghosts to be part of
+  the fetched revisions, not in the stacked-on ancestry.
+  (John Arbash Meinel)
+
+* Fix status and commit to work with content filtered trees, addressing
+  numerous bad bugs with line-ending support. (Ian Clatworthy, #362030)
+
+* Fix problem of "directory not empty" when contending for a lock over
+  SFTP.  (Martin Pool, #340352)
+
+* Fix rule handling so that eol is optional, not mandatory.
+  (Ian Clatworthy, #379370)
+
+* Pushing a new stacked branch to a 1.15 smart server was broken due to a
+  bug in the ``BzrDirFormat.initialize_ex`` smart verb.  This is fixed in
+  1.16, but required changes to the network protocol, so the
+  ``BzrDirFormat.initialize_ex`` verb has been removed and replaced with a
+  corrected ``BzrDirFormat.initialize_ex_1.16`` verb.  1.15 clients will
+  still work with a 1.16 server as they will fallback to slower (and
+  bug-free) methods.
+  (Jonathan Lange, Robert Collins, Andrew Bennetts, #385132)
+
+* Reconcile can now deal with text revisions that originated in revisions 
+  that are ghosts. (Jelmer Vernooij, #336749)
+
+* Support cloning of branches with ghosts in the left hand side history.
+  (Jelmer Vernooij, #248540)
+
+* The ''bzr diff'' now catches OSError from osutils.rmtree and logs a
+  helpful message to the trace file, unless the temp directory really was
+  removed (which would be very strange).  Since the diff operation has
+  succeeded from the user's perspective, no output is written to stderr 
+  or stdout.  (Maritza Mendez, #363837)
+
+* Translate errors received from a smart server in response to a
+  ``BzrDirFormat.initialize`` or ``BzrDirFormat.initialize_ex`` request.
+  This was causing tracebacks even for mundane errors like
+  ``PermissionDenied``.  (Andrew Bennetts, #381329)
+
+Documentation
+*************
+
+* Added directory structure and started translation of docs in Russian.
+  (Alexey Shtokalo, Alexander Iljin, Alexander Belchenko, Dmitry Vasiliev,
+  Volodymyr Kotulskyi)
+
+API Changes
+***********
+
+* Added osutils.parent_directories(). (Ian Clatworthy)
+
+* ``bzrlib.progress.ProgressBar``, ``ChildProgress``, ``DotsProgressBar``,
+  ``TTYProgressBar`` and ``child_progress`` are now deprecated; use
+  ``ui_factory.nested_progress_bar`` instead.  (Martin Pool)
+
+* ``graph.StackedParentsProvider`` is now a public API, replacing
+  ``graph._StackedParentsProvider``. The api is now considered stable and ready
+  for external users. (Gary van der Merwe)
+
+* ``bzrlib.user_encoding`` is deprecated in favor of
+  ``get_user_encoding``.  (Alexander Belchenko)
+
+* TreeTransformBase no longer assumes that limbo is provided via disk.
+  DiskTreeTransform now provides disk functionality.  (Aaron Bentley)
+
+Internals
+*********
+
+* Remove ``weave.py`` script for accessing internals of old weave-format
+  repositories.  (Martin Pool)
+
+Testing
+*******
+
+* ``make check`` no longer repeats the test run in ``LANG=C``.
+  (Martin Pool, #386180)
+
+* The number of cores is now correctly detected on OSX. (John Szakmeister)
+
+* The number of cores is also detected on Solaris and win32. (Vincent Ladeuil)
+
+* The number of cores is also detected on FreeBSD. (Matthew Fuller)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.17.txt'
--- doc/en/release-notes/bzr-1.17.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.17.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,259 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.17
+########
+:Codename: so-late-its-brunch
+:1.17rc1: 2009-07-13
+:1.17: 2009-07-20
+
+
+Bazaar continues to blaze a straight and shining path to the 2.0 release and
+the elevation of the ``2a`` beta format to the full glory of "supported and
+stable".
+
+Highlights in this release include greatly reduced memory consumption during
+commits, faster ``ls``, faster ``annotate``, faster network operations if
+you're specifying a revision number and the final destruction of those
+annoying progress bar artifacts.
+
+
+Changes from 1.17rc1 to 1.17final
+*********************************
+
+* Change an extension to call the python ``frozenset()`` rather than the C
+  api ``PyFrozenSet_New``. It turns out that python2.4 did not expose the
+  C api. (John Arbash Meinel, #399366)
+
+* Fixes for the Makefile and the rename of ``generate_docs.py`` to
+  ``tools/generate_docs.py`` to allow everything to be built on Windows.
+  (John Arbash Meinel, #399356)
+
+* ``bzr serve`` once again applies a ``ChrootServer`` to the given
+  directory before serving it. (Andrew Bennetts, #400535)
+
+
+Compatibility Breaks
+********************
+
+* ``bzr register-branch`` from the Launchpad plugin now refers to "project"
+  instead of "product" which is the correct Launchpad terminology.  The
+  --product option is deprecated and users should switch to using --project.
+  (Neil Martinsen-Burrell, #238764)
+
+
+New Features
+************
+
+* ``bzr push`` now aborts if uncommitted changes (including pending merges)
+  are present in the working tree (if one is present) and no revision is
+  specified. The configuration option ``push_strict`` can be used to set the
+  default for this behavior.  (Vincent Ladeuil, #284038, #322808, #65286)
+
+* ``bzr revno`` and ``bzr revision-info`` now have a ``--tree`` option to
+  show revision info for the working tree instead of the branch.
+  (Matthew Fuller, John Arbash Meinel)
+
+* ``bzr send`` now aborts if uncommitted changes (including pending merges)
+  are present in the working tree and no revision is specified. The
+  configuration option ``send_strict`` can be used to set the default for this
+  behavior.
+  (Vincent Ladeuil, #206577)
+
+* ``bzr switch --create-branch/-b`` can now be used to create and switch
+  to a new branch. Supplying a name without a ``/`` will create the branch
+  relative to the existing branch. (similar to how ``bzr switch name``
+  works when the branch already exists.) (John Arbash Meinel)
+
+
+Bug Fixes
+*********
+
+* Accept uppercase "Y/N" to prompts such as from break lock. 
+  (#335182, Tim Powell, Martin Pool)
+
+* Add documentation about diverged branches and how to fix them in the
+  centralized workflow with local commits.  Mention ``bzr help
+  diverged-branches`` when a push fails because the branches have
+  diverged.  (Neil Martinsen-Burrell, #269477)
+
+* Annotate would sometimes 'latch on' to trivial lines, causing important
+  lines to be incorrectly annotated. (John Arbash Meinel, #387952)
+
+* Automatic format upgrades triggered by default stacking policies on a
+  1.16rc1 (or later) smart server work again.
+  (Andrew Bennetts, #388675)
+
+* Avoid progress bar artifacts being left behind on the screen.
+  (Martin Pool, #321935)
+
+* Better message in ``bzr split`` error suggesting a rich root format.
+  (Neil Martinsen-Burrell, #220067)
+
+* ``Branch.set_append_revisions_only`` now works with branches on a smart
+  server. (Andrew Bennetts, #365865)
+
+* By default, ``bzr branch`` will fail if the target directory exists, but
+  does not already have a control directory.  The flag ``--use-existing-dir``
+  will allow operation to proceed.  (Alexander Belchenko, #307554)
+
+* ``bzr ls DIR --from-root`` now shows only things in DIR, not everything.
+  (Ian Clatworthy)
+
+* Fetch between repositories does not error if they have inconsistent data
+  that should be irrelevant to the fetch operation. (Aaron Bentley)
+
+* Fix ``AttributeError`` exception when reconfiguring lightweight checkout 
+  of a remote repository.
+  (Jelmer Vernooij, #332194)
+
+* Fix bug in decoding v3 smart server messages when receiving multiple
+  lots of excess bytes after an end-of-message.
+  (Andrew Bennetts)
+
+* Force deletion of readonly files during merge, update and other tree
+  transforms.
+  (Craig Hewetson, Martin Pool, #218206)
+
+* Force socket shutdown in threaded HTTP test servers to avoid client hangs
+  (pycurl).  (Vincent Ladeuil, #383920).
+
+* ``LRUCache`` will maintain the linked list pointers even if a nodes
+  cleanup function raises an exception. (John Arbash Meinel, #396838)
+
+* Progress bars are now suppressed again when the environment variable
+  ``BZR_PROGRESS_BAR`` is set to ``none``.
+  (Martin Pool, #339385)
+
+* Reduced memory consumption during ``bzr commit`` of large files. For
+  pre 2a formats, should be down to ~3x the size of a file.
+  For ``--2a`` format repositories, it is down to the size of the file
+  content plus the size of the compressed text.  Related to bug #109114.
+  (John Arbash Meinel)
+
+* Set hidden attribute on .bzr directory below unicode path should never
+  fail with error. The operation should succeed even if bzr unable to set 
+  the attribute.  (Alexander Belchenko, related to bug #335362).
+  
+* Stacking will no longer accept requests to stack on the same
+  branch/repository. Existing branches that incorrectly reference the same
+  repository in a stacking configuration will now raise
+  UnstackableLocationError when the branch is opened. This can be fixed by
+  removing the stacking location inside ``.bzr/branch``.
+  (Robert Collins, #376243)
+
+* The ``log+`` decorator, useful in debugging or profiling, could cause
+  "AttributeError: 'list' object has no attribute 'next'".  This is now
+  fixed.  The log decorator no longer shows the elapsed time or transfer
+  rate because they're available in the log prefixes and the transport
+  activity display respectively.
+  (Martin Pool, #340347)
+
+* Unshelve works correctly when multiple zero-length files are present on
+  the shelf. (Aaron Bentley, #363444)
+
+* Progress bars no longer show the network transport scheme or direction.
+  (Martin Pool)
+
+* launchpad-login now respects the 'verbose' option.
+  (Jonathan Lange, #217031)
+
+
+Internals
+*********
+
+* ``bzrlib.user_encoding`` is now officially deprecated. It is not
+  possible to write a deprecation wrapper, but the variable will be
+  removed in the near future. Use ``bzrlib.osutils.get_user_encoding()``
+  instead. (Alexander Belchenko)
+
+* Command lookup has had hooks added. ``bzrlib.Command.hooks`` has
+  three new hook points: ``get_command``, ``get_missing_command`` and
+  ``list_commands``, which allow just-in-time command name provision
+  rather than requiring all command names be known a-priori.
+  (Robert Collins)
+
+* ``get_app_path`` from win32utils.py now supports REG_EXPAND_SZ data type
+  and can read path to wordpad.exe. (Alexander Belchenko, #392046)
+
+* ``graph.KnownGraph`` has been added. This is a class that can give
+  answers to ``heads()`` very quickly. However, it has the assumption that
+  the whole graph has already been loaded. This is true during
+  ``annotate`` so it is used there with good success (as much as 2x faster
+  for files with long ancestry and 'cherrypicked' changes.)
+  (John Arbash Meinel, Vincent Ladeuil)
+
+* OS file locks are now taken out using ``CreateFile`` rather than
+  ``LockFileEx`` on Windows. The locking remains exclusive with
+  ``LockFileEx`` but now it also works on older versions of Windows (such
+  as Win98). (Martin <gzlist>)
+
+* pack <=> pack fetching is now done via a ``PackStreamSource`` rather
+  than the ``Packer`` code. The user visible change is that we now
+  properly fetch the minimum number of texts for non-smart fetching.
+  (John Arbash Meinel)
+
+
+* ``VersionedFiles._add_text`` is a new api that lets us insert text into
+  the repository as a single string, rather than a list of lines. This can
+  improve memory overhead and performance of committing large files.
+  (Currently a private api, used only by commit). (John Arbash Meinel)
+
+
+Improvements
+************
+
+* ``bzr annotate`` can now be significantly faster. The time for
+  ``bzr annotate NEWS`` is down to 7s from 22s in 1.16. Files with long
+  histories and lots of 'duplicate insertions' will be improved more than
+  others. (John Arbash Meinel, Vincent Ladeuil)
+
+* ``bzr ls`` is now faster. On OpenOffice.org, the time drops from 2.4
+  to 1.1 seconds. The improvement for ``bzr ls -r-1`` is more
+  substantial dropping from 54.3 to 1.1 seconds. (Ian Clatworthy)
+
+* Improve "Path(s) are not versioned" error reporting for some commands.
+  (Benoît PIERRE)
+
+* Initial commit performance in ``--2a`` repositories has been improved by
+  making it cheaper to build the initial CHKMap. (John Arbash Meinel)
+
+* Resolving a revno to a revision id on a branch accessed via ``bzr://``
+  or ``bzr+ssh://`` is now much faster and involves no VFS operations.
+  This speeds up commands like ``bzr pull -r 123``.  (Andrew Bennetts)
+
+* ``revision-info`` now properly aligns the revnos/revids in the output
+  and doesn't traceback when given revisions not in the current branch.
+  Performance is also significantly improved when requesting multiple revs
+  at once.  (Matthew Fuller, John Arbash Meinel)
+
+* Tildes are no longer escaped by Transports. (Andy Kilner)
+
+
+Documentation
+*************
+
+* Avoid bad text wrapping in generated documentation.  Slightly better
+  formatting in the user reference.
+  (Martin Pool, #249908)
+
+* Minor clarifications to the help for End-Of-Line conversions.
+  (Ian Clatworthy)
+
+API Changes
+***********
+
+* Removed overspecific error class ``InvalidProgressBarType``.
+  (Martin Pool)
+
+* The method ``ProgressView._show_transport_activity`` is now
+  ``show_transport_activity`` because it's part of the contract between
+  this class and the UI.  (Martin Pool)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.18.txt'
--- doc/en/release-notes/bzr-1.18.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.18.txt	2010-12-16 06:24:14 +0000
@@ -0,0 +1,393 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.18.1
+##########
+
+:Codename:     nein nein nein!
+:1.18.1:       2009-09-09
+
+This release fixes two small but worthwhile bugs relevant to users on
+Microsoft Windows: some commands that failed on with locking errors will
+now work, and a bug that caused poor performance after committing a file
+with line-ending conversion has now been fixed.  It also fixes a bug in
+pushing to older servers.
+
+Bug Fixes
+*********
+
+* Fixed a problem where using content filtering and especially end-of-line
+  conversion will commit too many copies a file.
+  (Martin Pool, #415508)
+
+* Fix assertion error about ``_remember_remote_is_before`` in
+  ``set_tags_bytes`` when pushing to older smart servers.  
+  (Andrew Bennetts, Alexander Belchenko, #418931)
+
+Improvements
+************
+
+* ``bzr push`` locally on Windows will no longer give a locking error with
+  dirstate based formats. (Robert Collins)
+
+* ``bzr shelve`` and ``bzr unshelve`` now work on Windows.
+  (Robert Collins, #305006)
+
+API Changes
+***********
+
+* ``bzrlib.shelf_ui`` has had the ``from_args`` convenience methods of its
+  classes changed to manage lock lifetime of the trees they open in a way
+  consistent with reader-exclusive locks. (Robert Collins, #305006)
+
+* ``Tree.path_content_summary`` may return a size of None, when called on
+  a tree with content filtering where the size of the canonical form
+  cannot be cheaply determined.  (Martin Pool)
+
+* When manually creating transport servers in test cases, a new helper
+  ``TestCase.start_server`` that registers a cleanup and starts the server
+  should be used. (Robert Collins)
+
+bzr 1.18
+########
+
+Compatibility Breaks
+********************
+
+* Committing directly to a stacked branch from a lightweight checkout will
+  no longer work. In previous versions this would appear to work but would
+  generate repositories with insufficient data to create deltas, leading
+  to later errors when branching or reading from the repository.
+  (Robert Collins, bug #375013)
+
+New Features
+************
+
+Bug Fixes
+*********
+
+* Fetching from 2a branches from a version-2 bzr protocol would fail to
+  copy the internal inventory pages from the CHK store. This cannot happen
+  in normal use as all 2a compatible clients and servers support the
+  version-3 protocol, but it does cause test suite failures when testing
+  downlevel protocol behaviour. (Robert Collins)
+
+* Fix a test failure on karmic by making a locale test more robust.
+  (Vincent Ladeuil, #413514)
+
+* Fixed "Pack ... already exists" error when running ``bzr pack`` on a
+  fully packed 2a repository.  (Andrew Bennetts, #382463)
+
+* Further tweaks to handling of ``bzr add`` messages about ignored files.
+  (Jason Spashett, #76616)
+
+* Properly handle fetching into a stacked branch while converting the
+  data, especially when there are also ghosts. The code was filling in
+  parent inventories incorrectly, and also not handling when one of the
+  parents was a ghost. (John Arbash Meinel, #402778, #412198)
+
+* ``RemoteStreamSource.get_stream_for_missing_keys`` will fetch CHK
+  inventory pages when appropriate (by falling back to the vfs stream
+  source).  (Andrew Bennetts, #406686)
+
+* StreamSource generates rich roots from non-rich root sources correctly
+  now.  (Andrew Bennetts, #368921)
+
+* When deciding whether a repository was compatible for upgrading or
+  fetching, we previously incorrectly checked the default repository
+  format for the bzrdir format, rather than the format that was actually
+  present on disk.  (Martin Pool, #408824)
+
+Improvements
+************
+
+* A better description of the platform is shown in crash tracebacks, ``bzr
+  --version`` and ``bzr selftest``.
+  (Martin Pool, #409137)
+
+* Cross-format fetches (such as between 1.9-rich-root and 2a) via the
+  smart server are more efficient now.  They send inventory deltas rather
+  than full inventories.  The smart server has two new requests,
+  ``Repository.get_stream_1.19`` and ``Repository.insert_stream_1.19`` to
+  support this.  (Andrew Bennetts, #374738, #385826)
+
+* Extracting the full ancestry and computing the ``merge_sort`` is now
+  significantly faster. This effects things like ``bzr log -n0``. (For
+  example, ``bzr log -r -10..-1 -n0 bzr.dev`` is 2.5s down to 1.0s.
+  (John Arbash Meinel)
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+* ``-Dstrict_locks`` can now be used to check that read and write locks
+  are treated properly w.r.t. exclusivity. (We don't try to take an OS
+  read lock on a file that we already have an OS write lock on.) This is
+  now set by default for all tests, if you have a test which cannot be
+  fixed, you can use ``self.thisFailsStrictLockCheck()`` as a
+  compatibility knob. (John Arbash Meinel)
+
+* InterDifferingSerializer is now only used locally.  Other fetches that
+  would have used InterDifferingSerializer now use the more network
+  friendly StreamSource, which now automatically does the same
+  transformations as InterDifferingSerializer.  (Andrew Bennetts)
+
+* ``KnownGraph`` now has a ``.topo_sort`` and ``.merge_sort`` member which
+  are implemented in pyrex and significantly faster. This is exposed along
+  with ``CombinedGraphIndex.find_ancestry()`` as
+  ``VersionedFiles.get_known_graph_ancestry(keys)``.
+  (John Arbash Meinel)
+
+* RemoteBranch.open now honours ignore_fallbacks correctly on bzr-v2
+  protocols. (Robert Collins)
+
+* The index code now has some specialized routines to extract the full
+  ancestry of a key in a more efficient manner.
+  ``CombinedGraphIndex.find_ancestry()``. (Time to get ancestry for
+  bzr.dev drops from 1.5s down to 300ms. For OOo from 33s => 10.5s) (John
+  Arbash Meinel)
+
+Testing
+*******
+
+* Install the test ssl certificate and key so that installed bzr
+  can run the https tests. (Denys Duchier, #392401)
+  
+
+bzr 1.18rc1
+###########
+
+:Codename: little traveller
+:1.18:    2009-08-20
+:1.18rc1: 2009-08-10
+
+This release of Bazaar marches on towards the 2.0 release in which the 2a
+'brisbane-core' format becomes generally recommended.  Most of the work in
+this release now focusses on bug fixes and stabilization, covering both 2a
+and previous formats.  There is a new text-mode interactive merge feature,
+a new guide to migration to 2a format in the user documentation, and
+pushing branches to a smart server is now much faster.  
+
+The Bazaar team decided that 2.0 will be a long-term supported release,
+with bugfix-only releases based on it continuing for at least six months
+or until the following stable release.
+
+There are no changes from 1.18rc1 to 1.18.
+
+New Features
+************
+
+* ``bzr merge --interactive`` applies a user-selected portion of the
+  merge.  The UI is similar to ``shelve``.  (Aaron Bentley)
+
+* ``bzr reconfigure`` now takes options ``--stacked-on URL`` and
+  ``--unstacked`` to change stacking of a branch.
+  (Martin Pool, #391411)
+
+Bug Fixes
+*********
+
+* Annotating on a stacked branch will now succeed in simple scenarios.
+  There are still some complex scenarios where it will fail (bug #399884)
+  (John Arbash Meinel, #393366)
+
+* A progress bar is no longer left dangling when ``bzr selftest``
+  completes, and the progress bar updates with zero latency so the
+  displayed test name is always the one that's actually running.
+  (Martin Pool, #123688)
+
+* Authenticating against an SSH server now uses ``auth_none`` to determine
+  if password authentication is even supported. This fixes a bug where
+  users would be prompted for a launchpad password, even though launchpad
+  only supports publickey authentication. (John Arbash Meinel, #375867)
+
+* BranchBuilder now accepts timezone to avoid test failures in countries far
+  from GMT. (Vincent Ladeuil, #397716)
+
+* ``bzr commit`` no longer saves the unversioning of missing files until
+  the commit has completed on the branch. This means that aborting a
+  commit that found a missing file will leave the tree unedited.
+  (Robert Collins, #282402)
+
+* ``bzr mv`` no longer takes out branch locks, which allows it to work
+  when the branch is readonly. (Robert Collins, #216541)
+
+* ``bzr revert .`` no longer generates an InconsistentDelta error when
+  there are missing subtrees. (Robert Collins, #367632)
+
+* ``bzr send`` now generates valid bundles with ``--2a`` formats. However,
+  do to internal changes necessary to support this, older clients will
+  fail when trying to insert them. For newer clients, the bundle can be
+  used to apply the changes to any rich-root compatible format.
+  (John Arbash Meinel, #393349)
+
+* Cope with FTP servers that don't support restart/append by falling back
+  to reading and then rewriting the whole file, such as TahoeLAFS.  (This
+  fallback may be slow for some access patterns.)  (Nils Durner, #294709)
+
+* Encode the paths in ``mbcs`` encoding on Windows when spawning an
+  external diff client. This at least allows supporting filenames that are
+  not ascii, but are present in the current locale. Ideally we would be
+  able to pass the Unicode path, but that would be client dependent.
+  (John Arbash Meinel, #382709)
+
+* Fix a compile bug on Solaris having to do with const and
+  pointer-to-pointers. (John Arbash Meinel, #408441)
+
+* Fixed a NameError that occurs when merging or pulling from a URL that
+  causes a redirection loop when bzr tries to read a URL as a bundle.
+  (Andrew Bennetts, #400847)
+
+* Fix ``AttributeError: 'TestUIFactory' object has no attribute 'tick'``
+  running send and similar commands on 2a formats.
+  (Martin Pool, #408201)
+  
+* Fix crash in some invocations of ``bzr status`` in format 2a.
+  (Martin Pool, #403523)
+
+* Fixed export to existing directory: if directory is empty then export 
+  will succeed, otherwise it fails with error.
+  (Alexander Belchenko, #406174)
+
+* Fixed spurious "Source branch does not support stacking" warning when
+  pushing. (Andrew Bennetts, #388908)
+
+* Fixed spurious transport activity indicator appearing while tests are
+  running.  (Martin Pool, #343532)
+
+* Merge now correctly handles empty right-hand revision specs.
+  (Aaron Bentley, #333961)
+
+* Renames to lexographically lower basenames in trees that have never been
+  committed to will no longer corrupt the dirstate. This was caused by an
+  bug in the dirstate update_minimal method. (Robert Collins, #395556)
+
+* Requests for unknown methods no longer cause the smart server to log
+  lots of backtraces about ``UnknownSmartMethod``, ``do_chunk`` or
+  ``do_end``.  (Andrew Bennetts, #338561)
+
+* Shelve will not shelve the initial add of the tree root.  (Aaron Bentley)
+
+* Streaming from bzr servers where there is a chain of stacked branches
+  (A stacked on B stacked on C) will now work. (Robert Collins, #406597)
+
+* The environment variable ``BZR_PROGRESS_BAR`` set to either ``text`` or ``none``
+  always forces progress bars either on or off respectively.  Otherwise,
+  they're turned on if ``TERM`` is not ``dumb`` and stderr is a terminal.
+  bzr always uses the 'text' user interface when run as a command, so
+  ``BZR_USE_TEXT_UI`` is no longer needed.
+  (Martin Pool, #339385, #387717)
+
+* The optional ``_knit_load_data_pyx`` C extension was never being
+  imported.  This caused significant slowdowns when reading data from
+  repositories.  (Andrew Bennetts, #405653)
+  
+* The ``--hardlink`` option to ``branch`` and ``checkout`` is not
+  supported at the moment on workingtree formats that can do content
+  filtering.  (See <https://bugs.launchpad.net/bzr/+bug/408193>.)
+  bzr now says so, rather than just ignoring the option.  (Martin Pool)
+
+* There was a bug in ``osutils.relpath`` that was only triggered on
+  Windows. Essentially if you were at the root of a drive, and did
+  something to a branch/repo on another drive, we would go into an
+  infinite loop while trying to find a 'relative path'.
+  (John Arbash Meinel, #394227)
+
+* ``WorkingTree4.unversion`` will no longer fail to unversion ids which
+  were present in a parent tree but renamed in the working tree.
+  (Robert Collins, #187207)
+
+Improvements
+************
+
+* Can now rename/move files even if they have been removed from the inventory.
+  (Marius Kruger)
+
+* Pushing branches with tags via ``bzr://`` and ``bzr+ssh://`` is much
+  faster, using a new ``Branch.set_tags_bytes`` smart server verb rather
+  than VFS methods.  For example, pushes of small branches with tags take
+  11 rather than 18 smart server requests.  (Andrew Bennetts, #398608)
+
+* Sending Ctrl-Break on Windows will now drop you into the debugger, in
+  the same way that sending Ctrl-\\ does on other platforms.
+  (John Arbash Meinel)
+
+Documentation
+*************
+
+* Added Bazaar 2.0 Upgrade Guide. (Ian Clatworthy)
+
+API Changes
+***********
+
+* ``CLIUIFactory`` is deprecated; use ``TextUIFactory`` instead if you
+  need to subclass or create a specific class, or better yet the existing
+  ``make_ui_for_terminal``.  ``SilentUIFactory`` is clarified to do no
+  user interaction at all, rather than trying to read from stdin but not
+  writing any output, which would be strange if reading prompts or
+  passwords.  (Martin Pool)
+
+* New TransformPreview.commit() allows committing without a working tree.
+  (Aaron Bentley)
+
+* ``pb`` parameter to ``TextTestResult`` is deprecated and ignored.
+  (Martin Pool)
+
+* ProgressTasks now prefer to talk direct to their ProgressView not to the
+  UIFactory. 
+  (Martin Pool)
+
+* ``WorkingTree._check`` now requires a references dict with keys matching
+  those returned by ``WorkingTree._get_check_refs``. (Robert Collins)
+
+Internals
+*********
+
+* ``CHKInventory.path2id`` uses the parent_id to basename hash to avoid
+  reading the entries along the path, reducing work to lookup ids from
+  paths. (Robert Collins)
+
+* ``CHKMap.apply_delta`` now raises ``InconsistentDelta`` if a delta adds
+  as new a key which was already mapped. (Robert Collins)
+
+* Inventory delta application catches more cases of corruption and can
+  prevent corrupt deltas from affecting consistency of data structures on
+  disk. (Robert Collins)
+
+* --subunit support now adds timestamps if the subunit version supports
+  it. (Robert Collins)
+
+* The Windows all-in-one installer now bundles the PyQt image format
+  plugins, which allows previewing more images as part of 'qdiff'.
+  (Alexander Belchenko)
+
+
+Testing
+*******
+
+* Merge directive cherrypick tests must use the same root id.
+  (Martin Pool, #409684)
+
+* Spurious failure in ``check`` tests on rich-root formats fixed.
+  (Martin Pool, #408199)
+
+* The ``bzrlib.tests.TextTestRunner`` will no longer call
+  ``countTestsCases`` on the test being run. Progress information is
+  instead handled by having the test passed in call ``result.progress``
+  before running its contents. This improves the behaviour when using
+  ``TextTestRunner`` with test suites that don't support
+  ``countTestsCases``. (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix
+

=== added file 'doc/en/release-notes/bzr-1.2.txt'
--- doc/en/release-notes/bzr-1.2.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.2.txt	2010-11-12 22:46:28 +0000
@@ -0,0 +1,224 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.2
+#######
+
+:Released: 2008-02-15
+
+Bug Fixes
+*********
+
+* Fix failing test in Launchpad plugin. (Martin Pool)
+
+
+bzr 1.2rc1
+##########
+
+:Released: 2008-02-13
+
+Notes When Upgrading
+********************
+
+* Fetching via the smart protocol may need to reconnect once during a fetch
+  if the remote server is running Bazaar 1.1 or earlier, because the client
+  attempts to use more efficient requests that confuse older servers.  You
+  may be required to re-enter a password or passphrase when this happens.
+  This won't happen if the server is upgraded to Bazaar 1.2.
+  (Andrew Bennetts)
+
+Changes
+*******
+
+* Fetching via bzr+ssh will no longer fill ghosts by default (this is
+  consistent with pack-0.92 fetching over SFTP). (Robert Collins)
+
+* Formatting of ``bzr plugins`` output is changed to be more human-
+  friendly. Full path of plugins locations will be shown only with
+  ``--verbose`` command-line option. (Alexander Belchenko)
+
+* ``merge`` now prefers to use the submit branch, but will fall back to
+  parent branch.  For many users, this has no effect.  But some users who
+  pull and merge on the same branch will notice a change.  This change
+  makes it easier to work on a branch on two different machines, pulling
+  between the machines, while merging from the upstream.
+  ``merge --remember`` can now be used to set the submit_branch.
+  (Aaron Bentley)
+
+Features
+********
+
+* ``merge --preview`` produces a diff of the changes merge would make,
+  but does not actually perform the merge.  (Aaron Bentley)
+
+* New smart method ``Repository.get_parent_map`` for getting revision
+  parent data. This returns additional parent information topologically
+  adjacent to the requested data to reduce round trip latency impacts.
+  (Robert Collins)
+
+* New smart method, ``Repository.stream_revisions_chunked``, for fetching
+  revision data that streams revision data via a chunked encoding.  This
+  avoids buffering large amounts of revision data on the server and on the
+  client, and sends less data to the server to request the revisions.
+  (Andrew Bennetts, Robert Collins, #178353)
+
+* The launchpad plugin now handles lp URLs of the form
+  ``lp://staging/``, ``lp://demo/``, ``lp://dev/`` to use the appropriate
+  launchpad instance to do the resolution of the branch identities.
+  This is primarily of use to Launchpad developers, but can also
+  be used by other users who want to try out Launchpad as
+  a branch location without messing up their public Launchpad
+  account.  Branches that are pushed to the staging environment
+  have an expected lifetime of one day. (Tim Penhey)
+
+Improvements
+************
+
+* Creating a new branch no longer tries to read the entire revision-history
+  unnecessarily over smart server operations. (Robert Collins)
+
+* Fetching between different repository formats with compatible models now
+  takes advantage of the smart method to stream revisions.  (Andrew Bennetts)
+
+* The ``--coverage`` option is now global, rather specific to ``bzr
+  selftest``.  (Andrew Bennetts)
+
+* The ``register-branch`` command will now use the public URL of the branch
+  containing the current directory, if one has been set and no explicit
+  branch is provided.  (Robert Collins)
+
+* Tweak the ``reannotate`` code path to optimize the 2-parent case.
+  Speeds up ``bzr annotate`` with a pack repository by approx 3:2.
+  (John Arbash Meinel)
+
+Bugfixes
+********
+
+* Calculate remote path relative to the shared medium in _SmartClient.  This
+  is related to the problem in bug #124089.  (Andrew Bennetts)
+
+* Cleanly handle connection errors in smart protocol version two, the same
+  way as they are handled by version one.  (Andrew Bennetts)
+
+* Clearer error when ``version-info --custom`` is used without
+  ``--template`` (Lukáš Lalinský)
+
+* Don't raise UnavailableFeature during test setup when medusa is not
+  available or tearDown is never called leading to nasty side effects.
+  (#137823, Vincent Ladeuil)
+
+* If a plugin's test suite cannot be loaded, for example because of a syntax
+  error in the tests, then ``selftest`` fails, rather than just printing
+  a warning.  (Martin Pool, #189771)
+
+* List possible values for BZR_SSH environment variable in env-variables
+  help topic. (Alexander Belchenko, #181842)
+
+* New methods ``push_log_file`` and ``pop_log_file`` to intercept messages:
+  popping the log redirection now precisely restores the previous state,
+  which makes it easier to use bzr log output from other programs.
+  TestCaseInTempDir no longer depends on a log redirection being established
+  by the test framework, which lets bzr tests cleanly run from a normal
+  unittest runner.
+  (#124153, #124849, Martin Pool, Jonathan Lange)
+
+* ``pull --quiet`` is now more quiet, in particular a message is no longer
+  printed when the remembered pull location is used. (James Westby,
+  #185907)
+
+* ``reconfigure`` can safely be interrupted while fetching.
+  (Aaron Bentley, #179316)
+
+* ``reconfigure`` preserves tags when converting to and from lightweight
+  checkouts.  (Aaron Bentley, #182040)
+
+* Stop polluting /tmp when running selftest.
+  (Vincent Ladeuil, #123363)
+
+* Switch from NFKC => NFC for normalization checks. NFC allows a few
+  more characters which should be considered valid.
+  (John Arbash Meinel, #185458)
+
+* The launchpad plugin now uses the ``edge`` XML-RPC server to avoid
+  interacting badly with a bug on the launchpad side. (Robert Collins)
+
+* Unknown hostnames when connecting to a ``bzr://`` URL no longer cause
+  tracebacks.  (Andrew Bennetts, #182849)
+
+API Breaks
+**********
+
+* Classes implementing Merge types like Merge3Merger must now accept (and
+  honour) a do_merge flag in their constructor.  (Aaron Bentley)
+
+* ``Repository.add_inventory`` and ``add_revision`` now require the caller
+  to previously take a write lock (and start a write group.)
+  (Martin Pool)
+
+Testing
+*******
+
+* selftest now accepts --load-list <file> to load a test id list. This
+  speeds up running the test suite on a limited set of tests.
+  (Vincent Ladeuil)
+
+Internals
+*********
+
+* Add a new method ``get_result`` to graph search objects. The resulting
+  ``SearchResult`` can be used to recreate the search later, which will
+  be useful in reducing network traffic. (Robert Collins)
+
+* Use convenience function to check whether two repository handles
+  are referring to the same repository in ``Repository.get_graph``.
+  (Jelmer Vernooij, #187162)
+
+* Fetching now passes the find_ghosts flag through to the
+  ``InterRepository.missing_revision_ids`` call consistently for all
+  repository types. This will enable faster missing revision discovery with
+  bzr+ssh. (Robert Collins)
+
+* Fix error handling in Repository.insert_data_stream. (Lukas Lalinsky)
+
+* ``InterRepository.missing_revision_ids`` is now deprecated in favour of
+  ``InterRepository.search_missing_revision_ids`` which returns a
+  ``bzrlib.graph.SearchResult`` suitable for making requests from the smart
+  server. (Robert Collins)
+
+* New error ``NoPublicBranch`` for commands that need a public branch to
+  operate. (Robert Collins)
+
+* New method ``iter_inventories`` on Repository for access to many
+  inventories. This is primarily used by the ``revision_trees`` method, as
+  direct access to inventories is discouraged. (Robert Collins)
+
+* New method ``next_with_ghosts`` on the Graph breadth-first-search objects
+  which will split out ghosts and present parents into two separate sets,
+  useful for code which needs to be aware of ghosts (e.g. fetching data
+  cares about ghosts during revision selection). (Robert Collins)
+
+* Record a timestamp against each mutter to the trace file, relative to the
+  first import of bzrlib.  (Andrew Bennetts)
+
+* ``Repository.get_data_stream`` is now deprecated in favour of
+  ``Repository.get_data_stream_for_search`` which allows less network
+  traffic when requesting data streams over a smart server. (Robert Collins)
+
+* ``RemoteBzrDir._get_tree_branch`` no longer triggers ``_ensure_real``,
+  removing one round trip on many network operations. (Robert Collins)
+
+* RemoteTransport's ``recommended_page_size`` method now returns 64k, like
+  SFTPTransport and HttpTransportBase.  (Andrew Bennetts)
+
+* Repository has a new method ``has_revisions`` which signals the presence
+  of many revisions by returning a set of the revisions listed which are
+  present. This can be done by index queries without reading data for parent
+  revision names etc. (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.3.txt'
--- doc/en/release-notes/bzr-1.3.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.3.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,239 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.3.1
+#########
+
+:Released: 2008-04-09
+
+No changes from 1.3.1rc1.
+
+
+bzr 1.3.1rc1
+############
+
+:Released: 2008-04-04
+
+Bug Fixes
+*********
+
+* Fix a bug causing a ValueError crash in ``parse_line_delta_iter`` when
+  fetching revisions from a knit to pack repository or vice versa using
+  bzr:// (including over HTTP or SSH).
+  (#208418, Andrew Bennetts, Martin Pool, Robert Collins)
+
+
+bzr 1.3
+#######
+
+:Released: 2008-03-20
+
+Bazaar has become part of the GNU project <http://www.gnu.org>
+
+Many operations that act on history, including ``log`` and ``annotate`` are now
+substantially faster.  Several bugs have been fixed and several new options and
+features have been added.
+
+Testing
+*******
+
+* Avoid spurious failure of ``TestVersion.test_version`` matching
+  directory names.
+  (#202778, Martin Pool)
+
+
+bzr 1.3rc1
+##########
+
+:Released: 2008-03-16
+
+Notes When Upgrading
+********************
+
+* The backup directory created by ``upgrade`` is now called
+  ``backup.bzr``, not ``.bzr.backup``. (Martin Albisetti)
+
+Changes
+*******
+
+* A new repository format 'development' has been added. This format will
+  represent the latest 'in-progress' format that the bzr developers are
+  interested in getting early-adopter testing and feedback on.
+  ``doc/developers/development-repo.txt`` has detailed information.
+  (Robert Collins)
+
+* BZR_LOG environment variable controls location of .bzr.log trace file.
+  User can suppress writing messages to .bzr.log by using '/dev/null'
+  filename (on Unix) or 'NUL' (on Windows). If BZR_LOG variable
+  is not defined but BZR_HOME is defined then default location
+  for .bzr.log trace file is ``$BZR_HOME/.bzr.log``.
+  (Alexander Belchenko, #106117)
+
+* ``launchpad`` builtin plugin now shipped as separate part in standalone
+  bzr.exe, installed to ``C:\Program Files\Bazaar\plugins`` directory,
+  and standalone installer allows user to skip installation of this plugin.
+  (Alexander Belchenko)
+
+* Restore auto-detection of plink.exe on Windows. (Dmitry Vasiliev)
+
+* Version number is now shown as "1.2" or "1.2pr2", without zeroed or
+  missing final fields.  (Martin Pool)
+
+Features
+********
+
+* ``branch`` and ``checkout`` can hard-link working tree files, which is
+  faster and saves space.  (Aaron Bentley)
+
+* ``bzr send`` will now also look at the ``child_submit_to`` setting in
+  the submit branch to determine the email address to send to.
+  (Jelmer Vernooij)
+
+Improvements
+************
+
+* BzrBranch._lefthand_history is faster on pack repos.  (Aaron Bentley)
+
+* Branch6.generate_revision_history is faster.  (Aaron Bentley)
+
+* Directory services can now be registered, allowing special URLs to be
+  dereferenced into real URLs.  This is a generalization and cleanup of
+  the lp: transport lookup.  (Aaron Bentley)
+
+* Merge directives that are automatically attached to emails have nicer
+  filenames, based on branch-nick + revno. (Aaron Bentley)
+
+* ``push`` has a ``--revision`` option, to specify what revision to push up
+  to.  (Daniel Watkins)
+
+* Significantly reducing execution time and network traffic for trivial
+  case of running ``bzr missing`` command for two identical branches.
+  (Alexander Belchenko)
+
+* Speed up operations that look at the revision graph (such as 'bzr log').
+  ``KnitPackRepositor.get_revision_graph`` uses ``Graph.iter_ancestry`` to
+  extract the revision history. This allows filtering ghosts while
+  stepping instead of needing to peek ahead. (John Arbash Meinel)
+
+* The ``hooks`` command lists installed hooks, to assist in debugging.
+  (Daniel Watkins)
+
+* Updates to how ``annotate`` work. Should see a measurable improvement in
+  performance and memory consumption for file with a lot of merges.
+  Also, correctly handle when a line is introduced by both parents (it
+  should be attributed to the first merge which notices this, and not
+  to all subsequent merges.) (John Arbash Meinel)
+
+Bugfixes
+********
+
+* Autopacking no longer holds the full set of inventory lines in
+  memory while copying. For large repositories, this can amount to
+  hundreds of MB of ram consumption.
+  (Ian Clatworthy, John Arbash Meinel)
+
+* Cherrypicking when using ``--format=merge3`` now explictly excludes
+  BASE lines. (John Arbash Meinel, #151731)
+
+* Disable plink's interactive prompt for password.
+  (#107593, Dmitry Vasiliev)
+
+* Encode command line arguments from unicode to user_encoding before
+  invoking external mail client in `bzr send` command.
+  (#139318, Alexander Belchenko)
+
+* Fixed problem connecting to ``bzr+https://`` servers.
+  (#198793, John Ferlito)
+
+* Improved error reporting in the Launchpad plugin. (Daniel Watkins,
+  #196618)
+
+* Include quick-start-summary.svg file to python-based installer(s)
+  for Windows. (#192924, Alexander Belchenko)
+
+* lca merge now respects specified files. (Aaron Bentley)
+
+* Make version-info --custom imply --all. (#195560, James Westby)
+
+* ``merge --preview`` now works for merges that add or modify
+  symlinks (James Henstridge)
+
+* Redirecting the output from ``bzr merge`` (when the remembered
+  location is used) now works. (John Arbash Meinel)
+
+* setup.py script explicitly checks for Python version.
+  (Jari Aalto, Alexander Belchenko, #200569)
+
+* UnknownFormatErrors no longer refer to branches regardless of kind of
+  unknown format. (Daniel Watkins, #173980)
+
+* Upgrade bundled ConfigObj to version 4.5.2, which properly quotes #
+  signs, among other small improvements. (Matt Nordhoff, #86838)
+
+* Use correct indices when emitting LCA conflicts.  This fixes IndexError
+  errors.  (Aaron Bentley, #196780)
+
+Documentation
+*************
+
+* Explained how to use ``version-info --custom`` in the User Guide.
+  (Neil Martinsen-Burrell)
+
+API Breaks
+**********
+
+* Support for loading plugins from zip files and
+  ``bzrlib.plugin.load_from_zip()`` function are deprecated.
+  (Alexander Belchenko)
+
+Testing
+*******
+
+* Added missing blackbox tests for ``modified`` (Adrian Wilkins)
+
+* The branch interface tests were invalid for branches using rich-root
+  repositories because the empty string is not a valid file-id.
+  (Robert Collins)
+
+Internals
+*********
+
+* ``Graph.iter_ancestry`` returns the ancestry of revision ids. Similar to
+  ``Repository.get_revision_graph()`` except it includes ghosts and you can
+  stop part-way through. (John Arbash Meinel)
+
+* New module ``tools/package_mf.py`` provide custom module finder for
+  python packages (improves standard python library's modulefinder.py)
+  used by ``setup.py`` script while building standalone bzr.exe.
+  (Alexander Belchenko)
+
+* New remote method ``RemoteBzrDir.find_repositoryV2`` adding support for
+  detecting external lookup support on remote repositories. This method is
+  now attempted first when lookup up repositories, leading to an extra
+  round trip on older bzr smart servers. (Robert Collins)
+
+* Repository formats have a new supported-feature attribute
+  ``supports_external_lookups`` used to indicate repositories which support
+  falling back to other repositories when they have partial data.
+  (Robert Collins)
+
+* ``Repository.get_revision_graph_with_ghosts`` and
+  ``bzrlib.revision.(common_ancestor,MultipleRevisionSources,common_graph)``
+  have been deprecated.  (John Arbash Meinel)
+
+* ``Tree.iter_changes`` is now a public API, replacing the work-in-progress
+  ``Tree._iter_changes``. The api is now considered stable and ready for
+  external users.  (Aaron Bentley)
+
+* The bzrdir format registry now accepts an ``alias`` keyword to
+  register_metadir, used to indicate that a format name is an alias for
+  some other format and thus should not be reported when describing the
+  format. (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.4.txt'
--- doc/en/release-notes/bzr-1.4.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.4.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,347 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.4 
+#######
+
+:Released: 2008-04-28
+
+This release of Bazaar includes handy improvements to the speed of log and
+status, new options for several commands, improved documentation, and better
+hooks, including initial code for server-side hooks.  A number of bugs have
+been fixed, particularly in interoperability between different formats or
+different releases of Bazaar over there network.  There's been substantial
+internal work in both the repository and network code to enable new features
+and faster performance.
+
+Bug Fixes
+*********
+
+* Pushing a branch in "dirstate" format (Branch5) over bzr+ssh would break
+  if the remote server was < version 1.2.  This was due to a bug in the
+  RemoteRepository.get_parent_map() fallback code.
+  (John Arbash Meinel, Andrew Bennetts, #214894)
+
+
+bzr 1.4rc2
+##########
+
+:Released: 2008-04-21
+
+Bug Fixes
+*********
+
+* ``bzr log -r ..X bzr://`` was failing, because it was getting a request
+  for ``revision_id=None`` which was not a string.
+  (John Arbash Meinel, #211661)
+
+* Fixed a bug in handling ghost revisions when logging changes in a
+  particular file.  (John Arbash Meinel, #209948)
+
+* Fix error about "attempt to add line-delta in non-delta knit".
+  (Andrew Bennetts, #205156)
+
+* Fixed performance degradation in fetching from knit repositories to
+  knits and packs due to parsing the entire revisions.kndx on every graph
+  walk iteration fixed by using the Repository.get_graph API.  There was
+  another regression in knit => knit fetching which re-read the index for
+  every revision each side had in common.
+  (Robert Collins, John Arbash Meinel)
+
+
+bzr 1.4rc1
+##########
+
+:Released: 2008-04-11
+
+Changes
+*******
+
+* bzr main script cannot be imported (Benjamin Peterson)
+
+* On GNU/Linux bzr additionally looks for plugins in arch-independent site
+  directory. (Toshio Kuratomi)
+
+* The ``set_rh`` branch hook is now deprecated. Please migrate
+  any plugins using this hook to use an alternative, e.g.
+  ``post_change_branch_tip``. (Ian Clatworthy)
+
+* When a plugin cannot be loaded as the file path is not a valid
+  python module name bzr will now strip a ``bzr_`` prefix from the
+  front of the suggested name, as many plugins (e.g. bzr-svn)
+  want to be installed without this prefix. It is a common mistake
+  to have a folder named "bzr-svn" for that plugin, especially
+  as this is what bzr branch lp:bzr-svn will give you. (James Westby,
+  Andrew Cowie)
+
+* UniqueIntegerBugTracker now appends bug-ids instead of joining
+  them to the base URL. Plugins that register bug trackers may
+  need a trailing / added to the base URL if one is not already there.
+  (James Wesby, Andrew Cowie)
+
+Features
+********
+
+* Added start_commit hook for mutable trees. (Jelmer Vernooij, #186422)
+
+* ``status`` now accepts ``--no-pending`` to show the status without
+  listing pending merges, which speeds up the command a lot on large
+  histories.  (James Westby, #202830)
+
+* New ``post_change_branch_tip`` hook that is called after the
+  branch tip is moved but while the branch is still write-locked.
+  See the User Reference for signature details.
+  (Ian Clatworthy, James Henstridge)
+
+* Reconfigure can convert a branch to be standalone or to use a shared
+  repository.  (Aaron Bentley)
+
+Improvements
+************
+
+* The smart protocol now has support for setting branches' revision info
+  directly.  This should make operations like push slightly faster, and is a
+  step towards server-side hooks.  The new request method name is
+  ``Branch.set_last_revision_info``.  (Andrew Bennetts)
+
+* ``bzr commit --fixes`` now recognises "gnome" as a tag by default.
+  (James Westby, Andrew Cowie)
+
+* ``bzr switch`` will attempt to find branches to switch to relative to the
+  current branch. E.g. ``bzr switch branchname`` will look for
+  ``current_branch/../branchname``. (Robert Collins, Jelmer Vernooij,
+  Wouter van Heyst)
+
+* Diff is now more specific about execute-bit changes it describes
+  (Chad Miller)
+
+* Fetching data over HTTP is a bit faster when urllib is used.  This is done
+  by forcing it to recv 64k at a time when reading lines in HTTP headers,
+  rather than just 1 byte at a time.  (Andrew Bennetts)
+
+* Log --short and --line are much faster when -r is not specified.
+  (Aaron Bentley)
+
+* Merge is faster.  We no longer check a file's existence unnecessarily
+  when merging the execute bit.  (Aaron Bentley)
+
+* ``bzr status`` on an explicit list of files no longer shows pending
+  merges, making it much faster on large trees. (John Arbash Meinel)
+
+* The launchpad directory service now warns the user if they have not set
+  their launchpad login and are trying to resolve a URL using it, just
+  in case they want to do a write operation with it.  (James Westby)
+
+* The smart protocol client is slightly faster, because it now only queries
+  the server for the protocol version once per connection.  Also, the HTTP
+  transport will now automatically probe for and use a smart server if
+  one is present.  You can use the new ``nosmart+`` transport decorator
+  to get the old behaviour.  (Andrew Bennetts)
+
+* The ``version`` command takes a ``--short`` option to print just the
+  version number, for easier use in scripts.  (Martin Pool)
+
+* Various operations with revision specs and commands that calculate
+  revnos and revision ids are faster.  (John A. Meinel, Aaron Bentley)
+
+Bugfixes
+********
+
+* Add ``root_client_path`` parameter to SmartWSGIApp and
+  SmartServerRequest.  This makes it possible to publish filesystem
+  locations that don't exactly match URL paths. SmartServerRequest
+  subclasses should use the new ``translate_client_path`` and
+  ``transport_from_client_path`` methods when dealing with paths received
+  from a client to take this into account.  (Andrew Bennetts, #124089)
+
+* ``bzr mv a b`` can be now used also to rename previously renamed
+  directories, not only files. (Lukáš Lalinský, #107967)
+
+* ``bzr uncommit --local`` can now remove revisions from the local
+  branch to be symmetric with ``bzr commit --local``.
+  (John Arbash Meinel, #93412)
+
+* Don't ask for a password if there is no real terminal.
+  (Alexander Belchenko, #69851)
+
+* Fix a bug causing a ValueError crash in ``parse_line_delta_iter`` when
+  fetching revisions from a knit to pack repository or vice versa using
+  bzr:// (including over HTTP or SSH).
+  (#208418, Andrew Bennetts, Martin Pool, Robert Collins)
+
+* Fixed ``_get_line`` in ``bzrlib.smart.medium``, which was buggy.  Also
+  fixed ``_get_bytes`` in the same module to use the push back buffer.
+  These bugs had no known impact in normal use, but were problematic for
+  developers working on the code, and were likely to cause real bugs sooner
+  or later.  (Andrew Bennetts)
+
+* Implement handling of basename parameter for DefaultMail.  (James Westby)
+
+* Incompatibility with Paramiko versions newer than 1.7.2 was fixed.
+  (Andrew Bennetts, #213425)
+
+* Launchpad locations (lp: URLs) can be pulled.  (Aaron Bentley, #181945)
+
+* Merges that add files to deleted root directories complete.  They
+  do create conflicts.  (Aaron Bentley, #210092)
+
+* vsftp's return ``550 RNFR command failed.`` supported.
+  (Marcus Trautwig, #129786)
+
+Documentation
+*************
+
+* Improved documentation on send/merge relationship. (Peter Schuller)
+
+* Minor fixes to the User Guide. (Matthew Fuller)
+
+* Reduced the evangelism in the User Guide. (Ian Clatworthy)
+
+* Added Integrating with Bazaar document for developers (Martin Albisetti)
+
+API Breaks
+**********
+
+* Attempting to pull data from a ghost aware repository (e.g. knits) into a
+  non-ghost aware repository such as weaves will now fail if there are
+  ghosts.  (Robert Collins)
+
+* ``KnitVersionedFile`` no longer accepts an ``access_mode`` parameter, and
+  now requires the ``index`` and ``access_method`` parameters to be
+  supplied. A compatible shim has been kept in the new function
+  ``knit.make_file_knit``. (Robert Collins)
+
+* Log formatters must now provide log_revision instead of show and
+  show_merge_revno methods. The latter had been deprecated since the 0.17
+  release. (James Westby)
+
+* ``LoopbackSFTP`` is now called ``SocketAsChannelAdapter``.
+  (Andrew Bennetts)
+
+* ``osutils.backup_file`` is removed. (Alexander Belchenko)
+
+* ``Repository.get_revision_graph`` is deprecated, with no replacement
+  method. The method was size(history) and not desirable. (Robert Collins)
+
+* ``revision.revision_graph`` is deprecated, with no replacement function.
+  The function was size(history) and not desirable. (Robert Collins)
+
+* ``Transport.get_shared_medium`` is deprecated.  Use
+  ``Transport.get_smart_medium`` instead.  (Andrew Bennetts)
+
+* ``VersionedFile`` factories now accept a get_scope parameter rather
+  than using a call to ``transaction_finished``, allowing the removal of
+  the fixed list of versioned files per repository. (Robert Collins)
+
+* ``VersionedFile.annotate_iter`` is deprecated. While in principle this
+  allowed lower memory use, all users of annotations wanted full file
+  annotations, and there is no storage format suitable for incremental
+  line-by-line annotation. (Robert Collins)
+
+* ``VersionedFile.clone_text`` is deprecated. This performance optimisation
+  is no longer used - reading the content of a file that is undergoing a
+  file level merge to identical state on two branches is rare enough, and
+  not expensive enough to special case. (Robert Collins)
+
+* ``VersionedFile.clear_cache`` and ``enable_cache`` are deprecated.
+  These methods added significant complexity to the ``VersionedFile``
+  implementation, but were only used for optimising fetches from knits -
+  which can be done from outside the knit layer, or via a caching
+  decorator. As knits are not the default format, the complexity is no
+  longer worth paying. (Robert Collins)
+
+* ``VersionedFile.create_empty`` is removed. This method presupposed a
+  sensible mapping to a transport for individual files, but pack backed
+  versioned files have no such mapping. (Robert Collins)
+
+* ``VersionedFile.get_graph`` is deprecated, with no replacement method.
+  The method was size(history) and not desirable. (Robert Collins)
+
+* ``VersionedFile.get_graph_with_ghosts`` is deprecated, with no
+  replacement method.  The method was size(history) and not desirable.
+  (Robert Collins)
+
+* ``VersionedFile.get_parents`` is deprecated, please use
+  ``VersionedFile.get_parent_map``. (Robert Collins)
+
+* ``VersionedFile.get_sha1`` is deprecated, please use
+  ``VersionedFile.get_sha1s``. (Robert Collins)
+
+* ``VersionedFile.has_ghost`` is now deprecated, as it is both expensive
+  and unused outside of a single test. (Robert Collins)
+
+* ``VersionedFile.iter_parents`` is now deprecated in favour of
+  ``get_parent_map`` which can be used to instantiate a Graph on a
+  VersionedFile. (Robert Collins)
+
+* ``VersionedFileStore`` no longer uses the transaction parameter given
+  to most methods; amongst other things this means that the
+  get_weave_or_empty method no longer guarantees errors on a missing weave
+  in a readonly transaction, and no longer caches versioned file instances
+  which reduces memory pressure (but requires more careful management by
+  callers to preserve performance). (Robert Collins)
+
+Testing
+*******
+
+* New -Dselftest_debug flag disables clearing of the debug flags during
+  tests.  This is useful if you want to use e.g. -Dhpss to help debug a
+  failing test.  Be aware that using this feature is likely to cause
+  spurious test failures if used with the full suite. (Andrew Bennetts)
+
+* selftest --load-list now uses a new more agressive test loader that will
+  avoid loading unneeded modules and building their tests. Plugins can use
+  this new loader by defining a load_tests function instead of a test_suite
+  function. (a forthcoming patch will provide many examples on how to
+  implement this).
+  (Vincent Ladeuil)
+
+* selftest --load-list now does some sanity checks regarding duplicate test
+  IDs and tests present in the list but not found in the actual test suite.
+  (Vincent Ladeuil)
+
+* Slightly more concise format for the selftest progress bar, so there's
+  more space to show the test name.  (Martin Pool) ::
+
+    [2500/10884, 1fail, 3miss in 1m29s] test_revisionnamespaces.TestRev
+
+* The test suite takes much less memory to run, and is a bit faster.  This
+  is done by clearing most attributes of TestCases after running them, if
+  they succeeded.  (Andrew Bennetts)
+
+Internals
+*********
+
+* Added ``_build_client_protocol`` to ``_SmartClient``.  (Andrew Bennetts)
+
+* Added basic infrastructure for automatic plugin suggestion.
+  (Martin Albisetti)
+
+* If a ``LockableFiles`` object is not explicitly unlocked (for example
+  because of a missing ``try/finally`` block, it will give a warning but
+  not automatically unlock itself.  (Previously they did.)  This
+  sometimes caused knock-on errors if for example the network connection
+  had already failed, and should not be relied upon by code.
+  (Martin Pool, #109520)
+
+* ``make dist`` target to build a release tarball, and also
+  ``check-dist-tarball`` and ``dist-upload-escudero``.  (Martin Pool)
+
+* The ``read_response_tuple`` method of ``SmartClientRequestProtocol*``
+  classes will now raise ``UnknownSmartMethod`` when appropriate, so that
+  callers don't need to try distinguish unknown request errors from other
+  errors.  (Andrew Bennetts)
+
+* ``set_make_working_trees`` is now implemented provided on all repository
+  implementations (Aaron Bentley)
+
+* ``VersionedFile`` now has a new method ``get_parent_map`` which, like
+  ``Graph.get_parent_map`` returns a dict of key:parents. (Robert Collins)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.5.txt'
--- doc/en/release-notes/bzr-1.5.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.5.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,206 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.5
+#######
+
+:Released: 2008-05-16
+
+This release of Bazaar includes several updates to the documentation, and fixes
+to prepare for making rich root support the default format. Many bugs have been
+squashed, including fixes to log, bzr+ssh inter-operation with older servers.
+
+Changes
+*******
+
+* Suppress deprecation warnings when bzrlib is a 'final' release. This way
+  users of packaged software won't be bothered with DeprecationWarnings,
+  but developers and testers will still see them. (John Arbash Meinel)
+
+Documentation
+*************
+
+* Incorporate feedback from Jelmer Vernooij and Neil Martinsen-Burrell
+  on the plugin and integration chapters of the User Guide.
+  (Ian Clatworthy)
+
+
+bzr 1.5rc1
+##########
+
+:Released: 2008-05-09
+
+Changes
+*******
+
+* Broader support of GNU Emacs mail clients. Set
+  ``mail_client=emacsclient`` in your bazaar.conf and ``send`` will pop the
+  bundle in a mail buffer according to the value of ``mail-user-agent``
+  variable. (Xavier Maillard)
+
+Improvements
+************
+
+* Diff now handles revision specs like "branch:" and "submit:" more
+  efficiently.  (Aaron Bentley, #202928)
+
+* More friendly error given when attempt to start the smart server
+  on an address already in use. (Andrea Corbellini, #200575)
+
+* Pull completes much faster when there is nothing to pull.
+  (Aaron Bentley)
+
+Bugfixes
+********
+
+* Authentication.conf can define sections without password.
+  (Vincent Ladeuil, #199440)
+
+* Avoid muttering every time a child update does not cause a progress bar
+  update. (John Arbash Meinel, #213771)
+
+* ``Branch.reconcile()`` is now implemented. This allows ``bzr reconcile``
+  to fix when a Branch has a non-canonical mainline history. ``bzr check``
+  also detects this condition. (John Arbash Meinel, #177855)
+
+* ``bzr log -r ..X bzr://`` was failing, because it was getting a request
+  for ``revision_id=None`` which was not a string.
+  (John Arbash Meinel, #211661)
+
+* ``bzr commit`` now works with Microsoft's FTP service.
+  (Andreas Deininger)
+
+* Catch definitions outside sections in authentication.conf.
+  (Vincent Ladeuil, #217650)
+
+* Conversion from non-rich-root to rich-root(-pack) updates inventory
+  sha1s, even when bundles are used.  (Aaron Bentley, #181391)
+
+* Conversion from non-rich-root to rich-root(-pack) works correctly even
+  though search keys are not topologically sorted.  (Aaron Bentley)
+
+* Conversion from non-rich-root to rich-root(-pack) works even when a
+  parent revision has a different root id.  (Aaron Bentley, #177874)
+
+* Disable strace testing until strace is fixed (see bug #103133) and emit a
+  warning when selftest ends to remind us of leaking tests.
+  (Vincent Ladeuil, #226769)
+
+* Fetching all revisions from a repository does not cause pack collisions.
+  (Robert Collins, Aaron Bentley, #212908)
+
+* Fix error about "attempt to add line-delta in non-delta knit".
+  (Andrew Bennetts, #217701)
+
+* Pushing a branch in "dirstate" format (Branch5) over bzr+ssh would break
+  if the remote server was < version 1.2. This was due to a bug in the
+  RemoteRepository.get_parent_map() fallback code.
+  (John Arbash Meinel, #214894)
+
+* Remove leftover code in ``bzr_branch`` that inappropriately creates
+  a ``branch-name`` file in the branch control directory.
+  (Martin Pool)
+
+* Set SO_REUSEADDR on server sockets of ``bzr serve`` to avoid problems
+  rebinding the socket when starting the server a second time.
+  (John Arbash Meinel, Martin Pool, #164288)
+
+* Severe performance degradation in fetching from knit repositories to
+  knits and packs due to parsing the entire revisions.kndx on every graph
+  walk iteration fixed by using the Repository.get_graph API.  There was
+  another regression in knit => knit fetching which re-read the index for
+  every revision each side had in common.
+  (Robert Collins, John Arbash Meinel)
+
+* When logging the changes to a particular file, there was a bug if there
+  were ghosts in the revision ancestry. (John Arbash Meinel, #209948)
+
+* xs4all's FTP server returns a temporary error when trying to list an
+  empty directory, rather than returning an empty list. Adding a
+  workaround so that we don't get spurious failures.
+  (John Arbash Meinel, #215522)
+
+Documentation
+*************
+
+* Expanded the User Guide to include new chapters on popular plugins and
+  integrating Bazaar into your environment. The *Best practices* chapter
+  was renamed to *Miscellaneous topics* as suggested by community
+  feedback as well. (Ian Clatworthy)
+
+* Document outlining strategies for TortoiseBzr. (Mark Hammond)
+
+* Improved the documentation on hooks. (Ian Clatworthy)
+
+* Update authentication docs regarding SSH agents.
+  (Vincent Ladeuil, #183705)
+
+Testing
+*******
+
+* Add ``thread_name_suffix`` parameter to SmartTCPServer_for_testing, to
+  make it easy to identify which test spawned a thread with an unhandled
+  exception. (Andrew Bennetts)
+
+* New ``--debugflag``/``-E`` option to ``bzr selftest`` for setting
+  options for debugging tests, these are complementary to the -D
+  options.  The ``-Dselftest_debug`` global option has been replaced by the
+  ``-E=allow_debug`` option for selftest. (Andrew Bennetts)
+
+* Parameterised test ids are preserved correctly to aid diagnosis of test
+  failures. (Robert Collins, Andrew Bennetts)
+
+* selftest now accepts --starting-with <id> to load only the tests whose id
+  starts with the one specified. This greatly speeds up running the test
+  suite on a limited set of tests and can be used to run the tests for a
+  single module, a single class or even a single test.  (Vincent Ladeuil)
+
+* The test suite modules have been modified to define load_tests() instead
+  of test_suite(). That speeds up selective loading (via --load-list)
+  significantly and provides many examples on how to migrate (grep for
+  load_tests).  (Vincent Ladeuil)
+
+Internals
+*********
+
+* ``Hooks.install_hook`` is now deprecated in favour of
+  ``Hooks.install_named_hook`` which adds a required ``name`` parameter, to
+  avoid having to call ``Hooks.name_hook``. (Daniel Watkins)
+
+* Implement xml8 serializer.  (Aaron Bentley)
+
+* New form ``@deprecated_method(deprecated_in(1, 5, 0))`` for making
+  deprecation wrappers.  (Martin Pool)
+
+* ``Repository.revision_parents`` is now deprecated in favour of
+  ``Repository.get_parent_map([revid])[revid]``. (Jelmer Vernooij)
+
+* The Python ``assert`` statement is no longer used in Bazaar source, and
+  a test checks this.  (Martin Pool)
+
+API Changes
+***********
+
+* ``bzrlib.status.show_pending_merges`` requires the repository to be
+  locked by the caller. Callers should have been doing it anyway, but it
+  will now raise an exception if they do not. (John Arbash Meinel)
+
+* Repository.get_data_stream, Repository.get_data_stream_for_search(),
+  Repository.get_deltas_for_revsions(), Repository.revision_trees(),
+  Repository.item_keys_introduced_by() no longer take read locks.
+  (Aaron Bentley)
+
+* ``LockableFiles.get_utf8`` and ``.get`` are deprecated, as a start
+  towards removing LockableFiles and ``.control_files`` entirely.
+  (Martin Pool)
+
+* Methods deprecated prior to 1.1 have been removed.
+  (Martin Pool)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.6.txt'
--- doc/en/release-notes/bzr-1.6.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.6.txt	2010-11-12 22:43:38 +0000
@@ -0,0 +1,818 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.6.1
+#########
+
+:Released: 2008-09-05
+
+A couple regressions were found in the 1.6 release. There was a
+performance issue when using ``bzr+ssh`` to branch large repositories,
+and some problems with stacking and ``rich-root`` capable repositories.
+
+
+bzr 1.6.1rc2
+############
+
+:Released: 2008-09-03
+
+Bug Fixes
+*********
+
+* Copying between ``rich-root`` and ``rich-root-pack`` (and vice
+  versa) was accidentally using the inter-model fetcher, instead of
+  recognizing that both were 'rich root' formats.
+  (John Arbash Meinel, #264321)
+
+
+bzr 1.6.1rc1
+############
+
+:Released: 2008-08-29
+
+This release fixes a few regressions found in the 1.6 client. Fetching
+changes was using an O(N^2) buffering algorithm, so for large projects it
+would cause memory thrashing. There is also a specific problem with the
+``--1.6-rich-root`` format, which prevented stacking on top of
+``--rich-root-pack`` repositories, and could allow users to accidentally
+fetch experimental data (``-subtree``) without representing it properly.
+The ``--1.6-rich-root`` format has been deprecated and users are
+recommended to upgrade to ``--1.6.1-rich-root`` immediately.  Also we
+re-introduced a workaround for users who have repositories with incorrect
+nodes (not possible if you only used official releases).
+I should also clarify that none of this is data loss level issues, but
+still sufficient enough to warrant an updated release.
+
+Bug Fixes
+*********
+
+* ``RemoteTransport.readv()`` was being inefficient about how it
+  buffered the readv data and processed it. It would keep appending to
+  the same string (causing many copies) and then pop bytes out of the
+  start of the string (causing more copies).
+  With this patch "bzr+ssh://local" can improve dramatically,
+  especially for projects with large files.
+  (John Arbash Meinel)
+
+* Revision texts were always meant to be stored as fulltexts. There
+  was a bug in a bzr.dev version that would accidentally create deltas
+  when copying from a Pack repo to a Knit repo. This has been fixed,
+  but to support those repositories, we know always request full texts
+  for Revision texts. (John Arbash Meinel, #261339)
+
+* The previous ``--1.6-rich-root`` format used an incorrect XML
+  serializer, which would accidentally support fetching from a
+  repository that supported subtrees, even though the local one would
+  not. We deprecated that format, and introduced a new one that uses
+  the correct serializer ``--1.6.1-rich-root``.
+  (John Arbash Meinel, #262333)
+
+
+bzr 1.6
+#######
+
+:Released: 2008-08-25
+
+Finally, the long awaited bzr 1.6 has been released. This release includes
+new features like Stacked Branches, improved weave merge, and an updated
+server protocol (now on v3) which will allow for better cross version
+compatibility. With this release we have deprecated Knit format
+repositories, and recommend that users upgrade them, we will continue to
+support reading and writing them for the forseeable future, but we will
+not be tuning them for performance as pack repositories have proven to be
+better at scaling. This will also be the first release to bundle
+TortoiseBzr in the standalone Windows installer.
+
+
+bzr 1.6rc5
+##########
+
+:Released: 2008-08-19
+
+Bug Fixes
+*********
+
+* Disable automatic detection of stacking based on a containing
+  directory of the target. It interacted badly with push, and needs a
+  bit more work to get the edges polished before it should happen
+  automatically. (John Arbash Meinel, #259275)
+  (This change was reverted when merged to bzr.dev)
+
+
+bzr 1.6rc4
+##########
+
+:Released: 2008-08-18
+
+Bug Fixes
+*********
+
+* Fix a regression in knit => pack fetching.  We had a logic
+  inversion, causing the fetch to insert fulltexts in random order,
+  rather than preserving deltas.  (John Arbash Meinel, #256757)
+
+
+bzr 1.6rc3
+##########
+
+:Released: 2008-08-14
+
+Changes
+*******
+
+* Disable reading ``.bzrrules`` as a per-branch rule preferences
+  file. The feature was not quite ready for a full release.
+  (Robert Collins)
+
+Improvements
+************
+
+* Update the windows installer to bundle TortoiseBzr and ``qbzr``
+  into the standalone installer. This will be the first official
+  windows release that installs Tortoise by default.
+  (Mark Hammond)
+
+Bug Fixes
+*********
+
+* Fix a regression in ``bzr+http`` support. There was a missing
+  function (``_read_line``) that needed to be carried over from
+  ``bzr+ssh`` support. (Andrew Bennetts)
+
+* ``GraphIndex`` objects will internally read an entire index if more
+  than 1/20th of their keyspace is requested in a single operation.
+  This largely mitigates a performance regression in ``bzr log FILE``
+  and completely corrects the performance regression in ``bzr log``.
+  The regression was caused by removing an accomodation which had been
+  supporting the index format in use. A newer index format is in
+  development which is substantially faster. (Robert Collins)
+
+
+bzr 1.6rc2
+##########
+
+:Released: 2008-08-13
+
+This release candidate has a few minor bug fixes, and some regression
+fixes for Windows.
+
+Bug Fixes
+*********
+
+* ``bzr upgrade`` on remote branches accessed via bzr:// and
+  bzr+ssh:// now works.  (Andrew Bennetts)
+
+* Change the ``get_format_description()`` strings for
+  ``RepositoryFormatKnitPack5`` et al to be single line messages.
+  (Aaron Bentley)
+
+* Fix for a regression on Win32 where we would try to call
+  ``os.listdir()`` on a file and not catch the exception properly.
+  (Windows raises a different exception.) This would manifest in
+  places like ``bzr rm file`` or ``bzr switch``.
+  (Mark Hammond, John Arbash Meinel)
+
+* ``Inventory.copy()`` was failing to set the revision property for
+  the root entry. (Jelmer Vernooij)
+
+* SFTP transport: added missing ``FileExists`` case to
+  ``_translate_io_exception`` (Christophe Troestler, #123475)
+
+* The help for ``bzr ignored`` now suggests ``bzr ls --ignored`` for
+  scripting use. (Robert Collins, #3834)
+
+* The default ``annotate`` logic will now always assign the
+  last-modified value of a line to one of the revisions that modified
+  it, rather than a merge revision. This would happen when both sides
+  claimed to have modified the line resulting in the same text. The
+  choice is arbitrary but stable, so merges in different directions
+  will get the same results.  (John Arbash Meinel, #232188)
+
+
+bzr 1.6rc1
+##########
+
+:Released: 2008-08-06
+
+This release candidate for bzr 1.6 solidifies the new branch stacking
+feature.  Bazaar now recommends that users upgrade all knit repositories,
+because later formats are much faster.  However, we plan to continue read/write and
+upgrade support for knit repostories for the forseeable future.  Several
+other bugs and performance issues were fixed.
+
+Changes
+*******
+
+* Knit format repositories are deprecated and bzr will now emit
+  warnings whenever it encounters one.  Use ``bzr upgrade`` to upgrade
+  knit repositories to pack format.  (Andrew Bennetts)
+
+Improvements
+************
+
+* ``bzr check`` can now be told which elements at a location it should
+  check.  (Daniel Watkins)
+
+* Commit now supports ``--exclude`` (or ``-x``) to exclude some files
+  from the commit. (Robert Collins, #3117)
+
+* Fetching data between repositories that have the same model but no
+  optimised fetcher will not reserialise all the revisions, increasing
+  performance. (Robert Collins, John Arbash Meinel)
+
+* Give a more specific error when target branch is not reachable.
+  (James Westby)
+
+* Implemented a custom ``walkdirs_utf8`` implementation for win32.
+  This uses a pyrex extension to get direct access to the
+  ``FindFirstFileW`` style apis, rather than using ``listdir`` +
+  ``lstat``. Shows a very strong improvement in commands like
+  ``status`` and ``diff`` which have to iterate the working tree.
+  Anywhere from 2x-6x faster depending on the size of the tree (bigger
+  trees, bigger benefit.) (John Arbash Meinel)
+
+* New registry for log properties handles  and the method in
+  LongLogFormatter to display the custom properties returned by the
+  registered handlers. (Guillermo Gonzalez, #162469)
+
+Bug Fixes
+*********
+
+* Add more tests that stacking does not create deltas spanning
+  physical repository boundaries.
+  (Martin Pool, #252428)
+
+* Better message about incompatible repositories.
+  (Martin Pool, #206258)
+
+* ``bzr branch --stacked`` ensures the destination branch format can
+  support stacking, even if the origin does not.
+  (Martin Pool)
+
+* ``bzr export`` no longer exports ``.bzrrules``.
+  (Ian Clatworthy)
+
+* ``bzr serve --directory=/`` now correctly allows the whole
+  filesystem to be accessed on Windows, not just the root of the drive
+  that Python is running from.
+  (Adrian Wilkins, #240910)
+
+* Deleting directories by hand before running ``bzr rm`` will not
+  cause subsequent errors in ``bzr st`` and ``bzr commit``.
+  (Robert Collins, #150438)
+
+* Fix a test case that was failing if encoding wasn't UTF-8.
+  (John Arbash Meinel, #247585)
+
+* Fix "no buffer space available" error when branching with the new
+  smart server protocol to or from Windows.
+  (Andrew Bennetts, #246180)
+
+* Fixed problem in branching from smart server.
+  (#249256, Michael Hudson, Martin Pool)
+
+* Handle a file turning in to a directory in TreeTransform.
+  (James Westby, #248448)
+
+API Changes
+***********
+
+* ``MutableTree.commit`` has an extra optional keywork parameter
+  ``exclude`` that will be unconditionally supplied by the command
+  line UI - plugins that add tree formats may need an update.
+  (Robert Collins)
+
+* The API minimum version for plugin compatibility has been raised to
+  1.6 - there are significant changes throughout the code base.
+  (Robert Collins)
+
+* The generic fetch code now uses three attributes on Repository objects
+  to control fetch. The streams requested are controlled via :
+  ``_fetch_order`` and ``_fetch_uses_deltas``. Setting these
+  appropriately allows different repository implementations to recieve
+  data in their optimial form. If the ``_fetch_reconcile`` is set then
+  a reconcile operation is triggered at the end of the fetch.
+  (Robert Collins)
+
+* The ``put_on_disk`` and ``get_tar_item`` methods in
+  ``InventoryEntry`` were deprecated. (Ian Clatworthy)
+
+* ``Repository.is_shared`` doesn't take a read lock. It didn't
+  need one in the first place (nobody cached the value, and
+  ``RemoteRepository`` wasn't taking one either). This saves a round
+  trip when probing Pack repositories, as they read the ``pack-names``
+  file when locked. And during probe, locking the repo isn't very
+  useful. (John Arbash Meinel)
+
+Internals
+*********
+
+* ``bzrlib.branchbuilder.BranchBuilder`` is now much more capable of
+  putting together a real history without having to create a full
+  WorkingTree. It is recommended that tests that are not directly
+  testing the WorkingTree use BranchBuilder instead.  See
+  ``BranchBuilder.build_snapshot`` or
+  ``TestCaseWithMemoryTree.make_branch_builder``.  (John Arbash Meinel)
+
+* ``bzrlib.builtins.internal_tree_files`` broken into two giving a new
+  helper ``safe_relpath_files`` - used by the new ``exclude``
+  parameter to commit. (Robert Collins)
+
+* Make it easier to introduce new WorkingTree formats.
+  (Ian Clatworthy)
+
+* The code for exporting trees was refactored not to use the
+  deprecated ``InventoryEntry`` methods. (Ian Clatworthy)
+
+* RuleSearchers return () instead of [] now when there are no matches.
+  (Ian Clatworthy)
+
+
+bzr 1.6beta3
+############
+
+:Released: 2008-07-17
+
+This release adds a new 'stacked branches' feature allowing branches to
+share storage without being in the same repository or on the same machine.
+(See the user guide for more details.)  It also adds a new hook, improved
+weaves, aliases for related locations, faster bzr+ssh push, and several
+bug fixes.
+
+Features
+********
+
+* New ``pre_change_branch_tip`` hook that is called before the
+  branch tip is moved, while the branch is write-locked.  See the User
+  Reference for signature details.  (Andrew Bennetts)
+
+* Rule-based preferences can now be defined for selected files in
+  selected branches, allowing commands and plugins to provide
+  custom behaviour for files matching defined patterns.
+  See ``Rule-based preferences`` (part of ``Configuring Bazaar``)
+  in the User Guide and ``bzr help rules`` for more information.
+  (Ian Clatworthy)
+
+* Sites may suggest a branch to stack new branches on.  (Aaron Bentley)
+
+* Stacked branches are now supported. See ``bzr help branch`` and
+  ``bzr help push``.  Branches must be in the ``development1`` format
+  to stack, though the stacked-on branch can be of any format.
+  (Robert Collins)
+
+Improvements
+************
+
+* ``bzr export --format=tgz --root=NAME -`` to export a gzipped tarball
+  to stdout; also ``tar`` and ``tbz2``.
+  (Martin Pool)
+
+* ``bzr (re)merge --weave`` will now use a standard Weave algorithm,
+  rather than the annotation-based merge it was using. It does so by
+  building up a Weave of the important texts, without needing to build
+  the full ancestry. (John Arbash Meinel, #238895)
+
+* ``bzr send`` documents and better supports ``emacsclient`` (proper
+  escaping of mail headers and handling of the MUA Mew).
+  (Christophe Troestler)
+
+* Remembered locations can be specified by aliases, e.g. :parent, :public,
+  :submit.  (Aaron Bentley)
+
+* The smart protocol now has improved support for setting branches'
+  revision info directly.  This makes operations like push
+  faster.  The new request method name is
+  ``Branch.set_last_revision_ex``.  (Andrew Bennetts)
+
+Bug Fixes
+*********
+
+* Bazaar is now able to be a client to the web server of IIS 6 and 7.
+  The broken implementations of RFC822 in Python and RFC2046 in IIS
+  combined with boundary-line checking in Bazaar previously made this
+  impossible. (NB, IIS 5 does not suffer from this problem).
+  (Adrian Wilkins, #247585)
+
+* ``bzr log --long`` with a ghost in your mainline now handles that
+  ghost properly. (John Arbash Meinel, #243536)
+
+* ``check`` handles the split-up .bzr layout correctly, so no longer
+  requires a branch to be present.
+  (Daniel Watkins, #64783)
+
+* Clearer message about how to set the PYTHONPATH if bzrlib can't be
+  loaded.
+  (Martin Pool, #205230)
+
+* Errors about missing libraries are now shown without a traceback,
+  and with a suggestion to install the library.  The full traceback is
+  still in ``.bzr.log`` and can be shown with ``-Derror``.
+  (Martin Pool, #240161)
+
+* Fetch from a stacked branch copies all required data.
+  (Aaron Bentley, #248506)
+
+* Handle URLs such as ftp://user@host.com@www.host.com where the user
+  name contains an @.
+  (Neil Martinsen-Burrell, #228058)
+
+* ``needs_read_lock`` and ``needs_write_lock`` now suppress an error during
+  ``unlock`` if there was an error in the original function. This helps
+  most when there is a failure with a smart server action, since often the
+  connection closes and we cannot unlock.
+  (Andrew Bennetts, John Arbash Meinel, #125784)
+
+* Obsolete hidden command ``bzr fetch`` removed.
+  (Martin Pool, #172870)
+
+* Raise the correct exception when doing ``-rbefore:0`` or ``-c0``.
+  (John Arbash Meinel, #239933)
+
+* You can now compare file revisions in Windows diff programs from
+  Cygwin Bazaar.
+  (Matt McClure, #209281)
+
+* revision_history now tolerates mainline ghosts for Branch format 6.
+  (Aaron Bentley, #235055)
+
+* Set locale from environment for third party libs.
+  (Martin von Gagern, #128496)
+
+Documentation
+*************
+
+* Added *Using stacked branches* to the User Guide.
+  (Ian Clatworthy)
+
+* Updated developer documentation.
+  (Martin Pool)
+
+Testing
+*******
+
+* ``-Dmemory`` will cause /proc/PID/status to be catted before bzr
+  exits, allowing low-key analysis of peak memory use. (Robert Collins)
+
+* ``TestCaseWithTransport.make_branch_and_tree`` tries harder to return
+  a tree with a ``branch`` attribute of the right format.  This was
+  preventing some ``RemoteBranch`` tests from actually running with
+  ``RemoteBranch`` instances.  (Andrew Bennetts)
+
+API Changes
+***********
+
+* Removed ``Repository.text_store``, ``control_store``, etc.  Instead,
+  there are new attributes ``texts, inventories, revisions,
+  signatures``, each of which is a ``VersionedFiles``.  See the
+  Repository docstring for more details.
+  (Robert Collins)
+
+* ``Branch.pull`` now accepts an ``_override_hook_target`` optional
+  parameter.  If you have a subclass of ``Branch`` that overrides
+  ``pull`` then you should add this parameter.  (Andrew Bennetts)
+
+* ``bzrlib.check.check()`` has been deprecated in favour of the more
+  aptly-named ``bzrlib.check.check_branch()``.
+  (Daniel Watkins)
+
+* ``Tree.print_file`` and ``Repository.print_file`` are deprecated.
+  These methods are bad APIs because they write directly to sys.stdout.
+  bzrlib does not use them internally, and there are no direct tests
+  for them. (Alexander Belchenko)
+
+Internals
+*********
+
+* ``cat`` command no longer uses ``Tree.print_file()`` internally.
+  (Alexander Belchenko)
+
+* New class method ``BzrDir.open_containing_tree_branch_or_repository``
+  which eases the discovery of the tree, the branch and the repository
+  containing a given location.
+  (Daniel Watkins)
+
+* New ``versionedfile.KeyMapper`` interface to abstract out the access to
+  underlying .knit/.kndx etc files in repositories with partitioned
+  storage. (Robert Collins)
+
+* Obsolete developer-use command ``weave-join`` has been removed.
+  (Robert Collins)
+
+* ``RemoteToOtherFetcher`` and ``get_data_stream_for_search`` removed,
+  to support new ``VersionedFiles`` layering.
+  (Robert Collins)
+
+
+bzr 1.6beta2
+############
+
+:Released: 2008-06-10
+
+This release contains further progress towards our 1.6 goals of shallow
+repositories, and contains a fix for some user-affecting bugs in the
+repository layer.  Building working trees during checkout and branch is
+now faster.
+
+Bug Fixes
+*********
+
+* Avoid KnitCorrupt error extracting inventories from some repositories.
+  (The data is not corrupt; an internal check is detecting a problem
+  reading from the repository.)
+  (Martin Pool, Andrew Bennetts, Robert Collins, #234748)
+
+* ``bzr status`` was breaking if you merged the same revision twice.
+  (John Arbash Meinel, #235407)
+
+* Fix infinite loop consuming 100% CPU when a connection is lost while
+  reading a response body via the smart protocol v1 or v2.
+  (Andrew Bennetts)
+
+* Inserting a bundle which changes the contents of a file with no trailing
+  end of line, causing a knit snapshot in a 'knits' repository will no longer
+  cause KnitCorrupt. (Robert Collins)
+
+* ``RemoteBranch.pull`` needs to return the ``self._real_branch``'s
+  pull result. It was instead just returning None, which breaks ``bzr
+  pull``. (John Arbash Meinel, #238149)
+
+* Sanitize branch nick before using it as an attachment filename in
+  ``bzr send``. (Lukáš Lalinský, #210218)
+
+* Squash ``inv_entry.symlink_target`` to a plain string when
+  generating DirState details. This prevents from getting a
+  ``UnicodeError`` when you have symlinks and non-ascii filenames.
+  (John Arbash Meinel, #135320)
+
+Improvements
+************
+
+* Added the 'alias' command to set/unset and display aliases. (Tim Penhey)
+
+* ``added``, ``modified``, and ``unknowns`` behaviour made consistent (all three
+  now quote paths where required). Added ``--null`` option to ``added`` and
+  ``modified`` (for null-separated unknowns, use ``ls --unknown --null``)
+  (Adrian Wilkins)
+
+* Faster branching (1.09x) and lightweight checkouts (1.06x) on large trees.
+  (Ian Clatworthy, Aaron Bentley)
+
+Documentation
+*************
+
+* Added *Bazaar Zen* section to the User Guide. (Ian Clatworthy)
+
+Testing
+*******
+
+* Fix the test HTTPServer to be isolated from chdir calls made while it is
+  running, allowing it to be used in blackbox tests. (Robert Collins)
+
+API Changes
+***********
+
+* ``WorkingTree.set_parent_(ids/trees)`` will now filter out revisions
+  which are in the ancestry of other revisions. So if you merge the same
+  tree twice, or merge an ancestor of an existing merge, it will only
+  record the newest. (If you merge a descendent, it will replace its
+  ancestor). (John Arbash Meinel, #235407)
+
+* ``RepositoryPolicy.__init__`` now requires stack_on and stack_on_pwd,
+  through the derived classes do not.  (Aaron Bentley)
+
+Internals
+*********
+
+* ``bzrlib.bzrdir.BzrDir.sprout`` now accepts ``stacked`` to control
+  creating stacked branches. (Robert Collins)
+
+* Knit record serialisation is now stricter on what it will accept, to
+  guard against potential internal bugs, or broken input. (Robert Collins)
+
+bzr 1.6beta1
+############
+
+:Released: 2008-06-02
+
+Commands that work on the revision history such as push, pull, missing,
+uncommit and log are now substantially faster.  This release adds a
+translation of some of the user documentation into Spanish.  (Contributions of
+other translations would be very welcome.)  Bazaar 1.6beta1 adds a new network
+protocol which is used by default and which allows for more efficient transfers
+and future extensions.
+
+
+Notes When Upgrading
+********************
+
+* There is a new version of the network protocol used for bzr://, bzr+ssh://
+  and bzr+http:// connections.  This will allow more efficient requests and
+  responses, and more graceful fallback when a server is too old to
+  recognise a request from a more recent client.  Bazaar 1.6 will
+  interoperate with 0.16 and later versions, but servers should be upgraded
+  when possible.  Bazaar 1.6 no longer interoperates with 0.15 and earlier via
+  these protocols.  Use alternatives like SFTP or upgrade those servers.
+  (Andrew Bennetts, #83935)
+
+Changes
+*******
+
+* Deprecation warnings will not be suppressed when running ``bzr selftest``
+  so that developers can see if their code is using deprecated functions.
+  (John Arbash Meinel)
+
+Features
+********
+
+* Adding ``-Derror`` will now display a traceback when a plugin fails to
+  load. (James Westby)
+
+Improvements
+************
+
+* ``bzr branch/push/pull -r XXX`` now have a helper function for finding
+  the revno of the new revision (``Graph.find_distance_to_null``). This
+  should make something like ``bzr branch -r -100`` in a shared, no-trees
+  repository much snappier. (John Arbash Meinel)
+
+* ``bzr log --short -r X..Y`` no longer needs to access the full revision
+  history. This makes it noticeably faster when logging the last few
+  revisions. (John Arbash Meinel)
+
+* ``bzr ls`` now accepts ``-V`` as an alias for ``--versioned``.
+  (Jerad Cramp, #165086)
+
+* ``bzr missing`` uses the new ``Graph.find_unique_ancestors`` and
+  ``Graph.find_differences`` to determine missing revisions without having
+  to search the whole ancestry. (John Arbash Meinel, #174625)
+
+* ``bzr uncommit`` now uses partial history access, rather than always
+  extracting the full revision history for a branch. This makes it
+  resolve the appropriate revisions much faster (in testing it drops
+  uncommit from 1.5s => 0.4s). It also means ``bzr log --short`` is one
+  step closer to not using full revision history.
+  (John Arbash Meinel, #172649)
+
+Bugfixes
+********
+
+* ``bzr merge --lca`` should handle when two revisions have no common
+  ancestor other than NULL_REVISION. (John Arbash Meinel, #235715)
+
+* ``bzr status`` was breaking if you merged the same revision twice.
+  (John Arbash Meinel, #235407)
+
+* ``bzr push`` with both ``--overwrite`` and ``-r NNN`` options no longer
+  fails.  (Andrew Bennetts, #234229)
+
+* Correctly track the base URL of a smart medium when using bzr+http://
+  URLs, which was causing spurious "No repository present" errors with
+  branches in shared repositories accessed over bzr+http.
+  (Andrew Bennetts, #230550)
+
+* Define ``_remote_is_at_least_1_2`` on ``SmartClientMedium`` so that all
+  implementations have the attribute.  Fixes 'PyCurlTransport' object has no
+  attribute '_remote_is_at_least_1_2' attribute errors.
+  (Andrew Bennetts, #220806)
+
+* Failure to delete an obsolete pack file should just give a warning
+  message, not a fatal error.  It may for example fail if the file is still
+  in use by another process.
+  (Martin Pool)
+
+* Fix MemoryError during large fetches over HTTP by limiting the amount of
+  data we try to read per ``recv`` call.  The problem was observed with
+  Windows and a proxy, but might affect other environments as well.
+  (Eric Holmberg, #215426)
+
+* Handle old merge directives correctly in Merger.from_mergeable.  Stricter
+  get_parent_map requirements exposed a latent bug here.  (Aaron Bentley)
+
+* Issue a warning and ignore passwords declared in authentication.conf when
+  used for an SSH scheme (sftp:// or bzr+ssh://).
+  (Vincent Ladeuil, #203186)
+
+* Make both HTTP implementations raise appropriate exceptions on 403
+  Forbidden when POSTing smart requests.
+  (Vincent Ladeuil, #230223)
+
+* Properly *title* header names in HTTP requests instead of capitalizing
+  them.
+  (Vincent Ladeuil, #229076)
+
+* The "Unable to obtain lock" error message now also suggests using
+  ``bzr break-lock`` to fix it.  (Martin Albisetti, #139202)
+
+* Treat an encoding of '' as ascii; this can happen when bzr is run
+  under vim on Mac OS X.
+  (Neil Martinsen-Burrell)
+
+* ``VersionedFile.make_mpdiffs()`` was raising an exception that wasn't in
+  scope. (Daniel Fischer #235687)
+
+Documentation
+*************
+
+* Added directory structure and started translation of docs in spanish.
+  (Martin Albisetti, Lucio Albenga)
+
+* Incorporate feedback from Jelmer Vernooij and Neil Martinsen-Burrell
+  on the plugin and integration chapters of the User Guide.
+  (Ian Clatworthy)
+
+* More Bazaar developer documentation about packaging and release process,
+  and about use of Python reprs.
+  (Martin Pool, Martin Albisetti)
+
+* Updated Tortise strategy document. (Mark Hammond)
+
+Testing
+*******
+
+* ``bzrlib.tests.adapt_tests`` was broken and unused - it has been fixed.
+  (Robert Collins)
+
+* Fix the test HTTPServer to be isolated from chdir calls made while it is
+  running, allowing it to be used in blackbox tests. (Robert Collins)
+
+* New helper function for splitting test suites
+  ``split_suite_by_condition``. (Robert Collins)
+
+Internals
+*********
+
+* ``Branch.missing_revisions`` has been deprecated. Similar functionality
+  can be obtained using ``bzrlib.missing.find_unmerged``. The api was
+  fairly broken, and the function was unused, so we are getting rid of it.
+  (John Arbash Meinel)
+
+API Changes
+***********
+
+* ``Branch.abspath`` is deprecated; use the Tree or Transport
+  instead.  (Martin Pool)
+
+* ``Branch.update_revisions`` now takes an optional ``Graph``
+  object. This can be used by ``update_revisions`` when it is
+  checking ancestry, and allows callers to prefer request to go to a
+  local branch.  (John Arbash Meinel)
+
+* Branch, Repository, Tree and BzrDir should expose a Transport as an
+  attribute if they have one, rather than having it indirectly accessible
+  as ``.control_files._transport``.  This doesn't add a requirement
+  to support a Transport in cases where it was not needed before;
+  it just simplifies the way it is reached.  (Martin Pool)
+
+* ``bzr missing --mine-only`` will return status code 0 if you have no
+  new revisions, but the remote does. Similarly for ``--theirs-only``.
+  The new code only checks one side, so it doesn't know if the other
+  side has changes. This seems more accurate with the request anyway.
+  It also changes the output to print '[This|Other] branch is up to
+  date.' rather than displaying nothing.  (John Arbash Meinel)
+
+* ``LockableFiles.put_utf8``, ``put_bytes`` and ``controlfilename``
+  are now deprecated in favor of using Transport operations.
+  (Martin Pool)
+
+* Many methods on ``VersionedFile``, ``Repository`` and in
+  ``bzrlib.revision``  deprecated before bzrlib 1.5 have been removed.
+  (Robert Collins)
+
+* ``RevisionSpec.wants_revision_history`` can be set to False for a given
+  ``RevisionSpec``. This will disable the existing behavior of passing in
+  the full revision history to ``self._match_on``. Useful for specs that
+  don't actually need access to the full history. (John Arbash Meinel)
+
+* The constructors of ``SmartClientMedium`` and its subclasses now require a
+  ``base`` parameter.  ``SmartClientMedium`` implementations now also need
+  to provide a ``remote_path_from_transport`` method.  (Andrew Bennetts)
+
+* The default permissions for creating new files and directories
+  should now be obtained from ``BzrDir._get_file_mode()`` and
+  ``_get_dir_mode()``, rather than from LockableFiles.  The ``_set_file_mode``
+  and ``_set_dir_mode`` variables on LockableFiles which were advertised
+  as a way for plugins to control this are no longer consulted.
+  (Martin Pool)
+
+* ``VersionedFile.join`` is deprecated. This method required local
+  instances of both versioned file objects and was thus hostile to being
+  used for streaming from a smart server. The new get_record_stream and
+  insert_record_stream are meant to efficiently replace this method.
+  (Robert Collins)
+
+* ``WorkingTree.set_parent_(ids/trees)`` will now filter out revisions
+  which are in the ancestry of other revisions. So if you merge the same
+  tree twice, or merge an ancestor of an existing merge, it will only
+  record the newest. (If you merge a descendent, it will replace its
+  ancestor). (John Arbash Meinel, #235407)
+
+* ``WorkingTreeFormat2.stub_initialize_remote`` is now private.
+  (Martin Pool)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.7.txt'
--- doc/en/release-notes/bzr-1.7.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.7.txt	2010-11-12 22:46:28 +0000
@@ -0,0 +1,282 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.7.1
+#########
+
+:Released:  2008-10-01
+
+No changes from 1.7.1rc1.
+
+
+bzr 1.7.1rc1
+############
+
+:Released: 2008-09-24
+
+This release just includes an update to how the merge algorithm handles
+file paths when we encounter complex history.
+
+Features
+********
+
+* If we encounter a criss-cross in history, use information from
+  direct Least Common Ancestors to resolve inventory shape (locations
+  of files, adds, deletes, etc). This is similar in concept to using
+  ``--lca`` for merging file texts, only applied to paths.
+  (John Arbash Meinel)
+
+
+bzr 1.7
+#######
+
+:Released: 2008-09-23
+
+This release includes many bug fixes and a few performance and feature
+improvements.  ``bzr rm`` will now scan for missing files and remove them,
+like how ``bzr add`` scans for unknown files and adds them. A bit more
+polish has been applied to the stacking code. The b-tree indexing code has
+been brought in, with an eye on using it in a future repository format.
+There are only minor installer changes since bzr-1.7rc2.
+
+Features
+********
+
+* Some small updates to the win32 installer. Include localization
+  files found in plugins, and include the builtin distutils as part of
+  packaging qbzr. (Mark Hammond)
+
+
+bzr 1.7rc2
+##########
+
+:Released: 2008-09-17
+
+A few bug fixes from 1.7rc1. The biggest change is a new
+``RemoteBranch.get_stacked_on_url`` RPC. This allows clients that are
+trying to access a Stacked branch over the smart protocol, to properly
+connect to the stacked-on location.
+
+Bug Fixes
+*********
+
+* Branching from a shared repository on a smart server into a new
+  repository now preserves the repository format.
+  (Andrew Bennetts, #269214)
+
+* Branching from a stacked branch via ``bzr+ssh`` can properly connect
+  to the stacked-on branch.  (Martin Pool, #261315)
+
+* ``bzr init`` no longer re-opens the BzrDir multiple times.
+  (Vincent Ladeuil)
+
+* Fix '_in_buffer' AttributeError when using the -Dhpss debug flag.
+  (Andrew Bennetts)
+
+
+bzr 1.7rc1
+##########
+
+:Released: 2008-09-09
+
+This release candidate for bzr 1.7 has several bug fixes and a few
+performance and feature improvements.  ``bzr rm`` will now scan for
+missing files and remove them, like how ``bzr add`` scans for unknown
+files and adds them. A bit more polish has been applied to the stacking
+code. The b-tree indexing code has been brought in, with an eye on using
+it in a future repository format.
+
+
+Changes
+*******
+
+* ``bzr export`` can now export a subdirectory of a project.
+  (Robert Collins)
+
+* ``bzr remove-tree`` will now refuse to remove a tree with uncommitted
+  changes, unless the ``--force`` option is specified.
+  (Lukáš Lalinský, #74101)
+
+* ``bzr rm`` will now scan for files that are missing and remove just
+  them automatically, much as ``bzr add`` scans for new files that
+  are not ignored and adds them automatically. (Robert Collins)
+
+Features
+********
+
+* Support for GSSAPI authentication when using FTP as documented in
+  RFC2228. (Jelmer Vernooij, #49623)
+
+* Add support for IPv6 in the smart server. (Jelmer Vernooij, #165014)
+
+Improvements
+************
+
+* A URL like ``log+file:///tmp`` will log all access to that Transport
+  to ``.bzr.log``, which may help in debugging or profiling.
+  (Martin Pool)
+
+* ``bzr branch`` and ``bzr push`` use the default stacking policy if the
+  branch format supports it. (Aaron Bentley)
+
+* ``bzr init`` and ``bzr init-repo`` will now print out the same as
+  ``bzr info`` if it completed successfully.
+  (Marius Kruger)
+
+* ``bzr uncommit`` logs the old tip revision id, and displays how to
+  restore the branch to that tip using ``bzr pull``.  This allows you
+  to recover if you realize you uncommitted the wrong thing.
+  (John Arbash Meinel)
+
+* Fix problems in accessing stacked repositories over ``bzr://``.
+  (Martin Pool, #261315)
+
+* ``SFTPTransport.readv()`` was accidentally using ``list += string``,
+  which 'works', but adds each character separately to the list,
+  rather than using ``list.append(string)``. Fixing this makes the
+  SFTP transport a little bit faster (~20%) and use a bit less memory.
+  (John Arbash Meinel)
+
+* When reading index files, if we happen to read the whole file in a
+  single request treat it as a ``_buffer_all`` request. This happens
+  most often on small indexes over remote transports, where we default
+  to reading 64kB. It saves a round trip for each small index during
+  fetch operations. Also, if we have read more than 50% of an index
+  file, trigger a ``_buffer_all`` on the next request. This works
+  around some inefficiencies because reads don't fall neatly on page
+  boundaries, so we would ignore those bytes, but request them again
+  later. This could trigger a total read size of more than the whole
+  file. (John Arbash Meinel)
+
+Bug Fixes
+*********
+
+* ``bzr rm`` is now aliased to ``bzr del`` for the convenience of svn
+  users. (Robert Collins, #205416)
+
+* Catch the infamous "select/poll returned error" which occurs when
+  pycurl try to send a body request to an HTTP/1.0 server which has
+  already refused to handle the request. (Vincent Ladeuil, #225020)
+
+* Fix ``ObjectNotLocked`` errors when using various commands
+  (including ``bzr cat`` and ``bzr annotate``) in combination with a
+  smart server URL.  (Andrew Bennetts, #237067)
+
+* ``FTPTransport.stat()`` would return ``0000`` as the permission bits
+  for the containing ``.bzr/`` directory (it does not implement
+  permissions). This would cause us to set all subdirectories to
+  ``0700`` and files to ``0600`` rather than leaving them unmodified.
+  Now we ignore ``0000`` as the permissions and assume they are
+  invalid. (John Arbash Meinel, #259855)
+
+* Merging from a previously joined branch will no longer cause
+  a traceback. (Jelmer Vernooij, #203376)
+
+* Pack operations on windows network shares will work even with large
+  files. (Robert Collins, #255656)
+
+* Running ``bzr st PATH_TO_TREE`` will no longer suppress merge
+  status. Status is also about 7% faster on mozilla sized trees
+  when the path to the root of the tree has been given. Users of
+  the internal ``show_tree_status`` function should be aware that
+  the show_pending flag is now authoritative for showing pending
+  merges, as it was originally. (Robert Collins, #255204)
+
+* Set valid default _param_name for Option so that ListOption can embed
+  '-' in names. (Vincent Ladeuil, #263249)
+
+* Show proper error rather than traceback when an unknown revision
+  id is specified to ``bzr cat-revision``. (Jelmer Vernooij, #175569)
+
+* Trailing text in the dirstate file could cause the C dirstate parser
+  to try to allocate an invalid amount of memory. We now properly
+  check and test for parsing a dirstate with invalid trailing data.
+  (John Arbash Meinel, #186014)
+
+* Unexpected error responses from a smart server no longer cause the
+  client to traceback.  (Andrew Bennetts, #263527)
+
+* Use a Windows api function to get a Unicode host name, rather than
+  assuming the host name is ascii.
+  (Mark Hammond, John Arbash Meinel, #256550)
+
+* ``WorkingTree4`` trees will now correctly report missing-and-new
+  paths in the output of ``iter_changes``. (Robert Collins)
+
+Documentation
+*************
+
+* Updated developer documentation.  (Martin Pool)
+
+API Changes
+***********
+
+* Exporters now take 4 parameters. (Robert Collins)
+
+* ``Tree.iter_changes`` will now return False for the content change
+  field when a file is missing in the basis tree and not present in
+  the target tree. Previously it returned True unconditionally.
+  (Robert Collins)
+
+* The deprecated ``Branch.abspath`` and unimplemented
+  ``Branch.rename_one`` and ``Branch.move`` were removed. (Jelmer Vernooij)
+
+* BzrDir.clone_on_transport implementations must now accept a stacked_on
+  parameter.  (Aaron Bentley)
+
+* BzrDir.cloning_metadir implementations must now take a require_stacking
+  parameter.  (Aaron Bentley)
+
+Testing
+*******
+
+* ``addCleanup`` now takes ``*arguments`` and ``**keyword_arguments``
+  which are then passed to the cleanup callable as it is run. In
+  addition, addCleanup no longer requires that the callables passed to
+  it be unique. (Jonathan Lange)
+
+* Fix some tests that fail on Windows because files are deleted while
+  still in use.
+  (Mark Hammond)
+
+* ``selftest``'s ``--starting-with`` option can now use predefined
+  prefixes so that one can say ``bzr selftest -s bp.loom`` instead of
+  ``bzr selftest -s bzrlib.plugins.loom``. (Vincent Ladeuil)
+
+* ``selftest``'s ``--starting-with`` option now accepts multiple values.
+  (Vincent Ladeuil)
+
+Internals
+*********
+
+* A new plugin interface, ``bzrlib.log.log_adapters``, has been added.
+  This allows dynamic log output filtering by plugins.
+  (Robert Collins)
+
+* ``bzrlib.btree_index`` is now available, providing a b-tree index
+  layer. The design is memory conservative (limited memory cache),
+  faster to seek (approx 100 nodes per page, gives 100-way fan out),
+  and stores compressed pages allowing more keys per page.
+  (Robert Collins, John Arbash Meinel)
+
+* ``bzrlib.diff.DiffTree.show_diff`` now skips changes where the kind
+  is unknown in both source and target.
+  (Robert Collins, Aaron Bentley)
+
+* ``GraphIndexBuilder.add_node`` and ``BTreeBuilder`` have been
+  streamlined a bit. This should make creating large indexes faster.
+  (In benchmarking, it now takes less time to create a BTree index than
+  it takes to read the GraphIndex one.) (John Arbash Meinel)
+
+* Mail clients for `bzr send` are now listed in a registry.  This
+  allows plugins to add new clients by registering them with
+  ``bzrlib.mail_client.mail_client_registry``.  All of the built-in
+  clients now use this mechanism.  (Neil Martinsen-Burrell)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.8.txt'
--- doc/en/release-notes/bzr-1.8.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.8.txt	2010-11-12 22:08:18 +0000
@@ -0,0 +1,234 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.8
+#######
+
+:Released: 2008-10-16
+
+Bazaar 1.8 includes several fixes that improve working tree performance,
+display of revision logs, and merges.  The bzr testsuite now passes on OS
+X and Python 2.6, and almost completely passes on Windows.  The
+smartserver code has gained several bug fixes and performance
+improvements, and can now run server-side hooks within an HTTP server.
+
+Bug Fixes
+*********
+
+* Fix "Must end write group" error when another error occurs during
+  ``bzr push``.  (Andrew Bennetts, #230902)
+
+Portability
+***********
+
+* Some Pyrex versions require the WIN32 macro defined to compile on
+  that platform.  (Alexander Belchenko, Martin Pool, #277481)
+
+
+bzr 1.8rc1
+##########
+
+:Released: 2008-10-07
+
+Changes
+*******
+
+* ``bzr log file`` has been changed. It now uses a different method
+  for determining which revisions to show as merging the changes to
+  the file. It now only shows revisions which merged the change
+  towards your mainline. This simplifies the output, makes it faster,
+  and reduces memory consumption.  (John Arbash Meinel)
+
+* ``bzr merge`` now defaults to having ``--reprocess`` set, whenever
+  ``--show-base`` is not supplied.  (John Arbash Meinel)
+
+* ``bzr+http//`` will now optionally load plugins and write logs on the
+  server. (Marius Kruger)
+
+* ``bzrlib._dirstate_helpers_c.pyx`` does not compile correctly with
+  Pyrex 0.9.4.1 (it generates C code which causes segfaults). We
+  explicitly blacklist that version of the compiler for that
+  extension. Packaged versions will include .c files created with
+  pyrex >= 0.9.6 so it doesn't effect releases, only users running
+  from the source tree. (John Arbash Meinel, #276868)
+
+Features
+********
+
+* bzr is now compatible with python-2.6. python-2.6 is not yet officially
+  supported (nor released, tests were conducted with the dev version of
+  python-2.6rc2), but all known problems have been fixed.  Feedback
+  welcome.
+  (Vincent Ladeuil, #269535)
+
+Improvements
+************
+
+* ``bzr annotate`` will now include uncommitted changes from the local
+  working tree by default. Such uncommitted changes are given the
+  revision number they would get if a commit was done, followed with a
+  ? to indicate that its not actually known. (Robert Collins, #3439)
+
+* ``bzr branch`` now accepts a ``--standalone`` option, which creates a
+  standalone branch regardless of the presence of shared repositories.
+  (Daniel Watkins)
+
+* ``bzr push`` is faster in the case there are no new revisions to
+  push.  It is also faster if there are no tags in the local branch.
+  (Andrew Bennetts)
+
+* File changes during a commit will update the tree stat cache.
+  (Robert Collins)
+
+* Location aliases can now accept a trailing path.  (Micheal Hudson)
+
+* New hooks ``Lock.hooks`` when LockDirs are acquired and released.
+  (Robert Collins, MartinPool)
+
+* Switching in heavyweight checkouts uses the master branch's context, not
+  the checkout's context.  (Adrian Wilkins)
+
+* ``status`` on large trees is now faster, due to optimisations in the
+  walkdirs code. Of particular note, the walkdirs code now performs
+  a temporary ``chdir()`` while reading a single directory; if your
+  platform has non thread-local current working directories (and is
+  not windows which has its own implementation), this may introduce a
+  race condition during concurrent uses of bzrlib. The bzrlib CLI
+  will not encounter this as it is single threaded for working tree
+  operations. (Robert Collins)
+
+* The C extensions now build on python 2.4 (Robert Collins, #271939)
+
+* The ``-Dhpss`` debug flag now reports the number of smart server
+  calls per medium to stderr.  This is in addition to the existing
+  detailed logging to the .bzr.log trace file.  (Andrew Bennetts)
+
+Bug Fixes
+*********
+
+* Avoid random failures arising from misinterpreted ``errno`` values
+  in ``_readdir_pyx.read_dir``.
+  (Martin Pool, #279381)
+
+* Branching from a shared repository on a smart server into a new
+  repository now preserves the repository format.
+  (Andrew Bennetts, #269214)
+
+* ``bzr log`` now accepts a ``--change`` option.
+  (Vincent Ladeuil, #248427)
+
+* ``bzr missing`` now accepts an ``--include-merges`` option.
+  (Vincent Ladeuil, #233817)
+
+* Don't try to filter (internally) '.bzr' from the files to be deleted if
+  it's not there.
+  (Vincent Ladeuil, #272648)
+
+* Fix '_in_buffer' AttributeError when using the -Dhpss debug flag.
+  (Andrew Bennetts)
+
+* Fix TooManyConcurrentRequests errors caused by a connection failure
+  when doing ``bzr pull`` or ``bzr merge`` from a ``bzr+ssh`` URL.
+  (Andrew Bennetts, #246233)
+
+* Fixed ``bzr st -r branch:PATH_TO_BRANCH`` where the other branch
+  is in a different repository than the current one.
+  (Lukáš Lalinský, #144421)
+
+* Make the first line of the manpage preamble a comment again.
+  (David Futcher, #242106)
+
+* Remove use of optional parameter in GSSAPI FTP support, since
+  it breaks newer versions of Python-Kerberos. (Jelmer Vernooij)
+
+* The autopacking logic will now always create a single new pack from
+  all of the content which it deems is worth moving. This avoids the
+  'repack a single pack' bug and should result in better packing
+  overall.  (John Arbash Meinel, #242510, #172644)
+
+* Trivial documentation fix.
+  (John Arbash Meinel, #270471)
+
+* ``bzr switch`` and ``bzr bind`` will now update the branch nickname if
+  it was previously set. All checkouts will now refer to the bound branch
+  for a nickname if one was not explicitly set.
+  (Marius Kruger, #230903)
+
+Documentation
+*************
+
+* Explain revision/range identifiers. (Daniel Clemente)
+
+API Changes
+***********
+
+* ``CommitBuilder.record_entry_contents`` returns one more element in
+  its result tuple - an optional file system hash for the hash cache
+  to use. (Robert Collins)
+
+* ``dirstate.DirState.update_entry`` will now only calculate the sha1
+  of a file if it is likely to be needed in determining the output
+  of iter_changes. (Robert Collins)
+
+* The PackRepository, RepositoryPackCollection, NewPack classes have a
+  slightly changed interface to support different index types; as a
+  result other users of these classes need to supply the index types
+  they want. (Robert Collins)
+
+Testing
+*******
+
+* ``bzrlib.tests.repository_implementations`` has been renamed to
+  ``bzrlib.tests.per_repository`` so that we have a common structure
+  (and it is shorter). (John Arbash Meinel, #239343)
+
+* ``LocalTransport.abspath()`` now returns a drive letter if the
+  transport has one, fixing numerous tests on Windows.
+  (Mark Hammond)
+
+* PreviewTree is now tested via intertree_implementations.
+  (Aaron Bentley)
+
+* The full test suite is passing again on OSX.
+  (Guillermo Gonzalez, Vincent Ladeuil)
+
+* The full test suite passes when run with ``-Eallow_debug``.
+  (Andrew Bennetts)
+
+Internals
+*********
+
+* A new hook, ``Branch.open``, has been added, which is called when
+  branch objects are opened. (Robert Collins)
+
+* ``bzrlib.osutils._walkdirs_utf8`` has been refactored into common
+  tree walking, and modular directory listing code to aid future
+  performance optimisations and refactoring. (Robert Collins)
+
+* ``bzrlib.trace.debug_memory`` can be used to get a quick memory dump
+  in the middle of processing. It only reports memory if
+  ``/proc/PID/status`` is available. (John Arbash Meinel)
+
+* New method ``RevisionSpec.as_tree`` for representing the revision
+  specifier as a revision tree object. (Lukáš Lalinský)
+
+* New race-free method on MutableTree ``get_file_with_stat`` for use
+  when generating stat cache results. (Robert Collins)
+
+* New win32utils.get_local_appdata_location() provides access to a local
+  directory for storing data.  (Mark Hammond)
+
+* To be compatible with python-2.6 a few new rules should be
+  observed. 'message' attribute can't be used anymore in exception
+  classes, 'sha' and 'md5' modules have been deprecated (use
+  osutils.[md5|sha]), object__init__ and object.__new__ don't accept
+  parameters anymore.
+  (Vincent Ladeuil)
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-1.9.txt'
--- doc/en/release-notes/bzr-1.9.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-1.9.txt	2010-11-12 22:36:19 +0000
@@ -0,0 +1,154 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 1.9
+#######
+
+:Released: 2008-11-07
+
+This release of Bazaar adds a new repository format, ``1.9``, with smaller
+and more efficient index files.  This format can be specified when
+creating a new repository, or used to losslessly upgrade an existing
+repository.  bzr 1.9 also speeds most operations over the smart server
+protocol, makes annotate faster, and uses less memory when making
+checkouts or pulling large amounts of data.
+
+Bug Fixes
+*********
+
+* Fix "invalid property value 'branch-nick' for None" regression with
+  branches bound to svn branches.  (Martin Pool, #293440)
+
+* Fix SSL/https on Python2.6.  (Vincent Ladeuil, #293054)
+
+* ``SFTPTransport.readv()`` had a bug when requests were out-of-order.
+  This only triggers some-of-the-time on Knit format repositories.
+  (John Arbash Meinel, #293746)
+
+
+bzr 1.9rc1
+##########
+
+:Released: 2008-10-31
+
+New Features
+************
+
+* New Branch hook ``transform_fallback_location`` allows a function to
+  be called when looking up the stacked source. (Michael Hudson)
+
+* New repository formats ``1.9`` and ``1.9-rich-root``. These have all
+  the functionality of ``1.6``, but use the new btree indexes.
+  These indexes are both smaller and faster for access to historical
+  information.  (John Arbash Meinel)
+
+Improvements
+************
+
+* ``BTreeIndex`` code now is able to prefetch extra pages to help tune
+  the tradeoff between bandwidth and latency. Should be tuned
+  appropriately to not impact commands which need minimal information,
+  but provide a significant boost to ones that need more context. Only
+  has a direct impact on the ``--development2`` format which uses
+  btree's for the indexes. (John Arbash Meinel)
+
+* ``bzr dump-btree`` is a hidden command introduced to allow dumping
+  the contents of a compressed btree file.  (John Arbash Meinel)
+
+* ``bzr pack`` now tells the index builders to optimize for size. For
+  btree index repositories, this can save 25% of the index size
+  (mostly in the text indexes). (John Arbash Meinel)
+
+* ``bzr push`` to an existing branch or repository on a smart server
+  is faster, due to Bazaar making more use of the ``get_parent_map``
+  RPC when querying the remote branch's revision graph.
+  (Andrew Bennetts)
+
+* default username for bzr+ssh:// and sftp:// can be configured in
+  authentication.conf. (Aaron Bentley)
+
+* launchpad-login now provides a default username for bzr+ssh and SFTP
+  URLs, allowing username-free URLs to work for everyone. (Aaron Bentley)
+
+* ``lp:`` lookups no longer include usernames, making them shareable and
+  shorter. (Aaron Bentley)
+
+* New ``PackRepository.autopack`` smart server RPC, which does
+  autopacking entirely on the server.  This is much faster than
+  autopacking via plain file methods, which downloads a large amount
+  of pack data and then re-uploads the same pack data into a single
+  file.  This fixes a major (although infrequent) cause of lengthy
+  delays when using a smart server.  For example, pushing the 10th
+  revision to a repository with 9 packs now takes 44 RPCs rather than
+  179, and much less bandwidth too.  This requires Bazaar 1.9 on both
+  the client and the server, otherwise the client will fallback to the
+  slower method.  (Andrew Bennetts)
+
+Bug Fixes
+*********
+
+* A failure to load a plugin due to an IncompatibleAPI exception is
+  now correctly reported. (Robert Collins, #279451)
+
+* API versioning support now has a multiple-version checking api
+  ``require_any_api``. (Robert Collins, #279447)
+
+* ``bzr branch --stacked`` from a smart server to a standalone branch
+  works again.  This fixes a regression in 1.7 and 1.8.
+  (Andrew Bennetts, #270397)
+
+* ``bzr co`` uses less memory. It used to unpack the entire WT into
+  memory before writing it to disk. This was a little bit faster, but
+  consumed lots of memory. (John Arbash Meinel, #269456)
+
+* ``bzr missing --quiet`` no longer prints messages about whether
+  there are missing revisions.  The exit code indicates whether there
+  were or not.  (Martin Pool, #284748)
+
+* Fixes to the ``annotate`` code. The fast-path which re-used the
+  stored deltas was accidentally disabled all the time, instead of
+  only when a branch was stacked. Second, the code would accidentally
+  re-use a delta even if it wasn't against the left-parent, this
+  could only happen if ``bzr reconcile`` decided that the parent
+  ordering was incorrect in the file graph.  (John Arbash Meinel)
+
+* "Permission denied" errors that occur when pushing a new branch to a
+  smart server no longer cause tracebacks.  (Andrew Bennetts, #278673)
+
+* Some compatibility fixes for building the extensions with MSVC and
+  for python2.4. (John Arbash Meinel, #277484)
+
+* The index logic is now able to reload the list of pack files if and
+  index ends up disappearing. We still don't reload if the pack data
+  itself goes missing after checking the index. This bug appears as a
+  transient failure (file not found) when another process is writing
+  to the repository.  (John Arbash Meinel, #153786)
+
+* ``bzr switch`` and ``bzr bind`` will now update the branch nickname if
+  it was previously set. All checkouts will now refer to the bound branch
+  for a nickname if one was not explicitly set.
+  (Marius Kruger, #230903)
+
+Documentation
+*************
+
+* Improved hook documentation. (Michael Ernst)
+
+API Changes
+***********
+
+* commands.plugins_cmds is now a CommandRegistry, not a dict.
+
+Internals
+*********
+
+* New AuthenticationConfig.set_credentials method allows easy programmatic
+  configuration of authetication credentials.
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-2.0.txt'
--- doc/en/release-notes/bzr-2.0.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-2.0.txt	2011-08-20 09:28:27 +0000
@@ -0,0 +1,655 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.0.7
+#########
+
+:2.0.7: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+* Launchpad has announced that the ``edge.launchpad.net`` instance is
+  deprecated and may be shut down in the future
+  <http://blog.launchpad.net/general/edge-is-deprecated>.  Bazaar has therefore
+  been updated in this release to talk to the main (``launchpad.net``) servers,
+  rather than the ``edge`` ones. (Vincent Ladeuil, #583667)
+
+New Features
+************
+
+Bug Fixes
+*********
+
+* Avoid spurious ValueErrors when autopacking a subset of the repository,
+  and seeing a revision without its related inventory
+  (John Arbash Meinel, #437003)
+
+* Avoid UnicodeDecodeError in ``bzr add`` with multiple files under a non-ascii
+  path on windows from symlink support addition. (Martin [gz], #686611)
+
+* Fix activity reporting for pycurl when using https with some
+  implementations of curl. (Vincent Ladeuil, #614713)
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+
+bzr 2.0.6
+#########
+
+:2.0.6: 2010-09-17
+
+The sixth release in our 2.0 series addresses several user-inconvenience
+bugs.  None are critical, but upgrading is recommended for all users on
+earlier 2.0 releases.
+
+Bug Fixes
+*********
+
+* Additional merges after an unrelated branch has been merged with its
+  history no longer crash when deleted files are involved.
+  (Vincent Ladeuil, John Arbash Meinel, #375898)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.  
+  (Martin Pool, #192859)
+
+* ``bzr commit SYMLINK`` now works, rather than trying to commit the
+  target of the symlink.
+  (Martin Pool, John Arbash Meinel, #128562)
+
+* ``bzr revert`` now only takes write lock on working tree, instead of on 
+  both working tree and branch.
+  (Danny van Heumen, #498409)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* Fix ``AttributeError on parent.children`` when adding a file under a 
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Selftest with versions of subunit that support ``stopTestRun`` will no longer
+  error. This error was caused by 2.0 not being updated when upstream
+  python merged the end of run patch, which chose ``stopTestRun`` rather than
+  ``done``. (Robert Collins, #571437)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+
+Testing
+*******
+
+* ``build_tree_contents`` can create symlinks.
+  (Martin Pool, John Arbash Meinel)
+
+
+bzr 2.0.5
+#########
+
+:2.0.5: 2010-03-23
+
+This fifth release in our 2.0 series addresses several user-inconvenience
+bugs.  None are critical, but upgrading is recommended for all users on
+earlier 2.0 releases.
+
+Bug Fixes
+*********
+
+* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
+  (Martin Pool, #331095)
+
+* Concurrent autopacking is more resilient to already-renamed pack files.
+  If we find that a file we are about to obsolete is already obsoleted, we
+  do not try to rename it, and we leave the file in ``obsolete_packs``.
+  The code is also fault tolerant if a file goes missing, assuming that
+  another process already removed the file.
+  (John Arbash Meinel, Gareth White, #507557)
+
+* Cope with the lockdir ``held/info`` file being empty, which seems to
+  happen fairly often if the process is suddenly interrupted while taking
+  a lock.
+  (Martin Pool, #185103)
+
+* Give the warning about potentially slow cross-format fetches much
+  earlier on in the fetch operation.  Don't show this message during
+  upgrades, and show the correct format indication for remote
+  repositories.
+  (Martin Pool, #456077, #515356, #513157)
+
+* Handle renames correctly when there are files or directories that 
+  differ only in case.  (Chris Jones, Martin Pool, #368931)
+
+* If ``bzr push --create-prefix`` triggers an unexpected ``NoSuchFile``
+  error, report that error rather than failing with an unhelpful
+  ``UnboundLocalError``.
+  (Andrew Bennetts, #423563)
+
+* Running ``bzr`` command without any arguments now shows bzr
+  version number along with rest of the help text.
+  (Parth Malwankar, #369501)
+
+* Use osutils.O_NOINHERIT for some files on win32 to avoid PermissionDenied
+  errors.
+  (Inada Naoki, #524560)
+
+Documentation
+*************
+
+* Added ``location-alias`` help topic.
+  (Andrew Bennetts, #337834)
+
+* Fixed CHM generation by moving the NEWS section template into
+  a separate file. (Ian Clatworthy, #524184)
+
+
+bzr 2.0.4
+#########
+
+:Codename: smooth sailing
+:2.0.4: 2010-01-21
+
+The fourth bugfix-only release in the 2.0 series contains more than a
+dozen bugfixes relative to 2.0.3. The primary focus is on handling
+interruptions and concurrent operations more cleanly, there is also a fair
+improvement to ``bzr export`` when exporting a remote branch.
+
+
+Bug Fixes
+*********
+
+* ``bzr annotate`` on another branch with ``-r branch:...`` no longer
+  fails with an ``ObjectNotLocked`` error.  (Andrew Bennetts, #496590)
+
+* ``bzr export dir`` now requests all file content as a record stream,
+  rather than requsting the file content one file-at-a-time. This can make
+  exporting over the network significantly faster (54min => 9min in one
+  case). (John Arbash Meinel, #343218)
+
+* ``bzr serve`` no longer slowly leaks memory. The compiled
+  ``bzrlib.bencode.Encoder()`` class was using ``__del__`` to cleanup and
+  free resources, and it should have been using ``__dealloc__``.
+  This will likely have an impact on any other process that is serving for
+  an extended period of time.  (John Arbash Meinel, #494406)
+
+* Check for SIGINT (Ctrl-C) and other signals immediately if ``readdir``
+  returns ``EINTR`` by calling ``PyErr_CheckSignals``.  This affected the
+  optional ``_readdir_pyx`` extension.  (Andrew Bennetts, #495023)
+
+* Concurrent autopacks will no longer lose a newly created pack file.
+  There was a race condition, where if the reload happened at the right
+  time, the second packer would forget the name of the newly added pack
+  file. (John Arbash Meinel, Gareth White, #507566)
+
+* Give a clearer message if the lockdir disappears after being apparently
+  successfully taken.  (Martin Pool, #498378)
+
+* Give a warning when fetching between repositories (local or remote) with
+  sufficiently different formats that the content will need to be
+  serialized (ie ``InterDifferingSerializer`` or ``inventory-deltas``), so
+  the user has a clue that upgrading could make it faster.
+  (Martin Pool, #456077)
+
+* If we fail to open ``~/.bzr.log`` write a clear message to stderr rather
+  than using ``warning()``. The log file is opened before logging is set
+  up, and it leads to very confusing: 'no handlers for "bzr"' messages for
+  users, rather than something nicer.
+  (John Arbash Meinel, Barry Warsaw, #503886)
+
+* Refuse to build with any Pyrex 0.9.4 release, as they have known bugs.
+  (Martin Pool, John Arbash Meinel, #449372)
+
+* ``setup.py bdist_rpm`` now properly finds extra files needed for the
+  build. (there is still the distutils bug
+  http://bugs.python.org/issue644744) (Joe Julian, #175839)
+
+* The 2a format wasn't properly restarting autopacks when something
+  changed underneath it (like another autopack). Now concurrent
+  autopackers will properly succeed. (John Arbash Meinel, #495000)
+
+* ``TreeTransform`` can now handle when a delta says that the file id for
+  the tree root changes. Rather than trying to rename your working
+  directory, or failing early saying that you can't have multiple
+  tree roots. This also fixes revert, update, and pull when the root id
+  changes.  (John Arbash Meinel, #494269, #504390)
+
+* ``_update_current_block`` no longer suppresses exceptions, so ^C at just
+  the right time will get propagated, rather than silently failing to move
+  the block pointer. (John Arbash Meinel, Gareth White, #495023)
+
+Testing
+*******
+
+* We have a new ``test_source`` that ensures all pyrex ``cdef`` functions
+  handle exceptions somehow. (Possibly by setting ``# cannot_raise``
+  rather than an ``except ?:`` clause.) This should help prevent bugs like
+  bug #495023. (John Arbash Meinel)
+
+
+bzr 2.0.3
+#########
+
+:Codename: little italy
+:2.0.3: 2009-12-14
+
+
+The third stable release of Bazaar has a small handful of bugfixes. As
+expected, this has no internal or external compatibility changes versus
+2.0.2 (or 2.0.0).
+
+Bug Fixes
+*********
+
+* ``bzr push --use-existing-dir`` no longer crashes if the directory
+  exists but contains an invalid ``.bzr`` directory.
+  (Andrew Bennetts, #423563)
+
+* Content filters are now applied correctly after pull, merge and switch.
+  (Ian Clatworthy, #385879)
+
+* Fix a potential segfault in the groupcompress hash map handling code.
+  When inserting new entries, if the final hash bucket was empty, we could
+  end up trying to access if ``(last_entry+1)->ptr == NULL``.
+  (John Arbash Meinel, #490228)
+
+* Improve "Binary files differ" hunk handling.  (Aaron Bentley, #436325)
+
+
+bzr 2.0.2
+#########
+
+:Codename: after the scare
+:2.0.2: 2009-11-02
+
+The second in our "let's keep the stable bugfixes flowing" series. As
+expected this has a few (~9) bugfixes relative to 2.0.1, and no major api
+changes or features.
+
+Bug Fixes
+*********
+
+* Avoid "NoneType has no attribute st_mode" error when files disappear
+  from a directory while it's being read.  (Martin Pool, #446033)
+
+* Content filters are now applied correctly after revert.
+  (Ian Clatworthy)
+
+* Diff parsing handles "Binary files differ" hunks.  (Aaron Bentley, #436325)
+
+* Fetching from stacked pre-2a repository via a smart server no longer
+  fails intermittently with "second push failed to complete".
+  (Andrew Bennetts, #437626)
+
+* Fix typos left after test_selftest refactoring.
+  (Vincent Ladeuil, Matt Nordhoff, #461149)
+
+* Fixed ``ObjectNotLocked`` errors during ``bzr log -r NNN somefile``.
+  (Andrew Bennetts, #445171)
+  
+* PreviewTree file names are not limited by the encoding of the temp
+  directory's filesystem. (Aaron Bentley, #436794)
+
+Improvements
+************
+
+* ``bzr log`` now read-locks branches exactly once, so makes better use of
+  data caches.  (Andrew Bennetts)
+
+Documentation
+*************
+
+* Filtered views user documentation upgraded to refer to format 2a
+  instead of pre-2.0 formats. (Ian Clatworthy)
+
+
+bzr 2.0.1
+#########
+
+:Codename: Stability First
+:2.0.1: 2009-10-14
+
+The first of our new ongoing bugfix-only stable releases has arrived. It
+includes a collection of 12 bugfixes applied to bzr 2.0.0, but does not
+include any of the feature development in the 2.1.0 series.
+
+
+Bug Fixes
+*********
+
+* ``bzr add`` in a tree that has files with ``\r`` or ``\n`` in the
+  filename will issue a warning and skip over those files.
+  (Robert Collins, #3918)
+
+* bzr will attempt to authenticate with SSH servers that support
+  ``keyboard-interactive`` auth but not ``password`` auth when using
+  Paramiko.   (Andrew Bennetts, #433846)
+
+* Fixed fetches from a stacked branch on a smart server that were failing
+  with some combinations of remote and local formats.  This was causing
+  "unknown object type identifier 60" errors.  (Andrew Bennetts, #427736)
+
+* Fixed ``ObjectNotLocked`` errors when doing some log and diff operations
+  on branches via a smart server.  (Andrew Bennetts, #389413)
+
+* Handle things like ``bzr add foo`` and ``bzr rm foo`` when the tree is
+  at the root of a drive. ``osutils._cicp_canonical_relpath`` always
+  assumed that ``abspath()`` returned a path that did not have a trailing
+  ``/``, but that is not true when working at the root of the filesystem.
+  (John Arbash Meinel, Jason Spashett, #322807)
+
+* Hide deprecation warnings for 'final' releases for python2.6.
+  (John Arbash Meinel, #440062)
+
+* Improve the time for ``bzr log DIR`` for 2a format repositories.
+  We had been using the same code path as for <2a formats, which required
+  iterating over all objects in all revisions.
+  (John Arbash Meinel, #374730)
+
+* Make sure that we unlock the tree if we fail to create a TreeTransform
+  object when doing a merge, and there is limbo, or pending-deletions
+  directory.  (Gary van der Merwe, #427773)
+
+* Occasional IndexError on renamed files have been fixed. Operations that
+  set a full inventory in the working tree will now go via the
+  apply_inventory_delta code path which is simpler and easier to
+  understand than dirstates set_state_from_inventory method. This may
+  have a small performance impact on operations built on _write_inventory,
+  but such operations are already doing full tree scans, so no radical
+  performance change should be observed. (Robert Collins, #403322)
+
+* Retrieving file text or mtime from a _PreviewTree has good performance when
+  there are many changes.  (Aaron Bentley)
+
+* The CHK index pages now use an unlimited cache size. With a limited
+  cache and a large project, the random access of chk pages could cause us
+  to download the entire cix file many times.
+  (John Arbash Meinel, #402623)
+
+* When a file kind becomes unversionable after being added, a sensible
+  error will be shown instead of a traceback. (Robert Collins, #438569)
+
+Documentation
+*************
+
+* Improved README. (Ian Clatworthy)
+
+* Improved upgrade documentation for Launchpad branches.
+  (Barry Warsaw)
+
+
+bzr 2.0.0
+#########
+
+:2.0.0: 2009-09-22
+:Codename: Instant Karma
+
+This release of Bazaar makes the 2a (previously 'brisbane-core') format
+the default when new branches or repositories are created.  This format is
+substantially smaller and faster for many operations.  Most of the work in
+this release focuses on bug fixes and stabilization, covering both 2a and
+previous formats.  (See the Upgrade Guide for information on migrating
+existing projects.)
+
+This release also improves the documentation content and presentation,
+including adding Windows HtmlHelp manuals.
+
+The Bazaar team decided that 2.0 will be a long-term supported release,
+with bugfix-only 2.0.x releases based on it, continuing for at least six
+months or until the following stable release.
+
+Changes from 2.0.0rc2 to final
+******************************
+
+* Officially branded as 2.0.0 rather than 2.0 to clarify between things
+  that "want to happen on the 2.0.x stable series" versus things that want
+  to "land in 2.0.0". (Changes how bzrlib._format_version_tuple() handles
+  micro = 0.) (John Arbash Meinel)
+
+
+bzr 2.0.0rc2
+############
+
+:2.0.0rc2: 2009-09-10
+
+New Features
+************
+
+* Added post_commit hook for mutable trees. This allows the keywords
+  plugin to expand keywords on files changed by the commit.
+  (Ian Clatworthy, #408841)
+
+Bug Fixes
+*********
+
+* Bazaar's native protocol code now correctly handles EINTR, which most
+  noticeably occurs if you break in to the debugger while connected to a
+  bzr+ssh server.  You can now can continue from the debugger (by typing
+  'c') and the process continues.  However, note that pressing C-\ in the
+  shell may still kill the SSH process, which is bug 162509, so you must
+  sent a signal to the bzr process specifically, for example by typing
+  ``kill -QUIT PID`` in another shell.  (Martin Pool, #341535)
+
+* ``bzr check`` in pack-0.92, 1.6 and 1.9 format repositories will no
+  longer report incorrect errors about ``Missing inventory ('TREE_ROOT', ...)``
+  (Robert Collins, #416732)
+
+* ``bzr info -v`` on a 2a format still claimed that it was a "Development
+  format" (John Arbash Meinel, #424392)
+
+* ``bzr log stacked-branch`` shows the full log including
+  revisions that are in the fallback repository. (Regressed in 2.0rc1).
+  (John Arbash Meinel, #419241)
+
+* Clearer message when Bazaar runs out of memory, instead of a ``MemoryError``
+  traceback.  (Martin Pool, #109115)
+
+* Conversion to 2a will create a single pack for all the new revisions (as
+  long as it ran without interruption). This improves both ``bzr upgrade``
+  and ``bzr pull`` or ``bzr merge`` from local branches in older formats.
+  The autopack logic that occurs every 100 revisions during local
+  conversions was not returning that pack's identifier, which resulted in
+  the partial packs created during the conversion not being consolidated
+  at the end of the conversion process. (Robert Collins, #423818)
+
+* Fetches from 2a to 2a are now again requested in 'groupcompress' order.
+  Groups that are seen as 'underutilized' will be repacked on-the-fly.
+  This means that when the source is fully packed, there is minimal
+  overhead during the fetch, but if the source is poorly packed the result
+  is a fairly well packed repository (not as good as 'bzr pack' but
+  good-enough.) (Robert Collins, John Arbash Meinel, #402652)
+
+* Fix a potential segmentation fault when doing 'log' of a branch that had
+  ghosts in its mainline.  (Evaluating None as a tuple is bad.)
+  (John Arbash Meinel, #419241)
+
+* ``groupcompress`` sort order is now more stable, rather than relying on
+  ``topo_sort`` ordering. The implementation is now
+  ``KnownGraph.gc_sort``. (John Arbash Meinel)
+
+* Local data conversion will generate correct deltas. This is a critical
+  bugfix vs 2.0rc1, and all 2.0rc1 users should upgrade to 2.0rc2 before
+  converting repositories. (Robert Collins, #422849)
+
+* Network streams now decode adjacent records of the same type into a
+  single stream, reducing layering churn. (Robert Collins)
+
+* Prevent some kinds of incomplete data from being committed to a 2a
+  repository, such as revisions without inventories, a missing chk_bytes
+  record for an inventory, or a missing text referenced by an inventory.
+  (Andrew Bennetts, #423506, #406687)
+  
+Documentation
+*************
+
+* Fix assertion error about "_remember_remote_is_before" when pushing to
+  older smart servers.
+  (Andrew Bennetts, #418931)
+
+* Help on hooks no longer says 'Not deprecated' for hooks that are
+  currently supported. (Ian Clatworthy, #422415)
+
+* PDF and CHM (Windows HtmlHelp) formats are now supported for the
+  user documentation. The HTML documentation is better broken up into
+  topics. (Ian Clatworthy)
+
+* The developer and foreign language documents are now separated
+  out so that searching in the HTML and CHM files produces more
+  useful results. (Ian Clatworthy)
+
+* The main table of contents now provides links to the new Migration Docs
+  and Plugins Guide. (Ian Clatworthy)
+
+
+bzr 2.0.0rc1
+############
+
+:Codename: no worries
+:2.0.0rc1: 2009-08-26
+
+Compatibility Breaks
+********************
+
+* The default format for bzr is now ``2a``. This format brings many
+  significant performance and size improvements. bzr can pull from
+  any existing repository into a ``2a`` one, but can only transfer
+  from ``2a`` into ``rich-root`` repositories. The Upgrade guide
+  has more information about this change. (Robert Collins)
+
+* On Windows auto-detection of Putty's plink.exe is disabled.
+  Default SSH client for Windows is paramiko. User still can force
+  usage of plink if explicitly set environment variable BZR_SSH=plink.
+  (#414743, Alexander Belchenko)
+
+New Features
+************
+
+* ``bzr branch --switch`` can now switch the checkout in the current directory
+  to the newly created branch. (Lukáš Lalinský)
+
+Bug Fixes
+*********
+
+* Further tweaks to handling of ``bzr add`` messages about ignored files.
+  (Jason Spashett, #76616)
+
+* Fetches were being requested in 'groupcompress' order, but weren't
+  recombining the groups. Thus they would 'fragment' to get the correct
+  order, but not 'recombine' to actually benefit from it. Until we get
+  recombining to work, switching to 'unordered' fetches avoids the
+  fragmentation. (John Arbash Meinel, #402645)
+
+* Fix a pycurl related test failure on karmic by recognizing an error
+  raised by newer versions of pycurl.
+  (Vincent Ladeuil, #306264)
+
+* Fix a test failure on karmic by making a locale test more robust.
+  (Vincent Ladeuil, #413514)
+
+* Fix IndexError printing CannotBindAddress errors.
+  (Martin Pool, #286871)
+
+* Fix "Revision ... not present" errors when upgrading stacked branches,
+  or when doing fetches from a stacked source to a stacked target.
+  (Andrew Bennetts, #399140)
+
+* ``bzr branch`` of 2a repositories over HTTP is much faster.  bzr now
+  batches together small fetches from 2a repositories, rather than
+  fetching only a few hundred bytes at a time.
+  (Andrew Bennetts, #402657)
+
+Improvements
+************
+
+* A better description of the platform is shown in crash tracebacks, ``bzr
+  --version`` and ``bzr selftest``.
+  (Martin Pool, #409137)
+
+* bzr can now (again) capture crash data through the apport library, 
+  so that a single human-readable file can be attached to bug reports.
+  This can be disabled by using ``-Dno_apport`` on the command line, or by
+  putting ``no_apport`` into the ``debug_flags`` section of
+  ``bazaar.conf``.
+  (Martin Pool, Robert Collins, #389328)
+
+* ``bzr push`` locally on windows will no longer give a locking error with
+  dirstate based formats. (Robert Collins)
+
+* ``bzr shelve`` and ``bzr unshelve`` now work on windows.
+  (Robert Collins, #305006)
+
+* Commit of specific files no longer prevents using the iter_changes
+  codepath. On 2a repositories, commit of specific files should now be as
+  fast, or slightly faster, than a full commit. (Robert Collins)
+
+* The internal core code that handles specific file operations like
+  ``bzr st FILENAME`` or ``bzr commit FILENAME`` has been changed to
+  include the parent directories if they have altered, and when a
+  directory stops being a directory its children are always included. This
+  fixes a number of causes for ``InconsistentDelta`` errors, and permits
+  faster commit of specific paths. (Robert Collins, #347649)
+
+Documentation
+*************
+
+* New developer documentation for content filtering.
+  (Martin Pool)
+
+API Changes
+***********
+
+* ``bzrlib.shelf_ui`` has had the ``from_args`` convenience methods of its
+  classes changed to manage lock lifetime of the trees they open in a way
+  consistent with reader-exclusive locks. (Robert Collins, #305006)
+
+Testing
+*******
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-2.1.txt'
--- doc/en/release-notes/bzr-2.1.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-2.1.txt	2011-08-20 09:28:27 +0000
@@ -0,0 +1,1305 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.1.5
+#########
+
+:2.1.5: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+New Features
+************
+
+Bug Fixes
+*********
+
+* Accessing a packaging branch on Launchpad (eg, ``lp:ubuntu/bzr``) now
+  checks to see if the most recent published source package version for
+  that project is present in the branch tags. This should help developers
+  trust whether the packaging branch is up-to-date and can be used for new
+  changes. The level of verbosity is controlled by the config item
+  ``launchpad.packaging_verbosity``. It can be set to one of
+
+  off
+    disable all checks
+
+
+  minimal
+    only display if the branch is out-of-date
+
+  short
+    also display single-line up-to-date and missing,
+
+
+  all
+    (default) display multi-line content for all states
+
+
+  (John Arbash Meinel, #609187, #812928)
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+
+bzr 2.1.4
+#########
+
+:2.1.4: 2011-05-16
+
+The fourth release in our 2.1 series addresses some user-inconvenience bugs.
+None are critical, but upgrading is recommended for all users on earlier 2.1
+releases.
+
+
+Compatibility Breaks
+********************
+
+* Launchpad has announced that the ``edge.launchpad.net`` instance is
+  deprecated and may be shut down in the future
+  <http://blog.launchpad.net/general/edge-is-deprecated>.  Bazaar has therefore
+  been updated in this release to talk to the main (``launchpad.net``) servers,
+  rather than the ``edge`` ones. (Vincent Ladeuil, #583667)
+
+New Features
+************
+
+None.
+
+Bug Fixes
+*********
+
+* Avoid UnicodeDecodeError in ``bzr add`` with multiple files under a non-ascii
+  path on windows from symlink support addition. (Martin [gz], #686611)
+
+* Skip tests that needs a bzr source tree when there isn't one. This is
+  needed to succesfully run the test suite for installed versions.
+  (Vincent Ladeuil, #644855).
+
+* Skip the tests that requires respecting the chmod bits when running as root.
+  (Vincent Ladeuil, #646133)
+
+* Using bzr with `lp:` URLs behind an HTTP proxy should work.
+  (Robert Collins, #558343)
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+
+bzr 2.1.3
+#########
+
+:Codename: Do run run
+:2.1.3: 2010-09-17
+
+The third release in our 2.1 series addresses several user-inconvenience bugs
+(and includes the fixes done in 2.0.6).  None are critical, but upgrading is
+recommended for all users on earlier 2.1 releases.
+
+Bug Fixes
+*********
+
+* Additional merges after an unrelated branch has been merged with its
+  history no longer crash when deleted files are involved.
+  (Vincent Ladeuil, John Arbash Meinel, #375898)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.  
+  (Martin Pool, #192859)
+
+* ``bzr commit SYMLINK`` now works, rather than trying to commit the
+  target of the symlink.
+  (Martin Pool, John Arbash Meinel, #128562)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* Fix ``AttributeError on parent.children`` when adding a file under a 
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Selftest with versions of subunit that support ``stopTestRun`` will no longer
+  error. This error was caused by 2.0 not being updated when upstream
+  python merged the end of run patch, which chose ``stopTestRun`` rather than
+  ``done``. (Robert Collins, #571437)
+
+* Stop ``AttributeError: 'module' object has no attribute 'ElementTree'``
+  being thrown from ``xml_serializer`` on certain cElementTree setups.
+  (Martin [gz], #254278)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+Testing
+*******
+
+* ``build_tree_contents`` can create symlinks.
+  (Martin Pool, John Arbash Meinel)
+
+
+bzr 2.1.2
+#########
+
+:2.1.2: 2010-05-28
+
+This release fixes two critical networking issues with older servers and
+with interrupted system call errors when pushing or pulling.  We recommend
+upgrading to anyone running a 2.1.x version of bzr.
+
+Bug Fixes
+*********
+
+* ``bzr clean-tree`` should not delete nested bzrdirs. Required for proper
+  support of bzr-externals and scmproj plugins.
+  (Alexander Belchenko, bug #572098)
+
+* ``bzr switch`` does not die if a ConfigurableFileMerger is used.
+  (Aaron Bentley, #559436)
+
+* Do not register a SIGWINCH signal handler, instead just poll for the
+  terminal width as needed.  This avoids the "Interrupted System Call"
+  problems that occur on POSIX with all currently released versions of
+  Python.
+  (Andrew Bennetts, #583941)
+
+* Fixed ``AssertionError`` when accessing smart servers running Bazaar
+  versions before 1.6.
+  (Andrew Bennetts, #528041)
+
+* Reset ``siginterrupt`` flag to False every time we handle a signal
+  installed with ``set_signal_handler(..., restart_syscall=True)`` (from
+  ``bzrlib.osutils``.  Reduces the likelihood of "Interrupted System Call"
+  errors compared to registering ``signal.signal`` directly.
+  (Andrew Bennetts)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Support Pyrex 0.9.9, required changing how we handle exceptions in Pyrex.
+  (John Arbash Meinel, #582656)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+Internals
+*********
+
+* ``_remember_remote_is_before`` no longer raises AssertionError when
+  suboptimal network behaviour is noticed; instead it just mutters to the
+  log file (and warns the user if they have set the ``hpss`` debug flag).
+  This was causing unnecessary aborts for performance bugs that are minor
+  at worst.
+  (Andrew Bennetts, #528041)
+
+
+bzr 2.1.1
+#########
+
+:2.1.1: 2010-03-24
+
+This is a small bugfix release.  Upgrading is recommended for anyone
+running 2.1.0 or earlier.
+
+Bug Fixes
+*********
+
+* Allow syscalls to automatically restart when ``TextUIFactory``'s
+  SIGWINCH handler is invoked, avoiding ``EINTR`` errors during blocking
+  IO, which are often poorly handled by Python's libraries and parts of
+  bzrlib.  (Andrew Bennetts, #496813)
+
+* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
+  (Martin Pool, #331095)
+
+* Fix plugin packaging on Windows. (Ian Clatworthy, #524162)
+
+* Fix stub SFTP test server to call os.getcwdu().
+  (Vincent Ladeuil, #526221, #526353)
+
+* Fixed CHM generation by moving the NEWS section template into
+  a separate file. (Ian Clatworthy, #524184)
+
+* Merge correctly when this_tree is not a WorkingTree.  (Aaron Bentley)
+
+* Register SIGWINCH handler only when creating a ``TextUIFactory``; avoids
+  problems importing bzrlib from a non-main thread.
+  (Elliot Murphy, #521989)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Standardize the error handling when creating a new ``StaticTuple``
+  (problems will raise TypeError). (Matt Nordhoff, #457979)
+
+* Warn if pyrex is too old to compile the new ``SimpleSet`` and
+  ``StaticTuple`` extensions, rather than having the build fail randomly.
+  (John Arbash Meinel, #449776)
+
+Documentation
+*************
+
+* Added a link to the Desktop Guide. (Ian Clatworthy)
+
+* Added What's New in Bazaar 2.1 document. (Ian Clatworthy)
+
+* Drop Google Analytics from the core docs as they caused problems
+  in the CHM files. (Ian Clatworthy, #502010)
+
+API Changes
+***********
+
+* Added ``bzrlib.osutils.set_signal_handler``, a convenience function that
+  can set a signal handler and call ``signal.siginterrupt(signum,
+  False)`` for it, if the platform and Python version supports it.
+  (Andrew Bennetts, #496813)
+
+
+bzr 2.1.0
+#########
+
+:Codename: Strasbourg
+:2.1.0: 2010-02-11
+
+This release marks our second long-term-stable series. The Bazaar team
+has decided that we will continue to make bugfix-only 2.0.x and 2.1.x
+releases, along with 2.2 development releases. 
+
+This is a fairly incremental update, focusing on polish and bugfixing.
+There are no changes for supported disk formats. Key updates include
+reduced memory consumption for many operations, a new per-file merge
+hook, ignore patterns can now include '!' to exclude files, globbing
+support for all commands on Windows, and support for addressing home
+directories via ``bzr+ssh://host/~/`` syntax.
+
+Users are encouraged to upgrade from the 2.0 stable series.
+
+Bug Fixes
+*********
+
+* Don't require testtools to use SFTP.
+  (Vincent Ladeuil, #516183)
+
+* Fix "AttributeError in Inter1and2Helper" during fetch.
+  (Martin Pool, #513432)
+
+* ``bzr update`` performs the two merges in a more logical order and will stop
+  when it encounters conflicts.  
+  (Gerard Krol, #113809)
+
+* Give a better error message when doing ``bzr bind`` in an already bound
+  branch.  (Neil Martinsen-Burrell, #513063)
+
+* Ignore ``KeyError`` from ``remove_index`` during ``_abort_write_group``
+  in a pack repository, which can happen harmlessly if the abort occurs during
+  finishing the write group.  Also use ``bzrlib.cleanup`` so that any
+  other errors that occur while aborting the individual packs won't be
+  hidden by secondary failures when removing the corresponding indices.
+  (Andrew Bennetts, #423015)
+
+* Set the mtime of files exported to a directory by ``bzr export`` all to
+  the same value to avoid confusing ``make`` and other date-based build
+  systems. (Robert Collins, #515631)
+
+Improvements
+************
+
+* Fetching into experimental formats will now print a warning. (Jelmer
+  Vernooij)
+
+API Changes
+***********
+
+* ``Repository.deserialise_inventory`` has been renamed to 
+  ``Repository._deserialise_inventory`` to indicate it is private.
+  (Jelmer Vernooij)
+
+* ``Repository.get_inventory_xml`` has been renamed to 
+  ``Repository._get_inventory_xml`` to indicate it is private. 
+  (Jelmer Vernooij)
+
+* ``Repository.serialise_inventory`` has been renamed to 
+  ``Repository._serialise_inventory`` to indicate it is private.
+
+* Using the ``bzrlib.chk_map`` module from within multiple threads at the
+  same time was broken due to race conditions with a module level page
+  cache. This shows up as a KeyError in the ``bzrlib.lru_cache`` code with
+  ``bzrlib.chk_map`` in the backtrace, and can be triggered without using
+  the same high level objects such as ``bzrlib.repository.Repository``
+  from different threads. chk_map now uses a thread local cache which may
+  increase memory pressure on processes using threads.
+  (Robert Collins, John Arbash Meinel, #514090)
+
+* The new ``merge_file_content`` should now be ok with tests to avoid
+  regressions.
+  (Vincent Ladeuil, #515597)
+
+Internals
+*********
+
+* Use ``bzrlib.cleanup`` rather than less robust ``try``/``finally``
+  blocks in several places in ``bzrlib.merge``.  This avoids masking prior
+  errors when errors like ``ImmortalPendingDeletion`` occur during cleanup
+  in ``do_merge``.
+  (Andrew Bennetts, #517275)
+
+API Changes
+***********
+
+* The ``remove_index`` method of
+  ``bzrlib.repofmt.pack_repo.AggregateIndex`` no longer takes a ``pack``
+  argument.  This argument was always ignored.
+  (Andrew Bennetts, #423015)
+
+bzr 2.1.0rc2
+############
+
+:Codename: after the bubbles
+:2.1.0rc2: 2010-01-29
+
+This is a quick-turn-around to update a small issue with our new per-file
+merge hook. We expect no major changes from this to the final 2.1.0.
+
+API Changes
+***********
+
+* The new ``merge_file_content`` hook point has been altered to provide a
+  better API where state for extensions can be stored rather than the
+  too-simple function based approach. This fixes a performance regression
+  where branch configuration would be parsed per-file during merge. As
+  part of this the included news_merger has been refactored into a base
+  helper class ``bzrlib.merge.ConfigurableFileMerger``.
+  (Robert Collins, John Arbash Meinel, #513822)
+
+
+bzr 2.1.0rc1
+############
+
+:Codename: the 'new' stable
+:2.1.0rc1: 2009-01-21
+
+This is the first stable release candidate for Bazaar's 2.1 series. From
+this point onwards, the 2.1 series will be considered stable (as the 2.0
+series) and only bugfixes are expected to be incorporated. The dozen or so
+bugfixes in the 2.0.4 release are also included in this release (along
+with more than 15 more bugfixes). Some of the interesting features are
+support for per-file merge hooks, ``bzr unshelve --preview``, support
+for using ! in ignore files to exclude files from being ignored, a small
+memory leak was squashed, and many ``ObjectNotLocked`` errors were fixed.
+This looks to be a very good start for a new stable series.
+
+
+New Features
+************
+
+* Add bug information to log output when available.
+  (Neil Martinsen-Burrell, Guillermo Gonzalez, #251729)
+
+* Added ``merge_file_content`` hook point to ``Merger``, allowing plugins
+  to register custom merge logic, e.g. to provide smarter merging for
+  particular files.
+
+* Bazaar now includes the ``news_merge`` plugin.  It is disabled by
+  default, to enable it add a ``news_merge_files`` option to your
+  configuration.  Consult ``bzr help news_merge`` for more information.
+  (Andrew Bennetts)
+  
+* ``bzr branch`` now takes a ``--bind`` option. This lets you
+  branch and bind all in one command. (Ian Clatworthy)
+
+* ``bzr switch`` now takes a ``--revision`` option, to allow switching to
+  a specific revision of a branch. (Daniel Watkins, #183559)
+
+* ``bzr unshelve --preview`` can now be used to show how a patch on the
+  shelf would be applied to the working tree.
+  (Guilherme Salgado, #308122)
+
+* ``bzr update`` now takes a ``--revision`` argument. This lets you
+  change the revision of the working tree to any revision in the
+  ancestry of the current or master branch. (Matthieu Moy, Mark Hammond,
+  Martin Pool, #45719)
+
+* ``-Dbytes`` can now be used to display the total number of bytes
+  transferred for the current command. This information is always logged
+  to ``.bzr.log`` for later inspection. (John Arbash Meinel)
+
+* New ignore patterns.  Patterns prefixed with '!' are exceptions to 
+  ignore patterns and take precedence over regular ignores.  Such 
+  exceptions are used to specify files that should be versioned which 
+  would otherwise be ignored.  Patterns prefixed with '!!' act as regular 
+  ignore patterns, but have highest precedence, even over the '!' 
+  exception patterns. (John Whitley, #428031)
+
+* The ``supress_warnings`` configuration option has been introduced to disable
+  various warnings (it currently only supports the ``format_deprecation``
+  warning). The new option can be set in any of the following locations:
+  ``bazaar.conf``, ``locations.conf`` and/or ``branch.conf``.
+  (Ted Gould, Matthew Fuller, Vincent Ladeuil)
+
+Bug Fixes
+*********
+
+* Always show a message if an OS error occurs while trying to run a
+  user-specified commit message editor.
+  (Martin Pool, #504842)
+
+* ``bzr diff`` will now use the epoch when it is unable to determine 
+  the timestamp of a file, if the revision it was introduced in is a
+  ghost. (Jelmer Vernooij, #295611)
+
+* ``bzr switch -b`` can now create branches that are located using directory
+  services such as ``lp:``, even when the branch name doesn't contain a
+  '/'.  (Neil Martinsen-Burrell, #495263)
+
+* ``bzr unshelve`` has improved messages about what it is doing.
+  (Neil Martinsen-Burrell, #496917)
+
+* Concurrent autopacking is more resilient to already-renamed pack files.
+  If we find that a file we are about to obsolete is already obsoleted, we
+  do not try to rename it, and we leave the file in ``obsolete_packs``.
+  The code is also fault tolerant if a file goes missing, assuming that
+  another process already removed the file.
+  (John Arbash Meinel, Gareth White, #507557)
+
+* Fix "Too many concurrent requests" in reconcile when network connection
+  fails.  (Andrew Bennetts, #503878)
+
+* Fixed a side effect mutation of ``RemoteBzrDirFormat._network_name``
+  that caused some tests to fail when run in a non-default order.
+  Probably no user impact.  (Martin Pool, #504102)
+
+* Fixed ``ObjectNotLocked`` error in ``bzr cat -rbranch:../foo FILE``.
+  (Andrew Bennetts, #506274)
+
+* FTP transports support Unicode paths by encoding/decoding them as utf8.
+  (Vincent Ladeuil, #472161)
+
+* Listen to the SIGWINCH signal to update the terminal width.
+  (Vincent Ladeuil, #316357)
+
+* Progress bars are now hidden when ``--quiet`` is given.
+  (Martin Pool, #320035)
+
+* ``SilentUIFactory`` now supports ``make_output_stream`` and discards
+  whatever is written to it.  This un-breaks some plugin tests that
+  depended on this behaviour.
+  (Martin Pool, #499757)
+
+* When operations update the working tree, all affected files should end
+  up with the same mtime. (eg. when versioning a generated file, if you
+  update the source and the generated file together, the generated file
+  should appear up-to-date.)
+  (John Arbash Meinel, Martin <gzlist>, #488724)
+
+Improvements
+************
+
+* Added ``add_cleanup`` and ``cleanup_now`` to ``bzrlib.command.Command``.
+  All the builtin commands now use ``add_cleanup`` rather than
+  ``try``/``finally`` blocks where applicable as it is simpler and more
+  robust.  (Andrew Bennetts)
+
+* All except a small number of storage formats are now hidden, making
+  the help for numerous commands far more digestible. (Ian Clatworthy)
+
+* Attempts to open a shared repository as a branch (e.g. ``bzr branch
+  path/to/repo``) will now include "location is a repository" as a hint in
+  the error message.  (Brian de Alwis, Andrew Bennetts, #440952)
+
+* Push will now inform the user when they are trying to push to a foreign 
+  VCS for which roundtripping is not supported, and will suggest them to 
+  use dpush. (Jelmer Vernooij)
+
+* The version of bzr being run is now written to the log file.
+  (__monty__, #257170)
+
+* Transport network activity indicator is shown more of the time when
+  Bazaar is doing network IO.
+  (Martin Pool)
+
+Documentation
+*************
+
+* Add documentation on creating merges with more than one parent.
+  (Neil Martinsen-Burrell, #481526)
+
+* Better explain the --uncommitted option of merge.
+  (Neil Martinsen-Burrell, #505088)
+
+* Improve discussion of pending merges in the documentation for
+  ``revert``.  (Neil Martinsen-Burrell, #505093)
+
+* Improved help for ``bzr send``. 
+  (Martin Pool, Bojan Nikolic)
+
+* There is a System Administrator's Guide in ``doc/en/admin-guide``,
+  including discussions of installation, relevant plugins, security and 
+  backup. (Neil Martinsen-Burrell)
+
+* The ``conflicts`` help topic has been renamed to ``conflict-types``.
+  (Ian Clatworthy)
+
+* The User Reference is now presented as a series of topics.
+  Many of the included topics have link and format tweaks applied.
+  (Ian Clatworthy)
+
+API Changes
+***********
+
+* Added ``cachedproperty`` decorator to ``bzrlib.decorators``.
+  (Andrew Bennetts)
+
+* Many test features were renamed from ``FooFeature`` to ``foo_feature``
+  to be consistent with instances being lower case and classes being
+  CamelCase. For the features that were more likely to be used, we added a
+  deprecation thunk, but not all. (John Arbash Meinel)
+
+* Merger classes (such as ``Merge3Merger``) now expect a ``this_branch``
+  parameter in their constructors, and provide ``this_branch`` as an
+  attribute. (Andrew Bennetts)
+  
+* The Branch hooks pre_change_branch_tip no longer masks exceptions raised
+  by plugins - the original exceptions are now preserved. (Robert Collins)
+
+* The Transport ``Server.tearDown`` method is now renamed to
+  ``stop_server`` and ``setUp`` to ``start_server`` for consistency with
+  our normal naming pattern, and to avoid confusion with Python's
+  ``TestCase.tearDown``.  (Martin Pool)
+
+* ``WorkingTree.update`` implementations must now accept a ``revision``
+  parameter.
+
+Internals
+*********
+
+* Added ``BzrDir.open_branchV3`` smart server request, which can receive
+  a string of details (such as "location is a repository") as part of a
+  ``nobranch`` response.  (Andrew Bennetts, #440952)
+  
+* New helper osutils.UnicodeOrBytesToBytesWriter which encodes unicode
+  objects but passes str objects straight through. This is used for
+  selftest but may be useful for diff and other operations that generate
+  mixed output. (Robert Collins)
+
+* New exception ``NoRoundtrippingSupport``, for use by foreign branch 
+  plugins. (Jelmer Vernooij)
+
+Testing
+*******
+
+* ``bzrlib.tests.permute_for_extension`` is a helper that simplifies
+  running all tests in the current module, once against a pure python
+  implementation, and once against an extension (pyrex/C) implementation.
+  It can be used to dramatically simplify the implementation of
+  ``load_tests``.  (John Arbash Meinel)
+
+* ``bzrlib.tests.TestCase`` now subclasses ``testtools.testcase.TestCase``.
+  This permits features in testtools such as getUniqueInteger and
+  getUniqueString to be used. Because of this, testtools version 0.9.2 or
+  newer is now a dependency to run bzr selftest. Running with versions of
+  testtools less than 0.9.2 will cause bzr to error while loading the test
+  suite. (Robert Collins)
+
+* Shell-like tests now support the command "mv" for moving files.  The
+  syntax for ``mv file1 file2``, ``mv dir1 dir2`` and ``mv file dir`` is
+  supported.  (Neil Martinsen-Burrell)
+
+* The test progress bar no longer distinguishes tests that 'errored' from
+  tests that 'failed' - they're all just failures.
+  (Martin Pool)
+
+
+bzr 2.1.0b4
+###########
+
+:Codename: san francisco airport
+:2.1.0b4: 2009-12-14
+
+The fourth beta release in the 2.1 series brings with it a significant
+number of bugfixes (~20). The test suite is once again (finally) "green"
+on Windows, and should remain that way for future releases. There are a
+few performance related updates (faster upgrade and log), and several UI
+tweaks. There has also been a significant number of tweaks to the runtime
+documentation. 2.1.0b4 include everything from the 2.0.3 release.
+
+
+Compatibility Breaks
+********************
+
+* The BZR_SSH environmental variable may now be set to the path of a secure
+  shell client. If currently set to the value ``ssh`` it will now guess the
+  vendor of the program with that name, to restore the old behaviour that
+  indicated the SSH Corporation client use ``sshcorp`` instead as the magic
+  string. (Martin <gzlist@googlemail.com>, #176292)
+
+New Features
+************
+
+* ``bzr commit`` now has a ``--commit-time`` option.
+  (Alexander Sack, #459276)
+
+* ``-Dhpss`` now increases logging done when run on the bzr server,
+  similarly to how it works on the client. (John Arbash Meinel)
+
+* New option ``bzr unshelve --keep`` applies the changes and leaves them
+  on the shelf.  (Martin Pool, Oscar Fuentes, #492091)
+
+* The ``BZR_COLUMNS`` envrionment variable can be set to force bzr to
+  respect a given terminal width. This can be useful when output is
+  redirected or in obscure cases where the default value is not
+  appropriate. Pagers can use it to get a better control of the line
+  lengths. 
+  (Vincent Ladeuil)
+
+* The new command ``bzr lp-mirror`` will request that Launchpad update its
+  mirror of a local branch. This command will only function if launchpadlib
+  is installed.
+  (Jonathan Lange)
+
+
+Bug Fixes
+*********
+
+* After renaming a file, the dirstate could accidentally reference
+  ``source\\path`` rather than ``source/path`` on Windows. This might be a
+  source of some dirstate-related failures. (John Arbash Meinel)
+
+* ``bzr commit`` now detects commit messages that looks like file names
+  and issues a warning.
+  (Gioele Barabucci, #73073)
+
+* ``bzr ignore /`` no longer causes an IndexError. (Gorden Tyler, #456036)
+
+* ``bzr log -n0 -rN`` should not return revisions beyond its merged revisions.
+  (#325618, #484109, Marius Kruger)
+
+* ``bzr merge --weave`` and ``--lca`` will now create ``.BASE`` files for
+  files with conflicts (similar to ``--merge3``). The contents of the file
+  is a synthesis of all bases used for the merge.
+  (John Arbash Meinel, #40412)
+
+* ``bzr mv --quiet`` really is quiet now.  (Gordon Tyler, #271790)
+
+* ``bzr serve`` is more clear about the risk of supplying --allow-writes.
+  (Robert Collins, #84659)
+
+* ``bzr serve --quiet`` really is quiet now.  (Gordon Tyler, #252834)
+
+* Fix bug with redirected URLs over authenticated HTTP.
+  (Glen Mailer, Neil Martinsen-Burrell, Vincent Ladeuil, #395714)
+
+* Interactive merge doesn't leave branch locks behind.  (Aaron Bentley)
+
+* Lots of bugfixes for the test suite on Windows. We should once again
+  have a test suite with no failures on Windows. (John Arbash Meinel)
+
+* ``osutils.terminal_width()`` obeys the BZR_COLUMNS environment
+  variable but returns None if the terminal is not a tty (when output is
+  redirected for example). Also fixes its usage under OSes that doesn't
+  provide termios.TIOCGWINSZ. Make sure the corresponding tests runs on
+  windows too.
+  (Joke de Buhr, Vincent Ladeuil, #353370, #62539)
+  (John Arbash Meinel, Vincent Ladeuil, #492561)
+
+* Terminate SSH subprocesses when no references to them remain, fixing
+  subprocess and file descriptor leaks.  (Andrew Bennetts, #426662)
+  
+* The ``--hardlink`` option of ``bzr branch`` and ``bzr checkout`` now
+  works for 2a format trees.  Only files unaffected by content filters
+  will be hardlinked.  (Andrew Bennetts, #408193)
+
+* The new glob expansion on Windows would replace all ``\`` characters
+  with ``/`` even if it there wasn't a glob to expand, the arg was quoted,
+  etc. Now only change slashes if there is something being glob expanded.
+  (John Arbash Meinel, #485771)
+
+* Use our faster ``KnownGraph.heads()`` functionality when computing the
+  new rich-root heads. This can cut a conversion time in half (mysql from
+  13.5h => 6.2h) (John Arbash Meinel, #487632)
+
+* When launching a external diff tool via bzr diff --using, temporary files
+  are no longer created, rather, the path to the file in the working tree is
+  passed to the external diff tool. This allows the file to be edited if the
+  diff tool provides for this. (Gary van der Merwe, #490738)
+  
+* The launchpad-open command can now be used from a subdirectory of a
+  branch, not just from the root of the branch. 
+  (Neil Martinsen-Burrell, #489102)
+
+
+Improvements
+************
+
+* ``bzr log`` is now faster. (Ian Clatworthy)
+
+* ``bzr update`` provides feedback on which branch it is up to date with.
+  (Neil Martinsen-Burrell)
+
+* ``bzr upgrade`` from pre-2a to 2a can be significantly faster (4x).
+  For details see the xml8 patch and heads() improvements.
+  (John Arbash Meinel)
+
+* ``bzrlib.urlutils.local_path_from_url`` now accepts
+  'file://localhost/' as well as 'file:///' URLs on POSIX.  (Michael
+  Hudson)
+
+* The progress bar now shows only a spinner and per-operation counts,
+  not an overall progress bar.  The previous bar was often not correlated
+  with real overall operation progress, either because the operations take
+  nonlinear time, or because at the start of the operation Bazaar couldn't
+  estimate how much work there was to do.  (Martin Pool)
+
+Documentation
+*************
+
+* Lots of documentation tweaks for inline help topics and command help
+  information.
+
+API Changes
+***********
+
+* ``bzrlib.textui`` (vestigial module) removed.  (Martin Pool)
+
+* The Launchpad plugin now has a function ``login`` which will log in to
+  Launchpad with launchpadlib, and ``load_branch`` which will return the
+  Launchpad Branch object corresponding to a given Bazaar Branch object.
+  (Jonathan Lange)
+
+Internals
+*********
+
+* New test Feature: ``ModuleAvailableFeature``. It is designed to make it
+  easier to handle what tests you want to run based on what modules can be
+  imported. (Rather than lots of custom-implemented features that were
+  basically copy-and-pasted.) (John Arbash Meinel)
+
+* ``osutils.timer_func()`` can be used to get either ``time.time()`` or
+  ``time.clock()`` when you want to do performance timing.
+  ``time.time()`` is limited to 15ms resolution on Windows, but
+  ``time.clock()`` gives CPU and not wall-clock time on other platforms.
+  (John Arbash Meinel)
+
+* Several code paths that were calling ``Transport.get().read()`` have
+  been changed to the equalivent ``Transport.get_bytes()``. The main
+  difference is that the latter will explicitly call ``file.close()``,
+  rather than expecting the garbage collector to handle it. This helps
+  with some race conditions on Windows during the test suite and SFTP
+  tests. (John Arbash Meinel)
+
+Testing
+*******
+
+* TestCaseWithMemoryTransport no longer sets $HOME and $BZR_HOME to
+  unicode strings. (Michael Hudson, #464174)
+
+
+bzr 2.1.0b3
+###########
+
+:Codename: after sprint recovery
+:2.1.0b3: 2009-11-16
+
+This release was pushed up from its normal release cycle due to a
+regression in python 2.4 compatibility in 2.1.0b2.  Since this regression
+was caught before 2.1.0b2 was officially announced, the full changelog
+includes both 2.1.0b3 and 2.1.0b2 changes.
+
+Highlights of 2.1.0b3 are: new globbing code for all commands on Windows,
+the test suite now conforms to python's trunk enhanced semantics (skip,
+etc.), and ``bzr info -v`` will now report the correct branch and repo
+formats for Remote objects.
+
+
+New Features
+************
+
+* Users can define a shelve editor to provide shelf functionality at a
+  granularity finer than per-patch-hunk. (Aaron Bentley)
+
+Bug Fixes
+*********
+
+* Fix for shell completion and short options.  (Benoît PIERRE)
+
+* Fix ``bzr --profile-imports`` with Python 2.6.  (Martin Pool)
+
+* Hooks daughter classes should always call the base constructor.
+  (Alexander Belchenko, Vincent Ladeuil, #389648) 
+
+* Improve "Binary files differ" hunk handling.  (Aaron Bentley, #436325)
+
+* On Windows, do glob expansion at the command-line level (as is usually
+  done in bash, etc.) This means that *all* commands get glob expansion
+  (bzr status, bzr add, bzr mv, etc). It uses a custom command line
+  parser, which allows us to know if a given section was quoted. It means
+  you can now do ``bzr ignore "*.py"``.
+  (John Arbash Meinel, #425510, #426410, #194450)
+
+* Sanitize commit messages that come in from the '-m' flag. We translate
+  '\r\n' => '\n' and a plain '\r' => '\n'. The storage layer doesn't
+  allow those because XML store silently translate it anyway. (The parser
+  auto-translates \r\n => \n in ways that are hard for us to catch.)
+
+* Show correct branch and repository format descriptions in 
+  ``bzr info -v`` on a smart server location.  (Andrew Bennetts, #196080)
+
+* The fix for bug #186920 accidentally broke compatibility with python
+  2.4.  (Vincent Ladeuil, #475585)
+
+* Using ``Repository.get_commit_builder().record_iter_changes()`` now
+  correctly sets ``self.inv_sha1`` to a sha1 string and
+  ``self.new_inventory`` to an Inventory instance after calling
+  ``self.finish_inventory()``. (Previously it accidently set both values
+  as a tuple on ``self.inv_sha1``. This was missed because
+  ``repo.add_revision`` ignores the supplied inventory sha1 and recomputes
+  the sha1 from the repo directly. (John Arbash Meinel)
+
+* Shelve command refuse to run if there is no real terminal.
+  (Alexander Belchenko)
+
+* Avoid unnecessarily flushing of trace file; it's now unbuffered at the
+  Python level.  (Martin Pool)
+
+Documentation
+*************
+
+* Include Japanese translations for documentation (Inada Naoki)
+
+* New API ``ui_factory.make_output_stream`` to be used for sending bulk
+  (rather than user-interaction) data to stdout.  This automatically
+  coordinates with progress bars or other terminal activity, and can be
+  overridden by GUIs.
+  (Martin Pool, 493944)
+
+Internals
+*********
+
+* Some of the core groupcompress functionality now releases the GIL before
+  operation. Similar to how zlib and bz2 operate without the GIL in the
+  core compression and decompression routines. (John Arbash Meinel)
+
+Testing
+*******
+
+* -Dhpssvfs will now trigger on ``RemoteBzrDir._ensure_real``, providing
+  more debugging of VFS access triggers. (Robert Collins)
+
+* KnownFailure is now signalled to ``ExtendedTestResult`` using the same
+  method that Python 2.7 uses - ``addExpectedFailure``. (Robert Collins)
+
+* ``--parallel=fork`` is now compatible with --subunit.
+  (Robert Collins, Vincent Ladeuil, #419776)
+
+* Reporting of failures shows test ids not descriptions and thus shows
+  parameterised tests correctly. (Robert Collins)
+
+* TestNotApplicable is now handled within the TestCase.run method rather
+  than being looked for within ``ExtendedTestResult.addError``. This
+  provides better handling with other ``TestResult`` objects, degrading to
+  sucess rather than error. (Robert Collins)
+
+* The private method ``_testConcluded`` on ``ExtendedTestResult`` has been
+  removed - it was empty and unused. (Robert Collins)
+
+* UnavailableFeature is now handled within the TestCase.run method rather
+  than being looked for within addError. If the Result object does not
+  have an addNotSupported method, addSkip is attempted instead, and
+  failing that addSuccess. (Robert Collins)
+
+* When a TestResult does not have an addSkip method, skipped tests are now
+  reported as successful tests, rather than as errors. This change is
+  to make it possible to get a clean test run with a less capable
+  TestResult. (Robert Collins)
+
+
+
+bzr 2.1.0b2
+###########
+
+:Codename: a load off my mind
+:2.1.0b2: 2009-11-02
+
+This is our second feature-filled release since 2.0, pushing us down the
+path to a 2.1.0. Once again, all bugfixes in 2.0.2 are present in 2.1.0b2.
+
+Key highlights in this release are: improved handling of
+failures-during-cleanup for commit, fixing a long-standing bug with
+``bzr+http`` and shared repositories, all ``lp:`` URLs to be resolved
+behind proxies, and a new StaticTuple datatype, allowing us to reduce
+memory consumption (50%) and garbage collector overhead (40% faster) for
+many operations.
+
+* A new ``--concurrency`` option has been added as well as an associated
+  BZR_CONCURRENCY environment variable to specify the number of
+  processes that can be run concurrently when running ``bzr selftest``. The
+  command-line option overrides the environment variable if both are
+  specified. If none is specified. the number of processes is obtained
+  from the OS as before.  (Matt Nordhoff, Vincent Ladeuil)
+
+Bug Fixes
+*********
+
+* ``bzr+http`` servers no longer give spurious jail break errors when
+  serving branches inside a shared repository.  (Andrew Bennetts, #348308)
+
+* Errors during commit are handled more robustly so that knock-on errors
+  are less likely to occur, and will not obscure the original error if
+  they do occur.  This fixes some causes of ``TooManyConcurrentRequests``
+  and similar errors.  (Andrew Bennetts, #429747, #243391)
+
+* Launchpad URLs can now be resolved from behind proxies.
+  (Gordon Tyler, Vincent Ladeuil, #186920)
+
+* Reduce the strictness for StaticTuple, instead add a debug flag
+  ``-Dstatic_tuple`` which will change apis to be strict and raise errors.
+  This way, most users won't see failures, but developers can improve
+  internals. (John Arbash Meinel, #471193)
+
+* TreeTransform.adjust_path updates the limbo paths of descendants of adjusted
+  files.  (Aaron Bentley)
+
+* Unicode paths are now handled correctly and consistently by the smart
+  server.  (Andrew Bennetts, Michael Hudson, #458762)
+
+Improvements
+************
+
+* When reading index files, we now use a ``StaticTuple`` rather than a
+  plain ``tuple`` object. This generally gives a 20% decrease in peak
+  memory, and can give a performance boost up to 40% on large projects.
+  (John Arbash Meinel)
+
+* Peak memory under certain operations has been reduced significantly.
+  (eg, 'bzr branch launchpad standalone' is cut in half)
+  (John Arbash Meinel)
+
+Documentation
+*************
+
+* Filtered views user documentation upgraded to refer to format 2a
+  instead of pre-2.0 formats. (Ian Clatworthy)
+
+API Changes
+***********
+
+* Remove deprecated ``CLIUIFactory``.  (Martin Pool)
+
+* ``UIFactory`` now has new ``show_error``, ``show_message`` and
+  ``show_warning`` methods, which can be hooked by non-text UIs.  
+  (Martin Pool)
+
+Internals
+*********
+
+* Added ``bzrlib._simple_set_pyx``. This is a hybrid between a Set and a
+  Dict (it only holds keys, but you can lookup the object located at a
+  given key). It has significantly reduced memory consumption versus the
+  builtin objects (1/2 the size of Set, 1/3rd the size of Dict). This is
+  used as the interning structure for StaticTuple objects.
+  (John Arbash Meinel)
+
+* ``bzrlib._static_tuple_c.StaticTuple`` is now available and used by
+  the btree index parser and the chk map parser. This class functions
+  similarly to ``tuple`` objects. However, it can only point to a limited
+  collection of types.  (Currently StaticTuple, str, unicode, None, bool,
+  int, long, float, but not subclasses).  This allows us to remove it from
+  the garbage collector (it cannot be in a cycle), it also allows us to
+  intern the objects. In testing, this can reduce peak memory by 20-40%,
+  and significantly improve performance by removing objects from being
+  inspected by the garbage collector.  (John Arbash Meinel)
+
+* ``GroupCompressBlock._ensure_content()`` will now release the
+  ``zlib.decompressobj()`` when the first request is for all of the
+  content. (Previously it would only be released if you made a request for
+  part of the content, and then all of it later.) This turns out to be a
+  significant memory savings, as a ``zstream`` carries around approx 260kB
+  of internal state and buffers. (For branching bzr.dev this drops peak
+  memory from 382MB => 345MB.) (John Arbash Meinel)
+
+* When streaming content between ``2a`` format repositories, we now clear
+  caches from earlier versioned files. (So 'revisions' is cleared when we
+  start reading 'inventories', etc.) This can have a significant impact on
+  peak memory for initial copies (~200MB). (John Arbash Meinel)
+
+
+bzr 2.1.0b1
+###########
+
+:Codename: While the cat is away
+:2.1.0b1: 2009-10-14
+
+This is the first development release in the new split "stable" and
+"development" series. As such, the release is a snapshot of bzr.dev
+without creating a release candidate first. This release includes a
+fair amount of internal changes, with deprecated code being removed,
+and several new feature developments. People looking for a stable code
+base with only bugfixes should focus on the 2.0.1 release. All bugfixes
+present in 2.0.1 are present in 2.1.0b1.
+
+Highlights include support for ``bzr+ssh://host/~/homedir`` style URLs,
+finer control over the plugin search path via extended BZR_PLUGIN_PATH
+syntax, visible warnings when extension modules fail to load, and improved
+error handling during unlocking.
+
+
+New Features
+************
+
+* Bazaar can now send mail through Apple OS X Mail.app. 
+  (Brian de Alwis)
+
+* ``bzr+ssh`` and ``bzr`` paths can now be relative to home directories
+  specified in the URL.  Paths starting with a path segment of ``~`` are
+  relative to the home directory of the user running the server, and paths
+  starting with ``~user`` are relative to the home directory of the named
+  user.  For example, for a user "bob" with a home directory of
+  ``/home/bob``, these URLs are all equivalent:
+
+  * ``bzr+ssh://bob@host/~/repo``
+  * ``bzr+ssh://bob@host/~bob/repo``
+  * ``bzr+ssh://bob@host/home/bob/repo``
+
+  If ``bzr serve`` was invoked with a ``--directory`` argument, then no
+  home directories outside that directory will be accessible via this
+  method.
+
+  This is a feature of ``bzr serve``, so pre-2.1 clients will
+  automatically benefit from this feature when ``bzr`` on the server is
+  upgraded.  (Andrew Bennetts, #109143)
+
+* Extensions can now be compiled if either Cython or Pyrex is available.
+  Currently Pyrex is preferred, but that may change in the future.
+  (Arkanes)
+
+* Give more control on BZR_PLUGIN_PATH by providing a way to refer to or
+  disable the user, site and core plugin directories.
+  (Vincent Ladeuil, #412930, #316192, #145612)
+
+Bug Fixes
+*********
+
+* Bazaar's native protocol code now correctly handles EINTR, which most
+  noticeably occurs if you break in to the debugger while connected to a
+  bzr+ssh server.  You can now can continue from the debugger (by typing
+  'c') and the process continues.  However, note that pressing C-\ in the
+  shell may still kill the SSH process, which is bug 162509, so you must
+  sent a signal to the bzr process specifically, for example by typing
+  ``kill -QUIT PID`` in another shell.  (Martin Pool, #341535)
+
+* ``bzr add`` in a tree that has files with ``\r`` or ``\n`` in the
+  filename will issue a warning and skip over those files.
+  (Robert Collins, #3918)
+
+* ``bzr dpush`` now aborts if uncommitted changes (including pending merges)
+  are present in the working tree. The configuration option ``dpush_strict``
+  can be used to set the default for this behavior.
+  (Vincent Ladeuil, #438158)
+
+* ``bzr merge`` and ``bzr remove-tree`` now requires --force if pending
+  merges are present in the working tree.
+  (Vincent Ladeuil, #426344)
+
+* Clearer message when Bazaar runs out of memory, instead of a ``MemoryError``
+  traceback.  (Martin Pool, #109115)
+
+* Don't give a warning on Windows when failing to import ``_readdir_pyx``
+  as it is never built. (John Arbash Meinel, #430645)
+
+* Don't restrict the command name used to run the test suite.
+  (Vincent Ladeuil, #419950)
+
+* FTP transports were built differently when the kerberos python module was
+  present leading to obscure failures related to ASCII/BINARY modes.
+  (Vincent Ladeuil, #443041)
+
+* Network streams now decode adjacent records of the same type into a
+  single stream, reducing layering churn. (Robert Collins)
+
+* PreviewTree behaves correctly when get_file_mtime is invoked on an unmodified
+  file. (Aaron Bentley, #251532)
+
+* Registry objects should not use iteritems() when asked to use items().
+  (Vincent Ladeuil, #430510)
+
+* Weave based repositories couldn't be cloned when committers were using
+  domains or user ids embedding '.sig'. Now they can.
+  (Matthew Fuller, Vincent Ladeuil, #430868)
+
+Improvements
+************
+
+* Revision specifiers can now be given in a more DWIM form, without
+  needing explicit prefixes for specifiers like tags or revision id's.
+  See ``bzr help revisionspec`` for full details.  (Matthew Fuller)
+
+* Bazaar gives a warning before exiting, and writes into ``.bzr.log``, if 
+  compiled extensions can't be loaded.  This typically indicates a
+  packaging or installation problem.  In this case Bazaar will keep
+  running using pure-Python versions, but this may be substantially
+  slower.  The warning can be disabled by setting
+  ``ignore_missing_extensions = True`` in ``bazaar.conf``.
+  See also <https://answers.launchpad.net/bzr/+faq/703>.
+  (Martin Pool, #406113, #430529)
+
+* Secondary errors that occur during Branch.unlock and Repository.unlock
+  no longer obscure the original error.  These methods now use a new
+  decorator, ``only_raises``.  This fixes many causes of
+  ``TooManyConcurrentRequests`` and similar errors.
+  (Andrew Bennetts, #429747)
+
+Documentation
+*************
+
+* Describe the new shell-like test feature. (Vincent Ladeuil)
+
+* Help on hooks no longer says 'Not deprecated' for hooks that are
+  currently supported. (Ian Clatworthy, #422415)
+
+API Changes
+***********
+
+* ``bzrlib.user_encoding`` has been removed; use
+  ``bzrlib.osutils.get_user_encoding`` instead.  (Martin Pool)
+
+* ``bzrlib.tests`` now uses ``stopTestRun`` for its ``TestResult``
+  subclasses - the same as python's unittest module. (Robert Collins)
+  
+* ``diff._get_trees_to_diff`` has been renamed to 
+  ``diff.get_trees_and_branches_to_diff``. It is now a public API, and it 
+  returns the old and new branches. (Gary van der Merwe)
+
+* ``bzrlib.trace.log_error``, ``error`` and ``info`` have been deprecated.
+  (Martin Pool)
+
+* ``MutableTree.has_changes()`` does not require a tree parameter anymore. It
+  now defaults to comparing to the basis tree. It now checks for pending
+  merges too.  ``Merger.check_basis`` has been deprecated and replaced by the
+  corresponding has_changes() calls. ``Merge.compare_basis``,
+  ``Merger.file_revisions`` and ``Merger.ensure_revision_trees`` have also
+  been deprecated.
+  (Vincent Ladeuil, #440631)
+
+* ``ProgressTask.note`` is deprecated.
+  (Martin Pool)
+
+Internals
+*********
+
+* Added ``-Drelock`` debug flag.  It will ``note`` a message every time a
+  repository or branch object is unlocked then relocked the same way.
+  (Andrew Bennetts)
+  
+* ``BTreeLeafParser.extract_key`` has been tweaked slightly to reduce
+  mallocs while parsing the index (approx 3=>1 mallocs per key read).
+  This results in a 10% speedup while reading an index.
+  (John Arbash Meinel)
+
+* The ``bzrlib.lsprof`` module has a new class ``BzrProfiler`` which makes
+  profiling in some situations like callbacks and generators easier.
+  (Robert Collins)
+
+Testing
+*******
+
+* Passing ``--lsprof-tests -v`` to bzr selftest will cause lsprof output to
+  be output for every test. Note that this is very verbose! (Robert Collins)
+
+* Setting ``BZR_TEST_PDB=1`` when running selftest will cause a pdb
+  post_mortem to be triggered when a test failure occurs. (Robert Collins)
+
+* Shell-like tests can now be written. Code in ``bzrlib/tests/script.py`` ,
+  documentation in ``developers/testing.txt`` for details.
+  (Vincent Ladeuil)
+
+* Some tests could end up with the same id, that was dormant for
+  a long time.
+  (Vincent Ladeuil, #442980)
+
+* Stop showing the number of tests due to missing features in the test
+  progress bar.  (Martin Pool)
+
+* Test parameterisation now does a shallow copy, not a deep copy of the test
+  to be parameterised. This is not expected to break external use of test
+  parameterisation, and is substantially faster. (Robert Collins)
+
+* Tests that try to open a bzr dir on an arbitrary transport will now
+  fail unless they have explicitly permitted the transport via
+  ``self.permit_url``. The standard test factories such as ``self.get_url``
+  will permit the URLs they provide automatically, so only exceptional
+  tests should need to do this. (Robert Collins)
+
+* The break-in test no longer cares about clean shutdown of the child,
+  instead it is happy if the debugger starts up. (Robert  Collins)
+
+* The full test suite is expected to pass when the C extensions are not
+  present. (Vincent Ladeuil, #430749)
+
+
+..
+   vim: tw=74 ft=rst ff=unix
+

=== added file 'doc/en/release-notes/bzr-2.2.txt'
--- doc/en/release-notes/bzr-2.2.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-2.2.txt	2011-09-09 13:30:12 +0000
@@ -0,0 +1,1340 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.2.6
+#########
+
+:2.2.6: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+New Features
+************
+
+Bug Fixes
+*********
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+
+bzr 2.2.5
+#########
+
+:2.2.5: 2011-09-01
+
+This is a bugfix release. One regression introduced in 2.2b1 has been fixed
+for some rare conflict resolutions. Also a warning is now emmitted when
+branching an out-of-date ubuntu packaging branch. Upgrading is recommended
+for all users on earlier 2.2 releases.
+
+Compatibility Breaks
+********************
+
+None.
+
+New Features
+************
+
+None.
+
+Bug Fixes
+*********
+
+* Correctly handle ``bzr log`` and `get_known_graph_ancestry` on a
+  doubly-stacked branch.
+  (James Westby, Martin Pool, #715000)
+
+* Don't crash while merging and encountering obscure path conflicts
+  involving different root-ids. (Vincent Ladeuil, #805809)
+
+Internals
+*********
+
+* Fixed bug in the bundled copy of ConfigObj with quoting of triple quotes
+  in the value string. Fix suggested by ConfigObj's author Michael Foord.
+  (Alexander Belchenko, #710410)
+
+
+bzr 2.2.4
+#########
+
+:2.2.4: 2011-02-04
+
+This is a bugfix release. Only one bug has been fixed, a regression from 2.2.3
+involving only certain operations with launchpad. Upgrading is recommended for
+all users on earlier 2.2 releases.
+
+Bug Fixes
+*********
+
+* Fix communications with the Launchpad web service when using
+  launchpadlib >= 1.5.5.  This was a latent bug in bzr's communication
+  with Launchpad's production instance, which only became a problem when
+  the default instance was switched from edge to production in bzr 2.2.3.
+  (Max Bowsher, #707075)
+
+
+bzr 2.2.3
+#########
+
+:2.2.3: 2011-01-20
+
+This is a bugfix release. Upgrading is recommended for all users
+on earlier 2.2 releases.
+
+Compatibility Breaks
+********************
+
+* Launchpad has announced that the ``edge.launchpad.net`` instance is
+  deprecated and may be shut down in the future
+  <http://blog.launchpad.net/general/edge-is-deprecated>.  Bazaar has therefore
+  been updated in this release to talk to the main (``launchpad.net``) servers,
+  rather than the ``edge`` ones. (Vincent Ladeuil, #583667)
+
+Bug Fixes
+*********
+
+* Avoid UnicodeDecodeError in ``bzr add`` with multiple files under a non-ascii
+  path on windows from symlink support addition. (Martin [gz], #686611)
+
+* Correctly resolve content (and path) conflicts for files in subdirs.
+  (Vincent Ladeuil, #660935)
+
+* Don't probe for a repository from within ``NotBranchError.__repr__``,
+  because this can cause knock-on errors at awkward times.
+  (Andrew Bennetts, #687653)
+
+* Fix a crash during ``RepositoryPackCollection.pack`` caused by a
+  concurrent repository pack operation.  This was particularly affecting
+  ``bzr-svn`` users.  (Andrew Bennetts, #701940)
+
+* ``https`` access works again with recent versions of python2.7.
+  (Vincent Ladeuil, #693880)
+
+* RevisionTree.is_executable no longer returns None for directories and
+  symlinks.  Instead, it returns False, like other Trees and methods.
+  (Aaron Bentley, #681885)
+
+
+bzr 2.2.2
+#########
+
+:2.2.2: 2010-11-25
+
+This is a bugfix release. None of these bugfixes are critical, but upgrading
+is recommended for all users on earlier 2.2 releases.
+
+Bug Fixes
+*********
+
+* ``bzr resolve --take-other <file>`` will not crash anymore if ``<file>``
+  is involved in a text conflict (but the conflict is still not
+  resolved). (Vincent Ladeuil, #646961)
+
+* Commit in a bound branch or heavyweight checkout now propagates tags
+  (e.g. from a merge) to the master branch (and informs the user if there
+  is a conflict).  (Andrew Bennetts, #603395)
+  
+* Correctly set the Content-Type header when HTTP POSTing to comply
+  with stricter web frameworks. (Vincent Ladeuil, #665100)
+
+* ``NotBranchError`` no longer allows errors from calling
+  ``bzrdir.open_repository()`` to propagate.  This is unhelpful at best,
+  and at worst can trigger infinite loops in callers.  (Andrew Bennetts)
+  
+* Skip tests that needs a bzr source tree when there isn't one. This is
+  needed to succesfully run the test suite for installed versions.
+  (Vincent Ladeuil, #644855).
+
+* Skip the tests that requires respecting the chmod bits when running as
+  root. Including the one that wasn't present in 2.1.
+  (Vincent Ladeuil, #646133)
+
+* Using bzr with `lp:` URLs behind an HTTP proxy should work.
+  (Robert Collins, #558343)
+
+* Windows installers no longer requires the Microsoft vcredist to be
+  installed.
+  (Martin [gz], Gary van der Merwe, #632465)
+
+* Close leaked socket to SSH subprocesses, which caused dput sftp uploads
+  to hang.  (Max Bowsher, #659590)
+
+Testing
+*******
+
+* Add ``tests/ssl_certs/ca.crt`` to the required test files list. Test
+  involving the pycurl https test server fail otherwise when running
+  selftest from an installed version. (Vincent Ladeuil, #651706)
+
+* Fix tests that failed when run under ``LANG=C``.
+  (Andrew Bennetts, #632387)
+
+
+bzr 2.2.1
+#########
+
+:2.2.1: 2010-09-17
+
+This is a bugfix release which also includes bugfixes from 2.0.6 and
+2.1.3. None are critical, but upgrading is recommended for all users on
+earlier 2.2 releases.
+
+Bug Fixes
+*********
+
+* Additional merges after an unrelated branch has been merged with its
+  history no longer crash when deleted files are involved.
+  (Vincent Ladeuil, John Arbash Meinel, #375898)
+
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.
+  (Martin Pool, #192859)
+
+* ``bzr commit SYMLINK`` now works, rather than trying to commit the
+  target of the symlink.
+  (Martin Pool, John Arbash Meinel, #128562)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* CommitBuilder now uses the committer instead of _config.username to generate
+  the revision-id.  (Aaron Bentley, #614404)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* Cope with Microsoft FTP server that returns reply '250 Directory
+  created' when mkdir succeeds.  (Martin Pool, #224373)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* Fix ``AttributeError on parent.children`` when adding a file under a
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Fix ``AttributeError: 'NoneType' object has no attribute 'close'`` in
+  ``_close_ssh_proc`` when using ``bzr+ssh://``.  This was causing
+  connections to pre-1.6 bzr+ssh servers to fail, and causing warnings on
+  stderr in some other circumstances.  (Andrew Bennetts, #633745)
+
+* Only call ``setlocale`` in the bzr startup script on posix systems. This
+  avoids an issue with the newer windows C runtimes used by Python 2.6 and
+  later which can mangle bytestrings printed to the console.
+  (Martin [gz], #631350)
+
+* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
+  which can result in "missing referenced chk root keys" errors when
+  fetching from repositories with affected revisions.
+  (Andrew Bennetts, #522637)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Repositories accessed via a smart server now reject being stacked on a
+  repository in an incompatible format, as is the case when accessing them
+  via other methods.  This was causing fetches from those repositories via
+  a smart server (e.g. using ``bzr branch``) to receive invalid data.
+  (Andrew Bennetts, #562380)
+
+* Selftest with versions of subunit that support ``stopTestRun`` will no longer
+  error. This error was caused by 2.0 not being updated when upstream
+  python merged the end of run patch, which chose ``stopTestRun`` rather than
+  ``done``. (Robert Collins, #571437)
+
+* Stop ``AttributeError: 'module' object has no attribute 'ElementTree'``
+  being thrown from ``xml_serializer`` on certain cElementTree setups.
+  (Martin [gz], #254278)
+
+* Upgrading or fetching from a non-rich-root repository to a rich-root
+  repository (e.g. from pack-0.92 to 2a) no longer fails with
+  ``'Inter1and2Helper' object has no attribute 'source_repo'``.  This was
+  a regression from Bazaar 2.1.  (Andrew Bennetts, #636930)
+  
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+Documentation
+*************
+
+* Fix a lot of references in the docs to the old http://bazaar-vcs.org to
+  the new http://bazaar.canonical.com or http://wiki.bazaar.canonical.com
+  (John Arbash Meinel, #617503)
+
+Internals
+*********
+
+* Remove used and broken code path in ``BranchInitHookParams.__repr__``.
+  (Andrew Bennetts)
+
+Testing
+*******
+
+* ``build_tree_contents`` can create symlinks.
+  (Martin Pool, John Arbash Meinel)
+
+* Tracebacks from a parameterized test are no longer reported against every
+  parameterization of that test.  This was done by adding a hack to
+  ``bzrlib.tests.clone_test`` so that it no longer causes
+  testtools.TestCase instances to share a details dict.
+  (Andrew Bennetts, #625574)
+
+bzr 2.2
+#######
+
+:Codename: La Hulpe
+:2.2: 2010-08-06
+
+This release marks the start of another long-term-stable series. From
+here, we will only make bugfix releases on the 2.2 series (2.2.1, etc),
+while 2.3 will become our new development series. The 2.0 and 2.1 series
+will also continue to get bugfixes. (Currently 2.0 is planned to be
+supported for another 6 months.)
+
+This is primarily a bugfix and polish release over the 2.1 series, with
+a large number of bugs fixed (>120), and some performance improvements.
+
+There are some compatibility changes in this release.  For users of bzrlib
+as a library, we now request that they call ``bzrlib.initialize`` and use
+the returned context manager appropriately. For commandline users we no
+longer guess user identity for ``bzr commit``, users must specify their
+identity using ``bzr whoami`` (you don't need to specify your identity for
+readonly operations).
+
+Users are encouraged to upgrade from the other stable series.
+
+Compatibility Breaks
+********************
+
+* BzrError subclasses no longer support the name "message" to be used
+  as an argument for __init__ or in _fmt format specification as this
+  breaks in some Python versions. errors.LockError.__init__ argument
+  is now named "msg" instead of earlier "message".
+  (Parth Malwankar, #603461)
+
+* The old ``bzr selftest --benchmark`` option has been removed.
+  <https://launchpad.net/bzr-usertest> is an actively-maintained
+  macrobenchmark suite.
+  (Martin Pool)
+
+Bug Fixes
+*********
+
+* ``bzr ignore PATTERNS`` exits with error if a bad pattern is supplied.
+  ``InvalidPattern`` exception error message now shows faulting
+  regular expression.
+  (Parth Malwankar #300062)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* Don't traceback trying to unversion children files of an already
+  unversioned directory.  (Vincent Ladeuil, #494221)
+
+* ``HTTP/1.1`` test servers now set a ``Content-Length`` header to comply
+  with pedantic ``HTTP/1.1`` clients. (Vincent Ladeuil, #568421)
+
+* Progress bars prefer to truncate the text message rather than the
+  counters.  The spinner is shown between the network transfer indicator
+  and the progress message.  Progress bars are correctly cleared off when 
+  they finish.  (Martin Pool, #611127)
+
+* Recursive binding for checkouts is now detected by bzr. A clear error
+  message is shown to the user. (Parth Malwankar, #405192)
+
+Improvements
+************
+
+* Add ``bzrlib.merge.MergeIntoMerger``, which can merge part or all of a
+  tree, and works with unrelated branches.  (Andrew Bennetts)
+
+* Add py2exe windows target ``bzrw.exe``. This allow for starting a Bazaar
+  GUI with out have a console open in the background.
+  (Gary van der Merwe, #433781)
+
+Documentation
+*************
+
+* ``bzr help patterns`` now explains case insensitive patterns and
+  points to Python regular expression documentation.
+  (Parth Malwankar, #594386)
+
+API Changes
+***********
+
+* Delete ``ProgressTask.note``, which was deprecated in 2.1.
+
+Testing
+*******
+
+* Unit test added to ensure that "message" is not uses as a format variable
+  name in BzrError subclasses as this conflicts with some Python versions.
+  (Parth Malwankar, #603461)
+
+bzr 2.2b4
+#########
+
+:Codename: Monkey Magic
+:2.2b4: 2010-07-10
+
+
+This fourth and final beta in the 2.2 series now stabilizes the internal
+APIs. Plugin authors are recommended to ensure their releases are
+compatible, so that 2.2rc1 can be a true release candidate, containing
+stable and compatible plugin versions. 
+
+For users of bzrlib as a library, one of the primary changes is to request
+that they call ``bzrlib.initialize`` and use the returned context manager
+appropriately.
+
+Better interaction with ``bzr-loom`` to make sure branching from a loom
+even over a smart server still yields a local loom. Not to mention lots of
+bugfixes over 2.2b3.
+
+Compatibility Breaks
+********************
+
+* bzrlib library users now need to call ``__enter__`` and ``__exit__`` on
+  the result of ``bzrlib.initialize``. This change was made when fixing
+  the bad habit recent bzr versions have had of leaving progress bars 
+  behind on the screen. That required calling another function before
+  exiting the program, and it made sense to provide a full context
+  manager at the same time. (Robert Collins)
+
+* The ``bzr`` front end now requires a ``bzrlib.ui.ui_factory`` which is a
+  context manager in the Python 2.5 and above sense. The bzrlib base class
+  is such a manager, but third party UI factories which do not derive from
+  ``bzrlib.ui.UIFactory`` will be incompatible with the command line front
+  end.
+
+* URLs like ``foo:bar/baz`` are now always parsed as a URL with scheme "foo"
+  and path "bar/baz", even if bzr does not recognize "foo" as a known URL
+  scheme.  Previously these URLs would be treated as local paths.
+  (Gordon Tyler)
+
+
+New Features
+************
+
+* Support ``--directory`` option for a number of additional commands:
+  conflicts, merge-directive, missing, resolve, shelve, switch,
+  unshelve, whoami. (Martin von Gagern, #527878)
+
+Bug Fixes
+*********
+
+* ``bzr branch`` to a new repository with a default stacking policy no
+  longer transfers the full history unnecessarily.
+  (Andrew Bennetts, #597942)
+
+* ``bzr init`` does not recursively scan directory contents anymore
+  leading to faster init for directories with existing content.
+  (Martin [gz], Parth Malwankar, #501307)
+
+* ``bzr log --exclude-common-ancestry`` is now taken into account for
+  linear ancetries. (Vincent Ladeuil, #575631)
+
+* ``bzr log -r branch:REMOTE`` can now properly log the remote branch,
+  rather than trying to fetch the data locally and failing because of a
+  readonly error. (Martin von Gagern, #149270)
+
+* ``bzr pull`` now works when a lp: URL is explicitly defined as the parent
+  or pull location in locations.conf or branch.conf.
+  (Gordon Tyler, #534787)
+
+* ``bzr reconfigure --unstacked`` now works with branches accessed via a
+  smart server. (Andrew Bennetts, #551525)
+
+* ``BzrDir.find_branches`` should ignore branches with missing repositories.
+  (Marius Kruger, Robert Collins)
+
+* ``BzrDir.find_bzrdirs`` should ignore dirs that raises PermissionDenied.
+  (Marius Kruger, Robert Collins)
+
+* Ensure that wrong path specifications in ``BZR_PLUGINS_AT`` display
+  proper error messages. (Vincent Ladeuil, #591215)
+
+* Explicitly removing ``--profile-imports`` option from parsed command-line
+  arguments on Windows, because bzr script does the same.
+  (Alexander Belchenko, #588277)
+
+* Fetching was slightly confused about the best code to use and was
+  using a new code path for all branches, resulting in more lookups than
+  necessary on old branches. (Robert Collins, #593515)
+
+* Final fix for 'no help for command' issue. We now show a clean message
+  when a command has no help, document how to set help more clearly, and
+  test that all commands available to the test suite have help.
+  (Robert Collins, #177500)
+
+* Invalid patterns supplied to ``Globster`` or ``lazy_regex`` now raise
+  ``InvalidPattern`` exception showing clear error message to the user.
+  (Parth Malwankar #300062)
+
+* Progress output is cleaned up when exiting.  (Aaron Bentley)
+
+* Raise ValueError instead of a string exception.
+  (John Arbash Meinel, #586926)
+
+* Relative imports in plugins are now handled correctly when using
+  BZR_PLUGINS_AT. (Vincent Ladeuil, #588959)
+
+* ``ScriptRunner`` now strips off leading indentation from test scripts,
+  which previously caused "SyntaxError: No command for line".
+  (Martin Pool)
+
+* Show unicode filenames in diff headers using terminal encoding. 
+  (Alexander Belchenko, Bug #382699)
+  NOTE for Windows users: If user need to save diff to file then user need to
+  change encoding of the terminal to ANSI encoding with command ``chcp XXX``
+  (e.g. ``chcp 1251`` for Russian Windows).
+
+* URL displayed for use with ``break-lock`` when smart server sees lock
+  contention are now valid. Default timeout for lock contention retry is
+  now 30 seconds instead of 300 seconds.
+  (Parth Malwankar, #250451)
+
+* ``walkdirs`` now raises a useful message when the filenames are not using
+  the filesystem encoding. (Eric Moritz, #488519)
+
+* Enable debugging of bzr on windows with pdb and other tools. This was 
+  broken because we call GetCommandLineW on windows. The fix adjusts the 
+  command line we get to be the same length as sys.argv.
+  (Jason Spashett, Alexander Belchenko, #587868)
+
+Improvements
+************
+
+* Bazaar now reads data from SSH connections more efficiently on platforms
+  that provide the ``socketpair`` function, and when using paramiko.
+  (Andrew Bennetts, #590637)
+
+* ``Branch.copy_content_into`` is now a convenience method dispatching to
+  a ``InterBranch`` multi-method. This permits ``bzr-loom`` and other
+  plugins to intercept this even when a ``RemoteBranch`` proxy is in use.
+  (Robert Collins, #201613)
+
+* ``Branch`` formats can now be loaded lazily by registering a
+  ``MetaDirBranchFormatFactory`` rather than an actual format. This will
+  cause the named format class to be loaded only when an enumeration of
+  formats is needed or when the format string for the object is
+  encountered. (Robert Collins, Jelmer Vernooij)
+
+* The encoding that bzr uses to output things other than file content can
+  now be overridden via the output_encoding configuration option.
+  (Martin Pool, #340394)
+
+* Use lazy imports in ``bzrlib/merge.py`` so that plugins like ``news_merge``
+  do not cause modules to be loaded unnecessarily just because the plugin
+  registers a merge hook.  This improves ``bzr rocks`` time by about 25%
+  in a default installation (with just the core plugins).
+  (Andrew Bennetts)
+
+Documentation
+*************
+
+* Added ``regression`` tag to our tags list. (Robert Collins)
+
+* Improved our release checklist to have a bit less churn and leave things
+  ready-to-go for the next action (including other people doing
+  development). (Robert Collins)
+
+* Remove obsolete discussion of PQM in documentation about how to
+  contribute to Bazaar.  (Martin Pool, #588444)
+
+API Changes
+***********
+
+* ``bzrlib.branch.InterBranch._get_branch_formats_to_test`` now returns
+  an iterable of format pairs, rather than just a single pair, permitting
+  InterBranch objects that work with multiple permutations to be
+  comprehensively tested. (Robert Collins)
+
+* ``bzrlib.lsprof.profile`` will no longer silently generate bad threaded
+  profiles when concurrent profile requests are made. Instead the profile
+  requests will be serialised. Reentrant requests will now deadlock.
+  (Robert Collins)
+
+* ``bzrlib.knit.KnitSequenceMatcher``, which has been deprecated since
+  2007, has been deleted.  Use ``PatienceSequenceMatcher`` from
+  ``bzrlib.patiencediff`` instead. (Andrew Bennetts)
+
+* ``bzrlib.re_compile_checked`` is now deprecated. Caller should handle
+  ``bzrlib.errors.InvalidPattern`` exception thrown by ``re.match`` in
+  case the default error message not suitable for the use case.
+  (Parth Malwankar)
+
+* ``bzrlib.tests.blackbox.ExternalBase`` is deprecated.  It provided only
+  one method ``check_output``, and we now recommend checking command
+  output using ``run_script``. (Martin Pool)
+
+* ``bzrlib.transport.ssh.SSHVendor.connect_ssh`` now returns an object
+  that implements the interface of ``bzrlib.transport.ssh.SSHConnection``.
+  Third-party implementations of ``SSHVendor`` may need to be updated
+  accordingly.  Similarly, any code using ``SSHConnection`` directly will
+  need to be updated.  (Andrew Bennetts)
+
+* The constructor of ``bzrilb.smart.medium.SmartSSHClientMedium`` has
+  changed to take an ``SSHParams`` instance (replacing many individual
+  values).  (Andrew Bennetts)
+
+Internals
+*********
+
+* ``bzrlib.osutils.get_terminal_encoding`` will now only mutter its
+  selection when explicitly requested; this avoids many duplicate calls
+  being logged when helpers, wrappers and older code that manually calls
+  it are executed it is now logged deliberately by the ui setup code.
+  (Robert Collins)
+
+* Improved ``bzrlib.urlutils`` to handle lp:foo/bar URLs. (Gordon Tyler)
+
+* ``bzrlib._c_static_tuple.StaticTuple`` now implements ``__sizeof__``, so
+  that ``sys.getsizeof`` and other memory analysis tools will report more
+  accurate results. (Andrew Bennetts)
+
+* The symbol_versioning module can now cleanup after itself -
+  ``suppress_deprecation_warnings`` now returns a cleanup function.
+  (Robert Collins)
+
+Testing
+*******
+
+* Add ``bzrlib.tests.fixtures`` to hold code for setting up objects
+  to test.  (Martin Pool)
+
+* ``test_import_tariff`` now respects BZR_PLUGINS_AT and BZR_PLUGINS_DISABLE.
+  (Vincent Ladeuil, #595587)
+
+
+bzr 2.2b3
+#########
+
+:2.2b3: 2010-05-28
+
+This third beta in the 2.2 series brings with it all the goodness of 2.1.2
+and 2.0.6 (though it preceeds 2.0.6 slightly). Of particular note for
+users are compatibility fixes with bzr 1.5 and below servers, a hopeful
+end to the EINTR errors caused by SIGWINCH interactions, a shiny new
+bash completion script and bzr will no longer guess at identity details -
+it was too unreliable in reality. Use ``bzr whoami`` on every new install.
+For developers we have some API changes which may impact plugins as well
+as a bunch of our regular improvements to internal clarity and test
+support.
+
+Compatibility Breaks
+********************
+
+* An API break has been made to the lock_write method of ``Branch`` and
+  ``Repository`` objects; they now return ``branch.BranchWriteLockResult``
+  and ``repository.RepositoryWriteLockResult`` objects. This makes
+  changing the API in future easier and permits some cleaner calling code.
+  The lock_read method has also changed from having no defined return
+  value to returning ``LogicalLockResult`` objects.
+  (Robert Collins)
+
+* ``bzr`` does not try to guess the username as ``username@hostname``
+  and requires it to be explictly set. This can be set using ``bzr
+  whoami``. (Parth Malwankar, #549310)
+
+* ``bzrlib.commands.Command`` will now raise ValueError during
+  construction if there is no __doc__ set. (Note, this will be reverted in
+  2.2b4) (Robert Collins)
+
+* The source tree no longer contains a contrib/zsh/_bzr completion
+  script. The new file contrib/zsh/README suggests alternatives.
+  (Martin von Gagern, #560030)
+
+New Features
+************
+
+* ``bzr commit`` accepts ``-p`` (for "patch") as a shorter name for
+  ``--show-diff``.
+  (Parth Malwankar, #571467)
+  
+* ``bzr ignore`` now supports a ``--default-rules`` option that displays
+  the default ignore rules used by bzr. The flag ``--old-default-rules``
+  is no longer supported by ``ignore``.
+  (Parth Malwankar, #538703)
+
+* ``bzr pack`` now supports a ``--clean-obsolete-packs`` option that
+  can save disk space by deleting obsolete pack files created during the
+  pack operation.
+  (Parth Malwankar, #304320)
+
+* New command line option ``--authors`` to ``bzr log`` allows users to
+  select which of the apparent authors and committer should be
+  included in the log. Defaults depend on format. (Martin von Gagern, #513322)
+
+* Support ``--directory`` option for a number of additional commands:
+  added, annotate, bind, cat, cat-revision, clean-tree, deleted,
+  export, ignore, ignored, lookup-revision, ls, modified, nick,
+  re-sign, unbind, unknowns.
+  (Martin von Gagern, #527878)
+
+* The bash_completion plugin from the bzr-bash-completion project has
+  been merged into the tree. It provides a bash-completion command and
+  replaces the outdated ``contrib/bash/bzr`` script with a version
+  using the plugin. (Martin von Gagern, #560030)
+
+* A new transport based on GIO (the Gnome I/O library) provides access to
+  Samba shares, WebDAV using gio+smb and gio+dav. It is also possible to
+  use gio for some already existing transport methods as gio+file,
+  gio+sftp, gio+ftp. 
+  (Mattias Eriksson)
+
+Bug Fixes
+*********
+
+* Alias information shown by ``bzr help`` is now accurate. This
+  was showing an internal object name for some plugin aliases.
+  (Parth Malwankar, #584650)
+
+* ``.bazaar``, ``.bazaar/bazaar.conf`` and ``.bzr.log`` inherit user and
+  group ownership from the containing directory. This allow bzr to work
+  better with sudo.
+  (Martin <gzlist@googlemail.com>, Parth Malwankar, #376388)
+
+* ``bzr clean-tree`` should not delete nested bzrdirs. Required for proper
+  support of bzr-externals and scmproj plugins.
+  (Alexander Belchenko, bug #572098)
+
+* ``bzr ignore`` will no longer add duplicate patterns to .bzrignore.
+  (Gordon Tyler, #572092)
+
+* ``bzr log --exclude-common-ancestry -r X..Y`` displays the revisions that
+  are part of Y ancestry but not part of X ancestry (aka the graph
+  difference).
+  (Vincent Ladeuil, #320119)
+
+* ``bzr lp-propose`` which was switched to use production Launchpad API
+  servers a few commits ago has been reverted to use edge: there is a
+  problem with using production which isn't trivially obvious, so we've
+  filed a bug to track it, and until thats fixed will be using edge.
+  (Robert Collins, #583667)
+
+* ``bzr rm`` should not refuse to delete directories which contained a file
+  which has been moved elsewhere in the tree after the previous commit.
+  (Marius Kruger, Daniel Watkins, #129880)
+
+* ``bzr selftest --parallel=fork`` wait for its children avoiding zombies.
+  (Vincent Ladeuil, #566670)
+
+* ``bzr selftest`` should not use ui.note() since it's not unicode safe.
+  (Vincent Ladeuil, #563997)
+
+* CommitBuilder refuses to create revisions whose trees have no root.
+  (Aaron Bentley)
+
+* Do not register a SIGWINCH signal handler, instead just poll for the
+  terminal width as needed.  This avoids the "Interrupted System Call"
+  problems that occur on POSIX with all currently released versions of
+  Python.
+  (Andrew Bennetts, #583941)
+
+* Don't mention --no-strict when we just issue the warning about unclean trees.
+  (Vincent Ladeuil, #401599)
+
+* Fixed ``AssertionError`` when accessing smart servers running Bazaar
+  versions before 1.6.
+  (Andrew Bennetts, #528041)
+
+* Improved progress bar for fetch (2a format only). Bazaar now shows an
+  estimate of the number of records to be fetched vs actually fetched.
+  (Parth Malwankar, #374740, #538868)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* ``RemoteBranch.lock_write`` raises ``ReadOnlyError`` if called during a
+  read lock, rather than causing an ``AttributeError``.
+  (Andrew Bennetts, Danilo Segan, #582781)
+
+* Selftest was failing with testtools 0.9.3, which caused an
+  AssertionError raised from a cleanUp to be reported as a Failure, not an
+  Error, breaking on of our test hygiene tests.
+  (Robert Collins, Vincent Ladeuil).
+
+* ``set_user_option`` with a dict on remote branches no longer fails with
+  an AttributeError.  There is a new ``Branch.set_config_option_dict`` RPC
+  to support this efficiently.
+  (Andrew Bennetts, #430382)
+  
+* Show the filenames when a file rename fails so that the error will be
+  more comprehensible.
+  (Martin Pool, #491763)
+
+* Support Pyrex 0.9.9, required changing how we handle exceptions in Pyrex.
+  (John Arbash Meinel, #582656)
+
+* Unicode characters in aliases are now handled correctly and do not cause
+  UnicodeEncodeError exception. (Parth Malwankar, #529930)
+
+* Unicode commit messages that are the same as a file name no longer cause
+  UnicodeEncodeError. ``ui.text.show_warning`` now handles unicode
+  messages.
+  (Parth Malwankar, #563646)
+
+* When passing a file to ``UTF8DirReader`` make sure to close the current
+  directory file handle after the chdir fails. Otherwise when passing many
+  filenames into a command line ``bzr status`` we would leak descriptors.
+  (John Arbash Meinel, #583486)
+
+Improvements
+************
+
+* ``append_revisions_only`` will now be interpreted as a boolean and a
+  warning emitted if illegal values are used. Note that for projects
+  that needs to maintain compatibility with previsous bzr versions,
+  only 'True' and 'False' strings must be used (previous versions of
+  bzr will interpret all strings differing from 'True'
+  (case-sensitive) as false.
+  (Brian de Alwis, Vincent Ladeuil)
+
+* ``bzr ls`` now supports short options for existing long options.
+  ``-k/--kind``, ``-i/--ignored``, ``-u/--unknown`` and ``-0/--null``.
+  (Parth Malwankar, #181124)
+
+* ``Config.get_user_option_as_bool`` will now warn if a value cannot
+  be interpreted as a boolean.
+  (Vincent Ladeuil)
+
+* The all-in-one Windows installer will now be built with docstrings stripped
+  from the library zip, reducing the size and slightly improving cold startup
+  time. Bundled plugins are unchanged for the moment, but if adding other new
+  plugins to an all-in-one installation, ensure they are compiled and
+  installed with -O1 or help may not work. (Martin [gz])
+
+API Changes
+***********
+
+* Added ``bzrlib.merge.PerFileMerger``, a more convenient way to write
+  some kinds of ``merge_file_content`` hook functions.
+  (Andrew Bennetts)
+  
+* `BzrDir`, `Branch`, `Repository` and `WorkingTree` now all support `user_url`,
+  `user_transport`, `control_url` and `control_transport` members pointing
+  respectively to the directory containing the ``.bzr`` control directory, 
+  and to the directory within ``.bzr`` used for the particular component.
+  All of them inherit from `ControlComponent` which provides default
+  implementations.
+  (Martin Pool)
+
+* Lock methods on ``Tree``, ``Branch`` and ``Repository`` are now
+  expected to return an object which can be used to unlock them. This reduces
+  duplicate code when using cleanups. The previous 'tokens's returned by
+  ``Branch.lock_write`` and ``Repository.lock_write`` are now attributes
+  on the result of the lock_write. ``repository.RepositoryWriteLockResult``
+  and ``branch.BranchWriteLockResult`` document this. (Robert Collins)
+
+* ``Repository.refresh_data`` may now be called in a write group on
+  pack-based repositories.  Older repositories will still raise an error
+  in this case.  Subclasses of ``Repository`` can still override
+  ``Repository._refresh_data``, but are now responsible for raising
+  ``bzrlib.repository.IsInWriteGroupError`` if they do not support
+  ``refresh_data`` during a write group.
+  (Andrew Bennetts, #574236)
+
+Internals
+*********
+
+* ``chk_map._bytes_to_text_key`` is now an optimized function to extract
+  the (file-id, revision-id) key from a CHKInventory entry. This can
+  potentially shave 5-10% time off during a large fetch. Related to bug
+  #562666. (John Arbash Meinel)
+
+* ``log._get_info_for_log_files`` now takes an add_cleanup callable.
+  (Robert Collins)
+
+* ``_remember_remote_is_before`` no longer raises AssertionError when
+  suboptimal network behaviour is noticed; instead it just mutters to the
+  log file (and warns the user if they have set the ``hpss`` debug flag).
+  This was causing unnecessary aborts for performance bugs that are minor
+  at worst.
+  (Andrew Bennetts, #528041)
+
+* Permit bzr to run under ``python -OO`` which reduces the size of bytecode
+  files loaded from disk. To ensure docstrings needed for help are never
+  stripped, the prefix ``__doc__ =`` should now be used.
+  (Martin <gzlist@googlemail.com>)
+
+* No longer require zlib headers to build extensions, and remove the need
+  for seperate copy of zlib library on windows.
+  (John Arbash Meinel, Martin <gzlist@googlemail.com>, #566923)
+
+Testing
+*******
+
+* Added ``bzrlib.tests.matchers`` as a place to put matchers, along with
+  our first in-tree matcher. See the module docstring for details.
+  (Robert Collins)
+
+* ``bzr selftest --parallel=subprocess`` now works correctly on win32.
+   (Gordon Tyler, #551332)
+
+* Workaround ``Crypto.Random`` check leading to spurious test
+  failures on Lucid, FreeBSD and gentoo.  
+  (Vincent Ladeuil, #528436)
+
+* New class ``ExecutableFeature`` for checking the availability of
+  executables on the ``PATH``. Migrated from bash_completion plugin.
+  (Martin von Gagern)
+
+
+bzr 2.2b2
+#########
+
+:2.2b2: 2010-04-16
+
+This is a somewhat early second beta of the 2.2 series, to fix a python2.4
+incompatibility in the 2.2b1 release.  It also includes a swag of
+performance, usability and correctness improvements: test feedback on all
+of these would be welcome.
+
+
+New Features
+************
+
+* ``bzr diff`` now supports a --format option, which can be used to 
+  select alternative diff formats. (Jelmer Vernooij, #555994)
+
+Bug Fixes
+*********
+
+* ``bzr dpush``, ``bzr push`` and ``bzr send`` will now issue a warning
+  instead of failing when dirty trees are involved. The corresponding
+  ``dpush_strict``, ``push_strict`` and ``send_strict`` should be set to
+  True explicitly to get the previous behaviour.  
+  (Vincent Ladeuil, #519319)
+
+* ``bzr export`` to tar file does not fail if any parent directory
+  contains unicode characters. This works around upstream Python bug
+  http://bugs.python.org/issue8396 .
+  (Parth Malwankar, #413406)
+
+* ``bzr switch`` does not die if a ConfigurableFileMerger is used.
+  (Aaron Bentley, #559436)
+
+* ``bzr update`` when a pending merge in the working tree has been merged
+  into the master branch will no longer claim that old commits have become
+  pending merges. (Robert Collins, #562079)
+
+* ``bzrlib.mutabletree.MutableTree.commit`` will now support a passed in
+  config as in previous versions of bzrlib. (Robert Collins)
+
+* Fix glitch in the warning about unclean trees display.
+  (Vincent Ladeuil, #562665)
+
+* Fixed Python2.4 incompatibilities in the bzr2.2b1 source tarball.
+  (Martin Pool)
+
+* Help messages generated by ``RegistryOption.from_kwargs`` list the
+  switches in alphabetical order, rather than in an undefined order.
+  (Martin von Gagern, #559409)
+
+* Make sure the ``ExecutablePath`` and ``InterpreterPath`` are set in
+  Apport crash reports, to avoid "This problem report applies to a program
+  which is not installed any more" error.
+  (Martin Pool, James Westby, #528114)
+
+* Reset ``siginterrupt`` flag to False every time we handle a signal
+  installed with ``set_signal_handler(..., restart_syscall=True)`` (from
+  ``bzrlib.osutils``.  Reduces the likelihood of "Interrupted System Call"
+  errors compared to registering ``signal.signal`` directly.
+  (Andrew Bennetts)
+
+* When invoked with a range revision, ``bzr log`` doesn't show revisions
+  that are not part of the Y revisions ancestry anymore when invoked with
+  -rX..Y.
+  (Vincent Ladeuil, #474807)
+
+* Properly handle ``param_name`` attribute for ``ListOption``.
+  (Martin von Gagern, #387117)
+
+Improvements
+************
+
+* ``bzr commit`` will prompt before using a commit message that was
+  generated by a template and not edited by the user.
+  (Robert Collins, #530265)
+
+* ``bzr diff`` read-locks the trees and branches only once, saving about
+  10-20ms on ``bzr diff`` in a bzr.dev tree.
+  (Andrew Bennetts)
+
+* ``bzr missing`` read-locks the branches only once.
+  (Andrew Bennetts)
+  
+* ``bzr pull`` locks the branches and tree only once.
+  (Andrew Bennetts)
+  
+* Index lookups in pack repositories search recently hit pack files first.  
+  In repositories with many pack files this can greatly reduce the
+  number of files accessed, the number of bytes read, and the number of
+  read calls.  An incremental pull via plain HTTP takes half the time and
+  bytes for a moderately large repository.  (Andrew Bennetts)
+
+* Index lookups only re-order the indexes when the hit files aren't
+  already first. Reduces the cost of reordering
+  (John Arbash Meinel, #562429)
+
+* Less code is loaded at startup.  (Cold-cache start time is about 10-20%
+  less.)
+  (Martin Pool, #553017)
+
+API Changes
+***********
+
+* ``bzrlib.diff.get_trees_and_branches_to_diff`` is deprecated.  Use
+  ``get_trees_and_branches_to_diff_locked`` instead.
+  (Andrew Bennetts)
+
+* ``TreeTransform.commit`` supports the full set of commit parameters, and
+  auto-determines branch nick if not supplied.  (Aaron Bentley)
+  
+Internals
+*********
+
+* ``bzrlib.commands.Command.run_direct`` is no longer needed - the pre
+  2.1 method of calling run() to perform testing or direct use via the API
+  is now possible again. As part of this, the _operation attribute on
+  Command is now transient and only exists for the duration of ``run()``.
+  (Robert Collins)
+
+bzr 2.2b1
+#########
+
+:2.2b1: 2010-04-01
+
+This is the first beta of the 2.2 series, leading up to a 2.2.0
+release in July or August.  Beta releases are suitable for everyday use
+but may cause some incompatibilities with plugins.  Some plugins may need
+small updates to work with 2.2b1.
+
+2.2b1 includes some changes to make merge conflicts easier to understand
+and resolve.  It also removes some old unnecessary code, and loads
+somewhat less code at startup.  It starts adding a common infrastructure
+for dealing with colocated named branches, which can be implemented in
+various ways in either bzr native or foreign formats.   On Ubuntu and
+other platforms with the apport bug-reporting library, there's an easier
+path to report problems with bzr.  We plan to continue with these themes
+through the 2.2 series.
+
+Over thirty bugs have been fixed, including in the log command, exporting
+to tarballs, restarting interrupted system calls, portability of compiled
+extensions, making backups during upgrade, and locking on FTP.
+
+Compatibility Breaks
+********************
+
+* BTreeGraphIndex can now take an offset to indicate that the data starts
+  somewhere other than then beginning of the file. (John Arbash Meinel)
+
+* Deleted very old hidden commands ``versionedfile-list``,
+  ``weave-plan-merge``, ``weave-merge-text``.
+  (Martin Pool)
+
+* ``Repository.get_inventory_sha1()`` and ``Repository.get_revision_xml()`` 
+  have been removed. (Jelmer Vernooij)
+
+* ``Repository.get_revision_inventory()`` has been removed in favor of
+  ``Repository.get_inventory()``. (Jelmer Vernooij)
+
+* All test servers have been moved out of the bzrlib.transport hierarchy to
+  bzrlib.tests.test_server *except* for MemoryServer, ChrootServer and
+  PathFilteringServer. ``bzrlib`` users may encounter test failures that can
+  be fixed by updating the related imports from ``bzrlib.transport.xxx`` to
+  ``bzrlib.tests.test_server``.
+  (Vincent Ladeuil)
+
+* ``BranchReferenceFormat.initialize()`` now takes an optional name argument
+  as its second parameter, for consistency with the initialize() method of
+  other formats. (Jelmer Vernooij)
+
+New Features
+************
+
+* Added ``bzr remove-branch`` command that can remove a local or remote 
+  branch. (Jelmer Vernooij, #276295)
+
+* ``bzr export`` now takes an optional argument ``--per-file-timestamps``
+  to set file mtimes to the last timestamp of the last revision in which
+  they were changed rather than the current time. (Jelmer Vernooij)
+
+* If the Apport crash-reporting tool is available, bzr crashes are now
+  stored into the ``/var/crash`` apport spool directory, and the user is
+  invited to report them to the developers from there, either
+  automatically or by running ``apport-bug``.  No information is sent
+  without specific permission from the user.  (Martin Pool, #515052)
+
+* Parsing of command lines, for example in ``diff --using``, no longer
+  treats backslash as an escape character on Windows.
+  (Gordon Tyler, #392428)
+
+* Plugins can be disabled by defining ``BZR_DISABLE_PLUGINS`` as
+  a list of plugin names separated by ':' (';' on windows).
+  (Vincent Ladeuil, #411413)
+
+* Plugins can be loaded from arbitrary locations by defining
+  ``BZR_PLUGINS_AT`` as a list of name@path separated by ':' (';' on
+  windows). This takes precedence over ``BZR_PLUGIN_PATH`` for the
+  specified plugins. This is targeted at plugin developers for punctual
+  needs and *not* intended to replace ``BZR_PLUGIN_PATH``.  
+  (Vincent Ladeuil, #82693)
+
+* Tag names can now be determined automatically by ``automatic_tag_name`` 
+  hooks on ``Branch`` if they are not specified on the command line.
+  (Jelmer Vernooij)
+
+* Tree-shape conflicts can be resolved by providing ``--take-this`` and
+  ``--take-other`` to the ``bzr resolve`` command. Just marking the conflict
+  as resolved is still accessible via the ``--done`` default action.
+  (Vincent Ladeuil)
+
+* Merges can be proposed on Launchpad with the new lp-propose-merge command.
+  (Aaron Bentley, Jonathan Lange)
+
+Bug Fixes
+*********
+
+* Added docstring for ``Tree.iter_changes``
+  (John Arbash Meinel, #304182)
+
+* Allow additional arguments to
+  ``RemoteRepository.add_inventory_by_delta()``. (Jelmer Vernooij, #532631)
+
+* Allow exporting a single file using ``bzr export``.
+  (Michal Junák, #511987)
+
+* Allow syscalls to automatically restart when ``TextUIFactory``'s
+  SIGWINCH handler is invoked, avoiding ``EINTR`` errors during blocking
+  IO, which are often poorly handled by Python's libraries and parts of
+  bzrlib.  (Andrew Bennetts, #496813)
+
+* Avoid infinite recursion when probing for apport.
+  (Vincent Ladeuil, #516934)
+
+* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
+  (Martin Pool, #331095)
+
+* Avoid truncating svn URLs.
+  (Martin Pool, Martin von Gagern, #545185)
+
+* ``bzr add`` will not add conflict related files unless explicitly required.
+  (Vincent Ladeuil, #322767, #414589)
+
+* ``bzr dump-btree`` now works on ``*.cix`` and ``*.six`` files.  Those
+  indices do not have reference lists, so ``dump-btree`` will simply show
+  ``None`` instead.  (Andrew Bennetts, #488607)
+
+* ``bzr help`` will no longer trigger the get_missing_command hook when
+  doing a topic lookup. This avoids prompting (like 'no command plugins/loom,
+  did you mean log?') when getting help. In future we may trigger the hook 
+  deliberately when no help topics match from any help index.
+  (Robert Collins, #396261)
+
+* ``bzr log -n0 -r..A.B.C`` should not crash but just consider the None
+  revspec as representing the first revision of the branch.
+  (Vincent Ladeuil, #519862)
+
+* ``bzr remove-tree`` can now remove multiple working trees.
+  (Jared Hance, Andrew Bennetts, #253137)
+
+* ``bzr resolve --take-this`` and ``--take-other`` now correctly renames
+  the kept file on content conflicts where one side deleted the file.
+  (Vincent Ladeuil, #529968)
+
+* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
+  permissions as ``.bzr`` directory on a POSIX OS.
+  (Parth Malwankar, #262450)
+
+* ``bzr upgrade`` now names backup directory as ``backup.bzr.~N~`` instead
+  of ``backup.bzr``. This directory is ignored by bzr commands such as
+  ``add``.
+  (Parth Malwankar, #335033, #300001)
+
+* Cope with non-utf8 characters inside ``.bzrignore``.
+  (Jason Spashett, #183504)
+
+* Correctly interpret "451 Rename/move failure: Directory not empty" from
+  FTP servers while trying to take a lock.
+  (Martin Pool, #528722)
+
+* DirStateRevisionTree.kind() was returning wrong result when 'kind'
+  changes occured between the workingtree and one of its parents.
+  (Vincent Ladeuil, #535547)
+
+* Fix ``log`` to better check ancestors even if merged revisions are involved.
+  (Vincent Ladeuil, #476293)
+
+* Loading a plugin from a given path with ``BZR_PLUGINS_AT`` doesn't depend
+  on os.lisdir() order and is now reliable.
+  (Vincent Ladeuil, #552922).
+
+* Many IO operations that returned ``EINTR`` were retried even if it
+  wasn't safe to do so via careless use of ``until_no_eintr``.  Bazaar now
+  only retries operations that are safe to retry, and in some cases has
+  switched to operations that can be retried (e.g. ``sock.send`` rather than
+  ``sock.sendall``).
+  (Andrew Bennetts, Martin <gzlist@googlemail.com>, #496813)
+
+* Path conflicts now support --take-this and --take-other even when a
+  deletion is involved.
+  (Vincent Ladeuil, #531967)
+
+* Network transfer amounts and rates are now displayed in SI units according
+  to the Ubuntu Units Policy <https://wiki.ubuntu.com/UnitsPolicy>.
+  (Gordon Tyler, #514399)
+
+* Support kind markers for socket and fifo filesystem objects. This
+  prevents ``bzr status --short`` from crashing when those files are
+  present.  (John Arbash Meinel, #303275)
+
+* ``bzr mkdir DIR`` will not create DIR unless DIR's parent is a versioned
+   directory. (Parth Malwankar, #138600)
+
+* SSH child processes will now ignore SIGQUIT on nix systems so breaking into
+  the debugger won't kill the session.
+  (Martin <gzlist@googlemail.com>, #162502)
+
+* Tolerate patches with leading noise in ``bzr-handle-patch``.
+  (Toshio Kuratomi, Martin Pool, #502076)
+
+* ``update -r`` now supports updating to revisions that are not on
+  mainline (i.e. it supports dotted revisions).
+  (Parth Malwankar, #517800)
+
+* Use first apparent author not committer in GNU Changelog format.
+  (Martin von Gagern, #513322)
+
+API Changes
+***********
+
+* ``bzrlib.merge_directive._BaseMergeDirective`` has been renamed to 
+  ``bzrlib.merge_directive.BaseMergeDirective`` and is now public.
+  (Jelmer Vernooij)
+
+* ``BranchFormat.initialize`` now takes an optional ``name`` of the colocated 
+  branch to create. (Jelmer Vernooij)
+
+* ``BzrDir.get_branch_transport`` now takes an optional ``name`` of the 
+  colocated branch to open. (Jelmer Vernooij)
+
+* Added ``bzrlib.osutils.set_signal_handler``, a convenience function that
+  can set a signal handler and call ``signal.siginterrupt(signum,
+  False)`` for it, if the platform and Python version supports it.
+  (Andrew Bennetts, #496813)
+
+* New ``bzrlib.initialize`` is recommended for programs using bzrlib to 
+  run when starting up; it sets up several things that previously needed
+  to be done separately.
+  (Martin Pool, #507710)
+
+* Exporters now support a ``per_file_timestamps`` argument to write out the 
+  timestamp of the commit in which a file revision was introduced.
+  (Jelmer Vernooij)
+
+* New method ``BzrDir.list_branches()`` that returns a sequence of branches 
+  present in a control directory. (Jelmer Vernooij)
+
+* New method ``Repository.get_known_graph_ancestry()``. 
+  (Jelmer Vernooij, #495502)
+
+* New transport methods ``readlink``, ``symlink`` and ``hardlink``.
+  (Neil Santos)
+
+* Remove unused ``CommandFailed`` exception.
+  (Martin Pool)
+
+Internals
+*********
+
+* ``bzrlib.branchbuilder.BranchBuilder.build_snapshot`` now accepts a
+  ``message_callback`` in the same way that commit does. (Robert Collins)
+
+* ``bzrlib.builtins.Commit.run`` raises ``bzrlib.errors.BoundBranchOutOfDate``
+  rather than ``bzrlib.errors.BzrCommandError`` when the bound branch is out
+  of date. (Gary van der Merwe)
+
+* ``bzrlib.commands.run_bzr`` is more extensible: callers can supply the
+  functions to load or disable plugins if they wish to use a different
+  plugin mechanism; the --help, --version and no-command name code paths
+  now use the generic pluggable command lookup infrastructure.
+  (Robert Collins)
+
+* ``bzrlib.errors.BoundBranchOutOfDate`` has a new field ``extra_help``
+  which can be set to add extra help to the error. (Gary van der Merwe)
+
+* New method ``Branch.automatic_tag_name`` that can be used to find the
+  tag name for a particular revision automatically. (Jelmer Vernooij)
+
+* The methods ``BzrDir.create_branch()``, ``BzrDir.destroy_branch()`` and 
+  ``BzrDir.open_branch()`` now take an optional ``name`` argument. 
+  (Jelmer Vernooij)
+
+Testing
+*******
+
+* bzr now has a ``.testr.conf`` file in its source tree configured
+  appropriately for running tests with Testrepository
+  (``https://launchpad.net/testrepository``). (Robert Collins)
+
+* Documentation about testing with ``subunit`` has been tweaked.
+  (Robert Collins)
+
+* Known failures has been added for resolve --take-other on ParentLoop
+  conflicts. This reflects bug #537956 without fixing it.
+  (Vincent Ladeuil)
+
+* New ``bzrlib.tests.test_import_tariff`` can make assertions about what
+  Python modules are loaded, to guard against startup time or library
+  dependency regressions.
+  (Martin Pool)
+
+* PQM will now run with subunit output. To analyze a PQM error use
+  tribunal, or cat log | subunit-filter | subunit2pyunit. (Robert Collins)
+
+* Stop sending apport crash files to ``.cache`` in the directory from
+  which ``bzr selftest`` was run.  (Martin Pool, #422350)
+
+* Tests no longer fail if "close() called during concurrent
+  operation on the same file object" occurs when closing the log file
+  (which can happen if a thread tries to write to the log file at the
+  wrong moment).  An warning will be written to ``stderr`` when this
+  happens, and another warning will be written if the log file could not
+  be closed after retrying 100 times.  (Andrew Bennetts, #531746)
+
+..
+   vim: tw=74 ft=rst ff=unix

=== renamed file 'NEWS' => 'doc/en/release-notes/bzr-2.3.txt'
--- NEWS	2010-07-29 11:17:57 +0000
+++ doc/en/release-notes/bzr-2.3.txt	2011-09-09 13:30:12 +0000
@@ -2,72 +2,927 @@
 Bazaar Release Notes
 ####################
 
-.. contents:: List of Releases
-   :depth: 1
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.3.5
+#########
+
+:2.3.5: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Cope cleanly with buggy HTTP proxies that close the socket in the middle
+  of a multipart response.  (Martin Pool, #198646).
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+
+bzr 2.3.4
+#########
+
+:Codename: One and counting
+:2.3.4: 2011-07-14
+
+This is a bugfix release. Upgrading is recommended for all users of earlier
+2.3 releases.
+
+This mainly fixes bug #786980 which blocked the SRU process for Ubuntu Natty.
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* Tweak an RPC implementation for ``Repository.get_parent_map``, it was
+  doing an inefficient ``small_set.difference_update(large_set)`` when we
+  can do ``small_set = small_set.difference(large_set)``. This speeds up
+  discovery time by about 10%. (John Arbash Meinel)
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Accept some differences for ``bound_location`` from the config files that
+  were leading to a 'ReadOnlyError: A write attempt was made in a read only
+  transaction' error.  (Vincent Ladeuil, #786980)
+
+* Don't fail with traceback if `bzr serve` is running as a service on Windows,
+  and there is no USERNAME, nor BZR_EMAIL or other whoami-related environment
+  variables set. (Alexander Belchenko, Bug #660174)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+* Updated the "Using stacked branches" section of the user guide to
+  describe committing to stacked branches and expanded its discussion of
+  pushing a stacked branch.  (Andrew Bennetts)
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* Remove the deprecation decorators for ``failUnlessExists`` and
+  ``failIfExists``. The deprecation "will" occur in 2.4, not
+  before. Providing the wrappers is enough as far as 2.3 is concerned.
+  (Vincent Ladeuil #794960)
+
+bzr 2.3.3
+#########
+
+:2.3.3: 2011-05-13
+
+This is a bugfix release. Upgrading is recommended for all users of earlier
+2.3 releases.
+
+This fixed a bug in the test suite triggered by python-2.7 deprecating some
+tests helpers.
+
+Testing
+*******
+
+* Stop using `failIf`, `failUnless`, `failIfEqual`, etc, that give
+  `PendingDeprecationWarnings` on Python2.7. 
+  (Martin Pool, #760435)
+
+
+bzr 2.3.2
+#########
+
+:2.3.2: 2011-05-12
+
+This is a bugfix release. Upgrading is recommended for all users of earlier
+2.3 releases.
+
+This was never released due to bug #760435 interrupting the release process by
+breaking the test suite under python-2.7 on natty.
+
+External Compatibility Breaks
+*****************************
+
+None
+
+New Features
+************
+
+None
+
+Improvements
+************
+
+* Getting all entries from ``CHKInventory.iter_entries_by_dir()`` has been
+  sped up dramatically for large trees. Iterating by dir is not the best
+  way to load data from a CHK inventory, so it preloads all the items in
+  the correct order. (With the gcc-tree, this changes it (re)reading 8GB
+  of CHK data, down to just 150MB.) This has noticeable affects for things
+  like building checkouts, etc.  (John Arbash Meinel, #737234)
+
+Bug Fixes
+*********
+
+* Bazaar now infers the default user email address on Unix from the local
+  account name plus the contents of ``/etc/mailname`` if that file exists.
+  In particular, this means that committing as root through etckeeper will
+  normally not require running ``bzr whoami`` first.
+  (Martin Pool, #616878)
+
+* ``bzr merge --preview --pull`` should respect the ``--preview`` option
+  first, and not actually change the branch tip revision.
+  (John Arbash Meinel, Dennis Duchier, #760152)
+
+* ``bzr push`` into a repository (that doesn't have a branch), will no
+  longer copy all revisions in the repository. Only the ones in the
+  ancestry of the source branch, like it does in all other cases.
+  (John Arbash Meinel, #465517)
+
+* Fix ``UnboundLocalError: local variable 'lock_url' in wait_lock`` error,
+  especially while trying to save configuration from QBzr.
+  (Martin Pool, #733136)
+
+* Fix "Unable to obtain lock" error when pushing to a bound branch if tags
+  had changed.  Bazaar was attempting to open and lock the master branch
+  twice in this case.  (Andrew Bennetts, #733350)
+
+* Standalone bzr.exe installation on Windows: user can put additional python 
+  libraries into ``site-packages`` subdirectory of the installation directory,
+  this might be required for "installing" extra dependencies for some plugins.
+  (Alexander Belchenko, #743256)
+
+* When reporting a crash without apport, don't print the full list of
+  plugins because it's often too long.
+  (Martin Pool, #716389)
+
+
+API Changes
+***********
+
+None.
+
+Testing
+*******
+
+* FreeBSD8 has switched to python-2.7 which revealed a re-occurrence of a test
+  failure in the launchpad plugin. ``xmlrpclib.py`` on natty carries a patch
+  that is not in python-2.7 upstream and masked the issue. An additional fix
+  has been added in the interim
+  (<http://psf.upfronthosting.co.za/roundup/tracker/issue8194> should be fixed
+  in python > 2.7.1).  (Vincent Ladeuil, #654733)
+
+bzr 2.3.1
+#########
+
+:2.3.1: 2011-03-10
+
+This is a bugfix release. Upgrading is recommended for all users of earlier
+2.3 releases.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Correctly resolve text conflicts for files in subdirs.
+  (Vincent Ladeuil, #715058)
+
+* Fix "AssertionError: repository.user_url ... does not match URL from
+  server response" when reusing a smart transport.
+  (Andrew Bennetts, #726584)
+
+* Restore proper logging of bytes transferred. We accidentally reset the
+  counter when commands finished before we logged the total transferred.
+  (John Arbash Meinel, #713258)
+
+bzr 2.3.0
+#########
+
+:2.3.0: 2011-02-03
+
+This release marks the start of another long-term-stable series. From here, we
+will only make bugfix releases on the 2.3 series (2.3.1, etc, and support it
+until August 2012), while 2.4 will become our new development series. The 2.1
+and 2.2 series will also continue to get bugfixes. (Currently 2.0 is planned
+to be EOLed circa September 2011 and will receive only critical bugfixes.)
+
+This is a bugfix and polish release over the 2.2 series, with a large number
+of bugs fixed (>130), and some performance improvements. Some features have
+been enhanced including commits on stacked branches, upgrades of related
+branches, shortcut URL schemes for ubuntu and debian on launchpad and better
+conflict resolution.
+
+Only bugfixes from other stables series have been included since 2.3b5 so all
+known fixed bugs are included here.
+
+Users are encouraged to upgrade from the other stable series.
+
+bzr 2.3b5
+#########
+
+:2.3.b5: 2011-01-13
+
+This is the fifth and **last** beta of the 2.3 series, leading up to a 2.3.0
+release in February. Beta releases are suitable for everyday use but may cause
+some incompatibilities with plugins.
+
+2.3b5 includes bug fixes for committing to stacked branches, smoother upgrades
+of multiple branches, compatibility with python-2.7, full test suite passing
+on Ubuntu Natty and windows, less round-trips for several smart server
+operations, better support text conflicts resolve actions and some more.
+
+All known fixed bugs in other series (2.0, 2.1, 2.2) are also included here.
+
+External Compatibility Breaks
+*****************************
+
+(none)
+
+Improvements
+************
+
+* A redundant parent inventories calculation was removed from
+  ``fetch.py``, as ``Repository.insert_stream`` already reports any
+  missing inventories.  This removes at least one network roundtrip when
+  pushing to a stacked branch.  (Andrew Bennetts)
+
+* ``ControlDir.sprout`` no longer opens the target repository more than
+  once.  This avoids some unnecessary IO, and removes a network roundtrip
+  when doing ``bzr branch`` to a smart server URL.  (Andrew Bennetts)
+
+* ``bzr modified`` now read-locks the working tree (and branch and
+  repository) just once.  (Andrew Bennetts)
+  
+* ``bzr resolve`` now accepts ``--take-this`` and ``--take-other`` actions
+  for text conflicts. This *replace* the whole file with the content
+  designated by the action. This will *ignore* all differences that would
+  have been merge cleanly otherwise. (Vincent Ladeuil, #638451)
+
+* ``bzr tags``'s "sort" argument now allows registering custom sort
+  methods using the ``bzrlib.tag.tag_sort_methods`` registry. 
+  (Jelmer Vernooij, #701244)
+
+* ``bt.test_http`` was breaking ``os.environ`` by erasing the values saved by
+  ``TestCase`` leading to ``bt.test_import_tariff`` failures.
+  (Vincent Ladeuil, #690563)
+
+* ``upgrade`` now upgrades dependent branches when a shared repository is
+  specified. It also supports new options: ``--dry-run`` for showing what
+  will happen and ``--clean`` to remove the backup directory on successful
+  completion. (Ian Clatworthy, Matthew Fuller, #89830, #374734, #422450)
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Avoid leaking SSH subprocess communication socket into unrelated child
+  processes, which could cause bzr to hang on exit. (Max Bowsher, #696285)
+
+* ``bzr break-lock`` on a corrupted lock file works correctly, rather than
+  raising a PermissionDenied error. We were accidentally holding open the
+  file we were trying to delete. (John Arbash Meinel, #659978)
+
+* ``bzr update`` in a checkout of a readonly branch works again, without
+  trying to set the tags in the master branch. This had been broken by the
+  bug fix for bug #603395.  (John Arbash Meinel, #701212)
+
+* Per-transport tests now prefer to use ``Transport.get_bytes()`` rather
+  than ``Transport.get().read()``. The SFTP code uses an async message to
+  close the file handle if you let the handle die from refcounting, while
+  it uses a synchronous message if you close it directly. This should help
+  prevent random test suite failures from race conditions.
+  (John Arbash Meinel, #681047)
+
+* Stop using ``bzrlib.tuned_gzip.GzipFile``. It is incompatible with
+  python-2.7 and was only used for Knit format repositories, which haven't
+  been recommended since 2007. The file itself will be removed in the next
+  release. (John Arbash Meinel)
+
+* The BZR_COLUMNS environment variable can be set to 0 to indicate no
+  limitation on the width of the terminal.  (Neil Martinsen-Burrell, #675652)
+
+* Treat WSAECONNABORTED the same as WSAECONNRESET for the purposes of
+  considering a smart data stream as being interrupted. This fixes a
+  failure in the windows test suite, that was trying to ensure we cleanly
+  handled a server disconnect.  (John Arbash Meinel, #581311, #686587)
+
+* Unshelving changes that occur in a now-unversioned directory now restore
+  the directory properly rather than crashing.
+  (John Arbash Meinel, #389674)
+
+* You are now able to commit directly to a stacked branch. Any needed
+  parent inventories will be filled in as part of the commit process.
+  (John Arbash Meinel, #375013)
+
+Documentation
+*************
+
+* Better document the rules to update the bzr freshmeat page when
+  doing a release. (Vincent Ladeuil, #690515)
+
+API Changes
+***********
+
+* ``Branch.sprout``, ``BranchFormat.initalize`` and
+  ``ControlDir.create_branch`` now take an optional ``repository`` keyword
+  argument, and ``BranchFormat.open`` now takes an optional
+  ``found_repository`` keyword argument.  These provide the repository
+  object for new branch object to use (for cases when the caller has
+  already opened that repository).  Implementations of these APIs will
+  need to be updated to accept these arguments.  (Andrew Bennetts)
+
+* ``bzrlib.tuned_gzip.GzipFile`` is now deprecated and will be removed in
+  the bzr-2.4 series. Code that was using it can just use the python
+  stdlib ``gzip.GzipFile``. (John Arbash Meinel)
+
+
+Testing
+*******
+
+* ``bzrlib.tests`` defines ``isolated_environ`` with the definitions of all
+  the environment variables the tests should care about. It also defines
+  ``override_os_environ`` and ``restore_os_environ`` to properly implement
+  isolation from ``os.environ`` for tests. ``bzrlib.tests`` now defines a
+  ``DocTestSuite`` class using this facility for all ``bzrlib``
+  doctests. (Vincent Ladeuil, #321320)
+
+* Catch exceptions related to bug #637821 during test cleanup to avoid
+  spurious failures. (Vincent Ladeuil, #686008).
+
+* Check sphinx compatibility for tests requiring older sphinx versions.
+  (Vincent Ladeuil, #688072)
+
+* ``test_onto_transport`` in the Launchpad plugin can now run with Python
+  2.7. (Vincent Ladeuil, #654733)
+
+* ``TestCase._captureVar`` and ``TestCase._old_env`` have been deleted due to
+  bug #690563. Test writers are encouraged to use ``TestCase.overrideEnv``
+  instead. (Vincent Ladeuil)
+
+* ``TestDebuntuExpansions`` was escaping the test isolation by calling the
+  wrong base class ``setUp``. (Vincent Ladeuil, #684662)
+
+bzr 2.3b4
+#########
+
+:2.3.b4: 2010-12-03
+
+This is the fourth beta of the 2.3 series, leading up to a 2.3.0 release in
+February. Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.
+
+2.3b4 includes bug fixes for the ``config`` command and conflict
+resolution. More changes were made for the test scripts handling to make it
+easier to add reproducing recipes to bugs.
+
+It also includes bug fixes from the 2.2.2 release as well as the bug fixes
+in the upcoming 2.0.7, 2.1.4 and 2.2.3 releases. This means that all known
+fixed bugs at the time of this release are included.
+
+
+External Compatibility Breaks
+*****************************
+
+  (none)
+
+Improvements
+************
+
+* Bazaar now caches a branch's tags while that branch is read-locked.
+  This removes 1 network roundtrip from most interactions with a remote
+  branch.  (Andrew Bennetts)
+
+* ``bzr config <option>`` will now display only the value itself so scripts
+  can use it to query the currently active configuration. Displaying several
+  options matching a given regular expression is now controlled via the
+  ``--all`` option.  (Vincent Ladeuil, bug #670251)
+
+* ``bzr resolve`` now reports the number of conflicts resolved and the
+  number of remaining conflicts. This provides a better feedback about the
+  whole resolution process. (Vincent Ladeuil)
+
+* Read configuration files in $XDG_CONFIG_HOME/bazaar on Unix if there is
+  already a directory there.  (Neil Martinsen-Burrell, #195397)
+
+Bug Fixes
+*********
+
+* Better message if there is an error while setting ownership of
+  ``.bazaar`` directory. (Parth Malwankar, #657553)
+
+* ``bzr config`` properly displays list values. (Vincent Ladeuil, #672382)
+
+* ``bzr config`` will now respect option policies when displaying the value
+  and display the definition sections when appropriate.
+  (Vincent Ladeuil, #671050)
+
+* Don't create commit message files in the current directory to avoid nasty
+  interactions with emacs (which tries to establish the status of the file
+  during the commit which breaks on windows). (Vincent Ladeuil, #673637)
+
+* ``bzr resolve --take-other <file>`` will not crash anymore if ``<file>``
+  is involved in a text conflict (but the conflict is still not
+  resolved). (Vincent Ladeuil, #646961)
+
+* Merge will now correctly locate a lca where there is a criss-cross merge
+  of a new root. (Gary van der Merwe, #588698)
+
+* Report error if non-ASCII command option given. (Rory Yorke, #140563)
+
+* ``tools/check-newsbug.py`` is now based on ``lp:hydrazine`` and no longer
+  crashes when encountering private bugs (they are just displayed as such).
+  (Vincent Ladeuil, #354985)
+
+Internals
+*********
+
+* ``BranchBuilder.build_snapshot`` now accepts parent_ids == [].
+  This can be used to create a new root in the graph. (Gary van der Merwe)
+
+* Old repository development formats
+  RepositoryFormatCHK1 and RepositoryFormatCHK2 have been removed, and so
+  have the corresponding metadir format options ``development-rich-root``,
+  ``development6-rich-root``, and ``development7-rich-root``.
+
+Testing
+*******
+
+* Add a null_output_matches_anything keyword argument with default False to
+  bzrlib.tests.script.ScriptRunner.run_script to specify that the command
+  output should not be checked (as opposed to expecting an empty output).
+  (Neil Martinsen-Burrell, #662509)
+
+* Blank output section in scriptrunner tests no longer match any output.
+  Instead, use '...' as a wildcard if you don't care about the output.
+  (Martin Pool, #637830)
+
+* Bump minimum testtools version required to run ``bzr selftest`` from 0.9.2
+  to 0.9.5 which will allow tests that need the fixed unicode handling to be
+  written. (Martin [gz])
+
+* Introduce an ``overrideEnv()`` helper for tests that needs to change the
+  environment variables while respecting the isolation rules. Get rid of
+  TestCase._restoreEnvironment which is now useless.
+  (Vincent Ladeuil, #690563)
+
+* Printing selftest results to a non-UTF-8 console will now escape characters
+  that can't be encoded rather than aborting the test run with an exception.
+  (Martin [gz], #633216)
+
+
+bzr 2.3b3
+#########
+
+:2.3b3: 2010-11-05
+
+External Compatibility Breaks
+*****************************
+
+(none)
+
+New Features
+************
+
+* Add --no-tree option to 'bzr push' and 'bzr init' for creating a
+  new or mirrored branch without working trees.
+  (Matthew Gordon, #506730)
+
+* ``bzr config`` is a new command that displays the configuration options for
+  a given directory. It accepts a glob to match against multiple options at
+  once. It can also be used to set or delete a configuration option in any
+  configuration file. (Vincent Ladeuil)
+
+* New shortcut URL schemes ``ubuntu:`` and ``debianlp:`` access source
+  branches on Launchpad.  E.g. ``bzr branch ubuntu:foo`` gives you the source
+  branch for project ``foo`` in the current distroseries for Ubuntu while
+  ``bzr branch debianlp:lenny/foo`` gives you the source branch (on Launchpad)
+  for project ``foo`` in Debian Lenny.
+  (Barry Warsaw, #609186)
+
+* Provide a configuration option "default_format" that controls the
+  default format for new branches created with ``bzr init``.
+  (Neil Martinsen-Burrell, #484101)
+
+Bug Fixes
+*********
+
+* Always set PATH in start_bzr.bat on Windows. (Matthäus G. Chajdas, #470264)
+
+* ``bzr status -r X..Y`` was failing because RevisionTree didn't implement
+  ``get_shelf_manager``. (John Arbash Meinel, #662053)
+
+* Correctly add directory contents when the name was previously added as a
+  normal file, rather than throwing ``AttributeError: children`` during
+  smart_add. (Martin [gz], #251864)
+
+* Correctly handle the ``--directory`` option for all code paths of
+  ``resolve`` and ``shelve``, this was previously ignored when paths were
+  provided as parameters. When both are provided, ``--directory`` becomes
+  the base directory for the other paths. (Vincent Ladeuil, #670851)
+
+* Correctly set the Content-Type  header when HTTP POSTing to comply
+  with stricter web frameworks. (Vincent Ladeuil, #665100)
+
+* Don't force openssh to use protocol=2, since that is now the default.
+  (Neil Martinsen-Burrell, #561061)
+
+* Fix ``KeyError: 'port'`` when getting the stored password for an HTTP  URL.
+  (Martin Pool, #654684)
+
+* Make ``bzr tag --quiet`` really quiet. (Neil Martinsen-Burrell, #239523)
+
+* Missing files (files bzr add'ed and then OS deleted) are now shown in ``bzr
+  status`` output. (Rory Yorke, #134168)
+
+* ``NotBranchError`` no longer allows errors from calling
+  ``bzrdir.open_repository()`` to propagate.  This is unhelpful at best,
+  and at worst can trigger infinite loops in callers.  (Andrew Bennetts)
+  
+* The ``branch.tags.merge_to(target_branch)`` API used by plugins such as
+  ``bzr-builddeb`` now propagates changes to the master branch of the
+  target branch (if there is one).  This makes it consistent with the
+  other tag APIs.  (Andrew Bennetts, #603395)
+
+* Windows installers no longer requires the Microsoft vcredist to be
+  installed. (Martin [gz], Gary van der Merwe, #632465)
+
+Documentation
+*************
+
+* Add documentation of the ability to edit hunks when shelving.
+  (Neil Martinsen-Burrell, #517660)
+
+* Be more specific about the meaning of revision ranges for ``bzr diff``.
+  (Neil Martinsen-Burrell, #247282)
+
+* Document the comment character in the .bzrignore file, including a
+  workaround for ignore patterns that begin with #.
+  (Neil Martinsen-Burrell, #631515)
+
+API Changes
+***********
+
+* Add ``bzrlib.pyutils`` module with helper functions for some Python
+  tasks such as resolving a dotted name to a Python object
+  (``get_named_object``).  (Andrew Bennetts)
+
+* ``bzrlib.tests.ForwardingResult`` no longer exists.  Use
+  ``testtools.ExtendedToOriginalDecorator`` instead.  (Andrew Bennetts)
+
+* ``known_hooks_key_to_parent_and_attribute`` in ``bzrlib.hooks`` has been
+  deprecated in favour of ``known_hooks.key_to_parent_and_attribute`` in
+  the same module.  (Andrew Bennetts)
+
+Internals
+*********
+
+* ``tools/fixed-in.py`` find a bug in NEWS from its number or a regexp
+  matching the news entry and display the corresponding release, date, fix
+  authors and the news entry itself. (Vincent Ladeuil)
+
+Testing
+*******
+
+* Blank output section in scriptrunner tests no longer match any output.
+  Instead, use '...' as a wildcard if you don't care about the output.
+  (Martin Pool, #637830)
+
+* ``bzr test-script script`` is a new command that runs a shell-like script
+  from an the ``script`` file. (Vincent Ladeuil)
+
+* Fix spurious test failures on babune related to the http pipe cleanup and
+  get rid of some 'bytes left on the HTTP socket' useless log messages.
+  (Vincent Ladeuil, #655557)
+
+* ``bzrlib.tests.per_workingtree.TestCaseWithWorkingTree.make_branch_builder``
+  respects its ``relpath`` parameter. (Vincent Ladeuil)
+
+bzr 2.3b2
+#########
+
+:2.3b2: 2010-10-08
+
+External Compatibility Breaks
+*****************************
+
+* The ``bzr tags`` command sorts tag names using a natural sort by
+  default (so tag2 sorts before tag10).  The previous default was
+  strictly "asciibetical".  That behavior is still available as ``bzr tags
+  --sort=alpha``. (Neil Martinsen-Burrell, #640760)
+
+* ``BzrDir.generate_backup_name`` has been deprecated and replaced by a
+  private method. ``osutils.available_backup_name`` provides an extensible
+  replacement. This allowed the deprecation of
+  ``bzrlib.transform.get_backup_name``,
+  ``bzrlib.transform._get_backup_name`` and
+  ``bzrlib.transform.TreeTransformBase.has_named_child``.
+  (Vincent Ladeuil)
+
+New Features
+************
+
+* Add ``mainline`` revision specifier, which selects the revision that
+  merged a specified revision into the mainline.  (Aaron Bentley)
+
+* Add ``annotate`` revision specifier, which selects the revision that
+  introduced a specified line of a file.  (Aaron Bentley)
+
+* Add ``-Dmem_dump`` debug flag, which uses meliae to dump memory to 
+  a file upon an out of memory error.
+  (Karl Bielefeldt, #551391)
+
+* ``bzr status`` now displays a summary of existing shelves after
+  the other status information. This is done using a ``post_status``
+  hook.
+  (Parth Malwankar, #403687)
+
+* GNU lsh is now a supported lsh client; just set BZR_SSH to 'lsh'.
+  Also, bzr will recognize if the 'ssh' comand is a symlink to lsh.
+  (Matthew Gordon, #374700)
+
+* The ``pull`` and ``update`` commands now take a ``--show-base``
+  option that, in the case of conflicts, shows the base revision text.
+  (Rory Yorke,  #202374)
+
+Improvements
+************
+
+* ``bzr break-lock --force`` breaks the lock without prompting.  (Before
+  using this, make sure the process holding the lock really is dead.)
+  (Martin Pool, #397315)
+
+* ``bzr remove`` now takes a ``--no-backup`` option for when you don't want it
+  to backup anything, just delete it. This option used to be called ``--force``
+  which is now deprecated.
+  (Marius Kruger, #400554)
+
+* When using the pycurl client module, Bazaar shows some of the text from
+  HTTP server error messages.
+  (Martin Pool, #656667)
+
+Bug Fixes
+*********
+
+* Don't force openssh to use protocol=2, since that is now the default.
+  (Neil Martinsen-Burrell, #561061)
+
+* Fix signature of RemoteBzrDir.create_workingtree to match that of its
+  superclass.  (Neil Martinsen-Burrell, Martin Pool, #524627)
+
+* Fix traceback with python-2.7's xmlrpclib
+  (Toshio Kuratomi, #612096)
+
+* Print junk rather than throwing a UnicodeDecodeError from ``bzr version-info``
+  when using the rio format (with non-ascii information) on a non-utf-8
+  terminal.  (Andrej A Antonov, #518609)
+
+* Treat all IO, OS, and socket errors consistently when establishing
+  SSH/SFTP connections via a subprocess.  (Andrew Bennetts)
+
+* ``unshelve --preview`` now can show diff in a non-ascii encoding.
+  (Andrej A Antonov, #518916)
+
+* Deleting a versioned directory can leave orphans: unversioned files that
+  were present don't have a parent anymore. The
+  ``bzr.transform.orphan_policy`` configuration option controls the ``bzr``
+  behaviour: ``conflict`` (the default) leave the orphans in place and
+  create a conflict for the directory, ``move`` create orphans named
+  ``<file>.~#~`` in a ``bzr-orphans`` directory at the root of the working
+  tree.  (Vincent Ladeuil, #323111)
+
+Improvements
+************
+
+Documentation
+*************
+
+* Correct the documentation for setting up the smart server with Apache.
+  (Neil Martinsen-Burrell, #311518)
+
+* Fix rst typos in bzrlib/help_topics/en/conflict-types.txt.
+  (Vincent Ladeuil, #660943)
+
+* Provide more detailed help on the Launchpad plugin at "bzr help
+  plugins/launchpad".  (Neil Martinsen-Burrell, #589379)
+
+* Suggest ``bzr revert`` for restoring locally deleted files in help text
+  for ``bzr update``.  (John C Barstow, #191466)
+
+API Changes
+***********
+
+* ``WorkingTree`` methods ``pull``, ``update``, and ``_update_tree``
+  now have an optional argument, ``show_base``, which is by default
+  False.  This is flag is ultimately passed to ``merge.merge_inner``
+  in each case. (Rory Yorke, #202374)
+
+Internals
+*********
+
+* Small change to GroupCompressBlock to work more in terms of 'chunks'
+  rather than 'content' for its compressed storage. (John Arbash Meinel)
+
+* When running ``bzr selftest --subunit`` the subunit stream will no
+  longer include the "log" information for tests which are considered to
+  be 'successes' (success, xfail, skip, etc) (John Arbash Meinel)
+
+Testing
+*******
+
+* Add a new simpler way to generate multiple test variations, by setting
+  the `scenarios` attribute of a test class to a list of scenarios
+  descriptions, then using `load_tests_apply_scenarios`.  (See the testing
+  guide and `bzrlib.tests.scenarios`.) Simplify `test_http` using this.
+  (Martin Pool, #597791)
+
+* Add ``tests/ssl_certs/ca.crt`` to the required test files list. Test
+  involving the pycurl https test server fail otherwise when running
+  selftest from an installed version. (Vincent Ladeuil, #651706)
+
+* Fix tests that failed when run under ``LANG=C``.
+  (Andrew Bennetts, #632387)
+
+* Skip tests that needs a bzr source tree when there isn't one. This is
+  needed to successfully run the test suite for installed versions.
+  (Vincent Ladeuil, #644855).
+
+* Skip the tests that requires respecting the chmod bits when running as root.
+  (Vincent Ladeuil, #646133)
+
+* Suppress the "maximum recursion depth exceeded in __subclasscheck__"
+  warning on stderr emitted during ``test_dict_deepnested`` in
+  ``bzrlib/tests/test__bencode.py``.  (Andrew Bennetts)
+
+* Use tests.TestCaseInTempDir for tests that requires disk resources.
+  (Vincent Ladeuil, #650001)
 
 bzr 2.3b1
 #########
 
-:Codename: ???
-:2.3b1: NOT RELEASED YET
-
-Compatibility Breaks
-********************
-
-* BzrError subclasses no longer support the name "message" to be used
-  as an argument for __init__ or in _fmt format specification as this
-  breaks in some Python versions. errors.LockError.__init__ argument
-  is now named "msg" instead of earlier "message".
-  (Parth Malwankar, #603461)
-
-* `FileInWrongBranch` is deprecated in favour of `PathNotChild` and no
-  longer raised.
-  (Martin Pool)
-
-* `tree_files` and `internal_tree_files` are now deprecated in favor of
-  `WorkingTree.open_containing_paths`.
-  (Martin Pool)
-
-* The old ``bzr selftest --benchmark`` option has been removed.
-  <https://launchpad.net/bzr-usertest> is an actively-maintained
-  macrobenchmark suite.
-  (Martin Pool)
+:2.3b1: 2010-09-20
+
+This is the first beta of the 2.3 series, leading up to a 2.3.0
+release in January or February.  Beta releases are suitable for everyday use
+but may cause some incompatibilities with plugins.  Some plugins may need
+small updates to work with 2.3b1.
+
+2.3b1 includes some performance improvements in both speed and memory
+consumption, some preliminary support for generating a texinfo version of
+the doc and better support for launchpad.  Many changes were made to make
+our test suite more robust as well as numerous documentation fixes.  It
+improves the common infrastructure for dealing with colocated named
+branches and foreign branches.  We plan to continue with these themes
+through the 2.3 series.
+
+It also includes bug fixes for 2.0.6, 2.1.3 and 2.2.1 and over 40 fixes of
+its own.
+
+
+External Compatibility Breaks
+*****************************
+
+(none)
 
 New Features
 ************
 
+* Added ``pre_status`` and ``post_status`` hooks. This allows plugins
+  to register custom handlers which will be invoked before/after the
+  standard status output is displayed. (Parth Malwankar)
+
+* ``bzr break-lock --config [location]`` can now break config files
+  locks. (Vincent Ladeuil, #525571)
+
+* ``bzrlib.config.LockableConfig`` is a base class for config files that
+  needs to be protected against multiple writers. All methods that
+  change a configuration variable value must be decorated with
+  @needs_write_lock (set_option() for example).
+  (Vincent Ladeuil,  #525571)
+
 * The ``lp:`` prefix will now use your known username (from
   ``bzr launchpad-login``) to expand ``~`` to your username.  For example:
   ``bzr launchpad-login user && bzr push lp:~/project/branch`` will now
   push to ``lp:~user/project/branch``.  (John Arbash Meinel)
 
-Bug Fixes
-*********
-
-* Check if both --diff-options and --using are set, and exit with error
-  in this case. (Matthäus G. Chajdas, #234708)
-
-* Don't print internal object name when print an invalid revision spec
-  error.  (Neil Martinsen-Burrell, #598701)
-
-* ``HTTP/1.1` test servers now set a ``Content-Length`` header to comply
-  with pedantic ``HTTP/1.1`` clients. (Vincent Ladeuil, #568421)
-
-* `PathNotChild` should not give a traceback.
-  (Martin Pool, #98735)
-
-* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
-  which can result in "missing referenced chk root keys" errors when
-  fetching from repositories with affected revisions.
-  (Andrew Bennetts, #522637)
-
-* ``Transport.stat`` on a symlink, including a transport pointing directly
-  to a symlink, now returns information about the symlink.
-  (Martin Pool)
+* New development format ``development8-subtree`` which is similar to the 
+  ``2a`` format and adds subtree support. (Jelmer Vernooij)
 
 Improvements
 ************
 
+* Allow using both ``--using`` and ``--diff-options``.
+  (Matthäus G. Chajdas, #234708)
+
+* Allow using non-integer bug ID with generic bug trackers.
+  (Alexandre Garnier, #440472)
+
+* ``bzr remove`` now just backs up changed files instead of exiting,
+  forcing you to choose to either keep or delete them. Bazaar will now delete
+  the files if they can easily be recovered using revert, otherwise they
+  will be backed up (adding an extension of the form .~#~).
+  (Marius Kruger, #400554)
+
+* ``bzr revert`` and ``bzr status`` are up to 15% faster on large trees
+  with many changes by not repeatedly building a list of all file-ids.
+  (Andrew Bennetts)
+
+* Decrease memory consumption when many chk index pages are loaded. (Such
+  as during ``bzr co`` or ``bzr ls -R`` of a large tree.) Often we need to
+  read many chk pages because the individual chk map nodes will be spread
+  randomly. Peak memory for 'bzr ls -R' on a large tree dropped from 396MB
+  down to 247MB, expect even more significant savings on 64-bit platforms.
+  (John Arbash Meinel)
+
+* Decrease peak memory during ``bzr send``. The old code was caching all
+  text content and all inventory strings for all revisions before
+  computing the diffs. Now we only cache as long as there is a child that
+  will need them. Sending 2000 bzr revisions drops from 1.2GB peak to
+  256MB peak. (John Arbash Meinel, #614576)
+
+* ``DirState`` internals use a little bit less memory. For bzr.dev it
+  drops the memory from 1MB down to about 800kB. And replaces a few
+  thousand tuples and sets with StaticTuple.  (John Arbash Meinel)
+
+* Inventory entries now consume less memory (on 32-bit Ubuntu file entries
+  have dropped from 68 bytes to 40, and directory entries from 120 bytes
+  to 48).  (Andrew Bennetts)
+
+* Reduce peak memory by one copy of compressed text when writing to pack
+  repositories.
+  (John Arbash Meinel, #566940)
+
 * When building new working trees, default to reading from the repository
   rather than the source tree unless explicitly requested. (via
   ``--files-from`` and ``--hardlink`` for ``bzr branch`` and
@@ -75,1582 +930,9 @@
   content faster than seeking and reading content from another tree,
   especially in cold-cache situations. (John Arbash Meinel, #607298)
 
-Documentation
-*************
-
-API Changes
-***********
-
-Internals
-*********
-
-Testing
-*******
-
-
-bzr 2.2rc1
-##########
-
-:Codename: ???
-:2.2rc1: NOT RELEASED YET
-
-Compatibility Breaks
-********************
-
-* BzrError subclasses no longer support the name "message" to be used
-  as an argument for __init__ or in _fmt format specification as this
-  breaks in some Python versions. errors.LockError.__init__ argument
-  is now named "msg" instead of earlier "message".
-  (Parth Malwankar, #603461)
-
-* The old ``bzr selftest --benchmark`` option has been removed.
-  <https://launchpad.net/bzr-usertest> is an actively-maintained
-  macrobenchmark suite.
-  (Martin Pool)
-
-New Features
-************
-
-Bug Fixes
-*********
-
-* ``bzr ignore PATTERNS`` exits with error if a bad pattern is supplied.
-  ``InvalidPattern`` exception error message now shows faulting
-  regular expression.
-  (Parth Malwankar #300062)
-
-* Configuration files in ``${BZR_HOME}`` are now written in an atomic
-  way which should help avoid problems with concurrent writers.
-  (Vincent Ladeuil, #525571)
-
-* Don't traceback trying to unversion children files of an already
-  unversioned directory.  (Vincent Ladeuil, #494221)
-
-Improvements
-************
-
-* Add ``bzrlib.merge.MergeIntoMerger``, which can merge part or all of a
-  tree, and works with unrelated branches.  (Andrew Bennetts)
-
-Documentation
-*************
-
-* ``bzr help patterns`` now explains case insensitive patterns and
-  points to Python regular expression documentation.
-  (Parth Malwankar, #594386)
-
-API Changes
-***********
-
-* Delete ``ProgressTask.note``, which was deprecated in 2.1.
-
-Internals
-*********
-
-Testing
-*******
-
-* Unit test added to ensure that "message" is not uses as a format variable
-  name in BzrError subclasses as this conflicts with some Python versions.
-  (Parth Malwankar, #603461)
-
-bzr 2.2b4
-#########
-
-:Codename: Monkey Magic
-:2.2b4: 2004-07-09
-
-
-This fourth and final beta in the 2.2 series now stabilizes the internal
-APIs. Plugin authors are recommended to ensure their releases are
-compatible, so that 2.2rc1 can be a true release candidate, containing
-stable and compatible plugin versions. 
-
-For users of bzrlib as a library, one of the primary changes is to request
-that they call ``bzrlib.initialize`` and use the returned context manager
-appropriately.
-
-Better interaction with ``bzr-loom`` to make sure branching from a loom
-even over a smart server still yields a local loom. Not to mention lots of
-bugfixes over 2.2b3.
-
-Compatibility Breaks
-********************
-
-* bzrlib library users now need to call ``__enter__`` and ``__exit__`` on
-  the result of ``bzrlib.initialize``. This change was made when fixing
-  the bad habit recent bzr versions have had of leaving progress bars 
-  behind on the screen. That required calling another function before
-  exiting the program, and it made sense to provide a full context
-  manager at the same time. (Robert Collins)
-
-* The ``bzr`` front end now requires a ``bzrlib.ui.ui_factory`` which is a
-  context manager in the Python 2.5 and above sense. The bzrlib base class
-  is such a manager, but third party UI factories which do not derive from
-  ``bzrlib.ui.UIFactory`` will be incompatible with the command line front
-  end.
-
-* URLs like ``foo:bar/baz`` are now always parsed as a URL with scheme "foo"
-  and path "bar/baz", even if bzr does not recognize "foo" as a known URL
-  scheme.  Previously these URLs would be treated as local paths.
-  (Gordon Tyler)
-
-
-New Features
-************
-
-* Support ``--directory`` option for a number of additional commands:
-  conflicts, merge-directive, missing, resolve, shelve, switch,
-  unshelve, whoami. (Martin von Gagern, #527878)
-
-Bug Fixes
-*********
-
-* ``bzr branch`` to a new repository with a default stacking policy no
-  longer transfers the full history unnecessarily.
-  (Andrew Bennetts, #597942)
-
-* ``bzr init`` does not recursively scan directory contents anymore
-  leading to faster init for directories with existing content.
-  (Martin [gz], Parth Malwankar, #501307)
-
-* ``bzr log --exclude-common-ancestry`` is now taken into account for
-  linear ancetries. (Vincent Ladeuil, #575631)
-
-* ``bzr log -r branch:REMOTE`` can now properly log the remote branch,
-  rather than trying to fetch the data locally and failing because of a
-  readonly error. (Martin von Gagern, #149270)
-
-* ``bzr pull`` now works when a lp: URL is explicitly defined as the parent
-  or pull location in locations.conf or branch.conf.
-  (Gordon Tyler, #534787)
-
-* ``bzr reconfigure --unstacked`` now works with branches accessed via a
-  smart server. (Andrew Bennetts, #551525)
-
-* ``BzrDir.find_branches`` should ignore branches with missing repositories.
-  (Marius Kruger, Robert Collins)
-
-* ``BzrDir.find_bzrdirs`` should ignore dirs that raises PermissionDenied.
-  (Marius Kruger, Robert Collins)
-
-* Ensure that wrong path specifications in ``BZR_PLUGINS_AT`` display
-  proper error messages. (Vincent Ladeuil, #591215)
-
-* Explicitly removing ``--profile-imports`` option from parsed command-line
-  arguments on Windows, because bzr script does the same.
-  (Alexander Belchenko, #588277)
-
-* Fetching was slightly confused about the best code to use and was
-  using a new code path for all branches, resulting in more lookups than
-  necessary on old branches. (Robert Collins, #593515)
-
-* Final fix for 'no help for command' issue. We now show a clean message
-  when a command has no help, document how to set help more clearly, and
-  test that all commands available to the test suite have help.
-  (Robert Collins, #177500)
-
-* Invalid patterns supplied to ``Globster`` or ``lazy_regex`` now raise
-  ``InvalidPattern`` exception showing clear error message to the user.
-  (Parth Malwankar #300062)
-
-* Progress output is cleaned up when exiting.  (Aaron Bentley)
-
-* Raise ValueError instead of a string exception.
-  (John Arbash Meinel, #586926)
-
-* Relative imports in plugins are now handled correctly when using
-  BZR_PLUGINS_AT. (Vincent Ladeuil, #588959)
-
-* ``ScriptRunner`` now strips off leading indentation from test scripts,
-  which previously caused "SyntaxError: No command for line".
-  (Martin Pool)
-
-* Show unicode filenames in diff headers using terminal encoding. 
-  (Alexander Belchenko, Bug #382699)
-  NOTE for Windows users: If user need to save diff to file then user need to
-  change encoding of the terminal to ANSI encoding with command ``chcp XXX``
-  (e.g. ``chcp 1251`` for Russian Windows).
-
-* URL displayed for use with ``break-lock`` when smart server sees lock
-  contention are now valid. Default timeout for lock contention retry is
-  now 30 seconds instead of 300 seconds.
-  (Parth Malwankar, #250451)
-
-* ``walkdirs`` now raises a useful message when the filenames are not using
-  the filesystem encoding. (Eric Moritz, #488519)
-
-* Enable debugging of bzr on windows with pdb and other tools. This was 
-  broken because we call GetCommandLineW on windows. The fix adjusts the 
-  command line we get to be the same length as sys.argv.
-  (Jason Spashett, Alexander Belchenko, #587868)
-
-Improvements
-************
-
-* Bazaar now reads data from SSH connections more efficiently on platforms
-  that provide the ``socketpair`` function, and when using paramiko.
-  (Andrew Bennetts, #590637)
-
-* ``Branch.copy_content_into`` is now a convenience method dispatching to
-  a ``InterBranch`` multi-method. This permits ``bzr-loom`` and other
-  plugins to intercept this even when a ``RemoteBranch`` proxy is in use.
-  (Robert Collins, #201613)
-
-* ``Branch`` formats can now be loaded lazily by registering a
-  ``MetaDirBranchFormatFactory`` rather than an actual format. This will
-  cause the named format class to be loaded only when an enumeration of
-  formats is needed or when the format string for the object is
-  encountered. (Robert Collins, Jelmer Vernooij)
-
-* The encoding that bzr uses to output things other than file content can
-  now be overridden via the output_encoding configuration option.
-  (Martin Pool, #340394)
-
-* Use lazy imports in ``bzrlib/merge.py`` so that plugins like ``news_merge``
-  do not cause modules to be loaded unnecessarily just because the plugin
-  registers a merge hook.  This improves ``bzr rocks`` time by about 25%
-  in a default installation (with just the core plugins).
-  (Andrew Bennetts)
-
-Documentation
-*************
-
-* Added ``regression`` tag to our tags list. (Robert Collins)
-
-* Improved our release checklist to have a bit less churn and leave things
-  ready-to-go for the next action (including other people doing
-  development). (Robert Collins)
-
-* Remove obsolete discussion of PQM in documentation about how to
-  contribute to Bazaar.  (Martin Pool, #588444)
-
-API Changes
-***********
-
-* ``bzrlib.branch.InterBranch._get_branch_formats_to_test`` now returns
-  an iterable of format pairs, rather than just a single pair, permitting
-  InterBranch objects that work with multiple permutations to be
-  comprehensively tested. (Robert Collins)
-
-* ``bzrlib.lsprof.profile`` will no longer silently generate bad threaded
-  profiles when concurrent profile requests are made. Instead the profile
-  requests will be serialised. Reentrant requests will now deadlock.
-  (Robert Collins)
-
-* ``bzrlib.knit.KnitSequenceMatcher``, which has been deprecated since
-  2007, has been deleted.  Use ``PatienceSequenceMatcher`` from
-  ``bzrlib.patiencediff`` instead. (Andrew Bennetts)
-
-* ``bzrlib.re_compile_checked`` is now deprecated. Caller should handle
-  ``bzrlib.errors.InvalidPattern`` exception thrown by ``re.match`` in
-  case the default error message not suitable for the use case.
-  (Parth Malwankar)
-
-* ``bzrlib.tests.blackbox.ExternalBase`` is deprecated.  It provided only
-  one method ``check_output``, and we now recommend checking command
-  output using ``run_script``. (Martin Pool)
-
-* ``bzrlib.transport.ssh.SSHVendor.connect_ssh`` now returns an object
-  that implements the interface of ``bzrlib.transport.ssh.SSHConnection``.
-  Third-party implementations of ``SSHVendor`` may need to be updated
-  accordingly.  Similarly, any code using ``SSHConnection`` directly will
-  need to be updated.  (Andrew Bennetts)
-
-* The constructor of ``bzrilb.smart.medium.SmartSSHClientMedium`` has
-  changed to take an ``SSHParams`` instance (replacing many individual
-  values).  (Andrew Bennetts)
-
-Internals
-*********
-
-* ``bzrlib.osutils.get_terminal_encoding`` will now only mutter its
-  selection when explicitly requested; this avoids many duplicate calls
-  being logged when helpers, wrappers and older code that manually calls
-  it are executed it is now logged deliberately by the ui setup code.
-  (Robert Collins)
-
-* Improved ``bzrlib.urlutils`` to handle lp:foo/bar URLs. (Gordon Tyler)
-
-* ``bzrlib._c_static_tuple.StaticTuple`` now implements ``__sizeof__``, so
-  that ``sys.getsizeof`` and other memory analysis tools will report more
-  accurate results. (Andrew Bennetts)
-
-* The symbol_versioning module can now cleanup after itself -
-  ``suppress_deprecation_warnings`` now returns a cleanup function.
-  (Robert Collins)
-
-Testing
-*******
-
-* Add ``bzrlib.tests.fixtures`` to hold code for setting up objects
-  to test.  (Martin Pool)
-
-* ``test_import_tariff`` now respects BZR_PLUGINS_AT and BZR_PLUGINS_DISABLE.
-  (Vincent Ladeuil, #595587)
-
-bzr 2.2b3
-#########
-
-:2.2b3: 2010-05-28
-
-This third beta in the 2.2 series brings with it all the goodness of 2.1.2
-and 2.0.6 (though it preceeds 2.0.6 slightly). Of particular note for
-users are compatibility fixes with bzr 1.5 and below servers, a hopeful
-end to the EINTR errors caused by SIGWINCH interactions, a shiny new
-bash completion script and bzr will no longer guess at identity details -
-it was too unreliable in reality. Use ``bzr whoami`` on every new install.
-For developers we have some API changes which may impact plugins as well
-as a bunch of our regular improvements to internal clarity and test
-support.
-
-Compatibility Breaks
-********************
-
-* An API break has been made to the lock_write method of ``Branch`` and
-  ``Repository`` objects; they now return ``branch.BranchWriteLockResult``
-  and ``repository.RepositoryWriteLockResult`` objects. This makes
-  changing the API in future easier and permits some cleaner calling code.
-  The lock_read method has also changed from having no defined return
-  value to returning ``LogicalLockResult`` objects.
-  (Robert Collins)
-
-* ``bzr`` does not try to guess the username as ``username@hostname``
-  and requires it to be explictly set. This can be set using ``bzr
-  whoami``. (Parth Malwankar, #549310)
-
-* ``bzrlib.commands.Command`` will now raise ValueError during
-  construction if there is no __doc__ set. (Note, this will be reverted in
-  2.2b4) (Robert Collins)
-
-* The source tree no longer contains a contrib/zsh/_bzr completion
-  script. The new file contrib/zsh/README suggests alternatives.
-  (Martin von Gagern, #560030)
-
-New Features
-************
-
-* ``bzr commit`` accepts ``-p`` (for "patch") as a shorter name for
-  ``--show-diff``.
-  (Parth Malwankar, #571467)
-  
-* ``bzr ignore`` now supports a ``--default-rules`` option that displays
-  the default ignore rules used by bzr. The flag ``--old-default-rules``
-  is no longer supported by ``ignore``.
-  (Parth Malwankar, #538703)
-
-* ``bzr pack`` now supports a ``--clean-obsolete-packs`` option that
-  can save disk space by deleting obsolete pack files created during the
-  pack operation.
-  (Parth Malwankar, #304320)
-
-* New command line option ``--authors`` to ``bzr log`` allows users to
-  select which of the apparent authors and committer should be
-  included in the log. Defaults depend on format. (Martin von Gagern, #513322)
-
-* Support ``--directory`` option for a number of additional commands:
-  added, annotate, bind, cat, cat-revision, clean-tree, deleted,
-  export, ignore, ignored, lookup-revision, ls, modified, nick,
-  re-sign, unbind, unknowns.
-  (Martin von Gagern, #527878)
-
-* The bash_completion plugin from the bzr-bash-completion project has
-  been merged into the tree. It provides a bash-completion command and
-  replaces the outdated ``contrib/bash/bzr`` script with a version
-  using the plugin. (Martin von Gagern, #560030)
-
-* A new transport based on GIO (the gnome i/o library) provides access to
-  samba shares, webdav using gio+smb and gio+dav. It is also possible to
-  use gio for some already existing transport methods as gio+file,
-  gio+sftp, gio+ftp. 
-  (Mattias Eriksson)
-
-Bug Fixes
-*********
-
-* Alias information shown by ``bzr help`` is now accurate. This
-  was showing an internal object name for some plugin aliases.
-  (Parth Malwankar, #584650)
-
-* ``.bazaar``, ``.bazaar/bazaar.conf`` and ``.bzr.log`` inherit user and
-  group ownership from the containing directory. This allow bzr to work
-  better with sudo.
-  (Martin <gzlist@googlemail.com>, Parth Malwankar, #376388)
-
-* ``bzr clean-tree`` should not delete nested bzrdirs. Required for proper
-  support of bzr-externals and scmproj plugins.
-  (Alexander Belchenko, bug #572098)
-
-* ``bzr ignore`` will no longer add duplicate patterns to .bzrignore.
-  (Gordon Tyler, #572092)
-
-* ``bzr log --exclude-common-ancestry -r X..Y`` displays the revisions that
-  are part of Y ancestry but not part of X ancestry (aka the graph
-  difference).
-  (Vincent Ladeuil, #320119)
-
-* ``bzr lp-propose`` which was switched to use production Launchpad API
-  servers a few commits ago has been reverted to use edge: there is a
-  problem with using production which isn't trivially obvious, so we've
-  filed a bug to track it, and until thats fixed will be using edge.
-  (Robert Collins, #583667)
-
-* ``bzr rm`` should not refuse to delete directories which contained a file
-  which has been moved elsewhere in the tree after the previous commit.
-  (Marius Kruger, Daniel Watkins, #129880)
-
-* ``bzr selftest --parallel=fork`` wait for its children avoiding zombies.
-  (Vincent Ladeuil, #566670)
-
-* ``bzr selftest`` should not use ui.note() since it's not unicode safe.
-  (Vincent Ladeuil, #563997)
-
-* CommitBuilder refuses to create revisions whose trees have no root.
-  (Aaron Bentley)
-
-* Do not register a SIGWINCH signal handler, instead just poll for the
-  terminal width as needed.  This avoids the "Interrupted System Call"
-  problems that occur on POSIX with all currently released versions of
-  Python.
-  (Andrew Bennetts, #583941)
-
-* Don't mention --no-strict when we just issue the warning about unclean trees.
-  (Vincent Ladeuil, #401599)
-
-* Fixed ``AssertionError`` when accessing smart servers running Bazaar
-  versions before 1.6.
-  (Andrew Bennetts, #528041)
-
-* Improved progress bar for fetch (2a format only). Bazaar now shows an
-  estimate of the number of records to be fetched vs actually fetched.
-  (Parth Malwankar, #374740, #538868)
-
-* Reduce peak memory by one copy of compressed text.
-  (John Arbash Meinel, #566940)
-
-* ``RemoteBranch.lock_write`` raises ``ReadOnlyError`` if called during a
-  read lock, rather than causing an ``AttributeError``.
-  (Andrew Bennetts, Данило Шеган, #582781)
-
-* Selftest was failing with testtools 0.9.3, which caused an
-  AssertionError raised from a cleanUp to be reported as a Failure, not an
-  Error, breaking on of our test hygiene tests.
-  (Robert Collins, Vincent Ladeuil).
-
-* ``set_user_option`` with a dict on remote branches no longer fails with
-  an AttributeError.  There is a new ``Branch.set_config_option_dict`` RPC
-  to support this efficiently.
-  (Andrew Bennetts, #430382)
-  
-* Show the filenames when a file rename fails so that the error will be
-  more comprehensible.
-  (Martin Pool, #491763)
-
-* Support Pyrex 0.9.9, required changing how we handle exceptions in Pyrex.
-  (John Arbash Meinel, #582656)
-
-* Unicode characters in aliases are now handled correctly and do not cause
-  UnicodeEncodeError exception. (Parth Malwankar, #529930)
-
-* Unicode commit messages that are the same as a file name no longer cause
-  UnicodeEncodeError. ``ui.text.show_warning`` now handles unicode
-  messages.
-  (Parth Malwankar, #563646)
-
-* Using bzr with `lp:` urls behind an http proxy should work.
-  (Robert Collins, #558343)
-
-* When passing a file to ``UTF8DirReader`` make sure to close the current
-  directory file handle after the chdir fails. Otherwise when passing many
-  filenames into a command line ``bzr status`` we would leak descriptors.
-  (John Arbash Meinel, #583486)
-
-Improvements
-************
-
-* ``append_revisions_only`` will now be interpreted as a boolean and a
-  warning emitted if illegal values are used. Note that for projects
-  that needs to maintain compatibility with previsous bzr versions,
-  only 'True' and 'False' strings must be used (previous versions of
-  bzr will interpret all strings differing from 'True'
-  (case-sensitive) as false.
-  (Brian de Alwis, Vincent Ladeuil)
-
-* ``bzr ls`` now supports short options for existing long options.
-  ``-k/--kind``, ``-i/--ignored``, ``-u/--unknown`` and ``-0/--null``.
-  (Parth Malwankar, #181124)
-
-* ``Config.get_user_option_as_bool`` will now warn if a value cannot
-  be interpreted as a boolean.
-  (Vincent Ladeuil)
-
-* The all-in-one Windows installer will now be built with docstrings stripped
-  from the library zip, reducing the size and slightly improving cold startup
-  time. Bundled plugins are unchanged for the moment, but if adding other new
-  plugins to an all-in-one installation, ensure they are compiled and
-  installed with -O1 or help may not work. (Martin [gz])
-
-Documentation
-*************
-
-* First tests defined for sphinx, including a new bzrlib.tests.features.sphinx
-  to make the tests conditional.
-  (Vincent Ladeuil)
-
-* Added a builder/writer sphinx extension that can generate texinfo files. The
-  generated files are syntactically correct but the info navigation nodes
-  needs more work. (Vincent Ladeuil, #219334)
-
-API Changes
-***********
-
-* Added ``bzrlib.merge.PerFileMerger``, a more convenient way to write
-  some kinds of ``merge_file_content`` hook functions.
-  (Andrew Bennetts)
-  
-* `BzrDir`, `Branch`, `Repository` and `WorkingTree` now all support `user_url`,
-  `user_transport`, `control_url` and `control_transport` members pointing
-  respectively to the directory containing the ``.bzr`` control directory, 
-  and to the directory within ``.bzr`` used for the particular component.
-  All of them inherit from `ControlComponent` which provides default
-  implementations.
-  (Martin Pool)
-
-* Lock methods on ``Tree``, ``Branch`` and ``Repository`` are now
-  expected to return an object which can be used to unlock them. This reduces
-  duplicate code when using cleanups. The previous 'tokens's returned by
-  ``Branch.lock_write`` and ``Repository.lock_write`` are now attributes
-  on the result of the lock_write. ``repository.RepositoryWriteLockResult``
-  and ``branch.BranchWriteLockResult`` document this. (Robert Collins)
-
-* ``Repository.refresh_data`` may now be called in a write group on
-  pack-based repositories.  Older repositories will still raise an error
-  in this case.  Subclasses of ``Repository`` can still override
-  ``Repository._refresh_data``, but are now responsible for raising
-  ``bzrlib.repository.IsInWriteGroupError`` if they do not support
-  ``refresh_data`` during a write group.
-  (Andrew Bennetts, #574236)
-
-Internals
-*********
-
-* ``chk_map._bytes_to_text_key`` is now an optimized function to extract
-  the (file-id, revision-id) key from a CHKInventory entry. This can
-  potentially shave 5-10% time off during a large fetch. Related to bug
-  #562666. (John Arbash Meinel)
-
-* ``log._get_info_for_log_files`` now takes an add_cleanup callable.
-  (Robert Collins)
-
-* ``_remember_remote_is_before`` no longer raises AssertionError when
-  suboptimal network behaviour is noticed; instead it just mutters to the
-  log file (and warns the user if they have set the ``hpss`` debug flag).
-  This was causing unnecessary aborts for performance bugs that are minor
-  at worst.
-  (Andrew Bennetts, #528041)
-
-* Permit bzr to run under ``python -OO`` which reduces the size of bytecode
-  files loaded from disk. To ensure docstrings needed for help are never
-  stripped, the prefix ``__doc__ =`` should now be used.
-  (Martin <gzlist@googlemail.com>)
-
-* No longer require zlib headers to build extensions, and remove the need
-  for seperate copy of zlib library on windows.
-  (John Arbash Meinel, Martin <gzlist@googlemail.com>, #66923)
-
-Testing
-*******
-
-* Added ``bzrlib.tests.matchers`` as a place to put matchers, along with
-  our first in-tree matcher. See the module docstring for details.
-  (Robert Collins)
-
-* ``bzr selftest --parallel=subprocess`` now works correctly on win32.
-   (Gordon Tyler, #551332)
-
-* Workaround ``Crypto.Random`` check leading to spurious test
-  failures on Lucid, FreeBSD and gentoo.  
-  (Vincent Ladeuil, #528436)
-
-* New class ``ExecutableFeature`` for checking the availability of
-  executables on the ``PATH``. Migrated from bash_completion plugin.
-  (Martin von Gagern)
-
-bzr 2.2b2
-#########
-
-:2.2b2: 2010-04-16
-
-This is a somewhat early second beta of the 2.2 series, to fix a python2.4
-incompatibility in the 2.2b1 release.  It also includes a swag of
-performance, usability and correctness improvements: test feedback on all
-of these would be welcome.
-
-
-New Features
-************
-
-* ``bzr diff`` now supports a --format option, which can be used to 
-  select alternative diff formats. (Jelmer Vernooij, #555994)
-
-Bug Fixes
-*********
-
-* ``bzr dpush``, ``bzr push`` and ``bzr send`` will now issue a warning
-  instead of failing when dirty trees are involved. The corresponding
-  ``dpush_strict``, ``push_strict`` and ``send_strict`` should be set to
-  True explicitly to get the previous behaviour.  
-  (Vincent Ladeuil, #519319)
-
-* ``bzr export`` to tar file does not fail if any parent directory
-  contains unicode characters. This works around upstream Python bug
-  http://bugs.python.org/issue8396 .
-  (Parth Malwankar, #413406)
-
-* ``bzr switch`` does not die if a ConfigurableFileMerger is used.
-  (Aaron Bentley, #559436)
-
-* ``bzr update`` when a pending merge in the working tree has been merged
-  into the master branch will no longer claim that old commits have become
-  pending merges. (Robert Collins, #562079)
-
-* ``bzrlib.mutabletree.MutableTree.commit`` will now support a passed in
-  config as in previous versions of bzrlib. (Robert Collins)
-
-* Fix glitch in the warning about unclean trees display.
-  (Vincent Ladeuil, #562665)
-
-* Fixed Python2.4 incompatibilities in the bzr2.2b1 source tarball.
-  (Martin Pool)
-
-* Help messages generated by ``RegistryOption.from_kwargs`` list the
-  switches in alphabetical order, rather than in an undefined order.
-  (Martin von Gagern, #559409)
-
-* Make sure the ``ExecutablePath`` and ``InterpreterPath`` are set in
-  Apport crash reports, to avoid "This problem report applies to a program
-  which is not installed any more" error.
-  (Martin Pool, James Westby, #528114)
-
-* Reset ``siginterrupt`` flag to False every time we handle a signal
-  installed with ``set_signal_handler(..., restart_syscall=True)`` (from
-  ``bzrlib.osutils``.  Reduces the likelihood of "Interrupted System Call"
-  errors compared to registering ``signal.signal`` directly.
-  (Andrew Bennetts)
-
-* When invoked with a range revision, ``bzr log`` doesn't show revisions
-  that are not part of the Y revisions ancestry anymore when invoked with
-  -rX..Y.
-  (Vincent Ladeuil, #474807)
-
-* Properly handle ``param_name`` attribute for ``ListOption``.
-  (Martin von Gagern, 387117)
-
-Improvements
-************
-
-* ``bzr commit`` will prompt before using a commit message that was
-  generated by a template and not edited by the user.
-  (Robert Collins, #530265)
-
-* ``bzr diff`` read-locks the trees and branches only once, saving about
-  10-20ms on ``bzr diff`` in a bzr.dev tree.
-  (Andrew Bennetts)
-
-* ``bzr missing`` read-locks the branches only once.
-  (Andrew Bennetts)
-  
-* ``bzr pull`` locks the branches and tree only once.
-  (Andrew Bennetts)
-  
-* Index lookups in pack repositories search recently hit pack files first.  
-  In repositories with many pack files this can greatly reduce the
-  number of files accessed, the number of bytes read, and the number of
-  read calls.  An incremental pull via plain HTTP takes half the time and
-  bytes for a moderately large repository.  (Andrew Bennetts)
-
-* Index lookups only re-order the indexes when the hit files aren't
-  already first. Reduces the cost of reordering
-  (John Arbash Meinel, #562429)
-
-* Less code is loaded at startup.  (Cold-cache start time is about 10-20%
-  less.)
-  (Martin Pool, #553017)
-
-API Changes
-***********
-
-* ``bzrlib.diff.get_trees_and_branches_to_diff`` is deprecated.  Use
-  ``get_trees_and_branches_to_diff_locked`` instead.
-  (Andrew Bennetts)
-
-* ``TreeTransform.commit`` supports the full set of commit parameters, and
-  auto-determines branch nick if not supplied.  (Aaron Bentley)
-  
-Internals
-*********
-
-* ``bzrlib.commands.Command.run_direct`` is no longer needed - the pre
-  2.1 method of calling run() to perform testing or direct use via the API
-  is now possible again. As part of this, the _operation attribute on
-  Command is now transient and only exists for the duration of ``run()``.
-  (Robert Collins)
-
-bzr 2.2b1
-#########
-
-:2.2b1: 2010-04-01
-
-This is the first beta of the 2.2 series, leading up to a 2.2.0
-release in July or August.  Beta releases are suitable for everyday use
-but may cause some incompatibilities with plugins.  Some plugins may need
-small updates to work with 2.2b1.
-
-2.2b1 includes some changes to make merge conflicts easier to understand
-and resolve.  It also removes some old unnecessary code, and loads
-somewhat less code at startup.  It starts adding a common infrastructure
-for dealing with colocated named branches, which can be implemented in
-various ways in either bzr native or foreign formats.   On Ubuntu and
-other platforms with the apport bug-reporting library, there's an easier
-path to report problems with bzr.  We plan to continue with these themes
-through the 2.2 series.
-
-Over thirty bugs have been fixed, including in the log command, exporting
-to tarballs, restarting interrupted system calls, portability of compiled
-extensions, making backups during upgrade, and locking on ftp.
-
-Compatibility Breaks
-********************
-
-* BTreeGraphIndex can now take an offset to indicate that the data starts
-  somewhere other than then beginning of the file. (John Arbash Meinel)
-
-* Deleted very old hidden commands ``versionedfile-list``,
-  ``weave-plan-merge``, ``weave-merge-text``.
-  (Martin Pool)
-
-* ``Repository.get_inventory_sha1()`` and ``Repository.get_revision_xml()`` 
-  have been removed. (Jelmer Vernooij)
-
-* ``Repository.get_revision_inventory()`` has been removed in favor of
-  ``Repository.get_inventory()``. (Jelmer Vernooij)
-
-* All test servers have been moved out of the bzrlib.transport hierarchy to
-  bzrlib.tests.test_server *except* for MemoryServer, ChrootServer and
-  PathFilteringServer. ``bzrlib`` users may encounter test failures that can
-  be fixed by updating the related imports from ``bzrlib.transport.xxx`` to
-  ``bzrlib.tests.test_server``.
-  (Vincent Ladeuil)
-
-* ``BranchReferenceFormat.initialize()`` now takes an optional name argument
-  as its second parameter, for consistency with the initialize() method of
-  other formats. (Jelmer Vernooij)
-
-New Features
-************
-
-* Added ``bzr remove-branch`` command that can remove a local or remote 
-  branch. (Jelmer Vernooij, #276295)
-
-* ``bzr export`` now takes an optional argument ``--per-file-timestamps``
-  to set file mtimes to the last timestamp of the last revision in which
-  they were changed rather than the current time. (Jelmer Vernooij)
-
-* If the Apport crash-reporting tool is available, bzr crashes are now
-  stored into the ``/var/crash`` apport spool directory, and the user is
-  invited to report them to the developers from there, either
-  automatically or by running ``apport-bug``.  No information is sent
-  without specific permission from the user.  (Martin Pool, #515052)
-
-* Parsing of command lines, for example in ``diff --using``, no longer
-  treats backslash as an escape character on Windows.   (Gordon Tyler,
-  #392248)
-
-* Plugins can be disabled by defining ``BZR_DISABLE_PLUGINS`` as
-  a list of plugin names separated by ':' (';' on windows).
-  (Vincent Ladeuil, #411413)
-
-* Plugins can be loaded from arbitrary locations by defining
-  ``BZR_PLUGINS_AT`` as a list of name@path separated by ':' (';' on
-  windows). This takes precedence over ``BZR_PLUGIN_PATH`` for the
-  specified plugins. This is targeted at plugin developers for punctual
-  needs and *not* intended to replace ``BZR_PLUGIN_PATH``.  
-  (Vincent Ladeuil, #82693)
-
-* Tag names can now be determined automatically by ``automatic_tag_name`` 
-  hooks on ``Branch`` if they are not specified on the command line.
-  (Jelmer Vernooij)
-
-* Tree-shape conflicts can be resolved by providing ``--take-this`` and
-  ``--take-other`` to the ``bzr resolve`` command. Just marking the conflict
-  as resolved is still accessible via the ``--done`` default action.
-  (Vincent Ladeuil)
-
-* Merges can be proposed on Launchpad with the new lp-propose-merge command.
-  (Aaron Bentley, Jonathan Lange)
-
-Bug Fixes
-*********
-
-* Added docstring for ``Tree.iter_changes``
-  (John Arbash Meinel, #304182)
-
-* Allow additional arguments to
-  ``RemoteRepository.add_inventory_by_delta()``. (Jelmer Vernooij, #532631)
-
-* Allow exporting a single file using ``bzr export``.
-  (Michal Junák, #511987)
-
-* Allow syscalls to automatically restart when ``TextUIFactory``'s
-  SIGWINCH handler is invoked, avoiding ``EINTR`` errors during blocking
-  IO, which are often poorly handled by Python's libraries and parts of
-  bzrlib.  (Andrew Bennetts, #496813)
-
-* Avoid infinite recursion when probing for apport.
-  (Vincent Ladeuil, #516934)
-
-* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
-  (Martin Pool, #331095)
-
-* Avoid truncating svn URLs.
-  (Martin Pool, Martin von Gagern, #545185)
-
-* ``bzr add`` will not add conflict related files unless explicitly required.
-  (Vincent Ladeuil, #322767, #414589)
-
-* ``bzr dump-btree`` now works on ``*.cix`` and ``*.six`` files.  Those
-  indices do not have reference lists, so ``dump-btree`` will simply show
-  ``None`` instead.  (Andrew Bennetts, #488607)
-
-* ``bzr help`` will no longer trigger the get_missing_command hook when
-  doing a topic lookup. This avoids prompting (like 'no command plugins/loom,
-  did you mean log?') when getting help. In future we may trigger the hook 
-  deliberately when no help topics match from any help index.
-  (Robert Collins, #396261)
-
-* ``bzr log -n0 -r..A.B.C`` should not crash but just consider the None
-  revspec as representing the first revision of the branch.
-  (Vincent Ladeuil, #519862)
-
-* ``bzr remove-tree`` can now remove multiple working trees.
-  (Jared Hance, Andrew Bennetts, #253137)
-
-* ``bzr resolve --take-this`` and ``--take-other`` now correctly renames
-  the kept file on content conflicts where one side deleted the file.
-  (Vincent Ladeuil, #529968)
-
-* ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
-  permissions as ``.bzr`` directory on a POSIX OS.
-  (Parth Malwankar, #262450)
-
-* ``bzr upgrade`` now names backup directory as ``backup.bzr.~N~`` instead
-  of ``backup.bzr``. This directory is ignored by bzr commands such as
-  ``add``.
-  (Parth Malwankar, #335033, #300001)
-
-* Cope with non-utf8 characters inside ``.bzrignore``.
-  (Jason Spashett, #183504)
-
-* Correctly interpret "451 Rename/move failure: Directory not empty" from
-  ftp servers while trying to take a lock.
-  (Martin Pool, #528722)
-
-* DirStateRevisionTree.kind() was returning wrong result when 'kind'
-  changes occured between the workingtree and one of its parents.
-  (Vincent Ladeuil, #535547)
-
-* Fix ``log`` to better check ancestors even if merged revisions are involved.
-  (Vincent Ladeuil, #476293)
-
-* Loading a plugin from a given path with ``BZR_PLUGINS_AT`` doesn't depend
-  on os.lisdir() order and is now reliable.
-  (Vincent Ladeuil, #552922).
-
-* Many IO operations that returned ``EINTR`` were retried even if it
-  wasn't safe to do so via careless use of ``until_no_eintr``.  Bazaar now
-  only retries operations that are safe to retry, and in some cases has
-  switched to operations that can be retried (e.g. ``sock.send`` rather than
-  ``sock.sendall``).
-  (Andrew Bennetts, Martin <gzlist@googlemail.com>, #496813)
-
-* Path conflicts now support --take-this and --take-other even when a
-  deletion is involved.
-  (Vincent Ladeuil, #531967)
-
-* Network transfer amounts and rates are now displayed in SI units according
-  to the Ubuntu Units Policy <https://wiki.ubuntu.com/UnitsPolicy>.
-  (Gordon Tyler, #514399)
-
-* Support kind markers for socket and fifo filesystem objects. This
-  prevents ``bzr status --short`` from crashing when those files are
-  present.  (John Arbash Meinel, #303275)
-
-* ``bzr mkdir DIR`` will not create DIR unless DIR's parent is a versioned
-   directory. (Parth Malwankar, #138600)
-
-* SSH child processes will now ignore SIGQUIT on nix systems so breaking into
-  the debugger won't kill the session.
-  (Martin <gzlist@googlemail.com>, #162502)
-
-* Tolerate patches with leading noise in ``bzr-handle-patch``.
-  (Toshio Kuratomi, Martin Pool, #502076)
-
-* ``update -r`` now supports updating to revisions that are not on
-  mainline (i.e. it supports dotted revisions).
-  (Parth Malwankar, #517800)
-
-* Use first apparent author not committer in GNU Changelog format.
-  (Martin von Gagern, #513322)
-
-API Changes
-***********
-
-* ``bzrlib.merge_directive._BaseMergeDirective`` has been renamed to 
-  ``bzrlib.merge_directive.BaseMergeDirective`` and is now public.
-  (Jelmer Vernooij)
-
-* ``BranchFormat.initialize`` now takes an optional ``name`` of the colocated 
-  branch to create. (Jelmer Vernooij)
-
-* ``BzrDir.get_branch_transport`` now takes an optional ``name`` of the 
-  colocated branch to open. (Jelmer Vernooij)
-
-* Added ``bzrlib.osutils.set_signal_handler``, a convenience function that
-  can set a signal handler and call ``signal.siginterrupt(signum,
-  False)`` for it, if the platform and Python version supports it.
-  (Andrew Bennetts, #496813)
-
-* New ``bzrlib.initialize`` is recommended for programs using bzrlib to 
-  run when starting up; it sets up several things that previously needed
-  to be done separately.
-  (Martin Pool, #507710)
-
-* Exporters now support a ``per_file_timestamps`` argument to write out the 
-  timestamp of the commit in which a file revision was introduced.
-  (Jelmer Vernooij)
-
-* New method ``BzrDir.list_branches()`` that returns a sequence of branches 
-  present in a control directory. (Jelmer Vernooij)
-
-* New method ``Repository.get_known_graph_ancestry()``. 
-  (Jelmer Vernooij, #495502)
-
-* New transport methods ``readlink``, ``symlink`` and ``hardlink``.
-  (Neil Santos)
-
-* Remove unused ``CommandFailed`` exception.
-  (Martin Pool)
-
-Internals
-*********
-
-* ``bzrlib.branchbuilder.BranchBuilder.build_snapshot`` now accepts a
-  ``message_callback`` in the same way that commit does. (Robert Collins)
-
-* ``bzrlib.builtins.Commit.run`` raises ``bzrlib.errors.BoundBranchOutOfDate``
-  rather than ``bzrlib.errors.BzrCommandError`` when the bound branch is out
-  of date. (Gary van der Merwe)
-
-* ``bzrlib.commands.run_bzr`` is more extensible: callers can supply the
-  functions to load or disable plugins if they wish to use a different
-  plugin mechanism; the --help, --version and no-command name code paths
-  now use the generic pluggable command lookup infrastructure.
-  (Robert Collins)
-
-* ``bzrlib.errors.BoundBranchOutOfDate`` has a new field ``extra_help``
-  which can be set to add extra help to the error. (Gary van der Merwe)
-
-* New method ``Branch.automatic_tag_name`` that can be used to find the
-  tag name for a particular revision automatically. (Jelmer Vernooij)
-
-* The methods ``BzrDir.create_branch()``, ``BzrDir.destroy_branch()`` and 
-  ``BzrDir.open_branch()`` now take an optional ``name`` argument. 
-  (Jelmer Vernooij)
-
-Testing
-*******
-
-* bzr now has a ``.testr.conf`` file in its source tree configured
-  appropriately for running tests with Testrepository
-  (``https://launchpad.net/testrepository``). (Robert Collins)
-
-* Documentation about testing with ``subunit`` has been tweaked.
-  (Robert Collins)
-
-* Known failures has been added for resolve --take-other on ParentLoop
-  conflicts. This reflects bug #537956 without fixing it.
-  (Vincent Ladeuil)
-
-* New ``bzrlib.tests.test_import_tariff`` can make assertions about what
-  Python modules are loaded, to guard against startup time or library
-  dependency regressions.
-  (Martin Pool)
-
-* PQM will now run with subunit output. To analyze a PQM error use
-  tribunal, or cat log | subunit-filter | subunit2pyunit. (Robert Collins)
-
-* Stop sending apport crash files to ``.cache`` in the directory from
-  which ``bzr selftest`` was run.  (Martin Pool, #422350)
-
-* Tests no longer fail if "close() called during concurrent
-  operation on the same file object" occurs when closing the log file
-  (which can happen if a thread tries to write to the log file at the
-  wrong moment).  An warning will be written to ``stderr`` when this
-  happens, and another warning will be written if the log file could not
-  be closed after retrying 100 times.  (Andrew Bennetts, #531746)
-
-bzr 2.1.3
-#########
-
-:Codename: Do run run
-:2.1.3: NOT RELEASED YET
-
-Compatibility Breaks
-********************
-
-New Features
-************
-
-Bug Fixes
-*********
-
-* ``bzr ignore PATTERNS`` exits with error if a bad pattern is supplied.
-  ``InvalidPattern`` exception error message now shows faulting
-  regular expression.
-  (Parth Malwankar #300062)
-
-* Configuration files in ``${BZR_HOME}`` are now written in an atomic
-  way which should help avoid problems with concurrent writers.
-  (Vincent Ladeuil, #525571)
-
-* Don't traceback trying to unversion children files of an already
-  unversioned directory.  (Vincent Ladeuil, #494221)
-
-* ``HTTP/1.1` test servers now set a ``Content-Length`` header to comply
-  with pedantic ``HTTP/1.1`` clients. (Vincent Ladeuil, #568421)
-
-* Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
-  which can result in "missing referenced chk root keys" errors when
-  fetching from repositories with affected revisions.
-  (Andrew Bennetts, #522637)
-
-* Progress bars prefer to truncate the text message rather than the
-  counters.  The spinner is shown between the network transfer indicator
-  and the progress message.  (Martin Pool)
-
-* Raise ValueError instead of a string exception.
-  (John Arbash Meinel, #586926)
-
-* Recursive binding for checkouts is now detected by bzr. A clear error
-  message is shown to the user. (Parth Malwankar, #405192)
-
-Improvements
-************
-
-* Add ``bzrlib.merge.MergeIntoMerger``, which can merge part or all of a
-  tree, and works with unrelated branches.  (Andrew Bennetts)
-
-Documentation
-*************
-
-* ``bzr help patterns`` now explains case insensitive patterns and
-  points to Python regular expression documentation.
-  (Parth Malwankar, #594386)
-
-API Changes
-***********
-
-* Delete ``ProgressTask.note``, which was deprecated in 2.1.
-
-Internals
-*********
-
-Testing
-*******
-
-* Unit test added to ensure that "message" is not uses as a format variable
-  name in BzrError subclasses as this conflicts with some Python versions.
-  (Parth Malwankar, #603461)
-
-bzr 2.1.2
-#########
-
-:2.1.2: 2010-05-28
-
-This release fixes two critical networking issues with older servers and
-with interrupted system call errors when pushing or pulling.  We recommend
-upgrading to anyone running a 2.1.x version of bzr.
-
-Bug Fixes
-*********
-
-* ``bzr clean-tree`` should not delete nested bzrdirs. Required for proper
-  support of bzr-externals and scmproj plugins.
-  (Alexander Belchenko, bug #572098)
-
-* ``bzr switch`` does not die if a ConfigurableFileMerger is used.
-  (Aaron Bentley, #559436)
-
-* Do not register a SIGWINCH signal handler, instead just poll for the
-  terminal width as needed.  This avoids the "Interrupted System Call"
-  problems that occur on POSIX with all currently released versions of
-  Python.
-  (Andrew Bennetts, #583941)
-
-* Fixed ``AssertionError`` when accessing smart servers running Bazaar
-  versions before 1.6.
-  (Andrew Bennetts, #528041)
-
-* Reset ``siginterrupt`` flag to False every time we handle a signal
-  installed with ``set_signal_handler(..., restart_syscall=True)`` (from
-  ``bzrlib.osutils``.  Reduces the likelihood of "Interrupted System Call"
-  errors compared to registering ``signal.signal`` directly.
-  (Andrew Bennetts)
-
-* Reduce peak memory by one copy of compressed text.
-  (John Arbash Meinel, #566940)
-
-* Support Pyrex 0.9.9, required changing how we handle exceptions in Pyrex.
-  (John Arbash Meinel, #582656)
-
-* When passing a file to ``UTF8DirReader`` make sure to close the current
-  directory file handle after the chdir fails. Otherwise when passing many
-  filenames into a command line ``bzr status`` we would leak descriptors.
-  (John Arbash Meinel, #583486)
-
-Internals
-*********
-
-* ``_remember_remote_is_before`` no longer raises AssertionError when
-  suboptimal network behaviour is noticed; instead it just mutters to the
-  log file (and warns the user if they have set the ``hpss`` debug flag).
-  This was causing unnecessary aborts for performance bugs that are minor
-  at worst.
-  (Andrew Bennetts, #528041)
-
-
-bzr 2.1.1
-#########
-
-:2.1.1: 2010-03-24
-
-This is a small bugfix release.  Upgrading is recommended for anyone
-running 2.1.0 or earlier.
-
-Bug Fixes
-*********
-
-* Allow syscalls to automatically restart when ``TextUIFactory``'s
-  SIGWINCH handler is invoked, avoiding ``EINTR`` errors during blocking
-  IO, which are often poorly handled by Python's libraries and parts of
-  bzrlib.  (Andrew Bennetts, #496813)
-
-* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
-  (Martin Pool, #331095)
-
-* Fix plugin packaging on Windows. (Ian Clatworthy, #524162)
-
-* Fix stub sftp test server to call os.getcwdu().
-  (Vincent Ladeuil, #526221, #526353)
-
-* Fixed CHM generation by moving the NEWS section template into
-  a separate file. (Ian Clatworthy, #524184)
-
-* Merge correctly when this_tree is not a WorkingTree.  (Aaron Bentley)
-
-* Register SIGWINCH handler only when creating a ``TextUIFactory``; avoids
-  problems importing bzrlib from a non-main thread.
-  (Elliot Murphy, #521989)
-
-* Repositories accessed via a smart server now reject being stacked on a
-  repository in an incompatible format, as is the case when accessing them
-  via other methods.  This was causing fetches from those repositories via
-  a smart server (e.g. using ``bzr branch``) to receive invalid data.
-  (Andrew Bennetts, #562380)
-
-* Standardize the error handling when creating a new ``StaticTuple``
-  (problems will raise TypeError). (Matt Nordhoff, #457979)
-
-* Warn if pyrex is too old to compile the new ``SimpleSet`` and
-  ``StaticTuple`` extensions, rather than having the build fail randomly.
-  (John Arbash Meinel, #449776)
-
-Documentation
-*************
-
-* Added a link to the Desktop Guide. (Ian Clatworthy)
-
-* Added What's New in Bazaar 2.1 document. (Ian Clatworthy)
-
-* Drop Google Analytics from the core docs as they caused problems
-  in the CHM files. (Ian Clatworthy, #502010)
-
-API Changes
-***********
-
-* Added ``bzrlib.osutils.set_signal_handler``, a convenience function that
-  can set a signal handler and call ``signal.siginterrupt(signum,
-  False)`` for it, if the platform and Python version supports it.
-  (Andrew Bennetts, #496813)
-
-
-bzr 2.1.0
-#########
-
-:Codename: Strasbourg
-:2.1.0: 2010-02-11
-
-This release marks our second long-term-stable series. The Bazaar team
-has decided that we will continue to make bugfix-only 2.0.x and 2.1.x
-releases, along with 2.2 development releases. 
-
-This is a fairly incremental update, focusing on polish and bugfixing.
-There are no changes for supported disk formats. Key updates include
-reduced memory consumption for many operations, a new per-file merge
-hook, ignore patterns can now include '!' to exclude files, globbing
-support for all commands on Windows, and support for addressing home
-directories via ``bzr+ssh://host/~/`` syntax.
-
-Users are encouraged to upgrade from the 2.0 stable series.
-
-Bug Fixes
-*********
-
-* Don't require testtools to use sftp.
-  (Vincent Ladeuil, #516183)
-
-* Fix "AttributeError in Inter1and2Helper" during fetch.
-  (Martin Pool, #513432)
-
-* ``bzr update`` performs the two merges in a more logical order and will stop
-  when it encounters conflicts.  
-  (Gerard Krol, #113809)
-
-* Give a better error message when doing ``bzr bind`` in an already bound
-  branch.  (Neil Martinsen-Burrell, #513063)
-
-* Ignore ``KeyError`` from ``remove_index`` during ``_abort_write_group``
-  in a pack repository, which can happen harmlessly if the abort occurs during
-  finishing the write group.  Also use ``bzrlib.cleanup`` so that any
-  other errors that occur while aborting the individual packs won't be
-  hidden by secondary failures when removing the corresponding indices.
-  (Andrew Bennetts, #423015)
-
-* Set the mtime of files exported to a directory by ``bzr export`` all to
-  the same value to avoid confusing ``make`` and other date-based build
-  systems. (Robert Collins, #515631)
-
-Improvements
-************
-
-* Fetching into experimental formats will now print a warning. (Jelmer
-  Vernooij)
-
-API Changes
-***********
-
-* ``Repository.deserialise_inventory`` has been renamed to 
-  ``Repository._deserialise_inventory`` to indicate it is private.
-  (Jelmer Vernooij)
-
-* ``Repository.get_inventory_xml`` has been renamed to 
-  ``Repository._get_inventory_xml`` to indicate it is private. 
-  (Jelmer Vernooij)
-
-* ``Repository.serialise_inventory`` has been renamed to 
-  ``Repository._serialise_inventory`` to indicate it is private.
-
-* Using the ``bzrlib.chk_map`` module from within multiple threads at the
-  same time was broken due to race conditions with a module level page
-  cache. This shows up as a KeyError in the ``bzrlib.lru_cache`` code with
-  ``bzrlib.chk_map`` in the backtrace, and can be triggered without using
-  the same high level objects such as ``bzrlib.repository.Repository``
-  from different threads. chk_map now uses a thread local cache which may
-  increase memory pressure on processes using threads.
-  (Robert Collins, John Arbash Meinel, #514090)
-
-* The new ``merge_file_content`` should now be ok with tests to avoid
-  regressions.
-  (Vincent Ladeuil, #515597)
-
-Internals
-*********
-
-* Use ``bzrlib.cleanup`` rather than less robust ``try``/``finally``
-  blocks in several places in ``bzrlib.merge``.  This avoids masking prior
-  errors when errors like ``ImmortalPendingDeletion`` occur during cleanup
-  in ``do_merge``.
-  (Andrew Bennetts, #517275)
-
-API Changes
-***********
-
-* The ``remove_index`` method of
-  ``bzrlib.repofmt.pack_repo.AggregateIndex`` no longer takes a ``pack``
-  argument.  This argument was always ignored.
-  (Andrew Bennetts, #423015)
-
-bzr 2.1.0rc2
-############
-
-:Codename: after the bubbles
-:2.1.0rc2: 2010-01-29
-
-This is a quick-turn-around to update a small issue with our new per-file
-merge hook. We expect no major changes from this to the final 2.1.0.
-
-API Changes
-***********
-
-* The new ``merge_file_content`` hook point has been altered to provide a
-  better API where state for extensions can be stored rather than the
-  too-simple function based approach. This fixes a performance regression
-  where branch configuration would be parsed per-file during merge. As
-  part of this the included news_merger has been refactored into a base
-  helper class ``bzrlib.merge.ConfigurableFileMerger``.
-  (Robert Collins, John Arbash Meinel, #513822)
-
-
-bzr 2.1.0rc1
-############
-
-:Codename: the 'new' stable
-:2.1.0rc1: 2009-01-21
-
-This is the first stable release candidate for Bazaar's 2.1 series. From
-this point onwards, the 2.1 series will be considered stable (as the 2.0
-series) and only bugfixes are expected to be incorporated. The dozen or so
-bugfixes in the 2.0.4 release are also included in this release (along
-with more than 15 more bugfixes). Some of the interesting features are
-support for per-file merge hooks, ``bzr unshelve --preview``, support
-for using ! in ignore files to exclude files from being ignored, a small
-memory leak was squashed, and many ``ObjectNotLocked`` errors were fixed.
-This looks to be a very good start for a new stable series.
-
-
-New Features
-************
-
-* Add bug information to log output when available.
-  (Neil Martinsen-Burrell, Guillermo Gonzalez, #251729)
-
-* Added ``merge_file_content`` hook point to ``Merger``, allowing plugins
-  to register custom merge logic, e.g. to provide smarter merging for
-  particular files.
-
-* Bazaar now includes the ``news_merge`` plugin.  It is disabled by
-  default, to enable it add a ``news_merge_files`` option to your
-  configuration.  Consult ``bzr help news_merge`` for more information.
-  (Andrew Bennetts)
-  
-* ``bzr branch`` now takes a ``--bind`` option. This lets you
-  branch and bind all in one command. (Ian Clatworthy)
-
-* ``bzr switch`` now takes a ``--revision`` option, to allow switching to
-  a specific revision of a branch. (Daniel Watkins, #183559)
-
-* ``bzr unshelve --preview`` can now be used to show how a patch on the
-  shelf would be applied to the working tree.
-  (Guilherme Salgado, #308122)
-
-* ``bzr update`` now takes a ``--revision`` argument. This lets you
-  change the revision of the working tree to any revision in the
-  ancestry of the current or master branch. (Matthieu Moy, Mark Hammond,
-  Martin Pool, #45719)
-
-* ``-Dbytes`` can now be used to display the total number of bytes
-  transferred for the current command. This information is always logged
-  to ``.bzr.log`` for later inspection. (John Arbash Meinel)
-
-* New ignore patterns.  Patterns prefixed with '!' are exceptions to 
-  ignore patterns and take precedence over regular ignores.  Such 
-  exceptions are used to specify files that should be versioned which 
-  would otherwise be ignored.  Patterns prefixed with '!!' act as regular 
-  ignore patterns, but have highest precedence, even over the '!' 
-  exception patterns. (John Whitley, #428031)
-
-* The ``supress_warnings`` configuration option has been introduced to disable
-  various warnings (it currently only supports the ``format_deprecation``
-  warning). The new option can be set in any of the following locations:
-  ``bazaar.conf``, ``locations.conf`` and/or ``branch.conf``.
-  (Ted Gould, Matthew Fuller, Vincent Ladeuil)
-
-Bug Fixes
-*********
-
-* Always show a message if an OS error occurs while trying to run a
-  user-specified commit message editor.
-  (Martin Pool, #504842)
-
-* ``bzr diff`` will now use the epoch when it is unable to determine 
-  the timestamp of a file, if the revision it was introduced in is a
-  ghost. (Jelmer Vernooij, #295611)
-
-* ``bzr switch -b`` can now create branches that are located using directory
-  services such as ``lp:``, even when the branch name doesn't contain a
-  '/'.  (Neil Martinsen-Burrell, #495263)
-
-* ``bzr unshelve`` has improved messages about what it is doing.
-  (Neil Martinsen-Burrell, #496917)
-
-* Concurrent autopacking is more resilient to already-renamed pack files.
-  If we find that a file we are about to obsolete is already obsoleted, we
-  do not try to rename it, and we leave the file in ``obsolete_packs``.
-  The code is also fault tolerant if a file goes missing, assuming that
-  another process already removed the file.
-  (John Arbash Meinel, Gareth White, #507557)
-
-* Fix "Too many concurrent requests" in reconcile when network connection
-  fails.  (Andrew Bennetts, #503878)
-
-* Fixed a side effect mutation of ``RemoteBzrDirFormat._network_name``
-  that caused some tests to fail when run in a non-default order.
-  Probably no user impact.  (Martin Pool, #504102)
-
-* Fixed ``ObjectNotLocked`` error in ``bzr cat -rbranch:../foo FILE``.
-  (Andrew Bennetts, #506274)
-
-* FTP transports support Unicode paths by encoding/decoding them as utf8.
-  (Vincent Ladeuil, #472161)
-
-* Listen to the SIGWINCH signal to update the terminal width.
-  (Vincent Ladeuil, #316357)
-
-* Progress bars are now hidden when ``--quiet`` is given.
-  (Martin Pool, #320035)
-
-* ``SilentUIFactory`` now supports ``make_output_stream`` and discards
-  whatever is written to it.  This un-breaks some plugin tests that
-  depended on this behaviour.
-  (Martin Pool, #499757)
-
-* When operations update the working tree, all affected files should end
-  up with the same mtime. (eg. when versioning a generated file, if you
-  update the source and the generated file together, the generated file
-  should appear up-to-date.)
-  (John Arbash Meinel, Martin <gzlist>, #488724)
-
-Improvements
-************
-
-* Added ``add_cleanup`` and ``cleanup_now`` to ``bzrlib.command.Command``.
-  All the builtin commands now use ``add_cleanup`` rather than
-  ``try``/``finally`` blocks where applicable as it is simpler and more
-  robust.  (Andrew Bennetts)
-
-* All except a small number of storage formats are now hidden, making
-  the help for numerous commands far more digestible. (Ian Clatworthy)
-
-* Attempts to open a shared repository as a branch (e.g. ``bzr branch
-  path/to/repo``) will now include "location is a repository" as a hint in
-  the error message.  (Brian de Alwis, Andrew Bennetts, #440952)
-
-* Push will now inform the user when they are trying to push to a foreign 
-  VCS for which roundtripping is not supported, and will suggest them to 
-  use dpush. (Jelmer Vernooij)
-
-* The version of bzr being run is now written to the log file.
-  (__monty__, #257170)
-
-* Transport network activity indicator is shown more of the time when
-  Bazaar is doing network IO.
-  (Martin Pool)
-
-Documentation
-*************
-
-* Add documentation on creating merges with more than one parent.
-  (Neil Martinsen-Burrell, #481526)
-
-* Better explain the --uncommitted option of merge.
-  (Neil Martinsen-Burrell, #505088)
-
-* Improve discussion of pending merges in the documentation for
-  ``revert``.  (Neil Martinsen-Burrell, #505093)
-
-* Improved help for ``bzr send``. 
-  (Martin Pool, Bojan Nikolic)
-
-* There is a System Administrator's Guide in ``doc/en/admin-guide``,
-  including discussions of installation, relevant plugins, security and 
-  backup. (Neil Martinsen-Burrell)
-
-* The ``conflicts`` help topic has been renamed to ``conflict-types``.
-  (Ian Clatworthy)
-
-* The User Reference is now presented as a series of topics.
-  Many of the included topics have link and format tweaks applied.
-  (Ian Clatworthy)
-
-API Changes
-***********
-
-* Added ``cachedproperty`` decorator to ``bzrlib.decorators``.
-  (Andrew Bennetts)
-
-* Many test features were renamed from ``FooFeature`` to ``foo_feature``
-  to be consistent with instances being lower case and classes being
-  CamelCase. For the features that were more likely to be used, we added a
-  deprecation thunk, but not all. (John Arbash Meinel)
-
-* Merger classes (such as ``Merge3Merger``) now expect a ``this_branch``
-  parameter in their constructors, and provide ``this_branch`` as an
-  attribute. (Andrew Bennetts)
-  
-* The Branch hooks pre_change_branch_tip no longer masks exceptions raised
-  by plugins - the original exceptions are now preserved. (Robert Collins)
-
-* The Transport ``Server.tearDown`` method is now renamed to
-  ``stop_server`` and ``setUp`` to ``start_server`` for consistency with
-  our normal naming pattern, and to avoid confusion with Python's
-  ``TestCase.tearDown``.  (Martin Pool)
-
-* ``WorkingTree.update`` implementations must now accept a ``revision``
-  parameter.
-
-Internals
-*********
-
-* Added ``BzrDir.open_branchV3`` smart server request, which can receive
-  a string of details (such as "location is a repository") as part of a
-  ``nobranch`` response.  (Andrew Bennetts, #440952)
-  
-* New helper osutils.UnicodeOrBytesToBytesWriter which encodes unicode
-  objects but passes str objects straight through. This is used for
-  selftest but may be useful for diff and other operations that generate
-  mixed output. (Robert Collins)
-
-* New exception ``NoRoundtrippingSupport``, for use by foreign branch 
-  plugins. (Jelmer Vernooij)
-
-Testing
-*******
-
-* ``bzrlib.tests.permute_for_extension`` is a helper that simplifies
-  running all tests in the current module, once against a pure python
-  implementation, and once against an extension (pyrex/C) implementation.
-  It can be used to dramatically simplify the implementation of
-  ``load_tests``.  (John Arbash Meinel)
-
-* ``bzrlib.tests.TestCase`` now subclasses ``testtools.testcase.TestCase``.
-  This permits features in testtools such as getUniqueInteger and
-  getUniqueString to be used. Because of this, testtools version 0.9.2 or
-  newer is now a dependency to run bzr selftest. Running with versions of
-  testtools less than 0.9.2 will cause bzr to error while loading the test
-  suite. (Robert Collins)
-
-* Shell-like tests now support the command "mv" for moving files.  The
-  syntax for ``mv file1 file2``, ``mv dir1 dir2`` and ``mv file dir`` is
-  supported.  (Neil Martinsen-Burrell)
-
-* The test progress bar no longer distinguishes tests that 'errored' from
-  tests that 'failed' - they're all just failures.
-  (Martin Pool)
-
-bzr 2.0.6
-#########
-
-:2.0.6: NOT RELEASED YET
+* Add ``__pycache__`` to the default ``ignores`` file. Future releases of
+  Python will use this directory to store bytecodes.
+  (Andrea Corbellini, #626687)
 
 Bug Fixes
 *********
@@ -1659,20 +941,100 @@
   history no longer crash when deleted files are involved.
   (Vincent Ladeuil, John Arbash Meinel, #375898)
 
+* ``bzr add SYMLINK/FILE`` now works properly when the symlink points to a
+  previously-unversioned directory within the tree: the directory is
+  marked versioned too.
+  (Martin Pool, #192859)
+
+* ``bzr clean-tree`` issues a warning if it is unable to delete a file
+  due to ``errno.EACCES`` instead of exiting with an error on Windows.
+  (Parth Malwankar, #430785)
+
 * ``bzr commit SYMLINK`` now works, rather than trying to commit the
   target of the symlink.
   (Martin Pool, John Arbash Meinel, #128562)
 
-* ``bzr revert`` now only takes write lock on working tree, instead of on 
-  both working tree and branch.
-  (Danny van Heumen, #498409)
+* ``bzr ignore PATTERNS`` exits with error if a bad pattern is supplied.
+  ``InvalidPattern`` exception error message now shows faulting
+  regular expression.
+  (Parth Malwankar #300062)
 
 * ``bzr upgrade`` now creates the ``backup.bzr`` directory with the same
   permissions as ``.bzr`` directory on a POSIX OS.
   (Parth Malwankar, #262450)
 
-* Don't traceback trying to unversion children files of an already
-  unversioned directory.  (Vincent Ladeuil, #494221)
+* CommitBuilder now uses the committer instead of _config.username to generate
+  the revision-id.  (Aaron Bentley, #614404)
+
+* Configuration files in ``${BZR_HOME}`` are now written in an atomic
+  way which should help avoid problems with concurrent writers.
+  (Vincent Ladeuil, #525571)
+
+* CommitBuilder now uses the committer instead of _config.username to generate
+  the revision-id.  (Aaron Bentley, #614404)
+
+* Configuration files in ``${BZR_HOME}`` are now protected against
+  concurrent writers by using a lock. (Vincent Ladeuil, #525571)
+
+* Cope with Microsoft FTP Server and VSFTPd that return reply '250
+  Directory created' when mkdir succeeds.  (Martin Pool, #224373)
+
+* `decode` parameter to get() method in FtpTransport and GioTransport classes
+  is deprecated. (Alexander Belchenko)
+
+* Don't print internal object name when print an invalid revision spec
+  error.  (Neil Martinsen-Burrell, #598701)
+
+* Don't traceback when a lockdir's ``held/info`` file is corrupt (e.g.
+  contains only NUL bytes).  Instead warn the user, and allow ``bzr
+  break-lock`` to remove it.  (Andrew Bennetts, #619872)
+  
+* ``EPIPE`` can be raised during test server shutdown. This happened on
+  gentoo only so far. (Vincent Ladeuil, #627277)
+
+* Errors occurring during HTTP(S) test server starts should now be
+  handled cleanly. (Vincent Ladeuil, #392402)
+
+* Fix ``AttributeError on parent.children`` when adding a file under a 
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Fix ``AttributeError: 'NoneType' object has no attribute 'close'`` in
+  ``_close_ssh_proc`` when using ``bzr+ssh://``.  This was causing
+  connections to pre-1.6 bzr+ssh servers to fail, and causing warnings on
+  stderr in some other circumstances.  (Andrew Bennetts, #633745)
+
+* Fix spurious paramiko warning on hardy by ensuring that ``selftest``
+  properly remove its warning filter. (Vincent Ladeuil, #625686)
+
+* Fix ``AttributeError on parent.children`` when adding a file under a
+  directory that was a symlink in the previous commit.
+  (Martin Pool, #192859)
+
+* Fix ``AttributeError: 'NoneType' object has no attribute 'close'`` in
+  ``_close_ssh_proc`` when using ``bzr+ssh://``.  This was causing
+  connections to pre-1.6 bzr+ssh servers to fail, and causing warnings on
+  stderr in some other circumstances.  (Andrew Bennetts, #633745)
+
+* Fix a problem in bzr's ``Makefile`` that meant syntax errors in some
+  parts of bzr's source code could cause ``make check-nodocs`` to
+  incorrectly return an exit code of 0.
+  (Vincent Ladeuil, #626667)
+
+* ``HTTP/1.1`` test servers now set a ``Content-Length`` header to comply
+  with pedantic ``HTTP/1.1`` clients. (Vincent Ladeuil, #568421)
+
+* Most of the leaked threads during selftest are now fixed, allowing the
+  full test suite to pass on gentoo.
+  (Vincent Ladeuil, #392127)
+
+* Only call ``setlocale`` in the bzr startup script on posix systems. This
+  avoids an issue with the newer windows C runtimes used by Python 2.6 and
+  later which can mangle bytestrings printed to the console.
+  (Martin [gz], #631350)
+
+* `PathNotChild` should not give a traceback.
+  (Martin Pool, #98735)
 
 * Prevent ``CHKMap.apply_delta`` from generating non-canonical CHK maps,
   which can result in "missing referenced chk root keys" errors when
@@ -1682,9 +1044,6 @@
 * Raise ValueError instead of a string exception.
   (John Arbash Meinel, #586926)
 
-* Reduce peak memory by one copy of compressed text.
-  (John Arbash Meinel, #566940)
-
 * Repositories accessed via a smart server now reject being stacked on a
   repository in an incompatible format, as is the case when accessing them
   via other methods.  This was causing fetches from those repositories via
@@ -1696,11557 +1055,175 @@
   python merged the end of run patch, which chose ``stopTestRun`` rather than
   ``done``. (Robert Collins, #571437)
 
+* Stop ``AttributeError: 'module' object has no attribute 'ElementTree'``
+  being thrown from ``xml_serializer`` on certain cElementTree setups.
+  (Martin [gz], #254278)
+
+* strace test-helper tests cope with the new Ubuntu policy of not allowing
+  users to attach to their own processes by default.
+  (Martin Pool, #626679)
+
+* Test classes like ``TestCase``, ``TestLoader``, and ``TestSuite`` should
+  be available from ``bzrlib.tests.*``. They used to be, but were
+  accidentally removed. (John Arbash Meinel, #627438)
+
+* ``Transport.stat`` on a symlink, including a transport pointing directly
+  to a symlink, now returns information about the symlink.
+  (Martin Pool)
+
+* Upgrading or fetching from a non-rich-root repository to a rich-root
+  repository (e.g. from pack-0.92 to 2a) no longer fails with
+  ``'Inter1and2Helper' object has no attribute 'source_repo'``.  This was
+  a regression from Bazaar 2.1.  (Andrew Bennetts, #636930)
+  
+Documentation
+*************
+
+* Added a builder/writer sphinx extension that can generate texinfo files. The
+  generated files are syntactically correct but the info navigation nodes
+  needs more work. (Vincent Ladeuil, #219334)
+
+* First tests defined for sphinx, including a new bzrlib.tests.features.sphinx
+  to make the tests conditional.
+  (Vincent Ladeuil)
+
+* Fix a lot of references in the docs to the old http://bazaar-vcs.org to
+  the new http://bazaar.canonical.com or http://wiki.bazaar.canonical.com
+  (John Arbash Meinel, #617503)
+
+API Changes
+***********
+
 * When passing a file to ``UTF8DirReader`` make sure to close the current
   directory file handle after the chdir fails. Otherwise when passing many
   filenames into a command line ``bzr status`` we would leak descriptors.
   (John Arbash Meinel, #583486)
 
+* `ControlDirFormat` and  `ControlDir` have been split out of `BzrDirFormat`
+  and `BzrDir`, respectively. `ControlDirFormat`
+  and `ControlDir` should be used as the base classes for new non-.bzr
+  implementations.
+
+  `BzrDirFormat.register_control_format` has been renamed to
+  `ControlDirFormat.register_format`.
+
+  `BzrDirFormat.register_server_control_format` has been removed.
+
+  Probing for control directories is now done by separate objects derived
+  from `bzrlib.controldir.Prober` and registered using
+  `bzrlib.controldir.ControlDirFormat.register_prober` or
+  `bzrlib.controldir.ControlDirFormat.register_server_prober`.
+  `BzrDirFormat.probe_transport` has been moved onto `Prober`.
+
+  `BzrDirFormat.register_format` has been renamed to
+  `BzrProber.register_bzrdir_format`.
+
+  `bzrlib.bzrdir.network_format_registry` has been moved to
+  `bzrlib.controldir`.
+
+  (Jelmer Vernooij)
+
+* ``bzrlib.transform.TreeTransformBase.final_kind``,
+  ``bzrlib.transform.TreeTransform.tree_kind`` and
+  ``bzrlib.transform.TransformPreview.tree_kind`` now return None instead
+  of raising NoSuchFile.  (Vincent Ladeuil)
+
+* BzrError subclasses no longer support the name "message" to be used
+  as an argument for __init__ or in _fmt format specification as this
+  breaks in some Python versions. errors.LockError.__init__ argument
+  is now named "msg" instead of earlier "message".
+  (Parth Malwankar, #603461)
+
+* Configuration files should now use the ``from_string`` constructor rather
+  than the ``file`` parameter of the ``_get_parser`` method. The later has
+  been deprecated. ``from_string`` also accept a ``save=True`` parameter to
+  have the configuration file immediately written to disk. 
+  (Vincent Ladeuil)
+
+* Deprecate treating a `PushResult` and `PullResult` as an integer for the
+  relative change in revno.
+  (Martin Pool)
+
+* `FileInWrongBranch` is deprecated in favour of `PathNotChild` and no
+  longer raised.
+  (Martin Pool)
+
+* ``IniBaseConfig`` objects should now use the ``from_string`` constructor
+  the rather than the ``file`` parameter of the ``_get_parser`` method. The
+  later has been deprecated. (Vincent Ladeuil)
+
+* InventoryEntry instances now raise AttributeError if you try to assign
+  to attributes that are irrelevant to that kind of entry.  e.g. setting
+  ``symlink_target`` on an InventoryFile will fail.  It is still okay to
+  read those attributes on any kind of InventoryEntry.  The complete list
+  of affected attributes is: ``executable``, ``text_id``, ``text_sha1``,
+  ``text_size`` (only valid for kind == file); ``symlink_target`` (only
+  valid for kind == link); and ``reference_revision`` (only valid for kind
+  == tree-reference).  (Andrew Bennetts)
+
+* InventoryEntry objects no longer have ``_put_in_tar`` or
+  ``_put_on_disk`` methods.  (Andrew Bennetts)
+
+* The ``get_filename`` parameter in the ``config.IniBaseConfig``
+  constructor has been deprecated, use the ``file_name`` parameter instead.
+  (Vincent Ladeuil)
+
+* `tree_files` and `internal_tree_files` are now deprecated in favor of
+  `WorkingTree.open_containing_paths`.
+  (Martin Pool)
+
+Internals
+*********
+
+* Remove used and broken code path in ``BranchInitHookParams.__repr__``.
+  (Andrew Bennetts)
 
 Testing
 *******
 
+* Avoid spurious failures in ssh tests: wait for the SSH server to
+  actually finish, rather than just waiting for it to negotiate the key
+  exchange. (John Arbash Meinel, #626876)
+
 * ``build_tree_contents`` can create symlinks.
   (Martin Pool, John Arbash Meinel)
 
-
-bzr 2.0.5
-#########
-
-:2.0.5: 2010-03-23
-
-This fifth release in our 2.0 series addresses several user-inconvenience
-bugs.  None are critical, but upgrading is recommended for all users on
-earlier 2.0 releases.
-
-Bug Fixes
-*********
-
-* Avoid ``malloc(0)`` in ``patiencediff``, which is non-portable.
-  (Martin Pool, #331095)
-
-* Concurrent autopacking is more resilient to already-renamed pack files.
-  If we find that a file we are about to obsolete is already obsoleted, we
-  do not try to rename it, and we leave the file in ``obsolete_packs``.
-  The code is also fault tolerant if a file goes missing, assuming that
-  another process already removed the file.
-  (John Arbash Meinel, Gareth White, #507557)
-
-* Cope with the lockdir ``held/info`` file being empty, which seems to
-  happen fairly often if the process is suddenly interrupted while taking
-  a lock.
-  (Martin Pool, #185103)
-
-* Give the warning about potentially slow cross-format fetches much
-  earlier on in the fetch operation.  Don't show this message during
-  upgrades, and show the correct format indication for remote
-  repositories.
-  (Martin Pool, #456077, #515356, #513157)
-
-* Handle renames correctly when there are files or directories that 
-  differ only in case.  (Chris Jones, Martin Pool, #368931)
-
-* If ``bzr push --create-prefix`` triggers an unexpected ``NoSuchFile``
-  error, report that error rather than failing with an unhelpful
-  ``UnboundLocalError``.
-  (Andrew Bennetts, #423563)
-
-* Running ``bzr`` command without any arguments now shows bzr
-  version number along with rest of the help text.
-  (Parth Malwankar, #369501)
-
-* Use osutils.O_NOINHERIT for some files on win32 to avoid PermissionDenied
-  errors.
-  (Inada Naoki, #524560)
-
-Documentation
-*************
-
-* Added ``location-alias`` help topic.
-  (Andrew Bennetts, #337834)
-
-* Fixed CHM generation by moving the NEWS section template into
-  a separate file. (Ian Clatworthy, #524184)
-
-
-bzr 2.0.4
-#########
-
-:Codename: smooth sailing
-:2.0.4: 2010-01-21
-
-The fourth bugfix-only release in the 2.0 series contains more than a
-dozen bugfixes relative to 2.0.3. The primary focus is on handling
-interruptions and concurrent operations more cleanly, there is also a fair
-improvement to ``bzr export`` when exporting a remote branch.
-
-
-Bug Fixes
-*********
-
-* ``bzr annotate`` on another branch with ``-r branch:...`` no longer
-  fails with an ``ObjectNotLocked`` error.  (Andrew Bennetts, #496590)
-
-* ``bzr export dir`` now requests all file content as a record stream,
-  rather than requsting the file content one file-at-a-time. This can make
-  exporting over the network significantly faster (54min => 9min in one
-  case). (John Arbash Meinel, #343218)
-
-* ``bzr serve`` no longer slowly leaks memory. The compiled
-  ``bzrlib.bencode.Encoder()`` class was using ``__del__`` to cleanup and
-  free resources, and it should have been using ``__dealloc__``.
-  This will likely have an impact on any other process that is serving for
-  an extended period of time.  (John Arbash Meinel, #494406)
-
-* Check for SIGINT (Ctrl-C) and other signals immediately if ``readdir``
-  returns ``EINTR`` by calling ``PyErr_CheckSignals``.  This affected the
-  optional ``_readdir_pyx`` extension.  (Andrew Bennetts, #495023)
-
-* Concurrent autopacks will no longer lose a newly created pack file.
-  There was a race condition, where if the reload happened at the right
-  time, the second packer would forget the name of the newly added pack
-  file. (John Arbash Meinel, Gareth White, #507566)
-
-* Give a clearer message if the lockdir disappears after being apparently
-  successfully taken.  (Martin Pool, #498378)
-
-* Give a warning when fetching between repositories (local or remote) with
-  sufficiently different formats that the content will need to be
-  serialized (ie ``InterDifferingSerializer`` or ``inventory-deltas``), so
-  the user has a clue that upgrading could make it faster.
-  (Martin Pool, #456077)
-
-* If we fail to open ``~/.bzr.log`` write a clear message to stderr rather
-  than using ``warning()``. The log file is opened before logging is set
-  up, and it leads to very confusing: 'no handlers for "bzr"' messages for
-  users, rather than something nicer.
-  (John Arbash Meinel, Barry Warsaw, #503886)
-
-* Refuse to build with any Pyrex 0.9.4 release, as they have known bugs.
-  (Martin Pool, John Arbash Meinel, #449372)
-
-* ``setup.py bdist_rpm`` now properly finds extra files needed for the
-  build. (there is still the distutils bug
-  http://bugs.python.org/issue644744) (Joe Julian, #175839)
-
-* The 2a format wasn't properly restarting autopacks when something
-  changed underneath it (like another autopack). Now concurrent
-  autopackers will properly succeed. (John Arbash Meinel, #495000)
-
-* ``TreeTransform`` can now handle when a delta says that the file id for
-  the tree root changes. Rather than trying to rename your working
-  directory, or failing early saying that you can't have multiple
-  tree roots. This also fixes revert, update, and pull when the root id
-  changes.  (John Arbash Meinel, #494269, #504390)
-
-* ``_update_current_block`` no longer suppresses exceptions, so ^C at just
-  the right time will get propagated, rather than silently failing to move
-  the block pointer. (John Arbash Meinel, Gareth White, #495023)
-
-Testing
-*******
-
-* We have a new ``test_source`` that ensures all pyrex ``cdef`` functions
-  handle exceptions somehow. (Possibly by setting ``# cannot_raise``
-  rather than an ``except ?:`` clause.) This should help prevent bugs like
-  bug #495023. (John Arbash Meinel)
-
-
-bzr 2.1.0b4
-###########
-
-:Codename: san francisco airport
-:2.1.0b4: 2009-12-14
-
-The fourth beta release in the 2.1 series brings with it a significant
-number of bugfixes (~20). The test suite is once again (finally) "green"
-on Windows, and should remain that way for future releases. There are a
-few performance related updates (faster upgrade and log), and several UI
-tweaks. There has also been a significant number of tweaks to the runtime
-documentation. 2.1.0b4 include everything from the 2.0.3 release.
-
-
-Compatibility Breaks
-********************
-
-* The BZR_SSH environmental variable may now be set to the path of a secure
-  shell client. If currently set to the value ``ssh`` it will now guess the
-  vendor of the program with that name, to restore the old behaviour that
-  indicated the SSH Corporation client use ``sshcorp`` instead as the magic
-  string. (Martin <gzlist@googlemail.com>, #176292)
-
-New Features
-************
-
-* ``bzr commit`` now has a ``--commit-time`` option.
-  (Alexander Sack, #459276)
-
-* ``-Dhpss`` now increases logging done when run on the bzr server,
-  similarly to how it works on the client. (John Arbash Meinel)
-
-* New option ``bzr unshelve --keep`` applies the changes and leaves them
-  on the shelf.  (Martin Pool, Oscar Fuentes, #492091)
-
-* The ``BZR_COLUMNS`` envrionment variable can be set to force bzr to
-  respect a given terminal width. This can be useful when output is
-  redirected or in obscure cases where the default value is not
-  appropriate. Pagers can use it to get a better control of the line
-  lengths. 
-  (Vincent Ladeuil)
-
-* The new command ``bzr lp-mirror`` will request that Launchpad update its
-  mirror of a local branch. This command will only function if launchpadlib
-  is installed.
-  (Jonathan Lange)
-
-
-Bug Fixes
-*********
-
-* After renaming a file, the dirstate could accidentally reference
-  ``source\\path`` rather than ``source/path`` on Windows. This might be a
-  source of some dirstate-related failures. (John Arbash Meinel)
-
-* ``bzr commit`` now detects commit messages that looks like file names
-  and issues a warning.
-  (Gioele Barabucci, #73073)
-
-* ``bzr ignore /`` no longer causes an IndexError. (Gorden Tyler, #456036)
-
-* ``bzr log -n0 -rN`` should not return revisions beyond its merged revisions.
-  (#325618, #484109, Marius Kruger)
-
-* ``bzr merge --weave`` and ``--lca`` will now create ``.BASE`` files for
-  files with conflicts (similar to ``--merge3``). The contents of the file
-  is a synthesis of all bases used for the merge.
-  (John Arbash Meinel, #40412)
-
-* ``bzr mv --quiet`` really is quiet now.  (Gordon Tyler, #271790)
-
-* ``bzr serve`` is more clear about the risk of supplying --allow-writes.
-  (Robert Collins, #84659)
-
-* ``bzr serve --quiet`` really is quiet now.  (Gordon Tyler, #252834)
-
-* Fix bug with redirected URLs over authenticated HTTP.
-  (Glen Mailer, Neil Martinsen-Burrell, Vincent Ladeuil, #395714)
-
-* Interactive merge doesn't leave branch locks behind.  (Aaron Bentley)
-
-* Lots of bugfixes for the test suite on Windows. We should once again
-  have a test suite with no failures on Windows. (John Arbash Meinel)
-
-* ``osutils.terminal_width()`` obeys the BZR_COLUMNS environment
-  variable but returns None if the terminal is not a tty (when output is
-  redirected for example). Also fixes its usage under OSes that doesn't
-  provide termios.TIOCGWINSZ. Make sure the corresponding tests runs on
-  windows too.
-  (Joke de Buhr, Vincent Ladeuil, #353370, #62539)
-  (John Arbash Meinel, Vincent Ladeuil, #492561)
-
-* Terminate ssh subprocesses when no references to them remain, fixing
-  subprocess and file descriptor leaks.  (Andrew Bennetts, #426662)
-  
-* The ``--hardlink`` option of ``bzr branch`` and ``bzr checkout`` now
-  works for 2a format trees.  Only files unaffected by content filters
-  will be hardlinked.  (Andrew Bennetts, #408193)
-
-* The new glob expansion on Windows would replace all ``\`` characters
-  with ``/`` even if it there wasn't a glob to expand, the arg was quoted,
-  etc. Now only change slashes if there is something being glob expanded.
-  (John Arbash Meinel, #485771)
-
-* Use our faster ``KnownGraph.heads()`` functionality when computing the
-  new rich-root heads. This can cut a conversion time in half (mysql from
-  13.5h => 6.2h) (John Arbash Meinel, #487632)
-
-* When launching a external diff tool via bzr diff --using, temporary files
-  are no longer created, rather, the path to the file in the working tree is
-  passed to the external diff tool. This allows the file to be edited if the
-  diff tool provides for this. (Gary van der Merwe, #490738)
-  
-* The launchpad-open command can now be used from a subdirectory of a
-  branch, not just from the root of the branch. 
-  (Neil Martinsen-Burrell, #489102)
-
-
-Improvements
-************
-
-* ``bzr log`` is now faster. (Ian Clatworthy)
-
-* ``bzr update`` provides feedback on which branch it is up to date with.
-  (Neil Martinsen-Burrell)
-
-* ``bzr upgrade`` from pre-2a to 2a can be significantly faster (4x).
-  For details see the xml8 patch and heads() improvements.
-  (John Arbash Meinel)
-
-* ``bzrlib.urlutils.local_path_from_url`` now accepts
-  'file://localhost/' as well as 'file:///' URLs on POSIX.  (Michael
-  Hudson)
-
-* The progress bar now shows only a spinner and per-operation counts,
-  not an overall progress bar.  The previous bar was often not correlated
-  with real overall operation progress, either because the operations take
-  nonlinear time, or because at the start of the operation Bazaar couldn't
-  estimate how much work there was to do.  (Martin Pool)
-
-Documentation
-*************
-
-* Lots of documentation tweaks for inline help topics and command help
-  information.
-
-API Changes
-***********
-
-* ``bzrlib.textui`` (vestigial module) removed.  (Martin Pool)
-
-* The Launchpad plugin now has a function ``login`` which will log in to
-  Launchpad with launchpadlib, and ``load_branch`` which will return the
-  Launchpad Branch object corresponding to a given Bazaar Branch object.
-  (Jonathan Lange)
-
-Internals
-*********
-
-* New test Feature: ``ModuleAvailableFeature``. It is designed to make it
-  easier to handle what tests you want to run based on what modules can be
-  imported. (Rather than lots of custom-implemented features that were
-  basically copy-and-pasted.) (John Arbash Meinel)
-
-* ``osutils.timer_func()`` can be used to get either ``time.time()`` or
-  ``time.clock()`` when you want to do performance timing.
-  ``time.time()`` is limited to 15ms resolution on Windows, but
-  ``time.clock()`` gives CPU and not wall-clock time on other platforms.
-  (John Arbash Meinel)
-
-* Several code paths that were calling ``Transport.get().read()`` have
-  been changed to the equalivent ``Transport.get_bytes()``. The main
-  difference is that the latter will explicitly call ``file.close()``,
-  rather than expecting the garbage collector to handle it. This helps
-  with some race conditions on Windows during the test suite and sftp
-  tests. (John Arbash Meinel)
-
-Testing
-*******
-
-* TestCaseWithMemoryTransport no longer sets $HOME and $BZR_HOME to
-  unicode strings. (Michael Hudson, #464174)
-
-
-bzr 2.0.3
-#########
-
-:Codename: little italy
-:2.0.3: 2009-12-14
-
-
-The third stable release of Bazaar has a small handful of bugfixes. As
-expected, this has no internal or external compatibility changes versus
-2.0.2 (or 2.0.0).
-
-Bug Fixes
-*********
-
-* ``bzr push --use-existing-dir`` no longer crashes if the directory
-  exists but contains an invalid ``.bzr`` directory.
-  (Andrew Bennetts, #423563)
-
-* Content filters are now applied correctly after pull, merge and switch.
-  (Ian Clatworthy, #385879)
-
-* Fix a potential segfault in the groupcompress hash map handling code.
-  When inserting new entries, if the final hash bucket was empty, we could
-  end up trying to access if ``(last_entry+1)->ptr == NULL``.
-  (John Arbash Meinel, #490228)
-
-* Improve "Binary files differ" hunk handling.  (Aaron Bentley, #436325)
-
-
-bzr 2.1.0b3
-###########
-
-:Codename: after sprint recovery
-:2.1.0b3: 2009-11-16
-
-This release was pushed up from its normal release cycle due to a
-regression in python 2.4 compatibility in 2.1.0b2.  Since this regression
-was caught before 2.1.0b2 was officially announced, the full changelog
-includes both 2.1.0b3 and 2.1.0b2 changes.
-
-Highlights of 2.1.0b3 are: new globbing code for all commands on Windows,
-the test suite now conforms to python's trunk enhanced semantics (skip,
-etc.), and ``bzr info -v`` will now report the correct branch and repo
-formats for Remote objects.
-
-
-New Features
-************
-
-* Users can define a shelve editor to provide shelf functionality at a
-  granularity finer than per-patch-hunk. (Aaron Bentley)
-
-Bug Fixes
-*********
-
-* Fix for shell completion and short options.  (Benoît PIERRE)
-
-* Fix ``bzr --profile-imports`` with Python 2.6.  (Martin Pool)
-
-* Hooks daughter classes should always call the base constructor.
-  (Alexander Belchenko, Vincent Ladeuil, #389648) 
-
-* Improve "Binary files differ" hunk handling.  (Aaron Bentley, #436325)
-
-* On Windows, do glob expansion at the command-line level (as is usually
-  done in bash, etc.) This means that *all* commands get glob expansion
-  (bzr status, bzr add, bzr mv, etc). It uses a custom command line
-  parser, which allows us to know if a given section was quoted. It means
-  you can now do ``bzr ignore "*.py"``.
-  (John Arbash Meinel, #425510, #426410, #194450)
-
-* Sanitize commit messages that come in from the '-m' flag. We translate
-  '\r\n' => '\n' and a plain '\r' => '\n'. The storage layer doesn't
-  allow those because XML store silently translate it anyway. (The parser
-  auto-translates \r\n => \n in ways that are hard for us to catch.)
-
-* Show correct branch and repository format descriptions in 
-  ``bzr info -v`` on a smart server location.  (Andrew Bennetts, #196080)
-
-* The fix for bug #186920 accidentally broke compatibility with python
-  2.4.  (Vincent Ladeuil, #475585)
-
-* Using ``Repository.get_commit_builder().record_iter_changes()`` now
-  correctly sets ``self.inv_sha1`` to a sha1 string and
-  ``self.new_inventory`` to an Inventory instance after calling
-  ``self.finish_inventory()``. (Previously it accidently set both values
-  as a tuple on ``self.inv_sha1``. This was missed because
-  ``repo.add_revision`` ignores the supplied inventory sha1 and recomputes
-  the sha1 from the repo directly. (John Arbash Meinel)
-
-* Shelve command refuse to run if there is no real terminal.
-  (Alexander Belchenko)
-
-* Avoid unnecessarily flushing of trace file; it's now unbuffered at the
-  Python level.  (Martin Pool)
-
-Documentation
-*************
-
-* Include Japanese translations for documentation (Inada Naoki)
-
-* New API ``ui_factory.make_output_stream`` to be used for sending bulk
-  (rather than user-interaction) data to stdout.  This automatically
-  coordinates with progress bars or other terminal activity, and can be
-  overridden by GUIs.
-  (Martin Pool, 493944)
-
-Internals
-*********
-
-* Some of the core groupcompress functionality now releases the GIL before
-  operation. Similar to how zlib and bz2 operate without the GIL in the
-  core compression and decompression routines. (John Arbash Meinel)
-
-Testing
-*******
-
-* -Dhpssvfs will now trigger on ``RemoteBzrDir._ensure_real``, providing
-  more debugging of VFS access triggers. (Robert Collins)
-
-* KnownFailure is now signalled to ``ExtendedTestResult`` using the same
-  method that Python 2.7 uses - ``addExpectedFailure``. (Robert Collins)
-
-* ``--parallel=fork`` is now compatible with --subunit.
-  (Robert Collins, Vincent Ladeuil, #419776)
-
-* Reporting of failures shows test ids not descriptions and thus shows
-  parameterised tests correctly. (Robert Collins)
-
-* TestNotApplicable is now handled within the TestCase.run method rather
-  than being looked for within ``ExtendedTestResult.addError``. This
-  provides better handling with other ``TestResult`` objects, degrading to
-  sucess rather than error. (Robert Collins)
-
-* The private method ``_testConcluded`` on ``ExtendedTestResult`` has been
-  removed - it was empty and unused. (Robert Collins)
-
-* UnavailableFeature is now handled within the TestCase.run method rather
-  than being looked for within addError. If the Result object does not
-  have an addNotSupported method, addSkip is attempted instead, and
-  failing that addSuccess. (Robert Collins)
-
-* When a TestResult does not have an addSkip method, skipped tests are now
-  reported as successful tests, rather than as errors. This change is
-  to make it possible to get a clean test run with a less capable
-  TestResult. (Robert Collins)
-
-
-
-bzr 2.1.0b2
-###########
-
-:Codename: a load off my mind
-:2.1.0b2: 2009-11-02
-
-This is our second feature-filled release since 2.0, pushing us down the
-path to a 2.1.0. Once again, all bugfixes in 2.0.2 are present in 2.1.0b2.
-
-Key highlights in this release are: improved handling of
-failures-during-cleanup for commit, fixing a long-standing bug with
-``bzr+http`` and shared repositories, all ``lp:`` urls to be resolved
-behind proxies, and a new StaticTuple datatype, allowing us to reduce
-memory consumption (50%) and garbage collector overhead (40% faster) for
-many operations.
-
-* A new ``--concurrency`` option has been added as well as an associated
-  BZR_CONCURRENCY environment variable to specify the number of
-  processes that can be run concurrently when running ``bzr selftest``. The
-  command-line option overrides the environment variable if both are
-  specified. If none is specified. the number of processes is obtained
-  from the OS as before.  (Matt Nordhoff, Vincent Ladeuil)
-
-Bug Fixes
-*********
-
-* ``bzr+http`` servers no longer give spurious jail break errors when
-  serving branches inside a shared repository.  (Andrew Bennetts, #348308)
-
-* Errors during commit are handled more robustly so that knock-on errors
-  are less likely to occur, and will not obscure the original error if
-  they do occur.  This fixes some causes of ``TooManyConcurrentRequests``
-  and similar errors.  (Andrew Bennetts, #429747, #243391)
-
-* Launchpad urls can now be resolved from behind proxies.
-  (Gordon Tyler, Vincent Ladeuil, #186920)
-
-* Reduce the strictness for StaticTuple, instead add a debug flag
-  ``-Dstatic_tuple`` which will change apis to be strict and raise errors.
-  This way, most users won't see failures, but developers can improve
-  internals. (John Arbash Meinel, #471193)
-
-* TreeTransform.adjust_path updates the limbo paths of descendants of adjusted
-  files.  (Aaron Bentley)
-
-* Unicode paths are now handled correctly and consistently by the smart
-  server.  (Andrew Bennetts, Michael Hudson, #458762)
-
-Improvements
-************
-
-* When reading index files, we now use a ``StaticTuple`` rather than a
-  plain ``tuple`` object. This generally gives a 20% decrease in peak
-  memory, and can give a performance boost up to 40% on large projects.
-  (John Arbash Meinel)
-
-* Peak memory under certain operations has been reduced significantly.
-  (eg, 'bzr branch launchpad standalone' is cut in half)
-  (John Arbash Meinel)
-
-Documentation
-*************
-
-* Filtered views user documentation upgraded to refer to format 2a
-  instead of pre-2.0 formats. (Ian Clatworthy)
-
-API Changes
-***********
-
-* Remove deprecated ``CLIUIFactory``.  (Martin Pool)
-
-* ``UIFactory`` now has new ``show_error``, ``show_message`` and
-  ``show_warning`` methods, which can be hooked by non-text UIs.  
-  (Martin Pool)
-
-Internals
-*********
-
-* Added ``bzrlib._simple_set_pyx``. This is a hybrid between a Set and a
-  Dict (it only holds keys, but you can lookup the object located at a
-  given key). It has significantly reduced memory consumption versus the
-  builtin objects (1/2 the size of Set, 1/3rd the size of Dict). This is
-  used as the interning structure for StaticTuple objects.
-  (John Arbash Meinel)
-
-* ``bzrlib._static_tuple_c.StaticTuple`` is now available and used by
-  the btree index parser and the chk map parser. This class functions
-  similarly to ``tuple`` objects. However, it can only point to a limited
-  collection of types.  (Currently StaticTuple, str, unicode, None, bool,
-  int, long, float, but not subclasses).  This allows us to remove it from
-  the garbage collector (it cannot be in a cycle), it also allows us to
-  intern the objects. In testing, this can reduce peak memory by 20-40%,
-  and significantly improve performance by removing objects from being
-  inspected by the garbage collector.  (John Arbash Meinel)
-
-* ``GroupCompressBlock._ensure_content()`` will now release the
-  ``zlib.decompressobj()`` when the first request is for all of the
-  content. (Previously it would only be released if you made a request for
-  part of the content, and then all of it later.) This turns out to be a
-  significant memory savings, as a ``zstream`` carries around approx 260kB
-  of internal state and buffers. (For branching bzr.dev this drops peak
-  memory from 382MB => 345MB.) (John Arbash Meinel)
-
-* When streaming content between ``2a`` format repositories, we now clear
-  caches from earlier versioned files. (So 'revisions' is cleared when we
-  start reading 'inventories', etc.) This can have a significant impact on
-  peak memory for initial copies (~200MB). (John Arbash Meinel)
-
-
-bzr 2.0.2
-#########
-
-:Codename: after the scare
-:2.0.2: 2009-11-02
-
-The second in our "let's keep the stable bugfixes flowing" series. As
-expected this has a few (~9) bugfixes relative to 2.0.1, and no major api
-changes or features.
-
-Bug Fixes
-*********
-
-* Avoid "NoneType has no attribute st_mode" error when files disappear
-  from a directory while it's being read.  (Martin Pool, #446033)
-
-* Content filters are now applied correctly after revert.
-  (Ian Clatworthy)
-
-* Diff parsing handles "Binary files differ" hunks.  (Aaron Bentley, #436325)
-
-* Fetching from stacked pre-2a repository via a smart server no longer
-  fails intermittently with "second push failed to complete".
-  (Andrew Bennetts, #437626)
-
-* Fix typos left after test_selftest refactoring.
-  (Vincent Ladeuil, Matt Nordhoff, #461149)
-
-* Fixed ``ObjectNotLocked`` errors during ``bzr log -r NNN somefile``.
-  (Andrew Bennetts, #445171)
-  
-* PreviewTree file names are not limited by the encoding of the temp
-  directory's filesystem. (Aaron Bentley, #436794)
-
-Improvements
-************
-
-* ``bzr log`` now read-locks branches exactly once, so makes better use of
-  data caches.  (Andrew Bennetts)
-
-Documentation
-*************
-
-* Filtered views user documentation upgraded to refer to format 2a
-  instead of pre-2.0 formats. (Ian Clatworthy)
-
-
-bzr 2.1.0b1
-###########
-
-:Codename: While the cat is away
-:2.1.0b1: 2009-10-14
-
-This is the first development release in the new split "stable" and
-"development" series. As such, the release is a snapshot of bzr.dev
-without creating a release candidate first. This release includes a
-fair amount of internal changes, with deprecated code being removed,
-and several new feature developments. People looking for a stable code
-base with only bugfixes should focus on the 2.0.1 release. All bugfixes
-present in 2.0.1 are present in 2.1.0b1.
-
-Highlights include support for ``bzr+ssh://host/~/homedir`` style urls,
-finer control over the plugin search path via extended BZR_PLUGIN_PATH
-syntax, visible warnings when extension modules fail to load, and improved
-error handling during unlocking.
-
-
-New Features
-************
-
-* Bazaar can now send mail through Apple OS X Mail.app. 
-  (Brian de Alwis)
-
-* ``bzr+ssh`` and ``bzr`` paths can now be relative to home directories
-  specified in the URL.  Paths starting with a path segment of ``~`` are
-  relative to the home directory of the user running the server, and paths
-  starting with ``~user`` are relative to the home directory of the named
-  user.  For example, for a user "bob" with a home directory of
-  ``/home/bob``, these URLs are all equivalent:
-
-  * ``bzr+ssh://bob@host/~/repo``
-  * ``bzr+ssh://bob@host/~bob/repo``
-  * ``bzr+ssh://bob@host/home/bob/repo``
-
-  If ``bzr serve`` was invoked with a ``--directory`` argument, then no
-  home directories outside that directory will be accessible via this
-  method.
-
-  This is a feature of ``bzr serve``, so pre-2.1 clients will
-  automatically benefit from this feature when ``bzr`` on the server is
-  upgraded.  (Andrew Bennetts, #109143)
-
-* Extensions can now be compiled if either Cython or Pyrex is available.
-  Currently Pyrex is preferred, but that may change in the future.
-  (Arkanes)
-
-* Give more control on BZR_PLUGIN_PATH by providing a way to refer to or
-  disable the user, site and core plugin directories.
-  (Vincent Ladeuil, #412930, #316192, #145612)
-
-Bug Fixes
-*********
-
-* Bazaar's native protocol code now correctly handles EINTR, which most
-  noticeably occurs if you break in to the debugger while connected to a
-  bzr+ssh server.  You can now can continue from the debugger (by typing
-  'c') and the process continues.  However, note that pressing C-\ in the
-  shell may still kill the SSH process, which is bug 162509, so you must
-  sent a signal to the bzr process specifically, for example by typing
-  ``kill -QUIT PID`` in another shell.  (Martin Pool, #341535)
-
-* ``bzr add`` in a tree that has files with ``\r`` or ``\n`` in the
-  filename will issue a warning and skip over those files.
-  (Robert Collins, #3918)
-
-* ``bzr dpush`` now aborts if uncommitted changes (including pending merges)
-  are present in the working tree. The configuration option ``dpush_strict``
-  can be used to set the default for this behavior.
-  (Vincent Ladeuil, #438158)
-
-* ``bzr merge`` and ``bzr remove-tree`` now requires --force if pending
-  merges are present in the working tree.
-  (Vincent Ladeuil, #426344)
-
-* Clearer message when Bazaar runs out of memory, instead of a ``MemoryError``
-  traceback.  (Martin Pool, #109115)
-
-* Don't give a warning on Windows when failing to import ``_readdir_pyx``
-  as it is never built. (John Arbash Meinel, #430645)
-
-* Don't restrict the command name used to run the test suite.
-  (Vincent Ladeuil, #419950)
-
-* ftp transports were built differently when the kerberos python module was
-  present leading to obscure failures related to ASCII/BINARY modes.
-  (Vincent Ladeuil, #443041)
-
-* Network streams now decode adjacent records of the same type into a
-  single stream, reducing layering churn. (Robert Collins)
-
-* PreviewTree behaves correctly when get_file_mtime is invoked on an unmodified
-  file. (Aaron Bentley, #251532)
-
-* Registry objects should not use iteritems() when asked to use items().
-  (Vincent Ladeuil, #430510)
-
-* Weave based repositories couldn't be cloned when committers were using
-  domains or user ids embedding '.sig'. Now they can.
-  (Matthew Fuller, Vincent Ladeuil, #430868)
-
-Improvements
-************
-
-* Revision specifiers can now be given in a more DWIM form, without
-  needing explicit prefixes for specifiers like tags or revision id's.
-  See ``bzr help revisionspec`` for full details.  (Matthew Fuller)
-
-* Bazaar gives a warning before exiting, and writes into ``.bzr.log``, if 
-  compiled extensions can't be loaded.  This typically indicates a
-  packaging or installation problem.  In this case Bazaar will keep
-  running using pure-Python versions, but this may be substantially
-  slower.  The warning can be disabled by setting
-  ``ignore_missing_extensions = True`` in ``bazaar.conf``.
-  See also <https://answers.launchpad.net/bzr/+faq/703>.
-  (Martin Pool, #406113, #430529)
-
-* Secondary errors that occur during Branch.unlock and Repository.unlock
-  no longer obscure the original error.  These methods now use a new
-  decorator, ``only_raises``.  This fixes many causes of
-  ``TooManyConcurrentRequests`` and similar errors.
-  (Andrew Bennetts, #429747)
-
-Documentation
-*************
-
-* Describe the new shell-like test feature. (Vincent Ladeuil)
-
-* Help on hooks no longer says 'Not deprecated' for hooks that are
-  currently supported. (Ian Clatworthy, #422415)
-
-API Changes
-***********
-
-* ``bzrlib.user_encoding`` has been removed; use
-  ``bzrlib.osutils.get_user_encoding`` instead.  (Martin Pool)
-
-* ``bzrlib.tests`` now uses ``stopTestRun`` for its ``TestResult``
-  subclasses - the same as python's unittest module. (Robert Collins)
-  
-* ``diff._get_trees_to_diff`` has been renamed to 
-  ``diff.get_trees_and_branches_to_diff``. It is now a public API, and it 
-  returns the old and new branches. (Gary van der Merwe)
-
-* ``bzrlib.trace.log_error``, ``error`` and ``info`` have been deprecated.
-  (Martin Pool)
-
-* ``MutableTree.has_changes()`` does not require a tree parameter anymore. It
-  now defaults to comparing to the basis tree. It now checks for pending
-  merges too.  ``Merger.check_basis`` has been deprecated and replaced by the
-  corresponding has_changes() calls. ``Merge.compare_basis``,
-  ``Merger.file_revisions`` and ``Merger.ensure_revision_trees`` have also
-  been deprecated.
-  (Vincent Ladeuil, #440631)
-
-* ``ProgressTask.note`` is deprecated.
-  (Martin Pool)
-
-Internals
-*********
-
-* Added ``-Drelock`` debug flag.  It will ``note`` a message every time a
-  repository or branch object is unlocked then relocked the same way.
-  (Andrew Bennetts)
-  
-* ``BTreeLeafParser.extract_key`` has been tweaked slightly to reduce
-  mallocs while parsing the index (approx 3=>1 mallocs per key read).
-  This results in a 10% speedup while reading an index.
-  (John Arbash Meinel)
-
-* The ``bzrlib.lsprof`` module has a new class ``BzrProfiler`` which makes
-  profiling in some situations like callbacks and generators easier.
-  (Robert Collins)
-
-Testing
-*******
-
-* Passing ``--lsprof-tests -v`` to bzr selftest will cause lsprof output to
-  be output for every test. Note that this is very verbose! (Robert Collins)
-
-* Setting ``BZR_TEST_PDB=1`` when running selftest will cause a pdb
-  post_mortem to be triggered when a test failure occurs. (Robert Collins)
-
-* Shell-like tests can now be written. Code in ``bzrlib/tests/script.py`` ,
-  documentation in ``developers/testing.txt`` for details.
-  (Vincent Ladeuil)
-
-* Some tests could end up with the same id, that was dormant for
-  a long time.
-  (Vincent Ladeuil, #442980)
-
-* Stop showing the number of tests due to missing features in the test
-  progress bar.  (Martin Pool)
-
-* Test parameterisation now does a shallow copy, not a deep copy of the test
-  to be parameterised. This is not expected to break external use of test
-  parameterisation, and is substantially faster. (Robert Collins)
-
-* Tests that try to open a bzr dir on an arbitrary transport will now
-  fail unless they have explicitly permitted the transport via
-  ``self.permit_url``. The standard test factories such as ``self.get_url``
-  will permit the urls they provide automatically, so only exceptional
-  tests should need to do this. (Robert Collins)
-
-* The break-in test no longer cares about clean shutdown of the child,
-  instead it is happy if the debugger starts up. (Robert  Collins)
-
-* The full test suite is expected to pass when the C extensions are not
-  present. (Vincent Ladeuil, #430749)
-
-
-bzr 2.0.1
-#########
-
-:Codename: Stability First
-:2.0.1: 2009-10-14
-
-The first of our new ongoing bugfix-only stable releases has arrived. It
-includes a collection of 12 bugfixes applied to bzr 2.0.0, but does not
-include any of the feature development in the 2.1.0 series.
-
-
-Bug Fixes
-*********
-
-* ``bzr add`` in a tree that has files with ``\r`` or ``\n`` in the
-  filename will issue a warning and skip over those files.
-  (Robert Collins, #3918)
-
-* bzr will attempt to authenticate with SSH servers that support
-  ``keyboard-interactive`` auth but not ``password`` auth when using
-  Paramiko.   (Andrew Bennetts, #433846)
-
-* Fixed fetches from a stacked branch on a smart server that were failing
-  with some combinations of remote and local formats.  This was causing
-  "unknown object type identifier 60" errors.  (Andrew Bennetts, #427736)
-
-* Fixed ``ObjectNotLocked`` errors when doing some log and diff operations
-  on branches via a smart server.  (Andrew Bennetts, #389413)
-
-* Handle things like ``bzr add foo`` and ``bzr rm foo`` when the tree is
-  at the root of a drive. ``osutils._cicp_canonical_relpath`` always
-  assumed that ``abspath()`` returned a path that did not have a trailing
-  ``/``, but that is not true when working at the root of the filesystem.
-  (John Arbash Meinel, Jason Spashett, #322807)
-
-* Hide deprecation warnings for 'final' releases for python2.6.
-  (John Arbash Meinel, #440062)
-
-* Improve the time for ``bzr log DIR`` for 2a format repositories.
-  We had been using the same code path as for <2a formats, which required
-  iterating over all objects in all revisions.
-  (John Arbash Meinel, #374730)
-
-* Make sure that we unlock the tree if we fail to create a TreeTransform
-  object when doing a merge, and there is limbo, or pending-deletions
-  directory.  (Gary van der Merwe, #427773)
-
-* Occasional IndexError on renamed files have been fixed. Operations that
-  set a full inventory in the working tree will now go via the
-  apply_inventory_delta code path which is simpler and easier to
-  understand than dirstates set_state_from_inventory method. This may
-  have a small performance impact on operations built on _write_inventory,
-  but such operations are already doing full tree scans, so no radical
-  performance change should be observed. (Robert Collins, #403322)
-
-* Retrieving file text or mtime from a _PreviewTree has good performance when
-  there are many changes.  (Aaron Bentley)
-
-* The CHK index pages now use an unlimited cache size. With a limited
-  cache and a large project, the random access of chk pages could cause us
-  to download the entire cix file many times.
-  (John Arbash Meinel, #402623)
-
-* When a file kind becomes unversionable after being added, a sensible
-  error will be shown instead of a traceback. (Robert Collins, #438569)
-
-Documentation
-*************
-
-* Improved README. (Ian Clatworthy)
-
-* Improved upgrade documentation for Launchpad branches.
-  (Barry Warsaw)
-
-
-bzr 2.0.0
-#########
-
-:2.0.0: 2009-09-22
-:Codename: Instant Karma
-
-This release of Bazaar makes the 2a (previously 'brisbane-core') format
-the default when new branches or repositories are created.  This format is
-substantially smaller and faster for many operations.  Most of the work in
-this release focuses on bug fixes and stabilization, covering both 2a and
-previous formats.  (See the Upgrade Guide for information on migrating
-existing projects.)
-
-This release also improves the documentation content and presentation,
-including adding Windows HtmlHelp manuals.
-
-The Bazaar team decided that 2.0 will be a long-term supported release,
-with bugfix-only 2.0.x releases based on it, continuing for at least six
-months or until the following stable release.
-
-Changes from 2.0.0rc2 to final
-******************************
-
-* Officially branded as 2.0.0 rather than 2.0 to clarify between things
-  that "want to happen on the 2.0.x stable series" versus things that want
-  to "land in 2.0.0". (Changes how bzrlib._format_version_tuple() handles
-  micro = 0.) (John Arbash Meinel)
-
-
-bzr 2.0.0rc2
-############
-
-:2.0.0rc2: 2009-09-10
-
-New Features
-************
-
-* Added post_commit hook for mutable trees. This allows the keywords
-  plugin to expand keywords on files changed by the commit.
-  (Ian Clatworthy, #408841)
-
-Bug Fixes
-*********
-
-* Bazaar's native protocol code now correctly handles EINTR, which most
-  noticeably occurs if you break in to the debugger while connected to a
-  bzr+ssh server.  You can now can continue from the debugger (by typing
-  'c') and the process continues.  However, note that pressing C-\ in the
-  shell may still kill the SSH process, which is bug 162509, so you must
-  sent a signal to the bzr process specifically, for example by typing
-  ``kill -QUIT PID`` in another shell.  (Martin Pool, #341535)
-
-* ``bzr check`` in pack-0.92, 1.6 and 1.9 format repositories will no
-  longer report incorrect errors about ``Missing inventory ('TREE_ROOT', ...)``
-  (Robert Collins, #416732)
-
-* ``bzr info -v`` on a 2a format still claimed that it was a "Development
-  format" (John Arbash Meinel, #424392)
-
-* ``bzr log stacked-branch`` shows the full log including
-  revisions that are in the fallback repository. (Regressed in 2.0rc1).
-  (John Arbash Meinel, #419241)
-
-* Clearer message when Bazaar runs out of memory, instead of a ``MemoryError``
-  traceback.  (Martin Pool, #109115)
-
-* Conversion to 2a will create a single pack for all the new revisions (as
-  long as it ran without interruption). This improves both ``bzr upgrade``
-  and ``bzr pull`` or ``bzr merge`` from local branches in older formats.
-  The autopack logic that occurs every 100 revisions during local
-  conversions was not returning that pack's identifier, which resulted in
-  the partial packs created during the conversion not being consolidated
-  at the end of the conversion process. (Robert Collins, #423818)
-
-* Fetches from 2a to 2a are now again requested in 'groupcompress' order.
-  Groups that are seen as 'underutilized' will be repacked on-the-fly.
-  This means that when the source is fully packed, there is minimal
-  overhead during the fetch, but if the source is poorly packed the result
-  is a fairly well packed repository (not as good as 'bzr pack' but
-  good-enough.) (Robert Collins, John Arbash Meinel, #402652)
-
-* Fix a potential segmentation fault when doing 'log' of a branch that had
-  ghosts in its mainline.  (Evaluating None as a tuple is bad.)
-  (John Arbash Meinel, #419241)
-
-* ``groupcompress`` sort order is now more stable, rather than relying on
-  ``topo_sort`` ordering. The implementation is now
-  ``KnownGraph.gc_sort``. (John Arbash Meinel)
-
-* Local data conversion will generate correct deltas. This is a critical
-  bugfix vs 2.0rc1, and all 2.0rc1 users should upgrade to 2.0rc2 before
-  converting repositories. (Robert Collins, #422849)
-
-* Network streams now decode adjacent records of the same type into a
-  single stream, reducing layering churn. (Robert Collins)
-
-* Prevent some kinds of incomplete data from being committed to a 2a
-  repository, such as revisions without inventories, a missing chk_bytes
-  record for an inventory, or a missing text referenced by an inventory.
-  (Andrew Bennetts, #423506, #406687)
-  
-Documentation
-*************
-
-* Fix assertion error about "_remember_remote_is_before" when pushing to
-  older smart servers.
-  (Andrew Bennetts, #418931)
-
-* Help on hooks no longer says 'Not deprecated' for hooks that are
-  currently supported. (Ian Clatworthy, #422415)
-
-* PDF and CHM (Windows HtmlHelp) formats are now supported for the
-  user documentation. The HTML documentation is better broken up into
-  topics. (Ian Clatworthy)
-
-* The developer and foreign language documents are now separated
-  out so that searching in the HTML and CHM files produces more
-  useful results. (Ian Clatworthy)
-
-* The main table of contents now provides links to the new Migration Docs
-  and Plugins Guide. (Ian Clatworthy)
-
-
-bzr 2.0.0rc1
-############
-
-:Codename: no worries
-:2.0.0rc1: 2009-08-26
-
-Compatibility Breaks
-********************
-
-* The default format for bzr is now ``2a``. This format brings many
-  significant performance and size improvements. bzr can pull from
-  any existing repository into a ``2a`` one, but can only transfer
-  from ``2a`` into ``rich-root`` repositories. The Upgrade guide
-  has more information about this change. (Robert Collins)
-
-* On Windows auto-detection of Putty's plink.exe is disabled.
-  Default SSH client for Windows is paramiko. User still can force
-  usage of plink if explicitly set environment variable BZR_SSH=plink.
-  (#414743, Alexander Belchenko)
-
-New Features
-************
-
-* ``bzr branch --switch`` can now switch the checkout in the current directory
-  to the newly created branch. (Lukáš Lalinský)
-
-Bug Fixes
-*********
-
-* Further tweaks to handling of ``bzr add`` messages about ignored files.
-  (Jason Spashett, #76616)
-
-* Fetches were being requested in 'groupcompress' order, but weren't
-  recombining the groups. Thus they would 'fragment' to get the correct
-  order, but not 'recombine' to actually benefit from it. Until we get
-  recombining to work, switching to 'unordered' fetches avoids the
-  fragmentation. (John Arbash Meinel, #402645)
-
-* Fix a pycurl related test failure on karmic by recognizing an error
-  raised by newer versions of pycurl.
-  (Vincent Ladeuil, #306264)
-
-* Fix a test failure on karmic by making a locale test more robust.
-  (Vincent Ladeuil, #413514)
-
-* Fix IndexError printing CannotBindAddress errors.
-  (Martin Pool, #286871)
-
-* Fix "Revision ... not present" errors when upgrading stacked branches,
-  or when doing fetches from a stacked source to a stacked target.
-  (Andrew Bennetts, #399140)
-
-* ``bzr branch`` of 2a repositories over HTTP is much faster.  bzr now
-  batches together small fetches from 2a repositories, rather than
-  fetching only a few hundred bytes at a time.
-  (Andrew Bennetts, #402657)
-
-Improvements
-************
-
-* A better description of the platform is shown in crash tracebacks, ``bzr
-  --version`` and ``bzr selftest``.
-  (Martin Pool, #409137)
-
-* bzr can now (again) capture crash data through the apport library, 
-  so that a single human-readable file can be attached to bug reports.
-  This can be disabled by using ``-Dno_apport`` on the command line, or by
-  putting ``no_apport`` into the ``debug_flags`` section of
-  ``bazaar.conf``.
-  (Martin Pool, Robert Collins, #389328)
-
-* ``bzr push`` locally on windows will no longer give a locking error with
-  dirstate based formats. (Robert Collins)
-
-* ``bzr shelve`` and ``bzr unshelve`` now work on windows.
-  (Robert Collins, #305006)
-
-* Commit of specific files no longer prevents using the iter_changes
-  codepath. On 2a repositories, commit of specific files should now be as
-  fast, or slightly faster, than a full commit. (Robert Collins)
-
-* The internal core code that handles specific file operations like
-  ``bzr st FILENAME`` or ``bzr commit FILENAME`` has been changed to
-  include the parent directories if they have altered, and when a
-  directory stops being a directory its children are always included. This
-  fixes a number of causes for ``InconsistentDelta`` errors, and permits
-  faster commit of specific paths. (Robert Collins, #347649)
-
-Documentation
-*************
-
-* New developer documentation for content filtering.
-  (Martin Pool)
-
-API Changes
-***********
-
-* ``bzrlib.shelf_ui`` has had the ``from_args`` convenience methods of its
-  classes changed to manage lock lifetime of the trees they open in a way
-  consistent with reader-exclusive locks. (Robert Collins, #305006)
-
-Testing
-*******
-
-bzr 1.18.1
-##########
-
-:Codename:     nein nein nein!
-:1.18.1:       2009-09-09
-
-This release fixes two small but worthwhile bugs relevant to users on
-Microsoft Windows: some commands that failed on with locking errors will
-now work, and a bug that caused poor performance after committing a file
-with line-ending conversion has now been fixed.  It also fixes a bug in
-pushing to older servers.
-
-Bug Fixes
-*********
-
-* Fixed a problem where using content filtering and especially end-of-line
-  conversion will commit too many copies a file.
-  (Martin Pool, #415508)
-
-* Fix assertion error about ``_remember_remote_is_before`` in
-  ``set_tags_bytes`` when pushing to older smart servers.  
-  (Andrew Bennetts, Alexander Belchenko, #418931)
-
-Improvements
-************
-
-* ``bzr push`` locally on Windows will no longer give a locking error with
-  dirstate based formats. (Robert Collins)
-
-* ``bzr shelve`` and ``bzr unshelve`` now work on Windows.
-  (Robert Collins, #305006)
-
-API Changes
-***********
-
-* ``bzrlib.shelf_ui`` has had the ``from_args`` convenience methods of its
-  classes changed to manage lock lifetime of the trees they open in a way
-  consistent with reader-exclusive locks. (Robert Collins, #305006)
-
-* ``Tree.path_content_summary`` may return a size of None, when called on
-  a tree with content filtering where the size of the canonical form
-  cannot be cheaply determined.  (Martin Pool)
-
-* When manually creating transport servers in test cases, a new helper
-  ``TestCase.start_server`` that registers a cleanup and starts the server
-  should be used. (Robert Collins)
-
-bzr 1.18
-########
-
-Compatibility Breaks
-********************
-
-* Committing directly to a stacked branch from a lightweight checkout will
-  no longer work. In previous versions this would appear to work but would
-  generate repositories with insufficient data to create deltas, leading
-  to later errors when branching or reading from the repository.
-  (Robert Collins, bug #375013)
-
-New Features
-************
-
-Bug Fixes
-*********
-
-* Fetching from 2a branches from a version-2 bzr protocol would fail to
-  copy the internal inventory pages from the CHK store. This cannot happen
-  in normal use as all 2a compatible clients and servers support the
-  version-3 protocol, but it does cause test suite failures when testing
-  downlevel protocol behaviour. (Robert Collins)
-
-* Fix a test failure on karmic by making a locale test more robust.
-  (Vincent Ladeuil, #413514)
-
-* Fixed "Pack ... already exists" error when running ``bzr pack`` on a
-  fully packed 2a repository.  (Andrew Bennetts, #382463)
-
-* Further tweaks to handling of ``bzr add`` messages about ignored files.
-  (Jason Spashett, #76616)
-
-* Properly handle fetching into a stacked branch while converting the
-  data, especially when there are also ghosts. The code was filling in
-  parent inventories incorrectly, and also not handling when one of the
-  parents was a ghost. (John Arbash Meinel, #402778, #412198)
-
-* ``RemoteStreamSource.get_stream_for_missing_keys`` will fetch CHK
-  inventory pages when appropriate (by falling back to the vfs stream
-  source).  (Andrew Bennetts, #406686)
-
-* StreamSource generates rich roots from non-rich root sources correctly
-  now.  (Andrew Bennetts, #368921)
-
-* When deciding whether a repository was compatible for upgrading or
-  fetching, we previously incorrectly checked the default repository
-  format for the bzrdir format, rather than the format that was actually
-  present on disk.  (Martin Pool, #408824)
-
-Improvements
-************
-
-* A better description of the platform is shown in crash tracebacks, ``bzr
-  --version`` and ``bzr selftest``.
-  (Martin Pool, #409137)
-
-* Cross-format fetches (such as between 1.9-rich-root and 2a) via the
-  smart server are more efficient now.  They send inventory deltas rather
-  than full inventories.  The smart server has two new requests,
-  ``Repository.get_stream_1.19`` and ``Repository.insert_stream_1.19`` to
-  support this.  (Andrew Bennetts, #374738, #385826)
-
-* Extracting the full ancestry and computing the ``merge_sort`` is now
-  significantly faster. This effects things like ``bzr log -n0``. (For
-  example, ``bzr log -r -10..-1 -n0 bzr.dev`` is 2.5s down to 1.0s.
-  (John Arbash Meinel)
-
-Documentation
-*************
-
-API Changes
-***********
-
-Internals
-*********
-
-* ``-Dstrict_locks`` can now be used to check that read and write locks
-  are treated properly w.r.t. exclusivity. (We don't try to take an OS
-  read lock on a file that we already have an OS write lock on.) This is
-  now set by default for all tests, if you have a test which cannot be
-  fixed, you can use ``self.thisFailsStrictLockCheck()`` as a
-  compatibility knob. (John Arbash Meinel)
-
-* InterDifferingSerializer is now only used locally.  Other fetches that
-  would have used InterDifferingSerializer now use the more network
-  friendly StreamSource, which now automatically does the same
-  transformations as InterDifferingSerializer.  (Andrew Bennetts)
-
-* ``KnownGraph`` now has a ``.topo_sort`` and ``.merge_sort`` member which
-  are implemented in pyrex and significantly faster. This is exposed along
-  with ``CombinedGraphIndex.find_ancestry()`` as
-  ``VersionedFiles.get_known_graph_ancestry(keys)``.
-  (John Arbash Meinel)
-
-* RemoteBranch.open now honours ignore_fallbacks correctly on bzr-v2
-  protocols. (Robert Collins)
-
-* The index code now has some specialized routines to extract the full
-  ancestry of a key in a more efficient manner.
-  ``CombinedGraphIndex.find_ancestry()``. (Time to get ancestry for
-  bzr.dev drops from 1.5s down to 300ms. For OOo from 33s => 10.5s) (John
-  Arbash Meinel)
-
-Testing
-*******
-
-* Install the test ssl certificate and key so that installed bzr
-  can run the https tests. (Denys Duchier, #392401)
-  
-
-bzr 1.18rc1
-###########
-
-:Codename: little traveller
-:1.18:    2009-08-20
-:1.18rc1: 2009-08-10
-
-This release of Bazaar marches on towards the 2.0 release in which the 2a
-'brisbane-core' format becomes generally recommended.  Most of the work in
-this release now focusses on bug fixes and stabilization, covering both 2a
-and previous formats.  There is a new text-mode interactive merge feature,
-a new guide to migration to 2a format in the user documentation, and
-pushing branches to a smart server is now much faster.  
-
-The Bazaar team decided that 2.0 will be a long-term supported release,
-with bugfix-only releases based on it continuing for at least six months
-or until the following stable release.
-
-There are no changes from 1.18rc1 to 1.18.
-
-New Features
-************
-
-* ``bzr merge --interactive`` applies a user-selected portion of the
-  merge.  The UI is similar to ``shelve``.  (Aaron Bentley)
-
-* ``bzr reconfigure`` now takes options ``--stacked-on URL`` and
-  ``--unstacked`` to change stacking of a branch.
-  (Martin Pool, #391411)
-
-Bug Fixes
-*********
-
-* Annotating on a stacked branch will now succeed in simple scenarios.
-  There are still some complex scenarios where it will fail (bug #399884)
-  (John Arbash Meinel, #393366)
-
-* A progress bar is no longer left dangling when ``bzr selftest``
-  completes, and the progress bar updates with zero latency so the
-  displayed test name is always the one that's actually running.
-  (Martin Pool, #123688)
-
-* Authenticating against an ssh server now uses ``auth_none`` to determine
-  if password authentication is even supported. This fixes a bug where
-  users would be prompted for a launchpad password, even though launchpad
-  only supports publickey authentication. (John Arbash Meinel, #375867)
-
-* BranchBuilder now accepts timezone to avoid test failures in countries far
-  from GMT. (Vincent Ladeuil, #397716)
-
-* ``bzr commit`` no longer saves the unversioning of missing files until
-  the commit has completed on the branch. This means that aborting a
-  commit that found a missing file will leave the tree unedited.
-  (Robert Collins, #282402)
-
-* ``bzr mv`` no longer takes out branch locks, which allows it to work
-  when the branch is readonly. (Robert Collins, #216541)
-
-* ``bzr revert .`` no longer generates an InconsistentDelta error when
-  there are missing subtrees. (Robert Collins, #367632)
-
-* ``bzr send`` now generates valid bundles with ``--2a`` formats. However,
-  do to internal changes necessary to support this, older clients will
-  fail when trying to insert them. For newer clients, the bundle can be
-  used to apply the changes to any rich-root compatible format.
-  (John Arbash Meinel, #393349)
-
-* Cope with FTP servers that don't support restart/append by falling back
-  to reading and then rewriting the whole file, such as TahoeLAFS.  (This
-  fallback may be slow for some access patterns.)  (Nils Durner, #294709)
-
-* Encode the paths in ``mbcs`` encoding on Windows when spawning an
-  external diff client. This at least allows supporting filenames that are
-  not ascii, but are present in the current locale. Ideally we would be
-  able to pass the Unicode path, but that would be client dependent.
-  (John Arbash Meinel, #382709)
-
-* Fix a compile bug on Solaris having to do with const and
-  pointer-to-pointers. (John Arbash Meinel, #408441)
-
-* Fixed a NameError that occurs when merging or pulling from a URL that
-  causes a redirection loop when bzr tries to read a URL as a bundle.
-  (Andrew Bennetts, #400847)
-
-* Fix ``AttributeError: 'TestUIFactory' object has no attribute 'tick'``
-  running send and similar commands on 2a formats.
-  (Martin Pool, #408201)
-  
-* Fix crash in some invocations of ``bzr status`` in format 2a.
-  (Martin Pool, #403523)
-
-* Fixed export to existing directory: if directory is empty then export 
-  will succeed, otherwise it fails with error.
-  (Alexander Belchenko, #406174)
-
-* Fixed spurious "Source branch does not support stacking" warning when
-  pushing. (Andrew Bennetts, #388908)
-
-* Fixed spurious transport activity indicator appearing while tests are
-  running.  (Martin Pool, #343532)
-
-* Merge now correctly handles empty right-hand revision specs.
-  (Aaron Bentley, #333961)
-
-* Renames to lexographically lower basenames in trees that have never been
-  committed to will no longer corrupt the dirstate. This was caused by an
-  bug in the dirstate update_minimal method. (Robert Collins, #395556)
-
-* Requests for unknown methods no longer cause the smart server to log
-  lots of backtraces about ``UnknownSmartMethod``, ``do_chunk`` or
-  ``do_end``.  (Andrew Bennetts, #338561)
-
-* Shelve will not shelve the initial add of the tree root.  (Aaron Bentley)
-
-* Streaming from bzr servers where there is a chain of stacked branches
-  (A stacked on B stacked on C) will now work. (Robert Collins, #406597)
-
-* The environment variable ``BZR_PROGRESS_BAR`` set to either ``text`` or ``none``
-  always forces progress bars either on or off respectively.  Otherwise,
-  they're turned on if ``TERM`` is not ``dumb`` and stderr is a terminal.
-  bzr always uses the 'text' user interface when run as a command, so
-  ``BZR_USE_TEXT_UI`` is no longer needed.
-  (Martin Pool, #339385, #387717)
-
-* The optional ``_knit_load_data_pyx`` C extension was never being
-  imported.  This caused significant slowdowns when reading data from
-  repositories.  (Andrew Bennetts, #405653)
-  
-* The ``--hardlink`` option to ``branch`` and ``checkout`` is not
-  supported at the moment on workingtree formats that can do content
-  filtering.  (See <https://bugs.edge.launchpad.net/bzr/+bug/408193>.)
-  bzr now says so, rather than just ignoring the option.  (Martin Pool)
-
-* There was a bug in ``osutils.relpath`` that was only triggered on
-  Windows. Essentially if you were at the root of a drive, and did
-  something to a branch/repo on another drive, we would go into an
-  infinite loop while trying to find a 'relative path'.
-  (John Arbash Meinel, #394227)
-
-* ``WorkingTree4.unversion`` will no longer fail to unversion ids which
-  were present in a parent tree but renamed in the working tree.
-  (Robert Collins, #187207)
-
-Improvements
-************
-
-* Can now rename/move files even if they have been removed from the inventory.
-  (Marius Kruger)
-
-* Pushing branches with tags via ``bzr://`` and ``bzr+ssh://`` is much
-  faster, using a new ``Branch.set_tags_bytes`` smart server verb rather
-  than VFS methods.  For example, pushes of small branches with tags take
-  11 rather than 18 smart server requests.  (Andrew Bennetts, #398608)
-
-* Sending Ctrl-Break on Windows will now drop you into the debugger, in
-  the same way that sending Ctrl-\\ does on other platforms.
-  (John Arbash Meinel)
-
-Documentation
-*************
-
-* Added Bazaar 2.0 Upgrade Guide. (Ian Clatworthy)
-
-API Changes
-***********
-
-* ``CLIUIFactory`` is deprecated; use ``TextUIFactory`` instead if you
-  need to subclass or create a specific class, or better yet the existing
-  ``make_ui_for_terminal``.  ``SilentUIFactory`` is clarified to do no
-  user interaction at all, rather than trying to read from stdin but not
-  writing any output, which would be strange if reading prompts or
-  passwords.  (Martin Pool)
-
-* New TransformPreview.commit() allows committing without a working tree.
-  (Aaron Bentley)
-
-* ``pb`` parameter to ``TextTestResult`` is deprecated and ignored.
-  (Martin Pool)
-
-* ProgressTasks now prefer to talk direct to their ProgressView not to the
-  UIFactory. 
-  (Martin Pool)
-
-* ``WorkingTree._check`` now requires a references dict with keys matching
-  those returned by ``WorkingTree._get_check_refs``. (Robert Collins)
-
-Internals
-*********
-
-* ``CHKInventory.path2id`` uses the parent_id to basename hash to avoid
-  reading the entries along the path, reducing work to lookup ids from
-  paths. (Robert Collins)
-
-* ``CHKMap.apply_delta`` now raises ``InconsistentDelta`` if a delta adds
-  as new a key which was already mapped. (Robert Collins)
-
-* Inventory delta application catches more cases of corruption and can
-  prevent corrupt deltas from affecting consistency of data structures on
-  disk. (Robert Collins)
-
-* --subunit support now adds timestamps if the subunit version supports
-  it. (Robert Collins)
-
-* The Windows all-in-one installer now bundles the PyQt image format
-  plugins, which allows previewing more images as part of 'qdiff'.
-  (Alexander Belchenko)
-
-
-Testing
-*******
-
-* Merge directive cherrypick tests must use the same root id.
-  (Martin Pool, #409684)
-
-* Spurious failure in ``check`` tests on rich-root formats fixed.
-  (Martin Pool, #408199)
-
-* The ``bzrlib.tests.TextTestRunner`` will no longer call
-  ``countTestsCases`` on the test being run. Progress information is
-  instead handled by having the test passed in call ``result.progress``
-  before running its contents. This improves the behaviour when using
-  ``TextTestRunner`` with test suites that don't support
-  ``countTestsCases``. (Robert Collins)
-
-
-bzr 1.17.1 (unreleased)
-#######################
-
-Bug Fixes
-*********
-
-* The optional ``_knit_load_data_pyx`` C extension was never being
-  imported.  This caused significant slowdowns when reading data from
-  knit format repositories.  (Andrew Bennetts, #405653)
-  
-
-bzr 1.17
-########
-:Codename: so-late-its-brunch
-:1.17rc1: 2009-07-13
-:1.17: 2009-07-20
-
-
-Bazaar continues to blaze a straight and shining path to the 2.0 release and
-the elevation of the ``2a`` beta format to the full glory of "supported and
-stable".
-
-Highlights in this release include greatly reduced memory consumption during
-commits, faster ``ls``, faster ``annotate``, faster network operations if
-you're specifying a revision number and the final destruction of those
-annoying progress bar artifacts.
-
-
-Changes from 1.17rc1 to 1.17final
-*********************************
-
-* Change an extension to call the python ``frozenset()`` rather than the C
-  api ``PyFrozenSet_New``. It turns out that python2.4 did not expose the
-  C api. (John Arbash Meinel, #399366)
-
-* Fixes for the Makefile and the rename of ``generate_docs.py`` to
-  ``tools/generate_docs.py`` to allow everything to be built on Windows.
-  (John Arbash Meinel, #399356)
-
-* ``bzr serve`` once again applies a ``ChrootServer`` to the given
-  directory before serving it. (Andrew Bennetts, #400535)
-
-
-Compatibility Breaks
-********************
-
-* ``bzr register-branch`` from the Launchpad plugin now refers to "project"
-  instead of "product" which is the correct Launchpad terminology.  The
-  --product option is deprecated and users should switch to using --project.
-  (Neil Martinsen-Burrell, #238764)
-
-
-New Features
-************
-
-* ``bzr push`` now aborts if uncommitted changes (including pending merges)
-  are present in the working tree (if one is present) and no revision is
-  specified. The configuration option ``push_strict`` can be used to set the
-  default for this behavior.  (Vincent Ladeuil, #284038, #322808, #65286)
-
-* ``bzr revno`` and ``bzr revision-info`` now have a ``--tree`` option to
-  show revision info for the working tree instead of the branch.
-  (Matthew Fuller, John Arbash Meinel)
-
-* ``bzr send`` now aborts if uncommitted changes (including pending merges)
-  are present in the working tree and no revision is specified. The
-  configuration option ``send_strict`` can be used to set the default for this
-  behavior.
-  (Vincent Ladeuil, #206577)
-
-* ``bzr switch --create-branch/-b`` can now be used to create and switch
-  to a new branch. Supplying a name without a ``/`` will create the branch
-  relative to the existing branch. (similar to how ``bzr switch name``
-  works when the branch already exists.) (John Arbash Meinel)
-
-
-Bug Fixes
-*********
-
-* Accept uppercase "Y/N" to prompts such as from break lock. 
-  (#335182, Tim Powell, Martin Pool)
-
-* Add documentation about diverged branches and how to fix them in the
-  centralized workflow with local commits.  Mention ``bzr help
-  diverged-branches`` when a push fails because the branches have
-  diverged.  (Neil Martinsen-Burrell, #269477)
-
-* Annotate would sometimes 'latch on' to trivial lines, causing important
-  lines to be incorrectly annotated. (John Arbash Meinel, #387952)
-
-* Automatic format upgrades triggered by default stacking policies on a
-  1.16rc1 (or later) smart server work again.
-  (Andrew Bennetts, #388675)
-
-* Avoid progress bar artifacts being left behind on the screen.
-  (Martin Pool, #321935)
-
-* Better message in ``bzr split`` error suggesting a rich root format.
-  (Neil Martinsen-Burrell, #220067)
-
-* ``Branch.set_append_revisions_only`` now works with branches on a smart
-  server. (Andrew Bennetts, #365865)
-
-* By default, ``bzr branch`` will fail if the target directory exists, but
-  does not already have a control directory.  The flag ``--use-existing-dir``
-  will allow operation to proceed.  (Alexander Belchenko, #307554)
-
-* ``bzr ls DIR --from-root`` now shows only things in DIR, not everything.
-  (Ian Clatworthy)
-
-* Fetch between repositories does not error if they have inconsistent data
-  that should be irrelevant to the fetch operation. (Aaron Bentley)
-
-* Fix ``AttributeError`` exception when reconfiguring lightweight checkout 
-  of a remote repository.
-  (Jelmer Vernooij, #332194)
-
-* Fix bug in decoding v3 smart server messages when receiving multiple
-  lots of excess bytes after an end-of-message.
-  (Andrew Bennetts)
-
-* Force deletion of readonly files during merge, update and other tree
-  transforms.
-  (Craig Hewetson, Martin Pool, #218206)
-
-* Force socket shutdown in threaded http test servers to avoid client hangs
-  (pycurl).  (Vincent Ladeuil, #383920).
-
-* ``LRUCache`` will maintain the linked list pointers even if a nodes
-  cleanup function raises an exception. (John Arbash Meinel, #396838)
-
-* Progress bars are now suppressed again when the environment variable
-  ``BZR_PROGRESS_BAR`` is set to ``none``.
-  (Martin Pool, #339385)
-
-* Reduced memory consumption during ``bzr commit`` of large files. For
-  pre 2a formats, should be down to ~3x the size of a file.
-  For ``--2a`` format repositories, it is down to the size of the file
-  content plus the size of the compressed text.  Related to bug #109114.
-  (John Arbash Meinel)
-
-* Set hidden attribute on .bzr directory below unicode path should never
-  fail with error. The operation should succeed even if bzr unable to set 
-  the attribute.  (Alexander Belchenko, related to bug #335362).
-  
-* Stacking will no longer accept requests to stack on the same
-  branch/repository. Existing branches that incorrectly reference the same
-  repository in a stacking configuration will now raise
-  UnstackableLocationError when the branch is opened. This can be fixed by
-  removing the stacking location inside ``.bzr/branch``.
-  (Robert Collins, #376243)
-
-* The ``log+`` decorator, useful in debugging or profiling, could cause
-  "AttributeError: 'list' object has no attribute 'next'".  This is now
-  fixed.  The log decorator no longer shows the elapsed time or transfer
-  rate because they're available in the log prefixes and the transport
-  activity display respectively.
-  (Martin Pool, #340347)
-
-* Unshelve works correctly when multiple zero-length files are present on
-  the shelf. (Aaron Bentley, #363444)
-
-* Progress bars no longer show the network transport scheme or direction.
-  (Martin Pool)
-
-* launchpad-login now respects the 'verbose' option.
-  (Jonathan Lange, #217031)
-
-
-Internals
-*********
-
-* ``bzrlib.user_encoding`` is now officially deprecated. It is not
-  possible to write a deprecation wrapper, but the variable will be
-  removed in the near future. Use ``bzrlib.osutils.get_user_encoding()``
-  instead. (Alexander Belchenko)
-
-* Command lookup has had hooks added. ``bzrlib.Command.hooks`` has
-  three new hook points: ``get_command``, ``get_missing_command`` and
-  ``list_commands``, which allow just-in-time command name provision
-  rather than requiring all command names be known a-priori.
-  (Robert Collins)
-
-* ``get_app_path`` from win32utils.py now supports REG_EXPAND_SZ data type
-  and can read path to wordpad.exe. (Alexander Belchenko, #392046)
-
-* ``graph.KnownGraph`` has been added. This is a class that can give
-  answers to ``heads()`` very quickly. However, it has the assumption that
-  the whole graph has already been loaded. This is true during
-  ``annotate`` so it is used there with good success (as much as 2x faster
-  for files with long ancestry and 'cherrypicked' changes.)
-  (John Arbash Meinel, Vincent Ladeuil)
-
-* OS file locks are now taken out using ``CreateFile`` rather than
-  ``LockFileEx`` on Windows. The locking remains exclusive with
-  ``LockFileEx`` but now it also works on older versions of Windows (such
-  as Win98). (Martin <gzlist>)
-
-* pack <=> pack fetching is now done via a ``PackStreamSource`` rather
-  than the ``Packer`` code. The user visible change is that we now
-  properly fetch the minimum number of texts for non-smart fetching.
-  (John Arbash Meinel)
-
-
-* ``VersionedFiles._add_text`` is a new api that lets us insert text into
-  the repository as a single string, rather than a list of lines. This can
-  improve memory overhead and performance of committing large files.
-  (Currently a private api, used only by commit). (John Arbash Meinel)
-
-
-Improvements
-************
-
-* ``bzr annotate`` can now be significantly faster. The time for
-  ``bzr annotate NEWS`` is down to 7s from 22s in 1.16. Files with long
-  histories and lots of 'duplicate insertions' will be improved more than
-  others. (John Arbash Meinel, Vincent Ladeuil)
-
-* ``bzr ls`` is now faster. On OpenOffice.org, the time drops from 2.4
-  to 1.1 seconds. The improvement for ``bzr ls -r-1`` is more
-  substantial dropping from 54.3 to 1.1 seconds. (Ian Clatworthy)
-
-* Improve "Path(s) are not versioned" error reporting for some commands.
-  (Benoît PIERRE)
-
-* Initial commit performance in ``--2a`` repositories has been improved by
-  making it cheaper to build the initial CHKMap. (John Arbash Meinel)
-
-* Resolving a revno to a revision id on a branch accessed via ``bzr://``
-  or ``bzr+ssh://`` is now much faster and involves no VFS operations.
-  This speeds up commands like ``bzr pull -r 123``.  (Andrew Bennetts)
-
-* ``revision-info`` now properly aligns the revnos/revids in the output
-  and doesn't traceback when given revisions not in the current branch.
-  Performance is also significantly improved when requesting multiple revs
-  at once.  (Matthew Fuller, John Arbash Meinel)
-
-* Tildes are no longer escaped by Transports. (Andy Kilner)
-
-
-Documentation
-*************
-
-* Avoid bad text wrapping in generated documentation.  Slightly better
-  formatting in the user reference.
-  (Martin Pool, #249908)
-
-* Minor clarifications to the help for End-Of-Line conversions.
-  (Ian Clatworthy)
-
-API Changes
-***********
-
-* Removed overspecific error class ``InvalidProgressBarType``.
-  (Martin Pool)
-
-* The method ``ProgressView._show_transport_activity`` is now
-  ``show_transport_activity`` because it's part of the contract between
-  this class and the UI.  (Martin Pool)
-
-
-bzr 1.16.1
-##########
-
-:Released: 2009-06-26
-
-End user testing of the 2a format revealed two serious bugs. The first,
-#365615, caused bzr to raise AbsentContentFactory errors when autopacking.
-This meant that commits or pushes to 2a-format repositories failed
-intermittently.
-
-The second bug, #390563, caused the smart server to raise AbsentContentFactory
-when streaming 2a stacked 2a-format branches. This particularly affected
-branches stored on Launchpad in the 2a format.
-
-Both of these bugs cause command failures only, neither of them cause data
-corruption or data loss. And, of course, both of these bugs are now fixed.
-
-Bug Fixes
-*********
-
-* We now properly request a more minimal set of file texts when fetching
-  multiple revisions. (Robert Collins, John Arbash Meinel, #390563)
-
-* Repositories using CHK pages (which includes the new 2a format) will no
-  longer error during commit or push operations when an autopack operation
-  is triggered. (Robert Collins, #365615)
-
-* ``chk_map.iter_interesting_nodes`` now properly uses the *intersection*
-  of referenced nodes rather than the *union* to determine what
-  uninteresting pages we still need to look at. Prior to this,
-  incrementally pushing to stacked branch would push the minimal data, but
-  fetching everything would request extra texts. There are some unhandled
-  cases wrt trees of different depths, but this fixes the common cases.
-  (Robert Collins, John Arbash Meinel, #390563)
-
-* ``GroupCompress`` repositories now take advantage of the pack hints
-  parameter to permit cross-format fetching to incrementally pack the
-  converted data. (Robert Collins)
-
-* ``Repository.commit_write_group`` now returns opaque data about what
-  was committed, for passing to the ``Repository.pack``. Repositories
-  without atomic commits will still return None. (Robert Collins)
-
-* ``Repository.pack`` now takes an optional ``hint`` parameter
-  which will support doing partial packs for repositories that can do
-  that. (Robert Collins)
-
-* RepositoryFormat has a new attribute 'pack_compresses' which is True
-  when doing a pack operation changes the compression of content in the
-  repository. (Robert Collins)
-
-* ``StreamSink`` and ``InterDifferingSerialiser`` will call
-  ``Repository.pack`` with the hint returned by
-  ``Repository.commit_write_group`` if the formats were different and the
-  repository can increase compression by doing a pack operation.
-  (Robert Collins, #376748)
-
-
-bzr 1.16
-########
-:Codename: yesterday-in-california
-:1.16rc1: 2009-06-11
-:1.16: 2009-06-18
-
-This version of Bazaar contains the beta release of the new ``2a`` repository
-format, suitable for testing by fearless, advanced users. This format or an
-updated version of it will become the default format in Bazaar 2.0. Please
-read the NEWS entry before even thinking about upgrading to the new format.
-
-Also included are speedups for many operations on huge projects, a bug fix for
-pushing stacked new stacked branches to smart servers and the usual bevy of
-bug fixes and improvements.
-
-
-Changes from 1.16rc1 to 1.16final
-*********************************
-
-* Fix the nested tree flag check so that upgrade from development formats to
-  2a can work correctly.
-  (Jelmer Vernooij, #388727)
-
-* Automatic format upgrades triggered by default stacking policies on a
-  1.16rc1 (or later) smart server work again.
-  (Andrew Bennetts, #388675)
-
-
-Compatibility Breaks
-********************
-
-* Display prompt on stderr (instead of stdout) when querying users so
-  that the output of commands can be safely redirected.
-  (Vincent Ladeuil, #376582)
-
-
-New Features
-************
-
-* A new repository format ``2a`` has been added.  This is a beta release
-  of the brisbane-core (aka group-compress) project.  This format now
-  suitable for wider testing by advanced users willing to deal with some
-  bugs.  We would appreciate test reports, either positive or negative.
-  Format 2a is substantially smaller and faster for many operations on
-  many trees.  This format or an updated version will become the default
-  in bzr 2.0.
-
-  This is a rich-root format, so this repository format can be used with
-  bzr-svn.  Bazaar branches in previous non-rich-root formats can be
-  converted (including by merge, push and pull) to format 2a, but not vice
-  versa.  We recommend upgrading previous development formats to 2a.
-
-  Upgrading to this format can take considerable time because it expands
-  and more concisely repacks the full history.
-
-  If you use stacked branches, you must upgrade the stacked branches
-  before the stacked-on branches.  (See <https://bugs.launchpad.net/bugs/374735>)
-
-* ``--development7-rich-root`` is a new dev format, similar to ``--dev6``
-  but using a Revision serializer using bencode rather than XML.
-  (Jelmer Vernooij, John Arbash Meinel)
-
-* mail_client=claws now supports --body (and message body hooks).  Also uses
-  configured from address.  (Barry Warsaw)
-
-Improvements
-************
-
-
-* ``--development6-rich-root`` can now stack. (Modulo some smart-server
-  bugs with stacking and non default formats.)
-  (John Arbash Meinel, #373455)
-
-* ``--development6-rich-root`` delays generating a delta index for the
-  first object inserted into a group. This has a beneficial impact on
-  ``bzr commit`` since each committed texts goes to its own group. For
-  committing a 90MB file, it drops peak memory by about 200MB, and speeds
-  up commit from 7s => 4s. (John Arbash Meinel)
-
-* Numerous operations are now faster for huge projects, i.e. those
-  with a large number of files and/or a large number of revisions,
-  particularly when the latest development format is used. These
-  operations (and improvements on OpenOffice.org) include:
-
-  * branch in a shared repository (2X faster)
-  * branch --no-tree (100X faster)
-  * diff (2X faster)
-  * tags (70X faster)
-
-  (Ian Clatworthy)
-
-* Pyrex version of ``bencode`` support. This provides optimized support
-  for both encoding and decoding, and is now found at ``bzrlib.bencode``.
-  ``bzrlib.utils.bencode`` is now deprecated.
-  (Alexander Belchenko, Jelmer Vernooij, John Arbash Meinel)
-
-
-Bug Fixes
-*********
-
-* Bazaar can now pass attachment files to the mutt email client.
-  (Edwin Grubbs, #384158)
-
-* Better message in ``bzr add`` output suggesting using ``bzr ignored`` to
-  see which files can also be added.  (Jason Spashett, #76616)
-
-* ``bzr pull -r 123`` from a stacked branch on a smart server no longer fails.
-  Also, the ``Branch.revision_history()`` API now works in the same
-  situation.  (Andrew Bennetts, #380314)
-  
-* ``bzr serve`` on Windows no longer displays a traceback simply because a
-  TCP client disconnected. (Andrew Bennetts)
-
-* Clarify the rules for locking and fallback repositories. Fix bugs in how
-  ``RemoteRepository`` was handling fallbacks along with the
-  ``_real_repository``. (Andrew Bennetts, John Arbash Meinel, #375496)
-
-* Fix a small bug with fetching revisions w/ ghosts into a new stacked
-  branch. Not often triggered, because it required ghosts to be part of
-  the fetched revisions, not in the stacked-on ancestry.
-  (John Arbash Meinel)
-
-* Fix status and commit to work with content filtered trees, addressing
-  numerous bad bugs with line-ending support. (Ian Clatworthy, #362030)
-
-* Fix problem of "directory not empty" when contending for a lock over
-  sftp.  (Martin Pool, #340352)
-
-* Fix rule handling so that eol is optional, not mandatory.
-  (Ian Clatworthy, #379370)
-
-* Pushing a new stacked branch to a 1.15 smart server was broken due to a
-  bug in the ``BzrDirFormat.initialize_ex`` smart verb.  This is fixed in
-  1.16, but required changes to the network protocol, so the
-  ``BzrDirFormat.initialize_ex`` verb has been removed and replaced with a
-  corrected ``BzrDirFormat.initialize_ex_1.16`` verb.  1.15 clients will
-  still work with a 1.16 server as they will fallback to slower (and
-  bug-free) methods.
-  (Jonathan Lange, Robert Collins, Andrew Bennetts, #385132)
-
-* Reconcile can now deal with text revisions that originated in revisions 
-  that are ghosts. (Jelmer Vernooij, #336749)
-
-* Support cloning of branches with ghosts in the left hand side history.
-  (Jelmer Vernooij, #248540)
-
-* The ''bzr diff'' now catches OSError from osutils.rmtree and logs a
-  helpful message to the trace file, unless the temp directory really was
-  removed (which would be very strange).  Since the diff operation has
-  succeeded from the user's perspective, no output is written to stderr 
-  or stdout.  (Maritza Mendez, #363837)
-
-* Translate errors received from a smart server in response to a
-  ``BzrDirFormat.initialize`` or ``BzrDirFormat.initialize_ex`` request.
-  This was causing tracebacks even for mundane errors like
-  ``PermissionDenied``.  (Andrew Bennetts, #381329)
-
-Documentation
-*************
-
-* Added directory structure and started translation of docs in Russian.
-  (Alexey Shtokalo, Alexander Iljin, Alexander Belchenko, Dmitry Vasiliev,
-  Volodymyr Kotulskyi)
-
-API Changes
-***********
-
-* Added osutils.parent_directories(). (Ian Clatworthy)
-
-* ``bzrlib.progress.ProgressBar``, ``ChildProgress``, ``DotsProgressBar``,
-  ``TTYProgressBar`` and ``child_progress`` are now deprecated; use
-  ``ui_factory.nested_progress_bar`` instead.  (Martin Pool)
-
-* ``graph.StackedParentsProvider`` is now a public API, replacing
-  ``graph._StackedParentsProvider``. The api is now considered stable and ready
-  for external users. (Gary van der Merwe)
-
-* ``bzrlib.user_encoding`` is deprecated in favor of
-  ``get_user_encoding``.  (Alexander Belchenko)
-
-* TreeTransformBase no longer assumes that limbo is provided via disk.
-  DiskTreeTransform now provides disk functionality.  (Aaron Bentley)
-
-Internals
-*********
-
-* Remove ``weave.py`` script for accessing internals of old weave-format
-  repositories.  (Martin Pool)
-
-Testing
-*******
-
-* ``make check`` no longer repeats the test run in ``LANG=C``.
-  (Martin Pool, #386180)
-
-* The number of cores is now correctly detected on OSX. (John Szakmeister)
-
-* The number of cores is also detected on Solaris and win32. (Vincent Ladeuil)
-
-* The number of cores is also detected on FreeBSD. (Matthew Fuller)
-
-
-bzr 1.15
-########
-:1.15rc1: 2009-05-16
-:1.15: 2009-05-22
-:1.15.1: 2009-06-09
-
-The smart server will no longer raise 'NoSuchRevision' when streaming content
-with a size mismatch in a reconstructed graph search. New command ``bzr
-dpush``. Plugins can now define their own annotation tie-breaker when two
-revisions introduce the exact same line.
-
-Changes from 1.15.1 to 1.15.2
-*****************************
-
-* Use zdll on Windows to build ``_chk_map_pyx`` extension.
-  (Alexander Belchenko)
-
-Changes from 1.15final to 1.15.1
-*********************************
-
-* Translate errors received from a smart server in response to a
-  ``BzrDirFormat.initialize`` or ``BzrDirFormat.initialize_ex`` request.
-  This was causing tracebacks even for mundane errors like
-  ``PermissionDenied``.  (Andrew Bennetts, #381329)
-
-Changes from 1.15rc1 to 1.15final
-*********************************
-
-* No changes
-
-Compatibility Breaks
-********************
-
-* ``bzr ls`` is no longer recursive by default. To recurse, use the
-  new ``-R`` option. The old ``--non-recursive`` option has been removed.
-  If you alias ``ls`` to ``ls -R``, you can disable recursion using
-  ``--no-recursive`` instead.  (Ian Clatworthy)
-
-New Features
-************
-
-* New command ``bzr dpush`` that can push changes to foreign 
-  branches (svn, git) without setting custom bzr-specific metadata.
-  (Jelmer Vernooij)
-
-* The new development format ``--development6-rich-root`` now supports
-  stacking. We chose not to use a new format marker, since old clients
-  will just fail to open stacked branches, the same as if we used a new
-  format flag. (John Arbash Meinel, #373455)
-
-* Plugins can now define their own annotation tie-breaker when two revisions
-  introduce the exact same line. See ``bzrlib.annotate._break_annotation_tie``
-  Be aware though that this is temporary, private (as indicated by the leading
-  '_') and a first step to address the problem. (Vincent Ladeuil, #348459)
-
-* New command ``bzr dpush`` that can push changes to foreign 
-  branches (svn, git) without setting custom bzr-specific metadata.
-  (Jelmer Vernooij)
-
-* ``bzr send`` will now check the ``child_submit_format`` setting in
-  the submit branch to determine what format to use, if none was 
-  specified on the command-line.  (Jelmer Vernooij)
-
-Improvements
-************
-
-* -Dhpss output now includes the number of VFS calls made to the remote
-  server. (Jonathan Lange)
-
-* ``--coverage`` works for code running in threads too.
-  (Andrew Bennets, Vincent Ladeuil)
-
-* ``bzr pull`` now has a ``--local`` option to only make changes to the
-  local branch, and not the bound master branch.
-  (Gary van der Merwe, #194716)
-
-* ``bzr rm *`` is now as fast as ``bzr rm * --keep``. (Johan Walles, #180116)
-
-Bug Fixes
-*********
-
-* Adding now works properly when path contains a symbolic link.
-  (Geoff Bache, #183831)
-
-* An error is now raised for unknown eol values. (Brian de Alwis, #358199)
-
-* ``bzr merge --weave`` will now generate a conflict if one side deletes a
-  line, and the other side modifies the line. (John Arbash Meinel, #328171)
-
-* ``bzr reconfigure --standalone`` no longer raises IncompatibleRepositories.
-  (Martin von Gagern, #248932)
-
-* ``bzr send`` works to send emails again using MAPI.
-  (Neil Martinsen-Burrell, #346998)
-
-* Check for missing parent inventories in StreamSink.  This prevents
-  incomplete stacked branches being created by 1.13 bzr:// and
-  bzr+ssh:// clients (which have bug #354036).  Instead, the server now
-  causes those clients to send the missing records.  (Andrew Bennetts)
-
-* Correctly handle http servers proposing multiple authentication schemes.
-  (Vincent Ladeuil, #366107)
-
-* End-Of-Line content filters are now loaded correctly.
-  (Ian Clatworthy, Brian de Alwis, #355280)
-
-* Fix a bug in the pure-python ``GroupCompress`` code when handling copies
-  longer than 64KiB. (John Arbash Meinel, #364900)
-
-* Fix TypeError in running ``bzr break-lock`` on some URLs.
-  (Alexander Belchenko, Martin Pool, #365891)
-
-* Non-recursive ``bzr ls`` now works properly when a path is specified.
-  (Jelmer Vernooij, #357863)
-
-* ssh usernames (defined in ~/.ssh/config) are honoured for bzr+ssh connections.
-  (Vincent Ladeuil, #367726)
-
-* Several bugs related to unicode symlinks have been fixed and the test suite
-  enhanced to better catch regressions for them. (Vincent Ladeuil)
-
-* The smart server will no longer raise 'NoSuchRevision' when streaming
-  content with a size mismatch in a reconstructed graph search: it assumes
-  that the client will make sure it is happy with what it got, and this
-  sort of mismatch is normal for stacked environments.
-  bzr 1.13.0/1 will stream from unstacked branches only - in that case not
-  getting all the content expected would be a bug. However the graph
-  search is how we figured out what we wanted, so a mismatch is both odd
-  and unrecoverable without starting over, and starting over will end up
-  with the same data as if we just permitted the mismatch. If data is
-  gc'd, doing a new search will find only the truncated data, so sending
-  only the truncated data seems reasonable. bzr versions newer than this
-  will stream from stacked branches and check the stream to find missing
-  content in the stacked-on branch, and thus will handle the situation
-  implicitly.  (Robert Collins, #360791)
-
-* Upgrading to, or fetching into a 'rich-root' format will now correctly
-  set the root data the same way that reconcile does.
-  (Robert Collins, #368921)
-
-* Using unicode Windows API to obtain command-line arguments.
-  (Alexander Belchenko, #375934)
-
-Documentation
-*************
-
-API Changes
-***********
-
-* ``InterPackRepo.fetch`` and ``RepoFetcher`` now raise ``NoSuchRevision``
-  instead of ``InstallFailed`` when they detect a missing revision.
-  ``InstallFailed`` itself has been deleted. (Jonathan Lange)
-
-* Not passing arguments to ``bzrlib.commands.main()`` will now grab the
-  arguments from ``osutils.get_unicode_argv()`` which has proper support
-  for unicode arguments on windows. Further, the supplied arguments are now 
-  required to be unicode strings, rather than user_encoded strings.
-  (Alexander Belchenko)
-
-Internals
-*********
-
-* ``bzrlib.branch.Branch.set_parent`` is now present on the base branch
-  class and will call ``_set_parent_location`` after doing unicode 
-  encoding. (Robert Collins)
-
-* ``bzrlib.remote.RemoteBranch._set_parent_location`` will use a new verb
-  ``Branch.set_parent_location`` removing further VFS operations.
-  (Robert Collins)
-
-* ``bzrlib.bzrdir.BzrDir._get_config`` now returns a ``TransportConfig``
-  or similar when the dir supports configuration settings. The base class
-  defaults to None. There is a matching new server verb
-  ``BzrDir.get-config_file`` to reduce roundtrips for getting BzrDir
-  configuration. (Robert Collins)
-
-* ``bzrlib.tests.ExtendedTestResult`` has new methods ``startTests``
-  called before the first test is started, ``done`` called after the last
-  test completes, and a new parameter ``strict``. (Robert Collins)
-
-* ``-Dhpss`` when passed to bzr will cause a backtrace to be printed when
-  VFS operations are started on a smart server repository. This should not
-  occur on regular push and pull operations, and is a key indicator for
-  performance regressions. (Robert Collins)
-
-* ``-Dlock`` when passed to the selftest (e.g. ``bzr -Dlock selftest``) will
-  cause mismatched physical locks to cause test errors rather than just
-  reporting to the screen. (Robert Collins)
-
-* -Dprogress will cause pdb to start up if a progress view jumps
-  backwards. (Robert Collins)
-
-* Fallback ``CredentialStore`` instances registered with ``fallback=True``
-  are now be able to provide credentials if obtaining credentials 
-  via ~/.bazaar/authentication.conf fails. (Jelmer Vernooij, 
-  Vincent Ladeuil, #321918)
-
-* New hook ``Lock.lock_broken`` which runs when a lock is
-  broken. This is mainly for testing that lock/unlock are
-  balanced in tests. (Vincent Ladeuil)
-
-* New MergeDirective hook 'merge_request_body' allows hooks to supply or
-  alter a body for the message produced by ``bzr send``.
-
-* New smart server verb ``BzrDir.initialize_ex`` which implements a
-  refactoring to the core of clone allowing less round trips on new
-  branches. (Robert Collins)
-
-* New method ``Tags.rename_revisions`` that can rename revision ids tags
-  are pointing at. (Jelmer Vernooij)
-
-* Updated the bundled ``ConfigObj`` library to 4.6.0 (Matt Nordhoff)
-
-Testing
-*******
-
-* ``bzr selftest`` will now fail if lock/unlock are not correctly balanced in
-  tests. Using ``-Dlock`` will turn the related failures into warnings.
-  (Vincent Ladeuil, Robert Collins)
-
-bzr 1.14
-########
-:Codename: brisbane-core
-:1.14rc1: 2009-04-06
-:1.14rc2: 2009-04-19
-:1.14: 2009-04-28
-:1.14.1: 2009-05-01
-
-New formats 1.14 and 1.14-rich-root supporting End-Of-Line (EOL) conversions,
-keyword templating (via the bzr-keywords plugin) and generic content filtering.
-End-of-line conversion is now supported for formats supporting content
-filtering.
-
-Changes from 1.14final to 1.14.1
-********************************
-
-* Change api_minimum_version back to api_minimum_version = (1, 13, 0)
-
-Changes from 1.14rc2 to 1.14final
-*********************************
-
-* Fix a bug in the pure-python ``GroupCompress`` code when handling copies
-  longer than 64KiB. (John Arbash Meinel, #364900)
-
-Changes from 1.14rc1 to 1.14rc2
-*******************************
-
-* Fix for bug 358037 Revision not in
-  bzrlib.groupcompress.GroupCompressVersionedFiles (Brian de Alwis, 
-  John A Meinel)
-
-* Fix for bug 354036 ErrorFromSmartServer - AbsentContentFactory object has no
-  attribute 'get_bytes_as' exception while pulling from Launchpad 
-  (Jean-Francois Roy, Andrew Bennetts, Robert Collins)
-
-* Fix for bug 355280 eol content filters are never loaded and thus never
-  applied (Brian de Alwis, Ian Clatworthy)
- 
-* bzr.dev -r4280  Change _fetch_uses_deltas = False for CHK repos until we can
-  write a better fix. (John Arbash Meinel, Robert Collins)
-
-* Fix for bug 361574 uncommit recommends undefined --levels and -n options
-  (Marius Kruger, Ian Clatworthy)
-
-* bzr.dev r4289 as cherrypicked at lp:~spiv/bzr/stacking-cherrypick-1.14 
-  (Andrew Bennetts, Robert Collins)
-
-Compatibility Breaks
-********************
-
-* A previously disabled code path to accelerate getting configuration
-  settings from a smart server has been reinstated. We think this *may*
-  cause a incompatibility with servers older than bzr 0.15. We intend
-  to issue a point release to address this if it turns out to be a
-  problem. (Robert Collins, Andrew Bennetts)
-
-* bzr no longer autodetects nested trees as 'tree-references'.  They
-  must now be explicitly added tree references.  At the commandline, use
-  join --reference instead of add.  (Aaron Bentley)
-
-* The ``--long`` log format (the default) no longer shows merged
-  revisions implicitly, making it consistent with the ``short`` and
-  ``line`` log formats.  To see merged revisions for just a given
-  revision, use ``bzr log -n0 -rX``. To see every merged revision,
-  use ``bzr log -n0``.  (Ian Clatworthy)
-
-New Features
-************
-
-* New formats ``1.14`` and ``1.14-rich-root`` supporting End-Of-Line
-  (EOL) conversions, keyword templating (via the bzr-keywords plugin)
-  and generic content filtering. These formats replace the experimental
-  ``development-wt5`` and ``development-wt5-rich-root`` formats
-  respectively, but have support for filtered views disabled.
-  (Ian Clatworthy)
-
-* New ``mv --auto`` option recognizes renames after they occur.
-  (Aaron Bentley)
-
-* ``bzr`` can now get passwords from stdin without requiring a controlling
-  terminal (i.e. by redirecting stdin). (Vincent Ladeuil)
-
-* ``bzr log`` now supports filtering of multiple files and directories
-  and will show changes that touch any of them. Furthermore,
-  directory filtering now shows the changes to any children of that
-  directory, not just the directory object itself.
-  (Ian Clatworthy, #97715)
-
-* ``bzr shelve`` can now apply changes without storing anything on the
-  shelf, via the new --destroy option.  (Aaron Bentley)
-
-* ``bzr send`` now accepts --body to specify an initial message body.
-  (Aaron bentley)
-
-* ``bzr xxx --usage`` where xxx is a command now shows a usage
-  message and the options without the descriptive help sections
-  (like Description and Examples). A message is also given
-  explaining how to see the complete help, i.e. ``bzr help xxx``.
-  (Ian Clatworthy)
-
-* Content filters can now be used to provide custom conversion
-  between the canonical format of content (i.e. as stored) and
-  the convenience format of content (i.e. as created in working
-  trees). See ``bzr help content-filters`` for further details.
-  (Ian Clatworthy, Alexander Belchenko)
-
-* End-of-line conversion is now supported for formats supporting
-  content filtering. See ``bzr help eol`` for details.
-  (Ian Clatworthy)
-
-* Newly-blessed `join` command allows combining two trees into one.
-  (Aaron Bentley)
-
-Improvements
-************
-
-* A new format name alias ``default-rich-root`` has been added and
-  points at the closest relative of the default format that supports 
-  rich roots. (Jelmer Vernooij, #338061)
-
-* Branching from a stacked branch using ``bzr*://`` will now stream
-  the data when the target repository does not need topological
-  ordering, reducing round trips and network overhead. This uses the
-  existing smart server methods added in 1.13, so will work on any
-  1.13 or newer server. (Robert Collins, Andrew Bennetts)
-
-* ``bzr cat`` and ``bzr export`` now supports a ``--filters`` option
-  that displays/saves the content after content filters are applied.
-  (Ian Clatworthy)
-
-* ``bzr ignore`` gives a more informative message when existing
-  version controlled files match the ignore pattern. (Neil
-  Martinsen-Burrell, #248895)
-
-* ``bzr log`` now has ``--include-merges`` as an alias for ``--levels 0``.
-  (Ian Clatworthy)
-
-* ``bzr send`` is faster on repositories with deep histories.
-  (Ian Clatworthy)
-
-* IPv6 literals are accepted in URLs.
-  (stlman, Martin Pool, Jelmer Vernooij, #165014)
-
-* Progress bars now show the rate of network activity for
-  ``bzr+ssh://`` and ``bzr://`` connections.  (Andrew Bennetts)
-
-* Prompt for user names if they are not in the configuration. 
-  (Jelmer Vernooij, #256612)
-
-* Pushing to a stacked pack-format branch on a 1.12 or older smart server
-  now takes many less round trips.  (Andrew Bennetts, Robert Collins,
-  #294479)
-  
-* Streaming push can be done to older repository formats.  This is
-  implemented using a new ``Repository.insert_stream_locked`` RPC.
-  (Andrew Bennetts, Robert Collins)
-
-* The "ignoring files outside view: .." message has been re-worded
-  to "Ignoring files outside view. View is .." to reduce confusion
-  about what was being considered and what was being ignored.
-  (Ian Clatworthy)
-
-* The ``long`` log formatter now shows [merge] indicators. If
-  only one level of revisions is displayed and merges are found,
-  the ``long`` and ``short`` log formatters now tell the user
-  how to see the hidden merged revisions.  (Ian Clatworthy)
-
-* The ``brisbane-core`` project has delivered its beta format
-  ``development6-rich-root``. This format is suitable for judicious
-  testing by early adopters. In particular if you are benchmarking bzr
-  performance please be sure to test using this format. At this stage
-  more information is best obtained by contacting the Bazaar mailing list
-  or IRC channel if you are interested in using this format. We will make
-  end user documentation available closer to blessing the format as
-  production ready. (Robert Collins, John Arbash Meinel, Ian Clatworthy,
-  Vincent Ladeuil, Andrew Bennetts, Martin Pool)
-
-* Tildes are no longer escaped. No more %7Euser/project/branch!
-  (Jonathan Lange)
-
-Bug Fixes
-*********
-
-* Pushing a new stacked branch will also push the parent inventories for
-  revisions at the stacking boundary.  This makes sure that the stacked
-  branch has enough data to calculate inventory deltas for all of its
-  revisions (without requiring the fallback branch).  This avoids
-  "'AbsentContentFactory' object has no attribute 'get_bytes_as'" errors
-  when fetching the stacked branch from a 1.13 (or later) smart server.
-  This partially fixes #354036.  (Andrew Bennetts, Robert Collins)
-
-* End-Of-Line content filters are now loaded correctly.
-  (Ian Clatworthy, Brian de Alwis, #355280)
-
-* Authentication plugins now receive all the parameters from the request
-  itself (aka host, port, realm, path, etc). Previously, only the 
-  authentication section name, username and encoded password were 
-  provided. (Jean-Francois Roy)
-
-* bzr gives a better message if an invalid regexp is passed to ``bzr log
-  -m``.  (Anne Mohsen, Martin Pool)
-
-* ``bzr split`` now says "See also: join" (Aaron Bentley, #335015)
-
-* ``bzr version-info`` now works in empty branches. (Jelmer Vernooij,
-  #313028)
-
-* Fix "is not a stackable format" error when pushing a
-  stackable-format branch with an unstackable-format repository to a
-  destination with a default stacking policy.  (Andrew Bennetts)
-
-* Fixed incorrect "Source format does not support stacking" warning
-  when pushing to a smart server.  (Andrew Bennetts, #334114)
-
-* Fix 'make check-dist-tarball' failure by converting paths to unicode when
-  needed. (Vincent Ladeuil, #355454)
-
-* Fixed "Specified file 'x/y/z' is outside current view: " occurring
-  on ``bzr add x/y/z`` in formats supporting views when no view is
-  defined.  (Ian Clatworthy, #344708)
-
-* It is no longer possible to fetch between repositories while the
-  target repository is in a write group. This prevents race conditions
-  that prevent the use of RPC's to perform fetch, and thus allows
-  optimising more operations. (Robert Collins, Andrew Bennetts)
-
-* ``merge --force`` works again. (Robert Collins, #342105)
-
-* No more warnings are issued about ``sha`` being deprecated under python-2.6.
-  (Vincent Ladeuil, #346593)
-
-* Pushing a new branch to a server that has a stacking policy will now
-  upgrade from the local branch format when the stacking policy points at
-  a branch which is itself stackable, because we know the client can read
-  both branches, we know that the trunk for the project can be read too,
-  so the upgrade will not inconvenience users. (Robert Collins, #345169)
-
-* Pushing a new stacked branch will also push the parent inventories for
-  revisions at the stacking boundary.  This makes sure that the stacked
-  branch has enough data to calculate inventory deltas for all of its
-  revisions (without requiring the fallback branch).  This avoids
-  "'AbsentContentFactory' object has no attribute 'get_bytes_as'" errors
-  when fetching the stacked branch from a 1.13 (or later) smart server.
-  This partially fixes #354036.  (Andrew Bennetts, Robert Collins)
-
-* The full test suite is passing again on OSX. Several minor issues (mostly
-  test related) have been fixed. (Vincent Ladeuil, #355273).
-
-* The GNU Changelog formatter is slightly improved in the case where
-  the delta is empty, and now correctly claims not to support tags.
-  (Andrea Bolognani)
-
-* Shelve can now shelve changes to a symlink target.
-  (James Westby, #341558)
-
-* The help for the ``info`` command has been corrected.
-  (Ian Clatworthy, #351931)
-
-* Upgrade will now use a sensible default format if the source repository
-  uses rich roots.  (Jelmer Vernooij, #252908)
-
-Documentation
-*************
-
-* Expanded the index of the developer documentation. (Eric Siegerman)
-
-* New topic `bzr help debug-flags`.  (Martin Pool)
-
-* The generated manpage now explicitly lists aliases as commands.
-  (James Westby, #336998)
-
-API Changes
-***********
-
-* APIs deprecated in 1.6 and previous versions of bzr are now removed.
-  (Martin Pool)
-
-* ``CommitReporter`` is no longer called with ``unchanged`` status during
-  commit - this was a full-tree overhead that bzr no longer performs.
-  (Robert Collins)
-
-* New abstract ``UIFactory`` method ``get_username`` which will be called to 
-  obtain the username to use when connecting to remote machines. 
-  (Jelmer Vernooij)
-
-* New API ``Inventory.filter()`` added that filters an inventory by
-  a set of file-ids so that only those fileids, their parents and
-  their children are included.  (Ian Clatworthy)
-
-* New sort order for ``get_record_stream`` ``groupcompress`` which
-  sorts optimally for use with groupcompress compressors. (John Arbash
-  Meinel, Robert Collins)
-
-* Repository APIs ``get_deltas_for_revisions()`` and
-  ``get_revision_delta()`` now support an optional ``specific_fileids``
-  parameter. If provided, the deltas are filtered so that only those
-  file-ids, their parents and their children are included.
-  (Ian Clatworthy)
-
-* The ``get_credentials`` and ``set_credentials`` methods of 
-  ``AuthenticationConfig`` now accept an optional realm argument.
-  (Jean-Francois Roy)
-
-* The ``pb`` argument to ``fetch()`` is deprecated.
-  (Martin Pool)
-
-* The ``Serializer`` class and the serializer ``format registry`` have moved
-  from ``bzrlib.xml_serializer`` to ``bzrlib.serializer``. (Jelmer Vernooij)
-
-* The smart server jail now hooks into BzrDir.open to prevent any BzrDir
-  that is not inside the backing transport from being opened.  See the
-  module documentation for ``bzrlib.smart.request`` for details.
-  (Andrew Bennetts, Robert Collins)
-
-* ``Tree.get_symlink_target`` now always returns a unicode string result
-  or None. Previously it would return the bytes from reading the link
-  which could be in any arbitrary encoding. (Robert Collins)
-
-Testing
-*******
-
-* ``bzrlib.tests.TestCase`` now fails the test if its own ``setUp``
-  and ``tearDown`` weren't called.  This catches faulty tests that
-  forget to upcall when overriding ``setUp`` and ``tearDown``.  Those
-  faulty tests were not properly isolated.
-  (Andrew Bennetts, Robert Collins)
-
-* Fix test_msgeditor.MsgEditorTest test isolation.
-  (Vincent Ladeuil, #347130)
-
-* ``medusa`` is not used anymore as an FTP test server starting with
-  python2.6. A new FTP test server based on ``pyftplib`` can be used
-  instead. This new server is a soft dependency as medusa which is still
-  preferred if both are available (modulo python version).
-  (Vincent Ladeuil)
-
-Internals
-*********
-
-* Added ``chk_map`` for fast, trie-based storage of tuple to string maps.
-  (Robert Collins, John Arbash Meinel, Vincent Ladeuil)
-
-* Added ``bzrlib.chk_map`` for fast, trie-based storage of tuple to string
-  maps.  (Robert Collins, John Arbash Meinel, Vincent Ladeuil)
-
-* Added ``bzrlib.inventory_delta`` module.  This will be used for
-  serializing and deserializing inventory deltas for more efficient
-  streaming on the network.  (Robert Collins, Andrew Bennetts)
-
-* ``Branch._get_config`` has been added, which splits out access to the
-  specific config file from the branch. This is used to let RemoteBranch
-  avoid constructing real branch objects to access configuration settings.
-  (Robert Collins, Andrew Bennetts)
-
-* ``Branch`` now implements ``set_stacked_on_url`` in the base class as
-  the implementation is generic and should impact foreign formats. This
-  helps performance for ``RemoteBranch`` push operations to new stacked
-  branches. (Robert Collins, Andrew Bennetts)
-
-* ``BtreeIndex._spill_mem_keys_to_disk()`` now generates disk index with
-  optmizations turned off. This only has effect when processing > 100,000
-  keys during something like ``bzr pack``. (John Arbash Meinel)
-
-* ``bzr selftest`` now accepts ``--subunit`` to run in subunit output
-  mode. Requires ``lp:subunit`` installed to work, but is not a hard
-  dependency. (Robert Collins)
-
-* ``BzrDir.open_branch`` now takes an optional ``ignore_fallbacks``
-  parameter for controlling opening of stacked branches.
-  (Andrew Bennetts, Robert Collins)
-  
-* ``CommitBuilder`` has a new method, ``record_iter_changes`` which works
-  in terms of an iter_changes iterator rather than full tree scanning.
-  (Robert Collins)
-
-* ``DirState`` can now be passed a custom ``SHA1Provider`` object
-  enabling it to store the SHA1 and stat of the canonical (post
-  content filtered) form. (Ian Clatworthy)
-
-* New ``assertLength`` method based on one Martin has squirreled away
-  somewhere. (Robert Collins, Martin Pool)
-
-* New hook ``BzrDir.pre_open`` which runs before opening ``BzrDir``
-  objects, allowing better enforcement of the smart server jail when
-  dealing with stacked branches. (Robert Collins, Andrew Bennetts)
-
-* New hook ``RioVersionInfoBuilder.revision``, allowing extra entries 
-  to be added to the stanza that is printed for a particular revision.
-  (Jelmer Vernooij)
-
-* New repository method ``refresh_data`` to cause any repository to
-  make visible data inserted into the repository by a smart server
-  fetch operation. (Robert Collins, Andrew Bennetts)
-
-* ``register_filter_stack_map`` now takes an optional fallback parameter,
-  a callable to invoke if a preference has a value not in the map
-  of filter stacks. This enhancement allows, for example,  bzr-svn to
-  handle existing svn properties that define a list of keywords to be
-  expanded.  (Ian Clatworthy)
-
-* ``RemoteBranchConfig`` will use a new verb ``Branch.set_config_option``
-  to write config settings to smart servers that support this, saving
-  5 round trips on the stacked streaming acceptance test.
-  (Robert Collins, Andrew Bennetts)
-
-* ``RemoteBranch`` now provides ``_get_config`` for access to just the
-  branch specific configuration from a remote server, which uses the 
-  already existing ``Branch.get_config_file`` smart verb.
-  (Robert Collins, Andrew Bennetts)
-
-* ``RemoteRepository`` will now negatively cache missing revisions during
-  ``get_parent_map`` while read-locked. Write-locks are unaffected.
-  (Robert Collins, Andrew Bennetts)
-
-* Removed ``InterRemoteToOther``, ``InterOtherToRemote`` and
-  ``InterPackToRemotePack`` classes, as they are now unnecessary.
-  (Andrew Bennetts)
-
-* ``RepositoryFormat`` as a new attribute ``fast_deltas`` to indicate
-  whether the repository can efficiently generate deltas between trees
-  regardless of tree size. (Robert Collins)
-
-* ``Repository.iter_files_bytes()`` now properly returns an "iterable of
-  byte strings" (aka 'chunked') for the content. It previously was
-  returning a plain string, which worked, but performed very poorly when
-  building a working tree (file.writelines(str) is very inefficient). This
-  can have a large effect on ``bzr checkout`` times. (John Arbash Meinel)
-
-* selftest now supports a --parallel option, with values of 'fork' or
-  'subprocess' to run the test suite in parallel. Currently only Linux
-  machines work, other platforms need patches submitted. (Robert Collins,
-  Vincent Ladeuil)
-
-* ``tests.run_suite`` has a new parameter ``suite_decorators``, a list of 
-  callables to use to decorate the test suite. Such decorators can add or
-  remove tests, or even remote the test suite to another machine if
-  desired. (Robert Collins)
-
-* The smart server verb ``Repository.get_parent_map`` can now include
-  information about ghosts when the special revision ``include-missing:``
-  is in the requested parents map list. With this flag, ghosts are
-  included as ``missing:REVISION_ID``. (Robert Collins, Andrew Bennetts)
-
-* ``_walk_to_common_revisions`` will now batch up at least 50
-  revisions before calling ``get_parent_map`` on the target,
-  regardless of ``InterRepository``.
-  (Andrew Bennetts, Robert Collins)
-
-bzr 1.13
-########
-
-:Codename: paraskavedekatriaphobia
-:1.13: 2009-03-14
-:1.13rc1: 2009-03-10
-:1.13.1: 2009-03-23
-:1.13.2: 2009-04-27
-
-GNU Changelog output can now be produced by ``bzr log --gnu-changelog``.  Debug
-flags can now be set in ``~/.bazaar/bazaar.conf``.  Lightweight checkouts and
-stacked branches should both be much faster over remote connections.  
-
-Changes From 1.13.1 to 1.13.2
-*****************************
-
-A regression was found in the 1.13.1 release. When bzr 1.13.1 and earlier push
-a stacked branch they do not take care to push all the parent inventories for
-the transferred revisions. This means that a smart server serving that branch
-often cannot calculate inventory deltas for the branch (because smart server
-does not/cannot open fallback repositories). Prior to 1.13 the server did not
-have a verb to stream revisions out of a repository, so that's why this bug has
-appeared now.
-
-Bug Fixes
-*********
-
-* Fix for bug 354036 ErrorFromSmartServer - AbsentContentFactory object has no
-  attribute 'get_bytes_as' exception while pulling from Launchpad 
-  (Jean-Francois Roy, Andrew Bennetts, Robert Collins)
-
-Changes From 1.13final to 1.13.1
-********************************
-
-A couple regessions where found in the 1.13 release. The pyrex-generated C
-extensions are missing from the .tar.gz and .zip files.  Documentation on how
-to generate GNU ChangeLogs is wrong.
-
-Bug Fixes
-*********
-
-* Change ``./bzr``'s ``_script_version`` to match ./bzrlib/__init__.py
-  version_info. (Bob Tanner, Martin Pool, #345232)
-
-* Distribution archives for 1.13 do not contain generated C extension modules
-  (Jean-Francois Roy, Bob Tanner, #344465)
-
-* GNU ChangeLog output can now be produced by bzr log --format gnu-changelog is
-  incorrect (Deejay, Bob Tanner, Martin Pool, Robert Collins, #343928)
-
-* ``merge --force`` works again. (Robert Collins, #342105)
-
-Changes From 1.13rc1 to 1.13final
-*********************************
-
-* Fix "is not a stackable format" error when pushing a
-  stackable-format branch with an unstackable-format repository to a
-  destination with a default stacking policy.  (Andrew Bennetts)
-
-* Progress bars now show the rate of network activity for
-  ``bzr+ssh://`` and ``bzr://`` connections.  (Andrew Bennetts)
-
-Compatibility Breaks
-********************
-
-* ``bzr log --line`` now indicates which revisions are merges with
-  `[merge]` after the date.  Scripts which parse the output of this
-  command may need to be adjusted.
-  (Neil Martinsen-Burrell)
-
-New Features
-************
-
-* ``bzr reconfigure`` now supports --with-trees and --with-no-trees
-  options to change the default tree-creation policy of shared
-  repositories.  (Matthew Fuller, Marius Kruger, #145033)
-
-* Debug flags can now be set in ``~/.bazaar/bazaar.conf``.
-  (Martin Pool)
-
-* Filtered views provide a mask over the tree so that users can focus
-  on a subset of a tree when doing their work. See ``Filtered views``
-  in chapter 7 of the User Guide and ``bzr help view`` for details.
-  (Ian Clatworthy)
-
-* GNU Changelog output can now be produced by ``bzr log --gnu-changelog``.
-  (Andrea Bolognani, Martin Pool)
-
-* The ``-Dmemory`` flag now gives memory information on Windows.
-  (John Arbash Meinel)
-
-* Multiple authors for a commit can now be recorded by using the "--author"
-  option multiple times. (James Westby, #185772)
-
-* New clean-tree command, from bzrtools.  (Aaron Bentley, Jelmer Vernooij)
-
-* New command ``bzr launchpad-open`` opens a Launchpad web page for that
-  branch in your web browser, as long as the branch is on Launchpad at all.
-  (Jonathan Lange)
-
-* New API for getting bugs fixed by a revision: Revision.iter_bugs().
-  (Jonathan Lange)
-
-Improvements
-************
-
-* All bzr ``Hooks`` classes are now registered in
-  ``bzrlib.hooks.known_hooks``. This removes the separate list from
-  ``bzrlib.tests`` and ensures that all hooks registered there are
-  correctly isolated by the test suite (previously
-  ``MutableTreeHooks`` were not being isolated correctly). Further, 
-  documentation for hooks is now dynamically generated from the
-  present HookPoints. ``bzr hooks`` will now also report on all the
-  hooks present in the ``bzrlib.hooks.known_hooks`` registry.
-  (Robert Collins)
-
-* ``bzr add`` no longer prints ``add completed`` on success. Failure
-  still prints an error message. (Robert Collins)
-
-* ``bzr branch`` now has a ``--no-tree`` option which turns off the
-  generation of a working tree in the new branch.
-  (Daniel Watkins, John Klinger, #273993)
-
-* Bazaar will now point out ``bzr+ssh://`` to the user when they 
-  use ssh://. (Jelmer Vernooij, #330535)
-
-* ``bzr -v info`` now omits the number of committers branch statistic,
-  making it many times faster for large projects. To include that
-  statistic in the output, use ``bzr -vv info``.
-  (Ian Clatworthy)
-
-* ``bzr push`` to a ``bzr`` url (``bzr://``, ``bzr+ssh://`` etc) will
-  stream if the server is version 1.13 or greater, reducing roundtrips
-  significantly. (Andrew Bennetts, Robert Collins)
-
-* Lightweight Checkouts and Stacked Branches should both be much
-  faster over remote connections. Building the working tree now
-  batches up requests into approx 5MB requests, rather than a separate
-  request for each file. (John Arbash Meinel)
-
-* Support for GSSAPI authentication when using HTTP or HTTPS. 
-  (Jelmer Vernooij)
-
-* The ``bzr shelve`` prompt now includes a '?' help option to explain the
-  short options better. (Daniel Watkins, #327429)
-
-* ``bzr lp-open`` now falls back to the push location if it cannot find a
-  public location. (Jonathan Lange, #332372)
-
-* ``bzr lp-open`` will try to find the Launchpad URL for the location
-  passed on the command line. This makes ``bzr lp-open lp:foo`` work as
-  expected. (Jonathan Lange, #332705)
-
-* ``bzr send`` now supports MH-E via ``emacsclient``. (Eric Gillespie)
-
-Bug Fixes
-*********
-
-* Allows ``bzr log <FILE>`` to be called in an empty branch without
-  backtracing. (Vincent Ladeuil, #346431)
-
-* Bazaar now gives a better message including the filename if it's
-  unable to read a file in the working directory, for example because
-  of a permission error.  (Martin Pool, #338653)
-
-* ``bzr cat -r<old> <path>`` doesn't traceback anymore when <path> has a
-  file id in the working tree different from the one in revision <old>.
-  (Vincent Ladeuil, #341517, #253806)
-
-* ``bzr send`` help is more specific about how to apply merge
-  directives.  (Neil Martinsen-Burrell, #253470)
-
-* ``bzr missing`` now uses ``Repository.get_revision_delta()`` rather
-  than fetching trees and determining a delta itself. (Jelmer
-  Vernooij, #315048)
-
-* ``bzr push`` to a smart server no longer causes "Revision
-  {set([('null:',)])} not present ..." errors when the branch has
-  multiple root revisions. (Andrew Bennetts, #317654)
-
-* ``bzr shelve`` now properly handle patches with no terminating newline.
-  (Benoît PIERRE, #303569)
-
-* ``bzr unshelve`` gives a more palatable error if passed a non-integer
-  shelf id. (Daniel Watkins)
-
-* Export now handles files that are not present in the tree.
-  (James Westby, #174539)
-
-* Fixed incorrect "Source format does not support stacking" warning
-  when pushing to a smart server.  (Andrew Bennetts, #334114)
-  
-* Fixed "sprout() got an unexpected keyword argument 'source_branch'"
-  error branching from old repositories.
-  (Martin Pool, #321695)
-
-* Make ``bzr push --quiet <non-local location>`` less chatty.
-  (Kent Gibson, #221461)
-
-* Many Branch hooks would not fire with ``bzr://`` and ``bzr+ssh://``
-  branches, and this was not noticed due to a bug in the test logic
-  for branches. This is now fixed and a test added to prevent it
-  reoccuring. (Robert Collins, Andrew Bennetts)
-
-* Restore the progress bar on Windows. We were disabling it when TERM
-  wasn't set, but Windows doesn't set TERM. (Alexander Belchenko,
-  #334808)
-
-* ``setup.py build_ext`` now gives a proper error when an extension
-  fails to build. (John Arbash Meinel)
-
-* Symlinks to non ascii file names are now supported.
-  (Robert Collins, Vincent Ladeuil, #339055, #272444)    
-
-* Under rare circumstances (aka nobody reported a bug about it), the ftp
-  transport could revert to ascii mode. It now stays in binary mode except
-  when needed.  (Vincent Ladeuil)
-
-* Unshelve does not generate warnings about progress bars.
-  (Aaron Bentley, #328148)
-
-* shelve cleans up properly when unversioned files are specified.
-  (Benoît Pierre, Aaron Bentley)
-
-Documentation
-*************
-
-* Added ``Organizing your workspace`` to the User Guide appendices,
-  summarizing some common ways of organizing trees, branches and
-  repositories and the processes/workflows implied/enabled by each.
-  (Ian Clatworthy)
-
-* Hooks can now be self documenting. ``bzrlib.hooks.Hooks.create_hook``
-  is the entry point for this feature. (Robert Collins)
-
-* The documentation for ``shelve`` and ``unshelve`` has been clarified.
-  (Daniel Watkins, #327421, #327425)
-
-API Changes
-***********
-
-* ``bzr selftest`` now fails if the bazaar sources contain trailing
-  whitespace, non-unix style line endings and files not ending in a
-  newline. About 372 files and 3243 lines with trailing whitespace was
-  updated to comply with this. The code already complied with the other
-  criteria, but now it is enforced. (Marius Kruger)
-
-* ``bzrlib.branch.PushResult`` was renamed to 
-  ``bzrlib.branch.BranchPushResult``. (Jelmer Vernooij)
-
-* ``Branch.fetch`` and ``Repository.fetch`` now return None rather
-  than a count of copied revisions and failed revisions. A while back
-  we stopped ever reporting failed revisions because we started
-  erroring instead, and the copied revisions count is not used in the
-  UI at all - indeed it only reflects the repository status not
-  changes to the branch itself. (Robert Collins)
-
-* ``Inventory.apply_delta`` now raises an AssertionError if a file-id
-  appears multiple times within the delta. (Ian Clatworthy)
-
-* MutableTree.commit now favours the "authors" argument, with the old
-  "author" argument being deprecated.
-
-* Remove deprecated EmptyTree.  (Martin Pool)
-
-* ``Repository.fetch`` now accepts an optional ``fetch_spec``
-  parameter.  A ``SearchResult`` or ``MiniSearchResult`` may be passed
-  to ``fetch_spec`` instead of a ``last_revision`` to specify exactly
-  which revisions to fetch. (Andrew Bennetts)
-
-* ``RepositoryAcquisitionPolicy.acquire_repository`` now returns a
-  tuple of ``(repository, is_new_flag)``, rather than just the
-  repository.  (Andrew Bennetts)
-
-* Revision.get_apparent_author() is now deprecated, replaced by
-  Revision.get_apparent_authors(), which returns a list. The former
-  now returns the first item that would be returned from the second.
-
-* The ``BranchBuilder`` test helper now accepts a ``timestamp``
-  parameter to ``build_commit`` and ``build_snapshot``.  (Martin Pool)
-
-* The ``_fetch_*`` attributes on ``Repository`` are now on
-  ``RepositoryFormat``, more accurately reflecting their intent (they
-  describe a disk format capability, not state of a particular
-  repository of that format). (Robert Collins)
-
-Internals
-*********
-
-* Branching from a non-stacked branch on a smart protocol is now
-  free of virtual file system methods.
-  (Robert Collins, Andrew Bennetts)
-
-* Branch and Repository creation on a bzr+ssh://server are now done
-  via RPC calls rather than VFS calls, reducing round trips for
-  pushing new branches substantially. (Robert Collins)
-
-* ``Branch.clone`` now takes the ``repository_policy`` formerly used
-  inside ``BzrDir.clone_on_transport``, allowing stacking to be
-  configured before the branch tags and revision tip are set. This
-  fixes a race condition cloning stacked branches that would cause
-  plugins to have hooks called on non-stacked instances.
-  (Robert Collins, #334187)
-
-* ``BzrDir.cloning_metadir`` now has a RPC call. (Robert Collins)
-
-* ``BzrDirFormat.__str__`` now uses the human readable description
-  rather than the sometimes-absent disk label. (Robert Collins)
-
-* ``bzrlib.fetch`` is now composed of a sender and a sink component
-  allowing for decoupling over a network connection. Fetching from
-  or into a RemoteRepository with a 1.13 server will use this to
-  stream the operation.
-  (Andrew Bennetts, Robert Collins)
-
-* ``bzrlib.tests.run_suite`` accepts a runner_class parameter
-  supporting the use of different runners. (Robert Collins)
-
-* Change how file_ids and revision_ids are interned as part of
-  inventory deserialization. Now we use the real ``intern()``, rather
-  than our own workaround that would also cache a Unicode copy of the
-  string, and never emptied the cache. This should slightly reduce
-  memory consumption. (John Arbash Meinel)
-
-* New branch method ``create_clone_on_transport`` that returns a
-  branch object. (Robert Collins)
-
-* New hook Commands['extend_command'] to allow plugins to access a
-  command object before the command is run (or help generated from
-  it), without overriding the command. (Robert Collins)
-
-* New version of the ``BzrDir.find_repository`` verb supporting
-  ``_network_name`` to support removing more _ensure_real calls.
-  (Robert Collins)
-
-* ``RemoteBranchFormat`` no longer claims to have a disk format string.
-  (Robert Collins)
-
-* ``Repository`` objects now have ``suspend_write_group`` and
-  ``resume_write_group`` methods.  These are currently only useful
-  with pack repositories. (Andrew Bennetts, Robert Collins)
-
-* ``BzrDirFormat``, ``BranchFormat`` and ``RepositoryFormat`` objects
-  now have a ``network_name`` for passing the format across RPC calls.
-  (Robert Collins, Andrew Bennetts)
-
-* ``RepositoryFormat`` objects now all have a new attribute
-  ``_serializer`` used by fetch when reserialising is required.
-  (Robert Collins, Andrew Bennetts)
-
-* Some methods have been pulled up from ``BzrBranch`` to ``Branch``
-  to aid branch types that are not bzr branch objects (like
-  RemoteBranch). (Robert Collins, Andrew Bennetts)
-
-* Test adaptation has been made consistent throughout the built in
-  tests. ``TestScenarioApplier``, ``multiply_tests_from_modules``,
-  ``adapt_tests``, ``adapt_modules`` have all been deleted. Please
-  use ``multiply_tests``, or for lower level needs ``apply_scenarios``
-  and ``apply_scenario``. (Robert Collins)
-
-* ``TestSkipped`` is now detected by TestCase and passed to the
-  ``TestResult`` by calling ``addSkip``. For older TestResult objects,
-  where ``addSkip`` is not available, ``addError`` is still called.
-  This permits test filtering in subunit to strip out skipped tests
-  resulting in a faster fix-shrink-list-run cycle. This is compatible
-  with the testtools protocol for skips. (Robert Collins)
-
-* The ``_index`` of ``KnitVersionedFiles`` now supports the ability
-  to scan an underlying index that is going to be incorporated into
-  the ``KnitVersionedFiles`` object, to determine if it has missing
-  delta references. The method is ``scan_unvalidated_index``.
-  (Andrew Bennetts, Robert Collins)
-
-* There is a RemoteSink object which handles pushing to smart servers.
-  (Andrew Bennetts, Robert Collins)
-
-* ``TransportTraceDecorator`` now logs ``put_bytes_non_atomic`` and
-  ``rmdir`` calls. (Robert Collins)
-
-* ``VersionedFiles`` record adapters have had their signature change
-  from ``(record, record.get_bytes_as(record.storage_kind))`` to
-  ``(record)`` reducing excess duplication and allowing adapters
-  to access private data in record to obtain content more
-  efficiently. (Robert Collins)
-
-* We no longer probe to see if we should create a working tree during
-  clone if we cannot get a local_abspath for the new bzrdir.
-  (Robert Collins)
-
-
-bzr 1.12
-########
-
-:Codename: 1234567890
-:1.12: 2009-02-13
-:1.12rc1: 2009-02-10
-
-This release of Bazaar contains many improvements to the speed,
-documentation and functionality of ``bzr log`` and the display of logged
-revisions by ``bzr status``.  bzr now also gives a better indication of
-progress, both in the way operations are drawn onto a text terminal, and
-by showing the rate of network IO.
-
-Changes from RC1 to Final
-*************************
-
-* ``bzr init --development-wt5[-rich-root]`` would fail because of
-  circular import errors. (John Arbash Meinel, #328135)
-
-* Expanded the help for log and added a new help topic called
-  ``log-formats``.  (Ian Clatworthy)
-
-Compatibility Breaks
-********************
-
-* By default, ``bzr status`` after a merge now shows just the pending
-  merge tip revisions. This improves the signal-to-noise ratio after
-  merging from trunk and completes much faster. To see all merged
-  revisions, use the new ``-v`` flag.  (Ian Clatworthy)
-
-* ``bzr log --line`` now shows any tags after the date and before
-  the commit message. If you have scripts which parse the output
-  from this command, you may need to adjust them accordingly.
-  (Ian Clatworthy)
-
-* ``bzr log --short`` now shows any additional revision properties
-  after the date and before the commit message.  Scripts that parse 
-  output of the log command in this situation may need to adjust.
-  (Neil Martinsen-Burrell)
-
-* The experimental formats ``1.12-preview`` and ``1.12-preview-rich-root``
-  have been renamed ``development-wt5`` and ``development-wt5-rich-root``
-  respectively, given they are not ready for release in 1.12.
-  (Ian Clatworthy)
-
-* ``read_bundle_from_url`` has been deprecated. (Vincent Ladeuil)
-
-New Features
-************
-
-* Add support for filtering ``bzr missing`` on revisions.  Remote revisions
-  can be filtered using ``bzr missing -r -20..-10`` and local revisions can
-  be filtered using ``bzr missing --my-revision -20..-10``.
-  (Marius Kruger)
-
-* ``bzr log -p`` displays the patch diff for each revision.
-  When logging a file, the diff only includes changes to that file.
-  (Ian Clatworthy, #202331, #227335)
-
-* ``bzr log`` supports a new option called ``-n N`` or ``--level N``.
-  A value of 0 (zero) means "show all nested merge revisions" while
-  a value of 1 (one) means "show just the top level". Values above
-  1 can be used to see a limited amount of nesting. That can be
-  useful for seeing the level or two below PQM submits for example.
-  To force the ``--short`` and ``--line`` formats to display all nested
-  merge revisions just like ``--long`` does by default, use a command
-  like ``bzr log --short -n0``. To display just the mainline using
-  ``--long`` format, ``bzr log --long -n1``.
-  (Ian Clatworthy)
-
-Improvements
-************
-
-* ``bzr add`` more clearly communicates success vs failure.
-  (Daniel Watkins)
-
-* ``bzr init`` will now print a little less verbose output.
-  (Marius Kruger)
-
-* ``bzr log`` is now much faster in many use cases, particularly
-  at incrementally displaying results and filtering by a
-  revision range. (Ian Clatworthy)
-
-* ``bzr log --short`` and ``bzr log --line`` now show tags, if any,
-  for each revision. The tags are shown comma-separated inside
-  ``{}``. For short format, the tags appear at the end of line
-  before the optional ``[merge]`` indicator. For line format,
-  the tags appear after the date. (Ian Clatworthy)
-
-* Progress bars now show the rate of activity for some sftp 
-  operations, and they are drawn different.  (Martin Pool, #172741)
-
-* Progress bars now show the rate of activity for urllib and pycurl based
-  http client implementations. The operations are tracked at the socket
-  level for better precision.
-  (Vincent Ladeuil)
-
-* Rule-based preferences can now accept multiple patterns for a set of
-  rules.  (Marius Kruger)
-
-* The ``ancestor:`` revision spec will now default to referring to the
-  parent of the branch if no other location is given.
-  (Daniel Watkins, #198417)
-
-* The debugger started as a result of setting ``$BZR_PDB`` works
-  around a bug in ``pdb``, http://bugs.python.org/issue4150.  The bug
-  can cause truncated tracebacks in Python versions before 2.6.
-  (Andrew Bennetts)
-
-* VirtualVersionedFiles now implements
-  ``iter_lines_added_or_present_in_keys``. This allows the creation of 
-  new branches based on stacked bzr-svn branches. (#311997)
-
-Bug Fixes
-*********
-
-* ``bzr annotate --show-ids`` doesn't give a backtrace on empty files
-  anymore.
-  (Anne Mohsen, Vincent Ladeuil, #314525)
-
-* ``bzr log FILE`` now correctly shows mainline revisions merging
-  a change to FILE when the ``--short`` and ``--line`` log formats
-  are used. (Ian Clatworthy, #317417)
-
-* ``bzr log -rX..Y FILE`` now shows the history of FILE provided
-  it existed in Y or X, even if the file has since been deleted or
-  renamed. If no range is given, the current/basis tree and
-  initial tree are searched in that order. More generally, log
-  now interprets filenames in their historical context.
-  (Ian Clatworthy, #175520)
-
-* ``bzr status`` now reports nonexistent files and continues, then
-  errors (with code 3) at the end.  (Karl Fogel, #306394)
-
-* Don't require the present compression base in knits to be the same
-  when adding records in knits. (Jelmer Vernooij, #307394)
-
-* Fix a problem with CIFS client/server lag on Windows colliding with
-  an invariant-per-process algorithm for generating AtomicFile names
-  (Adrian Wilkins, #304023)
-
-* Many socket operations now handle EINTR by retrying the operation.
-  Previously EINTR was treated as an unrecoverable failure.  There is
-  a new ``until_no_eintr`` helper function in ``bzrlib.osutils``.
-  (Andrew Bennetts)
-
-* Support symlinks with non-ascii characters in the symlink filename.
-  (Jelmer Vernooij, #319323)
-
-* There was a bug in how we handled resolving when a file is deleted
-  in one branch, and modified in the other. If there was a criss-cross
-  merge, we would cause the deletion to conflict a second time.
-  (Vincent Ladeuil, John Arbash Meinel)
-
-* There was another bug in how we chose the correct intermediate LCA in
-  criss-cross merges leading to several kind of changes be incorrectly
-  handled.
-  (John Arbash Meinel, Vincent Ladeuil)
-
-* Unshelve now handles deleted paths without crashing. (Robert Collins)
-
-Documentation
-*************
-
-* Improved plugin developer documentation.  (Martin Pool)
-
-API Changes
-***********
-
-* ``ProgressBarStack`` is deprecated; instead use
-  ``ui_factory.nested_progress_bar`` to create new progress bars.
-  (Martin Pool)
-
-* ForeignVcsMapping() now requires a ForeignVcs object as first
-  argument. (Jelmer Vernooij)
-
-* ForeignVcsMapping.show_foreign_revid() has been moved to
-  ForeignVcs. (Jelmer Vernooij)
-
-* ``read_bundle_from_url`` is deprecated in favor of
-  ``read_mergeable_from_url``.  (Vincent Ladeuil)
-
-* Revision specifiers are now registered in
-  ``bzrlib.revisionspec.revspec_registry``, and the old list of 
-  revisionspec classes (``bzrlib.revisionspec.SPEC_TYPES``) has been
-  deprecated. (Jelmer Vernooij, #321183)
-
-* The progress and UI classes have changed; the main APIs remain the
-  same but code that provides a new UI or progress bar class may
-  need to be updated.  (Martin Pool)
-
-Internals
-*********
-
-* Default User Interface (UI) is CLIUIFactory when bzr runs in a dumb
-  terminal. It is sometimes desirable do override this default by forcing
-  bzr to use TextUIFactory. This can be achieved by setting the
-  BZR_USE_TEXT_UI environment variable (emacs shells, as opposed to
-  compile buffers, are such an example).
-  (Vincent Ladeuil)
-
-* New API ``Branch.iter_merge_sorted_revisions()`` that iterates over
-  ``(revision_id, depth, revno, end_of_merge)`` tuples.
-  (Ian Clatworthy)
-
-* New ``Branch.dotted_revno_to_revision_id()`` and
-  ``Branch.revision_id_to_dotted_revno()`` APIs that pick the most
-  efficient way of doing the mapping.
-  (Ian Clatworthy)
-
-* Refactor cmd_serve so that it's a little easier to build commands that
-  extend it, and perhaps even a bit easier to read.  (Jonathan Lange)
-
-* ``TreeDelta.show()`` now accepts a ``filter`` parameter allowing log
-  formatters to retrict the output.
-  (Vincent Ladeuil)
-
-
-bzr 1.11
-########
-
-:Codename: "Eyes up!"
-:Released: 2009-01-19
-
-This first monthly release of Bazaar for 2009 improves Bazaar's operation
-in Windows, Mac OS X, and other situations where file names are matched
-without regard to capitalization: Bazaar tries to match the case of an
-existing file.  This release of Bazaar also improves the efficiency of
-Tortoise Windows Shell integration and lets it work on 64-bit platforms.
-
-The UI through which Bazaar supports historic formats has been improved,
-so 'bzr help formats' now gives a simpler and shorter list, with clear
-advice.
-
-This release also fixes a number of bugs, particularly a glitch that can
-occur when there are concurrent writes to a pack repository.
-
-Bug Fixes
-*********
-
-* Fix failing test when CompiledChunksToLines is not available.
-  (Vincent Ladeuil)
-
-* Stacked branches don't repeatedly open their transport connection.
-  (John Arbash Meinel)
-
-
-
-bzr 1.11rc1
-###########
-
-:Codename: "Eyes up!"
-:Released: 2009-01-09
-
-Changes
-*******
-
-* Formats using Knit-based repository formats are now explicitly
-  marked as deprecated. (Ian Clatworthy)
-
-New Features
-************
-
-* Add support for `bzr tags -r 1..2`, that is we now support showing
-  tags applicable for a specified revision range. (Marius Kruger)
-
-* ``authentication.conf`` now accepts pluggable read-only credential
-  stores. Such a plugin (``netrc_credential_store``) is now included,
-  handles the ``$HOME/.netrc`` file and can server as an example to
-  implement other plugins.
-  (Vincent Ladeuil)
-
-* ``shelve --list`` can now be used to list shelved changes.
-  (Aaron Bentley)
-
-Improvements
-************
-
-* Add trailing slash to directories in all output of ``bzr ls``, except
-  ``bzr ls --null``. (Gordon P. Hemsley, #306424)
-
-* ``bzr revision-info`` now supports a -d option to specify an
-  alternative branch. (Michael Hudson)
-
-* Add connection to a C++ implementation of the Windows Shell Extension
-  which is able to fully replace the current Python implemented one.
-  Advantages include 64bit support and reduction in overhead for
-  processes which drag in shell extensions.
-  (Mark Hammond)
-
-* Support the Claws mail client directly, rather than via
-  xdg-email. This prevents the display of an unnecessary modal
-  dialog in Claws, informing the user that a file has been
-  attached to the message, and works around bug #291847 in
-  xdg-utils which corrupts the destination address.
-
-* When working on a case-insensitive case-preserving file-system, as
-  commonly found with Windows, bzr will often ignore the case of the
-  arguments specified by the user in preference to the case of an existing
-  item on the file-system or in the inventory to help prevent
-  counter-intuitive behaviour on Windows. (Mark Hammond)
-
-Bug Fixes
-*********
-  
-* Allow BzrDir implementation to implement backing up of 
-  control directory. (#139691)
-
-* ``bzr push`` creating a new stacked branch will now only open a
-  single connection to the target machine. (John Arbash Meinel)
-
-* Don't call iteritems on transport_list_registry, because it may
-  change during iteration.  (Martin Pool, #277048)
-
-* Don't make a broken branch when pushing an unstackable-format branch
-  that's in a stackable shared repository to a location with default
-  stack-on location.  (Andrew Bennetts, #291046)
-
-* Don't require embedding user in HTTP(S) URLs do use authentication.conf.
-  (Ben Jansen, Vincent Ladeuil, #300347)
-
-* Fix a problem with CIFS client/server lag on windows colliding with
-  an invariant-per-process algorithm for generating AtomicFile names
-  (Adrian Wilkins, #304023)
-
-* Fix bogus setUp signature in UnavailableFTPServer.
-  (Gary van der Merwe, #313498)
-
-* Fix compilation error in ``_dirstate_helpers_c`` on SunOS/Solaris.
-  (Jari Aalto)
-
-* Fix SystemError in ``_patiencediff_c`` module by calling
-  PyErr_NoMemory() before returning NULL in PatienceSequenceMatcher_new.
-  (Andrew Bennetts, #303206)
-
-* Give proper error message for diff with non-existent dotted revno.
-  (Marius Kruger, #301969)
-
-* Handle EACCES (permission denied) errors when launching a message
-  editor, and emit warnings when a configured editor cannot be
-  started. (Andrew Bennetts)
-
-* ``$HOME/.netrc`` file is now recognized as a read-only credential store
-  if configured in ``authentication.conf`` with 'password_encoding=netrc'
-  in the appropriate sections.
-  (Vincent Ladeuil, #103029)
-
-* Opening a stacked branch now properly shares the connection, rather
-  than opening a new connection for the stacked-on branch.
-  (John Arbash meinel)
-
-* Preserve transport decorators while following redirections.
-  (Vincent Ladeuil, #245964, #270863)
-
-* Provides a finer and more robust filter for accepted redirections.
-  (Vincent Ladeuil, #303959, #265070)
-
-* ``shelve`` paths are now interpreted relative to the current working
-  tree.  (Aaron Bentley)
-
-* ``Transport.readv()`` defaults to not reading more than 100MB in a
-  single array. Further ``RemoteTransport.readv`` sets this to 5MB to
-  work better with how it splits its requests.
-  (John Arbash Meinel, #303538)
-
-* Pack repositories are now able to reload the pack listing and retry
-  the current operation if another action causes the data to be
-  repacked.  (John Arbash Meinel, #153786)
-
-* ``pull -v`` now respects the log_format configuration variable.
-  (Aaron Bentley)
-
-* ``push -v`` now works on non-initial pushes.  (Aaron Bentley)
-
-* Use the short status format when the short format is used for log.
-  (Vincent Ladeuil, #87179)
-
-* Allow files to be renamed or moved via remove + add-by-id. (Charles
-  Duffy, #314251)
-
-Documentation
-*************
-
-* Improved the formats help topic to explain why multiple formats
-  exist and to provide guidelines in selecting one. Introduced
-  two new supporting help topics: current-formats and other-formats.
-  (Ian Clatworthy)
-
-API Changes
-***********
-
-* ``LRUCache(after_cleanup_size)`` was renamed to
-  ``after_cleanup_count`` and the old name deprecated. The new name is
-  used for clarity, and to avoid confusion with
-  ``LRUSizeCache(after_cleanup_size)``. (John Arbash Meinel)
-
-* New ``ForeignRepository`` base class, to help with foreign branch 
-  support (e.g. svn).  (Jelmer Vernooij)
-
-* ``node_distances`` and ``select_farthest`` can no longer be imported
-  from ``bzrlib.graph``.  They can still be imported from
-  ``bzrlib.deprecated_graph``, which has been the preferred way to
-  import them since before 1.0.  (Andrew Bennetts)
-  
-* The logic in commit now delegates inventory basis calculations to
-  the ``CommitBuilder`` object; this requires that the commit builder
-  in use has been updated to support the new ``recording_deletes`` and
-  ``record_delete`` methods. (Robert Collins)
-
-Testing
-*******
-
-* An HTTPS server is now available (it requires python-2.6). Future bzr
-  versions will allow the use of the python-2.6 ssl module that can be
-  installed for 2.5 and 2.4.
-
-* ``bzr selftest`` now fails if new trailing white space is added to
-  the bazaar sources. It only checks changes not committed yet. This
-  means that PQM will now reject changes that introduce new trailing
-  whitespace. (Marius Kruger)
-
-* Introduced new experimental formats called ``1.12-preview`` and
-  ``1.12-preview-rich-root`` to enable testing of related pending
-  features, namely content filtering and filtered views.
-  (Ian Clatworthy)
-
-Internals
-*********
-
-* Added an ``InventoryEntry`` cache when deserializing inventories.
-  Can cut the time to iterate over multiple RevisionsTrees in half.
-  (John Arbash Meinel)
-
-* Added ``bzrlib.fifo_cache.FIFOCache`` which is designed to have
-  minimal overhead versus using a plain dict for cache hits, at the
-  cost of not preserving the 'active' set as well as an ``LRUCache``.
-  (John Arbash Meinel)
-
-* ``bzrlib.patience_diff.unified_diff`` now properly uses a tab
-  character to separate the filename from the date stamp, and doesn't
-  add trailing whitespace when a date stamp is not supplied.
-  (Adeodato Simó, John Arbash Meinel)
-
-* ``DirStateWorkingTree`` and ``DirStateWorkingTreeFormat`` added
-  as base classes of ``WorkingTree4`` and ``WorkingTreeFormat4``
-  respectively. (Ian Clatworthy)
-
-* ``KnitVersionedFiles._check_should_delta()`` now uses the
-  ``get_build_details`` api to avoid multiple hits to the index, and
-  to properly follow the ``compression_parent`` rather than assuming
-  it is the left-hand parent. (John Arbash Meinel)
-
-* ``KnitVersionedFiles.get_record_stream()`` will now chose a
-  more optimal ordering when the keys are requested 'unordered'.
-  Previously the order was fully random, now the records should be
-  returned from each pack in turn, in forward I/O order.
-  (John Arbash Meinel)
-    
-* ``mutter()`` will now flush the ``~/.bzr.log`` if it has been more
-  than 2s since the last time it flushed. (John Arbash Meinel)
-
-* New method ``bzrlib.repository.Repository.add_inventory_by_delta``
-  allows adding an inventory via an inventory delta, which can be
-  more efficient for some repository types. (Robert Collins)
-
-* Repository ``CommitBuilder`` objects can now accumulate an inventory
-  delta. To enable this functionality call ``builder.recording_deletes``
-  and additionally call ``builder.record_delete`` when a delete
-  against the basis occurs. (Robert Collins)
-
-* The default http handler has been changed from pycurl to urllib.
-  The default is still pycurl for https connections. (The only
-  advantage of pycurl is that it checks ssl certificates.)
-  (John Arbash Meinel)
-
-* ``VersionedFiles.get_record_stream()`` can now return objects with a
-  storage_kind of ``chunked``. This is a collection (list/tuple) of
-  strings. You can use ``osutils.chunks_to_lines()`` to turn them into
-  guaranteed 'lines' or you can use ``''.join(chunks)`` to turn it
-  into a fulltext. This allows for some very good memory savings when
-  asking for many texts that share ancestry, as the individual chunks
-  can be shared between versions of the file. (John Arbash Meinel)
-
-* ``pull -v`` and ``push -v`` use new function
-  ``bzrlib.log.show_branch_change`` (Aaron Bentley)
-
-
-
-bzr 1.10
-########
-
-:Released: 2008-12-05
-
-Bazaar 1.10 has several performance improvements for copying revisions
-(especially for small updates to large projects).  There has also been a
-significant amount of effort in polishing stacked branches.  The commands
-``shelve`` and ``unshelve`` have become core commands, with an improved
-implementation.
-
-The only changes versus bzr-1.10rc1 are bugfixes for stacked branches.
-
-bug Fixes
-*********
-
-* Don't set a pack write cache size from RepoFetcher, because the
-  cache is not coherent with reads and causes ShortReadvErrors.
-  This reverses the change that fixed #294479.
-  (Martin Pool, #303856)
-
-* Properly handle when a revision can be inserted as a delta versus
-  when it needs to be expanded to a fulltext for stacked branches.
-  There was a bug involving merge revisions. As a method to help
-  prevent future difficulties, also make stacked fetches sort
-  topologically. (John Arbash Meinel, #304841)
-
-
-bzr 1.10rc1
-###########
-
-:Released: 2008-11-28
-
-This release of Bazaar focuses on performance improvements when pushing
-and pulling revisions, both locally and to remote networks.  The popular
-``shelve`` and ``unshelve`` commands, used to interactively revert and
-restore work in progress, have been merged from bzrtools into the bzr
-core.  There are also bug fixes for portability, and for stacked branches.
-
-New Features
-************
-
-* New ``commit_message_template`` hook that is called by the commit
-  code to generate a template commit message. (Jelmer Vernooij)
-
-* New `shelve` and `unshelve` commands allow undoing and redoing changes.
-  (Aaron Bentley)
-
-Improvements
-************
-
-* ``(Remote)Branch.copy_content_into`` no longer generates the full revision
-  history just to set the last revision info.
-  (Andrew Bennetts, John Arbash Meinel)
-
-* Fetches between formats with different serializers (such as
-  pack-0.92-subtree and 1.9-rich-root) are faster now.  This is due to
-  operating on batches of 100 revisions at time rather than
-  one-by-one.  (Andrew Bennetts, John Arbash Meinel)
-
-* Search index files corresponding to pack files we've already used
-  before searching others, because they are more likely to have the
-  keys we're looking for.  This reduces the number of iix and tix
-  files accessed when pushing 1 new revision, for instance.
-  (John Arbash Meinel)
-
-* Signatures to transfer are calculated more efficiently in
-  ``item_keys_introduced_by``.  (Andrew Bennetts, John Arbash Meinel)
-
-* The generic fetch code can once again copy revisions and signatures
-  without extracting them completely to fulltexts and then serializing
-  them back down into byte strings. This is a significant performance
-  improvement when fetching from a stacked branch.
-  (John Arbash Meinel, #300289)
-
-* When making a large readv() request over ``bzr+ssh``, break up the
-  request into more manageable chunks. Because the RPC is not yet able
-  to stream, this helps keep us from buffering too much information at
-  once. (John Arbash Meinel)
-
-Bug Fixes
-*********
-
-* Better message when the user needs to set their Launchpad ID.
-  (Martin Pool, #289148)
-
-* ``bzr commit --local`` doesn't access the master branch anymore.
-  This fixes a regression introduced in 1.9.  (Marius Kruger, #299313)
-
-* Don't call the system ``chdir()`` with an empty path. Sun OS seems
-  to give an error in that case.  Also, don't count on ``getcwd()``
-  being able to allocate a new buffer, which is a gnu extension.
-  (John Arbash Meinel, Martin Pool, Harry Hirsch, #297831)
-
-* Don't crash when requesting log --forward <file> for a revision range
-  starting with a dotted revno.
-  (Vincent Ladeuil, #300055)
-
-* Don't create text deltas spanning stacked repositories; this could
-  cause "Revision X not present in Y" when later accessing them.
-  (Martin Pool, #288751)
-
-* Pack repositories are now able to reload the pack listing and retry
-  the current operation if another action causes the data to be
-  repacked.  (John Arbash Meinel, #153786)
-
-* PermissionDenied errors from smart servers no longer cause
-  "PermissionDenied: "None"" on the client.
-  (Andrew Bennetts, #299254)
-
-* Pushing to a stacked pack repository now batches writes, the same
-  way writes are batched to ordinary pack repository.  This makes
-  pushing to a stacked branch over the network much faster.
-  (Andrew Bennetts, #294479)
-
-* TooManyConcurrentRequests no longer occur when a fetch fails and
-  tries to abort a write group.  This allows the root cause (e.g. a
-  network interruption) to be reported.  (Andrew Bennetts, #297014)
-
-* RemoteRepository.get_parent_map now uses fallback repositories.
-  (Aaron Bentley, #297991?, #293679?)
-
-API Changes
-***********
-
-* ``CommitBuilder`` now validates the strings it will be committing,
-  to ensure that they do not have characters that will not be properly
-  round-tripped. For now, it just checks for characters that are
-  invalid in the XML form. (John Arbash Meinel, #295161)
-
-* Constructor parameters for NewPack (internal to pack repositories)
-  have changed incompatibly.
-
-* ``Repository.abort_write_group`` now accepts an optional
-  ``suppress_errors`` flag.  Repository implementations that override
-  ``abort_write_group`` will need to be updated to accept the new
-  argument.  Subclasses that only override ``_abort_write_group``
-  don't need to change.
-
-* Transport implementations must provide copy_tree_to_transport.  A default
-  implementation is provided for Transport subclasses.
-
-Testing
-*******
-
-* ``bzr selftest`` now fails if no doctests are found in a module
-  that's expected to have them.  (Martin Pool)
-
-* Doctests now only report the first failure.  (Martin Pool)
-
-
-bzr 1.9
-#######
-
-:Released: 2008-11-07
-
-This release of Bazaar adds a new repository format, ``1.9``, with smaller
-and more efficient index files.  This format can be specified when
-creating a new repository, or used to losslessly upgrade an existing
-repository.  bzr 1.9 also speeds most operations over the smart server
-protocol, makes annotate faster, and uses less memory when making
-checkouts or pulling large amounts of data.
-
-Bug Fixes
-*********
-
-* Fix "invalid property value 'branch-nick' for None" regression with
-  branches bound to svn branches.  (Martin Pool, #293440)
-
-* Fix SSL/https on Python2.6.  (Vincent Ladeuil, #293054)
-
-* ``SFTPTransport.readv()`` had a bug when requests were out-of-order.
-  This only triggers some-of-the-time on Knit format repositories.
-  (John Arbash Meinel, #293746)
-
-
-bzr 1.9rc1
-##########
-
-:Released: 2008-10-31
-
-New Features
-************
-
-* New Branch hook ``transform_fallback_location`` allows a function to
-  be called when looking up the stacked source. (Michael Hudson)
-
-* New repository formats ``1.9`` and ``1.9-rich-root``. These have all
-  the functionality of ``1.6``, but use the new btree indexes.
-  These indexes are both smaller and faster for access to historical
-  information.  (John Arbash Meinel)
-
-Improvements
-************
-
-* ``BTreeIndex`` code now is able to prefetch extra pages to help tune
-  the tradeoff between bandwidth and latency. Should be tuned
-  appropriately to not impact commands which need minimal information,
-  but provide a significant boost to ones that need more context. Only
-  has a direct impact on the ``--development2`` format which uses
-  btree's for the indexes. (John Arbash Meinel)
-
-* ``bzr dump-btree`` is a hidden command introduced to allow dumping
-  the contents of a compressed btree file.  (John Arbash Meinel)
-
-* ``bzr pack`` now tells the index builders to optimize for size. For
-  btree index repositories, this can save 25% of the index size
-  (mostly in the text indexes). (John Arbash Meinel)
-
-* ``bzr push`` to an existing branch or repository on a smart server
-  is faster, due to Bazaar making more use of the ``get_parent_map``
-  RPC when querying the remote branch's revision graph.
-  (Andrew Bennetts)
-
-* default username for bzr+ssh and sftp can be configured in
-  authentication.conf. (Aaron Bentley)
-
-* launchpad-login now provides a default username for bzr+ssh and sftp
-  URLs, allowing username-free URLs to work for everyone. (Aaron Bentley)
-
-* ``lp:`` lookups no longer include usernames, making them shareable and
-  shorter. (Aaron Bentley)
-
-* New ``PackRepository.autopack`` smart server RPC, which does
-  autopacking entirely on the server.  This is much faster than
-  autopacking via plain file methods, which downloads a large amount
-  of pack data and then re-uploads the same pack data into a single
-  file.  This fixes a major (although infrequent) cause of lengthy
-  delays when using a smart server.  For example, pushing the 10th
-  revision to a repository with 9 packs now takes 44 RPCs rather than
-  179, and much less bandwidth too.  This requires Bazaar 1.9 on both
-  the client and the server, otherwise the client will fallback to the
-  slower method.  (Andrew Bennetts)
-
-Bug Fixes
-*********
-
-* A failure to load a plugin due to an IncompatibleAPI exception is
-  now correctly reported. (Robert Collins, #279451)
-
-* API versioning support now has a multiple-version checking api
-  ``require_any_api``. (Robert Collins, #279447)
-
-* ``bzr branch --stacked`` from a smart server to a standalone branch
-  works again.  This fixes a regression in 1.7 and 1.8.
-  (Andrew Bennetts, #270397)
-
-* ``bzr co`` uses less memory. It used to unpack the entire WT into
-  memory before writing it to disk. This was a little bit faster, but
-  consumed lots of memory. (John Arbash Meinel, #269456)
-
-* ``bzr missing --quiet`` no longer prints messages about whether
-  there are missing revisions.  The exit code indicates whether there
-  were or not.  (Martin Pool, #284748)
-
-* Fixes to the ``annotate`` code. The fast-path which re-used the
-  stored deltas was accidentally disabled all the time, instead of
-  only when a branch was stacked. Second, the code would accidentally
-  re-use a delta even if it wasn't against the left-parent, this
-  could only happen if ``bzr reconcile`` decided that the parent
-  ordering was incorrect in the file graph.  (John Arbash Meinel)
-
-* "Permission denied" errors that occur when pushing a new branch to a
-  smart server no longer cause tracebacks.  (Andrew Bennetts, #278673)
-
-* Some compatibility fixes for building the extensions with MSVC and
-  for python2.4. (John Arbash Meinel, #277484)
-
-* The index logic is now able to reload the list of pack files if and
-  index ends up disappearing. We still don't reload if the pack data
-  itself goes missing after checking the index. This bug appears as a
-  transient failure (file not found) when another process is writing
-  to the repository.  (John Arbash Meinel, #153786)
-
-* ``bzr switch`` and ``bzr bind`` will now update the branch nickname if
-  it was previously set. All checkouts will now refer to the bound branch
-  for a nickname if one was not explicitly set.
-  (Marius Kruger, #230903)
-
-Documentation
-*************
-
-* Improved hook documentation. (Michael Ernst)
-
-API Changes
-***********
-
-* commands.plugins_cmds is now a CommandRegistry, not a dict.
-
-Internals
-*********
-
-* New AuthenticationConfig.set_credentials method allows easy programmatic
-  configuration of authetication credentials.
-
-
-bzr 1.8
-#######
-
-:Released: 2008-10-16
-
-Bazaar 1.8 includes several fixes that improve working tree performance,
-display of revision logs, and merges.  The bzr testsuite now passes on OS
-X and Python 2.6, and almost completely passes on Windows.  The
-smartserver code has gained several bug fixes and performance
-improvements, and can now run server-side hooks within an http server.
-
-Bug Fixes
-*********
-
-* Fix "Must end write group" error when another error occurs during
-  ``bzr push``.  (Andrew Bennetts, #230902)
-
-Portability
-***********
-
-* Some Pyrex versions require the WIN32 macro defined to compile on
-  that platform.  (Alexander Belchenko, Martin Pool, #277481)
-
-
-bzr 1.8rc1
-##########
-
-:Released: 2008-10-07
-
-Changes
-*******
-
-* ``bzr log file`` has been changed. It now uses a different method
-  for determining which revisions to show as merging the changes to
-  the file. It now only shows revisions which merged the change
-  towards your mainline. This simplifies the output, makes it faster,
-  and reduces memory consumption.  (John Arbash Meinel)
-
-* ``bzr merge`` now defaults to having ``--reprocess`` set, whenever
-  ``--show-base`` is not supplied.  (John Arbash Meinel)
-
-* ``bzr+http//`` will now optionally load plugins and write logs on the
-  server. (Marius Kruger)
-
-* ``bzrlib._dirstate_helpers_c.pyx`` does not compile correctly with
-  Pyrex 0.9.4.1 (it generates C code which causes segfaults). We
-  explicitly blacklist that version of the compiler for that
-  extension. Packaged versions will include .c files created with
-  pyrex >= 0.9.6 so it doesn't effect releases, only users running
-  from the source tree. (John Arbash Meinel, #276868)
-
-Features
-********
-
-* bzr is now compatible with python-2.6. python-2.6 is not yet officially
-  supported (nor released, tests were conducted with the dev version of
-  python-2.6rc2), but all known problems have been fixed.  Feedback
-  welcome.
-  (Vincent Ladeuil, #269535)
-
-Improvements
-************
-
-* ``bzr annotate`` will now include uncommitted changes from the local
-  working tree by default. Such uncommitted changes are given the
-  revision number they would get if a commit was done, followed with a
-  ? to indicate that its not actually known. (Robert Collins, #3439)
-
-* ``bzr branch`` now accepts a ``--standalone`` option, which creates a
-  standalone branch regardless of the presence of shared repositories.
-  (Daniel Watkins)
-
-* ``bzr push`` is faster in the case there are no new revisions to
-  push.  It is also faster if there are no tags in the local branch.
-  (Andrew Bennetts)
-
-* File changes during a commit will update the tree stat cache.
-  (Robert Collins)
-
-* Location aliases can now accept a trailing path.  (Micheal Hudson)
-
-* New hooks ``Lock.hooks`` when LockDirs are acquired and released.
-  (Robert Collins, MartinPool)
-
-* Switching in heavyweight checkouts uses the master branch's context, not
-  the checkout's context.  (Adrian Wilkins)
-
-* ``status`` on large trees is now faster, due to optimisations in the
-  walkdirs code. Of particular note, the walkdirs code now performs
-  a temporary ``chdir()`` while reading a single directory; if your
-  platform has non thread-local current working directories (and is
-  not windows which has its own implementation), this may introduce a
-  race condition during concurrent uses of bzrlib. The bzrlib CLI
-  will not encounter this as it is single threaded for working tree
-  operations. (Robert Collins)
-
-* The C extensions now build on python 2.4 (Robert Collins, #271939)
-
-* The ``-Dhpss`` debug flag now reports the number of smart server
-  calls per medium to stderr.  This is in addition to the existing
-  detailed logging to the .bzr.log trace file.  (Andrew Bennetts)
-
-Bug Fixes
-*********
-
-* Avoid random failures arising from misinterpreted ``errno`` values
-  in ``_readdir_pyx.read_dir``.
-  (Martin Pool, #279381)
-
-* Branching from a shared repository on a smart server into a new
-  repository now preserves the repository format.
-  (Andrew Bennetts, #269214)
-
-* ``bzr log`` now accepts a ``--change`` option.
-  (Vincent Ladeuil, #248427)
-
-* ``bzr missing`` now accepts an ``--include-merges`` option.
-  (Vincent Ladeuil, #233817)
-
-* Don't try to filter (internally) '.bzr' from the files to be deleted if
-  it's not there.
-  (Vincent Ladeuil, #272648)
-
-* Fix '_in_buffer' AttributeError when using the -Dhpss debug flag.
-  (Andrew Bennetts)
-
-* Fix TooManyConcurrentRequests errors caused by a connection failure
-  when doing ``bzr pull`` or ``bzr merge`` from a ``bzr+ssh`` URL.
-  (Andrew Bennetts, #246233)
-
-* Fixed ``bzr st -r branch:PATH_TO_BRANCH`` where the other branch
-  is in a different repository than the current one.
-  (Lukáš Lalinský, #144421)
-
-* Make the first line of the manpage preamble a comment again.
-  (David Futcher, #242106)
-
-* Remove use of optional parameter in GSSAPI FTP support, since
-  it breaks newer versions of Python-Kerberos. (Jelmer Vernooij)
-
-* The autopacking logic will now always create a single new pack from
-  all of the content which it deems is worth moving. This avoids the
-  'repack a single pack' bug and should result in better packing
-  overall.  (John Arbash Meinel, #242510, #172644)
-
-* Trivial documentation fix.
-  (John Arbash Meinel, #270471)
-
-* ``bzr switch`` and ``bzr bind`` will now update the branch nickname if
-  it was previously set. All checkouts will now refer to the bound branch
-  for a nickname if one was not explicitly set.
-  (Marius Kruger, #230903)
-
-Documentation
-*************
-
-* Explain revision/range identifiers. (Daniel Clemente)
-
-API Changes
-***********
-
-* ``CommitBuilder.record_entry_contents`` returns one more element in
-  its result tuple - an optional file system hash for the hash cache
-  to use. (Robert Collins)
-
-* ``dirstate.DirState.update_entry`` will now only calculate the sha1
-  of a file if it is likely to be needed in determining the output
-  of iter_changes. (Robert Collins)
-
-* The PackRepository, RepositoryPackCollection, NewPack classes have a
-  slightly changed interface to support different index types; as a
-  result other users of these classes need to supply the index types
-  they want. (Robert Collins)
-
-Testing
-*******
-
-* ``bzrlib.tests.repository_implementations`` has been renamed to
-  ``bzrlib.tests.per_repository`` so that we have a common structure
-  (and it is shorter). (John Arbash Meinel, #239343)
-
-* ``LocalTransport.abspath()`` now returns a drive letter if the
-  transport has one, fixing numerous tests on Windows.
-  (Mark Hammond)
-
-* PreviewTree is now tested via intertree_implementations.
-  (Aaron Bentley)
-
-* The full test suite is passing again on OSX.
-  (Guillermo Gonzalez, Vincent Ladeuil)
-
-* The full test suite passes when run with ``-Eallow_debug``.
-  (Andrew Bennetts)
-
-Internals
-*********
-
-* A new hook, ``Branch.open``, has been added, which is called when
-  branch objects are opened. (Robert Collins)
-
-* ``bzrlib.osutils._walkdirs_utf8`` has been refactored into common
-  tree walking, and modular directory listing code to aid future
-  performance optimisations and refactoring. (Robert Collins)
-
-* ``bzrlib.trace.debug_memory`` can be used to get a quick memory dump
-  in the middle of processing. It only reports memory if
-  ``/proc/PID/status`` is available. (John Arbash Meinel)
-
-* New method ``RevisionSpec.as_tree`` for representing the revision
-  specifier as a revision tree object. (Lukáš Lalinský)
-
-* New race-free method on MutableTree ``get_file_with_stat`` for use
-  when generating stat cache results. (Robert Collins)
-
-* New win32utils.get_local_appdata_location() provides access to a local
-  directory for storing data.  (Mark Hammond)
-
-* To be compatible with python-2.6 a few new rules should be
-  observed. 'message' attribute can't be used anymore in exception
-  classes, 'sha' and 'md5' modules have been deprecated (use
-  osutils.[md5|sha]), object__init__ and object.__new__ don't accept
-  parameters anymore.
-  (Vincent Ladeuil)
-
-
-bzr 1.7.1
-#########
-
-:Released:  2008-10-01
-
-No changes from 1.7.1rc1.
-
-
-bzr 1.7.1rc1
-############
-
-:Released: 2008-09-24
-
-This release just includes an update to how the merge algorithm handles
-file paths when we encounter complex history.
-
-Features
-********
-
-* If we encounter a criss-cross in history, use information from
-  direct Least Common Ancestors to resolve inventory shape (locations
-  of files, adds, deletes, etc). This is similar in concept to using
-  ``--lca`` for merging file texts, only applied to paths.
-  (John Arbash Meinel)
-
-
-bzr 1.7
-#######
-
-:Released: 2008-09-23
-
-This release includes many bug fixes and a few performance and feature
-improvements.  ``bzr rm`` will now scan for missing files and remove them,
-like how ``bzr add`` scans for unknown files and adds them. A bit more
-polish has been applied to the stacking code. The b-tree indexing code has
-been brought in, with an eye on using it in a future repository format.
-There are only minor installer changes since bzr-1.7rc2.
-
-Features
-********
-
-* Some small updates to the win32 installer. Include localization
-  files found in plugins, and include the builtin distutils as part of
-  packaging qbzr. (Mark Hammond)
-
-
-bzr 1.7rc2
-##########
-
-:Released: 2008-09-17
-
-A few bug fixes from 1.7rc1. The biggest change is a new
-``RemoteBranch.get_stacked_on_url`` rpc. This allows clients that are
-trying to access a Stacked branch over the smart protocol, to properly
-connect to the stacked-on location.
-
-Bug Fixes
-*********
-
-* Branching from a shared repository on a smart server into a new
-  repository now preserves the repository format.
-  (Andrew Bennetts, #269214)
-
-* Branching from a stacked branch via ``bzr+ssh`` can properly connect
-  to the stacked-on branch.  (Martin Pool, #261315)
-
-* ``bzr init`` no longer re-opens the BzrDir multiple times.
-  (Vincent Ladeuil)
-
-* Fix '_in_buffer' AttributeError when using the -Dhpss debug flag.
-  (Andrew Bennetts)
-
-
-bzr 1.7rc1
-##########
-
-:Released: 2008-09-09
-
-This release candidate for bzr 1.7 has several bug fixes and a few
-performance and feature improvements.  ``bzr rm`` will now scan for
-missing files and remove them, like how ``bzr add`` scans for unknown
-files and adds them. A bit more polish has been applied to the stacking
-code. The b-tree indexing code has been brought in, with an eye on using
-it in a future repository format.
-
-
-Changes
-*******
-
-* ``bzr export`` can now export a subdirectory of a project.
-  (Robert Collins)
-
-* ``bzr remove-tree`` will now refuse to remove a tree with uncommitted
-  changes, unless the ``--force`` option is specified.
-  (Lukáš Lalinský, #74101)
-
-* ``bzr rm`` will now scan for files that are missing and remove just
-  them automatically, much as ``bzr add`` scans for new files that
-  are not ignored and adds them automatically. (Robert Collins)
-
-Features
-********
-
-* Support for GSSAPI authentication when using FTP as documented in
-  RFC2228. (Jelmer Vernooij, #49623)
-
-* Add support for IPv6 in the smart server. (Jelmer Vernooij, #165014)
-
-Improvements
-************
-
-* A url like ``log+file:///tmp`` will log all access to that Transport
-  to ``.bzr.log``, which may help in debugging or profiling.
-  (Martin Pool)
-
-* ``bzr branch`` and ``bzr push`` use the default stacking policy if the
-  branch format supports it. (Aaron Bentley)
-
-* ``bzr init`` and ``bzr init-repo`` will now print out the same as
-  ``bzr info`` if it completed successfully.
-  (Marius Kruger)
-
-* ``bzr uncommit`` logs the old tip revision id, and displays how to
-  restore the branch to that tip using ``bzr pull``.  This allows you
-  to recover if you realize you uncommitted the wrong thing.
-  (John Arbash Meinel)
-
-* Fix problems in accessing stacked repositories over ``bzr://``.
-  (Martin Pool, #261315)
-
-* ``SFTPTransport.readv()`` was accidentally using ``list += string``,
-  which 'works', but adds each character separately to the list,
-  rather than using ``list.append(string)``. Fixing this makes the
-  SFTP transport a little bit faster (~20%) and use a bit less memory.
-  (John Arbash Meinel)
-
-* When reading index files, if we happen to read the whole file in a
-  single request treat it as a ``_buffer_all`` request. This happens
-  most often on small indexes over remote transports, where we default
-  to reading 64kB. It saves a round trip for each small index during
-  fetch operations. Also, if we have read more than 50% of an index
-  file, trigger a ``_buffer_all`` on the next request. This works
-  around some inefficiencies because reads don't fall neatly on page
-  boundaries, so we would ignore those bytes, but request them again
-  later. This could trigger a total read size of more than the whole
-  file. (John Arbash Meinel)
-
-Bug Fixes
-*********
-
-* ``bzr rm`` is now aliased to ``bzr del`` for the convenience of svn
-  users. (Robert Collins, #205416)
-
-* Catch the infamous "select/poll returned error" which occurs when
-  pycurl try to send a body request to an HTTP/1.0 server which has
-  already refused to handle the request. (Vincent Ladeuil, #225020)
-
-* Fix ``ObjectNotLocked`` errors when using various commands
-  (including ``bzr cat`` and ``bzr annotate``) in combination with a
-  smart server URL.  (Andrew Bennetts, #237067)
-
-* ``FTPTransport.stat()`` would return ``0000`` as the permission bits
-  for the containing ``.bzr/`` directory (it does not implement
-  permissions). This would cause us to set all subdirectories to
-  ``0700`` and files to ``0600`` rather than leaving them unmodified.
-  Now we ignore ``0000`` as the permissions and assume they are
-  invalid. (John Arbash Meinel, #259855)
-
-* Merging from a previously joined branch will no longer cause
-  a traceback. (Jelmer Vernooij, #203376)
-
-* Pack operations on windows network shares will work even with large
-  files. (Robert Collins, #255656)
-
-* Running ``bzr st PATH_TO_TREE`` will no longer suppress merge
-  status. Status is also about 7% faster on mozilla sized trees
-  when the path to the root of the tree has been given. Users of
-  the internal ``show_tree_status`` function should be aware that
-  the show_pending flag is now authoritative for showing pending
-  merges, as it was originally. (Robert Collins, #225204)
-
-* Set valid default _param_name for Option so that ListOption can embed
-  '-' in names. (Vincent Ladeuil, #263249)
-
-* Show proper error rather than traceback when an unknown revision
-  id is specified to ``bzr cat-revision``. (Jelmer Vernooij, #175569)
-
-* Trailing text in the dirstate file could cause the C dirstate parser
-  to try to allocate an invalid amount of memory. We now properly
-  check and test for parsing a dirstate with invalid trailing data.
-  (John Arbash Meinel, #186014)
-
-* Unexpected error responses from a smart server no longer cause the
-  client to traceback.  (Andrew Bennetts, #263527)
-
-* Use a Windows api function to get a Unicode host name, rather than
-  assuming the host name is ascii.
-  (Mark Hammond, John Arbash Meinel, #256550)
-
-* ``WorkingTree4`` trees will now correctly report missing-and-new
-  paths in the output of ``iter_changes``. (Robert Collins)
-
-Documentation
-*************
-
-* Updated developer documentation.  (Martin Pool)
-
-API Changes
-***********
-
-* Exporters now take 4 parameters. (Robert Collins)
-
-* ``Tree.iter_changes`` will now return False for the content change
-  field when a file is missing in the basis tree and not present in
-  the target tree. Previously it returned True unconditionally.
-  (Robert Collins)
-
-* The deprecated ``Branch.abspath`` and unimplemented
-  ``Branch.rename_one`` and ``Branch.move`` were removed. (Jelmer Vernooij)
-
-* BzrDir.clone_on_transport implementations must now accept a stacked_on
-  parameter.  (Aaron Bentley)
-
-* BzrDir.cloning_metadir implementations must now take a require_stacking
-  parameter.  (Aaron Bentley)
-
-Testing
-*******
-
-* ``addCleanup`` now takes ``*arguments`` and ``**keyword_arguments``
-  which are then passed to the cleanup callable as it is run. In
-  addition, addCleanup no longer requires that the callables passed to
-  it be unique. (Jonathan Lange)
-
-* Fix some tests that fail on Windows because files are deleted while
-  still in use.
-  (Mark Hammond)
-
-* ``selftest``'s ``--starting-with`` option can now use predefined
-  prefixes so that one can say ``bzr selftest -s bp.loom`` instead of
-  ``bzr selftest -s bzrlib.plugins.loom``. (Vincent Ladeuil)
-
-* ``selftest``'s ``--starting-with`` option now accepts multiple values.
-  (Vincent Ladeuil)
-
-Internals
-*********
-
-* A new plugin interface, ``bzrlib.log.log_adapters``, has been added.
-  This allows dynamic log output filtering by plugins.
-  (Robert Collins)
-
-* ``bzrlib.btree_index`` is now available, providing a b-tree index
-  layer. The design is memory conservative (limited memory cache),
-  faster to seek (approx 100 nodes per page, gives 100-way fan out),
-  and stores compressed pages allowing more keys per page.
-  (Robert Collins, John Arbash Meinel)
-
-* ``bzrlib.diff.DiffTree.show_diff`` now skips changes where the kind
-  is unknown in both source and target.
-  (Robert Collins, Aaron Bentley)
-
-* ``GraphIndexBuilder.add_node`` and ``BTreeBuilder`` have been
-  streamlined a bit. This should make creating large indexes faster.
-  (In benchmarking, it now takes less time to create a BTree index than
-  it takes to read the GraphIndex one.) (John Arbash Meinel)
-
-* Mail clients for `bzr send` are now listed in a registry.  This
-  allows plugins to add new clients by registering them with
-  ``bzrlib.mail_client.mail_client_registry``.  All of the built-in
-  clients now use this mechanism.  (Neil Martinsen-Burrell)
-
-
-bzr 1.6.1
-#########
-
-:Released: 2008-09-05
-
-A couple regressions were found in the 1.6 release. There was a
-performance issue when using ``bzr+ssh`` to branch large repositories,
-and some problems with stacking and ``rich-root`` capable repositories.
-
-
-bzr 1.6.1rc2
-############
-
-:Released: 2008-09-03
-
-Bug Fixes
-*********
-
-* Copying between ``rich-root`` and ``rich-root-pack`` (and vice
-  versa) was accidentally using the inter-model fetcher, instead of
-  recognizing that both were 'rich root' formats.
-  (John Arbash Meinel, #264321)
-
-
-bzr 1.6.1rc1
-############
-
-:Released: 2008-08-29
-
-This release fixes a few regressions found in the 1.6 client. Fetching
-changes was using an O(N^2) buffering algorithm, so for large projects it
-would cause memory thrashing. There is also a specific problem with the
-``--1.6-rich-root`` format, which prevented stacking on top of
-``--rich-root-pack`` repositories, and could allow users to accidentally
-fetch experimental data (``-subtree``) without representing it properly.
-The ``--1.6-rich-root`` format has been deprecated and users are
-recommended to upgrade to ``--1.6.1-rich-root`` immediately.  Also we
-re-introduced a workaround for users who have repositories with incorrect
-nodes (not possible if you only used official releases).
-I should also clarify that none of this is data loss level issues, but
-still sufficient enough to warrant an updated release.
-
-Bug Fixes
-*********
-
-* ``RemoteTransport.readv()`` was being inefficient about how it
-  buffered the readv data and processed it. It would keep appending to
-  the same string (causing many copies) and then pop bytes out of the
-  start of the string (causing more copies).
-  With this patch "bzr+ssh://local" can improve dramatically,
-  especially for projects with large files.
-  (John Arbash Meinel)
-
-* Revision texts were always meant to be stored as fulltexts. There
-  was a bug in a bzr.dev version that would accidentally create deltas
-  when copying from a Pack repo to a Knit repo. This has been fixed,
-  but to support those repositories, we know always request full texts
-  for Revision texts. (John Arbash Meinel, #261339)
-
-* The previous ``--1.6-rich-root`` format used an incorrect xml
-  serializer, which would accidentally support fetching from a
-  repository that supported subtrees, even though the local one would
-  not. We deprecated that format, and introduced a new one that uses
-  the correct serializer ``--1.6.1-rich-root``.
-  (John Arbash Meinel, #262333)
-
-
-bzr 1.6
-#######
-
-:Released: 2008-08-25
-
-Finally, the long awaited bzr 1.6 has been released. This release includes
-new features like Stacked Branches, improved weave merge, and an updated
-server protocol (now on v3) which will allow for better cross version
-compatibility. With this release we have deprecated Knit format
-repositories, and recommend that users upgrade them, we will continue to
-support reading and writing them for the forseeable future, but we will
-not be tuning them for performance as pack repositories have proven to be
-better at scaling. This will also be the first release to bundle
-TortoiseBzr in the standalone Windows installer.
-
-
-bzr 1.6rc5
-##########
-
-:Released: 2008-08-19
-
-Bug Fixes
-*********
-
-* Disable automatic detection of stacking based on a containing
-  directory of the target. It interacted badly with push, and needs a
-  bit more work to get the edges polished before it should happen
-  automatically. (John Arbash Meinel, #259275)
-  (This change was reverted when merged to bzr.dev)
-
-
-bzr 1.6rc4
-##########
-
-:Released: 2008-08-18
-
-Bug Fixes
-*********
-
-* Fix a regression in knit => pack fetching.  We had a logic
-  inversion, causing the fetch to insert fulltexts in random order,
-  rather than preserving deltas.  (John Arbash Meinel, #256757)
-
-
-bzr 1.6rc3
-##########
-
-:Released: 2008-08-14
-
-Changes
-*******
-
-* Disable reading ``.bzrrules`` as a per-branch rule preferences
-  file. The feature was not quite ready for a full release.
-  (Robert Collins)
-
-Improvements
-************
-
-* Update the windows installer to bundle TortoiseBzr and ``qbzr``
-  into the standalone installer. This will be the first official
-  windows release that installs Tortoise by default.
-  (Mark Hammond)
-
-Bug Fixes
-*********
-
-* Fix a regression in ``bzr+http`` support. There was a missing
-  function (``_read_line``) that needed to be carried over from
-  ``bzr+ssh`` support. (Andrew Bennetts)
-
-* ``GraphIndex`` objects will internally read an entire index if more
-  than 1/20th of their keyspace is requested in a single operation.
-  This largely mitigates a performance regression in ``bzr log FILE``
-  and completely corrects the performance regression in ``bzr log``.
-  The regression was caused by removing an accomodation which had been
-  supporting the index format in use. A newer index format is in
-  development which is substantially faster. (Robert Collins)
-
-
-bzr 1.6rc2
-##########
-
-:Released: 2008-08-13
-
-This release candidate has a few minor bug fixes, and some regression
-fixes for Windows.
-
-Bug Fixes
-*********
-
-* ``bzr upgrade`` on remote branches accessed via bzr:// and
-  bzr+ssh:// now works.  (Andrew Bennetts)
-
-* Change the ``get_format_description()`` strings for
-  ``RepositoryFormatKnitPack5`` et al to be single line messages.
-  (Aaron Bentley)
-
-* Fix for a regression on Win32 where we would try to call
-  ``os.listdir()`` on a file and not catch the exception properly.
-  (Windows raises a different exception.) This would manifest in
-  places like ``bzr rm file`` or ``bzr switch``.
-  (Mark Hammond, John Arbash Meinel)
-
-* ``Inventory.copy()`` was failing to set the revision property for
-  the root entry. (Jelmer Vernooij)
-
-* sftp transport: added missing ``FileExists`` case to
-  ``_translate_io_exception`` (Christophe Troestler, #123475)
-
-* The help for ``bzr ignored`` now suggests ``bzr ls --ignored`` for
-  scripting use. (Robert Collins, #3834)
-
-* The default ``annotate`` logic will now always assign the
-  last-modified value of a line to one of the revisions that modified
-  it, rather than a merge revision. This would happen when both sides
-  claimed to have modified the line resulting in the same text. The
-  choice is arbitrary but stable, so merges in different directions
-  will get the same results.  (John Arbash Meinel, #232188)
-
-
-bzr 1.6rc1
-##########
-
-:Released: 2008-08-06
-
-This release candidate for bzr 1.6 solidifies the new branch stacking
-feature.  Bazaar now recommends that users upgrade all knit repositories,
-because later formats are much faster.  However, we plan to continue read/write and
-upgrade support for knit repostories for the forseeable future.  Several
-other bugs and performance issues were fixed.
-
-Changes
-*******
-
-* Knit format repositories are deprecated and bzr will now emit
-  warnings whenever it encounters one.  Use ``bzr upgrade`` to upgrade
-  knit repositories to pack format.  (Andrew Bennetts)
-
-Improvements
-************
-
-* ``bzr check`` can now be told which elements at a location it should
-  check.  (Daniel Watkins)
-
-* Commit now supports ``--exclude`` (or ``-x``) to exclude some files
-  from the commit. (Robert Collins, #3117)
-
-* Fetching data between repositories that have the same model but no
-  optimised fetcher will not reserialise all the revisions, increasing
-  performance. (Robert Collins, John Arbash Meinel)
-
-* Give a more specific error when target branch is not reachable.
-  (James Westby)
-
-* Implemented a custom ``walkdirs_utf8`` implementation for win32.
-  This uses a pyrex extension to get direct access to the
-  ``FindFirstFileW`` style apis, rather than using ``listdir`` +
-  ``lstat``. Shows a very strong improvement in commands like
-  ``status`` and ``diff`` which have to iterate the working tree.
-  Anywhere from 2x-6x faster depending on the size of the tree (bigger
-  trees, bigger benefit.) (John Arbash Meinel)
-
-* New registry for log properties handles  and the method in
-  LongLogFormatter to display the custom properties returned by the
-  registered handlers. (Guillermo Gonzalez, #162469)
-
-Bug Fixes
-*********
-
-* Add more tests that stacking does not create deltas spanning
-  physical repository boundaries.
-  (Martin Pool, #252428)
-
-* Better message about incompatible repositories.
-  (Martin Pool, #206258)
-
-* ``bzr branch --stacked`` ensures the destination branch format can
-  support stacking, even if the origin does not.
-  (Martin Pool)
-
-* ``bzr export`` no longer exports ``.bzrrules``.
-  (Ian Clatworthy)
-
-* ``bzr serve --directory=/`` now correctly allows the whole
-  filesystem to be accessed on Windows, not just the root of the drive
-  that Python is running from.
-  (Adrian Wilkins, #240910)
-
-* Deleting directories by hand before running ``bzr rm`` will not
-  cause subsequent errors in ``bzr st`` and ``bzr commit``.
-  (Robert Collins, #150438)
-
-* Fix a test case that was failing if encoding wasn't UTF-8.
-  (John Arbash Meinel, #247585)
-
-* Fix "no buffer space available" error when branching with the new
-  smart server protocol to or from Windows.
-  (Andrew Bennetts, #246180)
-
-* Fixed problem in branching from smart server.
-  (#249256, Michael Hudson, Martin Pool)
-
-* Handle a file turning in to a directory in TreeTransform.
-  (James Westby, #248448)
-
-API Changes
-***********
-
-* ``MutableTree.commit`` has an extra optional keywork parameter
-  ``exclude`` that will be unconditionally supplied by the command
-  line UI - plugins that add tree formats may need an update.
-  (Robert Collins)
-
-* The API minimum version for plugin compatibility has been raised to
-  1.6 - there are significant changes throughout the code base.
-  (Robert Collins)
-
-* The generic fetch code now uses three attributes on Repository objects
-  to control fetch. The streams requested are controlled via :
-  ``_fetch_order`` and ``_fetch_uses_deltas``. Setting these
-  appropriately allows different repository implementations to recieve
-  data in their optimial form. If the ``_fetch_reconcile`` is set then
-  a reconcile operation is triggered at the end of the fetch.
-  (Robert Collins)
-
-* The ``put_on_disk`` and ``get_tar_item`` methods in
-  ``InventoryEntry`` were deprecated. (Ian Clatworthy)
-
-* ``Repository.is_shared`` doesn't take a read lock. It didn't
-  need one in the first place (nobody cached the value, and
-  ``RemoteRepository`` wasn't taking one either). This saves a round
-  trip when probing Pack repositories, as they read the ``pack-names``
-  file when locked. And during probe, locking the repo isn't very
-  useful. (John Arbash Meinel)
-
-Internals
-*********
-
-* ``bzrlib.branchbuilder.BranchBuilder`` is now much more capable of
-  putting together a real history without having to create a full
-  WorkingTree. It is recommended that tests that are not directly
-  testing the WorkingTree use BranchBuilder instead.  See
-  ``BranchBuilder.build_snapshot`` or
-  ``TestCaseWithMemoryTree.make_branch_builder``.  (John Arbash Meinel)
-
-* ``bzrlib.builtins.internal_tree_files`` broken into two giving a new
-  helper ``safe_relpath_files`` - used by the new ``exclude``
-  parameter to commit. (Robert Collins)
-
-* Make it easier to introduce new WorkingTree formats.
-  (Ian Clatworthy)
-
-* The code for exporting trees was refactored not to use the
-  deprecated ``InventoryEntry`` methods. (Ian Clatworthy)
-
-* RuleSearchers return () instead of [] now when there are no matches.
-  (Ian Clatworthy)
-
-
-bzr 1.6beta3
-############
-
-:Released: 2008-07-17
-
-This release adds a new 'stacked branches' feature allowing branches to
-share storage without being in the same repository or on the same machine.
-(See the user guide for more details.)  It also adds a new hook, improved
-weaves, aliases for related locations, faster bzr+ssh push, and several
-bug fixes.
-
-Features
-********
-
-* New ``pre_change_branch_tip`` hook that is called before the
-  branch tip is moved, while the branch is write-locked.  See the User
-  Reference for signature details.  (Andrew Bennetts)
-
-* Rule-based preferences can now be defined for selected files in
-  selected branches, allowing commands and plugins to provide
-  custom behaviour for files matching defined patterns.
-  See ``Rule-based preferences`` (part of ``Configuring Bazaar``)
-  in the User Guide and ``bzr help rules`` for more information.
-  (Ian Clatworthy)
-
-* Sites may suggest a branch to stack new branches on.  (Aaron Bentley)
-
-* Stacked branches are now supported. See ``bzr help branch`` and
-  ``bzr help push``.  Branches must be in the ``development1`` format
-  to stack, though the stacked-on branch can be of any format.
-  (Robert Collins)
-
-Improvements
-************
-
-* ``bzr export --format=tgz --root=NAME -`` to export a gzipped tarball
-  to stdout; also ``tar`` and ``tbz2``.
-  (Martin Pool)
-
-* ``bzr (re)merge --weave`` will now use a standard Weave algorithm,
-  rather than the annotation-based merge it was using. It does so by
-  building up a Weave of the important texts, without needing to build
-  the full ancestry. (John Arbash Meinel, #238895)
-
-* ``bzr send`` documents and better supports ``emacsclient`` (proper
-  escaping of mail headers and handling of the MUA Mew).
-  (Christophe Troestler)
-
-* Remembered locations can be specified by aliases, e.g. :parent, :public,
-  :submit.  (Aaron Bentley)
-
-* The smart protocol now has improved support for setting branches'
-  revision info directly.  This makes operations like push
-  faster.  The new request method name is
-  ``Branch.set_last_revision_ex``.  (Andrew Bennetts)
-
-Bug Fixes
-*********
-
-* Bazaar is now able to be a client to the web server of IIS 6 and 7.
-  The broken implementations of RFC822 in Python and RFC2046 in IIS
-  combined with boundary-line checking in Bazaar previously made this
-  impossible. (NB, IIS 5 does not suffer from this problem).
-  (Adrian Wilkins, #247585)
-
-* ``bzr log --long`` with a ghost in your mainline now handles that
-  ghost properly. (John Arbash Meinel, #243536)
-
-* ``check`` handles the split-up .bzr layout correctly, so no longer
-  requires a branch to be present.
-  (Daniel Watkins, #64783)
-
-* Clearer message about how to set the PYTHONPATH if bzrlib can't be
-  loaded.
-  (Martin Pool, #205230)
-
-* Errors about missing libraries are now shown without a traceback,
-  and with a suggestion to install the library.  The full traceback is
-  still in ``.bzr.log`` and can be shown with ``-Derror``.
-  (Martin Pool, #240161)
-
-* Fetch from a stacked branch copies all required data.
-  (Aaron Bentley, #248506)
-
-* Handle urls such as ftp://user@host.com@www.host.com where the user
-  name contains an @.
-  (Neil Martinsen-Burrell, #228058)
-
-* ``needs_read_lock`` and ``needs_write_lock`` now suppress an error during
-  ``unlock`` if there was an error in the original function. This helps
-  most when there is a failure with a smart server action, since often the
-  connection closes and we cannot unlock.
-  (Andrew Bennetts, John Arbash Meinel, #125784)
-
-* Obsolete hidden command ``bzr fetch`` removed.
-  (Martin Pool, #172870)
-
-* Raise the correct exception when doing ``-rbefore:0`` or ``-c0``.
-  (John Arbash Meinel, #239933)
-
-* You can now compare file revisions in Windows diff programs from
-  Cygwin Bazaar.
-  (Matt McClure, #209281)
-
-* revision_history now tolerates mainline ghosts for Branch format 6.
-  (Aaron Bentley, #235055)
-
-* Set locale from environment for third party libs.
-  (Martin von Gagern, #128496)
-
-Documentation
-*************
-
-* Added *Using stacked branches* to the User Guide.
-  (Ian Clatworthy)
-
-* Updated developer documentation.
-  (Martin Pool)
-
-Testing
-*******
-
-* ``-Dmemory`` will cause /proc/PID/status to be catted before bzr
-  exits, allowing low-key analysis of peak memory use. (Robert Collins)
-
-* ``TestCaseWithTransport.make_branch_and_tree`` tries harder to return
-  a tree with a ``branch`` attribute of the right format.  This was
-  preventing some ``RemoteBranch`` tests from actually running with
-  ``RemoteBranch`` instances.  (Andrew Bennetts)
-
-API Changes
-***********
-
-* Removed ``Repository.text_store``, ``control_store``, etc.  Instead,
-  there are new attributes ``texts, inventories, revisions,
-  signatures``, each of which is a ``VersionedFiles``.  See the
-  Repository docstring for more details.
-  (Robert Collins)
-
-* ``Branch.pull`` now accepts an ``_override_hook_target`` optional
-  parameter.  If you have a subclass of ``Branch`` that overrides
-  ``pull`` then you should add this parameter.  (Andrew Bennetts)
-
-* ``bzrlib.check.check()`` has been deprecated in favour of the more
-  aptly-named ``bzrlib.check.check_branch()``.
-  (Daniel Watkins)
-
-* ``Tree.print_file`` and ``Repository.print_file`` are deprecated.
-  These methods are bad APIs because they write directly to sys.stdout.
-  bzrlib does not use them internally, and there are no direct tests
-  for them. (Alexander Belchenko)
-
-Internals
-*********
-
-* ``cat`` command no longer uses ``Tree.print_file()`` internally.
-  (Alexander Belchenko)
-
-* New class method ``BzrDir.open_containing_tree_branch_or_repository``
-  which eases the discovery of the tree, the branch and the repository
-  containing a given location.
-  (Daniel Watkins)
-
-* New ``versionedfile.KeyMapper`` interface to abstract out the access to
-  underlying .knit/.kndx etc files in repositories with partitioned
-  storage. (Robert Collins)
-
-* Obsolete developer-use command ``weave-join`` has been removed.
-  (Robert Collins)
-
-* ``RemoteToOtherFetcher`` and ``get_data_stream_for_search`` removed,
-  to support new ``VersionedFiles`` layering.
-  (Robert Collins)
-
-
-bzr 1.6beta2
-############
-
-:Released: 2008-06-10
-
-This release contains further progress towards our 1.6 goals of shallow
-repositories, and contains a fix for some user-affecting bugs in the
-repository layer.  Building working trees during checkout and branch is
-now faster.
-
-Bug Fixes
-*********
-
-* Avoid KnitCorrupt error extracting inventories from some repositories.
-  (The data is not corrupt; an internal check is detecting a problem
-  reading from the repository.)
-  (Martin Pool, Andrew Bennetts, Robert Collins, #234748)
-
-* ``bzr status`` was breaking if you merged the same revision twice.
-  (John Arbash Meinel, #235407)
-
-* Fix infinite loop consuming 100% CPU when a connection is lost while
-  reading a response body via the smart protocol v1 or v2.
-  (Andrew Bennetts)
-
-* Inserting a bundle which changes the contents of a file with no trailing
-  end of line, causing a knit snapshot in a 'knits' repository will no longer
-  cause KnitCorrupt. (Robert Collins)
-
-* ``RemoteBranch.pull`` needs to return the ``self._real_branch``'s
-  pull result. It was instead just returning None, which breaks ``bzr
-  pull``. (John Arbash Meinel, #238149)
-
-* Sanitize branch nick before using it as an attachment filename in
-  ``bzr send``. (Lukáš Lalinský, #210218)
-
-* Squash ``inv_entry.symlink_target`` to a plain string when
-  generating DirState details. This prevents from getting a
-  ``UnicodeError`` when you have symlinks and non-ascii filenames.
-  (John Arbash Meinel, #135320)
-
-Improvements
-************
-
-* Added the 'alias' command to set/unset and display aliases. (Tim Penhey)
-
-* ``added``, ``modified``, and ``unknowns`` behaviour made consistent (all three
-  now quote paths where required). Added ``--null`` option to ``added`` and
-  ``modified`` (for null-separated unknowns, use ``ls --unknown --null``)
-  (Adrian Wilkins)
-
-* Faster branching (1.09x) and lightweight checkouts (1.06x) on large trees.
-  (Ian Clatworthy, Aaron Bentley)
-
-Documentation
-*************
-
-* Added *Bazaar Zen* section to the User Guide. (Ian Clatworthy)
-
-Testing
-*******
-
-* Fix the test HTTPServer to be isolated from chdir calls made while it is
-  running, allowing it to be used in blackbox tests. (Robert Collins)
-
-API Changes
-***********
-
-* ``WorkingTree.set_parent_(ids/trees)`` will now filter out revisions
-  which are in the ancestry of other revisions. So if you merge the same
-  tree twice, or merge an ancestor of an existing merge, it will only
-  record the newest. (If you merge a descendent, it will replace its
-  ancestor). (John Arbash Meinel, #235407)
-
-* ``RepositoryPolicy.__init__`` now requires stack_on and stack_on_pwd,
-  through the derived classes do not.  (Aaron Bentley)
-
-Internals
-*********
-
-* ``bzrlib.bzrdir.BzrDir.sprout`` now accepts ``stacked`` to control
-  creating stacked branches. (Robert Collins)
-
-* Knit record serialisation is now stricter on what it will accept, to
-  guard against potential internal bugs, or broken input. (Robert Collins)
-
-bzr 1.6beta1
-############
-
-:Released: 2008-06-02
-
-Commands that work on the revision history such as push, pull, missing,
-uncommit and log are now substantially faster.  This release adds a
-translation of some of the user documentation into Spanish.  (Contributions of
-other translations would be very welcome.)  Bazaar 1.6beta1 adds a new network
-protocol which is used by default and which allows for more efficient transfers
-and future extensions.
-
-
-Notes When Upgrading
-********************
-
-* There is a new version of the network protocol used for bzr://, bzr+ssh://
-  and bzr+http:// connections.  This will allow more efficient requests and
-  responses, and more graceful fallback when a server is too old to
-  recognise a request from a more recent client.  Bazaar 1.6 will
-  interoperate with 0.16 and later versions, but servers should be upgraded
-  when possible.  Bazaar 1.6 no longer interoperates with 0.15 and earlier via
-  these protocols.  Use alternatives like SFTP or upgrade those servers.
-  (Andrew Bennetts, #83935)
-
-Changes
-*******
-
-* Deprecation warnings will not be suppressed when running ``bzr selftest``
-  so that developers can see if their code is using deprecated functions.
-  (John Arbash Meinel)
-
-Features
-********
-
-* Adding ``-Derror`` will now display a traceback when a plugin fails to
-  load. (James Westby)
-
-Improvements
-************
-
-* ``bzr branch/push/pull -r XXX`` now have a helper function for finding
-  the revno of the new revision (``Graph.find_distance_to_null``). This
-  should make something like ``bzr branch -r -100`` in a shared, no-trees
-  repository much snappier. (John Arbash Meinel)
-
-* ``bzr log --short -r X..Y`` no longer needs to access the full revision
-  history. This makes it noticeably faster when logging the last few
-  revisions. (John Arbash Meinel)
-
-* ``bzr ls`` now accepts ``-V`` as an alias for ``--versioned``.
-  (Jerad Cramp, #165086)
-
-* ``bzr missing`` uses the new ``Graph.find_unique_ancestors`` and
-  ``Graph.find_differences`` to determine missing revisions without having
-  to search the whole ancestry. (John Arbash Meinel, #174625)
-
-* ``bzr uncommit`` now uses partial history access, rather than always
-  extracting the full revision history for a branch. This makes it
-  resolve the appropriate revisions much faster (in testing it drops
-  uncommit from 1.5s => 0.4s). It also means ``bzr log --short`` is one
-  step closer to not using full revision history.
-  (John Arbash Meinel, #172649)
-
-Bugfixes
-********
-
-* ``bzr merge --lca`` should handle when two revisions have no common
-  ancestor other than NULL_REVISION. (John Arbash Meinel, #235715)
-
-* ``bzr status`` was breaking if you merged the same revision twice.
-  (John Arbash Meinel, #235407)
-
-* ``bzr push`` with both ``--overwrite`` and ``-r NNN`` options no longer
-  fails.  (Andrew Bennetts, #234229)
-
-* Correctly track the base URL of a smart medium when using bzr+http://
-  URLs, which was causing spurious "No repository present" errors with
-  branches in shared repositories accessed over bzr+http.
-  (Andrew Bennetts, #230550)
-
-* Define ``_remote_is_at_least_1_2`` on ``SmartClientMedium`` so that all
-  implementations have the attribute.  Fixes 'PyCurlTransport' object has no
-  attribute '_remote_is_at_least_1_2' attribute errors.
-  (Andrew Bennetts, #220806)
-
-* Failure to delete an obsolete pack file should just give a warning
-  message, not a fatal error.  It may for example fail if the file is still
-  in use by another process.
-  (Martin Pool)
-
-* Fix MemoryError during large fetches over HTTP by limiting the amount of
-  data we try to read per ``recv`` call.  The problem was observed with
-  Windows and a proxy, but might affect other environments as well.
-  (Eric Holmberg, #215426)
-
-* Handle old merge directives correctly in Merger.from_mergeable.  Stricter
-  get_parent_map requirements exposed a latent bug here.  (Aaron Bentley)
-
-* Issue a warning and ignore passwords declared in authentication.conf when
-  used for an ssh scheme (sftp or bzr+ssh).
-  (Vincent Ladeuil, #203186)
-
-* Make both http implementations raise appropriate exceptions on 403
-  Forbidden when POSTing smart requests.
-  (Vincent Ladeuil, #230223)
-
-* Properly *title* header names in http requests instead of capitalizing
-  them.
-  (Vincent Ladeuil, #229076)
-
-* The "Unable to obtain lock" error message now also suggests using
-  ``bzr break-lock`` to fix it.  (Martin Albisetti, #139202)
-
-* Treat an encoding of '' as ascii; this can happen when bzr is run
-  under vim on Mac OS X.
-  (Neil Martinsen-Burrell)
-
-* ``VersionedFile.make_mpdiffs()`` was raising an exception that wasn't in
-  scope. (Daniel Fischer #235687)
-
-Documentation
-*************
-
-* Added directory structure and started translation of docs in spanish.
-  (Martin Albisetti, Lucio Albenga)
-
-* Incorporate feedback from Jelmer Vernooij and Neil Martinsen-Burrell
-  on the plugin and integration chapters of the User Guide.
-  (Ian Clatworthy)
-
-* More Bazaar developer documentation about packaging and release process,
-  and about use of Python reprs.
-  (Martin Pool, Martin Albisetti)
-
-* Updated Tortise strategy document. (Mark Hammond)
-
-Testing
-*******
-
-* ``bzrlib.tests.adapt_tests`` was broken and unused - it has been fixed.
-  (Robert Collins)
-
-* Fix the test HTTPServer to be isolated from chdir calls made while it is
-  running, allowing it to be used in blackbox tests. (Robert Collins)
-
-* New helper function for splitting test suites
-  ``split_suite_by_condition``. (Robert Collins)
-
-Internals
-*********
-
-* ``Branch.missing_revisions`` has been deprecated. Similar functionality
-  can be obtained using ``bzrlib.missing.find_unmerged``. The api was
-  fairly broken, and the function was unused, so we are getting rid of it.
-  (John Arbash Meinel)
-
-API Changes
-***********
-
-* ``Branch.abspath`` is deprecated; use the Tree or Transport
-  instead.  (Martin Pool)
-
-* ``Branch.update_revisions`` now takes an optional ``Graph``
-  object. This can be used by ``update_revisions`` when it is
-  checking ancestry, and allows callers to prefer request to go to a
-  local branch.  (John Arbash Meinel)
-
-* Branch, Repository, Tree and BzrDir should expose a Transport as an
-  attribute if they have one, rather than having it indirectly accessible
-  as ``.control_files._transport``.  This doesn't add a requirement
-  to support a Transport in cases where it was not needed before;
-  it just simplifies the way it is reached.  (Martin Pool)
-
-* ``bzr missing --mine-only`` will return status code 0 if you have no
-  new revisions, but the remote does. Similarly for ``--theirs-only``.
-  The new code only checks one side, so it doesn't know if the other
-  side has changes. This seems more accurate with the request anyway.
-  It also changes the output to print '[This|Other] branch is up to
-  date.' rather than displaying nothing.  (John Arbash Meinel)
-
-* ``LockableFiles.put_utf8``, ``put_bytes`` and ``controlfilename``
-  are now deprecated in favor of using Transport operations.
-  (Martin Pool)
-
-* Many methods on ``VersionedFile``, ``Repository`` and in
-  ``bzrlib.revision``  deprecated before bzrlib 1.5 have been removed.
-  (Robert Collins)
-
-* ``RevisionSpec.wants_revision_history`` can be set to False for a given
-  ``RevisionSpec``. This will disable the existing behavior of passing in
-  the full revision history to ``self._match_on``. Useful for specs that
-  don't actually need access to the full history. (John Arbash Meinel)
-
-* The constructors of ``SmartClientMedium`` and its subclasses now require a
-  ``base`` parameter.  ``SmartClientMedium`` implementations now also need
-  to provide a ``remote_path_from_transport`` method.  (Andrew Bennetts)
-
-* The default permissions for creating new files and directories
-  should now be obtained from ``BzrDir._get_file_mode()`` and
-  ``_get_dir_mode()``, rather than from LockableFiles.  The ``_set_file_mode``
-  and ``_set_dir_mode`` variables on LockableFiles which were advertised
-  as a way for plugins to control this are no longer consulted.
-  (Martin Pool)
-
-* ``VersionedFile.join`` is deprecated. This method required local
-  instances of both versioned file objects and was thus hostile to being
-  used for streaming from a smart server. The new get_record_stream and
-  insert_record_stream are meant to efficiently replace this method.
-  (Robert Collins)
-
-* ``WorkingTree.set_parent_(ids/trees)`` will now filter out revisions
-  which are in the ancestry of other revisions. So if you merge the same
-  tree twice, or merge an ancestor of an existing merge, it will only
-  record the newest. (If you merge a descendent, it will replace its
-  ancestor). (John Arbash Meinel, #235407)
-
-* ``WorkingTreeFormat2.stub_initialize_remote`` is now private.
-  (Martin Pool)
-
-
-bzr 1.5
-#######
-
-:Released: 2008-05-16
-
-This release of Bazaar includes several updates to the documentation, and fixes
-to prepare for making rich root support the default format. Many bugs have been
-squashed, including fixes to log, bzr+ssh inter-operation with older servers.
-
-Changes
-*******
-
-* Suppress deprecation warnings when bzrlib is a 'final' release. This way
-  users of packaged software won't be bothered with DeprecationWarnings,
-  but developers and testers will still see them. (John Arbash Meinel)
-
-Documentation
-*************
-
-* Incorporate feedback from Jelmer Vernooij and Neil Martinsen-Burrell
-  on the plugin and integration chapters of the User Guide.
-  (Ian Clatworthy)
-
-
-bzr 1.5rc1
-##########
-
-:Released: 2008-05-09
-
-Changes
-*******
-
-* Broader support of GNU Emacs mail clients. Set
-  ``mail_client=emacsclient`` in your bazaar.conf and ``send`` will pop the
-  bundle in a mail buffer according to the value of ``mail-user-agent``
-  variable. (Xavier Maillard)
-
-Improvements
-************
-
-* Diff now handles revision specs like "branch:" and "submit:" more
-  efficiently.  (Aaron Bentley, #202928)
-
-* More friendly error given when attempt to start the smart server
-  on an address already in use. (Andrea Corbellini, #200575)
-
-* Pull completes much faster when there is nothing to pull.
-  (Aaron Bentley)
-
-Bugfixes
-********
-
-* Authentication.conf can define sections without password.
-  (Vincent Ladeuil, #199440)
-
-* Avoid muttering every time a child update does not cause a progress bar
-  update. (John Arbash Meinel, #213771)
-
-* ``Branch.reconcile()`` is now implemented. This allows ``bzr reconcile``
-  to fix when a Branch has a non-canonical mainline history. ``bzr check``
-  also detects this condition. (John Arbash Meinel, #177855)
-
-* ``bzr log -r ..X bzr://`` was failing, because it was getting a request
-  for ``revision_id=None`` which was not a string.
-  (John Arbash Meinel, #211661)
-
-* ``bzr commit`` now works with Microsoft's FTP service.
-  (Andreas Deininger)
-
-* Catch definitions outside sections in authentication.conf.
-  (Vincent Ladeuil, #217650)
-
-* Conversion from non-rich-root to rich-root(-pack) updates inventory
-  sha1s, even when bundles are used.  (Aaron Bentley, #181391)
-
-* Conversion from non-rich-root to rich-root(-pack) works correctly even
-  though search keys are not topologically sorted.  (Aaron Bentley)
-
-* Conversion from non-rich-root to rich-root(-pack) works even when a
-  parent revision has a different root id.  (Aaron Bentley, #177874)
-
-* Disable strace testing until strace is fixed (see bug #103133) and emit a
-  warning when selftest ends to remind us of leaking tests.
-  (Vincent Ladeuil, #226769)
-
-* Fetching all revisions from a repository does not cause pack collisions.
-  (Robert Collins, Aaron Bentley, #212908)
-
-* Fix error about "attempt to add line-delta in non-delta knit".
-  (Andrew Bennetts, #217701)
-
-* Pushing a branch in "dirstate" format (Branch5) over bzr+ssh would break
-  if the remote server was < version 1.2. This was due to a bug in the
-  RemoteRepository.get_parent_map() fallback code.
-  (John Arbash Meinel, #214894)
-
-* Remove leftover code in ``bzr_branch`` that inappropriately creates
-  a ``branch-name`` file in the branch control directory.
-  (Martin Pool)
-
-* Set SO_REUSEADDR on server sockets of ``bzr serve`` to avoid problems
-  rebinding the socket when starting the server a second time.
-  (John Arbash Meinel, Martin Pool, #164288)
-
-* Severe performance degradation in fetching from knit repositories to
-  knits and packs due to parsing the entire revisions.kndx on every graph
-  walk iteration fixed by using the Repository.get_graph API.  There was
-  another regression in knit => knit fetching which re-read the index for
-  every revision each side had in common.
-  (Robert Collins, John Arbash Meinel)
-
-* When logging the changes to a particular file, there was a bug if there
-  were ghosts in the revision ancestry. (John Arbash Meinel, #209948)
-
-* xs4all's ftp server returns a temporary error when trying to list an
-  empty directory, rather than returning an empty list. Adding a
-  workaround so that we don't get spurious failures.
-  (John Arbash Meinel, #215522)
-
-Documentation
-*************
-
-* Expanded the User Guide to include new chapters on popular plugins and
-  integrating Bazaar into your environment. The *Best practices* chapter
-  was renamed to *Miscellaneous topics* as suggested by community
-  feedback as well. (Ian Clatworthy)
-
-* Document outlining strategies for TortoiseBzr. (Mark Hammond)
-
-* Improved the documentation on hooks. (Ian Clatworthy)
-
-* Update authentication docs regarding ssh agents.
-  (Vincent Ladeuil, #183705)
-
-Testing
-*******
-
-* Add ``thread_name_suffix`` parameter to SmartTCPServer_for_testing, to
-  make it easy to identify which test spawned a thread with an unhandled
-  exception. (Andrew Bennetts)
-
-* New ``--debugflag``/``-E`` option to ``bzr selftest`` for setting
-  options for debugging tests, these are complementary to the -D
-  options.  The ``-Dselftest_debug`` global option has been replaced by the
-  ``-E=allow_debug`` option for selftest. (Andrew Bennetts)
-
-* Parameterised test ids are preserved correctly to aid diagnosis of test
-  failures. (Robert Collins, Andrew Bennetts)
-
-* selftest now accepts --starting-with <id> to load only the tests whose id
-  starts with the one specified. This greatly speeds up running the test
-  suite on a limited set of tests and can be used to run the tests for a
-  single module, a single class or even a single test.  (Vincent Ladeuil)
-
-* The test suite modules have been modified to define load_tests() instead
-  of test_suite(). That speeds up selective loading (via --load-list)
-  significantly and provides many examples on how to migrate (grep for
-  load_tests).  (Vincent Ladeuil)
-
-Internals
-*********
-
-* ``Hooks.install_hook`` is now deprecated in favour of
-  ``Hooks.install_named_hook`` which adds a required ``name`` parameter, to
-  avoid having to call ``Hooks.name_hook``. (Daniel Watkins)
-
-* Implement xml8 serializer.  (Aaron Bentley)
-
-* New form ``@deprecated_method(deprecated_in(1, 5, 0))`` for making
-  deprecation wrappers.  (Martin Pool)
-
-* ``Repository.revision_parents`` is now deprecated in favour of
-  ``Repository.get_parent_map([revid])[revid]``. (Jelmer Vernooij)
-
-* The Python ``assert`` statement is no longer used in Bazaar source, and
-  a test checks this.  (Martin Pool)
-
-API Changes
-***********
-
-* ``bzrlib.status.show_pending_merges`` requires the repository to be
-  locked by the caller. Callers should have been doing it anyway, but it
-  will now raise an exception if they do not. (John Arbash Meinel)
-
-* Repository.get_data_stream, Repository.get_data_stream_for_search(),
-  Repository.get_deltas_for_revsions(), Repository.revision_trees(),
-  Repository.item_keys_introduced_by() no longer take read locks.
-  (Aaron Bentley)
-
-* ``LockableFiles.get_utf8`` and ``.get`` are deprecated, as a start
-  towards removing LockableFiles and ``.control_files`` entirely.
-  (Martin Pool)
-
-* Methods deprecated prior to 1.1 have been removed.
-  (Martin Pool)
-
-
-bzr 1.4 
-#######
-
-:Released: 2008-04-28
-
-This release of Bazaar includes handy improvements to the speed of log and
-status, new options for several commands, improved documentation, and better
-hooks, including initial code for server-side hooks.  A number of bugs have
-been fixed, particularly in interoperability between different formats or
-different releases of Bazaar over there network.  There's been substantial
-internal work in both the repository and network code to enable new features
-and faster performance.
-
-Bug Fixes
-*********
-
-* Pushing a branch in "dirstate" format (Branch5) over bzr+ssh would break
-  if the remote server was < version 1.2.  This was due to a bug in the
-  RemoteRepository.get_parent_map() fallback code.
-  (John Arbash Meinel, Andrew Bennetts, #214894)
-
-
-bzr 1.4rc2
-##########
-
-:Released: 2008-04-21
-
-Bug Fixes
-*********
-
-* ``bzr log -r ..X bzr://`` was failing, because it was getting a request
-  for ``revision_id=None`` which was not a string.
-  (John Arbash Meinel, #211661)
-
-* Fixed a bug in handling ghost revisions when logging changes in a
-  particular file.  (John Arbash Meinel, #209948)
-
-* Fix error about "attempt to add line-delta in non-delta knit".
-  (Andrew Bennetts, #205156)
-
-* Fixed performance degradation in fetching from knit repositories to
-  knits and packs due to parsing the entire revisions.kndx on every graph
-  walk iteration fixed by using the Repository.get_graph API.  There was
-  another regression in knit => knit fetching which re-read the index for
-  every revision each side had in common.
-  (Robert Collins, John Arbash Meinel)
-
-
-bzr 1.4rc1
-##########
-
-:Released: 2008-04-11
-
-Changes
-*******
-
-* bzr main script cannot be imported (Benjamin Peterson)
-
-* On GNU/Linux bzr additionally looks for plugins in arch-independent site
-  directory. (Toshio Kuratomi)
-
-* The ``set_rh`` branch hook is now deprecated. Please migrate
-  any plugins using this hook to use an alternative, e.g.
-  ``post_change_branch_tip``. (Ian Clatworthy)
-
-* When a plugin cannot be loaded as the file path is not a valid
-  python module name bzr will now strip a ``bzr_`` prefix from the
-  front of the suggested name, as many plugins (e.g. bzr-svn)
-  want to be installed without this prefix. It is a common mistake
-  to have a folder named "bzr-svn" for that plugin, especially
-  as this is what bzr branch lp:bzr-svn will give you. (James Westby,
-  Andrew Cowie)
-
-* UniqueIntegerBugTracker now appends bug-ids instead of joining
-  them to the base URL. Plugins that register bug trackers may
-  need a trailing / added to the base URL if one is not already there.
-  (James Wesby, Andrew Cowie)
-
-Features
-********
-
-* Added start_commit hook for mutable trees. (Jelmer Vernooij, #186422)
-
-* ``status`` now accepts ``--no-pending`` to show the status without
-  listing pending merges, which speeds up the command a lot on large
-  histories.  (James Westby, #202830)
-
-* New ``post_change_branch_tip`` hook that is called after the
-  branch tip is moved but while the branch is still write-locked.
-  See the User Reference for signature details.
-  (Ian Clatworthy, James Henstridge)
-
-* Reconfigure can convert a branch to be standalone or to use a shared
-  repository.  (Aaron Bentley)
-
-Improvements
-************
-
-* The smart protocol now has support for setting branches' revision info
-  directly.  This should make operations like push slightly faster, and is a
-  step towards server-side hooks.  The new request method name is
-  ``Branch.set_last_revision_info``.  (Andrew Bennetts)
-
-* ``bzr commit --fixes`` now recognises "gnome" as a tag by default.
-  (James Westby, Andrew Cowie)
-
-* ``bzr switch`` will attempt to find branches to switch to relative to the
-  current branch. E.g. ``bzr switch branchname`` will look for
-  ``current_branch/../branchname``. (Robert Collins, Jelmer Vernooij,
-  Wouter van Heyst)
-
-* Diff is now more specific about execute-bit changes it describes
-  (Chad Miller)
-
-* Fetching data over HTTP is a bit faster when urllib is used.  This is done
-  by forcing it to recv 64k at a time when reading lines in HTTP headers,
-  rather than just 1 byte at a time.  (Andrew Bennetts)
-
-* Log --short and --line are much faster when -r is not specified.
-  (Aaron Bentley)
-
-* Merge is faster.  We no longer check a file's existence unnecessarily
-  when merging the execute bit.  (Aaron Bentley)
-
-* ``bzr status`` on an explicit list of files no longer shows pending
-  merges, making it much faster on large trees. (John Arbash Meinel)
-
-* The launchpad directory service now warns the user if they have not set
-  their launchpad login and are trying to resolve a URL using it, just
-  in case they want to do a write operation with it.  (James Westby)
-
-* The smart protocol client is slightly faster, because it now only queries
-  the server for the protocol version once per connection.  Also, the HTTP
-  transport will now automatically probe for and use a smart server if
-  one is present.  You can use the new ``nosmart+`` transport decorator
-  to get the old behaviour.  (Andrew Bennetts)
-
-* The ``version`` command takes a ``--short`` option to print just the
-  version number, for easier use in scripts.  (Martin Pool)
-
-* Various operations with revision specs and commands that calculate
-  revnos and revision ids are faster.  (John A. Meinel, Aaron Bentley)
-
-Bugfixes
-********
-
-* Add ``root_client_path`` parameter to SmartWSGIApp and
-  SmartServerRequest.  This makes it possible to publish filesystem
-  locations that don't exactly match URL paths. SmartServerRequest
-  subclasses should use the new ``translate_client_path`` and
-  ``transport_from_client_path`` methods when dealing with paths received
-  from a client to take this into account.  (Andrew Bennetts, #124089)
-
-* ``bzr mv a b`` can be now used also to rename previously renamed
-  directories, not only files. (Lukáš Lalinský, #107967)
-
-* ``bzr uncommit --local`` can now remove revisions from the local
-  branch to be symmetric with ``bzr commit --local``.
-  (John Arbash Meinel, #93412)
-
-* Don't ask for a password if there is no real terminal.
-  (Alexander Belchenko, #69851)
-
-* Fix a bug causing a ValueError crash in ``parse_line_delta_iter`` when
-  fetching revisions from a knit to pack repository or vice versa using
-  bzr:// (including over http or ssh).
-  (#208418, Andrew Bennetts, Martin Pool, Robert Collins)
-
-* Fixed ``_get_line`` in ``bzrlib.smart.medium``, which was buggy.  Also
-  fixed ``_get_bytes`` in the same module to use the push back buffer.
-  These bugs had no known impact in normal use, but were problematic for
-  developers working on the code, and were likely to cause real bugs sooner
-  or later.  (Andrew Bennetts)
-
-* Implement handling of basename parameter for DefaultMail.  (James Westby)
-
-* Incompatibility with Paramiko versions newer than 1.7.2 was fixed.
-  (Andrew Bennetts, #213425)
-
-* Launchpad locations (lp: URLs) can be pulled.  (Aaron Bentley, #181945)
-
-* Merges that add files to deleted root directories complete.  They
-  do create conflicts.  (Aaron Bentley, #210092)
-
-* vsftp's return ``550 RNFR command failed.`` supported.
-  (Marcus Trautwig, #129786)
-
-Documentation
-*************
-
-* Improved documentation on send/merge relationship. (Peter Schuller)
-
-* Minor fixes to the User Guide. (Matthew Fuller)
-
-* Reduced the evangelism in the User Guide. (Ian Clatworthy)
-
-* Added Integrating with Bazaar document for developers (Martin Albisetti)
-
-API Breaks
-**********
-
-* Attempting to pull data from a ghost aware repository (e.g. knits) into a
-  non-ghost aware repository such as weaves will now fail if there are
-  ghosts.  (Robert Collins)
-
-* ``KnitVersionedFile`` no longer accepts an ``access_mode`` parameter, and
-  now requires the ``index`` and ``access_method`` parameters to be
-  supplied. A compatible shim has been kept in the new function
-  ``knit.make_file_knit``. (Robert Collins)
-
-* Log formatters must now provide log_revision instead of show and
-  show_merge_revno methods. The latter had been deprecated since the 0.17
-  release. (James Westby)
-
-* ``LoopbackSFTP`` is now called ``SocketAsChannelAdapter``.
-  (Andrew Bennetts)
-
-* ``osutils.backup_file`` is removed. (Alexander Belchenko)
-
-* ``Repository.get_revision_graph`` is deprecated, with no replacement
-  method. The method was size(history) and not desirable. (Robert Collins)
-
-* ``revision.revision_graph`` is deprecated, with no replacement function.
-  The function was size(history) and not desirable. (Robert Collins)
-
-* ``Transport.get_shared_medium`` is deprecated.  Use
-  ``Transport.get_smart_medium`` instead.  (Andrew Bennetts)
-
-* ``VersionedFile`` factories now accept a get_scope parameter rather
-  than using a call to ``transaction_finished``, allowing the removal of
-  the fixed list of versioned files per repository. (Robert Collins)
-
-* ``VersionedFile.annotate_iter`` is deprecated. While in principle this
-  allowed lower memory use, all users of annotations wanted full file
-  annotations, and there is no storage format suitable for incremental
-  line-by-line annotation. (Robert Collins)
-
-* ``VersionedFile.clone_text`` is deprecated. This performance optimisation
-  is no longer used - reading the content of a file that is undergoing a
-  file level merge to identical state on two branches is rare enough, and
-  not expensive enough to special case. (Robert Collins)
-
-* ``VersionedFile.clear_cache`` and ``enable_cache`` are deprecated.
-  These methods added significant complexity to the ``VersionedFile``
-  implementation, but were only used for optimising fetches from knits -
-  which can be done from outside the knit layer, or via a caching
-  decorator. As knits are not the default format, the complexity is no
-  longer worth paying. (Robert Collins)
-
-* ``VersionedFile.create_empty`` is removed. This method presupposed a
-  sensible mapping to a transport for individual files, but pack backed
-  versioned files have no such mapping. (Robert Collins)
-
-* ``VersionedFile.get_graph`` is deprecated, with no replacement method.
-  The method was size(history) and not desirable. (Robert Collins)
-
-* ``VersionedFile.get_graph_with_ghosts`` is deprecated, with no
-  replacement method.  The method was size(history) and not desirable.
-  (Robert Collins)
-
-* ``VersionedFile.get_parents`` is deprecated, please use
-  ``VersionedFile.get_parent_map``. (Robert Collins)
-
-* ``VersionedFile.get_sha1`` is deprecated, please use
-  ``VersionedFile.get_sha1s``. (Robert Collins)
-
-* ``VersionedFile.has_ghost`` is now deprecated, as it is both expensive
-  and unused outside of a single test. (Robert Collins)
-
-* ``VersionedFile.iter_parents`` is now deprecated in favour of
-  ``get_parent_map`` which can be used to instantiate a Graph on a
-  VersionedFile. (Robert Collins)
-
-* ``VersionedFileStore`` no longer uses the transaction parameter given
-  to most methods; amongst other things this means that the
-  get_weave_or_empty method no longer guarantees errors on a missing weave
-  in a readonly transaction, and no longer caches versioned file instances
-  which reduces memory pressure (but requires more careful management by
-  callers to preserve performance). (Robert Collins)
-
-Testing
-*******
-
-* New -Dselftest_debug flag disables clearing of the debug flags during
-  tests.  This is useful if you want to use e.g. -Dhpss to help debug a
-  failing test.  Be aware that using this feature is likely to cause
-  spurious test failures if used with the full suite. (Andrew Bennetts)
-
-* selftest --load-list now uses a new more agressive test loader that will
-  avoid loading unneeded modules and building their tests. Plugins can use
-  this new loader by defining a load_tests function instead of a test_suite
-  function. (a forthcoming patch will provide many examples on how to
-  implement this).
-  (Vincent Ladeuil)
-
-* selftest --load-list now does some sanity checks regarding duplicate test
-  IDs and tests present in the list but not found in the actual test suite.
-  (Vincent Ladeuil)
-
-* Slightly more concise format for the selftest progress bar, so there's
-  more space to show the test name.  (Martin Pool) ::
-
-    [2500/10884, 1fail, 3miss in 1m29s] test_revisionnamespaces.TestRev
-
-* The test suite takes much less memory to run, and is a bit faster.  This
-  is done by clearing most attributes of TestCases after running them, if
-  they succeeded.  (Andrew Bennetts)
-
-Internals
-*********
-
-* Added ``_build_client_protocol`` to ``_SmartClient``.  (Andrew Bennetts)
-
-* Added basic infrastructure for automatic plugin suggestion.
-  (Martin Albisetti)
-
-* If a ``LockableFiles`` object is not explicitly unlocked (for example
-  because of a missing ``try/finally`` block, it will give a warning but
-  not automatically unlock itself.  (Previously they did.)  This
-  sometimes caused knock-on errors if for example the network connection
-  had already failed, and should not be relied upon by code.
-  (Martin Pool, #109520)
-
-* ``make dist`` target to build a release tarball, and also
-  ``check-dist-tarball`` and ``dist-upload-escudero``.  (Martin Pool)
-
-* The ``read_response_tuple`` method of ``SmartClientRequestProtocol*``
-  classes will now raise ``UnknownSmartMethod`` when appropriate, so that
-  callers don't need to try distinguish unknown request errors from other
-  errors.  (Andrew Bennetts)
-
-* ``set_make_working_trees`` is now implemented provided on all repository
-  implementations (Aaron Bentley)
-
-* ``VersionedFile`` now has a new method ``get_parent_map`` which, like
-  ``Graph.get_parent_map`` returns a dict of key:parents. (Robert Collins)
-
-
-bzr 1.3.1
-#########
-
-:Released: 2008-04-09
-
-No changes from 1.3.1rc1.
-
-
-bzr 1.3.1rc1
-############
-
-:Released: 2008-04-04
-
-Bug Fixes
-*********
-
-* Fix a bug causing a ValueError crash in ``parse_line_delta_iter`` when
-  fetching revisions from a knit to pack repository or vice versa using
-  bzr:// (including over http or ssh).
-  (#208418, Andrew Bennetts, Martin Pool, Robert Collins)
-
-
-bzr 1.3
-#######
-
-:Released: 2008-03-20
-
-Bazaar has become part of the GNU project <http://www.gnu.org>
-
-Many operations that act on history, including ``log`` and ``annotate`` are now
-substantially faster.  Several bugs have been fixed and several new options and
-features have been added.
-
-Testing
-*******
-
-* Avoid spurious failure of ``TestVersion.test_version`` matching
-  directory names.
-  (#202778, Martin Pool)
-
-
-bzr 1.3rc1
-##########
-
-:Released: 2008-03-16
-
-Notes When Upgrading
-********************
-
-* The backup directory created by ``upgrade`` is now called
-  ``backup.bzr``, not ``.bzr.backup``. (Martin Albisetti)
-
-Changes
-*******
-
-* A new repository format 'development' has been added. This format will
-  represent the latest 'in-progress' format that the bzr developers are
-  interested in getting early-adopter testing and feedback on.
-  ``doc/developers/development-repo.txt`` has detailed information.
-  (Robert Collins)
-
-* BZR_LOG environment variable controls location of .bzr.log trace file.
-  User can suppress writing messages to .bzr.log by using '/dev/null'
-  filename (on Unix) or 'NUL' (on Windows). If BZR_LOG variable
-  is not defined but BZR_HOME is defined then default location
-  for .bzr.log trace file is ``$BZR_HOME/.bzr.log``.
-  (Alexander Belchenko, #106117)
-
-* ``launchpad`` builtin plugin now shipped as separate part in standalone
-  bzr.exe, installed to ``C:\Program Files\Bazaar\plugins`` directory,
-  and standalone installer allows user to skip installation of this plugin.
-  (Alexander Belchenko)
-
-* Restore auto-detection of plink.exe on Windows. (Dmitry Vasiliev)
-
-* Version number is now shown as "1.2" or "1.2pr2", without zeroed or
-  missing final fields.  (Martin Pool)
-
-Features
-********
-
-* ``branch`` and ``checkout`` can hard-link working tree files, which is
-  faster and saves space.  (Aaron Bentley)
-
-* ``bzr send`` will now also look at the ``child_submit_to`` setting in
-  the submit branch to determine the email address to send to.
-  (Jelmer Vernooij)
-
-Improvements
-************
-
-* BzrBranch._lefthand_history is faster on pack repos.  (Aaron Bentley)
-
-* Branch6.generate_revision_history is faster.  (Aaron Bentley)
-
-* Directory services can now be registered, allowing special URLs to be
-  dereferenced into real URLs.  This is a generalization and cleanup of
-  the lp: transport lookup.  (Aaron Bentley)
-
-* Merge directives that are automatically attached to emails have nicer
-  filenames, based on branch-nick + revno. (Aaron Bentley)
-
-* ``push`` has a ``--revision`` option, to specify what revision to push up
-  to.  (Daniel Watkins)
-
-* Significantly reducing execution time and network traffic for trivial
-  case of running ``bzr missing`` command for two identical branches.
-  (Alexander Belchenko)
-
-* Speed up operations that look at the revision graph (such as 'bzr log').
-  ``KnitPackRepositor.get_revision_graph`` uses ``Graph.iter_ancestry`` to
-  extract the revision history. This allows filtering ghosts while
-  stepping instead of needing to peek ahead. (John Arbash Meinel)
-
-* The ``hooks`` command lists installed hooks, to assist in debugging.
-  (Daniel Watkins)
-
-* Updates to how ``annotate`` work. Should see a measurable improvement in
-  performance and memory consumption for file with a lot of merges.
-  Also, correctly handle when a line is introduced by both parents (it
-  should be attributed to the first merge which notices this, and not
-  to all subsequent merges.) (John Arbash Meinel)
-
-Bugfixes
-********
-
-* Autopacking no longer holds the full set of inventory lines in
-  memory while copying. For large repositories, this can amount to
-  hundreds of MB of ram consumption.
-  (Ian Clatworthy, John Arbash Meinel)
-
-* Cherrypicking when using ``--format=merge3`` now explictly excludes
-  BASE lines. (John Arbash Meinel, #151731)
-
-* Disable plink's interactive prompt for password.
-  (#107593, Dmitry Vasiliev)
-
-* Encode command line arguments from unicode to user_encoding before
-  invoking external mail client in `bzr send` command.
-  (#139318, Alexander Belchenko)
-
-* Fixed problem connecting to ``bzr+https://`` servers.
-  (#198793, John Ferlito)
-
-* Improved error reporting in the Launchpad plugin. (Daniel Watkins,
-  #196618)
-
-* Include quick-start-summary.svg file to python-based installer(s)
-  for Windows. (#192924, Alexander Belchenko)
-
-* lca merge now respects specified files. (Aaron Bentley)
-
-* Make version-info --custom imply --all. (#195560, James Westby)
-
-* ``merge --preview`` now works for merges that add or modify
-  symlinks (James Henstridge)
-
-* Redirecting the output from ``bzr merge`` (when the remembered
-  location is used) now works. (John Arbash Meinel)
-
-* setup.py script explicitly checks for Python version.
-  (Jari Aalto, Alexander Belchenko, #200569)
-
-* UnknownFormatErrors no longer refer to branches regardless of kind of
-  unknown format. (Daniel Watkins, #173980)
-
-* Upgrade bundled ConfigObj to version 4.5.2, which properly quotes #
-  signs, among other small improvements. (Matt Nordhoff, #86838)
-
-* Use correct indices when emitting LCA conflicts.  This fixes IndexError
-  errors.  (Aaron Bentley, #196780)
-
-Documentation
-*************
-
-* Explained how to use ``version-info --custom`` in the User Guide.
-  (Neil Martinsen-Burrell)
-
-API Breaks
-**********
-
-* Support for loading plugins from zip files and
-  ``bzrlib.plugin.load_from_zip()`` function are deprecated.
-  (Alexander Belchenko)
-
-Testing
-*******
-
-* Added missing blackbox tests for ``modified`` (Adrian Wilkins)
-
-* The branch interface tests were invalid for branches using rich-root
-  repositories because the empty string is not a valid file-id.
-  (Robert Collins)
-
-Internals
-*********
-
-* ``Graph.iter_ancestry`` returns the ancestry of revision ids. Similar to
-  ``Repository.get_revision_graph()`` except it includes ghosts and you can
-  stop part-way through. (John Arbash Meinel)
-
-* New module ``tools/package_mf.py`` provide custom module finder for
-  python packages (improves standard python library's modulefinder.py)
-  used by ``setup.py`` script while building standalone bzr.exe.
-  (Alexander Belchenko)
-
-* New remote method ``RemoteBzrDir.find_repositoryV2`` adding support for
-  detecting external lookup support on remote repositories. This method is
-  now attempted first when lookup up repositories, leading to an extra
-  round trip on older bzr smart servers. (Robert Collins)
-
-* Repository formats have a new supported-feature attribute
-  ``supports_external_lookups`` used to indicate repositories which support
-  falling back to other repositories when they have partial data.
-  (Robert Collins)
-
-* ``Repository.get_revision_graph_with_ghosts`` and
-  ``bzrlib.revision.(common_ancestor,MultipleRevisionSources,common_graph)``
-  have been deprecated.  (John Arbash Meinel)
-
-* ``Tree.iter_changes`` is now a public API, replacing the work-in-progress
-  ``Tree._iter_changes``. The api is now considered stable and ready for
-  external users.  (Aaron Bentley)
-
-* The bzrdir format registry now accepts an ``alias`` keyword to
-  register_metadir, used to indicate that a format name is an alias for
-  some other format and thus should not be reported when describing the
-  format. (Robert Collins)
-
-
-bzr 1.2
-#######
-
-:Released: 2008-02-15
-
-Bug Fixes
-*********
-
-* Fix failing test in Launchpad plugin. (Martin Pool)
-
-
-bzr 1.2rc1
-##########
-
-:Released: 2008-02-13
-
-Notes When Upgrading
-********************
-
-* Fetching via the smart protocol may need to reconnect once during a fetch
-  if the remote server is running Bazaar 1.1 or earlier, because the client
-  attempts to use more efficient requests that confuse older servers.  You
-  may be required to re-enter a password or passphrase when this happens.
-  This won't happen if the server is upgraded to Bazaar 1.2.
-  (Andrew Bennetts)
-
-Changes
-*******
-
-* Fetching via bzr+ssh will no longer fill ghosts by default (this is
-  consistent with pack-0.92 fetching over SFTP). (Robert Collins)
-
-* Formatting of ``bzr plugins`` output is changed to be more human-
-  friendly. Full path of plugins locations will be shown only with
-  ``--verbose`` command-line option. (Alexander Belchenko)
-
-* ``merge`` now prefers to use the submit branch, but will fall back to
-  parent branch.  For many users, this has no effect.  But some users who
-  pull and merge on the same branch will notice a change.  This change
-  makes it easier to work on a branch on two different machines, pulling
-  between the machines, while merging from the upstream.
-  ``merge --remember`` can now be used to set the submit_branch.
-  (Aaron Bentley)
-
-Features
-********
-
-* ``merge --preview`` produces a diff of the changes merge would make,
-  but does not actually perform the merge.  (Aaron Bentley)
-
-* New smart method ``Repository.get_parent_map`` for getting revision
-  parent data. This returns additional parent information topologically
-  adjacent to the requested data to reduce round trip latency impacts.
-  (Robert Collins)
-
-* New smart method, ``Repository.stream_revisions_chunked``, for fetching
-  revision data that streams revision data via a chunked encoding.  This
-  avoids buffering large amounts of revision data on the server and on the
-  client, and sends less data to the server to request the revisions.
-  (Andrew Bennetts, Robert Collins, #178353)
-
-* The launchpad plugin now handles lp urls of the form
-  ``lp://staging/``, ``lp://demo/``, ``lp://dev/`` to use the appropriate
-  launchpad instance to do the resolution of the branch identities.
-  This is primarily of use to Launchpad developers, but can also
-  be used by other users who want to try out Launchpad as
-  a branch location without messing up their public Launchpad
-  account.  Branches that are pushed to the staging environment
-  have an expected lifetime of one day. (Tim Penhey)
-
-Improvements
-************
-
-* Creating a new branch no longer tries to read the entire revision-history
-  unnecessarily over smart server operations. (Robert Collins)
-
-* Fetching between different repository formats with compatible models now
-  takes advantage of the smart method to stream revisions.  (Andrew Bennetts)
-
-* The ``--coverage`` option is now global, rather specific to ``bzr
-  selftest``.  (Andrew Bennetts)
-
-* The ``register-branch`` command will now use the public url of the branch
-  containing the current directory, if one has been set and no explicit
-  branch is provided.  (Robert Collins)
-
-* Tweak the ``reannotate`` code path to optimize the 2-parent case.
-  Speeds up ``bzr annotate`` with a pack repository by approx 3:2.
-  (John Arbash Meinel)
-
-Bugfixes
-********
-
-* Calculate remote path relative to the shared medium in _SmartClient.  This
-  is related to the problem in bug #124089.  (Andrew Bennetts)
-
-* Cleanly handle connection errors in smart protocol version two, the same
-  way as they are handled by version one.  (Andrew Bennetts)
-
-* Clearer error when ``version-info --custom`` is used without
-  ``--template`` (Lukáš Lalinský)
-
-* Don't raise UnavailableFeature during test setup when medusa is not
-  available or tearDown is never called leading to nasty side effects.
-  (#137823, Vincent Ladeuil)
-
-* If a plugin's test suite cannot be loaded, for example because of a syntax
-  error in the tests, then ``selftest`` fails, rather than just printing
-  a warning.  (Martin Pool, #189771)
-
-* List possible values for BZR_SSH environment variable in env-variables
-  help topic. (Alexander Belchenko, #181842)
-
-* New methods ``push_log_file`` and ``pop_log_file`` to intercept messages:
-  popping the log redirection now precisely restores the previous state,
-  which makes it easier to use bzr log output from other programs.
-  TestCaseInTempDir no longer depends on a log redirection being established
-  by the test framework, which lets bzr tests cleanly run from a normal
-  unittest runner.
-  (#124153, #124849, Martin Pool, Jonathan Lange)
-
-* ``pull --quiet`` is now more quiet, in particular a message is no longer
-  printed when the remembered pull location is used. (James Westby,
-  #185907)
-
-* ``reconfigure`` can safely be interrupted while fetching.
-  (Aaron Bentley, #179316)
-
-* ``reconfigure`` preserves tags when converting to and from lightweight
-  checkouts.  (Aaron Bentley, #182040)
-
-* Stop polluting /tmp when running selftest.
-  (Vincent Ladeuil, #123363)
-
-* Switch from NFKC => NFC for normalization checks. NFC allows a few
-  more characters which should be considered valid.
-  (John Arbash Meinel, #185458)
-
-* The launchpad plugin now uses the ``edge`` xmlrpc server to avoid
-  interacting badly with a bug on the launchpad side. (Robert Collins)
-
-* Unknown hostnames when connecting to a ``bzr://`` URL no longer cause
-  tracebacks.  (Andrew Bennetts, #182849)
-
-API Breaks
-**********
-
-* Classes implementing Merge types like Merge3Merger must now accept (and
-  honour) a do_merge flag in their constructor.  (Aaron Bentley)
-
-* ``Repository.add_inventory`` and ``add_revision`` now require the caller
-  to previously take a write lock (and start a write group.)
-  (Martin Pool)
-
-Testing
-*******
-
-* selftest now accepts --load-list <file> to load a test id list. This
-  speeds up running the test suite on a limited set of tests.
-  (Vincent Ladeuil)
-
-Internals
-*********
-
-* Add a new method ``get_result`` to graph search objects. The resulting
-  ``SearchResult`` can be used to recreate the search later, which will
-  be useful in reducing network traffic. (Robert Collins)
-
-* Use convenience function to check whether two repository handles
-  are referring to the same repository in ``Repository.get_graph``.
-  (Jelmer Vernooij, #187162)
-
-* Fetching now passes the find_ghosts flag through to the
-  ``InterRepository.missing_revision_ids`` call consistently for all
-  repository types. This will enable faster missing revision discovery with
-  bzr+ssh. (Robert Collins)
-
-* Fix error handling in Repository.insert_data_stream. (Lukas Lalinsky)
-
-* ``InterRepository.missing_revision_ids`` is now deprecated in favour of
-  ``InterRepository.search_missing_revision_ids`` which returns a
-  ``bzrlib.graph.SearchResult`` suitable for making requests from the smart
-  server. (Robert Collins)
-
-* New error ``NoPublicBranch`` for commands that need a public branch to
-  operate. (Robert Collins)
-
-* New method ``iter_inventories`` on Repository for access to many
-  inventories. This is primarily used by the ``revision_trees`` method, as
-  direct access to inventories is discouraged. (Robert Collins)
-
-* New method ``next_with_ghosts`` on the Graph breadth-first-search objects
-  which will split out ghosts and present parents into two separate sets,
-  useful for code which needs to be aware of ghosts (e.g. fetching data
-  cares about ghosts during revision selection). (Robert Collins)
-
-* Record a timestamp against each mutter to the trace file, relative to the
-  first import of bzrlib.  (Andrew Bennetts)
-
-* ``Repository.get_data_stream`` is now deprecated in favour of
-  ``Repository.get_data_stream_for_search`` which allows less network
-  traffic when requesting data streams over a smart server. (Robert Collins)
-
-* ``RemoteBzrDir._get_tree_branch`` no longer triggers ``_ensure_real``,
-  removing one round trip on many network operations. (Robert Collins)
-
-* RemoteTransport's ``recommended_page_size`` method now returns 64k, like
-  SFTPTransport and HttpTransportBase.  (Andrew Bennetts)
-
-* Repository has a new method ``has_revisions`` which signals the presence
-  of many revisions by returning a set of the revisions listed which are
-  present. This can be done by index queries without reading data for parent
-  revision names etc. (Robert Collins)
-
-
-bzr 1.1
-#######
-
-:Released: 2008-01-15
-
-(no changes from 1.1rc1)
-
-bzr 1.1rc1
-##########
-
-:Released: 2008-01-05
-
-Changes
-*******
-
-* Dotted revision numbers have been revised. Instead of growing longer with
-  nested branches the branch number just increases. (eg instead of 1.1.1.1.1
-  we now report 1.2.1.) This helps scale long lived branches which have many
-  feature branches merged between them. (John Arbash Meinel)
-
-* The syntax ``bzr diff branch1 branch2`` is no longer supported.
-  Use ``bzr diff branch1 --new branch2`` instead. This change has
-  been made to remove the ambiguity where ``branch2`` is in fact a
-  specific file to diff within ``branch1``.
-
-Features
-********
-
-* New option to use custom template-based formats in  ``bzr version-info``.
-  (Lukáš Lalinský)
-
-* diff '--using' allows an external diff tool to be used for files.
-  (Aaron Bentley)
-
-* New "lca" merge-type for fast everyday merging that also supports
-  criss-cross merges.  (Aaron Bentley)
-
-Improvements
-************
-
-* ``annotate`` now doesn't require a working tree. (Lukáš Lalinský,
-  #90049)
-
-* ``branch`` and ``checkout`` can now use files from a working tree to
-  to speed up the process.  For checkout, this requires the new
-  --files-from flag.  (Aaron Bentley)
-
-* ``bzr diff`` now sorts files in alphabetical order.  (Aaron Bentley)
-
-* ``bzr diff`` now works on branches without working trees. Tree-less
-  branches can also be compared to each other and to working trees using
-  the new diff options ``--old`` and ``--new``. Diffing between branches,
-  with or without trees, now supports specific file filtering as well.
-  (Ian Clatworthy, #6700)
-
-* ``bzr pack`` now orders revision texts in topological order, with newest
-  at the start of the file, promoting linear reads for ``bzr log`` and the
-  like. This partially fixes #154129. (Robert Collins)
-
-* Merge directives now fetch prerequisites from the target branch if
-  needed.  (Aaron Bentley)
-
-* pycurl now handles digest authentication.
-  (Vincent Ladeuil)
-
-* ``reconfigure`` can now convert from repositories.  (Aaron Bentley)
-
-* ``-l`` is now a short form for ``--limit`` in ``log``.  (Matt Nordhoff)
-
-* ``merge`` now warns when merge directives cause cherrypicks.
-  (Aaron Bentley)
-
-* ``split`` now supported, to enable splitting large trees into smaller
-  pieces.  (Aaron Bentley)
-
-Bugfixes
-********
-
-* Avoid AttributeError when unlocking a pack repository when an error occurs.
-  (Martin Pool, #180208)
-
-* Better handle short reads when processing multiple range requests.
-  (Vincent Ladeuil, #179368)
-
-* build_tree acceleration uses the correct path when a file has been moved.
-  (Aaron Bentley)
-
-* ``commit`` now succeeds when a checkout and its master branch share a
-  repository.  (Aaron Bentley, #177592)
-
-* Fixed error reporting of unsupported timezone format in
-  ``log --timezone``. (Lukáš Lalinský, #178722)
-
-* Fixed Unicode encoding error in ``ignored`` when the output is
-  redirected to a pipe. (Lukáš Lalinský)
-
-* Fix traceback when sending large response bodies over the smart protocol
-  on Windows. (Andrew Bennetts, #115781)
-
-* Fix ``urlutils.relative_url`` for the case of two ``file:///`` URLs
-  pointed to different logical drives on Windows.
-  (Alexander Belchenko, #90847)
-
-* HTTP test servers are now compatible with the http protocol version 1.1.
-  (Vincent Ladeuil, #175524)
-
-* _KnitParentsProvider.get_parent_map now handles requests for ghosts
-  correctly, instead of erroring or attributing incorrect parents to ghosts.
-  (Aaron Bentley)
-
-* ``merge --weave --uncommitted`` now works.  (Aaron Bentley)
-
-* pycurl authentication handling was broken and incomplete. Fix handling of
-  user:pass embedded in the urls.
-  (Vincent Ladeuil, #177643)
-
-* Files inside non-directories are now handled like other conflict types.
-  (Aaron Bentley, #177390)
-
-* ``reconfigure`` is able to convert trees into lightweight checkouts.
-  (Aaron Bentley)
-
-* Reduce lockdir timeout to 0 when running ``bzr serve``.  (Andrew Bennetts,
-  #148087)
-
-* Test that the old ``version_info_format`` functions still work, even
-  though they are deprecated. (John Arbash Meinel, ShenMaq, #177872)
-
-* Transform failures no longer cause ImmortalLimbo errors (Aaron Bentley,
-  #137681)
-
-* ``uncommit`` works even when the commit messages of revisions to be
-  removed use characters not supported in the terminal encoding.
-  (Aaron Bentley)
-
-* When dumb http servers return whole files instead of the requested ranges,
-  read the remaining bytes by chunks to avoid overflowing network buffers.
-  (Vincent Ladeuil, #175886)
-
-Documentation
-*************
-
-* Minor tweaks made to the bug tracker integration documentation.
-  (Ian Clatworthy)
-
-* Reference material has now be moved out of the User Guide and added
-  to the User Reference. The User Reference has gained 4 sections as
-  a result: Authenication Settings, Configuration Settings, Conflicts
-  and Hooks. All help topics are now dumped into text format in the
-  doc/en/user-reference directory for those who like browsing that
-  information in their editor. (Ian Clatworthy)
-
-* *Using Bazaar with Launchpad* tutorial added. (Ian Clatworthy)
-
-Internals
-*********
-
-* find_* methods available for BzrDirs, Branches and WorkingTrees.
-  (Aaron Bentley)
-
-* Help topics can now be loaded from files.
-  (Ian Clatworthy, Alexander Belchenko)
-
-* get_parent_map now always provides tuples as its output.  (Aaron Bentley)
-
-* Parent Providers should now implement ``get_parent_map`` returning a
-  dictionary instead of ``get_parents`` returning a list.
-  ``Graph.get_parents`` is now deprecated. (John Arbash Meinel,
-  Robert Collins)
-
-* Patience Diff now supports arbitrary python objects, as long as they
-  support ``hash()``. (John Arbash Meinel)
-
-* Reduce selftest overhead to establish test names by memoization.
-  (Vincent Ladeuil)
-
-API Breaks
-**********
-
-Testing
-*******
-
-* Modules can now customise their tests by defining a ``load_tests``
-  attribute. ``pydoc bzrlib.tests.TestUtil.TestLoader.loadTestsFromModule``
-  for the documentation on this attribute. (Robert Collins)
-
-* New helper function ``bzrlib.tests.condition_id_re`` which helps
-  filter tests based on a regular expression search on the tests id.
-  (Robert Collins)
-
-* New helper function ``bzrlib.tests.condition_isinstance`` which helps
-  filter tests based on class. (Robert Collins)
-
-* New helper function ``bzrlib.tests.exclude_suite_by_condition`` which
-  generalises the ``exclude_suite_by_re`` function. (Robert Collins)
-
-* New helper function ``bzrlib.tests.filter_suite_by_condition`` which
-  generalises the ``filter_suite_by_re`` function. (Robert Collins)
-
-* New helper method ``bzrlib.tests.exclude_tests_by_re`` which gives a new
-  TestSuite that does not contain tests from the input that matched a
-  regular expression. (Robert Collins)
-
-* New helper method ``bzrlib.tests.randomize_suite`` which returns a
-  randomized copy of the input suite. (Robert Collins)
-
-* New helper method ``bzrlib.tests.split_suite_by_re`` which splits a test
-  suite into two according to a regular expression. (Robert Collins)
-
-* Parametrize all http tests for the transport implementations, the http
-  protocol versions (1.0 and 1.1) and the authentication schemes.
-  (Vincent Ladeuil)
-
-* The ``exclude_pattern`` and ``random_order`` parameters to the function
-  ``bzrlib.tests.filter_suite_by_re`` have been deprecated. (Robert Collins)
-
-* The method ``bzrlib.tests.sort_suite_by_re`` has been deprecated. It is
-  replaced by the new helper methods added in this release. (Robert Collins)
-
-
-bzr 1.0
-#######
-
-:Released: 2007-12-14
-
-Documentation
-*************
-
-* More improvements and fixes to the User Guide.  (Ian Clatworthy)
-
-* Add information on cherrypicking/rebasing to the User Guide.
-  (Ian Clatworthy)
-
-* Improve bug tracker integration documentation. (Ian Clatworthy)
-
-* Minor edits to ``Bazaar in five minutes`` from David Roberts and
-  to the rebasing section of the User Guide from Aaron Bentley.
-  (Ian Clatworthy)
-
-
-bzr 1.0rc3
-##########
-
-:Released: 2007-12-11
-
-Changes
-*******
-
-* If a traceback occurs, users are now asked to report the bug
-  through Launchpad (https://bugs.launchpad.net/bzr/), rather than
-  by mail to the mailing list.
-  (Martin Pool)
-
-Bugfixes
-********
-
-* Fix Makefile rules for doc generation. (Ian Clatworthy, #175207)
-
-* Give more feedback during long http downloads by making readv deliver data
-  as it arrives for urllib, and issue more requests for pycurl. High latency
-  networks are better handled by urllib, the pycurl implementation give more
-  feedback but also incur more latency.
-  (Vincent Ladeuil, #173010)
-
-* Implement _make_parents_provider on RemoteRepository, allowing generating
-  bundles against branches on a smart server.  (Andrew Bennetts, #147836)
-
-Documentation
-*************
-
-* Improved user guide.  (Ian Clatworthy)
-
-* The single-page quick reference guide is now available as a PDF.
-  (Ian Clatworthy)
-
-Internals
-*********
-
-* readv urllib http implementation is now a real iterator above the
-  underlying socket and deliver data as soon as it arrives. 'get' still
-  wraps its output in a StringIO.
-  (Vincent Ladeuil)
-
-
-bzr 1.0rc2
-##########
-
-:Released: 2007-12-07
-
-Improvements
-************
-
-* Added a --coverage option to selftest. (Andrew Bennetts)
-
-* Annotate merge (merge-type=weave) now supports cherrypicking.
-  (Aaron Bentley)
-
-* ``bzr commit`` now doesn't print the revision number twice. (Matt
-  Nordhoff, #172612)
-
-* New configuration option ``bugtracker_<tracker_abbrevation>_url`` to
-  define locations of bug trackers that are not directly supported by
-  bzr or a plugin. The URL will be treated as a template and ``{id}``
-  placeholders will be replaced by specific bug IDs.  (Lukáš Lalinský)
-
-* Support logging single merge revisions with short and line log formatters.
-  (Kent Gibson)
-
-* User Guide enhanced with suggested readability improvements from
-  Matt Revell and corrections from John Arbash Meinel. (Ian Clatworthy)
-
-* Quick Start Guide renamed to Quick Start Card, moved down in
-  the catalog, provided in pdf and png format and updated to refer
-  to ``send`` instead of ``bundle``. (Ian Clatworthy, #165080)
-
-* ``switch`` can now be used on heavyweight checkouts as well as
-  lightweight ones. After switching a heavyweight checkout, the
-  local branch is a mirror/cache of the new bound branch and
-  uncommitted changes in the working tree are merged. As a safety
-  check, if there are local commits in a checkout which have not
-  been committed to the previously bound branch, then ``switch``
-  fails unless the ``--force`` option is given. This option is
-  now also required if the branch a lightweight checkout is pointing
-  to has been moved. (Ian Clatworthy)
-
-Internals
-*********
-
-* New -Dhttp debug option reports http connections, requests and responses.
-  (Vincent Ladeuil)
-
-* New -Dmerge debug option, which emits merge plans for merge-type=weave.
-
-Bugfixes
-********
-
-* Better error message when running ``bzr cat`` on a non-existant branch.
-  (Lukáš Lalinský, #133782)
-
-* Catch OSError 17 (file exists) in final phase of tree transform and show
-  filename to user.
-  (Alexander Belchenko, #111758)
-
-* Catch ShortReadvErrors while using pycurl. Also make readv more robust by
-  allowing multiple GET requests to be issued if too many ranges are
-  required.
-  (Vincent Ladeuil, #172701)
-
-* Check for missing basis texts when fetching from packs to packs.
-  (John Arbash Meinel, #165290)
-
-* Fall back to showing e-mail in ``log --short/--line`` if the
-  committer/author has only e-mail. (Lukáš Lalinský, #157026)
-
-API Breaks
-**********
-
-* Deprecate not passing a ``location`` argument to commit reporters'
-  ``started`` methods. (Matt Nordhoff)
-
-
-bzr 1.0rc1
-##########
-
-:Released: 2007-11-30
-
-Notes When Upgrading
-********************
-
-* The default repository format is now ``pack-0.92``.  This
-  default is used when creating new repositories with ``init`` and
-  ``init-repo``, and when branching over bzr+ssh or bzr+hpss.
-  (See https://bugs.launchpad.net/bugs/164626)
-
-  This format can be read and written by Bazaar 0.92 and later, and
-  data can be transferred to and from older formats.
-
-  To upgrade, please reconcile your repository (``bzr reconcile``), and then
-  upgrade (``bzr upgrade``).
-
-  ``pack-0.92`` offers substantially better scaling and performance than the
-  previous knits format. Some operations are slower where the code already
-  had bad scaling characteristics under knits, the pack format makes such
-  operations more visible as part of being more scalable overall. We will
-  correct such operations over the coming releases and encourage the filing
-  of bugs on any operation which you observe to be slower in a packs
-  repository. One particular case that we do not intend to fix is pulling
-  data from a pack repository into a knit repository over a high latency
-  link;  downgrading such data requires reinsertion of the file texts, and
-  this is a classic space/time tradeoff. The current implementation is
-  conservative on memory usage because we need to support converting data
-  from any tree without problems.
-  (Robert Collins, Martin Pool, #164476)
-
-Changes
-*******
-
-* Disable detection of plink.exe as possible ssh vendor. Plink vendor
-  still available if user selects it explicitly with BZR_SSH environment
-  variable. (Alexander Belchenko, workaround for bug #107593)
-
-* The pack format is now accessible as "pack-0.92", or "pack-0.92-subtree"
-  to enable the subtree functions (for example, for bzr-svn).
-  (Martin Pool)
-
-Features
-********
-
-* New ``authentication.conf`` file holding the password or other credentials
-  for remote servers. This can be used for ssh, sftp, smtp and other
-  supported transports.
-  (Vincent Ladeuil)
-
-* New rich-root and rich-root-pack formats, recording the same data about
-  tree roots that's recorded for all other directories.
-  (Aaron Bentley, #164639)
-
-* ``pack-0.92`` repositories can now be reconciled.
-  (Robert Collins, #154173)
-
-* ``switch`` command added for changing the branch a lightweight checkout
-  is associated with and updating the tree to reflect the latest content
-  accordingly. This command was previously part of the BzrTools plug-in.
-  (Ian Clatworthy, Aaron Bentley, David Allouche)
-
-* ``reconfigure`` command can now convert branches, trees, or checkouts to
-  lightweight checkouts.  (Aaron Bentley)
-
-Performance
-***********
-
-* Commit updates the state of the working tree via a delta rather than
-  supplying entirely new basis trees. For commit of a single specified file
-  this reduces the wall clock time for commit by roughly a 30%.
-  (Robert Collins, Martin Pool)
-
-* Commit with many automatically found deleted paths no longer performs
-  linear scanning for the children of those paths during inventory
-  iteration. This should fix commit performance blowing out when many such
-  paths occur during commit. (Robert Collins, #156491)
-
-* Fetch with pack repositories will no longer read the entire history graph.
-  (Robert Collins, #88319)
-
-* Revert takes out an appropriate lock when reverting to a basis tree, and
-  does not read the basis inventory twice. (Robert Collins)
-
-* Diff does not require an inventory to be generated on dirstate trees.
-  (Aaron Bentley, #149254)
-
-* New annotate merge (--merge-type=weave) implementation is fast on
-  versionedfiles withough cached annotations, e.g. pack-0.92.
-  (Aaron Bentley)
-
-Improvements
-************
-
-* ``bzr merge`` now warns when it encounters a criss-cross merge.
-  (Aaron Bentley)
-
-* ``bzr send`` now doesn't require the target e-mail address to be
-  specified on the command line if an interactive e-mail client is used.
-  (Lukáš Lalinský)
-
-* ``bzr tags`` now prints the revision number for each tag, instead of
-  the revision id, unless --show-ids is passed. In addition, tags can be
-  sorted chronologically instead of lexicographically with --sort=time.
-  (Adeodato Simó, #120231)
-
-* Windows standalone version of bzr is able to load system-wide plugins from
-  "plugins" subdirectory in installation directory. In addition standalone
-  installer write to the registry (HKLM\SOFTWARE\Bazaar) useful info
-  about paths and bzr version. (Alexander Belchenko, #129298)
-
-Documentation
-*************
-
-Bug Fixes
-*********
-
-* A progress bar has been added for knitpack -> knitpack fetching.
-  (Robert Collins, #157789, #159147)
-
-* Branching from a branch via smart server now preserves the repository
-  format. (Andrew Bennetts,  #164626)
-
-* ``commit`` is now able to invoke an external editor in a non-ascii
-  directory. (Daniel Watkins, #84043)
-
-* Catch connection errors for ftp.
-  (Vincent Ladeuil, #164567)
-
-* ``check`` no longer reports spurious unreferenced text versions.
-  (Robert Collins, John A Meinel, #162931, #165071)
-
-* Conflicts are now resolved recursively by ``revert``.
-  (Aaron Bentley, #102739)
-
-* Detect invalid transport reuse attempts by catching invalid URLs.
-  (Vincent Ladeuil, #161819)
-
-* Deleting a file without removing it shows a correct diff, not a traceback.
-  (Aaron Bentley)
-
-* Do no use timeout in HttpServer anymore.
-  (Vincent Ladeuil, #158972).
-
-* Don't catch the exceptions related to the http pipeline status before
-  retrying an http request or some programming errors may be masked.
-  (Vincent Ladeuil, #160012)
-
-* Fix ``bzr rm`` to not delete modified and ignored files.
-  (Lukáš Lalinský, #172598)
-
-* Fix exception when revisionspec contains merge revisons but log
-  formatter doesn't support merge revisions. (Kent Gibson, #148908)
-
-* Fix exception when ScopeReplacer is assigned to before any members have
-  been retrieved.  (Aaron Bentley)
-
-* Fix multiple connections during checkout --lightweight.
-  (Vincent Ladeuil, #159150)
-
-* Fix possible error in insert_data_stream when copying between
-  pack repositories over bzr+ssh or bzr+http.
-  KnitVersionedFile.get_data_stream now makes sure that requested
-  compression parents are sent before any delta hunks that depend
-  on them.
-  (Martin Pool, #164637)
-
-* Fix typo in limiting offsets coalescing for http, leading to
-  whole files being downloaded instead of parts.
-  (Vincent Ladeuil, #165061)
-
-* FTP server errors don't error in the error handling code.
-  (Robert Collins, #161240)
-
-* Give a clearer message when a pull fails because the source needs
-  to be reconciled.
-  (Martin Pool, #164443)
-
-* It is clearer when a plugin cannot be loaded because of its name, and a
-  suggestion for an acceptable name is given. (Daniel Watkins, #103023)
-
-* Leave port as None in transport objects if user doesn't
-  specify a port in urls.
-  (vincent Ladeuil, #150860)
-
-* Make sure Repository.fetch(self) is properly a no-op for all
-  Repository implementations. (John Arbash Meinel, #158333)
-
-* Mark .bzr directories as "hidden" on Windows.
-  (Alexander Belchenko, #71147)
-
-* ``merge --uncommitted`` can now operate on a single file.
-  (Aaron Bentley, Lukáš Lalinský, #136890)
-
-* Obsolete packs are now cleaned up by pack and autopack operations.
-  (Robert Collins, #153789)
-
-* Operations pulling data from a smart server where the underlying
-  repositories are not both annotated/both unannotated will now work.
-  (Robert Collins, #165304).
-
-* Reconcile now shows progress bars. (Robert Collins, #159351)
-
-* ``RemoteBranch`` was not initializing ``self._revision_id_to_revno_map``
-  properly. (John Arbash Meinel, #162486)
-
-* Removing an already-removed file reports the file does not exist. (Daniel
-  Watkins, #152811)
-
-* Rename on Windows is able to change filename case.
-  (Alexander Belchenko, #77740)
-
-* Return error instead of a traceback for ``bzr log -r0``.
-  (Kent Gibson, #133751)
-
-* Return error instead of a traceback when bzr is unable to create
-  symlink on some platforms (e.g. on Windows).
-  (Alexander Belchenko, workaround for #81689)
-
-* Revert doesn't crash when restoring a single file from a deleted
-  directory. (Aaron Bentley)
-
-* Stderr output via logging mechanism now goes through encoded wrapper
-  and no more uses utf-8, but terminal encoding instead. So all unicode
-  strings now should be readable in non-utf-8 terminal.
-  (Alexander Belchenko, #54173)
-
-* The error message when ``move --after`` should be used makes how to do so
-  clearer. (Daniel Watkins, #85237)
-
-* Unicode-safe output from ``bzr info``. The output will be encoded
-  using the terminal encoding and unrepresentable characters will be
-  replaced by '?'. (Lukáš Lalinský, #151844)
-
-* Working trees are no longer created when pushing into a local no-trees
-  repo. (Daniel Watkins, #50582)
-
-* Upgrade util/configobj to version 4.4.0.
-  (Vincent Ladeuil, #151208).
-
-* Wrap medusa ftp test server as an FTPServer feature.
-  (Vincent Ladeuil, #157752)
-
-API Breaks
-**********
-
-* ``osutils.backup_file`` is deprecated. Actually it's not used in bzrlib
-  during very long time. (Alexander Belchenko)
-
-* The return value of
-  ``VersionedFile.iter_lines_added_or_present_in_versions`` has been
-  changed. Previously it was an iterator of lines, now it is an iterator of
-  (line, version_id) tuples. This change has been made to aid reconcile and
-  fetch operations. (Robert Collins)
-
-* ``bzrlib.repository.get_versioned_file_checker`` is now private.
-  (Robert Collins)
-
-* The Repository format registry default has been removed; it was previously
-  obsoleted by the bzrdir format default, which implies a default repository
-  format.
-  (Martin Pool)
-
-Internals
-*********
-
-* Added ``ContainerSerialiser`` and ``ContainerPushParser`` to
-  ``bzrlib.pack``.  These classes provide more convenient APIs for generating
-  and parsing containers from streams rather than from files.  (Andrew
-  Bennetts)
-
-* New module ``lru_cache`` providing a cache for use by tasks that need
-  semi-random access to large amounts of data. (John A Meinel)
-
-* InventoryEntry.diff is now deprecated.  Please use diff.DiffTree instead.
-
-
-bzr 0.92
-########
-
-:Released: 2007-11-05
-
-Changes
-*******
-
-  * New uninstaller on Win32.  (Alexander Belchenko)
-
-
-bzr 0.92rc1
-###########
-
-:Released: 2007-10-29
-
-Changes
-*******
-
-* ``bzr`` now returns exit code 4 if an internal error occurred, and
-  3 if a normal error occurred.  (Martin Pool)
-
-* ``pull``, ``merge`` and ``push`` will no longer silently correct some
-  repository index errors that occured as a result of the Weave disk format.
-  Instead the ``reconcile`` command needs to be run to correct those
-  problems if they exist (and it has been able to fix most such problems
-  since bzr 0.8). Some new problems have been identified during this release
-  and you should run ``bzr check`` once on every repository to see if you
-  need to reconcile. If you cannot ``pull`` or ``merge`` from a remote
-  repository due to mismatched parent errors - a symptom of index errors -
-  you should simply take a full copy of that remote repository to a clean
-  directory outside any local repositories, then run reconcile on it, and
-  finally pull from it locally. (And naturally email the repositories owner
-  to ask them to upgrade and run reconcile).
-  (Robert Collins)
-
-Features
-********
-
-* New ``knitpack-experimental`` repository format. This is interoperable with
-  the ``dirstate-tags`` format but uses a smarter storage design that greatly
-  speeds up many operations, both local and remote. This new format can be
-  used as an option to the ``init``, ``init-repository`` and ``upgrade``
-  commands. (Robert Collins)
-
-* For users of bzr-svn (and those testing the prototype subtree support) that
-  wish to try packs, a new ``knitpack-subtree-experimental`` format has also
-  been added. This is interoperable with the ``dirstate-subtrees`` format.
-  (Robert Collins)
-
-* New ``reconfigure`` command. (Aaron Bentley)
-
-* New ``revert --forget-merges`` command, which removes the record of a pending
-  merge without affecting the working tree contents.  (Martin Pool)
-
-* New ``bzr_remote_path`` configuration variable allows finer control of
-  remote bzr locations than BZR_REMOTE_PATH environment variable.
-  (Aaron Bentley)
-
-* New ``launchpad-login`` command to tell Bazaar your Launchpad
-  user ID.  This can then be used by other functions of the
-  Launchpad plugin. (James Henstridge)
-
-Performance
-***********
-
-* Commit in quiet mode is now slightly faster as the information to
-  output is no longer calculated. (Ian Clatworthy)
-
-* Commit no longer checks for new text keys during insertion when the
-  revision id was deterministically unique. (Robert Collins)
-
-* Committing a change which is not a merge and does not change the number of
-  files in the tree is faster by utilising the data about whether files are
-  changed to determine if the tree is unchanged rather than recalculating
-  it at the end of the commit process. (Robert Collins)
-
-* Inventory serialisation no longer double-sha's the content.
-  (Robert Collins)
-
-* Knit text reconstruction now avoids making copies of the lines list for
-  interim texts when building a single text. The new ``apply_delta`` method
-  on ``KnitContent`` aids this by allowing modification of the revision id
-  such objects represent. (Robert Collins)
-
-* Pack indices are now partially parsed for specific key lookup using a
-  bisection approach. (Robert Collins)
-
-* Partial commits are now approximately 40% faster by walking over the
-  unselected current tree more efficiently. (Robert Collins)
-
-* XML inventory serialisation takes 20% less time while being stricter about
-  the contents. (Robert Collins)
-
-* Graph ``heads()`` queries have been fixed to no longer access all history
-  unnecessarily. (Robert Collins)
-
-Improvements
-************
-
-* ``bzr+https://`` smart server across https now supported.
-  (John Ferlito, Martin Pool, #128456)
-
-* Mutt is now a supported mail client; set ``mail_client=mutt`` in your
-  bazaar.conf and ``send`` will use mutt. (Keir Mierle)
-
-* New option ``-c``/``--change`` for ``merge`` command for cherrypicking
-  changes from one revision. (Alexander Belchenko, #141368)
-
-* Show encodings, locale and list of plugins in the traceback message.
-  (Martin Pool, #63894)
-
-* Experimental directory formats can now be marked with
-  ``experimental = True`` during registration. (Ian Clatworthy)
-
-Documentation
-*************
-
-* New *Bazaar in Five Minutes* guide.  (Matthew Revell)
-
-* The hooks reference documentation is now converted to html as expected.
-  (Ian Clatworthy)
-
-Bug Fixes
-*********
-
-* Connection error reporting for the smart server has been fixed to
-  display a user friendly message instead of a traceback.
-  (Ian Clatworthy, #115601)
-
-* Make sure to use ``O_BINARY`` when opening files to check their
-  sha1sum. (Alexander Belchenko, John Arbash Meinel, #153493)
-
-* Fix a problem with Win32 handling of the executable bit.
-  (John Arbash Meinel, #149113)
-
-* ``bzr+ssh://`` and ``sftp://`` URLs that do not specify ports explicitly
-  no longer assume that means port 22.  This allows people using OpenSSH to
-  override the default port in their ``~/.ssh/config`` if they wish.  This
-  fixes a bug introduced in bzr 0.91.  (Andrew Bennetts, #146715)
-
-* Commands reporting exceptions can now be profiled and still have their
-  data correctly dumped to a file. For example, a ``bzr commit`` with
-  no changes still reports the operation as pointless but doing so no
-  longer throws away the profiling data if this command is run with
-  ``--lsprof-file callgrind.out.ci`` say. (Ian Clatworthy)
-
-* Fallback to ftp when paramiko is not installed and sftp can't be used for
-  ``tests/commands`` so that the test suite is still usable without
-  paramiko.
-  (Vincent Ladeuil, #59150)
-
-* Fix commit ordering in corner case. (Aaron Bentley, #94975)
-
-* Fix long standing bug in partial commit when there are renames
-  left in tree. (Robert Collins, #140419)
-
-* Fix selftest semi-random noise during http related tests.
-  (Vincent Ladeuil, #140614)
-
-* Fix typo in ftp.py making the reconnection fail on temporary errors.
-  (Vincent Ladeuil, #154259)
-
-* Fix failing test by comparing real paths to cover the case where the TMPDIR
-  contains a symbolic link.
-  (Vincent Ladeuil, #141382).
-
-* Fix log against smart server branches that don't support tags.
-  (James Westby, #140615)
-
-* Fix pycurl http implementation by defining error codes from
-  pycurl instead of relying on an old curl definition.
-  (Vincent Ladeuil, #147530)
-
-* Fix 'unprintable error' message when displaying BzrCheckError and
-  some other exceptions on Python 2.5.
-  (Martin Pool, #144633)
-
-* Fix ``Inventory.copy()`` and add test for it. (Jelmer Vernooij)
-
-* Handles default value for ListOption in cmd_commit.
-  (Vincent Ladeuil, #140432)
-
-* HttpServer and FtpServer need to be closed properly or a listening socket
-  will remain opened.
-  (Vincent Ladeuil, #140055)
-
-* Monitor the .bzr directory created in the top level test
-  directory to detect leaking tests.
-  (Vincent Ladeuil, #147986)
-
-* The basename, not the full path, is now used when checking whether
-  the profiling dump file begins with ``callgrind.out`` or not. This
-  fixes a bug reported by Aaron Bentley on IRC. (Ian Clatworthy)
-
-* Trivial fix for invoking command ``reconfigure`` without arguments.
-  (Rob Weir, #141629)
-
-* ``WorkingTree.rename_one`` will now raise an error if normalisation of the
-  new path causes bzr to be unable to access the file. (Robert Collins)
-
-* Correctly detect a NoSuchFile when using a filezilla server. (Gary van der
-  Merwe)
-
-API Breaks
-**********
-
-* ``bzrlib.index.GraphIndex`` now requires a size parameter to the
-  constructor, for enabling bisection searches. (Robert Collins)
-
-* ``CommitBuilder.record_entry_contents`` now requires the root entry of a
-  tree be supplied to it, previously failing to do so would trigger a
-  deprecation warning. (Robert Collins)
-
-* ``KnitVersionedFile.add*`` will no longer cache added records even when
-  enable_cache() has been called - the caching feature is now exclusively for
-  reading existing data. (Robert Collins)
-
-* ``ReadOnlyLockError`` is deprecated; ``LockFailed`` is usually more
-  appropriate.  (Martin Pool)
-
-* Removed ``bzrlib.transport.TransportLogger`` - please see the new
-  ``trace+`` transport instead. (Robert Collins)
-
-* Removed previously deprecated varargs interface to ``TestCase.run_bzr`` and
-  deprecated methods ``TestCase.capture`` and ``TestCase.run_bzr_captured``.
-  (Martin Pool)
-
-* Removed previous deprecated ``basis_knit`` parameter to the
-  ``KnitVersionedFile`` constructor. (Robert Collins)
-
-* Special purpose method ``TestCase.run_bzr_decode`` is moved to the test_non_ascii
-  class that needs it.
-  (Martin Pool)
-
-* The class ``bzrlib.repofmt.knitrepo.KnitRepository3`` has been folded into
-  ``KnitRepository`` by parameters to the constructor. (Robert Collins)
-
-* The ``VersionedFile`` interface now allows content checks to be bypassed
-  by supplying check_content=False.  This saves nearly 30% of the minimum
-  cost to store a version of a file. (Robert Collins)
-
-* Tree's with bad state such as files with no length or sha will no longer
-  be silently accepted by the repository XML serialiser. To serialise
-  inventories without such data, pass working=True to write_inventory.
-  (Robert Collins)
-
-* ``VersionedFile.fix_parents`` has been removed as a harmful API.
-  ``VersionedFile.join`` will no longer accept different parents on either
-  side of a join - it will either ignore them, or error, depending on the
-  implementation. See notes when upgrading for more information.
-  (Robert Collins)
-
-Internals
-*********
-
-* ``bzrlib.transport.Transport.put_file`` now returns the number of bytes
-  put by the method call, to allow avoiding stat-after-write or
-  housekeeping in callers. (Robert Collins)
-
-* ``bzrlib.xml_serializer.Serializer`` is now responsible for checking that
-  mandatory attributes are present on serialisation and deserialisation.
-  This fixes some holes in API usage and allows better separation between
-  physical storage and object serialisation. (Robert Collins)
-
-* New class ``bzrlib.errors.InternalBzrError`` which is just a convenient
-  shorthand for deriving from BzrError and setting internal_error = True.
-  (Robert Collins)
-
-* New method ``bzrlib.mutabletree.update_to_one_parent_via_delta`` for
-  moving the state of a parent tree to a new version via a delta rather than
-  a complete replacement tree. (Robert Collins)
-
-* New method ``bzrlib.osutils.minimum_path_selection`` useful for removing
-  duplication from user input, when a user mentions both a path and an item
-  contained within that path. (Robert Collins)
-
-* New method ``bzrlib.repository.Repository.is_write_locked`` useful for
-  determining if a repository is write locked. (Robert Collins)
-
-* New method on ``bzrlib.tree.Tree`` ``path_content_summary`` provides a
-  tuple containing the key information about a path for commit processing
-  to complete. (Robert Collins)
-
-* New method on xml serialisers, write_inventory_to_lines, which matches the
-  API used by knits for adding content. (Robert Collins)
-
-* New module ``bzrlib.bisect_multi`` with generic multiple-bisection-at-once
-  logic, currently only available for byte-based lookup
-  (``bisect_multi_bytes``). (Robert Collins)
-
-* New helper ``bzrlib.tuned_gzip.bytes_to_gzip`` which takes a byte string
-  and returns a gzipped version of the same. This is used to avoid a bunch
-  of api friction during adding of knit hunks. (Robert Collins)
-
-* New parameter on ``bzrlib.transport.Transport.readv``
-  ``adjust_for_latency`` which changes readv from returning strictly the
-  requested data to inserted return larger ranges and in forward read order
-  to reduce the effect of network latency. (Robert Collins)
-
-* New parameter yield_parents on ``Inventory.iter_entries_by_dir`` which
-  causes the parents of a selected id to be returned recursively, so all the
-  paths from the root down to each element of selected_file_ids are
-  returned. (Robert Collins)
-
-* Knit joining has been enhanced to support plain to annotated conversion
-  and annotated to plain conversion. (Ian Clatworthy)
-
-* The CommitBuilder method ``record_entry_contents`` now returns summary
-  information about the effect of the commit on the repository. This tuple
-  contains an inventory delta item if the entry changed from the basis, and a
-  boolean indicating whether a new file graph node was recorded.
-  (Robert Collins)
-
-* The python path used in the Makefile can now be overridden.
-  (Andrew Bennetts, Ian Clatworthy)
-
-Testing
-*******
-
-* New transport implementation ``trace+`` which is useful for testing,
-  logging activity taken to its _activity attribute. (Robert Collins)
-
-* When running bzr commands within the test suite, internal exceptions are
-  not caught and reported in the usual way, but rather allowed to propagate
-  up and be visible to the test suite.  A new API ``run_bzr_catch_user_errors``
-  makes this behavior available to other users.
-  (Martin Pool)
-
-* New method ``TestCase.call_catch_warnings`` for testing methods that
-  raises a Python warning.  (Martin Pool)
-
-
-bzr 0.91
-########
-
-:Released: 2007-09-26
-
-Bug Fixes
-*********
-
-* Print a warning instead of aborting the ``python setup.py install``
-  process if building of a C extension is not possible.
-  (Lukáš Lalinský, Alexander Belchenko)
-
-* Fix commit ordering in corner case (Aaron Bentley, #94975)
-
-* Fix ''bzr info bzr://host/'' and other operations on ''bzr://' URLs with
-  an implicit port.  We were incorrectly raising PathNotChild due to
-  inconsistent treatment of the ''_port'' attribute on the Transport object.
-  (Andrew Bennetts, #133965)
-
-* Make RemoteRepository.sprout cope gracefully with servers that don't
-  support the ``Repository.tarball`` request.
-  (Andrew Bennetts)
-
-
-bzr 0.91rc2
-###########
-
-:Released: 2007-09-11
-
-* Replaced incorrect tarball for previous release; a debug statement was left
-  in bzrlib/remote.py.
-
-
-bzr 0.91rc1
-###########
-
-:Released: 2007-09-11
-
-Changes
-*******
-
-* The default branch and repository format has changed to
-  ``dirstate-tags``, so tag commands are active by default.
-  This format is compatible with Bazaar 0.15 and later.
-  This incidentally fixes bug #126141.
-  (Martin Pool)
-
-* ``--quiet`` or ``-q`` is no longer a global option. If present, it
-  must now appear after the command name. Scripts doing things like
-  ``bzr -q missing`` need to be rewritten as ``bzr missing -q``.
-  (Ian Clatworthy)
-
-Features
-********
-
-* New option ``--author`` in ``bzr commit`` to specify the author of the
-  change, if it's different from the committer. ``bzr log`` and
-  ``bzr annotate`` display the author instead of the committer.
-  (Lukáš Lalinský)
-
-* In addition to global options and command specific options, a set of
-  standard options are now supported. Standard options are legal for
-  all commands. The initial set of standard options are:
-
-  * ``--help`` or ``-h`` - display help message
-  * ``--verbose`` or ``-v`` - display additional information
-  * ``--quiet``  or ``-q`` - only output warnings and errors.
-
-  Unlike global options, standard options can be used in aliases and
-  may have command-specific help. (Ian Clatworthy)
-
-* Verbosity level processing has now been unified. If ``--verbose``
-  or ``-v`` is specified on the command line multiple times, the
-  verbosity level is made positive the first time then increased.
-  If ``--quiet`` or ``-q`` is specified on the command line
-  multiple times, the verbosity level is made negative the first
-  time then decreased. To get the default verbosity level of zero,
-  either specify none of the above , ``--no-verbose`` or ``--no-quiet``.
-  Note that most commands currently ignore the magnitude of the
-  verbosity level but do respect *quiet vs normal vs verbose* when
-  generating output. (Ian Clatworthy)
-
-* ``Branch.hooks`` now supports ``pre_commit`` hook. The hook's signature
-  is documented in BranchHooks constructor. (Nam T. Nguyen, #102747)
-
-* New ``Repository.stream_knit_data_for_revisions`` request added to the
-  network protocol for greatly reduced roundtrips when retrieving a set of
-  revisions. (Andrew Bennetts)
-
-Bug Fixes
-*********
-
-* ``bzr plugins`` now lists the version number for each plugin in square
-  brackets after the path. (Robert Collins, #125421)
-
-* Pushing, pulling and branching branches with subtree references was not
-  copying the subtree weave, preventing the file graph from being accessed
-  and causing errors in commits in clones. (Robert Collins)
-
-* Suppress warning "integer argument expected, got float" from Paramiko,
-  which sometimes caused false test failures.  (Martin Pool)
-
-* Fix bug in bundle 4 that could cause attempts to write data to wrong
-  versionedfile.  (Aaron Bentley)
-
-* Diffs generated using "diff -p" no longer break the patch parser.
-  (Aaron Bentley)
-
-* get_transport treats an empty possible_transports list the same as a non-
-  empty one.  (Aaron Bentley)
-
-* patch verification for merge directives is reactivated, and works with
-  CRLF and CR files.  (Aaron Bentley)
-
-* Accept ..\ as a path in revision specifiers. This fixes for example
-  "-r branch:..\other-branch" on Windows.  (Lukáš Lalinský)
-
-* ``BZR_PLUGIN_PATH`` may now contain trailing slashes.
-  (Blake Winton, #129299)
-
-* man page no longer lists hidden options (#131667, Aaron Bentley)
-
-* ``uncommit --help`` now explains the -r option adequately.  (Daniel
-  Watkins, #106726)
-
-* Error messages are now better formatted with parameters (such as
-  filenames) quoted when necessary. This avoids confusion when directory
-  names ending in a '.' at the end of messages were confused with a
-  full stop that may or not have been there. (Daniel Watkins, #129791)
-
-* Fix ``status FILE -r X..Y``. (Lukáš Lalinský)
-
-* If a particular command is an alias, ``help`` will show the alias
-  instead of claiming there is no help for said alias. (Daniel Watkins,
-  #133548)
-
-* TreeTransform-based operations, like pull, merge, revert, and branch,
-  now roll back if they encounter an error.  (Aaron Bentley, #67699)
-
-* ``bzr commit`` now exits cleanly if a character unsupported by the
-  current encoding is used in the commit message.  (Daniel Watkins,
-  #116143)
-
-* bzr send uses default values for ranges when only half of an elipsis
-  is specified ("-r..5" or "-r5..").  (#61685, Aaron Bentley)
-
-* Avoid trouble when Windows ssh calls itself 'plink' but no plink
-  binary is present.  (Martin Albisetti, #107155)
-
-* ``bzr remove`` should remove clean subtrees.  Now it will remove (without
-  needing ``--force``) subtrees that contain no files with text changes or
-  modified files.  With ``--force`` it removes the subtree regardless of
-  text changes or unknown files. Directories with renames in or out (but
-  not changed otherwise) will now be removed without needing ``--force``.
-  Unknown ignored files will be deleted without needing ``--force``.
-  (Marius Kruger, #111665)
-
-* When two plugins conflict, the source of both the losing and now the
-  winning definition is shown.  (Konstantin Mikhaylov, #5454)
-
-* When committing to a branch, the location being committed to is
-  displayed.  (Daniel Watkins, #52479)
-
-* ``bzr --version`` takes care about encoding of stdout, especially
-  when output is redirected. (Alexander Belchenko, #131100)
-
-* Prompt for an ftp password if none is provided.
-  (Vincent Ladeuil, #137044)
-
-* Reuse bound branch associated transport to avoid multiple
-  connections.
-  (Vincent Ladeuil, #128076, #131396)
-
-* Overwrite conflicting tags by ``push`` and ``pull`` if the
-  ``--overwrite`` option is specified.  (Lukáš Lalinský, #93947)
-
-* In checkouts, tags are copied into the master branch when created,
-  changed or deleted, and are copied into the checkout when it is
-  updated.  (Martin Pool, #93856, #93860)
-
-* Print a warning instead of aborting the ``python setup.py install``
-  process if building of a C extension is not possible.
-  (Lukáš Lalinský, Alexander Belchenko)
-
-Improvements
-************
-
-* Add the option "--show-diff" to the commit command in order to display
-  the diff during the commit log creation. (Goffredo Baroncelli)
-
-* ``pull`` and ``merge`` are much faster at installing bundle format 4.
-  (Aaron Bentley)
-
-* ``pull -v`` no longer includes deltas, making it much faster.
-  (Aaron Bentley)
-
-* ``send`` now sends the directive as an attachment by default.
-  (Aaron Bentley, Lukáš Lalinský, Alexander Belchenko)
-
-* Documentation updates (Martin Albisetti)
-
-* Help on debug flags is now included in ``help global-options``.
-  (Daniel Watkins, #124853)
-
-* Parameters passed on the command line are checked to ensure they are
-  supported by the encoding in use. (Daniel Watkins)
-
-* The compression used within the bzr repository has changed from zlib
-  level 9 to the zlib default level. This improves commit performance with
-  only a small increase in space used (and in some cases a reduction in
-  space). (Robert Collins)
-
-* Initial commit no longer SHAs files twice and now reuses the path
-  rather than looking it up again, making it faster.
-  (Ian Clatworthy)
-
-* New option ``-c``/``--change`` for ``diff`` and ``status`` to show
-  changes in one revision.  (Lukáš Lalinský)
-
-* If versioned files match a given ignore pattern, a warning is now
-  given. (Daniel Watkins, #48623)
-
-* ``bzr status`` now has -S as a short name for --short and -V as a
-  short name for --versioned. These have been added to assist users
-  migrating from Subversion: ``bzr status -SV`` is now like
-  ``svn status -q``.  (Daniel Watkins, #115990)
-
-* Added C implementation of  ``PatienceSequenceMatcher``, which is about
-  10x faster than the Python version. This speeds up commands that
-  need file diffing, such as ``bzr commit`` or ``bzr diff``.
-  (Lukáš Lalinský)
-
-* HACKING has been extended with a large section on core developer tasks.
-  (Ian Clatworthy)
-
-* Add ``branches`` and ``standalone-trees`` as online help topics and
-  include them as Concepts within the User Reference.
-  (Paul Moore, Ian Clatworthy)
-
-* ``check`` can detect versionedfile parent references that are
-  inconsistent with revision and inventory info, and ``reconcile`` can fix
-  them.  These faulty references were generated by 0.8-era releases,
-  so repositories which were manipulated by old bzrs should be
-  checked, and possibly reconciled ASAP.  (Aaron Bentley, Andrew Bennetts)
-
-API Breaks
-**********
-
-* ``Branch.append_revision`` is removed altogether; please use
-  ``Branch.set_last_revision_info`` instead.  (Martin Pool)
-
-* CommitBuilder now advertises itself as requiring the root entry to be
-  supplied. This only affects foreign repository implementations which reuse
-  CommitBuilder directly and have changed record_entry_contents to require
-  that the root not be supplied. This should be precisely zero plugins
-  affected. (Robert Collins)
-
-* The ``add_lines`` methods on ``VersionedFile`` implementations has changed
-  its return value to include the sha1 and length of the inserted text. This
-  allows the avoidance of double-sha1 calculations during commit.
-  (Robert Collins)
-
-* ``Transport.should_cache`` has been removed.  It was not called in the
-  previous release.  (Martin Pool)
-
-Testing
-*******
-
-* Tests may now raise TestNotApplicable to indicate they shouldn't be
-  run in a particular scenario.  (Martin Pool)
-
-* New function multiply_tests_from_modules to give a simpler interface
-  to test parameterization.  (Martin Pool, Robert Collins)
-
-* ``Transport.should_cache`` has been removed.  It was not called in the
-  previous release.  (Martin Pool)
-
-* NULL_REVISION is returned to indicate the null revision, not None.
-  (Aaron Bentley)
-
-* Use UTF-8 encoded StringIO for log tests to avoid failures on
-  non-ASCII committer names.  (Lukáš Lalinský)
-
-Internals
-*********
-
-* ``bzrlib.plugin.all_plugins`` has been deprecated in favour of
-  ``bzrlib.plugin.plugins()`` which returns PlugIn objects that provide
-  useful functionality for determining the path of a plugin, its tests, and
-  its version information. (Robert Collins)
-
-* Add the option user_encoding to the function 'show_diff_trees()'
-  in order to move the user encoding at the UI level. (Goffredo Baroncelli)
-
-* Add the function make_commit_message_template_encoded() and the function
-  edit_commit_message_encoded() which handle encoded strings.
-  This is done in order to mix the commit messages (which is a unicode
-  string), and the diff which is a raw string. (Goffredo Baroncelli)
-
-* CommitBuilder now defaults to using add_lines_with_ghosts, reducing
-  overhead on non-weave repositories which don't require all parents to be
-  present. (Robert Collins)
-
-* Deprecated method ``find_previous_heads`` on
-  ``bzrlib.inventory.InventoryEntry``. This has been superseded by the use
-  of ``parent_candidates`` and a separate heads check via the repository
-  API. (Robert Collins)
-
-* New trace function ``mutter_callsite`` will print out a subset of the
-  stack to the log, which can be useful for gathering debug details.
-  (Robert Collins)
-
-* ``bzrlib.pack.ContainerWriter`` now tracks how many records have been
-  added via a public attribute records_written. (Robert Collins)
-
-* New method ``bzrlib.transport.Transport.get_recommended_page_size``.
-  This provides a hint to users of transports as to the reasonable
-  minimum data to read. In principle this can take latency and
-  bandwidth into account on a per-connection basis, but for now it
-  just has hard coded values based on the url. (e.g. http:// has a large
-  page size, file:// has a small one.) (Robert Collins)
-
-* New method on ``bzrlib.transport.Transport`` ``open_write_stream`` allows
-  incremental addition of data to a file without requiring that all the
-  data be buffered in memory. (Robert Collins)
-
-* New methods on ``bzrlib.knit.KnitVersionedFile``:
-  ``get_data_stream(versions)``, ``insert_data_stream(stream)`` and
-  ``get_format_signature()``.  These provide some infrastructure for
-  efficiently streaming the knit data for a set of versions over the smart
-  protocol.
-
-* Knits with no annotation cache still produce correct annotations.
-  (Aaron Bentley)
-
-* Three new methods have been added to ``bzrlib.trace``:
-  ``set_verbosity_level``, ``get_verbosity_level`` and ``is_verbose``.
-  ``set_verbosity_level`` expects a numeric value: negative for quiet,
-  zero for normal, positive for verbose. The size of the number can be
-  used to determine just how quiet or verbose the application should be.
-  The existing ``be_quiet`` and ``is_quiet`` routines have been
-  integrated into this new scheme. (Ian Clatworthy)
-
-* Options can now be delcared with a ``custom_callback`` parameter. If
-  set, this routine is called after the option is processed. This feature
-  is now used by the standard options ``verbose`` and ``quiet`` so that
-  setting one implicitly resets the other. (Ian Clatworthy)
-
-* Rather than declaring a new option from scratch in order to provide
-  custom help, a centrally registered option can be decorated using the
-  new ``bzrlib.Option.custom_help`` routine. In particular, this routine
-  is useful when declaring better help for the ``verbose`` and ``quiet``
-  standard options as the base definition of these is now more complex
-  than before thanks to their use of a custom callback. (Ian Clatworthy)
-
-* Tree._iter_changes(specific_file=[]) now iterates through no files,
-  instead of iterating through all files.  None is used to iterate through
-  all files.  (Aaron Bentley)
-
-* WorkingTree.revert() now accepts None to revert all files.  The use of
-  [] to revert all files is deprecated.  (Aaron Bentley)
-
-
-bzr 0.90
-########
-
-:Released: 2007-08-28
-
-Improvements
-************
-
-* Documentation is now organized into multiple directories with a level
-  added for different languages or locales. Added the Mini Tutorial
-  and Quick Start Summary (en) documents from the Wiki, improving the
-  content and readability of the former. Formatted NEWS as Release Notes
-  complete with a Table of Conents, one heading per release. Moved the
-  Developer Guide into the main document catalog and provided a link
-  from the developer document catalog back to the main one.
-  (Ian Clatworthy, Sabin Iacob, Alexander Belchenko)
-
-
-API Changes
-***********
-
-* The static convenience method ``BzrDir.create_repository``
-  is deprecated.  Callers should instead create a ``BzrDir`` instance
-  and call ``create_repository`` on that.  (Martin Pool)
-
-
-bzr 0.90rc1
-###########
-
-:Released: 2007-08-14
-
-Bugfixes
-********
-
-* ``bzr init`` should connect to the remote location one time only.  We
-  have been connecting several times because we forget to pass around the
-  Transport object. This modifies ``BzrDir.create_branch_convenience``,
-  so that we can give it the Transport we already have.
-  (John Arbash Meinel, Vincent Ladeuil, #111702)
-
-* Get rid of sftp connection cache (get rid of the FTP one too).
-  (Vincent Ladeuil, #43731)
-
-* bzr branch {local|remote} remote don't try to create a working tree
-  anymore.
-  (Vincent Ladeuil, #112173)
-
-* All identified multiple connections for a single bzr command have been
-  fixed. See bzrlib/tests/commands directory.
-  (Vincent Ladeuil)
-
-* ``bzr rm`` now does not insist on ``--force`` to delete files that
-  have been renamed but not otherwise modified.  (Marius Kruger,
-  #111664)
-
-* ``bzr selftest --bench`` no longer emits deprecation warnings
-  (Lukáš Lalinský)
-
-* ``bzr status`` now honours FILE parameters for conflict lists
-  (Aaron Bentley, #127606)
-
-* ``bzr checkout`` now honours -r when reconstituting a working tree.
-  It also honours -r 0.  (Aaron Bentley, #127708)
-
-* ``bzr add *`` no more fails on Windows if working tree contains
-  non-ascii file names. (Kuno Meyer, #127361)
-
-* allow ``easy_install bzr`` runs without fatal errors.
-  (Alexander Belchenko, #125521)
-
-* Graph._filter_candidate_lca does not raise KeyError if a candidate
-  is eliminated just before it would normally be examined.  (Aaron Bentley)
-
-* SMTP connection failures produce a nice message, not a traceback.
-  (Aaron Bentley)
-
-Improvements
-************
-
-* Don't show "dots" progress indicators when run non-interactively, such
-  as from cron.  (Martin Pool)
-
-* ``info`` now formats locations more nicely and lists "submit" and
-  "public" branches (Aaron Bentley)
-
-* New ``pack`` command that will trigger database compression within
-  the repository (Robert Collins)
-
-* Implement ``_KnitIndex._load_data`` in a pyrex extension. The pyrex
-  version is approximately 2-3x faster at parsing a ``.kndx`` file.
-  Which yields a measurable improvement for commands which have to
-  read from the repository, such as a 1s => 0.75s improvement in
-  ``bzr diff`` when there are changes to be shown.  (John Arbash Meinel)
-
-* Merge is now faster.  Depending on the scenario, it can be more than 2x
-  faster. (Aaron Bentley)
-
-* Give a clearer warning, and allow ``python setup.py install`` to
-  succeed even if pyrex is not available.
-  (John Arbash Meinel)
-
-* ``DirState._read_dirblocks`` now has an optional Pyrex
-  implementation. This improves the speed of any command that has to
-  read the entire DirState. (``diff``, ``status``, etc, improve by
-  about 10%).
-  ``bisect_dirblocks`` has also been improved, which helps all
-  ``_get_entry`` type calls (whenever we are searching for a
-  particular entry in the in-memory DirState).
-  (John Arbash Meinel)
-
-* ``bzr pull`` and ``bzr push`` no longer do a complete walk of the
-  branch revision history for ui display unless -v is supplied.
-  (Robert Collins)
-
-* ``bzr log -rA..B`` output shifted to the left margin if the log only
-  contains merge revisions. (Kent Gibson)
-
-* The ``plugins`` command is now public with improved help.
-  (Ian Clatworthy)
-
-* New bundle and merge directive formats are faster to generate, and
-
-* Annotate merge now works when there are local changes. (Aaron Bentley)
-
-* Commit now only shows the progress in terms of directories instead of
-  entries. (Ian Clatworthy)
-
-* Fix ``KnitRepository.get_revision_graph`` to not request the graph 2
-  times. This makes ``get_revision_graph`` 2x faster. (John Arbash
-  Meinel)
-
-* Fix ``VersionedFile.get_graph()`` to avoid using
-  ``set.difference_update(other)``, which has bad scaling when
-  ``other`` is large. This improves ``VF.get_graph([version_id])`` for
-  a 12.5k graph from 2.9s down to 200ms. (John Arbash Meinel)
-
-* The ``--lsprof-file`` option now generates output for KCacheGrind if
-  the file starts with ``callgrind.out``. This matches the default file
-  filtering done by KCacheGrind's Open Dialog. (Ian Clatworthy)
-
-* Fix ``bzr update`` to avoid an unnecessary
-  ``branch.get_master_branch`` call, which avoids 1 extra connection
-  to the remote server. (Partial fix for #128076, John Arbash Meinel)
-
-* Log errors from the smart server in the trace file, to make debugging
-  test failures (and live failures!) easier.  (Andrew Bennetts)
-
-* The HTML version of the man page has been superceded by a more
-  comprehensive manual called the Bazaar User Reference. This manual
-  is completed generated from the online help topics. As part of this
-  change, limited reStructuredText is now explicitly supported in help
-  topics and command help with 'unnatural' markup being removed prior
-  to display by the online help or inclusion in the man page.
-  (Ian Clatworthy)
-
-* HTML documentation now use files extension ``*.html``
-  (Alexander Belchenko)
-
-* The cache of ignore definitions is now cleared in WorkingTree.unlock()
-  so that changes to .bzrignore aren't missed. (#129694, Daniel Watkins)
-
-* ``bzr selftest --strict`` fails if there are any missing features or
-  expected test failures. (Daniel Watkins, #111914)
-
-* Link to registration survey added to README. (Ian Clatworthy)
-
-* Windows standalone installer show link to registration survey
-  when installation finished. (Alexander Belchenko)
-
-Library API Breaks
-******************
-
-* Deprecated dictionary ``bzrlib.option.SHORT_OPTIONS`` removed.
-  Options are now required to provide a help string and it must
-  comply with the style guide by being one or more sentences with an
-  initial capital and final period. (Martin Pool)
-
-* KnitIndex.get_parents now returns tuples. (Robert Collins)
-
-* Ancient unused ``Repository.text_store`` attribute has been removed.
-  (Robert Collins)
-
-* The ``bzrlib.pack`` interface has changed to use tuples of bytestrings
-  rather than just bytestrings, making it easier to represent multiple
-  element names. As this interface was not used by any internal facilities
-  since it was introduced in 0.18 no API compatibility is being preserved.
-  The serialised form of these packs is identical with 0.18 when a single
-  element tuple is in use. (Robert Collins)
-
-Internals
-*********
-
-* merge now uses ``iter_changes`` to calculate changes, which makes room for
-  future performance increases.  It is also more consistent with other
-  operations that perform comparisons, and reduces reliance on
-  Tree.inventory.  (Aaron Bentley)
-
-* Refactoring of transport classes connected to a remote server.
-  ConnectedTransport is a new class that serves as a basis for all
-  transports needing to connect to a remote server.  transport.split_url
-  have been deprecated, use the static method on the object instead. URL
-  tests have been refactored too.
-  (Vincent Ladeuil)
-
-* Better connection sharing for ConnectedTransport objects.
-  transport.get_transport() now accepts a 'possible_transports' parameter.
-  If a newly requested transport can share a connection with one of the
-  list, it will.
-  (Vincent Ladeuil)
-
-* Most functions now accept ``bzrlib.revision.NULL_REVISION`` to indicate
-  the null revision, and consider using ``None`` for this purpose
-  deprecated.  (Aaron Bentley)
-
-* New ``index`` module with abstract index functionality. This will be
-  used during the planned changes in the repository layer. Currently the
-  index layer provides a graph aware immutable index, a builder for the
-  same index type to allow creating them, and finally a composer for
-  such indices to allow the use of many indices in a single query. The
-  index performance is not optimised, however the API is stable to allow
-  development on top of the index. (Robert Collins)
-
-* ``bzrlib.dirstate.cmp_by_dirs`` can be used to compare two paths by
-  their directory sections. This is equivalent to comparing
-  ``path.split('/')``, only without having to split the paths.
-  This has a Pyrex implementation available.
-  (John Arbash Meinel)
-
-* New transport decorator 'unlistable+' which disables the list_dir
-  functionality for testing.
-
-* Deprecated ``change_entry`` in transform.py. (Ian Clatworthy)
-
-* RevisionTree.get_weave is now deprecated.  Tree.plan_merge is now used
-  for performing annotate-merge.  (Aaron Bentley)
-
-* New EmailMessage class to create email messages. (Adeodato Simó)
-
-* Unused functions on the private interface KnitIndex have been removed.
-  (Robert Collins)
-
-* New ``knit.KnitGraphIndex`` which provides a ``KnitIndex`` layered on top
-  of a ``index.GraphIndex``. (Robert Collins)
-
-* New ``knit.KnitVersionedFile.iter_parents`` method that allows querying
-  the parents of many knit nodes at once, reducing round trips to the
-  underlying index. (Robert Collins)
-
-* Graph now has an is_ancestor method, various bits use it.
-  (Aaron Bentley)
-
-* The ``-Dhpss`` flag now includes timing information. As well as
-  logging when a new connection is opened. (John Arbash Meinel)
-
-* ``bzrlib.pack.ContainerWriter`` now returns an offset, length tuple to
-  callers when inserting data, allowing generation of readv style access
-  during pack creation, without needing a separate pass across the output
-  pack to gather such details. (Robert Collins)
-
-* ``bzrlib.pack.make_readv_reader`` allows readv based access to pack
-  files that are stored on a transport. (Robert Collins)
-
-* New ``Repository.has_same_location`` method that reports if two
-  repository objects refer to the same repository (although with some risk
-  of false negatives).  (Andrew Bennetts)
-
-* InterTree.compare now passes require_versioned on correctly.
-  (Marius Kruger)
-
-* New methods on Repository - ``start_write_group``,
-  ``commit_write_group``, ``abort_write_group`` and ``is_in_write_group`` -
-  which provide a clean hook point for transactional Repositories - ones
-  where all the data for a fetch or commit needs to be made atomically
-  available in one step. This allows the write lock to remain while making
-  a series of data insertions.  (e.g. data conversion). (Robert Collins)
-
-* In ``bzrlib.knit`` the internal interface has been altered to use
-  3-tuples (index, pos, length) rather than two-tuples (pos, length) to
-  describe where data in a knit is, allowing knits to be split into
-  many files. (Robert Collins)
-
-* ``bzrlib.knit._KnitData`` split into cache management and physical access
-  with two access classes - ``_PackAccess`` and ``_KnitAccess`` defined.
-  The former provides access into a .pack file, and the latter provides the
-  current production repository form of .knit files. (Robert Collins)
-
-Testing
-*******
-
-* Remove selftest ``--clean-output``, ``--numbered-dirs`` and
-  ``--keep-output`` options, which are obsolete now that tests
-  are done within directories in $TMPDIR.  (Martin Pool)
-
-* The SSH_AUTH_SOCK environment variable is now reset to avoid
-  interaction with any running ssh agents.  (Jelmer Vernooij, #125955)
-
-* run_bzr_subprocess handles parameters the same way as run_bzr:
-  either a string or a list of strings should be passed as the first
-  parameter.  Varargs-style parameters are deprecated. (Aaron Bentley)
-
-
-bzr 0.18
-########
-
-:Released:  2007-07-17
-
-Bugfixes
-********
-
-* Fix 'bzr add' crash under Win32 (Kuno Meyer)
-
-
-bzr 0.18rc1
-###########
-
-:Released:  2007-07-10
-
-Bugfixes
-********
-
-* Do not suppress pipe errors, etc. in non-display commands
-  (Alexander Belchenko, #87178)
-
-* Display a useful error message when the user requests to annotate
-  a file that is not present in the specified revision.
-  (James Westby, #122656)
-
-* Commands that use status flags now have a reference to 'help
-  status-flags'.  (Daniel Watkins, #113436)
-
-* Work around python-2.4.1 inhability to correctly parse the
-  authentication header.
-  (Vincent Ladeuil, #121889)
-
-* Use exact encoding for merge directives. (Adeodato Simó, #120591)
-
-* Fix tempfile permissions error in smart server tar bundling under
-  Windows. (Martin _, #119330)
-
-* Fix detection of directory entries in the inventory. (James Westby)
-
-* Fix handling of http code 400: Bad Request When issuing too many ranges.
-  (Vincent Ladeuil, #115209)
-
-* Issue a CONNECT request when connecting to an https server
-  via a proxy to enable SSL tunneling.
-  (Vincent Ladeuil, #120678)
-
-* Fix ``bzr log -r`` to support selecting merge revisions, both
-  individually and as part of revision ranges.
-  (Kent Gibson, #4663)
-
-* Don't leave cruft behind when failing to acquire a lockdir.
-  (Martin Pool, #109169)
-
-* Don't use the '-f' strace option during tests.
-  (Vincent Ladeuil, #102019).
-
-* Warn when setting ``push_location`` to a value that will be masked by
-  locations.conf.  (Aaron Bentley, #122286)
-
-* Fix commit ordering in corner case (Aaron Bentley, #94975)
-
-*  Make annotate behave in a non-ASCII world (Adeodato Simó).
-
-Improvements
-************
-
-* The --lsprof-file option now dumps a text rendering of the profiling
-  information if the filename ends in ".txt". It will also convert the
-  profiling information to a format suitable for KCacheGrind if the
-  output filename ends in ".callgrind". Fixes to the lsprofcalltree
-  conversion process by Jean Paul Calderone and Itamar were also merged.
-  See http://ddaa.net/blog/python/lsprof-calltree. (Ian Clatworthy)
-
-* ``info`` now defaults to non-verbose mode, displaying only paths and
-  abbreviated format info.  ``info -v`` displays all the information
-  formerly displayed by ``info``.  (Aaron Bentley, Adeodato Simó)
-
-* ``bzr missing`` now has better option names ``--this`` and ``--other``.
-  (Elliot Murphy)
-
-* The internal ``weave-list`` command has become ``versionedfile-list``,
-  and now lists knits as well as weaves.  (Aaron Bentley)
-
-* Automatic merge base selection uses a faster algorithm that chooses
-  better bases in criss-cross merge situations (Aaron Bentley)
-
-* Progress reporting in ``commit`` has been improved. The various logical
-  stages are now reported on as follows, namely:
-
-  * Collecting changes [Entry x/y] - Stage n/m
-  * Saving data locally - Stage n/m
-  * Uploading data to master branch - Stage n/m
-  * Updating the working tree - Stage n/m
-  * Running post commit hooks - Stage n/m
-
-  If there is no master branch, the 3rd stage is omitted and the total
-  number of stages is adjusted accordingly.
-
-  Each hook that is run after commit is listed with a name (as hooks
-  can be slow it is useful feedback).
-  (Ian Clatworthy, Robert Collins)
-
-* Various operations that are now faster due to avoiding unnecessary
-  topological sorts. (Aaron Bentley)
-
-* Make merge directives robust against broken bundles. (Aaron Bentley)
-
-* The lsprof filename note is emitted via trace.note(), not standard
-  output.  (Aaron Bentley)
-
-* ``bzrlib`` now exports explicit API compatibility information to assist
-  library users and plugins. See the ``bzrlib.api`` module for details.
-  (Robert Collins)
-
-* Remove unnecessary lock probes when acquiring a lockdir.
-  (Martin Pool)
-
-* ``bzr --version`` now shows the location of the bzr log file, which
-  is especially useful on Windows.  (Martin Pool)
-
-* -D now supports hooks to get debug tracing of hooks (though its currently
-  minimal in nature). (Robert Collins)
-
-* Long log format reports deltas on merge revisions.
-  (John Arbash Meinel, Kent Gibson)
-
-* Make initial push over ftp more resilient. (John Arbash Meinel)
-
-* Print a summary of changes for update just like pull does.
-  (Daniel Watkins, #113990)
-
-* Add a -Dhpss option to trace smart protocol requests and responses.
-  (Andrew Bennetts)
-
-Library API Breaks
-******************
-
-* Testing cleanups -
-  ``bzrlib.repository.RepositoryTestProviderAdapter`` has been moved
-  to ``bzrlib.tests.repository_implementations``;
-  ``bzrlib.repository.InterRepositoryTestProviderAdapter`` has been moved
-  to ``bzrlib.tests.interrepository_implementations``;
-  ``bzrlib.transport.TransportTestProviderAdapter`` has moved to
-  ``bzrlib.tests.test_transport_implementations``.
-  ``bzrlib.branch.BranchTestProviderAdapter`` has moved to
-  ``bzrlib.tests.branch_implementations``.
-  ``bzrlib.bzrdir.BzrDirTestProviderAdapter`` has moved to
-  ``bzrlib.tests.bzrdir_implementations``.
-  ``bzrlib.versionedfile.InterVersionedFileTestProviderAdapter`` has moved
-  to ``bzrlib.tests.interversionedfile_implementations``.
-  ``bzrlib.store.revision.RevisionStoreTestProviderAdapter`` has moved to
-  ``bzrlib.tests.revisionstore_implementations``.
-  ``bzrlib.workingtree.WorkingTreeTestProviderAdapter`` has moved to
-  ``bzrlib.tests.workingtree_implementations``.
-  These changes are an API break in the testing infrastructure only.
-  (Robert Collins)
-
-* Relocate TestCaseWithRepository to be more central. (Robert Collins)
-
-* ``bzrlib.add.smart_add_tree`` will no longer perform glob expansion on
-  win32. Callers of the function should do this and use the new
-  ``MutableTree.smart_add`` method instead. (Robert Collins)
-
-* ``bzrlib.add.glob_expand_for_win32`` is now
-  ``bzrlib.win32utils.glob_expand``.  (Robert Collins)
-
-* ``bzrlib.add.FastPath`` is now private and moved to
-  ``bzrlib.mutabletree._FastPath``. (Robert Collins, Martin Pool)
-
-* ``LockDir.wait`` removed.  (Martin Pool)
-
-* The ``SmartServer`` hooks API has changed for the ``server_started`` and
-  ``server_stopped`` hooks. The first parameter is now an iterable of
-  backing URLs rather than a single URL. This is to reflect that many
-  URLs may map to the external URL of the server. E.g. the server interally
-  may have a chrooted URL but also the local file:// URL will be at the
-  same location. (Robert Collins)
-
-Internals
-*********
-
-* New SMTPConnection class to unify email handling.  (Adeodato Simó)
-
-* Fix documentation of BzrError. (Adeodato Simó)
-
-* Make BzrBadParameter an internal error. (Adeodato Simó)
-
-* Remove use of 'assert False' to raise an exception unconditionally.
-  (Martin Pool)
-
-* Give a cleaner error when failing to decode knit index entry.
-  (Martin Pool)
-
-* TreeConfig would mistakenly search the top level when asked for options
-  from a section. It now respects the section argument and only
-  searches the specified section. (James Westby)
-
-* Improve ``make api-docs`` output. (John Arbash Meinel)
-
-* Use os.lstat rather than os.stat for osutils.make_readonly and
-  osutils.make_writeable. This makes the difftools plugin more
-  robust when dangling symlinks are found. (Elliot Murphy)
-
-* New ``-Dlock`` option to log (to ~/.bzr.log) information on when
-  lockdirs are taken or released.  (Martin Pool)
-
-* ``bzrlib`` Hooks are now nameable using ``Hooks.name_hook``. This
-  allows a nicer UI when hooks are running as the current hook can
-  be displayed. (Robert Collins)
-
-* ``Transport.get`` has had its interface made more clear for ease of use.
-  Retrieval of a directory must now fail with either 'PathError' at open
-  time, or raise 'ReadError' on a read. (Robert Collins)
-
-* New method ``_maybe_expand_globs`` on the ``Command`` class for
-  dealing with unexpanded glob lists - e.g. on the win32 platform. This
-  was moved from ``bzrlib.add._prepare_file_list``. (Robert Collins)
-
-* ``bzrlib.add.smart_add`` and ``bzrlib.add.smart_add_tree`` are now
-  deprecated in favour of ``MutableTree.smart_add``. (Robert Collins,
-  Martin Pool)
-
-* New method ``external_url`` on Transport for obtaining the url to
-  hand to external processes. (Robert Collins)
-
-* Teach windows installers to build pyrex/C extensions.
-  (Alexander Belchenko)
-
-Testing
-*******
-
-* Removed the ``--keep-output`` option from selftest and clean up test
-  directories as they're used.  This reduces the IO load from
-  running the test suite and cuts the time by about half.
-  (Andrew Bennetts, Martin Pool)
-
-* Add scenarios as a public attribute on the TestAdapter classes to allow
-  modification of the generated scenarios before adaption and easier
-  testing. (Robert Collins)
-
-* New testing support class ``TestScenarioApplier`` which multiplies
-  out a single teste by a list of supplied scenarios. (RobertCollins)
-
-* Setting ``repository_to_test_repository`` on a repository_implementations
-  test will cause it to be called during repository creation, allowing the
-  testing of repository classes which are not based around the Format
-  concept. For example a repository adapter can be tested in this manner,
-  by altering the repository scenarios to include a scenario that sets this
-  attribute during the test parameterisation in
-  ``bzrlib.tests.repository.repository_implementations``. (Robert Collins)
-
-* Clean up many of the APIs for blackbox testing of Bazaar.  The standard
-  interface is now self.run_bzr.  The command to run can be passed as
-  either a list of parameters, a string containing the command line, or
-  (deprecated) varargs parameters.  (Martin Pool)
-
-* The base TestCase now isolates tests from -D parameters by clearing
-  ``debug.debug_flags`` and restores it afterwards. (Robert Collins)
-
-* Add a relpath parameter to get_transport methods in test framework to
-  avoid useless cloning.
-  (Vincent Ladeuil, #110448)
-
-
-bzr 0.17
-########
-
-:Released:  2007-06-18
-
-Bugfixes
-********
-
-* Fix crash of commit due to wrong lookup of filesystem encoding.
-  (Colin Watson, #120647)
-
-* Revert logging just to stderr in commit as broke unicode filenames.
-  (Aaron Bentley, Ian Clatworthy, #120930)
-
-
-bzr 0.17rc1
-###########
-
-:Released:  2007-06-12
-
-Notes When Upgrading
-********************
-
-* The kind() and is_executable() APIs on the WorkingTree interface no
-  longer implicitly (read) locks and unlocks the tree. This *might*
-  impact some plug-ins and tools using this part of the API. If you find
-  an issue that may be caused by this change, please let us know,
-  particularly the plug-in/tool maintainer. If encountered, the API
-  fix is to surround kind() and is_executable() calls with lock_read()
-  and unlock() like so::
-
-    work_tree.lock_read()
-    try:
-        kind = work_tree.kind(...)
-    finally:
-        work_tree.unlock()
-
-Internals
-*********
-* Rework of LogFormatter API to provide beginning/end of log hooks and to
-  encapsulate the details of the revision to be logged in a LogRevision
-  object.
-  In long log formats, merge revision ids are only shown when --show-ids
-  is specified, and are labelled "revision-id:", as per mainline
-  revisions, instead of "merged:". (Kent Gibson)
-
-* New ``BranchBuilder`` API which allows the construction of particular
-  histories quickly. Useful for testing and potentially other applications
-  too. (Robert Collins)
-
-Improvements
-************
-
-* There are two new help topics, working-trees and repositories that
-  attempt to explain these concepts. (James Westby, John Arbash Meinel,
-  Aaron Bentley)
-
-* Added ``bzr log --limit`` to report a limited number of revisions.
-  (Kent Gibson, #3659)
-
-* Revert does not try to preserve file contents that were originally
-  produced by reverting to a historical revision.  (Aaron Bentley)
-
-* ``bzr log --short`` now includes ``[merge]`` for revisions which
-  have more than one parent. This is a small improvement to help
-  understanding what changes have occurred
-  (John Arbash Meinel, #83887)
-
-* TreeTransform avoids many renames when contructing large trees,
-  improving speed.  3.25x speedups have been observed for construction of
-  kernel-sized-trees, and checkouts are 1.28x faster.  (Aaron Bentley)
-
-* Commit on large trees is now faster. In my environment, a commit of
-  a small change to the Mozilla tree (55k files) has dropped from
-  66 seconds to 32 seconds. For a small tree of 600 files, commit of a
-  small change is 33% faster. (Ian Clatworthy)
-
-* New --create-prefix option to bzr init, like for push.  (Daniel Watkins,
-  #56322)
-
-Bugfixes
-********
-
-* ``bzr push`` should only connect to the remote location one time.
-  We have been connecting 3 times because we forget to pass around
-  the Transport object. This adds ``BzrDir.clone_on_transport()``, so
-  that we can pass in the Transport that we already have.
-  (John Arbash Meinel, #75721)
-
-* ``DirState.set_state_from_inventory()`` needs to properly order
-  based on split paths, not just string paths.
-  (John Arbash Meinel, #115947)
-
-* Let TestUIFactoy encode the password prompt with its own stdout.
-  (Vincent Ladeuil, #110204)
-
-* pycurl should take use the range header that takes the range hint
-  into account.
-  (Vincent Ladeuil, #112719)
-
-* WorkingTree4.get_file_sha1 no longer raises an exception when invoked
-  on a missing file.  (Aaron Bentley, #118186)
-
-* WorkingTree.remove works correctly with tree references, and when pwd is
-  not the tree root. (Aaron Bentley)
-
-* Merge no longer fails when a file is renamed in one tree and deleted
-  in the other. (Aaron Bentley, #110279)
-
-* ``revision-info`` now accepts dotted revnos, doesn't require a tree,
-  and defaults to the last revision (Matthew Fuller, #90048)
-
-* Tests no longer fail when BZR_REMOTE_PATH is set in the environment.
-  (Daniel Watkins, #111958)
-
-* ``bzr branch -r revid:foo`` can be used to branch any revision in
-  your repository. (Previously Branch6 only supported revisions in your
-  mainline). (John Arbash Meinel, #115343)
-
-bzr 0.16
-########
-
-:Released:  2007-05-07
-
-Bugfixes
-********
-
-* Handle when you have 2 directories with similar names, but one has a
-  hyphen. (``'abc'`` versus ``'abc-2'``). The WT4._iter_changes
-  iterator was using direct comparison and ``'abc/a'`` sorts after
-  ``'abc-2'``, but ``('abc', 'a')`` sorts before ``('abc-2',)``.
-  (John Arbash Meinel, #111227)
-
-* Handle when someone renames a file on disk without telling bzr.
-  Previously we would report the first file as missing, but not show
-  the new unknown file. (John Arbash Meinel, #111288)
-
-* Avoid error when running hooks after pulling into or pushing from
-  a branch bound to a smartserver branch.  (Martin Pool, #111968)
-
-Improvements
-************
-
-* Move developer documentation to doc/developers/. This reduces clutter in
-  the root of the source tree and allows HACKING to be split into multiple
-  files. (Robert Collins, Alexander Belchenko)
-
-* Clean up the ``WorkingTree4._iter_changes()`` internal loops as well as
-  ``DirState.update_entry()``. This optimizes the core logic for ``bzr
-  diff`` and ``bzr status`` significantly improving the speed of
-  both. (John Arbash Meinel)
-
-bzr 0.16rc2
-###########
-
-:Released:  2007-04-30
-
-Bugfixes
-********
-
-* Handle the case when you delete a file, and then rename another file
-  on top of it. Also handle the case of ``bzr rm --keep foo``. ``bzr
-  status`` should show the removed file and an unknown file in its
-  place. (John Arbash Meinel, #109993)
-
-* Bundles properly read and write revision properties that have an
-  empty value. And when the value is not ASCII.
-  (John Arbash Meinel, #109613)
-
-* Fix the bzr commit message to be in text mode.
-  (Alexander Belchenko, #110901)
-
-* Also handle when you rename a file and create a file where it used
-  to be. (John Arbash Meinel, #110256)
-
-* ``WorkingTree4._iter_changes`` should not descend into unversioned
-  directories. (John Arbash Meinel, #110399)
-
-bzr 0.16rc1
-###########
-
-:Released:  2007-04-26
-
-Notes When Upgrading
-********************
-
-* ``bzr remove`` and ``bzr rm`` will now remove the working file, if
-  it could be recovered again.
-  This has been done for consistency with svn and the unix rm command.
-  The old ``remove`` behaviour has been retained in the new option
-  ``bzr remove --keep``, which will just stop versioning the file,
-  but not delete it.
-  ``bzr remove --force`` have been added which will always delete the
-  files.
-  ``bzr remove`` is also more verbose.
-  (Marius Kruger, #82602)
-
-Improvements
-************
-
-* Merge directives can now be supplied as input to `merge` and `pull`,
-  like bundles can.  (Aaron Bentley)
-
-* Sending the SIGQUIT signal to bzr, which can be done on Unix by
-  pressing Control-Backslash, drops bzr into a debugger.  Type ``'c'``
-  to continue.  This can be disabled by setting the environment variable
-  ``BZR_SIGQUIT_PDB=0``.  (Martin Pool)
-
-* selftest now supports --list-only to list tests instead of running
-  them. (Ian Clatworthy)
-
-* selftest now supports --exclude PATTERN (or -x PATTERN) to exclude
-  tests with names that match that regular expression.
-  (Ian Clatworthy, #102679)
-
-* selftest now supports --randomize SEED to run tests in a random order.
-  SEED is typically the value 'now' meaning 'use the current time'.
-  (Ian Clatworthy, #102686)
-
-* New option ``--fixes`` to commit, which stores bug fixing annotations as
-  revision properties. Built-in support for Launchpad, Debian, Trac and
-  Bugzilla bug trackers. (Jonathan Lange, James Henstridge, Robert Collins)
-
-* New API, ``bzrlib.bugtracker.tracker_registry``, for adding support for
-  other bug trackers to ``fixes``. (Jonathan Lange, James Henstridge,
-  Robert Collins)
-
-* ``selftest`` has new short options ``-f`` and ``-1``.  (Martin
-  Pool)
-
-* ``bzrlib.tsort.MergeSorter`` optimizations. Change the inner loop
-  into using local variables instead of going through ``self._var``.
-  Improves the time to ``merge_sort`` a 10k revision graph by
-  approximately 40% (~700->400ms).  (John Arbash Meinel)
-
-* ``make docs`` now creates a man page at ``man1/bzr.1`` fixing bug 107388.
-  (Robert Collins)
-
-* ``bzr help`` now provides cross references to other help topics using
-  the _see_also facility on command classes. Likewise the bzr_man
-  documentation, and the bzr.1 man page also include this information.
-  (Robert Collins)
-
-* Tags are now included in logs, that use the long log formatter.
-  (Erik Bågfors, Alexander Belchenko)
-
-* ``bzr help`` provides a clearer message when a help topic cannot be
-  found. (Robert Collins, #107656)
-
-* ``bzr help`` now accepts optional prefixes for command help. The help
-  for all commands can now be found at ``bzr help commands/COMMANDNAME``
-  as well as ``bzr help COMMANDNAME`` (which only works for commands
-  where the name is not the same as a more general help topic).
-  (Robert Collins)
-
-* ``bzr help PLUGINNAME`` will now return the module docstring from the
-  plugin PLUGINNAME. (Robert Collins, #50408)
-
-* New help topic ``urlspec`` which lists the availables transports.
-  (Goffredo Baroncelli)
-
-* doc/server.txt updated to document the default bzr:// port
-  and also update the blurb about the hpss' current status.
-  (Robert Collins, #107125).
-
-* ``bzr serve`` now listens on interface 0.0.0.0 by default, making it
-  serve out to the local LAN (and anyone in the world that can reach the
-  machine running ``bzr serve``. (Robert Collins, #98918)
-
-* A new smart server protocol version has been added.  It prefixes requests
-  and responses with an explicit version identifier so that future protocol
-  revisions can be dealt with gracefully.  (Andrew Bennetts, Robert Collins)
-
-* The bzr protocol version 2 indicates success or failure in every response
-  without depending on particular commands encoding that consistently,
-  allowing future client refactorings to be much more robust about error
-  handling. (Robert Collins, Martin Pool, Andrew Bennetts)
-
-* The smart protocol over HTTP client has been changed to always post to the
-  same ``.bzr/smart`` URL under the original location when it can.  This allows
-  HTTP servers to only have to pass URLs ending in .bzr/smart to the smart
-  server handler, and not arbitrary ``.bzr/*/smart`` URLs.  (Andrew Bennetts)
-
-* digest authentication is now supported for proxies and HTTP by the urllib
-  based http implementation. Tested against Apache 2.0.55 and Squid
-  2.6.5. Basic and digest authentication are handled coherently for HTTP
-  and proxy: if the user is provided in the url (bzr command line for HTTP,
-  proxy environment variables for proxies), the password is prompted for
-  (only once). If the password is provided, it is taken into account. Once
-  the first authentication is successful, all further authentication
-  roundtrips are avoided by preventively setting the right authentication
-  header(s).
-  (Vincent Ladeuil).
-
-Internals
-*********
-
-* bzrlib API compatability with 0.8 has been dropped, cleaning up some
-  code paths. (Robert Collins)
-
-* Change the format of chroot urls so that they can be safely manipulated
-  by generic url utilities without causing the resulting urls to have
-  escaped the chroot. A side effect of this is that creating a chroot
-  requires an explicit action using a ChrootServer.
-  (Robert Collins, Andrew Bennetts)
-
-* Deprecate ``Branch.get_root_id()`` because branches don't have root ids,
-  rather than fixing bug #96847.  (Aaron Bentley)
-
-* ``WorkingTree.apply_inventory_delta`` provides a better alternative to
-  ``WorkingTree._write_inventory``.  (Aaron Bentley)
-
-* Convenience method ``TestCase.expectFailure`` ensures that known failures
-  do not silently pass.  (Aaron Bentley)
-
-* ``Transport.local_abspath`` now raises ``NotLocalUrl`` rather than
-  ``TransportNotPossible``. (Martin Pool, Ian Clatworthy)
-
-* New SmartServer hooks facility. There are two initial hooks documented
-  in ``bzrlib.transport.smart.SmartServerHooks``. The two initial hooks allow
-  plugins to execute code upon server startup and shutdown.
-  (Robert Collins).
-
-* SmartServer in standalone mode will now close its listening socket
-  when it stops, rather than waiting for garbage collection. This primarily
-  fixes test suite hangs when a test tries to connect to a shutdown server.
-  It may also help improve behaviour when dealing with a server running
-  on a specific port (rather than dynamically assigned ports).
-  (Robert Collins)
-
-* Move most SmartServer code into a new package, bzrlib/smart.
-  bzrlib/transport/remote.py contains just the Transport classes that used
-  to be in bzrlib/transport/smart.py.  (Andrew Bennetts)
-
-* urllib http implementation avoid roundtrips associated with
-  401 (and 407) errors once the authentication succeeds.
-  (Vincent Ladeuil).
-
-* urlib http now supports querying the user for a proxy password if
-  needed. Realm is shown in the prompt for both HTTP and proxy
-  authentication when the user is required to type a password.
-  (Vincent Ladeuil).
-
-* Renamed SmartTransport (and subclasses like SmartTCPTransport) to
-  RemoteTransport (and subclasses to RemoteTCPTransport, etc).  This is more
-  consistent with its new home in ``bzrlib/transport/remote.py``, and because
-  it's not really a "smart" transport, just one that does file operations
-  via remote procedure calls.  (Andrew Bennetts)
-
-* The ``lock_write`` method of ``LockableFiles``, ``Repository`` and
-  ``Branch`` now accept a ``token`` keyword argument, so that separate
-  instances of those objects can share a lock if it has the right token.
-  (Andrew Bennetts, Robert Collins)
-
-* New method ``get_branch_reference`` on ``BzrDir`` allows the detection of
-  branch references - which the smart server component needs.
-
-* The Repository API ``make_working_trees`` is now permitted to return
-  False when ``set_make_working_trees`` is not implemented - previously
-  an unimplemented ``set_make_working_trees`` implied the result True
-  from ``make_working_trees``. This has been changed to accomodate the
-  smart server, where it does not make sense (at this point) to ever
-  make working trees by default. (Robert Collins)
-
-* Command objects can now declare related help topics by having _see_also
-  set to a list of related topic. (Robert Collins)
-
-* ``bzrlib.help`` now delegates to the Command class for Command specific
-  help. (Robert Collins)
-
-* New class ``TransportListRegistry``, derived from the Registry class, which
-  simplifies tracking the available Transports. (Goffredo Baroncelli)
-
-* New function ``Branch.get_revision_id_to_revno_map`` which will
-  return a dictionary mapping revision ids to dotted revnos. Since
-  dotted revnos are defined in the context of the branch tip, it makes
-  sense to generate them from a ``Branch`` object.
-  (John Arbash Meinel)
-
-* Fix the 'Unprintable error' message display to use the repr of the
-  exception that prevented printing the error because the str value
-  for it is often not useful in debugging (e.g. KeyError('foo') has a
-  str() of 'foo' but a repr of 'KeyError('foo')' which is much more
-  useful. (Robert Collins)
-
-* ``urlutils.normalize_url`` now unescapes unreserved characters, such as "~".
-  (Andrew Bennetts)
-
-Bugfixes
-********
-
-* Don't fail bundle selftest if email has 'two' embedded.
-  (Ian Clatworthy, #98510)
-
-* Remove ``--verbose`` from ``bzr bundle``. It didn't work anyway.
-  (Robert Widhopf-Fenk, #98591)
-
-* Remove ``--basis`` from the checkout/branch commands - it didn't work
-  properly and is no longer beneficial.
-  (Robert Collins, #53675, #43486)
-
-* Don't produce encoding error when adding duplicate files.
-  (Aaron Bentley)
-
-* Fix ``bzr log <file>`` so it only logs the revisions that changed
-  the file, and does it faster.
-  (Kent Gibson, John Arbash Meinel, #51980, #69477)
-
-* Fix ``InterDirstateTre._iter_changes`` to handle when we come across
-  an empty versioned directory, which now has files in it.
-  (John Arbash Meinel, #104257)
-
-* Teach ``common_ancestor`` to shortcut when the tip of one branch is
-  inside the ancestry of the other. Saves a lot of graph processing
-  (with an ancestry of 16k revisions, ``bzr merge ../already-merged``
-  changes from 2m10s to 13s).  (John Arbash Meinel, #103757)
-
-* Fix ``show_diff_trees`` to handle the case when a file is modified,
-  and the containing directory is renamed. (The file path is different
-  in this versus base, but it isn't marked as a rename).
-  (John Arbash Meinel, #103870)
-
-* FTP now works even when the FTP server does not support atomic rename.
-  (Aaron Bentley, #89436)
-
-* Correct handling in bundles and merge directives of timezones with
-  that are not an integer number of hours offset from UTC.  Always
-  represent the epoch time in UTC to avoid problems with formatting
-  earlier times on win32.  (Martin Pool, Alexander Belchenko, John
-  Arbash Meinel)
-
-* Typo in the help for ``register-branch`` fixed. (Robert Collins, #96770)
-
-* "dirstate" and "dirstate-tags" formats now produce branches compatible
-  with old versions of bzr. (Aaron Bentley, #107168))
-
-* Handle moving a directory when children have been added, removed,
-  and renamed. (John Arbash Meinel, #105479)
-
-* Don't preventively use basic authentication for proxy before receiving a
-  407 error. Otherwise people willing to use other authentication schemes
-  may expose their password in the clear (or nearly). This add one
-  roundtrip in case basic authentication should be used, but plug the
-  security hole.
-  (Vincent Ladeuil)
-
-* Handle http and proxy digest authentication.
-  (Vincent Ladeuil, #94034).
-
-Testing
-*******
-
-* Added ``bzrlib.strace.strace`` which will strace a single callable and
-  return a StraceResult object which contains just the syscalls involved
-  in running it. (Robert Collins)
-
-* New test method ``reduceLockdirTimeout`` to drop the default (ui-centric)
-  default time down to one suitable for tests. (Andrew Bennetts)
-
-* Add new ``vfs_transport_factory`` attribute on tests which provides the
-  common vfs backing for both the readonly and readwrite transports.
-  This allows the RemoteObject tests to back onto local disk or memory,
-  and use the existing ``transport_server`` attribute all tests know about
-  to be the smart server transport. This in turn allows tests to
-  differentiate between 'transport to access the branch', and
-  'transport which is a VFS' - which matters in Remote* tests.
-  (Robert Collins, Andrew Bennetts)
-
-* The ``make_branch_and_tree`` method for tests will now create a
-  lightweight checkout for the tree if the ``vfs_transport_factory`` is not
-  a LocalURLServer. (Robert Collins, Andrew Bennetts)
-
-* Branch implementation tests have been audited to ensure that all urls
-  passed to Branch APIs use proper urls, except when local-disk paths
-  are intended. This is so that tests correctly access the test transport
-  which is often not equivalent to local disk in Remote* tests. As part
-  of this many tests were adjusted to remove dependencies on local disk
-  access.
-  (Robert Collins, Andrew Bennetts)
-
-* Mark bzrlib.tests and bzrlib.tests.TestUtil as providing assertFOO helper
-  functions by adding a ``__unittest`` global attribute. (Robert Collins,
-  Andrew Bennetts, Martin Pool, Jonathan Lange)
-
-* Refactored proxy and authentication handling to simplify the
-  implementation of new auth schemes for both http and proxy.
-  (Vincent Ladeuil)
-
-bzr 0.15
-########
-
-:Released: 2007-04-01
-
-Bugfixes
-********
-
-* Handle incompatible repositories as a user issue when fetching.
-  (Aaron Bentley)
-
-* Don't give a recommendation to upgrade when branching or
-  checking out a branch that contains an old-format working tree.
-  (Martin Pool)
-
-bzr 0.15rc3
-###########
-
-:Released:  2007-03-26
-
-Changes
-*******
-
-* A warning is now displayed when opening working trees in older
-  formats, to encourage people to upgrade to WorkingTreeFormat4.
-  (Martin Pool)
-
-Improvements
-************
-
-* HTTP redirections are now taken into account when a branch (or a
-  bundle) is accessed for the first time. A message is issued at each
-  redirection to inform the user. In the past, http redirections were
-  silently followed for each request which significantly degraded the
-  performances. The http redirections are not followed anymore by
-  default, instead a RedirectRequested exception is raised. For bzrlib
-  users needing to follow http redirections anyway,
-  ``bzrlib.transport.do_catching_redirections`` provide an easy transition
-  path.  (vila)
-
-Internals
-*********
-
-* Added ``ReadLock.temporary_write_lock()`` to allow upgrading an OS read
-  lock to an OS write lock. Linux can do this without unlocking, Win32
-  needs to unlock in between. (John Arbash Meinel)
-
-* New parameter ``recommend_upgrade`` to ``BzrDir.open_workingtree``
-  to silence (when false) warnings about opening old formats.
-  (Martin Pool)
-
-* Fix minor performance regression with bzr-0.15 on pre-dirstate
-  trees. (We were reading the working inventory too many times).
-  (John Arbash Meinel)
-
-* Remove ``Branch.get_transaction()`` in favour of a simple cache of
-  ``revision_history``.  Branch subclasses should override
-  ``_gen_revision_history`` rather than ``revision_history`` to make use of
-  this cache, and call ``_clear_revision_history_cache`` and
-  ``_cache_revision_history`` at appropriate times. (Andrew Bennetts)
-
-Bugfixes
-********
-
-* Take ``smtp_server`` from user config into account.
-  (vila, #92195)
-
-* Restore Unicode filename handling for versioned and unversioned files.
-  (John Arbash Meinel, #92608)
-
-* Don't fail during ``bzr commit`` if a file is marked removed, and
-  the containing directory is auto-removed.  (John Arbash Meinel, #93681)
-
-* ``bzr status FILENAME`` failed on Windows because of an uncommon
-  errno. (``ERROR_DIRECTORY == 267 != ENOTDIR``).
-  (Wouter van Heyst, John Arbash Meinel, #90819)
-
-* ``bzr checkout source`` should create a local branch in the same
-  format as source. (John Arbash Meinel, #93854)
-
-* ``bzr commit`` with a kind change was failing to update the
-  last-changed-revision for directories.  The
-  InventoryDirectory._unchanged only looked at the ``parent_id`` and name,
-  ignoring the fact that the kind could have changed, too.
-  (John Arbash Meinel, #90111)
-
-* ``bzr mv dir/subdir other`` was incorrectly updating files inside
-  the directory. So that there was a chance it would break commit,
-  etc. (John Arbash Meinel, #94037)
-
-* Correctly handles mutiple permanent http redirections.
-  (vila, #88780)
-
-bzr 0.15rc2
-###########
-
-:Released:  2007-03-14
-
-Notes When Upgrading
-********************
-
-* Release 0.15rc2 of bzr changes the ``bzr init-repo`` command to
-  default to ``--trees`` instead of ``--no-trees``.
-  Existing shared repositories are not affected.
-
-Improvements
-************
-
-* New ``merge-directive`` command to generate machine- and human-readable
-  merge requests.  (Aaron Bentley)
-
-* New ``submit:`` revision specifier makes it easy to diff against the
-  common ancestor with the submit location (Aaron Bentley)
-
-* Added support for Putty's SSH implementation. (Dmitry Vasiliev)
-
-* Added ``bzr status --versioned`` to report only versioned files,
-  not unknowns. (Kent Gibson)
-
-* Merge now autodetects the correct line-ending style for its conflict
-  markers.  (Aaron Bentley)
-
-Internals
-*********
-
-* Refactored SSH vendor registration into SSHVendorManager class.
-  (Dmitry Vasiliev)
-
-Bugfixes
-********
-
-* New ``--numbered-dirs`` option to ``bzr selftest`` to use
-  numbered dirs for TestCaseInTempDir. This is default behavior
-  on Windows. Anyone can force named dirs on Windows
-  with ``--no-numbered-dirs``. (Alexander Belchenko)
-
-* Fix ``RevisionSpec_revid`` to handle the Unicode strings passed in
-  from the command line. (Marien Zwart, #90501)
-
-* Fix ``TreeTransform._iter_changes`` when both the source and
-  destination are missing. (Aaron Bentley, #88842)
-
-* Fix commit of merges with symlinks in dirstate trees.
-  (Marien Zwart)
-
-* Switch the ``bzr init-repo`` default from --no-trees to --trees.
-  (Wouter van Heyst, #53483)
-
-
-bzr 0.15rc1
-###########
-
-:Released:  2007-03-07
-
-Surprises
-*********
-
-* The default disk format has changed. Please run 'bzr upgrade' in your
-  working trees to upgrade. This new default is compatible for network
-  operations, but not for local operations. That is, if you have two
-  versions of bzr installed locally, after upgrading you can only use the
-  bzr 0.15 version. This new default does not enable tags or nested-trees
-  as they are incompatible with bzr versions before 0.15 over the network.
-
-* For users of bzrlib: Two major changes have been made to the working tree
-  api in bzrlib. The first is that many methods and attributes, including
-  the inventory attribute, are no longer valid for use until one of
-  ``lock_read``/``lock_write``/``lock_tree_write`` has been called,
-  and become invalid again after unlock is called. This has been done
-  to improve performance and correctness as part of the dirstate
-  development.
-  (Robert Collins, John A Meinel, Martin Pool, and others).
-
-* For users of bzrlib: The attribute 'tree.inventory' should be considered
-  readonly. Previously it was possible to directly alter this attribute, or
-  its contents, and have the tree notice this. This has been made
-  unsupported - it may work in some tree formats, but in the newer dirstate
-  format such actions will have no effect and will be ignored, or even
-  cause assertions. All operations possible can still be carried out by a
-  combination of the tree API, and the bzrlib.transform API. (Robert
-  Collins, John A Meinel, Martin Pool, and others).
-
-Improvements
-************
-
-* Support for OS Windows 98. Also .bzr.log on any windows system
-  saved in My Documents folder. (Alexander Belchenko)
-
-* ``bzr mv`` enhanced to support already moved files.
-  In the past the mv command would have failed if the source file doesn't
-  exist. In this situation ``bzr mv`` would now detect that the file has
-  already moved and update the repository accordingly, if the target file
-  does exist.
-  A new option ``--after`` has been added so that if two files already
-  exist, you could notify Bazaar that you have moved a (versioned) file
-  and replaced it with another. Thus in this case ``bzr move --after``
-  will only update the Bazaar identifier.
-  (Steffen Eichenberg, Marius Kruger)
-
-* ``ls`` now works on treeless branches and remote branches.
-  (Aaron Bentley)
-
-* ``bzr help global-options`` describes the global options.
-  (Aaron Bentley)
-
-* ``bzr pull --overwrite`` will now correctly overwrite checkouts.
-  (Robert Collins)
-
-* Files are now allowed to change kind (e.g. from file to symlink).
-  Supported by ``commit``, ``revert`` and ``status``
-  (Aaron Bentley)
-
-* ``inventory`` and ``unknowns`` hidden in favour of ``ls``
-  (Aaron Bentley)
-
-* ``bzr help checkouts`` descibes what checkouts are and some possible
-  uses of them. (James Westby, Aaron Bentley)
-
-* A new ``-d`` option to push, pull and merge overrides the default
-  directory.  (Martin Pool)
-
-* Branch format 6: smaller, and potentially faster than format 5.  Supports
-  ``append_history_only`` mode, where the log view and revnos do not change,
-  except by being added to.  Stores policy settings in
-  ".bzr/branch/branch.conf".
-
-* ``append_only`` branches:  Format 6 branches may be configured so that log
-  view and revnos are always consistent.  Either create the branch using
-  "bzr init --append-revisions-only" or edit the config file as descriped
-  in docs/configuration.txt.
-
-* rebind: Format 6 branches retain the last-used bind location, so if you
-  "bzr unbind", you can "bzr bind" to bind to the previously-selected
-  bind location.
-
-* Builtin tags support, created and deleted by the ``tag`` command and
-  stored in the branch.  Tags can be accessed with the revisionspec
-  ``-rtag:``, and listed with ``bzr tags``.  Tags are not versioned
-  at present. Tags require a network incompatible upgrade. To perform this
-  upgrade, run ``bzr upgrade --dirstate-tags`` in your branch and
-  repositories. (Martin Pool)
-
-* The ``bzr://`` transport now has a well-known port number, 4155,
-  which it will use by default.  (Andrew Bennetts, Martin Pool)
-
-* Bazaar now looks for user-installed plugins before looking for site-wide
-  plugins. (Jonathan Lange)
-
-* ``bzr resolve`` now detects and marks resolved text conflicts.
-  (Aaron Bentley)
-
-Internals
-*********
-
-* Internally revision ids and file ids are now passed around as utf-8
-  bytestrings, rather than treating them as Unicode strings. This has
-  performance benefits for Knits, since we no longer need to decode the
-  revision id for each line of content, nor for each entry in the index.
-  This will also help with the future dirstate format.
-  (John Arbash Meinel)
-
-* Reserved ids (any revision-id ending in a colon) are rejected by
-  versionedfiles, repositories, branches, and working trees
-  (Aaron Bentley)
-
-* Minor performance improvement by not creating a ProgressBar for
-  every KnitIndex we create. (about 90ms for a bzr.dev tree)
-  (John Arbash Meinel)
-
-* New easier to use Branch hooks facility. There are five initial hooks,
-  all documented in bzrlib.branch.BranchHooks.__init__ - ``'set_rh'``,
-  ``'post_push'``, ``'post_pull'``, ``'post_commit'``,
-  ``'post_uncommit'``. These hooks fire after the matching operation
-  on a branch has taken place, and were originally added for the
-  branchrss plugin. (Robert Collins)
-
-* New method ``Branch.push()`` which should be used when pushing from a
-  branch as it makes performance and policy decisions to match the UI
-  level command ``push``. (Robert Collins).
-
-* Add a new method ``Tree.revision_tree`` which allows access to cached
-  trees for arbitrary revisions. This allows the in development dirstate
-  tree format to provide access to the callers to cached copies of
-  inventory data which are cheaper to access than inventories from the
-  repository.
-  (Robert Collins, Martin Pool)
-
-* New ``Branch.last_revision_info`` method, this is being done to allow
-  optimization of requests for both the number of revisions and the last
-  revision of a branch with smartservers and potentially future branch
-  formats. (Wouter van Heyst, Robert Collins)
-
-* Allow ``'import bzrlib.plugins.NAME'`` to work when the plugin NAME has not
-  yet been loaded by ``load_plugins()``. This allows plugins to depend on each
-  other for code reuse without requiring users to perform file-renaming
-  gymnastics. (Robert Collins)
-
-* New Repository method ``'gather_stats'`` for statistic data collection.
-  This is expected to grow to cover a number of related uses mainly
-  related to bzr info. (Robert Collins)
-
-* Log formatters are now managed with a registry.
-  ``log.register_formatter`` continues to work, but callers accessing
-  the FORMATTERS dictionary directly will not.
-
-* Allow a start message to be passed to the ``edit_commit_message``
-  function.  This will be placed in the message offered to the user
-  for editing above the separator. It allows a template commit message
-  to be used more easily. (James Westby)
-
-* ``GPGStrategy.sign()`` will now raise ``BzrBadParameterUnicode`` if
-  you pass a Unicode string rather than an 8-bit string. Callers need
-  to be updated to encode first. (John Arbash Meinel)
-
-* Branch.push, pull, merge now return Result objects with information
-  about what happened, rather than a scattering of various methods.  These
-  are also passed to the post hooks.  (Martin Pool)
-
-* File formats and architecture is in place for managing a forest of trees
-  in bzr, and splitting up existing trees into smaller subtrees, and
-  finally joining trees to make a larger tree. This is the first iteration
-  of this support, and the user-facing aspects still require substantial
-  work.  If you wish to experiment with it, use ``bzr upgrade
-  --dirstate-with-subtree`` in your working trees and repositories.
-  You can use the hidden commands ``split`` and ``join`` and to create
-  and manipulate nested trees, but please consider using the nested-trees
-  branch, which contains substantial UI improvements, instead.
-  http://code.aaronbentley.com/bzr/bzrrepo/nested-trees/
-  (Aaron Bentley, Martin Pool, Robert Collins).
-
-Bugfixes
-********
-
-* ``bzr annotate`` now uses dotted revnos from the viewpoint of the
-  branch, rather than the last changed revision of the file.
-  (John Arbash Meinel, #82158)
-
-* Lock operations no longer hang if they encounter a permission problem.
-  (Aaron Bentley)
-
-* ``bzr push`` can resume a push that was canceled before it finished.
-  Also, it can push even if the target directory exists if you supply
-  the ``--use-existing-dir`` flag.
-  (John Arbash Meinel, #30576, #45504)
-
-* Fix http proxy authentication when user and an optional
-  password appears in the ``*_proxy`` vars. (Vincent Ladeuil,
-  #83954).
-
-* ``bzr log branch/file`` works for local treeless branches
-  (Aaron Bentley, #84247)
-
-* Fix problem with UNC paths on Windows 98. (Alexander Belchenko, #84728)
-
-* Searching location of CA bundle for PyCurl in env variable
-  (``CURL_CA_BUNDLE``), and on win32 along the PATH.
-  (Alexander Belchenko, #82086)
-
-* ``bzr init`` works with unicode argument LOCATION.
-  (Alexander Belchenko, #85599)
-
-* Raise ``DependencyNotPresent`` if pycurl do not support https.
-  (Vincent Ladeuil, #85305)
-
-* Invalid proxy env variables should not cause a traceback.
-  (Vincent Ladeuil, #87765)
-
-* Ignore patterns normalised to use '/' path separator.
-  (Kent Gibson, #86451)
-
-* bzr rocks. It sure does! Fix case. (Vincent Ladeuil, #78026)
-
-* Fix bzrtools shelve command for removed lines beginning with "--"
-  (Johan Dahlberg, #75577)
-
-Testing
-*******
-
-* New ``--first`` option to ``bzr selftest`` to run specified tests
-  before the rest of the suite.  (Martin Pool)
-
-
-bzr 0.14
-########
-
-:Released:  2007-01-23
-
-Improvements
-************
-
-* ``bzr help global-options`` describes the global options. (Aaron Bentley)
-
-Bug Fixes
-*********
-
-* Skip documentation generation tests if the tools to do so are not
-  available. Fixes running selftest for installled copies of bzr.
-  (John Arbash Meinel, #80330)
-
-* Fix the code that discovers whether bzr is being run from it's
-  working tree to handle the case when it isn't but the directory
-  it is in is below a repository. (James Westby, #77306)
-
-
-bzr 0.14rc1
-###########
-
-:Released:  2007-01-16
-
-Improvements
-************
-
-* New connection: ``bzr+http://`` which supports tunnelling the smart
-  protocol over an HTTP connection. If writing is enabled on the bzr
-  server, then you can write over the http connection.
-  (Andrew Bennetts, John Arbash Meinel)
-
-* Aliases now support quotation marks, so they can contain whitespace
-  (Marius Kruger)
-
-* PyCurlTransport now use a single curl object. By specifying explicitly
-  the 'Range' header, we avoid the need to use two different curl objects
-  (and two connections to the same server). (Vincent Ladeuil)
-
-* ``bzr commit`` does not prompt for a message until it is very likely to
-  succeed.  (Aaron Bentley)
-
-* ``bzr conflicts`` now takes --text to list pathnames of text conflicts
-  (Aaron Bentley)
-
-* Fix ``iter_lines_added_or_present_in_versions`` to use a set instead
-  of a list while checking if a revision id was requested. Takes 10s
-  off of the ``fileids_affected_by_revision_ids`` time, which is 10s
-  of the ``bzr branch`` time. Also improve ``fileids_...`` time by
-  filtering lines with a regex rather than multiple ``str.find()``
-  calls. (saves another 300ms) (John Arbash Meinel)
-
-* Policy can be set for each configuration key. This allows keys to be
-  inherited properly across configuration entries. For example, this
-  should enable you to do::
-
-    [/home/user/project]
-    push_location = sftp://host/srv/project/
-    push_location:policy = appendpath
-
-  And then a branch like ``/home/user/project/mybranch`` should get an
-  automatic push location of ``sftp://host/srv/project/mybranch``.
-  (James Henstridge)
-
-* Added ``bzr status --short`` to make status report svn style flags
-  for each file.  For example::
-
-    $ bzr status --short
-    A  foo
-    A  bar
-    D  baz
-    ?  wooley
-
-* 'bzr selftest --clean-output' allows easily clean temporary tests
-  directories without running tests. (Alexander Belchenko)
-
-* ``bzr help hidden-commands`` lists all hidden commands. (Aaron Bentley)
-
-* ``bzr merge`` now has an option ``--pull`` to fall back to pull if
-  local is fully merged into remote. (Jan Hudec)
-
-* ``bzr help formats`` describes available directory formats. (Aaron Bentley)
-
-Internals
-*********
-
-* A few tweaks directly to ``fileids_affected_by_revision_ids`` to
-  help speed up processing, as well allowing to extract unannotated
-  lines. Between the two ``fileids_affected_by_revision_ids`` is
-  improved by approx 10%. (John Arbash Meinel)
-
-* Change Revision serialization to only write out millisecond
-  resolution. Rather than expecting floating point serialization to
-  preserve more resolution than we need. (Henri Weichers, Martin Pool)
-
-* Test suite ends cleanly on Windows.  (Vincent Ladeuil)
-
-* When ``encoding_type`` attribute of class Command is equal to 'exact',
-  force sys.stdout to be a binary stream on Windows, and therefore
-  keep exact line-endings (without LF -> CRLF conversion).
-  (Alexander Belchenko)
-
-* Single-letter short options are no longer globally declared.  (Martin
-  Pool)
-
-* Before using detected user/terminal encoding bzr should check
-  that Python has corresponding codec. (Alexander Belchenko)
-
-* Formats for end-user selection are provided via a FormatRegistry (Aaron Bentley)
-
-Bug Fixes
-*********
-
-* ``bzr missing --verbose`` was showing adds/removals in the wrong
-  direction. (John Arbash Meinel)
-
-* ``bzr annotate`` now defaults to showing dotted revnos for merged
-  revisions. It cuts them off at a depth of 12 characters, but you can
-  supply ``--long`` to see the full number. You can also use
-  ``--show-ids`` to display the original revision ids, rather than
-  revision numbers and committer names. (John Arbash Meinel, #75637)
-
-* bzr now supports Win32 UNC path (e.g. ``\HOST\path``.
-  (Alexander Belchenko, #57869)
-
-* Win32-specific: output of cat, bundle and diff commands don't mangle
-  line-endings (Alexander Belchenko, #55276)
-
-* Replace broken fnmatch based ignore pattern matching with custom pattern
-  matcher.
-  (Kent Gibson, Jan Hudec #57637)
-
-* pycurl and urllib can detect short reads at different places. Update
-  the test suite to test more cases. Also detect http error code 416
-  which was raised for that specific bug. Also enhance the urllib
-  robustness by detecting invalid ranges (and pycurl's one by detecting
-  short reads during the initial GET). (Vincent Ladeuil, #73948)
-
-* The urllib connection sharing interacts badly with urllib2
-  proxy setting (the connections didn't go thru the proxy
-  anymore). Defining a proper ProxyHandler solves the
-  problem.  (Vincent Ladeuil, #74759)
-
-* Use urlutils to generate relative URLs, not osutils
-  (Aaron Bentley, #76229)
-
-* ``bzr status`` in a readonly directory should work without giving
-  lots of errors. (John Arbash Meinel, #76299)
-
-* Mention the revisionspec topic for the revision option help.
-  (Wouter van Heyst, #31663)
-
-* Allow plugins import from zip archives.
-  (Alexander Belchenko, #68124)
-
-
-bzr 0.13
-########
-
-:Released:  2006-12-05
-
-No changes from 0.13rc
-
-
-bzr 0.13rc1
-###########
-
-:Released:  2006-11-27
-
-Improvements
-************
-
-* New command ``bzr remove-tree`` allows the removal of the working
-  tree from a branch.
-  (Daniel Silverstone)
-
-* urllib uses shared keep-alive connections, so http
-  operations are substantially faster.
-  (Vincent Ladeuil, #53654)
-
-* ``bzr export`` allows an optional branch parameter, to export a bzr
-  tree from some other url. For example:
-  ``bzr export bzr.tar.gz http://bazaar-vcs.org/bzr/bzr.dev``
-  (Daniel Silverstone)
-
-* Added ``bzr help topics`` to the bzr help system. This gives a
-  location for general information, outside of a specific command.
-  This includes updates for ``bzr help revisionspec`` the first topic
-  included. (Goffredo Baroncelli, John Arbash Meinel, #42714)
-
-* WSGI-compatible HTTP smart server.  See ``doc/http_smart_server.txt``.
-  (Andrew Bennetts)
-
-* Knit files will now cache full texts only when the size of the
-  deltas is as large as the size of the fulltext. (Or after 200
-  deltas, whichever comes first). This has the most benefit on large
-  files with small changes, such as the inventory for a large project.
-  (eg For a project with 2500 files, and 7500 revisions, it changes
-  the size of inventory.knit from 11MB to 5.4MB) (John Arbash Meinel)
-
-Internals
-*********
-
-* New -D option given before the command line turns on debugging output
-  for particular areas.  -Derror shows tracebacks on all errors.
-  (Martin Pool)
-
-* Clean up ``bzr selftest --benchmark bundle`` to correct an import,
-  and remove benchmarks that take longer than 10min to run.
-  (John Arbash Meinel)
-
-* Use ``time.time()`` instead of ``time.clock()`` to decide on
-  progress throttling. Because ``time.clock()`` is actually CPU time,
-  so over a high-latency connection, too many updates get throttled.
-  (John Arbash Meinel)
-
-* ``MemoryTransport.list_dir()`` would strip the first character for
-  files or directories in root directory. (John Arbash Meinel)
-
-* New method ``get_branch_reference`` on 'BzrDir' allows the detection of
-  branch references - which the smart server component needs.
-
-* New ``ChrootTransportDecorator``, accessible via the ``chroot+`` url
-  prefix.  It disallows any access to locations above a set URL.  (Andrew
-  Bennetts)
-
-Bug Fixes
-*********
-
-* Now ``_KnitIndex`` properly decode revision ids when loading index data.
-  And optimize the knit index parsing code.
-  (Dmitry Vasiliev, John Arbash Meinel)
-
-* ``bzrlib/bzrdir.py`` was directly referencing ``bzrlib.workingtree``,
-  without importing it. This prevented ``bzr upgrade`` from working
-  unless a plugin already imported ``bzrlib.workingtree``
-  (John Arbash Meinel, #70716)
-
-* Suppress the traceback on invalid URLs (Vincent Ladeuil, #70803).
-
-* Give nicer error message when an http server returns a 403
-  error code. (Vincent Ladeuil, #57644).
-
-* When a multi-range http GET request fails, try a single
-  range one. If it fails too, forget about ranges. Remember that until
-  the death of the transport and propagates that to the clones.
-  (Vincent Ladeuil, #62276, #62029).
-
-* Handles user/passwords supplied in url from command
-  line (for the urllib implementation). Don't request already
-  known passwords (Vincent Ladeuil, #42383, #44647, #48527)
-
-* ``_KnitIndex.add_versions()`` dictionary compresses revision ids as they
-  are added. This fixes bug where fetching remote revisions records
-  them as full references rather than integers.
-  (John Arbash Meinel, #64789)
-
-* ``bzr ignore`` strips trailing slashes in patterns.
-  Also ``bzr ignore`` rejects absolute paths. (Kent Gibson, #4559)
-
-* ``bzr ignore`` takes multiple arguments. (Cheuksan Edward Wang, #29488)
-
-* mv correctly handles paths that traverse symlinks.
-  (Aaron Bentley, #66964)
-
-* Give nicer looking error messages when failing to connect over ssh.
-  (John Arbash Meinel, #49172)
-
-* Pushing to a remote branch does not currently update the remote working
-  tree. After a remote push, ``bzr status`` and ``bzr diff`` on the remote
-  machine now show that the working tree is out of date.
-  (Cheuksan Edward Wang #48136)
-
-* Use patiencediff instead of difflib for determining deltas to insert
-  into knits. This avoids the O(N^3) behavior of difflib. Patience
-  diff should be O(N^2). (Cheuksan Edward Wang, #65714)
-
-* Running ``bzr log`` on nonexistent file gives an error instead of the
-  entire log history. (Cheuksan Edward Wang #50793)
-
-* ``bzr cat`` can look up contents of removed or renamed files. If the
-  pathname is ambiguous, i.e. the files in the old and new trees have
-  different id's, the default is the file in the new tree. The user can
-  use "--name-from-revision" to select the file in the old tree.
-  (Cheuksan Edward Wang, #30190)
-
-Testing
-*******
-
-* TestingHTTPRequestHandler really handles the Range header
-  (previously it was ignoring it and returning the whole file,).
-
-bzr 0.12
-########
-
-:Released:  2006-10-30
-
-Internals
-*********
-
-* Clean up ``bzr selftest --benchmark bundle`` to correct an import,
-  and remove benchmarks that take longer than 10min to run.
-  (John Arbash Meinel)
-
-bzr 0.12rc1
-###########
-
-:Released:  2006-10-23
-
-Improvements
-************
-
-* ``bzr log`` now shows dotted-decimal revision numbers for all revisions,
-  rather than just showing a decimal revision number for revisions on the
-  mainline. These revision numbers are not yet accepted as input into bzr
-  commands such as log, diff etc. (Robert Collins)
-
-* revisions can now be specified using dotted-decimal revision numbers.
-  For instance, ``bzr diff -r 1.2.1..1.2.3``. (Robert Collins)
-
-* ``bzr help commands`` output is now shorter (Aaron Bentley)
-
-* ``bzr`` now uses lazy importing to reduce the startup time. This has
-  a moderate effect on lots of actions, especially ones that have
-  little to do. For example ``bzr rocks`` time is down to 116ms from
-  283ms. (John Arbash Meinel)
-
-* New Registry class to provide name-to-object registry-like support,
-  for example for schemes where plugins can register new classes to
-  do certain tasks (e.g. log formatters). Also provides lazy registration
-  to allow modules to be loaded on request.
-  (John Arbash Meinel, Adeodato Simó)
-
-API Incompatability
-*******************
-
-* LogFormatter subclasses show now expect the 'revno' parameter to
-  show() to be a string rather than an int. (Robert Collins)
-
-Internals
-*********
-
-* ``TestCase.run_bzr``, ``run_bzr_captured``, and ``run_bzr_subprocess``
-  can take a ``working_dir='foo'`` parameter, which will change directory
-  for the command. (John Arbash Meinel)
-
-* ``bzrlib.lazy_regex.lazy_compile`` can be used to create a proxy
-  around a regex, which defers compilation until first use.
-  (John Arbash Meinel)
-
-* ``TestCase.run_bzr_subprocess`` defaults to supplying the
-  ``--no-plugins`` parameter to ensure test reproducability, and avoid
-  problems with system-wide installed plugins. (John Arbash Meinel)
-
-* Unique tree root ids are now supported. Newly created trees still
-  use the common root id for compatibility with bzr versions before 0.12.
-  (Aaron Bentley)
-
-* ``WorkingTree.set_root_id(None)`` is now deprecated. Please
-  pass in ``inventory.ROOT_ID`` if you want the default root id value.
-  (Robert Collins, John Arbash Meinel)
-
-* New method ``WorkingTree.flush()`` which will write the current memory
-  inventory out to disk. At the same time, ``read_working_inventory`` will
-  no longer trash the current tree inventory if it has been modified within
-  the current lock, and the tree will now ``flush()`` automatically on
-  ``unlock()``. ``WorkingTree.set_root_id()`` has been updated to take
-  advantage of this functionality. (Robert Collins, John Arbash Meinel)
-
-* ``bzrlib.tsort.merge_sorted`` now accepts ``generate_revnos``. This
-  parameter will cause it to add another column to its output, which
-  contains the dotted-decimal revno for each revision, as a tuple.
-  (Robert Collins)
-
-* ``LogFormatter.show_merge`` is deprecated in favour of
-  ``LogFormatter.show_merge_revno``. (Robert Collins)
-
-Bug Fixes
-*********
-
-* Avoid circular imports by creating a deprecated function for
-  ``bzrlib.tree.RevisionTree``. Callers should have been using
-  ``bzrlib.revisontree.RevisionTree`` anyway. (John Arbash Meinel,
-  #66349)
-
-* Don't use ``socket.MSG_WAITALL`` as it doesn't exist on all
-  platforms. (Martin Pool, #66356)
-
-* Don't require ``Content-Type`` in range responses. Assume they are a
-  single range if ``Content-Type`` does not exist.
-  (John Arbash Meinel, #62473)
-
-* bzr branch/pull no longer complain about progress bar cleanup when
-  interrupted during fetch.  (Aaron Bentley, #54000)
-
-* ``WorkingTree.set_parent_trees()`` uses the trees to directly write
-  the basis inventory, rather than going through the repository. This
-  allows us to have 1 inventory read, and 2 inventory writes when
-  committing a new tree. (John Arbash Meinel)
-
-* When reverting, files that are not locally modified that do not exist
-  in the target are deleted, not just unversioned (Aaron Bentley)
-
-* When trying to acquire a lock, don't fail immediately. Instead, try
-  a few times (up to 1 hour) before timing out. Also, report why the
-  lock is unavailable (John Arbash Meinel, #43521, #49556)
-
-* Leave HttpTransportBase daughter classes decides how they
-  implement cloning. (Vincent Ladeuil, #61606)
-
-* diff3 does not indicate conflicts on clean merge. (Aaron Bentley)
-
-* If a commit fails, the commit message is stored in a file at the root of
-  the tree for later commit. (Cheuksan Edward Wang, Stefan Metzmacher,
-  #32054)
-
-Testing
-*******
-
-* New test base class TestCaseWithMemoryTransport offers memory-only
-  testing facilities: its not suitable for tests that need to mutate disk
-  state, but most tests should not need that and should be converted to
-  TestCaseWithMemoryTransport. (Robert Collins)
-
-* ``TestCase.make_branch_and_memory_tree`` now takes a format
-  option to set the BzrDir, Repository and Branch formats of the
-  created objects. (Robert Collins, John Arbash Meinel)
-
-bzr 0.11
-########
-
-:Released:  2006-10-02
-
-* Smart server transport test failures on windows fixed. (Lukáš Lalinský).
-
-bzr 0.11rc2
-###########
-
-:Released:  2006-09-27
-
-Bug Fixes
-*********
-
-* Test suite hangs on windows fixed. (Andrew Bennets, Alexander Belchenko).
-
-* Commit performance regression fixed. (Aaron Bentley, Robert Collins, John
-  Arbash Meinel).
-
-bzr 0.11rc1
-###########
-
-:Released:  2006-09-25
-
-Improvements
-************
-
-* Knit files now wait to create their contents until the first data is
-  added. The old code used to create an empty .knit and a .kndx with just
-  the header. However, this caused a lot of extra round trips over sftp.
-  This can change the time for ``bzr push`` to create a new remote branch
-  from 160s down to 100s. This also affects ``bzr commit`` performance when
-  adding new files, ``bzr commit`` on a new kernel-like tree drops from 50s
-  down to 40s (John Arbash Meinel, #44692)
-
-* When an entire subtree has been deleted, commit will now report that
-  just the top of the subtree has been deleted, rather than reporting
-  all the individual items. (Robert Collins)
-
-* Commit performs one less XML parse. (Robert Collins)
-
-* ``bzr checkout`` now operates on readonly branches as well
-  as readwrite branches. This fixes bug #39542. (Robert Collins)
-
-* ``bzr bind`` no longer synchronises history with the master branch.
-  Binding should be followed by an update or push to synchronise the
-  two branches. This is closely related to the fix for bug #39542.
-  (Robert Collins)
-
-* ``bzrlib.lazy_import.lazy_import`` function to create on-demand
-  objects.  This allows all imports to stay at the global scope, but
-  modules will not actually be imported if they are not used.
-  (John Arbash Meinel)
-
-* Support ``bzr://`` and ``bzr+ssh://`` urls to work with the new RPC-based
-  transport which will be used with the upcoming high-performance smart
-  server. The new command ``bzr serve`` will invoke bzr in server mode,
-  which processes these requests. (Andrew Bennetts, Robert Collins, Martin
-  Pool)
-
-* New command ``bzr version-info`` which can be used to get a summary
-  of the current state of the tree. This is especially useful as part
-  of a build commands. See ``doc/version_info.txt`` for more information
-  (John Arbash Meinel)
-
-Bug Fixes
-*********
-
-* ``'bzr inventory [FILE...]'`` allows restricting the file list to a
-  specific set of files. (John Arbash Meinel, #3631)
-
-* Don't abort when annotating empty files (John Arbash Meinel, #56814)
-
-* Add ``Stanza.to_unicode()`` which can be passed to another Stanza
-  when nesting stanzas. Also, add ``read_stanza_unicode`` to handle when
-  reading a nested Stanza. (John Arbash Meinel)
-
-* Transform._set_mode() needs to stat the right file.
-  (John Arbash Meinel, #56549)
-
-* Raise WeaveFormatError rather than StopIteration when trying to read
-  an empty Weave file. (John Arbash Meinel, #46871)
-
-* Don't access e.code for generic URLErrors, only HTTPErrors have .code.
-  (Vincent Ladeuil, #59835)
-
-* Handle boundary="" lines properly to allow access through a Squid proxy.
-  (John Arbash Meinel, #57723)
-
-* revert now removes newly-added directories (Aaron Bentley, #54172)
-
-* ``bzr upgrade sftp://`` shouldn't fail to upgrade v6 branches if there
-  isn't a working tree. (David Allouche, #40679)
-
-* Give nicer error messages when a user supplies an invalid --revision
-  parameter. (John Arbash Meinel, #55420)
-
-* Handle when LANG is not recognized by python. Emit a warning, but
-  just revert to using 'ascii'. (John Arbash Meinel, #35392)
-
-* Don't use ``preexec_fn`` on win32, as it is not supported by subprocess.
-  (John Arbash Meinel)
-
-* Skip specific tests when the dependencies aren't met. This includes
-  some ``setup.py`` tests when ``python-dev`` is not available, and
-  some tests that depend on paramiko. (John Arbash Meinel, Mattheiu Moy)
-
-* Fallback to Paramiko properly, if no ``ssh`` executable exists on
-  the system. (Andrew Bennetts, John Arbash Meinel)
-
-* ``Branch.bind(other_branch)`` no longer takes a write lock on the
-  other branch, and will not push or pull between the two branches.
-  API users will need to perform a push or pull or update operation if they
-  require branch synchronisation to take place. (Robert Collins, #47344)
-
-* When creating a tarball or zipfile export, export unicode names as utf-8
-  paths. This may not work perfectly on all platforms, but has the best
-  chance of working in the common case. (John Arbash Meinel, #56816)
-
-* When committing, only files that exist in working tree or basis tree
-  may be specified (Aaron Bentley, #50793)
-
-Portability
-***********
-
-* Fixes to run on Python 2.5 (Brian M. Carlson, Martin Pool, Marien Zwart)
-
-Internals
-*********
-
-* TestCaseInTempDir now creates a separate directory for HOME, rather
-  than having HOME set to the same location as the working directory.
-  (John Arbash Meinel)
-
-* ``run_bzr_subprocess()`` can take an optional ``env_changes={}`` parameter,
-  which will update os.environ inside the spawned child. It also can
-  take a ``universal_newlines=True``, which helps when checking the output
-  of the command. (John Arbash Meinel)
-
-* Refactor SFTP vendors to allow easier re-use when ssh is used.
-  (Andrew Bennetts)
-
-* ``Transport.list_dir()`` and ``Transport.iter_files_recursive()`` should always
-  return urlescaped paths. This is now tested (there were bugs in a few
-  of the transports) (Andrew Bennetts, David Allouche, John Arbash Meinel)
-
-* New utility function ``symbol_versioning.deprecation_string``. Returns the
-  formatted string for a callable, deprecation format pair. (Robert Collins)
-
-* New TestCase helper applyDeprecated. This allows you to call a callable
-  which is deprecated without it spewing to the screen, just by supplying
-  the deprecation format string issued for it. (Robert Collins)
-
-* Transport.append and Transport.put have been deprecated in favor of
-  ``.append_bytes``, ``.append_file``, ``.put_bytes``, and
-  ``.put_file``. This removes the ambiguity in what type of object the
-  functions take.  ``Transport.non_atomic_put_{bytes,file}`` has also
-  been added. Which works similarly to ``Transport.append()`` except for
-  SFTP, it doesn't have a round trip when opening the file. Also, it
-  provides functionality for creating a parent directory when trying
-  to create a file, rather than raise NoSuchFile and forcing the
-  caller to repeat their request.
-  (John Arbash Meinel)
-
-* WorkingTree has a new api ``unversion`` which allow the unversioning of
-  entries by their file id. (Robert Collins)
-
-* ``WorkingTree.pending_merges`` is deprecated.  Please use the
-  ``get_parent_ids`` (introduced in 0.10) method instead. (Robert Collins)
-
-* WorkingTree has a new ``lock_tree_write`` method which locks the branch for
-  read rather than write. This is appropriate for actions which only need
-  the branch data for reference rather than mutation. A new decorator
-  ``needs_tree_write_lock`` is provided in the workingtree module. Like the
-  ``needs_read_lock`` and ``needs_write_lock`` decorators this allows static
-  declaration of the locking requirements of a function to ensure that
-  a lock is taken out for casual scripts. (Robert Collins, #54107)
-
-* All WorkingTree methods which write to the tree, but not to the branch
-  have been converted to use ``needs_tree_write_lock`` rather than
-  ``needs_write_lock``. Also converted is the revert, conflicts and tree
-  transform modules. This provides a modest performance improvement on
-  metadir style trees, due to the reduce lock-acquisition, and a more
-  significant performance improvement on lightweight checkouts from
-  remote branches, where trivial operations used to pay a significant
-  penalty. It also provides the basis for allowing readonly checkouts.
-  (Robert Collins)
-
-* Special case importing the standard library 'copy' module. This shaves
-  off 40ms of startup time, while retaining compatibility. See:
-  ``bzrlib/inspect_for_copy.py`` for more details. (John Arbash Meinel)
-
-* WorkingTree has a new parent class MutableTree which represents the
-  specialisations of Tree which are able to be altered. (Robert Collins)
-
-* New methods mkdir and ``put_file_bytes_non_atomic`` on MutableTree that
-  mutate the tree and its contents. (Robert Collins)
-
-* Transport behaviour at the root of the URL is now defined and tested.
-  (Andrew Bennetts, Robert Collins)
-
-Testing
-*******
-
-* New test helper classs MemoryTree. This is typically accessed via
-  ``self.make_branch_and_memory_tree()`` in test cases. (Robert Collins)
-
-* Add ``start_bzr_subprocess`` and ``stop_bzr_subprocess`` to allow test
-  code to continue running concurrently with a subprocess of bzr.
-  (Andrew Bennetts, Robert Collins)
-
-* Add a new method ``Transport.get_smart_client()``. This is provided to
-  allow upgrades to a richer interface than the VFS one provided by
-  Transport. (Andrew Bennetts, Martin Pool)
-
-bzr 0.10
-########
-
-:Released:  2006-08-29
-
-Improvements
-************
-* 'merge' now takes --uncommitted, to apply uncommitted changes from a
-  tree.  (Aaron Bentley)
-
-* 'bzr add --file-ids-from' can be used to specify another path to use
-  for creating file ids, rather than generating all new ones. Internally,
-  the 'action' passed to ``smart_add_tree()`` can return ``file_ids`` that
-  will be used, rather than having bzrlib generate new ones.
-  (John Arbash Meinel, #55781)
-
-* ``bzr selftest --benchmark`` now allows a ``--cache-dir`` parameter.
-  This will cache some of the intermediate trees, and decrease the
-  setup time for benchmark tests. (John Arbash Meinel)
-
-* Inverse forms are provided for all boolean options.  For example,
-  --strict has --no-strict, --no-recurse has --recurse (Aaron Bentley)
-
-* Serialize out Inventories directly, rather than using ElementTree.
-  Writing out a kernel sized inventory drops from 2s down to ~350ms.
-  (Robert Collins, John Arbash Meinel)
-
-Bug Fixes
-*********
-
-* Help diffutils 2.8.4 get along with binary tests (Marien Zwart: #57614)
-
-* Change LockDir so that if the lock directory doesn't exist when
-  ``lock_write()`` is called, an attempt will be made to create it.
-  (John Arbash Meinel, #56974)
-
-* ``bzr uncommit`` preserves pending merges. (John Arbash Meinel, #57660)
-
-* Active FTP transport now works as intended. (ghozzy, #56472)
-
-* Really fix mutter() so that it won't ever raise a UnicodeError.
-  It means it is possible for ~/.bzr.log to contain non UTF-8 characters.
-  But it is a debugging log, not a real user file.
-  (John Arbash Meinel, #56947, #53880)
-
-* Change Command handle to allow Unicode command and options.
-  At present we cannot register Unicode command names, so we will get
-  BzrCommandError('unknown command'), or BzrCommandError('unknown option')
-  But that is better than a UnicodeError + a traceback.
-  (John Arbash Meinel, #57123)
-
-* Handle TZ=UTC properly when reading/writing revisions.
-  (John Arbash Meinel, #55783, #56290)
-
-* Use ``GPG_TTY`` to allow gpg --cl to work with gpg-agent in a pipeline,
-  (passing text to sign in on stdin). (John Arbash Meinel, #54468)
-
-* External diff does the right thing for binaries even in foreign
-  languages. (John Arbash Meinel, #56307)
-
-* Testament handles more cases when content is unicode. Specific bug was
-  in handling of revision properties.
-  (John Arbash Meinel, Holger Krekel, #54723)
-
-* The bzr selftest was failing on installed versions due to a bug in a new
-  test helper. (John Arbash Meinel, Robert Collins, #58057)
-
-Internals
-*********
-
-* ``bzrlib.cache_utf8`` contains ``encode()`` and ``decode()`` functions
-  which can be used to cache the conversion between utf8 and Unicode.
-  Especially helpful for some of the knit annotation code, which has to
-  convert revision ids to utf8 to annotate lines in storage.
-  (John Arbash Meinel)
-
-* ``setup.py`` now searches the filesystem to find all packages which
-  need to be installed. This should help make the life of packagers
-  easier. (John Arbash Meinel)
-
-bzr 0.9.0
-#########
-
-:Released:  2006-08-11
-
-Surprises
-*********
-
-* The hard-coded built-in ignore rules have been removed. There are
-  now two rulesets which are enforced. A user global one in
-  ``~/.bazaar/ignore`` which will apply to every tree, and the tree
-  specific one '.bzrignore'.
-  ``~/.bazaar/ignore`` will be created if it does not exist, but with
-  a more conservative list than the old default.
-  This fixes bugs with default rules being enforced no matter what.
-  The old list of ignore rules from bzr is available by
-  running 'bzr ignore --old-default-rules'.
-  (Robert Collins, Martin Pool, John Arbash Meinel)
-
-* 'branches.conf' has been changed to 'locations.conf', since it can apply
-  to more locations than just branch locations.
-  (Aaron Bentley)
-
-Improvements
-************
-
-* The revision specifier "revno:" is extended to accept the syntax
-  revno:N:branch. For example,
-  revno:42:http://bazaar-vcs.org/bzr/bzr.dev/ means revision 42 in
-  bzr.dev.  (Matthieu Moy)
-
-* Tests updates to ensure proper URL handling, UNICODE support, and
-  proper printing when the user's terminal encoding cannot display
-  the path of a file that has been versioned.
-  ``bzr branch`` can take a target URL rather than only a local directory.
-  ``Branch.get_parent()/set_parent()`` now save a relative path if possible,
-  and normalize the parent based on root, allowing access across
-  different transports. (John Arbash Meinel, Wouter van Heyst, Martin Pool)
-  (Malone #48906, #42699, #40675, #5281, #3980, #36363, #43689,
-  #42517, #42514)
-
-* On Unix, detect terminal width using an ioctl not just $COLUMNS.
-  Use terminal width for single-line logs from ``bzr log --line`` and
-  pending-merge display.  (Robert Widhopf-Fenk, Gustavo Niemeyer)
-  (Malone #3507)
-
-* On Windows, detect terminal width using GetConsoleScreenBufferInfo.
-  (Alexander Belchenko)
-
-* Speedup improvement for 'date:'-revision search. (Guillaume Pinot).
-
-* Show the correct number of revisions pushed when pushing a new branch.
-  (Robert Collins).
-
-* 'bzr selftest' now shows a progress bar with the number of tests, and
-  progress made. 'make check' shows tests in -v mode, to be more useful
-  for the PQM status window. (Robert Collins).
-  When using a progress bar, failed tests are printed out, rather than
-  being overwritten by the progress bar until the suite finishes.
-  (John Arbash Meinel)
-
-* 'bzr selftest --benchmark' will run a new benchmarking selftest.
-  'bzr selftest --benchmark --lsprof-timed' will use lsprofile to generate
-  profile data for the individual profiled calls, allowing for fine
-  grained analysis of performance.
-  (Robert Collins, Martin Pool).
-
-* 'bzr commit' shows a progress bar. This is useful for commits over sftp
-  where commit can take an appreciable time. (Robert Collins)
-
-* 'bzr add' is now less verbose in telling you what ignore globs were
-  matched by files being ignored. Instead it just tells you how many
-  were ignored (because you might reasonably be expecting none to be
-  ignored). 'bzr add -v' is unchanged and will report every ignored
-  file. (Robert Collins).
-
-* ftp now has a test server if medusa is installed. As part of testing,
-  ftp support has been improved, including support for supplying a
-  non-standard port. (John Arbash Meinel).
-
-* 'bzr log --line' shows the revision number, and uses only the
-  first line of the log message (#5162, Alexander Belchenko;
-  Matthieu Moy)
-
-* 'bzr status' has had the --all option removed. The 'bzr ls' command
-  should be used to retrieve all versioned files. (Robert Collins)
-
-* 'bzr bundle OTHER/BRANCH' will create a bundle which can be sent
-  over email, and applied on the other end, while maintaining ancestry.
-  This bundle can be applied with either 'bzr merge' or 'bzr pull',
-  the same way you would apply another branch.
-  (John Arbash Meinel, Aaron Bentley)
-
-* 'bzr whoami' can now be used to set your identity from the command line,
-  for a branch or globally.  (Robey Pointer)
-
-* 'bzr checkout' now aliased to 'bzr co', and 'bzr annotate' to 'bzr ann'.
-  (Michael Ellerman)
-
-* 'bzr revert DIRECTORY' now reverts the contents of the directory as well.
-  (Aaron Bentley)
-
-* 'bzr get sftp://foo' gives a better error when paramiko is not present.
-  Also updates things like 'http+pycurl://' if pycurl is not present.
-  (John Arbash Meinel) (Malone #47821, #52204)
-
-* New env variable ``BZR_PROGRESS_BAR``, sets the default progress bar type.
-  Can be set to 'none' or 'dummy' to disable the progress bar, 'dots' or
-  'tty' to create the respective type. (John Arbash Meinel, #42197, #51107)
-
-* Improve the help text for 'bzr diff' to explain what various options do.
-  (John Arbash Meinel, #6391)
-
-* 'bzr uncommit -r 10' now uncommits revisions 11.. rather than uncommitting
-  revision 10. This makes -r10 more in line with what other commands do.
-  'bzr uncommit' also now saves the pending merges of the revisions that
-  were removed. So it is safe to uncommit after a merge, fix something,
-  and commit again. (John Arbash Meinel, #32526, #31426)
-
-* 'bzr init' now also works on remote locations.
-  (Wouter van Heyst, #48904)
-
-* HTTP support has been updated. When using pycurl we now support
-  connection keep-alive, which reduces dns requests and round trips.
-  And for both urllib and pycurl we support multi-range requests,
-  which decreases the number of round-trips. Performance results for
-  ``bzr branch http://bazaar-vcs.org/bzr/bzr.dev/`` indicate
-  http branching is now 2-3x faster, and ``bzr pull`` in an existing
-  branch is as much as 4x faster.
-  (Michael Ellerman, Johan Rydberg, John Arbash Meinel, #46768)
-
-* Performance improvements for sftp. Branching and pulling are now up to
-  2x faster. Utilize paramiko.readv() support for async requests if it
-  is available (paramiko > 1.6) (John Arbash Meinel)
-
-Bug Fixes
-*********
-
-* Fix shadowed definition of TestLocationConfig that caused some
-  tests not to run.
-  (Erik Bågfors, Michael Ellerman, Martin Pool, #32587)
-
-* Fix unnecessary requirement of sign-my-commits that it be run from
-  a working directory.  (Martin Pool, Robert Collins)
-
-* 'bzr push location' will only remember the push location if it succeeds
-  in connecting to the remote location. (John Arbash Meinel, #49742)
-
-* 'bzr revert' no longer toggles the executable bit on win32
-  (John Arbash Meinel, #45010)
-
-* Handle broken pipe under win32 correctly. (John Arbash Meinel)
-
-* sftp tests now work correctly on win32 if you have a newer paramiko
-  (John Arbash Meinel)
-
-* Cleanup win32 test suite, and general cleanup of places where
-  file handles were being held open. (John Arbash Meinel)
-
-* When specifying filenames for 'diff -r x..y', the name of the file in the
-  working directory can be used, even if its name is different in both x
-  and y.
-
-* File-ids containing single- or double-quotes are handled correctly by
-  push. (Aaron Bentley, #52227)
-
-* Normalize unicode filenames to ensure cross-platform consistency.
-  (John Arbash Meinel, #43689)
-
-* The argument parser can now handle '-' as an argument. Currently
-  no code interprets it specially (it is mostly handled as a file named
-  '-'). But plugins, and future operations can use it.
-  (John Arbash meinel, #50984)
-
-* Bundles can properly read binary files with a plain '\r' in them.
-  (John Arbash Meinel, #51927)
-
-* Tuning ``iter_entries()`` to be more efficient (John Arbash Meinel, #5444)
-
-* Lots of win32 fixes (the test suite passes again).
-  (John Arbash Meinel, #50155)
-
-* Handle openbsd returning None for sys.getfilesystemencoding() (#41183)
-
-* Support ftp APPE (append) to allow Knits to be used over ftp (#42592)
-
-* Removals are only committed if they match the filespec (or if there is
-  no filespec).  (#46635, Aaron Bentley)
-
-* smart-add recurses through all supplied directories
-  (John Arbash Meinel, #52578)
-
-* Make the bundle reader extra lines before and after the bundle text.
-  This allows you to parse an email with the bundle inline.
-  (John Arbash Meinel, #49182)
-
-* Change the file id generator to squash a little bit more. Helps when
-  working with long filenames on windows. (Also helps for unicode filenames
-  not generating hidden files). (John Arbash Meinel, #43801)
-
-* Restore terminal mode on C-c while reading sftp password.  (#48923,
-  Nicholas Allen, Martin Pool)
-
-* Timestamps are rounded to 1ms, and revision entries can be recreated
-  exactly. (John Arbash Meinel, Jamie Wilkinson, #40693)
-
-* Branch.base has changed to a URL, but ~/.bazaar/locations.conf should
-  use local paths, since it is user visible (John Arbash Meinel, #53653)
-
-* ``bzr status foo`` when foo was unversioned used to cause a full delta
-  to be generated (John Arbash Meinel, #53638)
-
-* When reading revision properties, an empty value should be considered
-  the empty string, not None (John Arbash Meinel, #47782)
-
-* ``bzr diff --diff-options`` can now handle binary files being changed.
-  Also, the output is consistent when --diff-options is not supplied.
-  (John Arbash Meinel, #54651, #52930)
-
-* Use the right suffixes for loading plugins (John Arbash Meinel, #51810)
-
-* Fix ``Branch.get_parent()`` to handle the case when the parent is not
-  accessible (John Arbash Meinel, #52976)
-
-Internals
-*********
-
-* Combine the ignore rules into a single regex rather than looping over
-  them to reduce the threshold where  N^2 behaviour occurs in operations
-  like status. (Jan Hudec, Robert Collins).
-
-* Appending to ``bzrlib.DEFAULT_IGNORE`` is now deprecated. Instead, use
-  one of the add functions in bzrlib.ignores. (John Arbash Meinel)
-
-* 'bzr push' should only push the ancestry of the current revision, not
-  all of the history in the repository. This is especially important for
-  shared repositories. (John Arbash Meinel)
-
-* ``bzrlib.delta.compare_trees`` now iterates in alphabetically sorted order,
-  rather than randomly walking the inventories. (John Arbash Meinel)
-
-* Doctests are now run in temporary directories which are cleaned up when
-  they finish, rather than using special ScratchDir/ScratchBranch objects.
-  (Martin Pool)
-
-* Split ``check`` into separate methods on the branch and on the repository,
-  so that it can be specialized in ways that are useful or efficient for
-  different formats.  (Martin Pool, Robert Collins)
-
-* Deprecate ``Repository.all_revision_ids``; most methods don't really need
-  the global revision graph but only that part leading up to a particular
-  revision.  (Martin Pool, Robert Collins)
-
-* Add a BzrDirFormat ``control_formats`` list which allows for control formats
-  that do not use '.bzr' to store their data - i.e. '.svn', '.hg' etc.
-  (Robert Collins, Jelmer Vernooij).
-
-* ``bzrlib.diff.external_diff`` can be redirected to any file-like object.
-  Uses subprocess instead of spawnvp.
-  (James Henstridge, John Arbash Meinel, #4047, #48914)
-
-* New command line option '--profile-imports', which will install a custom
-  importer to log time to import modules and regex compilation time to
-  sys.stderr (John Arbash Meinel)
-
-* 'EmptyTree' is now deprecated, please use ``repository.revision_tree(None)``
-  instead. (Robert Collins)
-
-* "RevisionTree" is now in bzrlib/revisiontree.py. (Robert Collins)
-
-bzr 0.8.2
-#########
-
-:Released:  2006-05-17
-
-Bug Fixes
-*********
-
-* setup.py failed to install launchpad plugin.  (Martin Pool)
-
-bzr 0.8.1
-#########
-
-:Released:  2006-05-16
-
-Bug Fixes
-*********
-
-* Fix failure to commit a merge in a checkout.  (Martin Pool,
-  Robert Collins, Erik Bågfors, #43959)
-
-* Nicer messages from 'commit' in the case of renames, and correct
-  messages when a merge has occured. (Robert Collins, Martin Pool)
-
-* Separate functionality from assert statements as they are skipped in
-  optimized mode of python. Add the same check to pending merges.
-  (Olaf Conradi, #44443)
-
-Changes
-*******
-
-* Do not show the None revision in output of bzr ancestry. (Olaf Conradi)
-
-* Add info on standalone branches without a working tree.
-  (Olaf Conradi, #44155)
-
-* Fix bug in knits when raising InvalidRevisionId. (Olaf Conradi, #44284)
-
-Changes
-*******
-
-* Make editor invocation comply with Debian Policy. First check
-  environment variables VISUAL and EDITOR, then try editor from
-  alternatives system. If that all fails, fall back to the pre-defined
-  list of editors. (Olaf Conradi, #42904)
-
-New Features
-************
-
-* New 'register-branch' command registers a public branch into
-  Launchpad.net, where it can be associated with bugs, etc.
-  (Martin Pool, Bjorn Tillenius, Robert Collins)
-
-Internals
-*********
-
-* New public api in InventoryEntry - ``describe_change(old, new)`` which
-  provides a human description of the changes between two old and
-  new. (Robert Collins, Martin Pool)
-
-Testing
-*******
-
-* Fix test case for bzr info in upgrading a standalone branch to metadir,
-  uses bzrlib api now. (Olaf Conradi)
-
-bzr 0.8
-#######
-
-:Released:  2006-05-08
-
-Notes When Upgrading
-********************
-
-Release 0.8 of bzr introduces a new format for history storage, called
-'knit', as an evolution of to the 'weave' format used in 0.7.  Local
-and remote operations are faster using knits than weaves.  Several
-operations including 'init', 'init-repo', and 'upgrade' take a
---format option that controls this.  Branching from an existing branch
-will keep the same format.
-
-It is possible to merge, pull and push between branches of different
-formats but this is slower than moving data between homogenous
-branches.  It is therefore recommended (but not required) that you
-upgrade all branches for a project at the same time.  Information on
-formats is shown by 'bzr info'.
-
-bzr 0.8 now allows creation of 'repositories', which hold the history
-of files and revisions for several branches.  Previously bzr kept all
-the history for a branch within the .bzr directory at the root of the
-branch, and this is still the default.  To create a repository, use
-the new 'bzr init-repo' command.  Branches exist as directories under
-the repository and contain just a small amount of information
-indicating the current revision of the branch.
-
-bzr 0.8 also supports 'checkouts', which are similar to in cvs and
-subversion.  Checkouts are associated with a branch (optionally in a
-repository), which contains all the historical information.  The
-result is that a checkout can be deleted without losing any
-already-committed revisions.  A new 'update' command is also available.
-
-Repositories and checkouts are not supported with the 0.7 storage
-format.  To use them you must upgrad to either knits, or to the
-'metaweave' format, which uses weaves but changes the .bzr directory
-arrangement.
-
-
-Improvements
-************
-
-* sftp paths can now be relative, or local, according to the lftp
-  convention. Paths now take the form::
-
-      sftp://user:pass@host:port/~/relative/path
-      or
-      sftp://user:pass@host:port/absolute/path
-
-* The FTP transport now tries to reconnect after a temporary
-  failure. ftp put is made atomic. (Matthieu Moy)
-
-* The FTP transport now maintains a pool of connections, and
-  reuses them to avoid multiple connections to the same host (like
-  sftp did). (Daniel Silverstone)
-
-* The ``bzr_man.py`` file has been removed. To create the man page now,
-  use ``./generate_docs.py man``. The new program can also create other files.
-  Run ``python generate_docs.py --help`` for usage information.
-  (Hans Ulrich Niedermann & James Blackwell).
-
-* Man Page now gives full help (James Blackwell).
-  Help also updated to reflect user config now being stored in .bazaar
-  (Hans Ulrich Niedermann)
-
-* It's now possible to set aliases in bazaar.conf (Erik Bågfors)
-
-* Pull now accepts a --revision argument (Erik Bågfors)
-
-* ``bzr re-sign`` now allows multiple revisions to be supplied on the command
-  line. You can now use the following command to sign all of your old
-  commits::
-
-    find .bzr/revision-store// -name my@email-* \
-      | sed 's/.*\/\/..\///' \
-      | xargs bzr re-sign
-
-* Upgrade can now upgrade over the network. (Robert Collins)
-
-* Two new commands 'bzr checkout' and 'bzr update' allow for CVS/SVN-alike
-  behaviour.  By default they will cache history in the checkout, but
-  with --lightweight almost all data is kept in the master branch.
-  (Robert Collins)
-
-* 'revert' unversions newly-versioned files, instead of deleting them.
-
-* 'merge' is more robust.  Conflict messages have changed.
-
-* 'merge' and 'revert' no longer clobber existing files that end in '~' or
-  '.moved'.
-
-* Default log format can be set in configuration and plugins can register
-  their own formatters. (Erik Bågfors)
-
-* New 'reconcile' command will check branch consistency and repair indexes
-  that can become out of sync in pre 0.8 formats. (Robert Collins,
-  Daniel Silverstone)
-
-* New 'bzr init --format' and 'bzr upgrade --format' option to control
-  what storage format is created or produced.  (Robert Collins,
-  Martin Pool)
-
-* Add parent location to 'bzr info', if there is one.  (Olaf Conradi)
-
-* New developer commands 'weave-list' and 'weave-join'.  (Martin Pool)
-
-* New 'init-repository' command, plus support for repositories in 'init'
-  and 'branch' (Aaron Bentley, Erik Bågfors, Robert Collins)
-
-* Improve output of 'info' command. Show all relevant locations related to
-  working tree, branch and repository. Use kibibytes for binary quantities.
-  Fix off-by-one error in missing revisions of working tree.  Make 'info'
-  work on branches, repositories and remote locations.  Show locations
-  relative to the shared repository, if applicable.  Show locking status
-  of locations.  (Olaf Conradi)
-
-* Diff and merge now safely handle binary files. (Aaron Bentley)
-
-* 'pull' and 'push' now normalise the revision history, so that any two
-  branches with the same tip revision will have the same output from 'log'.
-  (Robert Collins)
-
-* 'merge' accepts --remember option to store parent location, like 'push'
-  and 'pull'. (Olaf Conradi)
-
-* bzr status and diff when files given as arguments do not exist
-  in the relevant trees.  (Martin Pool, #3619)
-
-* Add '.hg' to the default ignore list.  (Martin Pool)
-
-* 'knit' is now the default disk format. This improves disk performance and
-  utilization, increases incremental pull performance, robustness with SFTP
-  and allows checkouts over SFTP to perform acceptably.
-  The initial Knit code was contributed by Johan Rydberg based on a
-  specification by Martin Pool.
-  (Robert Collins, Aaron Bentley, Johan Rydberg, Martin Pool).
-
-* New tool to generate all-in-one html version of the manual.  (Alexander
-  Belchenko)
-
-* Hitting CTRL-C while doing an SFTP push will no longer cause stale locks
-  to be left in the SFTP repository. (Robert Collins, Martin Pool).
-
-* New option 'diff --prefix' to control how files are named in diff
-  output, with shortcuts '-p0' and '-p1' corresponding to the options for
-  GNU patch.  (Alexander Belchenko, Goffredo Baroncelli, Martin Pool)
-
-* Add --revision option to 'annotate' command.  (Olaf Conradi)
-
-* If bzr shows an unexpected revision-history after pulling (perhaps due
-  to a reweave) it can now be corrected by 'bzr reconcile'.
-  (Robert Collins)
-
-Changes
-*******
-
-* Commit is now verbose by default, and shows changed filenames and the
-  new revision number.  (Robert Collins, Martin Pool)
-
-* Unify 'mv', 'move', 'rename'.  (Matthew Fuller, #5379)
-
-* 'bzr -h' shows help.  (Martin Pool, Ian Bicking, #35940)
-
-* Make 'pull' and 'push' remember location on failure using --remember.
-  (Olaf Conradi)
-
-* For compatibility, make old format for using weaves inside metadir
-  available as 'metaweave' format.  Rename format 'metadir' to 'default'.
-  Clean up help for option --format in commands 'init', 'init-repo' and
-  'upgrade'.  (Olaf Conradi)
-
-Internals
-*********
-
-* The internal storage of history, and logical branch identity have now
-  been split into Branch, and Repository. The common locking and file
-  management routines are now in bzrlib.lockablefiles.
-  (Aaron Bentley, Robert Collins, Martin Pool)
-
-* Transports can now raise DependencyNotPresent if they need a library
-  which is not installed, and then another implementation will be
-  tried.  (Martin Pool)
-
-* Remove obsolete (and no-op) `decode` parameter to `Transport.get`.
-  (Martin Pool)
-
-* Using Tree Transform for merge, revert, tree-building
-
-* WorkingTree.create, Branch.create, ``WorkingTree.create_standalone``,
-  Branch.initialize are now deprecated. Please see ``BzrDir.create_*`` for
-  replacement API's. (Robert Collins)
-
-* New BzrDir class represents the .bzr control directory and manages
-  formatting issues. (Robert Collins)
-
-* New repository.InterRepository class encapsulates Repository to
-  Repository actions and allows for clean selection of optimised code
-  paths. (Robert Collins)
-
-* ``bzrlib.fetch.fetch`` and ``bzrlib.fetch.greedy_fetch`` are now
-  deprecated, please use ``branch.fetch`` or ``repository.fetch``
-  depending on your needs. (Robert Collins)
-
-* deprecated methods now have a ``is_deprecated`` flag on them that can
-  be checked, if you need to determine whether a given callable is
-  deprecated at runtime. (Robert Collins)
-
-* Progress bars are now nested - see
-  ``bzrlib.ui.ui_factory.nested_progress_bar``.
-  (Robert Collins, Robey Pointer)
-
-* New API call ``get_format_description()`` for each type of format.
-  (Olaf Conradi)
-
-* Changed ``branch.set_parent()`` to accept None to remove parent.
-  (Olaf Conradi)
-
-* Deprecated BzrError AmbiguousBase.  (Olaf Conradi)
-
-* WorkingTree.branch is now a read only property.  (Robert Collins)
-
-* bzrlib.ui.text.TextUIFactory now accepts a ``bar_type`` parameter which
-  can be None or a factory that will create a progress bar. This is
-  useful for testing or for overriding the bzrlib.progress heuristic.
-  (Robert Collins)
-
-* New API method ``get_physical_lock_status()`` to query locks present on a
-  transport.  (Olaf Conradi)
-
-* Repository.reconcile now takes a thorough keyword parameter to allow
-  requesting an indepth reconciliation, rather than just a data-loss
-  check. (Robert Collins)
-
-* ``bzrlib.ui.ui_factory protocol`` now supports ``get_boolean`` to prompt
-  the user for yes/no style input. (Robert Collins)
-
-Testing
-*******
-
-* SFTP tests now shortcut the SSH negotiation, reducing test overhead
-  for testing SFTP protocol support. (Robey Pointer)
-
-* Branch formats are now tested once per implementation (see ``bzrlib.
-  tests.branch_implementations``. This is analagous to the transport
-  interface tests, and has been followed up with working tree,
-  repository and BzrDir tests. (Robert Collins)
-
-* New test base class TestCaseWithTransport provides a transport aware
-  test environment, useful for testing any transport-interface using
-  code. The test suite option --transport controls the transport used
-  by this class (when its not being used as part of implementation
-  contract testing). (Robert Collins)
-
-* Close logging handler on disabling the test log. This will remove the
-  handler from the internal list inside python's logging module,
-  preventing shutdown from closing it twice.  (Olaf Conradi)
-
-* Move test case for uncommit to blackbox tests.  (Olaf Conradi)
-
-* ``run_bzr`` and ``run_bzr_captured`` now accept a 'stdin="foo"'
-  parameter which will provide String("foo") to the command as its stdin.
-
-bzr 0.7
-#######
-
-:Released: 2006-01-09
-
-Changes
-*******
-
-* .bzrignore is excluded from exports, on the grounds that it's a bzr
-  internal-use file and may not be wanted.  (Jamie Wilkinson)
-
-* The "bzr directories" command were removed in favor of the new
-  --kind option to the "bzr inventory" command.  To list all
-  versioned directories, now use "bzr inventory --kind directory".
-  (Johan Rydberg)
-
-* Under Windows configuration directory is now ``%APPDATA%\bazaar\2.0``
-  by default. (John Arbash Meinel)
-
-* The parent of Bzr configuration directory can be set by ``BZR_HOME``
-  environment variable. Now the path for it is searched in ``BZR_HOME``,
-  then in HOME. Under Windows the order is: ``BZR_HOME``, ``APPDATA``
-  (usually points to ``C:\Documents and Settings\User Name\Application Data``),
-  ``HOME``. (John Arbash Meinel)
-
-* Plugins with the same name in different directories in the bzr plugin
-  path are no longer loaded: only the first successfully loaded one is
-  used. (Robert Collins)
-
-* Use systems' external ssh command to open connections if possible.
-  This gives better integration with user settings such as ProxyCommand.
-  (James Henstridge)
-
-* Permissions on files underneath .bzr/ are inherited from the .bzr
-  directory. So for a shared repository, simply doing 'chmod -R g+w .bzr/'
-  will mean that future file will be created with group write permissions.
-
-* configure.in and config.guess are no longer in the builtin default
-  ignore list.
-
-* '.sw[nop]' pattern ignored, to ignore vim swap files for nameless
-  files.  (John Arbash Meinel, Martin Pool)
-
-Improvements
-************
-
-* "bzr INIT dir" now initializes the specified directory, and creates
-  it if it does not exist.  (John Arbash Meinel)
-
-* New remerge command (Aaron Bentley)
-
-* Better zsh completion script.  (Steve Borho)
-
-* 'bzr diff' now returns 1 when there are changes in the working
-  tree. (Robert Collins)
-
-* 'bzr push' now exists and can push changes to a remote location.
-  This uses the transport infrastructure, and can store the remote
-  location in the ~/.bazaar/branches.conf configuration file.
-  (Robert Collins)
-
-* Test directories are only kept if the test fails and the user requests
-  that they be kept.
-
-* Tweaks to short log printing
-
-* Added branch nicks, new nick command, printing them in log output.
-  (Aaron Bentley)
-
-* If ``$BZR_PDB`` is set, pop into the debugger when an uncaught exception
-  occurs.  (Martin Pool)
-
-* Accept 'bzr resolved' (an alias for 'bzr resolve'), as this is
-  the same as Subversion.  (Martin Pool)
-
-* New ftp transport support (on ftplib), for ftp:// and aftp://
-  URLs.  (Daniel Silverstone)
-
-* Commit editor temporary files now start with ``bzr_log.``, to allow
-  text editors to match the file name and set up appropriate modes or
-  settings.  (Magnus Therning)
-
-* Improved performance when integrating changes from a remote weave.
-  (Goffredo Baroncelli)
-
-* Sftp will attempt to cache the connection, so it is more likely that
-  a connection will be reused, rather than requiring multiple password
-  requests.
-
-* bzr revno now takes an optional argument indicating the branch whose
-  revno should be printed.  (Michael Ellerman)
-
-* bzr cat defaults to printing the last version of the file.
-  (Matthieu Moy, #3632)
-
-* New global option 'bzr --lsprof COMMAND' runs bzr under the lsprof
-  profiler.  (Denys Duchier)
-
-* Faster commits by reading only the headers of affected weave files.
-  (Denys Duchier)
-
-* 'bzr add' now takes a --dry-run parameter which shows you what would be
-  added, but doesn't actually add anything. (Michael Ellerman)
-
-* 'bzr add' now lists how many files were ignored per glob.  add --verbose
-  lists the specific files.  (Aaron Bentley)
-
-* 'bzr missing' now supports displaying changes in diverged trees and can
-  be limited to show what either end of the comparison is missing.
-  (Aaron Bently, with a little prompting from Daniel Silverstone)
-
-Bug Fixes
-*********
-
-* SFTP can walk up to the root path without index errors. (Robert Collins)
-
-* Fix bugs in running bzr with 'python -O'.  (Martin Pool)
-
-* Error when run with -OO
-
-* Fix bug in reporting http errors that don't have an http error code.
-  (Martin Pool)
-
-* Handle more cases of pipe errors in display commands
-
-* Change status to 3 for all errors
-
-* Files that are added and unlinked before committing are completely
-  ignored by diff and status
-
-* Stores with some compressed texts and some uncompressed texts are now
-  able to be used. (John A Meinel)
-
-* Fix for bzr pull failing sometimes under windows
-
-* Fix for sftp transport under windows when using interactive auth
-
-* Show files which are both renamed and modified as such in 'bzr
-  status' output.  (Daniel Silverstone, #4503)
-
-* Make annotate cope better with revisions committed without a valid
-  email address.  (Marien Zwart)
-
-* Fix representation of tab characters in commit messages.
-  (Harald Meland)
-
-* List of plugin directories in ``BZR_PLUGIN_PATH`` environment variable is
-  now parsed properly under Windows. (Alexander Belchenko)
-
-* Show number of revisions pushed/pulled/merged. (Robey Pointer)
-
-* Keep a cached copy of the basis inventory to speed up operations
-  that need to refer to it.  (Johan Rydberg, Martin Pool)
-
-* Fix bugs in bzr status display of non-ascii characters.
-  (Martin Pool)
-
-* Remove Makefile.in from default ignore list.
-  (Tollef Fog Heen, Martin Pool, #6413)
-
-* Fix failure in 'bzr added'.  (Nathan McCallum, Martin Pool)
-
-Testing
-*******
-
-* Fix selftest asking for passwords when there are no SFTP keys.
-  (Robey Pointer, Jelmer Vernooij)
-
-* Fix selftest run with 'python -O'.  (Martin Pool)
-
-* Fix HTTP tests under Windows. (John Arbash Meinel)
-
-* Make tests work even if HOME is not set (Aaron Bentley)
-
-* Updated ``build_tree`` to use fixed line-endings for tests which read
-  the file cotents and compare. Make some tests use this to pass under
-  Windows. (John Arbash Meinel)
-
-* Skip stat and symlink tests under Windows. (Alexander Belchenko)
-
-* Delay in selftest/testhashcash is now issued under win32 and Cygwin.
-  (John Arbash Meinel)
-
-* Use terminal width to align verbose test output.  (Martin Pool)
-
-* Blackbox tests are maintained within the bzrlib.tests.blackbox directory.
-  If adding a new test script please add that to
-  ``bzrlib.tests.blackbox.__init__``. (Robert Collins)
-
-* Much better error message if one of the test suites can't be
-  imported.  (Martin Pool)
-
-* Make check now runs the test suite twice - once with the default locale,
-  and once with all locales forced to C, to expose bugs. This is not
-  trivially done within python, so for now its only triggered by running
-  Make check. Integrators and packagers who wish to check for full
-  platform support should run 'make check' to test the source.
-  (Robert Collins)
-
-* Tests can now run TestSkipped if they can't execute for any reason.
-  (Martin Pool) (NB: TestSkipped should only be raised for correctable
-  reasons - see the wiki spec ImprovingBzrTestSuite).
-
-* Test sftp with relative, absolute-in-homedir and absolute-not-in-homedir
-  paths for the transport tests. Introduce blackbox remote sftp tests that
-  test the same permutations. (Robert Collins, Robey Pointer)
-
-* Transport implementation tests are now independent of the local file
-  system, which allows tests for esoteric transports, and for features
-  not available in the local file system. They also repeat for variations
-  on the URL scheme that can introduce issues in the transport code,
-  see bzrlib.transport.TransportTestProviderAdapter() for this.
-  (Robert Collins).
-
-* ``TestCase.build_tree`` uses the transport interface to build trees,
-  pass in a transport parameter to give it an existing connection.
-  (Robert Collins).
-
-Internals
-*********
-
-* WorkingTree.pull has been split across Branch and WorkingTree,
-  to allow Branch only pulls. (Robert Collins)
-
-* ``commands.display_command`` now returns the result of the decorated
-  function. (Robert Collins)
-
-* LocationConfig now has a ``set_user_option(key, value)`` call to save
-  a setting in its matching location section (a new one is created
-  if needed). (Robert Collins)
-
-* Branch has two new methods, ``get_push_location`` and
-  ``set_push_location`` to respectively, get and set the push location.
-  (Robert Collins)
-
-* ``commands.register_command`` now takes an optional flag to signal that
-  the registrant is planning to decorate an existing command. When
-  given multiple plugins registering a command is not an error, and
-  the original command class (whether built in or a plugin based one) is
-  returned to the caller. There is a new error 'MustUseDecorated' for
-  signalling when a wrapping command should switch to the original
-  version. (Robert Collins)
-
-* Some option parsing errors will raise 'BzrOptionError', allowing
-  granular detection for decorating commands. (Robert Collins).
-
-* ``Branch.read_working_inventory`` has moved to
-  ``WorkingTree.read_working_inventory``. This necessitated changes to
-  ``Branch.get_root_id``, and a move of ``Branch.set_inventory`` to
-  WorkingTree as well. To make it clear that a WorkingTree cannot always
-  be obtained ``Branch.working_tree()`` will raise
-  ``errors.NoWorkingTree`` if one cannot be obtained. (Robert Collins)
-
-* All pending merges operations from Branch are now on WorkingTree.
-  (Robert Collins)
-
-* The follow operations from Branch have moved to WorkingTree::
-
-      add()
-      commit()
-      move()
-      rename_one()
-      unknowns()
-
-  (Robert Collins)
-
-* ``bzrlib.add.smart_add_branch`` is now ``smart_add_tree``. (Robert Collins)
-
-* New "rio" serialization format, similar to rfc-822. (Martin Pool)
-
-* Rename selftests to ``bzrlib.tests.test_foo``.  (John A Meinel, Martin
-  Pool)
-
-* ``bzrlib.plugin.all_plugins`` has been changed from an attribute to a
-  query method. (Robert Collins)
-
-* New options to read only the table-of-contents of a weave.
-  (Denys Duchier)
-
-* Raise NoSuchFile when someone tries to add a non-existant file.
-  (Michael Ellerman)
-
-* Simplify handling of DivergedBranches in ``cmd_pull()``.
-  (Michael Ellerman)
-
-* Branch.controlfile* logic has moved to lockablefiles.LockableFiles, which
-  is exposed as ``Branch().control_files``. Also this has been altered with the
-  controlfile pre/suffix replaced by simple method names like 'get' and
-  'put'. (Aaron Bentley, Robert Collins).
-
-* Deprecated functions and methods can now be marked as such using the
-  ``bzrlib.symbol_versioning`` module. Marked method have their docstring
-  updated and will issue a DeprecationWarning using the warnings module
-  when they are used. (Robert Collins)
-
-* ``bzrlib.osutils.safe_unicode`` now exists to provide parameter coercion
-  for functions that need unicode strings. (Robert Collins)
-
-bzr 0.6
-#######
-
-:Released: 2005-10-28
-
-Improvements
-************
-
-* pull now takes --verbose to show you what revisions are added or removed
-  (John A Meinel)
-
-* merge now takes a --show-base option to include the base text in
-  conflicts.
-  (Aaron Bentley)
-
-* The config files are now read using ConfigObj, so '=' should be used as
-  a separator, not ':'.
-  (Aaron Bentley)
-
-* New 'bzr commit --strict' option refuses to commit if there are
-  any unknown files in the tree.  To commit, make sure all files are
-  either ignored, added, or deleted.  (Michael Ellerman)
-
-* The config directory is now ~/.bazaar, and there is a single file
-  ~/.bazaar/bazaar.conf storing email, editor and other preferences.
-  (Robert Collins)
-
-* 'bzr add' no longer takes a --verbose option, and a --quiet option
-  has been added that suppresses all output.
-
-* Improved zsh completion support in contrib/zsh, from Clint
-  Adams.
-
-* Builtin 'bzr annotate' command, by Martin Pool with improvements from
-  Goffredo Baroncelli.
-
-* 'bzr check' now accepts -v for verbose reporting, and checks for
-  ghosts in the branch. (Robert Collins)
-
-* New command 're-sign' which will regenerate the gpg signature for
-  a revision. (Robert Collins)
-
-* If you set ``check_signatures=require`` for a path in
-  ``~/.bazaar/branches.conf`` then bzr will invoke your
-  ``gpg_signing_command`` (defaults to gpg) and record a digital signature
-  of your commit. (Robert Collins)
-
-* New sftp transport, based on Paramiko.  (Robey Pointer)
-
-* 'bzr pull' now accepts '--clobber' which will discard local changes
-  and make this branch identical to the source branch. (Robert Collins)
-
-* Just give a quieter warning if a plugin can't be loaded, and
-  put the details in .bzr.log.  (Martin Pool)
-
-* 'bzr branch' will now set the branch-name to the last component of the
-  output directory, if one was supplied.
-
-* If the option ``post_commit`` is set to one (or more) python function
-  names (must be in the bzrlib namespace), then they will be invoked
-  after the commit has completed, with the branch and ``revision_id`` as
-  parameters. (Robert Collins)
-
-* Merge now has a retcode of 1 when conflicts occur. (Robert Collins)
-
-* --merge-type weave is now supported for file contents.  Tree-shape
-  changes are still three-way based.  (Martin Pool, Aaron Bentley)
-
-* 'bzr check' allows the first revision on revision-history to have
-  parents - something that is expected for cheap checkouts, and occurs
-  when conversions from baz do not have all history.  (Robert Collins).
-
-* 'bzr merge' can now graft unrelated trees together, if your specify
-  0 as a base. (Aaron Bentley)
-
-* 'bzr commit branch' and 'bzr commit branch/file1 branch/file2' now work
-  (Aaron Bentley)
-
-* Add '.sconsign*' to default ignore list.  (Alexander Belchenko)
-
-* 'bzr merge --reprocess' minimizes conflicts
-
-Testing
-*******
-
-* The 'bzr selftest --pattern' option for has been removed, now
-  test specifiers on the command line can be simple strings, or
-  regexps, or both. (Robert Collins)
-
-* Passing -v to selftest will now show the time each test took to
-  complete, which will aid in analysing performance regressions and
-  related questions. (Robert Collins)
-
-* 'bzr selftest' runs all tests, even if one fails, unless '--one'
-  is given. (Martin Pool)
-
-* There is a new method for TestCaseInTempDir, assertFileEqual, which
-  will check that a given content is equal to the content of the named
-  file. (Robert Collins)
-
-* Fix test suite's habit of leaving many temporary log files in $TMPDIR.
-  (Martin Pool)
-
-Internals
-*********
-
-* New 'testament' command and concept for making gpg-signatures
-  of revisions that are not tied to a particular internal
-  representation.  (Martin Pool).
-
-* Per-revision properties ('revprops') as key-value associated
-  strings on each revision created when the revision is committed.
-  Intended mainly for the use of external tools.  (Martin Pool).
-
-* Config options have moved from bzrlib.osutils to bzrlib.config.
-  (Robert Collins)
-
-* Improved command line option definitions allowing explanations
-  for individual options, among other things.  Contributed by
-  Magnus Therning.
-
-* Config options have moved from bzrlib.osutils to bzrlib.config.
-  Configuration is now done via the config.Config interface:
-  Depending on whether you have a Branch, a Location or no information
-  available, construct a ``*Config``, and use its ``signature_checking``,
-  ``username`` and ``user_email`` methods. (Robert Collins)
-
-* Plugins are now loaded under bzrlib.plugins, not bzrlib.plugin, and
-  they are made available for other plugins to use. You should not
-  import other plugins during the ``__init__`` of your plugin though, as
-  no ordering is guaranteed, and the plugins directory is not on the
-  python path. (Robert Collins)
-
-* Branch.relpath has been moved to WorkingTree.relpath. WorkingTree no
-  no longer takes an inventory, rather it takes an option branch
-  parameter, and if None is given will open the branch at basedir
-  implicitly. (Robert Collins)
-
-* Cleaner exception structure and error reporting.  Suggested by
-  Scott James Remnant.  (Martin Pool)
-
-* Branch.remove has been moved to WorkingTree, which has also gained
-  ``lock_read``, ``lock_write`` and ``unlock`` methods for convenience.
-  (Robert Collins)
-
-* Two decorators, ``needs_read_lock`` and ``needs_write_lock`` have been
-  added to the branch module. Use these to cause a function to run in a
-  read or write lock respectively. (Robert Collins)
-
-* ``Branch.open_containing`` now returns a tuple (Branch, relative-path),
-  which allows direct access to the common case of 'get me this file
-  from its branch'. (Robert Collins)
-
-* Transports can register using ``register_lazy_transport``, and they
-  will be loaded when first used.  (Martin Pool)
-
-* 'pull' has been factored out of the command as ``WorkingTree.pull()``.
-  A new option to WorkingTree.pull has been added, clobber, which will
-  ignore diverged history and pull anyway.
-  (Robert Collins)
-
-* config.Config has a ``get_user_option`` call that accepts an option name.
-  This will be looked up in branches.conf and bazaar.conf as normal.
-  It is intended that this be used by plugins to support options -
-  options of built in programs should have specific methods on the config.
-  (Robert Collins)
-
-* ``merge.merge_inner`` now has tempdir as an optional parameter.
-  (Robert Collins)
-
-* Tree.kind is not recorded at the top level of the hierarchy, as it was
-  missing on EmptyTree, leading to a bug with merge on EmptyTrees.
-  (Robert Collins)
-
-* ``WorkingTree.__del__`` has been removed, it was non deterministic and not
-  doing what it was intended to. See ``WorkingTree.__init__`` for a comment
-  about future directions. (Robert Collins/Martin Pool)
-
-* bzrlib.transport.http has been modified so that only 404 urllib errors
-  are returned as NoSuchFile. Other exceptions will propagate as normal.
-  This allows debuging of actual errors. (Robert Collins)
-
-* bzrlib.transport.Transport now accepts *ONLY* url escaped relative paths
-  to apis like 'put', 'get' and 'has'. This is to provide consistent
-  behaviour - it operates on url's only. (Robert Collins)
-
-* Transports can register using ``register_lazy_transport``, and they
-  will be loaded when first used.  (Martin Pool)
-
-* ``merge_flex`` no longer calls ``conflict_handler.finalize()``, instead that
-  is called by ``merge_inner``. This is so that the conflict count can be
-  retrieved (and potentially manipulated) before returning to the caller
-  of ``merge_inner``. Likewise 'merge' now returns the conflict count to the
-  caller. (Robert Collins)
-
-* ``revision.revision_graph`` can handle having only partial history for
-  a revision - that is no revisions in the graph with no parents.
-  (Robert Collins).
-
-* New ``builtins.branch_files`` uses the standard ``file_list`` rules to
-  produce a branch and a list of paths, relative to that branch
-  (Aaron Bentley)
-
-* New TestCase.addCleanup facility.
-
-* New ``bzrlib.version_info`` tuple (similar to ``sys.version_info``),
-  which can be used by programs importing bzrlib.
-
-Bug Fixes
-*********
-
-* Better handling of branches in directories with non-ascii names.
-  (Joel Rosdahl, Panagiotis Papadakos)
-
-* Upgrades of trees with no commits will not fail due to accessing
-  [-1] in the revision-history. (Andres Salomon)
-
-
-bzr 0.1.1
-#########
-
-:Released: 2005-10-12
-
-Bug Fixes
-*********
-
-* Fix problem in pulling over http from machines that do not
-  allow directories to be listed.
-
-* Avoid harmless warning about invalid hash cache after
-  upgrading branch format.
-
-Performance
-***********
-
-* Avoid some unnecessary http operations in branch and pull.
-
-
-bzr 0.1
-#######
-
-:Released: 2005-10-11
-
-Notes
-*****
-
-* 'bzr branch' over http initially gives a very high estimate
-  of completion time but it should fall as the first few
-  revisions are pulled in.  branch is still slow on
-  high-latency connections.
-
-Bug Fixes
-*********
-
-* bzr-man.py has been updated to work again. Contributed by
-  Rob Weir.
-
-* Locking is now done with fcntl.lockf which works with NFS
-  file systems. Contributed by Harald Meland.
-
-* When a merge encounters a file that has been deleted on
-  one side and modified on the other, the old contents are
-  written out to foo.BASE and foo.SIDE, where SIDE is this
-  or OTHER. Contributed by Aaron Bentley.
-
-* Export was choosing incorrect file paths for the content of
-  the tarball, this has been fixed by Aaron Bentley.
-
-* Commit will no longer commit without a log message, an
-  error is returned instead. Contributed by Jelmer Vernooij.
-
-* If you commit a specific file in a sub directory, any of its
-  parent directories that are added but not listed will be
-  automatically included. Suggested by Michael Ellerman.
-
-* bzr commit and upgrade did not correctly record new revisions
-  for files with only a change to their executable status.
-  bzr will correct this when it encounters it. Fixed by
-  Robert Collins
-
-* HTTP tests now force off the use of ``http_proxy`` for the duration.
-  Contributed by Gustavo Niemeyer.
-
-* Fix problems in merging weave-based branches that have
-  different partial views of history.
-
-* Symlink support: working with symlinks when not in the root of a
-  bzr tree was broken, patch from Scott James Remnant.
-
-Improvements
-************
-
-* 'branch' now accepts a --basis parameter which will take advantage
-  of local history when making a new branch. This allows faster
-  branching of remote branches. Contributed by Aaron Bentley.
-
-* New tree format based on weave files, called version 5.
-  Existing branches can be upgraded to this format using
-  'bzr upgrade'.
-
-* Symlinks are now versionable. Initial patch by
-  Erik Toubro Nielsen, updated to head by Robert Collins.
-
-* Executable bits are tracked on files. Patch from Gustavo
-  Niemeyer.
-
-* 'bzr status' now shows unknown files inside a selected directory.
-  Patch from Heikki Paajanen.
-
-* Merge conflicts are recorded in .bzr. Two new commands 'conflicts'
-  and 'resolve' have needed added, which list and remove those
-  merge conflicts respectively. A conflicted tree cannot be committed
-  in. Contributed by Aaron Bentley.
-
-* 'rm' is now an alias for 'remove'.
-
-* Stores now split out their content in a single byte prefixed hash,
-  dropping the density of files per directory by 256. Contributed by
-  Gustavo Niemeyer.
-
-* 'bzr diff -r branch:URL' will now perform a diff between two branches.
-  Contributed by Robert Collins.
-
-* 'bzr log' with the default formatter will show merged revisions,
-  indented to the right. Initial implementation contributed by Gustavo
-  Niemeyer, made incremental by Robert Collins.
-
-
-Internals
-*********
-
-* Test case failures have the exception printed after the log
-  for your viewing pleasure.
-
-* InventoryEntry is now an abstract base class, use one of the
-  concrete InventoryDirectory etc classes instead.
-
-* Branch raises an UnsupportedFormatError when it detects a
-  bzr branch it cannot understand. This allows for precise
-  handling of such circumstances.
-
-* Remove RevisionReference class; ``Revision.parent_ids`` is now simply a
-  list of their ids and ``parent_sha1s`` is a list of their corresponding
-  sha1s (for old branches only at the moment.)
-
-* New method-object style interface for Commit() and Fetch().
-
-* Renamed ``Branch.last_patch()`` to ``Branch.last_revision()``, since
-  we call them revisions not patches.
-
-* Move ``copy_branch`` to ``bzrlib.clone.copy_branch``.  The destination
-  directory is created if it doesn't exist.
-
-* Inventories now identify the files which were present by
-  giving the revision *of that file*.
-
-* Inventory and Revision XML contains a version identifier.
-  This must be consistent with the overall branch version
-  but allows for more flexibility in future upgrades.
-
-Testing
-*******
-
-* Removed testsweet module so that tests can be run after
-  bzr installed by 'bzr selftest'.
-
-* 'bzr selftest' command-line arguments can now be partial ids
-  of tests to run, e.g. ``bzr selftest test_weave``
-
-
-bzr 0.0.9
-#########
-
-:Released: 2005-09-23
-
-Bug Fixes
-*********
-
-* Fixed "branch -r" option.
-
-* Fix remote access to branches containing non-compressed history.
-  (Robert Collins).
-
-* Better reliability of http server tests.  (John Arbash-Meinel)
-
-* Merge graph maximum distance calculation fix.  (Aaron Bentley)
-
-* Various minor bug in windows support have been fixed, largely in the
-  test suite. Contributed by Alexander Belchenko.
-
-Improvements
-************
-
-* Status now accepts a -r argument to give status between chosen
-  revisions. Contributed by Heikki Paajanen.
-
-* Revision arguments no longer use +/-/= to control ranges, instead
-  there is a 'before' namespace, which limits the successive namespace.
-  For example '$ bzr log -r date:yesterday..before:date:today' will
-  select everything from yesterday and before today. Contributed by
-  Robey Pointer
-
-* There is now a bzr.bat file created by distutils when building on
-  Windows. Contributed by Alexander Belchenko.
-
-Internals
-*********
-
-* Removed uuid() as it was unused.
-
-* Improved 'fetch' code for pulling revisions from one branch into
-  another (used by pull, merged, etc.)
-
-
-bzr 0.0.8
-#########
-
-:Released: 2005-09-20
-
-
-Improvements
-************
-
-* Adding a file whose parent directory is not versioned will
-  implicitly add the parent, and so on up to the root. This means
-  you should never need to explictly add a directory, they'll just
-  get added when you add a file in the directory.  Contributed by
-  Michael Ellerman.
-
-* Ignore ``.DS_Store`` (contains Mac metadata) by default.
-  (Nir Soffer)
-
-* If you set ``BZR_EDITOR`` in the environment, it is checked in
-  preference to EDITOR and the config file for the interactive commit
-  editing program. Related to this is a bugfix where a missing program
-  set in EDITOR would cause editing to fail, now the fallback program
-  for the operating system is still tried.
-
-* Files that are not directories/symlinks/regular files will no longer
-  cause bzr to fail, it will just ignore them by default. You cannot add
-  them to the tree though - they are not versionable.
-
-
-Internals
-*********
-
-* Refactor xml packing/unpacking.
-
-Bug Fixes
-*********
-
-* Fixed 'bzr mv' by Ollie Rutherfurd.
-
-* Fixed strange error when trying to access a nonexistent http
-  branch.
-
-* Make sure that the hashcache gets written out if it can't be
-  read.
-
-
-Portability
-***********
-
-* Various Windows fixes from Ollie Rutherfurd.
-
-* Quieten warnings about locking; patch from Matt Lavin.
-
-
-bzr-0.0.7
-#########
-
-:Released: 2005-09-02
-
-New Features
-************
-
-* ``bzr shell-complete`` command contributed by Clint Adams to
-  help with intelligent shell completion.
-
-* New expert command ``bzr find-merge-base`` for debugging merges.
-
-
-Enhancements
-************
-
-* Much better merge support.
-
-* merge3 conflicts are now reported with markers like '<<<<<<<'
-  (seven characters) which is the same as CVS and pleases things
-  like emacs smerge.
-
-
-Bug Fixes
-*********
-
-* ``bzr upgrade`` no longer fails when trying to fix trees that
-  mention revisions that are not present.
-
-* Fixed bugs in listing plugins from ``bzr plugins``.
-
-* Fix case of $EDITOR containing options for the editor.
-
-* Fix log -r refusing to show the last revision.
-  (Patch from Goffredo Baroncelli.)
-
-
-Changes
-*******
-
-* ``bzr log --show-ids`` shows the revision ids of all parents.
-
-* Externally provided commands on your $BZRPATH no longer need
-  to recognize --bzr-usage to work properly, and can just handle
-  --help themselves.
-
-
-Library
-*******
-
-* Changed trace messages to go through the standard logging
-  framework, so that they can more easily be redirected by
-  libraries.
-
-
-
-bzr-0.0.6
-#########
-
-:Released: 2005-08-18
-
-New Features
-************
-
-* Python plugins, automatically loaded from the directories on
-  ``BZR_PLUGIN_PATH`` or ``~/.bzr.conf/plugins`` by default.
-
-* New 'bzr mkdir' command.
-
-* Commit mesage is fetched from an editor if not given on the
-  command line; patch from Torsten Marek.
-
-* ``bzr log -m FOO`` displays commits whose message matches regexp
-  FOO.
-
-* ``bzr add`` with no arguments adds everything under the current directory.
-
-* ``bzr mv`` does move or rename depending on its arguments, like
-  the Unix command.
-
-* ``bzr missing`` command shows a summary of the differences
-  between two trees.  (Merged from John Arbash-Meinel.)
-
-* An email address for commits to a particular tree can be
-  specified by putting it into .bzr/email within a branch.  (Based
-  on a patch from Heikki Paajanen.)
-
-
-Enhancements
-************
-
-* Faster working tree operations.
-
-
-Changes
-*******
-
-* 3rd-party modules shipped with bzr are copied within the bzrlib
-  python package, so that they can be installed by the setup
-  script without clashing with anything already existing on the
-  system.  (Contributed by Gustavo Niemeyer.)
-
-* Moved plugins directory to bzrlib/, so that there's a standard
-  plugin directory which is not only installed with bzr itself but
-  is also available when using bzr from the development tree.
-  ``BZR_PLUGIN_PATH`` and ``DEFAULT_PLUGIN_PATH`` are then added to the
-  standard plugins directory.
-
-* When exporting to a tarball with ``bzr export --format tgz``, put
-  everything under a top directory rather than dumping it into the
-  current directory.   This can be overridden with the ``--root``
-  option.  Patch from William Dodé and John Meinel.
-
-* New ``bzr upgrade`` command to upgrade the format of a branch,
-  replacing ``bzr check --update``.
-
-* Files within store directories are no longer marked readonly on
-  disk.
-
-* Changed ``bzr log`` output to a more compact form suggested by
-  John A Meinel.  Old format is available with the ``--long`` or
-  ``-l`` option, patched by William Dodé.
-
-* By default the commit command refuses to record a revision with
-  no changes unless the ``--unchanged`` option is given.
-
-* The ``--no-plugins``, ``--profile`` and ``--builtin`` command
-  line options must come before the command name because they
-  affect what commands are available; all other options must come
-  after the command name because their interpretation depends on
-  it.
-
-* ``branch`` and ``clone`` added as aliases for ``branch``.
-
-* Default log format is back to the long format; the compact one
-  is available with ``--short``.
-
-
-Bug Fixes
-*********
-
-* Fix bugs in committing only selected files or within a subdirectory.
-
-
-bzr-0.0.5
-#########
-
-:Released:  2005-06-15
-
-Changes
-*******
-
-* ``bzr`` with no command now shows help rather than giving an
-  error.  Suggested by Michael Ellerman.
-
-* ``bzr status`` output format changed, because svn-style output
-  doesn't really match the model of bzr.  Now files are grouped by
-  status and can be shown with their IDs.  ``bzr status --all``
-  shows all versioned files and unknown files but not ignored files.
-
-* ``bzr log`` runs from most-recent to least-recent, the reverse
-  of the previous order.  The previous behaviour can be obtained
-  with the ``--forward`` option.
-
-* ``bzr inventory`` by default shows only filenames, and also ids
-  if ``--show-ids`` is given, in which case the id is the second
-  field.
-
-
-Enhancements
-************
-
-* New 'bzr whoami --email' option shows only the email component
-  of the user identification, from Jo Vermeulen.
-
-* New ``bzr ignore PATTERN`` command.
-
-* Nicer error message for broken pipe, interrupt and similar
-  conditions that don't indicate an internal error.
-
-* Add ``.*.sw[nop] .git .*.tmp *,v`` to default ignore patterns.
-
-* Per-branch locks keyed on ``.bzr/branch-lock``, available in
-  either read or write mode.
-
-* New option ``bzr log --show-ids`` shows revision and file ids.
-
-* New usage ``bzr log FILENAME`` shows only revisions that
-  affected that file.
-
-* Changed format for describing changes in ``bzr log -v``.
-
-* New option ``bzr commit --file`` to take a message from a file,
-  suggested by LarstiQ.
-
-* New syntax ``bzr status [FILE...]`` contributed by Bartosz
-  Oler.  File may be in a branch other than the working directory.
-
-* ``bzr log`` and ``bzr root`` can be given an http URL instead of
-  a filename.
-
-* Commands can now be defined by external programs or scripts
-  in a directory on $BZRPATH.
-
-* New "stat cache" avoids reading the contents of files if they
-  haven't changed since the previous time.
-
-* If the Python interpreter is too old, try to find a better one
-  or give an error.  Based on a patch from Fredrik Lundh.
-
-* New optional parameter ``bzr info [BRANCH]``.
-
-* New form ``bzr commit SELECTED`` to commit only selected files.
-
-* New form ``bzr log -r FROM:TO`` shows changes in selected
-  range; contributed by John A Meinel.
-
-* New option ``bzr diff --diff-options 'OPTS'`` allows passing
-  options through to an external GNU diff.
-
-* New option ``bzr add --no-recurse`` to add a directory but not
-  their contents.
-
-* ``bzr --version`` now shows more information if bzr is being run
-  from a branch.
-
-
-Bug Fixes
-*********
-
-* Fixed diff format so that added and removed files will be
-  handled properly by patch.  Fix from Lalo Martins.
-
-* Various fixes for files whose names contain spaces or other
-  metacharacters.
-
-
-Testing
-*******
-
-* Converted black-box test suites from Bourne shell into Python;
-  now run using ``./testbzr``.  Various structural improvements to
-  the tests.
-
-* testbzr by default runs the version of bzr found in the same
-  directory as the tests, or the one given as the first parameter.
-
-* testbzr also runs the internal tests, so the only command
-  required to check is just ``./testbzr``.
-
-* testbzr requires python2.4, but can be used to test bzr running
-  under a different version.
-
-* Tests added for many other changes in this release.
-
-
-Internal
-********
-
-* Included ElementTree library upgraded to 1.2.6 by Fredrik Lundh.
-
-* Refactor command functions into Command objects based on HCT by
-  Scott James Remnant.
-
-* Better help messages for many commands.
-
-* Expose ``bzrlib.open_tracefile()`` to start the tracefile; until
-  this is called trace messages are just discarded.
-
-* New internal function ``find_touching_revisions()`` and hidden
-  command touching-revisions trace the changes to a given file.
-
-* Simpler and faster ``compare_inventories()`` function.
-
-* ``bzrlib.open_tracefile()`` takes a tracefilename parameter.
-
-* New AtomicFile class.
-
-* New developer commands ``added``, ``modified``.
-
-
-Portability
-***********
-
-* Cope on Windows on python2.3 by using the weaker random seed.
-  2.4 is now only recommended.
-
-
-bzr-0.0.4
-#########
-
-:Released:  2005-04-22
-
-Enhancements
-************
-
-* 'bzr diff' optionally takes a list of files to diff.  Still a bit
-  basic.  Patch from QuantumG.
-
-* More default ignore patterns.
-
-* New 'bzr log --verbose' shows a list of files changed in the
-  changeset.  Patch from Sebastian Cote.
-
-* Roll over ~/.bzr.log if it gets too large.
-
-* Command abbreviations 'ci', 'st', 'stat', '?' based on a patch
-  by Jason Diamon.
-
-* New 'bzr help commands' based on a patch from Denys Duchier.
-
-
-Changes
-*******
-
-* User email is determined by looking at $BZREMAIL or ~/.bzr.email
-  or $EMAIL.  All are decoded by the locale preferred encoding.
-  If none of these are present user@hostname is used.  The host's
-  fully-qualified name is not used because that tends to fail when
-  there are DNS problems.
-
-* New 'bzr whoami' command instead of username user-email.
-
-
-Bug Fixes
-*********
-
-* Make commit safe for hardlinked bzr trees.
-
-* Some Unicode/locale fixes.
-
-* Partial workaround for ``difflib.unified_diff`` not handling
-  trailing newlines properly.
-
-
-Internal
-********
-
-* Allow docstrings for help to be in PEP0257 format.  Patch from
-  Matt Brubeck.
-
-* More tests in test.sh.
-
-* Write profile data to a temporary file not into working
-  directory and delete it when done.
-
-* Smaller .bzr.log with process ids.
-
-
-Portability
-***********
-
-* Fix opening of ~/.bzr.log on Windows.  Patch from Andrew
-  Bennetts.
-
-* Some improvements in handling paths on Windows, based on a patch
-  from QuantumG.
-
-
-bzr-0.0.3
-#########
-
-:Released:  2005-04-06
-
-Enhancements
-************
-
-* New "directories" internal command lists versioned directories
-  in the tree.
-
-* Can now say "bzr commit --help".
-
-* New "rename" command to rename one file to a different name
-  and/or directory.
-
-* New "move" command to move one or more files into a different
-  directory.
-
-* New "renames" command lists files renamed since base revision.
-
-* New cat command contributed by janmar.
-
-Changes
-*******
-
-* .bzr.log is placed in $HOME (not pwd) and is always written in
-  UTF-8.  (Probably not a completely good long-term solution, but
-  will do for now.)
-
-Portability
-***********
-
-* Workaround for difflib bug in Python 2.3 that causes an
-  exception when comparing empty files.  Reported by Erik Toubro
-  Nielsen.
-
-Internal
-********
-
-* Refactored inventory storage to insert a root entry at the top.
-
-Testing
-*******
-
-* Start of shell-based black-box testing in test.sh.
-
-
-bzr-0.0.2.1
-###########
-
-Portability
-***********
-
-* Win32 fixes from Steve Brown.
-
-
-bzr-0.0.2
-#########
-
-:Codename: "black cube"
-:Released: 2005-03-31
-
-Enhancements
-************
-
-* Default ignore list extended (see bzrlib/__init__.py).
-
-* Patterns in .bzrignore are now added to the default ignore list,
-  rather than replacing it.
-
-* Ignore list isn't reread for every file.
-
-* More help topics.
-
-* Reinstate the 'bzr check' command to check invariants of the
-  branch.
-
-* New 'ignored' command lists which files are ignored and why;
-  'deleted' lists files deleted in the current working tree.
-
-* Performance improvements.
-
-* New global --profile option.
-
-* Ignore patterns like './config.h' now correctly match files in
-  the root directory only.
-
-
-bzr-0.0.1
-#########
-
-:Released:  2005-03-26
-
-Enhancements
-************
-
-* More information from info command.
-
-* Can now say "bzr help COMMAND" for more detailed help.
-
-* Less file flushing and faster performance when writing logs and
-  committing to stores.
-
-* More useful verbose output from some commands.
-
-Bug Fixes
-*********
-
-* Fix inverted display of 'R' and 'M' during 'commit -v'.
-
-Portability
-***********
-
-* Include a subset of ElementTree-1.2.20040618 to make
-  installation easier.
-
-* Fix time.localtime call to work with Python 2.3 (the minimum
-  supported).
-
-
-bzr-0.0.0.69
-############
-
-:Released:  2005-03-22
-
-Enhancements
-************
-
-* First public release.
-
-* Storage of local versions: init, add, remove, rm, info, log,
-  diff, status, etc.
+* Catch socket errors to avoid
+  bt.test_sftp_transport.SSHVendorBadConnection.test_bad_connection_ssh
+  random failures. (Vincent Ladeuil, #601804)
+
+* HTTP test servers will leak less threads (and sockets) and will not hang on
+  AIX anymore. (Vincent Ladeuil, #405745)
+
+* On platforms that don't support forking give a nice error message saying so
+  when ``bzr selftest --parallel=fork`` is used. (Martin [gz], #528730)
+
+* Rearrange thread leak detection code to eliminate global state and make it
+  possible to extend the reporting. (Martin [gz], #633462)
+
+* The old ``bzr selftest --benchmark`` option has been removed.
+  <https://launchpad.net/bzr-usertest> is an actively-maintained
+  macrobenchmark suite.
+  (Martin Pool)
+
+* The test suite now simply holds log files in memory, rather than writing them
+  out to disk and then reading them back in and deleting them.
+  (Andrew Bennetts)
+
+* The way ``bzr selftest --parallel`` generates N partitions of tests to
+  run in parallel has changed.  Instead of splitting the list of tests at
+  N-1 points, it distributes the tests one-by-one into the partitions in a
+  round robin fashion.  This reduces the total time to run the tests in
+  parallel because a series of slow tests in the test suite will be
+  distributed evenly among the parallel test suites, rather than slowing
+  down just one suite.  (Andrew Bennetts)
+
+* Tracebacks from a parameterized test are no longer reported against every
+  parameterization of that test.  This was done by adding a hack to
+  ``bzrlib.tests.clone_test`` so that it no longer causes
+  testtools.TestCase instances to share a details dict.
+  (Andrew Bennetts, #625574)
 
 
 ..
-   vim: tw=74 ft=rst ff=unix encoding=utf-8
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-10-10 13:59:22 +0000
@@ -0,0 +1,1276 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.4.2
+#########
+
+:2.4.2: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* Fixed a bug where ``bzr tags -r x..y`` loaded the branch history once for
+  every revision in the range; it's now much faster. (Vincent Ladeuil, #857335)
+
+Bug Fixes
+*********
+
+* Fixed an infinite loop when creating a repo at the root of the filesystem, 
+  i.e. "/", due to posixpath.normpath() not collapsing 2 leading slashes into 
+  one, thus respecting the POSIX standard, but making relpath() loop infinitely.
+  (Florian Vichot, #861008)
+
+* Fixed loading of external merge tools from config to properly decode
+  command-lines which contain embedded quotes. (Gordon Tyler, #828803)
+
+* Include declaration of 'changed' to avoid an UnboundLocalError in dirstate
+  pyrex code with new Cython versions. (Denys Duchier, #837221)
+
+* Prevent several kinds of OverflowError and other fallout from failing to fit
+  stat fields into four bytes in dirstate pack_stat implementations.
+  (Martin Packman, #683191 #706957)
+
+* Return early from create_delta_index_from_delta given tiny inputs. This
+  avoids raising a spurious MemoryError on certain platforms such as AIX.
+  (John Arbash Meinel, #856731)
+  
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* Fix the race for TestingThreadingTCPServer in
+  test_server_crash_while_responding. (Vincent Ladeuil, #869366)
+
+* Really corrupt the pack file without depending on a special length or value.
+  (Vincent Ladeuil, #807032)
+
+
+bzr 2.4.1
+#########
+
+:2.4.1: 2011-09-08
+
+This is a bugfix release. Upgrading is recommended for all users of earlier
+2.4 releases.
+
+It includes fixes from previous stable releases and address some issues with
+the test suite.
+
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* ``config.LocationMatcher`` properly excludes unrelated sections.
+  (Vincent Ladeuil, #829237)
+
+* ``dirstate.fdatasync`` and ``repository.fdatasync`` can now properly be
+  disabled. (Vincent Ladeuil, #824513)
+
+* Disable ``os.fsync`` and ``os.fdatasync`` by default when running
+  ``bzr selftest``. You can use ``--sync`` to re-enable them.
+  (John Arbash Meinel, #837293)
+
+* Fix i18n use when no environment variables are set. (Jelmer Vernooij, #810701)
+
+* Avoid UnicodeDecode error when reporting EINVAL from transports.
+  (IWATA Hidetaka, #829237)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+* Corrected documentation for BZR_PROGRESS_BAR. 
+  (Dennis Benzinger, #735417)
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* The test suite should now be able to run under weird environments where
+  ``/etc/passwd`` doesn't contain the ``uid`` for the user running selftest
+  or where ``fakeroot`` is used but ``/root`` is inacessible.
+  (Vincent Ladeuil, #825027)
+
+bzr 2.4.0
+#########
+
+:2.4.0: 2011-08-11
+
+This release marks the start of a new long-term-stable series. From here, we
+will only make bugfix releases on the 2.4 series (2.4.1, etc, and support it
+until February 2013), while 2.5 will become our new development series.
+
+This is a bugfix and polish release over the 2.3 series, with a large number
+of bugs fixed (>150 for the 2.4 series alone), and some performance
+improvements. Support for python 2.4 and 2.5 has been dropped, many large
+working tree operations have been optimized as well as some stacked branches
+operations.
+
+Only bugfixes from other stables series have been included since 2.4b5 so
+all known fixed bugs are included here.
+
+Users are encouraged to upgrade from the other stable series.
+
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* A call to CHKInventory's filter-method will not result in a
+  DuplicateFileId error, if you move a subfolder and change a file in
+  that subfolder.
+  (Bastian Bowe, #809901)
+
+* Accessing a packaging branch on Launchpad (eg, ``lp:ubuntu/bzr``) now
+  checks to see if the most recent published source package version for
+  that project is present in the branch tags. This should help developers
+  trust whether the packaging branch is up-to-date and can be used for new
+  changes. The level of verbosity is controlled by the config item
+  ``launchpad.packaging_verbosity``. It can be set to one of
+
+  off
+    disable all checks
+
+
+  minimal
+    only display if the branch is out-of-date
+
+  short
+    also display single-line up-to-date and missing,
+
+
+  all
+    (default) display multi-line content for all states
+
+
+  (John Arbash Meinel, #609187, #812928)
+
+* Cope with not all Python versions having a ``clear`` method on
+  ``TestCase._type_equality_funcs``.
+  (Martin [gz], Jelmer Vernooij, #809048)
+
+* Fetching tags when fetching the tip revision of a branch is now
+  controlled by the config setting ``branch.fetch_tags``. The behavior has
+  been reverted to 2.3's not-fetching tagged revisions by default.
+  (John Arbash Meinel, #771184)
+
+* The fix for bug #513709 caused us to open a new connection when
+  switching a lightweight checkout that was pointing at a bound branch.
+  This isn't necessary because we know the master URL without opening it,
+  avoiding an extra SSH connection, etc.
+  (John Arbash Meinel, #812285)
+
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* `BranchBuilder.build_snapshot` now supports a "flush" action.  This
+  cleanly and reliably allows tests using `BranchBuilder` to construct
+  branches that e.g. rename files out of a directory and unversion that
+  directory in the same revision.  Previously some changes were impossible
+  due to the order that `build_snapshot` performs its actions.
+  (Andrew Bennetts)
+
+* `TestCaseWithMemoryTransport` is faster now: `_check_safety_net` now
+  just compares the bytes in the dirstate file to its pristine state,
+  rather than opening the WorkingTree and calling ``last_revision()``.
+  This reduces the overall test suite time by about 10% on my laptop.
+  (Andrew Bennetts)
+
+
+bzr 2.4b5
+#########
+
+:2.4b5: 2011-07-07
+
+This is the fifth (and last) beta of the 2.4 series leading to
+2.4.0 release in August 2011. Beta releases are suitable for
+everyday use but may cause some incompatibilities with plugins.
+
+This release includes all bug fixed in previous series known at
+the time of this release.
+
+External Compatibility Breaks
+*****************************
+
+None.
+
+New Features
+************
+
+* New command ``verify-signatures`` to check if all commits or specified commits
+  have digital signatures from trusted keys.  Requires python-gpgme to be
+  installed.
+
+* New option ``--signatures`` for ``bzr log`` to display digital signature
+  verification results for each commit.
+
+* Config option acceptable_keys to list which GPG keys are verified as trusted.
+
+* Config option validate_signatures_in_log to always show signatures in 
+  ``bzr log``.
+
+Improvements
+************
+
+* ``Branch.open`` is now about 3x faster (about 2ms instead of 6.5ms).
+  (Andrew Bennetts).
+
+* Pack, dirstate, and index files are synced to persistent storage if 
+  possible when writing finishes, to reduce the risk of problems caused by
+  a machine crash or similar problem.  This can be turned off through the
+  ``dirstate.fdatasync`` and ``repository.fdatasync`` options, which can
+  be set in ``locations.conf`` or ``bazaar.conf``.  (Martin Pool,
+  #343427)
+
+Bug Fixes
+*********
+
+* Display a proper error message when a config file content cannot be
+  decoded as UTF-8 or when it cannot be parsed.
+  (Vincent Ladeuil, #502060, #688677, #797246)
+
+* Generate a single conflict (instead of two) when merging a branch
+  modifying and renaming a file in a branch that deleted it (or vice-versa).
+  (Vincent Ladeuil, #688101)
+
+* Give a more helpful message when the bzr executable doesn't match the
+  library.  (This typically happens because of a misconfigured PYTHONPATH
+  or half-installed bzr.)  
+  (Martin Pool, #804553)
+
+* Properly load utf8-encoded config files. (Vincent Ladeuil, #799212)
+
+* ``GraphThunkIdsToKeys.merge_sort`` now properly returns
+  keys rather than ids. (Jelmer Vernooij, #799677)
+
+* ``TreeTransformBase.fixup_new_roots`` can now check that a tree root
+  is present. (Jelmer Vernooij, #801257)
+
+API Changes
+***********
+
+* New attributes ``WorkingTreeFormat.supports_versioned_directories`` and
+  ``RepositoryFormat.supports_versioned_directories``.
+  (Jelmer Vernooij, #765815)
+
+* The "revno" field type when using the python version-info format is now
+  a string (to handle dotted revnos) (Benoît Pierre, #796259)
+
+Internals
+*********
+
+* Start implementing localization, starting with command help text (but not
+  the command options themselves). This will allow bootstrapping the bzr
+  internationalization process. (Inada Naoki)
+
+Testing
+*******
+
+* Fix test failures when running as a homeless user (debian buildd). Tests
+  leaking into ``${HOME}/.bzr.log`` should be detected properly now.
+  (Vincent Ladeuil, #798698)
+
+bzr 2.4b4
+#########
+
+:2.4b4: 2011-06-16
+
+This is the fourth beta of the 2.4 series, leading to a 2.4.0 release in
+August 2011. Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.
+
+This release includes all bug fixed in previous series known at the time of
+this release.
+
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+* Do not treat configuration option 'check_signatures = require' as if
+  it were 'create_signatures = always' (Jonathan Riddell)
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+* Hooks have been added for config stacks: ``get``, ``set`` and ``remove``
+  are called when an option is respectively read, modified or deleted. Also
+  added ``load`` and ``save`` hooks for config stores, called when the
+  stores are loaded or saved.  (Vincent Ladeuil)
+
+* New hook server_exception in bzrlib.smart.server to catch any
+  exception caused while running bzr serve.
+  (Jonathan Riddell, #274578)
+
+* New hook set_commit_message in bzrlib.msgeditor to set a commit message
+  and revision properties.  (Jonathan Riddell, #274578)
+
+* Support ``-S`` as an alias for ``--short`` for the ``log`` and
+  ``missing`` commands. (Martin von Gagern, #38655)
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* ``bzr annotate`` can be run without setting whoami data first.
+  (Jonathan Riddell, #667408)
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Bazaar can now detect when a lock file is held by a dead process
+  originating from the same machine, and steal the lock after printing a
+  message to the user.  This is off by default, for safety, but can be
+  turned on by setting the configuration variable ``locks.steal_dead`` to
+  ``True``.
+  (Martin Pool, #220464)
+
+* ``bzr version-info`` now works when the tree is on a dotted revno.
+  (Benoît Pierre, #796259)
+
+* Credentials in the log output produced by ``-Dhttp`` are masked so users
+  can more freely post them in bug reports. (Vincent Ladeuil, #723074)
+
+* Fix a race condition for ``server_started`` hooks leading to a spurious
+  test failure. (Vincent Ladeuil, #789167)
+
+* Fix exporting subdirectory with ``--per-file-timestamps``.
+  (Szilveszter Farkas, #795557)
+
+* Handle files that get created but don't get used during TreeTransform.
+  ``open()`` can create a file, and still raise an exception before it
+  returns. So anything we might have created, make sure we destroy during
+  ``finalize()``. (Martin [gz], #597686)
+
+* ``pack_repo`` now uses ``Transport.move`` instead of
+  ``Transport.rename``, deleting any existing targets even on SFTP.
+  (Martin von Gagern, #421776)
+
+* Pass the ``build_mo`` command to the rest of the setup() calls in
+  setup.py. The ``bdist_wininst`` and ``py2exe`` code paths were failing
+  because ``build_mo`` became a required step that they didn't know about.
+  (John Arbash Meinel, #787122)
+
+* Preserve existing ``root-id`` when merging an unrelated branch.
+  (Aaron Bentley, #806356)
+
+* Properly avoid re-adding a file after it changes case on CICP
+  filesystems. (John Arbash Meinel, #798130)
+
+* Reports the original error when an InvalidHttpResponse exception is
+  encountered to facilitate debug. (Vincent Ladeuil, #788530)
+
+* Reports a non-existent file error when trying to merge in a file
+  that does not exist. (Jonathan Riddell, #330063)
+
+* ``UIFactory.prompt``, ``UIFactory.get_username``,
+  ``UIFactory.get_password`` and ``UIFactory.get_boolean`` now require a
+  unicode prompt to be passed in. (Jelmer Vernooij, #592083)
+
+* Support merging into the empty tree. (Aaron Bentley, #595328)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+* Improve documentation of ``bzr merge --force``.
+  (Neil Martinsen-Burrell, #767307)
+
+* Make docs for configuration options for digital signatures match 
+  reality. (Jonathan Riddell)
+
+* Add user-guide page on GPG signatures. (Jonathan Riddell)
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+* Checking for a file id in a `Tree` or `Inventory` using ``in`` is now
+  deprecated.  Instead, use `has_id`.
+  (Martin Pool)
+
+* Exporters are now all exposed as generators, rather than as single-call
+  functions, so that calling code can take stream the output.
+  (Xaav, Martin Pool)
+
+* Information about held lockdir locks returned from eg `LockDir.peek` is
+  now represented as a `LockHeldInfo` object, rather than a plain
+  Python dict.
+  (Martin Pool)
+
+* Remove `file_status` function.
+  (Martin Pool)
+
+* ``Repository.iter_reverse_revision_history`` is now deprecated.
+  Use ``Graph.iter_lefthand_ancestry`` instead.
+  (Jelmer Vernooij, #739481)
+
+* ``Repository.get_ancestry`` has been deprecated. Use
+  ``Graph.iter_ancestry`` instead.
+  (Jelmer Vernooij, #784511)
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+* ``tools/check-newsbugs.py`` accepts a ``--browser`` option to open
+  corresponding launchpad pages in a browser. (Vincent Ladeuil)
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* A `ImportTariffTestCase` base class has been added in
+  ``bzrlib.tests.test_import_tariff``, which can be used for import tariff
+  tests in plugins. (Jelmer Vernooij, #793465)
+
+* Fix deadlock in `TestImportTariffs.test_simple_serve` when stderr gets
+  more output than fits in the default buffer.  This was happening on the
+  Windows buildslave, and could easily happen in other circumstances where
+  the default OS buffer size for pipes is small or the ``python -v``
+  output is large.  (Andrew Bennetts, #784802)
+
+* Fix spurious test failure on OSX for WorkingTreeFormat2.
+  (Vincent Ladeuil, #787942)
+
+* Re-target ``bb.test_merge.TestMerge.test_merge_reversed_revision_range``
+  and rewrite it as a parametrized test to avoid unrelated failures.
+  (Vincent Ladeuil, #795456)
+
+* Show log file contents from subprocesses started by
+  `start_bzr_subprocess` in test failure details.  This may help diagnose
+  strange hangs and failures involving subprocesses.  (Andrew Bennetts)
+
+* Skip ``utextwrap`` tests when ``sphinx`` breaks text_wrap by an hostile
+  monkey-patch to textwrap.TextWrapper.wordsep_re.
+  (Vincent Ladeuil, #785098)
+
+* Multiple ``selftest --exclude`` options are now combined instead of
+  overriding each other. (Vincent Ladeuil, #746991)
+
+* Restore some ``FTPTransport`` test coverage by allowing ``pyftpdlib
+  0.6.0`` to be used. Also restore ``medusa`` support while leaving it
+  disabled to make it easier to use if/when we can in the future.
+  (Vincent Ladeuil, #781140)
+
+* `TestImportTariffs` no longer uses the real ``$HOME``.  This prevents it
+  from polluting ``$HOME/.bzr.log`` or being accidentally influenced by
+  user configuration such as aliases.  It still runs with all the user's
+  plugins enabled, as intended.
+  (Vincent Ladeuil, Andrew Bennetts, #789505)
+
+
+bzr 2.4b3
+#########
+
+:2.4b3: 2011-05-26
+
+This is the third beta of the 2.4 series, leading to a 2.4.0 release in
+August 2011. Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.
+
+This release includes all bug fixed in previous series known at the time of
+this release.
+
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+* ``bzr-2.4`` has officially dropped support for python2.4 and python2.5.
+  We will continue to maintain ``bzr-2.3`` for people who still need to
+  use those versions of python. (John Arbash Meinel)
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+* The text compressor used for 2a repositories now has a tweakable
+  parameter that can be set in bazaar.conf.
+  ``bzr.groupcompress.max_entries_per_source`` default of 65536.
+  When doing compression, we build up an index of locations to match
+  against. Setting this higher will result in slightly better compression,
+  at a cost of more memory. Note that a value of 65k represents fully
+  sampling a 1MB file. So this only has an effect when compressing texts
+  larger than N*16 bytes. (John Arbash Meinel, #602614)
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* ``bzr branch --stacked`` from a smart server uses the network a little
+  more efficiently.  For a simple branch it reduces the number of
+  round-trips by about 20%.  (Andrew Bennetts)
+
+* ``bzr log --line`` scales the width of the author field with the size of
+  the line.  This means that the full author name is shown when the
+  environment variable BZR_COLUMNS=0.  (Neil Martinsen-Burrell)
+
+* ``bzr pull`` now properly triggers the fast
+  ``CHKInventory.iter_changes`` rather than the slow generic
+  inter-Inventory changes. It used to use a ``DirStateRevisionTree`` as
+  one of the source trees, which is faster when we have to read the whole
+  inventory anyway, but much slower when we can get just the delta out of
+  the repository. On a 70k record tree, this changes ``bzr pull`` from 28s
+  down to 17s. (John Arbash Meinel, #780677)
+
+* Slightly reduced memory consumption when fetching into a 2a repository
+  by reusing existing caching a little better.  (Andrew Bennetts)
+
+* Speed up ``bzr status`` by a little bit when there are a couple of
+  modified files. We now track how many files we have seen that need
+  updating, and only rewrite the dirstate file if enough of them have
+  changed. The default is 10, and can be overridden by setting the branch
+  option "``bzr.workingtree.worth_saving_limit``".
+  (Ian Clatworthy, John Arbash Meinel, #380202)
+
+* Speed up ``bzr uncommit``. Instead of resetting the dirstate from
+  scratch, use ``update_basis_by_delta``, computing the delta from the
+  repository. (John Arbash Meinel, #780544)
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* All Tree types can now be exported as tar.*, zip or directories.
+  (Aaron Bentley)
+  
+* ``bzr merge --no-remember location`` never sets ``submit_branch``.
+  (Vincent Ladeuil, #782169)
+
+* ``bzr pull --no-remember location`` never sets
+  ``parent_location``.  ``bzr push --no-remember location`` never
+  sets ``push_location``.  ``bzr send --no-remember
+  submit_location public_location`` never sets ``submit_branch``
+  nor ``public_branch``.  (Vincent Ladeuil)
+
+* Conflicts involving non-ascii filenames are now properly reported rather
+  than failing with a UnicodeEncodeError. (Martin [GZ], #686161)
+
+* Correct parent is now set when using 'switch -b' with bound branches.
+  (A. S. Budden, #513709)
+
+* Fix `bzr plugins` regression in bzr 2.4 which resulted in a traceback
+  from writelines on ckj terminals. (Martin [GZ], #754082)
+
+* ``WT.inventory`` and ``WT.iter_entries_by_dir()`` was not correctly
+  reporting subdirectories that were tree references (in formats that
+  supported them). (John Arbash Meinel, #764677)
+
+* Merging into empty branches now gives an error as this is currently
+  not supported. (Jonathan Riddell, #242175)
+
+* Do not show exception to user on pointless commit error.
+  (Jonathan Riddell #317357)
+
+* ``WT.update_basis_by_delta`` no longer requires that the deltas match
+  the current WT state. This allows ``update_basis_by_delta`` to be used
+  by more commands than just commit. Updating with a delta allows us to
+  not load the whole inventory, which can take 10+s with large trees.
+  (Jonathan Riddell, John Arbash Meinel, #781168)
+
+* ``bzr mv --after old_name new_name`` now works if "new_name" is newly
+  added. (Benoît Pierre)
+
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+* Restore the workaround for option names including dots (--1.14) which was
+  disabled when we stopped listing --1.9 as a format.
+  (Vincent Ladeuil, #782289)
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+* ``annotate_file`` has been deprecated in favor of
+  ``annotate_file_revision_tree``. (Jelmer Vernooij, #775598)
+
+* ``Branch.fetch`` now takes an optional ``limit`` argument.
+  (Andrew Bennetts, Jelmer Vernooij, #750175)
+
+* ``Inter.get`` now raises ``NoCompatibleInter`` if there are no
+  compatible optimisers rather than an instance of the class it is called
+  on. (Jelmer Vernooij)
+
+* ``Branch.push`` now takes a ``lossy`` argument.
+  ``Branch.lossy_push`` has been removed.
+  (Jelmer Vernooij)
+
+* New method ``Repository.get_file_graph`` which can return the
+  per-file revision graph. (Jelmer Vernooij, #775578)
+
+* The default implementation of ``Branch`` is now oriented to
+  storing the branch tip. Branch implementations which store the full
+  history should now subclass ``FullHistoryBzrBranch``.
+  ``Branch._last_revision_info`` has been renamed to
+  ``Branch._read_last_revision_info`` (Jelmer Vernooij)
+
+* ``Tree.__iter__`` has been deprecated; use ``Tree.all_file_ids``
+  instead.  (Jelmer Vernooij)
+
+* ``Tree.get_symlink_target`` now takes an optional ``path``
+  argument. (Jelmer Vernooij)
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+* ``MutableTree.smart_add`` now uses inventory deltas.
+  (Jelmer Vernooij, #146165)
+
+* Removed ``bzrlib.branch._run_with_write_locked_target`` as
+  ``bzrlib.cleanup`` provides the same functionality in a more general
+  way.  (Andrew Bennetts)
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* A test that was expected to fail but passes instead now counts as a failure
+  catching up with new testtools and subunit handling. (Martin [GZ], #654474)
+
+* Make it easier for plugins to reuse the per_workingtree scenarios by
+  restoring the wt_scenarios helper that was accidentally deleted.
+  (Vincent Ladeuil, #783472)
+
+* Removed ``test_breakin`` tests that were excessively prone to hanging,
+  did not work on Wine, and partly already disabled.
+  (Martin Pool, #408814, #746985)
+
+* Windows locations are different and should be tested accordingly.
+  (Vincent Ladeuil, #788131)
+
+bzr 2.4b2
+#########
+
+:2.4b2: 2011-04-28
+
+This is the second beta of the 2.4 series, leading to a 2.4.0 release in
+August 2011. Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.
+
+This release includes all bug fixed in previous series known at the time of
+this release.
+
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+* Two command synonyms for ``bzr branch`` have been deprecated, to avoid
+  confusion and to allow the names to later be reused.  The removed names
+  are: ``get`` and ``clone``.   (Martin Pool, #506265)
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+* ``bzr commit`` now supports a ``--lossy`` argument that can be used
+  to discard any data that can not be natively represented when committing
+  to a foreign VCS. (Jelmer Vernooij, #587721)
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* ``bzr merge`` in large trees is now significantly faster. On a 70k entry
+  tree, the time went from ~3min down to 30s. This also effects ``bzr pull``
+  and ``bzr update`` since they use the same merge logic to update the
+  WorkingTree.  (John Arbash Meinel, #759091)
+
+* ``bzr revert`` now properly uses ``bzr status``'s optimized
+  ``iter_changes``. This can be a significant performance difference (33s
+  to 5s on large trees). (John Arbash Meinel, #759096)
+
+* Resolve ``lp:FOO`` urls locally rather than doing an XMLRPC request if
+  the user has done ``bzr launchpad-login``. The bzr+ssh URLs were already
+  being handed off to the remote server anyway (xmlrpc has been mapping
+  ``lp:bzr`` to ``bzr+ssh://bazaar.launchpad.net/+branch/bzr``, rather
+  than ``bzr+ssh://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev`` for a few
+  months now.) By doing it ourselves, we can cut out substantial startup
+  time. From Netherlands to London it was taking 368ms to do the XMLRPC
+  call as much as 2s from Sydney. You can test the local logic by using
+  ``-Dlaunchpad``.  (John Arbash Meinel, #397739)
+
+* When building a new WorkingTree (such as during ``bzr co`` or
+  ``bzr branch``) we now properly store the stat and hash of files that
+  are old enough. This saves a fair amount of time on the first
+  ``bzr status`` (on a 500MB tree, it saves about 30+s).
+  (John Arbash Meinel, #740932)
+
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Arguments that can't be decoded to unicode in the current posix locale give
+  a clearer error message without a traceback. (Martin [gz], #745712)
+
+* ``bzrlib.log._DEFAULT_REQUEST_PARAMS`` is no longer accidentally
+  mutated by ``bzrlib.log._apply_log_request_defaults``.  In practice
+  these default values aren't relied on very often so this probably
+  wasn't causing any trouble.  (Andrew Bennetts)
+
+* ``bzr log`` now works on revisions which are not in the current branch.
+  (Matt Giuca, #241998)
+
+* Don't rewrite the dirstate file when non-interesting changes have
+  occurred. This can significantly improve 'bzr status' times when there
+  are only small changes to a large tree.
+  (Ian Clatworthy, John Arbash Meinel, #380202)
+
+* Lazy hooks are now reset between test runs. (Jelmer Vernooij, #745566)
+
+* ``bzrlib.merge.Merge`` now calls ``iter_changes`` without
+  ``include_unversioned=True``. This makes it significantly faster in many
+  cases, because it only looks at modified files, rather than building
+  information about all files. This can cause failures in other
+  TreeTransform code, because it had been expecting to know the names of
+  things which had not changed (such as parent directories). All cases we
+  know about so far have been fixed, but there may be fallout for edge
+  cases that we are missing. (John Arbash Meinel, #759091)
+
+* ``SFTPTransport`` is more pro-active about closing file-handles. This
+  reduces the chance of having threads fail from async requests while
+  running the test suite. (John Arbash Meinel, #656170)
+
+* Standalone bzr.exe installation on Windows: user can put additional python 
+  libraries into ``site-packages`` subdirectory of the installation directory,
+  this might be required for "installing" extra dependencies for some plugins.
+  (Alexander Belchenko, #743256)
+
+* ``transform.revert()`` has been updated to use
+  ``wt.iter_changes(basis_tree)`` rather than
+  ``basis_tree.iter_changes(wt)``. This allows the optimized code path to
+  kick in, improving ``bzr revert`` times significantly (33s to 4s on
+  large trees, 0.7s to 0.3s on small trees.) (John Arbash Meinel, #759096)
+
+* ``TreeTransform.create_file/new_file`` can now take an optional ``sha1``
+  parameter. If supplied, when the transform is applied, it will then call
+  ``self._tree._observed_sha1`` for those files. This lets us update the
+  hash-cache for content that we create, preventing us from re-reading the
+  content in the next ``bzr status``.  (John Arbash Meinel, #740932)
+
+Documentation
+*************
+
+* Added a section about using a shared SSH account on a server for bzr+ssh
+  access.  (Russell Smith)
+
+* The documentation now recommends using SSH rather than SFTP in the
+  tutorials and the examples, because that will generally be much faster
+  and better in cases where it can be used.  SFTP is still available and
+  mentioned as an alternative.  (Martin Pool, #636712)
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+* ``Branch.update_revisions`` has been made private and should no
+  longer be used by external users. Use ``Branch.pull`` or ``Branch.push``
+  instead. (Jelmer Vernooij, #771765)
+
+* Commands now have an `invoked_as` attribute, showing the name under
+  which they were called before alias expansion.
+  (Martin Pool)
+
+* ``Hooks.create_hook`` is now deprecated in favour of ``Hooks.add_hook``.
+  (Jelmer Vernooij)
+
+* If you call `bzrlib.initialize` but forget to enter the resulting object
+  as a context manager, bzrlib will now be initialized anyhow.
+  (Previously simple programs calling bzrlib might find the library was
+  mysteriously silent.)
+  (Martin Pool)
+
+* Inventory-specific functionality has been split out of ``Tree`` into
+  a new ``InventoryTree`` class. Tree instances no longer
+  necessarily provide an ``inventory`` attribute. (Jelmer Vernooij)
+
+* Inventory-specific functionality has been split out of ``RevisionTree``
+  into a new ``InventoryRevisionTree`` class. RevisionTree instances no
+  longer necessarily provide an ``inventory`` attribute. (Jelmer Vernooij)
+
+* New method ``Hooks.uninstall_named_hook``. (Jelmer Vernooij, #301472)
+
+* ``revision_graph_can_have_wrong_parents`` is now an attribute
+  on ``RepositoryFormat`` rather than a method on ``Repository``.
+  (Jelmer Vernooij)
+
+* ``Testament`` now takes a ``tree`` rather than an
+  ``inventory``. (Jelmer Vernooij, #762608)
+
+* ``TestCase.failUnlessExists`` and ``failIfExists`` are deprecated in
+  favour of ``assertPathExists`` and ``assertPathDoesNotExist`` 
+  respectively.
+  (Martin Pool)
+
+* The ``revno`` parameter of ``log.LogRevision`` may now be None,
+  representing a revision which is not in the current branch.
+  (Matt Giuca, #241998)
+
+* The various knit pack repository format classes have been moved
+  from ``bzrlib.repofmt.pack_repo`` to
+  ``bzrlib.repofmt.knitpack_repo``. (Jelmer Vernooij)
+
+* ``RevisionTree`` now has a new method ``get_file_revision``.
+  (Jelmer Vernooij)
+
+* ``WorkingTree`` no longer provides an ``inventory``. Instead,
+  all inventory-related functionality is now on the subclass
+  ``InventoryWorkingTree`` that all native Bazaar working tree
+  implementations derive from. (Jelmer Vernooij)
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+* Added ``osutils.lstat`` and ``osutils.fstat``. These are just the ``os``
+  functions on Linux, but they are wrapped on Windows so that fstat
+  matches lstat results across all python versions.
+  (John Arbash Meinel)
+
+* ``WorkingTree._observed_sha1`` also updates the 'size' column. It
+  happened to be updated as a side-effect of commit, but if we start using
+  the function elsewhere we might as well do it directly.
+  (John Arbash Meinel)
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* Stop using `failIf`, `failUnless`, `failIfEqual`, etc, that give
+  `PendingDeprecationWarnings` on Python2.7. 
+  (Martin Pool, #760435)
+
+
+bzr 2.4b1
+#########
+
+:2.4b1: 2011-03-17
+
+This is the first beta of the 2.4 series, leading up to a 2.4.0
+release in August 2011.  Beta releases are suitable for everyday use
+but may cause some incompatibilities with plugins.  Some plugins may need
+small updates to work with 2.4b1.
+
+External Compatibility Breaks
+*****************************
+
+(none)
+
+New Features
+************
+
+* Added ``changelog_merge`` plugin for merging changes to ``Changelog`` files
+  in GNU format.  See ``bzr help changelog_merge`` for details.
+  (Andrew Bennetts)
+  
+* Configuration options can now use references to other options in the same
+  file by enclosing them with curly brackets (``{other_opt}``). This makes it
+  possible to use, for example,
+  ``push_location=lp:~vila/bzr/config-{nickname}`` in ``branch.conf`` when
+  using a loom. During the beta period, the default behaviour is to disable
+  this feature. It can be activated by declaring ``bzr.config.expand = True``
+  in ``bazaar.conf``. (Vincent Ladeuil)
+
+* External merge tools can now be configured in bazaar.conf. See
+  ``bzr help configuration`` for more information.  (Gordon Tyler, #489915)
+
+* The ``lp:`` directory service now supports Launchpad's QA staging.
+  (Jelmer Vernooij, #667483)
+
+Improvements
+************
+
+* A new hidden command ``bzr repair-workingtree``. This is a way to force
+  the dirstate file to be rebuilt, rather than using a ``bzr checkout``
+  workaround. (John Arbash Meinel)
+
+* Added a ``Branch.heads_to_fetch`` RPC to the smart server protocol.
+  This allows formats from plugins (such as looms) to efficiently tell the
+  client which revisions need to be fetched.  (Andrew Bennetts)
+
+* Branching, merging and pulling a branch now copies revisions named in
+  tags, not just the tag metadata.  (Andrew Bennetts, #309682)
+
+* ``bzr cat-revision`` no longer requires a working tree.
+  (Jelmer Vernooij, #704405)
+
+* ``bzr export --per-file-timestamps`` for .tar.gz files will now
+  override the mtime for trees exported on Python 2.7 and later, which
+  expose the 'mtime' field in gzip files. This makes the output of
+  ``bzr export --per-file-timestamps`` for a particular tree
+  deterministic.  (Jelmer Vernooij, #711226)
+
+* ``bzr export --format=zip`` can now export to standard output,
+  like the other exporters can. (Jelmer Vernooij, #513752)
+
+* ``bzr export`` can now create ``.tar.xz`` and ``.tar.lzma`` files.
+  (Jelmer Vernooij, #551714)
+
+* Getting all entries from ``CHKInventory.iter_entries_by_dir()`` has been
+  sped up dramatically for large trees. Iterating by dir is not the best
+  way to load data from a CHK inventory, so it preloads all the items in
+  the correct order. (With the gcc-tree, this changes it (re)reading 8GB
+  of CHK data, down to just 150MB.) This has noticeable affects for things
+  like building checkouts, etc.  (John Arbash Meinel, #737234)
+
+Bug Fixes
+*********
+
+* A MemoryError thrown on the server during a remote operation will now be
+  usefully reported, and other unexpected errors will include the class name.
+  (Martin [gz], #722416)
+
+* ``bzr annotate -r-1 file`` will now properly annotate a deleted file.
+  (Andrew King, #537442)
+
+* ``bzr export`` to zip files will now set a mode on directories.
+  (Jelmer Vernooij, #207253)
+
+* ``bzr export`` to tgz files will only write out the basename of the
+  tarfile to the gzip file. (Jelmer Vernooij, #102234)
+
+* ``bzr push --overwrite`` with an older revision specified will now correctly
+  roll back the target branch. (Jelmer Vernooij, #386576)
+
+* ``bzr lp-propose`` can now propose merges against packaging branches on
+  Launchpad without requiring the target branch to be specified.
+  (Jelmer Vernooij, #704647)
+
+* ``bzr lp-propose`` no longer requires a reviewer to be specified. It will
+  instead leave setting the reviewer up to Launchpad if it was not specified.
+  (Jelmer Vernooij, #583772)
+
+* ``bzr pull`` will now exit with exit code 1 if there were tag conflicts.
+  (Jelmer Vernooij, #213185)
+
+* ``bzr mv`` user errors no longer throw UnicodeEncodeError with non-ascii
+  paths, however they may still print junk if not on a UTF-8 terminal.
+  (Martin [gz], #707954)
+
+* ``bzr reconfigure --unstacked`` now copies revisions (and their
+  ancestors) named in tags into the unstacked repository, not just the
+  ancestry of the branch's tip.  (Andrew Bennetts, #401646)
+
+* ``bzr serve`` no longer crashes when a server_started hook is installed and
+  IPv6 support is available on the system. (Jelmer Vernooij, #293697)
+
+* ``bzr status`` will not rewrite the dirstate file if it only has
+  'trivial' changes. (Currently limited to dir updates and newly-added
+  files changing state.) This saves a bit of time for regular operations.
+  eg. ``bzr status`` in a 100k tree takes 1.4s to compute the status, but 1s
+  to re-save the dirstate file. (John Arbash Meinel, #765881)
+
+* ``bzr tags`` will no longer choke on branches with ghost revisions in
+  their mainline and tags on revisions not in the branch ancestry. 
+  (Jelmer Vernooij, #397556)
+
+* ``bzr whoami`` will now display an error if both a new identity and
+  ``--email`` were specified. (Jelmer Vernooij, #680449)
+
+* ``launchpadlib`` doesn't provide the ``uris`` module in some old versions.
+  (Vincent Ladeuil, #706835)
+
+* Empty entries in the ``NO_PROXY`` variable are no longer treated as matching
+  every host.
+  (Martin Pool, #586341)
+
+* Plugins incompatible with the current version of bzr no longer produce a
+  warning on every command invocation.  Instead, a message is shown by
+  ``bzr plugins`` and in crash reports.
+  (#704195, Martin Pool)
+
+* The "pretty" version of ``needs_read_lock`` and ``needs_write_lock`` now
+  preserves the identity of default parameter values.
+  (Andrew Bennetts, #718569)
+
+* ``bzr dump-btree --raw`` no longer tracebacks on a B-Tree file
+  containing no rows. (Eric Siegerman, #715508)
+
+* Fix ``bzr lp-mirror`` to work on command line branch URLs and branches
+  without an explicit public location. (Max Bowsher)
+
+* On Python 2.6 and higher, use multiprocessing.cpu_count() to retrieve the
+  number of available processors. (Jelmer Vernooij, #693140)
+
+API Changes
+***********
+
+* Added ``Branch.heads_to_fetch`` method.  Implementations of the Branch API
+  must now inherit or implement this method.  (Andrew Bennetts, #721328)
+  
+* Added ``bzrlib.mergetools`` module with helper functions for working with
+  the list of external merge tools. (Gordon Tyler, #489915)
+
+* All methods and arguments that were deprecated before 2.0
+  have been removed. (Jelmer Vernooij)
+
+* Branch formats should now be registered on the format registry
+  (``bzrlib.branch.format_registry``) rather than using the class
+  methods on ``BranchFormat``. (Jelmer Vernooij, #714729)
+
+* ``Branch.set_revision_history`` is now deprecated.
+  (Jelmer Vernooij)
+
+* ``BranchFormat.supports_leaving_lock()`` and
+  ``RepositoryFormat.supports_leaving_lock`` flags have been added.
+  (Jelmer Vernooij)
+
+* ``Branch.fetch`` implementations must now accept an optional
+  ``fetch_tags`` keyword argument. (Andrew Bennetts)
+
+* ``Branch.import_last_revision_info`` is deprecated.  Use the
+  ``import_last_revision_info_and_tags`` method instead.
+  (Andrew Bennetts)
+
+* Because it was too specific to BzrDir implementations,
+  ``ControlDir.sprout`` no longer has a default implementation; it now
+  raises ``NotImplementedError``. (Jelmer Vernooij, #717937)
+
+* ``bzrlib.deprecated_graph`` has been removed. ``bzrlib.graph``
+  scales better tree and should be used instead.
+  (Jelmer Vernooij, #733612)
+
+* ``ControlDirFormat.register_format`` has been removed. Instead,
+  ``Prober`` implementations should now implement a ``known_formats``
+  method. (Jelmer Vernooij)
+
+* ControlDirFormats can now provide a ``check_status`` method and
+  raise a custom exception or warning when an unsupported or deprecated
+  format is being opened.  (Jelmer Vernooij, #731311)
+
+* ``bzrlib.revionspec.dwim_revspecs`` is deprecated.
+  Use ``bzrlib.revisionspec.RevisionSpec_dwim.append_possible_revspec`` and
+  ``bzrlib.revisionspec.RevisionSpec_dwim.append_possible_lazy_revspec``
+  instead.  (Jelmer Vernooij, #721971)
+
+* ``BzrDirFormat`` has a new attribute ``fixed_components`` that
+  indicates whether the components of the bzrdir can be upgraded
+  independent of the ``BzrDir``. (Jelmer Vernooij)
+
+* ``BzrProber.register_format`` and ``BzrProber.unregister_format`` are
+  now deprecated in favour of the ``BzrProber.formats`` format registry.
+  (Jelmer Vernooij)
+
+* ``ControlDir`` implementations no longer have to provide the
+  ``get_branch_transport``, ``get_workingtree_transport`` and
+  ``get_repository_transport`` methods.  (Jelmer Vernooij, #730325)
+
+* ``Converter`` has been moved from ``bzrlib.bzrdir`` to
+  ``bzrlib.controldir``. (Jelmer Vernooij)
+
+* Repository formats can now provide
+  ``_get_extra_interrepo_test_combinations`` in the same module 
+  to provide extra test combinations for ``bzrlib.tests.per_repository``.
+  (Jelmer Vernooij)
+
+* Repository formats should now be registered on the format registry
+  (``bzrlib.repository.format_registry``) rather than using the class
+  methods on ``RepositoryFormat``. (Jelmer Vernooij)
+
+* Repository formats can now indicate they do not support the full
+  VersionedFiles API by setting the ``supports_full_versioned_files``
+  attribute to False. A subset of the VersionedFiles API
+  (signatures and text graphs) still needs to be supported.
+  (Jelmer Vernooij)
+
+* Repository formats have a new method ``is_deprecated`` that
+  implementations can override to return True to trigger a deprecation
+  warning. (Jelmer Vernooij)
+
+* The ``revision_id`` parameter of
+  ``Repository.search_missing_revision_ids`` and
+  ``InterRepository.search_missing_revision_ids`` is deprecated.  It is
+  replaced by the ``revision_ids`` parameter.  (Andrew Bennetts)
+
+* Working tree formats should now be registered on the format registry
+  (``bzrlib.working_tree.format_registry``) rather than using the class
+  methods on ``WorkingTreeFormat``. (Jelmer Vernooij, #714730)
+
+* Exporting may now be done with a generator
+  (``bzrlib.export.get_export_generator``) (Geoff/xaav, #791005)
+
+Internals
+*********
+
+* ``CatchingExceptionThread`` (formerly ThreadWithException) has been moved
+  out of the ``bzrlib.tests`` hierarchy to make it clearer that it can be used
+  outside of tests. This class makes it easier to track exceptions in threads
+  by catching them so they can be re-raised in the controlling thread. It's
+  available in the ``bzrlib.cethread`` module.  (Vincent Ladeuil)
+
+* Correctly propagate malloc failures from diff-delta.c code as MemoryError
+  so OOM conditions during groupcompress are clearly reported. This entailed a
+  change to several function signatures. (Martin [gz], #633336)
+
+* ``HookPoint.lazy_hook`` and ``Hooks.install_named_lazy_hook`` can install 
+  hooks for which the callable is loaded lazily.  (Jelmer Vernooij)
+
+Testing
+*******
+
+* The Range parsing for HTTP requests will correctly parse incomplete ranges.
+  (Vincent Ladeuil, #731240)
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2011-10-14 13:56:45 +0000
@@ -0,0 +1,536 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr 2.5b3
+#########
+
+:2.5b3: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+* The ``log_format`` configuration can be used with ``-Olog_format=line`` to
+  change the format ``push`` and ``pull`` use to display the
+  revisions. I.e.: ``bzr pull -v -Olog_format=short`` will use the ``short``
+  format instead of the default ``long`` one. (Vincent Ladeuil, #861472)
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* ``bzr version-info`` now takes a ``--revision`` argument.
+  (Jelmer Vernooij, #238705)
+
+* ``bzr revno`` now takes a ``--revision`` argument.
+  (Jelmer Vernooij, #870649)
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* ``bzr mkdir --quiet`` now does not print a line for every created
+  directory. (Martin von Gagern, #869915)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+* ``Branch.revision_history`` is now deprecated. (Jelmer Vernooij, #799519)
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+* ``ControlDirFormat`` now has a new method ``supports_transport``
+  which format implementations can use whether or not they can access
+  a control dir over a particular transport. (Jelmer Vernooij)
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+*  Ensure TestCase instances are deallocated immediately after running where
+   possible. This greatly reduces the peak resource needs of a full test suite
+   run. The new ``-Euncollected_cases`` selftest flag will add failures if any
+   case which persists pasts its expected lifetime. (Martin Packman, #613247)
+
+
+bzr 2.5b2
+#########
+
+This is the second beta of the 2.5 series, leading to a 2.5.0 release in
+February 2012. Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.
+
+This release includes more filtering options for ``bzr log``, idle
+connections handling for ``bzr serve``, a ``development-colo`` experimental
+format to flesh out the colocated branches UI, better support for foreign
+formats, enhancements to the config framework and more.
+
+This release includes all bug fixed in previous series known at the time of
+this release.
+
+:2.5b2: 2011-10-06
+
+External Compatibility Breaks
+*****************************
+
+None
+
+New Features
+************
+
+* A new ``-O`` standard option (common to all commands) have been added. It
+  provides a value for a config option in the ``-Oname=value`` form that
+  takes precedence over all definitions found in config files.  It can be
+  used multiple times to override different options.
+  (Vincent Ladeuil, #491196)
+
+* ``bzr log`` now has an option called ``--omit-merges`` to omit
+  those commits that merged branches, i.e. those having more than one
+  parent.
+  In order to avoid confusion, the previous command line option
+  ``--include-merges`` has been renamed to ``--include-merged``.
+  The old name of the command line option will still be accepted.
+  The name change also affects ``bzr missing``.
+  (Martin von Gagern)
+
+* ``bzr serve`` will now disconnect clients if they have not issued an RPC
+  request after 5minutes. On POSIX platforms, this will also happen for
+  ``bzr serve --inet``. This can be overridden with the configuration
+  variable ``serve.client_timeout`` or in the command line parameter
+  ``bzr serve --client-timeout=X``. Further, it is possible to request
+  ``bzr serve [--inet]`` to shutdown gracefully by sending SIGHUP. It will
+  finish the current request, and then close the connection.
+  (John Arbash Meinel, #824797, #795025)
+
+* The new experimental format ``development-colo`` supports colocated
+  branches. This format will eventually be merged back into the ``2a``
+  format when it has stabilized and there is adequate UI support for
+  colocated branches.
+  (Jelmer Vernooij, #831481)
+
+Improvements
+************
+
+* Fixed a bug where ``bzr tags -r x..y`` loaded the branch history once for
+  every revision in the range; it's now much faster. (Vincent Ladeuil, #857335)
+
+* ``bzr info -v`` can now be run against branches that don't support
+  ``last_revision_info``, in which case the branch information will simply
+  not be displayed. (Jelmer Vernooij)
+
+Bug Fixes
+*********
+
+* ``bzr shelve`` can now be used in emacs shells as the input handling is
+  turned into a line-based one when ``INSIDE_EMACS`` is set (which is the
+  case for all recent emacs versions). (Vincent Ladeuil, #856261)
+
+* ``bzr tags`` can now be used against remote repositories that do
+  not provide access to the revision graph. (Jelmer Vernooij, #858942)
+
+* ``bzr update PATH`` will stop if you seem to be asking it to update
+  anything less than a whole tree, because that's not supported by ``bzr``'s
+  concept that the whole tree has a single basis revision.  Previously, it
+  would go ahead and update the whole tree, which was surprising.
+  (Martin Pool, #557886)
+
+* Don't crash if ``bzrlib.initialize()`` has not been called while accessing
+  configs.  (Vincent Ladeuil, #863401)
+
+* Redirects between http and https no longer discard path information
+  in some cases. (Jelmer Vernooij, #853765)
+
+* The ``--overwrite`` argument to ``bzr push`` and ``bzr pull`` no longer
+  reports all tags as changed. (Jelmer Vernooij, #845396)
+
+* ``WorkingTree.get_file_mtime`` now raises NoSuchId if a file id is
+  specified that is unknown. (Jelmer Vernooij, #847435)
+
+
+API Changes
+***********
+
+* ``Branch.get_revision_delta`` has been deprecated. Use
+  ``Repository.get_revision_delta`` instead. (Jelmer Vernooij, #859712)
+
+* Plugins that implement custom protocols for ``bzr serve`` should now
+  also take an argument ``timeout``. This is used by the the bzr protocol
+  to close a connection if a client has been idle for more than X seconds.
+  (Default 5minutes). (John Arbash Meinel)
+
+* ``Repository.fileids_altered_by_revision_ids`` has been moved to
+  ``VersionedFileRepository`` and is no longer part of the standard
+  ``Repository`` interface. (Jelmer Vernooij)
+
+* The argument ``include_merges`` to ``missing.find_unmerged`` has
+  been renamed to ``include_merged``. The old name is still supported
+  for now but will cause a deprecation warning. (Martin von Gagern)
+
+* The new method ``ControlDirFormat.is_initializable()`` returns a boolean
+  indicating whether or not it is possible to use any of the
+  initialization methods of that format to create a new control dir.
+  (Jelmer Vernooij)
+
+Internals
+*********
+
+* ``Branch`` objects can now use a config stack with the newly introduced
+  ``get_config_stack()``. Both ``get_config`` and ``get_config_stack`` can
+  be used for the same branch but it's recommended to stick to one for a
+  given option.
+
+Testing
+*******
+
+* Test scripts can now use ``bzr shelve`` and provide their input as
+  complete lines. (Vincent Ladeuil, #856261)
+
+* Really corrupt the pack file without depending on a special length or value.
+  (Vincent Ladeuil, #807032)
+
+
+bzr 2.5b1
+#########
+
+:2.5b1: 2011-09-15
+
+This is the first beta of the 2.5 series, leading up to a 2.5.0
+release in February 2012.
+
+This release includes better support for gpg signing, better support for
+i18n (mostly command help and error messages), more options to filter ``bzr
+log`` output, more support for colocated branches ("location,branch=XXX"
+syntax), better feedback on updated tags for various commands, faster
+branching into an empty repository, enhancements to the config framework and
+more.
+
+Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.  Some plugins may need small updates to work
+with 2.5b1.
+
+External Compatibility Breaks
+*****************************
+
+None
+
+New Features
+************
+
+* A ``from_unicode`` parameter can be specified when registering a config
+  option. This implements boolean, integer and list config options when the
+  provided ``bool_from_store``, ``int_from_store`` and ``list_from_store``
+  are used for this parameter.  (Vincent Ladeuil)
+
+* Accessing a packaging branch on Launchpad (eg, ``lp:ubuntu/bzr``) now
+  checks to see if the most recent published source package version for
+  that project is present in the branch tags. This should help developers
+  trust whether the packaging branch is up-to-date and can be used for new
+  changes. The level of verbosity is controlled by the config item
+  ``launchpad.packaging_verbosity``. It can be set to one of
+
+  off
+    disable all checks
+
+
+  minimal
+    only display if the branch is out-of-date
+
+  short
+    also display single-line up-to-date and missing,
+
+
+  all
+    (default) display multi-line content for all states
+
+
+  (John Arbash Meinel, #609187, #812928)
+
+* Add a config option gpg_signing_key for setting which GPG key should
+  be used to sign commits. Also default to using the gpg user identity
+  which matches user_email() as set by whoami.
+  (Jonathan Riddell, #68501)
+
+* An ``invalid`` parameter can be specified when registering a config option
+  to decide what should be done when invalid values are
+  encountered. 'warning' and 'error' will respectively emit a warning and
+  ignore the value or errors out. (Vincent Ladeuil)
+
+* bzr add now skips large files in recursive mode. The default "large"
+  size is 20MB, and is configurable via the add.maximum_file_size
+  option. A value of 0 disables skipping. Named items passed to add are
+  never skipped. (Shannon Weyrick, #54624)
+
+* ``bzr help configuration/<option>`` display the help for ``option`` for
+  all registered configuration options. (Vincent Ladeuil, #747050)
+
+* ``bzr log -m`` now matches message, author, committer and bugs instead
+  of just matching the message.  ``--message`` keeps its original meaning,
+  while ``--match-message, --match-author, --match-committer`` and
+  ``--match-bugs`` match each of those fields. (Jacek Sieka)
+
+* ``config.Option`` can now declare ``default_from_env``, a list of
+  environment variables to get a default value from. (Vincent Ladeuil)
+
+* ``config.NameMatcher`` can be used to implement config stores and stacks
+  that need to provide specific option values for arbitrary unique IDs (svn
+  repository UUIDs, etc).  (Vincent Ladeuil, #843638)
+
+* New builtin ``bzr branches`` command, which lists all colocated branches
+  in a directory. (Jelmer Vernooij, #826820)
+
+* Relative local paths can now be specified in URL syntax by using the
+  "file:" prefix.  (Jelmer Vernooij)
+
+* Report commits signed with expired keys in ``verify-signatures``.
+  (Jonathan Riddell, #804254)
+
+* Translations are now enabled for command help, errors and globally
+  for any message using ``gettext`` given on output.  (Jonathan Riddell,
+  INADA Naoki, #83941)
+
+Improvements
+************
+
+* ``bzr add`` will now warn about nested subtrees that are skipped.
+  (Jelmer Vernooij, #187342)
+
+* ``bzr commit -m ''`` can now be used to force an empty commit message.
+  Entering an empty commit message in the message editor still triggers
+  an error. (Jelmer Vernooij)
+
+* ``bzr pull`` will now mention how many tags it has updated.
+  (Jelmer Vernooij, #164450)
+
+* ``bzr tag`` no longer errors if a tag already exists but refers to the
+  same revision, and will mention when a tag has been updated
+  rather than created. (Jelmer Vernooij, #381203)
+
+* ``bzr uncommit`` will now remove tags that refer to removed revisions.
+  The ``--keep-tags`` option can be used to prevent this behaviour.
+  (Jelmer Vernooij, #605814)
+
+* Do not run i18n initialisation twice. (Jonathan Riddell)
+
+* Install translation .mo files. (Jonathan Riddell)
+
+* Locations printed by ``bzr upgrade`` are now formatted before display.
+  (Jelmer Vernooij)
+
+* ``Repository.get_parent_map`` now estimates the size of the returned
+  content more accurately. This means that we get closer to the desired
+  64kB/request. For repositories converted from svn, this can be an
+  improvement of approx 5:1 in round trips to discover the whole history.
+  (John Arbash Meinel)
+
+* Support a ``bugtracker`` option which is used by ``bzr commit --fixes``
+  if no bug tracker was specified on the command line.
+  (Jelmer Vernooij, #334860)
+
+* Use ``gettext.NullTranslations`` in i18n to allow use of i18n even when
+  translations are not turned on. (Jonathan Riddell)
+
+Bug Fixes
+*********
+
+* ``bzr commit`` now correctly reports missing files as "removed", not
+  "modified". (Jelmer Vernooij, #553955)
+
+* ``bzr reconfigure`` will now allow multiple non-conflicting requests
+  in a single invocation, e.g. ``--branch`` and ``--use-shared``.
+  (Martin von Gagern, #842993)
+
+* A call to CHKInventory's filter-method will not result in a
+  DuplicateFileId error, if you move a subfolder and change a file in
+  that subfolder.
+  (Bastian Bowe, #809901)
+
+* Branching from a stacked branch no longer does a ``get_parent_map``
+  request for each revisions that is in the stacked-on repository while
+  determining what revisions need to be fetched. This mostly impacts
+  branching initialy into an empty shared repository when the source is
+  not the development focus.  (John Arbash Meinel, #388269)
+
+* Decode ``BZR_HOME`` with fs encoding on posix platforms to avoid unicode
+  errors.  (Vincent Ladeuil, #822571)
+
+* Fix fallout from URL handling changes in 2.5 that caused an IndexError to be
+  raised whenever a transport at the drive root was opened on windows.
+  (Martin [gz], #841322)
+
+* Fixed loading of external merge tools from config to properly decode
+  command-lines which contain embedded quotes. (Gordon Tyler, #828803)
+
+* Rather than an error being raised, a warning is now printed when the
+  current user does not have permission to read a configuration file.
+  (Jelmer Vernooij, #837324)
+
+* The pull command will now always use separate connections for the
+  case where the destination is a heavyweight checkout of some remote
+  branch on the same host as the source branch.
+  (Martin von Gagern, #483661)
+
+* TreeTransformBase.fixup_new_roots no longer forces trees to have a root, so
+  operations that use it, like merge, can now create trees without a root.
+  (Aaron Bentley)
+
+Documentation
+*************
+
+* Release instructions refreshed. (Vincent Ladeuil)
+
+API Changes
+***********
+
+* ``BranchFormat.initialize`` now takes a ``append_revisions_only``
+  argument. (Jelmer Vernooij)
+
+* ``Branch._get_checkout_format`` now takes a ``lightweight`` argument
+  which indicates if the format should be for a lightweight or a
+  heavyweight checkout. (Jelmer Vernooij)
+
+* ``ControlDir.create_branch`` now takes a ``append_revisions_only`` argument.
+  (Jelmer Vernooij)
+
+* New class ``URL`` in ``bzrlib.utils`` for managing parsed URLs.
+  (Jelmer Vernooij)
+
+* New method ``Config.get_user_option_as_int_from_SI`` added for expanding a
+  value in SI format (i.e. "20MB", "1GB") into its integer equivalent. 
+  (Shannon Weyrick)
+
+* New method ``InterTree.file_content_matches`` which checks that
+  two files in different trees have the same contents.
+  (Jelmer Vernooij)
+
+* New method ``Tree.get_file_verifier`` which allows tree implementations
+  to return non-sha1 checksums to verify files.
+  (Jelmer Vernooij, #720831)
+
+* New methods ``get_transport_from_path`` and ``get_transport_from_url``
+  have been added that only support opening from a path or a URL,
+  unlike ``get_transport``. (Jelmer Vernooij)
+
+* New registry ``OptionRegistry`` specialized for configuration options.
+  (Vincent Ladeuil)
+
+* Remove ``AtomicFile.closed`` which has been deprecated in bzr 0.10.
+  (Vincent Ladeuil)
+
+* Remove ``commands._builtin_commands``, ``commands.shlex_split_unicode``,
+  ``Command._maybe_expand_globs`` and ``Command.run_direct`` deprecated in
+  2.10 and 2.2.0. (Vincent Ladeuil)
+
+* Remove ``diff.get_trees_and_branches_to_diff`` deprecated in 2.2.0.
+
+* Remove ``log.calculate_view_revisions``, ``log._filter_revision_range``,
+  ``log.get_view_revisions`` which have been deprecated in bzr 2.1.0. Also
+  remove ``log.show_one_log`` which was never properly deprecated but wasn't
+  used and is easy to inline if needed. (Vincent Ladeuil)
+
+* Remove ``trace.info``, ``trace.error`` and ``trace.show_log_error``
+  deprecated in 2.1.0. (Vincent Ladeuil)
+
+* Remove ``TransportListRegistry.set_default_transport``, as the concept of
+  a default transport is currently unused. (Jelmer Vernooij)
+
+* Remove ``UIFactory.warn_cross_format_fetch`` and
+  ``UIFactory.warn_experimental_format_fetch`` in favor of
+  ``UIFactory.show_user_warning``. (Jelmer Vernooij)
+
+* ``Tags`` containers can now declare whether they support versioned
+  tags and whether tags can refer to ghost tags.
+  (Jelmer Vernooij)
+
+* ``Tags.merge_to`` now returns a dictionary with the updated tags
+  and a set of conflicts, rather than just conflicts. (Jelmer Vernooij)
+
+* There is a new class `ContentFilterTree` that provides a facade for 
+  content filtering.  The `filtered` parameter to `export` is deprecated 
+  in favor of passing a filtered tree, and the specific exporter plugins
+  no longer support it.
+  (Martin Pool)
+
+* ``Transport`` now has a ``_parsed_url`` attribute instead of
+  separate ``_user``, ``_password``, ``_port``, ``_scheme``, ``_host``
+  and ``_path`` attributes. Proxies are provided for the moment but
+  may be removed in the future. (Jelmer Vernooij)
+
+Internals
+*********
+
+* A new debug flag ``hpss_client_no_vfs`` will now cause the HPSS client
+  to raise a ``HpssVfsRequestNotAllowed`` exception when a VFS request
+  is attempted. (Jelmer Vernooij)
+
+* New method ``ControlDir._get_selected_branch`` which returns the
+  colocated branch selected using path segment parameters.
+  (Jelmer Vernooij, #380871)
+
+Testing
+*******
+
+* Blackbox tests (including test scripts) can be debugged interactively (see
+  bzrlib.debug.BzrPdb for details). (Vincent Ladeuil)
+
+* `BranchBuilder.build_snapshot` now supports a "flush" action.  This
+  cleanly and reliably allows tests using `BranchBuilder` to construct
+  branches that e.g. rename files out of a directory and unversion that
+  directory in the same revision.  Previously some changes were impossible
+  due to the order that `build_snapshot` performs its actions.
+  (Andrew Bennetts)
+
+* Don't require ``os.fdatasync`` to be defined on all supported OSes
+  (BSD-based OSes don't define it).  (Vincent Ladeuil, #822649)
+
+* Fix compatibility with testtools 0.9.12. (Jelmer Vernooij, #815423)
+
+* ``LockDir`` can now be run when the local hostname is ``localhost``.
+  (Jelmer Vernooij, #825994)
+
+* ``ModuleAvailableFeature`` won't try to import already imported modules,
+  allowing it to be used for modules with side-effects.
+  (Vincent Ladeuil, #712474)
+
+* Output time stamps while running ``make check`` to get better timings from
+  pqm.  (Vincent Ladeuil, #837926)
+
+* `TestCaseWithMemoryTransport` is faster now: `_check_safety_net` now
+  just compares the bytes in the dirstate file to its pristine state,
+  rather than opening the WorkingTree and calling ``last_revision()``.
+  This reduces the overall test suite time by about 10% on my laptop.
+  (Andrew Bennetts)
+
+* Update `TestCase.knownFailure` to the testtools way of handling expected
+  failures to resolve Python 2.7 incompatibility. (Martin [gz], #607400)
+
+..
+   vim: tw=74 ft=rst ff=unix

=== renamed file 'NEWS-template.txt' => 'doc/en/release-notes/release-template.txt'
--- NEWS-template.txt	2010-02-19 07:13:45 +0000
+++ doc/en/release-notes/release-template.txt	2010-10-13 08:08:58 +0000
@@ -4,28 +4,50 @@
 :Codename: Nirvana
 :2.x.y: NOT RELEASED YET
 
-Compatibility Breaks
-********************
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
 
 New Features
 ************
 
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
 Bug Fixes
 *********
 
-Improvements
-************
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
 
 Documentation
 *************
 
+.. Improved or updated documentation.
+
 API Changes
 ***********
 
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
 Internals
 *********
 
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
 Testing
 *******
 
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
 

=== added file 'doc/en/release-notes/series-template.txt'
--- doc/en/release-notes/series-template.txt	1970-01-01 00:00:00 +0000
+++ doc/en/release-notes/series-template.txt	2010-10-13 08:08:58 +0000
@@ -0,0 +1,62 @@
+####################
+Bazaar Release Notes
+####################
+
+.. toctree::
+   :maxdepth: 1
+
+bzr ?.?.?
+#########
+
+:Codename: Nirvana
+:2.x.y: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+
+..
+   vim: tw=74 ft=rst ff=unix

=== modified file 'doc/en/tutorials/centralized_workflow.txt'
--- doc/en/tutorials/centralized_workflow.txt	2009-12-03 23:21:16 +0000
+++ doc/en/tutorials/centralized_workflow.txt	2011-04-06 04:58:43 +0000
@@ -21,7 +21,7 @@
 in sync.  This workflow is also applicable to a single developer working
 on several different machines.
 
-.. _Bazaar: http://bazaar-vcs.org
+.. _Bazaar: http://bazaar.canonical.com
 
 
 Initial Setup
@@ -80,7 +80,7 @@
 
 ::
 
-  % bzr init-repo --no-trees sftp://centralhost/srv/bzr/
+  % bzr init-repo --no-trees bzr+ssh://centralhost/srv/bzr/
 
 You can think of this step as similar to setting up a new cvsroot, or
 subversion repository.  The ``--no-trees`` option tells bzr to not
@@ -88,6 +88,10 @@
 since no one will be making changes directly in the branches within
 the central repository.
 
+Here we're using a ``bzr+ssh`` URL, which means to use Bazaar's own
+protocol on top of the SSH secure shell.  See the Administrator Guide for
+information about setting up a bzr+ssh server.  
+
 
 Migrating an existing project to Bazaar
 =======================================
@@ -100,7 +104,7 @@
 document. See `Tracking Upstream`_ for some possibilities (section
 "Converting and keeping history").
 
-.. _Tracking Upstream: http://bazaar-vcs.org/TrackingUpstream
+.. _Tracking Upstream: http://wiki.bazaar.canonical.com/TrackingUpstream
 
 ..
    XXX: We really need a different document for discussing conversion of a
@@ -116,7 +120,7 @@
 
 ::
 
-  % bzr init sftp://centralhost/srv/bzr/sigil
+  % bzr init bzr+ssh://centralhost/srv/bzr/sigil
 
 This can be thought of as the "HEAD" branch in CVS terms, or as the "trunk"
 in Subversion terms. We will call this the ``dev`` branch.
@@ -133,7 +137,7 @@
   % cd work
   % mkdir sigil
   % cd sigil
-  % bzr checkout sftp://centralhost/srv/bzr/sigil dev
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil dev
   % cd dev
   % cp -ar ~/sigil/* .
   % bzr add
@@ -162,10 +166,10 @@
 To get a copy of the current development tree::
 
   % cd ~/work/sigil
-  % bzr checkout sftp://centralhost/srv/bzr/sigil dev
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil dev
 
 Now that two people both have a checkout of
-``sftp://centralhost/srv/bzr/sigil``, there will be times when one of
+``bzr+ssh://centralhost/srv/bzr/sigil``, there will be times when one of
 the checkouts will be out of date with the current version.
 At commit time, Bazaar_ will inform the user of this and prevent them from
 committing. To get up to date, use ``bzr update`` to update the
@@ -201,9 +205,9 @@
 ::
 
   % cd ~/work/sigil
-  % bzr branch sftp://centralhost/srv/bzr/sigil \
-               sftp://centralhost/srv/bzr/sigil/doodle-fixes
-  % bzr checkout sftp://centralhost/srv/bzr/sigil/doodle-fixes doodle-fixes
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil \
+               bzr+ssh://centralhost/srv/bzr/sigil/doodle-fixes
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil/doodle-fixes doodle-fixes
   % cd doodle-fixes
 
 We now have a place to make any fixes we need to ``doodle``. And we would
@@ -231,7 +235,7 @@
 
    % bzr cbranch dev my-feature-branch
 
-.. _bzrtools: http://bazaar-vcs.org/BzrTools
+.. _bzrtools: http://wiki.bazaar.canonical.com/BzrTools
 
 
 Merging changes back
@@ -272,19 +276,19 @@
 developer their own branch, and their own place to work in the central
 location. This can be done with::
 
-  % bzr branch sftp://centralhost/srv/bzr/sigil \
-               sftp://centralhost/srv/bzr/sigil/user-a
-  % bzr branch sftp://centralhost/srv/bzr/sigil \
-               sftp://centralhost/srv/bzr/sigil/user-b
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil \
+               bzr+ssh://centralhost/srv/bzr/sigil/user-a
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil \
+               bzr+ssh://centralhost/srv/bzr/sigil/user-b
 
 This gives each developer their own branch to work on. And, they can
 easily create a new feature branch for themselves with just [#cbranch]_
 ::
 
-  % bzr branch sftp://centralhost/srv/bzr/sigil/user-a \
-               sftp://centralhost/srv/bzr/sigil/user-a/feature
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil/user-a \
+               bzr+ssh://centralhost/srv/bzr/sigil/user-a/feature
   % cd ~/work/sigil
-  % bzr checkout sftp://centralhost/srv/bzr/sigil/user-a/feature myfeature
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil/user-a/feature myfeature
 
 
 Glossary

=== modified file 'doc/en/tutorials/index.txt'
--- doc/en/tutorials/index.txt	2009-07-22 13:41:01 +0000
+++ doc/en/tutorials/index.txt	2011-05-16 10:22:06 +0000
@@ -8,4 +8,4 @@
    tutorial
    using_bazaar_with_launchpad
    centralized_workflow
-
+   licence

=== added file 'doc/en/tutorials/licence.txt'
--- doc/en/tutorials/licence.txt	1970-01-01 00:00:00 +0000
+++ doc/en/tutorials/licence.txt	2011-05-16 10:22:06 +0000
@@ -0,0 +1,7 @@
+Licence
+===============
+
+Copyright 2005-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later.  See
+<http://www.gnu.org/licenses/>.

=== modified file 'doc/en/tutorials/tutorial.txt'
--- doc/en/tutorials/tutorial.txt	2009-12-04 06:21:36 +0000
+++ doc/en/tutorials/tutorial.txt	2011-04-06 04:58:43 +0000
@@ -50,10 +50,10 @@
 These revisions have other uses as well. For example, one can comment
 revisions to record what the recent set of changes meant by providing an
 optional log message. Real life log messages include things like "Fixed
-the web template to close the table" and "Added sftp suppport. Fixes #595"
+the web template to close the table" and "Added SFTP suppport. Fixes #595"
 	
 We keep these logs so that if later there is some sort of problem with
-sftp, we can figure out when the problem probably happened.
+SFTP, we can figure out when the problem probably happened.
 
 How DVCS is different
 =====================
@@ -209,10 +209,11 @@
 
 You'll usually refer to branches on your computer's filesystem just by
 giving the name of the directory containing the branch.  bzr also supports
-accessing branches over http and sftp, for example::
+accessing branches over SSH, HTTP and SFTP, amongst other things::
 
-    % bzr log http://bazaar-vcs.org/bzr/bzr.dev/
-    % bzr log sftp://bazaar-vcs.org/bzr/bzr.dev/
+    % bzr log bzr+ssh://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log sftp://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
 
 By installing bzr plugins you can also access branches using the rsync
 protocol.
@@ -535,7 +536,7 @@
 the existing branch.  Because this new copy is potentially a new branch,
 the command is called **branch**::
 
-    % bzr branch http://bazaar-vcs.org/bzr/bzr.dev
+    % bzr branch lp:bzr bzr.dev
     % cd bzr.dev
 
 This copies down the complete history of this branch, so we can do all
@@ -608,7 +609,7 @@
 
   ::
 
-    % bzr push sftp://servername.com/path/to/directory
+    % bzr push bzr+ssh://servername.com/path/to/directory
 
   (The destination directory must already exist unless the
   ``--create-prefix`` option is used.)

=== modified file 'doc/en/tutorials/using_bazaar_with_launchpad.txt'
--- doc/en/tutorials/using_bazaar_with_launchpad.txt	2010-06-02 05:18:27 +0000
+++ doc/en/tutorials/using_bazaar_with_launchpad.txt	2010-10-07 21:03:55 +0000
@@ -92,7 +92,7 @@
 ------------------------------------------
 
 As well as sponsoring `Ubuntu <http://www.ubuntu.com>`_ and
-`Bazaar <http://bazaar-vcs.org>`_ development, Canonical
+`Bazaar <http://bazaar.canonical.com>`_ development, Canonical
 provides Launchpad, <https://launchpad.net/>, as a free service
 for the community. Launchpad is one of the most
 exciting CDEs around for several notable reasons:
@@ -244,6 +244,41 @@
 quality standards.
 
 
+Package source branches
+-----------------------
+
+When `maintaining packages for Ubuntu using Bazaar`_ you can easily access the
+package's source branch on Launchpad.  The package's source branch in the
+current (default) series can be downloaded like this::
+
+  bzr branch ubuntu:package
+
+where *package* is the name of the Ubuntu package you want to access.  To
+download the package branch for a specific series in Ubuntu (e.g. Maverick or
+Lucid), use this::
+
+  bzr branch ubuntu:maverick/package
+
+Ubuntu distroseries can also be abbreviated to just their first letter.  For
+example, the above could also be written::
+
+  bzr branch ubuntu:m/package
+
+You can also download the package source branch from Launchpad for several
+Debian series.  The default series can be downloaded like this::
+
+  bzr branch debianlp:package
+
+and a specific series can be downloaded like this::
+
+  bzr branch debianlp:lenny/package
+
+Note that the ``debianlp:`` scheme access the Debian source branch for a
+package from Launchpad only.
+
+.. _`maintaining packages for Ubuntu using Bazaar`: https://wiki.ubuntu.com/DistributedDevelopment
+
+
 Linking branches using Launchpad
 ================================
 

=== modified file 'doc/en/upgrade-guide/data_migration.txt'
--- doc/en/upgrade-guide/data_migration.txt	2009-10-02 18:12:29 +0000
+++ doc/en/upgrade-guide/data_migration.txt	2010-12-20 12:01:56 +0000
@@ -113,7 +113,7 @@
 
 In Launchpad, a project can define a *development series* and associate a
 branch with that series.  The branch then becomes the *focus of development*
-and gets special treatment and a shortcut url.  By default, if anybody
+and gets special treatment and a shortcut URL.  By default, if anybody
 branches your project's focus of development and pushes changes back to
 Launchpad, their branch will be stacked on your development focus branch.
 Also, branches can be associated with other Launchpad artifacts such as bugs
@@ -202,11 +202,15 @@
    name then **merge** your changes from the matching branch in the
    old repository.
 
-The first method will give you a branch which is identical (in terms
-of revision history) to the old branch, but it's parent branch will
-be set to the old branch, not your new trunk. If you use this method,
-you'll probably want to edit your branch.conf file to update the
-parent branch setting.
+The first method will give you a branch which is identical (in terms of
+revision history) to the old branch, but it's parent branch will be set to the
+old branch, not your new trunk. If you use this method, you'll probably update
+the ``parent_location`` configuration variable in the ``branch.conf`` file
+with::
+
+    bzr config parent_location=XXX
+
+``XXX`` being the URL to your new trunk.
 
 In contrast, the second approach sets up the parent branch correctly.
 However, it isn't ideal if you're not ready to include all the latest

=== modified file 'doc/en/upgrade-guide/index.txt'
--- doc/en/upgrade-guide/index.txt	2009-09-02 16:03:51 +0000
+++ doc/en/upgrade-guide/index.txt	2011-05-16 10:22:06 +0000
@@ -1,6 +1,6 @@
-########################
-Bazaar 2.0 Upgrade Guide
-########################
+####################
+Bazaar Upgrade Guide
+####################
 
 .. Please mark sections in included files as following:
 ..   level 1 ========
@@ -11,3 +11,5 @@
 .. include:: overview.txt
 .. include:: data_migration.txt
 .. include:: tips_and_tricks.txt
+.. include:: licence.txt
+

=== added file 'doc/en/upgrade-guide/licence.txt'
--- doc/en/upgrade-guide/licence.txt	1970-01-01 00:00:00 +0000
+++ doc/en/upgrade-guide/licence.txt	2011-05-16 10:22:06 +0000
@@ -0,0 +1,7 @@
+Licence
+===============
+
+Copyright 2009-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later.  See
+<http://www.gnu.org/licenses/>.

=== modified file 'doc/en/upgrade-guide/overview.txt'
--- doc/en/upgrade-guide/overview.txt	2010-06-02 05:03:31 +0000
+++ doc/en/upgrade-guide/overview.txt	2010-12-17 10:16:48 +0000
@@ -4,7 +4,7 @@
 High level upgrade process
 --------------------------
 
-In broad terms, there are 3 steps involved in upgrading to Bazaar 2.x:
+In broad terms, there are 3 steps involved in upgrading a new format:
 
 1. Upgrade the core software
 
@@ -12,34 +12,29 @@
 
 3. Migrate data to the new default format.
 
-Bazaar 2.x supports branches in earlier formats so the third step is
-strictly not required. However, the new default format in Bazaar 2.x
-is more space efficient, faster on large projects and provides a range
-of new features, so it is recommended that most projects migrate to it
-at a convenient time.
+Bazaar supports branches in earlier formats so the third step is strictly not
+required. However, when new default formats are introduced, they are more
+space efficient, faster on large projects and/or provide new features. So it
+is recommended that most projects migrate to it at a convenient time.
 
-For most users, upgrading to 2.x and migrating to the new format is
-straight forward. For projects with a large community of developers
-though, things become more complex. In these cases, careful planning
-and good communications become essential. This document provides
-general advice which aims to assist in this regard. If in doubt,
-please contact us on our mailing list or IRC channel with any
-questions or concerns you have.
+For most users, upgrading and migrating to the new format is straight
+forward. For projects with a large community of developers though, things
+become more complex. In these cases, careful planning and good communications
+become essential. This document provides general advice which aims to assist
+in this regard. If in doubt, please contact us on our mailing list or IRC
+channel with any questions or concerns you have.
 
 
 Upgrading the core software
 ---------------------------
 
-The steps required to upgrade the core software vary from operating
-system to operating system. There is nothing special about upgrading
-from Bazaar 1.x to Bazaar 2.0 compared to upgrading from Bazaar 1.x
-to Bazaar 1.y. In either case, a brief outline of the steps is given
-below.
+The steps required to upgrade the core software vary from operating system to
+operating system.  A brief outline of the steps is given below.
 
 To upgrade Bazaar on Ubuntu:
 
-1. Ensure your package manager is configured with the required
-   software sources, e.g. the official release PPA for Ubuntu:
+1. Ensure your package manager is configured with the required software
+   sources, e.g. the official stable release PPA for Ubuntu:
    https://launchpad.net/~bzr/+archive
 
 2. Use your package manager to upgrade to the latest version.
@@ -61,7 +56,7 @@
 2. Upgrade to the latest version using **sudo port upgrade bzr**
 
 For further information on installing and upgrading, see
-http://bazaar-vcs.org/Download.
+http://wiki.bazaar.canonical.com/Download.
 
 
 Upgrading required plugins

=== modified file 'doc/en/user-guide/branching_a_project.txt'
--- doc/en/user-guide/branching_a_project.txt	2010-06-02 05:03:31 +0000
+++ doc/en/user-guide/branching_a_project.txt	2011-04-06 04:58:43 +0000
@@ -9,25 +9,27 @@
 You may decide to make the top level directory of your branch
 a network share, an approach familiar to Windows users.
 Unix users might prefer access to be
-via SFTP, a secure protocol built-in to most SSH servers.
+via SSH, a secure protocol built-in to most SSH servers.
 Bazaar is *very* flexible in this regard with support for
 lots of protocols some of which are given below.
 
   ===========         ======================================================
   Prefix              Description
   ===========         ======================================================
-  \file://            Access using the standard filesystem (default)
+  \file://            Access using the standard filesystem (default).
+  \bzr+ssh:/          Access over SSH (best remote option).
   \sftp://            Access using SFTP (most SSH servers provide SFTP).
   \bzr://             Fast access using the Bazaar smart server.
   \ftp://             Access using passive FTP.
-  \http://            Read-only access to branches exported by a web server.
+  \http://            Access to branches exported by a web server.
+  \https://           Encrypted access to branches exported by a web server.
   ===========         ======================================================
 
 As indicated above, branches are identified using URLs with the
 prefix indicating the transfer technology. If no prefix is given,
 normal filenames are assumed. For a complete list of supported
 protocols, see the ``urlspec`` online help topic or the
-`URL Identifiers <../user-reference/bzr_man.html#url-identifiers>`_
+`URL Identifiers <../user-reference/index.html#url-identifiers>`_
 section of the Bazaar User Reference.
 
 URLs are normally resolved relative to the root directory of the server,
@@ -38,9 +40,14 @@
 which directory.)
 
 To address a path relative to your home directory on the server, use a
-tilde like so: ``sftp://example.com/~/public_html`` should map to
+tilde like so: ``bzr+ssh://example.com/~/public_html`` should map to
 ``public_html`` within your home directory.
 
+.. note:: Access over HTTP or HTTPS is read-only by default.
+   See `Pushing over the HTTP smart server
+   <http_smart_server.html#pushing-over-the-http-smart-server>`_ for
+   details on configuring read-write access.
+
 A reminder about shared repositories
 ------------------------------------
 
@@ -69,7 +76,7 @@
 
 If a directory is not given, one is created based on the last part of
 the URL. Here are some examples showing a drive qualified path (M:/) and an
-sftp URL respectively::
+SFTP URL respectively::
 
   bzr branch M:/cool-trunk
   bzr branch sftp://bill@mary-laptop/cool-repo/cool-trunk

=== modified file 'doc/en/user-guide/bzrtools_plugin.txt'
--- doc/en/user-guide/bzrtools_plugin.txt	2009-03-12 06:04:54 +0000
+++ doc/en/user-guide/bzrtools_plugin.txt	2010-08-13 19:08:57 +0000
@@ -6,7 +6,7 @@
 
 BzrTools is a collection of useful enhancements to Bazaar.
 For installation instructions, see the BzrTools home page:
-http://bazaar-vcs.org/BzrTools.
+http://wiki.bazaar.canonical.com/BzrTools.
 Here is a sample of the frequently used commands it provides.
 
 

=== modified file 'doc/en/user-guide/configuring_bazaar.txt'
--- doc/en/user-guide/configuring_bazaar.txt	2010-07-07 01:55:26 +0000
+++ doc/en/user-guide/configuring_bazaar.txt	2011-05-02 08:15:59 +0000
@@ -19,7 +19,7 @@
 Using a network proxy
 ---------------------
 
-If your network requires that you use an http proxy for outbound
+If your network requires that you use an HTTP proxy for outbound
 connections, you must set the ``http_proxy`` variable.  If the proxy is
 also required for https connections, you need to set ``https_proxy`` too.
 If you need these and don't have them set, you may find that connections
@@ -37,6 +37,24 @@
 which shouldn't be reached by the proxy.  (See
 <http://docs.python.org/library/urllib.html> for more details.)
 
+Various ways to configure
+-------------------------
+
+As shown in the example above, there are various ways to
+configure Bazaar, they all share some common properties though.
+An option has:
+
+- a name which is generally a valid python identifier,
+
+- a value which is a string. In some cases, Bazaar will be able
+  to recognize special values like 'True', 'False' to infer a
+  boolean type, but basically, as a user, you will always specify
+  a value as a string.
+
+Options are grouped in various contexts so the option name
+uniquely identifies it in this context. When needed, options can
+be made persistent by recording them in a configuration file.
+
 
 Configuration files
 -------------------
@@ -66,10 +84,63 @@
   email = Your Name <email@example.com>
 
 For further details on the syntax and configuration settings supported, see
-`Configuration Settings <../user-reference/bzr_man.html#configuration-settings>`_
+`Configuration Settings <../user-reference/index.html#configuration-settings>`_
 in the Bazaar User Reference.
 
 
+Looking at the active configuration
+-----------------------------------
+
+To look at all the currently defined options, you can use the following
+command::
+
+  bzr config
+
+``bzr`` implements some rules to decide where to get the value of a
+configuration option.
+
+The current policy is to examine the existing configurations files in a
+given order for matching definitions.
+
+  * ``locations.conf`` is searched first for a section whose name matches the
+    location considered (working tree, branch or remote branch),
+
+  * the current ``branch.conf`` is searched next,
+
+  * ``bazaar.conf`` is searched next,
+
+  * finally, some options can have default values generally defined in the
+    code itself and not displayed by ``bzr config`` (see `Configuration
+    Settings <../user-reference/index.html#configuration-settings>`_).
+
+This is better understood by using ```bzr config`` with no arguments, which
+will display some output of the form::
+
+  locations:
+    post_commit_to = commits@example.com
+    news_merge_files = NEWS
+  branch:
+    parent_location = bzr+ssh://bazaar.launchpad.net/+branch/bzr/
+    nickname = config-modify
+    push_location = bzr+ssh://bazaar.launchpad.net/~vila/bzr/config-modify/
+  bazaar:
+    debug_flags = hpss,
+
+Each configuration file is associated with a given scope whose name is
+displayed before each set of defined options.
+
+Modifying the active configuration
+----------------------------------
+
+To set an option to a given value use::
+
+  bzr config opt=value
+
+To remove an option use::
+
+  bzr config --remove opt
+
+
 Rule-based preferences
 ----------------------
 
@@ -78,5 +149,19 @@
 ``BZR_HOME/rules``.
 
 For further information on how rules are searched and the detailed syntax of
-the relevant files, see `Rules <../user-reference/bzr_man.html#rules>`_
+the relevant files, see `Rules <../user-reference/index.html#rules>`_
 in the Bazaar User Reference.
+
+
+Escaping command lines
+----------------------
+
+When you give a program name or command line in configuration, you can quote
+to include special characters or whitespace.  The same rules are used across
+all platforms.
+
+The rules are: strings surrounded by double-quotes are interpreted as single
+"words" even if they contain whitespace, and backslash may be used to quote
+quotation marks.  For example::
+
+    BZR_EDITOR="C:\Program Files\My Editor\myeditor.exe"

=== added file 'doc/en/user-guide/gpg_signatures.txt'
--- doc/en/user-guide/gpg_signatures.txt	1970-01-01 00:00:00 +0000
+++ doc/en/user-guide/gpg_signatures.txt	2011-08-16 11:19:00 +0000
@@ -0,0 +1,91 @@
+GnuPG Signatures
+=============================
+
+Reasons to Sign Your Repository
+--------------------------------
+
+Bazaar can sign revisions using GnuPG, a Free Software implementation of the
+OpenPGP digital signature format.  By signing commits a person wanting to
+make use of a branch can be confident where the code came from, assuming the
+GnuPG keys used can be verified.  This could for example prevent worry about
+compromised code in the case where a server hosting Bazaar branches has been
+hacked into.  It could also be used to verify that all code is written by a
+select group of people, such as if contributor agreements are needed.
+
+Signatures are passed around with commits during branch, push, merge and other
+operations.
+
+Setting up GnuPG
+----------------
+
+There are many guides to creating a digital signature key with GnuPG.  See
+for example the `GnuPG Handbook
+<http://www.gnupg.org/gph/en/manual.html#AEN26>`_ or the `Launchpad Wiki
+<https://help.launchpad.net/YourAccount/ImportingYourPGPKey>`_.
+
+
+Signing Commits
+---------------
+
+To sign commits as they are made turn on the ``create_signatures``
+configuration option in your ``bazaar.conf`` or ``locations.conf`` file::
+
+  create_signatures = always
+
+When you next make a commit it will ask for the pass phrase for your GnuPG key. 
+If you want GnuPG to remember your password ensure you have ``gnupg-agent``
+installed.
+
+To sign previous commits to a branch use ``sign-my-commits``.  This will go
+through all revisions in the branch and sign any which match your
+commit name.  You can also pass the name of a contributor to ``sign-my-commits``
+to sign someone else's commits or if your GnuPG key does not match your Bazaar
+name and e-mail::
+
+  bzr sign-my-commits . "Amy Pond <amy@example.com>"
+
+It will not sign commits which already have a signature.
+
+To sign a single commit or a range of commits use the (hidden) command
+``re-sign``::
+
+  bzr re-sign -r 24
+
+``re-sign`` is also useful to change an existing signature.
+
+By default Bazaar will tell GnuPG to use a key with the same user
+identity as the one set with ``whoami``.  To override this set
+``gpg_signing_key`` in bazaar.conf or locations.conf.
+
+  ``gpg_signing_key=DD4D5088``
+
+  ``gpg_signing_key=amy@example.com``
+
+Verifying Commits
+-----------------
+
+Signatures can be verified with the ``bzr verify-signatures`` command.  By
+default this will check all commits in the branch and notify that all commits
+are signed by known trusted signatures.  If not all commits have trusted
+signatures it will give a summary of the number of commits which are invalid,
+having missing keys or are not signed.
+
+The ``verify-signatures`` command can be given a comma separated list of key
+patters to specify a list of acceptable keys.  It can also take a range of
+commits to verify in the current branch.  Finally using the verbose option will
+list each key that is valid or authors for commits which failed::
+
+  $bzr verify-signatures -kamy -v -r 1..5
+  1 commit with valid signature
+    Amy Pond <amy@example.com> signed 4 commits
+  0 commits with unknown keys
+  1 commit not valid
+    1 commit by author The Doctor <doctor@example.com>
+  0 commits not signed
+
+Work in Progress
+----------------
+
+There is still a number of digital signature related features which
+are hoped to be added to Bazaar soon.  These include bzr explorer
+integration and setting branches to require signatures.

=== modified file 'doc/en/user-guide/hooks.txt'
--- doc/en/user-guide/hooks.txt	2010-04-22 07:35:06 +0000
+++ doc/en/user-guide/hooks.txt	2011-02-18 18:19:50 +0000
@@ -8,7 +8,7 @@
 perform actions before or after certain Bazaar operations.  The operations
 include ``commit``, ``push``, ``pull``, and ``uncommit``.
 For a complete list of hooks and their parameters, see `Hooks
-<../user-reference/hooks-help.html>`_ in the User Reference.
+<../user-reference/index.html#hooks>`_ in the User Reference.
 
 Most hooks are run on the client, but a few are run on the server.  (Also
 see the `push-and-update plugin`_ that handles one special case of
@@ -54,6 +54,20 @@
 itself.  The third argument is a name ``'My post_push hook'``, which can be
 used in progress messages and error messages.
 
+To reduce the start-up time of Bazaar it is also possible to "lazily" install hooks,
+using the ``bzrlib.hooks.install_lazy_named_hook`` function. This removes the need
+to load the module that contains the hook point just to install the hook. Here's lazy
+version of the example above::
+
+    from bzrlib import hooks
+
+    def post_push_hook(push_result):
+        print "The new revno is %d" % push_result.new_revno
+
+
+    hooks.install_lazy_named_hook('bzrlib.branch', 'Branch.hooks',
+        'post_push', post_push_hook, 'My post_push hook')
+
 Debugging hooks
 ---------------
 
@@ -81,7 +95,7 @@
   
   def merge_xml_files_hook(merger):
       """Hook to merge *.xml files"""
-      return MergeXMLFiles(merger)
+      return AlwaysConflictXMLMerger(merger)
   
   class AlwaysConflictXMLMerger(PerFileMerger):
   

=== modified file 'doc/en/user-guide/http_smart_server.txt'
--- doc/en/user-guide/http_smart_server.txt	2010-02-26 18:47:48 +0000
+++ doc/en/user-guide/http_smart_server.txt	2010-11-12 22:43:38 +0000
@@ -5,9 +5,7 @@
 using Apache 2.0 and FastCGI or mod_python or mod_wsgi.
 
 For more information on the smart server, and other ways to configure it
-see the main `smart server documentation`_.
-
-.. _smart server documentation: #running-a-smart-server
+see the main `smart server documentation <server.html>`_.
 
 Example
 -------
@@ -49,7 +47,7 @@
         Options Indexes FollowSymLinks
         RewriteEngine On
         RewriteBase /code
-        RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
+        RewriteRule ^(.*/)?\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
     </Directory>
 
     # bzr-smart.fcgi isn't under the DocumentRoot, so Alias it into the URL
@@ -82,11 +80,11 @@
 Define the rewrite rules with mod_rewrite the same way as for FastCGI, except
 change::
 
-    RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
+    RewriteRule ^(.*/)?\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
 
 to::
 
-    RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.py
+    RewriteRule ^(.*/)?\.bzr/smart$ /srv/example.com/scripts/bzr-smart.py
 
 Like with mod_fastcgi, we also define how our script is to be handled::
 
@@ -260,10 +258,10 @@
 .. _WSGI standard: http://www.python.org/dev/peps/pep-0333/
 
 
-Pushing over the http smart server
+Pushing over the HTTP smart server
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-It is possible to allow pushing data over the http smart server. The
+It is possible to allow pushing data over the HTTP smart server. The
 easiest way to do this, is to just supply ``readonly=False`` to the
 ``wsgi.make_app()`` call. But be careful, because the smart protocol does
 not contain any Authentication. So if you enable write support, you will
@@ -280,7 +278,7 @@
     </Location>
 
 At this time, it is not possible to allow some people to have read-only
-access and others to have read-write access to the same urls. Because at
+access and others to have read-write access to the same URLs. Because at
 the HTTP layer (which is doing the Authenticating), everything is just a
 POST request.  However, it would certainly be possible to have HTTPS
 require authentication and use a writable server, and plain HTTP allow

=== modified file 'doc/en/user-guide/index-plain.txt'
--- doc/en/user-guide/index-plain.txt	2010-07-07 11:32:03 +0000
+++ doc/en/user-guide/index-plain.txt	2011-05-16 10:22:06 +0000
@@ -114,6 +114,7 @@
 .. include:: setting_up_email.txt
 .. include:: http_smart_server.txt
 .. include:: writing_a_plugin.txt
+.. include:: licence.txt
 
 .. |--| unicode:: U+2014
 

=== modified file 'doc/en/user-guide/index.txt'
--- doc/en/user-guide/index.txt	2010-07-07 11:21:19 +0000
+++ doc/en/user-guide/index.txt	2011-06-13 14:08:28 +0000
@@ -103,6 +103,7 @@
    server
    hooks
    version_info
+   gpg_signatures
 
 
 A brief tour of some popular plugins
@@ -137,5 +138,6 @@
    setting_up_email
    http_smart_server
    writing_a_plugin
+   licence
 
 .. |--| unicode:: U+2014

=== modified file 'doc/en/user-guide/installing_bazaar.txt'
--- doc/en/user-guide/installing_bazaar.txt	2010-06-02 05:03:31 +0000
+++ doc/en/user-guide/installing_bazaar.txt	2011-08-30 09:27:59 +0000
@@ -6,7 +6,7 @@
 
 Bazaar packages are available for most popular GNU/Linux distributions
 including Ubuntu, Debian, Red Hat and Gentoo.
-See http://bazaar-vcs.org/Download for the latest instructions.
+See http://wiki.bazaar.canonical.com/Download for the latest instructions.
 
 Windows
 -------
@@ -14,7 +14,7 @@
 For Windows users, an installer is available that includes
 the core Bazaar package together with necessary pre-requisites
 and some useful plug-ins.
-See http://bazaar-vcs.org/Download for the latest instructions.
+See http://wiki.bazaar.canonical.com/Download for the latest instructions.
 
 Note: If you are running Cygwin on Windows, a Bazaar for Cygwin package
 is available and ought to be used instead of the Windows version.
@@ -24,7 +24,7 @@
 
 Beyond Linux and Windows, Bazaar packages are available for a large
 range of other operating systems include Mac OS X, FreeBSD and Solaris.
-See http://bazaar-vcs.org/Download for the latest instructions.
+See http://wiki.bazaar.canonical.com/Download for the latest instructions.
 
 
 Installing from scratch
@@ -33,10 +33,10 @@
 If you wish to install Bazaar from scratch rather than using a
 pre-built package, the steps are:
 
-  1. If it is not installed already, install Python 2.4 or later.
+  1. If it is not installed already, install Python 2.6 or later.
 
   2. Download the ``bazaar-xxx.tar.gz`` file (where xxx is the version
-     number) from http://bazaar-vcs.org/Download or from Launchpad
+     number) from http://wiki.bazaar.canonical.com/Download or from Launchpad
      (https://launchpad.net/~bzr/).
 
   3. Unpack the archive using tar, WinZip or equivalent.

=== modified file 'doc/en/user-guide/introducing_bazaar.txt'
--- doc/en/user-guide/introducing_bazaar.txt	2010-06-02 05:18:27 +0000
+++ doc/en/user-guide/introducing_bazaar.txt	2010-08-13 19:08:57 +0000
@@ -89,7 +89,7 @@
  * in companies, easier to work with distributed and outsourced teams.
 
 For a detailed look at the advantages of distributed VCS tools over
-centralized VCS tools, see http://bazaar-vcs.org/BzrWhy.
+centralized VCS tools, see http://wiki.bazaar.canonical.com/BzrWhy.
 
 
 Key features of Bazaar
@@ -98,12 +98,12 @@
 While Bazaar is not the only distributed VCS tool around, it does have some
 notable features that make it an excellent choice for many teams and
 communities. A summary of these and comparisons with other VCS tools
-can be found on the Bazaar Wiki, http://bazaar-vcs.org.
+can be found on the Bazaar Wiki, http://wiki.bazaar.canonical.com.
 
 Of the many features, one in particular is worth highlighting:
 Bazaar is completely free software written in Python. As a result,
 it is easy to contribute improvements. If you wish to get involved,
-please see http://bazaar-vcs.org/BzrSupport.
+please see http://wiki.bazaar.canonical.com/BzrSupport.
 
 
 Learning more

=== added file 'doc/en/user-guide/licence.txt'
--- doc/en/user-guide/licence.txt	1970-01-01 00:00:00 +0000
+++ doc/en/user-guide/licence.txt	2011-05-16 11:27:37 +0000
@@ -0,0 +1,7 @@
+Licence
+===============
+
+Copyright 2009-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later.  See
+<http://www.gnu.org/licenses/>.

=== modified file 'doc/en/user-guide/merging_changes.txt'
--- doc/en/user-guide/merging_changes.txt	2009-09-09 15:06:00 +0000
+++ doc/en/user-guide/merging_changes.txt	2011-05-16 17:48:58 +0000
@@ -29,10 +29,11 @@
 has done in his. In this case, she needs to explicitly give the URL the
 first time, e.g.::
 
-  bzr merge sftp://mary@bill-laptop/cool-repo/cool-trunk
+  bzr merge bzr+ssh://mary@bill-laptop/cool-repo/cool-trunk
 
-This sets the default merge branch if one is not already set. To change the
-default after it is set, use the ``--remember`` option.
+This sets the default merge branch if one is not already set.  Use
+``--no-remember`` to avoid setting it. To change the default after it is set,
+use the ``--remember`` option.
 
 How does merging work?
 ----------------------

=== modified file 'doc/en/user-guide/organizing_branches.txt'
--- doc/en/user-guide/organizing_branches.txt	2009-04-04 02:50:01 +0000
+++ doc/en/user-guide/organizing_branches.txt	2011-04-06 04:58:43 +0000
@@ -16,7 +16,7 @@
 
   bzr init-repo PROJECT
   cd PROJECT
-  bzr branch sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr branch bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 Task branches
 -------------

=== modified file 'doc/en/user-guide/organizing_your_workspace.txt'
--- doc/en/user-guide/organizing_your_workspace.txt	2009-09-09 15:30:59 +0000
+++ doc/en/user-guide/organizing_your_workspace.txt	2010-11-12 22:08:18 +0000
@@ -133,7 +133,7 @@
 As a variation, the trunk can be created as a checkout. If you have
 commit privileges on trunk, that lets you merge into trunk and the
 commit of the merge will implicitly publish your change. Alternatively,
-if the trunk URL is read-only (e.g. a http address), that prevents
+if the trunk URL is read-only (e.g. an HTTP address), that prevents
 accidental submission this way - ideal if the project workflow uses
 an automated gatekeeper like PQM, say.
 

=== modified file 'doc/en/user-guide/plugins.txt'
--- doc/en/user-guide/plugins.txt	2010-06-02 05:03:31 +0000
+++ doc/en/user-guide/plugins.txt	2010-09-27 19:31:45 +0000
@@ -28,7 +28,7 @@
 Where to find plugins
 ---------------------
 
-We keep our list of plugins on the http://bazaar-vcs.org/BzrPlugins page.
+We keep our list of plugins on the http://wiki.bazaar.canonical.com/BzrPlugins page.
 
 How to install a plugin
 -----------------------
@@ -59,8 +59,8 @@
 If you have the necessary permissions, plugins can also be installed on a
 system-wide basis.  One can additionally override the personal plugins
 location by setting the environment variable ``BZR_PLUGIN_PATH`` (see `User
-Reference <../user-reference/bzr_man.html#bzr-plugin-path>`_ for a detailed
-explanation).
+Reference <../user-reference/configuration-help.html#bzr-plugin-path>`_ 
+for a detailed explanation).
 
 Listing the installed plugins
 -----------------------------

=== modified file 'doc/en/user-guide/publishing_a_branch.txt'
--- doc/en/user-guide/publishing_a_branch.txt	2009-12-02 11:03:35 +0000
+++ doc/en/user-guide/publishing_a_branch.txt	2011-04-08 09:04:17 +0000
@@ -1,3 +1,5 @@
+.. _publishing_a_branch:
+
 Publishing a branch
 ===================
 
@@ -14,7 +16,7 @@
 store history, not working copies of files, so their enclosing
 repository is usually creating using the ``no-trees`` option, e.g.::
 
-  bzr init-repo --no-trees sftp://centralhost/srv/bzr/PROJECT
+  bzr init-repo --no-trees bzr+ssh://centralhost/srv/bzr/PROJECT
 
 You can think of this step as similar to setting up a new cvsroot or
 Subversion repository. However, Bazaar gives you more flexibility
@@ -42,14 +44,14 @@
   bzr add                (populate repository; start version control)
   bzr commit -m "Initial import"
                          (publish to central repository)
-  bzr push sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr push bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 Here is an example of the second way::
 
   bzr init-repo PROJECT  (prepare local repository)
   cd PROJECT
-  bzr init sftp://centralhost/srv/bzr/PROJECT/trunk
-  bzr checkout sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr init bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
+  bzr checkout bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
   cd trunk
   cp -ar ~/PROJECT .     (copy files in using OS-specific tools)
   bzr add                (populate repository; start version control)

=== modified file 'doc/en/user-guide/reusing_a_checkout.txt'
--- doc/en/user-guide/reusing_a_checkout.txt	2008-12-24 03:14:16 +0000
+++ doc/en/user-guide/reusing_a_checkout.txt	2011-04-06 04:58:43 +0000
@@ -63,7 +63,7 @@
 
   bzr init-repo --no-trees PROJECT
   cd PROJECT
-  bzr branch sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr branch bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
   bzr checkout --lightweight trunk my-sandbox
   cd my-sandbox
   (hack away)
@@ -74,7 +74,7 @@
 need there and switch between them as required. For example::
 
   (assuming in my-sandbox)
-  bzr branch sftp://centralhost/srv/bzr/PROJECT/PROJECT-1.0 ../PROJECT-1.0
+  bzr branch bzr+ssh://centralhost/srv/bzr/PROJECT/PROJECT-1.0 ../PROJECT-1.0
   bzr switch ../PROJECT-1.0
   (fix bug in 1.0)
   bzr commit -m "blah, blah blah"

=== modified file 'doc/en/user-guide/sending_changes.txt'
--- doc/en/user-guide/sending_changes.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/sending_changes.txt	2010-10-08 10:50:51 +0000
@@ -42,7 +42,7 @@
 ``send`` without options will create a merge directive, fire up your email
 tool and attach it, ready for you to add the explanatory text bit.
 (See the online help for ``send`` and
-`Configuration Settings <../user-reference/bzr_man.html#configuration-settings>`_
+`Configuration Settings <../user-reference/index.html#configuration-settings>`_
 in the User Reference for further details on how to configure this.)
 
 Most projects like people to add some explanation to the mail along with

=== modified file 'doc/en/user-guide/server.txt'
--- doc/en/user-guide/server.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/server.txt	2010-11-12 22:46:28 +0000
@@ -13,7 +13,7 @@
 over any of these protocols, Bazaar will allow you to read it remotely.  Just
 enter the URL to the branch in the Bazaar command you are running.::
 
-    bzr log http://bazaar-vcs.org/bzr/bzr.dev
+    bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev
 
 Bazaar supports writing over FTP, SFTP and (via a plugin) over HTTP-WebDAV.
 
@@ -64,7 +64,7 @@
 
 Running a Bazaar server from inetd requires an inetd.conf entry::
 
-    4155  stream  tcp  nowait  bzruser  /usr/bin/bzr /usr/bin/bzr serve --inet --directory=/srv/bzr/repo
+    4155  stream  TCP  nowait  bzruser  /usr/bin/bzr /usr/bin/bzr serve --inet --directory=/srv/bzr/repo
 
 When running client commands, the URL you supply is a `bzr://` URL relative to
 the ``--directory`` option given in inetd.conf::

=== modified file 'doc/en/user-guide/setting_up_email.txt'
--- doc/en/user-guide/setting_up_email.txt	2010-06-02 05:03:31 +0000
+++ doc/en/user-guide/setting_up_email.txt	2010-10-08 10:50:51 +0000
@@ -74,7 +74,7 @@
 For more information on the ini file format, see `Configuration Settings`_ in
 the Bazaar User Reference.
 
-.. _Configuration Settings: ../user-reference/bzr_man.html#configuration-settings
+.. _Configuration Settings: ../user-reference/index.html#configuration-settings
 
 Setting email on a per-branch basis
 -----------------------------------

=== modified file 'doc/en/user-guide/shared_repository_layouts.txt'
--- doc/en/user-guide/shared_repository_layouts.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/shared_repository_layouts.txt	2010-11-12 22:08:18 +0000
@@ -121,7 +121,7 @@
     Letting you ``bzr checkout`` one project, and have it grab all the necessary
     sub-projects.
 
-.. _NestedTreeSupport: http://bazaar-vcs.org/NestedTrees
+.. _NestedTreeSupport: http://wiki.bazaar.canonical.com/NestedTrees
 
 
 Nested Style (``project/branch/sub-branch/``)
@@ -237,7 +237,7 @@
 that could be changed.
 This is actually how `Launchpad`_ allows you to ``bzr checkout https://launchpad.net/bzr``.
 Effectively a ``BranchReference`` is a symlink, but it allows you to reference any other URL.
-If it is extended to support relative references, it would even work over http, sftp,
+If it is extended to support relative references, it would even work over HTTP, SFTP,
 and local paths.
 
 .. _Launchpad: https://launchpad.net

=== modified file 'doc/en/user-guide/specifying_revisions.txt'
--- doc/en/user-guide/specifying_revisions.txt	2009-12-02 23:09:40 +0000
+++ doc/en/user-guide/specifying_revisions.txt	2011-01-16 05:46:18 +0000
@@ -65,7 +65,7 @@
 For complete details, see `Revision Identifiers`_ in the
 Bazaar User Reference.
 
-.. _Revision Identifiers: ../user-reference/bzr_man.html#revision-identifiers
+.. _Revision Identifiers: ../user-reference/index.html#revision-identifiers
 
 Numbers
 ~~~~~~~
@@ -86,7 +86,7 @@
 revid
 ~~~~~
 
-**revid** allows specifying a an internal revision ID, as shown by ``bzr
+**revid** allows specifying an internal revision ID, as shown by ``bzr
 log --show-ids`` and some other commands.
 
 For example::

=== modified file 'doc/en/user-guide/stacked.txt'
--- doc/en/user-guide/stacked.txt	2010-03-25 07:34:15 +0000
+++ doc/en/user-guide/stacked.txt	2011-05-20 17:27:26 +0000
@@ -74,46 +74,55 @@
 This creates a new branch at ``my-url`` that is stacked on ``reference-url``
 and only contains the revisions in the current branch that are not already
 in the branch at ``reference-url``. In particular, ``my-url`` and
-``reference-url`` can be on the same host, and the ``--stacked`` option
+``reference-url`` can be on the same host, and the ``--stacked-on`` option
 can be used additionally to inform ``push`` to reference the
 revisions in ``reference-url``. For example::
 
-  bzr push --stacked-on sftp://host/project --stacked sftp://host/user/stacked-branch
+  bzr push --stacked-on bzr+ssh://host/project bzr+ssh://host/user/stacked-branch
 
 This usage fits the scenario described in the Motivation section.
 
-
-.. The following text is hidden because bug 375013 breaks the example.
-   When bug 375013 is fixed, we should unhide this text.
-      - Andrew Bennetts, 10 March 2010
-
-.. If the local branch was created as a stacked branch, then you can
-.. use the ``--stacked`` option to ``push`` and the *stacked-on* location
-.. will be implicit. For example::
-.. 
-..   bzr branch --stacked source-url my-dir
-..   cd my-dir
-..   (hack, hack, hack)
-..   bzr commit -m "fix bug"
-..   bzr push --stacked
+You can also use the ``--stacked`` option without specifying ``--stacked-on``.
+This will automatically set the *stacked-on* location to the parent branch of
+the branch you are pushing (or its ``public_location`` if configured).  For
+example::
+
+  bzr branch source-url my-dir
+  cd my-dir
+  (hack, hack, hack)
+  bzr commit -m "fix bug"
+  bzr push --stacked
+
+You can combine ``bzr branch --stacked`` and ``bzr push --stacked`` to work on a
+branch without downloading or uploading the whole history::
+
+  bzr branch --stacked source-url my-dir
+  cd my-dir
+  (hack, hack, hack)
+  bzr commit -m "fix bug"
+  bzr push --stacked
 
 
 Limitations of stacked branches
 -------------------------------
 
-Currently, you cannot commit to a stacked branch, due to `bug 375013`_.
-
-.. _bug 375013: https://bugs.launchpad.net/bzr/+bug/375013
-
-The important thing to remember about a stacked branch is that the
-stacked-on branch needs to be available for almost all operations. This is
-not an issue when both branches are local or both branches are on the
-same server.
+The important thing to remember about a stacked branch is that the stacked-on
+branch needs to be accessible for almost all operations.  This is not an issue
+when both branches are local, or when both branches are on the same server and
+the stacked-on location is a relative path.  But clearly a branch hosted on a
+server with a stacked-on location of ``file:///...`` is not going to work for
+anyone except the user that originally pushed it.  It's a good idea to configure
+``public_location`` to help prevent that.
 
 Similarly, because most of the history is stored in the stacked-on repository,
 operations like ``bzr log`` can be slower when the stacked-on repository is
 accessed via a network.
 
+If a stacked branch is in a format older than 2a, you cannot commit to it due to
+`bug 375013`_.
+
+.. _bug 375013: https://bugs.launchpad.net/bzr/+bug/375013
+
 
 Changing branch stacking
 ------------------------

=== modified file 'doc/en/user-guide/svn_plugin.txt'
--- doc/en/user-guide/svn_plugin.txt	2009-09-25 10:22:44 +0000
+++ doc/en/user-guide/svn_plugin.txt	2010-08-13 19:08:57 +0000
@@ -20,7 +20,7 @@
 who are unable to do so yet for timing or non-technical reasons.
 
 For installation instructions, see the bzr-svn home page:
-http://bazaar-vcs.org/BzrForeignBranches/Subversion.
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion.
 
 
 A simple example
@@ -110,5 +110,5 @@
  * Bazaar doesn't support tracking of file copies.
 
 See the bzr-svn web page,
-http://bazaar-vcs.org/BzrForeignBranches/Subversion,
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion,
 for the current list of constraints.

=== modified file 'doc/en/user-guide/undoing_mistakes.txt'
--- doc/en/user-guide/undoing_mistakes.txt	2009-11-23 15:29:24 +0000
+++ doc/en/user-guide/undoing_mistakes.txt	2011-08-24 09:31:18 +0000
@@ -93,6 +93,9 @@
 one or more files. Some users like to alias ``commit`` to ``commit --strict``
 so that commits fail if unknown files are found in the tree.
 
+Tags for uncommitted revisions are removed from the branch unless
+``--keep-tags`` was specified.
+
 Note: While the ``merge`` command is not introduced until the next
 chapter, it is worth noting now that ``uncommit`` restores any pending
 merges. (Running ``bzr status`` after ``uncommit`` will show these.)

=== modified file 'doc/en/user-guide/using_checkouts.txt'
--- doc/en/user-guide/using_checkouts.txt	2009-09-09 15:30:59 +0000
+++ doc/en/user-guide/using_checkouts.txt	2011-04-06 04:58:43 +0000
@@ -7,7 +7,7 @@
 If you have a local branch and wish to make it a checkout, use the
 ``bind`` command like this::
 
-  bzr bind sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr bind bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 This is necessary, for example, after creating a central branch using
 ``push`` as illustrated in the previous section.
@@ -56,7 +56,7 @@
 To get a history-less checkout in Bazaar, use the ``--lightweight``
 option like this::
 
-  bzr checkout --lightweight sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr checkout --lightweight bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 Of course, many of the benefits of a normal checkout are lost by doing
 this but that's a tradeoff you can make if and when it makes sense.

=== modified file 'doc/en/user-guide/version_info.txt'
--- doc/en/user-guide/version_info.txt	2010-06-02 05:03:31 +0000
+++ doc/en/user-guide/version_info.txt	2010-10-08 10:50:51 +0000
@@ -87,7 +87,7 @@
 variables that can be used in templates, see `Version Info`_ in the
 Bazaar User Reference.
 
-.. _Version Info: ../user-reference/bzr_man.html#version-info
+.. _Version Info: ../user-reference/index.html#version-info
 
 Predefined formats for dumping version information in specific languages
 are currently in development. Please contact us on the mailing list about

=== modified file 'doc/en/user-guide/web_browsing.txt'
--- doc/en/user-guide/web_browsing.txt	2009-12-02 20:34:07 +0000
+++ doc/en/user-guide/web_browsing.txt	2010-08-13 19:08:57 +0000
@@ -9,7 +9,7 @@
 can be found at https://launchpad.net/loggerhead.
 
 A list of alternative web viewers including download links can be found on
-http://bazaar-vcs.org/WebInterface.
+http://wiki.bazaar.canonical.com/WebInterface.
 
 Note: If your project is hosted or mirrored on Launchpad,
 Loggerhead code browsing is provided as part of the service.

=== modified file 'doc/en/user-guide/writing_a_plugin.txt'
--- doc/en/user-guide/writing_a_plugin.txt	2010-04-21 07:52:31 +0000
+++ doc/en/user-guide/writing_a_plugin.txt	2010-09-27 19:31:45 +0000
@@ -42,8 +42,8 @@
 
 Bzr will scan ``~/.bazaar/plugins``  and ``bzrlib/plugins`` for plugins
 by default.  You can override this with  ``BZR_PLUGIN_PATH``
-(see `User Reference <../user-reference/bzr_man.html#bzr-plugin-path>`_
-for details).
+(see `User Reference 
+<../user-reference/configuration-help.html#bzr-plugin-path>`_ for details).
 
 Plugins may be either modules or packages.  If your plugin is a single
 file, you can structure it as a module.  If it has multiple files, or if

=== modified file 'doc/en/user-reference/readme.txt'
--- doc/en/user-reference/readme.txt	2007-12-11 13:40:18 +0000
+++ doc/en/user-reference/readme.txt	2011-05-16 09:08:56 +0000
@@ -5,4 +5,4 @@
 registries, e.g. the set of available file formats. If you're
 not sure where to find the source content you're looking for,
 please contact the developers on the mailing list or IRC.
-See http://bazaar-vcs.org/BzrSupport for contact details.
+See http://wiki.bazaar.canonical.com/BzrSupport for contact details.

=== added file 'doc/en/whats-new/template.txt'
--- doc/en/whats-new/template.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/template.txt	2011-07-18 15:58:50 +0000
@@ -0,0 +1,26 @@
+*************************
+What's New in Bazaar x.y?
+*************************
+
+Bazaar x.y is still under development, and will be released in Month Year.
+This document accumulates a high level summary of what's changed.  See the
+:doc:`../release-notes/index` for a full list.
+
+<topics of interest here>
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the the
+:doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/x.y>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+* :doc:`whats-new-in-2.3`
+<intermediate series here>
+* :doc:`whats-new-in-x.(y-1)`

=== modified file 'doc/en/whats-new/whats-new-in-2.1.txt'
--- doc/en/whats-new/whats-new-in-2.1.txt	2010-02-19 05:08:31 +0000
+++ doc/en/whats-new/whats-new-in-2.1.txt	2010-12-02 09:21:11 +0000
@@ -62,6 +62,16 @@
 or just say ``today`` (instead of saying ``date:today``). Prefixes
 are now only required if the revision spec could be ambiguous.
 
+Launchpad compatibility
+-----------------------
+
+Launchpad has `announced
+<http://blog.launchpad.net/general/edge-is-deprecated>`_ that the
+``edge.launchpad.net`` instance is deprecated and may be shut down in the
+future . Bazaar has therefore been updated in this release to talk to the main
+(``launchpad.net``) servers, rather than the ``edge`` ones (the same code is
+running on both servers during the interim).
+
 
 New ignore patterns
 -------------------

=== modified file 'doc/en/whats-new/whats-new-in-2.2.txt'
--- doc/en/whats-new/whats-new-in-2.2.txt	2010-03-04 04:15:18 +0000
+++ doc/en/whats-new/whats-new-in-2.2.txt	2011-09-09 12:32:08 +0000
@@ -1,62 +1,306 @@
+*************************
 What's New in Bazaar 2.2?
-=========================
-
-This document outlines the major improvements in Bazaar 2.2
-vs Bazaar 2.1. As well as summarizing improvements made to
-the core product, it highlights enhancements within the broader
-Bazaar world of potential interest to those upgrading.
-
-.. warning::
-
-   Bazaar 2.2.0 is currently in development. While less stable than
-   Bazaar 2.1.x, we aim to keep quality at a high level:
-
-   * Monthly beta releases should be suitable for early adopters of new
-     features.
-
-   * Nightly builds are suitable for users interested in assisting
-     with testing.
-
-   If it doubt, please use the latest 2.1 release instead.
-
+*************************
+
+Bazaar 2.2.0, released on the 6th of August 2010, marks the start of
+another long-term-stable series. From here, we will only make bugfix
+releases on the 2.2 series (2.2.1, etc), while 2.3 will become our new
+development series. The 2.0 and 2.1 series will also continue to get
+bugfixes. (Currently 2.0 is planned to be supported for another 6 months.)
+
+The main changes in 2.2 are: **better local and network performance**,
+**reduced memory usage**, and several user-interface improvements.
+
+Users are encouraged to upgrade from the other stable series.  This
+document outlines the improvements in Bazaar 2.2 vs Bazaar 2.1. As well as
+summarizing improvements made to the core product, it highlights
+enhancements within the broader Bazaar world of potential interest to
+those upgrading.
+
+Bazaar 2.2.0 includes all the fixes from 2.1.2 and 2.0.6.
+
+Over 120 bugs have been fixed in total.  See the
+:doc:`../release-notes/index` for a full list.
+
+Bazaar 2.2.1 includes all the fixes from 2.1.3 and 2.0.6 (that
+weren't included in 2.2.0).
+
+See the :doc:`../release-notes/index` for details.
+
+Bazaar 2.2.2 focused on fixes to improve our Ubuntu release workflow (which
+should also help all other distributions).
+
+See the :doc:`../release-notes/index` for details.
+
+Bazaar 2.2.3 focused on fixes related to interactions with the launchpad
+server and python-2.7 compatibility.
+
+Bazaar 2.2.4 fixed a regression for some interactions with the launchpad
+server.
+
+Bazaar 2.2.5 fixed a regression in some rare conflict resolutions and warns
+when branching an out-of-date ubuntu packaging branch.
+
+See the :doc:`../release-notes/index` for details.
+
+Bazaar 2.2 is fully compatible both locally and on the network with 2.0
+and 2.1, and can read and write repositories generated by all previous
+versions.
+
+
+Behaviour changes
+*****************
+
+There are some compatibility changes in this release.  
+
+* For commandline users we no longer guess user identity for ``bzr
+  commit``: users must specify their identity using ``bzr whoami`` (you
+  don't need to specify your identity for readonly operations).
+  This avoids problems where the previous guessed default caused commits
+  be recorded as coming from, for example ``<sam@localhost>``.
 
 Improved conflict handling
---------------------------
+**************************
 
 Tree-shape conflicts can be resolved by providing ``--take-this`` and
 ``--take-other`` to the ``bzr resolve`` command. Just marking the conflict
 as resolved is still accessible via the ``--done`` default action.
 
+Local performance
+*****************
+
+* ``bzr init`` does not recursively scan directory contents anymore
+  leading to faster init for directories with existing content.
+  (Martin [gz], Parth Malwankar, #501307)
+
+* Less code is loaded at startup, so there's less overhead on running all
+  bzr commands.
+  (Andrew Bennetts, Martin Pool)
+
+* Reduce peak memory by one copy of compressed text.
+  (John Arbash Meinel, #566940)
+
+* Avoid repeated locking of local objects in ``diff``, ``missing``, and
+  ``pull``, so those options are faster.
+  (Andrew Bennetts)
+
+Network performance
+*******************
+
+* Bazaar now reads data from SSH connections more efficiently on platforms
+  that provide the ``socketpair`` function, and when using paramiko.
+  (Andrew Bennetts, #590637)
+
+* Index lookups in pack repositories search recently hit pack files
+  first.  In repositories with many pack files this can greatly reduce the
+  number of files accessed, the number of bytes read, and the number of
+  read calls.  An incremental pull via plain HTTP takes half the time and
+  bytes for a moderately large repository.  (Andrew Bennetts)
+
+* Index lookups only re-order the indexes when the hit files aren't
+  already first. Reduces the cost of reordering
+  (John Arbash Meinel, #562429)
+
+
+Command improvements
+********************
+
+* Added ``bzr remove-branch`` command that can remove a local or remote 
+  branch. (Jelmer Vernooij, #276295)
+
+* ``bzr export`` now takes an optional argument ``--per-file-timestamps``
+  to set file mtimes to the last timestamp of the last revision in which
+  they were changed rather than the current time. (Jelmer Vernooij)
+
+* Tag names can now be determined automatically by ``automatic_tag_name`` 
+  hooks on ``Branch`` if they are not specified on the command line.
+  (Jelmer Vernooij)
+
+* Tree-shape conflicts can be resolved by providing ``--take-this`` and
+  ``--take-other`` to the ``bzr resolve`` command. Just marking the conflict
+  as resolved is still accessible via the ``--done`` default action.
+  (Vincent Ladeuil)
+
+* The ``--directory`` option is supported for a number of additional
+  commands: added, annotate, bind, cat, cat-revision, clean-tree,
+  conflicts, deleted, export, ignore, ignored, lookup-revision, ls,
+  merge-directive, missing, modified, nick, re-sign, resolve, shelve,
+  switch, unbind, unknowns, unshelve, whoami.  
+  (Martin von Gagern, #527878)
+
+* ``bzr commit`` accepts ``-p`` (for "patch") as a shorter name for
+  ``--show-diff``.
+  (Parth Malwankar, #571467)
+  
+* ``bzr ignore`` now supports a ``--default-rules`` option that displays
+  the default ignore rules used by bzr. The flag ``--old-default-rules``
+  is no longer supported by ``ignore``.
+  (Parth Malwankar, #538703)
+
+* ``bzr pack`` now supports a ``--clean-obsolete-packs`` option that
+  can save disk space by deleting obsolete pack files created during the
+  pack operation.
+  (Parth Malwankar, #304320)
+
+* New command line option ``--authors`` to ``bzr log`` allows users to
+  select which of the apparent authors and committer should be
+  included in the log. Defaults depend on format. (Martin von Gagern, #513322)
+
+* The bash_completion plugin from the bzr-bash-completion project has
+  been merged into the tree. It provides a bash-completion command and
+  replaces the outdated ``contrib/bash/bzr`` script with a version
+  using the plugin. (Martin von Gagern, #560030)
+
+* A new transport based on GIO (the gnome i/o library) provides access to
+  samba shares, webdav using gio+smb and gio+dav. It is also possible to
+  use gio for some already existing transport methods as gio+file,
+  gio+sftp, gio+ftp. 
+  (Mattias Eriksson)
+
+
+Controlling plugins
+*******************
+
+* Plugins can be disabled by defining ``BZR_DISABLE_PLUGINS`` as
+  a list of plugin names separated by ':' (';' on windows).
+  (Vincent Ladeuil, #411413)
+
+* Plugins can be loaded from arbitrary locations by defining
+  ``BZR_PLUGINS_AT`` as a list of ``name@path`` separated by ':' (';' on
+  Microsoft
+  Windows). This takes precedence over ``BZR_PLUGIN_PATH`` for the
+  specified plugins, and is expected to be most useful for plugin
+  developers.
+  (Vincent Ladeuil, #82693)
+
+
+Apport crash reporting
+**********************
+
+* If the Apport crash-reporting tool is available, bzr crashes are now
+  stored into the ``/var/crash`` apport spool directory, and the user is
+  invited to report them to the developers from there, either
+  automatically or by running ``apport-bug``.  No information is sent
+  without specific permission from the user.  (Martin Pool, #515052)
+
 
 Improved Launchpad integration
-------------------------------
-
-Merges can be proposed on Launchpad with the new **lp-propose-merge**
-command.
-
-
-Notable plugin enhancements
----------------------------
-
-The grep plugin ...
+******************************
+
+* Merges can be proposed on Launchpad with the new ``lp-propose-merge``
+  command.
 
 
 Better documentation
---------------------
-
-Numerous improvements have been made to the developer documentation.
-
-
-Enhanced GUI clients
---------------------
-
-(Details coming soon.)
+********************
+
+* ``bzr help patterns`` now explains case insensitive patterns and
+  points to Python regular expression documentation.
+  (Parth Malwankar, #594386)
+
+* Numerous improvements have been made to the developer documentation.
+
+  
+Changes to plugins
+******************
+
+
+bzr grep
+========
+
+The `grep plugin <https://launchpad.net/bzr-grep>`_  has developed well
+during the bzr 2.2 cycle.  bzr grep can search the versioned files in the
+working tree, or in one or a series of revisions, or it can search through
+only the changes in a revision range.
+
+qbzr
+====
+
+`qbzr <https://launchpad.net/qbzr>`_, a cross-platform graphical interface
+to Bazaar, gained many features and fixes in its 0.19 release, including:
+
+* qannotate has new look and feel; with new features: find text and goto
+  to line.
+
+* Improved performance of qlog, and treewidget-based dialogs (qcommit,
+  qadd, qrevert etc.)
+
+* qpush, qmerge, etc.: when there are uncommitted changes in the working
+  tree, user has the option to commit, or revert.
+
+* qcommit: user can update bound branch/checkout if it is not up to date.
+
+* Better support of Mac OS X: dialog windows no more start in background.
+
+* qlog: Context menu actions for tag and revert will now show a branch
+  menu if more than one branch is open.
+
+* qlog: more context menu actions for update, cherry-pick, and reverse
+  cherry-pick.
+
+* Language of GUI can be set in DEFAULT section of bazaar.conf 
+  as ``language = code``. Language codes are the same 
+  as for ``LANG`` environment variable. 
+  Environment variable ``LANGUAGE`` still preferred over settings 
+  in bazaar.conf.
+
+
+Platform-specific changes
+*************************
+
+Microsoft Windows
+=================
+
+* There's a new py2exe windows program ``bzrw.exe``, which allows for starting a Bazaar GUI with out have a console open in the background.  (Gary van der Merwe, #433781`) 
+
+* The all-in-one Windows installer will now be built with docstrings stripped
+  from the library zip, reducing the size and slightly improving cold startup
+  time. Bundled plugins are unchanged for the moment, but if adding other new
+  plugins to an all-in-one installation, ensure they are compiled and
+  installed with -O1 or help may not work. (Martin [gz])
+
+* Parsing of command lines, for example in ``diff --using``, no longer
+  treats backslash as an escape character on Windows.   (Gordon Tyler,
+  #392248)
+
+
+API changes
+***********
+
+* BzrError subclasses no longer support the name "message" to be used
+  as an argument for __init__ or in _fmt format specification as this
+  breaks in some Python versions. errors.LockError.__init__ argument
+  is now named "msg" instead of earlier "message".
+  (Parth Malwankar, #603461)
+
+* The old ``bzr selftest --benchmark`` option has been removed.
+  <https://launchpad.net/bzr-usertest> is an actively-maintained
+  macrobenchmark suite.
+  (Martin Pool)
+
+* bzrlib library users now need to call ``__enter__`` and ``__exit__`` on
+  the result of ``bzrlib.initialize``. This change was made when fixing
+  the bad habit recent bzr versions have had of leaving progress bars 
+  behind on the screen. That required calling another function before
+  exiting the program, and it made sense to provide a full context
+  manager at the same time. (Robert Collins)
+
+* The ``bzr`` front end now requires a ``bzrlib.ui.ui_factory`` which is a
+  context manager in the Python 2.5 and above sense. The bzrlib base class
+  is such a manager, but third party UI factories which do not derive from
+  ``bzrlib.ui.UIFactory`` will be incompatible with the command line front
+  end.
+
+* URLs like ``foo:bar/baz`` are now always parsed as a URL with scheme "foo"
+  and path "bar/baz", even if bzr does not recognize "foo" as a known URL
+  scheme.  Previously these URLs would be treated as local paths.
+  (Gordon Tyler)
 
 
 Further information
--------------------
+*******************
 
-For more detailed information on the changes made, be sure to check
+For more detailed information on the changes made, see the
 the :doc:`../release-notes/index` for:
 
 * the interim bzr `milestones <https://launchpad.net/bzr/2.2>`_
@@ -66,5 +310,5 @@
 
 * :doc:`whats-new-in-2.1`
 
-Enjoy,
-The Bazaar Development Team
+
+.. vim: ft=rst

=== added file 'doc/en/whats-new/whats-new-in-2.3.txt'
--- doc/en/whats-new/whats-new-in-2.3.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/whats-new-in-2.3.txt	2011-07-15 08:25:00 +0000
@@ -0,0 +1,205 @@
+*************************
+What's New in Bazaar 2.3?
+*************************
+
+Bazaar 2.3 has been released on the 3rd of February 2011 and marks the start
+of another long-term-stable series. From here, we will only make bugfix
+releases on the 2.3 series (2.3.1, etc), while 2.4 will become our new
+development series. The 2.1 and 2.2 series will also continue to get
+bugfixes. (Currently 2.0 is planned to be EOLed circa September 2011.)
+
+This document accumulates a high level summary of what's changed.
+See the
+:doc:`../release-notes/index` for a full list.
+
+Users are encouraged to upgrade from the other stable series.  This document
+outlines the improvements in Bazaar 2.3 vs Bazaar 2.2. As well as summarizing
+improvements made to the core product, it highlights enhancements within the
+broader Bazaar world of potential interest to those upgrading.
+
+Bazaar 2.3.1 includes all the fixes in the un-released 2.0.7, 2.1.4 and 2.2.5
+versions that weren't included in 2.3.0 and fixes some bugs on its own.
+
+Bazaar 2.3.2 is a bugfix release that was never released.
+
+Bazaar 2.3.3 is a bugfix release including the fixes in 2.3.2 and
+fixing the test helpers deprecated by python-2.7.
+
+Bazaar 2.3.4 is a bugfix release.
+
+See the :doc:`../release-notes/index` for details.
+
+Bazaar 2.3 is fully compatible both locally and on the network with 2.0, 2.1,
+and 2.2. It can read and write repositories generated by all previous
+versions.
+
+Changed Behaviour
+*****************
+
+* Committing a new revision in a stacked branch is now supported, as long as
+  you are using the current repository format (2a). It will preserve the
+  stacking invariants, etc, so that fetching after commit is guaranteed to
+  work. (John Arbash Meinel, #375013)
+
+* Support for some old development formats have been removed:
+  ``development-rich-root``, ``development6-rich-root``, and
+  ``development7-rich-root``.  These formats were always labelled experimental
+  and not used unless the user specifically asked for them.  If you have
+  repositories using these old formats you should upgrade them to ``2a`` using
+  Bazaar 2.2.  (Andrew Bennetts)
+
+* The default ``ignore`` file created by Bazaar will contain ``__pycache__``,
+  which is the name of the directory that will be used by Python to store
+  bytecode files.
+  (Andrea Corbellini, #626687)
+
+* The default sort order for the ``bzr tags`` command now uses a natural sort
+  where numeric substrings are sorted numerically.  The previous default was
+  "asciibetical" where tags were sorted by the characters they contained.  To
+  get the old behavior, one can use ``bzr tags --sort=alpha``.
+  (Neil Martinsen-Burrell, #640760)
+
+* On platforms other than Windows and Mac OS X, Bazaar will use configuration
+  files that live in $XDG_CONFIG_HOME/bazaar if that directory exists.  This
+  allows interested individuals to conform to the XDG Base Directory
+  specification.  The plugin location has not changed and is still
+  ~/.bazaar/plugins.  To use a different directory for plugins, use the
+  environment variable BZR_PLUGIN_PATH.  (Neil Martinsen-Burrell, #195397)
+
+* ``bzr upgrade`` now operates recursively when run on a shared
+  repository, automatically upgrading the branches within it, and has
+  grown additional options for showing what it will do and cleaning up
+  after itself.  (Ian Clatworthy, Matthew Fuller, #89830, #374734, #422450)
+
+Launchpad integration
+*********************
+
+* The ``lp:`` prefix will now use your known username (from
+  ``bzr launchpad-login``) to expand ``~`` to your username.  For example:
+  ``bzr launchpad-login user && bzr push lp:~/project/branch`` will now
+  push to ``lp:~user/project/branch``.  (John Arbash Meinel)
+
+* Launchpad has announced that the ``edge.launchpad.net`` instance is
+  deprecated and may be shut down in the future
+  <http://blog.launchpad.net/general/edge-is-deprecated>.  Bazaar has therefore
+  been updated in this release to talk to the main (``launchpad.net``) servers,
+  rather than the ``edge`` ones.
+
+Performance improvements
+************************
+
+* ``bzr revert`` and ``bzr status`` are up to 15% faster on large trees
+  with many changes by not repeatedly building a list of all file-ids.
+  (Andrew Bennetts)
+
+* ``bzr send`` uses less memory.
+  (John Arbash Meinel, #614576)
+
+* Fetches involving stacked branches and branches with tags now do slightly less
+  I/O, and so does branching from an existing branch.  This also improves the
+  network performance of these operations.  (Andrew Bennetts)
+
+* Inventory entries now consume less memory (on 32-bit Ubuntu file entries
+  have dropped from 68 bytes to 40, and directory entries from 120 bytes
+  to 48).  This affects most operations, and depending on the size of the
+  tree may substantially improve the speed of operations like ``bzr
+  commit``.  (Andrew Bennetts)
+
+* Lower memory consumption when reading many chk index pages. Helpful for
+  things like ``bzr co`` or ``bzr ls -R`` on large trees.
+  (John Arbash Meinel)
+
+* When building new working trees, default to reading from the repository
+  rather than the source tree unless explicitly requested. (via
+  ``--files-from`` and ``--hardlink`` for ``bzr branch`` and
+  ``bzr checkout``. Generally, 2a format repositories extract
+  content faster than seeking and reading content from another tree,
+  especially in cold-cache situations. (John Arbash Meinel, #607298)
+
+New revision specifiers
+***********************
+
+* The ``mainline`` revision specifier has been added.  It takes another revision
+  spec as its input, and selects the revision which merged that revision into
+  the mainline.
+  
+  For example, ``bzr log -vp -r mainline:1.2.3`` will show the log of the
+  revision that merged revision 1.2.3 into mainline, along with its status
+  output and diff.  (Aaron Bentley)
+
+* The ``annotate`` revision specifier has been added.  It takes a path and a
+  line as its input (in the form ``path:line``), and selects the revision which
+  introduced that line of that file.
+
+  For example: ``bzr log -vp -r annotate:bzrlib/transform.py:500`` will select
+  the revision that introduced line 500 of transform.py, and display its log,
+  status output and diff.
+
+  It can be combined with ``mainline`` to select the revision that landed this
+  line into trunk, like so: 
+  ``bzr log -vp -r mainline:annotate:bzrlib/transform.py:500``
+  (Aaron Bentley)
+
+Testing/Bug reporting
+*********************
+
+* Shell-like scripts can now be run directly from the command line without
+  writing a python test. This should help users adding reproducing recipes
+  to bug reports. (Vincent Ladeuil)
+
+
+Improved conflict handling
+**************************
+
+* ``pull``, ``merge`` or ``switch`` can lead to conflicts when deleting a
+  versioned directory contains unversioned files. The cause of the conflict
+  is that deleting the directory will orphan the unversioned files so the
+  user needs to instruct ``bzr`` what do to do about these orpahns. This is
+  controlled by setting the ``bzr.transform.orphan_policy`` configuration
+  variable with a value of ``move``. In this case the unversioned files are
+  moved to a ``bzr-orphans`` directory at the root of the working tree. The
+  default behaviour is specified (if needed) by setting the variable to
+  ``conflict``.  (Vincent Ladeuil, #323111)
+
+* ``bzr resolve --take-this`` and ``bzr resolve --take-other`` can now be
+  used for text conflicts. This will ignore the differences that were merged
+  cleanly and replace the file with its content in the current branch
+  (``--take-this``) or with its content in the merged branch
+  (``--take-other``). (Vincent Ladeuil, #638451)
+
+* ``bzr resolve`` now provides more feedback about the conflicts just
+  resolved and the remaining ones. (Vincent Ladeuil)
+
+Documentation
+*************
+
+* A beta version of the documentation is now available in GNU TexInfo
+  format, used by emacs and the standalone ``info`` reader.
+  (Vincent Ladeuil, #219334)
+
+Configuration
+*************
+
+``bzr`` can be configured via environment variables, command-line options
+and configurations files. We've started working on unifying this and give
+access to more options. The first step is a new ``bzr config`` command that
+can be used to display the active configuration options in the current
+working tree or branch as well as the ability to set or remove an
+option. Scripts can also use it to get only the value for a given option.
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the
+the :doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/2.3>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+
+
+.. vim: ft=rst

=== added file 'doc/en/whats-new/whats-new-in-2.4.txt'
--- doc/en/whats-new/whats-new-in-2.4.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/whats-new-in-2.4.txt	2011-09-13 12:29:22 +0000
@@ -0,0 +1,159 @@
+**********************************
+What's New in Bazaar 2.4 (Oronsay)
+**********************************
+
+Bazaar 2.4 has been released on the 8th of August 2011 and marks the start
+of a new long-term-stable series. From here, we will only make bugfix
+releases on the 2.4 series (2.4.1, etc, and support it until February 2013),
+while 2.5 will become our new development series.
+
+This document accumulates a high level summary of what's changed.  See the
+:doc:`../release-notes/index` for a full list.
+
+Users are encouraged to upgrade from the other stable series.  This
+document outlines the improvements in Bazaar 2.4 vs Bazaar 2.3. As well as
+summarizing improvements made to the core product, it highlights
+enhancements within the broader Bazaar world of potential interest to
+those upgrading.
+
+Bazaar 2.4.0 is fully compatible both locally and on the network with 2.0,
+2.1, 2.2 and 2.3, and can read and write repositories generated by all
+previous versions.
+
+Bazaar 2.4.1 includes all the fixes in the un-released 2.0.7, 2.1.5, 2.2.6
+and 2.3.5 versions that weren't included in 2.4.0 and fixes some bugs on its
+own.
+
+
+Dropping Python2.4 and Python2.5 support
+****************************************
+
+Bazaar 2.4.0 is the first version of bzr to drop support for python
+versions before 2.6. The 2.3 series will still be maintained in bugfix
+mode for people who need older versions of python. You can also use
+Launchpad to nominate fixes for 2.3. We can't fix everything, but that can
+help guide us to what people are running into. Upgrading to python2.6
+allows us to clean up the syntax in our files, and makes it easier to work
+on python3 compatibility.
+
+External merge tools
+********************
+
+External merge tool configuration has been added to ``bzr`` core. The name
+and commandline of one or more external merge tools can be defined in
+bazaar.conf. See the help topic ``configuration`` for more details.
+
+Tagged revisions are copied
+***************************
+
+When tags are copied from a branch, the associated revisions are now copied
+too if the config entry ``branch.fetch_tags`` is set to True.  Operations
+like branching, merging or pulling will still always copy new tags visible
+in ``bzr tags``. When the config is set, it will now also copy the
+revisions and their ancestry.  This way tagged revisions will always be
+present, so that operations like ``bzr log -r tag:foo`` will always work.
+
+Deprecated command synonyms
+***************************
+
+Two built-in synonyms for ``bzr branch`` have been deprecated: ``clone`` and
+``get``.
+
+Command options
+***************
+
+* The ``bzr log`` and ``bzr missing`` commands now accept ``-S`` as a
+  shorthand for ``--short``.
+
+Configuration files
+*******************
+
+Option values can now refer to other options in the same configuration file
+by enclosing them in curly brackets (``{option}``). This is an opt-in
+feature controlled by the ``bzr.config.expand`` option that should be
+declared in ``bazaar.conf`` and no other file.
+
+Changelog merge plugin
+**********************
+
+The ``changelog_merge`` plugin has been added.  It provides a merge hook
+to automate merging of changes to ``ChangeLog`` files in GNU's change log
+format.  Refer to ``bzr help changelog_merge`` for documentation on how to
+enable it and what it can do.
+
+Faster operations on Large Trees
+********************************
+
+Many bzr commands used to run into pathological behavior on large trees
+(>10k files), reading the inventory data in random order causing cache
+thrashing (the fix was backported to bzr-2.3.2). We also updated several
+code paths that were updating the WT state using an O(tree) operation to
+one that was an O(changes) operation.  A possibly incomplete list is as
+follows for running commands on a 70k file tree::
+
+    bzr-2.3.1 bzr-2.3.2 bzr-2.4  action
+    3m39s         1m08s   1m03s  bzr co --lightweight
+      38s            8s      2s  bzr revert (in a clean tree)
+    4m47s         3m56s     15s  bzr merge
+    4m45s           20s      3s  bzr pull
+    4m58s         3m00s      2s  bzr up
+    9m33s           21s     19s  bzr uncommit (including a merge)
+    4m44s           17s      2s  bzr uncommit (simple commit)
+    
+This is a smaller table of times for the Launchpad tree (~8k items)::
+
+    bzr-2.3.1   bzr-2.4     action
+    5.3s        5.2s        bzr co --lightweight
+    0.9s        0.3s        bzr revert
+    1.4s        0.4s        bzr pull
+    3.9s        3.7s        bzr uncommit (with merge)
+    0.9s        0.3s        bzr uncommit (without merge)
+
+Faster stacked branches
+***********************
+
+Creating a stacked branch from a smart server with ``bzr branch
+--stacked`` is a bit faster now.  For small branches it does 20% fewer
+network roundtrips.  Other operations where a local branch is stacked on a
+branch hosted on a smart server will also benefit.
+
+More export control
+*******************
+
+When exporting a tree, you may now use get_export_generator() to 
+do an operation after each file is exported. 
+
+Testing
+*******
+
+The ``selftest --exclude`` option can now be specified multiple times and
+the tests that match any of the specified patterns will be excluded. Only
+the last specified pattern was previously taken into account.
+
+Digital Signature Verification
+******************************
+
+A new command ``bzr verify-signatures`` has been added to check that commits
+are correctly signed with trusted keys by GPG.  This requires python-gpgme to
+be installed.  ``bzr log`` has gained a ``--signatures`` option to list the
+validity of signatures for each commit.  New config options ``acceptable_keys``
+and ``validate_signatures_in_log`` can be set to control options to these
+commands.
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the the
+:doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/2.4>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+* :doc:`whats-new-in-2.3`
+
+..
+   vim: tw=74 ft=rst ff=unix

=== added file 'doc/en/whats-new/whats-new-in-2.5.txt'
--- doc/en/whats-new/whats-new-in-2.5.txt	1970-01-01 00:00:00 +0000
+++ doc/en/whats-new/whats-new-in-2.5.txt	2011-10-06 14:39:49 +0000
@@ -0,0 +1,49 @@
+*************************
+What's New in Bazaar 2.5?
+*************************
+
+Bazaar 2.5 is still under development, and will be released in February 2012.
+This document accumulates a high level summary of what's changed.  See the
+:doc:`../release-notes/index` for a full list.
+
+Users are encouraged to upgrade from the other stable series.  This
+document outlines the improvements in Bazaar 2.5 vs Bazaar 2.4. As well as
+summarizing improvements made to the core product, it highlights
+enhancements within the broader Bazaar world of potential interest to
+those upgrading.
+
+Bazaar 2.5.0 is fully compatible both locally and on the network with 2.0,
+2.1, 2.2, 2.3 and 2.4, and can read and write repositories generated by all
+previous versions.
+
+Overriding configuration options from the command line
+******************************************************
+
+The ``-O`` parameter available for all bzr commands allows a user to
+override a configuration option from the command line. For example::
+
+  bzr pull -v -Olog_format=line
+
+will change the way the pulled revisions are displayed (the default log
+format is ``long``). This a work in progress and only some options are
+supported so far.
+
+
+Further information
+*******************
+
+For more detailed information on the changes made, see the the
+:doc:`../release-notes/index` for:
+
+* the interim bzr `milestones <https://launchpad.net/bzr/2.5>`_
+* the plugins you use.
+
+For a summary of changes made in earlier releases, see:
+
+* :doc:`whats-new-in-2.1`
+* :doc:`whats-new-in-2.2`
+* :doc:`whats-new-in-2.3`
+* :doc:`whats-new-in-2.4`
+
+..
+   vim: tw=74 ft=rst ff=unix

=== modified file 'doc/es/index.txt'
--- doc/es/index.txt	2009-12-02 20:34:07 +0000
+++ doc/es/index.txt	2010-08-13 19:08:57 +0000
@@ -3,7 +3,7 @@
 =================================
 
 Las ultimas versiones de estos documentos se encuentran disponibles
-en la pagina de  Bazaar, http://doc.bazaar-vcs.org/en/.
+en la pagina de  Bazaar, http://doc.bazaar.canonical.com/en/.
 
 Documentacion Principal
 =======================
@@ -21,20 +21,20 @@
 
 Mudandose a Bazaar (enlances web):
 
-* `Guias de Migracion <http://doc.bazaar-vcs.org/migration/en/>`_
+* `Guias de Migracion <http://doc.bazaar.canonical.com/migration/en/>`_
   - para equipos que mudan el historial de otros SCV
 
-* `Guias de Plugins <http://doc.bazaar-vcs.org/plugins/en/>`_
+* `Guias de Plugins <http://doc.bazaar.canonical.com/plugins/en/>`_
 
 Otros Documentos (enlances web):
 
-* `Glosario <http://bazaar-vcs.org/BzrGlossary>`_
+* `Glosario <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
 * `Preguntas Frecuentes <https://answers.launchpad.net/bzr>`_
 
-* `Referencia del API bzrlib <http://bazaar-vcs.org/BzrLib>`_
+* `Referencia del API bzrlib <http://wiki.bazaar.canonical.com/BzrLib>`_
 
 Especificaciones tecnicas, planes a futuro y varias
 notas tecnicas varias en Ingles:
 
-* `Catalogo de Documentos para Desarrolladores <http://doc.bazaar-vcs.org/latest/developers/>`_
+* `Catalogo de Documentos para Desarrolladores <http://doc.bazaar.canonical.com/latest/developers/>`_

=== modified file 'doc/es/mini-tutorial/index.txt'
--- doc/es/mini-tutorial/index.txt	2009-12-02 20:34:07 +0000
+++ doc/es/mini-tutorial/index.txt	2010-11-12 22:36:19 +0000
@@ -30,10 +30,10 @@
 Para otras plataformas y para instalar desde el código fuente, vea las
 páginas de Descarga_ e Instalación_.
 
-.. _instrucciones de instalación para Windows: http://bazaar-vcs.org/WindowsDownloads
-.. _instrucciones de instalación para Mac OS X: http://bazaar-vcs.org/MacOSXBundle
-.. _Descarga: http://bazaar-vcs.org/Download
-.. _Instalación: http://bazaar-vcs.org/InstallationFaq
+.. _instrucciones de instalación para Windows: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _instrucciones de instalación para Mac OS X: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _Descarga: http://wiki.bazaar.canonical.com/Download
+.. _Instalación: http://wiki.bazaar.canonical.com/InstallationFaq
 
 
 Preséntese
@@ -77,7 +77,7 @@
 Si parece que no ha ocurrido nada no se preocupe. Bazaar ha creado un
 branch_ dónde guardará sus archivos y su histórico de revisiones.
 
-.. _branch: http://bazaar-vcs.org/Branch
+.. _branch: http://wiki.bazaar.canonical.com/Branch
 
 El siguiente paso es decirle a Bazaar a que archivos desea seguirles la pista.
 Ejecutando ``bzr add`` agregará recursivamente todos los elementos dentro del
@@ -143,7 +143,7 @@
     Importación inicial
 
 
-Publicando su branch con sftp
+Publicando su branch con SFTP
 =============================
 
 Hay un par de maneras para publicar su branch. Si ya tiene un servidor
@@ -166,8 +166,8 @@
 
  $ bzr branch http://www.example.com/miproyecto
 
-**Nota:** para utilizar sftp deberá instalar ``paramiko`` y
-``pyCrypto``. Vea http://bazaar-vcs.org/InstallationFaq para más información.
+**Nota:** para utilizar SFTP deberá instalar ``paramiko`` y
+``pyCrypto``. Vea http://wiki.bazaar.canonical.com/InstallationFaq para más información.
 
 
 Publicando su branch con Launchpad

=== modified file 'doc/index.es.txt'
--- doc/index.es.txt	2010-02-23 03:06:44 +0000
+++ doc/index.es.txt	2010-08-13 19:08:57 +0000
@@ -26,19 +26,19 @@
 
 Mudandose a Bazaar (enlances web):
 
-* `Guias de Mudanzas <http://bazaar-vcs.org/BzrSwitching>`_
+* `Guias de Mudanzas <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   - para los usuarios que se mudan de otros SCV
 
-* `Guias de Migracion <http://bazaar-vcs.org/BzrMigration>`_
+* `Guias de Migracion <http://wiki.bazaar.canonical.com/BzrMigration>`_
   - para equipos que mudan el historial de otros SCV
 
 Otros Documentos (enlances web):
 
-* `Glosario <http://bazaar-vcs.org/BzrGlossary>`_
-
-* `Preguntas Frecuentes <http://bazaar-vcs.org/FAQ>`_
-
-* `Referencia del API bzrlib <http://bazaar-vcs.org/BzrLib>`_
+* `Glosario <http://wiki.bazaar.canonical.com/BzrGlossary>`_
+
+* `Preguntas Frecuentes <http://wiki.bazaar.canonical.com/FAQ>`_
+
+* `Referencia del API bzrlib <http://wiki.bazaar.canonical.com/BzrLib>`_
 
 Especificaciones tecnicas, planes a futuro y varias
 notas tecnicas varias en Ingles:

=== modified file 'doc/index.ja.txt'
--- doc/index.ja.txt	2010-07-07 11:21:19 +0000
+++ doc/index.ja.txt	2010-08-17 06:45:33 +0000
@@ -3,7 +3,7 @@
 ===========================
 
 これらのドキュメントの最新版はBazaarのドキュメントのサイト、 http://doc.bazaar.canonical.com/ から入手可能で、
-詳しい情報は http://bazaar-vcs.org/Documentation のページからリンクされています。
+詳しい情報は http://wiki.bazaar.canonical.com/Documentation のページからリンクされています。
 
 
 コアドキュメント
@@ -42,15 +42,15 @@
 ウェブリンク
 =============
 
-* `切り替えガイド <http://bazaar-vcs.org/BzrSwitching>`_
+* `切り替えガイド <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| 別のVCSツールから移ってきたユーザー用
 
-* `移行ガイド <http://bazaar-vcs.org/BzrMigration>`_
+* `移行ガイド <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| 別のVCSツールから履歴を移行するチーム用
 
-* `用語 <http://bazaar-vcs.org/BzrGlossary>`_
+* `用語 <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
-* `よく聞かれる質問 <http://bazaar-vcs.org/FAQ>`_
+* `よく聞かれる質問 <http://wiki.bazaar.canonical.com/FAQ>`_
 
 
 TortoiseBzrをインストールするには？

=== modified file 'doc/index.ru.txt'
--- doc/index.ru.txt	2010-07-07 11:21:19 +0000
+++ doc/index.ru.txt	2010-08-17 06:45:33 +0000
@@ -4,7 +4,7 @@
 
 Последняя версия этих документов доступа со страницы документации
 на сайте Bazaar, <http://doc.bazaar.canonical.com/>. Еще больше информации
-можно получить по адресу <http://bazaar-vcs.org/Documentation>.
+можно получить по адресу <http://wiki.bazaar.canonical.com/Documentation>.
 
 Основная документация
 =====================
@@ -38,18 +38,18 @@
 Ссылки в сети
 =============
 
-* `Руководства по переходу <http://bazaar-vcs.org/BzrSwitching>`_
+* `Руководства по переходу <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| для пользователей переходящих с других систем контроля версий
 
-* `Руководство по миграции <http://bazaar-vcs.org/BzrMigration>`_
+* `Руководство по миграции <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| для команд переносящих историю с других систем контроля версий
 
-* `Словарь терминов <http://bazaar-vcs.org/BzrGlossary>`_ (англ.),
+* `Словарь терминов <http://wiki.bazaar.canonical.com/BzrGlossary>`_ (англ.),
   см. также `русскую версию`__
 
 __  http://groups.google.com/group/ru_bzr/web/%D0%B3%D0%BB%D0%BE%D1%81%D1%81%D0%B0%D1%80%D0%B8%D0%B9
 
-* `Часто задаваемые вопросы <http://bazaar-vcs.org/FAQ>`_ (англ.)
+* `Часто задаваемые вопросы <http://wiki.bazaar.canonical.com/FAQ>`_ (англ.)
 
 Другие языки
 ============

=== modified file 'doc/index.txt'
--- doc/index.txt	2010-07-07 11:21:19 +0000
+++ doc/index.txt	2010-08-17 06:45:33 +0000
@@ -4,7 +4,7 @@
 
 The latest version of these documents are available from Bazaar's
 documentation site, <http://doc.bazaar.canonical.com/>, and more documentation
-may be linked from <http://bazaar-vcs.org/Documentation>.
+may be linked from <http://wiki.bazaar.canonical.com/Documentation>.
 
 Core Documentation
 ==================
@@ -43,15 +43,15 @@
 Web links
 =========
 
-* `Switching Guides <http://bazaar-vcs.org/BzrSwitching>`_
+* `Switching Guides <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| for users moving from another VCS tool
 
-* `Migration Guide <http://bazaar-vcs.org/BzrMigration>`_
+* `Migration Guide <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| for teams migrating history from another VCS tool
 
-* `Glossary <http://bazaar-vcs.org/BzrGlossary>`_
+* `Glossary <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
-* `Frequently Asked Questions <http://bazaar-vcs.org/FAQ>`_
+* `Frequently Asked Questions <http://wiki.bazaar.canonical.com/FAQ>`_
 
 
 Other Languages

=== modified file 'doc/ja/index.txt'
--- doc/ja/index.txt	2010-07-07 11:21:19 +0000
+++ doc/ja/index.txt	2011-05-18 13:12:59 +0000
@@ -2,8 +2,10 @@
 Bazaarのメインドキュメント
 ===========================
 
-これらのドキュメントの最新版はBazaarのドキュメントのサイト、 http://doc.bazaar-vcs.org/ から入手可能で、
-詳しい情報は http://bazaar-vcs.org/Documentation のページからリンクされています。
+これらのドキュメントの最新版はBazaarのドキュメントのサイト、
+http://doc.bazaar.canonical.com/en/ から入手可能で、
+詳しい情報は http://wiki.bazaar.canonical.com/Documentation
+のページからリンクされています。
 
 
 目次
@@ -14,21 +16,25 @@
 
    tutorials/index
    user-guide/index
-   user-reference/bzr_man
+   upgrade-guide/index
+   user-reference/index
 
+.. no translated documents
+   quick-reference/index
+   admin-guide/index
 
 ウェブリンク
 =============
 
-* `切り替えガイド <http://bazaar-vcs.org/BzrSwitching>`_
+* `切り替えガイド <http://wiki.bazaar.canonical.com/BzrSwitching>`_
   |--| 別のVCSツールから移ってきたユーザー用
 
-* `移行ガイド <http://bazaar-vcs.org/BzrMigration>`_
+* `移行ガイド <http://wiki.bazaar.canonical.com/BzrMigration>`_
   |--| 別のVCSツールから履歴を移行するチーム用
 
-* `用語 <http://bazaar-vcs.org/BzrGlossary>`_
+* `用語 <http://wiki.bazaar.canonical.com/BzrGlossary>`_
 
-* `よく聞かれる質問 <http://bazaar-vcs.org/FAQ>`_
+* `よく聞かれる質問 <http://wiki.bazaar.canonical.com/FAQ>`_
 
 
 TortoiseBzrをインストールするには？
@@ -39,10 +45,6 @@
 その後の作業はインストールウィザードに従います。
 インストールウィザードが終了した後で再起動します。
 
-最新バージョンでも正常に動作しないことがあるので、
-その場合は古いバージョンをインストールします。
-2009年1月時点で筆者はbzr-setup-1.9.exeの基本的な動作を確認しています。
-
 もしPythonインタープリタにbzrをインストールした場合、インストールしたディレクトリによって
 bzr.exe(デフォルトでは ``C:\Program Files\Bazaar`` )よりもbzr.bat(``C:\PythonXX\Scripts``)が
 優先されるので、コマンドプロンプトでbzrと入力したときにbzr.exeが実行されるようにするには、

=== modified file 'doc/ja/mini-tutorial/index.txt'
--- doc/ja/mini-tutorial/index.txt	2009-10-29 16:52:18 +0000
+++ doc/ja/mini-tutorial/index.txt	2011-05-18 06:01:08 +0000
@@ -7,11 +7,12 @@
 イントロダクション
 ===================
 
-Bazaarは分散型バージョン管理システムで、ソフトウェアプロジェクトの共同作業を楽にしてくれます。
+Bazaarは分散型バージョン管理システムで、ソフトウェアプロジェクトの
+共同作業を楽にしてくれます。
 
-これから5分ほどで、ファイルをバージョン管理下に置き、変更をそれらに記録して、\
-作業内容を確認し、公開して作業内容をマージしてもらうためにプロジェクトのtrunkに\
-送る方法などを学びます。
+これから5分ほどで、ファイルをバージョン管理下に置き、変更を記録して、
+作業内容を確認し、公開して作業内容をマージしてもらうためにプロジェクトの
+trunk に送る方法などを学びます。
 
 詳細な紹介内容を望むのであれば、 `さらに学ぶ`_ をご覧ください。
 
@@ -29,10 +30,10 @@
 別のプラットフォームとソースコードからインストールする方法に関しては、 ダウンロード_
 と インストール方法_ のページを参照してください。
 
-.. _Windowsのためのインストールの手引き: http://bazaar-vcs.org/WindowsDownloads
-.. _Mac OS Xのためのインストールの手引き: http://bazaar-vcs.org/MacOSXBundle
-.. _ダウンロード: http://bazaar-vcs.org/Download
-.. _インストール方法: http://bazaar-vcs.org/InstallationFaq
+.. _Windowsのためのインストールの手引き: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _Mac OS Xのためのインストールの手引き: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _ダウンロード: http://wiki.bazaar.canonical.com/Download
+.. _インストール方法: http://wiki.bazaar.canonical.com/InstallationFaq
 
 
 まずは自己紹介
@@ -74,11 +75,11 @@
 何も起きていないように見えても心配しないでください。
 Bazaarはファイルとリビジョンの履歴を保存する branch_ を作りました。
 
-.. _branch: http://bazaar-vcs.org/Branch
+.. _branch: http://wiki.bazaar.canonical.com/Branch
 
 次のステップはBazaarに管理して欲しいファイルを教えることです。
 ``bzr add`` を実行するとすべてのディレクトリとファイルがプロジェクトに\
-再帰的に追加されます::
+再帰的に追加されます。 ::
 
  $ bzr add
  added subdirectory
@@ -88,15 +89,15 @@
  added subdirectory/test4.txt
 
 次に、これらをブランチにコミットしてスナップショットをとります。
-コミットを行った理由を説明するメッセージを追加します::
+コミットを行った理由を説明するメッセージを追加します。 ::
 
   $ bzr commit -m "Initial import"
 
 Bazaarは分散型バージョン管理システムなので、コミットするために\
 サーバーに接続する必要はありません。
 代わりに、Bazaarはブランチとすべてのコミットをあなたが作業している\
-ディレクトリ内部に保存します;
-``.bzr`` というサブディレクトリをご覧ください。
+ディレクトリ内部に ``.bzr`` というサブディレクトリを作ってそこに
+保存します。
 
 
 ファイルを変更する
@@ -104,7 +105,7 @@
 
 ファイルを変更してブランチにその変更をコミットしてみましょう。
 
-好きなエディタで ``test1.txt`` を編集し、何を行ったのかを確認します::
+好きなエディタで ``test1.txt`` を編集し、何を行ったのかを確認します。 ::
 
  $ bzr diff
  === modified file 'test1.txt'
@@ -122,7 +123,7 @@
 リビジョンのログを眺める
 =========================
 
-ログを閲覧することでブランチの履歴がわかります::
+ログを閲覧することでブランチの履歴を調べることができます。 ::
 
   $ bzr log
   ------------------------------------------------------------
@@ -141,58 +142,37 @@
     Initial import
 
 
-sftpでブランチを公開する
-=========================
-
-ブランチを公開する方法は複数あります。
-SFTPサーバーがすでにあるもしくは容易にセットアップできるのであれば、\
-ブランチをそこで公開できます。
-
-そうでなければ、このセクションをとばして、Bazaarのための無料ホスティング\
-サービスである、 Launchpad_ で公開しましょう。
-
-.. _Launchpad: https://launchpad.net/
-
-``www.example.com/myproject`` でブランチを公開することを前提とします::
-
- $ bzr push --create-prefix sftp://your.name@example.com/~/public_html/myproject
- 2 revision(s) pushed.
-
-Bazaarはリモートサーバー上で ``myproject`` ディレクトリを作りブランチを\
-そこにpushします。
-
-これで誰でも次のコマンドを入力すればあなたのブランチをコピーできます::
-
- $ bzr branch http://www.example.com/myproject
-
-**注:** sftpを使うためには、 ``paramiko`` と ``pyCrypto`` をインストールする必要があります。
-詳細は http://bazaar-vcs.org/InstallationFaq を参照してください。
-
-
-Launchpadでブランチを公開する
-==============================
-
-Launchpadはフリーソフトウェアのための開発とホスティングのためのツールが\
-ひとまとめになったものです。これをブランチを公開するために利用できます。
-
-Launchpadのアカウントを持っていなければ、 `アカウントのサインアップのガイド`_
-に従ってアカウントを作り、 `SSHキーを登録`_ してください。
-
-.. _アカウントのサインアップのガイド: https://help.launchpad.net/CreatingYourLaunchpadAccount
-.. _SSHキーを登録: https://launchpad.net/people/+me/+editsshkeys
-
-次のコマンドを(``john.doe`` を自分のLaunchpadアカウント名に変更して)実行してください::
-
- $ bzr push bzr+ssh://john.doe@bazaar.launchpad.net/~john.doe/+junk/myproject
-
-**注:** ``+junk`` はこのブランチがLaunchpad上の特定のプロジェクトに関連していないことを意味します。
-
-これで、誰でも次のコマンドを入力することでブランチのコピーを作ることができます::
-
- $ bzr branch http://bazaar.launchpad.net/~john.doe/+junk/myproject
-
-ブランチとリビジョンの履歴に関する情報は https://code.launchpad.net/people/+me/+junk/myproject
-でも見ることができます。
+ブランチを Launchpad で公開する
+===================================
+
+Launchpad はソフトウェアプロジェクトの開発と運営のためのツールをまとめた
+サイトです。自分のブランチを公開するために Launchpad を利用することができます。
+(もちろん、自分のサーバーや他のホスティングサービス上で公開することもできます)
+
+まだ Launchpad のアカウントを持っていないのであれば、 `account signup guide`_
+に従ってアカウントを作成し、 `SSH 鍵を登録`_ してください。
+
+.. _account signup guide: https://help.launchpad.net/CreatingYourLaunchpadAccount
+.. _SSH 鍵を登録: https://launchpad.net/people/+me/+editsshkeys
+
+次のように、 (``john.doe`` は自分のアカウントのユーザー名に置き換えて)
+タイプしてください。 [#]_ ::
+
+ $ bzr push lp:~john.doe/+junk/myproject
+
+.. [#] ``lp:`` という URL スキーマは bzr 0.92 以降でサポートされています。
+
+**注意**: ``+junk`` の部分は、このブランチが Launchpad 上の特定のプロジェクトに
+属していないことを意味しています。
+
+これで、誰でもあなたのブランチのコピーを、次のようなコマンドで入手できるようになりました。 ::
+
+ $ bzr branch lp:~john.doe/+junk/myproject
+
+ブランチの情報を、履歴も含めて
+https://code.launchpad.net/people/+me/+junk/myproject
+から閲覧することができます。
+
 
 別のブランチから自分用のコピーを作る
 =====================================
@@ -200,23 +180,23 @@
 他人のコードに取り組むために、ブランチのコピーを作ることができます。
 実際の世界の例として、BazaarのGTKインターフェイスを見てみましょう::
 
-  $ bzr branch http://bazaar.launchpad.net/~bzr/bzr-gtk/trunk bzr-gtk.john
+  $ bzr branch lp:~bzr/bzr-gtk/trunk bzr-gtk.john
   Branched 292 revision(s).
 
-Bazaarはbzr-gtkのtrunkブランチからすべてのファイルをダウンロードして\
+Bazaarはbzr-gtkのtrunkブランチからすべてのファイルをダウンロードして
 リビジョンの履歴をそろえ、bzr-gtk.johnというコピーを作ります。
 
-これで、ブランチのコピーを手に入れたのでネットの接続のあるなしに\
+これで、ブランチのコピーを手に入れたのでネットの接続のあるなしに
 関わらず変更をコミットできます。
-ブランチはいつでも公開することで共有でき、bzr-gtkチームがあなたの作品を\
-使いたいと思ったときにBazaarは彼らがあなたのブランチから彼らのブランチに\
+ブランチはいつでも公開することで共有でき、bzr-gtkチームがあなたの作品を
+使いたいと思ったときにBazaarは彼らがあなたのブランチから彼らのブランチに
 マージし直す作業を簡単にしてくれます。
 
 
 メインのブランチから自分のブランチを更新する
 =============================================
 
-変更を自分のブランチにコミットする一方で、他の人がコードを親のブランチに\
+変更を自分のブランチにコミットしている間に、他の人がコードを元のブランチに\
 コミットしているということもよくあります。
 
 自分のブランチを最新に維持するには、親ブランチから自分のブランチへと変更を\
@@ -273,3 +253,20 @@
 ''foo'' トピックもしくはコマンドを学ぶには::
 
   $ bzr help foo
+
+
+Licence
+=============
+
+Copyright 2007-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later. See
+<http://www.gnu.org/licenses/>.
+
+
+日本語訳について
+-----------------
+この日本語訳は、 `Bazaar-jaグループ <https://groups.google.com/group/bazaar-ja>`_
+がメンテナンスしています。
+
+日本語訳に着いて間違いや質問等ありましたらこちらへお願いします。

=== modified file 'doc/ja/tutorials/centralized_workflow.txt'
--- doc/ja/tutorials/centralized_workflow.txt	2010-02-17 07:15:12 +0000
+++ doc/ja/tutorials/centralized_workflow.txt	2011-05-18 13:12:59 +0000
@@ -1,276 +1,283 @@
-========================================
-集中型ワークフローのチュートリアル
-========================================
-
-
-概要
-====
-
-この文書では、 Bazaar_ を使用することで実現可能なワークフローについて説明します。
-つまり、分散バージョンコントロールシステムである Bazaar_ を、集中型のやり方で使用するためのワークフローです。 
-Bazaar_ は、非常にフレキシブルに設計されており、完全な分散型からほとんど集中型のワークフローまで、\
-いくつかの異なるワークフローを受け入れることができます。
-このワークフローは、初めてのユーザでも簡単に Bazaar_ を使いこなすことができ、\
-分散型と集中型のオペレーションを組み合わせた業務にも対応できるようになっています。
-
-基本的に、この文書はCVSやSubvresionのような集中型バージョンコントロールシステムの経験があるユーザ向けに書かれています。
-これらの環境では、一般的に、コードベースをホストする単一の中央サーバが存在し、複数の人々がこのコードベース上で作業して、\
-お互いの作業の同期をとることになります。 
-また、このワークフローは、一人の開発者が複数のマシーン上で作業をする場合にも適用できます。
-
-.. _Bazaar: http://bazaar-vcs.org
-
-
-初期セットアップ
-================
-
-Bazaar_ がうまく動くようにするための、まあまあ簡単なセットアップ手順があります。
-
-ユーザのEメールの設定
----------------------
-
-ユーザIDは各コミットに保存されます。これは正確だったり一意だったりする必要はありませんが、\
-ログメッセージや注釈で使用されるため、何かしら実在する値を設定しておいた方がよいでしょう。
-
-::  
-
-   % bzr whoami "John Doe <jdoe@organization.com>"
-
-
-ローカルリポジトリのセットアップ
---------------------------------
-
-Bazaar_ のブランチは、通常は履歴のコピーを持っています。そのおかげで分散型での作業ができるわけです。
-最適化の方法の一つとして、関連するブランチ同士の情報を統合して、\
-新しいブランチを作るたびに履歴情報を丸ごとコピーしなくてもいいようにすることができます。
-
-そのための一番いい方法は、 `共用リポジトリ`_ を作成することです。
-一般に、 `共用リポジトリ`_ のサブディレクトリ内に複数のブランチがある場合、お互いの記憶領域を共有します。
-ですので、ホームディレクトリに `共用リポジトリ`_ を作成するようにしましょう。
-そうすれば、その下に作成したすべてのブランチは、履歴情報の領域を共有することになります。
-
-::
-
-  % bzr init-repo --trees ~
-
-
-リモートリポジトリのセットアップ
----------------------------------
-
-作業用の領域とは別に、データを蓄積しておく領域が欲しいというのはよくあることです。
-集中型のシステム(CVS/SVN)では、このようなワークフローが必要になります。
-たいていは、これらは別々のマシーンに分けて配置されます。(そうでないこともあります。)
-実際のところ、これは、特に職場ではとてもよい設定です。
-蓄積されたデータはバックアップを確実にとることができ、開発者のマシーンに何かトラブルが起こっても\
-コミットされたデータはけっして無くなりません。
-
-それでは、プロジェクト内で共有する ``centralhost`` というマシーンを用意しましょう。
-先ほども言いましたが、ディスクの使用量を節約するために、 `共用リポジトリ`_ を使用します。
-
-::
-
-  % bzr init-repo --no-trees sftp://centralhost/srv/bzr/
-
-この手順は、新しくcvsrootやSubversionのリポジトリを作るのと同じようなものだと考えることができます。
-``--no-tree`` オプションの指定によって、ワーキングツリーを作らないようになります。
-中央リポジトリ内のブランチを直接変更することは無いので、このオプションを指定しておくとよいでしょう。
-
-
-既存のプロジェクトからの移行
-=============================
-
-リポジトリができましたので、プロジェクトの作成にかかりましょう。
-たいていの場合、 Bazaar_ でバージョン管理したい作業中のコードがすでにあるはずです。
-もし、そのコードがもともとソース管理されていたのであれば、履歴の情報を保ったまま Bazaar_ に\
-変換する方法がたくさんあります。
-しかしながら、それについてはここでは説明しません。詳しくは、 `Tracking Upstream`_ の\
-"Converting and keeping history"セクションを見てください。
-
-.. _Tracking Upstream: http://bazaar-vcs.org/TrackingUpstream
-
-.. 
-   XXX: プロジェクトの変換について話をするための別の文書が必要なのですが、\
-   今ある中ではTrackingUpstreamが一番よい文書です。
-
-
-Developer 1: 最初のリビジョンを作成する
------------------------------------------
-
-まず最初に、プロジェクトをホストするリモートリポジトリに、ブランチを作成したいと思います。
-仮に、"sigil"というプロジェクトをバージョン管理しようとしているとしましょう。
-
-::
-
-  % bzr init sftp://centralhost/srv/bzr/sigil
-
-これは、CVSで言うところの"HEAD"ブランチ、Subversionなら"trunk"にあたるものだと考えてかまいません。
-これを ``dev`` ブランチと呼ぶことにしましょう。
-
-他のファイルとの競合を避けるために、ホームディレクトリのサブディレクトリ内で作業するのが私の好みです。
-同じように、最終的にでき上がるすべてのブランチを格納できるプロジェクトディレクトリも欲しいですね。 
-::
-
-  % cd ~
-  % mkdir work
-  % cd work
-  % mkdir sigil
-  % cd sigil
-  % bzr checkout sftp://centralhost/srv/bzr/sigil dev
-  % cd dev
-  % cp -ar ~/sigil/* .
-  % bzr add
-  % bzr commit -m "Initial import of Sigil"
-
-前のセクションでは、空のブランチ(``sigil`` ブランチ)を ``centralhost`` に作成して、\
-それをクライアント端末にチェックアウトしたあと、元々あったプロジェクトのファイルを追加しています。
-作業ディレクトリをセットアップするための方法はたくさんありますが、この方法を使うと機能追加用/バグフィックス用\
-のブランチを使った作業が簡単にできます。
-そして、複数のブランチをとてもうまく扱えるというのが、 Bazaar_ の長所の一つなのです。
-
-この場合、リモートブランチのチェックアウトを持っているので、 ``~/work/sigil/dev/`` に対してコミットした内容が、\
-ローカルと ``centralhost`` の両方に自動的に保存される訳です。
-
-
-Developer N: プロジェクトの作業コピーを取得する
---------------------------------------------------
-
-プロジェクトの作成に関するすべての作業は1人目の開発者がしてしまっているので、\
-他のみんなは単にそのブランチをチェックアウトするだけです。
-**ただし、** `ユーザのEメールの設定`_ **と** `ローカルリポジトリのセットアップ`_ **は見ておいてください。**
-
-現在開発中のツリーのコピーを取得するためには::
-
-  % cd ~/work/sigil
-  % bzr checkout sftp://centralhost/srv/bzr/sigil dev
-
-今、二人の人が ``sftp://centralhost/srv/bzr/sigil`` のチェックアウトを持っている状態なので、\
-片方のチェックアウトが最新のものより古くなってしまうタイミングが出てきます。
-コミットの時に、チェックアウトが古いければ Bazaar_ がエラーを返して、コミットは失敗します。
-チェックアウトを最新にするには、よそで変更されたツリーに対して ``bzr update`` を使用します。
-この時、もし同じファイルが変更されていれば、競合の解決が必要になるかもしれません。
-
-
-別ブランチでの開発
-====================
-
-ここまでは、全員が同じブランチで作業して、変更をコミットしていました。
-これはつまり、全員が適正かつ定期的にアップデートを行い、他の人の変更を取り扱う必要があると\
-いうことです。
-また、誰か一人が何かコードを壊すような変更をコミットして、それが同期されれば、\
-全員が問題を抱えることになります。
-
-たいていの場合、別のブランチ上で開発を行い、コードが安定してからメインのブランチに統合する\
-というやり方の方が優れています。これが、CVSやSVNとの一番大きな違いです。
-CVSやSVNも別ブランチでの開発はできますが、マージのアルゴリズムがかなり貧弱なので、\
-ブランチ間できちんと同期がとれた状態に保つのは難しいことです。
-Bazaar_ は、何がマージ済みかを記憶し、たとえファイルが変名されてる場合でもちゃんと変更を\
-適用することができます。
-
-
-新しいブランチを作成してそこで作業する
----------------------------------------
-まだ変更が完了していない場合でも、他の人がその変更内容にアクセスできるようにしておきたいですよね。
-そこで、 ``centralhost`` 上に新しい公開ブランチを作成して、それを手元に持ってくることにしましょう。
-
-::
-
-  % cd ~/work/sigil
-  % bzr branch sftp://centralhost/srv/bzr/sigil \
-               sftp://centralhost/srv/bzr/sigil/doodle-fixes
-  % bzr checkout sftp://centralhost/srv/bzr/sigil/doodle-fixes doodle-fixes
-  % cd doodle-fixes
-
-これで、 ``doodle`` に必要な修正を当てるための場所ができました。
-また、他の部分の修正をする人に邪魔されることもありません。
-チェックアウトを持っているため、 ``~/work/sigil/doodle-fixes/`` に対してコミットした内容は\
-``centralhost`` 上にも現れます。
-[#nestedbranches]_ ``dev`` ブランチと同じように、このようなブランチ上で二人の開発者が共同で\
-作業することもできます。
-[#cbranch]_
-
-.. [#nestedbranches] あるブランチのサブディレクトリに別のブランチがあるというのはおかしなこと\
-   に見えるかもしれませんが、これは何もおかしくありません。入れ子になったブランチは外側のブランチ\
-   から派生しているという点で、名前空間の階層と同じようなものだと考えることができます。
-
-.. [#cbranch] たくさんの独立したブランチを使っている場合、毎回フルURLを入力するのは大変です。
-   このURLの入力を簡単にするために、ブランチエイリアスなどたくさんの方法を検討しています。
-   今のところ、 bzrtools_ プラグインが ``bzr cbranch`` コマンドを提供しています。
-   これは、ベースとなるブランチを指定して、新しく公開ブランチを作成し、そのチェックアウトを作成する\
-   ことを少ない入力でできるように設計されています。
-   ``cbranch`` の使い方についてはこの文書の範囲外ですが、最後のコマンドについてはこんな感じです。:
-
-::
-
-   % bzr cbranch dev my-feature-branch
-
-.. _bzrtools: http://bazaar-vcs.org/BzrTools
-
-
-変更内容をマージする
-----------------------
-
-``doodle-fixes`` での変更内容をメインのブランチにマージすることになったら、単にこうするだけです。::
-
-  % cd ~/work/sigil/dev
-  % bzr merge ../doodle-fixes
-
-これで、変更内容は ``dev`` ブランチ上でもアクセスできるようになりますが、まだコミットはされていません。
-最終的な変更内容をレビューして、コードがちゃんとコンパイルでき、テストをパスすることを確認したいならこの時です。
-``bzr status`` コマンドと ``bzr diff`` コマンドがここで役立ちます。
-また、競合を解決するのもこの時です。Bazaar_ では、競合を解決するまではコミットできないようになっています。
-そのため、間違って競合マーカーをコミットしてしまうことはありません。
-``bzr status`` コマンドを使えば、他の変更内容と一緒に競合の情報も表示されます。
-``bzr conflicts`` なら、競合の情報だけが表示されます。
-競合を解決し終わったら、``bzr resolve file/name`` か ``bzr resolve --all`` を実行してください。
-[#resolve]_ もし、解決が特に難しい競合がある場合は、 ``bzr remerge`` コマンドを使いたいと思うかもしれません。
-このコマンドで、別のマージアルゴリズムを試してみることができ、さらに元のソース行を表示する\
-こともできます。(``--show-base``)
-
-.. [#resolve] マージ実行中に競合の解決をさせるシステムもあります。
-   私たちは、ファイルごとに競合を解決するよりも、ツリー全体を見ながら競合を解決する方がたいていは簡単である\
-   ことに気づきました。
-   そうすれば、よりたくさんの情報を得られますし、問題を解決したときにテストを実行することもできます。
-
-
-推奨のブランチ構成
----------------------
-
-とても一般的なブランチ構成として、開発者ごとにひとつずつ専用のブランチを割り当て、中央サーバにも開発者ごとの\
-作業領域を用意するという方法があります。これは以下のコマンドでできます。::
-
-  % bzr branch sftp://centralhost/srv/bzr/sigil \
-               sftp://centralhost/srv/bzr/sigil/user-a
-  % bzr branch sftp://centralhost/srv/bzr/sigil \
-               sftp://centralhost/srv/bzr/sigil/user-b
-
-これで、開発者ごとに専用の作業用ブランチを割り当てています。
-さらに、開発者自身で [#cbranch]_ を使用して新しく新機能開発用ブランチを作成することも簡単にできます。
-::
-
-  % bzr branch sftp://centralhost/srv/bzr/sigil/user-a \
-               sftp://centralhost/srv/bzr/sigil/user-a/feature 
-  % cd ~/work/sigil
-  % bzr checkout sftp://centralhost/srv/bzr/sigil/user-a/feature myfeature
-
-
-用語解説
-=========
-
-共用リポジトリ
---------------
-
-Bazaar_ には、”共用リポジトリ”という概念があります。これは、CVSやSubversionのようなの他のRCSが持つ\
-旧来の概念に似ています。
-たとえば、Subversionでは、サーバ上のリポジトリにすべての履歴情報が保存されていて、手元には履歴情報は\
-全くなく、作業ツリーのファイルのチェックアウトがあるだけです。
-ここで言う”共用”というのは、ブランチ同士の間で共用するという意味であることに注意してください。
-開発者間でも共用される *かも知れません* が、開発者間での共用はスタンドアロンブランチでも可能です。
-
-Bazaar_ の用語では、"共用リポジトリ"とは複数のブランチが履歴情報を **共有** できる場所のことです。
-分散型のワークフローに対応するためには、それぞれのブランチが履歴情報を持っている必要があります。
-しかし、しばしばこれは非効率です。関連するブランチ同士は履歴を共有しており、ディスクも共有した方が\
-いいためです。
-
-
-.. 
-   vim: tw=74 ft=rst
+========================================
+集中型ワークフローのチュートリアル
+========================================
+
+
+概要
+====
+
+この文書では、 Bazaar_ を使用することで実現可能なワークフローについて説明します。
+つまり、分散バージョンコントロールシステムである Bazaar_ を、集中型のやり方で使用するためのワークフローです。 
+Bazaar_ は、非常にフレキシブルに設計されており、完全な分散型からほとんど集中型のワークフローまで、\
+いくつかの異なるワークフローを受け入れることができます。
+このワークフローは、初めてのユーザでも簡単に Bazaar_ を使いこなすことができ、\
+分散型と集中型のオペレーションを組み合わせた業務にも対応できるようになっています。
+
+基本的に、この文書はCVSや Subversion のような集中型バージョンコントロールシステムの経験があるユーザ向けに書かれています。
+これらの環境では、一般的に、コードベースをホストする単一の中央サーバが存在し、複数の人々がこのコードベース上で作業して、\
+お互いの作業の同期をとることになります。 
+また、このワークフローは、一人の開発者が複数のマシーン上で作業をする場合にも適用できます。
+
+.. _Bazaar: http://bazaar.canonical.com
+
+
+初期セットアップ
+================
+
+Bazaar_ がうまく動くようにするための、まあまあ簡単なセットアップ手順があります。
+
+ユーザのEメールの設定
+---------------------
+
+ユーザIDは各コミットに保存されます。これは正確だったり一意だったりする必要はありませんが、\
+ログメッセージや注釈で使用されるため、何かしら実在する値を設定しておいた方がよいでしょう。
+
+::
+
+   % bzr whoami "John Doe <jdoe@organization.com>"
+
+
+ローカルリポジトリのセットアップ
+--------------------------------
+
+Bazaar_ のブランチは、通常は履歴のコピーを持っています。そのおかげで分散型での作業ができるわけです。
+最適化の方法の一つとして、関連するブランチ同士の情報を統合して、\
+新しいブランチを作るたびに履歴情報を丸ごとコピーしなくてもいいようにすることができます。
+
+そのための一番いい方法は、 `共用リポジトリ`_ を作成することです。
+一般に、 `共用リポジトリ`_ のサブディレクトリ内に複数のブランチがある場合、お互いの記憶領域を共有します。
+ですので、ホームディレクトリに `共用リポジトリ`_ を作成するようにしましょう。
+そうすれば、その下に作成したすべてのブランチは、履歴情報の領域を共有することになります。
+
+::
+
+  % bzr init-repo --trees ~
+
+
+リモートリポジトリのセットアップ
+---------------------------------
+
+作業用の領域とは別に、データを蓄積しておく領域が欲しいというのはよくあることです。
+集中型のシステム(CVS/SVN)では、このようなワークフローが必要になります。
+たいていは、これらは別々のマシーンに分けて配置されます。(そうでないこともあります。)
+実際のところ、これは、特に職場ではとてもよい設定です。
+蓄積されたデータはバックアップを確実にとることができ、開発者のマシーンに何かトラブルが起こっても\
+コミットされたデータはけっして無くなりません。
+
+それでは、プロジェクト内で共有する ``centralhost`` というマシーンを用意しましょう。
+先ほども言いましたが、ディスクの使用量を節約するために、 `共用リポジトリ`_ を使用します。
+
+::
+
+  % bzr init-repo --no-trees bzr+ssh://centralhost/srv/bzr/
+
+この手順は、新しくcvsrootやSubversionのリポジトリを作るのと同じようなものだと考えることができます。
+``--no-tree`` オプションの指定によって、ワーキングツリーを作らないようになります。
+中央リポジトリ内のブランチを直接変更することは無いので、このオプションを指定しておくとよいでしょう。
+
+ここで、 ``bzr+ssh`` というURLを使っていますが、これは、 SSH セキュアシェ
+ル上の
+Bazaar 独自プロトコルを意味しています。 bzr+ssh サーバーのセットアップに関
+する
+情報に着いては、管理者向けガイドを参照してください。
+
+
+
+既存のプロジェクトからの移行
+=============================
+
+リポジトリができましたので、プロジェクトの作成にかかりましょう。
+たいていの場合、 Bazaar_ でバージョン管理したい作業中のコードがすでにあるはずです。
+もし、そのコードがもともとソース管理されていたのであれば、履歴の情報を保ったまま Bazaar_ に\
+変換する方法がたくさんあります。
+しかしながら、それについてはここでは説明しません。詳しくは、 `Tracking Upstream`_ の\
+"Converting and keeping history"セクションを見てください。
+
+.. _Tracking Upstream: http://wiki.bazaar.canonical.com/TrackingUpstream
+
+..
+   XXX: プロジェクトの変換について話をするための別の文書が必要なのですが、\
+   今ある中ではTrackingUpstreamが一番よい文書です。
+
+
+Developer 1: 最初のリビジョンを作成する
+-----------------------------------------
+
+まず最初に、プロジェクトをホストするリモートリポジトリに、ブランチを作成したいと思います。
+仮に、"sigil"というプロジェクトをバージョン管理しようとしているとしましょう。
+
+::
+
+  % bzr init bzr+ssh://centralhost/srv/bzr/sigil
+
+これは、CVSで言うところの"HEAD"ブランチ、Subversionなら"trunk"にあたるものだと考えてかまいません。
+これを ``dev`` ブランチと呼ぶことにしましょう。
+
+他のファイルとの競合を避けるために、ホームディレクトリのサブディレクトリ内で作業するのが私の好みです。
+同じように、最終的にでき上がるすべてのブランチを格納できるプロジェクトディレクトリも欲しいですね。 
+::
+
+  % cd ~
+  % mkdir work
+  % cd work
+  % mkdir sigil
+  % cd sigil
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil dev
+  % cd dev
+  % cp -ar ~/sigil/* .
+  % bzr add
+  % bzr commit -m "Initial import of Sigil"
+
+前のセクションでは、空のブランチ(``sigil`` ブランチ)を ``centralhost`` に作成して、\
+それをクライアント端末にチェックアウトしたあと、元々あったプロジェクトのファイルを追加しています。
+作業ディレクトリをセットアップするための方法はたくさんありますが、この方法を使うと機能追加用/バグフィックス用\
+のブランチを使った作業が簡単にできます。
+そして、複数のブランチをとてもうまく扱えるというのが、 Bazaar_ の長所の一つなのです。
+
+この場合、リモートブランチのチェックアウトを持っているので、 ``~/work/sigil/dev/`` に対してコミットした内容が、\
+ローカルと ``centralhost`` の両方に自動的に保存される訳です。
+
+
+Developer N: プロジェクトの作業コピーを取得する
+--------------------------------------------------
+
+プロジェクトの作成に関するすべての作業は1人目の開発者がしてしまっているので、\
+他のみんなは単にそのブランチをチェックアウトするだけです。
+**ただし、** `ユーザのEメールの設定`_ **と** `ローカルリポジトリのセットアップ`_ **は見ておいてください。**
+
+現在開発中のツリーのコピーを取得するためには::
+
+  % cd ~/work/sigil
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil dev
+
+今、二人の人が ``bzr+ssh://centralhost/srv/bzr/sigil`` のチェックアウトを持っている状態なので、\
+片方のチェックアウトが最新のものより古くなってしまうタイミングが出てきます。
+コミットの時に、チェックアウトが古いければ Bazaar_ がエラーを返して、コミットは失敗します。
+チェックアウトを最新にするには、よそで変更されたツリーに対して ``bzr update`` を使用します。
+この時、もし同じファイルが変更されていれば、競合の解決が必要になるかもしれません。
+
+
+別ブランチでの開発
+====================
+
+ここまでは、全員が同じブランチで作業して、変更をコミットしていました。
+これはつまり、全員が適正かつ定期的にアップデートを行い、他の人の変更を取り扱う必要があると\
+いうことです。
+また、誰か一人が何かコードを壊すような変更をコミットして、それが同期されれば、\
+全員が問題を抱えることになります。
+
+たいていの場合、別のブランチ上で開発を行い、コードが安定してからメインのブランチに統合する\
+というやり方の方が優れています。これが、CVSやSVNとの一番大きな違いです。
+CVSやSVNも別ブランチでの開発はできますが、マージのアルゴリズムがかなり貧弱なので、\
+ブランチ間できちんと同期がとれた状態に保つのは難しいことです。
+Bazaar_ は、何がマージ済みかを記憶し、たとえファイルが変名されてる場合でもちゃんと変更を\
+適用することができます。
+
+
+新しいブランチを作成してそこで作業する
+---------------------------------------
+まだ変更が完了していない場合でも、他の人がその変更内容にアクセスできるようにしておきたいですよね。
+そこで、 ``centralhost`` 上に新しい公開ブランチを作成して、それを手元に持ってくることにしましょう。
+
+::
+
+  % cd ~/work/sigil
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil \
+               bzr+ssh://centralhost/srv/bzr/sigil/doodle-fixes
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil/doodle-fixes doodle-fixes
+  % cd doodle-fixes
+
+これで、 ``doodle`` に必要な修正を当てるための場所ができました。
+また、他の部分の修正をする人に邪魔されることもありません。
+チェックアウトを持っているため、 ``~/work/sigil/doodle-fixes/`` に対してコミットした内容は\
+``centralhost`` 上にも現れます。
+[#nestedbranches]_ ``dev`` ブランチと同じように、このようなブランチ上で二人の開発者が共同で\
+作業することもできます。
+[#cbranch]_
+
+.. [#nestedbranches] あるブランチのサブディレクトリに別のブランチがあるというのはおかしなこと\
+   に見えるかもしれませんが、これは何もおかしくありません。入れ子になったブランチは外側のブランチ\
+   から派生しているという点で、名前空間の階層と同じようなものだと考えることができます。
+
+.. [#cbranch] たくさんの独立したブランチを使っている場合、毎回フルURLを入力するのは大変です。
+   このURLの入力を簡単にするために、ブランチエイリアスなどたくさんの方法を検討しています。
+   今のところ、 bzrtools_ プラグインが ``bzr cbranch`` コマンドを提供しています。
+   これは、ベースとなるブランチを指定して、新しく公開ブランチを作成し、そのチェックアウトを作成する\
+   ことを少ない入力でできるように設計されています。
+   ``cbranch`` の使い方についてはこの文書の範囲外ですが、最後のコマンドについてはこんな感じです。:
+
+::
+
+   % bzr cbranch dev my-feature-branch
+
+.. _bzrtools: http://wiki.bazaar.canonical.com/BzrTools
+
+
+変更内容をマージする
+----------------------
+
+``doodle-fixes`` での変更内容をメインのブランチにマージすることになったら、単にこうするだけです。::
+
+  % cd ~/work/sigil/dev
+  % bzr merge ../doodle-fixes
+
+これで、変更内容は ``dev`` ブランチ上でもアクセスできるようになりますが、まだコミットはされていません。
+最終的な変更内容をレビューして、コードがちゃんとコンパイルでき、テストをパスすることを確認したいならこの時です。
+``bzr status`` コマンドと ``bzr diff`` コマンドがここで役立ちます。
+また、競合を解決するのもこの時です。Bazaar_ では、競合を解決するまではコミットできないようになっています。
+そのため、間違って競合マーカーをコミットしてしまうことはありません。
+``bzr status`` コマンドを使えば、他の変更内容と一緒に競合の情報も表示されます。
+``bzr conflicts`` なら、競合の情報だけが表示されます。
+競合を解決し終わったら、``bzr resolve file/name`` か ``bzr resolve --all`` を実行してください。
+[#resolve]_ もし、解決が特に難しい競合がある場合は、 ``bzr remerge`` コマンドを使いたいと思うかもしれません。
+このコマンドで、別のマージアルゴリズムを試してみることができ、さらに元のソース行を表示する\
+こともできます。(``--show-base``)
+
+.. [#resolve] マージ実行中に競合の解決をさせるシステムもあります。
+   私たちは、ファイルごとに競合を解決するよりも、ツリー全体を見ながら競合を解決する方がたいていは簡単である\
+   ことに気づきました。
+   そうすれば、よりたくさんの情報を得られますし、問題を解決したときにテストを実行することもできます。
+
+
+推奨のブランチ構成
+---------------------
+
+とても一般的なブランチ構成として、開発者ごとにひとつずつ専用のブランチを割り当て、中央サーバにも開発者ごとの\
+作業領域を用意するという方法があります。これは以下のコマンドでできます。::
+
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil \
+               bzr+ssh://centralhost/srv/bzr/sigil/user-a
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil \
+               bzr+ssh://centralhost/srv/bzr/sigil/user-b
+
+これで、開発者ごとに専用の作業用ブランチを割り当てています。
+さらに、開発者自身で [#cbranch]_ を使用して新しく新機能開発用ブランチを作成することも簡単にできます。
+::
+
+  % bzr branch bzr+ssh://centralhost/srv/bzr/sigil/user-a \
+               bzr+ssh://centralhost/srv/bzr/sigil/user-a/feature
+  % cd ~/work/sigil
+  % bzr checkout bzr+ssh://centralhost/srv/bzr/sigil/user-a/feature myfeature
+
+
+用語解説
+=========
+
+共用リポジトリ
+--------------
+
+Bazaar_ には、”共用リポジトリ”という概念があります。これは、CVSやSubversionのようなの他のRCSが持つ\
+旧来の概念に似ています。
+たとえば、Subversionでは、サーバ上のリポジトリにすべての履歴情報が保存されていて、手元には履歴情報は\
+全くなく、作業ツリーのファイルのチェックアウトがあるだけです。
+ここで言う”共用”というのは、ブランチ同士の間で共用するという意味であることに注意してください。
+開発者間でも共用される *かも知れません* が、開発者間での共用はスタンドアロンブランチでも可能です。
+
+Bazaar_ の用語では、"共用リポジトリ"とは複数のブランチが履歴情報を **共有** できる場所のことです。
+分散型のワークフローに対応するためには、それぞれのブランチが履歴情報を持っている必要があります。
+しかし、しばしばこれは非効率です。関連するブランチ同士は履歴を共有しており、ディスクも共有した方が\
+いいためです。
+
+
+..
+   vim: tw=74 ft=rst

=== modified file 'doc/ja/tutorials/index.txt'
--- doc/ja/tutorials/index.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/tutorials/index.txt	2011-05-18 06:01:08 +0000
@@ -8,4 +8,4 @@
    tutorial
    using_bazaar_with_launchpad
    centralized_workflow
-
+   licence

=== added file 'doc/ja/tutorials/licence.txt'
--- doc/ja/tutorials/licence.txt	1970-01-01 00:00:00 +0000
+++ doc/ja/tutorials/licence.txt	2011-05-18 06:01:08 +0000
@@ -0,0 +1,14 @@
+Licence
+===============
+
+Copyright 2005-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later.  See
+<http://www.gnu.org/licenses/>.
+
+日本語訳
+==========
+日本語への翻訳は、
+Bazaar-ja <https://groups.google.com/group/bazaar-ja>
+が行っています。質問や間違いの指摘等はこちらへお願いします。
+

=== modified file 'doc/ja/tutorials/tutorial.txt'
--- doc/ja/tutorials/tutorial.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/tutorials/tutorial.txt	2011-05-18 06:01:08 +0000
@@ -25,6 +25,8 @@
 それ以外の場合、コーヒーか紅茶(訳注:日本茶でもいいですよ)を用意して、\
 リラックスして読み始めてください。
 
+.. purpose-of-version-control
+
 バージョン管理の目的
 ====================
 
@@ -38,8 +40,8 @@
 
 Bazaarをはじめとするバージョン管理システムは、ディレクトリに起こった変更を\
 **ブランチ (branch)** というディレクトリよりも複雑なものの中に入れて\
-追跡します。
-ブランチはディレクトリの中に何が入っているかだけでなく、過去のいろいろな\
+管理します。
+ブランチは今ディレクトリの中に何が入っているかだけでなく、過去のいろいろな\
 時点でのディレクトリの中身を格納しています。
 なので、望まぬ変更をしてしまったときには過去の時点の状態に戻すことができます。
 
@@ -53,9 +55,9 @@
 たとえば、オプションのログメッセージをつけることで、変更が何を意味して\
 いるのかというコメントを残すことができます。
 実際に利用されるログメッセージは、 "Webテンプレートをテーブルを閉じるように\
-修正" とか "sftpに対応した。 #595 を修正。" といったものです。
+修正" とか "SFTPに対応した。 #595 を修正。" といったものです。
 
-こういったログが保存されているおかげで、たとえば後になって sftp に問題が発生\
+こういったログが保存されているおかげで、たとえば後になって SFTP に問題が発生\
 したときに、問題が発生するようになったのがどの時点なのか目星をつけることが\
 できます。
 
@@ -236,11 +238,12 @@
 分離するといいでしょう。
 
 自分のコンピュータのファイルシステム上にあるブランチを参照するときは\
-ブランチを格納しているディレクトリ名で指定できます。 bzr は http や ftp\
-経由でブランチにアクセスすることもできます。例::
+ブランチを格納しているディレクトリ名で指定できます。 bzr は SSH, HTTP
+SFTP などを経由してブランチにアクセスすることもできます。例::
 
-    % bzr log http://bazaar-vcs.org/bzr/bzr.dev/
-    % bzr log sftp://bazaar-vcs.org/bzr/bzr.dev/
+    % bzr log bzr+ssh://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log sftp://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
 
 プラグインをインストールすれば、 rsync プロトコルを使ってブランチにアクセス\
 することもできます。
@@ -395,6 +398,39 @@
 コミットメッセージに diff を含めたい場合は、水平線より上にコピーペースト\
 してください。
 
+
+.. Marking bugs as fixed
+
+解決したバグの記録をつける
+----------------------------
+
+プロジェクトにおいて多くの変更はバグの修正のために行われます。
+Bazaar は、コミットするときに解決したバグについてメタデータに記録することが
+できます。
+これを行うには、 ``--fixes`` オプションを使います。
+このオプションは次のような形の引数を取ります。
+
+    % bzr commit --fixes <tracker>:<id>
+
+``<tracker>`` の部分にはバグ管理システムを指定するIDを書き、
+``<id>`` の部分にはそのバグ管理システム上で管理されているバグの
+IDを書きます。 ``<id>`` はたいてい数値になるでしょう。
+Bazaar は最初からいくつかの有名なバグ管理システムを知っています。
+bugs.launchpad.net, bugs.debian.org bugzilla.gnome.org です。
+これらは、それぞれ独自のIDとして lp, deb, gnome を持っています。
+例えば、 bugs.launchpad.net 上のバグ #1234 を解決する場合は、
+その解決をコミットするときに次のようなコマンドを利用できます。 ::
+
+    % bzr commit -m "fixed my first bug" --fixes lp:1234
+
+For more information on this topic or for information on how to configure
+other bug trackers please read `Bug Tracker Settings`_.
+
+このトピックに着いてのさらなる情報や、他のバグ管理システムを設定する方法
+については、 `Bug Tracker Settings`_ を参照してください。
+
+.. _Bug Tracker Settings: ../user-reference/index.html#bug-tracker-settings
+
 .. Selective commit
 
 選択コミット
@@ -519,6 +555,27 @@
     % bzr commit -m "Add ignore patterns"
 
 
+bzr ignore
+----------
+
+``.bzrignore`` ファイルを直接編集する代わりに、 ``bzr ignore`` コマンドを
+利用することができます。 ``bzr ignore`` コマンドはファイル名かパターンを
+引数に受け取って、それを ``.bzrignore`` ファイルに追加します。
+``.bzrignore`` ファイルが存在しない場合、 ``bzr ignore`` コマンドは
+自動的にそのファイルを作成してバージョン管理に追加します。 ::
+
+    % bzr ignore tags
+    % bzr status
+    added:
+      .bzrignore
+
+
+``.bzrignore`` ファイルを自分で修正したときと同じく、コマンドを実行したあとに
+``.bzrignore`` ファイルをコミットしなければなりません。 ::
+
+    % bzr commit -m "Added tags to ignore file"
+
+
 .. Global ignores
 
 グローバルの無視設定
@@ -655,7 +712,7 @@
 この場合、既存のブランチのコピーを取得する必要があります。
 このコピーは新しいブランチになるので、このコマンドは **branch** という名前になっています::
 
-    % bzr branch http://bazaar-vcs.org/bzr/bzr.dev
+    % bzr branch lp:bzr bzr.dev
     % cd bzr.dev
 
 .. This copies down the complete history of this branch, so we can do all
@@ -780,7 +837,7 @@
 
   ::
 
-    % bzr push sftp://servername.com/path/to/directory
+    % bzr push bzr+ssh://servername.com/path/to/directory
 
 ..   (The destination directory must already exist unless the
 ..   ``--create-prefix`` option is used.)
@@ -799,8 +856,10 @@
 ..   rsync, or other related file transfer methods.  This is usually less safe
 ..   than using ``push``, but may be faster or easier in some situations.
 
-* 手動で、 tarball を送ったり rsync を使ったりほかの転送方法を利用してファイルを\
-  送ることで
+* tarball を送ったり rsync を使ったりほかの転送方法を利用して、手動でファイルを\
+  コピーすることもできます。
+  これはたいてい ``push`` ほど安全ではありませんが、場合によって高速だったり、\
+  簡単だったりするかもしれません。
 
 
 .. Moving changes between trees
@@ -814,7 +873,7 @@
 .. expected, so you start a new branch for it.
 
 どんな人にでもありえることですが、適切ではないツリー上で変更してしまうことがあります。
-単純に作業するディレクトリを間違えたり、変更がよそうよりも大きくなってしまったりして、
+単純に作業するディレクトリを間違えたり、変更が予想よりも大きくなってしまったりして、
 その変更のために新しいブランチを作りなおすことがあります。
 
 .. To move your changes from one tree to another, use

=== modified file 'doc/ja/tutorials/using_bazaar_with_launchpad.txt'
--- doc/ja/tutorials/using_bazaar_with_launchpad.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/tutorials/using_bazaar_with_launchpad.txt	2011-05-18 06:01:08 +0000
@@ -17,50 +17,25 @@
 それに見合ったワークフローをサポートするBazaarのようなツールを使うことは、\
 大きな手助けになるでしょう。
 
-.. The team of people required to create the initial release
-   of a piece of software may vary in size from one person
-   to several thousand people. Depending on the requirements,
-   the challenges involved, both technical and managerial,
-   can be immense. As explained in the Bazaar User Guide, selecting
-   "just right" processes and using tools like Bazaar to support
-   matching workflows can greatly help.
-
 しかし、ソフトウェアによる成功のためには、すばらしいチーム以上のものが\
 必要です。 - それは、健全で活発な *コミュニティ* です。
 このグループは、通常はチームよりもはるかに大きく、なぜならそのソフトウェアに\
 関心のあるすべての人 - 開発チーム、ユーザ、トレーニングパートナー、\
 サポートパートナー、サードパーティの開発者など - を含むからです。
 
-.. Success with software though requires more than a great team - it
-   requires a healthy, active *community*. This group is typically
-   far larger than the team as it includes everyone interested in
-   the software: the team, users, training partners, support partners,
-   third-party developers and so on.
-
-すばらしいコミュニティというものはオープンソースの世界では良く理解されています。
-しかし、その適用はオープンソースの世界を越えて広がっています。: もっとも成功している\
-商業ソフトウェアのベンダーは、そのフラッグシッププロダクトと共に成長するコミュニティ\
+すばらしいコミュニティというものはフリー(自由)ソフトウェア
+コミュニティでは良く理解されています。
+しかし、その適用はオープンソースの世界を越えて広がっています。
+もっとも成功している商業ソフトウェアのベンダーは、
+そのフラッグシッププロダクトと共に成長するコミュニティ\
 を作り上げ、運営することがたくみなのです。
 
-.. Great communities are well understood in the open source world.
-   Their applicability extends well beyond that though: most
-   successful commercial software vendors are well skilled at
-   building and managing the communities that grow up around
-   their flagship products.
-
 すばらしいチームと同じように、すばらしいコミュニティも偶然できるものではありません。
 良いポリシーとガイドラインが、参加者同士の健全なコミュニケーションと正しい振る舞い\
 を育てるために不可欠です。
 この話題についてもっと深く知りたければ、Karl Fogelのすばらしい著書 - \
 `Producing Open Source Software <http://www.producingoss.com/>`_ - を見てください。
 
-.. Like great teams, great communities don't just happen.
-   Good policies and guidelines are essential for
-   fostering the right sort of behaviour and healthy
-   relationships between participants. For a deeper look at
-   this topic, see Karl Fogel's landmark book:
-   `Producing Open Source Software <http://www.producingoss.com/>`_.
-
 
 協調開発に必要なもの
 ---------------------------------------------------
@@ -73,25 +48,13 @@
 `java.net <http://java.net>`_ 、
 `SAP Community Network <https://www.sdn.sap.com/irj/sdn>`_ などに、CDEsの例があります。
 
-.. An intelligent toolset is also important for tracking and managing
-   community information and workflows. These tools are called
-   Collaborative Development Environments (CDEs). These toolsets are
-   typically web-based and manage things such as announcements,
-   issues/bugs, questions and answers, downloads, documents and
-   source code. Some examples of CDEs include
-   `Launchpad <https://launchpad.net>`_,
-   `SourceForge <http://sourceforge.net>`_,
-   `java.net <http://java.net>`_ and
-   `SAP Community Network <https://www.sdn.sap.com/irj/sdn>`_.
-
-
 関係するコミュニティとの協調を助ける
 -------------------------------------------------
 
 多くの成功しているプロダクトは、その下流にそれを使うたくさんのプロダクトがあります。
 言いかえると、他のコミュニティとやりとりをして、自分の変更が彼らにどんな影響を与えるかを\
 理解することで、新しい挑戦が成功するのです。
-これは、以下のようなプロダクトで特に明白です。:
+これは、以下のようなプロダクトでは特に明白です。:
 
 * プログラム言語、たとえばPyhon、PHP、Ruby、Java、Perlなど
 * コンパイラ、たとえばgcc、JDKなど
@@ -106,18 +69,9 @@
    communities and understanding how your changes will impact them. This is
    most obvious for projects like:
 
-.. * software languages, e.g. Python, PHP, Ruby, Java, Perl, etc.
-   * compilers, e.g. gcc, JDK, etc.
-   * libraries, e.g. zlib, openssl, etc.
-   * frameworks, e.g. Zope, Ruby on Rails, Spring, etc.
-
 しかし、アドオン機能を持つメジャーなアプリケーション、たとえばFirefox、Thundervird、\
 OpenOffice.org、Drupal、Wordpress、Joomlaなどにも、このことはあてはまります。
 
-.. However it applies equally for popular applications on which add-ons are
-   built, e.g. Firefox, Thunderbird, OpenOffice.org, Drupal, Wordpress, Joomla,
-   etc.
-
 コミュニティの境界をこえて案件や障害修正の追跡と管理をするための作業をサポートして\
 くれるツールが必要です。
 そのようなツールは、両極端にいるどちらのユーザも助けてくれます。:
@@ -130,38 +84,21 @@
   たとえば、「グラフィックライブラリのバグを修正することにより、これらの10個のOS上の\
   5つのアプリケーションに恩恵がある」など
 
-.. Tools that assist communities work together to track and manage
-   issues and fixes across community boundaries are required. These
-   tools help people at both ends of the spectrum:
-
-.. * users can report problems in their terms, e.g. rendering of image
-     type X is broken in application Y on operating system Z
-
-.. * developers can better appreciate the downstream impact of making a
-     change or fix, e.g. fixing this bug in a graphics library will
-     make the users of these 5 applications on these 10 operating
-     systems happy.
-
 その間にいる人々は、 *点線をつなぎ* 、上流と下流との間のコミュニケーションを担うという\
 重要な役割を果たします。
 多くの場合、彼らはエンドユーザのためにバグを修正したり、パッチをリリースしたり、上流の\
 開発チームに修正内容を提示したりします。
 それらすべてを持続可能な方法で常に追跡しつづけることは、簡単なことではありません。
 
-.. People in the middle play the essential role of *joining the dots* and
-   communicating up and down the line. In many cases, they may also fix the
-   problem for end users, releasing a patch and pushing a suggested fix
-   to the upstream development team. Keeping track of all that over time
-   in a sustainable way is no easy task.
-
-
 Launchpad: 開発をもっと効果的に、摩擦は少なく
 -------------------------------------------------
 
-`Ubuntu <http://www.ubuntu.com>`_ の開発に出資しているのと同じように、Canonicalは\
-オープンソースコミュニティ向けの無料のサービスとしてLaunchpad(https:launchpad.net)も\
-提供しています。
-Launchpadは、以下の注目すべき理由から、もっともエキサイティングなCDEsのひとつです。:
+Canonical は、 `Ubuntu <http://www.ubuntu.com>`_ や `Bazaar <http://bazaar.canonical.com>`_
+の開発に出資しているのと同じように、
+オープンソースコミュニティ向けの無料のサービスとして
+Launchpad <https:launchpad.net> も提供しています。
+Launchpadは、以下の注目すべき理由から、もっともエキサイティングな
+CDEsのひとつです。
 
 * トラッキング対象のたくさんのもの同士の関係を具体化しています。
   たとえば、ソースコードのブランチをバグ修正に関連づけることができます。
@@ -175,38 +112,11 @@
 * 違うコミュニティ同士が、関連する案件やロードマップに対してともに作業するための結びつきを\
   提供します。
 
-.. As well as sponsoring `Ubuntu <http://www.ubuntu.com>`_ and
-   `Bazaar <http://bazaar-vcs.org>`_ development, Canonical
-   provides Launchpad, https://launchpad.net, as a free service
-   for the open source community. Launchpad is one of the most
-   exciting CDEs around for several notable reasons:
-
-.. * it models relationships between many of things tracked, e.g.
-     source code branches can be associated with bug fixes
-
-.. * as well are managing historical knowledge,
-     it supports future development planning and tracking by providing
-     features such as roadmaps, milestones and blueprints
-
-.. * it provides translation tools and packaging services so that
-     barriers are reduced for translators and testers wishing to
-     join your community and help out
-
-.. * it provides a nexus for different communities to work
-     together on related issues and roadmaps.
-
 言いかえると、Launchpadは、あなたのコミュニティの成長を助け、 *コミュニティ内* と\
 *コミュニティ間* との両方でワークフローの摩擦を減らすようにデザインされています。
 究極的には、機械的なタスクにつかう時間をなくし、興味ぶかい開発により多くの時間を\
 さけるようにすることを意味しています。
 
-.. In other words, Launchpad has been designed to help your
-   community grow and to reduce the workflow friction both
-   *within* your community and *between* communities. Ultimately,
-   that means less time on mechanical tasks and more time for
-   interesting development.
-
-
 Bazaar: Launchpadのバージョン管理クライアント
 ---------------------------------------------
 
@@ -217,76 +127,62 @@
 1. BazaarはLaunchpadなしで使うこともできます。
 2. LaunchpadはBazaarなしで使うこともできます。
 
-それでも、別々に使うよりも一緒に使った方がよりパワフルになるよう設計されています。
-
-.. This tutorial looks at how Bazaar and Launchpad can be used together
-   and how they complement each other. It is important to remember that:
-
-.. 1. Bazaar can be used without Launchpad
-.. 2. Launchpad can be used without Bazaar.
-
-.. By design though, their sum is greater than the individual
-   parts.
-
-
-Finding and browsing branches using Launchpad
-=============================================
-
-Finding available branches
+それでも、別々に使うよりも一緒に使った方がより大きな力を発揮するように設計されています。
+
+
+Launchpadでのブランチの検索、閲覧
+===================================
+
+利用できるブランチの検索
 --------------------------
 
-While there are many advantages in adopting distributed version
-control, one of the things that disappears is the all-knowing
-central server with knowledge about all available branches. Indeed
-in a distributed environment, interesting branches can literally
-exist in 100s of locations across the Internet (or within an
-Intranet for that matter).
-
-Launchpad fills this gap by providing a registry of branches.
-
-
-Registering branches
+分散型バージョン管理を導入することには多くの利点がありますが、できなくなってしまうこと\
+のひとつとして、利用できるすべてのブランチについて知っている中央サーバがないという点が\
+あります。
+実際、分散環境では、関連するブランチは、インターネット中の文字どおり100もの場所に存在する\
+可能性があります。(もしくは、イントラネットの中でも同様です。)
+
+Launchpadは、ブランチのデータベースを提供することによって、このギャップをなくします。
+
+ブランチの登録
 --------------------
 
-Branches can be uploaded to Launchpad or simply registered
-as being available in an external location. Branches can also
-be given a Status such as *New*, *Development*, *Mature* or
-*Abandoned*.
-
-Note: External branches can even be hosted in legacy version control
-tools, i.e. CVS and Subversion. Code in these systems will be
-scanned and converted to Bazaar branches on a periodic basis.
-For maximum fidelity of course, it is preferable for external
-branches to be hosted in Bazaar.
-
-
-Browsing branches
+ブランチはLaunchPadにアップロードすることもできますし、別の場所にあるブランチを単に登録する\
+こともできます。
+また、ブランチには *New(新規)* 、 *Development(開発バージョン)* 、 *Mature(安定)* 、 *Abandoned(破棄)*
+というステータスを付加することができます。
+
+Note: 外部のブランチは、CVSやSubversionのような従来のバージョン管理ツールでホストされて\
+いても構いません。
+これらのシステム上のコードは定期的にスキャンされ、Bazaarのブランチに変換されます。
+もちろん、最大限の正確さを求めるのであれば、外部のブランチをBazaarでホストすることが望ましいです。
+
+ブランチの閲覧
 -----------------
 
-Branches can be listed, filtered and sorted by numerous
-attributes including Name, Registrant, Author, Status, Age and
-time of last commit. Browsing of branches is also provided making
-it easy to see things such as:
-
-* where the branch can be downloaded from
-* how to upload changes
-* recent commits and the changes made by each
-* the source code of individual files for a given version.
-
-
-Accessing code in Launchpad using Bazaar
-========================================
-
-Getting the code for an open source project
--------------------------------------------
-
-As Launchpad keeps track of thousands of open source projects
-and their latest code whether it be managed by Bazaar, CVS or Subversion,
-Bazaar users can grab that code as easily as this::
+ブランチは、名前、登録者、作者、ステータス、世代、最終コミット時刻などの多くの属性によって\
+リストアップやフィルタリング、並べ替えができます。
+また、ブランチを閲覧することによって、以下のようなことが簡単に分かります。:
+
+* ブランチがどこからダウンロードできるか
+* 変更をアップロードする方法
+* それぞれで行った最近のコミットと変更内容
+* 指定したバージョンの個々のファイルのソースコード
+
+
+Launchpad上のコードへのBazaarでのアクセス
+=========================================
+
+プロジェクトのコードの取得
+----------------------------
+
+Launchpad 上には多数のプロジェクトがあり、その最新のコードがBazaarで
+管理されていても、CVSやSubversionで管理されていても、Bazaarを使って
+以下のように簡単にコードを取得することができます。 ::
 
   bzr branch lp:project-name
 
-where `project-name` is the Launchpad project ID. Here are some examples::
+`project-name` は、Launchpad上のプロジェクトIDです。以下にいくつか例を挙げます。::
 
   bzr branch lp:inkscape
   bzr branch lp:amarok
@@ -294,259 +190,270 @@
   bzr branch lp:rails
   bzr branch lp:java-gnome
 
-You can then browse the code locally using your favorite editor or IDE and
-change the code if you wish.
-
-If a project has multiple series registered (e.g. a development series and a
-maintenance series), the latest code for a given series can be fetched using::
-
-  bzr branch lp:project-name/series
-
-Publishing your changes
+そのあと、好きなエディタやIDEを使って、コードを手元で参照したり、必要があれば編集したり\
+することができます。
+
+もし、プロジェクトに複数のシリーズ(例えば、開発用とメンテナンス用)が登録されている場合は、\
+以下のコマンドで指定したシリーズの最新のコードを取得することができます。::
+
+ bzr branch lp:project-name/series
+
+変更内容の公開
 -----------------------
 
-Having fixed that annoying bug or added that cool feature you've always
-wanted, it's time to impress your friends and make the world a better
-place by making your code available to others. As explained earlier,
-Launchpad is a free Bazaar code hosting service so you can push your
-branch to it and others can access your code from there. For example,
-assuming you are a member of the relevant team, login to launchpad like this::
+イライラするバグを修正したり、ずっと欲しかったクールな機能を追加したりしたら、それを友達に\
+アピールしたり、そのコードを他の人に公開して世の中をもっと良くするときです。
+これまでに説明したとおり、LaunchpadはBazaarによる無料のコードホスティングサービスなので、自分の\
+ブランチをプッシュして他の人がそこからコードにアクセスできるようにすることができます。
+例えば、あなたが関連するチームのメンバーだとすると、以下のようにLaunchpadにログインします。::
 
   bzr launchpad-login userid
 
-where `userid` is your Launchpad user ID.
-You can then push your changes to a team branch like this::
+`userid` は、LaunchpadのユーザIDです。
+そのあと、以下のように変更をチームのブランチにプッシュすることができます。::
 
   bzr push lp:~team-name/project-name/branch-name
 
-Others can then download your code like this::
+他の人は、以下のようにそのコードをダウンロードすることができます。::
 
   bzr branch lp:~team-name/project-name/branch-name
 
 
-Personal branches
+自分用のブランチ
 -----------------
 
-Even if you are not a member of a team, Launchpad can be used to publish
-your changes. In this case, simply create a personal branch like this::
+あなたがチームのメンバーではなかったとしても、Launcpadであなたの変更内容を公開することができます。
+この場合、以下のようにして単純に自分用のブランチを作成します。::
 
   bzr push lp:~userid/project-name/branch-name
 
-Others can then download your code like this::
+他の人は、以下のようにそのコードをダウンロードすることができます。::
 
   bzr branch lp:~userid/project-name/branch-name
 
-Note: Even when publishing to a personal branch, it is polite to notify the
-upstream developers about your branch so they can pull your changes from
-it if they are generally applicable to all users and meet the project's
-quality standards.
-
-
-Linking branches using Launchpad
-================================
-
-Associating a branch with a bug
+Note: 自分用のブランチに公開したときも、ちゃんと上流の開発者にあなたのブランチについて通知されるので、\
+全てのユーザに一般的に適用できる内容で、かつプロジェクトの品質基準を満たしていれば、彼らはその変更を\
+プルすることができます。
+
+.. Package source branches
+
+パッケージのソースブランチ
+-----------------------------
+
+`maintaining packages for Ubuntu using Bazaar` に書かれているとおり、
+Launchpad から簡単にパッケージのソースブランチにアクセスできます。
+現在の(デフォルトの)系列のパッケージのソースブランチは、次のようなコマンドで
+ダウンロードできます。 ::
+
+  bzr branch ubuntu:package
+
+ここで、 *package* はアクセスしたい Ubuntu のパッケージ名です。
+Ubuntu の特定の系列 (例: Maverick, Lucid) のパッケージブランチを
+ダウンロードするには、次のコマンドを使います。 ::
+
+  bzr branch ubuntu:maverick/package
+
+Ubuntu の系列は単に最初の文字を使う形に省略することができます。
+たとえば、上の例は次のようにも書けます。
+
+  bzr branch ubuntu:m/package
+
+いくつかの Debian の系列でも、 Launchpad からソースブランチをダウンロードする
+ことができます。デフォルトの系列であれば次のようにダウンロードできます。 ::
+
+  bzr branch debianlp:package
+
+そして、特定の系列の場合は次のようになります。 ::
+
+  bzr branch debianlp:lenny/package
+
+``debianlp:`` スキーマは　Launchpad にある Debian のソースブランチにしか
+アクセスできないので注意してください。
+
+.. _`maintaining packages for Ubuntu using Bazaar`: https://wiki.ubuntu.com/DistributedDevelopment
+
+
+Lanchpadでのブランチの関連づけ
+===============================
+
+ブランチをバグと関連付ける
 -------------------------------
 
-After registering a branch, you can associate it to a bug so that
-people interested in that bug can track and download the fix as
-it becomes available.
-
-To do this, the steps are:
-
-1. Navigate to the bug in question.
-
-2. Select `Add branch` under `Actions`.
-
-3. Select the branch.
-
-4. Optionally set the State of the relationship. This is
-   *Fix In Progress* by default but you may wish to set it
-   to another state such as *Fix Available* if the branch already
-   addresses the issue.
-
-If you wish, you can also provide some arbitrary comments about
-the relationship between the bug and the branch.
-
-
-Changing the state in Launchpad while committing in Bazaar
+ブランチを登録したあと、それにバグを関連づけることができます。
+そうすることで、そのバグに関心をもつ人々がそのブランチを追いかけ、修正が公開されたらダウンロード\
+することができます。
+
+そのための手順は以下のとおりです。
+
+1. Questionから、そのバグに移動します。
+
+2. `Actions` から `Add branch` を選択します。
+
+3. ブランチを選択します。
+
+4. 必要に応じて、関連づけのステータスを設定します。
+   *Fix In Progress(修正中)* がデフォルトですが、すでにブランチ内にで問題に対処しているのなら、\
+   *Fix Available(修正済)* に設定することもできます。
+
+もし望むのなら、バグとブランチとの関連づけに好きなコメントをつけることもできます。
+
+BazaarでのコミットによるLaunchpadのステータスの変化
 ----------------------------------------------------------
 
-Bazaar and Launchpad can work together to reduce some of
-the status housekeeping for you. When you commit using Bazaar,
-use the --fixes option like this::
+BazzarとLaunchpadを一緒につかうことで、ステータスのメンテナンス作業をへらすことができます。
+Bazaarでコミットしたときに、以下のように--fixesオプションを指定します。::
 
   bzr commit --fixes lp:1234 -m "..."
 
-where 1234 is the bug ID. This will changes the State of the
-bug-branch relationship to *Fix Available*. If the one commit
-fixes multiple issues, the --fixes option can be specified multiple
-times.
-
-One of the cool things about this feature is that Launchpad does
-not need to be accessible when making the commit. The ``--fixes``
-option works by storing metadata which Launchpad will detect next
-time the branch is pushed to it or scanned once online again.
-
-Note: Launchpad will not implicitly close a bug just because a
-branch is available that fixes it. There are several reasons for this.
-Firstly, the branch usually needs to be merged into the trunk
-(main development branch) before most teams consider it fixed.
-Secondly, many teams have a separate process for confirming
-bugs are fixed over and above a developer saying so.
-
-As explained later, merge control features are currently under
-development in Launchpad and automatically changing the status of
-bugs to *Fix Committed* will be more appropriate once those features
-are in place.
-
-
-Associating a branch with a blueprint
+この1234というのはバグIDです。こうすると、バグ-ブランチ間の関連づけのステータスが *Fix Available* \
+に変わります。
+一回のコミットで複数の問題を修正する場合には、--fixesオプションを複数回指定できます。
+
+この機能のすばらしい点のひとつは、コミットするときにLaunchpadにアクセスできなくてもいいということです。
+``--fixes`` オプションではメタデータを保存し、次にそのブランチがLaunchpadにプッシュされたときか、\
+オンラインで再スキャンされたときに、Launchpadはそのメタデータを検出します。
+
+Note: Launchpadは、ブランチでバグが修正されたからといって勝手にバグをクローズすることはありません。
+これにはいくつかの理由があります。
+一つ目の理由は、ほとんどのチームでは、たいていブランチをトランク(メインの開発ブランチ)にマージしないと\
+バグが修正されたとはみなさないためです。
+二つ目の理由は、多くのチームでは、バグが修正されたことを確認するためには、「開発者がそう言っている」と\
+いうだけではなくそれ以外のプロセスがあるためです。
+
+あとで説明しますが、マージ管理機能が現在Launchpadで開発されており、この機能がリリースされれば、バグの\
+ステータスを *Fix Committed* に自動変更する機能がもっと適切なものになります。
+
+
+ブランチをブループリントと関連づける
 -------------------------------------
 
-After registering a branch, you can associate it to a blueprint so that
-people interested in that blueprint can track and test the feature as
-it develops.
-
-To do this, the steps are:
-
-1. Navigate to the blueprint in question.
-
-2. Select `Link branch` under `Actions`.
-
-3. Select the branch.
-
-If you wish, you can also provide some arbitrary comments about
-the relationship between the blueprint and the branch.
-
-
-Managing releases using Launchpad
+ブランチを登録したあと、それにブループリントを関連付けることができます。
+そうすることで、そのブループリントに関心を持つ人々がそのブランチを追いかけ、開発中の新機能をテストする\
+ことができます。
+
+そのための手順は以下の通りです。
+
+1. Questionから、そのブループリントに移動します。
+
+2. `Actions` から `Link branch` を選択します。
+
+3. ブランチを選択します。
+
+もし望むのなら、ブループリントとブランチとの関連づけに好きなコメントをつけることもできます。
+
+
+Launchpadをつかったリリースの管理
 =================================
 
-Integrating changes
+変更内容の統合
 -------------------
 
-Once a branch has been developed and published, communities
-typically go through a rigorous process before those changes
-are integrated into the core product and rolled out to end users.
-Some of the steps involved may include:
-
-* peer review of the changes
-
-* deciding which releases to include the changes in, e.g. the
-  next maintenance release, the next major release, or both
-
-* running functional regression tests
-
-* benchmarking to ensure performance remains acceptable
-
-* packaging into early access releases for end user testing
-
-* documentation updates, e.g. Release Notes for the targeted
-  releases
-
-* translation of the user interface and documentation into
-  multiple languages.
-
-This section briefly looks at some of the features in Launchpad that
-help get good quality code into production. Strong integration with
-Bazaar is core to making this happen smoothly.
-
-Note: Where indicated, some of the features below are still under
-development. If one or more of these features interest you, please
-consider joining the Launchpad beta test team at this link:
-https://help.launchpad.net/JoiningLaunchpadBetaTesters. You can
-then get early access to features and provide feedback to the
-developers before wider roll-out.
-
-
-Branch merge proposals
+ブランチが開発され、公開されたら、コミュニティでは一般的に、その変更内容をコアプロダクトに統合して\
+エンドユーザにリリースする前に厳格なプロセスを通します。
+その中には、以下のような手順が含まれるでしょう。:
+
+* 仲間による変更内容のレビュー
+
+* どのリリースにその変更を含めるかの決定。たとえば、次のメンテナンスリリース、次のメジャーリリース、\
+  もしくはその両方。
+
+* 機能の回帰テストの実行
+
+* パフォーマンスが基準を満たすかどうかを確認するためのベンチマーキング
+
+* エンドユーザテストのための早期提供リリースの作成
+
+* ドキュメントの更新。たとえば、そのリリースのためのリリースノートなど
+
+* ユーザインターフェイスやドキュメントの他言語への翻訳
+
+このセクションでは、プロダクトのコードの品質を高めるために役立つLaunchpadの機能のいくつかについて簡単に\
+説明します。
+Bazaarとの強力な一体化が、それをスムーズにするためのポイントです。
+
+Note: 以下にあげる機能の中には、まだ開発中のものもあります。
+もし、これらの機能に興味があるのなら、以下のリンクでLaunchpadベータテストチームへの参加を検討してください。:
+https://help.launchpad.net/JoiningLaunchpadBetaTesters
+そうすれば、各機能の早期提供版を手に入れて、広くリリースされる前に開発者にフィードバックを送ることができます。
+
+
+ブランチのマージの提案
+-----------------------
+
+Launchpadでブランチに移動したあとに実行できるアクションのひとつに、 *Propose for merging(マージの提案)*
+があります。
+この機能では、どのブランチがこのコードを取り入れるべきかを指定します。
+
+どのブランチがコードラインへのマージ提案中なのかという情報を追跡することは、リリース管理者が、\
+リリース日までに何を完了させなければならないか、または何を完了させることができるかを常に把握するために役立ちます。
+この情報をもとに、必要なレビューを実行してからブランチをマージすることを確実に行うことができます。
+単純なケースでは、リリース管理者は手作業でブランチをマージします。
+もっと高度なケースでは、ブランチがあるべきステータス(たとえば、 *Review completed(レビュー完了)*)になった\
+ときにロボット(`PQM`_ のような)に自動的にマージさせることができます。
+
+.. _PQM: https://launchpad.net/pqm
+
+
+コードレビューの追跡
 ----------------------
 
-After navigating to a branch in Launchpad, one of the available actions
-is *Propose for merging*. This lets you nominate which branch this code
-ought to be merged into.
-
-Tracking the knowledge about which branches are proposed to be merged
-into a codeline helps Release Managers keep on top of what still needs
-to be completed, or can be completed, before a ship date. Using this
-information, they can ensure branches are merged after completing any
-necessary reviews. In the simple case, the Release Manager may manually
-merge branches. In more advanced cases, the merging could be automatically
-done by a robot (like `PQM`_) when the branch reaches the right state
-(e.g. *Review completed*).
-
-.. _PQM: https://launchpad.net/pqm
-
-
-Code review tracking
---------------------
-
-A number of features are under development in Launchpad to track the
-states, conversations and outcomes of code reviews. These features are
-expected to be integrated with branch merge proposals and branch
-browsing features.
-
-
-Personal Package Archives (PPAs)
---------------------------------
-
-PPAs help developers and development teams get custom builds into the
-hands of users for early testing and feedback. In other words, a PPA
-allows a developer to form a community of testers who are interested
-in their changes. The testing community can install the packages,
-run them for the test period and then remove them cleanly from their
-system.
-
-See https://help.launchpad.net/PPAQuickStart for further details.
-
-
-Translations
+コードレビューの状態や会話の内容、成果物を追跡するために、Launchpadでたくさんの機能が開発されています。
+これらの機能は、ブランチのマージ提案やブランチ閲覧の機能に統合されると思われます。
+
+
+パーソナルパッケージアーカイブ(PPA)
+------------------------------------
+
+PPAは、開発者や開発チームが、早期提供版でテストとフィードバックを行うユーザにカスタムビルドモジュールを渡す\
+手助けをします。
+言い方を変えると、PPAによって、開発者はその変更内容に関心を持つテスターのコミュニティを結成することが\
+できるようになります。
+テスティングコミュニティは、パッケージをインストールし、テスト期間中にそれを実行し、その後はシステムから\
+きれいに削除することができます。
+
+より詳細な情報は、 https://help.launchpad.net/PPAQuickStart を参照してください。
+
+
+翻訳
 ------------
 
-The Translations module in Launchpad is designed to make it easy for
-anyone to get involved translating applications to languages they know.
-Translators are shielded from the low level details.
-
-Launchpad keeps track of the translations for each major version of a
-project separately, allowing translators to continue to improve the
-translations of your stable releases while others start work on newer
-versions that are still in development. Translation speed in reduced
-by sharing resources across projects. Automatic suggestions, from a
-library of 750,000 translated strings, and a community of 19,000
-registered translators can radically cut the time required to
-localise your project into many languages.
-
-
-Summary
+Launchpadの翻訳モジュールは、誰もがアプリケーションを自分が知る言語に簡単に翻訳できるように設計されています。
+翻訳者は、深いレベルの詳細を知る必要はありません。
+
+Launchpadは、プロジェクトの各メジャーバージョンに対する翻訳を別々に記録するので、だれかが新しい開発バージョンの\
+作業を始めていても、安定版の翻訳の改良を続けることができます。
+プロジェクトをまたがってリソースを共有することにより、翻訳のスピードを短縮しています。
+75万件の翻訳済みのテキストのライブラリからの自動抽出機能と、1万9千人の翻訳者のコミュニティとが、プロジェクトを\
+多くの言語に翻訳する時間を短縮してくれます。
+
+.. XXX Translation speed in reduces by sharing resources across projects. > in は isの間違いだと思われる。
+
+要約
 =======
 
-The communities we join, whether off-line or on-line,
-say a lot about the sort of people we are. The flip-side
-to this is that the tools you choose for your community - particularly
-the CDE and version control tool - can have a large impact on who
-joins and how easily they can contribute.
-
-In their own right, Launchpad and Bazaar are highly useful tools.
-Together, they can:
-
-* help your community track major assets such as source code and knowledge
-* help it grow by reducing barriers to entry
-* help it interact with related communities.
-
-In particular, Launchpad is a free code hosting service for your Bazaar
-branches, branches can be browsed online, branches can be linked to bugs
-and blueprints, and the status of bug-branch relationships can be
-automatically managed by mentioning the bug while committing in Bazaar.
-Further integration is under development with the aim of streamlining
-the process from *great idea* to *running code in the hands of end users*.
-
-If you have any feedback on how you'd like to see Bazaar and Launchpad
-further integrated, please contact us on the Bazaar mailing list,
+私たちが参加するコミュニティは、それがオフラインであれオンラインであれ、私たちがどのような種類の人間であるかを\
+表すものです。
+逆に言うと、あなたがコミュニティのために選ぶツール - 特に、CDEとバージョン管理ツール - は、誰がそのコミュニティ\
+に参加するのかということ、また、その人たちがどれだけ簡単にコミュニティに貢献できるかということに大きな影響を与えます。
+
+LaunchpadとBazaarは、単独でもとても役に立つツールです。
+一緒に使えば、以下のことが可能になります。:
+
+* コミュニティの、ソースコードやナレッジのような資産の追跡を助ける。
+* コミュニティへの参加の妨げを軽減して、コミュニティの成長を促す。
+* 関係するコミュニティとのやりとりを助ける。
+
+具体的には、LaunchpadはあなたのBazaarブランチを管理する無料のコードホスティングサービスであり、ブランチをオンラインで\
+閲覧でき、ブランチとバグやブループリントを関連づけることができ、Bazaarへのコミット時にバグについて記述することによって\
+ブランチ-バグ間のステータスを自動的に変更することができます。
+*すばらしいアイデア* を、 *エンドユーザの元で実行されるコード* にするまでのプロセスを合理化するための、より進んだ統合\
+機能が現在開発中です。
+
+もし、BazaarとLaunchpadがさらにどのように統合されてほしいかについて、何かフィードバックすることがあるのなら、\
+Bazzarメーリングリストで私たちに連絡してください。
 bazaar@lists.canonical.com.
 
-While designed as a free service to support open source projects,
-Canonical may make Launchpad available to commercial software developers
-depending on their requirements. We would be happy to hear from you
-if you think Launchpad would be useful for managing your community,
-open source or otherwise.
+Launchpadは自由ソフトウェアプロジェクトをサポートするための無料のサービスとしてデザインされていますが、Canonicalはこれを\
+商業ソフトウェアの開発者にも、その要望次第で提供することができます。
+オープンソースであってもなくても、Launchpadがあなたのコミュニティの運営に役立つと考えていただけるのならば幸いです。
+

=== modified file 'doc/ja/upgrade-guide/data_migration.txt'
--- doc/ja/upgrade-guide/data_migration.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/upgrade-guide/data_migration.txt	2011-05-18 06:27:34 +0000
@@ -13,7 +13,7 @@
 完全なバックアップはなにか悪いことがおきたときのセーフティネットとなります。
 
 古いブランチの消去は移行対象となるデータを減らすことができます。これをおこなうためのいくつかのコツを
-`Finding obsolete branches`_　でご覧ください。
+`Finding obsolete branches`_ 　でご覧ください。
 
 移行に関連するコマンドの紹介
 ----------------------------------
@@ -117,11 +117,13 @@
 
 2. 中央リポジトリから最新のブランチを作成した共用リポジトリへ格納します。
 
-3. あなたのローカルブランチで移行したいものを決定します。(もし決定していなければ、もちろんバックアップした後、  `Finding obsolete branches`_　してそのブランチを消去するのによい時間です。)
+3. あなたのローカルブランチで移行したいものを決定します。
+   (もし決定していなければ、もちろんバックアップした後、
+   `Finding obsolete branches`_ をしてそのブランチを消去するのによい時間です。)
 
 4. 各ローカルブランチの移行に関して2つ選択肢があります。
 
- * 新しいリポジトリで1つ空のブランチを **init** して古いリポジトリからリビジョンを **pull** する。
+ * 新しいリポジトリで1つ空のブランチを **init** して古いリポジトリからリビジョンを **pull** する。 
 
  * 新リポジトリにおいて、 trunkから新しいブランチに **branch** した後古いリポジトリにおいてあなたが行った変更を **merge** する。
 

=== modified file 'doc/ja/upgrade-guide/tips_and_tricks.txt'
--- doc/ja/upgrade-guide/tips_and_tricks.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/upgrade-guide/tips_and_tricks.txt	2011-05-18 06:01:08 +0000
@@ -1,7 +1,7 @@
 役立つヒントとコツ
 ######################
 
-古いブランチを見つけるには
+古いブランチを見つける
 -----------------------------
 
 もしあなたが開発における各変更や個々に行っている拡張のためにブランチの機能を使っている場合、いくつかのもう必要とされないブランチを持つことになるかもしれません。たいていの場合、関係のある変更は trunkにマージされているはずです。そのほかの場合ブランチは自分あるいは他人が行った別の変更の結果、古くなっているはずです。

=== modified file 'doc/ja/user-guide/adv_merging.txt'
--- doc/ja/user-guide/adv_merging.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/adv_merging.txt	2011-05-18 06:01:08 +0000
@@ -40,6 +40,31 @@
 フルマージが後で行われることはけっしてないと言える十分な理由があるからです。
 そうではない場面では、変更が再びマージされたときにまた衝突を解消する必要があります。
 
+.. Merging without parents
+
+親のないマージ
+------------------
+
+チェリーピックに関連したテクニックとして、マージ元のリビジョンを参照せずに、
+コミット前に親リビジョンを忘れることができます。
+こうすると、このマージで行われた全ての変更が、1回のコミットで行われたような
+効果があります。
+マージしたあと、コミットする前に次のコマンドを実行します。 ::
+
+  bzr revert --forget-merges
+
+これで、作業ツリー内の変更は残したまま、その変更がどこからマージされたかの
+記録だけを削除します。そして、次のコミットではマージされたリビジョンに
+関する情報が一切ない状態で、全ての変更を記録します。
+
+この機能は、「きれいな」履歴を作りたいユーザーに取って便利なものですが、
+Bazaarはもともとマージされた履歴を段階的に表示する機能を持っているので、
+失った履歴の価値が履歴のきれいさの価値を上回らないように気をつける必要があります。
+特に、この機能を使うとマージ元の変更だけを取り込んでその変更のマージ元を
+記録しないので、後で同じマージ元からマージしようとすると余計なコンフリクトを
+発生させる可能性があります。
+
+
 .. _reverse-cherrypicking:
 
 リバースチェリーピッキング

=== modified file 'doc/ja/user-guide/branching_a_project.txt'
--- doc/ja/user-guide/branching_a_project.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/branching_a_project.txt	2011-05-18 06:01:08 +0000
@@ -14,20 +14,38 @@
   ===========   ==========================================================================
   スキーム      説明
   ===========   ==========================================================================
-  file://       標準ファイルシステムを利用してアクセスする (デフォルト)
-  sftp://       SFTPを利用してアクセスする (大抵のSSHサーバーはSFTPを提供する)
-  bzr://        Bazaarのスマートサーバーを利用した高速のアクセス
-  ftp://        パッシブFTPを利用してアクセスする
-  http://       ウェブサーバーによってエクスポートされたブランチへの読み込み限定のアクセス
+  \file://      標準ファイルシステムを利用してアクセスする (デフォルト)
+  \bzr+ssh://   SSH ごしのアクセス   (リモートで最良の選択肢).
+  \sftp://      SFTPを利用してアクセスする (大抵のSSHサーバーはSFTPを提供する)
+  \bzr://       Bazaarのスマートサーバーを利用した高速のアクセス
+  \ftp://       パッシブFTPを利用してアクセスする
+  \http://      Webサーバーによって公開されたブランチへのアクセス
+  \https://     Webサーバーによって公開されたブランチへの、暗号化されたアクセス
   ===========   ==========================================================================
 
 上記で示されるように、ブランチは転送プロトコルを示すスキームを伴うURLを使用して識別されます。
 スキームが無ければ、通常のファイルの名前が想定されます。
 サポートされるプロトコルの完全なリストに関しては、
 ``urlspec`` のオンライントピックもしくはBazaarのユーザーリファレンスの
-`URLの識別子 <../user-reference/bzr_man.html#url-identifiers>`_
+`URLの識別子 <../user-reference/index.html#url-identifiers>`_
 のセクションを参照してください。
 
+URL は通常サーバーのルートディレクトリから解決されます。
+なので、 ``ftp://example.com/repo/foo`` はそのホストの ``/repo/foo``
+ディレクトリを意味しています。(「通常」といっているのは、Apacheの
+ようないくつかのサーバーソフトウェアはURLを任意の場所にリマップ
+できるからです。この場合はサーバーの設定ファイルを確認して、
+どのURLがどのディレクトリを参照しているかを探さないといけないでしょう)
+
+サーバー上の自分のホームディレクトリからの相対パスを使うには、チルダを
+使います。たとえば ``bzr+ssh://example.com/~/public_html`` と書くと、
+ホームディレクトリの中の ``public_html`` ディレクトリにマップされます。
+
+.. note:: HTTP や HTTPS ごしのアクセスはデフォルトでは読み込み専用です。
+   読み書き両方に対応するための設定については、
+   `HTTP スマートサーバーごしの push <http_smart_server.html#pushing-over-the-http-smart-server>`_
+   を参照してください。
+
 .. _a-reminder-about-shared-repositories:
 
 共用レポジトリに関するリマインダ

=== modified file 'doc/ja/user-guide/bug_trackers.txt'
--- doc/ja/user-guide/bug_trackers.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/bug_trackers.txt	2011-05-18 06:01:08 +0000
@@ -17,7 +17,7 @@
 Bugzilla、Trac、Roundupとその他のバグ/問題トラッカーに対してこれを設定する方法の詳細に関しては、
 Bazaarユーザーリファレンスの `バグトラッカーの設定`_ を参照してください。
 
-.. _バグトラッカーの設定: ../user-reference/bzr_man.html#bug-tracker-settings
+.. _バグトラッカーの設定: ../user-reference/index.html#bug-tracker-settings
 
 メタデータの記録 vs バグトラッカーの更新
 ------------------------------------------

=== modified file 'doc/ja/user-guide/bzrtools_plugin.txt'
--- doc/ja/user-guide/bzrtools_plugin.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/bzrtools_plugin.txt	2010-08-13 19:08:57 +0000
@@ -6,7 +6,7 @@
 
 BzrToolsは便利なBazaar強化機能のコレクションです。
 インストールの手引きに関しては、BzrToolsのホームページを参照してください:
-http://bazaar-vcs.org/BzrTools.
+http://wiki.bazaar.canonical.com/BzrTools.
 よく使われるコマンドのサンプルは下記のとおりです。
 
 

=== modified file 'doc/ja/user-guide/configuring_bazaar.txt'
--- doc/ja/user-guide/configuring_bazaar.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/configuring_bazaar.txt	2011-05-18 06:01:08 +0000
@@ -17,11 +17,66 @@
 
 ``whoami`` は引数なしで使われると、現在の値が表示されます。
 
+
+.. Using a network proxy
+
+ネットワークプロクシを使う
+---------------------------
+
+ネットワークが外部への接続に HTTP プロクシを必要とする場合、
+``http_proxy`` という環境変数を設定しなければなりません。
+https 接続にもプロクシが必要なら、 ``https_proxy`` も設定しなければなりません。
+プロクシが必要なのにこれらの環境変数が設定されていない場合、
+Launchpad やその他の外部のサーバーへの接続ができなかったりタイムアウトしたりします。
+
+Unix では、たいていこれらの設定は ``/etc/environment`` か
+``~/.bash_profile`` に書いて、 Windows ではたいていユーザープロファイルで
+設定します。
+
+::
+
+  http_proxy=http://proxy.example.com:3128/
+  https_proxy=http://proxy.example.com:3128/
+
+The ``no_proxy`` variable can be set to a comma-separated list of hosts
+which shouldn't be reached by the proxy.  (See
+<http://docs.python.org/library/urllib.html> for more details.)
+
+``no_proxy`` という環境変数に、プロクシを利用しないで到達するホスト名の
+リストをカンマ区切りで設定できます。
+(詳細は <http://docs.python.org/library/urllib.html> を参照してください)
+
+
+.. Various ways to configure
+
+いろいろな設定方法
+-------------------------
+
+上の例で示したように Bazaar を設定する方法はたくさんありますが、
+全てに共通している属性があります。オプションは全て以下のように
+なっています。
+
+- 名前は有効な Python の識別子です。
+
+- a value which is a string. In some cases, Bazaar will be able
+  to recognize special values like 'True', 'False' to infer a
+  boolean type, but basically, as a user, you will always specify
+  a value as a string.
+
+- 値は文字列です。いくつかの場面では、真偽値を得るために Bazaar は `True`,
+  `False` のような特別な値を認識しますが、基本的にはユーザーは値として
+  ただの文字列を渡します。
+
+オプションはコンテキストによってグループ化されており、オプション名は
+そのコンテキスト内ではユニークに識別することができます。
+必要な場合、オプションは設定ファイルに保存され永続化されます。
+
+
 設定ファイル
 -------------
 
-設定ファイルはLinux/Unixの場合 ``$HOME/.bazaar`` に、
-Windowsの場合 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0`` にあります。
+設定ファイルは Unix の場合 ``$HOME/.bazaar`` に、 Windows の場合
+``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0`` にあります。
 この場所には3つの主要な設定ファイルがあります:
 
 * ``bazaar.conf`` はデフォルトの設定オプションを記述します。
@@ -42,7 +97,66 @@
   email = Your Name <email@example.com>
 
 サポートされる構文と構成設定の詳細については、
-Bazaarのユーザーリファレンスの `構成設定 <../user-reference/bzr_man.html#configuration-settings>`_ の項目を参照してください。
+Bazaar のユーザーリファレンスの
+`構成設定 <../user-reference/index.html#configuration-settings>`_
+の項目を参照してください。
+
+
+.. Looking at the active configuration
+
+アクティブな設定を確認する
+-----------------------------------
+
+現在定義されている全てのオプションを確認するには、次のコマンドを実行します。 ::
+
+  bzr config
+
+``bzr`` は設定オプションをどこから取得するかを決定するためのいくつかのルールを
+持っています。
+
+現在のポリシーでは、以下の順序でマッチする定義を設定ファイルから探します。
+
+  * 最初に ``location.conf`` の中の、セクション名が場所(作業ツリー、ブランチ、
+    リモートブランチ)にマッチするセクションが探されます。
+
+  * 次に現在の ``branch.conf`` が探されます。
+
+  * 次に ``bazaar.conf`` が探されます。
+
+  * 最後に、いくつかのオプションはコード中で定義されたデフォルト値が設定され、
+    この設定は ``bzr config`` には表示されません。
+    (`構成設定 <../user-reference/index.html#configuration-settings>`_
+    を参照してください。)
+
+この動作は、 ``bzr config`` を引数なしで実行すると理解しやすいはずです。
+このコマンドを実行すると次のような表示をします。 ::
+
+  locations:
+    post_commit_to = commits@example.com
+    news_merge_files = NEWS
+  branch:
+    parent_location = bzr+ssh://bazaar.launchpad.net/+branch/bzr/
+    nickname = config-modify
+    push_location = bzr+ssh://bazaar.launchpad.net/~vila/bzr/config-modify/
+  bazaar:
+    debug_flags = hpss,
+
+各オプション定義のグループの前に表示されているスコープが、
+そのオプションを定義している構成設定ファイルを表しています。
+
+
+.. _modifying-the-active-configuration:
+
+有効な設定を変更する
+----------------------------------
+
+オプションに値を設定するには::
+
+  bzr config opt=value
+
+オプションの利用を止めるには::
+
+  bzr config --remove opt
 
 
 ルールベースのプリファレンス
@@ -50,5 +164,24 @@
 
 いくつかのコマンドとプラグインは特定のパターンにマッチするファイルのカスタムの処理機能を提供します。
 ユーザーごとにルールベースのプリファレンスが ``BZR_HOME/rules`` で定義されます。
+
 ルールが検索される検索方法と関連ファイルの詳細な構文に関する詳細については、
-Bazaarのユーザープリファレンスの `ルール <../user-reference/bzr_man.html#rules>`_ の項目を参照してください。
+Bazaarのユーザープリファレンスの `ルール <../user-reference/index.html#rules>`_
+の項目を参照してください。
+
+
+.. _escaping-command-lines:
+
+コマンドラインのエスケープ
+--------------------------------
+
+設定ファイルの中にプログラム名やコマンドラインを記述する場合、特殊な文字や
+スペースをその中に含めるためにクォートすることができます。
+同じルールが全てのプラットフォームで有効です。
+
+そのルールとは、ダブルクォートで囲まれた文字列はスペースが含まれていたとしても
+1つの「語」として認識され、クォート文字をクォートの中に含めるためにバックスラッシュ
+(訳注: 日本語環境では多くの場合バックスラッシュではなく円記号(ASCII文字の0x5c)です)
+を使います。例えば::
+
+    BZR_EDITOR="C:\Program Files\My Editor\myeditor.exe"

=== modified file 'doc/ja/user-guide/entering_commands.txt'
--- doc/ja/user-guide/entering_commands.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-guide/entering_commands.txt	2011-05-18 06:01:08 +0000
@@ -32,5 +32,5 @@
 
 Quietモードはエラーと警告のみを表示します。これはスクリプトで使う際に便利です。
 
-注: 大抵のコマンドは1つのレベルの冗長性だけをサポートします。今後これは変更されるかもしれません。
+注: 大抵のコマンドは1つのレベルの冗長性だけをサポートします。今後これは変更されるかもしれません。 
 高度な冗長性を求めるためには、-vオプションを複数回指定するだけです。

=== modified file 'doc/ja/user-guide/filtered_views.txt'
--- doc/ja/user-guide/filtered_views.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/filtered_views.txt	2011-05-18 06:01:08 +0000
@@ -25,11 +25,8 @@
 どちらのケースでも、Bazaarはユーザーに毎回viewが使われていることを\
 報告するので、操作や出力がマスクされていることが判ります。
 
-.. 2aになったらこのnoteは要らない
-.. Note: Bazaar's default format does not yet support filtered views. That
-   is likely to change in the near future. To use filtered views in the
-   meantime, you currently need to upgrade to ``development-wt6`` (or
-   ``development-wt6-rich-root``) format first.
+ノート: filtered view は 2a フォーマット(Bazaar 2.0 以降でデフォルトの
+フォーマット)でのみ有効です。
 
 
 view を作る

=== modified file 'doc/ja/user-guide/getting_help.txt'
--- doc/ja/user-guide/getting_help.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/getting_help.txt	2011-05-18 06:01:08 +0000
@@ -1,18 +1,21 @@
 helpを表示する
 ==============
 
-Bazaarは組み込みのオンラインヘルプシステムを搭載しています。
-トピックの一覧を見るためには::
-
+Bazaar は組み込みのオンラインヘルプシステムを搭載していて、
+次のコマンドでアクセスできます。 ::
+
+   bzr help
+
+help コマンドで、コマンドやコマンド以外のトピックに着いて調べることができます。
+それぞれのヘルプの一覧を見るには次のようにします。 ::
+
+  bzr help commands
   bzr help topics
 
-コマンドの一覧を見るためには::
-
-  bzr help commands
-
-トピックのxxxもしくはコマンドのxxxのヘルプを表示するためには::
-
-  bzr help xxx
+特定のコマンドのヘルプを見るには、次の２つの形式を使うことができます。 ::
+
+  bzr help status
+  bzr status --help
 
 ヘルプを検索するもしくは大きなドキュメントとして読みたい場合、
 情報はBazaarのユーザーリファレンスでも手に入ります。

=== modified file 'doc/ja/user-guide/hooks.txt'
--- doc/ja/user-guide/hooks.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/hooks.txt	2011-05-18 06:01:08 +0000
@@ -6,20 +6,22 @@
 
 Bazaarのふるまいをカスタマイズする1つの方法は *フック(hook)* です。
 フックによって特定のBazaarの特定のオペレーションの前後でアクションを実行できます。
-オペレーションは ``commit`` 、 ``push`` 、 ``pull`` 、と ``uncommit`` を含みます。
+オペレーションは ``commit``, ``push``, ``pull`` と ``uncommit`` を含みます。
 フックとパラメータの完全なリストに関しては、ユーザーリファレンスの
-`フック <../user-reference/bzr_man.html#hooks>`_ を参照してください。
-
-大抵のフックはクライアントで実行されますが、サーバーで実行されるものもわずかにあります。
-(サーバーサイドのオペレーションの特殊なケースを扱うものは `bzr-push-and-update`_ プラグインも参照。)
-
-.. _bzr-push-and-update: https://launchpad.net/bzr-push-and-update/
+`フック <../user-reference/index.html#hooks>`_ を参照してください。
+
+大抵のフックはクライアントで実行されますが、サーバーで実行されるものもわずかに
+あります。
+(サーバーサイドのオペレーションの特殊なケースを扱うものは
+`push-and-update plugin`_ も参照。)
+
+.. _push-and-update plugin: http://doc.bazaar.canonical.com/plugins/en/push-and-update-plugin.html
 
 
 フックを使用する
 -----------------
 
-フックを使用するには、 `プラグインを書きます <#writing-a-plugin>`_ 。
+フックを使用するには、 `プラグインを書きます`_ 。
 新しいコマンドを作成する代わりに、このプラグインはフックを定義してインストールします。例です::
 
     from bzrlib import branch
@@ -32,6 +34,8 @@
     branch.Branch.hooks.install_named_hook('post_push', post_push_hook,
                                      'My post_push hook')
 
+.. _プラグインを書きます: http://doc.bazaar.canonical.com/plugins/en/plugin-development.html
+
 この例を使用するには、 ``push_hook.py`` という名前のファイルを作り
 ``plugins`` サブディレクトリに設置します。
 (プラグインをインストールしていなければ、 ``plugins`` ディレクトリを作る必要があります)。
@@ -50,10 +54,72 @@
 2番目の引数はフック自身です。3番目の引数は ``'My post_push hook'`` という名前で、
 これは進行メッセージとエラーメッセージで使用されます。
 
+To reduce the start-up time of Bazaar it is also possible to "lazily" install hooks,
+using the ``bzrlib.hooks.install_lazy_named_hook`` function. This removes the need
+to load the module that contains the hook point just to install the hook. Here's lazy
+version of the example above:
+
+Bazaar のスタートアップ時間を短縮するために、フックを "遅延" インストールすることができます。
+遅延インストールには ``bzrlib.hooks.install_lazy_named_hook`` 関数を使います。
+遅延インストールを使えば、フックをインストールするためだけにフックポイントを含むモジュールを
+ロードする必要がなくなります。
+次の例は、上の例の遅延バージョンです。 ::
+
+    from bzrlib import hooks
+
+    def post_push_hook(push_result):
+        print "The new revno is %d" % push_result.new_revno
+
+
+    hooks.install_lazy_named_hook('bzrlib.branch', 'Branch.hooks',
+        'post_push', post_push_hook, 'My post_push hook')
+
 
 フックをデバッグする
 ---------------------
 
-インストールされたフックの一覧を表示するには、 ``hooks`` コマンドを使います::
+インストールされたフックの一覧 (と、利用可能なフックポイントの一覧) を表示するには、
+隠しコマンドである ``hooks`` コマンドを使います::
 
     bzr hooks
+
+
+例: マージプラグイン
+-----------------------
+
+次の例は ``Merger.merge_file_content`` フックのデモのための、完全なプラグインです。
+このプラグインは、 ``*.xml`` の名前のファイルに対する全てのマージに着いて、
+Bazaar がそのマージがクリーンだと判断しても必ず「衝突」状態にします。
+
+``merge_xml.py``::
+
+  """Custom 'merge' logic for *.xml files.
+  
+  Always conflicts if both branches have changed the file.
+  """
+  
+  from bzrlib.merge import PerFileMerger, Merger
+  
+  def merge_xml_files_hook(merger):
+      """Hook to merge *.xml files"""
+      return AlwaysConflictXMLMerger(merger)
+  
+  class AlwaysConflictXMLMerger(PerFileMerger):
+  
+      def file_matches(self, params):
+          filename = self.get_filename(params, self.merger.this_tree)
+          return filename.endswith('.xml')
+  
+      def merge_matching(self, params):
+          return 'conflicted', params.this_lines
+  
+  Merger.hooks.install_named_hook(
+      'merge_file_content', merge_xml_files_hook, '*.xml file merge')
+
+``merge_file_content`` hooks are executed for each file to be merged.  For
+a more a complex example look at the ``news_merge`` plugin that's bundled with
+Bazaar in the ``bzrlib/plugins`` directory.
+
+``merge_file_content`` フックは各ファイルがマージされるたびに呼ばれます。
+もっと複雑な例として、 Bazaar の ``bzrlib/plugins`` ディレクトリに同梱されている
+``news_merge`` プラグインも参照してください。

=== modified file 'doc/ja/user-guide/http_smart_server.txt'
--- doc/ja/user-guide/http_smart_server.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-guide/http_smart_server.txt	2011-05-18 06:01:08 +0000
@@ -1,11 +1,12 @@
-FastCGIでBazaarを提供する
-==========================
+Apache を使って Bazaar サーバーをたてる
+=========================================
 
-このドキュメントでは、Apache 2.0とFastCGIもしくはmod_pythonを利用して
-Bazaar HTTPスマートサーバーをセットアップする1つの方法を説明します。
+このドキュメントでは、 Apache 2.0 と FastCGI, mod_python, mod_wsgi の
+どれかを利用して Bazaar の HTTP スマートサーバーをセットアップする方法を
+説明します。
 
 スマートサーバーに関する詳細な情報とそれを設定する他の方法に関しては、
-メインのスマートサーバーのドキュメントを参照してください。
+`スマートサーバーのドキュメント <server.html>`_ を参照してください。
 
 例
 ---
@@ -26,7 +27,7 @@
 
     LoadModule fastcgi_module /usr/lib/apache2/modules/mod_fastcgi.so
     FastCgiIpcDir /var/lib/apache2/fastcgi
-
+    
 我々の例では、`http://example.com/code` で `/srv/example.com/www/code` をすでに提供しているので
 既存のApacheの設定は次のようになります::
 
@@ -44,9 +45,9 @@
         Options Indexes FollowSymLinks
         RewriteEngine On
         RewriteBase /code
-        RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
+        RewriteRule ^(.*/)?\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
     </Directory>
-
+    
     # bzr-smart.fcgiはDocumentRootの元に存在しないので、実行されるように
     # AliasはこれをURLの名前空間のエイリアスにする。
     Alias /srv/example.com/scripts/bzr-smart.fcgi /srv/example.com/scripts/bzr-smart.fcgi
@@ -56,7 +57,7 @@
             SetHandler fastcgi-script
         </Files>
     </Directory>
-
+    
 この設定はFastCGIを通して `/code` 内部の `/.bzr/smart` で終わるURLに対する
 Bazaarのスマートサーバーへのリクエストを扱うようにApacheに指示します。
 
@@ -68,19 +69,19 @@
 mod_python
 ~~~~~~~~~~
 
-最初に、次のようなコードをhttpd.confに追加してmod_pythonを設定します::
+最初に、次のようなコードを httpd.conf に追加して mod_python を設定します::
 
     LoadModule python_module /usr/lib/apache2/modules/mod_python.so
 
-FastCGIと同じ方法でmod_rewriteを用いて書き換えルールを定義します::
+FastCGI と同じ方法で mod_rewrite を用いて書き換えルールを定義します::
 
-    RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
+    RewriteRule ^(.*/)?\.bzr/smart$ /srv/example.com/scripts/bzr-smart.fcgi
 
 変更は次のようになります::
 
-    RewriteRule ^(.*/|)\.bzr/smart$ /srv/example.com/scripts/bzr-smart.py
+    RewriteRule ^(.*/)?\.bzr/smart$ /srv/example.com/scripts/bzr-smart.py
 
-mod_fastcgiのように、スクリプトがどのように扱われるのかも定義します::
+mod_fastcgi のように、スクリプトがどのように扱われるのかも定義します::
 
     Alias /srv/example.com/scripts/bzr-smart.py /srv/example.com/scripts/bzr-smart.py
     <Directory /srv/example.com/scripts>
@@ -91,10 +92,10 @@
         </Files>
     </Directory>
 
-この設定はmod_pythonを通して `/code` 内部の `/.bzr/smart` で終わるURLに対するリクエストを
-Bazaarのスマートサーバーに渡すように指示します。
+この設定は mod_python を通して `/code` 内部の `/.bzr/smart` で終わるURLに対するリクエストを
+Bazaar のスマートサーバーに渡すように指示します。
 
-注: bzrlibがPATHの中に存在しない場合、次の行を変更する必要があります::
+注: bzrlib が PATH の中に存在しない場合、次の行を変更する必要があります::
 
             PythonPath "sys.path+['/srv/example.com/scripts']"
 
@@ -108,6 +109,32 @@
 .. _mod_python: http://www.modpython.org/
 
 
+mod_wsgi
+~~~~~~~~
+
+最初に、 a2enmod wsgi などで mod_wsgi を有効にしておきます。
+次に、 `.bzr/smart` で終わる全ての URL に対するリクエストを mod_wsgi 経由
+で処理するように設定します。設定例は次のようになります。 ::
+
+    WSGIScriptAliasMatch ^/code/.*/\.bzr/smart$ /srv/example.com/scripts/bzr.wsgi
+
+    #The three next lines allow regular GETs to work too
+    RewriteEngine On
+    RewriteCond %{REQUEST_URI} !^/code/.*/\.bzr/smart$
+    RewriteRule ^/code/(.*/\.bzr/.*)$ /srv/example.com/www/code/$1 [L]
+
+    <Directory /srv/example.com/www/code>
+        WSGIApplicationGroup %{GLOBAL}
+    </Directory>
+
+この設定では、 Apache は `/code` 以下の `/.bzr/smart` で終わる URL に
+対する全てのリクエストを WSGI 経由で Bazaar のスマートサーバーに渡し、
+それ以外の全てのリクエストは Apache が直接扱うようにしています。
+
+詳細は mod_wsgi_ のドキュメントを参照してください。
+
+.. _mod_wsgi: http://code.google.com/p/modwsgi/
+
 Bazaarを設定する
 -----------------
 
@@ -130,7 +157,7 @@
         enable_logging=True)
 
     fcgi.WSGIServer(smart_server_app).run()
-
+        
 　`fcgi`のモジュールはhttp://svn.saddi.com/py-lib/trunk/fcgi.pyで見つかります。
 これは flup_ の一部です。
 
@@ -158,18 +185,37 @@
         """Handle a single request."""
         wsgi_server = modpywsgi.WSGIServer(smart_server_app)
         return wsgi_server.run(request)
-
+        
 `modpywsgi` モジュールは
-`http://dev.pocoo.org/projects/pocoo/browser/pocoo/wrappers/modpy.py` で見つかります。
-これは pocoo_ の一部です。modpywsgi.pyをbzr-smart.pyと同じディレクトリ
+http://ice.usq.edu.au/svn/ice/trunk/apps/ice-server/modpywsgi.py で見つかります。
+これは pocoo_ の一部でした。 modpywsgi.py を bzr-smart.py と同じディレクトリ
 (すなわち/srv/example.com/scripts/)に設置していることを確認してください。
 
 .. _pocoo: http://dev.pocoo.org/projects/pocoo/
 
+
+mod_wsgi
+~~~~~~~~
+
+We've configured Apache to run the smart server at
+`/srv/example.com/scripts/bzr.wsgi`.  This is just a simple script we need
+to write to configure a smart server, and glue it to the WSGI gateway.
+Here's what it looks like::
+
+    from bzrlib.transport.http import wsgi
+
+    def application(environ, start_response):
+        app = wsgi.make_app(
+            root="/srv/example.com/www/code/",
+            prefix="/code",
+            readonly=True,
+            enable_logging=False)
+        return app(environ, start_response)
+
 クライアント
 ------------
 
-これで `bzr+http://` 形式のURLを利用できます::
+これで `bzr+http://` 形式のURLやただの `http://` のURLを利用できます::
 
     bzr log bzr+http://example.com/code/my-branch
 
@@ -185,13 +231,13 @@
 WSGI標準に準拠するサードパーティのWSGIのミドルウェアもしくはサーバーで利用できます。
 唯一の要件は以下のとおりです:
 
-  * 　`SmartWSGIApp` をコンストラクトするためには、それが提供する **root transport** を指定する必要があります。
+  * `SmartWSGIApp` をコンストラクトするためには、それが提供する **root transport** を指定する必要があります。
   * それぞれのリクエストの `environ` dict は **'bzrlib.relpath'** 変数の設定を持たなければなりません。
 
 この例で使われている `make_app` ヘルパーは それに渡される `root` パスに基づいたトランスポートを伴う
 `SmartWSGIApp` をコンストラクトし、引数 `prefix` と`path_var` に基づくそれぞれのリクエストに対する
 　`bzrlib.relpath` を算出します。
-上記の例において、これは (Apacheによって設定される)'REQUEST_URI' を取り、接頭辞の '/code/' と接尾辞の '/.bzr/smart'
+上記の例において、これは (Apacheによって設定される)'REQUEST_URI' を取り、接頭辞の '/code/' と接尾辞の '/.bzr/smart' 
 をはぎ取り、それを 'bzrlib.relpath' として設定するので、 '/code/foo/bar/.bzr/smart' に対するリクエストは
 'foo/bzr' の 'bzrlib.relpath' になります。
 
@@ -202,20 +248,48 @@
 .. _WSGI標準: http://www.python.org/dev/peps/pep-0333/
 
 
-``bzr+http://`` を通してpushする
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+HTTP スマートサーバー経由で push する
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-httpスマートサーバーを通してデータをプッシュすることは可能です。
+HTTP スマートサーバーを通してデータをプッシュすることは可能です。
 これを行うための最も簡単な方法は、 ``wsgi.make_app()`` コールに ``readonly=False`` を
 提供するだけです。ただし、スマートプロトコルは認証機能が含まれないので注意してください。
 書き込みのサポートを有効にする場合、
-実際にシステム上のデータを書き込みできる人を制限するために、 ``.bzr/smart`` URLへの権限を制限するとよいでしょう。
-
-現時点では、同じURLに対して読み込み限定の人と読み込みと書き込みの人を分けることはできません。
+実際にシステム上のデータを書き込みできる人を制限するために、 ``.bzr/smart``
+URLへの権限を制限するとよいでしょう。例えば Apache で次のような設定を
+します。 ::
+
+    <Location /code>
+        AuthType Basic
+        AuthName "example"
+        AuthUserFile /srv/example.com/conf/auth.passwd
+        <LimitExcept GET>
+            Require valid-user
+        </LimitExcept>
+    </Location>
+
+
+現時点では、同じURLに対して読み込み限定の人と読み込みと書き込みの人を
+分けることはできません。
 (認証を行う)HTTPレイヤーにおいて、すべては単なるPOSTリクエストだからです。
 しかしながら、HTTPSアクセスの場合に認証が必要な書き込みサーバーを使い、
-プレーンなHTTPは読み込み限定のアクセスを許可することは確実にできます。
-
-
-..
+プレーンなHTTPは読み込み限定のアクセスを許可することはできます。
+
+HTTPS サイトに対してアクセスしたときに bzr が次のようなエラーを表示する
+場合::
+
+  bzr: ERROR: Connection error: curl connection error (server certificate verification failed.
+  CAfile:/etc/ssl/certs/ca-certificates.crt CRLfile: none)
+
+You can workaround it by using ``https+urllib`` rather than ``http`` in your
+URL, or by uninstalling pycurl.  See `bug 82086`_ for more details.
+
+URL に ``https`` の代わりに ``https+urllib`` を使うことで問題を回避
+できます。
+詳細については `bug 82086`_ を参照してください。
+
+.. _bug 82086: https://bugs.launchpad.net/bzr/+bug/82086
+
+
+.. 
    vim: ft=rst tw=74 et

=== added file 'doc/ja/user-guide/index-plain.txt'
--- doc/ja/user-guide/index-plain.txt	1970-01-01 00:00:00 +0000
+++ doc/ja/user-guide/index-plain.txt	2011-05-18 06:01:08 +0000
@@ -0,0 +1,120 @@
+####################
+Bazaarユーザーガイド
+####################
+
+.. Please mark sections in included files as following:
+..   level 1 ========
+..   level 2 --------
+..   level 3 ~~~~~~~~
+..   level 4 ^^^^^^^^ (it is better not to use nesting deeper than 3 levels)
+
+.. contents:: :depth: 3
+
+
+紹介
+############
+
+.. include:: introducing_bazaar.txt
+.. include:: core_concepts.txt
+.. include:: bazaar_workflows.txt
+
+
+始める
+###############
+
+.. include:: installing_bazaar.txt
+.. include:: entering_commands.txt
+.. include:: getting_help.txt
+.. include:: configuring_bazaar.txt
+.. include:: using_aliases.txt
+.. include:: plugins.txt
+.. include:: zen.txt
+
+
+個人用途のバージョン管理
+########################
+
+.. include:: solo_intro.txt
+.. include:: starting_a_project.txt
+.. include:: controlling_registration.txt
+.. include:: reviewing_changes.txt
+.. include:: recording_changes.txt
+.. include:: browsing_history.txt
+.. include:: releasing_a_project.txt
+.. include:: undoing_mistakes.txt
+
+
+同僚と共有する
+##################
+
+.. include:: partner_intro.txt
+.. include:: branching_a_project.txt
+.. include:: merging_changes.txt
+.. include:: resolving_conflicts.txt
+.. include:: annotating_changes.txt
+
+
+集中スタイルの、チームコラボレーション
+############################################
+
+.. include:: central_intro.txt
+.. include:: publishing_a_branch.txt
+.. include:: using_checkouts.txt
+.. include:: working_offline_central.txt
+.. include:: reusing_a_checkout.txt
+
+
+分散スタイルの、チームコラボレーション
+########################################
+
+.. include:: distributed_intro.txt
+.. include:: organizing_branches.txt
+.. include:: using_gatekeepers.txt
+.. include:: sending_changes.txt
+
+
+その他のトピック
+####################
+
+.. include:: part2_intro.txt
+.. include:: adv_merging.txt
+.. include:: shelving_changes.txt
+.. include:: filtered_views.txt
+.. include:: stacked.txt
+.. include:: server.txt
+.. include:: hooks.txt
+.. include:: version_info.txt
+
+
+人気のあるプラグインの手短なツアー
+####################################
+
+.. include:: bzrtools_plugin.txt
+.. include:: svn_plugin.txt
+.. include later looms_plugin.txt
+
+
+Bazaarを環境に統合する
+#######################
+
+.. include:: web_browsing.txt
+.. include later - file_explorers.txt
+.. include later - desktop_integration.txt
+.. include later - editors_and_ides.txt
+.. include later - email.txt
+.. include:: bug_trackers.txt
+
+
+付録
+#####
+
+.. include:: specifying_revisions.txt
+.. include:: organizing_your_workspace.txt
+.. include:: shared_repository_layouts.txt
+.. include:: setting_up_email.txt
+.. include:: http_smart_server.txt
+.. include:: writing_a_plugin.txt
+.. include:: licence.txt
+
+.. |--| unicode:: U+2014
+

=== modified file 'doc/ja/user-guide/index.txt'
--- doc/ja/user-guide/index.txt	2010-07-07 11:21:19 +0000
+++ doc/ja/user-guide/index.txt	2011-05-18 06:19:42 +0000
@@ -136,5 +136,6 @@
    setting_up_email
    http_smart_server
    writing_a_plugin
+   licence
 
 .. |--| unicode:: U+2014

=== modified file 'doc/ja/user-guide/installing_bazaar.txt'
--- doc/ja/user-guide/installing_bazaar.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/installing_bazaar.txt	2011-05-18 06:01:08 +0000
@@ -1,19 +1,19 @@
 Bazaarをインストールする
 =========================
 
-Linux
------
+GNU/Linux
+----------
 
-BazaarのパッケージはUbuntu/Debian、Red HatとGentooを含む人気のある
-大抵のLinuxディストリビューションで利用できます。
-最新の手引きは http://bazaar-vcs.org/Download を参照してください。
+Bazaar のパッケージは Ubuntu/Debian, Red Hat と Gentoo を含む人気のある
+大抵の GNU/Linux ディストリビューションで利用できます。
+最新の手引きは http://wiki.bazaar.canonical.com/Download を参照してください。
 
 Windows
 -------
 
 Windowsユーザーは、Bazaarのコアパッケージと一緒に必要な前提要件と
 いくつかの便利なプラグインを含むインストーラが利用できます。
-最新の手引きは http://bazaar-vcs.org/Download を参照してください。
+最新の手引きは http://wiki.bazaar.canonical.com/Download を参照してください。
 
 注: Windows上でCygwinを動かしている場合、Cygwin用のBazaarパッケージを\
 Windows版の代わりに使わなければなりません。
@@ -34,8 +34,9 @@
 
   1. まだインストールしていなければ、バージョン2.4以降のPythonをインストールします。
 
-  2. http://bazaar-vcs.org/Download もしくはLaunchpad (https://launchpad.net/~bzr/)から
-     ``bazaar-xxx.tar.gz`` ファイル(xxxはバージョン番号)をダウンロードします。
+  2. http://wiki.bazaar.canonical.com/Download もしくは Launchpad
+     (https://launchpad.net/~bzr/)から ``bazaar-xxx.tar.gz`` ファイル
+     (xxxはバージョン番号)をダウンロードします。
 
   3. tar、WinZipもしくは同等のコマンドを用いてアーカイブを解凍します。
 

=== modified file 'doc/ja/user-guide/introducing_bazaar.txt'
--- doc/ja/user-guide/introducing_bazaar.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-guide/introducing_bazaar.txt	2011-05-18 06:01:08 +0000
@@ -43,13 +43,13 @@
 **チェックアウト** (*checkout*) する必要があります。
 チェックアウトにより、ユーザーが修正できるツリー(**作業ツリー**, *working tree*)ができます。
 これらの変更を記録(**コミット**, *commit*)するためには、集中型のサーバーにアクセスする権限が必要で、\
-コミットをする前に作業内容を最新バージョンとマージしなければなりません。
+コミットをする前に作業内容を最新バージョンとマージしなければなりません。 
 このアプローチは集中型モデルとして知られます。
 
 集中型モデルは長期にわたって有用であることを示してきましたが顕著な欠点がいくつかあります。
 先ず第一に、集中型のVCSはバージョン管理機能を利用するためにはサーバーに接続できることを要求します。
 二番目に、集中型のモデルは変更の **スナップショットの記録** のふるまいと変更の **公開**
-のふるまいと密接にリンクさせます。
+のふるまいと密接にリンクさせます。 
 これは状況によってはよいこともありますが別の状況では品質にわるい影響を与えることがあります。
 
 分散型のVCSツールは単独の集中型のレポジトリではなく複数のレポジトリをユーザーとチームに持たせます。
@@ -64,14 +64,15 @@
  * ルーチンタスクの時間が減ります - より創造的な活動に時間を割けます
  * "feature-wide" コミットの利用を通してリリース管理の柔軟性が増します
  * トランクの質と安定性を高い水準で維持でき、それによって作業者のストレスが少なくなります
- * オープンソースのコミュニティにおいて:
+ * オープンなコミュニティにおいて:
 
    * コア開発者ではない人が作成と変更の維持をしやすくなります
    * コア開発者が外部の開発者と共同作業をしてコアのチームに組み込むことが楽になります
 
  * 会社において、分散されたチームや外部のチームと連携するのが簡単になります
 
-分散型VCSツールの集中型より優れた点の詳細な一覧に関しては、 http://bazaar-vcs.org/BzrWhy を参照してください。
+分散型VCSツールの集中型より優れた点の詳細な一覧に関しては、 http://wiki.bazaar.canonical.com/BzrWhy
+を参照してください。
 
 
 Bazaarの主要な機能
@@ -79,12 +80,12 @@
 
 Bazaarは現存する唯一のVCSツールではありませんが、多くのチームとコミュニティにとって\
 よい選択肢になる優れた機能をいくつか持ちます。
-別のVCSツールとの比較の要約はBazaarのWiki、 http://bazaar-vcs.org/ で見つかります。
+別のVCSツールとの比較の要約はBazaarのWiki、 http://wiki.bazaar.canonical.com で見つかります。
 
 多くの機能の中で、とりわけ強調するものがあります:
 BazaarはPythonで書かれた完全にフリーなソフトウェアです。
 結果として、改善に貢献することは簡単です。
-手を貸して頂けるのであれば、 http://bazaar-vcs.org/BzrSupport を参照して頂くようお願いします。
+手を貸して頂けるのであれば、 http://wiki.bazaar.canonical.com/BzrSupport を参照して頂くようお願いします。
 
 
 さらに学ぶ
@@ -110,8 +111,8 @@
 は利用可能なコマンドとオプションのすべての詳細を提供します。
 
 .. _5分でBazaar: ../mini-tutorial/index.html
-.. _Bazaarクィックスタートカード: ../quick-reference/quick-start-summary.svg
-.. _Bazaarユーザーリファレンス: ../user-reference/bzr_man.html
+.. _Bazaarクィックスタートカード: ../quick-reference/index.html
+.. _Bazaarユーザーリファレンス: ../user-reference/index.html
 
 このマニュアルがお役に立てることを願っております。Bazaarの残りのドキュメントの改善を提案したいのであれば、
 メーリングリスト(bazaar@lists.canonical.com)に連絡して頂けるようお願いします。

=== added file 'doc/ja/user-guide/licence.txt'
--- doc/ja/user-guide/licence.txt	1970-01-01 00:00:00 +0000
+++ doc/ja/user-guide/licence.txt	2011-05-18 06:01:08 +0000
@@ -0,0 +1,7 @@
+Licence
+===============
+
+Copyright 2009-2011 Canonical Ltd. Bazaar is free software, and you
+may use, modify and redistribute both Bazaar and this document under
+the terms of the GNU General Public License version 2 or later.  See
+<http://www.gnu.org/licenses/>.

=== modified file 'doc/ja/user-guide/merging_changes.txt'
--- doc/ja/user-guide/merging_changes.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/merging_changes.txt	2011-05-18 06:01:08 +0000
@@ -28,7 +28,7 @@
 したいかもしれません。
 この場合、彼女は最初に明示的にURLを渡す必要があります::
 
-  bzr merge sftp://mary@bill-laptop/cool-repo/cool-trunk
+  bzr merge bzr+ssh://mary@bill-laptop/cool-repo/cool-trunk
 
 マージブランチがまだ設定されていなければ、これによってデフォルトの\
 マージブランチが設定されます。

=== modified file 'doc/ja/user-guide/organizing_branches.txt'
--- doc/ja/user-guide/organizing_branches.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/organizing_branches.txt	2011-05-18 06:01:08 +0000
@@ -15,7 +15,7 @@
 
   bzr init-repo PROJECT
   cd PROJECT
-  bzr branch sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr branch bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 タスクのブランチ
 -----------------

=== modified file 'doc/ja/user-guide/organizing_your_workspace.txt'
--- doc/ja/user-guide/organizing_your_workspace.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/organizing_your_workspace.txt	2011-05-18 06:01:08 +0000
@@ -132,7 +132,7 @@
 変化形として、trunkをチェックアウトとして作ることもできます。
 もしtrunkへのコミット権限を持っているのであれば、trunkへマージして\
 コミットするとその変更は暗黙的に公開されます。
-他には、trunkのURLが読み込み専用だった場合(例: httpアドレス)、チェックアウトを\
+他には、trunkのURLが読み込み専用だった場合(例: HTTP アドレス)、チェックアウトを\
 利用することはローカルのtrunkミラーに間違えて変更を登録してしまうことを\
 防止します。これはプロジェクトがPQMなどのゲートキーパー型のワークフロー\
 を利用していたときに有用です。

=== modified file 'doc/ja/user-guide/plugins.txt'
--- doc/ja/user-guide/plugins.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/plugins.txt	2011-05-18 06:01:08 +0000
@@ -26,19 +26,21 @@
 プラグインが見つかる場所
 -------------------------
 
-http://bazaar-vcs.org/BzrPlugins ページでプラグインのリストが見つかります。
+http://wiki.bazaar.canonical.com/BzrPlugins ページで
+プラグインのリストが見つかります。
 
 プラグインをインストールする方法
 ---------------------------------
 
 プラグインのインストール作業はとても簡単です！　まだ作られていなければ、
 Bazaarの設定ディレクトリの元で ``plugins`` ディレクトリを作ります。
-Linuxの場合は ``~/.bazaar/`` でWindowsの場合は
+Unix の場合は ``~/.bazaar/`` でWindowsの場合は
 ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0\`` です。
 このディレクトリの範囲内では(下記では$BZR_HOMEとして言及される) それぞれのプラグインは独自のサブディレクトリに設置されます。
 
 プラグインはとりわけBazaarのブランチとよく連携します。
-たとえば、Linuxのメインのユーザーアカウント用にbzrtoolsプラグインをインストールするためには、次のコマンドを実行します::
+たとえば、 GNU/Linux のメインのユーザーアカウント用に bzrtools
+プラグインをインストールするためには、次のコマンドを実行します::
 
     bzr branch http://panoramicfeedback.com/opensource/bzr/bzrtools
     ~/.bazaar/plugins/bzrtools
@@ -50,20 +52,14 @@
 プラグインの代替用の場所
 -------------------------
 
-必要なパーミッションがあれば、プラグインをシステム全体のベースにインストールすることもできます。
-現在2つの場所がプラグインのためにチェックされます:
-
- 1. システムの場所 - bzrlib/plugins
- 2. 個人の場所 - $BZR_HOME/plugins.
-
-Linuxにおいて、これらの典型的な場所は
-``/usr/lib/python2.4/site-packages/bzrlib/plugins/`` と
-``$HOME/.bazaar/plugins/`` です。
-Windowsにおいて、システムの場所は
-``C:\\Program Files\\Bazaar\\plugins`` になり、
-個人の場所は ``C:\Documents and Settings\<username>\Application Data\Bazaar\2.0\plugins`` です。
-
-環境変数 ``BZR_PLUGIN_PATH`` をプラグインが含まれるディレクトリに設定することで\個人のプラグインの場所を上書きできます。
+必要なパーミッションがあれば、プラグインをシステム全体のベースに
+インストールすることもできます。
+
+環境変数 ``BZR_PLUGIN_PATH`` をプラグインが含まれるディレクトリに
+設定することで個人のプラグインの場所を上書きできます。
+(詳細な解説は
+`ユーザーリファレンス <../user-reference/configuration-help.html#bzr-plugin-path>`_ 
+を参照してください。)
 
 
 インストールされたプラグインの一覧を表示する

=== modified file 'doc/ja/user-guide/publishing_a_branch.txt'
--- doc/ja/user-guide/publishing_a_branch.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/publishing_a_branch.txt	2011-05-18 06:01:08 +0000
@@ -1,3 +1,5 @@
+.. _publishing_a_branch:
+
 ブランチを公開する
 ===================
 
@@ -16,7 +18,7 @@
 なので、そのような共有リポジトリを作るときには通常 ``no-trees``
 オプションを使います::
 
-  bzr init-repo --no-trees sftp://centralhost/srv/bzr/PROJECT
+  bzr init-repo --no-trees bzr+ssh://centralhost/srv/bzr/PROJECT
 
 この手順をcvsrootもしくはSubversionのリポジトリのセットアップとして\
 似たようなものとして考えることができます。
@@ -43,20 +45,19 @@
   bzr add                (リポジトリを投入する; バージョン管理を始める)
   bzr commit -m "Initial import"
                          (中心リポジトリに公開する)
-  bzr push sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr push bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 2番目のやり方の例です::
 
   bzr init-repo PROJECT  (ローカルリポジトリを準備する)
   cd PROJECT
-  bzr init sftp://centralhost/srv/bzr/PROJECT/trunk
-  bzr checkout sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr init bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
+  bzr checkout bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
   cd trunk
   cp -ar ~/PROJECT .     (OS固有のツールを使用してファイルをコピーする)
   bzr add                (リポジトリを投入する; バージョン管理を始める)
   bzr commit -m "Initial import"
                          (中心リポジトリに公開する)
-  bzr push sftp://centralhost/srv/bzr/project/trunk
 
 ``checkout`` コミットを使って作られた作業ツリー内部でコミットすると\
 ローカルと同様に内容は暗黙の内に中心位置にコミットされることに注意してください。

=== modified file 'doc/ja/user-guide/reusing_a_checkout.txt'
--- doc/ja/user-guide/reusing_a_checkout.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/reusing_a_checkout.txt	2011-05-18 06:01:08 +0000
@@ -60,7 +60,7 @@
 
   bzr init-repo --no-trees PROJECT
   cd PROJECT
-  bzr branch sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr branch bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
   bzr checkout --lightweight trunk my-sandbox
   cd my-sandbox
   (hack away)
@@ -71,7 +71,7 @@
 応じてそれらの間で切り替えることができます。例です::
 
   (my-sandboxの中にいることを前提とする)
-  bzr branch sftp://centralhost/srv/bzr/PROJECT/PROJECT-1.0 ../PROJECT-1.0
+  bzr branch bzr+ssh://centralhost/srv/bzr/PROJECT/PROJECT-1.0 ../PROJECT-1.0
   bzr switch ../PROJECT-1.0
   (fix bug in 1.0)
   bzr commit -m "blah, blah blah"

=== modified file 'doc/ja/user-guide/reviewing_changes.txt'
--- doc/ja/user-guide/reviewing_changes.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/reviewing_changes.txt	2011-05-18 06:01:08 +0000
@@ -37,8 +37,15 @@
 
 ``-r`` オプションによって、ツリーは前のリビジョン、もしくは示された2つのリビジョンの違いを表示します::
 
-    % bzr diff -r 1000..          # everything since r1000
-    % bzr diff -r 1000..1100      # changes from 1000 to 1100
+    % bzr diff -r 1000..          # r1000 からの全ての変更
+    % bzr diff -r 1000..1100      # 1000 から 1100 までの変更
+
+1つのリビジョンの変更だけを見たい場合は、 ``-c`` オプションを利用します。
+
+::
+
+    % bzr diff -c 1000            # r1000 による変更
+                                  # -r999..1000 と同じ意味
 
 ``--diff-options`` オプションによってbzrは外部のdiffプログラムにオプションを渡して実行します。例です::
 

=== modified file 'doc/ja/user-guide/sending_changes.txt'
--- doc/ja/user-guide/sending_changes.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/sending_changes.txt	2011-05-18 06:01:08 +0000
@@ -39,7 +39,7 @@
 オプションなしの ``send`` はマージディレクティブを作り、Eメールツールを作動させて添付し、\
 少しの説明文を追加するための準備をします。
 (この設定方法の詳細に関してはオンラインヘルプの ``send`` と
-リファレンスマニュアルの `構成設定 <../user-reference/bzr_man.html#configuration-settings>`_
+リファレンスマニュアルの `構成設定 <../user-reference/index.html#configuration-settings>`_
 を参照)
 
 大抵のプロジェクトではパッチ付きのメールに説明文を追加し、パッチの理由と内容を説明します。

=== modified file 'doc/ja/user-guide/server.txt'
--- doc/ja/user-guide/server.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/server.txt	2011-05-18 06:01:08 +0000
@@ -15,7 +15,7 @@
 Bazaarはリモートからの読み込みを許可します。
 実行しているBazaarコマンドの中でブランチへのURLを入力するだけです。::
 
-    bzr log http://bazaar-vcs.org/bzr/bzr.dev
+    bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev
 
 BazaarはFTP、SFTPと(プラグインを通した)HTTP-WebDAVを通した書き込みをサポートします。
 
@@ -38,17 +38,27 @@
 SSH
 ~~~
 
-SSHを通してBazaarを利用する際にサーバー上の特別な設定は必要ありません::
+SSHを通してBazaarを利用する際にサーバー上の特別な設定は必要ありません。
+サーバーに Bazaar がインストールされていれば、 ``bzr+ssh`` の URL を
+利用することができます。 例::
+
+    bzr log bzr+ssh://host/path/to/branch
+
+`bzr` がサーバーのシステムワイドな場所にインストールされていない場合、
+リモートの `bzr` がどこにあるかをローカルの `bzr` に教える必要があるかも
+しれません。 ::
 
     BZR_REMOTE_PATH=~/bin/bzr bzr log bzr+ssh://host/path/to/branch
 
 ``BZR_REMOTE_PATH`` 環境変数はリモートシステムで `bzr` が起動する方法を調整します。
 デフォルトでは単に `bzr` として起動するので、 `bzr` 実行ファイルはデフォルトの\
 検索パス上にあることが要求されます。
+この設定を場所ごとの設定ファイルである ``locations.conf`` に書いて
+永続化することもできます。
 
-``bzr+ssh://`` URLスキームはファイルシステムのrootからの絶対パスだけをサポートします。
-将来のバージョンでは ``sftp://`` URL (https://bugs.launchpad.net/bzr/+bug/109143) と同じように
-``~`` もサポートされる予定です。
+SFTP と同じく、 ``~`` で始まるパスはホームディレクトリからの相対パスになります。
+例えば ``bzr+ssh://example.com/~code/proj`` のようになります。
+加えて、 ``~user`` は user のホームディレクトリからの相対パスになります。
 
 inetd
 ~~~~~
@@ -58,13 +68,17 @@
 
 inetdからBazaarサーバーを動かすにはinetd.confエントリが必要です::
 
-    4155  stream  tcp  nowait  bzruser  /usr/bin/bzr /usr/bin/bzr serve --inet --directory=/srv/bzr/repo
+    4155  stream  TCP  nowait  bzruser  /usr/bin/bzr /usr/bin/bzr serve --inet --directory=/srv/bzr/repo
 
 クライアントコマンドを実行するとき、提供するURLは
 inetd.confに渡される ``--directory`` オプションに相対的な `bzr://` です::
 
     bzr log bzr://host/branchname
 
+可能であれば、 ``~`` や ``~user`` で始まるパスは ``bzr+ssh`` と同じように展開されますが、
+``bzr serve`` に指定された ``--directory`` オプションの外にあるホームディレクトリには
+アクセスできません。
+
 専用サーバー
 ~~~~~~~~~~~~~
 

=== modified file 'doc/ja/user-guide/setting_up_email.txt'
--- doc/ja/user-guide/setting_up_email.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-guide/setting_up_email.txt	2011-05-18 06:01:08 +0000
@@ -37,7 +37,7 @@
 
     % bzr whoami
     Joe Cool <joe@example.com>
-
+    
 'whoami'コマンドでEメールを設定する
 ------------------------------------
 
@@ -55,7 +55,8 @@
 --------------------------------------------
 
 デフォルトのiniファイルを使うには、 ``bazaar.conf`` ファイル
-(Linuxでは ``~/.bazaar/`` 、Windowsでは ``%APPDATA%\bazaar\2.0\`` )を作成し編集して下記で示すようにEメールアドレスを設定します。
+(Unix では ``~/.bazaar/`` 、Windowsでは ``%APPDATA%\bazaar\2.0\`` )を作成し編集して
+下記で示すようにEメールアドレスを設定します。
 DEFAULTは大文字と個別を区別して、大文字でなければならないことに注意をお願いします。
 ::
 
@@ -63,7 +64,8 @@
     email=Your Name <name@isp.com>
 
 
-iniファイルのフォーマットの詳細情報に関しては、Bazaarユーザーリファレンスの `構成設定 <../user-reference/bzr_man.html#id21>`_ を参照してください。
+iniファイルのフォーマットの詳細情報に関しては、Bazaarユーザーリファレンスの
+`構成設定 <../user-reference/index.html#configuration-settings>`_ を参照してください。
 
 ブランチ単位でEメールを設定する
 --------------------------------
@@ -88,6 +90,6 @@
 
 スパムの標的にされないようにEメールアドレスを共有したくない人達がいます。
 公開された場所で自分でブランチもしくはチェンジセットを公開しない限り、BazaarはけっしてEメールアドレスを露出しません。
-他の人があなたに作業内容に関して連絡できるように実際のアドレスを使うことを *強く* お勧めしますが、必須ではありません。
+他の人があなたに作業内容に関して連絡できるように実際のアドレスを使うことを *強く* お勧めしますが、必須ではありません。  
 メールアドレスは難読にしたり、宛先がわからず戻ってくるようにする、もしくは `spamgourmet.cm` のような
 アンチスパムサービスの検査をうけさせたりします。

=== modified file 'doc/ja/user-guide/shared_repository_layouts.txt'
--- doc/ja/user-guide/shared_repository_layouts.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/shared_repository_layouts.txt	2011-05-18 06:01:08 +0000
@@ -121,7 +121,7 @@
     メタプロジェクトはリポジトリのレイアウトにかかわらず複数のプロジェクトを集約します。
     1つのプロジェクトを ``bzr checkout`` すれば、必要なサブプロジェクトがすべて手に入ります。
 
-.. _NestedTreeSupport: http://bazaar-vcs.org/NestedTrees
+.. _NestedTreeSupport: http://wiki.bazaar.canonical.com/NestedTrees
 
 
 入れ子形式 (``project/branch/sub-branch/``)
@@ -239,7 +239,7 @@
 ``bzr`` コマンドを通してこれらを作るのは現時点では難しいですが、便利だと思う人がいれば変わるかもしれません。
 これは実際には `Launchpad`_ が ``bzr checkout https://launchpad.net/bzr`` をできるようにしている方法です。
 ``BranchReference`` は機能的にはシンボリックリンクですが、他のURLの参照ができます。
-相対パスによる参照ができるように拡張されれば、http、sftp、ローカルパスを通しても\
+相対パスによる参照ができるように拡張されれば、HTTP、SFTP、ローカルパスを通しても\
 動作するでしょう。
 
 .. _Launchpad: https://launchpad.net

=== modified file 'doc/ja/user-guide/specifying_revisions.txt'
--- doc/ja/user-guide/specifying_revisions.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/specifying_revisions.txt	2011-05-18 06:01:08 +0000
@@ -35,34 +35,40 @@
 リビジョン、もしくは範囲の境界、は
 下記に示される異なるフォーマットを利用して渡すことができます。
 
- +----------------------+--------------------------------------------+
- |  引数の型            | 説明                                       |
- +----------------------+--------------------------------------------+
- | *number*             | リビジョン番号                             |
- +----------------------+--------------------------------------------+
- | **revno**:*number*   | 正のリビジョン番号                         |
- +----------------------+--------------------------------------------+
- | **last**:*number*    | 負のリビジョン番号                         |
- +----------------------+--------------------------------------------+
- | **revid**:*guid*     | グローバルでユニークなリビジョンID         |
- +----------------------+--------------------------------------------+
- | **before**:*rev*     | ''rev''の左端の親                          |
- +----------------------+--------------------------------------------+
- | **date**:*value*     | 渡された日付の後の最初のエントリ           |
- +----------------------+--------------------------------------------+
- | **tag**:*value*      | 渡されたタグにマッチするリビジョン         |
- +----------------------+--------------------------------------------+
- | **ancestor**:*path*  | ブランチからのマージされた最新のリビジョン |
- +----------------------+--------------------------------------------+
- | **branch**:*path*    | 別のブランチの最新リビジョン               |
- +----------------------+--------------------------------------------+
- | **submit**:*path*    | 投稿ブランチの共通の祖先                   |
- +----------------------+--------------------------------------------+
+ +----------------------------+--------------------------------------------+
+ |  引数の型                  | 説明                                       |
+ +----------------------------+--------------------------------------------+
+ | *number*                   | リビジョン番号                             |
+ +----------------------------+--------------------------------------------+
+ | **revno**:*number*         | リビジョン番号                             |
+ +----------------------------+--------------------------------------------+
+ | **last**:*number*          | 負のリビジョン番号                         |
+ +----------------------------+--------------------------------------------+
+ | *guid*                     | グローバルでユニークなリビジョンID         |
+ +----------------------------+--------------------------------------------+
+ | **revid**:*guid*           | グローバルでユニークなリビジョンID         |
+ +----------------------------+--------------------------------------------+
+ | **before**:*rev*           | ''rev''の左端の親                          |
+ +----------------------------+--------------------------------------------+
+ | *date-value*               | 渡された日付の後の最初のエントリ           |
+ +----------------------------+--------------------------------------------+
+ | **date**:*date-value*      | 渡された日付の後の最初のエントリ           |
+ +----------------------------+--------------------------------------------+
+ | *tag-name*                 | 渡されたタグにマッチするリビジョン         |
+ +----------------------------+--------------------------------------------+
+ | **tag**:*tag-name*         | 渡されたタグにマッチするリビジョン         |
+ +----------------------------+--------------------------------------------+
+ | **ancestor**:*path*        | ブランチからのマージされた最新のリビジョン |
+ +----------------------------+--------------------------------------------+
+ | **branch**:*path*          | 別のブランチの最新リビジョン               |
+ +----------------------------+--------------------------------------------+
+ | **submit**:*path*          | 投稿ブランチの共通の祖先                   |
+ +----------------------------+--------------------------------------------+
 
 これらのフォーマットの手短な紹介は下記のとおりです。
 完全な詳細内容に関しては、 Bazaarユーザーリファレンスの `リビジョンの識別子`_ を参照してください。
 
-.. _リビジョンの識別子: ../user-reference/bzr_man.html#revision-identifiers
+.. _リビジョンの識別子: ../user-reference/index.html#revision-identifiers
 
 番号
 ~~~~~
@@ -82,7 +88,8 @@
 revid
 ~~~~~
 
-**revid** は ``bzr log`` と他のコマンドによって示される内部のリビジョンIDの指定を可能にします。
+**revid** は ``bzr log --show-ids`` や他のコマンドによって示される内部の
+リビジョンIDの指定を可能にします。
 
 例です::
 

=== modified file 'doc/ja/user-guide/stacked.txt'
--- doc/ja/user-guide/stacked.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/stacked.txt	2011-05-18 06:27:34 +0000
@@ -1,28 +1,43 @@
 スタックブランチを利用する
 ==========================
 
+.. _motivation:
+
+ユースケース
+--------------
+
+あるプロジェクトで作業しようとしていて、公開されているリポジトリに対して
+読み込みアクセスはできるものの書き込みができないとしましょう。
+公開されているリポジトリと同じホストで自分のブランチを公開したりバックアップ
+したりする場合、スタックブランチを使うことができるかもしれません。
+
+スタックブランチの他のユースケースとしては、実験的なブランチと、コード
+ホスティングサイトが挙げられます。これらのシナリオではスタックブランチの
+特性がぴったりあいます。
+
+
 スタックブランチとは？
 -------------------------
 
-スタックブランチ(stacked branch)は別のブランチのリビジョンを見つける方法を知っています。
-スタックブランチはユニークなリビジョンのみを保存することで、速く作れたり\
-効率的に保存したりします。
+スタックブランチ(stacked branch)は別の(スタック先)ブランチのリビジョンを
+見つける方法を知っています。
+スタックブランチはスタック先ブランチには存在しないユニークなリビジョンのみを
+保存することで、ブランチの作成を高速にしたり、ディスク利用効率を向上します。
 これらの観点から、スタックブランチは共用リポジトリと似ています。
 しかしながら、スタックブランチは追加の利点があります:
 
 * 新しいブランチはスタックされたブランチとは完全に異なる位置に設置できます。
 
 * スタックブランチを削除すれば(共用リポジトリだと残ってしまう)
-  本当のリビジョンも削除されます
+  リビジョンの情報も削除されます
 
 * セキュリティは共用リポジトリよりも向上しています。
-  スタックドオンリポジトリはスタックブランチにコミットする開発者に対して物理的にリードオンリーだからです。
+  スタック先のリポジトリはスタックブランチにコミットする開発者に対して
+  物理的にリードオンリーだからです。
 
-これらの恩恵によって実験ブランチとコードホスティングサイトを含めたさまざまな\
-シナリオに対してスタックブランチは理想的なものになります。
 
 スタックブランチを作成する
--------------------------
+--------------------------
 
 スタックブランチを作成するには、branchコマンドの ``stacked`` オプションを使用します。
 例です::
@@ -58,12 +73,17 @@
 
   bzr push --stacked-on reference-url my-url
 
-このコマンドによって ``reference-url`` にスタックされた ``my-url`` で新しいブランチが作成され
-``reference-url`` でまだブランチにない現在のブランチのリビジョンのみを格納します。
+このコマンドでは、 ``reference-url`` にスタックした新しいブランチを ``my-url``
+に作成し、 ``reference-url`` には無いリビジョンだけをそこに格納します。
+``my-url`` は ``reference-url`` と同じホストでも構いません。
+
+.. The following text is hidden because bug 375013 breaks the example.
+   When bug 375013 is fixed, we should unhide this text.
+      - Andrew Bennetts, 10 March 2010
 
 ローカルブランチがスタックブランチとして作成された場合、
 ``push`` するには ``--stacked`` オプションを使うことが可能で
-*stacked-on* な位置が明確になります。例です::
+スタック先の位置を省略できます。::
 
   bzr branch --stacked source-url my-dir
   cd my-dir
@@ -71,15 +91,21 @@
   bzr commit -m "fix bug"
   bzr push --stacked
 
+この使い方は、上述したユースケースにマッチしています。
+
 
 スタックブランチの制限
 ----------------------
 
 スタックブランチに関して覚えておくべき大事なことは、ほとんど全ての\
-オペレーションでスタックドオンブランチが必要になることです。
+オペレーションでスタック先ブランチが必要になることです。
 これは両方のブランチがローカルもしくは同じサーバーにあるときは\
 問題にはなりません。
 
+また、ほとんどの履歴がスタック先リポジトリに格納されているので、スタック先
+リポジトリへのアクセスがネットワーク経由だった場合に ``bzr log``
+のようなコマンドが遅くなるかもしれません。
+ 
 スタックするブランチを変更する
 -------------------------------
 

=== modified file 'doc/ja/user-guide/svn_plugin.txt'
--- doc/ja/user-guide/svn_plugin.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/svn_plugin.txt	2011-05-18 06:01:08 +0000
@@ -18,7 +18,7 @@
 利用からまだ採用していないチームのための共通の暫定ステップです
 
 インストールの手引きに関しては、bzr-svnのホームページをご覧ください:
-http://bazaar-vcs.org/BzrForeignBranches/Subversion.
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion
 
 
 シンプルな例
@@ -28,14 +28,10 @@
 最初に、ブランチの保存用のローカルな共用リポジトリをセットアップして\
 トランクをチェックアウトします::
 
-  bzr init-repo --rich-root-pack beagle-repo
+  bzr init-repo beagle-repo
   cd beagle-repo
   bzr checkout svn+ssh://svn.gnome.org/svn/beagle/trunk beagle-trunk
 
-
-``init-repo`` に対して ``rich-root-pack`` オプションを使うことが重要です。
-bzr-svnは追加のメタデータがいくつか必要ですが、Bazaarのデフォルトのリポジトリフォーマットではまだサポートされないからです。
-(訳注: Bazaar 2.0以降を使っているのであれば、 --rich-root-pack オプションは不要です。)
 次に、フィーチャブランチを作成してハックします::
 
   bzr branch beagle-trunk beagle-feature1
@@ -70,7 +66,7 @@
 
 中央のBazaarミラーを利用するために更新された上記からのレシピです::
 
-  bzr init-repo --rich-root-pack beagle-repo
+  bzr init-repo beagle-repo
   cd beagle-repo
   bzr branch bzr+ssh://bzr.gnome.org/beagle.bzr/trunk beagle-trunk
   bzr branch beagle-trunk beagle-feature1
@@ -108,5 +104,5 @@
 
  * Bazaarはファイルのコピーのトラッキングをサポートしません
 
-現在の制約の一覧に関しては、bzr-svnのウェブページ、\
-http://bazaar-vcs.org/BzrForeignBranches/Subversion を参照してください。
+現在の制約の一覧に関しては、bzr-svnのウェブページ、
+http://wiki.bazaar.canonical.com/BzrForeignBranches/Subversion を参照してください。

=== modified file 'doc/ja/user-guide/undoing_mistakes.txt'
--- doc/ja/user-guide/undoing_mistakes.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/undoing_mistakes.txt	2011-05-18 06:01:08 +0000
@@ -4,7 +4,7 @@
 間違いは起きる
 ---------------
 
-Bazaarは下記で説明されるような間違いから簡単にリカバーできるように\
+Bazaarは下記で説明されるような間違いから簡単にリカバーできるように
 設計されています。
 
 プロジェクトのリビジョンの履歴をドロップする
@@ -118,10 +118,10 @@
   bzr commit -m "Backout fix for bug #5"
 
 この作業によってツリー全体をリビジョン19の状態に変更します。
-それ以降は新しいコミットをしていなければおそらくこれがあなたのしたい唯一のことです。
-そうであれば、 ``revert`` はこれらをきれいに消去します。
-この場合、わるい修正を取り消す代わりに
-`リバースチェリーピッキング <advanced_merging.html#reverse-cherrypicking>`_
+それ以降は新しいコミットをしていなければ、おそらくこれがあなたのしたい唯一のことです。
+新しいコミットをしている場合は、 ``revert`` はそれも消してしまいます。
+その場合は、わるい修正を取り消す代わりに
+`リバースチェリーピック <adv_merging.html#reverse-cherrypicking>`_
 を使う方がよいでしょう。
 
 注: 19のような絶対的なリビジョン番号の代わりに、

=== modified file 'doc/ja/user-guide/using_checkouts.txt'
--- doc/ja/user-guide/using_checkouts.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/using_checkouts.txt	2011-05-18 06:01:08 +0000
@@ -7,7 +7,7 @@
 ローカルのブランチを作りチェックアウトに変更したいのであれば、
 ``bind`` コマンドを使います::
 
-  bzr bind sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr bind bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 たとえば以前のセクションで説明されたように ``push`` を使って集中型の\
 ブランチを作った後でこれは必要です。
@@ -57,7 +57,7 @@
 Bazaarで履歴のないチェックアウトを入手するには、
 ``--lightweight`` オプションを使います::
 
-  bzr checkout --lightweight sftp://centralhost/srv/bzr/PROJECT/trunk
+  bzr checkout --lightweight bzr+ssh://centralhost/srv/bzr/PROJECT/trunk
 
 もちろん、これによって通常のチェックアウトの多くの利点は失われますが、
 役に立つ場合と時を選ぶトレードオフです。

=== modified file 'doc/ja/user-guide/version_info.txt'
--- doc/ja/user-guide/version_info.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-guide/version_info.txt	2011-05-18 06:01:08 +0000
@@ -23,7 +23,7 @@
   branch-nick: bzr.dev
 
 オペレーティングシステムツールもしくはスクリプトを使用して出力を簡単にフィルタリングできます。
-例です(Linux/Unix)::
+例です::
 
   $ bzr version-info | grep ^date
   date: 2007-12-11 17:51:18 +0000
@@ -56,7 +56,7 @@
 
   * `file_revisions`: プロジェクトのすべてのファイルに対する最終修正のリビジョンのリストを示すディクショナリ。
     これは ``$Id$`` キーワードがCVSで管理されたファイルと同じように使われます。
-    最終修正の日付は ``revisions`` マップで探すことで決定されます。
+    最終修正の日付は ``revisions`` マップで探すことで決定されます。 
     デフォルトではこれは空で、 ``--all`` もしくは ``--include-file-revisions`` によってのみ有効になります
 
 
@@ -74,11 +74,11 @@
       > version_info.h
 
 ``{revno}`` は作業ツリーのリビジョン番号に置き換えされます。
-(上記の例があなたのOSで動作しない場合、一行ですべてのコマンドを入力してみてください)
+(上記の例があなたのOSで動作しない場合、一行ですべてのコマンドを入力してみてください) 
 テンプレートの中で利用できる変数の詳細な情報に関しては、
 Bazaarのユーザーリファレンスの `Version Info`_ を参照してください。
 
-.. _Version Info: ../user-reference/bzr_man.html#version-info
+.. _Version Info: ../user-reference/index.html#version-info
 
 特定の言語でバージョン情報をダンプするために予め定義されるフォーマットはは現在開発段階にあります。
 この領域の要求に関してはメーリングリストで私達開発者に連絡して下さるようお願いします。
@@ -93,5 +93,5 @@
 ``version_info`` ``clean`` を設定します。
 同様に ``modified`` が適切である場合に ``file_revisions`` でエントリを設定します。
 
-..
+.. 
    vim: tw=74 ft=rst spell spelllang=en_us

=== modified file 'doc/ja/user-guide/web_browsing.txt'
--- doc/ja/user-guide/web_browsing.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/web_browsing.txt	2011-05-18 06:01:08 +0000
@@ -9,7 +9,8 @@
 Loggerhead のホームページは https://launchpad.net/loggerhead にあります。
 
 ダウンロードリンクを含めてこれらのパッケージの最新情報に関しては
-http://bazaar-vcs.org/WebInterface を参照してください。
+http://wiki.bazaar.canonical.com/WebInterface
+を参照してください。
 
 注: プロジェクトがホストされているもしくはLaunchpadでミラーリングされている場合、
 Loggerheadのコードブラウジング機能はサービスの一部として提供されます。

=== modified file 'doc/ja/user-guide/writing_a_plugin.txt'
--- doc/ja/user-guide/writing_a_plugin.txt	2009-10-29 10:01:19 +0000
+++ doc/ja/user-guide/writing_a_plugin.txt	2011-05-18 06:01:08 +0000
@@ -27,6 +27,16 @@
 ``bzrlib.commands.register_command(cmd_foo)`` でコマンドを登録しなければなりません。
 さもなければbzrはコマンドを見つけることはありません。
 
+.. _installing-a-hook:
+
+フックをインストールする
+-------------------------
+
+`Using hooks`_ を参照してください。
+
+.. _Using hooks: hooks.txt
+
+
 .. _specifying-a-plugin-version-number:
 
 プラグインのバージョン番号を指定する
@@ -41,8 +51,13 @@
 プラグインの検索ルール
 ------------------------
 
-デフォルトではbzrはプラグインを見つけるために ``bzrlib/plugins`` と ``~/.bazaar/plugins`` をスキャンします。
+デフォルトではbzrはプラグインを見つけるために ``~/.bazaar/plugins`` と
+``bzrlib/plugins`` をスキャンします。
 ``BZR_PLUGIN_PATH`` でこれを上書きできます。
+(詳細は、
+`ユーザーリファレンス <../user-reference/configuration-help.html#bzr-plugin-path>`_
+を参照してください。)
+
 プラグインはモジュールもしくはパッケージの形態をとることができます。
 プラグインが単独のファイルであれば、構造をモジュールにできます。
 プラグインが複数のファイルを持つ場合やbzrのブランチとして配布したい場合は、

=== modified file 'doc/ja/user-reference/index.txt'
--- doc/ja/user-reference/index.txt	2009-12-02 20:34:07 +0000
+++ doc/ja/user-reference/index.txt	2011-05-18 06:01:08 +0000
@@ -50,7 +50,7 @@
 ブランチ
 =========
 
-ブランチは、すべての履歴を含む、プロジェクトの状態で構成されます。
+ブランチは、すべての履歴を含む、プロジェクトの状態で構成されます。 
 すべてのブランチは関連づけされたリポジトリ(ブランチの履歴が保存される場所)を持ちますが、
 複数のブランチは同じリポジトリを共有することもあります(共用リポジトリ)。
 ブランチはコピーしたりマージしたりできます。
@@ -174,7 +174,7 @@
 * さもなければ、デフォルトのフォーマットを使用します。大抵のプロジェクトはこれで十分です。
 
 (ディストロのパッケージを利用しているなどで)最新のBazaarを利用できない開発者がいるのであれば、
-それに応じてガイドラインを調整してください。
+それに応じてガイドラインを調整してください。 
 たとえば、プロジェクトがBazaar 1.7を標準化している場合、1.9の代わりに1.6を選ぶことが必要です。
 
 注: 現在サポートされるフォーマットの多くは2つのバリアントを持ちます:
@@ -308,7 +308,7 @@
 
 チェックアウト、ツリーもしくはブランチを軽量ブランチに再設定するとき、
 ローカルのブランチを破壊しなければなりません。
-(チェックアウトに関して、これはキャッシュとして最初に提供するローカルブランチです。)
+(チェックアウトに関して、これはキャッシュとして最初に提供するローカルブランチです。)  
 破壊されるブランチが同じ最終リビジョンを持たなければ、
 軽量用チェックアウト用の新しい参照ブランチ、データが失われる可能性があるので、
 Bazaarは拒否します。
@@ -342,8 +342,8 @@
 ブランチから 'bzr checkout .' を実行すると作業ツリーが作成されます。
 リモートでブランチが更新されると、そのディレクトリの中で'bzr update'を実行することで作業ツリーを更新できます。
 
-望まない作業ツリーを持つブランチがある場合、安全であれば'remove-tree'コマンドはツリーを除外します。
-ブランチにプッシュするとき更新されないリモート作業ツリーに関する警告を回避することでこれは可能です。
+望まない作業ツリーを持つブランチがある場合、安全であれば'remove-tree'コマンドはツリーを除外します。 
+ブランチにプッシュするとき更新されないリモート作業ツリーに関する警告を回避することでこれは可能です。 
 これは'--no-trees'リポジトリ('bzr help repositories'を参照)に取り組むときにも便利です。
 
 プッシュするリモートマシン上で作業ブランチを持ちたい場合、
@@ -406,7 +406,7 @@
   host=host.com
   user=joe
   password=secret
-
+  
 
 認証の定義
 -----------
@@ -423,7 +423,7 @@
 2. ユーザー、領域とパスワード
 
 ホストに対して認証するために ``HTTP`` と ``HTTPS`` は(``user, realm, password``)を必要とします。
-``.htaccess`` ファイルを利用することで、たとえば、任意の ``host``
+``.htaccess`` ファイルを利用することで、たとえば、任意の ``host`` 
 に対して (``user, realm, password``) をいくつか定義することが可能です。
 ですので本当に必要なのは (``user``, ``password``, ``host``, ``path``)です。
 ``realm`` は定義で考慮されませんが、bzrにパスワードを催促される場合表示されます。
@@ -501,7 +501,7 @@
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 すべての接続は同じ ``user`` で行われ
-(デフォルトのbzrのものが適切でない場合のためのリモートの接続)
+(デフォルトのbzrのものが適切でない場合のためのリモートの接続) 
 パスワードはいくつかの例外とともに常に催促されます::
 
         # hobby.netのPetプロジェクト
@@ -509,14 +509,14 @@
         host=r.hobby.net
         user=jim
         password=obvious1234
-
+        
         # ホームサーバー
         [home]
         scheme=https
         host=home.net
         user=joe
         password=1essobV10us
-
+        
         [DEFAULT]
         # ローカルユーザーがbarbazで、すべてのリモートサイト上ではfoobarとして
         user=foobar
@@ -556,7 +556,7 @@
         host=dev.company.com
         path=/dev/integration
         user=user2
-
+        
         # プロキシ
         [proxy]
         scheme=http
@@ -644,6 +644,8 @@
 上記はCPANのRTバグトラッカー用です。
 
 
+.. _configuration-settings:
+
 構成設定
 =========
 
@@ -827,7 +829,7 @@
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 ``locations.conf`` によって特定のブランチ用に設定を上書きできます。
-フォーマットは1つの重大な変更を伴うbazaar.confのデフォルトセクションに対してほとんど理想的です:
+フォーマットは1つの重大な変更を伴うbazaar.confのデフォルトセクションに対してほとんど理想的です: 
 デフォルトを記述する代わりに、セクションヘッダーは値を上書きしたいブランチへのパスになります。
 ワイルドカードの '?' と '*' がサポートされます::
 
@@ -963,7 +965,7 @@
 
 :default: 上記を参照。
 :editor: マージリクエストを書くエディタを使用します。
-    これはコミットID( ``bzr whoami`` を参照),
+    これはコミットID( ``bzr whoami`` を参照), 
     smtp_serverと(オプションで)smtp_usernameとsmtp_passwordも使用します。
 :mapi: Windowsで好きなメールクライアントを使用します。
 :xdg-email: 好きなメールプラグラムを実行するためにxdg-emailを使用する
@@ -1180,7 +1182,7 @@
   $ bzr merge ../other
 
 この状況において、Bazaarは移動をキャンセルして、"a"を"b"の中に残しておきます。
-望むのであればディレクトリをリネームできれば
+望むのであればディレクトリをリネームできれば 
 衝突を解消したものとしてマークするために "bzr resolve FILE" を実行します。
 
 ディレクトリではない親
@@ -1219,7 +1221,7 @@
 ============================
 
 :pack-0.92:
-    (ネイティブ) (デフォルト) 0.92で新しく導入:
+    (ネイティブ) (デフォルト) 0.92で新しく導入: 
     dirstate-tagsフォーマットリポジトリと互換性のあるデータを持つパックベースのフォーマット。
     0.92以前のbzrリポジトリと相互運用できますが0.92以前のbzrでは読めません。
     以前はknitpack-experimentalと呼ばれていました。詳細な情報に関しては、
@@ -1596,7 +1598,7 @@
     (ネイティブ) 現在の開発フォーマット。
     pack-0.92 (とpack-0.92と互換性のある)フォーマットリポジトリにデータを変換できます。
     このフォーマットのリポジトリとブランチはbzr.devによってのみ読み込みできます。
-    使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html
+    使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html 
     をご覧下さるようお願いします。
 
 
@@ -1605,7 +1607,7 @@
     pack-0.92-subtree (pack-0.92-subtreeと互換性のあるものなら何でも)フォーマットリポジトリ
     から/へのデータの変換ができる。このフォーマットのリポジトリとブランチは
     bzr.devによってのみ読み込みできます。
-    使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html
+    使用する前に http://doc.bazaar-vcs.org/latest/developers/development-repo.html 
     をご覧下さるようお願いします。
 
 
@@ -1644,7 +1646,7 @@
 
 'REV1' と 'REV2' がリビジョンの識別子である場合、
 'REV1..REV2' はリビジョンの範囲を表示します。
-例: '3647..3649' 、 'date:yesterday..-1' と 'branch:/path/to/branch1/..branch:/branch2'
+例: '3647..3649' 、 'date:yesterday..-1' と 'branch:/path/to/branch1/..branch:/branch2' 
 ('..'周辺にはクォートもしくはスペースが存在しません)。
 
 範囲は異なるコマンドによって異なる解釈がなされます。
@@ -1777,7 +1779,7 @@
     投稿ブランチはbundleとmergeコマンドによって使用されます。
     投稿ブランチが指定されなければ、親ブランチが代わりに使用されます。
 
-    共通の祖先は両方のブランチ内に存在する最終リビジョンです。
+    共通の祖先は両方のブランチ内に存在する最終リビジョンです。 
     通常これはブランチポイントですが、マージされたリビジョンにもなり得ます。
 
     例::
@@ -1789,7 +1791,7 @@
 ===============
 
 標準オプションはすべてのコマンドで有効です。
-
+      
 --help, -h     ヘルプメッセージを表示する。
 --verbose, -v  詳細な情報を表示する。
 --quiet, -q    エラーと警告のみ表示する。
@@ -1876,7 +1878,7 @@
   non-recursiveモードでは、以前無視されたファイルであっても、
   名前つきのすべてのアイテムが追加されます。
   名前つきファイルがすでにバージョン管理されている場合は警告が表示されます。
-
+  
   recursiveモード(デフォルト)では、ファイルは同じ方法で扱われますが、
   ディレクトリに対するふるまいは異なります。
   すでにバージョン管理されているディレクトリであれば警告は表示されません。
@@ -1885,16 +1887,16 @@
   これらは追加されます。
   この検索はバージョン管理されたディレクトリにも再帰的に進められます。
   名前が渡されなければ、'.'が想定されます。
-
+  
   それゆえ単に 'bzr add' を実行すると現在未知であるファイルのすべてがバージョン管理されます。
-
+  
   親ディレクトリがバージョン管理されていないファイルを追加すると
   親およびそのrootまでも暗黙の内に追加されます。
   このことが意味するのはディレクトリを明示的に追加する必要はなく、
   ディレクトリの中のファイルを1つ追加するときにそれらのディレクトリも追加されます。
-
+  
   --dry-runは追加されるファイルを表示しますが、それらを実際に追加しません。
-
+  
   --file-ids-fromは提供されたパスからファイルのidの使用を試みます。
   これは同じファイル名を持ちマッチするディレクトリの発見をするためにidを探し、また純粋なパスによって行います。
   このオプションが必要になるのはめったにありませんが、後でマージする2つのブランチに同じ論理ファイルを追加する
@@ -1952,7 +1954,7 @@
 :説明:
   このコマンドは与えられたファイルの、変更によって導入されたリビジョン、
   筆者と日付を示す注釈を左側で表示します。
-
+  
   連続した行の続きに関してoriginが同じ場合、
   --allオプションが渡されない限り、これはトップでのみ表示されます。
 
@@ -2004,7 +2006,7 @@
   冒頭のスキームもしくはドライブの識別子を剥ぎ取ることで、
   FROM_LOCATIONからTO_LOCATIONが得られます。
   たとえば、"branch lp:foo-bar"は./foo-barを作成しようとします。
-
+  
   ブランチの特定のリビジョンを読み取るには、
   "branch foo/bar -r 5"のような、--revisionパラメータを提供します。
 
@@ -2024,7 +2026,7 @@
 
 :説明:
   警告: ロックを保持するプロセスが停止したときにのみロックはブレークします。
-
+  
   'bzr info'コマンドを通して開いているロックに関する情報を得ることができます。
 
 :例:
@@ -2047,7 +2049,7 @@
 
 :説明:
   リビジョンが指定されなければ、最後のリビジョンが使用されます。
-
+  
   注意: バイナリファイルでこのコマンドを使用する際には
         標準出力をリダイレクトするように気をつけてください。
 
@@ -2070,10 +2072,10 @@
 :説明:
   このコマンドはデータの破損もしくはbzrのバグを検出するために
   ブランチとリポジトリストレージに関するさまざまな不変量をチェックします・
-
+  
   問題が検出された場合のみ作業ツリーとブランチのチェックは出力をします。
   リポジトリチェックの出力フィールドは次のとおりです:
-
+  
       revisions: これはチェックされるリビジョンの番号です。
           これは問題を示しません。
       versionedfiles: これはチェックされるバージョン管理されたファイルの数です。
@@ -2088,7 +2090,7 @@
           テキストのエントリが修正されたときにテキストは繰り返しできますが、
           ファイルの内容は繰り返しできません。
           これは問題を示しません。
-
+  
   制限が指定されなければ、すべてのBazaarデータはチェックされる位置で見つかります。
 
 :例:
@@ -2131,16 +2133,16 @@
   チェックアウトは作業ツリーを再構成します。
   作業ツリーを削除した場合もしくはこれがけっして作成されない場合
   - すなわち、SFTPを使用してブランチを現在の位置にpushする場合、これは便利です。
-
+  
   TO_LOCATIONが省略されると、BRANCH_LOCATIONの最後のコンポーネントが使用されます。
   言い換えると、"checkout ../foo/bar" は./barを作成しようとします。
   BRANCH_LOCATIONが has no /を持たないもしくはパスの区切り文字が埋め込まれている場合、
   先頭のスキームもしくはドライブの識別子を除去することでBRANCH_LOCATIONからTO_LOCATIONが
   得られます。たとえば、"checkout lp:foo-bar"は./foo-barを作成しようとします。
-
+  
   特定のリビジョンのブランチを読み取るには、
-  "checkout foo/bar -r 5"のように--revisionパラメータを提供します。
-  これはすぐに時代遅れになりますが[なのでコミットできない]
+  "checkout foo/bar -r 5"のように--revisionパラメータを提供します。 
+  これはすぐに時代遅れになりますが[なのでコミットできない] 
   役に立つことがあります(すなわち古いコードを検査するため)。
 
 :エイリアス:  co
@@ -2175,22 +2177,22 @@
 
 :説明:
   引数が渡されなければ、ツリー全体がコミットされます。
-
+  
   選択されたファイルが指定されると、これらのファイルへの変更のみがコミットされます。
   ディレクトリが指定されるとディレクトリとその中のすべてがコミットされます。
-
+  
   excludesが渡されるとき、これらは選択されたファイルよりも優先されます。
   たとえば、fooの範囲での変更のみコミットされますが、foo/barの範囲の変更はコミットされません::
-
+  
     bzr commit foo -x foo/bar
-
+  
   変更の著者がコミッターと同じ人物でなければ、
   --authorオプションを使用して著者の名前を指定できます。
   名前はコミッターidと同じフォーマットです。たとえば"John Doe <jdoe@example.com>"です。
-
+  
   コミットされるツリーが無効である場合選択されたファイルのコミットが失敗することがあります。
   次の一連のコマンドを考えてみましょう::
-
+  
     bzr init foo
     mkdir foo/bar
     bzr add foo/bar
@@ -2199,12 +2201,12 @@
     mkdir foo/bar
     bzr add foo/bar
     bzr commit foo/bar -m "committing bar but not baz"
-
+  
   上記の例において、最終コミットは故意に失敗します。
   これによってユーザーは、同時に、最初に個別に、もしくはまったく
   リネームしたくないかどうかを決める機会を得ます。
   (一般的なルールとして、判断がつかないとき、Bazaarは安全に物事を行う方針を持ちます。)
-
+  
   注: マージの後の選択されたファイルのコミットはまだサポートされていません。
 
 :エイリアス:  ci, checkin
@@ -2227,11 +2229,11 @@
   人間だけが修正できるある種の問題が存在します。
   この問題に遭遇するとき、衝突がマークされます。
   衝突はコミットする前に何かを修正する必要があることを意味します。
-
+  
   通常の衝突は短く人間が読めるメッセージとして一覧が示されます。
   --textが提供される場合、代わりに、テキストの衝突を持つファイルのパスの一覧が表示されます。
   (これはテキストの衝突を持つファイルをすべて編集するために便利です。)
-
+  
   問題を修正したらbzr resolveを使用します。
 
 :関連コマンド: `resolve`_
@@ -2278,7 +2280,7 @@
   リモートと複数のブランチは--oldと--newオプションを使用して比較できます。
   これらのオプションが提供されなければ、両方のデフォルトは、存在すれば最初の引数、
   引数が渡されなければ現在のツリーから得られます。
-
+  
   "bzr diff -p1" は "bzr diff --prefix old/:new/"と同等で、
   "patch -p1"に最適なパッチが生成されます。
 
@@ -2346,19 +2348,19 @@
 
 :説明:
   リビジョンが指定されなければこれは最終コミットのリビジョンをエクスポートします。
-
+  
   フォーマットはtar、tgz、tbz2のような"exporter"の名前になることができます。
   何も渡されなければ、拡張子かｒフォーマットを見つけようとします。
   拡張子が見つからなければディレクトリにエクスポートします(--format=dirと同等)。
-
+  
   rootが提供されると、これはコンテナフォーマット(tar、zip、その他)内部のrootディレクトリとして使用されます。
   これが提供されなければエクスポートされるファイル名へのデフォルトになります。
   rootオプションは'dir'フォーマットに対して効果がありません。
-
+  
   ブランチが省略されると現在の作業ディレクトリを含むブランチが使用されます。
-
+  
   注: ASCIIではないファイル名でのツリーのエクスポートはサポートされません。
-
+  
     ==========================       =========================
     サポートされるフォーマット        拡張子で自動検出
     ==========================       =========================
@@ -2399,11 +2401,11 @@
 
 :説明:
   パターンの構文の詳細は ``bzr help patterns`` を参照。
-
+  
   無視リストからパターンを除外するには、.bzrignoreファイルを編集します。
   追加した後で、コマンドを使用して間接的に、もしくはエディタを使用して
   直接そのファイルを編集もしくは削除した後で、そのコミットを確認してください。
-
+  
   注: シェルのワイルドカードを含む無視パターンはUnixのシェルからクォートされなければなりません。
 
 :例:
@@ -2442,9 +2444,9 @@
 
 :説明:
   無視されるファイルと無視パターンのすべての一覧を表示します。
-
+  
   代わりにファイルの一覧だけを表示するには::
-
+  
       bzr ls --ignored
 
 :関連コマンド: `ignore`_, `ls`_
@@ -2463,7 +2465,7 @@
 :説明:
   このコマンドはツリー、ブランチもしくはリポジトリに関連する既知の位置とフォーマットのすべてを表示します。
   統計情報はそれぞれのレポートに含まれます。
-
+  
   ブランチと作業ツリーは見つからないリビジョンもレポートします。
 
 :関連項目: `リポジトリ`_, `revno`_, `作業ツリー`_
@@ -2514,8 +2516,8 @@
                         repo.html をご覧いただくようお願いします。
     --dirstate          0.15の新しいフォーマット: 速いローカルオペレーション。
                         ネットワークを通したアクセスのときbzr 0.8とそれ以降と互換性がある。
-    --dirstate-tags     0.15の新しいフォーマット: 速いローカルオペレーションで
-                        ネットワークオペレーションに関するスケーリングを改善。
+    --dirstate-tags     0.15の新しいフォーマット: 速いローカルオペレーションで 
+                        ネットワークオペレーションに関するスケーリングを改善。 
                         タグのサポートを追加。bzr < 0.15とは互換性がない。
     --knit              knitsを使用するフォーマット。bzr <= 0.14との相互運用に推奨。
     --metaweave         0.8での暫定フォーマット。knitよりも遅い。
@@ -2534,15 +2536,15 @@
 
 :説明:
   空のブランチを作成するもしくは既存のプロジェクトをインポートする前に使用します。
-
+  
   親ディレクトリの位置にリポジトリが存在する場合、
   ブランチの履歴はリポジトリに保存されます。
   さもなければinitは.bzrのディレクトリに独自の履歴を運ぶスタンドアロンのブランチを作成します。
-
+  
   ロケーションにブランチがすでにあるが作業ツリーがない場合、ツリーは'bzr checkout'で投入できます。
-
+  
   ファイルのツリーをインポートする方法のレシピです::
-
+  
       cd ~/project
       bzr init
       bzr add .
@@ -2594,8 +2596,8 @@
                         repo.html をご覧いただくようお願いします。
     --dirstate          0.15の新しいフォーマット: 速いローカルオペレーション。
                         ネットワークを通したアクセスのときbzr 0.8とそれ以降と互換性がある。
-    --dirstate-tags     0.15の新しいフォーマット: 速いローカルオペレーションで
-                        ネットワークオペレーションに関するスケーリングを改善。
+    --dirstate-tags     0.15の新しいフォーマット: 速いローカルオペレーションで 
+                        ネットワークオペレーションに関するスケーリングを改善。 
                         タグのサポートを追加。bzr < 0.15とは互換性がない。
     --knit              knitsを使用するフォーマット。bzr <= 0.14との相互運用に推奨。
     --metaweave         0.8での暫定フォーマット。knitよりも遅い。
@@ -2615,7 +2617,7 @@
 :説明:
   ブランチのディレクトリではなく、リポジトリにリビジョンを保存する
   リポジトリディレクトリの元で作成された新しいブランチ。
-
+  
   --no-treesオプションが使用されるとリポジトリのブランチはデフォルトで作業ツリーを持ちません。
 
 :例:
@@ -2661,7 +2663,7 @@
 
 :説明:
   デフォルトでは作業ディレクトリを含むブランチのログを表示する。
-
+  
   ログの範囲をリクエストするには、-r begin..endコマンドを使用できます。
   -r revisionは指定したリビジョンをリクエストし、
   -r ..end もしくは -r begin.. も有効です。
@@ -2742,35 +2744,35 @@
   mergeディレクティブを含むファイルへのパスの形式で指定できます。
   どちらも指定されなければ、デフォルトは上流ブランチもしくは
   --rememberを使用して最近マージされたブランチです。
-
+  
   マージをブランチするとき、デフォルトではチップがマージされます。
   異なるリビジョンをピックするには、--revisionを渡します。
   2つの値を指定する場合、最初の値はBASEとして、2番目の値はOTHERとして使用されます。
   個別のリビジョン、もしくはこのように利用可能なリビジョンのサブセットをマージすることは
   一般的に"チェリーピック"として言及されます。
-
+  
   リビジョン番号はマージされるブランチに対して常に相対的です。
-
+  
   デフォルトでは、bzrは、自動的に適切なベースを検出して、
   他のブランチからすべてのネットワークでマージしようとします。
   これが失敗すると、明示的なベースを渡すことが必要になることがあります。
-
+  
   マージは2つのブランチの変更を結合するために最善を尽くしますが、
   人間だけが修正できる種類の問題があります。
   この問題に遭遇するとき、衝突マークがつけられます。
   衝突はコミットする前に何かを修正する必要があることを意味します。
-
+  
   問題を修正したらbzr resolveを使用します。bzr conflictsも参照してください。
-
+  
   デフォルトのブランチセットが存在しない場合、最初のマージはそれを設定します。
   その後で、デフォルトを使用するブランチを省略できます。
   デフォルトを変更するには、--rememberを使用します。
   リモートロケーションがアクセス可能場合のみ値は保存されます。
-
+  
   マージの結果は目的の作業ディレクトリに設置されます。
   このディレクトリは(bzr diffで)閲覧、テスト、
   とマージの結果を記録するためにコミット可能です。
-
+  
   --forceが渡されない限り、コミットされていない変更が存在すればマージの実行は拒否されます。
 
 :例:
@@ -2857,13 +2859,13 @@
   他のすべての名前はそこに移動します。
   さもなければ、2つの引数だけにしなければならず
   ファイルは新しい名前に変更されます。
-
+  
   OLDNAMEがファイルシステム上に存在しないがバージョン管理されていて
   NEWNAMEはファイルシステム上に存在せずバージョン管理もされていない場合、
   mvはファイルが手動で移動させられその変更を反映するために
   内部インベントリだけを更新することを想定します。
   多くのSOURCEファイルをDESTINATIONに移動させるときも同じです。
-
+  
   ブランチの間でファイルを移動させることはできません。
 
 :エイリアス:  move, rename
@@ -2914,9 +2916,9 @@
 :説明:
   このコマンドはプラグインのバージョンとそれぞれの手短な説明を含めて
   インストールされたプラグインの一覧を表示します
-
+  
   --verboseはそれぞれのプラグインが設置されたパスを表示します。
-
+  
   プラグインはコードを追加したり置き換えたりすることで
   リビジョン管理システムを拡張する外部コンポーネントです。
   コマンドの上書き、新しいコマンドの追加、追加のネットワーク転送の提供や
@@ -2950,19 +2952,19 @@
   このコマンドは分岐されていないブランチのみで動作します。
   目的のブランチの最新の変更コミットが親にマージされなかった場合(直接もしくは間接)、
   ブランチは分岐したものとして見なされます。
-
+  
   ブランチが分岐していれば、あるブランチからの変更を他のブランチに統合するために
   'bzr merge'を使用できます。
   一旦あるブランチがマージされると、他のブランチは再びそれをpullできるようになります。
-
+  
   ローカルの変更を忘れてリモートブランチを満たすようにブランチを更新したいだけなら、
   pull --overwriteを使用します。
-
+  
   デフォルトのロケーションが存在しない場合、最初のpullはこれを設定します。
   その後で、デフォルトを使用するロケーションを省略できます。
   デフォルトを変更するには、--rememberを使用します。
   リモートロケーションがアクセスできる場合のみ値は保存されます。
-
+  
   注: 位置がブランチのフォーマットもしくはbzr sendで生成されたmergeディレクトリを格納する
       ファイルへのパスで指定できます。
 
@@ -2995,20 +2997,20 @@
 :説明:
   これは割高でリモートファイルシステムではサポートされないので、
   ターゲットブランチは投入された作業ツリーを持ちません。
-
+  
   スマートサーバーもしくはプロトコルの中には将来作業ツリーを導入しないものがあります。
-
+  
   このコマンドは分岐されていないブランチでのみ動作します。
   目的のブランチの最新のコミットがソースブランチによって(直接もしくは間接的に)マージされなければ
   ブランチは分岐されたものとして見なされます。
-
+  
   ブランチが分岐されると、他のブランチを完全に置き換えるために
   'bzr push --overwrite'を使用できます。この場合、マージされていない変更は廃棄されます。
-
+  
   他のブランチに異なる変更があることを保証したい場合は、
   他のブランチからマージを行い(bzr help mergeを参照)、それをコミットします。
   その後で'--overwrite'なしでpushを行うことができるようになります。
-
+  
   デフォルトのpush位置の設定がなければ、最初のpushはこれを設定します。
   その後では、デフォルトを省略できます。
   デフォルトを変更するには、--rememberを使用します。
@@ -3036,9 +3038,9 @@
   2番目のブランチが提供されると、
   ブランチをまたがる調整も行われます。これによってbzrの初期のバージョンでは存在しなかった
   ツリーのroot idのようなデータがチェックされ、両方のブランチで正しく表示されます。
-
+  
   同時にデータが再圧縮されるのでディスクスペースの節約やパフォーマンスのゲインにつながります。
-
+  
   ブランチはローカルディスクもしくはsftpのようにリスト可能なシステム上になければなりません。
 
 :関連項目: `check`_
@@ -3067,7 +3069,7 @@
 
 :説明:
   ターゲットの設定を指定しなければなりません。
-
+  
   チェックアウトに関しては指定されていなければ、bind-toのロケーションは自動検出されます。
   優先順位は
   1. 軽量チェックアウトに関しては、現在バインドされているロケーション。
@@ -3101,7 +3103,7 @@
 :説明:
   衝突を解消している間に異なるマージテクニックを試したい場合はこれを使用します。
   マージテクニックの中には他のものよりもすぐれたものがあり、
-  remergeによって異なるファイルで異なるテクニックを試すことができます。
+  remergeによって異なるファイルで異なるテクニックを試すことができます。  
   remergeのオプションはmergeのものと同じ意味とデフォルトを持ちます。
   違いは未解決のマージが存在するときのみremergeは実行できて
   特定のファイルを指定できることです。
@@ -3109,12 +3111,12 @@
 :例:
     すべてのファイルのマージを再実行し、通常のTHISとOTHERテキストに加えて、
     衝突領域のベーステキストを表示する::
-
+  
         bzr remerge --show-base
 
     weaveマージアルゴリズムを使用して"foobar"のマージを再実行して、
     衝突領域のサイズを減らすために追加処理を行う::
-
+  
         bzr remerge --merge-type weave --reprocess foobar
 
 
@@ -3138,7 +3140,7 @@
 
 :説明:
   このコマンドによってbzrは指定されたファイルへの変更の追跡を止めます。
-  rebertコマンドで容易に復元できるのであればbzrはこれらのファイルを削除します。
+  rebertコマンドで容易に復元できるのであればbzrはこれらのファイルを削除します。 
   オプションもしくはパラメータが与えられなければbzrは追跡されているファイルをスキャンしますが
   ツリーの中で見つからなければそれらの追跡を停止します。
 
@@ -3158,7 +3160,7 @@
 
 :説明:
   軽量チェックアウトは作業ツリーと大差ないので、これはそれに対する実行を拒絶します。
-
+  
   作業ツリーを再現するには、"bzr checkout"を使用します。
 
 :関連項目: `checkout`_, `working-trees`_
@@ -3193,11 +3195,11 @@
   人間だけが修正できる種類の問題があります。
   この問題に遭遇するとき、衝突がマークされます。
   衝突はコミットする前に何かを修正する必要があることを意味します。
-
+  
   一旦問題を修正すれば、自動的にテキストの衝突を修正したものとしてマークするために"bzr resolve"を使用し、
   特定の衝突を解消したものとしてマークするためにFILEをresolveします。
   すべての衝突が解消されたものとしてマークするにはor "bzr resolve --all"を行います。
-
+  
 :関連コマンド: `bzr conflicts`
 
 :エイリアス:  resolved
@@ -3221,29 +3223,29 @@
   指定されたテキストだけを差し戻すファイルのリストを渡します。
   さもなければ、すべてのファイルが差し戻されます。
   '--revision'でリビジョンが指定されなければ、最後にコミットされたリビジョンが使用されます。
-
+  
   以前のリビジョンに差し戻さずに、いくつかの変更を除外するには、代わりにmergeを使用します。
   たとえば、"merge . --revision -2..-3"は-2で導入された変更を除外しますが、
   -1で導入された変更には影響を与えません。
   hunk-by-hunkベースである変更を除外するには、Shelfプラグインを参照してください。
-
+  
   デフォルトでは、手動で変更されてきたファイルは最初にバックアップされます。
   (マージのみで変更されたファイルはバックアップされません。)
   バックアップファイルの名前には '.~#~' が追加されます。#は番号です。
-
+  
   ファイルを提供する場合、現在のパス名もしくはターゲットリビジョンからのパス名を使用できます。
   名前でファイルを"undelete"するためにrevertを使用できます。
   ディレクトリに名前をつけると、そのディレクトリのすべての内容が差し戻されます。
-
+  
   そのリビジョン以降に新しく追加されたファイルは削除されます。適切であればバックアップは維持されます。
   未知のファイルを持つディレクトリは削除されません。
-
+  
   作業ツリーは未解決のマージされたリビジョンのリストを含みます。
   これは次のコミットで親として含まれます。
   通常は、ファイルを差し戻すのと同様にrevertはそのリストをクリーンにします。
   ファイルが指定されていれば、revertは未解決のマージリストをそのままにしてファイルだけを差し戻します。
   すべてのファイルを差し戻すがマージの記録を維持するにはツリーのrootで"bzr revert ."を使用し、
-  ファイルの差し戻しを行わずに未解決のマージリストをクリアするには"bzr revert --forget-merges"を使用します。
+  ファイルの差し戻しを行わずに未解決のマージリストをクリアするには"bzr revert --forget-merges"を使用します。 
 
 :関連項目: `cat`_, `export`_
 
@@ -3302,41 +3304,41 @@
 
 :説明:
   mergeディレクティブはmergeリクエストを行うために必要な多くのものを提供します:
-
+  
   * 実行するマージのマシンが理解できる説明
-
+  
   * リクエストされた変更のプレビューであるオプションのパッチ
-
+  
   * リビジョンデータのオプションバンドル、
     ブランチからデータを読み込まずに、mergeディレクトリからの変更を直接適用できるようになります。
-
+  
   --no-bundleが指定されると、public_branchが必要です(また最新でなければなりません)、
   受け取り手がpublic_branchを使用するマージを実行できるように
   後で他の人がチェックできるように、知っているのであればpublic_branchを常に含まれていなければなりません。
-
+  
   投稿ブランチのデフォルトは親ですが、上書きできます。
   提供されれば投稿ブランチと公開ブランチの両方が記録されます。
-
+  
   public_branchがsubmit_branchに知られていれば、
   その公開と投稿ブランチはマージのインストラクションで使用されます。
   これはそのローカルミラーに対してpublic_branchを設定すれば、
   そのミラーは実際の投稿ブランチとして使用できることを意味します。
-
+  
   メールは好きなプログラムで送信されます。
   Windowsではこれは透過的です(MAPIが使用される)。
   Linuxでは、xdg-emailユーティリティを必要とします。
   望ましいクライアントが見つからなければ(もしくは使用できなければ)、エディタが使用されます。
-
+  
   特定のメールプログラムを使用するには、mail_client設定オプションを設定します。
-  (Thunderbird 1.5に関して、これはいくつかのバグに対処します。)
+  (Thunderbird 1.5に関して、これはいくつかのバグに対処します。) 
   特定のクライアント用にサポートされる値は"claws"、"evolution"、"kmail"、"mutt"、と"thunderbird";
   一般的なオプションは"default"、"editor"、"emacsclient"、"mapi"、と"xdg-email"です。
   プラグインがサポートされるクライアントを追加することもあります。
-
+  
   メールが送信されている場合、to addressが必要になります。
   これはコマンドライン、submit_to設定オプションをブランチ自身に設定するか、
   投稿ブランチでchild_submit_to設定オプションを設定することで提供できます。
-
+  
   現在2つのフォーマットがサポートされています: "4"はリビジョンバンドルフォーマット4
   とマージディレクトリフォーマット2です。
   これは古いフォーマットよりも顕著に速く小さいです。
@@ -3344,7 +3346,7 @@
   これはデフォルトです。
   "0.9"はリビジョンバンドルフォーマット0.9とマージディレクティブフォーマット1を使用します。
   これは0.12 - 0.18と互換性があります。
-
+  
   mergeもしくはpullコマンドを使用することでmergeディレクトリが適用されます。
 
 :関連項目: `merge`_, `pull`_
@@ -3393,18 +3395,18 @@
 :説明:
   shelveによって変更を一時的に"棚に上げる"、すなわち邪魔にならない場所に置くことができます。
   'unshelve'コマンドで後で元に戻すことができます。
-
+  
   shelve --listが指定されると、以前退避された変更の一覧が表示されます。
-
+  
   shelveは不適切に混ぜられた変更のいくつかのセットの分離を手助けすることを目的としています。
   すべての変更を除去したいだけで後で退避する必要がなければ、revertを使用します。
   一度にすべてのテキストの変更をshelveするには、shelve --allを使用します。
-
+  
   ファイル名が指定されると、それらのファイルの変更のみ退避されます。
   他のファイルは手つかずのままです。
-
+  
   リビジョンが指定されれば、そのリビジョン以降の変更は退避されます。
-
+  
   複数のアイテムを退避することができ、デフォルトでは、
   'unshelve'は最近shelveされた変更を復元します。
 
@@ -3425,7 +3427,7 @@
 :説明:
   位置が指定されなければローカルツリーが使用されます。
   コミッターが指定されなければデフォルトのコミッターが使用されます。
-
+  
   これはすでにシグネチャを持つコミットには署名しません。
 
 
@@ -3444,7 +3446,7 @@
   このコマンドは'rich-root' もしくは 'rich-root-pack'のように
   リッチrootをサポートするフォーマットでターゲットツリーを生み出します。
   これらのフォーマットは'dirstate-tags'のような初期のフォーマットに変換できません。
-
+  
   TREEの引数は作業ツリーのサブディレクトリになります。
   そのサブディレクトリは独自のブランチを持つ独立したツリーに変換されます。
   トップレベルツリーのコミットは新しいサブツリーに適用されません。
@@ -3472,38 +3474,38 @@
 :説明:
   これはバージョン管理されたファイルと未知のファイルを状態で
   分類してレポートします。利用可能な状態は次のとおりです:
-
+  
   added
       作業ツリーでバージョン管理されているが以前のリビジョンではない。
-
+  
   removed
       以前のリビジョンでバージョン管理されているが作業コピーでは移動もしくは削除されている。
-
+  
   renamed
       以前のリビジョンから変更されたファイルのパス;
       テキストも変更されていることがある。
       これは親ディレクトリがリネームされたファイルを含む。
-
+  
   modified
       以前のリビジョン以降変更されたテキスト。
-
+  
   kind changed
       変更されたファイルの種類(たとえばファイルからディレクトリへ)。
-
+  
   unknown
       バージョン管理されていないかつ無視パターンにマッチしない。
-
+  
   無視されるファイルを見るには'bzr ignored'を使用します。
   ファイルテキストへの詳細な変更に関しては、'bzr diff'を使用します。
-
+  
   --shortもしくは-Sは、Subversionのstatusコマンドに似た、
   それぞれのアイテムに対するステータスフラグを提供することに注意してください。
   svn -qと似たような出力を得るには、bzr status -SVを使用します。
-
+  
   引数が指定されなければ、作業ディレクトリ全体のステータスが示されます。
   さもなければ、指定されたファイルもしくはディレクトリのステータスのみが報告されます。
   ディレクトリが渡されれば、そのディレクトリ内部のすべてに関するステータスが報告されます。
-
+  
   1つのリビジョンの引数が渡されれば、ステータスはそのリビジョンに対して
   2つの引数の場合は2つのリビジョンの間で算出されます。
 
@@ -3527,12 +3529,12 @@
   重量チェックアウトに対して、これはローカルコミットがなく、
   バインドされたブランチがないことを確認して、
   ローカルブランチを新しいロケーションのミラーにしてそれにバインドします。
-
+  
   両方の場合において、作業ツリーはupdateされコミットされてない変更はマージされます。
   ユーザーは望むのであればcommitもしくはrevertできます。
-
+  
   マージの追加にはswithを使用する前にcommitもしくはrevertする必要があります。
-
+  
   swithするブランチへのパスは現在のブランチの親ディレクトリに対して相対的に指定できます。
   たとえば、現在/path/to/branchのチェックアウトの中にいるのであれば
   'newbranch'を指定すれば/path/to/newbranchでのブランチが発見されます。
@@ -3561,13 +3563,13 @@
   タグはリビジョンに人間が理解できる名前を与えます。
   -r (--revision)オプションをとるコマンドは-rtag:Xに渡されます。
   Xは以前作成されたタグです。
-
+  
   タグはブランチに保存されます。
   branch、push、pullもしくはmergeを行うときタグはあるブランチから他のブランチにコピーされます。
-
+  
   --forceを渡さない限り、既存のタグ名を与えるとエラーになります。
   この場合新しいリビジョンを示すようにタグは移動します。
-
+  
   タグをリネームする(名前を変更するが同じリビジョンで維持する)には、
   ``bzr tag new-name -r tag:old-name`` と ``bzr tag --delete oldname`` を実行します。
 
@@ -3649,11 +3651,11 @@
 :説明:
   --verboseは削除されているものを表示します。
   --dry-runはすべてのモーションを経験しますが、実際には何も削除しません。
-
+  
   --revisionが指定されると、指定されたリビジョンでブランチをそのままにするために
   リビジョンをuncommitします。たとえば、"bzr uncommit -r 15"はリビジョン15でのブランチを
   そのままにします。
-
+  
   uncommitは新しいコミットの準備ができている作業ツリーをそのままにします。
   唯一行われる変更はコミット以前に存在していた追加マージをリストアすることです。
 
@@ -3696,7 +3698,7 @@
   このコマンドは作業ツリーでマージを実行し、衝突を生成することがあります。
   ローカルの変更がある場合、updateを完了させるために
   updateの後でそれらをコミットする必要があります。
-
+  
   ローカルの変更を破棄したい場合、updateの後で'bzr commit'の代わりに
   'bzr revert'を使用できます。
 
@@ -3745,8 +3747,8 @@
                         repo.html をご覧いただくようお願いします。
     --dirstate          0.15の新しいフォーマット: 速いローカルオペレーション。
                         ネットワークを通したアクセスのときbzr 0.8とそれ以降と互換性がある。
-    --dirstate-tags     0.15の新しいフォーマット: 速いローカルオペレーションで
-                        ネットワークオペレーションに関するスケーリングを改善。
+    --dirstate-tags     0.15の新しいフォーマット: 速いローカルオペレーションで 
+                        ネットワークオペレーションに関するスケーリングを改善。 
                         タグのサポートを追加。bzr < 0.15とは互換性がない。
     --knit              knitsを使用するフォーマット。bzr <= 0.14との相互運用に推奨。
     --metaweave         0.8での暫定フォーマット。knitよりも遅い。
@@ -3809,15 +3811,15 @@
   バージョンに関する情報をアプリケーションのソースコードに追加するために
   このコマンドを使用できます。出力のフォーマットはサポートされているもの1つか
   テンプレートに基づいてカスタマイズされたものです。
-
+  
   例::
-
+  
     bzr version-info --custom \
       --template="#define VERSION_INFO \"Project 1.2.3 (r{revno})\"\n"
-
+  
   現在のリビジョン番号を含むフォーマットされた文字列でCヘッダファイルを生成します。
   テンプレートでのサポートされた他の変数は次のとおりです:
-
+  
     * {date} - 最終リビジョンの日付
     * {build_date} - 現在の日付
     * {revno} - リビジョン番号

=== modified file 'doc/ru/index.txt'
--- doc/ru/index.txt	2010-07-07 11:21:19 +0000
+++ doc/ru/index.txt	2010-08-17 06:45:33 +0000
@@ -3,7 +3,7 @@
 =================================
 
 Последняя версия этих документов доступа со страницы документации
-на сайте Bazaar, <http://doc.bazaar-vcs.org/ru/>.
+на сайте Bazaar, <http://doc.bazaar.canonical.com/ru/>.
 
 Основная документация
 =====================
@@ -22,10 +22,10 @@
 Ссылки в сети
 =============
 
-* `Руководство по миграции <http://doc.bazaar-vcs.org/migration/en/>`_
+* `Руководство по миграции <http://doc.bazaar.canonical.com/migration/en/>`_
   |--| для команд переносящих историю с других систем контроля версий
 
-* `Словарь терминов <http://bazaar-vcs.org/BzrGlossary>`_ (англ.),
+* `Словарь терминов <http://wiki.bazaar.canonical.com/BzrGlossary>`_ (англ.),
   см. также `русскую версию`__
 
 __  http://groups.google.com/group/ru_bzr/web/%D0%B3%D0%BB%D0%BE%D1%81%D1%81%D0%B0%D1%80%D0%B8%D0%B9

=== modified file 'doc/ru/mini-tutorial/index.txt'
--- doc/ru/mini-tutorial/index.txt	2010-07-07 11:21:19 +0000
+++ doc/ru/mini-tutorial/index.txt	2010-08-17 06:45:33 +0000
@@ -31,10 +31,10 @@
 Для других платформ и для установки из исходных кодов, обратитесь к страницам
 Загрузка_ и Установка_.
 
-.. _инструкции по установке для Windows: http://bazaar-vcs.org/WindowsDownloads
-.. _инструкции по установке для Mac OS X: http://bazaar-vcs.org/MacOSXBundle
-.. _Загрузка: http://bazaar-vcs.org/Download
-.. _Установка: http://bazaar-vcs.org/InstallationFaq
+.. _инструкции по установке для Windows: http://wiki.bazaar.canonical.com/WindowsDownloads
+.. _инструкции по установке для Mac OS X: http://wiki.bazaar.canonical.com/MacOSXBundle
+.. _Загрузка: http://wiki.bazaar.canonical.com/Download
+.. _Установка: http://wiki.bazaar.canonical.com/InstallationFaq
 
 Представьтесь
 =============
@@ -78,7 +78,7 @@
 Если всё выглядит так, как будто ничего не случилось |--| не волнуйтесь. Bazaar
 создал ветку_, в которой он будет хранить рабочие файлы и историю их изменений.
 
-.. _ветку: http://bazaar-vcs.org/Branch
+.. _ветку: http://wiki.bazaar.canonical.com/Branch
 
 Следующий шаг |--| сказать Bazaar какие файлы вы хотите контролировать. Команда
 ``bzr add`` рекурсивно добавит все файлы в проект::
@@ -172,7 +172,7 @@
 
 **Замечание:** чтобы использовать SFTP, может понадобиться установить
 ``paramiko`` и ``pyCrypto``. За подробностями обращайтесь к
-http://bazaar-vcs.org/InstallationFaq.
+http://wiki.bazaar.canonical.com/InstallationFaq.
 
 
 Публикация ветки на Launchpad

=== modified file 'doc/ru/tutorials/centralized_workflow.txt'
--- doc/ru/tutorials/centralized_workflow.txt	2010-02-17 07:15:12 +0000
+++ doc/ru/tutorials/centralized_workflow.txt	2010-08-13 19:08:57 +0000
@@ -24,7 +24,7 @@
 работу. Такой режим так же подходит для разработчика-одиночки,
 работающего на нескольких различных машинах.
 
-.. _Bazaar: http://bazaar-vcs.org
+.. _Bazaar: http://bazaar.canonical.com
 
 .. contents::
         Содержание
@@ -107,7 +107,7 @@
 `Отслеживание изменений на основной ветке`_ для некоторых способов (секция
 "Конвертирование и сохранение истории").
 
-.. _Отслеживание изменений на основной ветке: http://bazaar-vcs.org/TrackingUpstream
+.. _Отслеживание изменений на основной ветке: http://wiki.bazaar.canonical.com/TrackingUpstream
 
 ..
    TODO: На самом деле нам нужен другой документ для описания
@@ -238,7 +238,7 @@
 
    % bzr cbranch dev my-feature-branch
 
-.. _bzrtools: http://bazaar-vcs.org/BzrTools
+.. _bzrtools: http://wiki.bazaar.canonical.com/BzrTools
 
 
 Объединение изменений обратно

=== modified file 'doc/ru/tutorials/tutorial.txt'
--- doc/ru/tutorials/tutorial.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/tutorials/tutorial.txt	2010-11-12 22:36:19 +0000
@@ -201,11 +201,11 @@
 большого количества веток для проекта среднего размера.
 
 Мы обычно обращаемся к веткам на нашем компьютере просто передав имя каталога
-содержащего ветку. bzr также поддерживает доступ к веткам через http и sftp,
+содержащего ветку. bzr также поддерживает доступ к веткам через HTTP и SFTP,
 например::
 
-    % bzr log http://bazaar-vcs.org/bzr/bzr.dev/
-    % bzr log sftp://bazaar-vcs.org/bzr/bzr.dev/
+    % bzr log http://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
+    % bzr log sftp://bazaar.launchpad.net/~bzr-pqm/bzr/bzr.dev/
 
 Установив для bzr плагины можно также осуществлять доступ к веткам с
 использованием rsync.
@@ -482,7 +482,7 @@
 готовой ветки. Так как эта копия может быть потенциальной новой веткой эта
 команда называется **branch**::
 
-    % bzr branch http://bazaar-vcs.org/bzr/bzr.dev
+    % bzr branch lp:bzr bzr.dev
     % cd bzr.dev
 
 Эта команда копирует полную историю ветки и после этого вы можете делать все

=== modified file 'doc/ru/tutorials/using_bazaar_with_launchpad.txt'
--- doc/ru/tutorials/using_bazaar_with_launchpad.txt	2010-07-07 11:21:19 +0000
+++ doc/ru/tutorials/using_bazaar_with_launchpad.txt	2010-08-17 06:45:33 +0000
@@ -93,7 +93,7 @@
 -------------------------------------------
 
 Кроме спонсорства разработки `Ubuntu <http://www.ubuntu.com>`_ и `Bazaar
-<http://bazaar-vcs.org>`_, Canonical так же предоставляет Launchpad,
+<http://bazaar.canonical.com>`_, Canonical так же предоставляет Launchpad,
 https://launchpad.net, как бесплатный сервис для сообществ с открытым исходным
 кодом. Launchpad является одной из самых интересных сред совместной разработки
 по следующим причинам:

=== modified file 'doc/ru/user-guide/branching_a_project.txt'
--- doc/ru/user-guide/branching_a_project.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/user-guide/branching_a_project.txt	2010-11-12 22:36:19 +0000
@@ -59,7 +59,7 @@
 
 If a directory is not given, one is created based on the last part of
 the URL. Here are some examples showing a drive qualified path (M:/) and an
-sftp URL respectively::
+SFTP URL respectively::
 
   bzr branch M:/cool-trunk
   bzr branch sftp://bill@mary-laptop/cool-repo/cool-trunk

=== modified file 'doc/ru/user-guide/introducing_bazaar.txt'
--- doc/ru/user-guide/introducing_bazaar.txt	2009-12-02 20:34:07 +0000
+++ doc/ru/user-guide/introducing_bazaar.txt	2010-08-13 19:08:57 +0000
@@ -94,7 +94,7 @@
  * для компаний - упрощение работы с распределенными и внешними командами.
 
 Для более детального взгляда на преимущества распределенных систем контроля
-версий по сравнению с централизованными смотрите http://bazaar-vcs.org/BzrWhy.
+версий по сравнению с централизованными смотрите http://wiki.bazaar.canonical.com/BzrWhy.
 
 
 Ключевые особенности Bazaar
@@ -104,12 +104,12 @@
 некоторые значимые преимущества, которые делают ее прекрасным выбором для
 многих команд и сообществ. Описание этих особенностей и сравнение с другими
 системами контроля версий может быть найдено на Wiki Bazaar -
-http://bazaar-vcs.org.
+http://wiki.bazaar.canonical.com.
 
 Из большинства особенностей, одна требует особого упоминания: Bazaar - это
 полностью свободное ПО написанное на языке Python. Это упрощает сотрудничество
 для внесения улучшений. Если вы хотите помочь, обратите внимание на
-http://bazaar-vcs.org/BzrSupport.
+http://wiki.bazaar.canonical.com/BzrSupport.
 
 
 Продолжаем изучение

=== added directory 'po'
=== added file 'po/bzr.pot'
--- po/bzr.pot	1970-01-01 00:00:00 +0000
+++ po/bzr.pot	2011-09-20 09:41:21 +0000
@@ -0,0 +1,12747 @@
+# SOME DESCRIPTIVE TITLE.
+# Copyright (C) YEAR Canonical
+# This file is distributed under the same license as the PACKAGE package.
+# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
+#
+#, fuzzy
+msgid ""
+msgstr ""
+"Project-Id-Version: bzr\n"
+"Report-Msgid-Bugs-To: <bazaar@canonical.com>\n"
+"POT-Creation-Date: 2011-09-20 10:41+0100\n"
+"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
+"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
+"Language-Team: LANGUAGE <LL@li.org>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=CHARSET\n"
+"Content-Transfer-Encoding: 8bit\n"
+"Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;\n"
+
+#: bzrlib/add.py:95
+msgid "skipping {0} (larger than {1} of {2} bytes)"
+msgstr ""
+
+#: bzrlib/branch.py:852
+msgid "Unstacking"
+msgstr ""
+
+#: bzrlib/branch.py:3138
+#, python-format
+msgid "Pushed up to revision %d."
+msgstr ""
+
+#: bzrlib/branch.py:3140
+#, python-format
+msgid "%d tag updated."
+msgid_plural "%d tags updated."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/branch.py:3143
+msgid "No new revisions or tags to push."
+msgstr ""
+
+#: bzrlib/branch.py:3145
+msgid "No new revisions to push."
+msgstr ""
+
+#: bzrlib/branch.py:3165
+msgid "checked branch {0} format {1}"
+msgstr ""
+
+#: bzrlib/branch.py:3168
+#, python-format
+msgid "found error:%s"
+msgstr ""
+
+#: bzrlib/builtins.py:116 bzrlib/builtins.py:2782 bzrlib/workingtree.py:306
+#, python-format
+msgid "Ignoring files outside view. View is %s"
+msgstr ""
+
+#: bzrlib/builtins.py:125
+#, python-format
+msgid "bzr %s --revision takes exactly one revision identifier"
+msgstr ""
+
+#: bzrlib/builtins.py:210
+msgid "Display status summary."
+msgstr ""
+
+#: bzrlib/builtins.py:212
+msgid ""
+"This reports on versioned and unknown files, reporting them\n"
+"grouped by state.  Possible states are:"
+msgstr ""
+
+#: bzrlib/builtins.py:215
+msgid ""
+"added\n"
+"    Versioned in the working copy but not in the previous revision."
+msgstr ""
+
+#: bzrlib/builtins.py:218
+msgid ""
+"removed\n"
+"    Versioned in the previous revision but removed or deleted\n"
+"    in the working copy."
+msgstr ""
+
+#: bzrlib/builtins.py:222
+msgid ""
+"renamed\n"
+"    Path of this file changed from the previous revision;\n"
+"    the text may also have changed.  This includes files whose\n"
+"    parent directory was renamed."
+msgstr ""
+
+#: bzrlib/builtins.py:227
+msgid ""
+"modified\n"
+"    Text has changed since the previous revision."
+msgstr ""
+
+#: bzrlib/builtins.py:230
+msgid ""
+"kind changed\n"
+"    File kind has been changed (e.g. from file to directory)."
+msgstr ""
+
+#: bzrlib/builtins.py:233
+msgid ""
+"unknown\n"
+"    Not versioned and not matching an ignore pattern."
+msgstr ""
+
+#: bzrlib/builtins.py:236
+msgid ""
+"Additionally for directories, symlinks and files with a changed\n"
+"executable bit, Bazaar indicates their type using a trailing\n"
+"character: '/', '@' or '*' respectively. These decorations can be\n"
+"disabled using the '--no-classify' option."
+msgstr ""
+
+#: bzrlib/builtins.py:241
+msgid ""
+"To see ignored files use 'bzr ignored'.  For details on the\n"
+"changes to file texts, use 'bzr diff'."
+msgstr ""
+
+#: bzrlib/builtins.py:244
+msgid ""
+"Note that --short or -S gives status flags for each item, similar\n"
+"to Subversion's status command. To get output similar to svn -q,\n"
+"use bzr status -SV."
+msgstr ""
+
+#: bzrlib/builtins.py:248
+msgid ""
+"If no arguments are specified, the status of the entire working\n"
+"directory is shown.  Otherwise, only the status of the specified\n"
+"files or directories is reported.  If a directory is given, status\n"
+"is reported for everything inside that directory."
+msgstr ""
+
+#: bzrlib/builtins.py:253
+msgid ""
+"Before merges are committed, the pending merge tip revisions are\n"
+"shown. To see all pending merge revisions, use the -v option.\n"
+"To skip the display of pending merge information altogether, use\n"
+"the no-pending option or specify a file/directory."
+msgstr ""
+
+#: bzrlib/builtins.py:258
+msgid ""
+"To compare the working directory to a specific revision, pass a\n"
+"single revision to the revision argument."
+msgstr ""
+
+#: bzrlib/builtins.py:261
+msgid ""
+"To see which files have changed in a specific revision, or between\n"
+"two revisions, pass a revision range to the revision argument.\n"
+"This will produce the same results as calling 'bzr diff --summarize'."
+msgstr ""
+
+# help of 'short' option of 'status' command
+#: bzrlib/builtins.py:270
+msgid "Use short status indicators."
+msgstr ""
+
+# help of 'versioned' option of 'status' command
+#: bzrlib/builtins.py:272
+msgid "Only show versioned files."
+msgstr ""
+
+# help of 'no-pending' option of 'status' command
+#: bzrlib/builtins.py:274
+msgid "Don't show pending merges."
+msgstr ""
+
+# help of 'no-classify' option of 'status' command
+#: bzrlib/builtins.py:277
+msgid "Do not mark object type using indicator."
+msgstr ""
+
+#: bzrlib/builtins.py:292
+msgid "bzr status --revision takes exactly one or two revision specifiers"
+msgstr ""
+
+#: bzrlib/builtins.py:336 bzrlib/builtins.py:4927
+msgid "You can only supply one of revision_id or --revision"
+msgstr ""
+
+#: bzrlib/builtins.py:339 bzrlib/builtins.py:4929
+msgid "You must supply either --revision or a revision_id"
+msgstr ""
+
+#: bzrlib/builtins.py:346
+#, python-format
+msgid "Repository %r does not support access to raw revision texts"
+msgstr ""
+
+#: bzrlib/builtins.py:357
+msgid "The repository {0} contains no revision {1}."
+msgstr ""
+
+#: bzrlib/builtins.py:364
+msgid "You cannot specify a NULL revision."
+msgstr ""
+
+#: bzrlib/builtins.py:456
+msgid "Remove the working tree from a given branch/checkout."
+msgstr ""
+
+#: bzrlib/builtins.py:458
+msgid ""
+"Since a lightweight checkout is little more than a working tree\n"
+"this will refuse to run against one."
+msgstr ""
+
+#: bzrlib/builtins.py:461
+msgid "To re-create the working tree, use \"bzr checkout\"."
+msgstr ""
+
+# help of 'force' option of 'remove-tree' command
+#: bzrlib/builtins.py:467
+msgid "Remove the working tree even if it has uncommitted or shelved changes."
+msgstr ""
+
+#: bzrlib/builtins.py:481
+msgid "No working tree to remove"
+msgstr ""
+
+#: bzrlib/builtins.py:483
+msgid "You cannot remove the working tree of a remote path"
+msgstr ""
+
+#: bzrlib/builtins.py:492
+msgid "You cannot remove the working tree from a lightweight checkout"
+msgstr ""
+
+#: bzrlib/builtins.py:532
+msgid ""
+"The tree does not appear to be corrupt. You probably want \"bzr revert\" "
+"instead. Use \"--force\" if you are sure you want to reset the working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:543
+msgid ""
+", the header appears corrupt, try passing -r -1 to set the state to the last "
+"commit"
+msgstr ""
+
+#: bzrlib/builtins.py:547
+msgid "failed to reset the tree state{0}"
+msgstr ""
+
+#: bzrlib/builtins.py:551
+msgid "Show current revision number."
+msgstr ""
+
+#: bzrlib/builtins.py:553
+msgid "This is equal to the number of revisions on this branch."
+msgstr ""
+
+# help of 'tree' option of 'revno' command
+#: bzrlib/builtins.py:594
+msgid "Show revno of working tree"
+msgstr ""
+
+#: bzrlib/builtins.py:642
+msgid "Add specified files or directories."
+msgstr ""
+
+#: bzrlib/builtins.py:644
+msgid ""
+"In non-recursive mode, all the named items are added, regardless\n"
+"of whether they were previously ignored.  A warning is given if\n"
+"any of the named files are already versioned."
+msgstr ""
+
+#: bzrlib/builtins.py:648
+msgid ""
+"In recursive mode (the default), files are treated the same way\n"
+"but the behaviour for directories is different.  Directories that\n"
+"are already versioned do not give a warning.  All directories,\n"
+"whether already versioned or not, are searched for files or\n"
+"subdirectories that are neither versioned or ignored, and these\n"
+"are added.  This search proceeds recursively into versioned\n"
+"directories.  If no names are given '.' is assumed."
+msgstr ""
+
+#: bzrlib/builtins.py:656
+msgid ""
+"Therefore simply saying 'bzr add' will version all files that\n"
+"are currently unknown."
+msgstr ""
+
+#: bzrlib/builtins.py:659
+msgid ""
+"Adding a file whose parent directory is not versioned will\n"
+"implicitly add the parent, and so on up to the root. This means\n"
+"you should never need to explicitly add a directory, they'll just\n"
+"get added when you add a file in the directory."
+msgstr ""
+
+#: bzrlib/builtins.py:664
+msgid ""
+"--dry-run will show which files would be added, but not actually\n"
+"add them."
+msgstr ""
+
+#: bzrlib/builtins.py:667
+msgid ""
+"--file-ids-from will try to use the file ids from the supplied path.\n"
+"It looks up ids trying to find a matching parent directory with the\n"
+"same filename, and then by pure path. This option is rarely needed\n"
+"but can be useful when adding the same logical file into two\n"
+"branches that will be merged later (without showing the two different\n"
+"adds as a conflict). It is also useful when merging another project\n"
+"into a subdirectory of this one."
+msgstr ""
+
+#: bzrlib/builtins.py:675
+msgid ""
+"Any files matching patterns in the ignore list will not be added\n"
+"unless they are explicitly mentioned."
+msgstr ""
+
+#: bzrlib/builtins.py:678
+msgid ""
+"In recursive mode, files larger than the configuration option \n"
+"add.maximum_file_size will be skipped. Named items are never skipped due\n"
+"to file size."
+msgstr ""
+
+# help of 'no-recurse' option of 'add' command
+#: bzrlib/builtins.py:685
+msgid "Don't recursively add the contents of directories."
+msgstr ""
+
+# help of 'file-ids-from' option of 'add' command
+#: bzrlib/builtins.py:691
+msgid "Lookup file ids from this tree."
+msgstr ""
+
+#: bzrlib/builtins.py:727
+msgid "ignored {0} matching \"{1}\"\n"
+msgstr ""
+
+#: bzrlib/builtins.py:732
+msgid "Create a new versioned directory."
+msgstr ""
+
+#: bzrlib/builtins.py:734
+msgid "This is equivalent to creating the directory and then adding it."
+msgstr ""
+
+#: bzrlib/builtins.py:748
+#, python-format
+msgid "added %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:792
+#, python-format
+msgid "invalid kind %r specified"
+msgstr ""
+
+#: bzrlib/builtins.py:829
+msgid "Move or rename a file."
+msgstr ""
+
+#: bzrlib/builtins.py:831
+msgid "    bzr mv SOURCE... DESTINATION"
+msgstr ""
+
+#: bzrlib/builtins.py:833
+msgid ""
+"If the last argument is a versioned directory, all the other names\n"
+"are moved into it.  Otherwise, there must be exactly two arguments\n"
+"and the file is changed to a new name."
+msgstr ""
+
+#: bzrlib/builtins.py:837
+msgid ""
+"If OLDNAME does not exist on the filesystem but is versioned and\n"
+"NEWNAME does exist on the filesystem but is not versioned, mv\n"
+"assumes that the file has been manually moved and only updates\n"
+"its internal inventory to reflect that change.\n"
+"The same is valid when moving many SOURCE files to a DESTINATION."
+msgstr ""
+
+#: bzrlib/builtins.py:843
+msgid "Files cannot be moved between branches."
+msgstr ""
+
+# help of 'after' option of 'mv' command
+#: bzrlib/builtins.py:850
+msgid ""
+"Move only the bzr identifier of the file, because the file has already been "
+"moved."
+msgstr ""
+
+# help of 'auto' option of 'mv' command
+#: bzrlib/builtins.py:852
+msgid "Automatically guess renames."
+msgstr ""
+
+# help of 'dry-run' option of 'mv' command
+#: bzrlib/builtins.py:853
+msgid "Avoid making changes when guessing renames."
+msgstr ""
+
+#: bzrlib/builtins.py:862
+msgid "--dry-run requires --auto."
+msgstr ""
+
+#: bzrlib/builtins.py:866
+msgid "missing file argument"
+msgstr ""
+
+#: bzrlib/builtins.py:873
+msgid "Only one path may be specified to --auto."
+msgstr ""
+
+#: bzrlib/builtins.py:876
+msgid "--after cannot be specified with --auto."
+msgstr ""
+
+#: bzrlib/builtins.py:913
+msgid "to mv multiple files the destination must be a versioned directory"
+msgstr ""
+
+#: bzrlib/builtins.py:965
+msgid "Turn this branch into a mirror of another branch."
+msgstr ""
+
+#: bzrlib/builtins.py:967
+msgid ""
+"By default, this command only works on branches that have not diverged.\n"
+"Branches are considered diverged if the destination branch's most recent \n"
+"commit is one that has not been merged (directly or indirectly) into the \n"
+"parent."
+msgstr ""
+
+#: bzrlib/builtins.py:972
+msgid ""
+"If branches have diverged, you can use 'bzr merge' to integrate the changes\n"
+"from one into the other.  Once one branch has merged, the other should\n"
+"be able to pull it again."
+msgstr ""
+
+#: bzrlib/builtins.py:976
+msgid ""
+"If you want to replace your local changes and just want your branch to\n"
+"match the remote one, use pull --overwrite. This will work even if the two\n"
+"branches have diverged."
+msgstr ""
+
+#: bzrlib/builtins.py:980
+msgid ""
+"If there is no default location set, the first pull will set it (use\n"
+"--no-remember to avoid setting it). After that, you can omit the\n"
+"location to use the default.  To change the default, use --remember. The\n"
+"value will only be saved if the remote location can be accessed."
+msgstr ""
+
+#: bzrlib/builtins.py:985
+msgid ""
+"Note: The location can be specified either in the form of a branch,\n"
+"or in the form of a path to a file containing a merge directive generated\n"
+"with bzr send."
+msgstr ""
+
+# help of 'verbose' option of 'pull' command
+#: bzrlib/builtins.py:993
+msgid "Show logs of pulled revisions."
+msgstr ""
+
+# help of 'directory' option of 'pull' command
+#: bzrlib/builtins.py:995
+msgid ""
+"Branch to pull into, rather than the one containing the working directory."
+msgstr ""
+
+# help of 'local' option of 'pull' command
+#: bzrlib/builtins.py:998
+msgid ""
+"Perform a local pull in a bound branch.  Local pulls are not applied to the "
+"master branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1027
+msgid "Need working tree for --show-base."
+msgstr ""
+
+#: bzrlib/builtins.py:1043
+msgid "No pull location known or specified."
+msgstr ""
+
+#: bzrlib/builtins.py:1049
+#, python-format
+msgid "Using saved parent location: %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1056 bzrlib/builtins.py:4137
+msgid "Cannot use -r with merge directives or bundles"
+msgstr ""
+
+#: bzrlib/builtins.py:1097
+msgid "Update a mirror of this branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1099
+msgid ""
+"The target branch will not have its working tree populated because this\n"
+"is both expensive, and is not supported on remote file systems."
+msgstr ""
+
+#: bzrlib/builtins.py:1102
+msgid ""
+"Some smart servers or protocols *may* put the working tree in place in\n"
+"the future."
+msgstr ""
+
+#: bzrlib/builtins.py:1105
+msgid ""
+"This command only works on branches that have not diverged.  Branches are\n"
+"considered diverged if the destination branch's most recent commit is one\n"
+"that has not been merged (directly or indirectly) by the source branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1109
+msgid ""
+"If branches have diverged, you can use 'bzr push --overwrite' to replace\n"
+"the other branch completely, discarding its unmerged changes."
+msgstr ""
+
+#: bzrlib/builtins.py:1112
+msgid ""
+"If you want to ensure you have the different changes in the other branch,\n"
+"do a merge (see bzr help merge) from the other branch, and commit that.\n"
+"After that you will be able to do a push without '--overwrite'."
+msgstr ""
+
+#: bzrlib/builtins.py:1116
+msgid ""
+"If there is no default push location set, the first push will set it (use\n"
+"--no-remember to avoid setting it).  After that, you can omit the\n"
+"location to use the default.  To change the default, use --remember. The\n"
+"value will only be saved if the remote location can be accessed."
+msgstr ""
+
+# help of 'directory' option of 'push' command
+#: bzrlib/builtins.py:1128
+msgid ""
+"Branch to push from, rather than the one containing the working directory."
+msgstr ""
+
+# help of 'use-existing-dir' option of 'push' command
+#: bzrlib/builtins.py:1131
+msgid ""
+"By default push will fail if the target directory exists, but does not "
+"already have a control directory.  This flag will allow push to proceed."
+msgstr ""
+
+# help of 'stacked' option of 'push' command
+#: bzrlib/builtins.py:1136
+msgid ""
+"Create a stacked branch that references the public location of the parent "
+"branch."
+msgstr ""
+
+# help of 'stacked-on' option of 'push' command
+#: bzrlib/builtins.py:1139
+msgid ""
+"Create a stacked branch that refers to another branch for the commit "
+"history. Only the work not present in the referenced branch is included in "
+"the branch created."
+msgstr ""
+
+# help of 'strict' option of 'push' command
+#: bzrlib/builtins.py:1144
+msgid ""
+"Refuse to push if there are uncommitted changes in the working tree, --no-"
+"strict disables the check."
+msgstr ""
+
+# help of 'no-tree' option of 'push' command
+#: bzrlib/builtins.py:1147
+msgid "Don't populate the working tree, even for protocols that support it."
+msgstr ""
+
+#: bzrlib/builtins.py:1191
+msgid "Could not determine branch to refer to."
+msgstr ""
+
+#: bzrlib/builtins.py:1198 bzrlib/foreign.py:306
+msgid "No push location known or specified."
+msgstr ""
+
+#: bzrlib/builtins.py:1202
+#, python-format
+msgid "Using saved push location: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:1212
+msgid "Create a new branch that is a copy of an existing branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1214
+msgid ""
+"If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will\n"
+"be used.  In other words, \"branch ../foo/bar\" will attempt to create ./"
+"bar.\n"
+"If the FROM_LOCATION has no / or path separator embedded, the TO_LOCATION\n"
+"is derived from the FROM_LOCATION by stripping a leading scheme or drive\n"
+"identifier, if any. For example, \"branch lp:foo-bar\" will attempt to\n"
+"create ./foo-bar."
+msgstr ""
+
+#: bzrlib/builtins.py:1221
+msgid ""
+"To retrieve the branch as of a particular revision, supply the --revision\n"
+"parameter, as in \"branch foo/bar -r 5\"."
+msgstr ""
+
+#: bzrlib/builtins.py:1224
+msgid "The synonyms 'clone' and 'get' for this command are deprecated."
+msgstr ""
+
+# help of 'no-tree' option of 'branch' command
+#: bzrlib/builtins.py:1234
+msgid "Create a branch without a working-tree."
+msgstr ""
+
+# help of 'switch' option of 'branch' command
+#: bzrlib/builtins.py:1236
+msgid "Switch the checkout in the current directory to the new branch."
+msgstr ""
+
+# help of 'stacked' option of 'branch' command
+#: bzrlib/builtins.py:1239
+msgid ""
+"Create a stacked branch referring to the source branch. The new branch will "
+"depend on the availability of the source branch for all operations."
+msgstr ""
+
+# help of 'standalone' option of 'branch' command
+#: bzrlib/builtins.py:1243
+msgid "Do not use a shared repository, even if available."
+msgstr ""
+
+# help of 'use-existing-dir' option of 'branch' command
+#: bzrlib/builtins.py:1245
+msgid ""
+"By default branch will fail if the target directory exists, but does not "
+"already have a control directory.  This flag will allow branch to proceed."
+msgstr ""
+
+# help of 'bind' option of 'branch' command
+#: bzrlib/builtins.py:1250
+msgid "Bind new branch to from location."
+msgstr ""
+
+#: bzrlib/builtins.py:1291
+#, python-format
+msgid "Target directory \"%s\" already exists."
+msgstr ""
+
+#: bzrlib/builtins.py:1301
+#, python-format
+msgid "Parent of \"%s\" does not exist."
+msgstr ""
+
+#: bzrlib/builtins.py:1315
+msgid "The branch {0} has no revision {1}."
+msgstr ""
+
+#: bzrlib/builtins.py:1323 bzrlib/push.py:51
+#, python-format
+msgid "Created new stacked branch referring to %s."
+msgstr ""
+
+#: bzrlib/builtins.py:1327
+#, python-format
+msgid "Branched %d revision."
+msgid_plural "Branched %d revisions."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:1332
+#, python-format
+msgid "New branch bound to %s"
+msgstr ""
+
+#: bzrlib/builtins.py:1337 bzrlib/builtins.py:5968
+#, python-format
+msgid "Switched to branch: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:1342
+msgid "List the branches available at the current location."
+msgstr ""
+
+#: bzrlib/builtins.py:1344
+msgid ""
+"This command will print the names of all the branches at the current "
+"location."
+msgstr ""
+
+#: bzrlib/builtins.py:1353
+msgid " (default)\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1359
+msgid "Create a new checkout of an existing branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1361
+msgid ""
+"If BRANCH_LOCATION is omitted, checkout will reconstitute a working tree "
+"for\n"
+"the branch found in '.'. This is useful if you have removed the working "
+"tree\n"
+"or if it was never created - i.e. if you pushed the branch to its current\n"
+"location using SFTP."
+msgstr ""
+
+#: bzrlib/builtins.py:1366
+msgid ""
+"If the TO_LOCATION is omitted, the last component of the BRANCH_LOCATION "
+"will\n"
+"be used.  In other words, \"checkout ../foo/bar\" will attempt to create ./"
+"bar.\n"
+"If the BRANCH_LOCATION has no / or path separator embedded, the TO_LOCATION\n"
+"is derived from the BRANCH_LOCATION by stripping a leading scheme or drive\n"
+"identifier, if any. For example, \"checkout lp:foo-bar\" will attempt to\n"
+"create ./foo-bar."
+msgstr ""
+
+#: bzrlib/builtins.py:1373
+msgid ""
+"To retrieve the branch as of a particular revision, supply the --revision\n"
+"parameter, as in \"checkout foo/bar -r 5\". Note that this will be "
+"immediately\n"
+"out of date [so you cannot commit] but it may be useful (i.e. to examine "
+"old\n"
+"code.)"
+msgstr ""
+
+# help of 'lightweight' option of 'checkout' command
+#: bzrlib/builtins.py:1383
+msgid ""
+"Perform a lightweight checkout.  Lightweight checkouts depend on access to "
+"the branch for every operation.  Normal checkouts can perform common "
+"operations like diff and status without such access, and also support local "
+"commits."
+msgstr ""
+
+# help of 'files-from' option of 'branch' command
+#: bzrlib/builtins.py:1390
+msgid "Get file contents from this tree."
+msgstr ""
+
+# help of 'hardlink' option of 'branch' command
+#: bzrlib/builtins.py:1392
+msgid "Hard-link working tree files where possible."
+msgstr ""
+
+#: bzrlib/builtins.py:1433
+msgid ""
+"Show list of renamed files.\n"
+"    "
+msgstr ""
+
+#: bzrlib/builtins.py:1463
+msgid "Update a tree to have the latest code committed to its branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1465
+msgid ""
+"This will perform a merge into the working tree, and may generate\n"
+"conflicts. If you have any local changes, you will still\n"
+"need to commit them after the update for the update to be complete."
+msgstr ""
+
+#: bzrlib/builtins.py:1469
+msgid ""
+"If you want to discard your local changes, you can just do a\n"
+"'bzr revert' instead of 'bzr commit' after the update."
+msgstr ""
+
+#: bzrlib/builtins.py:1472
+msgid ""
+"If you want to restore a file that has been removed locally, use\n"
+"'bzr revert' instead of 'bzr update'."
+msgstr ""
+
+#: bzrlib/builtins.py:1475
+msgid ""
+"If the tree's branch is bound to a master branch, it will also update\n"
+"the branch from the master."
+msgstr ""
+
+#: bzrlib/builtins.py:1490
+msgid "bzr update --revision takes exactly one revision"
+msgstr ""
+
+#: bzrlib/builtins.py:1521
+msgid "Tree is up to date at revision {0} of branch {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:1537
+#, python-format
+msgid ""
+"branch has no revision %s\n"
+"bzr update --revision only works for a revision in the branch history"
+msgstr ""
+
+#: bzrlib/builtins.py:1543
+msgid "Updated to revision {0} of branch {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:1547
+msgid ""
+"Your local commits will now show as pending merges with 'bzr status', and "
+"can be committed with 'bzr commit'."
+msgstr ""
+
+#: bzrlib/builtins.py:1556
+msgid "Show information about a working tree, branch or repository."
+msgstr ""
+
+#: bzrlib/builtins.py:1558
+msgid ""
+"This command will show all known locations and formats associated to the\n"
+"tree, branch or repository."
+msgstr ""
+
+#: bzrlib/builtins.py:1561
+msgid ""
+"In verbose mode, statistical information is included with each report.\n"
+"To see extended statistic information, use a verbosity level of 2 or\n"
+"higher by specifying the verbose option multiple times, e.g. -vv."
+msgstr ""
+
+#: bzrlib/builtins.py:1565
+msgid "Branches and working trees will also report any missing revisions."
+msgstr ""
+
+#: bzrlib/builtins.py:1569
+msgid "  Display information on the format and related locations:"
+msgstr ""
+
+#: bzrlib/builtins.py:1571
+msgid "    bzr info"
+msgstr ""
+
+#: bzrlib/builtins.py:1573
+msgid ""
+"  Display the above together with extended format information and\n"
+"  basic statistics (like the number of files in the working tree and\n"
+"  number of revisions in the branch and repository):"
+msgstr ""
+
+#: bzrlib/builtins.py:1577
+msgid "    bzr info -v"
+msgstr ""
+
+#: bzrlib/builtins.py:1579
+msgid "  Display the above together with number of committers to the branch:"
+msgstr ""
+
+#: bzrlib/builtins.py:1581
+msgid "    bzr info -vv"
+msgstr ""
+
+#: bzrlib/builtins.py:1600
+msgid "Remove files or directories."
+msgstr ""
+
+#: bzrlib/builtins.py:1602
+msgid ""
+"This makes Bazaar stop tracking changes to the specified files. Bazaar will\n"
+"delete them if they can easily be recovered using revert otherwise they\n"
+"will be backed up (adding an extention of the form .~#~). If no options or\n"
+"parameters are given Bazaar will scan for files that are being tracked by\n"
+"Bazaar but missing in your tree and stop tracking them for you."
+msgstr ""
+
+# help of 'new' option of 'remove' command
+#: bzrlib/builtins.py:1610
+msgid "Only remove files that have never been committed."
+msgstr ""
+
+# help of 'file-deletion-strategy' option of 'remove' command
+#: bzrlib/builtins.py:1612
+msgid "The file deletion mode to be used."
+msgstr ""
+
+# title of 'file-deletion-strategy' option of 'remove' command
+#: bzrlib/builtins.py:1613
+msgid "Deletion Strategy"
+msgstr ""
+
+#: bzrlib/builtins.py:1626
+msgid "(The --force option is deprecated, rather use --no-backup in future.)"
+msgstr ""
+
+#: bzrlib/builtins.py:1643
+msgid "No matching files."
+msgstr ""
+
+#: bzrlib/builtins.py:1704
+msgid "Reconcile bzr metadata in a branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1706
+msgid ""
+"This can correct data mismatches that may have been caused by\n"
+"previous ghost operations or bzr upgrades. You should only\n"
+"need to run this command if 'bzr check' or a bzr developer\n"
+"advises you to run it."
+msgstr ""
+
+#: bzrlib/builtins.py:1711
+msgid ""
+"If a second branch is provided, cross-branch reconciliation is\n"
+"also attempted, which will check that data like the tree root\n"
+"id which was not present in very early bzr versions is represented\n"
+"correctly in both branches."
+msgstr ""
+
+#: bzrlib/builtins.py:1716
+msgid ""
+"At the same time it is run it may recompress data resulting in\n"
+"a potential saving in disk space or performance gain."
+msgstr ""
+
+#: bzrlib/builtins.py:1719
+msgid "The branch *MUST* be on a listable system such as local disk or sftp."
+msgstr ""
+
+#: bzrlib/builtins.py:1783
+msgid "Make a directory into a versioned branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1785
+msgid ""
+"Use this to create an empty branch, or before importing an\n"
+"existing project."
+msgstr ""
+
+#: bzrlib/builtins.py:1788
+msgid ""
+"If there is a repository in a parent directory of the location, then\n"
+"the history of the branch will be stored in the repository.  Otherwise\n"
+"init creates a standalone branch which carries its own history\n"
+"in the .bzr directory."
+msgstr ""
+
+#: bzrlib/builtins.py:1793
+msgid ""
+"If there is already a branch at the location but it has no working tree,\n"
+"the tree can be populated with 'bzr checkout'."
+msgstr ""
+
+#: bzrlib/builtins.py:1796
+msgid "Recipe for importing a tree of files::"
+msgstr ""
+
+#: bzrlib/builtins.py:1798
+msgid ""
+"    cd ~/project\n"
+"    bzr init\n"
+"    bzr add .\n"
+"    bzr status\n"
+"    bzr commit -m \"imported project\""
+msgstr ""
+
+# help of 'create-prefix' option of 'init' command
+#: bzrlib/builtins.py:1809
+msgid "Create the path leading up to the branch if it does not already exist."
+msgstr ""
+
+# help of 'format' option of 'init' command
+#: bzrlib/builtins.py:1812
+msgid "Specify a format for this branch. See \"help formats\"."
+msgstr ""
+
+# help of 'append-revisions-only' option of 'init' command
+#: bzrlib/builtins.py:1820
+msgid "Never change revnos or the existing log.  Append revisions to it only."
+msgstr ""
+
+# help of 'no-tree' option of 'init' command
+#: bzrlib/builtins.py:1823
+msgid "Create a branch without a working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:1843 bzrlib/push.py:108
+#, python-format
+msgid ""
+"Parent directory of %s does not exist.\n"
+"You may supply --create-prefix to create all leading parent directories."
+msgstr ""
+
+#: bzrlib/builtins.py:1877
+msgid ""
+"This branch format cannot be set to append-revisions-only.  Try --default."
+msgstr ""
+
+#: bzrlib/builtins.py:1888
+msgid "Created a {0} (format: {1})\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1897
+#, python-format
+msgid "Using shared repository: %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1901
+msgid "Create a shared repository for branches to share storage space."
+msgstr ""
+
+#: bzrlib/builtins.py:1903
+msgid ""
+"New branches created under the repository directory will store their\n"
+"revisions in the repository, not in the branch directory.  For branches\n"
+"with shared history, this reduces the amount of storage needed and \n"
+"speeds up the creation of new branches."
+msgstr ""
+
+#: bzrlib/builtins.py:1908
+msgid ""
+"If the --no-trees option is given then the branches in the repository\n"
+"will not have working trees by default.  They will still exist as \n"
+"directories on disk, but they will not have separate copies of the \n"
+"files at a certain revision.  This can be useful for repositories that\n"
+"store branches which are interacted with through checkouts or remote\n"
+"branches, such as on a server."
+msgstr ""
+
+#: bzrlib/builtins.py:1915
+msgid ""
+":Examples:\n"
+"    Create a shared repository holding just branches::"
+msgstr ""
+
+#: bzrlib/builtins.py:1918
+msgid ""
+"        bzr init-repo --no-trees repo\n"
+"        bzr init repo/trunk"
+msgstr ""
+
+#: bzrlib/builtins.py:1921
+msgid "    Make a lightweight checkout elsewhere::"
+msgstr ""
+
+#: bzrlib/builtins.py:1923
+msgid ""
+"        bzr checkout --lightweight repo/trunk trunk-checkout\n"
+"        cd trunk-checkout\n"
+"        (add files here)"
+msgstr ""
+
+# help of 'format' option of 'init-repository' command
+#: bzrlib/builtins.py:1931
+msgid ""
+"Specify a format for this repository. See \"bzr help formats\" for details."
+msgstr ""
+
+# title of 'format' option of 'init-repository' command
+#: bzrlib/builtins.py:1935
+msgid "Repository format"
+msgstr ""
+
+# help of 'no-trees' option of 'init-repository' command
+#: bzrlib/builtins.py:1937
+msgid "Branches in the repository will default to not having a working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:1961
+msgid "Show differences in the working tree, between revisions or branches."
+msgstr ""
+
+#: bzrlib/builtins.py:1963
+msgid ""
+"If no arguments are given, all changes for the current tree are listed.\n"
+"If files are given, only the changes in those files are listed.\n"
+"Remote and multiple branches can be compared by using the --old and\n"
+"--new options. If not provided, the default for both is derived from\n"
+"the first argument, if any, or the current tree if no arguments are\n"
+"given."
+msgstr ""
+
+#: bzrlib/builtins.py:1970
+msgid ""
+"\"bzr diff -p1\" is equivalent to \"bzr diff --prefix old/:new/\", and\n"
+"produces patches suitable for \"patch -p1\"."
+msgstr ""
+
+#: bzrlib/builtins.py:1973
+msgid ""
+"Note that when using the -r argument with a range of revisions, the\n"
+"differences are computed between the two specified revisions.  That\n"
+"is, the command does not show the changes introduced by the first \n"
+"revision in the range.  This differs from the interpretation of \n"
+"revision ranges used by \"bzr log\" which includes the first revision\n"
+"in the range."
+msgstr ""
+
+#: bzrlib/builtins.py:1980
+msgid ""
+":Exit values:\n"
+"    1 - changed\n"
+"    2 - unrepresentable changes\n"
+"    3 - error\n"
+"    0 - no change"
+msgstr ""
+
+#: bzrlib/builtins.py:1986
+msgid ""
+":Examples:\n"
+"    Shows the difference in the working tree versus the last commit::"
+msgstr ""
+
+#: bzrlib/builtins.py:1989
+msgid "        bzr diff"
+msgstr ""
+
+#: bzrlib/builtins.py:1991
+msgid "    Difference between the working tree and revision 1::"
+msgstr ""
+
+#: bzrlib/builtins.py:1993
+msgid "        bzr diff -r1"
+msgstr ""
+
+#: bzrlib/builtins.py:1995
+msgid "    Difference between revision 3 and revision 1::"
+msgstr ""
+
+#: bzrlib/builtins.py:1997
+msgid "        bzr diff -r1..3"
+msgstr ""
+
+#: bzrlib/builtins.py:1999
+msgid "    Difference between revision 3 and revision 1 for branch xxx::"
+msgstr ""
+
+#: bzrlib/builtins.py:2001
+msgid "        bzr diff -r1..3 xxx"
+msgstr ""
+
+#: bzrlib/builtins.py:2003
+msgid "    The changes introduced by revision 2 (equivalent to -r1..2)::"
+msgstr ""
+
+#: bzrlib/builtins.py:2005
+msgid "        bzr diff -c2"
+msgstr ""
+
+#: bzrlib/builtins.py:2007
+msgid ""
+"    To see the changes introduced by revision X::\n"
+"    \n"
+"        bzr diff -cX"
+msgstr ""
+
+#: bzrlib/builtins.py:2011
+msgid ""
+"    Note that in the case of a merge, the -c option shows the changes\n"
+"    compared to the left hand parent. To see the changes against\n"
+"    another parent, use::"
+msgstr ""
+
+#: bzrlib/builtins.py:2015
+msgid "        bzr diff -r<chosen_parent>..X"
+msgstr ""
+
+#: bzrlib/builtins.py:2017
+msgid ""
+"    The changes between the current revision and the previous revision\n"
+"    (equivalent to -c-1 and -r-2..-1)"
+msgstr ""
+
+#: bzrlib/builtins.py:2020
+msgid "        bzr diff -r-2.."
+msgstr ""
+
+#: bzrlib/builtins.py:2022
+msgid "    Show just the differences for file NEWS::"
+msgstr ""
+
+#: bzrlib/builtins.py:2024
+msgid "        bzr diff NEWS"
+msgstr ""
+
+#: bzrlib/builtins.py:2026
+msgid "    Show the differences in working tree xxx for file NEWS::"
+msgstr ""
+
+#: bzrlib/builtins.py:2028
+msgid "        bzr diff xxx/NEWS"
+msgstr ""
+
+#: bzrlib/builtins.py:2030
+msgid "    Show the differences from branch xxx to this working tree:"
+msgstr ""
+
+#: bzrlib/builtins.py:2032
+msgid "        bzr diff --old xxx"
+msgstr ""
+
+#: bzrlib/builtins.py:2034
+msgid "    Show the differences between two branches for file NEWS::"
+msgstr ""
+
+#: bzrlib/builtins.py:2036
+msgid "        bzr diff --old xxx --new yyy NEWS"
+msgstr ""
+
+#: bzrlib/builtins.py:2038
+msgid "    Same as 'bzr diff' but prefix paths with old/ and new/::"
+msgstr ""
+
+#: bzrlib/builtins.py:2040
+msgid ""
+"        bzr diff --prefix old/:new/\n"
+"        \n"
+"    Show the differences using a custom diff program with options::\n"
+"    \n"
+"        bzr diff --using /usr/bin/diff --diff-options -wu"
+msgstr ""
+
+# help of 'diff-options' option of 'diff' command
+#: bzrlib/builtins.py:2050
+msgid "Pass these options to the external diff program."
+msgstr ""
+
+# help of 'prefix' option of 'diff' command
+#: bzrlib/builtins.py:2053
+msgid ""
+"Set prefixes added to old and new filenames, as two values separated by a "
+"colon. (eg \"old/:new/\")."
+msgstr ""
+
+# help of 'old' option of 'diff' command
+#: bzrlib/builtins.py:2056
+msgid "Branch/tree to compare from."
+msgstr ""
+
+# help of 'new' option of 'diff' command
+#: bzrlib/builtins.py:2060
+msgid "Branch/tree to compare to."
+msgstr ""
+
+# help of 'using' option of 'diff' command
+#: bzrlib/builtins.py:2066
+msgid "Use this command to compare files."
+msgstr ""
+
+# help of 'format' option of 'diff' command
+#: bzrlib/builtins.py:2071
+msgid "Diff format to use."
+msgstr ""
+
+# title of 'format' option of 'diff' command
+#: bzrlib/builtins.py:2073
+msgid "Diff format"
+msgstr ""
+
+#: bzrlib/builtins.py:2095
+msgid "--prefix expects two values separated by a colon (eg \"old/:new/\")"
+msgstr ""
+
+#: bzrlib/builtins.py:2099
+msgid "bzr diff --revision takes exactly one or two revision specifiers"
+msgstr ""
+
+#: bzrlib/builtins.py:2104 bzrlib/builtins.py:2502 bzrlib/builtins.py:2515
+#: bzrlib/builtins.py:2523 bzrlib/builtins.py:4663
+msgid "{0} and {1} are mutually exclusive"
+msgstr ""
+
+#: bzrlib/builtins.py:2124
+msgid ""
+"List files deleted in the working tree.\n"
+"    "
+msgstr ""
+
+#: bzrlib/builtins.py:2202
+msgid "Show the tree root directory."
+msgstr ""
+
+#: bzrlib/builtins.py:2204
+msgid ""
+"The root is the nearest enclosing directory with a .bzr control\n"
+"directory."
+msgstr ""
+
+#: bzrlib/builtins.py:2219
+msgid "The limit argument must be an integer."
+msgstr ""
+
+#: bzrlib/builtins.py:2227
+msgid "The levels argument must be an integer."
+msgstr ""
+
+#: bzrlib/builtins.py:2232
+msgid "Show historical log for a branch or subset of a branch."
+msgstr ""
+
+#: bzrlib/builtins.py:2234
+msgid ""
+"log is bzr's default tool for exploring the history of a branch.\n"
+"The branch to use is taken from the first parameter. If no parameters\n"
+"are given, the branch containing the working directory is logged.\n"
+"Here are some simple examples::"
+msgstr ""
+
+#: bzrlib/builtins.py:2239
+msgid ""
+"  bzr log                       log the current branch\n"
+"  bzr log foo.py                log a file in its branch\n"
+"  bzr log http://server/branch  log a branch on a server"
+msgstr ""
+
+#: bzrlib/builtins.py:2243
+msgid ""
+"The filtering, ordering and information shown for each revision can\n"
+"be controlled as explained below. By default, all revisions are\n"
+"shown sorted (topologically) so that newer revisions appear before\n"
+"older ones and descendants always appear before ancestors. If displayed,\n"
+"merged revisions are shown indented under the revision in which they\n"
+"were merged."
+msgstr ""
+
+#: bzrlib/builtins.py:2250
+msgid ":Output control:"
+msgstr ""
+
+#: bzrlib/builtins.py:2252
+msgid ""
+"  The log format controls how information about each revision is\n"
+"  displayed. The standard log formats are called ``long``, ``short``\n"
+"  and ``line``. The default is long. See ``bzr help log-formats``\n"
+"  for more details on log formats."
+msgstr ""
+
+#: bzrlib/builtins.py:2257
+msgid ""
+"  The following options can be used to control what information is\n"
+"  displayed::"
+msgstr ""
+
+#: bzrlib/builtins.py:2260
+msgid ""
+"    -l N        display a maximum of N revisions\n"
+"    -n N        display N levels of revisions (0 for all, 1 for collapsed)\n"
+"    -v          display a status summary (delta) for each revision\n"
+"    -p          display a diff (patch) for each revision\n"
+"    --show-ids  display revision-ids (and file-ids), not just revnos"
+msgstr ""
+
+#: bzrlib/builtins.py:2266
+msgid ""
+"  Note that the default number of levels to display is a function of the\n"
+"  log format. If the -n option is not used, the standard log formats show\n"
+"  just the top level (mainline)."
+msgstr ""
+
+#: bzrlib/builtins.py:2270
+msgid ""
+"  Status summaries are shown using status flags like A, M, etc. To see\n"
+"  the changes explained using words like ``added`` and ``modified``\n"
+"  instead, use the -vv option."
+msgstr ""
+
+#: bzrlib/builtins.py:2274
+msgid ":Ordering control:"
+msgstr ""
+
+#: bzrlib/builtins.py:2276
+msgid ""
+"  To display revisions from oldest to newest, use the --forward option.\n"
+"  In most cases, using this option will have little impact on the total\n"
+"  time taken to produce a log, though --forward does not incrementally\n"
+"  display revisions like --reverse does when it can."
+msgstr ""
+
+#: bzrlib/builtins.py:2281
+msgid ":Revision filtering:"
+msgstr ""
+
+#: bzrlib/builtins.py:2283
+msgid ""
+"  The -r option can be used to specify what revision or range of revisions\n"
+"  to filter against. The various forms are shown below::"
+msgstr ""
+
+#: bzrlib/builtins.py:2286
+msgid ""
+"    -rX      display revision X\n"
+"    -rX..    display revision X and later\n"
+"    -r..Y    display up to and including revision Y\n"
+"    -rX..Y   display from X to Y inclusive"
+msgstr ""
+
+#: bzrlib/builtins.py:2291
+msgid ""
+"  See ``bzr help revisionspec`` for details on how to specify X and Y.\n"
+"  Some common examples are given below::"
+msgstr ""
+
+#: bzrlib/builtins.py:2294
+msgid ""
+"    -r-1                show just the tip\n"
+"    -r-10..             show the last 10 mainline revisions\n"
+"    -rsubmit:..         show what's new on this branch\n"
+"    -rancestor:path..   show changes since the common ancestor of this\n"
+"                        branch and the one at location path\n"
+"    -rdate:yesterday..  show changes since yesterday"
+msgstr ""
+
+#: bzrlib/builtins.py:2301
+msgid ""
+"  When logging a range of revisions using -rX..Y, log starts at\n"
+"  revision Y and searches back in history through the primary\n"
+"  (\"left-hand\") parents until it finds X. When logging just the\n"
+"  top level (using -n1), an error is reported if X is not found\n"
+"  along the way. If multi-level logging is used (-n0), X may be\n"
+"  a nested merge revision and the log will be truncated accordingly."
+msgstr ""
+
+#: bzrlib/builtins.py:2308
+msgid ":Path filtering:"
+msgstr ""
+
+#: bzrlib/builtins.py:2310
+msgid ""
+"  If parameters are given and the first one is not a branch, the log\n"
+"  will be filtered to show only those revisions that changed the\n"
+"  nominated files or directories."
+msgstr ""
+
+#: bzrlib/builtins.py:2314
+msgid ""
+"  Filenames are interpreted within their historical context. To log a\n"
+"  deleted file, specify a revision range so that the file existed at\n"
+"  the end or start of the range."
+msgstr ""
+
+#: bzrlib/builtins.py:2318
+msgid ""
+"  Historical context is also important when interpreting pathnames of\n"
+"  renamed files/directories. Consider the following example:"
+msgstr ""
+
+#: bzrlib/builtins.py:2321
+msgid ""
+"  * revision 1: add tutorial.txt\n"
+"  * revision 2: modify tutorial.txt\n"
+"  * revision 3: rename tutorial.txt to guide.txt; add tutorial.txt"
+msgstr ""
+
+#: bzrlib/builtins.py:2325
+msgid "  In this case:"
+msgstr ""
+
+#: bzrlib/builtins.py:2327
+msgid "  * ``bzr log guide.txt`` will log the file added in revision 1"
+msgstr ""
+
+#: bzrlib/builtins.py:2329
+msgid "  * ``bzr log tutorial.txt`` will log the new file added in revision 3"
+msgstr ""
+
+#: bzrlib/builtins.py:2331
+msgid ""
+"  * ``bzr log -r2 -p tutorial.txt`` will show the changes made to\n"
+"    the original file in revision 2."
+msgstr ""
+
+#: bzrlib/builtins.py:2334
+msgid ""
+"  * ``bzr log -r2 -p guide.txt`` will display an error message as there\n"
+"    was no file called guide.txt in revision 2."
+msgstr ""
+
+#: bzrlib/builtins.py:2337
+msgid ""
+"  Renames are always followed by log. By design, there is no need to\n"
+"  explicitly ask for this (and no way to stop logging a file back\n"
+"  until it was last renamed)."
+msgstr ""
+
+#: bzrlib/builtins.py:2341
+msgid ":Other filtering:"
+msgstr ""
+
+#: bzrlib/builtins.py:2343
+msgid ""
+"  The --match option can be used for finding revisions that match a\n"
+"  regular expression in a commit message, committer, author or bug.\n"
+"  Specifying the option several times will match any of the supplied\n"
+"  expressions. --match-author, --match-bugs, --match-committer and\n"
+"  --match-message can be used to only match a specific field."
+msgstr ""
+
+#: bzrlib/builtins.py:2349
+msgid ":Tips & tricks:"
+msgstr ""
+
+#: bzrlib/builtins.py:2351
+msgid ""
+"  GUI tools and IDEs are often better at exploring history than command\n"
+"  line tools: you may prefer qlog or viz from qbzr or bzr-gtk, the\n"
+"  bzr-explorer shell, or the Loggerhead web interface.  See the Plugin\n"
+"  Guide <http://doc.bazaar.canonical.com/plugins/en/> and\n"
+"  <http://wiki.bazaar.canonical.com/IDEIntegration>.  "
+msgstr ""
+
+#: bzrlib/builtins.py:2357
+msgid "  You may find it useful to add the aliases below to ``bazaar.conf``::"
+msgstr ""
+
+#: bzrlib/builtins.py:2359
+msgid ""
+"    [ALIASES]\n"
+"    tip = log -r-1\n"
+"    top = log -l10 --line\n"
+"    show = log -v -p"
+msgstr ""
+
+#: bzrlib/builtins.py:2364
+msgid ""
+"  ``bzr tip`` will then show the latest revision while ``bzr top``\n"
+"  will show the last 10 mainline revisions. To see the details of a\n"
+"  particular revision X,  ``bzr show -rX``."
+msgstr ""
+
+#: bzrlib/builtins.py:2368
+msgid ""
+"  If you are interested in looking deeper into a particular merge X,\n"
+"  use ``bzr log -n0 -rX``."
+msgstr ""
+
+#: bzrlib/builtins.py:2371
+msgid ""
+"  ``bzr log -v`` on a branch with lots of history is currently\n"
+"  very slow. A fix for this issue is currently under development.\n"
+"  With or without that fix, it is recommended that a revision range\n"
+"  be given when using the -v option."
+msgstr ""
+
+#: bzrlib/builtins.py:2376
+msgid ""
+"  bzr has a generic full-text matching plugin, bzr-search, that can be\n"
+"  used to find revisions matching user names, commit messages, etc.\n"
+"  Among other features, this plugin can find all revisions containing\n"
+"  a list of words but not others."
+msgstr ""
+
+#: bzrlib/builtins.py:2381
+msgid ""
+"  When exploring non-mainline history on large projects with deep\n"
+"  history, the performance of log can be greatly improved by installing\n"
+"  the historycache plugin. This plugin buffers historical information\n"
+"  trading disk space for faster speed."
+msgstr ""
+
+# help of 'forward' option of 'log' command
+#: bzrlib/builtins.py:2390
+msgid "Show from oldest to newest."
+msgstr ""
+
+# help of 'verbose' option of 'log' command
+#: bzrlib/builtins.py:2393
+msgid "Show files changed in each revision."
+msgstr ""
+
+# help of 'change' option of 'log' command
+#: bzrlib/builtins.py:2399
+msgid "Show just the specified revision. See also \"help revisionspec\"."
+msgstr ""
+
+# help of 'authors' option of 'log' command
+#: bzrlib/builtins.py:2403
+msgid "What names to list as authors - first, all or committer."
+msgstr ""
+
+# title of 'authors' option of 'log' command
+#: bzrlib/builtins.py:2404
+msgid "Authors"
+msgstr ""
+
+# help of 'levels' option of 'log' command
+#: bzrlib/builtins.py:2409
+msgid "Number of levels to display - 0 for all, 1 for flat."
+msgstr ""
+
+# help of 'limit' option of 'log' command
+#: bzrlib/builtins.py:2419
+msgid "Limit the output to the first N revisions."
+msgstr ""
+
+# help of 'show-diff' option of 'log' command
+#: bzrlib/builtins.py:2424
+msgid "Show changes made in each revision as a patch."
+msgstr ""
+
+# help of 'include-merged' option of 'log' command
+#: bzrlib/builtins.py:2426
+msgid "Show merged revisions like --levels 0 does."
+msgstr ""
+
+# help of 'omit-merges' option of 'log' command
+#: bzrlib/builtins.py:2430
+msgid "Do not report commits with more than one parent."
+msgstr ""
+
+# help of 'exclude-common-ancestry' option of 'log' command
+#: bzrlib/builtins.py:2432
+msgid ""
+"Display only the revisions that are not part of both ancestries (require -"
+"rX..Y)"
+msgstr ""
+
+# help of 'signatures' option of 'log' command
+#: bzrlib/builtins.py:2436
+msgid "Show digital signature validity"
+msgstr ""
+
+# help of 'match' option of 'log' command
+#: bzrlib/builtins.py:2439
+msgid "Show revisions whose properties match this expression."
+msgstr ""
+
+# help of 'match-message' option of 'log' command
+#: bzrlib/builtins.py:2443
+msgid "Show revisions whose message matches this expression."
+msgstr ""
+
+# help of 'match-committer' option of 'log' command
+#: bzrlib/builtins.py:2447
+msgid "Show revisions whose committer matches this expression."
+msgstr ""
+
+# help of 'match-author' option of 'log' command
+#: bzrlib/builtins.py:2451
+msgid "Show revisions whose authors match this expression."
+msgstr ""
+
+# help of 'match-bugs' option of 'log' command
+#: bzrlib/builtins.py:2455
+msgid "Show revisions whose bugs match this expression."
+msgstr ""
+
+#: bzrlib/builtins.py:2509
+msgid "--exclude-common-ancestry requires -r with two revisions"
+msgstr ""
+
+#: bzrlib/builtins.py:2537
+#, python-format
+msgid "Path unknown at end or start of revision range: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:2654
+#, python-format
+msgid "bzr %s doesn't accept two revisions in different branches."
+msgstr ""
+
+#: bzrlib/builtins.py:2670
+#, python-format
+msgid "bzr %s --revision takes one or two values."
+msgstr ""
+
+#: bzrlib/builtins.py:2715
+msgid ""
+"List files in a tree.\n"
+"    "
+msgstr ""
+
+# help of 'recursive' option of 'ls' command
+#: bzrlib/builtins.py:2724
+msgid "Recurse into subdirectories."
+msgstr ""
+
+# help of 'from-root' option of 'ls' command
+#: bzrlib/builtins.py:2726
+msgid "Print paths relative to the root of the branch."
+msgstr ""
+
+# help of 'unknown' option of 'ls' command
+#: bzrlib/builtins.py:2728
+msgid "Print unknown files."
+msgstr ""
+
+# help of 'versioned' option of 'ls' command
+#: bzrlib/builtins.py:2729
+msgid "Print versioned files."
+msgstr ""
+
+# help of 'ignored' option of 'ls' command
+#: bzrlib/builtins.py:2732
+msgid "Print ignored files."
+msgstr ""
+
+# help of 'kind' option of 'ls' command
+#: bzrlib/builtins.py:2734
+msgid "List entries of a particular kind: file, directory, symlink."
+msgstr ""
+
+#: bzrlib/builtins.py:2747
+msgid "invalid kind specified"
+msgstr ""
+
+#: bzrlib/builtins.py:2750
+msgid "Cannot set both --verbose and --null"
+msgstr ""
+
+#: bzrlib/builtins.py:2759
+msgid "cannot specify both --from-root and PATH"
+msgstr ""
+
+#: bzrlib/builtins.py:2846
+msgid "Ignore specified files or patterns."
+msgstr ""
+
+#: bzrlib/builtins.py:2848
+msgid "See ``bzr help patterns`` for details on the syntax of patterns."
+msgstr ""
+
+#: bzrlib/builtins.py:2850
+msgid ""
+"If a .bzrignore file does not exist, the ignore command\n"
+"will create one and add the specified files or patterns to the newly\n"
+"created file. The ignore command will also automatically add the \n"
+".bzrignore file to be versioned. Creating a .bzrignore file without\n"
+"the use of the ignore command will require an explicit add command."
+msgstr ""
+
+#: bzrlib/builtins.py:2856
+msgid ""
+"To remove patterns from the ignore list, edit the .bzrignore file.\n"
+"After adding, editing or deleting that file either indirectly by\n"
+"using this command or directly by using an editor, be sure to commit\n"
+"it."
+msgstr ""
+
+#: bzrlib/builtins.py:2861
+msgid ""
+"Bazaar also supports a global ignore file ~/.bazaar/ignore. On Windows\n"
+"the global ignore file can be found in the application data directory as\n"
+"C:\\Documents and Settings\\<user>\\Application Data\\Bazaar\\2.0\\ignore.\n"
+"Global ignores are not touched by this command. The global ignore file\n"
+"can be edited directly using an editor."
+msgstr ""
+
+#: bzrlib/builtins.py:2867
+msgid ""
+"Patterns prefixed with '!' are exceptions to ignore patterns and take\n"
+"precedence over regular ignores.  Such exceptions are used to specify\n"
+"files that should be versioned which would otherwise be ignored."
+msgstr ""
+
+#: bzrlib/builtins.py:2871
+msgid ""
+"Patterns prefixed with '!!' act as regular ignore patterns, but have\n"
+"precedence over the '!' exception patterns."
+msgstr ""
+
+#: bzrlib/builtins.py:2874
+msgid ""
+":Notes: \n"
+"    \n"
+"* Ignore patterns containing shell wildcards must be quoted from\n"
+"  the shell on Unix."
+msgstr ""
+
+#: bzrlib/builtins.py:2879
+msgid ""
+"* Ignore patterns starting with \"#\" act as comments in the ignore file.\n"
+"  To ignore patterns that begin with that character, use the \"RE:\" prefix."
+msgstr ""
+
+#: bzrlib/builtins.py:2882
+msgid ""
+":Examples:\n"
+"    Ignore the top level Makefile::"
+msgstr ""
+
+#: bzrlib/builtins.py:2885
+msgid "        bzr ignore ./Makefile"
+msgstr ""
+
+#: bzrlib/builtins.py:2887
+msgid "    Ignore .class files in all directories...::"
+msgstr ""
+
+#: bzrlib/builtins.py:2889
+msgid "        bzr ignore \"*.class\""
+msgstr ""
+
+#: bzrlib/builtins.py:2891
+msgid "    ...but do not ignore \"special.class\"::"
+msgstr ""
+
+#: bzrlib/builtins.py:2893
+msgid "        bzr ignore \"!special.class\""
+msgstr ""
+
+#: bzrlib/builtins.py:2895
+msgid "    Ignore files whose name begins with the \"#\" character::"
+msgstr ""
+
+#: bzrlib/builtins.py:2897
+msgid "        bzr ignore \"RE:^#\""
+msgstr ""
+
+#: bzrlib/builtins.py:2899
+msgid "    Ignore .o files under the lib directory::"
+msgstr ""
+
+#: bzrlib/builtins.py:2901
+msgid "        bzr ignore \"lib/**/*.o\""
+msgstr ""
+
+#: bzrlib/builtins.py:2905
+msgid "        bzr ignore \"RE:lib/.*\\.o\""
+msgstr ""
+
+#: bzrlib/builtins.py:2907
+msgid "    Ignore everything but the \"debian\" toplevel directory::"
+msgstr ""
+
+#: bzrlib/builtins.py:2909
+msgid ""
+"        bzr ignore \"RE:(?!debian/).*\"\n"
+"    \n"
+"    Ignore everything except the \"local\" toplevel directory,\n"
+"    but always ignore autosave files ending in ~, even under local/::\n"
+"    \n"
+"        bzr ignore \"*\"\n"
+"        bzr ignore \"!./local\"\n"
+"        bzr ignore \"!!*~\""
+msgstr ""
+
+# help of 'default-rules' option of 'ignore' command
+#: bzrlib/builtins.py:2923
+msgid "Display the default ignore rules that bzr uses."
+msgstr ""
+
+#: bzrlib/builtins.py:2935
+msgid "ignore requires at least one NAME_PATTERN or --default-rules."
+msgstr ""
+
+#: bzrlib/builtins.py:2946
+#, python-format
+msgid "Invalid ignore pattern found. %s"
+msgid_plural "Invalid ignore patterns found. %s"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:2955
+msgid "NAME_PATTERN should not be an absolute path"
+msgstr ""
+
+#: bzrlib/builtins.py:2968
+#, python-format
+msgid ""
+"Warning: the following files are version controlled and match your ignore "
+"pattern:\n"
+"%s\n"
+"These files will continue to be version controlled unless you 'bzr remove' "
+"them.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:2975
+msgid "List ignored files and the patterns that matched them."
+msgstr ""
+
+#: bzrlib/builtins.py:2977
+msgid ""
+"List all the ignored files and the ignore pattern that caused the file to\n"
+"be ignored."
+msgstr ""
+
+#: bzrlib/builtins.py:2980
+msgid "Alternatively, to list just the files::"
+msgstr ""
+
+#: bzrlib/builtins.py:2982
+msgid "    bzr ls --ignored"
+msgstr ""
+
+#: bzrlib/builtins.py:3016
+#, python-format
+msgid "not a valid revision-number: %r"
+msgstr ""
+
+#: bzrlib/builtins.py:3023
+msgid "Export current or past revision to a destination directory or archive."
+msgstr ""
+
+#: bzrlib/builtins.py:3025
+msgid "If no revision is specified this exports the last committed revision."
+msgstr ""
+
+#: bzrlib/builtins.py:3027
+msgid ""
+"Format may be an \"exporter\" name, such as tar, tgz, tbz2.  If none is\n"
+"given, try to find the format with the extension. If no extension\n"
+"is found exports to a directory (equivalent to --format=dir)."
+msgstr ""
+
+#: bzrlib/builtins.py:3031
+msgid ""
+"If root is supplied, it will be used as the root directory inside\n"
+"container formats (tar, zip, etc). If it is not supplied it will default\n"
+"to the exported filename. The root option has no effect for 'dir' format."
+msgstr ""
+
+#: bzrlib/builtins.py:3035
+msgid ""
+"If branch is omitted then the branch containing the current working\n"
+"directory will be used."
+msgstr ""
+
+#: bzrlib/builtins.py:3038
+msgid "Note: Export of tree with non-ASCII filenames to zip is not supported."
+msgstr ""
+
+#: bzrlib/builtins.py:3040
+msgid ""
+"  =================       =========================\n"
+"  Supported formats       Autodetected by extension\n"
+"  =================       =========================\n"
+"     dir                         (none)\n"
+"     tar                          .tar\n"
+"     tbz2                    .tar.bz2, .tbz2\n"
+"     tgz                      .tar.gz, .tgz\n"
+"     zip                          .zip\n"
+"  =================       ========================="
+msgstr ""
+
+# help of 'format' option of 'export' command
+#: bzrlib/builtins.py:3054
+msgid "Type of file to export to."
+msgstr ""
+
+# help of 'filters' option of 'export' command
+#: bzrlib/builtins.py:3057
+msgid "Apply content filters to export the convenient form."
+msgstr ""
+
+# help of 'root' option of 'export' command
+#: bzrlib/builtins.py:3061
+msgid "Name of the root directory inside the exported file."
+msgstr ""
+
+# help of 'per-file-timestamps' option of 'export' command
+#: bzrlib/builtins.py:3063
+msgid ""
+"Set modification time of files to that of the last revision in which it was "
+"changed."
+msgstr ""
+
+#: bzrlib/builtins.py:3083
+#, python-format
+msgid "Unsupported export format: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:3087
+msgid "Write the contents of a file as of a given revision to standard output."
+msgstr ""
+
+#: bzrlib/builtins.py:3089
+msgid "If no revision is nominated, the last revision is used."
+msgstr ""
+
+#: bzrlib/builtins.py:3091
+msgid ""
+"Note: Take care to redirect standard output when using this command on a\n"
+"binary file."
+msgstr ""
+
+# help of 'filters' option of 'cat' command
+#: bzrlib/builtins.py:3098
+msgid "Apply content filters to display the convenience form."
+msgstr ""
+
+#: bzrlib/builtins.py:3109
+msgid "bzr cat --revision takes exactly one revision specifier"
+msgstr ""
+
+#: bzrlib/builtins.py:3134 bzrlib/builtins.py:3146
+msgid "{0!r} is not present in revision {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:3169
+msgid "Commit changes into a new revision."
+msgstr ""
+
+#: bzrlib/builtins.py:3171
+msgid ""
+"An explanatory message needs to be given for each commit. This is\n"
+"often done by using the --message option (getting the message from the\n"
+"command line) or by using the --file option (getting the message from\n"
+"a file). If neither of these options is given, an editor is opened for\n"
+"the user to enter the message. To see the changed files in the\n"
+"boilerplate text loaded into the editor, use the --show-diff option."
+msgstr ""
+
+#: bzrlib/builtins.py:3178
+msgid ""
+"By default, the entire tree is committed and the person doing the\n"
+"commit is assumed to be the author. These defaults can be overridden\n"
+"as explained below."
+msgstr ""
+
+#: bzrlib/builtins.py:3182
+msgid ":Selective commits:"
+msgstr ""
+
+#: bzrlib/builtins.py:3184
+msgid ""
+"  If selected files are specified, only changes to those files are\n"
+"  committed.  If a directory is specified then the directory and\n"
+"  everything within it is committed."
+msgstr ""
+
+#: bzrlib/builtins.py:3188
+msgid ""
+"  When excludes are given, they take precedence over selected files.\n"
+"  For example, to commit only changes within foo, but not changes\n"
+"  within foo/bar::"
+msgstr ""
+
+#: bzrlib/builtins.py:3192
+msgid "    bzr commit foo -x foo/bar"
+msgstr ""
+
+#: bzrlib/builtins.py:3194
+msgid "  A selective commit after a merge is not yet supported."
+msgstr ""
+
+#: bzrlib/builtins.py:3196
+msgid ":Custom authors:"
+msgstr ""
+
+#: bzrlib/builtins.py:3198
+msgid ""
+"  If the author of the change is not the same person as the committer,\n"
+"  you can specify the author's name using the --author option. The\n"
+"  name should be in the same format as a committer-id, e.g.\n"
+"  \"John Doe <jdoe@example.com>\". If there is more than one author of\n"
+"  the change you can specify the option multiple times, once for each\n"
+"  author."
+msgstr ""
+
+#: bzrlib/builtins.py:3205
+msgid ":Checks:"
+msgstr ""
+
+#: bzrlib/builtins.py:3207
+msgid ""
+"  A common mistake is to forget to add a new file or directory before\n"
+"  running the commit command. The --strict option checks for unknown\n"
+"  files and aborts the commit if any are found. More advanced pre-commit\n"
+"  checks can be implemented by defining hooks. See ``bzr help hooks``\n"
+"  for details."
+msgstr ""
+
+#: bzrlib/builtins.py:3213
+msgid ":Things to note:"
+msgstr ""
+
+#: bzrlib/builtins.py:3215
+msgid ""
+"  If you accidentially commit the wrong changes or make a spelling\n"
+"  mistake in the commit message say, you can use the uncommit command\n"
+"  to undo it. See ``bzr help uncommit`` for details."
+msgstr ""
+
+#: bzrlib/builtins.py:3219
+msgid ""
+"  Hooks can also be configured to run after a commit. This allows you\n"
+"  to trigger updates to external systems like bug trackers. The --fixes\n"
+"  option can be used to record the association between a revision and\n"
+"  one or more bugs. See ``bzr help bugs`` for details."
+msgstr ""
+
+# help of 'exclude' option of 'commit' command
+#: bzrlib/builtins.py:3229
+msgid "Do not consider changes made to a given path."
+msgstr ""
+
+# help of 'message' option of 'commit' command
+#: bzrlib/builtins.py:3232
+msgid "Description of the new revision."
+msgstr ""
+
+# help of 'unchanged' option of 'commit' command
+#: bzrlib/builtins.py:3235
+msgid "Commit even if nothing has changed."
+msgstr ""
+
+# help of 'file' option of 'commit' command
+#: bzrlib/builtins.py:3239
+msgid "Take commit message from this file."
+msgstr ""
+
+# help of 'strict' option of 'commit' command
+#: bzrlib/builtins.py:3241
+msgid "Refuse to commit if there are unknown files in the working tree."
+msgstr ""
+
+# help of 'commit-time' option of 'commit' command
+#: bzrlib/builtins.py:3244
+msgid ""
+"Manually set a commit time using commit date format, e.g. '2009-10-10 "
+"08:00:00 +0100'."
+msgstr ""
+
+# help of 'fixes' option of 'commit' command
+#: bzrlib/builtins.py:3247
+msgid "Mark a bug as being fixed by this revision (see \"bzr help bugs\")."
+msgstr ""
+
+# help of 'author' option of 'commit' command
+#: bzrlib/builtins.py:3250
+msgid "Set the author's name, if it's different from the committer."
+msgstr ""
+
+# help of 'local' option of 'commit' command
+#: bzrlib/builtins.py:3253
+msgid ""
+"Perform a local commit in a bound branch.  Local commits are not pushed to "
+"the master branch until a normal commit is performed."
+msgstr ""
+
+# help of 'show-diff' option of 'commit' command
+#: bzrlib/builtins.py:3259
+msgid ""
+"When no message is supplied, show the diff along with the status summary in "
+"the message editor."
+msgstr ""
+
+# help of 'lossy' option of 'commit' command
+#: bzrlib/builtins.py:3262
+msgid ""
+"When committing to a foreign version control system do not push data that "
+"can not be natively represented."
+msgstr ""
+
+#: bzrlib/builtins.py:3280
+#, python-format
+msgid ""
+"No tracker specified for bug %s. Use the form 'tracker:id' or specify a "
+"default bug tracker using the `bugtracker` option.\n"
+"See \"bzr help bugs\" for more information on this feature. Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3289
+#, python-format
+msgid ""
+"Invalid bug %s. Must be in the form of 'tracker:id'. See \"bzr help bugs\" "
+"for more information on this feature.\n"
+"Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3298
+#, python-format
+msgid "Unrecognized bug %s. Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3301
+#, python-format
+msgid ""
+"%s\n"
+"Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3325
+msgid "Could not parse --commit-time: "
+msgstr ""
+
+#: bzrlib/builtins.py:3365
+msgid "please specify either --message or --file"
+msgstr ""
+
+#: bzrlib/builtins.py:3394
+msgid "please specify a commit message with either --message or --file"
+msgstr ""
+
+#: bzrlib/builtins.py:3397
+msgid ""
+"Empty commit message specified. Please specify a commit message with either "
+"--message or --file or leave a blank message with --message \"\"."
+msgstr ""
+
+#: bzrlib/builtins.py:3417
+msgid ""
+"No changes to commit. Please 'bzr add' the files you want to commit, or use "
+"--unchanged to force an empty commit."
+msgstr ""
+
+#: bzrlib/builtins.py:3421
+msgid ""
+"Conflicts detected in working tree.  Use \"bzr conflicts\" to list, \"bzr "
+"resolve FILE\" to resolve."
+msgstr ""
+
+#: bzrlib/builtins.py:3425
+msgid "Commit refused because there are unknown files in the working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:3428
+msgid ""
+"\n"
+"To commit to master branch, run update and then commit.\n"
+"You can also pass --local to commit to continue working disconnected."
+msgstr ""
+
+#: bzrlib/builtins.py:3436
+msgid ""
+"Validate working tree structure, branch consistency and repository history."
+msgstr ""
+
+#: bzrlib/builtins.py:3438
+msgid ""
+"This command checks various invariants about branch and repository storage\n"
+"to detect data corruption or bzr bugs."
+msgstr ""
+
+#: bzrlib/builtins.py:3441
+msgid ""
+"The working tree and branch checks will only give output if a problem is\n"
+"detected. The output fields of the repository check are:"
+msgstr ""
+
+#: bzrlib/builtins.py:3444
+msgid ""
+"revisions\n"
+"    This is just the number of revisions checked.  It doesn't\n"
+"    indicate a problem."
+msgstr ""
+
+#: bzrlib/builtins.py:3448
+msgid ""
+"versionedfiles\n"
+"    This is just the number of versionedfiles checked.  It\n"
+"    doesn't indicate a problem."
+msgstr ""
+
+#: bzrlib/builtins.py:3452
+msgid ""
+"unreferenced ancestors\n"
+"    Texts that are ancestors of other texts, but\n"
+"    are not properly referenced by the revision ancestry.  This is a\n"
+"    subtle problem that Bazaar can work around."
+msgstr ""
+
+#: bzrlib/builtins.py:3457
+msgid ""
+"unique file texts\n"
+"    This is the total number of unique file contents\n"
+"    seen in the checked revisions.  It does not indicate a problem."
+msgstr ""
+
+#: bzrlib/builtins.py:3461
+msgid ""
+"repeated file texts\n"
+"    This is the total number of repeated texts seen\n"
+"    in the checked revisions.  Texts can be repeated when their file\n"
+"    entries are modified, but the file contents are not.  It does not\n"
+"    indicate a problem."
+msgstr ""
+
+#: bzrlib/builtins.py:3467
+msgid ""
+"If no restrictions are specified, all Bazaar data that is found at the "
+"given\n"
+"location will be checked."
+msgstr ""
+
+#: bzrlib/builtins.py:3470
+msgid ":Examples:"
+msgstr ""
+
+#: bzrlib/builtins.py:3472
+msgid "    Check the tree and branch at 'foo'::"
+msgstr ""
+
+#: bzrlib/builtins.py:3474
+msgid "        bzr check --tree --branch foo"
+msgstr ""
+
+#: bzrlib/builtins.py:3476
+msgid "    Check only the repository at 'bar'::"
+msgstr ""
+
+#: bzrlib/builtins.py:3478
+msgid "        bzr check --repo bar"
+msgstr ""
+
+#: bzrlib/builtins.py:3480
+msgid "    Check everything at 'baz'::"
+msgstr ""
+
+#: bzrlib/builtins.py:3482
+msgid "        bzr check baz"
+msgstr ""
+
+# help of 'branch' option of 'check' command
+#: bzrlib/builtins.py:3488
+msgid "Check the branch related to the current directory."
+msgstr ""
+
+# help of 'repo' option of 'check' command
+#: bzrlib/builtins.py:3490
+msgid "Check the repository related to the current directory."
+msgstr ""
+
+# help of 'tree' option of 'check' command
+#: bzrlib/builtins.py:3492
+msgid "Check the working tree related to the current directory."
+msgstr ""
+
+#: bzrlib/builtins.py:3506
+msgid "Upgrade a repository, branch or working tree to a newer format."
+msgstr ""
+
+#: bzrlib/builtins.py:3508
+msgid ""
+"When the default format has changed after a major new release of\n"
+"Bazaar, you may be informed during certain operations that you\n"
+"should upgrade. Upgrading to a newer format may improve performance\n"
+"or make new features available. It may however limit interoperability\n"
+"with older repositories or with older versions of Bazaar."
+msgstr ""
+
+#: bzrlib/builtins.py:3514
+msgid ""
+"If you wish to upgrade to a particular format rather than the\n"
+"current default, that can be specified using the --format option.\n"
+"As a consequence, you can use the upgrade command this way to\n"
+"\"downgrade\" to an earlier format, though some conversions are\n"
+"a one way process (e.g. changing from the 1.x default to the\n"
+"2.x default) so downgrading is not always possible."
+msgstr ""
+
+#: bzrlib/builtins.py:3521
+msgid ""
+"A backup.bzr.~#~ directory is created at the start of the conversion\n"
+"process (where # is a number). By default, this is left there on\n"
+"completion. If the conversion fails, delete the new .bzr directory\n"
+"and rename this one back in its place. Use the --clean option to ask\n"
+"for the backup.bzr directory to be removed on successful conversion.\n"
+"Alternatively, you can delete it by hand if everything looks good\n"
+"afterwards."
+msgstr ""
+
+#: bzrlib/builtins.py:3529
+msgid ""
+"If the location given is a shared repository, dependent branches\n"
+"are also converted provided the repository converts successfully.\n"
+"If the conversion of a branch fails, remaining branches are still\n"
+"tried."
+msgstr ""
+
+#: bzrlib/builtins.py:3534
+msgid ""
+"For more information on upgrades, see the Bazaar Upgrade Guide,\n"
+"http://doc.bazaar.canonical.com/latest/en/upgrade-guide/."
+msgstr ""
+
+# help of 'format' option of 'upgrade' command
+#: bzrlib/builtins.py:3542
+msgid "Upgrade to a specific format.  See \"bzr help formats\" for details."
+msgstr ""
+
+# title of 'format' option of 'init' command
+#: bzrlib/builtins.py:3546
+msgid "Branch format"
+msgstr ""
+
+# help of 'clean' option of 'upgrade' command
+#: bzrlib/builtins.py:3548
+msgid "Remove the backup.bzr directory if successful."
+msgstr ""
+
+#: bzrlib/builtins.py:3565
+msgid "Show or set bzr user id."
+msgstr ""
+
+#: bzrlib/builtins.py:3567
+msgid ""
+":Examples:\n"
+"    Show the email of the current user::"
+msgstr ""
+
+#: bzrlib/builtins.py:3570
+msgid "        bzr whoami --email"
+msgstr ""
+
+#: bzrlib/builtins.py:3572
+msgid "    Set the current user::"
+msgstr ""
+
+#: bzrlib/builtins.py:3574
+msgid "        bzr whoami \"Frank Chu <fchu@example.com>\""
+msgstr ""
+
+# help of 'email' option of 'whoami' command
+#: bzrlib/builtins.py:3578
+msgid "Display email address only."
+msgstr ""
+
+# help of 'branch' option of 'whoami' command
+#: bzrlib/builtins.py:3580
+msgid "Set identity for the current branch instead of globally."
+msgstr ""
+
+#: bzrlib/builtins.py:3604
+msgid "--email can only be used to display existing identity"
+msgstr ""
+
+#: bzrlib/builtins.py:3626
+msgid "Print or set the branch nickname."
+msgstr ""
+
+#: bzrlib/builtins.py:3628
+msgid ""
+"If unset, the tree root directory name is used as the nickname.\n"
+"To print the current nickname, execute with no argument."
+msgstr ""
+
+#: bzrlib/builtins.py:3631
+msgid ""
+"Bound branches use the nickname of its master branch unless it is set\n"
+"locally."
+msgstr ""
+
+#: bzrlib/builtins.py:3651
+msgid "Set/unset and display aliases."
+msgstr ""
+
+#: bzrlib/builtins.py:3653
+msgid ""
+":Examples:\n"
+"    Show the current aliases::"
+msgstr ""
+
+#: bzrlib/builtins.py:3656
+msgid "        bzr alias"
+msgstr ""
+
+#: bzrlib/builtins.py:3658
+msgid "    Show the alias specified for 'll'::"
+msgstr ""
+
+#: bzrlib/builtins.py:3660
+msgid "        bzr alias ll"
+msgstr ""
+
+#: bzrlib/builtins.py:3662
+msgid "    Set an alias for 'll'::"
+msgstr ""
+
+#: bzrlib/builtins.py:3664
+msgid "        bzr alias ll=\"log --line -r-10..-1\""
+msgstr ""
+
+#: bzrlib/builtins.py:3666
+msgid "    To remove an alias for 'll'::"
+msgstr ""
+
+#: bzrlib/builtins.py:3668
+msgid "        bzr alias --remove ll"
+msgstr ""
+
+# help of 'remove' option of 'alias' command
+#: bzrlib/builtins.py:3673
+msgid "Remove the alias."
+msgstr ""
+
+#: bzrlib/builtins.py:3691
+msgid "bzr alias --remove expects an alias to remove."
+msgstr ""
+
+#: bzrlib/builtins.py:3863
+msgid "subunit not available. subunit needs to be installed to use --subunit."
+msgstr ""
+
+#: bzrlib/builtins.py:3880
+msgid ""
+"--benchmark is no longer supported from bzr 2.2; use bzr-usertest instead"
+msgstr ""
+
+#: bzrlib/builtins.py:3927
+msgid "Show version of bzr."
+msgstr ""
+
+# help of 'short' option of 'version' command
+#: bzrlib/builtins.py:3931
+msgid "Print just the version number."
+msgstr ""
+
+#: bzrlib/builtins.py:3950
+msgid "It sure does!\n"
+msgstr ""
+
+#: bzrlib/builtins.py:3974
+#, python-format
+msgid "merge base is revision %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:3978
+msgid "Perform a three-way merge."
+msgstr ""
+
+#: bzrlib/builtins.py:3980
+msgid ""
+"The source of the merge can be specified either in the form of a branch,\n"
+"or in the form of a path to a file containing a merge directive generated\n"
+"with bzr send. If neither is specified, the default is the upstream branch\n"
+"or the branch most recently merged using --remember.  The source of the\n"
+"merge may also be specified in the form of a path to a file in another\n"
+"branch:  in this case, only the modifications to that file are merged into\n"
+"the current working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:3988
+msgid ""
+"When merging from a branch, by default bzr will try to merge in all new\n"
+"work from the other branch, automatically determining an appropriate base\n"
+"revision.  If this fails, you may need to give an explicit base."
+msgstr ""
+
+#: bzrlib/builtins.py:3992
+msgid ""
+"To pick a different ending revision, pass \"--revision OTHER\".  bzr will\n"
+"try to merge in all new work up to and including revision OTHER."
+msgstr ""
+
+#: bzrlib/builtins.py:3995
+msgid ""
+"If you specify two values, \"--revision BASE..OTHER\", only revisions BASE\n"
+"through OTHER, excluding BASE but including OTHER, will be merged.  If this\n"
+"causes some revisions to be skipped, i.e. if the destination branch does\n"
+"not already contain revision BASE, such a merge is commonly referred to as\n"
+"a \"cherrypick\". Unlike a normal merge, Bazaar does not currently track\n"
+"cherrypicks. The changes look like a normal commit, and the history of the\n"
+"changes from the other branch is not stored in the commit."
+msgstr ""
+
+#: bzrlib/builtins.py:4003
+msgid "Revision numbers are always relative to the source branch."
+msgstr ""
+
+#: bzrlib/builtins.py:4010
+msgid "Use bzr resolve when you have fixed a problem.  See also bzr conflicts."
+msgstr ""
+
+#: bzrlib/builtins.py:4012
+msgid ""
+"If there is no default branch set, the first merge will set it (use\n"
+"--no-remember to avoid setting it). After that, you can omit the branch\n"
+"to use the default.  To change the default, use --remember. The value will\n"
+"only be saved if the remote location can be accessed."
+msgstr ""
+
+#: bzrlib/builtins.py:4017
+msgid ""
+"The results of the merge are placed into the destination working\n"
+"directory, where they can be reviewed (with bzr diff), tested, and then\n"
+"committed to record the result of the merge."
+msgstr ""
+
+#: bzrlib/builtins.py:4021
+msgid ""
+"merge refuses to run if there are any uncommitted changes, unless\n"
+"--force is given.  If --force is given, then the changes from the source \n"
+"will be merged with the current working tree, including any uncommitted\n"
+"changes in the tree.  The --force option can also be used to create a\n"
+"merge revision which has more than two parents."
+msgstr ""
+
+#: bzrlib/builtins.py:4027
+msgid ""
+"If one would like to merge changes from the working tree of the other\n"
+"branch without merging any committed revisions, the --uncommitted option\n"
+"can be given."
+msgstr ""
+
+#: bzrlib/builtins.py:4031
+msgid ""
+"To select only some changes to merge, use \"merge -i\", which will prompt\n"
+"you to apply each diff hunk and file change, similar to \"shelve\"."
+msgstr ""
+
+#: bzrlib/builtins.py:4034
+msgid ""
+":Examples:\n"
+"    To merge all new revisions from bzr.dev::"
+msgstr ""
+
+#: bzrlib/builtins.py:4037
+msgid "        bzr merge ../bzr.dev"
+msgstr ""
+
+#: bzrlib/builtins.py:4039
+msgid "    To merge changes up to and including revision 82 from bzr.dev::"
+msgstr ""
+
+#: bzrlib/builtins.py:4041
+msgid "        bzr merge -r 82 ../bzr.dev"
+msgstr ""
+
+#: bzrlib/builtins.py:4043
+msgid "    To merge the changes introduced by 82, without previous changes::"
+msgstr ""
+
+#: bzrlib/builtins.py:4045
+msgid "        bzr merge -r 81..82 ../bzr.dev"
+msgstr ""
+
+#: bzrlib/builtins.py:4047
+msgid "    To apply a merge directive contained in /tmp/merge::"
+msgstr ""
+
+#: bzrlib/builtins.py:4049
+msgid "        bzr merge /tmp/merge"
+msgstr ""
+
+#: bzrlib/builtins.py:4051
+msgid ""
+"    To create a merge revision with three parents from two branches\n"
+"    feature1a and feature1b:"
+msgstr ""
+
+#: bzrlib/builtins.py:4054
+msgid ""
+"        bzr merge ../feature1a\n"
+"        bzr merge ../feature1b --force\n"
+"        bzr commit -m 'revision with three parents'"
+msgstr ""
+
+# help of 'force' option of 'merge' command
+#: bzrlib/builtins.py:4066
+msgid "Merge even if the destination tree has uncommitted changes."
+msgstr ""
+
+# help of 'uncommitted' option of 'merge' command
+#: bzrlib/builtins.py:4072
+msgid ""
+"Apply uncommitted changes from a working copy, instead of branch changes."
+msgstr ""
+
+# help of 'pull' option of 'merge' command
+#: bzrlib/builtins.py:4074
+msgid ""
+"If the destination is already completely merged into the source, pull from "
+"the source rather than merging.  When this happens, you do not need to "
+"commit the result."
+msgstr ""
+
+# help of 'directory' option of 'merge' command
+#: bzrlib/builtins.py:4079
+msgid ""
+"Branch to merge into, rather than the one containing the working directory."
+msgstr ""
+
+# help of 'preview' option of 'merge' command
+#: bzrlib/builtins.py:4081
+msgid "Instead of merging, show a diff of the merge."
+msgstr ""
+
+# help of 'interactive' option of 'merge' command
+#: bzrlib/builtins.py:4083
+msgid "Select changes interactively."
+msgstr ""
+
+#: bzrlib/builtins.py:4105
+msgid ""
+"Merging into empty branches not currently supported, https://bugs.launchpad."
+"net/bzr/+bug/308562"
+msgstr ""
+
+#: bzrlib/builtins.py:4132
+msgid "Cannot use --uncommitted with bundles or merge directives."
+msgstr ""
+
+#: bzrlib/builtins.py:4143
+msgid "Cannot use --uncommitted and --revision at the same time."
+msgstr ""
+
+#: bzrlib/builtins.py:4163
+msgid "merger: "
+msgstr ""
+
+#: bzrlib/builtins.py:4165 bzrlib/bundle/apply_bundle.py:67
+msgid "Nothing to do."
+msgstr ""
+
+#: bzrlib/builtins.py:4169
+msgid "Cannot pull individual files"
+msgstr ""
+
+#: bzrlib/builtins.py:4177
+msgid "This branch has no commits. (perhaps you would prefer 'bzr pull')"
+msgstr ""
+
+#: bzrlib/builtins.py:4235
+#, python-format
+msgid "Show-base is not supported for this merge type. %s"
+msgstr ""
+
+#: bzrlib/builtins.py:4244
+#, python-format
+msgid "Conflict reduction is not supported for merge type %s."
+msgstr ""
+
+#: bzrlib/builtins.py:4248
+msgid "Cannot do conflict reduction and show base."
+msgstr ""
+
+#: bzrlib/builtins.py:4359
+msgid "No location specified or remembered"
+msgstr ""
+
+#: bzrlib/builtins.py:4361
+msgid "{0} remembered {1} location {2}"
+msgstr ""
+
+#: bzrlib/builtins.py:4367
+msgid "Redo a merge."
+msgstr ""
+
+#: bzrlib/builtins.py:4369
+msgid ""
+"Use this if you want to try a different merge technique while resolving\n"
+"conflicts.  Some merge techniques are better than others, and remerge\n"
+"lets you try different ones on different files."
+msgstr ""
+
+#: bzrlib/builtins.py:4373
+msgid ""
+"The options for remerge have the same meaning and defaults as the ones for\n"
+"merge.  The difference is that remerge can (only) be run when there is a\n"
+"pending merge, and it lets you specify particular files."
+msgstr ""
+
+#: bzrlib/builtins.py:4377
+msgid ""
+":Examples:\n"
+"    Re-do the merge of all conflicted files, and show the base text in\n"
+"    conflict regions, in addition to the usual THIS and OTHER texts::"
+msgstr ""
+
+#: bzrlib/builtins.py:4381
+msgid "        bzr remerge --show-base"
+msgstr ""
+
+#: bzrlib/builtins.py:4383
+msgid ""
+"    Re-do the merge of \"foobar\", using the weave merge algorithm, with\n"
+"    additional processing to reduce the size of conflict regions::"
+msgstr ""
+
+#: bzrlib/builtins.py:4386
+msgid "        bzr remerge --merge-type weave --reprocess foobar"
+msgstr ""
+
+# help of 'show-base' option of 'merge' command
+#: bzrlib/builtins.py:4393
+msgid "Show base revision text in conflicts."
+msgstr ""
+
+#: bzrlib/builtins.py:4405
+msgid ""
+"Sorry, remerge only works after normal merges.  Not cherrypicking or multi-"
+"merges."
+msgstr ""
+
+#: bzrlib/builtins.py:4461
+msgid "Revert files to a previous revision."
+msgstr ""
+
+#: bzrlib/builtins.py:4463
+msgid ""
+"Giving a list of files will revert only those files.  Otherwise, all files\n"
+"will be reverted.  If the revision is not specified with '--revision', the\n"
+"last committed revision is used."
+msgstr ""
+
+#: bzrlib/builtins.py:4467
+msgid ""
+"To remove only some changes, without reverting to a prior version, use\n"
+"merge instead.  For example, \"merge . -r -2..-3\" (don't forget the \".\")\n"
+"will remove the changes introduced by the second last commit (-2), without\n"
+"affecting the changes introduced by the last commit (-1).  To remove\n"
+"certain changes on a hunk-by-hunk basis, see the shelve command."
+msgstr ""
+
+#: bzrlib/builtins.py:4473
+msgid ""
+"By default, any files that have been manually changed will be backed up\n"
+"first.  (Files changed only by merge are not backed up.)  Backup files have\n"
+"'.~#~' appended to their name, where # is a number."
+msgstr ""
+
+#: bzrlib/builtins.py:4477
+msgid ""
+"When you provide files, you can use their current pathname or the pathname\n"
+"from the target revision.  So you can use revert to \"undelete\" a file by\n"
+"name.  If you name a directory, all the contents of that directory will be\n"
+"reverted."
+msgstr ""
+
+#: bzrlib/builtins.py:4482
+msgid ""
+"If you have newly added files since the target revision, they will be\n"
+"removed.  If the files to be removed have been changed, backups will be\n"
+"created as above.  Directories containing unknown files will not be\n"
+"deleted."
+msgstr ""
+
+#: bzrlib/builtins.py:4487
+msgid ""
+"The working tree contains a list of revisions that have been merged but\n"
+"not yet committed. These revisions will be included as additional parents\n"
+"of the next commit.  Normally, using revert clears that list as well as\n"
+"reverting the files.  If any files are specified, revert leaves the list\n"
+"of uncommitted merges alone and reverts only the files.  Use ``bzr revert\n"
+".`` in the tree root to revert all files but keep the recorded merges,\n"
+"and ``bzr revert --forget-merges`` to clear the pending merge list without\n"
+"reverting any files."
+msgstr ""
+
+#: bzrlib/builtins.py:4496
+msgid ""
+"Using \"bzr revert --forget-merges\", it is possible to apply all of the\n"
+"changes from a branch in a single revision.  To do this, perform the merge\n"
+"as desired.  Then doing revert with the \"--forget-merges\" option will "
+"keep\n"
+"the content of the tree as it was, but it will clear the list of pending\n"
+"merges.  The next commit will then contain all of the changes that are\n"
+"present in the other branch, but without any other parent revisions.\n"
+"Because this technique forgets where these changes originated, it may\n"
+"cause additional conflicts on later merges involving the same source and\n"
+"target branches."
+msgstr ""
+
+# help of 'no-backup' option of 'revert' command
+#: bzrlib/builtins.py:4510
+msgid "Do not save backups of reverted files."
+msgstr ""
+
+# help of 'forget-merges' option of 'revert' command
+#: bzrlib/builtins.py:4512
+msgid "Remove pending merge marker, without changing any files."
+msgstr ""
+
+#: bzrlib/builtins.py:4543
+msgid ""
+"Show help on a command or other topic.\n"
+"    "
+msgstr ""
+
+# help of 'long' option of 'help' command
+#: bzrlib/builtins.py:4548
+msgid "Show help on all commands."
+msgstr ""
+
+#: bzrlib/builtins.py:4577
+msgid "Show unmerged/unpulled revisions between two branches."
+msgstr ""
+
+#: bzrlib/builtins.py:4579
+msgid "OTHER_BRANCH may be local or remote."
+msgstr ""
+
+#: bzrlib/builtins.py:4581
+msgid ""
+"To filter on a range of revisions, you can use the command -r begin..end\n"
+"-r revision requests a specific revision, -r ..end or -r begin.. are\n"
+"also valid.\n"
+"        \n"
+":Exit values:\n"
+"    1 - some missing revisions\n"
+"    0 - no missing revisions"
+msgstr ""
+
+#: bzrlib/builtins.py:4591
+msgid ""
+"    Determine the missing revisions between this and the branch at the\n"
+"    remembered pull location::"
+msgstr ""
+
+#: bzrlib/builtins.py:4594
+msgid "        bzr missing"
+msgstr ""
+
+#: bzrlib/builtins.py:4596
+msgid "    Determine the missing revisions between this and another branch::"
+msgstr ""
+
+#: bzrlib/builtins.py:4598
+msgid "        bzr missing http://server/branch"
+msgstr ""
+
+#: bzrlib/builtins.py:4600
+msgid ""
+"    Determine the missing revisions up to a specific revision on the other\n"
+"    branch::"
+msgstr ""
+
+#: bzrlib/builtins.py:4603
+msgid "        bzr missing -r ..-10"
+msgstr ""
+
+#: bzrlib/builtins.py:4605
+msgid ""
+"    Determine the missing revisions up to a specific revision on this\n"
+"    branch::"
+msgstr ""
+
+#: bzrlib/builtins.py:4608
+msgid "        bzr missing --my-revision ..-10"
+msgstr ""
+
+# help of 'reverse' option of 'missing' command
+#: bzrlib/builtins.py:4615
+msgid "Reverse the order of revisions."
+msgstr ""
+
+# help of 'mine-only' option of 'missing' command
+#: bzrlib/builtins.py:4617
+msgid "Display changes in the local branch only."
+msgstr ""
+
+# help of 'this' option of 'missing' command
+#: bzrlib/builtins.py:4618
+msgid "Same as --mine-only."
+msgstr ""
+
+# help of 'theirs-only' option of 'missing' command
+#: bzrlib/builtins.py:4620
+msgid "Display changes in the remote branch only."
+msgstr ""
+
+# help of 'other' option of 'missing' command
+#: bzrlib/builtins.py:4621
+msgid "Same as --theirs-only."
+msgstr ""
+
+# help of 'revision' option of 'missing' command
+#: bzrlib/builtins.py:4626
+msgid ""
+"Filter on other branch revisions (inclusive). See \"help revisionspec\" for "
+"details."
+msgstr ""
+
+# help of 'my-revision' option of 'missing' command
+#: bzrlib/builtins.py:4630
+msgid ""
+"Filter on local branch revisions (inclusive). See \"help revisionspec\" for "
+"details."
+msgstr ""
+
+# help of 'include-merged' option of 'missing' command
+#: bzrlib/builtins.py:4633
+msgid "Show all revisions in addition to the mainline ones."
+msgstr ""
+
+#: bzrlib/builtins.py:4687
+msgid "No peer location known or specified."
+msgstr ""
+
+#: bzrlib/builtins.py:4691
+msgid "Using saved parent location: {0}\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4724
+#, python-format
+msgid "You have %d extra revision:\n"
+msgid_plural "You have %d extra revisions:\n"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:4740
+#, python-format
+msgid "You are missing %d revision:\n"
+msgid_plural "You are missing %d revisions:\n"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:4752
+msgid "This branch has no new revisions.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4755
+msgid "Other branch has no new revisions.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4760
+msgid "Branches are up to date.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4771
+msgid "Compress the data within a repository."
+msgstr ""
+
+#: bzrlib/builtins.py:4773
+msgid ""
+"This operation compresses the data within a bazaar repository. As\n"
+"bazaar supports automatic packing of repository, this operation is\n"
+"normally not required to be done manually."
+msgstr ""
+
+#: bzrlib/builtins.py:4777
+msgid ""
+"During the pack operation, bazaar takes a backup of existing repository\n"
+"data, i.e. pack files. This backup is eventually removed by bazaar\n"
+"automatically when it is safe to do so. To save disk space by removing\n"
+"the backed up pack files, the --clean-obsolete-packs option may be\n"
+"used."
+msgstr ""
+
+#: bzrlib/builtins.py:4783
+msgid ""
+"Warning: If you use --clean-obsolete-packs and your machine crashes\n"
+"during or immediately after repacking, you may be left with a state\n"
+"where the deletion has been written to disk but the new packs have not\n"
+"been. In this case the repository may be unusable."
+msgstr ""
+
+# help of 'clean-obsolete-packs' option of 'pack' command
+#: bzrlib/builtins.py:4792
+msgid "Delete obsolete packs to save disk space."
+msgstr ""
+
+#: bzrlib/builtins.py:4806
+msgid "List the installed plugins."
+msgstr ""
+
+#: bzrlib/builtins.py:4808
+msgid ""
+"This command displays the list of installed plugins including\n"
+"version of plugin and a short description of each."
+msgstr ""
+
+#: bzrlib/builtins.py:4811
+msgid "--verbose shows the path where each plugin is located."
+msgstr ""
+
+#: bzrlib/builtins.py:4813
+msgid ""
+"A plugin is an external component for Bazaar that extends the\n"
+"revision control system, by adding or replacing code in Bazaar.\n"
+"Plugins can do a variety of things, including overriding commands,\n"
+"adding new commands, providing additional network transports and\n"
+"customizing log output."
+msgstr ""
+
+#: bzrlib/builtins.py:4819
+msgid ""
+"See the Bazaar Plugin Guide <http://doc.bazaar.canonical.com/plugins/en/>\n"
+"for further information on plugins including where to find them and how to\n"
+"install them. Instructions are also provided there on how to write new\n"
+"plugins using the Python programming language."
+msgstr ""
+
+#: bzrlib/builtins.py:4835
+msgid "Show testament (signing-form) of a revision."
+msgstr ""
+
+# help of 'long' option of 'testament' command
+#: bzrlib/builtins.py:4838
+msgid "Produce long-format testament."
+msgstr ""
+
+# help of 'strict' option of 'testament' command
+#: bzrlib/builtins.py:4840
+msgid "Produce a strict-format testament."
+msgstr ""
+
+#: bzrlib/builtins.py:4866
+msgid "Show the origin of each line in a file."
+msgstr ""
+
+#: bzrlib/builtins.py:4868
+msgid ""
+"This prints out the given file with an annotation on the left side\n"
+"indicating which revision, author and date introduced the change."
+msgstr ""
+
+#: bzrlib/builtins.py:4871
+msgid ""
+"If the origin is the same for a run of consecutive lines, it is\n"
+"shown only at the top, unless the --all option is given."
+msgstr ""
+
+# help of 'all' option of 'annotate' command
+#: bzrlib/builtins.py:4879
+msgid "Show annotations on all lines."
+msgstr ""
+
+# help of 'long' option of 'annotate' command
+#: bzrlib/builtins.py:4880
+msgid "Show commit date in annotations."
+msgstr ""
+
+#: bzrlib/builtins.py:4967
+msgid "Cannot sign a range of non-revision-history revisions"
+msgstr ""
+
+#: bzrlib/builtins.py:4979
+msgid "Please supply either one revision, or a range."
+msgstr ""
+
+#: bzrlib/builtins.py:4983
+msgid ""
+"Convert the current branch into a checkout of the supplied branch.\n"
+"If no branch is supplied, rebind to the last bound location."
+msgstr ""
+
+#: bzrlib/builtins.py:4986
+msgid ""
+"Once converted into a checkout, commits must succeed on the master branch\n"
+"before they will be applied to the local branch."
+msgstr ""
+
+#: bzrlib/builtins.py:4989
+msgid ""
+"Bound branches use the nickname of its master branch unless it is set\n"
+"locally, in which case binding will update the local nickname to be\n"
+"that of the master."
+msgstr ""
+
+#: bzrlib/builtins.py:5004
+msgid "No location supplied.  This format does not remember old locations."
+msgstr ""
+
+#: bzrlib/builtins.py:5009
+msgid "Branch is already bound"
+msgstr ""
+
+#: bzrlib/builtins.py:5011
+msgid "No location supplied and no previous location known"
+msgstr ""
+
+#: bzrlib/builtins.py:5017
+msgid "These branches have diverged. Try merging, and then bind again."
+msgstr ""
+
+#: bzrlib/builtins.py:5024
+msgid "Convert the current checkout into a regular branch."
+msgstr ""
+
+#: bzrlib/builtins.py:5026
+msgid ""
+"After unbinding, the local branch is considered independent and subsequent\n"
+"commits will be local only."
+msgstr ""
+
+#: bzrlib/builtins.py:5037
+msgid "Local branch is not bound"
+msgstr ""
+
+#: bzrlib/builtins.py:5041
+msgid "Remove the last committed revision."
+msgstr ""
+
+#: bzrlib/builtins.py:5043
+msgid ""
+"--verbose will print out what is being removed.\n"
+"--dry-run will go through all the motions, but not actually\n"
+"remove anything."
+msgstr ""
+
+#: bzrlib/builtins.py:5047
+msgid ""
+"If --revision is specified, uncommit revisions to leave the branch at the\n"
+"specified revision.  For example, \"bzr uncommit -r 15\" will leave the\n"
+"branch at revision 15."
+msgstr ""
+
+#: bzrlib/builtins.py:5051
+msgid ""
+"Uncommit leaves the working tree ready for a new commit.  The only change\n"
+"it may make is to restore any pending merges that were present before\n"
+"the commit."
+msgstr ""
+
+# help of 'dry-run' option of 'uncommit' command
+#: bzrlib/builtins.py:5062
+msgid "Don't actually make changes."
+msgstr ""
+
+# help of 'force' option of 'uncommit' command
+#: bzrlib/builtins.py:5063
+msgid "Say yes to all questions."
+msgstr ""
+
+# help of 'keep-tags' option of 'uncommit' command
+#: bzrlib/builtins.py:5065
+msgid "Keep tags that point to removed revisions."
+msgstr ""
+
+# help of 'local' option of 'uncommit' command
+#: bzrlib/builtins.py:5067
+msgid "Only remove the commits from the local branch when in a checkout."
+msgstr ""
+
+#: bzrlib/builtins.py:5116
+msgid "No revisions to uncommit.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5131
+msgid "Dry-run, pretending to remove the above revisions.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5134
+msgid "The above revision(s) will be removed.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5138
+msgid "Uncommit these revisions"
+msgstr ""
+
+#: bzrlib/builtins.py:5141
+msgid "Canceled\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5148
+#, python-format
+msgid ""
+"You can restore the old tip by running:\n"
+"  bzr pull . -r revid:%s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5153
+msgid "Break a dead lock."
+msgstr ""
+
+#: bzrlib/builtins.py:5155
+msgid ""
+"This command breaks a lock on a repository, branch, working directory or\n"
+"config file."
+msgstr ""
+
+#: bzrlib/builtins.py:5158
+msgid ""
+"CAUTION: Locks should only be broken when you are sure that the process\n"
+"holding the lock has been stopped."
+msgstr ""
+
+#: bzrlib/builtins.py:5161
+msgid ""
+"You can get information on what locks are open via the 'bzr info\n"
+"[location]' command."
+msgstr ""
+
+#: bzrlib/builtins.py:5164
+msgid ""
+":Examples:\n"
+"    bzr break-lock\n"
+"    bzr break-lock bzr+ssh://example.com/bzr/foo\n"
+"    bzr break-lock --conf ~/.bazaar"
+msgstr ""
+
+# help of 'config' option of 'break-lock' command
+#: bzrlib/builtins.py:5173
+msgid "LOCATION is the directory where the config lock is."
+msgstr ""
+
+# help of 'force' option of 'break-lock' command
+#: bzrlib/builtins.py:5175
+msgid "Do not ask for confirmation before breaking the lock."
+msgstr ""
+
+#: bzrlib/builtins.py:5211
+msgid "Run the bzr server."
+msgstr ""
+
+# help of 'inet' option of 'serve' command
+#: bzrlib/builtins.py:5217
+msgid "Serve on stdin/out for use from inetd or sshd."
+msgstr ""
+
+# title of 'protocol' option of 'serve' command
+#: bzrlib/builtins.py:5218
+msgid "protocol"
+msgstr ""
+
+# help of 'protocol' option of 'serve' command
+#: bzrlib/builtins.py:5219
+msgid "Protocol to serve."
+msgstr ""
+
+# help of 'port' option of 'serve' command
+#: bzrlib/builtins.py:5223
+msgid ""
+"Listen for connections on nominated port of the form [hostname:]portnumber.  "
+"Passing 0 as the port number will result in a dynamically allocated port.  "
+"The default port depends on the protocol."
+msgstr ""
+
+# help of 'directory' option of 'serve' command
+#: bzrlib/builtins.py:5229
+msgid "Serve contents of this directory."
+msgstr ""
+
+# help of 'allow-writes' option of 'serve' command
+#: bzrlib/builtins.py:5231
+msgid ""
+"By default the server is a readonly server.  Supplying --allow-writes "
+"enables write access to the contents of the served directory and below.  "
+"Note that ``bzr serve`` does not perform authentication, so unless some form "
+"of external authentication is arranged supplying this option leads to global "
+"uncontrolled write access to your file system."
+msgstr ""
+
+#: bzrlib/builtins.py:5276
+msgid "Combine a tree into its containing tree."
+msgstr ""
+
+#: bzrlib/builtins.py:5278
+msgid "This command requires the target tree to be in a rich-root format."
+msgstr ""
+
+#: bzrlib/builtins.py:5280
+msgid ""
+"The TREE argument should be an independent tree, inside another tree, but\n"
+"not part of it.  (Such trees can be produced by \"bzr split\", but also by\n"
+"running \"bzr branch\" with the target inside a tree.)"
+msgstr ""
+
+#: bzrlib/builtins.py:5284
+msgid ""
+"The result is a combined tree, with the subtree no longer an independent\n"
+"part.  This is marked as a merge of the subtree into the containing tree,\n"
+"and all history is preserved."
+msgstr ""
+
+#: bzrlib/builtins.py:5302
+#, python-format
+msgid ""
+"Can't join trees because %s doesn't support rich root data.\n"
+"You can use bzr upgrade on the repository."
+msgstr ""
+
+#: bzrlib/builtins.py:5312 bzrlib/builtins.py:5318
+msgid "Cannot join {0}.  {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:5322
+msgid "Split a subdirectory of a tree into a separate tree."
+msgstr ""
+
+#: bzrlib/builtins.py:5324
+msgid ""
+"This command will produce a target tree in a format that supports\n"
+"rich roots, like 'rich-root' or 'rich-root-pack'.  These formats cannot be\n"
+"converted into earlier formats like 'dirstate-tags'."
+msgstr ""
+
+#: bzrlib/builtins.py:5328
+msgid ""
+"The TREE argument should be a subdirectory of a working tree.  That\n"
+"subdirectory will be converted into an independent tree, with its own\n"
+"branch.  Commits in the top-level tree will not apply to the new subtree."
+msgstr ""
+
+#: bzrlib/builtins.py:5408
+msgid "No submit branch specified or known"
+msgstr ""
+
+#: bzrlib/builtins.py:5416 bzrlib/send.py:105
+msgid "No public branch specified or known"
+msgstr ""
+
+#: bzrlib/builtins.py:5421
+msgid "bzr merge-directive takes at most two one revision identifiers"
+msgstr ""
+
+#: bzrlib/builtins.py:5430
+msgid "No revisions to bundle."
+msgstr ""
+
+#: bzrlib/builtins.py:5449
+msgid "Mail or create a merge-directive for submitting changes."
+msgstr ""
+
+#: bzrlib/builtins.py:5451
+msgid "A merge directive provides many things needed for requesting merges:"
+msgstr ""
+
+#: bzrlib/builtins.py:5453
+msgid "* A machine-readable description of the merge to perform"
+msgstr ""
+
+#: bzrlib/builtins.py:5455
+msgid "* An optional patch that is a preview of the changes requested"
+msgstr ""
+
+#: bzrlib/builtins.py:5457
+msgid ""
+"* An optional bundle of revision data, so that the changes can be applied\n"
+"  directly from the merge directive, without retrieving data from a\n"
+"  branch."
+msgstr ""
+
+#: bzrlib/builtins.py:5461
+msgid ""
+"`bzr send` creates a compact data set that, when applied using bzr\n"
+"merge, has the same effect as merging from the source branch.  "
+msgstr ""
+
+#: bzrlib/builtins.py:5464
+msgid ""
+"By default the merge directive is self-contained and can be applied to any\n"
+"branch containing submit_branch in its ancestory without needing access to\n"
+"the source branch."
+msgstr ""
+
+#: bzrlib/builtins.py:5468
+msgid ""
+"If --no-bundle is specified, then Bazaar doesn't send the contents of the\n"
+"revisions, but only a structured request to merge from the\n"
+"public_location.  In that case the public_branch is needed and it must be\n"
+"up-to-date and accessible to the recipient.  The public_branch is always\n"
+"included if known, so that people can check it later."
+msgstr ""
+
+#: bzrlib/builtins.py:5474
+msgid ""
+"The submit branch defaults to the parent of the source branch, but can be\n"
+"overridden.  Both submit branch and public branch will be remembered in\n"
+"branch.conf the first time they are used for a particular branch.  The\n"
+"source branch defaults to that containing the working directory, but can\n"
+"be changed using --from."
+msgstr ""
+
+#: bzrlib/builtins.py:5480
+msgid ""
+"Both the submit branch and the public branch follow the usual behavior with\n"
+"respect to --remember: If there is no default location set, the first send\n"
+"will set it (use --no-remember to avoid setting it). After that, you can\n"
+"omit the location to use the default.  To change the default, use\n"
+"--remember. The value will only be saved if the location can be accessed."
+msgstr ""
+
+#: bzrlib/builtins.py:5486
+msgid ""
+"In order to calculate those changes, bzr must analyse the submit branch.\n"
+"Therefore it is most efficient for the submit branch to be a local mirror.\n"
+"If a public location is known for the submit_branch, that location is used\n"
+"in the merge directive."
+msgstr ""
+
+#: bzrlib/builtins.py:5491
+msgid ""
+"The default behaviour is to send the merge directive by mail, unless -o is\n"
+"given, in which case it is sent to a file."
+msgstr ""
+
+#: bzrlib/builtins.py:5494
+msgid ""
+"Mail is sent using your preferred mail program.  This should be transparent\n"
+"on Windows (it uses MAPI).  On Unix, it requires the xdg-email utility.\n"
+"If the preferred client can't be found (or used), your editor will be used."
+msgstr ""
+
+#: bzrlib/builtins.py:5498
+msgid ""
+"To use a specific mail program, set the mail_client configuration option.\n"
+"(For Thunderbird 1.5, this works around some bugs.)  Supported values for\n"
+"specific clients are \"claws\", \"evolution\", \"kmail\", \"mail.app"
+"\" (MacOS X's\n"
+"Mail.app), \"mutt\", and \"thunderbird\"; generic options are \"default\",\n"
+"\"editor\", \"emacsclient\", \"mapi\", and \"xdg-email\".  Plugins may also "
+"add\n"
+"supported clients."
+msgstr ""
+
+#: bzrlib/builtins.py:5505
+msgid ""
+"If mail is being sent, a to address is required.  This can be supplied\n"
+"either on the commandline, by setting the submit_to configuration\n"
+"option in the branch itself or the child_submit_to configuration option\n"
+"in the submit branch."
+msgstr ""
+
+#: bzrlib/builtins.py:5510
+msgid ""
+"Two formats are currently supported: \"4\" uses revision bundle format 4 "
+"and\n"
+"merge directive format 2.  It is significantly faster and smaller than\n"
+"older formats.  It is compatible with Bazaar 0.19 and later.  It is the\n"
+"default.  \"0.9\" uses revision bundle format 0.9 and merge directive\n"
+"format 1.  It is compatible with Bazaar 0.12 - 0.18."
+msgstr ""
+
+#: bzrlib/builtins.py:5516
+msgid ""
+"The merge directives created by bzr send may be applied using bzr merge or\n"
+"bzr pull by specifying a file containing a merge directive as the location."
+msgstr ""
+
+#: bzrlib/builtins.py:5519
+msgid ""
+"bzr send makes extensive use of public locations to map local locations "
+"into\n"
+"URLs that can be used by other people.  See `bzr help configuration` to\n"
+"set them, and use `bzr info` to display them."
+msgstr ""
+
+# help of 'output' option of 'send' command
+#: bzrlib/builtins.py:5543
+msgid "Write merge directive to this file or directory; use - for stdout."
+msgstr ""
+
+# help of 'strict' option of 'send' command
+#: bzrlib/builtins.py:5547
+msgid ""
+"Refuse to send if there are uncommitted changes in the working tree, --no-"
+"strict disables the check."
+msgstr ""
+
+# help of 'mail-to' option of 'send' command
+#: bzrlib/builtins.py:5549
+msgid "Mail the request to this address."
+msgstr ""
+
+# help of 'body' option of 'send' command
+#: bzrlib/builtins.py:5553
+msgid "Body for the email."
+msgstr ""
+
+# help of 'no-bundle' option of 'send' command
+#: bzrlib/builtins.py:5606
+msgid "Do not include a bundle in the merge directive."
+msgstr ""
+
+# help of 'no-patch' option of 'send' command
+#: bzrlib/builtins.py:5607
+msgid "Do not include a preview patch in the merge directive."
+msgstr ""
+
+# help of 'remember' option of 'send' command
+#: bzrlib/builtins.py:5610
+msgid "Remember submit and public branch."
+msgstr ""
+
+# help of 'from' option of 'send' command
+#: bzrlib/builtins.py:5612
+msgid ""
+"Branch to generate the submission from, rather than the one containing the "
+"working directory."
+msgstr ""
+
+# title of 'format' option of 'send' command
+#: bzrlib/builtins.py:5622
+msgid "format"
+msgstr ""
+
+# help of 'format' option of 'send' command
+#: bzrlib/builtins.py:5623
+msgid "Use the specified output format."
+msgstr ""
+
+#: bzrlib/builtins.py:5645
+msgid "Create, remove or modify a tag naming a revision."
+msgstr ""
+
+#: bzrlib/builtins.py:5647
+msgid ""
+"Tags give human-meaningful names to revisions.  Commands that take a -r\n"
+"(--revision) option can be given -rtag:X, where X is any previously\n"
+"created tag."
+msgstr ""
+
+#: bzrlib/builtins.py:5651
+msgid ""
+"Tags are stored in the branch.  Tags are copied from one branch to another\n"
+"along when you branch, push, pull or merge."
+msgstr ""
+
+#: bzrlib/builtins.py:5654
+msgid ""
+"It is an error to give a tag name that already exists unless you pass\n"
+"--force, in which case the tag is moved to point to the new revision."
+msgstr ""
+
+#: bzrlib/builtins.py:5657
+msgid ""
+"To rename a tag (change the name but keep it on the same revsion), run "
+"``bzr\n"
+"tag new-name -r tag:old-name`` and then ``bzr tag --delete oldname``."
+msgstr ""
+
+#: bzrlib/builtins.py:5660
+msgid ""
+"If no tag name is specified it will be determined through the \n"
+"'automatic_tag_name' hook. This can e.g. be used to automatically tag\n"
+"upstream releases by reading configure.ac. See ``bzr help hooks`` for\n"
+"details."
+msgstr ""
+
+# help of 'delete' option of 'tag' command
+#: bzrlib/builtins.py:5670
+msgid "Delete this tag rather than placing it."
+msgstr ""
+
+# help of 'directory' option of 'tag' command
+#: bzrlib/builtins.py:5673
+msgid "Branch in which to place the tag."
+msgstr ""
+
+# help of 'force' option of 'tag' command
+#: bzrlib/builtins.py:5675
+msgid "Replace existing tags."
+msgstr ""
+
+#: bzrlib/builtins.py:5690
+msgid "No tag specified to delete."
+msgstr ""
+
+#: bzrlib/builtins.py:5692
+#, python-format
+msgid "Deleted tag %s."
+msgstr ""
+
+#: bzrlib/builtins.py:5697
+msgid "Tags can only be placed on a single revision, not on a range"
+msgstr ""
+
+#: bzrlib/builtins.py:5706
+msgid "Please specify a tag name."
+msgstr ""
+
+#: bzrlib/builtins.py:5714
+#, python-format
+msgid "Tag %s already exists for that revision."
+msgstr ""
+
+#: bzrlib/builtins.py:5718
+#, python-format
+msgid "Created tag %s."
+msgstr ""
+
+#: bzrlib/builtins.py:5720
+#, python-format
+msgid "Updated tag %s."
+msgstr ""
+
+#: bzrlib/builtins.py:5724
+msgid "List tags."
+msgstr ""
+
+#: bzrlib/builtins.py:5726
+msgid ""
+"This command shows a table of tag names and the revisions they reference."
+msgstr ""
+
+# help of 'directory' option of 'tags' command
+#: bzrlib/builtins.py:5732
+msgid "Branch whose tags should be displayed."
+msgstr ""
+
+# help of 'sort' option of 'tags' command
+#: bzrlib/builtins.py:5734
+msgid "Sort tags by different criteria."
+msgstr ""
+
+# title of 'sort' option of 'tags' command
+#: bzrlib/builtins.py:5734
+msgid "Sorting"
+msgstr ""
+
+#: bzrlib/builtins.py:5779
+msgid "Reconfigure the type of a bzr directory."
+msgstr ""
+
+#: bzrlib/builtins.py:5781
+msgid "A target configuration must be specified."
+msgstr ""
+
+#: bzrlib/builtins.py:5783
+msgid ""
+"For checkouts, the bind-to location will be auto-detected if not specified.\n"
+"The order of preference is\n"
+"1. For a lightweight checkout, the current bound location.\n"
+"2. For branches that used to be checkouts, the previously-bound location.\n"
+"3. The push location.\n"
+"4. The parent location.\n"
+"If none of these is available, --bind-to must be specified."
+msgstr ""
+
+# title of 'tree_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5797
+msgid "Tree type"
+msgstr ""
+
+# help of 'tree_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5798
+msgid "The relation between branch and tree."
+msgstr ""
+
+# title of 'repository_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5808
+msgid "Repository type"
+msgstr ""
+
+# help of 'repository_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5809
+msgid "Location fo the repository."
+msgstr ""
+
+# title of 'repository_trees' option of 'reconfigure' command
+#: bzrlib/builtins.py:5817
+msgid "Trees in Repository"
+msgstr ""
+
+# help of 'repository_trees' option of 'reconfigure' command
+#: bzrlib/builtins.py:5818
+msgid "Whether new branches in the repository have trees."
+msgstr ""
+
+# help of 'bind-to' option of 'reconfigure' command
+#: bzrlib/builtins.py:5825
+msgid "Branch to bind checkout to."
+msgstr ""
+
+# help of 'force' option of 'reconfigure' command
+#: bzrlib/builtins.py:5827
+msgid "Perform reconfiguration even if local changes will be lost."
+msgstr ""
+
+# help of 'stacked-on' option of 'reconfigure' command
+#: bzrlib/builtins.py:5830
+msgid "Reconfigure a branch to be stacked on another branch."
+msgstr ""
+
+# help of 'unstacked' option of 'reconfigure' command
+#: bzrlib/builtins.py:5834
+msgid ""
+"Reconfigure a branch to be unstacked.  This may require copying substantial "
+"data into it."
+msgstr ""
+
+#: bzrlib/builtins.py:5844
+msgid "Can't use both --stacked-on and --unstacked"
+msgstr ""
+
+#: bzrlib/builtins.py:5858
+msgid "No target configuration specified"
+msgstr ""
+
+#: bzrlib/builtins.py:5893
+msgid "Set the branch of a checkout and update."
+msgstr ""
+
+#: bzrlib/builtins.py:5895
+msgid ""
+"For lightweight checkouts, this changes the branch being referenced.\n"
+"For heavyweight checkouts, this checks that there are no local commits\n"
+"versus the current bound branch, then it makes the local branch a mirror\n"
+"of the new location and binds to it."
+msgstr ""
+
+#: bzrlib/builtins.py:5900
+msgid ""
+"In both cases, the working tree is updated and uncommitted changes\n"
+"are merged. The user can commit or revert these as they desire."
+msgstr ""
+
+#: bzrlib/builtins.py:5903
+msgid "Pending merges need to be committed or reverted before using switch."
+msgstr ""
+
+#: bzrlib/builtins.py:5905
+msgid ""
+"The path to the branch to switch to can be specified relative to the parent\n"
+"directory of the current branch. For example, if you are currently in a\n"
+"checkout of /path/to/branch, specifying 'newbranch' will find a branch at\n"
+"/path/to/newbranch."
+msgstr ""
+
+#: bzrlib/builtins.py:5910
+msgid ""
+"Bound branches use the nickname of its master branch unless it is set\n"
+"locally, in which case switching will update the local nickname to be\n"
+"that of the master."
+msgstr ""
+
+# help of 'force' option of 'switch' command
+#: bzrlib/builtins.py:5918
+msgid "Switch even if local commits will be lost."
+msgstr ""
+
+# help of 'create-branch' option of 'switch' command
+#: bzrlib/builtins.py:5921
+msgid "Create the target branch from this one before switching to it."
+msgstr ""
+
+#: bzrlib/builtins.py:5933
+msgid "You must supply either a revision or a location"
+msgstr ""
+
+#: bzrlib/builtins.py:5944
+msgid "cannot create branch without source branch"
+msgstr ""
+
+#: bzrlib/builtins.py:5990
+msgid "Manage filtered views."
+msgstr ""
+
+#: bzrlib/builtins.py:5992
+msgid ""
+"Views provide a mask over the tree so that users can focus on\n"
+"a subset of a tree when doing their work. After creating a view,\n"
+"commands that support a list of files - status, diff, commit, etc -\n"
+"effectively have that list of files implicitly given each time.\n"
+"An explicit list of files can still be given but those files\n"
+"must be within the current view."
+msgstr ""
+
+#: bzrlib/builtins.py:5999
+msgid ""
+"In most cases, a view has a short life-span: it is created to make\n"
+"a selected change and is deleted once that change is committed.\n"
+"At other times, you may wish to create one or more named views\n"
+"and switch between them."
+msgstr ""
+
+#: bzrlib/builtins.py:6004
+msgid ""
+"To disable the current view without deleting it, you can switch to\n"
+"the pseudo view called ``off``. This can be useful when you need\n"
+"to see the whole tree for an operation or two (e.g. merge) but\n"
+"want to switch back to your view after that."
+msgstr ""
+
+#: bzrlib/builtins.py:6009
+msgid ""
+":Examples:\n"
+"  To define the current view::"
+msgstr ""
+
+#: bzrlib/builtins.py:6012
+msgid "    bzr view file1 dir1 ..."
+msgstr ""
+
+#: bzrlib/builtins.py:6014
+msgid "  To list the current view::"
+msgstr ""
+
+#: bzrlib/builtins.py:6016
+msgid "    bzr view"
+msgstr ""
+
+#: bzrlib/builtins.py:6018
+msgid "  To delete the current view::"
+msgstr ""
+
+#: bzrlib/builtins.py:6020
+msgid "    bzr view --delete"
+msgstr ""
+
+#: bzrlib/builtins.py:6022
+msgid "  To disable the current view without deleting it::"
+msgstr ""
+
+#: bzrlib/builtins.py:6024
+msgid "    bzr view --switch off"
+msgstr ""
+
+#: bzrlib/builtins.py:6026
+msgid "  To define a named view and switch to it::"
+msgstr ""
+
+#: bzrlib/builtins.py:6028
+msgid "    bzr view --name view-name file1 dir1 ..."
+msgstr ""
+
+#: bzrlib/builtins.py:6030
+msgid "  To list a named view::"
+msgstr ""
+
+#: bzrlib/builtins.py:6032
+msgid "    bzr view --name view-name"
+msgstr ""
+
+#: bzrlib/builtins.py:6034
+msgid "  To delete a named view::"
+msgstr ""
+
+#: bzrlib/builtins.py:6036
+msgid "    bzr view --name view-name --delete"
+msgstr ""
+
+#: bzrlib/builtins.py:6038
+msgid "  To switch to a named view::"
+msgstr ""
+
+#: bzrlib/builtins.py:6040
+msgid "    bzr view --switch view-name"
+msgstr ""
+
+#: bzrlib/builtins.py:6042
+msgid "  To list all views defined::"
+msgstr ""
+
+#: bzrlib/builtins.py:6044
+msgid "    bzr view --all"
+msgstr ""
+
+#: bzrlib/builtins.py:6046
+msgid "  To delete all views::"
+msgstr ""
+
+#: bzrlib/builtins.py:6048
+msgid "    bzr view --delete --all"
+msgstr ""
+
+# help of 'all' option of 'view' command
+#: bzrlib/builtins.py:6055
+msgid "Apply list or delete action to all views."
+msgstr ""
+
+# help of 'delete' option of 'view' command
+#: bzrlib/builtins.py:6058
+msgid "Delete the view."
+msgstr ""
+
+# help of 'name' option of 'view' command
+#: bzrlib/builtins.py:6061
+msgid "Name of the view to define, list or delete."
+msgstr ""
+
+# help of 'switch' option of 'view' command
+#: bzrlib/builtins.py:6065
+msgid "Name of the view to switch to."
+msgstr ""
+
+#: bzrlib/builtins.py:6084
+msgid "Both --delete and a file list specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6087
+msgid "Both --delete and --switch specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6090
+msgid "Deleted all views.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6092
+msgid "No current view to delete"
+msgstr ""
+
+#: bzrlib/builtins.py:6095
+#, python-format
+msgid "Deleted '%s' view.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6099
+msgid "Both --switch and a file list specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6102
+msgid "Both --switch and --all specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6105
+msgid "No current view to disable"
+msgstr ""
+
+#: bzrlib/builtins.py:6107
+#, python-format
+msgid "Disabled '%s' view.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6111 bzrlib/builtins.py:6133
+msgid "Using '{0}' view: {1}\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6114
+msgid "Views defined:\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6123
+msgid "No views defined.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6130
+msgid "Cannot change the 'off' pseudo view"
+msgstr ""
+
+#: bzrlib/builtins.py:6138
+msgid "No current view.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6141
+msgid "'{0}' view is: {1}\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6161
+msgid "    <no hooks installed>\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6165
+msgid "Remove a branch."
+msgstr ""
+
+#: bzrlib/builtins.py:6167
+msgid ""
+"This will remove the branch from the specified location but \n"
+"will keep any working tree or repository in place."
+msgstr ""
+
+#: bzrlib/builtins.py:6172
+msgid "  Remove the branch at repo/trunk::"
+msgstr ""
+
+#: bzrlib/builtins.py:6174
+msgid "    bzr remove-branch repo/trunk"
+msgstr ""
+
+#: bzrlib/builtins.py:6190
+msgid "Temporarily set aside some changes from the current tree."
+msgstr ""
+
+#: bzrlib/builtins.py:6192
+msgid ""
+"Shelve allows you to temporarily put changes you've made \"on the shelf\",\n"
+"ie. out of the way, until a later time when you can bring them back from\n"
+"the shelf with the 'unshelve' command.  The changes are stored alongside\n"
+"your working tree, and so they aren't propagated along with your branch nor\n"
+"will they survive its deletion."
+msgstr ""
+
+#: bzrlib/builtins.py:6198
+msgid "If shelve --list is specified, previously-shelved changes are listed."
+msgstr ""
+
+#: bzrlib/builtins.py:6200
+msgid ""
+"Shelve is intended to help separate several sets of changes that have\n"
+"been inappropriately mingled.  If you just want to get rid of all changes\n"
+"and you don't need to restore them later, use revert.  If you want to\n"
+"shelve all text changes at once, use shelve --all."
+msgstr ""
+
+#: bzrlib/builtins.py:6205
+msgid ""
+"If filenames are specified, only the changes to those files will be\n"
+"shelved. Other files will be left untouched."
+msgstr ""
+
+#: bzrlib/builtins.py:6208
+msgid ""
+"If a revision is specified, changes since that revision will be shelved."
+msgstr ""
+
+#: bzrlib/builtins.py:6210
+msgid ""
+"You can put multiple items on the shelf, and by default, 'unshelve' will\n"
+"restore the most recently shelved changes."
+msgstr ""
+
+#: bzrlib/builtins.py:6213
+msgid ""
+"For complicated changes, it is possible to edit the changes in a separate\n"
+"editor program to decide what the file remaining in the working copy\n"
+"should look like.  To do this, add the configuration option"
+msgstr ""
+
+#: bzrlib/builtins.py:6217
+msgid "    change_editor = PROGRAM @new_path @old_path"
+msgstr ""
+
+#: bzrlib/builtins.py:6219
+msgid ""
+"where @new_path is replaced with the path of the new version of the \n"
+"file and @old_path is replaced with the path of the old version of \n"
+"the file.  The PROGRAM should save the new file with the desired \n"
+"contents of the file in the working tree.\n"
+"    "
+msgstr ""
+
+# help of 'all' option of 'shelve' command
+#: bzrlib/builtins.py:6231
+msgid "Shelve all changes."
+msgstr ""
+
+# help of 'writer' option of 'shelve' command
+#: bzrlib/builtins.py:6233
+msgid "Method to use for writing diffs."
+msgstr ""
+
+# title of 'writer' option of 'shelve' command
+#: bzrlib/builtins.py:6233
+msgid "writer"
+msgstr ""
+
+# help of 'list' option of 'shelve' command
+#: bzrlib/builtins.py:6237
+msgid "List shelved changes."
+msgstr ""
+
+# help of 'destroy' option of 'shelve' command
+#: bzrlib/builtins.py:6239
+msgid "Destroy removed changes instead of shelving them."
+msgstr ""
+
+#: bzrlib/builtins.py:6268
+msgid "No shelved changes."
+msgstr ""
+
+#: bzrlib/builtins.py:6279
+msgid "Restore shelved changes."
+msgstr ""
+
+#: bzrlib/builtins.py:6281
+msgid ""
+"By default, the most recently shelved changes are restored. However if you\n"
+"specify a shelf by id those changes will be restored instead.  This works\n"
+"best when the changes don't depend on each other."
+msgstr ""
+
+# help of 'action' option of 'unshelve' command
+#: bzrlib/builtins.py:6290
+msgid "The action to perform."
+msgstr ""
+
+#: bzrlib/builtins.py:6312
+msgid "Remove unwanted files from working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:6314
+msgid ""
+"By default, only unknown files, not ignored files, are deleted.  Versioned\n"
+"files are never deleted."
+msgstr ""
+
+#: bzrlib/builtins.py:6317
+msgid ""
+"Another class is 'detritus', which includes files emitted by bzr during\n"
+"normal operations and selftests.  (The value of these files decreases with\n"
+"time.)"
+msgstr ""
+
+#: bzrlib/builtins.py:6321
+msgid ""
+"If no options are specified, unknown files are deleted.  Otherwise, option\n"
+"flags are respected, and may be combined."
+msgstr ""
+
+#: bzrlib/builtins.py:6324
+msgid "To check what clean-tree will do, use --dry-run."
+msgstr ""
+
+# help of 'ignored' option of 'clean-tree' command
+#: bzrlib/builtins.py:6327
+msgid "Delete all ignored files."
+msgstr ""
+
+# help of 'detritus' option of 'clean-tree' command
+#: bzrlib/builtins.py:6328
+msgid ""
+"Delete conflict files, merge and revert backups, and failed selftest dirs."
+msgstr ""
+
+# help of 'unknown' option of 'clean-tree' command
+#: bzrlib/builtins.py:6331
+msgid "Delete files unknown to bzr (default)."
+msgstr ""
+
+# help of 'dry-run' option of 'clean-tree' command
+#: bzrlib/builtins.py:6332
+msgid "Show files to delete instead of deleting them."
+msgstr ""
+
+# help of 'force' option of 'clean-tree' command
+#: bzrlib/builtins.py:6334
+msgid "Do not prompt before deleting."
+msgstr ""
+
+#: bzrlib/bundle/__init__.py:57
+msgid "A directory cannot be a bundle"
+msgstr ""
+
+#: bzrlib/bundle/serializer/v4.py:326
+#, python-format
+msgid "Bundling %d revision."
+msgid_plural "Bundling %d revisions."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/bzrdir.py:636
+msgid ""
+"making backup of {0}\n"
+"  to {1}"
+msgstr ""
+
+#: bzrlib/bzrdir.py:659
+msgid "renamed {0} to {1}"
+msgstr ""
+
+#: bzrlib/bzrdir.py:851
+msgid "{0} is{1} redirected to {2}"
+msgstr ""
+
+#: bzrlib/bzrdir.py:1727
+#, python-format
+msgid ""
+"Source repository format does not support stacking, using format:\n"
+"  %s"
+msgstr ""
+
+#: bzrlib/bzrdir.py:1747
+#, python-format
+msgid ""
+"Source branch format does not support stacking, using format:\n"
+"  %s"
+msgstr ""
+
+#: bzrlib/bzrdir.py:1858
+msgid "starting repository conversion"
+msgstr ""
+
+#: bzrlib/check.py:106
+msgid "check"
+msgstr ""
+
+#: bzrlib/check.py:108
+msgid "checking revisions"
+msgstr ""
+
+#: bzrlib/check.py:110
+msgid "checking commit contents"
+msgstr ""
+
+#: bzrlib/check.py:112
+msgid "checking file graphs"
+msgstr ""
+
+#: bzrlib/check.py:116
+msgid "checking branches and trees"
+msgstr ""
+
+#: bzrlib/check.py:203
+msgid "checked repository {0} format {1}"
+msgstr ""
+
+#: bzrlib/check.py:206
+#, python-format
+msgid "%6d revisions"
+msgstr ""
+
+#: bzrlib/check.py:207
+#, python-format
+msgid "%6d file-ids"
+msgstr ""
+
+#: bzrlib/check.py:209
+#, python-format
+msgid "%6d unreferenced text versions"
+msgstr ""
+
+#: bzrlib/check.py:213
+msgid "unreferenced version: {{{0}}} in {1}"
+msgstr ""
+
+#: bzrlib/check.py:216
+#, python-format
+msgid "%6d revisions are missing inventory_sha1"
+msgstr ""
+
+#: bzrlib/check.py:219
+#, python-format
+msgid "%6d revisions are mentioned but not present"
+msgstr ""
+
+#: bzrlib/check.py:222
+#, python-format
+msgid "%6d ghost revisions"
+msgstr ""
+
+#: bzrlib/check.py:227
+#, python-format
+msgid "%6d revisions missing parents in ancestry"
+msgstr ""
+
+#: bzrlib/check.py:231
+#, python-format
+msgid "      %s should be in the ancestry for:"
+msgstr ""
+
+#: bzrlib/check.py:235
+#, python-format
+msgid "%6d inconsistent parents"
+msgstr ""
+
+#: bzrlib/check.py:239
+msgid "      * {0} version {1} has parents {2!r} but should have {3!r}"
+msgstr ""
+
+#: bzrlib/check.py:245
+#, python-format
+msgid "%6d revisions have incorrect parents in the revision index"
+msgstr ""
+
+#: bzrlib/check.py:251
+msgid "       {0} has wrong parents in index: {1!r} should be {2!r}"
+msgstr ""
+
+#: bzrlib/check.py:265
+msgid "Mismatched internal revid {{{0}}} and index revid {{{1}}}"
+msgstr ""
+
+#: bzrlib/check.py:294
+msgid ""
+"Multiple expected sha1s for {0}. {{{1}}} expects {{{2}}}, {{{3}}} expects "
+"{{{4}}}"
+msgstr ""
+
+#: bzrlib/check.py:352
+#, python-format
+msgid "Checking branch at '%s'."
+msgstr ""
+
+#: bzrlib/check.py:372
+#, python-format
+msgid "Checking working tree at '%s'."
+msgstr ""
+
+#: bzrlib/check.py:425 bzrlib/check.py:439
+msgid "No branch found at specified location."
+msgstr ""
+
+#: bzrlib/check.py:427 bzrlib/check.py:437
+msgid "No working tree found at specified location."
+msgstr ""
+
+#: bzrlib/check.py:430
+#, python-format
+msgid "Checking repository at '%s'."
+msgstr ""
+
+#: bzrlib/check.py:441
+msgid "No repository found at specified location."
+msgstr ""
+
+#: bzrlib/clean_tree.py:62
+msgid "Nothing to delete."
+msgstr ""
+
+#: bzrlib/clean_tree.py:108
+#, python-format
+msgid "unable to remove %s"
+msgstr ""
+
+#: bzrlib/clean_tree.py:112
+msgid "deleting paths:"
+msgstr ""
+
+#: bzrlib/clean_tree.py:126
+msgid "unable to remove \"{0}\": {1}."
+msgstr ""
+
+#: bzrlib/clean_tree.py:131
+msgid "No files deleted."
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:45
+msgid "No known version info format {0}. Supported types are: {1}"
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:51
+msgid "Show version information about this tree."
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:53
+msgid ""
+"You can use this command to add information about version into\n"
+"source code of an application. The output can be in one of the\n"
+"supported formats or in a custom format based on a template."
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:57
+msgid "For example::"
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:59
+msgid ""
+"  bzr version-info --custom \\\n"
+"    --template=\"#define VERSION_INFO \\\"Project 1.2.3 (r{revno})\\\"\\n\""
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:62
+msgid ""
+"will produce a C header file with formatted string containing the\n"
+"current revision number. Other supported variables in templates are:"
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:65
+msgid ""
+"  * {date} - date of the last revision\n"
+"  * {build_date} - current date\n"
+"  * {revno} - revision number\n"
+"  * {revision_id} - revision id\n"
+"  * {branch_nick} - branch nickname\n"
+"  * {clean} - 0 if the source tree contains uncommitted changes,\n"
+"              otherwise 1"
+msgstr ""
+
+# help of 'format' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:75
+msgid "Select the output format."
+msgstr ""
+
+# help of 'all' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:79
+msgid "Include all possible information."
+msgstr ""
+
+# help of 'check-clean' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:80
+msgid "Check if tree is clean."
+msgstr ""
+
+# help of 'include-history' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:82
+msgid "Include the revision-history."
+msgstr ""
+
+# help of 'include-file-revisions' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:84
+msgid "Include the last revision for each file."
+msgstr ""
+
+# help of 'template' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:85
+msgid "Template for the output."
+msgstr ""
+
+#: bzrlib/commands.py:232
+#, python-format
+msgid "unknown command \"%s\""
+msgstr ""
+
+#: bzrlib/commands.py:479
+msgid "No help for this command."
+msgstr ""
+
+#: bzrlib/commands.py:492
+#, python-format
+msgid ":Purpose: %s\n"
+msgstr ""
+
+#: bzrlib/commands.py:494
+#, python-format
+msgid ""
+":Usage:\n"
+"%s\n"
+msgstr ""
+
+#: bzrlib/commands.py:496
+#, python-format
+msgid ":Usage:   %s\n"
+msgstr ""
+
+#: bzrlib/commands.py:515
+#, python-format
+msgid ":Options:%s"
+msgstr ""
+
+#: bzrlib/commands.py:526
+#, python-format
+msgid ""
+":Description:\n"
+"  %s\n"
+"\n"
+msgstr ""
+
+#: bzrlib/commands.py:536
+#, python-format
+msgid ""
+"See bzr help %s for more details and examples.\n"
+"\n"
+msgstr ""
+
+#: bzrlib/commands.py:541
+msgid ":Aliases:  "
+msgstr ""
+
+#: bzrlib/commands.py:545
+#, python-format
+msgid ":From:     plugin \"%s\"\n"
+msgstr ""
+
+#: bzrlib/commands.py:557
+msgid ":doc:`{0} <{1}-help>`"
+msgstr ""
+
+#: bzrlib/commands.py:561
+#, python-format
+msgid ":See also: %s"
+msgstr ""
+
+#: bzrlib/commands.py:819 bzrlib/gpg.py:317
+msgid "Only ASCII permitted in option names"
+msgstr ""
+
+#: bzrlib/commands.py:844 bzrlib/commands.py:852
+msgid "command {0!r} needs one or more {1}"
+msgstr ""
+
+#: bzrlib/commands.py:861
+msgid "command {0!r} requires argument {1}"
+msgstr ""
+
+#: bzrlib/commands.py:868
+msgid "extra argument to command {0}: {1}"
+msgstr ""
+
+#: bzrlib/commands.py:965
+#, python-format
+msgid "Profile data written to \"%s\"."
+msgstr ""
+
+#: bzrlib/commit.py:116 bzrlib/commit.py:709
+msgid "added"
+msgstr ""
+
+#: bzrlib/commit.py:116 bzrlib/commit.py:717
+msgid "modified"
+msgstr ""
+
+#: bzrlib/commit.py:130
+#, python-format
+msgid "Committing%s"
+msgstr ""
+
+#: bzrlib/commit.py:133
+#, python-format
+msgid "Committed revision %d."
+msgstr ""
+
+#: bzrlib/commit.py:142
+#, python-format
+msgid "deleted %s"
+msgstr ""
+
+#: bzrlib/commit.py:145
+#, python-format
+msgid "missing %s"
+msgstr ""
+
+#: bzrlib/commit.py:471
+msgid ""
+"Conflicting tags in bound branch:\n"
+"{0}"
+msgstr ""
+
+#: bzrlib/commit.py:711
+msgid "renamed"
+msgstr ""
+
+#: bzrlib/commit.py:946
+msgid "unchanged"
+msgstr ""
+
+#: bzrlib/commit.py:968
+msgid "{0} [{1}] - Stage"
+msgstr ""
+
+#: bzrlib/commit.py:971
+#, python-format
+msgid "%s - Stage"
+msgstr ""
+
+#: bzrlib/commit_signature_commands.py:34
+msgid "Sign all commits by a given committer."
+msgstr ""
+
+#: bzrlib/commit_signature_commands.py:36
+msgid ""
+"If location is not specified the local tree is used.\n"
+"If committer is not specified the default committer is used."
+msgstr ""
+
+#: bzrlib/commit_signature_commands.py:39
+msgid "This does not sign commits that already have signatures."
+msgstr ""
+
+# help of 'dry-run' option of 'sign-my-commits' command
+#: bzrlib/commit_signature_commands.py:47
+msgid ""
+"Don't actually sign anything, just print the revisions that would be signed."
+msgstr ""
+
+#: bzrlib/commit_signature_commands.py:101
+msgid "Verify all commit signatures."
+msgstr ""
+
+#: bzrlib/commit_signature_commands.py:103
+msgid "Verifies that all commits in the branch are signed by known GnuPG keys."
+msgstr ""
+
+# help of 'acceptable-keys' option of 'verify-signatures' command
+#: bzrlib/commit_signature_commands.py:108
+msgid ""
+"Comma separated list of GPG key patterns which are acceptable for "
+"verification."
+msgstr ""
+
+#: bzrlib/commit_signature_commands.py:145
+msgid "Cannot verify a range of non-revision-history revisions"
+msgstr ""
+
+#: bzrlib/commit_signature_commands.py:166
+msgid "All commits signed with verifiable keys"
+msgstr ""
+
+#: bzrlib/config.py:448
+msgid "Invalid config value for \"{0}\"  value {1!r} is not an SI unit."
+msgstr ""
+
+#: bzrlib/config.py:3293
+msgid "Display, set or remove a configuration option."
+msgstr ""
+
+#: bzrlib/config.py:3295
+msgid "Display the active value for a given option."
+msgstr ""
+
+#: bzrlib/config.py:3297
+msgid ""
+"If --all is specified, NAME is interpreted as a regular expression and all\n"
+"matching options are displayed mentioning their scope. The active value\n"
+"that bzr will take into account is the first one displayed for each option."
+msgstr ""
+
+#: bzrlib/config.py:3301
+msgid "If no NAME is given, --all .* is implied."
+msgstr ""
+
+#: bzrlib/config.py:3303
+msgid ""
+"Setting a value is achieved by using name=value without spaces. The value\n"
+"is set in the most relevant scope and can be checked by displaying the\n"
+"option again."
+msgstr ""
+
+# help of 'scope' option of 'config' command
+#: bzrlib/config.py:3314
+msgid "Reduce the scope to the specified configuration file"
+msgstr ""
+
+# help of 'all' option of 'config' command
+#: bzrlib/config.py:3318
+msgid "Display all the defined values for the matching options."
+msgstr ""
+
+# help of 'remove' option of 'config' command
+#: bzrlib/config.py:3320
+msgid "Remove the option from the configuration file"
+msgstr ""
+
+#: bzrlib/conflicts.py:48
+msgid "List files with conflicts."
+msgstr ""
+
+#: bzrlib/conflicts.py:50
+msgid ""
+"Merge will do its best to combine the changes in two branches, but there\n"
+"are some kinds of problems only a human can fix.  When it encounters those,\n"
+"it will mark a conflict.  A conflict means that you need to fix something,\n"
+"before you should commit."
+msgstr ""
+
+#: bzrlib/conflicts.py:55
+msgid ""
+"Conflicts normally are listed as short, human-readable messages.  If --text\n"
+"is supplied, the pathnames of files with text conflicts are listed,\n"
+"instead.  (This is useful for editing all files with text conflicts.)"
+msgstr ""
+
+#: bzrlib/conflicts.py:59
+msgid "Use bzr resolve when you have fixed a problem."
+msgstr ""
+
+# help of 'text' option of 'conflicts' command
+#: bzrlib/conflicts.py:64
+msgid "List paths of files with text conflicts."
+msgstr ""
+
+# help of 'action' option of 'resolve' command
+#: bzrlib/conflicts.py:96
+msgid "How to resolve the conflict."
+msgstr ""
+
+#: bzrlib/conflicts.py:102
+msgid "Mark a conflict as resolved."
+msgstr ""
+
+#: bzrlib/conflicts.py:109
+msgid ""
+"Once you have fixed a problem, use \"bzr resolve\" to automatically mark\n"
+"text conflicts as fixed, \"bzr resolve FILE\" to mark a specific conflict "
+"as\n"
+"resolved, or \"bzr resolve --all\" to mark all conflicts as resolved."
+msgstr ""
+
+# help of 'all' option of 'resolve' command
+#: bzrlib/conflicts.py:117
+msgid "Resolve all conflicts in this tree."
+msgstr ""
+
+#: bzrlib/conflicts.py:124
+msgid "If --all is specified, no FILE may be provided"
+msgstr ""
+
+#: bzrlib/conflicts.py:149
+#, python-format
+msgid "%d conflict auto-resolved."
+msgid_plural "%d conflicts auto-resolved."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/conflicts.py:152
+msgid "Remaining conflicts:"
+msgstr ""
+
+#: bzrlib/conflicts.py:157
+msgid "All conflicts resolved."
+msgstr ""
+
+#: bzrlib/conflicts.py:166
+msgid "{0} conflict resolved, {1} remaining"
+msgid_plural "{0} conflicts resolved, {1} remaining"
+msgstr[0] ""
+msgstr[1] ""
+
+# title of 'action' option of 'resolve' command
+#: bzrlib/conflicts.py:692
+msgid "action"
+msgstr ""
+
+#: bzrlib/diff.py:397
+#, python-format
+msgid "*** Ignoring files outside view. View is %s"
+msgstr ""
+
+#: bzrlib/errors.py:215
+msgid "The tree builder is already building a tree."
+msgstr ""
+
+#: bzrlib/errors.py:236
+msgid "The dirstate file (%(state)s) appears to be corrupt: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:255
+msgid ""
+"The API for \"%(api)s\" is not compatible with \"%(wanted)s\". It supports "
+"versions \"%(minimum)s\" to \"%(current)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:267
+msgid "The transport '%(transport)s' is only accessible within this process."
+msgstr ""
+
+#: bzrlib/errors.py:285
+msgid "Invalid revision number %(revno)s"
+msgstr ""
+
+#: bzrlib/errors.py:294
+msgid "Invalid revision-id {%(revision_id)s} in %(branch)s"
+msgstr ""
+
+#: bzrlib/errors.py:305
+msgid "Reserved revision-id {%(revision_id)s}"
+msgstr ""
+
+#: bzrlib/errors.py:319
+msgid "There is no public branch set for \"%(branch_url)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:329
+msgid ""
+"No help could be found for '%(topic)s'. Please use 'bzr help topics' to "
+"obtain a list of topics."
+msgstr ""
+
+#: bzrlib/errors.py:338
+msgid "The file id \"%(file_id)s\" is not present in the tree %(tree)s."
+msgstr ""
+
+#: bzrlib/errors.py:348
+msgid ""
+"The file id \"%(file_id)s\" is not present in the repository %(repository)r"
+msgstr ""
+
+#: bzrlib/errors.py:357
+msgid "The branch '%(branch)s' is not stacked."
+msgstr ""
+
+#: bzrlib/errors.py:371
+msgid "No WorkingTree exists for \"%(base)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:380
+msgid "Not currently building a tree."
+msgstr ""
+
+#: bzrlib/errors.py:385
+msgid "%(url)s is not a local path."
+msgstr ""
+
+#: bzrlib/errors.py:413
+msgid "%(not_locked)r is not write locked but needs to be."
+msgstr ""
+
+#: bzrlib/errors.py:421
+msgid "Error in command line options"
+msgstr ""
+
+#: bzrlib/errors.py:426
+msgid "%(value)s is not an index of type %(_type)s."
+msgstr ""
+
+#: bzrlib/errors.py:436
+msgid "Error in data for index %(value)s."
+msgstr ""
+
+#: bzrlib/errors.py:445
+msgid "The key '%(key)s' is already in index '%(index)s'."
+msgstr ""
+
+#: bzrlib/errors.py:455
+msgid "The key '%(key)s' is not a valid key."
+msgstr ""
+
+#: bzrlib/errors.py:464
+msgid "Could not parse options for index %(value)s."
+msgstr ""
+
+#: bzrlib/errors.py:473
+msgid "The value '%(value)s' is not a valid value."
+msgstr ""
+
+#: bzrlib/errors.py:501
+msgid "Generic path error: %(path)r%(extra)s)"
+msgstr ""
+
+#: bzrlib/errors.py:514
+msgid "No such file: %(path)r%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:519
+msgid "File exists: %(path)r%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:525
+msgid ""
+"Could not rename %(source)s => %(dest)s because both files exist. (Use --"
+"after to tell bzr about a rename that has already happened)%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:541
+msgid "\"%(path)s\" is not a directory %(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:546
+msgid "\"%(path)s\" is not in the working directory %(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:551
+msgid "Directory not empty: \"%(path)s\"%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:556
+msgid "Hard-linking \"%(path)s\" is not supported"
+msgstr ""
+
+#: bzrlib/errors.py:571
+msgid "Device or resource busy: \"%(path)s\"%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:576
+msgid "Permission denied: \"%(path)s\"%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:581
+msgid "Invalid url supplied to transport: \"%(path)s\"%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:586
+msgid "Invalid URL join request: %(reason)s: %(base)r + %(join_args)r"
+msgstr ""
+
+#: bzrlib/errors.py:597
+msgid "URLs differ by more than path: %(from_)r and %(to)r"
+msgstr ""
+
+#: bzrlib/errors.py:619
+msgid "The %(type)s hook '%(hook)s' is unknown in this version of bzrlib."
+msgstr ""
+
+#: bzrlib/errors.py:629
+msgid "Unsupported protocol for url \"%(path)s\"%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:637
+msgid ""
+"The branch '%(url)s'(%(format)s) is not a stackable format. You will need to "
+"upgrade the branch to permit branch stacking."
+msgstr ""
+
+#: bzrlib/errors.py:648
+msgid "The branch '%(branch_url)s' cannot be stacked on '%(target_url)s'."
+msgstr ""
+
+#: bzrlib/errors.py:658
+msgid ""
+"The repository '%(url)s'(%(format)s) is not a stackable format. You will "
+"need to upgrade the repository to permit branch stacking."
+msgstr ""
+
+#: bzrlib/errors.py:669
+msgid "Error reading from %(path)r."
+msgstr ""
+
+#: bzrlib/errors.py:688
+msgid "Path \"%(path)s\" is not a child of path \"%(base)s\"%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:704
+msgid "Path \"%(path)s\" is not unicode normalized"
+msgstr ""
+
+#: bzrlib/errors.py:712
+msgid "Not a branch: \"%(path)s\"%(detail)s."
+msgstr ""
+
+#: bzrlib/errors.py:754
+msgid "No submit branch available for branch \"%(path)s\""
+msgstr ""
+
+#: bzrlib/errors.py:763
+msgid "Already a branch: \"%(path)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:769
+msgid ""
+"Directory contains a branch, but no working tree (use bzr checkout if you "
+"wish to build a working tree): \"%(path)s\""
+msgstr ""
+
+#: bzrlib/errors.py:774
+msgid ""
+"\"%(function)s\" called on an AtomicFile after it was closed: \"%(path)s\""
+msgstr ""
+
+#: bzrlib/errors.py:784
+msgid ""
+"Parent not accessible given base \"%(base)s\" and relative path \"%(path)s\""
+msgstr ""
+
+#: bzrlib/errors.py:794
+msgid "No repository present: \"%(path)s\""
+msgstr ""
+
+#: bzrlib/errors.py:802
+msgid "File \"%(path)s\" is not in branch %(branch_base)s."
+msgstr ""
+
+#: bzrlib/errors.py:814
+msgid ""
+"Unsupported branch format: %(format)s\n"
+"Please run 'bzr upgrade'"
+msgstr ""
+
+#: bzrlib/errors.py:820
+msgid "Unknown %(kind)s format: %(format)r"
+msgstr ""
+
+#: bzrlib/errors.py:829
+msgid "Format %(format)s is not compatible with .bzr version %(bzrdir)s."
+msgstr ""
+
+#: bzrlib/errors.py:842
+msgid ""
+"%(target)s\n"
+"is not compatible with\n"
+"%(source)s\n"
+"%(details)s"
+msgstr ""
+
+#: bzrlib/errors.py:858
+msgid "Revision is not compatible with %(repo_format)s"
+msgstr ""
+
+#: bzrlib/errors.py:868
+msgid "%(context_info)s%(path)s is already versioned."
+msgstr ""
+
+#: bzrlib/errors.py:889
+msgid "%(context_info)s%(path)s is not versioned."
+msgstr ""
+
+#: bzrlib/errors.py:910
+msgid "Path(s) are not versioned: %(paths_as_string)s"
+msgstr ""
+
+#: bzrlib/errors.py:921
+msgid "Path(s) do not exist: %(paths_as_string)s%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:940
+msgid "Cannot operate on \"%(filename)s\" of unsupported kind \"%(kind)s\""
+msgstr ""
+
+#: bzrlib/errors.py:948
+msgid ""
+"Filename %(filename)r is not valid in your current filesystem encoding "
+"%(fs_encoding)s"
+msgstr ""
+
+#: bzrlib/errors.py:959
+msgid "Cannot operate on \"%(filename)s\" because it is a control file"
+msgstr ""
+
+#: bzrlib/errors.py:977
+msgid "The lock for '%(lock_description)s' is in use and cannot be broken."
+msgstr ""
+
+#: bzrlib/errors.py:1016
+msgid "Cannot lock %(lock)s: %(why)s"
+msgstr ""
+
+#: bzrlib/errors.py:1026
+msgid ""
+"A transaction related operation was attempted after the transaction finished."
+msgstr ""
+
+#: bzrlib/errors.py:1053
+msgid "Cannot lock: transport is read only: %(transport)s"
+msgstr ""
+
+#: bzrlib/errors.py:1061
+msgid "Could not acquire lock \"%(lock)s\": %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1072
+msgid "Lock was broken while still open: %(lock)s - check storage consistency!"
+msgstr ""
+
+#: bzrlib/errors.py:1083
+msgid ""
+"Lock was released and re-acquired before being broken: %(lock)s: held by "
+"%(holder)r, wanted to break %(target)r"
+msgstr ""
+
+#: bzrlib/errors.py:1095
+msgid ""
+"Lock is apparently held, but corrupted: %(corruption_info)s\n"
+"Use 'bzr break-lock' to clear it"
+msgstr ""
+
+#: bzrlib/errors.py:1108
+msgid "Lock not held: %(lock)s"
+msgstr ""
+
+#: bzrlib/errors.py:1137
+msgid "No changes to commit"
+msgstr ""
+
+#: bzrlib/errors.py:1142
+msgid ""
+"Selected-file commit of merges is not supported yet: files %(files_str)s"
+msgstr ""
+
+#: bzrlib/errors.py:1152
+msgid ""
+"Excluding paths during commit is not supported by repository at %(repository)"
+"r."
+msgstr ""
+
+#: bzrlib/errors.py:1161
+msgid ""
+"The specified commit message contains characters unsupported by the current "
+"encoding."
+msgstr ""
+
+#: bzrlib/errors.py:1167
+msgid "Upgrade URL cannot work with readonly URLs."
+msgstr ""
+
+#: bzrlib/errors.py:1172
+msgid "The branch format %(format)s is already at the most recent format."
+msgstr ""
+
+#: bzrlib/errors.py:1195
+msgid "Option --change does not accept revision ranges"
+msgstr ""
+
+#: bzrlib/errors.py:1200
+msgid "No namespace registered for string: %(spec)r"
+msgstr ""
+
+#: bzrlib/errors.py:1219
+msgid ""
+"Requested revision: '%(spec)s' does not exist in branch: %(branch_url)s"
+"%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:1233
+msgid "%(branch)s is missing %(object_type)s {%(object_id)s}"
+msgstr ""
+
+#: bzrlib/errors.py:1238
+msgid ""
+"Operation denied because it would change the main history, which is not "
+"permitted by the append_revisions_only setting on branch \"%(location)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:1249
+msgid ""
+"These branches have diverged. Use the missing command to see how.\n"
+"Use the merge command to reconcile them."
+msgstr ""
+
+#: bzrlib/errors.py:1269
+msgid ""
+"Branches have no common ancestor, and no merge base revision was specified."
+msgstr ""
+
+#: bzrlib/errors.py:1275
+msgid ""
+"Selected merge cannot perform reverse cherrypicks.  Try merge3 or diff3."
+msgstr ""
+
+#: bzrlib/errors.py:1281
+msgid "Revisions have no common ancestor: %(revision_a)s %(revision_b)s"
+msgstr ""
+
+#: bzrlib/errors.py:1290
+msgid ""
+"Revisions are not derived from the same root: %(revision_a)s %(revision_b)s."
+msgstr ""
+
+#: bzrlib/errors.py:1299
+msgid "Revision %(rev_id)s is not an ancestor of %(not_ancestor_id)s"
+msgstr ""
+
+#: bzrlib/errors.py:1319
+msgid "Branch %(branch)s has no commits."
+msgstr ""
+
+#: bzrlib/errors.py:1337
+msgid ""
+"Bound branch %(branch)s is out of date with master branch %(master)s."
+"%(extra_help)s"
+msgstr ""
+
+#: bzrlib/errors.py:1349
+msgid ""
+"Cannot commit to branch %(branch)s. It is bound to %(master)s, which is "
+"bound to %(remote)s."
+msgstr ""
+
+#: bzrlib/errors.py:1361
+msgid "Cannot pull --overwrite to a branch which is bound %(branch)s"
+msgstr ""
+
+#: bzrlib/errors.py:1370
+msgid ""
+"Unable to connect to target of bound branch %(branch)s => %(target)s: "
+"%(error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1382
+msgid "Error in processing weave: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1391
+msgid "Revision {%(revision_id)s} already present in %(weave)s"
+msgstr ""
+
+#: bzrlib/errors.py:1402
+msgid "Revision {%(revision_id)s} not present in %(weave)s"
+msgstr ""
+
+#: bzrlib/errors.py:1412
+msgid "Weave invariant violated: %(what)s"
+msgstr ""
+
+#: bzrlib/errors.py:1421
+msgid "Parents are mismatched between two revisions. %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1443
+msgid ""
+"Weaves differ on text content. Revision: {%(revision_id)s}, %(weave_a)s, "
+"%(weave_b)s"
+msgstr ""
+
+#: bzrlib/errors.py:1455
+msgid "Versioned file error"
+msgstr ""
+
+#: bzrlib/errors.py:1460
+msgid "Revision {%(revision_id)s} not present in \"%(file_id)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:1470
+msgid "Revision {%(revision_id)s} already present in \"%(file_id)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:1480
+msgid "Text did not match its checksum: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1605
+msgid "Export format %(format)r not supported"
+msgstr ""
+
+#: bzrlib/errors.py:1614
+msgid "Transport error: %(msg)s %(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1640
+msgid "Generic bzr smart protocol error: %(details)s"
+msgstr ""
+
+#: bzrlib/errors.py:1648
+msgid "Received bad protocol version marker: %(marker)r"
+msgstr ""
+
+#: bzrlib/errors.py:1679
+msgid "Transport operation not possible: %(msg)s %(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1684
+msgid "Connection error: %(msg)s %(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1689
+msgid "%(msg)s %(host)s%(port)s%(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1710
+msgid "Connection closed: %(msg)s %(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1715
+msgid "Invalid range access in %(path)s at %(offset)s: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1725
+msgid "Invalid http response for %(path)s: %(msg)s%(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1740
+msgid "Invalid http range %(range)r for %(path)s: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1754
+msgid "HTTP MIME Boundary missing for %(path)s: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1762
+msgid "Invalid http Content-type \"%(ctype)s\" for %(path)s: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1771
+msgid "%(source)s is%(permanently)s redirected to %(target)s"
+msgstr ""
+
+#: bzrlib/errors.py:1785
+msgid "Too many redirections"
+msgstr ""
+
+#: bzrlib/errors.py:1790
+msgid "Working tree has conflicts."
+msgstr ""
+
+#: bzrlib/errors.py:1794
+msgid "Config file %(filename)s is not UTF-8 encoded\n"
+msgstr ""
+
+#: bzrlib/errors.py:1803
+msgid ""
+"Error(s) parsing config file %(filename)s:\n"
+"%(errors)s"
+msgstr ""
+
+#: bzrlib/errors.py:1814
+msgid "Bad value \"%(value)s\" for option \"%(name)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:1822
+msgid "%(username)r does not seem to contain a reasonable email address"
+msgstr ""
+
+#: bzrlib/errors.py:1831
+msgid "Failed to GPG sign data with command \"%(command_line)s\""
+msgstr ""
+
+#: bzrlib/errors.py:1839
+msgid "Failed to verify GPG signature data with error \"%(error)s\""
+msgstr ""
+
+#: bzrlib/errors.py:1847
+msgid "Unable to import library \"%(library)s\": %(error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1855
+msgid "python-gpgme is not installed, it is needed to verify signatures"
+msgstr ""
+
+#: bzrlib/errors.py:1863
+msgid ""
+"The working tree for %(basedir)s has changed since the last commit, but "
+"weave merge requires that it be unchanged"
+msgstr ""
+
+#: bzrlib/errors.py:1873
+msgid ""
+"Can't reprocess and show base, because reprocessing obscures the "
+"relationship of conflicting lines to the base"
+msgstr ""
+
+#: bzrlib/errors.py:1879
+msgid "Cycle in graph %(graph)r"
+msgstr ""
+
+#: bzrlib/errors.py:1907
+msgid "File %(filename)s is not conflicted."
+msgstr ""
+
+#: bzrlib/errors.py:1929
+msgid "No bundle was found in \"%(filename)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:1938
+msgid "Unable to handle bundle version %(version)s: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:1948
+msgid "Branch %(base)s is missing revision %(text_revision)s of %(file_id)s"
+msgstr ""
+
+#: bzrlib/errors.py:1961
+msgid "File id {%(file_id)s} already exists in inventory as %(entry)s"
+msgstr ""
+
+#: bzrlib/errors.py:1971
+msgid "Key %(key)s is already present in map"
+msgstr ""
+
+#: bzrlib/errors.py:1976
+msgid "The prefix %(prefix)s is in the help search path twice."
+msgstr ""
+
+#: bzrlib/errors.py:1984
+msgid "Tree transform is malformed %(conflicts)r"
+msgstr ""
+
+#: bzrlib/errors.py:1986
+msgid ""
+"No final name for trans_id %(trans_id)r\n"
+"file-id: %(file_id)r\n"
+"root trans-id: %(root_trans_id)r\n"
+msgstr ""
+
+#: bzrlib/errors.py:2018
+msgid "Attempt to reuse a transform that has already been applied."
+msgstr ""
+
+#: bzrlib/errors.py:2023
+msgid "Moving the root directory is not supported at this time"
+msgstr ""
+
+#: bzrlib/errors.py:2028
+msgid "Failed to rename %(from_path)s to %(to_path)s: %(why)s"
+msgstr ""
+
+#: bzrlib/errors.py:2039
+msgid ""
+"Could not move %(from_path)s%(operator)s %(to_path)s%(_has_extra)s%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:2075
+msgid ""
+"Could not rename %(from_path)s%(operator)s %(to_path)s%(_has_extra)s%(extra)s"
+msgstr ""
+
+#: bzrlib/errors.py:2084
+msgid ""
+"Can't safely remove modified or unknown files:\n"
+"%(changes_as_text)sUse --keep to not delete them, or --force to delete them "
+"regardless."
+msgstr ""
+
+#: bzrlib/errors.py:2121
+msgid "Unable to import paramiko (required for sftp support): %(error)s"
+msgstr ""
+
+#: bzrlib/errors.py:2129
+msgid "Nothing to merge."
+msgstr ""
+
+#: bzrlib/errors.py:2134
+msgid "Format %(format)s cannot be initialised by this version of bzr."
+msgstr ""
+
+#: bzrlib/errors.py:2143
+msgid ""
+"Cannot convert from format %(from_format)s to format %(format)s.    "
+"%(problem)s"
+msgstr ""
+
+#: bzrlib/errors.py:2155
+msgid "Could not find an appropriate Differ for file \"%(path)s\""
+msgstr ""
+
+#: bzrlib/errors.py:2163
+msgid "%(exe_name)s could not be found on this machine"
+msgstr ""
+
+#: bzrlib/errors.py:2171
+msgid "Diff is not installed on this machine: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:2179
+msgid "Diff3 is not installed on this machine."
+msgstr ""
+
+#: bzrlib/errors.py:2185
+msgid "The content being inserted is already present."
+msgstr ""
+
+#: bzrlib/errors.py:2190
+msgid ""
+"This tree contains left-over files from a failed operation.\n"
+"    Please examine %(limbo_dir)s to see if it contains any files you wish "
+"to\n"
+"    keep, and delete it when you are done."
+msgstr ""
+
+#: bzrlib/errors.py:2201
+msgid ""
+"This tree contains left-over files from a failed operation.\n"
+"    Please examine %(pending_deletion)s to see if it contains any files you\n"
+"    wish to keep, and delete it when you are done."
+msgstr ""
+
+#: bzrlib/errors.py:2211
+msgid ""
+"Unable to delete transform temporary directory %(limbo_dir)s.\n"
+"    Please examine %(limbo_dir)s to see if it contains any files you wish "
+"to\n"
+"    keep, and delete it when you are done."
+msgstr ""
+
+#: bzrlib/errors.py:2222
+msgid ""
+"Unable to delete transform temporary directory %(pending_deletion)s.  Please "
+"examine %(pending_deletion)s to see if it contains any files you wish to "
+"keep, and delete it when you are done."
+msgstr ""
+
+#: bzrlib/errors.py:2232
+msgid "Working tree is out of date, please run 'bzr update'.%(more)s"
+msgstr ""
+
+#: bzrlib/errors.py:2246
+msgid "Public branch \"%(public_location)s\" lacks revision \"%(revstring)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:2259
+msgid "Error in merge modified format"
+msgstr ""
+
+#: bzrlib/errors.py:2264
+msgid "Format error in conflict listings"
+msgstr ""
+
+#: bzrlib/errors.py:2268
+msgid ""
+"Inconsistency in dirstate file %(dirstate_path)s.\n"
+"Error: %(description)s"
+msgstr ""
+
+#: bzrlib/errors.py:2279
+msgid ""
+"An error has been detected in the repository %(repo_path)s.\n"
+"Please run bzr reconcile on this repository."
+msgstr ""
+
+#: bzrlib/errors.py:2290
+msgid ""
+"An inconsistent delta was supplied involving %(path)r, %(file_id)r\n"
+"reason: %(reason)s"
+msgstr ""
+
+#: bzrlib/errors.py:2303
+msgid ""
+"An inconsistent delta was supplied: %(delta)r\n"
+"reason: %(reason)s"
+msgstr ""
+
+#: bzrlib/errors.py:2315
+msgid "To use this feature you must upgrade your branch at %(path)s."
+msgstr ""
+
+#: bzrlib/errors.py:2324
+msgid "To use this feature you must upgrade your repository at %(path)s."
+msgstr ""
+
+#: bzrlib/errors.py:2329
+msgid ""
+"To use this feature you must upgrade your branch at %(path)s to a format "
+"which supports rich roots."
+msgstr ""
+
+#: bzrlib/errors.py:2335
+msgid "Cannot perform local-only commits on unbound branches."
+msgstr ""
+
+#: bzrlib/errors.py:2340
+msgid "The method %(mname)s is not supported on objects of type %(tname)s."
+msgstr ""
+
+#: bzrlib/errors.py:2361
+msgid "Ghost tags not supported by format %(format)r."
+msgstr ""
+
+#: bzrlib/errors.py:2369
+msgid "File is binary but should be text."
+msgstr ""
+
+#: bzrlib/errors.py:2374
+msgid "The path %(path)s is not permitted on this platform"
+msgstr ""
+
+#: bzrlib/errors.py:2383
+msgid ""
+"Testament did not match expected value.\n"
+"       For revision_id {%(revision_id)s}, expected {%(expected)s}, measured\n"
+"       {%(measured)s}"
+msgstr ""
+
+#: bzrlib/errors.py:2395
+msgid "Not a bzr revision-bundle: %(text)r"
+msgstr ""
+
+#: bzrlib/errors.py:2404
+msgid "Bad bzr revision-bundle: %(text)r"
+msgstr ""
+
+#: bzrlib/errors.py:2413
+msgid "Malformed bzr revision-bundle header: %(text)r"
+msgstr ""
+
+#: bzrlib/errors.py:2418
+msgid "Malformed patches in bzr revision-bundle: %(text)r"
+msgstr ""
+
+#: bzrlib/errors.py:2423
+msgid "Malformed footer in bzr revision-bundle: %(text)r"
+msgstr ""
+
+#: bzrlib/errors.py:2428
+msgid "End of line marker was not \\n in bzr revision-bundle"
+msgstr ""
+
+#: bzrlib/errors.py:2438
+msgid "Bundle format %(bundle_format)s is incompatible with %(other)s"
+msgstr ""
+
+#: bzrlib/errors.py:2448
+msgid "Root class for inventory serialization errors"
+msgstr ""
+
+#: bzrlib/errors.py:2452
+msgid ""
+"The inventory was not in the expected format:\n"
+" %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:2461
+msgid "This operation requires rich root data storage"
+msgstr ""
+
+#: bzrlib/errors.py:2474
+msgid "Unrecognised value for BZR_SSH environment variable: %(vendor)s"
+msgstr ""
+
+#: bzrlib/errors.py:2483
+msgid ""
+"Don't know how to handle SSH connections. Please set BZR_SSH environment "
+"variable."
+msgstr ""
+
+#: bzrlib/errors.py:2490
+msgid ""
+"Could not determine revno for {%(revision_id)s} because its ancestry shows a "
+"ghost at {%(ghost_revision_id)s}"
+msgstr ""
+
+#: bzrlib/errors.py:2500
+msgid "Ghost revision {%(revision_id)s} cannot be used here."
+msgstr ""
+
+#: bzrlib/errors.py:2551
+msgid ""
+"A merge directive must provide either a bundle or a public branch location."
+msgstr ""
+
+#: bzrlib/errors.py:2558
+msgid "Bad merge directive payload %(start)r"
+msgstr ""
+
+#: bzrlib/errors.py:2568
+msgid "Preview patch does not match requested changes."
+msgstr ""
+
+#: bzrlib/errors.py:2574
+msgid "Patch_type was %(patch_type)s, but no patch was supplied."
+msgstr ""
+
+#: bzrlib/errors.py:2584
+msgid ""
+"Your branch does not have all of the revisions required in order to merge "
+"this merge directive and the target location specified in the merge "
+"directive is not a branch: %(location)s."
+msgstr ""
+
+#: bzrlib/errors.py:2596
+msgid "Unsupported entry kind %(kind)s"
+msgstr ""
+
+#: bzrlib/errors.py:2604
+msgid "Can't subsume %(other_tree)s into %(tree)s. %(reason)s"
+msgstr ""
+
+#: bzrlib/errors.py:2614
+msgid "Subsume target %(other_tree)s needs to be upgraded."
+msgstr ""
+
+#: bzrlib/errors.py:2633
+msgid "No such tag: %(tag_name)s"
+msgstr ""
+
+#: bzrlib/errors.py:2641
+msgid "Tags not supported by %(branch)s; you may be able to use bzr upgrade."
+msgstr ""
+
+#: bzrlib/errors.py:2650
+msgid "Tag %(tag_name)s already exists."
+msgstr ""
+
+#: bzrlib/errors.py:2658
+msgid ""
+"Did not understand bug identifier %(bug_id)s: %(reason)s. See \"bzr help bugs"
+"\" for more information on this feature."
+msgstr ""
+
+#: bzrlib/errors.py:2668
+msgid ""
+"The URL for bug tracker \"%(abbreviation)s\" doesn't contain {id}: %(url)s"
+msgstr ""
+
+#: bzrlib/errors.py:2678
+msgid ""
+"Cannot find registered bug tracker called %(abbreviation)s on %(branch)s"
+msgstr ""
+
+#: bzrlib/errors.py:2688
+msgid "Invalid line in bugs property: '%(line)s'"
+msgstr ""
+
+#: bzrlib/errors.py:2696
+msgid "Invalid bug status: '%(status)s'"
+msgstr ""
+
+#: bzrlib/errors.py:2704
+msgid "Could not understand response from smart server: %(response_tuple)r"
+msgstr ""
+
+#: bzrlib/errors.py:2743
+msgid "Server sent an unexpected error: %(error_tuple)r"
+msgstr ""
+
+#: bzrlib/errors.py:2762
+msgid "Unrecognised container format: %(container_format)r"
+msgstr ""
+
+#: bzrlib/errors.py:2770
+msgid "Unexpected end of container stream"
+msgstr ""
+
+#: bzrlib/errors.py:2775
+msgid "Unknown record type: %(record_type)r"
+msgstr ""
+
+#: bzrlib/errors.py:2783
+msgid "Invalid record: %(reason)s"
+msgstr ""
+
+#: bzrlib/errors.py:2791
+msgid "Container has data after end marker: %(excess)r"
+msgstr ""
+
+#: bzrlib/errors.py:2799
+msgid "Container has multiple records with the same name: %(name)s"
+msgstr ""
+
+#: bzrlib/errors.py:2812
+msgid "Corrupt or incompatible data stream: %(reason)s"
+msgstr ""
+
+#: bzrlib/errors.py:2820
+msgid "SMTP error: %(error)s"
+msgstr ""
+
+#: bzrlib/errors.py:2828
+msgid "No message supplied."
+msgstr ""
+
+#: bzrlib/errors.py:2833
+msgid "No mail-to address (--mail-to) or output (-o) specified."
+msgstr ""
+
+#: bzrlib/errors.py:2838
+msgid "Unknown mail client: %(mail_client)s"
+msgstr ""
+
+#: bzrlib/errors.py:2846
+msgid ""
+"Unable to find mail client with the following names: "
+"%(mail_command_list_string)s"
+msgstr ""
+
+#: bzrlib/errors.py:2856
+msgid "SMTP connection to %(host)s refused"
+msgstr ""
+
+#: bzrlib/errors.py:2865
+msgid "Please specify smtp_server.  No server at default %(host)s."
+msgstr ""
+
+#: bzrlib/errors.py:2879
+msgid ""
+"'%(display_url)s' is not in sync with %(target_url)s.  See bzr help sync-for-"
+"reconfigure."
+msgstr ""
+
+#: bzrlib/errors.py:2891
+msgid "'%(display_url)s' is already a branch."
+msgstr ""
+
+#: bzrlib/errors.py:2896
+msgid "'%(display_url)s' is already a tree."
+msgstr ""
+
+#: bzrlib/errors.py:2901
+msgid "'%(display_url)s' is already a checkout."
+msgstr ""
+
+#: bzrlib/errors.py:2906
+msgid "'%(display_url)s' is already a lightweight checkout."
+msgstr ""
+
+#: bzrlib/errors.py:2911
+msgid "'%(display_url)s' is already using a shared repository."
+msgstr ""
+
+#: bzrlib/errors.py:2916
+msgid "'%(display_url)s' is already standalone."
+msgstr ""
+
+#: bzrlib/errors.py:2921
+msgid "Shared repository '%(display_url)s' already creates working trees."
+msgstr ""
+
+#: bzrlib/errors.py:2927
+msgid ""
+"Shared repository '%(display_url)s' already doesn't create working trees."
+msgstr ""
+
+#: bzrlib/errors.py:2933
+msgid "Requested reconfiguration of '%(display_url)s' is not supported."
+msgstr ""
+
+#: bzrlib/errors.py:2938
+msgid "No location could be found to bind to at %(display_url)s."
+msgstr ""
+
+#: bzrlib/errors.py:2943
+msgid ""
+"Working tree \"%(display_url)s\" has uncommitted changes (See bzr status)."
+"%(more)s"
+msgstr ""
+
+#: bzrlib/errors.py:2962
+msgid ""
+"Working tree \"%(display_url)s\" has shelved changes (See bzr shelve --list)."
+"%(more)s"
+msgstr ""
+
+#: bzrlib/errors.py:2968
+msgid "Variable {%(name)s} is not available."
+msgstr ""
+
+#: bzrlib/errors.py:2976
+msgid "No template specified."
+msgstr ""
+
+#: bzrlib/errors.py:2981
+msgid "Unable to create symlink %(path_str)son this platform"
+msgstr ""
+
+#: bzrlib/errors.py:2996
+msgid ""
+"Unsupported timezone format \"%(timezone)s\", options are \"utc\", \"original"
+"\", \"local\"."
+msgstr ""
+
+#: bzrlib/errors.py:3030
+msgid ""
+"Unable to encode %(kind)s path %(path)r in user encoding %(user_encoding)s"
+msgstr ""
+
+#: bzrlib/errors.py:3042
+msgid "The \"%(config_id)s\" configuration does not exist."
+msgstr ""
+
+#: bzrlib/errors.py:3050
+msgid "The \"%(option_name)s\" configuration option does not exist."
+msgstr ""
+
+#: bzrlib/errors.py:3058
+msgid "The alias \"%(alias_name)s\" does not exist."
+msgstr ""
+
+#: bzrlib/errors.py:3072
+msgid "\"%(alias_name)s\" is not a valid location alias."
+msgstr ""
+
+#: bzrlib/errors.py:3080
+msgid "No %(alias_name)s location assigned."
+msgstr ""
+
+#: bzrlib/errors.py:3088
+msgid "Cannot bind address \"%(host)s:%(port)i\": %(orig_error)s."
+msgstr ""
+
+#: bzrlib/errors.py:3098
+msgid "Unknown rules detected: %(unknowns_str)s."
+msgstr ""
+
+#: bzrlib/errors.py:3111
+msgid ""
+"Hook '%(hook_name)s' during %(hook_stage)s failed:\n"
+"%(traceback_text)s%(exc_value)s"
+msgstr ""
+
+#: bzrlib/errors.py:3134
+msgid "Tip change rejected: %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:3142
+msgid "Shelf corrupt."
+msgstr ""
+
+#: bzrlib/errors.py:3147
+msgid "Corruption while decompressing repository file%(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:3159
+msgid "No changes are shelved with id \"%(shelf_id)d\"."
+msgstr ""
+
+#: bzrlib/errors.py:3167
+msgid "\"%(invalid_id)s\" is not a valid shelf id, try a number instead."
+msgstr ""
+
+#: bzrlib/errors.py:3175
+msgid "An attempt to access a url outside the server jail was made: '%(url)s'."
+msgstr ""
+
+#: bzrlib/errors.py:3183
+msgid "The user aborted the operation."
+msgstr ""
+
+#: bzrlib/errors.py:3188
+msgid "Branching '%(url)s'(%(format)s) must create a working tree."
+msgstr ""
+
+#: bzrlib/errors.py:3198
+msgid "No such view: %(view_name)s."
+msgstr ""
+
+#: bzrlib/errors.py:3208
+msgid ""
+"Views are not supported by %(tree)s; use 'bzr upgrade' to change your tree "
+"to a later format."
+msgstr ""
+
+#: bzrlib/errors.py:3217
+msgid ""
+"Specified file \"%(file_name)s\" is outside the current view: %(view_str)s"
+msgstr ""
+
+#: bzrlib/errors.py:3284
+msgid "%(bzrdir)r does not support co-located branches."
+msgstr ""
+
+#: bzrlib/errors.py:3290
+msgid ""
+"Unable to determine your name.\n"
+"Please, set your name with the 'whoami' command.\n"
+"E.g. bzr whoami \"Your Name <name@example.com>\""
+msgstr ""
+
+#: bzrlib/errors.py:3299
+msgid "Invalid pattern(s) found. %(msg)s"
+msgstr ""
+
+#: bzrlib/errors.py:3307
+msgid ""
+"Branch \"%(branch_url)s\" appears to be bound to itself. Please use `bzr "
+"unbind` to fix."
+msgstr ""
+
+#: bzrlib/errors.py:3318
+msgid "Loop involving %(refs)r while expanding \"%(string)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:3327
+msgid "Option %(name)s is not defined while expanding \"%(string)s\"."
+msgstr ""
+
+#: bzrlib/errors.py:3336
+msgid ""
+"No compatible object available for operations from %(source)r to %(target)r."
+msgstr ""
+
+#: bzrlib/errors.py:3346
+msgid ""
+"VFS requests over the smart server are not allowed. Encountered: %(method)s, "
+"%(arguments)s."
+msgstr ""
+
+#: bzrlib/export_pot.py:123 bzrlib/export_pot.py:129
+#, python-format
+msgid "%r is not found in bzrlib/option.py"
+msgstr ""
+
+#: bzrlib/export_pot.py:185
+#, python-format
+msgid "Exporting messages from builtin command: %s"
+msgstr ""
+
+#: bzrlib/export_pot.py:201
+msgid "Exporting messages from plugin command: {0} in {1}"
+msgstr ""
+
+#: bzrlib/export_pot.py:226
+#, python-format
+msgid "Exporting message from error: %s"
+msgstr ""
+
+#: bzrlib/fetch.py:97
+msgid "Finding revisions"
+msgstr ""
+
+#: bzrlib/fetch.py:102
+msgid "Fetching revisions"
+msgstr ""
+
+#: bzrlib/foreign.py:311
+#, python-format
+msgid "Using saved location: %s\n"
+msgstr ""
+
+#: bzrlib/foreign.py:321
+msgid ""
+"{0!r} and {1!r} are in the same VCS, lossy push not necessary. Please use "
+"regular push."
+msgstr ""
+
+#: bzrlib/gpg.py:116 bzrlib/gpg.py:454
+msgid "{0} commits with valid signatures"
+msgstr ""
+
+#: bzrlib/gpg.py:120 bzrlib/gpg.py:459
+msgid "{0} commit with unknown key"
+msgid_plural "{0} commits with unknown keys"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/gpg.py:126 bzrlib/gpg.py:466
+msgid "{0} commit not valid"
+msgid_plural "{0} commits not valid"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/gpg.py:132 bzrlib/gpg.py:473
+msgid "{0} commit not signed"
+msgid_plural "{0} commits not signed"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/gpg.py:138 bzrlib/gpg.py:480
+msgid "{0} commit with key now expired"
+msgid_plural "{0} commits with key now expired"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/gpg.py:336
+msgid "No GnuPG key results for pattern: {0}"
+msgstr ""
+
+#: bzrlib/gpg.py:379
+msgid "{0} signed {1} commit"
+msgid_plural "{0} signed {1} commits"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/gpg.py:396 bzrlib/gpg.py:412
+msgid "{0} commit by author {1}"
+msgid_plural "{0} commits by author {1}"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/gpg.py:426
+msgid "Unknown key {0} signed {1} commit"
+msgid_plural "Unknown key {0} signed {1} commits"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/gpg.py:444
+msgid "{0} commit by author {1} with key {2} now expired"
+msgid_plural "{0} commits by author {1} with key {2} now expired"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/groupcompress.py:1758
+#, python-format
+msgid "Insert claimed random_id=True, but then inserted %r two times"
+msgstr ""
+
+#: bzrlib/knit.py:1764 bzrlib/knit.py:1800
+msgid "Walking content"
+msgstr ""
+
+#: bzrlib/knit.py:3467
+msgid "annotating"
+msgstr ""
+
+#: bzrlib/lock.py:547
+msgid "{0!r} was {1} locked again"
+msgstr ""
+
+#: bzrlib/log.py:569
+msgid "--exclude-common-ancestry requires two different revisions"
+msgstr ""
+
+#: bzrlib/log.py:571
+#, python-format
+msgid "invalid direction %r"
+msgstr ""
+
+#: bzrlib/log.py:618
+msgid "Start revision not found in left-hand history of end revision."
+msgstr ""
+
+#: bzrlib/log.py:664
+msgid "Start revision not found in history of end revision."
+msgstr ""
+
+#: bzrlib/log.py:1082 bzrlib/log.py:1140
+msgid "Logging revision 0 is invalid."
+msgstr ""
+
+#: bzrlib/log.py:1084 bzrlib/log.py:1142
+msgid "Start revision must be older than the end revision."
+msgstr ""
+
+#: bzrlib/log.py:1828
+#, python-format
+msgid "unknown log formatter: %r"
+msgstr ""
+
+#: bzrlib/log.py:2123
+msgid "fixes bug"
+msgid_plural "fixes bugs"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/merge.py:715
+msgid "All changes applied successfully."
+msgstr ""
+
+#: bzrlib/merge.py:717
+#, python-format
+msgid "%d conflicts encountered."
+msgstr ""
+
+#: bzrlib/merge.py:862 bzrlib/merge.py:1910
+msgid "Preparing file merge"
+msgstr ""
+
+#: bzrlib/multiparent.py:424
+msgid "Importing revisions"
+msgstr ""
+
+# help of 'help' option
+#: bzrlib/option.py:530
+msgid "Show help message."
+msgstr ""
+
+# help of 'usage' option
+#: bzrlib/option.py:532
+msgid "Show usage message and options."
+msgstr ""
+
+# help of 'verbose' option
+#: bzrlib/option.py:534
+msgid "Display more information."
+msgstr ""
+
+# help of 'quiet' option
+#: bzrlib/option.py:537
+msgid "Only display errors and warnings."
+msgstr ""
+
+# help of 'overwrite' option
+#: bzrlib/option.py:542
+msgid "Ignore differences between branches and overwrite unconditionally."
+msgstr ""
+
+# help of 'message' option
+#: bzrlib/option.py:553
+msgid "Message string."
+msgstr ""
+
+# help of 'null' option
+#: bzrlib/option.py:556
+msgid "Use an ASCII NUL (\\0) separator rather than a newline."
+msgstr ""
+
+# help of 'profile' option
+#: bzrlib/option.py:559
+msgid "Show performance profiling information."
+msgstr ""
+
+# help of 'revision' option
+#: bzrlib/option.py:563
+msgid "See \"help revisionspec\" for details."
+msgstr ""
+
+# help of 'change' option
+#: bzrlib/option.py:568
+msgid ""
+"Select changes introduced by the specified revision. See also \"help "
+"revisionspec\"."
+msgstr ""
+
+# help of 'show-ids' option
+#: bzrlib/option.py:570
+msgid "Show internal object ids."
+msgstr ""
+
+# help of 'timezone' option
+#: bzrlib/option.py:573
+msgid "Display timezone as local, original, or utc."
+msgstr ""
+
+# help of 'log-format' option
+#: bzrlib/option.py:578
+msgid "Use specified log format."
+msgstr ""
+
+# title of 'log-format' option
+#: bzrlib/option.py:580
+msgid "Log format"
+msgstr ""
+
+# help of 'long' option
+#: bzrlib/option.py:582
+msgid "Use detailed log format. Same as --log-format long"
+msgstr ""
+
+# help of 'short' option
+#: bzrlib/option.py:584
+msgid "Use moderately short log format. Same as --log-format short"
+msgstr ""
+
+# help of 'line' option
+#: bzrlib/option.py:585
+msgid "Use log format with one line per revision. Same as --log-format line"
+msgstr ""
+
+# help of 'merge-type' option
+#: bzrlib/option.py:588
+msgid "Select a particular merge algorithm."
+msgstr ""
+
+# title of 'merge-type' option
+#: bzrlib/option.py:590
+msgid "Merge algorithm"
+msgstr ""
+
+# help of 'remember' option
+#: bzrlib/option.py:592
+msgid "Remember the specified location as a default."
+msgstr ""
+
+# help of 'reprocess' option
+#: bzrlib/option.py:594
+msgid "Reprocess to reduce spurious conflicts."
+msgstr ""
+
+# help of 'dry-run' option
+#: bzrlib/option.py:597
+msgid "Show what would be done, but don't actually do anything."
+msgstr ""
+
+# help of 'name-from-revision' option
+#: bzrlib/option.py:598
+msgid "The path name in the old tree."
+msgstr ""
+
+# help of 'directory' option
+#: bzrlib/option.py:600
+msgid "Branch to operate on, instead of working directory"
+msgstr ""
+
+#: bzrlib/osutils.py:93
+msgid "Parameter {0!r} encoding is unsupported by {1} application locale."
+msgstr ""
+
+#: bzrlib/osutils.py:194
+msgid "lstat/stat of ({0!r}): {1!r}"
+msgstr ""
+
+#: bzrlib/osutils.py:929 bzrlib/osutils.py:940
+#, python-format
+msgid "sorry, %r not allowed in path"
+msgstr ""
+
+#: bzrlib/osutils.py:1158
+#, python-format
+msgid "%r is too short to calculate a relative path"
+msgstr ""
+
+# help of 'plugin' option of 'bash-completion' command
+#: bzrlib/plugins/bash_completion/bashcomp.py:407
+msgid "Enable completions for the selected plugin (default: all plugins)"
+msgstr ""
+
+#: bzrlib/plugins/bash_completion/bashcomp.py:409
+msgid "Generate a shell function for bash command line completion."
+msgstr ""
+
+#: bzrlib/plugins/bash_completion/bashcomp.py:411
+msgid ""
+"This command generates a shell function which can be used by bash to\n"
+"automatically complete the currently typed command when the user presses\n"
+"the completion key (usually tab)."
+msgstr ""
+
+#: bzrlib/plugins/bash_completion/bashcomp.py:415
+msgid ""
+"Commonly used like this:\n"
+"    eval \"`bzr bash-completion`\""
+msgstr ""
+
+# help of 'function-name' option of 'bash-completion' command
+#: bzrlib/plugins/bash_completion/bashcomp.py:421
+msgid "Name of the generated function (default: _bzr)"
+msgstr ""
+
+# help of 'function-only' option of 'bash-completion' command
+#: bzrlib/plugins/bash_completion/bashcomp.py:423
+msgid "Generate only the shell function, don't enable it"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:78
+msgid "Register a branch with launchpad.net."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:80
+msgid ""
+"This command lists a bzr branch in the directory of branches on\n"
+"launchpad.net.  Registration allows the branch to be associated with\n"
+"bugs or specifications."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:84
+msgid ""
+"Before using this command you must register the project to which the\n"
+"branch belongs, and create an account for yourself on launchpad.net."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:87
+msgid ""
+"arguments:\n"
+"    public_url: The publicly visible url for the branch to register.\n"
+"                This must be an http or https url (which Launchpad can read\n"
+"                from to access the branch). Local file urls, SFTP urls, and\n"
+"                bzr+ssh urls will not work.\n"
+"                If no public_url is provided, bzr will use the configured\n"
+"                public_url if there is one for the current branch, and\n"
+"                otherwise error."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:96
+msgid ""
+"example:\n"
+"    bzr register-branch http://foo.com/bzr/fooproject.mine \\\n"
+"            --project fooproject"
+msgstr ""
+
+# help of 'project' option of 'register-branch' command
+#: bzrlib/plugins/launchpad/__init__.py:103
+msgid "Launchpad project short name to associate with the branch."
+msgstr ""
+
+# help of 'branch-name' option of 'register-branch' command
+#: bzrlib/plugins/launchpad/__init__.py:110
+msgid ""
+"Short name for the branch; by default taken from the last component of the "
+"url."
+msgstr ""
+
+# help of 'branch-title' option of 'register-branch' command
+#: bzrlib/plugins/launchpad/__init__.py:114
+msgid "One-sentence description of the branch."
+msgstr ""
+
+# help of 'branch-description' option of 'register-branch' command
+#: bzrlib/plugins/launchpad/__init__.py:117
+msgid "Longer description of the purpose or contents of the branch."
+msgstr ""
+
+# help of 'author' option of 'register-branch' command
+#: bzrlib/plugins/launchpad/__init__.py:120
+msgid "Branch author's email address, if not yourself."
+msgstr ""
+
+# help of 'link-bug' option of 'register-branch' command
+#: bzrlib/plugins/launchpad/__init__.py:123
+msgid "The bug this branch fixes."
+msgstr ""
+
+# help of 'dry-run' option of 'register-branch' command
+#: bzrlib/plugins/launchpad/__init__.py:126
+msgid "Prepare the request but don't actually send it."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:186
+msgid "Open a Launchpad branch page in your web browser."
+msgstr ""
+
+# help of 'dry-run' option of 'launchpad-open' command
+#: bzrlib/plugins/launchpad/__init__.py:191
+msgid "Do not actually open the browser. Just say the URL we would use."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:236
+msgid "Show or set the Launchpad user ID."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:238
+msgid ""
+"When communicating with Launchpad, some commands need to know your\n"
+"Launchpad user ID.  This command can be used to set or show the\n"
+"user ID that Bazaar will use for such communication."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:242
+msgid ""
+":Examples:\n"
+"  Show the Launchpad ID of the current user::"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:245
+msgid "      bzr launchpad-login"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:247
+msgid "  Set the Launchpad ID of the current user to 'bob'::"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:249
+msgid "      bzr launchpad-login bob"
+msgstr ""
+
+# help of 'no-check' option of 'launchpad-login' command
+#: bzrlib/plugins/launchpad/__init__.py:256
+msgid "Don't check that the user name is valid."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:292
+msgid "Ask Launchpad to mirror a branch now."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:312
+msgid "Propose merging a branch on Launchpad."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:314
+msgid ""
+"This will open your usual editor to provide the initial comment.  When it\n"
+"has created the proposal, it will open it in your default web browser."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:317
+msgid ""
+"The branch will be proposed to merge into SUBMIT_BRANCH.  If SUBMIT_BRANCH\n"
+"is not supplied, the remembered submit branch will be used.  If no submit\n"
+"branch is remembered, the development focus will be used."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:321
+msgid ""
+"By default, the SUBMIT_BRANCH's review team will be requested to review\n"
+"the merge proposal.  This can be overriden by specifying --review (-R).\n"
+"The parameter the launchpad account name of the desired reviewer.  This\n"
+"may optionally be followed by '=' and the review type.  For example:"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:326
+msgid "  bzr lp-propose-merge --review jrandom --review review-team=qa"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:328
+msgid ""
+"This will propose a merge,  request \"jrandom\" to perform a review of\n"
+"unspecified type, and request \"review-team\" to perform a \"qa\" review."
+msgstr ""
+
+# help of 'staging' option of 'lp-propose-merge' command
+#: bzrlib/plugins/launchpad/__init__.py:333
+msgid "Propose the merge on staging."
+msgstr ""
+
+# help of 'message' option of 'lp-propose-merge' command
+#: bzrlib/plugins/launchpad/__init__.py:335
+msgid "Commit message."
+msgstr ""
+
+# help of 'approve' option of 'lp-propose-merge' command
+#: bzrlib/plugins/launchpad/__init__.py:337
+msgid "Mark the proposal as approved immediately."
+msgstr ""
+
+# help of 'review' option of 'lp-propose-merge' command
+#: bzrlib/plugins/launchpad/__init__.py:339
+msgid "Requested reviewer and optional type."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:376
+msgid "Find the proposal to merge this revision."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:378
+msgid ""
+"Finds the merge proposal(s) that discussed landing the specified revision.\n"
+"This works only if the selected branch was the merge proposal target, and\n"
+"if the merged_revno is recorded for the merge proposal.  The proposal(s)\n"
+"are opened in a web browser."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:383
+msgid ""
+"Any revision involved in the merge may be specified-- the revision in\n"
+"which the merge was performed, or one of the revisions that was merged."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:386
+msgid "So, to find the merge proposal that reviewed line 1 of README::"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:388
+msgid "  bzr lp-find-proposal -r annotate:README:1"
+msgstr ""
+
+#: bzrlib/push.py:54
+msgid "Created new branch."
+msgstr ""
+
+#: bzrlib/push.py:96
+#, python-format
+msgid ""
+"Target directory %s already contains a .bzr directory, but it is not valid."
+msgstr ""
+
+#: bzrlib/push.py:99
+#, python-format
+msgid ""
+"Target directory %s already exists, but does not have a .bzr directory. "
+"Supply --use-existing-dir to push there anyway."
+msgstr ""
+
+#: bzrlib/push.py:120
+#, python-format
+msgid "Too many redirections trying to make %s."
+msgstr ""
+
+#: bzrlib/push.py:145
+msgid ""
+"These branches have diverged.  See \"bzr help diverged-branches\" for more "
+"information."
+msgstr ""
+
+#: bzrlib/push.py:149
+#, python-format
+msgid ""
+"It is not possible to losslessly push to %s. You may want to use dpush "
+"instead."
+msgstr ""
+
+#: bzrlib/push.py:155
+#, python-format
+msgid ""
+"At %s you have a valid .bzr control directory, but not a branch or "
+"repository. This is an unsupported configuration. Please move the target "
+"directory out of the way and try again."
+msgstr ""
+
+#: bzrlib/reconcile.py:96
+#, python-format
+msgid "Reconciling branch %s"
+msgstr ""
+
+#: bzrlib/reconcile.py:102
+#, python-format
+msgid "Reconciling repository %s"
+msgstr ""
+
+#: bzrlib/reconcile.py:104
+msgid "Reconciling repository"
+msgstr ""
+
+#: bzrlib/reconcile.py:110
+#, python-format
+msgid "%s cannot canonicalize CHKs."
+msgstr ""
+
+#: bzrlib/reconcile.py:118
+msgid "Reconcile aborted: revision index has inconsistent parents."
+msgstr ""
+
+#: bzrlib/reconcile.py:120
+msgid "Run \"bzr check\" for more details."
+msgstr ""
+
+#: bzrlib/reconcile.py:122
+msgid "Reconciliation complete."
+msgstr ""
+
+#: bzrlib/reconcile.py:165
+msgid "Fixing last revision info {0}  => {1}"
+msgstr ""
+
+#: bzrlib/reconcile.py:172
+msgid "revision_history ok."
+msgstr ""
+
+#: bzrlib/reconcile.py:233
+msgid "Reading inventory data"
+msgstr ""
+
+#: bzrlib/reconcile.py:253 bzrlib/reconcile.py:383
+msgid "Inventory ok."
+msgstr ""
+
+#: bzrlib/reconcile.py:255 bzrlib/reconcile.py:385
+msgid "Backing up inventory"
+msgstr ""
+
+#: bzrlib/reconcile.py:257
+msgid "Backup inventory created."
+msgstr ""
+
+#: bzrlib/reconcile.py:273 bzrlib/reconcile.py:404
+msgid "Writing weave"
+msgstr ""
+
+#: bzrlib/reconcile.py:276 bzrlib/reconcile.py:407
+msgid "Inventory regenerated."
+msgstr ""
+
+#: bzrlib/reconcile.py:370 bzrlib/reconcile.py:372 bzrlib/reconcile.py:375
+msgid "Reading indexes"
+msgstr ""
+
+#: bzrlib/reconcile.py:379 bzrlib/reconcile.py:381
+msgid "Checking unused inventories"
+msgstr ""
+
+#: bzrlib/reconcile.py:387
+msgid "Backup Inventory created"
+msgstr ""
+
+#: bzrlib/reconcile.py:445
+msgid "Fixing text parents"
+msgstr ""
+
+#: bzrlib/reconfigure.py:52
+msgid "{0} is now stacked on {1}\n"
+msgstr ""
+
+#: bzrlib/reconfigure.py:67
+#, python-format
+msgid "%s is now not stacked\n"
+msgstr ""
+
+#: bzrlib/remote.py:1886
+msgid "Copying repository content as tarball..."
+msgstr ""
+
+#: bzrlib/rename_map.py:68
+msgid "Calculating hashes"
+msgstr ""
+
+#: bzrlib/rename_map.py:106
+msgid "Determining hash hits"
+msgstr ""
+
+#: bzrlib/rename_map.py:241
+msgid "{0} => {1}"
+msgstr ""
+
+#: bzrlib/repository.py:284
+#, python-format
+msgid "bzr: ERROR (ignored): %s"
+msgstr ""
+
+#: bzrlib/repository.py:1806
+msgid "Moving repository to repository.backup"
+msgstr ""
+
+#: bzrlib/repository.py:1813
+msgid "Creating new repository"
+msgstr ""
+
+#: bzrlib/repository.py:1818
+msgid "Copying content"
+msgstr ""
+
+#: bzrlib/repository.py:1822
+msgid "Deleting old repository content"
+msgstr ""
+
+#: bzrlib/repository.py:1824
+msgid "repository converted"
+msgstr ""
+
+#: bzrlib/revisionspec.py:914
+msgid "Using {0} {1}"
+msgstr ""
+
+#: bzrlib/send.py:56
+#, python-format
+msgid "Mail client \"%s\" does not support specifying body"
+msgstr ""
+
+#: bzrlib/send.py:60
+msgid "--remember requires a branch to be specified."
+msgstr ""
+
+#: bzrlib/send.py:74
+msgid "No submit branch known or specified"
+msgstr ""
+
+#: bzrlib/send.py:77
+msgid "Using saved {0} location \"{1}\" to determine what changes to submit."
+msgstr ""
+
+#: bzrlib/send.py:94
+#, python-format
+msgid "No such send format '%s'."
+msgstr ""
+
+#: bzrlib/send.py:111
+msgid "bzr send takes at most two one revision identifiers"
+msgstr ""
+
+#: bzrlib/send.py:124
+msgid "No revisions to submit."
+msgstr ""
+
+#: bzrlib/send.py:135
+msgid ""
+"- not supported for merge directives that use more than one output file."
+msgstr ""
+
+#: bzrlib/send.py:177
+msgid "Format 0.9 does not permit bundle with no patch"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:45
+#, python-format
+msgid "Shelve adding file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:46
+msgid "Shelve binary changes?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:47
+msgid "Shelve changing \"%s\" from %(other)s to %(this)s?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:49
+#, python-format
+msgid "Shelve removing file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:50
+#, python-format
+msgid "Shelve %d change(s)?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:51
+msgid "Shelve?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:52
+#, python-format
+msgid ""
+"Shelve changing target of \"%(path)s\" from \"%(other)s\" to \"%(this)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:54
+#, python-format
+msgid "Shelve renaming \"%(other)s\" => \"%(this)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:69
+#, python-format
+msgid "Changes shelved with id \"%d\"."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:73
+msgid "Selected changes destroyed."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:77
+msgid "Selected changes:"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:97
+#, python-format
+msgid "Delete file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:98
+msgid "Apply binary changes?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:99
+#, python-format
+msgid "Change \"%(path)s\" from %(this)s to %(other)s?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:101
+#, python-format
+msgid "Add file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:102
+#, python-format
+msgid "Apply %d change(s)?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:103
+msgid "Apply change?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:104
+#, python-format
+msgid "Change target of \"%(path)s\" from \"%(this)s\" to \"%(other)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:106
+#, python-format
+msgid "Rename \"%(this)s\" => \"%(other)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:265
+msgid "You need a controlling terminal."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:415
+msgid "No changes are shelved."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:473
+#, python-format
+msgid "Using changes with id \"%d\"."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:477
+#, python-format
+msgid "Message: %s"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:489
+#, python-format
+msgid "Deleted changes with id \"%d\"."
+msgstr ""
+
+#: bzrlib/smart/medium.py:565
+msgid "HPSS calls: {0} ({1} vfs) {2}"
+msgstr ""
+
+#: bzrlib/smart/server.py:355
+#, python-format
+msgid "listening on port: %s"
+msgstr ""
+
+#: bzrlib/switch.py:67
+msgid "Pending merges must be committed or reverted before using switch."
+msgstr ""
+
+#: bzrlib/switch.py:94
+msgid ""
+"Cannot switch as local commits found in the checkout. Commit these to the "
+"bound branch or use --force to throw them away."
+msgstr ""
+
+#: bzrlib/switch.py:99
+#, python-format
+msgid ""
+"Unable to connect to current master branch %(target)s: %(error)s To switch "
+"anyway, use --force."
+msgstr ""
+
+#: bzrlib/switch.py:108
+msgid "Cannot switch a branch, only a checkout."
+msgstr ""
+
+#: bzrlib/switch.py:144
+#, python-format
+msgid "Tree is up to date at revision %d."
+msgstr ""
+
+#: bzrlib/switch.py:150
+#, python-format
+msgid "Updated to revision %d."
+msgstr ""
+
+#: bzrlib/transform.py:1730 bzrlib/transform.py:1741 bzrlib/transform.py:1743
+msgid "Apply phase"
+msgstr ""
+
+#: bzrlib/transform.py:1769 bzrlib/transform.py:1837
+msgid "removing file"
+msgstr ""
+
+#: bzrlib/transform.py:1787 bzrlib/transform.py:1872
+msgid "adding file"
+msgstr ""
+
+#: bzrlib/transform.py:2587
+msgid "Building tree"
+msgstr ""
+
+#: bzrlib/transform.py:2677 bzrlib/transform.py:2704
+msgid "Adding file contents"
+msgstr ""
+
+#: bzrlib/transform.py:3051
+msgid "Resolution pass"
+msgstr ""
+
+#: bzrlib/tree.py:635
+msgid "*** {0} content-filter: {1} => {2!r}"
+msgstr ""
+
+#: bzrlib/upgrade.py:76
+#, python-format
+msgid "This is a checkout. The branch (%s) needs to be upgraded separately."
+msgstr ""
+
+#: bzrlib/upgrade.py:99
+#, python-format
+msgid "cannot upgrade from bzrdir format %s"
+msgstr ""
+
+#: bzrlib/upgrade.py:102
+#, python-format
+msgid "starting upgrade of %s"
+msgstr ""
+
+#: bzrlib/upgrade.py:109
+msgid "finished"
+msgstr ""
+
+#: bzrlib/upgrade.py:119
+msgid "Deleting backup.bzr"
+msgstr ""
+
+#: bzrlib/upgrade.py:150
+msgid ""
+"\n"
+"SUMMARY: {0} upgrades attempted, {1} succeeded, {2} failed"
+msgstr ""
+
+#: bzrlib/upgrade.py:207
+#, python-format
+msgid "Found %d dependent branches - upgrading ..."
+msgstr ""
+
+#: bzrlib/upgrade.py:275
+msgid "Upgrading bzrdirs"
+msgstr ""
+
+#: bzrlib/upgrade.py:281
+#, python-format
+msgid "Upgrading %s"
+msgstr ""
+
+#: bzrlib/upgrade.py:282
+msgid "Upgrading {0} {1} ..."
+msgstr ""
+
+#: bzrlib/upgrade.py:296
+msgid "Removing backup ..."
+msgstr ""
+
+#: bzrlib/upgrade.py:300
+msgid "failed to clean-up {0}: {1}"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1091
+msgid "inventories"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1117
+msgid "texts"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1572 bzrlib/vf_repository.py:1581
+msgid "Calculating text parents"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1657
+msgid "Fetch texts"
+msgstr ""
+
+#: bzrlib/vf_repository.py:2468
+msgid "loading text store"
+msgstr ""
+
+#: bzrlib/vf_repository.py:2474
+msgid "checking text graph"
+msgstr ""
+
+#: bzrlib/vf_repository.py:2924 bzrlib/vf_repository.py:2938
+#: bzrlib/vf_repository.py:3053
+msgid "Transferring revisions"
+msgstr ""
+
+#: bzrlib/win32utils.py:132
+msgid "Failed to GetProcessMemoryInfo()"
+msgstr ""
+
+#: bzrlib/win32utils.py:153
+msgid "Cannot debug memory on win32 without ctypes or win32process"
+msgstr ""
+
+#: bzrlib/win32utils.py:158
+msgid "WorkingSize {0:>7}KiB\tPeakWorking {1:>7}KiB\t{2}"
+msgstr ""
+
+#: bzrlib/win32utils.py:166
+#, python-format
+msgid "WorkingSize       %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:167
+#, python-format
+msgid "PeakWorking       %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:168
+#, python-format
+msgid "PagefileUsage     %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:169
+#, python-format
+msgid "PeakPagefileUsage %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:171
+#, python-format
+msgid "PrivateUsage      %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:172
+#, python-format
+msgid "PageFaultCount    %8d"
+msgstr ""
+
+#: bzrlib/workingtree.py:1502
+msgid "Rerun update after fixing the conflicts."
+msgstr ""
+
+#: dummy/help_topics/authentication/summary.txt:1
+msgid "Information on configuring authentication"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:1
+msgid ""
+"Bazaar 2.5.0dev2 -- a free distributed version-control tool\n"
+"http://bazaar.canonical.com/"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:4
+msgid ""
+"Basic commands:\n"
+"  bzr init           makes this directory a versioned branch\n"
+"  bzr branch         make a copy of another branch"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:8
+msgid ""
+"  bzr add            make files or directories versioned\n"
+"  bzr ignore         ignore a file or pattern\n"
+"  bzr mv             move or rename a versioned file"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:12
+msgid ""
+"  bzr status         summarize changes in working copy\n"
+"  bzr diff           show detailed diffs"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:15
+msgid ""
+"  bzr merge          pull in changes from another branch\n"
+"  bzr commit         save some or all changes\n"
+"  bzr send           send changes via email"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:19
+msgid ""
+"  bzr log            show history of changes\n"
+"  bzr check          validate storage"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:22
+msgid ""
+"  bzr help init      more help on e.g. init command\n"
+"  bzr help commands  list all commands\n"
+"  bzr help topics    list all help topics\n"
+msgstr ""
+
+#: dummy/help_topics/basic/summary.txt:1
+msgid "Basic commands"
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:1
+msgid "Branches"
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:3
+msgid ""
+"A branch consists of the state of a project, including all of its\n"
+"history. All branches have a repository associated (which is where the\n"
+"branch history is stored), but multiple branches may share the same\n"
+"repository (a shared repository). Branches can be copied and merged."
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:8
+msgid ""
+"In addition, one branch may be bound to another one.  Binding to another\n"
+"branch indicates that commits which happen in this branch must also \n"
+"happen in the other branch.  Bazaar ensures consistency by not allowing \n"
+"commits when the two branches are out of date.  In order for a commit \n"
+"to succeed, it may be necessary to update the current branch using \n"
+"``bzr update``."
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:15
+msgid "Related commands::"
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:17
+msgid ""
+"  init    Change a directory into a versioned branch.\n"
+"  branch  Create a new branch that is a copy of an existing branch.\n"
+"  merge   Perform a three-way merge.\n"
+"  bind    Bind a branch to another one.\n"
+msgstr ""
+
+#: dummy/help_topics/branches/summary.txt:1
+msgid "Information on what a branch is"
+msgstr ""
+
+#: dummy/help_topics/bugs/summary.txt:1
+msgid "Bug tracker settings"
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:1
+msgid "Checkouts"
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:3
+msgid ""
+"Checkouts are source trees that are connected to a branch, so that when\n"
+"you commit in the source tree, the commit goes into that branch.  They\n"
+"allow you to use a simpler, more centralized workflow, ignoring some of\n"
+"Bazaar's decentralized features until you want them. Using checkouts\n"
+"with shared repositories is very similar to working with SVN or CVS, but\n"
+"doesn't have the same restrictions.  And using checkouts still allows\n"
+"others working on the project to use whatever workflow they like."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:11
+msgid ""
+"A checkout is created with the bzr checkout command (see \"help checkout"
+"\").\n"
+"You pass it a reference to another branch, and it will create a local copy\n"
+"for you that still contains a reference to the branch you created the\n"
+"checkout from (the master branch). Then if you make any commits they will "
+"be\n"
+"made on the other branch first. This creates an instant mirror of your work, "
+"or\n"
+"facilitates lockstep development, where each developer is working together,\n"
+"continuously integrating the changes of others."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:19
+msgid ""
+"However the checkout is still a first class branch in Bazaar terms, so that\n"
+"you have the full history locally.  As you have a first class branch you "
+"can\n"
+"also commit locally if you want, for instance due to the temporary loss af "
+"a\n"
+"network connection. Use the --local option to commit to do this. All the "
+"local\n"
+"commits will then be made on the master branch the next time you do a non-"
+"local\n"
+"commit."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:26
+msgid ""
+"If you are using a checkout from a shared branch you will periodically want "
+"to\n"
+"pull in all the changes made by others. This is done using the \"update\"\n"
+"command. The changes need to be applied before any non-local commit, but\n"
+"Bazaar will tell you if there are any changes and suggest that you use this\n"
+"command when needed."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:32
+msgid ""
+"It is also possible to create a \"lightweight\" checkout by passing the\n"
+"--lightweight flag to checkout. A lightweight checkout is even closer to an\n"
+"SVN checkout in that it is not a first class branch, it mainly consists of "
+"the\n"
+"working tree. This means that any history operations must query the master\n"
+"branch, which could be slow if a network connection is involved. Also, as "
+"you\n"
+"don't have a local branch, then you cannot commit locally."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:39
+msgid ""
+"Lightweight checkouts work best when you have fast reliable access to the\n"
+"master branch. This means that if the master branch is on the same disk or "
+"LAN\n"
+"a lightweight checkout will be faster than a heavyweight one for any "
+"commands\n"
+"that modify the revision history (as only one copy of the branch needs to\n"
+"be updated). Heavyweight checkouts will generally be faster for any command\n"
+"that uses the history but does not change it, but if the master branch is "
+"on\n"
+"the same disk then there won't be a noticeable difference."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:47
+msgid ""
+"Another possible use for a checkout is to use it with a treeless repository\n"
+"containing your branches, where you maintain only one working tree by\n"
+"switching the master branch that the checkout points to when you want to\n"
+"work on a different branch."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:52
+msgid ""
+"Obviously to commit on a checkout you need to be able to write to the "
+"master\n"
+"branch. This means that the master branch must be accessible over a "
+"writeable\n"
+"protocol , such as sftp://, and that you have write permissions at the "
+"other\n"
+"end. Checkouts also work on the local file system, so that all that matters "
+"is\n"
+"file permissions."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:58
+msgid ""
+"You can change the master of a checkout by using the \"switch\" command "
+"(see\n"
+"\"help switch\").  This will change the location that the commits are sent "
+"to.\n"
+"The \"bind\" command can also be used to turn a normal branch into a heavy\n"
+"checkout. If you would like to convert your heavy checkout into a normal\n"
+"branch so that every commit is local, you can use the \"unbind\" command. To "
+"see\n"
+"whether or not a branch is bound or not you can use the \"info\" command. If "
+"the\n"
+"branch is bound it will tell you the location of the bound branch."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:68
+msgid ""
+"  checkout    Create a checkout. Pass --lightweight to get a lightweight\n"
+"              checkout\n"
+"  update      Pull any changes in the master branch in to your checkout\n"
+"  commit      Make a commit that is sent to the master branch. If you have\n"
+"              a heavy checkout then the --local option will commit to the\n"
+"              checkout without sending the commit to the master\n"
+"  switch      Change the master branch that the commits in the checkout "
+"will\n"
+"              be sent to\n"
+"  bind        Turn a standalone branch into a heavy checkout so that any\n"
+"              commits will be sent to the master branch\n"
+"  unbind      Turn a heavy checkout into a standalone branch so that any\n"
+"              commits are only made locally\n"
+"  info        Displays whether a branch is bound or unbound. If the branch "
+"is\n"
+"              bound, then it will also display the location of the bound "
+"branch\n"
+msgstr ""
+
+#: dummy/help_topics/checkouts/summary.txt:1
+msgid "Information on what a checkout is"
+msgstr ""
+
+#: dummy/help_topics/commands/summary.txt:1
+msgid "Basic help for all commands"
+msgstr ""
+
+#: dummy/help_topics/configuration/summary.txt:1
+msgid "Details on the configuration settings available"
+msgstr ""
+
+#: dummy/help_topics/conflict-types/summary.txt:1
+msgid "Types of conflicts and what to do about them"
+msgstr ""
+
+#: dummy/help_topics/content-filters/summary.txt:1
+msgid "Conversion of content into/from working trees"
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:1
+msgid "Criss-Cross"
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:3
+msgid ""
+"A criss-cross in the branch history can cause the default merge technique\n"
+"to emit more conflicts than would normally be expected."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:6
+msgid ""
+"In complex merge cases, ``bzr merge --lca`` or ``bzr merge --weave`` may "
+"give\n"
+"better results.  You may wish to ``bzr revert`` the working tree and merge\n"
+"again.  Alternatively, use ``bzr remerge`` on particular conflicted files."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:10
+msgid ""
+"Criss-crosses occur in a branch's history if two branches merge the same "
+"thing\n"
+"and then merge one another, or if two branches merge one another at the "
+"same\n"
+"time.  They can be avoided by having each branch only merge from or into a\n"
+"designated central branch (a \"star topology\")."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:15
+msgid ""
+"Criss-crosses cause problems because of the way merge works.  Bazaar's "
+"default\n"
+"merge is a three-way merger; in order to merge OTHER into THIS, it must\n"
+"find a basis for comparison, BASE.  Using BASE, it can determine whether\n"
+"differences between THIS and OTHER are due to one side adding lines, or\n"
+"from another side removing lines."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:21
+msgid ""
+"Criss-crosses mean there is no good choice for a base.  Selecting the "
+"recent\n"
+"merge points could cause one side's changes to be silently discarded.\n"
+"Selecting older merge points (which Bazaar does) mean that extra conflicts\n"
+"are emitted."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:26
+msgid ""
+"The ``weave`` merge type is not affected by this problem because it uses\n"
+"line-origin detection instead of a basis revision to determine the cause of\n"
+"differences.\n"
+msgstr ""
+
+#: dummy/help_topics/criss-cross/summary.txt:1
+msgid "Information on criss-cross merging"
+msgstr ""
+
+#: dummy/help_topics/current-formats/summary.txt:1
+msgid "Current storage formats"
+msgstr ""
+
+#: dummy/help_topics/debug-flags/summary.txt:1
+msgid "Options to show or record debug information"
+msgstr ""
+
+#: dummy/help_topics/diverged-branches/summary.txt:1
+msgid "How to fix diverged branches"
+msgstr ""
+
+#: dummy/help_topics/env-variables/detail.txt:1
+msgid "Environment Variables"
+msgstr ""
+
+#: dummy/help_topics/env-variables/detail.txt:3
+msgid ""
+"=================== "
+"===========================================================\n"
+"BZRPATH             Path where bzr is to look for shell plugin external\n"
+"                    commands.\n"
+"BZR_EMAIL           E-Mail address of the user. Overrides EMAIL.\n"
+"EMAIL               E-Mail address of the user.\n"
+"BZR_EDITOR          Editor for editing commit messages. Overrides EDITOR.\n"
+"EDITOR              Editor for editing commit messages.\n"
+"BZR_PLUGIN_PATH     Paths where bzr should look for plugins.\n"
+"BZR_DISABLE_PLUGINS Plugins that bzr should not load.\n"
+"BZR_PLUGINS_AT      Plugins to load from a directory not in "
+"BZR_PLUGIN_PATH.\n"
+"BZR_HOME            Directory holding .bazaar config dir. Overrides HOME.\n"
+"BZR_HOME (Win32)    Directory holding bazaar config dir. Overrides APPDATA "
+"and\n"
+"                    HOME.\n"
+"BZR_REMOTE_PATH     Full name of remote 'bzr' command (for bzr+ssh:// "
+"URLs).\n"
+"BZR_SSH             Path to SSH client, or one of paramiko, openssh, "
+"sshcorp,\n"
+"                    plink or lsh.\n"
+"BZR_LOG             Location of .bzr.log (use '/dev/null' to suppress log).\n"
+"BZR_LOG (Win32)     Location of .bzr.log (use 'NUL' to suppress log).\n"
+"BZR_COLUMNS         Override implicit terminal width.\n"
+"BZR_CONCURRENCY     Number of processes that can be run concurrently "
+"(selftest)\n"
+"BZR_PROGRESS_BAR    Override the progress display. Values are 'none' or "
+"'text'.\n"
+"BZR_PDB             Control whether to launch a debugger on error.\n"
+"BZR_SIGQUIT_PDB     Control whether SIGQUIT behaves normally or invokes a\n"
+"                    breakin debugger.\n"
+"=================== "
+"===========================================================\n"
+msgstr ""
+
+#: dummy/help_topics/env-variables/summary.txt:1
+msgid "Environment variable names and values"
+msgstr ""
+
+#: dummy/help_topics/eol/summary.txt:1
+msgid "Information on end-of-line handling"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:1
+msgid "Files"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:3
+msgid ""
+":On Unix:   ~/.bazaar/bazaar.conf\n"
+":On Windows: C:\\\\Documents and Settings\\\\username\\\\Application Data\\"
+"\\bazaar\\\\2.0\\\\bazaar.conf"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:6
+msgid ""
+"Contains the user's default configuration. The section ``[DEFAULT]`` is\n"
+"used to define general configuration that will be applied everywhere.\n"
+"The section ``[ALIASES]`` can be used to create command aliases for\n"
+"commonly used options."
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:11
+msgid "A typical config file might look something like::"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:13
+msgid ""
+"  [DEFAULT]\n"
+"  email=John Doe <jdoe@isp.com>"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:16
+msgid ""
+"  [ALIASES]\n"
+"  commit = commit --strict\n"
+"  log10 = log --short -r -10..-1\n"
+msgstr ""
+
+#: dummy/help_topics/files/summary.txt:1
+msgid "Information on configuration and log files"
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:1
+msgid "Storage Formats"
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:3
+msgid ""
+"To ensure that older clients do not access data incorrectly,\n"
+"Bazaar's policy is to introduce a new storage format whenever\n"
+"new features requiring new metadata are added. New storage\n"
+"formats may also be introduced to improve performance and\n"
+"scalability."
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:9
+msgid ""
+"The newest format, 2a, is highly recommended. If your\n"
+"project is not using 2a, then you should suggest to the\n"
+"project owner to upgrade."
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:13
+msgid ""
+"\n"
+".. note::"
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:16
+msgid ""
+"   Some of the older formats have two variants:\n"
+"   a plain one and a rich-root one. The latter include an additional\n"
+"   field about the root of the tree. There is no performance cost\n"
+"   for using a rich-root format but you cannot easily merge changes\n"
+"   from a rich-root format into a plain format. As a consequence,\n"
+"   moving a project to a rich-root format takes some co-ordination\n"
+"   in that all contributors need to upgrade their repositories\n"
+"   around the same time. 2a and all future formats will be\n"
+"   implicitly rich-root."
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:26
+msgid ""
+"See :doc:`current-formats-help` for the complete list of\n"
+"currently supported formats. See :doc:`other-formats-help` for\n"
+"descriptions of any available experimental and deprecated formats.\n"
+msgstr ""
+
+#: dummy/help_topics/formats/summary.txt:1
+msgid "Information on choosing a storage format"
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:1
+msgid "Global Options"
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:3
+msgid ""
+"These options may be used with any command, and may appear in front of any\n"
+"command.  (e.g. ``bzr --profile help``)."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:6
+msgid ""
+"--version      Print the version number. Must be supplied before the "
+"command.\n"
+"--no-aliases   Do not process command aliases when running this command.\n"
+"--builtin      Use the built-in version of a command, not the plugin "
+"version.\n"
+"               This does not suppress other plugin effects.\n"
+"--no-plugins   Do not process any plugins.\n"
+"--no-l10n      Do not translate messages.\n"
+"--concurrency  Number of processes that can be run concurrently (selftest)."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:14
+msgid ""
+"--profile      Profile execution using the hotshot profiler.\n"
+"--lsprof       Profile execution using the lsprof profiler.\n"
+"--lsprof-file  Profile execution using the lsprof profiler, and write the\n"
+"               results to a specified file.  If the filename ends with \".txt"
+"\",\n"
+"               text format will be used.  If the filename either starts "
+"with\n"
+"               \"callgrind.out\" or end with \".callgrind\", the output will "
+"be\n"
+"               formatted for use with KCacheGrind. Otherwise, the output\n"
+"               will be a pickle.\n"
+"--coverage     Generate line coverage report in the specified directory."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:24
+msgid ""
+"See http://doc.bazaar.canonical.com/developers/profiling.html for more\n"
+"information on profiling."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:27
+msgid ""
+"A number of debug flags are also available to assist troubleshooting and\n"
+"development.  See :doc:`debug-flags-help`.\n"
+msgstr ""
+
+#: dummy/help_topics/global-options/summary.txt:1
+msgid "Options that control how Bazaar runs"
+msgstr ""
+
+#: dummy/help_topics/hidden-commands/summary.txt:1
+msgid "All hidden commands"
+msgstr ""
+
+#: dummy/help_topics/hooks/summary.txt:1
+msgid "Points at which custom processing can be added"
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:1
+msgid "Integration with Launchpad.net"
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:3
+msgid ""
+"Launchpad.net provides free Bazaar branch hosting with integrated bug and\n"
+"specification tracking."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:6
+msgid ""
+"The bzr client (through the plugin called 'launchpad') has special\n"
+"features to communicate with Launchpad:"
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:9
+msgid ""
+"    * The launchpad-login command tells Bazaar your Launchpad user name. "
+"This\n"
+"      is then used by the 'lp:' transport to download your branches using\n"
+"      bzr+ssh://."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:13
+msgid ""
+"    * The 'lp:' transport uses Launchpad as a directory service: for "
+"example\n"
+"      'lp:bzr' and 'lp:python' refer to the main branches of the relevant\n"
+"      projects and may be branched, logged, etc. You can also use the 'lp:'\n"
+"      transport to refer to specific branches, e.g. lp:~bzr/bzr/trunk."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:18
+msgid ""
+"    * The 'lp:' bug tracker alias can expand launchpad bug numbers to their\n"
+"      URLs for use with 'bzr commit --fixes', e.g. 'bzr commit --fixes "
+"lp:12345'\n"
+"      will record a revision property that marks that revision as fixing\n"
+"      Launchpad bug 12345. When you push that branch to Launchpad it will\n"
+"      automatically be linked to the bug report."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:24
+msgid ""
+"    * The register-branch command tells Launchpad about the url of a\n"
+"      public branch.  Launchpad will then mirror the branch, display\n"
+"      its contents and allow it to be attached to bugs and other\n"
+"      objects."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:29
+msgid "For more information see http://help.launchpad.net/\n"
+msgstr ""
+
+#: dummy/help_topics/launchpad/summary.txt:1
+msgid "Using Bazaar with Launchpad.net"
+msgstr ""
+
+#: dummy/help_topics/location-alias/summary.txt:1
+msgid "Aliases for remembered locations"
+msgstr ""
+
+#: dummy/help_topics/log-formats/summary.txt:1
+msgid "Details on the logging formats available"
+msgstr ""
+
+#: dummy/help_topics/other-formats/summary.txt:1
+msgid "Experimental and deprecated storage formats"
+msgstr ""
+
+#: dummy/help_topics/patterns/summary.txt:1
+msgid "Information on the pattern syntax"
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:1
+msgid "Repositories"
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:3
+msgid ""
+"Repositories in Bazaar are where committed information is stored. There is\n"
+"a repository associated with every branch."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:6
+msgid ""
+"Repositories are a form of database. Bzr will usually maintain this for\n"
+"good performance automatically, but in some situations (e.g. when doing\n"
+"very many commits in a short time period) you may want to ask bzr to\n"
+"optimise the database indices. This can be done by the 'bzr pack' command."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:11
+msgid ""
+"By default just running 'bzr init' will create a repository within the new\n"
+"branch but it is possible to create a shared repository which allows "
+"multiple\n"
+"branches to share their information in the same location. When a new branch "
+"is\n"
+"created it will first look to see if there is a containing shared repository "
+"it\n"
+"can use."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:17
+msgid ""
+"When two branches of the same project share a repository, there is\n"
+"generally a large space saving. For some operations (e.g. branching\n"
+"within the repository) this translates in to a large time saving."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:21
+msgid ""
+"To create a shared repository use the init-repository command (or the alias\n"
+"init-repo). This command takes the location of the repository to create. "
+"This\n"
+"means that 'bzr init-repository repo' will create a directory named 'repo',\n"
+"which contains a shared repository. Any new branches that are created in "
+"this\n"
+"directory will then use it for storage."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:27
+msgid ""
+"It is a good idea to create a repository whenever you might create more\n"
+"than one branch of a project. This is true for both working areas where you\n"
+"are doing the development, and any server areas that you use for hosting\n"
+"projects. In the latter case, it is common to want branches without working\n"
+"trees. Since the files in the branch will not be edited directly there is "
+"no\n"
+"need to use up disk space for a working tree. To create a repository in "
+"which\n"
+"the branches will not have working trees pass the '--no-trees' option to\n"
+"'init-repository'."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:38
+msgid ""
+"  init-repository   Create a shared repository. Use --no-trees to create "
+"one\n"
+"                    in which new branches won't get a working tree.\n"
+msgstr ""
+
+#: dummy/help_topics/repositories/summary.txt:1
+msgid "Basic information on shared repositories."
+msgstr ""
+
+#: dummy/help_topics/revisionspec/summary.txt:1
+msgid "Explain how to use --revision"
+msgstr ""
+
+#: dummy/help_topics/rules/summary.txt:1
+msgid "Information on defining rule-based preferences"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:1
+msgid "Standalone Trees"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:3
+msgid ""
+"A standalone tree is a working tree with an associated repository. It\n"
+"is an independently usable branch, with no dependencies on any other.\n"
+"Creating a standalone tree (via bzr init) is the quickest way to put\n"
+"an existing project under version control."
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:8
+msgid "Related Commands::"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:10
+msgid "  init    Make a directory into a versioned branch.\n"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/summary.txt:1
+msgid "Information on what a standalone tree is"
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:1
+msgid "Standard Options"
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:3
+msgid "Standard options are legal for all commands."
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:5
+msgid ""
+"--help, -h     Show help message.\n"
+"--verbose, -v  Display more information.\n"
+"--quiet, -q    Only display errors and warnings."
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:9
+msgid "Unlike global options, standard options can be used in aliases.\n"
+msgstr ""
+
+#: dummy/help_topics/standard-options/summary.txt:1
+msgid "Options that can be used with any command"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:1
+msgid "Status Flags"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:3
+msgid ""
+"Status flags are used to summarise changes to the working tree in a concise\n"
+"manner.  They are in the form::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:6
+msgid "   xxx   <filename>"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:8
+msgid "where the columns' meanings are as follows."
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:10
+msgid "Column 1 - versioning/renames::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:12
+msgid ""
+"  + File versioned\n"
+"  - File unversioned\n"
+"  R File renamed\n"
+"  ? File unknown\n"
+"  X File nonexistent (and unknown to bzr)\n"
+"  C File has conflicts\n"
+"  P Entry for a pending merge (not a file)"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:20
+msgid "Column 2 - contents::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:22
+msgid ""
+"  N File created\n"
+"  D File deleted\n"
+"  K File kind changed\n"
+"  M File modified"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:27
+msgid "Column 3 - execute::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:29
+msgid "  * The execute bit was changed\n"
+msgstr ""
+
+#: dummy/help_topics/status-flags/summary.txt:1
+msgid "Help on status flags"
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:1
+msgid "Branches Out of Sync"
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:3
+msgid ""
+"When reconfiguring a checkout, tree or branch into a lightweight checkout,\n"
+"a local branch must be destroyed.  (For checkouts, this is the local branch\n"
+"that serves primarily as a cache.)  If the branch-to-be-destroyed does not\n"
+"have the same last revision as the new reference branch for the lightweight\n"
+"checkout, data could be lost, so Bazaar refuses."
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:9
+msgid "How you deal with this depends on *why* the branches are out of sync."
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:11
+msgid ""
+"If you have a checkout and have done local commits, you can get back in "
+"sync\n"
+"by running \"bzr update\" (and possibly \"bzr commit\")."
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:14
+msgid ""
+"If you have a branch and the remote branch is out-of-date, you can push\n"
+"the local changes using \"bzr push\".  If the local branch is out of date, "
+"you\n"
+"can do \"bzr pull\".  If both branches have had changes, you can merge, "
+"commit\n"
+"and then push your changes.  If you decide that some of the changes aren't\n"
+"useful, you can \"push --overwrite\" or \"pull --overwrite\" instead.\n"
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/summary.txt:1
+msgid "Steps to resolve \"out-of-sync\" when reconfiguring"
+msgstr ""
+
+#: dummy/help_topics/topics/summary.txt:1
+msgid "Topics list"
+msgstr ""
+
+#: dummy/help_topics/url-special-chars/summary.txt:1
+msgid "Special character handling in URLs"
+msgstr ""
+
+#: dummy/help_topics/urlspec/summary.txt:1
+msgid "Supported transport protocols"
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:1
+msgid "Working Trees"
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:3
+msgid ""
+"A working tree is the contents of a branch placed on disk so that you can\n"
+"see the files and edit them. The working tree is where you make changes to "
+"a\n"
+"branch, and when you commit the current state of the working tree is the\n"
+"snapshot that is recorded in the commit."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:8
+msgid ""
+"When you push a branch to a remote system, a working tree will not be\n"
+"created. If one is already present the files will not be updated. The\n"
+"branch information will be updated and the working tree will be marked\n"
+"as out-of-date. Updating a working tree remotely is difficult, as there\n"
+"may be uncommitted changes or the update may cause content conflicts that "
+"are\n"
+"difficult to deal with remotely."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:15
+msgid ""
+"If you have a branch with no working tree you can use the 'checkout' "
+"command\n"
+"to create a working tree. If you run 'bzr checkout .' from the branch it "
+"will\n"
+"create the working tree. If the branch is updated remotely, you can update "
+"the\n"
+"working tree by running 'bzr update' in that directory."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:20
+msgid ""
+"If you have a branch with a working tree that you do not want the 'remove-"
+"tree'\n"
+"command will remove the tree if it is safe. This can be done to avoid the\n"
+"warning about the remote working tree not being updated when pushing to the\n"
+"branch. It can also be useful when working with a '--no-trees' repository\n"
+"(see 'bzr help repositories')."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:26
+msgid ""
+"If you want to have a working tree on a remote machine that you push to you\n"
+"can either run 'bzr update' in the remote branch after each push, or use "
+"some\n"
+"other method to update the tree during the push. There is an 'rspush' "
+"plugin\n"
+"that will update the working tree using rsync as well as doing a push. "
+"There\n"
+"is also a 'push-and-update' plugin that automates running 'bzr update' via "
+"SSH\n"
+"after each push."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:33
+msgid "Useful commands::"
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:35
+msgid ""
+"  checkout     Create a working tree when a branch does not have one.\n"
+"  remove-tree  Removes the working tree from a branch when it is safe to do "
+"so.\n"
+"  update       When a working tree is out of sync with its associated "
+"branch\n"
+"               this will update the tree to match the branch.\n"
+msgstr ""
+
+#: dummy/help_topics/working-trees/summary.txt:1
+msgid "Information on working trees"
+msgstr ""
+
+#: en/help_topics/authentication.txt:1
+msgid ""
+"Authentication Settings\n"
+"======================="
+msgstr ""
+
+#: en/help_topics/authentication.txt:4
+msgid ""
+"\n"
+"Intent\n"
+"------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:8
+msgid ""
+"Many different authentication policies can be described in the\n"
+"``authentication.conf`` file but a particular user should need only a few\n"
+"definitions to cover his needs without having to specify a user and a "
+"password\n"
+"for every branch he uses."
+msgstr ""
+
+#: en/help_topics/authentication.txt:13
+msgid ""
+"The definitions found in this file are used to find the credentials to use "
+"for\n"
+"a given url. The same credentials can generally be used for as many branches "
+"as\n"
+"possible by grouping their declaration around the remote servers that need\n"
+"them. It's even possible to declare credentials that will be used by "
+"different\n"
+"servers."
+msgstr ""
+
+#: en/help_topics/authentication.txt:19
+msgid ""
+"The intent is to make this file as small as possible to minimize maintenance."
+msgstr ""
+
+#: en/help_topics/authentication.txt:21
+msgid ""
+"Once the relevant credentials are declared in this file you may use branch "
+"urls\n"
+"without embedding passwords (security hazard) or even users (enabling "
+"sharing\n"
+"of your urls with others)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:25
+msgid "Instead of using::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:27
+msgid "  bzr branch ftp://joe:secret@host.com/path/to/my/branch"
+msgstr ""
+
+#: en/help_topics/authentication.txt:29
+msgid "you simply use::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:31
+msgid "  bzr branch ftp://host.com/path/to/my/branch"
+msgstr ""
+
+#: en/help_topics/authentication.txt:33
+msgid "provided you have created the following ``authentication.conf`` file::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:35
+msgid ""
+"  [myprojects]\n"
+"  scheme=ftp\n"
+"  host=host.com\n"
+"  user=joe\n"
+"  password=secret\n"
+"  "
+msgstr ""
+
+#: en/help_topics/authentication.txt:42
+msgid ""
+"Authentication definitions\n"
+"--------------------------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:45
+msgid ""
+"There are two kinds of authentication used by the various schemes supported "
+"by\n"
+"bzr:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:48
+msgid "1. user and password"
+msgstr ""
+
+#: en/help_topics/authentication.txt:50
+msgid ""
+"``FTP`` needs a (``user``, ``password``) to authenticate against a ``host``\n"
+"``SFTP`` can use either a password or a host key to authenticate. However,\n"
+"ssh agents are a better, more secure solution. So we have chosen to not "
+"provide\n"
+"our own less secure method."
+msgstr ""
+
+#: en/help_topics/authentication.txt:55
+msgid "2. user, realm and password"
+msgstr ""
+
+#: en/help_topics/authentication.txt:57
+msgid ""
+"``HTTP`` and ``HTTPS`` needs a (``user, realm, password``) to authenticate\n"
+"against a host. But, by using ``.htaccess`` files, for example, it is "
+"possible\n"
+"to define several (``user, realm, password``) for a given ``host``. So what "
+"is\n"
+"really needed is (``user``, ``password``, ``host``, ``path``). The ``realm`` "
+"is\n"
+"not taken into account in the definitions, but will displayed if bzr "
+"prompts\n"
+"you for a password."
+msgstr ""
+
+#: en/help_topics/authentication.txt:64
+msgid ""
+"``HTTP proxy`` can be handled as ``HTTP`` (or ``HTTPS``) by explicitly\n"
+"specifying the appropriate port."
+msgstr ""
+
+#: en/help_topics/authentication.txt:67
+msgid ""
+"To take all schemes into account, the password will be deduced from a set "
+"of\n"
+"authentication definitions (``scheme``, ``host``, ``port``, ``path``, "
+"``user``,\n"
+"``password``)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:71
+msgid ""
+"  * ``scheme``: can be empty (meaning the rest of the definition can be "
+"used\n"
+"    for any scheme), ``SFTP`` and ``bzr+ssh`` should not be used here, "
+"``ssh``\n"
+"    should be used instead since this is the real scheme regarding\n"
+"    authentication,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:76
+msgid "  * ``host``: can be empty (to act as a default for any host),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:78
+msgid ""
+"  * ``port`` can be empty (useful when an host provides several servers for "
+"the\n"
+"    same scheme), only numerical values are allowed, this should be used "
+"only\n"
+"    when the server uses a port different than the scheme standard port,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:82
+msgid "  * ``path``: can be empty (FTP or SFTP will never user it),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:84
+msgid ""
+"  * ``user``: can be empty (``bzr`` will defaults to python's\n"
+"    ``getpass.get_user()``),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:87
+msgid ""
+"  * ``password``: can be empty if you prefer to always be prompted for your\n"
+"    password."
+msgstr ""
+
+#: en/help_topics/authentication.txt:90
+msgid ""
+"Multiple definitions can be provided and, for a given URL, bzr will select "
+"a\n"
+"(``user`` [, ``password``]) based on the following rules :"
+msgstr ""
+
+#: en/help_topics/authentication.txt:93
+msgid " 1. the first match wins,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:95
+msgid " 2. empty fields match everything,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:97
+msgid ""
+" 3. ``scheme`` matches even if decorators are used in the requested URL,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:99
+msgid ""
+" 4. ``host`` matches exactly or act as a domain if it starts with '.'\n"
+"    (``project.bzr.sf.net`` will match ``.bzr.sf.net`` but ``projectbzr.sf."
+"net``\n"
+"    will not match ``bzr.sf.net``)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:103
+msgid ""
+" 5. ``port`` matches if included in the requested URL (exact matches only)"
+msgstr ""
+
+#: en/help_topics/authentication.txt:105
+msgid ""
+" 6. ``path`` matches if included in the requested URL (and by rule #2 "
+"above,\n"
+"    empty paths will match any provided path)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:110
+msgid ""
+"File format\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:113
+msgid ""
+"The general rules for :doc:`configuration files <configuration-help>`\n"
+"apply except for the variable policies."
+msgstr ""
+
+#: en/help_topics/authentication.txt:116
+msgid "Each section describes an authentication definition."
+msgstr ""
+
+#: en/help_topics/authentication.txt:118
+msgid ""
+"The section name is an arbitrary string, only the ``DEFAULT`` value is "
+"reserved\n"
+"and should appear as the *last* section."
+msgstr ""
+
+#: en/help_topics/authentication.txt:121
+msgid "Each section should define:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:123
+msgid "* ``user``: the login to be used,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:125
+msgid "Each section could define:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:127
+msgid "* ``host``: the remote server,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:129
+msgid "* ``port``: the port the server is listening,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:131
+msgid "* ``path``: the branch location,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:133
+msgid "* ``password``: the password."
+msgstr ""
+
+#: en/help_topics/authentication.txt:135
+msgid ""
+"\n"
+"Examples\n"
+"--------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:139
+msgid ""
+"\n"
+"Personal projects hosted outside\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/authentication.txt:143
+msgid ""
+"All connections are done with the same ``user`` (the remote one for which "
+"the\n"
+"default bzr one is not appropriate) and the password is always prompted "
+"with\n"
+"some exceptions::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:147
+msgid ""
+"        # Pet projects on hobby.net\n"
+"        [hobby]\n"
+"        host=r.hobby.net\n"
+"        user=jim\n"
+"        password=obvious1234\n"
+"        \n"
+"        # Home server\n"
+"        [home]\n"
+"        scheme=https\n"
+"        host=home.net\n"
+"        user=joe\n"
+"        password=1essobV10us\n"
+"        \n"
+"        [DEFAULT]\n"
+"        # Our local user is barbaz, on all remote sites we're known as "
+"foobar\n"
+"        user=foobar"
+msgstr ""
+
+#: en/help_topics/authentication.txt:164
+msgid ""
+"\n"
+"Source hosting provider\n"
+"~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/authentication.txt:168
+msgid "In the shp.net (fictitious) domain, each project has its own site::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:170
+msgid ""
+"        [shpnet domain]\n"
+"        # we use sftp, but ssh is the scheme used for authentication\n"
+"        scheme=ssh\n"
+"        # The leading '.' ensures that 'shp.net' alone doesn't match\n"
+"        host=.shp.net\n"
+"        user=joe\n"
+"        # bzr don't support supplying a password for sftp,\n"
+"        # consider using an ssh agent if you don't want to supply\n"
+"        # a password interactively. (pageant, ssh-agent, etc)"
+msgstr ""
+
+#: en/help_topics/authentication.txt:180
+msgid ""
+"HTTPS, SFTP servers and their proxy\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/authentication.txt:183
+msgid ""
+"At company.com, the server hosting release and integration branches is "
+"behind a\n"
+"proxy, and the two branches use different authentication policies::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:186
+msgid ""
+"        [reference code]\n"
+"        scheme=https\n"
+"        host=dev.company.com\n"
+"        path=/dev\n"
+"        user=user1\n"
+"        password=pass1"
+msgstr ""
+
+#: en/help_topics/authentication.txt:193
+msgid ""
+"        # development branches on dev server\n"
+"        [dev]\n"
+"        scheme=ssh # bzr+ssh and sftp are available here\n"
+"        host=dev.company.com\n"
+"        path=/dev/integration\n"
+"        user=user2\n"
+"        \n"
+"        # proxy\n"
+"        [proxy]\n"
+"        scheme=http\n"
+"        host=proxy.company.com\n"
+"        port=3128\n"
+"        user=proxyuser1\n"
+"        password=proxypass1"
+msgstr ""
+
+#: en/help_topics/authentication.txt:208
+msgid ""
+"\n"
+"Planned enhancements\n"
+"--------------------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:212
+msgid ""
+"The following are not yet implemented but planned as parts of a work in\n"
+"progress:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:215
+msgid "* add a  ``password_encoding`` field allowing:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:217
+msgid ""
+"  - storing the passwords in various obfuscating encodings (base64 for one),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:219
+msgid "  - delegate password storage to plugins (.netrc for example)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:221
+msgid ""
+"* update the credentials when the user is prompted for user or password,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:223
+msgid "* add a ``verify_certificates`` field for ``HTTPS``."
+msgstr ""
+
+#: en/help_topics/authentication.txt:225
+msgid ""
+"The ``password_encoding`` and ``verify_certificates`` fields are recognized "
+"but\n"
+"ignored in the actual implementation.\n"
+msgstr ""
+
+#: en/help_topics/bugs.txt:1
+msgid "Bug Tracker Settings"
+msgstr ""
+
+#: en/help_topics/bugs.txt:3
+msgid ""
+"When making a commit, metadata about bugs fixed by that change can be\n"
+"recorded by using the ``--fixes`` option. For each bug marked as fixed, an\n"
+"entry is included in the 'bugs' revision property stating '<url> <status>'.\n"
+"(The only ``status`` value currently supported is ``fixed.``)"
+msgstr ""
+
+#: en/help_topics/bugs.txt:8
+msgid ""
+"The ``--fixes`` option allows you to specify a bug tracker and a bug "
+"identifier\n"
+"rather than a full URL. This looks like::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:11
+msgid "    bzr commit --fixes <tracker>:<id>"
+msgstr ""
+
+#: en/help_topics/bugs.txt:13
+msgid "or::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:15
+msgid "    bzr commit --fixes <id>"
+msgstr ""
+
+#: en/help_topics/bugs.txt:17
+msgid ""
+"where \"<tracker>\" is an identifier for the bug tracker, and \"<id>\" is "
+"the\n"
+"identifier for that bug within the bugtracker, usually the bug number.\n"
+"If \"<tracker>\" is not specified the ``bugtracker`` set in the branch\n"
+"or global configuration is used."
+msgstr ""
+
+#: en/help_topics/bugs.txt:22
+msgid ""
+"Bazaar knows about a few bug trackers that have many users. If\n"
+"you use one of these bug trackers then there is no setup required to\n"
+"use this feature, you just need to know the tracker identifier to use.\n"
+"These are the bugtrackers that are built in:"
+msgstr ""
+
+#: en/help_topics/bugs.txt:27
+msgid ""
+"  ============================ ============ ============\n"
+"  URL                          Abbreviation Example\n"
+"  ============================ ============ ============\n"
+"  https://bugs.launchpad.net/  lp           lp:12345\n"
+"  http://bugs.debian.org/      deb          deb:12345\n"
+"  http://bugzilla.gnome.org/   gnome        gnome:12345\n"
+"  ============================ ============ ============"
+msgstr ""
+
+#: en/help_topics/bugs.txt:35
+msgid ""
+"For the bug trackers not listed above configuration is required.\n"
+"Support for generating the URLs for any project using Bugzilla or Trac\n"
+"is built in, along with a template mechanism for other bugtrackers with\n"
+"simple URL schemes. If your bug tracker can't be described by one\n"
+"of the schemes described below then you can write a plugin to support\n"
+"it."
+msgstr ""
+
+#: en/help_topics/bugs.txt:42
+msgid ""
+"If you use Bugzilla or Trac, then you only need to set a configuration\n"
+"variable which contains the base URL of the bug tracker. These options\n"
+"can go into ``bazaar.conf``, ``branch.conf`` or into a branch-specific\n"
+"configuration section in ``locations.conf``.  You can set up these values\n"
+"for each of the projects you work on."
+msgstr ""
+
+#: en/help_topics/bugs.txt:48
+msgid ""
+"Note: As you provide a short name for each tracker, you can specify one or\n"
+"more bugs in one or more trackers at commit time if you wish."
+msgstr ""
+
+#: en/help_topics/bugs.txt:51
+msgid ""
+"Launchpad\n"
+"---------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:54
+msgid "Use ``bzr commit --fixes lp:2`` to record that this commit fixes bug 2."
+msgstr ""
+
+#: en/help_topics/bugs.txt:56
+msgid ""
+"bugzilla_<tracker>_url\n"
+"----------------------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:59
+msgid ""
+"If present, the location of the Bugzilla bug tracker referred to by\n"
+"<tracker>. This option can then be used together with ``bzr commit\n"
+"--fixes`` to mark bugs in that tracker as being fixed by that commit. For\n"
+"example::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:64
+msgid "    bugzilla_squid_url = http://bugs.squid-cache.org"
+msgstr ""
+
+#: en/help_topics/bugs.txt:66
+msgid ""
+"would allow ``bzr commit --fixes squid:1234`` to mark Squid's bug 1234 as\n"
+"fixed."
+msgstr ""
+
+#: en/help_topics/bugs.txt:69
+msgid ""
+"trac_<tracker>_url\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:72
+msgid ""
+"If present, the location of the Trac instance referred to by\n"
+"<tracker>. This option can then be used together with ``bzr commit\n"
+"--fixes`` to mark bugs in that tracker as being fixed by that commit. For\n"
+"example::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:77
+msgid "    trac_twisted_url = http://www.twistedmatrix.com/trac"
+msgstr ""
+
+#: en/help_topics/bugs.txt:79
+msgid ""
+"would allow ``bzr commit --fixes twisted:1234`` to mark Twisted's bug 1234 "
+"as\n"
+"fixed."
+msgstr ""
+
+#: en/help_topics/bugs.txt:82
+msgid ""
+"bugtracker_<tracker>_url\n"
+"------------------------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:85
+msgid ""
+"If present, the location of a generic bug tracker instance referred to by\n"
+"<tracker>. The location must contain an ``{id}`` placeholder,\n"
+"which will be replaced by a specific bug ID. This option can then be used\n"
+"together with ``bzr commit --fixes`` to mark bugs in that tracker as being\n"
+"fixed by that commit. For example::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:91
+msgid "    bugtracker_python_url = http://bugs.python.org/issue{id}"
+msgstr ""
+
+#: en/help_topics/bugs.txt:93
+msgid ""
+"would allow ``bzr commit --fixes python:1234`` to mark bug 1234 in Python's\n"
+"Roundup bug tracker as fixed, or::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:96
+msgid ""
+"    bugtracker_cpan_url = http://rt.cpan.org/Public/Bug/Display.html?id={id}"
+msgstr ""
+
+#: en/help_topics/bugs.txt:98
+msgid ""
+"would allow ``bzr commit --fixes cpan:1234`` to mark bug 1234 in CPAN's\n"
+"RT bug tracker as fixed, or::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:101
+msgid "    bugtracker_hudson_url = http://issues.hudson-ci.org/browse/{id}"
+msgstr ""
+
+#: en/help_topics/bugs.txt:103
+msgid ""
+"would allow ``bzr commit --fixes hudson:HUDSON-1234`` to mark bug "
+"HUDSON-1234\n"
+"in Hudson's JIRA bug tracker as fixed.\n"
+msgstr ""
+
+#: en/help_topics/commands.txt:1
+msgid ""
+"add                Add specified files or directories.\n"
+"alias              Set/unset and display aliases.\n"
+"annotate           Show the origin of each line in a file.\n"
+"bash-completion    Generate a shell function for bash command line "
+"completion.\n"
+"                   [bash_completion]\n"
+"bd-do              Run a command in an exported package, copying the result\n"
+"                   back. [builddeb]\n"
+"bind               Convert the current branch into a checkout of the "
+"supplied\n"
+"                   branch.\n"
+"branch             Create a new branch that is a copy of an existing "
+"branch.\n"
+"branch-history     Display the development history of a branch. [bzrtools]\n"
+"branches           Scan a location for branches [bzrtools]\n"
+"break-lock         Break a dead lock.\n"
+"bug-url            Print full URL to a specific bug, or open it in your\n"
+"                   browser. [qbzr]\n"
+"builddeb           Builds a Debian package from a branch. [builddeb]\n"
+"cat                Write the contents of a file as of a given revision to\n"
+"                   standard output.\n"
+"cbranch            Create a new checkout, associated with a new repository\n"
+"                   branch. [bzrtools]\n"
+"cdiff              A color version of bzr's diff [bzrtools]\n"
+"check              Validate working tree structure, branch consistency and\n"
+"                   repository history.\n"
+"checkout           Create a new checkout of an existing branch.\n"
+"clean-tree         Remove unwanted files from working tree.\n"
+"commit             Commit changes into a new revision.\n"
+"config             Display, set or remove a configuration option.\n"
+"conflict-diff      Compare a conflicted file against BASE. [bzrtools]\n"
+"conflicts          List files with conflicts.\n"
+"create-mirror      Create a mirror of another branch. [bzrtools]\n"
+"deleted            List files deleted in the working tree.\n"
+"dep3-patch         Format the changes in a branch as a DEP-3 patch. "
+"[builddeb]\n"
+"dh-make            Helps you create a new package. [builddeb]\n"
+"diff               Show differences in the working tree, between revisions "
+"or\n"
+"                   branches.\n"
+"explorer           Desktop application for Bazaar. [explorer]\n"
+"export             Export current or past revision to a destination "
+"directory\n"
+"                   or archive.\n"
+"fetch-ghosts       Attempt to retrieve ghosts from another branch. "
+"[bzrtools]\n"
+"graph-ancestry     Produce ancestry graphs using dot. [bzrtools]\n"
+"heads              Show all revisions in a repository not having "
+"descendants.\n"
+"                   [bzrtools]\n"
+"help               Show help on a command or other topic.\n"
+"ignore             Ignore specified files or patterns.\n"
+"ignored            List ignored files and the patterns that matched them.\n"
+"import             Import sources from a directory, tarball or zip file\n"
+"                   [bzrtools]\n"
+"import-dsc         Import a series of source packages. [builddeb]\n"
+"import-upstream    Imports an upstream tarball. [builddeb]\n"
+"info               Show information about a working tree, branch or "
+"repository.\n"
+"init               Make a directory into a versioned branch.\n"
+"init-repository    Create a shared repository for branches to share storage\n"
+"                   space.\n"
+"init-workspace     Start a new workspace. [explorer]\n"
+"join               Combine a tree into its containing tree.\n"
+"launchpad-login    Show or set the Launchpad user ID. [launchpad]\n"
+"launchpad-mirror   Ask Launchpad to mirror a branch now. [launchpad]\n"
+"launchpad-open     Open a Launchpad branch page in your web browser.\n"
+"                   [launchpad]\n"
+"link-tree          Hardlink matching files to another tree. [bzrtools]\n"
+"log                Show historical log for a branch or subset of a branch.\n"
+"lp-find-proposal   Find the proposal to merge this revision. [launchpad]\n"
+"lp-propose-merge   Propose merging a branch on Launchpad. [launchpad]\n"
+"ls                 List files in a tree.\n"
+"mark-uploaded      Mark that this branch has been uploaded, prior to "
+"pushing\n"
+"                   it. [builddeb]\n"
+"merge              Perform a three-way merge.\n"
+"merge-package      Merges source packaging branch into target packaging "
+"branch.\n"
+"                   [builddeb]\n"
+"merge-upstream     Merges a new upstream version into the current branch.\n"
+"                   [builddeb]\n"
+"missing            Show unmerged/unpulled revisions between two branches.\n"
+"mkdir              Create a new versioned directory.\n"
+"multi-pull         Pull all the branches under a location, e.g. a "
+"repository.\n"
+"                   [bzrtools]\n"
+"mv                 Move or rename a file.\n"
+"nick               Print or set the branch nickname.\n"
+"pack               Compress the data within a repository.\n"
+"patch              Apply a named patch to the current tree. [bzrtools]\n"
+"plugins            List the installed plugins.\n"
+"pull               Turn this branch into a mirror of another branch.\n"
+"push               Update a mirror of this branch.\n"
+"qadd               GUI for adding files or directories. [qbzr]\n"
+"qannotate          Show the origin of each line in a file. [qbzr]\n"
+"qbind              Convert the current branch into a checkout of the "
+"supplied\n"
+"                   branch. [qbzr]\n"
+"qbranch            Create a new copy of a branch. [qbzr]\n"
+"qbrowse            Show inventory or working tree. [qbzr]\n"
+"qcat               View the contents of a file as of a given revision. "
+"[qbzr]\n"
+"qcommit            GUI for committing revisions. [qbzr]\n"
+"qconfig            Configure Bazaar and QBzr. [qbzr]\n"
+"qconflicts         Show conflicts. [qbzr]\n"
+"qdiff              Show differences in working tree in a GUI window. [qbzr]\n"
+"qexport            Export current or past revision to a destination "
+"directory\n"
+"                   or archive. [qbzr]\n"
+"qgetnew            Creates a new working tree (either a checkout or full\n"
+"                   branch). [qbzr]\n"
+"qgetupdates        Fetches external changes into the working tree. [qbzr]\n"
+"qinfo              Shows information about the current location. [qbzr]\n"
+"qinit              Initializes a new branch or shared repository. [qbzr]\n"
+"qinit-workspace    Start a new workspace using a GUI. [explorer]\n"
+"qlog               Show log of a repository, branch, file, or directory in a "
+"Qt\n"
+"                   window. [qbzr]\n"
+"qmerge             Perform a three-way merge. [qbzr]\n"
+"qplugins           Show information about installed plugins. [qbzr]\n"
+"qpull              Turn this branch into a mirror of another branch. [qbzr]\n"
+"qpush              Update a mirror of this branch. [qbzr]\n"
+"qrevert            Revert changes files. [qbzr]\n"
+"qrun               Run arbitrary bzr command. [qbzr]\n"
+"qsend              Mail or create a merge-directive for submitting changes.\n"
+"                   [qbzr]\n"
+"qshelve            Shelve selected changes away. [qbzr]\n"
+"qswitch            Set the branch of a checkout and update. [qbzr]\n"
+"qtag               Edit tags. [qbzr]\n"
+"qunbind            Convert the current checkout into a regular branch. "
+"[qbzr]\n"
+"quncommit          Move the tip of a branch to an earlier revision. [qbzr]\n"
+"qunshelve          Restore shalved changes. [qbzr]\n"
+"qupdate            Update working tree with latest changes in the branch.\n"
+"                   [qbzr]\n"
+"qverify-signatures Shows digital signature statuses for branch commits "
+"[qbzr]\n"
+"qversion           Show version/system information. [qbzr]\n"
+"qviewer            Simple file viewer. [qbzr]\n"
+"reconcile          Reconcile bzr metadata in a branch.\n"
+"reconfigure        Reconfigure the type of a bzr directory.\n"
+"register-branch    Register a branch with launchpad.net. [launchpad]\n"
+"remerge            Redo a merge.\n"
+"remove             Remove files or directories.\n"
+"remove-branch      Remove a branch.\n"
+"remove-tree        Remove the working tree from a given branch/checkout.\n"
+"renames            Show list of renamed files.\n"
+"resolve            Mark a conflict as resolved.\n"
+"revert             Revert files to a previous revision.\n"
+"revno              Show current revision number.\n"
+"root               Show the tree root directory.\n"
+"rspush             Upload this branch to another location using rsync.\n"
+"                   [bzrtools]\n"
+"send               Mail or create a merge-directive for submitting changes.\n"
+"serve              Run the bzr server.\n"
+"shelf1             Perform various operations on your shelved patches. See "
+"also\n"
+"                   shelve1. [bzrtools]\n"
+"shell              Begin an interactive shell tailored for bzr. [bzrtools]\n"
+"shelve             Temporarily set aside some changes from the current "
+"tree.\n"
+"shelve1            Temporarily set aside some changes from the current "
+"tree.\n"
+"                   [bzrtools]\n"
+"sign-my-commits    Sign all commits by a given committer.\n"
+"split              Split a subdirectory of a tree into a separate tree.\n"
+"status             Display status summary.\n"
+"switch             Set the branch of a checkout and update.\n"
+"tag                Create, remove or modify a tag naming a revision.\n"
+"tags               List tags.\n"
+"testament          Show testament (signing-form) of a revision.\n"
+"trees              Scan a location for trees [bzrtools]\n"
+"unbind             Convert the current checkout into a regular branch.\n"
+"uncommit           Remove the last committed revision.\n"
+"unshelve           Restore shelved changes.\n"
+"unshelve1          Restore shelved changes. [bzrtools]\n"
+"update             Update a tree to have the latest code committed to its\n"
+"                   branch.\n"
+"upgrade            Upgrade a repository, branch or working tree to a newer\n"
+"                   format.\n"
+"verify-signatures  Verify all commit signatures.\n"
+"version            Show version of bzr.\n"
+"version-info       Show version information about this tree.\n"
+"view               Manage filtered views.\n"
+"whoami             Show or set bzr user id.\n"
+"zap                Remove a lightweight checkout, if it can be done safely.\n"
+"                   [bzrtools]\n"
+msgstr ""
+
+#: en/help_topics/configuration.txt:1
+msgid ""
+"Configuration Settings\n"
+"======================="
+msgstr ""
+
+#: en/help_topics/configuration.txt:4
+msgid ""
+"Environment settings\n"
+"---------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:7
+msgid ""
+"While most configuration is handled by configuration files, some options\n"
+"which may be semi-permanent can also be controlled through the environment."
+msgstr ""
+
+#: en/help_topics/configuration.txt:10
+msgid ""
+"BZR_EMAIL\n"
+"~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:13
+msgid "Override the email id used by Bazaar.  Typical format::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:15
+msgid "  \"John Doe <jdoe@example.com>\""
+msgstr ""
+
+#: en/help_topics/configuration.txt:17
+msgid "See also the ``email`` configuration option."
+msgstr ""
+
+#: en/help_topics/configuration.txt:19
+msgid ""
+"BZR_PROGRESS_BAR\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:22
+msgid ""
+"Override the progress display.  Possible values are \"none\" or \"text\".  "
+"If\n"
+"the value is \"none\" then no progress bar is displayed.  The value \"text\" "
+"draws\n"
+"the ordinary command line progress bar."
+msgstr ""
+
+#: en/help_topics/configuration.txt:26
+msgid ""
+"BZR_SIGQUIT_PDB\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:29
+msgid "Control whether SIGQUIT behaves normally or invokes a breakin debugger."
+msgstr ""
+
+#: en/help_topics/configuration.txt:31
+msgid ""
+"* 0 = Standard SIGQUIT behavior (normally, exit with a core dump)\n"
+"* 1 = Invoke breakin debugger (default)"
+msgstr ""
+
+#: en/help_topics/configuration.txt:34
+msgid ""
+"BZR_HOME\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:37
+msgid "Override the home directory used by Bazaar."
+msgstr ""
+
+#: en/help_topics/configuration.txt:39
+msgid ""
+"BZR_SSH\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:42
+msgid "Select a different SSH implementation."
+msgstr ""
+
+#: en/help_topics/configuration.txt:44
+msgid ""
+"BZR_PDB\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:47
+msgid "Control whether to launch a debugger on error."
+msgstr ""
+
+#: en/help_topics/configuration.txt:49
+msgid ""
+"* 0 = Standard behavior\n"
+"* 1 = Launch debugger"
+msgstr ""
+
+#: en/help_topics/configuration.txt:52
+msgid ""
+"BZR_REMOTE_PATH\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:55
+msgid "Path to the Bazaar executable to use when using the bzr+ssh protocol."
+msgstr ""
+
+#: en/help_topics/configuration.txt:57
+msgid "See also the ``bzr_remote_path`` configuration option."
+msgstr ""
+
+#: en/help_topics/configuration.txt:59
+msgid ""
+"BZR_EDITOR\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:62
+msgid "Path to the editor Bazaar should use for commit messages, etc."
+msgstr ""
+
+#: en/help_topics/configuration.txt:64
+msgid ""
+"BZR_LOG\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:67
+msgid ""
+"Location of the Bazaar log file. You can check the current location by\n"
+"running ``bzr version``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:70
+msgid ""
+"The log file contains debug information that is useful for diagnosing or\n"
+"reporting problems with Bazaar."
+msgstr ""
+
+#: en/help_topics/configuration.txt:73
+msgid ""
+"Setting this to ``NUL`` on Windows or ``/dev/null`` on other platforms\n"
+"will disable logging."
+msgstr ""
+
+#: en/help_topics/configuration.txt:76
+msgid ""
+"\n"
+"BZR_PLUGIN_PATH\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:80
+msgid ""
+"The path to the plugins directory that Bazaar should use.\n"
+"If not set, Bazaar will search for plugins in:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:83
+msgid "* the user specific plugin directory (containing the ``user`` plugins),"
+msgstr ""
+
+#: en/help_topics/configuration.txt:85
+msgid "* the bzrlib directory (containing the ``core`` plugins),"
+msgstr ""
+
+#: en/help_topics/configuration.txt:87
+msgid ""
+"* the site specific plugin directory if applicable (containing\n"
+"  the ``site`` plugins)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:90
+msgid ""
+"If ``BZR_PLUGIN_PATH`` is set in any fashion, it will change the\n"
+"the way the plugin are searched. "
+msgstr ""
+
+#: en/help_topics/configuration.txt:93
+msgid ""
+"As for the ``PATH`` variables, if multiple directories are\n"
+"specified in ``BZR_PLUGIN_PATH`` they should be separated by the\n"
+"platform specific appropriate character (':' on Unix,\n"
+"';' on windows)"
+msgstr ""
+
+#: en/help_topics/configuration.txt:98
+msgid ""
+"By default if ``BZR_PLUGIN_PATH`` is set, it replaces searching\n"
+"in ``user``.  However it will continue to search in ``core`` and\n"
+"``site`` unless they are explicitly removed."
+msgstr ""
+
+#: en/help_topics/configuration.txt:102
+msgid ""
+"If you need to change the order or remove one of these\n"
+"directories, you should use special values:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:105
+msgid ""
+"* ``-user``, ``-core``, ``-site`` will remove the corresponding\n"
+"  path from the default values,"
+msgstr ""
+
+#: en/help_topics/configuration.txt:108
+msgid ""
+"* ``+user``, ``+core``, ``+site`` will add the corresponding path\n"
+"  before the remaining default values (and also remove it from\n"
+"  the default values)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:112
+msgid ""
+"Note that the special values 'user', 'core' and 'site' should be\n"
+"used literally, they will be substituted by the corresponding,\n"
+"platform specific, values."
+msgstr ""
+
+#: en/help_topics/configuration.txt:116
+msgid ""
+"The examples below use ':' as the separator, windows users\n"
+"should use ';'."
+msgstr ""
+
+#: en/help_topics/configuration.txt:119
+msgid "Overriding the default user plugin directory::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:121
+msgid "  BZR_PLUGIN_PATH='/path/to/my/other/plugins'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:123
+msgid "Disabling the site directory while retaining the user directory::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:125
+msgid "  BZR_PLUGIN_PATH='-site:+user'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:127
+msgid "Disabling all plugins (better achieved with --no-plugins)::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:129
+msgid "  BZR_PLUGIN_PATH='-user:-core:-site'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:131
+msgid "Overriding the default site plugin directory::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:133
+msgid "  BZR_PLUGIN_PATH='/path/to/my/site/plugins:-site':+user"
+msgstr ""
+
+#: en/help_topics/configuration.txt:135
+msgid ""
+"BZR_DISABLE_PLUGINS\n"
+"~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:138
+msgid ""
+"Under special circumstances (mostly when trying to diagnose a\n"
+"bug), it's better to disable a plugin (or several) rather than\n"
+"uninstalling them completely. Such plugins can be specified in\n"
+"the ``BZR_DISABLE_PLUGINS`` environment variable."
+msgstr ""
+
+#: en/help_topics/configuration.txt:143
+msgid ""
+"In that case, ``bzr`` will stop loading the specified plugins and\n"
+"will raise an import error if they are explicitly imported (by\n"
+"another plugin that depends on them for example)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:147
+msgid "Disabling ``myplugin`` and ``yourplugin`` is achieved by::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:149
+msgid "  BZR_DISABLE_PLUGINS='myplugin:yourplugin'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:151
+msgid ""
+"BZR_PLUGINS_AT\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:154
+msgid ""
+"When adding a new feature or working on a bug in a plugin,\n"
+"developers often need to use a specific version of a given\n"
+"plugin. Since python requires that the directory containing the\n"
+"code is named like the plugin itself this make it impossible to\n"
+"use arbitrary directory names (using a two-level directory scheme\n"
+"is inconvenient). ``BZR_PLUGINS_AT`` allows such directories even\n"
+"if they don't appear in ``BZR_PLUGIN_PATH`` ."
+msgstr ""
+
+#: en/help_topics/configuration.txt:162
+msgid ""
+"Plugins specified in this environment variable takes precedence\n"
+"over the ones in ``BZR_PLUGIN_PATH``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:165
+msgid ""
+"The variable specified a list of ``plugin_name@plugin path``,\n"
+"``plugin_name`` being the name of the plugin as it appears in\n"
+"python module paths, ``plugin_path`` being the path to the\n"
+"directory containing the plugin code itself\n"
+"(i.e. ``plugins/myplugin`` not ``plugins``).  Use ':' as the list\n"
+"separator, use ';' on windows."
+msgstr ""
+
+#: en/help_topics/configuration.txt:172
+msgid ""
+"Example:\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:175
+msgid ""
+"Using a specific version of ``myplugin``:\n"
+"``BZR_PLUGINS_AT='myplugin@/home/me/bugfixes/123456-myplugin``"
+msgstr ""
+
+#: en/help_topics/configuration.txt:178
+msgid ""
+"BZRPATH\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:181
+msgid "The path where Bazaar should look for shell plugin external commands."
+msgstr ""
+
+#: en/help_topics/configuration.txt:183
+msgid ""
+"\n"
+"http_proxy, https_proxy\n"
+"~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:187
+msgid "Specifies the network proxy for outgoing connections, for example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:189
+msgid ""
+"  http_proxy=http://proxy.example.com:3128/ \n"
+"  https_proxy=http://proxy.example.com:3128/"
+msgstr ""
+
+#: en/help_topics/configuration.txt:192
+msgid ""
+"\n"
+"Configuration files\n"
+"-------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:196
+msgid ""
+"Location\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:199
+msgid ""
+"Configuration files are located in ``$HOME/.bazaar`` on Unix and\n"
+"``C:\\Documents and Settings\\<username>\\Application Data\\Bazaar\\2.0`` "
+"on\n"
+"Windows. (You can check the location for your system by using\n"
+"``bzr version``.)"
+msgstr ""
+
+#: en/help_topics/configuration.txt:204
+msgid "There are three primary configuration files in this location:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:206
+msgid "* ``bazaar.conf`` describes default configuration options,"
+msgstr ""
+
+#: en/help_topics/configuration.txt:208
+msgid ""
+"* ``locations.conf`` describes configuration information for\n"
+"  specific branch locations,"
+msgstr ""
+
+#: en/help_topics/configuration.txt:211
+msgid ""
+"* ``authentication.conf`` describes credential information for\n"
+"  remote servers."
+msgstr ""
+
+#: en/help_topics/configuration.txt:214
+msgid ""
+"Each branch can also contain a configuration file that sets values specific\n"
+"to that branch. This file is found at ``.bzr/branch/branch.conf`` within "
+"the\n"
+"branch. This file is visible to all users of a branch, if you wish to "
+"override\n"
+"one of the values for a branch with a setting that is specific to you then "
+"you\n"
+"can do so in ``locations.conf``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:220
+msgid ""
+"General format\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:223
+msgid ""
+"An ini file has three types of contructs: section headers, section\n"
+"variables and comments."
+msgstr ""
+
+#: en/help_topics/configuration.txt:226
+msgid ""
+"Comments\n"
+"^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:229
+msgid ""
+"A comment is any line that starts with a \"#\" (sometimes called a \"hash\n"
+"mark\", \"pound sign\" or \"number sign\"). Comment lines are ignored by\n"
+"Bazaar when parsing ini files."
+msgstr ""
+
+#: en/help_topics/configuration.txt:233
+msgid ""
+"Section headers\n"
+"^^^^^^^^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:236
+msgid ""
+"A section header is a word enclosed in brackets that starts at the begining\n"
+"of a line.  A typical section header looks like this::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:239
+msgid "    [DEFAULT]"
+msgstr ""
+
+#: en/help_topics/configuration.txt:241
+msgid ""
+"The only valid section headers for bazaar.conf currently are [DEFAULT] and\n"
+"[ALIASES].  Section headers are case sensitive. The default section provides "
+"for\n"
+"setting variables which can be overridden with the branch config file."
+msgstr ""
+
+#: en/help_topics/configuration.txt:245
+msgid ""
+"For ``locations.conf``, the variables from the section with the\n"
+"longest matching section header are used to the exclusion of other\n"
+"potentially valid section headers. A section header uses the path for\n"
+"the branch as the section header. Some examples include::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:250
+msgid ""
+"    [http://mybranches.isp.com/~jdoe/branchdir]\n"
+"    [/home/jdoe/branches/]"
+msgstr ""
+
+#: en/help_topics/configuration.txt:253
+msgid ""
+"\n"
+"Section variables\n"
+"^^^^^^^^^^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:257
+msgid ""
+"A section variable resides within a section. A section variable contains a\n"
+"variable name, an equals sign and a value.  For example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:260
+msgid ""
+"    email            = John Doe <jdoe@isp.com>\n"
+"    gpg_signing_key  = Amy Pond <amy@example.com>"
+msgstr ""
+
+#: en/help_topics/configuration.txt:263
+msgid ""
+"A variable can reference other variables **in the same configuration file** "
+"by\n"
+"enclosing them in curly brackets::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:266
+msgid ""
+"    my_branch_name = feature_x\n"
+"    my_server      = bzr+ssh://example.com\n"
+"    push_location   = {my_server}/project/{my_branch_name}"
+msgstr ""
+
+#: en/help_topics/configuration.txt:270
+msgid ""
+"\n"
+"Variable policies\n"
+"^^^^^^^^^^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:274
+msgid ""
+"Variables defined in a section affect the named directory or URL plus\n"
+"any locations they contain.  Policies can be used to change how a\n"
+"variable value is interpreted for contained locations.  Currently\n"
+"there are three policies available:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:279
+msgid ""
+" none:\n"
+"   the value is interpreted the same for contained locations.  This is\n"
+"   the default behaviour.\n"
+" norecurse:\n"
+"   the value is only used for the exact location specified by the\n"
+"   section name.\n"
+" appendpath:\n"
+"   for contained locations, any additional path components are\n"
+"   appended to the value."
+msgstr ""
+
+#: en/help_topics/configuration.txt:289
+msgid ""
+"Policies are specified by keys with names of the form \"$var:policy\".\n"
+"For example, to define the push location for a tree of branches, the\n"
+"following could be used::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:293
+msgid ""
+"  [/top/location]\n"
+"  push_location = sftp://example.com/location\n"
+"  push_location:policy = appendpath"
+msgstr ""
+
+#: en/help_topics/configuration.txt:297
+msgid ""
+"With this configuration, the push location for ``/top/location/branch1``\n"
+"would be ``sftp://example.com/location/branch1``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:300
+msgid ""
+"\n"
+"The main configuration file, bazaar.conf\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:304
+msgid ""
+"``bazaar.conf`` allows two sections: ``[DEFAULT]`` and ``[ALIASES]``.\n"
+"The default section contains the default\n"
+"configuration options for all branches. The default section can be\n"
+"overriden by providing a branch-specific section in ``locations.conf``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:309
+msgid "A typical ``bazaar.conf`` section often looks like the following::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:311
+msgid ""
+"    [DEFAULT]\n"
+"    email             = John Doe <jdoe@isp.com>\n"
+"    editor            = /usr/bin/vim\n"
+"    create_signatures = when-required"
+msgstr ""
+
+#: en/help_topics/configuration.txt:316
+msgid ""
+"\n"
+"The branch location configuration file, locations.conf\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:320
+msgid ""
+"``locations.conf`` allows one to specify overriding settings for\n"
+"a specific branch. The format is almost identical to the default section in\n"
+"bazaar.conf with one significant change: The section header, instead of "
+"saying\n"
+"default, will be the path to a branch that you wish to override a value\n"
+"for. The '?' and '*' wildcards are supported::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:326
+msgid ""
+"    [/home/jdoe/branches/nethack]\n"
+"    email = Nethack Admin <nethack@nethack.com>"
+msgstr ""
+
+#: en/help_topics/configuration.txt:329
+msgid ""
+"    [http://hypothetical.site.com/branches/devel-branch]\n"
+"    create_signatures = always"
+msgstr ""
+
+#: en/help_topics/configuration.txt:332
+msgid ""
+"The authentication configuration file, authentication.conf\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:335
+msgid ""
+"``authentication.conf`` allows one to specify credentials for\n"
+"remote servers. This can be used for all the supported transports and any "
+"part\n"
+"of bzr that requires authentication (smtp for example)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:339
+msgid ""
+"The syntax of the file obeys the same rules as the others except for the\n"
+"variable policies which don't apply."
+msgstr ""
+
+#: en/help_topics/configuration.txt:342
+msgid ""
+"For more information on the possible uses of the authentication "
+"configuration\n"
+"file see :doc:`authentication-help`."
+msgstr ""
+
+#: en/help_topics/configuration.txt:345
+msgid ""
+"\n"
+"Common variable options\n"
+"-----------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:349
+msgid ""
+"debug_flags\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:352
+msgid ""
+"A comma-separated list of debugging options to turn on.  The same values\n"
+"can be used as with the -D command-line option (see `help global-options`).\n"
+"For example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:356
+msgid "    debug_flags = hpss"
+msgstr ""
+
+#: en/help_topics/configuration.txt:360
+msgid "    debug_flags = hpss,evil"
+msgstr ""
+
+#: en/help_topics/configuration.txt:362
+msgid ""
+"email\n"
+"~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:365
+msgid ""
+"The email address to use when committing a branch. Typically takes the form\n"
+"of::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:368
+msgid "    email = Full Name <account@hostname.tld>"
+msgstr ""
+
+#: en/help_topics/configuration.txt:370
+msgid ""
+"editor\n"
+"~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:373
+msgid ""
+"The path of the editor that you wish to use if *bzr commit* is run without\n"
+"a commit message. This setting is trumped by the environment variable\n"
+"``BZR_EDITOR``, and overrides the ``VISUAL`` and ``EDITOR`` environment\n"
+"variables."
+msgstr ""
+
+#: en/help_topics/configuration.txt:378
+msgid ""
+"log_format\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:381
+msgid ""
+"The default log format to use. Standard log formats are ``long``, ``short``\n"
+"and ``line``. Additional formats may be provided by plugins. The default\n"
+"value is ``long``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:385
+msgid ""
+"check_signatures\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:388
+msgid ""
+"Reserved for future use.  These options will allow a policy for branches to\n"
+"require signatures."
+msgstr ""
+
+#: en/help_topics/configuration.txt:391
+msgid ""
+"require\n"
+"    The gnupg signature for revisions must be present and must be valid."
+msgstr ""
+
+#: en/help_topics/configuration.txt:394
+msgid ""
+"ignore\n"
+"    Do not check gnupg signatures of revisions."
+msgstr ""
+
+#: en/help_topics/configuration.txt:397
+msgid ""
+"check-available\n"
+"    (default) If gnupg signatures for revisions are present, check them.\n"
+"    Bazaar will fail if it finds a bad signature, but will not fail if\n"
+"    no signature is present."
+msgstr ""
+
+#: en/help_topics/configuration.txt:402
+msgid ""
+"create_signatures\n"
+"~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:405
+msgid ""
+"Defines the behaviour of signing revisions on commits.  By default bzr will "
+"not\n"
+"sign new commits."
+msgstr ""
+
+#: en/help_topics/configuration.txt:408
+msgid ""
+"always\n"
+"    Sign every new revision that is committed.  If the signing fails then "
+"the\n"
+"    commit will not be made."
+msgstr ""
+
+#: en/help_topics/configuration.txt:412
+msgid ""
+"when-required\n"
+"    Reserved for future use."
+msgstr ""
+
+#: en/help_topics/configuration.txt:415
+msgid ""
+"never\n"
+"    Reserved for future use."
+msgstr ""
+
+#: en/help_topics/configuration.txt:418
+msgid ""
+"In future it is planned that ``when-required`` will sign newly\n"
+"committed revisions only when the branch requires them.  ``never`` will "
+"refuse\n"
+"to sign newly committed revisions, even if the branch requires signatures."
+msgstr ""
+
+#: en/help_topics/configuration.txt:422
+msgid ""
+"dirstate.fdatasync\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:425
+msgid ""
+"If true (default), working tree metadata changes are flushed through the\n"
+"OS buffers to physical disk.  This is somewhat slower, but means data\n"
+"should not be lost if the machine crashes.  See also repository.fdatasync."
+msgstr ""
+
+#: en/help_topics/configuration.txt:429
+msgid ""
+"gpg_signing_key\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:432
+msgid ""
+"The GnuPG user identity to use when signing commits.  Can be an e-mail\n"
+"address, key fingerprint or full key ID.  When unset or when set to\n"
+"\"default\" Bazaar will use the user e-mail set with ``whoami``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:436
+msgid ""
+"recurse\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:439
+msgid ""
+"Only useful in ``locations.conf``. Defines whether or not the\n"
+"configuration for this section applies to subdirectories:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:442
+msgid ""
+"true\n"
+"    (default) This section applies to subdirectories as well."
+msgstr ""
+
+#: en/help_topics/configuration.txt:445
+msgid ""
+"false\n"
+"    This section only applies to the branch at this directory and not\n"
+"    branches below it."
+msgstr ""
+
+#: en/help_topics/configuration.txt:449
+msgid ""
+"gpg_signing_command\n"
+"~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:452
+msgid ""
+"(Default: \"gpg\"). Which program should be used to sign and check "
+"revisions.\n"
+"For example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:455
+msgid "    gpg_signing_command = /usr/bin/gnpg"
+msgstr ""
+
+#: en/help_topics/configuration.txt:457
+msgid ""
+"The specified command must accept the options \"--clearsign\" and \"-u "
+"<email>\"."
+msgstr ""
+
+#: en/help_topics/configuration.txt:459
+msgid ""
+"bzr_remote_path\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:462
+msgid ""
+"(Default: \"bzr\").  The path to the command that should be used to run the "
+"smart\n"
+"server for bzr.  This value may only be specified in locations.conf, because:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:465
+msgid ""
+"- it's needed before branch.conf is accessible\n"
+"- allowing remote branch.conf files to specify commands would be a security\n"
+"  risk"
+msgstr ""
+
+#: en/help_topics/configuration.txt:469
+msgid "It is overridden by the BZR_REMOTE_PATH environment variable."
+msgstr ""
+
+#: en/help_topics/configuration.txt:471
+msgid ""
+"smtp_server\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:474
+msgid ""
+"(Default: \"localhost\"). SMTP server to use when Bazaar needs to send\n"
+"email, eg. with ``merge-directive --mail-to``, or the bzr-email plugin."
+msgstr ""
+
+#: en/help_topics/configuration.txt:477
+msgid ""
+"smtp_username, smtp_password\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:480
+msgid ""
+"User and password to authenticate to the SMTP server. If smtp_username\n"
+"is set, and smtp_password is not, Bazaar will prompt for a password.\n"
+"These settings are only needed if the SMTP server requires authentication\n"
+"to send mail."
+msgstr ""
+
+#: en/help_topics/configuration.txt:485
+msgid ""
+"locks.steal_dead\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:488
+msgid ""
+"If set to true, bzr will automatically break locks held by processes from\n"
+"the same machine and user that are no longer alive.  Otherwise, it will\n"
+"print a message and you can break the lock manually, if you are satisfied\n"
+"the object is no longer in use."
+msgstr ""
+
+#: en/help_topics/configuration.txt:493
+msgid ""
+"mail_client\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:496
+msgid ""
+"A mail client to use for sending merge requests.\n"
+"By default, bzr will try to use ``mapi`` on Windows.  On other platforms, "
+"it\n"
+"will try ``xdg-email``. If either of these fails, it will fall back to\n"
+"``editor``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:501
+msgid "Supported values for specific clients:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:503
+msgid ""
+":claws: Use Claws.  This skips a dialog for attaching files.\n"
+":evolution: Use Evolution.\n"
+":kmail: Use KMail.\n"
+":mutt: Use Mutt.\n"
+":thunderbird: Use Mozilla Thunderbird or Icedove.  For Thunderbird/Icedove "
+"1.5,\n"
+"    this works around some bugs that xdg-email doesn't handle."
+msgstr ""
+
+#: en/help_topics/configuration.txt:510
+msgid "Supported generic values are:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:512
+msgid ""
+":default: See above.\n"
+":editor: Use your editor to compose the merge request.  This also uses\n"
+"    your commit id, (see ``bzr whoami``), smtp_server and (optionally)\n"
+"    smtp_username and smtp_password.\n"
+":mapi: Use your preferred e-mail client on Windows.\n"
+":xdg-email: Use xdg-email to run your preferred mail program"
+msgstr ""
+
+#: en/help_topics/configuration.txt:519
+msgid ""
+"repository.fdatasync\n"
+"~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:522
+msgid ""
+"If true (default), repository changes are flushed through the OS buffers\n"
+"to physical disk.  This is somewhat slower, but means data should not be\n"
+"lost if the machine crashes.  See also dirstate.fdatasync."
+msgstr ""
+
+#: en/help_topics/configuration.txt:526
+msgid ""
+"submit_branch\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:529
+msgid ""
+"The branch you intend to submit your current work to.  This is "
+"automatically\n"
+"set by ``bzr send``, and is also used by the ``submit:`` revision spec.  "
+"This\n"
+"should usually be set on a per-branch or per-location basis."
+msgstr ""
+
+#: en/help_topics/configuration.txt:533
+msgid ""
+"public_branch\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:536
+msgid ""
+"A publically-accessible version of this branch (implying that this version "
+"is\n"
+"not publically-accessible).  Used (and set) by ``bzr send``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:539
+msgid ""
+"suppress_warnings\n"
+"~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:542
+msgid ""
+"A list of strings, each string represent a warning that can be emitted by\n"
+"bzr. Mentioning a warning in this list tells bzr to not emit it."
+msgstr ""
+
+#: en/help_topics/configuration.txt:545
+msgid "Valid values:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:547
+msgid ""
+"* ``format_deprecation``:\n"
+"    whether the format deprecation warning is shown on repositories that "
+"are\n"
+"    using deprecated formats."
+msgstr ""
+
+#: en/help_topics/configuration.txt:551
+msgid ""
+"default_format\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:554
+msgid ""
+"A format name for the default format used when creating branches.  See "
+"``bzr\n"
+"help formats`` for possible values."
+msgstr ""
+
+#: en/help_topics/configuration.txt:557
+msgid ""
+"\n"
+"Unicode options\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:561
+msgid ""
+"output_encoding\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:564
+msgid ""
+"A Python unicode encoding name for text output from bzr, such as log\n"
+"information.  Values include: utf8, cp850, ascii, iso-8859-1.  The default\n"
+"is the terminal encoding prefered by the operating system."
+msgstr ""
+
+#: en/help_topics/configuration.txt:568
+msgid ""
+"\n"
+"Branch type specific options\n"
+"----------------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:572
+msgid ""
+"These options apply only to branches that use the ``dirstate-tags`` or\n"
+"later format.  They\n"
+"are usually set in ``.bzr/branch/branch.conf`` automatically, but may be\n"
+"manually set in ``locations.conf`` or ``bazaar.conf``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:577
+msgid ""
+"append_revisions_only\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:580
+msgid ""
+"If set to \"True\" then revisions can only be appended to the log, not\n"
+"removed.  A branch with this setting enabled can only pull from another\n"
+"branch if the other branch's log is a longer version of its own.  This is\n"
+"normally set by ``bzr init --append-revisions-only``. If you set it\n"
+"manually, use either 'True' or 'False' (case-sensitive) to maintain\n"
+"compatibility with previous bzr versions (older than 2.2)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:587
+msgid ""
+"parent_location\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:590
+msgid ""
+"If present, the location of the default branch for pull or merge.  This "
+"option\n"
+"is normally set when creating a branch, the first ``pull`` or by ``pull\n"
+"--remember``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:594
+msgid ""
+"push_location\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:597
+msgid ""
+"If present, the location of the default branch for push.  This option\n"
+"is normally set by the first ``push`` or ``push --remember``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:600
+msgid ""
+"push_strict\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:603
+msgid ""
+"If present, defines the ``--strict`` option default value for checking\n"
+"uncommitted changes before pushing."
+msgstr ""
+
+#: en/help_topics/configuration.txt:606
+msgid ""
+"dpush_strict\n"
+"~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:609
+msgid ""
+"If present, defines the ``--strict`` option default value for checking\n"
+"uncommitted changes before pushing into a different VCS without any\n"
+"custom bzr metadata."
+msgstr ""
+
+#: en/help_topics/configuration.txt:613
+msgid ""
+"bound_location\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:616
+msgid ""
+"The location that commits should go to when acting as a checkout.\n"
+"This option is normally set by ``bind``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:619
+msgid ""
+"bound\n"
+"~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:622
+msgid ""
+"If set to \"True\", the branch should act as a checkout, and push each "
+"commit to\n"
+"the bound_location.  This option is normally set by ``bind``/``unbind``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:625
+msgid ""
+"send_strict\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:628
+msgid ""
+"If present, defines the ``--strict`` option default value for checking\n"
+"uncommitted changes before sending a merge directive."
+msgstr ""
+
+#: en/help_topics/configuration.txt:631
+msgid ""
+"add.maximum_file_size\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:634
+msgid ""
+"Defines the maximum file size the command line \"add\" operation will allow\n"
+"in recursive mode, with files larger than this value being skipped. You "
+"may \n"
+"specify this value as an integer (in which case it is interpreted as "
+"bytes), \n"
+"or you may specify the value using SI units, i.e. 10KB, 20MB, 1G. A value of "
+"0 \n"
+"will disable skipping."
+msgstr ""
+
+#: en/help_topics/configuration.txt:640
+msgid ""
+"External Merge Tools\n"
+"--------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:643
+msgid ""
+"bzr.mergetool.<name>\n"
+"~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:646
+msgid ""
+"Defines an external merge tool called <name> with the given command-line.\n"
+"Arguments containing spaces should be quoted using single or double quotes. "
+"The\n"
+"executable may omit its path if it can be found on the PATH."
+msgstr ""
+
+#: en/help_topics/configuration.txt:650
+msgid ""
+"The following markers can be used in the command-line to substitute "
+"filenames\n"
+"involved in the merge conflict::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:653
+msgid ""
+"  {base}      file.BASE\n"
+"  {this}      file.THIS\n"
+"  {other}     file.OTHER\n"
+"  {result}    output file\n"
+"  {this_temp} temp copy of file.THIS, used to overwrite output file if "
+"merge\n"
+"              succeeds."
+msgstr ""
+
+#: en/help_topics/configuration.txt:662
+msgid "  bzr.mergetool.kdiff3 = kdiff3 {base} {this} {other} -o {result}"
+msgstr ""
+
+#: en/help_topics/configuration.txt:664
+msgid ""
+"bzr.default_mergetool\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:667
+msgid ""
+"Specifies which external merge tool (as defined above) should be selected "
+"by\n"
+"default in tools such as ``bzr qconflicts``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:672
+msgid "  bzr.default_mergetool = kdiff3\n"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:1
+msgid ""
+"Conflict Types\n"
+"=============="
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:4
+msgid ""
+"Some operations, like merge, revert and pull, modify the contents of your\n"
+"working tree.  These modifications are programmatically generated, and so "
+"they\n"
+"may conflict with the current state of your working tree. "
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:8
+msgid ""
+"When conflicts are present in your working tree (as shown by ``bzr\n"
+"conflicts``), you should resolve them and then inform bzr that the "
+"conflicts\n"
+"have been resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:12
+msgid ""
+"Resolving conflicts is sometimes not obvious. Either because the user that\n"
+"should resolve them is not the one responsible for their occurrence, as is "
+"the\n"
+"case when merging other people's work or because some conflicts are "
+"presented\n"
+"in a way that is not easy to understand."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:17
+msgid ""
+"Bazaar tries to avoid conflicts ; its aim is to ask you to resolve the\n"
+"conflict if and only if there's an actual conceptual conflict in the source\n"
+"tree.  Because Bazaar doesn't understand the real meaning of the files "
+"being\n"
+"versioned, it can, when faced with ambiguities, fall short in either "
+"direction\n"
+"trying to resolve the conflict itself. Many kinds of changes can be "
+"combined\n"
+"programmatically, but sometimes only a human can determine the right thing "
+"to\n"
+"do."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:25
+msgid ""
+"When Bazaar generates a conflict, it adds information into the working tree "
+"to\n"
+"present the conflicting versions, and it's up to you to find the correct\n"
+"resolution."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:29
+msgid "Whatever the conflict is, resolving it is roughly done in two steps:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:31
+msgid ""
+"1. Modify the working tree content so that the conflicted item is now in "
+"the\n"
+"   state you want to keep, then"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:34
+msgid ""
+"2. Inform Bazaar that the conflict is now solved and ask to cleanup any\n"
+"   remaining generated information (``bzr resolve <item>``)."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:37
+msgid ""
+"For most conflict types, there are some obvious ways to modify the working\n"
+"tree and put it into the desired state. For some types of conflicts, Bazaar\n"
+"itself already made a choice, when possible."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:41
+msgid ""
+"Yet, whether Bazaar makes a choice or not, there are some other simple but\n"
+"different ways to resolve the conflict."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:44
+msgid ""
+"Each type of conflict is explained below, and the action which must be done "
+"to\n"
+"resolve the conflict is outlined."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:47
+msgid ""
+"Various actions are available depending on the kind of conflict, for some "
+"of\n"
+"these actions, Bazaar can provide some help. In the end you should at least\n"
+"inform Bazaar that you're done with the conflict with::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:51
+msgid "  ``bzr resolve FILE --action=done'"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:53
+msgid ""
+"Note that this is the default action when a single file is involved so you "
+"can\n"
+"simply use::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:56
+msgid "   ``bzr resolve FILE``"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:58
+msgid "See ``bzr help resolve`` for more details."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:60
+msgid ""
+"Text conflicts\n"
+"--------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:63
+msgid "Typical message::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:65
+msgid "  Text conflict in FILE"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:67
+msgid ""
+"These are produced when a text merge cannot completely reconcile two sets "
+"of\n"
+"text changes.  Bazaar will emit files for each version with the extensions\n"
+"THIS, OTHER, and BASE.  THIS is the version of the file from the target "
+"tree,\n"
+"i.e. the tree that you are merging changes into.  OTHER is the version that "
+"you\n"
+"are merging into the target.  BASE is an older version that is used as a "
+"basis\n"
+"for comparison."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:74
+msgid ""
+"In the main copy of the file, Bazaar will include all the changes that it\n"
+"could reconcile, and any un-reconciled conflicts are surrounded by\n"
+"\"herringbone\" markers like ``<<<<<<<``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:78
+msgid ""
+"Say the initial text is \"The project leader released it.\", and THIS "
+"modifies it\n"
+"to \"Martin Pool released it.\", while OTHER modifies it to \"The project "
+"leader\n"
+"released Bazaar.\"  A conflict would look like this::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:82
+msgid ""
+"  <<<<<<< TREE\n"
+"  Martin Pool released it.\n"
+"  =======\n"
+"  The project leader released Bazaar.\n"
+"  >>>>>>> MERGE-SOURCE"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:88
+msgid "The correct resolution would be \"Martin Pool released Bazaar.\""
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:90
+msgid ""
+"You can handle text conflicts either by editing the main copy of the file,\n"
+"or by invoking external tools on the THIS, OTHER and BASE versions.  It's\n"
+"worth mentioning that resolving text conflicts rarely involves picking one\n"
+"set of changes over the other (but see below when you encounter these\n"
+"cases).  More often, the two sets of changes must be intelligently combined."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:96
+msgid ""
+"If you edit the main copy, be sure to remove the herringbone markers.  When\n"
+"you are done editing, the file should look like it never had a conflict, and "
+"be\n"
+"ready to commit."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:100
+msgid ""
+"When you have resolved text conflicts, just run ``bzr resolve --auto``, and\n"
+"Bazaar will auto-detect which conflicts you have resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:103
+msgid ""
+"When the conflict is resolved, Bazaar deletes the previously generated\n"
+"``.BASE``, ``.THIS`` and ``.OTHER`` files if they are still present in the\n"
+"working tree."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:107
+msgid ""
+"\n"
+"When you want to pick one set of changes over the other, you can use ``bzr\n"
+"resolve`` with one of the following actions:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:111
+msgid ""
+"* ``--action=take-this`` will issue ``mv FILE.THIS FILE``,\n"
+"* ``--action=take-other`` will issue ``mv FILE.OTHER FILE``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:114
+msgid ""
+"Note that if you have modified ``FILE.THIS`` or ``FILE.OTHER``, these\n"
+"modifications will be taken into account."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:117
+msgid ""
+"Content conflicts\n"
+"-----------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:122
+msgid "  Contents conflict in FILE"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:124
+msgid ""
+"This conflict happens when there are conflicting changes in the working "
+"tree\n"
+"and the merge source, but the conflicted items are not text files.  They "
+"may\n"
+"be binary files, or symlinks, or directories.  It can even happen with "
+"files\n"
+"that are deleted on one side, and modified on the other."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:129
+msgid ""
+"Like text conflicts, Bazaar will emit THIS, OTHER and BASE files.  (They may "
+"be\n"
+"regular files, symlinks or directories).  But it will not include a \"main "
+"copy\"\n"
+"of the file with herringbone conflict markers.  It will appear that the "
+"\"main\n"
+"copy\" has been renamed to THIS or OTHER."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:134
+msgid ""
+"To resolve that kind of conflict, you should rebuild FILE from either "
+"version\n"
+"or a combination of both."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:137
+msgid "``bzr resolve`` recognizes the following actions:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:139
+msgid ""
+"* ``--action=take-this`` will issue ``bzr mv FILE.THIS FILE``,\n"
+"* ``--action=take-other`` will issue ``bzr mv FILE.OTHER FILE``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:143
+msgid ""
+"Any action will also delete the previously generated ``.BASE``, ``.THIS`` "
+"and\n"
+"``.OTHER`` files if they are still present in the working tree."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:146
+msgid ""
+"Bazaar cannot auto-detect when conflicts of this kind have been resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:148
+msgid ""
+"Tag conflicts\n"
+"-------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:153
+msgid ""
+"  Conflicting tags:\n"
+"      version-0.1"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:156
+msgid ""
+"When pulling from or pushing to another branch, Bazaar informs you about "
+"tags\n"
+"that conflict between the two branches; that is the same tag points to two\n"
+"different revisions.  You need not resolve these conflicts, but subsequent\n"
+"uses of pull or push will result in the same message."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:161
+msgid ""
+"To resolve the conflict, you must apply the correct tags to either the "
+"target\n"
+"branch or the source branch as appropriate.  Use \"bzr tags --show-ids -d\n"
+"SOURCE_URL\" to see the tags in the source branch.  If you want to make the\n"
+"target branch's tags match the source branch, then in the target branch do\n"
+"``bzr tag --force -r revid:REVISION_ID CONFLICTING_TAG`` for each of the\n"
+"CONFLICTING_TAGs, where REVISION_ID comes from the list of tags in the "
+"source\n"
+"branch.  You need not call \"bzr resolve\" after doing this.  To resolve in\n"
+"favor of the target branch, you need to similarly use ``tag --force`` in "
+"the\n"
+"source branch.  (Note that pulling or pushing using --overwrite will "
+"overwrite\n"
+"all tags as well.)"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:172
+msgid ""
+"Duplicate paths\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:177
+msgid "  Conflict adding file FILE.  Moved existing file to FILE.moved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:179
+msgid ""
+"Sometimes Bazaar will attempt to create a file using a pathname that has\n"
+"already been used.  The existing file will be renamed to \"FILE.moved\".  "
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:187
+msgid ""
+"* ``--action=take-this`` will issue ``bzr rm FILE ; bzr mv FILE.moved "
+"FILE``,\n"
+"* ``--action=take-other`` will issue ``bzr rm FILE.moved``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:191
+msgid ""
+"Note that you must get rid of FILE.moved before using ``--action=done``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:195
+msgid ""
+"Unversioned parent\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:200
+msgid "    Conflict because FILE is not versioned, but has versioned children."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:202
+msgid ""
+"Sometimes Bazaar will attempt to create a file whose parent directory is "
+"not\n"
+"versioned.  This happens when the directory has been deleted in the target,\n"
+"but has a new child in the source, or vice versa.  In this situation, "
+"Bazaar\n"
+"will version the parent directory as well.  Resolving this issue depends\n"
+"very much on the particular scenario.  You may wish to rename or delete "
+"either\n"
+"the file or the directory.  When you are satisfied, you can run \"bzr "
+"resolve\n"
+"FILE\" to mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:210
+msgid ""
+"Missing parent\n"
+"--------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:215
+msgid "  Conflict adding files to FILE.  Created directory."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:217
+msgid ""
+"This happens when a directory has been deleted in the target, but has new\n"
+"children in the source.  This is similar to the \"unversioned parent\" "
+"conflict,\n"
+"except that the parent directory does not *exist*, instead of just being\n"
+"unversioned.  In this situation, Bazaar will create the missing parent.\n"
+"Resolving this issue depends very much on the particular scenario."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:223
+msgid ""
+"To resolve that kind of conflict, you should either remove or rename the\n"
+"children or the directory or a combination of both."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:228
+msgid ""
+"* ``--action=take-this`` will issue ``bzr rm directory`` including the\n"
+"  children,\n"
+"* ``--action=take-other`` will acknowledge Bazaar choice to keep the "
+"children\n"
+"  and restoring the directory,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:236
+msgid ""
+"Deleting parent\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:241
+msgid "  Conflict: can't delete DIR because it is not empty.  Not deleting."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:243
+msgid ""
+"This is the opposite of \"missing parent\".  A directory is deleted in the\n"
+"source, but has new children in the target (either because a directory\n"
+"deletion is merged or because the merge introduce new children).  Bazaar\n"
+"will retain the directory.  Resolving this issue depends very much on the\n"
+"particular scenario."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:254
+msgid ""
+"* ``--action=take-this`` will acknowledge Bazaar choice to keep the "
+"directory,"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:256
+msgid ""
+"* ``--action=take-other`` will issue ``bzr rm directory`` including the \n"
+"  children,"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:259
+msgid "* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:261
+msgid ""
+"Note that when merging a directory deletion, if unversioned files are\n"
+"present, they become potential orphans has they don't have a directory\n"
+"parent anymore."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:265
+msgid ""
+"Handling such orphans, *before* the conflict is created, is controlled by\n"
+"setting the ``bzr.transform.orphan_policy`` configuration option."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:268
+msgid "There are two possible values for this option:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:270
+msgid ""
+"* ``conflict`` (the default): will leave the orphans in place and\n"
+"  generate a conflicts,"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:273
+msgid ""
+"* ``move``: will move the orphans to a ``bzr-orphans`` directory at the "
+"root\n"
+"  of the working tree with names like ``<file>.~#~``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:278
+msgid ""
+"Path conflict\n"
+"-------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:283
+msgid "  Path conflict: PATH1 / PATH2"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:285
+msgid ""
+"This happens when the source and target have each modified the name or "
+"parent\n"
+"directory of a file.  Bazaar will use the path elements from the source."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:288
+msgid ""
+"To resolve that kind of conflict, you just have to decide what name should "
+"be\n"
+"retained for the file involved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:293
+msgid ""
+"* ``--action=take-this`` will revert Bazaar choice and keep ``PATH1`` by\n"
+"  issuing ``bzr mv PATH2 PATH1``,\n"
+"* ``--action=take-other`` will acknowledge Bazaar choice of keeping "
+"``PATH2``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:300
+msgid ""
+"Parent loop\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:305
+msgid "  Conflict moving FILE into DIRECTORY.  Cancelled move."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:307
+msgid ""
+"This happens when the source and the target have each moved directories, so\n"
+"that, if the change could be applied, a directory would be contained by "
+"itself.\n"
+"For example::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:311
+msgid ""
+"  $ bzr init\n"
+"  $ bzr mkdir white\n"
+"  $ bzr mkdir black\n"
+"  $ bzr commit -m \"BASE\"\n"
+"  $ bzr branch . ../other\n"
+"  $ bzr mv white black\n"
+"  $ bzr commit -m \"THIS\"\n"
+"  $ bzr mv ../other/black ../other/white\n"
+"  $ bzr commit ../other -m \"OTHER\"\n"
+"  $ bzr merge ../other"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:322
+msgid ""
+"In this situation, Bazaar will cancel the move, and leave ``white`` in\n"
+"``black``.  To resolve that kind of conflict, you just have to decide what\n"
+"name should be retained for the directories involved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:328
+msgid ""
+"* ``--action=take-this`` will acknowledge Bazaar choice of leaving "
+"``white`` \n"
+"  in ``black``,\n"
+"* ``--action=take-other`` will revert Bazaar choice and move ``black`` in\n"
+"   ``white`` by issuing ``bzr mv black/white white ; bzr mv black white``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:336
+msgid ""
+"Non-directory parent\n"
+"--------------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:341
+msgid ""
+"  Conflict: foo.new is not a directory, but has files in it.\n"
+"  Created directory."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:344
+msgid ""
+"This happens when one side has added files to a directory, and the other "
+"side\n"
+"has changed the directory into a file or symlink.  For example::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:347
+msgid ""
+"  $ bzr init\n"
+"  $ bzr mkdir foo\n"
+"  $ bzr commit -m \"BASE\"\n"
+"  $ bzr branch . ../other\n"
+"  $ rmdir foo\n"
+"  $ touch foo\n"
+"  $ bzr commit -m \"THIS\"\n"
+"  $ bzr mkdir ../other/foo/bar\n"
+"  $ bzr commit ../other -m \"OTHER\"\n"
+"  $ bzr merge ../other"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:358
+msgid ""
+"To resolve that kind of conflict, you have to decide what name should be\n"
+"retained for the file, directory or symlink involved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:363
+msgid ""
+"* ``--action=take-this`` will issue ``bzr rm --force foo.new`` and \n"
+"  ``bzr add foo``,\n"
+"* ``--action=take-other`` will issue ``bzr rm --force foo`` and \n"
+"  ``bzr mv foo.new foo``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:371
+msgid ""
+"MalformedTransform\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:374
+msgid ""
+"It is possible (though very rare) for Bazaar to raise a MalformedTransform\n"
+"exception.  This means that Bazaar encountered a filesystem conflict that it "
+"was\n"
+"unable to resolve.  This usually indicates a bug.  Please let us know if "
+"you\n"
+"encounter this.  Our bug tracker is at https://launchpad.net/bzr/+bugs\n"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:1
+msgid ""
+"Content Filters\n"
+"==============="
+msgstr ""
+
+#: en/help_topics/content-filters.txt:4
+msgid ""
+"Content formats\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:7
+msgid ""
+"Bazaar's content filtering allows you to store files in a different\n"
+"format from the copy in your working tree.  This lets you, or your\n"
+"co-developers, use Windows development tools that expect CRLF files\n"
+"on projects that use other line-ending conventions. Among other things,\n"
+"content filters also let Unix developers more easily work on projects\n"
+"using Windows line-ending conventions, keyword expansion/compression,\n"
+"and trailing spaces on lines in text files to be implicitly stripped\n"
+"when committed."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:16
+msgid "To generalize, there are two content formats supported by Bazaar:"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:18
+msgid ""
+"* a canonical format - how files are stored internally\n"
+"* a convenience format - how files are created in a working tree."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:21
+msgid ""
+"\n"
+"Format conversion\n"
+"-----------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:25
+msgid ""
+"The conversion between these formats is done by content filters.\n"
+"A content filter has two parts:"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:28
+msgid ""
+"* a read converter - converts from convenience to canonical format\n"
+"* a write converter - converts from canonical to convenience format."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:31
+msgid ""
+"Many of these converters will provide *round-trip* conversion,\n"
+"i.e. applying the read converter followed by the write converter\n"
+"gives back the original content. However, others may provide an\n"
+"asymmetric conversion. For example, a read converter might strip\n"
+"trailing whitespace off lines in source code while the matching\n"
+"write converter might pass content through unchanged."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:38
+msgid ""
+"\n"
+"Enabling content filters\n"
+"------------------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:42
+msgid ""
+"Content filters are typically provided by plugins, so the first step\n"
+"in using them is to install the relevant plugins and read their\n"
+"documentation. Some plugins may be very specific about which files\n"
+"they filter, e.g. only files ending in ``.java`` or ``.php``.\n"
+"In other cases, the plugin may leave it in the user's hands to\n"
+"define which files are to be filtered. This is typically done\n"
+"using rule-based preferences. See ``bzr help rules`` for general\n"
+"information about defining these."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:51
+msgid ""
+"\n"
+"Impact on commands\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:55
+msgid ""
+"Read converters are only applied to commands that read content from\n"
+"a working tree, e.g. status, diff and commit. For example, ``bzr diff``\n"
+"will apply read converters to files in the working tree, then compare\n"
+"the results to the content last committed."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:60
+msgid ""
+"Write converters are only applied by commands that **create files in a\n"
+"working tree**, e.g. branch, checkout, update. If you wish to see the\n"
+"canonical format of a file or tree, use ``bzr cat`` or ``bzr export``\n"
+"respectively."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:65
+msgid ""
+"Note: ``bzr commit`` does not implicitly apply write converters after\n"
+"comitting files. If this makes sense for a given plugin providing\n"
+"a content filter, the plugin can usually achieve this effect by using a\n"
+"``start_commit`` or ``post_commit`` hook say. See :doc:`hooks-help`\n"
+"for more information on hooks."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:71
+msgid ""
+"\n"
+"Refreshing your working tree\n"
+"----------------------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:75
+msgid ""
+"For performance reasons, Bazaar caches the timestamps of files in\n"
+"a working tree, and assumes files are unchanged if their timestamps\n"
+"match the cached values. As a consequence, there are times when\n"
+"you may need to explicitly ask for content filtering to be reapplied\n"
+"in one or both directions, e.g. after installing or reconfiguring\n"
+"plugins providing it."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:82
+msgid "Here are some general guidelines for doing this:"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:84
+msgid ""
+" * To reapply read converters, ``touch`` files, i.e. update their\n"
+"   timestamp. Operations like ``bzr status`` should then reapply the\n"
+"   relevant read converters and compare the end result with the\n"
+"   canonical format."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:89
+msgid ""
+" * To reapply write converters, ensure there are no local changes,\n"
+"   delete the relevant files and run ``bzr revert`` on those files."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:92
+msgid ""
+"Note: In the future, it is likely that additional options will be added\n"
+"to commands to make this refreshing process faster and safer.\n"
+msgstr ""
+
+#: en/help_topics/current-formats.txt:1
+msgid "Current Storage Formats"
+msgstr ""
+
+#: en/help_topics/current-formats.txt:3
+msgid ""
+":2a:\n"
+"    (native) (default) First format for bzr 2.0 series. Uses group-\n"
+"    compress storage. Provides rich roots which are a one-way\n"
+"    transition."
+msgstr ""
+
+#: en/help_topics/current-formats.txt:8
+msgid ""
+":pack-0.92:\n"
+"    (native) New in 0.92: Pack-based format with data compatible with\n"
+"    dirstate-tags format repositories. Interoperates with bzr\n"
+"    repositories before 0.92 but cannot be read by bzr < 0.92."
+msgstr ""
+
+#: en/help_topics/current-formats.txt:13
+msgid ""
+":1.14:\n"
+"    (native) A working-tree format that supports content filtering."
+msgstr ""
+
+#: en/help_topics/current-formats.txt:16
+msgid ""
+":1.14-rich-root:\n"
+"    (native) A variant of 1.14 that supports rich-root data (needed\n"
+"    for bzr-svn and bzr-git)."
+msgstr ""
+
+#: en/help_topics/current-formats.txt:20
+msgid ""
+"\n"
+"See :doc:`formats-help` for more about storage formats."
+msgstr ""
+
+#: en/help_topics/debug-flags.txt:1
+msgid ""
+"Debug Flags\n"
+"==========="
+msgstr ""
+
+#: en/help_topics/debug-flags.txt:4
+msgid ""
+"These flags can be passed on the bzr command line or (without the ``-D``\n"
+"prefix) put in the ``debug_flags`` variable in ``bazaar.conf``."
+msgstr ""
+
+#: en/help_topics/debug-flags.txt:7
+msgid ""
+"-Dauth            Trace authentication sections used.\n"
+"-Dbytes           Print out how many bytes were transferred\n"
+"-Ddirstate        Trace dirstate activity (verbose!)\n"
+"-Derror           Instead of normal error handling, always print a "
+"traceback\n"
+"                  on error.\n"
+"-Devil            Capture call sites that do expensive or badly-scaling\n"
+"                  operations.\n"
+"-Dfetch           Trace history copying between repositories.\n"
+"-Dfilters         Emit information for debugging content filtering.\n"
+"-Dforceinvdeltas  Force use of inventory deltas during generic streaming "
+"fetch.\n"
+"-Dgraph           Trace graph traversal.\n"
+"-Dhashcache       Log every time a working file is read to determine its "
+"hash.\n"
+"-Dhooks           Trace hook execution.\n"
+"-Dhpss            Trace smart protocol requests and responses.\n"
+"-Dhpssdetail      More hpss details.\n"
+"-Dhpssvfs         Traceback on vfs access to Remote objects.\n"
+"-Dhttp            Trace http connections, requests and responses.\n"
+"-Dindex           Trace major index operations.\n"
+"-Dknit            Trace knit operations.\n"
+"-Dlock            Trace when lockdir locks are taken or released.\n"
+"-Dprogress        Trace progress bar operations.\n"
+"-Dmem_dump        Dump memory to a file upon an out of memory error.\n"
+"-Dmerge           Emit information for debugging merges.\n"
+"-Dno_apport       Don't use apport to report crashes.\n"
+"-Dno_activity \t  Don't show transport activity indicator in progress bar.\n"
+"-Dpack            Emit information about pack operations.\n"
+"-Drelock          Emit a message every time a branch or repository object "
+"is\n"
+"                  unlocked then relocked the same way.\n"
+"-Dsftp            Trace SFTP internals.\n"
+"-Dstatic_tuple    Error when a tuple is used where a StaticTuple is "
+"expected\n"
+"-Dstream          Trace fetch streams.\n"
+"-Dstrict_locks    Trace when OS locks are potentially used in a non-"
+"portable\n"
+"                  manner.\n"
+"-Dunlock          Some errors during unlock are treated as warnings.\n"
+"-DIDS_never       Never use InterDifferingSerializer when fetching.\n"
+"-DIDS_always      Always use InterDifferingSerializer to fetch if "
+"appropriate\n"
+"                  for the format, even for non-local fetches.\n"
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:1
+msgid ""
+"Diverged Branches\n"
+"================="
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:4
+msgid ""
+"When Bazaar tries to push one branch onto another, it requires that the\n"
+"destination branch must be ready to receive the source branch.  If this "
+"isn't\n"
+"the case, then we say that the branches have ``diverged``.  Branches are\n"
+"considered diverged if the destination branch's most recent commit is one "
+"that\n"
+"has not been merged (directly or indirectly) by the source branch.  To "
+"recover\n"
+"from diverged branches, one must merge the missing revisions into the "
+"source\n"
+"branch."
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:12
+msgid ""
+"This situation commonly arises when using a centralized workflow with local\n"
+"commits.  If someone else has committed new work to the mainline since your\n"
+"last pull and you have local commits that have not yet been pushed to the\n"
+"mainline, then your local branch and the mainline have diverged."
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:17
+msgid ""
+"Discovering What Has Diverged\n"
+"-----------------------------"
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:20
+msgid ""
+"The ``bzr missing`` command is used to find out what revisions are in "
+"another\n"
+"branch that are not present in the current branch, and vice-versa.  It shows "
+"a\n"
+"summary of  which extra revisions exist in each branch.  If you want to see "
+"the\n"
+"precise effects of those revisions, you can use ``bzr diff --"
+"old=other_branch``\n"
+"to show the differences between other_branch and your current branch."
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:26
+msgid ""
+"A Solution\n"
+"----------"
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:29
+msgid ""
+"The solution is to merge the revisions from the mainline into your local\n"
+"branch.  To do so, use ``bzr merge`` to get the new revisions from the\n"
+"mainline.  This merge may result in conflicts if the other developer's "
+"changes\n"
+"overlap with your changes.  These conflicts should be resolved before\n"
+"continuing. After any conflicts have been resolved, or even if there were "
+"no\n"
+"conflicts, Bazaar requires that you explicitly commit these new revisions\n"
+"to your local branch.  This requirement gives you an opportunity to test "
+"the\n"
+"resulting working tree for correctness, since the merged revisions could "
+"have\n"
+"made arbitrary changes.  After testing, you should commit the merge using\n"
+"``bzr commit``.  This clears up the diverged branches situation.  Your "
+"local\n"
+"branch can now be pushed to the mainline.\n"
+msgstr ""
+
+#: en/help_topics/eol.txt:1
+msgid ""
+"End of Line Conversion\n"
+"======================"
+msgstr ""
+
+#: en/help_topics/eol.txt:4
+msgid ""
+"EOL conversion is provided as a content filter where Bazaar internally\n"
+"stores a canonical format but outputs a convenience format. See\n"
+"``bzr help content-filters`` for general information about using these."
+msgstr ""
+
+#: en/help_topics/eol.txt:8
+msgid ""
+"Note: Content filtering is only supported in recently added formats,\n"
+"e.g. 1.14. Be sure that both the repository *and* the branch are\n"
+"in a recent format. (Just setting the format on the repository\n"
+"is not enough.) If content filtering does not appear to be working, use\n"
+"'bzr info -v' to confirm that the branch is using \"Working tree format 5\"\n"
+"or later."
+msgstr ""
+
+#: en/help_topics/eol.txt:15
+msgid ""
+"EOL conversion needs to be enabled for selected file patterns using\n"
+"rules. See ``bzr help rules`` for general information on defining rules.\n"
+"Currently, rules are only supported in $BZR_HOME/.bazaar/rules (or\n"
+"%BZR_HOME%/bazaar/2.0/rules on Windows). Branch specific rules will be\n"
+"supported in a future verison of Bazaar."
+msgstr ""
+
+#: en/help_topics/eol.txt:21
+msgid ""
+"To configure which files to filter, set ``eol`` to one of the values below.\n"
+"(If a value is not set, ``exact`` is the default.)"
+msgstr ""
+
+#: en/help_topics/eol.txt:24
+msgid ""
+" ========== ===================================== ======================\n"
+" Value      Checkout end-of-lines as              Commit end-of-lines as\n"
+" ========== ===================================== ======================\n"
+" ``native`` ``crlf`` on Windows, ``lf`` otherwise ``lf``\n"
+" ---------- ------------------------------------- ----------------------\n"
+" ``lf``     ``lf``                                ``lf``\n"
+" ---------- ------------------------------------- ----------------------\n"
+" ``crlf``   ``crlf``                              ``lf``\n"
+" ---------- ------------------------------------- ----------------------\n"
+" ``exact``  No conversion                         Exactly as in file\n"
+" ========== ===================================== ======================"
+msgstr ""
+
+#: en/help_topics/eol.txt:36
+msgid ""
+"Note: For safety reasons, no conversion is applied to any file where a null\n"
+"byte is detected in the file."
+msgstr ""
+
+#: en/help_topics/eol.txt:39
+msgid ""
+"For users working on a cross-platform project, here is a suggested rule\n"
+"to use as a starting point::"
+msgstr ""
+
+#: en/help_topics/eol.txt:42
+msgid ""
+"  [name *]\n"
+"  eol = native"
+msgstr ""
+
+#: en/help_topics/eol.txt:45
+msgid ""
+"If you have binary files that do not contain a null byte though, be\n"
+"sure to add ``eol = exact`` rules for those as well. You can do this\n"
+"by giving more explicit patterns earlier in the rules file. For example::"
+msgstr ""
+
+#: en/help_topics/eol.txt:49
+msgid ""
+"  [name *.png]\n"
+"  eol = exact"
+msgstr ""
+
+#: en/help_topics/eol.txt:55
+msgid ""
+"If your working tree is on a network drive shared by users on different\n"
+"operating systems, you typically want to force certain conventions for\n"
+"certain files. In that way, if a file is created with the wrong line\n"
+"endings or line endings get mixed during editing, it gets committed\n"
+"correctly and gets checked out correctly. For example::"
+msgstr ""
+
+#: en/help_topics/eol.txt:61
+msgid ""
+"  [name *.bat]\n"
+"  eol = crlf"
+msgstr ""
+
+#: en/help_topics/eol.txt:64
+msgid ""
+"  [name *.sh]\n"
+"  eol = lf"
+msgstr ""
+
+#: en/help_topics/eol.txt:70
+msgid ""
+"If you take the care to create files with their required endings, you can\n"
+"achieve *almost* the same thing by using ``eol = exact``. It is slightly\n"
+"safer to use ``lf`` and ``crlf`` though because edits accidentally\n"
+"introducing mixed line endings will be corrected during commit for files\n"
+"with those settings."
+msgstr ""
+
+#: en/help_topics/eol.txt:76
+msgid ""
+"If you have sample test data that deliberately has text files with mixed\n"
+"newline conventions, you can ask for those to be left alone like this::"
+msgstr ""
+
+#: en/help_topics/eol.txt:79
+msgid ""
+"  [name test_data/]\n"
+"  eol = exact"
+msgstr ""
+
+#: en/help_topics/eol.txt:85
+msgid ""
+"Note that ``exact`` does not imply the file is binary but it does mean\n"
+"that no conversion of end-of-lines will be done. (Bazaar currently relies\n"
+"of content analysis to detect binary files for commands like ``diff``.\n"
+"In the future, a ``binary = true`` rule may be added but it is not\n"
+"supported yet.)"
+msgstr ""
+
+#: en/help_topics/eol.txt:91
+msgid ""
+"If you have an existing repository with text files already stored using\n"
+"Windows newline conventions (``crlf``), then you may want to keep using "
+"that\n"
+"convention in the repository. Forcing certain files to this convention\n"
+"may also help users who do not have rules configured. To do this, set\n"
+"``eol`` to one of the values below."
+msgstr ""
+
+#: en/help_topics/eol.txt:97
+msgid ""
+" ============================ ======================== "
+"======================\n"
+" Value                        Checkout end-of-lines as Commit end-of-lines "
+"as\n"
+" ============================ ======================== "
+"======================\n"
+" ``native-with-crlf-in-repo`` ``crlf`` on Windows,     ``crlf``\n"
+"                              ``lf`` otherwise\n"
+" ---------------------------- ------------------------ "
+"----------------------\n"
+" ``lf-with-crlf-in-repo``     ``lf``                   ``crlf``\n"
+" ---------------------------- ------------------------ "
+"----------------------\n"
+" ``crlf-with-crlf-in-repo``   ``crlf``                 ``crlf``\n"
+" ============================ ======================== ======================"
+msgstr ""
+
+#: en/help_topics/eol.txt:108
+msgid ""
+"For users working on an existing project that uses Windows newline\n"
+"conventions in their Bazaar repository, this rule is suggested as a\n"
+"starting point::"
+msgstr ""
+
+#: en/help_topics/eol.txt:112
+msgid ""
+"  [name *]\n"
+"  eol = native-with-crlf-in-repo"
+msgstr ""
+
+#: en/help_topics/eol.txt:115
+msgid ""
+"For new projects, it is recommended that end-of-lines be stored as ``lf``\n"
+"and that users stick to the basic settings, i.e. ``native``, ``lf``,\n"
+"``crlf`` and ``exact``."
+msgstr ""
+
+#: en/help_topics/eol.txt:119
+msgid ""
+"Note: Bazaar's EOL conversion will convert the content of files but\n"
+"never reject files because a given line ending or mixed line endings\n"
+"are found. A precommit hook should be used if you wish to validate\n"
+"(and not just convert) content before committing.\n"
+msgstr ""
+
+#: en/help_topics/hidden-commands.txt:1
+msgid ""
+"added                  List files added in working tree.\n"
+"ancestry               List all revisions merged into this branch.\n"
+"assert-fail            Test reporting of assertion failures\n"
+"bundle-info            Output interesting stats about a bundle\n"
+"bundle-revisions       Create a merge-directive for submitting changes.\n"
+"cat-revision           Write out metadata for a revision.\n"
+"dpush                  Push into a different VCS without any custom bzr\n"
+"                       metadata.\n"
+"dump-btree             Dump the contents of a btree index file to stdout.\n"
+"export-pot             Export command helps and error messages in po "
+"format.\n"
+"file-id                Print file_id of a particular file or directory.\n"
+"file-path              Print path of file_ids to a file or directory.\n"
+"find-merge-base        Find and print a base revision for merging two "
+"branches.\n"
+"hooks                  Show hooks.\n"
+"inventory              Show inventory of the current working copy or a\n"
+"                       revision.\n"
+"is-ignored             Check if a path is ignored. [qbzr]\n"
+"is-versioned           Check if a path is versioned. [qbzr]\n"
+"local-time-offset      Show the offset in seconds from GMT to local time.\n"
+"lookup-revision        Lookup the revision-id from a revision-number\n"
+"merge-directive        Generate a merge directive for auto-merge tools.\n"
+"modified               List files modified in working tree.\n"
+"qcheckout-ala-explorer Checkout a branch. The checkout is always "
+"lightweight.\n"
+"                       [explorer]\n"
+"qhelp                  Shows a help window [qbzr]\n"
+"qmain                  The QBzr main application. [qbzr]\n"
+"qsubprocess            Run some bzr command as subprocess. [qbzr]\n"
+"re-sign                Create a digital signature for an existing revision.\n"
+"reference              list, view and set branch locations for nested "
+"trees.\n"
+"relpath                Show path of a file relative to root\n"
+"repair-workingtree     Reset the working tree state file.\n"
+"revision-history       Display the list of revision ids on a branch.\n"
+"revision-info          Show revision number and revision id for a given\n"
+"                       revision identifier.\n"
+"rocks                  Statement of optimism.\n"
+"selftest               Run internal test suite.\n"
+"shell-complete         Show appropriate completions for context.\n"
+"test-script            Run a shell-like test from a file.\n"
+"touching-revisions     Return revision-ids which affected a particular "
+"file.\n"
+"unknowns               List unknown files.\n"
+"wait-until-signalled   Test helper for\n"
+"                       test_start_and_stop_bzr_subprocess_send_signal.\n"
+msgstr ""
+
+#: en/help_topics/hooks.txt:1
+msgid ""
+"\n"
+"Hooks\n"
+"====="
+msgstr ""
+
+#: en/help_topics/hooks.txt:5
+msgid ""
+"Introduction\n"
+"------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:8
+msgid "A hook of type *xxx* of class *yyy* needs to be registered using::"
+msgstr ""
+
+#: en/help_topics/hooks.txt:10
+msgid "  yyy.hooks.install_named_hook(\"xxx\", ...)"
+msgstr ""
+
+#: en/help_topics/hooks.txt:12
+msgid ""
+"See :doc:`Using hooks<../user-guide/hooks>` in the User Guide for examples."
+msgstr ""
+
+#: en/help_topics/hooks.txt:14
+msgid ""
+"The class that contains each hook is given before the hooks it supplies. "
+"For\n"
+"instance, BranchHooks as the class is the hooks class for\n"
+"`bzrlib.branch.Branch.hooks`."
+msgstr ""
+
+#: en/help_topics/hooks.txt:18
+msgid ""
+"Each description also indicates whether the hook runs on the client (the\n"
+"machine where bzr was invoked) or the server (the machine addressed by\n"
+"the branch URL).  These may be, but are not necessarily, the same machine."
+msgstr ""
+
+#: en/help_topics/hooks.txt:22
+msgid ""
+"Plugins (including hooks) are run on the server if all of these is true:"
+msgstr ""
+
+#: en/help_topics/hooks.txt:24
+msgid ""
+"  * The connection is via a smart server (accessed with a URL starting with\n"
+"    \"bzr://\", \"bzr+ssh://\" or \"bzr+http://\", or accessed via a "
+"\"http://\"\n"
+"    URL when a smart server is available via HTTP)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:28
+msgid ""
+"  * The hook is either server specific or part of general infrastructure "
+"rather\n"
+"    than client specific code (such as commit)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:31
+msgid ""
+"\n"
+"BranchHooks\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:35
+msgid ""
+"automatic_tag_name\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:38
+msgid "Introduced in: 2.2"
+msgstr ""
+
+#: en/help_topics/hooks.txt:40
+msgid ""
+"Called to determine an automatic tag name for a revision.\n"
+"automatic_tag_name is called with (branch, revision_id) and should\n"
+"return a tag name or None if no tag name could be determined. The\n"
+"first non-None tag name returned will be used."
+msgstr ""
+
+#: en/help_topics/hooks.txt:45
+msgid ""
+"open\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:48
+msgid "Introduced in: 1.8"
+msgstr ""
+
+#: en/help_topics/hooks.txt:50
+msgid ""
+"Called with the Branch object that has been opened after a branch is\n"
+"opened."
+msgstr ""
+
+#: en/help_topics/hooks.txt:53
+msgid ""
+"post_branch_init\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:58
+msgid ""
+"Called after new branch initialization completes. post_branch_init is\n"
+"called with a bzrlib.branch.BranchInitHookParams. Note that init,\n"
+"branch and checkout (both heavyweight and lightweight) will all\n"
+"trigger this hook."
+msgstr ""
+
+#: en/help_topics/hooks.txt:63
+msgid ""
+"post_change_branch_tip\n"
+"~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:66
+msgid "Introduced in: 1.4"
+msgstr ""
+
+#: en/help_topics/hooks.txt:68
+msgid ""
+"Called in bzr client and server after a change to the tip of a branch\n"
+"is made. post_change_branch_tip is called with a\n"
+"bzrlib.branch.ChangeBranchTipParams. Note that push, pull, commit,\n"
+"uncommit will all trigger this hook."
+msgstr ""
+
+#: en/help_topics/hooks.txt:73
+msgid ""
+"post_commit\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:76
+msgid "Introduced in: 0.15"
+msgstr ""
+
+#: en/help_topics/hooks.txt:78
+msgid ""
+"Called in the bzr client after a commit has completed. post_commit is\n"
+"called with (local, master, old_revno, old_revid, new_revno,\n"
+"new_revid). old_revid is NULL_REVISION for the first commit to a\n"
+"branch."
+msgstr ""
+
+#: en/help_topics/hooks.txt:83
+msgid ""
+"post_pull\n"
+"~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:88
+msgid ""
+"Called after a pull operation completes. post_pull is called with a\n"
+"bzrlib.branch.PullResult object and only runs in the bzr client."
+msgstr ""
+
+#: en/help_topics/hooks.txt:91
+msgid ""
+"post_push\n"
+"~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:96
+msgid ""
+"Called after a push operation completes. post_push is called with a\n"
+"bzrlib.branch.BranchPushResult object and only runs in the bzr client."
+msgstr ""
+
+#: en/help_topics/hooks.txt:99
+msgid ""
+"post_switch\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:104
+msgid ""
+"Called after a checkout switches branch. post_switch is called with a\n"
+"bzrlib.branch.SwitchHookParams."
+msgstr ""
+
+#: en/help_topics/hooks.txt:107
+msgid ""
+"post_uncommit\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:112
+msgid ""
+"Called in the bzr client after an uncommit completes. post_uncommit is\n"
+"called with (local, master, old_revno, old_revid, new_revno,\n"
+"new_revid) where local is the local branch or None, master is the\n"
+"target branch, and an empty branch receives new_revno of 0, new_revid\n"
+"of None."
+msgstr ""
+
+#: en/help_topics/hooks.txt:118
+msgid ""
+"pre_change_branch_tip\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:121
+msgid "Introduced in: 1.6"
+msgstr ""
+
+#: en/help_topics/hooks.txt:123
+msgid ""
+"Called in bzr client and server before a change to the tip of a branch\n"
+"is made. pre_change_branch_tip is called with a\n"
+"bzrlib.branch.ChangeBranchTipParams. Note that push, pull, commit,\n"
+"uncommit will all trigger this hook."
+msgstr ""
+
+#: en/help_topics/hooks.txt:128
+msgid ""
+"pre_commit\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:131
+msgid "Introduced in: 0.91"
+msgstr ""
+
+#: en/help_topics/hooks.txt:133
+msgid ""
+"Called after a commit is calculated but before it is completed.\n"
+"pre_commit is called with (local, master, old_revno, old_revid,\n"
+"future_revno, future_revid, tree_delta, future_tree). old_revid is\n"
+"NULL_REVISION for the first commit to a branch, tree_delta is a\n"
+"TreeDelta object describing changes from the basis revision. hooks\n"
+"MUST NOT modify this delta.  future_tree is an in-memory tree obtained\n"
+"from CommitBuilder.revision_tree() and hooks MUST NOT modify this\n"
+"tree."
+msgstr ""
+
+#: en/help_topics/hooks.txt:142
+msgid ""
+"set_rh\n"
+"~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:147
+msgid ""
+"Invoked whenever the revision history has been set via\n"
+"set_revision_history. The api signature is (branch, revision_history),\n"
+"and the branch will be write-locked. The set_rh hook can be expensive\n"
+"for bzr to trigger, a better hook to use is\n"
+"Branch.post_change_branch_tip."
+msgstr ""
+
+#: en/help_topics/hooks.txt:153
+msgid ""
+"transform_fallback_location\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:156
+msgid "Introduced in: 1.9"
+msgstr ""
+
+#: en/help_topics/hooks.txt:158
+msgid ""
+"Called when a stacked branch is activating its fallback locations.\n"
+"transform_fallback_location is called with (branch, url), and should\n"
+"return a new url. Returning the same url allows it to be used as-is,\n"
+"returning a different one can be used to cause the branch to stack on\n"
+"a closer copy of that fallback_location. Note that the branch cannot\n"
+"have history accessing methods called on it during this hook because\n"
+"the fallback locations have not been activated. When there are\n"
+"multiple hooks installed for transform_fallback_location, all are\n"
+"called with the url returned from the previous hook.The order is\n"
+"however undefined."
+msgstr ""
+
+#: en/help_topics/hooks.txt:169
+msgid ""
+"BzrDirHooks\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:172
+msgid ""
+"post_repo_init\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:177
+msgid ""
+"Invoked after a repository has been initialized. post_repo_init is\n"
+"called with a bzrlib.bzrdir.RepoInitHookParams."
+msgstr ""
+
+#: en/help_topics/hooks.txt:180
+msgid ""
+"pre_open\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:183
+msgid "Introduced in: 1.14"
+msgstr ""
+
+#: en/help_topics/hooks.txt:185
+msgid ""
+"Invoked before attempting to open a BzrDir with the transport that the\n"
+"open will use."
+msgstr ""
+
+#: en/help_topics/hooks.txt:188
+msgid ""
+"CommandHooks\n"
+"------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:191
+msgid ""
+"extend_command\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:194
+msgid "Introduced in: 1.13"
+msgstr ""
+
+#: en/help_topics/hooks.txt:196
+msgid ""
+"Called after creating a command object to allow modifications such as\n"
+"adding or removing options, docs etc. Called with the new\n"
+"bzrlib.commands.Command object."
+msgstr ""
+
+#: en/help_topics/hooks.txt:200
+msgid ""
+"get_command\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:203
+msgid "Introduced in: 1.17"
+msgstr ""
+
+#: en/help_topics/hooks.txt:205
+msgid ""
+"Called when creating a single command. Called with (cmd_or_None,\n"
+"command_name). get_command should either return the cmd_or_None\n"
+"parameter, or a replacement Command object that should be used for the\n"
+"command. Note that the Command.hooks hooks are core infrastructure.\n"
+"Many users will prefer to use bzrlib.commands.register_command or\n"
+"plugin_cmds.register_lazy."
+msgstr ""
+
+#: en/help_topics/hooks.txt:212
+msgid ""
+"get_missing_command\n"
+"~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:217
+msgid ""
+"Called when creating a single command if no command could be found.\n"
+"Called with (command_name). get_missing_command should either return\n"
+"None, or a Command object to be used for the command."
+msgstr ""
+
+#: en/help_topics/hooks.txt:221
+msgid ""
+"list_commands\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:226
+msgid ""
+"Called when enumerating commands. Called with a set of cmd_name\n"
+"strings for all the commands found so far. This set  is safe to mutate\n"
+"- e.g. to remove a command. list_commands should return the updated\n"
+"set of command names."
+msgstr ""
+
+#: en/help_topics/hooks.txt:231
+msgid ""
+"_ConfigHooks\n"
+"------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:234
+msgid ""
+"get\n"
+"~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:237
+msgid "Introduced in: 2.4"
+msgstr ""
+
+#: en/help_topics/hooks.txt:239
+msgid ""
+"Invoked when a config option is read. The signature is (stack, name,\n"
+"value)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:242
+msgid ""
+"load\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:247
+msgid "Invoked when a config store is loaded. The signature is (store)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:249
+msgid ""
+"remove\n"
+"~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:254
+msgid ""
+"Invoked when a config option is removed. The signature is (stack,\n"
+"name)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:257
+msgid ""
+"save\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:262
+msgid "Invoked when a config store is saved. The signature is (store)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:264
+msgid ""
+"set\n"
+"~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:269
+msgid ""
+"Invoked when a config option is set. The signature is (stack, name,\n"
+"value)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:272
+msgid ""
+"InfoHooks\n"
+"---------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:275
+msgid ""
+"repository\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:278
+msgid "Introduced in: 1.15"
+msgstr ""
+
+#: en/help_topics/hooks.txt:280
+msgid ""
+"Invoked when displaying the statistics for a repository. repository is\n"
+"called with a statistics dictionary as returned by the repository and\n"
+"a file-like object to write to."
+msgstr ""
+
+#: en/help_topics/hooks.txt:284
+msgid ""
+"LockHooks\n"
+"---------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:287
+msgid ""
+"lock_acquired\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:292
+msgid "Called with a bzrlib.lock.LockResult when a physical lock is acquired."
+msgstr ""
+
+#: en/help_topics/hooks.txt:294
+msgid ""
+"lock_broken\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:299
+msgid "Called with a bzrlib.lock.LockResult when a physical lock is broken."
+msgstr ""
+
+#: en/help_topics/hooks.txt:301
+msgid ""
+"lock_released\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:306
+msgid "Called with a bzrlib.lock.LockResult when a physical lock is released."
+msgstr ""
+
+#: en/help_topics/hooks.txt:308
+msgid ""
+"MergeHooks\n"
+"----------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:311
+msgid ""
+"merge_file_content\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:314
+msgid "Introduced in: 2.1"
+msgstr ""
+
+#: en/help_topics/hooks.txt:316
+msgid ""
+"Called with a bzrlib.merge.Merger object to create a per file merge\n"
+"object when starting a merge. Should return either None or a subclass\n"
+"of ``bzrlib.merge.AbstractPerFileMerger``. Such objects will then be\n"
+"called per file that needs to be merged (including when one side has\n"
+"deleted the file and the other has changed it). See the\n"
+"AbstractPerFileMerger API docs for details on how it is used by merge."
+msgstr ""
+
+#: en/help_topics/hooks.txt:323
+msgid ""
+"MergeDirectiveHooks\n"
+"-------------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:326
+msgid ""
+"merge_request_body\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:329
+msgid "Introduced in: 1.15.0"
+msgstr ""
+
+#: en/help_topics/hooks.txt:331
+msgid ""
+"Called with a MergeRequestBodyParams when a body is needed for a merge\n"
+"request.  Callbacks must return a body.  If more than one callback is\n"
+"registered, the output of one callback is provided to the next."
+msgstr ""
+
+#: en/help_topics/hooks.txt:335
+msgid ""
+"MessageEditorHooks\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:338
+msgid ""
+"commit_message_template\n"
+"~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:341
+msgid "Introduced in: 1.10"
+msgstr ""
+
+#: en/help_topics/hooks.txt:343
+msgid ""
+"Called when a commit message is being generated.\n"
+"commit_message_template is called with the bzrlib.commit.Commit object\n"
+"and the message that is known so far. commit_message_template must\n"
+"return a new message to use (which could be the same as it was given).\n"
+"When there are multiple hooks registered for commit_message_template,\n"
+"they are chained with the result from the first passed into the\n"
+"second, and so on."
+msgstr ""
+
+#: en/help_topics/hooks.txt:351
+msgid ""
+"set_commit_message\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:356
+msgid ""
+"Set a fixed commit message. set_commit_message is called with the\n"
+"bzrlib.commit.Commit object (so you can also change e.g. revision\n"
+"properties by editing commit.builder._revprops) and the message so\n"
+"far. set_commit_message must return the message to use or None if it\n"
+"should use the message editor as normal."
+msgstr ""
+
+#: en/help_topics/hooks.txt:362
+msgid ""
+"MutableTreeHooks\n"
+"----------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:368
+msgid "Introduced in: 2.0"
+msgstr ""
+
+#: en/help_topics/hooks.txt:370
+msgid ""
+"Called after a commit is performed on a tree. The hook is called with\n"
+"a bzrlib.mutabletree.PostCommitHookParams object. The mutable tree the\n"
+"commit was performed on is available via the mutable_tree attribute of\n"
+"that object."
+msgstr ""
+
+#: en/help_topics/hooks.txt:375
+msgid ""
+"start_commit\n"
+"~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:380
+msgid ""
+"Called before a commit is performed on a tree. The start commit hook\n"
+"is able to change the tree before the commit takes place. start_commit\n"
+"is called with the bzrlib.mutabletree.MutableTree that the commit is\n"
+"being performed on."
+msgstr ""
+
+#: en/help_topics/hooks.txt:385
+msgid ""
+"SmartClientHooks\n"
+"----------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:388
+msgid ""
+"call\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:391
+msgid "Introduced in: unknown"
+msgstr ""
+
+#: en/help_topics/hooks.txt:393
+msgid ""
+"Called when the smart client is submitting a request to the smart\n"
+"server. Called with a bzrlib.smart.client.CallHookParams object.\n"
+"Streaming request bodies, and responses, are not accessible."
+msgstr ""
+
+#: en/help_topics/hooks.txt:397
+msgid ""
+"SmartServerHooks\n"
+"----------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:400
+msgid ""
+"server_exception\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:405
+msgid ""
+"Called by the bzr server when an exception occurs. server_exception is\n"
+"called with the sys.exc_info() tuple return true for the hook if the\n"
+"exception has been handled, in which case the server will exit\n"
+"normally."
+msgstr ""
+
+#: en/help_topics/hooks.txt:410
+msgid ""
+"server_started\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:413
+msgid "Introduced in: 0.16"
+msgstr ""
+
+#: en/help_topics/hooks.txt:415
+msgid ""
+"Called by the bzr server when it starts serving a directory.\n"
+"server_started is called with (backing urls, public url), where\n"
+"backing_url is a list of URLs giving the server-specific directory\n"
+"locations, and public_url is the public URL for the directory being\n"
+"served."
+msgstr ""
+
+#: en/help_topics/hooks.txt:421
+msgid ""
+"server_started_ex\n"
+"~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:426
+msgid ""
+"Called by the bzr server when it starts serving a directory.\n"
+"server_started is called with (backing_urls, server_obj)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:429
+msgid ""
+"server_stopped\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:434
+msgid ""
+"Called by the bzr server when it stops serving a directory.\n"
+"server_stopped is called with the same parameters as the\n"
+"server_started hook: (backing_urls, public_url)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:438
+msgid ""
+"StatusHooks\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:441
+msgid ""
+"post_status\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:444
+msgid "Introduced in: 2.3"
+msgstr ""
+
+#: en/help_topics/hooks.txt:446
+msgid ""
+"Called with argument StatusHookParams after Bazaar has displayed the\n"
+"status. StatusHookParams has the attributes (old_tree, new_tree,\n"
+"to_file, versioned, show_ids, short, verbose). The last four arguments\n"
+"correspond to the command line options specified by the user for the\n"
+"status command. to_file is the output stream for writing."
+msgstr ""
+
+#: en/help_topics/hooks.txt:452
+msgid ""
+"pre_status\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:457
+msgid ""
+"Called with argument StatusHookParams before Bazaar displays the\n"
+"status. StatusHookParams has the attributes (old_tree, new_tree,\n"
+"to_file, versioned, show_ids, short, verbose). The last four arguments\n"
+"correspond to the command line options specified by the user for the\n"
+"status command. to_file is the output stream for writing."
+msgstr ""
+
+#: en/help_topics/hooks.txt:463
+msgid ""
+"RioVersionInfoBuilderHooks\n"
+"--------------------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:466
+msgid ""
+"revision\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:471
+msgid ""
+"Invoked when adding information about a revision to the RIO stanza\n"
+"that is printed. revision is called with a revision object and a RIO\n"
+"stanza.\n"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:1
+msgid ""
+"Location aliases\n"
+"================"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:4
+msgid ""
+"Bazaar defines several aliases for locations associated with a branch.  "
+"These\n"
+"can be used with most commands that expect a location, such as `bzr push`."
+msgstr ""
+
+#: en/help_topics/location-alias.txt:7
+msgid "The aliases are::"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:9
+msgid ""
+"  :parent    the parent of this branch\n"
+"  :submit    the submit branch for this branch\n"
+"  :public    the public location of this branch\n"
+"  :bound     the branch this branch is bound to, for bound branches\n"
+"  :push      the saved location used for `bzr push` with no arguments\n"
+"  :this      this branch"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:16
+msgid "For example, to push to the parent location::"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:18
+msgid "    bzr push :parent"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:1
+msgid ""
+"Log Formats\n"
+"==========="
+msgstr ""
+
+#: en/help_topics/log-formats.txt:4
+msgid ""
+"A log format controls how information about each revision is displayed.\n"
+"The standard log formats are compared below::"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:7
+msgid ""
+"  Feature                 long           short         line\n"
+"  ----------------------  -------------  ------------  -------------------\n"
+"  design goal             detailed view  concise view  1 revision per line\n"
+"  committer               name+email     name only     name only\n"
+"  author                  name+email     -             -\n"
+"  date-time format        full           date only     date only\n"
+"  commit message          full           full          top line\n"
+"  tags                    yes            yes           yes\n"
+"  merges indicator        -              yes           -\n"
+"  status/delta            optional       optional      -\n"
+"  diff/patch              optional       optional      -\n"
+"  revision-id             optional       optional      -\n"
+"  branch nick             yes            -             -\n"
+"  foreign vcs properties  yes            yes           -\n"
+"  preferred levels        all            1             1\n"
+"  digital signature       optional       -             -"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:24
+msgid ""
+"The default format is ``long``. To change this, define the ``log_format``\n"
+"setting in the ``[DEFAULT]`` section of ``bazaar.conf`` like this (say)::"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:27
+msgid ""
+"  [DEFAULT]\n"
+"  log_format = short"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:30
+msgid ""
+"Alternatively, to change the log format used for a given query, use the\n"
+"--long, --short or --line options."
+msgstr ""
+
+#: en/help_topics/log-formats.txt:33
+msgid ""
+"If one of the standard log formats does not meet your needs, additional\n"
+"formats can be provided by plugins.\n"
+msgstr ""
+
+#: en/help_topics/other-formats.txt:1
+msgid "Other Storage Formats"
+msgstr ""
+
+#: en/help_topics/other-formats.txt:3
+msgid "No experimental formats are available."
+msgstr ""
+
+#: en/help_topics/other-formats.txt:5
+msgid ""
+"\n"
+"No deprecated formats are available."
+msgstr ""
+
+#: en/help_topics/patterns.txt:1
+msgid ""
+"Patterns\n"
+"========"
+msgstr ""
+
+#: en/help_topics/patterns.txt:4
+msgid ""
+"Bazaar uses patterns to match files at various times. For example,\n"
+"the ``add`` command skips over files that match ignore patterns\n"
+"and preferences can be associated with files using rule patterns.\n"
+"The pattern syntax is described below."
+msgstr ""
+
+#: en/help_topics/patterns.txt:9
+msgid ""
+"Trailing slashes on patterns are ignored. If the pattern contains a\n"
+"slash or is a regular expression, it is compared to the whole path\n"
+"from the branch root. Otherwise, it is compared to only the last\n"
+"component of the path. To match a file only in the root directory,\n"
+"prepend ``./``. Patterns specifying absolute paths are not allowed."
+msgstr ""
+
+#: en/help_topics/patterns.txt:15
+msgid "Patterns may include globbing wildcards such as::"
+msgstr ""
+
+#: en/help_topics/patterns.txt:17
+msgid ""
+"  ? - Matches any single character except '/'\n"
+"  * - Matches 0 or more characters except '/'\n"
+"  /**/ - Matches 0 or more directories in a path\n"
+"  [a-z] - Matches a single character from within a group of characters"
+msgstr ""
+
+#: en/help_topics/patterns.txt:22
+msgid ""
+"Patterns may also be `Python regular expressions`_. Regular expression\n"
+"patterns are identified by a ``RE:`` prefix followed by the regular\n"
+"expression.  Regular expression patterns may not include named or\n"
+"numbered groups."
+msgstr ""
+
+#: en/help_topics/patterns.txt:27
+msgid ""
+"Case insensitive ignore patterns can be specified with regular expressions\n"
+"by using the ``i`` (for ignore case) flag in the pattern."
+msgstr ""
+
+#: en/help_topics/patterns.txt:30
+msgid "For example, a case insensitive match for ``foo`` may be specified as::"
+msgstr ""
+
+#: en/help_topics/patterns.txt:32
+msgid "  RE:(?i)foo"
+msgstr ""
+
+#: en/help_topics/patterns.txt:34
+msgid ""
+"Ignore patterns may be prefixed with ``!``, which means that a filename\n"
+"matched by that pattern will not be ignored."
+msgstr ""
+
+#: en/help_topics/patterns.txt:37
+msgid ".. _Python regular expressions: http://docs.python.org/library/re.html"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:1
+msgid "Revision Identifiers"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:3
+msgid ""
+"A revision identifier refers to a specific state of a branch's history.  It\n"
+"can be expressed in several ways.  It can begin with a keyword to\n"
+"unambiguously specify a given lookup type; some examples are 'last:1',\n"
+"'before:yesterday' and 'submit:'."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:8
+msgid ""
+"Alternately, it can be given without a keyword, in which case it will be\n"
+"checked as a revision number, a tag, a revision id, a date specification, or "
+"a\n"
+"branch specification, in that order.  For example, 'date:today' could be\n"
+"written as simply 'today', though if you have a tag called 'today' that "
+"will\n"
+"be found first."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:14
+msgid ""
+"If 'REV1' and 'REV2' are revision identifiers, then 'REV1..REV2' denotes a\n"
+"revision range. Examples: '3647..3649', 'date:yesterday..-1' and\n"
+"'branch:/path/to/branch1/..branch:/branch2' (note that there are no quotes "
+"or\n"
+"spaces around the '..')."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:19
+msgid ""
+"Ranges are interpreted differently by different commands. To the \"log\" "
+"command,\n"
+"a range is a sequence of log messages, but to the \"diff\" command, the "
+"range\n"
+"denotes a change between revisions (and not a sequence of changes).  In\n"
+"addition, \"log\" considers a closed range whereas \"diff\" and \"merge\" "
+"consider it\n"
+"to be open-ended, that is, they include one end but not the other.  For "
+"example:\n"
+"\"bzr log -r 3647..3649\" shows the messages of revisions 3647, 3648 and "
+"3649,\n"
+"while \"bzr diff -r 3647..3649\" includes the changes done in revisions 3648 "
+"and\n"
+"3649, but not 3647."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:28
+msgid "The keywords used as revision selection methods are the following:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:30
+msgid ""
+":revid:\n"
+"\tSelects a revision using the revision id.\n"
+":submit:\n"
+"\tSelects a common ancestor with the submit branch.\n"
+":ancestor:\n"
+"\tSelects a common ancestor with a second branch.\n"
+":date:\n"
+"\tSelects a revision on the basis of a datestamp.\n"
+":branch:\n"
+"\tSelects the last revision of a specified branch.\n"
+":tag:\n"
+"\tSelects a revision identified by a tag name.\n"
+":package:\n"
+"\tSelects the revision corresponding to a version of a package.\n"
+":revno:\n"
+"\tSelects a revision using a number.\n"
+":before:\n"
+"\tSelects the parent of the revision specified.\n"
+":annotate:\n"
+"\tSelect the revision that last modified the specified line.\n"
+":mainline:\n"
+"\tSelect mainline revision that merged the specified revision.\n"
+":last:\n"
+"\tSelects the nth revision from the end."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:55
+msgid "In addition, plugins can provide other keywords."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:57
+msgid "A detailed description of each keyword is given below."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:59
+msgid ":revid:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:61
+msgid ""
+"    Supply a specific revision id, that can be used to specify any\n"
+"    revision id in the ancestry of the branch.\n"
+"    Including merges, and pending merges.\n"
+"    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:66
+msgid ""
+"      revid:aaaa@bbbb-123456789 -> Select revision 'aaaa@bbbb-123456789'"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:68
+msgid ":submit:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:70
+msgid ""
+"    Diffing against this shows all the changes that were made in this "
+"branch,\n"
+"    and is a good predictor of what merge will do.  The submit branch is\n"
+"    used by the bundle and merge directive commands.  If no submit branch\n"
+"    is specified, the parent branch is used instead."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:75
+msgid ""
+"    The common ancestor is the last revision that existed in both\n"
+"    branches. Usually this is the branch point, but it could also be\n"
+"    a revision that was merged."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:79
+msgid "    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:81
+msgid "      $ bzr diff -r submit:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:83
+msgid ":ancestor:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:85
+msgid "    Supply the path to a branch to select the common ancestor."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:91
+msgid ""
+"    This is frequently used with 'diff' to return all of the changes\n"
+"    that your branch introduces, while excluding the changes that you\n"
+"    have not merged from the remote branch."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:97
+msgid ""
+"      ancestor:/path/to/branch\n"
+"      $ bzr diff -r ancestor:../../mainline/branch"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:100
+msgid ":date:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:102
+msgid ""
+"    Supply a datestamp to select the first revision that matches the date.\n"
+"    Date can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.\n"
+"    Matches the first entry after a given date (either at midnight or\n"
+"    at a specified time)."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:107
+msgid "    One way to display all the changes since yesterday would be::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:109
+msgid "        bzr log -r date:yesterday.."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:113
+msgid ""
+"      date:yesterday            -> select the first revision since "
+"yesterday\n"
+"      date:2006-08-14,17:10:14  -> select the first revision after\n"
+"                                   August 14th, 2006 at 5:10pm."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:117
+msgid ":branch:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:119
+msgid "    Supply the path to a branch to select its last revision."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:123
+msgid "      branch:/path/to/branch"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:125
+msgid ":tag:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:127
+msgid "    Tags are stored in the branch and created by the 'tag' command."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:129
+msgid ":package:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:131
+msgid ""
+"    Given a package version number this revision specifier will allow you\n"
+"    specify the revision corresponding to the upload of that version of\n"
+"    the package."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:135
+msgid ":revno:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:137
+msgid ""
+"    Use an integer to specify a revision in the history of the branch.\n"
+"    Optionally a branch can be specified.  A negative number will count\n"
+"    from the end of the branch (-1 is the last revision, -2 the previous\n"
+"    one). If the negative number is larger than the branch's history, the\n"
+"    first revision is returned.\n"
+"    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:144
+msgid ""
+"      revno:1                   -> return the first revision of this branch\n"
+"      revno:3:/path/to/branch   -> return the 3rd revision of\n"
+"                                   the branch '/path/to/branch'\n"
+"      revno:-1                  -> The last revision in a branch.\n"
+"      -2:http://other/branch    -> The second to last revision in the\n"
+"                                   remote branch.\n"
+"      -1000000                  -> Most likely the first revision, unless\n"
+"                                   your history is very long."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:153
+msgid ":before:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:155
+msgid ""
+"    Supply any revision spec to return the parent of that revision.  This "
+"is\n"
+"    mostly useful when inspecting revisions that are not in the revision "
+"history\n"
+"    of a branch."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:159
+msgid ""
+"    It is an error to request the parent of the null revision (before:0)."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:163
+msgid ""
+"      before:1913    -> Return the parent of revno 1913 (revno 1912)\n"
+"      before:revid:aaaa@bbbb-1234567890  -> return the parent of revision\n"
+"                                            aaaa@bbbb-1234567890\n"
+"      bzr diff -r before:1913..1913\n"
+"            -> Find the changes between revision 1913 and its parent "
+"(1912).\n"
+"               (What changes did revision 1913 introduce).\n"
+"               This is equivalent to:  bzr diff -c 1913"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:171
+msgid ":annotate:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:173
+msgid ""
+"    Select the revision that last modified the specified line.  Line is\n"
+"    specified as path:number.  Path is a relative path to the file.  "
+"Numbers\n"
+"    start at 1, and are relative to the current version, not the last-\n"
+"    committed version of the file."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:178
+msgid ":mainline:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:180
+msgid ""
+"    Select the revision that merged the specified revision into mainline."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:182
+msgid ":last:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:184
+msgid ""
+"    Supply a positive number to get the nth revision from the end.\n"
+"    This is the same as supplying negative numbers to the 'revno:' spec.\n"
+"    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:188
+msgid ""
+"      last:1        -> return the last revision\n"
+"      last:3        -> return the revision 2 before the end.\n"
+msgstr ""
+
+#: en/help_topics/rules.txt:1
+msgid ""
+"Rules\n"
+"====="
+msgstr ""
+
+#: en/help_topics/rules.txt:7
+msgid ""
+"Rules are defined in ini file format where the sections are file glob\n"
+"patterns and the contents of each section are the preferences for files\n"
+"matching that pattern(s). For example::"
+msgstr ""
+
+#: en/help_topics/rules.txt:11
+msgid ""
+"    [name *.bat]\n"
+"    eol = native"
+msgstr ""
+
+#: en/help_topics/rules.txt:14
+msgid ""
+"    [name *.html *.xml]\n"
+"    keywords = xml_escape"
+msgstr ""
+
+#: en/help_topics/rules.txt:17
+msgid ""
+"Preferences like these are useful for commands and plugins wishing to\n"
+"provide custom behaviour for selected files.  For more information on \n"
+"end of line conversion see :doc:`eol-help`.\n"
+"Keyword support is provided by the `keywords plugin\n"
+"<http://doc.bazaar.canonical.com/plugins/en/keywords-plugin.html>`_."
+msgstr ""
+
+#: en/help_topics/rules.txt:23
+msgid ""
+"Files\n"
+"-----"
+msgstr ""
+
+#: en/help_topics/rules.txt:26
+msgid ""
+"Default rules for all branches are defined in the optional file\n"
+"``BZR_HOME/rules``. "
+msgstr ""
+
+#: en/help_topics/rules.txt:29
+msgid ""
+"Rule Patterns\n"
+"-------------"
+msgstr ""
+
+#: en/help_topics/rules.txt:32
+msgid ""
+"Patterns are ordered and searching stops as soon as one matches.\n"
+"As a consequence, more explicit patterns should be placed towards\n"
+"the top of the file. Rule patterns use exactly the same conventions\n"
+"as ignore patterns. See :doc:`patterns-help` for details."
+msgstr ""
+
+#: en/help_topics/rules.txt:37
+msgid ".. note::"
+msgstr ""
+
+#: en/help_topics/rules.txt:39
+msgid ""
+"   Patterns containing square brackets or spaces should be\n"
+"   surrounded in quotes to ensure they are correctly parsed.\n"
+msgstr ""
+
+#: en/help_topics/topics.txt:1
+msgid ""
+"authentication       Information on configuring authentication\n"
+"basic                Basic commands\n"
+"branches             Information on what a branch is\n"
+"bugs                 Bug tracker settings\n"
+"checkouts            Information on what a checkout is\n"
+"commands             Basic help for all commands\n"
+"configuration        Details on the configuration settings available\n"
+"conflict-types       Types of conflicts and what to do about them\n"
+"content-filters      Conversion of content into/from working trees\n"
+"criss-cross          Information on criss-cross merging\n"
+"current-formats      Current storage formats\n"
+"debug-flags          Options to show or record debug information\n"
+"diverged-branches    How to fix diverged branches\n"
+"env-variables        Environment variable names and values\n"
+"eol                  Information on end-of-line handling\n"
+"files                Information on configuration and log files\n"
+"formats              Information on choosing a storage format\n"
+"global-options       Options that control how Bazaar runs\n"
+"hidden-commands      All hidden commands\n"
+"hooks                Points at which custom processing can be added\n"
+"launchpad            Using Bazaar with Launchpad.net\n"
+"location-alias       Aliases for remembered locations\n"
+"log-formats          Details on the logging formats available\n"
+"other-formats        Experimental and deprecated storage formats\n"
+"patterns             Information on the pattern syntax\n"
+"repositories         Basic information on shared repositories.\n"
+"revisionspec         Explain how to use --revision\n"
+"rules                Information on defining rule-based preferences\n"
+"standalone-trees     Information on what a standalone tree is\n"
+"standard-options     Options that can be used with any command\n"
+"status-flags         Help on status flags\n"
+"sync-for-reconfigure Steps to resolve \"out-of-sync\" when reconfiguring\n"
+"topics               Topics list\n"
+"url-special-chars    Special character handling in URLs\n"
+"urlspec              Supported transport protocols\n"
+"working-trees        Information on working trees\n"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:1
+msgid ""
+"Special character handling in URLs\n"
+"=================================="
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:4
+msgid "Bazaar allows locations to be specified in multiple ways, either:"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:6
+msgid " * Fully qualified URLs"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:8
+msgid " * File system paths, relative or absolute"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:10
+msgid ""
+"Internally bzr treats all locations as URLs. For any file system paths\n"
+"that are specified it will automatically determine the appropriate URL\n"
+"representation, and escape special characters where necessary."
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:14
+msgid ""
+"There are a few characters which have special meaning in URLs and need "
+"careful\n"
+"handling to avoid ambiguities. Characters can be escaped with a % and a hex\n"
+"value in URLs. Any non-ASCII characters in a file path will automatically "
+"be\n"
+"urlencoded when the path is converted to a URL."
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:19
+msgid ""
+"URLs represent non-ASCII characters in an encoding defined by the server, "
+"but\n"
+"usually UTF-8.  The % escapes should be of the UTF-8 bytes.  Bazaar tries to "
+"be\n"
+"generous in what it accepts as a URL and to print them in a way that\n"
+"will be readable."
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:24
+msgid ""
+"For example, if you have a directory named '/tmp/%2False' these are all "
+"valid\n"
+"ways of accessing the content (0x2F, or 47, is the ASCII code for forward "
+"slash)::"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:27
+msgid ""
+"  cd /tmp\n"
+"  bzr log /tmp/%2False\n"
+"  bzr log %2False\n"
+"  bzr log file:///tmp/%252False\n"
+"  bzr log file://localhost/tmp/%252False\n"
+"  bzr log file:%252False"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:34
+msgid "These are valid but do not refer to the same file::"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:36
+msgid ""
+"  bzr log file:///tmp/%2False (refers to a file called /tmp/\\/alse)\n"
+"  bzr log %252False (refers to a file called /tmp/%252False)"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:39
+msgid ""
+"Comma also has special meaning in URLs, because it denotes `segment "
+"parameters`_"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:41
+msgid ""
+"_`segment parameters`: http://www.ietf.org/rfc/rfc3986.txt (section 3.3)"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:43
+msgid ""
+"Comma is also special in any file system paths that are specified. To use a "
+"literal\n"
+"comma in a file system path, specify a URL and URL encode the comma::"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:46
+msgid ""
+"  bzr log foo,branch=bla # path \"foo\" with the segment parameter \"branch"
+"\" set to \"bla\"\n"
+"  bzr log file:foo%2Cbranch=bla # path \"foo,branch=bla\"\n"
+"  bzr log file:foo,branch=bla # path \"foo\" with segment parameter \"branch"
+"\" set to \"bla\"\n"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:1
+msgid "URL Identifiers"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:3
+msgid "Supported URL prefixes::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:5
+msgid ""
+"  aftp://             Access using active FTP.\n"
+"  bzr://              Fast access using the Bazaar smart server.\n"
+"  bzr+ssh://          Fast access using the Bazaar smart server over SSH.\n"
+"  file://             Access using the standard filesystem (default)\n"
+"  ftp://              Access using passive FTP.\n"
+"  http://             Read-only access of branches exported on the web.\n"
+"  https://            Read-only access of branches exported on the web using "
+"SSL.\n"
+"  sftp://             Access using SFTP (most SSH servers provide SFTP)."
+msgstr ""
+
+#: en/help_topics/urlspec.txt:14
+msgid "Supported modifiers::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:16
+msgid "  gio+                Access using any GIO supported protocols."
+msgstr ""
+
+#: en/help_topics/urlspec.txt:18
+msgid "Bazaar supports all of the standard parts within the URL::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:20
+msgid "  <protocol>://[user[:password]@]host[:port]/[path]"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:22
+msgid "allowing URLs such as::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:24
+msgid "  http://bzruser:BadPass@bzr.example.com:8080/bzr/trunk"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:26
+msgid ""
+"For bzr+ssh:// and sftp:// URLs, Bazaar also supports paths that begin\n"
+"with '~' as meaning that the rest of the path should be interpreted\n"
+"relative to the remote user's home directory.  For example if the user\n"
+"``remote`` has a  home directory of ``/home/remote`` on the server\n"
+"shell.example.com, then::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:32
+msgid "  bzr+ssh://remote@shell.example.com/~/myproject/trunk"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:34
+msgid "would refer to ``/home/remote/myproject/trunk``."
+msgstr ""
+
+#: en/help_topics/urlspec.txt:36
+msgid ""
+"Many commands that accept URLs also accept location aliases too.\n"
+"See :doc:`location-alias-help` and :doc:`url-special-chars-help`.\n"
+msgstr ""

=== modified file 'profile_imports.py'
--- profile_imports.py	2010-02-11 04:02:41 +0000
+++ profile_imports.py	2011-05-11 11:35:28 +0000
@@ -21,11 +21,7 @@
 import time
 
 
-if sys.version_info < (2, 5, 0):
-    import sre
-    re = sre
-else:
-    import re
+import re
 
 
 _parent_stack = []

=== modified file 'setup.py'
--- setup.py	2010-07-26 10:15:32 +0000
+++ setup.py	2011-10-06 16:41:45 +0000
@@ -9,9 +9,11 @@
 import os
 import os.path
 import sys
+import copy
+import glob
 
-if sys.version_info < (2, 4):
-    sys.stderr.write("[ERROR] Not a supported Python version. Need 2.4+\n")
+if sys.version_info < (2, 6):
+    sys.stderr.write("[ERROR] Not a supported Python version. Need 2.6+\n")
     sys.exit(1)
 
 # NOTE: The directory containing setup.py, whether run by 'python setup.py' or
@@ -65,12 +67,17 @@
             'package_data': {'bzrlib': ['doc/api/*.txt',
                                         'tests/test_patches_data/*',
                                         'help_topics/en/*.txt',
+                                        'tests/ssl_certs/ca.crt',
                                         'tests/ssl_certs/server_without_pass.key',
                                         'tests/ssl_certs/server_with_pass.key',
-                                        'tests/ssl_certs/server.crt'
+                                        'tests/ssl_certs/server.crt',
                                        ]},
            }
-
+I18N_FILES = []
+for filepath in glob.glob("bzrlib/locale/*/LC_MESSAGES/*.mo"):
+    langfile = filepath[len("bzrlib/locale/"):]
+    targetpath = os.path.dirname(os.path.join("share/locale", langfile))
+    I18N_FILES.append((targetpath, [filepath]))
 
 def get_bzrlib_packages():
     """Recurse through the bzrlib directory, and extract the package names"""
@@ -150,6 +157,10 @@
     Generate bzr.1.
     """
 
+    sub_commands = build.sub_commands + [
+            ('build_mo', lambda _: True),
+            ]
+
     def run(self):
         build.run(self)
 
@@ -161,20 +172,24 @@
 ## Setup
 ########################
 
+from bzrlib.bzr_distutils import build_mo
+
 command_classes = {'install_scripts': my_install_scripts,
-                   'build': bzr_build}
+                   'build': bzr_build,
+                   'build_mo': build_mo,
+                   }
 from distutils import log
 from distutils.errors import CCompilerError, DistutilsPlatformError
 from distutils.extension import Extension
 ext_modules = []
 try:
     try:
+        from Cython.Distutils import build_ext
+        from Cython.Compiler.Version import version as pyrex_version
+    except ImportError:
+        print("No Cython, trying Pyrex...")
         from Pyrex.Distutils import build_ext
         from Pyrex.Compiler.Version import version as pyrex_version
-    except ImportError:
-        print("No Pyrex, trying Cython...")
-        from Cython.Distutils import build_ext
-        from Cython.Compiler.Version import version as pyrex_version
 except ImportError:
     have_pyrex = False
     # try to build the extension from the prior generated source.
@@ -187,7 +202,7 @@
     from distutils.command.build_ext import build_ext
 else:
     have_pyrex = True
-    pyrex_version_info = tuple(map(int, pyrex_version.split('.')))
+    pyrex_version_info = tuple(map(int, pyrex_version.rstrip("+").split('.')))
 
 
 class build_ext_if_possible(build_ext):
@@ -289,11 +304,11 @@
         # The code it generates re-uses a "local" pointer and
         # calls "PY_DECREF" after having set it to NULL. (It mixes PY_XDECREF
         # which is NULL safe with PY_DECREF which is not.)
-        # <https://bugs.edge.launchpad.net/bzr/+bug/449372>
-        # <https://bugs.edge.launchpad.net/bzr/+bug/276868>
+        # <https://bugs.launchpad.net/bzr/+bug/449372>
+        # <https://bugs.launchpad.net/bzr/+bug/276868>
         print('Cannot build extension "bzrlib._dirstate_helpers_pyx" using')
-        print('your version of pyrex "%s". Please upgrade your pyrex' % (
-            pyrex_version,))
+        print('your version of pyrex "%s". Please upgrade your pyrex'
+              % (pyrex_version,))
         print('install. For now, the non-compiled (python) version will')
         print('be used instead.')
     else:
@@ -393,7 +408,8 @@
     # ditto for the tbzrcommand tool
     tbzrcommand = dict(
         script = os.path.join(tbzr_root, "scripts", "tbzrcommand.py"),
-        icon_resources = [(0,'bzr.ico')],
+        icon_resources = icon_resources,
+        other_resources = other_resources,
     )
     console_targets.append(tbzrcommand)
     tbzrcommandw = tbzrcommand.copy()
@@ -466,6 +482,12 @@
     packages.append('sqlite3')
 
 
+def get_fastimport_py2exe_info(includes, excludes, packages):
+    # This is the python-fastimport package, not to be confused with the
+    # bzr-fastimport plugin.
+    packages.append('fastimport')
+
+
 if 'bdist_wininst' in sys.argv:
     def find_docs():
         docs = []
@@ -490,17 +512,17 @@
             # help pages
             'data_files': find_docs(),
             # for building pyrex extensions
-            'cmdclass': {'build_ext': build_ext_if_possible},
+            'cmdclass': command_classes,
            }
 
     ARGS.update(META_INFO)
     ARGS.update(BZRLIB)
+    PKG_DATA['package_data']['bzrlib'].append('locale/*/LC_MESSAGES/*.mo')
     ARGS.update(PKG_DATA)
-    
+
     setup(**ARGS)
 
 elif 'py2exe' in sys.argv:
-    import glob
     # py2exe setup
     import py2exe
 
@@ -557,6 +579,8 @@
                                      company_name = "Canonical Ltd.",
                                      comments = META_INFO['description'],
                                     )
+    gui_target = copy.copy(target)
+    gui_target.dest_base = "bzrw"
 
     packages = BZRLIB['packages']
     packages.remove('bzrlib')
@@ -647,8 +671,8 @@
     console_targets = [target,
                        'tools/win32/bzr_postinstall.py',
                        ]
-    gui_targets = []
-    data_files = topics_files + plugins_files
+    gui_targets = [gui_target]
+    data_files = topics_files + plugins_files + I18N_FILES
 
     if 'qbzr' in plugins:
         get_qbzr_py2exe_info(includes, excludes, packages, data_files)
@@ -656,6 +680,9 @@
     if 'svn' in plugins:
         get_svn_py2exe_info(includes, excludes, packages)
 
+    if 'fastimport' in plugins:
+        get_fastimport_py2exe_info(includes, excludes, packages)
+
     if "TBZR" in os.environ:
         # TORTOISE_OVERLAYS_MSI_WIN32 must be set to the location of the
         # TortoiseOverlays MSI installer file. It is in the TSVN svn repo and
@@ -685,13 +712,19 @@
 
     # MSWSOCK.dll is a system-specific library, which py2exe accidentally pulls
     # in on Vista.
-    dll_excludes.extend(["MSWSOCK.dll", "MSVCP60.dll", "powrprof.dll"])
+    dll_excludes.extend(["MSWSOCK.dll",
+                         "MSVCP60.dll",
+                         "MSVCP90.dll",
+                         "powrprof.dll",
+                         "SHFOLDER.dll"])
     options_list = {"py2exe": {"packages": packages + list(additional_packages),
                                "includes": includes,
                                "excludes": excludes,
                                "dll_excludes": dll_excludes,
                                "dist_dir": "win32_bzr.exe",
                                "optimize": 2,
+                               "custom_boot_script":
+                                        "tools/win32/py2exe_boot_common.py",
                               },
                    }
 
@@ -705,13 +738,14 @@
             self.optimize = 2
 
     if __name__ == '__main__':
+        command_classes['install_data'] = install_data_with_bytecompile
+        command_classes['py2exe'] = py2exe_no_oo_exe
         setup(options=options_list,
               console=console_targets,
               windows=gui_targets,
               zipfile='lib/library.zip',
               data_files=data_files,
-              cmdclass={'install_data': install_data_with_bytecompile,
-                        'py2exe': py2exe_no_oo_exe},
+              cmdclass=command_classes,
               )
 
 else:
@@ -722,19 +756,7 @@
         # easy_install one
         DATA_FILES = [('man/man1', ['bzr.1'])]
 
-    if sys.platform != 'win32':
-        # see https://wiki.kubuntu.org/Apport/DeveloperHowTo
-        #
-        # checking the paths and hardcoding the check for root is a bit gross,
-        # but I don't see a cleaner way to find out the locations in a way
-        # that's going to align with the hardcoded paths in apport.
-        if os.geteuid() == 0:
-            DATA_FILES += [
-                ('/usr/share/apport/package-hooks',
-                    ['apport/source_bzr.py']),
-                ('/etc/apport/crashdb.conf.d/',
-                    ['apport/bzr-crashdb.conf']),]
-
+    DATA_FILES = DATA_FILES + I18N_FILES
     # std setup
     ARGS = {'scripts': ['bzr'],
             'data_files': DATA_FILES,

=== modified file 'tools/check-newsbugs.py'
--- tools/check-newsbugs.py	2010-06-01 16:46:47 +0000
+++ tools/check-newsbugs.py	2011-05-26 20:42:13 +0000
@@ -5,23 +5,33 @@
 import getopt, re, sys
 try:
     from launchpadlib.launchpad import Launchpad
+    from lazr.restfulclient import errors
 except ImportError:
     print "Please install launchpadlib from lp:launchpadlib"
     sys.exit(1)
-
-options, args = getopt.gnu_getopt(sys.argv, "l", ["launchpad"])
+try:
+    import hydrazine
+except ImportError:
+    print "Please install hydrazine from lp:hydrazine"
+    sys.exit(1)
+
+
+options, args = getopt.gnu_getopt(sys.argv, "lw", ["launchpad", 'webbrowser'])
 options = dict(options)
 
 if len(args) == 1:
-    print "Usage: check-newsbugs [--launchpad] NEWS"
+    print ("Usage: check-newsbugs [--launchpad][--webbrowser] "
+           "doc/en/release-notes/bzr-x.y.txt")
     print "Options:"
     print "--launchpad     Print out Launchpad mail commands for closing bugs "
     print "                that are already fixed."
+    print "--webbrowser    Open launchpad bug pages for bugs that are already "
+    print "                fixed."
     sys.exit(1)
 
 
 def report_notmarked(bug, task, section):
-    print 
+    print
     print "Bug %d was mentioned in NEWS but is not marked fix released:" % (bug.id, )
     print "Launchpad title: %s" % bug.title
     print "NEWS summary: "
@@ -30,6 +40,9 @@
         print "  bug %d" % bug.id
         print "  affects %s" % task.bug_target_name
         print "  status fixreleased"
+    if "--webbrowser" in options or "-w" in options:
+        import webbrowser
+        webbrowser.open('http://pad.lv/%s>' % (bug.id,))
 
 
 def read_news_bugnos(path):
@@ -61,16 +74,42 @@
         f.close()
 
 
-lp = Launchpad.login_anonymously('bzr-check-newsbugs', 'edge', version='1.0')
+def print_bug_url(bugno):
+    print '<URL:http://pad.lv/%s>' % (bugno,)
 
+launchpad = hydrazine.create_session()
 bugnos = read_news_bugnos(args[1])
 for bugno, section in bugnos:
-    bug = lp.bugs[bugno]
+    try:
+        bug = launchpad.bugs[bugno]
+    except errors.HTTPError, e:
+        if e.response.status == 401:
+            print_bug_url(bugno)
+            # Private, we can't access the bug content
+            print '%s is private and cannot be accessed' % (bugno,)
+            continue
+        raise
+
     found_bzr = False
+    fix_released = False
     for task in bug.bug_tasks:
-        if task.bug_target_name == "bzr":
+        parts = task.bug_target_name.split('/')
+        if len(parts) == 1:
+            project = parts[0]
+            distribution = None
+        else:
+            project = parts[0]
+            distribution = parts[1]
+        if project == "bzr":
             found_bzr = True
-            if task.status != "Fix Released":
-                report_notmarked(bug, task, section)
+            if not fix_released and task.status == "Fix Released":
+                # We could check that the NEWS section and task_status are in
+                # sync, but that would be overkill. (case at hand: bug #416732)
+                fix_released = True
+
     if not found_bzr:
+        print_bug_url(bugno)
         print "Bug %d was mentioned in NEWS but is not marked as affecting bzr" % bugno
+    elif not fix_released:
+        print_bug_url(bugno)
+        report_notmarked(bug, task, section)

=== removed file 'tools/convertfile.py'
--- tools/convertfile.py	2009-03-23 14:59:43 +0000
+++ tools/convertfile.py	1970-01-01 00:00:00 +0000
@@ -1,87 +0,0 @@
-#! /usr/bin/python
-
-# Copyright (C) 2005 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Experiment in converting existing bzr branches to weaves."""
-
-
-import sys
-import bzrlib.branch
-from bzrlib.weave import Weave
-from bzrlib.weavefile import write_weave
-
-import hotshot
-import tempfile
-
-
-def convert():
-    WEAVE_NAME = "test.weave"
-
-    wf = Weave()
-
-    toconvert = sys.argv[1]
-    
-    b = bzrlib.branch.find_branch(toconvert)
-    rp = b.relpath(toconvert)
-
-    print 'converting...'
-
-    fid = b.read_working_inventory().path2id(rp)
-
-    last_lines = None
-    parents = set()
-    revno = 0
-    for rev_id in b.revision_history():
-        revno += 1
-        print revno
-        tree = b.revision_tree(rev_id)
-        inv = tree.inventory
-
-        if fid not in tree:
-            print '  (not present)'
-            continue
-
-        text = tree.get_file(fid).readlines()
-
-        if text == last_lines:
-            continue
-        last_lines = text
-        
-        weave_id = wf.add(parents, text)
-        parents = [weave_id]
-
-        print '  %4d lines' % len(text)
-
-    write_weave(wf, file(WEAVE_NAME, 'wb'))
-
-
-prof_f = tempfile.NamedTemporaryFile()
-
-prof = hotshot.Profile(prof_f.name)
-
-prof.runcall(convert) 
-prof.close()
-
-import hotshot.stats
-stats = hotshot.stats.load(prof_f.name)
-#stats.strip_dirs()
-stats.sort_stats('time')
-## XXX: Might like to write to stderr or the trace file instead but
-## print_stats seems hardcoded to stdout
-stats.print_stats(20)
-            
-

=== removed file 'tools/convertinv.py'
--- tools/convertinv.py	2009-03-23 14:59:43 +0000
+++ tools/convertinv.py	1970-01-01 00:00:00 +0000
@@ -1,75 +0,0 @@
-#! /usr/bin/python
-
-# Copyright (C) 2005 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Experiment in converting existing bzr branches to weaves."""
-
-
-import bzrlib.branch
-from bzrlib.weave import Weave
-from bzrlib.weavefile import write_weave
-from bzrlib.progress import ProgressBar
-import tempfile
-import hotshot, hotshot.stats
-import sys
-
-def convert():
-    WEAVE_NAME = "inventory.weave"
-
-    pb = ProgressBar()
-
-    wf = Weave()
-
-    b = bzrlib.branch.find_branch('.')
-
-    parents = set()
-    revno = 1
-    rev_history = b.revision_history()
-    for rev_id in rev_history:
-        pb.update('converting inventory', revno, len(rev_history))
-        inv_xml = b.inventory_store[rev_id].readlines()
-        weave_id = wf.add(parents, inv_xml)
-        parents = set([weave_id])       # always just one parent
-        revno += 1
-
-    pb.update('write weave', None, None)
-    write_weave(wf, file(WEAVE_NAME, 'wb'))
-
-    pb.clear()
-
-
-def profile_convert(): 
-    prof_f = tempfile.NamedTemporaryFile()
-
-    prof = hotshot.Profile(prof_f.name)
-
-    prof.runcall(convert) 
-    prof.close()
-
-    stats = hotshot.stats.load(prof_f.name)
-    #stats.strip_dirs()
-    stats.sort_stats('time')
-    ## XXX: Might like to write to stderr or the trace file instead but
-    ## print_stats seems hardcoded to stdout
-    stats.print_stats(20)
-            
-
-if '-p' in sys.argv[1:]:
-    profile_convert()
-else:
-    convert()
-    

=== added file 'tools/fixed-in.py'
--- tools/fixed-in.py	1970-01-01 00:00:00 +0000
+++ tools/fixed-in.py	2010-09-28 08:57:31 +0000
@@ -0,0 +1,172 @@
+#!/usr/bin/python
+
+# Simple script that will output the release where a given bug was fixed
+# searching the NEWS file
+
+import optparse
+import re
+import sys
+
+
+class NewsParser(object):
+
+    paren_exp_re = re.compile('\(([^)]+)\)')
+    release_re = re.compile("bzr[ -]")
+    release_prefix_length = len('bzr ')
+    bugs_re = re.compile('#([0-9]+)')
+
+    def __init__(self, news):
+        self.news = news
+        # Temporary attributes used by the parser
+        self.release = None
+        self.date = None
+        self.may_be_release = None
+        self.release_markup = None
+        self.entry = ''
+        self.line = None
+        self.lrs = None
+
+    def set_line(self, line):
+        self.line = line
+        self.lrs = line.rstrip()
+
+    def try_release(self):
+        if self.release_re.match(self.lrs) is not None:
+            # May be a new release
+            self.may_be_release = self.lrs
+            # We know the markup will have the same length as the release
+            self.release_markup = '#' * len(self.may_be_release)
+            return True
+        return False
+
+    def confirm_release(self):
+        if self.may_be_release is not None and self.lrs == self.release_markup:
+            # The release is followed by the right markup
+            self.release = self.may_be_release[self.release_prefix_length:]
+            # Wait for the associated date
+            self.date = None
+            return True
+        return False
+
+    def try_date(self):
+        if self.release is None:
+            return False
+        date_re = re.compile(':%s: (NOT RELEASED YET|\d{4}-\d{2}-\d{2})'
+                             % (self.release,))
+        match = date_re.match(self.lrs)
+        if  match is not None:
+            self.date = match.group(1)
+            return True
+        # The old fashion way
+        released_re = re.compile(':Released:\s+(\d{4}-\d{2}-\d{2})')
+        match = released_re.match(self.lrs)
+        if  match is not None:
+            self.date = match.group(1)
+            return True
+        return False
+
+    def add_line_to_entry(self):
+        if self.lrs == '':
+            return False
+        self.entry += self.line
+        return True
+
+    def extract_bugs_from_entry(self):
+        """Possibly extract bugs from a NEWS entry and yield them.
+
+        Not all entries will contain bugs and some entries are even garbage and
+        we don't try to parse them (yet). The trigger is a '#' and what looks
+        like a bug number inside parens to start with. From that we extract
+        authors (when present) and multiple bugs if needed.
+        """
+        # FIXME: Malone entries are different
+        # Join all entry lines to simplify multiple line matching
+        flat_entry = ' '.join(self.entry.splitlines())
+        # Fixed bugs are always inside parens
+        for par in self.paren_exp_re.findall(flat_entry):
+            sharp = par.find('#')
+            if sharp is not None:
+                # We have at least one bug inside parens.
+                bugs = list(self.bugs_re.finditer(par))
+                if bugs:
+                    # See where the first bug is mentioned
+                    start = bugs[0].start()
+                    end = bugs[-1].end()
+                    if start == 0:
+                        # (bugs/authors)
+                        authors = par[end:]
+                    else:
+                        # (authors/bugs)
+                         authors = par[:start]
+                    for bug_match in bugs:
+                        bug_number = bug_match.group(0)
+                        yield (bug_number, authors,
+                               self.release, self.date, self.entry)
+        # We've consumed the entry
+        self.entry = ''
+
+    def parse_bugs(self):
+        for line in self.news:
+            self.set_line(line)
+            if self.try_release():
+                continue # line may a be release
+            try:
+                if self.confirm_release():
+                    continue # previous line was indeed a release
+            finally:
+                self.may_be_release = None
+            if self.try_date():
+                continue # The release date has been seen
+            if self.add_line_to_entry():
+                continue # accumulate in self.enrty
+            for b in self.extract_bugs_from_entry():
+                yield b # all bugs in the news entry
+
+def main():
+    opt_parser = optparse.OptionParser(
+        usage="""Usage: %prog [options] BUG_NUMBER
+    """)
+    opt_parser.add_option(
+        '-f', '--file', type='str', dest='news_file',
+        help='NEWS file (defaults to ./NEWS)')
+    opt_parser.add_option(
+        '-m', '--message', type='str', dest='msg_re',
+        help='A regexp to search for in the news entry '
+        '(BUG_NUMBER should not be specified in this case)')
+    opt_parser.set_defaults(news_file='./NEWS')
+    (opts, args) = opt_parser.parse_args(sys.argv[1:])
+    if opts.msg_re is not None:
+        if len(args) != 0:
+            opt_parser.error('BUG_NUMBER and -m are mutually exclusive')
+        bug = None
+        msg_re = re.compile(opts.msg_re)
+    elif len(args) != 1:
+        opt_parser.error('Expected a single bug number, got %r' % args)
+    else:
+        bug = args[0]
+
+    news = open(opts.news_file)
+    parser = NewsParser(news)
+    try:
+        seen = 0
+        for b in parser.parse_bugs():
+            (number, authors, release, date, entry,) = b
+            # indent entry
+            entry = '\n'.join(['    ' + l for l in entry.splitlines()])
+            found = False
+            if bug is not None:
+                if number[1:] == bug: # Strip the leading '#'
+                    found = True
+            elif msg_re.search(entry) is not None:
+                found = True
+            if found:
+                print 'Bug %s was fixed in bzr-%s/%s by %s:' % (
+                    number, release, date, authors)
+                print entry
+            seen += 1
+    finally:
+        print '%s bugs seen' % (seen,)
+        news.close()
+
+
+main()

=== modified file 'tools/generate_docs.py'
--- tools/generate_docs.py	2009-07-14 15:01:19 +0000
+++ tools/generate_docs.py	2011-09-26 15:40:02 +0000
@@ -38,7 +38,12 @@
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
 
-from bzrlib import commands, doc_generate
+import bzrlib
+from bzrlib import (
+    commands,
+    doc_generate,
+    )
+
 
 def main(argv):
     parser = OptionParser(usage="""%prog [options] OUTPUT_FORMAT
@@ -71,26 +76,23 @@
         parser.print_help()
         sys.exit(1)
 
-    commands.install_bzr_command_hooks()
-
-    infogen_type = args[1]
-    infogen_mod = doc_generate.get_module(infogen_type)
-
-    if options.filename:
-        outfilename = options.filename
-    else:
-        outfilename = infogen_mod.get_filename(options)
-
-    if outfilename == "-":
-        outfile = sys.stdout
-    else:
-        outfile = open(outfilename,"w")
-
-    if options.show_filename and (outfilename != "-"):
-        sys.stdout.write(outfilename)
-        sys.stdout.write('\n')
-    
-    infogen_mod.infogen(options, outfile)
+    with bzrlib.initialize():
+        commands.install_bzr_command_hooks()
+        infogen_type = args[1]
+        infogen_mod = doc_generate.get_module(infogen_type)
+        if options.filename:
+            outfilename = options.filename
+        else:
+            outfilename = infogen_mod.get_filename(options)
+        if outfilename == "-":
+            outfile = sys.stdout
+        else:
+            outfile = open(outfilename,"w")
+        if options.show_filename and (outfilename != "-"):
+            sys.stdout.write(outfilename)
+            sys.stdout.write('\n')
+        infogen_mod.infogen(options, outfile)
+
 
 def print_extended_help(option, opt, value, parser):
     """ Program help examples

=== modified file 'tools/generate_release_notes.py'
--- tools/generate_release_notes.py	2010-05-05 17:04:30 +0000
+++ tools/generate_release_notes.py	2010-10-13 06:14:37 +0000
@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2010 Canonical Ltd.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,77 +16,121 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import os
+"""Generate doc/en/release-notes/index.txt from the per-series NEWS files.
+
+NEWS files are kept in doc/en/release-notes/, one file per series, e.g.
+doc/en/release-notes/bzr-2.3.txt
+"""
+
+# XXX: add test_source test that latest doc/en/release-notes/bzr-*.txt has the
+# NEWS file-id (so that merges of new work will tend to always land new NEWS
+# entries in the latest series).
+
+
+import os.path
+import re
 import sys
 from optparse import OptionParser
 
 
-def split_into_topics(lines, out_file, out_dir):
-    """Split a large NEWS file into topics, one per release.
-
-    Releases are detected by matching headings that look like
-    release names. Topics are created with matching names
-    replacing spaces with dashes.
+preamble_plain = """\
+####################
+Bazaar Release Notes
+####################
+
+
+.. contents:: List of Releases
+   :depth: 2
+
+"""
+
+preamble_sphinx = """\
+####################
+Bazaar Release Notes
+####################
+
+
+.. toctree::
+   :maxdepth: 2
+
+"""
+
+
+def natural_sort_key(file_name):
+    """Split 'aaa-N.MMbbb' into ('aaa-', N, '.' MM, 'bbb')
+    
+    e.g. 1.10b1 will sort as greater than 1.2::
+
+        >>> natural_sort_key('bzr-1.10b1.txt') > natural_sort_key('bzr-1.2.txt')
+        True
     """
-    topic_file = None
-    for index, line in enumerate(lines):
-        maybe_new_topic = line[:4] in ('bzr ', 'bzr-',)
-        if maybe_new_topic and lines[index + 1].startswith('####'):
-            release = line.strip()
-            if topic_file is None:
-                # First topic found
-                out_file.write(".. toctree::\n   :maxdepth: 1\n\n")
-            else:
-                # close the current topic
-                topic_file.close()
-            topic_file = open_topic_file(out_file, out_dir, release)
-        elif topic_file:
-            topic_file.write(line)
-        else:
-            # FIXME: the 'content' directive is used for rst2html (and
-            # conflicts with the 'toctree' we insert), we should get rid of
-            # that once we fully switch to sphinx -- vila 20100505
-            if (line.startswith('.. contents::')
-                or line.startswith('   :depth:')):
-                    continue
-            # Still in the header - dump content straight to output
-            out_file.write(line)
-    if topic_file is not None:
-        # Close the last topic_file opened
-        topic_file.close()
-
-
-def open_topic_file(out_file, out_dir, release):
-    topic_name = release.replace(' ', '-')
-    out_file.write("   %s <%s>\n" % (release, topic_name,))
-    topic_path = os.path.join(out_dir, "%s.txt" % (topic_name,))
-    result = open(topic_path, 'w')
-    result.write("%s\n" % (release,))
-    return result
+    file_name = os.path.basename(file_name)
+    parts = re.findall(r'(?:[0-9]+|[^0-9]+)', file_name)
+    result = []
+    for part in parts:
+        if re.match('^[0-9]+$', part) is not None:
+            part = int(part)
+        result.append(part)
+    return tuple(result)
+
+
+def output_news_file_sphinx(out_file, news_file_name):
+    news_file_name = os.path.basename(news_file_name)
+    if not news_file_name.endswith('.txt'):
+        raise AssertionError(
+            'NEWS file %s does not have .txt extension.'
+            % (news_file_name,))
+    doc_name = news_file_name[:-4]
+    link_text = doc_name.replace('-', ' ')
+    out_file.write('   %s <%s>\n' % (link_text, doc_name))
+
+
+def output_news_file_plain(out_file, news_file_name):
+    f = open(news_file_name, 'rb')
+    try:
+        lines = f.readlines()
+    finally:
+        f.close()
+    title = os.path.basename(news_file_name)[len('bzr-'):-len('.txt')]
+    for line in lines:
+        if line == '####################\n':
+            line = '#' * len(title) + '\n'
+        elif line == 'Bazaar Release Notes\n':
+            line = title + '\n'
+        elif line == '.. toctree::\n':
+            continue
+        elif line == '   :maxdepth: 1\n':
+            continue
+        out_file.write(line)
+    out_file.write('\n\n')
 
 
 def main(argv):
     # Check usage
-    parser = OptionParser(usage="%prog SOURCE DESTINATION")
+    parser = OptionParser(usage="%prog OUTPUT_FILE NEWS_FILE [NEWS_FILE ...]")
     (options, args) = parser.parse_args(argv)
-    if len(args) != 2:
+    if len(args) < 2:
         parser.print_help()
         sys.exit(1)
 
     # Open the files and do the work
-    infile_name = args[0]
-    outfile_name = args[1]
-    outdir = os.path.dirname(outfile_name)
-    infile = open(infile_name, 'r')
-    try:
-        lines = infile.readlines()
-    finally:
-        infile.close()
-    outfile = open(outfile_name, 'w')
-    try:
-        split_into_topics(lines, outfile, outdir)
-    finally:
-        outfile.close()
+    out_file_name = args[0]
+    news_file_names = sorted(args[1:], key=natural_sort_key, reverse=True)
+
+    if os.path.basename(out_file_name) == 'index.txt':
+        preamble = preamble_sphinx
+        output_news_file = output_news_file_sphinx
+    else:
+        preamble = preamble_plain
+        output_news_file = output_news_file_plain
+
+    out_file = open(out_file_name, 'w')
+    try:
+        out_file.write(preamble)
+        for news_file_name in news_file_names:
+            output_news_file(out_file, news_file_name)
+    finally:
+        out_file.close()
 
 
 if __name__ == '__main__':

=== modified file 'tools/packaging/build-packages.sh'
--- tools/packaging/build-packages.sh	2009-05-25 10:59:42 +0000
+++ tools/packaging/build-packages.sh	2011-01-11 11:26:48 +0000
@@ -1,5 +1,5 @@
 #!/bin/bash
-# A helper script to build packages for the various distributions
+# Helper to build packages for the various distributions
 
 if [ -z "$UBUNTU_RELEASES" ]; then
     echo "Configure the distro platforms that you want to"

=== modified file 'tools/packaging/update-changelogs.sh'
--- tools/packaging/update-changelogs.sh	2009-06-03 10:26:06 +0000
+++ tools/packaging/update-changelogs.sh	2011-01-11 11:26:48 +0000
@@ -1,4 +1,5 @@
 #!/bin/bash
+# Helper to insert a new entry in debian/changelog
 
 if [ -z "$UBUNTU_RELEASES" ]; then
     echo "Configure the distro platforms that you want to"
@@ -27,7 +28,7 @@
     (
         echo "Updating changelog for $DISTRO"
         cd "$PACKAGE-$DISTRO" &&
-            dch -v $PPAVERSION -D $DISTRO -c changelog "$MSG." &&
+            dch -v $PPAVERSION -D $DISTRO "$MSG." &&
             bzr commit -m "$MSG: $PPAVERSION"
     )
 done

=== modified file 'tools/packaging/update-control.sh'
--- tools/packaging/update-control.sh	2009-09-02 04:08:08 +0000
+++ tools/packaging/update-control.sh	2011-01-11 11:26:48 +0000
@@ -1,4 +1,5 @@
 #!/bin/bash
+# Helper to run sed on versions in dependencies in debian/control
 
 if [ -z "$UBUNTU_RELEASES" ]; then
     echo "Configure the distro platforms that you want to"

=== modified file 'tools/packaging/update-packaging-branches.sh'
--- tools/packaging/update-packaging-branches.sh	2009-06-03 10:20:19 +0000
+++ tools/packaging/update-packaging-branches.sh	2010-08-27 00:33:07 +0000
@@ -18,10 +18,7 @@
             cd ..
         fi
     else
-        SRC="lp:~bzr/$PACKAGE/packaging-$DISTRO"
-        if [ "$PACKAGE" = "bzr-svn" ] ; then
-            SRC="lp:~bzr/$PACKAGE/beta-ppa-$DISTRO"
-        fi
+        SRC="lp:~bzr/ubuntu/$DISTRO/$PACKAGE/bzr-ppa"
         echo "Checking out $SRC"
         bzr co $SRC $PACKAGE-$DISTRO
     fi

=== added file 'tools/subunit-sum'
--- tools/subunit-sum	1970-01-01 00:00:00 +0000
+++ tools/subunit-sum	2011-06-16 10:45:17 +0000
@@ -0,0 +1,61 @@
+#!/usr/bin/env python
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Displays the sum of a counter found in a subunit stream.
+
+Each test have (or not) the counter as a named detail in the stream.
+"""
+
+import subunit
+import sys
+import testtools
+import unittest
+
+
+class TestSumCounter(testtools.TestResult):
+
+    def __init__(self, counter_names):
+        """Create a FilterResult object outputting to stream."""
+        testtools.TestResult.__init__(self)
+        self.counter_names = counter_names
+        self.totals = {}
+        self.longest = 0
+        for name in counter_names:
+            l = len(name)
+            if l > self.longest: self.longest = l
+            self.totals[name] = 0
+
+    def addSuccess(self, test, details):
+        for name in self.counter_names:
+            try:
+                counter_text = ''.join(details[name].iter_text())
+            except KeyError, e:
+                # this counter doesn't exist for the test
+                continue
+            self.totals[name] += int(counter_text)
+
+    def display_totals(self, stream):
+        for name in self.counter_names:
+            stream.write('%-*s: %s\n'
+                         % (self.longest, name, self.totals[name]))
+
+
+counter_names = sys.argv[1:]
+result = TestSumCounter(counter_names)
+test = subunit.ProtocolTestCase(sys.stdin)
+test.run(result)
+result.display_totals(sys.stdout)

=== removed file 'tools/trace-revisions'
--- tools/trace-revisions	2007-07-17 13:47:45 +0000
+++ tools/trace-revisions	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-#! /usr/bin/env python
-
-import sys
-
-from bzrlib.branch import Branch
-
-b = Branch.open('.')
-rev_id = sys.argv[1]
-rh = []
-while True:
-    rh.append(rev_id)
-    rev = b.get_revision(rev_id)
-    if rev.parent_ids:
-        rev_id = rev.parent_ids[0]
-    else:
-        break
-rh.reverse()
-for rev_id in rh:
-    print rev_id

=== removed file 'tools/weavebench.py'
--- tools/weavebench.py	2009-03-23 14:59:43 +0000
+++ tools/weavebench.py	1970-01-01 00:00:00 +0000
@@ -1,106 +0,0 @@
-#! /usr/bin/python
-
-# Copyright (C) 2005 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-
-"""Weave algorithms benchmark"""
-
-import bzrlib.branch
-from bzrlib.weave import Weave
-from bzrlib.weavefile import write_weave
-from bzrlib.progress import ProgressBar
-from random import randrange, randint, seed
-import tempfile
-import hotshot, hotshot.stats
-import sys
-
-WEAVE_NAME = "bench.weave"
-NUM_REVS = 2000
-
-seed(0)
-
-def build():
-    pb = ProgressBar(show_eta=False)
-
-    wf = Weave()
-    lines = []
-
-    parents = []
-    for i in xrange(NUM_REVS):
-        pb.update('building', i, NUM_REVS)
-
-        for j in range(randint(0, 4)):
-            o = randint(0, len(lines))
-            lines.insert(o, "new in version %i\n" % i)
-
-        for j in range(randint(0, 2)):
-            if lines:
-                del lines[randrange(0, len(lines))]
-
-        rev_id = wf.add("%s" % i, parents, lines)
-        parents = [rev_id]
-
-    write_weave(wf, file(WEAVE_NAME, 'wb'))
-
-        
-#     parents = set()
-#     revno = 1
-#     rev_history = b.revision_history()
-#     for rev_id in rev_history:
-#         pb.update('converting inventory', revno, len(rev_history))
-#         inv_xml = b.inventory_store[rev_id].readlines()
-#         weave_id = wf.add(parents, inv_xml)
-#         parents = set([weave_id])       # always just one parent
-#         revno += 1
-
-#     pb.update('write weave', None, None)
-#     write_weave(wf, file(WEAVE_NAME, 'wb'))
-
-    pb.clear()
-
-
-
-def profileit(fn): 
-    prof_f = tempfile.NamedTemporaryFile()
-
-    prof = hotshot.Profile(prof_f.name)
-
-    prof.runcall(fn) 
-    prof.close()
-
-    stats = hotshot.stats.load(prof_f.name)
-    #stats.strip_dirs()
-    stats.sort_stats('time')
-    ## XXX: Might like to write to stderr or the trace file instead but
-    ## print_stats seems hardcoded to stdout
-    stats.print_stats(20)
-            
-
-if '-p' in sys.argv[1:]:
-    opt_p = True
-    sys.argv.remove('-p')
-else:
-    opt_p = False
-
-if len(sys.argv) > 1:
-    NUM_REVS = int(sys.argv[1])
-
-if opt_p:
-    profileit(build)
-else:
-    build()
-    

=== modified file 'tools/win32/bazaar.url'
--- tools/win32/bazaar.url	2006-06-29 08:41:31 +0000
+++ tools/win32/bazaar.url	2010-12-07 09:06:39 +0000
@@ -1,5 +1,5 @@
 [InternetShortcut]
-URL=http://bazaar-vcs.org/
+URL=http://bazaar.canonical.com
 Hotkey=0
 IconIndex=0
 IconFile=

=== modified file 'tools/win32/build_release.py'
--- tools/win32/build_release.py	2009-07-31 19:09:31 +0000
+++ tools/win32/build_release.py	2010-12-07 09:06:39 +0000
@@ -85,7 +85,6 @@
     bzr_dir = get_bzr_dir()
     if not os.path.isdir(bzr_dir):
         bzr_version = VERSIONS['bzr']
-        # bzr_url = 'http://bazaar-vcs.org/bzr/bzr.' + bzr_version
         bzr_url = 'lp:bzr/' + bzr_version
         print "Getting bzr release %s from %s" % (bzr_version, bzr_url)
         call_or_fail([bzr(), 'co', bzr_url, bzr_dir])

=== modified file 'tools/win32/buildout.cfg'
--- tools/win32/buildout.cfg	2009-08-20 09:30:34 +0000
+++ tools/win32/buildout.cfg	2010-12-07 09:06:39 +0000
@@ -50,7 +50,7 @@
 # on Launchpad
 # bzr-release-url = http://bazaar-vcs.org/bzr/bzr.${settings:bzr-release}
 bzr-release-url = lp:bzr/${settings:bzr-release}
-bzr-trunk-url = http://bazaar-vcs.org/bzr/bzr.dev
+bzr-trunk-url = lp:bzr
 
 bzrtools-release = 1.18.0
 bzrtools-release-tag = tag:release-${settings:bzrtools-release}

=== modified file 'tools/win32/bzr.iss.cog'
--- tools/win32/bzr.iss.cog	2009-08-05 18:56:37 +0000
+++ tools/win32/bzr.iss.cog	2010-12-07 09:06:39 +0000
@@ -71,9 +71,9 @@
 
 AppComments="Bazaar: Friendly distributed version control system"
 AppPublisher="Canonical Ltd."
-AppPublisherURL="http://www.bazaar-vcs.org"
-AppSupportURL="http://www.bazaar-vcs.org/BzrSupport"
-AppUpdatesURL="http://www.bazaar-vcs.org/WindowsDownloads"
+AppPublisherURL="http://bazaar.canonical.com"
+AppSupportURL="http://wiki.bazaar.canonical.com/BzrSupport"
+AppUpdatesURL="http://wiki.bazaar.canonical.com/WindowsDownloads"
 ; [[[cog cog.outl('AppVersion=%s' % VERSION) ]]]
 ; [[[end]]]
 
@@ -177,7 +177,7 @@
 
 [Icons]
 Name: "{group}\Documentation index"; Filename: "{app}\doc\index.html"; WorkingDir: "{app}\doc";
-Name: "{group}\Bazaar Home Page"; Filename: "{app}\bazaar.url"; Comment: "http://www.bazaar-vcs.org";
+Name: "{group}\Bazaar Home Page"; Filename: "{app}\bazaar.url"; Comment: "http://bazaar.canonical.com";
 Name: "{group}\Start Bzr in cmd shell"; Filename: "{cmd}"; Parameters: "/K start_bzr.bat"; WorkingDir: "{app}"; IconFilename: "{app}\bzr.exe"; Comment: "Open new Bzr session";
 ; NOTE: Intent is to change the log file location - the line below will need to change to reflect that.
 Name: "{group}\Open Bzr log file"; Filename: "notepad.exe"; Parameters: "{userdocs}\.bzr.log"; Comment: "Launch notepad to view the bzr log file";

=== modified file 'tools/win32/bzr_postinstall.py'
--- tools/win32/bzr_postinstall.py	2010-02-04 16:06:36 +0000
+++ tools/win32/bzr_postinstall.py	2010-10-12 01:18:01 +0000
@@ -173,6 +173,9 @@
                 content[ix] = s.replace('bzr.exe',
                                         '"%s"' % os.path.join(bzr_dir,
                                                               'bzr.exe'))
+            elif s.find(r'C:\Program Files\Bazaar') != -1:
+                content[ix] = s.replace(r'C:\Program Files\Bazaar',
+                                        bzr_dir)
 
         if dry_run:
             print "*** Write file: start_bzr.bat"

=== added file 'tools/win32/py2exe_boot_common.py'
--- tools/win32/py2exe_boot_common.py	1970-01-01 00:00:00 +0000
+++ tools/win32/py2exe_boot_common.py	2011-04-11 09:38:59 +0000
@@ -0,0 +1,40 @@
+# Common py2exe boot script - executed for all target types.
+
+# In the standard py2exe boot script, it setup stderr so that anything written
+# to it will be written to exe.log, and a message dialog is shown.
+# For Bazaar, we log most things to .bzr.log, and there are many things that
+# write to stderr, that are not errors, and so we don't want the py2exe dialog
+# message, So also blackhole stderr.
+
+import sys
+if sys.frozen == "windows_exe":
+    class Blackhole(object):
+        softspace = 0
+        def write(self, text):
+            pass
+        def flush(self):
+            pass
+    sys.stdout = Blackhole()
+    sys.stderr = Blackhole()
+    del Blackhole
+
+# add more directories to sys.path to allow "installing" third-party libs
+# required by some plugins (see bug #743256)
+import os
+sys.path.append(os.path.join(os.path.dirname(sys.executable), 'site-packages'))
+del os
+del sys
+
+# Disable linecache.getline() which is called by
+# traceback.extract_stack() when an exception occurs to try and read
+# the filenames embedded in the packaged python code.  This is really
+# annoying on windows when the d: or e: on our build box refers to
+# someone elses removable or network drive so the getline() call
+# causes it to ask them to insert a disk in that drive.
+import linecache
+def fake_getline(filename, lineno, module_globals=None):
+    return ''
+linecache.orig_getline = linecache.getline
+linecache.getline = fake_getline
+
+del linecache, fake_getline

=== modified file 'tools/win32/start_bzr.bat'
--- tools/win32/start_bzr.bat	2007-01-30 23:05:35 +0000
+++ tools/win32/start_bzr.bat	2010-10-09 18:18:19 +0000
@@ -5,9 +5,8 @@
 REM    **  that affects on bzr behaviour                   **
 REM    ******************************************************
 
-REM If you don't add Bzr directory to system-wide PATH environment variable
-REM you can modify local PATH
-REM SET PATH=C:\Program Files\Bazaar;%PATH%
+REM Add the Bzr directory to system-wide PATH environment variable
+SET PATH=C:\Program Files\Bazaar;%PATH%
 
 REM Change next line to set-up e-mail to identify yourself in bzr
 REM SET BZREMAIL=
@@ -28,4 +27,4 @@
 REM --------------------------------------------------------------------------
 
 @ECHO ON
-@bzr.exe help
\ No newline at end of file
+@bzr.exe help

